/*******************************************
*
*
*	ANOVALine.h
*
*	A class that implements a line in an ANOVA table
*
*
************************************************/

#ifndef ANOVALineh
#define ANOVALineh = 1

#include "Element.h"
#include "aList.h"
#include "StringClass.h"
#include "StatList.h"
#include <iostream.h>
#include <fstream.h>
#include "aVector.h"
#include "aMatrix.h"
#include "MatrixList.h"
#include "dataColumnRec.h"
#include "dataColumnRecNum.h"
#include "dataColumnRecStr.h"
#include "dataSetRec.h"
#include "FDistribution.h"
#include "BootList.cpp"
#include "AdamLibraryDebug.h"
#include "ANOVALineBase.h"
#include "MemoryManager.h"

//20030815 updated to use 'long double' for all calculations
//20030821 core functions shared with class MANOVALine broken out into class ANOVALineBase


class ANOVALine: public ANOVALineBase{
#ifdef MemoryLeakDebug
	private:
		MemLeakIDType memID_;
	public:
		static memAllocList memIDList_;
		static  MemLeakIDType memIDCount_;
#endif
	public:
//		ObjList<StringList> factor;	//this will hold interactions
//		aString title_;
//		effectTypeSet effectType_;
//		int df_;
		long double SS_, bootSS;
		long double MS_, bootMS;
//		aString denomTitle_;
//		long double F_, bootF;
//		long double P_;
//		BootList<long double> bootFList;
		long double reducedFormSS_, reducedFormbootSS_;
		long double varianceComponent_, bootvarianceComponent_;
//		int levels_;//total number of factor levels present
	public:
		ANOVALine(void): SS_(0.0), bootSS(0.0), MS_(0.0),
				 bootMS(0.0),
				 reducedFormSS_(0.0), reducedFormbootSS_(0.0),
				 varianceComponent_(0.0), bootvarianceComponent_(0.0){
			#ifdef MemoryLeakDebug
				memID_=++memIDCount_;
				memIDList_.Append(memID_);
			#endif
				 	}
		ANOVALine(ANOVALine& a): ANOVALineBase(a), SS_(a.SS_), bootSS(a.bootSS), MS_(a.MS_),
				 bootMS(a.bootMS), 
				 reducedFormSS_(a.reducedFormSS_), reducedFormbootSS_(a.reducedFormbootSS_),
				 varianceComponent_(a.varianceComponent_), 
				 bootvarianceComponent_(a.bootvarianceComponent_){
			#ifdef MemoryLeakDebug
				memID_=++memIDCount_;
				memIDList_.Append(memID_);
			#endif
				 	}
		~ANOVALine(void){
			#ifdef MemoryLeakDebug
				long i=memIDList_.Loc(memID_);
				memIDList_.Delete(i);
			#endif
			}
			
	virtual ANOVALineBase*
					Duplicate(void){
							return NEW ANOVALine(*this);}
	void 			CalcRegressionSS (StatList<long double>* dependentVarList,StatList<int>* independentVarList);
	long double 	F(long bootRep) {return ANOVALineBase::F(bootRep);}
	void 			F(ANOVALineBase& denomLine, long bootRep);
	long double 	MS(long bootRep) {if(bootRep==0){return MS_;}else{return bootMS;}}
	void 			MS(long double val, long bootRep);
	virtual void 	Print (ostream& outFile, int titleLen, StringList& /*dependentList*/, long bootRep=0);
	long double 	SS(long bootRep) {if(bootRep==0){return SS_;}else{return bootSS;}}
	void 			SS(long double val, long bootRep);
	long double 	reducedFormSS(long bootRep) {
							if(bootRep==0){return reducedFormSS_;}else{return reducedFormbootSS_;}}
	void 			reducedFormSS(long double val, long bootRep);
	double 			sumXY (StatList<int>& Xlist, StatList<double>& Ylist);
	long double 	sumXY (StatList<int>& Xlist, StatList<long double>& Ylist);
	long double 	varianceComponent(long bootRep) {if(bootRep==0){
							return varianceComponent_;}else{return bootvarianceComponent_;}}
	void 			varianceComponent(long double val, long bootRep);

	friend int operator== (ANOVALine& line1, ANOVALine& line2);

	friend int operator!= (ANOVALine& line1, ANOVALine& line2);
#ifdef MemoryLeakDebug
	static void PrintIDList(ostream& outFile=cout);
	static void PrintIDListLen(ostream& outFile=cout);
#endif
	};//end class ANOVALine

int operator== (ANOVALine& line1, ANOVALine& line2);

int operator!= (ANOVALine& line1, ANOVALine& line2);



#endif
