/*******************************************
*
*
*	ANOVALineBase.h
*
*	A base class that implements common elements of a line in an ANOVA or MANOVA table
*
*
************************************************/

#ifndef ANOVALineBaseh
#define ANOVALineBaseh = 1

#include "Element.h"
#include "aList.h"
#include "StringClass.h"
#include "StatList.h"
#include <iostream.h>
#include <fstream.h>
#include "aVector.h"
#include "aMatrix.h"
#include "MatrixList.h"
#include "dataColumnRec.h"
#include "dataColumnRecNum.h"
#include "dataColumnRecStr.h"
#include "dataSetRec.h"
#include "FDistribution.h"
#include "BootList.h"
#include "MemoryManager.h"
#include "InlineTemplates.h"


enum effectTypeSet  {fixedEffect, randomEffect, errorLine, totalLine};
enum ANOVALineType  {factorialLine, simpleNestLine, interactionLine, complexNestLine};




class ANOVALineBase{
	public:
		ObjList<StringList> factor;	//this will hold interactions
		aString title_;
		effectTypeSet effectType_;
		int df_;
		int dependentVariables_;
		aString denomTitle_;
		long double F_, bootF;
		long double P_;
		BootList<long double> bootFList;
//		int levels_;//total number of factor levels present
		StringList levelNames_;//names of each factor level
		ObjList<aList<int> > levelRows_;//data rows corresponding to each factor level
	public:
		ANOVALineBase(void): df_(0), dependentVariables_(1), F_(zero), bootF(zero), P_(-one)/*, levels_(0)*/{}
		
		ANOVALineBase(ANOVALineBase& a): factor(a.factor), title_(a.title_),effectType_(a.effectType_),
				df_(a.df_), dependentVariables_(a.dependentVariables_), F_(a.F_), bootF(a.bootF),  P_(a.P_),
				 bootFList(a.bootFList),/*levels_(a.levels_),*/levelNames_(a.levelNames_),levelRows_(a.levelRows_){}

	virtual ~ANOVALineBase(void){
			factor.Clear();
			bootFList.Clear();
			levelNames_.Clear();
			levelRows_.Clear();
			}
			
	virtual ANOVALineBase*
			Duplicate(void){
					return NEW ANOVALineBase(*this);}
	virtual void 		AdjustSearchPosition (aVector<int>& presentPosition,aVector<int>&  setDims, int loc);
	virtual void 		AppendBootF(long double val){bootFList.Append(val);}
	virtual void 		CalcDesignAndDF (ObjList<StringList>& independentVarSet, dataSetRec& theDataSet, 
								aMatrix<long double>* designMatrix, aMatrix<long double>* incedenceMatrix, long n);
	virtual void 		CalcDesignAndDFComplexNest (/*ObjList<StringList>& independentVarSet,*/ dataSetRec& theDataSet, 
							aMatrix<long double>* designMatrix, aMatrix<long double>* incedenceMatrix, long n);
	virtual void 		CalcDesignAndDFInteraction (ObjList<StringList>& independentVarSet, dataSetRec& theDataSet, 
							aMatrix<long double>* designMatrix, aMatrix<long double>* incedenceMatrix, 
							long n, aMatrix<long double>* Ymatrix);
	virtual void 		CalcDesignAndDFNest (ObjList<StringList>& independentVarSet, dataSetRec& theDataSet, 
							aMatrix<long double>* designMatrix, aMatrix<long double>* incedenceMatrix, long n);
	virtual void 		ConvertTitleString (aString& theTitleStr, StringList& theList);
	virtual int 		df(void){return df_;}
	virtual void 		df(int d){df_ = d;}
	virtual int 		dependentVariables(void){return dependentVariables_;}
	virtual void 		dependentVariables(int dv){dependentVariables_=dv;}
	virtual effectTypeSet 
						effectType (void){return effectType_;}
	virtual void 		effectType (effectTypeSet val){effectType_=val;}
	virtual long double 	F(long bootRep) {if(bootRep==0){return F_;}else{return bootF;}}
	virtual void 		F(ANOVALineBase& denomLine, long bootRep);
	virtual ANOVALineType 	
			GetANOVALineType (aString& factorString);
	virtual int 		levels(void){return levelNames_.len();}
//	virtual void 		levels(int d){levels_ = d;}
	virtual aMatrix<long double>* 	
			MakeDesignMatrix (MatrixList& matrixList, int omitFactor);
	virtual int 		MeetsSearchCriteria (long indiv, StringList& searchCriteriaList, 
							ObjList<StringList>& independentVarSet);
	virtual aString 	MSDenomTitle (void){return denomTitle_;}
	virtual void 		MSDenomTitle (aString val){denomTitle_=val;}
	virtual long double 	P(void){return P_;}
	virtual void 		PartitionTitleString (aString& titleStr, aString& inParenStr, aString& outsideParenStr);
	virtual void		 PrepForCalcs (dataSetRec& theDataSet, aMatrix<long double>* factorDesign, 
								aMatrix<long double>* incedenceMatrix, long n, aMatrix<long double>* Ymatrix);
	virtual void 		Print (ostream& outFile, int titleLen, StringList& dependentList, long bootRep=0);
	virtual void 		PrintFactorLevels(ostream& outFile);
	virtual void 		PrintFactorLevelsAndDataRows(ostream& outFile);
	virtual void 		SetP(int denomDF, int boot=0);//boot == 0 gives standard P, boot==1 gives bootstrap P value
	virtual aString 	title(void){return title_;}
	virtual void 		title(aString val){title_=val;}
	friend int operator== (ANOVALineBase& line1, ANOVALineBase& line2);

	friend int operator!= (ANOVALineBase& line1, ANOVALineBase& line2);
	};//end class ANOVALineBase

int operator== (ANOVALineBase& line1, ANOVALineBase& line2);

int operator!= (ANOVALineBase& line1, ANOVALineBase& line2);



#endif
