/*******************************************
*
*
*	ANOVATable.h
*
*	A class that implements an ANOVA table
*
*
************************************************/

#ifndef ANOVATableh
#define ANOVATableh = 1

#include "Element.h"
#include "aList.h"
#include "StringClass.h"
#include "StatList.h"
#include <iostream.h>
#include <fstream.h>
#include "aVector.h"
#include "aMatrix.h"
#include "dataColumnRec.h"
#include "dataColumnRecNum.h"
#include "dataColumnRecStr.h"
#include "dataSetRec.h"
#include "ANOVALineBase.h"
#include "ANOVALine.h"
#include "GeneralLinearModel.h"
#include "MemoryManager.h"

//20030818 GLM moved to a class variable; design matrix calculations moved to separate function
//20030821 updated to reflect new base class for ANOVALine

class ANOVATable: public ObjList<ANOVALineBase>{
	public:
		GLM linearModel;

	public:
	ANOVATable(void){
		}
	ANOVATable(ANOVATable& a): ObjList<ANOVALineBase>(a){
		}
	ANOVATable(ANOVATable* a): ObjList<ANOVALineBase>(a){
		}
	~ANOVATable(void){
		}
	ANOVALineBase& 		GetANOVALine (int loc){return Nth(loc);}
	ANOVALineBase& 		GetANOVALine(aString& title);
	void AppendANOVATableFlist (long double val, int loc);
	long df(aString& theTitle);
	long df(int loc);
	void DoANOVA (dataSetRec& allData, aString& commandFileName, aString& outFileName){
			Ran3 ran(-1);//dummy
			DoBootANOVA (allData,  commandFileName, outFileName, 0, ran);}
	void DoBootANOVA (dataSetRec& allData, aString& commandFileName, aString& outFileName, long reps, Ran3& ran);
	long levels(aString& theTitle);
	long levels(int loc);
	void Print(ostream& outFile, int titleLen);
	void PrintFactorLevels(ostream& outFile);
	int 	ReadCommandFile (StringList& factorList, StringList& MSDivisorList, aString& commandFileName);
	void SetDF (int val, int loc);
	void SetEffect (effectTypeSet val, int loc);
	void SetF (StringList& MSDivisorList, long bootRep);
//	void SetLevels (int val, int loc);
	void SetMS (long double val, long loc, long bootRep);
	void SetSS (long double val, long loc, long bootRep);
	void SetReducedFormSS (long double val, long loc, long bootRep);
	void SetVarianceComponent (long double val, long loc, long bootRep);
	void SetP (StringList& MSDivisorList, int boot = 0);//boot == 0 gives standard P, boot==1 gives bootstrap P value
	void SetTitle (aString val, int loc);
	void SetMSDenomTitle (aString val, int loc);
	int  SetUpTableAndDesignMatrix (dataSetRec& allData, aString& commandFileName, 
										StringList& factorList, StringList& MSDivisorList, 
										long& dfSum, int& longTitle);
	long double SS(aString& theTitle);
	long double MS(aString& theTitle);
	long double reducedSS(aString& theTitle);
	long double varianceComponent(aString& theTitle);
	
	
	};//end class ANOVATable


/*
{ANOVA functions & procedures}
	function GetSum (theList: ListStructureType): extended;
	function GetSumValsSqd (theList: ListStructureType): extended;
	function GetSumXY (Xlist, Ylist: ListStructureType): extended;
	function SSQToGivenMean (theList: ListStructureType; givenMean: extended; var df: longint): extended;

	procedure Statistics (theList: ListStructureType; var n: longint; var mean, vari, samVari, kurt: extended; var df: longint; var t: extended);
*/



#endif

