/*******************************************
*
*
*	GeneralLinearModel
*
*	A class that implements least squares estimation using matrix algebra
*
*
************************************************/

#ifndef GeneralLinearModelh
#define GeneralLinearModelh = 1

#include "Element.h"
#include "aList.h"
#include "MemoryManager.h"
#include "StringClass.h"
#include "StatList.h"
#include <iostream.h>
#include "aVector.h"
#include "aMatrix.h"
//#include "aMatrix2.h"
#include "MatrixList.h"

class ANOVATable;
class MANOVATable;



class GLM{
	public:
		MatrixList* designList;
		MatrixList* incedenceMatrixList;
		MatrixList* invXprimeXList;
		ObjList<aList<long double> > varCompCoefficients;//used in variance component estimation
		int reusingDesign_;//will use this design again
		int recyclablesCalculated_;//designList, etc. calculated and stored, or not
		int varCompCoeffsSupplied_;//1 if supplied externally, 0 if to be calculated here
	public: 
		GLM(void):designList(NEW MatrixList()),
				   incedenceMatrixList(NEW MatrixList()),
				   invXprimeXList(NEW MatrixList()),
				   reusingDesign_(0),recyclablesCalculated_(0),varCompCoeffsSupplied_(0)
				   {
				   	}
		GLM(GLM& glm):designList(NEW MatrixList(glm.designList)),
				   incedenceMatrixList(NEW MatrixList(glm.incedenceMatrixList)),
				   invXprimeXList(NEW MatrixList(glm.invXprimeXList)),
				   varCompCoefficients(glm.varCompCoefficients),
				   reusingDesign_(glm.reusingDesign_),
				   recyclablesCalculated_(glm.recyclablesCalculated_),
				   varCompCoeffsSupplied_(glm.varCompCoeffsSupplied_)
				   {
				   	}
		~GLM(void){
			Clear();
			}
	virtual GLM* Duplicate(void){
				return NEW GLM(*this);}

	void 			ANOVASSdfMS (aMatrix<long double>* Ymatrix, ANOVATable& theANOVATable,
										long  bootRep, MatrixList *residuals = NULL);
	void 			AppendDesign(aMatrix<long double>* theDesign){
						designList->Append(theDesign);}
	void 			AppendIncedenceMatrix(aMatrix<long double>* theIncedenceMatrix){
						incedenceMatrixList->Append(theIncedenceMatrix);}
	void 			AppendInvXprimeXList(aMatrix<long double>* invXprimeXmatrix){
						invXprimeXList->Append(invXprimeXmatrix);}
	void 			Clear(void);
	aMatrix<long double>* 	MakeDesignMatrix (MatrixList& mList, const int omitFactor);
	aMatrix<long double>* 	MakeDesignMatrix (MatrixList& mList, aList<int>& omitFactor);
	void 			MANOVASSdfVarCov (aMatrix<long double>* Ymatrix, MANOVATable& theMANOVATable, 
										long  bootRep, MatrixList *residuals = NULL);
	void 			RegressionSS (aMatrix<long double>*Xmatrix, 
							       aMatrix<long double>*Ymatrix, 
							       long double& SSR, long double& SSE, long double& SSTO, 
							      	matrixHandle<long double>& invXprimeXmatrixH);
	void 			RegressionSS (aMatrix<long double>*Xmatrix, 
							       aMatrix<long double>*Ymatrix, 
							       long double& SSR, long double& SSE, long double& SSTO, 
							      	matrixHandle<long double>& invXprimeXmatrixH, 
								matrixHandle<long double>& BmatrixH,  
								matrixHandle<long double>& EmatrixH);
	void 			RegressionSS (aMatrix<long double>*Xmatrix, 
							       aMatrix<long double>*Ymatrix, 
							       matrixHandle<long double>& SSRmatrix, matrixHandle<long double>& SSEmatrix, 
							       matrixHandle<long double>& SSTOmatrix, 
							      	matrixHandle<long double>& invXprimeXmatrixH, 
								matrixHandle<long double>& BmatrixH,  
								matrixHandle<long double>& EmatrixH);
	int	 			recyclablesCalculated(void){return recyclablesCalculated_;}
	int	 			reuseDesign(void){return reusingDesign_;}
	void 			SetToReuseDesign(int TorF){reusingDesign_=TorF;}
	void 			SetRecyclablesCalculated(int TorF){recyclablesCalculated_=TorF;}
	void 			VarianceComponentsANOVA (aMatrix<long double>* Ymatrix, ANOVATable& theANOVATable, long  bootRep);
	int 				varCompCoeffsSupplied(void){return varCompCoeffsSupplied_;}
	void 			varCompCoeffsSupplied(int TorF){varCompCoeffsSupplied_=TorF;}

	friend int		operator==(GLM& a, GLM& b);
	friend int		operator!=(GLM& a, GLM& b);


	};//end class GLM









#endif

