/*******************************************
*
*
*	MANOVALine.h
*
*	A class that implements a line in a MANOVA table
*
*
************************************************/

#ifndef MANOVALineh
#define MANOVALineh = 1

#include "Element.h"
#include "aList.h"
#include "StringClass.h"
#include "StatList.h"
#include <iostream.h>
#include <fstream.h>
#include "aVector.h"
#include "aMatrix.h"
#include "MatrixList.h"
#include "dataColumnRec.h"
#include "dataColumnRecNum.h"
#include "dataColumnRecStr.h"
#include "dataSetRec.h"
#include "FDistribution.h"
#include "BootList.h"
#include "ANOVALineBase.h"
#include "MemoryManager.h"


class GLM;

class MANOVALine: public ANOVALineBase{
	public:
		aMatrix<long double> SS_, bootSS_;
		aMatrix<long double> VarCov_, bootVarCov_, VarCovP_;//equivalent to MS in ANOVA
		aMatrix<long double> Corr_, bootCorr_, CorrP_;//equivalent to MS in ANOVA
		long double WilksLambda_, bootWilksLambda_;
		long double F_Wilks_, bootF_Wilks_;
		long double P_Wilks_, bootP_Wilks_;//P holds parametric P, boot holds P from bootstrap reps
		long double PillaisTrace_, bootPillaisTrace_;
		long double F_Pillai_, bootF_Pillai_;
		long double P_Pillai_, bootP_Pillai_;//P holds parametric P, boot holds P from bootstrap reps
		long double HotellingLawleyTrace_, bootHotellingLawleyTrace_;
		long double F_HLtrace_, bootF_HLtrace_;
		long double P_HLtrace_, bootP_HLtrace_;//P holds parametric P, boot holds P from bootstrap reps
		long double RoysLargestRoot_, bootRoysLargestRoot_;
		long double F_Roys_, bootF_Roys_;
		long double P_Roys_, bootP_Roys_;//P holds parametric P, boot holds P from bootstrap reps
		long double df1WL_, df2WL_;///degrees of freedom for multivariate F calculation on Wilks' Lambda
		long double df1PT_, df2PT_;///degrees of freedom for multivariate F calculation on Pillai's Trace
		long double df1HLT_, df2HLT_;///degrees of freedom for multivariate F calculation on Hotelling-Lawley Trce
		long double df1RMR_, df2RMR_;///degrees of freedom for multivariate F calculation on Roy's Max Root
		ObjList<ObjList<BootList<long double> > > VarCovBootList_, CorrBootList_;//for pooled levels
		MatrixList CovLevelList_;//partial covariances, indexed by [factor level]
		ObjList<MatrixList> CovLevelBootList_;//indexed by [rep][factor level]
		MatrixList CorrLevelList_;//partial correlations, indexed by [factor level]
		ObjList<MatrixList> CorrLevelBootList_;//indexed by [rep][factor level]
		BootList<long double> WilksLambdaBootList_;
		BootList<long double> PillaisTraceBootList_;
		BootList<long double> HLtraceBootList_;
		BootList<long double> RoysRootBootList_;
//		aMatrix<long double> covarianceComponents_,bootCovarianceComponents_;
//		ObjList<ObjList<BootList<long double> > > VarCovComponentBootList_;//index [row][col][rep]
		int SS_flag,VarCov_flag,VarCovComponent_flag, wilks_flag, df_flag;//true if matrices created, false if not
	public:
		MANOVALine(void): ANOVALineBase(),WilksLambda_((long double)0.0),bootWilksLambda_((long double)0.0),
				F_Wilks_((long double)0.0), bootF_Wilks_((long double)0.0),P_Wilks_((long double)-1.0), bootP_Wilks_((long double)-1.0),
				PillaisTrace_((long double)0.0),bootPillaisTrace_((long double)0.0),
				F_Pillai_((long double)0.0),bootF_Pillai_((long double)0.0),P_Pillai_((long double)-1.0),bootP_Pillai_((long double)-1.0),
				HotellingLawleyTrace_((long double)0.0),bootHotellingLawleyTrace_((long double)0.0),
				F_HLtrace_((long double)0.0),bootF_HLtrace_((long double)0.0),P_HLtrace_((long double)-1.0),bootP_HLtrace_((long double)-1.0),
				RoysLargestRoot_((long double)0.0),bootRoysLargestRoot_((long double)0.0),
				F_Roys_((long double)0.0), bootF_Roys_((long double)0.0),P_Roys_((long double)-1.0), bootP_Roys_((long double)-1.0),
				df1WL_((long double)0.0), df2WL_((long double)0.0),
				df1PT_((long double)0.0), df2PT_((long double)0.0),
				df1HLT_((long double)0.0), df2HLT_((long double)0.0),
				df1RMR_((long double)0.0), df2RMR_((long double)0.0),
				SS_flag(false),VarCov_flag(false),VarCovComponent_flag(false), wilks_flag(false), df_flag(false){}
		
		MANOVALine(MANOVALine& a): ANOVALineBase(a),SS_(a.SS_), bootSS_(a.bootSS_), 
				 VarCov_(a.VarCov_),bootVarCov_(a.bootVarCov_),VarCovP_(a.VarCovP_),
				 Corr_(a.Corr_),bootCorr_(a.bootCorr_),CorrP_(a.CorrP_),
			 	 WilksLambda_(a.WilksLambda_),bootWilksLambda_(a.bootWilksLambda_),
				 F_Wilks_(a.F_Wilks_), bootF_Wilks_(a.bootF_Wilks_), P_Wilks_(a.P_Wilks_), bootP_Wilks_(a.bootP_Wilks_),
				 PillaisTrace_(a.PillaisTrace_),bootPillaisTrace_(a.bootPillaisTrace_),
				 F_Pillai_(a.F_Pillai_),bootF_Pillai_(a.bootF_Pillai_),P_Pillai_(a.P_Pillai_),bootP_Pillai_(a.bootP_Pillai_),
				 HotellingLawleyTrace_(a.HotellingLawleyTrace_),bootHotellingLawleyTrace_(a.bootHotellingLawleyTrace_),
				 F_HLtrace_(a.F_HLtrace_),bootF_HLtrace_(a.bootF_HLtrace_),P_HLtrace_(a.P_HLtrace_),bootP_HLtrace_(a.bootP_HLtrace_),
				 RoysLargestRoot_(a.RoysLargestRoot_),bootRoysLargestRoot_(a.bootRoysLargestRoot_),
				 F_Roys_(a.F_Roys_), bootF_Roys_(a.bootF_Roys_), P_Roys_(a.P_Roys_), bootP_Roys_(a.bootP_Roys_),
				 df1WL_(a.df1WL_), df2WL_(a.df2WL_),
				 df1PT_(a.df1PT_), df2PT_(a.df2PT_),
				 df1HLT_(a.df1HLT_), df2HLT_(a.df2HLT_),
				 df1RMR_(a.df1RMR_), df2RMR_(a.df2RMR_),
				 VarCovBootList_(a.VarCovBootList_),CorrBootList_(a.CorrBootList_),
				 CovLevelList_(a.CovLevelList_),CovLevelBootList_(a.CovLevelBootList_),
				 CorrLevelList_(a.CorrLevelList_),CorrLevelBootList_(a.CorrLevelBootList_),
				 WilksLambdaBootList_(a.WilksLambdaBootList_),PillaisTraceBootList_(a.PillaisTraceBootList_),
				 HLtraceBootList_(a.HLtraceBootList_),RoysRootBootList_(a.RoysRootBootList_),
//				 covarianceComponents_(a.covarianceComponents_),bootCovarianceComponents_(a.bootCovarianceComponents_),
//				 VarCovComponentBootList_(a.VarCovComponentBootList_),
				 SS_flag(a.SS_flag),VarCov_flag(a.VarCov_flag),VarCovComponent_flag(a.VarCovComponent_flag),
				 wilks_flag(a.wilks_flag), df_flag(a.df_flag){}
	virtual ~MANOVALine(void){
			WilksLambdaBootList_.Clear();
			VarCovBootList_.Clear();
			CorrBootList_.Clear();
			CovLevelList_.Clear();
			CovLevelBootList_.Clear();
			CorrLevelList_.Clear();
			CorrLevelBootList_.Clear();
			WilksLambdaBootList_.Clear();
			PillaisTraceBootList_.Clear();
			HLtraceBootList_.Clear();
			RoysRootBootList_.Clear();
//			VarCovComponentBootList_.Clear();
			}
			
	virtual ANOVALineBase*
				Duplicate(void){
					return NEW MANOVALine(*this);}
	void 		CalcRegressionSS (StatList<long double>* dependentVarList,StatList<int>* independentVarList);
//	aMatrix<long double>& 	
//				covarianceComponents(long bootRep) {if(bootRep==0){return covarianceComponents_;}else{return bootCovarianceComponents_;}}
//	void 		covarianceComponents(aMatrix<long double>& vals, long bootRep);
	long double F(long bootRep) {if(bootRep==0){return F_;}else{return bootF;}}//returns F for Wilk's Lambda only
//	void 		F(ANOVALineBase& denomLine, long bootRep);
	void 		F(ANOVALineBase& total_Line, long bootRep);
	long double F_HLtrace(long bootRep) {if(bootRep==0){return F_HLtrace_;}else{return bootF_HLtrace_;}}//returns F for Wilk's Lambda only
	long double F_Pillai(long bootRep) {if(bootRep==0){return F_Pillai_;}else{return bootF_Pillai_;}}//returns F for Wilk's Lambda only
	long double F_Roys(long bootRep) {if(bootRep==0){return F_Roys_;}else{return bootF_Roys_;}}//returns F for Wilk's Lambda only
	long double F_Wilks(long bootRep) {if(bootRep==0){return F_Wilks_;}else{return bootF_Wilks_;}}//returns F for Wilk's Lambda only
	long double HotellingLawley(long bootRep) {if(bootRep==0){return HotellingLawleyTrace_;}else{return bootHotellingLawleyTrace_;}}
	long double PillaisTrace(long bootRep) {if(bootRep==0){return PillaisTrace_;}else{return bootPillaisTrace_;}}
	void 		Print (ostream& outFile, int titleLen, StringList& dependentList, long bootRep=0);
	void 		PrintWithinLevelCorrelations (ostream& outFile, StringList& dependentList, long bootRep=0);
	long double RoysMaxRoot(long bootRep) {if(bootRep==0){return RoysLargestRoot_;}else{return bootRoysLargestRoot_;}}
//	void 		SetMultivariateDF(ANOVALineBase& denomLine);//handled in F and WilksLambda now
	void 		SetP(int /*denomDF*/, int boot);//boot == 0 gives standard P, boot==1 gives bootstrap P value
	void 		SetVarCovDenomTitle (aString& val, int loc);
	aMatrix<long double>& 	
				SS(long bootRep) {if(bootRep==0){return SS_;}else{return bootSS_;}}
	void 		SS(aMatrix<long double>& vals, long bootRep);
//	long double 	reducedFormSS(long bootRep) {
//					if(bootRep==0){return reducedFormSS_;}else{return reducedFormbootSS_;}}
//	void 		reducedFormSS(long double val, long bootRep);
	double 		sumXY (StatList<int>& Xlist, StatList<double>& Ylist);
	long double sumXY (StatList<int>& Xlist, StatList<long double>& Ylist);
	aMatrix<long double>& 	
				VarCov(long bootRep) {if(bootRep==0){return VarCov_;}else{return bootVarCov_;}}
	void 		VarCov(aMatrix<long double>& vals, long bootRep, aMatrix<long double> *residuals);
	long double WilksLambda(long bootRep) {if(bootRep==0){return WilksLambda_;}else{return bootWilksLambda_;}}
	void 		WilksLambda(ANOVALineBase& denomLine, long bootRep);

	friend int operator== (MANOVALine& line1, MANOVALine& line2);

	friend int operator!= (MANOVALine& line1, MANOVALine& line2);
	};//end class MANOVALine

int operator== (MANOVALine& line1, MANOVALine& line2);

int operator!= (MANOVALine& line1, MANOVALine& line2);



#endif
