/*******************************************
*
*
*	MANOVATable.h
*
*	A class that implements a MANOVA table
*
*
************************************************/

#ifndef MANOVATableh
#define MANOVATableh = 1

#include "Element.h"
#include "aList.h"
#include "StringClass.h"
#include "StatList.h"
#include <iostream.h>
#include <fstream.h>
#include "aVector.h"
#include "aMatrix.h"
#include "dataColumnRec.h"
#include "dataColumnRecNum.h"
#include "dataColumnRecStr.h"
#include "dataSetRec.h"
#include "ANOVALineBase.h"
#include "MANOVALine.h"
#include "GeneralLinearModel.h"
#include "MemoryManager.h"


class MANOVATable: public ObjList<ANOVALineBase>{
	public:
		GLM linearModel;
		StringList dependentList;
	public:
	MANOVATable(void){} 
	
	MANOVATable(MANOVATable& a): ObjList<ANOVALineBase>(a){}
	
	MANOVATable(MANOVATable* a): ObjList<ANOVALineBase>(a){	}
	
	~MANOVATable(void){}
	
	ANOVALineBase& 		GetMANOVALine (int loc){return Nth(loc);}
	ANOVALineBase& 		GetMANOVALine(aString& title);
	void AppendMANOVATableFlist (long double val, int loc);
	long df(aString& theTitle);
	long df(int loc);
	void DoMANOVA (dataSetRec& allData, aString& commandFileName, aString& outFileName, 
						int factorLevelCorrelations){
			Ran3 ran(-1);//dummy
			DoBootMANOVA (allData,  commandFileName, outFileName, 0, ran,false,factorLevelCorrelations);}
	void DoBootMANOVA (dataSetRec& allData, aString& commandFileName, 
						aString& outFileName, long reps, Ran3& ran, 
						int resampleColumnsIndependently, int factorLevelCorrelations);
	long levels(aString& theTitle);
	long levels(int loc);
	void Print (ostream& outFile, int titleLen, int boot);
	void PrintFactorLevels(ostream& outFile);
	void PrintWithinLevelCorrelations (ostream& outFile, long bootReps);
	int 	ReadCommandFile (StringList& factorList, 
								StringList& MSDivisorList, aString& commandFileName);
	void SetDF (int val, int loc);
	void SetEffect (effectTypeSet val, int loc);
	void SetF (StringList& MSDivisorList, long bootRep);
//	void SetLevels (int val, int loc);
//	void SetMultivariateDF (StringList& MSDivisorList);//now handled in WilksLambda
	void SetVarCov (aMatrix<long double>& vals, long loc, long bootRep, MatrixList *residuals);//now handled in SetF and WilksLambda
	void SetSS (aMatrix<long double>& vals, long loc, long bootRep);
//	void SetReducedFormSS (aMatrix<long double>& vals, long loc, long bootRep);
//	void SetCovarianceComponents (aMatrix<long double>& vals, long loc, long bootRep);
	void SetP (StringList& /*MSDivisorList*/, int boot = 0);//boot == 0 gives standard P, boot==1 gives bootstrap P value
	void SetTitle (aString val, int loc);
	int  SetUpTableAndDesignMatrix (dataSetRec& allData, aString& commandFileName, 
										StringList& factorList, StringList& MSDivisorList, 
										long& dfSum, int& longTitle);
	void SetVarCovDenomTitle (aString& val, int loc);
	void SetWilksLambda (StringList& MSDivisorList, long bootRep);
	aMatrix<long double>& SS(aString& theTitle);
	aMatrix<long double>& VarCov(aString& theTitle);
//	aMatrix<long double>& reducedSS(aString& theTitle);
//	aMatrix<long double>& covarianceComponents(aString& theTitle);
	
	
	};//end class MANOVATable




#endif

