/*******************************************
*
*
*	dataColumnRecNum.h
*
*	A class that implements a numerical data column in an ANOVA table
*
*
************************************************/

#ifndef dataColumnRecNumh
#define dataColumnRecNumh = 1

#include "Element.h"
#include "aList.h"
#include "StringClass.h"
#include "StatList.h"
#include <iostream.h>
#include <fstream.h>
#include "aVector.h"
#include "aMatrix.h"
#include "dataColumnRec.h"
#include "MemoryManager.h"


//	200709043	comment out MemoryLeakDebug code
//	20070903	move inlines to .cpp file

template <class T> class dataColumnRecNum: public dataColumnRec{




	public:
		aList<T> dataList_;
	public:
		dataColumnRecNum(int source=0);
//		dataColumnRecNum(dataColumnRecNum<T>&);//no copying possible; use Clone instead
//		dataColumnRecNum(dataColumnRecNum<T>*);//no copying possible; use Clone instead
		~dataColumnRecNum(void);

	dataColumnRec* 
				Duplicate(void);//returns a duplicate copy of 'this'

	dataColumnRecNum<T>& operator=(dataColumnRecNum<T>& dcrn);
	T& 			operator[](long i);
	
	virtual void	SetPointerToDerivedSelf(void);
	virtual long	GetPointerToDerivedSelf(void);

	//data column handling functions
	void 			Append(T x);
	T&	 			First(void);
	T&	 			Last(void);
	aList<T>&		dataList(void);
	void 			DeleteRow(long r);
	aList<T>*  	GetResampledList (sampleRegimeType sampleRegime, Ran3& ran);
	aVector<T>*  	GetResampledVector(sampleRegimeType sampleRegime, Ran3& ran);
	aVector<double>*  
				GetResampledVectorDouble(sampleRegimeType sampleRegime, Ran3& ran);
	aVector<long double>*  
				GetResampledVectorLongDouble(sampleRegimeType sampleRegime, Ran3& ran);
	T  			GetResampleValue(aList<T>& theList, sampleRegimeType sampleRegime, Ran3& ran);
//	T  			GetResampleValue(vector<T>& theList, sampleRegimeType sampleRegime, Ran3& ran);
	long 			len();
//	int 			MissingData(long double missingDataFlag);
	aList<long>* 	MissingDataPos(char, long double missingDataFlag);
	aList<long>* 	MissingDataPos(long double missingDataFlag);
	 void 		PrintValue(ostream& outFile, long item);
	virtual long 	SampleSize(long double missingDataFlag);
	virtual void	SetValue(aString& str, long item);

	friend int operator== (dataColumnRecNum<T>& dataCol1, dataColumnRecNum<T>& dataCol2);

	friend int operator!= (dataColumnRecNum<T>& dataCol1, dataColumnRecNum<T>& dataCol2);


	};//end class dataColumnRecNum

template <class T> 
	int operator== (dataColumnRecNum<T>& dataCol1, dataColumnRecNum<T>& dataCol2){
		return ((dataColumnRec)dataCol1 == (dataColumnRec) dataCol2);}

template <class T> 
	int operator!= (dataColumnRecNum<T>& dataCol1, dataColumnRecNum<T>& dataCol2){
		return ((dataColumnRec)dataCol1 != (dataColumnRec) dataCol2);}


#endif

