/*******************************************
*
*
*	dataSetRec.h
*
*	A class that implements a data set held by an ANOVA table
*
*
************************************************/

#ifndef dataSetRechh
#define dataSetRechh = 1

#include "Element.h"
#include "aList.h"
#include "StringClass.h"
#include "StatList.h"
#include <iostream.h>
#include <fstream.h>
#include "aVector.h"
#include "aMatrix.h"
#include "InlineTemplates.h"
#include "dataColumnRec.h"
#include "dataColumnRecNum.h"
#include "dataColumnRecStr.h"
#include "MatrixList.h"
#include "MemoryManager.h"


//20070903	comment out MemoryLeakDebug code
//20070903	move all inllines to .cpp file


class dataSetRec : public ObjList<dataColumnRec> {
	public:
	dataSetRec(void);
	dataSetRec(dataSetRec& d);
//	dataSetRec(dataSetRec* d);
	~dataSetRec(void);
	
	
	void 			ClearNullFactorLevels (void);
	aList<int>* 	ColumnsWithMissingData(char missingStrDataFlag =  '\0', long double missingNumDataFlag=(- one));
						//returns columns numbers with missing data
						//returned list must be disposed elsewhere
	aList<int>* 	ColumnsWithOutMissingData(char missingStrDataFlag =  '\0', long double missingNumDataFlag=(-one));
		//returns columns numbers without missing data
		//returned list must be disposed elsewhere
	void  			ConstructData(MatrixList& dataClassification, MatrixList& dataItself,
						StringList& dataClassificationNames,StringList& dataColumnNames);
	dataSetRec* 	CopyIndependentColumns(StringList& dependentColNames);
	void 			CullMissingData(char missingStrDataFlag, long double missingNumDataFlag);
	void			DeleteRow(long rownum);
	dataColumnContentType 
					DetermineColumnContents (aString& word);
	dataColumnRec& 	GetDataColumn (aString& colTitle);
	dataColumnRec& 	GetDataColumn (int loc);
	StringList& 	GetFactorLevelList (aString& colTitle);
	void 			GetResampledData(aMatrix<long double>& Ymatrix,int sampleWithReplacement, Ran3& ran);
						//only call GetResampledData if whole data set is of type 'long double'
	int 			IncludesCol(aString& colTitle);
	int 			MissingData(char missingStrDataFlag = '\0', long double missingNumDataFlag=(- one));
						//returns 0 if no missing data, 1 otherwise
	ObjList<dataSetRec>* 
					ParseMissingDataIntoSets(StringList& dependentColNames, 
											char missingStrDataFlag , long double missingNumDataFlag);
	void 			Print(int printTempData = 0);			
	void 			Print(aString& outFileName, char separator=' ', int printTempData = 0);
	void 			Print(ostream& outFile, char separator=' ', int printTempData = 0);
	void 			PrintMathematicaFormat(aString& outFileName, bool printHeadings = false, bool printTempData = false);
	void 			PrintMathematicaFormat(ostream& outFile, bool printHeadings = false, bool printTempData = false);
	void 			ReadDataFile (aString& dataFileName);
	aString* 		SynthesizeNewStrDataCol(StringList& colTitles);//returns new col title name
	
	};//end class dataSetRec


#endif
