/******************************************
*
*	memAllocList.h
*
*	This implements a specialized list containing
*	information for debugging memory leak problems.
*
*******************************************/

#ifndef memAllocListh
#define memAllocListh=1


#include <iostream.h>
#include "InlineTemplates.h"

typedef  unsigned long MemLeakIDType;


class memElem {
	protected:
		MemLeakIDType data_;
		memElem *prev, *next;
	public:
		memElem(MemLeakIDType x):data_(x), prev(NULL), next(NULL){}
		~memElem(void){
			prev=next=NULL;}//these must be followed and disposed properly elsewhere
		MemLeakIDType& data(void){return data_;}
		memElem* nextElem(void){return next;}
		memElem* prevElem(void){return prev;}
	friend class memAllocList;
};//end memElem


class memAllocList{
  protected:
  	long len_;
  	memElem *first,*last,*recent;
  	long recentLoc_;
  public:
    				memAllocList(void);//empty list
				memAllocList(memAllocList& toCopy);//constucts a new copy
				memAllocList(memAllocList* toCopy);//constructs a new copy

virtual 			~memAllocList(void);//deletes remaining contents
virtual void 					Append (MemLeakIDType x);
virtual int					BadIndex(long loc);
virtual void					Clear(void);
virtual void					Copy(memAllocList& toCopy);
virtual void					Copy(memAllocList* toCopy);
virtual void 					DeleteFirst (void);
virtual void 					DeleteLast (void);
virtual void 					Delete (long loc);
						//range of loc=0 (=first) to len()-1 (=last)
virtual MemLeakIDType&   		First (void);
	memElem* 				GoToElem (long loc);
virtual int					Includes (MemLeakIDType x);
virtual void 					Insert (MemLeakIDType x, long loc);
						//range of loc=0 (=first) to len()-1 (=last)
virtual MemLeakIDType&   		Last (void);
virtual long 					len(void);
virtual long 					Loc (MemLeakIDType x);
						//range of loc=0 (=first) to len()-1 (=last)
virtual MemLeakIDType&	  	Nth (long loc);
						//range of loc=0 (=first) to len()-1 (=last)
virtual void 					Print(ostream& outFile=cout);
virtual void 					PrintLen(ostream& outFile=cout);
virtual void 					Prepend (MemLeakIDType x);
virtual void 					Replace (MemLeakIDType x, long loc);
						//range of loc=0 (=first) to len()-1 (=last)
virtual void 					Sort (void);
						//sorts entire list using Sort(long,long)
virtual void 					Sort (long start, long finish);
						//a QuickSort algorithm
virtual memAllocList&			operator= (memAllocList& z);
virtual MemLeakIDType&		operator[] (const long loc);



	friend int operator== (memAllocList& list1, memAllocList& list2);

	friend int operator!= (memAllocList& list1, memAllocList& list2);
	
    };//end class memAllocList
	
int operator== (memAllocList& list1, memAllocList& list2);

int operator!= (memAllocList& list1, memAllocList& list2);



#endif





