/******************************************
*
*
*	ANOVAGenetics.h
*
*
*	This class performs calculations needed to get variance components for
*	genetic data analysis.  It is a derived class of ANOVATable.h, so it employs
*	linear algebra methods to analyze the data.
*
*
************************************************/

#ifndef ANOVAGeneticsh
#define ANOVAGeneticsh = 1


# include "ANOVATable.h"
# include "GeneralLinearModel.h"
#include "PopulationGeneticData.h"
#include "CodominantMarkerData.h"
#include "DominantMarkerData2.h"
#include "SexLinkedCodomMarkerData.h"
#include "MemoryManager.h"
#include "Element.h"
#include "aList.h"
#include "Stringclass.h"
#include "StringList.h"

class PopulationGeneticData;
class CodominantMarkerData;
class DominantMarkerData2;
class SexLinkedData;
class SexLinkedCodomMarkerData;
class SexLinkedDomMarkerData2;

class ANOVAGenetics: public ANOVATable{
	public:
	ANOVAGenetics(void){
		}
	~ANOVAGenetics(void){
		}
	
	
	ObjList<aList<long double> >* CalculateMeanSquares (/*PopulationGeneticData& geneticData, */
								dataSetRec& ANOVAData, StringList& factorList, 
								StringList& dependentColNames/*, long reps, Ran3& ran*/);
					//factorList holds the analysis inut lines
	ObjList<aList<long double> >* CalculateVarianceComponents (/*PopulationGeneticData& geneticData, */
								dataSetRec& ANOVAData, StringList& factorList, 
								StringList& dependentColNames/*, long reps, Ran3& ran*/){
							GLM linearModel;
							return CalculateVarianceComponents (/*geneticData,*/ ANOVAData, factorList,dependentColNames, 
											/* reps, ran, */linearModel);
							}
	ObjList<aList<long double> >* CalculateVarianceComponents (/*PopulationGeneticData& geneticData, */
								dataSetRec& ANOVAData, StringList& factorList, 
								StringList& dependentColNames,/* long reps, Ran3& ran,*/ GLM& linearModel);
					//factorList holds the analysis inut lines
	void 		ConvertTitleString (aString& theTitleStr, StringList& theList);
	dataSetRec*	 CreateRawANOVADataFormat(PopulationGeneticData& geneticData, StringList& dependentColNames);
					//returns a data set that can be analyzed using linear algebra
					//missing data represented by '-1'
					//missing data must be culled elsewhere before analysis
	
	dataSetRec*	 CreateRawANOVADataFormat(CodominantMarkerData& geneticData, StringList& dependentColNames);
					//returns a data set that can be analyzed using linear algebra
					//missing data represented by '-1'
					//missing data must be culled elsewhere before analysis
	
	dataSetRec*	 CreateRawANOVADataFormat(DominantMarkerData2& geneticData, StringList& dependentColNames);
					//returns a data set that can be analyzed using linear algebra
					//missing data represented by '-1'
					//missing data must be culled elsewhere before analysis
	
	dataSetRec*	 CreateRawANOVADataFormat(SexLinkedCodomMarkerData& geneticData, StringList& dependentColNames);
					//returns a data set that can be analyzed using linear algebra
					//missing data represented by '-1'
					//missing data must be culled elsewhere before analysis
	


	
};//end ANOVAGenetics





















#endif
