// FstatisticAnalysis.h

#ifndef FstatisticAnalysish
#define FstatisticAnalysish = 1


#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "MemoryManager.h"
#include "PopulationGeneticData.h"
//#include "FstatMethodType.h"

//enum FstatMethodType {WandC=0,hierWandC=1,linearAlg=2,hierLinearAlg=3,linearIntrog=4,linearIntrogNoInt=5,factorialFstats=6,factorialFstatsII=7,factorialFstatsIII=8,linearAlgII = 9, domStd=10, hierDom=11, unspecified=12};

class FstatisticAnalysis: public virtual PopulationGeneticData{
  public:	//FstatMethodType values
		  const static int unspecified =0;
		  const static int WandC =1;
		  const static int hierWandC =2;
		  const static int linearAlg =3;
		  const static int hierLinearAlg =4;
		  const static int linearIntrog =5;
		  const static int linearIntrogNoInt =6;
		  const static int factorialFstats =7;
		  const static int factorialFstatsII =8;
		  const static int factorialFstatsIII =9;
		  const static int linearAlgII =10;
		  //higher numbers in derived classes

  public:
	int FstatsCurrent_;//boolean
	aList<int>  Fmethod_; //all F-stats now sorted by method so >1 method's results can be stored 
	ObjList<StringList> Flevels_;//list of Fit,Fst, Fis, etc sequence
	ObjList<aList<long double> > Fstats_;//the values in sequence corresponding to the F list above
	ObjList<ObjList<aList<long double> > > FstatsLoc_, jackFstatsLoc_;
		//list of loci, each with F statistics in sequence above
	ObjList<ObjList<ObjList<aList<long double> > > > FstatsAll_;
		//list of loci, each with list of alleles, each with F statistics in sequence above

	int vcCurrent_;//boolean
	aList<int>  vcMethod_; //all variance comps sorted by method so >1 method's results can be stored 
	ObjList<StringList> vcLevels_;//list of variance components by name
	ObjList<aList<long double> > vc_;//the variance components in sequence corresponding to the vc list above
	ObjList<ObjList<aList<long double> > > vcLoc_;//the variance components by locus
	ObjList<ObjList<ObjList<aList<long double> > > > vcAll_;//the variance components by locus & allele

  public:
		FstatisticAnalysis(void): FstatsCurrent_(0), vcCurrent_(0) {
			PopulationGeneticData::Initialize();}
		FstatisticAnalysis(aString& s): PopulationGeneticData(s), 
				FstatsCurrent_(0), vcCurrent_(0) {
			PopulationGeneticData::Initialize(s);}
		FstatisticAnalysis(PopulationGeneticData& pgd): 
					PopulationGeneticData(pgd), 
					FstatsCurrent_(0), vcCurrent_(0) {
			PopulationGeneticData::Initialize(pgd);}
		virtual	~FstatisticAnalysis(void){
			Clear();
			}

		FstatisticAnalysis(FstatisticAnalysis& parentData);//copy constructor

		FstatisticAnalysis(FstatisticAnalysis& parentData, aList<int>& popSubsetNums);
		FstatisticAnalysis(PopulationGeneticData& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(FstatisticAnalysis& pgd);
	FstatisticAnalysis&	operator=(FstatisticAnalysis& pgd);

	void 		CalculateFactorialVarCompCoeffs(GLM& lm); //species x locality
	void 		CalculateFactorialVarCompCoeffsII(GLM& lm);//locality x species
	void 		CalculateFactorialVarCompCoeffsIII(aList<long double>& nList, aList<long double>& dfList);
	void 		CalculateFactorialVarCompCoeffsIII(aList<long double>& nList, aList<long double>& dfList, int loc);
	void 		CalculateStdVarCompCoeffs(aList<long double>& nList, aList<long double>& dfList);
	void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					ClearFstats();
					PopulationGeneticData::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampled,ran);
					}
	void 		CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
					ClearFstats();
					PopulationGeneticData::CreateRandomData(Fst,Fis,pBar,numIndivsPerPopLoc,ran);
					}
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					ClearFstats();
					PopulationGeneticData::CreateRandomData(Fspt,Flt,Fis,Flspt,pBar,numSpeciesSampled,numLocalitiesSampled,numIndivsPerPop,numLociSampled,ran);
					}
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					ClearFstats();
					PopulationGeneticData::CreateRandomData(Fspt,Flt,Fis,pBar,numSpeciesSampled,numLocalitiesSampled,numIndivsPerPop,numLociSampled,ran);
					}
	void 		CreateRandomDataLocFirst(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					ClearFstats();
					PopulationGeneticData::CreateRandomData(Fspt,Flt,Fis,pBar,numSpeciesSampled,numLocalitiesSampled,numIndivsPerPop,numLociSampled,ran);
					}

	virtual void Clear();
	virtual void ClearFstats();//only clears arrays in this class, base classes intact
	long double 	dL_over_dS(const int Fmethod);
	long double 	dL_over_dS(const int Fmethod, int locus);
	long double 	dL_over_dSjack(const int Fmethod, int locus);//average dropping locus
	long double 	dL_over_dS(const int Fmethod, int locus, int allele);
	long double 	dL_over_d(const int Fmethod);
	long double 	dL_over_d(const int Fmethod, int locus);
	long double 	dL_over_djack(const int Fmethod, int locus);//average dropping locus
	long double 	dL_over_d(const int Fmethod, int locus, int allele);
	long double 	dS_over_d(const int Fmethod);
	long double 	dS_over_d(const int Fmethod, int locus);
	long double 	dS_over_djack(const int Fmethod, int locus);//average dropping locus
	long double 	dS_over_d(const int Fmethod, int locus, int allele);
	long double 	F(aString& Fword, const int Fmethod);
	long double 	F(aString& Fword, int loc, const int Fmethod);
	long double 	F(aString& Fword, int loc, int allele, const int Fmethod);
	long double 	Fis(const int Fmethod);
	long double 	Fis(int loc, const int Fmethod);
	long double 	Fis(int loc, int allele, const int Fmethod);
	long double 	Fit(const int Fmethod);
	long double 	Fit(int loc, const int Fmethod);
	long double 	Fit(int loc, int allele, const int Fmethod);
	long double 	Fst(const int Fmethod);
	long double 	Fst(int loc, const int Fmethod);
	long double 	Fst(int loc, int allele, const int Fmethod);
	long double 	JackF(aString& Fword, const int Fmethod);//jackknife stdev of F(Fword) above, taken over loci
	long double 	NL_over_N(const int Fmethod);
	long double 	NL_over_N(const int Fmethod, int locus);
	long double 	NL_over_Njack(const int Fmethod, int locus);//average dropping locus
	long double 	NL_over_N(const int Fmethod, int locus, int allele);
	long double 	NSp_over_N(const int Fmethod);
	long double 	NSp_over_N(const int Fmethod, int locus);
	long double 	NSp_over_Njack(const int Fmethod, int locus);//average dropping locus
	long double 	NSp_over_N(const int Fmethod, int locus, int allele);
	virtual int PrintFMethodHeading(ostream& outFile, const int Fmethod);
	void 		PrintFStatistics(ostream& outFile, const int Fmethod, char colSpacer = '\t');
	virtual void PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, char colSpacer = '\t');
	virtual void PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, char colSpacer = '\t');
	void 		PrintFactorialFstatsDerivedStats(aString& outFileName);
	void 		PrintFactorialFstatsDerivedStats(ostream& outFile);
	int 		PrintVCmethodHeading(ostream& outFile, const int vcMethod);
	void 		PrintVarianceComponents(ostream& outFile, const int vcMethod, char colSpacer='\t');
	void 		PrintVarianceComponentsByLocus(ostream& outFile, const int vcMethod, char colSpacer='\t');
	void 		PrintVarianceComponentsByAllele(ostream& outFile, const int vcMethod, char colSpacer='\t');
	virtual void	ResetFlags(void);
	void		WeirTheta (void);
	void		HierarchicalWeirTheta(void);
	void 		FstatsLinearAlg(void);//uses linear algebra to get variance components for F-statistics
	void 		FstatsLinearAlg(ObjList<GLM>& LinearModelList);//uses linear algebra to get variance components for F-statistics
	void 		FstatsLinearAlgII(void);
	void 		FstatsIntrogression(int includeInteraction=1);
					//uses linear algebra to get variance components for F-statistics
					//in a factorial, nested design
	void 		FstatsIntrogression(ObjList<GLM>& LinearModelList, int includeInteraction = 1);
					//uses linear algebra to get variance components for F-statistics
					//in a factorial, nested design
	void  	FactorialFstats(ObjList<GLM>& LinearModelList, int includeInteraction);

	void 		FactorialFstats(void);
					//uses quick computational forms where possible to get variance components for F-statistics
					//in a factorial, nested design
	void  	FactorialFstatsII(ObjList<GLM>& LinearModelList, int includeInteraction);
					//this method estimates locality after species variance component
	void 		FactorialFstatsII(void);
					//this method estimates locality after species variance component
					//uses quick computational forms where possible to get variance components for F-statistics
					//in a factorial, nested design
	void 		FactorialFstatsIII(void);
	void 		FactorialFstatsIIIGuts(dataSetRec& rawAnovaData, 
										ObjList<aList<long double> >& varianceComponents, 
										aList<int>& locusNums, StringList& locusByAlleleNames);

};


#endif
