// FstatisticAnalysisCodom.h

#ifndef FstatisticAnalysisCodomh
#define FstatisticAnalysisCodomh = 1


#include "StringClass.h"
#include "StringList.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "MemoryManager.h"
#include "ANOVAGenetics.h"
#include "CodominantMarkerData.h"
#include "EvolutionaryGeneticsData.h"
#include "FstatisticAnalysisCore.h"
//#include "FstatMethodType.h"

//20081202: updated Nm() to allow for scaling by the number of demes, and for zygote vs gamete migration
//20081202: updated NmFromDiploidFst() to allow for scaling by the number of demes, and for zygote vs gamete migration
//20081202: updated DiploidFstFromNm() to allow for scaling by the number of demes, and for zygote vs gamete migration

//enum FstatMethodType {WandC=0,hierWandC=1,linearAlg=2,hierLinearAlg=3,linearIntrog=4,linearIntrogNoInt=5,factorialFstats=6,factorialFstatsII=7,factorialFstatsIII=8,linearAlgII = 9, domStd=10, hierDom=11, unspecified=12};

class FstatisticAnalysisCodom: 
					public virtual FstatisticAnalysisCore,
					public virtual CodominantMarkerData,
					public virtual EvolutionaryGeneticsData{

  public:
		FstatisticAnalysisCodom(void){
			CodominantMarkerData::Initialize();
			FstatisticAnalysisCore::Initialize();
			}
		FstatisticAnalysisCodom(aString& s){
			CodominantMarkerData::Initialize(s);
			FstatisticAnalysisCore::Initialize();
			}
		FstatisticAnalysisCodom(CodominantMarkerData& pgd){
			CodominantMarkerData::Initialize(pgd);
			FstatisticAnalysisCore::Initialize();
			}
		virtual	~FstatisticAnalysisCodom(void){
			Clear();
			}

		FstatisticAnalysisCodom(FstatisticAnalysisCodom& parentData);//copy constructor

		FstatisticAnalysisCodom(CodominantMarkerData& parentData, aList<int>& popSubsetNums);
		FstatisticAnalysisCodom(FstatisticAnalysisCodom& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(FstatisticAnalysisCodom& pgd);
	void		Initialize(FstatisticAnalysisCodom& parentData, aList<int>& popSubsetNums);
	FstatisticAnalysisCodom&	operator=(FstatisticAnalysisCodom& pgd);
	virtual EvolutionaryGeneticsData* Duplicate(void){
						return NEW FstatisticAnalysisCodom(*this);}

	void 		CalculateFactorialVarCompCoeffs(GLM& lm); //species x locality
	void 		CalculateFactorialVarCompCoeffsII(GLM& lm);//locality x species
	void 		CalculateFactorialVarCompCoeffsIII(aList<long double>& nList, aList<long double>& dfList);
	void 		CalculateFactorialVarCompCoeffsIII(aList<long double>& nList, aList<long double>& dfList, int loc);
	void 		CalculateStdVarCompCoeffs(aList<long double>& nList, aList<long double>& dfList);
	void		CalculateIBDestimates(Ran3& ran,int sampleSize);//sampled
	void		CalculateIBDexhaustive(void);//only do this for very small population samples
	virtual void Clear();
	virtual void ClearFstats(void){//only clears arrays in this class, base classes intact
					FstatisticAnalysisCore::ClearFstats();}
	virtual void ClearIBDs(void);
	void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					ClearFstats();
					CodominantMarkerData::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampled,ran);
					}
	void 		CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
					ClearFstats();
					CodominantMarkerData::CreateRandomData(Fst,Fis,pBar,numIndivsPerPopLoc,ran);
					}
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					ClearFstats();
					CodominantMarkerData::CreateRandomData(Fspt,Flt,Fis,Flspt,pBar,numSpeciesSampled,numLocalitiesSampled,numIndivsPerPop,numLociSampled,ran);
					}
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					ClearFstats();
					CodominantMarkerData::CreateRandomData(Fspt,Flt,Fis,pBar,numSpeciesSampled,numLocalitiesSampled,numIndivsPerPop,numLociSampled,ran);
					}
	void 		CreateRandomDataLocFirst(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					ClearFstats();
					CodominantMarkerData::CreateRandomData(Fspt,Flt,Fis,pBar,numSpeciesSampled,numLocalitiesSampled,numIndivsPerPop,numLociSampled,ran);
					}

	long double DiploidFstFromNm(long double Nm, int numTrueDemes=0, bool zygoteMigration=false);
	void 		FstatsLinearAlg(void);//uses linear algebra to get variance components for F-statistics
	void 		FstatsLinearAlg(ObjList<GLM>& LinearModelList);//uses linear algebra to get variance components for F-statistics
	void 		FstatsLinearAlgII(void);
	void 		FstatsIntrogression(int includeInteraction=1);
					//uses linear algebra to get variance components for F-statistics
					//in a factorial, nested design
	void 		FstatsIntrogression(ObjList<GLM>& LinearModelList, int includeInteraction = 1);
					//uses linear algebra to get variance components for F-statistics
					//in a factorial, nested design
	void  	FactorialFstats(ObjList<GLM>& LinearModelList, int includeInteraction);

	void 		FactorialFstats(void);
					//uses quick computational forms where possible to get variance components for F-statistics
					//in a factorial, nested design
	void  	FactorialFstatsII(ObjList<GLM>& LinearModelList, int includeInteraction);
					//this method estimates locality after species variance component
	void 		FactorialFstatsII(void);
					//this method estimates locality after species variance component
					//uses quick computational forms where possible to get variance components for F-statistics
					//in a factorial, nested design
	void 		FactorialFstatsIII(void);
	void 		FactorialFstatsIIIGuts(dataSetRec& rawAnovaData, 
										ObjList<aList<long double> >& varianceComponents, 
										aList<int>& locusNums, StringList& locusByAlleleNames);
	void		HierarchicalWeirTheta(void);
	
	virtual long double Nm(long double FstVal,int /*Fmethod=WandC*/, int numTrueDemes=0, bool zygotMigration=false);//{//use default 4Nm method

	long double 	NmFromDiploidFst(long double FstDip, int /*Fmethod*/, int numTrueDemes=0, bool zygoteMigration=false);
							//numTrueDemes is whether to weigh by a known finite number of demes (not sampled number)
							//numTrueDemes<2 gives infinite island model; also numTrueDemes = MAX(numTrueDemes,npop_);

	virtual void	ResetFlags(void);
	void		WeirTheta (void);
//	virtual int PrintFMethodHeading(ostream& outFile, const int Fmethod);
//	void 		PrintFStatistics(ostream& outFile, const int Fmethod, char colSpacer = '\t');
	virtual void PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');
	virtual void PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');
//	void 		PrintFactorialFstatsDerivedStats(aString& outFileName);
//	void 		PrintFactorialFstatsDerivedStats(ostream& outFile);
//	int 		PrintVCmethodHeading(ostream& outFile, const int vcMethod);
//	void 		PrintVarianceComponents(ostream& outFile, const int vcMethod, char colSpacer='\t');
//	void 		PrintVarianceComponentsByLocus(ostream& outFile, const int vcMethod, char colSpacer='\t');
//	void 		PrintVarianceComponentsByAllele(ostream& outFile, const int vcMethod, char colSpacer='\t');

};


#endif
