// FstatisticAnalysisCodomSexLink.h

//only use for haplodiploid species, or X-linked loci of diploid species

#ifndef FstatisticAnalysisCodomSexLinkh
#define FstatisticAnalysisCodomSexLinkh = 1


#include "InlineTemplates.h"
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "MemoryManager.h"
#include "ANOVAGenetics.h"
#include "SexLinkedCodomMarkerData.h"
#include "EvolutionaryGeneticsData.h"
#include "FstatisticAnalysisCore.h"
//#include "FstatMethodType.h"

//20081202: updated Nm() to allow for scaling by the number of demes, and for zygote vs gamete migration
//20081202: updated NmFromSexLinkedCodomFst() to allow for scaling by the number of demes, and for zygote vs gamete migration
//20081202: updated SexLinkedCodomFstFromNm() to allow for scaling by the number of demes, and for zygote vs gamete migration
//20081209: updated SexLinkedCodomMarkerFstats() to allow effective Fst to be adjusted the number of demes, and for zygote vs gamete migration
//20081229  fixed a bug in SexLinkedCodomFstFromNm(long double, int, bool) that affects cases where primarySexRatio_!=0.5

class FstatisticAnalysisCodomSexLink: 
					public virtual FstatisticAnalysisCore,
					public virtual SexLinkedCodomMarkerData,
					public virtual EvolutionaryGeneticsData{

  public://more FstatMethodTypes
  		const static int sexLinkStd=13;
		const static int sexLinkHier=14;//not supported yet 
  		const static int sexLinkStdEffective=23;//equivalent values for codomnant diploid

		  const static int IBDexhaustSLCodom=105;
		  const static int IBDestimSLCodom=106;
  public:
		FstatisticAnalysisCodomSexLink(void){
			SexLinkedCodomMarkerData::Initialize();
			FstatisticAnalysisCore::Initialize();
			}
		FstatisticAnalysisCodomSexLink(aString& s){
			SexLinkedCodomMarkerData::Initialize(s);
			FstatisticAnalysisCore::Initialize();
			}
		FstatisticAnalysisCodomSexLink(SexLinkedCodomMarkerData& pgd){
			SexLinkedCodomMarkerData::Initialize(pgd);
			FstatisticAnalysisCore::Initialize();
			}
		virtual	~FstatisticAnalysisCodomSexLink(void){
			Clear();
			}

		FstatisticAnalysisCodomSexLink(FstatisticAnalysisCodomSexLink& parentData);//copy constructor

		FstatisticAnalysisCodomSexLink(SexLinkedCodomMarkerData& parentData, aList<int>& popSubsetNums);
		FstatisticAnalysisCodomSexLink(FstatisticAnalysisCodomSexLink& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(SexLinkedCodomMarkerData& pgd);
	void		Initialize(FstatisticAnalysisCodomSexLink& pgd);
	void		Initialize(FstatisticAnalysisCodomSexLink& parentData, aList<int>& popSubsetNums);
	FstatisticAnalysisCodomSexLink&	operator=(FstatisticAnalysisCodomSexLink& pgd);
	virtual EvolutionaryGeneticsData* Duplicate(void){
						return NEW FstatisticAnalysisCodomSexLink(*this);}

	void 		CalculateFactorialVarCompCoeffs(GLM& lm); //species x locality
	void 		CalculateFactorialVarCompCoeffsII(GLM& lm);//locality x species
	void 		CalculateFactorialVarCompCoeffsIII(aList<long double>& nList, aList<long double>& dfList);
	void 		CalculateFactorialVarCompCoeffsIII(aList<long double>& nList, aList<long double>& dfList, int loc);
	void 		CalculateIBDestimates(Ran3& ran,int sampleSize);//sampled with replacement, per locus
	void 		CalculateIBDexhaustive(void);//only do this for very small population samples
	void 		CalculateStdVarCompCoeffs(aList<long double>& nList, aList<long double>& dfList);
	virtual void ClearIBDs(void);
	virtual void Clear();
	void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					ClearFstats();
					ClearIBDs();
					SexLinkedCodomMarkerData::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampled,ran);
					}
	void 		CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
					ClearFstats();
					ClearIBDs();
					SexLinkedCodomMarkerData::CreateRandomData(Fst,Fis,pBar,numIndivsPerPopLoc,ran);
					}
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					ClearFstats();
					ClearIBDs();
					SexLinkedCodomMarkerData::CreateRandomData(Fspt,Flt,Fis,Flspt,pBar,numSpeciesSampled,numLocalitiesSampled,numIndivsPerPop,numLociSampled,ran);
					}
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					ClearFstats();
					ClearIBDs();
					SexLinkedCodomMarkerData::CreateRandomData(Fspt,Flt,Fis,pBar,numSpeciesSampled,numLocalitiesSampled,numIndivsPerPop,numLociSampled,ran);
					}
	void 		CreateRandomDataLocFirst(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					ClearFstats();
					ClearIBDs();
					SexLinkedCodomMarkerData::CreateRandomData(Fspt,Flt,Fis,pBar,numSpeciesSampled,numLocalitiesSampled,numIndivsPerPop,numLociSampled,ran);
					}
	long double	EffectiveFstFromSLcodomFst(long double Fstval);
							//the corresponding value of Fst for a diploid codominant locus 
							//scaling by migration mode and deme number is accounted for in deriving the ratio underlying this conversion

	void 		FstatsLinearAlg(void);//uses linear algebra to get variance components for F-statistics
	void 		FstatsLinearAlg(ObjList<GLM>& LinearModelList);//uses linear algebra to get variance components for F-statistics
	void 		FstatsLinearAlgII(void);
	void 		FstatsIntrogression(int includeInteraction=1);
					//uses linear algebra to get variance components for F-statistics
					//in a factorial, nested design
	void 		FstatsIntrogression(ObjList<GLM>& LinearModelList, int includeInteraction = 1);
					//uses linear algebra to get variance components for F-statistics
					//in a factorial, nested design
	void  	FactorialFstats(ObjList<GLM>& LinearModelList, int includeInteraction);

	void 		FactorialFstats(void);
					//uses quick computational forms where possible to get variance components for F-statistics
					//in a factorial, nested design
	void  	FactorialFstatsII(ObjList<GLM>& LinearModelList, int includeInteraction);
					//this method estimates locality after species variance component
	void 		FactorialFstatsII(void);
					//this method estimates locality after species variance component
					//uses quick computational forms where possible to get variance components for F-statistics
					//in a factorial, nested design
	void 		FactorialFstatsIII(void);
	void 		FactorialFstatsIIIGuts(dataSetRec& rawAnovaData, 
										ObjList<aList<long double> >& varianceComponents, 
										aList<int>& locusNums, StringList& locusByAlleleNames);
	virtual long double Nm(long double FstVal, int /*Fmethod*/, int numTrueDemes=0, bool zygoteMigration=false){
							return NmFromSexLinkedCodomFst(FstVal,numTrueDemes, zygoteMigration);}
	long double 	NmFromSexLinkedCodomFst(long double FstSLCodom, int numTrueDemes=0, bool zygoteMigration=false);
	virtual int PrintFMethodHeading(ostream& outFile, const int Fmethod);
	virtual void PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');
	virtual void PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');
	virtual void	ResetFlags(void);
	long double SexLinkedCodomFstFromNm(long double Nm, int numTrueDemes=0, bool zygoteMigration=false);
	
	void 		SexLinkedCodomMarkerFstats(int numTrueDemes=0, bool zygoteMigration=false);

};


#endif
