// FstatisticAnalysisCore.h

#ifndef FstatisticAnalysisCoreh
#define FstatisticAnalysisCoreh = 1


#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "StringList.h"
#include "INlineTemplates.h"
#include "MemoryManager.h"
#include <fstream.h>
//#include "EvolutionaryGeneticsData.h"
//#include "FstatMethodType.h"

//enum FstatMethodType {WandC=0,hierWandC=1,linearAlg=2,hierLinearAlg=3,linearIntrog=4,linearIntrogNoInt=5,factorialFstats=6,
//factorialFstatsII=7,factorialFstatsIII=8,linearAlgII = 9, domStd=10, hierDom=11, unspecified=12};

//20081209:  Added function EffectiveFstFromNm()
//20081209:  Added function NmFromEffectiveFst()

class FstatisticAnalysisCore{
  public:	//FstatMethodType values
		  const static int unspecified =0;
		  const static int WandC =1;
		  const static int hierWandC =2;
		  const static int linearAlg =3;
		  const static int hierLinearAlg =4;
		  const static int linearIntrog =5;
		  const static int linearIntrogNoInt =6;
		  const static int factorialFstats =7;
		  const static int factorialFstatsII =8;
		  const static int factorialFstatsIII =9;
		  const static int linearAlgII =10;
		  //higher numbers in derived classes
			
		  const static int IBDexhaustCodom=100;
		  const static int IBDestimCodom=101;
		  
  public:
	int FstatsCurrent_;//boolean
	aList<int>  Fmethod_; //all F-stats now sorted by method so >1 method's results can be stored 
	ObjList<StringList> Flevels_;//list of Fit,Fst, Fis, etc sequence
	ObjList<aList<long double> > Fstats_;//the values in sequence corresponding to the F list above [Fmethod][Flevel]
	ObjList<ObjList<aList<long double> > > FstatsLoc_, jackFstatsLoc_;
		//list of loci, each with F statistics in sequence above [Fmethod][loc][Flevel]
	ObjList<ObjList<ObjList<aList<long double> > > > FstatsAll_;
		//list of loci, each with list of alleles, each with F statistics in sequence above [Fmethod][loc][allele][Flevel]

	int vcCurrent_;//boolean
	aList<int>  vcMethod_; //all variance comps sorted by method so >1 method's results can be stored 
	ObjList<StringList> vcLevels_;//list of variance components by name
	ObjList<aList<long double> > vc_;//the variance components in sequence corresponding to the vc list above
	ObjList<ObjList<aList<long double> > > vcLoc_;//the variance components by locus
	ObjList<ObjList<ObjList<aList<long double> > > > vcAll_;//the variance components by locus & allele

	int IBDcurrent_;//boolean
	aList<int>  IBDmethod_; //all IBD comps sorted by method so >1 method's results can be stored 
	ObjList<StringList> IBDlevels_;//list of IBD components by name
					// "withinIndivs"
					// "betwIndivsInPops"
					// "betwIndivsBetwPops"
	ObjList<aList<long double> > IBDvals_;//the IBD components in sequence corresponding to the IBD list above
	ObjList<ObjList<aList<long double> > > IBDvalsByLoc_, jackIBDvalsLoc_;//the IBD components by locus [IBDmethod][loc][IBDlevel]

//	long double ;//modify later for hierarchical, factorial, etc. structures
//	long double IBD_betwIndivInPop_;
//	long double IBD_betwIndivBetwPops_;
//	aList<long double> IBD_inIndiv_ByLoc_;//modify later for hierarchical, factorial, etc. structures
//	aList<long double> IBD_betwIndivInPop_ByLoc_;
//	aList<long double> IBD_betwIndivBetwPops_ByLoc_;
  public:
		FstatisticAnalysisCore(void): FstatsCurrent_(0), vcCurrent_(0), IBDcurrent_(0) {}

		virtual	~FstatisticAnalysisCore(void){
			Clear();
			}

		FstatisticAnalysisCore(FstatisticAnalysisCore& parentData);//copy constructor

	void 			Initialize(void);
	void 			Initialize(FstatisticAnalysisCore& pgd);
	FstatisticAnalysisCore&	operator=(FstatisticAnalysisCore& pgd);


	virtual void Clear();
	virtual void ClearFstats();//only clears arrays in this class, base classes intact
	virtual void ClearIBDs();//only clears arrays in this class, base classes intact
	long double 	dL_over_dS(const int Fmethod);
	long double 	dL_over_dS(const int Fmethod, int locus);
	long double 	dL_over_dSjack(const int Fmethod, int locus);//average dropping locus
	long double 	dL_over_dS(const int Fmethod, int locus, int allele);
	long double 	dL_over_d(const int Fmethod);
	long double 	dL_over_d(const int Fmethod, int locus);
	long double 	dL_over_djack(const int Fmethod, int locus);//average dropping locus
	long double 	dL_over_d(const int Fmethod, int locus, int allele);
	long double 	dS_over_d(const int Fmethod);
	long double 	dS_over_d(const int Fmethod, int locus);
	long double 	dS_over_djack(const int Fmethod, int locus);//average dropping locus
	long double 	dS_over_d(const int Fmethod, int locus, int allele);
	long double		EffectiveFstFromNm(long double Nmval, int numTrueDemes=0, bool zygoteMigration=false);
	long double 	F(aString& Fword, const int Fmethod);
	long double 	F(aString& Fword, int loc, const int Fmethod);
	long double 	F(aString& Fword, int loc, int allele, const int Fmethod);
	long double 	Fis(const int Fmethod);
	long double 	Fis(int loc, const int Fmethod);
	long double 	Fis(int loc, int allele, const int Fmethod);
	long double 	Fit(const int Fmethod);
	long double 	Fit(int loc, const int Fmethod);
	long double 	Fit(int loc, int allele, const int Fmethod);
	long double 	Fst(const int Fmethod);
	long double 	Fst(int loc, const int Fmethod);
	long double 	Fst(int loc, int allele, const int Fmethod);
	long double 	JackF(aString& Fword, const int Fmethod);//jackknife stdev of F(Fword) above, taken over loci
	long double 	JackIBD(aString& IBDword, const int IBDmethod);//jackknife stdev of F(Fword) above, taken over loci
	long double 	IBD(aString& IBDword, const int IBDmethod);
					//basic IBDwords are:
					// "withinIndivs"
					// "betwIndivsInPops"
					// "betwIndivsBetwPops"	
	long double 	IBD(aString& IBDword, int loc, const int IBDmethod);
	long double 	IBD_inIndiv(const int IBDmethod);
	long double 	IBD_betwIndivInPop(const int IBDmethod);
	long double 	IBD_betwIndivBetwPops(const int IBDmethod);
	long double 	IBD_inIndiv(int loc, const int IBDmethod);//modify later for hierarchical, factorial, etc. structures
	long double 	IBD_betwIndivInPop(int loc, const int IBDmethod);
	long double 	IBD_betwIndivBetwPops(int loc, const int IBDmethod);
	long double 	NL_over_N(const int Fmethod);
	long double 	NL_over_N(const int Fmethod, int locus);
	long double 	NL_over_Njack(const int Fmethod, int locus);//average dropping locus
	long double 	NL_over_N(const int Fmethod, int locus, int allele);

	virtual long double Nm(long double /*FstVal*/,int /*Fmethod=unspecified*/, int /*numTrueDemes = 0*/, bool /*zygoteMigration=false*/){
								//override in derived classes
								aError("calculate Nm using drived classes, not FstatisticAnalysisCore");
								return -one;}
					//			return NmFromDiploidFst(FstVal,Fmethod,numTrueDemes,zygoteMigration);}
	long double		NmFromEffectiveFst(long double Fstval, int numTrueDemes=false, bool zygoteMigration=false);

	long double 	NSp_over_N(const int Fmethod);
	long double 	NSp_over_N(const int Fmethod, int locus);
	long double 	NSp_over_Njack(const int Fmethod, int locus);//average dropping locus
	long double 	NSp_over_N(const int Fmethod, int locus, int allele);
	virtual int PrintFMethodHeading(ostream& outFile, const int Fmethod);
	virtual void 		PrintFStatistics(ostream& outFile, aString& titleStr, char colSpacer = '\t', 
															int numTrueDemes=0, bool zygoteMigration=false);//all methods
	virtual void 		PrintFStatistics(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t',
															int numTrueDemes=0, bool zygoteMigration=false);
	virtual void PrintFStatisticsByLocus(ostream& , aString& , char ){}//dummy virtual function
	virtual void PrintFStatisticsByLocus(ostream& , const int , aString& , char ){}//dummy virtual function
	virtual void PrintFStatisticsByLocus(ostream& outFile, aString& titleStr, StringList& locNames, char colSpacer = '\t');//all methods
	virtual void PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, StringList& locNames, char colSpacer = '\t');
	virtual void PrintFStatisticsByAllele(ostream& , aString& , char ){}//dummy virtual function
	virtual void PrintFStatisticsByAllele(ostream& , const int , aString& , char ){}//dummy virtual function
	virtual void PrintFStatisticsByAllele(ostream& outFile, aString& titleStr,  
														StringList& locNames, ObjList<StringList>& alleleNames, char colSpacer = '\t');//all methods
	virtual void PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, 
														StringList& locNames, ObjList<StringList>& alleleNames, char colSpacer = '\t');
	void 		PrintFactorialFstatsDerivedStats(aString& outFileName, aString& titleStr,aList<int>& variableLoci,StringList& locNames, ObjList<StringList>& alleleNames);
	void 		PrintFactorialFstatsDerivedStats(ostream& outFile, aString& titleStr,aList<int>& variableLoci,StringList& locNames, ObjList<StringList>& alleleNames);
	virtual int PrintIBDMethodHeading(ostream& outFile, const int IBDmethod);
	virtual void 		PrintIBDs(ostream& outFile, aString& titleStr, char colSpacer = '\t');//all methods
	virtual void 		PrintIBDs(ostream& outFile, const int IBDmethod, aString& titleStr, char colSpacer = '\t');
	virtual void PrintIBDByLocus(ostream& , aString& , char ){}//dummy virtual function
	virtual void PrintIBDByLocus(ostream& , const int , aString& , char ){}//dummy virtual function
	virtual void PrintIBDByLocus(ostream& outFile, aString& titleStr, StringList& locNames, char colSpacer = '\t');//all methods
	virtual void PrintIBDByLocus(ostream& outFile, const int IBDmethod, aString& titleStr, StringList& locNames, char colSpacer = '\t');
	int 		PrintVCmethodHeading(ostream& outFile, const int vcMethod);
	void 		PrintVarianceComponents(ostream& outFile, const int vcMethod, aString& titleStr, char colSpacer='\t');
	void 		PrintVarianceComponentsByLocus(ostream& outFile, const int vcMethod, aString& titleStr, StringList& locNames, char colSpacer='\t');
	void 		PrintVarianceComponentsByAllele(ostream& outFile, const int vcMethod, aString& titleStr, 
														StringList& locNames, ObjList<StringList>& alleleNames, char colSpacer='\t');
	virtual void	ResetFlags(void);
	virtual void	ResetGeneticFlags(void) { ResetFlags();}
	long double 	varianceComponent(aString& Fword, const int Fmethod);
	long double 	varianceComponent(aString& Fword, int loc, const int Fmethod);
	long double 	varianceComponent(aString& Fword, int loc, int allele, const int Fmethod);

};


#endif
