// FstatisticAnalysisCytoFemale.h

//Fstats for cytoplasmic markers inherited through females

#ifndef FstatisticAnalysisCytoFemaleh
#define FstatisticAnalysisCytoFemaleh = 1


#include <ostream.h>
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "MemoryManager.h"
#include "EvolutionaryGeneticsData.h"
#include "CytoplasmicFemaleMarkerData.h"
#include "FstatisticAnalysisCore.h"

//20081202: updated Nm() to adjust for the number of demes in the species
//20081202: updated NmFromCytoFemaleFst() to adjust for the number of demes in the species
//20081202: updated CytoplasmicFstFromNm() to adjust for the number of demes in the species
//20081209: updated CytoFemaleMarkerFstats() to allow effective Fst to be adjusted the number of demes
//20081229	fixed bug in Nm(long double, int, int, bool)


class FstatisticAnalysisCytoFemale: public virtual FstatisticAnalysisCore,
							  public virtual CytoplasmicFemaleMarkerData,
							  public virtual EvolutionaryGeneticsData {
  public://more FstatMethodTypes
  		const static int CYfemale=25;
		const static int hierCYfemale=26;
  		const static int CYfemaleEffective=27;//equivalent values for codominant diploid

		  const static int IBDexhaustCytoFemale=107;
		  const static int IBDestimCytoFemale=108;

  public:
		FstatisticAnalysisCytoFemale(void){
						FstatisticAnalysisCore::Initialize();
						CytoplasmicFemaleMarkerData::Initialize();
						EvolutionaryGeneticsData::Initialize();
						}
		FstatisticAnalysisCytoFemale(aString& s);
		FstatisticAnalysisCytoFemale(EvolutionaryGeneticsData& egd);
		FstatisticAnalysisCytoFemale(CytoplasmicFemaleMarkerData& cyfmd);
		FstatisticAnalysisCytoFemale(FstatisticAnalysisCytoFemale& cyfmd);//copy constructor

			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
		FstatisticAnalysisCytoFemale(FstatisticAnalysisCytoFemale& parentData, aList<int>& popSubsetNums);
		FstatisticAnalysisCytoFemale(CytoplasmicFemaleMarkerData& parentData, aList<int>& popSubsetNums);


		virtual	~FstatisticAnalysisCytoFemale(void){
			Clear();
			}

	void 	Initialize(FstatisticAnalysisCytoFemale& cyfmd);//copies


	virtual void 		Clear(void);
	virtual void CalculateIBDestimates(Ran3& ran,int sampleSize);
	virtual void CalculateIBDexhaustive(void);
	virtual void ClearIBDs(void);
	virtual void CreateBootstrapData(CytoplasmicFemaleMarkerData& origData, Ran3& ran){
						//individuals resampled keeping multilocus genotypes intact
						CytoplasmicFemaleMarkerData::CreateBootstrapData(origData, ran);}
	virtual void CreateRandomData(const long double Fst, 
						const long double Fis,
						const long double pBar, 
						const int numPopsSampled,
						const int numIndivsPerPop,
						const int numLociSampled,
						Ran3List& ran){
					CytoplasmicFemaleMarkerData::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampled,ran);
					}
	virtual void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
					CytoplasmicFemaleMarkerData::CreateRandomData(Fst,Fis,pBar,numIndivsPerPopLoc,ran);
					}

	virtual void	ResetFlags(void);
	long double CytoplasmicFstFromNm(long double Nm, int numTrueDemes=0);
	void		CytoFemaleMarkerFstats (int numTrueDemes=0);
	void		HierarchicalCytoFemaleMarkerFstats(void);
	virtual int loci(void){return CytoplasmicFemaleMarkerData::loci();}

	virtual long double Nm(long double FstVal, int Fmethod, int numTrueDemes=0, bool zygoteMigration=false);
	long double NmFromCytoFemaleFst(long double FstCYF, int numTrueDemes=0);//,bool zygoteMigration=false);

	virtual int PrintFMethodHeading(ostream& outFile, const int Fmethod);
	virtual void PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');
	virtual void PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');
	virtual void PrintNEXUSFile(aString& outFileName, StringList& textToAppend);
	virtual void PrintNEXUSFile (ostream& outFile, StringList& textToAppend);



	virtual void ReadAFLPDataFile (aString& dataFileName){
									//input file assumed closed
					CytoplasmicFemaleMarkerData::ReadAFLPDataFile(dataFileName);}
//	virtual void ReadAndAppendAFLPDataFile (aString& dataFileName){
//									//input file assumed closed
//					CytoplasmicFemaleMarkerData::ReadAndAppendAFLPDataFile(dataFileName);}
	FstatisticAnalysisCytoFemale& operator=(FstatisticAnalysisCytoFemale& cyfmd);
	FstatisticAnalysisCytoFemale& operator=(CytoplasmicFemaleMarkerData& cyfmd);
	FstatisticAnalysisCytoFemale& operator=(EvolutionaryGeneticsData& egd);

};


#endif
