// FstatisticAnalysisCytoMale.h

//Fstats for cytoplasmic markers inherited through males

#ifndef FstatisticAnalysisCytoMaleh
#define FstatisticAnalysisCytoMaleh = 1


#include <ostream.h>
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "MemoryManager.h"
#include "EvolutionaryGeneticsData.h"
#include "CytoplasmicMaleMarkerData.h"
#include "FstatisticAnalysisCore.h"

//20081202: updated Nm() to adjust for the number of demes in the species
//20081202: updated NmFromCytoMaleFst() to adjust for the number of demes in the species
//20081202: updated CytoplasmicFstFromNm() to adjust for the number of demes in the species
//20081209: updated CytoMaleMarkerFstats() to allow effective Fst to be adjusted the number of demes
//20081229	fixed bug in NmFromCytoMaleFst(long double, int)


class FstatisticAnalysisCytoMale: public virtual FstatisticAnalysisCore,
							  public virtual CytoplasmicMaleMarkerData,
							  public virtual EvolutionaryGeneticsData {
  public://more FstatMethodTypes
  		const static int CYmale=28;
		const static int hierCYmale=29;
  		const static int CYmaleEffective=30;//equivalent values for codominant diploid

		  const static int IBDexhaustCytoMale=109;
		  const static int IBDestimCytoMale=110;

  public:
		FstatisticAnalysisCytoMale(void){
						FstatisticAnalysisCore::Initialize();
						CytoplasmicMaleMarkerData::Initialize();
						EvolutionaryGeneticsData::Initialize();
						}
		FstatisticAnalysisCytoMale(aString& s);
		FstatisticAnalysisCytoMale(EvolutionaryGeneticsData& egd);
		FstatisticAnalysisCytoMale(CytoplasmicMaleMarkerData& cyfmd);
		FstatisticAnalysisCytoMale(FstatisticAnalysisCytoMale& cyfmd);//copy constructor

			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
		FstatisticAnalysisCytoMale(FstatisticAnalysisCytoMale& parentData, aList<int>& popSubsetNums);
		FstatisticAnalysisCytoMale(CytoplasmicMaleMarkerData& parentData, aList<int>& popSubsetNums);


		virtual	~FstatisticAnalysisCytoMale(void){
			Clear();
			}

	void 	Initialize(FstatisticAnalysisCytoMale& cyfmd);//copies


	void 		Clear(void);
	virtual void CalculateIBDestimates(Ran3& ran,int sampleSize);
	virtual void CalculateIBDexhaustive(void);
	virtual void ClearIBDs(void);
	virtual void CreateBootstrapData(CytoplasmicMaleMarkerData& origData, Ran3& ran){
						//individuals resampled keeping multilocus genotypes intact
						CytoplasmicMaleMarkerData::CreateBootstrapData(origData, ran);}
	virtual void CreateRandomData(const long double Fst, 
						const long double Fis,
						const long double pBar, 
						const int numPopsSampled,
						const int numIndivsPerPop,
						const int numLociSampled,
						Ran3List& ran){
					CytoplasmicMaleMarkerData::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampled,ran);
					}
	virtual void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
					CytoplasmicMaleMarkerData::CreateRandomData(Fst,Fis,pBar,numIndivsPerPopLoc,ran);
					}

	virtual void	ResetFlags(void);
	void		CytoMaleMarkerFstats (int numTrueDemes=0);
	void		HierarchicalCytoMaleMarkerFstats(void);
	long double CytoplasmicFstFromNm(long double Nm, int numTrueDemes=0);
	virtual int loci(void){return CytoplasmicMaleMarkerData::loci();}
	virtual long double Nm(long double FstVal,int Fmethod, int numTrueDemes=0, bool zygoteMigration=false);
	long double NmFromCytoMaleFst(long double FstCYM, int numTrueDemes=0);//, bool /*zygoteMigration=false*/);
	virtual int PrintFMethodHeading(ostream& outFile, const int Fmethod);
	virtual void PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');
	virtual void PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');
	virtual void PrintNEXUSFile(aString& outFileName, StringList& textToAppend);
	virtual void PrintNEXUSFile (ostream& outFile, StringList& textToAppend);



	virtual void ReadAFLPDataFile (aString& dataFileName){
									//input file assumed closed
					CytoplasmicMaleMarkerData::ReadAFLPDataFile(dataFileName);}
//	virtual void ReadAndAppendAFLPDataFile (aString& dataFileName){
//									//input file assumed closed
//					CytoplasmicMaleMarkerData::ReadAndAppendAFLPDataFile(dataFileName);}
	FstatisticAnalysisCytoMale& operator=(FstatisticAnalysisCytoMale& cyfmd);
	FstatisticAnalysisCytoMale& operator=(CytoplasmicMaleMarkerData& cyfmd);
	FstatisticAnalysisCytoMale& operator=(EvolutionaryGeneticsData& egd);

};


#endif
