// FstatisticAnalysisDom.h

#ifndef FstatisticAnalysisDomh
#define FstatisticAnalysisDomh = 1


#include <ostream.h>
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "MemoryManager.h"
#include "PopulationGeneticData.h"
#include "DominantMarkerData.h"
#include "FstatisticAnalysis.h"



class FstatisticAnalysisDom: public FstatisticAnalysis, 
							 public DominantMarkerData,
							 public virtual PopulationGeneticData {
  public://more FstatMethodTypes
  		const static int domStd=11;
		const static int hierDom=12;

  public:
		FstatisticAnalysisDom(void){
						FstatisticAnalysis::Initialize();
						DominantMarkerData::Initialize();
						PopulationGeneticData::Initialize();
						}
		FstatisticAnalysisDom(aString& s);
		FstatisticAnalysisDom(PopulationGeneticData& pgd);
		FstatisticAnalysisDom(DominantMarkerData& pgd);
		FstatisticAnalysisDom(FstatisticAnalysis& pgd);
		FstatisticAnalysisDom(FstatisticAnalysisDom& pgd);//copy constructor

			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
		FstatisticAnalysisDom(FstatisticAnalysisDom& parentData, aList<int>& popSubsetNums);
//		FstatisticAnalysisDom(FstatisticAnalysis& parentData, aList<int>& popSubsetNums);
		FstatisticAnalysisDom(DominantMarkerData& parentData, aList<int>& popSubsetNums);
//		FstatisticAnalysisDom(PopulationGeneticData& parentData, aList<int>& popSubsetNums);


		virtual	~FstatisticAnalysisDom(void){
			Clear();
			}


	void 		Clear();
	virtual void CreateBootstrapData(DominantMarkerData& origData, Ran3& ran){
						//individuals resampled keeping multilocus genotypes intact
						DominantMarkerData::CreateBootstrapData(origData, ran);}
	virtual void CreateRandomData(const long double Fst, 
						const long double Fis,
						const long double pBar, 
						const int numPopsSampled,
						const int numIndivsPerPop,
						const int numLociSampled,
						Ran3List& ran){
					DominantMarkerData::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampled,ran);
					}
	virtual void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
					DominantMarkerData::CreateRandomData(Fst,Fis,pBar,numIndivsPerPopLoc,ran);
					}

	virtual void	ResetFlags(void);
	void		DomMarkerFstats (void);
	void		HierarchicalDomMarkerFstats(void);
	virtual int loci(void){return DominantMarkerData::loci();}
	virtual int PrintFMethodHeading(ostream& outFile, const int Fmethod);
	virtual void PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, char colSpacer = '\t');
	virtual void PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, char colSpacer = '\t');
	virtual void PrintNEXUSFile(aString& outFileName, StringList& textToAppend);
	virtual void PrintNEXUSFile (ostream& outFile, StringList& textToAppend);



	virtual void ReadAFLPDataFile (aString& dataFileName){
									//input file assumed closed
					DominantMarkerData::ReadAFLPDataFile(dataFileName);}
	virtual void ReadAndAppendAFLPDataFile (aString& dataFileName){
									//input file assumed closed
					DominantMarkerData::ReadAndAppendAFLPDataFile(dataFileName);}
	FstatisticAnalysisDom& operator=(FstatisticAnalysisDom& pgd);
	FstatisticAnalysisDom& operator=(FstatisticAnalysis& pgd);
	FstatisticAnalysisDom& operator=(DominantMarkerData& pgd);
	FstatisticAnalysisDom& operator=(PopulationGeneticData& pgd);

};


#endif
