// FstatisticAnalysisDom2.h

#ifndef FstatisticAnalysisDom2h
#define FstatisticAnalysisDom2h = 1


#include <ostream.h>
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "MemoryManager.h"
#include "EvolutionaryGeneticsData.h"
#include "DominantMarkerData2.h"
#include "FstatisticAnalysisCore.h"

//20081202: updated Nm() to allow for scaling by the number of demes, and for zygote vs gamete migration
//20081202: updated NmFromDomFst() to allow for scaling by the number of demes, and for zygote vs gamete migration
//20081202: updated DomFstFromNm() to allow for scaling by the number of demes, and for zygote vs gamete migration
//20081209: updated DomMarkerFstats() to allow effective Fst to be adjusted the number of demes, and for zygote vs gamete migration

class FstatisticAnalysisDom2: public virtual FstatisticAnalysisCore,
							  public virtual DominantMarkerData2,
							  public virtual EvolutionaryGeneticsData {
  public://more FstatMethodTypes
  		const static int domStd=11;
		const static int hierDom=12;
  		const static int domStdEffective=22;//equivalent values for codominant diploid

  public:
		FstatisticAnalysisDom2(void){
						FstatisticAnalysisCore::Initialize();
						DominantMarkerData2::Initialize();
						EvolutionaryGeneticsData::Initialize();
						}
		FstatisticAnalysisDom2(aString& s);
		FstatisticAnalysisDom2(EvolutionaryGeneticsData& egd);
		FstatisticAnalysisDom2(DominantMarkerData2& dmd);
		FstatisticAnalysisDom2(FstatisticAnalysisDom2& dmd);//copy constructor

			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
		FstatisticAnalysisDom2(FstatisticAnalysisDom2& parentData, aList<int>& popSubsetNums);
		FstatisticAnalysisDom2(DominantMarkerData2& parentData, aList<int>& popSubsetNums);


		virtual	~FstatisticAnalysisDom2(void){
			Clear();
			}

	void 	Initialize(FstatisticAnalysisDom2& dmd);//copies


	void 		Clear();
	virtual void CreateBootstrapData(DominantMarkerData2& origData, Ran3& ran){
						//individuals resampled keeping multilocus genotypes intact
						DominantMarkerData2::CreateBootstrapData(origData, ran);}
	virtual void CreateRandomData(const long double Fst, 
						const long double Fis,
						const long double pBar, 
						const int numPopsSampled,
						const int numIndivsPerPop,
						const int numLociSampled,
						Ran3List& ran){
					DominantMarkerData2::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampled,ran);
					}
	virtual void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
					DominantMarkerData2::CreateRandomData(Fst,Fis,pBar,numIndivsPerPopLoc,ran);
					}

	virtual void	ResetFlags(void);

	long double DomFstFromNm(long double NmVal,int numTrueDemes=0, bool zygoteMigration=false);
	void		DomMarkerFstats (int numTrueDemes=0, bool zygoteMigration=false);
	void		HierarchicalDomMarkerFstats(void);
	virtual int loci(void){return DominantMarkerData2::loci();}
	virtual long double Nm(long double FstVal,int Fmethod, int numTrueDemes=0, bool zygoteMigration=false);
	long double NmFromDomFst(long double FstDom, int numTrueDemes=0, bool zygoteMigration=false);
	virtual int PrintFMethodHeading(ostream& outFile, const int Fmethod);
	virtual void PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer);
	virtual void PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer);
	virtual void PrintNEXUSFile(aString& outFileName, StringList& textToAppend);
	virtual void PrintNEXUSFile (ostream& outFile, StringList& textToAppend);



	virtual int ReadAFLPDataFile (aString& dataFileName){
									//input file assumed closed
					return DominantMarkerData2::ReadAFLPDataFile(dataFileName);}
//	virtual int ReadAndAppendAFLPDataFile (aString& dataFileName){
//									//input file assumed closed
//					return DominantMarkerData2::ReadAndAppendAFLPDataFile(dataFileName);}
	FstatisticAnalysisDom2& operator=(FstatisticAnalysisDom2& dmd);
	FstatisticAnalysisDom2& operator=(DominantMarkerData2& dmd);
	FstatisticAnalysisDom2& operator=(EvolutionaryGeneticsData& egd);

};


#endif
