// FstatisticAnalysisDomSexLink.h

#ifndef FstatisticAnalysisDomSexLinkh
#define FstatisticAnalysisDomSexLinkh = 1


#include <ostream.h>
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "MemoryManager.h"
#include "EvolutionaryGeneticsData.h"
#include "SexLinkedDomMarkerData2.h"
#include "FstatisticAnalysisCore.h"

//20081202: updated Nm() to allow for scaling by the number of demes, and for zygote vs gamete migration
//20081202: updated NmFromSexLinkedDomFst() to allow for scaling by the number of demes, and for zygote vs gamete migration
//20081202: updated SexLinkedDomFstFromNm() to allow for scaling by the number of demes, and for zygote vs gamete migration
//20081209: updated DomMarkerFstats() to allow effective Fst to be adjusted the number of demes, and for zygote vs gamete migration
//20081210: added a 'variance component' [#F(i,weighted)] to DomMarkerFstats() that permits averaging Fis over loci of different inheritance types
//20081229	fixed a bug in SexLinkedDomFstFromNm(long double, int, bool) that affects cases where primarySexRatio_!=0.5


class FstatisticAnalysisDomSexLink: public virtual FstatisticAnalysisCore,
							  public virtual SexLinkedDomMarkerData2,
							  public virtual EvolutionaryGeneticsData {
  public://more FstatMethodTypes
  		const static int sexLinkDomStd=15;
		const static int sexLinkHierDomStd=16;
  		const static int sexLinkDomStdEffective=24;//equivalent values for codominant diploid

  public:
		FstatisticAnalysisDomSexLink(void){
						FstatisticAnalysisCore::Initialize();
						SexLinkedDomMarkerData2::Initialize();
						EvolutionaryGeneticsData::Initialize();
						}
		FstatisticAnalysisDomSexLink(aString& s);
		FstatisticAnalysisDomSexLink(EvolutionaryGeneticsData& egd);
		FstatisticAnalysisDomSexLink(SexLinkedDomMarkerData2& dmd);
		FstatisticAnalysisDomSexLink(FstatisticAnalysisDomSexLink& dmd);//copy constructor

			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
		FstatisticAnalysisDomSexLink(FstatisticAnalysisDomSexLink& parentData, aList<int>& popSubsetNums);
		FstatisticAnalysisDomSexLink(SexLinkedDomMarkerData2& parentData, aList<int>& popSubsetNums);


		virtual	~FstatisticAnalysisDomSexLink(void){
			Clear();
			}

	void 	Initialize(FstatisticAnalysisDomSexLink& dmd);//copies


	void 		Clear();
	virtual void CreateBootstrapData(SexLinkedDomMarkerData2& origData, Ran3& ran){
						//individuals resampled keeping multilocus genotypes intact
						SexLinkedDomMarkerData2::CreateBootstrapData(origData, ran);}
	virtual void CreateRandomData(const long double Fst, 
						const long double Fis,
						const long double pBar, //recessive *marker* freq 
						const int numPopsSampled,
						const int numIndivsPerPop,
						const int numLociSampled,
						Ran3List& ran){
					SexLinkedDomMarkerData2::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampled,ran);
					}
	virtual void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles; recessive *marker* freq first
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
					SexLinkedDomMarkerData2::CreateRandomData(Fst,Fis,pBar,numIndivsPerPopLoc,ran);
					}

	virtual void	ResetFlags(void);
	void		DomMarkerFstats (int numTrueDemes=0, bool zygoteMigration=false);
	void		HierarchicalDomMarkerFstats(void);
	virtual int loci(void){return SexLinkedDomMarkerData2::loci();}
	virtual long double Nm(long double FstVal,int Fmethod, int numTrueDemes=0, bool zygoteMigration=false);
	long double 	NmFromSexLinkedDomFst(long double FstSLDom, int numTrueDemes=0, bool zygoteMigration=false);

	virtual int PrintFMethodHeading(ostream& outFile, const int Fmethod);
	virtual void PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');
	virtual void PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');
	virtual void PrintNEXUSFile(aString& outFileName, StringList& textToAppend);
	virtual void PrintNEXUSFile (ostream& outFile, StringList& textToAppend);



	virtual int ReadAFLPDataFile (aString& dataFileName){
									//input file assumed closed
					return SexLinkedDomMarkerData2::ReadAFLPDataFile(dataFileName);}
//	virtual void ReadAndAppendAFLPDataFile (aString& dataFileName){
//									//input file assumed closed
//					SexLinkedDomMarkerData2::ReadAndAppendAFLPDataFile(dataFileName);}
	long double SexLinkedDomFstFromNm(long double Nm,int numTrueDemes=0,bool zygoteMigration=false);


	FstatisticAnalysisDomSexLink& operator=(FstatisticAnalysisDomSexLink& dmd);
	FstatisticAnalysisDomSexLink& operator=(SexLinkedDomMarkerData2& dmd);
	FstatisticAnalysisDomSexLink& operator=(EvolutionaryGeneticsData& egd);

};


#endif
