// FstatisticAnalysisHap.h

#ifndef FstatisticAnalysisHaph
#define FstatisticAnalysisHaph = 1


#include <ostream.h>
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "MemoryManager.h"
#include "EvolutionaryGeneticsData.h"
#include "HaploidMarkerData.h"
#include "FstatisticAnalysisCore.h"


//20081202: updated Nm() to adjust for the number of demes in the species
//20081202: updated NmFromHaploid() to adjust for the number of demes in the species
//20081202: updated HaploidFstFromNm() to adjust for the number of demes in the species
//20081209: updated HapMarkerFstats() to allow effective Fst to be adjusted the number of demes

class FstatisticAnalysisHap: public virtual FstatisticAnalysisCore,
							  public virtual HaploidMarkerData,
							  public virtual EvolutionaryGeneticsData {
  public://more FstatMethodTypes
  		const static int hapStd=17;
		const static int hierHap=18;
  		const static int hapStdEffective=21;//equivalent values for codominant diploid

		  const static int IBDexhaustHap=102;
		  const static int IBDestimHap=103;

  public:
		FstatisticAnalysisHap(void){
						FstatisticAnalysisCore::Initialize();
						HaploidMarkerData::Initialize();
						EvolutionaryGeneticsData::Initialize();
						}
		FstatisticAnalysisHap(aString& s);
		FstatisticAnalysisHap(EvolutionaryGeneticsData& egd);
		FstatisticAnalysisHap(HaploidMarkerData& dmd);
		FstatisticAnalysisHap(FstatisticAnalysisHap& dmd);//copy constructor

			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
		FstatisticAnalysisHap(FstatisticAnalysisHap& parentData, aList<int>& popSubsetNums);
		FstatisticAnalysisHap(HaploidMarkerData& parentData, aList<int>& popSubsetNums);


		virtual	~FstatisticAnalysisHap(void){
			Clear();
			}

	void 	Initialize(FstatisticAnalysisHap& dmd);//copies


	void 		Clear();
	virtual void ClearIBDs();
	void		CalculateIBDestimates(Ran3& ran,int sampleSize);//sampled
	void		CalculateIBDexhaustive(void);//only do this for very small population samples
	virtual void CreateBootstrapData(HaploidMarkerData& origData, Ran3& ran){
						//individuals resampled keeping multilocus genotypes intact
						HaploidMarkerData::CreateBootstrapData(origData, ran);}
	virtual void CreateRandomData(const long double Fst, 
						const long double pBar, 
						const int numPopsSampled,
						const int numIndivsPerPop,
						const int numLociSampled,
						Ran3List& ran){
					HaploidMarkerData::CreateRandomData(Fst,pBar,numPopsSampled,numIndivsPerPop,numLociSampled,ran);
					}
	virtual void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
					HaploidMarkerData::CreateRandomData(Fst,pBar,numIndivsPerPopLoc,ran);
					}

	virtual void	ResetFlags(void);
	long double HaploidFstFromNm(long double Nm, int numTrueDemes=0);
	void		HapMarkerFstats (int numTrueDemes=0);
	void		HierarchicalHapMarkerFstats(void);
	virtual int loci(void){return HaploidMarkerData::loci();}
	virtual long double Nm(long double FstVal,int Fmethod, int numTrueDemes=0, bool zygoteMigration=false);
	long double NmFromHaploidFst(long double FstHap, int numTrueDemes=0);//, bool /*zygotMigration=false*/);
	virtual int PrintFMethodHeading(ostream& outFile, const int Fmethod);
	virtual void PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');
	virtual void PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');
	virtual void PrintNEXUSFile(aString& outFileName, StringList& textToAppend);
	virtual void PrintNEXUSFile (ostream& outFile, StringList& textToAppend);



	virtual int ReadHaploidDataFile (aString& dataFileName){
									//input file assumed closed
					return HaploidMarkerData::ReadHaploidDataFile(dataFileName);}
//	virtual int ReadAndAppendHaploidDataFile (aString& dataFileName){
//									//input file assumed closed
//					return HaploidMarkerData::ReadAndAppendHaploidDataFile(dataFileName);}
	FstatisticAnalysisHap& operator=(FstatisticAnalysisHap& dmd);
	FstatisticAnalysisHap& operator=(HaploidMarkerData& dmd);
	FstatisticAnalysisHap& operator=(EvolutionaryGeneticsData& egd);

};


#endif
