// FstatisticAnalysisMultiFormat.h

//Fstatistics calculated simultaneously for all data types

#ifndef FstatisticAnalysisMultiFormath
#define FstatisticAnalysisMultiFormath = 1


#include "StringClass.h"
#include "StringList.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "RandomList.h"
#include "ANOVAGenetics.h"
#include "EvolutionaryGeneticsData.h"
#include "MultiformatMarkerData.h"
#include "HaploidMarkerData.h"
#include "CodominantMarkerData.h"
#include "DominantMarkerData2.h"
#include "SexLinkedData.h"
#include "SexLinkedCodomMarkerData.h"
#include "SexLinkedDomMarkerData2.h"
#include "CytoplasmicFemaleMarkerData.h"
#include "CytoplasmicMaleMarkerData.h"
#include "FstatisticAnalysisCore.h"
#include "FstatisticAnalysisHap.h"
#include "FstatisticAnalysisCodom.h"
#include "FstatisticAnalysisDom2.h"
#include "FstatisticAnalysisCodomSexLink.h"
#include "FstatisticAnalysisDomSexLink.h"
#include "FstatisticAnalysisCytoFemale.h"
#include "FstatisticAnalysisCytoMale.h"
#include "MemoryManager.h"
//#include "FstatMethodType.h"


//20081209: updated Fstats_StandardLeastSquaresMethod() to allow effective Fst to be adjusted the number of demes, and for zygote vs gamete migration



class FstatisticAnalysisMultiFormat: 
					public virtual FstatisticAnalysisHap,
					public virtual FstatisticAnalysisCodom,
					public virtual FstatisticAnalysisDom2,
					public virtual FstatisticAnalysisCodomSexLink,
					public virtual FstatisticAnalysisDomSexLink,
					public virtual FstatisticAnalysisCytoFemale,
					public virtual FstatisticAnalysisCytoMale,
					public virtual FstatisticAnalysisCore,
					public virtual MultiformatMarkerData,
					public virtual HaploidMarkerData,
					public virtual CodominantMarkerData,
					public virtual DominantMarkerData2,
					public virtual SexLinkedCodomMarkerData,
					public virtual SexLinkedDomMarkerData2,
					public virtual CytoplasmicFemaleMarkerData,
					public virtual CytoplasmicMaleMarkerData,
					public virtual SexLinkedData,
					public virtual EvolutionaryGeneticsData{
  public:
		static const int effectiveFstOverLociStd = 19;
		static const int effectiveFstOverLociHier = 20;
  public:
		FstatisticAnalysisMultiFormat(void){
			Initialize();
			}
		FstatisticAnalysisMultiFormat(aString& s){
			Initialize(s);
			}
		FstatisticAnalysisMultiFormat(MultiformatMarkerData& pgd){
			Initialize(pgd);
			}
		virtual	~FstatisticAnalysisMultiFormat(void){
			Clear();
			}

		FstatisticAnalysisMultiFormat(FstatisticAnalysisMultiFormat& parentData){//copy constructor
			Initialize(parentData);
			}

		FstatisticAnalysisMultiFormat(MultiformatMarkerData& parentData, aList<int>& popSubsetNums){
						Initialize(parentData, popSubsetNums);}
						
		FstatisticAnalysisMultiFormat(FstatisticAnalysisMultiFormat& parentData, aList<int>& popSubsetNums){
						Initialize(parentData, popSubsetNums);}
						
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void){
						Clear();
						MultiformatMarkerData::Initialize();}
	void		Initialize(aString& s){
						Initialize();
						title_=s;}
	void		Initialize(MultiformatMarkerData& pgd);
	void		Initialize(FstatisticAnalysisMultiFormat& pgd);
	void		Initialize(MultiformatMarkerData& parentData, aList<int>& popSubsetNums);
	void		Initialize(FstatisticAnalysisMultiFormat& parentData, aList<int>& popSubsetNums);
	FstatisticAnalysisMultiFormat&	operator=(FstatisticAnalysisMultiFormat& pgd){
						Initialize(pgd);
						return *this;}
	virtual EvolutionaryGeneticsData* Duplicate(void){
						return NEW FstatisticAnalysisMultiFormat(*this);}

	virtual int  	alleles(int loc){
						return MultiformatMarkerData::alleles(loc);}
	void		CalculateAlleleFreqs(void){
						MultiformatMarkerData::CalculateAlleleFreqs();}
	ObjList<ObjList<aList<long double> > >*	CalculateMarkerFreqsByPopSet(aList<int>& popSet, 	////[trtype][pop][loc][allele]	
									ObjList<aList<long double> >& sampleSizeListbyTypeAndLoc, bool splitCytotypeBySex){
						return MultiformatMarkerData::CalculateMarkerFreqsByPopSet(popSet,sampleSizeListbyTypeAndLoc, splitCytotypeBySex);}
	void 		CalculateFactorialVarCompCoeffs(GLM& lm){ //species x locality
						FstatisticAnalysisCodom::CalculateFactorialVarCompCoeffs(lm);}
	void 		CalculateFactorialVarCompCoeffsII(GLM& lm){//locality x species
						FstatisticAnalysisCodom::CalculateFactorialVarCompCoeffsII(lm);}
	void 		CalculateFactorialVarCompCoeffsIII(aList<long double>& nList, aList<long double>& dfList){
						FstatisticAnalysisCodom::CalculateFactorialVarCompCoeffsIII(nList,dfList);}
	void 		CalculateFactorialVarCompCoeffsIII(aList<long double>& nList, aList<long double>& dfList, int loc){
						FstatisticAnalysisCodom::CalculateFactorialVarCompCoeffsIII(nList,dfList,loc);}
	void 		CalculateStdVarCompCoeffs(aList<long double>& nList, aList<long double>& dfList){
						FstatisticAnalysisCodom::CalculateStdVarCompCoeffs(nList,dfList);}

	virtual void CalculateIBDestimates(Ran3& ran,int sampleSize);
	virtual void CalculateIBDexhaustive(void);

	void 		ConvertCodominantData(FstatisticAnalysisMultiFormat& mfData){//never use self-reference mfData=*this
						MultiformatMarkerData::ConvertCodominantData(mfData);}
	virtual void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,
							long double sexratio,
							int numTrueDemes,//default=0 for infinite island model
							bool zygoteMigration){//default=false
					MultiformatMarkerData::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampledByDataType,ran,
																	sexratio,numTrueDemes,zygoteMigration);
					}
	virtual void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,//for each data type
							Ran3List& ran,
							long double sexratio,
							int numTrueDemes,//default=0 for infinite island model
							bool zygoteMigration){//default=false
					ClearFstats();
					ClearIBDs();
					StatList<int> numLociSampledByDataType;
					for(int dt=0;dt<7;++dt){numLociSampledByDataType.Append(numLociSampled);}
					MultiformatMarkerData::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampledByDataType,ran,sexratio,
																	numTrueDemes,zygoteMigration);
					}
	virtual void 		CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<ObjList<aList<long double> > >& pBar, //loci by alleles
							ObjList<ObjList<aList<int> > >& numIndivsPerPopLocDataType,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,
							long double sexratio,
							int numTrueDemes,//default=0 for infinite island model
							bool zygoteMigration){//default=false
					ClearFstats();
					ClearIBDs();
					MultiformatMarkerData::CreateRandomData(Fst,Fis,pBar,numIndivsPerPopLocDataType,numLociSampledByDataType,ran,sexratio,
																	numTrueDemes,zygoteMigration);
					}
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,
							long double sexratio,
							int numTrueDemesWithinSpecies,//default=0 for infinite island model
							int numTrueDemesBetweenSpecies,//d3fault=2
							bool zygoteMigration){//default=false
					ClearFstats();
					ClearIBDs();
					MultiformatMarkerData::CreateRandomData(Fspt,Flt,Fis,Flspt,pBar,numSpeciesSampled,numLocalitiesSampled,
																	numIndivsPerPop,numLociSampledByDataType,ran,sexratio,
																	numTrueDemesWithinSpecies,numTrueDemesBetweenSpecies,zygoteMigration);
					}


	virtual void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran
							){//default=false
					FstatisticAnalysisMultiFormat::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampledByDataType,ran,half,
																	0,false);
					}
	virtual void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,//for each data type
							Ran3List& ran){
					FstatisticAnalysisMultiFormat::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampled,ran,half,
																	0,false);
					}
	virtual void 		CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<ObjList<aList<long double> > >& pBar, //loci by alleles
							ObjList<ObjList<aList<int> > >& numIndivsPerPopLocDataType,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran){
					FstatisticAnalysisMultiFormat::CreateRandomData(Fst,Fis,pBar,numIndivsPerPopLocDataType,numLociSampledByDataType,ran,half,
																	0,false);
					}
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran){
					FstatisticAnalysisMultiFormat::CreateRandomData(Fspt,Flt,Fis,Flspt,pBar,numSpeciesSampled,numLocalitiesSampled,
																	numIndivsPerPop,numLociSampledByDataType,ran,half,
																	0,2,false);
					}
/*	virtual void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,aList<int>& femaleOnlyLoci, aList<int>& maleOnlyLoci){
					ClearFstats();
					MultiformatMarkerData::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampledByDataType,ran,
																	femaleOnlyLoci,maleOnlyLoci);
					}
	virtual void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,//for each data type
							Ran3List& ran,aList<int>& femaleOnlyLoci, aList<int>& maleOnlyLoci){
					ClearFstats();
					StatList<int> numLociSampledByDataType;
					for(int dt=0;dt<7;++dt){numLociSampledByDataType.Append(numLociSampled);}
					MultiformatMarkerData::CreateRandomData(Fst,Fis,pBar,numPopsSampled,numIndivsPerPop,numLociSampledByDataType,ran,
																	femaleOnlyLoci,maleOnlyLoci);
					}
	virtual void 		CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<ObjList<aList<long double> > >& pBar, //loci by alleles
							ObjList<ObjList<aList<int> > >& numIndivsPerPopLocDataType,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,aList<int>& femaleOnlyLoci, aList<int>& maleOnlyLoci){
					ClearFstats();
					MultiformatMarkerData::CreateRandomData(Fst,Fis,pBar,numIndivsPerPopLocDataType,numLociSampledByDataType,ran,
																	femaleOnlyLoci,maleOnlyLoci);
					}
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,aList<int>& femaleOnlyLoci, aList<int>& maleOnlyLoci){
					ClearFstats();
					MultiformatMarkerData::CreateRandomData(Fspt,Flt,Fis,Flspt,pBar,numSpeciesSampled,numLocalitiesSampled,
																	numIndivsPerPop,numLociSampledByDataType,ran,
																	femaleOnlyLoci,maleOnlyLoci);
					}
*/
	virtual void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double /*Fst*/, 
							const long double /*Fis*/,
							ObjList<aList<long double> >& /*pBar*/, //loci by alleles
							ObjList<aList<int> >& /*numIndivsPerPopLoc*/,
							Ran3List& /*ran*/){
					aError("Error: use a different constructor format for FstatisticAnalysisMultiFormat::CreateRandomData()");
					}
	virtual void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double /*Fst*/, 
							ObjList<aList<long double> >& /*pBar*/, //loci by alleles
							ObjList<aList<int> >& /*numIndivsPerPopLoc*/,
							Ran3List& /*ran*/){
					aError("Error: use a different constructor format for FstatisticAnalysisMultiFormat::CreateRandomData()");
					}
	virtual void 		CreateRandomData(
							const long double /*Fst*/, 
							const long double /*pBar*/, 
							const int /*numPopsSampled*/,
							const int /*numIndivsPerPop*/,
							const int /*numLociSampled*/,
							Ran3List& /*ran*/){
					aError("Error: use a different constructor format for FstatisticAnalysisMultiFormat::CreateRandomData()");
					}

	virtual void Clear(){//some redundancies may occur here
						FstatisticAnalysisHap::Clear();
						FstatisticAnalysisCodom::Clear();
						FstatisticAnalysisDom2::Clear();
						FstatisticAnalysisCodomSexLink::Clear();
						FstatisticAnalysisDomSexLink::Clear();
						FstatisticAnalysisCore::Clear();
						MultiformatMarkerData::Clear();
						HaploidMarkerData::Clear();
						CodominantMarkerData::Clear();
						DominantMarkerData2::Clear();
						SexLinkedCodomMarkerData::Clear();
						SexLinkedDomMarkerData2::Clear();
						SexLinkedData::Clear();
						EvolutionaryGeneticsData::Clear();						
						}
						
	virtual void ClearFstats(){//only clears arrays in Fstat classes, base data classes remain intact
//						FstatisticAnalysisHap::ClearFstats();
//						FstatisticAnalysisCodom::ClearFstats();
//						FstatisticAnalysisDom2::ClearFstats();
//						FstatisticAnalysisCodomSexLink::ClearFstats();
//						FstatisticAnalysisDomSexLink::ClearFstats();
						FstatisticAnalysisCore::ClearFstats();
						}
	virtual void ClearIBDs(){//only clears arrays in IBD classes, base data classes remain intact
						FstatisticAnalysisHap::ClearIBDs();
						FstatisticAnalysisCodom::ClearIBDs();
						FstatisticAnalysisCodomSexLink::ClearIBDs();
						FstatisticAnalysisCytoFemale::ClearIBDs();
						FstatisticAnalysisCytoMale::ClearIBDs();
						}
	virtual void 		CullLoci (aList<int>& lociToCull){
						if(lociToCull.len()>0){
							ResetGeneticFlags();
							MultiformatMarkerData::CullLoci(lociToCull);}
						}
	virtual void 		CullLoci (StringList& lociToCull){
						if(lociToCull.len()>0){
							ResetGeneticFlags();
							MultiformatMarkerData::CullLoci(lociToCull);}
						}
	virtual void 		CullLocus (aString& locusToCull){
							ResetGeneticFlags();
							MultiformatMarkerData::CullLocus(locusToCull);
						}
	virtual void 		CullLocus (char* locusToCull){
							ResetGeneticFlags();
							MultiformatMarkerData::CullLocus(locusToCull);
						}
	virtual void 		CullLocus (int locusToCull){
							ResetGeneticFlags();
							MultiformatMarkerData::CullLocus(locusToCull);
						}
	virtual void 		CullMonomorphicLoci (void){
						MultiformatMarkerData::CullMonomorphicLoci();
						}
	void 		CullToOneSex (FstatisticAnalysisMultiFormat& parentData, genderSet sexVal){
						//makes a subset data structure from the specified group
						MultiformatMarkerData::CullToOneSex(parentData, sexVal);
						}
	void 		CullToTheseSexes (FstatisticAnalysisMultiFormat& parentData, aList<int>& sexValList){
						//makes a subset data structure from the specified group
						MultiformatMarkerData::CullToTheseSexes(parentData, sexValList);
						}
						
//	virtual int PrintFMethodHeading(ostream& outFile, const int Fmethod);
//	void 		PrintFStatistics(ostream& outFile, const int Fmethod, char colSpacer = '\t');
//	virtual void PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, char colSpacer = '\t');
//	virtual void PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, char colSpacer = '\t');
//	void 		PrintFactorialFstatsDerivedStats(aString& outFileName);
//	void 		PrintFactorialFstatsDerivedStats(ostream& outFile);
//	int 		PrintVCmethodHeading(ostream& outFile, const int vcMethod);
//	void 		PrintVarianceComponents(ostream& outFile, const int vcMethod, char colSpacer='\t');
//	void 		PrintVarianceComponentsByLocus(ostream& outFile, const int vcMethod, char colSpacer='\t');
//	void 		PrintVarianceComponentsByAllele(ostream& outFile, const int vcMethod, char colSpacer='\t');
	void 		FstatsLinearAlg(void){//uses linear algebra to get variance components for F-statistics; codom only is supported
						FstatisticAnalysisCodom::FstatsLinearAlg();}
	void 		FstatsLinearAlg(ObjList<GLM>& LinearModelList){//uses linear algebra to get variance components for F-statistics; codom only is supported
						FstatisticAnalysisCodom::FstatsLinearAlg(LinearModelList);}
	void 		FstatsLinearAlgII(void){// codom only is supported
						FstatisticAnalysisCodom::FstatsLinearAlg();}
	void 		FstatsIntrogression(int includeInteraction=1){// codom only is supported
					//uses linear algebra to get variance components for F-statistics
					//in a factorial, nested design
						FstatisticAnalysisCodom::FstatsIntrogression(includeInteraction);}
	void 		FstatsIntrogression(ObjList<GLM>& LinearModelList, int includeInteraction = 1){// codom only is supported
					//uses linear algebra to get variance components for F-statistics
					//in a factorial, nested design
						FstatisticAnalysisCodom::FstatsIntrogression(LinearModelList,includeInteraction);}
	void  	FactorialFstats(ObjList<GLM>& LinearModelList, int includeInteraction){// codom only is supported
						FstatisticAnalysisCodom::FactorialFstats(LinearModelList,includeInteraction);}

	void 		FactorialFstats(void){// codom only is supported
					//uses quick computational forms where possible to get variance components for F-statistics
					//in a factorial, nested design
						FstatisticAnalysisCodom::FactorialFstats();}
	void  	FactorialFstatsII(ObjList<GLM>& LinearModelList, int includeInteraction){// codom only is supported
					//this method estimates locality after species variance component
						FstatisticAnalysisCodom::FactorialFstatsII(LinearModelList,includeInteraction);}
	void 		FactorialFstatsII(void){// codom only is supported
					//this method estimates locality after species variance component
					//uses quick computational forms where possible to get variance components for F-statistics
					//in a factorial, nested design
						FstatisticAnalysisCodom::FactorialFstatsII();}
	void 		FactorialFstatsIII(void){// codom only is supported
						FstatisticAnalysisCodom::FactorialFstatsIII();}

	void		Fstats_StandardLeastSquaresMethod (int numTrueDemes=0, bool zygoteMigration=false);//all data formats supported
						//Weir-Cockerham formulas and analogs for other data types

	virtual void 	GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
							//uploads data to pass to genetic analysis classes
							MultiformatMarkerData::GetGenome(pgcList);}
	virtual void 	GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID=true){
							//uploads this indiv's data to MultiInheritanceModeContainer<int>
							MultiformatMarkerData::GetGenome(gc,pop,indiv,setID);}
	void		HierarchicalWeirTheta(void){// codom only is supported
						FstatisticAnalysisCodom::HierarchicalWeirTheta();}
	virtual int  	loci(void){
						return MultiformatMarkerData::loci();}
	virtual int  	loci(int traitType){
						return MultiformatMarkerData::loci(traitType);}
	virtual long double Nm(long double FstVal,int Fmethod,int numTrueDemes=0,bool zygoteMigration=false);

	void		PrintAlleleFreqs(ostream& outFile, int ncols, char colSpacer = ' '){
						MultiformatMarkerData::PrintAlleleFreqs(outFile, ncols, colSpacer);}

	virtual int PrintFMethodHeading(ostream& outFile, const int Fmethod);
//	virtual void 		PrintFStatistics(ostream& outFile, aString& titleStr, char colSpacer = '\t');//all methods
	virtual void PrintFStatisticsByLocus(ostream& outFile, aString& titleStr, char colSpacer = '\t');//all methods
	virtual void PrintFStatisticsByLocus(ostream& outFile, aString& titleStr, StringList& /*locNames*/, char colSpacer = '\t');//all methods
	virtual void PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer = '\t');//all methods
	virtual void PrintFStatisticsByAllele(ostream& outFile, aString& titleStr, char colSpacer = '\t');//all methods
	virtual void PrintFStatisticsByAllele(ostream& outFile, aString& titleStr,  
										StringList& /*locNames*/, ObjList<StringList>& /*alleleNames*/, char colSpacer = '\t');//all methods
	virtual void PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr,  char colSpacer = '\t');//all methods

	virtual int PrintIBDMethodHeading(ostream& outFile, const int IBDmethod);
	virtual void PrintIBDByLocus(ostream& outFile, aString& titleStr, char colSpacer = '\t');//all methods
	virtual void PrintIBDByLocus(ostream& outFile, aString& titleStr, StringList& /*locNames*/, char colSpacer = '\t');//all methods
	virtual void PrintIBDByLocus(ostream& outFile, const int IBDmethod, aString& titleStr, char colSpacer = '\t');//all methods

	virtual void	 	PrintArlequinFile(aString& outFileName){
							MultiformatMarkerData::PrintArlequinFile(outFileName);}
	virtual void 		PrintArlequinFile (ostream& outFile){
							MultiformatMarkerData::PrintArlequinFile(outFile);}
	virtual void	 	PrintGelJockFile(aString& outFileName){
							MultiformatMarkerData::PrintGelJockFile(outFileName);}
	virtual void 		PrintGelJockFile (ostream& outFile){
							MultiformatMarkerData::PrintGelJockFile(outFile);}

	virtual int 		ReadGelJockDataFile (aString* dataFileName){//input file assumed closed
							return MultiformatMarkerData::ReadGelJockDataFile(dataFileName);}
	virtual int 		ReadGelJockmtDNADataFile (aString* dataFileName){//input file assumed closed
							return MultiformatMarkerData::ReadGelJockmtDNADataFile(dataFileName);}

	virtual void	ResetFlags(void){//some of these may be redundant
						FstatisticAnalysisHap::ResetFlags();
						FstatisticAnalysisCodom::ResetFlags();
						FstatisticAnalysisDom2::ResetFlags();
						FstatisticAnalysisCodomSexLink::ResetFlags();
						FstatisticAnalysisDomSexLink::ResetFlags();
						FstatisticAnalysisCytoFemale::ResetFlags();
						FstatisticAnalysisCytoMale::ResetFlags();
						FstatisticAnalysisCore::ResetFlags();
						MultiformatMarkerData::ResetFlags();
						HaploidMarkerData::ResetFlags();
						CodominantMarkerData::ResetFlags();
						DominantMarkerData2::ResetFlags();
						SexLinkedCodomMarkerData::ResetFlags();
						SexLinkedDomMarkerData2::ResetFlags();
						CytoplasmicFemaleMarkerData::ResetFlags();
						CytoplasmicMaleMarkerData::ResetFlags();
						SexLinkedData::ResetFlags();
						EvolutionaryGeneticsData::ResetFlags();						
						}

	virtual void	ResetGeneticFlags(void) { ResetFlags();}

	virtual void 		RestoreLoci (aList<int>& lociToRestore){
							if(lociToRestore.len()>0){
								ResetGeneticFlags();
								MultiformatMarkerData::RestoreLoci(lociToRestore);
								}
							}
							

	virtual void 		RestoreLoci (StringList& lociToRestore){
							if(lociToRestore.len()>0){
								ResetGeneticFlags();
								MultiformatMarkerData::RestoreLoci(lociToRestore);
								}
							}
	virtual void 		RestoreLocus (aString& locusToRestore){
							StringList lociToRestore;
							lociToRestore.Append(NEW aString(locusToRestore));
							RestoreLoci(lociToRestore);
							}
	virtual void 		RestoreLocus (char* locusToRestore){
							StringList lociToRestore;
							lociToRestore.Append(NEW aString(locusToRestore));
							RestoreLoci(lociToRestore);
							}
	virtual void 		RestoreLocus (int locusToRestore){
							aList<int> lociToRestore;
							lociToRestore.Append(locusToRestore);
							RestoreLoci(lociToRestore);
							}
							

	virtual void 		SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
							//records all data, passed along from genetic analysis classes
							//empties old data and creates new data structures
							MultiformatMarkerData::SetGenome(pgcList);}
	virtual void 		SetGenome(GeneContainerCore& gc, int pop, int indiv){
							//records this indiv's data from MultiInheritanceModeContainer<int>
							//assumes empty data structure already created in EvolutionaryGeneticData
							MultiformatMarkerData::SetGenome(gc,pop,indiv);}

};


#endif
