/*******************************************
*
*
*	Fstatistics.h
*
*	This class calculates F-statistics using various methods
*
********************************************/
#ifndef Fstatisticsh
#define Fstatisticsh = 1

#include "Element.h"
#include "aList.h"
#include "StringClass.h"
#include "StringList.h"
#include "GeneticPopulation.h"
#include "StatList.h"


class Fstatistics;

class Fstatistics {
  protected:
  	ObjList<ObjList<GeneticPopulation> > 	popList_;//sets of populations
  
  	ObjList<StringList >			alleleList_;//assumes alleles are int
	int						allelesCurrent_;//flag
	aList<int> 					variableLoci_;//indices of variable loci
	int						varLociCurrent_;//flag

  	ObjList<aString>			Flevels_;//levels
	aList<long double>				Fstats_;//F-level
	ObjList<aList<long double> >		FstatsLoc_;//loc, F-level
	ObjList<aList<long double> >		jackFstatsLoc_;//loc, F-level
	ObjList<ObjList<aList<long double> > >
							FstatsAll_;//loc,allele, F-level
	int						FstatsCurrent_;//flag
	aString					FstatsType;
  public:
  				Fstatistics(void);
  					
  				Fstatistics(ObjList<GeneticPopulation>& popList);
  
 				Fstatistics(ObjList<ObjList<GeneticPopulation> >& popList);
 				Fstatistics(ObjList<ObjList<GeneticPopulation> >* popList);
 
 				Fstatistics(Fstatistics& a):
  					alleleList_(a.alleleList_),
  					allelesCurrent_(a.allelesCurrent_), 
  					variableLoci_(a.variableLoci_),
  					varLociCurrent_(a.varLociCurrent_), 
  					Flevels_(a.Flevels_),
  					Fstats_(a.Fstats_),
  					FstatsLoc_(a.FstatsLoc_),
  					jackFstatsLoc_(a.jackFstatsLoc_),
  					FstatsAll_(a.FstatsAll_),
  					FstatsCurrent_(a.FstatsCurrent_){
  					FstatsType=a.FstatsType;}//end constructor
 
 				Fstatistics(Fstatistics* a):
  					alleleList_(a->alleleList_),
  					allelesCurrent_(a->allelesCurrent_), 
  					variableLoci_(a->variableLoci_),
  					varLociCurrent_(a->varLociCurrent_), 
  					Flevels_(a->Flevels_),
  					Fstats_(a->Fstats_),
  					FstatsLoc_(a->FstatsLoc_),
  					jackFstatsLoc_(a->jackFstatsLoc_),
  					FstatsAll_(a->FstatsAll_),
  					FstatsCurrent_(a->FstatsCurrent_){
  					FstatsType=a->FstatsType;}//end constructor
 
 virtual 			~Fstatistics(void) {
 					  for (int g=0;g<popList_.len();++g){//detach each popList
 						ObjList<GeneticPopulation>& group =popList_[g];
 						group.Detach();
 						}
 					}//end destructor

virtual Fstatistics* Duplicate(void){
						return NEW Fstatistics(*this);}

virtual long double	Fis(void);
virtual long double	Fis(int loc);
virtual long double	Fis(int loc, int alleleVal);
virtual long double	Fit(void);
virtual long double	Fit(int loc);
virtual long double	Fit(int loc, int alleleVal);
virtual long double	Fst(void);
virtual long double	Fst(int loc);
virtual long double	Fst(int loc, int alleleVal);
virtual long double	Fig(void);
virtual long double	Fig(int loc);
virtual long double	Fig(int loc, int alleleVal);
virtual long double	Fsg(void);
virtual long double	Fsg(int loc);
virtual long double	Fsg(int loc, int alleleVal);
virtual long double	Fgt(void);
virtual long double	Fgt(int loc);
virtual long double	Fgt(int loc, int alleleVal);
virtual void		FAndErrorString(aString& FRow, aString& Fword,int calcNm=0);
virtual void		NoLongerCurrent(void){
					FstatsCurrent_=allelesCurrent_=varLociCurrent_=0;
					FstatsType='\0';}
virtual void		PrintFstatSummary (ostream& out=cout);
virtual void		UpdateAlleleList(void);//makes a list of all alleles in the popList
virtual void		UpdateVariableLoci(void);
virtual void 		WeirTheta (int group);//standard Weir & Cockerham F-stats on a single population group
virtual void 		HierarchicalWeirTheta (void);

friend int operator== (Fstatistics& a, Fstatistics& b);
friend int operator!= (Fstatistics& a, Fstatistics& b);

	};//end class Fstatistics
	

int operator== (Fstatistics& a, Fstatistics& b);
int operator!= (Fstatistics& a, Fstatistics& b);


#endif
