/******************************************************
*
*	Fis_Fit_Q_ParamRec.h
*
*	Parameters for estimating Fis or Fit and allele frequencies Q using a multinomial distribution.
*	Use for diploid codominant or dominant markers; sex-linked/haplodiploid markers in derived class
*
******************************************************/

#ifndef Fis_Fit_Q_ParamRech
#define Fis_Fit_Q_ParamRech = 1


#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "ParamRec.h"
#include "Gamma.h"
#include "InlineTemplates.h"
#include "MemoryManager.h"

class Fis_Fit_Q_ParamRec;
class Fis_Fit_Q_ParamRec: virtual public ParamRec{
  public:
  	long double lnL_Fis_;
  	long double lnL_Fit_;
	long double Fis_;
	long double Fit_;
	long double Fst_;//derived parameter; Fst = 1-(1-Fit)/(1-Fis)
	long double sexRatio_;//F::M  set separately
	bool sexRatioVaries_;//T|F -- default F: treated as constant by default
	bool qParamsVary_;//T|F -- default T; 
	ObjList<ObjList<aList<long double> > > q_estimated_;//[pop][loc][allele]
	int numqParams_, numIndependentqParams_;
	ObjList<aList<long double> > Q_estimated_;//[loc][allele]
	int numQParams_, numIndependentQParams_;
	aList<int> paramChoices_;//lists all params for random selection
	int paramsToChoose_;//num params not yet chosen
  	
  public:
  	Fis_Fit_Q_ParamRec(void);
  	Fis_Fit_Q_ParamRec(long double Fis_val, long double Fit_val,long double sr, 
								 ObjList<ObjList<aList<long double> > >& init_q_estimated);
  	Fis_Fit_Q_ParamRec(Fis_Fit_Q_ParamRec& p);
  	virtual ~Fis_Fit_Q_ParamRec(void);

  	virtual ParamRec* 				Duplicate(void);
  	virtual Fis_Fit_Q_ParamRec* 	_Duplicate(void);
	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
  	


	virtual long double Fis(void);//returns Fis_
	virtual void 		Fis(long double Fis_val);//sets Fis_
	virtual long double Fit(void);//returns Fit_
	virtual void 		Fit(long double Fit_val);//sets Fit_
	virtual long double Fst(void);//returns Fst_
	virtual int 		NumqParams(void){return numqParams_;}
	virtual int 		NumIndependentqParams(void){return numIndependentqParams_;}
	virtual int 		NumQParams(void){return numQParams_;}
	virtual int 		NumIndependentQParams(void){return numIndependentQParams_;}
	long double 		Q(int loc, int allele);//returns global estimated frequency
	long double 		q(int pop, int loc, int allele);//returns local estimated frequency

	virtual int 		ChangeFis(Ran3& ran, long double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeFit(Ran3& ran, long double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		Change_q(int paramNum, Ran3& ran, Fis_Fit_Q_ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		Change_Q(int paramNum, Ran3& ran, Fis_Fit_Q_ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double /*weight=1.0*/);
//	virtual int 	 	ChangeParams(Ran3& ran, //returns number of parameter changed
//									ParamRec* paramDeviations, 
//									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 	 	ChangeAnnealingParams(Ran3& ran, //returns number of parameter changed
									ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeSexRatio(Ran3& ran, long double sigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual long double ExpectedValue(aList<long double>& /*coords*/);
			void		InitializeQ(void);
	long double			lnLFis(void){return lnL_Fis_;}
	long double			lnLFit(void){return lnL_Fit_;}
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumDerivedParams(void);
	virtual int 		NumParams(void);//number of parameters
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double 		ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
			bool		qParamsVary(void){return qParamsVary_;}
	virtual int 		RandomParameter(Ran3& ran);
	virtual void		ResetParamChoices(void){
							paramsToChoose_ = 2 + numIndependentqParams_ + numIndependentQParams_;//num params not yet chosen
							if(sexRatioVaries_) ++paramsToChoose_;
							}
	void				SetFst(void){
							Fst_=zero;
							if(Fis_<(long double)0.999){
									Fst_ = one - (one-Fit_)/(one-Fis_);}
								else if(Fis_<one){//handles rounding error problems near 1.0
									Fst_ = Fit_ - Fis_;}
								else{}
							}
	void				SetDependentAllele_q(int pop, int loc){
							long double summ=zero;
							int a,last_a = q_estimated_[pop][loc].len()-1;
							for(a=0;a<last_a;++a){
								summ += q_estimated_[pop][loc][a];}
							q_estimated_[pop][loc][last_a] = one - summ;
							}
	void				SetDependentAllele_Q(int loc){
							long double summ=zero;
							int a,last_a = Q_estimated_[loc].len()-1;
							for(a=0;a<last_a;++a){
								summ += Q_estimated_[loc][a];}
							Q_estimated_[loc][last_a] = one - summ;
							}
			void 		SetNumqParams(void);
			void 		SetNumQParams(void);
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
			void		SetQtoVary(bool tf){qParamsVary_=tf;}
			void		SetSexRatio(long double sr, bool variableSexRatio){sexRatio_=sr;sexRatioVaries_=variableSexRatio;}
	long double			SexRatio(void){return sexRatio_;}
			bool		SexRatioVaries(void){return sexRatioVaries_;}
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
	virtual int 		TotalParams(void);//number of parameters that could vary, plus derived paramerters

	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(Fis_Fit_Q_ParamRec& p, Fis_Fit_Q_ParamRec& q);
	friend int	operator!=(Fis_Fit_Q_ParamRec& p, Fis_Fit_Q_ParamRec& q);

	};//end class Fis_Fit_Q_ParamRec



#endif