/**************************************
*
*
*	GeneFrequencyDistMultiAllele.h
*
*
*	This class implements an algorithm for sampling
*	allele frequencies of populations with a given Fst (beta) distribution
*	when there are an arbitrary number of alleles.    
*
**************************************/

#ifndef GeneFrequencyDistMultiAlleleh
#define GeneFrequencyDistMultiAlleleh = 1


#include "InlineTemplates.h"
#include <iostream.h>
#include <math.h>
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "StringClass.h"
#include "aError.h"
#include "Randomness.h"
#include "GeneFrequencyDistribution.h"
#include "MemoryManager.h"

//20081229  bug fix in RandomFrequencies() to handle ploidy != 2
//20081229  added note that ploidy should be set to 0.5 when creating distributions for cytoplasmically inherited loci


class GeneFrequencyDistMultiAllele: GeneFrequencyDistribution{
	public:
	 StatList<long double> pBarList_;
	 int fixed_;//True if any allele has freq = 1.0; False otherwise
	 int fixedAllele_;
	 
	 public:
	 GeneFrequencyDistMultiAllele(const long double ploidy=two):
	 	//ploidy should be set to 0.5 when creating distributions for cytoplasmically inherited loci
	 	GeneFrequencyDistribution(ploidy),fixed_(0), fixedAllele_(-1){}
	 GeneFrequencyDistMultiAllele(const long double Fst, aList<long double>& pBar, const long double ploidy):
	 	//ploidy should be set to 0.5 when creating distributions for cytoplasmically inherited loci
	 			GeneFrequencyDistribution(Fst, pBar[0],ploidy),
	 			pBarList_(pBar), fixed_(0), fixedAllele_(-1){
			for(int i=0;i<pBarList_.len();++i){
				if(pBarList_[i]==one){
					fixed_=1;fixedAllele_=i;break;}
				}//end for i
	 				}//end ctor
	 GeneFrequencyDistMultiAllele(GeneFrequencyDistMultiAllele& d):
	 			GeneFrequencyDistribution(d),
	 			pBarList_(d.pBarList_), fixed_(d.fixed_), fixedAllele_(d.fixedAllele_){
	 				}
	 GeneFrequencyDistMultiAllele(GeneFrequencyDistMultiAllele* d):
	 			GeneFrequencyDistribution(d),
	 			pBarList_(d->pBarList_), fixed_(d->fixed_), fixedAllele_(d->fixedAllele_){
	 				}
	 			
	 ~GeneFrequencyDistMultiAllele(){
			pBarList_.Clear();			
	 	}

	virtual void Initialize(const long double Fst, aList<long double>& pBar);

	virtual void Initialize(const long double Fst, const long double pBar);

	virtual void Initialize(GeneFrequencyDistribution& d);//assumes d => GeneFrequencyDistMultiAllele

	virtual int fixed(void){return fixed_;}
			long double ProbabilityAtFrequencyP(aList<long double>& p, long double dp=1.0e-6);
			long double FreqP_AtProb_x(long double x);
	long double		RandomFrequency(Ran3& /*ran*/);
	aList<long double>*	RandomFrequencies(Ran3& ran);
	
	
};//end class GeneFrequencyDistMultiAllele


#endif


