// CytoplasmicFemaleMarkerData.h

// Holds data for markers that are transmitted through females only (usually mtDNA or chloroplasts)
// Markers may be present in females only, or in both sexes

#ifndef CytoplasmicFemaleMarkerDatah
#define CytoplasmicFemaleMarkerDatah = 1


#include "EvolutionaryGeneticsData.h"
#include "SexLinkedData.h"
//#include "SexLinkedCodomMarkerData.h"
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "RandomList.h"
#include "SortList.h"
#include "Stringclass.h"
#include "StringList.h"
#include "MemoryManager.h"
#include "GeneContainers.h"
#include <fstream.h>

class CytoplasmicFemaleMarkerData;

class CytoplasmicFemaleMarkerData: public virtual SexLinkedData,
							   public virtual EvolutionaryGeneticsData{
		public:
			static enum {presentBothSexes=0, presentFonly=1, allLoci=2};
		public:
			long nlociCYfemale_;//for cytoplasmic female marker loci
			int ploidy_, alleleLen_;
			aList<int> allCYfemaleLoci_,locusNumCYfemale_, skippedLocusNumCYfemale_;
					//these hold places in (=indices of) EvolutionaryGeneticsData::traitName_[]
			aList<int> allPresentInFemalesOnly_;//index by [0...allCYfemale_.len()] --> includes skipped loci
			aList<int> presentInFemalesOnly_;//index by [0...nlociCYfemale_] --> drops skipped loci
			aList<int> allPresentInBothSexes_;//index by [0...allCYfemale_.len()] --> includes skipped loci
			aList<int> presentInBothSexes_;//index by [0...nlociCYfemale_] --> drops skipped loci
			aList<int> variableLocusNumCYfemale_;//holds places in (=indices of) locusNumCYfemale_[]
//			ObjList<ObjList<TextDataList> > rawPhenotypes;//{0,1} presence absence data as read from data file
					//list of pops, each with list of individual phenotype strings
			ObjList<StringList> allelesPresentCYfemale_;//corresponds to order of locusNumCYfemale_[]
					//list of loci, each with sorted list of all alleles encountered
					//loci correspond to order of locusNumCYfemale_[]
			ObjList<ObjList<aList<long double> > > genotypeFreqCYfemale_;//0 is absent, 1 present
					//indexed [pop][loc][marker]
					//loci correspond to order of locusNumCYfemale_[]
					//list of pops, each with list of cytoplasmic male marker loci, each with list of marker freqs
			ObjList<aList<long double> > genotypeFreqCYfemaleGlobal_;//0 is absent, 1 present
					//indexed [loc][marker]
					//loci correspond to order of locusNumCYfemale_[]
					//list of cytoplasmic male marker loci, each with list of marker freqs
			ObjList<aList<int> > nByLocusCYfemale_;//cytoplasmic female markers only	
					//indexed [pop][loc]
					//loci correspond to order of locusNumCYfemale_[]
					//list of pops, each with list of #individuals sampled for each locus
			int allelesCurrentCYfemale_;
			int varLociCurrentCYfemale_;//boolean
			int genotypeFreqsCurrentCYfemale_;//boolean
		public:
		CytoplasmicFemaleMarkerData(void): EvolutionaryGeneticsData(),nlociCYfemale_(0),
							ploidy_(1),alleleLen_(1),varLociCurrentCYfemale_(0),
							allelesCurrentCYfemale_(0),genotypeFreqsCurrentCYfemale_(0) {
			EvolutionaryGeneticsData::Initialize();
			}
		CytoplasmicFemaleMarkerData(aString& s): EvolutionaryGeneticsData(s),nlociCYfemale_(0),
							ploidy_(1),alleleLen_(1),varLociCurrentCYfemale_(0),
							allelesCurrentCYfemale_(0),genotypeFreqsCurrentCYfemale_(0){
			EvolutionaryGeneticsData::Initialize(s);
			}

virtual	~CytoplasmicFemaleMarkerData(void){
			Clear();
			}

		CytoplasmicFemaleMarkerData(CytoplasmicFemaleMarkerData& parentData);//copy constructor

		CytoplasmicFemaleMarkerData(CytoplasmicFemaleMarkerData& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(CytoplasmicFemaleMarkerData& pgd);
	void		Initialize(CytoplasmicFemaleMarkerData& pgd, aList<int>& popSubsetNums);

	CytoplasmicFemaleMarkerData&	operator=(CytoplasmicFemaleMarkerData& pgd);


	EvolutionaryGeneticsData* Duplicate(void){
								return NEW CytoplasmicFemaleMarkerData(*this);}

	virtual int alleleLen(void) {return alleleLen_;}
	virtual int alleleID(int loc, TextDatum& alleleVal){//#of this allele in locus list
						if(!allelesCurrentCYfemale_) {CheckForNewAlleles();}
						return allelesPresentCYfemale_[loc].Loc(&alleleVal);}//location of the allele
	int alleleID(int pop, int indiv, int loc){//#of this allele in locus list
						TextDatum& alleleVal = CytoplasmicFemaleMarkerData::GenotypicVal(pop,indiv,loc,0);
						return CytoplasmicFemaleMarkerData::alleleID(loc,alleleVal);}//location of the allele
	virtual int alleles(int loc){
						if(!allelesCurrentCYfemale_) {CheckForNewAlleles();}
						return allelesPresentCYfemale_[loc].len();}//number of alleles
	void		CalculateGenotypeFreqs (void);
	ObjList<aList<long double> >* 
				CalculateGenotypeFreqsByPopSet(aList<int>& popSet, aList<long double>& nList, int locusTypesToInclude);
	void 		CheckForNewAlleles (void);
	virtual void		Clear(void);//clears all information
	void CreateBootstrapData(CytoplasmicFemaleMarkerData& origData, Ran3& ran);
						//pops & individuals resampled keeping multilocus genotypes intact
	void CreateBootstrapDataConstPop(CytoplasmicFemaleMarkerData& origData, Ran3& ran);
						//individuals resampled keeping multilocus genotypes intact
	void CreateRandomData(const long double Fst, 
							const long double dummyFis,
							const long double pBar,  //recessive *marker* freq 
							const int numPopsSampled,
							const int numIndivsPerPop,//female markers carried by both sexes
							const int numLociSampled,
							Ran3List& ran){
				aList<int> femaleOnlyLoci;//default empty
				CreateRandomData(Fst,dummyFis,pBar,numPopsSampled,numIndivsPerPop,numLociSampled,ran,femaleOnlyLoci);
				}
	void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double dummyFis,
							ObjList<aList<long double> >& pBar, //loci by alleles //recessive *marker* freq first
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
				aList<int> femaleOnlyLoci;//default empty
				CreateRandomData(Fst,dummyFis,pBar,numIndivsPerPopLoc,ran,femaleOnlyLoci);
				}
	void CreateRandomData(const long double Fst, 
							const long double /*Fis*/,
							const long double pBar,  //recessive *marker* freq 
							const int numPopsSampled,
							const int numIndivsPerPop,//female markers carried by both sexes
							const int numLociSampled,
							Ran3List& ran,
							aList<int>& femaleOnlyLoci);
	void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double /*Fis*/,
							ObjList<aList<long double> >& pBar, //loci by alleles //recessive *marker* freq first
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran,
							aList<int>& femaleOnlyLoci);
	void 		CullCYfemaleLoci (StringList& lociToCull);
	void 		CullCYfemaleLoci (aList<int>& lociToCull);
	void 		CullCYfemaleLocus (aString& locusToCull);
	void 		CullCYfemaleLocus (char* locusToCull);
	void 		CullMonomorphicLoci (void);
	void 		CullToOneSex (CytoplasmicFemaleMarkerData& parentData, genderSet sexVal);//makes a subset data structure from the specified group
	void 		CullToTheseSexes (CytoplasmicFemaleMarkerData& parentData, aList<int>& sexValList);//makes a subset data structure from the specified group
/*	virtual EvolutionaryGeneticsData* 		
				CullToOneGroup (int groupNum);//makes a subset data structure from the specified group
*/
	void 		ExpungeLoci (StringList& lociToCull);//transfer to skipped
	void 		ExpungeLoci (aList<int>& lociToCull);
	void 		ExpungeLocus (aString& locusToCull);
	void 		ExpungeLocus (char* locusToCull);
	bool		femaleLimitedTraitsIncluded(void){
							bool tf = false;
							if(presentInFemalesOnly_.len()>0) tf = true;
							return tf;}
	virtual TextDatum&	GenotypicVal(int pop, int indiv, int loc, int /*allele*/){// loc refers to cytoplasmic female loci only
							return rawTraits_[pop][indiv][locusNumCYfemale_[loc]];}
	int 		GenotypeSize(void){return alleleLen_* ploidy_;}
	long double GetExpectedHeterozygosity (int pop, int loc, int allele);
	long double GetExpectedHeterozygosity (int pop, int loc);
	long double 	GetFreqCYfemale (int loc, int marker);
	long double 	GetFreqCYfemale (int pop, int loc, int marker);

	long double 	GetFreqMostCommonAllele (int pop, int loc);
	virtual void 	GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList);
							//uploads data to pass to genetic analysis classes
	virtual void 	GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID=true);
							//uploads this indiv's data to HaplotypeContainer<int>
	long 		GetIndividualsSampledCYfemale (int pop, int loc);
	void		GetLocusNames(StringList& names){
					TraitNamesFromSublist(names,locusNumCYfemale_);}
/*	long 		GetLocusSampleSize (int loc);//pooling populations
*/
	long 		GetLocusSampleSizeCYfemale (int loc);// loc refers to cytoplasmic female loci only
	long 		GetLocusSampleSizeCYfemale (int pop, int loc);// loc refers to cytoplasmic female loci only
	long 		GetLocusSampleSizeCYfemale (int pop, int indiv, int loc);// loc refers to cytoplasmic female loci only
	int 		GetNumAllelesCYfemale (int loc);// loc refers to cytoplasmic female loci only
	int 		GetNumAllelesCYfemale (int pop, int loc);// loc refers to cytoplasmic female loci only
	aString* 	GetTraitString(int pop, int indiv, int loc);//loc = index of locusNumCYfemale_
	aString* 	GetTraitString(int pop, int indiv, int loc, int /*allele*/);
	int 		individuals(int pop){return EvolutionaryGeneticsData::individuals(pop);}//make this male only?
//	int 		individuals(int pop, int loc);// loc refers to cytoplasmic female loci only
	virtual int		loci(void){return nlociCYfemale_;}
	virtual int		lociCYfemale(void){return nlociCYfemale_;}//for compatibility
	virtual int 	lociSkipped(void){return skippedLocusNumCYfemale_.len();}
	aString& 	locusNameCYfemale(int loc){return traitName_[locusNumCYfemale_[loc]];}
	virtual long	maxall(void);
	virtual int 	Ploidy(int /*pop*/, int /*indiv*/, int /*loc*/){
							return ploidy_;}
	virtual int 	ploidy(void){return ploidy_;}
	virtual int 	ploidy(int pop, int indiv){return Ploidy(pop,indiv,0);}
	
	virtual long double PloidyEffective(void){
							return primarySexRatio();}

	void 				PrintGenotypeFreqs (aString& outFileName, int ncols, char colSpacer = '\t');//outfile assumed open
	void 				PrintGenotypeFreqs (ostream& outFile, int ncols, char colSpacer = '\t');//outfile assumed open
	virtual void	 	PrintCYfemaleMarkerFile(aString& outFileName);
	virtual void 		PrintCYfemaleMarkerFile (ostream& outFile);
	virtual void	 	PrintNEXUSFile(aString& outFileName, StringList& textToAppend);
	virtual void 		PrintNEXUSFile (ostream& outFile, StringList& textToApend);
	virtual void 		PrintLociCYfemale (ostream& outFile, int lociPerLine);//outfile assumed open
	virtual void 		ReadAFLPDataFile (aString& dataFileName);//input file assumed closed
//	virtual void 		ReadAndAppendAFLPDataFile (aString& dataFileName);//input file assumed closed
	virtual void		ResetFlags(void){
							CytoplasmicFemaleMarkerData::ResetGeneticFlags();
							SexLinkedData::ResetFlags();}
	virtual void		ResetGeneticFlags(void){//skips sex-linked flags
							allelesCurrentCYfemale_=varLociCurrentCYfemale_=genotypeFreqsCurrentCYfemale_=0;
							}
	virtual void 		RestoreLociCYfemale (aList<int>& lociToRestore);
	virtual void 		RestoreLociCYfemale (StringList& lociToRestore);
	virtual void 		RestoreLocusCYfemale (aString& locusToRestore);
	virtual void		SetAlleleLength(int allelelength){
							alleleLen_=allelelength;}
	virtual void 		SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList);
							//records all data, passed along from genetic analysis classes
							//empties old data and creates new data structures
	virtual void 		SetGenome(GeneContainerCore& gc, int pop, int indiv);
							//records data from HaplotypeContainer<int>
							//assumes blank data records already created in EvolutionaryGeneticsData
	void				SetLoci(aList<int>& lociToInclude);//constrains the CYfemale data set to these loci only
							//skipped loci status unchanged (use CullLoci to skip more; RestoreLoci to skip fewer)
	virtual void		SetLociPresentInFemalesOnly(aList<int>& locusList);//assumes males contain missing data as placeholders
	virtual int 		traits(void){return locusNumCYfemale_.len();}	
	virtual int 		traits(int /*traitType*/){return CytoplasmicFemaleMarkerData::traits();}	
	virtual void		UpdateVariableLociCYfemale(void);
	virtual aString&	VariableLocusName(int loc){
							return traitName_[locusNumCYfemale_[variableLocusNumCYfemale_[loc]]];}
		};//end class CytoplasmicFemaleMarkerData
		
		
#endif

