// CytoplasmicMaleMarkerData.h

// Holds data for markers that are transmitted through males only (usually Y chromosomes)
// Markers may be present in males only, or in both sexes

#ifndef CytoplasmicMaleMarkerDatah
#define CytoplasmicMaleMarkerDatah = 1


#include "EvolutionaryGeneticsData.h"
#include "SexLinkedData.h"
//#include "SexLinkedCodomMarkerData.h"
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "RandomList.h"
#include "Stringclass.h"
#include "StringList.h"
#include "SortList.cpp"
#include "MemoryManager.h"
#include "GeneContainers.h"
#include <fstream.h>

class CytoplasmicMaleMarkerData;

class CytoplasmicMaleMarkerData: public virtual SexLinkedData,
							   public virtual EvolutionaryGeneticsData{
		public:
			static enum {presentBothSexes=0, presentMonly=1, allLoci=2};
		public:
			long nlociCYmale_;//for cytoplasmic male marker loci
			int ploidy_, alleleLen_;
			aList<int> allCYmaleLoci_,locusNumCYmale_, skippedLocusNumCYmale_;
					//these hold places in (=indices of) EvolutionaryGeneticsData::traitName_[]
			aList<int> allPresentInMalesOnly_;//index by [0...allCYmaleLoci_]  --> includes skipped loci
			aList<int> presentInMalesOnly_;//index by [0...nlociCYmale_]  --> drops skipped loci
			aList<int> allPresentInBothSexes_;//index by [0...allCYmaleLoci_]  --> includes skipped loci
			aList<int> presentInBothSexes_;//index by [0...nlociCYmale_]  --> drops skipped loci
			aList<int> variableLocusNumCYmale_;//holds places in (=indices of) locusNumCYmale_[]
//			ObjList<ObjList<TextDataList> > rawPhenotypes;//{0,1} presence absence data as read from data file
					//list of pops, each with list of individual phenotype strings
			ObjList<StringList> allelesPresentCYmale_;//corresponds to order of locusNumCYmale_[]
					//list of loci, each with sorted list of all alleles encountered
					//loci correspond to order of locusNumCYmale_[]
			ObjList<ObjList<aList<long double> > > genotypeFreqCYmale_;//0 is absent, 1 present
					//indexed [pop][loc][marker]
					//loci correspond to order of locusNumCYmale_[]
					//list of pops, each with list of cytoplasmic male marker loci, each with list of marker freqs
			ObjList<aList<long double> > genotypeFreqCYmaleGlobal_;//0 is absent, 1 present
					//indexed [loc][marker]
					//loci correspond to order of locusNumCYmale_[]
					//list of cytoplasmic male marker loci, each with list of marker freqs
			ObjList<aList<int> > nByLocusCYmale_;//cytoplasmic male markers only	
					//indexed [pop][loc]
					//loci correspond to order of locusNumCYmale_[]
					//list of pops, each with list of #individuals sampled for each locus
			int allelesCurrentCYmale_;
			int varLociCurrentCYmale_;//boolean
			int genotypeFreqsCurrentCYmale_;//boolean
		public:
		CytoplasmicMaleMarkerData(void): EvolutionaryGeneticsData(),nlociCYmale_(0),
							ploidy_(1),alleleLen_(1),varLociCurrentCYmale_(0),
							allelesCurrentCYmale_(0),genotypeFreqsCurrentCYmale_(0) {
			EvolutionaryGeneticsData::Initialize();
			}
		CytoplasmicMaleMarkerData(aString& s): EvolutionaryGeneticsData(s),nlociCYmale_(0),
							ploidy_(1),alleleLen_(1),varLociCurrentCYmale_(0),
							allelesCurrentCYmale_(0),genotypeFreqsCurrentCYmale_(0){
			EvolutionaryGeneticsData::Initialize(s);
			}

virtual	~CytoplasmicMaleMarkerData(void){
			Clear();
			}

		CytoplasmicMaleMarkerData(CytoplasmicMaleMarkerData& parentData);//copy constructor

		CytoplasmicMaleMarkerData(CytoplasmicMaleMarkerData& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(CytoplasmicMaleMarkerData& pgd);
	void		Initialize(CytoplasmicMaleMarkerData& pgd, aList<int>& popSubsetNums);

	CytoplasmicMaleMarkerData&	operator=(CytoplasmicMaleMarkerData& pgd);


	EvolutionaryGeneticsData* Duplicate(void){
								return NEW CytoplasmicMaleMarkerData(*this);}

	virtual int alleleLen(void) {return alleleLen_;}
	virtual int alleleID(int loc, TextDatum& alleleVal){//#of this allele in locus list
						if(!allelesCurrentCYmale_) {CheckForNewAlleles();}
						return allelesPresentCYmale_[loc].Loc(&alleleVal);}//location of the allele
	int alleleID(int pop, int indiv, int loc){//#of this allele in locus list
						TextDatum& alleleVal = CytoplasmicMaleMarkerData::GenotypicVal(pop,indiv,loc,0);
						return CytoplasmicMaleMarkerData::alleleID(loc,alleleVal);}//location of the allele
	virtual int alleles(int loc){
						if(!allelesCurrentCYmale_) {CheckForNewAlleles();}
						return allelesPresentCYmale_[loc].len();}//number of alleles
	void		CalculateGenotypeFreqs (void);
	ObjList<aList<long double> >* 
				CalculateGenotypeFreqsByPopSet(aList<int>& popSet, aList<long double>& nList, int locusTypesToInclude);
	void 		CheckForNewAlleles (void);
	virtual void		Clear(void);//clears all information
	void CreateBootstrapData(CytoplasmicMaleMarkerData& origData, Ran3& ran);
						//pops & individuals resampled keeping multilocus genotypes intact
	void CreateBootstrapDataConstPop(CytoplasmicMaleMarkerData& origData, Ran3& ran);
						//individuals resampled keeping multilocus genotypes intact
	void CreateRandomData(const long double Fst, 
							const long double dummyFis,
							const long double pBar,  //recessive *marker* freq 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
					aList<int> dummyList;//empty == present in both sexes
					CreateRandomData(Fst,dummyFis,pBar,numPopsSampled,numIndivsPerPop,numLociSampled,ran,dummyList);}
	void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double dummyFis,
							ObjList<aList<long double> >& pBar, //loci by alleles //recessive *marker* freq first
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
					aList<int> dummyList;//empty == present in both sexes
					CreateRandomData(Fst,dummyFis,pBar,numIndivsPerPopLoc,ran,dummyList);}
	void CreateRandomData(const long double Fst, 
							const long double /*Fis*/,
							const long double pBar,  //recessive *marker* freq 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran,
							aList<int>& lociInMalesOnly);
	void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double /*Fis*/,
							ObjList<aList<long double> >& pBar, //loci by alleles //recessive *marker* freq first
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran,
							aList<int>& lociInMalesOnly);
	void 		CullCYmaleLoci (StringList& lociToCull);
	void 		CullCYmaleLoci (aList<int>& lociToCull);
	void 		CullCYmaleLocus (aString& locusToCull);
	void 		CullCYmaleLocus (char* locusToCull);
	void 		CullMonomorphicLoci (void);
/*	virtual EvolutionaryGeneticsData* 		
				CullToOneGroup (int groupNum);//makes a subset data structure from the specified group
*/
	void 		CullToOneSex (CytoplasmicMaleMarkerData& parentData, genderSet sexVal);//makes a subset data structure from the specified group
	void 		CullToTheseSexes (CytoplasmicMaleMarkerData& parentData, aList<int>& sexValList);//makes a subset data structure from the specified group
	void 		ExpungeLoci (StringList& lociToCull);//transfer to skipped
	void 		ExpungeLoci (aList<int>& lociToCull);
	void 		ExpungeLocus (aString& locusToCull);
	void 		ExpungeLocus (char* locusToCull);
	virtual TextDatum&	GenotypicVal(int pop, int indiv, int loc, int /*allele*/){// loc refers to cytoplasmic male loci only
							return rawTraits_[pop][indiv][locusNumCYmale_[loc]];}
	int 		GenotypeSize(void){return alleleLen_* ploidy_;}
	long double GetExpectedHeterozygosity (int pop, int loc, int allele);
	long double GetExpectedHeterozygosity (int pop, int loc);
	long double 	GetFreqCYmale (int loc, int marker);
	long double 	GetFreqCYmale (int pop, int loc, int marker);

	long double 	GetFreqMostCommonAllele (int pop, int loc);
	virtual void 	GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList);
							//uploads data to pass to genetic analysis classes
	virtual void 	GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID=true);
							//uploads this indiv's data to HaplotypeContainer<int>
	long 		GetIndividualsSampledCYmale (int pop, int loc);
	void		GetLocusNames(StringList& names){
					TraitNamesFromSublist(names,locusNumCYmale_);}
/*	long 		GetLocusSampleSize (int loc);//pooling populations
*/
	long 		GetLocusSampleSizeCYmale (int loc);// loc refers to cytoplasmic male loci only
	long 		GetLocusSampleSizeCYmale (int pop, int loc);// loc refers to cytoplasmic male loci only
	long 		GetLocusSampleSizeCYmale (int pop, int indiv, int loc);// loc refers to cytoplasmic male loci only
	int 		GetNumAllelesCYmale (int loc);// loc refers to cytoplasmic male loci only
	int 		GetNumAllelesCYmale (int pop, int loc);// loc refers to cytoplasmic male loci only
	aString* 	GetTraitString(int pop, int indiv, int loc);//loc is item in locusNumCYmale_
	aString* 	GetTraitString(int pop, int indiv, int loc, int /*allele*/);

	int 		individuals(int pop){return EvolutionaryGeneticsData::individuals(pop);}//make this male only?
//	int 		individuals(int pop, int loc);// loc refers to cytoplasmic male loci only
	virtual int		loci(void){return nlociCYmale_;}
	virtual int		lociCYmale(void){return nlociCYmale_;}//for compatibility
	virtual int 	lociSkipped(void){return skippedLocusNumCYmale_.len();}
	aString& 	locusNameCYmale(int loc){return traitName_[locusNumCYmale_[loc]];}
	bool		maleLimitedTraitsIncluded(void){
							bool tf=false;
							if(presentInMalesOnly_.len()>0) {tf= true;}
							return tf;}
	virtual long 	maxall(void);
	virtual int 	Ploidy(int /*pop*/, int /*indiv*/, int /*loc*/){
							return ploidy_;}
	virtual int 	ploidy(void){return ploidy_;}
	virtual int 	ploidy(int pop, int indiv){return Ploidy(pop,indiv,0);}
	virtual long double PloidyEffective(void){
							return (long double)1.0 - primarySexRatio();}
	void 				PrintGenotypeFreqs (aString& outFileName, int ncols, char colSpacer = '\t');//outfile assumed open
	void 				PrintGenotypeFreqs (ostream& outFile, int ncols, char colSpacer = '\t');//outfile assumed open
	virtual void	 	PrintCYmaleMarkerFile(aString& outFileName);
	virtual void 		PrintCYmaleMarkerFile (ostream& outFile);
	virtual void	 	PrintNEXUSFile(aString& outFileName, StringList& textToAppend);
	virtual void 		PrintNEXUSFile (ostream& outFile, StringList& textToApend);
	virtual void 		PrintLociCYmale (ostream& outFile, int lociPerLine);//outfile assumed open
	virtual void 		ReadAFLPDataFile (aString& dataFileName);//input file assumed closed
//	virtual void 		ReadAndAppendAFLPDataFile (aString& dataFileName);//input file assumed closed
	virtual void		ResetFlags(void){
							CytoplasmicMaleMarkerData::ResetGeneticFlags();
							SexLinkedData::ResetFlags();}
	virtual void		ResetGeneticFlags(void){//skips sex-linked flags
							allelesCurrentCYmale_=varLociCurrentCYmale_=genotypeFreqsCurrentCYmale_=0;
							}
	virtual void 		RestoreLociCYmale (aList<int>& lociToRestore);
	virtual void 		RestoreLociCYmale (StringList& lociToRestore);
	virtual void 		RestoreLocusCYmale (aString& locusToRestore);
	virtual void		SetAlleleLength(int allelelength){
							alleleLen_=allelelength;}
	virtual void 		SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList);
							//records all data, passed along from genetic analysis classes
							//empties old data and creates new data structures
	virtual void 		SetGenome(GeneContainerCore& gc, int pop, int indiv);
							//records data from HaplotypeContainer<int>
							//assumes blank data records already created in EvolutionaryGeneticsData
	void				SetLoci(aList<int>& lociToInclude);//constrains the CYmale data set to these loci only
							//skipped loci status unchanged (use CullLoci to skip more; RestoreLoci to skip fewer)
	virtual void		SetLociPresentInMalesOnly(aList<int>& locusList);//assumes females contain missing data as placeholders
	virtual int 		traits(void){return locusNumCYmale_.len();}	
	virtual int 		traits(int /*traitType*/){return CytoplasmicMaleMarkerData::traits();}	
	virtual void		UpdateVariableLociCYmale(void);
	virtual aString&	VariableLocusName(int loc){
							return traitName_[locusNumCYmale_[variableLocusNumCYmale_[loc]]];}
		};//end class CytoplasmicMaleMarkerData
		
		
#endif

