// DominantMarkerData2.h

#ifndef DominantMarkerData2h
#define DominantMarkerData2h = 1


#include "EvolutionaryGeneticsData.h"
#include "CodominantMarkerData.h"
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "RandomList.h"
#include "Stringclass.h"
#include "StringList.h"
#include "SortList.h"
#include "MemoryManager.h"
#include <fstream.h>

class CodominantMarkerData;

class DominantMarkerData2: public virtual EvolutionaryGeneticsData{
		public:
			long nlociDom_;//for dominant marker loci
			int ploidy_, alleleLen_;
			aList<int> allDomLoci_,locusNumDom_, skippedLocusNumDom_;
					//these hold places in (=indices of) EvolutionaryGeneticsData::traitName_[]
			aList<int> variableLocusNumDom_;//holds places in (=indices of) locusNumDom_[]
//			ObjList<ObjList<TextDataList> > rawPhenotypes;//{0,1} presence absence data as read from data file
					//list of pops, each with list of individual phenotype strings
			ObjList<StringList> allelesPresentDom_;//corresponds to order of locusNumDom_[]
					//list of loci, each with sorted list of all alleles encountered
					//loci correspond to order of locusNumDom_[]
			ObjList<ObjList<aList<int> > > phenotypeCount_;//0 is recessive, 1 dominant
					//indexed [pop][loc][marker]
					//loci correspond to order of locusNumDom_[]
					//list of pops, each with list of dominant marker loci, each with list of marker counts
			ObjList<aList<int> > phenotypeCountGlobal_;//0 is recessive, 1 dominant
					//indexed [pop][loc][marker]
					//loci correspond to order of locusNumDom_[]
					//list of pops, each with list of dominant marker loci, each with list of marker counts
			ObjList<ObjList<aList<long double> > > phenotypeFreq_;//0 is recessive, 1 dominant
					//indexed [pop][loc][marker]
					//loci correspond to order of locusNumDom_[]
					//list of pops, each with list of dominant marker loci, each with list of marker freqs
			ObjList<aList<long double> > phenotypeFreqGlobal_;//0 is recessive, 1 dominant
					//indexed [pop][loc][marker]
					//loci correspond to order of locusNumDom_[]
					//list of pops, each with list of dominant marker loci, each with list of marker freqs
			ObjList<aList<int> > nByLocusDom_;//dominant markers only	
					//indexed [pop][loc]
					//loci correspond to order of locusNumDom_[]
					//list of pops, each with list of #individuals sampled for each locus
			int allelesCurrentDom_;
			int varLociCurrentDom_;//boolean
			int phenotypeFreqsCurrent_;//boolean
		public:
		DominantMarkerData2(void): EvolutionaryGeneticsData(),nlociDom_(0),
							ploidy_(2),alleleLen_(1),varLociCurrentDom_(0),
							allelesCurrentDom_(0),phenotypeFreqsCurrent_(0) {
			EvolutionaryGeneticsData::Initialize();
			}
		DominantMarkerData2(aString& s): EvolutionaryGeneticsData(s),nlociDom_(0),
							ploidy_(2),alleleLen_(1),varLociCurrentDom_(0),
							allelesCurrentDom_(0),phenotypeFreqsCurrent_(0){
			EvolutionaryGeneticsData::Initialize(s);
			}

virtual	~DominantMarkerData2(void){
			Clear();
			}

		DominantMarkerData2(DominantMarkerData2& parentData);//copy constructor

		DominantMarkerData2(DominantMarkerData2& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(DominantMarkerData2& pgd);
	void		Initialize(DominantMarkerData2& pgd, aList<int>& popSubsetNums);
	DominantMarkerData2&	operator=(DominantMarkerData2& pgd);
	EvolutionaryGeneticsData* Duplicate(void){
								return NEW DominantMarkerData2(*this);}

	virtual int alleleLen(void) {return alleleLen_;}
	virtual int alleleID(int loc, TextDatum& alleleVal){//#of this allele in locus list
						if(!allelesCurrentDom_) {CheckForNewAlleles();}
						return allelesPresentDom_[loc].Loc(&alleleVal);}//location of the allele
	virtual int alleles(int loc){
						if(!allelesCurrentDom_) {CheckForNewAlleles();}
						return allelesPresentDom_[loc].len();}//number of alleles
	void		CalculatePhenotypeFreqs (void);
	ObjList<aList<long double> >* 
				CalculatePhenotypeFreqsByPopSet(aList<int>& popSet, aList<long double>& nList);
	void 		CheckForNewAlleles (void);
	virtual void		Clear(void);//clears all information
	void 		ConvertCodominantData(CodominantMarkerData& codomData);
						//appends converted data; don't let codomData be part of the same data object
	void 		CreateBootstrapData(DominantMarkerData2& origData, Ran3& ran);
						//pops & individuals resampled keeping multilocus genotypes intact
	void 		CreateBootstrapDataConstPop(DominantMarkerData2& origData, Ran3& ran);
						//individuals resampled keeping multilocus genotypes intact
	void CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double betaBar, //just one locus ->dominant marker frequency (not allele freq)
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void CreateRandomData(//dominant marker freqs (not allele freqs) that vary among loci; variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& betaBar, //--> only dominant marker freqs allowed
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran);
	void 		CullDomLoci (StringList& lociToCull);
	void 		CullDomLoci (aList<int>& lociToCull);
	void 		CullDomLocus (aString& locusToCull);
	void 		CullDomLocus (char* locusToCull);
	void 		CullMonomorphicLociDom (void);
//	virtual EvolutionaryGeneticsData* 		
//				CullToOneGroup (int groupNum);//makes a subset data structure from the specified group

	long double DomFstFromNm(long double Nm){
					return ((long double)1.0)/((long double)2.0*Nm + (long double)1.0);}
	long double dominantAlleleFreqFromFisBeta (long double beta, long double FisVal);
	void 		ExpungeLoci (StringList& lociToCull);//transfer to skipped
	void 		ExpungeLoci (aList<int>& lociToCull);
	void 		ExpungeLocus (aString& locusToCull);
	void 		ExpungeLocus (char* locusToCull);
	int 		GenotypeSize(void){return alleleLen_;}
	int		 		GetCountDom (int loc, int marker){
							if(!phenotypeFreqsCurrent_){CalculatePhenotypeFreqs();}
							return phenotypeCountGlobal_[loc][marker];}
	long double 	GetFreqDom (int loc, int marker);//uses weighted average over populations
	aString* 	GetTraitString(int pop, int indiv, int loc);//loc = index of locusNumDom_
	aString* 	GetTraitString(int pop, int indiv, int loc, int /*allele*/);
	long double 	GetUnweightedFreqDom (int loc, int marker){
							if(!phenotypeFreqsCurrent_){CalculatePhenotypeFreqs();}
							return phenotypeFreqGlobal_[loc][marker];}
	int 			GetCountDom (int pop, int loc, int marker){
							if(!phenotypeFreqsCurrent_){CalculatePhenotypeFreqs();}
							return phenotypeCount_[pop][loc][marker];}
	long double 	GetFreqDom (int pop, int loc, int marker);

	long double 	GetFreqMostCommonAllele (int pop, int loc);
//	long 		GetLocusSampleSize (int loc);//pooling populations

	long 		GetIndividualsSampledDom (int pop, int loc);
	void		GetLocusNames(StringList& names){
					TraitNamesFromSublist(names,locusNumDom_);}
	long 		GetLocusSampleSizeDom (int loc);// loc refers to dominant loci only
	long 		GetLocusSampleSizeDom (int pop, int loc);// loc refers to dominant loci only
	long 		GetLocusSampleSizeDom (int pop, int indiv, int loc);// loc refers to dominant loci only
	int 		GetNumAllelesDom (int loc);// loc refers to dominant loci only
	int 		GetNumAllelesDom (int pop, int loc);// loc refers to dominant loci only
	int			IsPolymorphic(int pop, int loc);
	int			IsPolymorphic(int loc);
	int			IsPolymorphic(int pop, int loc, double criterion);
	int			IsPolymorphic(int loc, double criterion);

	int 		individuals(int pop){return EvolutionaryGeneticsData::individuals(pop);}
	int 		individuals(int pop, int loc);// loc refers to dominant loci only
	virtual int		loci(void){return nlociDom_;}
	virtual int		lociDom(void){return nlociDom_;}//for compatibility
	virtual int 	lociSkipped(void){return skippedLocusNumDom_.len();}
	aString& 	locusNameDom(int loc){return traitName_[locusNumDom_[loc]];}
	virtual long maxall(void);
	long double NmFromDomFst(long double FstDom){
					return ((long double)1.0 - FstDom)/((long double)2.0*FstDom);}
	virtual TextDatum&	PhenotypicVal(int pop, int indiv, int loc){// loc refers to dominant loci only
							return rawTraits_[pop][indiv][locusNumDom_[loc]];}
	virtual int 	ploidy(void){return ploidy_;}
	virtual int 	Ploidy(int /*pop*/, int /*indiv*/, int /*loc*/){return ploidy_;}
	void 				PrintPhenotypeFreqs (aString& outFileName, int ncols, char colSpacer = '\t');//outfile assumed open
	void 				PrintPhenotypeFreqs (ostream& outFile, int ncols, char colSpacer = '\t');//outfile assumed open
	virtual void	 	PrintDomMarkerFile(aString& outFileName);
	virtual void 		PrintDomMarkerFile (ostream& outFile);
	virtual void	 	PrintNEXUSFile(aString& outFileName, StringList& textToAppend);
	virtual void 		PrintNEXUSFile (ostream& outFile, StringList& textToApend);
	virtual void 		PrintLociDom (ostream& outFile, int lociPerLine);//outfile assumed open
	virtual int 		ReadAFLPDataFile (aString& dataFileName);//input file assumed closed
//	virtual int 		ReadAndAppendAFLPDataFile (aString& dataFileName);//input file assumed closed
	long double			recessiveAlleleFreqFromFisBeta(long double beta, long double FisVal){
								return (long double)1.0 - dominantAlleleFreqFromFisBeta(beta,FisVal);}
	virtual void		ResetFlags(void){
							DominantMarkerData2::ResetGeneticFlags();
							EvolutionaryGeneticsData::ResetFlags();}
	virtual void		ResetGeneticFlags(void){
							allelesCurrentDom_=varLociCurrentDom_=phenotypeFreqsCurrent_=0;}
	virtual void 		RestoreLociDom (aList<int>& lociToRestore);
	virtual void 		RestoreLociDom (StringList& lociToRestore);
	virtual void 		RestoreLocusDom (aString& locusToRestore);
	void				SetLoci(aList<int>& lociToInclude);//constrains the diploid dominant data set to these loci only
							//skipped loci status unchanged (use CullLoci to skip more; RestoreLoci to skip fewer)
	virtual int 		traits(void){return locusNumDom_.len();}	
	virtual int 		traits(int /*traitType*/){return DominantMarkerData2::traits();}	
	virtual void		UpdateVariableLociDom(void);
	virtual aString&	VariableLocusName(int loc){
							return traitName_[locusNumDom_[variableLocusNumDom_[loc]]];}

		};//end class DominantMarkerData2
		
		
#endif

