// HaploidMarkerData.h

// Handles data that are haploid, including nuclear markers only

#ifndef HaploidMarkerDatah
#define HaploidMarkerDatah = 1


#include "EvolutionaryGeneticsData.h"
//#include "CodominantMarkerData.h"
#include "GeneFrequencyDistribution.h"
#include "GeneFrequencyDistMultiAllele.h"
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "Stringclass.h"
#include "RandomList.h"
#include "SortList.h"
#include "MemoryManager.h"
#include "GeneContainers.h"
#include <fstream.h>


class HaploidMarkerData: public virtual EvolutionaryGeneticsData{
		public:
			long nlociHap_;//for haploid marker loci
			int alleleLen_;
			aList<int> allHapLoci_,locusNumHap_, skippedLocusNumHap_;
					//these hold places in (=indices of) EvolutionaryGeneticsData::traitName_[]
			aList<int> variableLocusNumHap_;//holds places in (=indices of) locusNumHap_[]
//			ObjList<ObjList<TextDataList> > rawPhenotypes;//{0,1} presence absence data as read from data file
					//list of pops, each with list of individual phenotype strings
			ObjList<StringList> allelesPresentHap_;//corresponds to order of locusNumHap_[]
					//list of loci, each with sorted list of all alleles encountered
					//loci correspond to order of locusNumHap_[]
			ObjList<ObjList<aList<long double> > > alleleFreqHap_;//0 is recessive, 1 haploid
					//indexed [pop][loc][marker]
					//loci correspond to order of locusNumHap_[]
					//list of pops, each with list of dominant marker loci, each with list of marker freqs
			ObjList<aList<long double> > alleleFreqHapGlobal_;//0 is recessive, 1 haploid
					//indexed [pop][loc][marker]
					//loci correspond to order of locusNumHap_[]
					//list of pops, each with list of dominant marker loci, each with list of marker freqs
			ObjList<aList<int> > nByLocusHap_;//haploid markers only	
					//indexed [pop][loc]
					//loci correspond to order of locusNumHap_[]
					//list of pops, each with list of #individuals sampled for each locus
			int allelesCurrentHap_;
			int varLociCurrentHap_;//boolean
			int alleleFreqsCurrentHap_;//boolean
		public:
		HaploidMarkerData(void): EvolutionaryGeneticsData(),nlociHap_(0),
							alleleLen_(1),varLociCurrentHap_(0),
							allelesCurrentHap_(0),alleleFreqsCurrentHap_(0) {
			EvolutionaryGeneticsData::Initialize();
			}
		HaploidMarkerData(aString& s): EvolutionaryGeneticsData(s),nlociHap_(0),
							alleleLen_(1),varLociCurrentHap_(0),
							allelesCurrentHap_(0),alleleFreqsCurrentHap_(0){
			EvolutionaryGeneticsData::Initialize(s);
			}

		HaploidMarkerData(int alelelen): EvolutionaryGeneticsData(),nlociHap_(0),
							alleleLen_(alelelen),varLociCurrentHap_(0),
							allelesCurrentHap_(0),alleleFreqsCurrentHap_(0){
			EvolutionaryGeneticsData::Initialize();
			}

virtual	~HaploidMarkerData(void){
			Clear();
			}

		HaploidMarkerData(HaploidMarkerData& parentData);//copy constructor

		HaploidMarkerData(HaploidMarkerData& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(HaploidMarkerData& pgd);
	void		Initialize(HaploidMarkerData& pgd, aList<int>& popSubsetNums);
	HaploidMarkerData&	operator=(HaploidMarkerData& pgd);
	EvolutionaryGeneticsData* Duplicate(void){
								return NEW HaploidMarkerData(*this);}

	virtual int alleleLen(void) {return alleleLen_;}
	int alleleID(int loc, TextDatum& alleleVal){//#of this allele in locus list
						if(!allelesCurrentHap_) {CheckForNewAlleles();}
						return allelesPresentHap_[loc].Loc(&alleleVal);}//location of the allele
	int alleleID(int pop, int indiv, int loc){//#of this allele in locus list
						TextDatum& alleleVal = HaploidMarkerData::GenotypicVal(pop,indiv,loc);
						return HaploidMarkerData::alleleID(loc,alleleVal);}//location of the allele
	virtual int alleles(int loc){
						if(!allelesCurrentHap_) {CheckForNewAlleles();}
						return allelesPresentHap_[loc].len();}//number of alleles
	void		CalculateHaplotypeFreqs (void);
	ObjList<aList<long double> >* 
				CalculateHaplotypeFreqsByPopSet(aList<int>& popSet, aList<long double>& nList);
	void 		CheckForNewAlleles (void);
	virtual void		Clear(void);//clears all information
	void CreateBootstrapData(HaploidMarkerData& origData, Ran3& ran);
						//pops & individuals resampled keeping multilocus genotypes intact
	void CreateBootstrapDataConstPop(HaploidMarkerData& origData, Ran3& ran);
						//individuals resampled keeping multilocus genotypes intact
	void CreateRandomData(const long double Fst, 
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran);
	void 		CullHapLoci (StringList& lociToCull);//transfer to skipped
	void 		CullHapLoci (aList<int>& lociToCull);
	void 		CullHapLocus (aString& locusToCull);
	void 		CullHapLocus (char* locusToCull);
	void 		CullMonomorphicLociHap (void);

/*	virtual EvolutionaryGeneticsData* 		
				CullToOneGroup (int groupNum);//makes a subset data structure from the specified group
*/
	void 		ExpungeLoci (StringList& lociToCull);//transfer to skipped
	void 		ExpungeLoci (aList<int>& lociToCull);
	void 		ExpungeLocus (aString& locusToCull);
	void 		ExpungeLocus (char* locusToCull);
	TextDatum&	GenotypicVal(int pop, int indiv, int loc){// loc refers to haploid loci only
							return rawTraits_[pop][indiv][locusNumHap_[loc]];}
	int 		GenotypeSize(void){return alleleLen_;}
	long double GetExpectedHeterozygosity (int pop, int loc, int allele);
	long double GetExpectedHeterozygosity (int pop, int loc);
	long double 	GetFreqHap (int pop, int loc, int marker);
	long double 	GetFreqHap (int loc, int marker);

	long double 	GetFreqMostCommonAllele (int pop, int loc);
	virtual void 	GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList);
							//uploads data to pass to genetic analysis classes
	virtual void 	GetGenome(GeneContainerCore& gc, int pop, int indiv,bool setID=true);
							//uploads this indiv's data to HaplotypeContainer<int>
	long 		GetIndividualsSampledHap (int pop, int loc);
/*	long 		GetLocusSampleSize (int loc);//pooling populations
*/
	void		GetLocusNames(StringList& names){
					TraitNamesFromSublist(names,locusNumHap_);}
	long 		GetLocusSampleSizeHap (int loc);// loc refers to haploid loci only
	long 		GetLocusSampleSizeHap (int pop, int loc);// loc refers to haploid loci only
	long 		GetLocusSampleSizeHap (int pop, int indiv, int loc);// loc refers to haploid loci only
	int 		GetNumAllelesHap (int loc);// loc refers to haploid loci only
	int 		GetNumAllelesHap (int pop, int loc);// loc refers to haploid loci only
	aString* 	GetTraitString(int pop, int indiv, int loc);//loc = index of locusNumHap_
	aString* 	GetTraitString(int pop, int indiv, int loc, int /*allele*/);
	int 		individuals(int pop){return EvolutionaryGeneticsData::individuals(pop);}
	int 		individuals(int pop, int loc);// loc refers to haploid loci only
	virtual int		loci(void){return nlociHap_;}
	virtual int		lociHap(void){return nlociHap_;}//for compatibility
	virtual int 	lociSkipped(void){return skippedLocusNumHap_.len();}
	aString& 	locusNameHap(int loc){return traitName_[locusNumHap_[loc]];}
	virtual long	maxall();
	virtual int 	ploidy(void){return 1;}
	virtual int 	Ploidy(int /*pop*/, int /*indiv*/, int /*loc*/){return 1;}
	void 				PrintHaplotypeFreqs (aString& outFileName, int ncols, char colSpacer = '\t');//outfile assumed open
	void 				PrintHaplotypeFreqs (ostream& outFile, int ncols, char colSpacer = '\t');//outfile assumed open
	virtual void	 	PrintHapMarkerFile(aString& outFileName);
	virtual void 		PrintHapMarkerFile (ostream& outFile);
	virtual void	 	PrintNEXUSFile(aString& outFileName, StringList& textToAppend);
	virtual void 		PrintNEXUSFile (ostream& outFile, StringList& textToApend);
	virtual void 		PrintLociHap (ostream& outFile, int lociPerLine);//outfile assumed open
	virtual int 		ReadHaploidDataFile (aString& dataFileName);//input file assumed closed
//	virtual int 		ReadAndAppendHaploidDataFile (aString& dataFileName);//input file assumed closed
	virtual void		ResetFlags(void){
							HaploidMarkerData::ResetGeneticFlags();
							EvolutionaryGeneticsData::ResetFlags();}
	virtual void		ResetGeneticFlags(void){
							allelesCurrentHap_=varLociCurrentHap_=alleleFreqsCurrentHap_=0;}
	virtual void 		RestoreLociHap (aList<int>& lociToRestore);
	virtual void 		RestoreLociHap (StringList& lociToRestore);
	virtual void 		RestoreLocusHap (aString& locusToRestore);
	virtual void		SetAlleleLength(int allelelength){
							alleleLen_=allelelength;}
	virtual void 		SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList);
							//records all data, passed along from genetic analysis classes
							//empties old data and creates new data structures
	virtual void 		SetGenome(GeneContainerCore& gc, int pop, int indiv);
							//records data from HaplotypeContainer<int>
							//assumes blank data records already created in EvolutionaryGeneticsData
	void				SetLoci(aList<int>& lociToInclude);//constrains the haploid data set to these loci only
							//skipped loci status unchanged (use CullLoci to skip more; RestoreLoci to skip fewer)
	virtual int 		traits(void){return locusNumHap_.len();}	
	virtual int 		traits(int /*traitType*/){return HaploidMarkerData::traits();}	
	virtual void		UpdateVariableLociHap(void);
	virtual aString&	VariableLocusName(int loc){
							return traitName_[locusNumHap_[variableLocusNumHap_[loc]]];}

		};//end class HaploidMarkerData
		
		
#endif

