// PopulationGeneticData.h

#ifndef PopulationGeneticDatah
#define PopulationGeneticDatah = 1


#include "StringClass.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StringList.h"
#include "StatList.cpp"
#include "GenotypeFreqDistribution.h"
#include "GenotypeFreqMultiAlleleDist.h"
#include "GeneFrequencyDistribution.h"
#include "GeneFrequencyDistMultiAllele.h"
#include "ANOVAGenetics.h"
#include "GeneralLinearModel.h"
#include "aVector.h"
#include "aMatrix.h"
#include "MatrixList.h"
#include "PartitionedMatrix.h"
#include <fstream.h>
#include <iostream.h>
#include <math.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"


//enum hetType {observed=0, expected=1};

//const char endChr = '\n';
//const char endChr = '\r';

	class PopulationGeneticData{
		public://hetTypes
			const static int observed=0;
			const static int expected=1;
		public:
			aString title_;	//title_ of the data set from the first data line
			long ngroups, npop_, nloci, maxall,  variableLoci;
			int ploidy_, alleleLen_;
			aList<int> popSize;
			StringList groupName, popName, localityName, rawLocusName, locusName, skippedLocusName;
			ObjList<aList<int> > groupHierarchy, popsByLocality;	
					//each group gets its own list of populations, by number
			int varLociCurrent_;//boolean
			aList<int> skippedLocusNum, variableLoci_;
			int lociSkipped;
			ObjList<StringList> rawGtypes;//as read in from data file
			ObjList<StringList> gtypes;//after skipped loci are culled
					//list of pops, each with list of individual genotype strings
			int gtypesCurrent_;//boolean
			ObjList<ObjList<ObjList<StringList> > > gtypesPopIndLocAll;//after skipped loci are culled
					//single-locus genotypes, indexed [pop][indiv][loc][allele]
			int allelesCurrent_;//boolean
			ObjList<StringList> allelesPresent;
					//list of loci, each with list of all alleles encountered
			int alleleFreqCurrent_;//boolean
			ObjList<ObjList<aList<long double> > > alleleFreq;	
					//list of pops, each with list of loci, each with list of allele freqs
			ObjList<aList<int> > twoN;	
					//list of pops, each with list of sample sizes for each locus
			int HetObsCurrent_;//boolean
			ObjList<ObjList<aList<long double> > > HetObsByAllele;	
					//list of pops, each with list of loci, each with list of observed heterozygosities
			ObjList<aList<long double> > HetObsByLocus;	
					//list of pops, each with list of observed heterozygosities for each locus
			aList<long double> diseqD;
			ObjList<ObjList<aList<long double> > > locusD;	
					//list of pops, each with list of gametic disequilibria for each locus
		public:
		PopulationGeneticData(void): ngroups(0),npop_(0), nloci(0), maxall(0), ploidy_(2),alleleLen_(1), variableLoci(0), 
							varLociCurrent_(0),gtypesCurrent_(0), allelesCurrent_(0), alleleFreqCurrent_(0), 
							lociSkipped(0), HetObsCurrent_(0), diseqD() {}
		PopulationGeneticData(aString& s): title_(s), ngroups(0),npop_(0), nloci(0), maxall(0), ploidy_(2),alleleLen_(1), 
							variableLoci(0), varLociCurrent_(0), gtypesCurrent_(0), allelesCurrent_(0), lociSkipped(0), 
							alleleFreqCurrent_(0), HetObsCurrent_(0), diseqD() {}
virtual	~PopulationGeneticData(void){
			Clear();
			}

		PopulationGeneticData(PopulationGeneticData& parentData);//copy constructor

		PopulationGeneticData(PopulationGeneticData& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(PopulationGeneticData& pgd);

	int 		alleleLen(void){return alleleLen_;}
	void		Calculate2LocusDisequilibrium(void);
	void		CalculateAlleleFreqs (void);
	ObjList<aList<long double> >* 
				CalculateAlleleFreqsByPopSet(aList<int>& popSet, aList<double>& twoNList);
	void		CalculateObservedHeterozygosity (void);
	void		CalculateTwoN(void);
	void 		CheckForNewAlleles (void);
	void 		CheckForNewHaplotypes (void);
	virtual void		Clear(void);//clears all information
	void		CreateBootstrapData(PopulationGeneticData& origData, Ran3& ran);//individuals resampled
	void 		CreateGtypeList(void);
	virtual void 		CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	virtual void 		CreateRandomData(//multiple alleles per locus, variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran);
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void 		CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void 		CreateRandomDataLocFirst(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void 		CullLoci (StringList& lociToCull);
	void 		CullLocus (aString& locusToCull);
	void 		CullLocus (char* locusToCull);
	void 		CullMonomorphicLoci (void);
	void 		CullSkippedLoci (void);
	virtual PopulationGeneticData* 		
				CullToOneGroup (int groupNum);//makes a subset data structure from the specified group
	void	FitAllozymeCline(aString& dataFileName);
	StringList& 	Genotype(int pop, int indiv, int loc);
	aString& 	Genotype(int pop, int indiv, int loc, int allele);
	int 		GenotypeSize(void){return alleleLen_ * ploidy_;}
	void 	GenotypeTable(ObjList<MatrixList>& gtypeTableList);//stored pops within loci; square matrices of genotypes
	long double 	GetExpectedHeterozygosity (int pop, int loc, int allele);
	long double 	GetExpectedHeterozygosity (int pop, int loc);
	long double 	GetExpectedHeterozygosity (int pop);//returns mean over loci
	long double 	GetFreq (int loc, int allele);//pooling populations
	long double 	GetFreq (int pop, int loc, int allele);
	long double 	GetFreqMostCommonAllele (int pop, int loc);
	aString* 	GetGenotypeString(int pop, int indiv);//drops code & returns new string; must be deleted elsewhere
	aString* 	GetGenotypeString(int pop, int indiv, int loc);//returns new string; must be deleted elsewhere
	aString* 	GetGenotypeString(int pop, int indiv, int loc, int chrom);//returns new string; must be deleted elsewhere
	long 	GetIndividualsSampled (int pop, int loc);
	int	 	GetLocality (int pop);
	long 		GetLocusSampleSize (int loc);//pooling populations
	long 		GetLocusSampleSize (int pop, int loc);
	int	 	GetLocusSampleSize (int pop, int indiv, int loc);//returns ploidy
	long double	GetMeanAllelesPerLocus (int pop);
	int	 	GetNumAlleles (int loc);//gives total in data set
	int	 	GetNumAlleles (int pop, int loc);//gives extant alleles in pop
	long double 	GetObservedHeterozygosity (int pop, int loc, int allele);
	long double 	GetObservedHeterozygosity (int pop, int loc);
	long double 	GetObservedHeterozygosity (int pop);//returns mean over loci
	long double 	GetPercentPolymorphicLoci (int pop, double criterion = 100.0);
	long double	GetVarJackAllelesPerLocus (int pop);//returns jackknife variance
	long double 	GetVarJackExpectedHeterozygosity (int pop);//returns jackknife variance
	long double 	GetVarJackObservedHeterozygosity (int pop);//returns jackknife variance
	long double 	GetVarJackPercentPolyLoci (int pop, double criterion = 100.0);
	virtual void 		GroupMembership (void);
	virtual int 	 	GroupMembership (int pop);
	int 		individuals(int pop){return gtypes[pop].len();}
	int 		individuals(int pop, int loc);
	int		IsPolymorphic(int pop, int loc);//boolean
	int		IsPolymorphic(int pop, int loc, double criterion);//boolean
	int 		localities(void){return popsByLocality.len();}
	void 		LocalityMembership (void);
	int	 	LocalityMembership (int pop);
	virtual int 	loci(void){return nloci;}
	int 		Ploidy(int pop, int indiv, int loc){return GetLocusSampleSize(pop, indiv, loc);}
	void 		PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer = ' ');//outfile assumed open
	void  		PrintAlleleFreqsByPopList (ostream& outFile, 
										ObjList<ObjList<aList<double> > >& popFreqList, 
										ObjList<aList<double> >& popTwoNList, 
										int ncols);
	virtual void	 	PrintArlequinFile(aString& outFileName);
	virtual void 		PrintArlequinFile (ostream& outFile);
	virtual void	 	PrintGelJockFile(aString& outFileName);
	virtual void 		PrintGelJockFile (ostream& outFile);
	void 		PrintHeterozygosity (ostream& outFile, const int hetType, int ncols);//outfile assumed open
	virtual void 		PrintLoci (ostream& outFile, int lociPerLine);//outfile assumed open
	virtual void 		PrintPopKey (ostream& outFile, char colSpacer = ' ');//outfile assumed open
	void		PrintSASFormatData(aString& outFileName);//output file assumed closed
	void		PrintSASFormatData(ostream& outFile);//outfile assumed open
				//missing data represented by '.', columns separated by spaces
	void 		PrintVariabilityScores(aString& outFileName, char sepChar='\0');
	void 		PrintVariabilityScores(ostream& outFile, char sepChar='\0');
	virtual void 		ReadArlequinDataFile (aString* dataFileName);//input file assumed closed
	virtual void 		ReadGelJockDataFile (aString* dataFileName);//input file assumed closed
	virtual void 		ReadGelJockmtDNADataFile (aString* dataFileName);//input file assumed closed
	virtual void		ResetFlags(void){
				varLociCurrent_=gtypesCurrent_=allelesCurrent_=0;
				alleleFreqCurrent_=HetObsCurrent_=0;}
	virtual void 		RestoreLoci (StringList& lociToRestore);
	virtual void 		RestoreLocus (aString& locusToRestore);
	virtual void 		RestoreLocus (char* locusToRestore);
	virtual void 		SnipArlequinComment(aString& arqStr);
	virtual void		UpdateVariableLoci(void);
	virtual void		UpdateVariableHaploidLoci(void);
	PopulationGeneticData& operator=(PopulationGeneticData& pgd);
#ifdef MemoryLeakDebug
	static void PrintIDList(ostream& outFile=cout);
	static void PrintIDListLen(ostream& outFile=cout);
#endif
		};//end class PopulationGeneticData
		
		
#endif

