/*********************************************

	 PopulationPhenotypicData.h

**********************************************/


#ifndef PopulationPhenotypicDatah
#define PopulationPhenotypicDatah = 1


#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "StringList.h"
#include "aVector.h"
#include <fstream.h>
#include <iostream.h>
#include <math.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"


	class PopulationPhenotypicData{
		public:
			aString title;	//title of the data set from the first data line
			long ngroups, npop_, ntraits;
			aList<int> popSize;
			StringList groupName, popName, localityName, rawTraitName, traitName, skippedTraitName;
			ObjList<aList<int> > groupHierarchy, popsByLocality;	
					//each group gets its own list of populations, by number
			aList<int> skippedTraitNum;
			int traitsSkipped;

			ObjList<StringList> rawTraits;//as read in from data file
			ObjList<StringList> traitStrs;//after skipped traits are culled
					//list of pops, each with list of individual phenotype strings
			ObjList<ObjList<aList<long double> > > traits_;	
					//list of pops, each with list of individuals, each with list of trait values
			ObjList<ObjList<aList<int> > > traitsScored;//to handle missing data
					//list of pops, each with list of individuals, each with list of traits scored
			ObjList<ObjList<StatList<long double> > > popTraitStats;	
					//list of pops, with list of traits, each with StatList of scores
		public:
		PopulationPhenotypicData(void): ngroups(0),npop_(0), ntraits(0), traitsSkipped(0){
			}
		PopulationPhenotypicData(aString& s): title(s), ngroups(0),npop_(0), ntraits(0),traitsSkipped(0) {
			}
virtual	~PopulationPhenotypicData(void){
			Clear();
			}

		PopulationPhenotypicData(PopulationPhenotypicData& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets

	void		Clear(void);//clears all information
	void 		CullSkippedTraits (void);
	void 		CullTraits (StringList& traitsToCull);
	void 		CullTrait (char* traitToCull);
	void 		CullTrait (aString& traitToCull);
	aString* 	GetPhenotypeString(int pop, int indiv);//drops code & returns new string; must be deleted elsewhere
	aString* 	GetPhenotypeString(int pop, int indiv, int trait);//returns new string; must be deleted elsewhere
	int	 		GetLocality (int pop);
	long 		GetTraitSampleSize (int pop, int trait);
	aString*	GetTraitStr(int pop, int indiv, int trait);
	void 		GroupMembership (void);
	int 	 	GroupMembership (int pop);
	int 		individuals(int pop){return traitStrs[pop].len();}
	int 		individuals(int pop, int trait);
	int 		localities(void){return popsByLocality.len();}
	void 		LocalityMembership (void);
	int	 		LocalityMembership (int pop);
	int			npop(){return npop_;}
	void 		PrintPhenotypeFile(aString& outFileName);
	void 		PrintPhenotypeFile (ostream& outFile);
	void 		PrintTraits (ostream& outFile, int lociPerLine);//outfile assumed open
	void 		PrintPopKey (ostream& outFile);//outfile assumed open
	void		PrintSASFormatData(aString& outFileName);//output file assumed closed
	void		PrintSASFormatData(ostream& outFile);//outfile assumed open
				//missing data represented by '.', columns separated by spaces
	void 		PrintVariabilityScores(aString& outFileName, char sepChar='\0');
	void 		PrintVariabilityScores(ostream& outFile, char sepChar='\0');
	void 		ReadDataFile (aString* dataFileName);//input file assumed closed
	void		ReadTraitStrings(void);
	void		ResetFlags(void){}
	void 		RestoreTraits (StringList& traitsToRestore);
	void 		RestoreTrait (aString& traitToRestore);
	void 		RestoreTrait (char* traitToRestore);
	int 		traits(void){return ntraits;}
	virtual aVector<long double>* traitVector(int pop, int indiv);//vector of scores for each trait
		};//end class PopulationPhenotypicData
		
		
#endif

