// SexLinkedDomMarkerData2.h

// Holds phenotypic data for dominant sex-linked or haplodiploid markers

#ifndef SexLinkedDomMarkerData2h
#define SexLinkedDomMarkerData2h = 1


#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "EvolutionaryGeneticsData.h"
#include "SexLinkedData.h"
//#include "SexLinkedCodomMarkerData.h"
#include "GenotypeFreqDistSexLinkDom.h"
#include "RandomList.h"
#include "SortList.h"
#include "MemoryManager.h"
#include <fstream.h>

class SexLinkedCodomMarkerData;

class SexLinkedDomMarkerData2: public virtual SexLinkedData,
							   public virtual EvolutionaryGeneticsData{
		public:
			long nlociSLDom_;//for dominant marker loci
			int ploidy_, alleleLen_;//ploidy_ holds homogametic if sexes differ
			aList<int> allSLDomLoci_,locusNumSLDom_, skippedLocusNumSLDom_;
					//these hold places in (=indices of) EvolutionaryGeneticsData::traitName_[]
			aList<int> variableLocusNumSLDom_;//holds places in (=indices of) locusNumSLDom_[]
//			ObjList<ObjList<TextDataList> > rawPhenotypes;//{0,1} presence absence data as read from data file
					//list of pops, each with list of individual phenotype strings
			ObjList<StringList> allelesPresentSLDom_;//corresponds to order of locusNumSLDom_[]
					//list of loci, each with sorted list of all alleles encountered
					//loci correspond to order of locusNumSLDom_[]
			ObjList<ObjList<aList<long double> > > phenotypeFreqSLDom_;//0 is recessive, 1 dominant
					//indexed [pop][loc][marker]
					//loci correspond to order of locusNumSLDom_[]
					//list of pops, each with list of dominant marker loci, each with list of marker freqs
			ObjList<aList<long double> > phenotypeFreqSLDomGlobal_;//0 is recessive, 1 dominant
					//indexed [loc][marker]
					//loci correspond to order of locusNumSLDom_[]
					//list of dominant marker loci, each with list of marker freqs
			ObjList<ObjList<aList<int> > > phenotypeCountSLDom_;//0 is recessive, 1 dominant
					//indexed [pop][loc][marker]
					//loci correspond to order of locusNumSLDom_[]
					//list of pops, each with list of dominant marker loci, each with list of marker freqs
			ObjList<aList<int> > phenotypeCountSLDomGlobal_;//0 is recessive, 1 dominant
					//indexed [loc][marker]
					//loci correspond to order of locusNumSLDom_[]
					//list of dominant marker loci, each with list of marker freqs
			ObjList<ObjList<aList<long double> > > phenotypeFreqSLHeterogametic_,phenotypeFreqSLHomogametic_;//separately for each sex
					//list of pops, each with list of loci, each with list of allele freqs
			ObjList<aList<long double> > phenotypeFreqSLHeterogameticGlobal_,phenotypeFreqSLHomogameticGlobal_;//separately for each sex
					//list of loci, each with list of allele freqs
			ObjList<ObjList<aList<int> > > phenotypeCountSLHeterogametic_,phenotypeCountSLHomogametic_;//separately for each sex
					//list of pops, each with list of loci, each with list of allele freqs
			ObjList<aList<int> > phenotypeCountSLHeterogameticGlobal_,phenotypeCountSLHomogameticGlobal_;//separately for each sex
					//list of loci, each with list of allele freqs
			ObjList<aList<int> > nByLocusSLDom_;//dominant markers only	
					//indexed [pop][loc]
					//loci correspond to order of locusNumSLDom_[]
					//list of pops, each with list of #individuals sampled for each locus
			ObjList<aList<int> > nByLocusSLHeterogametic_,nByLocusSLHomogametic_;	//for each sex
					//list of pops, each with list of sample sizes for each locus
			int allelesCurrentSLDom_;
			int varLociCurrentSLDom_;//boolean
			int phenotypeFreqsCurrentSLDom_;//boolean
		public:
		SexLinkedDomMarkerData2(void): EvolutionaryGeneticsData(),nlociSLDom_(0),
							ploidy_(2),alleleLen_(1),varLociCurrentSLDom_(0),
							allelesCurrentSLDom_(0),phenotypeFreqsCurrentSLDom_(0) {
			EvolutionaryGeneticsData::Initialize();
			}
		SexLinkedDomMarkerData2(aString& s): EvolutionaryGeneticsData(s),nlociSLDom_(0),
							ploidy_(2),alleleLen_(1),varLociCurrentSLDom_(0),
							allelesCurrentSLDom_(0),phenotypeFreqsCurrentSLDom_(0){
			EvolutionaryGeneticsData::Initialize(s);
			}

virtual	~SexLinkedDomMarkerData2(void){
			Clear();
			}

		SexLinkedDomMarkerData2(SexLinkedDomMarkerData2& parentData);//copy constructor

		SexLinkedDomMarkerData2(SexLinkedDomMarkerData2& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets
	//Initialize() functions for handling construction of shared virtual base classes
	void		Initialize(void);
	void		Initialize(aString& s);
	void		Initialize(SexLinkedDomMarkerData2& pgd);
	void		Initialize(SexLinkedDomMarkerData2& pgd, aList<int>& popSubsetNums);

	SexLinkedDomMarkerData2&	operator=(SexLinkedDomMarkerData2& pgd);


	EvolutionaryGeneticsData* Duplicate(void){
								return NEW SexLinkedDomMarkerData2(*this);}

	virtual int alleleLen(void) {return alleleLen_;}
	virtual int alleleID(int loc, TextDatum& alleleVal){//#of this allele in locus list
						if(!allelesCurrentSLDom_) {CheckForNewAlleles();}
						return allelesPresentSLDom_[loc].Loc(&alleleVal);}//location of the allele
	virtual int alleles(int loc){
						if(!allelesCurrentSLDom_) {CheckForNewAlleles();}
						return allelesPresentSLDom_[loc].len();}//number of alleles
	void		CalculatePhenotypeFreqs (void);
	ObjList<aList<long double> >* 
				CalculatePhenotypeFreqsByPopSet(aList<int>& popSet, aList<long double>& nList);
	void 		CheckForNewAlleles (void);
	virtual void		Clear(void);//clears all information
	void 		ConvertCodominantData(SexLinkedCodomMarkerData& codomData);
						//appends converted data; don't let codomData be part of the same data object
	void CreateBootstrapData(SexLinkedDomMarkerData2& origData, Ran3& ran);
						//pops & individuals resampled keeping multilocus genotypes intact
	void CreateBootstrapDataConstPop(SexLinkedDomMarkerData2& origData, Ran3& ran);
						//individuals resampled keeping multilocus genotypes intact
	void CreateRandomData(const long double Fst, 
							const long double Fis,
							const long double pBar,  //just one locus ->dominant marker frequency (not allele freq)
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void CreateRandomData(//dominant marker freqs (not allele freqs) that vary among loci; variable sample sizes per pop
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& betaBar, 
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran);
	void CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double betaBar, //just one locus ->dominant marker frequency (not allele freq)
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void CreateRandomData(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double betaBar,//just one locus ->dominant marker frequency (not allele freq)
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void CreateRandomDataLocFirst(const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double betaBar, //just one locus ->dominant marker frequency (not allele freq) 
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran);
	void 		CullDomLoci (StringList& lociToCull);
	void 		CullDomLoci (aList<int>& lociToCull);
	void 		CullDomLocus (aString& locusToCull);
	void 		CullDomLocus (char* locusToCull);
	void 		CullMonomorphicLociDom (void);
/*	virtual EvolutionaryGeneticsData* 		
				CullToOneGroup (int groupNum);//makes a subset data structure from the specified group
*/
	void 		CullToOneSex (SexLinkedDomMarkerData2& parentData, genderSet sexVal);//makes a subset data structure from the specified group
	void 		CullToTheseSexes (SexLinkedDomMarkerData2& parentData, aList<int>& sexValList);//makes a subset data structure from the specified group
	long double sexLinkedDominantAlleleFreqFromFisBeta (long double beta, long double FisVal);
	void 		ExpungeLoci (StringList& lociToCull);//transfer to skipped
	void 		ExpungeLoci (aList<int>& lociToCull);
	void 		ExpungeLocus (aString& locusToCull);
	void 		ExpungeLocus (char* locusToCull);
	int 		GenotypeSize(void){return alleleLen_* ploidy_;}//scale to homogametic
	long double GetExpectedHeterozygosity (int pop, int loc, int allele);
	long double GetExpectedHeterozygosity (int pop, int loc);

	int 		GetCountDom (int loc, int marker){
						if(!phenotypeFreqsCurrentSLDom_){CalculatePhenotypeFreqs();}
						return phenotypeCountSLDomGlobal_[loc][marker];}
	int			 GetCountDom (int pop, int loc, int marker){
						if(!phenotypeFreqsCurrentSLDom_){CalculatePhenotypeFreqs();}
						return phenotypeCountSLDom_[pop][loc][marker];}
	int			 GetCountDomHeterogametic (int pop, int loc, int marker){
						if(!phenotypeFreqsCurrentSLDom_){CalculatePhenotypeFreqs();}
						return phenotypeCountSLHeterogametic_[pop][loc][marker];}
	int			 GetCountDomHeterogametic (int loc, int marker){
						if(!phenotypeFreqsCurrentSLDom_){CalculatePhenotypeFreqs();}
						return phenotypeCountSLHeterogameticGlobal_[loc][marker];}
	int			 GetCountDomHomogametic (int pop, int loc, int marker){
						if(!phenotypeFreqsCurrentSLDom_){CalculatePhenotypeFreqs();}
						return phenotypeCountSLHomogametic_[pop][loc][marker];}
	int			 GetCountDomHomogametic (int loc, int marker){
						if(!phenotypeFreqsCurrentSLDom_){CalculatePhenotypeFreqs();}
						return phenotypeCountSLHomogameticGlobal_[loc][marker];}

	long double 	GetFreqDom (int loc, int marker);//uses weighted aveage over loci
	long double 	GetUnweightedFreqDom (int loc, int marker){
							if(!phenotypeFreqsCurrentSLDom_){CalculatePhenotypeFreqs();}
							return phenotypeFreqSLDomGlobal_[loc][marker];}
	long double 	GetFreqDom (int pop, int loc, int marker);
	long double 	GetFreqDomHeterogametic (int pop, int loc, int marker);
	long double 	GetFreqDomHomogametic (int pop, int loc, int marker);
	aString* 	GetTraitString(int pop, int indiv, int loc);//loc = index of locusNumSLdom_
	aString* 	GetTraitString(int pop, int indiv, int loc, int /*allele*/);
	long double 	GetUnweightedFreqDomHeterogametic (int loc, int marker){
							if(!phenotypeFreqsCurrentSLDom_){CalculatePhenotypeFreqs();}
							return phenotypeFreqSLHeterogameticGlobal_[loc][marker];}
	long double 	GetUnweightedFreqDomHomogametic (int loc, int marker){
							if(!phenotypeFreqsCurrentSLDom_){CalculatePhenotypeFreqs();}
							return phenotypeFreqSLHomogameticGlobal_[loc][marker];}

	long double 	GetFreqMostCommonAllele (int pop, int loc);
/*	long 		GetLocusSampleSize (int loc);//pooling populations
*/
	long 		GetIndividualsSampledDom (int pop, int loc);
	void		GetLocusNames(StringList& names){
					TraitNamesFromSublist(names,locusNumSLDom_);}
	long 		GetLocusSampleSizeDom (int loc);// loc refers to dominant loci only
	long 		GetLocusSampleSizeDom (int pop, int loc);// loc refers to dominant loci only
	long 		GetLocusSampleSizeDom (int pop, int indiv, int loc);// loc refers to dominant loci only
	long 		GetLocusSampleSizeDomHeterogametic (int pop, int loc);// loc refers to dominant loci only
	long 		GetLocusSampleSizeDomHomogametic (int pop, int loc);// loc refers to dominant loci only
	int 		GetNumAllelesDom (int loc);// loc refers to dominant loci only
	int 		GetNumAllelesDom (int pop, int loc);// loc refers to dominant loci only
	int 		individuals(int pop){return EvolutionaryGeneticsData::individuals(pop);}
	int 		individuals(int pop, int loc);// loc refers to dominant loci only
	int			IsPolymorphic(int loc){
							if(! allelesCurrentSLDom_){CheckForNewAlleles();}
							if(allelesPresentSLDom_[loc].len()>1){return true;}
							return false;}
	int			IsPolymorphic(int loc, double criterion){
							if(! phenotypeFreqsCurrentSLDom_){CalculatePhenotypeFreqs();}
							long double freq = phenotypeFreqSLDomGlobal_[loc][0];
							if(freq>(long double)0.5){
									if(freq >= (long double)1.0-criterion){return false;}}
								else{
									if(freq <= criterion){return false;}}
							return true;}
	int			IsPolymorphic(int pop, int loc){
							if(! phenotypeFreqsCurrentSLDom_){CalculatePhenotypeFreqs();}
							long double freq = phenotypeFreqSLDom_[pop][loc][0];
							if(freq==(long double)0.0 || freq == (long double)1.0){return false;}
							return true;}
	int			IsPolymorphic(int pop, int loc, double criterion){
							if(! phenotypeFreqsCurrentSLDom_){CalculatePhenotypeFreqs();}
							long double freq = phenotypeFreqSLDom_[pop][loc][0];
							if(freq>(long double)0.5){
									if(freq >= (long double)1.0-criterion){return false;}}
								else{
									if(freq <= criterion){return false;}}
							return true;}
	virtual int		loci(void){return nlociSLDom_;}
	virtual int		lociDom(void){return nlociSLDom_;}//for compatibility
	virtual int		lociSkipped(void){return skippedLocusNumSLDom_.len();}
	aString& 	locusNameDom(int loc){return traitName_[locusNumSLDom_[loc]];}
	virtual long	maxall(void);
	virtual TextDatum&	PhenotypicVal(int pop, int indiv, int loc){// loc refers to dominant loci only
							return rawTraits_[pop][indiv][locusNumSLDom_[loc]];}
	virtual int 	Ploidy(int pop, int indiv, int /*loc*/){
							if(heterogametic(pop,indiv)){ return ploidy_/2;}
							return ploidy_;}
	virtual int 	ploidy(void){return ploidy_;}
	virtual int 	ploidy(int pop, int indiv){return Ploidy(pop,indiv,0);}
	void 				PrintPhenotypeCounts (aString& outFileName, int ncols, char colSpacer = '\t');//outfile assumed open
	void 				PrintPhenotypeCountsBySex (aString& outFileName, int ncols, char colSpacer = '\t');//outfile assumed open
	void 				PrintPhenotypeCounts (ostream& outFile, int ncols, char colSpacer = '\t');//outfile assumed open
	void 				PrintPhenotypeCountsBySex (ostream& outFile, int ncols, char colSpacer = '\t');//outfile assumed open
	void 				PrintPhenotypeFreqs (aString& outFileName, int ncols, char colSpacer = '\t');//outfile assumed open
	void 				PrintPhenotypeFreqsBySex (aString& outFileName, int ncols, char colSpacer = '\t');//outfile assumed open
	void 				PrintPhenotypeFreqs (ostream& outFile, int ncols, char colSpacer = '\t');//outfile assumed open
	void 				PrintPhenotypeFreqsBySex (ostream& outFile, int ncols, char colSpacer = '\t');//outfile assumed open
	virtual void	 	PrintDomMarkerFile(aString& outFileName);
	virtual void 		PrintDomMarkerFile (ostream& outFile);
	virtual void	 	PrintNEXUSFile(aString& outFileName, StringList& textToAppend);
	virtual void 		PrintNEXUSFile (ostream& outFile, StringList& textToApend);
	virtual void 		PrintLociDom (ostream& outFile, int lociPerLine);//outfile assumed open
	virtual int 		ReadAFLPDataFile (aString& dataFileName);//input file assumed closed
//	virtual int 		ReadAndAppendAFLPDataFile (aString& dataFileName);//input file assumed closed
	long double			sexLinkedRecessiveAlleleFreqFromFisBeta(long double beta, long double FisVal){
								return (long double)1.0 - sexLinkedDominantAlleleFreqFromFisBeta(beta,FisVal);}
	virtual void		ResetFlags(void){
							SexLinkedDomMarkerData2::ResetGeneticFlags();
							SexLinkedData::ResetFlags();}
	virtual void		ResetGeneticFlags(void){//skips sex-linked flags
							allelesCurrentSLDom_=varLociCurrentSLDom_=phenotypeFreqsCurrentSLDom_=0;
							}
	virtual void 		RestoreLociDom (aList<int>& lociToRestore);
	virtual void 		RestoreLociDom (StringList& lociToRestore);
	virtual void 		RestoreLocusDom (aString& locusToRestore);
	void				SetLoci(aList<int>& lociToInclude);//constrains the sex-linked dominant data set to these loci only
							//skipped loci status unchanged (use CullLoci to skip more; RestoreLoci to skip fewer)
	virtual int 		traits(void){return locusNumSLDom_.len();}	
	virtual int 		traits(int /*traitType*/){return SexLinkedDomMarkerData2::traits();}	
	virtual void		UpdateVariableLociDom(void);
	virtual aString&	VariableLocusName(int loc){
							return traitName_[locusNumSLDom_[variableLocusNumSLDom_[loc]]];}
		};//end class SexLinkedDomMarkerData2
		
		
#endif

