/*****
GenotypeFreqDistSexLinkDom.h

Sex-linked dominant markers: p_ is the (dominant) allele frequency, but the class holds marker frequencies in the
order {dominant,recessive}.  The constructor should be set up like this:

GenotypeFreqDistribution someDist(dominantMarkerFreq, Fis, //SLdom=// true, //heterogameticSexIsMale=// true, 
			sexratio);

*****/

#ifndef GenotypeFreqDistSexLinkDomh
#define GenotypeFreqDistSexLinkDomh = 1


#include "aError.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "StringClass.h"
#include "StringList.h"
#include "RandomList.h"
#include "InlineTemplates.h"
#include "GenotypeFreqDistribution.h"
#include "GenotypeFreqDistDom.h"
#include <iostream.h>
#include "MemoryManager.h"


class GenotypeFreqDistSexLinkDom: public GenotypeFreqDistDom{
	public:
	bool heterogameticSexIsMale_;
	long double sr_;//sex ratio = F/(F+M)
	long double betaHomogametic_, betaHeterogametic_;
	aList<long double> heterogameticSublist_, homogameticSublist_;
	
	public:
	GenotypeFreqDistSexLinkDom(const long double betaBar, const long double Fis=(long double)0.0,
							 const bool heterogameticSexIsMale = true,
							 const long double sexRatio=(long double)0.5):
			GenotypeFreqDistDom(), sr_(sexRatio), heterogameticSexIsMale_(heterogameticSexIsMale){
					beta_=betaBar;
					Fis_=Fis;
					CreateDistributionSexLinkedDominantMarker();
					}//ctr
			
	GenotypeFreqDistSexLinkDom(void):
			GenotypeFreqDistDom(), sr_((long double) 0.5), heterogameticSexIsMale_(true){
					betaHomogametic_ = betaHeterogametic_ = (long double)0.0;
					}//ctr
			
	GenotypeFreqDistSexLinkDom(GenotypeFreqDistSexLinkDom& d):
					GenotypeFreqDistDom(d),
					sr_(d.sr_), heterogameticSexIsMale_(d.heterogameticSexIsMale_),
					heterogameticSublist_(d.heterogameticSublist_),
					betaHomogametic_(d.betaHomogametic_),betaHeterogametic_(d.betaHeterogametic_),
					homogameticSublist_(d.homogameticSublist_){
			}	
	GenotypeFreqDistSexLinkDom(GenotypeFreqDistSexLinkDom* d):
					GenotypeFreqDistDom(*d),
					sr_(d->sr_), heterogameticSexIsMale_(d->heterogameticSexIsMale_),
					heterogameticSublist_(d->heterogameticSublist_),
					betaHomogametic_(d->betaHomogametic_),betaHeterogametic_(d->betaHeterogametic_),
					homogameticSublist_(d->homogameticSublist_){
			}
					
	~GenotypeFreqDistSexLinkDom(void){}



	virtual GenotypeFreqDistribution* Duplicate(void){
		return NEW GenotypeFreqDistSexLinkDom(*this);
		}

	virtual void Initialize(long double betaBar, long double Fis=(long double)0.0){
				Initialize(betaBar,Fis,true,0.5);}
	virtual void Initialize(long double betaBar, long double Fis=(long double)0.0,
							bool heterogameticSexIsMale = true, long double sexRatio=(long double)0.5){
		Clear();
		beta_=betaBar;
		Fis_ = Fis;
		heterogameticSexIsMale_=heterogameticSexIsMale;
		sr_ = sexRatio;
		CreateDistributionSexLinkedDominantMarker();
		}


	virtual void CreateDistribution2(void){
				CreateDistributionSexLinkedDominantMarker();}
	void CreateDistributionSexLinkedDominantMarker(void);//0-1 data, sampled from Fis, sex ratio & marker freq
	virtual long double markerFrequency(int markerDom0recessive1, bool homogametic){
			//returns marker frequency for dominant/recessive markers
			if(homogametic){
					if(markerDom0recessive1==0){
							return betaHomogametic_;}
					return (long double)1.0 - betaHomogametic_;}
				else{
					if(markerDom0recessive1==0){
								return betaHeterogametic_;}
						return (long double)1.0 - betaHeterogametic_;}
			}
	virtual long double markerFrequency(int markerDom0recessive1, int sexFemale0male1){
			return markerFrequency(markerDom0recessive1, (bool) sexFemale0male1);}
//	void PrintToScreen(void);
	virtual int RandomGenotype(Ran3List& /*ran*/){
			aError("ERROR:  need to specify sex in when calling  sex-linked dominant RandomGenotype()");
			return 0;}
	virtual int RandomGenotype(Ran3List& ran, bool homogametic);
	void SetHeterogameticFemaleFMaleT(bool femaleFmaleT){
			if(heterogameticSexIsMale_ == femaleFmaleT) return;
			heterogameticSexIsMale_=femaleFmaleT;
			SWAP(homogameticSublist_,heterogameticSublist_);
			SWAP(betaHomogametic_,betaHeterogametic_);
			}
	void SetSexRatio(long double newSexRatio){
			if(sr_==newSexRatio) return;
			sr_=newSexRatio;
			//sex ratio affects genotype frequencies
			CreateDistributionSexLinkedDominantMarker();
			}
	
};//end GenotypeFreqDistSexLinkDom




#endif
