/*****
GenotypeFreqDistribution.h

Codominant diploid markers:  p_ is the allele frequency, and the class holds a list of genotype frequencies.

Derived classes hold dominant & sex-linked dominant markers
*****/

#ifndef GenotypeFreqDistributionh
#define GenotypeFreqDistributionh = 1


#include "aError.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "StringClass.h"
#include "StringList.h"
#include "RandomList.h"
#include "InlineTemplates.h"
#include <iostream.h>
#include "MemoryManager.h"


class GenotypeFreqDistribution: public StatList<long double>{
	public:
	long double p_;//always represents allele frequency
	long double Fis_;
	
	public:
	GenotypeFreqDistribution(const long double p, const long double Fis=(long double)0.0):
					p_(p), Fis_(Fis){
			CreateDistribution2();
			}//ctr
			
	GenotypeFreqDistribution(void){
			p_=Fis_ = (long double)0.0;
			}

	GenotypeFreqDistribution(GenotypeFreqDistribution& d):
					StatList<long double>(d), p_(d.p_), Fis_(d.Fis_){
			}	
	GenotypeFreqDistribution(GenotypeFreqDistribution* d):
					StatList<long double>(d), p_(d->p_), Fis_(d->Fis_){
			}
					
	~GenotypeFreqDistribution(void){
			}

	virtual GenotypeFreqDistribution* Duplicate(void){
		return NEW GenotypeFreqDistribution(*this);
		}

	virtual void Initialize(long double p, long double Fis=(long double)0.0){
		Clear();
		p_=p;
		Fis_ = Fis;
		CreateDistribution2();
		}

//	void CreateDistribution(void);//buggy for Fis<0 when p_ != 0.5
	virtual void CreateDistribution2(void);
	virtual long double alleleFrequency(int alleleVal);//returns marker frequency for sex-lnked dominant
	virtual void PrintToScreen(void);
	virtual int RandomGenotype(Ran3List& ran){
			return (int) ran.weighted(*this);}
	virtual int RandomGenotype(Ran3List& ran, bool /* homogametic*/){
			return RandomGenotype(ran);}
	

};//end GenotypeFreqDistribution




#endif
