// AllozymeClineShapeData.cp;
//This program calculates cline shape by fitting data in the form (x,y) to
//a composite sigmoid function.  The theory is in Szymura & Barton 1986.
//The program reads in a data set of allozymes and calculates the hybrid
//index and gametic disequilibria for each population.
#ifndef AllozymeClineShapeDatacp
#define AllozymeClineShapeDatacp = 1


#include "AllozymeClineShapeData.h"




void AllozymeClineShapeData::CalculateHybridIndex (void){
	//calcuate the harmonic mean population size for weighting
	popLocation.Clear();
	hybridIndex.Clear();
	StatList<int> sizePop(popSize);
	double amongPopHarMean = sizePop.harmonicMean();

	//get the population's location from the end of the title string
	for (int pop=0;pop<npop;++pop){
		aString *popTitleWord = popName[pop];
		aString popTitleWordCopy(popTitleWord);
		aString *popXword = popTitleWordCopy.ChopOffLastWord();
		double popX = popXword->toDouble();
		popLocation.Append(popX);
		delete popXword;popXword=0;
		//get each individual's genotype
		StringList* genotypesList = gtypes[pop];
		for(long i=0;i<genotypesList->len();++i){
			double indexVal = 0.0;
			long nAlleles = 0;
			for (int loc=0;loc<nloci;++loc){
				StringList *locusList = diagnosticAllelesR[loc];
				for (int chrom=1;chrom<=2;++chrom){
					aString *gene =GetGenotypeString(pop, i, loc, chrom);
					if ((*gene != ' ') && (gene->len()>0)) {
						++nAlleles;
						if (locusList->Includes(gene)){
							++indexVal;}
						}//end if gene<>blank
					delete gene;gene=0;
					}//end for chrom
				}//end for loc
			if (nAlleles > 0){
					indexVal /= nAlleles;
//					hybridIndex.Append(indexVal, nAlleles, popX, amongPopHarMean/sizePop[pop]);
					hybridIndex.Append(indexVal, nAlleles, popX, 1.0);
					}
				else{//no data for this individual
					hybridIndex.Append(0.0, 0, popX,0.0);//zero weight ensures it will be ignored later
					}//end if nAlleles					
			}//end for i
		//count the diagnostic alleles at each locus
		//divide by the number of alleles encountered
		}//end for pop
	sizePop.Clear();
	}//end AllozymeClineShapeData::CalculateHybridIndex


void AllozymeClineShapeData::CalculateHybridIndexByLocus (void){
	//calcuate the harmonic mean population size for weighting
	popLocation.Clear();
	hybridIndexByLocus.Clear();
	StatList<int> sizePop(popSize);
	double amongPopHarMean = sizePop.harmonicMean();


	for (int loc=0;loc<nloci;++loc){
		//create and initialize by-locus data structure
		HybridIndexByLocData* hiByLocus = new HybridIndexByLocData();
		hybridIndexByLocus.Append(hiByLocus);}

	//get the population's location from the end of the title string
	for (int pop=0;pop<npop;++pop){
		aString *popTitleWord = popName[pop];
		aString popTitleWordCopy(popTitleWord);
		aString *popXword = popTitleWordCopy.ChopOffLastWord();
		double popX = popXword->toDouble();
		popLocation.Append(popX);
		delete popXword;popXword=0;
		//get each individual's genotype
		StringList* genotypesList = gtypes[pop];
		for(long i=0;i<genotypesList->len();++i){
			for (int loc=0;loc<nloci;++loc){
				StringList *locusList = diagnosticAllelesR[loc];
				double indexVal = 0.0; long nAlleles = 0;
				for (int chrom=1;chrom<=2;++chrom){
					aString *gene =GetGenotypeString(pop, i, loc, chrom);
					if ((*gene != ' ') && (gene->len()>0)) {
						++nAlleles;
						if (locusList->Includes(gene)){
							++indexVal;}
						}//end if gene<>blank
					delete gene;gene=0;
					}//end for chrom
				HybridIndexByLocData* hi = hybridIndexByLocus[loc];
				if (nAlleles > 0){
						indexVal /= nAlleles;
//						hi->Append(indexVal, nAlleles, popX, amongPopHarMean/sizePop[pop]);
						hi->Append(indexVal, nAlleles, popX, 1.0);
						}
					else{//no data for this individual at this locus
						hi->Append(0.0, 0, popX,0.0);//zero weight ensures it will be ignored later
						}//end if nAlleles					
				}//end for loc
			}//end for i
		//count the diagnostic alleles at each locus
		//divide by the number of alleles encountered
		}//end for pop
	sizePop.Clear();
	}//end AllozymeClineShapeData::CalculateHybridIndexByLocus



void	 AllozymeClineShapeData::IncrementSuccessfulTries (const int paramChanged){
	int nParams = acceptedChanges.len();
	if(nParams<paramChanged+1){
		for(int i=0;i<paramChanged+1-nParams;++i){
			acceptedChanges.Append(0);
			}}
	acceptedChanges.Increment(paramChanged);
	}//end AllozymeClineShapeData::IncrementSuccessfulTries



void	 AllozymeClineShapeData::IncrementTotalTries (const int paramChanged){
	int nParams = totalTries.len();
	if(nParams<paramChanged+1){
		for(int i=0;i<paramChanged+1-nParams;++i){
			totalTries.Append(0);
			acceptedChanges.Append(0);
			}}
	totalTries.Increment(paramChanged);
	}//end AllozymeClineShapeData::IncrementTotalTries



void	 AllozymeClineShapeData::InitializeCounters (const int params){
	acceptedChanges.Clear(); totalTries.Clear();//reset these counters
	allTries=0;
	for (int i=0; i<params;++i){ 
		acceptedChanges.Append(0);
		totalTries.Append(0);}
	}//end AllozymeClineShapeData::InitializeCounters




int  AllozymeClineShapeData::PoolDiagnosticAlleles (void){
	ObjList<StringList>* gtypesCopy= new ObjList<StringList>(gtypes);
	gtypes.Clear();
	rawDiagnosticAllelesR.Clear();
	diagnosticAllelesR.Clear();

	//Determine what the diagnostic alleles will be for the right side of the transect.
	//These will be found tacked onto the ends of the locus names, in parentheses;
	//the remainder will be diagnostic for the left side.
	for (int loc=0;loc<nloci;++loc){
		StringList *locusList = new StringList();
		rawDiagnosticAllelesR.Append(locusList);
		aString *locusWord = locusName[loc];
		int locLparen = locusWord->pos('(');
		int locRparen = locusWord->pos(')');
		if(locLparen==0){
			cout<<"Data formatted improperly."<<endl;
			cout<<"The locus names need to have the diagnostic alleles indicated,"<<endl;
			cout<<"using the form LOC1(B;C), where alleles B and C are diagnostic"<<endl;
			cout<<"for the 'pure' genotypes on the right side of the cline.  Note that"<<endl;
			cout<<"they are separated by a semicolon within the parentheses."<<endl;
			return 0;}
		aString *locusStr = new aString(locusWord, locLparen + 1, locRparen - 1);
		while (locusStr->len() > 0){
			int locSemicolon = locusStr->pos(';');
			aString *alleleWord;// = new aString(locusStr,1,locSemicolon-1);
			if (locSemicolon > 0){
					alleleWord = new aString(locusStr,1,locSemicolon-1);
					locusStr->cut(0,locSemicolon);}
				else{
					alleleWord = new aString(locusStr);
					locusStr->clear();}
			locusList->Append(alleleWord);
			}//end while
		}//end for loc


	//Go through each individual and change its genotype.
	//An allele diagnostic on the left side is A, and on the right side is B.
	//Alleles unique to the central part of the zone are pooled with the left side as A
	for (int  pop=0;pop<npop;++pop){
		StringList *genotypesList = gtypesCopy->Nth(pop);
		StringList *newGtypesList = new StringList();
		gtypes.Append(newGtypesList);
		for (long i=0;i<genotypesList->len();++i){
			aString *dataStr = genotypesList->Nth(i);
			aString *dataStrCopy = new aString(dataStr);
			aString *codeWord = dataStrCopy->ChopOffFirstWord();
			int codeLen = codeWord->len();
			delete dataStrCopy;dataStrCopy=0;
			dataStrCopy = dataStr;
			aString *newGtypeWord = new aString(*codeWord);
			newGtypeWord->PrependCharToLen(' ',4);
			newGtypeWord->cat(' ');
			newGtypesList->Append(newGtypeWord);
			for (int loc=0;loc<nloci;++loc){
				StringList *locusList =rawDiagnosticAllelesR[loc];
				int posGtypeWord = dataStrCopy->pos(*codeWord) + codeLen + 1 + (3 * loc);
				aString gtypeWord(dataStrCopy, posGtypeWord, posGtypeWord+1);
				for (int chrom=1;chrom<=2;++chrom){
					aString gene = aString(gtypeWord, chrom, chrom);
					if ((gene == ' ') || (gene.len()==0)) {
							newGtypeWord->cat(' ');}
						else{
							if (locusList->Includes(&gene)) {
									newGtypeWord->cat('B');}
								else{
									newGtypeWord->cat('A');}
						}//end if gene==blank...else
					}//end for chrom
				if( loc<nloci-1){newGtypeWord->cat(' ');}
				}//end for loc
			delete codeWord;codeWord=0;
			}//end for i
		}//end for pop

	//now replace the diagnostic alleles list for the revised data
	for (loc=0;loc<nloci;++loc){
		StringList *rawLocusList = rawDiagnosticAllelesR[loc];
		StringList *locusList = new StringList();
		diagnosticAllelesR.Append(locusList);
		if(rawLocusList->len()>0){//does nothing for monomorphic loci
			aString *allName = new aString('B');
			locusList->Append(allName);}
		}//end for loc
	gtypesCopy->Clear();
	delete gtypesCopy;gtypesCopy=0;
	ResetFlags();
	return 1;
	}//end AllozymeClineShapeData::PoolDiagnosticAlleles



void AllozymeClineShapeData::PrintHybridIndexFile (aString& outFileName){
	fstream outFile;
	char* filename = outFileName.raw();
	outFile.open(filename);
	PrintHybridIndexFile (outFile);
	outFile.close();
	delete [] filename; filename=0;
	}//end AllozymeClineShapeData::PrintHybridIndexFile (aString&)
	
	
	
void AllozymeClineShapeData::PrintHybridIndexFile (ostream& outFile){
	outFile<<"location index alleles weight"<<endl;
	for(long i=0; i<hybridIndex.len(); ++i){
		HybridIndexByLocation* hi = hybridIndex[i];
		outFile<<hi->x()<<" ";
		outFile<<hi->p()<<" ";
		outFile<<hi->items()<<" ";
		outFile<<hi->weight()<<endl;
		}//end for i
	}//end AllozymeClineShapeData::PrintHybridIndexFile (ostream&)

#endif
