// AllozymeClineShapeData.h

//This ABSTRACT base class calculates cline shape by fitting data in the form (x,y) to
//a tanh function.  The theory is in Szymura & Barton 1986.
//The program reads in a data set of allozymes and calculates the hybrid
//index and gametic disequilibria for each population.

//derived classes implement the appropriate cline shape functions
#ifndef AllozymeClineShapeDatah
#define AllozymeClineShapeDatah = 1

#include <aList.h>
#include <StatList.h>
#include <Randomness.h>
#include <StringClass.h>
#include <HarmonicMeanDiseq.h>
#include <HybridIndexData.h>
#include <LikelihoodList.h>
#include <ClinePopList.h>
#include <ReadAndManageGeljock.h>





class AllozymeClineShapeData: public PopulationGeneticData{
		public:
			ObjList<StringList> rawDiagnosticAllelesR, diagnosticAllelesR;
					//list of loci, each with list of all diagnostic alleles for right side of HZ
					
			HybridIndexByLocData  hybridIndex;
			ObjList<HybridIndexByLocData>  hybridIndexByLocus;
			ClinePopList popLocation;
			aList<long> acceptedChanges, totalTries;
			double harMeanRecomb;
			double LcurrentCline, LmaxCline;
			long allTries;


			
		public:
			AllozymeClineShapeData(void): 
					PopulationGeneticData(),
					harMeanRecomb(0.0),
					allTries(0) {}
			AllozymeClineShapeData(double harmonicMeanRecomb): 
					PopulationGeneticData(),
					harMeanRecomb(harmonicMeanRecomb),
					allTries(0) {}
			AllozymeClineShapeData(double numChromosomes, double mapDistBetweenLoci): 
					PopulationGeneticData(),
					harMeanRecomb(HarmonicDiseqMarkerVsSelLoci(numChromosomes, mapDistBetweenLoci)),
					allTries(0) {}
			AllozymeClineShapeData(aString& s, double harmonicMeanRecomb):
					PopulationGeneticData(s),
					harMeanRecomb(harmonicMeanRecomb),
					allTries(0) {}
			AllozymeClineShapeData(aString& s,double numChromosomes, double mapDistBetweenLoci):
					PopulationGeneticData(s),
					harMeanRecomb(HarmonicDiseqMarkerVsSelLoci(numChromosomes, mapDistBetweenLoci)),
					allTries(0) {}
			~AllozymeClineShapeData(void){
				rawDiagnosticAllelesR.Clear();
				diagnosticAllelesR.Clear();
				popLocation.Clear();
				hybridIndex.Clear();
				hybridIndexByLocus.Clear();
//				LcurrentList.Clear();
//				LcurrentDiseqList.Clear();
//				LcurrentTotalList.Clear();
				acceptedChanges.Clear();
				totalTries.Clear();
				}
			
	virtual void 		CalculateHybridIndex(void);
	virtual void 		CalculateHybridIndexByLocus(void);
	virtual void		IncrementSuccessfulTries (const int paramChanged);
	virtual void		IncrementTotalTries (const int paramChanged);
	virtual void		InitializeCounters (const int params);
	virtual int		PoolDiagnosticAlleles (void);//returns 0 if unsuccessful
	virtual void		PrintHybridIndexFile(aString& outFileName);//file assumed closed
	virtual void		PrintHybridIndexFile(ostream& outFile);// stream assumed open

		};//end class AllozymeClineShapeData


#endif
