// AllozymeClineShapeData2.cp;
//This program calculates cline shape by fitting data in the form (x,y) to
//a composite sigmoid function.  The theory is in Szymura & Barton 1986.
//The program reads in a data set of allozymes and calculates the hybrid
//index and gametic disequilibria for each population.
#pragma once
#include "AllozymeClineShapeData2.h"



void AllozymeClineShapeData2::AnnealClineShape(const long repsBetweenAnnealSteps, 
						const double startTemp, const double endTemp, 
						Ran3& ran){
	int paramChanged = 0, unsuccessfulTries = 0;
	int savingNow = False,  allDone = False, annealing = True, switchParams; //boolean flags
	long rep=0;
	double temperatureNow = startTemp;//anneal from a wide variance
	do{//here is the annealing loop
		if(rep > repsBetweenAnnealSteps){//anneal the temperature, which narrows the sampling variance
			temperatureNow /= 10.0;
			rep = 0;
			paramsInUse = bestParams;
			}//end if rep
		DoOneShapeIteration(temperatureNow, annealing, ran, rep, switchParams);
		if (! switchParams){	//catches occasional dead-end traps when 
			//an unlikely parameter combination is accepted
				++unsuccessfulTries;
				if(unsuccessfulTries>=25){
					//this is a particularly bad parameter combination
					unsuccessfulTries=0;
					paramsInUse = bestParams;}
				}
			else{unsuccessfulTries=0;}
		if (temperatureNow < endTemp){
				allDone = True;}
		}while (! allDone);//ends the annealing loop
	}//AllozymeClineShapeData2::AnnealClineShape





int AllozymeClineShapeData2::DoOneShapeIteration(const double temperature, 
									const int annealing, Ran3& ran, long& rep, int& switchParams){
	int allDone = False, paramChanged = 0;
	int centerExists = False;
	ClinePopList *centerPopList = new ClinePopList();
	do{//only accept if a population falls in the central region
		delete paramsToTry; paramsToTry=0;
		paramsToTry = paramsInUse->ChangeParams(*paramDeviations, ran, 
										paramChanged, temperature,annealing);
		centerPopList->Clear();
		centerExists = paramsToTry->FindCenterPopulations(*centerPopList, popLocation);
		}while(! centerExists);
	double LnewCline = MaxLikeDevFromCline(*paramsToTry, maxScore);
	paramsToTry->SetClineLikelihoodRatios(LmaxCline);
	++allTries;
		totalTries.Increment(paramChanged-1);
	switchParams = False;//if the fit is better, then save the new parameters

	if (paramsToTry->LtoLmaxCline() < 1.0){
			//if it's worse, then only save them with a probability proportional to their L ratio
			if (ran() < paramsToTry->LtoLmaxCline()) {
					switchParams = True;}}
												
		else{//new is better, so always switch
			switchParams = True;
			if (LnewCline > LmaxCline){//save new bests in appropriate places
				LmaxCline = LnewCline;
				paramsToTry->SetNewBest();//adjusts Lratios
				*bestParams = *paramsToTry;
				//replace initial values for diseq and total parameter estimates
				if (annealing){
					LcurrentList.Clear();}//abolish all old information
				bestParams->PrintToScreen(allTries);
				}//end if LnewCline > LmaxCline
		}//end if LtoLmaxCline... else...
	if(switchParams){
		acceptedChanges.Increment(paramChanged-1);
		*paramsInUse = *paramsToTry;
		LcurrentCline = LnewCline;
		allDone = LcurrentList.Append(paramsToTry, centerPopList);
		}//end if switchParams
	centerPopList->Clear();
	delete centerPopList; centerPopList=0;
	delete paramsToTry; paramsToTry=0;
	++rep;
	return allDone;
	}//end AllozymeClineShapeData2::DoOneShapeIteration
	
	
void AllozymeClineShapeData2::Find2unitParameterList(Ran3& ran){
	cout<<endl<<"Beginning cline shape error estimation"<<endl<<endl;
	double temperatureNow = temperature * 2.0; 
	int allDone = False;
	LcurrentList.SetSave(True);
//	LcurrentDiseqList.SetSave(True);
//	LcurrentTotalList.SetSave(True);

	LcurrentCline = bestParams->Lcline();
	*paramsInUse = *bestParams;
	int unsuccessfulTries = 0;
	InitializeCounters(params);

	long rep=0;
	do{//random walk sampling-error iteration loop
//		changeDiseq =  False;	//change cline shape parameters first
		int switchParams = False, annealing = False;
		allDone = DoOneShapeIteration(temperatureNow, annealing, ran, rep, switchParams);
			
		if (! switchParams)	{//catches occasional dead-end traps when 
				//an unlikely parameter combination is accepted
				++unsuccessfulTries;
				if(unsuccessfulTries>=25){
					//this is a particularly bad parameter combination
					unsuccessfulTries=0;
					paramsInUse = bestParams;}
				}
			else{//new shape parameters chosen
				unsuccessfulTries = 0;
				}//end if ! switchParams...else...
		++rep;
		allDone = True;
		long lenSum = LcurrentList.totalSize;//compare SavedReps to the total of all pop combination lists}
		if (lenSum < savedReps){
			allDone = False;}
		}while (! allDone);//end error-sampling iteration
	cout<<endl<<"Done with cline shape error estimation"<<endl<<endl;
	}//end AllozymeClineShapeData2::Find2unitParameterList



void AllozymeClineShapeData2::FitAverageAllozymeCline(String& dataFileName, Ran3& ran){
	PrepareDataForClineAnalysis(dataFileName);
	InitializeCounters(params);

	paramsInUse->SetStartingValues();
	paramDeviations->SetDeviationValues();

	//calculate a maximum likelihood fit function
	ClinePopList centerPopList;
	paramsInUse->FindCenterPopulations(centerPopList, popLocation);
	LcurrentCline = MaxLikeDevFromCline(*paramsInUse, maxScore);
	LcurrentList.Append(*paramsInUse, centerPopList);
	centerPopList.Clear();


	LmaxCline = LcurrentCline;
	SetCurrentAsBest();
	bestParams->PrintToScreen(allTries);




	//ANNEAL TO FIND THE MAXIMUM LIKELIHOOD ESTIMATE OF CLINE SHAPE

	AnnealClineShape(annealAfterRep,temperature * 100.0, lowTemp, ran);



	//NOW DO THE RANDOM-WALK ERROR ESTIMATION 
	Find2unitParameterList(ran);
	LcurrentList.Find2UnitSupportLimits (harMeanRecomb);

/*
	PrintBestParameters(acceptedChanges, totalTries, LmaxCline, bestParams, lowerCIset, upperCIset, bestParamsDiseq, bestParamsTotal, popCombListDiseq, popCombListTotal, Output, False);
	WriteLn(Output, 'chromosomes: ', numChromosomes : 6 : 3, '   harmonic mean recombination rate: ', harMeanRecomb : 8 : 5);
	popListFileName := Concat(outputFileName1, '.sum');
	open(outputData2, popListFileName);
	PrintBestParameters(acceptedChanges, totalTries, LmaxCline, bestParams, lowerCIset, upperCIset, bestParamsDiseq, bestParamsTotal, popCombListDiseq, popCombListTotal, outputData2, False);
	WriteLn(outputData2, 'chromosomes: ', numChromosomes : 6 : 3, '   harmonic mean recombination rate: ', harMeanRecomb : 8 : 5);
	close(outputData2);
	PrintSavedParameters(bestParams, savedParamList, outputData, outputFileName1);
	if alloData^^.nloci > 1 then	{provided there are enough loci}
		begin
			popListFileName := Concat(outputFileName2, '.D pops');
			open(outputData2, popListFileName);
			writeLn(outputData2, 'file, population set');
			for j := 1 to popCombListDiseq.len do
				begin
					outName := Concat('.', IntegerToString(j));
					outName := Concat(outputFileName2, outName);
					PrintSavedParameters(bestParamsDiseq^[j], diseqSavedParamList^[j], outputData, outName);
					centerPopList := ListStructureTypePtr(GetSubListAt(popCombListDiseq, j));
					writeLn(Output, 'In data file ', outName, ':');
					PrintPopComblist(centerPopList^, Output);
					writeLn(outputData2, outName, ':');
					PrintPopComblist(centerPopList^, outputData2);
				end;	{end for j}
			close(outputData2);
		end;	{end if nloci}
	if alloData^^.nloci > 1 then	{provided there are enough loci}
		begin
			popListFileName := Concat(outputFileName3, '.T pops');
			open(outputData2, popListFileName);
			writeLn(outputData2, 'file, population set');
			for j := 1 to popCombListTotal.len do
				begin
					outName := Concat('.', IntegerToString(j));
					outName := Concat(outputFileName3, outName);
					PrintSavedParameters(bestParamsTotal^[j], totalSavedParamList^[j], outputData, outName);
					centerPopList := ListStructureTypePtr(GetSubListAt(popCombListTotal, j));
					writeLn(Output, 'In data file ', outName, ':');
					PrintPopComblist(centerPopList^, Output);
					writeLn(outputData2, outName, ',');
					PrintPopComblist(centerPopList^, outputData2);
					centerPopList := nil;	{forget this value}
				end;	{end for j}
			close(outputData2);
		end; {end if nloci>1}




{check for differential introgression by locus}
	if introgressionBreakdownByLocus then
		if alloData^^.nloci > 1 then
			begin
				New(dummyPopList);
				InitializeListOfLists(dummyPopList^, primaryList);
				New(tempParamList);
				New(tempBestParams);
				for loc := 1 to alloData^^.nloci do
					begin
						RescaleLtoLmaxRatios(bestParamsLoc^[loc], savedParamListLocDev^[loc], 1);	{uses Lcline as criterion}
						tempParamList^[1] := savedParamListLocDev^[loc];
						tempBestParams^[1] := bestParamsLoc^[loc];
						Find2UnitSupportLimits(bestParamsLoc^[loc], lowerCIset, upperCIset, tempBestParams, tempBestParams, 1, 1, savedParamListLocDev^[loc], tempParamList, tempParamList);
						PrintBestParameters(acceptedChanges, totalTries, bestParamsLoc^[loc].LCline, bestParamsLoc^[loc], lowerCIset, upperCIset, tempBestParams, tempBestParams, dummyPopList^, dummyPopList^, Output, True);
						WriteLn(Output, 'chromosomes: ', numChromosomes : 6 : 3, '   harmonic mean recombination rate: ', harMeanRecomb : 8 : 5);
						popListFileName := Concat(outputFileName1, '.');
						popListFileName := Concat(outputFileName1, GetStrValueAt(alloData^^.locusName^, loc));
						popListFileName := Concat(popListFileName, '.sum');
						open(outputData2, popListFileName);
						PrintBestParameters(acceptedChanges, totalTries, bestParamsLoc^[loc].LCline, bestParamsLoc^[loc], lowerCIset, upperCIset, tempBestParams, tempBestParams, dummyPopList^, dummyPopList^, outputData2, True);
						WriteLn(outputData2, 'chromosomes: ', numChromosomes : 6 : 3, '   harmonic mean recombination rate: ', harMeanRecomb : 8 : 5);
						close(outputData2);
						popListFileName := Concat(outputFileName1, GetStrValueAt(alloData^^.locusName^, loc));
						PrintSavedParameters(bestParamsLoc^[loc], savedParamListLocDev^[loc], outputData2, popListFileName);
					end;	{end for loc}
				Dispose(dummyPopList);
				Dispose(tempParamList);
				Dispose(tempBestParams);

{for each locus's data, calculate the likelihood of the average and other loci's best fits}
				popListFileName := Concat(outputFileName1, '.L1overL2');
				open(outputData2, popListFileName);
				write(Output, 'loc,  ave,');
				write(outputData2, 'loc,  ave,');
				for loc1 := 1 to alloData^^.nloci do
					begin
						write(Output, Spaces(3), ', ', loc1 : 2);
						write(outputData2, Spaces(3), ', ', loc1 : 2);
					end;
				writeLn(Output);
				writeLn(outputData2);
				for loc1 := 1 to alloData^^.nloci do
					begin
						write(Output, loc1 : 2, Spaces(3), ', ');
						write(outputData2, loc1 : 2, Spaces(3), ', ');
						theRatio := exp(bestParamsLoc^[loc1].Lcline - bestParams.Lcline);
						write(Output, theRatio : 15 : 10);
						write(outputData2, theRatio : 15 : 10);
						for loc2 := 1 to alloData^^.nloci do
							begin
								write(Output, Spaces(3), ', ');
								write(outputData2, Spaces(3), ', ');
								theRatio := exp(bestParamsLoc^[loc1].Lcline - bestParamsLoc^[loc2].Lcline);
								write(Output, theRatio : 15 : 10);
								write(outputData2, theRatio : 15 : 10);
							end;	{end for loc2}
						writeLn(Output);
						writeLn(outputData2);
					end;	{end for loc1}
				close(outputData2);


			end;	{end if introgressionBreakdownByLocus & if nloci>1]}

{dispose everything}
	DisposeHybridIndexListsWithPtrs;
	DisposeSavedParamSet(savedParamList);
	for i := 1 to PopCombinations do
		DisposeSavedParamSet(totalSavedParamList^[i]);
	Dispose(totalSavedParamList);
	for i := 1 to PopCombinations do
		DisposeSavedParamSet(diseqSavedParamList^[i]);
	Dispose(diseqSavedParamList);
	while popCombListTotal.len > 0 do
		DisposeFirstSublist(popCombListTotal, primaryList);
	while popCombListDiseq.len > 0 do
		DisposeFirstSublist(popCombListDiseq, primaryList);
	Dispose(bestParamsLoc);
	for loc := 1 to maxLoci do
		DisposeSavedParamSet(savedParamListLocDev^[loc]);
	Dispose(savedParamListLocDev);
	DisposeAlloListsWithPointers;
	DisposeList(popLocation);
	Dispose(bestParamsDiseq);
	Dispose(bestParamsTotal);
*/
}//end AllozymeClineShapeData2::FitAverageAllozymeCline



double AllozymeClineShapeData2::MaxLikeDevFromCline (Shape2ParamRec& paramSet, double highScore){
	double runningTotal = 0.0;
	for (int loc=0;loc<nloci;++loc){
			runningTotal += MaxLikeDevFromCline(paramSet, loc, highScore);}
	paramSet.Lcline_ = runningTotal;
	if(runningTotal == 0.0){
			int dummyToForceDebugStop = 1;}
	return runningTotal;
	}//end AllozymeClineShapeData2::MaxLikeDevFromCline  (Shape2ParamRec&, double)



double AllozymeClineShapeData2::MaxLikeDevFromCline (Shape2ParamRec& paramSet, int loc, double highScore){
	//sample-size weights removed from this analysis
	// obs, expected, logL, temp, score: extended;
	double logL = 0.0;
	HybridIndexByLocData* hiData = hybridIndexByLocus[loc];
	for (long i=0;i<hiData->len();++i){
		double obs = hiData->p(i);
		double expected = paramSet.PonCline( hiData->x(i), highScore);
		if ((obs > 0.0) && (expected > 0.0)) {
			//this effectively sets obs*ln(E)=0 when obs=0 or exp=0
			logL +=  obs * ln(expected);}// * hiData->weight(i);
		if ((obs < 1.0) && (expected < 1.0)) {
			logL +=  (1.0 - obs) * ln(1.0 - expected);}// * hiData->weight(i);
		}//end for i
	paramSet.Lcline_ = logL;
	return logL;
	}//end AllozymeClineShapeData2::MaxLikeDevFromCline (Shape2ParamRec&, int, double)



void AllozymeClineShapeData2::PrepareDataForClineAnalysis(String& dataFileName){
	PoolDiagnosticAlleles();

	String convertFileName(dataFileName);
	convertFileName.cat(".pooled");
	PrintGelJockFile(convertFileName);

	CalculateAlleleFreqs();

	CalculateHybridIndex();//popLocation created
	CalculateHybridIndexByLocus();//popLocation re-created
	convertFileName = dataFileName;
	convertFileName.cat(".hyb index data");
	PrintHybridIndexFile(convertFileName);
}//end AllozymeClineShapeData2::PrepareDataForClineAnalysis

