// AllozymeClineShapeData3.h

//This class calculates cline shape by fitting data in the form (x,y) to
//a 2-parameter tanh function.  The theory is in Szymura & Barton 1986.
//The program reads in a data set of allozymes and calculates the hybrid
//index and gametic disequilibria for each population.

//derived classes implement the appropriate cline shape functions
#ifndef AllozymeClineShapeData3h
#define AllozymeClineShapeData3h = 1

//#include <iostream.h>
#include <Shape3ParamRec.h>
//#include <ClinePopList.h>
//#include <LikelihoodList.h>
//#include <LikelihoodListDiseq.h>
//#include <LikelihoodListTotal.h>
//#include <AllozymeClineShapeData.h>
#include <AllozymeClineShapePlus.h>


class AllozymeClineShapeData3: public AllozymeClineShapePlus<Shape3ParamRec>{
		public:
			AllozymeClineShapeData3(double harmonicMeanRecomb):
				AllozymeClineShapePlus<Shape3ParamRec>(harmonicMeanRecomb){
						harMeanRecomb = harmonicMeanRecomb;
						params = 3;}
			AllozymeClineShapeData3(aString& s, double harmonicMeanRecomb): 
				AllozymeClineShapePlus<Shape3ParamRec>(s, harmonicMeanRecomb){
						harMeanRecomb = harmonicMeanRecomb;
						params = 3;}
			AllozymeClineShapeData3(double numChromosomes, double mapDistBetweenLoci):
				AllozymeClineShapePlus<Shape3ParamRec>(numChromosomes, mapDistBetweenLoci){
						params = 3;}
			AllozymeClineShapeData3(aString& s, double numChromosomes, double mapDistBetweenLoci): 
				AllozymeClineShapePlus<Shape3ParamRec>(s, numChromosomes, mapDistBetweenLoci){
						params = 3;}
			~AllozymeClineShapeData3(void){}
			
		};//end class AllozymeClineShapeData3

/*

class AllozymeClineShapeData3: virtual public AllozymeClineShapeData{
		public:
			int params;
			Shape3ParamRec *bestParams, *paramsInUse, *paramDeviations, *paramsToTry;			
			LikelihoodList<Shape3ParamRec> LcurrentList;
			LikelihoodListDiseq<Shape3ParamRec> LcurrentDiseqList;
			LikelihoodListTotal<Shape3ParamRec> LcurrentTotalList;
			double LcurrentDiseq, LmaxDiseq;
			double LcurrentTotal, LmaxTotal;


			
		public:
			AllozymeClineShapeData3(void): AllozymeClineShapeData(),
						LcurrentList(cline), LcurrentDiseqList(disequilibrium), LcurrentTotalList(total), 
						params(3), bestParams(0), paramsInUse(0), paramDeviations(0), paramsToTry(0) {
							IShapeData();}
			AllozymeClineShapeData3(String& s): AllozymeClineShapeData(s),
						LcurrentList(cline), LcurrentDiseqList(disequilibrium), LcurrentTotalList(total), 
						params(3), bestParams(0), paramsInUse(0), paramDeviations(0), paramsToTry(0) {
							IShapeData();}
			~AllozymeClineShapeData3(void){
				delete bestParams; bestParams=0;
				delete paramsInUse; paramsInUse=0;
				delete paramDeviations; paramDeviations=0;
				delete paramsToTry; paramsToTry=0;
				}
			
	//VIRTUAL FUNCTIONS
	virtual void 		AnnealClineShape(const long repsBetweenAnnealSteps, 
						const double startTemp, const double endTemp, 
						Ran3& ran);
	virtual double 	DiseqLikelihood (Shape3ParamRec& paramSet, ClinePopList& centerPopList);
	virtual int 		DoOneShapeIteration(const double temperature,  
							const int annealing, Ran3& ran, long& rep, int& switchParams);
	virtual void 		Find2unitParameterList(Ran3& ran);//should first call AnnealClineShape
	virtual void		FitAverageAllozymeCline(String& dataFileName, Ran3& ran);//averaged across loci
	virtual void 		IShapeData(void ){
						delete bestParams; bestParams= new Shape3ParamRec();
						delete paramsInUse; paramsInUse= new Shape3ParamRec();
						delete paramDeviations; paramDeviations= new Shape3ParamRec();
						delete paramsToTry; paramsToTry= new Shape3ParamRec();
						}//end IShapeData		
	virtual double	MaxLikeDevFromCline (Shape3ParamRec& paramSet, double highScore);
	virtual double 	MaxLikeDevFromCline (Shape3ParamRec& paramSet, int loc, double highScore);
	virtual void 		PrepareDataForClineAnalysis(String& dataFileName);
	virtual void		SetCurrentAsBest(void){
						delete bestParams; bestParams = 0;
						bestParams = new Shape3ParamRec(*paramsInUse);
						}//end SetCurrentAsBest
		};//end class AllozymeClineShapeData3
		
*/

#endif
