// AllozymeClineShapePlus<T>.cp;
//This program calculates cline shape by fitting data in the form (x,y) to
//a composite sigmoid function.  The theory is in Szymura & Barton 1986.
//The program reads in a data set of allozymes and calculates the hybrid
//index and gametic disequilibria for each population.
#pragma once
#include <AllozymeClineShapePlus.h>



template<class T> void AllozymeClineShapePlus<T>::AnnealClineShape(const long repsBetweenAnnealSteps, 
						const double startTemp, const double endTemp, 
						Ran3& ran, long& savedReps){
	int paramChanged = 0, unsuccessfulTries = 0;
	int savingNow = False,  allDone = False, annealing = True, switchParams; //boolean flags
	long rep=0;
	double temperatureNow = startTemp;//anneal from a wide variance
	do{//here is the annealing loop
		if(rep > repsBetweenAnnealSteps){//anneal the temperature, which narrows the sampling variance
			temperatureNow /= 10.0;
			rep = 0;
			*paramsInUse = *bestParams;
			}//end if rep
		DoOneShapeIteration(temperatureNow, annealing, ran, rep, switchParams, savedReps);
		if (! switchParams){	//catches occasional dead-end traps when 
			//an unlikely parameter combination is accepted
				++unsuccessfulTries;
				if(unsuccessfulTries>=25){
					//this is a particularly bad parameter combination
					unsuccessfulTries=0;
					paramsInUse = bestParams;}
				}
			else{unsuccessfulTries=0;}
		if (temperatureNow < endTemp){
				allDone = True;}
		}while (! allDone);//ends the annealing loop
	}//AllozymeClineShapePlus<T>::AnnealClineShape




template<class T> double AllozymeClineShapePlus<T>::DiseqLikelihood (T& paramSet, ClinePopList& centerPopList){
	//sample-size based weighting removed
	long N[5][5];//Hill's intermediate variable counting individuals by 2-locus genotype, modified to permit blanks
	long X[2][2];//Hill's intermediate variable counting individuals by 2-locus genotype, modified to permit blanks
	int index1, index2;
	double  f[2][2];
	double logL = 0.0, adjustedD = 0.0;
	aString blank(" ");
	for (int popNum=0;popNum<centerPopList.len();++popNum){
		int pop = centerPopList[popNum];
		long sizePop = popSize[pop];
		double thisPopLocation = popLocation[pop];
		double pAtCline = paramSet.PonCline(thisPopLocation, maxScore);
		if ((pAtCline == 0.0) || (pAtCline == 1.0)) {
				adjustedD = 1.0;} //was = paramSet.D
			else{
				adjustedD = paramSet.D() / (pAtCline * (1.0 - pAtCline));}
		if (adjustedD > 1.0){
			adjustedD = 1.0;}
		for (int loc1=0;loc1<nloci - 1;++loc1){
			StringList* alleleNamesList1 = allelesPresent[loc1];
			for (int loc2=loc1+1;loc2<nloci;++loc2){
				StringList* alleleNamesList2 = allelesPresent[loc2];
				for (int allele1=0;allele1<alleleNamesList1->len();++allele1){
					//get freq of allele1 (at loc1) in pop
					double freq1 = GetFreq(pop, loc1, allele1);
					if ((freq1 > 0.0) && (freq1 < 1.0)) {
						aString* alleleName1 = alleleNamesList1->Nth(allele1);
						for (int allele2=0;allele2<alleleNamesList2->len();++allele2){
							long nSample = 0;//set sample size to zero, to count for blanks
							//get freq of allele2 (at loc2) in pop
							double freq2 = GetFreq( pop, loc2, allele2);
							if ((freq2 > 0.0) && (freq2 < 1.0)) {
								f[0][0] = freq1 * freq2 + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
								f[0][1] = freq1 * (1.0 - freq2) + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
								f[1][0] = (1.0 - freq1) * freq2 + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
								f[1][1] = (1.0 - freq1) * (1.0 - freq2) + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
								//Sometimes these values can be less than zero due to sampling error,
								//implying that it is infinitely improbable that an observed value
								//could occur under the model.
								//Therefore, values <=0 are set to a very small number, and will
								//give a very low finite likelihood
								for(index1=0;index1<2;++index1){
									for (index2=0;index2<2;++index2){
										if (f[index1][index2] <= 0.0){
											f[index1][index2] = 1.0e-50;}}}

								for(index1=0;index1<5;++index1){
									for (index2=0;index2<5;++index2){
										N[index1][index2] = 0;}}
								for(index1=0;index1<2;++index1){
									for (index2=0;index2<2;++index2){
										X[index1][index2] = 0;}}
								//get genotype strings for each individual and  read joint frequencies
								aString* alleleName2 = alleleNamesList1->Nth(allele2);
								for (long i=0;i<sizePop;++i){//count up genotypes & store in matrix N
									aString* spot1 = GetGenotypeString(pop,i,loc1);//spot1 created in function call
									//make sure it has been scored
									if (((*spot1)[0] == blank) && ((*spot1)[1] == blank)) {//not scored
										delete spot1; spot1=0;
										break;}//go to next individual
									aString* spot2 = GetGenotypeString(pop,i,loc2);//spot2 created in function call
									if (((*spot2)[0] == blank) && ((*spot2)[1] == blank)) {//not scored
										delete spot1; spot1=0;
										delete spot2; spot2=0;
										break;}//go to next individual
									++nSample;//use this indiv in calculation

									//count instances of target alleles at each locus in this indiv
									if (((*spot1)[0] == blank) || ((*spot1)[1] == blank)) {//haploid individual or sex-linked locus
											index1 = 4;}
										else{index1=2;}//diploid individual
									for (int chrom=0;chrom<2;++chrom){
										if (*alleleName1==(*spot1)[chrom]){
											--index1;}}
									if (((*spot2)[0] == blank) || ((*spot2)[1] == blank)) {//haploid individual or sex-linked locus
											index2 = 4;}
										else{index2=2;}//diploid individual
									for (chrom=0;chrom<2;++chrom){
										if (*alleleName2==(*spot2)[chrom]){
											--index2;}}
									++N[index1][index2];
									delete spot1; spot1=0;
									delete spot2; spot2=0;
									}//end for i
								//calculate Hill's (1974) intermediate variables, modified for haploid & sex-linked cases
								X[0][0] = 2 * N[0][0] + N[0][1] + N[1][0] + N[0][3] + N[3][0] + N[3][3];
								X[1][0] = 2 * N[2][0] + N[1][0] + N[2][1] + N[2][3] + N[4][0] + N[4][3];
								X[0][1] = 2 * N[0][2] + N[0][1] + N[1][2] + N[0][4] + N[3][2] + N[3][4];
								X[1][1] = 2 * N[2][2] + N[1][2] + N[2][1] + N[2][4] + N[4][2] + N[4][4];
								for(index1=0;index1<2;++index1){
									for (index2=0;index2<2;++index2){
										if (f[index1][index2] > 0.0){
											logL += X[index1][index2] * ln(f[index1][index2]);
											logL += N[1][1] * ln(f[0][0] * f[1][1] + f[0][1] * f[1][0]);
											}}}//end if & for both indexes
								}//end if freq2>0
							}//end for allele2
						}//end if freq1>0
					}//end for allele1
				}//end for loc2
			}//end for loc1
		}//end for popNum
	//finally, standardize by the proportion of populations within the center region, 
	//as this will vary with the model parameters.  This will decrease logL.
	//if (centerPopList.len() > 0){
	//logL /= centerPopList.len() / npop;}
	paramSet.Ldiseq_ = logL;
	return logL;
	}//end AllozymeClineShapePlus<T>::DiseqLikelihood

	



template<class T> int AllozymeClineShapePlus<T>::DoOneShapeIteration(const double temperature, 
						const int annealing, Ran3& ran, long& rep, int& switchParams, long& savedReps){
	int allDone = False, paramChanged = 0;
	int centerExists = False;
	ClinePopList *centerPopList = new ClinePopList();
	do{//only accept if a population falls in the central region
		delete paramsToTry; paramsToTry=0;
		paramsToTry = paramsInUse->ChangeParams(*paramDeviations, ran, 
										paramChanged, temperature,True);
		centerPopList->Clear();
		centerExists = paramsToTry->FindCenterPopulations(*centerPopList, popLocation);
		IncrementTotalTries(paramChanged-1);
		}while(! centerExists);
	double LnewDiseq = DiseqLikelihood(*paramsToTry, centerPopList);
	double LnewCline = MaxLikeDevFromCline(*paramsToTry, maxScore);
	paramsToTry->SetTotal();
	paramsToTry->SetClineLikelihoodRatios(LmaxCline);
	++allTries;
//	totalTries.Increment(paramChanged-1);
	switchParams = False;//if the fit is better, then save the new parameters

	if (paramsToTry->LtoLmaxCline() < 1.0){
			//if it's worse, then only save them with a probability proportional to their L ratio
			if (ran() < paramsToTry->LtoLmaxCline()) {
					switchParams = True;}}
		else{//new is better, so always switch
			switchParams = True;
			}//end if LtoLmaxCline... else...
	if(switchParams){
		IncrementSuccessfulTries(paramChanged-1);
		if (LnewCline > LmaxCline){//save new bests in appropriate places
			LmaxCline = LnewCline;
			paramsToTry->SetNewBest();//adjusts Lratios
			*bestParams = *paramsToTry;
			//replace initial values for diseq and total parameter estimates
			if (annealing){
				LcurrentList.Clear();
				LcurrentDiseqList.Clear();
				LcurrentTotalList.Clear();
				}//abolish all old information
			bestParams->PrintToScreen(allTries, harMeanRecomb);
			}//end if LnewCline > LmaxCline
		*paramsInUse = *paramsToTry;
		LcurrentCline = LnewCline;
		allDone = LcurrentList.Append(paramsToTry, centerPopList, savedReps);
		}//end if switchParams
	centerPopList->Clear();
	delete centerPopList; centerPopList=0;
	delete paramsToTry; paramsToTry=0;
	++rep;
	return allDone;
	}//end AllozymeClineShapePlus<T>::DoOneShapeIteration
	
	
	
	
template<class T> void AllozymeClineShapePlus<T>::Find2unitParameterList(Ran3& ran, long& savedReps, long& annealAfterRep){
	cout<<endl<<"Beginning cline shape error estimation"<<endl<<endl;
	double temperatureNow = temperature * 2.0; 
	int allDone = False;
	LcurrentList.SetSave(True);
	LcurrentDiseqList.SetSave(True);
	LcurrentTotalList.SetSave(True);

	LcurrentCline = bestParams->Lcline();
	*paramsInUse = *bestParams;
	int unsuccessfulTries = 0;
	InitializeCounters(params);

	long rep=0;
	do{//random walk sampling-error iteration loop
		//change cline shape parameters first
		int switchParams = False, annealing = False;
		allDone = DoOneShapeIteration(temperatureNow, annealing, ran, rep, switchParams, savedReps);
			
		if (! switchParams)	{//catches occasional dead-end traps when 
				//an unlikely parameter combination is accepted
				++unsuccessfulTries;
				if(unsuccessfulTries>=25){
					//this is a particularly bad parameter combination
					unsuccessfulTries=0;
					paramsInUse = bestParams;}
				}
			else{//new shape parameters chosen
				unsuccessfulTries = 0;
				//test a new disequilibrium value
				allDone = VaryDiseq(temperatureNow, ran, rep, switchParams,savedReps, annealAfterRep);
				}//end if ! switchParams...else...
		++rep;
		//compare savedReps to the total of all pop combination lists
		allDone =  ((LcurrentList.totalSize >= savedReps) &&
		 		 (LcurrentDiseqList.totalSize >= savedReps) &&
				 (LcurrentTotalList.totalSize >= savedReps) );

		}while (! allDone);//end error-sampling iteration
	cout<<endl<<"Done with cline shape error estimation"<<endl<<endl;
	}//end AllozymeClineShapePlus<T>::Find2unitParameterList



template<class T> int AllozymeClineShapePlus<T>::FitAverageAllozymeCline(aString& dataFileName, 
											Ran3& ran, long& savedReps, long& annealAfterRep){
	if (! PrepareDataForClineAnalysis(dataFileName) ){
		return 0;}
	InitializeCounters(params);

	paramsInUse->SetStartingValues();
	paramDeviations->SetDeviationValues();

	//calculate a maximum likelihood fit function
	ClinePopList centerPopList;
	paramsInUse->FindCenterPopulations(centerPopList, popLocation);
	LcurrentDiseq = DiseqLikelihood(*paramsInUse, centerPopList);
	LcurrentCline = MaxLikeDevFromCline(*paramsInUse, maxScore);
	paramsInUse->SetTotal();
	paramsInUse->SetNewBest();
	paramsInUse->SetNewBestDiseq();
	paramsInUse->SetNewBestTotal();
	LcurrentTotal = LcurrentDiseq + LcurrentCline;
	LcurrentList.Append(*paramsInUse, centerPopList, savedReps);
	LcurrentDiseqList.Append(*paramsInUse, centerPopList, savedReps);
	LcurrentTotalList.Append(*paramsInUse, centerPopList, savedReps);
	centerPopList.Clear();


	LmaxCline = LcurrentCline;
	LmaxDiseq = LcurrentDiseq;
	LmaxTotal = LcurrentTotal;
	SetCurrentAsBest();
	bestParams->PrintToScreen(allTries, harMeanRecomb);




	//ANNEAL TO FIND THE MAXIMUM LIKELIHOOD ESTIMATE OF CLINE SHAPE

	AnnealClineShape(annealAfterRep,temperature * 100.0, lowTemp, ran, savedReps);



	//NOW DO THE RANDOM-WALK ERROR ESTIMATION 
	bestParams->FindCenterPopulations(centerPopList, popLocation);
	LcurrentDiseqList.Append(bestParams, centerPopList, savedReps);
	LcurrentTotalList.Append(bestParams, centerPopList, savedReps);
	centerPopList.Clear();

	Find2unitParameterList(ran, savedReps, annealAfterRep);
	LcurrentList.Find2UnitSupportLimits (harMeanRecomb);
	LcurrentDiseqList.Find2UnitSupportLimits (harMeanRecomb);
	LcurrentTotalList.Find2UnitSupportLimits (harMeanRecomb);

/*
	PrintBestParameters(acceptedChanges, totalTries, LmaxCline, bestParams, lowerCIset, upperCIset, bestParamsDiseq, bestParamsTotal, popCombListDiseq, popCombListTotal, Output, False);
	WriteLn(Output, 'chromosomes: ', numChromosomes : 6 : 3, '   harmonic mean recombination rate: ', harMeanRecomb : 8 : 5);
	popListFileName := Concat(outputFileName1, '.sum');
	open(outputData2, popListFileName);
	PrintBestParameters(acceptedChanges, totalTries, LmaxCline, bestParams, lowerCIset, upperCIset, bestParamsDiseq, bestParamsTotal, popCombListDiseq, popCombListTotal, outputData2, False);
	WriteLn(outputData2, 'chromosomes: ', numChromosomes : 6 : 3, '   harmonic mean recombination rate: ', harMeanRecomb : 8 : 5);
	close(outputData2);
	PrintSavedParameters(bestParams, savedParamList, outputData, outputFileName1);
	if alloData^^.nloci > 1 then	{provided there are enough loci}
		begin
			popListFileName := Concat(outputFileName2, '.D pops');
			open(outputData2, popListFileName);
			writeLn(outputData2, 'file, population set');
			for j := 1 to popCombListDiseq.len do
				begin
					outName := Concat('.', IntegerToString(j));
					outName := Concat(outputFileName2, outName);
					PrintSavedParameters(bestParamsDiseq^[j], diseqSavedParamList^[j], outputData, outName);
					centerPopList := ListStructureTypePtr(GetSubListAt(popCombListDiseq, j));
					writeLn(Output, 'In data file ', outName, ':');
					PrintPopComblist(centerPopList^, Output);
					writeLn(outputData2, outName, ':');
					PrintPopComblist(centerPopList^, outputData2);
				end;	{end for j}
			close(outputData2);
		end;	{end if nloci}
	if alloData^^.nloci > 1 then	{provided there are enough loci}
		begin
			popListFileName := Concat(outputFileName3, '.T pops');
			open(outputData2, popListFileName);
			writeLn(outputData2, 'file, population set');
			for j := 1 to popCombListTotal.len do
				begin
					outName := Concat('.', IntegerToString(j));
					outName := Concat(outputFileName3, outName);
					PrintSavedParameters(bestParamsTotal^[j], totalSavedParamList^[j], outputData, outName);
					centerPopList := ListStructureTypePtr(GetSubListAt(popCombListTotal, j));
					writeLn(Output, 'In data file ', outName, ':');
					PrintPopComblist(centerPopList^, Output);
					writeLn(outputData2, outName, ',');
					PrintPopComblist(centerPopList^, outputData2);
					centerPopList := nil;	{forget this value}
				end;	{end for j}
			close(outputData2);
		end; {end if nloci>1}




{check for differential introgression by locus}
	if introgressionBreakdownByLocus then
		if alloData^^.nloci > 1 then
			begin
				New(dummyPopList);
				InitializeListOfLists(dummyPopList^, primaryList);
				New(tempParamList);
				New(tempBestParams);
				for loc := 1 to alloData^^.nloci do
					begin
						RescaleLtoLmaxRatios(bestParamsLoc^[loc], savedParamListLocDev^[loc], 1);	{uses Lcline as criterion}
						tempParamList^[1] := savedParamListLocDev^[loc];
						tempBestParams^[1] := bestParamsLoc^[loc];
						Find2UnitSupportLimits(bestParamsLoc^[loc], lowerCIset, upperCIset, tempBestParams, tempBestParams, 1, 1, savedParamListLocDev^[loc], tempParamList, tempParamList);
						PrintBestParameters(acceptedChanges, totalTries, bestParamsLoc^[loc].LCline, bestParamsLoc^[loc], lowerCIset, upperCIset, tempBestParams, tempBestParams, dummyPopList^, dummyPopList^, Output, True);
						WriteLn(Output, 'chromosomes: ', numChromosomes : 6 : 3, '   harmonic mean recombination rate: ', harMeanRecomb : 8 : 5);
						popListFileName := Concat(outputFileName1, '.');
						popListFileName := Concat(outputFileName1, GetStrValueAt(alloData^^.locusName^, loc));
						popListFileName := Concat(popListFileName, '.sum');
						open(outputData2, popListFileName);
						PrintBestParameters(acceptedChanges, totalTries, bestParamsLoc^[loc].LCline, bestParamsLoc^[loc], lowerCIset, upperCIset, tempBestParams, tempBestParams, dummyPopList^, dummyPopList^, outputData2, True);
						WriteLn(outputData2, 'chromosomes: ', numChromosomes : 6 : 3, '   harmonic mean recombination rate: ', harMeanRecomb : 8 : 5);
						close(outputData2);
						popListFileName := Concat(outputFileName1, GetStrValueAt(alloData^^.locusName^, loc));
						PrintSavedParameters(bestParamsLoc^[loc], savedParamListLocDev^[loc], outputData2, popListFileName);
					end;	{end for loc}
				Dispose(dummyPopList);
				Dispose(tempParamList);
				Dispose(tempBestParams);

{for each locus's data, calculate the likelihood of the average and other loci's best fits}
				popListFileName := Concat(outputFileName1, '.L1overL2');
				open(outputData2, popListFileName);
				write(Output, 'loc,  ave,');
				write(outputData2, 'loc,  ave,');
				for loc1 := 1 to alloData^^.nloci do
					begin
						write(Output, Spaces(3), ', ', loc1 : 2);
						write(outputData2, Spaces(3), ', ', loc1 : 2);
					end;
				writeLn(Output);
				writeLn(outputData2);
				for loc1 := 1 to alloData^^.nloci do
					begin
						write(Output, loc1 : 2, Spaces(3), ', ');
						write(outputData2, loc1 : 2, Spaces(3), ', ');
						theRatio := exp(bestParamsLoc^[loc1].Lcline - bestParams.Lcline);
						write(Output, theRatio : 15 : 10);
						write(outputData2, theRatio : 15 : 10);
						for loc2 := 1 to alloData^^.nloci do
							begin
								write(Output, Spaces(3), ', ');
								write(outputData2, Spaces(3), ', ');
								theRatio := exp(bestParamsLoc^[loc1].Lcline - bestParamsLoc^[loc2].Lcline);
								write(Output, theRatio : 15 : 10);
								write(outputData2, theRatio : 15 : 10);
							end;	{end for loc2}
						writeLn(Output);
						writeLn(outputData2);
					end;	{end for loc1}
				close(outputData2);


			end;	{end if introgressionBreakdownByLocus & if nloci>1]}

{dispose everything}
	DisposeHybridIndexListsWithPtrs;
	DisposeSavedParamSet(savedParamList);
	for i := 1 to PopCombinations do
		DisposeSavedParamSet(totalSavedParamList^[i]);
	Dispose(totalSavedParamList);
	for i := 1 to PopCombinations do
		DisposeSavedParamSet(diseqSavedParamList^[i]);
	Dispose(diseqSavedParamList);
	while popCombListTotal.len > 0 do
		DisposeFirstSublist(popCombListTotal, primaryList);
	while popCombListDiseq.len > 0 do
		DisposeFirstSublist(popCombListDiseq, primaryList);
	Dispose(bestParamsLoc);
	for loc := 1 to maxLoci do
		DisposeSavedParamSet(savedParamListLocDev^[loc]);
	Dispose(savedParamListLocDev);
	DisposeAlloListsWithPointers;
	DisposeList(popLocation);
	Dispose(bestParamsDiseq);
	Dispose(bestParamsTotal);
*/
	return 1;
}//end AllozymeClineShapePlus<T>::FitAverageAllozymeCline



template<class T> double AllozymeClineShapePlus<T>::MaxLikeDevFromCline (T& paramSet, double highScore){
	double runningTotal = 0.0;
	for (int loc=0;loc<nloci;++loc){
			runningTotal += MaxLikeDevFromCline(paramSet, loc, highScore);}
	paramSet.Lcline_ = runningTotal;
	if(runningTotal == 0.0){
			int dummyToForceDebugStop = 1;}
	return runningTotal;
	}//end AllozymeClineShapePlus<T>::MaxLikeDevFromCline  (T&, double)



template<class T> double AllozymeClineShapePlus<T>::MaxLikeDevFromCline (T& paramSet, int loc, double highScore){
	//sample-size weights removed from this analysis
	// obs, expected, logL, temp, score: extended;
	double logL = 0.0;
	HybridIndexByLocData* hiData = hybridIndexByLocus[loc];
	for (long i=0;i<hiData->len();++i){
		double obs = hiData->p(i);
		double expected = paramSet.PonCline( hiData->x(i), highScore);
		if ((obs > 0.0) && (expected > 0.0)) {
			//this effectively sets obs*ln(E)=0 when obs=0 or exp=0
			logL +=  obs * ln(expected);}// * hiData->weight(i);
		if ((obs < 1.0) && (expected < 1.0)) {
			logL +=  (1.0 - obs) * ln(1.0 - expected);}// * hiData->weight(i);
		}//end for i
	paramSet.Lcline_ = logL;
	return logL;
	}//end AllozymeClineShapePlus<T>::MaxLikeDevFromCline (T&, int, double)



template<class T> int AllozymeClineShapePlus<T>::PrepareDataForClineAnalysis(aString& dataFileName){
	if( !  PoolDiagnosticAlleles()){
		return 0;}

	aString convertFileName(dataFileName);
	convertFileName.cat(".pooled");
	PrintGelJockFile(convertFileName);

	CalculateAlleleFreqs();

	CalculateHybridIndex();//popLocation created
	CalculateHybridIndexByLocus();//popLocation re-created
	convertFileName = dataFileName;
	convertFileName.cat(".hyb index data");
	PrintHybridIndexFile(convertFileName);
	return 1;
	}//end AllozymeClineShapePlus<T>::PrepareDataForClineAnalysis


template<class T> void AllozymeClineShapePlus<T>::PrintBestParameters (aString& fileName){
	fstream outFile;
	char* fileNameStr = fileName.raw();
	outFile.open(fileNameStr);
	PrintBestParameters (outFile);
	outFile.close();
	delete [] fileNameStr; fileNameStr=0;
	}//end AllozymeClineShapePlus<T>::PrintBestParameters(aString&)





template<class T> void AllozymeClineShapePlus<T>::PrintBestParameters (ostream& outFile){
	//a, n: paramTriesArrayType; LmaxCline: extended; bestparams, lowerCIset, upperCIset: paramRec; bestParamsDiseq, bestParamsTotal: bestParamRecListTypePtr; popCombListDiseq, popCombListTotal: ListOfListsType; var outputData: text; byLocus: boolean);
	LcurrentList.PrintBestParameters(allTries,acceptedChanges,totalTries,outFile);
	LcurrentDiseqList.PrintBestParameters(allTries,acceptedChanges,totalTries,outFile);
	LcurrentTotalList.PrintBestParameters(allTries,acceptedChanges,totalTries,outFile);





/*	aList<double> successRate;
	long sumA = 0, sumN = 0;
	for (long i=0;i<totalTries.len();++i){
		if (totalTries[i] > 0){ 
			successRate.Append( (double)acceptedChanges[i] / (double) totalTries[i] );
			sumA += acceptedChanges[i];
			sumN += totalTries[i];
			}//end for i
	aString LmaxClineStr(LmaxCline,8,12);
	LmaxClineStr.prepend("cline L max: ");
	outFile<<LmaxClineStr<<endl;

	aString totalReps("total replicates tried: "); totalReps.cat(sumN);
	outFile<<totalReps<<endl;
	
	double totalSuccessRate = 0.0;
	if (sumN> 0) {
		totalSuccessRate = (double)sumA / (double) sumN;}

	aString totalSucess(totalSuccessRate,1,6); totalSucess.prepend("overall acceptance rate: ");
	outFile<<totalSucess<<endl;

	aString cLine("center: ");cLine.cat(bestparams.center(),5,10);
	cLine.separator(3);cLine.cat('(');cLine.cat(LcurrentList.lowerCI.center(),5,10);cLine.cat(" - ");
	cLine.cat(LcurrentList.upperCI.center(),5,10);cLine.cat(')');cLine.cat(" rate: ");cLine.cat(successRate[0],1,6);
	outFile<<cLine<<endl;
	
	aString wLine("width: ");wLine.cat(bestparams.w(),5,10);
	wLine.separator(3);wLine.cat('(');wLine.cat(LcurrentList.lowerCI.w(),5,10);wLine.cat(" - ");
	wLine.cat(LcurrentList.upperCI.w(),5,10);wLine.cat(')');wLine.cat(" rate: ");wLine.cat(successRate[1],1,6);
	outFile<<wLine<<endl;

	aString slopeLine("max slope: ");slopeLine.cat(1.0/bestparams.w(),5,10);
	slopeLine.separator(3);slopeLine.cat('(');slopeLine.cat(1.0/LcurrentList.lowerCI.w(),5,10);slopeLine.cat(" - ");
	slopeLine.cat(1.0/LcurrentList.upperCI.w(),5,10);slopeLine.cat(')');
	outFile<<slopeLine<<endl;
	
	aString thetaLLine("theta L: ");thetaLLine.cat(bestparams.thetaL(),5,10);
	thetaLLine.separator(3);thetaLLine.cat('(');thetaLLine.cat(LcurrentList.lowerCI.thetaL(),5,10);thetaLLine.cat(" - ");
	thetaLLine.cat(LcurrentList.upperCI.thetaL(),5,10);thetaLLine.cat(')');thetaLLine.cat(" rate: ");thetaLLine.cat(successRate[3],1,6);
	outFile<<thetaLLine<<endl;

	aString thetaRLine("theta R: ");thetaRLine.cat(bestparams.thetaR(),5,10);
	thetaRLine.separator(3);thetaRLine.cat('(');thetaRLine.cat(LcurrentList.lowerCI.thetaR(),5,10);thetaRLine.cat(" - ");
	thetaRLine.cat(LcurrentList.upperCI.thetaR(),5,10);thetaRLine.cat(')');thetaRLine.cat(" rate: ");thetaRLine.cat(successRate[3],1,6);
	outFile<<thetaRLine<<endl;

	aString asLLine("asymptote L: ");asLLine.cat(bestparams.asymptoteL(),5,10);
	asLLine.separator(3);asLLine.cat('(');asLLine.cat(LcurrentList.lowerCI.asymptoteL(),5,10);asLLine.cat(" - ");
	asLLine.cat(LcurrentList.upperCI.asymptoteL(),5,10);asLLine.cat(')');asLLine.cat(" rate: ");asLLine.cat(successRate[5],1,6);
	outFile<<asLLine<<endl;

	aString asRLine("asymptote R: ");asRLine.cat(bestparams.asymptoteR(),5,10);
	asRLine.separator(3);asRLine.cat('(');asRLine.cat(LcurrentList.lowerCI.asymptoteR(),5,10);asRLine.cat(" - ");
	asRLine.cat(LcurrentList.upperCI.asymptoteR(),5,10);asRLine.cat(')');asRLine.cat(" rate: ");asRLine.cat(successRate[6],1,6);
	outFile<<asRLine<<endl;

	aString bottomLLine("bottom L: ");bottomLLine.cat(bestparams.bottomL(),5,10);
	bottomLLine.separator(3);bottomLLine.cat('(');bottomLLine.cat(LcurrentList.lowerCI.bottomL(),5,10);bottomLLine.cat(" - ");
	bottomLLine.cat(LcurrentList.upperCI.bottomL(),5,10);bottomLLine.cat(')');bottomLLine.cat(" rate: ");bottomLLine.cat(successRate[7],1,6);
	outFile<<bottomLLine<<endl;

	aString topRLine("top R: ");topRLine.cat(bestparams.topR(),5,10);
	topRLine.separator(3);topRLine.cat('(');topRLine.cat(LcurrentList.lowerCI.topR(),5,10);topRLine.cat(" - ");
	topRLine.cat(LcurrentList.upperCI.topR(),5,10);topRLine.cat(')');topRLine.cat(" rate: ");topRLine.cat(successRate[8],1,6);
	outFile<<topRLine<<endl;

	aString xLLine("xL: ");xLLine.cat(bestparams.xL(),5,10);
	xLLine.separator(3);xLLine.cat('(');xLLine.cat(LcurrentList.lowerCI.xL(),5,10);xLLine.cat(" - ");
	xLLine.cat(LcurrentList.upperCI.xL(),5,10);xLLine.cat(')');
	outFile<<xLLine<<endl;

	aString xRLine("xR: ");xRLine.cat(bestparams.xR(),5,10);
	xRLine.separator(3);xRLine.cat('(');xRLine.cat(LcurrentList.lowerCI.xR(),5,10);xRLine.cat(" - ");
	xRLine.cat(LcurrentList.upperCI.xR(),5,10);xRLine.cat(')');
	outFile<<xRLine<<endl;

	aString betaLLine("beta L: ");betaLLine.cat(bestparams.betaL(),5,10);
	betaLLine.separator(3);betaLLine.cat('(');betaLLine.cat(LcurrentList.lowerCI.betaL(),5,10);betaLLine.cat(" - ");
	betaLLine.cat(LcurrentList.upperCI.betaL(),5,10);betaLLine.cat(')');
	outFile<<betaLLine<<endl;

	aString betaRLine("beta R: ");betaRLine.cat(bestparams.betaR(),5,10);
	betaRLine.separator(3);betaRLine.cat('(');betaRLine.cat(LcurrentList.lowerCI.betaR(),5,10);betaRLine.cat(" - ");
	betaRLine.cat(LcurrentList.upperCI.betaR(),5,10);betaRLine.cat(')');
	outFile<<betaRLine<<endl;

	aString WhWpLine("Wh/Wp: ");WhWpLine.cat(bestparams.WhybridOverWpure(harMeanRecomb),5,10);
	WhWpLine.separator(3);WhWpLine.cat('(');
	WhWpLine.cat(LcurrentList.lowerCI.WhybridOverWpure(harMeanRecomb),5,10);WhWpLine.cat(" - ");
	WhWpLine.cat(LcurrentList.upperCI.WhybridOverWpure(harMeanRecomb),5,10);WhWpLine.cat(')');
	outFile<<WhWpLine<<endl;

	aString bigSLine("S: ");
	if (bestparams.bigS(harMeanRecomb) == 200.0){
					bigSLine.cat("100%");}
				else{
					bigSLine.cat(bestparams.bigS(harMeanRecomb),5,10);}
	bigSLine.separator(3);bigSLine.cat('(');
	if (bestparams.bigS(harMeanRecomb) == 200.0){
					bigSLine.cat("100%");}
				else{
					bigSLine.cat(LcurrentList.lowerCI.bigS(harMeanRecomb),5,10);}
	bigSLine.cat(" - ");
	if (bestparams.bigS(harMeanRecomb) == 200.0){
					bigSLine.cat("100%");}
				else{
					bigSLine.cat(LcurrentList.upperCI.bigS(harMeanRecomb),5,10);}
	bigSLine.cat(')');
	outFile<<bigSLine<<endl<<endl;

	if (! byLocus){//printing averages over loci
			if popCombListDiseq.len > 0 then
					writeLn(outputData, 'Population combinations in disequilibrium list:', popCombListDiseq.len : 3);
				if popCombListTotal.len > 0 then
					writeLn(outputData, 'Population combinations in totals list:', popCombListTotal.len : 3);
				if popCombListDiseq.len > 0 then
					writeln(outputData, '(Support limits taken from all population combinations pooled.)');
				for i := 1 to popCombListDiseq.len do
					begin
						centerPopList := ListStructureTypePtr(GetSubListAt(popCombListDiseq, i));
						writeln(outputData, 'Populations included in this disequilibrium estimate:');
						PrintPopCombList(centerPopList^, outputData);
						writeln(outputData, 'diseq Lmax: ', bestparamsDiseq^[i].Ldiseq : 20 : 12);
						writeln(outputData, 'D: ', bestparamsDiseq^[i].D : 15 : 10, Spaces(3), '(', lowerCIset.D : 15 : 10, ' - ', upperCIset.D : 15 : 10, ')');
						centerPopList := ListStructureTypePtr(GetSubListAt(popCombListTotal, i));
						writeln(outputData, 'Populations included in these estimates of derived cline parameters:');
						PrintPopCombList(centerPopList^, outputData);
						writeln(outputData, 'total Lmax: ', bestparamsTotal^[i].Ldiseq + bestparamsTotal^[i].Lcline : 20 : 12);
						writeln(outputData, 'sigma: ', bestparamsTotal^[i].sigma : 15 : 10, Spaces(3), '(', lowerCIset.sigma : 15 : 10, ' - ', upperCIset.sigma : 15 : 10, ')');
						writeln(outputData, 'timeL: ', bestparamsTotal^[i].timeL : 15 : 3, Spaces(3), '(', lowerCIset.timeL : 15 : 3, ' - ', upperCIset.timeL : 15 : 3, ')');
						writeln(outputData, 'timeR: ', bestparamsTotal^[i].timeR : 15 : 3, Spaces(3), '(', lowerCIset.timeR : 15 : 3, ' - ', upperCIset.timeR : 15 : 3, ')');
						writeln(outputData, 'vs. hets. s*: ', bestparamsTotal^[i].sVsHets : 15 : 10, Spaces(3), '(', lowerCIset.sVsHets : 15 : 10, ' - ', upperCIset.sVsHets : 15 : 10, ')');
						writeln(outputData, 'ecotone s*: ', bestparamsTotal^[i].sStarEcotone : 15 : 10, Spaces(3), '(', lowerCIset.sStarEcotone : 15 : 10, ' - ', upperCIset.sStarEcotone : 15 : 10, ')');
						writeln(outputData, 'Dom. ecotone s*: ', bestparamsTotal^[i].sStarDomEcotone : 15 : 10, Spaces(3), '(', lowerCIset.sStarDomEcotone : 15 : 10, ' - ', upperCIset.sStarDomEcotone : 15 : 10, ')');
						writeln(outputData, 'num. Loci selected: ', bestparamsTotal^[i].nLocSel : 15 : 3, Spaces(3), '(', lowerCIset.nLocSel : 15 : 3, ' - ', upperCIset.nLocSel : 15 : 3, ')');
						writeln(outputData, 'sel. per locus L: ', bestparamsTotal^[i].sPerMarkerL : 15 : 10, Spaces(3), '(', lowerCIset.sPerMarkerL : 15 : 10, ' - ', upperCIset.sPerMarkerL : 15 : 10, ')');
						writeln(outputData, 'sel. per locus R: ', bestparamsTotal^[i].sPerMarkerR : 15 : 10, Spaces(3), '(', lowerCIset.sPerMarkerR : 15 : 10, ' - ', upperCIset.sPerMarkerR : 15 : 10, ')');
						writeln(outputData, 'sel. per genetic isolation locus: ', bestparamsTotal^[i].sVsHetsRI : 15 : 10, Spaces(3), '(', lowerCIset.sVsHetsRI : 15 : 10, ' - ', upperCIset.sVsHetsRI : 15 : 10, ')');
						writeln(outputData);
					end;	{end for i}
			end
		else  {printing locus data only}
			begin
				writeln(outputData, 'diseq Lmax: ', bestparams.Ldiseq : 20 : 12);
				writeln(outputData, 'D: ', bestparams.D : 15 : 10, Spaces(3), '(', lowerCIset.D : 15 : 10, ' - ', upperCIset.D : 15 : 10, ')');
				writeln(outputData, 'total Lmax: ', bestparams.Ldiseq + bestparamsTotal^[i].Lcline : 20 : 12);
				writeln(outputData, 'sigma: ', bestparams.sigma : 15 : 10, Spaces(3), '(', lowerCIset.sigma : 15 : 10, ' - ', upperCIset.sigma : 15 : 10, ')');
				writeln(outputData, 'timeL: ', bestparams.timeL : 15 : 3, Spaces(3), '(', lowerCIset.timeL : 15 : 3, ' - ', upperCIset.timeL : 15 : 3, ')');
				writeln(outputData, 'timeR: ', bestparams.timeR : 15 : 3, Spaces(3), '(', lowerCIset.timeR : 15 : 3, ' - ', upperCIset.timeR : 15 : 3, ')');
				writeln(outputData, 'vs. hets. s*: ', bestparams.sVsHets : 15 : 10, Spaces(3), '(', lowerCIset.sVsHets : 15 : 10, ' - ', upperCIset.sVsHets : 15 : 10, ')');
				writeln(outputData, 'ecotone s*: ', bestparams.sStarEcotone : 15 : 10, Spaces(3), '(', lowerCIset.sStarEcotone : 15 : 10, ' - ', upperCIset.sStarEcotone : 15 : 10, ')');
				writeln(outputData, 'Dom. ecotone s*: ', bestparams.sStarDomEcotone : 15 : 10, Spaces(3), '(', lowerCIset.sStarDomEcotone : 15 : 10, ' - ', upperCIset.sStarDomEcotone : 15 : 10, ')');
				writeln(outputData, 'num. Loci selected: ', bestparams.nLocSel : 15 : 3, Spaces(3), '(', lowerCIset.nLocSel : 15 : 3, ' - ', upperCIset.nLocSel : 15 : 3, ')');
				writeln(outputData, 'sel. per locus L: ', bestparams.sPerMarkerL : 15 : 10, Spaces(3), '(', lowerCIset.sPerMarkerL : 15 : 10, ' - ', upperCIset.sPerMarkerL : 15 : 10, ')');
				writeln(outputData, 'sel. per locus R: ', bestparams.sPerMarkerR : 15 : 10, Spaces(3), '(', lowerCIset.sPerMarkerR : 15 : 10, ' - ', upperCIset.sPerMarkerR : 15 : 10, ')');
				writeln(outputData, 'sel. per genetic isolation locus: ', bestparams.sVsHetsRI : 15 : 10, Spaces(3), '(', lowerCIset.sVsHetsRI : 15 : 10, ' - ', upperCIset.sVsHetsRI : 15 : 10, ')');
				writeln(outputData);
			end;	{end if not byLocus...else}
*/
	
	}//end AllozymeClineShapePlus<T>::PrintBestParameters(ostream&)



template<class T> int AllozymeClineShapePlus<T>::VaryDiseq(const double temperature, 
									Ran3& ran, long& rep, int& switchParams, long& savedReps, long& annealAfterRep){
	int allDone = False, paramChanged = 0, annealing = False;
	ClinePopList *centerPopList = new ClinePopList();
	paramsInUse->FindCenterPopulations(*centerPopList, popLocation);
	//if this is a change from the previous centerPopList, then D and associated parameters must be
	//the most recent set from the current centerPopList
	if (! LcurrentDiseqList.IncludesPopComb(*centerPopList)){
		LcurrentDiseqList.Append(*paramsInUse, *centerPopList, savedReps);}
	if (! LcurrentTotalList.IncludesPopComb(*centerPopList)){
		LcurrentTotalList.Append(*paramsInUse, *centerPopList, savedReps);}
		
	LcurrentDiseqList.SetToCurrentDiseq(*paramsInUse, *centerPopList);

	if (! LcurrentDiseqList.Annealed(*centerPopList)){
			AnnealDiseq(annealAfterRep,temperature * 10.0, lowTemp, ran, paramsInUse, savedReps);
			allDone = False;
			}
		else{
			T* paramsToUse = new T(*paramsInUse);
			T* localBestParams = new T(LcurrentDiseqList.best(*centerPopList));
			allDone = DoOneDiseqIteration(temperature, annealing, ran, rep, 
						switchParams, paramsToUse, localBestParams, savedReps);
			*paramsInUse = *paramsToUse;
			delete paramsToUse; paramsToUse=0;
			delete localBestParams; localBestParams=0;
			}

	centerPopList->Clear();
	delete centerPopList; centerPopList=0;

	return allDone;
	}//end AllozymeClineShapePlus<T>::VaryDiseq
	
	
	
template<class T> void AllozymeClineShapePlus<T>::AnnealDiseq(const long repsBetweenAnnealSteps, 
						const double startTemp, const double endTemp, 
						Ran3& ran, T* defaultParams, long& savedReps){
	int DsaveValue = LcurrentDiseqList.saving();
	int TsaveValue = LcurrentTotalList.saving();
	LcurrentDiseqList.SetSave(False);
	LcurrentTotalList.SetSave(False);
	int paramChanged = 0, unsuccessfulTries = 0;
	int savingNow = False,  allDone = False, annealing = True, switchParams; //boolean flags
	long rep=0;
	double temperatureNow = startTemp;//anneal from a wide variance
	T* paramsToUse = new T(*defaultParams);
	T* localBestParams = new T(*defaultParams);
	//work with the parameters appropriate for this centerPopList
	ClinePopList centerPopList;
	defaultParams->FindCenterPopulations(centerPopList, popLocation);
	do{//here is the annealing loop
		if(rep > repsBetweenAnnealSteps){//anneal the temperature, which narrows the sampling variance
			temperatureNow /= 10.0;
			rep = 0;
			*paramsToUse = *localBestParams;
			}//end if rep
		DoOneDiseqIteration(temperatureNow, annealing, ran, rep, 
							switchParams, paramsToUse, localBestParams, savedReps);
		if (! switchParams){	//catches occasional dead-end traps when 
			//an unlikely parameter combination is accepted
				++unsuccessfulTries;
				if(unsuccessfulTries>=25){
					//this is a particularly bad parameter combination
					unsuccessfulTries=0;
					*paramsToUse = *localBestParams;}
				}
			else{unsuccessfulTries=0;}
		if (temperatureNow < endTemp){
				allDone = True;}
		}while (! allDone);//ends the annealing loop
	LcurrentDiseqList.SetAnnealed(centerPopList, True);
	*defaultParams = *localBestParams;
	delete paramsToUse; paramsToUse=0;
	delete localBestParams; localBestParams=0;
	LcurrentDiseqList.SetSave(DsaveValue);//rstore these
	LcurrentTotalList.SetSave(TsaveValue);
	}//AllozymeClineShapePlus<T>::AnnealDiseq




template<class T> int AllozymeClineShapePlus<T>::DoOneDiseqIteration(const double temperature, 
									const int annealing, Ran3& ran, long& rep, int& switchParams,
									T* paramsToUse, T* localBestParams, long& savedReps){
	int allDone = False, paramChanged = 0;
	delete paramsToTry; paramsToTry=0;
	paramsToTry = paramsToUse->ChangeDiseqParam(*paramDeviations, ran, 
										paramChanged, temperature);
	ClinePopList *centerPopList = new ClinePopList();
	paramsToTry->FindCenterPopulations(*centerPopList, popLocation);
	double LmaxDiseq = LcurrentDiseqList.bestL(*centerPopList);
	double LnewDiseq = DiseqLikelihood(*paramsToTry, *centerPopList);
	paramsToTry->SetDiseqLikelihoodRatios(LmaxDiseq);
	paramsToTry->SetTotal();
	++allTries;
	totalTries.Increment(paramChanged-1);
	switchParams = False;//if the fit is better, then save the new parameters

	if (paramsToTry->LtoLmaxDiseq() < 1.0){
			//if it's worse, then only save them with a probability proportional to their L ratio
			if (ran() < paramsToTry->LtoLmaxDiseq()) {
					switchParams = True;}}
		else{//new is better, so always switch
			switchParams = True;
			}//end if LtoLmaxCline... else...
	if(switchParams){
		acceptedChanges.Increment(paramChanged-1);
		if (LnewDiseq > LmaxDiseq){//save new bests in appropriate places
			LmaxDiseq = LnewDiseq;
			paramsToTry->SetNewBestDiseq();//adjusts Lratios
			*localBestParams = *paramsToTry;
			//replace initial values for diseq and total parameter estimates
			if (annealing){
					LcurrentDiseqList.Clear(*centerPopList);
					LcurrentTotalList.Clear(*centerPopList);
					cout<<"Annealing disequilibrium:"<<endl;
					}//abolish all old information
				else{
					cout<<"New best disequilibrium:"<<endl;}
			centerPopList->Print(cout); cout<<endl;
			localBestParams->PrintToScreen(allTries, harMeanRecomb);
			}//end if LnewCline > LmaxCline
		*paramsToUse = *paramsToTry;
//		LcurrentDiseq = LnewDiseq;
		int allDone1 = LcurrentDiseqList.Append(paramsToTry, centerPopList, savedReps);
		int allDone2 = LcurrentTotalList.Append(paramsToTry, centerPopList, savedReps);
		allDone = (allDone1 && allDone2);
		}//end if switchParams
	centerPopList->Clear();
	delete centerPopList; centerPopList=0;
	delete paramsToTry; paramsToTry=0;
	++rep;
	return allDone;
	}//end AllozymeClineShapePlus<T>::DoOneDiseqIteration






