//CurrentLikelihoodList.h


#pragma once
#include <aList.h>
#include <Boolean.h>
#include <math.h>
#include <Hyperbolics.h>
#include <InlineTemplates.h>
#include <Randomness.h>
#include <ParamRec.h>
#include <FitMultilocusCline.h>
#include <AbstractLikelihoodList.h>
#include "AllozymeClineGlobals.h"



class CurrentLikelihoodList:  public AbstractLikelihoodList{
	//For each set of center populations found in the cline-fitting iterations.
	//this class stores the center populations themselves, and
	//the list of acdeptable parameters for those populations.
	//The best-fitting set of parameters is always the 0th element,
	//and the current set of parameters is the last element.
	//The intervening elements in the list are unsorted.
	public:
		ParamRec overallBest;
		ObjList<ParamRec> bestParamList, currentParamList;//one for each centerPopList
		ObjList<ObjList<ParamRec> > listOfParamLists;

	public:
		CurrentLikelihoodList(likelihoodType z): AbstractLikelihoodList(z) {};
		CurrentLikelihoodList(CurrentLikelihoodList& c):
				AbstractLikelihoodList(c),
				overallBest(c.overallBest),
				bestParamList(c.bestParamList),
				currentParamList(c.currentParamList),
				listOfParamLists(c.listOfParamLists){};
		CurrentLikelihoodList(CurrentLikelihoodList* c): 
				AbstractLikelihoodList(c),
				overallBest(c->overallBest),
				bestParamList(c->bestParamList),
				currentParamList(c->currentParamList),
				listOfParamLists(c->listOfParamLists) {};
		~CurrentLikelihoodList(void){
				bestParamList.Clear();
				currentParamList.Clear();
				listOfParamLists.Clear();
				AbstractLikelihoodList::Clear();
				}
		
	int 			Append(ParamRec& x, ClinePopList& centerPopList){
						ParamRec* z = new ParamRec(x);
						Append(z,centerPopList);
						delete z; z=0;
						return saving;}
	int 			Append(ParamRec* x, ClinePopList& centerPopList);//appends a copy; old unchanged
	ParamRec& 	best(void){return overallBest;}
	double 		bestL(void){return overallBest.Lcline;}
	ParamRec*	best(ClinePopList& centerPopList);
	ParamRec*	current(ClinePopList& centerPopList);
	void 		Clear(void){
						bestParamList.Clear();
						currentParamList.Clear();
						listOfParamLists.Clear();
						ParamRec x;
						overallBest=x;
						AbstractLikelihoodList::Clear();
						}

	void 		PurgeUnlikelyParameters(void);
					//removes paramRec's that are unlikely relative to overall best
	void 		PurgeUnlikelyParameters(int loc);
					//removes paramRec's that are unlikely for centerPopList at loc
	void			SetBest(ClinePopList& popCombList, ParamRec* newBest);
		};//end class CurrentLikelihoodList

