//LikelihoodList.h


#ifndef LikelihoodListh
#define LikelihoodListh = 1

#include <aList.h>
#include <Boolean.h>
#include <math.h>
#include <Hyperbolics.h>
#include <InlineTemplates.h>
#include <Randomness.h>
//#include <Shape2ParamRec.h>
#include <ClinePopList.h>
#include <AbstractLikelihoodList.h>
#include "AllozymeClineGlobals.h"



template<class T> class LikelihoodList:  public AbstractLikelihoodList{
	//For each set of center populations found in the cline-fitting iterations.
	//this class stores the center populations themselves, and
	//the list of acdeptable parameters for those populations.
	//The best-fitting set of parameters is always the 0th element,
	//and the current set of parameters is the last element.
	//The intervening elements in the list are unsorted.
	public:
		T overallBest;
		ObjList<T> bestParamList, currentParamList;//one for each centerPopList
		ObjList<ObjList<T> > listOfParamLists;
		T lowerCI, upperCI;
		aList<double> bestDerived, lowerDerivedCI, upperDerivedCI;

	public:
		LikelihoodList(likelihoodType z): AbstractLikelihoodList(z) {}
		LikelihoodList(LikelihoodList<T>& c):
				AbstractLikelihoodList(c),
				overallBest(c.overallBest),
				bestParamList(c.bestParamList),
				currentParamList(c.currentParamList),
				listOfParamLists(c.listOfParamLists),
				lowerCI(c.lowerCI), upperCI(c.upperCI),
				bestDerived(c.bestDerived), lowerDerivedCI(c.lowerDerivedCI),
				upperDerivedCI(c.upperDerivedCI) {}
		LikelihoodList(LikelihoodList<T>* c): 
				AbstractLikelihoodList(c),
				overallBest(c->overallBest),
				bestParamList(c->bestParamList),
				currentParamList(c->currentParamList),
				listOfParamLists(c->listOfParamLists),
				lowerCI(c->lowerCI), upperCI(c->upperCI),
				bestDerived(c->bestDerived), lowerDerivedCI(c->lowerDerivedCI),
				upperDerivedCI(c->upperDerivedCI) {}
		~LikelihoodList(void){
				bestParamList.Clear();
				currentParamList.Clear();
				listOfParamLists.Clear();
				bestDerived.Clear();
				lowerDerivedCI.Clear();
				upperDerivedCI.Clear();
				AbstractLikelihoodList::Clear();
				}
		
	virtual int 	Append(T& x, ClinePopList& centerPopList, long& savedReps){
						T* z = new T(x);
						Append(z,centerPopList, savedReps);
						delete z; z=0;
						return saving();}
	virtual int 	Append(T* x, ClinePopList& centerPopList,long& savedReps);
						//should append a copy; old unchanged
	T& 			best(void){return overallBest;}
	virtual double 	bestL(void){return overallBest.Lcline();}
	virtual double 	bestL(ClinePopList& popCombList);
	virtual T*	best(ClinePopList& centerPopList);
	virtual T*	current(ClinePopList& centerPopList);
	virtual void 	Clear(void){
						bestParamList.Clear();
						currentParamList.Clear();
						listOfParamLists.Clear();
						bestDerived.Clear();
						lowerDerivedCI.Clear();
						upperDerivedCI.Clear();
						AbstractLikelihoodList::Clear();
						}
	virtual void 		Clear(ClinePopList& popCombList){
						int loc = listOfCenterPopLists.Loc(&popCombList);
						bestParamList.Delete(loc);
						currentParamList.Delete(loc);
						ObjList<T>* paramList = listOfParamLists[loc];
						totalSize -= paramList->len(); paramList=0;
						listOfParamLists.Delete(loc);
						bestDerived.Clear();
						lowerDerivedCI.Clear();
						upperDerivedCI.Clear();
						AbstractLikelihoodList::Clear(popCombList);
						}
	virtual void 	Find2UnitSupportLimits (const double recombinationRate);

	virtual void 	PrintBestParameters(long& allTries,
								aList<long>& acceptedChanges, 
								aList<long>& totalTries,
								ostream& outFile);
	virtual void 	PurgeUnlikelyParameters(void);
					//removes paramRec's that are unlikely relative to overall best
	virtual void 	PurgeUnlikelyParameters(int loc);
					//removes paramRec's that are unlikely for centerPopList at loc
	virtual void	SetBest(ClinePopList& popCombList, T* newBest);
		};//end class LikelihoodList

#endif
