// Shape2ParamRec.h

#ifndef Shape2ParamRech
#define Shape2ParamRech = 1


#include <aList.h>
#include <Boolean.h>
#include <math.h>
#include <Hyperbolics.h>
#include <InlineTemplates.h>
#include <Randomness.h>
#include "AllozymeClineGlobals.h"

class ClinePopList;
class Shape2ParamRec;

class Shape2ParamRec{//a set of cline shape parameters and their derived parameters
		//for only width and center location
	    public:
		double c_;//when fitting, these are randomly chosen
		double w_;		//width
		double Lcline_;	//likelihood of this parameter set
		double LtoLmaxCline_;
		double LmaxMinusLcline_;
	public:
		Shape2ParamRec(void):
			Lcline_(0.0), LtoLmaxCline_(0.0), LmaxMinusLcline_(0.0),
			c_(0.0), w_(0.0)
			{}
		Shape2ParamRec(Shape2ParamRec& x):	//copy constructor
				Lcline_(x.Lcline()), LtoLmaxCline_(x.LtoLmaxCline()), 
				LmaxMinusLcline_(x.LmaxMinusLcline()),
				c_(x.center()), w_(x.w())
				{}
		Shape2ParamRec(Shape2ParamRec* x):	//copy constructor
				Lcline_(x->Lcline()), LtoLmaxCline_(x->LtoLmaxCline()), 
				LmaxMinusLcline_(x->LmaxMinusLcline()),
				c_(x->center()), w_(x->w())
				{}
		virtual ~Shape2ParamRec(void){}

		virtual double center(void){return c_;}
		virtual double w(void) {return w_;}	
		virtual double bottomL(void){return 0.0;}//average allele freq on left side
		virtual double topR(void){return 1.0;} //average allele freq on right side
		virtual double Lcline(void){return Lcline_;}
		virtual double LtoLmaxCline(void) {return LtoLmaxCline_;}	
		virtual double LmaxMinusLcline(void){return LmaxMinusLcline_;}
		virtual void	 center(double c){c_ = c;}
		virtual void	 w(double w) {w_ = w;}	
		virtual void	 Lcline(double L){Lcline_ = L;}
		virtual void	 LtoLmaxCline(double ratio) {LtoLmaxCline_ = ratio;}	
		virtual void	 LmaxMinusLcline(double diff){LmaxMinusLcline_ = diff;}


	//some stand-in functions used in derived classes Shape6ParamRec to Shape9ParamRec
	virtual double thetaL(void){return 0.0;}//these apply to derived classes
	virtual double thetaR(void){return 0.0;}
	virtual double asymptoteL(void){return 0.0;}
	virtual double asymptoteR(void){return 0.0;}
	virtual double xL(void){return 0.0;}
	virtual double xR(void){return 0.0;}
	virtual double betaL(void){return 0.0;}
	virtual double betaR(void){return 0.0;}


	//THESE RETURN DERIVED PARAMETERS
	virtual double sVsHets (const double sigma){
						return (2.0*sigma / w())*(2.0*sigma / w());}
	virtual double sStarEcotone (const double sigma){	//selection in an ecotone, =(1.732*sigma/w)^2
						return (1.732*sigma / w())*(1.732*sigma / w());}
	virtual double sStarDomEcotone (const double sigma){//selection in an ecotone vs dominant locus, =(1.782*sigma/w)^2
						return (1.782*sigma / w())*(1.782*sigma / w());}


	//THESE ARE STANDARD FUNCTIONS
	Shape2ParamRec* 	
					ChangeParams (Shape2ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t, const int annealingShape);
	virtual int		ChooseParameter(Ran3& ran, const int annealingShape=0);
	virtual void		Clear(void){
						c_=w_=Lcline_=LtoLmaxCline_=LmaxMinusLcline_=0.0;}
	virtual Shape2ParamRec* 	Copy(void){//must be deleted elsewhere
						return new Shape2ParamRec(*this);}
	virtual void		DerivedValues(aList<double>& derivedList, double recombinationRate){//returns a list of derived variables
									derivedList.Clear();//none calculated at this level
									}
	virtual int 		FindCenterPopulations(ClinePopList& centerPopList, ClinePopList& popLocation);
	virtual int		HasHigherLikelihood(Shape2ParamRec& param, likelihoodType compare, const double ci = 0.0){//boolean
						//ci is usually zero, or 2.0 for 95% ci
						//always compares Lcline(); always greater if param.Lcline() = 0
						return ((param.Lcline() == 0.0) ||  (Lcline() > param.Lcline() - ci));}
	virtual void		max(Shape2ParamRec& param){//sets c_ & w_ to the maximum of this & param
							if(param.center()>center()){c_=param.center();}
							if(param.w()>w()){w_=param.w();}
							Lcline_=LtoLmaxCline_=LmaxMinusLcline_=0.0;}
	virtual int 		MeetsParamCriteria (void);//boolean for legal parameters
	virtual void		min(Shape2ParamRec& param){//sets c_ & w_ to the minimum of this & param
							if(param.center()<center()){c_=param.center();}
							if(param.w()<w()){w_=param.w();}
							Lcline_=LtoLmaxCline_=LmaxMinusLcline_=0.0;}
	virtual double 	PonCline(double x, double score);
	virtual void 		PrintShapeWithCI(const long allTries, 
							const double overallSuccessRate, 
							aList<double>& successRate,
							Shape2ParamRec& lowerCI, 
							Shape2ParamRec& upperCI, 
							aList<double>& bestDerived, 
							aList<double>& lowerDerivedCI, 
							aList<double>& upperDerivedCI, 
							ostream& outFile);//called by the best parameter set only
	virtual void		PrintToScreen(const long tries);
	virtual void		PrintToScreen(const long tries, const double sigma);
	virtual void 		SetClineLikelihoodRatios(double LmaxCline);
	virtual void		SetDeviationValues(void);
	virtual void 		SetNewBest(void){LtoLmaxCline_ = 1.0; LmaxMinusLcline_ = 0.0;}
	virtual int    		SetShape(void);//returns boolean false if illegal shape
	virtual void		SetStartingValues(void);
	virtual void 	operator=(Shape2ParamRec& x){
				Lcline_=x.Lcline(); LtoLmaxCline_=x.LtoLmaxCline(); 
				LmaxMinusLcline_=x.LmaxMinusLcline();
				c_=x.center(); w_=x.w();
				}

	friend int operator== (Shape2ParamRec& a, Shape2ParamRec& b){
		if(	(a.center()==b.center())&&			(a.w()==b.w())	
		    )		 {return 1;}else{ return 0;}
		}//end operator ==

	friend int operator!= (Shape2ParamRec& a, Shape2ParamRec& b){
		return (!(a==b));}

	};//end class Shape2ParamRec

#endif

