// Shape6ParamRec.h

#ifndef Shape6ParamRech
#define Shape6ParamRech = 1

#include <Shape2ParamRec.h>

class ClinePopList;
class Shape6ParamRec;

class Shape6ParamRec: public Shape2ParamRec{//a set of cline shape parameters and their derived parameters
		//for width, center location, theta, and the asymptotic edges of the single-locus clines on the ends
	    public:
		double thetaL_;		//decay on left side
		double thetaR_;		//decay on right side
		double asymptoteL_;	//asymptote of right side, which is left of center
		double asymptoteR_;	//asymptote of left side, which is right of center
		double xL_;	//these are calculated values
		double xR_;
		double betaL_;	
		double betaR_;
	public:
		Shape6ParamRec(void):
			thetaL_(0.0), thetaR_(0.0),
			asymptoteL_(0.0), asymptoteR_(0.0),
			xL_(0.0), xR_(0.0),
			betaL_(0.0), betaR_(0.0)
			{}
		Shape6ParamRec(Shape6ParamRec& x){	//copy constructor
				c_= x.center();
				w_= x.w();
				thetaL_ = x.thetaL();
				thetaR_ = x.thetaR();
				asymptoteL_ = x.asymptoteL();
				asymptoteR_ = x.asymptoteR();
				xL_ = x.xL();
				xR_ = x.xR();
				betaL_ = x.betaL();
				betaR_ = x.betaR();
				Lcline_= x.Lcline();
				LtoLmaxCline_= x.LtoLmaxCline();
				LmaxMinusLcline_= x.LmaxMinusLcline();
				}
		Shape6ParamRec(Shape6ParamRec* x){	//copy constructor
				c_= x->center();
				w_= x->w();
				thetaL_ = x->thetaL();
				thetaR_ = x->thetaR();
				asymptoteL_ = x->asymptoteL();
				asymptoteR_ = x->asymptoteR();
				xL_ = x->xL();
				xR_ = x->xR();
				betaL_ = x->betaL();
				betaR_ = x->betaR();
				Lcline_= x->Lcline();
				LtoLmaxCline_= x->LtoLmaxCline();
				LmaxMinusLcline_= x->LmaxMinusLcline();
				}
		virtual ~Shape6ParamRec(void){}

		virtual double thetaL(void){return thetaL_;}
		virtual double thetaR(void){return thetaR_;}
		virtual double asymptoteL(void){return asymptoteL_;}
		virtual double asymptoteR(void){return asymptoteR_;}
		virtual double xL(void){return xL_;}
		virtual double xR(void){return xR_;}
		virtual double betaL(void){return betaL_;}
		virtual double betaR(void){return betaR_;}
		virtual void	 thetaL(double t){thetaL_ = t;}
		virtual void	 thetaR(double t){thetaR_ = t;}
		virtual void	 asymptoteL(double x){asymptoteL_ = x;}
		virtual void 	 asymptoteR(double x){asymptoteR_ = x;}
		virtual void	 xL(double x){xL_ = x;}
		virtual void	 xR(double x){xR_ = x;}
		virtual void	 betaL(double b){betaL_ = b;}
		virtual void	 betaR(double b){betaR_ = b;}


	//DERIVED VARIABLES
	virtual double WhybridOverWpure (const double recombinationRate);
	virtual double bigS (const double recombinationRate);



	//STANDARD FUNCTION CALLS
	virtual Shape6ParamRec* 	
					ChangeParams(Shape2ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t, const int annealingShape);
	virtual int		ChooseParameter(Ran3& ran, const int annealingShape=0);
	virtual void		Clear(void){
						Shape2ParamRec::Clear();
						thetaL_=thetaR_=asymptoteL_=asymptoteR_=xL_=xR_=betaL_=betaR_=0.0;
						}
	virtual Shape6ParamRec* 	Copy(void){//must be deleted elsewhere
						return new Shape6ParamRec(*this);}
	virtual void		DerivedValues(aList<double>& derivedList, double recombinationRate){
						derivedList.Clear();
						derivedList.Append(WhybridOverWpure(recombinationRate));
						derivedList.Append(bigS(recombinationRate));
						}		//listed in order of calls above
	virtual int 		FindCenterPopulations(ClinePopList& centerPopList, ClinePopList& popLocation);
	virtual int 		LJunction (const double score); // sets xL_ and betaL_, returns boolean re success
	virtual void		max(Shape6ParamRec& param){//sets parameters to the maximum of this & param
							Shape2ParamRec::max(param);
							if(param.thetaL()>thetaL()){thetaL_=param.thetaL();}
							if(param.thetaR()>thetaR()){thetaR_=param.thetaR();}
							if(param.asymptoteL()>asymptoteL()){asymptoteL_=param.asymptoteL();}
							if(param.asymptoteR()>asymptoteR()){asymptoteR_=param.asymptoteR();}
							if(param.xL()>xL()){xL_=param.xL();}
							if(param.xR()>xR()){xR_=param.xR();}
							if(param.betaL()>betaL()){betaL_=param.betaL();}
							if(param.betaR()>betaR()){betaR_=param.betaR();}
							}
	virtual int 		MeetsParamCriteria (void);//boolean for legal parameters
	virtual void		min(Shape6ParamRec& param){//sets parameters to the minimum of this & param
							Shape2ParamRec::min(param);
							if(param.thetaL()<thetaL()){thetaL_=param.thetaL();}
							if(param.thetaR()<thetaR()){thetaR_=param.thetaR();}
							if(param.asymptoteL()<asymptoteL()){asymptoteL_=param.asymptoteL();}
							if(param.asymptoteR()<asymptoteR()){asymptoteR_=param.asymptoteR();}
							if(param.xL()<xL()){xL_=param.xL();}
							if(param.xR()<xR()){xR_=param.xR();}
							if(param.betaL()<betaL()){betaL_=param.betaL();}
							if(param.betaR()<betaR()){betaR_=param.betaR();}
							}
	virtual double 	PonCline(const double x, const double score);
	virtual void 		PrintShapeWithCI(const long allTries, 
							const double overallSuccessRate, 
							aList<double>& successRate,
							Shape6ParamRec& lowerCI, 
							Shape6ParamRec& upperCI, 
							aList<double>& bestDerived, 
							aList<double>& lowerDerivedCI, 
							aList<double>& upperDerivedCI, 
							ostream& outFile);//called by the best parameter set only
	virtual void 		PrintToScreen (const long tries, const double recombinationRate);
	virtual int 		RJunction (const double score); // sets xR_ and betaR_, returns boolean re success
	virtual void 		SetBarrierStrength(void);
	virtual void		SetDeviationValues(void);
	virtual int 		SetShape(void);
	virtual void		SetStartingValues(void);
	virtual void 		operator=(Shape6ParamRec& x){
						c_= x.center();
						w_= x.w();
						thetaL_ = x.thetaL();
						thetaR_ = x.thetaR();
						asymptoteL_ = x.asymptoteL();
						asymptoteR_ = x.asymptoteR();
						xL_ = x.xL();
						xR_ = x.xR();
						betaL_ = x.betaL();
						betaR_ = x.betaR();
						Lcline_= x.Lcline();
						LtoLmaxCline_= x.LtoLmaxCline();
						LmaxMinusLcline_= x.LmaxMinusLcline();
						}

	friend int operator== (Shape6ParamRec& a, Shape6ParamRec& b);

	friend int operator!= (Shape6ParamRec& a, Shape6ParamRec& b){
		return (!(a==b));}

	};//end class Shape6ParamRec

#endif

