/*******************************************************
*
*	GeneticGridCell
*
*	Base class: GridCell
*
*	This class handles the interface for genetic individuals
*	of different species.  It adds only functions.

*******************************************************/
#ifndef GeneticGridCellh
#define GeneticGridCellh = 1

#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "GeneticPopulation.h"
#include "GridCell.h"
#include "StringClass.h"
#include "MemoryManager.h"

class GridCell;
class LandScape;


class GeneticGridCell: public GridCell{

  public:
  	GeneticGridCell(LandScape* r, CoordinateType coords, int habitat):
  		GridCell(r,coords,habitat){
		}
 
  	GeneticGridCell(LandScape* r, CoordinateType coords):
  		GridCell(r,coords){
		}
  	
  	GeneticGridCell(void){
		}
  		
	GeneticGridCell(GeneticGridCell& g)://neighbors & pops not copied
  		GridCell(g){
		}
	GeneticGridCell(GeneticGridCell* g)://neighbors & pops not copied
  		GridCell(g){
		}
  	
virtual ~GeneticGridCell(void) {
		}
  	
virtual GridCell* 		Duplicate(LandScape* r, CoordinateType coords, int habitat);
virtual GridCell* 		Duplicate(LandScape* r, CoordinateType coords);
virtual GridCell* 		Duplicate(void);//virtual copy constructor


//virtual double		AlleleFrequency(int species,int loc, T alleleVal)
virtual double		AlleleFrequency(int species,int loc, aString& alleleVal){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							return pop->AlleleFrequency(loc,alleleVal);}

virtual void		AssignMutations(int species){//derive this
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							pop->AssignMutations();}
							
virtual void		CalcAlleleFrequencies(int species){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							pop->CalcAlleleFrequencies();}
							
virtual void		CalcExpHeterozygosity(int species){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							pop->CalcExpHeterozygosity();}
							
virtual void		CalcObsHeterozygosity(int species){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							pop->CalcObsHeterozygosity();}
							
virtual void 		CoordsOfNthLocus(int species, int nthLoc, int& onChromNum, int& locNumOnChrom);

							
virtual  aString*	GetPhenotypeString(int species, long indiv, char* separator = " ");

virtual int		loci(int species);
							
virtual int		NthLocus(int species, int chrom, int locOnChrom);//0 indexed

virtual int		ploidy(int species, int locus);

virtual void		PrintAlleleFreqs(ostream& outFile, int species){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							pop->PrintAlleleFreqs(outFile);}
							
virtual void		PrintAlleleFreqs(ostream& outFile, int species, int locus){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							pop->PrintAlleleFreqs(outFile, locus);}
							
//virtual void		PrintAlleleFreqs(ostream& outFile, int species, int locus, T alleleVal)
virtual void		PrintAlleleFreqs(ostream& outFile, int species, int locus, aString& alleleVal){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							pop->PrintAlleleFreqs(outFile, locus,alleleVal);}
							
virtual void		PrintAlleleList(ostream& outFile, int species){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							pop->PrintAlleleList(outFile);}
							
virtual void		PrintAlleleList(ostream& outFile, int species, int locus){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							pop->PrintAlleleList(outFile, locus);}
							
virtual void		PrintGeneStrings(ostream& outFile, int species, int alleleLen=4, int codeLen=5){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							pop->PrintGeneStrings(outFile, alleleLen,codeLen);}
							
//virtual void 		Reproduce (int species, long& idCount, Ran3List& ran, double mutationRate, double sigma);

virtual void 		Reproduce (int species, long& idCount, Ran3List& ran, aList<long double>& specialInfo);

//virtual void 		Reproduce (int /*species*/, long& /*idCount*/, Ran3List& /*ran*/){
//							cout<<"ERROR: Shouldn't arrive here"<<endl;
//							cout<<"Call GeneticGridCell<T>::Reproduce(int,long&,Ran3List&,double,double) instead"<<endl;}


virtual void		UpdateAlleleList(int species){
							//makes a list of all alleles in the pop
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							pop->UpdateAlleleList();}
							
//virtual int		AllelePresent (int species, int loc, T alleleVal)
virtual int			AllelePresent (int species, int loc, aString& alleleVal){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							return pop->AllelePresent(loc,alleleVal);}
							
virtual long		Alleles (int species, int loc){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							return pop->Alleles(loc);}


//virtual aString*	AlleleValue (int species, int loc, int alleleNum)
virtual aString&	AlleleValue (int species, int loc, int alleleNum){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							return pop->AlleleValue(loc,alleleNum);}


//virtual long		CountAlleles (int species, int loc, T alleleVal)
virtual long		CountAlleles (int species, int loc, aString& alleleVal){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							return pop->CountAlleles(loc,alleleVal);}
							
/*virtual long		CountHets (int species, int loc, T alleleVal)*/
virtual long		CountHets (int species, int loc, aString& alleleVal){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							return pop->CountHets(loc, alleleVal);}
							
virtual long		CountHets (int species, int loc){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							return pop->CountHets(loc);}
virtual aString* 	GenotypeString(int species, int loc, int alleleNum, char separator=' '){
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							return pop->GenotypeString(loc,alleleNum, separator);}


virtual long		SampleSize (int species, int loc){//handles ploidy differences
							GeneticPopulation* pop = (GeneticPopulation*) &(pops_[species]);
							return pop->SampleSize(loc);}
							
  	};//end GeneticGridCell

#endif

