/***************************************************
*																														}
*	GeneticIndividual.h

	Base Class: Individual																					}
*																														}
*	Handles processes involving individuals with genetic information.
*	Genotypes are held in a container class called Phenotype.
*	Assumes asexual reproduction.  Used derived classes for sex.
*																														}
****************************************************/

#ifndef GeneticIndividualh
#define GeneticIndividualh = 1

#include "aError.h"
#include "Element.h"
#include "aList.h"
#include "InlineTemplates.h"
#include "Randomness.h"
#include "Individual.h"
#include "Population.h"
#include <iostream.h>
#include "SelectionIntf.h"
#include "StringClass.h"
#include "RandomList.h"
#include "CoreGenome.h"
#include "MemoryManager.h"


class GeneticIndividual: public Individual{
	public:
		Phenotype*	genes_;	
	public:
						GeneticIndividual(Phenotype& newGenes,long id=0, Population* site=NULL);
						GeneticIndividual(long id,Population* site=NULL);
						GeneticIndividual(void);
						GeneticIndividual(GeneticIndividual& indiv);
		 				GeneticIndividual(GeneticIndividual* indiv);
virtual		 		~GeneticIndividual(void);
virtual GeneticIndividual* 	Duplicate(Phenotype& newGenes, long id, Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual Individual* 		Duplicate(long id,Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual Individual* 		Duplicate(void);//VIRTUAL COPY CONSTRUCTOR

/*virtual double 			additiveRelativePhenotype(void){
							CoreGenome* g = (CoreGenome*) genes_;
							return g->additiveRelativePhenotype();}
*/
//virtual int				AllelePresent (int chrom, int loc, T alleleVal);
virtual int				AllelePresent (int loc, aString& alleleVal);
/*virtual T 				AlleleValue(int loc, int alleleNum){
							CoreGenome* g = (CoreGenome*) genes_;
							return g->GetGene(loc, alleleNum);}
*/
//virtual void			Clone (long& idCount, Ran3& ran, double mutationRate, double sigma);
virtual void			Clone (long& idCount, Ran3List& ran, aList<long double>& specialInfo);
//virtual void			Clone (long& /*idCount*/, Ran3& /*ran*/, aList<double>& /*specialInfo*/){
//							aError("ERROR: Shouldn't arrive here.");
//							aError(" Call GeneticIndividual<T>::Clone(long&,Ran3&,aList<double>&) instead.");}
//virtual void			Clone (Individual*){
//virtual void			Clone (Individual* /*indiv*/, Ran3& /*ran*/, aList<double>& /*specialInfo*/){
//							aError("ERROR: Shouldn't arrive here.");
//							aError(" Call GeneticIndividual<T>::Clone(Individual*,Ran3&,aList<double>&) instead.");}
//virtual void			Clone (Individual* mom, Ran3& ran, double mutationRate, double sigma);
virtual void			Clone (Individual* mom, Ran3List& ran, aList<long double>& specialInfo);
void					Copy (Individual* it){
							Individual::Copy(it);
							delete genes_; genes_ = NULL;}
virtual void			Copy (GeneticIndividual* it){
							Individual::Copy((Individual*) it);
							genes_ = it->genes_->Duplicate();
							}
//virtual int				CountAlleles (int chrom, int loc, T alleleVal)
virtual int				CountAlleles (int loc, aString& alleleVal);
//virtual double 			fitness(aList<double>& selParams);
virtual long double 	fitness(void);
//virtual T				GetGene (int locus, int alleleNum);//returns allele's value
virtual void			GetGenome(GeneContainerCore& gc){//refills container class at appropriate level
							if(genes_) genes_->GetGenome(gc,ID());}
virtual void			SetGenome(GeneContainerCore& gc){//initializes genome using appropriate level of container class
							if(genes_) genes_->SetGenome(gc);}
virtual CoreGenome&		GetGenotype (void);
virtual aString*		GenotypeString(char* separator = " ");
virtual aString*		GenotypeString(int locus, char* separator = " ");
virtual aString*		GenotypeString(int locus, int alleleNum, char* separator = " ");
/*virtual aString*		GenotypeString(int chrom, int locOnChrom, int alleleNum, char* separator = " "){
							CoreGenome* g = (CoreGenome*) genes_;
							return g->GenotypeString(chrom, locOnChrom, alleleNum, separator);}
*/
virtual aString*		GetPhenotypeString(char* separator = " "){
							return genes_->GetPhenotypeString(separator);}
//virtual void 			IIIndividual (genderSet gender, lifeStageType lifeStage, 
//								double rate, Ran3List& ran, LandScapeInitializer00& init);
virtual void 			IIIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran, LandScapeInitializer01& init, aList<long double>& extraInfo);
virtual int				IsHeterozygote (int loc);//returns false=0 if haploid
//virtual int				IsHeterozygote (int chrom, int loc, T alleleVal); //returns false=0 if haploid
virtual int				IsHeterozygote (int loc, aString& alleleVal); //returns false=0 if haploid
virtual int 			loci(void);
//virtual void 			MakeABaby (long& idCount, Ran3List& ran, double mutationRate, double sigma);
//virtual void 			MakeABaby (long& idCount, Ran3List& ran, aList<double>& specialInfo);
//virtual void 			MakeABaby (long& /*idCount*/, Ran3List& /*ran*/){
//							aError("ERROR: Shouldn't arrive here.");
//							aError(" Call Individual::MakeABaby(long&,Ran3List&,aList<double>&) instead.");}
//virtual void			Mutate (Ran3& ran, double mutationRate, double sigma=0.0);
virtual void			Mutate (Ran3& ran, aList<long double>& specialInfo);
//virtual void			Mutate (int chrom, int loc, Ran3& ran, T minVal, T maxVal);
virtual int 			ploidy(int loc);
/*virtual int				PloidyAtChrom (int chrom){//derive genetic sex determination etc.
							CoreGenome* g = (CoreGenome*) genes_;
							return g->PloidyAtChrom(chrom);}
*/
virtual long double		phenotype(int i=0);
virtual void 			PrintGenes(ostream& outFile, int alleleLen=4, int codeLen=6);
virtual genderSet 		RealizedSexOfClassMember(long double /*sexRatio*/, Ran3&){
							return A;}
						//returns any one of the possible sexes of this class
//virtual void 			Reproduce (long& idCount, Ran3List& ran, double mutationRate, double sigma);
virtual void 			Reproduce (long& idCount, Ran3List& ran, aList<long double>& specialInfo);
//virtual void 			Reproduce (long& /*idCount*/, Ran3List& /*ran*/){
//							aError("ERROR: Shouldn't arrive here.  Call GeneticIndividual::Reproduce(long&,Ran3List&,aList<double>&) instead.");}
virtual void 			SetGenotypeString(aString& geneStr,int alleleLen=4);
virtual void			SetPhenotype(void);
virtual void			SetPhenotype(Ran3& ran, aList<long double>& heritabilities);
virtual void 			SetSelectionParams (aList<long double>& selParams);
virtual int 			SurvivesViabilitySelection  (aList<long double>& selParams, Ran3& ran, long double& fitness);
							/*0=dead; 1=alive; used in natural selection*/
virtual int 			traits(void){
							return genes_->traits();}
	};		//end GeneticIndividual


#endif
	
