/***************************************************
*
*	Individual
*
*	Handles processes involving individuals within populations.
*	Genetic stuff done by derived class GeneticIndividual.
*	Collect data (dispersal etc.) using derived classes.
*
****************************************************/

#ifndef Individualh
#define Individualh = 1

//class LandScapeInitializer00;
class LandScapeInitializer01;

#include <iostream.h>
#include <ostream.h>
#include "StringClass.h"
#include "StringList.h"
#include "Element.h"
#include "aList.h"
#include "RandomList.h"
#include "InlineTemplates.h"
#include "PopulationIntf.h"
#include "SelectionIntf.h"
#include "CoordinateClass.h"
#include "GeneContainers.h"
#include "LandScapeInitializer01.h"
#include "MemoryManager.h"

class Population;/*declared here but defined elsewhere*/
class GridCell;
//class CoreGenome;
class Individual;

class Individual{
	protected:
		long 			id_;/*unique to each individual*/
	public:
		int				age_;//0-indexed
		int				alive_;
		int 			causeOfDeath_;
		lifeStageType	lifeStage_;
		long double 	moveRate_;/*stdev of movement ability*/
		genderSet		sex_;
		Population*		pop_;
		int				movedThisGen_;
/*		CoordinateType	endLoc_;		//derive: these for recording dispersal data
		float			lifetimeDisplace_Xaxis;
		float			lifetimeDisplace_Yaxis;
		float			lifetimeDisplace;
		float			lifetimeDist;
*/
	public:
						Individual(long id,Population* site=NULL);
						Individual(void);
		 				Individual(Individual& indiv);
		 				Individual(Individual* indiv);
virtual	 				~Individual(void);
virtual Individual* 		Duplicate(long id,Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual Individual* 		Duplicate(void);//VIRTUAL COPY CONSTRUCTOR
virtual long			GetPointerToDerivedSelf(void){
								return (long) this;}


/*FUNCTIONS*/
virtual int				age(void) {return age_;}
virtual void			age(int newAge){ age_=newAge;}//use for initializing
virtual int 			alive(void) {return alive_;}
virtual void			AdultMortality (aList<long double>& survivalSchedule,Ran3List& ran);
virtual void			Birthday (int maxage);
virtual int 			causeOfDeath(void){return causeOfDeath_;}
virtual void			Clone (Individual* mom, Ran3List& /*ran*/, aList<long double>& /*specialInfo*/);
virtual void			Clone (long& idCount, Ran3List& /*ran*/, aList<long double>& /*specialInfo*/);
virtual CoordinateType&	coords(void);
virtual void			Copy (Individual* it);
virtual int				dead(void){return ! alive_;}
virtual void 			Dies(void){alive_=0;}
virtual char*			GetGenderChar (void);
virtual int				Fecundity(Ran3List& ran);/*number of babies to produce*/
		long			id(void){return id_;}
virtual long double 	gridScale(void);
virtual int				habitat(void);/*looks at the site's habitat*/
		long			ID(void){return id_;}
virtual void			ImmatureMortality (aList<long double>& survivalSchedule,Ran3List& ran);
virtual long double		intrinsicSexRatio(void);
virtual void 			IIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List&);
//virtual void 			IIndividual (genderSet gender, lifeStageType lifeStage, 
//								long double rate, Ran3List& ran, LandScapeInitializer00&);
virtual void 			IIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran, LandScapeInitializer01& /*init*/,aList<long double>& /*extraInfo*/);
virtual lifeStageType	lifeStage(void){return lifeStage_;}
virtual void			lifeStage(lifeStageType newStage){lifeStage_=newStage;}//use for initializing
virtual void			MakeABaby (long& idCount, Ran3List& ran, aList<long double>& specialInfo) {
								Clone(idCount, ran, specialInfo);}
virtual migrationModelSet migrationModel(void);
virtual long double		MigrationProb(int currentHabitat, int potentialHabitat);
virtual int				movedThisGen(void) {return movedThisGen_;}
virtual void			movedThisGen(int moved) {movedThisGen_=moved;}
//virtual CoordinateType&	MoveRandomly (Ran3List& ran, double /*kurtosisFactor=0.0*/);
virtual CoordinateType&	MoveRandomly (Ran3List& ran, aList<long double>& dispersalInfo);
virtual CoordinateType& MoveRandomlyKurtoticDiffusion (Ran3List& ran, long double kurtosisFactor);
virtual CoordinateType&	MoveRandomlyDiffusion (Ran3List& ran, aList<long double>& /*dispersalInfo*/);
virtual CoordinateType&	MoveRandomlyIsland (Ran3List& ran, aList<long double>& /*dispersalInfo*/);
virtual CoordinateType&	MoveRandomlyStepStone (Ran3List& ran, aList<long double>& /*dispersalInfo*/);
//virtual CoordinateType&	MoveRandomlyKurtoticDiffusion (Ran3List& ran, double kurtosisFactor=0.0);
virtual long double		moveRate(void);
virtual void			moveRate(long double r) {moveRate_=r;}
//virtual void 			NaturalSelection(Ran3List&, selectionSourceSet){}
						/*default does nothing -- depends on phenotypes
						  in derived classes   */
virtual Population*		pop (void);
virtual void			pop (Population* p);
virtual void			PrepareForBurial (void);
virtual void			Print(ostream& outFile, char sepChar='\t');
virtual genderSet 		RealizedSexOfClassMember(long double, Ran3&){return A;}
						//returns any one of the possible sexes of this class
//virtual void			Reproduce (long& idCount, Ran3List& ran);
virtual void			Reproduce (long& idCount, Ran3List& ran, aList<long double>& specialInfo);
virtual void			ReproduceBySyngamy (long& /*idCount*/, Ran3List&, aList<long double>& /*specialInfo*/){}
virtual long double 	selectionStrength(void) { return zero;}/*default
						baseline selection in simple selection models*/
virtual genderSet		sex (void) {return sex_;}
virtual void			sex (genderSet gender) {sex_=gender;}
virtual void			sex (char genderChar);
virtual GridCell&		site (void);
virtual int				species(void);
virtual StringList* 	stateNames(void);//column headings: ="id, age, sex, etc."
virtual StringList* 	states(void);// indiv values for stateNames() above
		long 			time(void);//returns time from Landscape via Population & GridCell
virtual void			ToNewLoc (long, long);//use to record movement data; (long xdev, long ydev)
virtual int 			WillEnterHabitat(GridCell* g, long&, Ran3List&);


friend int		operator== (Individual& x, Individual& y) {return (y.id_==x.id_);}
friend int		operator!= (Individual& x, Individual& y) {return (!(y==x));}
friend int		operator< (Individual& x, Individual& y) {return (y.id_<x.id_);}
friend int		operator<= (Individual& x, Individual& y) {return (y.id_<=x.id_);}
friend int		operator> (Individual& x, Individual& y) {return (y.id_>x.id_);}
friend int		operator>= (Individual& x, Individual& y) {return (y.id_>=x.id_);}
	
/*THESE FUNCTIONS USED BY DERIVED PHENOTYPIC/GENETIC CLASSES ONLY*/
//virtual int			AllelePresent (int, int, int){return 0;}
//virtual int			CountAlleles (int, int, int){return 0;}
//virtual int			GetGene (int, int, int){return (-1);}
//virtual int			IsHeterozygote (int, int){return 0;}
//virtual int			IsHeterozygote (int, int, int){return 0;}
//virtual void		Mutate (Ran3&, int, int){}
//virtual void		Mutate (int, int, Ran3&, int, int){}
//virtual int			PloidyAtChrom (int){return 0;}
//virtual void 		PrintGenes(ostream&, int, int){}
virtual void		GetGenome(GeneContainerCore& /*gc*/){//refills container class at appropriate level
						aError("Error: base class Individual has no genome");}
virtual void		SetGenome(GeneContainerCore& /*gc*/){//initializes genome using appropriate level of container class
						aError("Error: base class Individual has no genome");}
//virtual void 		SetGenotypeString(aString&,int){}
virtual int 		SurvivesViabilitySelection  (aList<long double>&, Ran3&, long double& fitness) {
							fitness = one;
							return alive_;}/*0=dead; 1=alive; used in natural selection*/

/*THESE FUNCTIONS USED BY DERIVED SEXUAL CLASSES ONLY*/
virtual int			CanMate (int){return 0;}
virtual void		FindMate (Ran3List&, aList<long double>&, int, int){}
virtual genderSet	gameteType(void){ return A;}
virtual void		mated(void){}
virtual int			matings(void){return 0;}
virtual int			matingsThisAgeClass(void){return 0;}
virtual void		SetMate (Individual*, int){}
virtual long double	sexualAttractiveness(void){return one;}/*used in sexual selection*/

/*THESE USED BY DERIVED DIOECIOUS CLASSES ONLY*/


/*THESE USED BY DERIVED SEXUAL GENETIC CLASSES ONLY*/
virtual void			ClearMates (void){}
//virtual Genome&		GetMate (Ran3List&){return NULL;}
//virtual Genome&		GetMate (int ){return NULL;}
//virtual ObjList<Genome>&	GetMates (void) {return NULL;}
virtual int				mates (void) {return 0;}


	};		/*end Individual*/


#endif
