/***************************************************
*																														}
*	QuickAutoSexLinkSibmateIndiv.h

	Base Class: QuickAutosomeSexChromIndividual, QuickDiplSexGenIndividual, Individual																			}
*																														}
*	Handles processes involving individuals with autosomal and X-linked genetic information.
*	Genotype is an array of short integers, stored separately for sex-linked genes.
*	Sexual reproduction, but mate identity not stored; must mate at 
*	site of reproduction.  Loci autosomal or X-linked, but assumed to be unlinked.
*	Note:  assumes male is XO.
*																														}
****************************************************/

#ifndef QuickAutoSexLinkSibmateIndivh
#define QuickAutoSexLinkSibmateIndivh = 1

#include "aError.h"
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "Population.h"
#include <iostream.h>
#include "SelectionIntf.h"
#include "StringClass.h"
#include "RandomList.h"
#include "BetaDistribution.h"
#include "MemoryManager.h"
#include "Individual.h"
#include "GeneContainers.h"
#include "QuickDiplSexGenIndividual.h"
#include "QuickAutosomeSexChromIndividual.h"

// 20090828 created

//#ifndef DEBUG_REPRODUCTION
//#define DEBUG_REPRODUCTION = 1
//#endif

class QuickAutoSexLinkSibmateIndiv;

class QuickAutoSexLinkSibmateIndiv: virtual public QuickAutosomeSexChromIndividual,
									virtual public QuickDiplSexGenIndividual, 
									virtual public Individual{
	public:
		QuickAutoSexLinkSibmateIndiv *mom_, *dad_;//null if they are dead
		ObjList<QuickAutoSexLinkSibmateIndiv> sibsF_, sibsM_;
		ObjList<QuickAutoSexLinkSibmateIndiv> offspringF_, offspringM_;
		long double sibMatingProb_, sexRatio_;
	public:
						QuickAutoSexLinkSibmateIndiv(const int nlociA, const int nlociX, const int ploidyX, int* newGenesA, 
									int* newGenesX, long double sexratio, long idVal=-1, Population* popSite=NULL);
						QuickAutoSexLinkSibmateIndiv(const int nlociA, const int nlociX, const int ploidyX, int* newGenesA, 
									int* newGenesX, long idVal=-1, Population* popSite=NULL);
						QuickAutoSexLinkSibmateIndiv(long id,Population* site=NULL);
						QuickAutoSexLinkSibmateIndiv(void);
						QuickAutoSexLinkSibmateIndiv(QuickAutoSexLinkSibmateIndiv& indiv);
		 				QuickAutoSexLinkSibmateIndiv(QuickAutoSexLinkSibmateIndiv* indiv);
virtual		 		~QuickAutoSexLinkSibmateIndiv(void);
						//NOTE that Duplicate() functions don't copy family info - use Copy() to store that info
virtual QuickAutoSexLinkSibmateIndiv* 	Duplicate(int* newGenesA, int* newGenesX, const int ploidyX, 
							long id, Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual QuickAutoSexLinkSibmateIndiv* 	Duplicate(int* newGenesA, int* newGenesX, const int ploidyX, 
							long double sexratio, long double sibmatingprob, long id, Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual QuickAutoSexLinkSibmateIndiv* 	Duplicate(int* /*newGenesA*/, long /*id*/, Population* /*site=NULL*/);//returns error message
								
virtual Individual* 		Duplicate(long id, Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual Individual* 		Duplicate(void);//VIRTUAL COPY CONSTRUCTOR
virtual long			GetPointerToDerivedSelf(void){
								return (long) this;}

virtual void			Clone (long& idCount, Ran3List& ran, aList<long double>& specialInfo);
virtual void			Clone (Individual* mom, Ran3List& ran, aList<long double>& specialInfo);
void					Copy (Individual* it);//copies only base class; leaves genetic component uncopied
virtual void			Copy (QuickDiplSexGenIndividual* it);
virtual void			Copy (QuickAutosomeSexChromIndividual* it);
virtual void 			Copy (QuickAutoSexLinkSibmateIndiv* it);
virtual void 			IIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran, LandScapeInitializer01& init, aList<long double>& extraInfo);
								//extraInfo holds pop & indiv number of this individual, for getting the genotype info
								//parents are NULL pointers
virtual void 			MakeABaby (long& idCount, Ran3List& ran, aList<long double>& specialInfo);
								//assumes baby has the same moveRate_ as the calling parent, probably female
virtual void			RemoveDad(void){dad_=NULL;}//forgets identity -- must delete elsewhere
virtual void			RemoveMom(void){mom_=NULL;}//forgets identity -- must delete elsewhere
virtual void			RemoveParent(QuickAutoSexLinkSibmateIndiv* parent);//forgets identity -- must delete elsewhere
virtual void			RemoveOffspring(QuickAutoSexLinkSibmateIndiv* child);//forgets identity -- must delete elsewhere
virtual void			RemoveSibF(QuickAutoSexLinkSibmateIndiv* sib);//forgets identity -- must delete elsewhere
virtual void			RemoveSibM(QuickAutoSexLinkSibmateIndiv* sib);//forgets identity -- must delete elsewhere
virtual void			SetMom(QuickAutoSexLinkSibmateIndiv* mom){mom_ = mom;}
virtual void			SetDad(QuickAutoSexLinkSibmateIndiv* dad){dad_ = dad;}
virtual void			SetOffspring(QuickAutoSexLinkSibmateIndiv* child);
virtual void			SetSib(QuickAutoSexLinkSibmateIndiv* sib);
virtual int				sibs(void){return sibsMale()+sibsFemale();}//returns # of sibs
virtual int				sibsMale(void){return sibsM_.len();}
virtual int				sibsFemale(void){return sibsF_.len();}//returns true if femalesibs are available

Individual& 			operator=(QuickAutoSexLinkSibmateIndiv& indiv){
								QuickAutoSexLinkSibmateIndiv::Copy(&indiv);
								return *this;}
	
	};		//end QuickAutoSexLinkSibmateIndiv


#endif
	
