/***************************************************
*																														}
*	QuickAutosomeSexChromIndividual.h

	Base Class: QuickDiplSexGenIndividual, Individual																			}
*																														}
*	Handles processes involving individuals with autosomal and X-linked genetic information.
*	Genotype is an array of short integers, stored separately for sex-linked genes.
*	Sexual reproduction, but mate identity not stored; must mate at 
*	site of reproduction.  Loci autosomal or X-linked, but assumed to be unlinked.
*	Note:  assumes male is XO.
*																														}
****************************************************/

#ifndef QuickAutosomeSexChromIndividualh
#define QuickAutosomeSexChromIndividualh = 1

#include "aError.h"
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "Population.h"
#include <iostream.h>
#include "SelectionIntf.h"
#include "StringClass.h"
#include "RandomList.h"
#include "BetaDistribution.h"
#include "MemoryManager.h"
#include "Individual.h"
#include "GeneContainers.h"
#include "QuickDiplSexGenIndividual.h"

// 2090612 debugged errors in setting moveRate_ during copying
// 2090612 debugged error in Copy

//#ifndef DEBUG_REPRODUCTION
//#define DEBUG_REPRODUCTION = 1
//#endif

class QuickAutosomeSexChromIndividual: virtual public QuickDiplSexGenIndividual, virtual public Individual{
	public:
		int nlociX_;//count loci autosomally first, then X-linked
	protected:
		int*	genesX_;	
	public:
						QuickAutosomeSexChromIndividual(const int nlociA, const int nlociX, const int ploidyX, int* newGenesA, 
									int* newGenesX, long idVal=-1, Population* popSite=NULL);
						QuickAutosomeSexChromIndividual(long id,Population* site=NULL);
						QuickAutosomeSexChromIndividual(void);
						QuickAutosomeSexChromIndividual(QuickAutosomeSexChromIndividual& indiv);
		 				QuickAutosomeSexChromIndividual(QuickAutosomeSexChromIndividual* indiv);
virtual		 		~QuickAutosomeSexChromIndividual(void);
virtual QuickAutosomeSexChromIndividual* 	Duplicate(int* newGenesA, int* newGenesX, const int ploidyX, long id, Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual QuickDiplSexGenIndividual* 	Duplicate(int* /*newGenesA*/, long /*id*/, Population* /*site=NULL*/){//VIRTUAL COPY CONSTRUCTOR
								aError("ERROR: can\'t use the construction QuickAutosomeSexChromIndividual:Duplicate(int*, long,Population*)");
								return NEW QuickAutosomeSexChromIndividual(*this);}
virtual Individual* 		Duplicate(long id, Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual Individual* 		Duplicate(void);//VIRTUAL COPY CONSTRUCTOR
virtual long			GetPointerToDerivedSelf(void){
								return (long) this;}

virtual int				AllelePresent (int loc, aString& alleleVal);
//virtual void			Clone (long& /*idCount*/){
//							aError("ERROR: Shouldn't arrive here.");
//							aError(" Call QuickDiplSexGenIndividual::Clone(long&,Ran3&,double,double) instead.");}
//virtual void			Clone (Individual*){
//							aError("ERROR: Shouldn't arrive here.");
//							aError(" Call QuickDiplSexGenIndividual::Clone(Individual*,Ran3&,double,double) instead.");}
//virtual void			Clone (long& idCount, Ran3& ran, double mutationRate, double sigma);
//virtual void			Clone (Individual* mom, Ran3& ran, double mutationRate, double sigma);
virtual void			Clone (long& idCount, Ran3List& ran, aList<long double>& specialInfo);
virtual void			Clone (Individual* mom, Ran3List& ran, aList<long double>& specialInfo);
void					Copy (Individual* it);//copies only base class; leaves genetic component uncopied
virtual void			Copy (QuickDiplSexGenIndividual* it);
virtual void			Copy (QuickAutosomeSexChromIndividual* it);
virtual int				CountAlleles (int loc, aString& alleleVal);
//virtual double 			fitness(void);
virtual genderSet		gameteType(void){return B;}
virtual void 			GetGenome(GeneContainerCore& gc);
							//refills container class as a MultiInheritanceModeContainer<int>[mode][ploidy][locus]
virtual int*			GetGenotype (void);//returns a copy that must be disposed of elsewhere
virtual int				GetGenotype (int locus, int allele){
							return QuickDiplSexGenIndividual::GetGenotype(locus, allele);}
virtual int				GetGenotypeX (int locX, int allele=0){
							int ploidyX=2;
							if(sex_==M) --ploidyX;
							return genesX_[locX*ploidyX+allele];
							}
virtual aString*		GenotypeString(char* separator = " ");
virtual aString*		GenotypeString(int locus, char* separator = " ");
virtual aString*		GenotypeString(int locus, int alleleNum, char* separator = " ");
virtual void 			IIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran, LandScapeInitializer01& init, aList<long double>& extraInfo);
								//extraInfo holds pop & indiv number of this individual, for getting the genotype info
virtual int				IsHeterozygote (int loc);//returns false=0 if haploid
virtual int				IsHeterozygote (int loc, aString& alleleVal); //returns false=0 if haploid
virtual int 			loci(void);
virtual void 			MakeABaby (long& idCount, Ran3List& ran, aList<long double>& specialInfo);
								//assumes baby has the same moveRate_ as the calling parent, probably female

//virtual void 			MakeABaby (long& idCount, Ran3List& ran, double mutationRate, double sigma);
//virtual void 			MakeABaby (long& /*idCount*/, Ran3List& /*ran*/){
//							aError("ERROR: Shouldn't arrive here.");
//							aError(" Call QuickDiplSexGenIndividual::MakeABaby(long&,Ran3List&,double,double) instead.");}
virtual int				mates(void);//returns true if potential mates are available
//virtual void			Mutate (Ran3& ran, double mutationRate, double sigma=0.0);
virtual void			Mutate (Ran3& ran, aList<long double>& specialInfo);
							//specialInfo[0] = mutationRate
							//specialInfo[1] = 2-allele model (0), infinite-allele model (1)
							//specialInfo[2] = infinite allele counter (increment for each new mutation)
virtual int 			ploidy(int loc);
virtual long double			phenotype(int i=0);//returns value of locus as a dominant: 00->0, 01->1, 11->1
//virtual void 			PrintGenes(ostream& outFile, int /*alleleLen=4*/, int codeLen=6);
virtual void 			Print(ostream& outFile, char sepChar='\t');
virtual genderSet 		RealizedSexOfClassMember(long double sexRatio, Ran3& ran){
							return (ran()<sexRatio)? F:M;}
						//returns any one of the possible sexes of this class
//virtual void 			Reproduce (long& idCount, Ran3List& ran, double mutationRate, double sigma);
//virtual void 			Reproduce (long& /*idCount*/, Ran3List& /*ran*/){
//							aError("ERROR: Shouldn't arrive here.");
//							aError(" Call GeneticIndividual::Reproduce(long&,Ran3List&,double,double) instead.");}
virtual void 			SetGenome(GeneContainerCore& gc);
							//initializes genome using MultiInheritanceModeContainer<int>[mode][ploidy][locus]
virtual void 			SetGenotypeX(int loc, int allele, int genotype);
virtual void 			SetXLinkedGenotypeFemale(int *genesX, int nlociX){//genes must be the same size array or hell breaks loose!
									if(nlociX!=nlociX_){
										aError("ERROR: setting a female genotype of the wrong size in QuickAutosomeSexChromIndividual");}
									if(sex_ != F){
											aError("Setting sex-linked female genotype in a non-female-gendered individual");}
										else{
											int loc;
											for(loc=0;loc<nlociX_;++loc){
												genesX_[loc*2] = genesX[loc*2];
												genesX_[loc*2+1] = genesX[loc*2+1];
												}}
									}
virtual void 			SetXLinkedGenotypeMale(int *genesX, int nlociX){//genes must be the same size array or hell breaks loose!
									if(nlociX!=nlociX_){
										aError("ERROR: setting a male genotype of the wrong size in QuickAutosomeSexChromIndividual");}
									if(sex_ != M){
											aError("Setting sex-linked female genotype in a non-female-gendered individual");}
										else{
											int loc;
											for(loc=0;loc<nlociX_;++loc){
												genesX_[loc] = genesX[loc];
												}}
									}
virtual void 			SetSexLinkedGenome(GeneContainerCore& gc);
							//sex-linked genes only
							//initializes genome using MultiInheritanceModeContainer<int>[mode][ploidy][locus]


	};		//end QuickAutosomeSexChromIndividual


#endif
	
