/***************************************************
*																														}
*	QuickHaplSexGenIndividual_LD.h

	Base Class: Individual																					}
*																														}
*	Handles processes involving individuals with genetic information.
*	Genotype is an array of long doubles.
*	Sexual reproduction, but mate identity not stored; must mate at 
*	site of reproduction.  Loci autosomal, unlinked.
*																														}
****************************************************/

#ifndef QuickHaplSexGenIndividual_LDh
#define QuickHaplSexGenIndividual_LDh = 1

#include "aError.h"
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "Individual.h"
#include "Population.h"
#include <iostream.h>
#include "SelectionIntf.h"
#include "StringClass.h"
#include "RandomList.h"
#include "GeneContainers.h"
#include "InlineTemplates.h"
#include "MemoryManager.h"

class QuickHaplSexGenIndividual_LD;

typedef void (*PhenotypeFunc)(QuickHaplSexGenIndividual_LD& , aList<long double>& specialInfo);
	// Calculates the phenotype

void PhenotypeOneToOne(aList<long double>& specialInfo);
	//one gene = one trait

class QuickHaplSexGenIndividual_LD: virtual public Individual{
	public:
		int nloci_;
		int ntraits_;
		bool phenotypeSet_;
		PhenotypeFunc SetTraits;
		long double*	genes_;	
		long double*	traits_;
		aList<long double> constants_;	//use to calculate phenotypes
	public:
						QuickHaplSexGenIndividual_LD(const int nloci, long double* newGenes, 
													 const int ntraits, long double* newTraits, aList<long double>& constants, 
													 PhenotypeFunc thePhenotypeFunction, long id=-1, 
													 Population* site=NULL);
						QuickHaplSexGenIndividual_LD(aList<long double>& constants, long id,Population* site=NULL);
						QuickHaplSexGenIndividual_LD(void);
						QuickHaplSexGenIndividual_LD(QuickHaplSexGenIndividual_LD& indiv);
		 				QuickHaplSexGenIndividual_LD(QuickHaplSexGenIndividual_LD* indiv);
virtual		 		~QuickHaplSexGenIndividual_LD(void);
virtual QuickHaplSexGenIndividual_LD* 	Duplicate(long double* newGenes, long double* newTraits, long id, Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual Individual* 		Duplicate(long id, Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual Individual* 		Duplicate(void);//VIRTUAL COPY CONSTRUCTOR
virtual long			GetPointerToDerivedSelf(void){
								return (long) this;}

void 					Initialize(const int nloci, long double* newGenes, const int ntraits, long double* newTraits, aList<long double>& constants,
											PhenotypeFunc thePhenotypeFunction, long id, Population* site);
void 					Initialize(const int nloci, long double* newGenes, const int ntraits, long double* newTraits, aList<long double>& constants,
											PhenotypeFunc thePhenotypeFunction);

virtual int				AllelePresent (int loc, aString& alleleVal);
virtual void			Clone (long& idCount, Ran3List& ran, aList<long double>& specialInfo);
virtual void			Clone (Individual* mom, Ran3List& ran, aList<long double>& specialInfo);
void					Copy (Individual* it);//copies only base class; leaves genetic component uncopied
virtual void			Copy (QuickHaplSexGenIndividual_LD* it);
virtual int				CountAlleles (int loc, aString& alleleVal);
virtual genderSet 		gameteType(void){return B;}
virtual aString*		GenotypeString(char* separator = " ");
virtual aString*		GenotypeString(int locus, char* separator = " ");
virtual aString*		GenotypeString(int locus, int alleleNum, char* separator = " ");
virtual void 			GetGenome(GeneContainerCore& gc);
							//refills container class as a HaplotypeContainer<int>[locus]
virtual long double*	GetGenotype (void);//returns a copy that must be disposed of elsewhere
virtual long double		GetGenotype (int locus, int allele);//returns a copy that must be disposed of elsewhere
virtual void 			IIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran, LandScapeInitializer01& init, aList<long double>& extraInfo);
virtual int 			loci(void);
virtual void 			MakeABaby (long& idCount, Ran3List& ran, aList<long double>& specialInfo);
virtual void			Mutate (Ran3& ran, aList<long double>& specialInfo);
virtual int 			ploidy(int /*loc*/){return 1;}
virtual int 			ploidy(void);
virtual long double		phenotype(int i=0);//returns value of locus 
virtual aString*		PhenotypeString(char* separator = " ");
virtual aString*		PhenotypeString(int trait, char* separator = " ");
virtual aString*		PhenotypeString(int trait, int alleleNum, char* separator = " ");
virtual void 			PrintGenes(ostream& outFile, int /*alleleLen=4*/, int codeLen=6);
virtual void 			PrintPhenotypes(ostream& outFile, int /*alleleLen=4*/, int codeLen=6);
virtual genderSet 		RealizedSexOfClassMember(long double /*sexRatio*/, Ran3&){
							return B;}
virtual void 			SetGenome(GeneContainerCore& gc);
							//initializes genome using MultiHaplotypeContainer<int>[ploidy][locus]
virtual void			SetPhenotype(void);							
void 					SetPhenotypeFunction (PhenotypeFunc thePhenotypeFunction){
							( *SetTraits ) = thePhenotypeFunction;}
							
	};		//end QuickHaplSexGenIndividual_LD


#endif
	
