/***************************************************
*																														}
*	SexChromIndividual.h
*
*	Superclass:  DioeciousGeneticIndividual
*
*	Handles processes involving individuals with genetic sex determination
*	Genotypes have sex chromosomes and are held in a container class, SexChromGenome,
	which is a derived class of Phenotype & CoreGenome
*
****************************************************/

#ifndef SexChromIndividualh
#define SexChromIndividualh = 1

#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "CoreGenome.h"
#include "DioeciousGeneticIndividual.h"


class SexChromIndividual: public DioeciousGeneticIndividual{
	public:

		 				SexChromIndividual(Phenotype& newGenes, long id=0, Population* site=NULL):
		 								DioeciousGeneticIndividual(newGenes,id,site) {}

						SexChromIndividual(long id,Population* site=NULL):
										DioeciousGeneticIndividual(id,site) {}

						SexChromIndividual(void):
										DioeciousGeneticIndividual() {}

						SexChromIndividual(SexChromIndividual& indiv){
							Copy(&indiv);}//creates, then copies

		 				SexChromIndividual(SexChromIndividual* indiv){
							Copy(indiv);}//creates, then copies

virtual 	 			~SexChromIndividual(void){}

virtual GeneticIndividual* 	Duplicate(Phenotype& newGenes, long id, Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual Individual* 		Duplicate(long id,Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR


//THESE OVERRIDE FUNCTIONS IN THE BASE CLASS SexualIndividual

virtual void			Clone (long& idCount);

void					Copy (Individual* it);
void					Copy (GeneticIndividual* it);
void					Copy (SexualGeneticIndividual* it);
void					Copy (DioeciousGeneticIndividual* it);
virtual void			Copy (SexChromIndividual* it);

virtual void			MakeABaby (long& idCount, Ran3List& ran, double mutationRate, double mutationStdev);

virtual genderSet 		RealizedSexOfClassMember(double sexRatio, Ran3& ran){
							return (ran.next()<sexRatio) ? F:M;}
						//returns any one of the possible sexes of this class
virtual void			Reproduce (long& idCount, Ran3List& ran, double mutationRate, double mutationStdev);

virtual void			SetGender(void);


	
	};		//end SexChromIndividual









/*
//SEND TO DERIVED Individual-BASED SEXUAL CLASSES WITH SPECIAL MATING SYSTEMS
void CPopulation::AssignMates (Ran3List& ran){
	CIndividual she = he = NULL;
	long num, i;
	for (i=0,i<females.len,i++){
		she = (CIndividual) females.NthItem(i);
		switch (matingSystem) {
			case clonal: 
				break;	//only arrive here when comparing mating systems
			case selfing: 
				she.SetMate(she);
				break;	
			case idealHermaphrodite: 
				if ((!she.mated) && (females.len + males.len > 0)) {
					num = (long)(RandomProb(idum) * (females.len + males.len)) + 1;
					if (num > females.len) {
							he = (CIndividual) males.NthItem(num - females.len);}
						else{he = (CIndividual)females.NthItem(num);}
					if (!he) {cout<<"problem choosing a mate"<<endl;}
						else {she.SetMate(he);}
						
					}// end if females+males
				break;	//end case idealHermaphrodite

			case strictMonogamy: 
			case femaleMonogamy: 
				if ((!she.mated) && (males.len > 0)) {
					num = (long)(RandomProb(idum) * males.len) + 1;
					he = (CIndividual) males.NthItem(num);
					if (!he) {cout<< "problem choosing a male mate"<<endl;}
						else if (!he.mated) {
								she.SetMate(he);
								he.SetMate(she);} //useful for data collection on mating freqs later
						}	// end if males
				break;	//end case strictMonogamy


			case maleMonogamy: 
			case polygamyWMixing: 
				if (males.len > 0) {
					num = (long)(RandomProb(idum) * males.len) + 1;
					he = (CIndividual) males.NthItem(num);
					if (!he) {cout<<"problem choosing a male mate"<<endl;}
						else if (!he.mated) {
							she.SetMate(he);
							he.SetMate(she); //useful for data collection on mating freqs later
						}	//if !he.mated
					}	//end if males
				break;	//end case maleMonogamy

			case polygamyWLastPrecedence: 
				if (males.len > 0) {
					num = (long)(RandomProb(idum) * males.len) + 1;
					he = (CIndividual) males.NthItem(num);
					if (!he) {cout<<"problem choosing a male mate"<<endl;}
						else if (!he.mated) {
							she.mates.DisposeList;	//previous sperm are removed
							she.SetMate(he);
							he.SetMate(she); //useful for data collection on mating freqs later
						}	//if !he.mated
					}	//end if males
				break;	//end case polygamyWLastPrecedence
			}; //end switch matingSystem

	}	//end if nFemales>0 etc. else
	SetReproductiveStatus();
	}	//end CPopulation::AssignMates
*/



#endif

