/***************************************************
*																														}
*	SexualGeneticIndividual.h
*
*	Superclass:  GeneticIndividual
*
*	Handles processes involving individuals within populations
*	genotypes of mates are held in a container class
*
****************************************************/

#ifndef SexGenIndividualh
#define SexGenIndividualh = 1

#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "StatList.cpp"
#include "GeneticIndividual.h"
#include "Individual.h"
#include "Population.h"
#include <iostream.h>
#include "CoreGenome.h"
#include "MemoryManager.h"


class SexualGeneticIndividual: public GeneticIndividual{
	protected:
		int 				matings_;
		int				matingsThisAgeClass_;// for limits on mating rate
		ObjList<CoreGenome>	mates_;	//carries a genotype list of each mate
	public:
		 				SexualGeneticIndividual(Phenotype& newGenes, long id=0, Population* site=NULL);

						SexualGeneticIndividual(long id,Population* site=NULL);

						SexualGeneticIndividual(void);

						SexualGeneticIndividual(SexualGeneticIndividual& indiv){
							Copy(&indiv);}//creates then copies

		 				SexualGeneticIndividual(SexualGeneticIndividual* indiv){
							Copy(indiv);}//creates then copies

virtual 	 			~SexualGeneticIndividual(void);

virtual GeneticIndividual* 	Duplicate(Phenotype& newGenes, long id, Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual Individual* 		Duplicate(long id,Population* site=NULL);//VIRTUAL COPY CONSTRUCTOR
virtual Individual* 		Duplicate(void);//VIRTUAL COPY CONSTRUCTOR


virtual void			Birthday (int maxage);

virtual int				CanMate (int maxThisAgeClass);

virtual void			ClearMates (void){
							mates_.Clear();}

//virtual void			Clone (long& id, Ran3& ran, double mutationRate, double sigma);//use for vegetative reproduction by sexual individual
virtual void			Clone (long& id, Ran3List& ran, aList<long double>& specialInfo);//use for vegetative reproduction by sexual individual

//virtual void			Clone (Individual* mom, Ran3& ran, double mutationRate, double sigma);
virtual void			Clone (Individual* mom, Ran3List& ran, aList<long double>& specialInfo);

void					Copy (Individual* it);//assumes 'it' is SexualGeneticIndividual
void					Copy (GeneticIndividual* it);//assumes 'it' is SexualGeneticIndividual
virtual void			Copy (SexualGeneticIndividual* it);

virtual void			FindMate (Ran3List& ran, aList<long double>& reproductionInfo, int tries, int maxThisAgeClass);

virtual CoreGenome&		GetMate (Ran3List& /*ran*/);	//returns a genotype list of a mate
							//depends on mating system
							//default here is complete last-male sperm precedence

virtual CoreGenome&		GetMate (int mateNum){	//returns a genotype list of a mate
							return mates_[mateNum];}

virtual ObjList<CoreGenome>&	GetMates (void) {//current list of mates
							return mates_;}

virtual void			IIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran);

//virtual void			IIndividual (genderSet gender, lifeStageType lifeStage, 
//								long double rate, Ran3List& ran, LandScapeInitializer00& init);

virtual void			IIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran, LandScapeInitializer01& init, aList<long double>& extraInfo);

//virtual void			MakeABaby (long& idCount, Ran3List& ran, double mutationRate, double sigma);
virtual void			MakeABaby (long& idCount, Ran3List& ran, aList<long double>& specialInfo);

virtual int				mates (void) {//currently listed mates
							return mates_.len();}

virtual int				matings(void){return matings_;}

virtual int				matingsThisAgeClass(void){return matingsThisAgeClass_;}

virtual genderSet 		RealizedSexOfClassMember(long double /*sexRatio*/, Ran3&){
							return B;}
						//returns any one of the possible sexes of this class
//virtual void			Reproduce (long& idCount, Ran3List& ran, double mutationRate, double sigma);
virtual void			Reproduce (long& idCount, Ran3List& ran, aList<long double>& specialInfo);

virtual void			SetMate (Individual* sweetie, int );
							//depends on mating system
							//default is complete sperm precedence
							//derive for longer lists of mates
virtual StringList* 	stateNames(void);//column headings: ="id, age, sex, etc."
virtual StringList* 	states(void);// indiv values for stateNames() above

	
	};		//end SexualGeneticIndividual

#endif

