/******************************************
*
*	DioeciousGeneticLandScape.h
*
*	Base class:  SexualGeneticLandScape
*
*   This header defines the derived LandScape class
*	that forms the basis for dioecious individuals with genetic information.
*	It seeds the landscape with dioecious genetic individuals,
*	and presumes constant habitat quality.  Derived
*	classes add features and/or take data.
******************************************/


#ifndef DioeciousGeneticLandScapeh
#define DioeciousGeneticLandScapeh = 1

#include "RandomList.h"
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "StatList.h"
#include "GridCell.h"
#include "DioeciousGeneticPopulation.h"
#include "DioeciousGeneticIndividual.h"
#include "GeneticGridCell.h"
#include "SexualGeneticLandScape.h"
#include "CoreGenome.h"


class DioeciousGeneticLandScape: public SexualGeneticLandScape {
	
  public:/*DATA*/
  	aList<double> intrinsicSexRatio_;/*one for each species*/

	public:	
	DioeciousGeneticLandScape(LandScapeInitializer01& init);
					/*general purpose initializer*/

	DioeciousGeneticLandScape(int len, /*ONE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 double fSurvivalRate, /*CONSTANT THROUGH LIFE*/
  					 double fFecundityRate, /*CONSTANT THROUGH LIFE*/
  					 double femaleMovementRate,
  					 double maleMovementRate,
  					 double intrinSexRatio,
  					 ObjList<Phenotype>& initGenes,
					 aList<double>& initGenotypeFreqs
					 ):
		SexualGeneticLandScape(len,w,migrationModel,initSize,maxCapacity,maxage, 
					 fSurvivalRate,fFecundityRate,femaleMovementRate,
					 initGenes,initGenotypeFreqs){
	intrinsicSexRatio_.Append(intrinSexRatio);
	maleMovementRate_.Clear();
	ObjList<aList<double> >* habitatList=new ObjList<aList<double> >();
	maleMovementRate_.Append(habitatList);
	aList<double>* ageList=new aList<double>();
	habitatList->Append(ageList);
	for (int i=0;i<maxage_[0];++i){ageList->Append(maleMovementRate);}
	}/*end constructor DioeciousGeneticLandScape*/

	DioeciousGeneticLandScape(int len, /*ONE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 aList<double>* fSurvivalSchedule, /*VARIABLE SURVIVAL*/
  					 aList<double>* fFecunditySchedule, /*VARIABLE FECUNDITY*/
 					 ObjList<aList<double> >* femaleMovementRate,
 					 ObjList<aList<double> >* maleMovementRate,
  					 double intrinSexRatio,
  					 ObjList<Phenotype>& initGenes,
					 aList<double>& initGenotypeFreqs
					 ):
		SexualGeneticLandScape(len,w,migrationModel,initSize,maxCapacity,maxage, 
					 fSurvivalSchedule,fFecunditySchedule,femaleMovementRate,
					 initGenes,initGenotypeFreqs){
	intrinsicSexRatio_.Append(intrinSexRatio);
	maleMovementRate_.Clear();
	ObjList<aList<double> >* habitatList=new ObjList<aList<double> >(maleMovementRate);
	maleMovementRate_.Append(habitatList);
	}/*end constructor DioeciousGeneticLandScape*/

	DioeciousGeneticLandScape(int len, /*ONE OR MORE SPECIES, 1 HABITAT*/
					 int w, 	
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<double> >* fSurvivalSchedule,
  					 ObjList<aList<double> >* fFecunditySchedule,
 					 ObjList<ObjList<aList<double> > >* femaleMovementRate,
 					 ObjList<ObjList<aList<double> > >* maleMovementRate,
  					 aList<double>& intrinSexRatio,
  					 ObjList<ObjList<Phenotype> >& initGenes,
					 ObjList<aList<double> >& initGenotypeFreqs
					 ):
	SexualGeneticLandScape(len,w,migrationModel,species,initSize,maxCapacity,maxage, 
					 fSurvivalSchedule,fFecunditySchedule,femaleMovementRate,
					 initGenes,initGenotypeFreqs),
  	intrinsicSexRatio_(intrinSexRatio){
  	maleMovementRate_.Copy(maleMovementRate);
  	}/*end constructor DioeciousGeneticLandScape*/

	DioeciousGeneticLandScape(int len, /*ONE OR MORE SPECIES  1 HABITAT*/
					 int w, 	
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<double> >* fSurvivalSchedule,
  					 ObjList<aList<double> >* fFecunditySchedule,
 					 ObjList<ObjList<aList<double> > >* femaleMovementRate,
 					 ObjList<ObjList<aList<double> > >* maleMovementRate,
  					 aList<double>& intrinSexRatio,
  					 ObjList<ObjList<ObjList<Phenotype> > >& initGenes,
					 ObjList<ObjList<aList<double> > >& initGenotypeFreqs
					 ):
	SexualGeneticLandScape(len,w,migrationModel,species,initSize,maxCapacity,maxage, 
					 fSurvivalSchedule,fFecunditySchedule,femaleMovementRate,
					 initGenes,initGenotypeFreqs),
  	intrinsicSexRatio_(intrinSexRatio){
  	maleMovementRate_.Copy(maleMovementRate);
  	}/*end constructor DioeciousGeneticLandScape*/



virtual ~DioeciousGeneticLandScape(void){}

/*	THESE ARE FUNCTIONS OVERRIDDEN FROM THE BASE GeneticLandScape CLASS*/
virtual void DoOneCycle (int species, long gen, Ran3List& ran, double mutationRate, double sigma);
virtual double	intrinsicSexRatio(int species){
				return intrinsicSexRatio_[species];}
virtual void ILandScape(Ran3List& ran);
virtual void ILandScapeByHabitat (Ran3List& ran, 
					ObjList<aList<int> >& habitatGrid,
					ObjList<ObjList<ObjList<Phenotype> > >& genesByFreqByHabitatBySpecies,
					ObjList<ObjList<aList<double> > >& freqsByHabitatBySpecies);
virtual void SeedLandScape (int species, long initSize, Ran3List& ran);
virtual void SeedLandScapeByHabitat (int species, long initSize, Ran3List& ran, 
					ObjList<ObjList<ObjList<Phenotype> > >& genesByFreqByHabitatBySpecies,
					ObjList<ObjList<aList<double> > >& freqsByHabitatBySpecies);

	};/*end DioeciousGeneticLandScape*/


#endif

