/******************************************
*
*	DioeciousLandScape
*
*   This header defines the derived LandScape class
*	that forms the basis for individuals that must mate with opposite sex individuals to reproduce.
	No genetics are included.
*	It seeds the landscape with dioecious individuals,
*	and presumes constant habitat quality.  Derived
*	classes add features and/or take data.
******************************************/


#ifndef DioeciousLandScapeh
#define DioeciousLandScapeh = 1

#include <iostream.h>
#include "RandomList.h"
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "StatList.h"
#include "GridCell.h"
#include "SexualLandScape.h"
#include "DioeciousPopulation.h"
#include "DioeciousIndividual.h"
#include "MemoryManager.h"


class DioeciousLandScape: public SexualLandScape {
  public:
  	aList<double> intrinsicSexRatio_;//one for each species*/
	

  public:
	// CONSTRUCTORS*/
	DioeciousLandScape(LandScapeInitializer01& init):/*general purpose initializer*/
			SexualLandScape(init),
			intrinsicSexRatio_(init.intrinSexRatio_)
			{}
	DioeciousLandScape(int len, //ONE SPECIES*/
					 int w, 	//default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 double fSurvivalRate, //CONSTANT THROUGH LIFE*/
  					 double fFecundityRate, //CONSTANT THROUGH LIFE*/
 					 double femaleMovementRate,//constant across habitat & age class*/
 					 double maleMovementRate,/*constant across habitat & age class*/
 					// selectionSourceSet sSource,
  					 double intrinSexRatio=0.5
					 ):
			SexualLandScape(len,w,migrationModel,initSize,maxCapacity, 
					  maxage,fSurvivalRate,fFecundityRate,femaleMovementRate//,sSource
					  ){
		intrinsicSexRatio_.Append(intrinSexRatio);
	maleMovementRate_.Clear();
	ObjList<aList<double> >* habitatList=NEW ObjList<aList<double> >();
	maleMovementRate_.Append(habitatList);
	aList<double>* ageList=NEW aList<double>();
	habitatList->Append(ageList);
	for (int i=0;i<maxage_[0];++i){ageList->Append(maleMovementRate);}
	}/*end constructor DioeciousLandScape*/
	
	DioeciousLandScape(int len, /*ONE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 aList<double>* fSurvivalSchedule, /*VARIABLE SURVIVAL*/
  					 aList<double>* fFecunditySchedule, /*VARIABLE FECUNDITY*/
 					 ObjList<aList<double> >* femaleMovementRate,/*by habitat & age class*/
 					 ObjList<aList<double> >* maleMovementRate,/*by habitat & age class*/
 					// selectionSourceSet sSource,
  					 double intrinSexRatio=0.5
 					 ):
			SexualLandScape(len,w,migrationModel,initSize,maxCapacity, 
					  maxage,fSurvivalSchedule,fFecunditySchedule,femaleMovementRate//,sSource
					  ) {
	intrinsicSexRatio_.Append(intrinSexRatio);
	maleMovementRate_.Clear();
	ObjList<aList<double> >* habitatList=new ObjList<aList<double> >(maleMovementRate);
	maleMovementRate_.Append(habitatList);
	}/*end constructor DioeciousLandScape*/
	
	DioeciousLandScape(int len, /*ONE OR MORE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<double> >* fSurvivalSchedule,
  					 ObjList<aList<double> >* fFecunditySchedule,
 					 ObjList<ObjList<aList<double> > >* femaleMovementRate,
 					 ObjList<ObjList<aList<double> > >* maleMovementRate,
 					// selectionSourceSet sSource,
  					 aList<double>& intrinSexRatio
					 ):
			SexualLandScape(len,w,migrationModel,species,initSize,maxCapacity, 
					  maxage,fSurvivalSchedule,fFecunditySchedule,femaleMovementRate//,sSource
					  ),
			intrinsicSexRatio_(intrinSexRatio) {
//				maleMovementRate_.Copy(femaleMovementRate);
				maleMovementRate_.Copy(maleMovementRate);
				}



virtual ~DioeciousLandScape(void){}



/*	THESE ARE FUNCTIONS OVERRIDDEN FROM THE BASE LANDSCAPE CLASS*/
virtual void 		ILandScape(Ran3List& ran, int seeded = true);
virtual void 		ILandScape(LandScapeInitializer01& init, Ran3List& ran, int seeded = true);
virtual double	intrinsicSexRatio(int species){
						return intrinsicSexRatio_[species];}
virtual void 		SeedLandScape (int species, long initSize, Ran3List& ran);
virtual void 		SeedLandScape (LandScapeInitializer01& init, int species, long initSize, Ran3List& ran);


	};/*end DioeciousLandScape*/


#endif
