/******************************************
*
*	GeneticLandScape
*
*   This header defines the derived LandScape class
*	that forms the basis for individuas with genetic information.
*	It seeds the landscape with base-class genetic individuals,
*	and presumes constant habitat quality.  Derived
*	classes add features and/or take data.
******************************************/


#ifndef GeneticLandScapeh
#define GeneticLandScapeh = 1

#include <iostream.h>
#include "RandomList.h"
#include "Element.h"
#include "aList.h"
#include "StatList.h"
#include "GridCell.h"
#include "GeneticPopulation.h"
#include "GeneticIndividual.h"
#include "GeneticGridCell.h"
#include "LandScape.h"
#include "LandScapeInitializer01.h"
#include "Phenotype.h"
#include "GeneContainers.h"
#include "InlineTemplates.h"
#include "MemoryManager.h"


class GeneticLandScape: public LandScape {

  public:
  	/* DATA FOR GENETIC ANALYSES, ONE SET FOR EACH SPECIES*/
  	ObjList<ObjList<StringList > >		allelesInScape_;/*allele values stored as strings to permit allele complexity*/
	aList<int>						allelesCurrent_;/*flags*/
	ObjList<StatList<int> >			lociPerChromosome_;/*i=species, j=chromosome; val[i,j]=loci*/
	ObjList<ObjList<aList<long double> > > 	allFreq_;
	aList<int>						allFreqCurrent_;/*flags*/
	ObjList<StatList<long double> > 		hetObsLoc_;
	ObjList<StatList<long double> >			hetExpLoc_;
	StatList<long double>				hetObs_;
	StatList<long double>				hetExp_;
	aList<int>						hetExpDataCurrent_;/*flags*/
	aList<int>						hetObsDataCurrent_;/*flags*/
//	T			dummy1;
	
	/*LISTS OF 'GLOBAL' CONSTANTS FOR EACH SPECIES*/
	ObjList<ObjList<ObjList<Phenotype > > > seededGenotypes_;
			/*initial genotypes by species,habitat-->NOTE THAT THIS IS GENOTYPES*/
	ObjList<ObjList<aList<long double> > > seededGenotypeFreqs_;
			/*initial frequencies by habitat -->NOTE THAT THIS IS GENOTYPE FREQUENCIES*/



  public: /*still*/
	/* CONSTRUCTORS*/
//	GeneticLandScape(LandScapeInitializer00& init);/*general purpose initializer*/
	GeneticLandScape(LandScapeInitializer01& init);/*general purpose initializer*/

	GeneticLandScape(int len, /*ONE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 long double fSurvivalRate, /*CONSTANT THROUGH LIFE*/
  					 long double fFecundityRate, /*CONSTANT THROUGH LIFE*/
 					 long double femaleMovementRate,/*constant across habitats, sexes & age classes*/
// 					 selectionSourceSet sSource,
  					 ObjList<Phenotype >& initGenes,
					 aList<long double>& initGenotypeFreqs
					 );
	GeneticLandScape(int len, /*ONE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 aList<long double>* fSurvivalSchedule, /*VARIABLE SURVIVAL*/
  					 aList<long double>* fFecunditySchedule, /*VARIABLE FECUNDITY*/
 					 ObjList<aList<long double> >* femaleMovementRate,
 //					 selectionSourceSet sSource,
 					 ObjList<Phenotype >& initGenes,
					 aList<long double>& initGenotypeFreqs
					 );
	GeneticLandScape(int len, /*ONE OR MORE SPECIES, 1 HABITAT*/
					 int w, 	
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<long double> >* fSurvivalSchedule,
  					 ObjList<aList<long double> >* fFecunditySchedule,
 					 ObjList<ObjList<aList<long double> > >* femaleMovementRate,
// 					 selectionSourceSet sSource,
 					 ObjList<ObjList<Phenotype > >& initGenes,
					 ObjList<aList<long double> >& initGenotypeFreqs
					 );
	GeneticLandScape(int len, /*ONE OR MORE SPECIES, >1 HABITAT*/
					 int w, 
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<long double> >* fSurvivalSchedule,
  					 ObjList<aList<long double> >* fFecunditySchedule,
 					 ObjList<ObjList<aList<long double> > >* femaleMovementRate,
// 					 selectionSourceSet sSource,
 					 ObjList<ObjList<ObjList<Phenotype > > >& initGenes,
					 ObjList<ObjList<aList<long double> > >& initGenotypeFreqs
					 );



virtual ~GeneticLandScape(void);

/*	THESE ARE FUNCTIONS OVERRIDDEN FROM THE BASE LANDSCAPE CLASS*/
virtual void AgeOneStep(int species);

//virtual void DoOneCycle (long /*gen*/, Ran3List& /*ran*/){
//				cout<<"ERROR:  Shouldn't arrive here."<<endl;
//				cout<<"Call GeneticLandScape::DoOneCycle(long,Ran3List&,double,double,double) instead."<<endl;}
//virtual void DoOneCycle (long /*gen*/, Ran3List& /*ran*/, double , double ){
//				cout<<"ERROR:  Shouldn't arrive here."<<endl;
//				cout<<"Call GeneticLandScape::DoOneCycle(long,Ran3List&,double,double,double) instead."<<endl;}
//virtual void DoOneCycle (long /*gen*/, Ran3List& ran, double m, double mutationRate, double sigma);
//virtual void DoOneCycle (long gen, Ran3List& ran, double m, double mutationRate, double sigma, double /*m*/, double /*kurtosisFactor*/){
//				 GeneticLandScape::DoOneCycle (gen, ran, m, mutationRate, sigma);}
//virtual void DoOneCycle (int species, long gen, Ran3List& ran, double m, double mutationRate, double sigma, double /* m*/, 
//						   double /*kurtosisFactor*/){
//				 GeneticLandScape::DoOneCycle (species, gen, ran, m, mutationRate, sigma);}
//virtual void DoOneCycle (int species, long /*gen*/, Ran3List& ran, double m, double mutationRate, double sigma);
//virtual void DoOneCycle (int /*species*/, long /*gen*/, Ran3List& /*ran*/){
//				cout<<"ERROR:  Shouldn't arrive here."<<endl;
//				cout<<"Call GeneticLandScape::DoOneCycle(int,long,Ran3List&,double,double) instead."<<endl;}
//virtual void DoOneCycle (int /*species*/, long /*gen*/, Ran3List& /*ran*/,double,double){
//				cout<<"ERROR:  Shouldn't arrive here."<<endl;
//				cout<<"Call GeneticLandScape::DoOneCycle(int,long,Ran3List&,double,double,double) instead."<<endl;}


//virtual void DoOneCycle (long /*gen*/, Ran3List& ran, ObjList<aList<long double> >& dispersalInfo,
//							 ObjList<aList<long double> >& reproductionInfo);
//virtual void DoOneCycle (int species, long /*gen*/, Ran3List& ran, ObjList<aList<long double> >& dispersalInfo,
//							 ObjList<aList<long double> >& reproductionInfo);


virtual void GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList, int species);
virtual void GetGenome(PopulationGeneContainer<int>& pgc, int species, int popID);
virtual void GetGenome(MultiInheritanceModeContainer<int>& mimc, int species, int popID, long indiv);
virtual void ILandScape (Ran3List& ran, int seeded);
//virtual void ILandScape (LandScapeInitializer00& init, Ran3List& ran);
virtual void ILandScape (LandScapeInitializer01& init, Ran3List& ran, int seeded);
//virtual void ILandScapeByHabitat (Ran3List& ran, LandScapeInitializer00& init);
virtual void ILandScapeByHabitat (Ran3List& ran, LandScapeInitializer01& init, int seeded);
virtual void ILandScapeByHabitat (Ran3List& ran, 
					ObjList<aList<int> >& habitatGrid, 
					ObjList<ObjList<ObjList<Phenotype > > >& genesByFreqByHabitatBySpecies,
					ObjList<ObjList<aList<long double> > >& freqsByHabitatBySpecies);
virtual void ILandScapeByHabitat (Ran3List& /*ran*/, ObjList<aList<int> >& /*habitatGrid*/, int /*seeded*/){
				cout<<"ERROR:  Shouldn't arrive here."<<endl;
				cout<<"Call GeneticLandScape::ILandScapeByHabitat(Ran3List&,ObjList<aList<int>>&,ObjList<ObjList<ObjList<Phenotype>>>&,ObjList<ObjList<aList<double> > >&) instead."<<endl;}
//virtual void NaturalSelection (int species, Ran3List& ran);
virtual void Reproduce (int species, Ran3List& ran, aList<long double>& specialInfo);
				//specialInfo[0]=mutationRate, specialInfo[1]= sigma
virtual void Reproduce (int /*species*/, Ran3List& /*ran*/){
				cout<<"ERROR:  Shouldn't arrive here."<<endl;
				cout<<"Call GeneticLandScape::Reproduce(int,Ran3List&,double,double) instead."<<endl;}
virtual void SeedLandScape (int species, long initSize, Ran3List& ran);
//virtual void SeedLandScape (LandScapeInitializer00& init, int species, long initSize, Ran3List& ran);
virtual void SeedLandScape (LandScapeInitializer01& init, int species, long initSize, Ran3List& ran);
virtual void SeedLandScapeByHabitat (int species, long initSize, Ran3List& ran, 
					ObjList<ObjList<ObjList<Phenotype > > >& genesByFreqByHabitatBySpecies,
					ObjList<ObjList<aList<long double> > >& freqsByHabitatBySpecies);
//virtual void SeedLandScapeByHabitat (LandScapeInitializer00& init, int species, long initSize, Ran3List& ran);
virtual void SeedLandScapeByHabitat (LandScapeInitializer01& init, int species, long initSize, Ran3List& ran);
virtual void SetLociByChromosome(LandScapeInitializer01& init);
virtual void SomeAdultsDie (int species, Ran3List& ran);



	/* THESE ARE FUNCTIONS FOR HANDLING GENETIC SUMMARY INFORMATION*/
	
//virtual int			AllelePosition (int species, int loc, T alleleVal);
						//position in allelesInScape_
virtual int			AllelePosition (int species, int loc, aString& alleleVal);
						//position in allelesInScape_
virtual int			AllelePresent (int species, int loc, aString& alleleVal);//boolean
virtual aString&	AlleleAtLoc (int species, int loc, int alleleNum);
						//looks in allelesInScape_ & returns allele's Value
virtual int			AllelesAtLoc (int species, int loc);//returns number of alleles
virtual void		AssignMutations(int /*species*/);//derive this
virtual void		CalcAlleleFrequencies(int species);
virtual void		CoordsOfNthLocus(int species, int nthLoc, int& onChromNum, int& locNumOnChrom);
//virtual long		CountAlleles (int species, int loc, T alleleVal);
//virtual long		CountHets (int species, int loc, T alleleVal);
virtual long		CountAlleles (int species, int loc, aString& alleleVal);
virtual long		CountHets (int species, int loc, aString& alleleVal);
virtual long		CountHets (int species, int loc);
virtual aString*	GetPhenotypeString(int species, int pop, long indiv, char* separator = " ");
//virtual int 		inheritanceType(int species, int locus);
						//returns value matching inheritanceTypes in MultiInheritanceModeContainer<T> (GeneContainers.h)
virtual int			loci(int species);
virtual int			NthLocus(int species, int chrom, int locOnChrom);/*returns 0 indexed value*/
virtual int			ploidy(int species, int locus);
virtual void		PrintAlleleFreqs(ostream&, int /*species*/);
virtual void		PrintAlleleFreqs(ostream&, int /*species*/, int /*locus*/);
//virtual void		PrintAlleleFreqs(ostream& outFile, int species, int locus, T alleleVal);
virtual void		PrintAlleleFreqs(ostream& outFile, int species, int locus, aString& alleleVal);
virtual void		PrintAlleleList(ostream&, int /*species*/);
virtual void		PrintAlleleList(ostream&, int /*species*/, int /*locus*/);
virtual void		PrintGeneStrings(ostream& outFile, int species, int alleleLen=4, int codeLen=5);
virtual void		PrintHetExp(ostream&, int /*species*/);
virtual void		PrintHetExp(ostream&, int /*species*/, int /*loc*/);
virtual void		PrintHetObs(ostream&, int /*species*/);
virtual void		PrintHetObs(ostream&, int /*species*/, int /*loc*/);
virtual long		SampleSize (int species, int loc);/*handles ploidy differences*/
virtual ObjList<Phenotype >&	/*genotypes seeded into the habitat, by species*/
				SeededGeneList(int species, int habitat);
virtual aList<long double>&	/*freqs of genotypes seeded into the habitat, by species*/
				SeededGeneFreqList(int species, int habitat);
virtual int			traits(int species);
virtual void		UpdateAlleleList(void);/*makes a List of all alleles in the LandScape*/
virtual void		UpdateAlleleList(int species);/*makes a List of all alleles in the LandScape*/
	
  
	

	};/*end GeneticLandScape*/


#endif
