/******************************************
*
*	LandScape.h
*
*   This header defines the base LandScape class
*	It seeds the landscape with base-class individuals,
*	and presumes constant habitat quality.  Derived
*	classes add features and/or take data.
******************************************/


#ifndef LandScapeh
#define LandScapeh = 1

#include <iostream.h>
#include <fstream.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"
#include "RandomList.h"
#include "Element.h"
#include "aList.h"
#include "CoordinateClass.h"
#include "Randomness.h"
#include "GridCell.h"
#include "PopulationIntf.h"
#include "SelectionIntf.h"
#include "Population.h"
#include "Individual.h"
#include "LandScapeInitializer00.h"
#include "LandScapeInitializer01.h"
#include "InlineTemplates.h"


class GelJockLandScapeInterface;

class LandScape {
	friend class GelJockLandScapeInterface;
	friend class FstatLandScapeInterface;
   protected:
  	unsigned int 					len_;	/*ORDER DEPENDENCY*/
  	unsigned int 					width_;/*ORDER DEPENDENCY*/
	ObjList<ObjList<GridCell> > 		grid_;
	int 							species_;/*number on landscape*/
	long double						gridScale_;/*distance between grid cells*/
	migrationModelSet				migrationModel_;/*island, steppingStone, diffusion, kurtotic*/
//	selectionSourceSet				selectionSource_;/* {	intrinsic, extrinsic, 
//				    									IntrinsicAndExtrinsic,
//				    									frequencyDependence  }*/
	
	public:
	long totalIndivsCreated_;/*assigns id numbers to individuals*/
	protected:
	long totalPopsCreated_;/*assigns unique numbers to populations*/

	//time units equal here, but may differ in derived classes
	long globalTime_;//time since beginning of simulation
	long localTime_;//time that may cycle such as day/yr or secs/day

	/*LISTS OF 'GLOBAL' CONSTANTS FOR EACH SPECIES*/
	aList<long> initSize_;/*initial number of individuals on grid*/
	aList<int> maxage_;/*maximum age of individuals on grid*/
	aList<long> maxCapacity_;/*carryingCapacity of pops on grid*/

  	ObjList<aList<long double> > fSurvivalSchedule_;
  	ObjList<aList<long double> > mSurvivalSchedule_;/*just a dummy in this base class*/
  	ObjList<aList<long double> > fFecunditySchedule_;
 	
 	ObjList<ObjList<aList<long double> > > femaleMovementRate_;/*by species, habitat, and age class*/
 	ObjList<ObjList<aList<long double> > > maleMovementRate_;/*by species, habitat, and age class*/

 	ObjList<ObjList<ObjList<aList<long double> > > > migrationMatrix_;
 		/*probability of moving from one habitat type to another
 		by species, age class, originating habitat, and target habitat*/


  public:
	/*CONSTRUCTORS*/
//	LandScape(LandScapeInitializer00& init);/*general purpose initializer*/
	LandScape(LandScapeInitializer01& init);/*general purpose initializer*/
	LandScape(int len, /*ONE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 long double fSurvivalRate, /*CONSTANT THROUGH LIFE*/
  					 long double fFecundityRate, /*CONSTANT THROUGH LIFE*/
  					 long double femaleMovementRate
					 );
	LandScape(int len, /*ONE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 aList<long double>* fSurvivalSchedule, /*VARIABLE SURVIVAL*/
  					 aList<long double>* fFecunditySchedule, /*VARIABLE FECUNDITY*/
 					 ObjList<aList<long double> >* femaleMovementRate
					 );
	LandScape(int len, /*ONE OR MORE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<long double> >* fSurvivalSchedule,
  					 ObjList<aList<long double> >* fFecunditySchedule,
 					 ObjList<ObjList<aList<long double> > >* femaleMovementRate
					 		/*by species, habitat, and age class*/
  					 );



virtual ~LandScape(void);

/*	THESE ARE FUNCTIONS USED ON THE LANDSCAPE*/
virtual void 		AdultDispersal (int species, Ran3List& ran, aList<long double>& dispersalInfo);
//virtual void 		AdultDispersal (int species, Ran3List& ran, double m, 
//							   double kurtosisFactor=0.0);
//virtual void 		AdultDispersal (int species, Ran3List& ran);
virtual long 		adults(int species);
virtual void 		AgeOneStep(int species);
virtual int 		area(void){return len_ * width_;}
virtual long 		babies(int species);
virtual void 		BuryTheDead(int species);
virtual long 		capacity(int species);
virtual void 		ClearOffspringLists(int species);
virtual long 		dead(int species);
//virtual void 		Dispersal (int species, Ran3List& ran, double m, 
//							   double kurtosisFactor=0.0);
virtual void 		Dispersal (int species, Ran3List& ran, aList<long double>& dispersalInfo);
virtual void 		DoOneCycle (aList<int>& cycleSequence, Ran3List& ran, ObjList<aList<long double> >& dispersalInfo,
							 ObjList<aList<long double> >& reproductionInfo);
virtual void 		DoOneCycle (aList<int>& cycleSequence, int species, Ran3List& ran, ObjList<aList<long double> >& dispersalInfo,
							 ObjList<aList<long double> >& reproductionInfo);
virtual void 		DropNeighbors(GridCell* g1, GridCell* g2);
virtual long		extantPops(int species);
virtual int  		extinct(int species);
virtual void 		ExtractFromGrid (int species, CoordinateType loc, Individual* indiv);
virtual long double	Fecundity(int species, int age);
virtual void 		GameteDispersal (int species, Ran3List& ran, aList<long double>& dispersalInfo);
virtual void 		GetAgeDistribution (int species, aList<int>& ageDist);
virtual void 		GetCoordinates (int popID, CoordinateType& coords);
virtual void 		GetCoordinates (int species, int popID, CoordinateType& coords);
virtual CoordinateType 
				GetNewLocCoordinates (CoordinateType oldLoc, 
											  long xdev, long ydev);
virtual GridCell& 	GetGridCell (CoordinateType loc);
virtual Population& 
					GetPopulation (int species, CoordinateType loc);
		long& 		globalTime(void){return globalTime_;}
virtual long double gridScale(void){return gridScale_;}
virtual int			habitat(CoordinateType loc);
virtual long		habitats(void);/*number of habitats in landscape*/
virtual ObjList<aList<int> >*
					habitatGrid(void);//returns grid of habitat scores
virtual void 		ILandScape (Ran3List& ran, int seeded = true);
//virtual void 		ILandScape (LandScapeInitializer00& init, Ran3List& ran);
virtual void 		ILandScape (LandScapeInitializer01& init, Ran3List& ran, int seeded = true);
//virtual void 		ILandScapeByHabitat (Ran3List& ran, LandScapeInitializer00& init);
virtual void 		ILandScapeByHabitat (Ran3List& ran, LandScapeInitializer01& init, int seeded = true);
virtual void 		ILandScapeByHabitat (Ran3List& ran, ObjList<aList<int> >& habitatGrid, int seeded = true);
//virtual void 		ILandScapeByHabitat (LandScapeInitializer00& init, Ran3List& ran, ObjList<aList<int> >& habitatGrid);
//virtual void 		ILandScapeByHabitat (LandScapeInitializer01& init, Ran3List& ran, ObjList<aList<int> >& habitatGrid);
virtual void 		ImmatureDispersal (int species, Ran3List& ran, aList<long double>& dispersalInfo);
virtual long 		immigrants(int species);//number of new migrants arriving
virtual long double	intrinsicSexRatio(int){return one;}/*all females in default*/
virtual void 		InstallInGrid (int species, Individual* indiv, CoordinateType loc);
virtual int 		length(void){return len_;}
		long& 		localTime(void){return localTime_;}
virtual void 		MateOnGrid (int species, Ran3List& ran, aList<long double>& reproductionInfo);
virtual migrationModelSet 
					migrationModel(void) {return migrationModel_;}
virtual long double MigrationProb(int species, int ageClass, int currentHabitat, int potentialHabitat);
virtual long double	moveRate(int species, int habitat, genderSet sex, int age);
//virtual void 		NaturalSelection(int species, Ran3List& ran);
virtual ObjList<aList<int> >*
					PopIDGrid(int species);//returns grid of pop IDs
virtual void 		PrintAgeDistribution (ostream& outFile, int species, aString* msg = NULL);
virtual void 		PrintGridConnections(ostream& outFile, aString* msg = NULL);
virtual void 		PrintGridOccupancy (ostream& outFile, int species, aString* msg = NULL);
virtual void 		Recruit (int species, Ran3List& ran);//babies->adults
virtual void 		ReplaceTheDead (int species, Ran3List& ran);
virtual void 		ReplaceTheDeadUsingLandscapeWideCarryingCapacity (int species, Ran3List& ran);
							//Use for continuous dispersal models where landscape represents deme area 
							//and the grid describes location within a deme
//virtual void 		Reproduce (int species, Ran3List& ran);
//virtual void 		Reproduce (int /*species*/, Ran3List& /*ran*/,double,double){}//derived genetic classes
virtual void 		Reproduce (int species, Ran3List& ran, aList<long double>& reproductionInfo);
							//derived classes add info as needed
virtual void 		ReproduceBySyngamy (int species, Ran3List& ran, aList<long double>& reproductionInfo);
							//derived classes add info as needed
//virtual selectionSourceSet
//				selectionSource(void){return selectionSource_;}
virtual void 		SeedLandScape (int species, long initSize, Ran3List& ran);
virtual void 		SeedLandScape (int species, Individual* indiv, Ran3List& ran);
//virtual void 		SeedLandScape (LandScapeInitializer00& init, int species, long initSize, Ran3List& ran);
virtual void 		SeedLandScape (LandScapeInitializer01& init, int species, long initSize, Ran3List& ran);
virtual void 		SetCapacity (long N, int species);
virtual void 		SetDispersalMatrix(ObjList<ObjList<aList<long double> > >& dispersalMatrix);
virtual void 		SetGridNeighbors(void);/*does all except across edges; 4 directions*/
virtual void 		SetHabitatTypes(void);/*all default habitat type = 0*/
virtual void 		SetHabitatTypes(ObjList<aList<int> >& habitatGrid);
virtual void 		SetHabitatTypes(aString& dataFileName);/*reads data file*/
virtual void 		SetHexGridNeighbors(void);/*does all except across edges; 6 directions*/
virtual void 		SetIslandGridNeighbors(void);/*for island model, all cells connected*/
virtual void 		SetLoopHabitat(void);/*edges link in only the longer of 2 directions*/
virtual void 		SetLoopHabitatLongway(void);/*edges link in only the shorter of 2 directions*/
virtual void 		SetNeighbors(GridCell& g1, GridCell& g2);
virtual void 		SetTorusHabitat(void);/*edges link in all 4 directions*/
virtual void 		SetHexTorusHabitat(void);/*edges link in all 6 directions*/
virtual void 		SomeAdultsDie (int species, Ran3List& ran);
virtual void 		SomeImmaturesDie (int species, Ran3List& ran);
virtual void 		SomeGametesDie (int species, Ran3List& ran);
virtual int 		species(void){return species_;}
virtual void 		TimeIncrement(void){++globalTime_; ++localTime_;}
virtual long 		time(void){return localTime();}
virtual void		ViabilitySelection( Ran3& ran);
virtual void		ViabilitySelection(int species, Ran3& ran);
virtual int 		width(void){return width_;}

/*	void CalculateExpectedDispersal(void);//derive this for dispersal class*/
virtual void 		CreateMigrationMatrix(void);
virtual void 		CreateMigrationMatrix(LandScapeInitializer00& init);
virtual void 		CreateMigrationMatrix(LandScapeInitializer01& init);
virtual void 		CreateMigrationMatrix(ObjList<ObjList<ObjList<aList<long double> > > >& migrationMatrix);
virtual void 		CreateDataMatrices(int){}
virtual void 		StartupInitializations(int){}



	};/*end LandScape*/

#endif
