/******************************************
*
*	LandScapeInitializer00.h
*
*   This header holds all the variables that may be used to
*	initialize a LandScape-based simulation.  Derived classes
*	may be created to add to this list, to initialize classes newly derived
*	from LandScape simulation bases.  Use sequential numbers to
*	represent derived classes.
*
******************************************/


#ifndef LandScapeInitializer00h
#define LandScapeInitializer00h = 1

class Individual;
class Population;
class GridCell;

#include <iostream.h>
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "PopulationIntf.h"
#include "GridCell.h"
#include "Population.h"
#include "Individual.h"
#include "Genome.h"
#include "MemoryManager.h"


class LandScapeInitializer00 {
   public:
  	unsigned int 					len_;
  	unsigned int 					width_;
	unsigned int 					species_;/*number on landscape*/
	unsigned int 					habitats_;/*number of habitat types*/
	long double						gridScale_;/*distance between grid cells*/
	migrationModelSet				migrationModel_;/*island, steppingStone, diffusion, kurtotic*/
//	selectionSourceSet				selectionSource_;/* {	intrinsic, extrinsic, 
//				    									IntrinsicAndExtrinsic,
//				    									frequencyDependence  }*/
	
	/*LISTS OF 'GLOBAL' CONSTANTS FOR EACH SPECIES*/
	aList<long> initSize_;/*by species, initial number of individuals on grid*/
	aList<int> maxage_;/*by species, maximum age of individuals on grid*/
	aList<long> maxCapacity_;/*by species, carryingCapacity of pops on grid*/
	
	ObjList<aList<int> > habitatGrid_;//habitat1 by habitat2	

  	ObjList<aList<long double> > fSurvivalSchedule_;/*by species, age class*/
  	ObjList<aList<long double> > mSurvivalSchedule_;/*by species, age class*/
  	ObjList<aList<long double> > fFecunditySchedule_;/*by species, age class*/
 	
 	ObjList<ObjList<aList<long double> > > femaleMovementRate_;/*by species, habitat, and age class*/
 	ObjList<ObjList<aList<long double> > > maleMovementRate_;/*by species, habitat, and age class*/

 	ObjList<ObjList<ObjList<aList<long double> > > > migrationMatrix_;
 		/*probability of moving from one habitat type to another
 		by species, age class, originating habitat, and target habitat*/
 	long double kurtosisFactor_;//non-normal dispersal

  	aList<long double>  intrinSexRatio_;//by species, {ratio is female/male}
  	aList<long double>  assortMatingProb_;//by species, prob of mating with other species, phenotype, whatever

 	ObjList<ObjList<ObjList<Genome<int> > > > initGenes_;//species, habitat, list of genomes
  	ObjList<ObjList<aList<long double> > > initGenotypeFreqs_;//species, habitat, list of frequencies
  	Individual* indivOfCorrectType;
  	Population* popOfCorrectType;
  	GridCell* gridcellOfCorrectType;


  public:
	/*CONSTRUCTORS*/
	LandScapeInitializer00(void):	//default settings
		len_(1),
		width_(1),
		species_(1),
		habitats_(1),
		gridScale_(1.0),
		migrationModel_(steppingStone),
		kurtosisFactor_(0.0),
//		selectionSource_(intrinsic),
		indivOfCorrectType(NULL),
		popOfCorrectType(NULL),	
		gridcellOfCorrectType(NULL)		
		{}

virtual ~LandScapeInitializer00(void);

	//functions to return stored values
	unsigned int length(void){return len_;}
	unsigned int width(void){return width_;}
	unsigned int species(void){return species_;}
	unsigned int  habitats(void){return habitats_;}
	long double gridScale(void){return gridScale_;}
	migrationModelSet migrationModel(void){return migrationModel_;}
							/*island, steppingStone, diffusion, kurtotic*/
//	selectionSourceSet selectionSource(void){return selectionSource_;}
				/* {	intrinsic, extrinsic, IntrinsicAndExtrinsic, frequencyDependence  }*/
	long double kurtosisFactor(void){return kurtosisFactor_;}
	long double sexRatio(int species){return intrinSexRatio_[species];}
	long double AssortativeMatingProb(int species){return assortMatingProb_[species];}



	// functions to add or change initializer variables
	void length(unsigned int x){len_ = x;}
	void width(unsigned int x){width_ = x;}
	void species(unsigned int s){species_ = s;}
	void habitats(unsigned int h){habitats_ = h;}
	void gridScale(long double d){gridScale_ = d;}
	void migrationModel(migrationModelSet m){migrationModel_ = m;}
							/*island, steppingStone, diffusion, kurtotic*/
//	void selectionSource(selectionSourceSet s){selectionSource_ = s;}
				/* {	intrinsic, extrinsic, IntrinsicAndExtrinsic, frequencyDependence  }*/
	void kurtosisFactor(long double k){kurtosisFactor_ = k;}
	void initSize(long n);//all species the same
	void initSize(long n, int species);
	void maxage(int a);//all species the same
	void maxage(int a, int species);
	void maxCapacity(long n);//all species the same
	void maxCapacity(long n, int species);
	void sexRatio(long double x);//all species the same
	void sexRatio(long double x, int species);

	//set species_ and maxage_ first		
	void femaleFecundity(long double b);//all species & ages the same
	void femaleFecundity(long double b, int species);//all ages the same
	void femaleFecundity(long double b, int species, int ageClass);
	void femaleSurvival(long double m);//all species & ages the same
	void femaleSurvival(long double m, int species);//all ages the same
	void femaleSurvival(long double m, int species, int ageClass);
	void maleSurvival(long double m);//all species & ages the same
	void maleSurvival(long double m, int species);//all ages the same
	void maleSurvival(long double m, int species, int ageClass);
		//sets both sexes
	void survival(long double m);//all species & ages the same
	void survival(long double m, int species);//all ages the same
	void survival(long double m, int species, int ageClass);
 	void femaleMovementRate(long double m);//all species, habitats & ages the same
	void femaleMovementRate(long double m, int species);//all habitats & ages the same
	void femaleMovementRate(long double m, int species, int ageClass);//all habitats the same
	void femaleMovementRate(long double m, int species, int ageClass, int habitat);
	void maleMovementRate(long double m);//all species, habitats & ages the same
	void maleMovementRate(long double m, int species);//all habitats & ages the same
	void maleMovementRate(long double m, int species, int ageClass);//all habitats the same
	void maleMovementRate(long double m, int species, int ageClass, int habitat);
		//sets both sexes
	void movementRate(long double m);//all species, habitats & ages the same
	void movementRate(long double m, int species);//all habitats & ages the same
	void movementRate(long double m, int species, int ageClass);//all habitats the same
	void movementRate(long double m, int species, int ageClass, int habitat);
 		/*by species, age class, originating habitat, and target habitat*/
	void migrationProb(long double m);//all species, ages & habitats the same
	void migrationProb(long double m, int species);//all ages & habitats the same
	void migrationProb(long double m, int species, int ageClass);//all ages the same
	void migrationProb(long double m, int species, int fromHabitat, int toHabitat);//all ages the same
	void migrationProb(long double m, int species, int ageClass, int fromHabitat, int toHabitat);
	
	void setHabitat(int habitatValue, int row, int column);
	void SetIndividual(Individual* indiv);
	void SetGridCell(GridCell* g);
	void SetPopulation(Population* pop);
	
	void AppendGenes(Genome<int>& genes, long double freq, int species);//all habitats the same
	void AppendGenes(Genome<int>& genes, long double freq, int species, int habitat);
	
	void AssortativeMatingProb(long double prob);//all species the same
	void AssortativeMatingProb(long double prob, int species);

	};//end LandScapeInitializer00

#endif
