/******************************************
*
*	SexualGeneticLandScape
*
*   This header defines the derived LandScape class
*	that forms the basis for individuas with genetic information.
*	It seeds the landscape with base-class genetic individuals,
*	and presumes constant habitat quality.  Derived
*	classes add features and/or take data.
******************************************/


#ifndef SexGenLandScapeh
#define SexGenLandScapeh = 1

#include <iostream.h>
#include "RandomList.h"
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "StatList.h"
#include "GridCell.h"
#include "GeneticPopulation.h"
#include "SexualGeneticPopulation.h"
#include "SexualGeneticIndividual.h"
#include "GeneticGridCell.h"
#include "GeneticLandScape.h"
#include "LandScapeInitializer01.h"
//#include "CoreGenome.h"
#include "MemoryManager.h"


class SexualGeneticLandScape: public GeneticLandScape {
  public:
	/* CONSTRUCTORS*/
//	SexualGeneticLandScape(LandScapeInitializer00& init);
					/*general purpose initializer*/
	SexualGeneticLandScape(LandScapeInitializer01& init);
					/*general purpose initializer*/
	SexualGeneticLandScape(int len, /*ONE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 long double fSurvivalRate, /*CONSTANT THROUGH LIFE*/
  					 long double fFecundityRate, /*CONSTANT THROUGH LIFE*/
   					 long double femaleMovementRate,
//   					 selectionSourceSet sSource,
  					 ObjList<Phenotype>& initGenes,
					 aList<long double>& initGenotypeFreqs
					 );
	SexualGeneticLandScape(int len, /*ONE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 aList<long double>* fSurvivalSchedule, /*VARIABLE SURVIVAL*/
  					 aList<long double>* fFecunditySchedule, /*VARIABLE FECUNDITY*/
 					 ObjList<aList<long double> >* femaleMovementRate,
// 					 selectionSourceSet sSource,
  					 ObjList<Phenotype>& initGenes,
					 aList<long double>& initGenotypeFreqs
					 );
	SexualGeneticLandScape(int len, /*ONE OR MORE SPECIES, 1 HABITAT*/
					 int w, 	
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<long double> >* fSurvivalSchedule,
  					 ObjList<aList<long double> >* fFecunditySchedule,
 					 ObjList<ObjList<aList<long double> > >* femaleMovementRate,
// 					 selectionSourceSet sSource,
   					 ObjList<ObjList<Phenotype> >& initGenes,
					 ObjList<aList<long double> >& initGenotypeFreqs
					 );
	SexualGeneticLandScape(int len, /*ONE OR MORE SPECIES, >1 HABITAT*/
					 int w, 	
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<long double> >* fSurvivalSchedule,
  					 ObjList<aList<long double> >* fFecunditySchedule,
 					 ObjList<ObjList<aList<long double> > >* femaleMovementRate,
// 					 selectionSourceSet sSource,
   					 ObjList<ObjList<ObjList<Phenotype > > >& initGenes,
					 ObjList<ObjList<aList<long double> > >& initGenotypeFreqs
					 );



virtual ~SexualGeneticLandScape(void);

/*	THESE ARE FUNCTIONS OVERRIDDEN FROM THE BASE GeneticLandScape CLASS*/
//virtual void DoOneCycle (long /*gen*/, Ran3List& ran, double m, double mutationRate, double sigma);
//virtual void DoOneCycle (int species, long /*gen*/, Ran3List& ran, double m, double mutationRate, double sigma);
//virtual void DoOneCycle (long /*gen*/, Ran3List& ran, ObjList<aList<long double> >& dispersalInfo,
//							 ObjList<aList<long double> >& reproductionInfo);
//virtual void DoOneCycle (int species, long /*gen*/, Ran3List& ran, ObjList<aList<long double> >& dispersalInfo,
//							 ObjList<aList<long double> >& reproductionInfo);
virtual long double	intrinsicSexRatio(int /*species*/){return half;}
virtual void ILandScape(Ran3List& ran, int seeded);
//virtual void ILandScape (LandScapeInitializer00& init, Ran3List& ran);
virtual void ILandScape (LandScapeInitializer01& init, Ran3List& ran, int seeded);
//virtual void ILandScapeByHabitat (Ran3List& ran, LandScapeInitializer00& init);
virtual void ILandScapeByHabitat (Ran3List& ran, 
					LandScapeInitializer01& init, int seeded);
virtual void ILandScapeByHabitat (Ran3List& ran, ObjList<aList<int> >& habitatGrid,
					ObjList<ObjList<ObjList<Phenotype > > >& genesByFreqByHabitatBySpecies,
					ObjList<ObjList<aList<long double> > >& freqsByHabitatBySpecies);
virtual void SeedLandScape (int species, long initSize, Ran3List& ran);
//virtual void SeedLandScape (LandScapeInitializer00& init, int species, long initSize, Ran3List& ran);
virtual void SeedLandScape (LandScapeInitializer01& init, int species, long initSize, Ran3List& ran);
virtual void SeedLandScapeByHabitat (int species, long initSize, Ran3List& ran, 
					ObjList<ObjList<ObjList<Phenotype > > >& genesByFreqByHabitatBySpecies,
					ObjList<ObjList<aList<long double> > >& freqsByHabitatBySpecies);
//virtual void SeedLandScapeByHabitat (LandScapeInitializer00& init, int species, 
//					long initSize, Ran3List& ran);
virtual void SeedLandScapeByHabitat (LandScapeInitializer01& init, int species, 
					long initSize, Ran3List& ran);


	};/*end SexualGeneticLandScape*/



#endif
