/*******************************************
*
*
*	aMatrix.h
*
*	A class that implements matrices, and does matrix algebra
*
*	Matrices can be converted from  ObjList<aList<T>>
*
*
************************************************/

#ifndef aMatrixh
#define aMatrixh = 1

#include <iostream.h>
#include <fstream.h>
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "InlineTemplates.h"
#include "MemoryManager.h"
#include "StringClass.h"
#include "aVector.h"
#include "aError.h"


template<class T> class matrixHandle;

template <class T> class aMatrix{
	public://ORDER DEPENDENCY; DON'T SWITCH THESE
		unsigned long rows_, cols_;
		ObjList<aVector<T> >  data;
		int nullity_;
	public:
		aMatrix(T x,   unsigned long r=1, unsigned long c=1);
		aMatrix(unsigned long r=0, unsigned long c=1);
		aMatrix(aMatrix<T>& m);
		aMatrix(aMatrix<T>* m);
		virtual ~aMatrix(void);
		virtual void BadOperation(aString err/*, ostream& outFile=cout*/){
						aError(err);
//						outFile<<err<<endl;
						}

	virtual aMatrix<T>* 
				Duplicate(void){
					return NEW aMatrix<T>(*this);}
			
			
	aMatrix<T>* 	Add (aMatrix<T>& m);
					//adds m to this and returns result in a new matrix
					//Sizes must match or NULL pointer returned
	void 			CalcNullity(void);
	int 			CanBeAdded(aMatrix<T>& m){
					return Size(m.rows(),m.cols());}
	int 			CanBeMultiplied(aMatrix<T>& m){
					return cols()==m.rows();}
	int 			CanBeMultiplied(aVector<T>& v){
					return cols()==v.len();}
	void 			cdiv(long double xr, long double  xi,//called in some eigenvalue calcs 
						 long double yr, long double  yi,//modified from JAMA library, in jama_eig.h
						 long double cdivr, long double cdivi);
	void 			Chop(void);//rounds to zero any number less than 10^-6 times the maximum value
	void 			Clear(void){
					rows_=cols_=0;
					nullity_=-1;
					data.Clear();}
	unsigned long	cols(void){return cols_;}
	int 			Compatible(aVector<T>& v){//can be added, multiplied, etc.
					if(cols()==v.len()){return 1;}else{return 0;}}
	aMatrix<T>* 	Concat (aMatrix<T>& m);
					//appends m to this and returns result in a new matrix
					//rows_ must match or NULL pointer returned
	void 	 		ConcatCol (aVector<T>& v);
					//appends column v to this 
					//rows_ must match len_ orerror returned
	void 	 		ConcatRow (aVector<T>& v);
					//appends row v to this 
					//cols_ must match len_ or error returned
	aVector<T>* 	Column (long col);//returns the column in a new vector
	void 			Copy(aMatrix<T>& m);//copies the contents of m into 'this'
	void 			Copy(aMatrix<T>* m){Copy(*m);}
	aMatrix<T>* 	DeleteColOfCopy (long col);//deletes col and returns result in a new matrix
	void 			DeleteCol (long col);//deletes column
	aMatrix<T>*	DeleteRowOfCopy (long row);//deletes row and returns result in a new matrix
	void 			DeleteRow (long row);//deletes row
	long double 	Determinant (void);// can easily overflow
	int 			EigenCalcs(aVector<long double>& eigenValues, //returns eigenvalues
							   aMatrix<long double>& eigenVectorCols);//returns corresponding eigenvectors in columns
							   //returns 0 if successful; error code if not
	int 			Empty(void){
					if((rows_==0)&&(cols_==0)){return 1;}else{return 0;}}
	int 			GetCofactorSign (long num);
					//recursive; calculates (-1)^(num)
					//used in calculating determinants
	aMatrix<T>* 	HorizontalDirectProduct(aMatrix<T>& m);//returns direct product of row vectors
	int 			hqr2(aVector<long double>& diagonalElements,//modified from JAMA library, in jama_eig.h
						 aVector<long double>& imaginaryElements,//called in some eigenvalue calcs
						 aMatrix<long double>& V,
						 aMatrix<long double>& H);
	void 	 		InsertColumn (aVector<T>& v, int colNum);
					//inserts copy of column v into this 
					//rows_ must match len_ or error returned
	void 	 		InsertRow (aVector<T>& v, int rowNum);
					//inserts copy of row v into this 
					//cols_ must match len_ or error returned
	aMatrix<long double>* 	
				Inverse (void){
						//using minors method; long computing time; result in new matrix
//						return InverseByMinors();
						return SVInverse();
						}
	aMatrix<long double>* 	
				InverseByMinors (void);
						//using minors method; long computing time; result in new matrix
	void 			LUBackSubstitution (aVector<long double>& b, aVector<long>& indx, int sign);
					//debug first;  overwrites the original matrix
	void 			LUDecomposition (aVector<long>& indx, int& sign);
					//debug first;  overwrites the original matrix
	aMatrix<T>* 	LUInverse (void);	//debug first;  returns result in a new matrix
	aMatrix<long double>* 	
				MatrixPlusForm(void);//returns [Inverse(X'.X)].X'; NULL if inverse can't be found
	aMatrix<long double>* 	
				MatrixPlusForm(aMatrix<long double>& invXprimeX);//returns [Inverse(X'.X)].X'; NULL if inverse can't be found
	T 			Max(void);//highest value
	T 			Min(void);//lowest value
	aMatrix<T>* 	Minor (long  row, long col);//returns result in a new matrix
	aMatrix<long double>* 	
				Multiply(aMatrix<T>& m);//premultiplies m to this and returns result in a new matrix
	aMatrix<long double>* 	
				Multiply(aVector<T>& v);//premultiplies v to this and returns result in a new matrix
	void 			Multiply (T scalar);
	int 		Nullity(void){
					if(nullity_ == -1){CalcNullity();}
					return nullity_;}
	int 		orthes(aMatrix<long double>& V,//modified from JAMA library, in jama_eig.h
					   aVector<long double>& ort,//called in some eigenvalue calcs
					   aMatrix<long double>& H);
	void 			Print (char* outFileName, char separator=' '){//file assumed closed
						aString emptyMessage;
						Print(emptyMessage, outFileName, separator);}
	void 			Print (aString& message, char* outFileName, char separator=' ');//file assumed closed
	void 			Print (aString& outFileName, char separator=' '){//file assumed closed
						aString emptyMessage;
						Print(emptyMessage, outFileName, separator);}
	void 			Print (aString& message, aString& outFileName, char separator=' '){//file assumed closed
					char* rawFileName = outFileName.raw();
					Print (message, rawFileName, separator);
					delete [] rawFileName; rawFileName=NULL;}
	void 			Print (aString& message, ostream& outFile=cout, char separator=' ');//file assumed open
	void 			Print (ostream& outFile=cout, char separator=' '){//file assumed open
						aString emptyMessage;
						Print(emptyMessage, outFile, separator);}
	double 			Pythag (double a, double b);
	long double 	Pythag (long double a, long double b);
					//gives back (a^2 +b^2)^(.5) without overflow problems; Press et al p.70
	void 			RandomPermutation(Ran3& ran);
	long 			Rank(void){
					return MIN(rows_,cols_)-Nullity();}
	void 			Replace (T val, long row, long col){data[row][col]=val;}
	unsigned long	rows(void){return rows_;}
	aVector<T>* 	Row (long  row);//returns the row in a new vector
	T 			sesq(void);//sum of squared elements of matrix
	double 			SignProc (double a, double b);
	long double 	SignProc (long double a, long double b);
	int 			Size (long rows, long cols){//boolean
					if((rows==rows_)&&(cols==cols_)){return 1;}else{return 0;}}
	aVector<long double>* 	
				SolutionOfLinearEquations (aMatrix<T>&coefficients, aVector<T>& rightHandSide);
						//debug first;  returns result in a new vector
	int 			Square(void){//boolean
					if (rows_ == cols_){return 1;}else{return 0;}}
	aMatrix<T>* 	submatrix(long rStart, long cStart, long rEnd, long cEnd);//returns new submatrix
	aMatrix<T>* 	Subtract (aMatrix<T>& m);
					//subtracts m from this and returns result in a new matrix
					//Sizes must match or NULL pointer returned
	aVector<long double>*  	
				SVBackSubstitution (matrixHandle<long double>& VmatrixH, 
										aVector<long double>&  b, aVector<long double>& w);	
					//debug first;  returns result in a new vector
	aMatrix<long double>* 	
				SVDecomposition (matrixHandle<long double>& VmatrixH, aVector<long double>& w);
	aMatrix<long double>*	
				SVInverse (void);	//returns result in a new matrix
	void 			SwapCols(long c1, long c2);
	void 			SwapRows(long r1, long r2);
	void 			SwapRowsCols(long rc1, long rc2){
						SwapRows(rc1, rc2);
						SwapCols(rc1, rc2);}
	int 		symmetric(void);//returns true/false value
	aMatrix<double>* 	
				ToDouble(void);//converts all values to type 'double' and returns result as a new matrix
	aMatrix<long double>* 	
				ToLongDouble(void);//converts all values to type 'long double' and returns result as a new matrix
//	int 		tqli(aVector<long double>& diagonalElements,//buggy
//					 aVector<long double>& offdiagonalElements,
//					 aMatrix<long double>& eigenVectorCols, int getEigenVectors);//called by EigenCalcs()
//	int 		tred2(aVector<long double>& diagonalElements,//buggy
//					 aVector<long double>& offdiagonalElements, 
//					 aMatrix<long double>& V, int getEigenVectors);//called by EigenCalcs()
	int 		tql2(aVector<long double>& diagonalElements,//modified from JAMA library, in jama_eig.h
					 aVector<long double>& imaginaryElements,
					 aMatrix<long double>& eigenVectorsInCols);
	int 		tred2(aVector<long double>& diagonalElements,//modified from JAMA library, in jama_eig.h
					  aVector<long double>& imaginaryElements, 
					  aMatrix<long double>& V);

	T			Trace(void);
	aMatrix<T>* 	Transpose (void);	//returns result in a new matrix


	aVector<T>& 	operator[](long row){
					return data[row];}

	friend int operator== (aMatrix< T >& matrix1, aMatrix< T >& matrix2);

	friend int operator!= (aMatrix< T >& matrix1, aMatrix< T >& matrix2);


	};//end class aMatrix




template <class T > int operator== (aMatrix< T >& matrix1, aMatrix< T >& matrix2);

template <class T > int operator!= (aMatrix< T >& matrix1, aMatrix< T >& matrix2);





template<class T> class matrixHandle{
	public:
		aMatrix<T>* mat;
	public:
		matrixHandle(void): mat(NULL){
			}
		matrixHandle(aMatrix<T>* m):mat(NULL){
			mat = m;}
		matrixHandle(aMatrix<T>& m):mat(NULL){
			mat = &m;}
		~matrixHandle(void){//detach if you want to keep the matrix
			}
		
		matrixHandle* Duplicate(void){
			return NEW matrixHandle(*this);}
		
		aMatrix<T>* operator->(void){
			return mat;}
			
		void Detach(void){
			mat = NULL;}
		
		void Bind(aMatrix<T>* m){
			delete mat; mat=NULL;
			mat=m;}

		void Bind(aMatrix<T>& m){
			delete mat; mat=NULL;
			mat=&m;}
//		#ifdef MemoryLeakDebug
//			static void PrintIDList(ostream& outFile=cout);
//			static void PrintIDListLen(ostream& outFile=cout);
//		#endif
		
		friend int operator==(matrixHandle& mh1, matrixHandle& mh2);
		friend int operator!=(matrixHandle& mh1, matrixHandle& mh2);
		
		
	};//end matrixHandle<T>


template <class T> class identityMatrix: public aMatrix<T>{
	public:
		identityMatrix(long len): aMatrix<T>(T(0.0), len,len){
			identityMatrix<T>& x = *this;
			for(long r=0;r<rows_;++r){
				x[r][r]=T(1.0);}
			nullity_=len;}
		identityMatrix(identityMatrix<T>& m):aMatrix<T>(m){}
		identityMatrix(identityMatrix<T>* m):aMatrix<T>(m){}
		virtual ~identityMatrix(void){}
	};//end identityMatrix<T>





//functions that return new matrices
	//defined in MatrixFunctions.cpp
aMatrix<long double>* 	CMatrix(unsigned long len);

template <class T> aMatrix<T>* IdentityMatrix(T x, unsigned long len);
		//x is a dummy that gives the return type


aMatrix<long double>* 	JbarMatrix(unsigned long len);

template <class T> aMatrix<T>* JMatrix(T x, unsigned long len);
		//x is a dummy that gives the return type

template <class T> aMatrix<T>* JMatrix(T x, unsigned long rows, unsigned long cols);
		//x is a dummy that gives the return type

template <class T> aMatrix<T>* OneMatrix(T x,unsigned long rows, unsigned long cols);
		//x is a dummy that gives the return type

template <class T> aMatrix<T>* ZeroMatrix(T x,unsigned long rows, unsigned long cols);
		//x is a dummy that gives the return type


#endif

