/*******************************************
*
*
*	aVector.h
*
*	A class that implements vectors, and does vector algebra
*
*	Vectors can be converted from    aList<T>
*
*
************************************************/

#ifndef aVectorh
#define aVectorh = 1

#include "aError.h"
#include "Element.h"
#include "aList.h"
#include <iostream.h>
#include "MemoryManager.h"
#include "StringClass.h"
#include "InlineTemplates.h"



template <class T> class aVector{
	public:
		unsigned long len_;//ORDER DEPENDENCY; DON'T SWITCH THESE
		T*  data;
	public:
		aVector(T x, unsigned long n=1);
		aVector(unsigned long n);
		aVector(void);
		aVector(aList<T>& x);
		aVector(aVector<T>& v);
		aVector(aVector<T>* v);
		virtual  ~aVector(void);
		virtual void BadOperation(aString& err/*, ostream& outFile=cout*/);
		
	virtual aVector<T>* 
			Duplicate(void){
				return NEW aVector<T>(*this);}
			
	aVector<T>* 	Add(aVector<T>& v);//adds this to v and returns result as a new vector
	void 		Append (T val);//lengthens the current vector
	void 		Append (aVector<T>& v);//lengthens the current vector; v unchanged
	void  		Clear(void);
	int 		Compatible(aVector<T>& v);//can be added, multiplied, etc.
	void 		Delete (long loc);//shortens the vector
	void 		Initialize(T val = (T)0.0);
	void 		Insert (T val, const long loc);//lengthens the current vector
	unsigned long len(void);
	void 		Multiply (T scalar);
	long double Multiply (aVector<T>& v);//multiplies this by v and returns result as a new vector
	T 			Sum (void);

	void 		Print (ostream& outFile, int columnVector=1, char separator=' ');
					//default 1 is column vector, 0 is row vector
					//outFile assumed open
	void 		Replace (T val, long loc);

	aVector<double>* ToDouble(void);//returns copy of values as type 'double'
	aVector<long double>* ToLongDouble(void);//returns copy of values as type 'long double'
	T& 			operator[](unsigned long i);
	aVector<T>* 	operator+(aVector<T>& v);
	aVector<T>*	 operator-(aVector<T>& v);
	friend int operator== (aVector< T >& vector1, aVector< T >& vector2);

	friend int operator!= (aVector< T >& vector1, aVector< T >& vector2);
	};//end class vector



//template <class T > int operator== (aVector< T >& vector1, aVector< T >& vector2);

//template <class T > int operator!= (aVector< T >& vector1, aVector< T >& vector2);


#endif

