/***************************************************************
*
*
*	StatList.h
*
*	base class:   aList<T>
*
*	Does basic statistics on lists of numbers
*
***************************************************************/


#ifndef StatListh
#define StatListh = 1


#include "Element.h"
#include "aList.h"
#include <iostream.h>
#include <math.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"

template <class T> class StatList:  public aList<T> {
 protected://flags
  	int flagsFlag_;
	int harmonicMeanFlag_;
  	int kurtosisFlag_;
  	int maxFlag_;
  	int meanFlag_;
  	int medianFlag_;
  	int minFlag_;
  	int stdevFlag_;
  	int stdevPFlag_;
  	int sumFlag_;
  	int sumSqdFlag_;
  	int sumCubedFlag_;
  	int sum4thedFlag_;
  	int sqdSumFlag_;
  	int varFlag_;
  	int varPFlag_;
	long double	harmonicMean_;
	long double	kurtosis_;
	T		max_;
	long double	mean_;
	long double	median_;
	T 		min_;
	long double	stdev_;//sample standard deviation using (n-1)
	long double	stdevP_;//total population standard deviation using (n)
	long double	sum_;
	long double	sumSqdVals_;//=sum (x^2)
	long double	sumCubedVals_;//=sum (x^3)
	long double	sum4thedVals_;//=sum (x^4)
	long double	sqdSum_;//(sum x)^2
	long double	var_;//sample variance using (n-1)
	long double	varP_;//total population variance using (n)

  	
  public:
    StatList(void);//empty list
    
	StatList(StatList<T>& toCopy);//constucts a new copy
	StatList(StatList<T>* toCopy);//constructs a new copy

	StatList(aList<T>& toCopy) : aList<T>(toCopy){//upgrades a aList<T> as a new copy
					ResetFlags();}
	StatList(aList<T>* toCopy) : aList<T>(toCopy){//upgrades a aList<T> as a new copy
					ResetFlags();}

virtual ~StatList(void) {
	}

virtual void 		Append (T x){if (flagsFlag_){ResetFlags();} aList<T>::Append(x);}
virtual void		Clear(void){if (flagsFlag_){ResetFlags();} aList<T>::Clear();}
virtual void		CullMissingData(T missingDataVal);
virtual void 		DeleteFirst (void){
						if (flagsFlag_){ResetFlags();} 
						aList<T>::DeleteFirst();}
virtual void 		DeleteLast (void){
						if (flagsFlag_){ResetFlags();} 
						aList<T>::DeleteLast();}
virtual void 		Decrement (long loc){
						Replace(Nth(loc)-1,loc);}
virtual void 		Increment (long loc){
						Replace(Nth(loc)+1,loc);}
virtual void 		Insert (T x, long loc){
						if (flagsFlag_){ResetFlags();} 
						aList<T>::Insert(x,loc);}
virtual void 		Prepend (T x){
						if (flagsFlag_){ResetFlags();} 
						aList<T>::Prepend(x);}
virtual void 		Replace (T x, long loc){
						if (flagsFlag_){ResetFlags();} 
						aList<T>::Replace(x,loc);}

virtual void		ResetFlags(void){
					flagsFlag_=kurtosisFlag_=maxFlag_=medianFlag_=meanFlag_=minFlag_=stdevFlag_=stdevPFlag_=
						sumFlag_=sumSqdFlag_=sumCubedFlag_=sum4thedFlag_=
						harmonicMeanFlag_=sqdSumFlag_=varFlag_=varPFlag_=0;
					kurtosis_=harmonicMean_=max_=median_=mean_=min_=stdev_=stdevP_=sum_=sumSqdVals_=
						sumCubedVals_=sum4thedVals_=sqdSum_=var_=varP_=zero;
					}//end ResetFlags
StatList<T>& 		operator= (StatList<T>& z);
virtual aList<T>* 		Duplicate(void){return NEW StatList(*this);}

long		df(void) {return n()-1;}
long double	kurtosis(void);
long double 	harmonicMean(void);
long		n(void) {return len();}
T 		max(void);
long double	mean(void);
long double	median(void);
T		min(void);
long double 	stdErrMean(void);
long double	stdev(void);//sample standard deviation using (n-1)
long double	stdevP(void);//total population standard deviation using (n)
long double	sum(void);
long double	sumSqdVals(void);//=sum (x^2)
long double	sumCubedVals(void);//=sum (x^3)
long double	sum4thedVals(void);//=sum (x^4)
long double	sqdSum(void);//(sum x)^2
long double	t(long double givenMean);//one-sample t-test statistic with df=n-1
long double	var(void);//sample variance using (n-1)
long double	varP(void);//total population variance using (n)

};//end StatList

#endif
