/******************************************
*
*	aList.h
*
*	This implements a template container class
*	for growable lists of unknown size.
*
*******************************************/

#ifndef aListh
#define aListh=1


#include <iostream.h>
#include "Element.h"
#include "aError.h"
#include "InlineTemplates.h"
#include "MemoryManager.h"


template<class T> class aList;


template<class T> class aList{
	// for int, float, etc. standard objects
  protected:
  	long len_;
  	NumElem<T> *first,*last,*recent;
  	long recentLoc_;
  public:
    aList(void);//empty list
	aList(aList<T>& toCopy);//constucts a new copy
	aList(aList<T>* toCopy);//constructs a new copy

virtual ~aList(void);//deletes remaining contents
virtual void 		Append (T x);
virtual int			BadIndex(long loc);
virtual int			CatchBadAlloc(const char* fname);//takes name of member function where error ocurred
virtual void		Clear(void);
virtual void		Copy(aList<T>& toCopy);
virtual void		Copy(aList<T>* toCopy);
virtual void		Decrement (long loc);
virtual void 		DeleteFirst (void);
virtual void 		DeleteLast (void);
virtual void 		Delete (long loc);
						//range of loc=0 (=first) to len()-1 (=last)
virtual aList<T>* 	Duplicate(void);
virtual T&   		First (void);
	NumElem<T>* 	GoToElem (long loc);
virtual int			Includes (T x);
virtual void		Increment (long loc);
virtual void 		Insert (T x, long loc);
						//range of loc=0 (=first) to len() (=last+1)
						//x inserted at loc; elements  loc are shifted up
						//x simply Appended if loc==len_
virtual T&   		Last (void);
virtual long 		len(void);
virtual long 		Loc (T x);
						//range of loc=0 (=first) to len()-1 (=last)
virtual T&	  		Nth (long loc);
						//range of loc=0 (=first) to len()-1 (=last)
virtual void 		Prepend (T x);
virtual void 		Print(ostream& outfile, char* seperator);
virtual void 		Replace (T x, long loc);
						//range of loc=0 (=first) to len()-1 (=last)
virtual void 		Sort (void);
						//sorts entire list using Sort(long,long)
virtual void 		Sort (long start, long finish);
						//a QuickSort algorithm
aList<T>&			operator= (aList<T>& z);
virtual T&			operator[] (const long loc);



	friend int operator== (aList< T >& list1, aList< T >& list2);

	friend int operator!= (aList< T >& list1, aList< T >& list2);
	
    };//end class aList<T>
	
template <class T > int operator== (aList< T >& list1, aList< T >& list2);

template <class T > int operator!= (aList< T >& list1, aList< T >& list2);






template<class T> class ObjList {
	//for user-defined classes
  protected:
  	long len_;
  	ObjElem<T> *first,*last, *recent;
  	long recentLoc_;
  public:
    ObjList(void);//empty list
	ObjList(ObjList<T>& toCopy);
				//constucts a new copy
	ObjList(ObjList<T>* toCopy);
				//constructs a new copy
    		
virtual ~ObjList(void);//deletes remaining contents

virtual void 		Append (T* x);
virtual int			BadIndex(long loc);
virtual int			CatchBadAlloc(const char* fname);//takes name of member function where error ocurred
virtual void		Clear(void);
						//frees memory of all contents
void				Copy(ObjList< T >& toCopy);
						//makes a new copy of the contents of toCopy
void				Copy(ObjList< T >* toCopy);
						//makes a new copy of the contents of toCopy
virtual void 		DeleteFirst (void);
virtual void 		DeleteLast (void);
virtual void 		Delete (long loc);
						//range of loc=0 (=first) to len()-1 (=last)
virtual void 		DetachFirst (void);
virtual void 		DetachLast (void);
virtual void 		Detach (long loc);
						//range of loc=0 (=first) to len()-1 (=last)
virtual void 		Detach (T* x);
virtual void 		Detach (void);
						//releases but does not delete contents
virtual ObjList<T>* Duplicate(void);
virtual T&   		First (void);
	ObjElem<T>* 	GoToElem (long loc);
virtual int			Includes (T* x);
virtual void 		Insert (T* x, long loc);
						//range of loc=0 (=first) to len() (=last+1)
						//x inserted at loc; elements  loc are shifted up
						//x simply Appended if loc==len_
virtual T&   		Last (void);
virtual const long 	len(void);
virtual long 		Loc (T* x);
						//range of loc=0 (=first) to len()-1 (=last)
virtual T&   		Nth (long loc);
						//range of loc=0 (=first) to len()-1 (=last)
virtual void 		Prepend (T* x);
virtual void 		Replace (T* x, long loc);
						//range of loc=0 (=first) to len()-1 (=last)
ObjList<T>&			operator= (ObjList<T>& z);
virtual T&			operator[] (const long loc);

	friend int operator== (ObjList< T >& list1, ObjList< T >& list2);

	friend int operator!= (ObjList< T >& list1, ObjList< T >& list2);

	
    };//end class ObjList<T>

/*
template <class T > int operator== (ObjList< T >& list1, ObjList< T >& list2);

template <class T > 
	int operator!= (ObjList< T >& list1, ObjList< T >& list2);
*/



#endif





