/*********************************************
*
*
*		BetaDistribution.h
*
*
*		Functions for calculating beta-distribution statistics
*
*
**********************************************/


#ifndef BetaDistributionh
#define BetaDistributionh = 1

#include <math.h>
#include <iostream.h>
#include "aError.h"
#include "Gamma.h"
#include "Element.h"
#include "aList.h"
#include "InlineTemplates.h"
#include "Randomness.h"
#include "MemoryManager.h"


long double Beta(long double alpha, long double beta);
		//beta function
		
long double BinomialDistributionCDF(long n, long r, long double p);
		//n = sample size, r = #found, p = probability
		//calculated using the incomplete beta function
		
long double lnBeta(long double alpha, long double beta);
		//natural log of beta function -- useful when Beta is an intermediate value
		
long double BetaDistributionPDF(long double alpha, long double beta, long double x);
		//probability density: calculates 1/Beta(alpha,beta) * x^(aplha-1) * (1-x)^(beta-1)
		
long double lnBetaDistributionPDF(long double alpha, long double beta, long double x);
		//natural log of the probability density
		
long double lnBetaDistributionPDF(long double alpha, long double beta, long double x, long double dx);
		//natural log of the probability density scaled by dx
		
long double BetaDistributionCDF(long double alpha, long double beta, long double x);
		//cumulative probability density 
		

long double BetaIncomplete(long double alpha, long double beta, long double x);
	//Beta function integrated from 0 to  x (0x1)


long double BetaContinuedFraction(long double alpha, long double beta, long double x);
	//used to calculate BetaIncomplete

long double BetaRegularized(long double alpha, long double beta, long double x);
	//BetaIncomplete as a proportion of Beta


inline long double RandomBeta(long double alpha, long double beta, Ran3& ran){
	return ran.BetaDev(alpha,beta);}


#endif

