/***************************************************************
*
*
*		Bisection_GoldenRatioFns.h
*
*		Bisection narrows in on a value along a line, including root-finding in 1 dimension.
*		GoldenRatio finds the minimum or maximum of a function in 1 dimension.
*
*		From Press et al. 1992. Numerical recipes in C
*
****************************************************************/



#ifndef Bisection_GoldenRatioFnsh
#define Bisection_GoldenRatioFnsh

#include "MemoryManager.h"
#include "aError.h"
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "InlineTemplates.h"
#include <math.h>



typedef long double (*MinMaxRootFn)(long double, aList<long double>&);
	/* Returns the minimum, maximum, or root, as apropritate, given the data in the list*/
	/* Must specify the exact form of the MinMaxRootFn function*/
	/* before feeding it into the functions below*/

//long double dummyfunction(long double, aList<long double>&);

inline long double dummyfunction(long double, aList<long double>&){
	return zero;}


class Bisection_GoldenRatioFns{
	public:
		MinMaxRootFn func;
	public:
		Bisection_GoldenRatioFns(void){
			(*func) = dummyfunction;
			}
		Bisection_GoldenRatioFns(MinMaxRootFn theFunction){
			(*func) = theFunction;
			}
		Bisection_GoldenRatioFns(Bisection_GoldenRatioFns& bgfn){
			(*func) = bgfn.func;
			}
		~Bisection_GoldenRatioFns(void){
			func = NULL;
			}
		public:
	void	SetFunction(MinMaxRootFn theGenericFunction){
					( *func ) = theGenericFunction;}
					
	Bisection_GoldenRatioFns& operator=(Bisection_GoldenRatioFns& bgfn){
		SetFunction(bgfn.func);
		return *this;}
	
	inline void Shift2(long double& pp, long double& qq, long double rr){
		pp=qq; qq = rr;}
		
	inline void Shift3(long double& pp, long double& qq, long double& rr, long double ss){
		pp=qq; qq = rr; rr=ss;}


	long double  FindRootViaBisection(long double x1, long double x2, aList<long double>& variables); 
			//the function returns a point between x1 and x2 to accuracy tol (tlerance), using variables in variableList
			//fn rtbis in Press et al


	long double  FindPointViaBisection(long double yTarget, long double x1, long double x2, aList<long double>& variables); 
			//the function returns a point between x1 and x2 to accuracy tol (tlerance), using variables in variableList
		//before calling, it is critical identify which section of the curve to investigate, as y may occure at more than one x vbalue n a curve




	//   !!!!!    NOT WORKING SATISFACTORALLY YET    !!!!!     
	long double FindMinMaxViaGoldenRatio(bool minimize, long double ax, long double bx, long double cx, 
											long double tol, long double& xmin, aList<long double>& variables);
						//minimize=true to minimize the function, minimize=false to maximize the function
			//the function returns the min/max between ax and cx, with ax<bx<cx, using variables in variableList
			//fn golden in Press et al


	};//end class Bisection_GoldenRatioFns














#endif