/*********************************************
*
*
*		FDistribution.h
*
*
*		Functions for calculating F-distribution statistics
*
*
**********************************************/


#ifndef FDistributionh
#define FDistributionh = 1

#include <math.h>
#include "Gamma.h"
#include <iostream.h>
#include "Element.h"
#include "aList.h"
#include "InlineTemplates.h"
#include "MemoryManager.h"


long double FDistribution(int df1, int df2, long double x);
		//calculates the height of F(df1,df2) at point x
		
long double Fdistribution1TailedP(int df1, int df2, long double observedF);


long double qrombFDistribution(int df1, int df2, long double a, long double b, int n);
	//see Press et al. p. 129, a method called Romberg's integration procedure	
	//returns integral from a to b of the Fdistribution; n is a scaling constant 

long double trapzdFDistribution(int df1, int df2, long double a, long double b, long double s, int n, 
							int& trapzdIterator);
	//see Press et al. p. 124, a rough integration procedure	
	//returns approximate integral from a to b of the Fdistribution
	//n is a scaling constant that determines the accuracy of the integral

void polintFDistribution(aList<long double>& xa, aList<long double>& ya, const int n, 
						 int np, long double x, long double& y, long double& dy);
	//see Press et al. p. 92, an interpolation/extrapolation procedure for polynomials
	//given x, and arrays xa and ya, it returns an approximate y and an approximate error dy


#endif

