/******************************************************
*
*	CumNormalDistParamRec.h
*
*	Parameters for a cumulative normal distribution defined by mean and stdev.
*
*	Use this as a base class for derived classes that add parameters.
*
******************************************************/

#ifndef CumNormalDistParamRech
#define CumNormalDistParamRech = 1


#include "Element.h"
#include "aList.h"
#include "ParamRec.h"
#include "InlineTemplates.h"
#include "NormalDistParamRec.h"
#include "MemoryManager.h"

class CumNormalDistParamRec;
class CumNormalDistParamRec: virtual public NormalDistParamRec{
  public:
  	CumNormalDistParamRec(void){
  				SetPointerToDerivedSelf();
  				}
  	CumNormalDistParamRec(long double m, long double s){
  				SetPointerToDerivedSelf();
				mu_= m;
				sigma_ = s;
  				}
  	CumNormalDistParamRec(CumNormalDistParamRec& p){
  				SetPointerToDerivedSelf();
				lnL_= p.lnL_;
				lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;	
				mu_= p.mu_;
				sigma_ = p.sigma_;
				}
  	virtual ~CumNormalDistParamRec(void){
  				pointerToDerivedSelf = NULL;//forget
  				}

  	virtual ParamRec* 				Duplicate(void);
  	virtual CumNormalDistParamRec* 	_Duplicate(void);
	virtual void 					SetPointerToDerivedSelf(void);//always override in derived

  	virtual long double		CumulativeNormalProbabilityAtX(long double x);
	virtual long double 		XforCumulativeNormalProbability(long double prob);


	virtual long double 		ExpectedValue(aList<long double>& coords);
	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(CumNormalDistParamRec& p, CumNormalDistParamRec& q);
	friend int	operator!=(CumNormalDistParamRec& p, CumNormalDistParamRec& q);

	};//end class CumNormalDistParamRec



#endif