/***************************************************
																							
	 GeneticPopulation
	 
	 Base Class:  Population															
																							
	Handles population processes for genetic individuals	
																							
***************************************************/

#ifndef GeneticPopulationh
#define GeneticPopulationh = 1

#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "RandomList.h"
#include "StatList.h"
#include "StringList.h"
#include "PopulationIntf.h"
#include "Individual.h"
#include "GeneticIndividual.h"
#include "GeneContainers.h"
#include <iostream.h>
#include "Population.h"
#include "InlineTemplates.h"
#include "MemoryManager.h"





class GeneticPopulation: public Population {
  public:
	ObjList<StringList> 		allelesInPop_;//values of alleles stored as strings; complex alleles possible
		//now organized onto chromosomes
	int							allelesCurrent_;//flag
	ObjList<aList<long double> > 			allFreq_;
	int							allFreqCurrent_;//flag
	StatList<long> 					sampleSize_;
	int							sampleSizeCurrent_;//flag
	StatList<long double> 				hetObsLoc_;
	StatList<long double>				hetExpLoc_;
	long double						hetObs_;
	long double						hetExp_;
	int							hetExpDataCurrent_;//flag
	int							hetObsDataCurrent_;//flag


		GeneticPopulation(long id, int species, GridCell* g,long k);
		GeneticPopulation(long id);
	 	GeneticPopulation(GeneticPopulation& p);
		GeneticPopulation(GeneticPopulation* p);
virtual ~GeneticPopulation(void);
virtual  Population* Duplicate(long id, int species, GridCell* g, long k){/*virtual constructor*/
				return NEW GeneticPopulation(id,species,g,k);}
virtual  Population* Duplicate(long id){/*virtual constructor*/
				return NEW GeneticPopulation(id);}
virtual  Population* 	Duplicate(void){/*virtual constructor*/
				return NEW GeneticPopulation(*this);}

/*FUNCTIONS*/
virtual void		AgeOneStep(int maxage);
virtual void		AssignMutations(void);//derive this
virtual void		CalcAlleleFrequencies(void);
virtual void		CalcExpHeterozygosity(void);
virtual void		CalcObsHeterozygosity(void);
virtual void		CoordsOfNthLocus(int nthLoc, int& onChromNum, int& locNumOnChrom);
virtual void 		Extract (Individual* indiv);
virtual int			loci(void);
virtual double		fitness(int indiv);
//virtual double		hetExp (int loc, T alleleVal);
virtual long double		hetExp (int loc, aString& alleleVal);
virtual long double		hetExp (int loc);
//virtual double		hetObs (int loc, T alleleVal);
virtual long double		hetObs (int loc, aString& alleleVal);
virtual long double		hetObs (int loc);
virtual void 		Install (Individual* indiv);
virtual int			NthLocus(int chrom, int locOnChrom);//0 indexed
virtual void		PrintAlleleFreqs(ostream&);
virtual void		PrintAlleleFreqs(ostream&, int /*locus*/);
//virtual void		PrintAlleleFreqs(ostream&, int /*locus*/, T /*alleleVal*/);
virtual void		PrintAlleleFreqs(ostream&, int /*locus*/, aString& /*alleleVal*/);
virtual void		PrintAlleleList(ostream&);
virtual void		PrintAlleleList(ostream&, int /*locus*/);
virtual void		PrintGeneStrings(ostream& outFile, int alleleLen=4, int codeLen=5);
virtual void		PrintGeneStrings(ostream& outFile, ObjList<Individual>& theList, int alleleLen=4, int codeLen=5);
//virtual void		Reproduce (long& idCount, Ran3List& ran, double mutationRate, double sigma);
virtual void		Reproduce (long& idCount, Ran3List& ran, aList<long double>& specialInfo);
//virtual void		Reproduce (long& /*idCount*/, Ran3List&/* ran*/){
//							cout<<"ERROR: Shouldn't arrive here"<<endl;
//							cout<<"Call GeneticPopulation::Reproduce(long&,Ran3List&,double,double) instead"<<endl;}

virtual void		UpdateAlleleList(void);//makes a list of all alleles in the pop
//		void		UpdateAlleleList(int loc, T alleleVal);//derive this
virtual void		UpdateAlleleList(ObjList<Individual>& theList);
virtual void		UpdateAlleleList(Individual& indiv);
virtual void		ViabilitySelection (Ran3& ran);
virtual void		ViabilitySelection (ObjList<Individual>& theList, Ran3& ran);

//virtual double		AlleleFrequency (int loc, T alleleVal);
virtual long double		AlleleFrequency (int loc, aString& alleleVal);
//virtual int			AllelePresent (int loc, T alleleVal);
virtual int			AllelePresent (int loc, aString& alleleVal);
//virtual int			AllelePresent (ObjList<Individual>& theList, int loc, T alleleVal);
virtual int			AllelePresent (ObjList<Individual>& theList, int loc, aString& alleleVal);
virtual long		Alleles (int loc);
virtual aString&	AlleleValue (int loc, int alleleNum);
//virtual T			AlleleValue (int indivNum, int loc, int alleleNum);
//virtual long		CountAlleles (int loc, T alleleVal);
virtual long		CountAlleles (int loc, aString& alleleVal);
//virtual long		CountAlleles (ObjList<Individual>& theList, int loc, T alleleVal);
virtual long		CountAlleles (ObjList<Individual>& theList, int loc, aString& alleleVal);
//virtual long		CountHets (int loc, T alleleVal);
virtual long		CountHets (int loc, aString& alleleVal);
//virtual long		CountHets (ObjList<Individual>& theList, int loc, T alleleVal);
virtual long		CountHets (ObjList<Individual>& theList, int loc, aString& alleleVal);
virtual long		CountHets (int loc);
virtual long		CountHets (ObjList<Individual>& theList, int loc);
virtual void 		GetGenome(PopulationGeneContainer<int>& pgc);
virtual void 		GetGenome(MultiInheritanceModeContainer<int>& mimc, long indiv);
virtual aString*	GenotypeString(long indiv, char* separator = " ");
virtual aString*	GenotypeString(long indiv, int locus, char* separator = " ");
virtual aString*	GenotypeString(long indiv, int locus, int alleleNum, char* separator = " ");
virtual aString*	GetPhenotypeString(long indiv, char* separator = " ");
virtual int			ploidy(int loc);
virtual void		SampleSize (void);//handles ploidy differences
virtual long		SampleSize (int loc);//handles ploidy differences
virtual long		SampleSize (int loc, ObjList<Individual>& theList);
virtual void 		SetSelectionParams (aList<long double>& selParams);//just those that relate to this population
virtual void 		SetSelectionParams (aList<long double>& selParams, GeneticIndividual& indiv);
virtual void		TallyAlleleFrequencies(ObjList<Individual>& theList);
virtual int			NoInconsistentSettings(void);  //returns an error code if settings are inconsistent

	};	//end GeneticPopulation

#endif
