/***************************************************
																							
	 Population																	
																							
	Handles population processes, for analysis of geographic and demographic structure.	
	Data analysis handled using derived classes.		
																							
***************************************************/

#ifndef Populationh
#define Populationh = 1


#include <iostream.h>
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "RandomList.h"
#include "CoordinateClass.h"
#include "PopulationIntf.h"
#include "SelectionIntf.h"
#include "Individual.h"
#include "InlineTemplates.h"
#include "MemoryManager.h"


class GridCell;
class Genotype;
class Individual;


class Population{
	protected:
		long 			id_;
		int 			species_;
		long			carryingCapacity_; /*ie, K in mathematical ecologese*/
		GridCell*		site_;
//		double selectionStrength_;

		ObjList<Individual>	asexuals_;
		ObjList<Individual>	females_;
		ObjList<Individual>	males_;
		ObjList<Individual>	herms_;		/*hermaphrodites*/
		ObjList<Individual>	asexualBabies_;
		ObjList<Individual>	maleBabies_;
		ObjList<Individual>	femaleBabies_;
		ObjList<Individual>	hermBabies_;
		ObjList<Individual>	femaleGametes_;//unikte with male
		ObjList<Individual>	maleGametes_;//unite with female
		ObjList<Individual>	hermGametes_;//unite with other herms
		ObjList<Individual>	deadList_;
	public:		
		Population(long id, int species, GridCell* g, long k);
		Population(long id);
		Population(void);
		Population(Population& p);
		Population(Population* p);
virtual ~Population(void);
virtual long 		GetPointerToDerivedSelf(void){
								return (long) this;}
virtual  Population* 		Duplicate(long id, int species, GridCell* g, long k);/*virtual constructor*/
virtual  Population* 		Duplicate(long id);/*virtual constructor*/
virtual  Population* 		Duplicate(void);/*virtual constructor*/

virtual Individual& 	Adult(long i);//returns an adult, order asexuals, herms, females, males
virtual long 			adults(void);
//virtual void 			AdultDispersal(Ran3List& ran, double m=1.0,
//									   double kurtosisFactor=0.0);
virtual void 			AdultDispersal(Ran3List& ran, aList<long double>& dispersalInfo);
virtual void			AgeOneStep(int maxage);
virtual void			AgeOneStep(ObjList<Individual>& theList, int maxage);
virtual long 			asexualBabies(void){return asexualBabies_.len();}
virtual Individual& 	asexualBaby(long i){return asexualBabies_[i];}
virtual long 			asexuals(void){return asexuals_.len();}
virtual Individual& 		asexual(long i){return asexuals_[i];}
virtual void			AssignMates (Ran3List& ran, aList<long double>& reproductionInfo, int tries=1, int maxThisAgeClass=1);
virtual long			babies(void);
virtual Individual& 	Baby(long i);
virtual void			BuryTheDead(void);
virtual long			carryingCapacity(void) {return carryingCapacity_;}
virtual void			ClearGameteLists(void);
virtual void			ClearOffspringLists(void);
virtual CoordinateType&  coords(void);
virtual long			dead(void);
virtual void			dead(Individual* deadguy);
virtual void			dead(Individual* deadgamete, genderSet typeOfGamete);
virtual Individual& 		dead(long i){return deadList_[i];}
//virtual void 			Dispersal(Ran3List& ran, double m, double kurtosisFactor);
//virtual void			Dispersal (ObjList<Individual>& theList, Ran3List& ran,
//								   double m=1.0, double kurtosisFactor=0.0);
virtual void 			Dispersal(Ran3List& ran, aList<long double>& dispersalInfo);
virtual void			Dispersal (ObjList<Individual>& theList, Ran3List& ran,
								   aList<long double>& dispersalInfo);
virtual int				extinct(void) {return (size()==0);}
virtual void 			Extract (Individual* indiv);
virtual void 			Extract (Individual* indiv, genderSet parentGender);//extracts gametes; defaults to Extract(individual)
virtual long double		Fecundity(int age);
virtual long 			fathers(void){return herms()+males();}//potential male-role parents
virtual Individual& 	father(long i);//out of potential male-role parents
virtual long 			femaleBabies(void){return femaleBabies_.len();}
virtual Individual& 		femaleBaby(long i){return femaleBabies_[i];}
virtual long 			femaleGametes(void){return femaleGametes_.len();}
virtual Individual& 		femaleGamete(long i){return femaleGametes_[i];}
virtual long 			females(void){return females_.len();}
virtual Individual& 		female(long i){return females_[i];}
virtual Individual& 	Gamete(long i);//returns a gamete, order herms, females, males
virtual long 			gametes(void);
virtual void 			GameteDispersal(Ran3List& ran, aList<long double>& dispersalInfo);
virtual void			GetAgeDistribution(aList<int>& ageDist);
virtual void			GetAgeDistribution(ObjList<Individual>& theList, 
										   aList<int>& ageDist);
virtual Individual& 		GetAdult(long i);
virtual Individual& 		GetBaby(long i);
virtual Individual& 		GetGamete(long i);
//virtual Individual& 		GetIndividual(long i){return GetAdult(i);}
virtual Individual& 		GetIndividual(long i);
virtual long double 	gridScale(void);
virtual int				habitat(void);
virtual long 			hermBabies(void){return hermBabies_.len();}
virtual Individual& 		hermBaby(long i){return hermBabies_[i];}
virtual long 			hermGametes(void){return hermGametes_.len();}
virtual Individual& 	hermGamete(long i){return hermGametes_[i];}
virtual long 			herms(void){return herms_.len();}
virtual Individual& 		herm(long i){return herms_[i];}
		 long			id(void){return id_;}
virtual void 			ImmatureDispersal(Ran3List& ran, aList<long double>& dispersalInfo);
virtual long 			immigrants(void);//number of new immigrants
virtual long 			immigrantAdults(void);//number of new immigrants
virtual long 			immigrantGametes(void);//number of new immigrants
virtual long 			immigrantImmatures(void);//number of new immigrants
virtual void			InitializeANewGeneration(void);
virtual void 			Install (Individual* indiv);
virtual void 			Install (Individual* indiv, genderSet parentGender);//installs gametes; defaults to Install(individual)
virtual long double		intrinsicSexRatio(void);
virtual void			IPopulation (CoordinateType, long k, Ran3List&);
virtual int				IsAllOneSex(void);
//virtual void 			JuvenileDispersal(Ran3List& ran, double m=1.0,
//									   double kurtosisFactor=0.0);
virtual void 			JuvenileDispersal(Ran3List& ran, aList<long double>& dispersalInfo);
virtual long 			maleBabies(void){return maleBabies_.len();}
virtual Individual& 	maleBaby(long i){return maleBabies_[i];}
virtual long 			maleGametes(void){return maleGametes_.len();}
virtual Individual& 	maleGamete(long i){return maleGametes_[i];}
virtual long 			males(void){return males_.len();}
virtual Individual& 	male(long i){return males_[i];}
virtual migrationModelSet migrationModel(void);
virtual long double 	MigrationProb(int ageClass, int currentHabitat, int potentialHabitat);
virtual long 			mothers(void){return asexuals()+herms()+females();}//potential female-role parents
virtual Individual& 	mother(long i);//out of potential female-role parents
virtual long double		moveRate(genderSet sex, int age);
//virtual void 			NaturalSelection(Ran3List& ran, selectionSourceSet sSource);
//virtual void			NaturalSelection (ObjList<Individual>& theList, Ran3List& ran,
//								   selectionSourceSet sSource);
virtual void			PrintAgeDistribution(ostream& outFile, int maxage);
virtual void			PrintPopHeading(ostream& outFile);
virtual void 			RandomDeathOfAdult(Ran3List& ran, int numToDie);
virtual void			Recruit(Ran3List& ran);
virtual void			Recruit(long babyIndex);
virtual void			ReplaceTheDead (Ran3List& ran);
//virtual void			Reproduce (long& idCount, Ran3List& ran);
virtual void			Reproduce (long& idCount, Ran3List& ran, aList<long double>& specialInfo);
virtual void			ReproduceBySyngamy (long& idCount, Ran3List& ran, aList<long double>& specialInfo);
//virtual double 			selectionStrength(void){/*override in derived classes*/
//						return selectionStrength_;}
		void			SetID (int IDnum) {id_ = IDnum;}
virtual void			SetReproductiveStatus(void);
virtual GridCell&		site(void);
virtual long			size(void);
virtual void 			SomeAdultsDie (aList<long double>& fSurvivalSchedule,
									aList<long double>& mSurvivalSchedule, Ran3List& ran);
virtual void 			SomeGametesDie (aList<long double>& fSurvivalSchedule,
									aList<long double>& mSurvivalSchedule, Ran3List& ran);
virtual void 			SomeImmaturesDie (aList<long double>& fSurvivalSchedule,
									aList<long double>& mSurvivalSchedule, Ran3List& ran);
virtual int			species(void) {return species_;}
		long 			time(void);// returns time from Landscape via GridCell
virtual int			NoInconsistentSettings(void);  
							/*returns an error code if settings are inconsistent*/
virtual void			ViabilitySelection (Ran3&){}

friend int 		operator== (Population& x, Population& y){
							return (x.id_==y.id_);}

friend int 		operator!= (Population& x, Population& y){
							return (!(x==y));}

	/* SOME VIRTUAL FUNCTIONS TO BE CALLED ONLY BY DERIVED GENETIC CLASSES*/
	/* these return nonsense values, or false, or 0   */
/*virtual double		AlleleFrequency (int, int){return (-2.0);}
//virtual int			AllelePresent (int, int){return 0;}
//virtual int			AllelePresent (ObjList<Individual>&, int,int){return 0;}
virtual long			Alleles (int){return 0;}
//virtual int			AlleleValue (int, int){return 0;}
virtual void			AssignMutations(void){}
virtual void			CalcAlleleFrequencies(void){}
virtual void			CalcExpHeterozygosity(void){}
virtual void			CalcObsHeterozygosity(void){}
virtual long			CountAlleles (int, int){return 0;}
virtual long			CountAlleles (ObjList<Individual>&, int, int){return 0;}
virtual long			CountHets (int, int){return 0;}
virtual long			CountHets (ObjList<Individual>&, int,int){return 0;}
virtual long			CountHets (int){return 0;}
virtual long			CountHets (ObjList<Individual>&, int){return 0;}
virtual double		Disequilibrium(void){return (-2.0);}
virtual double		Disequilibrium(int, int){return (-2.0);}
virtual double		Fis(void){return (-2.0);}
virtual double		Fis(int){return (-2.0);}
virtual double		Fis(int, int){return (-2.0);}
virtual double		hetExp (int, int){return (-2.0);}
virtual double		hetExp (int){return (-2.0);}
virtual double		hetObs (int, int){return (-2.0);}
virtual double		hetObs (int){return (-2.0);}
virtual int			ploidy (int){return 0;}
virtual void			PrintAlleleFreqs(ostream&){}
virtual void			PrintAlleleFreqs(ostream&, int){}
virtual void			PrintAlleleFreqs(ostream&, int, int){}
virtual void			PrintAlleleList(ostream&){}
virtual void			PrintAlleleList(ostream&, int){}
virtual void			PrintGeneStrings(ostream&, int, int){}
virtual void			PrintGeneStrings(ostream&, ObjList<Individual>&,int, int){}
virtual void			PrintHetExp(ostream&){}
virtual void			PrintHetExp(ostream&, int){}
virtual void			PrintHetObs(ostream&){}
virtual void			PrintHetObs(ostream&, int){}
virtual void			SampleSize (void){}
virtual long			SampleSize (int){return 0;}
virtual long			SampleSize (int, ObjList<Individual>&){return 0;}
virtual void			TallyAlleleFrequencies(ObjList<Individual>&){}
virtual void			UpdateAlleleList(void){}
virtual void			UpdateAlleleList(int, int){}
virtual void			UpdateAlleleList(ObjList<Individual>&){}
virtual void			UpdateAlleleList(Individual&){}
virtual void			UpdateAlleleList(Genome<int>&){}
*/

/*SOME VIRTUAL FUNCTIONS TO BE CALLED BY DERIVED DIOECIOUS CLASSES*/
virtual long double sexRatio(void){return -1.0;}/*no sexes in base class*/
virtual long double adultSexRatio(void){return -1.0;}/*no sexes in base class*/
virtual long double offspringSexRatio(void){return -1.0;}/*no sexes in base class*/
		
	};	/*end Population*/


#endif
