/**************************************************** 
* 	PopulationIntf.h
* 
*	interface for the Population and Individual classes, 
* 	the body of the LandScape program. 
* 
*	
*****************************************************/

#ifndef PopulationIntfh
#define PopulationIntfh = 1

#include "RandomList.h"
#include "aError.h"
#include "StringClass.h"
#include "CoordinateClass.cpp"

class CoordinateType;


  	
class CoordinateType: public CoordinateClass<int> {
	public:
	CoordinateType(void):CoordinateClass<int>(){}
 	CoordinateType(int x, int y=0, int z=0):CoordinateClass<int>(x,y,z){}
	CoordinateType(CoordinateType& ct):CoordinateClass<int>(ct){}
	CoordinateType(CoordinateClass<int>& ct):CoordinateClass<int>(ct){}
	~CoordinateType(void){}

	CoordinateType* Duplicate(void){
			return NEW CoordinateType();}
			
	friend int operator== (CoordinateType& a, CoordinateType& b);
	friend int operator!= (CoordinateType& a, CoordinateType& b);
			
  	};//end class CoordinateType*/
			
			

	enum lifeCycleSteps {age, 
						 disperse, 
						 adultsDisperse,
						 babiesDisperse,
						 gametesDisperse,
						 mate, 
						 recruitToAdult,
						 reproduce, //meiosis in gamete-bearing classes
						 reproduceSyngamy, //syngamy in gamete-bearing classes
						 viabilitySelection, 
						 adultMortality, 
						 immatureMortality, 
						 gameteMortality, 
						 clearOffspringLists,
						 replaceDead, 
						 buryDead};
				//feed a list of these to LandScape::DoOneCycle(...)

	enum lifeStageType {gamete, zygote, juvenile, adult};//others may be added as needed


	/*the kinds of mating systems available:*/
	enum matingSystemSet {clonal, 					/* only females reproduce, by cloning*/
						   selfing, 				/* only hermaphrodites reproduce, by self fertilization*/
						   idealHermaphrodite,		/* random mating, including selfing with prob 1/N*/
						   strictMonogamy,			/* each male and female mates once and only once*/
						   femaleMonogamy,			/* males can mate randomly, but females mate only once*/
						   maleMonogamy,			/* males mate only once; females can mate multiply.*/
						   polygamyWMixing,			/* multiple mating by both sexes, and sperm mixing occurs*/
						   polygamyWLastPrecedence};/* multiple mating; last male to mate sires offspring*/

	enum migrationModelSet {island, 	/*migrants have equal prob of going 
										to any other population in group;
										may be modified by habitat quality or other rules*/
							 steppingStone,	/*migrants have equal probability 
														of going to any adjacent population;
										may be modified by habitat quality or other rules*/
							 diffusion,		/*migrants go a number of grid cells 
							 						determined from normal distribution,
												with mean and sigma required for each population;
										may be modified by habitat quality or other rules*/
							 kurtotic,	/*migrants follow above diffusion rules, but sigma
							 				is determined from a normal distribution each move;
										may be modified by habitat quality or other rules*/
							 weibull};  /*migrants diffuse according to a Weibull distribution*/

	
	enum genderSet {	A=0, /*=Asexual female*/
					B=1, /*=both (=hermaphrodite)*/
					F=2, /*=female*/
					M=3 /*=male*/
					};

//these functions return a genderSet-formatted gender given a text-formatted value
genderSet genderCode(char genChar);

genderSet genderCode(aString& genStr);

genderSet genderCode(char* rawGenStr);
	
	



//	enum selectionSourceSet {	intrinsic, /*genetic background, epistasis*/
//				    		extrinsic,  /*ecological*/
//				    		IntrinsicAndExtrinsic,  /*both */
//				    		frequencyDependence  
//				    };




#endif
