/******************************************
*
*	Randomness.h
*
*	handles 4-bit random number classes
*
******************************************/

//2001.08.08  modified to store initial seed
//2001.08.08  modified to store last returned value

#ifndef Randomnessh
#define Randomnessh = 1


#include <math.h>
#include "InlineTemplates.h"
#include "aError.h"
#include "Gamma.h"
#include "MemoryManager.h"

class Ran3 {
	//4-bit pseudorandom number generator based on Pascal Recipes p. 221
	//0.0  result  1.0
  	//default constructor always yields the same sequence
  protected:
  	const long initialSeed_;
  	long long double lastReturned_;
  private:
  	const long double mbig_;	//4-bit: const double 
  	const long double mseed_;	//4-bit: const double 
  	const long double mz_;		//4-bit: const double 
  	const long double fac_;		//4-bit: const double 
  	long double *ran3ma_;		//4-bit: double 
  	int idum_;				
  	int ran3Inext_, ran3Inextp_, gasdevIset_;  //int
  	long double gasdevGset_;		//4-bit:double 
  public:
  	Ran3(int idum=(-1001)) :initialSeed_(-idum),
  						 lastReturned_(-one),
  						 idum_(idum),
  						 mbig_(1000000000),	//4-bit integers & float
  						 mseed_(1618033.0), //4-bit integers & float
  						 mz_(zero),
  						 fac_(1.0e-9),	//4-bit integers & double
  						 ran3ma_(NEW long double[55]),	//4-bit integers & double
  						 gasdevIset_(0),
  						 gasdevGset_(zero) {
  				IRan3();}
	virtual ~Ran3(void){
		delete [] ran3ma_;ran3ma_=0;
		}

	int 	BernoulliDev(const long double p);//0p1
					//returns 0 if a random number p and 1 otherwise
	long double  BetaDev(const long double a, const long double b);
					//returns a random number from a Beta distribution
	long double 	CauchyDev(void);
					//returns a random number from a Cauchy distribution
	long double 	ChiSquareDev(long double v);
					//returns a random number from a Chi-Square distribution 
					//with degrees of freedom = v
	double 	doubleBetween (double bottom, double top);
	long double  GammaDev(const long double alpha, const long double beta);
					//returns a random number from a Gamma distribution	
	long double GasDev (void);//standard normal deviate
	long double  gkm1Gamma(const long double b, const long double c);//intermediate calc in GammaDev
	long double  gkm2Gamma(const long double k, const long double b, const long double c);//intermediate calc in GammaDev
	long double	gsGamma(const long double alpha, const long double r);//intermediate calc in GammaDev
  	void 		IRan3(void);
  	long double lastReturned(void){return lastReturned_;}
	long 		longBetween (long bottom, long top);
	long double longdoubleBetween (long double bottom, long double top);
	long double LogNormalDev(const long double mu, const long double sigma);
				//returns a random number from a log-normal distribution
				//with mean = mu and st.dev = sigma
  	long double next(void);//uniform distribution 0.0 <= x < 1.0
	long double NormDev (long double mu, long double sigma); 
					//returns a random normal deviate 
					//from dist. of mean=mu and st.dev=sigma
	int 	PoissonDev(const long double mu);
					//returns a random integer from a Poisson distribution with mean mu
	long double		RandomProb(void) {return next();}//here for compatibility
	long double		seed(void) {return initialSeed_;}
	long double		WeibullDev(long double kappa, long double lambda, long double theta=zero);









	
	
	long double operator() (void) {return next();}

#ifdef MemoryLeakDebug
	static void PrintIDList(ostream& outFile=cout);
	static void PrintIDListLen(ostream& outFile=cout);
#endif
  	};//end Ran3
  	

#endif
