/******************************************************
*
*	StringClass
*
*	Implements a string class, from C++ FAQs book, FAQ#459
*	Additional functions added.
*
*
********************************************************/

//20020906 modified numerical constructors to return 'undefined' for NAN errors
//20030806 included support for long double conversions
//20071120 removed obsolete code using class MemoryLeakDebug
//20100208 revised [double/float/long double]-to-string conversion

#ifndef StringClassh
#define StringClassh = 1


#include <string>
#include <string.h>
#include <iostream.h>
#include <sstream.h>
#include <fenv.h>
#include <math.h>
#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "InlineTemplates.h"
#include "MemoryManager.h"

//#ifdef __MACOS__
//const char endchr='\r';
		//MACOS9 needs '\r' for and endofline character
//#else
//const char endchr='\n';
//#endif
//#ifdef __applelinker__
const char endchr='\n';
		//MACOSX needs '\n' for and endofline character
//#endif 



int Digits (long i);//defined in StringClass.cp

//int max(int,int);

class aString {


  private:
  	unsigned len_;/*ORDER DEPENDENCY, DON'T SWITCH THESE*/
  	char* data_;


  public:
  	aString (void) :len_(0), data_(NEW char[1]) {
  		data_[0] = '\0';}
  	
  	aString(const char* s): len_(strlen(s)), data_(NEW char[len_+1]){
  		memcpy(data_, s, len_+1);}
  		
  	aString(const char c): len_(1), data_(NEW char[len_+1]){
  		data_[0]=c; data_[1]='\0';}
  		
  	aString(int n);//converts int to aString
  	
  	aString(long n);//converts long to aString
  	
  	aString(const float n, int preDecPlaces=0, int decPlaces=6);
  			//converts float to aString
  	
  	aString(const double n, int preDecPlaces=0, int decPlaces=6);
  			//converts double to aString
  		
  	aString(long double n, int preDecPlaces=0, int decPlaces=10);
  			//converts long double to aString
  		
virtual ~aString(void) {
		delete [] data_;data_=0;
		}
  	
	aString(const aString& s);//copy constructor
  		
	aString(const aString* s);//copy constructor
  		
  		
	aString(const aString& s, const int len);
  		//use for trunctation or elongation
  	
	aString(aString* s, const int len);
  		//use for trunctation or elongation
  	
	aString(aString& s, const int start, const int fin);
  		//use to copy out a part of a string
  	
	aString(aString* s, const int start, const int fin);
  		//use to copy out a part of a string
  	
	aString(const char c, const int len);
		//use to make a string repeating character c to length len 

	aString(int len, Ran3& ran);//creates string of random characters
	
	
	
virtual void AppendCharToLen (char c, int newLen);
	
virtual void cat(const aString& s);//concatenate s to current string
  	
virtual void cat(const char c){//concatenate c to current string
				aString chrStr(c); cat(chrStr);}

virtual void cat(const char *c){//concatenate *c to current string
				aString chrStr(c); cat(chrStr);}

virtual void cat(const int c){//concatenate c to current string
				aString s(c); cat(s);}

virtual void cat(const long c){//concatenate c to current string
				aString s(c); cat(s);}

virtual void cat(const float c, int preDecPlaces=0, int decPlaces=6){
				//concatenate c to current string
				aString s(c,preDecPlaces, decPlaces); cat(s);}

virtual void cat(const double c, int preDecPlaces=0, int decPlaces=6){
				//concatenate c to current string
				aString s(c,preDecPlaces, decPlaces); cat(s);}

virtual void cat(const long double c, int preDecPlaces=0, int decPlaces=10){
				//concatenate c to current string
				aString s(c,preDecPlaces, decPlaces); cat(s);}

virtual void catChar(const char c, int n){//concatenate n copies of c to current string
		for(int i=0;i<n;++i){cat(c);}}

virtual aString* ChopOffFirstWord (void);
		// returns a new string pointer which must be deleted elsewhere				
			//splits off and returns the first word in the string
			//default endCharList = " \t,.;:\"(){}[]!?"
	
virtual aString* ChopOffFirstWord (aString& endCharList);
		// returns a new string pointer which must be deleted elsewhere				
			//splits off and returns the first word in the string
	
virtual aString* ChopOffLastWord (void);
		// returns a new string pointer which must be deleted elsewhere				
			//splits off and returns the last word in the string
			//default endCharList = " \t,.;:\"(){}[]!?"
	
virtual aString* ChopOffLastWord (aString& endCharList);
		// returns a new string pointer which must be deleted elsewhere				
			//splits off and returns the last word in the string
	
virtual void CullChar (char c);
	
virtual void CullLeadingChar (char c);
	
virtual void CullTrailingChar (char c);
	
virtual void CullLeadingSpaces (void){
					CullLeadingChar(' ');}
	
virtual void CullTrailingSpaces (void){
					CullTrailingChar(' ');}
	
virtual void CullLeadingWhiteSpace (void);//tabs and spaces
	
virtual void CullTrailingWhiteSpace (void);
	
virtual void cut(int loc){//deletes the character at location loc
					cut(loc,loc);}

virtual void cut(int start, int fin);
		//cut chars from start to fin, inclusive, from current string
		  
  	
virtual void clear(void){trunc(0);}
		//clears the string down to zero len	

virtual aString* Duplicate(void){
			return NEW aString(*this);}

virtual void invert(void);
			//reverses the order of characters in the string	

virtual int pos(const aString& s);

virtual int pos(const char* c){aString chrStr(c); return pos(chrStr);}

virtual int pos(const char c){aString chrStr(c); return pos(chrStr);}

virtual int posEndFirstWord(void);

virtual int posEndFirstWord(aString& endCharList);

virtual void prepend(const aString& insertedStr);
		//prepends the inserted string

virtual void prepend(const char *c){//prepends the char*
				aString chrStr(c); prepend(chrStr);}
	
virtual void prepend(const char c){//prepends the char
				aString chrStr(c); prepend(chrStr);}
	
virtual void prepend(const int c){//prepends the value
				aString s(c); prepend(s);}
	
virtual void prepend(const long c){//prepends the value
				aString s(c); prepend(s);}
	
virtual void prepend(const float c, int decPlaces=6){
				//prepends the value
				aString s(c,decPlaces); prepend(s);}
	
virtual void prepend(const double c, int decPlaces=6){
				//prepends the value
				aString s(c,decPlaces); prepend(s);}
	
virtual void PrependCharToLen (char c, int newLen);

virtual char* raw(void){//the raw form, just the character array
				char* rawForm = NEW char[len_+1];
				memcpy(rawForm, data_, len_+1);
				return rawForm;}

virtual void replace (aString& insertedStr, int start, int fin);
	
virtual void replace (const char *c, int start, int fin){
				aString chrStr(c); replace(chrStr,start,fin);}

virtual void replace (const char c, int start, int fin){
				aString chrStr(c); replace(chrStr,start,fin);}

virtual void replace (const int c, int start, int fin){
				aString s(c); replace(s,start,fin);}

virtual void replace (const long c, int start, int fin){
				aString s(c); replace(s,start,fin);}

virtual void replace (const float c, int start, int fin, int decPlaces=6){
				aString s(c,decPlaces); replace(s,start,fin);}

virtual void replace (const double c, int start, int fin, int decPlaces=6){
				aString s(c,decPlaces); replace(s,start,fin);}
				
virtual void replaceAll(const char oldChar, const char newChar);
			//replaces all instances of oldChar with newChar

virtual void separator(const int numSpaces, char sepChar = '\0');
			//appends sepchar if sepChar != ''
			//appends spaces to numSpaces otherwise

virtual double toDouble(void);//converts string to double

virtual long double toLongDouble(void);//converts string to double

virtual float toFloat(void){//converts string to float
	return (float) toDouble();}//should check for errors

virtual int toInt(void){//converts string to integer
	return (int) toLong();}//should check for errors


virtual long toLong(void);//converts string to long integer

virtual void trunc (int newLen);//truncate a string

virtual int isNumber(void);

virtual int isDecimal(void);

virtual aString& operator+(const aString& a);
virtual aString& operator+(const char* a);
virtual aString& operator+(const char a);
  		
virtual aString& operator+=(const aString& a){cat(a); return *this;}
virtual aString& operator+=(const char* a){cat(a); return *this;}
virtual aString& operator+=(const char a){cat(a); return *this;}
virtual aString& operator+=(const int a){cat(a); return *this;}
virtual aString& operator+=(const long a){cat(a); return *this;}
  		
virtual const unsigned len() {return len_;}
  	
virtual aString& operator= (const aString& s);
  	
virtual aString& operator= (const char* c);
  	
virtual aString& operator= (const char c);
  	
virtual aString& operator= (const int n);
  	
virtual aString& operator= (const long n);
  	
virtual aString& operator= (const float n);
  	
virtual aString& operator= (const double n);
  	
virtual aString& operator= (const long double n);
  	
//  	char& operator[] (unsigned i) {return data_[i];}
  			//can't handle bad index!
virtual char  operator[] (unsigned i) {return data_[i];}
virtual char  operator[] (int i) {return data_[i];}
  			//can't handle bad index!
  			
  	friend ostream& operator<< (ostream& o, const aString& s){
  		return o.write(s.data_, s.len_);}

  	friend ostream& operator<< (ostream& o, const aString* s){
  		return o.write(s->data_, s->len_);}

friend int operator== (const aString& a, const aString& b){
  		return ((a.len_== b.len_) && (memcmp(a.data_, b.data_, a.len_) == 0));}
  		
friend int operator== (const aString* a, const aString& b){
  		return ((a->len_== b.len_) && (memcmp(a->data_, b.data_, a->len_) == 0));}
  		
friend int operator== (const aString& a, const aString* b){
  		return ((a.len_== b->len_) && (memcmp(a.data_, b->data_, a.len_) == 0));}
  		
friend int operator== (const aString& a, const char* c){
  		aString b(c);
  		return (a==b);}
  		
friend int operator== (const aString& a, const char c){
  		aString b(c);
  		return (a==b);}
  		
friend int operator!=  (const aString& a, const aString& b){
  		return !(a==b);}
  		
friend int operator!=  (const aString* a, const aString& b){
  		return !(a==b);}
  		
friend int operator!=  (const aString& a, const aString* b){
  		return !(a==b);}
  		
friend int operator!=  (const aString& a, const char* c){
  		aString b(c);
  		return !(a==b);}
  		
friend int operator!=  (const aString& a, const char c){
 		aString b(c);
   		return !(a==b);}

friend int operator< (const aString& a, const aString& b){
		for(int i=0;i<a.len_;++i){
			if (b.len_==i){return 0;}
			if (a.data_[i]<b.data_[i]){return 1;}}
		return 0;}

friend int operator< (const aString* a, const aString& b){
		for(int i=0;i<a->len_;++i){
			if (b.len_==i){return 0;}
			if (a->data_[i]<b.data_[i]){return 1;}}
		return 0;}

friend int operator< (const aString& a, const aString* b){
		for(int i=0;i<a.len_;++i){
			if (b->len_==i){return 0;}
			if (a.data_[i]<b->data_[i]){return 1;}}
		return 0;}

friend int operator< (const aString& a, const char* c){
 		aString b(c);
		return (a<b);}

friend int operator< (const aString& a, const char c){
 		aString b(c);
		return (a<b);}

friend int operator> (const aString& a, const aString& b){
	return !((a<b) || (a==b));}

friend int operator> (const aString* a, const aString& b){
	return !((a<b) || (a==b));}

friend int operator> (const aString& a, const aString* b){
	return !((a<b) || (a==b));}

friend int operator> (const aString& a, const char* c){
 		aString b(c);
		return !((a<b) || (a==b));}

friend int operator> (const aString& a, const char c){
 		aString b(c);
		return !((a<b) || (a==b));}

friend int operator<= (const aString& a, const aString& b){
		return !(a>b);}

friend int operator<= (const aString* a, const aString& b){
		return !(a>b);}

friend int operator<= (const aString& a, const aString* b){
		return !(a>b);}

friend int operator>= (const aString& a, const aString& b){
		return !(a<b);}

friend int operator>= (const aString* a, const aString& b){
		return !(a<b);}

friend int operator>= (const aString& a, const aString* b){
		return !(a<b);}

  	
  	};//end aString



//class StringList is a sortable list
//class ObjList<aString>;  		
  		


#endif
