/************************************


	TextDatum.h
	
	This class holds data in text format, with a flag for missing data
	
*************************************/


#ifndef TextDatumh
#define TextDatumh=1


#include "StringClass.h"
#include "Randomness.h"
#include "MemoryManager.h"


class TextDatum: public aString{
	private:
		bool missingFlag_;//1 if missing, 0 if present
	
	public:
  	TextDatum(void):missingFlag_(true),aString() {}  	
  	TextDatum(const char* s):missingFlag_(false),aString(s){}
  		
  	TextDatum(const char c):missingFlag_(false),aString(c){}
  		
  	TextDatum(int n):missingFlag_(false),aString(n){}//converts int to TextDatum
  	
  	TextDatum(long n):missingFlag_(false),aString(n){}//converts long to TextDatum
  	
  	TextDatum(const float n, int preDecPlaces=0, int decPlaces=6):
  			missingFlag_(false),aString(n,preDecPlaces,decPlaces){}
  			//converts float to aString
  	
  	TextDatum(const double n, int preDecPlaces=0, int decPlaces=6):
  			missingFlag_(false),aString(n,preDecPlaces,decPlaces){}
  			//converts double to TextDatum
  		
  	TextDatum(const long double n, int preDecPlaces=0, int decPlaces=10):
  			missingFlag_(false),aString(n,preDecPlaces,decPlaces){}
  			//converts long double to TextDatum
  		
	TextDatum(const TextDatum& s, const int len):missingFlag_(s.missingFlag_),aString(s,len){}
  		//use for trunctation or elongation
  	
	TextDatum(TextDatum* s, const int len):missingFlag_(s->missingFlag_),aString(s,len){}
  		//use for trunctation or elongation
  	
	TextDatum(TextDatum& s, const int start, const int fin):missingFlag_(s.missingFlag_),aString(s,start,fin){}
  		//use to copy out a part of a string
  	
	TextDatum(TextDatum* s, const int start, const int fin):missingFlag_(s->missingFlag_),aString(s,start,fin){}
  		//use to copy out a part of a string
  	
	TextDatum(aString& s):missingFlag_(false),aString(s){}
  	
	TextDatum(aString& s, const int len):missingFlag_(false),aString(s,len){}
  		//use for trunctation or elongation
  	
	TextDatum(aString* s, const int len):missingFlag_(false),aString(s,len){}
  		//use for trunctation or elongation
  	
	TextDatum(aString& s, const int start, const int fin):missingFlag_(false),aString(s,start,fin){}
  		//use to copy out a part of a string
  	
	TextDatum(aString* s, const int start, const int fin):missingFlag_(false),aString(s,start,fin){}
  		//use to copy out a part of a string
  	
	TextDatum(const char c, const int len):missingFlag_(false),aString(c,len){}
		//use to make a string repeating character c to length len 

	TextDatum(int len, Ran3& ran):missingFlag_(false),aString(len,ran){}//creates string of random characters
  		

		//in case of missing data, might want to store it anyway, perhaps for later regurgitation or error checking
		TextDatum(aString& s,bool missing): missingFlag_(missing),aString(s){}
		TextDatum(char* c,bool missing): missingFlag_(missing),aString(c){}
		TextDatum(char& c,bool missing): missingFlag_(missing),aString(c){}

		TextDatum(TextDatum& input): missingFlag_(input.missingFlag_),aString(input){}//copy
		TextDatum(TextDatum* input): missingFlag_(input->missingFlag_),aString(input){}//copy
		~TextDatum(void){}
		
		TextDatum* Duplicate(void){//copy
			return NEW TextDatum(*this);}

		void clear(void){
			missing(true);
			aString::clear();}

		bool OK(void){
			return (! missingFlag_);}

		bool missing(void){
			return (missingFlag_);}
			
		void missing(bool tf){
			missingFlag_=tf;}
			
		TextDatum& operator=(const TextDatum& t){
			missingFlag_=t.missingFlag_;
			aString::operator=(t);
			return *this;}
					
		TextDatum& operator=(const aString& s){
			missingFlag_=false;
			aString::operator=(s);
			return *this;}
					
		TextDatum& operator=(const char* c){
			missingFlag_=false;
			aString::operator=(c);
			return *this;}
					
		TextDatum& operator=(const char c){
			missingFlag_=false;
			aString::operator=(c);
			return *this;}
					
		TextDatum& operator=(const int c){
			missingFlag_=false;
			aString::operator=(c);
			return *this;}
					
		TextDatum& operator=(const long c){
			missingFlag_=false;
			aString::operator=(c);
			return *this;}
					
		TextDatum& operator=(const float c){
			missingFlag_=false;
			aString::operator=(c);
			return *this;}
					
		TextDatum& operator=(const double c){
			missingFlag_=false;
			aString::operator=(c);
			return *this;}
					
		TextDatum& operator=(const long double c){
			missingFlag_=false;
			aString::operator=(c);
			return *this;}
					
		//need to sort data; missing data goes to end of any list
		friend int operator== (TextDatum& a, TextDatum& b){
				if(a.missing() && b.missing()) {return 1;}
		  		return ((a.OK() == b.OK()) && (aString(a) == aString(b)));}
		  		
		friend int operator!=  (TextDatum& a, TextDatum& b){
		  		return !(a==b);}
		  		
		friend int operator== (const TextDatum& a, const char& bb){
				TextDatum b(bb);
				return (a==b);}
		  		
		friend int operator!=  (const TextDatum& a, const char& bb){
		  		return !(a==bb);}
		  		
		friend int operator== (const TextDatum& a, const char* bb){
				TextDatum b(bb);
				return (a==b);}
		  		
		friend int operator!=  (const TextDatum& a, const char* bb){
		  		return !(a==bb);}
		  		
		friend int operator< (TextDatum& a, TextDatum& b){
				if(a.missing() && b.missing()) {return 0;}//treated as equal
				if(a.missing() && b.OK()) {return 0;}
				if(b.missing() && a.OK()) {return 1;}
				return (aString(a)<aString(b));}

		friend int operator> (TextDatum& a, TextDatum& b){
				if(a.missing() && b.missing()) {return 0;}//treated as equal
				if(a.missing() && b.OK()) {return 1;}
				if(b.missing() && a.OK()) {return 0;}
				return (aString(a)>aString(b));}

		friend int operator<= (TextDatum& a, TextDatum& b){
				return !(a>b);}

		friend int operator>= (TextDatum& a, TextDatum& b){
				return !(a<b);}
		
		
	};//end class TextDatum


#endif