/*******************************************
*
*
*	ANOVALine.cpp
*
*	A class that implements a line in an ANOVA table
*
*
************************************************/

#ifndef ANOVALinecp
#define ANOVALinecp = 1


#include "ANOVALine.h"
#include "GeneralLinearModel.h"
#include "ANOVATable.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "aVector.cpp"
#include "aMatrix.cpp"



void ANOVALine::CalcRegressionSS (StatList<long double>* dependentVarList,StatList<int>* independentVarList){
	//from Neder & Wasserman pp 37 & 80
	//the data table is already constructed; this calcs the regression and returns SS in the ANOVA Line
	if (dependentVarList->len() == independentVarList->len()){
			long double n = dependentVarList->len();
			long double sumX = independentVarList->sum();
			long double SSX = independentVarList->sumSqdVals() - (sumX * sumX) /  n;
			long double b1 = ((sumXY(*independentVarList, *dependentVarList) - (sumX * dependentVarList->sum())) / n) / SSX;
			SS_ = b1 * b1 * SSX;
			}
		else{
			SS_ = (long double)-1.0;}//impossible value indicates error
	}//end ANOVALine::CalcRegressionSS 





void ANOVALine::F(ANOVALineBase& denomLine, long bootRep){
	ANOVALine *denomLine_ = (ANOVALine*) &denomLine;
	if (bootRep == 0){
			if (denomLine_->MS_ > (long double)0.0){
					F_= MS_ / denomLine_->MS_;}
				else {F_ = (long double)0.0;}
			}
		else {//bootRep>0
			if (denomLine_->bootMS > (long double)0.0){
					bootF = bootMS / denomLine_->bootMS;}
				else{bootF=(long double)0.0;}
			bootFList.Append(bootF);}
	}//end ANOVALine::F(ANOVALineBase&, long)





void ANOVALine::Print (ostream& outFile, int titleLen, StringList& /*dependentList*/, long bootRep){
	//titleLen is the desired number of characters in the title line
	aString printTitle(title());
	if (title_.len() < titleLen){
			printTitle.AppendCharToLen(' ', titleLen);}
		else if (title_.len() > titleLen){
			printTitle.trunc(titleLen);}
	aString wholeLine(printTitle);
	wholeLine.separator(1);
	aString dfword(df());dfword.PrependCharToLen(' ',4);
	wholeLine.cat(dfword); wholeLine.separator(1);
	wholeLine.cat(SS(bootRep), 8,8);wholeLine.separator(1);
	if (effectType() !=  totalLine){
		wholeLine.cat(MS(bootRep), 8,4);wholeLine.separator(1);
		if (effectType() != errorLine){
			wholeLine.cat(F(bootRep), 6,4);wholeLine.separator(1);
			wholeLine.cat(P(), 2,6);wholeLine.separator(1);
			wholeLine.cat(MSDenomTitle());}
		}
	outFile<<wholeLine<<endl;
	}//end ANOVALine::Print



void ANOVALine::SS (long double val, long bootRep){
	if( bootRep == 0){SS_ = val;}
		else{bootSS = val;}
	}//end ANOVALine::SS


void ANOVALine::reducedFormSS (long double val, long bootRep){
	if( bootRep == 0){reducedFormSS_ = val;}
		else{reducedFormbootSS_ = val;}
	}//end ANOVALine::reducedFormSS


void ANOVALine::MS (long double val, long bootRep){
	if( bootRep == 0){MS_ = val;}
		else{bootMS = val;}
	}//end ANOVALine::MS



void ANOVALine::varianceComponent (long double val, long bootRep){
	if( bootRep == 0){varianceComponent_ = val;}
		else{bootvarianceComponent_ = val;}
	}//end ANOVALine::varianceComponent



double ANOVALine::sumXY (StatList<int>& Xlist, StatList<double>& Ylist){
	double sum = 0.0;
	if (Xlist.len() == Ylist.len()) {
		for(long i=0;i< Xlist.len();++i){
			sum += Xlist[i] * Ylist[i];}}
	return sum;
	}//end ANOVALine::sumXY 


long double ANOVALine::sumXY (StatList<int>& Xlist, StatList<long double>& Ylist){
	long double sum = 0.0;
	if (Xlist.len() == Ylist.len()) {
		for(long i=0;i< Xlist.len();++i){
			sum += Xlist[i] * Ylist[i];}}
	return sum;
	}//end ANOVALine::sumXY 


int operator== (ANOVALine& line1, ANOVALine& line2){
		return (line1.title() == line2.title());}

int operator!= (ANOVALine& line1, ANOVALine& line2){
		return !(line1 == line2);}


#endif
