/*******************************************
*
*
*	ANOVALineBase.cpp
*
*	A class that implements a line in an ANOVA table
*
*
************************************************/

#ifndef ANOVALineBasecpp
#define ANOVALineBasecpp = 1


#include "ANOVALineBase.h"
#include "ANOVATable.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "aVector.cpp"
#include "aMatrix.cpp"
#include "BootList.cpp"


void ANOVALineBase::AdjustSearchPosition (aVector<int>& presentPosition, aVector<int>& setDims, int loc){
	if (loc >= 0){
		int testVal = presentPosition[loc];
		++testVal;
		int templateVal = setDims[loc];
		if (testVal < templateVal){
				presentPosition[loc]=testVal;}
			else{
				presentPosition[loc]=0;
				AdjustSearchPosition(presentPosition, setDims, --loc);
				}//
		}//end if loc>=0
	}//end ANOVALineBase::AdjustSearchPosition 



void ANOVALineBase::CalcDesignAndDF (ObjList<StringList>& independentVarSet, dataSetRec& theDataSet, 
				aMatrix<long double>* designMatrix, aMatrix<long double>* incedenceMatrix, long n){
	//this creates a design matrix of (0,1,-1) values, an incedence matrix of (0,1) values,  
	//and determines df for main effects only

	//read the title and convert it to a list of string values
	StringList titleStrList;//these are the factors to use for the analysis
	aString theTitle = title();
	ConvertTitleString(theTitle, titleStrList);
	int numFactors = titleStrList.len();
	levelNames_.Clear();
	levelRows_.Clear();

	//first determine the search criteria
	aVector<int> searchCriteriaPresentPosition;//these are counters
	aVector<int> searchCriteriaSetDims;//these are counters
	int searchCategorySetLen = 0;		//the product of the levels of each factor
	for(int k=0;k<numFactors;++k){//set up the search list size
		//cycle through each combination of factor levels
		aString& factorStr =titleStrList[k];
		StringList& factorLevelList = theDataSet.GetFactorLevelList(factorStr);//the category names of the factor levels
		int numlevels = factorLevelList.len();
		searchCriteriaSetDims.Append(numlevels);
		searchCriteriaPresentPosition.Append(0);
		if (k == 0) {
				searchCategorySetLen = numlevels;
//				levels_ = numlevels;
				df_ =numlevels - 1;}
			else{
				searchCategorySetLen *=  numlevels;
//				levels_ *= numlevels;//assumes fully factorial model
				df_ *= numlevels - 1;}
		}//end for k

	//make a generic design vector
	aVector<long double> designVector((long double)0.0, searchCategorySetLen - 1);
	aVector<long double> altDesignVec((long double)-1.0, searchCategorySetLen - 1);
	aVector<long double> incedenceVector((long double)0.0, searchCategorySetLen);

	//make up search criteria for organizing design-vector information
	ObjList<StringList> searchCriteriaListSet;//these are the factors to use for the analysis
	for (int level=0;level<searchCategorySetLen;++level){
		//levels of interaction analyses are combinations of primary levels
		StringList* searchCriteriaList = NEW StringList();//these are the factors to use for the analysis
		for(int k=0;k<numFactors;++k){
			//generate the search criteria list for each level
			aString& factorStr = titleStrList[k];
			StringList& factorLevelList = theDataSet.GetFactorLevelList(factorStr);
			int loc = searchCriteriaPresentPosition[k];
//			aString* levelStr = NEW aString(factorLevelList[loc]);
			searchCriteriaList->Append(NEW aString(factorLevelList[loc]));
			}//end for k
		searchCriteriaListSet.Append(searchCriteriaList);
		searchCriteriaList = NULL;//forget this
		AdjustSearchPosition(searchCriteriaPresentPosition, searchCriteriaSetDims, numFactors-1);
		}//end for level

	for(int lev=0;lev<searchCriteriaListSet.len();++lev){
		levelRows_.Append(NEW aList<int>());
		aString *levname = NEW aString(searchCriteriaListSet[lev][0]);
		for(int fac=1;fac<searchCriteriaListSet[lev].len();++fac){//append additional factors as interactions 
			levname->cat("*"); levname->cat(searchCriteriaListSet[lev][fac]);}
		levelNames_.Append(levname);
		}//lev
	//use the search criteria to build the design matrix
	for (long i=0;i< n;++i){
		int testBool = 0;//false
		for (int level =0;level<searchCategorySetLen;++level){
			if (! testBool){
				StringList& searchCriteriaList = searchCriteriaListSet[level];
				testBool = MeetsSearchCriteria(i, searchCriteriaList, independentVarSet);
				if (testBool){//if true
					if (level == searchCategorySetLen-1){
							designMatrix->ConcatRow(altDesignVec);	//copy it to the matrix
							}
						else{
							aVector<long double> rowVec(designVector);
							rowVec[level] = 1.0;
							designMatrix->ConcatRow(rowVec);	//copy it to the matrix
							}
					aVector<long double> rowVecI(incedenceVector);
					rowVecI[level] = 1.0;
					incedenceMatrix->ConcatRow(rowVecI);	//copy it to the matrix
					levelRows_[level].Append(i);
					}//end if testBool
				}//end if ! testBool
			}//end for level
		}//end for i

	//free up the memory
	titleStrList.Clear();
	searchCriteriaListSet.Clear();
	}//end ANOVALineBase::CalcDesignAndDF



void ANOVALineBase::CalcDesignAndDFInteraction (ObjList<StringList>& independentVarSet, dataSetRec& theDataSet, 
									aMatrix<long double>* designMatrix, aMatrix<long double>* incedenceMatrix, 
									long n, aMatrix<long double>* Ymatrix){
	//this creates a design matrix of (0,1,-1) values, an incedence matrix of (0,1) values, 
	//and determines df for interaction effects only
	//read the title and convert it to a list of string values
	StringList titleStrList;
	aString theTitle(title());
	ConvertTitleString(theTitle, titleStrList);
	int numFactors = titleStrList.len();
	levelNames_.Clear();
	levelRows_.Clear();
	int k, kk, level;

	//first get the design matrices for each of the main effects & store in temp ANOVA table
	//then multiply out the levels to get the design matrix for the interaction effects
	ANOVATable tempANOVATable;
	for(k=0;k< numFactors;++k){//	set up the search list size
		ANOVALine* tempANOVALine = NEW ANOVALine();
		tempANOVALine->effectType(fixedEffect);
		tempANOVATable.Append(tempANOVALine);
		tempANOVALine = NULL;
		}//end for k

	MatrixList tempDesignMatrixComponents;
	MatrixList tempIncedenceMatrixComponents;
	for (k=0;k< numFactors;++k){
		aMatrix<long double>* tempFactorDesign = NEW aMatrix<long double>();
		aMatrix<long double>* tempIncedenceMatrix = NEW aMatrix<long double>();
		aString& factorString = titleStrList[k];
		tempANOVATable.SetTitle(factorString, k);
		ANOVALineBase& tempANOVALine = tempANOVATable.GetANOVALine(k);
		tempANOVALine.PrepForCalcs(theDataSet, tempFactorDesign, tempIncedenceMatrix,Ymatrix->rows(), Ymatrix);
//		tempFactorDesign->Print();
		tempDesignMatrixComponents.Append(tempFactorDesign);
		tempIncedenceMatrixComponents.Append(tempIncedenceMatrix);
		tempFactorDesign = NULL;
		tempIncedenceMatrix=NULL;
		}//end for k
	tempANOVATable.Clear();

	//this below does nothing except set up the search criteria dimensions, used later
	//leave it in for now, pare down later

	//first determine the search criteria
	aVector<int> searchCriteriaPresentPosition;
	aVector<int> searchCriteriaSetDims;
	int searchCategorySetLen = 0;	//the product of the levels of each factor
	for(k=0;k<numFactors;++k){//set up the search list size
		//cycle through each combination of factor levels
		aString& factorStr = titleStrList[k];
		StringList& factorLevelList = theDataSet.GetFactorLevelList(factorStr);
		int numLevels = factorLevelList.len();
		searchCriteriaSetDims.Append(numLevels);
		searchCriteriaPresentPosition.Append(0);
		if (k == 0){ 
				searchCategorySetLen = numLevels;
//				levels_ = numLevels;
				df_ = numLevels - 1;}
			else{
				searchCategorySetLen *= numLevels;
//				levels_ *= numLevels;
				df_ *= numLevels - 1;}
		}//end for k

	//make a generic design vector
	aVector<long double> designVector((long double)0.0, searchCategorySetLen-1);
	aVector<long double> altDesignVec((long double)-1.0, searchCategorySetLen-1);
	aVector<long double> incedenceVector((long double)0.0, searchCategorySetLen);


	ObjList<StringList> searchCriteriaListSet; 
	for (level=0;level< searchCategorySetLen;++level){
		//levels of interaction analyses are combinations of primary levels
		StringList* searchCriteriaList = NEW StringList();
		for (k=0;k<numFactors;++k){//generate the search criteria list for each level
			aString& factorStr = titleStrList[k];
			StringList& factorLevelList = theDataSet.GetFactorLevelList(factorStr);
			int loc = searchCriteriaPresentPosition[k];
//			aString* levelStr = NEW aString(factorLevelList[loc]);
			searchCriteriaList->Append(NEW aString(factorLevelList[loc]));
			}//end for k
		searchCriteriaListSet.Append(searchCriteriaList);
		searchCriteriaList = NULL;//forget this
		AdjustSearchPosition(searchCriteriaPresentPosition, searchCriteriaSetDims, numFactors-1);
		}//end for level

	for(int lev=0;lev<searchCriteriaListSet.len();++lev){
		levelRows_.Append(NEW aList<int>());
		aString *levname = NEW aString(searchCriteriaListSet[lev][0]);
		for(int fac=1;fac<searchCriteriaListSet[lev].len();++fac){//append additional factors as interactions 
			levname->cat("*"); levname->cat(searchCriteriaListSet[lev][fac]);}
		levelNames_.Append(levname);
		}//lev

	//use the search criteria to build a preliminary set of single-factor design matrices
	for(long i =0;i<n;++i){
		int testBool = false;
		for (level =0;level<searchCategorySetLen; ++level){
			if (! testBool){
				StringList& searchCriteriaList = searchCriteriaListSet[level];
				testBool = MeetsSearchCriteria(i, searchCriteriaList, independentVarSet);
				if (testBool){//if true
					if (level == searchCategorySetLen-1){
							designMatrix->ConcatRow(altDesignVec);	//copy it to the matrix
							}
						else{
							aVector<long double> rowVec(designVector);
							rowVec[level] = (long double)1.0;
							designMatrix->ConcatRow(rowVec);	//copy it to the matrix
							}
					aVector<long double> rowVecI(incedenceVector);
					rowVecI[level] = (long double)1.0;
					incedenceMatrix->ConcatRow(rowVecI);	//copy it to the matrix
					levelRows_[level].Append(i);
					}//end if testBool
				}//end if not testBool
			}//end for level
		}//end for i


	//now the business starts
	//recalculate the interaction terms in the design matrix by multiplying single-factor components together


 	//get the vector of values for each factor level
	//then multiply the vectors to get the interactions
	//finally, group these into one long row vector and put into the design matrix


	//Must multiply components out combinatorially in pairs, 3's, etc.,
	//out to the dimensions of numFactors.
	//This sets up the general list of combinations that must be multiplied
	ObjList<ObjList<aList<int> > > interactionTerms;
	int groupSize, slot1, slot2;
	for (groupSize=2; groupSize<=numFactors;++groupSize){
		//make empty list of interaction-term sublevels (2-way, 3-way, ..., numFactors-way)
		interactionTerms.Append(NEW ObjList<aList<int> >());}
	//always do the 2-way combinations
	ObjList<aList<int> >& termsForGroupList1 = interactionTerms[0];
	for (slot1 =0;slot1<numFactors - 1;++slot1){
		for (slot2 = slot1 + 1; slot2< numFactors;++slot2){
			aList<int>* combSet0 = NEW aList<int>();
			termsForGroupList1.Append(combSet0);
			combSet0->Append(slot1);
			combSet0->Append(slot2);
			combSet0 = NULL;//forget this
			}}//end for slot1 & for slot2
	for (groupSize = 3; groupSize<= numFactors;++groupSize){
//		ObjList<aList<int> >& termsForGroupList2 = interactionTerms[groupSize - 1]; 
//		ObjList<aList<int> >& termsForPrevGroupList = interactionTerms[groupSize - 2]; 
		ObjList<aList<int> >& termsForGroupList2 = interactionTerms[groupSize - 2]; 
		ObjList<aList<int> >& termsForPrevGroupList = interactionTerms[groupSize - 3]; 
		for(k=0;k<termsForPrevGroupList.len();++k){
			aList<int>& prevCombSet = termsForPrevGroupList[k];
			int lastVal = (int) prevCombSet.Last();
//			for (kk=lastVal; kk<= numFactors;++kk){
			for (kk=lastVal; kk< numFactors;++kk){
				aList<int>* combSet1 = NEW aList<int>(prevCombSet);
				combSet1->Append(kk);
				termsForGroupList2.Append(combSet1);
				combSet1 = NULL;
				}//end for kk
			}//end for k
		}//end for groupSize; all combinations now determined

	
	//Need to pick apart the design matrix.  First, 
	//make a list of starting coordinates for each factor level

	//Then make a new design matrix
	designMatrix->Clear();
/*	for (int i=0;i<n;++i){ //{for each individual in the data, build the design vector
		aVector<long double> newDesignVector;
		int j = interactionTerms.len()-1;
		ObjList<aList<int> >& termsForGroupList3 = interactionTerms[j];
		k = termsForGroupList3.len()-1;
		
		//start by getting all pairwise combinations of first combSet
		aList<int>& combSet3 = termsForGroupList3[k];
		slot1 = combSet3[0];
		aMatrix<long double>& tempFactorDesign1 = tempDesignMatrixComponents[slot1];
		aVector<long double>* tempVector1 = tempFactorDesign1.Row(i);
		aList<long double> tempVectorList3;
		for (kk=1;kk<combSet3.len();++kk){ 
			slot2 = combSet3[kk];
			aMatrix<long double>& tempFactorDesign2 = tempDesignMatrixComponents[slot2];
			aVector<long double>* tempVector2 = tempFactorDesign2.Row(i);
			for (int level1=0;level1<tempVector1->len();++level1){ 
				for (int level2=0;level2<tempVector2->len();++level2){
					long double val1 = (*tempVector1)[level1];
					long double val2 = (*tempVector2)[level2];
					long double val12 = val1*val2;  
					if(val12==((long double)-0.0)){
						val12=(long double)0.0;}
					tempVectorList3.Append(val12);
					}}//end for level1 && level2
			delete tempVector2; tempVector2=NULL;
			delete tempVector1; tempVector1=NULL;
			//replacing facilitates iteration when calculating for >2-length combSets
			tempVector1 = NEW aVector<long double> (tempVectorList3);
			tempVectorList3.Clear();//empties it out for reloading
			}//end for kk
		newDesignVector.Append(*tempVector1);
		delete tempVector1; tempVector1=NULL;


		designMatrix->ConcatRow(newDesignVector);
		}//end for i
*/
	while(tempDesignMatrixComponents.len()>1){
		aMatrix<long double> *interactionMatrix = tempDesignMatrixComponents[0].HorizontalDirectProduct(tempDesignMatrixComponents[1]);
		tempDesignMatrixComponents.DeleteFirst();
		tempDesignMatrixComponents.DeleteFirst();//delete the first two
		tempDesignMatrixComponents.Prepend(interactionMatrix);
		interactionMatrix=NULL;
		}
	*designMatrix = tempDesignMatrixComponents[0];

	//free up the memory
	tempDesignMatrixComponents.Clear();
	interactionTerms.Clear();
	titleStrList.Clear();
	searchCriteriaListSet.Clear();
	}//end ANOVALineBase::CalcDesignAndDFInteraction




void ANOVALineBase::CalcDesignAndDFNest (ObjList<StringList>& independentVarSet, dataSetRec& theDataSet, 
								aMatrix<long double>* designMatrix, aMatrix<long double>* incedenceMatrix, long n){
	//this does nested ANOVA line of the form '_____$(______$)'
	//partition the title
	StringList titleStrList;
	aString theTitle(title());
	ConvertTitleString(theTitle, titleStrList);
	aString  inParenStr, outsideParenStr;
	PartitionTitleString(theTitle, inParenStr, outsideParenStr);
	StringList outsideTitleStrList;
	ConvertTitleString(outsideParenStr, outsideTitleStrList);
	StringList insideTitleStrList;
	ConvertTitleString(inParenStr, insideTitleStrList);
	int numInsideFactors = insideTitleStrList.len();
	int numOutsideFactors = outsideTitleStrList.len();	//should always be 1
	levelNames_.Clear();
	levelRows_.Clear();

	//first determine the search criteria
	
	//First generate the search list and  means for the inside-parentheses factors,
	//then append the search list for the first outside-paren factors and calculate SS,
	//then replace the search list for the next outside-paren factor level and calculate, etc.
	ObjList<StringList> searchCriteriaSet;
	aVector<int> inSearchCriteriaPresentPosition;
	aVector<int> inSearchCriteriaSetDims;
	int inSearchCategorySetLen = 0;
	for (int k=0;k<numInsideFactors;++k){//set up the search list size for the in-parentheses factor
		//cycle through each combination of factor levels
		aString& inFactorStr = insideTitleStrList[k];
		StringList& inFactorLevelList = theDataSet.GetFactorLevelList(inFactorStr);
		int numlevels = inFactorLevelList.len();
//		if(k==0){levels_ = numlevels;}
//			else{levels_ *= numlevels;}
		inSearchCriteriaSetDims.Append(numlevels);
		inSearchCriteriaPresentPosition.Append(0);
		inSearchCategorySetLen += numlevels;
		}//end for k

	aVector<int> outSearchCriteriaPresentPosition;
	aVector<int> outSearchCriteriaSetDims;//each row is a search vector for a differnt factor
	int outSearchCategorySetLen = 0;
	for(int k=0;k<numOutsideFactors;++k){//set up the search list size for the outside-parentheses factor
		//cycle through each combination of outside factor levels 
		aString& outFactorStr = outsideTitleStrList[k];
		StringList& outFactorLevelList = theDataSet.GetFactorLevelList(outFactorStr);
		int numlevels = outFactorLevelList.len();
		outSearchCriteriaSetDims.Append(numlevels);
		outSearchCriteriaPresentPosition.Append(0);
		outSearchCategorySetLen += numlevels;
		}//end for k

	StringList* searchCriteriaList =NEW StringList();
	MatrixList desMatList;
	MatrixList incedenceMatrixComponents;//incedenceMatrix built from 'direct sum' of these components

	int startinglevel = 0;//cumulative levels up to this inLevel, for indexing later
	for (int inLevel=0;inLevel<inSearchCategorySetLen;++inLevel){
		//levels of interaction analyses are combinations of primary levels
		//for each in-paren factor, get the array of out-paren factor level values
		aMatrix<long double>* thisIncidenceComponent = NEW aMatrix<long double>();
		incedenceMatrixComponents.Append(thisIncidenceComponent);
		StringList currentInLevel;
		for(int k=0;k<numInsideFactors;++k){//generate the search criteria list for this level
			aString& inFactorStr = insideTitleStrList[k];
			StringList& inFactorLevelList =  theDataSet.GetFactorLevelList(inFactorStr);
			int loc = inSearchCriteriaPresentPosition[k];
//			aString* levelStr = NEW aString(inFactorLevelList[loc]);
			searchCriteriaList->Append(NEW aString(inFactorLevelList[loc]));
			currentInLevel.Append(NEW aString(inFactorLevelList[loc]));
			}//end for k


		for (int outLevel=0;outLevel<outSearchCategorySetLen;++outLevel){
			StringList currentOutLevel;
			for (int kk=0;kk<numOutsideFactors;++kk){
				aString& outFactorStr = outsideTitleStrList[kk];
				StringList& outFactorLevelList = theDataSet.GetFactorLevelList(outFactorStr);
				int loc = outSearchCriteriaPresentPosition[kk];
				aString* levelStr1 = NEW aString(outFactorLevelList[loc]);
				loc = titleStrList.Loc(&outFactorStr);//this gets them in the right order for later
				searchCriteriaList->Insert(levelStr1, loc);//add them here, delete before going to next inLevel
				currentOutLevel.Append(NEW aString(*levelStr1));
				levelStr1=NULL;//forget
				}//end for kk

			//copy the search string to a separate list
			//reset the original search criteria counter
			int testBool = 0;// false;	  testBool just checks for empty searches
			for (long i =0;i<n;++i){
				if (! testBool){
					testBool = MeetsSearchCriteria(i, *searchCriteriaList, independentVarSet);
					if (testBool){
						StringList* searchCriteriaToCopy = NEW StringList(searchCriteriaList);
						searchCriteriaSet.Append(searchCriteriaToCopy);
						searchCriteriaToCopy=NULL;//forget
						}//end if testBool
					}//end if not testBool
				}//end for i

			for (int kk=0;kk<numOutsideFactors;++kk){
				//delete the out-paren search string elements
				aString& outFactorStr = outsideTitleStrList[kk];
				StringList& outFactorLevelList = theDataSet.GetFactorLevelList(outFactorStr);
				int loc = outSearchCriteriaPresentPosition[kk];
				aString& levelStr2 =outFactorLevelList[loc];
				loc = searchCriteriaList->Loc(&levelStr2);//this gets them in the right order
				searchCriteriaList->Delete(loc);
				}//end for kk
			AdjustSearchPosition(outSearchCriteriaPresentPosition, outSearchCriteriaSetDims, numOutsideFactors-1);
			
			if(testBool){//individuals with this level exist 
				aString levNameIn(currentInLevel[0]);
				for(int lev=1;lev<currentInLevel.len();++lev){
					levNameIn.cat("*"); levNameIn.cat(currentInLevel[lev]);}
				levNameIn.prepend('(');levNameIn.cat(')');
				aString *levNameOut = NEW aString(currentOutLevel[0]);
				for(int lev=1;lev<currentOutLevel.len();++lev){
					levNameOut->cat("*"); levNameOut->cat(currentOutLevel[lev]);}
				levNameOut->cat(levNameIn);
				levelNames_.Append(levNameOut);
				levelRows_.Append(NEW aList<int>());
				}
			}//end for outLevel



		//make a generic design vector
		aVector<long double> designVector((long double)0.0, searchCriteriaSet.len() - 1);
		aVector<long double> altDesignVec((long double)-1.0, searchCriteriaSet.len() - 1);
		aVector<long double> incedenceVector((long double)0.0, searchCriteriaSet.len());
		
		aMatrix<long double>* tempDesignMatrix = NEW aMatrix<long double>();

		for (long i =0;i<n;++i){	//use the search criteria
			int testBool = 0;//False
			for (int level =0;level< searchCriteriaSet.len();++level){
				if (! testBool){
					StringList& criteriaList = searchCriteriaSet[level];
					testBool = MeetsSearchCriteria(i, criteriaList, independentVarSet);
					if (testBool){
						if (level == searchCriteriaSet.len()-1){
								tempDesignMatrix->ConcatRow(altDesignVec);}
							else{
								aVector<long double> rowVec (designVector);
								rowVec[level] = (long double)1.0;
								tempDesignMatrix->ConcatRow(rowVec);
								}

						aVector<long double> rowVecI (incedenceVector);
						rowVecI[level] = (long double)1.0;
						thisIncidenceComponent->ConcatRow(rowVecI);
						levelRows_[level+startinglevel].Append(i);//account for nesting structure
						}//end if testBool
					}//end if not testBool
				}//end for level
			if (! testBool){//no match was found, so enter a line of 0's
				tempDesignMatrix->ConcatRow(designVector);}
			}//end for i
		desMatList.Prepend(tempDesignMatrix);
		tempDesignMatrix = NULL;//forget this

		startinglevel += searchCriteriaSet.len();
		searchCriteriaSet.Clear();


		for (int kk=0;kk<numInsideFactors;++kk){//usually just 1
			//delete the in-paren search string elements
			aString& inFactorStr = insideTitleStrList[kk];
			StringList& inFactorLevelList = theDataSet.GetFactorLevelList(inFactorStr);
			int loc = inSearchCriteriaPresentPosition[kk];
			aString& levelStr3 = inFactorLevelList[loc];
			loc = searchCriteriaList->Loc(&levelStr3);//this gets them in the right order
			searchCriteriaList->Delete(loc);
			}//end for kk
		AdjustSearchPosition(inSearchCriteriaPresentPosition, inSearchCriteriaSetDims, numInsideFactors-1);
		thisIncidenceComponent=NULL;//forget
		}//end for inLevel

	designMatrix->Clear();
	aMatrix<long double>* tempDesignMatrix = MakeDesignMatrix(desMatList, 0);
//	tempDesignMatrix->Print("nest design");
	for(long ii=0;ii<tempDesignMatrix->rows();++ii){
		aVector<long double>& rowVec = (*tempDesignMatrix)[ii];
		designMatrix->ConcatRow(rowVec);}
	delete tempDesignMatrix; tempDesignMatrix = NULL;
	aMatrix<long double>* incedenceMatrixBasis = incedenceMatrixComponents.DirectSum();
	incedenceMatrix->Copy(incedenceMatrixBasis);
	delete incedenceMatrixBasis; incedenceMatrixBasis=NULL;

	df_ = designMatrix->cols();
//	levels_ += df_;

	//free up the memory
//	searchCriteriaList->Clear();
	delete searchCriteriaList; searchCriteriaList=NULL;
	insideTitleStrList.Clear();
	outsideTitleStrList.Clear();
	searchCriteriaSet.Clear();
	titleStrList.Clear();
	}//end ANOVALineBase::CalcDesignAndDFNest






void ANOVALineBase::CalcDesignAndDFComplexNest(/*ObjList<StringList>& independentVarSet,*/ dataSetRec& theDataSet, 
										aMatrix<long double>* designMatrix, 
									  aMatrix<long double>* incedenceMatrix, long n){
	//this does complex nested ANOVA line of the form '_______$ x _____$(______$ x _______$)'
	//decompose the within- and outside-parentheses sets of factors, and construct new independent variables from each
			//yields a direct-sum matrix (if sorted right), with an interaction-matrix block corresponding to each level of nesting
			//nesting levels are for each factor combination in the in-parentheses term
	//first partition the title
	StringList titleStrList;
	aString theTitle(title());
	ConvertTitleString(theTitle, titleStrList);
	aString  inParenStr, outsideParenStr;
	PartitionTitleString(theTitle, inParenStr, outsideParenStr);
	StringList outsideTitleStrList;
	ConvertTitleString(outsideParenStr, outsideTitleStrList);
	StringList insideTitleStrList;
	ConvertTitleString(inParenStr, insideTitleStrList);
	int numInsideFactors = insideTitleStrList.len();
	int numOutsideFactors = outsideTitleStrList.len();
	levelNames_.Clear();
	levelRows_.Clear();

	//Strategy: first determine the search criteria:
	//Generate the search list for the inside-parentheses factors,
	//then sequentially append the outside-paren factors and search for each separately.
	//This will give single-factor list for each inside-parentheses factor level.  
	//Accumulate such a list for each outside-parentheses factor.
	//For each of the inside-paren levels, multiply together the values of the ouside-perentheses factors
	//to get an interaction-term design matrix for each inside-parentheses factor level.
	//The last step will be to cobble these together into a single design matrix.

	ObjList<StringList> searchCriteriaSet;
	aVector<int> inSearchCriteriaPresentPosition;
	aVector<int> inSearchCriteriaSetDims;
	int inSearchCategorySetLen = 0;
	for (int k0=0;k0<numInsideFactors;++k0){//set up the search list size for the in-parentheses factor
		//cycle through each combination of factor levels
		aString& inFactorStr0 = insideTitleStrList[k0];
		StringList& inFactorLevelList0 = theDataSet.GetFactorLevelList(inFactorStr0);
		int numlevels0 = inFactorLevelList0.len();
//		if(k0==0){levels_ = numlevels0;}
//			else{levels_ *= numlevels0;}
		inSearchCriteriaSetDims.Append(numlevels0);
		inSearchCriteriaPresentPosition.Append(0);
		inSearchCategorySetLen += numlevels0;
		}//end for k0

	aMatrix<int> outSearchCriteriaPresentPosition(0,numOutsideFactors,1);//each row is a search vector for a different factor
	aMatrix<int> outSearchCriteriaSetDims(0,numOutsideFactors,1);//each row is a search vector for a different factor
	aList<int> outSearchCategorySetLen;
	for(int k1=0;k1<numOutsideFactors;++k1){//set up the search list size for the outside-parentheses factor
		//cycle through each combination of outside factor levels 
		aString& outFactorStr1 = outsideTitleStrList[k1];
		StringList& outFactorLevelList1 = theDataSet.GetFactorLevelList(outFactorStr1);
		int numlevels1 = outFactorLevelList1.len();
		outSearchCriteriaPresentPosition[k1][0] = 0;
		outSearchCriteriaSetDims[k1][0] = numlevels1;
		outSearchCategorySetLen.Append(numlevels1);
		}//end for k

	StringList* searchCriteriaList =NEW StringList();
	MatrixList desMatList;
	MatrixList incedenceMatrixComponents;

	for (int inLevel=0;inLevel<inSearchCategorySetLen;++inLevel){
		//levels of interaction analyses are combinations of primary levels
		//for each in-paren factor, get the array of out-paren factor level values
		aMatrix<long double>* thisIncidenceComponent = NEW aMatrix<long double>();
		incedenceMatrixComponents.Append(thisIncidenceComponent);
		ObjList<StringList> independentVarSetTemp;//for the next step
		StringList currentInLevel;
		for(int k3=0;k3<numInsideFactors;++k3){//generate the search criteria list for this level
			aString& inFactorStr3 = insideTitleStrList[k3];
			StringList& inFactorLevelList3 =  theDataSet.GetFactorLevelList(inFactorStr3);
			int loc3 = inSearchCriteriaPresentPosition[k3];
			searchCriteriaList->Append(NEW aString(inFactorLevelList3[loc3]));
			currentInLevel.Append(NEW aString(inFactorLevelList3[loc3]));

			dataColumnRec& indepVarListRec3 = theDataSet.GetDataColumn(inFactorStr3);
			dataColumnRecStr* indepVarList3 = (dataColumnRecStr*) &indepVarListRec3;
			independentVarSetTemp.Append(NEW StringList(indepVarList3->dataList()));
			}//end for k3


		//get a main-effect design matrix for each factor, as if it were nested separately
		MatrixList desMatListTemp(desMatList);
		MatrixList incedenceMatrixComponentsTemp;
		ObjList<StringList> outLevelNames;
		ObjList<ObjList<aList<int> > > outLevelRows;
		for (int k4=0;k4<numOutsideFactors;++k4){
			outLevelNames.Append(NEW StringList());
			outLevelRows.Append(NEW ObjList<aList<int> >());
			aMatrix<long double>* thisIncidenceComponentTemp = NEW aMatrix<long double>();
			incedenceMatrixComponentsTemp.Append(thisIncidenceComponentTemp);
			ObjList<StringList> searchCriteriaSetTemp(searchCriteriaSet);
			long i4=0;
			aString& outFactorStr4 = outsideTitleStrList[k4];
			StringList& outFactorLevelList4 = theDataSet.GetFactorLevelList(outFactorStr4);

			dataColumnRec& indepVarListRec = theDataSet.GetDataColumn(outFactorStr4);
			dataColumnRecStr* indepVarList = (dataColumnRecStr*) &indepVarListRec;
			StringList& indepVarColToCopy = indepVarList->dataList();
			independentVarSetTemp.Prepend(NEW StringList(indepVarColToCopy));


			StringList currentOutLevel;
			for (int outLevel=0;outLevel<outSearchCategorySetLen[k4];++outLevel){
				int loc4 = outSearchCriteriaPresentPosition[k4][0];
				aString* levelStr4 = NEW aString(outFactorLevelList4[loc4]);
				loc4 = titleStrList.Loc(&outFactorStr4);//this gets them in the right order for later
//				searchCriteriaList->Insert(levelStr4, loc4);//add them here, delete before going to next inLevel
				searchCriteriaList->Prepend(levelStr4);
				currentOutLevel.Append(NEW aString(*levelStr4));
				levelStr4=NULL;//forget
				//copy the search string, if valid, to a separate list
				int testBool = 0;// false;	  testBool just checks for empty searches
				for (i4=0;i4<n;++i4){
					testBool = MeetsSearchCriteria(i4, *searchCriteriaList, independentVarSetTemp);
					if (testBool){
						StringList* searchCriteriaToCopy = NEW StringList(searchCriteriaList);
						searchCriteriaSetTemp.Append(searchCriteriaToCopy);
						searchCriteriaToCopy=NULL;//forget
						break;
						}//end if testBool
					}//end for i4
				//reset the original search criteria counter
				loc4 = outSearchCriteriaPresentPosition[k4][0];
				aString& levelStr4a =outFactorLevelList4[loc4];
				loc4 = searchCriteriaList->Loc(&levelStr4a);//this gets them in the right order
				searchCriteriaList->Delete(loc4);
				AdjustSearchPosition(outSearchCriteriaPresentPosition[k4], outSearchCriteriaSetDims[k4], 0);
				if(testBool){//individuals with this level exist
					aString *levNameOut = NEW aString(currentOutLevel[outLevel]);
					outLevelNames[k4].Append(levNameOut);
					outLevelRows[k4].Append(NEW aList<int>());
					}
				}//end for outLevel


			//make a generic design vector
			aVector<long double> designVector4((long double)0.0, searchCriteriaSetTemp.len() - 1);
			aVector<long double> altDesignVec4((long double)-1.0, searchCriteriaSetTemp.len() - 1);
			aVector<long double> incedenceVector4((long double)0.0, searchCriteriaSetTemp.len());
			
			aMatrix<long double>* tempDesignMatrix4 = NEW aMatrix<long double>();

			for (i4 =0;i4<n;++i4){	//use the search criteria
				int testBool4 = false;
				for (int level=0;level<searchCriteriaSetTemp.len();++level){
						StringList& criteriaList4 = searchCriteriaSetTemp[level];
						testBool4 = MeetsSearchCriteria(i4, criteriaList4, independentVarSetTemp);
						if (testBool4){
							if (level == searchCriteriaSetTemp.len()-1){
									tempDesignMatrix4->ConcatRow(altDesignVec4);}
								else{
									aVector<long double> rowVec4 (designVector4);
									rowVec4[level] = (long double)1.0;
									tempDesignMatrix4->ConcatRow(rowVec4);
									}

							aVector<long double> rowVecI4 (incedenceVector4);
							rowVecI4[level] = (long double)1.0;
							thisIncidenceComponentTemp->ConcatRow(rowVecI4);
							outLevelRows[k4][level].Append(i4);
							break;
							}//end if testBool4
					}//end for level
				if (! testBool4){//no match was found, so enter a line of 0's
					tempDesignMatrix4->ConcatRow(designVector4);}
				}//end for i4
			desMatListTemp.Prepend(tempDesignMatrix4);
			tempDesignMatrix4 = NULL;//forget this
			searchCriteriaSetTemp.Clear();
			independentVarSetTemp.DeleteFirst();
			}//k4

		//recursively cobble together the actual level names from individual factor level names
		while(outLevelNames.len()>1){
			StringList *levelNamesTemp = NEW StringList();
			ObjList<aList<int> > *levelRowsTemp = NEW ObjList<aList<int> >();
			for(int outFacLev0=0;outFacLev0<outLevelNames[0].len();++outFacLev0){
				aString levNameOut0(outLevelNames[0][outFacLev0]);
				for(int outFacLev1=0;outFacLev1<outLevelNames[1].len();++outFacLev1){
					aString *levNameOut = NEW aString(levNameOut0);
					levNameOut->cat("*");
					levNameOut->cat(outLevelNames[1][outFacLev1]);
					levelNamesTemp->Append(levNameOut);
					levelRowsTemp->Append(NEW aList<int>());
					for(int r1=0;r1<outLevelRows[1][outFacLev1].len();++r1){
						//find rows that are in both factor level lists
						int row1 = outLevelRows[1][outFacLev1][r1];
						if(outLevelRows[0][outFacLev0].Includes(row1)){
							levelRowsTemp->Last().Append(row1);}
						}//r1
					if(levelRowsTemp->Last().len()==0){//no data, so drop it
							levelNamesTemp->DeleteLast();
							levelRowsTemp->DeleteLast();
							}
					}//outFacLev1
				}//outFacLev0
			outLevelNames.DeleteFirst();
			outLevelNames.DeleteFirst();//first two factors
			outLevelNames.Prepend(levelNamesTemp);
			outLevelRows.DeleteFirst();
			outLevelRows.DeleteFirst();//first two factors
			outLevelRows.Prepend(levelRowsTemp);
			}//while outLevelNames.len()
		aString levNameIn(currentInLevel[0]);
		for(int lev=1;lev<currentInLevel.len();++lev){
			levNameIn.cat("*"); levNameIn.cat(currentInLevel[lev]);}
		levNameIn.prepend('(');levNameIn.cat(')');
		for(int lev2=0;lev2<outLevelNames[0].len();++lev2){
			aString * levnam = NEW aString(outLevelNames[0][lev2]);
			levnam->cat(levNameIn);
			levelNames_.Append(levnam);
			levelRows_.Append(NEW aList<int>(outLevelRows[0][lev2]));
			}
					
			
			
		
		//multiply together the combinations of levels among factors to get the interaction design vector
//***
	 	//get the vector of values for each factor level
		//then multiply the vectors to get the interactions
		//finally, group these into one long row vector and put into the design matrix


		//Must multiply components out combinatorially in pairs, 3's, etc.,
		//out to the dimensions of numFactors.
		//This sets up the general list of combinations that must be multiplied
		ObjList<ObjList<aList<int> > > interactionTerms;
		int groupSize, slot1, slot2;
		for (groupSize=2; groupSize<=numOutsideFactors;++groupSize){
			//make empty list of interaction-term sublevels (2-way, 3-way, ..., numFactors-way)
			interactionTerms.Append(NEW ObjList<aList<int> >());}
		//always do the 2-way combinations
		ObjList<aList<int> >& termsForGroupList1 = interactionTerms[0];
		for (slot1 =0;slot1<numOutsideFactors - 1;++slot1){
			for (slot2 = slot1 + 1; slot2< numOutsideFactors;++slot2){
				aList<int>* combSet0 = NEW aList<int>();
				termsForGroupList1.Append(combSet0);
				combSet0->Append(slot1);
				combSet0->Append(slot2);
				combSet0 = NULL;//forget this
				}}//end for slot1 & for slot2
		for (groupSize = 3; groupSize<= numOutsideFactors;++groupSize){
			ObjList<aList<int> >& termsForGroupList2 = interactionTerms[groupSize - 2]; 
			ObjList<aList<int> >& termsForPrevGroupList = interactionTerms[groupSize - 3]; 
			for(int k5=0;k5<termsForPrevGroupList.len();++k5){
				aList<int>& prevCombSet = termsForPrevGroupList[k5];
				int lastVal = (int) prevCombSet.Last();
				for (int k6=lastVal; k6< numOutsideFactors;++k6){
					aList<int>* combSet1 = NEW aList<int>(prevCombSet);
					combSet1->Append(k6);
					termsForGroupList2.Append(combSet1);
					combSet1 = NULL;
					}//end for k6
				}//end for k5
			}//end for groupSize; all combinations now determined

		
		//Need to pick apart the design matrix.  First, 
		//make a list of starting coordinates for each factor level

		//Then make a new design matrix
		aMatrix<long double> *nestedDesMatrix = NEW aMatrix<long double>();
		for (long i5=0;i5<n;++i5){ //{for each individual in the data, build the design vector
			aVector<long double> newDesignVector;
			int j = interactionTerms.len()-1;
			ObjList<aList<int> >& termsForGroupList3 = interactionTerms[j];
			int k7 = termsForGroupList3.len()-1;
			
			//start by getting all pairwise combinations of first combSet
			aList<int>& combSet3 = termsForGroupList3[k7];
			slot1 = combSet3[0];
			aMatrix<long double>& tempFactorDesign1 = desMatListTemp[slot1];
			aVector<long double>* tempVector1 = tempFactorDesign1.Row(i5);
			aList<long double> tempVectorList3;
			for (int k8=1;k8<combSet3.len();++k8){ 
				slot2 = combSet3[k8];
				aMatrix<long double>& tempFactorDesign2 = desMatListTemp[slot2];
				aVector<long double>* tempVector2 = tempFactorDesign2.Row(i5);
				for (int level1=0;level1<tempVector1->len();++level1){ 
					for (int level2=0;level2<tempVector2->len();++level2){
						long double val1 = (*tempVector1)[level1];
						long double val2 = (*tempVector2)[level2];
						long double val12 = val1*val2;  
						if(val12==(long double)(-0.0)){val12=(long double)0.0;}
						tempVectorList3.Append(val12);
						}}//end for level1 && level2
				delete tempVector2; tempVector2=NULL;
				delete tempVector1; tempVector1=NULL;
				//replacing facilitates iteration when calculating for >2-length combSets
				tempVector1 = NEW aVector<long double> (tempVectorList3);
				tempVectorList3.Clear();//empties it out for reloading
				}//end for k8
			newDesignVector.Append(*tempVector1);
			delete tempVector1; tempVector1=NULL;


			nestedDesMatrix->ConcatRow(newDesignVector);
			}//end for i
		desMatList.Append(nestedDesMatrix);

//***
		//reset and go to the next level within parentheses
		for (int kk=0;kk<numInsideFactors;++kk){
			//delete the in-paren search string elements
			aString& inFactorStr = insideTitleStrList[kk];
			StringList& inFactorLevelList = theDataSet.GetFactorLevelList(inFactorStr);
			int loc = inSearchCriteriaPresentPosition[kk];
			aString& levelStr3 = inFactorLevelList[loc];
			loc = searchCriteriaList->Loc(&levelStr3);//this gets them in the right order
			searchCriteriaList->Delete(loc);
			}//end for kk
		AdjustSearchPosition(inSearchCriteriaPresentPosition, inSearchCriteriaSetDims, numInsideFactors-1);
		thisIncidenceComponent=NULL;//forget
		}//end for inLevel

	designMatrix->Clear();
	aMatrix<long double>* tempDesignMatrix = MakeDesignMatrix(desMatList, 0);
//	tempDesignMatrix->Print("nest design");
	for(long ii=0;ii<tempDesignMatrix->rows();++ii){
		aVector<long double>& rowVec = (*tempDesignMatrix)[ii];
		designMatrix->ConcatRow(rowVec);}
	delete tempDesignMatrix; tempDesignMatrix = NULL;
	aMatrix<long double>* incedenceMatrixBasis = incedenceMatrixComponents.DirectSum();
	incedenceMatrix->Copy(incedenceMatrixBasis);
	delete incedenceMatrixBasis; incedenceMatrixBasis=NULL;

	df_ = designMatrix->cols();
//	levels_ += df_;

	//free up the memory
//	searchCriteriaList->Clear();
	delete searchCriteriaList; searchCriteriaList=NULL;
	insideTitleStrList.Clear();
	outsideTitleStrList.Clear();
	searchCriteriaSet.Clear();
	titleStrList.Clear();
	}//end ANOVALineBase::CalcDesignAndDFComplexNest




void ANOVALineBase::ConvertTitleString (aString& theTitleStr, StringList& theList){
	//recursive, so clear the list before the first call
	aString xWord('x'), startParen('('), endParen(')');
	aString endCharList(" \t,;:\"(){}[]!?");
	aString theTitle(theTitleStr);//work with a copy because original is destroyed
	while (theTitle.len() > 0){
		if((startParen == theTitle[0])&&(endParen == theTitle[theTitle.len()-1])){//must be format (______$ x _______$)
				theTitle.cut(1);
				theTitle.cut(theTitle.len());
				aString* titleCopy = NEW aString(theTitle);
				theList.Append(titleCopy);//this will be disassembled in a later call to this algorithm
				theTitle = "";
				}
			else{
				aString* word = theTitle.ChopOffFirstWord(endCharList);
//				if (*word != xWord){//ignore the 'x'
				if (*word == xWord){//ignore the 'x'
						delete word; word=NULL;}
					else{
						if (startParen == (*word)[0]){
								//remove parentheses and later, handle nesting
								word->cut(0);
								if (endParen == (*word)[word->len()]){
									word->cut(word->len());}
								ConvertTitleString(*word, theList);//checks for multiple words inside parentheses
								delete word; word=NULL;
								}//end if startParen==first char
							else if (word->pos(startParen) > 0){ //handle the pieces separately
								aString part1(*word), part2(*word);
								 part1.cut(part1.pos(startParen), part1.len());//trims off last ')' too
								 part2.cut(0, part2.pos(startParen));
								 part2.trunc(part2.len()-1);
								ConvertTitleString(part1, theList);
								ConvertTitleString(part2, theList);
								delete word; word=NULL;
								}
							else	{//it's a regular word
								if (endParen == (*word)[word->len()]){
									word->cut(word->len());}
								theList.Append(word);}
						}//end if word != xWord
				}//end else
		}//end while
	}//end  ANOVALineBase::ConvertTitleString



void ANOVALineBase::F(ANOVALineBase& /*denomLine*/, long /*bootRep*/){
	aString err("Error: calling ANOVALineBase::F() by mistake to calculate F.");
	aError(err);
	}//end ANOVALineBase::F(ANOVALine*, long)




ANOVALineType ANOVALineBase::GetANOVALineType (aString& factorString){
	ANOVALineType answer = factorialLine;
	char startParen = '(';
	char endParen = ')';
	aString xWord = " x ";//note the blanks 
	if (factorString.pos(endParen) == 0){
			if (factorString.pos(xWord) > 0){
				answer = interactionLine;}}
		else	{//nesting is involved
			if (factorString.pos(xWord) == 0){
					answer = simpleNestLine;}
				else{
					answer = complexNestLine;}
			}
	return answer;
	}//end ANOVALineBase::GetANOVALineType



aMatrix<long double>* ANOVALineBase::MakeDesignMatrix (MatrixList& matrixList, int omitFactor){
	//matrix list comes back empty
//			tempMatrix1, tempMatrix2, finalMatrix: matrixPtrType;
	if (omitFactor != 0){//=0 in full model
		matrixList.Delete(omitFactor);}//for the reduced model
	aMatrix<long double>* finalMatrix = &(matrixList.Last());
	matrixList.DetachLast();	//detach it
	int j = matrixList.len();
	for(int i=0;i<j;++i){
		aMatrix<long double>* tempMatrix1 = &(matrixList.Last());
		matrixList.DetachLast();	//detach it
		aMatrix<long double>* tempMatrix2 = finalMatrix->Concat(*tempMatrix1);
		delete finalMatrix; finalMatrix = NULL;
		delete tempMatrix1; tempMatrix1 = NULL;
		finalMatrix = tempMatrix2;
		tempMatrix2 = NULL;
		}//end for i

	return finalMatrix;
	}//end ANOVALineBase::MakeDesignMatrix



int ANOVALineBase::MeetsSearchCriteria (long indiv, StringList& searchCriteriaList, ObjList<StringList>& independentVarSet){
	for (int i=0;i<searchCriteriaList.len();++i){
		aString& searchStr = searchCriteriaList[i];
		StringList& independentVarList = independentVarSet[i];
		aString& indivStr = independentVarList[indiv];
		if (searchStr != indivStr){
			return 0;}//false
		}//end for i
	return 1;//true by default
	}//end  ANOVALineBase::MeetsSearchCriteria


void ANOVALineBase::PartitionTitleString (aString& titleStr, aString& inParenStr, aString& outsideParenStr){
	aString startParen('('), endParen(')');
	
	int loc1 = titleStr.pos(startParen);
	int loc2 = titleStr.pos(endParen);
	aString inStr(titleStr,loc1+1,loc2-1);
	inParenStr = inStr;
	aString oStr(titleStr,1,loc1-1);
	outsideParenStr = oStr;
	outsideParenStr.CullTrailingSpaces();
	}//end ANOVALineBase::PartitionTitleString


void ANOVALineBase::PrepForCalcs (dataSetRec& theDataSet, aMatrix<long double>* factorDesign, 
				aMatrix<long double>* incedenceMatrix, long n, aMatrix<long double>* Ymatrix){
//	procedure PrepANOVALine (var ANOVALine: ANOVALineRecTypePtr; theDataSet: dataSetRec; var factorDesign: matrixPtrType; n: longint; Ymatrix: matrixPtrType);
	//takes the factor string and decomposes it into factors, then
	//creates the independent variable list and passes it on to get design matrix & df
	aString theTitle(title());
	ANOVALineType analysisToDo = GetANOVALineType(theTitle);
	StringList titleStrList;
	aString totalString(title());
	ConvertTitleString(totalString, titleStrList);
	int numFactors = titleStrList.len();
//	if(analysisToDo == complexNestLine){
//		--numFactors;}//the last factor in the titleStrList is a within-parentheses set of factors, handled separately

	ObjList<StringList> independentVarSet;
	for (int i=0;i<numFactors;++i){
		aString& colTitle = titleStrList[i];
//		dataColumnRecStr& indepVarList = (dataColumnRecStr) theDataSet.GetDataColumn(colTitle);
		dataColumnRec& indepVarListRec = theDataSet.GetDataColumn(colTitle);
		dataColumnRecStr* indepVarListPtr = (dataColumnRecStr*) &indepVarListRec;
		dataColumnRecStr& indepVarList =  *indepVarListPtr;
		StringList& indepVarColToCopy = indepVarList.dataList();
		StringList* independentVarList = NEW StringList(indepVarColToCopy);
		independentVarSet.Append(independentVarList);
		independentVarList = NULL;//forget this
		}//end for i
		
	if (analysisToDo == factorialLine){
			CalcDesignAndDF(independentVarSet, theDataSet, factorDesign, incedenceMatrix, n);}
		else if (analysisToDo == simpleNestLine){
			CalcDesignAndDFNest(independentVarSet, theDataSet, factorDesign, incedenceMatrix, n);}
		else if (analysisToDo == interactionLine){
			CalcDesignAndDFInteraction(independentVarSet, theDataSet, factorDesign, incedenceMatrix, n, Ymatrix);}
		else if(analysisToDo == complexNestLine){
			CalcDesignAndDFComplexNest(/*independentVarSet,*/theDataSet, factorDesign, incedenceMatrix, n);
			}
		else{//no other options yet available
			}
	independentVarSet.Clear();
	titleStrList.Clear();
	}//end  ANOVALineBase::PrepForCalcs 


void ANOVALineBase::Print (ostream& /*outFile*/, int /*titleLen*/, StringList& /*dependentList*/, long /*bootRep*/){
	//titleLen is the desired number of characters in the title line
	aString err("Error: calling ANOVALineBase::Print() by mistake to print a data line.");
	aError(err);
	}//end ANOVALineBase::Print


void ANOVALineBase::PrintFactorLevels (ostream& outFile){
	//titleLen is the desired number of characters in the title line
	outFile<<title_<<endl;
	outFile<<"factor level\tn"<<endl;
	for(int i=0;i<levelNames_.len();++i){
		outFile<<levelNames_[i]<<'\t'<<levelRows_[i].len()<<endl;}
	outFile<<endl;
	}//end ANOVALineBase::PrintFactorLevels


void ANOVALineBase::PrintFactorLevelsAndDataRows (ostream& outFile){
	//titleLen is the desired number of characters in the title line
	outFile<<title_<<endl;
	outFile<<"factor level\tn\tdata rows"<<endl;
	for(int i=0;i<levelNames_.len();++i){
		outFile<<levelNames_[i]<<'\t'<<levelRows_[i].len()<<'\t'<<levelRows_[i][0]+1;
		for(int r=1;r<levelRows_[i].len();++r){
			outFile<<','<<levelRows_[i][r]+1;}
		outFile<<endl;}
	outFile<<endl;
	}//end ANOVALineBase::PrintFactorLevelsAndDataRows



void ANOVALineBase::SetP(int denomDF, int boot){//boot == 0 gives standard P, boot==1 gives bootstrap P value
	if(boot>0){
			P_ = bootFList.Get1TailP(F_,1);}
		else{
			P_ = Fdistribution1TailedP(df_, denomDF, F_);}
	}//end ANOVALineBase::SetP
			



int operator== (ANOVALineBase& line1, ANOVALineBase& line2){
		return (line1.title() == line2.title());}

int operator!= (ANOVALineBase& line1, ANOVALineBase& line2){
		return !(line1 == line2);}


#endif
