/*******************************************
*
*
*	ANOVATable.cp
*
*	A class that implements an ANOVA table
*
*
************************************************/

#ifndef ANOVATablecp
#define ANOVATablecp = 1


#include "ANOVATable.h"
//#include "GeneralLinearModel.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "aVector.cpp"
#include "aMatrix.cpp"
#include "dataColumnRecNum.cpp"






void ANOVATable::AppendANOVATableFlist (long double val, int loc){
	ANOVALineBase& theLine = Nth(loc);
	theLine.AppendBootF(val);
	}//end ANOVATable::AppendANOVATableFlist


long ANOVATable::df(aString& theTitle){
	ANOVALineBase& theLine = GetANOVALine(theTitle);
	return theLine.df();
	}//end ANOVATable::df


long ANOVATable::df(int loc){
	ANOVALineBase& theLine = Nth(loc);
	return theLine.df();
	}//end ANOVATable::df



long ANOVATable::levels(aString& theTitle){
	ANOVALineBase& theLine = GetANOVALine(theTitle);
	return theLine.levels();
	}//end ANOVATable::levels


long ANOVATable::levels(int loc){
	ANOVALineBase& theLine = Nth(loc);
	return theLine.levels();
	}//end ANOVATable::levels




void ANOVATable::DoBootANOVA (dataSetRec& allData, aString& commandFileName, 
							aString& outFileName, long reps, Ran3& ran){
	//among species only to start
	StringList factorList, MSDivisorList;
	long dfSum = 0;
	int longTitle;
	int err = SetUpTableAndDesignMatrix (allData, commandFileName, factorList, MSDivisorList, dfSum,longTitle);
	if(err){return;}

	//make a new ordered list of the dependent variable
	aString& colTitle = factorList[0];
	dataColumnRec& depVarColRec = allData.GetDataColumn(colTitle);
	dataColumnRecNum<long double>* depVarColPtr = (dataColumnRecNum<long double>*) &depVarColRec;
	dataColumnRecNum<long double>& depVarCol = *depVarColPtr;
	aList<long double>& varList = depVarCol.dataList();
	

	aMatrix<long double> Ymatrix;
	aVector<long double> aVecPtr(varList);
	Ymatrix.ConcatCol(aVecPtr);
	aMatrix<long double>* origYmatrix = NEW aMatrix<long double>(Ymatrix);
//    	cout<<"matrix of Y values:"<<endl;
//	Ymatrix.Print();

	long n = Ymatrix.rows();
	SetDF(n - 1, len()-1);	//total df
	SetDF((n - 1) - dfSum, len() - 2);//error df


	//do the analysis
	for (long rep = 0; rep<= reps;++rep){
		aString repWord(rep); repWord.PrependCharToLen(' ',6);
		cout<<"rep: "<< repWord<<endl;
		if (rep > 0){
			//resample the data
			Ymatrix.Clear();//original Ymatrix is stored in origYmatrix
			aVector<long double>* resampledVec = depVarCol.GetResampledVectorLongDouble(withReplacement, ran);
			Ymatrix.ConcatCol(*resampledVec);//copies to matrix
//			cout<<"resampled Ymatrix:"<<endl;
//			Ymatrix.Print(cout);
			delete resampledVec; resampledVec = NULL;
			}//end if rep>0
		linearModel.ANOVASSdfMS(&Ymatrix, *this, rep);
		SetF(MSDivisorList, rep);
		}//end for rep
	SetP(MSDivisorList, reps);
	Print(cout, longTitle + 2);
	PrintFactorLevels(cout);
	fstream outFile;
	char* filename = outFileName.raw();
	outFile.open(filename,ios::out);
	Print(outFile, longTitle + 2);
	PrintFactorLevels(outFile);
	outFile.close();
	delete [] filename; filename=NULL;

	delete origYmatrix; origYmatrix=NULL;
	}//end ANOVATable::DoBootANOVA 



ANOVALineBase& ANOVATable::GetANOVALine(aString& lineTitle){
	for (int i=0;i<len();++i){
		ANOVALineBase& theLine = Nth(i);
		if (theLine.title() == lineTitle){
			return theLine;}
		}//end for i
	//if no match found
	aString err("There is no ANOVA line named ");
	err.cat(lineTitle);
	err.cat(" in the ANOVA table.");
	aError(err);
	return Nth(len()-2);//error line
	}//end ANOVATable::GetANOVALine



void ANOVATable::Print (ostream& outFile, int titleLen){
	aString headings("factor");
	headings.separator(titleLen - 3);
	headings.cat("df");
	headings.separator(8);
	headings.cat("SS");
	headings.separator(12);
	headings.cat("MS");
	headings.separator(10);
	headings.cat("F");
	headings.separator(9);
	headings.cat("P");
	headings.separator(9);
	headings.cat("expected MS");
	outFile<<headings<<endl;
	StringList dummy_;
	for (int i=0;i<len();++i){
		ANOVALineBase& theLine =Nth(i);
		theLine.Print(outFile, titleLen, dummy_);
		}//end for i
	outFile<<endl;
	}//end ANOVATable::Print



void ANOVATable::PrintFactorLevels (ostream& outFile){
	for (int i=0;i<len()-2;++i){
		ANOVALineBase& theLine =Nth(i);
		theLine.PrintFactorLevels(outFile);
		}//end for i
	}//end ANOVATable::PrintFactorLevels



int ANOVATable::ReadCommandFile (StringList& factorList, StringList& MSDivisorList, aString& commandFileName){
	//returns 0 if data format error encountered
	aString depWord ="dependent:";
	aString facWord = "factors:";
	aString Fword = "F-divisors:";
	aString eWord = "Error";
	aString endCharList(" \t,;\"(){}[]!?");
	factorList.Clear();  MSDivisorList.Clear();
	int n = 0;
	aString aStr;
	fstream commandFile;
	char* commandName = commandFileName.raw();
	commandFile.open(commandName);
	char* dataLine = NEW char [1001]; dataLine[0]='\0';
	commandFile.getline(dataLine,1000);
	aString indivDataLine(dataLine);
	cout<<indivDataLine<<endl;//prints dependent variable
	aString* word = indivDataLine.ChopOffFirstWord(endCharList);
	if (*word != depWord){
			cout<<"Error in command file setup:  dependent variable not preceded by the word 'dependent:'"<<endl;
			cout<<"Press return to continue..."<<endl;
			char* a = NEW char[5];a[0]='\0';
			cin >> a;  //to keyboard: press any key to continue
			delete [] a; a = NULL;
			delete word; word=NULL;
			return 1;//error occurred
			}
		else{
			aString* depColname = NEW aString(indivDataLine);
			depColname->CullChar(' ');//removes any spaces
			factorList.Append(depColname);}
	delete word; word=NULL;
	do{//skips over facWord and stores effects to be tested
		commandFile.getline(dataLine,1000);
		indivDataLine = dataLine;
		if ((indivDataLine != Fword) && (indivDataLine != facWord)){
			aString* word = NEW aString(indivDataLine);  //Copy(indivDataLine, 1, WordLength(indivDataLine));
			factorList.Append(word);
			++n;}
		}while (indivDataLine != Fword);
	if(factorList.Includes(&eWord)){
		--n;}//skip error line in reading MS divisors
	for(int j =0;j< n;++j){
		commandFile.getline(dataLine,1000);
		indivDataLine = dataLine;
//		indivDataLine.CullChar(' ');
		aString* word = NEW aString(indivDataLine);  //Copy(indivDataLine, 1, WordLength(indivDataLine));
		MSDivisorList.Append(word);
		}//end for i
	commandFile.close();
	delete [] commandName; commandName=NULL;
	delete [] dataLine; dataLine=NULL;
	return 0;//no errors
	}//end ANOVATable::ReadCommandFile 



void ANOVATable::SetDF  (int val, int loc){
	ANOVALineBase& theLine = Nth(loc);
	theLine.df(val);
	}//end ANOVATable::SetDF


//void ANOVATable::SetLevels  (int val, int loc){
//	ANOVALineBase& theLine = Nth(loc);
//	theLine.levels(val);
//	}//end ANOVATable::SetLevels


void ANOVATable::SetEffect (effectTypeSet val, int loc){
	ANOVALineBase& theLine = Nth(loc);
	theLine.effectType(val);
	}//end ANOVATable::SetEffect


void ANOVATable::SetF (StringList& MSDivisorList, long bootRep){
	//need to determine denominator for each line
	for(int i=0;i<len() - 2;++i){
		ANOVALineBase& theLine = Nth(i);
		aString& denomTitle = MSDivisorList[i];
		ANOVALineBase& denomLine = GetANOVALine(denomTitle);
		theLine.F(denomLine, bootRep);
			}//end for i
	}//end ANOVATable::SetF


void ANOVATable::SetMS (long double val, long loc, long bootRep){
	ANOVALineBase& theLine_ = Nth(loc);
	ANOVALine *theLine = (ANOVALine*) &theLine_;
	theLine->MS(val,bootRep);
	}//end ANOVATable::SetMS


void ANOVATable::SetMSDenomTitle (aString val, int loc){
	ANOVALineBase& theLine_ = Nth(loc);
	ANOVALine *theLine = (ANOVALine*) &theLine_;
	theLine->MSDenomTitle(val);
	}//end ANOVATable::SetMSDenomTitle


void ANOVATable::SetP (StringList& MSDivisorList, int boot){
	//need to determine denominator for each line
	for(int i=0;i<len() - 2;++i){
		ANOVALineBase& theLine = Nth(i);
		aString& denomTitle = MSDivisorList[i];
		ANOVALineBase& denomLine = GetANOVALine(denomTitle);
		int denomDF = denomLine.df();
		theLine.SetP(denomDF, boot);
		}//end for i
	}//end ANOVATable::SetP



void ANOVATable::SetSS (long double val, long loc, long bootRep){
	ANOVALineBase& theLine_ = Nth(loc);
	ANOVALine *theLine = (ANOVALine*) &theLine_;
	theLine->SS(val,bootRep);
	}//end ANOVATable::SetSS


void ANOVATable::SetReducedFormSS (long double val, long loc, long bootRep){
	ANOVALineBase& theLine_ = Nth(loc);
	ANOVALine *theLine = (ANOVALine*) &theLine_;
	theLine->reducedFormSS(val,bootRep);
	}//end ANOVATable::SetReducedFormSS



int ANOVATable::SetUpTableAndDesignMatrix (dataSetRec& allData, aString& commandFileName,
									StringList& factorList, StringList& MSDivisorList, long& dfSum, 
									int& longTitle){
	Clear();//removes old analyses
	factorList.Clear();
	MSDivisorList.Clear();
	int err = ReadCommandFile (factorList, MSDivisorList, commandFileName);
	if(err){return 1;}
	int factorsInModel = factorList.len() - 1;
	int doingRepeatedMeasures = false;

	aString& colTitle = factorList[0];
	dataColumnRec& depVarColRec = allData.GetDataColumn(colTitle);
	dataColumnRecNum<long double>* depVarColPtr = (dataColumnRecNum<long double>*) &depVarColRec;
	dataColumnRecNum<long double>& depVarCol = *depVarColPtr;
	aList<long double>& varList = depVarCol.dataList();

	aMatrix<long double> Ymatrix;
	aVector<long double> aVecPtr(varList);
	Ymatrix.ConcatCol(aVecPtr);

	//make the vector of constants
	aMatrix<long double>* ConstMatrix = NEW aMatrix<long double>((long double)1.0,varList.len(),1);
	aMatrix<long double>* ConstMatrix1 = NEW aMatrix<long double>((long double)1.0,varList.len(),1);

	for (int i =0;i<factorsInModel + 1;++i){//+ total line
		ANOVALine* theANOVALine = NEW ANOVALine();
		theANOVALine->effectType(fixedEffect);
		Append(theANOVALine);
		theANOVALine = NULL;//forget
		}//end for i
	aString titleWord = "Error";

	if(factorList.Includes(&titleWord)){//it's not a repeated-measures ANOVA
			factorList.Delete(factorList.Loc(&titleWord));
			--factorsInModel;
			SetEffect(errorLine, len() - 2);	//fixedEffect, randomEffect, errorLine, totalLine
			SetTitle(titleWord, len() - 2);
			}
		else{//last line in repeated measures ANOVA is assumed to contain error variance only
			SetEffect(errorLine, len() - 2);	//fixedEffect, randomEffect, errorLine, totalLine
			doingRepeatedMeasures = true;
			}			

	SetEffect(totalLine, len()-1);	
	titleWord = "Total";
	SetTitle(titleWord, len()-1);
	longTitle = 5;	//number of characters in longest title

	//make the full design matrix components list
//	GLM linearModel;
//	linearModel.Clear();
	MatrixList* invXprimeXList = linearModel.invXprimeXList;
	MatrixList* designMatrixComponents = linearModel.designList;
	MatrixList* incedenceMatrixComponents = linearModel.incedenceMatrixList;
	designMatrixComponents->Append(ConstMatrix);
	incedenceMatrixComponents->Append(ConstMatrix1);

	dfSum = 0;
	//for each ANOVA table row
	for(int i =0;i<factorsInModel;++i){
		aString& factorString = factorList[i+1];
		SetTitle(factorString, i);
		aString& denomTitle = MSDivisorList[i];
		SetMSDenomTitle(denomTitle, i);
		if (longTitle < factorString.len()){ 
			longTitle = factorString.len();}
		ANOVALineBase& theLine = Nth(i);
		//this skips the last factor line if repeated measures ANOVA is used
		if(doingRepeatedMeasures && (i==factorsInModel-1)){break;}
			else{
				aMatrix<long double>* factorDesign = NEW aMatrix<long double>();
				aMatrix<long double>* incedenceMatrix = NEW aMatrix<long double>();
				theLine.PrepForCalcs(allData, factorDesign, incedenceMatrix, Ymatrix.rows(), &Ymatrix);
		
		//		factorDesign->Print();
		
				designMatrixComponents->Append(factorDesign);
				incedenceMatrixComponents->Append(incedenceMatrix);
				dfSum += theLine.df();
				factorDesign=NULL;//forget this too
				incedenceMatrix=NULL;//forget this too
				}//end if...else
		}//end for i
	return 0;
	}//end ANOVATable::SetUpTableAndDesignMatrix 



void ANOVATable::SetVarianceComponent (long double val, long loc, long bootRep){
	ANOVALineBase& theLine_ = Nth(loc);
	ANOVALine *theLine = (ANOVALine*) &theLine_;
	theLine->varianceComponent(val,bootRep);
	}//end ANOVATable::SetVarianceComponent


void ANOVATable::SetTitle (aString val, int loc){
	ANOVALineBase& theLine = Nth(loc);
	theLine.title(val);
	}//end ANOVATable::SetTitle


long double ANOVATable::SS(aString& theTitle){
	ANOVALineBase& theLine_ = GetANOVALine(theTitle);
	ANOVALine *theLine = (ANOVALine*) &theLine_;
	return theLine->SS(0);
	}//end ANOVATable::SS


long double ANOVATable::reducedSS(aString& theTitle){
	ANOVALineBase& theLine_ = GetANOVALine(theTitle);
	ANOVALine *theLine = (ANOVALine*) &theLine_;
	return theLine->reducedFormSS(0);
	}//end ANOVATable::reducedSS


long double ANOVATable::MS(aString& theTitle){
	ANOVALineBase& theLine_ = GetANOVALine(theTitle);
	ANOVALine *theLine = (ANOVALine*) &theLine_;
	return theLine->MS(0);
	}//end ANOVATable::MS


long double ANOVATable::varianceComponent(aString& theTitle){
	ANOVALineBase& theLine_ = GetANOVALine(theTitle);
	ANOVALine *theLine = (ANOVALine*) &theLine_;
	return theLine->varianceComponent(0);
	}//end ANOVATable::varianceComponent




#endif


