/*******************************************
*
*
*	GeneralLinearModel.cpp
*
*	A class that implements least squares estimation using mastrix algebra
*
*
************************************************/


#include "Element.cpp"
#include "aList.cpp"
#include "GeneralLinearModel.h"
#include "ANOVATable.h"
#include "MANOVATable.h"
#include "aVector.cpp"
#include "aMatrix.cpp"
//#include "aMatrix2.cpp"
#include "StatList.cpp"




void GLM::ANOVASSdfMS (aMatrix<long double>* Ymatrix, ANOVATable& theANOVATable, long  bootRep, MatrixList *residuals){
	if(residuals){
		residuals->Clear();}
	long double fullSSR=0.0, fullSSE=0.0, fullSSTO=0.0, factorSS=0.0;
	long double reducedSSR=0.0, reducedSSE=0.0, reducedSSTO=0.0;
	//make the full design matrix
	MatrixList* reducedDesignComponents = NEW MatrixList(designList);
	aMatrix<long double>* Xmatrix = MakeDesignMatrix(*reducedDesignComponents, 0);
//	Xmatrix->Print(cout);
	delete reducedDesignComponents; reducedDesignComponents = NULL;


	aMatrix<long double>* fullEmatrix = NEW aMatrix<long double>();	//residuals
	matrixHandle<long double> fullEmatrixH(fullEmatrix);	//residuals
	aMatrix<long double>* fullBmatrix = NEW aMatrix<long double>();	//reg. coeffs.
	matrixHandle<long double> fullBmatrixH(fullBmatrix);	//reg. coeffs.
	if (bootRep > 0){//doing bootstrap replicates using same design matrices
			//use the saved invXprimeXmatrix
			aMatrix<long double>& invXprimeXmatrixA = (*invXprimeXList)[0];
			matrixHandle<long double> invXprimeXmatrixH(invXprimeXmatrixA);
			RegressionSS(Xmatrix, Ymatrix, fullSSR, fullSSE, fullSSTO, invXprimeXmatrixH, fullBmatrixH, fullEmatrixH);
			invXprimeXmatrixH.Detach();
			}
		else{
			// create the invXprimeXmatrix
			aMatrix<long double>* invXprimeXmatrixB = NEW aMatrix<long double>();//send empty matrix
			matrixHandle<long double> invXprimeXmatrixH(invXprimeXmatrixB);
			RegressionSS(Xmatrix, Ymatrix, fullSSR, fullSSE, fullSSTO, invXprimeXmatrixH, fullBmatrixH, fullEmatrixH);
//			Xmatrix->Print("Xmatrices");
			invXprimeXmatrixB = invXprimeXmatrixH.mat;
			invXprimeXList->Append(invXprimeXmatrixB);  //full model goes in position 0
			invXprimeXmatrixH.Detach();
			invXprimeXmatrixB = NULL;//forget it here; destroyed elsewhere
			}
	delete Xmatrix; Xmatrix=NULL;

	theANOVATable.SetSS(fullSSE, theANOVATable.len() - 2, bootRep);
	theANOVATable.SetSS(fullSSTO, theANOVATable.len()-1, bootRep);
	long double MS = 0.0;
	ANOVALineBase& errorLine_ = theANOVATable.GetANOVALine(theANOVATable.len() - 2);
	long errorDF = errorLine_.df();
	if (errorDF > 0){
		MS = fullSSE / (long double) errorDF;}
	theANOVATable.SetMS(MS, theANOVATable.len() - 2, bootRep);



	for(int i=1;i< designList->len();++i){
		//make the reduced design matrix omitting factor i
		reducedDesignComponents = NEW MatrixList(designList);
		aMatrix<long double>* Xmatrix = MakeDesignMatrix(*reducedDesignComponents, i);
//		Xmatrix->Print("Xmatrices");
//		(*designList)[i].Print("Xmatrices");
		delete reducedDesignComponents; reducedDesignComponents = NULL;

		aMatrix<long double>* reducedEmatrix =  NEW aMatrix<long double>();	//residuals
		matrixHandle<long double> reducedEmatrixH(reducedEmatrix);	//residuals
		
		aMatrix<long double>* reducedBmatrix =  NEW aMatrix<long double>();	//reg. coeffs.
		matrixHandle<long double> reducedBmatrixH(reducedBmatrix);	//reg. coeffs.

		if (bootRep > 0){//then this must be a bootstrap analysis
				//use a saved invXprimeXmatrix
				aMatrix<long double>& invXprimeXmatrixC =(*invXprimeXList)[i];
				matrixHandle<long double> invXprimeXmatrixH(invXprimeXmatrixC);
				RegressionSS(Xmatrix, Ymatrix, reducedSSR, reducedSSE, reducedSSTO, invXprimeXmatrixH, reducedBmatrixH, reducedEmatrixH);
				invXprimeXmatrixH.Detach();//invXprimeXmatrix returns unchanged, and is deleted elsewhere
				}
			else{
				//create the invXprimeXmatrix
				aMatrix<long double>* invXprimeXmatrixD = NEW aMatrix<long double>();//send empty matrix
				matrixHandle<long double> invXprimeXmatrixH(invXprimeXmatrixD);
				RegressionSS(Xmatrix, Ymatrix, reducedSSR, reducedSSE, reducedSSTO, invXprimeXmatrixH, reducedBmatrixH, reducedEmatrixH);
				invXprimeXmatrixD = invXprimeXmatrixH.mat;//this is newly created
				invXprimeXList->Append(invXprimeXmatrixD);	//this model goes in position i
				invXprimeXmatrixH.Detach();
				invXprimeXmatrixD = NULL;
				}
		delete Xmatrix; Xmatrix = NULL;
		if(residuals){
			residuals->Append(reducedEmatrixH.mat);
			reducedEmatrixH.Detach();}
		reducedEmatrix = NULL;//forget here; deleted with the handle
		reducedBmatrix = NULL;//forget here; deleted with the handle
		factorSS = reducedSSE - fullSSE;
		theANOVATable.SetSS(factorSS, i-1, bootRep);
		ANOVALineBase& theLine = theANOVATable.GetANOVALine(i-1);
		ANOVALine *anLine = (ANOVALine*) &theLine;
		anLine->MS(0.0, bootRep);
		if (anLine->df() > 0){
			anLine->MS(anLine->SS(bootRep) / (long double) anLine->df(), bootRep);}
		}//end for i

	errorLine_ = theANOVATable.GetANOVALine(theANOVATable.len() - 2);
	ANOVALine *eLine = (ANOVALine*) &errorLine_;
	if (eLine->df() > 0){
		eLine->MS(eLine->SS(bootRep) / (long double) eLine->df(), bootRep);}
	if(residuals){//this goes last
		residuals->Append(fullEmatrixH.mat);
		fullEmatrixH.Detach();}

	fullEmatrix = NULL;//forget here; deleted with the handle
	fullBmatrix = NULL;//forget here; deleted with the handle
	}//end GLM::ANOVASSdfMS



void GLM::Clear(void){
	if(invXprimeXList != NULL){invXprimeXList->Clear();}
	if(incedenceMatrixList != NULL){incedenceMatrixList->Clear();}
	if(designList != NULL){designList->Clear();}
	delete invXprimeXList; invXprimeXList=NULL;
	delete incedenceMatrixList; incedenceMatrixList=NULL;
	delete designList; designList=NULL;
	varCompCoefficients.Clear();
	reusingDesign_= recyclablesCalculated_=varCompCoeffsSupplied_=false;
	}//GLM:::Clear()


aMatrix<long double>* GLM::MakeDesignMatrix (MatrixList& mList, const int omitFactor){
	//mList comes back empty
	if (omitFactor != 0){//=0 in full model
			mList.Delete(omitFactor);}	//for the reduced model
	aMatrix<long double>* finalMatrix = &(mList.Last());
	mList.DetachLast();	//detach it
	aMatrix<long double>* tempMatrix2 = NULL;
	int j = mList.len();
	for(int i=0;i<j;++i){
		aMatrix<long double>* tempMatrix1 =&(mList.Last());
		mList.DetachLast();	//detach it
		tempMatrix2 = tempMatrix1->Concat(*finalMatrix);
		delete tempMatrix1; tempMatrix1 = NULL;
		delete finalMatrix; finalMatrix = NULL;
		finalMatrix = tempMatrix2;
		tempMatrix2 = NULL;
		}//end for i

	return finalMatrix;
	}//end GLM::MakeDesignMatrix



aMatrix<long double>* GLM::MakeDesignMatrix (MatrixList& mList, aList<int>& omitFactor){
	//mList comes back empty
	for(int f=0;f<omitFactor.len();++f){//=0 in full model
			mList.Delete(omitFactor[f]);}	//for the reduced model
	aMatrix<long double>* finalMatrix = &(mList.Last());
	mList.DetachLast();	//detach it
	aMatrix<long double>* tempMatrix2 = NULL;
	int j = mList.len();
	for(int i=0;i<j;++i){
		aMatrix<long double>* tempMatrix1 =&(mList.Last());
		mList.DetachLast();	//detach it
		tempMatrix2 = tempMatrix1->Concat(*finalMatrix);
		delete tempMatrix1; tempMatrix1 = NULL;
		delete finalMatrix; finalMatrix = NULL;
		finalMatrix = tempMatrix2;
		tempMatrix2 = NULL;
		}//end for i

	return finalMatrix;
	}//end GLM::MakeDesignMatrix



void GLM::MANOVASSdfVarCov (aMatrix<long double>* Ymatrix, MANOVATable& theMANOVATable, long  bootRep, MatrixList *residuals){
	if(residuals){
		residuals->Clear();}
	aMatrix<long double>* fullSSRmatrix = NEW aMatrix<long double>();	
	aMatrix<long double>* fullSSEmatrix = NEW aMatrix<long double>();	
	aMatrix<long double>* fullSSTOmatrix = NEW aMatrix<long double>();
	matrixHandle<long double> fullSSRmatrixH(fullSSRmatrix), fullSSEmatrixH(fullSSEmatrix), fullSSTOmatrixH(fullSSTOmatrix);
	aMatrix<long double>* reducedSSRmatrix = NEW aMatrix<long double>();
	aMatrix<long double>* reducedSSEmatrix = NEW aMatrix<long double>();	
	aMatrix<long double>* reducedSSTOmatrix = NEW aMatrix<long double>();	
	matrixHandle<long double> reducedSSRmatrixH(reducedSSRmatrix), reducedSSEmatrixH(reducedSSEmatrix), reducedSSTOmatrixH(reducedSSTOmatrix);
	//make the full design matrix
	MatrixList* reducedDesignComponents = NEW MatrixList(designList);
	aMatrix<long double>* Xmatrix = MakeDesignMatrix(*reducedDesignComponents, 0);
//	Xmatrix->Print(cout);
	delete reducedDesignComponents; reducedDesignComponents = NULL;


	aMatrix<long double>* fullEmatrix = NEW aMatrix<long double>();	//residuals
	matrixHandle<long double> fullEmatrixH(fullEmatrix);	//residuals
	aMatrix<long double>* fullBmatrix = NEW aMatrix<long double>();	//reg. coeffs.
	matrixHandle<long double> fullBmatrixH(fullBmatrix);	//reg. coeffs.
	if (bootRep > 0){//doing bootstrap replicates using same design matrices
			//use the saved invXprimeXmatrix
			aMatrix<long double>& invXprimeXmatrixA = (*invXprimeXList)[0];
			matrixHandle<long double> invXprimeXmatrixH(invXprimeXmatrixA);
			RegressionSS(Xmatrix, Ymatrix, fullSSRmatrixH, fullSSEmatrixH, fullSSTOmatrixH, invXprimeXmatrixH, fullBmatrixH, fullEmatrixH);
			invXprimeXmatrixH.Detach();
			}
		else{
			// create the invXprimeXmatrix
			aMatrix<long double>* invXprimeXmatrixB = NEW aMatrix<long double>();//send empty matrix
			matrixHandle<long double> invXprimeXmatrixH(invXprimeXmatrixB);
			RegressionSS(Xmatrix, Ymatrix, fullSSRmatrixH, fullSSEmatrixH, fullSSTOmatrixH, invXprimeXmatrixH, fullBmatrixH, fullEmatrixH);
//			Xmatrix->Print("Xmatrices");
			invXprimeXmatrixB = invXprimeXmatrixH.mat;
			invXprimeXList->Append(invXprimeXmatrixB);  //full model goes in position 0
			invXprimeXmatrixH.Detach();
			invXprimeXmatrixB = NULL;//forget it here; destroyed elsewhere
			}
	delete Xmatrix; Xmatrix=NULL;
	fullSSEmatrix = fullSSEmatrixH.mat;
	fullSSTOmatrix = fullSSTOmatrixH.mat;
	
	theMANOVATable.SetSS(*fullSSEmatrix, theMANOVATable.len() - 2, bootRep);
	theMANOVATable.SetSS(*fullSSTOmatrix, theMANOVATable.len()-1, bootRep);
	aMatrix<long double> varCov_(*fullSSEmatrix);
	ANOVALineBase& errorLine_ = theMANOVATable.GetMANOVALine(theMANOVATable.len() - 2);
	long errorDF = errorLine_.df();
	if (errorDF > 0){
		long double dfdivisor = (long double)1.0 / (long double) errorDF;
		varCov_.Multiply( dfdivisor);}
//	theMANOVATable.SetVarCov(varCov_, theMANOVATable.len() - 2, bootRep,residuals);//replaced by the lines below
	MANOVALine *errorLineM_ = (MANOVALine*) &errorLine_;
	if(residuals){
		errorLineM_->VarCov(varCov_, bootRep,fullEmatrixH.mat);}
	else{
		errorLineM_->VarCov(varCov_, bootRep,NULL);}
		


	for(int i=1;i< designList->len();++i){
		//make the reduced design matrix omitting factor i
		reducedDesignComponents = NEW MatrixList(designList);
		Xmatrix = MakeDesignMatrix(*reducedDesignComponents, i);
//		Xmatrix->Print("Xmatrices");
		delete reducedDesignComponents; reducedDesignComponents = NULL;

		aMatrix<long double>* reducedEmatrix =  NEW aMatrix<long double>();	//residuals
		matrixHandle<long double> reducedEmatrixH(reducedEmatrix);	//residuals
		
		aMatrix<long double>* reducedBmatrix =  NEW aMatrix<long double>();	//reg. coeffs.
		matrixHandle<long double> reducedBmatrixH(reducedBmatrix);	//reg. coeffs.

		if (bootRep > 0){//then this must be a bootstrap analysis
				//use a saved invXprimeXmatrix
				aMatrix<long double>& invXprimeXmatrixC =(*invXprimeXList)[i];
				matrixHandle<long double> invXprimeXmatrixH(invXprimeXmatrixC);
				RegressionSS(Xmatrix, Ymatrix, reducedSSRmatrixH, reducedSSEmatrixH, reducedSSTOmatrixH, 
									invXprimeXmatrixH, reducedBmatrixH, reducedEmatrixH);
				invXprimeXmatrixH.Detach();//invXprimeXmatrix returns unchanged, and is deleted elsewhere
				}
			else{
				//create the invXprimeXmatrix
				aMatrix<long double>* invXprimeXmatrixD = NEW aMatrix<long double>();//send empty matrix
				matrixHandle<long double> invXprimeXmatrixH(invXprimeXmatrixD);
				RegressionSS(Xmatrix, Ymatrix, reducedSSRmatrixH, reducedSSEmatrixH, reducedSSTOmatrixH, invXprimeXmatrixH, reducedBmatrixH, reducedEmatrixH);
				invXprimeXmatrixD = invXprimeXmatrixH.mat;//this is newly created
				invXprimeXList->Append(invXprimeXmatrixD);	//this model goes in position i
				invXprimeXmatrixH.Detach();
				invXprimeXmatrixD = NULL;
				}
		delete Xmatrix; Xmatrix = NULL;
//		reducedSSRmatrix =  reducedSSRmatrixH.mat;
		reducedSSEmatrix =reducedSSEmatrixH.mat;	
//		reducedSSTOmatrix = reducedSSTOmatrixH.mat;	
		aMatrix<long double>* factorSSmatrix = reducedSSEmatrix->Subtract(*fullSSEmatrix);
		theMANOVATable.SetSS(*factorSSmatrix, i-1, bootRep);
		reducedSSRmatrix =  reducedSSEmatrix = reducedSSTOmatrix = NULL;///forget here; deleted with handles
		delete factorSSmatrix; factorSSmatrix=NULL;
		
		ANOVALineBase& theLine = theMANOVATable.GetMANOVALine(i-1);
		MANOVALine *manLine = (MANOVALine*) &theLine;
		aMatrix<long double> vcmat(manLine->SS(bootRep));
		if (manLine->df() > 0){
				long double dfdivisor = (long double)1.0/(long double) manLine->df();
				vcmat.Multiply(dfdivisor);
				}
			else{//if no degrees of freedom left, just replace with zeros
				for(int r=0;r<vcmat.rows();++r) 
					for(int c=0;c<vcmat.cols();++c)
						vcmat[r][c]=(long double) 0.0;
				}
//		manLine->VarCov(vcmat, bootRep);
		if(residuals){
			aMatrix<long double> *factorResiduals = reducedEmatrixH.mat->Subtract(*(fullEmatrixH.mat));
			manLine->VarCov(vcmat, bootRep,factorResiduals);
			residuals->Append(factorResiduals);
			}
		else{
			manLine->VarCov(vcmat, bootRep,NULL);}
		reducedEmatrix = NULL;//forget here; auto-deleted later with the handle
		reducedBmatrix = NULL;//forget here; auto-deleted later with the handle
		}//end for i

	errorLine_ = theMANOVATable.GetMANOVALine(theMANOVATable.len() - 2);
	MANOVALine *eLine = (MANOVALine*) &errorLine_;
	if (eLine->df() > 0){
		aMatrix<long double> eVarCov(eLine->SS(bootRep) );
		long double edfdivisor = (long double)1.0/(long double) eLine->df();
		eVarCov.Multiply(edfdivisor);
		if(residuals){
				eLine->VarCov(eVarCov, bootRep,fullEmatrixH.mat);}//fix
			else{
				eLine->VarCov(eVarCov, bootRep,NULL);}
		}
	if(residuals){//this goes last
		residuals->Append(fullEmatrixH.mat);
		fullEmatrixH.Detach();}

	fullEmatrix = NULL;//forget here; deleted with the handle
	fullBmatrix = NULL;//forget here; deleted with the handle
	}//end GLM::MANOVASSdfVarCov



void GLM::RegressionSS (aMatrix<long double>*Xmatrix, 
					aMatrix<long double>*Ymatrix, 
					long double& SSR, long double& SSE, long double& SSTO,
					matrixHandle<long double>& invXprimeXmatrixH, 
					matrixHandle<long double>& BmatrixH,  
					matrixHandle<long double>& EmatrixH){
	matrixHandle<long double> SSRmatrixH, SSEmatrixH, SSTOmatrixH;//empty
	RegressionSS (Xmatrix, Ymatrix,SSRmatrixH,SSEmatrixH, SSTOmatrixH,invXprimeXmatrixH, BmatrixH, EmatrixH);
	aMatrix<long double>* SSRmat_ = SSRmatrixH.mat;
	aMatrix<long double>* SSEmat_ = SSEmatrixH.mat;
	aMatrix<long double>* SSTOmat_ = SSTOmatrixH.mat;
	aMatrix<long double>& SSRmat = *SSRmat_;
	aMatrix<long double>& SSEmat = *SSEmat_;
	aMatrix<long double>& SSTOmat = *SSTOmat_;
	SSR = SSRmat[0][0];
	SSE = SSEmat[0][0];
	SSTO = SSTOmat[0][0];
	SSRmat_ =  SSEmat_ =  SSTOmat_ = NULL;//forget; disposed with handle
	}//end  GLM::RegressionSS

/*
void GLM::RegressionSS (aMatrix<long double>*Xmatrix, 
					aMatrix<long double>*Ymatrix, 
					long double& SSR, long double& SSE, long double& SSTO,
					matrixHandle<long double>& invXprimeXmatrixH, 
					matrixHandle<long double>& BmatrixH,  
					matrixHandle<long double>& EmatrixH){
	//variable matrices entered as empty matrices bound to handles
	aMatrix<long double>* XprimeMatrix = Xmatrix->Transpose();
	aMatrix<long double>* XprimeXmatrix = NULL;

	aMatrix<long double>* invXprimeXmatrix = invXprimeXmatrixH.mat;
	aMatrix<long double>* Bmatrix = BmatrixH.mat;
	aMatrix<long double>* Ematrix = EmatrixH.mat;
	BmatrixH.Detach();//these will be replaced below
	EmatrixH.Detach();//these will be replaced below
	int XprimeXexists = 0;//false
	if (invXprimeXmatrix->Empty()){ 
		//make on the first analysis & save elsewhere, then re-use in bootstrap analyses to save time
		XprimeXexists = 1;//true
		XprimeXmatrix = XprimeMatrix->Multiply(*Xmatrix);
		invXprimeXmatrixH.Detach();
		delete invXprimeXmatrix; invXprimeXmatrix=NULL;
		invXprimeXmatrix = XprimeXmatrix->Inverse();//slow, but no bugs!
		invXprimeXmatrixH.Bind(invXprimeXmatrix);
//		cout<<"matrix of X: "<<endl;
//		Xmatrix->Print(cout);
//		cout<<"matrix of (X''X)^-1 : "<<endl;
//		invXprimeXmatrix->Print(cout);
		}//end if invXprimeXmatrix =NULL

	aMatrix<long double>* YprimeMatrix = Ymatrix->Transpose();  //y = bX + e

	aMatrix<long double>* YprimeYmatrix = YprimeMatrix->Multiply(*Ymatrix);

	aMatrix<long double>* XprimeYmatrix = XprimeMatrix->Multiply(*Ymatrix);

	delete Bmatrix; Bmatrix = NULL;
	Bmatrix = invXprimeXmatrix->Multiply(*XprimeYmatrix); //b = ((X'X)^-1) X'y
	BmatrixH.Bind(Bmatrix);

	aMatrix<long double>* BprimeMatrix = Bmatrix->Transpose();
	aMatrix<long double>* XBmatrix = Xmatrix->Multiply(*Bmatrix);
	XBmatrix->Multiply(-1);

	delete Ematrix; Ematrix = NULL;
	Ematrix = Ymatrix->Add(*XBmatrix); // e = y-Xb
	EmatrixH.Bind(Ematrix);

	aMatrix<long double>* BprimeXprimeYmatrix = BprimeMatrix->Multiply(*XprimeYmatrix);

	aMatrix<long double>* oneMatrix = NEW aMatrix<long double>((long double)1.0,YprimeMatrix->cols(), 1);
	aMatrix<long double>* onePrimeMatrix = oneMatrix->Transpose();

	aMatrix<long double>* Yprime1matrix = YprimeMatrix->Multiply(*oneMatrix);

	aMatrix<long double>* onePrimeYmatrix = onePrimeMatrix->Multiply(*Ymatrix);

	aMatrix<long double>* Yprime11primeYmatrix = Yprime1matrix->Multiply(*onePrimeYmatrix);
	Yprime11primeYmatrix->Multiply(- 1.0 / YprimeMatrix->cols());


	aMatrix<long double>* SSTOmatrix  = YprimeYmatrix->Add(*Yprime11primeYmatrix);
	SSTO = (*SSTOmatrix)[0][0];
	aMatrix<long double>* SSRmatrix  = BprimeXprimeYmatrix->Add(*Yprime11primeYmatrix);
	SSR =  (*SSRmatrix)[0][0];
	BprimeXprimeYmatrix->Multiply(-1.0);
	aMatrix<long double>* SSEmatrix = YprimeYmatrix->Add(*BprimeXprimeYmatrix);
	SSE =  (*SSEmatrix)[0][0];

	delete SSTOmatrix; SSTOmatrix = NULL;
	delete SSRmatrix; SSRmatrix = NULL;
	delete SSEmatrix; SSEmatrix = NULL;
	delete XBmatrix; XBmatrix = NULL;
	delete BprimeMatrix; BprimeMatrix = NULL;
	delete BprimeXprimeYmatrix; BprimeXprimeYmatrix = NULL;
	delete XprimeYmatrix; XprimeYmatrix = NULL;
	delete XprimeMatrix; XprimeMatrix = NULL;
	delete YprimeMatrix; YprimeMatrix = NULL;
	if (XprimeXexists){
		delete XprimeXmatrix; XprimeXmatrix = NULL;}
	delete YprimeYmatrix; YprimeYmatrix = NULL;
	delete oneMatrix; oneMatrix = NULL;
	delete onePrimeMatrix; onePrimeMatrix = NULL;
	delete Yprime1matrix; Yprime1matrix = NULL;
	delete onePrimeYmatrix; onePrimeYmatrix = NULL;
	delete Yprime11primeYmatrix; Yprime11primeYmatrix = NULL;
	Bmatrix = NULL;//forget this; it is attached elsewhere
	Ematrix = NULL;//forget this; it is attached elsewhere
	}//end  GLM::RegressionSS
*/

void GLM::RegressionSS (aMatrix<long double>*Xmatrix, 
					aMatrix<long double>*Ymatrix, 
					matrixHandle<long double>& SSRmatrixH, 
					matrixHandle<long double>& SSEmatrixH, 
					matrixHandle<long double>& SSTOmatrixH,
					matrixHandle<long double>& invXprimeXmatrixH, 
					matrixHandle<long double>& BmatrixH,  
					matrixHandle<long double>& EmatrixH){
	//variable matrices entered as empty matrices bound to handles
	aMatrix<long double>* XprimeMatrix = Xmatrix->Transpose();
	aMatrix<long double>* XprimeXmatrix = NULL;

	aMatrix<long double>* invXprimeXmatrix = invXprimeXmatrixH.mat;
	aMatrix<long double>* Bmatrix = BmatrixH.mat;
	aMatrix<long double>* Ematrix = EmatrixH.mat;
	aMatrix<long double>* SSRmatrix = SSRmatrixH.mat;
	aMatrix<long double>* SSEmatrix = SSEmatrixH.mat;
	aMatrix<long double>* SSTOmatrix = SSTOmatrixH.mat;
	BmatrixH.Detach();//these will be replaced below
	EmatrixH.Detach();
	SSRmatrixH.Detach();
	SSEmatrixH.Detach();
	SSTOmatrixH.Detach();
	int XprimeXexists = 0;//false
	if (invXprimeXmatrix->Empty()){ 
		//make on the first analysis & save elsewhere, then re-use in bootstrap analyses to save time
		XprimeXexists = 1;//true
		XprimeXmatrix = XprimeMatrix->Multiply(*Xmatrix);
		invXprimeXmatrixH.Detach();
		delete invXprimeXmatrix; invXprimeXmatrix=NULL;
		invXprimeXmatrix = XprimeXmatrix->Inverse();//slow, but no bugs!
		invXprimeXmatrixH.Bind(invXprimeXmatrix);
//		cout<<"matrix of X: "<<endl;
//		Xmatrix->Print(cout);
//		cout<<"matrix of (X''X)^-1 : "<<endl;
//		invXprimeXmatrix->Print(cout);
		}//end if invXprimeXmatrix =NULL

	aMatrix<long double>* YprimeMatrix = Ymatrix->Transpose();  //y = bX + e

	aMatrix<long double>* YprimeYmatrix = YprimeMatrix->Multiply(*Ymatrix);

	aMatrix<long double>* XprimeYmatrix = XprimeMatrix->Multiply(*Ymatrix);

	delete Bmatrix; Bmatrix = NULL;
	Bmatrix = invXprimeXmatrix->Multiply(*XprimeYmatrix); //b = ((X'X)^-1) X'y
	BmatrixH.Bind(Bmatrix);

	aMatrix<long double>* BprimeMatrix = Bmatrix->Transpose();
	aMatrix<long double>* XBmatrix = Xmatrix->Multiply(*Bmatrix);
	XBmatrix->Multiply(-1);

	delete Ematrix; Ematrix = NULL;
	Ematrix = Ymatrix->Add(*XBmatrix); // e = y-Xb
	EmatrixH.Bind(Ematrix);

	aMatrix<long double>* BprimeXprimeYmatrix = BprimeMatrix->Multiply(*XprimeYmatrix);

	aMatrix<long double>* oneMatrix = NEW aMatrix<long double>((long double)1.0,YprimeMatrix->cols(), 1);
	aMatrix<long double>* onePrimeMatrix = oneMatrix->Transpose();

	aMatrix<long double>* Yprime1matrix = YprimeMatrix->Multiply(*oneMatrix);

	aMatrix<long double>* onePrimeYmatrix = onePrimeMatrix->Multiply(*Ymatrix);

	aMatrix<long double>* Yprime11primeYmatrix = Yprime1matrix->Multiply(*onePrimeYmatrix);
	Yprime11primeYmatrix->Multiply((long double)(- 1.0) / (long double)(YprimeMatrix->cols()));

	delete SSRmatrix; SSRmatrix=NULL;
	delete SSEmatrix; SSEmatrix=NULL;
	delete SSTOmatrix; SSTOmatrix=NULL;
	SSTOmatrix  = YprimeYmatrix->Add(*Yprime11primeYmatrix);
	SSRmatrix  = BprimeXprimeYmatrix->Add(*Yprime11primeYmatrix);
	BprimeXprimeYmatrix->Multiply((long double)-1.0);
	SSEmatrix = YprimeYmatrix->Add(*BprimeXprimeYmatrix);
	SSTOmatrixH.Bind(SSTOmatrix);
	SSRmatrixH.Bind(SSRmatrix);
	SSEmatrixH.Bind(SSEmatrix);

	delete XBmatrix; XBmatrix = NULL;
	delete BprimeMatrix; BprimeMatrix = NULL;
	delete BprimeXprimeYmatrix; BprimeXprimeYmatrix = NULL;
	delete XprimeYmatrix; XprimeYmatrix = NULL;
	delete XprimeMatrix; XprimeMatrix = NULL;
	delete YprimeMatrix; YprimeMatrix = NULL;
	if (XprimeXexists){
		delete XprimeXmatrix; XprimeXmatrix = NULL;}
	delete YprimeYmatrix; YprimeYmatrix = NULL;
	delete oneMatrix; oneMatrix = NULL;
	delete onePrimeMatrix; onePrimeMatrix = NULL;
	delete Yprime1matrix; Yprime1matrix = NULL;
	delete onePrimeYmatrix; onePrimeYmatrix = NULL;
	delete Yprime11primeYmatrix; Yprime11primeYmatrix = NULL;
	Bmatrix = NULL;//forget this; it is attached elsewhere
	Ematrix = NULL;//forget this; it is attached elsewhere
	}//end  GLM::RegressionSS


void GLM::RegressionSS (aMatrix<long double>*Xmatrix, 
					aMatrix<long double>*Ymatrix, 
					long double& SSR, long double& SSE, long double& SSTO,
					matrixHandle<long double>& invXprimeXmatrixH){
	matrixHandle<long double> SSRmatrixH, SSEmatrixH, SSTOmatrixH, BmatrixH, EmatrixH;//empty
	RegressionSS (Xmatrix, Ymatrix,SSRmatrixH,SSEmatrixH, SSTOmatrixH,invXprimeXmatrixH,BmatrixH, EmatrixH);
	aMatrix<long double>* SSRmat_ = SSRmatrixH.mat;
	aMatrix<long double>* SSEmat_ = SSEmatrixH.mat;
	aMatrix<long double>* SSTOmat_ = SSTOmatrixH.mat;
	aMatrix<long double>& SSRmat = *SSRmat_;//notational convenience
	aMatrix<long double>& SSEmat = *SSEmat_;
	aMatrix<long double>& SSTOmat = *SSTOmat_;
	SSR = SSRmat[0][0];
	SSE = SSEmat[0][0];
	SSTO = SSTOmat[0][0];
	SSRmat_ =  SSEmat_ =  SSTOmat_ = NULL;//forget; disposed with handle
	}//end  GLM::RegressionSS


/*
void GLM::RegressionSS (aMatrix<long double>*Xmatrix, 
					aMatrix<long double>*Ymatrix, 
					long double& SSR, long double& SSE, long double& SSTO,
					matrixHandle<long double>& invXprimeXmatrixH){
	//variable matrices entered as empty matrices bound to handles
	aMatrix<long double>* XprimeMatrix = Xmatrix->Transpose();
	aMatrix<long double>* XprimeXmatrix = NULL;

	aMatrix<long double>* invXprimeXmatrix = invXprimeXmatrixH.mat;
	int XprimeXexists = 0;//false
	if (invXprimeXmatrix->Empty()){ 
		//make on the first analysis & save elsewhere, then re-use in bootstrap analyses to save time
		XprimeXexists = 1;//true
		XprimeXmatrix = XprimeMatrix->Multiply(*Xmatrix);
		invXprimeXmatrixH.Detach();
		delete invXprimeXmatrix; invXprimeXmatrix=NULL;
		invXprimeXmatrix = XprimeXmatrix->Inverse();//slow, but no bugs!
		invXprimeXmatrixH.Bind(invXprimeXmatrix);
		}//end if invXprimeXmatrix =NULL

	aMatrix<long double>* YprimeMatrix = Ymatrix->Transpose();  //y = bX + e

	aMatrix<long double>* YprimeYmatrix = YprimeMatrix->Multiply(*Ymatrix);

	aMatrix<long double>* XprimeYmatrix = XprimeMatrix->Multiply(*Ymatrix);

	aMatrix<long double>* Bmatrix = invXprimeXmatrix->Multiply(*XprimeYmatrix); //b = ((X'X)^-1) X'y

	aMatrix<long double>* BprimeMatrix = Bmatrix->Transpose();

	aMatrix<long double>* BprimeXprimeYmatrix = BprimeMatrix->Multiply(*XprimeYmatrix);

	aMatrix<long double>* oneMatrix = NEW aMatrix<long double>((long double)1.0,YprimeMatrix->cols(), 1);
	aMatrix<long double>* onePrimeMatrix = oneMatrix->Transpose();

	aMatrix<long double>* Yprime1matrix = YprimeMatrix->Multiply(*oneMatrix);

	aMatrix<long double>* onePrimeYmatrix = onePrimeMatrix->Multiply(*Ymatrix);

	aMatrix<long double>* Yprime11primeYmatrix = Yprime1matrix->Multiply(*onePrimeYmatrix);
	Yprime11primeYmatrix->Multiply((long double)- 1.0 / YprimeMatrix->cols());


	aMatrix<long double>* SSTOmatrix  = YprimeYmatrix->Add(*Yprime11primeYmatrix);
	SSTO = (*SSTOmatrix)[0][0];
	aMatrix<long double>* SSRmatrix  = BprimeXprimeYmatrix->Add(*Yprime11primeYmatrix);
	SSR =  (*SSRmatrix)[0][0];
	BprimeXprimeYmatrix->Multiply(-1.0);
	aMatrix<long double>* SSEmatrix = YprimeYmatrix->Add(*BprimeXprimeYmatrix);
	SSE =  (*SSEmatrix)[0][0];

	delete SSTOmatrix; SSTOmatrix = NULL;
	delete SSRmatrix; SSRmatrix = NULL;
	delete SSEmatrix; SSEmatrix = NULL;
	delete BprimeMatrix; BprimeMatrix = NULL;
	delete BprimeXprimeYmatrix; BprimeXprimeYmatrix = NULL;
	delete XprimeYmatrix; XprimeYmatrix = NULL;
	delete XprimeMatrix; XprimeMatrix = NULL;
	delete YprimeMatrix; YprimeMatrix = NULL;
	if (XprimeXexists){
		delete XprimeXmatrix; XprimeXmatrix = NULL;}
	delete YprimeYmatrix; YprimeYmatrix = NULL;
	delete oneMatrix; oneMatrix = NULL;
	delete onePrimeMatrix; onePrimeMatrix = NULL;
	delete Yprime1matrix; Yprime1matrix = NULL;
	delete onePrimeYmatrix; onePrimeYmatrix = NULL;
	delete Yprime11primeYmatrix; Yprime11primeYmatrix = NULL;
	delete Bmatrix; Bmatrix = NULL;
	}//end  GLM::RegressionSS
*/


void GLM::VarianceComponentsANOVA (aMatrix<long double>* Ymatrix, ANOVATable& theANOVATable, long  bootRep){
	//this calculates Reduced forms of sums of squares arranged
	//hierarchically, so that variance components can be calculated off of them
	//df, MS, etc are not calculated
	//Reduces sums of squares are stored in SS_; variance components are stored in MS_
	long double fullSSR=0.0, fullSSE=0.0, fullSSTO=0.0, factorSS=0.0;
	long double reducedSSR=0.0, reducedSSE=0.0, reducedSSTO=0.0;
	int calculateRecyclables=0;//first time through, store info; next times through, use stored info
	if(reuseDesign() && ! recyclablesCalculated()){
		calculateRecyclables=1;}//true
	//make the full design matrix
//	MatrixList* reducedDesignComponents = NEW MatrixList(designList);
	MatrixList* reducedDesignComponents = NEW MatrixList(incedenceMatrixList);
	aMatrix<long double>* Xmatrix = MakeDesignMatrix(*reducedDesignComponents, 0);//=[1 Xa Xb ...Xf] with f factors in model
//	aString xmessage("X matrix for full model ");
//	Xmatrix->Print(xmessage, "Xmatrices");
	delete reducedDesignComponents; reducedDesignComponents = NULL;
	long n = Ymatrix->rows();

	aMatrix<long double>* fullEmatrix = NEW aMatrix<long double>();	//residuals
	matrixHandle<long double> fullEmatrixH(fullEmatrix);	//residuals
	aMatrix<long double>* fullBmatrix = NEW aMatrix<long double>();	//reg. coeffs.
	matrixHandle<long double> fullBmatrixH(fullBmatrix);	//reg. coeffs.
	if (calculateRecyclables == 0){// reusing same design matrices
			//use the saved invXprimeXmatrix
//			aMatrix<double>& invXprimeXmatrixA = (*invXprimeXList)[0];
			aMatrix<long double>& invXprimeXmatrixA = invXprimeXList->Last();
			matrixHandle<long double> invXprimeXmatrixH(invXprimeXmatrixA);
			RegressionSS(Xmatrix, Ymatrix, fullSSR, fullSSE, fullSSTO, invXprimeXmatrixH, fullBmatrixH, fullEmatrixH);
			invXprimeXmatrixH.Detach();
			}
		else{
			// create the invXprimeXmatrix
			aMatrix<long double>* invXprimeXmatrixB = NEW aMatrix<long double>();//send empty matrix
			matrixHandle<long double> invXprimeXmatrixH(invXprimeXmatrixB);
			RegressionSS(Xmatrix, Ymatrix, fullSSR, fullSSE, fullSSTO, invXprimeXmatrixH, fullBmatrixH, fullEmatrixH);
			invXprimeXmatrixB = invXprimeXmatrixH.mat;
			invXprimeXList->Append(invXprimeXmatrixB);  //full model goes in position 0
			invXprimeXmatrixH.Detach();
			invXprimeXmatrixB = NULL;//forget it here; destroyed elsewhere
			}
	delete Xmatrix; Xmatrix=NULL;

	theANOVATable.SetReducedFormSS(fullSSE, theANOVATable.len() - 2, bootRep);
	theANOVATable.SetReducedFormSS(fullSSTO, theANOVATable.len()-1, bootRep);


	aList<int> omitFactors;//factors to omit when constructing reduced design matrices
	MatrixList bigMList;//functions of reduced design matrices
	aList<int> rankList;//ranks of design matrices
	MatrixList ZZprimeList;//functions of reduced design matrices
	for(int i= designList->len()-1;i>=1;i--){
		//make the reduced design matrix cumulatively omitting factor i
		//also accumulate some information to use below
		omitFactors.Append(i);
//		reducedDesignComponents = NEW MatrixList(designList);
		reducedDesignComponents = NEW MatrixList(incedenceMatrixList);
		aMatrix<long double>* Xmatrix = MakeDesignMatrix(*reducedDesignComponents, omitFactors);//=[1 Xa Xb ... X(f-i)]
//		aString xmessage("X matrix for i = ");xmessage.cat(i);
//		Xmatrix->Print(xmessage, "Xmatrices");
		delete reducedDesignComponents; reducedDesignComponents = NULL;

		aMatrix<long double>* reducedEmatrix =  NEW aMatrix<long double>();	//residuals
		matrixHandle<long double> reducedEmatrixH(reducedEmatrix);	//residuals
		
		aMatrix<long double>* reducedBmatrix =  NEW aMatrix<long double>();	//reg. coeffs.
		matrixHandle<long double> reducedBmatrixH(reducedBmatrix);	//reg. coeffs.

		if (calculateRecyclables == 0){//then this must be an analysis using a previously formulated design
				//use a saved invXprimeXmatrix
				aMatrix<long double>& invXprimeXmatrixC =(*invXprimeXList)[i-1];
				matrixHandle<long double> invXprimeXmatrixH(invXprimeXmatrixC);
				RegressionSS(Xmatrix, Ymatrix, reducedSSR, reducedSSE, reducedSSTO, invXprimeXmatrixH, reducedBmatrixH, reducedEmatrixH);
				invXprimeXmatrixH.Detach();//invXprimeXmatrix returns unchanged, and is deleted elsewhere
				}
			else{
				//create the invXprimeXmatrix
				aMatrix<long double>* invXprimeXmatrixD = NEW aMatrix<long double>();//send empty matrix
				matrixHandle<long double> invXprimeXmatrixH(invXprimeXmatrixD);
				RegressionSS(Xmatrix, Ymatrix, reducedSSR, reducedSSE, reducedSSTO, invXprimeXmatrixH, reducedBmatrixH, reducedEmatrixH);
				invXprimeXmatrixD = invXprimeXmatrixH.mat;//this is newly created
				invXprimeXList->Prepend(invXprimeXmatrixD);	//this model goes in position i
				invXprimeXmatrixH.Detach();
				invXprimeXmatrixD = NULL;
				}
		delete Xmatrix; Xmatrix = NULL;
		reducedEmatrix = NULL;//forget here; deleted with the handle
		reducedBmatrix = NULL;//forget here; deleted with the handle
		factorSS = fullSSR-reducedSSR;
		theANOVATable.SetReducedFormSS(factorSS, i-1, bootRep);

		if(varCompCoeffsSupplied()==0){//do some calculations and save the answers for later
			aMatrix<long double>* imat =IdentityMatrix((long double)0.0, n);//used in calculations of bigM
			MatrixList *reducedbigMComponents = NEW MatrixList(incedenceMatrixList);
			aList<int> omitFactorsM(omitFactors);
			omitFactorsM.DeleteLast();
			aMatrix<long double>* combZmatrix = MakeDesignMatrix(*reducedbigMComponents, omitFactorsM);//Z=[1 Xa Xb ... X(f-i+1)]
			delete reducedbigMComponents; reducedbigMComponents = NULL;
			aMatrix<long double>* combZPlus = combZmatrix->MatrixPlusForm();
			int thisRank = combZmatrix->Rank();
			aMatrix<long double>* combZcombZplus =combZmatrix->Multiply(*combZPlus);
			combZcombZplus->Multiply((long double)-1.0);
			aMatrix<long double>* bigM = imat->Add(*combZcombZplus);
			bigMList.Prepend(bigM );
			rankList.Prepend(thisRank);
			bigM = NULL;//forget these
			delete  combZmatrix; combZmatrix=NULL;
			delete  combZPlus; combZPlus=NULL;
			delete  combZcombZplus; combZcombZplus=NULL;

			aMatrix<long double>& Z = (*incedenceMatrixList)[i];//=X[Xi]
			aMatrix<long double>* Zprime0 = Z.Transpose();
			aMatrix<long double>* ZZprime0 =Z.Multiply(*Zprime0);
			ZZprimeList.Prepend(ZZprime0);
			delete Zprime0; Zprime0=ZZprime0=NULL;//forget ZZprime0
			delete imat; imat = NULL;
			}//end if varCompCoeffsSupplied()==0
		}//end for i

	if(varCompCoeffsSupplied()==0){
		bigMList.Prepend(CMatrix(n));//item 0 is the C-matrix of size n

		rankList.Prepend(1);//the first item is the rank of the mean-vector 1
		rankList.Append(n);//the last item is the total sample size

		varCompCoefficients.Clear();
		aList<long double> *errorCoeff1= NEW aList<long double>();
		varCompCoefficients.Prepend(errorCoeff1);
		int prevLineRank = rankList[theANOVATable.len()-2];
		errorCoeff1->Append((long double)prevLineRank);

		for(int i = theANOVATable.len()-3;i>=0;i--){
			aList<long double> *thisLineCoeffs1= NEW aList<long double>();
			varCompCoefficients.Prepend(thisLineCoeffs1);
			aMatrix<long double>& prevBigM = bigMList[i];
			aMatrix<long double>& ZZprime = ZZprimeList[i];
			for(int j=i+1;j<theANOVATable.len()-2;++j){
				aMatrix<long double>& thisBigM = bigMList[i+1];
				aMatrix<long double>* num1Mat = prevBigM.Multiply(ZZprimeList[j]);
				aMatrix<long double>* num2Mat = thisBigM.Multiply(ZZprimeList[j]);
//				double coeffA= num1Mat->Trace();
//				double coeffB= num2Mat->Trace();
				long double coeff = num1Mat->Trace() - num2Mat->Trace();
				thisLineCoeffs1->Append(coeff);
				delete num1Mat; delete num2Mat; num1Mat=num2Mat=NULL;
				}//end for j
			int rankdif = rankList[i+1]-rankList[i];
			thisLineCoeffs1->Append((long double)rankdif);
			aMatrix<long double>* denomMat = prevBigM.Multiply(ZZprime);
			double denom = denomMat->Trace();
			thisLineCoeffs1->Append(denom);
			delete denomMat; denomMat=NULL;
			thisLineCoeffs1=NULL;//forget this
			}//end for i
		errorCoeff1=NULL;//forget this
		}//end if varCompCoeffsSupplied()==0

	//calculate the variance components

	//The corrects the reduced form of SS; those beyond len()-3 are already correct
	for(int i=0;i<theANOVATable.len()-3;++i){
		ANOVALineBase& thisLineR_=theANOVATable.GetANOVALine(i);
		ANOVALineBase& nextLineR_=theANOVATable.GetANOVALine(i+1);
		ANOVALine *thisLineR = (ANOVALine*) &thisLineR_;
		ANOVALine *nextLineR = (ANOVALine*) &nextLineR_;
		thisLineR->reducedFormSS(thisLineR->reducedFormSS(bootRep)-nextLineR->reducedFormSS(bootRep), bootRep);
		}//end for i

	//calculate the coefficients of the variance components
	//calculate the error variance component
	ANOVALineBase& errorLine_=theANOVATable.GetANOVALine(theANOVATable.len()-2);
	ANOVALine *eLine = (ANOVALine*) &errorLine_;
	aList<long double>& errorCoeff=varCompCoefficients.Last();
	double prevLineRank =errorCoeff[0];
	eLine->varianceComponent(eLine->reducedFormSS(bootRep)/((long double)n-prevLineRank),bootRep);

	//calculate the remaining variance components
	for(int i = theANOVATable.len()-3;i>=0;i--){
		aList<long double>& thisLineCoeff=varCompCoefficients[i];
		ANOVALineBase& thisLine_=theANOVATable.GetANOVALine(i);
		ANOVALineBase& nextLine_=theANOVATable.GetANOVALine(i+1);
		ANOVALine *thisLine = (ANOVALine*) &thisLine_;
		ANOVALine *nextLine = (ANOVALine*) &nextLine_;
		long double sum=thisLine->reducedFormSS(bootRep);
		long double varComp=0.0;
		for(int j=i+1;j<theANOVATable.len()-2;++j){
			ANOVALineBase& nextLineJ_=theANOVATable.GetANOVALine(j);
			ANOVALine *nextLineJ = (ANOVALine*) &nextLineJ_;
			varComp = nextLineJ->varianceComponent(bootRep);
			long double coeff = thisLineCoeff[j-(i+1)];
			sum -= coeff * varComp;
			}//end for j
		varComp = eLine->varianceComponent(bootRep);
		long double rankdif = thisLineCoeff[thisLineCoeff.len()-2];//second to last
		sum -= varComp * rankdif;
		long double denom = thisLineCoeff.Last();
		varComp = sum/denom;
		thisLine->varianceComponent(varComp, bootRep);
		}//end for i

	SetRecyclablesCalculated(1);//true
	varCompCoeffsSupplied(1);//true
	fullEmatrix = NULL;//forget here; deleted with the handle
	fullBmatrix = NULL;//forget here; deleted with the handle
	}//end GLM::VarianceComponentsANOVA





 int operator== (GLM& a, GLM& b){
		return ((a.designList == b.designList)&&
			      (a.incedenceMatrixList==b.incedenceMatrixList)&&
			      (a.invXprimeXList == b.invXprimeXList)&&
			      (a.varCompCoefficients ==b.varCompCoefficients));
		}


int	operator!= (GLM& a, GLM& b){
						return ! (a==b);}
