/*******************************************
*
*
*	MANOVALine.cpp
*
*	A class that implements a line in a MANOVA table
*
*
************************************************/

#ifndef MANOVALinecp
#define MANOVALinecp = 1


#include "MANOVALine.h"
#include "GeneralLinearModel.h"
#include "MANOVATable.h"
#include "aVector.cpp"
#include "aMatrix.cpp"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "BootList.cpp"




void MANOVALine::F(ANOVALineBase& total_Line, long bootRep){
	//use Rao's (1952) approximation of F, given in Cooley & Lohnes 1971 p.227-228
//	SetMultivariateDF(total_Line);
//	MANOVALine *denomLine_ = (MANOVALine*) &denomLine;
	long double p = dependentVariables_;//number of design parameters 
	long double q = df_;// =(g-1) in C&L notation --> number of groups - 1
//	long double q = levels()-1;// =(g-1) in C&L notation --> number of groups - 1
	long double s = 1.0, zero = (long double)0.0, one = (long double)1.0, two = (long double) 2.0;
	if(p*p + q*q > (long double) 5.0){
		s =sqrt( (p*p*q*q - (long double)4.0)/(p*p + q*q - (long double)5.0) );}
	long double y = exp( ln(WilksLambda_)/s);
	long double df_t = total_Line.df_;//should be N-1
	df1WL_ = p * q;
	df2WL_ = s * (df_t - (p + q + one)/two) - (p * q - two)/two;
	if((y==zero) || df1WL_ == zero) return;
	long double ff = ((one - y)/y) * (df2WL_/df1WL_);
	if(bootRep == 0){
			F_Wilks_ = ff;}
		else{
			bootF_Wilks_ = ff;}	
	}//end MANOVALine::F(ANOVALineBase&, long)





void MANOVALine::Print (ostream& outFile, int /*titleLen*/, StringList& dependentList, long bootRep){
	if(effectType() ==  totalLine || effectType() ==  errorLine) return;//not printed in MANOVA tables
	StringList toprint;
	toprint.Append(NEW aString());
	toprint.Append(NEW aString(title()));
	aString wholeLine, titleline("test\tvalue\tdf[num]\tdf[denom]");
	if(df_==1){titleline.cat("\tF[exact]");}else{titleline.cat("\tF[approx]");}
	titleline.cat("\tProb>F");
	if(bootRep>0) titleline.cat("\tProb(boot)>F");
	toprint.Append(NEW aString(titleline));
	if(df_==1){
			wholeLine="F test\t";
			wholeLine.cat(RoysMaxRoot(0),6,8);
			wholeLine.cat("\t");  wholeLine.cat(df1RMR_);
			wholeLine.cat("\t");  wholeLine.cat(df2RMR_);
			wholeLine.cat("\t");  wholeLine.cat(F_Roys(0));
			wholeLine.cat("\t");  wholeLine.cat(P_Roys_);
			wholeLine.cat("\t");  wholeLine.cat(bootP_Roys_);
			toprint.Append(NEW aString(wholeLine));}
		else{
			wholeLine ="Wilk\'s Lambda\t";
			wholeLine.cat(WilksLambda(0),6,8);
			wholeLine.cat("\t");  wholeLine.cat(df1WL_,3,1);
			wholeLine.cat("\t");  wholeLine.cat(df2WL_,3,1);
			wholeLine.cat("\t");  wholeLine.cat(F_Wilks(0));
			wholeLine.cat("\t");  wholeLine.cat(P_Wilks_);
			wholeLine.cat("\t");  wholeLine.cat(bootP_Wilks_);
			toprint.Append(NEW aString(wholeLine));
			wholeLine = "Pillai\'s Trace\t";
			wholeLine.cat(PillaisTrace(0),6,8);
			wholeLine.cat("\t");	wholeLine.cat(df1PT_);
			wholeLine.cat("\t");	wholeLine.cat(df2PT_);
			wholeLine.cat("\t");	wholeLine.cat(F_Pillai(0));
			wholeLine.cat("\t");	wholeLine.cat(P_Pillai_);
			wholeLine.cat("\t");	wholeLine.cat(bootP_Pillai_);
			toprint.Append(NEW aString(wholeLine));
			wholeLine = "Hotelling Lawley\t";
			wholeLine.cat(HotellingLawley(0),6,8);
			wholeLine.cat("\t");	wholeLine.cat(df1HLT_);
			wholeLine.cat("\t");	wholeLine.cat(df2HLT_);
			wholeLine.cat("\t");	wholeLine.cat(F_HLtrace(0));
			wholeLine.cat("\t");	wholeLine.cat(P_HLtrace_);
			wholeLine.cat("\t");	wholeLine.cat(bootP_HLtrace_);
			toprint.Append(NEW aString(wholeLine));
			wholeLine = "Roy\'s Max Root\t";
			wholeLine.cat(RoysMaxRoot(0),6,8);
			wholeLine.cat("\t");	wholeLine.cat(df1RMR_);
			wholeLine.cat("\t");	wholeLine.cat(df2RMR_);
			wholeLine.cat("\t");	wholeLine.cat(F_Roys(0));
			wholeLine.cat("\t");	wholeLine.cat(P_Roys_);
			wholeLine.cat("\t");	wholeLine.cat(bootP_Roys_);
			toprint.Append(NEW aString(wholeLine));
			}
	int dv,c;
	wholeLine = "covariance matrix:";
	if(bootRep>0) {wholeLine.cat("\t\tP[cov]:");}
	toprint.Append(NEW aString(wholeLine));
/*	wholeLine = "\t";
	for(dv=0;dv<dependentList.len();++dv){
		wholeLine.cat("\t");wholeLine.cat(dependentList[dv]);}
	if(bootRep>0) {
		wholeLine.cat("\t");
		for(dv=0;dv<dependentList.len();++dv){
			wholeLine.cat("\t");wholeLine.cat(dependentList[dv]);}}
	toprint.Append(NEW aString(wholeLine));
*/	for(dv=0;dv<VarCov_.rows();++dv){
		wholeLine = "\t\t";wholeLine.cat(dependentList[dv]);
		for(c=0;c<VarCov_.cols();++c){
			wholeLine.cat("\t");wholeLine.cat(VarCov_[dv][c]);}
		if(bootRep>0){
			wholeLine.cat("\t");wholeLine.cat(dependentList[dv]);
			for(c=0;c<VarCov_.cols();++c){
				wholeLine.cat("\t");wholeLine.cat(VarCovP_[dv][c],2,4);}
			}
		toprint.Append(NEW aString(wholeLine));
		}//rows
	wholeLine = "correlation matrix:";
	if(bootRep>0) {wholeLine.cat("\t\tP[corr]:");}
	toprint.Append(NEW aString(wholeLine));
/*	wholeLine = "\t";
	for(dv=0;dv<dependentList.len();++dv){
		wholeLine.cat("\t");wholeLine.cat(dependentList[dv]);}
	if(bootRep>0) {
		wholeLine.cat("\t");
		for(dv=0;dv<dependentList.len();++dv){
			wholeLine.cat("\t");wholeLine.cat(dependentList[dv]);}}
	toprint.Append(NEW aString(wholeLine));
*/	for(dv=0;dv<Corr_.rows();++dv){
		wholeLine = "\t\t";wholeLine.cat(dependentList[dv]);
		for(c=0;c<Corr_.cols();++c){
			wholeLine.cat("\t");wholeLine.cat(Corr_[dv][c]);}
		if(bootRep>0){
			wholeLine.cat("\t");wholeLine.cat(dependentList[dv]);
			for(c=0;c<Corr_.cols();++c){
				wholeLine.cat("\t");wholeLine.cat(CorrP_[dv][c],2,4);}
			}
		toprint.Append(NEW aString(wholeLine));
		}//rows
	
	for(int s=0;s<toprint.len();++s){
		outFile<<toprint[s]<<endl;}
	}//end MANOVALine::Print


void MANOVALine::PrintWithinLevelCorrelations (ostream& outFile, StringList& dependentList, long bootReps){
	if(effectType() ==  totalLine || effectType() ==  errorLine) return;//not printed in MANOVA tables
	StringList toprint;
	toprint.Append(NEW aString());
	toprint.Append(NEW aString(title()));
	toprint.Append(NEW aString("covariances & correlations for each factor level:"));
	aString wholeLine;
	for(int level=0;level<levels();++level){
		wholeLine = "\t";
		wholeLine.cat(levelNames_[level]);  
		toprint.Append(NEW aString(wholeLine));
		toprint.Append(NEW aString("\tvariance-covariance matrix"));
		for(int dv=0;dv<CovLevelList_[level].rows();++dv){
			wholeLine = "\t\t\t";wholeLine.cat(dependentList[dv]);
			for(int c=0;c<CovLevelList_[level].cols();++c){
				wholeLine.cat("\t");wholeLine.cat(CovLevelList_[level][dv][c],1,4);}
			toprint.Append(NEW aString(wholeLine));
			}//dv rows
		if(bootReps>0) {
			wholeLine="\t\tcorresponding P values from resampling:";
			toprint.Append(NEW aString(wholeLine));
			for(int dvB=0;dvB<CovLevelList_[level].rows();++dvB){
				wholeLine = "\t\t\t";wholeLine.cat(dependentList[dvB]);
				for(int cB=0;cB<CovLevelList_[level].cols();++cB){
					//make up a bootList and calculate the P value
					double pval=1.0;
//					if(dvB != cB){//it's a meaningful test
					BootList<long double> bootCov;
					for(long b=0;b<bootReps;++b){
						bootCov.Append(CovLevelBootList_[b][level][dvB][cB]);}
					pval = bootCov.Get2TailP(CovLevelList_[level][dvB][cB]);
//						}
					wholeLine.cat("\t");wholeLine.cat(pval,1,6);
					}//cB cols
				toprint.Append(NEW aString(wholeLine));
				}//dvB rows			
			}//bootRep>0
		toprint.Append(NEW aString("\tcorrelation matrix"));
		for(int dv=0;dv<CorrLevelList_[level].rows();++dv){
			wholeLine = "\t\t\t";wholeLine.cat(dependentList[dv]);
			for(int c=0;c<CorrLevelList_[level].cols();++c){
				wholeLine.cat("\t");wholeLine.cat(CorrLevelList_[level][dv][c],1,4);}
			toprint.Append(NEW aString(wholeLine));
			}//dv rows
		if(bootReps>0) {
			wholeLine="\t\tcorresponding P values from resampling:";
			toprint.Append(NEW aString(wholeLine));
			for(int dvB=0;dvB<CorrLevelList_[level].rows();++dvB){
				wholeLine = "\t\t\t";wholeLine.cat(dependentList[dvB]);
				for(int cB=0;cB<CorrLevelList_[level].cols();++cB){
					//make up a bootList and calculate the P value
					double pval=1.0;
					if(dvB != cB){//it's a meaningful test
						BootList<long double> bootCorr;
						for(long b=0;b<bootReps;++b){
							bootCorr.Append(CorrLevelBootList_[b][level][dvB][cB]);}
						pval = bootCorr.Get2TailP(CorrLevelList_[level][dvB][cB]);
						}
					wholeLine.cat("\t");wholeLine.cat(pval,1,4);
					}//cB cols
				toprint.Append(NEW aString(wholeLine));
				}//dvB rows			
			}//bootRep>0
		}//level
	for(int s=0;s<toprint.len();++s){
		outFile<<toprint[s]<<endl;}

	}//MANOVALine::PrintWithinLevelCorrelations

/*
void MANOVALine::SetMultivariateDF(ANOVALineBase& total_Line){
	MANOVALine *total_Line_ = (MANOVALine*) &total_Line;
//	aMatrix<long double> *SSerrorM, *SSeffectM;
//	SSerrorM = &(denomLine_->SS_);
//	SSeffectM = &(SS_);
	long double p = dependentVariables_;
	long double df_effect = df_;//=g-1 
//	long double df_error = (long double)denomLine_->df() * (p - (long double)1.0);//is this right? no
	long double df_total = (long double)total_Line_->df();
	df1WL_ = p * df_effect;
	long double s = 1.0;
	if(p*p + df_effect*df_effect > (long double) 5.0){
		s =sqrt( (p*p*df_effect*df_effect - (long double)4.0)/(p*p + df_effect*df_effect - (long double)5.0) );}
	df2WL_ = s*(df_error - (p - df_effect + (long double)1.0)/((long double)2.0));
	df2WL_ -= (p*df_effect - (long double)2.0)/((long double)2.0);
	}//end MANOVALine::SetMultivariateDF(ANOVALineBase&)
*/


void MANOVALine::SS (aMatrix<long double>& vals, long bootRep){
	if( bootRep == 0){SS_ = vals;}
		else{bootSS_ = vals;}
	}//end MANOVALine::SS

/*
void MANOVALine::reducedFormSS (aMatrix<long double>& valS, long bootRep){
	if( bootRep == 0){reducedFormSS_ = valS;}
		else{reducedFormbootSS_ = valS;}
	}//end MANOVALine::reducedFormSS
*/

void MANOVALine::VarCov (aMatrix<long double>& vals, long bootRep, aMatrix<long double> *residuals){
	int r,c;
	long double rowSigma, rowVar,colSigma,colVar;
	if( bootRep == 0){
			VarCov_ = Corr_ = vals;
			for(r=0;r<vals.rows();++r){
				Corr_[r][r] = (long double) 1.0;
				rowSigma=(long double)0.0;
				rowVar = VarCov_[r][r];
				if(rowVar>(long double)0.0) rowSigma = sqrt(rowVar);			
				for(c=0;c<r;++c){
					colSigma=(long double)0.0;
					colVar = VarCov_[c][c];
					if(colVar>(long double)0.0) colSigma = sqrt(colVar);			
//					Corr_[c][r] = Corr_[r][c] = VarCov_[r][c] / (sqrt(VarCov_[r][r])*sqrt(VarCov_[c][c]));
					if(rowSigma > (long double)0.0 && colSigma>(long double)0.0){
							Corr_[c][r] = Corr_[r][c] = VarCov_[r][c] / (rowSigma*colSigma);}
						else{//set to zero: cases with variances having undefined non-positive values
							Corr_[c][r] = Corr_[r][c] = (long double)0.0;}
					}}

			VarCovBootList_.Clear();//set up bootlists
			for(r=0;r<vals.rows();++r){
				VarCovBootList_.Append(NEW ObjList<BootList<long double> >());
				for(c=0;c<vals.cols();++c){
					VarCovBootList_[r].Append(NEW BootList<long double>());}}
			CorrBootList_.Clear();//set up bootlists
			for(r=0;r<vals.rows();++r){
				CorrBootList_.Append(NEW ObjList<BootList<long double> >());
				for(c=0;c<vals.cols();++c){
					CorrBootList_[r].Append(NEW BootList<long double>());}}
			}
		else{
			bootVarCov_ = bootCorr_ = vals;
			for(r=0;r<vals.rows();++r){
				bootCorr_[r][r] = (long double) 1.0;
				rowSigma=(long double)0.0;
				rowVar = bootVarCov_[r][r];
				if(rowVar>(long double)0.0) rowSigma = sqrt(rowVar);			
				for(c=0;c<r;++c){
					colSigma=(long double)0.0;
					colVar = bootVarCov_[c][c];
					if(colVar>(long double)0.0) colSigma = sqrt(colVar);			
//					bootCorr_[c][r] = bootCorr_[r][c] = bootVarCov_[r][c] / (sqrt(bootVarCov_[r][r])*sqrt(bootVarCov_[c][c]));
					if(rowSigma > (long double)0.0 && colSigma>(long double)0.0){
							bootCorr_[c][r] = bootCorr_[r][c] = bootVarCov_[r][c] / (rowSigma*colSigma);}
						else{//set to zero: cases with variances having undefined non-positive values
							bootCorr_[c][r] = bootCorr_[r][c] = (long double)0.0;}
					}}
//			if(VarCovBootList_.len() < bootRep){
			for(r=0;r<vals.rows();++r){
				for(c=0;c<vals.cols();++c){
					VarCovBootList_[r][c].Append(bootVarCov_[r][c]);
					CorrBootList_[r][c].Append(bootCorr_[r][c]);
					}}
//				}
			}
	if(residuals == NULL) return;
	//otherwise process residuals to get partial correlations
	if(bootRep) {
		CovLevelBootList_.Append(NEW MatrixList());
		CorrLevelBootList_.Append(NEW MatrixList());}
	for(int level=0;level<levels();++level){
		aMatrix<long double> levelResiduals;
		int nn = levelRows_[level].len();
		aMatrix<long double> *cov = NULL;
		aMatrix<long double> *corr = NEW aMatrix<long double>(0.0,dependentVariables_,dependentVariables_);
		if(nn<=1) {cov = NEW aMatrix<long double>(0.0,dependentVariables_,dependentVariables_);}//empty
		if(nn>1){
			for(int r0=0;r0<nn;++r0){
				int row = levelRows_[level][r0];
				aVector<long double>& residualRow = (*residuals)[row];
				levelResiduals.ConcatRow(residualRow);
				}//r0
			aMatrix<long double> *LRprime = levelResiduals.Transpose();
//			aMatrix<long double> *LRprime_LR = LRprime->Multiply(levelResiduals);//SSCP matrix, since these are already residuals
			cov = LRprime->Multiply(levelResiduals);//SSCP matrix, since these are already residuals
//			LRprime_LR->Multiply(1.0/(long double) (nn-1));
			cov->Multiply(1.0/(long double) (nn-1));
//			aMatrix<long double> cov = *LRprime_LR;
			for(int r1=0;r1<dependentVariables_;++r1){
				long double s0=(long double)0.0,s1=(long double)0.0;
				 if((*cov)[r1][r1]>(long double)0.0) s0 = sqrt((*cov)[r1][r1]);
				for(int c1=0;c1<dependentVariables_;++c1){
					if(r1==c1) {(*corr)[r1][c1] = (long double)1.0; continue;}
					if((*cov)[c1][c1]>(long double)0.0) s1 = sqrt((*cov)[c1][c1]);
					if(s0 == (long double)0.0 || s1 == (long double)0.0){
							(*corr)[r1][c1] = (long double)0.0;}
						else{
							(*corr)[r1][c1] = (*cov)[r1][c1]/ (s0*s1);}
					}}
			delete LRprime; LRprime=0;
//			delete LRprime_LR; LRprime_LR=0;
			}//if nn
		if(bootRep == 0) {
				CovLevelList_.Append(cov);
				CorrLevelList_.Append(corr);
				}
			else {
				CovLevelBootList_.Last().Append(cov);
				CorrLevelBootList_.Last().Append(corr);
				}
		}//level
	}//end MANOVALine::VarCov


/*
void MANOVALine::covarianceComponents (aMatrix<long double>& vals, long bootRep){
	if( bootRep == 0){covarianceComponents_ = vals;}
		else{bootCovarianceComponents_ = vals;}
	}//end MANOVALine::varianceComponents
*/

void MANOVALine::WilksLambda(ANOVALineBase& denomLine, long bootRep){
	MANOVALine *denomLine_ = (MANOVALine*) &denomLine;
	aMatrix<long double> *SSerrorM, *SSeffectM;
	if (bootRep == 0){
			SSerrorM = &(denomLine_->SS_);
			SSeffectM = &(SS_);
			}
		else {//bootRep>0
			SSerrorM = &(denomLine_->bootSS_);
			SSeffectM = &(bootSS_);
			}
	aMatrix<long double>& SSerror = *SSerrorM;
	aMatrix<long double>& SSeffect = *SSeffectM;
	aMatrix<long double>* SSsum = SSeffect.Add(SSerror);
	long double det1 = SSerror.Determinant();
	long double det2 = SSsum->Determinant();
	aMatrix<long double> *invSSsum = SSsum->Inverse();
	aMatrix<long double> *pillaiMatrix = SSeffect.Multiply(*invSSsum);
	aMatrix<long double> *invSSerror = SSerror.Inverse();
	aMatrix<long double> *hotellingLawleyMatrix = SSeffect.Multiply(*invSSerror);
//	long double maxEigenvalue = (*hotellingLawleyMatrix)[0][0];
//	for(int r=0;r<hotellingLawleyMatrix->rows();++r){
//			maxEigenvalue = MAX(maxEigenvalue,(*hotellingLawleyMatrix)[r][r]);}
	aVector<long double> HLmatrixEigenValues((long double)0.0,hotellingLawleyMatrix->rows());
	aMatrix<long double> HLmatrixEigenVectors;
//	hotellingLawleyMatrix->Print(cout);
	hotellingLawleyMatrix->EigenCalcs(HLmatrixEigenValues,HLmatrixEigenVectors);
	long double maxEigenvalue = HLmatrixEigenValues[0];
	for(int r=0;r<HLmatrixEigenValues.len();++r){
			maxEigenvalue = MAX(maxEigenvalue,HLmatrixEigenValues[r]);}
	//set variables
	if(bootRep==0){
			if(det2>(long double) 0.0){WilksLambda_=det1/det2;}
			PillaisTrace_ = pillaiMatrix->Trace();
			HotellingLawleyTrace_ = hotellingLawleyMatrix->Trace();
			RoysLargestRoot_ = maxEigenvalue;
			}
		else{
			if(det2>(long double) 0.0){bootWilksLambda_=det1/det2;}
			WilksLambdaBootList_.Append(bootWilksLambda_);
			bootPillaisTrace_ = pillaiMatrix->Trace();
			PillaisTraceBootList_.Append(bootPillaisTrace_);
			bootHotellingLawleyTrace_ = hotellingLawleyMatrix->Trace();
			HLtraceBootList_.Append(bootHotellingLawleyTrace_);
			bootRoysLargestRoot_ = maxEigenvalue;
			RoysRootBootList_.Append(bootRoysLargestRoot_);
			}
	//set df for all but Wilks' Lambda
	long p = SSsum->Rank();
//	aMatrix<long double>* LinvXprimeX = design.Multiply(invXprimeX);
//	aMatrix<long double>* LinvXprimeXL = LinvXprimeX->Multiply(design);
//	long q = LinvXprimeXL->Rank();
	long q = df_;
	long v = denomLine.df();
	long m = (ABS(p-q)-1)/2;
	long nn = (v - p - 1)/2;
	long s = MIN(p,q);
	df1PT_ = s * (2 * m + s + 1);
	df2PT_ = s * (2 * nn + s + 1);
	df1HLT_ = s * (2 * m + s + 1);
	df2HLT_ = 2 * (nn * s + 1);
	df1RMR_ = MAX(p,q);
	df2RMR_ = v - MAX(p,q) + q;
	//set F for non-Wilks' lambda stats
	if(bootRep==0){
			F_Pillai_ = (PillaisTrace_/(s-PillaisTrace_))*(df2PT_/df1PT_);// 2/1 is right
			F_HLtrace_ = (HotellingLawleyTrace_ * df2HLT_) / (s * df1HLT_);// 2/1 is right
			F_Roys_ = (RoysLargestRoot_ * df2RMR_) / df1RMR_;// 2/1 is right
			}
		else{
			bootF_Pillai_ = (bootPillaisTrace_/(s-bootPillaisTrace_))*(df2PT_/df1PT_);// 2/1 is right
			bootF_HLtrace_ = (bootHotellingLawleyTrace_ * df2HLT_) / (s * df1HLT_);// 2/1 is right
			bootF_Roys_ = (bootRoysLargestRoot_ * df2RMR_) / df1RMR_;// 2/1 is right
			}
	//clean up
	delete SSsum; SSsum=NULL;
	delete invSSsum; invSSsum=NULL;
	delete invSSerror; invSSerror=NULL;
	delete pillaiMatrix; pillaiMatrix=NULL;
	delete hotellingLawleyMatrix; hotellingLawleyMatrix=NULL;
//	delete LinvXprimeX; LinvXprimeX=NULL;
//	delete LinvXprimeXL; LinvXprimeXL=NULL;
	}//end MANOVALine::WilksLambda(ANOVALineBase&, long)





void MANOVALine::SetP(int /*denomDF*/, int boot){//boot == 0 gives standard P, boot==1 gives bootstrap P value
	if(boot>0){
			if(df_==1){//model degenerates to an exact F test
					bootP_Roys_ = RoysRootBootList_.Get1TailP(RoysLargestRoot_,1);}
				else{
					bootP_Wilks_ = WilksLambdaBootList_.Get1TailP(WilksLambda_,0);//low not high tail
					bootP_Pillai_ = PillaisTraceBootList_.Get1TailP(PillaisTrace_,1);
					bootP_HLtrace_ = HLtraceBootList_.Get1TailP(HotellingLawleyTrace_,1);
					bootP_Roys_ = RoysRootBootList_.Get1TailP(RoysLargestRoot_,1);
//					bootP_Wilks_ = WilksLambdaBootList_.Get2TailP(WilksLambda_);
//					bootP_Pillai_ = PillaisTraceBootList_.Get2TailP(PillaisTrace_);
//					bootP_HLtrace_ = HLtraceBootList_.Get2TailP(HotellingLawleyTrace_);
//					bootP_Roys_ = RoysRootBootList_.Get2TailP(RoysLargestRoot_);
				}
			VarCovP_=VarCov_;//sets size
			for(int r=0;r<VarCov_.rows();++r){
				for(int c=0;c<VarCov_.cols();++c){
					VarCovP_[r][c] = VarCovBootList_[r][c].Get2TailP(VarCov_[r][c]);}}
			CorrP_=Corr_;//sets size
			for(int r=0;r<Corr_.rows();++r){
				for(int c=0;c<Corr_.cols();++c){
					CorrP_[r][c] = CorrBootList_[r][c].Get2TailP(Corr_[r][c]);}}
			}
//		else{
	if(df_==1){//model degenerates to an exact F test
			P_ = P_Roys_ = Fdistribution1TailedP(df1RMR_, df2RMR_, F_Roys_);}
		else{
			P_Wilks_ = Fdistribution1TailedP(df1WL_, df2WL_, F_Wilks_);
			P_HLtrace_ = Fdistribution1TailedP(df1HLT_, df2HLT_, F_HLtrace_);
			P_Pillai_ = Fdistribution1TailedP(df1PT_, df2PT_, F_Pillai_);
			P_Roys_ = Fdistribution1TailedP(df1RMR_, df2RMR_, F_Roys_);}
//			}
	}//end MANOVALine::SetP




double MANOVALine::sumXY (StatList<int>& Xlist, StatList<double>& Ylist){
	double sum = 0.0;
	if (Xlist.len() == Ylist.len()) {
		for(long i=0;i< Xlist.len();++i){
			sum += Xlist[i] * Ylist[i];}}
	return sum;
	}//end MANOVALine::sumXY 


long double MANOVALine::sumXY (StatList<int>& Xlist, StatList<long double>& Ylist){
	long double sum = 0.0;
	if (Xlist.len() == Ylist.len()) {
		for(long i=0;i< Xlist.len();++i){
			sum += Xlist[i] * Ylist[i];}}
	return sum;
	}//end MANOVALine::sumXY 


int operator== (MANOVALine& line1, MANOVALine& line2){
		return (line1.title() == line2.title());}

int operator!= (MANOVALine& line1, MANOVALine& line2){
		return !(line1 == line2);}


#endif
