/*******************************************
*
*
*	MANOVATable.cpp
*
*	A class that implements a MANOVA table
*
*
************************************************/

#ifndef MANOVATablecpp
#define MANOVATablecpp = 1


#include "MANOVATable.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "aVector.cpp"
#include "aMatrix.cpp"
#include "dataColumnRecNum.cpp"




void MANOVATable::AppendMANOVATableFlist (long double val, int loc){
	ANOVALineBase& theLine = Nth(loc);
	theLine.AppendBootF(val);
	}//end MANOVATable::AppendANOVATableFlist


long MANOVATable::df(aString& theTitle){
	ANOVALineBase& theLine = GetMANOVALine(theTitle);
	return theLine.df();
	}//end MANOVATable::df


long MANOVATable::df(int loc){
	ANOVALineBase& theLine = Nth(loc);
	return theLine.df();
	}//end MANOVATable::df



long MANOVATable::levels(aString& theTitle){
	ANOVALineBase& theLine = GetMANOVALine(theTitle);
	return theLine.levels();
	}//end MANOVATable::levels


long MANOVATable::levels(int loc){
	ANOVALineBase& theLine = Nth(loc);
	return theLine.levels();
	}//end MANOVATable::levels




void MANOVATable::DoBootMANOVA (dataSetRec& allData, aString& commandFileName, 
							aString& outFileName, long reps, Ran3& ran, 
							int resampleColumnsIndependently, int factorLevelCorrelations){
	//among species only to start
	StringList factorList, MSDivisorList;
	long dfSum = 0;
	int longTitle;
	int err = SetUpTableAndDesignMatrix (allData, commandFileName, factorList, MSDivisorList, dfSum,longTitle);
	if(err){return;}
	//make a new ordered list of the dependent variable
	aMatrix<long double> Ymatrix;
	dataSetRec tempDependentVars;
	
	for(int d=0;d<dependentList.len();++d){
		aString& colTitle = dependentList[d];
		dataColumnRec& depVarColRec = allData.GetDataColumn(colTitle);
		dataColumnRecNum<long double>* depVarColPtr = (dataColumnRecNum<long double>*) &depVarColRec;
		dataColumnRecNum<long double>& depVarCol = *depVarColPtr;
		tempDependentVars.Append(depVarColPtr->Duplicate());
		aList<long double>& varList = depVarCol.dataList();
		aVector<long double> aVecPtr(varList);
		Ymatrix.ConcatCol(aVecPtr);
		}//f
	aMatrix<long double>* origYmatrix = NEW aMatrix<long double>(Ymatrix);
//  cout<<"matrix of Y values:"<<endl;
//	Ymatrix.Print();

	long n_ = Ymatrix.rows();
	SetDF(n_ - 1, len()-1);	//total df
	SetDF((n_ - 1) - dfSum, len() - 2);//error df


	//do the analysis
	int sampleWithReplacement=true;
	MatrixList *residuals = NULL;
	if(factorLevelCorrelations){
		residuals = NEW MatrixList();}
	for (long rep = 0; rep<= reps;++rep){//resampling under null hypothesis of randomly assorted dependent variable states
		aString repWord(rep); repWord.PrependCharToLen(' ',6);
		cout<<"rep: "<< repWord<<endl;
		if (rep > 0){
			//resample the data
			Ymatrix.Clear();//original Ymatrix is stored in origYmatrix
			if(resampleColumnsIndependently){
					for(int dv=0;dv<tempDependentVars.len();++dv){
						dataColumnRec& depVarColRecR = tempDependentVars.GetDataColumn(dv);
						dataColumnRecNum<long double>* depVarColPtrR = (dataColumnRecNum<long double>*) &depVarColRecR;
						dataColumnRecNum<long double>& depVarColR = *depVarColPtrR;
						aVector<long double>* resampledVecR = depVarColR.GetResampledVectorLongDouble(withReplacement, ran);
						Ymatrix.ConcatCol(*resampledVecR);//copies to matrix
						delete resampledVecR; resampledVecR = NULL;
						}//dv
					}
				else{
					tempDependentVars.GetResampledData(Ymatrix,sampleWithReplacement, ran);}//maintains correlations in original data
//			cout<<"resampled Ymatrix:"<<endl;
//			Ymatrix.Print(cout);
			}//end if rep>0
		linearModel.MANOVASSdfVarCov(&Ymatrix, *this, rep, residuals);
		SetWilksLambda(MSDivisorList, rep);
		SetF(MSDivisorList, rep);
		
		}//end for rep
	SetP(MSDivisorList, reps);
	Print(cout, longTitle + 2, reps>1);
	if(factorLevelCorrelations){
		PrintWithinLevelCorrelations(cout, reps);}
	PrintFactorLevels(cout);
	fstream outFile;
	char* filename = outFileName.raw();
	outFile.open(filename,ios::out);
	Print(outFile, longTitle + 2,reps>1);
	if(factorLevelCorrelations){
		PrintWithinLevelCorrelations(outFile, reps);}
 	PrintFactorLevels(outFile);
	outFile.close();
	delete [] filename; filename=NULL;

	delete origYmatrix; origYmatrix=NULL;
	delete residuals; residuals=NULL;
	}//end MANOVATable::DoBootMANOVA 



ANOVALineBase& MANOVATable::GetMANOVALine(aString& lineTitle){
	for (int i=0;i<len();++i){
		ANOVALineBase& theLine = Nth(i);
		if (theLine.title() == lineTitle){
			return theLine;}
		}//end for i
	//if no match found
	aString err("There is no MANOVA line named ");
	err.cat(lineTitle); 
	err.cat(" in the MANOVA table.");
	aError(err);
	return First();
	}//end MANOVATable::GetMANOVALine



void MANOVATable::Print (ostream& outFile, int titleLen, int boot){
	for (int i=0;i<len();++i){
		ANOVALineBase& theLine =Nth(i);
		theLine.Print(outFile, titleLen, dependentList, boot);
		}//end for i
	outFile<<endl;
	}//end MANOVATable::Print



void MANOVATable::PrintWithinLevelCorrelations (ostream& outFile, long bootReps){
	for (int i=0;i<len();++i){
		ANOVALineBase& theLine =Nth(i);
		MANOVALine *theLineM = (MANOVALine*) &theLine;
		theLineM->PrintWithinLevelCorrelations(outFile, dependentList, bootReps);
		}//end for i
	outFile<<endl;
	}//end MANOVATable::PrintPrintWithinLevelCorrelations



void MANOVATable::PrintFactorLevels (ostream& outFile){
	for (int i=0;i<len()-2;++i){
		ANOVALineBase& theLine =Nth(i);
		theLine.PrintFactorLevels(outFile);
		}//end for i
	}//end MANOVATable::PrintFactorLevels



int MANOVATable::ReadCommandFile (StringList& factorList, StringList& MSDivisorList, aString& commandFileName){
	//returns 0 if data format error encountered
	aString depWord ="dependent:";
	aString facWord = "factors:";
	aString Fword = "F-divisors:";
	aString eWord = "Error";
	aString endCharList(" \t,;\"(){}[]!?");
	dependentList.Clear(); factorList.Clear();  MSDivisorList.Clear();
	int n_ = 0;
	aString aStr;
	fstream commandFile;
	char* commandName = commandFileName.raw();
	commandFile.open(commandName,ios::in);
	char* dataLine = NEW char [1001]; dataLine[0]='\0';
	commandFile.getline(dataLine,1000);
	aString indivDataLine(dataLine);
	cout<<indivDataLine<<endl;//prints dependent variables
	aString* word = indivDataLine.ChopOffFirstWord(endCharList);
	if (*word != depWord){
			cout<<"Error in command file setup:  dependent variables not preceded by the word 'dependent:'"<<endl;
			cout<<"Press return to continue..."<<endl;
			char* a = NEW char[5];a[0]='\0';
			cin >> a;  //to keyboard: press any key to continue
			delete [] a; a = NULL;
			delete word; word = NULL;
			return 1;//error occurred
			}
		else{
			while(indivDataLine.len()>0){
				aString* depColname = indivDataLine.ChopOffFirstWord(endCharList);
				depColname->CullChar(' ');//removes any spaces
				if(depColname->len()>0){
						dependentList.Append(depColname);}
					else{
						delete depColname; depColname=NULL;}
				}//end while
			}
	delete word; word=NULL;
	do{//skips over facWord and stores effects to be tested
		commandFile.getline(dataLine,1000);
		indivDataLine = dataLine;
		if ((indivDataLine != Fword) && (indivDataLine != facWord)){
			word = NEW aString(indivDataLine);  //Copy(indivDataLine, 1, WordLength(indivDataLine));
			factorList.Append(word);
			++n_;}
		}while (indivDataLine != Fword);
	if(factorList.Includes(&eWord)){
		--n_;}//skip error line in reading MS divisors
	for(int j =0;j< n_;++j){
		commandFile.getline(dataLine,1000);
		indivDataLine = dataLine;
//		indivDataLine.CullChar(' ');
		aString* word = NEW aString(indivDataLine);  //Copy(indivDataLine, 1, WordLength(indivDataLine));
		MSDivisorList.Append(word);
		}//end for i
	commandFile.close();
	delete [] commandName; commandName=NULL;
	delete [] dataLine; dataLine=NULL;
	return 0;//no errors
	}//end MANOVATable::ReadCommandFile 



void MANOVATable::SetDF  (int val, int loc){
	ANOVALineBase& theLine = Nth(loc);
	theLine.df(val);
	}//end MANOVATable::SetDF


//void MANOVATable::SetLevels  (int val, int loc){
//	ANOVALineBase& theLine = Nth(loc);
//	theLine.levels(val);
//	}//end MANOVATable::SetLevels


void MANOVATable::SetEffect (effectTypeSet val, int loc){
	ANOVALineBase& theLine = Nth(loc);
	theLine.effectType(val);
	}//end MANOVATable::SetEffect


void MANOVATable::SetF (StringList& MSDivisorList, long bootRep){
	//need to determine denominator for each line
	for(int i=0;i<len() - 2;++i){
		ANOVALineBase& theLine = Nth(i);
		aString& denomTitle = MSDivisorList[i];
//		ANOVALineBase& denomLine = GetMANOVALine(denomTitle);
//		theLine.F(denomLine, bootRep);
		ANOVALineBase& total_Line = GetMANOVALine(len()-1);//last line is total linne
		theLine.F(total_Line, bootRep);
		}//end for i
	}//end MANOVATable::SetF

/*
void MANOVATable::SetMultivariateDF (StringList& MSDivisorList){
	//need to determine denominator for each line
	for(int i=0;i<len() - 2;++i){
		ANOVALineBase& theLine = Nth(i);
		aString& denomTitle = MSDivisorList[i];
		ANOVALineBase& denomLine = GetMANOVALine(denomTitle);
		MANOVALine *theLineMAN = (MANOVALine*) &theLine;
		theLineMAN->SetMultivariateDF(denomLine);
		}//end for i
	}//end MANOVATable::SetMultivariateDF
*/

void MANOVATable::SetVarCov (aMatrix<long double>& vals, long loc, long bootRep, MatrixList *residuals){
	ANOVALineBase& theLine_ = Nth(loc);
	MANOVALine *theLine = (MANOVALine*) &theLine_;
	aMatrix<long double> *theResiduals = NULL;
	if(residuals){
		theResiduals = &( (*residuals)[loc] );}
	theLine->VarCov(vals,bootRep, theResiduals);
	}//end MANOVATable::SetVarCov


void MANOVATable::SetVarCovDenomTitle (aString& val, int loc){
	ANOVALineBase& theLine_ = Nth(loc);
	MANOVALine *theLine = (MANOVALine*) &theLine_;
	theLine->MSDenomTitle(val);
	}//end MANOVATable::SetMSDenomTitle


void MANOVATable::SetP (StringList& /*MSDivisorList*/, int boot){
	for(int i=0;i<len() - 2;++i){
		ANOVALineBase& theLine = Nth(i);
//		aString& denomTitle = MSDivisorList[i];
//		ANOVALineBase& denomLine = GetMANOVALine(denomTitle);
//		int denomDF = denomLine.df();
//		theLine.SetP(denomDF, boot);
		theLine.SetP(0, boot);//dummy for denom df
		}//end for i
	}//end MANOVATable::SetP



void MANOVATable::SetSS (aMatrix<long double>& vals, long loc, long bootRep){
	ANOVALineBase& theLine_ = Nth(loc);
	MANOVALine *theLine = (MANOVALine*) &theLine_;
	theLine->SS(vals,bootRep);
	}//end MANOVATable::SetSS


/*
void MANOVATable::SetReducedFormSS (aMatrix<long double>& vals, long loc, long bootRep){
	ANOVALineBase& theLine_ = Nth(loc);
	ANOVALine *theLine = (ANOVALine*) &theLine_;
	theLine->reducedFormSS(vals,bootRep);
	}//end MANOVATable::SetReducedFormSS
*/


int MANOVATable::SetUpTableAndDesignMatrix (dataSetRec& allData, aString& commandFileName,
									StringList& factorList, StringList& MSDivisorList, long& dfSum, 
									int& longTitle){
	Clear();//removes old analyses
	dependentList.Clear();
	factorList.Clear();
	MSDivisorList.Clear();
	int err = ReadCommandFile (factorList, MSDivisorList, commandFileName);
	if(err){return 1;}
	int factorsInModel = factorList.len();
	int doingRepeatedMeasures = false;

	aString& colTitle = dependentList[0];
	dataColumnRec& depVarColRec = allData.GetDataColumn(colTitle);
	dataColumnRecNum<long double>* depVarColPtr = (dataColumnRecNum<long double>*) &depVarColRec;
	dataColumnRecNum<long double>& depVarCol = *depVarColPtr;
	aList<long double>& varList = depVarCol.dataList();

	aMatrix<long double> Ymatrix;//dummy temporarily constructed using first dependent variable only
	aVector<long double> aVecPtr(varList);
	Ymatrix.ConcatCol(aVecPtr);

	//make the vector of constants
	aMatrix<long double>* ConstMatrix = NEW aMatrix<long double>((long double)1.0,varList.len(),1);
	aMatrix<long double>* ConstMatrix1 = NEW aMatrix<long double>((long double)1.0,varList.len(),1);

	for (int i=0;i<factorsInModel + 1;++i){//+ total line
		MANOVALine* theMANOVALine = NEW MANOVALine();
		theMANOVALine->effectType(fixedEffect);
		theMANOVALine->dependentVariables(dependentList.len());
		Append(theMANOVALine);
		theMANOVALine = NULL;//forget
		}//end for i
	aString titleWord = "Error";

	if(factorList.Includes(&titleWord)){//it's not a repeated-measures MANOVA
			factorList.Delete(factorList.Loc(&titleWord));
			--factorsInModel;
			SetEffect(errorLine, len() - 2);	//fixedEffect, randomEffect, errorLine, totalLine
			SetTitle(titleWord, len() - 2);
			}
		else{//last line in repeated measures MANOVA is assumed to contain error variance only
			SetEffect(errorLine, len() - 2);	//fixedEffect, randomEffect, errorLine, totalLine
			doingRepeatedMeasures = true;
			}			

	SetEffect(totalLine, len()-1);	
	titleWord = "Total";
	SetTitle(titleWord, len()-1);
	longTitle = 5;	//number of characters in longest title

	//make the full design matrix components list
//	linearModel.Clear();
	MatrixList* invXprimeXList = linearModel.invXprimeXList;
	MatrixList* designMatrixComponents = linearModel.designList;
	MatrixList* incedenceMatrixComponents = linearModel.incedenceMatrixList;
	designMatrixComponents->Append(ConstMatrix);
	incedenceMatrixComponents->Append(ConstMatrix1);

	dfSum = 0;
	//for each MANOVA table row
	for(int i=0;i<factorsInModel;++i){
		aString& factorString = factorList[i];
		SetTitle(factorString, i);
		aString& denomTitle = MSDivisorList[i];
		SetVarCovDenomTitle(denomTitle, i);
		if (longTitle < factorString.len()){ 
			longTitle = factorString.len();}
		ANOVALineBase& theLine = Nth(i);
		//this skips the last factor line if repeated measures MANOVA is used
		if(doingRepeatedMeasures && (i==factorsInModel-1)){break;}
			else{
				aMatrix<long double>* factorDesign = NEW aMatrix<long double>();
				aMatrix<long double>* incedenceMatrix = NEW aMatrix<long double>();
				theLine.PrepForCalcs(allData, factorDesign, incedenceMatrix, Ymatrix.rows(), &Ymatrix);
		
		//		factorDesign->Print();
		
				designMatrixComponents->Append(factorDesign);
				incedenceMatrixComponents->Append(incedenceMatrix);
				dfSum += theLine.df();
				factorDesign=NULL;//forget this too
				incedenceMatrix=NULL;//forget this too
				}//end if...else
		}//end for i
	return 0;
	}//end MANOVATable::SetUpTableAndDesignMatrix 


/*
void MANOVATable::SetCovarianceComponents (aMatrix<long double>& vals, long loc, long bootRep){
	ANOVALineBase& theLine_ = Nth(loc);
	MANOVALine *theLine = (MANOVALine*) &theLine_;
	theLine->covarianceComponents(vals,bootRep);
	}//end MANOVATable::SetVarianceComponents
*/

void MANOVATable::SetTitle (aString val, int loc){
	ANOVALineBase& theLine = Nth(loc);
	theLine.title(val);
	}//end MANOVATable::SetTitle


void MANOVATable::SetWilksLambda (StringList& MSDivisorList, long bootRep){
	//need to determine denominator for each line
	for(int i=0;i<len() - 2;++i){
		ANOVALineBase& theLine = Nth(i);
		aString& denomTitle = MSDivisorList[i];
		ANOVALineBase& denomLine = GetMANOVALine(denomTitle);
		MANOVALine *theLineMAN = (MANOVALine*) &theLine;
		theLineMAN->WilksLambda(denomLine, bootRep);
		}//end for i
	}//end MANOVATable::SetWilksLambda


aMatrix<long double>& MANOVATable::SS(aString& theTitle){
	ANOVALineBase& theLine_ = GetMANOVALine(theTitle);
	MANOVALine *theLine = (MANOVALine*) &theLine_;
	return theLine->SS(0);
	}//end ANOVATable::SS

/*
aMatrix<long double>& MANOVATable::reducedSS(aString& theTitle){
	ANOVALineBase& theLine_ = GetMANOVALine(theTitle);
	MANOVALine *theLine = (MANOVALine*) &theLine_;
	return theLine->reducedFormSS(0);
	}//end MANOVATable::reducedSS
*/

aMatrix<long double>& MANOVATable::VarCov(aString& theTitle){
	ANOVALineBase& theLine_ = GetMANOVALine(theTitle);
	MANOVALine *theLine = (MANOVALine*) &theLine_;
	return theLine->VarCov(0);
	}//end MANOVATable::MS

/*
aMatrix<long double>& MANOVATable::covarianceComponents(aString& theTitle){
	ANOVALineBase& theLine_ = GetMANOVALine(theTitle);
	MANOVALine *theLine = (MANOVALine*) &theLine_;
	return theLine->covarianceComponents(0);
	}//end MANOVATable::varianceComponent
*/


#endif


