/*******************************************
*
*
*	dataColumnRecNum.cpp
*
*	A class that implements a numerical data column in an ANOVA table
*
*
************************************************/

#ifndef dataColumnRecNumcp
#define dataColumnRecNumcp = 1


#include "dataColumnRecNum.h"
#include "Element.cpp"
#include "aList.cpp"
#include "aVector.cpp"
#include "aMatrix.cpp"




template<class T > dataColumnRecNum<T>::dataColumnRecNum(int source):dataColumnRec(source){
	content_ = numbers;
	SetPointerToDerivedSelf();
	}
			
			
//		dataColumnRecNum(dataColumnRecNum<T>&);//no copying possible; use Clone instead
//		dataColumnRecNum(dataColumnRecNum<T>*);//no copying possible; use Clone instead


template<class T > dataColumnRecNum<T>::~dataColumnRecNum(void){
	dataList_.Clear();
	pointerToDerivedSelf_=NULL;
	}//end ~dataColumnRec


template<class T > dataColumnRec*   dataColumnRecNum< T >::Duplicate(void){
	//returns a duplicate copy of 'this'
	dataColumnRecNum< T >* dup = NEW dataColumnRecNum< T >();
	dup->title_ = title_;
	dup->content_ = content_;
	dup->levelsList_ = levelsList_;//this copies each value separately
	dup->dataList_ = dataList_;//this copies each value separately
	return dup;
	}//end dataColumnRecNum< T >::Duplicate 



template<class T> dataColumnRecNum<T>& dataColumnRecNum<T>::operator=(dataColumnRecNum<T>& dcrn){
	dataList_=dcrn.dataList_;
	}//operator=

template<class T> void dataColumnRecNum<T>::SetPointerToDerivedSelf(void){
	pointerToDerivedSelf_ = (long)this;
	}

template<class T> long dataColumnRecNum<T>::GetPointerToDerivedSelf(void){
	long pdsf = (long)this;
	return pdsf;
	}



template<class T> T& dataColumnRecNum<T>::operator[](long i){
					return dataList_[i];}



template<class T > void dataColumnRecNum<T>::Append(T x){
	dataList_.Append(x);
	}
	
template<class T > T& dataColumnRecNum<T>::First(void){
	return dataList_.First();
	}
	
	
template<class T > T& dataColumnRecNum<T>::Last(void){
	return dataList_.Last();
	}
	
	
template<class T > aList<T>& dataColumnRecNum<T>::dataList(void){
	return dataList_;
	}
	
	
template<class T > void dataColumnRecNum<T>::DeleteRow(long r){
	dataList_.Delete(r);
	}


template<class T > aList< T >*  dataColumnRecNum< T >::GetResampledList (sampleRegimeType sampleRegime, Ran3& ran){
	aList<T>* newList = NEW aList<T>();
	aList<T> copiedList(dataList_);
	long i;
	for(i=0;i<dataList_.len();++i){
		T value = GetResampleValue(copiedList, sampleRegime, ran);
		newList->Append(value);
		}//end for i
	return newList;
	}//end  dataColumnRecNum<T>::GetResampledList


template<class T > aVector< T >*  dataColumnRecNum< T >::GetResampledVector(sampleRegimeType sampleRegime, Ran3& ran){
	aVector<T>* newList = NEW aVector<T>(dataList_);
	aList<T> copiedList(dataList_);
	long i;
	for(i=0;i<dataList_.len();++i){
		(*newList)[i] = GetResampleValue(copiedList, sampleRegime, ran);
		}//end for i
	return newList;
	}//end  dataColumnRecNum<T>::GetResampledVector


template<class T > aVector< double >*  dataColumnRecNum< T >::GetResampledVectorDouble(sampleRegimeType sampleRegime, Ran3& ran){
	aVector<T>* tempVector = GetResampledVector(sampleRegime, ran);
	aVector<double>* newVecDbl = tempVector->ToDouble();
	delete tempVector; tempVector = NULL;
	return newVecDbl;
	}//end  dataColumnRecNum<T>::GetResampledVectorDouble


template<class T > aVector< long double >*  dataColumnRecNum< T >::GetResampledVectorLongDouble(sampleRegimeType sampleRegime, Ran3& ran){
	aVector<T>* tempVector = GetResampledVector(sampleRegime, ran);
	aVector<long double>* newVecDbl = tempVector->ToLongDouble();
	delete tempVector; tempVector = NULL;
	return newVecDbl;
	}//end  dataColumnRecNum<T>::GetResampledVectorLongDouble


template<class T> T   dataColumnRecNum<T>::GetResampleValue(aList<T>& theList, sampleRegimeType sampleRegime, Ran3& ran){
	//if w/o replacement, then be sure to send a copy of theList if you want the original list intact
	if (theList.len() > 0){
		long loc = ran.longBetween(0, theList.len()-1);
		T val = theList[loc];
		if (sampleRegime == withoutReplacement){
			theList.Delete(loc);}
		return val;
		}//end if len>0
	return (T) 0.0;//default
	}//end dataColumnRecNum<T>::GetResampleValue(aList<T>...)

/*
template<class T> T   dataColumnRecNum<T>::GetResampleValue(vector<T>& theList, sampleRegimeType sampleRegime, Ran3& ran){
	//if w/o replacement, then be sure to send a copy of theList if you want the original list intact
	if (theList.len() > 0){
		long loc = ran.longBetween(0, theList.len()-1);
		T val = theList[loc];
		if (sampleRegime == withoutReplacement){
			theList.Delete(loc);}
		return val;
		}//end if len>0
	return (T) 0.0;//default
	}//end dataColumnRecNum<T>::GetResampleValue(vector<T>...)
*/


template<class T> long 	dataColumnRecNum<T>::len(){
	return dataList_.len();
	}
	
	
//template<class T> int dataColumnRecNum<T>::MissingData(long double missingDataFlag){
//	int retval=0;
//	if(len()>SampleSize(missingDataFlag)){
//		retval= 1;}
//	return retval;
//	}
	
	
template<class T> aList<long>* 	dataColumnRecNum<T>::MissingDataPos(char, long double missingDataFlag){
	return MissingDataPos(missingDataFlag);
	}


template<class T> aList<long>* dataColumnRecNum<T>::MissingDataPos(long double missingDataFlag){
	aList<long>* nulldataList = NEW aList<long>();
	long i;
	for(i=0;i<dataList_.len();++i){
		if(dataList_[i] == (T) missingDataFlag){
			nulldataList->Append(i);}
		}//end for i
	return nulldataList;
	}//end dataColumnRecNum<T>::MissingDataPos


template<class T> void dataColumnRecNum<T>::PrintValue(ostream& outFile, long item){
	if (dataList_.len()>0){
			outFile<<dataList_[item];}
		else{outFile<<".";}
	}

template<class T> long   dataColumnRecNum<T>::SampleSize(long double missingDataFlag){
	long nullRows = 0, i;
	for(i=0;i<dataList_.len();++i){
		if(dataList_[i] == (T) missingDataFlag){
			nullRows++;}
		}//end for i
	return dataList_.len() - nullRows;
	}//end dataColumnRecNum<T>::SampleSize
	

template<class T> void dataColumnRecNum<T>::SetValue(aString& str, long item){
	if (dataList_.len()>0){
		str.cat(dataList_[item]);}
	}




#endif

