/******************************************
*
*	memAllocList.cpp
*
*	This implements a specialized list containing
*	information for debugging memory leak problems.
*
*******************************************/

#ifndef memAllocListalistcp
#define memAllocListalistcp = 1

#include <iostream>
#include "memAllocList.h"


memAllocList::memAllocList(void): 
	first(0), last(0), recent(0), len_(0), recentLoc_(0) {
    	//cout<<"create aList"<<endl;
    	}//empty list



memAllocList::memAllocList (memAllocList& toCopy):
		first(NULL), last(NULL), recent(NULL), len_(0), recentLoc_(0) {//constucts a new copy
	for(long i=0;i<toCopy.len();++i){
		MemLeakIDType x=toCopy[i];
		Append(x);
		}
	}//end copy constructor &


memAllocList::memAllocList (memAllocList* toCopy):
		first(NULL), last(NULL), recent(NULL), len_(0), recentLoc_(0) {//constucts a new copy
	for(long i=0;i<toCopy->len();++i){
		MemLeakIDType x=toCopy->Nth(i);
		Append(x);
		}
	}//end copy constructor *

memAllocList::~memAllocList(void) {
	Clear();//deletes remaining contents
	}//end destructor






void memAllocList::Append (MemLeakIDType x) {
	//attaches x to the end of the list
	memElem	*elem=new memElem(x);
	if (len_ == 0) {first = elem;}
		else {		//connect it to the end of the list
			elem->prev = last;
			last->next = elem;}
	elem->next = NULL;	//there is no next element
	last = recent = elem; //resets the last element
	recentLoc_ = len_;//the most recent item manipulated
	++len_;
	}	//end memAllocList::Append


int memAllocList::BadIndex(long loc){
	//prints error message if index is bad
	if ((len_<0)||(loc<0)||(loc>=len_)){
		cout<<"Bad index: sought item "<<loc<<" in a 0-indexed aList of len "<<len_<<endl;
		return 1;}
	return 0;
	}//end memAllocList::BadIndex
	


void memAllocList::Clear (void) {
	// deletes each element & frees it's memory
	while (len_ > 0) {
		DeleteLast();}
	}	// end memAllocList::Clear


void memAllocList::Copy (memAllocList& toCopy) {
	// copies each element 
	Clear();
    for (long i=0;i<toCopy.len();++i){
    	Append(toCopy.Nth(i));}
	}	// end memAllocList::Copy


void memAllocList::Copy (memAllocList* toCopy) {
	// copies each element 
	Clear();
    for (long i=0;i<toCopy->len();++i){
    	Append(toCopy->Nth(i));}
	}	// end memAllocList::Copy


void memAllocList::DeleteFirst (void) {
	// deletes first element from a list & frees it's memory
	//use DetachFirst if you want to save it from destruction
	if (len_ > 0) {
		memElem *elem = first;
		if (len_ == 1) {//no first or last elements will be left
				first = recent = last = NULL;}
			else if (len_ == 2) {
				//the first & last elements are now the same
				first = elem->next;
				first->prev = NULL;
				last = recent = first;}
			else {//the 2nd element is now the first
				elem->next->prev = NULL;
				first = recent = elem->next;
				}
		--len_ ;
		recentLoc_=0;
		delete elem;elem=0;
		} //end if len>0
		else {BadIndex(0);}
	}	// end memAllocList::DeleteFirst


void memAllocList::DeleteLast (void) {
	//deletes an element from a list & frees it's memory
	//use DetachLast if you want to save it from destruction
	if (len_ > 0) {
		memElem *elem = last;
		if (len_ == 1) {//no first or last elements will be left
				first = recent = last =NULL;
				recentLoc_ = len_ = 0;
				delete elem;elem=0;
				return;}
			else if (len_ == 2) {//the first & last elements are now the same
				last = elem->prev;
				last->next = NULL;
				first = recent = last;}
			else {//the 2nd-last element is now the last
				elem->prev->next = NULL;
				last = recent = elem->prev;}
		--len_;
		recentLoc_ = len_ - 1;
		delete elem;elem=0;
		} //end if len>0
		else {BadIndex(0);}
	}	// end memAllocList::DeleteLast
	

void memAllocList::Delete (long loc) {
	//range of loc=0 (=first) to len_-1 (=last)
	//deletes an element from a list & frees it's memory
	//use DetachNth if you want to save contents from destruction
	if (!(BadIndex(loc))&&(len_ > 0) && (loc >=0) && (loc<len_ )) {
		if (loc == len_-1) {
				DeleteLast();}
			else if (loc == 0) {
				DeleteFirst();}
			else {  //delete an internal element
				memElem *elem = GoToElem(loc);
				elem->prev->next = elem->next;
				elem->next->prev = elem->prev;
				--len_;
				 recent = elem->prev;//recent drops to one earlier in list
				--recentLoc_;
				delete elem;elem=0;
				}
		} //end if len_>0
	} //end memAllocList::Delete



MemLeakIDType& memAllocList::First(void) {
	//returns a copy of the item, and leaves original in list
	//returns unitialized reference if the index is no good
	BadIndex(0);
	recent = first; recentLoc_=0;
	return first->data();
	}	//end memAllocList::First


memElem* memAllocList::GoToElem (long loc) {
	if (!(BadIndex(loc))&&(len() > 0)) {
			if (recentLoc_==loc) {return recent;}
			memElem *elem = NULL;
			if(ABS(recentLoc_-loc)<loc){//if loc is closer to recentLoc_ than to zero
					elem = recent;
					if(recentLoc_<loc){//count up to loc
							for(long i=recentLoc_;i<loc;++i){elem = elem->next;}
							}
						else{//count down to loc
							for(long i=recentLoc_;i>loc;--i){elem = elem->prev;}
							}
					}
				else{//count from zero
					elem = first;
					for(long i=0;i<loc;++i){elem = elem->next;}
					}
			recentLoc_=loc;
			recent = elem;
			return elem;}
		else{//default returns the first item
			recentLoc_=0;
			recent = first;
			return first;}
	}//end memAllocList::GoToElem 


void memAllocList::Insert (MemLeakIDType x, long loc) {
	//x inserted after loc; elements > loc are shifted
	//x simply Appended if loclen_
  if (!(BadIndex(loc))&&(0<=loc)&&(loc<len_)){
	if (loc == 0) {
			Prepend(x);}
		else if (loc == len()-1) {
			Append(x);}
		else {	//insert in the middle of the list
			memElem *elem = GoToElem(loc);
			memElem *newElem = new memElem(x);
			newElem->next = elem->next;
			elem->next->prev = newElem;
			elem->next = newElem;
			newElem->prev = elem;
			++len_;
			recent = newElem;
			recentLoc_=loc+1;
			}
	  } //end if 0loc<len_
	}	//end memAllocList::Insert



int memAllocList::Includes (MemLeakIDType x) {
	// true if x is in the list
	if (len_ > 0) {
		memElem *elem = first;
		for(long i=0;i<len_;++i){
			if (elem->data() == x) {
				recent = elem;
				recentLoc_=i;
				return 1;}
			elem = elem->next;}
		}
	recent = first;
	recentLoc_=0;
	return 0;
	}	//end memAllocList::Includes


MemLeakIDType& memAllocList::Last (void) {
	//returns a reference the item, and leaves original in list
	//returns uninitialized reference if the index is no good
	BadIndex(len_-1);
	recent = last;
	recentLoc_=len_-1;
	return last->data();
	}	//end memAllocList::Last



long memAllocList::len(void) {
	return len_;}



long memAllocList::Loc (MemLeakIDType x) {
	//returns the location if the object in the list
	//returns -1 if not present, so be careful!
	if (len_ > 0) {
		memElem *elem = first;
		for(long i=0;i<len_;++i) {
			if (elem->data() == x) {
				recent = elem;
				recentLoc_=i;
				return i;}
			elem = elem->next;
			}
		} //end if len_>0
	recent = first;
	recentLoc_=0;
	return (-1);
	}//end memAllocList::Loc



MemLeakIDType& memAllocList::Nth (long loc) {
	//range of loc=0 (=first) to len_-1 (=last)
	//returns a reference to the item at loc, and leaves original in list
	//returns uninitialized reference if the index is no good
	memElem *elem = GoToElem(loc);
	return elem->data();
	} //end memAllocList::Nth


void memAllocList::Print (ostream& outFile) {
	for(long i=0;i<len();++i){
		outFile<<Nth(i)<<endl;}
	} //end memAllocList::Print

void memAllocList::PrintLen (ostream& outFile) {
		outFile<<len()<<endl;
	} //end memAllocList::Nth



void memAllocList::Prepend (MemLeakIDType x) {
	//attaches x to the beginning of the list
	memElem *elem = new memElem(x);
	if (len_ == 0) {
			last = elem;}
		else { //connect it to the beginning of the list
			elem->next = first;
			first->prev = elem;}
	first = recent = elem; //reset the val of the first element
	++len_;
	recentLoc_=0;
	}//end memAllocList::Prepend 



void memAllocList::Replace (MemLeakIDType x, long loc) {
	//range of loc=0 (=first) to len_-1 (=last)
	//the old element must be freed elsewhere from memory
	memElem *elem = GoToElem(loc);
	if(elem !=NULL){elem->data_ = x;}
	}// memAllocList::Replace


void memAllocList::Sort(void){
	if (len_<2){return;}
	Sort(0,len_-1);
	}// memAllocList::Sort

	
	
void memAllocList::Sort(long start, long finish){
	//implements QuickSort
	//patched to accomodate 0-indexing
	if (len_<2) {return;}
	long left= start+1, right= finish+1;
	MemLeakIDType startValue = Nth(DIV((left + right),(long) 2)-1);
	do{
			while (Nth(left-1) < startValue) {++left;}
			while (startValue < Nth(right-1)) {--right;}
			if (left <= right) {
				MemLeakIDType temp = Nth(left-1);
				Replace(Nth(right-1),left-1);
				Replace(temp,right-1);
				++left;
				--right;
				}//end if
		}while(right > left);
	if (start < right-1) {
		Sort(start, right-1);}
	if (left-1 < finish) {
		Sort(left-1, finish);}
	}//end  memAllocList::Sort



memAllocList& memAllocList::operator= (memAllocList& z){
    //copies list z element by element
    for (long i=0;i<z.len_;++i){
    	Append(z[i]);}
    return *this;
    }// memAllocList::operator=


MemLeakIDType& memAllocList::operator[] (const long loc) {
	return Nth(loc);
    }// memAllocList::operator[]


int operator== (memAllocList& list1, memAllocList& list2){
	if (list1.len() != list2.len()){return 0;}
	for (long i=0;i<list1.len();++i){
		if(list1[i] != list2[i]){return 0;}}
	return 1;
	}//end friend operator==

int operator!= (memAllocList& list1, memAllocList& list2){
	return !(list1==list2);}


#endif
