/******************************************
*
*
*	ANOVAGenetics.cpp
*
*
*	This class performs calculations needed to get variance components for
*	genetic data analysis.  It is a derived class of ANOVATable.h, so it employs
*	linear algebra methods to anaklyze the data.
*
*
************************************************/

#ifndef ANOVAGeneticscp
#define ANOVAGeneticscp = 1

#include "ANOVAGenetics.h"
#include "GeneralLinearModel.h"
#include "PopulationGeneticData.h"
#include "Element.cpp"
#include "aList.cpp"
//#include "StringList.cpp"




ObjList<aList<long double> >* ANOVAGenetics::CalculateMeanSquares (/*PopulationGeneticData& geneticData, */
							dataSetRec& ANOVAData, StringList& factorList, 
							StringList& dependentColNames /*, long reps, Ran3& ran*/){
	//missing data must be culled prior to this step
	// the first ANOVAData columns correspond to the factors in the model
	//the remaining ANOVAData columns are dependent variables to be separately analyzed
	int factorsInModel = factorList.len();
	int f,i,fff;
	ObjList<aList<long double> >* meanSquares = NEW ObjList<aList<long double> >();
	for(f=0;f<factorsInModel+1;++f){// plus error variance
		aList<long double>* a = NEW aList<long double>();
		meanSquares->Append(a); a=NULL;}
	
	//make a new ordered list of a dependent variable, to be used in data setup below
	aString& colTitle0 = dependentColNames[0];
//	dataColumnRecNum<double>& depVarCol0 =(dataColumnRecNum<double>) ANOVAData.GetDataColumn(colTitle0);
	dataColumnRec& depVarColRec0 = ANOVAData.GetDataColumn(colTitle0);
	dataColumnRecNum<long double>* depVarColPtr0 =(dataColumnRecNum<long double>*) &depVarColRec0;
	dataColumnRecNum<long double>& depVarCol0 = *depVarColPtr0;
	aList<long double>& varList0 = depVarCol0.dataList();
	long n = varList0.len();
	
	aMatrix<long double> Ymatrix;
	aVector<long double> aVecPtr0(varList0);
	Ymatrix.ConcatCol(aVecPtr0);
//    	cout<<"matrix of Y values:"<<endl;
//	Ymatrix.Print();

	//make the vector of constants
	aMatrix<long double>* ConstMatrix = NEW aMatrix<long double>((long double)1.0,n,1);
	aMatrix<long double>* ConstMatrix1 = NEW aMatrix<long double>((long double)1.0,n,1);


	Clear();//removes old analyses
	for (i=0;i<factorsInModel + 2;++i){//+ error line, + total line
		ANOVALine* theANOVALine = NEW ANOVALine();
		theANOVALine->effectType( fixedEffect);
		Append(theANOVALine);
		theANOVALine = NULL;//forget
		}//end for i
	SetEffect(errorLine, len() - 2);	//fixedEffect, randomEffect, errorLine, totalLine
	aString titleWord = "gam$(indiv$)";
	SetTitle(titleWord, len() - 2);
	SetEffect(totalLine, len()-1);	
	titleWord = "Total";
	SetTitle(titleWord, len()-1);
	int longTitle = 15;	//number of characters in longest title



	//make the full design matrix components list
	GLM linearModel;
	MatrixList* invXprimeXList = linearModel.invXprimeXList;
	MatrixList* designMatrixComponents = linearModel.designList;
	MatrixList* incedenceMatrixComponents = linearModel.incedenceMatrixList;
	designMatrixComponents->Append(ConstMatrix);
	incedenceMatrixComponents->Append(ConstMatrix1);

	long dfSum = 0;
	//for each ANOVA table row
	for(i =0;i<factorsInModel;++i){
		aMatrix<long double>* factorDesign = NEW aMatrix<long double>();
		aMatrix<long double>* incedenceMatrix = NEW aMatrix<long double>();
		aString& factorString = factorList[i];
		SetTitle(factorString, i);
		if (longTitle < factorString.len()){ 
			longTitle = factorString.len();}
		ANOVALineBase& theLine = Nth(i);
		theLine.PrepForCalcs(ANOVAData, factorDesign, incedenceMatrix, n, &Ymatrix);
		
		designMatrixComponents->Append(factorDesign);
		incedenceMatrixComponents->Append(incedenceMatrix);
		dfSum += theLine.df();
		factorDesign=NULL;//forget this too
		incedenceMatrix=NULL;//forget this too
		}//end for i
	SetDF(n - 1, len()-1);	//total df
	SetDF((n - 1) - dfSum, len() - 2);//error df


	//do the analysis; bootstrap replication not yet included
	long allele;
	for (allele = 0; allele< dependentColNames.len();++allele){
		Ymatrix.Clear();
		aString& colTitle = dependentColNames[allele];
//		dataColumnRecNum<double>& depVarCol =(dataColumnRecNum<double>) ANOVAData.GetDataColumn(colTitle);
		dataColumnRec& depVarColRec = ANOVAData.GetDataColumn(colTitle);
		dataColumnRecNum<long double>* depVarColPtr =(dataColumnRecNum<long double>*) &depVarColRec;
		dataColumnRecNum<long double>& depVarCol = *depVarColPtr;
		aList<long double>& varList = depVarCol.dataList();
		aVector<long double> aVecPtr(varList);
		Ymatrix.ConcatCol(aVecPtr);		
		linearModel.ANOVASSdfMS(&Ymatrix, *this, 0);
		for(fff=0;fff<factorsInModel+1;++fff){// plus error variance
			aList<long double>& levelList = (*meanSquares)[fff];
			ANOVALineBase& theLine = Nth(fff);
			ANOVALine* theLineA = (ANOVALine*) &theLine;
			levelList.Append(theLineA->MS(0));}
		Print(cout, longTitle + 2);
		}//end for allele
	return meanSquares;
	}//end ANOVAGenetics::CalculateMeanSquares


ObjList<aList<long double> >* ANOVAGenetics::CalculateVarianceComponents (/*PopulationGeneticData& geneticData, */
							dataSetRec& ANOVAData, StringList& factorList, 
							StringList& dependentColNames, /*long reps, Ran3& ran, */ GLM& linearModel){
	//missing data must be culled prior to this step
	// the first ANOVAData columns correspond to the factors in the model
	//the remaining ANOVAData columns are dependent variables to be separately analyzed
	
	//notation follows Searle et al.1992
	int factorsInModel = factorList.len();
	int f,i,fff;
	ObjList<aList<long double> >* varianceComponents = NEW ObjList<aList<long double> >();
	for(f=0;f<factorsInModel+1;++f){// plus error variance
		aList<long double>* a = NEW aList<long double>();
		varianceComponents->Append(a); a=NULL;}
	
	//make a new ordered list of a dependent variable, to be used in data setup below
	aString& colTitle0 = dependentColNames[0];
//	dataColumnRecNum<double>& depVarCol0 =(dataColumnRecNum<double>) ANOVAData.GetDataColumn(colTitle0);
	dataColumnRec& depVarColRec0 = ANOVAData.GetDataColumn(colTitle0);
	dataColumnRecNum<long double>* depVarColPtr0 =(dataColumnRecNum<long double>*) &depVarColRec0;
	dataColumnRecNum<long double>& depVarCol0 = *depVarColPtr0;
	aList<long double>& varList0 = depVarCol0.dataList();
	long n = varList0.len();
	
	aMatrix<long double> Ymatrix;
	aVector<long double> aVecPtr0(varList0);
	Ymatrix.ConcatCol(aVecPtr0);

	//make the vector of constants


	Clear();//removes old analyses
	for (i =0;i<factorsInModel + 2;++i){//+ error line, + total line
		ANOVALine* theANOVALine = NEW ANOVALine();
		theANOVALine->effectType( fixedEffect);
		Append(theANOVALine);
		theANOVALine = NULL;//forget
		}//end for i
	SetEffect(errorLine, len() - 2);	//fixedEffect, randomEffect, errorLine, totalLine
	aString titleWord = "gam$(indiv$)";
	SetTitle(titleWord, len() - 2);
	SetEffect(totalLine, len()-1);	
	titleWord = "Total";
	SetTitle(titleWord, len()-1);
	int longTitle = 15;	//number of characters in longest title



	//make the full design matrix components list
	MatrixList* invXprimeXList = linearModel.invXprimeXList;
	MatrixList* designMatrixComponents = linearModel.designList;
	MatrixList* incedenceMatrixComponents = linearModel.incedenceMatrixList;
	if(linearModel.recyclablesCalculated()==0){
		aMatrix<long double>* ConstMatrix = NEW aMatrix<long double>((long double)1.0,n,1);
		aMatrix<long double>* ConstMatrix1 = NEW aMatrix<long double>((long double)1.0,n,1);
		designMatrixComponents->Append(ConstMatrix);
		incedenceMatrixComponents->Append(ConstMatrix1);
		ConstMatrix = ConstMatrix1 = NULL;//forget these
		}
	long dfSum = 0;
	//for each ANOVA table row
	for(i =0;i<factorsInModel;++i){
		aMatrix<long double>* factorDesign = NEW aMatrix<long double>();
		aMatrix<long double>* incedenceMatrix = NEW aMatrix<long double>();
		aString& factorString = factorList[i];
		SetTitle(factorString, i);
		if (longTitle < factorString.len()){ 
			longTitle = factorString.len();}
		ANOVALineBase& theLine = Nth(i);
		theLine.PrepForCalcs(ANOVAData, factorDesign, incedenceMatrix, n, &Ymatrix);
		
		
		if(linearModel.recyclablesCalculated()==0){
				designMatrixComponents->Append(factorDesign);
				incedenceMatrixComponents->Append(incedenceMatrix);}
			else{//already have these stored
				delete factorDesign;
				delete incedenceMatrix;
				}
		dfSum += theLine.df();
		factorDesign=NULL;//forget this too
		incedenceMatrix=NULL;//forget this too
		}//end for i
	SetDF(n - 1, len()-1);	//total df
	SetDF((n - 1) - dfSum, len() - 2);//error df


	//do the analysis; bootstrap replication not yet included
	aString checkDataName("Ymatrices");
	for (long allele = 0; allele< dependentColNames.len();++allele){
		Ymatrix.Clear();
		aString& colTitle = dependentColNames[allele];
//		dataColumnRecNum<double>& depVarCol =(dataColumnRecNum<double>) ANOVAData.GetDataColumn(colTitle);
		dataColumnRec& depVarColRec = ANOVAData.GetDataColumn(colTitle);
		dataColumnRecNum<long double>* depVarColPtr =(dataColumnRecNum<long double>*) &depVarColRec;
		dataColumnRecNum<long double>& depVarCol = *depVarColPtr;
		aList<long double>& varList = depVarCol.dataList();
		aVector<long double> aVecPtr(varList);
		Ymatrix.ConcatCol(aVecPtr);	
		aString ymessage("data for allele "); ymessage.cat(dependentColNames[allele]);
		linearModel.VarianceComponentsANOVA(&Ymatrix, *this, 0);
		for(fff=0;fff<factorsInModel+1;++fff){// plus error variance
			aList<long double>& levelList = (*varianceComponents)[fff];
			ANOVALineBase& theLine = Nth(fff);
			ANOVALine* theLineA = (ANOVALine*) &theLine;
			levelList.Append(theLineA->varianceComponent(0));}
		}//end for allele
	invXprimeXList =NULL;//forget these
	designMatrixComponents = NULL;
	incedenceMatrixComponents = NULL;
	return varianceComponents;
	}//end ANOVAGenetics::CalculateVarianceComponents



void ANOVAGenetics::ConvertTitleString (aString& theTitleStr, StringList& theList){
	//recursive, so clear thelist before the first call
	aString xWord('x'), startParen('('), endParen(')');
	aString endCharList(" \t,;:\"(){}[]!?");
aString theTitle(theTitleStr);//work with a copy because original is destroyed
	while (theTitle.len() > 0){
		aString* word = theTitle.ChopOffFirstWord(endCharList);
		if (*word != xWord){//ignore the 'x'
			if (startParen == (*word)[0]){
					//remove parentheses and later, handle nesting
					word->cut(0);
					word->cut(word->len()-1);
					ConvertTitleString(*word, theList);//checks for multiple words inside parentheses
					delete word; word=NULL;
					}//end if startParen==first char
				else if (word->pos(startParen) > 0){ //handle the pieces separately
					aString part1(*word), part2(*word);
					 part1.cut(part1.pos(startParen), part1.len());//trins off last ')' too
					 part2.cut(0, part2.pos(startParen));
					 part2.trunc(part2.len()-1);
					ConvertTitleString(part1, theList);
					ConvertTitleString(part2, theList);
					delete word; word=NULL;
					}
				else	{//it's a regular word
					theList.Append(word);}
			}//end if word<>xWord
		}//end while
	}//end  ANOVAGenetics::ConvertTitleString



dataSetRec*	 ANOVAGenetics::CreateRawANOVADataFormat(PopulationGeneticData& geneticData, StringList& dependentColNames){
	//returns a data set that can be analyzed using linear algebra
	//missing data represented by '-1'
	//missing data must be culled elsewhere before analysis
	dependentColNames.Clear();
	dataSetRec* ANOVAData = NEW dataSetRec();
//	outFile<<"pop$ indiv$ gam$ ";//this and next loop comprise the column titles

	//construct the columns for indicator variables
	dataColumnRecStr* columnPtr = NEW dataColumnRecStr();
	aString grword("group$");
	columnPtr->title(grword);
	ANOVAData->Append(columnPtr);
	columnPtr = NEW dataColumnRecStr();
	aString lword("locality$");
	columnPtr->title(lword);
	ANOVAData->Append(columnPtr);
	columnPtr = NEW dataColumnRecStr();
	aString pword("pop$");
	columnPtr->title(pword);
	ANOVAData->Append(columnPtr);
	columnPtr = NEW dataColumnRecStr();
	aString iword("indiv$");
	columnPtr->title(iword);
	ANOVAData->Append(columnPtr);
	columnPtr=NULL;//forget this

	//construct columns for locus and allele data
	int vloc,loc,allele, group,gpop,pop, gam,i;
	for(vloc=0;vloc<geneticData.variableLoci_.len();++vloc){
		loc = geneticData.variableLoci_[vloc];
		aString& locName=geneticData.locusName[loc];
		StringList& alleleList1=geneticData.allelesPresent[loc];
		for(allele=0;allele<alleleList1.len();++allele){
			dataColumnRecNum<long double>* columnNumPtr = NEW dataColumnRecNum<long double>();
			aString alleleTitle(locName);
			alleleTitle.cat('_');
			alleleTitle.cat(alleleList1[allele]);
			columnNumPtr->title(alleleTitle);
			ANOVAData->Append(columnNumPtr);columnNumPtr=NULL;
			aString* alleleTitle2=NEW aString(alleleTitle);
			dependentColNames.Append(alleleTitle2);alleleTitle2=NULL;
			}
		}//end for loc
	for(group=0;group<geneticData.ngroups;++group){
		aList<int>& popsInGroup = geneticData.groupHierarchy[group];
		dataColumnRec& groupCol = ANOVAData->GetDataColumn(grword);
		for(gpop=0;gpop<popsInGroup.len();++gpop){
			pop = popsInGroup[gpop];
			dataColumnRec& popCol = ANOVAData->GetDataColumn(pword);
			int locality = geneticData.GetLocality(pop);
			dataColumnRec& localityCol = ANOVAData->GetDataColumn(lword);
			for(i=0;i<geneticData.popSize[pop];++i){
				dataColumnRec& indivCol = ANOVAData->GetDataColumn(iword);
				for (gam=0;gam<2;++gam){
					aString* groupStr = NEW aString(group+1);
					groupCol.CheckCategoryMembership(*groupStr);
					groupCol.Append(groupStr); groupStr=NULL;
					aString* localityStr = NEW aString(locality+1);
					localityCol.CheckCategoryMembership(*localityStr);
					localityCol.Append(localityStr); localityStr=NULL;
					aString* popStr = NEW aString(pop+1);
					popCol.CheckCategoryMembership(*popStr);
					popCol.Append(popStr); popStr=NULL;
					aString* indivStr = NEW aString(i+1);
					indivCol.CheckCategoryMembership(*indivStr);
					indivCol.Append(indivStr); indivStr=NULL;
					int columnCounter=3;
					for(vloc=0;vloc<geneticData.variableLoci_.len();++vloc){
						loc = geneticData.variableLoci_[vloc];
						StringList& alleleList2=geneticData.allelesPresent[loc];
						for(allele=0;allele<alleleList2.len();++allele){
							columnCounter++;
							dataColumnRec& dataCol = ANOVAData->GetDataColumn(columnCounter);
							aString& alleleName = alleleList2[allele];
							aString* gene =geneticData.GetGenotypeString(pop, i, loc, gam);
							if(*gene==alleleName){//allele present gets 1
									dataCol.Append(1.0);
									}
								else if(*gene==' '){	// missing data	 gets -1; use this later to cull					
									dataCol.Append(-1.0);
									}
								else{//allele absent gets 0
									dataCol.Append(0.0);
									}
							delete gene; gene=0;
							}//end for allele
						}//end for loc
					}//end for gam
				}//end for i
			}//end for gpop
		}//end for gpop
	return ANOVAData;
	}//end ANOVAGenetics::CreateANOVADataFormat





dataSetRec*	 ANOVAGenetics::CreateRawANOVADataFormat(CodominantMarkerData& geneticData, StringList& dependentColNames){
	//returns a data set that can be analyzed using linear algebra
	//missing data represented by '-1'
	//missing data must be culled elsewhere before analysis
	dependentColNames.Clear();
	dataSetRec* ANOVAData = NEW dataSetRec();
//	outFile<<"pop$ indiv$ gam$ ";//this and next loop comprise the column titles

	//construct the columns for indicator variables
	dataColumnRecStr* columnPtr = NEW dataColumnRecStr();
	aString grword("group$");
	columnPtr->title(grword);
	ANOVAData->Append(columnPtr);
	columnPtr = NEW dataColumnRecStr();
	aString lword("locality$");
	columnPtr->title(lword);
	ANOVAData->Append(columnPtr);
	columnPtr = NEW dataColumnRecStr();
	aString pword("pop$");
	columnPtr->title(pword);
	ANOVAData->Append(columnPtr);
	columnPtr = NEW dataColumnRecStr();
	aString iword("indiv$");
	columnPtr->title(iword);
	ANOVAData->Append(columnPtr);
	columnPtr=NULL;//forget this

	int vloc,loc,group,gpop,pop,i,gam,allele;
	//construct columns for locus and allele data
	for(vloc=0;vloc<geneticData.variableLocusNumCodom_.len();++vloc){
		aString& locName=geneticData.VariableLocusName(vloc);
		loc = geneticData.variableLocusNumCodom_[vloc];
		StringList& alleleList1=geneticData.allelesPresent_[loc];
		for(allele=0;allele<alleleList1.len();++allele){
			dataColumnRecNum<long double>* columnNumPtr = NEW dataColumnRecNum<long double>();
			aString alleleTitle(locName);
			alleleTitle.cat('_');
			alleleTitle.cat(alleleList1[allele]);
			columnNumPtr->title(alleleTitle);
			ANOVAData->Append(columnNumPtr);columnNumPtr=NULL;
			aString* alleleTitle2=NEW aString(alleleTitle);
			dependentColNames.Append(alleleTitle2);alleleTitle2=NULL;
			}
		}//end for loc
	for(group=0;group<geneticData.ngroups_;++group){
		aList<int>& popsInGroup = geneticData.groupHierarchy_[group];
		dataColumnRec& groupCol = ANOVAData->GetDataColumn(grword);
		for(gpop=0;gpop<popsInGroup.len();++gpop){
			pop = popsInGroup[gpop];
			dataColumnRec& popCol = ANOVAData->GetDataColumn(pword);
			int locality = geneticData.GetLocality(pop);
			dataColumnRec& localityCol = ANOVAData->GetDataColumn(lword);
			for(i=0;i<geneticData.popSize_[pop];++i){
				dataColumnRec& indivCol = ANOVAData->GetDataColumn(iword);
				for (gam=0;gam<2;++gam){
					aString* groupStr = NEW aString(group+1);
					groupCol.CheckCategoryMembership(*groupStr);
					groupCol.Append(groupStr); groupStr=NULL;
					aString* localityStr = NEW aString(locality+1);
					localityCol.CheckCategoryMembership(*localityStr);
					localityCol.Append(localityStr); localityStr=NULL;
					aString* popStr = NEW aString(pop+1);
					popCol.CheckCategoryMembership(*popStr);
					popCol.Append(popStr); popStr=NULL;
					aString* indivStr = NEW aString(i+1);
					indivCol.CheckCategoryMembership(*indivStr);
					indivCol.Append(indivStr); indivStr=NULL;
					int columnCounter=3;
					for(vloc=0;vloc<geneticData.variableLocusNumCodom_.len();++vloc){
						loc = geneticData.variableLocusNumCodom_[vloc];
						StringList& alleleList2=geneticData.allelesPresent_[loc];
						for(allele=0;allele<alleleList2.len();++allele){
							columnCounter++;
							dataColumnRec& dataCol = ANOVAData->GetDataColumn(columnCounter);
							aString& alleleName = alleleList2[allele];
							TextDatum& gene =geneticData.GenotypicVal(pop, i, loc, gam);
							if(gene.missing()){
									dataCol.Append(-1.0);	// missing data	 gets -1; use this later to cull
									}
								else if(gene==alleleName){					
									dataCol.Append(1.0);//allele present gets 1
									}
								else{//allele absent gets 0
									dataCol.Append(0.0);
									}
							}//end for allele
						}//end for loc
					}//end for gam
				}//end for i
			}//end for gpop
		}//end for gpop
	return ANOVAData;
	}//end ANOVAGenetics::CreateANOVADataFormat



dataSetRec*	 ANOVAGenetics::CreateRawANOVADataFormat(DominantMarkerData2& geneticData, StringList& dependentColNames){
	//returns a data set that can be analyzed using linear algebra
	//missing data represented by '-1'
	//missing data must be culled elsewhere before analysis
	dependentColNames.Clear();
	dataSetRec* ANOVAData = NEW dataSetRec();
//	outFile<<"pop$ indiv$ gam$ ";//this and next loop comprise the column titles

	//construct the columns for indicator variables
	dataColumnRecStr* columnPtr = NEW dataColumnRecStr();
	aString grword("group$");
	columnPtr->title(grword);
	ANOVAData->Append(columnPtr);
	columnPtr = NEW dataColumnRecStr();
	aString lword("locality$");
	columnPtr->title(lword);
	ANOVAData->Append(columnPtr);
	columnPtr = NEW dataColumnRecStr();
	aString pword("pop$");
	columnPtr->title(pword);
	ANOVAData->Append(columnPtr);
	columnPtr = NEW dataColumnRecStr();
	aString iword("indiv$");
	columnPtr->title(iword);
	ANOVAData->Append(columnPtr);
	columnPtr=NULL;//forget this

	int vloc,loc,allele,group,gpop,pop,i;
	//construct columns for locus and allele data
	for(vloc=0;vloc<geneticData.variableLocusNumDom_.len();++vloc){
		aString& locName=geneticData.VariableLocusName(vloc);
		loc = geneticData.variableLocusNumDom_[vloc];
		StringList& alleleList1=geneticData.allelesPresentDom_[loc];
		for(allele=0;allele<alleleList1.len();++allele){
			dataColumnRecNum<long double>* columnNumPtr = NEW dataColumnRecNum<long double>();
			aString alleleTitle(locName);
			alleleTitle.cat('_');
			alleleTitle.cat(alleleList1[allele]);
			columnNumPtr->title(alleleTitle);
			ANOVAData->Append(columnNumPtr);columnNumPtr=NULL;
			aString* alleleTitle2=NEW aString(alleleTitle);
			dependentColNames.Append(alleleTitle2);alleleTitle2=NULL;
			}
		}//end for vloc
	for(group=0;group<geneticData.ngroups_;++group){
		aList<int>& popsInGroup = geneticData.groupHierarchy_[group];
		dataColumnRec& groupCol = ANOVAData->GetDataColumn(grword);
		for(gpop=0;gpop<popsInGroup.len();++gpop){
			pop = popsInGroup[gpop];
			dataColumnRec& popCol = ANOVAData->GetDataColumn(pword);
			int locality = geneticData.GetLocality(pop);
			dataColumnRec& localityCol = ANOVAData->GetDataColumn(lword);
			for(i=0;i<geneticData.popSize_[pop];++i){
				dataColumnRec& indivCol = ANOVAData->GetDataColumn(iword);
					aString* groupStr = NEW aString(group+1);
					groupCol.CheckCategoryMembership(*groupStr);
					groupCol.Append(groupStr); groupStr=NULL;
					aString* localityStr = NEW aString(locality+1);
					localityCol.CheckCategoryMembership(*localityStr);
					localityCol.Append(localityStr); localityStr=NULL;
					aString* popStr = NEW aString(pop+1);
					popCol.CheckCategoryMembership(*popStr);
					popCol.Append(popStr); popStr=NULL;
					aString* indivStr = NEW aString(i+1);
					indivCol.CheckCategoryMembership(*indivStr);
					indivCol.Append(indivStr); indivStr=NULL;
					int columnCounter=3;
					for(vloc=0;vloc<geneticData.variableLocusNumDom_.len();++vloc){
						loc = geneticData.variableLocusNumDom_[vloc];
						StringList& alleleList2=geneticData.allelesPresentDom_[loc];
						for(allele=0;allele<alleleList2.len();++allele){
							columnCounter++;
							dataColumnRec& dataCol = ANOVAData->GetDataColumn(columnCounter);
							aString& alleleName = alleleList2[allele];
							TextDatum& gene =geneticData.PhenotypicVal(pop, i, loc);
							if(gene.missing()){
									dataCol.Append(-1.0);	// missing data	 gets -1; use this later to cull
									}
								else if(gene==alleleName){					
									dataCol.Append(1.0);//allele present gets 1
									}
								else{//allele absent gets 0
									dataCol.Append(0.0);
									}
							}//end for allele
						}//end for loc
				}//end for i
			}//end for gpop
		}//end for gpop
	return ANOVAData;
	}//end ANOVAGenetics::CreateANOVADataFormat



dataSetRec*	 ANOVAGenetics::CreateRawANOVADataFormat(SexLinkedCodomMarkerData& geneticData, StringList& dependentColNames){
	//returns a data set that can be analyzed using linear algebra
	//missing data represented by '-1'
	//missing data must be culled elsewhere before analysis
	dependentColNames.Clear();
	dataSetRec* ANOVAData = NEW dataSetRec();
//	outFile<<"pop$ indiv$ gam$ ";//this and next loop comprise the column titles

	//construct the columns for indicator variables
	dataColumnRecStr* columnPtr = NEW dataColumnRecStr();
	aString grword("group$");
	columnPtr->title(grword);
	ANOVAData->Append(columnPtr);
	columnPtr = NEW dataColumnRecStr();
	aString lword("locality$");
	columnPtr->title(lword);
	ANOVAData->Append(columnPtr);
	columnPtr = NEW dataColumnRecStr();
	aString pword("pop$");
	columnPtr->title(pword);
	ANOVAData->Append(columnPtr);
	columnPtr = NEW dataColumnRecStr();
	aString iword("indiv$");
	columnPtr->title(iword);
	ANOVAData->Append(columnPtr);
	columnPtr=NULL;//forget this

	int vloc,loc,allele,group,gpop,pop,i, gam;
	//construct columns for locus and allele data
	for(vloc=0;vloc<geneticData.variableLocusNumSLCodom_.len();++vloc){
		aString& locName=geneticData.VariableLocusName(vloc);
		loc = geneticData.variableLocusNumSLCodom_[vloc];
		StringList& alleleList1=geneticData.allelesPresentSLCodom_[loc];
		for(allele=0;allele<alleleList1.len();++allele){
			dataColumnRecNum<long double>* columnNumPtr = NEW dataColumnRecNum<long double>();
			aString alleleTitle(locName);
			alleleTitle.cat('_');
			alleleTitle.cat(alleleList1[allele]);
			columnNumPtr->title(alleleTitle);
			ANOVAData->Append(columnNumPtr);columnNumPtr=NULL;
			aString* alleleTitle2=NEW aString(alleleTitle);
			dependentColNames.Append(alleleTitle2);alleleTitle2=NULL;
			}
		}//end for loc
	for(group=0;group<geneticData.ngroups_;++group){
		aList<int>& popsInGroup = geneticData.groupHierarchy_[group];
		dataColumnRec& groupCol = ANOVAData->GetDataColumn(grword);
		for(gpop=0;gpop<popsInGroup.len();++gpop){
			pop = popsInGroup[gpop];
			dataColumnRec& popCol = ANOVAData->GetDataColumn(pword);
			int locality = geneticData.GetLocality(pop);
			dataColumnRec& localityCol = ANOVAData->GetDataColumn(lword);
			for(i=0;i<geneticData.popSize_[pop];++i){
				dataColumnRec& indivCol = ANOVAData->GetDataColumn(iword);
				for (gam=0;gam<geneticData.Ploidy(pop,i,0);++gam){
					aString* groupStr = NEW aString(group+1);
					groupCol.CheckCategoryMembership(*groupStr);
					groupCol.Append(groupStr); groupStr=NULL;
					aString* localityStr = NEW aString(locality+1);
					localityCol.CheckCategoryMembership(*localityStr);
					localityCol.Append(localityStr); localityStr=NULL;
					aString* popStr = NEW aString(pop+1);
					popCol.CheckCategoryMembership(*popStr);
					popCol.Append(popStr); popStr=NULL;
					aString* indivStr = NEW aString(i+1);
					indivCol.CheckCategoryMembership(*indivStr);
					indivCol.Append(indivStr); indivStr=NULL;
					int columnCounter=3;
					for(vloc=0;vloc<geneticData.variableLocusNumSLCodom_.len();++vloc){
						loc = geneticData.variableLocusNumSLCodom_[vloc];
						StringList& alleleList2=geneticData.allelesPresentSLCodom_[loc];
						for(allele=0;allele<alleleList2.len();++allele){
							columnCounter++;
							dataColumnRec& dataCol = ANOVAData->GetDataColumn(columnCounter);
							aString& alleleName = alleleList2[allele];
							TextDatum& gene =geneticData.GenotypicVal(pop, i, loc, gam);
							if(gene.missing()){
									dataCol.Append(-1.0);	// missing data	 gets -1; use this later to cull
									}
								else if(gene==alleleName){					
									dataCol.Append(1.0);//allele present gets 1
									}
								else{//allele absent gets 0
									dataCol.Append(0.0);
									}
							}//end for allele
						}//end for loc
					}//end for gam
				}//end for i
			}//end for gpop
		}//end for gpop
	return ANOVAData;
	}//end ANOVAGenetics::CreateANOVADataFormat






#endif
