/*

FstatLandScapeInterface.cpp

*/
#ifndef FstatLandScapeInterfacecpp
#define FstatLandScapeInterfacecpp = 1


#include "FstatLandScapeInterface.h"
#include "Element.cpp"
#include "aList.cpp"



void FstatLandScapeInterface::Clear(){
 	FstatisticAnalysisMultiFormat::Clear();
	GelJockLandScapeInterface::Clear();
 	}

void FstatLandScapeInterface::PrintAlleleFreqs(ostream& outFile, int ncols, char colSpacer){
 	MultiformatMarkerData::PrintAlleleFreqs(outFile, ncols, colSpacer);
 	}


/* REST MADE OBSOLETE BY MAKING THIS A DERIVED CLASS OF GelJockLandScaspeInterface

void FstatLandScapeInterface::PrintAlleleFreqs (ostream& outFile, int ncols){
	//outfile assumed open
	MultiformatMarkerData::PrintAlleleFreqs(outFile,ncols);
	PrintAlleleKey(outFile);
	}//FstatLandScapeInterface::PrintAlleleFreqs




void FstatLandScapeInterface::PrintAlleleKey (ostream& outFile){
	//outfile assumed open
	outFile<<"Key to genotypes of allele letter scores."<<endl<<endl;
	outFile<<"Locus & Allele    genotype"<<endl;
	for(int loc=0;loc<loci();++loc){
		outFile<<MultiformatMarkerData::locusName(loc)<<endl;
		StringList& locList = actualAlleleNames_[loc];
		for(int a=0;a<locList.len();++a){
			char alleleCh = 'A' + a;
			aString allele(alleleCh);
			allele.prepend("  ");
			allele.separator(10);
			allele.cat(locList[a]);
			outFile<<allele<<endl;
			}
		}
	outFile<<endl<<endl;
	}//FstatLandScapeInterface::PrintAlleleKey


void FstatLandScapeInterface::ReadGeneticLandScape(GeneticLandScape& theLandScape, 
				int species, aString& simTitle, aList<int>& skippedLocusNumbers){
	aList<int> popNumbers;
	for(int i=0;i<theLandScape.area();++i){
		popNumbers.Append(i);}
	ReadGeneticLandScape(theLandScape,species, simTitle, skippedLocusNumbers, popNumbers);
	}//end 



void FstatLandScapeInterface::ReadGeneticLandScape(GeneticLandScape& theLandScape, 
				int species, aString& simTitle, aList<int>& skippedLocusNumbers, aList<int>& popNumbers){
	Clear();
	int lociToSortThrough = theLandScape.loci(species);
	title_ = simTitle;

//	npop = theLandScape.area();
	npop_ = popNumbers.len();
	nloci = theLandScape.loci(species);
	maxall= 0;
	for(int loc=0;loc<nloci;++loc){
		maxall = MAX(maxall, (long) theLandScape.AllelesAtLoc(species,loc));}
	ngroups = 1;//default value

	//make locus names
	for(int loc=0;loc<nloci;++loc){
		aString *locName = NEW aString("LOC_");
		aString locNum(loc+1);
		locNum.PrependCharToLen('0', Digits(nloci));
		locName->cat(locNum);
		rawLocusName.Append(locName);
		actualAlleleNames_.Append(NEW StringList());
		}//end for loc
	//read in the number of loci to be skipped in the analyses
	lociSkipped = skippedLocusNumbers.len();
	skippedLocusNum = skippedLocusNumbers;
	//and read in the names of skipped loci, if any
	for(int loc=0;loc<lociSkipped;++loc){
		aString *skipName = NEW aString("LOC_");
		aString skipNum(skippedLocusNumbers[loc]+1);
		skipNum.PrependCharToLen('0', Digits(nloci));
		skipName->cat(skipNum);
		skippedLocusName.Append(skipName);
		}//end for loc

	locusName.Copy(rawLocusName);
	aList<int> actualLocusNumbers;
	for(int loc=0;loc<nloci;++loc){actualLocusNumbers.Append(loc);}
	for (int loc=0;loc<skippedLocusNumbers.len();++loc){
		aString& toSkip = skippedLocusName[loc];
		int location = locusName.Loc(&toSkip);
		locusName.Delete(location);
		actualLocusNumbers.Delete(location);
		}//end for loc
	nloci -= lociSkipped;

	//the read-in populations might not match the popID numbers
	ObjList<aList<int> >* popIDgridPtr = theLandScape.PopIDGrid(species);
	ObjList<aList<int> >& popIDgrid = *popIDgridPtr;
	aList<int> popIDs;//linearize the list
	int popcount=-1;
	for(int r=0;r<popIDgrid.len();++r){
		for(int c=0;c<popIDgrid[r].len();++c){
			++popcount;
			if(popNumbers.Includes(popcount)){
				popIDs.Append(popIDgrid[r][c]);
				}
			}}
	
	//read in population data, converting alleles to A, B, C, etc.
	int emptyPops = 0;
	for (int pop=0;pop<npop_;++pop){
		CoordinateType xy(-1,-1);
		theLandScape.GetCoordinates(species,popIDs[pop],xy);
		Population& thePopP = theLandScape.GetPopulation(species, xy);
		GeneticPopulation* thePopG = (GeneticPopulation*) &thePopP;
		GeneticPopulation& thePop = *thePopG;
		if(thePop.size()==0){
				++emptyPops;}
			else{
				//put the {x,y} coordinates of the population onto its name
				aString *s = NEW aString("Population ");
				s->cat(pop+1);
				s->cat(" y= "); s->cat(xy[1]);
				s->cat(" x= "); s->cat(xy[0]);
				popName.Append(s);

				//make a new sublist for the genotypes of this population
				StringList *rawGList = NEW StringList();
				rawGtypes.Append(rawGList);				
				popSize.Append(thePop.size());
				twoN.Append( NEW aList<int>());
				for(int loc10=0;loc10<lociToSortThrough;++loc10){//fix for haplodiploidy/sex-linkage
					(twoN.Last()).Append(thePop.size() * thePop.ploidy(loc10));}

				// BEAST is the string for a particular individual
				for(int i=0;i<thePop.size();++i){
					aString* beast = NEW aString(i+1);
					rawGList->Append(beast);
					beast->PrependCharToLen('0',4);beast->cat(' ');
					for(int loc=0;loc<lociToSortThrough;++loc){
						for(int a=0;a<thePop.ploidy(loc);++a){
							aString* alleleVal = thePop.GenotypeString(i,loc,a, " ");
							if(!(actualAlleleNames_[loc].Includes(alleleVal))){
									actualAlleleNames_[loc].Append(alleleVal);
									int allelePos = theLandScape.AllelePosition(species,loc,*alleleVal);
									char alleleChr = 'A' + allelePos;
									beast->cat(alleleChr);
									}
								else{
									int allelePos = theLandScape.AllelePosition(species,loc,*alleleVal);
									char alleleChr = 'A' + allelePos;
									beast->cat(alleleChr);
									delete alleleVal; alleleVal = NULL;
									}
							}//end for a
						if(thePop.ploidy(loc)<2){beast->cat(' ');}//handles haploidy
						if(loc<lociToSortThrough-1){beast->cat(' ');}
						}//end for loc					
					beast=NULL;
					}//end for i
				rawGList=NULL;
			}//end if size()...else
		}//end for pop
	npop_ -= emptyPops;

	GroupMembership();
	CullSkippedLoci();
	CheckForNewAlleles();
	delete popIDgridPtr; popIDgridPtr=NULL;
	}//end class FstatLandScapeInterface::ReadGeneticLandScape



void FstatLandScapeInterface::ReadGeneticLandScape(GeneticLandScape& theLandScape, 
				aString& simTitle, aList<int>& skippedLocusNumbers){
	aList<int> popNumbers;
	for(int i=0;i<theLandScape.area();++i){
		popNumbers.Append(i);}
	ReadGeneticLandScape(theLandScape, simTitle, skippedLocusNumbers, popNumbers);
	}//end 



void FstatLandScapeInterface::ReadGeneticLandScape(GeneticLandScape& theLandScape, 
				aString& simTitle, aList<int>& skippedLocusNumbers, aList<int>& popNumbers){
		//all species; can't handle haplodiploid or sex-linked loci
	Clear();
	title_ = simTitle;
	npop_=nloci=maxall=0; ngroups=theLandScape.species();
	aList<int> popsBySpecies;
	aList<int> totalLociBySpecies;
	for(int s=0;s<theLandScape.species();++s){
//		npop += theLandScape.area();
//		popsBySpecies.Append(theLandScape.area());
		npop_ += popNumbers.len();
		popsBySpecies.Append(popNumbers.len());
		int spLoci = theLandScape.loci(s); //for this species
		totalLociBySpecies.Append(spLoci);
		nloci = MAX(nloci, (long)spLoci);

		//make locus names
		for(int loc=0;loc<spLoci;++loc){
			aString *locName = NEW aString("LOC_");
			aString locNum(loc+1);
			locNum.PrependCharToLen('0', Digits(nloci));
			locName->cat(locNum);
			if(rawLocusName.Includes(locName)){
					delete locName;}
				else{
					rawLocusName.Append(locName);}
			 locName=NULL;
			 actualAlleleNames_.Append(NEW StringList());
			}//end for loc
		//read in the number of loci to be skipped in the analyses
		lociSkipped = skippedLocusNumbers.len();
		skippedLocusNum = skippedLocusNumbers;
		//and read in the names of skipped loci, if any
		for(int loc=0;loc<lociSkipped;++loc){
			aString *skipName = NEW aString("LOC_");
			aString skipNum(skippedLocusNumbers[loc]+1);
			skipNum.PrependCharToLen('0', Digits(nloci));
			skipName->cat(skipNum);
			if(skippedLocusName.Includes(skipName)){
					delete skipName;}
				else{
					skippedLocusName.Append(skipName);}
			 skipName=NULL;
			}//end for loc
		}//end for s

	locusName.Copy(rawLocusName);
	aList<int> actualLocusNumbers;
	for(int loc=0;loc<nloci;++loc){actualLocusNumbers.Append(loc);}
	for (int loc=0;loc<skippedLocusNumbers.len();++loc){
		aString& toSkip = skippedLocusName[loc];
		int location = locusName.Loc(&toSkip);
		locusName.Delete(location);
		actualLocusNumbers.Delete(location);
		}//end for loc
	nloci -= lociSkipped;

	//read in population data, converting integer alleles to A, B, C, etc.
	ObjList<aList<int> > popID;
	int count=0;
	for(int s=0;s<theLandScape.species();++s){
		aList<int>* theList = NEW aList<int>();
		popID.Append(theList);
		for (int pop=0;pop<popsBySpecies[s];++pop){
			theList->Append(0);
			}}
	for(int s=0;s<theLandScape.species();++s){
		int pop0=0;
		int popcount=-1;
		for(int x=0;x<theLandScape.length();++x){
			for(int y=0;y<theLandScape.width();++y){
				++popcount;
				if(popNumbers.Includes(popcount)){
					CoordinateType popcoords(x,y);
					int idd =theLandScape.GetPopulation(s,popcoords).id();
					popID[s][pop0] = idd;
					pop0++;
					}
				}}
//		popID[s][pop]=count++;
		}//s

			
	int emptyPops = 0;
	for(int s=0;s<theLandScape.species();++s){
		for (int pop=0;pop<popsBySpecies[s];++pop){
			CoordinateType xy(-1,-1);
			theLandScape.GetCoordinates(popID[s][pop],xy);
			Population& thePopP = theLandScape.GetPopulation(s, xy);
			GeneticPopulation* thePopG = (GeneticPopulation*) &thePopP;
			GeneticPopulation& thePop = *thePopG;
			
			if(thePop.size()==0){
					++emptyPops;}
				else{
					//put the {x,y} coordinates of the population onto its name
					aString *str = NEW aString("Population_");//locality first as one word
//					str->cat(pop+1);
					str->cat("y="); str->cat(xy[1]);
					str->cat("_x="); str->cat(xy[0]);
					str->cat(" of species_"); str->cat(s);//species last, as one word
					popName.Append(str);

					//make a new sublist for the genotypes of this population
					StringList *rawGList = NEW StringList();
					rawGtypes.Append(rawGList);				
					popSize.Append(thePop.size());
					twoN.Append( NEW aList<int>());
					for(int loc10=0;loc10<totalLociBySpecies[s];++loc10){//fix for haplodiploidy/sex-linkage
						(twoN.Last()).Append(thePop.size() * thePop.ploidy(loc10));}

					// BEAST is the string for a particular individual
					for(int i=0;i<thePop.size();++i){
						aString* beast = NEW aString(i+1);
						rawGList->Append(beast);
						beast->PrependCharToLen('0',4);beast->cat(' ');
						for(int loc0=0;loc0<totalLociBySpecies[s];++loc0){
							for(int a=0;a<thePop.ploidy(loc0);++a){//fix for haplodiploidy/sex-linkage
							aString* alleleVal = thePop.GenotypeString(i,loc0,a, " ");
							if(!(actualAlleleNames_[loc0].Includes(alleleVal))){
									actualAlleleNames_[loc0].Append(alleleVal);
									int allelePos = theLandScape.AllelePosition(s,loc0,*alleleVal);
									char alleleChr = 'A' + allelePos;
									beast->cat(alleleChr);
									}
								else{
									int allelePos = theLandScape.AllelePosition(s,loc0,*alleleVal);
									char alleleChr = 'A' + allelePos;
									beast->cat(alleleChr);
									delete alleleVal; alleleVal = NULL;
									}
								}//end for a
							if(thePop.ploidy(loc0)<2){beast->cat(' ');}//handles haploidy
							if(loc0<totalLociBySpecies[s]-1){beast->cat(' ');}
							}//end for loc0					
						beast=NULL;
						}//end for i
					rawGList=NULL;
				}//end if size()...else
			}//end for pop
		}//end for species
	npop_ -= emptyPops;

	GroupMembership();
	LocalityMembership();
	CullSkippedLoci();
	CheckForNewAlleles();
	for(int loc=0;loc<nloci;++loc){
		maxall = MAX(maxall, (long)GetNumAlleles(loc));}

	 }//end class FstatLandScapeInterface::ReadGeneticLandScape



void FstatLandScapeInterface::SeedGeneticLandScape(GeneticLandScape& theLandScape, 
 						int species, Ran3List& ran, LandScapeInitializer01& init, int useFileCoords){
 	//fills a properly configured GeneticLandScape with stored, unlinked genotypes
 	//if useFileCoords, then the coordinates of each population is read off the data file
 	aString endCharList(" ,{}();");
 	//get coordinates from population title
	GeneticGridCell* g;
 	theLandScape.grid_.Clear();
 	int x=0,y=-1, popp, i, loc, a;
	theLandScape.grid_.Append(NEW ObjList<GridCell>());
 	for(popp=0;popp<npop_;++popp){
 		if(useFileCoords){
 				aString poptitle(popName[popp]);
 				aString *ycoord = poptitle.ChopOffLastWord(endCharList);
 				aString *xcoord = poptitle.ChopOffLastWord(endCharList);
 				x=xcoord->toInt();
 				y=ycoord->toInt();
				delete xcoord; delete ycoord;
				CoordinateType xycoords(x,y);
				if(x>=theLandScape.grid_.len()){
					for(int xx=theLandScape.grid_.len();xx<x;++xx){
						theLandScape.grid_.Append(NEW ObjList<GridCell>());}}
				if(y<=theLandScape.grid_[x].len()){
					for(int yy=theLandScape.grid_[x].len();yy<y;++yy){
						theLandScape.grid_[x].Append(NEW GeneticGridCell(&theLandScape, xycoords,0));}}
 				}
 			else{//just go sequentially through grid
 				y++;
 				if(y>=theLandScape.width()){
 					y=0;x++;
 					theLandScape.grid_.Append(NEW ObjList<GridCell>());}
 				CoordinateType xycoords(x,y);
 				g = NEW GeneticGridCell(&theLandScape, xycoords,0);//habitat=0
				theLandScape.grid_[x].Append(g);
 				}
 		CoordinateType coords(x,y);
		Population* pop= init.popOfCorrectType->Duplicate(
								theLandScape.totalPopsCreated_++,species,g,
								theLandScape.maxCapacity_[species]);
		g->Append(pop);
		int hab = g->habitat();
 		aList<int> ploidyList;
 		//make Genome<int> or Genotype objects and fill with correct genotypes, then Install
		for (i=0;i<popSize(popp);++i){
//			Genome<int> newGenes(1, nloci, 2, (int) 0);//makes a null genotype
			Genotype newGenes(ploidyList, 0);//makes a null genotype
			for(loc=0;loc<loci();++loc){
				for(int a=0;a<ploidyList[loc];++a){
					TextDatum& gVal = GenotypicVal(popp,i,loc,a);
					int alleleVal = alleleID(loc,gVal);
					newGenes.set(loc,a,alleleVal);
					}//end for a
				}//end for loc
			//install into population
			Individual* indiv = init.indivOfCorrectType->Duplicate(++theLandScape.totalIndivsCreated_, NULL);
			GeneticIndividual* gIndiv = (GeneticIndividual*) indiv;
			Phenotype& phen =init.initPhenes_[species][hab][0];
			ViabilitySelectionFunc viabSelection = phen.selectionFunc;
			newGenes.SetSelectionFunction(viabSelection);
			gIndiv->genes_ = newGenes.Duplicate();
			double m = theLandScape.moveRate(species,theLandScape.habitat(coords),B,1);
			gIndiv->IIndividual(B, adult, m, ran, init);//age=1
			theLandScape.InstallInGrid(species, gIndiv, coords);
			}//end for i
		}//end for pop
 	}//end  FstatLandScapeInterface::SeedGeneticLandScape


*/



#endif





