//unit FstatisticAnalysisCore.cpp


#ifndef FstatisticAnalysisCorecpp
#define FstatisticAnalysisCorecpp = 1

#include "FstatisticAnalysisCore.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"



FstatisticAnalysisCore::FstatisticAnalysisCore(FstatisticAnalysisCore& parentData):
			FstatsCurrent_(parentData.FstatsCurrent_),
			Fmethod_(parentData.Fmethod_),
			Flevels_(parentData.Flevels_),
			Fstats_(parentData.Fstats_),
			FstatsLoc_(parentData.FstatsLoc_),
			jackFstatsLoc_(parentData.jackFstatsLoc_),
			FstatsAll_(parentData.FstatsAll_),
			vcCurrent_(parentData.vcCurrent_),
			vcMethod_(parentData.vcMethod_),
			vcLevels_(parentData.vcLevels_),
			vc_(parentData.vc_),
			vcLoc_(parentData.vcLoc_),
			vcAll_(parentData.vcAll_),
			IBDcurrent_(parentData.IBDcurrent_),
			IBDmethod_(parentData.IBDmethod_),
			IBDlevels_(parentData.IBDlevels_),
			IBDvals_(parentData.IBDvals_),
			IBDvalsByLoc_(parentData.IBDvalsByLoc_),
			jackIBDvalsLoc_(parentData.jackIBDvalsLoc_)
			{	
		}//end ctor FstatisticAnalysisCore(FstatisticAnalysisCore&)



void  FstatisticAnalysisCore::Initialize(void){
	FstatisticAnalysisCore::Clear();
	}//Initialize()
	
	
	
void  FstatisticAnalysisCore::Initialize(FstatisticAnalysisCore& pgd){
	FstatsCurrent_=pgd.FstatsCurrent_;
	Fmethod_=pgd.Fmethod_;
	Flevels_=pgd.Flevels_;
	Fstats_=pgd.Fstats_;
	FstatsLoc_=pgd.FstatsLoc_;
	jackFstatsLoc_=pgd.jackFstatsLoc_;
	FstatsAll_=pgd.FstatsAll_;
	vcCurrent_=pgd.vcCurrent_;
	vcMethod_=pgd.vcMethod_;
	vcLevels_=pgd.vcLevels_;
	vc_=pgd.vc_;
	vcLoc_=pgd.vcLoc_;
	vcAll_=pgd.vcAll_;
	IBDcurrent_=pgd.IBDcurrent_;
	IBDmethod_=pgd.IBDmethod_;
	IBDlevels_=pgd.IBDlevels_;
	IBDvals_=pgd.IBDvals_;
	IBDvalsByLoc_=pgd.IBDvalsByLoc_;
	jackIBDvalsLoc_=pgd.jackIBDvalsLoc_;
	}//Initialize(FstatisticAnalysisCore&)
	
	
FstatisticAnalysisCore&  FstatisticAnalysisCore::operator=(FstatisticAnalysisCore& pgd){
	FstatsCurrent_=pgd.FstatsCurrent_;
	Fmethod_=pgd.Fmethod_;
	Flevels_=pgd.Flevels_;
	Fstats_=pgd.Fstats_;
	FstatsLoc_=pgd.FstatsLoc_;
	jackFstatsLoc_=pgd.jackFstatsLoc_;
	FstatsAll_=pgd.FstatsAll_;
	vcCurrent_=pgd.vcCurrent_;
	vcMethod_=pgd.vcMethod_;
	vcLevels_=pgd.vcLevels_;
	vc_=pgd.vc_;
	vcLoc_=pgd.vcLoc_;
	vcAll_=pgd.vcAll_;
	IBDcurrent_=pgd.IBDcurrent_;
	IBDmethod_=pgd.IBDmethod_;
	IBDlevels_=pgd.IBDlevels_;
	IBDvals_=pgd.IBDvals_;
	IBDvalsByLoc_=pgd.IBDvalsByLoc_;
	jackIBDvalsLoc_=pgd.jackIBDvalsLoc_;
	return *this;
	}//operator=(FstatisticAnalysisCore&)




void FstatisticAnalysisCore::Clear(void){
	ClearFstats();
	ClearIBDs();
	}//end FstatisticAnalysisCore::Clear()



void FstatisticAnalysisCore::ClearFstats(void){
	Fmethod_.Clear();
	Flevels_.Clear();
	Fstats_.Clear();
	FstatsLoc_.Clear();
	jackFstatsLoc_.Clear();
	FstatsAll_.Clear();
	vcMethod_.Clear(); 
	vcLevels_.Clear();
	vc_.Clear();
	vcLoc_.Clear();
	vcAll_.Clear();
	FstatsCurrent_=vcCurrent_=0;
	}//end FstatisticAnalysisCore::ClearFstats()



void FstatisticAnalysisCore::ClearIBDs(void){
		//derived classes pick and delete Fstats calculated from IBD
	IBDmethod_.Clear();
	IBDlevels_.Clear();
	IBDvals_.Clear();
	IBDvalsByLoc_.Clear();
	jackIBDvalsLoc_.Clear();
	IBDcurrent_=0;
	}//end FstatisticAnalysisCore::ClearIBDs()



long double FstatisticAnalysisCore::dL_over_dS(const int Fmethod){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::dL_over_dS(). Zero value returned.");
		aError(err);
		return (long double) 0.0;}
	aString FsspWord("Fssp"), FslWord("Fsl");
	int fm = Fmethod_.Loc(Fmethod);
	int FsspLevel  = Flevels_[fm].Loc(&FsspWord);
	int FslLevel  = Flevels_[fm].Loc(&FslWord);
	long double Fssp = Fstats_[fm][FsspLevel];
	long double Fsl = Fstats_[fm][FslLevel];
	if((Fssp == one) || (Fsl == zero)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fsl) * Fssp) / ((one - Fssp) * Fsl);
	return ratio;
	}//end FstatisticAnalysisCore::dL_over_dS(const int)



long double FstatisticAnalysisCore::dL_over_dS(const int Fmethod, int locus){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::dL_over_dS(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FsspWord("Fssp"), FslWord("Fsl");
	int fm = Fmethod_.Loc(Fmethod);
	int FsspLevel  = Flevels_[fm].Loc(&FsspWord);
	int FslLevel  = Flevels_[fm].Loc(&FslWord);
	long double Fssp = FstatsLoc_[fm][locus][FsspLevel];
	long double Fsl = FstatsLoc_[fm][locus][FslLevel];
	if((Fssp == one) || (Fsl == zero)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fsl) * Fssp) / ((one - Fssp) * Fsl);
	return ratio;
	}//end FstatisticAnalysisCore::dL_over_dS(const int, int)



long double FstatisticAnalysisCore::dL_over_dSjack(const int Fmethod, int locus){
	//gives value dropping 'locus' for jackknife variance calculations
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::dL_over_dSjack(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FsspWord("Fssp"), FslWord("Fsl");
	int fm = Fmethod_.Loc(Fmethod);
	int FsspLevel  = Flevels_[fm].Loc(&FsspWord);
	int FslLevel  = Flevels_[fm].Loc(&FslWord);
	long double Fssp = jackFstatsLoc_[fm][locus][FsspLevel];
	long double Fsl = jackFstatsLoc_[fm][locus][FslLevel];
	if((Fssp == one) || (Fsl ==(long double) 0.0)){
		return 0.0;}//divide by zero gives undefined value
	long double ratio = ((one - Fsl) * Fssp) / ((one - Fssp) * Fsl);
	return ratio;
	}//end FstatisticAnalysisCore::dL_over_dSjack(const int, int)



long double FstatisticAnalysisCore::dL_over_dS(const int Fmethod, int locus, int allele){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::dL_over_dS(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FsspWord("Fssp"), FslWord("Fsl");
	int fm = Fmethod_.Loc(Fmethod);
	int FsspLevel  = Flevels_[fm].Loc(&FsspWord);
	int FslLevel  = Flevels_[fm].Loc(&FslWord);
	long double Fssp = FstatsAll_[fm][locus][allele][FsspLevel];
	long double Fsl = FstatsAll_[fm][locus][allele][FslLevel];
	if((Fssp == one) || (Fsl == zero)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fsl) * Fssp) / ((one - Fssp) * Fsl);
	return ratio;
	}//end FstatisticAnalysisCore::dL_over_dS(const int, int, int)



long double FstatisticAnalysisCore::dL_over_d(const int Fmethod){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::dL_over_d(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FlsptWord("Flspt"), FslWord("Fsl");
	int fm = Fmethod_.Loc(Fmethod);
	int FlsptLevel  = Flevels_[fm].Loc(&FlsptWord);
	int FslLevel  = Flevels_[fm].Loc(&FslWord);
	long double Flspt = Fstats_[fm][FlsptLevel];
	long double Fsl = Fstats_[fm][FslLevel];
	if((Flspt == one) || (Fsl == zero)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fsl) * Flspt) / ((one - Flspt) * Fsl);
	return ratio;
	}//end FstatisticAnalysisCore::dL_over_d(const int)



long double FstatisticAnalysisCore::dL_over_d(const int Fmethod, int locus){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::dL_over_d(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FlsptWord("Flspt"), FslWord("Fsl");
	int fm = Fmethod_.Loc(Fmethod);
	int FlsptLevel  = Flevels_[fm].Loc(&FlsptWord);
	int FslLevel  = Flevels_[fm].Loc(&FslWord);
	long double Flspt = FstatsLoc_[fm][locus][FlsptLevel];
	long double Fsl = FstatsLoc_[fm][locus][FslLevel];
	if((Flspt == one) || (Fsl == zero)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fsl) * Flspt) / ((one - Flspt) * Fsl);
	return ratio;
	}//end FstatisticAnalysisCore::dL_over_d(const int, int)



long double FstatisticAnalysisCore::dL_over_djack(const int Fmethod, int locus){
	//gives value dropping 'locus' for jackknife variance calculations
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::dL_over_djack(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FlsptWord("Flspt"), FslWord("Fsl");
	int fm = Fmethod_.Loc(Fmethod);
	int FlsptLevel  = Flevels_[fm].Loc(&FlsptWord);
	int FslLevel  = Flevels_[fm].Loc(&FslWord);
	long double Flspt = jackFstatsLoc_[fm][locus][FlsptLevel];
	long double Fsl = jackFstatsLoc_[fm][locus][FslLevel];
	if((Flspt == one) || (Fsl == zero)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fsl) * Flspt) / ((one - Flspt) * Fsl);
	return ratio;
	}//end FstatisticAnalysisCore::dL_over_djack(const int, int)



long double FstatisticAnalysisCore::dL_over_d(const int Fmethod, int locus, int allele){
	//must be caled after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::dL_over_d(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FlsptWord("Flspt"), FslWord("Fsl");
	int fm = Fmethod_.Loc(Fmethod);
	int FlsptLevel  = Flevels_[fm].Loc(&FlsptWord);
	int FslLevel  = Flevels_[fm].Loc(&FslWord);
	long double Flspt = FstatsAll_[fm][locus][allele][FlsptLevel];
	long double Fsl = FstatsAll_[fm][locus][allele][FslLevel];
	if((Flspt == one) || (Fsl == zero)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fsl) * Flspt) / ((one - Flspt) * Fsl);
	return ratio;
	}//end FstatisticAnalysisCore::dL_over_d(const int, int, int)




long double FstatisticAnalysisCore::dS_over_d(const int Fmethod){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::dS_over_d(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FlsptWord("Flspt"), FsspWord("Fssp");
	int fm = Fmethod_.Loc(Fmethod);
	int FlsptLevel  = Flevels_[fm].Loc(&FlsptWord);
	int FsspLevel  = Flevels_[fm].Loc(&FsspWord);
	long double Flspt = Fstats_[fm][FlsptLevel];
	long double Fssp = Fstats_[fm][FsspLevel];
	if((Flspt == one) || (Fssp == zero)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fssp) * Flspt) / ((one - Flspt) * Fssp);
	return ratio;
	}//end FstatisticAnalysisCore::dS_over_d(const int)



long double FstatisticAnalysisCore::dS_over_d(const int Fmethod, int locus){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::dS_over_d(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FlsptWord("Flspt"), FsspWord("Fssp");
	int fm = Fmethod_.Loc(Fmethod);
	int FlsptLevel  = Flevels_[fm].Loc(&FlsptWord);
	int FsspLevel  = Flevels_[fm].Loc(&FsspWord);
	long double Flspt = FstatsLoc_[fm][locus][FlsptLevel];
	long double Fssp = FstatsLoc_[fm][locus][FsspLevel];
	if((Flspt == one) || (Fssp == zero)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fssp) * Flspt) / ((one - Flspt) * Fssp);
	return ratio;
	}//end FstatisticAnalysisCore::dS_over_d(const int, int)



long double FstatisticAnalysisCore::dS_over_djack(const int Fmethod, int locus){
	//gives value dropping 'locus' for jackknife variance calculations
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::dS_over_djack(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FlsptWord("Flspt"), FsspWord("Fssp");
	int fm = Fmethod_.Loc(Fmethod);
	int FlsptLevel  = Flevels_[fm].Loc(&FlsptWord);
	int FsspLevel  = Flevels_[fm].Loc(&FsspWord);
	long double Flspt = jackFstatsLoc_[fm][locus][FlsptLevel];
	long double Fssp = jackFstatsLoc_[fm][locus][FsspLevel];
	if((Flspt == one) || (Fssp == zero)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fssp) * Flspt) / ((one - Flspt) * Fssp);
	return ratio;
	}//end FstatisticAnalysisCore::dS_over_djack(const int, int)



long double FstatisticAnalysisCore::dS_over_d(const int Fmethod, int locus, int allele){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::dS_over_d(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FlsptWord("Flspt"), FsspWord("Fssp");
	int fm = Fmethod_.Loc(Fmethod);
	int FlsptLevel  = Flevels_[fm].Loc(&FlsptWord);
	int FsspLevel  = Flevels_[fm].Loc(&FsspWord);
	long double Flspt = FstatsAll_[fm][locus][allele][FlsptLevel];
	long double Fssp = FstatsAll_[fm][locus][allele][FsspLevel];
	if((Flspt == one) || (Fssp == zero)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fssp) * Flspt) / ((one - Flspt) * Fssp);
	return ratio;
	}//end FstatisticAnalysisCore::dS_over_d(const int, int, int)



long double FstatisticAnalysisCore::EffectiveFstFromNm(long double Nmval, int numTrueDemes, bool zygoteMigration){
	long double Fstval = zero, scale=four;
	if(zygoteMigration){scale=eight;}
	if(numTrueDemes>1){
		long double d =(long double)numTrueDemes;
		scale *= d*d/((d-one)*(d-one));
		}
	Fstval = one/(scale*Nmval + one);
	return Fstval;
	}//FstatisticAnalysisCore::EffectiveFstFromNm(long double,int,bool)





long double FstatisticAnalysisCore::F(aString& Fword, const int Fmethod){
	//Wright's F-statistics regardless of method; assumes values already calculated
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	if (! Flevels_[m].Includes(&Fword)){
		aString err("ERROR:  ");err.cat(Fword);err.cat(" isn't included in the current Fstatistics calculated.");
		aError(err);
		return -2.0;}
	int pos=Flevels_[m].Loc(&Fword);
	return Fstats_[m][pos];
	}//end FstatisticAnalysisCore::F(aString&, const int)


long double FstatisticAnalysisCore::F(aString& Fword, int loc, const int Fmethod){
	//Wright's F-statistics regardless of method; assumes values already calculated
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");;
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	if (! Flevels_[m].Includes(&Fword)){
		aString err("ERROR:  ");err.cat(Fword);err.cat(" isn't included in the current Fstatistics calculated.");
		aError(err);
		return -2.0;}//a nonsense value
	int pos=Flevels_[m].Loc(&Fword);
	return  FstatsLoc_[m][loc][pos];
	}//end FstatisticAnalysisCore::F(aString&, int,const int)


long double FstatisticAnalysisCore::F(aString& Fword, int loc, int allele, const int Fmethod){
	//Wright's F-statistics regardless of method; assumes values already calculated
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	if (! Flevels_[m].Includes(&Fword)){
		aString err("ERROR:  ");err.cat(Fword);err.cat(" isn't included in the current Fstatistics calculated.");
		aError(err);
		return -2.0;}//a nonsense value
	int pos=Flevels_[m].Loc(&Fword);
	return  FstatsAll_[m][loc][allele][pos];
	}//end FstatisticAnalysisCore::F(aString&, int, int, const int)


long double FstatisticAnalysisCore::JackF(aString& Fword, const int Fmethod){
	//Wright's F-statistics regardless of method; assumes values already calculated
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	if (! Flevels_[m].Includes(&Fword)){
		aString err("ERROR:  ");err.cat(Fword);err.cat(" isn't included in the current Fstatistics calculated.");
		aError(err);
		return -2.0;}//a nonsense value
	int pos=Flevels_[m].Loc(&Fword);
	StatList<long double> jackFvals;
	for(int loc=0;loc<jackFstatsLoc_[m].len();++loc){
		jackFvals.Append(jackFstatsLoc_[m][loc][pos]);}
	return jackFvals.stdev();
	}//end FstatisticAnalysisCore::JackF(aString&, const int)



long double FstatisticAnalysisCore::JackIBD(aString& IBDword, const int IBDmethod){
	//Identity by descent value regardless of method; assumes values already calculated
	if(! IBDmethod_.Includes((int)IBDmethod)){
		aError("ERROR:  the IBD requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int loc, m = IBDmethod_.Loc((int)IBDmethod);
	if (! IBDlevels_[m].Includes(&IBDword)){
		aString err("ERROR:  ");err.cat(IBDword);err.cat(" isn't included in the current IBD calculated.");
		aError(err);
		return -2.0;}//a nonsense value
	int pos=IBDlevels_[m].Loc(&IBDword);
	StatList<long double> jackIBDvals;
	for(loc=0;loc<jackIBDvalsLoc_[m].len();++loc){
		jackIBDvals.Append(jackIBDvalsLoc_[m][loc][pos]);}
	return jackIBDvals.stdev();
	}//end FstatisticAnalysisCore::JackIBD(aString&, const int)



long double FstatisticAnalysisCore::Fis(const int Fmethod){	
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	aString Fword("Fis");
	int pos=Flevels_[m].Loc(&Fword);
	return Fstats_[m][pos];
	}//end FstatisticAnalysisCore::Fis(const int)



long double FstatisticAnalysisCore::Fis(int loc, const int Fmethod){	
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	aString Fword("Fis");
	int pos=Flevels_[m].Loc(&Fword);
	return FstatsLoc_[m][loc][pos];
	}//end FstatisticAnalysisCore::Fis(int, const int)



long double FstatisticAnalysisCore::Fis(int loc, int allele, const int Fmethod){	
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	aString Fword("Fis");
	int pos=Flevels_[m].Loc(&Fword);
	return FstatsAll_[m][loc][allele][pos];
	}//end FstatisticAnalysisCore::Fis(int, int,const int)



long double FstatisticAnalysisCore::Fit(const int Fmethod){	
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	aString Fword("Fit");
	int pos=Flevels_[m].Loc(&Fword);
	return Fstats_[m][pos];
	}//end FstatisticAnalysisCore::Fit(const int)



long double FstatisticAnalysisCore::Fit(int loc, const int Fmethod){	
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	aString Fword("Fit");
	int pos=Flevels_[m].Loc(&Fword);
	return FstatsLoc_[m][loc][pos];
	}//end FstatisticAnalysisCore::Fit(int, int,const int)



long double FstatisticAnalysisCore::Fit(int loc, int allele, const int Fmethod){	
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	aString Fword("Fit");
	int pos=Flevels_[m].Loc(&Fword);
	return FstatsAll_[m][loc][allele][pos];
	}//end FstatisticAnalysisCore::Fit(int, int,const int)



long double FstatisticAnalysisCore::Fst(const int Fmethod){	
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	aString Fword("Fst");
	int pos=Flevels_[m].Loc(&Fword);
	return Fstats_[m][pos];
	}//end FstatisticAnalysisCore::Fst(const int)



long double FstatisticAnalysisCore::Fst(int loc, const int Fmethod){
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	aString Fword("Fst");
	int pos=Flevels_[m].Loc(&Fword);
	return FstatsLoc_[m][loc][pos];
	}//end FstatisticAnalysisCore::Fst(int, int,const int)



long double FstatisticAnalysisCore::Fst(int loc, int allele, const int Fmethod){	
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	aString Fword("Fst");
	int pos=Flevels_[m].Loc(&Fword);
	return FstatsAll_[m][loc][allele][pos];
	}//end FstatisticAnalysisCore::Fst(int, int,const int)



long double FstatisticAnalysisCore::IBD(aString& IBDword, const int IBDmethod){
	//Identity by descent regardless of method; assumes values already calculated
	if(! IBDmethod_.Includes((int)IBDmethod)){
		aError("ERROR:  the requested identity-by-descent calculation hasn't been calculated.");
		return -two;}//a nonsense value
	int m = IBDmethod_.Loc((int)IBDmethod);
	if (! IBDlevels_[m].Includes(&IBDword)){
		aString err("ERROR:  ");err.cat(IBDword);err.cat(" isn't included in the current IBD calculated.");
		aError(err);
		return -two;}
	int pos=IBDlevels_[m].Loc(&IBDword);
	return IBDvals_[m][pos];
	}//end FstatisticAnalysisCore::IBD(aString&, const int)


long double FstatisticAnalysisCore::IBD(aString& IBDword, int loc, const int IBDmethod){
	//Identity by descent regardless of method; assumes values already calculated
	if(! IBDmethod_.Includes((int)IBDmethod)){
		aError("ERROR:  the requested identity-by-descent calculation hasn't been calculated.");
		return -two;}//a nonsense value
	int m = IBDmethod_.Loc((int)IBDmethod);
	if (! IBDlevels_[m].Includes(&IBDword)){
		aString err("ERROR:  ");err.cat(IBDword);err.cat(" isn't included in the current IBD calculated.");
		aError(err);
		return -two;}//a nonsense value
	int pos=IBDlevels_[m].Loc(&IBDword);
	return  IBDvalsByLoc_[m][loc][pos];
	}//end FstatisticAnalysisCore::IBD(aString&, int,const int)



long double FstatisticAnalysisCore::IBD_inIndiv(const int IBDmethod){
	if(! IBDmethod_.Includes((int)IBDmethod)){
		aError("ERROR:  the IBD method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = IBDmethod_.Loc((int)IBDmethod);
	aString IBDword("withinIndiv");
	int pos=IBDlevels_[m].Loc(&IBDword);
	return IBDvals_[m][pos];
	}

long double FstatisticAnalysisCore::IBD_inIndiv(int loc, const int IBDmethod){
	if(! IBDmethod_.Includes((int)IBDmethod)){
		aError("ERROR:  the IBD method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = IBDmethod_.Loc((int)IBDmethod);
	aString IBDword("withinIndiv");
	int pos=IBDlevels_[m].Loc(&IBDword);
	return IBDvalsByLoc_[m][loc][pos];
	}


long double FstatisticAnalysisCore::IBD_betwIndivInPop(const int IBDmethod){
	if(! IBDmethod_.Includes((int)IBDmethod)){
		aError("ERROR:  the IBD method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = IBDmethod_.Loc((int)IBDmethod);
	aString IBDword("betwIndivsInPop");
	int pos=IBDlevels_[m].Loc(&IBDword);
	return IBDvals_[m][pos];
	}



long double FstatisticAnalysisCore::IBD_betwIndivInPop(int loc, const int IBDmethod){
	if(! IBDmethod_.Includes((int)IBDmethod)){
		aError("ERROR:  the IBD method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = IBDmethod_.Loc((int)IBDmethod);
	aString IBDword("betwIndivsInPop");
	int pos=IBDlevels_[m].Loc(&IBDword);
	return IBDvalsByLoc_[m][loc][pos];
	}



long double FstatisticAnalysisCore::IBD_betwIndivBetwPops(const int IBDmethod){
	if(! IBDmethod_.Includes((int)IBDmethod)){
		aError("ERROR:  the IBD method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = IBDmethod_.Loc((int)IBDmethod);
	aString IBDword("betwIndivsBetwPops");
	int pos=IBDlevels_[m].Loc(&IBDword);
	return IBDvals_[m][pos];
	}



long double FstatisticAnalysisCore::IBD_betwIndivBetwPops(int loc, const int IBDmethod){
	if(! IBDmethod_.Includes((int)IBDmethod)){
		aError("ERROR:  the IBD method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = IBDmethod_.Loc((int)IBDmethod);
	aString IBDword("betwIndivsBetwPops");
	int pos=IBDlevels_[m].Loc(&IBDword);
	return IBDvalsByLoc_[m][loc][pos];
	}



long double FstatisticAnalysisCore::NSp_over_N(const int Fmethod){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::NSp_over_N(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FsptWord("Fspt"), FslWord("Fsl");
	int fm = Fmethod_.Loc(Fmethod);
	int FsptLevel  = Flevels_[fm].Loc(&FsptWord);
	int FslLevel  = Flevels_[fm].Loc(&FslWord);
	long double Fspt = Fstats_[fm][FsptLevel];
	long double Fsl = Fstats_[fm][FslLevel];
	if((Fspt == zero) || (Fsl == one)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fspt) * Fsl) / ((one - Fsl) * Fspt);
	return ratio;
	}//end FstatisticAnalysisCore::NSp_over_N(const int)



long double FstatisticAnalysisCore::NSp_over_Njack(const int Fmethod, int locus){
	//returns jackknife Fmethod dropping 'locus' for calculating variance
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::NSp_over_Njack(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FsptWord("Fspt"), FslWord("Fsl");
	int fm = Fmethod_.Loc(Fmethod);
	int FsptLevel  = Flevels_[fm].Loc(&FsptWord);
	int FslLevel  = Flevels_[fm].Loc(&FslWord);
	long double Fspt = jackFstatsLoc_[fm][locus][FsptLevel];
	long double Fsl = jackFstatsLoc_[fm][locus][FslLevel];
	if((Fspt ==zero) || (Fsl == one)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fspt) * Fsl) / ((one - Fsl) * Fspt);
	return ratio;
	}//end FstatisticAnalysisCore::NSp_over_N(const int, int)



long double FstatisticAnalysisCore::NSp_over_N(const int Fmethod, int locus){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::NSp_over_N(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FsptWord("Fspt"), FslWord("Fsl");
	int fm = Fmethod_.Loc(Fmethod);
	int FsptLevel  = Flevels_[fm].Loc(&FsptWord);
	int FslLevel  = Flevels_[fm].Loc(&FslWord);
	long double Fspt = FstatsLoc_[fm][locus][FsptLevel];
	long double Fsl = FstatsLoc_[fm][locus][FslLevel];
	if((Fspt == zero) || (Fsl == one)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fspt) * Fsl) / ((one - Fsl) * Fspt);
	return ratio;
	}//end FstatisticAnalysisCore::NSp_over_N(const int, int)



long double FstatisticAnalysisCore::NSp_over_N(const int Fmethod, int locus, int allele){
	//must be caled after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::NSp_over_N(). Zero value returned.");
		aError(err);
		return zero;}
	aString FsptWord("Fspt"), FslWord("Fsl");
	int fm = Fmethod_.Loc(Fmethod);
	int FsptLevel  = Flevels_[fm].Loc(&FsptWord);
	int FslLevel  = Flevels_[fm].Loc(&FslWord);
	long double Fspt = FstatsAll_[fm][locus][allele][FsptLevel];
	long double Fsl = FstatsAll_[fm][locus][allele][FslLevel];
	if((Fspt == zero) || (Fsl == one)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Fspt) * Fsl) / ((one - Fsl) * Fspt);
	return ratio;
	}//end FstatisticAnalysisCore::NSp_over_N(const int, int, int)



long double FstatisticAnalysisCore::NL_over_N(const int Fmethod){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::NL_over_N(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FsspWord("Fssp"), FltWord("Flt");
	int fm = Fmethod_.Loc(Fmethod);
	int FsspLevel  = Flevels_[fm].Loc(&FsspWord);
	int FltLevel  = Flevels_[fm].Loc(&FltWord);
	long double Fssp = Fstats_[fm][FsspLevel];
	long double Flt = Fstats_[fm][FltLevel];
	if((Flt == zero) || (Fssp == one)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Flt) * Fssp) / ((one - Fssp) * Flt);
	return ratio;
	}//end FstatisticAnalysisCore::NL_over_N(const int)



long double FstatisticAnalysisCore::NL_over_N(const int Fmethod, int locus){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::NL_over_N(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FsspWord("Fssp"), FltWord("Flt");
	int fm = Fmethod_.Loc(Fmethod);
	int FsspLevel  = Flevels_[fm].Loc(&FsspWord);
	int FltLevel  = Flevels_[fm].Loc(&FltWord);
	long double Fssp = FstatsLoc_[fm][locus][FsspLevel];
	long double Flt = FstatsLoc_[fm][locus][FltLevel];
	if((Flt == zero) || (Fssp ==one)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Flt) * Fssp) / ((one - Fssp) * Flt);
	return ratio;
	}//end FstatisticAnalysisCore::NL_over_N(const int, int)



long double FstatisticAnalysisCore::NL_over_Njack(const int Fmethod, int locus){
	//returns jackknife Fmethod dropping 'locus' for calculating variance
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::NL_over_Njack(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FsspWord("Fssp"), FltWord("Flt");
	int fm = Fmethod_.Loc(Fmethod);
	int FsspLevel  = Flevels_[fm].Loc(&FsspWord);
	int FltLevel  = Flevels_[fm].Loc(&FltWord);
	long double Fssp = jackFstatsLoc_[fm][locus][FsspLevel];
	long double Flt = jackFstatsLoc_[fm][locus][FltLevel];
	if((Flt == zero) || (Fssp == one)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Flt) * Fssp) / ((one - Fssp) * Flt);
	return ratio;
	}//end FstatisticAnalysisCore::NL_over_Njack(const int, int)



long double FstatisticAnalysisCore::NL_over_N(const int Fmethod, int locus, int allele){
	//must be called after FactorialFstatsIII()
	//no checking done to confirm
	if ((Fmethod < factorialFstats) || (Fmethod > factorialFstatsIII)){
		aString err("ERROR:  must use factorial F-statistics with the function FstatisticAnalysisCore::NL_over_N(). Zero value returned.");
		aError(err);
		return 0.0;}
	aString FsspWord("Fssp"), FltWord("Flt");
	int fm = Fmethod_.Loc(Fmethod);
	int FsspLevel  = Flevels_[fm].Loc(&FsspWord);
	int FltLevel  = Flevels_[fm].Loc(&FltWord);
	long double Fssp = FstatsAll_[fm][locus][allele][FsspLevel];
	long double Flt = FstatsAll_[fm][locus][allele][FltLevel];
	if((Flt == zero) || (Fssp == one)){
		return zero;}//divide by zero gives undefined value
	long double ratio = ((one - Flt) * Fssp) / ((one - Fssp) * Flt);
	return ratio;
	}//end FstatisticAnalysisCore::NL_over_N(const int, int, int)


long double FstatisticAnalysisCore::NmFromEffectiveFst(long double Fstval, int numTrueDemes, bool zygoteMigration){
	//assumes diploid inheritance
	if(Fstval<=zero) return (long double)1.0e6;
	if(Fstval>=one) return zero;
	long double Nmval=zero;
	long double scale=one;
	if(numTrueDemes>1){
		long double d=(long double)numTrueDemes;
		scale = (one-d)*(one-d)/(d*d);}
	if(zygoteMigration){
			Nmval = scale * (one - Fstval)/(eight*Fstval);}
		else{
			Nmval = scale * (one - Fstval)/(four*Fstval);}
	return Nmval;
	}//FstatisticAnalysisCodom::NmFromEffectiveFst(long double, int, bool, bool)




int  FstatisticAnalysisCore::PrintFMethodHeading(ostream& outFile, const int Fmethod){
	if(Fmethod==WandC){
			outFile<<"Weir and Cockerham's (1984) estimators of Wright's F-statistics."<<endl;}
		else if(Fmethod ==hierWandC ){
			outFile<<"Weir and Cockerham's (1984) estimators of Wright's hierarchical F-statistics."<<endl;}
		else if(Fmethod ==linearAlg ){
			outFile<<"Estimators of Wright's F-statistics using matrix methods to isolate variance components."<<endl;}
		else if(Fmethod ==hierLinearAlg ){
			outFile<<"Estimators of Wright's hierarchical F-statistics using matrix methods to isolate variance components."<<endl;}
		else if(Fmethod ==linearIntrog ){
			outFile<<"Estimators of fully Factorial F-statistics using matrix methods to isolate variance components."<<endl;}
		else if(Fmethod ==linearIntrogNoInt ){
			outFile<<"Estimators of Factorial F-statistics using matrix methods to isolate variance components."<<endl;}
		else if(Fmethod ==factorialFstats ){
			outFile<<"Estimators of fully Factorial F-statistics using quick computational forms (method I) to isolate variance components, species first."<<endl;}
		else if(Fmethod ==factorialFstatsII ){
			outFile<<"Estimators of fully Factorial F-statistics using quick computational forms (method II) to isolate variance components, locality first."<<endl;}
		else if(Fmethod ==factorialFstatsIII ){
			outFile<<"Estimators of fully Factorial F-statistics using quick computational forms (method III) to isolate variance components."<<endl;}
		else if(Fmethod ==IBDexhaustCodom ){
			outFile<<"Estimators of fully Factorial F-statistics for autosomal loci using Identity By Descent values calculated by exhaustive comparisons."<<endl;}
		else if(Fmethod ==IBDestimCodom ){
			outFile<<"Estimators of fully Factorial F-statistics for autosomal loci using Identity By Descent values estimated by sampling."<<endl;}
		else if(Fmethod ==unspecified ){
			outFile<<"Estimators of Factorial F-statistics using an unspecified method."<<endl;}
		else{
			outFile<<"ERROR: FstatisticAnalysisCore::PrintFStatistics(); shouldn't reach this step!"<<endl;
			return 0;}
	return 1;//no error
	}//end FstatisticAnalysisCore::PrintFMethodHeading



void  FstatisticAnalysisCore::PrintFStatistics(ostream& outFile, aString& titleStr, char colSpacer,int numTrueDemes, bool zygoteMigration){
	for(int m=0;m<Fmethod_.len();++m){
		PrintFStatistics(outFile,Fmethod_[m],titleStr,colSpacer, numTrueDemes, zygoteMigration);}
	}//FstatisticAnalysisCore::PrintFStatistics(ostream&, aString&, char)


void  FstatisticAnalysisCore::PrintFStatistics(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer,
												int numTrueDemes, bool zygoteMigration){
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return ;}
	int m = Fmethod_.Loc((int) Fmethod);

	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied
	char c = colSpacer;
	outFile<<titleStr<<endl<<endl;
	if(PrintFMethodHeading(outFile, Fmethod)==0){return;}//returns if error found

	if(asciiFormat){//headings
			outFile<<endl<<"F-stat    value  jackknife s.d.   Nm    Nm[low ci]    Nm[high ci]"<<endl;}
		else{
			outFile<<endl<<"F-stat"<<c<<"value"<<c<<"jackknife s.d."<<c<<"Nm"<<c<<"Nm[low ci]"<<c<<"Nm[high ci]"<<endl;}
		
	for(int f=0;f<Flevels_[m].len();++f){
		aString& fword = Flevels_[m][f];
		aString line(fword);  if(asciiFormat){line.separator(7-line.len());}else{line.cat(colSpacer);}
		long double Fval = F(fword, Fmethod);
		if(Fval>=zero) {line.separator(1);}//space for negative sign
		line.cat(Fval); if(asciiFormat){line.separator(20-line.len());}else{line.cat(colSpacer);}
		long double jackFval = JackF(fword, Fmethod);
		line.cat(jackFval); if(asciiFormat){line.separator(30-line.len());}else{line.cat(colSpacer);}
		if (fword.pos('i')==0){//appropriate for Nm calculation
			long double Nmval;
			if((Fval>zero)&&(Fval<one)){
//					Nmval = (1.0/Fval - 1.0)/4.0;
					Nmval = Nm(Fval, Fmethod, numTrueDemes, zygoteMigration);
					line.cat(Nmval,0,3); if(asciiFormat){line.separator(45-line.len());}else{line.cat(colSpacer);}
					}
				else if(Fval>=one){
					line.cat("none"); if(asciiFormat){line.separator(45-line.len());}else{line.cat(colSpacer);}
					}
				else{
					line.cat("infinite"); if(asciiFormat){line.separator(45-line.len());}else{line.cat(colSpacer);}
					}
			if((Fval+two*jackFval>zero)&&(Fval+two*jackFval<one)){
//					Nmval = (1.0/(Fval+2.0*jackFval) - 1.0)/4.0;
					Nmval = Nm(Fval+two*jackFval, Fmethod, numTrueDemes, zygoteMigration);
					line.cat(Nmval,0,3); if(asciiFormat){line.separator(59-line.len());}else{line.cat(colSpacer);}
					}
				else if(Fval+two*jackFval>=one){
					line.cat("none"); if(asciiFormat){line.separator(59-line.len());}else{line.cat(colSpacer);}
					}
				else{
					line.cat("infinite"); if(asciiFormat){line.separator(59-line.len());}else{line.cat(colSpacer);}
					}
			if(Fval-two*jackFval>zero){
//					Nmval = (1.0/(Fval-2.0*jackFval) - 1.0)/4.0;
					Nmval = Nm(Fval-two*jackFval, Fmethod, numTrueDemes, zygoteMigration);
					line.cat(Nmval,0,3); //if(asciiFormat){line.separator(86-line.len());}else{line.cat(colSpacer);}
					}
				else{
					line.cat("infinite");//if(asciiFormat){line.separator(86-line.len());}else{line.cat(colSpacer);}
					}
			}//end if Nm appropriate
		outFile<<line<<endl;
		}//end for f
	outFile<<"Nm values based on migration of ";
	if(zygoteMigration){
			outFile<<"zygotes."<<endl;}
		else{
			outFile<<"gametes."<<endl;}
	if(numTrueDemes>0){
		outFile<<"Nm values scaled to account for a finite island model with "<<numTrueDemes<<" demes.";}
	outFile<<endl<<endl;
	}//end FstatisticAnalysisCore::PrintFStatistics



void  FstatisticAnalysisCore::PrintFStatisticsByLocus(ostream& outFile, aString& titleStr, StringList& locNames, char colSpacer){
	for(int m=0;m<Fmethod_.len();++m){
		PrintFStatisticsByLocus(outFile,Fmethod_[m],titleStr,locNames,colSpacer);}
	}//FstatisticAnalysisCore::PrintFStatisticsByLocus(ostream&, aString&, StringList&, char)



void  FstatisticAnalysisCore::PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, StringList& locNames, char colSpacer){
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return ;}
	int m = Fmethod_.Loc((int) Fmethod);

	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied

	outFile<<titleStr<<endl<<endl;
	if(PrintFMethodHeading(outFile, Fmethod)==0){return;}//returns if error found

	aString tableHeadingLine("Locus");
	for(int f0=0;f0<Flevels_[m].len();++f0){
		if(asciiFormat){tableHeadingLine.separator(11*(f0+1)-tableHeadingLine.len(), '\0');}//each 11 characters
			else{tableHeadingLine.cat(colSpacer);}
		tableHeadingLine.cat(Flevels_[m][f0]);
		}
	outFile<<endl<<tableHeadingLine<<endl;
	for(int loc=0;loc<locNames.len();++loc){
		aString line(locNames[loc]); if(asciiFormat){line.separator(8-line.len());}else{line.cat(colSpacer);}
		for(int f1=0;f1<Flevels_[m].len();++f1){
			aString& fword = Flevels_[m][f1];
			long double Fval = F(fword, loc, Fmethod);
			if(Fval>=0.0) {line.separator(1);}//space for negative sign
			line.cat(Fval,1,6); if(asciiFormat){line.separator(11*(f1+1)+8-line.len());}else{line.cat(colSpacer);}
			}
		outFile<<line<<endl;
		}//end for loc
	outFile<<endl<<endl;
	}//end FstatisticAnalysisCore::PrintFStatisticsByLocus



void  FstatisticAnalysisCore::PrintFStatisticsByAllele(ostream& outFile, aString& titleStr,  
														StringList& locNames, ObjList<StringList>& alleleNames, char colSpacer){
	for(int m=0;m<Fmethod_.len();++m){
		PrintFStatisticsByAllele(outFile,Fmethod_[m],titleStr,locNames,alleleNames,colSpacer);}
	}//FstatisticAnalysisCore::PrintFStatisticsByAllele(ostream&, aString&, StringList&, char)



void  FstatisticAnalysisCore::PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, 
														StringList& locNames, ObjList<StringList>& alleleNames, char colSpacer){
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return ;}
	int m = Fmethod_.Loc((int) Fmethod);

	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied

	outFile<<titleStr<<endl<<endl;
	if(PrintFMethodHeading(outFile, Fmethod)==0){return;}//returns if error found

	aString tableHeadingLine("Locus");
	for(int f0=0;f0<Flevels_[m].len();++f0){
		if(asciiFormat){tableHeadingLine.separator(11*(f0+1)-tableHeadingLine.len(), '\0');}//each 11 characters
			else{tableHeadingLine.cat(colSpacer);}
		tableHeadingLine.cat(Flevels_[m][f0]);
		}
	outFile<<endl<<tableHeadingLine<<endl;
	for(int loc=0;loc<locNames.len();++loc){
		aString locLine(locNames[loc]);  locLine.cat(":");
		outFile<<locLine<<endl;
		for(int allele=0;allele<alleleNames[loc].len();++allele){  
			aString line(alleleNames[loc][allele]);  
			line.prepend("  ");  if(asciiFormat){line.separator(8-line.len());}else{line.cat(colSpacer);}
			for(int f1=0;f1<Flevels_[m].len();++f1){
				aString& fword = Flevels_[m][f1];
				long double Fval = F(fword, loc, allele, Fmethod);
				if(Fval>=zero) {line.separator(1);}//space for negative sign
				line.cat(Fval,1,6); if(asciiFormat){line.separator(11*(f1+1)+8-line.len());}else{line.cat(colSpacer);}
				}
			outFile<<line<<endl;
			}//end for allele		
		}//end for loc
	outFile<<endl<<endl;
	}//end FstatisticAnalysisCore::PrintFStatisticsByAllele



void FstatisticAnalysisCore::PrintFactorialFstatsDerivedStats(aString& outFileName, aString& titleStr, 
									aList<int>& variableLoci, StringList& locNames, ObjList<StringList>& alleleNames){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName);
	PrintFactorialFstatsDerivedStats(dataFile,titleStr,variableLoci,locNames,alleleNames);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end FstatisticAnalysisCore::PrintFactorialFstatsDerivedStats(aString&)



void FstatisticAnalysisCore::PrintFactorialFstatsDerivedStats(ostream& outFile, aString& titleStr,
									aList<int>& variableLoci, StringList& locNames, ObjList<StringList>& alleleNames){
//	FactorialFstatsIII();
	StatList<double> jdL_over_dS1, jdL_over_d1, jdS_over_d1, jNL_over_N1, jNS_over_N1;
	StatList<double> jdL_over_dS2, jdL_over_d2, jdS_over_d2, jNL_over_N2, jNS_over_N2;
	StatList<double> jdL_over_dS3, jdL_over_d3, jdS_over_d3, jNL_over_N3, jNS_over_N3;
	for(int loc0=0;loc0<variableLoci.len();++loc0){
		int loc = variableLoci[loc0];
		jdL_over_dS1.Append(dL_over_dSjack(factorialFstats, loc) );
		jdL_over_dS2.Append(dL_over_dSjack(factorialFstatsII, loc) );
		jdL_over_dS3.Append(dL_over_dSjack(factorialFstatsIII, loc) );

		jdL_over_d1.Append(dL_over_djack(factorialFstats, loc) );
		jdL_over_d2.Append(dL_over_djack(factorialFstatsII, loc) );
		jdL_over_d3.Append(dL_over_djack(factorialFstatsIII, loc) );

		jdS_over_d1.Append(dS_over_djack(factorialFstats, loc) );
		jdS_over_d2.Append(dS_over_djack(factorialFstatsII, loc) );
		jdS_over_d3.Append(dS_over_djack(factorialFstatsIII, loc) );

		jNL_over_N1.Append(NL_over_Njack(factorialFstats, loc) );
		jNL_over_N2.Append(NL_over_Njack(factorialFstatsII, loc) );
		jNL_over_N3.Append(NL_over_Njack(factorialFstatsIII, loc) );

		jNS_over_N1.Append(NSp_over_Njack(factorialFstats, loc) );
		jNS_over_N2.Append(NSp_over_Njack(factorialFstatsII, loc) );
		jNS_over_N3.Append(NSp_over_Njack(factorialFstatsIII, loc) );
		}

	outFile<<titleStr<<endl;
	outFile<<"Statistics derived from ratios of factorial F-statistics"<<endl<<endl;
	outFile<<"Ratios of among-species to within-species evolutionary rates."<<endl;
	outFile<<" Weighted averages over loci."<<endl;
	outFile<<"  For neutral loci,"<<endl;
	outFile<<"     m(L) is the proportional introgression rate"<<endl;
	outFile<<"     m(S) is the proportional gene flow rate"<<endl;
	outFile<<"     m = (m(L) + m(S)."<<endl<<endl;
	outFile<<"ratio \tvalue \ts.e. (jacknife over loci) \tF-stat method"<<endl;
	outFile<<"m(L)/m(S)\t"<<dL_over_dS(factorialFstats)<<'\t'<<jdL_over_dS1.stdev()<<"\tI"<<endl;
	outFile<<"m(L)/m\t"<<dL_over_d(factorialFstats)<<'\t'<<jdL_over_d1.stdev()<<"\tI"<<endl;
	outFile<<"m(S)/m\t"<<dS_over_d(factorialFstats)<<'\t'<<jdS_over_d1.stdev()<<"\tI"<<endl<<endl;

	outFile<<"m(L)/m(S)\t"<<dL_over_dS(factorialFstatsII)<<'\t'<<jdL_over_dS2.stdev()<<"\tII"<<endl;
	outFile<<"m(L)/m\t"<<dL_over_d(factorialFstatsII)<<'\t'<<jdL_over_d2.stdev()<<"\tII"<<endl;
	outFile<<"m(S)/m\t"<<dS_over_d(factorialFstatsII)<<'\t'<<jdS_over_d2.stdev()<<"\tII"<<endl<<endl;

	outFile<<"m(L)/m(S)\t"<<dL_over_dS(factorialFstatsIII)<<'\t'<<jdL_over_dS3.stdev()<<"\tIII"<<endl;
	outFile<<"m(L)/m\t"<<dL_over_d(factorialFstatsIII)<<'\t'<<jdL_over_d3.stdev()<<"\tIII"<<endl;
	outFile<<"m(S)/m\t"<<dS_over_d(factorialFstatsIII)<<'\t'<<jdS_over_d3.stdev()<<"\tIII"<<endl<<endl;
	

	outFile<<"Ratios of effective population sizes."<<endl;
	outFile<<" Weighted averages over loci."<<endl;
	outFile<<"  For neutral loci,"<<endl;
	outFile<<"     N is the effective size of a subpopulation"<<endl;
	outFile<<"     N(L) is the effective size of all the subpopulations pooled at a locality"<<endl;
	outFile<<"     N(Sp) is the efective size of al the subpopulations pooled for a species"<<endl<<endl;
	outFile<<"ratio \tvalue \ts.e. (jacknife over loci) \tF-stat method"<<endl;
	outFile<<"N(L)/N\t"<<NL_over_N(factorialFstats)<<'\t'<<jNL_over_N1.stdev()<<"\tI"<<endl;
	outFile<<"N(Sp)/N\t"<<NSp_over_N(factorialFstats)<<'\t'<<jNS_over_N1.stdev()<<"\tI"<<endl<<endl;

	outFile<<"N(L)/N\t"<<NL_over_N(factorialFstatsII)<<'\t'<<jNL_over_N2.stdev()<<"\tII"<<endl;
	outFile<<"N(Sp)/N\t"<<NSp_over_N(factorialFstatsII)<<'\t'<<jNS_over_N2.stdev()<<"\tII"<<endl<<endl;

	outFile<<"N(L)/N\t"<<NL_over_N(factorialFstatsIII)<<'\t'<<jNL_over_N3.stdev()<<"\tIII"<<endl;
	outFile<<"N(Sp)/N\t"<<NSp_over_N(factorialFstatsIII)<<'\t'<<jNS_over_N3.stdev()<<"\tIII"<<endl<<endl;


	//BY LOCUS
	outFile<<titleStr<<endl;
	outFile<<"Statistics derived from ratios of factorial F-statistics"<<endl<<endl;
	outFile<<"Ratios of among-species to within-species evolutionary rates."<<endl;
	outFile<<" Values for each locus"<<endl;
	outFile<<"  For neutral loci,"<<endl;
	outFile<<"     m(L) is the proportional introgression rate"<<endl;
	outFile<<"     m(S) is the proportional gene flow rate"<<endl;
	outFile<<"     m = (m(L) + m(S)."<<endl<<endl;
	outFile<<"locus \tratio \tvalue \tF-stat method"<<endl;
	for(int loc1=0;loc1<locNames.len();++loc1){
		int loc=variableLoci[loc1];
		outFile<<locNames[loc]<<endl;
		outFile<<"\tm(L)/m(S)\t"<<dL_over_dS(factorialFstats,loc)<<"\tI"<<endl;
		outFile<<"\tm(L)/m\t"<<dL_over_d(factorialFstats,loc)<<'\t'<<"\tI"<<endl;
		outFile<<"\tm(S)/m\t"<<dS_over_d(factorialFstats,loc)<<'\t'<<"\tI"<<endl<<endl;

		outFile<<"\tm(L)/m(S)\t"<<dL_over_dS(factorialFstatsII,loc)<<"\tII"<<endl;
		outFile<<"\tm(L)/m\t"<<dL_over_d(factorialFstatsII,loc)<<"\tII"<<endl;
		outFile<<"\tm(S)/m\t"<<dS_over_d(factorialFstatsII,loc)<<"\tII"<<endl<<endl;

		outFile<<"\tm(L)/m(S)\t"<<dL_over_dS(factorialFstatsIII,loc)<<"\tIII"<<endl;
		outFile<<"\tm(L)/m\t"<<dL_over_d(factorialFstatsIII,loc)<<"\tIII"<<endl;
		outFile<<"\tm(S)/m\t"<<dS_over_d(factorialFstatsIII,loc)<<"\tIII"<<endl<<endl;
		}//end for loc1
	

	outFile<<"Ratios of effective population sizes for each locus."<<endl;
	outFile<<" Weighted averages over loci."<<endl;
	outFile<<"  For neutral loci,"<<endl;
	outFile<<"     N is the effective size of a subpopulation"<<endl;
	outFile<<"     N(L) is the effective size of all the subpopulations pooled at a locality"<<endl;
	outFile<<"     N(Sp) is the efective size of al the subpopulations pooled for a species"<<endl<<endl;
	outFile<<"locus \tratio \tvalue \tF-stat method"<<endl;
	for(int loc2=0;loc2<variableLoci.len();++loc2){
		int loc=variableLoci[loc2];
		outFile<<locNames[loc]<<endl;
		outFile<<"\tN(L)/N\t"<<NL_over_N(factorialFstats,loc)<<"\tI"<<endl;
		outFile<<"\tN(Sp)/N\t"<<NSp_over_N(factorialFstats,loc)<<"\tI"<<endl<<endl;

		outFile<<"\tN(L)/N\t"<<NL_over_N(factorialFstatsII,loc)<<"\tII"<<endl;
		outFile<<"\tN(Sp)/N\t"<<NSp_over_N(factorialFstatsII,loc)<<"\tII"<<endl<<endl;

		outFile<<"\tN(L)/N\t"<<NL_over_N(factorialFstatsIII,loc)<<"\tIII"<<endl;
		outFile<<"\tN(Sp)/N\t"<<NSp_over_N(factorialFstatsIII,loc)<<"\tIII"<<endl<<endl;
		}//end for loc2


	//BY ALLELE
	outFile<<titleStr<<endl;
	outFile<<"Statistics derived from ratios of factorial F-statistics"<<endl<<endl;
	outFile<<"Ratios of among-species to within-species evolutionary rates."<<endl;
	outFile<<" Values for each allele"<<endl;
	outFile<<"  For neutral loci,"<<endl;
	outFile<<"     m(L) is the proportional introgression rate"<<endl;
	outFile<<"     m(S) is the proportional gene flow rate"<<endl;
	outFile<<"     m = (m(L) + m(S)."<<endl<<endl;
	outFile<<"locus /tallele \tratio \tvalue \tF-stat method"<<endl;
	for(int loc3=0;loc3<variableLoci.len();++loc3){
		int loc=variableLoci[loc3];
		outFile<<locNames[loc]<<endl;
		for(int a=0;a<alleleNames[loc].len();++a){
			outFile<<"\t"<<alleleNames[loc][a]<<endl;
			outFile<<"\t\tm(L)/m(S)\t"<<dL_over_dS(factorialFstats,loc,a)<<"\tI"<<endl;
			outFile<<"\t\tm(L)/m\t"<<dL_over_d(factorialFstats,loc)<<'\t'<<"\tI"<<endl;
			outFile<<"\t\tm(S)/m\t"<<dS_over_d(factorialFstats,loc)<<'\t'<<"\tI"<<endl<<endl;

			outFile<<"\t\tm(L)/m(S)\t"<<dL_over_dS(factorialFstatsII,loc,a)<<"\tII"<<endl;
			outFile<<"\t\tm(L)/m\t"<<dL_over_d(factorialFstatsII,loc,a)<<"\tII"<<endl;
			outFile<<"\t\tm(S)/m\t"<<dS_over_d(factorialFstatsII,loc,a)<<"\tII"<<endl<<endl;

			outFile<<"\t\tm(L)/m(S)\t"<<dL_over_dS(factorialFstatsIII,loc,a)<<"\tIII"<<endl;
			outFile<<"\t\tm(L)/m\t"<<dL_over_d(factorialFstatsIII,loc,a)<<"\tIII"<<endl;
			outFile<<"\t\tm(S)/m\t"<<dS_over_d(factorialFstatsIII,loc,a)<<"\tIII"<<endl<<endl;
			}//end for a
		}//end for loc3
	

	outFile<<"Ratios of effective population sizes for each allele."<<endl;
	outFile<<" Weighted averages over loci."<<endl;
	outFile<<"  For neutral loci,"<<endl;
	outFile<<"     N is the effective size of a subpopulation"<<endl;
	outFile<<"     N(L) is the effective size of all the subpopulations pooled at a locality"<<endl;
	outFile<<"     N(Sp) is the efective size of al the subpopulations pooled for a species"<<endl<<endl;
	outFile<<"locus /talele \tratio \tvalue \tF-stat method"<<endl;
	for(int loc4=0;loc4<variableLoci.len();++loc4){
		int loc=variableLoci[loc4];
		outFile<<locNames[loc]<<endl;
		for(int a=0;a<alleleNames[loc].len();++a){
			outFile<<"\t"<<alleleNames[loc][a]<<endl;
			outFile<<"\t\tN(L)/N\t"<<NL_over_N(factorialFstats,loc,a)<<"\tI"<<endl;
			outFile<<"\t\tN(Sp)/N\t"<<NSp_over_N(factorialFstats,loc,a)<<"\tI"<<endl<<endl;

			outFile<<"\t\tN(L)/N\t"<<NL_over_N(factorialFstatsII,loc,a)<<"\tII"<<endl;
			outFile<<"\t\tN(Sp)/N\t"<<NSp_over_N(factorialFstatsII,loc,a)<<"\tII"<<endl<<endl;

			outFile<<"\t\tN(L)/N\t"<<NL_over_N(factorialFstatsIII,loc,a)<<"\tIII"<<endl;
			outFile<<"\t\tN(Sp)/N\t"<<NSp_over_N(factorialFstatsIII,loc,a)<<"\tIII"<<endl<<endl;
			}//end for a
		}//end for loc4


	}//end FstatisticAnalysisCore::PrintFactorialFstatsDerivedStats(ostream&)




int  FstatisticAnalysisCore::PrintIBDMethodHeading(ostream& outFile, const int IBDmethod){
	if(IBDmethod==IBDexhaustCodom){
			outFile<<"Identity by descent for autosomal codominant markers: exhaustive calculation."<<endl;}
		else if(IBDmethod ==IBDestimCodom){
			outFile<<"Identity by descent for autosomal codominant markers: estimated by random sampling."<<endl;}
		else{
			outFile<<"ERROR: FstatisticAnalysisCore::PrintIBDs(); shouldn't reach this step!"<<endl;
			return 0;}
	return 1;//no error
	}//end FstatisticAnalysisCore::PrintFMethodHeading



void  FstatisticAnalysisCore::PrintIBDs(ostream& outFile, aString& titleStr, char colSpacer){
	int m;
	for(m=0;m<IBDmethod_.len();++m){
		PrintIBDs(outFile,IBDmethod_[m],titleStr,colSpacer);}
	}//FstatisticAnalysisCore::PrintFStatistics(ostream&, aString&, char)


void  FstatisticAnalysisCore::PrintIBDs(ostream& outFile, const int IBDmethod, aString& titleStr, char colSpacer){
	if(! IBDmethod_.Includes((int)IBDmethod)){
		aError("ERROR:  the identity-by-descent formulation requested hasn't been calculated.");
		return ;}
	int ibd, m = IBDmethod_.Loc((int) IBDmethod);

	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied
	char c = colSpacer;
	outFile<<titleStr<<endl<<endl;
	if(PrintIBDMethodHeading(outFile, IBDmethod)==0){return;}//returns if error found

	if(asciiFormat){//headings
			outFile<<endl<<"IBD type    value  jackknife s.d."<<endl;}
		else{
			outFile<<endl<<"IBD type"<<c<<"value"<<c<<"jackknife s.d."<<endl;}
		
	for(ibd=0;ibd<IBDlevels_[m].len();++ibd){
		aString& IBDword = IBDlevels_[m][ibd];
		aString line(IBDword);  if(asciiFormat){line.separator(7-line.len());}else{line.cat(colSpacer);}
		long double IBDval = IBD(IBDword, IBDmethod);
		if(IBDval>=zero) {line.separator(1);}//space for negative sign
		line.cat(IBDval); if(asciiFormat){line.separator(20-line.len());}else{line.cat(colSpacer);}
		long double jackIBDval = JackIBD(IBDword, IBDmethod);
		line.cat(jackIBDval); if(asciiFormat){line.separator(30-line.len());}else{line.cat(colSpacer);}
		outFile<<line<<endl;
		}//end for ibd
	outFile<<endl<<endl;
	}//end FstatisticAnalysisCore::PrintIBDs



void  FstatisticAnalysisCore::PrintIBDByLocus(ostream& outFile, aString& titleStr, StringList& locNames, char colSpacer){
	int m;
	for(m=0;m<IBDmethod_.len();++m){
		PrintIBDByLocus(outFile,IBDmethod_[m],titleStr,locNames,colSpacer);}
	}//FstatisticAnalysisCore::PrintIBDByLocus(ostream&, aString&, StringList&, char)



void  FstatisticAnalysisCore::PrintIBDByLocus(ostream& outFile, const int IBDmethod, aString& titleStr, StringList& locNames, char colSpacer){
	if(! IBDmethod_.Includes((int)IBDmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return ;}
	int loc, f0, f1, m = IBDmethod_.Loc((int) IBDmethod);

	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied

	outFile<<titleStr<<endl<<endl;
	if(PrintIBDMethodHeading(outFile, IBDmethod)==0){return;}//returns if error found

	aString tableHeadingLine("Locus");
	for(f0=0;f0<IBDlevels_[m].len();++f0){
		if(asciiFormat){tableHeadingLine.separator(11*(f0+1)-tableHeadingLine.len(), '\0');}//each 11 characters
			else{tableHeadingLine.cat(colSpacer);}
		tableHeadingLine.cat(IBDlevels_[m][f0]);
		}
	outFile<<endl<<tableHeadingLine<<endl;
	for(loc=0;loc<locNames.len();++loc){
		aString line(locNames[loc]); if(asciiFormat){line.separator(8-line.len());}else{line.cat(colSpacer);}
		for(f1=0;f1<IBDlevels_[m].len();++f1){
			aString& IBDword = IBDlevels_[m][f1];
			long double IBDval = IBD(IBDword, loc, IBDmethod);
			if(IBDval>=zero) {line.separator(1);}//space for negative sign
			line.cat(IBDval,1,6); if(asciiFormat){line.separator(11*(f1+1)+8-line.len());}else{line.cat(colSpacer);}
			}
		outFile<<line<<endl;
		}//end for loc
	outFile<<endl<<endl;
	}//end FstatisticAnalysisCore::PrintIBDByLocus




int  FstatisticAnalysisCore::PrintVCmethodHeading(ostream& outFile, const int vcMethod){
	if(vcMethod==WandC){
			outFile<<"Weir and Cockerham's (1984) estimators of variance components."<<endl;}
		else if(vcMethod ==hierWandC ){
			outFile<<"Weir and Cockerham's (1984) estimators of hierarchical variance components."<<endl;}
		else if(vcMethod ==linearAlg ){
			outFile<<"Estimators using matrix methods to isolate variance components."<<endl;}
		else if(vcMethod ==hierLinearAlg ){
			outFile<<"Estimators using matrix methods to isolate hierarchical variance components."<<endl;}
		else if(vcMethod ==linearIntrog ){
			outFile<<"Estimators using matrix methods to isolate fully factorial variance components."<<endl;}
		else if(vcMethod ==linearIntrogNoInt ){
			outFile<<"Estimators using matrix methods to isolate variance components."<<endl;}
		else if(vcMethod ==factorialFstats ){
			outFile<<"Estimators using quick computational forms (method I) to isolate variance components, species first."<<endl;}
		else if(vcMethod ==factorialFstatsII ){
			outFile<<"Estimators using quick computational forms (method II) to isolate variance components, locality first."<<endl;}
		else if(vcMethod ==factorialFstatsIII ){
			outFile<<"Estimators using quick computational forms (method III) to isolate variance components."<<endl;}
		else if(vcMethod ==unspecified ){
			outFile<<"Estimators of variance components using an unspecified method."<<endl;}
		else{
			outFile<<"ERROR: FstatisticAnalysisCore::PrintVCmethodHeading(); shouldn't reach this step!"<<endl;
			return 0;}
	return 1;//no error
	}//end FstatisticAnalysisCore::PrintVCmethodHeading


void  FstatisticAnalysisCore::PrintVarianceComponents(ostream& outFile, const int vcMethod, aString& titleStr, char colSpacer){
	if(! vcMethod_.Includes((int)vcMethod)){
		aError("ERROR:  the vc-method requested hasn't been calculated.");
		return ;}
	int m = vcMethod_.Loc((int) vcMethod);

	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied
	char c = colSpacer;
	outFile<<titleStr<<endl<<endl;
	if(PrintVCmethodHeading(outFile, vcMethod)==0){return;}//returns if error found

	if(asciiFormat){//headings
			outFile<<endl<<"var comp    value"<<endl;}
		else{
			outFile<<endl<<"var comp"<<c<<"value"<<endl;}
		
	for(int vc=0;vc<vcLevels_[m].len();++vc){
		aString& vcWord = vcLevels_[m][vc];
		aString line(vcWord);  if(asciiFormat){line.separator(7-line.len());}else{line.cat(colSpacer);}
		double vcVal = vc_[m][vc];
		if(vcVal>=0.0) {line.separator(1);}//space for negative sign
		line.cat(vcVal); 
		outFile<<line<<endl;
		}//end for vc
	outFile<<endl<<endl;
	}//end FstatisticAnalysisCore::PrintVarianceComponents



void  FstatisticAnalysisCore::PrintVarianceComponentsByLocus(ostream& outFile, const int vcMethod, aString& titleStr, StringList& locNames, char colSpacer){
	if(! vcMethod_.Includes((int)vcMethod)){
		aError("ERROR:  the vc-method requested hasn't been calculated.");
		return ;}
	int m = vcMethod_.Loc((int) vcMethod);

	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied
	char c = colSpacer;
	outFile<<titleStr<<endl<<endl;
	if(PrintVCmethodHeading(outFile, vcMethod)==0){return;}//returns if error found

	aString tableHeadingLine("Locus");
	for(int vc0=0;vc0<vcLevels_[m].len();++vc0){
		if(asciiFormat){tableHeadingLine.separator(11*(vc0+1)-tableHeadingLine.len(), '\0');}//each 11 characters
			else{tableHeadingLine.cat(colSpacer);}
		tableHeadingLine.cat(vcLevels_[m][vc0]);
		}
	outFile<<endl<<tableHeadingLine<<endl;
	for(int loc=0;loc<locNames.len();++loc){
		aString line(locNames[loc]); 
		if(asciiFormat){line.separator(8-line.len());}else{line.cat(colSpacer);}
		for(int vc1=0;vc1<vcLevels_[m].len();++vc1){
			aString& vcword = vcLevels_[m][vc1];
			double vcVal = vcLoc_[m][loc][vc1];
			if(vcVal>=0.0) {line.separator(1);}//space for negative sign
			line.cat(vcVal,1,6); 
			if(asciiFormat){line.separator(11*(vc1+1)+8-line.len());}else{line.cat(colSpacer);}
			}//vc1
		outFile<<line<<endl;
		}//end for loc
	outFile<<endl<<endl;
	}//end FstatisticAnalysisCore::PrintVarianceComponentsByLocus


void  FstatisticAnalysisCore::PrintVarianceComponentsByAllele(ostream& outFile, const int vcMethod, aString& titleStr, 
														StringList& locNames, ObjList<StringList>& alleleNames, char colSpacer){
	if(! vcMethod_.Includes((int)vcMethod)){
		aError("ERROR:  the vc-method requested hasn't been calculated.");
		return ;}
	int m = vcMethod_.Loc((int) vcMethod);

	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied

	char c = colSpacer;
	outFile<<titleStr<<endl<<endl;
	if(PrintVCmethodHeading(outFile, vcMethod)==0){return;}//returns if error found

	aString tableHeadingLine("Locus");
	for(int vc0=0;vc0<vcLevels_[m].len();++vc0){
		if(asciiFormat){tableHeadingLine.separator(11*(vc0+1)-tableHeadingLine.len(), '\0');}//each 11 characters
			else{tableHeadingLine.cat(colSpacer);}
		tableHeadingLine.cat(vcLevels_[m][vc0]);
		}
	outFile<<endl<<tableHeadingLine<<endl;
	for(int loc=0;loc<locNames.len();++loc){
		aString locLine(locNames[loc]);  locLine.cat(":");
		outFile<<locLine<<endl;
		for(int allele=0;allele<alleleNames[loc].len();++allele){  
			aString line(alleleNames[loc][allele]);  
			line.prepend("  ");  if(asciiFormat){line.separator(8-line.len());}else{line.cat(colSpacer);}
			for(int vc1=0;vc1<vcLevels_[m].len();++vc1){
				aString& vcword = vcLevels_[m][vc1];
				double vcVal = vcAll_[m][loc][allele][vc1];//F(fword, loc, allele, Fmethod);
				if(vcVal>=0.0) {line.separator(1);}//space for negative sign
				line.cat(vcVal,1,6); 
				if(asciiFormat){line.separator(11*(vc1+1)+8-line.len());}else{line.cat(colSpacer);}
				}
			outFile<<line<<endl;
			}//end for allele		
		}//end for loc
	outFile<<endl<<endl;
	}//end FstatisticAnalysisCore::PrintFStatisticsByAllele



void FstatisticAnalysisCore::ResetFlags(void){
	FstatsCurrent_=0;
	vcCurrent_=0;
	}


long double FstatisticAnalysisCore::varianceComponent(aString& vcWord, int loc, int allele, const int Fmethod){
	//Wright's variance components for F-statistics regardless of method; assumes values already calculated
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	if (! vcLevels_[m].Includes(&vcWord)){
		aString err("ERROR:  ");err.cat(vcWord);err.cat(" isn't included in the current variance components list.  Wrong name?");
		aError(err);
		return -2.0;}//a nonsense value
	int pos=vcLevels_[m].Loc(&vcWord);
	return  vcAll_[m][loc][allele][pos];
	}//end FstatisticAnalysisCore::varianceComponent(aString&, int, int, const int)


long double FstatisticAnalysisCore::varianceComponent(aString& vcWord, int loc, const int Fmethod){
	//Wright's variance components for F-statistics regardless of method; assumes values already calculated
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	if (! vcLevels_[m].Includes(&vcWord)){
		aString err("ERROR:  ");err.cat(vcWord);err.cat(" isn't included in the current variance components list.  Wrong name?");
		aError(err);
		return -2.0;}//a nonsense value
	int pos=vcLevels_[m].Loc(&vcWord);
	return  vcLoc_[m][loc][pos];
	}//end FstatisticAnalysisCore::varianceComponent(aString&, int, const int)


long double FstatisticAnalysisCore::varianceComponent(aString& vcWord, const int Fmethod){
	//Wright's variance components for F-statistics regardless of method; assumes values already calculated
	if(! Fmethod_.Includes((int)Fmethod)){
		aError("ERROR:  the F-method requested hasn't been calculated.");
		return -2.0;}//a nonsense value
	int m = Fmethod_.Loc((int)Fmethod);
	if (! vcLevels_[m].Includes(&vcWord)){
		aString err("ERROR:  ");err.cat(vcWord);err.cat(" isn't included in the current variance components list.  Wrong name?");
		aError(err);
		return -2.0;}//a nonsense value
	int pos=vcLevels_[m].Loc(&vcWord);
	return  vc_[m][pos];
	}//end FstatisticAnalysisCore::varianceComponent(aString&, const int)



#endif


