//FstatisticAnalysisDom2.cpp

#ifndef FstatisticAnalysisDom2cpp
#define FstatisticAnalysisDom2cpp=1

#include "FstatisticAnalysisDom2.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"






FstatisticAnalysisDom2::FstatisticAnalysisDom2(aString& s){
	FstatisticAnalysisCore::Initialize(); 
	DominantMarkerData2::Initialize();
	EvolutionaryGeneticsData::Initialize(s); 
	}


FstatisticAnalysisDom2::FstatisticAnalysisDom2(EvolutionaryGeneticsData& egd){
	DominantMarkerData2::Initialize();
	EvolutionaryGeneticsData::Initialize(egd);//do this second to overwrite blank initialization
	FstatisticAnalysisCore::Initialize();
	}//FstatisticAnalysisDom2::FstatisticAnalysisDom2(EvolutionaryGeneticsData&)



FstatisticAnalysisDom2::FstatisticAnalysisDom2(DominantMarkerData2& dmd){
	DominantMarkerData2::Initialize(dmd);
	FstatisticAnalysisCore::Initialize();
	}//FstatisticAnalysisDom2::FstatisticAnalysisDom2(DominantMarkerData2&)




FstatisticAnalysisDom2::FstatisticAnalysisDom2(FstatisticAnalysisDom2& dmd){//copy constructor
	DominantMarkerData2::Initialize(dmd);
	FstatisticAnalysisCore::Initialize(dmd);
	}//FstatisticAnalysisDom2::FstatisticAnalysisDom2(FstatisticAnalysisDom2&)





FstatisticAnalysisDom2::FstatisticAnalysisDom2(FstatisticAnalysisDom2& parentData, aList<int>& popSubsetNums) {
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	DominantMarkerData2::Initialize(parentData, popSubsetNums);//copies
	}//end ctor FstatisticAnalysisDom2(FstatisticAnalysisDom2&, aList<int>&)


FstatisticAnalysisDom2::FstatisticAnalysisDom2(DominantMarkerData2& parentData, aList<int>& popSubsetNums) {
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	DominantMarkerData2::Initialize(parentData, popSubsetNums);//copies
	}//end ctor FstatisticAnalysisDom2(DominantMarkerData2&, aList<int>&)





void FstatisticAnalysisDom2::Clear(){
	FstatisticAnalysisCore::Clear();
	DominantMarkerData2::Clear();
	}


void FstatisticAnalysisDom2::Initialize(FstatisticAnalysisDom2& dmd){//copy constructor
	DominantMarkerData2::Initialize(dmd);
	FstatisticAnalysisCore::Initialize(dmd);
	}//FstatisticAnalysisDom2::FstatisticAnalysisDom2(FstatisticAnalysisDom2&)



long double FstatisticAnalysisDom2::DomFstFromNm(long double NmVal, int numTrueDemes, bool zygoteMigration){
	long double scale = two;
	if(zygoteMigration){
		scale=four;}
	if(numTrueDemes<2){numTrueDemes=0;}
		else{numTrueDemes=MAX((int)npop_,numTrueDemes);}
	if(numTrueDemes>1){
		long double d = (long double)numTrueDemes;
		scale *= d*d/((d-one)*(d-one));}
	return one / (scale * NmVal + one);
	}//FstatisticAnalysisDom2::DomFstFromNm(long double,bool,bool)



void FstatisticAnalysisDom2::DomMarkerFstats (int numTrueDemes, bool zygoteMigration){
	//calculates F-statistics among populations using dominant markers,
	//to be used when there is only one group of pops.
	//Notation follows Porter MS in prep

	//clear out any previous calculations if necessary
  	if(! FstatsCurrent_){
  		ClearFstats();}
	if((Fmethod_.Includes((int) domStd))){return;}
	if(nlociDom_==0){return;}
	FstatsCurrent_=vcCurrent_=0;
	Fmethod_.Append((int) domStd);
	int m = Fmethod_.len()-1;
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, F-level
	Fstats_.Append(NEW aList<long double>());//F-level
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	Flevels_.Append(NEW StringList());//F-level

	vcMethod_.Append((int) domStd);
	vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
	vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
	vc_.Append(NEW aList<long double>());//vc-level
	vcLevels_.Append(NEW StringList());//vc-level

	Fmethod_.Append((int) domStdEffective);
	int mEff = Fmethod_.len()-1;
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, F-level
	Fstats_.Append(NEW aList<long double>());//F-level
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	Flevels_.Append(NEW StringList());//F-level

	vcMethod_.Append((int) domStdEffective);
	vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
	vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
	vc_.Append(NEW aList<long double>());//vc-level
	vcLevels_.Append(NEW StringList());//vc-level

	//set up all the F-stat arrays
	// first the array describing the Fstat structure:
	aString* FWord=NEW aString("Fst");
	Flevels_[m].Append(FWord);
	Flevels_[mEff].Append(NEW aString(*FWord));

	aString* vcWord=NEW aString("var(p)");
	vcLevels_[m].Append(vcWord);
	vcLevels_[mEff].Append(NEW aString(*vcWord));
	vcWord=NEW aString("var(i)");
	vcLevels_[m].Append(vcWord);
	vcLevels_[mEff].Append(NEW aString(*vcWord));
	vcWord=NEW aString("var(t)");
	vcLevels_[m].Append(vcWord);
	vcLevels_[mEff].Append(NEW aString(*vcWord));
	int Flevel,vclevel,pop,j,loc,a;
	//make and initialize the jackknife & by-locus Fstat lists
	for (Flevel=0;Flevel<1;++Flevel){
		Fstats_[m].Append(zero);
		Fstats_[mEff].Append(zero);
		}
	for (vclevel=0;vclevel<3;++vclevel){
		vc_[m].Append(zero);
		vc_[mEff].Append(zero);
		}
	//some intermediate variables
	aList<long double> sumILoc;
	aList<long double> sumSLoc;
	aList<long double> sumISLoc;

	DominantMarkerData2::CheckForNewAlleles();
	DominantMarkerData2::CalculatePhenotypeFreqs();
	DominantMarkerData2::UpdateVariableLociDom();
	aList<long double> Q;//global freqs for each marker
	for (j=0;j<nlociDom_;++j){
		//make and initialize the jackknife & by-locus Fstat lists for this locus 
		FstatsLoc_[m].Append(NEW aList<long double>());
		jackFstatsLoc_[m].Append(NEW aList<long double>());
		FstatsLoc_[mEff].Append(NEW aList<long double>());
		jackFstatsLoc_[mEff].Append(NEW aList<long double>());
		for (int Flevel=0;Flevel<1;++Flevel){
			FstatsLoc_[m][j].Append(zero);
			jackFstatsLoc_[m][j].Append(zero);
			FstatsLoc_[mEff][j].Append(zero);
			jackFstatsLoc_[mEff][j].Append(zero);
			}
		//make and initialize the jackknife & by-allele Fstat lists for this locus 
		FstatsAll_[m].Append(NEW ObjList<aList<long double> >());
		FstatsAll_[mEff].Append(NEW ObjList<aList<long double> >());
		//initialize variables
		sumILoc.Append(zero);
		sumSLoc.Append(zero);
		sumISLoc.Append(zero);

		//make and initialize the by-allele F-stat list for each allele and append to this locus list
		StringList& locAlleleList = allelesPresentDom_[j];
		for (a=0;a<locAlleleList.len();++a){
			FstatsAll_[m][j].Append(NEW aList<long double>());
			FstatsAll_[mEff][j].Append(NEW aList<long double>());
			for (Flevel=0;Flevel<1;++Flevel){
				FstatsAll_[m][j][a].Append(zero);
				FstatsAll_[mEff][j][a].Append(zero);
				}
			}//end for a							

		//make and initialize the by-allele var comp list
		vcLoc_[m].Append(NEW aList<long double>());
		vcLoc_[mEff].Append(NEW aList<long double>());
		for (vclevel=0;vclevel<3;++vclevel){
			vcLoc_[m][j].Append(zero);
			vcLoc_[mEff][j].Append(zero);
				}
		vcAll_[m].Append(NEW ObjList<aList<long double> >());
		vcAll_[mEff].Append(NEW ObjList<aList<long double> >());
		for (a=0;a<locAlleleList.len();++a){
			vcAll_[m][j].Append(NEW aList<long double>());
			vcAll_[mEff][j].Append(NEW aList<long double>());
			for (vclevel=0;vclevel<3;++vclevel){
				vcAll_[m][j][a].Append(zero);
				vcAll_[mEff][j][a].Append(zero);
					}
			}//end for a							
		Q.Append(zero);
		}//end for j

	//the main body of the calculations	
	long double n,n0,nbar,nsq,dfI,ssqI,ssqS,MSI,MSS,vcI,vcS,NmfromFst,FstEffFromNm,fstVal;
	long double r = (long double) (npop_);
	long double dfS = r - one;
	long double Isum,Ssum,ISsum;
	Isum=Ssum=ISsum=zero;
	aString levelName;
	for (j=0;j<variableLocusNumDom_.len();++j){
		loc = variableLocusNumDom_[j];//the actual locus
//		StringList& locAlleleList = allelesPresentDom[loc];
		n=n0=nbar=nsq=dfI=ssqI=ssqS=MSI=MSS=vcI=vcS=zero;
		StatList<long double> q;//marker frequencies for this locus at each pop
		StatList<long double> nn;//sample sizes for this locus at each pop
		
		//Calculate for the whole population set:
		for (pop=0;pop<npop_;++pop){
			q.Append(DominantMarkerData2::GetFreqDom(pop,loc, 1));
			nn.Append((double) (DominantMarkerData2::GetLocusSampleSizeDom(pop,loc)) );
			//average sample size among pops:
			nsq += nn[pop] * nn[pop];//running total
			// Average marker frequency (=Q)
			Q[loc] += nn[pop] * q[pop];
			}//pop
		n = nn.sum();//total sample size
		if ((r>one)&&(n>zero)) {//this weeds out zero denominators
			nbar = n/r;
			n0 = (n - nsq/n) / dfS;
			Q[loc] /=n;

			//Now get the term accounting for variance among allelic frequencies
			for (pop=0;pop<npop_;++pop){
				ssqI += nn[pop] * q[pop]*(one - q[pop]);
				ssqS += nn[pop] * (q[pop] - Q[loc]) * (q[pop] - Q[loc]);
				}//end for pop
			dfI = n - r;
			
			MSI = ssqI/dfI;
			MSS = ssqS/dfS;
			
			//variance components
			vcI = MSI;
			if (n0 != zero){
				vcS = (MSS - MSI)/n0;
				}
			}//end if r>1, n>0

		for(a=0;a<allelesPresentDom_[loc].len();++a){//same for both marker phenotypes
			vcAll_[m][loc][a][0] = vcS;
			vcAll_[m][loc][a][1] = vcI;
			vcAll_[m][loc][a][2] = vcS+vcI;
			vcAll_[mEff][loc][a][0] = vcS;
			vcAll_[mEff][loc][a][1] = vcI;
			vcAll_[mEff][loc][a][2] = vcS+vcI;
			//A, B, &C remain zero if their components have any zero denominators
			if ((vcS+vcI) != zero) {
				levelName="Fst";
				Flevel = Flevels_[m].Loc(&levelName);
//				FstatsAll_[m][loc][a].Replace(vcS / (vcS+vcI), Flevel);
				fstVal = FstatsAll_[m][loc][a][Flevel] = vcS / (vcS+vcI);
//				FstatsAll_[mEff][loc][a][Flevel] = vcS / (two*(vcS+vcI) - vcS);
				NmfromFst = NmFromDomFst(fstVal,numTrueDemes,zygoteMigration);
				FstEffFromNm = EffectiveFstFromNm(NmfromFst,numTrueDemes,zygoteMigration);
				FstatsAll_[mEff][loc][a][Flevel] = FstEffFromNm;
				}//end if vcS+vcI
			}//a
		//save these in an array for a quicker calculation of the jackknife variance estimates
		sumILoc[loc] = vcI;
		sumSLoc[loc] = vcS;
		sumISLoc[loc] = vcI+vcS;

		Isum += vcI;
		Ssum += vcS;
		ISsum += vcI+vcS;
			
		vcLoc_[m][loc][0] = sumSLoc[loc];
		vcLoc_[m][loc][1] = sumILoc[loc];
		vcLoc_[m][loc][2] = sumISLoc[loc];
		vcLoc_[mEff][loc][0] = sumSLoc[loc];
		vcLoc_[mEff][loc][1] = sumILoc[loc];
		vcLoc_[mEff][loc][2] = sumISLoc[loc];
//		aList<long double>& locFstat = FstatsLoc_[m][loc];
		long double temp = sumISLoc[loc];
		if (sumISLoc[loc] != zero) {
			levelName="Fst";
			Flevel = Flevels_[m].Loc(&levelName);
			fstVal = FstatsLoc_[m][loc][Flevel]=sumSLoc[loc] / sumISLoc[loc];
//			FstatsLoc_[mEff][loc][Flevel]=sumSLoc[loc] / (two* sumISLoc[loc] - sumSLoc[loc]);
			NmfromFst = NmFromDomFst(fstVal,numTrueDemes,zygoteMigration);
			FstEffFromNm = EffectiveFstFromNm(NmfromFst,numTrueDemes,zygoteMigration);
			FstatsLoc_[mEff][loc][Flevel] = FstEffFromNm;
			}
		}//end for j

	vc_[m][0] = Ssum;
	vc_[m][1] = Isum;
	vc_[m][2] = ISsum;
	vc_[mEff][0] = Ssum;
	vc_[mEff][1] = Isum;
	vc_[mEff][2] = ISsum;
	if (ISsum != zero){
		levelName="Fst";
		Flevel = Flevels_[m].Loc(&levelName);
		fstVal = Fstats_[m][Flevel] = Ssum/ISsum;
//		Fstats_[mEff][Flevel] = Ssum/(two * ISsum - Ssum);
		NmfromFst = NmFromDomFst(fstVal,numTrueDemes,zygoteMigration);
		FstEffFromNm = EffectiveFstFromNm(NmfromFst,numTrueDemes,zygoteMigration);
		Fstats_[mEff][Flevel] = FstEffFromNm;
		}//end if ISsum

	//now do the jackknife calculations, over loci
	for (j=0;j<variableLocusNumDom_.len();++j){
		loc = variableLocusNumDom_[j];//the actual locus		
		if (ISsum != sumISLoc[loc]) {
			levelName="Fst";
			Flevel = Flevels_[m].Loc(&levelName);
			fstVal = jackFstatsLoc_[m][loc][Flevel] = (Ssum-sumSLoc[loc]) / (ISsum - sumISLoc[loc]);
//			jackFstatsLoc_[mEff][loc][Flevel] = (Ssum-sumSLoc[loc]) / (ISsum - (two* sumISLoc[loc] - sumSLoc[loc]));
			NmfromFst = NmFromDomFst(fstVal,numTrueDemes,zygoteMigration);
			FstEffFromNm = EffectiveFstFromNm(NmfromFst,numTrueDemes,zygoteMigration);
			jackFstatsLoc_[mEff][loc][Flevel] = FstEffFromNm;
			}//end if 
		}//end for j

	FstatsCurrent_=vcCurrent_=1;
	}//end FstatisticAnalysisDom2::DomMarkerFstats


long double FstatisticAnalysisDom2::Nm(long double FstVal,int Fmethod, int numTrueDemes, bool zygoteMigration){
	long double Nmval = zero;
	if(Fmethod==domStdEffective){
			//requires a diploid analog
			if(FstVal<=zero) return (long double)1.0e6;
			if(FstVal>=one) return zero;
			long double scale=four;
			if(zygoteMigration){
				scale=eight;}
				if(numTrueDemes<2){numTrueDemes=0;}
					else{numTrueDemes=MAX((int)npop_,numTrueDemes);}
			if(numTrueDemes>1){
				long double d=(long double)numTrueDemes;
				scale *= d*d/(d-one)*(d-one);}
			Nmval = (one - FstVal)/(scale*FstVal);
			return Nmval;
			}
		else{
			Nmval = NmFromDomFst(FstVal,numTrueDemes,zygoteMigration);}
	return Nmval;
	}//FstatisticAnalysisDom2::Nm(long double,int, int, bool)



long double FstatisticAnalysisDom2::NmFromDomFst(long double FstDom, int numTrueDemes, bool zygoteMigration){
	if(FstDom<=zero) return (long double)1.0e6;
	if(FstDom>=one) return zero;
	long double Nmval=zero, scale=two;
	if(zygoteMigration){
		scale=four;}
	if(numTrueDemes<2){numTrueDemes=0;}
		else{numTrueDemes=MAX((int)npop_,numTrueDemes);}
	if(numTrueDemes>1){
		long double d=(long double)numTrueDemes;
		scale *= d*d/(d-one)*(d-one);}
	Nmval = (one - FstDom)/(scale*FstDom);
	return Nmval;
	}//FstatisticAnalysisDom2::NmFromDomFst(long double, int, bool)




int  FstatisticAnalysisDom2::PrintFMethodHeading(ostream& outFile, const int Fmethod){
	if(Fmethod==domStd){
			outFile<<"Least-squares estimators of Wright's F-statistics for dominant markers."<<endl;}
		else if(Fmethod ==hierDom){
			outFile<<"Least-squares estimators of Wright's hierarchical F-statistics for dominant markers."<<endl;}
		else {
			return FstatisticAnalysisCore::PrintFMethodHeading(outFile,Fmethod);}
	return 1;//no error
	}//end FstatisticAnalysisDom2::PrintFMethodHeading



void FstatisticAnalysisDom2::HierarchicalDomMarkerFstats(void){}



void FstatisticAnalysisDom2::PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer){
	StringList namesOfLoci;
	DominantMarkerData2::GetLocusNames(namesOfLoci);
	FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod, titleStr, namesOfLoci, colSpacer);
	}
	
	
void FstatisticAnalysisDom2::PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer){
	StringList namesOfLoci;
	DominantMarkerData2::GetLocusNames(namesOfLoci);
	FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod, titleStr, namesOfLoci, allelesPresentDom_, colSpacer);
	}


void FstatisticAnalysisDom2::PrintNEXUSFile(aString& outFileName, StringList& textToAppend){
	DominantMarkerData2::PrintNEXUSFile(outFileName, textToAppend);
	}//end FstatisticAnalysisDom2::PrintNEXUSFile(aString&, StringList&)



void FstatisticAnalysisDom2::PrintNEXUSFile (ostream& outFile, StringList& textToAppend){
	DominantMarkerData2::PrintNEXUSFile(outFile, textToAppend);
	}//end FstatisticAnalysisDom2::PrintNEXUSFile(ostream&,StringList&)




void FstatisticAnalysisDom2::ResetFlags(void){
	FstatisticAnalysisCore::ResetFlags();
	DominantMarkerData2::ResetFlags();
	}


FstatisticAnalysisDom2& FstatisticAnalysisDom2::operator=(EvolutionaryGeneticsData& egd){
	Clear();
	EvolutionaryGeneticsData::Initialize(egd);
	return *this;
	}//FstatisticAnalysisDom2::operator=(EvolutionaryGeneticsData&)


FstatisticAnalysisDom2& FstatisticAnalysisDom2::operator=(DominantMarkerData2& dmd){
	Clear();
	DominantMarkerData2::Initialize(dmd);
	return *this;
	}//FstatisticAnalysisDom2::operator=(DominantMarkerData2&)



FstatisticAnalysisDom2& FstatisticAnalysisDom2::operator=(FstatisticAnalysisDom2& dmd){
	Clear();
	DominantMarkerData2::Initialize(dmd);
	FstatisticAnalysisCore::Initialize(dmd);
	return *this;
	}//FstatisticAnalysisDom2::operator=(FstatisticAnalysisDom2&)


#endif
