//FstatisticAnalysisDomSexLink.cpp

#ifndef FstatisticAnalysisDomSexLinkcpp
#define FstatisticAnalysisDomSexLinkcpp=1

#include "FstatisticAnalysisDomSexLink.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"


//20081229	fixed a bug in SexLinkedDomFstFromNm(long double, int, bool) that affects cases where primarySexRatio_!=0.5



FstatisticAnalysisDomSexLink::FstatisticAnalysisDomSexLink(aString& s){
	FstatisticAnalysisCore::Initialize(); 
	SexLinkedDomMarkerData2::Initialize();
	EvolutionaryGeneticsData::Initialize(s); 
	}


FstatisticAnalysisDomSexLink::FstatisticAnalysisDomSexLink(EvolutionaryGeneticsData& egd){
	SexLinkedDomMarkerData2::Initialize();
	EvolutionaryGeneticsData::Initialize(egd);//do this second to overwrite blank initialization
	FstatisticAnalysisCore::Initialize();
	}//FstatisticAnalysisDomSexLink::FstatisticAnalysisDomSexLink(EvolutionaryGeneticsData&)



FstatisticAnalysisDomSexLink::FstatisticAnalysisDomSexLink(SexLinkedDomMarkerData2& dmd){
	SexLinkedDomMarkerData2::Initialize(dmd);
	FstatisticAnalysisCore::Initialize();
	}//FstatisticAnalysisDomSexLink::FstatisticAnalysisDomSexLink(SexLinkedDomMarkerData2&)




FstatisticAnalysisDomSexLink::FstatisticAnalysisDomSexLink(FstatisticAnalysisDomSexLink& dmd){//copy constructor
	SexLinkedDomMarkerData2::Initialize(dmd);
	FstatisticAnalysisCore::Initialize(dmd);
	}//FstatisticAnalysisDomSexLink::FstatisticAnalysisDomSexLink(FstatisticAnalysisDomSexLink&)





FstatisticAnalysisDomSexLink::FstatisticAnalysisDomSexLink(FstatisticAnalysisDomSexLink& parentData, aList<int>& popSubsetNums) {
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	SexLinkedDomMarkerData2::Initialize(parentData, popSubsetNums);//copies
	}//end ctor FstatisticAnalysisDomSexLink(FstatisticAnalysisDomSexLink&, aList<int>&)


FstatisticAnalysisDomSexLink::FstatisticAnalysisDomSexLink(SexLinkedDomMarkerData2& parentData, aList<int>& popSubsetNums) {
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	SexLinkedDomMarkerData2::Initialize(parentData, popSubsetNums);//copies
	}//end ctor FstatisticAnalysisDomSexLink(SexLinkedDomMarkerData2&, aList<int>&)





void FstatisticAnalysisDomSexLink::Clear(){
	FstatisticAnalysisCore::Clear();
	SexLinkedDomMarkerData2::Clear();
	}


void FstatisticAnalysisDomSexLink::Initialize(FstatisticAnalysisDomSexLink& dmd){//copy constructor
	SexLinkedDomMarkerData2::Initialize(dmd);
	FstatisticAnalysisCore::Initialize(dmd);
	}//FstatisticAnalysisDomSexLink::FstatisticAnalysisDomSexLink(FstatisticAnalysisDomSexLink&)





void FstatisticAnalysisDomSexLink::DomMarkerFstats (int numTrueDemes, bool zygoteMigration){
	//calculates F-statistics among populations using dominant markers,
	//to be used when there is only one group of pops.
	//Notation follows Porter MS in prep

	//clear out any previous calculations if necessary
  	if(! FstatsCurrent_){
  		ClearFstats();}
	if((Fmethod_.Includes((int) sexLinkDomStd))){return;}
	if(nlociSLDom_==0){return;}
	FstatsCurrent_=vcCurrent_=0;
	Fmethod_.Append((int) sexLinkDomStd);
	int m = Fmethod_.len()-1;
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, F-level
	Fstats_.Append(NEW aList<long double>());//F-level
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	Flevels_.Append(NEW StringList());//F-level

	vcMethod_.Append((int) sexLinkDomStd);
	vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
	vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
	vc_.Append(NEW aList<long double>());//vc-level
	vcLevels_.Append(NEW StringList());//vc-level

	Fmethod_.Append((int) sexLinkDomStdEffective);
	int mEff = Fmethod_.len()-1;
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, F-level
	Fstats_.Append(NEW aList<long double>());//F-level
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	Flevels_.Append(NEW StringList());//F-level

	vcMethod_.Append((int) sexLinkDomStdEffective);
	vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
	vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
	vc_.Append(NEW aList<long double>());//vc-level
	vcLevels_.Append(NEW StringList());//vc-level

	//set up all the F-stat arrays
	// first the array describing the Fstat structure:
	aString* FWord=NEW aString("Fst");
	Flevels_[m].Append(FWord);
	Flevels_[mEff].Append(NEW aString(*FWord));
	FWord=NEW aString("Fis");
	Flevels_[m].Append(FWord);
	Flevels_[mEff].Append(NEW aString(*FWord));
	FWord=NEW aString("Fit");
	Flevels_[m].Append(FWord);
	Flevels_[mEff].Append(NEW aString(*FWord));
	int flevelsToUse=3;

	aString* vcWord=NEW aString("var(pop,bBar)");
	vcLevels_[m].Append(vcWord);
	vcLevels_[mEff].Append(NEW aString(*vcWord));
	vcWord=NEW aString("var(i)");
	vcLevels_[m].Append(vcWord);
	vcLevels_[mEff].Append(NEW aString(*vcWord));
	vcWord=NEW aString("var(t)");
	vcLevels_[m].Append(vcWord);
	vcLevels_[mEff].Append(NEW aString(*vcWord));
	vcWord=NEW aString("var(i,bBar)");
	vcLevels_[m].Append(vcWord);
	vcLevels_[mEff].Append(NEW aString(*vcWord));
	vcWord=NEW aString("var(i,heterogametic)");
	vcLevels_[m].Append(vcWord);
	vcLevels_[mEff].Append(NEW aString(*vcWord));
	vcWord=NEW aString("diff(bm,bf)");
	vcLevels_[m].Append(vcWord);
	vcLevels_[mEff].Append(NEW aString(*vcWord));
	vcWord=NEW aString("var(i,weighted)");
	vcLevels_[m].Append(vcWord);
	vcLevels_[mEff].Append(NEW aString(*vcWord));
	vcWord=NEW aString("F(i,weighted)");//a variance ratio not a component, used to average Fis over inheritance classes
	vcLevels_[m].Append(vcWord);
	vcLevels_[mEff].Append(NEW aString(*vcWord));
	vcWord=NEW aString("weightFactor");
	vcLevels_[m].Append(vcWord);
	vcLevels_[mEff].Append(NEW aString(*vcWord));
	int vcLevelsToUse = vcLevels_[m].len();

	//make and initialize the jackknife & by-locus Fstat lists
	int Flevel, vclevel, j, pop, a;
	for (Flevel=0;Flevel<flevelsToUse;++Flevel){
		Fstats_[m].Append(zero);
		Fstats_[mEff].Append(zero);
		}
	for (vclevel=0;vclevel<vcLevelsToUse;++vclevel){
		vc_[m].Append(zero);
		vc_[mEff].Append(zero);
		}
	//some intermediate variables
	aList<long double> sumILoc, sumSLoc, sumISLoc, sumIbetaBar, sumMSIHetero, sumSSIHetero, sumBetaBarDiff;

	SexLinkedDomMarkerData2::CheckForNewAlleles();
	SexLinkedDomMarkerData2::CalculatePhenotypeFreqs();
	SexLinkedDomMarkerData2::UpdateVariableLociDom();
	//run a check for sufficient variability & report if failure
	ObjList<aList<int> > popsToUse, failedSets;//index [loc][pop]
	aList<int> lociToUse, lociToDrop;//index [loc][pop]
	bool dropSome=false, dropLoci=false;
	for(j=0;j<variableLocusNumSLDom_.len();++j){
		failedSets.Append(NEW aList<int>());
		popsToUse.Append(NEW aList<int>());
		for(pop=0;pop<npop_;++pop){
			int nhets = SexLinkedDomMarkerData2::GetLocusSampleSizeDomHeterogametic(pop,j);
			int nhoms = SexLinkedDomMarkerData2::GetLocusSampleSizeDomHomogametic(pop,j);
			if(nhets>0 && nhoms>0){
					popsToUse[j].Append(pop);}
				else{
					failedSets[j].Append(pop);
					dropSome=true;}
			}//pop
		if(popsToUse[j].len()<3){//df = r-2
				dropLoci=true;
				for(pop=0;pop<popsToUse[j].len();++pop){
					failedSets[j].Append(popsToUse[j][pop]);}
				popsToUse[j].Clear();
				failedSets[j].Sort();
				lociToDrop.Append(variableLocusNumSLDom_[j]);}
			else{
				lociToUse.Append(variableLocusNumSLDom_[j]);}
		}//j
	//Report result of error check
	if(lociToUse.len()==0){
		aError("ERROR:  Can\'t calculate Fstatistics when both males and females aren\'t sampled from at least 3 populations.");
		FstatsCurrent_=vcCurrent_=true;
		return;}
	//drop pops that are not sampled in both males and females
	if(dropSome){
		aString errStr="Some loci were not sampled for both sexes in all populations.  These combinations were dropped from the analysis.";
		errStr+"They are [locus: pops]:";
		aString locspacing=' ', popspacing="";
		for(j=0;j<failedSets.len();++j){
			if(failedSets[j].len()>0){
				errStr+locspacing+(variableLocusNumSLDom_[j]+1)+":";
				locspacing="; ";
				popspacing="";
				for(pop=0;pop<failedSets[j].len();++pop){
					errStr+popspacing+(pop+1);
					popspacing=",";}
				}}
		cout<<errStr<<endl;
		
		}
	
	
	StatList<long double> BetaBar, BetaBarHet, BetaBarHomo,sumBetaBarDiffGlobal,sumSSIHeteroGlobal;//global freqs for each marker
	for (j=0;j<nlociSLDom_;++j){//initialize for all loci, even loci dropped for poor sampling
//	for (int j=0;j<variableLocusNumSLDom_.len();++j){//initialize for all variable loci, even loci dropped for poor sampling
//		int loc=variableLocusNumSLDom_[j];
		//make and initialize the jackknife & by-locus Fstat lists for this locus 
		FstatsLoc_[m].Append(NEW aList<long double>());
		jackFstatsLoc_[m].Append(NEW aList<long double>());
		FstatsLoc_[mEff].Append(NEW aList<long double>());
		jackFstatsLoc_[mEff].Append(NEW aList<long double>());
		for (Flevel=0;Flevel<flevelsToUse;++Flevel){
			FstatsLoc_[m][j].Append(zero);
			jackFstatsLoc_[m][j].Append(zero);
			FstatsLoc_[mEff][j].Append(zero);
			jackFstatsLoc_[mEff][j].Append(zero);
			}
		//make and initialize the jackknife & by-allele Fstat lists for this locus 
		FstatsAll_[m].Append(NEW ObjList<aList<long double> >());
		FstatsAll_[mEff].Append(NEW ObjList<aList<long double> >());
		//initialize variables
		sumILoc.Append(zero);
		sumSLoc.Append(zero);
		sumISLoc.Append(zero);
		sumIbetaBar.Append(zero);
		sumMSIHetero.Append(zero);
		sumSSIHetero.Append(zero);
		sumBetaBarDiff.Append(zero);

		//make and initialize the by-allele F-stat list for both alleles and append to this locus list
		StringList& locAlleleList = allelesPresentSLDom_[j];
		for (a=0;a<locAlleleList.len();++a){
			FstatsAll_[m][j].Append(NEW aList<long double>());
			FstatsAll_[mEff][j].Append(NEW aList<long double>());
			for (Flevel=0;Flevel<flevelsToUse;++Flevel){
				FstatsAll_[m][j][a].Append(zero);
				FstatsAll_[mEff][j][a].Append(zero);
				}
			}//end for a							

		//make and initialize the by-allele var comp list
		vcLoc_[m].Append(NEW aList<long double>());
		vcLoc_[mEff].Append(NEW aList<long double>());
		for (vclevel=0;vclevel<vcLevelsToUse;++vclevel){
			vcLoc_[m][j].Append(zero);
			vcLoc_[mEff][j].Append(zero);
			}
		vcAll_[m].Append(NEW ObjList<aList<long double> >());
		vcAll_[mEff].Append(NEW ObjList<aList<long double> >());
		for (a=0;a<locAlleleList.len();++a){
			vcAll_[m][j].Append(NEW aList<long double>());
			vcAll_[mEff][j].Append(NEW aList<long double>());
			for (vclevel=0;vclevel<vcLevelsToUse;++vclevel){
				vcAll_[m][j][a].Append(zero);
				vcAll_[mEff][j][a].Append(zero);
				}
			}//end for a							
		BetaBar.Append(zero);
		BetaBarHomo.Append(zero);
		BetaBarHet.Append(zero);
		sumSSIHeteroGlobal.Append(zero);
		sumBetaBarDiffGlobal.Append(zero);
		}//end for j

	//the main body of the calculations	
	long double Isum,Ssum,ISsum,IbetaBarsum,IHeterosum,BetaBarDiffsum,freqHomo,freqHet,sr;
	Isum=Ssum=ISsum=IbetaBarsum=IHeterosum=BetaBarDiffsum=freqHomo=freqHet=zero;
	sr=SexLinkedDomMarkerData2::primarySexRatio();//female first; assume female is heterogametic
//	if(heterogameticSex()=='M'){sr = one - sr;}//if male heterogametic, use complement
	if(heterogameticSex()=='F'){sr = one - sr;}//if male heterogametic, use complement
	int jj,loc,pp;
	long double Fval,nDom,nhets,nhomos,varBetaM,r,dfPop,NeBySexGlobal,FisSum, FitSum,fstVal,NmfromFst,FstEffFromNm;
	long double n,nH,n0,nbar,nsq,dfI,ssqI,ssqPop,ssqIbetaBar,ssqIHetero,ssqIHetero2,ssqIHetero3,diffIBeta,diffIBeta3,NeBySex,
				MSI,MSIHetero,MSPop,vcI,vcPop,ssqIHetero2Global,ssqIHetero3Global,betaBarDiffGlobal,betaBarDiff3Global,
				weightedMeanFis,scaleFactor, weightedFisNum,FisDenom;
	long double NeBySexPopScale;
	if(heterogameticSex_=='F'){
			NeBySexPopScale = two*sr + (one-sr);}
		else{
			NeBySexPopScale = sr + two*(one-sr);}
	aString levelName;
	for (jj=0;jj<lociToUse.len();++jj){
		loc = lociToUse[jj];//the actual locus, used to look up information in the raw data
		j = variableLocusNumSLDom_.Loc(loc);//j used to store and look up values in this class' F-stat arrays
		r = (long double) (popsToUse[jj].len());
		dfPop = r - two;//2 (sex-specific) means calculated
		n=nH=n0=nbar=nsq=dfI=ssqI=ssqPop=ssqIbetaBar=ssqIHetero=ssqIHetero3=diffIBeta=diffIBeta3=NeBySex=betaBarDiff3Global=weightedMeanFis=
				MSI=MSIHetero=ssqIHetero=ssqIHetero2=MSPop=vcI=vcPop=ssqIHetero2Global=ssqIHetero3Global=betaBarDiffGlobal=
				scaleFactor=weightedFisNum=FisDenom=zero;
		StatList<long double> beta, betaBar, betaHet, betaHomo,FisByPop;//marker frequencies for this locus at each pop
		StatList<long double> FisByPopNumLoc, FisByPopDenomLoc;//for handling weighted 'variance components'
		StatList<long double> nn, nnHets, NeBySexPop, scaleFactorLoc;//sample sizes for this locus at each pop
		
		//Calculate for the whole population set:
		for (pp=0;pp<popsToUse[jj].len();++pp){
			pop=popsToUse[jj][pp];
			beta.Append(SexLinkedDomMarkerData2::GetFreqDom(pop,loc, 1));
			freqHomo=SexLinkedDomMarkerData2::GetFreqDomHomogametic(pop,loc, 1);
			freqHet=SexLinkedDomMarkerData2::GetFreqDomHeterogametic(pop,loc, 1);
			betaBar.Append(sr*freqHomo + (one-sr)*freqHet);//parametrically weighted mean
			betaHet.Append(freqHet);
			betaHomo.Append(freqHomo);
			nDom = (long double) SexLinkedDomMarkerData2::GetLocusSampleSizeDom(pop,loc);
			nhets = (long double) SexLinkedDomMarkerData2::GetLocusSampleSizeDomHeterogametic(pop,loc);
			nhomos = (long double) SexLinkedDomMarkerData2::GetLocusSampleSizeDomHomogametic(pop,loc);
			nn.Append(nDom);
			nnHets.Append(nhets);
//			NeBySexPop.Append(four *  nhets * nhomos / nDom);//may be wrong for haplodiploids = effective sample size
			NeBySexPop.Append(NeBySexPopScale *  nhets * nhomos / nDom);//may be wrong for haplodiploids = effective sample size
			diffIBeta += (freqHet-freqHomo);//diff between sexes
			diffIBeta3 += NeBySexPop[pp]*(freqHet-freqHomo);//weighted diff between sexes
//			diffIBeta += nn[pp]*(freqHet-freqHomo);//diff between sexes
			//average sample size among pops:
			nsq += nn[pp] * nn[pp];//running total
			// Average marker frequency (=BetaBar)
			BetaBar[jj] += nn[pp] * betaBar[pp];
			BetaBarHomo[jj] += (nn[pp]-nnHets[pp]) * betaHomo[pp];
			BetaBarHet[jj] += nnHets[pp] * betaHet[pp];
			varBetaM = freqHet*(one-freqHet);
			if(freqHet==zero || freqHet==one){
					FisByPop.Append(zero);
					FisByPopNumLoc.Append(zero);
					FisByPopDenomLoc.Append(zero);
					scaleFactorLoc.Append(zero);//undefined - so weight as zero
					}
				else{
					scaleFactorLoc.Append(one - freqHet*freqHet);
					FisByPop.Append(scaleFactorLoc[pp] * (varBetaM+freqHet-freqHomo)/varBetaM);
					FisByPopNumLoc.Append(scaleFactorLoc[pp]*(varBetaM+freqHet-freqHomo));
					FisByPopDenomLoc.Append(varBetaM);
					}
			}//pop
		n = nn.sum();//total sample size
		nH = nnHets.sum();//hetergametic sample size
		NeBySexGlobal = four * nH * (n-nH) / n;//effective sample size
		if ((r>one)&&(n>two*r) && nH>zero && n>nH) {//this weeds out zero denominators
			nbar = n/r;
			n0 = (n - nsq/n) / dfPop;
			BetaBar[jj] /=n;
			BetaBarHomo[jj] /=(n-nH);
			BetaBarHet[jj] /=nH;

			//Now get the terms accounting for adjusted variance among marker frequencies
			for (pp=0;pp<popsToUse[jj].len();++pp){
				ssqI += nn[pp] * beta[pp]*(one - beta[pp]);
				ssqIHetero += nnHets[pp] * betaHet[pp]*(one - betaHet[pp]);
				ssqIHetero3 += NeBySexPop[pp]*betaHet[pp]*(one - betaHet[pp]);
				ssqIHetero2 += betaHet[pp]*(one - betaHet[pp]);
				ssqIbetaBar += nn[pp] * (beta[pp] - betaBar[pp]) * (beta[pp] - betaBar[pp]);
				ssqPop += nn[pp] * (beta[pp] - BetaBar[jj]) * (beta[pp] - BetaBar[jj]);
				}//end for pp
			dfI = n - two*r;
			
			MSI = (ssqI + ssqIbetaBar)/dfI;
			MSIHetero = ssqIHetero/((long double)nnHets.sum()-r);
			MSPop = ssqPop/dfPop;
			
			ssqIHetero2Global = BetaBarHet[jj]*(one - BetaBarHet[jj]);
			ssqIHetero3Global = NeBySexGlobal*BetaBarHet[jj]*(one - BetaBarHet[jj]);
			betaBarDiff3Global = NeBySexGlobal*(BetaBarHet[jj] - BetaBarHomo[jj]);
			betaBarDiffGlobal = BetaBarHet[jj] - BetaBarHomo[jj];
			//variance components
			vcI = MSI;
			if (n0 != zero){
				vcPop = (MSPop - MSI)/n0;
				}
			}//end if r>1, n>0
		weightedFisNum += FisByPopNumLoc.sum();
		scaleFactor += scaleFactorLoc.sum();
		for(a=0;a<allelesPresentSLDom_[j].len();++a){//same for both marker phenotypes
			vcAll_[m][j][a][0] = vcPop;
			vcAll_[m][j][a][1] = vcI;
			vcAll_[m][j][a][2] = vcPop+vcI;
			vcAll_[m][j][a][3] = ssqIbetaBar/(n-two*r);
			vcAll_[m][j][a][4] = ssqIHetero;
			vcAll_[m][j][a][5] = diffIBeta;
			vcAll_[m][j][a][6] = FisByPopNumLoc.sum();
			vcAll_[m][j][a][7] = FisByPop.sum();
			vcAll_[m][j][a][8] = scaleFactorLoc.sum();

			vcAll_[mEff][j][a][0] = vcPop;
			vcAll_[mEff][j][a][1] = vcI;
			vcAll_[mEff][j][a][2] = vcPop+vcI;
			vcAll_[mEff][j][a][3] = ssqIbetaBar/(n-two*r);
			vcAll_[mEff][j][a][4] = ssqIHetero;
			vcAll_[mEff][j][a][5] = diffIBeta;
			vcAll_[mEff][j][a][6] = FisByPopNumLoc.sum();
			vcAll_[mEff][j][a][7] = FisByPop.sum();
			vcAll_[mEff][j][a][8] = scaleFactorLoc.sum();
			//variance components remain zero if their components have any zero denominators
			if ((vcPop+vcI) != zero) {
				levelName="Fst";
				Flevel = Flevels_[m].Loc(&levelName);
				fstVal = FstatsAll_[m][j][a][Flevel] = Fval = vcPop / (vcPop+vcI);
//				FstatsAll_[mEff][j][a][Flevel] = Fval = vcPop / (two*(vcPop+vcI)-vcPop);
				NmfromFst = NmFromSexLinkedDomFst(fstVal,numTrueDemes,zygoteMigration);
				FstEffFromNm = EffectiveFstFromNm(NmfromFst,numTrueDemes,zygoteMigration);
				FstatsAll_[mEff][loc][a][Flevel] = FstEffFromNm;
				}
			if (ssqIHetero2 != zero) {
				levelName = "Fis";
				Flevel = Flevels_[m].Loc(&levelName);
				FstatsAll_[m][j][a][Flevel] = Fval = FisByPop.sum()/scaleFactorLoc.sum();
				FstatsAll_[mEff][j][a][Flevel] = Fval = FisByPop.sum()/scaleFactorLoc.sum();
				}
			if (ssqIHetero2Global != zero) {
				levelName = "Fit";
				Flevel = Flevels_[m].Loc(&levelName);
				FstatsAll_[m][j][a][Flevel] = Fval = (ssqIHetero2Global + betaBarDiffGlobal) / ssqIHetero2Global;
				FstatsAll_[mEff][j][a][Flevel] = Fval = (ssqIHetero2Global + betaBarDiffGlobal) / ssqIHetero2Global;
				}
			}//a
		//save these in an array for a quicker calculation of the jackknife variance estimates
		sumILoc[j] = vcI;
		sumSLoc[j] = vcPop;
		sumISLoc[j] = vcI+vcPop;
		sumIbetaBar[j] = ssqIbetaBar/(n-two*r);
		sumMSIHetero[j] = MSIHetero;
		sumSSIHetero[j] = ssqIHetero2;
		sumBetaBarDiff[j] = diffIBeta;
		sumSSIHeteroGlobal[j] = ssqIHetero2Global;
		sumBetaBarDiffGlobal[j] = betaBarDiffGlobal;

		Isum += vcI;
		Ssum += vcPop;
		ISsum += vcI+vcPop;
		IbetaBarsum += ssqIbetaBar/(n-two*r);
		IHeterosum += ssqIHetero2;
		BetaBarDiffsum += diffIBeta;
			
		vcLoc_[m][j][0] = sumSLoc[j];
		vcLoc_[m][j][1] = sumILoc[j];
		vcLoc_[m][j][2] = sumISLoc[j];
		vcLoc_[m][j][3] = sumIbetaBar[j];
		vcLoc_[m][j][4] = sumSSIHetero[j];
		vcLoc_[m][j][5] = sumBetaBarDiff[j];
		vcLoc_[m][j][6] = FisByPopNumLoc.sum();
		vcLoc_[m][j][7] = FisByPop.sum();
		vcLoc_[m][j][8] = scaleFactorLoc.sum();

		vcLoc_[mEff][j][0] = sumSLoc[j];
		vcLoc_[mEff][j][1] = sumILoc[j];
		vcLoc_[mEff][j][2] = sumISLoc[j];
		vcLoc_[mEff][j][3] = sumIbetaBar[j];
		vcLoc_[mEff][j][4] = sumSSIHetero[j];
		vcLoc_[mEff][j][5] = sumBetaBarDiff[j];
		vcLoc_[mEff][j][6] = FisByPopNumLoc.sum();
		vcLoc_[mEff][j][7] = FisByPop.sum();
		vcLoc_[mEff][j][8] = scaleFactorLoc.sum();
		
		aList<long double>& locFstat = FstatsLoc_[m][j];
		if (sumISLoc[j] != zero) {
			levelName="Fst";
			Flevel = Flevels_[m].Loc(&levelName);
			fstVal = FstatsLoc_[m][j][Flevel] = Fval=sumSLoc[j] / sumISLoc[j];
//			FstatsLoc_[mEff][j][Flevel] = Fval=sumSLoc[j] / (two*sumISLoc[j] - sumSLoc[j]);
			NmfromFst = NmFromSexLinkedDomFst(fstVal,numTrueDemes,zygoteMigration);
			FstEffFromNm = EffectiveFstFromNm(NmfromFst,numTrueDemes,zygoteMigration);
			FstatsLoc_[mEff][loc][Flevel] = FstEffFromNm;
			}
		if (sumSSIHetero[j] != zero) {
			levelName="Fis";
			Flevel = Flevels_[m].Loc(&levelName);
			FstatsLoc_[m][j][Flevel] = Fval= FisByPop.sum()/scaleFactorLoc.sum();
			FstatsLoc_[mEff][j][Flevel] = Fval= FisByPop.sum()/scaleFactorLoc.sum();
			}
		if (sumSSIHeteroGlobal[j] != zero) {
			levelName="Fit";
			Flevel = Flevels_[m].Loc(&levelName);
			FstatsLoc_[m][j][Flevel] = Fval= (sumSSIHeteroGlobal[j]+sumBetaBarDiffGlobal[j]) / sumSSIHeteroGlobal[j];
			FstatsLoc_[mEff][j][Flevel] = Fval= (sumSSIHeteroGlobal[j]+sumBetaBarDiffGlobal[j]) / sumSSIHeteroGlobal[j];
			}
		}//end for j

	if (ISsum != zero){
		levelName="Fst";
		Flevel = Flevels_[m].Loc(&levelName);
		fstVal = Fstats_[m][Flevel] = Fval = Ssum/ISsum;
//		Fstats_[mEff][Flevel] = Fval = Ssum/(two*ISsum-Ssum);
		NmfromFst = NmFromSexLinkedDomFst(fstVal,numTrueDemes,zygoteMigration);
		FstEffFromNm = EffectiveFstFromNm(NmfromFst,numTrueDemes,zygoteMigration);
		Fstats_[mEff][Flevel] = FstEffFromNm;
		}//end if ISsum
	FisSum = FitSum = zero; 
	if (FstatsLoc_[m].len() != 0){
		levelName="Fis";
		Flevel = Flevels_[m].Loc(&levelName);
		for(j=0;j<FstatsLoc_[m].len();++j){
			FisSum += FstatsLoc_[m][j][Flevel];}
		Fstats_[m][Flevel] = Fval = FisSum/(long double)FstatsLoc_[m].len();
		Fstats_[mEff][Flevel] = Fval = FisSum/(long double)FstatsLoc_[m].len();

		levelName="Fit";
		Flevel = Flevels_[m].Loc(&levelName);
		for(j=0;j<FstatsLoc_[m].len();++j){
			FitSum += FstatsLoc_[m][j][Flevel];}
		Fstats_[m][Flevel] = Fval = FitSum/(long double)FstatsLoc_[m].len();
		Fstats_[mEff][Flevel] = Fval = FitSum/(long double)FstatsLoc_[m].len();
		}//end if FstatsLoc_[m].len()

	vc_[m][0] = vc_[mEff][0] = Ssum;
	vc_[m][1] = vc_[mEff][1] = Isum;
	vc_[m][2] = vc_[mEff][2] = ISsum;
	vc_[m][3] = vc_[mEff][3] = IbetaBarsum;
	vc_[m][4] = vc_[mEff][4] = IHeterosum;
	vc_[m][5] = vc_[mEff][5] = BetaBarDiffsum;
	vc_[m][6] = vc_[mEff][6] = weightedFisNum;
	vc_[m][7] = vc_[mEff][6] = FisSum;
	vc_[m][8] = vc_[mEff][7] = scaleFactor;

	//now do the jackknife calculations, over loci
	for (jj=0;jj<lociToUse.len();++jj){
		loc = lociToUse[jj];//the actual locus
		j = variableLocusNumSLDom_.Loc(loc);	
		if (ISsum != sumISLoc[j]) {
			levelName="Fst";
			Flevel = Flevels_[m].Loc(&levelName);
			fstVal = jackFstatsLoc_[m][loc][Flevel] = Fval = (Ssum-sumSLoc[j]) / (ISsum - sumISLoc[j]);
//			jackFstatsLoc_[mEff][loc][Flevel] = Fval = (Ssum-sumSLoc[j]) / (two*(ISsum - sumISLoc[j])-sumSLoc[j]);
			NmfromFst = NmFromSexLinkedDomFst(fstVal,numTrueDemes,zygoteMigration);
			FstEffFromNm = EffectiveFstFromNm(NmfromFst,numTrueDemes,zygoteMigration);
			jackFstatsLoc_[mEff][loc][Flevel] = FstEffFromNm;
			}//end if 
		if (FstatsLoc_[m].len()>1) {
			levelName="Fis";
			Flevel = Flevels_[m].Loc(&levelName);
			jackFstatsLoc_[m][loc][Flevel] = Fval = (FstatsLoc_[m].len()*Fstats_[m][Flevel] -FstatsLoc_[m][loc][Flevel])/(long double)(FstatsLoc_[m].len()-1) ;
			jackFstatsLoc_[mEff][loc][Flevel] = Fval = (FstatsLoc_[mEff].len()*Fstats_[mEff][Flevel] -FstatsLoc_[mEff][loc][Flevel])/(long double)(FstatsLoc_[mEff].len()-1) ;
			levelName="Fit";
			Flevel = Flevels_[m].Loc(&levelName);
			jackFstatsLoc_[m][loc][Flevel] = Fval = (FstatsLoc_[m].len()*Fstats_[m][Flevel] -FstatsLoc_[m][loc][Flevel])/(long double)(FstatsLoc_[m].len()-1) ;
			jackFstatsLoc_[mEff][loc][Flevel] = Fval = (FstatsLoc_[mEff].len()*Fstats_[mEff][Flevel] -FstatsLoc_[mEff][loc][Flevel])/(long double)(FstatsLoc_[mEff].len()-1) ;
			}//end if 
		}//end for j

	FstatsCurrent_=vcCurrent_=1;
	}//end FstatisticAnalysisDomSexLink::DomMarkerFstats






void FstatisticAnalysisDomSexLink::HierarchicalDomMarkerFstats(void){}


long double FstatisticAnalysisDomSexLink::Nm(long double FstVal,int Fmethod, int numTrueDemes, bool zygoteMigration){
	long double Nmval=zero;
	if(Fmethod==sexLinkDomStdEffective){
			if(FstVal<=zero) return (long double)1.0e6;
			if(FstVal>=one) return zero;
			long double scale=two;
			if(zygoteMigration){
				scale=four;}
			if(numTrueDemes<2){numTrueDemes=0;}
				else{numTrueDemes=MAX((int)npop_,numTrueDemes);}
			if(numTrueDemes>1){
				long double d=(long double)numTrueDemes;
				scale *= (d-one)*(d-one)/(d*d);}
			Nmval = (one - FstVal)/(scale*FstVal);
			return Nmval;
			}
		else{
			Nmval = NmFromSexLinkedDomFst(FstVal,numTrueDemes,zygoteMigration);}
	return Nmval;
	}//FstatisticAnalysisDomSexLink::Nm(long double,int, bool, bool)


long double FstatisticAnalysisDomSexLink::NmFromSexLinkedDomFst(long double FstSLDom, int numTrueDemes, bool zygoteMigration){
	if(FstSLDom<=zero) return (long double)1.0e6;
	if(FstSLDom>=one) return zero;
	long double Nmval=zero,scale=two, sr = primarySexRatio_;
	scale *= one + sr;
	if(zygoteMigration){
			scale*=two;}
	if(numTrueDemes<2){numTrueDemes=0;}
		else{numTrueDemes=MAX((int)npop_,numTrueDemes);}
	if(numTrueDemes>1){
		long double d = (long double)numTrueDemes;
		scale *= (d-one)*(d-one)/(d*d);}
	Nmval = (one - FstSLDom)/(scale*FstSLDom);
	return Nmval;
	}//FstatisticAnalysisDomSexLink::NmFromSexLinkedDomFst(long double )



int  FstatisticAnalysisDomSexLink::PrintFMethodHeading(ostream& outFile, const int Fmethod){
	if(Fmethod==sexLinkDomStd){
			outFile<<"Least-squares estimators of Wright's F-statistics for sex-linked codominant markers."<<endl;}
		else if(Fmethod ==sexLinkHierDomStd){
			outFile<<"Least-squares estimators of Wright's hierarchical F-statistics for sex-linked codominant markers."<<endl;}
		else if(Fmethod ==sexLinkDomStdEffective){
			outFile<<"Least-squares estimators of Wright's F-statistics for sex-linked codominant markers, presented as effective values for diploid codominant markers."<<endl;}
		else {
			return FstatisticAnalysisCore::PrintFMethodHeading(outFile, Fmethod);}
	return 1;//no error
	}//end FstatisticAnalysisDomSexLink::PrintFMethodHeading


void FstatisticAnalysisDomSexLink::PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer){
	StringList namesOfLoci;
	SexLinkedDomMarkerData2::GetLocusNames(namesOfLoci);
	FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod, titleStr, namesOfLoci, colSpacer);
	}
	
	
void FstatisticAnalysisDomSexLink::PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer){
	StringList namesOfLoci;
	SexLinkedDomMarkerData2::GetLocusNames(namesOfLoci);
	FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod, titleStr, namesOfLoci, allelesPresentSLDom_, colSpacer);
	}







void FstatisticAnalysisDomSexLink::PrintNEXUSFile(aString& outFileName, StringList& textToAppend){
	SexLinkedDomMarkerData2::PrintNEXUSFile(outFileName, textToAppend);
	}//end FstatisticAnalysisDomSexLink::PrintNEXUSFile(aString&, StringList&)



void FstatisticAnalysisDomSexLink::PrintNEXUSFile (ostream& outFile, StringList& textToAppend){
	SexLinkedDomMarkerData2::PrintNEXUSFile(outFile, textToAppend);
	}//end FstatisticAnalysisDomSexLink::PrintNEXUSFile(ostream&,StringList&)




void FstatisticAnalysisDomSexLink::ResetFlags(void){
	FstatisticAnalysisCore::ResetFlags();
	SexLinkedDomMarkerData2::ResetFlags();
	}


long double FstatisticAnalysisDomSexLink::SexLinkedDomFstFromNm(long double Nm, int numTrueDemes, bool zygoteMigration){
	long double scale = two, sr = primarySexRatio_;
	scale *= (one + sr);
	if(zygoteMigration){
		scale=three;}
	if(numTrueDemes<2){numTrueDemes=0;}
		else{numTrueDemes=MAX((int)npop_,numTrueDemes);}
	if(numTrueDemes>1){
		long double d=(long double)numTrueDemes;
		scale *= d*d/((d-one)*(d-one));}
	if(zygoteMigration){
//		scale *= (one+sr)/(two-r);
		scale /= (two-sr);//factor (1+sr) is accounted for above
		}
	return one/(scale*Nm+one);
	}//FstatisticAnalysisDomSexLink::SexLinkedDomFstFromNm(long double,bool,bool)




FstatisticAnalysisDomSexLink& FstatisticAnalysisDomSexLink::operator=(EvolutionaryGeneticsData& egd){
	Clear();
	EvolutionaryGeneticsData::Initialize(egd);
	return *this;
	}//FstatisticAnalysisDomSexLink::operator=(EvolutionaryGeneticsData&)


FstatisticAnalysisDomSexLink& FstatisticAnalysisDomSexLink::operator=(SexLinkedDomMarkerData2& dmd){
	Clear();
	SexLinkedDomMarkerData2::Initialize(dmd);
	return *this;
	}//FstatisticAnalysisDomSexLink::operator=(SexLinkedDomMarkerData2&)



FstatisticAnalysisDomSexLink& FstatisticAnalysisDomSexLink::operator=(FstatisticAnalysisDomSexLink& dmd){
	Clear();
	SexLinkedDomMarkerData2::Initialize(dmd);
	FstatisticAnalysisCore::Initialize(dmd);
	return *this;
	}//FstatisticAnalysisDomSexLink::operator=(FstatisticAnalysisDomSexLink&)


#endif
