// FstatisticAnalysisMultiFormat.cpp

//Fstatistics calculated simultaneously for all data types

#ifndef FstatisticAnalysisMultiFormatcpp
#define FstatisticAnalysisMultiFormatcpp = 1


#include "FstatisticAnalysisMultiFormat.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"



FstatisticAnalysisMultiFormat::FstatisticAnalysisMultiFormat(FstatisticAnalysisMultiFormat& parentData, aList<int>& popSubsetNums);
			//makes a copy of the original data using a subset of populations
			//useful for separate analyses of data subsets



	//Initialize() functions for handling construction of shared virtual base classes

void FstatisticAnalysisMultiFormat::Initialize(MultiformatMarkerData& pgd){
	ClearFstats();
	MultiformatMarkerData::Initialize(pgd);
	HaploidMarkerData::Initialize(pgd);
	CodominantMarkerData::Initialize(pgd);
	DominantMarkerData2::Initialize(pgd);
	SexLinkedCodomMarkerData::Initialize(pgd);
	SexLinkedDomMarkerData2::Initialize(pgd);
	CytoplasmicFemaleMarkerData::Initialize(pgd);
	CytoplasmicMaleMarkerData::Initialize(pgd);
	SexLinkedData::Initialize(pgd);
	EvolutionaryGeneticsData::Initialize(pgd);		
	}//FstatisticAnalysisMultiFormat::Initialize(FstatisticAnalysisMultiFormat&)


void FstatisticAnalysisMultiFormat::Initialize(FstatisticAnalysisMultiFormat& pgd){
	FstatisticAnalysisHap::Initialize(pgd);
	FstatisticAnalysisCodom::Initialize(pgd);
	FstatisticAnalysisDom2::Initialize(pgd);
	FstatisticAnalysisCodomSexLink::Initialize(pgd);
	FstatisticAnalysisDomSexLink::Initialize(pgd);
	FstatisticAnalysisCytoFemale::Initialize(pgd);
	FstatisticAnalysisCytoMale::Initialize(pgd);
	FstatisticAnalysisCore::Initialize(pgd);
	MultiformatMarkerData::Initialize(pgd);
	HaploidMarkerData::Initialize(pgd);
	CodominantMarkerData::Initialize(pgd);
	DominantMarkerData2::Initialize(pgd);
	SexLinkedCodomMarkerData::Initialize(pgd);
	SexLinkedDomMarkerData2::Initialize(pgd);
	CytoplasmicFemaleMarkerData::Initialize(pgd);
	CytoplasmicMaleMarkerData::Initialize(pgd);
	SexLinkedData::Initialize(pgd);
	EvolutionaryGeneticsData::Initialize(pgd);		
	}//FstatisticAnalysisMultiFormat::Initialize(FstatisticAnalysisMultiFormat&)



void FstatisticAnalysisMultiFormat::Initialize(MultiformatMarkerData& parentData, aList<int>& popSubsetNums){
	ClearFstats();
	ResetFlags();
	MultiformatMarkerData::Initialize(parentData,popSubsetNums);
	}


void FstatisticAnalysisMultiFormat::Initialize(FstatisticAnalysisMultiFormat& parentData, aList<int>& popSubsetNums){
	ClearFstats();
	ResetFlags();
	MultiformatMarkerData::Initialize(parentData,popSubsetNums);
	}


void FstatisticAnalysisMultiFormat::CalculateIBDestimates(Ran3& ran,int sampleSize){
	FstatisticAnalysisHap::CalculateIBDestimates(ran,sampleSize);
	FstatisticAnalysisCodom::CalculateIBDestimates(ran,sampleSize);
	FstatisticAnalysisCodomSexLink::CalculateIBDestimates(ran,sampleSize);
	FstatisticAnalysisCytoFemale::CalculateIBDestimates(ran,sampleSize);
	FstatisticAnalysisCytoMale::CalculateIBDestimates(ran,sampleSize);
	}//FstatisticAnalysisMultiFormat::CalculateIBDestimates(Ran3&,int)
	
	
	
void FstatisticAnalysisMultiFormat::CalculateIBDexhaustive(void){
	FstatisticAnalysisHap::CalculateIBDexhaustive();
	FstatisticAnalysisCodom::CalculateIBDexhaustive();
	FstatisticAnalysisCodomSexLink::CalculateIBDexhaustive();
	FstatisticAnalysisCytoFemale::CalculateIBDexhaustive();
	FstatisticAnalysisCytoMale::CalculateIBDexhaustive();
	}//FstatisticAnalysisMultiFormat::CalculateIBDexhaustive()


void FstatisticAnalysisMultiFormat::Fstats_StandardLeastSquaresMethod (int numTrueDemes, bool zygoteMigration){//all data formats supported
	//Weir-Cockerham formulas and analogs for other data types
	FstatisticAnalysisHap::HapMarkerFstats(numTrueDemes);
	FstatisticAnalysisCodom::WeirTheta();
	FstatisticAnalysisDom2::DomMarkerFstats(numTrueDemes,zygoteMigration);
	FstatisticAnalysisCodomSexLink::SexLinkedCodomMarkerFstats(numTrueDemes,zygoteMigration);
	FstatisticAnalysisDomSexLink::DomMarkerFstats(numTrueDemes,zygoteMigration);
	FstatisticAnalysisCytoFemale::CytoFemaleMarkerFstats(numTrueDemes);
	FstatisticAnalysisCytoMale::CytoMaleMarkerFstats(numTrueDemes);
	
	if(Fmethod_.Includes(effectiveFstOverLociStd)){
		return;}
	Fmethod_.Append((int)effectiveFstOverLociStd);
	vcMethod_.Append((int) effectiveFstOverLociStd);//fill with dummy values, since symmetry with F-stats is assumed
	int Flevel;
	int mEff = Fmethod_.len()-1;
	Flevels_.Append(NEW StringList());//F-level
	Flevels_[mEff].Append(NEW aString("Fst"));
	Flevels_[mEff].Append(NEW aString("Fis"));
	Flevels_[mEff].Append(NEW aString("Fit"));

	Fstats_.Append(NEW aList<long double>());//F-level
	for (Flevel=0;Flevel<Flevels_[mEff].len();++Flevel){
		Fstats_[mEff].Append(zero);}

	FstatsLoc_.Append(NEW ObjList<aList<long double> >());//Fmethod,loc,F-level
	FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//Fmethod,loc,allele,F-level
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//Fmethod,loc, F-level

	vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
	vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
	vc_.Append(NEW aList<long double>());//vc-level
	vcLevels_.Append(NEW StringList());//vc-level
	
	long double sr = (long double)primarySexRatio();
	
/*	I don't think this is legal, because the weighting doesn't account for different effective population sizes among inheritance classes
	// now calculate the weighted average effective Fst from the variance components
	long double FSTeffective, FSTnum, FSTdenom, FSTnumLoc, FSTdenomLoc, FSTnumLocAll, FSTdenomLocAll;
	FSTnum = FSTdenom = FSTeffective = FSTnumLoc = FSTdenomLoc = FSTnumLocAll = FSTdenomLocAll = zero;

	long double FISval, FISnum, FISdenom, FISnumLoc, FISdenomLoc, FISnumLocAll, FISdenomLocAll;
	FISval = FISnum = FISdenom = FISnumLoc = FISdenomLoc = FISnumLocAll = FISdenomLocAll = zero;

	aString vcPopWord = "var(p)", vcIndivWord = "var(i)", vcGeneWord = "var(g)", vctotalWord = "var(t)", vcSLword = "var(pop,bBar)";
	aString vcHetWord = "var(i,heterogametic)", vcDiffWord = "diff(bm,bf)", vcIWeigntWord = "var(i,weighted)", vcScaleWord = "weightFactor";
	int popLevel, indivLevel, geneLevel, totalLevel, diffLevel, heterogameticLevel,IWeightLevel, scaleLevel;
	int m = Fmethod_.Loc(hapStd);
	if(m>=0){
		popLevel = vcLevels_[m].Loc(&vcPopWord);
		totalLevel = vcLevels_[m].Loc(&vctotalWord);
		FSTnum += vc_[m][popLevel];
		FSTdenom += two * vc_[m][totalLevel] - vc_[m][popLevel];
		}
	m = Fmethod_.Loc(WandC);
	if(m>=0){
		geneLevel = vcLevels_[m].Loc(&vcGeneWord);
		indivLevel = vcLevels_[m].Loc(&vcIndivWord);
		popLevel = vcLevels_[m].Loc(&vcPopWord);
		totalLevel = vcLevels_[m].Loc(&vctotalWord);
		FSTnum += vc_[m][popLevel];
		FSTdenom += vc_[m][totalLevel];
		FISnum += vc_[m][indivLevel];
		FISdenom += vc_[m][geneLevel] + vc_[m][indivLevel];
		}
	m = Fmethod_.Loc(domStd);
	if(m>=0){
		popLevel = vcLevels_[m].Loc(&vcPopWord);
		totalLevel = vcLevels_[m].Loc(&vctotalWord);
		FSTnum += vc_[m][popLevel];
		FSTdenom += two*vc_[m][totalLevel] - vc_[m][popLevel];
		}
	m = Fmethod_.Loc(sexLinkStd);
	if(m>=0){
		geneLevel = vcLevels_[m].Loc(&vcGeneWord);
		indivLevel = vcLevels_[m].Loc(&vcIndivWord);
		popLevel = vcLevels_[m].Loc(&vcPopWord);
		totalLevel = vcLevels_[m].Loc(&vctotalWord);
		FSTnum += (long double)6.0*sr*vc_[m][popLevel];
		FSTdenom += four*vc_[m][totalLevel] - vc_[m][popLevel];
		FISnum += vc_[m][indivLevel];
		FISdenom += vc_[m][geneLevel] + vc_[m][indivLevel];
		}
	m = Fmethod_.Loc(sexLinkDomStd);
	if(m>=0){
		diffLevel = vcLevels_[m].Loc(&vcDiffWord);//not really a variance component, but stored here
		heterogameticLevel = vcLevels_[m].Loc(&vcHetWord);
		popLevel = vcLevels_[m].Loc(&vcSLword);
		totalLevel = vcLevels_[m].Loc(&vctotalWord);
		IWeightLevel = vcLevels_[m].Loc(&vcIWeigntWord);
		scaleLevel = vcLevels_[m].Loc(&vcScaleWord);
		FSTnum += vc_[m][popLevel];
		FSTdenom += two*vc_[m][totalLevel] - vc_[m][popLevel];
//		FISnum += vc_[m][heterogameticLevel] + vc_[m][diffLevel];
//		FISdenom += vc_[m][heterogameticLevel];
		FISnum += vc_[m][IWeightLevel];
		FISdenom += vcLevels_[m].Loc(&vcIWeigntWord) * vc_[m][heterogameticLevel];
		}
	m = Fmethod_.Loc(CYfemale);
	if(m>=0){
		popLevel = vcLevels_[m].Loc(&vcPopWord);
		totalLevel = vcLevels_[m].Loc(&vctotalWord);
//		FSTnum += vc_[m][popLevel];
//		FSTdenom += two*sr*vc_[m][totalLevel] - vc_[m][popLevel];
		FSTnum += sr * vc_[m][popLevel];
		FSTdenom += two*vc_[m][totalLevel] - (two - sr) * vc_[m][popLevel];
		}
	m = Fmethod_.Loc(CYmale);
	if(m>=0){
		popLevel = vcLevels_[m].Loc(&vcPopWord);
		totalLevel = vcLevels_[m].Loc(&vctotalWord);
//		FSTnum += vc_[m][popLevel];
//		FSTdenom += two*(one-sr)*vc_[m][totalLevel] - vc_[m][popLevel];
		FSTnum += (one - sr) * vc_[m][popLevel];
		FSTdenom += two * vc_[m][totalLevel] - (one + sr) * vc_[m][popLevel];
		}
	if(FSTdenom!=zero){
		FSTeffective=FSTnum/FSTdenom;}
	if(FISdenom!=zero){
		FISval= FISnum/FISdenom;}
	Fstats_[mEff][0]=FSTeffective;
	Fstats_[mEff][1]=FISval;
	Fstats_[mEff][2]=one - (one - FISval) * (one - FSTeffective);
*/

	//calculate effective Fst from the average over loci of all inheritance classes	
	//calculate Fis from the weighted average variance components
	//calculate effective Fit from Wright's relationship
	StatList<long double> FSTeffective,vcI,vcG,vcSB,vcIF,vcScale;
	int loc,FIlevel,scaleLevel;//popLevel, indivLevel, geneLevel, totalLevel, diffLevel, heterogameticLevel,IWeightLevel;
	long double fstVal=zero, fisVal=zero;
	aString levelName = "Fst";
	long double FISval, vcGval, vcIval, vcSBval, vcIFval, vcScaleVal;
	FISval = zero;

	aString vcIndivWord = "var(i)", vcGeneWord = "var(g)", vcSBword = "var(pop,bBar)";
	aString vcFword = "F(i,weighted)", vcScaleWord = "weightFactor";
	int indivLevel, geneLevel, sbLevel;


	int m = Fmethod_.Loc(hapStdEffective);
	if(m>=0){
		Flevel = Flevels_[m].Loc(&levelName);
		for(loc=0;loc<nlociHap_;++loc){
			fstVal = FstatsLoc_[m][loc][Flevel];
			FSTeffective.Append(fstVal);
			}//loc
		}
	m = Fmethod_.Loc(WandC);
	if(m>=0){
		Flevel = Flevels_[m].Loc(&levelName);
		indivLevel = vcLevels_[m].Loc(&vcIndivWord);
		geneLevel = vcLevels_[m].Loc(&vcGeneWord);
		for(loc=0;loc<nlociCodom_;++loc){
			fstVal = FstatsLoc_[m][loc][Flevel];
			FSTeffective.Append(fstVal);
			vcGval = vcLoc_[m][loc][geneLevel];
			vcIval = vcLoc_[m][loc][indivLevel];
			vcG.Append(vcGval);
			vcI.Append(vcIval);
			}//loc
		}
	m = Fmethod_.Loc(domStd);
	if(m>=0){
		Flevel = Flevels_[m].Loc(&levelName);
		for(loc=0;loc<nlociDom_;++loc){
			fstVal = FstatsLoc_[m][loc][Flevel];
			FSTeffective.Append(fstVal);
			}//loc
		}
	m = Fmethod_.Loc(sexLinkStd);
	if(m>=0){
		Flevel = Flevels_[m].Loc(&levelName);
		indivLevel = vcLevels_[m].Loc(&vcIndivWord);
		geneLevel = vcLevels_[m].Loc(&vcGeneWord);
		for(loc=0;loc<nlociSLCodom_;++loc){
			fstVal = FstatsLoc_[m][loc][Flevel];
			FSTeffective.Append(fstVal);
			vcGval = vcLoc_[m][loc][geneLevel];
			vcIval = vcLoc_[m][loc][indivLevel];
			vcG.Append(vcGval);
			vcI.Append(vcIval);
			}//loc
		}
	m = Fmethod_.Loc(sexLinkDomStd);
	if(m>=0){
		Flevel = Flevels_[m].Loc(&levelName);
		indivLevel = vcLevels_[m].Loc(&vcIndivWord);
		sbLevel = vcLevels_[m].Loc(&vcSBword);
		FIlevel = vcLevels_[m].Loc(&vcFword);
		scaleLevel = vcLevels_[m].Loc(&vcScaleWord);
		for(loc=0;loc<nlociSLDom_;++loc){
			fstVal = FstatsLoc_[m][loc][Flevel];
			FSTeffective.Append(fstVal);
			vcSBval = vcLoc_[m][loc][sbLevel];
			vcIval = vcLoc_[m][loc][indivLevel];
			vcIFval = vcLoc_[m][loc][FIlevel];
			vcScaleVal = vcLoc_[m][loc][scaleLevel];
			vcSB.Append(vcSBval);
			vcI.Append(vcIval);
			vcIF.Append(vcIFval);
			vcScale.Append(vcScaleVal);
			}//loc
		}
	m = Fmethod_.Loc(CYfemale);
	if(m>=0){
		Flevel = Flevels_[m].Loc(&levelName);
		for(loc=0;loc<nlociCYfemale_;++loc){
			fstVal = FstatsLoc_[m][loc][Flevel];
			FSTeffective.Append(fstVal);
			}//loc
		}
	m = Fmethod_.Loc(CYmale);
	if(m>=0){
		Flevel = Flevels_[m].Loc(&levelName);
		for(loc=0;loc<nlociCYmale_;++loc){
			fstVal = FstatsLoc_[m][loc][Flevel];
			FSTeffective.Append(fstVal);
			}//loc
		}

	Fstats_[mEff][Flevel] = fstVal = FSTeffective.mean();
	levelName = "Fis";
	Flevel = Flevels_[mEff].Loc(&levelName);
	//fisVal constructed from non-variance-components in SLdom case
	//that should bias the weighted average over loci with different inheritance classes
	//so, weight each construction method by the number of loci contributing
	fisVal = vcG.sum()/(vcG.sum()+vcI.sum());
//	fisVal = vcG.n() * ( vcG.sum()/(vcG.sum()+vcI.sum()) );
//	fisVal += vcIF.n() * ( vcIF.sum()/vcScale.sum() );
//	fisVal /= vcG.n() + vcIF.n();
	Fstats_[mEff][Flevel] = fisVal;
	levelName = "Fit";
	Flevel = Flevels_[mEff].Loc(&levelName);
	long double fitVal = one - (one - fisVal) * (one - fstVal);
	Fstats_[mEff][Flevel]= fitVal;
	}//FstatisticAnalysisMultiFormat::Fstats_StandardLeastSquaresMethod()




long double FstatisticAnalysisMultiFormat::Nm(long double FstVal,int Fmethod,int numTrueDemes,bool zygoteMigration){
	//override in derived classes
	switch(Fmethod){
		case 0://unspecified =0;
		case 1://WandC =1;
		case 2://hierWandC =2;
		case 3://linearAlg =3;
		case 4://hierLinearAlg =4;
		case 5://linearIntrog =5;
		case 6://linearIntrogNoInt =6;
		case 7://factorialFstats =7;
		case 8://factorialFstatsII =8;
		case 9://factorialFstatsIII =9;
		case 10://linearAlgII =10;
			return FstatisticAnalysisCodom::NmFromDiploidFst(FstVal,numTrueDemes,zygoteMigration);
			break;
		case 11://domStd=11;
		case 12://hierDom=12;
			return FstatisticAnalysisDom2::NmFromDomFst(FstVal,numTrueDemes,zygoteMigration);
			break;
		case 13://sexLinkStd=13;
		case 14://sexLinkHier=14;//not supported yet 
			return FstatisticAnalysisCodomSexLink::NmFromSexLinkedCodomFst(FstVal,numTrueDemes,zygoteMigration);
			break;
		case 15://sexLinkDomStd=15;
		case 16://sexLinkHierDomStd=16;
			return FstatisticAnalysisDomSexLink::NmFromSexLinkedDomFst(FstVal,numTrueDemes,zygoteMigration);
			break;
		case 17://hapStd=17;
		case 18://hierHap=18;
			return FstatisticAnalysisHap::NmFromHaploidFst(FstVal,numTrueDemes);
			break;
		case 21://hapStdEffective=21;//equivalent values for codomnant diploid 
		case 22://domStdEffective=22;//equivalent values for codominant diploid
		case 23://sexLinkStdEffective=23;//equivalent values for codominant diploid
		case 24://sexLinkDomStdEffective=24;//equivalent values for codominant diploid
			return FstatisticAnalysisCodom::NmFromDiploidFst(FstVal,numTrueDemes,zygoteMigration);
			break;
		case 25://CYfemale=25;
		case 26://hierCYfemale=26;
			return FstatisticAnalysisCytoFemale::NmFromCytoFemaleFst(FstVal,numTrueDemes);
			break;
		case 27://CYfemaleEffective=27;//equivalent values for codominant diploid
			return FstatisticAnalysisCodom::NmFromDiploidFst(FstVal,numTrueDemes,zygoteMigration);
			break;
	  	case 28://CYmale=28;
		case 29://hierCYmale=29;
			return FstatisticAnalysisCytoMale::NmFromCytoMaleFst(FstVal,numTrueDemes);
			break;
		case 30://CYmaleEffective=30;//equivalent values for codominant diploid
			return FstatisticAnalysisCodom::NmFromDiploidFst(FstVal,numTrueDemes,zygoteMigration);
			break;
		case 100://IBDexhaustCodom=100;//equivalent values for codominant diploid
		case 101://IBDestimCodom;//equivalent values for codominant diploid
			return FstatisticAnalysisCodom::NmFromDiploidFst(FstVal,numTrueDemes,zygoteMigration);
		case 105://IBDexhaustSLCodom=105;
		case 106://IBDestimSLCodom=106;//not supported yet 
			return FstatisticAnalysisCodomSexLink::NmFromSexLinkedCodomFst(FstVal,numTrueDemes,zygoteMigration);
			break;
		default: break;
		}
		
	return zero;
	}//FstatisticAnalysisMultiFormat::Nm(long double ,int, bool, bool)


int FstatisticAnalysisMultiFormat::PrintFMethodHeading(ostream& outFile, const int Fmethod){
	if(Fmethod==hapStd){
			outFile<<"Least-squares estimators of Wright's F-statistics for haploid markers."<<endl;}
		else if(Fmethod ==hierHap){
			outFile<<"Least-squares estimators of Wright's hierarchical F-statistics for haploid markers."<<endl;}
		else if(Fmethod==domStd){
			outFile<<"Least-squares estimators of Wright's F-statistics for dominant markers."<<endl;}
		else if(Fmethod ==hierDom){
			outFile<<"Least-squares estimators of Wright's hierarchical F-statistics for dominant markers."<<endl;}
		else if(Fmethod==sexLinkStd){
			outFile<<"Least-squares estimators of Wright's F-statistics for sex-linked codominant markers."<<endl;}
		else if(Fmethod==CYfemale){
			outFile<<"Least-squares estimators of Wright's F-statistics for female-inherited cytoplasmic markers."<<endl;}
		else if(Fmethod==CYmale){
			outFile<<"Least-squares estimators of Wright's F-statistics for male-inherited cytoplasmic markers."<<endl;}
		else if(Fmethod ==sexLinkHier){
			outFile<<"Least-squares estimators of Wright's hierarchical F-statistics for sex-linked codominant markers."<<endl;}
		else if(Fmethod==sexLinkDomStd){
			outFile<<"Least-squares estimators of Wright's F-statistics for sex-linked dominant markers."<<endl;}
		else if(Fmethod ==sexLinkHierDomStd){
			outFile<<"Least-squares estimators of Wright's hierarchical F-statistics for sex-linked dominant markers."<<endl;}
		else if(Fmethod==hapStdEffective){
			outFile<<"Least-squares estimators of Wright's F-statistics for haploid markers, EFFECTIVE VALUES relative to diploid codominant markers."<<endl;}
		else if(Fmethod==domStdEffective){
			outFile<<"Least-squares estimators of Wright's F-statistics for dominant markers, EFFECTIVE VALUES relative to diploid codominant markers."<<endl;}
		else if(Fmethod==sexLinkStdEffective){
			outFile<<"Least-squares estimators of Wright's F-statistics for sex-linked codominant markers, EFFECTIVE VALUES relative to diploid codominant markers."<<endl;}
		else if(Fmethod==sexLinkDomStdEffective){
			outFile<<"Least-squares estimators of Wright's F-statistics for sex-linked dominant markers, EFFECTIVE VALUES relative to diploid codominant markers."<<endl;}
		else if(Fmethod==CYfemaleEffective){
			outFile<<"Least-squares estimators of Wright's F-statistics for female-inherited cytoplasmic markers, EFFECTIVE VALUES relative to diploid codominant markers."<<endl;}
		else if(Fmethod==CYmaleEffective){
			outFile<<"Least-squares estimators of Wright's F-statistics for male-inherited cytoplasmic markers, EFFECTIVE VALUES relative to diploid codominant markers."<<endl;}
		else if(Fmethod==effectiveFstOverLociStd){
			outFile<<"Least-squares estimators of Wright's F-statistics over all markers, EFFECTIVE VALUES relative to diploid codominant markers."<<endl;}
		else if(Fmethod ==IBDexhaustCodom ){
			outFile<<"Estimators of fully Factorial F-statistics for autosomal loci using Identity By Descent values calculated by exhaustive comparisons."<<endl;}
		else if(Fmethod ==IBDexhaustSLCodom ){
			outFile<<"Estimators of fully Factorial F-statistics for sex-linked loci using Identity By Descent values calculated by exhaustive comparisons."<<endl;}
		else if(Fmethod ==IBDestimCodom ){
			outFile<<"Estimators of fully Factorial F-statistics for autosomal loci using Identity By Descent values estimated by sampling."<<endl;}
		else if(Fmethod ==IBDestimSLCodom ){
			outFile<<"Estimators of fully Factorial F-statistics for sex-linked loci using Identity By Descent values estimated by sampling."<<endl;}
		else {
			return FstatisticAnalysisCore::PrintFMethodHeading(outFile, Fmethod);}
	return 1;//no error
	}//end FstatisticAnalysisMultiFormat::PrintFMethodHeading



//void FstatisticAnalysisMultiFormat::PrintFStatistics(ostream& outFile, const int Fmethod, char colSpacer = '\t');
//void FstatisticAnalysisMultiFormat::PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, char colSpacer = '\t');
//void FstatisticAnalysisMultiFormat::PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, char colSpacer = '\t');
//void FstatisticAnalysisMultiFormat::PrintFactorialFstatsDerivedStats(aString& outFileName);
//void FstatisticAnalysisMultiFormat::PrintFactorialFstatsDerivedStats(ostream& outFile);
//int FstatisticAnalysisMultiFormat::PrintVCmethodHeading(ostream& outFile, const int vcMethod);
//void FstatisticAnalysisMultiFormat::PrintVarianceComponents(ostream& outFile, const int vcMethod, char colSpacer='\t');
//void FstatisticAnalysisMultiFormat::PrintVarianceComponentsByLocus(ostream& outFile, const int vcMethod, char colSpacer='\t');
//void FstatisticAnalysisMultiFormat::PrintVarianceComponentsByAllele(ostream& outFile, const int vcMethod, char colSpacer='\t');




void FstatisticAnalysisMultiFormat::PrintFStatisticsByLocus(ostream& outFile, aString& titleStr, 
														StringList& /*locNames*/, char colSpacer){
	FstatisticAnalysisMultiFormat::PrintFStatisticsByLocus(outFile, titleStr, colSpacer);
	}//FstatisticAnalysisMultiFormat::PrintFStatisticsByLocus()



void FstatisticAnalysisMultiFormat::PrintFStatisticsByLocus(ostream& outFile, aString& titleStr, char colSpacer){
	//all methods -- add new ones here as they are invented
	StringList locNames; 
	for(int m=0;m<Fmethod_.len();++m){
		switch(Fmethod_[m]){
			case hapStd:
			case hapStdEffective:
				for(int loc=0;loc<variableLocusNumHap_.len();++loc){
					locNames.Append(NEW aString(traitName_[locusNumHap_[variableLocusNumHap_[loc]]]));}
				FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod_[m], titleStr, locNames, colSpacer);
				break;
			case WandC:
				for(int loc=0;loc<variableLocusNumCodom_.len();++loc){
					locNames.Append(NEW aString(traitName_[locusNumCodom_[variableLocusNumCodom_[loc]]]));}
				FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod_[m], titleStr, locNames, colSpacer);
				break;
			case domStd:
			case domStdEffective:
				for(int loc=0;loc<variableLocusNumDom_.len();++loc){
					locNames.Append(NEW aString(traitName_[locusNumDom_[variableLocusNumDom_[loc]]]));}
				FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod_[m], titleStr, locNames, colSpacer);
				break;
			case sexLinkStd:
			case sexLinkStdEffective:
				for(int loc=0;loc<variableLocusNumSLCodom_.len();++loc){
					locNames.Append(NEW aString(traitName_[locusNumSLCodom_[variableLocusNumSLCodom_[loc]]]));}
				FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod_[m], titleStr, locNames, colSpacer);
				break;
			case sexLinkDomStd:
			case sexLinkDomStdEffective:
				for(int loc=0;loc<variableLocusNumSLDom_.len();++loc){
					locNames.Append(NEW aString(traitName_[locusNumSLDom_[variableLocusNumSLDom_[loc]]]));}
				FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod_[m], titleStr, locNames, colSpacer);
				break;
			case CYfemale:
			case CYfemaleEffective:
				for(int loc=0;loc<variableLocusNumCYfemale_.len();++loc){
					locNames.Append(NEW aString(traitName_[locusNumCYfemale_[variableLocusNumCYfemale_[loc]]]));}
				FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod_[m], titleStr, locNames, colSpacer);
				break;
			case CYmale:
			case CYmaleEffective:
				for(int loc=0;loc<variableLocusNumCYmale_.len();++loc){
					locNames.Append(NEW aString(traitName_[locusNumCYmale_[variableLocusNumCYmale_[loc]]]));}
				FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod_[m], titleStr, locNames, colSpacer);
				break;
			default://ignore
				break;
			}//switch
		locNames.Clear();
		}//m
	}//FstatisticAnalysisMultiFormat::PrintFStatisticsByLocus()



void FstatisticAnalysisMultiFormat::PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer){
	//all methods -- add new ones here as they are invented
	StringList locNames; 
	switch(Fmethod){
		case hapStd:
		case hapStdEffective:
			for(int loc=0;loc<variableLocusNumHap_.len();++loc){
				locNames.Append(NEW aString(traitName_[locusNumHap_[variableLocusNumHap_[loc]]]));}
			FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod, titleStr, locNames, colSpacer);
			break;
		case WandC:
			for(int loc=0;loc<variableLocusNumCodom_.len();++loc){
				locNames.Append(NEW aString(traitName_[locusNumCodom_[variableLocusNumCodom_[loc]]]));}
			FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod, titleStr, locNames, colSpacer);
			break;
		case domStd:
		case domStdEffective:
			for(int loc=0;loc<variableLocusNumDom_.len();++loc){
				locNames.Append(NEW aString(traitName_[locusNumDom_[variableLocusNumDom_[loc]]]));}
			FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod, titleStr, locNames, colSpacer);
			break;
		case sexLinkStd:
		case sexLinkStdEffective:
			for(int loc=0;loc<variableLocusNumSLCodom_.len();++loc){
				locNames.Append(NEW aString(traitName_[locusNumSLCodom_[variableLocusNumSLCodom_[loc]]]));}
			FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod, titleStr, locNames, colSpacer);
			break;
		case sexLinkDomStd:
		case sexLinkDomStdEffective:
			for(int loc=0;loc<variableLocusNumSLDom_.len();++loc){
				locNames.Append(NEW aString(traitName_[locusNumSLDom_[variableLocusNumSLDom_[loc]]]));}
			FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod, titleStr, locNames, colSpacer);
			break;
		case CYfemale:
		case CYfemaleEffective:
			for(int loc=0;loc<variableLocusNumCYfemale_.len();++loc){
				locNames.Append(NEW aString(traitName_[locusNumCYfemale_[variableLocusNumCYfemale_[loc]]]));}
			FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod, titleStr, locNames, colSpacer);
			break;
		case CYmale:
		case CYmaleEffective:
			for(int loc=0;loc<variableLocusNumCYmale_.len();++loc){
				locNames.Append(NEW aString(traitName_[locusNumCYmale_[variableLocusNumCYmale_[loc]]]));}
			FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod, titleStr, locNames, colSpacer);
			break;
		default://ignore
			break;
		}//switch
	locNames.Clear();
	}//FstatisticAnalysisMultiFormat::PrintFStatisticsByLocus()



void FstatisticAnalysisMultiFormat::PrintFStatisticsByAllele(ostream& outFile, aString& titleStr,  
							StringList& /*locNames*/, ObjList<StringList>& /*alleleNames*/, char colSpacer){
	FstatisticAnalysisMultiFormat::PrintFStatisticsByAllele(outFile, titleStr, colSpacer);
	}


void FstatisticAnalysisMultiFormat::PrintFStatisticsByAllele(ostream& outFile, aString& titleStr, char colSpacer){
	//all methods
	StringList locNames; 
	ObjList<StringList> alleleNames;
	for(int m=0;m<Fmethod_.len();++m){
		switch(Fmethod_[m]){
			case hapStd:
			case hapStdEffective:
				for(int loc=0;loc<nlociHap_;++loc){
					locNames.Append(NEW aString(traitName_[locusNumHap_[loc]]));
					alleleNames.Append(NEW StringList(allelesPresentHap_[loc]));
					}
				FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod_[m], titleStr, locNames, alleleNames, colSpacer);
				break;
			case WandC:
				for(int loc=0;loc<nlociCodom_;++loc){
					locNames.Append(NEW aString(traitName_[locusNumCodom_[loc]]));
					alleleNames.Append(NEW StringList(allelesPresent_[loc]));
					}
				FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, WandC, titleStr, locNames, alleleNames, colSpacer);
				break;
			case domStd:
			case domStdEffective:
				for(int loc=0;loc<nlociDom_;++loc){
					locNames.Append(NEW aString(traitName_[locusNumDom_[loc]]));
					alleleNames.Append(NEW StringList(allelesPresentDom_[loc]));
					}
				FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod_[m], titleStr, locNames, alleleNames, colSpacer);
				break;
			case sexLinkStd:
			case sexLinkStdEffective:
				for(int loc=0;loc<nlociSLCodom_;++loc){
					locNames.Append(NEW aString(traitName_[locusNumSLCodom_[loc]]));
					alleleNames.Append(NEW StringList(allelesPresentSLCodom_[loc]));
					}
				FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod_[m], titleStr, locNames, alleleNames, colSpacer);
				break;
			case sexLinkDomStd:
			case sexLinkDomStdEffective:
				for(int loc=0;loc<nlociSLDom_;++loc){
					locNames.Append(NEW aString(traitName_[locusNumSLDom_[loc]]));
					alleleNames.Append(NEW StringList(allelesPresentSLDom_[loc]));
					}
				FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod_[m], titleStr, locNames, alleleNames, colSpacer);
				break;
			case CYfemale:
			case CYfemaleEffective:
				for(int loc=0;loc<nlociCYfemale_;++loc){
					locNames.Append(NEW aString(traitName_[locusNumCYfemale_[loc]]));
					alleleNames.Append(NEW StringList(allelesPresentCYfemale_[loc]));
					}
				FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod_[m], titleStr, locNames, alleleNames, colSpacer);
				break;
			case CYmale:
			case CYmaleEffective:
				for(int loc=0;loc<nlociCYmale_;++loc){
					locNames.Append(NEW aString(traitName_[locusNumCYmale_[loc]]));
					alleleNames.Append(NEW StringList(allelesPresentCYmale_[loc]));
					}
				FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod_[m], titleStr, locNames, alleleNames, colSpacer);
				break;
			default://ignore
				break;
			}//switch
		locNames.Clear();
		alleleNames.Clear();
		}//m
	}//FstatisticAnalysisMultiFormat::PrintFStatisticsByAllele()




void FstatisticAnalysisMultiFormat::PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer){
	//all methods
	StringList locNames; 
	ObjList<StringList> alleleNames;
	switch(Fmethod){
		case hapStd:
		case hapStdEffective:
			for(int loc=0;loc<nlociHap_;++loc){
				locNames.Append(NEW aString(traitName_[locusNumHap_[loc]]));
				alleleNames.Append(NEW StringList(allelesPresentHap_[loc]));
				}
			FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod, titleStr, locNames, alleleNames, colSpacer);
			break;
		case WandC:
			for(int loc=0;loc<nlociCodom_;++loc){
				locNames.Append(NEW aString(traitName_[locusNumCodom_[loc]]));
				alleleNames.Append(NEW StringList(allelesPresent_[loc]));
				}
			FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, WandC, titleStr, locNames, alleleNames, colSpacer);
			break;
		case domStd:
		case domStdEffective:
			for(int loc=0;loc<nlociDom_;++loc){
				locNames.Append(NEW aString(traitName_[locusNumDom_[loc]]));
				alleleNames.Append(NEW StringList(allelesPresentDom_[loc]));
				}
			FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod, titleStr, locNames, alleleNames, colSpacer);
			break;
		case sexLinkStd:
		case sexLinkStdEffective:
			for(int loc=0;loc<nlociSLCodom_;++loc){
				locNames.Append(NEW aString(traitName_[locusNumSLCodom_[loc]]));
				alleleNames.Append(NEW StringList(allelesPresentSLCodom_[loc]));
				}
			FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod, titleStr, locNames, alleleNames, colSpacer);
			break;
		case sexLinkDomStd:
		case sexLinkDomStdEffective:
			for(int loc=0;loc<nlociSLDom_;++loc){
				locNames.Append(NEW aString(traitName_[locusNumSLDom_[loc]]));
				alleleNames.Append(NEW StringList(allelesPresentSLDom_[loc]));
				}
			FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod, titleStr, locNames, alleleNames, colSpacer);
			break;
		case CYfemale:
		case CYfemaleEffective:
			for(int loc=0;loc<nlociCYfemale_;++loc){
				locNames.Append(NEW aString(traitName_[locusNumCYfemale_[loc]]));
				alleleNames.Append(NEW StringList(allelesPresentCYfemale_[loc]));
				}
			FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod, titleStr, locNames, alleleNames, colSpacer);
			break;
		case CYmale:
		case CYmaleEffective:
			for(int loc=0;loc<nlociCYmale_;++loc){
				locNames.Append(NEW aString(traitName_[locusNumCYmale_[loc]]));
				alleleNames.Append(NEW StringList(allelesPresentCYmale_[loc]));
				}
			FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod, titleStr, locNames, alleleNames, colSpacer);
			break;
		default://ignore
			break;
		}//switch
	locNames.Clear();
	alleleNames.Clear();
	}//FstatisticAnalysisMultiFormat::PrintFStatisticsByAllele()




int FstatisticAnalysisMultiFormat::PrintIBDMethodHeading(ostream& outFile, const int IBDmethod){
	if(IBDmethod==IBDexhaustCodom){
			outFile<<"Identity by descent for autosomal codominant markers: exhaustive calculation."<<endl;}
		else if(IBDmethod ==IBDestimCodom){
			outFile<<"Identity by descent for autosomal codominant markers: estimated by random sampling."<<endl;}
		else if(IBDmethod==IBDexhaustSLCodom){
			outFile<<"Identity by descent for sex-linked codominant markers: exhaustive calculation."<<endl;}
		else if(IBDmethod==IBDestimSLCodom){
			outFile<<"Identity by descent for sex-linked codominant markers: estimated by random sampling."<<endl;}
		else if(IBDmethod==IBDexhaustHap){
			outFile<<"Identity by descent for haploid markers: exhaustive calculation."<<endl;}
		else if(IBDmethod ==IBDestimHap){
			outFile<<"Identity by descent for haploid markers: estimated by random sampling."<<endl;}
		else if(IBDmethod==IBDexhaustCytoFemale){
			outFile<<"Identity by descent for exclusively female-inherited markers: exhaustive calculation."<<endl;}
		else if(IBDmethod ==IBDestimCytoFemale){
			outFile<<"Identity by descent for exclusively female-inherited markers: estimated by random sampling."<<endl;}
		else if(IBDmethod==IBDexhaustCytoMale){
			outFile<<"Identity by descent for exclusively male-inherited markers: exhaustive calculation."<<endl;}
		else if(IBDmethod==IBDestimCytoMale){
			outFile<<"Identity by descent for exclusively male-inherited markers: estimated by random sampling."<<endl;}
		else {
			return FstatisticAnalysisCore::PrintIBDMethodHeading(outFile, IBDmethod);}
	return 1;//no error
	}//end FstatisticAnalysisMultiFormat::PrintIBDMethodHeading




void FstatisticAnalysisMultiFormat::PrintIBDByLocus(ostream& outFile, aString& titleStr, 
														StringList& /*locNames*/, char colSpacer){
	FstatisticAnalysisMultiFormat::PrintIBDByLocus(outFile, titleStr, colSpacer);
	}//FstatisticAnalysisMultiFormat::PrintIBDByLocus()



void FstatisticAnalysisMultiFormat::PrintIBDByLocus(ostream& outFile, aString& titleStr, char colSpacer){
	//all methods -- add new ones here as they are invented
	StringList locNames; 
	int m,loc;
	for(m=0;m<IBDmethod_.len();++m){
		switch(IBDmethod_[m]){
			case IBDexhaustHap:
			case IBDestimHap:
				for(loc=0;loc<variableLocusNumHap_.len();++loc){
					locNames.Append(NEW aString(traitName_[locusNumHap_[variableLocusNumHap_[loc]]]));}
				FstatisticAnalysisCore::PrintIBDByLocus(outFile, Fmethod_[m], titleStr, locNames, colSpacer);
				break;
			case IBDexhaustCodom:
			case IBDestimCodom:
				for(loc=0;loc<variableLocusNumCodom_.len();++loc){
					locNames.Append(NEW aString(traitName_[locusNumCodom_[variableLocusNumCodom_[loc]]]));}
				FstatisticAnalysisCore::PrintIBDByLocus(outFile, Fmethod_[m], titleStr, locNames, colSpacer);
				break;
			case IBDexhaustSLCodom:
			case IBDestimSLCodom:
				for(loc=0;loc<variableLocusNumSLCodom_.len();++loc){
					locNames.Append(NEW aString(traitName_[locusNumSLCodom_[variableLocusNumSLCodom_[loc]]]));}
				FstatisticAnalysisCore::PrintIBDByLocus(outFile, Fmethod_[m], titleStr, locNames, colSpacer);
				break;
			case IBDexhaustCytoFemale:
			case IBDestimCytoFemale:
				for(loc=0;loc<variableLocusNumCYfemale_.len();++loc){
					locNames.Append(NEW aString(traitName_[locusNumCYfemale_[variableLocusNumCYfemale_[loc]]]));}
				FstatisticAnalysisCore::PrintIBDByLocus(outFile, Fmethod_[m], titleStr, locNames, colSpacer);
				break;
			case IBDexhaustCytoMale:
			case IBDestimCytoMale:
				for(loc=0;loc<variableLocusNumCYmale_.len();++loc){
					locNames.Append(NEW aString(traitName_[locusNumCYmale_[variableLocusNumCYmale_[loc]]]));}
				FstatisticAnalysisCore::PrintIBDByLocus(outFile, Fmethod_[m], titleStr, locNames, colSpacer);
				break;
			default://ignore
				break;
			}//switch
		locNames.Clear();
		}//m
	}//FstatisticAnalysisMultiFormat::PrintIBDByLocus()



void FstatisticAnalysisMultiFormat::PrintIBDByLocus(ostream& outFile, const int IBDmethod, aString& titleStr, char colSpacer){
	//all methods -- add new ones here as they are invented
	StringList locNames; 
	int loc;
	switch(IBDmethod){
		case IBDexhaustHap:
		case IBDestimHap:
			for(loc=0;loc<variableLocusNumHap_.len();++loc){
				locNames.Append(NEW aString(traitName_[locusNumHap_[variableLocusNumHap_[loc]]]));}
			FstatisticAnalysisCore::PrintIBDByLocus(outFile, IBDmethod, titleStr, locNames, colSpacer);
			break;
		case IBDexhaustCodom:
		case IBDestimCodom:
			for(loc=0;loc<variableLocusNumCodom_.len();++loc){
				locNames.Append(NEW aString(traitName_[locusNumCodom_[variableLocusNumCodom_[loc]]]));}
			FstatisticAnalysisCore::PrintIBDByLocus(outFile, IBDmethod, titleStr, locNames, colSpacer);
			break;
		case IBDexhaustSLCodom:
		case IBDestimSLCodom:
			for(loc=0;loc<variableLocusNumSLCodom_.len();++loc){
				locNames.Append(NEW aString(traitName_[locusNumSLCodom_[variableLocusNumSLCodom_[loc]]]));}
			FstatisticAnalysisCore::PrintIBDByLocus(outFile, IBDmethod, titleStr, locNames, colSpacer);
			break;
		case IBDexhaustCytoFemale:
		case IBDestimCytoFemale:
			for(loc=0;loc<variableLocusNumCYfemale_.len();++loc){
				locNames.Append(NEW aString(traitName_[locusNumCYfemale_[variableLocusNumCYfemale_[loc]]]));}
			FstatisticAnalysisCore::PrintIBDByLocus(outFile, IBDmethod, titleStr, locNames, colSpacer);
			break;
		case IBDexhaustCytoMale:
		case IBDestimCytoMale:
			for(loc=0;loc<variableLocusNumCYmale_.len();++loc){
				locNames.Append(NEW aString(traitName_[locusNumCYmale_[variableLocusNumCYmale_[loc]]]));}
			FstatisticAnalysisCore::PrintIBDByLocus(outFile, IBDmethod, titleStr, locNames, colSpacer);
			break;
		default://ignore
			break;
		}//switch
	locNames.Clear();
	}//FstatisticAnalysisMultiFormat::PrintIBDByLocus()








#endif
