/*******************************************
*
*
*	Fstatistics.cp
*
*	This class calculates F-statistics using various methods
*
********************************************/
#ifndef Fstatisticscp
#define Fstatisticscp = 1

#include "Fstatistics.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"

Fstatistics::Fstatistics(void):
  	allelesCurrent_(0), varLociCurrent_(0), FstatsCurrent_(0) {};
  					
Fstatistics::Fstatistics(ObjList<GeneticPopulation>& popList):
  	allelesCurrent_(0), varLociCurrent_(0), FstatsCurrent_(0) {
  		ObjList<GeneticPopulation>* group0 = NEW ObjList<GeneticPopulation>();
  		popList_.Append(group0);
  		for(int p=0;p<popList.len();++p){
  			group0->Append(&(popList[p]));}
  		}//end constructor
  
Fstatistics::Fstatistics(ObjList<ObjList<GeneticPopulation> >& popList):
  	allelesCurrent_(0), varLociCurrent_(0), FstatsCurrent_(0) {
  		for (int g=0;g<popList.len();++g){//copy popList
  			ObjList<GeneticPopulation>* group = NEW ObjList<GeneticPopulation>();
  			ObjList<GeneticPopulation>& pops =popList[g];
  			popList_.Append(group);
  			for(int p=0;p<pops.len();++p){
  				group->Append(&(pops[p]));}
  			}
  		}//end constructor

Fstatistics::Fstatistics(ObjList<ObjList<GeneticPopulation> >* popList):
  	allelesCurrent_(0), varLociCurrent_(0), FstatsCurrent_(0) {
  		for (int g=0;g<popList->len();++g){//copy popList
  			ObjList<GeneticPopulation>* group = NEW ObjList<GeneticPopulation>();
  			ObjList<GeneticPopulation>& pops =(*popList)[g];
  			popList_.Append(group);
  			for(int p=0;p<pops.len();++p){
  				group->Append(&(pops[p]));}
  			}
  		}/*end constructor*/
 



long double Fstatistics::Fis(void){
	if(!FstatsCurrent_){WeirTheta(0);}/*default*/
	aString Fword="Fis";
	int Flevel = Flevels_.Loc(&Fword);
	return Fstats_[ Flevel];
	}/*end  Fstatistics::Fis*/

long double Fstatistics::Fis(int loc){
	if(!FstatsCurrent_){WeirTheta(0);}/*default*/
	aString Fword="Fis";
	int Flevel = Flevels_.Loc(&Fword);
	return FstatsLoc_[loc][Flevel];
	}/*end  Fstatistics::Fis*/

long double Fstatistics::Fis(int loc, int alleleVal){
	if(!FstatsCurrent_){WeirTheta(0);}/*default*/
	StringList& locAlleleList=alleleList_[loc];
	long alleleAt;
	aString aVal(alleleVal);
	if (locAlleleList.Includes(&aVal)){
			alleleAt=locAlleleList.Loc(&aVal);}
		else{return (long double)0.0;}	
	aString Fword="Fis";
	int Flevel = Flevels_.Loc(&Fword);
	return FstatsAll_[loc][alleleAt][Flevel];
	}/*end  Fstatistics::Fis*/

long double Fstatistics::Fit(void){
	if(!FstatsCurrent_){WeirTheta(0);}/*default*/
	aString Fword="Fit";
	int Flevel = Flevels_.Loc(&Fword);
	return Fstats_[ Flevel];
	}/*end  Fstatistics::Fit*/

long double Fstatistics::Fit(int loc){
	if(!FstatsCurrent_){WeirTheta(0);}/*default*/
	aString Fword="Fit";
	int Flevel = Flevels_.Loc(&Fword);
	return FstatsLoc_[loc][Flevel];
	}/*end  Fstatistics::Fit*/

long double Fstatistics::Fit(int loc, int alleleVal){
	if(!FstatsCurrent_){WeirTheta(0);}/*default*/
	StringList& locAlleleList=alleleList_[loc];
	long alleleAt;
	aString aVal(alleleVal);
	if (locAlleleList.Includes(&aVal)){
			alleleAt=locAlleleList.Loc(&aVal);}
		else{return (long double)0.0;}	
	aString Fword="Fit";
	int Flevel = Flevels_.Loc(&Fword);
	return FstatsAll_[loc][alleleAt][Flevel];
	}/*end  Fstatistics::Fit*/

long double Fstatistics::Fst(void){
	if(!FstatsCurrent_){WeirTheta(0);}//default
	aString Fword="Fst";
	int Flevel = Flevels_.Loc(&Fword);
	return Fstats_[ Flevel];
	}/*end  Fstatistics::Fst*/

long double Fstatistics::Fst(int loc){
	if(!FstatsCurrent_){WeirTheta(0);}/*default*/
	aString Fword="Fst";
	int Flevel = Flevels_.Loc(&Fword);
	return FstatsLoc_[loc][Flevel];
	}/*end  Fstatistics::Fst*/

long double Fstatistics::Fst(int loc, int alleleVal){
	if(!FstatsCurrent_){WeirTheta(0);}/*default*/
	StringList& locAlleleList=alleleList_[loc];
	long alleleAt;
	aString aVal(alleleVal);
	if (locAlleleList.Includes(&aVal)){
			alleleAt=locAlleleList.Loc(&aVal);}
		else{return (long double)0.0;}	
	aString Fword="Fst";
	int Flevel = Flevels_.Loc(&Fword);
	return FstatsAll_[loc][alleleAt][Flevel];
	}/*end  Fstatistics::Fst*/


long double Fstatistics::Fgt(void){
	if(!FstatsCurrent_){HierarchicalWeirTheta();}/*default*/
	aString Fword="Fgt";
	int Flevel = Flevels_.Loc(&Fword);
	return Fstats_[ Flevel];
	}/*end  Fstatistics::Fgt*/

long double Fstatistics::Fgt(int loc){
	if(!FstatsCurrent_){HierarchicalWeirTheta();}/*default*/
	aString Fword="Fst";
	int Flevel = Flevels_.Loc(&Fword);
	return FstatsLoc_[loc][Flevel];
	}/*end  Fstatistics::Fgt*/

long double Fstatistics::Fgt(int loc, int alleleVal){
	if(!FstatsCurrent_){HierarchicalWeirTheta();}/*default*/
	StringList& locAlleleList=alleleList_[loc];
	long alleleAt;
	aString aVal(alleleVal);
	if (locAlleleList.Includes(&aVal)){alleleAt=locAlleleList.Loc(&aVal);}
		else{return (long double)0.0;}	
	aString Fword="Fgt";
	int Flevel = Flevels_.Loc(&Fword);
	return FstatsAll_[loc][alleleAt][Flevel];
	}/*end  Fstatistics::Fgt*/


long double Fstatistics::Fsg(void){
	if(!FstatsCurrent_){HierarchicalWeirTheta();}/*default*/
	aString Fword="Fsg";
	int Flevel = Flevels_.Loc(&Fword);
	return Fstats_[ Flevel];
	}/*end  Fstatistics::Fsg*/

long double Fstatistics::Fsg(int loc){
	if(!FstatsCurrent_){HierarchicalWeirTheta();}/*default*/
	aString Fword="Fsg";
	int Flevel = Flevels_.Loc(&Fword);
	return FstatsLoc_[loc][Flevel];
	}/*end  Fstatistics::Fsg*/

long double Fstatistics::Fsg(int loc, int alleleVal){
	if(!FstatsCurrent_){HierarchicalWeirTheta();}/*default*/
	StringList& locAlleleList=alleleList_[loc];
	long alleleAt;
	aString aVal(alleleVal);
	if (locAlleleList.Includes(&aVal)){alleleAt=locAlleleList.Loc(&aVal);}
		else{return (long double)0.0;}	
	aString Fword="Fsg";
	int Flevel = Flevels_.Loc(&Fword);
	return FstatsAll_[loc][alleleAt][Flevel];
	}/*end  Fstatistics::Fsg*/


long double Fstatistics::Fig(void){
	if(!FstatsCurrent_){HierarchicalWeirTheta();}/*default*/
	aString Fword="Fig";
	int Flevel = Flevels_.Loc(&Fword);
	return Fstats_[ Flevel];
	}/*end  Fstatistics::Fig*/

long double Fstatistics::Fig(int loc){
	if(!FstatsCurrent_){HierarchicalWeirTheta();}/*default*/
	aString Fword="Fig";
	int Flevel = Flevels_.Loc(&Fword);
	return FstatsLoc_[loc][Flevel];
	}/*end  Fstatistics::Fig*/

long double Fstatistics::Fig(int loc, int alleleVal){
	if(!FstatsCurrent_){HierarchicalWeirTheta();}/*default*/
	StringList& locAlleleList=alleleList_[loc];
	long alleleAt;
	aString aVal(alleleVal);
	if (locAlleleList.Includes(&aVal)){alleleAt=locAlleleList.Loc(&aVal);}
		else{return (long double)0.0;}	
	aString Fword="Fig";
	int Flevel = Flevels_.Loc(&Fword);
	return FstatsAll_[loc][alleleAt][Flevel];
	}/*end  Fstatistics::Fig*/

void Fstatistics::FAndErrorString(aString& FRow, aString& Fword,int calcNm){
	FRow.AppendCharToLen(' ',17);
	int Flevel = Flevels_.Loc(&Fword);
	long double Fvalue = Fstats_[ Flevel];
	FRow.cat(Fvalue,2,5);
	FRow.AppendCharToLen(' ',29);
	/*Write(Output, Fvalue : 7 : 5, Spaces(5));*/
	long double stdev=(long double)0.0;long double stdErr=(long double)0.0;
	if (variableLoci_.len() > 1) {
			StatList<long double> locJack;
			for (int j=0;j<variableLoci_.len();++j) {
				aList<long double>& jackFList =jackFstatsLoc_[j];
				locJack.Append(jackFList[Flevel]);
				}/*end for j*/
			stdev = locJack.stdev();
			stdErr = stdev /  sqrt((long double) locJack.len());
			}
		else{
			stdev=stdErr = (long double)0.0;
			}//end if variableLoci..else
	//Write(Output, sqrt(sampleVariance) : 7 : 5, Spaces(5), stdErr : 7 : 5, Spaces(4));
	FRow.cat(stdev,2,5);
	FRow.AppendCharToLen(' ',41);
	FRow.cat(stdErr,2,5);	
	if (calcNm){
		FRow.AppendCharToLen(' ',52);
		if((Fvalue + 1.96 * stdev) <= 0.0) {
				FRow.cat("    ");}
			else if (((Fvalue + 1.96 * stdev) > 0.0) && ((Fvalue + 1.96 * stdev) < 1.0)) {
				//Write(Output, ((1.0 / (Fvalue + 1.96 * stdev) - 1.0) / 4.0) : 10 : 4)
				FRow.cat(((1.0 / (Fvalue + 1.96 * stdev) - 1.0) / 4.0),6, 4);}			
			else if ((Fvalue + 1.96 * stdev) >= 1.0) {
				//Write(Output, 0.0 : 10 : 4)
				FRow.cat( 0.0,6, 4);}
			else{
				FRow.cat("panmictic");}
		FRow.AppendCharToLen(' ',67);
		//Write(Output, Spaces(4));

		if(Fvalue <= 0.0){
				FRow.cat("    ");}
			else if ((Fvalue > 0.0) && (Fvalue < 1.0)) {
				//Write(Output, ((1.0 / Fvalue - 1.0) / 4.0) : 10 : 4)
				FRow.cat(((1.0 / Fvalue - 1.0) / 4.0),6, 4);}			
			else if( Fvalue >= 1.0) {
				//Write(Output, 0.0 : 10 : 4)
				FRow.cat( 0.0, 6,4);}
			else{
				FRow.cat("panmictic");}
		FRow.AppendCharToLen(' ',82);
		//Write(Output, Spaces(4));

		if((Fvalue - 1.96 * stdev) <= 0.0) {
				FRow.cat("    ");}
			else if (((Fvalue - 1.96 * stdev) > 0.0) && ((Fvalue - 1.96 * stdev) < 1.0)) {
				//Write(Output, ((1.0 / (Fvalue - 1.96 * stdev) - 1.0) / 4.0) : 10 : 4)
				FRow.cat(((1.0 / (Fvalue - 1.96 * stdev) - 1.0) / 4.0),6, 4);}			
			else if ((Fvalue - 1.96 * stdev) >= 1.0) {
				//Write(Output, 0.0 : 10 : 4)
				FRow.cat( 0.0,6, 4);}
			else{
				FRow.cat("panmictic");}
		
		}//end if calcNm
	}//end Fstatistics::FAndErrorString


void Fstatistics::PrintFstatSummary (ostream& out){
	//This is a general format for printing out F-statistics, giving
	//a summary table of values over loci and alleles. Errors are taken from jackknife estimates.
//	char endl=13;//mac os needs '\r' for and endofline character
	if(!FstatsCurrent_){WeirTheta(0);}
	out<<"Summary table of F-statistics, averaged over all loci and alleles."<<endl;
	out<<"Method used:  "<<FstatsType<<endl<<endl;
	aString colheadings;
	colheadings.AppendCharToLen(' ',20);
	colheadings.cat("value");
	colheadings.AppendCharToLen(' ',32);
	colheadings.cat("(s.d.)");
	colheadings.AppendCharToLen(' ',43);
	colheadings.cat("(s.e.)");
	colheadings.AppendCharToLen(' ',53);
	colheadings.cat("lower bound");
	colheadings.AppendCharToLen(' ',70);
	colheadings.cat("Nm");
	colheadings.AppendCharToLen(' ',80);
	colheadings.cat("upper bound");

	aString Fword("Fgt");
	aString FgtRow(Fword);
	if (Flevels_.len()==6){//only follow through if Fgt was calculated
		if ((FstatsType == "Weir & Cockerham")||(FstatsType == "Hierarchical Weir & Cockerham")) {
			FgtRow.cat(" (=theta 2)");}//Spaces(4));
		FAndErrorString(FgtRow, Fword,1);//do Nm
		}

	Fword="Fst";
	aString FstRow(Fword);
	if ((FstatsType == "Weir & Cockerham")||(FstatsType == "Hierarchical Weir & Cockerham")) {
		FstRow.cat(" (=theta 1)");}//Spaces(4));
	FAndErrorString(FstRow,  Fword,1);//do Nm

	Fword="Fsg";
	aString FsgRow(Fword);
	if (Flevels_.len()==6){//only follow through if Fsg was calculated
		FAndErrorString(FsgRow, Fword,1);//do Nm	
		}

	Fword="Fis";
	aString FisRow(Fword);
	if ((FstatsType == "Weir & Cockerham")||(FstatsType == "Hierarchical Weir & Cockerham")) {
		FisRow.cat(" (=f)");}//Spaces(4));
	FAndErrorString(FisRow, Fword,0);//don't do Nm	

	Fword="Fig";
	aString FigRow(Fword);
	if (Flevels_.len()==6){//only follow through if Fig was calculated
		FAndErrorString(FigRow,Fword,0);//don't do Nm	
		}
	

	Fword="Fit";
	aString FitRow(Fword);
	if ((FstatsType == "Weir & Cockerham")||(FstatsType == "Hierarchical Weir & Cockerham")) {
		FitRow.cat(" (=F)");}//Spaces(4));
	FAndErrorString(FitRow,  Fword,0);//don't do Nm	

	out<<colheadings<<endl;
	if (Flevels_.len()==6){out<<FgtRow<<endl;}
	out<<FstRow<<endl;
	if (Flevels_.len()==6){out<<FsgRow<<endl;}
	out<<FisRow<<endl;
	if (Flevels_.len()==6){out<<FigRow<<endl;}
	out<<FitRow<<endl;
	out<<endl<<endl;
	}//end Fstatistics::PrintFstatSummary




void Fstatistics::UpdateAlleleList(void){
	alleleList_.Clear();
	ObjList<GeneticPopulation>&  pops1=popList_[0];
	GeneticPopulation&  pop0=pops1[0];
	int nloci=pop0.loci();//determine from the first population
	for (int j=0;j<nloci;++j){
		StringList* locList=NEW StringList();
		alleleList_.Append(locList);
		for (int g=0;g<popList_.len();++g){
			ObjList<GeneticPopulation>& pops2=popList_[g];
			for (int p=0;p<pops2.len();++p){
				GeneticPopulation& pop= pops2[p];
				int nAlleles=pop.Alleles(j);
				for (int a=0;a<nAlleles;++a){
					aString alleleVal=pop.AlleleValue(j,a);
					if (!(locList->Includes(&alleleVal))){
						locList->Append(NEW aString(alleleVal));}
					}//end for a
				}//end for y
			}//end for x
		locList->Sort();
		}//end for j
	allelesCurrent_=1;
	}//end Fstatistics::UpdateAlleleList


void Fstatistics::UpdateVariableLoci (void){
	//determine which listed populations are variable
	variableLoci_.Clear();
	int nloci=alleleList_.len();
	aString alleleVal1;
	for (int j=0;j<nloci;++j){
		int firstAlleleFound=0;
		for (int g=0;g<popList_.len();++g){
			ObjList<GeneticPopulation>& group=popList_[g];
			int gBreak=0;
			for (int p=0;p<group.len();++p){
				GeneticPopulation& pop=group[p];
				if (pop.Alleles(j)>1){
				variableLoci_.Append(j);
				gBreak=1;
				break;
				}//end if
				if(!(pop.extinct())){
					if (firstAlleleFound==0){
							alleleVal1=pop.AlleleValue(j,0);
							firstAlleleFound=1;
							}
						else{
							aString& nextAlleleVal=pop.AlleleValue(j,0);
							if (alleleVal1!=nextAlleleVal){
								//fixed for different alleles
								variableLoci_.Append(j);
								gBreak=1;
								break;
								}//end if
							}//end else
						}//end if not extinct
				}//end for p
			if (gBreak){break;}
			}//end for g
		}//end for j
	varLociCurrent_=1;
	}//end  Fstatistics::UpdateVariableLoci



void Fstatistics::WeirTheta (int group){
	//calculates Weir & Cockerham's  F-statistics among populations,
	//to be used when there is only one group of pops.
	//Notation follows Wier & Cockerham 1984, eqns. 1-4, p.1359-60

	//clear out any previous calculations
	if( (FstatsCurrent_)&&(FstatsType=="Weir & Cockerham")){return;}
	FstatsType="Weir & Cockerham";
	FstatsLoc_.Clear();//loc, F-level
	FstatsAll_.Clear();//loc,allele, F-level
	Fstats_.Clear();//F-level
	jackFstatsLoc_.Clear();//loc, F-level
	Flevels_.Clear();//F-level

	//set up all the F-stat arrays
	// first the array describing the Fstat structure:
	aString* FWord=NEW aString("Fit");
	Flevels_.Append(FWord);
	FWord=NEW aString("Fis");
	Flevels_.Append(FWord);
	FWord=NEW aString("Fst");
	Flevels_.Append(FWord);
	//make and initialize the jackknife & by-locus Fstat lists
	for (int Flevel=0;Flevel<3;++Flevel){Fstats_.Append(0.0);}
	//some intermediate variables
	aList<long double>* ALoc=NEW aList<long double>();
	aList<long double>* BLoc=NEW aList<long double>();
	aList<long double>* CLoc=NEW aList<long double>();
	aList<long double>* BCLoc=NEW aList<long double>();
	aList<long double>* ABCLoc=NEW aList<long double>();
	if (!allelesCurrent_){UpdateAlleleList();}
	if (!varLociCurrent_){UpdateVariableLoci();}
	int nloci=alleleList_.len();
	for (int j=0;j<nloci;++j){
		//make and initialize the jackknife & by-locus Fstat lists for this locus 
		aList<long double>* locFstat = NEW aList<long double>();
		FstatsLoc_.Append(locFstat);
		aList<long double>* locJackFstat = NEW aList<long double>();
		jackFstatsLoc_.Append(locJackFstat);
		for (int Flevel=0;Flevel<3;++Flevel){
			locFstat->Append((long double)0.0);
			locJackFstat->Append((long double)0.0);
			}
		//make and initialize the jackknife & by-allele Fstat lists for this locus 
		ObjList<aList<long double> >* locAlleleFstat = NEW ObjList<aList<long double> >();
		FstatsAll_.Append(locAlleleFstat);
		//initialize variables
		ALoc->Append((long double)0.0);
		BLoc->Append((long double)0.0);
		CLoc->Append((long double)0.0);
		BCLoc->Append((long double)0.0);
		ABCLoc->Append((long double)0.0);

		//make and initialize the by-allele F-stat aList for each allele and append to this locus aList
		StringList& locAlleleList = alleleList_[j];
		for (int a=0;a<locAlleleList.len();++a){
			aList<long double>* alleleFstat =NEW aList<long double>();
			locAlleleFstat->Append(alleleFstat);
			for (int Flevel=0;Flevel<3;++Flevel){alleleFstat->Append((long double)0.0);}
			}//end for a							
		}//end for j

	//the main body of the calculations	
	ObjList<GeneticPopulation>& pops=popList_[group];
	long double r = (long double) (pops.len());
	long double Asum,Bsum,Csum,BCsum,ABCsum;
	Asum=Bsum=Csum=BCsum=ABCsum=(long double)0.0;
	for (int j=0;j<variableLoci_.len();++j){
		int loc = variableLoci_[j];//the actual locus
		StringList& locAlleleList = alleleList_[loc];
		for (int a=0;a<locAlleleList.len();++a){
			long double Nsq,Nc,Nbar,pBar,ssq,Hbar,A,B,C;
			Nsq=Nc=Nbar=pBar=ssq=Hbar=A=B=C=(long double)0.0;
			aString alleleVal=locAlleleList[a];
			aList<long double> q;//allele frequencies
			//Calculate for the whole population set:
			for (int p=0;p<pops.len();++p){
				GeneticPopulation& pop=pops[p];
				q.Append(pop.AlleleFrequency(loc, alleleVal));
				long double nSample = (long double) pop.SampleSize(loc);
				//average sample size among pops:
				Nbar += nSample;
				//preliminary variable for term accounting for the variance in sample sizes:
				Nsq += nSample * nSample;
				// Average allelic frequency (=pBar)
				pBar += nSample * q[p];
				//go get the observed heterozygosity for this allele and locus
				long double hObs = pop.hetObs(loc,alleleVal);
				Hbar += (nSample * hObs);
				}//end for p

			if ((r > (long double)1.0)&&(Nbar>(long double)0.0)) {//this weeds out zero denominators
				Nbar /= r;
				Nc = (r * Nbar - Nsq / (r * Nbar)) / (r - (long double)1.0);
				pBar /= (r * Nbar);
				Hbar /= (r * Nbar);

				//Now get the term accounting for variance among allelic frequencies
				for (int p=0;p<pops.len();++p){
					GeneticPopulation& pop=pops[p];
					long double nSample = (long double) pop.SampleSize(loc);
					long double freq=q[p];
					ssq += (nSample * (freq - pBar) * (freq - pBar));
					}//end for p
				ssq /= ((r - 1) * Nbar);

				//Calculate final variables in steps, because the equations are long:
				if ((Nc != (long double)0.0)&&(Nbar != (long double)1.0)){//this weeds out zero denominators
					A = (pBar * ((long double)1.0 - pBar)) - (ssq * ((r - (long double)1.0) / r)) - (Hbar / (long double)4.0);
					A = (Nbar / Nc) * (ssq - (A / (Nbar - 1)));
					}//end if Nc & Nbar

				if (Nbar != (long double)1.0){//this weeds out zero denominators
					B = Hbar * (((long double)2.0 * Nbar - (long double)1.0) / ((long double)4.0 * Nbar));
					B = (pBar * ((long double)1.0 - pBar)) - (ssq * ((r - (long double)1.0) / r)) - B;
					B *= (Nbar / (Nbar - (long double)1.0));
					}//end if Nbar!=1
				C = Hbar / (long double)2.0;
				}//end if r<>0, r<>1, Nbar<>0

			//A, B, &C remain zero if their components have any zero denominators
			ObjList<aList<long double> >& locAlleleFstat=FstatsAll_[loc];
			aList<long double>& alleleFstat =locAlleleFstat[a];
			if ((A + B + C) != (long double)0.0) {
				aString levelName="Fst";
				int Flevel = Flevels_.Loc(&levelName);
				alleleFstat.Replace(A / (A + B + C), Flevel);
				levelName="Fit";
				Flevel = Flevels_.Loc(&levelName);
				alleleFstat.Replace((long double)1.0 - (C / (A + B + C)),Flevel);
				}//end if A+B+C
			if ((B + C) != (long double)0.0) {
				aString levelName="Fis";
				int Flevel = Flevels_.Loc(&levelName);
				alleleFstat.Replace((long double)1.0 - (C / (B + C)), Flevel);
				}//end if (B+C)
			//save these in an array for a quicker calculation of the jackknife variance estimates
			ALoc->Replace(ALoc->Nth(loc)+A,loc);
			BLoc->Replace(BLoc->Nth(loc)+B,loc);
			CLoc->Replace(CLoc->Nth(loc)+C,loc);
			BCLoc->Replace(BCLoc->Nth(loc)+B+C,loc);
			ABCLoc->Replace(ABCLoc->Nth(loc)+A+B+C,loc);

			Asum += A;
			Bsum += B;
			Csum += C;
			BCsum += B + C;
			ABCsum += A + B + C;

			}//end for a
		aList<long double>& locFstat = FstatsLoc_[loc];
		long double temp = ABCLoc->Nth(loc);
		if (temp != (long double)0.0) {
			aString levelName="Fst";
			int Flevel = Flevels_.Loc(&levelName);
			locFstat.Replace(ALoc->Nth(loc) / temp, Flevel);
			levelName="Fit";
			Flevel = Flevels_.Loc(&levelName);
			locFstat.Replace((long double)1.0 - (CLoc->Nth(loc) / temp), Flevel);
			}//end if temp
		temp= BCLoc->Nth(loc);
		if (temp != 0.0){
			aString levelName="Fis";
			int Flevel = Flevels_.Loc(&levelName);
			locFstat.Replace((long double)1.0 - (CLoc->Nth(loc) / temp), Flevel);
			}//end if temp
		}//end for j

	if (ABCsum != (long double)0.0){
		aString levelName="Fst";
		int Flevel = Flevels_.Loc(&levelName);
		Fstats_.Replace((Asum / ABCsum), Flevel);
		levelName="Fit";
		Flevel = Flevels_.Loc(&levelName);
		Fstats_.Replace(((long double)1.0 - (Csum / ABCsum)), Flevel);
		}//end if ABCsum
	if (BCsum != 0.0){
		aString levelName="Fis";
		int Flevel = Flevels_.Loc(&levelName);
		Fstats_.Replace(((long double)1.0 - (Csum / BCsum)), Flevel);
		}//end if BCSum

	//now do the jackknife calculations, over loci
	for (int j=0;j<variableLoci_.len();++j){
		int loc = variableLoci_[j];//the actual locus		
		aList<long double>& locJackFstat = jackFstatsLoc_[loc];
		double temp = ABCLoc->Nth(loc);
		if ((ABCsum - temp) != (long double)0.0) {
			aString levelName="Fst";
			int Flevel = Flevels_.Loc(&levelName);
			locJackFstat.Replace(((Asum - ALoc->Nth(loc)) / (ABCsum - temp)), Flevel);
			levelName="Fit";
			Flevel = Flevels_.Loc(&levelName);
			locJackFstat.Replace((long double)1.0 - (Csum - CLoc->Nth(loc)) / (ABCsum - temp), Flevel);
			}//end if ABCsum
		temp = BCLoc->Nth(loc);
		if (BCsum - temp != (long double)0.0) {
			aString levelName="Fis";
			int Flevel = Flevels_.Loc(&levelName);
			locJackFstat.Replace((long double)1.0 - (Csum - CLoc->Nth(loc)) / (BCsum - temp), Flevel);
			}//end if BCSum
		}//end for j
	delete ALoc;
	delete BLoc;
	delete CLoc;
	delete BCLoc;
	delete ABCLoc;
	FstatsCurrent_=1;
	}//end  Fstatistics::WeirTheta



void  Fstatistics::HierarchicalWeirTheta(void){
    //this routine calculates Weir & Cockerham's hierarchical
    // F-statistics among groups of populations
    //Notation follows Wier & Cockerham 1984, p.1365
    
    //can't handle anything but diploids with all loci scored
    
    //Getting fishy results:  Fsg values don't match up with
    //Fst values from groups taken singly.  Debug this:  check
    //in the subroutines for sample size changes which might do it.
    //but remember, this happened in the Limenitis paper.
    

	if( (FstatsCurrent_)&&(FstatsType=="Hierarchical Weir & Cockerham")){return;}
	FstatsType="Hierarchical Weir & Cockerham";
	FstatsLoc_.Clear();//loc, F-level
	FstatsAll_.Clear();//loc,allele, F-level
	Fstats_.Clear();//F-level
	jackFstatsLoc_.Clear();//loc, F-level
	Flevels_.Clear();//F-level

	//set up all the F-stat arrays
	// first the array describing the Fstat structure:
	aString* FWord=NEW aString("Fit"); Flevels_.Append(FWord);
	FWord=NEW aString("Fis"); Flevels_.Append(FWord);
	FWord=NEW aString("Fig"); Flevels_.Append(FWord);
	FWord=NEW aString("Fsg"); Flevels_.Append(FWord);
	FWord=NEW aString("Fst"); Flevels_.Append(FWord);
	FWord=NEW aString("Fgt"); Flevels_.Append(FWord);
	//make and initialize the jackknife & by-locus Fstat lists
	for (int Flevel=0;Flevel<6;++Flevel){Fstats_.Append(0.0);}
	//some intermediate variables
	aList<long double>* ALoc=NEW aList<long double>();
	aList<long double>* BLoc=NEW aList<long double>();
	aList<long double>* CLoc=NEW aList<long double>();
	aList<long double>* ABLoc=NEW aList<long double>();
	aList<long double>* BCLoc=NEW aList<long double>();
	aList<long double>* BBCLoc=NEW aList<long double>();
	aList<long double>* ABBCLoc=NEW aList<long double>();
	if (!allelesCurrent_){UpdateAlleleList();}
	if (!varLociCurrent_){UpdateVariableLoci();}
	int nloci=alleleList_.len();
	for (int j=0;j<nloci;++j){
		ALoc->Append((long double)0.0);
		BLoc->Append((long double)0.0);
		CLoc->Append((long double)0.0);
		ABLoc->Append((long double)0.0);
		BCLoc->Append((long double)0.0);
		BBCLoc->Append((long double)0.0);
		ABBCLoc->Append((long double)0.0);
		aList<long double>* locFstats = NEW aList<long double>();
		FstatsLoc_.Append(locFstats);//loc, F-level
		aList<long double>* jackLocFstats = NEW aList<long double>();
		jackFstatsLoc_.Append(jackLocFstats);//loc, F-level
		for (int Flevel=0;Flevel<6;++Flevel){
			locFstats->Append((long double)0.0);
			jackLocFstats->Append(0.0);
			}//end for Flevel
		ObjList<aList<long double> >* alleleFstats = NEW ObjList<aList<long double> >();
		FstatsAll_.Append(alleleFstats);//loc,allele, F-level
		StringList& locAlleleList=alleleList_[j];
		for (int a=0;a<locAlleleList.len();++a){
			aList<long double>* locAlleleFstats = NEW aList<long double>();
			alleleFstats->Append(locAlleleFstats);
			for (int Flevel=0;Flevel<6;++Flevel){locAlleleFstats->Append(0.0);}
			}//end for a
		}//end for j
		
	long double Asum,Bsum,Csum,ABsum,BCsum,BBCsum,ABBCsum;
	Asum=Bsum=Csum=ABsum=BCsum=BBCsum=ABBCsum=(long double)0.0;
	for (int j=0;j<variableLoci_.len();++j){
		int loc=variableLoci_[j];
		StringList& locAlleleList=alleleList_[loc];
	        for(int a=0;a<locAlleleList.len();++a){
	        	aString& alleleVal = locAlleleList[a];
				StatList<long double> qDot, nDot, nSample;
	        	ObjList<StatList<long double> > allFreqs;//group, pop
            		long double mDot,qDotDot,nDotDot,MSP,MSI,MSG,MSD,n1,n2,n3,A,B1,B2,C;
            		mDot=qDotDot=nDotDot=MSP=MSI=MSG=MSD=n1=n2=n3=A=B1=B2=C=(long double)0.0;
           			for (int g=0;g<popList_.len();++g){//FOR GROUP=1 TO NGROUPS
           			ObjList<GeneticPopulation>& pops = popList_[g];
                		mDot += (long double) pops.len();// += SIZE(GROUP)
                		qDot.Append((long double)0.0);//PDOT!(GROUP)=0
                		nDot.Append((long double)0.0);//NDOT!(GROUP)=0
                		StatList<long double>* q=NEW StatList<long double>();
                		allFreqs.Append(q);
              			}//end for g
          			//Calculate for the whole population set:
           			for (int g=0;g<popList_.len();++g){//FOR GROUP=1 TO NGROUPS
           				ObjList<GeneticPopulation>& pops = popList_[g];
           				StatList<long double>& q = allFreqs[g];
           				for(int p=0;p<pops.len();++p){//FOR GPOP=1 TO SIZE(GROUP)
           					GeneticPopulation& pop=pops[p];
							long double n=(long double) pop.size();//n=nSample is for 1N sample size, not 2N
							nSample.Append(n);//nSample is for 1N sample size, not 2N
							long double freq=pop.AlleleFrequency(loc,alleleVal);
							q.Append(freq);
	                   		//Average allelic frequency (PDOTDOT!) across groups
                   			qDotDot += n*freq;

                    			//Average allelic frequency within this group (PDOT!(GROUP))
                    			qDot.Replace(qDot[g]+n*freq, g);

                    			//Total sample size per locus (NDOTDOT)
                    			nDotDot += n;

                    			//Sample size per locus within this group
                    			nDot.Replace(nDot[g]+n, g);
                    			}//next p
                    		if (nDot[g]>(long double)0.0){
                    			qDot.Replace(qDot[g]/nDot[g], g);}
                    		}//end for g
			qDotDot /= nDotDot;

            		//Calculate intermediate variables:
                	//if all the naturally occurring groups are being used, then use R=NGROUPS
                	//if just a sample of natural groups are used, use R=(NGROUPS-1)
                	long double r = popList_.len();
           		for (int g=0;g<popList_.len();++g){//FOR GROUP=1 TO NGROUPS
                    		long double NSQ=(long double)0.0;
                    		MSP += nDot[g] * (qDot[g] - qDotDot) * (qDot[g] - qDotDot);
          			ObjList<GeneticPopulation>& pops = popList_[g];
           			StatList<long double>& q = allFreqs[g];
           			for(int p=0;p<pops.len();++p){//FOR GPOP=1 TO SIZE(GROUP)
           				GeneticPopulation& pop=pops[p];
           				double n=nSample[g];
                        		MSD += n*(q[p]-qDot[g])*(q[p]-qDot[g]);
                        		//Be sure to divide heterozygosity by 2 when summing over alleles.
                        		long double Hexp = (long double)2.0*q[p]*((long double)1.0-q[p]);
                        		long double Hobs = pop.hetObs(loc,alleleVal);
                        		MSI += n*Hexp;
                        		MSI -= 0.5*n*Hobs; if(MSI==(long double)(-0.0)){ MSI=(long double)0.0;}
                        		MSG += n*Hobs;
                        		NSQ += n*n;
                        		}//end for p
                   		n1 += ((nDotDot-nDot[g])*NSQ)/(nDot[g]*nDotDot);
                   		n2 += nDot[g]*nDot[g];
                    		n3 +=  NSQ/nDot[g];
                    		}//end for g
			MSP *= (long double)2.0/(r-(long double)1.0);
               		MSD *= (long double)2.0/(mDot-r); if(MSD==(long double)(-0.0)){ MSD=(long double)0.0;}
                	MSI /= (nDotDot - mDot);if(MSI==(long double)(-0.0)){ MSI=(long double)0.0;}
                	MSG /= (long double)2.0*nDotDot;
                	n1 /= (r-(long double)1.0);if(n1==(long double)(-0.0)){ n1=(long double)0.0;}
               		n2 = (nDotDot - n2/nDotDot)/(r-(long double)1.0);if(n2==(long double)(-0.0)){ n2=(long double)0.0;}
                	n3 = (nDotDot - n3)/(mDot-r);if(n3==(long double)(-0.0)){ n3=(long double)0.0;}

                	//Calculate final variables:
                	A = (n3*MSP - n1*MSD - MSI*(n3-n1))/((long double)2.0*n2*n3); if(A==(long double)(-0.0)){ A=(long double)0.0;}
                	B1 = (MSI - MSG)/(long double)2.0; if(B1==(long double)(-0.0)){ B1=(long double)0.0;}
                	B2 = (MSD - MSI)/((long double)2.0*n3); if(B2==(long double)(-0.0)){ B2=(long double)0.0;}
                	C = MSG; if(C==(long double)(-0.0)){ C=(long double)0.0;}

                	//store Fstat values for each allele:
			ObjList<aList<long double> >& alleleFstats =FstatsAll_[loc];
			aList<long double>& locAlleleFstats = alleleFstats[a];
                	if ((A+B1+B2+C)!=(long double)0.0) {
				aString levelName="Fgt"; 
				int Flevel = Flevels_.Loc(&levelName);
				locAlleleFstats.Replace(A/(A+B1+B2+C), Flevel);//WEIRFgtALL!(L,M)= A/(A+B1+B2+C)
				levelName="Fst"; 
				Flevel = Flevels_.Loc(&levelName);
				locAlleleFstats.Replace((A+B2)/(A+B1+B2+C), Flevel);//WEIRFstALL!(L,M)=(A+B2)/(A+B1+B2+C)
				levelName="Fit"; 
				Flevel = Flevels_.Loc(&levelName);
				locAlleleFstats.Replace((long double)1.0-C/(A+B1+B2+C), Flevel);//WEIRFitALL!(L,M)=1.0-C/(A+B1+B2+C)
				}//end if A+B1+B2+C
                	if ((B1+B2+C)!=(long double)0.0) {
				aString levelName="Fig"; int Flevel = Flevels_.Loc(&levelName);
				locAlleleFstats.Replace((long double)1.0-C/(B1+B2+C), Flevel);//WEIRFigALL!(L,M)=1.0-C/(B1+B2+C)
				levelName="Fsg"; Flevel = Flevels_.Loc(&levelName);
				locAlleleFstats.Replace((long double)1.0-(B1+C)/(B1+B2+C), Flevel);//WEIRFsgALL!(L,M)=1.0-(B1+C)/(B1+B2+C)
				}//end if (B1+B2+C)
                	if ((B1+C)!=(long double)0.0) {
				aString levelName="Fis"; int Flevel = Flevels_.Loc(&levelName);
				locAlleleFstats.Replace((long double)1.0-C/(B1+C), Flevel);//WEIRFisALL!(L,M)=1.0-C/(B1+C)
                		}//end if (B1+C)

                	//accumulate locus sums for weighted averages & jackknife
                	ALoc->Replace(ALoc->Nth(loc) + A,loc);
                	BLoc->Replace(BLoc->Nth(loc) + B2,loc);
                	CLoc->Replace(CLoc->Nth(loc) + C,loc);
                	ABLoc->Replace(ABLoc->Nth(loc) + A+B2,loc);
                	BCLoc->Replace(BCLoc->Nth(loc) + B1+C,loc);
                	BBCLoc->Replace(BBCLoc->Nth(loc) + B1+B2+C,loc);
                	ABBCLoc->Replace(ABBCLoc->Nth(loc) + A+B1+B2+C,loc);

	                //accumulate overall sums for weighted averages
	                Asum += A;
                	Bsum += B2;
                	Csum += C;
                	ABsum += A+B2;
                	BCsum += BCsum+B1+C;
                	BBCsum += BBCsum+B1+B2+C;
                	ABBCsum += A+B1+B2+C;
                	}//end for a

		//store Fstat values for each locus:
		aList<long double>& locFstats = FstatsLoc_[loc];//loc, F-level
                if (ABBCLoc->Nth(loc) != (long double)0.0) {
			aString levelName="Fgt"; int Flevel = Flevels_.Loc(&levelName);
			locFstats.Replace(ALoc->Nth(loc)/ABBCLoc->Nth(loc), Flevel);//WEIRFgtLOC!(L)=ALOC!(L)/ABBCLOC!(L)
			levelName="Fst"; Flevel = Flevels_.Loc(&levelName);
			locFstats.Replace(ABLoc->Nth(loc)/ABBCLoc->Nth(loc), Flevel);//WEIRFstLOC!(L)=ABLOC!(L)/ABBCLOC!(L)
			levelName="Fit"; Flevel = Flevels_.Loc(&levelName);
			locFstats.Replace((long double)1.0-CLoc->Nth(loc)/ABBCLoc->Nth(loc), Flevel);//WEIRFitLOC!(L)=1-(CLOC!(L)/ABBCLOC!(L))
			}//end if ABBCLoc->Nth(loc)
                if (BBCLoc->Nth(loc) != (long double)0.0) {
			aString levelName="Fig"; int Flevel = Flevels_.Loc(&levelName);
			locFstats.Replace((long double)1.0-CLoc->Nth(loc)/BBCLoc->Nth(loc), Flevel);//WEIRFigLOC!(L)=1-(CLOC!(L)/BBCLOC!(L))
			levelName="Fsg"; Flevel = Flevels_.Loc(&levelName);
			locFstats.Replace((long double)1.0-BCLoc->Nth(loc)/BBCLoc->Nth(loc), Flevel);//WEIRFsgLOC!(L)=1-(BCLOC!(L)/BBCLOC!(L))
			}//end if BBCLoc->Nth(loc)
               	if (BCLoc->Nth(loc) != (long double)0.0) {
			aString levelName="Fis"; int Flevel = Flevels_.Loc(&levelName);
			locFstats.Replace((long double)1.0-CLoc->Nth(loc)/BCLoc->Nth(loc), Flevel);//WEIRFisLOC!(L)=1-(CLOC!(L)/BCLOC!(L))
                	}//end if BCLoc->Nth(loc)
                }//next j

	//store overall Fstat values:
	if (ABBCsum != (long double)0.0) {
		aString levelName="Fgt"; int Flevel = Flevels_.Loc(&levelName);
		Fstats_.Replace(Asum/ABBCsum, Flevel);//WEIRFgt!=ASUM!/ABBCSUM!
		levelName="Fst"; Flevel = Flevels_.Loc(&levelName);
		Fstats_.Replace(ABsum/ABBCsum, Flevel);//WEIRFst!=ABSUM!/ABBCSUM!
		levelName="Fit"; Flevel = Flevels_.Loc(&levelName);
		Fstats_.Replace((long double)1.0-Csum/ABBCsum, Flevel);//WEIRFit!=1-(CSUM!/ABBCSUM!)
		}//end if ABBCsum
	if (BBCsum != (long double)0.0) {
		aString levelName="Fig"; int Flevel = Flevels_.Loc(&levelName);
		Fstats_.Replace((long double)1.0-Csum/BBCsum, Flevel);// WEIRFig!=1-(CSUM!/BBCSUM!)
		levelName="Fsg"; Flevel = Flevels_.Loc(&levelName);
		Fstats_.Replace((long double)1.0-BCsum/BBCsum, Flevel);//WEIRFsg!=1-(BCSUM!/BBCSUM!)
		}//end if BBCsum
	if (BCsum != (long double)0.0) {
		aString levelName="Fis"; int Flevel = Flevels_.Loc(&levelName);
		Fstats_.Replace((long double)1.0-Csum/BCsum, Flevel);//WEIRFis!=1-(CSUM!/BCSUM!)
		}//end if BCsum



	//for jackknife variance estimates
	for (int j=0;j<variableLoci_.len();++j){
		int loc=variableLoci_[j];
		aList<long double>& jackLocFstats = jackFstatsLoc_[loc];//loc, F-level
		if (ABBCsum !=ABBCLoc->Nth(loc)){
			aString levelName="Fgt"; int Flevel = Flevels_.Loc(&levelName);
			long double val=(Asum-ALoc->Nth(loc))/(ABBCsum-ABBCLoc->Nth(loc));
			jackLocFstats.Replace(val, Flevel);
			levelName="Fst"; Flevel = Flevels_.Loc(&levelName);
			val=(ABsum-ABLoc->Nth(loc))/(ABBCsum-ABBCLoc->Nth(loc));
			jackLocFstats.Replace(val, Flevel);
			levelName="Fit"; Flevel = Flevels_.Loc(&levelName);
			val=(long double)1.0-(Csum-CLoc->Nth(loc))/(ABBCsum-ABBCLoc->Nth(loc));
			jackLocFstats.Replace(val, Flevel);//WEIRFit!=1-(CSUM!/ABBCSUM!)
			}//end if ABBCsum
    	if (BBCsum != BBCLoc->Nth(loc)) {
			aString levelName="Fig"; int Flevel = Flevels_.Loc(&levelName);
			long double val=(long double)1.0 - (Csum-CLoc->Nth(loc))/(BBCsum-BBCLoc->Nth(loc));
			jackLocFstats.Replace(val, Flevel);
			levelName="Fsg"; Flevel = Flevels_.Loc(&levelName);
			val=(long double)1.0-(BCsum-BCLoc->Nth(loc))/(BBCsum-BBCLoc->Nth(loc));
			jackLocFstats.Replace(val, Flevel);
			}//end if BBCsum
		if (BCsum != BCLoc->Nth(loc)) {
			aString levelName="Fis"; int Flevel = Flevels_.Loc(&levelName);
			long double val=1.0 - (Csum-CLoc->Nth(loc))/(BCsum-BCLoc->Nth(loc));
			jackLocFstats.Replace(val, Flevel);
			}//end if BCsum
		}//end for j
	FstatsCurrent_=1;
	}//end  Fstatistics::HierarchicalWeirTheta


int operator== (Fstatistics& a, Fstatistics& b){
	return (a.popList_==b.popList_);}


int operator!= (Fstatistics& a, Fstatistics& b){
	return !(a==b);}

#endif







