/******************************************************
*
*	Fis_Fit_Q_DomParamRec.cpp
*
*	Parameters for estimating Fis or Fit and allele frequencies Q using a binomial distribution.
*	Use for diploid sex-linked/haplodiploid markers only
*
******************************************************/
#ifndef Fis_Fit_Q_DomParamReccpp
#define Fis_Fit_Q_DomParamReccpp = 1

#include "Fis_Fit_Q_DomParamRec.h"
#include "Element.cpp"
#include "aList.cpp"


Fis_Fit_Q_DomParamRec::Fis_Fit_Q_DomParamRec(void):Fis_Fit_Q_ParamRec(),
					numqHeteroParams_(0), numIndependentqHeteroParams_(0),numQHeteroParams_(0), numIndependentQHeteroParams_(0){
  	SetPointerToDerivedSelf();
//  	cout<<"constructing Fis_Fit_Q_DomParamRec()"<<endl;
  	}


Fis_Fit_Q_DomParamRec::Fis_Fit_Q_DomParamRec(long double Fis_val, long double Fit_val, long double sr,
								 ObjList<ObjList<aList<long double> > >& init_q_estimated, 
								 ObjList<ObjList<aList<long double> > >& init_qHeterogametic_estimated){
	lnL_Fis_=(long double)0.0;
	lnL_Fit_=(long double)0.0;
	Fis_=Fis_val;
	Fit_=Fit_val;
	Fst_=(long double)0.0;
	sexRatio_=sr;
	q_estimated_=init_q_estimated;
	q_hetero_estimated_=init_qHeterogametic_estimated;
	SetNumqParams();
	InitializeQ();
	SetFst();
  	SetPointerToDerivedSelf();
	ResetParamChoices();
	paramChoices_.Clear();
	for(int i=0;i<paramsToChoose_;++i){
		paramChoices_.Append(i);}
  	}


Fis_Fit_Q_DomParamRec::Fis_Fit_Q_DomParamRec(long double Fis_val, long double Fit_val, long double sr,
								 ObjList<ObjList<aList<long double> > >& init_q_estimated, 
								 ObjList<ObjList<aList<long double> > >& init_qHeterogametic_estimated,
								 ObjList<aList<long double> >& init_QHetero_estimated_){
	lnL_Fis_=(long double)0.0;
	lnL_Fit_=(long double)0.0;
	Fis_=Fis_val;
	Fit_=Fit_val;
	Fst_=(long double)0.0;
	sexRatio_=sr;
	q_estimated_=init_q_estimated;
	q_hetero_estimated_=init_qHeterogametic_estimated;
	QHetero_estimated_=init_QHetero_estimated_;
	SetNumqParams();
//	InitializeQ();
	numQHeteroParams_=0;
	int loc,all;
//	long double freq;
	for(loc=0;loc<q_hetero_estimated_[0].len();++loc){
		for(all=0;all<q_hetero_estimated_[0][loc].len();++all){
			++numQHeteroParams_;}}
	numIndependentQHeteroParams_ = numQHeteroParams_ - q_hetero_estimated_[0].len();

	SetFst();
  	SetPointerToDerivedSelf();
	ResetParamChoices();
	paramChoices_.Clear();
	for(int i=0;i<paramsToChoose_;++i){
		paramChoices_.Append(i);}
  	}


Fis_Fit_Q_DomParamRec::Fis_Fit_Q_DomParamRec(Fis_Fit_Q_DomParamRec& p):
  					Fis_Fit_Q_ParamRec(p),
  					q_hetero_estimated_(p.q_hetero_estimated_),
					numqHeteroParams_(p.numqHeteroParams_), 
					numIndependentqHeteroParams_(p.numIndependentqHeteroParams_),
					QHetero_estimated_(p.QHetero_estimated_),
					numQHeteroParams_(p.numQHeteroParams_), 
					numIndependentQHeteroParams_(p.numIndependentQHeteroParams_){
  	SetPointerToDerivedSelf();
	}
	

Fis_Fit_Q_DomParamRec::~Fis_Fit_Q_DomParamRec(void){
	q_hetero_estimated_.Clear();
	QHetero_estimated_.Clear();
  	pointerToDerivedSelf = NULL;//forget
  	}

	
ParamRec* Fis_Fit_Q_DomParamRec::Duplicate(void){
	Fis_Fit_Q_DomParamRec* copy = NEW Fis_Fit_Q_DomParamRec(*this);
	return copy;
	}//end Fis_Fit_Q_DomParamRec::Duplicate()


Fis_Fit_Q_DomParamRec* Fis_Fit_Q_DomParamRec::_Duplicate(void){
	Fis_Fit_Q_DomParamRec* copy = NEW Fis_Fit_Q_DomParamRec(*this);
	return copy;
	}//end Fis_Fit_Q_DomParamRec::_Duplicate()


void Fis_Fit_Q_DomParamRec::SetPointerToDerivedSelf(void){
  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in Fis_Fit_Q_DomParamRec"<<endl;
  	}//end


	
void Fis_Fit_Q_DomParamRec::SetNumqParams(void){
	Fis_Fit_Q_ParamRec::SetNumqParams();
	numqHeteroParams_=numIndependentqHeteroParams_=0;
	int loci_estimated=0;
	for(int pop=0;pop<q_hetero_estimated_.len();++pop){
		for(int loc=0;loc<q_hetero_estimated_[pop].len();++loc){
			loci_estimated++;
			numqHeteroParams_ += q_hetero_estimated_[pop][loc].len();}}
	numIndependentqHeteroParams_ = numqHeteroParams_-loci_estimated;
	}//end Fis_Fit_Q_DomParamRec::SetNumqParams()



void Fis_Fit_Q_DomParamRec::SetNumQParams(void){
	Fis_Fit_Q_ParamRec::SetNumQParams();
	int numQHeteroParams_=numIndependentQHeteroParams_=0;
	int loci_estimated=0;
	for(int loc=0;loc<QHetero_estimated_.len();++loc){
		loci_estimated++;
		numQHeteroParams_ += QHetero_estimated_[loc].len();}
	numIndependentQHeteroParams_= numQHeteroParams_-loci_estimated;
	}//end Fis_Fit_Q_DomParamRec::SetNumQParams()


long double Fis_Fit_Q_DomParamRec::Q_hetero(int loc, int allele){
	//returns global estimated average
	return QHetero_estimated_[loc][allele];
	}//end Fis_Fit_Q_DomParamRec::Q_hetero(int, int)


long double Fis_Fit_Q_DomParamRec::q_hetero(int pop, int loc, int allele){
	//returns local estimated frequency; 0:recessive, 1:dominant
	long double qq = q_hetero_estimated_[pop][loc][allele];
	return qq;
	}//end Fis_Fit_Q_DomParamRec::q_hetero(int,int,int)


int Fis_Fit_Q_DomParamRec::Change_q_hetero(int paramNum, Ran3& ran, Fis_Fit_Q_DomParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary q
	//return 0 for failure, 1 for success
	int validChange=false, found=false, thepop=0, theloc=0, theAllele=0, p = paramNum;
	int pcount=0;
	for(int pop=0;pop<q_hetero_estimated_.len();++pop){
		for(int loc=0;loc<q_hetero_estimated_[pop].len();++loc){
			for(int a=0;a<q_hetero_estimated_[pop][loc].len()-1;++a){
				if(pcount==p){
					thepop = pop; theloc=loc; theAllele=a; 
					found=true; break;
					}
				pcount++;
				}//end for a
			if(found){break;}
			}//end for loc
		if(found){break;}
		}//end for pop
	long double old_q = q_hetero_estimated_[thepop][theloc][theAllele], new_q;
	aList<long double> old_qList = q_hetero_estimated_[thepop][theloc];
	long double sigma = paramDeviations->q_hetero_estimated_[thepop][theloc][theAllele];
	do{
		q_hetero_estimated_[thepop][theloc]=old_qList;//restore
		new_q = q_hetero_estimated_[thepop][theloc][theAllele] = ran.NormDev(old_q, sigma);
		if(new_q<(long double)0.0){
			validChange=false; continue;}
		SetDependentAllele_q_hetero(thepop,theloc);
		if(q_hetero_estimated_[thepop][theloc].Last()<(long double)0.0){
			validChange=false; continue;}
		//now adjust remaining alleles to compensate
/*		double freqRemaining = 1.0-new_q;
		aList<double> partialFreqs;aList<int> geneOrder;
		for(int a=0;a<q_estimated_[thepop][theloc].len();++a){
			if(a != theAllele){ 
				partialFreqs.Append(q_estimated_[thepop][theloc][a]);
				geneOrder.Append(a);
				}//end if
			}//end for a
		for(int aa=0;aa<geneOrder.len()-1;++aa){
			double new_qAdj=0.0;
			while ((new_qAdj>=0.0)&&(new_qAdj<1.0)){
				new_qAdj = ran.NormDev(partialFreqs[aa]/freqRemaining, sigma/freqRemaining);}
			q_estimated_[thepop][theloc][geneOrder[aa]]= new_qAdj*freqRemaining;
			freqRemaining -= new_qAdj*freqRemaining;
			}//end for aa
		if(geneOrder.len()>0){
			q_estimated_[thepop][theloc][geneOrder.Last()] = freqRemaining;}
		
*/		
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end Fis_Fit_Q_DomParamRec::Change_q_hetero(Ran3&, double, ParamRec*, ParamRec*)


int Fis_Fit_Q_DomParamRec::Change_Q_hetero(int paramNum, Ran3& ran, Fis_Fit_Q_DomParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary Q
	//return 0 for failure, 1 for success
	int validChange=false, found=false, theloc=0, theAllele=0, p = paramNum;
	int pcount=0;
	for(int loc=0;loc<QHetero_estimated_.len();++loc){
		for(int a=0;a<QHetero_estimated_[loc].len()-1;++a){
			if(pcount==p){
				theloc=loc; theAllele=a; 
				found=true; break;
				}
			pcount++;
			}//end for a
		if(found){break;}
		}//end for loc
	long double old_Q = QHetero_estimated_[theloc][theAllele], new_Q;
	aList<long double> old_QList = QHetero_estimated_[theloc];
	long double sigma = paramDeviations->QHetero_estimated_[theloc][theAllele];
	do{
		QHetero_estimated_[theloc]=old_QList;//restore
		new_Q = QHetero_estimated_[theloc][theAllele] = ran.NormDev(old_Q, sigma);
		if(new_Q<(long double)0.0){
			validChange=false; continue;}
		SetDependentAllele_Q_hetero(theloc);
		if(QHetero_estimated_[theloc].Last()<(long double)0.0){
			validChange=false; continue;}
		//now adjust remaining alleles to compensate
/*		double freqRemaining = 1.0-new_q;
		aList<double> partialFreqs;aList<int> geneOrder;
		for(int a=0;a<q_estimated_[thepop][theloc].len();++a){
			if(a != theAllele){ 
				partialFreqs.Append(q_estimated_[thepop][theloc][a]);
				geneOrder.Append(a);
				}//end if
			}//end for a
		for(int aa=0;aa<geneOrder.len()-1;++aa){
			double new_qAdj=0.0;
			while ((new_qAdj>=0.0)&&(new_qAdj<1.0)){
				new_qAdj = ran.NormDev(partialFreqs[aa]/freqRemaining, sigma/freqRemaining);}
			q_estimated_[thepop][theloc][geneOrder[aa]]= new_qAdj*freqRemaining;
			freqRemaining -= new_qAdj*freqRemaining;
			}//end for aa
		if(geneOrder.len()>0){
			q_estimated_[thepop][theloc][geneOrder.Last()] = freqRemaining;}
		
*/		
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end Fis_Fit_Q_DomParamRec::Change_Q_hetero(Ran3&, double, ParamRec*, ParamRec*)



int Fis_Fit_Q_DomParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double /*weight*/){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	Fis_Fit_Q_DomParamRec* deviations = (Fis_Fit_Q_DomParamRec*) paramDeviations->pointerToDerivedSelf;
	switch (paramNum){
	 case 0:
	 	 return ChangeFis(ran, deviations->Fis(), lowerLimits, upperLimits);
	 	 break;
	 case 1:
	 	 return ChangeFit(ran, deviations->Fit(), lowerLimits, upperLimits);
	 	 break;
	 default: break;}
	if(sexRatioVaries_ && paramNum==2){
	 	 return ChangeSexRatio(ran, deviations->SexRatio(), lowerLimits, upperLimits);}
	int origParamNum = paramNum;
	paramNum -= 2;
	if(sexRatioVaries_){
		--paramNum;}
	if(paramNum<NumqParams()){
	 	 return Change_q(paramNum, ran, deviations, lowerLimits, upperLimits);
	 	 }
	paramNum -= NumIndependentqParams();
	if(paramNum<NumQParams()){
	 	 return Change_Q(paramNum, ran, deviations, lowerLimits, upperLimits);
	 	 }
	paramNum -= NumIndependentQParams();
	if(paramNum<NumqHeteroParams()){
	 	 return Change_q_hetero(paramNum, ran, deviations, lowerLimits, upperLimits);
	 	 }
	paramNum -= NumIndependentqHeteroParams();
	if(paramNum<NumQHeteroParams()){
	 	 return Change_Q_hetero(paramNum, ran, deviations, lowerLimits, upperLimits);
	 	 }
	paramNum -= NumIndependentqParams();
	aString errstr("bug: can't change param #");
		errstr.cat(origParamNum+1);
		errstr.cat(" in a ");errstr.cat(NumParams());
		errstr.cat("-parameter model fitting Fis, Fit & allele frequencies");
	aError(errstr);
	 return 0;
	}//end Fis_Fit_Q_DomParamRec::ChangeParam(int)

/*
int Fis_Fit_Q_DomParamRec::ChangeParams(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	int validchange = false;
	int paramToChange = RandomParameter(ran);
	do{
		validchange = ChangeParam(paramToChange, ran, 
										paramDeviations, lowerLimits, upperLimits);
		}while(!(validchange));
	return paramToChange;
	}//end Fis_Fit_Q_DomParamRec::ChangeParams(Ran3&, ParamRec*, ParamRec*, ParamRec*)
*/

int Fis_Fit_Q_DomParamRec::ChangeAnnealingParams(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){

	if(qParamsVary_){
		return ParamRec::ChangeParams(ran,paramDeviations,lowerLimits, upperLimits);}

	int validchange = false, paramToChange, tries=0;
//	SetQtoVary(false);
	do{
		if(sexRatioVaries_ && ran()<0.33333333){
				paramToChange = 2;}
			else{
//				if(Fis_==0.0 && Fit_==0.0){
//						paramToChange = 1;}//just Fit_, to keep Fst_ legal
				if(Fis_== Fit_ && Fit_<(long double)1.0){
						paramToChange = 1;}//just Fit, to keep Fst legal
					else if(Fis_== Fit_ && Fit_==(long double)1.0){
						paramToChange = 0;}//just Fis, to keep Fst legal
					else{
						paramToChange = ran.longBetween(0,1);}//just Fis and Fit
				}
//		paramToChange = 0;//debug -Fis only

		validchange = ChangeParam(paramToChange, ran, 
										paramDeviations, lowerLimits, upperLimits,1.0);
		if(! validchange){
			tries++;
			if(tries>100){//bail out
				return -1;}
			long double val = ParamValue(paramToChange);
			long double devVal = paramDeviations->ParamValue(paramToChange);
			if(0.9*devVal >val/10000.0){//set a minimum deviation scaled to the parameter's value
				paramDeviations->MultiplyParam(paramToChange,0.9);}
				}
		}while(!(validchange));
	return paramToChange;

//	return ChangeParams(ran, paramDeviations,lowerLimits, upperLimits);
	}//end Fis_Fit_Q_ParamRec::ChangeAnnealingParams(Ran3&, ParamRec*, ParamRec*, ParamRec*)




void Fis_Fit_Q_DomParamRec::InitializeQ(void){//calculate means from pop frequencies
	Fis_Fit_Q_ParamRec::InitializeQ();
	QHetero_estimated_.Clear();
	numQHeteroParams_=0;
	int pop,loc,all;
	long double freq;
	for(loc=0;loc<q_hetero_estimated_[0].len();++loc){
		QHetero_estimated_.Append(NEW aList<long double>());
		for(all=0;all<q_hetero_estimated_[0][loc].len();++all){
			freq=(long double)0.0;
			++numQHeteroParams_;
			for(pop=0;pop<q_hetero_estimated_.len();++pop){
				freq += q_hetero_estimated_[pop][loc][all];}
			freq /= (long double) q_hetero_estimated_.len();
			QHetero_estimated_[loc].Append(freq);
			}//all
		SetDependentAllele_Q_hetero(loc);//in case they don't add up to 1.0
		}//loc
	numIndependentQHeteroParams_ = numQHeteroParams_ - q_hetero_estimated_[0].len();
	}//Fis_Fit_Q_DomParamRec::InitializeQ()
			


int Fis_Fit_Q_DomParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//use type conversion that circumvents usual virtual base class restrictions
	Fis_Fit_Q_DomParamRec* lower = (Fis_Fit_Q_DomParamRec*) lowerLimits->pointerToDerivedSelf;
	Fis_Fit_Q_DomParamRec* upper = (Fis_Fit_Q_DomParamRec*) upperLimits->pointerToDerivedSelf;
	long double asum=0;
	int loc,a,pop;
//	return Fis_Fit_Q_ParamRec::MeetsParamCriteria (lowerLimits, upperLimits);
	if(Fis_ < lower->Fis_){return 0;}
		else if (Fis_ > upper->Fis_){return 0;}
		else {}
	if(Fit_ < lower->Fit_){return 0;}
		else if (Fit_ > upper->Fit_){return 0;}
		else {}
	if(Fst_ < lower->Fst_){return 0;}
		else if (Fst_ > upper->Fst_){return 0;}
		else {}
	if(sexRatioVaries_){
		if(sexRatio_ <= lower->sexRatio_){return 0;}
			else if (sexRatio_ > upper->sexRatio_){return 0;}//all female population OK by default
			else {}}
	if(! qParamsVary_) {return 1;}
	for(pop=0;pop<q_estimated_.len();++pop){
		for(loc=0;loc<q_estimated_[pop].len();++loc){
			long double asum=0;
			for(a=0;a<q_estimated_[pop][loc].len();++a){
				if((q_estimated_[pop][loc][a]<(long double)0.0)||(q_estimated_[pop][loc][a]>(long double)1.0)){
					return 0;}
				asum += q_estimated_[pop][loc][a];
				}//a
			if(asum != (float) 1.0){//handles rounding error
				return 0;}
			}}
		for(loc=0;loc<Q_estimated_.len();++loc){
			long double asum=0;
			for(a=0;a<Q_estimated_[loc].len();++a){
				if((Q_estimated_[loc][a]<(long double)0.0)||(Q_estimated_[loc][a]>(long double)1.0)){
					return 0;}
				asum += Q_estimated_[loc][a];
				}//a
			if(asum != (float) 1.0){//handles rounding error
				return 0;}
			}	
	for(pop=0;pop<q_hetero_estimated_.len();++pop){
		for(loc=0;loc<q_hetero_estimated_[pop].len();++loc){
			asum=(long double)0.0;
			for(a=0;a<q_hetero_estimated_[pop][loc].len();++a){
				if((q_hetero_estimated_[pop][loc][a]<(long double)0.0)||(q_hetero_estimated_[pop][loc][a]>(long double)1.0)){
					return 0;}
				asum += q_hetero_estimated_[pop][loc][a];
				}//a
			if(asum != (float) 1.0){//handles rounding error
				return 0;}
			}}
		for(loc=0;loc<QHetero_estimated_.len();++loc){
			asum=(long double)0.0;
			for(a=0;a<QHetero_estimated_[loc].len();++a){
				if((QHetero_estimated_[loc][a]<(long double)0.0)||(QHetero_estimated_[loc][a]>(long double)1.0)){
					return 0;}
				asum += QHetero_estimated_[loc][a];
				}//a
			if(asum != (float) 1.0){//handles rounding error
				return 0;}
			}	
	return 1;
	}//end Fis_Fit_Q_DomParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void Fis_Fit_Q_DomParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	switch (paramNum){
	 case 0:
	 	 Fis_ *= factor;
	 	 SetFst();
	 	 return; break;
	 case 1:
	 	 Fit_ *= factor;
	 	 SetFst();
	 	 return; break;
	 default: break;}
	if(sexRatioVaries_ && paramNum==2){
	 	 sexRatio_ *= factor;
	 	 return;}
	int origParamNum = paramNum, pop,loc;
	paramNum -= 2;
	for(pop=0;pop<q_estimated_.len();++pop){
		for(loc=0;loc<q_estimated_[pop].len();++loc){
			if(paramNum<q_estimated_[pop][loc].len()){
					q_estimated_[pop][loc][paramNum] *= factor;
					return;}
				else{paramNum -= q_estimated_[pop][loc].len()-1;}
			}}
	for(loc=0;loc<Q_estimated_.len();++loc){
		if(paramNum<Q_estimated_[loc].len()){
				Q_estimated_[loc][paramNum] *= factor;
				return;}
			else{paramNum -= Q_estimated_[loc].len()-1;}
		}
	for(pop=0;pop<q_hetero_estimated_.len();++pop){
		for(loc=0;loc<q_hetero_estimated_[pop].len();++loc){
			if(paramNum<q_hetero_estimated_[pop][loc].len()){
					q_hetero_estimated_[pop][loc][paramNum] *= factor;
					return;}
				else{paramNum -= q_hetero_estimated_[pop][loc].len()-1;}
			}}
	for(loc=0;loc<QHetero_estimated_.len();++loc){
		if(paramNum<QHetero_estimated_[loc].len()){
				QHetero_estimated_[loc][paramNum] *= factor;
				return;}
			else{paramNum -= QHetero_estimated_[loc].len()-1;}
		}
	aString errstr("Fis_Fit_Q_DomParamRec::MultiplyParam(): can't rescale param #");
	errstr.cat(origParamNum+1);
	errstr.cat(" in a ");errstr.cat(NumParams());errstr.cat("-parameter model");
	aError(errstr);
	}//Fis_Fit_Q_DomParamRec::MultiplyParam(int, long double)



int Fis_Fit_Q_DomParamRec::NumDerivedParams(void){//Fst_ & last allele freq of each locus in each pop is a derived param
	int nd = Fis_Fit_Q_ParamRec::NumDerivedParams();
	if(qParamsVary_){
		nd += NumqHeteroParams() - NumIndependentqHeteroParams()  + NumQHeteroParams() - NumIndependentQHeteroParams();}
	return nd;
	}//end Fis_Fit_Q_DomParamRec::NumDerivedParams()


int Fis_Fit_Q_DomParamRec::NumParams(void){//independently varying parameters
	int np = Fis_Fit_Q_ParamRec::NumParams();
	if(qParamsVary_){
		np += NumIndependentqHeteroParams() + NumIndependentQHeteroParams();}
	return np;
	}//end Fis_Fit_Q_DomParamRec::NumParams()



aString* Fis_Fit_Q_DomParamRec::ParamName(int paramNum){//includes derived parameters
	switch (paramNum){
	 case 0://primary parameters
	 	 return NEW aString("Fis");
	 	 break;
	 case 1://primary parameters
	 	 return NEW aString("Fit");
	 	 break;
	 	default: break;}
	if(sexRatioVaries_ && paramNum==2){
	 	 return NEW aString("sex ratio");
	 	 }
	int origParamNum = paramNum;
	paramNum -= 2;
	if(sexRatioVaries_){
		--paramNum;}
	switch(paramNum){
		case 0:
		 	 return NEW aString("Fst");
		 	 break;
		case 1:
		 	 return NEW aString("lnL[Fis]");
		 	 break;
		case 2:
		 	 return NEW aString("lnL[Fit]");
		 	 break;
		default: break; }
	paramNum-=3;
	int pop,loc;
	if(qParamsVary_){
		for(pop=0;pop<q_estimated_.len();++pop){
			for(loc=0;loc<q_estimated_[pop].len();++loc){
				if(paramNum<q_estimated_[pop][loc].len()){
					aString *qstr = NEW aString("q_homogam(est) pop[");
					aString popNum(pop+1);
					aString maxpNum(q_estimated_.len()+1);
					popNum.PrependCharToLen(' ',maxpNum.len());
					qstr->cat(popNum);qstr->cat("] loc[");
					aString locNum(loc+1);
					aString maLNum(q_estimated_[pop].len()+1);
					locNum.PrependCharToLen(' ',maLNum.len());
					qstr->cat(locNum);qstr->cat("] allele");
					aString allNum(paramNum+1);
					aString maxLNum(q_estimated_[pop][loc].len()+1);
					locNum.PrependCharToLen(' ',maxLNum.len());
					qstr->cat(allNum);qstr->cat("]");
					return qstr;}
				else{paramNum -= q_estimated_[pop][loc].len();}
			}}
			
		for(loc=0;loc<Q_estimated_.len();++loc){
			if(paramNum<Q_estimated_[loc].len()){
				aString *Qstr = NEW aString("Q_homogam(est)loc[");
				aString locNum(loc+1);
				aString maLNum(Q_estimated_.len()+1);
				locNum.PrependCharToLen(' ',maLNum.len());
				Qstr->cat(locNum);Qstr->cat("] allele");
				aString allNum(paramNum+1);
				aString maxLNum(Q_estimated_[loc].len()+1);
				locNum.PrependCharToLen(' ',maxLNum.len());
				Qstr->cat(allNum);Qstr->cat("]");
				return Qstr;}
			else{paramNum -= Q_estimated_[loc].len();}
			}
		for(pop=0;pop<q_hetero_estimated_.len();++pop){
			for(loc=0;loc<q_hetero_estimated_[pop].len();++loc){
				if(paramNum<q_hetero_estimated_[pop][loc].len()){
					aString *qstr = NEW aString("q_heterogam(est) pop[");
					aString popNum(pop+1);
					aString maxpNum(q_hetero_estimated_.len()+1);
					popNum.PrependCharToLen(' ',maxpNum.len());
					qstr->cat(popNum);qstr->cat("] loc[");
					aString locNum(loc+1);
					aString maLNum(q_hetero_estimated_[pop].len()+1);
					locNum.PrependCharToLen(' ',maLNum.len());
					qstr->cat(locNum);qstr->cat("] allele");
					aString allNum(paramNum+1);
					aString maxLNum(q_hetero_estimated_[pop][loc].len()+1);
					locNum.PrependCharToLen(' ',maxLNum.len());
					qstr->cat(allNum);qstr->cat("]");
					return qstr;}
				else{paramNum -= q_hetero_estimated_[pop][loc].len();}
			}}
			
		for(loc=0;loc<QHetero_estimated_.len();++loc){
			if(paramNum<QHetero_estimated_[loc].len()){
				aString *Qstr = NEW aString("Q_heterogam(est)loc[");
				aString locNum(loc+1);
				aString maLNum(QHetero_estimated_.len()+1);
				locNum.PrependCharToLen(' ',maLNum.len());
				Qstr->cat(locNum);Qstr->cat("] allele");
				aString allNum(paramNum+1);
				aString maxLNum(QHetero_estimated_[loc].len()+1);
				locNum.PrependCharToLen(' ',maxLNum.len());
				Qstr->cat(allNum);Qstr->cat("]");
				return Qstr;}
			else{paramNum -= QHetero_estimated_[loc].len();}
			}
		}//qParamsVary_
	aString errstr("bug: can't return name of param #");
	errstr.cat(origParamNum+1);
	errstr.cat(" in a ");errstr.cat(NumParams()-NumDerivedParams());errstr.cat("-parameter model");
	aError(errstr);
	return NEW aString();
	}//end Fis_Fit_Q_DomParamRec::ParamName(int )



void Fis_Fit_Q_DomParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::ParamString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat(lnL_Fis_,9,9);
	dataline.cat(sepchar);
	dataline.cat(lnL_Fit_,9,9);
	dataline.cat(sepchar);
	dataline.cat(Fis_,1,9);
	dataline.cat(sepchar);
	dataline.cat(Fit_,1,9);
	dataline.cat(sepchar);
	dataline.cat(Fst_,1,9);
	if(sexRatioVaries_){
		dataline.cat(sepchar);
		dataline.cat(sexRatio_,1,9);
		}
	int pop,loc,a;
	for(pop=0;pop<q_estimated_.len();++pop){
		for(loc=0;loc<q_estimated_[pop].len();++loc){
			for(a=0;a<q_estimated_[pop][loc].len();++a){
				dataline.cat(sepchar);
				dataline.cat(q_estimated_[pop][loc][a],1,9);
		}}}
	for(loc=0;loc<Q_estimated_.len();++loc){
		for(a=0;a<Q_estimated_[loc].len();++a){
			dataline.cat(sepchar);
			dataline.cat(Q_estimated_[loc][a],1,9);
		}}
	for(pop=0;pop<q_hetero_estimated_.len();++pop){
		for(loc=0;loc<q_hetero_estimated_[pop].len();++loc){
			for(a=0;a<q_hetero_estimated_[pop][loc].len();++a){
				dataline.cat(sepchar);
				dataline.cat(q_hetero_estimated_[pop][loc][a],1,9);
		}}}
	for(loc=0;loc<QHetero_estimated_.len();++loc){
		for(a=0;a<QHetero_estimated_[loc].len();++a){
			dataline.cat(sepchar);
			dataline.cat(QHetero_estimated_[loc][a],1,9);
		}}
	}//end Fis_Fit_Q_DomParamRec::ParamString(aString&, char)



long double Fis_Fit_Q_DomParamRec::ParamValue(int paramNum){
	//returns value of parameter
	switch (paramNum){
	 case 0://primary parameters
	 	 return Fis();
	 	 break;
	 case 1://primary parameters
	 	 return Fit();
	 	 break;
	 	default: break;}
	if(sexRatioVaries_ && paramNum==2){
	 	 return SexRatio();
	 	 }
	int origParamNum = paramNum, pop,loc;
	paramNum-=2;
	if(sexRatioVaries_){
		--paramNum;}
	switch(paramNum){
		case 0:
		 	 return Fst();
		 	 break;
		case 1:
		 	 return lnLFis();
		 	 break;
		case 2:
		 	 return lnLFit();
		 	 break;
		default: break; }
	paramNum-=3;
	if(qParamsVary_){
		for(pop=0;pop<q_estimated_.len();++pop){
			for(loc=0;loc<q_estimated_[pop].len();++loc){
				if(paramNum<q_estimated_[pop][loc].len()){
						return q_estimated_[pop][loc][paramNum];}
					else{paramNum -= q_estimated_[pop][loc].len();}
				}}
		for(loc=0;loc<Q_estimated_.len();++loc){
			if(paramNum<Q_estimated_[loc].len()){
					return Q_estimated_[loc][paramNum];}
				else{paramNum -= Q_estimated_[loc].len();}
			}
		for(pop=0;pop<q_hetero_estimated_.len();++pop){
			for(loc=0;loc<q_hetero_estimated_[pop].len();++loc){
				if(paramNum<q_hetero_estimated_[pop][loc].len()){
						return q_hetero_estimated_[pop][loc][paramNum];}
					else{paramNum -= q_hetero_estimated_[pop][loc].len();}
				}}
		for(loc=0;loc<QHetero_estimated_.len();++loc){
			if(paramNum<QHetero_estimated_[loc].len()){
					return QHetero_estimated_[loc][paramNum];}
				else{paramNum -= QHetero_estimated_[loc].len();}
			}
		}//qParamsVary_
	aString errstr("bug: can't return name of param #");
	errstr.cat(origParamNum+1);
	errstr.cat(" in a ");errstr.cat(TotalParams());errstr.cat("-parameter model");
	aError(errstr);
	return (long double) 0.0;
	}//end Fis_Fit_Q_DomParamRec::ParamValue(int )


void Fis_Fit_Q_DomParamRec::Print(ostream& outFile, long rep, aString& message){
//	if (rep>70){
//		outFile<<endl;}//dummy
	ParamRec::Print(outFile, rep, message);
	aString outStr("lnL[Fis]: ");
	outStr.cat(lnLFis(),1,9);
	outStr.separator(5);
	outStr += "lnL[Fit]: ";
	outStr.cat(lnLFit(),1,9);
//	outStr.separator(5);
	outFile<<outStr<<endl;
	outStr = "Fis: ";
	outStr.cat(Fis(),1,9);
	outStr.separator(5);
	outStr += "Fit: ";
	outStr.cat(Fit(),1,9);
	outStr.separator(5);
	outStr += "Fst: ";
	outStr.cat(Fst(),1,9);
//	outStr.separator(5);
	outFile<<outStr<<endl;
	outStr= "sex ratio: ";
	outStr.cat(SexRatio(),1,9);
	outStr.separator(5);
	if(!SexRatioVaries()) outStr +="   does not vary";
	outFile<<outStr<<endl;
	//printing of q_estimated_ can be suppressed by commenting this out
/*	int pop,loc,a;
	for(pop=0;pop<q_estimated_.len();++pop){
		aString qstr("q_homogam(est)[");
		aString popNum(pop+1);
		aString maxpop(q_estimated_.len()+1);
		popNum.PrependCharToLen(' ',maxpop.len());
		qstr+popNum;
		qstr+"][";
		for(loc=0;loc<q_estimated_[pop].len();++loc){
			aString locNum(loc+1);
			aString maxNum(q_estimated_[pop].len()+1);
			locNum.PrependCharToLen(' ',maxNum.len());
			locNum.prepend(qstr);
			locNum.cat("]:");
			for(a=0;a<q_estimated_[pop][loc].len();++a){
				aString alleleNum(a+1);
				aString maxANum(q_estimated_[pop][loc].len()+1);
				alleleNum.PrependCharToLen(' ',maxANum.len());
				alleleNum.cat("]: ");
				alleleNum.prepend(locNum);
				alleleNum.cat(q_estimated_[pop][loc][a]);
				outFile<<alleleNum<<endl;
				}/end for a
			}//for loc
		}//for pop
	aString Qstr("Q_homogam(est)[");
	for(loc=0;loc<Q_estimated_.len();++loc){
		aString locNum(loc+1);
		aString maxNum(Q_estimated_.len()+1);
		locNum.PrependCharToLen(' ',maxNum.len());
		locNum.prepend(qstr);
		locNum.cat("]:");
		for(a=0;a<Q_estimated_[loc].len();++a){
			aString alleleNum(a+1);
			aString maxANum(Q_estimated_[loc].len()+1);
			alleleNum.PrependCharToLen(' ',maxANum.len());
			alleleNum.cat("]: ");
			alleleNum.prepend(locNum);
			alleleNum.cat(Q_estimated_[loc][a]);
			outFile<<alleleNum<<endl;
			}/end for a
		}//for loc
	for(pop=0;pop<q_hetero_estimated_.len();++pop){
		aString qstr("q_heterogam(est)[");
		aString popNum(pop+1);
		aString maxpop(q_estimated_.len()+1);
		popNum.PrependCharToLen(' ',maxpop.len());
		qstr+popNum;
		qstr+"][";
		for(loc=0;loc<q_hetero_estimated_[pop].len();++loc){
			aString locNum(loc+1);
			aString maxNum(q_hetero_estimated_[pop].len()+1);
			locNum.PrependCharToLen(' ',maxNum.len());
			locNum.prepend(qstr);
			locNum.cat("]:");
			for(a=0;a<q_hetero_estimated_[pop][loc].len();++a){
				aString alleleNum(a+1);
				aString maxANum(q_hetero_estimated_[pop][loc].len()+1);
				alleleNum.PrependCharToLen(' ',maxANum.len());
				alleleNum.cat("]: ");
				alleleNum.prepend(locNum);
				alleleNum.cat(q_hetero_estimated_[pop][loc][a]);
				outFile<<alleleNum<<endl;
				}/end for a
			}//for loc
		}//for pop
	aString Qstr("Q_heterogam(est)[");
	for(loc=0;loc<QHetero_estimated_.len();++loc){
		aString locNum(loc+1);
		aString maxNum(Q_estimated_.len()+1);
		locNum.PrependCharToLen(' ',maxNum.len());
		locNum.prepend(qstr);
		locNum.cat("]:");
		for(a=0;a<QHetero_estimated_[loc].len();++a){
			aString alleleNum(a+1);
			aString maxANum(QHetero_estimated_[loc].len()+1);
			alleleNum.PrependCharToLen(' ',maxANum.len());
			alleleNum.cat("]: ");
			alleleNum.prepend(locNum);
			alleleNum.cat(QHetero_estimated_[loc][a]);
			outFile<<alleleNum<<endl;
			}/end for a
		}//for loc
*/
	}//Fis_Fit_Q_DomParamRec::Print


/*
int Fis_Fit_Q_DomParamRec::RandomParameter(Ran3& ran){
	long double rannum=ran();
	if(rannum<(long double)1.0/(long double)3.0){
		return 0;}//chooses Fis 33% of the time
	if(rannum<(long double)2.0/(long double)3.0){
		return 1;}//chooses Fit 33% of the time
	int ranparam = ran.longBetween(1,NumParams()-2);//chooses among q scores
	return ranparam;
	}//	Fis_Fit_Q_DomParamRec::RandomParameter()
*/




void Fis_Fit_Q_DomParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter
	switch (paramNum){
	 case 0:
	 	 Fis(newVal);
	 	 SetFst();
	 	 return;break;
	 case 1:
	 	 Fit(newVal);
	 	 SetFst();
	 	 return;break;
	 default: break;}
	if(sexRatioVaries_ && paramNum==2){
	 	 SetSexRatio(newVal,true);
	 	 return;
	 	 }
	int origParamNum = paramNum,pop,loc;
	paramNum-=2;
	if(sexRatioVaries_){
		--paramNum;}
	for(pop=0;pop<q_estimated_.len();++pop){
		for(loc=0;loc<q_estimated_[pop].len();++loc){
			if(paramNum<q_estimated_[pop][loc].len()){
					q_estimated_[pop][loc][paramNum]=newVal;
					SetDependentAllele_q(pop,loc);
					return;}
				else{paramNum -= q_estimated_[pop][loc].len()-1;}
			}}	
	for(loc=0;loc<Q_estimated_.len();++loc){
		if(paramNum<Q_estimated_[loc].len()){
				Q_estimated_[loc][paramNum]=newVal;
				SetDependentAllele_Q(loc);
				return;}
			else{paramNum -= Q_estimated_[loc].len()-1;}
		}
	for(pop=0;pop<q_hetero_estimated_.len();++pop){
		for(loc=0;loc<q_hetero_estimated_[pop].len();++loc){
			if(paramNum<q_hetero_estimated_[pop][loc].len()){
					q_hetero_estimated_[pop][loc][paramNum]=newVal;
					SetDependentAllele_q_hetero(pop,loc);
					return;}
				else{paramNum -= q_hetero_estimated_[pop][loc].len()-1;}
			}}	
	for(loc=0;loc<QHetero_estimated_.len();++loc){
		if(paramNum<QHetero_estimated_[loc].len()){
				QHetero_estimated_[loc][paramNum]=newVal;
				SetDependentAllele_Q_hetero(loc);
				return;}
			else{paramNum -= QHetero_estimated_[loc].len()-1;}
		}
	aString errstr("bug: can't set value of param #");
	errstr.cat(origParamNum+1);
	errstr.cat(" in a ");errstr.cat(NumParams());errstr.cat("-parameter model");
	aError(errstr);
	}//end Fis_Fit_Q_DomParamRec::SetParamValue(int,long double)



void Fis_Fit_Q_DomParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::TitleString(dataline, sepchar);
	dataline + sepchar;
	dataline + "lnL[Fis]";
	dataline + sepchar;
	dataline + "lnL[Fit]";
	dataline + sepchar;
	dataline + "Fis";
	dataline + sepchar;
	dataline + "Fit";
	dataline + sepchar;
	dataline + "Fst";
	if(sexRatioVaries_){
		dataline + sepchar;
		dataline + "sex_ratio";
		}
	int pop,loc,a;
	for(pop=0;pop<q_estimated_.len();++pop){
		aString qstr("q_homogam(est)[");
		aString popNum(pop+1);
		aString maxpop(q_estimated_.len()+1);
		popNum.PrependCharToLen(' ',maxpop.len());
		qstr+popNum;qstr+"][";
		for(loc=0;loc<q_estimated_[pop].len();++loc){
			aString locNum(loc+1);
			aString maxNum(q_estimated_[pop].len()+1);
			locNum.prepend(qstr);locNum+"][";
			for(a=0;a<q_estimated_[pop][loc].len();++a){
				aString aNum(a+1);
				aString maxaNum(q_estimated_[pop][loc].len()+1);
				aNum.prepend(locNum);aNum+"]";
				dataline+sepchar;
				dataline+aNum;
				}//for a
			}//for loc
		}//for pop
	aString Qstr("Q_homogam(est)[");
	for(loc=0;loc<Q_estimated_.len();++loc){
		aString locNum(loc+1);
		aString maxNum(Q_estimated_.len()+1);
		locNum.prepend(Qstr);locNum+"][";
		for(a=0;a<Q_estimated_[loc].len();++a){
			aString aNum(a+1);
			aString maxaNum(Q_estimated_[loc].len()+1);
			aNum.prepend(locNum);aNum+"]";
			dataline+sepchar;
			dataline+aNum;
			}//for a
		}//for loc
	for(pop=0;pop<q_hetero_estimated_.len();++pop){
		aString qstr("q_heterogam(est)[");
		aString popNum(pop+1);
		aString maxpop(q_hetero_estimated_.len()+1);
		popNum.PrependCharToLen(' ',maxpop.len());
		qstr+popNum;qstr+"][";
		for(loc=0;loc<q_hetero_estimated_[pop].len();++loc){
			aString locNum(loc+1);
			aString maxNum(q_hetero_estimated_[pop].len()+1);
			locNum.prepend(qstr);locNum+"][";
			for(a=0;a<q_hetero_estimated_[pop][loc].len();++a){
				aString aNum(a+1);
				aString maxaNum(q_hetero_estimated_[pop][loc].len()+1);
				aNum.prepend(locNum);aNum+"]";
				dataline+sepchar;
				dataline+aNum;
				}//for a
			}//for loc
		}//for pop
	Qstr="Q_heterogam(est)[";
	for(loc=0;loc<QHetero_estimated_.len();++loc){
		aString locNum(loc+1);
		aString maxNum(QHetero_estimated_.len()+1);
		locNum.prepend(Qstr);locNum+"][";
		for(a=0;a<QHetero_estimated_[loc].len();++a){
			aString aNum(a+1);
			aString maxaNum(QHetero_estimated_[loc].len()+1);
			aNum.prepend(locNum);aNum+"]";
			dataline+sepchar;
			dataline+aNum;
			}//for a
		}//for loc
	}//end Fis_Fit_Q_DomParamRec::TitleString(aString&, char)


int Fis_Fit_Q_DomParamRec::TotalParams(void){
	return NumParams() + NumDerivedParams();}





void Fis_Fit_Q_DomParamRec::operator=(ParamRec& p){
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class Fis_Fit_Q_DomParamRec
	Fis_Fit_Q_DomParamRec* pp = (Fis_Fit_Q_DomParamRec*) p.pointerToDerivedSelf;
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
	lnL_Fis_= pp->lnL_Fis_;
	lnL_Fit_= pp->lnL_Fit_;
	Fis_= pp->Fis_;
	Fit_= pp->Fit_;
	Fst_= pp->Fst_;
	sexRatio_= pp->sexRatio_;
	sexRatioVaries_= pp->sexRatioVaries_;
	qParamsVary_= pp->qParamsVary_;
	q_estimated_ = pp->q_estimated_;
	Q_estimated_ = pp->Q_estimated_;
	numqParams_ = pp->numqParams_;
	numIndependentqParams_ = pp->numIndependentqParams_;
	numQParams_ = pp->numQParams_;
	numIndependentQParams_ = pp->numIndependentQParams_;
	q_hetero_estimated_ = pp->q_hetero_estimated_;
	QHetero_estimated_ = pp->QHetero_estimated_;
	numqHeteroParams_ = pp->numqHeteroParams_;
	numIndependentqHeteroParams_ = pp->numIndependentqHeteroParams_;
	numQHeteroParams_ = pp->numQHeteroParams_;
	numIndependentQHeteroParams_ = pp->numIndependentQHeteroParams_;
	}//end  Fis_Fit_Q_DomParamRec::operator=(ParamRec&)





int	operator==(Fis_Fit_Q_DomParamRec& p, Fis_Fit_Q_DomParamRec& q){
	Fis_Fit_Q_ParamRec& a = p;
	Fis_Fit_Q_ParamRec& b = q;
	return ((a == b) &&
			(p.q_hetero_estimated_ == q.q_hetero_estimated_)  && (p.QHetero_estimated_ == q.QHetero_estimated_));
	}//end operator==(Fis_Fit_Q_DomParamRec&, Fis_Fit_Q_DomParamRec&)


int	operator!=(Fis_Fit_Q_DomParamRec& p, Fis_Fit_Q_DomParamRec& q){
	return !(p==q);
	}//end operator!=(Fis_Fit_Q_DomParamRec&, Fis_Fit_Q_DomParamRec&)



#endif


