/******************************************************
*
*	Fis_Fit_Q_ParamRec.cpp
*
*	Parameters for estimating Fis or Fit and allele frequencies Q using a multinomial distribution.
*	Use for diploid codominant or dominant markers; sex-linked/haplodiploid markers in derived class
*
******************************************************/
#ifndef Fis_Fit_Q_ParamReccpp
#define Fis_Fit_Q_ParamReccpp = 1

#include "Fis_Fit_Q_ParamRec.h"
#include "Element.cpp"
#include "aList.cpp"


Fis_Fit_Q_ParamRec::Fis_Fit_Q_ParamRec(void):ParamRec(), lnL_Fis_((long double)0.0),lnL_Fit_((long double)0.0),Fis_((long double)0.0), 
					Fit_((long double)0.0), Fst_((long double)0.0),sexRatio_((long double)0.5),sexRatioVaries_(false),qParamsVary_(true),
					numqParams_(0), numIndependentqParams_(0),numQParams_(0), numIndependentQParams_(0){
  	SetPointerToDerivedSelf();
//  	cout<<"constructing Fis_Fit_Q_ParamRec()"<<endl;
  	}


Fis_Fit_Q_ParamRec::Fis_Fit_Q_ParamRec(long double Fis_val, long double Fit_val, long double sr,
								 ObjList<ObjList<aList<long double> > >& init_q_estimated):
				ParamRec(), lnL_Fis_((long double)0.0),lnL_Fit_((long double)0.0),Fis_(Fis_val),Fit_(Fit_val),Fst_((long double)0.0),
				sexRatio_((long double)sr),sexRatioVaries_(false),qParamsVary_(true),q_estimated_(init_q_estimated){
	SetNumqParams();
	InitializeQ();
	SetFst();
  	SetPointerToDerivedSelf();
	ResetParamChoices();
	for(int i=0;i<paramsToChoose_;++i){
		paramChoices_.Append(i);}
  	}


Fis_Fit_Q_ParamRec::Fis_Fit_Q_ParamRec(Fis_Fit_Q_ParamRec& p):
  					ParamRec(p), lnL_Fis_(p.lnL_Fis_),lnL_Fit_(p.lnL_Fit_),Fis_(p.Fis_),Fit_(p.Fit_),
  					Fst_(p.Fst_),sexRatio_(p.sexRatio_),sexRatioVaries_(p.sexRatioVaries_),q_estimated_(p.q_estimated_),
  					qParamsVary_(p.qParamsVary_),numqParams_(p.numqParams_),numIndependentqParams_(p.numIndependentqParams_),
  					Q_estimated_(p.Q_estimated_),numQParams_(p.numQParams_),numIndependentQParams_(p.numIndependentQParams_),
  					paramChoices_(p.paramChoices_),paramsToChoose_(p.paramsToChoose_) {
  	SetPointerToDerivedSelf();
	}
	

Fis_Fit_Q_ParamRec::~Fis_Fit_Q_ParamRec(void){
	q_estimated_.Clear();
	Q_estimated_.Clear();
  	pointerToDerivedSelf = NULL;//forget
  	}

	
ParamRec* Fis_Fit_Q_ParamRec::Duplicate(void){
	Fis_Fit_Q_ParamRec* copy = NEW Fis_Fit_Q_ParamRec(*this);
	return copy;
	}//end Fis_Fit_Q_ParamRec::Duplicate()


Fis_Fit_Q_ParamRec* Fis_Fit_Q_ParamRec::_Duplicate(void){
	Fis_Fit_Q_ParamRec* copy = NEW Fis_Fit_Q_ParamRec(*this);
	return copy;
	}//end Fis_Fit_Q_ParamRec::_Duplicate()


void Fis_Fit_Q_ParamRec::SetPointerToDerivedSelf(void){
  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in Fis_Fit_Q_ParamRec"<<endl;
  	}//end


long double Fis_Fit_Q_ParamRec::Fst(void){
	return Fst_;}


void Fis_Fit_Q_ParamRec::Fis(long double Fis_val){
	Fis_ = Fis_val;
	SetFst();
	}//end Nm_Fst_ParamRec::Fis(long double )


long double Fis_Fit_Q_ParamRec::Fis(void){
	return Fis_;}


void Fis_Fit_Q_ParamRec::Fit(long double Fit_val){
	Fit_ = Fit_val;
	SetFst();
	}//end Nm_Fst_ParamRec::Fit(long double )


long double Fis_Fit_Q_ParamRec::Fit(void){
	return Fit_;}


	
void Fis_Fit_Q_ParamRec::SetNumqParams(void){
	numqParams_=numIndependentqParams_=0;
	int loci_estimated=0;
	for(int pop=0;pop<q_estimated_.len();++pop){
		for(int loc=0;loc<q_estimated_[pop].len();++loc){
			loci_estimated++;
			numqParams_ += q_estimated_[pop][loc].len();}}
	numIndependentqParams_ = numqParams_-loci_estimated;
	}//end Fis_Fit_Q_ParamRec::SetNumqParams()



void Fis_Fit_Q_ParamRec::SetNumQParams(void){
	numQParams_=numIndependentQParams_=0;
	int loci_estimated=0;
	for(int loc=0;loc<Q_estimated_.len();++loc){
		loci_estimated++;
		numQParams_ += Q_estimated_[loc].len();}
	numIndependentQParams_= numQParams_-loci_estimated;
	}//end Fis_Fit_Q_ParamRec::SetNumQParams()


long double Fis_Fit_Q_ParamRec::Q(int loc, int allele){
	//returns global estimated average
	return Q_estimated_[loc][allele];
	}//end Fis_Fit_Q_ParamRec::Q(int, int)


long double Fis_Fit_Q_ParamRec::q(int pop, int loc, int allele){
	//returns local estimated frequency; 0:recessive, 1:dominant
	long double qq = q_estimated_[pop][loc][allele];
	return qq;
	}//end Fis_Fit_Q_ParamRec::g(int,int,int)





int Fis_Fit_Q_ParamRec::ChangeFis(Ran3& ran, long double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary Fis
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	long double oldFis = Fis_, oldFst = Fst_;
	do{
		++tries;
		Fis_ = ran.NormDev(oldFis, sigma);
		SetFst();
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		if(!validChange && tries>100){
			Fis_ = oldFis;
			Fst_ = oldFst;
			return validChange;}
		}while(!(validChange));
	return validChange;
	}//end Fis_Fit_Q_ParamRec::ChangeFis(Ran3&, double, ParamRec*, ParamRec*)



int Fis_Fit_Q_ParamRec::ChangeFit(Ran3& ran, long double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary Fit
	//return 0 for failure, 1 for success
	int validChange=false, tries=0;
	long double oldFit = Fit_, oldFst = Fst_;
	do{
		++tries;
		Fit_ = ran.NormDev(oldFit, sigma);
		SetFst();
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		if(!validChange && tries>100){
			Fit_ = oldFit;
			Fst_ = oldFst;
			return validChange;}
		}while(!(validChange));
	return validChange;
	}//end Fis_Fit_Q_ParamRec::ChangeFit(Ran3&, double, ParamRec*, ParamRec*)



int Fis_Fit_Q_ParamRec::Change_q(int paramNum, Ran3& ran, Fis_Fit_Q_ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary q
	//return 0 for failure, 1 for success
	int validChange=false, found=false, thepop=0, theloc=0, theAllele=0, p = paramNum;
	int pcount=0;
	for(int pop=0;pop<q_estimated_.len();++pop){
		for(int loc=0;loc<q_estimated_[pop].len();++loc){
			for(int a=0;a<q_estimated_[pop][loc].len()-1;++a){
				if(pcount==p){
					thepop = pop; theloc=loc; theAllele=a; 
					found=true; break;
					}
				pcount++;
				}//end for a
			if(found){break;}
			}//end for loc
		if(found){break;}
		}//end for pop
	long double old_q = q_estimated_[thepop][theloc][theAllele], new_q;
	aList<long double> old_qList = q_estimated_[thepop][theloc];
	long double sigma = paramDeviations->q_estimated_[thepop][theloc][theAllele];
	do{
		q_estimated_[thepop][theloc]=old_qList;//restore
		new_q = q_estimated_[thepop][theloc][theAllele] = ran.NormDev(old_q, sigma);
		if(new_q<(long double)0.0){
			validChange=false; continue;}
		SetDependentAllele_q(thepop,theloc);
		if(q_estimated_[thepop][theloc].Last()<(long double)0.0){
			validChange=false; continue;}
		//now adjust remaining alleles to compensate
/*		double freqRemaining = 1.0-new_q;
		aList<double> partialFreqs;aList<int> geneOrder;
		for(int a=0;a<q_estimated_[thepop][theloc].len();++a){
			if(a != theAllele){ 
				partialFreqs.Append(q_estimated_[thepop][theloc][a]);
				geneOrder.Append(a);
				}//end if
			}//end for a
		for(int aa=0;aa<geneOrder.len()-1;++aa){
			double new_qAdj=0.0;
			while ((new_qAdj>=0.0)&&(new_qAdj<1.0)){
				new_qAdj = ran.NormDev(partialFreqs[aa]/freqRemaining, sigma/freqRemaining);}
			q_estimated_[thepop][theloc][geneOrder[aa]]= new_qAdj*freqRemaining;
			freqRemaining -= new_qAdj*freqRemaining;
			}//end for aa
		if(geneOrder.len()>0){
			q_estimated_[thepop][theloc][geneOrder.Last()] = freqRemaining;}
		
*/		
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end Fis_Fit_Q_ParamRec::Change_q(Ran3&, double, ParamRec*, ParamRec*)


int Fis_Fit_Q_ParamRec::Change_Q(int paramNum, Ran3& ran, Fis_Fit_Q_ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary Q
	//return 0 for failure, 1 for success
	int validChange=false, found=false, theloc=0, theAllele=0, p = paramNum;
	int pcount=0;
	for(int loc=0;loc<Q_estimated_.len();++loc){
		for(int a=0;a<Q_estimated_[loc].len()-1;++a){
			if(pcount==p){
				theloc=loc; theAllele=a; 
				found=true; break;
				}
			pcount++;
			}//end for a
		if(found){break;}
		}//end for loc
	long double old_Q = Q_estimated_[theloc][theAllele], new_Q;
	aList<long double> old_QList = Q_estimated_[theloc];
	long double sigma = paramDeviations->Q_estimated_[theloc][theAllele];
	do{
		Q_estimated_[theloc]=old_QList;//restore
		new_Q = Q_estimated_[theloc][theAllele] = ran.NormDev(old_Q, sigma);
		if(new_Q<(long double)0.0){
			validChange=false; continue;}
		SetDependentAllele_Q(theloc);
		if(Q_estimated_[theloc].Last()<(long double)0.0){
			validChange=false; continue;}
		//now adjust remaining alleles to compensate
/*		double freqRemaining = 1.0-new_q;
		aList<double> partialFreqs;aList<int> geneOrder;
		for(int a=0;a<q_estimated_[thepop][theloc].len();++a){
			if(a != theAllele){ 
				partialFreqs.Append(q_estimated_[thepop][theloc][a]);
				geneOrder.Append(a);
				}//end if
			}//end for a
		for(int aa=0;aa<geneOrder.len()-1;++aa){
			double new_qAdj=0.0;
			while ((new_qAdj>=0.0)&&(new_qAdj<1.0)){
				new_qAdj = ran.NormDev(partialFreqs[aa]/freqRemaining, sigma/freqRemaining);}
			q_estimated_[thepop][theloc][geneOrder[aa]]= new_qAdj*freqRemaining;
			freqRemaining -= new_qAdj*freqRemaining;
			}//end for aa
		if(geneOrder.len()>0){
			q_estimated_[thepop][theloc][geneOrder.Last()] = freqRemaining;}
		
*/		
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end Fis_Fit_Q_ParamRec::Change_q(Ran3&, double, ParamRec*, ParamRec*)


int Fis_Fit_Q_ParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double /*weight*/){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	Fis_Fit_Q_ParamRec* deviations = (Fis_Fit_Q_ParamRec*) paramDeviations->pointerToDerivedSelf;
	switch (paramNum){
	 case 0:
	 	 return ChangeFis(ran, deviations->Fis(), lowerLimits, upperLimits);
	 	 break;
	 case 1:
	 	 return ChangeFit(ran, deviations->Fit(), lowerLimits, upperLimits);
	 	 break;
	 default: break;}
	if(sexRatioVaries_ && paramNum==2){
	 	 return ChangeSexRatio(ran, deviations->SexRatio(), lowerLimits, upperLimits);}
	int origParamNum = paramNum;
	paramNum -= 2;
	if(sexRatioVaries_){
		--paramNum;}
	if(paramNum<NumqParams()){
	 	 return Change_q(paramNum, ran, deviations, lowerLimits, upperLimits);
	 	 }
	paramNum -= NumIndependentqParams();
	if(paramNum<NumQParams()){
	 	 return Change_Q(paramNum, ran, deviations, lowerLimits, upperLimits);
	 	 }
	aString errstr("bug: can't change param #");
		errstr.cat(origParamNum+1);
		errstr.cat(" in a ");errstr.cat(NumParams());
		errstr.cat("-parameter model fitting Fis, Fit & allele frequencies");
	aError(errstr);
	 return 0;
	}//end Fis_Fit_Q_ParamRec::ChangeParam(int)

/*
int Fis_Fit_Q_ParamRec::ChangeParams(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	int validchange = false;
	int paramToChange = RandomParameter(ran);
	do{
		validchange = ChangeParam(paramToChange, ran, 
										paramDeviations, lowerLimits, upperLimits);
		}while(!(validchange));
	return paramToChange;
	}//end Fis_Fit_Q_ParamRec::ChangeParams(Ran3&, ParamRec*, ParamRec*, ParamRec*)
*/

int Fis_Fit_Q_ParamRec::ChangeAnnealingParams(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	if(qParamsVary_){
		return ParamRec::ChangeParams(ran,paramDeviations,lowerLimits, upperLimits);}

	int validchange = false, paramToChange, tries=0;
//	SetQtoVary(false);
	do{
//		if(Fis_==(long double)0.0 && Fit_==(long double)0.0){
		if(sexRatioVaries_ && ran()<0.33333333){
				paramToChange = 2;}
			else{
				if(Fis_== Fit_ && Fit_<(long double)1.0){
						paramToChange = 1;}//just Fit, to keep Fst legal
					else if(Fis_== Fit_ && Fit_==(long double)1.0){
						paramToChange = 0;}//just Fis, to keep Fst legal
					else{
						paramToChange = ran.longBetween(0,1);}//just Fis and Fit
				}
		validchange = ChangeParam(paramToChange, ran, 
										paramDeviations, lowerLimits, upperLimits,1.0);
		if(! validchange){
			tries++;
			if(tries>100){//bail out
				return -1;}
			long double val = ParamValue(paramToChange);
			long double devVal = paramDeviations->ParamValue(paramToChange);
			if(0.9*devVal >val/10000.0){//set a minimum deviation scaled to the parameter's value
				paramDeviations->MultiplyParam(paramToChange,0.9);}
				}
		}while(!(validchange));
	return paramToChange;
	}//end Fis_Fit_Q_ParamRec::ChangeAnnealingParams(Ran3&, ParamRec*, ParamRec*, ParamRec*)


int Fis_Fit_Q_ParamRec::ChangeSexRatio(Ran3& ran, long double sigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary sex ratio
	//return 0 for failure, 1 for success
	int validChange=false;
	long double oldSR = sexRatio_;
	do{
		sexRatio_ = ran.NormDev(oldSR, sigma);
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end Fis_Fit_Q_ParamRec::ChangeSexRatio(Ran3&, double, ParamRec*, ParamRec*)




long double Fis_Fit_Q_ParamRec::ExpectedValue(aList<long double>& /*coords*/){
//	return Cline_at_X(coords[0],0);
	return (long double) 0.0;
	}//end Fis_Fit_Q_ParamRec::ExpectedValue(aList<double>&)



void Fis_Fit_Q_ParamRec::InitializeQ(void){//acaculate means from pop frequencies
	Q_estimated_.Clear();
	int pop,loc,all;
	long double freq;
	for(loc=0;loc<q_estimated_[0].len();++loc){
		Q_estimated_.Append(NEW aList<long double>());
		for(all=0;all<q_estimated_[0][loc].len();++all){
			freq=(long double)0.0;
			for(pop=0;pop<q_estimated_.len();++pop){
				freq += q_estimated_[pop][loc][all];}
			freq /= (long double) q_estimated_.len();
			Q_estimated_[loc].Append(freq);
			}//all
		SetDependentAllele_Q(loc);//in case they don't add up to 1.0
		}//loc
	SetNumQParams();
	}//Fis_Fit_Q_ParamRec::InitializeQ()
			


int Fis_Fit_Q_ParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//use type conversion that circumvents usual virtual base class restrictions
	Fis_Fit_Q_ParamRec* lower = (Fis_Fit_Q_ParamRec*) lowerLimits->pointerToDerivedSelf;
	Fis_Fit_Q_ParamRec* upper = (Fis_Fit_Q_ParamRec*) upperLimits->pointerToDerivedSelf;
	if(Fis_ < lower->Fis_){return 0;}
		else if (Fis_ > upper->Fis_){return 0;}
		else {}
	if(Fit_ < lower->Fit_){return 0;}
		else if (Fit_ > upper->Fit_){return 0;}
		else {}
	if(Fst_ < lower->Fst_){return 0;}
		else if (Fst_ > upper->Fst_){return 0;}
		else {}
	if(sexRatioVaries_){
		if(sexRatio_ <= lower->sexRatio_){return 0;}
			else if (sexRatio_ > upper->sexRatio_){return 0;}//all female population OK by default
			else {}}
	if(! qParamsVary_) {return 1;}
	int pop,loc,a;
	for(pop=0;pop<q_estimated_.len();++pop){
		for(loc=0;loc<q_estimated_[pop].len();++loc){
			long double asum=0;
			for(a=0;a<q_estimated_[pop][loc].len();++a){
				if((q_estimated_[pop][loc][a]<(long double)0.0)||(q_estimated_[pop][loc][a]>(long double)1.0)){
					return 0;}
				asum += q_estimated_[pop][loc][a];
				}//a
			if(asum != (float) 1.0){//handles rounding error
				return 0;}
			}}
	for(loc=0;loc<Q_estimated_.len();++loc){
		long double asum=0;
		for(a=0;a<Q_estimated_[loc].len();++a){
			if((Q_estimated_[loc][a]<(long double)0.0)||(Q_estimated_[loc][a]>(long double)1.0)){
				return 0;}
			asum += Q_estimated_[loc][a];
			}//a
		if(asum != (float) 1.0){//handles rounding error
			return 0;}
		}
	return 1;
	}//end Fis_Fit_Q_ParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void Fis_Fit_Q_ParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	switch (paramNum){
	 case 0:
	 	 Fis_ *= factor;
	 	 SetFst();
	 	 return; break;
	 case 1:
	 	 Fit_ *= factor;
	 	 SetFst();
	 	 return; break;
	 default: break;}
	if(sexRatioVaries_ && paramNum==2){
	 	 sexRatio_ *= factor;
	 	 return;}
	int origParamNum = paramNum;
	paramNum -= 2;
	if(sexRatioVaries_){
		--paramNum;}
	for(int pop=0;pop<q_estimated_.len();++pop){
		for(int loc=0;loc<q_estimated_[pop].len();++loc){
			if(paramNum<q_estimated_[pop][loc].len()){
					q_estimated_[pop][loc][paramNum] *= factor;
					return;}
				else{paramNum -= q_estimated_[pop][loc].len()-1;}
			}}
//	paramNum =- NumIndependentqParams();
	for(int loc=0;loc<Q_estimated_.len();++loc){
		if(paramNum<Q_estimated_[loc].len()){
				Q_estimated_[loc][paramNum] *= factor;
				return;}
			else{paramNum -= Q_estimated_[loc].len()-1;}
		}
	aString errstr("Fis_Fit_Q_ParamRec::MultiplyParam(): can't rescale param #");
	errstr.cat(origParamNum+1);
	errstr.cat(" in a ");errstr.cat(NumParams());errstr.cat("-parameter model");
	aError(errstr);
	}//Fis_Fit_Q_ParamRec::MultiplyParam(int, long double)


int Fis_Fit_Q_ParamRec::NumDerivedParams(void){//Fst_ & last allele freq of each locus in each pop is a derived param
	if(! qParamsVary_){ return 3;}//Fst, lnLFis], lnL[Fit] 
	return 1 + NumqParams() - NumIndependentqParams()  + NumQParams() - NumIndependentQParams();//Nm
	}//end Fis_Fit_Q_ParamRec::NumDerivedParams()


int Fis_Fit_Q_ParamRec::NumParams(void){//independently varying parameters
	int par = 2;
	if(qParamsVary_){
		par += NumIndependentqParams() + NumIndependentQParams();}
	if(sexRatioVaries_){ ++par;}
	return par;
	}//end Fis_Fit_Q_ParamRec::NumParams()



aString* Fis_Fit_Q_ParamRec::ParamName(int paramNum){
	switch (paramNum){
	 case 0://primary parameters
	 	 return NEW aString("Fis");
	 	 break;
	 case 1://primary parameters
	 	 return NEW aString("Fit");
	 	 break;
	 	default: break;}
	if(sexRatioVaries_ && paramNum==2){
	 	 return NEW aString("sex ratio");
	 	 }
	int origParamNum = paramNum;
	paramNum -= 2;
	if(sexRatioVaries_){
		--paramNum;}
	switch(paramNum){
		case 0:
		 	 return NEW aString("Fst");
		 	 break;
		case 1:
		 	 return NEW aString("lnL[Fis]");
		 	 break;
		case 2:
		 	 return NEW aString("lnL[Fit]");
		 	 break;
		default: break; }
	paramNum-=3;
	int pop,loc;
	if(qParamsVary_){
		for(pop=0;pop<q_estimated_.len();++pop){
			for(loc=0;loc<q_estimated_[pop].len();++loc){
				if(paramNum<q_estimated_[pop][loc].len()){
					aString *qstr = NEW aString("q(est) pop[");
					aString popNum(pop+1);
					aString maxpNum(q_estimated_.len()+1);
					popNum.PrependCharToLen(' ',maxpNum.len());
					qstr->cat(popNum);qstr->cat("] loc[");
					aString locNum(loc+1);
					aString maLNum(q_estimated_[pop].len()+1);
					locNum.PrependCharToLen(' ',maLNum.len());
					qstr->cat(locNum);qstr->cat("] allele");
					aString allNum(paramNum+1);
					aString maxLNum(q_estimated_[pop][loc].len()+1);
					locNum.PrependCharToLen(' ',maxLNum.len());
					qstr->cat(allNum);qstr->cat("]");
					return qstr;}
				else{paramNum -= q_estimated_[pop][loc].len();}
			}}
			
		for(loc=0;loc<Q_estimated_.len();++loc){
			if(paramNum<Q_estimated_[loc].len()){
					aString *Qstr = NEW aString("Q(est)loc[");
					aString locNum(loc+1);
					aString maLNum(Q_estimated_.len()+1);
					locNum.PrependCharToLen(' ',maLNum.len());
					Qstr->cat(locNum);Qstr->cat("] allele");
					aString allNum(paramNum+1);
					aString maxLNum(Q_estimated_[loc].len()+1);
					locNum.PrependCharToLen(' ',maxLNum.len());
					Qstr->cat(allNum);Qstr->cat("]");
					return Qstr;}
				else{paramNum -= Q_estimated_[loc].len();}
			}//loc			
		}//qParamsVary_
	//got here by mistake
	aString errstr("bug: can't return name of param #");
	errstr.cat(origParamNum+1);
	errstr.cat(" in a ");errstr.cat(NumParams()-NumDerivedParams());errstr.cat("-parameter model");
	aError(errstr);
	return NEW aString();
	}//end Fis_Fit_Q_ParamRec::ParamName(int )



void Fis_Fit_Q_ParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::ParamString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat(lnL_Fis_,9,9);
	dataline.cat(sepchar);
	dataline.cat(lnL_Fit_,9,9);
	dataline.cat(sepchar);
	dataline.cat(Fis_,1,9);
	dataline.cat(sepchar);
	dataline.cat(Fit_,1,9);
	dataline.cat(sepchar);
	dataline.cat(Fst_,1,9);
	if(sexRatioVaries_){
		dataline.cat(sepchar);
		dataline.cat(sexRatio_,1,9);
		}
	int pop,loc,a;
	for(pop=0;pop<q_estimated_.len();++pop){
		for(loc=0;loc<q_estimated_[pop].len();++loc){
			for(a=0;a<q_estimated_[pop][loc].len();++a){
				dataline.cat(sepchar);
				dataline.cat(q_estimated_[pop][loc][a],1,9);
		}}}
	for(loc=0;loc<Q_estimated_.len();++loc){
		for(a=0;a<Q_estimated_[loc].len();++a){
			dataline.cat(sepchar);
			dataline.cat(Q_estimated_[loc][a],1,9);
		}}
	}//end Fis_Fit_Q_ParamRec::ParamString(aString&, char)



long double Fis_Fit_Q_ParamRec::ParamValue(int paramNum){
	//returns value of parameter
	switch (paramNum){
	 case 0://primary parameters
	 	 return Fis();
	 	 break;
	 case 1://primary parameters
	 	 return Fit();
	 	 break;
	 	default: break;}
	if(sexRatioVaries_ && paramNum==2){
	 	 return SexRatio();
	 	 }
	int origParamNum = paramNum, pop,loc;
	paramNum -= 2;
	if(sexRatioVaries_){
		--paramNum;}
	switch(paramNum){
		case 0:
		 	 return Fst();
		 	 break;
		case 1:
		 	 return lnLFis();
		 	 break;
		case 2:
		 	 return lnLFit();
		 	 break;
		default: break; }
	paramNum-=3;
	if(qParamsVary_){
		for(pop=0;pop<q_estimated_.len();++pop){
			for(loc=0;loc<q_estimated_[pop].len();++loc){
				if(paramNum<q_estimated_[pop][loc].len()){
						return q_estimated_[pop][loc][paramNum];}
					else{paramNum -= q_estimated_[pop][loc].len();}
				}}
		for(int loc=0;loc<Q_estimated_.len();++loc){
			if(paramNum<Q_estimated_[loc].len()){
					return Q_estimated_[loc][paramNum];}
				else{paramNum -= Q_estimated_[loc].len();}
			}
		}
	aString errstr("bug: can't return name of param #");
	errstr.cat(origParamNum+1);
	errstr.cat(" in a ");errstr.cat(TotalParams());errstr.cat("-parameter model");
	aError(errstr);
	return (long double) 0.0;
	}//end Fis_Fit_Q_ParamRec::ParamValue(int )


void Fis_Fit_Q_ParamRec::Print(ostream& outFile, long rep, aString& message){
//	if (rep>70){
//		outFile<<endl;}//dummy
	ParamRec::Print(outFile, rep, message);
	aString outStr("lnL[Fis]: ");
	outStr.cat(lnLFis(),1,9);
	outStr.separator(5);
	outStr + "lnL[Fit]: ";
	outStr.cat(lnLFit(),1,9);
//	outStr.separator(5);
	outFile<<outStr<<endl;
	outStr="Fis: ";
	outStr.cat(Fis(),1,9);
	outStr.separator(5);
	outStr + "Fit: ";
	outStr.cat(Fit(),1,9);
	outStr.separator(5);
	outStr + "Fst: ";
	outStr.cat(Fst(),1,9);
//	outStr.separator(5);
	outFile<<outStr<<endl;
	outStr= "sex ratio: ";
	outStr.cat(SexRatio(),1,9);
	outStr.separator(5);
	if(!SexRatioVaries()) outStr +="   does not vary";
	outFile<<outStr<<endl;
	//printing of q_estimated_ can be suppressed by commenting this out
/*	nt pop,loc,a;
	for(pop=0;pop<q_estimated_.len();++pop){
		aString qstr("q(est)[");
		aString popNum(pop+1);
		aString maxpop(q_estimated_.len()+1);
		popNum.PrependCharToLen(' ',maxpop.len());
		qstr+popNum;
		qstr+"][";
		for(loc=0;loc<q_estimated_[pop].len();++loc){
			aString locNum(loc+1);
			aString maxNum(q_estimated_[pop].len()+1);
			locNum.PrependCharToLen(' ',maxNum.len());
			locNum.prepend(qstr);
			locNum.cat("]:");
			for(a=0;a<q_estimated_[pop][loc].len();++a){
				aString alleleNum(a+1);
				aString maxANum(q_estimated_[pop][loc].len()+1);
				alleleNum.PrependCharToLen(' ',maxANum.len());
				alleleNum.cat("]: ");
				alleleNum.prepend(locNum);
				alleleNum.cat(q_estimated_[pop][loc][a]);
				outFile<<alleleNum<<endl;
				}/end for a
			}//for loc
		}//for pop
	aString Qstr("Q_homogam(est)[");
	for(loc=0;loc<Q_estimated_.len();++loc){
		aString locNum(loc+1);
		aString maxNum(Q_estimated_.len()+1);
		locNum.PrependCharToLen(' ',maxNum.len());
		locNum.prepend(qstr);
		locNum.cat("]:");
		for(a=0;a<Q_estimated_[loc].len();++a){
			aString alleleNum(a+1);
			aString maxANum(Q_estimated_[loc].len()+1);
			alleleNum.PrependCharToLen(' ',maxANum.len());
			alleleNum.cat("]: ");
			alleleNum.prepend(locNum);
			alleleNum.cat(Q_estimated_[loc][a]);
			outFile<<alleleNum<<endl;
			}/end for a
		}//for loc
*/
	}//Fis_Fit_Q_ParamRec::Print



int Fis_Fit_Q_ParamRec::RandomParameter(Ran3& ran){
/*	long double rannum=ran();
	long double denominator = (long double)3.0;
	if(sexRatioVaries_){ denominator++;}
	if(rannum<(long double)1.0/denominator){
		return 0;}//chooses Fis 33% of the time
	if(rannum<(long double)2.0/denominator){
		return 1;}//chooses Fit 33% of the time
	if(sexRatioVaries_ && rannum<(long double)3.0/denominator){
		return 2;}//chooses sexRatio 25% of the time
	int ranparam = ran.longBetween((long)(denominator-1),NumParams());//chooses among q scores
	return ranparam;
*/
	//choose without relacement
	if(paramsToChoose_==0){
		ResetParamChoices();}
	int ranparamSpot = ran.longBetween(0,paramsToChoose_-1);
	int ranparam = paramChoices_[ranparamSpot];
	paramChoices_[ranparamSpot] = paramChoices_[paramsToChoose_-1];//swap to end of list
	paramChoices_[paramsToChoose_-1] = ranparam;
	--paramsToChoose_;
	return ranparam;
	}//	Fis_Fit_Q_ParamRec::RandomParameter()





void Fis_Fit_Q_ParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter
	switch (paramNum){
	 case 0:
	 	 Fis(newVal);
	 	 SetFst();
	 	 return;break;
	 case 1:
	 	 Fit(newVal);
	 	 SetFst();
	 	 return;break;
	 default: break;}
	if(sexRatioVaries_ && paramNum==2){
	 	 SetSexRatio(newVal,true);
	 	 return;
	 	 }
	int origParamNum = paramNum, pop,loc;
	paramNum -= 2;
	if(sexRatioVaries_){
		--paramNum;}
	for(pop=0;pop<q_estimated_.len();++pop){
		for(loc=0;loc<q_estimated_[pop].len();++loc){
			if(paramNum<q_estimated_[pop][loc].len()){
					q_estimated_[pop][loc][paramNum]=newVal;
					SetDependentAllele_q(pop,loc);
					return;}
				else{paramNum -= q_estimated_[pop][loc].len()-1;}
			}}	
	for(loc=0;loc<Q_estimated_.len();++loc){
		if(paramNum<Q_estimated_[loc].len()){
				Q_estimated_[loc][paramNum]=newVal;
				SetDependentAllele_Q(loc);
				return;}
			else{paramNum -= Q_estimated_[loc].len()-1;}
		}
	aString errstr("bug: can't set value of param #");
	errstr.cat(origParamNum+1);
	errstr.cat(" in a ");errstr.cat(NumParams());errstr.cat("-parameter model");
	aError(errstr);
	}//end Fis_Fit_Q_ParamRec::SetParamValue(int,long double)



void Fis_Fit_Q_ParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::TitleString(dataline, sepchar);
	dataline + sepchar;
	dataline + "lnL[Fis]";
	dataline + sepchar;
	dataline + "lnL[Fit]";
	dataline + sepchar;
	dataline + "Fis";
	dataline + sepchar;
	dataline + "Fit";
	dataline + sepchar;
	dataline + "Fst";
	if(sexRatioVaries_){
		dataline + sepchar;
		dataline + "sex_ratio";
		}
	int pop,loc,a;
	for(pop=0;pop<q_estimated_.len();++pop){
		aString qstr("q(est)[");
		aString popNum(pop+1);
		aString maxpop(q_estimated_.len()+1);
		popNum.PrependCharToLen(' ',maxpop.len());
		qstr+popNum;qstr+"][";
		for(loc=0;loc<q_estimated_[pop].len();++loc){
			aString locNum(loc+1);
			aString maxNum(q_estimated_[pop].len()+1);
			locNum.prepend(qstr);locNum+"][";
			for(a=0;a<q_estimated_[pop][loc].len();++a){
				aString aNum(a+1);
				aString maxaNum(q_estimated_[pop][loc].len()+1);
				aNum.prepend(locNum);aNum+"]";
				dataline+sepchar;
				dataline+aNum;
				}//for a
			}//for loc
		}//for pop
	aString Qstr("Q(est)[");
	for(loc=0;loc<Q_estimated_.len();++loc){
		aString locNum(loc+1);
		aString maxNum(Q_estimated_.len()+1);
		locNum.prepend(Qstr);locNum+"][";
		for(a=0;a<Q_estimated_[loc].len();++a){
			aString aNum(a+1);
			aString maxaNum(Q_estimated_[loc].len()+1);
			aNum.prepend(locNum);aNum+"]";
			dataline+sepchar;
			dataline+aNum;
			}//for a
		}//for loc
	}//end Fis_Fit_Q_ParamRec::TitleString(aString&, char)


int Fis_Fit_Q_ParamRec::TotalParams(void){
	return NumParams() + NumDerivedParams();}





void Fis_Fit_Q_ParamRec::operator=(ParamRec& p){
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class Fis_Fit_Q_ParamRec
	Fis_Fit_Q_ParamRec* pp = (Fis_Fit_Q_ParamRec*) p.pointerToDerivedSelf;
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
	lnL_Fis_= pp->lnL_Fis_;
	lnL_Fit_= pp->lnL_Fit_;
	Fis_= pp->Fis_;
	Fit_= pp->Fit_;
	Fst_= pp->Fst_;
	sexRatio_= pp->sexRatio_;
	sexRatioVaries_= pp->sexRatioVaries_;
	qParamsVary_= pp->qParamsVary_;
	q_estimated_ = pp->q_estimated_;
	Q_estimated_ = pp->Q_estimated_;
	numqParams_ = pp->numqParams_;
	numIndependentqParams_ = pp->numIndependentqParams_;
	numQParams_ = pp->numQParams_;
	numIndependentQParams_ = pp->numIndependentQParams_;
	}//end  Fis_Fit_Q_ParamRec::operator=(ParamRec&)





int	operator==(Fis_Fit_Q_ParamRec& p, Fis_Fit_Q_ParamRec& q){
	ParamRec& a = p;
	ParamRec& b = q;
	return ((a == b) && (p.Fis_ == q.Fis_) && (p.Fit_ == q.Fit_) &&(p.sexRatio_ == q.sexRatio_) &&
			(p.q_estimated_ == q.q_estimated_)  && (p.Q_estimated_ == q.Q_estimated_));
	}//end operator==(Fis_Fit_Q_ParamRec&, Fis_Fit_Q_ParamRec&)


int	operator!=(Fis_Fit_Q_ParamRec& p, Fis_Fit_Q_ParamRec& q){
	return !(p==q);
	}//end operator!=(Fis_Fit_Q_ParamRec&, Fis_Fit_Q_ParamRec&)



#endif


