/*

GelJockLandScapeInterface.cpp

*/

//20070129  Completely revamped to account for new genetic analysis data structures

#include "GelJockLandScapeInterface.h"
#include "Genotype.h"
#include "Element.cpp"
#include "aList.cpp"
#include "CoordinateClass.cpp"




void GelJockLandScapeInterface::Clear(){
	actualAlleleNames_.Clear();
 	MultiformatMarkerData::Clear();
 	}


void GelJockLandScapeInterface::PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer){
	//outfile assumed open
	MultiformatMarkerData::PrintAlleleFreqs(outFile,ncols,colSpacer);
	PrintAlleleKey(outFile);
	}//GelJockLandScapeInterface::PrintAlleleFreqs




void GelJockLandScapeInterface::PrintAlleleKey (ostream& outFile){
	//outfile assumed open
	outFile<<"Key to genotypes of allele letter scores."<<endl<<endl;
	outFile<<"Locus & Allele    genotype"<<endl;
	int loc,a;
	for(loc=0;loc<ntraits_;++loc){
		outFile<<traitName_[loc]<<endl;
		StringList& locList = actualAlleleNames_[loc];
		for(a=0;a<locList.len();++a){
			char alleleCh = 'A' + a;
			aString allele(alleleCh);
			allele.prepend("  ");
			allele.separator(10);
			allele.cat(locList[a]);
			outFile<<allele<<endl;
			}
		}
	outFile<<endl<<endl;
	}//GelJockLandScapeInterface::PrintAlleleKey




//void GelJockLandScapeInterface::ReadGeneticLandScape(GeneticLandScape& theLandScape, 
void GelJockLandScapeInterface::ReadGeneticLandScape(LandScape& theLandScape, 
				int species, aString& simTitle, aList<int>& skippedLocusNumbers, 
				int maxAlleleLen, bool expectMissingData){
	aList<int> popNumbers;
	int i;
	for(i=0;i<theLandScape.area();++i){
		popNumbers.Append(i);}
	ReadGeneticLandScape(theLandScape,species, simTitle, skippedLocusNumbers, popNumbers, maxAlleleLen, expectMissingData);
	}//end 




//void GelJockLandScapeInterface::ReadGeneticLandScape(GeneticLandScape& theLandScape, 
void GelJockLandScapeInterface::ReadGeneticLandScape(LandScape& theLandScape, 
				int species, aString& simTitle, aList<int>& skippedLocusNumbers, aList<int>& popNumbers, 
				int maxAlleleLen, bool expectMissingData){
	Clear();
	int r, c, pop, i, loc, trt;
	for(trt=0;trt<numTraitTypes_;++trt){
		traitTypesIncluded_.Append(false);
		}
	title_ = simTitle;

	npop_ = popNumbers.len();
	ngroups_ = 1;//default value
	SetAlleleLength(maxAlleleLen);

	ntraits_ = 2;//ID & sex
	int runningTraits=-1;
	traitName_.Append(NEW aString("ID"));
	traitNum_.Append(++runningTraits);
	traitTypesIncluded_[0]=true;
	traitType_.Append(0);
	traitName_.Append(NEW aString("sex"));//default sex is B = hermaphrodite
	traitNum_.Append(++runningTraits);
	traitTypesIncluded_[1]=true;
	traitType_.Append(1);
	SetSexTrait();

	//the read-in populations might not match the popID numbers
	ObjList<aList<int> >* popIDgridPtr = theLandScape.PopIDGrid(species);
	ObjList<aList<int> >& popIDgrid = *popIDgridPtr;
	aList<int> popIDs;//linearize the list
	int popcount=-1;
	for(r=0;r<popIDgrid.len();++r){
		for(c=0;c<popIDgrid[r].len();++c){
			++popcount;
			if(popNumbers.Includes(popcount)){
				popIDs.Append(popIDgrid[r][c]);
				}
			}}
	//set up data arrays
	Individual* sampleIndividual;
	MultiInheritanceModeContainer<int> sampleGenome, maxGenome;
	CoordinateType xy(-1,-1);
	int lociToSortThrough = 0;
	if(expectMissingData){
			//run once through the data and count up the loci,
			//then make up the datas arrays and run through later to fill them
			StatList<int> nLociByType;
			for(trt=0;trt<MultiInheritanceModeContainer<int>::inheritanceModes_;++trt){
				nLociByType.Append(0);}
			for (pop=0;pop<npop_;++pop){
				theLandScape.GetCoordinates(species,popIDs[pop],xy);
				Population& samplePop = theLandScape.GetPopulation(species, xy);
				for(i=0;i<samplePop.size();++i){
					sampleIndividual = &(samplePop.GetIndividual(i));
					sampleIndividual->GetGenome(sampleGenome);
					for(trt=0;trt<MultiInheritanceModeContainer<int>::inheritanceModes_;++trt){
						int lociByType = sampleGenome.loci((MultiInheritanceModeContainer<int>::inheritanceMode)trt);
						if(lociByType > nLociByType[trt]){
							nLociByType[trt] = lociByType;
							if(trt==MultiInheritanceModeContainer<int>::diploid || trt==MultiInheritanceModeContainer<int>::sexLink){
									maxGenome.SetSize((MultiInheritanceModeContainer<int>::inheritanceMode)trt,2,lociByType);}
								else {
									maxGenome.SetSize((MultiInheritanceModeContainer<int>::inheritanceMode)trt,1,lociByType);}
							}
						}//trt
					}//i
				}//pop
			}//expectMissingData=true
		else{//just use the first individual to size the data arrays 
			for (pop=0;pop<npop_;++pop){
				theLandScape.GetCoordinates(species,popIDs[pop],xy);
				Population& samplePop = theLandScape.GetPopulation(species, xy);
				if(samplePop.size()>0){
					sampleIndividual = &(samplePop.GetIndividual(0));
					break;}
				}//pop
			sampleIndividual->GetGenome(maxGenome);
			}
	lociToSortThrough = maxGenome.loci();

	ntraits_ += lociToSortThrough;
	for(loc=0;loc<lociToSortThrough;++loc){
		aString *locName = NEW aString("LOC_");
		aString locNum(loc+1);
		locNum.PrependCharToLen('0', Digits(ntraits_));
		locName->cat(locNum);
		traitName_.Append(locName);
		traitNum_.Append(++runningTraits);
		actualAlleleNames_.Append(NEW StringList());
		int trtType = maxGenome.inheritanceType(loc);
		switch (trtType){//translate to MultiformatMarkerData categories
			case 0://haploid
				traitTypesIncluded_[hap]=true;
				traitType_.Append(hap); 
				nlociHap_++;
				locusNumHap_.Append(loc+2);
				allHapLoci_.Append(loc+2);
				break;
			case 1://diploid
				traitTypesIncluded_[codom]=true;
				traitType_.Append(codom); 
				nlociCodom_++;
				locusNumCodom_.Append(loc+2);
				allCodomLoci_.Append(loc+2);
				break;
			case 2://sexLink
				traitTypesIncluded_[SLcodom]=true;
				traitType_.Append(SLcodom); 
				nlociSLCodom_++;
				locusNumSLCodom_.Append(loc+2);
				allSLCodomLoci_.Append(loc+2);
				break;
			case 3://cytoFemale
				traitTypesIncluded_[CYfemale]=true;
				traitType_.Append(CYfemale); 
				nlociCYfemale_++;
				locusNumCYfemale_.Append(loc+2);
				allCYfemaleLoci_.Append(loc+2);
				break;
			case 4://cytoMale
				traitTypesIncluded_[CYmale]=true;
				traitType_.Append(CYmale); 
				nlociCYmale_++;
				locusNumCYmale_.Append(loc+2);
				allCYmaleLoci_.Append(loc+2);
				break;
			default://other
				traitType_.Append(0); 
				break;
			}//switch
		}//end for loc
	sampleIndividual = NULL; //forget this

	//read in the loci to be skipped in the analyses
	skippedTraitNum_ = skippedLocusNumbers;
	for(loc=0;loc<skippedTraitNum_.len();++loc){
		skippedTraitNum_[loc]+=2;}//adjust for ID and sex traits
	ntraits_ -= skippedLocusNumbers.len();

	
	//read in population data
	MultiInheritanceModeContainer<int> thisGenome;
	int emptyPops = 0, geneVal;
	aString alleleVal;
	for (pop=0;pop<npop_;++pop){
		theLandScape.GetCoordinates(species,popIDs[pop],xy);
		Population& thePop = theLandScape.GetPopulation(species, xy);
//		GeneticPopulation* thePopG = (GeneticPopulation*) &thePopP;
//		GeneticPopulation& thePop = *thePopG;
		if(thePop.size()==0){
				++emptyPops;}
			else{
				//put the {x,y} coordinates of the population onto its name
				aString *s = NEW aString("Population ");
				s->cat(pop+1);
				s->cat(" y= "); s->cat(xy[1]);
				s->cat(" x= "); s->cat(xy[0]);
				popName_.Append(s);
				popSize_.Append(thePop.size());

				//make a new sublist for the genotypes of this population
				ObjList<TextDataList> *rawGList = NEW ObjList<TextDataList>();
				rawTraits_.Append(rawGList);				

				// BEAST is the string for a particular individual
				for(i=0;i<thePop.size();++i){
					Individual& indiv = thePop.GetIndividual(i);
					indiv.GetGenome(thisGenome);
					TextDataList* beast = NEW TextDataList();
					rawGList->Append(beast);
					beast->Append(NEW TextDatum(thisGenome.ID()));//ID
					beast->Append(NEW TextDatum(thisGenome.sexChar()));//ID
					if(maxGenome.haploidDataIncluded()){
						for(loc=0;loc<maxGenome.haploidData().loci();++loc){
							if(thisGenome.missing(MultiInheritanceModeContainer<int>::haploid,0)){
									alleleVal='?';
									alleleVal.PrependCharToLen('?',alleleLen());}
								else{
									geneVal = thisGenome.haplotype(0,loc,0);
									alleleVal = geneVal;
									alleleVal.PrependCharToLen('0',alleleLen());
									}
							beast->Append(NEW TextDatum(alleleVal));
							}}
					if(maxGenome.diploidDataIncluded()){
						for(loc=0;loc<maxGenome.diploidData().loci();++loc){
							aString gtype;
							for(c=0;c<2;++c){
								if(thisGenome.missing(MultiInheritanceModeContainer<int>::diploid,c)){
										alleleVal='?';
										alleleVal.PrependCharToLen('?',alleleLen());}
									else{
										geneVal = thisGenome.haplotype(1,loc,c);
										alleleVal = geneVal;
										alleleVal.PrependCharToLen('0',alleleLen());
										}
								gtype.cat(alleleVal);
								}
							beast->Append(NEW TextDatum(gtype));
							}}
					if(maxGenome.sexLinkedDataIncluded()){
						for(loc=0;loc<maxGenome.sexLinkedData().loci();++loc){
							aString gtype;							
							if(thisGenome.missing(MultiInheritanceModeContainer<int>::sexLink,0)){
									alleleVal='?';
									alleleVal.PrependCharToLen('?',alleleLen());}
								else{
									geneVal = thisGenome.haplotype(2,loc,0);
									alleleVal = geneVal;
									alleleVal.PrependCharToLen('0',alleleLen());
									}
							gtype.cat(alleleVal);
							if(thisGenome.IsHeterogameticSex()){
									gtype.AppendCharToLen(' ',alleleLen());
									}
								else{
									if(thisGenome.missing(MultiInheritanceModeContainer<int>::sexLink,1)){
											alleleVal='?';
											alleleVal.PrependCharToLen('?',alleleLen());}
										else{
											geneVal = thisGenome.haplotype(2,loc,1);
											alleleVal = geneVal;
											alleleVal.PrependCharToLen('0',alleleLen());
											}
									gtype.cat(alleleVal);
									}
							beast->Append(NEW TextDatum(gtype));
							}}
					if(maxGenome.cytoFemaleDataIncluded()){
						for(loc=0;loc<maxGenome.cytoFemaleData().loci();++loc){
							if(thisGenome.missing(MultiInheritanceModeContainer<int>::cytoFemale,0)){
									alleleVal='?';
									alleleVal.PrependCharToLen('?',alleleLen());}
								else{
									geneVal = thisGenome.haplotype(3,loc,0);
									alleleVal = geneVal;
									alleleVal.PrependCharToLen('0',alleleLen());
									}
							beast->Append(NEW TextDatum(alleleVal));
							}}
					if(maxGenome.cytoMaleDataIncluded()){
						for(loc=0;loc<maxGenome.cytoMaleData().loci();++loc){
							if(thisGenome.missing(MultiInheritanceModeContainer<int>::cytoMale,0)){
									alleleVal='?';
									alleleVal.PrependCharToLen('?',alleleLen());}
								else{
									geneVal = thisGenome.haplotype(4,loc,0);
									alleleVal = geneVal;
									alleleVal.PrependCharToLen('0',alleleLen());
									}
							beast->Append(NEW TextDatum(alleleVal));
							}}		
					beast=NULL;
					}//end for i
				rawGList=NULL;
			}//end if size()...else
		}//end for pop
	npop_ -= emptyPops;

	GroupMembership();
	LocalityMembership();
	CullTraits(skippedTraitNum_);
	ParseGenotypicData();
	CheckForNewAlleles();
	delete popIDgridPtr; popIDgridPtr=NULL;
	}//end class GelJockLandScapeInterface::ReadGeneticLandScape




//void GelJockLandScapeInterface::ReadGeneticLandScape(GeneticLandScape& theLandScape, 
void GelJockLandScapeInterface::ReadGeneticLandScape(LandScape& theLandScape, 
				aString& simTitle, aList<int>& skippedLocusNumbers, int maxAlleleLen, bool expectMissingData){
	aList<int> popNumbers;
	int i;
	for(i=0;i<theLandScape.area();++i){
		popNumbers.Append(i);}
	ReadGeneticLandScape(theLandScape, simTitle, skippedLocusNumbers, popNumbers, maxAlleleLen, expectMissingData);
	}//end 




//void GelJockLandScapeInterface::ReadGeneticLandScape(GeneticLandScape& theLandScape, 
void GelJockLandScapeInterface::ReadGeneticLandScape(LandScape& theLandScape, 
				aString& simTitle, aList<int>& skippedLocusNumbers, aList<int>& popNumbers, 
				int maxAlleleLen, bool expectMissingData){
		//all species
	Clear();
	int s, x, y, c, pop, i, loc, trt;
	for(trt=0;trt<numTraitTypes_;++trt){
		traitTypesIncluded_.Append(false);
		}
	title_ = simTitle;

	npop_ = 0;
	ngroups_ = theLandScape.species();
	SetAlleleLength(maxAlleleLen);
	
	int runningTraits = -1;
	ntraits_ = 2;//ID & sex
	traitName_.Append(NEW aString("ID"));
	traitNum_.Append(++runningTraits);
	traitTypesIncluded_[0]=true;
	traitType_.Append(0);
	traitName_.Append(NEW aString("sex"));//default sex is B = hermaphrodite
	traitNum_.Append(++runningTraits);
	traitTypesIncluded_[1]=true;
	traitType_.Append(1);
	SetSexTrait();

	aList<int> popsBySpecies;
	for(s=0;s<theLandScape.species();++s){
		npop_ += popNumbers.len();
		popsBySpecies.Append(popNumbers.len());
		}//s
		
	//collect population IDs
	ObjList<aList<int> > popID;
	int count=0;
	for(s=0;s<theLandScape.species();++s){
		popID.Append(NEW aList<int>());
		for (pop=0;pop<popsBySpecies[s];++pop){
			popID[s].Append(0);
			}}
	for(s=0;s<theLandScape.species();++s){
		int pop0=0,popcount=-1;
		for(x=0;x<theLandScape.length();++x){
			for(y=0;y<theLandScape.width();++y){
				++popcount;
				if(popNumbers.Includes(popcount)){
					CoordinateType popcoords(x,y);
					int idd =theLandScape.GetPopulation(s,popcoords).id();
					popID[s][pop0] = idd;
					pop0++;
					}
				}}
		}//s

	//set up data arrays
	Individual* sampleIndividual=NULL;
	MultiInheritanceModeContainer<int> sampleGenome, maxGenome;
	CoordinateType xy(-1,-1);
	int lociToSortThrough = 0, lociByType;
	StatList<int> nLociByType;
	for(trt=0;trt<MultiInheritanceModeContainer<int>::inheritanceModes_;++trt){
		nLociByType.Append(0);}
	if(expectMissingData){
			//run once through the data and count up the loci,
			//then make up the datas arrays and run through later to fill them
			for(s=0;s<theLandScape.species();++s){
				for (pop=0;pop<popsBySpecies[s];++pop){
					theLandScape.GetCoordinates(s,popID[s][pop],xy);
					Population& samplePop = theLandScape.GetPopulation(s, xy);
					for(i=0;i<samplePop.size();++i){
						sampleIndividual = &(samplePop.GetIndividual(i));
						sampleIndividual->GetGenome(sampleGenome);
						for(trt=0;trt<MultiInheritanceModeContainer<int>::inheritanceModes_;++trt){
							lociByType = sampleGenome.loci((MultiInheritanceModeContainer<int>::inheritanceMode)trt);
							if(lociByType > nLociByType[trt]){
								nLociByType[trt] = lociByType;
								if(trt==MultiInheritanceModeContainer<int>::diploid || trt==MultiInheritanceModeContainer<int>::sexLink){
										maxGenome.SetSize((MultiInheritanceModeContainer<int>::inheritanceMode)trt,2,lociByType);}
									else {
										maxGenome.SetSize((MultiInheritanceModeContainer<int>::inheritanceMode)trt,1,lociByType);}
								}
							}//trt
						}//i
					}//pop
				}//s
			}//expectMissingData=true
		else{//just use the first individual of each species to size the data arrays 
			for(s=0;s<theLandScape.species();++s){
				for (pop=0;pop<popsBySpecies[s];++pop){
					theLandScape.GetCoordinates(s,popID[s][pop],xy);
					Population& samplePop = theLandScape.GetPopulation(s, xy);
					if(samplePop.size()>0){
						sampleIndividual = &(samplePop.GetIndividual(0));
						break;}
					}//pop
				sampleIndividual->GetGenome(sampleGenome);
				for(trt=0;trt<MultiInheritanceModeContainer<int>::inheritanceModes_;++trt){
					lociByType = sampleGenome.loci((MultiInheritanceModeContainer<int>::inheritanceMode)trt);
					if(lociByType > nLociByType[trt]){
						nLociByType[trt] = lociByType;
						if(trt==MultiInheritanceModeContainer<int>::diploid || trt==MultiInheritanceModeContainer<int>::sexLink){
								maxGenome.SetSize((MultiInheritanceModeContainer<int>::inheritanceMode)trt,2,lociByType);}
							else {
								maxGenome.SetSize((MultiInheritanceModeContainer<int>::inheritanceMode)trt,1,lociByType);}
						}
					}//trt
				}//s
			}//expectMissingData=false
	lociToSortThrough = maxGenome.loci();

	ntraits_ += lociToSortThrough;
	for(loc=0;loc<lociToSortThrough;++loc){
		aString *locName = NEW aString("LOC_");
		aString locNum(loc+1);
		locNum.PrependCharToLen('0', Digits(ntraits_));
		locName->cat(locNum);
		traitName_.Append(locName);
		traitNum_.Append(++runningTraits);
		actualAlleleNames_.Append(NEW StringList());
		int trtType = maxGenome.inheritanceType(loc);
		switch (trtType){//translate to MultiformatMarkerData categories
			case 0://haploid
				traitTypesIncluded_[hap]=true;
				traitType_.Append(hap); 
				nlociHap_++;
				locusNumHap_.Append(loc+2);
				allHapLoci_.Append(loc+2);
				break;
			case 1://diploid
				traitTypesIncluded_[codom]=true;
				traitType_.Append(codom); 
				nlociCodom_++;
				locusNumCodom_.Append(loc+2);
				allCodomLoci_.Append(loc+2);
				break;
			case 2://sexLink
				traitTypesIncluded_[SLcodom]=true;
				traitType_.Append(SLcodom); 
				nlociSLCodom_++;
				locusNumSLCodom_.Append(loc+2);
				allSLCodomLoci_.Append(loc+2);
				break;
			case 3://cytoFemale
				traitTypesIncluded_[CYfemale]=true;
				traitType_.Append(CYfemale); 
				nlociCYfemale_++;
				locusNumCYfemale_.Append(loc+2);
				allCYfemaleLoci_.Append(loc+2);
				break;
			case 4://cytoMale
				traitTypesIncluded_[CYmale]=true;
				traitType_.Append(CYmale); 
				nlociCYmale_++;
				locusNumCYmale_.Append(loc+2);
				allCYmaleLoci_.Append(loc+2);
				break;
			default://other
				traitType_.Append(0); 
				break;
			}//switch
		}//end for loc
	sampleIndividual = NULL; //forget this


	//read in the loci to be skipped in the analyses
	skippedTraitNum_ = skippedLocusNumbers;
	for(loc=0;loc<skippedTraitNum_.len();++loc){
		skippedTraitNum_[loc]+=2;}//adjust for ID and sex traits
	ntraits_ -= skippedLocusNumbers.len();


			
	//read in population data
	MultiInheritanceModeContainer<int> thisGenome;
	int emptyPops = 0, geneVal;
	aString alleleVal, gtype;
	for(s=0;s<theLandScape.species();++s){
		for (pop=0;pop<popsBySpecies[s];++pop){
			theLandScape.GetCoordinates(s,popID[s][pop],xy);
			Population& thePop = theLandScape.GetPopulation(s, xy);
			if(thePop.size()==0){
					++emptyPops;}
				else{
					//put the {x,y} coordinates of the population onto its name
					aString *str = NEW aString("Population ");
					str->cat(pop+1);
					str->cat(" species= "); str->cat(s+1);
					str->cat(" y= "); str->cat(xy[1]);
					str->cat(" x= "); str->cat(xy[0]);
					popName_.Append(str);
					popSize_.Append(thePop.size());

					//make a new sublist for the genotypes of this population
					ObjList<TextDataList> *rawGList = NEW ObjList<TextDataList>();
					rawTraits_.Append(rawGList);				

					// BEAST is the string for a particular individual
					for(i=0;i<thePop.size();++i){
						Individual& indiv = thePop.GetIndividual(i);
						indiv.GetGenome(thisGenome);
//						indiv.Print(cout,' ');
//						thisGenome.Print(cout,' ');
						TextDataList* beast = NEW TextDataList();
						rawGList->Append(beast);
						beast->Append(NEW TextDatum(thisGenome.ID()));//ID
						beast->Append(NEW TextDatum(thisGenome.sexChar()));//ID
						if(maxGenome.haploidDataIncluded()){
							for(loc=0;loc<maxGenome.haploidData().loci();++loc){
								if(thisGenome.missing(MultiInheritanceModeContainer<int>::haploid,0)){
										alleleVal='?';
										alleleVal.PrependCharToLen('?',alleleLen());}
									else{
										geneVal = thisGenome.haplotype(0,loc,0);
										alleleVal = geneVal;
										alleleVal.PrependCharToLen('0',alleleLen());}
								beast->Append(NEW TextDatum(alleleVal));
								}}
						if(maxGenome.diploidDataIncluded()){
							for(loc=0;loc<maxGenome.diploidData().loci();++loc){
								gtype.clear();
								for(c=0;c<2;++c){
									if(thisGenome.missing(MultiInheritanceModeContainer<int>::diploid,c)){
											alleleVal='?';
											alleleVal.PrependCharToLen('?',alleleLen());}
										else{
											geneVal = thisGenome.haplotype(1,loc,c);
											alleleVal = geneVal;
											alleleVal.PrependCharToLen('0',alleleLen());
											}
									gtype.cat(alleleVal);
									}
								beast->Append(NEW TextDatum(gtype));
								}}
						if(maxGenome.sexLinkedDataIncluded()){
							for(loc=0;loc<maxGenome.sexLinkedData().loci();++loc){
								gtype.clear();							
								if(thisGenome.missing(MultiInheritanceModeContainer<int>::sexLink,0)){
										alleleVal='?';
										alleleVal.PrependCharToLen('?',alleleLen());}
									else{
										geneVal = thisGenome.haplotype(2,loc,0);
										alleleVal = geneVal;
										alleleVal.PrependCharToLen('0',alleleLen());
										}
								gtype.cat(alleleVal);
								if(thisGenome.IsHeterogameticSex()){
						//				gtype.AppendCharToLen(' ',alleleLen());}
										gtype.catChar(' ',alleleLen());}
									else{
										if(thisGenome.missing(MultiInheritanceModeContainer<int>::sexLink,1)){
												alleleVal='?';
												alleleVal.PrependCharToLen('?',alleleLen());}
											else{
												geneVal = thisGenome.haplotype(2,loc,1);
												alleleVal = geneVal;
												alleleVal.PrependCharToLen('0',alleleLen());}
										gtype.cat(alleleVal);
										}
								beast->Append(NEW TextDatum(gtype));
								}}
						if(maxGenome.cytoFemaleDataIncluded()){
							for(loc=0;loc<maxGenome.cytoFemaleData().loci();++loc){
								if(thisGenome.missing(MultiInheritanceModeContainer<int>::cytoFemale,0)){
										alleleVal='?';
										alleleVal.PrependCharToLen('?',alleleLen());}
									else{
										geneVal = thisGenome.haplotype(3,loc,0);
										alleleVal = geneVal;
										alleleVal.PrependCharToLen('0',alleleLen());}
								beast->Append(NEW TextDatum(alleleVal));
								}}
						if(maxGenome.cytoMaleDataIncluded()){
							for(loc=0;loc<maxGenome.cytoMaleData().loci();++loc){
								if(thisGenome.missing(MultiInheritanceModeContainer<int>::cytoMale,0)){
										alleleVal='?';
										alleleVal.PrependCharToLen('?',alleleLen());}
									else{
										geneVal = thisGenome.haplotype(4,loc,0);
										alleleVal = geneVal;
										alleleVal.PrependCharToLen('0',alleleLen());}
								beast->Append(NEW TextDatum(alleleVal));
								}}		
						beast=NULL;
						}//end for i
					rawGList=NULL;
				}//end if size()...else
			}//end for pop
		}//s
	npop_ -= emptyPops;

	GroupMembership();
	LocalityMembership();
	CullTraits(skippedLocusNumbers);
	ParseGenotypicData();
	CheckForNewAlleles();
	 }//end class GelJockLandScapeInterface::ReadGeneticLandScape








//void GelJockLandScapeInterface::SeedGeneticLandScape(GeneticLandScape& theLandScape, 
void GelJockLandScapeInterface::SeedGeneticLandScape(LandScape& theLandScape, 
 						int species, Ran3List& ran, LandScapeInitializer01& init, int useFileCoords){
 	//fills a properly configured GeneticLandScape with stored, unlinked genotypes
 	//if useFileCoords, then the coordinates of each population is read off the data file
 	aString endCharList(" ,{}();");
 	//get coordinates from population title
//	GeneticGridCell* g;
	GridCell* g;
 	theLandScape.grid_.Clear();
 	int x=0, y=-1, popp, i, loc, a, xx, yy;
	theLandScape.grid_.Append(NEW ObjList<GridCell>());
	aList<long double> extraInfo;//[species][pop][indiv]
	extraInfo.Append((long double)species);
	extraInfo.Append(zero);
	extraInfo.Append(zero);
 	for(popp=0;popp<npop_;++popp){
 		if(useFileCoords){
 				aString poptitle(popName_[popp]);
 				aString *ycoord = poptitle.ChopOffLastWord(endCharList);
 				aString *xcoord = poptitle.ChopOffLastWord(endCharList);
 				x=xcoord->toInt();
 				y=ycoord->toInt();
				delete xcoord, ycoord;
				CoordinateType xycoords(x,y);
				if(x>=theLandScape.grid_.len()){
					for(xx=theLandScape.grid_.len();xx<x;++xx){
						theLandScape.grid_.Append(NEW ObjList<GridCell>());}}
				if(y<=theLandScape.grid_[x].len()){
					for(yy=theLandScape.grid_[x].len();yy<y;++yy){
//						theLandScape.grid_[x].Append(NEW GeneticGridCell(&theLandScape, xycoords,0));}}
						theLandScape.grid_[x].Append(NEW GridCell(&theLandScape, xycoords,0));}}
 				}
 			else{//just go sequentially through grid
 				y++;
 				if(y>=theLandScape.width()){
 					y=0;x++;
 					theLandScape.grid_.Append(NEW ObjList<GridCell>());}
 				CoordinateType xycoords(x,y);
// 				g = NEW GeneticGridCell(&theLandScape, xycoords,0);//habitat=0
 				g = NEW GridCell(&theLandScape, xycoords,0);//habitat=0
				theLandScape.grid_[x].Append(g);
 				}
 		CoordinateType coords(x,y);
		Population* pop= init.popOfCorrectType->Duplicate(
								theLandScape.totalPopsCreated_++,species,g,
								theLandScape.maxCapacity_[species]);
		g->Append(pop);
		int hab = g->habitat();
 		//make Genotype objects and fill with correct genotypes, then Install
 		aList<int> ploidyList;
 		for(loc=0;loc<loci();++loc){
 			ploidyList.Append(Ploidy(loc));}
		extraInfo[1]=(double) popp;
		for (i=0;i<popSize(popp);++i){//GENERALIZE THIS SO ANY INDIVIDUAL SUBCLASS WITH GENES CAN BE INITIALIZED
			Individual* indiv = init.indivOfCorrectType->Duplicate(++theLandScape.totalIndivsCreated_, NULL);
			if(init.initPhenes_.len()>0){//treat as GeneticIndvidual and use genotype classes 
				GeneticIndividual* gIndiv = (GeneticIndividual*) indiv;
				Genotype newGenes(ploidyList, 0);//makes a null genotype
				for(loc=0;loc<loci();++loc){
					for(a=0;a<ploidyList[loc];++a){
						TextDatum& gStr = GenotypicVal(popp,i,loc,a);
						int alleleVal = alleleID(loc, gStr);
						newGenes.set(loc,a,alleleVal);
						}//end for a
					}//end for loc
				//install into population
				Phenotype& phen =init.initPhenes_[species][hab][0];
				ViabilitySelectionFunc viabSelection = phen.selectionFunc;
				newGenes.SetSelectionFunction(viabSelection);
				gIndiv->genes_ = newGenes.Duplicate();
				}//otherwise assume indiv uses its own genotypic structure
			long double m = theLandScape.moveRate(species,theLandScape.habitat(coords),B,1);
			extraInfo[2]=(double) i;
			indiv->IIndividual(B, adult, m, ran, init,extraInfo);//age=1
			theLandScape.InstallInGrid(species, indiv, coords);
			}//end for i
		}//end for popp
 	}//end  GelJockLandScapeInterface::SeedGeneticLandScape



//void GelJockLandScapeInterface::SeedLandScape(GeneticLandScape& theLandScape, 
void GelJockLandScapeInterface::SeedLandScape(LandScape& theLandScape, 
 						int species, Ran3List& ran, LandScapeInitializer01& init, int useFileCoords){
 	//generic Individual subclasses assumed
 	//fills a properly configured LandScape with stored genotypes
 	//if useFileCoords, then the coordinates of each population is read off the data file
 	aString endCharList(" ,{}();");
 	//get coordinates from population title
//	GeneticGridCell* g;
	GridCell* g;
 	theLandScape.grid_.Clear();
 	int x=0, y=-1, popp, i, loc, a;
	theLandScape.grid_.Append(NEW ObjList<GridCell>());
	aList<long double> extraInfo;//[species][pop][indiv]
	extraInfo.Append((long double)species);
	extraInfo.Append(zero);
	extraInfo.Append(zero);
 	for(popp=0;popp<npop_;++popp){
 		if(useFileCoords){
 				aString poptitle(popName_[popp]);
 				aString *ycoord = poptitle.ChopOffLastWord(endCharList);
 				aString *xcoord = poptitle.ChopOffLastWord(endCharList);
 				x=xcoord->toInt();
 				y=ycoord->toInt();
				delete xcoord, ycoord;
				CoordinateType xycoords(x,y);
				if(x>=theLandScape.grid_.len()){
					for(int xx=theLandScape.grid_.len();xx<x;++xx){
						theLandScape.grid_.Append(NEW ObjList<GridCell>());}}
				if(y<=theLandScape.grid_[x].len()){
					for(int yy=theLandScape.grid_[x].len();yy<y;++yy){
//						theLandScape.grid_[x].Append(NEW GeneticGridCell(&theLandScape, xycoords,0));}}
						theLandScape.grid_[x].Append(NEW GridCell(&theLandScape, xycoords,0));}}
 				}
 			else{//just go sequentially through grid
 				y++;
 				if(y>=theLandScape.width()){
 					y=0;x++;
 					theLandScape.grid_.Append(NEW ObjList<GridCell>());}
 				CoordinateType xycoords(x,y);
// 				g = NEW GeneticGridCell(&theLandScape, xycoords,0);//habitat=0
 				g = NEW GridCell(&theLandScape, xycoords,0);//habitat=0
				theLandScape.grid_[x].Append(g);
 				}
 		CoordinateType coords(x,y);
		Population* pop= init.popOfCorrectType->Duplicate(
								theLandScape.totalPopsCreated_++,species,g,
								theLandScape.maxCapacity_[species]);
		g->Append(pop);
		int hab = g->habitat();
 		//make Individual objects and fill with correct genotypes, then Install
 		aList<int> ploidyList;
 		for(loc=0;loc<loci();++loc){
 			ploidyList.Append(Ploidy(loc));}
		extraInfo[1]=(long double)popp;
		for (i=0;i<popSize(popp);++i){//GENERALIZE THIS SO ANY INDIVIDUAL SUBCLASS WITH GENES CAN BE INITIALIZED
			Individual* indiv = init.indivOfCorrectType->Duplicate(++theLandScape.totalIndivsCreated_, NULL);
			if(init.initPhenes_.len()>0){//treat as GeneticIndvidual and use genotype classes 
				GeneticIndividual* gIndiv = (GeneticIndividual*) indiv;
				Genotype newGenes(ploidyList, 0);//makes a null genotype
				for(loc=0;loc<loci();++loc){
					for(a=0;a<ploidyList[loc];++a){
						TextDatum& gStr = GenotypicVal(popp,i,loc,a);
						int alleleVal = alleleID(loc, gStr);
						newGenes.set(loc,a,alleleVal);
						}//end for a
					}//end for loc
				//install into population
				Phenotype& phen =init.initPhenes_[species][hab][0];
				ViabilitySelectionFunc viabSelection = phen.selectionFunc;
				newGenes.SetSelectionFunction(viabSelection);
				gIndiv->genes_ = newGenes.Duplicate();
				}//otherwise assume indiv uses its own genotypic structure
			long double m = theLandScape.moveRate(species,theLandScape.habitat(coords),B,1);
			extraInfo[2]=(long double) i;
			indiv->IIndividual(B, adult, m, ran, init, extraInfo);//age=1
			theLandScape.InstallInGrid(species, indiv, coords);
			}//end for i
		}//end for popp
 	}//end  GelJockLandScapeInterface::SeedLandScape









