//GeneFrequencyDistMultiAllele.cp

#ifndef GeneFrequencyDistMultiAllelecpp
#define GeneFrequencyDistMultiAllelecpp = 1

#include "GeneFrequencyDistMultiAllele.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"

//20081229  bug fix in RandomFrequencies() to handle ploidy != 2

void GeneFrequencyDistMultiAllele::Initialize(const long double Fst, aList<long double>& pBar){
	Fst_ = Fst;  
	pBarList_.Clear();
	int i;
	for(i=0;i<pBar.len();++i){
		pBarList_.Append(pBar[i]);}
	for(i=0;i<pBarList_.len();++i){
		if(pBarList_[i]==(long double) 1.0){
			fixed_=1;fixedAllele_=i;break;}
		}//end for i
	}//Initialize

void GeneFrequencyDistMultiAllele::Initialize(const long double Fst, const long double pBar){
	Fst_ = Fst;  
	pBarList_.Clear();
	pBarList_.Append(pBar);
	pBarList_.Append(one - pBar);
	fixed_=0; fixedAllele_=-1;
	int i;
	for(i=0;i<pBarList_.len();++i){
		if(pBarList_[i]==one){
			fixed_=1;fixedAllele_=i;break;}
		}//end for i
	}//Initialize


void GeneFrequencyDistMultiAllele::Initialize(GeneFrequencyDistribution& d){
	//assumes d => GeneFrequencyDistMultiAllele
	GeneFrequencyDistMultiAllele* dmp = (GeneFrequencyDistMultiAllele*) &d;
	Fst_ = dmp->Fst_;
	pBarList_ = dmp->pBarList_;
	}//GeneFrequencyDistMultiAllele::Initialize(GeneFrequencyDistribution& )




long double GeneFrequencyDistMultiAllele::ProbabilityAtFrequencyP(aList<long double>& pList, long double dp){
	//implements Wright Vol.2 eqn 13.48, p. 362
	if(pList.len() != pBarList_.len()){
		aString err("ERROR in GeneFrequencyDistMultiAll2::ProbabilityAtFrequency() -- lists of different lengths.");
		aError(err); 
		return zero;}
	long double fourNm = two*ploidy_*Nm();//scaled properly for any ploidy
	long double lncoefficientNumerator = gammaln(fourNm);
	long double lncoefficientDenominator = zero;
	long double lnFreqToPower = zero;
	int a;
	for(a=0;a<pList.len();++a){
		lncoefficientDenominator += gammaln(fourNm * pBarList_[a]);
		lnFreqToPower += (fourNm * pBarList_[a] - one) * ln(pList[a]);
		}
	long double lncoefficient = lncoefficientNumerator - lncoefficientDenominator;
	long double answer=(long double)1000.0, sum =lncoefficient + lnFreqToPower + (pList.len()-1)*ln(dp);//integrates over independent alleles
	if (sum<ln(answer) ){//handles discontinuity at 0 and 1 when Nm<0.5
		answer = exp(sum);}
	return answer;
}//end GeneFrequencyDistMultiAllele::ProbabilityAtFrequencySet


aList<long double>*	GeneFrequencyDistMultiAllele::RandomFrequencies(Ran3& ran){
	// uses p[i] = X[i]/(X[1]+X[2]+...+X[n]) where X[i] is a random deviate from a gamma distribution
	//with gamma parameters a=Pbar*(1/Fst - 1) and b=1
	aList<long double>* ranFreqsPtr = NEW aList<long double>();
	aList<long double>& ranFreqs = *ranFreqsPtr;
	
	int a, alleles = pBarList_.len();
	if(Fst_ == zero){
		for(a=0;a<alleles;++a){//just copy
			ranFreqs.Append(pBarList_[a]);}
		return ranFreqsPtr;
		}//end if Fst_=0

//	long double NmAdj=(one-Fst_)/Fst_;
	long double alph, g, NmAdj=(one-Fst_)/(ploidy_*Fst_);
	StatList<long double> gammaVars;
	for(a=0;a<alleles;++a){
		alph = pBarList_[a] * NmAdj;
		g = ran.GammaDev(alph, one);
			gammaVars.Append(g);
			}//end a
	for(a=0;a<alleles;++a){//scale to total
		ranFreqs.Append(gammaVars[a]/gammaVars.sum());}

	return ranFreqsPtr;
	}//end GeneFrequencyDistMultiAllele::RandomFrequencies
	
	
/*	old version 
aList<double>*	GeneFrequencyDistMultiAllele::RandomFrequencies(Ran3& ran){
	aList<double>* ranFreqsPtr = NEW aList<double>();
	aList<double>& ranFreqs = *ranFreqsPtr;
//	long double NmAdj = Nm();//very old version
//	long double FstAdj = 1.0/(2.0*ploidy_*NmAdj + 1.0);
//	GeneFrequencyDistribution subDistribution0(FstAdj, pBarList_[0], ploidy_);
//	long double p = subDistribution0.RandomFrequency(ran);
//	ranFreqs.Append(p);
//	long double remainingParametricFreqSpace = 1.0 - pBarList_[0];
//	long double remainingFreqSpace = 1.0 - p;
//	long double sumFreqSamples = p, sumUnSampledFreq = 1.0 - p;

	long double NmAdj, p;
	long double remainingParametricFreqSpace = 1.0;
	long double remainingFreqSpace = 1.0;
	long double sumFreqSamples = 0.0, sumUnSampledFreq = 1.0;
	int lastExtantFreq = 0;//handle cases where pBarList_ has terminal zeros
	for(int aa=pBarList_.len()-1;aa>=0;--aa){
		if(pBarList_[aa]>0.0){
			lastExtantFreq = aa;
			break;}
		}//aa
	for(int a=0;a<lastExtantFreq;++a){
		long double pp=pBarList_[a];
		if((remainingFreqSpace<=0.0) || (pp==0.0)) {//all alleles found or this one absent
				ranFreqs.Append(0.0);}
			else if(pp==1.0){//fixed
				ranFreqs.Append(1.0);
				}
			else{//sample more alleles
				//make a biallelic distribution after reducing the sample size 
				//by the amount already selected
//				NmAdj = Nm() / (1.0 - remainingFreqSpace);//scales it down
				NmAdj = Nm() / remainingFreqSpace;//scales it down
//				double adjGlobalFreq =  pp / (1.0-remainingFreqSpace);//scales remaining distribution to length 1
				long double ppAdj = pp/remainingParametricFreqSpace;
				long double qqAdj = 1.0 - ppAdj;
				long double fourNm = (2.0*ploidy_) * NmAdj;//4Nm or 2Nm, but this notation is convenient
				double fourNmP = fourNm * ppAdj;
				double fourNmQ = fourNm * qqAdj;
//				p = RandomBetaDistribution(fourNmP,fourNmQ,ran);
				p = ran.BetaDev(fourNmP,fourNmQ);
				p *= remainingFreqSpace;//scales it back up
				ranFreqs.Append(p);
				sumFreqSamples += p;
				sumUnSampledFreq -= p;
				remainingFreqSpace -= p;
				}
		remainingParametricFreqSpace -= pp;
		}//end a
	ranFreqs.Append(remainingFreqSpace);
	for(int b=lastExtantFreq+1;b<pBarList_.len();++b){
		ranFreqs.Append(0.0);}	
	return ranFreqsPtr;
	}//end GeneFrequencyDistMultiAllele::RandomFrequencies
*/	
	
long double	GeneFrequencyDistMultiAllele::RandomFrequency(Ran3& /*ran*/){
	aString err("ERROR:  use RandomFrequencies(Ran3& ) when sampling from multi-allele distributions.  0 returned.");
	aError(err);
	return zero;
	}////end GeneFrequencyDistMultiAllele::RandomFrequency




#endif







