//GeneFrequencyDistribution.cpp


#ifndef GeneFrequencyDistributioncpp
#define GeneFrequencyDistributioncpp = 1


#include "GeneFrequencyDistribution.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"


long double GeneFrequencyDistribution::FreqP_AtProb_x(long double x){
	int estimate_x = 2;
	long double p = half;//unknown; parameter of beta distribution; dummy parameter here
	long double q = one - p; //unknown; complementary parameter of beta distribution
	long double a = two * ploidy_ * Nm() * pBar_;
	long double b = two * ploidy_ * Nm() * (one - pBar_);
	int errorStatus = 0;
	long double boundaryStatus = zero;
	long double y = one - x;//unknown here; complementary integral under beta distribution
	cdfbet(&estimate_x, &p, &q, &x, &y, &a, &b, &errorStatus, &boundaryStatus);
	if(errorStatus != zero){
		aString er(errorStatus);
		er.prepend("cumulative beta function cdfbet threw error #");
		aError(er);}
	if(boundaryStatus != zero){
		aString be(boundaryStatus); 
		be.prepend("cumulative beta function cdfbet threw boundary error #");
		aError(be);}
	return p;
	}//end GeneFrequencyDistribution::FreqP_AtProb_x(const long double)


long double GeneFrequencyDistribution::Nm(){
	return (one/Fst_ - one)/(two*ploidy_);
	}//GeneFrequencyDistribution::Nm()



long double GeneFrequencyDistribution::ProbabilityAtFrequencyP(long double p){
	//implements Wright Vol.2 eqn 13.48, p. 362
	if(Fst_==zero){//all pops identical
		if(p==pBar_){return one;}else{return zero;}
		}
	if(Fst_==one){//all pops fixed
		if(p==zero || p==one){return one;}else{return zero;}
		}
	long double Qbar = one - pBar_;
	long double fourNm = two * ploidy_ * Nm();//4Nm or 2Nm, but this notation is convenient
	long double lncoefficient = gammaln(fourNm);
	long double fourNmP = fourNm * pBar_;
	long double fourNmQ = fourNm * Qbar;
	lncoefficient -= gammaln(fourNmP);
	lncoefficient -= gammaln(fourNmQ);
	long double lnq = (fourNmP - one)*ln(p);
	long double ln1Minusq = (fourNmQ - one)*ln(one-p);
	long double sum = lncoefficient + lnq + ln1Minusq;
	long double answer = exp(sum);
	return answer;
}//end ProbabilityAtFrequencyP


long double GeneFrequencyDistribution::RandomFrequency(Ran3& ran){
	if(Fst_==zero){
		return pBar_;}
	if(Fst_==one){
		return (ran()<pBar_)? one:zero;}
	long double Qbar = one - pBar_;
	long double fourNm = (two*ploidy_) * Nm();//4Nm or 2Nm or even k*Nm, but this notation is convenient
	long double fourNmP = fourNm * pBar_;
	long double fourNmQ = fourNm * Qbar;
//	return RandomBetaDistribution(fourNmP,fourNmQ,ran);//using Gamma2.h
	return ran.BetaDev(fourNmP,fourNmQ);
	}//end GeneFrequencyDistribution::RandomFrequency(ran)



#endif
