// ClineGeneticData.h

#ifndef ClineGeneticDatacpp
#define ClineGeneticDatacpp = 1

#include "ClineGeneticData.h"
#include "Element.cpp"
#include "aList.cpp"


ClineGeneticData::ClineGeneticData(ClineGeneticData& parentData):
			MultiformatMarkerData(parentData), 
			MetropolisHastingsData(parentData), 
			diagnosticTraitNum_(parentData.diagnosticTraitNum_),
			diagnosticAllelesR_(parentData.diagnosticAllelesR_)
			{	
		}//end ctor ClineGeneticData(ClineGeneticData&, aList<int>&)


ClineGeneticData::ClineGeneticData(ClineGeneticData& parentData, aList<int>& popSubsetNums):
			MultiformatMarkerData(parentData,popSubsetNums), 
			diagnosticTraitNum_(parentData.diagnosticTraitNum_),
			diagnosticAllelesR_(parentData.diagnosticAllelesR_)
		{
	}//end ctor ClineGeneticData(ClineGeneticData&, aList<int>&)



void ClineGeneticData::AddDiagnosticAlleles(aString& aLocusName){
	//attaches list of diagnistic alleles to aString
	int traitloc, diagnosticPosition;
	aString err;
	if(traitName_.Includes(&aLocusName)){
			traitloc = traitName_.Loc(&aLocusName);
			}
		else{
			err="ERROR:  ClineGeneticData::AddDiagnosticAlleles(): no locus named ";
			err.cat(aLocusName);
			aError(err);
			return;
			}
	if(diagnosticTraitNum_.Includes(traitloc)){
			diagnosticPosition = diagnosticTraitNum_.Loc(traitloc);
			}
		else{
			err="ERROR:  ClineGeneticData::AddDiagnosticAlleles(): ";
			err.cat(aLocusName);
			err.cat(" is not diagnostic");
			aError(err);
			return;
			}
	StringList& diagAlleles = diagnosticAllelesR_[diagnosticPosition];
	if (diagAlleles.len()==0) return;
	aLocusName.cat("(");
	int a;
	for(a=0;a<diagAlleles.len();++a){
		aLocusName.cat(diagAlleles[a]);
		if(a<diagAlleles.len()-1){
			aLocusName.cat(";");
			}
		}//end for a
	aLocusName.cat(")");
	}//end ClineGeneticData::AddDiagnosticAlleles(aString&)



void ClineGeneticData::CalculateHybridIndex(aList<long double>& popLocation){
	hybridIndices_.Clear();
	HybridIndicesByLocus_.Clear();
	popLocation.Clear();
	long double hi,locHI,indexVal,nAlleles,locIndexVal,locPloidy, popX;
	aString endcharList(" ;:,/");
	int pop,i,tr,a;
	for(pop=0;pop<npop_;++pop){
		//get the population's location from the end of the title string
		aString popTitleWord(popName_[pop]);
	    aString* popXword = popTitleWord.ChopOffLastWord(endcharList);
    	popX = popXword->toLongDouble();
    	popLocation.Append(popX);
    	delete popXword; popXword=NULL;
		//get each individual's genotype and compare to diagnostic alleles
    	for (i=0;i<popSize_[pop];++i){
    		HybridIndicesByLocus_.Append(NEW HybridIndexByLocus());
      		indexVal = nAlleles = zero;
      		hi = -one;//use as error flag
    		for(tr=0;tr<traits();++tr){
    			if(traitType_[tr]>=2 && traitType_[tr]<=10){//it's a genetic trait
	    			locIndexVal = locPloidy = zero;
	    			locHI = -one;//use as error flag
	    			for(a=0;a<Ploidy(pop,i,tr);++a){
	    				aString* gene = GetTraitString(pop, i, tr, a);
	    				if((gene->len() != 0)&&(*gene != ' ')){
	    					++nAlleles;++locPloidy;
	    					if(diagnosticAllelesR_[tr].Includes(gene)){
	    						++indexVal;++locIndexVal;}
	    					}
	    				delete gene; gene = 0;
	    				}//end for a
	    			if(locPloidy>zero) {locHI = locIndexVal/locPloidy;}
	    			HybridIndicesByLocus_.Last().Append(NEW HybridIndexByLocation(locHI, locPloidy, popX));
	    			}//if a locus
    			}//end for tr
    		if(nAlleles >zero){hi = indexVal/nAlleles;}
    		hybridIndices_.Append(NEW HybridIndexByLocation(hi, nAlleles, popX));
    		}//end for i
    	}//end for pop
	}// end ClineGeneticData::CalculateHybridIndex(aList<double>&)




void ClineGeneticData::Clear(void){
	MultiformatMarkerData::Clear();
	MetropolisHastingsData::Clear();
	diagnosticTraitNum_.Clear();
	diagnosticAllelesR_.Clear();
	hybridIndices_.Clear();
	HybridIndicesByLocus_.Clear();
	}//end ClineGeneticData::Clear()


void ClineGeneticData::CullLoci (aList<int>& lociToCull){
	//removes relevant loci from the current data structures befre passing to the base class
	int trait, loc;
	for(trait=0;trait<lociToCull.len();++trait){
		if(diagnosticTraitNum_.Includes(trait)){
			loc = diagnosticTraitNum_[trait];
			diagnosticAllelesR_.Delete(loc);
			}
		}//trait
	MultiformatMarkerData::CullLoci (lociToCull);
	}//end procedure ClineGeneticData::CullLoci(StringList&)



void ClineGeneticData::Initialize(){
	MultiformatMarkerData::Initialize();
	}
	
	

void ClineGeneticData::Initialize(aString& a){
	MultiformatMarkerData::Initialize(a);
	}
	
	

ClineGeneticData* ClineGeneticData::PoolDiagnosticAlleles (void){
	if (diagnosticAllelesR_.len()==0){
		StoreDiagnosticAlleles();}
	ClineGeneticData* alloData = NEW ClineGeneticData(*this);
	alloData->ResetFlags();
	alloData->title_.cat(": diagnostic alleles pooled");
	//Go through each individual and change its genotype in alloData
		//entails going through each class of loci
	//An allele diagnostic on the left side is A, and on the right side is B
	//Alleles unique to the central part of the zone are arbitrarily pooled with the left side as A
	int pop,i,loc,a,b,tr, cLoc, slLoc;
	bool found;
	aString lName;
	TextDataList locusVals;
	TextDatum locVal;
	for(pop=0;pop<npop_;++pop){
	    for(i=0;i<alloData->rawTraits_[pop].len();++i){
			for(tr=0;tr<traits();++tr){
				if(diagnosticTraitNum_.Includes(tr)){
					lName=traitName_[tr];
					StripDiagnosticAlleles(lName);
					if(lName != alloData->traitName_[tr]){//lName had diagnostic alleles that were stripped
						//data not changed in this case
						//now figure out which inheritance pattern the trait has
						switch(traitType_[tr]){
							case 2://one marker
							case 4:
							case 6:
							case 7:
							case 8:
							case 9:
							case 10:
								locVal = rawTraits_[pop][i][tr][0];
								loc = diagnosticTraitNum_.Loc(tr);
								if(locVal.OK() && diagnosticAllelesR_[loc].Includes(&locVal)){
									//pool 
									found=false;
									for(b=0;b<diagnosticAllelesR_[loc].len();++b){
										if (diagnosticAllelesR_[loc][b] == rawTraits_[pop][i][tr]){
											rawTraits_[pop][i][tr] = "B";
											found=true;
											break;}//escape 'b' loop
										}//end for b
									if(!found){
										rawTraits_[pop][i][tr] = "A";}									
									}//ok & in list
								break;
							case 3://codom - two markers
								loc = diagnosticTraitNum_.Loc(tr);
								cLoc = allCodomLoci_.Loc(tr);
								for(a=0;a<Ploidy(pop,i,loc);++a){
									locVal = rawCodomGtypes_[pop][i][loc][a];
									if(locVal.OK() && diagnosticAllelesR_[loc].Includes(&locVal)){
										//pool 
										found=false;
										for(b=0;b<diagnosticAllelesR_[loc].len();++b){
											if (diagnosticAllelesR_[loc][b] == rawCodomGtypes_[pop][i][cLoc][a]){
												rawCodomGtypes_[pop][i][cLoc][a] = "B";
												found=true;
												break;}//escape 'b' loop
											}//end for b
										if(!found){
											rawCodomGtypes_[pop][i][cLoc][a] = "A";}									
										}//ok & in list
									}//a
								break;
							case 5:// <=1 marker
								loc = diagnosticTraitNum_.Loc(tr);
								slLoc = locusNumSLCodom_.Loc(tr);
								for(a=0;a<Ploidy(pop,i,loc);++a){
									locVal = rawTraits_[pop][i][tr][a];
									if(locVal.OK() && diagnosticAllelesR_[loc].Includes(&locVal)){
										//pool 
										found=false;
										for(b=0;b<diagnosticAllelesR_[loc].len();++b){
											if (diagnosticAllelesR_[loc][b] == rawSLCodomGtypes_[pop][i][slLoc][a]){
												rawSLCodomGtypes_[pop][i][slLoc][a] = "B";
												found=true;
												break;}//escape 'b' loop
											}//end for b
										if(!found){
											rawSLCodomGtypes_[pop][i][slLoc][a] = "A";}									
										}//ok & in list
									}//a
								break;
							default://skip non-genetic traits
								continue;
								break;
							}//switch
						}//trait has dagnostic alleles
					}//if diagnostic trait
				}//end for tr	
			}//end for i
		}//end for pop					
				
	//now replace the diagnostic alleles list for the revised data
	aString rName;
	for (tr=0;tr<alloData->traitName_.len();++tr){
		switch(traitType_[tr]){
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 7:
			case 8:
			case 9:
			case 10:		
				rName = alloData->traitName_[tr];
				StripDiagnosticAlleles(rName);
				if(rName != alloData->traitName_[loc]){//diagnostic alleles given
					StripDiagnosticAlleles(alloData->traitName_[tr]);
					alloData->traitName_[tr].cat("(B)");
					}				
				break;
			default:
				continue;
				break;
			}
		}//end for tr
	//now replace the alleles present list for the revised data
	alloData->CheckForNewAlleles();
	alloData->StoreDiagnosticAlleles();
	return alloData;
 	}//end ClineGeneticData::PoolDiagnosticAlleles()


void ClineGeneticData::PrintGelJockFile (aString& outFileName){
	MultiformatMarkerData::PrintGelJockFile(outFileName);}


void ClineGeneticData::PrintGelJockFile (ostream& outFile){
	MultiformatMarkerData::PrintGelJockFile(outFile);}


/*
void ClineGeneticData::PrintGelJockFile (ostream& outFile){
	//		theGenotypes: ListStrStructureTypePtr;
	aString dataLine = title_;
	dataLine.cat(":  skipped loci omitted");
	outFile<<dataLine<<endl;
	dataLine = "NPOP=";
	dataLine.cat(npop_);
	dataLine.cat(", NLOCI=");
	dataLine.cat(ntraits_);
	dataLine.cat(", MAXALL=");
	dataLine.cat(maxall());
	dataLine.cat(", NGROUPS=");
	dataLine.cat(ngroups_);
	outFile<<dataLine<<endl;
	dataLine.clear();
	int loc, pop,i;
	for (loc=0;loc<traitName_.len();++loc){
		aString lName(traitName_[loc]);
		AddDiagnosticAlleles(loc,lName);
		dataLine.cat(lName);
		if( ( ((long double)(loc+1)) / five) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < traitName_.len()){
				dataLine.cat(", ");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	dataLine = "SKIP=0";
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (pop=0;pop<npop_;++pop){
		outFile<<popName_[pop]<<endl;
		StringList& theGenotypes = gtypes_[pop];
		for (i=0;i<theGenotypes.len();++i){
			dataLine = theGenotypes[i];
			outFile<<dataLine<<endl;
			}//end for i
		outFile<<"NEXT"<<endl;
		}//end for pop
	outFile<<"END"<<endl;
//	return 1;
	}//end ClineGeneticData::PrintGelJockFile
*/


int ClineGeneticData::PrintHybridIndexFile(aString& outFileName){
	char* rawfilename = outFileName.raw();
	fstream outputData;
	outputData.open(rawfilename, ios::out /*,filebuf::openprot*/);
//	int opQ = outputData.is_open();
//	if(opQ==0) return 0;
	int i;
	for(i=0;i<hybridIndices_.len();++i){
		aString outline(hybridIndices_[i].x(),10,5); outline.cat('\t');	
		outline.cat(hybridIndices_[i].hi(),2,6); outline.cat('\t');	
		outline.cat(hybridIndices_[i].items());
		outputData<<outline<<endl; 
		}//end for i	
	outputData.close();
	delete [] rawfilename; rawfilename=0;
	return 1;
	}//end ClineGeneticData::PrintHybridIndexFile(aString&)



int ClineGeneticData::ReadGelJockDataFile (aString* dataFileName){
	int valid = MultiformatMarkerData::ReadGelJockDataFile (dataFileName);
	if(! valid) return valid;
	int loc;
	for (loc=0;loc<traitName_.len();++loc){
		aString& lName = traitName_[loc];
		StripDiagnosticAlleles(lName);
		}//end for loc
	StoreDiagnosticAlleles();
	return valid;
	}//end ClineGeneticData::ReadGelJockDataFile()


void ClineGeneticData::RestoreLoci (StringList& lociToRestore){
	//This function restores the designated loci to each genotype data string
	//and removes them from the skipped loci
	if(lociToRestore.len()==0){return;}//none to cull
	int rLoc,item,j,k,item1,i,inserted;
	for(rLoc=0;rLoc<lociToRestore.len();++rLoc){
		aString rLocusname = lociToRestore[rLoc];
		aString rLocusnameCopy(rLocusname);
		AddDiagnosticAlleles(rLocusname);
		item = traitName_.Loc(&rLocusname);
		aString rLnm;//stripped raw name stored here, processed below
		if(item>=0){//the locus exists in the list
			//retore to original location
			inserted = 0;
			for(j=traitName_.len()-1;j>=0;--j){
				//compare to locations of currently used loci
				aString& currLocusname = traitName_[j];
				//find the raw locus name's location
				for(k=0;k<traitName_.len();++k){
					aString& rLocName = traitName_[k];
					rLnm = rLocName;
					StripDiagnosticAlleles(rLnm);
					if(rLnm == currLocusname){//insert here
						traitName_.Insert(NEW aString(rLnm), j+1);
						inserted = 1;
						break;
						}//end if
					}//end for k
				}//j		
			if(! inserted){//goes at location 0
				traitName_.Prepend(NEW aString(rLnm));}
							
			item1 = skippedTraitNum_.Loc(item);
			if(item1>=0){//it's in the skipped list
				skippedTraitNum_.Delete(item1);
//				item2 = skippedLocusName.Loc(&rLocusname);
//				skippedLocusName.Delete(item2);
//				--lociSkipped;++nloci;
				}//if item exists in skipped list
			}//if item exists
		}//cLoc
	for(i=0;i<traitName_.len();++i){
		cout<<traitName_[i]<<",";}
	cout<<endl<<endl;
	for(i=0;i<traitName_.len();++i){
		cout<<traitName_[i]<<",";}
	cout<<endl;
	ResetFlags();
//	CullSkippedTraits();
	CheckForNewAlleles();
	}//end procedure ClineGeneticData::RestoreLoci


void ClineGeneticData::StoreDiagnosticAlleles(void){
	diagnosticAllelesR_.Clear();
	aString endCharList("()[]{}; ");
	int loc,num;
	for(loc=0;loc<traitName_.len();++loc){//do for every locus first
		diagnosticAllelesR_.Append(NEW StringList());
		aString nameAndDiagAlleles(traitName_[loc]);
		aString* lname = nameAndDiagAlleles.ChopOffFirstWord(endCharList);
		delete lname; lname = 0;//throw away
		do{
			aString* allname = nameAndDiagAlleles.ChopOffFirstWord(endCharList);
			allname->CullChar(' ');//cull spaces
			if(allname->len()==0) {
					delete allname; allname = 0;//throw away
					}
				else{
					diagnosticAllelesR_[loc].Append(allname);
					allname = 0;//forget this
					}
			}while(nameAndDiagAlleles.len()>0);
		}//end for loc
	for(loc=skippedTraitNum_.len()-1;loc>=0;--loc){//pull out skipped loci
		num = skippedTraitNum_[loc];
		diagnosticAllelesR_.Delete(num);
		}//end for loc	
	}//ClineGeneticData::StoreDiagnosticAlleles()


void ClineGeneticData::StripDiagnosticAlleles(aString& aLocusName){
	//strips list of diagnistic alleles to aString
	aString endCharList("()[]{}");
	int endOfName = aLocusName.posEndFirstWord(endCharList);
	aLocusName.trunc(endOfName-1);
	}//end StripDiagnosticAlleles(aString& aLocusName)


#endif



