//unit CodominantMarkerData.cpp

#ifndef CodominantMarkerDatacpp
#define CodominantMarkerDatacpp = 1

#include "CodominantMarkerData.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "GeneContainers.cpp"
#include "aVector.cpp"
#include "aMatrix.cpp"



CodominantMarkerData::CodominantMarkerData(CodominantMarkerData& parentData):
			nlociCodom_(parentData.nlociCodom_), 
			ploidy_(parentData.ploidy_), 
			alleleLen_(parentData.alleleLen_), 
			allCodomLoci_(parentData.allCodomLoci_),
			locusNumCodom_(parentData.locusNumCodom_),
			skippedLocusNumCodom_(parentData.skippedLocusNumCodom_),
			variableLocusNumCodom_(parentData.variableLocusNumCodom_),
			rawCodomGtypes_(parentData.rawCodomGtypes_),
			codomIndivs_(parentData.codomIndivs_),
			allelesPresent_(parentData.allelesPresent_),
			alleleFreq_(parentData.alleleFreq_),
			alleleFreqGlobal_(parentData.alleleFreqGlobal_),
			twoN_(parentData.twoN_),
			genotypeCount_(parentData.genotypeCount_),
			genotypeCountGlobal_(parentData.genotypeCountGlobal_),
			genotypeFreq_(parentData.genotypeFreq_),
			genotypeFreqGlobal_(parentData.genotypeFreqGlobal_),
			HetObsByAllele_(parentData.HetObsByAllele_),
			HetObsByLocus_(parentData.HetObsByLocus_),
			diseqD_(parentData.diseqD_),
			locusD_(parentData.locusD_),
			allelesCurrent_(parentData.allelesCurrent_),
			alleleFreqCurrent_(parentData.alleleFreqCurrent_),
			genotypeFreqCurrent_(parentData.genotypeFreqCurrent_),
			HetObsCurrent_(parentData.HetObsCurrent_), 
			varLociCurrent_(parentData.varLociCurrent_), 
			DCurrent_(parentData.DCurrent_) 
			{	
		}//end ctor CodominantMarkerData(CodominantMarkerData&)


CodominantMarkerData::CodominantMarkerData(CodominantMarkerData& parentData, aList<int>& popSubsetNums):
			EvolutionaryGeneticsData(parentData,popSubsetNums), nlociCodom_(parentData.nlociCodom_), 
			 ploidy_(parentData.ploidy_), alleleLen_(parentData.alleleLen_), 
			allCodomLoci_(parentData.allCodomLoci_),locusNumCodom_(parentData.locusNumCodom_),
			skippedLocusNumCodom_(parentData.skippedLocusNumCodom_){
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	for(int p=0;p<popSubsetNums.len();++p){
		int pp = popSubsetNums[p];//population num in parentData
		rawCodomGtypes_.Append(parentData.rawCodomGtypes_[pp].Duplicate());
		codomIndivs_.Append(parentData.codomIndivs_[pp].Duplicate());
		}//end for p
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();
	}//end ctor CodominantMarkerData(CodominantMarkerData&, aList<int>&)


void  CodominantMarkerData::Initialize(void){
	CodominantMarkerData::Clear();
	}//Initialize()
	
	
void  CodominantMarkerData::Initialize(aString& s){
	CodominantMarkerData::Clear();
	EvolutionaryGeneticsData::Initialize(s); 
	}//Initialize(aString&)
	
	
void  CodominantMarkerData::Initialize(CodominantMarkerData& cmd){
	EvolutionaryGeneticsData::Initialize(cmd); 
	nlociCodom_=cmd.nlociCodom_; 
	ploidy_=cmd.ploidy_; 
	alleleLen_=cmd.alleleLen_;
	allCodomLoci_=cmd.allCodomLoci_; 
	locusNumCodom_=cmd.locusNumCodom_;
	skippedLocusNumCodom_=cmd.skippedLocusNumCodom_;
	variableLocusNumCodom_=cmd.variableLocusNumCodom_;
	rawCodomGtypes_=cmd.rawCodomGtypes_;
	codomIndivs_=cmd.codomIndivs_;
	allelesPresent_=cmd.allelesPresent_;
	alleleFreq_=cmd.alleleFreq_;
	alleleFreqGlobal_=cmd.alleleFreqGlobal_;
	twoN_=cmd.twoN_;
	genotypeCount_=cmd.genotypeCount_;
	genotypeCountGlobal_=cmd.genotypeCountGlobal_;
	genotypeFreq_=cmd.genotypeFreq_;
	genotypeFreqGlobal_=cmd.genotypeFreqGlobal_;
	HetObsByAllele_=cmd.HetObsByAllele_;
	HetObsByLocus_=cmd.HetObsByLocus_;
	diseqD_=cmd.diseqD_;
	locusD_=cmd.locusD_;
	allelesCurrent_=cmd.allelesCurrent_;
	alleleFreqCurrent_=cmd.alleleFreqCurrent_;
	genotypeFreqCurrent_=cmd.genotypeFreqCurrent_;
	HetObsCurrent_=cmd.HetObsCurrent_; 
	varLociCurrent_=cmd.varLociCurrent_; 
	DCurrent_=cmd.DCurrent_; 
	}//Initialize(CodominantMarkerData& )
	
	
	
void  CodominantMarkerData::Initialize(CodominantMarkerData& parentData, aList<int>& popSubsetNums){
	EvolutionaryGeneticsData::Initialize(parentData,popSubsetNums);
	nlociCodom_=parentData.nlociCodom_; 
	ploidy_=parentData.ploidy_; 
	alleleLen_=parentData.alleleLen_;
	allCodomLoci_=parentData.allCodomLoci_; 
	locusNumCodom_=parentData.locusNumCodom_;
	skippedLocusNumCodom_=parentData.skippedLocusNumCodom_;
	ResetFlags();
//	ParseGenotypicData(allCodomLoci_);
	ParseGenotypicData();
	CheckForNewAlleles();
	UpdateVariableLoci();
	}//end Initialize(DominantMarkerData2&, aList<int>&)


	
	
	
CodominantMarkerData&  CodominantMarkerData::operator=(CodominantMarkerData& cmd){
	EvolutionaryGeneticsData::operator=(cmd); 
	npop_=cmd.npop_; 
	nlociCodom_=cmd.nlociCodom_; 
	ploidy_=cmd.ploidy_; 
	alleleLen_=cmd.alleleLen_;
	allCodomLoci_=cmd.allCodomLoci_; 
	locusNumCodom_=cmd.locusNumCodom_;
	skippedLocusNumCodom_=cmd.skippedLocusNumCodom_;
	variableLocusNumCodom_=cmd.variableLocusNumCodom_;
	rawCodomGtypes_=cmd.rawCodomGtypes_;
	codomIndivs_=cmd.codomIndivs_;
	allelesPresent_=cmd.allelesPresent_;
	alleleFreq_=cmd.alleleFreq_;
	alleleFreqGlobal_=cmd.alleleFreqGlobal_;
	twoN_=cmd.twoN_;
	genotypeCount_=cmd.genotypeCount_;
	genotypeCountGlobal_=cmd.genotypeCountGlobal_;
	genotypeFreq_=cmd.genotypeFreq_;
	genotypeFreqGlobal_=cmd.genotypeFreqGlobal_;
	HetObsByAllele_=cmd.HetObsByAllele_;
	HetObsByLocus_=cmd.HetObsByLocus_;
	diseqD_=cmd.diseqD_;
	locusD_=cmd.locusD_;
	allelesCurrent_=cmd.allelesCurrent_;
	alleleFreqCurrent_=cmd.alleleFreqCurrent_;
	genotypeFreqCurrent_=cmd.genotypeFreqCurrent_;
	HetObsCurrent_=cmd.HetObsCurrent_; 
	varLociCurrent_=cmd.varLociCurrent_; 
	DCurrent_=cmd.DCurrent_; 
	return *this;
	}//operator=(CodominantMarkerData&)



void CodominantMarkerData::Calculate2LocusDisequilibrium (void){
	if(DCurrent_) return;
	if(ploidy_ != 2){
		aString err("ERROR: CodominantMarkerData::Calculate2LocusDisequilibrium(): can\'t calculate D when ploidy=");
		err+ploidy_;
		aError(err);
		return;}
	int doingHillsMethod = 0;//false
	//first make up an empty data array for locusD_ and D_
	long double initValue = zero, gameteValue,freq1,freq2,diseq,HillsDiseq,initialF11hat,f11hat,diff,count1,count2;
//	aString blank; blank.AppendCharToLen(' ',alleleLen_);
	int pop,i,loc1,loc2,tr1,tr2,allele1,allele2,nSample,kComb,index1,index2,blankAt1,blankAt2,chrom,iteration;
	diseqD_.Clear();
	locusD_.Clear();//locusD_ contains locusByLocusDiseqInPop as subLists
	for (pop=0;pop<npop_;++pop){
		//locusByLocusDiseqInPop contains locusDiseqLists as sublists
		ObjList<aList<long double> >* locusByLocusDiseqInPop = NEW ObjList<aList<long double> >();
		locusD_.Append(locusByLocusDiseqInPop);
		for (loc1=0;loc1<nlociCodom_ - 1;++loc1){
			//locusDiseqList contains D values
			aList<long double>* locusDiseqList = NEW aList<long double>();
			locusByLocusDiseqInPop->Append(locusDiseqList);
			for (loc2=loc1 + 1;loc2<nlociCodom_;++loc2){
				locusDiseqList->Append(initValue);}
			}//end for loc1
		diseqD_.Append(initValue);
		}//end for pop
	StatList<long double> Dlist;
	for (pop=0;pop<npop_;++pop){
		for (loc1=0;loc1<nlociCodom_ - 1;++loc1){
			if (IsPolymorphic(pop,loc1)){ 
				tr1 = locusNumCodom_[loc1];
				StringList& alleleNamesList1=allelesPresent_[loc1];
				for (loc2 = (loc1 + 1); loc2<nlociCodom_;++loc2){
					if (IsPolymorphic(pop,loc2)){ 
						tr2=locusNumCodom_[loc2];
						StringList& alleleNamesList2=allelesPresent_[loc2];
						kComb = 0;  diseq=zero;
						for (allele1=0;allele1<alleleNamesList1.len();++allele1){
							//get freq of allele1 (at loc1) in pop
							freq1 = CodominantMarkerData::GetFreq(pop, loc1, allele1);
							if ((freq1>zero)&&(freq1<one)){
								aString& alleleName1 = alleleNamesList1[allele1];
								for (allele2=0;allele2<alleleNamesList2.len();++allele2){
									nSample = 0;/*set sample size to zero, to count for blanks*/
									/*get freq of allele2 (at loc2) in pop*/
									freq2 = CodominantMarkerData::GetFreq(pop, loc2, allele2);
									if ((freq2>zero)&&(freq2<one)){
									aString& alleleName2 = alleleNamesList2[allele2];
									/*get genotype strings for each individual and  read joint frequencies*/
									gameteValue = zero;
									int bigN[3][3], bigX[2][2];
									for (index1=0;index1<3;++index1){
										for (index2=0;index2<3;++index2){
											bigN[index1][index2] = 0;}}
									for (index1=0;index1<2;++index1){
										for (index2=0;index2<2;++index2){
											bigX[index1][index2] = 0;}}
									for (i=0;i<individuals(pop);++i){//each individual
								//		TextDataList& spot1 = CodominantMarkerData::GenotypicVal(pop,i,loc1);
										TextDataList& spot1 = CodominantMarkerData::GenotypicVal(pop,i,tr1);
										/*' make sure it doesn't have any blanks*/
										blankAt1 = 0;//false
										if (spot1[0].missing() || spot1[1].missing()) {blankAt1 = 1;}//true
										if (!(blankAt1)){
//											TextDataList& spot2 = CodominantMarkerData::GenotypicVal(pop,i,loc2);
											TextDataList& spot2 = CodominantMarkerData::GenotypicVal(pop,i,tr2);
											//make sure it doesn' t have any blanks
											blankAt2 = 0;//false
											if (spot2[0].missing() || spot2[1].missing()) {blankAt2 = 1;}//true
											if (!(blankAt2)){
												nSample++;
												//use this indiv in calculation
												//count instances of target alleles at each locus in this indiv
												count1 = 0;
												index1 = 3;
												for(chrom=0;chrom<2;++chrom){
													if (spot1[chrom] == alleleName1){
														count1 += half;
														index1--;
													}}
												count2 = 0;
												index2 = 3;
												for(chrom=0;chrom<2;++chrom){
													if (spot2[chrom] == alleleName2){
														count2 += half;
														index2--;
													}}
												//get a gametic value, which is the proportion
												//of gametes this individual will produce that
												//share target alleles at both loci.
												//Keep a sum of gametic values for the population
												gameteValue += count1 * count2;
												//this counts genotype freqs
												bigN[index1][index2]++;
												}//end if ! blankAt2
											}//end if ! blankAt1
										}//end for i
									//calculate frequency of joint genotype=gametic value/nsample}
									if (nSample == 0){
											gameteValue = (long double)-100;}
										else{
											gameteValue /= (long double)nSample;}
									//calculate Hill's (1974) intermediate variables & iterative method}
									HillsDiseq=zero;
									if (doingHillsMethod){
										bigX[1][1] = 2 * bigN[1][1] + bigN[1][2] + bigN[2][1];
										bigX[2][1] = 2 * bigN[3][1] + bigN[2][1] + bigN[3][2];
										bigX[1][2] = 2 * bigN[1][3] + bigN[1][2] + bigN[2][3];
										bigX[2][2] = 2 * bigN[3][3] + bigN[2][3] + bigN[3][2];
										initialF11hat = (long double)((2 * bigX[1][1] + bigN[2][2]) / (2 * nSample)) - freq1 * freq2;
										diff = ten; f11hat=zero;
										iteration = 0;
										while (diff > (long double) 0.00001){
											//calculate in steps since it's long
											iteration++;
											f11hat = (long double)bigN[2][2] * initialF11hat * ((long double)1.0 - freq1 - freq2 + initialF11hat);
											f11hat /= (initialF11hat * ((long double)1.0 - freq1 - freq2 + initialF11hat) + (freq1 - initialF11hat) * (freq2 - initialF11hat));
											f11hat = ((long double)bigX[1][1] + f11hat) / (long double)(2 * nSample);
											diff = ABS(f11hat - initialF11hat);
											initialF11hat = f11hat;
											if (iteration > 40){
												initialF11hat *= (long double)1.01;
												iteration = 0;}
											}//end while
										HillsDiseq = (long double)nSample * (f11hat - freq1 * freq2) / (long double)(nSample - 1);
										}//end if doingHillsMethod
									//calculate D
									diseq = gameteValue - freq1 * freq2;
									//was gameteValue -2* freq1 * freq2
									kComb++;
									ObjList<aList<long double> >& locusByLocusDiseqInPop = locusD_[pop];
									aList<long double>& locusDiseqList = locusByLocusDiseqInPop[loc1];
									if (doingHillsMethod){
											diseq = HillsDiseq + locusDiseqList[loc2 - loc1];}
										else{
											diseq += locusDiseqList[loc2 - loc1];}
									locusDiseqList.Replace(diseq, loc2 - loc1);
									}//end if freq2>0 &&freq2<1
								}//end for allele2
							}//end if freq1>0 && freq1<1
						}//end for allele1
					ObjList<aList<long double> >& locusByLocusDiseqInPop = locusD_[pop];
					aList<long double>& locusDiseqList = locusByLocusDiseqInPop[loc1];
					diseq += locusDiseqList.Nth(loc2 - loc1) / (long double)kComb;
					locusDiseqList.Replace(diseq, loc2 - loc1);
					Dlist.Append(diseq);
					}//end if IsPolymorphic(poploc2)
				}//end for loc2
			}//end if IsPolymorphic(pop,loc1)
		}//end for loc1
	if (diseqD_.len() > 0){diseqD_.Append(Dlist.mean());} else{diseqD_.Append(zero);}
	Dlist.Clear();
	}//end for pop
	DCurrent_=true;
	}//end CodominantMarkerData::Calculate2LocusDisequilibrium






void CodominantMarkerData::CalculateAlleleFreqs (void){
	//This routine counts allelic frequencies, taking into account
	//alleles left blank in the data set.
	if (alleleFreqCurrent_){return;}
	CheckForNewAlleles();
	int pop, i, chrom, loc, allele, tr, allelesSampled;
	alleleFreq_.Clear();//clear allele frequencies to start over
	twoN_.Clear();//clear locus sample sizes to start over
	codomIndivs_.Clear();
	long double freq;
	for (pop=0;pop< npop_;++pop){ 
		//set up a frequency list for this population
		ObjList<aList<long double> > *popAlleleFreqs = NEW ObjList<aList<long double> >();
		alleleFreq_.Append(popAlleleFreqs);
		aList<int> *locList = NEW aList<int>();
		twoN_.Append(locList);
		codomIndivs_.Append(NEW aList<int>());
		for (loc=0;loc<nlociCodom_;++loc){
			tr =locusNumCodom_[loc];
			//first set up a blank array of frequencies for each possible allele
			aList<long double> *locAlleleFreqs = NEW aList<long double>();
			popAlleleFreqs->Append(locAlleleFreqs);
			StringList& alleleNamesList = allelesPresent_[loc];
			freq = zero;
			for (allele=0;allele<alleleNamesList.len();++allele){
				locAlleleFreqs->Append(freq);}

			//next look at the genotype of each individual and tally the frequency
			allelesSampled = 0;	//the number of alleles, not individuals
			for (i=0;i< individuals(pop);++i){
				bool goodIndiv=true;
				for (chrom=0; chrom<ploidy_;++chrom){
//					TextDatum& gene = Genotype(pop,i,locusNumCodom_[loc],chrom);
					TextDatum& gene = CodominantMarkerData::GenotypicVal(pop,i,tr,chrom);
					if (gene.OK()) {//increment the frequency & sample size -- divide later}
							allele = alleleNamesList.Loc(&gene);
							(*locAlleleFreqs)[allele]++;
							++allelesSampled;
							}//end if not missing data
						else{
							goodIndiv=false;}//at least one allele not scored
					}//end for chrom
				if(! codomIndivs_[pop].Includes(i)){
					codomIndivs_[pop].Append(i);}
				}//end for i

			//now go through and divide by the sample size to restore the frequencies
			if(allelesSampled>0){
				for (allele=0;allele<alleleNamesList.len();++allele){
					(*locAlleleFreqs)[allele] /= (long double)allelesSampled;
					}
				}
			//store the sample size for this locus
			locList->Append(allelesSampled);
			}//end for loc
		}//end for pop
	alleleFreqGlobal_.Clear();//clear global allele frequencies to start over
	for(loc=0;loc<nlociCodom_;++loc){//unweighted aveages
		alleleFreqGlobal_.Append(NEW aList<long double>());
		for (allele=0;allele<allelesPresent_[loc].len();++allele){
			freq = zero;
			for(pop=0;pop<npop_;++pop){
				freq += alleleFreq_[pop][loc][allele];}
			alleleFreqGlobal_[loc].Append(freq/(long double)npop_);
			}}
	alleleFreqCurrent_ = indivsCodomCurrent_ = 1;
	}//end CodominantMarkerData::CalculateAlleleFreqs



ObjList<aList<long double> >* CodominantMarkerData::CalculateAlleleFreqsByPopSet(aList<int>& popSet, aList<long double>& twoNList){
 	//returns lost of loci, each with list of allele freqs, using total allele set
 	CalculateAlleleFreqs();
 	twoNList.Clear();
 	ObjList<aList<long double> >* popSetFreqsPtr = NEW ObjList<aList<long double> >(alleleFreq_[0]);//copies structure
 	ObjList<aList<long double> >& popSetFreqs = *popSetFreqsPtr;//notational convenience
 	for(int loc=0;loc<nlociCodom_;++loc){
 		int alleles = GetNumAlleles(loc);
 		long double samSizeSet = 0;
 		for(int a=0;a<alleles;++a){
 			long double nCopies = 0.0;
 			for(int popSetPop=0;popSetPop<popSet.len();++popSetPop){
 				int pop = popSet[popSetPop];
 				long double nSample = (long double)twoN_[pop][loc];
 				if(a==0){samSizeSet += nSample;}
 				nCopies += nSample * GetFreq(pop,loc,a);
 				}
 			if(a==0){twoNList.Append(samSizeSet);}
 			if(nCopies==0.0){
 					popSetFreqs[loc][a]=0.0;}
 				else if(nCopies==samSizeSet){
 					popSetFreqs[loc][a]=1.0;}
 				else{
 					popSetFreqs[loc][a] = nCopies / samSizeSet;
 					}
 			}//end for a
 		}//end for loc
 	return popSetFreqsPtr;
	}//end CodominantMarkerData::CalculateAlleleFreqsByPopSet



void CodominantMarkerData::CalculateGenotypeFreqs (void){
	//This routine counts genotypic frequencies, taking into account
	//alleles left blank in the data set.
	if (genotypeFreqCurrent_){return;}
	CalculateAlleleFreqs();//use to set sample sizes
	CheckForNewAlleles();
	long pop, i, loc, allele1, allele2, summ, tr;
	genotypeCount_.Clear();//clear genotype counts to start over
	genotypeFreq_.Clear();//clear genotype frequencies to start over
	long double freq, gtypesSampled;
	for (pop=0;pop< npop_;++pop){ 
		//set up a frequency list for this population
		genotypeCount_.Append(NEW ObjList<ObjList<aList<int> > >());
		genotypeFreq_.Append(NEW ObjList<ObjList<aList<long double> > >());
		for (loc=0;loc<nlociCodom_;++loc){
			tr=locusNumCodom_[loc];
			//first set up a blank array of frequencies for each possible allele combination
			genotypeCount_[pop].Append(NEW ObjList<aList<int> >());
			genotypeFreq_[pop].Append(NEW ObjList<aList<long double> >());
			StringList& alleleNamesList = allelesPresent_[loc];
			freq = zero;
			for (allele1=0;allele1<alleleNamesList.len();++allele1){
				genotypeCount_[pop][loc].Append(NEW aList<int> ());
				genotypeFreq_[pop][loc].Append(NEW aList<long double> ());
				for (allele2=0;allele2<alleleNamesList.len();++allele2){
					genotypeCount_[pop][loc][allele1].Append(0);
					genotypeFreq_[pop][loc][allele1].Append(freq);
					}}
				

			//next look at the genotype of each individual and tally the frequency
			gtypesSampled = (long double)0.0;	//the number of alleles, not individuals
			for (i=0;i< individuals(pop);++i){
//				TextDataList& gtpe = CodominantMarkerData::GenotypicVal(pop,i,loc);
				TextDataList& gtpe = CodominantMarkerData::GenotypicVal(pop,i,tr);
				if(gtpe.len()==2){//ignore unscored or partly scored individuals
					TextDatum& gene1 = gtpe[0];
					TextDatum& gene2 = gtpe[1];
					if (gene1.OK() && gene2.OK()) {//ignore missing data
						++gtypesSampled;//increment the frequency & sample size
						allele1 = alleleNamesList.Loc(&gene1);
						allele2 = alleleNamesList.Loc(&gene2);
						if(allele2<allele1){
							SWAP(allele1,allele2);}
						genotypeCount_[pop][loc][allele1][allele2]++;
						genotypeFreq_[pop][loc][allele1][allele2]++;
						}//end if not missing data
					}//if OK
				}//end for i

			//now go through and divide by the sample size to restore the frequencies
			if(gtypesSampled>zero){
				for (allele1=0;allele1<alleleNamesList.len();++allele1){
					for (allele2=0;allele2<alleleNamesList.len();++allele2){
						genotypeFreq_[pop][loc][allele1][allele2] /= gtypesSampled;
						}}
				}
			}//end for loc
		}//end for pop
	genotypeCountGlobal_.Clear();//clear global frequencies to start over
	genotypeFreqGlobal_.Clear();//clear global frequencies to start over
	for(loc=0;loc<nlociCodom_;++loc){//unweighted aveages
		genotypeCountGlobal_.Append(NEW ObjList<aList<int> >());
		genotypeFreqGlobal_.Append(NEW ObjList<aList<long double> >());
		for (allele1=0;allele1<allelesPresent_[loc].len();++allele1){
			genotypeCountGlobal_[loc].Append(NEW aList<int>());
			genotypeFreqGlobal_[loc].Append(NEW aList<long double>());
			for (allele2=0;allele2<allelesPresent_[loc].len();++allele2){
				freq = zero;
				summ=0;
				for(pop=0;pop<npop_;++pop){
					summ += genotypeCount_[pop][loc][allele1][allele2];
					freq += genotypeFreq_[pop][loc][allele1][allele2];
					}
				genotypeCountGlobal_[loc][allele1].Append(summ);
				genotypeFreqGlobal_[loc][allele1].Append(freq/(long double)npop_);
				}}}
	genotypeFreqCurrent_=1;
	}//end CodominantMarkerData::CalculateGenotypeFreqs


void CodominantMarkerData::CalculateObservedHeterozygosity (void){
	//Counts observed heterozygosity values for each population, locus & allele
	//directly from the genotype data strings for each individual.  
//	long i, codeLen, chrom, loc, allele, posGtypeWord, nSample;
	if(ploidy_ != 2){
		aString err("ERROR: CodominantMarkerData::CalculateObservedHeterozygosity(): can\'t calculate when ploidy=");
		err+ploidy_;
		aError(err);
		return;}
	long i, loc, allele,tr,nSample;
	long double hetsByAllele, hetsByLoc;
	//clear observed hets to start over
	HetObsByAllele_.Clear();
	HetObsByLocus_.Clear();
	for (int pop=0;pop<npop_;++pop){
		//set up a frequency list for this population
		ObjList<aList<long double> > *popHetObsAll = NEW ObjList<aList<long double> >();
		HetObsByAllele_.Append(popHetObsAll);
		aList<long double> *popHetObsLoc = NEW aList<long double>();
		HetObsByLocus_.Append(popHetObsLoc);
		for (loc=0;loc<nlociCodom_;++loc){
			tr=locusNumCodom_[loc];
			//first set up a blank array of frequencies for each possible allele
			aList<long double> *locHetObs = NEW aList<long double>();
			popHetObsAll->Append(locHetObs);
			StringList& alleleNamesList = allelesPresent_[loc];
			hetsByAllele = hetsByLoc = 0.0;
			popHetObsLoc->Append(hetsByLoc);
			for (allele=0;allele<alleleNamesList.len();++allele){
				locHetObs->Append(hetsByAllele);}
			//next look at the genotype of each individual and tally the frequency
			for (i=0;i<individuals(pop);++i){
//				TextDatum& gene1 = CodominantMarkerData::GenotypicVal(pop,i,loc,0);		
//				TextDatum& gene2 = CodominantMarkerData::GenotypicVal(pop,i,loc,1);		
				TextDatum& gene1 = CodominantMarkerData::GenotypicVal(pop,i,tr,0);		
				TextDatum& gene2 = CodominantMarkerData::GenotypicVal(pop,i,tr,1);		
				if ((gene1 != gene2) && gene1.OK() && gene2.OK()) {// it's a heterozygote
					//increment the het frequencies of both alleles -- divide later
					++hetsByLoc;
					allele = alleleNamesList.Loc(&gene1);
					hetsByAllele = locHetObs->Nth(allele) + 1.0;
					locHetObs->Replace(hetsByAllele, allele);
					allele = alleleNamesList.Loc(&gene2);
					hetsByAllele = locHetObs->Nth(allele) + 1.0;
					locHetObs->Replace(hetsByAllele, allele);
					}//end if match & no blanks
				}//end for i
			//now go through and divide by the sample size to restore the frequencies
			nSample = long(GetLocusSampleSize(pop, loc));
			if (nSample > 0){
				popHetObsLoc->Replace(hetsByLoc /((long double)nSample / two), loc);
				for (allele=0;allele<alleleNamesList.len();++allele){
					hetsByAllele = locHetObs->Nth(allele) / ((long double)nSample / (long double)2.0);
					locHetObs->Replace(hetsByAllele, allele);
					}//end for allele
				}//end if nSample
			}//end for loc
		}//end for pop
	HetObsCurrent_=1;
	}//end CodominantMarkerData::CalculateObservedHeterozygosity



void CodominantMarkerData::CalculateTwoN (void){
	//makes table of sample sizes for each pop, locus
	twoN_.Clear();
	for(int pop=0;pop<npop_;++pop){
		twoN_.Append(NEW aList<int>());
		for(int loc=0;loc<nlociCodom_;++loc){
			twoN_[pop].Append(0);
			for(int i=0;i<popSize_[pop];++i){
				twoN_[pop][loc] += GetLocusSampleSize(pop, i, loc);
				}//end for i
			}//end for loc
		}//end for pop
	}//end CodominantMarkerData::CalculateTwoN



void CodominantMarkerData::CheckForNewAlleles (void){
	if (allelesCurrent_){return;}
	allelesPresent_.Clear();
	int pop,loc,i,chrom,tr;
	for (loc=0;loc<nlociCodom_;++loc){
		allelesPresent_.Append(NEW StringList());}
	for (pop=0;pop<npop_;++pop){
		for(i=0;i<individuals(pop);++i){
			for (loc=0;loc<nlociCodom_;++loc){
				StringList& alleleList = allelesPresent_[loc];
				tr=locusNumCodom_[loc];
				for (chrom=0;chrom<ploidy_;++chrom){
//					TextDatum& gene = CodominantMarkerData::GenotypicVal(pop,i,loc,chrom);
					TextDatum& gene = CodominantMarkerData::GenotypicVal(pop,i,tr,chrom);
					if (gene.OK() &&!(alleleList.Includes(&gene))) {
						aString *newGene = NEW aString(gene);
						alleleList.Append(newGene);}
					}//end for chrom
				}//end for loc
			}//end for i
		}//end for pop
	for (int loc=0;loc<nlociCodom_;++loc){
		allelesPresent_[loc].Sort();}
	allelesCurrent_=1;
	CodominantMarkerData::UpdateVariableLoci();
	}//end CodominantMarkerData::CheckForNewAlleles



void CodominantMarkerData::Clear(void){
	EvolutionaryGeneticsData::Clear();
	allCodomLoci_.Clear();
	locusNumCodom_.Clear();
	skippedLocusNumCodom_.Clear();
	variableLocusNumCodom_.Clear();
	rawCodomGtypes_.Clear();
	codomIndivs_.Clear();
	allelesPresent_.Clear();
	alleleFreq_.Clear();
	alleleFreqGlobal_.Clear();
	twoN_.Clear();
	genotypeCount_.Clear();
	genotypeCountGlobal_.Clear();
	genotypeFreq_.Clear();
	genotypeFreqGlobal_.Clear();
	HetObsByAllele_.Clear();
	HetObsByLocus_.Clear();
	diseqD_.Clear();
	locusD_.Clear();
	ResetFlags();
	nlociCodom_= 0;
	alleleLen_=1;
	ploidy_=2;
	}//end CodominantMarkerData::Clear()



int CodominantMarkerData::codomIndividuals(int pop){
	if(! indivsCodomCurrent_){
		CountScoredIndividuals();}
	return codomIndivs_[pop].len();
	}//end  CodominantMarkerData::individuals(int,int)
	

int CodominantMarkerData::codomIndividuals(int pop, int loc){
	if(! indivsCodomCurrent_){
		CountScoredIndividuals();}
	int i,ncodom=0;
	for(i=0;i<codomIndivs_[pop].len();++i){
		if(! individualIsUnscored(pop,codomIndivs_[pop][i],loc)){
			++ncodom;}
		}//i
	return ncodom;
	}//end  CodominantMarkerData::individuals(int,int)
	

void CodominantMarkerData::CountScoredIndividuals(void){
	if(! indivsCodomCurrent_){
		int pop,i,loc;
		codomIndivs_.Clear();
		for(pop=0;pop<npop_;++pop){
			codomIndivs_.Append(NEW aList<int>());
			for(i=0;i<individuals(pop);++i){
				for(loc=0;loc<nlociCodom_;++loc){
					if(GetLocusSampleSize ( pop, i, loc)>0){
						codomIndivs_[pop].Append(i);
						continue;}
					}//loc
				}//i
			}//pop
		indivsCodomCurrent_=true;
		}//! current
	}//CodominantMarkerData::CountScoredIndividuals(void)
	


void  CodominantMarkerData::CreateBootstrapData(CodominantMarkerData& origData, Ran3& ran){
	//individuals resampled
//	EvolutionaryGeneticsData::CreateBootstrapData(origData,ran);//faster to do it all here
	Clear();
	title_ = origData.title_;	//title_ of the data set from the first data line
	title_.cat(" bootstrap replicate");
	ngroups_ = origData.ngroups_;
	npop_ = origData.npop_;
	ntraits_ = origData.ntraits_;
	popName_ = origData.popName_;
	groupName_ = origData.groupName_;
	localityName_ = origData.localityName_;
	traitName_ = origData.traitName_;
	traitNum_ = origData.traitNum_;
	skippedTraitNum_ = origData.skippedTraitNum_;
	groupHierarchy_ = origData.groupHierarchy_;
	popsByLocality_ = origData.popsByLocality_;
	localityCoordinates_ = origData.localityCoordinates_;
	missing_ = origData.missing_;

	nlociCodom_ = origData.nlociCodom_;
	ploidy_ = origData.ploidy_;
	alleleLen_ = origData.alleleLen_;
	allCodomLoci_ = origData.allCodomLoci_;
	locusNumCodom_ = origData.locusNumCodom_;
	skippedLocusNumCodom_ = origData.skippedLocusNumCodom_;
	variableLocusNumCodom_ = origData.variableLocusNumCodom_;	
	
		
	for(int pop=0;pop<npop_;++pop){//randomly resamples within populations
		rawTraits_.Append(NEW ObjList<TextDataList>());
		rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
		int newpop = ran.longBetween(0,npop_-1);
		int newpopsize = origData.rawTraits_[newpop].len();
		popSize_.Append(newpopsize);
		for (int i=0;i<newpopsize;++i){
			int ranIndivID = ran.longBetween(0,newpopsize-1);
//			TextDataList* ranIndiv = NEW TextDataList(origData.rawTraits_[newpop][ranIndivID]);
			rawTraits_[pop].Append((TextDataList*)origData.rawTraits_[newpop][ranIndivID].Duplicate());
//			ranIndiv = NULL;//forget
			rawCodomGtypes_[pop].Append(origData.rawCodomGtypes_[newpop][ranIndivID].Duplicate());
		}}

	CheckForNewAlleles();	
	}//end  CodominantMarkerData::CreateBootstrapData(CodominantMarkerData&)




void CodominantMarkerData::CreateRandomData(
							const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Fst=";
	title_.cat(Fst); title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numPopsSampled;
	nlociCodom_ = numLociSampled;
	ngroups_ = 1;
	traitName_.Append(NEW aString("ID"));
	int locuscount=0;
	traitNum_.Append(locuscount++);
	for(int loc = 0;loc<nlociCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociCodom_);
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(locuscount++);
		allCodomLoci_.Append(loc+1);
		locusNumCodom_.Append(loc+1);
		}
	ntraits_ =traitNum_.len();

	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
	GeneFrequencyDistribution parametricFreqDistrib(Fst, pBar, ploidy_);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	aString genotype;
	ObjList<StatList<long double> > locusFrequenciesByPop;
	int loc,pop,i;
	for(pop=0;pop<npop_;++pop){
		StatList<long double>* locusFrequencies = NEW StatList<long double>();
		locusFrequenciesByPop.Append(locusFrequencies);
		for(loc=0;loc<nlociCodom_;++loc){
			long double pf = parametricFreqDistrib.RandomFrequency(ran);
			locusFrequencies->Append(pf);
			}//end for loc
		}//end for pop

	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("randomly generated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
		rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());	
		for (i=0;i<numIndivsPerPop;++i){
			rawTraits_[pop].Append(NEW TextDataList());
			aString indiv(i+1);
			indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));
			rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
			for(int loc=0;loc<nlociCodom_;++loc){
				rawCodomGtypes_[pop][i].Append(NEW TextDataList());}					
			}//end for i
		popSize_.Append(numIndivsPerPop);

		//create and append genotypes
		for(loc=0;loc<nlociCodom_;++loc){
			StatList<long double>& locusFrequencies = locusFrequenciesByPop[pop];
			long double p = locusFrequencies[loc];
			GenotypeFreqDistribution popGenotypes(p, Fis);
			for(i=0;i<numIndivsPerPop;++i){
				int gtype = popGenotypes.RandomGenotype(ran);
				if (gtype==0){genotype = "AA";}
					else if(gtype==1){genotype = "AB";}
					else if(gtype==2){genotype = "BB";}	
				rawTraits_[pop][i].Append(NEW TextDatum(genotype));
				rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[0]));		
				rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[1]));		
				}//end for i
			}//end for loc
		}//end for pop	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end CodominantMarkerData::CreateRandomData



void CodominantMarkerData::CreateRandomData(
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Fst=";
	title_.cat(Fst); title_.cat(", Fis="); title_.cat(Fis); 
	npop_ = numIndivsPerPopLoc.len();
	nlociCodom_ = pBar.len();
	ngroups_ = 1;
	traitName_.Append(NEW aString("ID"));
	int locuscount=0;
	traitNum_.Append(locuscount++);
	for(int loc = 0;loc<nlociCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociCodom_);
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(locuscount++);
		allCodomLoci_.Append(loc+1);
		locusNumCodom_.Append(loc+1);
		}
	ntraits_ =traitNum_.len();


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();

	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	ObjList<ObjList<StatList<long double> > > frequenciesByPopLocusAllele;
	for(int pop=0;pop<npop_;++pop){
		frequenciesByPopLocusAllele.Append(NEW ObjList<StatList<long double> >());}
	for(int loc=0;loc<nlociCodom_;++loc){
		if(pBar[loc].len()>1){//locus is variable
//				GeneFrequencyDistMultiAllele parametricFreqDistrib(Fst, pBar[loc], 1000);//old version sends dp
				GeneFrequencyDistMultiAllele parametricFreqDistrib(Fst, pBar[loc], 2);//new version sends ploidy
				for(int pop=0;pop<npop_;++pop){
					aList<long double>* p = parametricFreqDistrib.RandomFrequencies(ran);
					frequenciesByPopLocusAllele[pop].Append(NEW StatList<long double>(*p));//copy as StatList
					delete p;p=NULL;
					}//end for pop
				}
			else{//locus is monomorphic
				for(int pop=0;pop<npop_;++pop){
					frequenciesByPopLocusAllele[pop].Append(NEW StatList<long double>(pBar[loc]));//copy as StatList
					}//end for pop
				}				
		}//end for loc

	for (int pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("randomly generated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		int maxIndivPop = 0;
		for (int loc=0;loc<numIndivsPerPopLoc[pop].len();++loc){
			maxIndivPop = MAX(maxIndivPop,numIndivsPerPopLoc[pop][loc]);}

		//create id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
		rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());							
		for (int i=0;i<maxIndivPop;++i){
			rawTraits_[pop].Append(NEW TextDataList());
			aString indiv(i+1);//first create id numbers for each individual
			indiv.PrependCharToLen(' ', MAX(4,Digits(maxIndivPop)));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));
			rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
			for(int loc=0;loc<nlociCodom_;++loc){
				rawCodomGtypes_[pop][i].Append(NEW TextDataList());}					
			}//end for i
		popSize_.Append(maxIndivPop);

		for(int loc=0;loc<nlociCodom_;++loc){
			StatList<long double>& p = frequenciesByPopLocusAllele[pop][loc];
//			for(int a=0;a<p.len();++a){
//				cout<<"pop:  "<<pop+1<<"   "<<"loc:  "<<loc+1<<"   "<<"allele:  "<<a+1<<"   "<<p[a]<<endl;}
			if(p.len()>1){//sampling required
					GenotypeFreqMultiAlleleDist popGenotypes(p, Fis);
//					popGenotypes.PrintToScreen();
					for(int i=0;i<numIndivsPerPopLoc[pop][loc];++i){
						aString* genotype = popGenotypes.RandomGenotypeString(ran);
						rawTraits_[pop][i].Append(NEW TextDatum(*genotype));
						rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum((*genotype)[0]));		
						rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum((*genotype)[1]));		
						delete genotype; genotype=NULL;
						}//end for i
					for(int i=numIndivsPerPopLoc[pop][loc];i<maxIndivPop;++i){//unsampled individuals blank
						rawTraits_[pop][i].Append(NEW TextDatum());//add missing data
						rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum());//add missing data	
						rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum());//add missing data	
						}//end for i
					}
				else{//monomorphic locus
					for(int i=0;i<numIndivsPerPopLoc[pop][loc];++i){
						rawTraits_[pop][i].Append(NEW TextDatum("AA"));
						rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum("A"));		
						rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum("A"));		
						}//end for i
					for(int i=numIndivsPerPopLoc[pop][loc];i<maxIndivPop;++i){//unsampled individuals blank
						rawTraits_[pop][i].Append(NEW TextDatum());//add missing data
						rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum());//add missing data	
						rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum());//add missing data	
						}//end for i
					}					
			}//end for loc
		}//end for pop	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end CodominantMarkerData::CreateRandomData



void CodominantMarkerData::CreateRandomData(
							const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Flt=";
	title_.cat(Flt); title_.cat(", Fspt="); title_.cat(Fspt);  title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numSpeciesSampled * numLocalitiesSampled;
	nlociCodom_ = numLociSampled;
	ngroups_ = numSpeciesSampled;
	traitName_.Append(NEW aString("ID"));
	int locuscount=0;
	traitNum_.Append(locuscount++);
	for(int loc = 0;loc<nlociCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociCodom_);
		locName.cat(locStr);
		traitNum_.Append(locuscount++);
		traitName_.Append(NEW aString(locName));//append a copy
		allCodomLoci_.Append(loc+1);
		locusNumCodom_.Append(loc+1);
		}
	ntraits_ =traitNum_.len();


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE SPECIES
//	GeneFrequencyDistribution parametricFreqDistribSP(Fspt, pBar, 1000);//old version sends dp
	GeneFrequencyDistribution parametricFreqDistribSP(Fspt, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<aList<long double> > pBarBySpeciesAndLocus;
	for(int sp0=0;sp0<numSpeciesSampled;++sp0){
		aList<long double>* locusPList = NEW aList<long double>();
		pBarBySpeciesAndLocus.Append(locusPList);
		for(int locus0=0;locus0<nlociCodom_;++locus0){
			long double p0 = parametricFreqDistribSP.RandomFrequency(ran);
			locusPList->Append(p0);
			}//end for locus0
		}//end for sp0
		
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	for(int sp1=0; sp1<numSpeciesSampled;++sp1){
		//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG populations within the species
		ObjList<StatList<long double> > locusFrequenciesByLocality;
		for(int locality1=0;locality1<numLocalitiesSampled; ++locality1){
			StatList<long double>* locusFrequencies = NEW StatList<long double>();
			locusFrequenciesByLocality.Append(locusFrequencies);
			for(int loc=0;loc<nlociCodom_;++loc){
				GeneFrequencyDistribution parametricFreqDistrib(Flt, pBarBySpeciesAndLocus[sp1][loc], 2);
				long double p = parametricFreqDistrib.RandomFrequency(ran);
				locusFrequencies->Append(p);
				}//end for loc
			}//end for locality1

		for (int locality1=0;locality1<numLocalitiesSampled; ++locality1){
			// NAME THE POPULATION
			aString pName("LOCALITY");pName.cat(locality1+1);pName.cat(" (");pName.cat(locality1+1);
			pName.cat(") randomly generated population of SPECIES"); pName.cat(sp1+1);
			popName_.Append(NEW aString(pName));//append a copy
			int pop = popName_.len()-1;
			//create id numbers for each individual
			rawTraits_.Append(NEW ObjList<TextDataList>());	
			rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());	
			for (int i=0;i<numIndivsPerPop;++i){
				rawTraits_[pop].Append(NEW TextDataList());
				aString indiv(i+1);//first create id numbers for each individual
				indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
				rawTraits_[pop][i].Append(NEW TextDatum(indiv));
				rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				for(int loc=0;loc<nlociCodom_;++loc){
					rawCodomGtypes_[pop][i].Append(NEW TextDataList());}					
				}//end for i
			popSize_.Append(numIndivsPerPop);

			for(int loc=0;loc<nlociCodom_;++loc){
				StatList<long double>& locusFrequencies = locusFrequenciesByLocality[locality1];
				double p = locusFrequencies[loc];
				GenotypeFreqDistribution popGenotypes(p, Fis);
				for(int i=0;i<numIndivsPerPop;++i){
					aString genotype = '\0';
					int gtype = popGenotypes.RandomGenotype(ran);
					if (gtype==0){genotype = "AA";}
						else if(gtype==1){genotype = "AB";}
						else if(gtype==2){genotype = "BB";}			
					rawTraits_[pop][i].Append(NEW TextDatum(genotype));
					rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[0]));		
					rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[1]));		
					}//end for i
				}//end for loc
			}//end for locality1
		}//end for sp1	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end CodominantMarkerData::CreateRandomData




void CodominantMarkerData::CreateRandomData(//not debugged
							const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Flt=";
	title_.cat(Flt); title_.cat(", Fspt="); title_.cat(Fspt);  
	title_.cat(", Flspt="); title_.cat(Flspt); title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numSpeciesSampled * numLocalitiesSampled;
	nlociCodom_ = numLociSampled;
	ngroups_ = numSpeciesSampled;
	traitName_.Append(NEW aString("ID"));
	int locuscount=0;
	traitNum_.Append(locuscount++);
	for(int loc = 0;loc<nlociCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociCodom_);
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(locuscount++);
		allCodomLoci_.Append(loc+1);
		locusNumCodom_.Append(loc+1);
		}
	ntraits_ =traitNum_.len();


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE SPECIES
	GeneFrequencyDistribution parametricFreqDistribSP(Fspt, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<aList<long double> > pBarBySpeciesAndLocus;
	for(int sp0=0;sp0<numSpeciesSampled;++sp0){
		aList<long double>* locusPList = NEW aList<long double>();
		pBarBySpeciesAndLocus.Append(locusPList);
		for(int locus0=0;locus0<nlociCodom_;++locus0){
			long double p0 = parametricFreqDistribSP.RandomFrequency(ran);
			locusPList->Append(p0);
			}//end for locus0
		}//end for sp0
		
	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE LOCALITIES
	GeneFrequencyDistribution parametricFreqDistribLocality(Flt, pBar, 2);
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<aList<long double> > pBarByLocalityAndLocus;
	for(int L0=0;L0<numLocalitiesSampled;++L0){
		aList<long double>* locusPList = NEW aList<long double>();
		pBarByLocalityAndLocus.Append(locusPList);
		for(int locus0=0;locus0<nlociCodom_;++locus0){
			long double p0 = parametricFreqDistribLocality.RandomFrequency(ran);
			locusPList->Append(p0);
			}//end for locus0
		}//end for L0
		
	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE SUBPOPULATIONS
//	GeneFrequencyDistribution parametricFreqDistribSubpop(Flspt, pBar, 1000);//old version sends dp
	GeneFrequencyDistribution parametricFreqDistribSubpop(Flspt, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<ObjList<aList<long double> > > pBarBySpeciesLocalityAndLocus;
	for(int sp1=0;sp1<numSpeciesSampled;++sp1){
		pBarBySpeciesLocalityAndLocus.Append(NEW ObjList<aList<long double> >());
		for(int L1=0;L1<numLocalitiesSampled;++L1){
			aList<long double>* locusPList = NEW aList<long double>();
			pBarBySpeciesLocalityAndLocus[sp1].Append(locusPList);
			for(int locus0=0;locus0<nlociCodom_;++locus0){
				long double p0 = parametricFreqDistribSubpop.RandomFrequency(ran);
				long double devLocality = (pBarByLocalityAndLocus[L1][locus0] - pBar);
				long double devSpecies = (pBarBySpeciesAndLocus[sp1][locus0] - pBar);
				long double p = p0 + devLocality + devSpecies;
				locusPList->Append(p);
				}//end for locus0
			}//end for L0
		}//end for sp0
		
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	for(int sp1=0; sp1<numSpeciesSampled;++sp1){
		//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG populations within the species
/*		ObjList<StatList<double> > locusFrequenciesByLocality;
		for(int locality1=0;locality1<numLocalitiesSampled; ++locality1){
			StatList<double>* locusFrequencies = new StatList<double>();
			locusFrequenciesByLocality.Append(locusFrequencies);
			for(int loc=0;loc<nloci;++loc){
				GeneFrequencyDistribution parametricFreqDistrib(Flspt, pBarBySpeciesLocalityAndLocus[sp1][locality1][loc], 1000);
				double p = parametricFreqDistrib.RandomFrequency(ran);
				locusFrequencies->Append(p);
				}//end for loc
			}//end for locality1
*/
		for (int locality1=0;locality1<numLocalitiesSampled; ++locality1){
			// NAME THE POPULATION
			aString pName("LOCALITY");pName.cat(locality1+1);pName.cat(" (");pName.cat(locality1+1);
			pName.cat(") randomly generated population of SPECIES"); pName.cat(sp1+1);
			popName_.Append(NEW aString(pName));//append a copy
			int pop = popName_.len()-1;
			//create id numbers for each individual
			rawTraits_.Append(NEW ObjList<TextDataList>());	
			rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());	
			for (int i=0;i<numIndivsPerPop;++i){
				rawTraits_[pop].Append(NEW TextDataList());
				aString indiv(i+1);//first create id numbers for each individual
				indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
				rawTraits_[pop][i].Append(NEW TextDatum(indiv));
				rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				for(int loc=0;loc<nlociCodom_;++loc){
					rawCodomGtypes_[pop][i].Append(NEW TextDataList());}					
				}//end for i
			popSize_.Append(numIndivsPerPop);

			for(int loc=0;loc<nlociCodom_;++loc){
				long double p = pBarBySpeciesLocalityAndLocus[sp1][locality1][loc];
				GenotypeFreqDistribution popGenotypes(p, Fis);
				for(int i=0;i<numIndivsPerPop;++i){
					aString genotype = '\0';
					int gtype = popGenotypes.RandomGenotype(ran);
					if (gtype==0){genotype = "AA";}
						else if(gtype==1){genotype = "AB";}
						else if(gtype==2){genotype = "BB";}			
					rawTraits_[pop][i].Append(NEW TextDatum(genotype));
					rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[0]));		
					rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[1]));		
					}//end for i
				}//end for loc
			}//end for locality1
		}//end for sp1	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end CodominantMarkerData::CreateRandomData


void CodominantMarkerData::CreateRandomDataLocFirst(
							const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Flt=";
	title_.cat(Flt); title_.cat(", Fspt="); title_.cat(Fspt);  title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numSpeciesSampled * numLocalitiesSampled;
	nlociCodom_ = numLociSampled;
	ngroups_ = numSpeciesSampled;
	traitName_.Append(NEW aString("ID"));
	int locuscount=0;
	traitNum_.Append(locuscount++);
	for(int loc = 0;loc<nlociCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociCodom_);
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(locuscount++);
		allCodomLoci_.Append(loc+1);
		locusNumCodom_.Append(loc+1);
		}
	ntraits_ =traitNum_.len();


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE LOCALITIES
//	GeneFrequencyDistribution parametricFreqDistribL(Flt, pBar, 1000);//old version sends dp
	GeneFrequencyDistribution parametricFreqDistribL(Flt, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	aList<long double> pBarByLocality;
	for(int L0=0;L0<numLocalitiesSampled;++L0){
		long double p0 = parametricFreqDistribL.RandomFrequency(ran);
		pBarByLocality.Append(p0);
		}//end for L0
		
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	for(int L1=0; L1<numLocalitiesSampled;++L1){
		//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG SPECIES WITHIN THE LOCALITIES
//		GeneFrequencyDistribution parametricFreqDistrib(Fspt, pBarByLocality[L1], 1000);//old version sends dp
		GeneFrequencyDistribution parametricFreqDistrib(Fspt, pBarByLocality[L1], 2);//new version sends ploidy
		ObjList<StatList<long double> > locusFrequenciesBySpecies;
		for(int species1=0;species1<numSpeciesSampled; ++species1){
			StatList<long double>* locusFrequencies = NEW StatList<long double>();
			locusFrequenciesBySpecies.Append(locusFrequencies);
			for(int loc=0;loc<nlociCodom_;++loc){
				long double p = parametricFreqDistrib.RandomFrequency(ran);
				locusFrequencies->Append(p);
				}//end for loc
			}//end for species1

		for (int species1=0;species1<numSpeciesSampled; ++species1){
			// NAME THE POPULATION
			aString pName("LOCALITY");pName.cat(L1+1);pName.cat(" (");pName.cat(L1+1);
			pName.cat(") randomly generated population of SPECIES"); pName.cat(species1+1);
			popName_.Append(NEW aString(pName));//append a copy
			int pop = popName_.len()-1;
			//create id numbers for each individual
			rawTraits_.Append(NEW ObjList<TextDataList>());	
			rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());	
			for (int i=0;i<numIndivsPerPop;++i){
				rawTraits_[pop].Append(NEW TextDataList());
				aString indiv(i+1);//first create id numbers for each individual
				indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
				rawTraits_[pop][i].Append(NEW TextDatum(indiv));
				rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				for(int loc=0;loc<nlociCodom_;++loc){
					rawCodomGtypes_[pop][i].Append(NEW TextDataList());}					
				}//end for i
			popSize_.Append(numIndivsPerPop);

			for(int loc=0;loc<nlociCodom_;++loc){
				StatList<long double>& locusFrequencies = locusFrequenciesBySpecies[species1];
				long double p = locusFrequencies[loc];
				GenotypeFreqDistribution popGenotypes(p, Fis);
				for(int i=0;i<numIndivsPerPop;++i){
					aString genotype = '\0';
					int gtype = popGenotypes.RandomGenotype(ran);
					if (gtype==0){genotype = "AA";}
						else if(gtype==1){genotype = "AB";}
						else if(gtype==2){genotype = "BB";}			
					rawTraits_[pop][i].Append(NEW TextDatum(genotype));
					rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[0]));		
					rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[1]));		
					}//end for i
				}//end for loc
			}//end for species1
		}//end for L1	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end CodominantMarkerData::CreateRandomDataLocFirst


void CodominantMarkerData::CullLoci (aList<int>& lociToCull){
	//This function shifts trait numbers from from traitNums_ to the skippedTraitNums_ list
	if(lociToCull.len()==0){return;}//none to cull
	for(int tr=0;tr<lociToCull.len();++tr){
		int cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(traitNum_.Includes(cLoc) && cLoc>=0 && cLoc<traitName_.len()){
			long item = traitNum_.Loc(cLoc);
			skippedTraitNum_.Append(cLoc);
			skippedLocusNumCodom_.Append(cLoc);
			traitNum_.Delete(item);
			item = locusNumCodom_.Loc(cLoc);
			locusNumCodom_.Delete(item);
			}//if item not skipped already
		}//tr
	nlociCodom_ = locusNumCodom_.len();
	skippedTraitNum_.Sort();
	skippedLocusNumCodom_.Sort();
	ResetFlags();
	CodominantMarkerData::CheckForNewAlleles();
//	CodominantMarkerData::ParseGenotypicData(locusNumCodom_,0);
	CodominantMarkerData::ParseGenotypicData(0);
	}//end procedure CodominantMarkerData::CullLoci(aList<int>&)


void CodominantMarkerData::CullLoci (StringList& lociToCull){
	//This function culls the designated loci from each genotype data string
	//and includes them among the skipped loci
	if(lociToCull.len()==0){return;}//none to cull
	//convert names to numbers and cull
	aList<int> cullListByNum;
	for(int tr=0;tr<lociToCull.len();++tr){
		aString cTraitname = lociToCull[tr];
		int item = traitName_.Loc(&cTraitname);
		if((item>=0)&&(!(skippedTraitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already skipped
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	CullLoci(cullListByNum);
	}//end procedure CodominantMarkerData::CullLoci(StringList&)



void CodominantMarkerData::CullLocus (char* locusToCull){
	aString locToCull(locusToCull);
	CullLocus(locToCull);
	}//end CodominantMarkerData::CullLocus (char* )


void CodominantMarkerData::CullLocus (int locusToCull){
	aList<int> lociToCull;
	lociToCull.Append(locusToCull);
	CullLoci(lociToCull);
	}//end CodominantMarkerData::CullLocus (int )


void CodominantMarkerData::CullLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	CullLoci(lociToCull);
	}//end CodominantMarkerData::CullLocus (aString& )


void CodominantMarkerData::CullMonomorphicLoci (void){	
	//This function culls the monomorphic loci
	//and includes them among the skipped loci
	UpdateVariableLoci();
	if(variableLocusNumCodom_.len()==locusNumCodom_.len()){return;}//none monomorphic
	aList<int> monomorphicLoci(locusNumCodom_);
	for(int vLoc=variableLocusNumCodom_.len()-1; vLoc>=0;--vLoc){
		int vLocusID = variableLocusNumCodom_[vLoc];
		int item = monomorphicLoci.Loc(vLocusID);
		monomorphicLoci.Delete(item);
		}//vLoc
	CullLoci(monomorphicLoci);
	}//end procedure CodominantMarkerData::CullMonomorphicLoci




void CodominantMarkerData::CullToOneGroup (EvolutionaryGeneticsData& parentData,int groupNum){
	//makes a subset data structure from the specified group
	//individuals resampled
	EvolutionaryGeneticsData::CullToOneGroup(parentData,groupNum);
	EvolutionaryGeneticsData* parentDataPtr =  &parentData;
	unsigned long parentDataUSL = (unsigned long) parentDataPtr;
	CodominantMarkerData* parentDataCodomPtr = (CodominantMarkerData*) parentDataUSL;
	CodominantMarkerData& parentDataC = *parentDataCodomPtr;

	nlociCodom_ = parentDataC.nlociCodom_;
	ploidy_ = parentDataC.ploidy_;
	alleleLen_ = parentDataC.alleleLen_;
	allCodomLoci_ = parentDataC.allCodomLoci_;
	locusNumCodom_ = parentDataC.locusNumCodom_;
	skippedLocusNumCodom_ = parentDataC.skippedLocusNumCodom_;
	
	for(int pop=0;pop<npop_;++pop){
		int theRightPop = parentDataC.groupHierarchy_[groupNum][pop];
		rawCodomGtypes_.Append(parentDataC.rawCodomGtypes_[theRightPop].Duplicate());
		}

	CheckForNewAlleles();
	}//end CodominantMarkerData::CullToOneGroup (int)



void CodominantMarkerData::ExpungeLoci (StringList& lociToCull){
	if(lociToCull.len()==0){return;}//none to cull
	aList<int> cullListByNum;
	int tr,item;
	aString cTraitname;
	for(tr=0;tr<lociToCull.len();++tr){
		cTraitname = lociToCull[tr];
		item = traitName_.Loc(&cTraitname);
		if(item>=0){//the trait exists in the rawlist
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	ExpungeLoci(cullListByNum);
	}//end procedure CodominantMarkerData::ExpungeLoci(StringList&)


void CodominantMarkerData::ExpungeLoci (aList<int>& lociToCull){
	//This function removes trait numbers from the haploid data set; called during reclassification
	if(lociToCull.len()==0){return;}//none to cull
	CodominantMarkerData::PrintGelJockFile(cout);
	int item,tr,cLoc;
	for(tr=0;tr<lociToCull.len();++tr){
		cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(allCodomLoci_.Includes(cLoc)){
			item = allCodomLoci_.Loc(cLoc);
			allCodomLoci_.Delete(item);}
		if(locusNumCodom_.Includes(cLoc)){
			item = locusNumCodom_.Loc(cLoc);
			locusNumCodom_.Delete(item);}
		if(skippedLocusNumCodom_.Includes(cLoc)){
			item = skippedLocusNumCodom_.Loc(cLoc);
			skippedLocusNumCodom_.Delete(item);}
		}//tr
	nlociCodom_ = locusNumCodom_.len();
	ResetFlags();
//	CodominantMarkerData::ParseGenotypicData(allCodomLoci_,0);

	CodominantMarkerData::PrintGelJockFile(cout);

	CodominantMarkerData::ParseGenotypicData(0);

	CodominantMarkerData::PrintGelJockFile(cout);

	CodominantMarkerData::CheckForNewAlleles();
	}//CodominantMarkerData::ExpungeLoci (aList<int>&)



void CodominantMarkerData::ExpungeLocus (char* locusToCull){
	aString locToCull(locusToCull);
	ExpungeLocus(locToCull);
	}//end CodominantMarkerData::ExpungeLocus (char* )


void CodominantMarkerData::ExpungeLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	ExpungeLoci(lociToCull);
	}//end CodominantMarkerData::ExpungeLocus (aString& )


TextDataList& CodominantMarkerData::GenotypicVal(int pop, int indiv, int trait){
	//does not notice skipped loci
//	return rawCodomGtypes_[pop][indiv][locusNumCodom_[loc]];
//	return rawCodomGtypes_[pop][indiv][loc];
//	int correctLoc = allCodomLoci_.Loc(locusNumCodom_[loc]);
	int correctLoc = allCodomLoci_.Loc(trait);
	return rawCodomGtypes_[pop][indiv][correctLoc];
	}//end  CodominantMarkerData::GenotypicVal(int,int,int)


TextDatum& CodominantMarkerData::GenotypicVal(int pop, int indiv, int trait, int allele){
	//does not notice skipped loci
//	return rawCodomGtypes_[pop][indiv][locusNumCodom_[loc]][allele];
//	return rawCodomGtypes_[pop][indiv][loc][allele];
//	int correctLoc = allCodomLoci_.Loc(locusNumCodom_[loc]);
	int correctLoc = allCodomLoci_.Loc(trait);
	return rawCodomGtypes_[pop][indiv][correctLoc][allele];
	}//end  CodominantMarkerData::GenotypicVal(int,int,int,int)




long double CodominantMarkerData::GetExpectedHeterozygosity (int pop, int loc, int allele){
	long double freq = GetFreq(pop, loc, allele);
	return (two * freq * (one - freq));
	}//end CodominantMarkerData::GetExpectedHeterozygosity(pop,loc,allele)



long double CodominantMarkerData::GetExpectedHeterozygosity (int pop, int loc){
	long double freq, expHomoFreq = zero;
	int allele;
	for (allele=0;allele<GetNumAlleles(loc);++allele){
		freq = GetFreq(pop, loc, allele);
		expHomoFreq += freq*freq;}
	return (one - expHomoFreq);
	}//end CodominantMarkerData::GetExpectedHeterozygosity(pop,loc)



long double CodominantMarkerData::GetExpectedHeterozygosity (int pop){
	//returns mean Hexp over loci
	StatList<long double> hetExp;
	for (int loc=0;loc<nlociCodom_;++loc){
		hetExp.Append(GetExpectedHeterozygosity(pop,loc));}
	return hetExp.mean();
	}//end CodominantMarkerData::GetExpectedHeterozygosity(pop)



long double CodominantMarkerData::GetFreq (int loc, int allele){
	long double n, numAlleleCopies = zero;
	long double numThisAllele = zero;
	int pop;
	for(pop=0;pop<npop_;++pop){
		n = (long double)GetLocusSampleSize(pop,loc);
		numAlleleCopies += n;
		numThisAllele += n * GetFreq(pop,loc,allele);
		}
	if(numAlleleCopies==zero){return zero;}
	return numThisAllele/numAlleleCopies;
	}//end CodominantMarkerData::GetFreq


long double CodominantMarkerData::GetFreq (int pop, int loc, int allele){
	if (!alleleFreqCurrent_){CalculateAlleleFreqs();}
	return alleleFreq_[pop][loc][allele];
	}//end CodominantMarkerData::GetFreq


long double CodominantMarkerData::GetFreqMostCommonAllele (int pop, int loc){
	long double freq, mostCommon = zero;
	int allele;
	for(allele=0;allele<GetNumAlleles(loc);++allele){
		freq = GetFreq(pop,loc,allele);
		if (freq>half) {return freq;}
		if (mostCommon<freq){mostCommon = freq;}
		}//and for allele
	return mostCommon;
	}//end CodominantMarkerData::GetFreqMostCommonAllele



long double CodominantMarkerData::GetFreqMostCommonAllele (int loc){
	long double freq,mostCommon = zero;
	for(int allele=0;allele<GetNumAlleles(loc);++allele){
		freq = GetUnweightedFreq(loc,allele);
		if (freq>half) {return freq;}
		if (mostCommon<freq){mostCommon = freq;}
		}//and for allele
	return mostCommon;
	}//end CodominantMarkerData::GetFreqMostCommonAllele



void CodominantMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
	//uploads data to pass to genetic analysis classes
	pgcList.Clear();
	int pop,i, dt;
	//initialize
	aList<int> lociByType;
	for(dt=0;dt<MultiInheritanceModeContainer<int>::inheritanceModes_;++dt){
		lociByType.Append(0);}
	lociByType[MultiInheritanceModeContainer<int>::diploid]=nlociCodom_;
	for(pop=0;pop<npop_;++pop){
		pgcList.Append(NEW PopulationGeneContainer<int>(popSize(pop),lociByType));}
	
	//fill up
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<popSize(pop);++i){
			GetGenome(pgcList[pop][i], pop, i);}
		}
	}//CodominantMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >&)



void CodominantMarkerData::GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID){
	//uploads this indiv's data to MultiHaplotypeContainer<int>
	MultiHaplotypeContainer<int>* mhc = (MultiHaplotypeContainer<int>*) &gc;
	mhc->SetSize(ploidy(),nlociCodom_);
	int loc,c,tr;
	if(setID){
		mhc->SetID(1+GetIndivNumInDataSequence(pop,indiv));}
	for(loc=0;loc<nlociCodom_;++loc){
		tr=locusNumCodom_[loc];
		for(c=0;c<ploidy();++c){
//			TextDatum& gval = CodominantMarkerData::GenotypicVal(pop,indiv,loc,c);
			TextDatum& gval = CodominantMarkerData::GenotypicVal(pop,indiv,tr,c);
			if(gval.missing()){
					mhc->SetMissing(c);}
				else{
					mhc->setHaplotype(loc,c,CodominantMarkerData::alleleID(loc,gval));}
			}}	
	}//CodominantMarkerData::GetGenome(GeneContainerCore&, int, int)



/* see how this is used before debugging
aString* CodominantMarkerData::GetGenotypeString(int pop, int indiv){
	StringList& genotypesList=gtypes[pop];
	aString& dataStr = genotypesList[indiv];
	aString *gtypeWord = NEW aString(dataStr);
	aString *codeWord = gtypeWord->ChopOffFirstWord();
	delete codeWord; codeWord=0;
	return gtypeWord;
	}//end CodominantMarkerData::GetGenotypeString(pop,indiv)



aString* CodominantMarkerData::GetGenotypeString(int pop, int indiv, int loc){
	aString *gtypeWord=GetGenotypeString(pop, indiv);
	int posGtypeWord = 1 + (2+1) * loc;
	aString *gTypeString = NEW aString(*gtypeWord,posGtypeWord,posGtypeWord +1);
	delete gtypeWord;gtypeWord=0;//this was created in GetGenotypeString(pop, indiv);
	return gTypeString;
	}//end CodominantMarkerData::GetGenotypeString(pop,indiv,loc)


aString* CodominantMarkerData::GetGenotypeString(int pop, int indiv, int loc, int chrom){
//	aString *gtypeString=GetGenotypeString(pop, indiv, loc);
			//	aString *gTypeChrom = NEW aString(*gtypeString,chrom,chrom);
//	aString *gTypeChrom = NEW aString( (*gtypeString)[chrom]);
//	delete gtypeString;gtypeString=0;//this was created in GetGenotypeString(pop, indiv, loc);
//	return gTypeChrom;

	StringList gtypeString=Genotype(pop, indiv, loc);
	aString *gTypeChrom = NEW aString( gtypeString[chrom]);
	return gTypeChrom;
	}//end CodominantMarkerData::GetGenotypeString(pop,indiv,loc,chrom)
*/

long CodominantMarkerData::GetIndividualsSampled (int pop, int loc){
	long i,n=0;
	for(i=0;i<individuals(pop);++i){
		if(GetLocusSampleSize (pop, i, loc)>0){
			++n;}}
	return n;
	}//end  CodominantMarkerData::GetIndividualsSampled


long CodominantMarkerData::GetLocusSampleSize (int loc){
	if(! alleleFreqCurrent_){
		CalculateAlleleFreqs();}//calculates twoN_
	double numAlleleCopies = 0.0;
	for(int pop=0;pop<npop_;++pop){
		numAlleleCopies += twoN_[pop][loc];
		}
	return numAlleleCopies;
	}//end  CodominantMarkerData::GetLocusSampleSize


long CodominantMarkerData::GetLocusSampleSize (int pop, int loc){
	if(! alleleFreqCurrent_){
		CalculateAlleleFreqs();}//calculates twoN_
	return twoN_[pop][loc];
	}//end  CodominantMarkerData::GetLocusSampleSize


int CodominantMarkerData::GetLocusSampleSize (int pop, int indiv, int loc){
	int tr=locusNumCodom_[loc];
//	StringList g(CodominantMarkerData::GenotypicVal(pop,indiv,loc));
	StringList g(CodominantMarkerData::GenotypicVal(pop,indiv,tr));
	return g.len();//assumes that missing data have been culled
	}//end  CodominantMarkerData::GetLocusSampleSize


long double CodominantMarkerData::GetMeanAllelesPerLocus (int pop){
	long double meanAlleles = zero;
	int loc;
	for(loc=0;loc<nlociCodom_;++loc){
		meanAlleles += (long double)GetNumAlleles(pop,loc);}
	meanAlleles /= (long double)nlociCodom_;
	return meanAlleles;
	}//end  CodominantMarkerData::GetMeanAllelesPerLocus


int CodominantMarkerData::GetNumAlleles (int loc){
	//total in data set
	if(! allelesCurrent_){CheckForNewAlleles();}
	StringList& locList = allelesPresent_[loc];
	return locList.len();
	}//end  CodominantMarkerData::GetNumAlleles(loc)


int CodominantMarkerData::GetNumAlleles (int pop, int loc){
	//extant alleles in population
	if(! alleleFreqCurrent_){CalculateAlleleFreqs();}
	ObjList<aList<long double> >& popList = alleleFreq_[pop];
	aList<long double>& locList = popList[loc];
	int count=0;
	for(int a=0;a<locList.len();++a){
		if (locList[a] >0.0){++count;}}
	return count;
	}//end  CodominantMarkerData::GetNumAlleles(pop,loc)


long double CodominantMarkerData::GetObservedHeterozygosity(int pop, int loc, int allele){
	if (!HetObsCurrent_){CalculateObservedHeterozygosity();}
	ObjList<aList<long double> >& popList = HetObsByAllele_[pop];
	aList<long double>& locList = popList[loc];
	return locList[allele];
	}//end CodominantMarkerData::GetObservedHeterozygosity(pop,loc,allele)


long double CodominantMarkerData::GetObservedHeterozygosity(int pop, int loc){
	if (!HetObsCurrent_){CalculateObservedHeterozygosity();}
	aList<long double>& popList = HetObsByLocus_[pop];
	return popList[loc];
	}//end CodominantMarkerData::GetObservedHeterozygosity(pop,loc)


long double CodominantMarkerData::GetObservedHeterozygosity (int pop){
	//returns mean Hobs over loci
	StatList<long double> hetObs;
	for (int loc=0;loc<nlociCodom_;++loc){
		hetObs.Append(GetObservedHeterozygosity(pop,loc));}
	return hetObs.mean();
	}//end CodominantMarkerData::GetObservedHeterozygosity(pop)



long double CodominantMarkerData::GetPercentPolymorphicLoci(int pop, long double criterion){
	//default criterion is 100.0% = any polymorphism
	long double percentP = 0.0;
	criterion /= 100.0; 
	for(int loc=0;loc<nlociCodom_;++loc){
		long double mostCommon = GetFreqMostCommonAllele(pop,loc);
		if(mostCommon<(long double)criterion){++percentP;}}
	return (percentP/(long double)nlociCodom_) *(long double)100.0;
	}//end CodominantMarkerData::GetPercentPolymorphicLoci(pop,loc)


aString* CodominantMarkerData::GetTraitString(int pop, int indiv, int loc, int allele){
	//loc indexed as in locusNumCodom_ 
	aString *aStr = NEW aString();
	int tr = locusNumCodom_[loc];
	int tra = allCodomLoci_.Loc(tr);
	aStr->cat(rawCodomGtypes_[pop][indiv][tra][allele]);
	return aStr;
	}//CodominantMarkerData::GetTraitString


long double CodominantMarkerData::GetVarJackAllelesPerLocus (int pop){
	//returns jackknife variance of means, dropping loci
	StatList<long double> allelesPerLoc;
	for (int loc=0;loc<nlociCodom_;++loc){
		allelesPerLoc.Append((long double)GetNumAlleles(pop,loc));}
	StatList<long double> jackAllelesPerLoc;
	for (int loc=0;loc<nlociCodom_;++loc){
		long double storeValue = allelesPerLoc.First();
		allelesPerLoc.DeleteFirst();
		jackAllelesPerLoc.Append(allelesPerLoc.mean());
		allelesPerLoc.Append(storeValue);
		}
	return jackAllelesPerLoc.var();
	}//end CodominantMarkerData::GetVarJackAllelesPerLocus(pop)



long double CodominantMarkerData::GetVarJackExpectedHeterozygosity (int pop){
	//returns jackknife variance of means, dropping loci
	StatList<long double> hetExpLoc;
	for (int loc=0;loc<nlociCodom_;++loc){
		hetExpLoc.Append(GetExpectedHeterozygosity(pop, loc));}
	StatList<long double> jackHetExpLoc;
	for (int loc=0;loc<nlociCodom_;++loc){
		long double storeValue = hetExpLoc.First();
		hetExpLoc.DeleteFirst();
		jackHetExpLoc.Append(hetExpLoc.mean());
		hetExpLoc.Append(storeValue);
		}
	return jackHetExpLoc.var();
	}//end CodominantMarkerData::GetVarJackExpectedHeterozygosity(pop)



long double CodominantMarkerData::GetVarJackObservedHeterozygosity (int pop){
	//returns jackknife variance of means, dropping loci
	StatList<long double> hetObsLoc;
	for (int loc=0;loc<nlociCodom_;++loc){
		hetObsLoc.Append(GetObservedHeterozygosity(pop, loc));}
	StatList<long double> jackHetObsLoc;
	for (int loc=0;loc<nlociCodom_;++loc){
		long double storeValue = hetObsLoc.First();
		hetObsLoc.DeleteFirst();
		jackHetObsLoc.Append(hetObsLoc.mean());
		hetObsLoc.Append(storeValue);
		}
	return jackHetObsLoc.var();
	}//end CodominantMarkerData::GetVarJackObservedHeterozygosity(pop)



long double CodominantMarkerData::GetVarJackPercentPolyLoci (int pop, long double criterion){
	//returns jackknife variance of means, dropping loci
	StatList<long double> jackPercentP;
	for (int loc=0;loc<nlociCodom_;++loc){
		jackPercentP.Append(GetPercentPolymorphicLoci(pop,criterion));}
	for (int loc=0;loc<nlociCodom_;++loc){
		double long temp = (jackPercentP[loc]/100.0)*nlociCodom_;
		if(IsPolymorphic(pop,loc,criterion)){--temp;}
		jackPercentP.Replace((temp/(long double)(nlociCodom_-1))*(long double)100.0,loc);
		}
	return jackPercentP.var();
	}//end CodominantMarkerData::GetVarJackPercentPolyLoci(pop,criterion)


bool CodominantMarkerData::individualIsUnscored(int pop, int indiv, int loc){
	bool scored=false;
	TextDatum& g0 = GenotypicVal(pop, indiv, locusNumCodom_[loc], 0);
	TextDatum& g1 = GenotypicVal(pop, indiv, locusNumCodom_[loc], 1);
	if(g0.OK() && g1.OK()){
		scored = true;}
	return ! scored;
	}


int  CodominantMarkerData::IsPolymorphic(int loc){//boolean
	if(! allelesCurrent_) {CheckForNewAlleles();}
	if (allelesPresent_[loc].len()>1){ return 1;}
	return 0;
	}// end CodominantMarkerData::IsPolymorphic


int  CodominantMarkerData::IsPolymorphic(int loc, long double criterion){//boolean
	long double freq = GetFreqMostCommonAllele(loc);
	if (freq<(criterion/100.0)){return 1;}else {return 0;}
	}// end CodominantMarkerData::IsPolymorphic


int  CodominantMarkerData::IsPolymorphic(int pop, int loc){//boolean
	double freq = GetFreqMostCommonAllele(pop,loc);
	if (freq==0.0){return 0;}
	if (freq<1.0){return 1;}else {return 0;}
	}// end CodominantMarkerData::IsPolymorphic


int  CodominantMarkerData::IsPolymorphic(int pop, int loc, long double criterion){//boolean
	double freq = GetFreqMostCommonAllele(pop,loc);
	if (freq==0.0){return 0;}
	if (freq<(criterion/100.0)){return 1;}else {return 0;}
	}// end CodominantMarkerData::IsPolymorphic


long CodominantMarkerData::maxall(void){
	//maximum alleles per locus in data set
	CheckForNewAlleles();
	long maxalleles=0;
	for(int loc=0;loc<nlociCodom_;++loc){
		maxalleles = MAX(maxalleles,allelesPresent_[loc].len());}
	return maxalleles;
	}//CodominantMarkerData::maxall()


//void CodominantMarkerData::ParseGenotypicData(aList<int>& codominantLoci,int startingLocus){
void CodominantMarkerData::ParseGenotypicData(int /*startingLocus*/){
	//convert from raw trait data
	//assumes 
	rawCodomGtypes_.Clear();
//	if(codominantLoci.len()==0) return;
	ResetFlags();
//	codominantLoci.Sort();
	int loc,pop,i,c,tr,item,sLoc;
//	for(loc=0;loc<codominantLoci.len();++loc){
//		codominantLoci[loc]+=startingLocus;}//because traitName_[0] is the id number
//	allCodomLoci_=locusNumCodom_=codominantLoci;
//	nlociCodom_=codominantLoci.len();
	
	for(pop=0;pop<npop_;++pop){//parse skipped loci too
		rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
		for(i=0;i<individuals(pop);++i){
			rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
			for(loc=0;loc<allCodomLoci_.len();++loc){
				tr = allCodomLoci_[loc];
				TextDatum& genotype = rawTraits_[pop][i][tr];
				rawCodomGtypes_[pop][i].Append(NEW TextDataList());
				for(c=0;c<ploidy_;++c){
					if(genotype.missing()){//append all missing data flags
							rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum());}
						else{//parse the genotype and handle each allele copy separately
							aString gene(genotype,c*alleleLen_+1,(c+1)*alleleLen_);//check this
							gene.CullLeadingWhiteSpace();
							gene.CullTrailingWhiteSpace();
							if(missing_.Includes(&gene)){
									rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum());}
								else{
									rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(gene));}
							}
					}//c
				}//loc
			}//i
		}//pop
	for(sLoc=0;sLoc<skippedTraitNum_.len();++sLoc){
		if(locusNumCodom_.Includes(skippedTraitNum_[sLoc])){
			item = locusNumCodom_.Loc(skippedTraitNum_[sLoc]);
			locusNumCodom_.Delete(item);
			--nlociCodom_;
			if(! skippedLocusNumCodom_.Includes(skippedTraitNum_[sLoc])){
				skippedLocusNumCodom_.Append(skippedTraitNum_[sLoc]);}
			}
		}//sLoc
	skippedLocusNumCodom_.Sort();
	CheckForNewAlleles();
	}//CodominantMarkerData::ParseGenotypicData(aList<int>& )


void  CodominantMarkerData::PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of allelic frequencies for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculateAlleleFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied


	outFile<<title_<<endl<<endl<<endl;
	outFile<<"ALLELE FREQUENCY DATA"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop_<<endl;
	outFile<<"NUMBER OF LOCI SURVEYED = ";outFile.width(3);outFile<<nlociCodom_<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (skippedLocusNumCodom_.len() == 0){
			outFile<<colSpacer<<"none";}
		else{
			outFile<<colSpacer;
			for (int loc=0;loc<skippedLocusNumCodom_.len();++loc){
				aString& locName = traitName_[skippedLocusNumCodom_[loc]];
				outFile<<locName;
				if (loc < skippedLocusNumCodom_.len()-1){outFile<<colSpacer;}
				}//end for loc
		}//end if lociSkipped
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop_ / ncols;
	int npop0 = npop_;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Allele frequencies of CODOMINANT MARKERS for each population."<<endl;
				outFile<<"Number of alleles sampled in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop_, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;

		for (int loc=0;loc<nlociCodom_;++loc) {//print the locus and locus sample size
			aString& locWord = traitName_[locusNumCodom_[loc]];
			outFile<< locWord;
			if (asciiFormat){outFile.width(11 + (5 - locWord.len()));}else{outFile<<colSpacer;}
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
//				int samSize = GetLocusSampleSize(pop, loc) / ploidy_;
				int samSize = GetLocusSampleSize(pop, loc);
				outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
				if (pop < endPop){
					if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
					}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent_[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					long double freq = GetFreq(pop, loc, allele);
					aString freqStr(freq,0,4);
					if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									if (asciiFormat){
											aString blanks(' ',12);
											if (pop==x) {
												blanks.PrependCharToLen(' ',17);}
											outFile<<blanks;
											}
										else{outFile<<colSpacer;}
									}
								else{
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end CodominantMarkerData::PrintAlleleFreqs




void  CodominantMarkerData::PrintAlleleFreqsByPopList (ostream& outFile, 
											ObjList<ObjList<aList<long double> > >& popFreqList, 
											ObjList<aList<long double> >& popTwoNList, 
											int ncols){
	//popFreqList must contain the entire set of alleles in the data set
	//Sends a table of allelic frequencies to the output file.
	//ncols indexes the number of population columns per page

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = popFreqList.len() / ncols;
	int npop0 = popFreqList.len();
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Number of alleles sampled in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)popFreqList.len(), (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			outFile.width(12);outFile<<pop+1;}
		outFile<<endl;

		for (int loc=0;loc<nlociCodom_;++loc) {//print the locus and locus sample size
			aString& locWord = traitName_[locusNumCodom_[loc]];
			outFile<< locWord;outFile.width(11 + (5 - locWord.len()));
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
//				int samSize = popTwoNList[pop][loc] / ploidy_;
				int samSize = popTwoNList[pop][loc];
				outFile<<"[";outFile.width(2);outFile<<samSize<<"]";
				if (pop < endPop){outFile.width(9);}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent_[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = popFreqList[pop][loc][allele];
					aString freqStr(freq,0,4);
					freqStr.PrependCharToLen(' ',12);
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									aString blanks(' ',12);
									if (pop==x) {
										blanks.PrependCharToLen(' ',17);}
									outFile<<blanks;}
								else{
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end CodominantMarkerData::PrintAlleleFreqsByPopList



void	 CodominantMarkerData::PrintGelJockFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintGelJockFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end CodominantMarkerData::PrintGelJockFile(aString&)



void	 CodominantMarkerData::PrintArlequinFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintArlequinFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end CodominantMarkerData::PrintArlequinFile(aString&)



void CodominantMarkerData::PrintGelJockFile (ostream& outFile){
	aString dataLine = title_;
	dataLine.cat(":  skipped loci omitted");
	outFile<<dataLine<<endl;
	dataLine = "NPOP=";
	dataLine.cat(npop_);
	dataLine.cat(", NLOCI=");
	dataLine.cat(nlociCodom_);
	dataLine.cat(", MAXALL=");
	dataLine.cat(maxall());
	dataLine.cat(", NGROUPS=");
	dataLine.cat(ngroups_);
	outFile<<dataLine<<endl;
	dataLine.clear();
	int pop,i,loc,c,tr;
	for (loc=0;loc<locusNumCodom_.len();++loc){
		dataLine.cat(traitName_[locusNumCodom_[loc]]);
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < locusNumCodom_.len()){
				dataLine.cat(", ");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	dataLine = "SKIP=0";
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (pop=0;pop<npop_;++pop){
		outFile<<popName_[pop]<<endl;
		for (i=0;i<individuals(pop);++i){
			dataLine = rawTraits_[pop][i][0];//individual's ID code is the first trait
			for(loc=0;loc<nlociCodom_;++loc){
				dataLine+' ';
				tr=locusNumCodom_[loc];
				for(c=0;c<ploidy_;++c){
//					dataLine + rawCodomGtypes_[pop][i][locusNumCodom_[loc]][c];}
					dataLine + CodominantMarkerData::GenotypicVal(pop,i,tr,c);}
				}//loc
			outFile<<dataLine<<endl;
			}//end for i
		outFile<<"NEXT"<<endl;
		}//end for pop
	outFile<<"END"<<endl;
	}//end CodominantMarkerData::PrintGelJockFile


void CodominantMarkerData::PrintArlequinFile (ostream& outFile){
	//		theGenotypes: ListStrStructureTypePtr;
	outFile<<"[Profile]"<<endl;
	aString dataLine = "\tTitle=\"";
	dataLine.cat(title_);
	dataLine.cat('\"');
	outFile<<dataLine<<endl;
	dataLine = "\t\tNbSamples=";
	dataLine.cat(npop_);
	outFile<<dataLine<<endl;
	dataLine = "\t\tGenotypicData=1";
	outFile<<dataLine<<endl;
	dataLine = "\t\tGameticPhase=0";
	outFile<<dataLine<<endl;
	dataLine = "\t\tRecessiveData=0";
	outFile<<dataLine<<endl;
	dataLine = "\t\tIndividualLevel=1";
	outFile<<dataLine<<endl;
	dataLine = "\t\tMissingData=\"\?\"";
	outFile<<dataLine<<endl;
	dataLine = "\t\tCompDistMatrix=1";
	outFile<<dataLine<<endl<<endl;

	dataLine = "\t\tDataType=STANDARD";
	outFile<<dataLine<<endl;
	dataLine = "\t\tFrequency=ABS";
	outFile<<dataLine<<endl;
	dataLine = "\t\tLocusSeparator=WHITESPACE";
	outFile<<dataLine<<endl<<endl;
		
	dataLine = "[Data]";
	outFile<<dataLine<<endl<<endl;
	dataLine = "#Locus names:";
	outFile<<dataLine<<endl;

	dataLine= "\t# ";
	for (int loc=0;loc<locusNumCodom_.len();++loc){
		dataLine.cat(traitName_[locusNumCodom_[loc]]);
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine= "\t# ";}
			else if (loc+1 < locusNumCodom_.len()){
				dataLine.cat("\t");}
			else{
				outFile<<dataLine<<endl;
				dataLine= "\t# ";}
		}//end for loc
	outFile<<endl<<endl;
	dataLine = "[[Samples]]";
	outFile<<dataLine<<endl<<endl;
	dataLine.clear();
	int pop,i,loc,tr,c;
	for (pop=0;pop<npop_;++pop){
		dataLine = "\t\tSampleName=\"";
		dataLine.cat(popName_[pop]);
		dataLine.cat('\"');
		outFile<<dataLine<<endl;
//		StringList& theGenotypes = gtypes[pop];
		dataLine = "\t\tSampleSize=";
		dataLine.cat(individuals(pop));
		outFile<<dataLine<<endl;
		dataLine = "\t\tSampleData={";
		outFile<<dataLine<<endl;
		StringList pseudohaps;
		for (i=0;i<individuals(pop);++i){
			pseudohaps.Clear();
			//split each genotype into ploidy_ pieces, = "pseudohaplotypes", on separate rows
			pseudohaps.Append(NEW aString(rawTraits_[pop][i][0]));//ID code
			pseudohaps[0].cat(" 1 ");//frequency of this genotype
//			aString g(theGenotypes[i]);
//			aString endCharList=" \t";
//			aString *phaplo0 = g.ChopOffFirstWord(endCharList);//ID code
//			phaplo0->cat(" 1 ");
			for(c=1;c<ploidy_;++c){
				pseudohaps.Append(NEW aString());
				pseudohaps[c].PrependCharToLen(' ',pseudohaps[0].len());}
//			aString  phaplo1;phaplo1.PrependCharToLen(' ',phaplo0->len());
//			outFile<<phaplo1<<endl;
			for(loc=0;loc<nlociCodom_;++loc){
				tr=locusNumCodom_[loc];
				for(c=0;c<ploidy_;++c){
//					TextDatum& allele = rawCodomGtypes_[pop][i][locusNumCodom_[loc]][c];
					TextDatum& allele = CodominantMarkerData::GenotypicVal(pop,i,tr,c);
					if(allele.OK()){
							pseudohaps[c].cat(allele);}
						else{//missing data 
							pseudohaps[c].cat('\?');}
					pseudohaps[c].cat(' ');
					}//c
/*				aString *gLoc0 = GetGenotypeString(pop,i,loc,0);
				aString *gLoc1 = GetGenotypeString(pop,i,loc,1);
				if(*gLoc0==' '){*gLoc0='\?';}
				if(*gLoc1==' '){*gLoc1='\?';}
				phaplo0->cat(*gLoc0);phaplo0->cat(' ');
				phaplo1.cat(*gLoc1);phaplo1.cat(' ');
				delete gLoc0; delete gLoc1;
*/				}//loc
			pseudohaps.Print(outFile);
//			outFile<<*phaplo0<<endl;
//			outFile<< phaplo1<<endl;
//			delete phaplo0;
			}//end for i
		outFile<<"\t\t}"<<endl<<endl;
		}//end for pop
	outFile<<endl<<endl;
	dataLine = "[[Structure]]";
	outFile<<dataLine<<endl<<endl;

	dataLine = "StructureName = \"Structure from population titles\"";
	outFile<<dataLine<<endl;
	dataLine = "NbGroups = 1";
	outFile<<dataLine<<endl;
	dataLine = "IndividualLevel = 1";
	outFile<<dataLine<<endl;
	for(int gr=0;gr<ngroups_;++gr){
		dataLine = "\t#";
		dataLine.cat(groupName_[gr]);
		outFile<<dataLine<<endl;
		dataLine = "\tGroup =  {";
		outFile<<dataLine<<endl;
		for(int pop=0;pop<groupHierarchy_[gr].len();++pop){
			outFile<<"\t\t\""<<popName_[groupHierarchy_[gr][pop]]<<'\"'<<endl;
			}
		outFile<<"\t\t}"<<endl;
	}


	outFile<<endl;
	}//end CodominantMarkerData::PrintArlequinFile





void CodominantMarkerData::PrintHeterozygosity (ostream& outFile, const int hetType, int ncols){
	//file assumed open
	//Sends a table of the designated heterozygosities for each population to the output file.
	//ncols indexes the number of population columns per page
	outFile<<title_<<endl<<endl<<endl;
	if (hetType == observed) {
			outFile<<"OBSERVED HETEROZYGOSITY TABLE";}
		else{
			outFile<<"EXPECTED HETEROZYGOSITY TABLE";}
	outFile<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop_<<endl;
	outFile<<"NUMBER OF LOCI SURVEYED = ";outFile.width(3);outFile<<nlociCodom_<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (lociSkipped() == 0){
			outFile<<"none";}
		else{
			for (int loc=0;loc<lociSkipped();++loc){
				aString& locName = traitName_[skippedLocusNumCodom_[loc]];
				outFile<<locName;
				if (loc < lociSkipped()-1){outFile<<", ";}
				}//end for loc
		}//end if lociSkipped
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop_ / ncols;
	int npop0 = npop_;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				if (hetType == expected) {
						outFile<<"Expected ";}
					else{
						outFile<<"Observed ";}
				outFile<<"heterozygosities for each allele in each population."<<endl;
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				if (hetType == expected) {
						outFile<<"Expected ";}
					else{
						outFile<<"Observed ";}
				outFile<<"heterozygosity table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop_, (ncols * (block+1)))-1;
		outFile<<"pop. ";		
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			outFile.width(12);outFile<<pop+1;}
		outFile<<endl;

		for (int loc=0;loc<nlociCodom_;++loc) {//print the locus and locus sample size
			aString& locWord = traitName_[locusNumCodom_[loc]];
			outFile<< locWord;outFile.width(11 + (5 - locWord.len()));
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = GetLocusSampleSize(pop, loc) / ploidy_;
				outFile<<"[";outFile.width(2);outFile<<samSize<<"]";
				if (pop < endPop){
					outFile.width(9);}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent_[locusNumCodom_[loc]];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double hets=0.0;
					if (hetType == observed) {
							hets = GetObservedHeterozygosity(pop, loc, allele);}
						else{
							hets = GetExpectedHeterozygosity(pop, loc, allele);}
					aString hetStr(hets,0,4);
					hetStr.PrependCharToLen(' ',12);
					if (pop == endPop){//just write the freq if >0
							if (hets > 0.0){
									if (pop==x) {
										hetStr.PrependCharToLen(' ',17);}
								 	outFile<<hetStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if hets=0}
							if (hets == 0.0){
									aString blanks(' ',12);
									if (pop==x) {
										blanks.PrependCharToLen(' ',17);}
									outFile<<blanks;}
								else{
									if (pop==x) {
										hetStr.PrependCharToLen(' ',17);}
									outFile<<hetStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block
	}//end CodominantMarkerData::PrintHeterozygosity



void CodominantMarkerData::PrintLoci (ostream& outFile, int lociPerLine){//file assumed open
	int lineSpot = 0;
	for (int loc=0;loc<locusNumCodom_.len();++loc){ 
		aString& locName = traitName_[locusNumCodom_[loc]];
		outFile<<locName;
		++lineSpot;
		if ((loc == locusNumCodom_.len()-1) || (lineSpot == lociPerLine)) {
				outFile<<endl;}
			else{
				outFile<<", ";}
		}//end for loc
	outFile<<endl;
	}//end CodominantMarkerData::PrintLoci





void	 CodominantMarkerData::PrintSASFormatData(ostream& outFile, char colSpacer){
	//prints a data set that can be read using SAS
	//missing data represented by '.', columns separated by spaces
	//possible alleles for each locus in separate columns

	//this and next loop comprise the column titles
	int pop,i,gamete,loc,allele,tr;
	outFile<<"pop"<<colSpacer<<"indiv"<<colSpacer<<"gamete"<<colSpacer;
	for(loc=0;loc<nlociCodom_;++loc){
		aString& locName=traitName_[locusNumCodom_[loc]];
		StringList& alleleList=allelesPresent_[loc];
		for(allele=0;allele<alleleList.len();++allele){
			aString alleleTitle(locName);
			alleleTitle.cat('_');
			alleleTitle.cat(alleleList[allele]);
			alleleTitle.cat(colSpacer);
			outFile<<alleleTitle;}
		}//end for loc
	outFile<<endl;
	for(pop=0;pop<npop_;++pop){
		aString popStr(pop+1);
		popStr.cat(colSpacer);
		for(i=0;i<individuals(pop);++i){
			aString indivStr(popStr);
			indivStr.cat(i+1);
			indivStr.cat(colSpacer);
			for (gamete=0;gamete<ploidy_;++gamete){
				aString genotypeStr(indivStr);
				genotypeStr.cat(gamete+1);
				genotypeStr.cat(colSpacer);
				for(loc=0;loc<nlociCodom_;++loc){
					StringList& alleleList=allelesPresent_[loc];
					tr=locusNumCodom_[loc];
					for(allele=0;allele<alleleList.len();++allele){
						aString& alleleName = alleleList[allele];
						TextDatum& gene = CodominantMarkerData::GenotypicVal(pop, i, tr, gamete);
						if(gene.missing()){
								genotypeStr.cat('.');}
							else if(gene==alleleName){
								genotypeStr.cat('1');}
							else{
								genotypeStr.cat('0');}
						genotypeStr.cat(colSpacer);
						}//end for allele
					}//end for loc
				outFile<<genotypeStr<<endl;
				}//end for gamete
			}//end for i
		}//end for pop
	outFile<<endl;		
	}//end  CodominantMarkerData::PrintSASFormatData(ostream&,char)
	
	
void	 CodominantMarkerData::PrintSASFormatData(aString& outFileName, char colSpacer){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintSASFormatData(dataFile,colSpacer);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end CodominantMarkerData::PrintSASFormatData(aString&,char)



void CodominantMarkerData::PrintVariabilityScores(aString& outFileName, char sepChar){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintVariabilityScores(dataFile, sepChar);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end CodominantMarkerData::PrintVariabilityScores(aString&, char)



void CodominantMarkerData::PrintVariabilityScores(ostream& outFile, char sepChar){
	//This calls for subroutines which calculate standard genetic variability measures:
	//>  "POLYLOCI" =  % polymorphic loci and maan number of alleles per locus
	//>  "HETEXPECTED" = heterozygosity expected from Hardy-Weinberg ratios,
	//         with a jackknife variance calculation
	// >  "HETOBSERVED" = heterozygosity obtained directly from genotype frequencies,
	//         with jackknife variance calculation
	// >  And a summary table of these values for each population.

	// The following is the summary table
	// The populations are printed out by number and name, with
	// only the first 20 characters of the name used.
	int charsInPrintedPopName = 10;

	// title_ and column headings
	outFile<<endl<<endl;
	outFile<<"Genetic variability scores for all populations."<<endl;
	outFile<<"(Standard errors in parentheses.)"<<endChr<<endl;
	aString headings = " # ";
	headings.separator(1, sepChar);                 					 //   5 characters
	headings.cat("population"); headings.separator(4, sepChar);            // 13 characters
	headings.cat("alleles/loc"); headings.separator(4, sepChar);           // 13 characters
	headings.cat("%P(all)"); headings.separator(7, sepChar);            	 // 13 characters
	headings.cat("%P(99%)"); headings.separator(7, sepChar);            // 13 characters
	headings.cat("%P(95%)"); headings.separator(8, sepChar);            // 13 characters
	headings.cat("H (obs)"); headings.separator(9, sepChar);            	 // 16 characters
	headings.cat("H (exp)");                                				 // 16 characters
	outFile<<headings<<endl;
	for(int pop=0;pop<npop_;++pop){
		// population number
		aString outLine(pop+1);
		outLine.PrependCharToLen(' ', MAX(2,Digits(npop_)));
    		outLine.separator(1, sepChar); 

       		aString popStr(popName_[pop]);
        	popStr.trunc(charsInPrintedPopName);
		popStr.separator(charsInPrintedPopName - popStr.len() + 3, sepChar);

		//mean number of alleles per locus
		aString meanAllPerLoc(GetMeanAllelesPerLocus(pop),2,2);
		popStr.cat(meanAllPerLoc);
		popStr.cat(" (");
		double stErr = sqrt(GetVarJackAllelesPerLocus(pop));
		aString varAllPerLoc(stErr,1,2);
		popStr.cat(varAllPerLoc);
		popStr.cat(")");popStr.separator(1, sepChar); 

		// %P detectable
		double p = GetPercentPolymorphicLoci(pop, 100.0);
		aString percentP(p,3,1);
		popStr.cat(percentP); popStr.cat(" (");
		stErr = sqrt(GetVarJackPercentPolyLoci(pop, 100.0) );
		aString varPercentP(stErr,2,1);
		popStr.cat(varPercentP);
		popStr.cat(")");popStr.separator(2, sepChar); 

		// %P at 99%
		p = GetPercentPolymorphicLoci(pop, 99.0);
		aString percentP99(p,3,1);
		popStr.cat(percentP99); popStr.cat(" (");
		stErr = sqrt(GetVarJackPercentPolyLoci(pop, 99.0) );
		aString varPercentP99(stErr,2,1);
		popStr.cat(varPercentP99);
		popStr.cat(")");popStr.separator(3, sepChar); 

		// %P at 95%
		p = GetPercentPolymorphicLoci(pop, 95.0);
		aString percentP95(p,3,1);
		popStr.cat(percentP95); popStr.cat(" (");
		stErr = sqrt(GetVarJackPercentPolyLoci(pop, 95.0) );
		aString varPercentP95(stErr,2,1);
		popStr.cat(varPercentP95);
		popStr.cat(")");popStr.separator(3, sepChar); 

		// Hobs with standard error in parentheses
		aString meanHetObs(GetObservedHeterozygosity(pop),1,3);
		popStr.cat(meanHetObs);
		popStr.cat(" (");
		stErr = sqrt(GetVarJackObservedHeterozygosity(pop));
		aString varHetObs(stErr,1,3);
		popStr.cat(varHetObs);
		popStr.cat(")");popStr.separator(3, sepChar); 

		// Hexp with standard error in parentheses
		aString meanHetExp(GetExpectedHeterozygosity(pop),1,3);
		popStr.cat(meanHetExp);
		popStr.cat(" (");
		stErr = sqrt(GetVarJackExpectedHeterozygosity(pop));
		aString varHetExp(stErr,1,3);
		popStr.cat(varHetExp);
		popStr.cat(")");

		outLine.cat(popStr);
		outFile<<outLine<<endl;
		}// end for pop

	outFile<<endl<<"Variances computed using a jackknife algorithm, dropping loci."<<endl<<endl;
	}//end  CodominantMarkerData::PrintVariabilityScores



int CodominantMarkerData::ReadArlequinDataFile (aString* dataFileName){
	fstream dataFile;	
	char *dataLine=NEW char[10001];dataLine[0]='\0';
//	aString beast, popLine;
	aString endCharList(" \t,:\"{}[]!?\n\r"), eq("="), parameterLine;
	char* filename=dataFileName->raw();
	dataFile.open(filename,ios::in);
	int dataFileOpen = dataFile.is_open();
	if(! dataFileOpen){
		aString errstr("couldn\'t open the file ");
		errstr.cat(filename);
		aError(errstr);
		return 0;}

	StringList rawArlequinData0;//read in everything as strings, then sort and process them
	ObjList<StringList> arlequinDataBySection;//sections and subsections
	int nArlSections=6;
	enum arlSections {profile=0,haplotype=1,distmatrices=2,samples=3,genStructure=4,mantelTests=5};
	aList<int> arlSectionStartLines, arlSectionEndLines;
	for(int sec0=0;sec0<nArlSections;++sec0){//initialize
		arlequinDataBySection.Append(NEW StringList());
		arlSectionStartLines.Append(-1);//flag
		arlSectionEndLines.Append(-1);//flag
		}

	aString rawDataLine;
	int lineNum=-1;//stores current line of valid data
	while(!dataFile.eof()){
		dataFile.getline(dataLine, 10000);
		rawDataLine = dataLine;
		SnipArlequinComment(rawDataLine);//removes comments
		rawDataLine.CullLeadingWhiteSpace();
		rawDataLine.CullTrailingWhiteSpace();
		if(rawDataLine.len()>0){//it's not blank
			rawArlequinData0.Append(NEW aString(rawDataLine));
			++lineNum;
			if(rawDataLine[0]=='['){//assume it's a section heading line
				aString linecopy(rawDataLine);//work with a copy
				linecopy.CullLeadingChar('[');
				linecopy.CullTrailingChar(']');
				linecopy.CullLeadingWhiteSpace();//in case of sloppy text
				linecopy.CullTrailingWhiteSpace();
				if(linecopy=="Profile" || linecopy=="profile" || linecopy=="PROFILE"){
						arlSectionStartLines[profile]=lineNum;
						arlSectionEndLines[profile]=lineNum-1;//end of previous section
						}
					else if(linecopy=="HaplotypeDefinition" || 
							linecopy=="haplotypedefinition" || 
							linecopy=="HAPLOTYPEDEFINITION"){
						arlSectionStartLines[haplotype]=lineNum;
						arlSectionEndLines[haplotype]=lineNum-1;//end of previous section
						}
					else if(linecopy=="DistanceMatrix" || 
							linecopy=="distancematrix" || 
							linecopy=="DISTANCEMATRIX"){
						arlSectionStartLines[distmatrices]=lineNum;
						arlSectionEndLines[distmatrices]=lineNum-1;//end of previous section
						}
					else if(linecopy=="Samples" || 
							linecopy=="samples" || 
							linecopy=="SAMPLES"){
						arlSectionStartLines[samples]=lineNum;
						arlSectionEndLines[samples]=lineNum-1;//end of previous section
						}
					else if(linecopy=="Structure" || 
							linecopy=="structure" || 
							linecopy=="STRUCTURE"){
						arlSectionStartLines[genStructure]=lineNum;
						arlSectionEndLines[genStructure]=lineNum-1;//end of previous section
						}
					else if(linecopy=="Mantel" || 
							linecopy=="mantel" || 
							linecopy=="MANTEL"){
						arlSectionStartLines[mantelTests]=lineNum;
						arlSectionEndLines[mantelTests]=lineNum-1;//end of previous section
						}
					else{}
				}//if '['
			}//len>0
		}//end while !eof
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;

	aList<int> sortedStartLines(arlSectionStartLines);
	aList<int> tempStartLines,tempEndLines;
	for(int sec2=0;sec2<arlSectionEndLines.len();++sec2){
		sortedStartLines.Append(arlSectionEndLines[sec2]);
		tempStartLines.Append(0);tempEndLines.Append(0);
		}
	sortedStartLines.Append(rawArlequinData0.len()-1);
	sortedStartLines.Sort();
	for(int sec3=0;sec3<arlSectionStartLines.len();++sec3){
		int posStart = sortedStartLines.Loc(arlSectionStartLines[sec3]);
		int endSection = sortedStartLines[posStart+1];
		arlSectionEndLines[sec3] = endSection;
		}//end for sec3
	
	if(arlSectionStartLines.Last() == -1){
		arlSectionStartLines.Last() = sortedStartLines.Last();
		arlSectionEndLines.Last() = sortedStartLines.Last();}
	for(int sec4 = nArlSections-2;sec4>=0;--sec4){
		if(arlSectionStartLines[sec4] == -1){
			arlSectionStartLines[sec4] = arlSectionStartLines[sec4+1];
			arlSectionEndLines[sec4] = arlSectionStartLines[sec4];}
		}//sec4
//	arlSectionStartLines.Print(cout," ");
//	arlSectionEndLines.Print(cout," ");
	
	//finish putting raw data into sections
	for(int sec5 = 0;sec5<nArlSections;++sec5){
		int start = arlSectionStartLines[sec5], end = arlSectionEndLines[sec5];
		for(int i=start+1; i<=end;++i){
			aString& processedDataLine = rawArlequinData0[i];
			arlequinDataBySection[sec5].Append(NEW aString(processedDataLine));
			}//i
//		arlequinDataBySection[sec5].Print(cout);
		}//sec5
		
	//now process each section separately
	// "[Profile]"//title_
	aString *tword=0, pline;
	for(int p0=0;p0<arlequinDataBySection[profile].len();++p0){
		pline = arlequinDataBySection[profile][p0];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "title_" || *tword == "Title" || *tword == "TITLE" ){
			pline.CullLeadingChar('\"');
			pline.CullTrailingChar('\"');
			title_ = pline;
			arlequinDataBySection[profile].Delete(p0);//no longer needed
			delete tword;tword=0;
			break;
			}//if==title_
		delete tword;tword=0;
		}

	// "[Profile]"//NbSamples
	for(int p1=0;p1<arlequinDataBySection[profile].len();++p1){
		pline = arlequinDataBySection[profile][p1];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "NbSamples" || *tword == "nbsamples" || *tword == "NBSAMPLES"){
			pline.CullLeadingWhiteSpace();
			npop_ = pline.toInt();
			arlequinDataBySection[profile].Delete(p1);//no longer needed
			delete tword;tword=0;
			break;
			}//if==NbSamples
		delete tword;tword=0;
		}

	// "[Profile]"//GenotypicData
	for(int p2=0;p2<arlequinDataBySection[profile].len();++p2){
		pline = arlequinDataBySection[profile][p2];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "GenotypicData" || *tword == "genotypicdata" || *tword == "GENOTYPICDATA"){
			pline.CullLeadingWhiteSpace();
			int gtypedata = pline.toInt();
			if(gtypedata != 1){
				aString err("Can't read in this type of data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p2);//no longer needed
			delete tword;tword=0;
			break;
			}//if==GenotypicData
		delete tword;tword=0;
		}

	// "[Profile]"//GameticPhase
	for(int p3=0;p3<arlequinDataBySection[profile].len();++p3){
		pline = arlequinDataBySection[profile][p3];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "GameticPhase" || *tword == "gameticphase" || *tword == "GAMETICPHASE"){
			pline.CullLeadingWhiteSpace();
			int gphasedata = pline.toInt();
			if(gphasedata != 0){
				aString err("Can't read this type of data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p3);//no longer needed
			delete tword;tword=0;
			break;
			}//if==GameticPhase
		delete tword;tword=0;
		}

	// "[Profile]"//RecessiveData
	for(int p4=0;p4<arlequinDataBySection[profile].len();++p4){
		pline = arlequinDataBySection[profile][p4];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "RecessiveData" || *tword == "recessivedata" || *tword == "RECESSIVEDATA"){
			pline.CullLeadingWhiteSpace();
			int recdata = pline.toInt();
			if(recdata != 0){
				aString err("Can't read this type of data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p4);//no longer needed
			delete tword;tword=0;
			break;
			}//if==RecessiveData
		delete tword;tword=0;
		}

	// "[Profile]"//DataType
	for(int p5=0;p5<arlequinDataBySection[profile].len();++p5){
		pline = arlequinDataBySection[profile][p5];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "DataType" || *tword == "datatype" || *tword == "DATATYPE"){
			pline.CullLeadingWhiteSpace();
			if(pline != "STANDARD"){
				aString err("Can only read DataType=STANDARD data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p5);//no longer needed
			delete tword;tword=0;
			break;
			}//if==DataType
		delete tword;tword=0;
		}

	// "[Profile]"//MissingData
	aString missingDataFlag("?");//default
	for(int p6=0;p6<arlequinDataBySection[profile].len();++p6){
		pline = arlequinDataBySection[profile][p6];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "MissingData" || *tword == "missingdata" || *tword == "MISSINGDATA"){
			pline.CullLeadingWhiteSpace();
			pline.CullLeadingChar('\"');//can be single or double quotes
			pline.CullTrailingChar('\"');
			pline.CullLeadingChar('\'');
			pline.CullTrailingChar('\'');
			missingDataFlag = pline;
			if(missingDataFlag.len() != 1){
				aString err("Missing data must be denoted by a single character.  Default used.");
				aError(err);
				missingDataFlag="?";
				}
			arlequinDataBySection[profile].Delete(p6);//no longer needed
			delete tword;tword=0;
			break;
			}//if==IndividualLevel
		delete tword;tword=0;
		}

	// "[Profile]"//RecessiveAllele
	aString RecessiveAlleleFlag("?");//default
	for(int p7=0;p7<arlequinDataBySection[profile].len();++p7){
		pline = arlequinDataBySection[profile][p7];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "RecessiveAllele" || *tword == "recessiveallele" || *tword == "RECESSIVEALLELE"){
			pline.CullLeadingWhiteSpace();
			pline.CullLeadingChar('\"');//can be single or double quotes
			pline.CullTrailingChar('\"');
			pline.CullLeadingChar('\'');
			pline.CullTrailingChar('\'');
			RecessiveAlleleFlag = pline;
			if(RecessiveAlleleFlag.len() == 0){
				aString err("Recessive allele (homozygote) not identified.  Default used.");
				aError(err);
				RecessiveAlleleFlag="null";
				}
			arlequinDataBySection[profile].Delete(p7);//no longer needed
			delete tword;tword=0;
			break;
			}//if==IndividualLevel
		delete tword;tword=0;
		}

	// "[Profile]"//Frequency
	for(int p8=0;p8<arlequinDataBySection[profile].len();++p8){
		pline = arlequinDataBySection[profile][p8];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "Frequency" || *tword == "frequency" || *tword == "FREQUENCY"){
			pline.CullLeadingWhiteSpace();
			if(!(pline == "ABS" || pline == "abs")){
				aString err("Can only read Frequency=ABS data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p8);//no longer needed
			delete tword;tword=0;
			break;
			}//if==DataType
		delete tword;tword=0;
		}


	// "[Profile]"//LocusSeparator
	aString locSep(" \t");//default
	for(int p9=0;p9<arlequinDataBySection[profile].len();++p9){
		pline = arlequinDataBySection[profile][p9];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "LocusSeparator" || *tword == "locusseparator" || *tword == "LOCUSSEPARATOR"){
			pline.CullLeadingWhiteSpace();
			if(pline == "WHITESPACE" || pline == "whitespace"){
					locSep = " \t";}
				else if(pline == "TAB" || pline == "tab"){
					locSep = "\t";}
				else if(pline == "COMMA" || pline == "comma"){
					locSep = ",";}
				else if(pline == "NONE" || pline == "none"){
					aString err("Need locus separator to read data from the Arlequin file");
					aError(err);
					delete tword;tword=0;
					return 2;}
				else{
					locSep = pline;
					if(locSep.len()==0){
						aString err("LocusSeparator not specified correctly.  Default used.");
						aError(err);
						locSep = " \t";}
					}
			arlequinDataBySection[profile].Delete(p9);//no longer needed
			delete tword;tword=0;
			break;
			}//if==DataType
		delete tword;tword=0;
		}

	//[Profile] //CompDistMatrix=  ignored
	//[Profile] //FrequencyThreshold=  ignored
	//[Profile] //EpsilonValue=  ignored

	//section [Data]

	//[Data] [[HaplotypeDesignation]]  ignored
	//[Data] [[DistanceMatrix]]  ignored


	//[Data] [[Sample]] 
	//read in population data
	long nloci=0;
	unsigned int alleleLength=1;
	aString popLine;
	ObjList<ObjList<ObjList<StringList> > > rawArlequinData;//temp storage [pop][i][loc][a]
	ObjList<StringList> rawIDnames;//temp storage [pop][id]
	ObjList<aList<long> > popGenotypeFreqs;//temp storage [pop][id]
	long dline=0;//data line counter
	for (int pop=0;pop<npop_;++pop){
		rawArlequinData.Append(NEW ObjList<ObjList<StringList> >());//set up raw data 
		rawIDnames.Append(NEW StringList());//set up raw data 
		popGenotypeFreqs.Append(NEW aList<long>());//set up raw data 
		
		//get the title_ of this population
		popLine = arlequinDataBySection[samples][dline++];
		aString* junk = popLine.ChopOffFirstWord(eq);
		delete junk; junk=0;
		popLine.CullLeadingWhiteSpace();
		popLine.CullLeadingChar('\"');
		popLine.CullTrailingChar('\"');
		aString *s = NEW aString(popLine);
		popName_.Append(s);//append a copy

		//read sample size to expect
		popLine = arlequinDataBySection[samples][dline++];
		junk = popLine.ChopOffFirstWord(eq);
		delete junk; junk=0;
		popLine.CullLeadingWhiteSpace();
		long indivs=popLine.toLong();
		long i=(-1);//a counter
		popLine.clear();
		popLine = arlequinDataBySection[samples][dline++];
		popLine.CullTrailingWhiteSpace();
		if(popLine[popLine.len()-1] != '{'){
			aString err("Expected \'SampleData={\' at ");err.cat(*s);//adds pop title_
			aError(err);}
		bool thisPopDone = false;
		while (!thisPopDone){//LOOK AT EACH INDIVIDUAL
			popLine.clear();
			popLine = arlequinDataBySection[samples][dline++];
			if(popLine[0] == '}'){
				thisPopDone = true;
				continue;}//goes to end of while !thisPopDone

			rawArlequinData[pop].Append(NEW ObjList<StringList>());//set up raw data
			i++; 
			
			//individuals come as 2 consecutive lines in diploid data
			//the first line contains an ID number and a frequency
			aString indivline1(popLine);
			aString *id=indivline1.ChopOffFirstWord();
			aString *fr=indivline1.ChopOffFirstWord();
			long gfreq = fr->toLong();
			delete fr; fr=0;
			rawIDnames[pop].Append(id);
			popGenotypeFreqs[pop].Append(gfreq);

			aString indivline2(arlequinDataBySection[samples][dline]);//sneak a peek ahead to check ploidy
			bool diploid = false;
			if(indivline2[0]=='}'){
					thisPopDone = true; break;}
				else if(indivline2.len() == indivline1.len()){//assume diploid based on symmetry in data file
					diploid=true;
					dline++;//increment the counter
					}
			long loc0=0;
			aString *gene=0;			
			while(indivline1.len()>0){
				rawArlequinData[pop][i].Append(NEW StringList());//new for each locus
				gene = indivline1.ChopOffFirstWord(locSep);
				alleleLength = MAX(alleleLength,gene->len());
				rawArlequinData[pop][i][loc0].Append(gene);
				++loc0;
				}//end while
			nloci = MAX(loc0,nloci);
			if(diploid){
				long loc01=0;
				while(indivline2.len()>0){
					gene = indivline2.ChopOffFirstWord(locSep);
					alleleLength = MAX(alleleLength,gene->len());
					rawArlequinData[pop][i][loc01].Append(gene);
					++loc01;
					}//end while
				nloci = MAX(loc01,nloci);
				}//if diploid
			}//end while !thisPopDone
		}//end for pop

	//make up some locus names
	traitName_.Clear();
	traitName_.Append(NEW aString("id"));
	ObjList<StringList> alleleNames;
	int maxloclen = Digits(nloci);
	for(int loc=0;loc<nloci;++loc){
		alleleNames.Append(NEW StringList());
		aString *lname = NEW aString("loc");
		aString lnum(loc+1);lnum.PrependCharToLen('0',maxloclen);
		lname->cat(lnum);
		traitName_.Append(lname);
		}//for loc

	//recode data, using a list of alleles/locus
	for(int pop1=0;pop1<npop_;++pop1){
		for(int i=0;i<rawArlequinData[pop1].len();++i){
			for(int loc1=0;loc1<nloci;++loc1){
				for(int h=0;h<rawArlequinData[pop1][i][loc1].len();++h){
					aString gene(rawArlequinData[pop1][i][loc1][h]);//work with copy
					if(! (alleleNames[loc1].Includes(&gene))){
						alleleNames[loc1].Append(NEW aString(gene));}
					}//end h
				}//loc1
			}//i
		}//pop1
	for(int loc1=0;loc1<nloci;++loc1){//sort; clear missing data flags
		alleleNames[loc1].Sort();
		if(alleleNames[loc1].Includes(&missingDataFlag)){
			int site = alleleNames[loc1].Loc(&missingDataFlag);
			alleleNames[loc1].Delete(site);}
		}//loc1
			
	//finish recoding
	alleleLen_=alleleLength;
	ploidy_=2;//assumed for now -- may be able to infer this from data
	rawTraits_.Clear();
	nlociCodom_ = nloci;
	aString missingDataToPrint(missingDataFlag);
	missingDataToPrint.PrependCharToLen(' ',alleleLen_);
	for(int pop2=0;pop2<npop_;++pop2){
		rawTraits_.Append(NEW ObjList<TextDataList>());				
		for(int i2=0;i2<rawArlequinData[pop2].len();++i2){
			TextDataList gtyp(missingDataFlag);
			gtyp.Append(NEW TextDatum(rawIDnames[pop2][i2]));
			for(int loc2=0;loc2<nloci;++loc2){
				bool datapresent=false;
				TextDatum *gtypLoc = NEW TextDatum();//starts out as missing
				for(int h2=0;h2<rawArlequinData[pop2][i2][loc2].len();++h2){
					aString gene2(rawArlequinData[pop2][i2][loc2][h2]);//work with copy
					if(gene2 == missingDataFlag){
							gtypLoc->cat(missingDataToPrint);}
						else{
							datapresent=true;
							gtypLoc->missing(false);
							gene2.PrependCharToLen(' ',alleleLen_);//standard size for all loci
							gtypLoc->cat(gene2);}
					}//end h2
				gtyp.Append(gtypLoc);
				}//loc2
			for(long n=0;n<popGenotypeFreqs[pop2][i2];++n){//use frequency info here
				rawTraits_[pop2].Append(NEW TextDataList(gtyp));}
			}//i	
		popSize_.Append(rawTraits_[pop2].len());
		}//pop2
		



	//[Data] [[Structure]] 
	//read in group information, if any
	bool readGroupInfo = false;
	if(arlequinDataBySection[genStructure].len() > 0){
		readGroupInfo=true;}

	if(! readGroupInfo){
			ngroups_=1;
			GroupMembership();}
		else{//read group info explicitly
			for(int s0=0;s0<arlequinDataBySection[genStructure].len();++s0){
				parameterLine = arlequinDataBySection[genStructure][s0];
				aString *pword=parameterLine.ChopOffFirstWord(eq);
				pword->CullTrailingWhiteSpace();
				if(*pword == "StructureName" || *pword == "structurename" || *pword == "STRUCTURENAME"){
					//just ignore
					arlequinDataBySection[genStructure].Delete(s0);//no longer needed
					delete pword;pword=0;
					break;
					}
				delete pword;pword=0;
				}//s0

			// "[Structure]"//NbGroups
			for(int s1=0;s1<arlequinDataBySection[genStructure].len();++s1){
				parameterLine = arlequinDataBySection[genStructure][s1];
				tword=parameterLine.ChopOffFirstWord(eq);
				tword->CullTrailingWhiteSpace();
				if(*tword == "NbGroups" || *tword == "nbgroups" || *tword == "NBGROUPS"){
					parameterLine.CullLeadingWhiteSpace();
					ngroups_ = parameterLine.toInt();
					arlequinDataBySection[genStructure].Delete(s1);//no longer needed
					delete tword;tword=0;
					break;
					}//if==NbGroups
				delete tword;tword=0;
				}//s1

			// "[Structure]"//IndividualLevel
			for(int s2=0;s2<arlequinDataBySection[genStructure].len();++s2){
				parameterLine = arlequinDataBySection[genStructure][s2];
				tword=parameterLine.ChopOffFirstWord(eq);
				tword->CullTrailingWhiteSpace();
				if(*tword == "IndividualLevel" || *tword == "individuallevel" || *tword == "INDIVIDUALLEVEL"){
					parameterLine.CullLeadingWhiteSpace();
					int ilevel = parameterLine.toInt();
					if(ilevel != 1){
						aString err("In [[Structure]], must set IndividualLevel=1 in order to read Arlequin data");
						aError(err);
						delete tword; tword=0;
						return 2;
						}
					arlequinDataBySection[genStructure].Delete(s2);//no longer needed
					delete tword;tword=0;
					break;
					}//if==IndividualLevel
				delete tword;tword=0;
				}//s2

			groupName_.Clear();
			groupHierarchy_.Clear();
			long sline=0;
			for(int gr=0;gr<ngroups_;++gr){
				aString gpnum(gr+1);
				gpnum.PrependCharToLen('0',Digits(ngroups_));
				gpnum.prepend("group");
				groupName_.Append(NEW aString(gpnum));
				groupHierarchy_.Append(NEW aList<int>());
				//read group membership intro lines
				parameterLine = arlequinDataBySection[genStructure][sline++];
				if(parameterLine[parameterLine.len()-1] != '{'){
						aString err("Expected \"Group={\" in [[Structure]]");
						aError(err);
						continue;}
					else{//read names in group
						bool thisGpDone = false;
						while (!thisGpDone){//LOOK AT EACH title_
							aString gpLine = arlequinDataBySection[genStructure][sline++];
							if(gpLine[0] == '}'){
								thisGpDone = true;
								continue;}//goes to end of while !thisGpDone
			
							gpLine.CullLeadingChar('\"');
							gpLine.CullTrailingChar('\"');
							int popnum = popName_.Loc(&gpLine);
							groupHierarchy_[gr].Append(popnum);
							}//end while !thisGpDone
						}//end else
				}//for gr
		}//end if readGroupInfo
		
	LocalityMembership();

	aList<int> codomLoci;
	for(int loc=0;loc<nlociCodom_;++loc){
		codomLoci.Append(loc);}//all loci are assumed codominant  - may adjust this later
//	ParseGenotypicData(codomLoci);
	ParseGenotypicData();
	CheckForNewAlleles();
	return 1;
	}//end CodominantMarkerData::ReadArlequinDataFile



void CodominantMarkerData::SnipArlequinComment(aString& arqStr){
	const char pound='#';
	int compos = arqStr.pos(pound);
	if(compos==0 && arqStr[0] != pound){//no comment found
		return;}
	if(arqStr.len()==1 && arqStr[0] == pound){
		arqStr.clear();return;
		}
	arqStr.trunc(compos-1);
	}//end CodominantMarkerData::SnipArlequinComment(aString&)





int CodominantMarkerData::ReadGelJockDataFile (aString* dataFileName){
	fstream dataFile;	
	char *dataLine=NEW char[1001];dataLine[0]='\0';
//	aString beast, popLine;
	aString endCharList(" \t,:\"{}[]!?\n\r");
	char* filename=dataFileName->raw();
	
//	char endChar = '\r';
	char endChar = endchr;//defined in StringClass.h
	dataFile.open(filename,ios::in);
	int dataFileOpen = dataFile.is_open();
	if(! dataFileOpen){
		aString errstr("couldn\'t open the file ");
		errstr.cat(filename);
		aError(errstr);
		return 0;}
//	dataFile.getline(dataLine, 1000);
	dataFile.getline(dataLine, 1000, endChar);
	title_ = dataLine;

	dataFile.getline(dataLine, 1000, endChar); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	npop_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 6);
	long nloci = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 7);
	int maxall_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	ngroups_ =  paramWord->toInt();
	delete paramWord; paramWord=0;

	if(endCharList.len()>11){//ALLELECHAR may be missing 
		paramWord = parameterLine.ChopOffFirstWord(endCharList);
		paramWord->cut(0, 11);
		alleleLen_ =  paramWord->toInt();
		delete paramWord; paramWord=0;
		}


	//get locus information
	traitName_.Clear();
	traitName_.Append(NEW aString("id"));
	int moreLoci = 1;//true
	int loc = 0;
	while (moreLoci){
		dataFile.getline(dataLine, 1000, endChar); 
		aString locusLine(dataLine);
		while (locusLine.len() > 0){
			++loc;
			aString *locName = locusLine.ChopOffFirstWord(endCharList);
			if(locName->len()>0){//there could be trailing blanks on locusLine
				aString *s = NEW aString(locName);
				traitName_.Append(s);//append a copy
				traitNum_.Append(loc);}
			delete locName; locName=0;
			}//end while locusLine.len()
		if (loc >= nloci){//data may be on more than one line
			moreLoci = 0;}//false
		}//end while moreLoci
	//read in the number of loci to be skipped in the analyses
	dataFile.getline(dataLine, 1000, endChar); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	int lociSkipped = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped loci, if any
	loc = 0;
	moreLoci = 0;//false
	if (lociSkipped > 0){
			moreLoci = 1;}//true
	StringList skippedLocusName;
	while (moreLoci){
		while (skipLine.len() > 0){
			++loc;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				aString *s = NEW aString(skipWord);
				skippedLocusName.Append(s);//append a copy
				int toSkip = traitName_.Loc(s);
				skippedTraitNum_.Append(toSkip);
				int item = traitNum_.Loc(toSkip);
				traitNum_.Delete(item);
				}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (loc >= lociSkipped){
				moreLoci = 0;}//false
			else	{//more loci on the next line
				dataFile.getline(dataLine, 1000); 
				skipLine = dataLine;}
		}//end while moreLoci
	skippedTraitNum_.Sort();
	traitNum_.Sort();

	nloci = traitNum_.len();

	//read in population data
	aString popLine;
	aString beast, beastTest;
	for (int pop=0;pop<npop_;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 1000, endChar); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		aString *s = NEW aString(popLine);
		popName_.Append(s);//append a copy

		//make a new sublist for the genotypes of this population
		rawTraits_.Append(NEW ObjList<TextDataList>());	

		// BEAST is the string for a particular individual
		int i=0;
		beast.clear(); beastTest.clear();
		while (beastTest != "NEXT"){ 
			dataFile.getline(dataLine, 1000, endChar); 
			beastTest = beast = dataLine;
			beastTest.CullLeadingWhiteSpace();
			beastTest.CullTrailingWhiteSpace();
			if (beastTest != "NEXT"){
					rawTraits_[pop].Append(NEW TextDataList("?"));//sets missingFlag=?
					aString s(beast);//work on a copy
					aString *idval=s.ChopOffFirstWord();
					rawTraits_[pop][i].Append(NEW TextDatum(*idval));
					delete idval;idval=0;
					//parse the genotype string
					for(int loc=0;loc<nloci;++loc){
						aString g(s,loc*alleleLen_*ploidy_,(loc+1)*alleleLen_*ploidy_);//check this
						bool allmissing=true;
						for(int j=0;j<alleleLen_*ploidy_;++j){
							if(g[j] != ' '){
								allmissing=false;break;}}
						if(allmissing){
								rawTraits_[pop][i].Append(NEW TextDatum());}
							else{
								rawTraits_[pop][i].Append(NEW TextDatum(g));}
						}//loc	
					++i;}
				else{
					popSize_.Append(rawTraits_[pop].len());}
			}//end while beast..else
		}//end for pop
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	GroupMembership();
	LocalityMembership();
	nlociCodom_=nloci;//by assumption -- modify this in other classes
	aList<int> codominantloci = traitNum_;
//	ParseGenotypicData(codominantloci);
	ParseGenotypicData();
	CheckForNewAlleles();
	return 1;
	}//end CodominantMarkerData::ReadGelJockDataFile




void CodominantMarkerData::RestoreLoci (aList<int>& lociToRestore){
	//This function restores the designated loci to the analysis.
	//It shifts trait numbers from from skippedTraitNums_ to the traitNums_ list
	//and keps track of the changes in the locusNumCodom_ and skippedLocusNumCodom_ lists
	if(lociToRestore.len()==0){return;}//none to cull
	ResetFlags();
	for(int tr=0;tr<lociToRestore.len();++tr){
		int rLoc = lociToRestore[tr];//make sure it's a valid locus to restore
		if(skippedTraitNum_.Includes(rLoc) && rLoc>=0 && rLoc<traitName_.len()){
			traitNum_.Append(rLoc);
			locusNumCodom_.Append(rLoc);
			long item = skippedTraitNum_.Loc(rLoc);
			skippedTraitNum_.Delete(item);
			item = skippedLocusNumCodom_.Loc(rLoc);
			skippedLocusNumCodom_.Delete(item);
			}//if item not skipped already
		}//tr
	traitNum_.Sort();
	locusNumCodom_.Sort();
	CheckForNewAlleles();
	}//end procedure CodominantMarkerData::RestoreLoci(aList<int>&)


void CodominantMarkerData::RestoreLoci (StringList& lociToRestore){
	//This function restores the designated loci to the analysis
	if(lociToRestore.len()==0){return;}//none to cull
	//convert names to numbers and cull
	aList<int> restoreListByNum;
	for(int tr=0;tr<lociToRestore.len();++tr){
		aString rTraitname = lociToRestore[tr];
		int item = traitName_.Loc(&rTraitname);
		if((item>=0)&&(!(traitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already included
			restoreListByNum.Append(item);
			}//if item exists
		}//tr
	RestoreLoci(restoreListByNum);
	}//end procedure CodominantMarkerData::RestoreLoci(StringList&)



void CodominantMarkerData::RestoreLocus (aString& locusToRestore){
	StringList lociToRestore;
	lociToRestore.Append(NEW aString(locusToRestore));
	RestoreLoci(lociToRestore);
	}//end CodominantMarkerData::RestoreLocus (aString& )


void CodominantMarkerData::RestoreLocus (char* locusToRestore){
	StringList lociToRestore;
	lociToRestore.Append(NEW aString(locusToRestore));
	RestoreLoci(lociToRestore);
	}//end CodominantMarkerData::RestoreLocus (char* )


void CodominantMarkerData::RestoreLocus (int locusToRestore){
	aList<int> lociToRestore;
	lociToRestore.Append(locusToRestore);
	RestoreLoci(lociToRestore);
	}//end CodominantMarkerData::RestoreLocus (int )


void CodominantMarkerData::SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
	//records all data, passed along from genetic analysis classes
	//empties old data and creates new data structures
	Clear();
	title_ = "Simulated data set";
	npop_ = pgcList.len();
	nlociCodom_ = pgcList[0].loci(MultiInheritanceModeContainer<int>::diploid);
	ngroups_ = 1;
	SetAlleleLength(1);//bug if maxall>9
	traitName_.Append(NEW aString("ID"));
	int locusCount=0;
	int pop,i, loc;
	traitNum_.Append(locusCount++);
	for(loc = 0;loc<nlociCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',Digits(nlociCodom_));
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(locusCount++);
		allCodomLoci_.Append(loc+1);
		locusNumCodom_.Append(loc+1);
		}
	ntraits_ = traitNum_.len();
	
	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("simulated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create dummy id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
//		rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());

		for (i=0;i<pgcList[pop].popSize();++i){
			rawTraits_[pop].Append(NEW TextDataList());
			rawTraits_[pop][i].Append(NEW TextDatum());//empty ID
//			for(loc=0;loc<nlociCodom_;++loc){
//				rawCodomGtypes_[pop][i].Append(NEW TextDataList());}
			}
		popSize_.Append(pgcList[pop].popSize());


		//create and append dummy genotypes
		for(loc=0;loc<nlociCodom_;++loc){
			for(i=0;i<pgcList[pop].popSize();++i){
				rawTraits_[pop][i].Append(NEW TextDatum("  "));
				}//end for i
			}//end for loc
		}//end for pop	

	//Fill up with data
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<popSize(pop);++i){
			SetGenome(pgcList[pop][i], pop, i);}
		}
	GroupMembership();
	LocalityMembership();
//	aList<int> lociToParse = allCodomLoci_;
//	ParseGenotypicData(lociToParse,0);
	ParseGenotypicData(0);
	CheckForNewAlleles();		
	}//CodominantMarkerData::SetGenome(ObjList<PopulationGeneContainer<int> >&)




void CodominantMarkerData::SetGenome(GeneContainerCore& gc, int pop, int indiv){
	//records data from MultiHaplotypeContainer<int>
	//assumes blank data records already created in EvolutionaryGeneticsData
	MultiHaplotypeContainer<int>* mhc = (MultiHaplotypeContainer<int>*) &gc;
	if(mhc->loci() != nlociCodom_){
		aError("ERROR: CodominantMarkerData::SetGenome(): size mismatch between data classes");
		return;}
	ResetFlags();
	rawTraits_[pop][indiv][0] = mhc->ID();
	int loc,c;
	if(mhc->missing()){//for whatever reason
		for(loc=0;loc<nlociCodom_;++loc){
			rawTraits_[pop][indiv][locusNumCodom_[loc]].clear();}
		return;}
	for(loc=0;loc<nlociCodom_;++loc){
		aString gvals;//initializes as missing
		for(c=0;c<ploidy();++c){
			if(mhc->missing(c)){
				gvals.AppendCharToLen(' ',alleleLen());}//missing value
			else{
				gvals.cat(mhc->haplotype(loc,c));}
			}	
		rawTraits_[pop][indiv][locusNumCodom_[loc]] = gvals;
		}
	}//CodominantMarkerData::SetGenome(GeneContainerCore&, int, int)



void CodominantMarkerData::SetLoci(aList<int>& lociToInclude){
	if(lociToInclude.len()==0) return;
	CodominantMarkerData::ResetGeneticFlags();
//	lociToInclude.Print(cout," ");
//	allCodomLoci_.Print(cout," ");
//	locusNumCodom_.Print(cout," ");
//	skippedLocusNumCodom_.Print(cout," ");
	int tr,item;//cLoc
	for(tr=0;tr<lociToInclude.len();++tr){
		if(! allCodomLoci_.Includes(lociToInclude[tr])){
			allCodomLoci_.Append(lociToInclude[tr]);
			locusNumCodom_.Append(lociToInclude[tr]);
			}}
	allCodomLoci_.Sort();
	locusNumCodom_.Sort();
	skippedLocusNumCodom_.Clear();
//	for(tr=0;tr<lociToInclude.len();++tr){
//		cLoc = lociToInclude[tr];
//		if(skippedTraitNum_.Includes(cLoc)){
//			item = locusNumCodom_.Loc(cLoc);
//			if(item>=0){
//				locusNumCodom_.Delete(item);}
//			if(! skippedLocusNumCodom_.Includes(cLoc)){
//				skippedLocusNumCodom_.Append(cLoc);}
//			}
//		}//tr
	for(tr=0;tr<allCodomLoci_.len();++tr){
		if(skippedTraitNum_.Includes(allCodomLoci_[tr])){
			skippedLocusNumCodom_.Append(allCodomLoci_[tr]);
			item = locusNumCodom_.Loc(allCodomLoci_[tr]);
			if(item>=0){
				locusNumCodom_.Delete(item);}
			}//skipped
		}//tr
	skippedLocusNumCodom_.Sort();
	cout<<endl;
	nlociCodom_ = locusNumCodom_.len();
//	lociToInclude.Print(cout," ");
//	allCodomLoci_.Print(cout," ");
//	locusNumCodom_.Print(cout," ");
//	skippedLocusNumCodom_.Print(cout," ");

	CodominantMarkerData::ParseGenotypicData(0);
//	CodominantMarkerData::ParseGenotypicData(allCodomLoci_,0);
	CodominantMarkerData::CheckForNewAlleles();
	}//CodominantMarkerData::SetLoci(aList<int>& )



void CodominantMarkerData::UpdateVariableLoci(void){
	if (varLociCurrent_){return;}
	CheckForNewAlleles();
	variableLocusNumCodom_.Clear();
	int loc;
	for(loc=0;loc<nlociCodom_;++loc){
		StringList& locList = allelesPresent_[loc];
		if (locList.len()>1){
//			variableLocusNumCodom_.Append(locusNumCodom_[loc]);}}
			variableLocusNumCodom_.Append(loc);}}
	varLociCurrent_=true;
	}//end CodominantMarkerData::UpdateVariableLoci




#endif






