// CytoplasmicFemaleMarkerData.cpp

// Holds data for markers that are transmitted through males only (usually Y chromosomes)

#ifndef CytoplasmicFemaleMarkerDatacpp
#define CytoplasmicFemaleMarkerDatacpp = 1

#include "CytoplasmicFemaleMarkerData.h"
#include "Element.cpp"
#include "aList.cpp"
#include "SortList.cpp"
#include "GeneContainers.cpp"





CytoplasmicFemaleMarkerData::CytoplasmicFemaleMarkerData(CytoplasmicFemaleMarkerData& parentData):
			SexLinkedData(parentData),
			nlociCYfemale_(parentData.nlociCYfemale_),
			ploidy_(parentData.ploidy_),
			alleleLen_(parentData.alleleLen_),
			allCYfemaleLoci_(parentData.allCYfemaleLoci_),
			locusNumCYfemale_(parentData.locusNumCYfemale_),
			skippedLocusNumCYfemale_(parentData.skippedLocusNumCYfemale_),
			allPresentInFemalesOnly_(parentData.allPresentInFemalesOnly_),
			presentInFemalesOnly_(parentData.presentInFemalesOnly_),
			allPresentInBothSexes_(parentData.allPresentInBothSexes_),
			presentInBothSexes_(parentData.presentInBothSexes_),
			variableLocusNumCYfemale_(parentData.variableLocusNumCYfemale_), 
			allelesPresentCYfemale_(parentData.allelesPresentCYfemale_),
			genotypeFreqCYfemale_(parentData.genotypeFreqCYfemale_),
			genotypeFreqCYfemaleGlobal_(parentData.genotypeFreqCYfemaleGlobal_),
			nByLocusCYfemale_(parentData.nByLocusCYfemale_),
			allelesCurrentCYfemale_(parentData.allelesCurrentCYfemale_),
			varLociCurrentCYfemale_(parentData.varLociCurrentCYfemale_),
			genotypeFreqsCurrentCYfemale_(parentData.genotypeFreqsCurrentCYfemale_)
			{	
		}//end ctor CytoplasmicFemaleMarkerData(CytoplasmicFemaleMarkerData&)


CytoplasmicFemaleMarkerData::CytoplasmicFemaleMarkerData(CytoplasmicFemaleMarkerData& parentData, 
						aList<int>& popSubsetNums):
			nlociCYfemale_(parentData.nlociCYfemale_),
			ploidy_(parentData.ploidy_),
			alleleLen_(parentData.alleleLen_),
			allCYfemaleLoci_(parentData.allCYfemaleLoci_),
			locusNumCYfemale_(parentData.locusNumCYfemale_),
			skippedLocusNumCYfemale_(parentData.skippedLocusNumCYfemale_),
			allPresentInFemalesOnly_(parentData.allPresentInFemalesOnly_),
			presentInFemalesOnly_(parentData.presentInFemalesOnly_),
			allPresentInBothSexes_(parentData.allPresentInBothSexes_),
			presentInBothSexes_(parentData.presentInBothSexes_)
			{
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	Initialize(parentData,popSubsetNums);
	title_.prepend("subset of dataset: ");
	ResetFlags();
	SortBySex();
	CheckForNewAlleles();
	UpdateVariableLociCYfemale();
	}//end ctor CytoplasmicFemaleMarkerData(CytoplasmicFemaleMarkerData&, aList<int>&)


void  CytoplasmicFemaleMarkerData::Initialize(void){
	CytoplasmicFemaleMarkerData::Clear();
	}//Initialize()
	
	
void  CytoplasmicFemaleMarkerData::Initialize(aString& s){
	CytoplasmicFemaleMarkerData::Clear();
	title_=s; 
	}//Initialize(aString&)
	
	
void  CytoplasmicFemaleMarkerData::Initialize(CytoplasmicFemaleMarkerData& cymd){
	SexLinkedData::Initialize(cymd);
	nlociCYfemale_=cymd.nlociCYfemale_;
	ploidy_=cymd.ploidy_;
	alleleLen_=cymd.alleleLen_;
	allCYfemaleLoci_=cymd.allCYfemaleLoci_;
	locusNumCYfemale_=cymd.locusNumCYfemale_; 
	skippedLocusNumCYfemale_=cymd.skippedLocusNumCYfemale_;
	variableLocusNumCYfemale_=cymd.variableLocusNumCYfemale_;
	allPresentInFemalesOnly_=cymd.allPresentInFemalesOnly_;
	presentInFemalesOnly_=cymd.presentInFemalesOnly_;
	allelesPresentCYfemale_=cymd.allelesPresentCYfemale_;
	allPresentInBothSexes_=cymd.allPresentInBothSexes_;
	presentInBothSexes_=cymd.presentInBothSexes_;
	genotypeFreqCYfemale_=cymd.genotypeFreqCYfemale_;
	genotypeFreqCYfemaleGlobal_=cymd.genotypeFreqCYfemaleGlobal_;
	nByLocusCYfemale_=cymd.nByLocusCYfemale_;
	allelesCurrentCYfemale_=cymd.allelesCurrentCYfemale_;
	varLociCurrentCYfemale_=cymd.varLociCurrentCYfemale_;
	genotypeFreqsCurrentCYfemale_=cymd.genotypeFreqsCurrentCYfemale_;
	}//Initialize(CytoplasmicFemaleMarkerData&)
	

void  CytoplasmicFemaleMarkerData::Initialize(CytoplasmicFemaleMarkerData& parentData, aList<int>& popSubsetNums){
	SexLinkedData::Initialize(parentData,popSubsetNums);
	nlociCYfemale_=parentData.nlociCYfemale_;
	ploidy_=parentData.ploidy_;
	alleleLen_=parentData.alleleLen_;
	allCYfemaleLoci_=parentData.allCYfemaleLoci_;
	locusNumCYfemale_=parentData.locusNumCYfemale_;
	skippedLocusNumCYfemale_=parentData.skippedLocusNumCYfemale_;
	allPresentInFemalesOnly_=parentData.allPresentInFemalesOnly_;
	presentInFemalesOnly_=parentData.presentInFemalesOnly_;
	allPresentInBothSexes_=parentData.allPresentInBothSexes_;
	presentInBothSexes_=parentData.presentInBothSexes_;
	SortBySex();
	CheckForNewAlleles();
	UpdateVariableLociCYfemale();
	}//end Initialize(CytoplasmicFemaleMarkerData&, aList<int>&)


	
CytoplasmicFemaleMarkerData&  CytoplasmicFemaleMarkerData::operator=(CytoplasmicFemaleMarkerData& cymd){
	Initialize(cymd);
	return *this;
	}//operator=(CytoplasmicFemaleMarkerData&)




void CytoplasmicFemaleMarkerData::CalculateGenotypeFreqs (void){
	//This routine counts marker frequencies, taking into account
	//markers left blank in the data set.
	if (genotypeFreqsCurrentCYfemale_){return;}
	SortBySex();
	CheckForNewAlleles();
	int pop, i, loc, marker;
	long double freq;
//	char blank = ' ', tab='\t';	//don't count these
	aString missingDataFlags = ".?-*";//characters used to signify missing data
	aString endCharList = " \t,";//characters used to separate data fields
	genotypeFreqCYfemale_.Clear();
	nByLocusCYfemale_.Clear();
	for (pop=0;pop< npop_;++pop){ 
		//set up a frequency list for this population
		genotypeFreqCYfemale_.Append( NEW ObjList<aList<long double> >());
		nByLocusCYfemale_.Append(NEW aList<int>());
		ObjList<TextDataList>& genotypesList = rawTraits_[pop];
		aList<int> *femalesPtr=0;
		if(heterogameticSex()==F){
				femalesPtr = &heterogameticIndivs_[pop];}
			else{
				femalesPtr = &homogameticIndivs_[pop];}
		aList<int>& females = *femalesPtr;
		for (loc=0;loc<nlociCYfemale_;++loc){//loops through dominant loci
			//first set up a blank array of frequencies for each possible phenotype
			genotypeFreqCYfemale_[pop].Append(NEW aList<long double>());
			StringList& markerNamesList = allelesPresentCYfemale_[loc];
			freq = zero;
			for (marker=0;marker<markerNamesList.len();++marker){//initialize
				genotypeFreqCYfemale_[pop][loc].Append(freq);
				}

			//next look at the genotype of each individual and tally the frequency
			long nSampled=0;	//the number of individuals
//			if(presentInFemalesOnly_[loc]){
//					for (i=0;i< females.len();++i){//quicker
//						TextDatum& gene = genotypesList[females[i]][locusNumCYfemale_[loc]];
//						if (gene.OK()) {//if not, it's missing data
//							//increment the frequency & sample size -- divide later
//							marker = markerNamesList.Loc(&gene);
//							genotypeFreqCYfemale_[pop][loc][marker]++;
//							nSampled++;
//							}//end if legal data
//						}//end for i
//					}
//				else{
					for (i=0;i<individuals(pop);++i){
						TextDatum& gene = genotypesList[i][locusNumCYfemale_[loc]];
						if (gene.OK()) {//if not, it's missing data
							//increment the frequency & sample size -- divide later
							marker = markerNamesList.Loc(&gene);
							genotypeFreqCYfemale_[pop][loc][marker]++;
							nSampled++;
							}//end if legal data
						}//end for i
//					}

			//now divide by the sample size to restore the frequencies
			if(nSampled>0){
				for (marker=0;marker<markerNamesList.len();++marker){
					genotypeFreqCYfemale_[pop][loc][marker] /= (long double) nSampled;
					}}
			//store the sample size for this locus
			nByLocusCYfemale_[pop].Append((int)nSampled);
			}//end for loc
		}//end for pop
	genotypeFreqCYfemaleGlobal_.Clear();
	for(loc=0;loc<nlociCYfemale_;++loc){
		genotypeFreqCYfemaleGlobal_.Append(NEW aList<long double>());
		for(marker=0;marker<allelesPresentCYfemale_[loc].len();++marker){
			freq=zero;
			for(pop=0;pop<npop_;++pop){
				freq += genotypeFreqCYfemale_[pop][loc][marker];}
			genotypeFreqCYfemaleGlobal_[loc].Append(freq/(long double)npop_);
			}}
	genotypeFreqsCurrentCYfemale_=1;
	}//end CytoplasmicFemaleMarkerData::CalculateGenotypeFreqs



ObjList<aList<long double> >* CytoplasmicFemaleMarkerData::CalculateGenotypeFreqsByPopSet(aList<int>& popSet, 
										aList<long double>& nList, int locusTypesToInclude){
 	//returns list of loci, each with list of marker freqs, using total marker set
 	CalculateGenotypeFreqs();
 	nList.Clear();
 	ObjList<aList<long double> >* popSetFreqsPtr = NEW ObjList<aList<long double> >();//copies structure
 	ObjList<aList<long double> >& popSetFreqs = *popSetFreqsPtr;//notational convenience
	int loc,markers,pop,popSetPop,m,item;
	long double nCopies, nSample, samSizeSet;
	ObjList<aList<long double> > popSetFreqsBothSexes, popSetFreqsFonly;
	aList<long double> nListB, nListF;
	if(locusTypesToInclude != presentFonly){
	 	for(item=0;item<presentInBothSexes_.len();++item){
	 		loc = presentInBothSexes_[item];
//	 		loc = locusNumCYfemale_.Loc(presentInBothSexes_[item]);
	 		popSetFreqsBothSexes.Append(NEW aList<long double>());
	 		markers = GetNumAllelesCYfemale(item);
//	 		markers = GetNumAllelesCYfemale(loc);
	 		samSizeSet = zero;
	 		for(m=0;m<markers;++m){
	 			nCopies = zero;
				popSetFreqsBothSexes[item].Append(zero);
	 			for(popSetPop=0;popSetPop<popSet.len();++popSetPop){
	 				pop = popSet[popSetPop];
	 				nSample = (long double) (nByLocusCYfemale_[pop][item]);
	 				if(m==0){samSizeSet += nSample;}
	 				nCopies += nSample * GetFreqCYfemale(pop,item,m);
	 				}
	 			if(m==0){nListB.Append(samSizeSet);}
	 			if(nCopies==samSizeSet){
	 					popSetFreqsBothSexes[item][m]=one;}
	 				else{
	 					popSetFreqsBothSexes[item][m] = nCopies / samSizeSet;
	 					}
	 			}//end for m
	 		}//end for item
	 	
	 	}// !presentFonly
	if(locusTypesToInclude != presentBothSexes){
	 	for(item=0;item<presentInFemalesOnly_.len();++item){
	 		loc = presentInFemalesOnly_[item];
	 		int fOnlyItem = item + presentInBothSexes_.len();
//	 		loc = locusNumCYfemale_.Loc(presentInFemalesOnly_[item]);
	 		popSetFreqsFonly.Append(NEW aList<long double>());
	 		markers = GetNumAllelesCYfemale(fOnlyItem);
//	 		markers = GetNumAllelesCYfemale(loc);
	 		samSizeSet = zero;
	 		for(m=0;m<markers;++m){
	 			nCopies = zero;
				popSetFreqsFonly[item].Append(zero);
	 			for(popSetPop=0;popSetPop<popSet.len();++popSetPop){
	 				pop = popSet[popSetPop];
	 				nSample = (long double) (nByLocusCYfemale_[pop][fOnlyItem]);
	 				if(m==0){samSizeSet += nSample;}
	 				nCopies += nSample * GetFreqCYfemale(pop,fOnlyItem,m);
	 				}
	 			if(m==0){nListF.Append(samSizeSet);}
	 			if(nCopies==samSizeSet){
	 					popSetFreqsFonly[item][m]=one;}
	 				else{
	 					popSetFreqsFonly[item][m] = nCopies / samSizeSet;
	 					}
	 			}//end for m
	 		}//end for item
	 	}// !presentBothSexes
	//sort these into the right spots
	if(locusTypesToInclude == presentBothSexes){
			nList = nListB;
			popSetFreqs = popSetFreqsBothSexes;}
		else if (locusTypesToInclude == presentFonly){
			nList = nListF;
			popSetFreqs = popSetFreqsFonly;}
		else{//sort into right order
			for(loc=0;loc<nlociCYfemale_;++loc){
				item = presentInBothSexes_.Loc(loc);
				if(item>=0){
						nList.Append(nListB[item]);
						popSetFreqs.Append(NEW aList<long double>(popSetFreqsBothSexes[item]));
						}
					else{
						item = presentInFemalesOnly_.Loc(loc);
						if(item>=0){
							nList.Append(nListF[item]);
							popSetFreqs.Append(NEW aList<long double>(popSetFreqsFonly[item]));
							}
						}
				}//loc
			}//sort
 	return popSetFreqsPtr;
	}//end CytoplasmicFemaleMarkerData::CalculateGenotypeFreqsByPopSet



void CytoplasmicFemaleMarkerData::CheckForNewAlleles (void){
	if (allelesCurrentCYfemale_){return;}
	allelesPresentCYfemale_.Clear();
	int loc,pop,i;
	for (loc=0;loc<nlociCYfemale_;++loc){//initialize
		allelesPresentCYfemale_.Append(NEW StringList());}
	for (pop=0;pop<npop_;++pop){
		ObjList<TextDataList>& ptypeList = rawTraits_[pop];
		for(i=0;i<individuals(pop);++i){
			for (loc=0;loc<nlociCYfemale_;++loc){
				TextDatum& gene = ptypeList[i][locusNumCYfemale_[loc]];
//				TextDatum& gene = ptypeList[i][loc];
				if(gene.OK()){//ignore missing data
					StringList& alleleList = allelesPresentCYfemale_[loc];
					if(!(alleleList.Includes(&gene))){
						alleleList.Append(NEW aString(gene));}
					}//OK
				}//end for loc
			}//end for i
		}//end for pop
	for (loc=0;loc<nlociCYfemale_;++loc){
		allelesPresentCYfemale_[loc].Sort();}
	allelesCurrentCYfemale_=true;
	CytoplasmicFemaleMarkerData::UpdateVariableLociCYfemale();
	}//end CytoplasmicFemaleMarkerData::CheckForNewAlleles



void CytoplasmicFemaleMarkerData::Clear(void){
	EvolutionaryGeneticsData::Clear();
	allCYfemaleLoci_.Clear();
	locusNumCYfemale_.Clear();
	skippedLocusNumCYfemale_.Clear();
	variableLocusNumCYfemale_.Clear();
	allPresentInFemalesOnly_.Clear();
	presentInFemalesOnly_.Clear();
	allPresentInBothSexes_.Clear();
	presentInBothSexes_.Clear();
	allelesPresentCYfemale_.Clear();
	genotypeFreqCYfemale_.Clear();
	genotypeFreqCYfemaleGlobal_.Clear();
	nByLocusCYfemale_.Clear();
	nlociCYfemale_=allelesCurrentCYfemale_=varLociCurrentCYfemale_=genotypeFreqsCurrentCYfemale_=0;
	}//end CytoplasmicFemaleMarkerData::Clear()





void  CytoplasmicFemaleMarkerData::CreateBootstrapData(CytoplasmicFemaleMarkerData& origData, Ran3& ran){
	EvolutionaryGeneticsData::CreateBootstrapData(origData,ran);
	SortBySex();
	CheckForNewAlleles();	
	}//end  CytoplasmicFemaleMarkerData::CreateBootstrapData(EvolutionaryGeneticsData&)



void  CytoplasmicFemaleMarkerData::CreateBootstrapDataConstPop(CytoplasmicFemaleMarkerData& origData, Ran3& ran){
	EvolutionaryGeneticsData::CreateBootstrapDataConstPop(origData,ran);
	SortBySex();
	CheckForNewAlleles();	
	}//end  CytoplasmicFemaleMarkerData::CreateBootstrapDataConstPop(EvolutionaryGeneticsData&)




void CytoplasmicFemaleMarkerData::CreateRandomData(
							const long double Fst, 
							const long double /*Fis*/,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran,
							aList<int>& femaleOnlyLoci){
	char hetSex = heterogameticSex_;
	long double sr = primarySexRatio_;
	Clear();
	heterogameticSex_ = hetSex;//restore these
	primarySexRatio_ = sr;
	femaleOnlyLoci.Sort();
	int pop,loc,i;
	allPresentInFemalesOnly_ = presentInFemalesOnly_ = femaleOnlyLoci;
	for(loc=0;loc<numLociSampled;++loc){
		if(! allPresentInFemalesOnly_.Includes(loc)){
			allPresentInBothSexes_.Append(loc);
			presentInBothSexes_.Append(loc);
			}}
			
	title_ = "Randomly generated cytoplasmic female-inherited marker data set with parametric Fst=";
	title_.cat(Fst);  
	title_.cat(" and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numPopsSampled;
	nlociCYfemale_ = numLociSampled;
	ngroups_ = 1;
	traitName_.Append(NEW aString("ID"));
	int locusCount=0;
	traitNum_.Append(locusCount++);
	if(allPresentInFemalesOnly_.len()>0){
		traitName_.Append(NEW aString("sex"));
		traitNum_.Append(locusCount++);
		sexTrait_=1;
		}
	for(loc = 0;loc<nlociCYfemale_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',Digits(nlociCYfemale_));
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(locusCount++);
		allCYfemaleLoci_.Append(loc+1);
		locusNumCYfemale_.Append(loc+1);
		}
	ntraits_ = traitNum_.len();

	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
	GeneFrequencyDistribution parametricFreqDistrib(Fst, pBar, 1);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	aString genotype;
	long double pf, pp;
	ObjList<StatList<long double> > locusFrequenciesByPop;
	for(pop=0;pop<npop_;++pop){
		StatList<long double>* locusFrequencies = NEW StatList<long double>();
		locusFrequenciesByPop.Append(locusFrequencies);
		for(loc=0;loc<nlociCYfemale_;++loc){
			pf = parametricFreqDistrib.RandomFrequency(ran);
			locusFrequencies->Append(pf);
			}//end for loc
		}//end for pop

	aString *s, indiv, pName;
	aList<int> female1male0;
	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		pName ="randomly generated population # ";
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());
		female1male0.Clear();	
		for (i=0;i<numIndivsPerPop;++i){
			rawTraits_[pop].Append(NEW TextDataList());
			indiv = i+1;
			indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));
			if(presentInFemalesOnly_.len()>0){
				female1male0.Append(1);
				rawTraits_[pop][i].Append(NEW TextDatum('F'));
				if(ran()>=sr){
					female1male0[i]=0;
					rawTraits_[pop][i].Last()="M";}}		
			}//end for i
		popSize_.Append(numIndivsPerPop);

		//create and append genotypes
		for(loc=0;loc<nlociCYfemale_;++loc){
			StatList<long double>& locusFrequencies = locusFrequenciesByPop[pop];
			pp = locusFrequencies[loc];
			for(i=0;i<numIndivsPerPop;++i){
				if(presentInFemalesOnly_.Includes(loc) && female1male0[i]==0){
						rawTraits_[pop][i].Append(NEW TextDatum());}//missing: placeholder
					else{
						if (ran()<pp){genotype = "A";}
							else {genotype = "B";}				
						rawTraits_[pop][i].Append(NEW TextDatum(genotype));
						}
				}//end for i
			}//end for loc
		}//end for pop	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end CytoplasmicFemaleMarkerData::CreateRandomData(...)




void CytoplasmicFemaleMarkerData::CreateRandomData(
							const long double Fst, 
							const long double /*Fis*/,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran,
							aList<int>& femaleOnlyLoci){
	//multiple alleles per locus, variable sample sizes per pop
	//	first set up initial variables
	char hetSex = heterogameticSex_;
	long double sr = primarySexRatio_;
	Clear();
	heterogameticSex_ = hetSex;//restore these
	primarySexRatio_ = sr;
	femaleOnlyLoci.Sort();
	int pop,loc,i;
	allPresentInFemalesOnly_ = presentInFemalesOnly_ = femaleOnlyLoci;
	nlociCYfemale_ = pBar.len();
	for(loc=0;loc<nlociCYfemale_;++loc){
		if(! allPresentInFemalesOnly_.Includes(loc)){
			allPresentInBothSexes_.Append(loc);
			presentInBothSexes_.Append(loc);
			}}
	title_ = "Randomly generated cytoplasmic male marker data set with parametric Fst=";
	title_.cat(Fst); 
	npop_ = numIndivsPerPopLoc.len();
	ngroups_ = 1;
	traitName_.Append(NEW aString("ID"));
	int locusCount=0;
	traitNum_.Append(locusCount++);
	for(loc = 0;loc<nlociCYfemale_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',Digits(nlociCYfemale_));
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(locusCount++);
		allCYfemaleLoci_.Append(loc+1);
		locusNumCYfemale_.Append(loc+1);
		}
	ntraits_ = traitNum_.len();


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();

	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	ObjList<ObjList<StatList<long double> > > frequenciesByPopLocusAllele;
	for(pop=0;pop<npop_;++pop){
		frequenciesByPopLocusAllele.Append(NEW ObjList<StatList<long double> >());}
	for(loc=0;loc<nlociCYfemale_;++loc){
		if(pBar[loc].len()>1){//locus is variable
				GeneFrequencyDistMultiAllele parametricFreqDistrib(Fst, pBar[loc], 1);//new version sends ploidy
				for(pop=0;pop<npop_;++pop){
					aList<long double>* pp = parametricFreqDistrib.RandomFrequencies(ran);
					frequenciesByPopLocusAllele[pop].Append(NEW StatList<long double>(*pp));//copy as StatList
					delete pp;pp=NULL;
					}//end for pop
				}
			else{//locus is monomorphic
				for(pop=0;pop<npop_;++pop){
					frequenciesByPopLocusAllele[pop].Append(NEW StatList<long double>(pBar[loc]));//copy as StatList
					}//end for pop
				}				
		}//end for loc

	int maxIndivPop;
	aString *s, indiv, pName;
	char genotype;
	long gtypeVal;
	aList<int> female1male0;
	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		pName = "randomly generated population # ";
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		s = NEW aString(pName);
		popName_.Append(s);//append a copy

		maxIndivPop = 0;
		for (loc=0;loc<numIndivsPerPopLoc[pop].len();++loc){
			maxIndivPop = MAX(maxIndivPop,numIndivsPerPopLoc[pop][loc]);}

		//create id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
		for (i=0;i<maxIndivPop;++i){
			rawTraits_[pop].Append(NEW TextDataList());
			aString indiv(i+1);//first create id numbers for each individual
			indiv.PrependCharToLen(' ', MAX(4,Digits(maxIndivPop)));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));
			female1male0.Append(1);
			if(presentInFemalesOnly_.len()>0){
				rawTraits_[pop][i].Append(NEW TextDatum('F'));
				if(ran()>=sr){
					female1male0[i]=0;
					rawTraits_[pop][i].Last()='M';}}		
			}//end for i
		popSize_.Append(maxIndivPop);

		for(loc=0;loc<nlociCYfemale_;++loc){
			StatList<long double>& pp = frequenciesByPopLocusAllele[pop][loc];
			if(pp.len()>1){//sampling required
					for(i=0;i<numIndivsPerPopLoc[pop][loc];++i){
						if(presentInFemalesOnly_.Includes(loc) && female1male0[i]==0){
								rawTraits_[pop][i].Append(NEW TextDatum());}
							else{
								gtypeVal = ran.weighted(pp);
								genotype = char('A' + (int)gtypeVal);
								rawTraits_[pop][i].Append(NEW TextDatum(genotype));
								}
						}//end for i
					for(i=numIndivsPerPopLoc[pop][loc];i<maxIndivPop;++i){//unsampled individuals blank
						rawTraits_[pop][i].Append(NEW TextDatum());//adds missing data
						}//end for i
					}
				else{//monomorphic locus
					for(i=0;i<numIndivsPerPopLoc[pop][loc];++i){
						if(presentInFemalesOnly_.Includes(loc) && female1male0[i]==0){
								rawTraits_[pop][i].Append(NEW TextDatum());}//missing
							else{
								rawTraits_[pop][i].Append(NEW TextDatum("A"));}
						}//end for i
					for(i=numIndivsPerPopLoc[pop][loc];i<maxIndivPop;++i){//unsampled individuals blank
						rawTraits_[pop][i].Append(NEW TextDatum());//add missing data
						}//end for i
					}					
			}//end for loc
		}//end for pop	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end CytoplasmicFemaleMarkerData::CreateRandomData(...)




void CytoplasmicFemaleMarkerData::CullCYfemaleLoci (StringList& lociToCull){
	//This function culls the designated loci from each dominant genotype data string
	//and includes them among the skipped loci
	if(lociToCull.len()==0){return;}//none to cull
	aList<int> cullListByNum;
	int tr, item;
	for(tr=0;tr<lociToCull.len();++tr){
		aString cTraitname = lociToCull[tr];
		item = traitName_.Loc(&cTraitname);
		if((item>=0)&&(!(skippedTraitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already skipped
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	CullCYfemaleLoci(cullListByNum);
	}//end procedure CytoplasmicFemaleMarkerData::CullCYfemaleLoci(StringList&)


void CytoplasmicFemaleMarkerData::CullCYfemaleLoci (aList<int>& lociToCull){
	//This function shifts trait numbers from from traitNums_ to the skippedTraitNums_ list
	if(lociToCull.len()==0){return;}//none to cull
	int tr,cLoc;
	long item;
	for(tr=0;tr<lociToCull.len();++tr){
		cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(traitNum_.Includes(cLoc) && cLoc>=0 && cLoc<traitName_.len()){
			item = traitNum_.Loc(cLoc);
			skippedTraitNum_.Append(cLoc);
			skippedLocusNumCYfemale_.Append(cLoc);
			traitNum_.Delete(item);
			item = locusNumCYfemale_.Loc(cLoc);
			locusNumCYfemale_.Delete(item);
			if(presentInFemalesOnly_.Includes(cLoc)){
				item = presentInFemalesOnly_.Loc(cLoc);
				presentInFemalesOnly_.Delete(item);}
			if(presentInBothSexes_.Includes(cLoc)){
				item = presentInBothSexes_.Loc(cLoc);
				presentInBothSexes_.Delete(item);}
			}//if item not skipped already
		}//tr
	nlociCYfemale_ = locusNumCYfemale_.len();
	skippedTraitNum_.Sort();
	skippedLocusNumCYfemale_.Sort();
	ResetFlags();
	CheckForNewAlleles();
	}//CytoplasmicFemaleMarkerData::CullCYfemaleLoci (aList<int>&)



void CytoplasmicFemaleMarkerData::CullCYfemaleLocus (char* locusToCull){
	aString locToCull(locusToCull);
	CullCYfemaleLocus(locToCull);
	}//end CytoplasmicFemaleMarkerData::CullCYfemaleLocus (char* )


void CytoplasmicFemaleMarkerData::CullCYfemaleLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	CullCYfemaleLoci(lociToCull);
	}//end CytoplasmicFemaleMarkerData::CullCYfemaleLocus (aString& )


void CytoplasmicFemaleMarkerData::CullMonomorphicLoci (void){	
	//This function culls the monomorphic loci from each genotype data string
	//and includes them among the skipped loci
	UpdateVariableLociCYfemale();
	if(variableLocusNumCYfemale_.len()==locusNumCYfemale_.len()){return;}//none monomorphic
	aList<int> monomorphicLoci(locusNumCYfemale_);
	for(int vLoc=0; vLoc<variableLocusNumCYfemale_.len();++vLoc){
		if(monomorphicLoci.Includes(variableLocusNumCYfemale_[vLoc])){//polymorphic
			int item = monomorphicLoci.Loc(variableLocusNumCYfemale_[vLoc]);
			monomorphicLoci.Delete(item);}
		}//vLoc
	CullCYfemaleLoci(monomorphicLoci);
	}//end procedure CytoplasmicFemaleMarkerData::CullMonomorphicLoci


void CytoplasmicFemaleMarkerData::CullToOneSex (CytoplasmicFemaleMarkerData& parentData,genderSet sexVal){
	//makes a subset data structure from the specified group
	aList<int> sexValList;
	sexValList.Append((int)sexVal);
	CullToTheseSexes(parentData,sexValList);
	}//end CytoplasmicFemaleMarkerData::CullToOneSex (CytoplasmicFemaleMarkerData& , genderSet )


void CytoplasmicFemaleMarkerData::CullToTheseSexes (CytoplasmicFemaleMarkerData& parentData,aList<int>& sexValList){
	//makes a subset data structure from the specified group
	Clear();
	if(sexValList.len()==0) return;
	title_ = parentData.title_;
	title_.cat(" -- ");
	for(int s=0;s<sexValList.len();++s){
		genderSet sexVal = (genderSet) sexValList[s];
		switch(sexVal){
			case A:
				title_.cat("asexuals"); break;
			case B:
				title_.cat("hermaphrodites"); break;
			case F:
				title_.cat("females"); break;
			case M:
				title_.cat("males"); break;
			default: break;
			}
		if(s<sexValList.len()-2){
				title_.cat(", ");}
			else if (s == sexValList.len()-2){
				title_.cat(" & ");}
			else{}
		}
	title_.cat(" only");


	npop_ = parentData.npop_;
	ngroups_ = parentData.ngroups_;
	ntraits_ = parentData.ntraits_;
	groupName_ = parentData.groupName_;
	localityName_ = parentData.localityName_;
	traitName_ = parentData.traitName_;
	traitNum_ = parentData.traitNum_;
	skippedTraitNum_ = parentData.skippedTraitNum_;
	missing_ = parentData.missing_;
	popName_ = parentData.popName_;
	localityCoordinates_ = parentData.localityCoordinates_;
	missing_ = parentData.missing_;
	
	heterogameticSex_ = parentData.heterogameticSex_;
	sexTrait_ = parentData.sexTrait_;
	primarySexRatio_ = parentData.primarySexRatio_;
	
	nlociCYfemale_ = parentData.nlociCYfemale_;
	locusNumCYfemale_ = parentData.locusNumCYfemale_;
	allCYfemaleLoci_ = parentData.allCYfemaleLoci_;
	skippedLocusNumCYfemale_ = parentData.skippedLocusNumCYfemale_;
	if(!sexValList.Includes ((int)F)){
		allPresentInFemalesOnly_ = parentData.allPresentInFemalesOnly_;
		presentInFemalesOnly_ = parentData.presentInFemalesOnly_;
		allPresentInBothSexes_ = parentData.allPresentInBothSexes_;
		presentInBothSexes_ = parentData.presentInBothSexes_;
		}

	genderSet thisSexVal=B;
	int pop, i, thisSexValInt;
	for(pop=0;pop<parentData.npop_;++pop){
		popSize_.Append(0);
		rawTraits_.Append(NEW ObjList<TextDataList>());
		for(i=0;i<parentData.popSize(pop);++i){
			TextDatum& thisSex = parentData.rawTraits_[pop][i][sexTrait_];
			thisSexVal = genderCode(thisSex);
			thisSexValInt = (int)thisSexVal;
			if(sexValList.Includes(thisSexValInt)){
				rawTraits_[pop].Append((TextDataList*)parentData.rawTraits_[pop][i].Duplicate());
				popSize_[pop]++;}
			}//i
		}//pop
	
	//cull empty populations from data
	for(pop=npop_-1;pop>=0;--pop){
		if(parentData.rawTraits_[pop].len()==0){
			parentData.rawTraits_.Delete(pop);
			popName_.Delete(pop);
			popSize_.Delete(pop);
			}
		}//pop
	npop_ = rawTraits_.len();
		
	ResetFlags();
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();
	}//end CytoplasmicFemaleMarkerData::CullToTheseSexes (CytoplasmicFemaleMarkerData& ,aList<int>& )


void CytoplasmicFemaleMarkerData::ExpungeLoci (StringList& lociToCull){
	if(lociToCull.len()==0){return;}//none to cull
	aList<int> cullListByNum;
	int tr, item;
	for(tr=0;tr<lociToCull.len();++tr){
		aString cTraitname = lociToCull[tr];
		item = traitName_.Loc(&cTraitname);
		if(item>=0){//the trait exists in the rawlist
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	ExpungeLoci(cullListByNum);
	}//end procedure CytoplasmicFemaleMarkerData::ExpungeLoci(StringList&)


void CytoplasmicFemaleMarkerData::ExpungeLoci (aList<int>& lociToCull){
	//This function removes trait numbers from the haploid data set; called during reclassification
	if(lociToCull.len()==0){return;}//none to cull
	int tr,cLoc;
	long item;
	for(tr=0;tr<lociToCull.len();++tr){
		cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(allCYfemaleLoci_.Includes(cLoc)){
			item = allCYfemaleLoci_.Loc(cLoc);
			allCYfemaleLoci_.Delete(item);}
		if(locusNumCYfemale_.Includes(cLoc)){
			item = locusNumCYfemale_.Loc(cLoc);
			locusNumCYfemale_.Delete(item);}
		if(skippedLocusNumCYfemale_.Includes(cLoc)){
			item = skippedLocusNumCYfemale_.Loc(cLoc);
			skippedLocusNumCYfemale_.Delete(item);}
		if(allPresentInFemalesOnly_.Includes(cLoc)){
			item = allPresentInFemalesOnly_.Loc(cLoc);
			allPresentInFemalesOnly_.Delete(item);}
		if(presentInFemalesOnly_.Includes(cLoc)){
			item = presentInFemalesOnly_.Loc(cLoc);
			presentInFemalesOnly_.Delete(item);}
		if(allPresentInBothSexes_.Includes(cLoc)){
			item = allPresentInBothSexes_.Loc(cLoc);
			allPresentInBothSexes_.Delete(item);}
		if(presentInBothSexes_.Includes(cLoc)){
			item = presentInBothSexes_.Loc(cLoc);
			presentInBothSexes_.Delete(item);}
		}//tr
	nlociCYfemale_ = locusNumCYfemale_.len();
	ResetFlags();
	CytoplasmicFemaleMarkerData::CheckForNewAlleles();
	}//CytoplasmicFemaleMarkerData::ExpungeLoci (aList<int>&)



void CytoplasmicFemaleMarkerData::ExpungeLocus (char* locusToCull){
	aString locToCull(locusToCull);
	ExpungeLocus(locToCull);
	}//end CytoplasmicFemaleMarkerData::ExpungeLocus (char* )


void CytoplasmicFemaleMarkerData::ExpungeLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	ExpungeLoci(lociToCull);
	}//end CytoplasmicFemaleMarkerData::ExpungeLocus (aString& )


long double CytoplasmicFemaleMarkerData::GetExpectedHeterozygosity (int pop, int loc, int allele){
	long double freq = GetFreqCYfemale(pop,loc,allele);
	return freq * ((long double)1.0 - freq);
	}//CytoplasmicFemaleMarkerData::GetExpectedHeterozygosity (int, int, int)
	
	
long double CytoplasmicFemaleMarkerData::GetExpectedHeterozygosity (int pop, int loc){
	long double freq, homosum=(long double)0.0;
	for(int allele=0;allele<allelesPresentCYfemale_[loc].len();++allele){
		freq = GetFreqCYfemale(pop,loc,allele);
		homosum += freq*freq;}
	return ((long double)1.0 - homosum);
	}//CytoplasmicFemaleMarkerData::GetExpectedHeterozygosity (int, int)




long double CytoplasmicFemaleMarkerData::GetFreqCYfemale (int loc, int allele){
	long double numAlleleCopies = 0.0;
	long double numThisAllele = 0.0;
	for(int pop=0;pop<npop_;++pop){
		long double n = (long double)GetLocusSampleSizeCYfemale(pop,loc);
		numAlleleCopies += n;
		numThisAllele += n * GetFreqCYfemale(pop,loc,allele);
		}
	if(numAlleleCopies==0.0){return (long double)0.0;}
	return numThisAllele/numAlleleCopies;
	}//end CytoplasmicFemaleMarkerData::GetFreqCYfemale


long double CytoplasmicFemaleMarkerData::GetFreqCYfemale (int pop, int loc, int marker){
	if (!genotypeFreqsCurrentCYfemale_){CalculateGenotypeFreqs();}
	return genotypeFreqCYfemale_[pop][loc][marker];
	}//end CytoplasmicFemaleMarkerData::GetFreqCYfemale


long double CytoplasmicFemaleMarkerData::GetFreqMostCommonAllele (int pop, int loc){
	long double mostCommon = 0.0;
	for(int allele=0;allele<GetNumAllelesCYfemale(loc);++allele){
		long double freq = GetFreqCYfemale(pop,loc,allele);
		if (freq>(long double)0.5) {return freq;}
		if (mostCommon<freq){mostCommon = freq;}
		}//and for allele
	return mostCommon;
	}//end CytoplasmicFemaleMarkerData::GetFreqMostCommonAllele


void CytoplasmicFemaleMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
	//uploads data to pass to genetic analysis classes
	pgcList.Clear();
	int pop,i, dt;
	//initialize
	aList<int> lociByType;
	for(dt=0;dt<MultiInheritanceModeContainer<int>::inheritanceModes_;++dt){
		lociByType.Append(0);}
	lociByType[MultiInheritanceModeContainer<int>::cytoFemale]=nlociCYfemale_;
	for(pop=0;pop<npop_;++pop){
		pgcList.Append(NEW PopulationGeneContainer<int>(popSize(pop),lociByType));}
	
	//fill up
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<popSize(pop);++i){
			GetGenome(pgcList[pop][i], pop, i);}
		}
	}//MultiformatMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >&)



void CytoplasmicFemaleMarkerData::GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID){
	//uploads this indiv's data to HaplotypeContainer<int>[locus]
	HaplotypeContainer<int>* hc = (HaplotypeContainer<int>*) &gc;
	hc->SetSize(CytoplasmicFemaleMarkerData::loci());
	if(setID){
		hc->SetID(1+GetIndivNumInDataSequence(pop,indiv));}
	for(int loc=0;loc<nlociCYfemale_;++loc){
		if(rawTraits_[pop][indiv][locusNumCYfemale_[loc]].missing()){
				hc->Clear(); return;}
			else{
				hc->setHaplotype(loc,CytoplasmicFemaleMarkerData::alleleID(pop,indiv,loc));}
		}	
	}//CytoplasmicFemaleMarkerData::GetGenome(GeneContainerCore&, int, int)


long CytoplasmicFemaleMarkerData::GetIndividualsSampledCYfemale (int pop, int loc){
	// loc refers to cytoplasmic male marker loci only
	long n=0;
	for(long i=0;i<individuals(pop);++i){
		if(GetLocusSampleSizeCYfemale (pop, i, loc)>0){//checks for correct sex in the called function
			++n;}}
	return n;
	}//end  CytoplasmicFemaleMarkerData::GetIndividualsSampledCYfemale


long CytoplasmicFemaleMarkerData::GetLocusSampleSizeCYfemale (int loc){
	if(! genotypeFreqsCurrentCYfemale_){
		CalculateGenotypeFreqs();}//calculates twoN_
	double numMarkersFound = 0.0;
	for(int pop=0;pop<npop_;++pop){
		numMarkersFound += nByLocusCYfemale_[pop][loc];
		}
	return numMarkersFound;
	}//end  CytoplasmicFemaleMarkerData::GetLocusSampleSizeCYfemale


long CytoplasmicFemaleMarkerData::GetLocusSampleSizeCYfemale (int pop, int loc){
	// loc refers to sex-linkeddominant loci only
	return nByLocusCYfemale_[pop][loc];
	}//end  CytoplasmicFemaleMarkerData::GetLocusSampleSizeCYfemale


long CytoplasmicFemaleMarkerData::GetLocusSampleSizeCYfemale (int pop, int indiv, int loc){
	// loc refers to dominant loci only
	if(isFemale(pop,indiv)) {return 0;}
	TextDatum& score = GenotypicVal(pop,indiv,loc,0);
	return (long) score.OK();//false=0 if missing, true=1 if present
	}//end  CytoplasmicFemaleMarkerData::GetLocusSampleSizeCYfemale



int CytoplasmicFemaleMarkerData::GetNumAllelesCYfemale (int loc){
	// loc refers to dominant loci only
	//total in data set
	if(! allelesCurrentCYfemale_){CheckForNewAlleles();}
	return allelesPresentCYfemale_[loc].len();
	}//end  CytoplasmicFemaleMarkerData::GetNumAllelesCYfemale(loc)


int CytoplasmicFemaleMarkerData::GetNumAllelesCYfemale (int pop, int loc){
	// loc refers to cytoplasmic male marker loci only
	//extant alleles in population
	if(! genotypeFreqsCurrentCYfemale_){CalculateGenotypeFreqs();}
	aList<long double>& locList = genotypeFreqCYfemale_[pop][loc];
	int count=0;
	for(int a=0;a<locList.len();++a){
		if (locList[a] >(long double) 0.0){++count;}}
	return count;
	}//end  CytoplasmicFemaleMarkerData::GetNumAllelesCYfemale(pop,loc)



aString* CytoplasmicFemaleMarkerData::GetTraitString(int pop, int indiv, int loc){
	//loc indexed as in locusNumCYfemale_ 
	aString *aStr = NEW aString();
	int tr = locusNumCYfemale_[loc];
	aStr->cat(rawTraits_[pop][indiv][tr][0]);
	return aStr;
	}//CytoplasmicFemaleMarkerData::GetTraitString


aString* CytoplasmicFemaleMarkerData::GetTraitString(int pop, int indiv, int loc, int /*allele*/){
	return GetTraitString(pop, indiv, loc);
	}//CytoplasmicFemaleMarkerData::GetTraitString



/*
int CytoplasmicFemaleMarkerData::individuals(int pop, int loc){//returns males only
	int indivs=0;
	for(int i=0;i<individuals(pop);++i){
		if (GetLocusSampleSizeCYfemale ( pop, i, loc)>0){++indivs;}}
	return indivs;
	}//end  CytoplasmicFemaleMarkerData::individuals(int,int)
*/	

long CytoplasmicFemaleMarkerData::maxall(void){
	//maximum alleles per locus in data set
	CheckForNewAlleles();
	long maxalleles=0;
	int loc;
	for(loc=0;loc<nlociCYfemale_;++loc){
		maxalleles = MAX(maxalleles,allelesPresentCYfemale_[loc].len());}
	return maxalleles;
	}//CytoplasmicFemaleMarkerData::maxall()






/*
void  EvolutionaryGeneticsData::PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of allelic frequencies for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculateAlleleFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied


	outFile<<title<<endl<<endl<<endl;
	outFile<<"ALLELE FREQUENCY DATA"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop<<endl;
	outFile<<"NUMBER OF LOCI SURVEYED = ";outFile.width(3);outFile<<nloci<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (lociSkipped == 0){
			outFile<<"none";}
		else{
			for (int loc=0;loc< lociSkipped;++loc){
				aString& locName = skippedLocusName[loc];
				outFile<<locName;
				if (loc < lociSkipped-1){outFile<<", ";}
				}//end for loc
		}//end if lociSkipped
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop / ncols;
	int npop0 = npop;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Allele frequencies for each population."<<endl;
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;

		for (int loc=0;loc<nloci;++loc) {//print the locus and locus sample size
			aString& locWord = locusName[loc];
			outFile<< locWord;
			if (asciiFormat){outFile.width(11 + (5 - locWord.len()));}else{outFile<<colSpacer;}
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = GetLocusSampleSize(pop, loc) / 2;
				outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
				if (pop < endPop){
					if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
					}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = GetFreq(pop, loc, allele);
					aString freqStr(freq,0,4);
					if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									if (asciiFormat){
											aString blanks(' ',12);
											if (pop==x) {
												blanks.PrependCharToLen(' ',17);}
											outFile<<blanks;
											}
										else{outFile<<colSpacer;}
									}
								else{
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end EvolutionaryGeneticsData::PrintAlleleFreqs




void  EvolutionaryGeneticsData::PrintAlleleFreqsByPopList (ostream& outFile, 
											ObjList<ObjList<aList<double> > >& popFreqList, 
											ObjList<aList<double> >& popTwoNList, 
											int ncols){
	//popFreqList must contain the entire set of alleles in the data set
	//Sends a table of allelic frequencies to the output file.
	//ncols indexes the number of population columns per page

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = popFreqList.len() / ncols;
	int npop0 = popFreqList.len();
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)popFreqList.len(), (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			outFile.width(12);outFile<<pop+1;}
		outFile<<endl;

		for (int loc=0;loc<nloci;++loc) {//print the locus and locus sample size
			aString& locWord = locusName[loc];
			outFile<< locWord;outFile.width(11 + (5 - locWord.len()));
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = popTwoNList[pop][loc] / 2;
				outFile<<"[";outFile.width(2);outFile<<samSize<<"]";
				if (pop < endPop){outFile.width(9);}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = popFreqList[pop][loc][allele];
					aString freqStr(freq,0,4);
					freqStr.PrependCharToLen(' ',12);
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									aString blanks(' ',12);
									if (pop==x) {
										blanks.PrependCharToLen(' ',17);}
									outFile<<blanks;}
								else{
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end EvolutionaryGeneticsData::PrintAlleleFreqsByPopList
*/


void	 CytoplasmicFemaleMarkerData::PrintCYfemaleMarkerFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintCYfemaleMarkerFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end CytoplasmicFemaleMarkerData::PrintCYfemaleMarkerFile(aString&)



void CytoplasmicFemaleMarkerData::PrintCYfemaleMarkerFile (ostream& outFile){
	aString dataLine = title_;
	dataLine.cat(":  skipped loci omitted");
	outFile<<dataLine<<endl;
	dataLine = "NPOP=";
	dataLine.cat(npop_);
	dataLine.cat(", NLOCI=");
	dataLine.cat(nlociCYfemale_);
	long maxall_=0;
	for(int loc=0;loc<allelesPresentCYfemale_.len();++loc){
		maxall_=MAX(maxall_,allelesPresentCYfemale_[loc].len());}
	dataLine.cat(", MAXALL=");
	dataLine.cat(maxall_);
	dataLine.cat(", NGROUPS=");
	dataLine.cat(ngroups_);
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int loc=0;loc<nlociCYfemale_;++loc){
		dataLine.cat(locusNameCYfemale(loc));
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < nlociCYfemale_){
				dataLine.cat("\t");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	dataLine = "SKIP=0";
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int pop=0;pop<npop_;++pop){
		outFile<<popName_[pop]<<endl;
		//cobble together a phenotype list for cytoplasmic male markers only
		aList<int> *malesPtr=0;
		if(heterogameticSex()==M){
				malesPtr = &heterogameticIndivs_[pop];}
			else{
				malesPtr = &homogameticIndivs_[pop];}
		aList<int>& males = *malesPtr;

		for (int i=0;i<males.len();++i){
			StringList idata;
			idata.Append(NEW TextDatum(rawTraits_[pop][males[i]][0]));
			for(int dloc=0;dloc<nlociCYfemale_;++dloc){
				TextDatum& score=rawTraits_[pop][males[i]][locusNumCYfemale_[dloc]];
				if(score.OK()){
						idata.Append(NEW aString(score));}
					else{//record as missing data
						idata.Append(NEW aString(rawTraits_[pop][males[i]].missingDataStr_));}
				}//dLoc
			aString outStr;
			for(int j=0;j<idata.len();++j){
				outStr+idata[j];
				if(j<idata.len()-1){
					outStr+" ";}}
			outFile<<outStr<<endl;
			}//end for i
		outFile<<"NEXT"<<endl;
		}//end for pop
	outFile<<"END"<<endl;
	}//end CytoplasmicFemaleMarkerData::PrintCYfemaleMarkerFile(ostream&)



void	 CytoplasmicFemaleMarkerData::PrintNEXUSFile(aString& outFileName, StringList& textToAppend){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintNEXUSFile(dataFile,textToAppend);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end CytoplasmicFemaleMarkerData::PrintNEXUSFile(aString&,StringList&)



void CytoplasmicFemaleMarkerData::PrintNEXUSFile (ostream& outFile, StringList& textToAppend){
	outFile<<"#NEXUS"<<endl;
	aString dataLine = title_;
	dataLine.prepend("[ ! ");
	outFile<<dataLine<<endl<<"]"<<endl<<endl;
	dataLine = "begin alleles;";
	outFile<<dataLine<<endl;
	dataLine = "\tdimensions newpops nloci=";
	dataLine.cat(nlociCYfemale_);
	dataLine.cat(" npops=");
	dataLine.cat(npop_);
	dataLine.cat(";");
	outFile<<dataLine<<endl;
	dataLine = "\tdominant all:1;";
	outFile<<dataLine<<endl;
	dataLine = "\tformat labels missing=? ;";
	outFile<<dataLine<<endl;
	dataLine = "\tmatrix";
	outFile<<dataLine<<endl;
	dataLine.clear();
	/* need code for locus names
	for (int loc=0;loc<nlociCYfemale_;++loc){
		dataLine.cat(locusNameDom_(loc));
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < nlociCYfemale_){
				dataLine.cat("\t");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	*/
	aString endCharList("\t, ");
	for (int pop=0;pop<npop_;++pop){
		aString pname = popName_[pop];
		pname.replaceAll(' ','_');
		outFile<<'\t'<<pname<<":"<<endl;
		aList<int> *malesPtr=0;
		if(heterogameticSex()==M){
				malesPtr = &heterogameticIndivs_[pop];}
			else{
				malesPtr = &homogameticIndivs_[pop];}
		aList<int>& males = *malesPtr;

		for (int i=0;i<males.len();++i){
			StringList idata;
			idata.Append(NEW TextDatum(rawTraits_[pop][males[i]][0]));
			idata[0].PrependCharToLen('0',3);//tweak the format of the ID code
			idata[0].prepend('_');
			idata[0].cat("_\t");
			for(int dloc=0;dloc<nlociCYfemale_;++dloc){
				TextDatum& score=rawTraits_[pop][males[i]][locusNumCYfemale_[dloc]];
				if(score.OK()){
						idata.Append(NEW aString(score));}
					else{//record as missing data
						idata.Append(NEW aString("?"));}
				}//dLoc
			aString outStr;
			for(int j=0;j<idata.len();++j){
				outStr+idata[j];
				if(j<idata.len()-1){
					outStr+"\t";}}
			outFile<<"\t\t"<<outStr<<endl;

			}//end for i
		if(pop==npop_-1){
				outFile<<";"<<endl;}
			else{
				outFile<<","<<endl;}
		}//end for pop
	outFile<<"end;"<<endl<<endl;
	
	for(int i=0;i<textToAppend.len();++i){
		outFile<<textToAppend[i]<<endl;}
	outFile<<endl;
	}//end CytoplasmicFemaleMarkerData::PrintNEXUSFile(ostream&, StringList&)



void CytoplasmicFemaleMarkerData::PrintLociCYfemale (ostream& outFile, int lociPerLine){//file assumed open
	int lineSpot = 0;
	for (int loc=0;loc<nlociCYfemale_;++loc){ 
		aString& locName = locusNameCYfemale(loc);
		outFile<<locName;
		++lineSpot;
		if ((loc == nlociCYfemale_-1) || (lineSpot == lociPerLine)) {
				outFile<<endl;}
			else{
				outFile<<"\t";}
		}//end for loc
	outFile<<endl;
	}//end CytoplasmicFemaleMarkerData::PrintLociCYfemale


void CytoplasmicFemaleMarkerData::PrintGenotypeFreqs(aString& outFileName, int ncols, char colSpacer){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintGenotypeFreqs(dataFile, ncols, colSpacer);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end CytoplasmicFemaleMarkerData::PrintGenotypeFreqs(aString&, int, char)



void  CytoplasmicFemaleMarkerData::PrintGenotypeFreqs (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of phenotypic frequencies for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculateGenotypeFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied
	int significantDigits = 4;

	outFile<<title_<<endl<<endl<<endl;
	outFile<<"ALLELE FREQUENCY DATA"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop_<<endl;
	outFile<<"NUMBER OF MARKERS SURVEYED = ";outFile.width(3);outFile<<nlociCYfemale_<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (skippedLocusNumCYfemale_.len() == 0){
			outFile<<colSpacer<<"none";}
		else{
			for (int loc=0;loc< skippedLocusNumCYfemale_.len();++loc){
				aString& locName = traitName_[skippedLocusNumCYfemale_[loc]];
				outFile<<locName;
				if(MOD(loc+1,20)==0){outFile<<endl;}
					else if (loc < skippedLocusNumCYfemale_.len()-1){outFile<<colSpacer;}
					else{}
				}//end for loc
		}//end if lociSkippedDom
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop_ / ncols;
	int npop0 = npop_;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Allele frequencies of CYTOPLASMICALLY INHERITED FEMALE markers for each population."<<endl;
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop_, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;

		for (int loc=0;loc<nlociCYfemale_;++loc) {//print the locus and locus sample size
			aString& locWord = locusNameCYfemale(loc);
			outFile<< locWord;
			if (asciiFormat){outFile.width(11 + (5 - locWord.len()));}else{outFile<<colSpacer;}
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = nByLocusCYfemale_[pop][loc];
				outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
				if (pop < endPop){
					if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
					}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& markerNameList = allelesPresentCYfemale_[loc];
			for (int marker=0; marker<markerNameList.len();++marker){
				aString& markerName=markerNameList[marker];
				aString markerNameCopy(markerName);
				markerNameCopy.PrependCharToLen(' ',3);
				outFile<<markerNameCopy;
				// print the frequency of the marker for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = genotypeFreqCYfemale_[pop][loc][marker];
					aString freqStr(freq,0,significantDigits);
					if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
										}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									if (asciiFormat){
											aString blanks(' ',12);
											if (pop==x) {
												blanks.PrependCharToLen(' ',17);}
											outFile<<blanks;
											}
										else{outFile<<colSpacer;}
									}
								else{
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end EvolutionaryGeneticsData::PrintGenotypeFreqs




void CytoplasmicFemaleMarkerData::ReadAFLPDataFile (aString& dataFileName){
	Clear();
	fstream dataFile;
	aString endCharList(" \t,\r"), missingDataListElems("?. -_");
	char endchar = '\n';	
	StringList missingDataList;
	for(int s=0;s<missingDataListElems.len();++s){
		missingDataList.Append(NEW aString(missingDataListElems[s]));}
	char *dataLine=NEW char[10001];dataLine[0]='\0';
	char* filename=dataFileName.raw();
	dataFile.open(filename,ios::in);

	dataFile.getline(dataLine, 10000,endchar);
	title_ = dataLine;

	dataFile.getline(dataLine, 10000,endchar); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	npop_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 6);
	nlociCYfemale_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 7);
	int maxall_ = paramWord->toInt();//now ignored
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	ngroups_ =  paramWord->toInt();
	delete paramWord; paramWord=0;


	//get locus information
	int moreLoci = 1;//true
	long loc = 0;
	StringList rawLocusNameCYfemale,skippedLocusNameCYfemale;//temporary
	while (moreLoci){
		dataFile.getline(dataLine, 10000,endchar); 
		aString locusLine(dataLine);
		while (locusLine.len() > 0){
			++loc;
			aString *locName = locusLine.ChopOffFirstWord(endCharList);
			if(locName->len()>0){//there could be trailing blanks on locusLine
				aString *s = NEW aString(locName);
				rawLocusNameCYfemale.Append(s);}//append a copy
			delete locName; locName=0;
			}//end while locusLine.len()
		if (loc >= nlociCYfemale_){//data may be on more than one line
			moreLoci = 0;}//false
		}//end while moreLoci
	//check for duplicates	
	StringList uniqueNames;
	for(long i=0;i<rawLocusNameCYfemale.len();++i){
		if(!(uniqueNames.Includes(&(rawLocusNameCYfemale[i])))){
			uniqueNames.Append(NEW aString(rawLocusNameCYfemale[i]));}}

	long numuni=uniqueNames.len();
	if(rawLocusNameCYfemale.len()>numuni){
		//figure out the names and positions of duplicated loci & report
		StringList dupNames(rawLocusNameCYfemale);
		for(loc=0;loc<uniqueNames.len();++loc){
			long uItem=dupNames.Loc(&(uniqueNames[loc]));
			dupNames.Delete(uItem);
			}//loc
		ObjList<aList<long> > dupPositions;
		for(loc=0;loc<dupNames.len();++loc){
			StringList locNamesCopy(rawLocusNameCYfemale);//work with a copy
			dupPositions.Append(NEW aList<long>());
			long dupcount=0;
			long dupitem = locNamesCopy.Loc(&(dupNames[loc]));
			while (dupitem>=0){
				locNamesCopy.Delete(dupitem);
				dupPositions[loc].Append(dupitem+dupcount);//because the locNamesCopy list shrinks from the bottom
				++dupcount;
				dupitem = locNamesCopy.Loc(&(dupNames[loc]));
				}//while
			}//loc
		cout<<"You have duplicated locus names in your data set."<<endl;
		cout<<"Please check that these are not mistakes, and give all your loci unique names."<<endl;
		cout<<"duplicated loci:"<<endl;
		dataFile<<"You have duplicated locus names in your data set."<<endl;
		dataFile<<"Please check that these are not mistakes, and give all your loci unique names."<<endl;
		dataFile<<"duplicated loci:"<<endl;
		for(loc=0;loc<dupNames.len();++loc){
			aString dupLine(dupNames[loc]);
			dupLine.cat(" at positions ");
			for(int i=0;i<dupPositions[loc].len();++i){
				dupLine.cat(dupPositions[loc][i]+1);
				if(i<dupPositions[loc].len()-1){
					dupLine.cat(", ");}
				}//i
			cout<<dupLine<<endl;
			dataFile<<dupLine<<endl;}
		aError("The duplicated names are in your output file. Please edit your data file and start again from the beginning.");
		return;
		}//unique names
	//read in the number of loci to be skipped in the analyses
	dataFile.getline(dataLine, 10000,endchar); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	int lociSkippedCYfemale = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped loci, if any
	loc = 0;
	moreLoci = 0;//false
	if (lociSkippedCYfemale > 0){
			moreLoci = 1;}//true
	while (moreLoci){
		while (skipLine.len() > 0){
			++loc;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				aString *s = NEW aString(skipWord);
				skippedLocusNameCYfemale.Append(s);//append a copy
				}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (loc >= lociSkippedCYfemale){
				moreLoci = 0;}//false
			else	{//more loci on the next line
				dataFile.getline(dataLine, 10000,endchar); 
				skipLine = dataLine;}
		}//end while moreLoci

	traitName_=rawLocusNameCYfemale;
	for(int tr=0;tr<traitName_.len();++tr){
		allCYfemaleLoci_.Append(tr);}
	locusNumCYfemale_=allCYfemaleLoci_;
		
	//read in population data
	aString popLine;
	aString beast;
	ObjList<StringList> rawGenotypes;
	for (int pop=0;pop<npop_;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 10000,endchar); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		aString *s = NEW aString(popLine);
		popName_.Append(s);//append a copy

		//make a new sublist for the genotypes of this population
		rawGenotypes.Append(NEW StringList());
		// BEAST is the string for a particular individual
		beast.clear();
		while (beast != "NEXT"){ 
			dataFile.getline(dataLine, 10000,endchar); 
			beast = dataLine;
			beast.CullTrailingChar('\t');
			beast.CullTrailingChar(' ');
			beast.CullTrailingChar(',');
			if (beast != "NEXT"){ 
					aString *s = NEW aString(beast);
					rawGenotypes[pop].Append(s);
					s=0;}//appends a copy
			}//end while beast..else
		}//end for pop
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	//Now parse the data by trait
	for(int pop=0;pop<npop_;++pop){	
		rawTraits_.Append(NEW ObjList<TextDataList>());
		for(int i=0;i<rawGenotypes[pop].len();++i){
			aString dataStrCopy(rawGenotypes[pop][i]);
			if(dataStrCopy.len()==0) continue;
			rawTraits_[pop].Append(NEW TextDataList("?"));
			aString *codeWord = dataStrCopy.ChopOffFirstWord(endCharList);
			rawTraits_[pop][i].Append(NEW TextDatum(*codeWord));
			for(int loc=0;loc<nlociCYfemale_;++loc){
				int posGtypeWord = 1 + ((alleleLen_+1) * loc);
				if(posGtypeWord>=dataStrCopy.len()){
						rawTraits_[pop][i].Append(NEW TextDatum("",true));
						continue;}
				aString gene = aString(dataStrCopy, posGtypeWord, posGtypeWord+alleleLen_-1);
				if(missingDataList.Includes(&gene)){
						rawTraits_[pop][i].Append(NEW TextDatum("",true));}
					else{
						rawTraits_[pop][i].Append(NEW TextDatum(gene));}
				}//loc
			delete codeWord;codeWord=0;
			}//i		
		}//pop		
	CullCYfemaleLoci(skippedLocusNameCYfemale);
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();
	}//end CytoplasmicFemaleMarkerData::ReadAFLPDataFile


/*

void CytoplasmicFemaleMarkerData::ReadAndAppendAFLPDataFile (aString& dataFileName){
	fstream dataFile;	
	char endchar = '\n';	
	char *dataLine=NEW char[10001];dataLine[0]='\0';
	aString endCharList(" \t,\r");
	char* filename=dataFileName.raw();
	dataFile.open(filename,ios::in);

	dataFile.getline(dataLine, 10000,endchar);
	title_.prepend("Combined data: [");
	title_.cat("] & [");
	title_.cat(dataLine);
	title_.cat("]");

	dataFile.getline(dataLine, 10000,endchar); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	int npopThisFile = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 6);
	int nlociDomThisFile = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 7);
	long maxallThisFile = paramWord->toInt();
	delete paramWord; paramWord=0;
	maxall = MAX(maxall, maxallThisFile);

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	int ngroupsThisFile =  paramWord->toInt();
	delete paramWord; paramWord=0;


	//get locus information
	int moreLoci = 1;//true
	int loc = 0;
	while (moreLoci){
		dataFile.getline(dataLine, 10000,endchar); 
		aString locusLine(dataLine);
		while (locusLine.len() > 0){
			++loc;
			aString *locName = locusLine.ChopOffFirstWord(endCharList);
			if(locName->len()>0){//there could be trailing blanks on locusLine
				if(!(rawLocusNameDom.Includes(locName))){//appended data can repeat loci
					aString *s = NEW aString(locName);
					rawLocusNameDom.Append(s);}//append a copy
				}
			delete locName; locName=0;
			}//end while locusLine.len()
		if (loc >= nlociDomThisFile){//data may be on more than one line
			moreLoci = 0;}//false
		}//end while moreLoci
	nlociCYfemale_ = rawLocusNameDom.len();
	//read in the number of loci to be skipped in the analyses
	dataFile.getline(dataLine, 10000,endchar); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	int lociSkippedDomThisFile = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped loci, if any
	loc = 0;
	moreLoci = 0;//false
	if (lociSkippedDom > 0){
			moreLoci = 1;}//true
	while (moreLoci){
		while (skipLine.len() > 0){
			++loc;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				if(!(skippedLocusNameDom.Includes(skipWord))){
					aString *s = NEW aString(skipWord);//skipped loci can be repeated
					skippedLocusNameDom.Append(s);//append a copy
					int toSkip = rawLocusNameDom.Loc(s);
					skippedLocusNumDom.Append(toSkip);}
				}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (loc >= lociSkippedDom){
				moreLoci = 0;}//false
			else	{//more loci on the next line
				dataFile.getline(dataLine, 10000,endchar); 
				skipLine = dataLine;}
		}//end while moreLoci
	lociSkippedDom = skippedLocusNameDom.len();

	locusNameDom.Copy(rawLocusNameDom);
	for (loc=0;loc<skippedLocusNumDom.len();++loc){
		aString& toSkip = skippedLocusNameDom[loc];
		int location = locusNameDom.Loc(&toSkip);
		locusNameDom.Delete(location);
		}//end for loc
	nlociCYfemale_ -= lociSkippedDom;

	//read in population data
	aString popLine;
	aString beast;
	for (int pop=0;pop<npopThisFile;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 10000,endchar); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		int thepop = pop;
		if(!(popName.Includes(&popLine))){
				aString *s = NEW aString(popLine);
				popName.Append(s);//append a copy
				thepop = popName.len()-1;//the last pop in list
				//make a new sublist for the genotypes of this population
				rawPhenotypes.Append(NEW StringList());				
				}
			else{
				thepop = popName.Loc(&popLine);}


		// BEAST is the string for a particular individual
		beast.clear();
		while (beast != "NEXT"){ 
			dataFile.getline(dataLine, 10000,endchar); 
			beast = dataLine;
			beast.CullTrailingChar('\t');
			beast.CullTrailingChar(' ');
			beast.CullTrailingChar(',');
			if (beast != "NEXT"){ 
				//check to see if this individual already has data
				aString beastCopy(beast);
				aString* IDbeast = beastCopy.ChopOffFirstWord(endCharList);
				int hasData	= 0, indivNum=0;
				for(int i=0;i<rawPhenotypes[thepop].len();++i){
					aString indivData(rawPhenotypes[thepop][i]);//copies
					aString *IDindiv = indivData.ChopOffFirstWord(endCharList);
					if(*IDbeast == *IDindiv){//already in data set
						hasData = 1;//true
						indivNum = i;
						delete IDindiv;IDindiv=0;
						break;//escape for i loop
						}
					delete IDindiv;IDindiv=0;
					}//end for i
				if(hasData){//append to end of this individual -- buggy if loci repeated
						beastCopy.prepend('\t');
						rawPhenotypes[thepop][indivNum].cat(beastCopy);
						}
					else{
						aString *s = NEW aString(beast);
						rawPhenotypes[thepop].Append(s);
						s=0;}//appends a copy
				delete IDbeast; IDbeast=0;
				}//end if beast != NEXT
			}//end while beast..else


		}//end for pop
	npop_ = rawPhenotypes.len();
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	ResetFlags();
	GroupMembership();
	LocalityMembership();
	CullSkippedLociDom();
	CheckForNewAlleles();
	}//end CytoplasmicFemaleMarkerData::ReadAndAppendAFLPDataFile
*/




void CytoplasmicFemaleMarkerData::RestoreLociCYfemale (aList<int>& lociToRestore){
	//This function restores the designated loci to the analysis.
	//It shifts trait numbers from from skippedTraitNums_ to the traitNums_ list
	//and keps track of the changes in the locusNumCYfemale_ and skippedLocusNumCYfemale_ lists
	if(lociToRestore.len()==0){return;}//none to cull
	ResetFlags();
	int tr,rLoc;
	long item;
	for(tr=0;tr<lociToRestore.len();++tr){
		rLoc = lociToRestore[tr];//make sure it's a valid locus to restore
		if(skippedTraitNum_.Includes(rLoc) && rLoc>=0 && rLoc<traitName_.len()){
			traitNum_.Append(rLoc);
			locusNumCYfemale_.Append(rLoc);
			item = skippedTraitNum_.Loc(rLoc);
			skippedTraitNum_.Delete(item);
			item = skippedLocusNumCYfemale_.Loc(rLoc);
			skippedLocusNumCYfemale_.Delete(item);
			if(allPresentInFemalesOnly_.Includes(rLoc) && ! presentInFemalesOnly_.Includes(rLoc)){
				presentInFemalesOnly_.Append(rLoc);}
			if(allPresentInBothSexes_.Includes(rLoc) && ! presentInBothSexes_.Includes(rLoc)){
				presentInBothSexes_.Append(rLoc);}
			}//if item not skipped already
		}//tr
	traitNum_.Sort();
	locusNumCYfemale_.Sort();
	presentInFemalesOnly_.Sort();
	presentInBothSexes_.Sort();
	CheckForNewAlleles();
	}//end  CytoplasmicFemaleMarkerData::RestoreLociCYfemale(aList<int>&)


void CytoplasmicFemaleMarkerData::RestoreLociCYfemale (StringList& lociToRestore){
	//This function restores the designated loci to the analysis
	if(lociToRestore.len()==0){return;}//none to cull
	//convert names to numbers and cull
	aList<int> restoreListByNum;
	for(int tr=0;tr<lociToRestore.len();++tr){
		aString rTraitname = lociToRestore[tr];
		int item = traitName_.Loc(&rTraitname);
		if((item>=0)&&(!(traitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already included
			restoreListByNum.Append(item);
			}//if item exists
		}//tr
	RestoreLociCYfemale(restoreListByNum);
	}//end CytoplasmicFemaleMarkerData::RestoreLociCYfemale(StringList&)


void CytoplasmicFemaleMarkerData::RestoreLocusCYfemale (aString& locusToRestore){
	StringList lociToRestore;
	lociToRestore.Append(new aString(locusToRestore));
	RestoreLociCYfemale(lociToRestore);
	}//end CytoplasmicFemaleMarkerData::RestoreLocusCYfemale (aString& )




void CytoplasmicFemaleMarkerData::SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
	//records all data, passed along from genetic analysis classes
	//empties old data and creates new data structures
	Clear();
	title_ = "Simulated data set";
	npop_ = pgcList.len();
	nlociCYfemale_ = pgcList[0].loci(MultiInheritanceModeContainer<int>::cytoFemale);
	ngroups_ = 1;
	int runningTraitNum=0;
	int pop,i, loc;
	SetAlleleLength(1);//bug if maxall>9
	traitName_.Append(NEW aString("ID"));
	traitNum_.Append(runningTraitNum++);

	if(pgcList[0].sexLinkedDataIncluded()){
		traitName_.Append(NEW aString("sex"));
		traitNum_.Append(runningTraitNum++);
		SetSexTrait();		
		for(pop=0;pop<npop_;++pop){
			homogameticIndivs_.Append(NEW aList<int>());
			heterogameticIndivs_.Append(NEW aList<int>());}
		}
	int locusCount=0;
	traitNum_.Append(locusCount++);
	for(loc = 0;loc<nlociCYfemale_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',Digits(nlociCYfemale_));
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(runningTraitNum++);
		allCYfemaleLoci_.Append(loc+1);
		locusNumCYfemale_.Append(loc+1);
		}
	ntraits_ = traitNum_.len();
	
	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("simulated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create dummy id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	

		for (i=0;i<pgcList[pop].popSize();++i){
			rawTraits_[pop].Append(NEW TextDataList());
			rawTraits_[pop][i].Append(NEW TextDatum());}//empty ID
		popSize_.Append(pgcList[pop].popSize());


		//create and append dummy genotypes
		for(i=0;i<pgcList[pop].popSize();++i){
			//determine sex, if sex-linked
			if(pgcList[pop].sexLinkedDataIncluded()){
			rawTraits_[pop][i].Append(NEW TextDatum(" "));}//empty sexTrait

			for(loc=0;loc<nlociCYfemale_;++loc){
				rawTraits_[pop][i].Append(NEW TextDatum(" "));
				}//end for loc
			}//end for i
		}//end for pop	

	//Fill up with data
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<popSize(pop);++i){
			SetGenome(pgcList[pop][i], pop, i);}
		}
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();		
	}//CytoplasmicFemaleMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >&)



void CytoplasmicFemaleMarkerData::SetGenome(GeneContainerCore& gc, int pop, int indiv){
	//records data from HaplotypeContainer<int>
	HaplotypeContainer<int>* hc = (HaplotypeContainer<int>*) &gc;
	if(hc->loci() != nlociCYfemale_){
		aError("ERROR: CytoplasmicFemaleMarkerData::SetGenome(): size mismatch between data classes");
		return;}
	ResetFlags();
	rawTraits_[pop][indiv][0] = hc->ID();
	for(int loc=0;loc<nlociCYfemale_;++loc){
		if(hc->missing()){//males may have missing data if sex limited
				rawTraits_[pop][indiv][locusNumCYfemale_[loc]].clear();}
			else{
				rawTraits_[pop][indiv][locusNumCYfemale_[loc]] = hc->haplotype(loc);}	
		}
	}//CytoplasmicFemaleMarkerData::GetGenome(GeneContainerCore&, int, int)



void CytoplasmicFemaleMarkerData::SetLoci(aList<int>& lociToInclude){
	if(lociToInclude.len()==0) return;
	CytoplasmicFemaleMarkerData::ResetGeneticFlags();
	allCYfemaleLoci_=lociToInclude;
	locusNumCYfemale_=lociToInclude;
	aList<int> femalesOnly;
	int tr,cLoc;
	long item;
	for(tr=0;tr<lociToInclude.len();++tr){
		cLoc = lociToInclude[tr];
		if(skippedTraitNum_.Includes(cLoc)){
			item = locusNumCYfemale_.Loc(cLoc);
			if(item>=0){
				locusNumCYfemale_.Delete(item);}
			if(! skippedLocusNumCYfemale_.Includes(cLoc)){
				skippedLocusNumCYfemale_.Append(cLoc);}
			}
		if(allPresentInFemalesOnly_.Includes(cLoc)){
			femalesOnly.Append(cLoc);}
		}//tr
	skippedLocusNumCYfemale_.Sort();
	nlociCYfemale_ = locusNumCYfemale_.len();
	SetLociPresentInFemalesOnly(femalesOnly);
	CytoplasmicFemaleMarkerData::CheckForNewAlleles();
	}//CytoplasmicFemaleMarkerData::SetLoci(aList<int>& )



void CytoplasmicFemaleMarkerData::SetLociPresentInFemalesOnly(aList<int>& locusList){
	//assumes males contain missing data as placeholders
	allPresentInFemalesOnly_ = locusList;
	allPresentInFemalesOnly_.Sort();
	presentInFemalesOnly_.Clear();
	allPresentInBothSexes_.Clear();
	presentInBothSexes_.Clear();
	int loc;
	for(loc=0;loc<allCYfemaleLoci_.len();++loc){
		if(! allPresentInFemalesOnly_.Includes(allCYfemaleLoci_[loc])){
			allPresentInBothSexes_.Append(allCYfemaleLoci_[loc]);}
		}//loc
	for(loc=0;loc<locusNumCYfemale_.len();++loc){
		if(allPresentInFemalesOnly_.Includes(locusNumCYfemale_[loc])){
				presentInFemalesOnly_.Append(locusNumCYfemale_[loc]);}
			else if(allPresentInBothSexes_.Includes(locusNumCYfemale_[loc])){
				presentInBothSexes_.Append(locusNumCYfemale_[loc]);}
			else{}
		}//loc
	}//CytoplasmicFemaleMarkerData::SetLociPresentInFemalesOnly(aList<int>& )



void CytoplasmicFemaleMarkerData::UpdateVariableLociCYfemale(void){
	if (varLociCurrentCYfemale_){return;}
	CheckForNewAlleles();
	variableLocusNumCYfemale_.Clear();
	for(int loc=0;loc<nlociCYfemale_;++loc){
		if (allelesPresentCYfemale_[loc].len()>1){
			variableLocusNumCYfemale_.Append(loc);}}
	varLociCurrentCYfemale_=1;
	}//end CytoplasmicFemaleMarkerData::UpdateVariableLociCYfemale



#endif









