// CytoplasmicMaleMarkerData.cpp

// Holds data for markers that are transmitted through males only (usually Y chromosomes)

#ifndef CytoplasmicMaleMarkerDatacpp
#define CytoplasmicMaleMarkerDatacpp = 1

#include "CytoplasmicMaleMarkerData.h"
#include "Element.cpp"
#include "aList.cpp"
#include "SortList.cpp"
#include "GeneContainers.cpp"





CytoplasmicMaleMarkerData::CytoplasmicMaleMarkerData(CytoplasmicMaleMarkerData& parentData):
			SexLinkedData(parentData),
			nlociCYmale_(parentData.nlociCYmale_),
			ploidy_(parentData.ploidy_),
			alleleLen_(parentData.alleleLen_),
			allCYmaleLoci_(parentData.allCYmaleLoci_),
			locusNumCYmale_(parentData.locusNumCYmale_),
			skippedLocusNumCYmale_(parentData.skippedLocusNumCYmale_),
			allPresentInMalesOnly_(parentData.allPresentInMalesOnly_),
			presentInMalesOnly_(parentData.presentInMalesOnly_),
			allPresentInBothSexes_(parentData.allPresentInBothSexes_),
			presentInBothSexes_(parentData.presentInBothSexes_),
			variableLocusNumCYmale_(parentData.variableLocusNumCYmale_), 
			allelesPresentCYmale_(parentData.allelesPresentCYmale_),
			genotypeFreqCYmale_(parentData.genotypeFreqCYmale_),
			genotypeFreqCYmaleGlobal_(parentData.genotypeFreqCYmaleGlobal_),
			nByLocusCYmale_(parentData.nByLocusCYmale_),
			allelesCurrentCYmale_(parentData.allelesCurrentCYmale_),
			varLociCurrentCYmale_(parentData.varLociCurrentCYmale_),
			genotypeFreqsCurrentCYmale_(parentData.genotypeFreqsCurrentCYmale_)
			{	
		}//end ctor CytoplasmicMaleMarkerData(CytoplasmicMaleMarkerData&)


CytoplasmicMaleMarkerData::CytoplasmicMaleMarkerData(CytoplasmicMaleMarkerData& parentData, 
						aList<int>& popSubsetNums):
			nlociCYmale_(parentData.nlociCYmale_),
			ploidy_(parentData.ploidy_),
			alleleLen_(parentData.alleleLen_),
			allCYmaleLoci_(parentData.allCYmaleLoci_),
			locusNumCYmale_(parentData.locusNumCYmale_),
			skippedLocusNumCYmale_(parentData.skippedLocusNumCYmale_),
			allPresentInMalesOnly_(parentData.allPresentInMalesOnly_),
			presentInMalesOnly_(parentData.presentInMalesOnly_),
			allPresentInBothSexes_(parentData.allPresentInBothSexes_),
			presentInBothSexes_(parentData.presentInBothSexes_)
			{
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	Initialize(parentData,popSubsetNums);
	title_.prepend("subset of dataset: ");
	ResetFlags();
	SortBySex();
	CheckForNewAlleles();
	UpdateVariableLociCYmale();
	}//end ctor CytoplasmicMaleMarkerData(CytoplasmicMaleMarkerData&, aList<int>&)


void  CytoplasmicMaleMarkerData::Initialize(void){
	CytoplasmicMaleMarkerData::Clear();
	}//Initialize()
	
	
void  CytoplasmicMaleMarkerData::Initialize(aString& s){
	CytoplasmicMaleMarkerData::Clear();
	title_=s; 
	}//Initialize(aString&)
	
	
void  CytoplasmicMaleMarkerData::Initialize(CytoplasmicMaleMarkerData& cymd){
	SexLinkedData::Initialize(cymd);
	nlociCYmale_=cymd.nlociCYmale_;
	ploidy_=cymd.ploidy_;
	alleleLen_=cymd.alleleLen_;
	allCYmaleLoci_=cymd.allCYmaleLoci_;
	locusNumCYmale_=cymd.locusNumCYmale_; 
	skippedLocusNumCYmale_=cymd.skippedLocusNumCYmale_;
	variableLocusNumCYmale_=cymd.variableLocusNumCYmale_;
	allPresentInMalesOnly_=cymd.allPresentInMalesOnly_;
	presentInMalesOnly_=cymd.presentInMalesOnly_;
	allPresentInBothSexes_=cymd.allPresentInBothSexes_;
	presentInBothSexes_=cymd.presentInBothSexes_;
	allelesPresentCYmale_=cymd.allelesPresentCYmale_;
	genotypeFreqCYmale_=cymd.genotypeFreqCYmale_;
	genotypeFreqCYmaleGlobal_=cymd.genotypeFreqCYmaleGlobal_;
	nByLocusCYmale_=cymd.nByLocusCYmale_;
	allelesCurrentCYmale_=cymd.allelesCurrentCYmale_;
	varLociCurrentCYmale_=cymd.varLociCurrentCYmale_;
	genotypeFreqsCurrentCYmale_=cymd.genotypeFreqsCurrentCYmale_;
	}//Initialize(CytoplasmicMaleMarkerData&)
	

void  CytoplasmicMaleMarkerData::Initialize(CytoplasmicMaleMarkerData& parentData, aList<int>& popSubsetNums){
	SexLinkedData::Initialize(parentData,popSubsetNums);
	nlociCYmale_=parentData.nlociCYmale_;
	ploidy_=parentData.ploidy_;
	alleleLen_=parentData.alleleLen_;
	allCYmaleLoci_=parentData.allCYmaleLoci_;
	locusNumCYmale_=parentData.locusNumCYmale_;
	skippedLocusNumCYmale_=parentData.skippedLocusNumCYmale_;
	allPresentInMalesOnly_=parentData.allPresentInMalesOnly_;
	presentInMalesOnly_=parentData.presentInMalesOnly_;
	allPresentInBothSexes_=parentData.allPresentInBothSexes_;
	presentInBothSexes_=parentData.presentInBothSexes_;
	SortBySex();
	CheckForNewAlleles();
	UpdateVariableLociCYmale();
	}//end Initialize(CytoplasmicMaleMarkerData&, aList<int>&)


	
CytoplasmicMaleMarkerData&  CytoplasmicMaleMarkerData::operator=(CytoplasmicMaleMarkerData& cymd){
	Initialize(cymd);
	return *this;
	}//operator=(CytoplasmicMaleMarkerData&)




void CytoplasmicMaleMarkerData::CalculateGenotypeFreqs (void){
	//This routine counts marker frequencies, taking into account
	//markers left blank in the data set.
	if (genotypeFreqsCurrentCYmale_){return;}
	SortBySex();
	CheckForNewAlleles();
	int pop, i, loc, marker;
	long double freq;
//	char blank = ' ', tab='\t';	//don't count these
	aString missingDataFlags = ".?-*";//characters used to signify missing data
	aString endCharList = " \t,";//characters used to separate data fields
	genotypeFreqCYmale_.Clear();
	nByLocusCYmale_.Clear();
	for (pop=0;pop< npop_;++pop){ 
		//set up a frequency list for this population
		genotypeFreqCYmale_.Append( NEW ObjList<aList<long double> >());
		nByLocusCYmale_.Append(NEW aList<int>());
		ObjList<TextDataList>& genotypesList = rawTraits_[pop];
		aList<int> *malesPtr=0;
		if(heterogameticSex()==M){
				malesPtr = &heterogameticIndivs_[pop];}
			else{
				malesPtr = &homogameticIndivs_[pop];}
		aList<int>& males = *malesPtr;
		for (loc=0;loc<nlociCYmale_;++loc){//loops through dominant loci
			//first set up a blank array of frequencies for each possible phenotype
			genotypeFreqCYmale_[pop].Append(NEW aList<long double>());
			StringList& markerNamesList = allelesPresentCYmale_[loc];
			freq = zero;
			for (marker=0;marker<markerNamesList.len();++marker){//initialize
				genotypeFreqCYmale_[pop][loc].Append(freq);
				}

			//next look at the genotype of each individual and tally the frequency
			long nSampled=0;	//the number of individuals
//			if(presentInMalesOnly_[loc]){
//					for (i=0;i< males.len();++i){//quicker
//						TextDatum& gene = genotypesList[males[i]][locusNumCYmale_[loc]];
//						if (gene.OK()) {//if not, it's missing data
//							//increment the frequency & sample size -- divide later
//							marker = markerNamesList.Loc(&gene);
//							genotypeFreqCYmale_[pop][loc][marker]++;
//							nSampled++;
//							}//end if legal data
//						}//end for i
//					}
//				else{
					for (i=0;i<individuals(pop);++i){
						TextDatum& gene = genotypesList[i][locusNumCYmale_[loc]];
						if (gene.OK()) {//if not, it's missing data
							//increment the frequency & sample size -- divide later
							marker = markerNamesList.Loc(&gene);
							genotypeFreqCYmale_[pop][loc][marker]++;
							nSampled++;
							}//end if legal data
						}//end for i
//					}

			//now divide by the sample size to restore the frequencies
			if(nSampled>0){
				for (marker=0;marker<markerNamesList.len();++marker){
					genotypeFreqCYmale_[pop][loc][marker] /= (long double) nSampled;
					}}
			//store the sample size for this locus
			nByLocusCYmale_[pop].Append((int)nSampled);
			}//end for loc
		}//end for pop
	genotypeFreqCYmaleGlobal_.Clear();
	for(loc=0;loc<nlociCYmale_;++loc){
		genotypeFreqCYmaleGlobal_.Append(NEW aList<long double>());
		for(marker=0;marker<allelesPresentCYmale_[loc].len();++marker){
			freq=zero;
			for(pop=0;pop<npop_;++pop){
				freq += genotypeFreqCYmale_[pop][loc][marker];}
			genotypeFreqCYmaleGlobal_[loc].Append(freq/(long double)npop_);
			}}
	genotypeFreqsCurrentCYmale_=1;
	}//end CytoplasmicMaleMarkerData::CalculateGenotypeFreqs



ObjList<aList<long double> >* CytoplasmicMaleMarkerData::CalculateGenotypeFreqsByPopSet(aList<int>& popSet, 
										aList<long double>& nList, int locusTypesToInclude){
 	//returns list of loci, each with list of marker freqs, using total marker set
 	CalculateGenotypeFreqs();
 	nList.Clear();
 	ObjList<aList<long double> >* popSetFreqsPtr = NEW ObjList<aList<long double> >();
 	ObjList<aList<long double> >& popSetFreqs = *popSetFreqsPtr;//notational convenience
	int loc,markers,pop,popSetPop,m,item;
	long double nCopies, nSample, samSizeSet;
	ObjList<aList<long double> > popSetFreqsBothSexes, popSetFreqsMonly;
	aList<long double> nListB, nListM;
	if(locusTypesToInclude != presentMonly){
	 	for(item=0;item<presentInBothSexes_.len();++item){
	 		loc = locusNumCYmale_.Loc(presentInBothSexes_[item]);
	 		popSetFreqsBothSexes.Append(NEW aList<long double>());
	 		markers = GetNumAllelesCYmale(item);
	 		samSizeSet = zero;
	 		for(m=0;m<markers;++m){
	 			nCopies = zero;
				popSetFreqsBothSexes[item].Append(zero);
	 			for(popSetPop=0;popSetPop<popSet.len();++popSetPop){
	 				pop = popSet[popSetPop];
	 				nSample = (long double) (nByLocusCYmale_[pop][item]);
	 				if(m==0){samSizeSet += nSample;}
	 				nCopies += nSample * GetFreqCYmale(pop,item,m);
	 				}
	 			if(m==0){nListB.Append(samSizeSet);}
	 			if(nCopies==samSizeSet){
	 					popSetFreqsBothSexes[item][m]=one;}
	 				else{
	 					popSetFreqsBothSexes[item][m] = nCopies / samSizeSet;
	 					}
	 			}//end for m
	 		}//end for item
	 	
	 	}// !presentMonly
	if(locusTypesToInclude != presentBothSexes){
	 	for(item=0;item<presentInMalesOnly_.len();++item){
	 		loc = locusNumCYmale_.Loc(presentInMalesOnly_[item]);
	 		int mOnlyItem=item+presentInBothSexes_.len();
	 		popSetFreqsMonly.Append(NEW aList<long double>());
	 		markers = GetNumAllelesCYmale(mOnlyItem);
	 		samSizeSet = zero;
	 		for(m=0;m<markers;++m){
	 			nCopies = zero;
				popSetFreqsMonly[item].Append(zero);
	 			for(popSetPop=0;popSetPop<popSet.len();++popSetPop){
	 				pop = popSet[popSetPop];
	 				nSample = (long double) (nByLocusCYmale_[pop][mOnlyItem]);
	 				if(m==0){samSizeSet += nSample;}
	 				nCopies += nSample * GetFreqCYmale(pop,mOnlyItem,m);
	 				}
	 			if(m==0){nListM.Append(samSizeSet);}
	 			if(nCopies==samSizeSet){
	 					popSetFreqsMonly[item][m]=one;}
	 				else{
	 					popSetFreqsMonly[item][m] = nCopies / samSizeSet;
	 					}
	 			}//end for m
	 		}//end for item
	 	}// !presentBothSexes
	//sort these into the right spots
	if(locusTypesToInclude == presentBothSexes){
			nList = nListB;
			popSetFreqs = popSetFreqsBothSexes;}
		else if (locusTypesToInclude == presentMonly){
			nList = nListM;
			popSetFreqs = popSetFreqsMonly;}
		else{//sort into right order
			for(loc=0;loc<nlociCYmale_;++loc){
				item = presentInBothSexes_.Loc(loc);
				if(item>=0){
						nList.Append(nListB[item]);
						popSetFreqs.Append(NEW aList<long double>(popSetFreqsBothSexes[item]));
						}
					else{
						item = presentInMalesOnly_.Loc(loc);
						if(item>=0){
							nList.Append(nListM[item]);
							popSetFreqs.Append(NEW aList<long double>(popSetFreqsBothSexes[item]));
							}
						}
				}//loc
			}//sort
 	return popSetFreqsPtr;
	}//end CytoplasmicMaleMarkerData::CalculateGenotypeFreqsByPopSet



void CytoplasmicMaleMarkerData::CheckForNewAlleles (void){
	if (allelesCurrentCYmale_){return;}
	allelesPresentCYmale_.Clear();
	int loc,pop,i;
	for (loc=0;loc<nlociCYmale_;++loc){//initialize
		allelesPresentCYmale_.Append(NEW StringList());}
	for (pop=0;pop<npop_;++pop){
		ObjList<TextDataList>& ptypeList = rawTraits_[pop];
		for(i=0;i<individuals(pop);++i){
			for (loc=0;loc<nlociCYmale_;++loc){
				TextDatum& gene = ptypeList[i][locusNumCYmale_[loc]];
				if(gene.OK()){//ignore missing data
					StringList& alleleList = allelesPresentCYmale_[loc];
					if(!(alleleList.Includes(&gene))){
						alleleList.Append(NEW aString(gene));}
					}//OK
				}//end for loc
			}//end for i
		}//end for pop
	for (loc=0;loc<nlociCYmale_;++loc){
		allelesPresentCYmale_[loc].Sort();}
	allelesCurrentCYmale_=true;
	CytoplasmicMaleMarkerData::UpdateVariableLociCYmale();
	}//end CytoplasmicMaleMarkerData::CheckForNewAlleles



void CytoplasmicMaleMarkerData::Clear(void){
	EvolutionaryGeneticsData::Clear();
	allCYmaleLoci_.Clear();
	locusNumCYmale_.Clear();
	skippedLocusNumCYmale_.Clear();
	variableLocusNumCYmale_.Clear();
	allPresentInMalesOnly_.Clear();
	presentInMalesOnly_.Clear();
	allPresentInBothSexes_.Clear();
	presentInBothSexes_.Clear();
	allelesPresentCYmale_.Clear();
	genotypeFreqCYmale_.Clear();
	genotypeFreqCYmaleGlobal_.Clear();
	nByLocusCYmale_.Clear();
	nlociCYmale_=allelesCurrentCYmale_=varLociCurrentCYmale_=genotypeFreqsCurrentCYmale_=0;
	}//end CytoplasmicMaleMarkerData::Clear()





void  CytoplasmicMaleMarkerData::CreateBootstrapData(CytoplasmicMaleMarkerData& origData, Ran3& ran){
	EvolutionaryGeneticsData::CreateBootstrapData(origData,ran);
	SortBySex();
	CheckForNewAlleles();	
	}//end  CytoplasmicMaleMarkerData::CreateBootstrapData(EvolutionaryGeneticsData&)



void  CytoplasmicMaleMarkerData::CreateBootstrapDataConstPop(CytoplasmicMaleMarkerData& origData, Ran3& ran){
	EvolutionaryGeneticsData::CreateBootstrapDataConstPop(origData,ran);
	SortBySex();
	CheckForNewAlleles();	
	}//end  CytoplasmicMaleMarkerData::CreateBootstrapDataConstPop(EvolutionaryGeneticsData&)




void CytoplasmicMaleMarkerData::CreateRandomData(
							const long double Fst, 
							const long double /*Fis*/,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran,
							aList<int>& lociInMalesOnly){
	char hetSex = heterogameticSex_;
	long double sr = primarySexRatio_;
	Clear();
	heterogameticSex_ = hetSex;//restore these
	primarySexRatio_ = sr;
	lociInMalesOnly.Sort();
	int pop,loc,i;
	presentInMalesOnly_ = allPresentInMalesOnly_ = lociInMalesOnly;
	for(loc=0;loc<numLociSampled;++loc){
		if(! allPresentInMalesOnly_.Includes(loc)){
			allPresentInBothSexes_.Append(loc);
			allPresentInBothSexes_.Append(loc);
			}}
	title_ = "Randomly generated cytoplasmic male marker data set with parametric Fst=";
	title_.cat(Fst);  
	title_.cat(" and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numPopsSampled;
	nlociCYmale_ = numLociSampled;
	ngroups_ = 1;
	traitName_.Append(NEW aString("ID"));
	int locusCount=0;
	traitNum_.Append(locusCount++);
	if(allPresentInMalesOnly_.len()>0){
		traitName_.Append(NEW aString("sex"));
		traitNum_.Append(locusCount++);
		sexTrait_=1;
		}
	for(loc = 0;loc<nlociCYmale_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',Digits(nlociCYmale_));
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(locusCount++);
		allCYmaleLoci_.Append(loc+1);
		locusNumCYmale_.Append(loc+1);
		}
	ntraits_ = traitNum_.len();

	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
	GeneFrequencyDistribution parametricFreqDistrib(Fst, pBar, 1);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	aString genotype;
	long double pf, pp;
	ObjList<StatList<long double> > locusFrequenciesByPop;
	for(pop=0;pop<npop_;++pop){
		StatList<long double>* locusFrequencies = NEW StatList<long double>();
		locusFrequenciesByPop.Append(locusFrequencies);
		for(loc=0;loc<nlociCYmale_;++loc){
			pf = parametricFreqDistrib.RandomFrequency(ran);
			locusFrequencies->Append(pf);
			}//end for loc
		}//end for pop

	aList<int> female1male0;
	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("randomly generated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
		for (i=0;i<numIndivsPerPop;++i){
			rawTraits_[pop].Append(NEW TextDataList());
			aString indiv(i+1);
			indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));
			if(presentInMalesOnly_.len()>0){
				female1male0.Append(0);
				rawTraits_[pop][i].Append(NEW TextDatum('M'));
				if(ran()<sr){
					female1male0[i]=1;
					rawTraits_[pop][i].Last()='F';}}		
			}//end for i
		popSize_.Append(numIndivsPerPop);

		//create and append genotypes
		for(loc=0;loc<nlociCYmale_;++loc){
			StatList<long double>& locusFrequencies = locusFrequenciesByPop[pop];
			pp = locusFrequencies[loc];
			for(i=0;i<numIndivsPerPop;++i){
				if(presentInMalesOnly_.Includes(loc) && female1male0[i]==1){
						rawTraits_[pop][i].Append(NEW TextDatum());}//missing: placeholder
					else{
						if (ran()<pp){genotype = "A";}
							else {genotype = "B";}				
						rawTraits_[pop][i].Append(NEW TextDatum(genotype));
						}
				}//end for i
			}//end for loc
		}//end for pop	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end CytoplasmicMaleMarkerData::CreateRandomData(...)




void CytoplasmicMaleMarkerData::CreateRandomData(
							const long double Fst, 
							const long double /*Fis*/,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,//males only
							Ran3List& ran,
							aList<int>& lociInMalesOnly){
	//multiple alleles per locus, variable sample sizes per pop
	//	first set up initial variables
	char hetSex = heterogameticSex_;
	long double sr = primarySexRatio_;
	Clear();
	heterogameticSex_ = hetSex;//restore these
	primarySexRatio_ = sr;
	lociInMalesOnly.Sort();
	int pop,loc,i;
	presentInMalesOnly_ = allPresentInMalesOnly_ = lociInMalesOnly;
	nlociCYmale_ = pBar.len();
	for(loc=0;loc<nlociCYmale_;++loc){
		if(! allPresentInMalesOnly_.Includes(loc)){
			allPresentInBothSexes_.Append(loc);
			allPresentInBothSexes_.Append(loc);
			}}
	title_ = "Randomly generated cytoplasmic male marker data set with parametric Fst=";
	title_.cat(Fst); 
	npop_ = numIndivsPerPopLoc.len();
	ngroups_ = 1;
	traitName_.Append(NEW aString("ID"));
	int locusCount=0;
	traitNum_.Append(locusCount++);
	if(allPresentInMalesOnly_.len()>0){
		traitName_.Append(NEW aString("sex"));
		traitNum_.Append(locusCount++);
		sexTrait_=1;
		}
	for(loc = 0;loc<nlociCYmale_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',Digits(nlociCYmale_));
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(locusCount++);
		allCYmaleLoci_.Append(loc+1);
		locusNumCYmale_.Append(loc+1);
		}
	ntraits_ = traitNum_.len();


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();

	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	ObjList<ObjList<StatList<long double> > > frequenciesByPopLocusAllele;
	for(pop=0;pop<npop_;++pop){
		frequenciesByPopLocusAllele.Append(NEW ObjList<StatList<long double> >());}
	for(loc=0;loc<nlociCYmale_;++loc){
		if(pBar[loc].len()>1){//locus is variable
				GeneFrequencyDistMultiAllele parametricFreqDistrib(Fst, pBar[loc], 1);//new version sends ploidy
				for(pop=0;pop<npop_;++pop){
					aList<long double>* pp = parametricFreqDistrib.RandomFrequencies(ran);
					frequenciesByPopLocusAllele[pop].Append(NEW StatList<long double>(*pp));//copy as StatList
					delete pp;pp=NULL;
					}//end for pop
				}
			else{//locus is monomorphic
				for(pop=0;pop<npop_;++pop){
					frequenciesByPopLocusAllele[pop].Append(NEW StatList<long double>(pBar[loc]));//copy as StatList
					}//end for pop
				}				
		}//end for loc

	int maxIndivPop;
	aString *s, indiv, pName;
	char genotype;
	long gtypeVal;
	aList<int> female1male0;
	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		pName = "randomly generated population # ";
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		s = NEW aString(pName);
		popName_.Append(s);//append a copy

		maxIndivPop = 0;
		for (loc=0;loc<numIndivsPerPopLoc[pop].len();++loc){
			maxIndivPop = MAX(maxIndivPop,numIndivsPerPopLoc[pop][loc]);}

		//create id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
		for (i=0;i<maxIndivPop;++i){
			rawTraits_[pop].Append(NEW TextDataList());
			indiv = i+1;//first create id numbers for each individual
			indiv.PrependCharToLen(' ', MAX(4,Digits(maxIndivPop)));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));
			female1male0.Append(0);
			if(presentInMalesOnly_.len()>0){
				rawTraits_[pop][i].Append(NEW TextDatum('M'));
				if(ran()<sr){
					female1male0[i]=1;
					rawTraits_[pop][i].Last()='F';}}		
			}//end for i
		popSize_.Append(maxIndivPop);

		for(loc=0;loc<nlociCYmale_;++loc){
			StatList<long double>& pp = frequenciesByPopLocusAllele[pop][loc];
			if(pp.len()>1){//sampling required
					for(i=0;i<numIndivsPerPopLoc[pop][loc];++i){
						if(presentInMalesOnly_.Includes(loc) && female1male0[i]==1){
								rawTraits_[pop][i].Append(NEW TextDatum());}
							else{
								gtypeVal = ran.weighted(pp);
								genotype = char('A' + (int)gtypeVal);
								rawTraits_[pop][i].Append(NEW TextDatum(genotype));
								}
						}//end for i
					for(i=numIndivsPerPopLoc[pop][loc];i<maxIndivPop;++i){//unsampled individuals blank at this locus
						rawTraits_[pop][i].Append(NEW TextDatum());//adds missing data
						}//end for i
					}
				else{//monomorphic locus
					for(i=0;i<numIndivsPerPopLoc[pop][loc];++i){
						if(presentInMalesOnly_.Includes(loc) && female1male0[i]==1){
								rawTraits_[pop][i].Append(NEW TextDatum());}//missing
							else{
								rawTraits_[pop][i].Append(NEW TextDatum("A"));}
						}//end for i
					for(i=numIndivsPerPopLoc[pop][loc];i<maxIndivPop;++i){//unsampled individuals blank at this locus
						rawTraits_[pop][i].Append(NEW TextDatum());//add missing data
						}//end for i
					}					
			}//end for loc
		}//end for pop	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end CytoplasmicMaleMarkerData::CreateRandomData(...)




void CytoplasmicMaleMarkerData::CullCYmaleLoci (StringList& lociToCull){
	//This function culls the designated loci from each genotype data string
	//and includes them among the skipped loci
	if(lociToCull.len()==0){return;}//none to cull
	aList<int> cullListByNum;
	int tr, item;
	for(tr=0;tr<lociToCull.len();++tr){
		aString cTraitname = lociToCull[tr];
		item = traitName_.Loc(&cTraitname);
		if((item>=0)&&(!(skippedTraitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already skipped
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	CullCYmaleLoci(cullListByNum);
	}//end procedure CytoplasmicMaleMarkerData::CullCYmaleLoci(StringList&)


void CytoplasmicMaleMarkerData::CullCYmaleLoci (aList<int>& lociToCull){
	//This function shifts trait numbers from from traitNums_ to the skippedTraitNums_ list
	if(lociToCull.len()==0){return;}//none to cull
	int tr,cLoc;
	long item;
	for(tr=0;tr<lociToCull.len();++tr){
		cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(traitNum_.Includes(cLoc) && cLoc>=0 && cLoc<traitName_.len()){
			item = traitNum_.Loc(cLoc);
			skippedTraitNum_.Append(cLoc);
			skippedLocusNumCYmale_.Append(cLoc);
			traitNum_.Delete(item);
			item = locusNumCYmale_.Loc(cLoc);
			locusNumCYmale_.Delete(item);
			if(presentInMalesOnly_.Includes(cLoc)){
				item = presentInMalesOnly_.Loc(cLoc);
				presentInMalesOnly_.Delete(item);}
			if(presentInBothSexes_.Includes(cLoc)){
				item = presentInBothSexes_.Loc(cLoc);
				presentInBothSexes_.Delete(item);}
			}//if item not skipped already
		}//tr
	nlociCYmale_ = locusNumCYmale_.len();
	skippedTraitNum_.Sort();
	skippedLocusNumCYmale_.Sort();
	ResetFlags();
	CheckForNewAlleles();
	}//CytoplasmicMaleMarkerData::CullCYmaleLoci (aList<int>&)



void CytoplasmicMaleMarkerData::CullCYmaleLocus (char* locusToCull){
	aString locToCull(locusToCull);
	CullCYmaleLocus(locToCull);
	}//end CytoplasmicMaleMarkerData::CullCYmaleLocus (char* )


void CytoplasmicMaleMarkerData::CullCYmaleLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	CullCYmaleLoci(lociToCull);
	}//end CytoplasmicMaleMarkerData::CullCYmaleLocus (aString& )


void CytoplasmicMaleMarkerData::CullToOneSex (CytoplasmicMaleMarkerData& parentData,genderSet sexVal){
	//makes a subset data structure from the specified group
	aList<int> sexValList;
	sexValList.Append((int)sexVal);
	CullToTheseSexes(parentData,sexValList);
	}//end CytoplasmicMaleMarkerData::CullToOneSex (CytoplasmicMaleMarkerData& , genderSet )


void CytoplasmicMaleMarkerData::CullToTheseSexes (CytoplasmicMaleMarkerData& parentData,aList<int>& sexValList){
	//makes a subset data structure from the specified group
	Clear();
	if(sexValList.len()==0) return;
	title_ = parentData.title_;
	title_.cat(" -- ");
	for(int s=0;s<sexValList.len();++s){
		genderSet sexVal = (genderSet) sexValList[s];
		switch(sexVal){
			case A:
				title_.cat("asexuals"); break;
			case B:
				title_.cat("hermaphrodites"); break;
			case F:
				title_.cat("females"); break;
			case M:
				title_.cat("males"); break;
			default: break;
			}
		if(s<sexValList.len()-2){
				title_.cat(", ");}
			else if (s == sexValList.len()-2){
				title_.cat(" & ");}
			else{}
		}
	title_.cat(" only");


	npop_ = parentData.npop_;
	ngroups_ = parentData.ngroups_;
	ntraits_ = parentData.ntraits_;
	groupName_ = parentData.groupName_;
	localityName_ = parentData.localityName_;
	traitName_ = parentData.traitName_;
	traitNum_ = parentData.traitNum_;
	skippedTraitNum_ = parentData.skippedTraitNum_;
	missing_ = parentData.missing_;
	popName_ = parentData.popName_;
	localityCoordinates_ = parentData.localityCoordinates_;
	missing_ = parentData.missing_;
	
	heterogameticSex_ = parentData.heterogameticSex_;
	sexTrait_ = parentData.sexTrait_;
	primarySexRatio_ = parentData.primarySexRatio_;
	
	nlociCYmale_ = parentData.nlociCYmale_;
	locusNumCYmale_ = parentData.locusNumCYmale_;
	allCYmaleLoci_ = parentData.allCYmaleLoci_;
	skippedLocusNumCYmale_ = parentData.skippedLocusNumCYmale_;
	if(sexValList.Includes((int)M)){
		allPresentInMalesOnly_ = parentData.allPresentInMalesOnly_;
		presentInMalesOnly_ = parentData.presentInMalesOnly_;
		allPresentInBothSexes_ = parentData.allPresentInBothSexes_;
		presentInBothSexes_ = parentData.presentInBothSexes_;
		}
		

	genderSet thisSexVal=B;
	int pop, i, thisSexValInt;
	for(pop=0;pop<parentData.npop_;++pop){
		popSize_.Append(0);
		rawTraits_.Append(NEW ObjList<TextDataList>());
		for(i=0;i<parentData.popSize(pop);++i){
			TextDatum& thisSex = parentData.rawTraits_[pop][i][sexTrait_];
			thisSexVal = genderCode(thisSex);
			thisSexValInt = (int)thisSexVal;
			if(sexValList.Includes(thisSexValInt)){
				rawTraits_[pop].Append((TextDataList*)parentData.rawTraits_[pop][i].Duplicate());
				popSize_[pop]++;}
			}//i
		}//pop
	
	//cull empty populations from data
	for(pop=npop_-1;pop>=0;--pop){
		if(parentData.rawTraits_[pop].len()==0){
			parentData.rawTraits_.Delete(pop);
			popName_.Delete(pop);
			popSize_.Delete(pop);
			}
		}//pop
	npop_ = rawTraits_.len();
		
	ResetFlags();
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();
	}//end CytoplasmicMaleMarkerData::CullToTheseSexes (CytoplasmicMaleMarkerData& ,aList<int>& )


void CytoplasmicMaleMarkerData::CullMonomorphicLoci (void){	
	//This function culls the monomorphic loci from each genotype data string
	//and includes them among the skipped loci
	UpdateVariableLociCYmale();
	if(variableLocusNumCYmale_.len()==locusNumCYmale_.len()){return;}//none monomorphic
	aList<int> monomorphicLoci(locusNumCYmale_);
	for(int vLoc=0; vLoc<variableLocusNumCYmale_.len();++vLoc){
		if(monomorphicLoci.Includes(variableLocusNumCYmale_[vLoc])){//polymorphic
			int item = monomorphicLoci.Loc(variableLocusNumCYmale_[vLoc]);
			monomorphicLoci.Delete(item);}
		}//vLoc
	CullCYmaleLoci(monomorphicLoci);
	}//end procedure CytoplasmicMaleMarkerData::CullMonomorphicLoci


void CytoplasmicMaleMarkerData::ExpungeLoci (StringList& lociToCull){
	if(lociToCull.len()==0){return;}//none to cull
	aList<int> cullListByNum;
	int tr,item;
	for(tr=0;tr<lociToCull.len();++tr){
		aString cTraitname = lociToCull[tr];
		item = traitName_.Loc(&cTraitname);
		if(item>=0){//the trait exists in the rawlist
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	ExpungeLoci(cullListByNum);
	}//end procedure CytoplasmicMaleMarkerData::ExpungeLoci(StringList&)


void CytoplasmicMaleMarkerData::ExpungeLoci (aList<int>& lociToCull){
	//This function removes trait numbers from the CYmale data set; called during reclassification
	if(lociToCull.len()==0){return;}//none to cull
	long item, tr, cLoc;
	for(tr=0;tr<lociToCull.len();++tr){
		cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(allCYmaleLoci_.Includes(cLoc)){
			item = allCYmaleLoci_.Loc(cLoc);
			allCYmaleLoci_.Delete(item);}
		if(locusNumCYmale_.Includes(cLoc)){
			item = locusNumCYmale_.Loc(cLoc);
			locusNumCYmale_.Delete(item);}
		if(skippedLocusNumCYmale_.Includes(cLoc)){
			item = skippedLocusNumCYmale_.Loc(cLoc);
			skippedLocusNumCYmale_.Delete(item);}
		if(allPresentInMalesOnly_.Includes(cLoc)){
			item = allPresentInMalesOnly_.Loc(cLoc);
			allPresentInMalesOnly_.Delete(item);}
		if(presentInMalesOnly_.Includes(cLoc)){
			item = presentInMalesOnly_.Loc(cLoc);
			presentInMalesOnly_.Delete(item);}
		if(allPresentInBothSexes_.Includes(cLoc)){
			item = allPresentInBothSexes_.Loc(cLoc);
			allPresentInBothSexes_.Delete(item);}
		if(presentInBothSexes_.Includes(cLoc)){
			item = presentInBothSexes_.Loc(cLoc);
			presentInBothSexes_.Delete(item);}
		}//tr
	nlociCYmale_ = locusNumCYmale_.len();
	ResetFlags();
	CytoplasmicMaleMarkerData::CheckForNewAlleles();
	}//CytoplasmicMaleMarkerData::ExpungeLoci (aList<int>&)



void CytoplasmicMaleMarkerData::ExpungeLocus (char* locusToCull){
	aString locToCull(locusToCull);
	ExpungeLocus(locToCull);
	}//end CytoplasmicMaleMarkerData::ExpungeLocus (char* )


void CytoplasmicMaleMarkerData::ExpungeLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	ExpungeLoci(lociToCull);
	}//end CytoplasmicMaleMarkerData::ExpungeLocus (aString& )


long double CytoplasmicMaleMarkerData::GetExpectedHeterozygosity (int pop, int loc, int allele){
	long double freq = GetFreqCYmale(pop,loc,allele);
	return freq * ((long double)1.0 - freq);
	}//CytoplasmicMaleMarkerData::GetExpectedHeterozygosity (int, int, int)
	
	
long double CytoplasmicMaleMarkerData::GetExpectedHeterozygosity (int pop, int loc){
	long double freq, homosum=(long double)0.0;
	for(int allele=0;allele<allelesPresentCYmale_[loc].len();++allele){
		freq = GetFreqCYmale(pop,loc,allele);
		homosum += freq*freq;}
	return ((long double)1.0 - homosum);
	}//CytoplasmicMaleMarkerData::GetExpectedHeterozygosity (int, int)




long double CytoplasmicMaleMarkerData::GetFreqCYmale (int loc, int allele){
	long double numAlleleCopies = 0.0;
	long double numThisAllele = 0.0;
	for(int pop=0;pop<npop_;++pop){
		long double n = (long double)GetLocusSampleSizeCYmale(pop,loc);
		numAlleleCopies += n;
		numThisAllele += n * GetFreqCYmale(pop,loc,allele);
		}
	if(numAlleleCopies==0.0){return (long double)0.0;}
	return numThisAllele/numAlleleCopies;
	}//end CytoplasmicMaleMarkerData::GetFreqCYmale


long double CytoplasmicMaleMarkerData::GetFreqCYmale (int pop, int loc, int marker){
	if (!genotypeFreqsCurrentCYmale_){CalculateGenotypeFreqs();}
	return genotypeFreqCYmale_[pop][loc][marker];
	}//end CytoplasmicMaleMarkerData::GetFreqCYmale


long double CytoplasmicMaleMarkerData::GetFreqMostCommonAllele (int pop, int loc){
	long double mostCommon = 0.0;
	for(int allele=0;allele<GetNumAllelesCYmale(loc);++allele){
		long double freq = GetFreqCYmale(pop,loc,allele);
		if (freq>(long double)0.5) {return freq;}
		if (mostCommon<freq){mostCommon = freq;}
		}//and for allele
	return mostCommon;
	}//end CytoplasmicMaleMarkerData::GetFreqMostCommonAllele


void CytoplasmicMaleMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
	//uploads data to pass to genetic analysis classes
	pgcList.Clear();
	int pop,i, dt;
	//initialize
	aList<int> lociByType;
	for(dt=0;dt<MultiInheritanceModeContainer<int>::inheritanceModes_;++dt){
		lociByType.Append(0);}
	lociByType[MultiInheritanceModeContainer<int>::cytoMale]=nlociCYmale_;
	for(pop=0;pop<npop_;++pop){
		pgcList.Append(NEW PopulationGeneContainer<int>(popSize(pop),lociByType));}
	
	//fill up
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<popSize(pop);++i){
			GetGenome(pgcList[pop][i], pop, i);}
		}
	}//CytoplasmicMaleMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >&)



void CytoplasmicMaleMarkerData::GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID){
	//uploads this indiv's data to HaplotypeContainer<int>[locus]
	HaplotypeContainer<int>* hc = (HaplotypeContainer<int>*) &gc;
	hc->SetSize(CytoplasmicMaleMarkerData::loci());
	if(setID){
		hc->SetID(1+GetIndivNumInDataSequence(pop,indiv));}
	for(int loc=0;loc<nlociCYmale_;++loc){
		if(rawTraits_[pop][indiv][locusNumCYmale_[loc]].missing()){
				hc->Clear(); return;}
			else{
				hc->setHaplotype(loc,CytoplasmicMaleMarkerData::alleleID(pop,indiv,loc));}
		}	
	}//CytoplasmicMaleMarkerData::GetGenome(GeneContainerCore&, int, int)


long CytoplasmicMaleMarkerData::GetIndividualsSampledCYmale (int pop, int loc){
	// loc refers to cytoplasmic male marker loci only
	long n=0;
	for(long i=0;i<individuals(pop);++i){
		if(GetLocusSampleSizeCYmale (pop, i, loc)>0){//checks for correct sex in the called function
			++n;}}
	return n;
	}//end  CytoplasmicMaleMarkerData::GetIndividualsSampledCYmale


long CytoplasmicMaleMarkerData::GetLocusSampleSizeCYmale (int loc){
	if(! genotypeFreqsCurrentCYmale_){
		CalculateGenotypeFreqs();}//calculates twoN_
	double numMarkersFound = 0.0;
	for(int pop=0;pop<npop_;++pop){
		numMarkersFound += nByLocusCYmale_[pop][loc];
		}
	return numMarkersFound;
	}//end  CytoplasmicMaleMarkerData::GetLocusSampleSizeCYmale


long CytoplasmicMaleMarkerData::GetLocusSampleSizeCYmale (int pop, int loc){
	// loc refers to sex-linkeddominant loci only
	return nByLocusCYmale_[pop][loc];
	}//end  CytoplasmicMaleMarkerData::GetLocusSampleSizeCYmale


long CytoplasmicMaleMarkerData::GetLocusSampleSizeCYmale (int pop, int indiv, int loc){
	// loc refers to dominant loci only
	if(isFemale(pop,indiv)) {return 0;}
	TextDatum& score = GenotypicVal(pop,indiv,loc,0);
	return (long) score.OK();//false=0 if missing, true=1 if present
	}//end  CytoplasmicMaleMarkerData::GetLocusSampleSizeCYmale



int CytoplasmicMaleMarkerData::GetNumAllelesCYmale (int loc){
	// loc refers to dominant loci only
	//total in data set
	if(! allelesCurrentCYmale_){CheckForNewAlleles();}
	return allelesPresentCYmale_[loc].len();
	}//end  CytoplasmicMaleMarkerData::GetNumAllelesCYmale(loc)


int CytoplasmicMaleMarkerData::GetNumAllelesCYmale (int pop, int loc){
	// loc refers to cytoplasmic male marker loci only
	//extant alleles in population
	if(! genotypeFreqsCurrentCYmale_){CalculateGenotypeFreqs();}
	aList<long double>& locList = genotypeFreqCYmale_[pop][loc];
	int count=0;
	for(int a=0;a<locList.len();++a){
		if (locList[a] >zero){++count;}}
	return count;
	}//end  CytoplasmicMaleMarkerData::GetNumAllelesCYmale(pop,loc)

aString* CytoplasmicMaleMarkerData::GetTraitString(int pop, int indiv, int loc){
	//loc indexed as in locusNumCYmale_ 
	aString *aStr = NEW aString();
	int tr = locusNumCYmale_[loc];
	aStr->cat(rawTraits_[pop][indiv][tr][0]);
	return aStr;
	}//CytoplasmicMaleMarkerData::GetTraitString


aString* CytoplasmicMaleMarkerData::GetTraitString(int pop, int indiv, int loc, int /*allele*/){
	return GetTraitString(pop, indiv, loc);
	}//CytoplasmicMaleMarkerData::GetTraitString



/*
int CytoplasmicMaleMarkerData::individuals(int pop, int loc){//returns males only
	int indivs=0;
	for(int i=0;i<individuals(pop);++i){
		if (GetLocusSampleSizeCYmale ( pop, i, loc)>0){++indivs;}}
	return indivs;
	}//end  CytoplasmicMaleMarkerData::individuals(int,int)
*/


long CytoplasmicMaleMarkerData::maxall(void){
	//maximum alleles per locus in data set
	CheckForNewAlleles();
	long maxalleles=0;
	int loc;
	for(loc=0;loc<nlociCYmale_;++loc){
		maxalleles = MAX(maxalleles,allelesPresentCYmale_[loc].len());}
	return maxalleles;
	}//CytoplasmicMaleMarkerData::maxall()




/*
void  EvolutionaryGeneticsData::PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of allelic frequencies for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculateAlleleFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied


	outFile<<title<<endl<<endl<<endl;
	outFile<<"ALLELE FREQUENCY DATA"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop<<endl;
	outFile<<"NUMBER OF LOCI SURVEYED = ";outFile.width(3);outFile<<nloci<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (lociSkipped == 0){
			outFile<<"none";}
		else{
			for (int loc=0;loc< lociSkipped;++loc){
				aString& locName = skippedLocusName[loc];
				outFile<<locName;
				if (loc < lociSkipped-1){outFile<<", ";}
				}//end for loc
		}//end if lociSkipped
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop / ncols;
	int npop0 = npop;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Allele frequencies for each population."<<endl;
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;

		for (int loc=0;loc<nloci;++loc) {//print the locus and locus sample size
			aString& locWord = locusName[loc];
			outFile<< locWord;
			if (asciiFormat){outFile.width(11 + (5 - locWord.len()));}else{outFile<<colSpacer;}
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = GetLocusSampleSize(pop, loc) / 2;
				outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
				if (pop < endPop){
					if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
					}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = GetFreq(pop, loc, allele);
					aString freqStr(freq,0,4);
					if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									if (asciiFormat){
											aString blanks(' ',12);
											if (pop==x) {
												blanks.PrependCharToLen(' ',17);}
											outFile<<blanks;
											}
										else{outFile<<colSpacer;}
									}
								else{
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end EvolutionaryGeneticsData::PrintAlleleFreqs




void  EvolutionaryGeneticsData::PrintAlleleFreqsByPopList (ostream& outFile, 
											ObjList<ObjList<aList<double> > >& popFreqList, 
											ObjList<aList<double> >& popTwoNList, 
											int ncols){
	//popFreqList must contain the entire set of alleles in the data set
	//Sends a table of allelic frequencies to the output file.
	//ncols indexes the number of population columns per page

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = popFreqList.len() / ncols;
	int npop0 = popFreqList.len();
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)popFreqList.len(), (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			outFile.width(12);outFile<<pop+1;}
		outFile<<endl;

		for (int loc=0;loc<nloci;++loc) {//print the locus and locus sample size
			aString& locWord = locusName[loc];
			outFile<< locWord;outFile.width(11 + (5 - locWord.len()));
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = popTwoNList[pop][loc] / 2;
				outFile<<"[";outFile.width(2);outFile<<samSize<<"]";
				if (pop < endPop){outFile.width(9);}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = popFreqList[pop][loc][allele];
					aString freqStr(freq,0,4);
					freqStr.PrependCharToLen(' ',12);
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									aString blanks(' ',12);
									if (pop==x) {
										blanks.PrependCharToLen(' ',17);}
									outFile<<blanks;}
								else{
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end EvolutionaryGeneticsData::PrintAlleleFreqsByPopList
*/


void	 CytoplasmicMaleMarkerData::PrintCYmaleMarkerFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintCYmaleMarkerFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end CytoplasmicMaleMarkerData::PrintCYmaleMarkerFile(aString&)



void CytoplasmicMaleMarkerData::PrintCYmaleMarkerFile (ostream& outFile){
	aString dataLine = title_;
	dataLine.cat(":  skipped loci omitted");
	outFile<<dataLine<<endl;
	dataLine = "NPOP=";
	dataLine.cat(npop_);
	dataLine.cat(", NLOCI=");
	dataLine.cat(nlociCYmale_);
	long maxall_=0;
	for(int loc=0;loc<allelesPresentCYmale_.len();++loc){
		maxall_=MAX(maxall_,allelesPresentCYmale_[loc].len());}
	dataLine.cat(", MAXALL=");
	dataLine.cat(maxall_);
	dataLine.cat(", NGROUPS=");
	dataLine.cat(ngroups_);
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int loc=0;loc<nlociCYmale_;++loc){
		dataLine.cat(locusNameCYmale(loc));
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < nlociCYmale_){
				dataLine.cat("\t");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	dataLine = "SKIP=0";
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int pop=0;pop<npop_;++pop){
		outFile<<popName_[pop]<<endl;
		//cobble together a phenotype list for cytoplasmic male markers only
		aList<int> *malesPtr=0;
		if(heterogameticSex()==M){
				malesPtr = &heterogameticIndivs_[pop];}
			else{
				malesPtr = &homogameticIndivs_[pop];}
		aList<int>& males = *malesPtr;

		for (int i=0;i<males.len();++i){
			StringList idata;
			idata.Append(NEW TextDatum(rawTraits_[pop][males[i]][0]));
			for(int dloc=0;dloc<nlociCYmale_;++dloc){
				TextDatum& score=rawTraits_[pop][males[i]][locusNumCYmale_[dloc]];
				if(score.OK()){
						idata.Append(NEW aString(score));}
					else{//record as missing data
						idata.Append(NEW aString(rawTraits_[pop][males[i]].missingDataStr_));}
				}//dLoc
			aString outStr;
			for(int j=0;j<idata.len();++j){
				outStr+idata[j];
				if(j<idata.len()-1){
					outStr+" ";}}
			outFile<<outStr<<endl;
			}//end for i
		outFile<<"NEXT"<<endl;
		}//end for pop
	outFile<<"END"<<endl;
	}//end CytoplasmicMaleMarkerData::PrintCYmaleMarkerFile(ostream&)



void	 CytoplasmicMaleMarkerData::PrintNEXUSFile(aString& outFileName, StringList& textToAppend){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintNEXUSFile(dataFile,textToAppend);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end CytoplasmicMaleMarkerData::PrintNEXUSFile(aString&,StringList&)



void CytoplasmicMaleMarkerData::PrintNEXUSFile (ostream& outFile, StringList& textToAppend){
	outFile<<"#NEXUS"<<endl;
	aString dataLine = title_;
	dataLine.prepend("[ ! ");
	outFile<<dataLine<<endl<<"]"<<endl<<endl;
	dataLine = "begin alleles;";
	outFile<<dataLine<<endl;
	dataLine = "\tdimensions newpops nloci=";
	dataLine.cat(nlociCYmale_);
	dataLine.cat(" npops=");
	dataLine.cat(npop_);
	dataLine.cat(";");
	outFile<<dataLine<<endl;
	dataLine = "\tdominant all:1;";
	outFile<<dataLine<<endl;
	dataLine = "\tformat labels missing=? ;";
	outFile<<dataLine<<endl;
	dataLine = "\tmatrix";
	outFile<<dataLine<<endl;
	dataLine.clear();
	/* need code for locus names
	for (int loc=0;loc<nlociCYmale_;++loc){
		dataLine.cat(locusNameDom_(loc));
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < nlociCYmale_){
				dataLine.cat("\t");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	*/
	aString endCharList("\t, ");
	for (int pop=0;pop<npop_;++pop){
		aString pname = popName_[pop];
		pname.replaceAll(' ','_');
		outFile<<'\t'<<pname<<":"<<endl;
		aList<int> *malesPtr=0;
		if(heterogameticSex()==M){
				malesPtr = &heterogameticIndivs_[pop];}
			else{
				malesPtr = &homogameticIndivs_[pop];}
		aList<int>& males = *malesPtr;

		for (int i=0;i<males.len();++i){
			StringList idata;
			idata.Append(NEW TextDatum(rawTraits_[pop][males[i]][0]));
			idata[0].PrependCharToLen('0',3);//tweak the format of the ID code
			idata[0].prepend('_');
			idata[0].cat("_\t");
			for(int dloc=0;dloc<nlociCYmale_;++dloc){
				TextDatum& score=rawTraits_[pop][males[i]][locusNumCYmale_[dloc]];
				if(score.OK()){
						idata.Append(NEW aString(score));}
					else{//record as missing data
						idata.Append(NEW aString("?"));}
				}//dLoc
			aString outStr;
			for(int j=0;j<idata.len();++j){
				outStr+idata[j];
				if(j<idata.len()-1){
					outStr+"\t";}}
			outFile<<"\t\t"<<outStr<<endl;

			}//end for i
		if(pop==npop_-1){
				outFile<<";"<<endl;}
			else{
				outFile<<","<<endl;}
		}//end for pop
	outFile<<"end;"<<endl<<endl;
	
	for(int i=0;i<textToAppend.len();++i){
		outFile<<textToAppend[i]<<endl;}
	outFile<<endl;
	}//end CytoplasmicMaleMarkerData::PrintNEXUSFile(ostream&, StringList&)



void CytoplasmicMaleMarkerData::PrintLociCYmale (ostream& outFile, int lociPerLine){//file assumed open
	int lineSpot = 0;
	for (int loc=0;loc<nlociCYmale_;++loc){ 
		aString& locName = locusNameCYmale(loc);
		outFile<<locName;
		++lineSpot;
		if ((loc == nlociCYmale_-1) || (lineSpot == lociPerLine)) {
				outFile<<endl;}
			else{
				outFile<<"\t";}
		}//end for loc
	outFile<<endl;
	}//end CytoplasmicMaleMarkerData::PrintLociCYmale


void CytoplasmicMaleMarkerData::PrintGenotypeFreqs(aString& outFileName, int ncols, char colSpacer){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintGenotypeFreqs(dataFile, ncols, colSpacer);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end CytoplasmicMaleMarkerData::PrintGenotypeFreqs(aString&, int, char)



void  CytoplasmicMaleMarkerData::PrintGenotypeFreqs (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of phenotypic frequencies for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculateGenotypeFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied
	int significantDigits = 4;

	outFile<<title_<<endl<<endl<<endl;
	outFile<<"ALLELE FREQUENCY DATA"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop_<<endl;
	outFile<<"NUMBER OF MARKERS SURVEYED = ";outFile.width(3);outFile<<nlociCYmale_<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (skippedLocusNumCYmale_.len() == 0){
			outFile<<colSpacer<<"none";}
		else{
			for (int loc=0;loc< skippedLocusNumCYmale_.len();++loc){
				aString& locName = traitName_[skippedLocusNumCYmale_[loc]];
				outFile<<locName;
				if(MOD(loc+1,20)==0){outFile<<endl;}
					else if (loc < skippedLocusNumCYmale_.len()-1){outFile<<colSpacer;}
					else{}
				}//end for loc
		}//end if lociSkippedDom
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop_ / ncols;
	int npop0 = npop_;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Allele frequencies of CYTOPLASMICALLY INHERITED MALE markers for each population."<<endl;
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop_, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;

		for (int loc=0;loc<nlociCYmale_;++loc) {//print the locus and locus sample size
			aString& locWord = locusNameCYmale(loc);
			outFile<< locWord;
			if (asciiFormat){outFile.width(11 + (5 - locWord.len()));}else{outFile<<colSpacer;}
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = nByLocusCYmale_[pop][loc];
				outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
				if (pop < endPop){
					if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
					}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& markerNameList = allelesPresentCYmale_[loc];
			for (int marker=0; marker<markerNameList.len();++marker){
				aString& markerName=markerNameList[marker];
				aString markerNameCopy(markerName);
				markerNameCopy.PrependCharToLen(' ',3);
				outFile<<markerNameCopy;
				// print the frequency of the marker for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = genotypeFreqCYmale_[pop][loc][marker];
					aString freqStr(freq,0,significantDigits);
					if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
										}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									if (asciiFormat){
											aString blanks(' ',12);
											if (pop==x) {
												blanks.PrependCharToLen(' ',17);}
											outFile<<blanks;
											}
										else{outFile<<colSpacer;}
									}
								else{
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end EvolutionaryGeneticsData::PrintGenotypeFreqs




void CytoplasmicMaleMarkerData::ReadAFLPDataFile (aString& dataFileName){
	Clear();
	fstream dataFile;
	aString endCharList(" \t,\r"), missingDataListElems("?. -_");
	char endchar = '\n';	
	StringList missingDataList;
	for(int s=0;s<missingDataListElems.len();++s){
		missingDataList.Append(NEW aString(missingDataListElems[s]));}
	char *dataLine=NEW char[10001];dataLine[0]='\0';
	char* filename=dataFileName.raw();
	dataFile.open(filename,ios::in);

	dataFile.getline(dataLine, 10000,endchar);
	title_ = dataLine;

	dataFile.getline(dataLine, 10000,endchar); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	npop_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 6);
	nlociCYmale_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 7);
	int maxall_ = paramWord->toInt();//now ignored
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	ngroups_ =  paramWord->toInt();
	delete paramWord; paramWord=0;


	//get locus information
	int moreLoci = 1;//true
	long loc = 0;
	StringList rawLocusNameCYmale,skippedLocusNameCYmale;//temporary
	while (moreLoci){
		dataFile.getline(dataLine, 10000,endchar); 
		aString locusLine(dataLine);
		while (locusLine.len() > 0){
			++loc;
			aString *locName = locusLine.ChopOffFirstWord(endCharList);
			if(locName->len()>0){//there could be trailing blanks on locusLine
				aString *s = NEW aString(locName);
				rawLocusNameCYmale.Append(s);}//append a copy
			delete locName; locName=0;
			}//end while locusLine.len()
		if (loc >= nlociCYmale_){//data may be on more than one line
			moreLoci = 0;}//false
		}//end while moreLoci
	//check for duplicates	
	StringList uniqueNames;
	for(long i=0;i<rawLocusNameCYmale.len();++i){
		if(!(uniqueNames.Includes(&(rawLocusNameCYmale[i])))){
			uniqueNames.Append(NEW aString(rawLocusNameCYmale[i]));}}

	long numuni=uniqueNames.len();
	if(rawLocusNameCYmale.len()>numuni){
		//figure out the names and positions of duplicated loci & report
		StringList dupNames(rawLocusNameCYmale);
		for(loc=0;loc<uniqueNames.len();++loc){
			long uItem=dupNames.Loc(&(uniqueNames[loc]));
			dupNames.Delete(uItem);
			}//loc
		ObjList<aList<long> > dupPositions;
		for(loc=0;loc<dupNames.len();++loc){
			StringList locNamesCopy(rawLocusNameCYmale);//work with a copy
			dupPositions.Append(NEW aList<long>());
			long dupcount=0;
			long dupitem = locNamesCopy.Loc(&(dupNames[loc]));
			while (dupitem>=0){
				locNamesCopy.Delete(dupitem);
				dupPositions[loc].Append(dupitem+dupcount);//because the locNamesCopy list shrinks from the bottom
				++dupcount;
				dupitem = locNamesCopy.Loc(&(dupNames[loc]));
				}//while
			}//loc
		cout<<"You have duplicated locus names in your data set."<<endl;
		cout<<"Please check that these are not mistakes, and give all your loci unique names."<<endl;
		cout<<"duplicated loci:"<<endl;
		dataFile<<"You have duplicated locus names in your data set."<<endl;
		dataFile<<"Please check that these are not mistakes, and give all your loci unique names."<<endl;
		dataFile<<"duplicated loci:"<<endl;
		for(loc=0;loc<dupNames.len();++loc){
			aString dupLine(dupNames[loc]);
			dupLine.cat(" at positions ");
			for(int i=0;i<dupPositions[loc].len();++i){
				dupLine.cat(dupPositions[loc][i]+1);
				if(i<dupPositions[loc].len()-1){
					dupLine.cat(", ");}
				}//i
			cout<<dupLine<<endl;
			dataFile<<dupLine<<endl;}
		aError("The duplicated names are in your output file. Please edit your data file and start again from the beginning.");
		return;
		}//unique names
	//read in the number of loci to be skipped in the analyses
	dataFile.getline(dataLine, 10000,endchar); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	int lociSkippedCYmale = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped loci, if any
	loc = 0;
	moreLoci = 0;//false
	if (lociSkippedCYmale > 0){
			moreLoci = 1;}//true
	while (moreLoci){
		while (skipLine.len() > 0){
			++loc;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				aString *s = NEW aString(skipWord);
				skippedLocusNameCYmale.Append(s);//append a copy
				}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (loc >= lociSkippedCYmale){
				moreLoci = 0;}//false
			else	{//more loci on the next line
				dataFile.getline(dataLine, 10000,endchar); 
				skipLine = dataLine;}
		}//end while moreLoci

	traitName_=rawLocusNameCYmale;
	for(int tr=0;tr<traitName_.len();++tr){
		allCYmaleLoci_.Append(tr);}
	locusNumCYmale_=allCYmaleLoci_;
		
	//read in population data
	aString popLine;
	aString beast;
	ObjList<StringList> rawGenotypes;
	for (int pop=0;pop<npop_;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 10000,endchar); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		aString *s = NEW aString(popLine);
		popName_.Append(s);//append a copy

		//make a new sublist for the genotypes of this population
		rawGenotypes.Append(NEW StringList());
		// BEAST is the string for a particular individual
		beast.clear();
		while (beast != "NEXT"){ 
			dataFile.getline(dataLine, 10000,endchar); 
			beast = dataLine;
			beast.CullTrailingChar('\t');
			beast.CullTrailingChar(' ');
			beast.CullTrailingChar(',');
			if (beast != "NEXT"){ 
					aString *s = NEW aString(beast);
					rawGenotypes[pop].Append(s);
					s=0;}//appends a copy
			}//end while beast..else
		}//end for pop
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	//Now parse the data by trait
	for(int pop=0;pop<npop_;++pop){	
		rawTraits_.Append(NEW ObjList<TextDataList>());
		for(int i=0;i<rawGenotypes[pop].len();++i){
			aString dataStrCopy(rawGenotypes[pop][i]);
			if(dataStrCopy.len()==0) continue;
			rawTraits_[pop].Append(NEW TextDataList("?"));
			aString *codeWord = dataStrCopy.ChopOffFirstWord(endCharList);
			rawTraits_[pop][i].Append(NEW TextDatum(*codeWord));
			for(int loc=0;loc<nlociCYmale_;++loc){
				int posGtypeWord = 1 + ((alleleLen_+1) * loc);
				if(posGtypeWord>=dataStrCopy.len()){
						rawTraits_[pop][i].Append(NEW TextDatum("",true));
						continue;}
				aString gene = aString(dataStrCopy, posGtypeWord, posGtypeWord+alleleLen_-1);
				if(missingDataList.Includes(&gene)){
						rawTraits_[pop][i].Append(NEW TextDatum("",true));}
					else{
						rawTraits_[pop][i].Append(NEW TextDatum(gene));}
				}//loc
			delete codeWord;codeWord=0;
			}//i		
		}//pop		
	CullCYmaleLoci(skippedLocusNameCYmale);
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();
	}//end CytoplasmicMaleMarkerData::ReadAFLPDataFile


/*

void CytoplasmicMaleMarkerData::ReadAndAppendAFLPDataFile (aString& dataFileName){
	fstream dataFile;	
	char endchar = '\n';	
	char *dataLine=NEW char[10001];dataLine[0]='\0';
	aString endCharList(" \t,\r");
	char* filename=dataFileName.raw();
	dataFile.open(filename,ios::in);

	dataFile.getline(dataLine, 10000,endchar);
	title_.prepend("Combined data: [");
	title_.cat("] & [");
	title_.cat(dataLine);
	title_.cat("]");

	dataFile.getline(dataLine, 10000,endchar); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	int npopThisFile = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 6);
	int nlociDomThisFile = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 7);
	long maxallThisFile = paramWord->toInt();
	delete paramWord; paramWord=0;
	maxall = MAX(maxall, maxallThisFile);

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	int ngroupsThisFile =  paramWord->toInt();
	delete paramWord; paramWord=0;


	//get locus information
	int moreLoci = 1;//true
	int loc = 0;
	while (moreLoci){
		dataFile.getline(dataLine, 10000,endchar); 
		aString locusLine(dataLine);
		while (locusLine.len() > 0){
			++loc;
			aString *locName = locusLine.ChopOffFirstWord(endCharList);
			if(locName->len()>0){//there could be trailing blanks on locusLine
				if(!(rawLocusNameDom.Includes(locName))){//appended data can repeat loci
					aString *s = NEW aString(locName);
					rawLocusNameDom.Append(s);}//append a copy
				}
			delete locName; locName=0;
			}//end while locusLine.len()
		if (loc >= nlociDomThisFile){//data may be on more than one line
			moreLoci = 0;}//false
		}//end while moreLoci
	nlociCYmale_ = rawLocusNameDom.len();
	//read in the number of loci to be skipped in the analyses
	dataFile.getline(dataLine, 10000,endchar); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	int lociSkippedDomThisFile = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped loci, if any
	loc = 0;
	moreLoci = 0;//false
	if (lociSkippedDom > 0){
			moreLoci = 1;}//true
	while (moreLoci){
		while (skipLine.len() > 0){
			++loc;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				if(!(skippedLocusNameDom.Includes(skipWord))){
					aString *s = NEW aString(skipWord);//skipped loci can be repeated
					skippedLocusNameDom.Append(s);//append a copy
					int toSkip = rawLocusNameDom.Loc(s);
					skippedLocusNumDom.Append(toSkip);}
				}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (loc >= lociSkippedDom){
				moreLoci = 0;}//false
			else	{//more loci on the next line
				dataFile.getline(dataLine, 10000,endchar); 
				skipLine = dataLine;}
		}//end while moreLoci
	lociSkippedDom = skippedLocusNameDom.len();

	locusNameDom.Copy(rawLocusNameDom);
	for (loc=0;loc<skippedLocusNumDom.len();++loc){
		aString& toSkip = skippedLocusNameDom[loc];
		int location = locusNameDom.Loc(&toSkip);
		locusNameDom.Delete(location);
		}//end for loc
	nlociCYmale_ -= lociSkippedDom;

	//read in population data
	aString popLine;
	aString beast;
	for (int pop=0;pop<npopThisFile;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 10000,endchar); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		int thepop = pop;
		if(!(popName.Includes(&popLine))){
				aString *s = NEW aString(popLine);
				popName.Append(s);//append a copy
				thepop = popName.len()-1;//the last pop in list
				//make a new sublist for the genotypes of this population
				rawPhenotypes.Append(NEW StringList());				
				}
			else{
				thepop = popName.Loc(&popLine);}


		// BEAST is the string for a particular individual
		beast.clear();
		while (beast != "NEXT"){ 
			dataFile.getline(dataLine, 10000,endchar); 
			beast = dataLine;
			beast.CullTrailingChar('\t');
			beast.CullTrailingChar(' ');
			beast.CullTrailingChar(',');
			if (beast != "NEXT"){ 
				//check to see if this individual already has data
				aString beastCopy(beast);
				aString* IDbeast = beastCopy.ChopOffFirstWord(endCharList);
				int hasData	= 0, indivNum=0;
				for(int i=0;i<rawPhenotypes[thepop].len();++i){
					aString indivData(rawPhenotypes[thepop][i]);//copies
					aString *IDindiv = indivData.ChopOffFirstWord(endCharList);
					if(*IDbeast == *IDindiv){//already in data set
						hasData = 1;//true
						indivNum = i;
						delete IDindiv;IDindiv=0;
						break;//escape for i loop
						}
					delete IDindiv;IDindiv=0;
					}//end for i
				if(hasData){//append to end of this individual -- buggy if loci repeated
						beastCopy.prepend('\t');
						rawPhenotypes[thepop][indivNum].cat(beastCopy);
						}
					else{
						aString *s = NEW aString(beast);
						rawPhenotypes[thepop].Append(s);
						s=0;}//appends a copy
				delete IDbeast; IDbeast=0;
				}//end if beast != NEXT
			}//end while beast..else


		}//end for pop
	npop_ = rawPhenotypes.len();
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	ResetFlags();
	GroupMembership();
	LocalityMembership();
	CullSkippedLociDom();
	CheckForNewAlleles();
	}//end CytoplasmicMaleMarkerData::ReadAndAppendAFLPDataFile
*/




void CytoplasmicMaleMarkerData::RestoreLociCYmale (aList<int>& lociToRestore){
	//This function restores the designated loci to the analysis.
	//It shifts trait numbers from from skippedTraitNums_ to the traitNums_ list
	//and keeps track of the changes in the lists locusNumCYmale_, skippedLocusNumCYmale_ & presentInMalesOnly_
	if(lociToRestore.len()==0){return;}//none to cull
	ResetFlags();
	int tr,rLoc;
	long item;
	for(tr=0;tr<lociToRestore.len();++tr){
		rLoc = lociToRestore[tr];//make sure it's a valid locus to restore
		if(skippedTraitNum_.Includes(rLoc) && rLoc>=0 && rLoc<traitName_.len()){
			traitNum_.Append(rLoc);
			locusNumCYmale_.Append(rLoc);
			item = skippedTraitNum_.Loc(rLoc);
			skippedTraitNum_.Delete(item);
			item = skippedLocusNumCYmale_.Loc(rLoc);
			skippedLocusNumCYmale_.Delete(item);
			if(allPresentInMalesOnly_.Includes(rLoc) && ! presentInMalesOnly_.Includes(rLoc)){
				presentInMalesOnly_.Append(rLoc);}
			if(allPresentInBothSexes_.Includes(rLoc) && ! presentInBothSexes_.Includes(rLoc)){
				presentInBothSexes_.Append(rLoc);}
			}//if item not skipped already
		}//tr
	traitNum_.Sort();
	locusNumCYmale_.Sort();
	presentInMalesOnly_.Sort();
	CheckForNewAlleles();
	}//end  CytoplasmicMaleMarkerData::RestoreLociCYmale(aList<int>&)


void CytoplasmicMaleMarkerData::RestoreLociCYmale (StringList& lociToRestore){
	//This function restores the designated loci to the analysis
	if(lociToRestore.len()==0){return;}//none to cull
	//convert names to numbers and cull
	aList<int> restoreListByNum;
	for(int tr=0;tr<lociToRestore.len();++tr){
		aString rTraitname = lociToRestore[tr];
		int item = traitName_.Loc(&rTraitname);
		if((item>=0)&&(!(traitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already included
			restoreListByNum.Append(item);
			}//if item exists
		}//tr
	RestoreLociCYmale(restoreListByNum);
	}//end CytoplasmicMaleMarkerData::RestoreLociCYmale(StringList&)


void CytoplasmicMaleMarkerData::RestoreLocusCYmale (aString& locusToRestore){
	StringList lociToRestore;
	lociToRestore.Append(new aString(locusToRestore));
	RestoreLociCYmale(lociToRestore);
	}//end CytoplasmicMaleMarkerData::RestoreLocusCYmale (aString& )




void CytoplasmicMaleMarkerData::SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
	//records all data, passed along from genetic analysis classes
	//empties old data and creates new data structures
	Clear();
	title_ = "Simulated data set";
	npop_ = pgcList.len();
	nlociCYmale_ = pgcList[0].loci(MultiInheritanceModeContainer<int>::cytoMale);
	ngroups_ = 1;
	SetAlleleLength(1);//bug if maxall>9
	int runningTraitNum=0;
	int pop,i, loc;
	traitName_.Append(NEW aString("ID"));
	traitNum_.Append(runningTraitNum++);
	
	if(pgcList[0].sexLinkedDataIncluded()){
		traitName_.Append(NEW aString("sex"));
		traitNum_.Append(runningTraitNum++);
		SetSexTrait();		
		for(pop=0;pop<npop_;++pop){
			homogameticIndivs_.Append(NEW aList<int>());
			heterogameticIndivs_.Append(NEW aList<int>());}
		}
		
	int locusCount=0;
	for(loc = 0;loc<nlociCYmale_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',Digits(nlociCYmale_));
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(runningTraitNum++);
		allCYmaleLoci_.Append(loc+1);
		locusNumCYmale_.Append(loc+1);
		locusCount++;
		}
	ntraits_ = traitNum_.len();
	
	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("simulated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create dummy id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	

		for (i=0;i<pgcList[pop].popSize();++i){
			rawTraits_[pop].Append(NEW TextDataList());
			rawTraits_[pop][i].Append(NEW TextDatum());}//empty ID
		popSize_.Append(pgcList[pop].popSize());

		//create and append dummy genotypes
		for(i=0;i<pgcList[pop].popSize();++i){
			//determine sex, if sex-linked
			if(pgcList[pop].sexLinkedDataIncluded()){
			rawTraits_[pop][i].Append(NEW TextDatum(" "));}//empty sexTrait

			for(loc=0;loc<nlociCYmale_;++loc){
				rawTraits_[pop][i].Append(NEW TextDatum(" "));
				}//end for loc
			}//end for i
		}//end for pop	

	//Fill up with data
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<popSize(pop);++i){
			SetGenome(pgcList[pop][i], pop, i);}
		}
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();		
	}//CytoplasmicMaleMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >&)



void CytoplasmicMaleMarkerData::SetGenome(GeneContainerCore& gc, int pop, int indiv){
	//records data from HaplotypeContainer<int>
	//assumes blank data records already created in EvolutionaryGeneticsData
	HaplotypeContainer<int>* hc = (HaplotypeContainer<int>*) &gc;
	if(hc->loci() != nlociCYmale_){
		aError("ERROR: CytoplasmicMaleMarkerData::SetGenome(): size mismatch between data classes");
		return;}
	ResetFlags();
	rawTraits_[pop][indiv][0] = hc->ID();
	for(int loc=0;loc<nlociCYmale_;++loc){
		if(hc->missing()){//males may have missing data if sex limited
				rawTraits_[pop][indiv][locusNumCYmale_[loc]].clear();}
			else{
				rawTraits_[pop][indiv][locusNumCYmale_[loc]] = hc->haplotype(loc);}	
		}
	}//CytoplasmicMaleMarkerData::GetGenome(GeneContainerCore&, int, int)



void CytoplasmicMaleMarkerData::SetLoci(aList<int>& lociToInclude){
	if(lociToInclude.len()==0) return;
	CytoplasmicMaleMarkerData::ResetGeneticFlags();
	allCYmaleLoci_=lociToInclude;
	locusNumCYmale_=lociToInclude;
	aList<int> malesOnly;
	int tr,cLoc;
	long item;
	for(tr=0;tr<lociToInclude.len();++tr){
		cLoc = lociToInclude[tr];
		if(skippedTraitNum_.Includes(cLoc)){
			item = locusNumCYmale_.Loc(cLoc);
			if(item>=0){
				locusNumCYmale_.Delete(item);}
			if(! skippedLocusNumCYmale_.Includes(cLoc)){
				skippedLocusNumCYmale_.Append(cLoc);}
			}
		if(allPresentInMalesOnly_.Includes(cLoc)){
			malesOnly.Append(cLoc);}
		}//tr
	skippedLocusNumCYmale_.Sort();
	nlociCYmale_ = locusNumCYmale_.len();
	SetLociPresentInMalesOnly(malesOnly);
	CytoplasmicMaleMarkerData::CheckForNewAlleles();
	}//CytoplasmicMaleMarkerData::SetLoci(aList<int>& )



void CytoplasmicMaleMarkerData::SetLociPresentInMalesOnly(aList<int>& locusList){
	//assumes females contain missing data as placeholders
	allPresentInMalesOnly_ = locusList;
	presentInMalesOnly_.Clear();
	allPresentInBothSexes_.Clear();
	presentInBothSexes_.Clear();
	int loc;
	for(loc=0;loc<allCYmaleLoci_.len();++loc){
		if(! allPresentInMalesOnly_.Includes(allCYmaleLoci_[loc])){
			allPresentInBothSexes_.Append(allCYmaleLoci_[loc]);
			}}//loc
	for(loc=0;loc<locusNumCYmale_.len();++loc){
		if(allPresentInMalesOnly_.Includes(locusNumCYmale_[loc])){
				presentInMalesOnly_.Append(locusNumCYmale_[loc]);}
			else if(allPresentInBothSexes_.Includes(locusNumCYmale_[loc])){
				presentInBothSexes_.Append(locusNumCYmale_[loc]);}
			else{}
		}//loc
	}//CytoplasmicMaleMarkerData::SetLociPresentInMalesOnly(aList<int>& )


void CytoplasmicMaleMarkerData::UpdateVariableLociCYmale(void){
	if (varLociCurrentCYmale_){return;}
	CheckForNewAlleles();
	variableLocusNumCYmale_.Clear();
	for(int loc=0;loc<nlociCYmale_;++loc){
		if (allelesPresentCYmale_[loc].len()>1){
			variableLocusNumCYmale_.Append(loc);}}
	varLociCurrentCYmale_=1;
	}//end CytoplasmicMaleMarkerData::UpdateVariableLociCYmale



#endif









