//unit DominantMarkerData2.cpp

#ifndef DominantMarkerData2cpp
#define DominantMarkerData2cpp = 1



#include "DominantMarkerData2.h"
#include "Element.cpp"
#include "aList.cpp"
#include "SortList.cpp"




DominantMarkerData2::DominantMarkerData2(DominantMarkerData2& parentData):
			EvolutionaryGeneticsData(parentData),
			nlociDom_(parentData.nlociDom_),
			ploidy_(parentData.ploidy_),
			alleleLen_(parentData.alleleLen_),
			allDomLoci_(parentData.allDomLoci_),
			locusNumDom_(parentData.locusNumDom_),
			skippedLocusNumDom_(parentData.skippedLocusNumDom_),
			variableLocusNumDom_(parentData.variableLocusNumDom_), 
			allelesPresentDom_(parentData.allelesPresentDom_),
			phenotypeCount_(parentData.phenotypeCount_),
			phenotypeCountGlobal_(parentData.phenotypeCountGlobal_),
			phenotypeFreq_(parentData.phenotypeFreq_),
			phenotypeFreqGlobal_(parentData.phenotypeFreqGlobal_),
			nByLocusDom_(parentData.nByLocusDom_),
			allelesCurrentDom_(parentData.allelesCurrentDom_),
			varLociCurrentDom_(parentData.varLociCurrentDom_),
			phenotypeFreqsCurrent_(parentData.phenotypeFreqsCurrent_)
			{	
		}//end ctor DominantMarkerData2(DominantMarkerData2&)


DominantMarkerData2::DominantMarkerData2(DominantMarkerData2& parentData, 
						aList<int>& popSubsetNums):
			EvolutionaryGeneticsData(parentData,popSubsetNums),
			nlociDom_(parentData.nlociDom_),
			ploidy_(parentData.ploidy_),
			alleleLen_(parentData.alleleLen_),
			allDomLoci_(parentData.allDomLoci_),
			locusNumDom_(parentData.locusNumDom_),
			skippedLocusNumDom_(parentData.skippedLocusNumDom_)
			{
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	title_.prepend("subset of dataset: ");
	ResetFlags();
	CheckForNewAlleles();
	UpdateVariableLociDom();
	}//end ctor DominantMarkerData2(DominantMarkerData2&, aList<int>&)


void  DominantMarkerData2::Initialize(void){
	DominantMarkerData2::Clear();
	}//Initialize()
	
	
void  DominantMarkerData2::Initialize(aString& s){
	DominantMarkerData2::Clear();
	title_=s; 
	}//Initialize(aString&)
	
	
void  DominantMarkerData2::Initialize(DominantMarkerData2& dmd){
	EvolutionaryGeneticsData::Initialize(dmd);
	nlociDom_=dmd.nlociDom_;
	ploidy_=dmd.ploidy_;
	alleleLen_=dmd.alleleLen_;
	allDomLoci_=dmd.allDomLoci_;
	locusNumDom_=dmd.locusNumDom_; 
	skippedLocusNumDom_=dmd.skippedLocusNumDom_;
	variableLocusNumDom_=dmd.variableLocusNumDom_;
	allelesPresentDom_=dmd.allelesPresentDom_;
	phenotypeFreq_=dmd.phenotypeFreq_;
	phenotypeFreqGlobal_=dmd.phenotypeFreqGlobal_;
	nByLocusDom_=dmd.nByLocusDom_;
	allelesCurrentDom_=dmd.allelesCurrentDom_;
	varLociCurrentDom_=dmd.varLociCurrentDom_;
	phenotypeFreqsCurrent_=dmd.phenotypeFreqsCurrent_;
	}//Initialize(DominantMarkerData2&)
	

void  DominantMarkerData2::Initialize(DominantMarkerData2& parentData, aList<int>& popSubsetNums){
	EvolutionaryGeneticsData::Initialize(parentData,popSubsetNums);
	nlociDom_=parentData.nlociDom_;
	ploidy_=parentData.ploidy_;
	alleleLen_=parentData.alleleLen_;
	allDomLoci_=parentData.allDomLoci_;
	locusNumDom_=parentData.locusNumDom_;
	skippedLocusNumDom_=parentData.skippedLocusNumDom_;
	CheckForNewAlleles();
	UpdateVariableLociDom();
	}//end Initialize(DominantMarkerData2&, aList<int>&)


	
DominantMarkerData2&  DominantMarkerData2::operator=(DominantMarkerData2& dmd){
	EvolutionaryGeneticsData::operator=(dmd);
	nlociDom_=dmd.nlociDom_;
	ploidy_=dmd.ploidy_;
	alleleLen_=dmd.alleleLen_;
	allDomLoci_=dmd.allDomLoci_;
	locusNumDom_=dmd.locusNumDom_; 
	skippedLocusNumDom_=dmd.skippedLocusNumDom_;
	variableLocusNumDom_=dmd.variableLocusNumDom_;
	allelesPresentDom_=dmd.allelesPresentDom_;
	phenotypeCount_=dmd.phenotypeCount_;
	phenotypeCountGlobal_=dmd.phenotypeCountGlobal_;
	phenotypeFreq_=dmd.phenotypeFreq_;
	phenotypeFreqGlobal_=dmd.phenotypeFreqGlobal_;
	nByLocusDom_=dmd.nByLocusDom_;
	allelesCurrentDom_=dmd.allelesCurrentDom_;
	varLociCurrentDom_=dmd.varLociCurrentDom_;
	phenotypeFreqsCurrent_=dmd.phenotypeFreqsCurrent_;
	return *this;
	}//operator=(DominantMarkerData2&)




void DominantMarkerData2::CalculatePhenotypeFreqs (void){
	//This routine counts marker frequencies, taking into account
	//markers left blank in the data set.
	if (phenotypeFreqsCurrent_){return;}
	CheckForNewAlleles();
	long pop, i, loc, marker;
	long double freq;
//	char blank = ' ', tab='\t';	//don't count these
	aString missingDataFlags = ".?-*";//characters used to signify missing data
	aString endCharList = " \t,";//characters used to separate data fields
	phenotypeCount_.Clear();
	phenotypeFreq_.Clear();
	nByLocusDom_.Clear();
	for (pop=0;pop< npop_;++pop){ 
		//set up a frequency list for this population
		phenotypeCount_.Append( NEW ObjList<aList<int> >());
		phenotypeFreq_.Append( NEW ObjList<aList<long double> >());
		nByLocusDom_.Append(NEW aList<int>());
		for (loc=0;loc<nlociDom_;++loc){//loops through dominant loci
			//first set up a blank array of frequencies for each possible phenotype
			phenotypeCount_[pop].Append(NEW aList<int>());
			phenotypeFreq_[pop].Append(NEW aList<long double>());
			StringList& markerNamesList = allelesPresentDom_[loc];
			freq = (long double) 0.0;
			for (marker=0;marker<markerNamesList.len();++marker){
				phenotypeCount_[pop][loc].Append(0);
				phenotypeFreq_[pop][loc].Append(freq);
				}//initialize

			//next look at the phenotype of each individual and tally the frequency
			long nSampled = 0;	//the number of individuals
			ObjList<TextDataList>& phenotypesList = rawTraits_[pop];
			for (i=0;i< phenotypesList.len();++i){
				TextDatum& phene = phenotypesList[i][locusNumDom_[loc]];
				if (phene.OK()) {//if not, it's missing data
					//increment the frequency & sample size -- divide later
					marker = markerNamesList.Loc(&phene);
					phenotypeCount_[pop][loc][marker]++;
					phenotypeFreq_[pop][loc][marker]++;
					nSampled++;
					}//end if legal data
				}//end for i


			//now go through and divide by the sample size to restore the frequencies
			if(nSampled>0){
				for (marker=0;marker<markerNamesList.len();++marker){
					phenotypeFreq_[pop][loc][marker] /= (long double) nSampled;
					}
				}
			//store the sample size for this locus
			nByLocusDom_[pop].Append((int)nSampled);
			}//end for loc
		}//end for pop
	phenotypeCountGlobal_.Clear();
	phenotypeFreqGlobal_.Clear();
	for(loc=0;loc<nlociDom_;++loc){
		phenotypeCountGlobal_.Append(NEW aList<int>());
		phenotypeFreqGlobal_.Append(NEW aList<long double>());
		for(marker=0;marker<allelesPresentDom_[loc].len();++marker){
			freq = (long double)0.0;
			int summ=0;
			for(pop=0;pop<npop_;++pop){
				summ += phenotypeCount_[pop][loc][marker];
				freq += phenotypeFreq_[pop][loc][marker];
				}
			phenotypeCountGlobal_[loc].Append(summ);
			phenotypeFreqGlobal_[loc].Append(freq/(long double)npop_);
			}}
	phenotypeFreqsCurrent_=1;
	}//end DominantMarkerData2::CalculatePhenotypeFreqs



ObjList<aList<long double> >* DominantMarkerData2::CalculatePhenotypeFreqsByPopSet(aList<int>& popSet, aList<long double>& nList){
 	//returns list of loci, each with list of marker freqs, using total marker set
 	CalculatePhenotypeFreqs();
 	nList.Clear();
 	ObjList<aList<long double> >* popSetFreqsPtr = NEW ObjList<aList<long double> >(phenotypeFreq_[0]);//copies structure
 	ObjList<aList<long double> >& popSetFreqs = *popSetFreqsPtr;//notational convenience
 	for(int loc=0;loc<nlociDom_;++loc){
 		int markers = GetNumAllelesDom(loc);
 		long double samSizeSet = (long double) 0.0;
 		for(int m=0;m<markers;++m){
 			long double nCopies = (long double) 0.0;
 			for(int popSetPop=0;popSetPop<popSet.len();++popSetPop){
 				int pop = popSet[popSetPop];
 				long double nSample = (long double) (nByLocusDom_[pop][loc]);
 				if(m==0){samSizeSet += nSample;}
 				nCopies += nSample * GetFreqDom(pop,loc,m);
 				}
 			if(m==0){nList.Append(samSizeSet);}
 			if(nCopies==0.0){
 					popSetFreqs[loc][m]=(long double) 0.0;}
 				else if(nCopies==samSizeSet){
 					popSetFreqs[loc][m]=(long double) 1.0;}
 				else{
 					popSetFreqs[loc][m] = nCopies / samSizeSet;
 					}
 			}//end for m
 		}//end for loc
 	return popSetFreqsPtr;
	}//end DominantMarkerData2::CalculateAlleleFreqsByPopSet



void DominantMarkerData2::CheckForNewAlleles (void){
	if (allelesCurrentDom_){return;}
	allelesPresentDom_.Clear();
	for (int loc=0;loc<nlociDom_;++loc){//initialize
		allelesPresentDom_.Append(NEW StringList());}
	for (int pop=0;pop<npop_;++pop){
		ObjList<TextDataList>& ptypeList = rawTraits_[pop];
		for(int i=0;i<ptypeList.len();++i){
			for (int loc=0;loc<nlociDom_;++loc){
				TextDatum& phene = ptypeList[i][locusNumDom_[loc]];
				if(phene.OK()){//ignore missing data
					StringList& alleleList = allelesPresentDom_[loc];
					if(!(alleleList.Includes(&phene))){
						alleleList.Append(NEW aString(phene));}
					}//OK
				}//end for loc
			}//end for i
		}//end for pop
	for (int loc=0;loc<nlociDom_;++loc){
		allelesPresentDom_[loc].Sort();}
	allelesCurrentDom_=true;
	DominantMarkerData2::UpdateVariableLociDom();
	}//end DominantMarkerData2::CheckForNewAlleles



void DominantMarkerData2::Clear(void){
	EvolutionaryGeneticsData::Clear();
	allDomLoci_.Clear();
	locusNumDom_.Clear();
	skippedLocusNumDom_.Clear();
	variableLocusNumDom_.Clear();
	allelesPresentDom_.Clear();
	phenotypeCount_.Clear();
	phenotypeCountGlobal_.Clear();
	phenotypeFreq_.Clear();
	phenotypeFreqGlobal_.Clear();
	nByLocusDom_.Clear();
	nlociDom_=allelesCurrentDom_=varLociCurrentDom_=phenotypeFreqsCurrent_=0;
	}//end DominantMarkerData2::Clear()




void DominantMarkerData2::ConvertCodominantData(CodominantMarkerData& codomData){
	//converts extant codominant data into dominant marker phenotypes; skipped loci omitted
	//first allele is recessive to second allele; missing data treated as unscored
	//don't let codomData be part of the same data object
	codomData.CheckForNewAlleles();	
	int cloc;
	bool tooManyAlleles=false;
	for(cloc=0;cloc<codomData.nlociCodom_;++cloc){
		if(codomData.allelesPresent_[cloc].len() >2){
			tooManyAlleles=true; break;}}
	if(tooManyAlleles){
		aError("ERROR: DominantMarkerData2::ConvertCodominantData only works when there are 1-2 allele copies/locus in the data");
		return;}
	Clear();
	title_=codomData.title_ + "--converted from codominant data";
	npop_=codomData.npop_;
	ngroups_=codomData.ngroups_;
	popSize_=codomData.popSize_;
	popName_=codomData.popName_;
	groupName_=codomData.groupName_;
	localityName_=codomData.localityName_;
	groupHierarchy_=codomData.groupHierarchy_;
	popsByLocality_=codomData.popsByLocality_;
	localityCoordinates_=codomData.localityCoordinates_;
	ploidy_=codomData.ploidy_;
	alleleLen_=codomData.alleleLen_;
	nlociDom_ += codomData.nlociCodom_;
	ntraits_ = codomData.ntraits_;
	for(cloc=0;cloc<codomData.nlociCodom_;++cloc){
		traitName_.Append(NEW aString(codomData.traitName_[codomData.locusNumCodom_[cloc]]));
		traitNum_.Append(cloc);
		locusNumDom_.Append(cloc);//last one created
		}
	for(int pop=0;pop<npop_;++pop){
		rawTraits_.Append(NEW ObjList<TextDataList>());//{0,1} presence-absence data as read from data file
		nByLocusDom_.Append(NEW aList<int>());
		for(int loc=0;loc<nlociDom_;++loc){
			nByLocusDom_[pop].Append(0);}//initialize		
		for(int i=0;i<codomData.individuals(pop);++i){
			rawTraits_[pop].Append(NEW TextDataList());
			rawTraits_[pop][i].Append(NEW TextDatum(codomData.Trait(pop,i,0)));//ID code
			for(cloc=0;cloc<codomData.nlociCodom_;++cloc){
//				TextDataList codomGenes(codomData.rawCodomGtypes_[pop][i][codomData.locusNumCodom_[cloc]]);//a copy
				TextDataList codomGenes(codomData.rawCodomGtypes_[pop][i][cloc]);//a copy
				bool missingflag=false;
				for(int p=0;p<ploidy_;++p){
					if(codomGenes[p].missing()){missingflag=true;break;}}
				if(missingflag){//treat as unscored data
					rawTraits_[pop][i].Append(NEW TextDatum("",true));//store as missing
					continue;
					}
				codomGenes.Sort();//to order of codomData.allelesPresent_
				bool recessive=true;
				for(int p=0;p<ploidy_;++p){
					if(codomGenes[p] != codomData.allelesPresent_[cloc][0]){//recessive allele A present
						recessive=false;
						break;
						}}
				if (recessive){
						rawTraits_[pop][i].Append(NEW TextDatum("0",false));}
					else {
						rawTraits_[pop][i].Append(NEW TextDatum("1",false));}
				nByLocusDom_[pop][cloc]++;
				}//end for cloc
			}//end for i
		}//end for pop
	CheckForNewAlleles();	
	}//end DominantMarkerData2::ConvertCodominantData()



void  DominantMarkerData2::CreateBootstrapData(DominantMarkerData2& origData, Ran3& ran){
	EvolutionaryGeneticsData::CreateBootstrapData(origData,ran);
	CheckForNewAlleles();	
	}//end  DominantMarkerData2::CreateBootstrapData(EvolutionaryGeneticsData&)



void  DominantMarkerData2::CreateBootstrapDataConstPop(DominantMarkerData2& origData, Ran3& ran){
	EvolutionaryGeneticsData::CreateBootstrapDataConstPop(origData,ran);
	CheckForNewAlleles();	
	}//end  DominantMarkerData2::CreateBootstrapDataConstPop(EvolutionaryGeneticsData&)




void DominantMarkerData2::CreateRandomData(
							const long double Fst, 
							const long double Fis,
							const long double betaBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first do this using codominant data, then convert
	//	those data to dominant marker phenotypes
	//first convert from phenotypeto allele frequencies
	//assume first allele (pBar) is the dominant one
	long double dominantAlleleFreq =  dominantAlleleFreqFromFisBeta(betaBar,Fis);
	
	CodominantMarkerData codomData;
	codomData.CreateRandomData(Fst,Fis,dominantAlleleFreq,numPopsSampled,
											numIndivsPerPop,numLociSampled,ran);
	ConvertCodominantData(codomData);
	}//end DominantMarkerData2::CreateRandomData(...)


void DominantMarkerData2::CreateRandomData(
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& betaBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
	//dominant marker freqs (not allele freqs) that vary among loci; variable sample sizes per pop
	//multiple alleles per locus, variable sample sizes per pop
	//first convert from phenotype to allele frequencies
	//assume first allele (betaBar) is the dominant one
	ObjList<aList<long double> > dominantAlleleFreqs(betaBar);
	long double freq, one = (long double)1.0;
	long double two = one + one;
	long double four = two * two;
	for(long loc=0;loc<dominantAlleleFreqs.len();++loc){
		freq = dominantAlleleFreqFromFisBeta(dominantAlleleFreqs[loc][0],Fis);
		dominantAlleleFreqs[loc][0] = freq;
		if(dominantAlleleFreqs[loc].len()>1){			
			dominantAlleleFreqs[loc][1] = (long double)1.0 - freq;}
		}//i
	CodominantMarkerData codomData;
	codomData.CreateRandomData(Fst,Fis,dominantAlleleFreqs,
											numIndivsPerPopLoc,ran);
	ConvertCodominantData(codomData);
	}//end DominantMarkerData2::CreateRandomData(...)



void DominantMarkerData2::CullDomLoci (StringList& lociToCull){
	//This function culls the designated loci from each dominant phenotype data string
	//and includes them among the skipped loci
	if(lociToCull.len()==0){return;}//none to cull
	aList<int> cullListByNum;
	for(int tr=0;tr<lociToCull.len();++tr){
		aString cTraitname = lociToCull[tr];
		int item = traitName_.Loc(&cTraitname);
		if((item>=0)&&(!(skippedTraitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already skipped
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	CullDomLoci(cullListByNum);
	}//end procedure DominantMarkerData2::CullDomLoci(StringList&)


void DominantMarkerData2::CullDomLoci (aList<int>& lociToCull){
	//This function shifts trait numbers from from traitNums_ to the skippedTraitNums_ list
	if(lociToCull.len()==0){return;}//none to cull
	for(int tr=0;tr<lociToCull.len();++tr){
		int cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(traitNum_.Includes(cLoc) && cLoc>=0 && cLoc<traitName_.len()){
			long item = traitNum_.Loc(cLoc);
			skippedTraitNum_.Append(cLoc);
			skippedLocusNumDom_.Append(cLoc);
			traitNum_.Delete(item);
			item = locusNumDom_.Loc(cLoc);
			locusNumDom_.Delete(item);
			}//if item not skipped already
		}//tr
	nlociDom_ = locusNumDom_.len();
	skippedTraitNum_.Sort();
	skippedLocusNumDom_.Sort();
	ResetFlags();
	CheckForNewAlleles();
	}//DominantMarkerData2::CullDomLoci (aList<int>&)



void DominantMarkerData2::CullDomLocus (char* locusToCull){
	aString locToCull(locusToCull);
	CullDomLocus(locToCull);
	}//end DominantMarkerData2::CullLocus (char* )


void DominantMarkerData2::CullDomLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	CullDomLoci(lociToCull);
	}//end DominantMarkerData2::CullLocus (aString& )


void DominantMarkerData2::CullMonomorphicLociDom (void){	
	//This function culls the monomorphic loci from each genotype data string
	//and includes them among the skipped loci
	UpdateVariableLociDom();
	if(variableLocusNumDom_.len()==locusNumDom_.len()){return;}//none monomorphic
	aList<int> monomorphicLoci(locusNumDom_);
	for(int vLoc=0; vLoc<variableLocusNumDom_.len();++vLoc){
		if(monomorphicLoci.Includes(variableLocusNumDom_[vLoc])){//polymorphic
			int item = monomorphicLoci.Loc(variableLocusNumDom_[vLoc]);
			monomorphicLoci.Delete(item);}
		}//vLoc
	CullDomLoci(monomorphicLoci);
	}//end procedure DominantMarkerData2::CullMonomorphicLociDom




void DominantMarkerData2::ExpungeLoci (StringList& lociToCull){
	if(lociToCull.len()==0){return;}//none to cull
	aList<int> cullListByNum;
	for(int tr=0;tr<lociToCull.len();++tr){
		aString cTraitname = lociToCull[tr];
		int item = traitName_.Loc(&cTraitname);
		if(item>=0){//the trait exists in the rawlist
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	ExpungeLoci(cullListByNum);
	}//end procedure DominantMarkerData2::ExpungeLoci(StringList&)


long double DominantMarkerData2::dominantAlleleFreqFromFisBeta (long double beta, long double FisVal){
	long double one = (long double)1.0;
	long double two = one + one;
	long double four = two * two;
	long double dominantAlleleFreq =  sqrt((two-FisVal)*(two-FisVal) - four * beta * (one-FisVal) );
	dominantAlleleFreq += (two-FisVal);
	dominantAlleleFreq /= two * (one-FisVal);
	return dominantAlleleFreq;
	}//DominantMarkerData2::dominantAlleleFreqFromFisBeta (long double, long double) 



void DominantMarkerData2::ExpungeLoci (aList<int>& lociToCull){
	//This function removes trait numbers from the haploid data set; called during reclassification
	if(lociToCull.len()==0){return;}//none to cull
	long item;
	for(int tr=0;tr<lociToCull.len();++tr){
		int cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(allDomLoci_.Includes(cLoc)){
			item = allDomLoci_.Loc(cLoc);
			allDomLoci_.Delete(item);}
		if(locusNumDom_.Includes(cLoc)){
			item = locusNumDom_.Loc(cLoc);
			locusNumDom_.Delete(item);}
		if(skippedLocusNumDom_.Includes(cLoc)){
			item = skippedLocusNumDom_.Loc(cLoc);
			skippedLocusNumDom_.Delete(item);}
		}//tr
	nlociDom_ = locusNumDom_.len();
	ResetFlags();
	DominantMarkerData2::CheckForNewAlleles();
	}//DominantMarkerData2::ExpungeLoci (aList<int>&)



void DominantMarkerData2::ExpungeLocus (char* locusToCull){
	aString locToCull(locusToCull);
	ExpungeLocus(locToCull);
	}//end DominantMarkerData2::ExpungeLocus (char* )


void DominantMarkerData2::ExpungeLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	ExpungeLoci(lociToCull);
	}//end DominantMarkerData2::ExpungeLocus (aString& )


long double DominantMarkerData2::GetFreqDom (int loc, int allele){
	long double numAlleleCopies = 0.0;
	long double numThisAllele = 0.0;
	for(int pop=0;pop<npop_;++pop){
		long double n = (long double)GetLocusSampleSizeDom(pop,loc);
		numAlleleCopies += n;
		numThisAllele += n * GetFreqDom(pop,loc,allele);
		}
	if(numAlleleCopies==0.0){return (long double)0.0;}
	return numThisAllele/numAlleleCopies;
	}//end DominantMarkerData2::GetFreq


long double DominantMarkerData2::GetFreqDom (int pop, int loc, int marker){
	if (!phenotypeFreqsCurrent_){CalculatePhenotypeFreqs();}
	return phenotypeFreq_[pop][loc][marker];
	}//end DominantMarkerData2::GetFreqDom


long double DominantMarkerData2::GetFreqMostCommonAllele (int pop, int loc){
	long double mostCommon = 0.0;
	for(int allele=0;allele<GetNumAllelesDom(loc);++allele){
		long double freq = GetFreqDom(pop,loc,allele);
		if (freq>(long double)0.5) {return freq;}
		if (mostCommon<freq){mostCommon = freq;}
		}//and for allele
	return mostCommon;
	}//end DominantMarkerData2::GetFreqMostCommonAllele


long DominantMarkerData2::GetIndividualsSampledDom (int pop, int loc){
	// loc refers to dominant loci only
	long n=0;
	for(long i=0;i<individuals(pop);++i){
		if(GetLocusSampleSizeDom (pop, i, loc)>0){
			++n;}}
	return n;
	}//end  DominantMarkerData2::GetIndividualsSampled


long DominantMarkerData2::GetLocusSampleSizeDom (int loc){
	if(! phenotypeFreqsCurrent_){
		CalculatePhenotypeFreqs();}//calculates twoN_
	double numMarkersFound = 0.0;
	for(int pop=0;pop<npop_;++pop){
		numMarkersFound += nByLocusDom_[pop][loc];
		}
	return numMarkersFound;
	}//end  DominantMarkerData2::GetLocusSampleSize


long DominantMarkerData2::GetLocusSampleSizeDom (int pop, int loc){
	// loc refers to dominant loci only
	return nByLocusDom_[pop][loc];
	}//end  DominantMarkerData2::GetLocusSampleSizeDom


long DominantMarkerData2::GetLocusSampleSizeDom (int pop, int indiv, int loc){
	// loc refers to dominant loci only
	TextDatum& score = PhenotypicVal(pop,indiv,loc);
	return score.OK();//false=0 if missing, true=1 if present
	}//end  DominantMarkerData2::GetLocusSampleSizeDom


int DominantMarkerData2::GetNumAllelesDom (int loc){
	// loc refers to dominant loci only
	//total in data set
	if(! allelesCurrentDom_){CheckForNewAlleles();}
	return allelesPresentDom_[loc].len();
	}//end  DominantMarkerData2::GetNumAlleles(loc)


int DominantMarkerData2::GetNumAllelesDom (int pop, int loc){
	// loc refers to dominant loci only
	//extant alleles in population
	if(! phenotypeFreqsCurrent_){CalculatePhenotypeFreqs();}
	aList<long double>& locList = phenotypeFreq_[pop][loc];
	int count=0;
	for(int a=0;a<locList.len();++a){
		if (locList[a] >(long double) 0.0){++count;}}
	return count;
	}//end  DominantMarkerData2::GetNumAllelesDom(pop,loc)


aString* DominantMarkerData2::GetTraitString(int pop, int indiv, int loc){
	//loc indexed as in locusNumCodom_ 
	aString *aStr = NEW aString();
	int tr = locusNumDom_[loc];
	aStr->cat(rawTraits_[pop][indiv][tr][0]);
	return aStr;
	}//DominantMarkerData2::GetTraitString


aString* DominantMarkerData2::GetTraitString(int pop, int indiv, int loc, int /*allele*/){
	//for compatibility with derived classes
	return GetTraitString(pop, indiv, loc);
	}//DominantMarkerData2::GetTraitString


int DominantMarkerData2::individuals(int pop, int loc){
	int indivs=0;
	for(int i=0;i<individuals(pop);++i){
		if (GetLocusSampleSizeDom ( pop, i, loc)>0){++indivs;}}
	return indivs;
	}//end  DominantMarkerData2::individuals(int,int)
	


int	DominantMarkerData2::IsPolymorphic(int pop, int loc){
	return IsPolymorphic(pop, loc, 0.0);
	}//DominantMarkerData2::IsPolymorphic(int, int)



int	DominantMarkerData2::IsPolymorphic(int pop, int loc, double criterion){
	if(! phenotypeFreqsCurrent_){CalculatePhenotypeFreqs();}
	long double freq, maxfreq=(long double)0.0;
	for(int allele=0;allele<phenotypeFreq_[pop][loc].len();++allele){
		freq = phenotypeFreq_[pop][loc].len();
		if(freq>0.5) {return true;}
			else {maxfreq = MAX(maxfreq,freq);}}
	if(maxfreq>criterion){ return true;}
	return false;
	}//DominantMarkerData2::IsPolymorphic(int, int, double)


int	DominantMarkerData2::IsPolymorphic(int loc){
	if(! allelesCurrentDom_){CheckForNewAlleles();}
	if(allelesPresentDom_[loc].len()>1){ return true;}
	return false;
	}//DominantMarkerData2::IsPolymorphic(int)



int	DominantMarkerData2::IsPolymorphic(int loc, double criterion){
	if(! phenotypeFreqsCurrent_){CalculatePhenotypeFreqs();}
	long double freq, maxfreq=(long double)0.0;
	for(int allele=0;allele<phenotypeFreqGlobal_[loc].len();++allele){
		freq = phenotypeFreqGlobal_[loc].len();
		if(freq>0.5) {return true;}
			else {maxfreq = MAX(maxfreq,freq);}}
	if(maxfreq>criterion){ return true;}
	return false;
	}//DominantMarkerData2::IsPolymorphic(int, double)


long DominantMarkerData2::maxall(void){
	//maximum alleles per locus in data set
	CheckForNewAlleles();
	long maxalleles=0;
	int loc;
	for(loc=0;loc<nlociDom_;++loc){
		maxalleles = MAX(maxalleles,allelesPresentDom_[loc].len());}
	return maxalleles;
	}//DominantMarkerData2::maxall()



/*
void  EvolutionaryGeneticsData::PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of allelic frequencies for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculateAlleleFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied


	outFile<<title<<endl<<endl<<endl;
	outFile<<"ALLELE FREQUENCY DATA"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop<<endl;
	outFile<<"NUMBER OF LOCI SURVEYED = ";outFile.width(3);outFile<<nloci<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (lociSkipped == 0){
			outFile<<"none";}
		else{
			for (int loc=0;loc< lociSkipped;++loc){
				aString& locName = skippedLocusName[loc];
				outFile<<locName;
				if (loc < lociSkipped-1){outFile<<", ";}
				}//end for loc
		}//end if lociSkipped
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop / ncols;
	int npop0 = npop;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Allele frequencies for each population."<<endl;
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;

		for (int loc=0;loc<nloci;++loc) {//print the locus and locus sample size
			aString& locWord = locusName[loc];
			outFile<< locWord;
			if (asciiFormat){outFile.width(11 + (5 - locWord.len()));}else{outFile<<colSpacer;}
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = GetLocusSampleSize(pop, loc) / 2;
				outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
				if (pop < endPop){
					if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
					}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = GetFreq(pop, loc, allele);
					aString freqStr(freq,0,4);
					if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									if (asciiFormat){
											aString blanks(' ',12);
											if (pop==x) {
												blanks.PrependCharToLen(' ',17);}
											outFile<<blanks;
											}
										else{outFile<<colSpacer;}
									}
								else{
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end EvolutionaryGeneticsData::PrintAlleleFreqs




void  EvolutionaryGeneticsData::PrintAlleleFreqsByPopList (ostream& outFile, 
											ObjList<ObjList<aList<double> > >& popFreqList, 
											ObjList<aList<double> >& popTwoNList, 
											int ncols){
	//popFreqList must contain the entire set of alleles in the data set
	//Sends a table of allelic frequencies to the output file.
	//ncols indexes the number of population columns per page

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = popFreqList.len() / ncols;
	int npop0 = popFreqList.len();
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)popFreqList.len(), (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			outFile.width(12);outFile<<pop+1;}
		outFile<<endl;

		for (int loc=0;loc<nloci;++loc) {//print the locus and locus sample size
			aString& locWord = locusName[loc];
			outFile<< locWord;outFile.width(11 + (5 - locWord.len()));
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = popTwoNList[pop][loc] / 2;
				outFile<<"[";outFile.width(2);outFile<<samSize<<"]";
				if (pop < endPop){outFile.width(9);}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = popFreqList[pop][loc][allele];
					aString freqStr(freq,0,4);
					freqStr.PrependCharToLen(' ',12);
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									aString blanks(' ',12);
									if (pop==x) {
										blanks.PrependCharToLen(' ',17);}
									outFile<<blanks;}
								else{
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end EvolutionaryGeneticsData::PrintAlleleFreqsByPopList
*/


void	 DominantMarkerData2::PrintDomMarkerFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintDomMarkerFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end DominantMarkerData2::PrintDomMarkerFile(aString&)



void DominantMarkerData2::PrintDomMarkerFile (ostream& outFile){
	aString dataLine = title_;
	dataLine.cat(":  skipped loci omitted");
	outFile<<dataLine<<endl;
	dataLine = "NPOP=";
	dataLine.cat(npop_);
	dataLine.cat(", NLOCI=");
	dataLine.cat(nlociDom_);
	long maxall_=0;
	for(int loc=0;loc<allelesPresentDom_.len();++loc){
		maxall_=MAX(maxall_,allelesPresentDom_[loc].len());}
	dataLine.cat(", MAXALL=");
	dataLine.cat(maxall_);
	dataLine.cat(", NGROUPS=");
	dataLine.cat(ngroups_);
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int loc=0;loc<nlociDom_;++loc){
		dataLine.cat(locusNameDom(loc));
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < nlociDom_){
				dataLine.cat("\t");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	dataLine = "SKIP=0";
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int pop=0;pop<npop_;++pop){
		outFile<<popName_[pop]<<endl;
		//cobble together a phenotype list for dominant markers only
		for (int i=0;i<individuals(pop);++i){
			StringList idata;
			idata.Append(NEW TextDatum(rawTraits_[pop][i][0]));
			for(int dloc=0;dloc<nlociDom_;++dloc){
				TextDatum& score=rawTraits_[pop][i][locusNumDom_[dloc]];
				if(score.OK()){
						idata.Append(NEW aString(score));}
					else{//record as missing data
						idata.Append(NEW aString(rawTraits_[pop][i].missingDataStr_));}
				}//dLoc
			aString outStr;
			for(int j=0;j<idata.len();++j){
				outStr+idata[j];
				if(j<idata.len()-1){
					outStr+" ";}}
			outFile<<outStr<<endl;
			}//end for i
		outFile<<"NEXT"<<endl;
		}//end for pop
	outFile<<"END"<<endl;
	}//end DominantMarkerData2::PrintDomMarkerFile(ostream&)



void	 DominantMarkerData2::PrintNEXUSFile(aString& outFileName, StringList& textToAppend){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintNEXUSFile(dataFile,textToAppend);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end DominantMarkerData2::PrintNEXUSFile(aString&,StringList&)



void DominantMarkerData2::PrintNEXUSFile (ostream& outFile, StringList& textToAppend){
	outFile<<"#NEXUS"<<endl;
	aString dataLine = title_;
	dataLine.prepend("[ ! ");
	outFile<<dataLine<<endl<<"]"<<endl<<endl;
	dataLine = "begin alleles;";
	outFile<<dataLine<<endl;
	dataLine = "\tdimensions newpops nloci=";
	dataLine.cat(nlociDom_);
	dataLine.cat(" npops=");
	dataLine.cat(npop_);
	dataLine.cat(";");
	outFile<<dataLine<<endl;
	dataLine = "\tdominant all:1;";
	outFile<<dataLine<<endl;
	dataLine = "\tformat labels missing=? ;";
	outFile<<dataLine<<endl;
	dataLine = "\tmatrix";
	outFile<<dataLine<<endl;
	dataLine.clear();
	/* need code for locus names
	for (int loc=0;loc<nlociDom_;++loc){
		dataLine.cat(locusNameDom_(loc));
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < nlociDom_){
				dataLine.cat("\t");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	*/
	aString endCharList("\t, ");
	for (int pop=0;pop<npop_;++pop){
		aString pname = popName_[pop];
		pname.replaceAll(' ','_');
		outFile<<'\t'<<pname<<":"<<endl;
		for (int i=0;i<individuals(pop);++i){
			StringList idata;
			idata.Append(NEW TextDatum(rawTraits_[pop][i][0]));
			idata[0].PrependCharToLen('0',3);//tweak the format of the ID code
			idata[0].prepend('_');
			idata[0].cat("_\t");
			for(int dloc=0;dloc<nlociDom_;++dloc){
				TextDatum& score=rawTraits_[pop][i][locusNumDom_[dloc]];
				if(score.OK()){
						idata.Append(NEW aString(score));}
					else{//record as missing data
						idata.Append(NEW aString("?"));}
				}//dLoc
			aString outStr;
			for(int j=0;j<idata.len();++j){
				outStr+idata[j];
				if(j<idata.len()-1){
					outStr+"\t";}}
			outFile<<"\t\t"<<outStr<<endl;

			}//end for i
		if(pop==npop_-1){
				outFile<<";"<<endl;}
			else{
				outFile<<","<<endl;}
		}//end for pop
	outFile<<"end;"<<endl<<endl;
	
	for(int i=0;i<textToAppend.len();++i){
		outFile<<textToAppend[i]<<endl;}
	outFile<<endl;
	}//end DominantMarkerData2::PrintNEXUSFile(ostream&, StringList&)



void DominantMarkerData2::PrintLociDom (ostream& outFile, int lociPerLine){//file assumed open
	int lineSpot = 0;
	for (int loc=0;loc<nlociDom_;++loc){ 
		aString& locName = locusNameDom(loc);
		outFile<<locName;
		++lineSpot;
		if ((loc == nlociDom_-1) || (lineSpot == lociPerLine)) {
				outFile<<endl;}
			else{
				outFile<<"\t";}
		}//end for loc
	outFile<<endl;
	}//end DominantMarkerData2::PrintLociDom


void DominantMarkerData2::PrintPhenotypeFreqs(aString& outFileName, int ncols, char colSpacer){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintPhenotypeFreqs(dataFile, ncols, colSpacer);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end DominantMarkerData2::PrintPhenotypeFreqs(aString&, int, char)



void  DominantMarkerData2::PrintPhenotypeFreqs (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of phenotypic frequencies for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculatePhenotypeFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied
	int significantDigits = 4;

	outFile<<title_<<endl<<endl<<endl;
	outFile<<"PHENOTYPIC FREQUENCY DATA"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop_<<endl;
	outFile<<"NUMBER OF MARKERS SURVEYED = ";outFile.width(3);outFile<<nlociDom_<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (skippedLocusNumDom_.len() == 0){
			outFile<<colSpacer<<"none";}
		else{
			outFile<<colSpacer;
			for (int loc=0;loc< skippedLocusNumDom_.len();++loc){
				aString& locName = traitName_[skippedLocusNumDom_[loc]];
				outFile<<locName;
				if(MOD(loc+1,20)==0){outFile<<endl;}
					else if (loc < skippedLocusNumDom_.len()-1){outFile<<colSpacer;}
					else{}
				}//end for loc
		}//end if lociSkippedDom
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop_ / ncols;
	int npop0 = npop_;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Phenotypic frequencies of DOMINANT markers for each population."<<endl;
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Phenotypic frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop_, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;

		for (int loc=0;loc<nlociDom_;++loc) {//print the locus and locus sample size
			aString& locWord = locusNameDom(loc);
			outFile<< locWord;
			if (asciiFormat){outFile.width(11 + (5 - locWord.len()));}else{outFile<<colSpacer;}
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = nByLocusDom_[pop][loc];
				outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
				if (pop < endPop){
					if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
					}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& markerNameList = allelesPresentDom_[loc];
			for (int marker=0; marker<markerNameList.len();++marker){
				aString& markerName=markerNameList[marker];
				aString markerNameCopy(markerName);
				markerNameCopy.PrependCharToLen(' ',3);
				outFile<<markerNameCopy;
				// print the frequency of the marker for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = phenotypeFreq_[pop][loc][marker];
					aString freqStr(freq,0,significantDigits);
					if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
										}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									if (asciiFormat){
											aString blanks(' ',12);
											if (pop==x) {
												blanks.PrependCharToLen(' ',17);}
											outFile<<blanks;
											}
										else{outFile<<colSpacer;}
									}
								else{
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end EvolutionaryGeneticsData::PrintPhenotypeFreqs




int DominantMarkerData2::ReadAFLPDataFile (aString& dataFileName){
	Clear();
	fstream dataFile;
	aString endCharList(" \t,\r"), missingDataListElems("?. -_");
	char endchar = '\n';	
	StringList missingDataList;
	for(int s=0;s<missingDataListElems.len();++s){
		missingDataList.Append(NEW aString(missingDataListElems[s]));}
	char *dataLine=NEW char[10001];dataLine[0]='\0';
	char* filename=dataFileName.raw();
	dataFile.open(filename,ios::in);
	int dataFileOpen = dataFile.is_open();
	if(! dataFileOpen){
		aString errstr("couldn\'t open the file ");
		errstr.cat(filename);
		aError(errstr);
		return 0;}

	dataFile.getline(dataLine, 10000,endchar);
	title_ = dataLine;

	dataFile.getline(dataLine, 10000,endchar); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	npop_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 6);
	nlociDom_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 7);
	int maxall_ = paramWord->toInt();//now ignored
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	ngroups_ =  paramWord->toInt();
	delete paramWord; paramWord=0;


	//get locus information
	int moreLoci = 1;//true
	long loc = 0;
	StringList rawLocusNameDom,skippedLocusNameDom;//temporary
	while (moreLoci){
		dataFile.getline(dataLine, 10000,endchar); 
		aString locusLine(dataLine);
		while (locusLine.len() > 0){
			++loc;
			aString *locName = locusLine.ChopOffFirstWord(endCharList);
			if(locName->len()>0){//there could be trailing blanks on locusLine
				aString *s = NEW aString(locName);
				rawLocusNameDom.Append(s);}//append a copy
			delete locName; locName=0;
			}//end while locusLine.len()
		if (loc >= nlociDom_){//data may be on more than one line
			moreLoci = 0;}//false
		}//end while moreLoci
	//check for duplicates	
	StringList uniqueNames;
	for(long i=0;i<rawLocusNameDom.len();++i){
		if(!(uniqueNames.Includes(&(rawLocusNameDom[i])))){
			uniqueNames.Append(NEW aString(rawLocusNameDom[i]));}}

	long numuni=uniqueNames.len();
	if(rawLocusNameDom.len()>numuni){
		//figure out the names and positions of duplicated loci & report
		StringList dupNames(rawLocusNameDom);
		for(loc=0;loc<uniqueNames.len();++loc){
			long uItem=dupNames.Loc(&(uniqueNames[loc]));
			dupNames.Delete(uItem);
			}//loc
		ObjList<aList<long> > dupPositions;
		for(loc=0;loc<dupNames.len();++loc){
			StringList locNamesCopy(rawLocusNameDom);//work with a copy
			dupPositions.Append(NEW aList<long>());
			long dupcount=0;
			long dupitem = locNamesCopy.Loc(&(dupNames[loc]));
			while (dupitem>=0){
				locNamesCopy.Delete(dupitem);
				dupPositions[loc].Append(dupitem+dupcount);//because the locNamesCopy list shrinks from the bottom
				++dupcount;
				dupitem = locNamesCopy.Loc(&(dupNames[loc]));
				}//while
			}//loc
		cout<<"You have duplicated locus names in your data set."<<endl;
		cout<<"Please check that these are not mistakes, and give all your loci unique names."<<endl;
		cout<<"duplicated loci:"<<endl;
		dataFile<<"You have duplicated locus names in your data set."<<endl;
		dataFile<<"Please check that these are not mistakes, and give all your loci unique names."<<endl;
		dataFile<<"duplicated loci:"<<endl;
		for(loc=0;loc<dupNames.len();++loc){
			aString dupLine(dupNames[loc]);
			dupLine.cat(" at positions ");
			for(int i=0;i<dupPositions[loc].len();++i){
				dupLine.cat(dupPositions[loc][i]+1);
				if(i<dupPositions[loc].len()-1){
					dupLine.cat(", ");}
				}//i
			cout<<dupLine<<endl;
			dataFile<<dupLine<<endl;}
		aError("The duplicated names are in your output file. Please edit your data file and start again from the beginning.");
		return 2;
		}//unique names
	//read in the number of loci to be skipped in the analyses
	dataFile.getline(dataLine, 10000,endchar); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	int lociSkippedDom = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped loci, if any
	loc = 0;
	moreLoci = 0;//false
	if (lociSkippedDom > 0){
			moreLoci = 1;}//true
	while (moreLoci){
		while (skipLine.len() > 0){
			++loc;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				aString *s = NEW aString(skipWord);
				skippedLocusNameDom.Append(s);//append a copy
				}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (loc >= lociSkippedDom){
				moreLoci = 0;}//false
			else	{//more loci on the next line
				dataFile.getline(dataLine, 10000,endchar); 
				skipLine = dataLine;}
		}//end while moreLoci

	traitName_=rawLocusNameDom;
	for(int tr=0;tr<traitName_.len();++tr){
		allDomLoci_.Append(tr);}
	locusNumDom_=allDomLoci_;
		
	//read in population data
	aString popLine;
	aString beast;
	ObjList<StringList> rawPhenotypes;
	for (int pop=0;pop<npop_;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 10000,endchar); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		aString *s = NEW aString(popLine);
		popName_.Append(s);//append a copy

		//make a new sublist for the genotypes of this population
		rawPhenotypes.Append(NEW StringList());
		// BEAST is the string for a particular individual
		beast.clear();
		while (beast != "NEXT"){ 
			dataFile.getline(dataLine, 10000,endchar); 
			beast = dataLine;
			beast.CullTrailingChar('\t');
			beast.CullTrailingChar(' ');
			beast.CullTrailingChar(',');
			if (beast != "NEXT"){ 
					aString *s = NEW aString(beast);
					rawPhenotypes[pop].Append(s);
					s=0;}//appends a copy
			}//end while beast..else
		}//end for pop
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	//Now parse the data by trait
	for(int pop=0;pop<npop_;++pop){	
		rawTraits_.Append(NEW ObjList<TextDataList>());
		for(int i=0;i<rawPhenotypes[pop].len();++i){
			aString dataStrCopy(rawPhenotypes[pop][i]);
			if(dataStrCopy.len()==0) continue;
			rawTraits_[pop].Append(NEW TextDataList("?"));
			aString *codeWord = dataStrCopy.ChopOffFirstWord(endCharList);
			rawTraits_[pop][i].Append(NEW TextDatum(*codeWord));
			for(int loc=0;loc<nlociDom_;++loc){
				int posPtypeWord = 1 + ((alleleLen_+1) * loc);
				if(posPtypeWord>=dataStrCopy.len()){
						rawTraits_[pop][i].Append(NEW TextDatum("",true));
						continue;}
				aString phene = aString(dataStrCopy, posPtypeWord, posPtypeWord+alleleLen_-1);
				if(missingDataList.Includes(&phene)){
						rawTraits_[pop][i].Append(NEW TextDatum("",true));}
					else{
						rawTraits_[pop][i].Append(NEW TextDatum(phene));}
				}//loc
			delete codeWord;codeWord=0;
			}//i		
		}//pop		
	CullDomLoci(skippedLocusNameDom);
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();
	return 1;
	}//end DominantMarkerData2::ReadAFLPDataFile


/*

int DominantMarkerData2::ReadAndAppendAFLPDataFile (aString& dataFileName){
	fstream dataFile;	
	char endchar = '\n';	
	char *dataLine=NEW char[10001];dataLine[0]='\0';
	aString endCharList(" \t,\r");
	char* filename=dataFileName.raw();
	dataFile.open(filename,ios::in);
	int dataFileOpen = dataFile.is_open();
	if(! dataFileOpen){
		aString errstr("couldn\'t open the file ");
		errstr.cat(filename);
		aError(errstr);
		return 0;}

	dataFile.getline(dataLine, 10000,endchar);
	title_.prepend("Combined data: [");
	title_.cat("] & [");
	title_.cat(dataLine);
	title_.cat("]");

	dataFile.getline(dataLine, 10000,endchar); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	int npopThisFile = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 6);
	int nlociDomThisFile = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 7);
	long maxallThisFile = paramWord->toInt();
	delete paramWord; paramWord=0;
	maxall = MAX(maxall, maxallThisFile);

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	int ngroupsThisFile =  paramWord->toInt();
	delete paramWord; paramWord=0;


	//get locus information
	int moreLoci = 1;//true
	int loc = 0;
	while (moreLoci){
		dataFile.getline(dataLine, 10000,endchar); 
		aString locusLine(dataLine);
		while (locusLine.len() > 0){
			++loc;
			aString *locName = locusLine.ChopOffFirstWord(endCharList);
			if(locName->len()>0){//there could be trailing blanks on locusLine
				if(!(rawLocusNameDom.Includes(locName))){//appended data can repeat loci
					aString *s = NEW aString(locName);
					rawLocusNameDom.Append(s);}//append a copy
				}
			delete locName; locName=0;
			}//end while locusLine.len()
		if (loc >= nlociDomThisFile){//data may be on more than one line
			moreLoci = 0;}//false
		}//end while moreLoci
	nlociDom_ = rawLocusNameDom.len();
	//read in the number of loci to be skipped in the analyses
	dataFile.getline(dataLine, 10000,endchar); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	int lociSkippedDomThisFile = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped loci, if any
	loc = 0;
	moreLoci = 0;//false
	if (lociSkippedDom > 0){
			moreLoci = 1;}//true
	while (moreLoci){
		while (skipLine.len() > 0){
			++loc;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				if(!(skippedLocusNameDom.Includes(skipWord))){
					aString *s = NEW aString(skipWord);//skipped loci can be repeated
					skippedLocusNameDom.Append(s);//append a copy
					int toSkip = rawLocusNameDom.Loc(s);
					skippedLocusNumDom.Append(toSkip);}
				}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (loc >= lociSkippedDom){
				moreLoci = 0;}//false
			else	{//more loci on the next line
				dataFile.getline(dataLine, 10000,endchar); 
				skipLine = dataLine;}
		}//end while moreLoci
	lociSkippedDom = skippedLocusNameDom.len();

	locusNameDom.Copy(rawLocusNameDom);
	for (loc=0;loc<skippedLocusNumDom.len();++loc){
		aString& toSkip = skippedLocusNameDom[loc];
		int location = locusNameDom.Loc(&toSkip);
		locusNameDom.Delete(location);
		}//end for loc
	nlociDom_ -= lociSkippedDom;

	//read in population data
	aString popLine;
	aString beast;
	for (int pop=0;pop<npopThisFile;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 10000,endchar); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		int thepop = pop;
		if(!(popName.Includes(&popLine))){
				aString *s = NEW aString(popLine);
				popName.Append(s);//append a copy
				thepop = popName.len()-1;//the last pop in list
				//make a new sublist for the genotypes of this population
				rawPhenotypes.Append(NEW StringList());				
				}
			else{
				thepop = popName.Loc(&popLine);}


		// BEAST is the string for a particular individual
		beast.clear();
		while (beast != "NEXT"){ 
			dataFile.getline(dataLine, 10000,endchar); 
			beast = dataLine;
			beast.CullTrailingChar('\t');
			beast.CullTrailingChar(' ');
			beast.CullTrailingChar(',');
			if (beast != "NEXT"){ 
				//check to see if this individual already has data
				aString beastCopy(beast);
				aString* IDbeast = beastCopy.ChopOffFirstWord(endCharList);
				int hasData	= 0, indivNum=0;
				for(int i=0;i<rawPhenotypes[thepop].len();++i){
					aString indivData(rawPhenotypes[thepop][i]);//copies
					aString *IDindiv = indivData.ChopOffFirstWord(endCharList);
					if(*IDbeast == *IDindiv){//already in data set
						hasData = 1;//true
						indivNum = i;
						delete IDindiv;IDindiv=0;
						break;//escape for i loop
						}
					delete IDindiv;IDindiv=0;
					}//end for i
				if(hasData){//append to end of this individual -- buggy if loci repeated
						beastCopy.prepend('\t');
						rawPhenotypes[thepop][indivNum].cat(beastCopy);
						}
					else{
						aString *s = NEW aString(beast);
						rawPhenotypes[thepop].Append(s);
						s=0;}//appends a copy
				delete IDbeast; IDbeast=0;
				}//end if beast != NEXT
			}//end while beast..else


		}//end for pop
	npop_ = rawPhenotypes.len();
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	ResetFlags();
	GroupMembership();
	LocalityMembership();
	CullSkippedLociDom();
	CheckForNewAlleles();
	return 1;
	}//end DominantMarkerData2::ReadAndAppendAFLPDataFile
*/




void DominantMarkerData2::RestoreLociDom (aList<int>& lociToRestore){
	//This function restores the designated loci to the analysis.
	//It shifts trait numbers from from skippedTraitNums_ to the traitNums_ list
	//and keps track of the changes in the locusNumDom_ and skippedLocusNumDom_ lists
	if(lociToRestore.len()==0){return;}//none to cull
	ResetFlags();
	for(int tr=0;tr<lociToRestore.len();++tr){
		int rLoc = lociToRestore[tr];//make sure it's a valid locus to restore
		if(skippedTraitNum_.Includes(rLoc) && rLoc>=0 && rLoc<traitName_.len()){
			traitNum_.Append(rLoc);
			locusNumDom_.Append(rLoc);
			long item = skippedTraitNum_.Loc(rLoc);
			skippedTraitNum_.Delete(item);
			item = skippedLocusNumDom_.Loc(rLoc);
			skippedLocusNumDom_.Delete(item);
			}//if item not skipped already
		}//tr
	traitNum_.Sort();
	locusNumDom_.Sort();
	CheckForNewAlleles();
	}//end  DominantMarkerData2::RestoreLociDom(aList<int>&)


void DominantMarkerData2::RestoreLociDom (StringList& lociToRestore){
	//This function restores the designated loci to the analysis
	if(lociToRestore.len()==0){return;}//none to cull
	//convert names to numbers and cull
	aList<int> restoreListByNum;
	for(int tr=0;tr<lociToRestore.len();++tr){
		aString rTraitname = lociToRestore[tr];
		int item = traitName_.Loc(&rTraitname);
		if((item>=0)&&(!(traitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already included
			restoreListByNum.Append(item);
			}//if item exists
		}//tr
	RestoreLociDom(restoreListByNum);
	}//end DominantMarkerData2::RestoreLociDom(StringList&)


void DominantMarkerData2::RestoreLocusDom (aString& locusToRestore){
	StringList lociToRestore;
	lociToRestore.Append(new aString(locusToRestore));
	RestoreLociDom(lociToRestore);
	}//end DominantMarkerData2::RestoreLocus (aString& )



void DominantMarkerData2::SetLoci(aList<int>& lociToInclude){
	if(lociToInclude.len()==0) return;
	DominantMarkerData2::ResetGeneticFlags();
	allDomLoci_=lociToInclude;
	locusNumDom_=lociToInclude;
	for(int tr=0;tr<lociToInclude.len();++tr){
		int cLoc = lociToInclude[tr];
		if(skippedTraitNum_.Includes(cLoc)){
			long item = locusNumDom_.Loc(cLoc);
			if(item>=0){
				locusNumDom_.Delete(item);}
			if(! skippedLocusNumDom_.Includes(cLoc)){
				skippedLocusNumDom_.Append(cLoc);}
			}
		}//tr
	skippedLocusNumDom_.Sort();
	nlociDom_ = locusNumDom_.len();
	DominantMarkerData2::CheckForNewAlleles();
	}//DominantMarkerData2::SetLoci(aList<int>& )



void DominantMarkerData2::UpdateVariableLociDom(void){
	if (varLociCurrentDom_){return;}
	CheckForNewAlleles();
	variableLocusNumDom_.Clear();
	for(int loc=0;loc<nlociDom_;++loc){
		if (allelesPresentDom_[loc].len()>1){
			variableLocusNumDom_.Append(loc);}}
	varLociCurrentDom_=1;
	}//end DominantMarkerData2::UpdateVariableLociDom


	

#endif




