//unit EvolutionaryGeneticsData.cpp


#ifndef EvolutionaryGeneticsDatacpp
#define EvolutionaryGeneticsDatacpp = 1

#include "EvolutionaryGeneticsData.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "aVector.cpp"
#include "aMatrix.cpp"
//#include "StringList.h"
//#include "TextDataList.h"



EvolutionaryGeneticsData::EvolutionaryGeneticsData(EvolutionaryGeneticsData& sourceData):
			title_(sourceData.title_), 
			npop_(sourceData.npop_), 
			ntraits_(sourceData.ntraits_), 
			ngroups_(sourceData.ngroups_),
			popSize_(sourceData.popSize_),
			groupName_(sourceData.groupName_),
			popName_(sourceData.popName_),
			localityName_(sourceData.localityName_),
			groupHierarchy_(sourceData.groupHierarchy_),
			popsByLocality_(sourceData.popsByLocality_),	
			localityCoordinates_(sourceData.localityCoordinates_),
			traitName_(sourceData.traitName_),
			traitNum_(sourceData.traitNum_),
			skippedTraitNum_(sourceData.skippedTraitNum_),
			rawTraits_(sourceData.rawTraits_),
			missing_(sourceData.missing_)
			{	
		}//end ctor EvolutionaryGeneticsData(EvolutionaryGeneticsData&)


EvolutionaryGeneticsData::EvolutionaryGeneticsData(EvolutionaryGeneticsData& sourceData, aList<int>& popSubsetNums):
			title_(sourceData.title_), 
			npop_(popSubsetNums.len()), 
			ntraits_(sourceData.ntraits_), 
			ngroups_(0),
			traitName_(sourceData.traitName_),
			traitNum_(sourceData.traitNum_),
			skippedTraitNum_(sourceData.skippedTraitNum_),
			rawTraits_(sourceData.rawTraits_),
			missing_(sourceData.missing_)
			{
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	popSubsetNums.Sort();
	int p, pp;
	for(p=0;p<popSubsetNums.len();++p){
		pp = popSubsetNums[p];//population num in sourceData
		aString* pn = NEW aString(sourceData.popName_[pp]);
		popName_.Append(pn);
		popSize_.Append(sourceData.popSize_[pp]);
		aVector<long double>& v = sourceData.localityCoordinates_[pp];
		localityCoordinates_.ConcatRow(v);
		rawTraits_.Append(NEW ObjList<TextDataList>(sourceData.rawTraits_[pp]));
		}//end for p
	ResetFlags();
	GroupMembership();
	LocalityMembership();
	}//end ctor EvolutionaryGeneticsData(EvolutionaryGeneticsData&, aList<int>&)


void  EvolutionaryGeneticsData::Initialize(void){
	EvolutionaryGeneticsData::Clear();
	}//Initialize()
	
	
void  EvolutionaryGeneticsData::Initialize(aString& s){
	EvolutionaryGeneticsData::Initialize();
	title_=s; 
	}//Initialize(aString&)
	
	
void  EvolutionaryGeneticsData::Initialize(EvolutionaryGeneticsData& egd){
	title_=egd.title_;
	npop_=egd.npop_;
	ntraits_=egd.ntraits_;
	ngroups_=egd.ngroups_;
	popSize_=egd.popSize_;
	groupName_=egd.groupName_;
	popName_=egd.popName_;
	localityName_=egd.localityName_;
	groupHierarchy_=egd.groupHierarchy_;
	popsByLocality_=egd.popsByLocality_;
	localityCoordinates_=egd.localityCoordinates_;
	traitName_=egd.traitName_;
	traitNum_=egd.traitNum_;
	skippedTraitNum_=egd.skippedTraitNum_;
	rawTraits_=egd.rawTraits_;
	missing_=egd.missing_;
	}//Initialize(EvolutionaryGeneticsData&)
	
	
void EvolutionaryGeneticsData::Initialize(EvolutionaryGeneticsData& sourceData, aList<int>& popSubsetNums){
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	title_=sourceData.title_; 
	npop_=popSubsetNums.len();
	ntraits_=sourceData.ntraits_;
	ngroups_=0;
	traitName_=sourceData.traitName_;
	traitNum_=sourceData.traitNum_;
	skippedTraitNum_=sourceData.skippedTraitNum_;
	rawTraits_=sourceData.rawTraits_;
	missing_=sourceData.missing_;
	popSubsetNums.Sort();
	int p, pp;
	for(p=0;p<popSubsetNums.len();++p){
		pp = popSubsetNums[p];//population num in sourceData
		aString* pn = NEW aString(sourceData.popName_[pp]);
		popName_.Append(pn);
		popSize_.Append(sourceData.popSize_[pp]);
		aVector<long double>& v = sourceData.localityCoordinates_[pp];
		localityCoordinates_.ConcatRow(v);
		rawTraits_.Append(NEW ObjList<TextDataList>(sourceData.rawTraits_[pp]));
		}//end for p
	ResetFlags();
	GroupMembership();
	LocalityMembership();
	}//end ctor EvolutionaryGeneticsData(EvolutionaryGeneticsData&, aList<int>&)


EvolutionaryGeneticsData&  EvolutionaryGeneticsData::operator=(EvolutionaryGeneticsData& egd){
	title_=egd.title_;
	npop_=egd.npop_;
	ntraits_=egd.ntraits_;
	ngroups_=egd.ngroups_;
	popSize_=egd.popSize_;
	groupName_=egd.groupName_;
	popName_=egd.popName_;
	localityName_=egd.localityName_;
	groupHierarchy_=egd.groupHierarchy_;
	popsByLocality_=egd.popsByLocality_;
	localityCoordinates_=egd.localityCoordinates_;
	traitName_=egd.traitName_;
	traitNum_=egd.traitNum_;
	skippedTraitNum_=egd.skippedTraitNum_;
	rawTraits_=egd.rawTraits_;
	missing_=egd.missing_;
	return *this;
	}//operator=(EvolutionaryGeneticsData&)




void EvolutionaryGeneticsData::Clear(void){
	popSize_.Clear();
	groupName_.Clear();
	popName_.Clear();
	localityName_.Clear();
	groupHierarchy_.Clear();
	popsByLocality_.Clear();
	localityCoordinates_.Clear();
	traitName_.Clear();
	traitNum_.Clear();
	skippedTraitNum_.Clear();
	rawTraits_.Clear();
	ResetFlags();
	title_.clear();	//title_ of the data set from the first data line
	missing_.Clear();
	npop_ = ntraits_ = ngroups_  = 0;
	}//end EvolutionaryGeneticsData::Clear()




void  EvolutionaryGeneticsData::CreateBootstrapData(EvolutionaryGeneticsData& origData, Ran3& ran){
	//populaitons and individuals resampled
	Clear();
	title_ = origData.title_;	//title_ of the data set from the first data line
	title_.cat(" bootstrap replicate");
	ngroups_ = origData.ngroups_;
	npop_ = origData.npop_;
	ntraits_ = origData.ntraits_;
	popName_ = origData.popName_;
	groupName_ = origData.groupName_;
	localityName_ = origData.localityName_;
	traitName_ = origData.traitName_;
	traitNum_ = origData.traitNum_;
	skippedTraitNum_ = origData.skippedTraitNum_;
	groupHierarchy_ = origData.groupHierarchy_;
	popsByLocality_ = origData.popsByLocality_;
	localityCoordinates_ = origData.localityCoordinates_;
	missing_ = origData.missing_;
	int pop, i;
	for(pop=0;pop<npop_;++pop){//randomly resamples among, then within, populations
		rawTraits_.Append(NEW ObjList<TextDataList>());
		int newpop = ran.longBetween(0,npop_-1);
		int newpopsize = origData.rawTraits_[newpop].len();
		popSize_.Append(newpopsize);
		for (i=0;i<newpopsize;++i){
			int ranIndivID = ran.longBetween(0,newpopsize-1);
			TextDataList* ranIndiv = NEW TextDataList(origData.rawTraits_[newpop][ranIndivID]);
			rawTraits_[pop].Append(ranIndiv);
			ranIndiv = NULL;//forget
		}}
	}//end  EvolutionaryGeneticsData::CreateBootstrapData(EvolutionaryGeneticsData&, Ran3&)



void  EvolutionaryGeneticsData::CreateBootstrapDataConstPop(EvolutionaryGeneticsData& origData, Ran3& ran){
	//populaitons and individuals resampled
	Clear();
	title_ = origData.title_;	//title_ of the data set from the first data line
	title_.cat(" bootstrap replicate");
	ngroups_ = origData.ngroups_;
	npop_ = origData.npop_;
	ntraits_ = origData.ntraits_;
	popName_ = origData.popName_;
	groupName_ = origData.groupName_;
	localityName_ = origData.localityName_;
	traitName_ = origData.traitName_;
	traitNum_ = origData.traitNum_;
	skippedTraitNum_ = origData.skippedTraitNum_;
	groupHierarchy_ = origData.groupHierarchy_;
	popsByLocality_ = origData.popsByLocality_;
	localityCoordinates_ = origData.localityCoordinates_;
	missing_ = origData.missing_;
	int pop,i;
	for(pop=0;pop<npop_;++pop){//randomly resamples among, then within, populations
		rawTraits_.Append(NEW ObjList<TextDataList>());
		int newpop = pop;//i.e., not resampled
		int newpopsize = origData.rawTraits_[newpop].len();
		popSize_.Append(newpopsize);
		for (i=0;i<newpopsize;++i){
			int ranIndivID = ran.longBetween(0,newpopsize-1);
			TextDataList* ranIndiv = NEW TextDataList(origData.rawTraits_[newpop][ranIndivID]);
			rawTraits_[pop].Append(ranIndiv);
			ranIndiv = NULL;//forget
		}}
	}//end  EvolutionaryGeneticsData::CreateBootstrapDataConstPop(EvolutionaryGeneticsData&, Ran3&)




void EvolutionaryGeneticsData::CullTraits (aList<int>& traitsToCull){
	//This function shifts trait numbers from from traitNums_ to the skippedTraitNums_ list
	if(traitsToCull.len()==0){return;}//none to cull
	for(int tr=0;tr<traitsToCull.len();++tr){
		int cTrait = traitsToCull[tr];//make sure it's a valid cull step
		if(traitNum_.Includes(cTrait) && cTrait>=0 && cTrait<traitName_.len()){
			long item = traitNum_.Loc(cTrait);
			skippedTraitNum_.Append(cTrait);
			traitNum_.Delete(item);
			}//if item not skipped already
		}//tr
	skippedTraitNum_.Sort();
	ResetFlags();
	}//end procedure EvolutionaryGeneticsData::CullTraits(aList<int>&)



void EvolutionaryGeneticsData::CullTraits (StringList& traitsToCull){
	//convert names to numbers and cull
	aList<int> cullListByNum;
	int tr, item;
	for(tr=0;tr<traitsToCull.len();++tr){
		aString cTraitname = traitsToCull[tr];
		item = traitName_.Loc(&cTraitname);
		if((item>=0)&&(!(skippedTraitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already skipped
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	CullTraits(cullListByNum);
	}//end procedure EvolutionaryGeneticsData::CullTraits(StringList&)



void EvolutionaryGeneticsData::CullTrait (char* traitToCull){
	aString trToCull(traitToCull);
	CullTrait(trToCull);
	}//end EvolutionaryGeneticsData::CullTrait (char* )



void EvolutionaryGeneticsData::CullTrait (aString& traitToCull){
	StringList traitsToCull;
	traitsToCull.Append(NEW aString(traitToCull));
	CullTraits(traitsToCull);
	}//end EvolutionaryGeneticsData::CullTrait (aString& )



void EvolutionaryGeneticsData::CullTrait (int traitToCull){
	aList<int> traitsToCull;
	traitsToCull.Append(traitToCull);
	CullTraits(traitsToCull);
	}//end EvolutionaryGeneticsData::CullTrait (int)


/*
void EvolutionaryGeneticsData::CullMonomorphicTraits (void){	
	//This function culls the monomorphic traits from each trait-data string
	//and includes them among the skipped traits
	UpdateVariableTraits();
	if(variableTraits_.len()==traitName_.len()){return;}//none monomorphic
	StringList monomorphicTraits(traitName_);
	for(int vTr=0; vLoc<variableTraits_.len();++vTr){
		aString vtraitname = traitName_[variableTraits_[vTr]];
		int item = monomorphicTraits.Loc(&vtraitname);
		monomorphicTraits.Delete(item);
		}//vTr
	CullTraits(monomorphicTraits);
	}//end procedure EvolutionaryGeneticsData::CullMonomorphicTraits
*/




EvolutionaryGeneticsData* 	EvolutionaryGeneticsData::CullToOneGroup (int groupNum){
	//virtual-copy and do the work in derived classes
	EvolutionaryGeneticsData *subsetData = Duplicate();
	subsetData->Clear();
	subsetData->CullToOneGroup(*this, groupNum);
	return subsetData;	
	}//end EvolutionaryGeneticsData::CullToOneGroup (int)



void 	EvolutionaryGeneticsData::CullToOneGroup (EvolutionaryGeneticsData& parentData, int groupNum){
	//makes a subset data structure from the specified group
	//individuals resampled
	title_ = parentData.title_;
	title_.cat(" -- ");
	title_.cat(parentData.groupName_[groupNum]);
	title_.cat(" only");

	npop_ = parentData.groupHierarchy_[groupNum].len();
	ngroups_ = 1;
	groupName_.Append(NEW aString(parentData.groupName_[groupNum]));
	localityName_ = parentData.localityName_;//is this right?
	traitName_ = parentData.traitName_;
	traitNum_ = parentData.traitNum_;
	skippedTraitNum_ = parentData.skippedTraitNum_;
	missing_ = parentData.missing_;
	int pop;
	for(pop=0;pop<npop_;++pop){
		int theRightPop = parentData.groupHierarchy_[groupNum][pop];
		popName_.Append(NEW aString(parentData.popName_[theRightPop]));
		localityCoordinates_.ConcatRow(parentData.localityCoordinates_[theRightPop]);
		rawTraits_.Append(parentData.rawTraits_[theRightPop].Duplicate());
		}
	
	
	GroupMembership();
	LocalityMembership();
	ResetFlags();
	}//end EvolutionaryGeneticsData::CullToOneGroup (EvolutionaryGeneticsData&,int)


	
	

int	EvolutionaryGeneticsData::GetIndivNumInDataSequence(int pop, int indiv){
	//returns a surrogate ID number, when numerical ID is needed: 0-indexed
	int idVal=0, p;
	for(p=0;p<pop;++p){
		idVal += popSize(p);}
	idVal += indiv;
	return idVal;
	}//EvolutionaryGeneticsData::GetIndivNumInDataSequence(int pop, int indiv)



long EvolutionaryGeneticsData::GetIndividualsSampled (int pop, int trait){
	long n=0, i;
	for(i=0;i<individuals(pop);++i){
		TextDatum *dat = (TextDatum*) &(rawTraits_[pop][i][traitNum_[trait]]);
		if(dat->OK()){
			++n;}}
	return n;
	}//end  EvolutionaryGeneticsData::GetIndividualsSampled


int EvolutionaryGeneticsData::GetLocality (int pop){
	for(int L=0;L<popsByLocality_.len();++L){
		if (popsByLocality_[L].Includes(pop)){
			return L;}}
	return (-1);//default if failure
	}//end  EvolutionaryGeneticsData::GetLocality


long EvolutionaryGeneticsData::GetTraitSampleSize (int trait){//pooling populations
	long n=0;
	int pop;
	for(pop=0;pop<npop_;++pop){
		n += GetIndividualsSampled(pop,trait);
		}
	return n;
	}//end  PopulationGeneticData::GetTraitSampleSize


long EvolutionaryGeneticsData::GetTraitSampleSize (int pop, int trait){
	return GetIndividualsSampled(pop,trait);
	}//end  EvolutionaryGeneticsData::GetTraitSampleSize



void EvolutionaryGeneticsData::GroupMembership (void){
	//This routine determines group membership by the last full word in
	//the population title_.  The first population determines the name of the 
	//first group, the next different population determines the name of the secondgroup, etc.

	//groupName saves the names of the groups, in the order encountered
	//groupHierarchy saves the population numbers from the
	//	original population series, so they can be retrieved from group membership data.
	//groupSize saves the number of populations in a group, indexed by group number
	groupName_.Clear();
	groupHierarchy_.Clear();
	if (ngroups_ <= 1){//all populations belong to the same group
		groupName_.Append(NEW aString("pooled populations"));
		groupHierarchy_.Append(NEW aList<int>());
		ngroups_ = 1;//reset this
		}
	int groupNum=0, pop;
	for (pop=0;pop<npop_;++pop){
		if (ngroups_>1){
			// there is more than one group		
			// first get the name of the group from the population name string
			aString popStringCopy(popName_[pop]);
			aString *thisGName = popStringCopy.ChopOffLastWord();
			//does this group already exist?
			if(groupName_.Includes(thisGName)){
					groupNum = groupName_.Loc(thisGName);}
				else{//add it to the list & add slots to corresponding lists
					++groupNum;
					groupName_.Append(NEW aString(thisGName));
					groupHierarchy_.Append(NEW aList<int>());
					}
			delete thisGName; thisGName=0;
			}//end if ngroups>1
		groupHierarchy_[groupNum].Append(pop);
		}//end for pop
	}//end  EvolutionaryGeneticsData::GroupMembership


int EvolutionaryGeneticsData::GroupMembership(int pop){
	if(groupHierarchy_.len()==0){
		GroupMembership();}
	int g;
	for(g=0;g<groupHierarchy_.len();++g){
		if (groupHierarchy_[g].Includes(pop)){
			return g;}}
	aString err("ERROR!  Population # ");err.cat(pop);err.cat(" (0-indexed) not in any group.  group=-1 returned.");
	aError(err);
	return -1;
	}//end  EvolutionaryGeneticsData::GroupMembership(int)



int EvolutionaryGeneticsData::individuals(int pop, int trait){
	return GetIndividualsSampled(pop,trait);
	}//end  EvolutionaryGeneticsData::individuals(int,int)
	


void EvolutionaryGeneticsData::LocalityMembership (void){
	//This routine determines locality membership by the first full word in
	//the population title_.  The first population determines the name of the 
	//first locality, the next different population determines the name of the second locality, etc.

	//localityName_ saves the names of the localities, in the order encountered
	//popsByLocality saves the population numbers from the
	//	original population series, so they can be retrieved from locality membership data
	
	//localityCoordinates_ is determined by what is in (parentheses) in the pop title line
	localityName_.Clear();
	popsByLocality_.Clear();
	localityCoordinates_.Clear();
	aString endCharList(" \t,;:\"(){}[]!?\n\r"),openParen("("),closeParen(")");
	int localityNum=0;
	int pop;
	for (pop=0;pop<npop_;++pop){
		// there is more than one group		
		// first get the name of the group from the population name string
		aString popStringCopy(popName_[pop]);
		int firstbracePos = popStringCopy.pos(openParen);
		int lastbracePos = popStringCopy.pos(closeParen);
		aString coordStr(popStringCopy,firstbracePos,lastbracePos);
		aVector<long double> coords;
		while(coordStr.len()>0){
			aString *coordVal=coordStr.ChopOffFirstWord(endCharList);
			long double coord = coordVal->toLongDouble();
			coords.Append(coord);
			delete coordVal; coordVal=NULL;
			}//end while
		if(coords.len()==0){
			coords.Append(zero);}//dummy value
		localityCoordinates_.ConcatRow(coords);
		
		popStringCopy.cut(firstbracePos,lastbracePos);//ignore coordinates in further processing
		aString *thisLName = popStringCopy.ChopOffFirstWord(endCharList);

		//does this locality already exist?
		if(localityName_.Includes(thisLName)){
				localityNum = localityName_.Loc(thisLName);}
			else{//add it to the list & add slots to corresponding lists
				localityName_.Append(NEW aString(thisLName));
				popsByLocality_.Append(NEW aList<int>());
//				++localityNum;
				localityNum = localityName_.len()-1;
				}
		delete thisLName; thisLName=0;
		popsByLocality_[localityNum].Append(pop);
		}//end for pop
	}//end  EvolutionaryGeneticsData::LocalityMembership()


int EvolutionaryGeneticsData::LocalityMembership(int pop){
	if(popsByLocality_.len()==0){
		LocalityMembership();}
	int L;
	for(L=0;L<popsByLocality_.len();++L){
		if (popsByLocality_[L].Includes(pop)){
			return L;}}
	aString err("ERROR!  Population # ");err.cat(pop);err.cat(" not in any locality.  Locality=-1 returned.");
	aError(err);
	return -1;
	}//end  EvolutionaryGeneticsData::LocalityMembership(int)




void EvolutionaryGeneticsData::PrintTraits (ostream& outFile, int traitsPerLine){//file assumed open
	int lineSpot = 0, tr;
	for (tr=0;tr<traitNum_.len();++tr){ 
		aString& trName = traitName_[traitNum_[tr]];
		outFile<<trName;
		++lineSpot;
		if ((tr == traitNum_.len()-1) || (lineSpot == traitsPerLine)) {
				outFile<<endl;}
			else{
				outFile<<", ";}
		}//end for tr
	outFile<<endl;
	}//end EvolutionaryGeneticsData::PrintTraits




void EvolutionaryGeneticsData::PrintPopKey (ostream& outFile, char colSpacer){//file assumed open
	//Makes a key to the populations studied, for use in
	//interpreting some of the data output matrices.
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied
	char c = colSpacer;

	outFile<<title_<<endl<<endl;
	outFile<<"Key to populations"<<endl<<endl;
	outFile<<"Number";
	if(asciiFormat){outFile.width(14);}else{outFile<<c;}
	outFile<<"Name"<<endl;
	aString blanks = aString(' ',7);
	for (int pop=0;pop<npop_;++pop){
		aString& popTitle = popName_[pop];
		if(asciiFormat){outFile.width(3);outFile<< pop+1<<blanks<<popTitle<<endl;}
			else{outFile<< pop+1<<c<<popTitle<<endl;}
		}
	outFile<<endl<<endl;
	if (ngroups_ > 1){
		outFile<<"Key to group membership for hierarchical analyses"<<endl<<endl;
		int group, gpop,pop;
		for (group=0;group<ngroups_;++group){
			aString& groupTitle = groupName_[group];
			if(asciiFormat){outFile.width(2);outFile<<group+1<<".  "<<groupTitle<<endl;}
				else{outFile<<group+1<<"."<<c<<groupTitle<<endl;}
			aList<int>& groupList = groupHierarchy_[group];
			for (gpop=0;gpop<groupList.len();++gpop){
				pop = groupList[gpop];
				aString& popTitle = popName_[pop];
				if(asciiFormat){outFile<<blanks<<popTitle<<endl;}
					else{outFile<<c<<popTitle<<endl;}
				}//end for gpop
			outFile<<endl;
			}//end for group
		outFile<<endl<<endl;
		}//end if ngroups
	}//end EvolutionaryGeneticsData::PrintPopKey




void	 EvolutionaryGeneticsData::PrintSASFormatData(ostream& outFile, char colSpacer){
	//prints a data set that can be read using SAS
	//missing data represented by '.', columns separated by tabs (default)
	
	//first reset the missing-data character if needed
	bool resetMissing=false;
	StringList oldMissing, newMissing;
	if(missing_[0] != "."){
		resetMissing=true;
		oldMissing = newMissing = missing_;
		newMissing.Prepend(NEW aString("."));
		UpdateMissingDataFlags(newMissing);
		}
	aString titleLine("pop");//create relevant column titles
	titleLine + colSpacer + "group" + colSpacer + "locality";
	int c0,pop,tr0, i, c1,tr1;;
	for(c0=0;c0<localityCoordinates_[0].len();++c0){
		titleLine + colSpacer + "coord[";
		titleLine.cat(c0+1);
		titleLine + "]";}
	titleLine + colSpacer + "indiv";
	for(tr0=0;tr0<traits();++tr0){
		aString& trName = traitName_[traitNum_[tr0]];
		titleLine + colSpacer + trName;
		}//tr0
	outFile<<titleLine<<endl;

	for(pop=0;pop<npop_;++pop){
		aString popStr(pop+1);
		popStr+colSpacer;
		popStr.cat(groupName_[GroupMembership(pop)]);
		popStr+colSpacer;
		popStr.cat(localityName_[LocalityMembership(pop)]);
		for(c1=0;c1<localityCoordinates_[pop].len();++c1){
			popStr + colSpacer;
			popStr.cat(localityCoordinates_[pop][c1]);}
			popStr + colSpacer;
		for(i=0;i<popSize_[pop];++i){
			aString indivStr(popStr);
			indivStr.cat(i+1);
			for(tr1=0;tr1<traits();++tr1){
				aString& trValue = rawTraits_[pop][i][traitNum_[tr1]];
				indivStr + colSpacer + trValue;
				}//tr1
			outFile<<indivStr<<endl;
			}//i
		}//pop
	outFile<<endl;		
	if(resetMissing){//restore it
		UpdateMissingDataFlags(oldMissing);}
	}//end  EvolutionaryGeneticsData::PrintSASFormatData(ostream&,char)



	
void	 EvolutionaryGeneticsData::PrintSASFormatData(aString& outFileName,char colSpacer){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintSASFormatData(dataFile, colSpacer);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end EvolutionaryGeneticsData::PrintSASFormatData(aString&,char)






void EvolutionaryGeneticsData::RestoreTraits (aList<int>& traitsToRestore){
	//This function shifts trait numbers from from skippedTraitNums_ to the traitNums_ list
	if(traitsToRestore.len()==0){return;}//none to cull
	int tr, rTrait;
	for(tr=0;tr<traitsToRestore.len();++tr){
		rTrait = traitsToRestore[tr];//make sure it's a valid cull step
		if(skippedTraitNum_.Includes(rTrait) && rTrait>=0 && rTrait<traitName_.len()){
			long item = skippedTraitNum_.Loc(rTrait);
			traitNum_.Append(rTrait);
			skippedTraitNum_.Delete(item);
			}//if item not restored already
		}//tr
	traitNum_.Sort();
	ResetFlags();
	}//end procedure EvolutionaryGeneticsData::RestoreTraits(aList<int>&)



void EvolutionaryGeneticsData::RestoreTraits (StringList& traitsToRestore){
	//convert names to numbers and restore
	aList<int> restoreListByNum;
	int tr, item;
	for(tr=0;tr<traitsToRestore.len();++tr){
		aString rTraitname = traitsToRestore[tr];
		item = traitName_.Loc(&rTraitname);
		if((item>=0)&&(!(traitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already restored
			restoreListByNum.Append(item);
			}//if item exists
		}//tr
	RestoreTraits(restoreListByNum);
	}//end procedure EvolutionaryGeneticsData::RestoreTraits(StringList&)



void EvolutionaryGeneticsData::RestoreTrait (char* traitToRestore){
	aString trToCull(traitToRestore);
	RestoreTrait(trToCull);
	}//end EvolutionaryGeneticsData::RestoreTrait (char* )



void EvolutionaryGeneticsData::RestoreTrait (aString& traitToRestore){
	StringList traitsToRestore;
	traitsToRestore.Append(NEW aString(traitToRestore));
	RestoreTraits(traitsToRestore);
	}//end EvolutionaryGeneticsData::RestoreTrait (aString& )



void EvolutionaryGeneticsData::RestoreTrait (int traitToRestore){
	aList<int> traitsToRestore;
	traitsToRestore.Append(traitToRestore);
	RestoreTraits(traitsToRestore);
	}//end EvolutionaryGeneticsData::RestoreTrait (int)



aString& EvolutionaryGeneticsData::Trait(int pop, int indiv, int trait){
	//use for printing; skips traits appropriately & returns 'missing character' for indivs without data
	return rawTraits_[pop][indiv][traitNum_[trait]];
	}//end  EvolutionaryGeneticsData::Trait(int,int,int)


void EvolutionaryGeneticsData::TraitNamesFromSublist(StringList& names, aList<int>& itemNumbers){
	//use to get trait names
	names.Clear();
	for(int i=0;i<itemNumbers.len();++i){
		if(itemNumbers[i]<traitName_.len()){
			names.Append(NEW aString(traitName_[itemNumbers[i]]));}}
	}//EvolutionaryGeneticsData::TraitNamesFromSublist(StringList&,aList<int>&)


void EvolutionaryGeneticsData::UpdateMissingDataFlags(StringList& useForMissing){
	if(useForMissing.len()==0){
		aError("ERROR in EvolutionaryGeneticsData::UpdateMissingDataFlag(): missing data flags not specified");}
	missing_ = useForMissing;
	long pop,i;
	for(pop=0;pop<rawTraits_.len();++pop){
		for(i=0;i<rawTraits_[pop].len();++i){
			rawTraits_[pop][i].SetMissingStr(useForMissing[0]);}}
	}//EvolutionaryGeneticsData::UpdateMissingDataFlags(StringList&)




#endif




