//unit HaploidMarkerData.cpp

#ifndef HaploidMarkerDatacpp
#define HaploidMarkerDatacpp = 1


#include "HaploidMarkerData.h"
#include "Element.cpp"
#include "aList.cpp"
#include "SortList.cpp"
#include "StatList.cpp"
#include "GeneContainers.cpp"




HaploidMarkerData::HaploidMarkerData(HaploidMarkerData& parentData):
			EvolutionaryGeneticsData(parentData),
			nlociHap_(parentData.nlociHap_),
			alleleLen_(parentData.alleleLen_),
			allHapLoci_(parentData.allHapLoci_),
			locusNumHap_(parentData.locusNumHap_),
			skippedLocusNumHap_(parentData.skippedLocusNumHap_),
			variableLocusNumHap_(parentData.variableLocusNumHap_), 
			allelesPresentHap_(parentData.allelesPresentHap_),
			alleleFreqHap_(parentData.alleleFreqHap_),
			alleleFreqHapGlobal_(parentData.alleleFreqHapGlobal_),
			nByLocusHap_(parentData.nByLocusHap_),
			allelesCurrentHap_(parentData.allelesCurrentHap_),
			varLociCurrentHap_(parentData.varLociCurrentHap_),
			alleleFreqsCurrentHap_(parentData.alleleFreqsCurrentHap_)
			{	
		}//end ctor HaploidMarkerData(HaploidMarkerData&)


HaploidMarkerData::HaploidMarkerData(HaploidMarkerData& parentData, 
						aList<int>& popSubsetNums):
			EvolutionaryGeneticsData(parentData,popSubsetNums),
			nlociHap_(parentData.nlociHap_),
			alleleLen_(parentData.alleleLen_),
			allHapLoci_(parentData.allHapLoci_),
			locusNumHap_(parentData.locusNumHap_),
			skippedLocusNumHap_(parentData.skippedLocusNumHap_)
			{
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	title_.prepend("subset of dataset: ");
	ResetFlags();
	CheckForNewAlleles();
	UpdateVariableLociHap();
	}//end ctor HaploidMarkerData(HaploidMarkerData&, aList<int>&)


void  HaploidMarkerData::Initialize(void){
	HaploidMarkerData::Clear();
	}//Initialize()
	
	
void  HaploidMarkerData::Initialize(aString& s){
	HaploidMarkerData::Clear();
	title_=s; 
	}//Initialize(aString&)
	
	
void  HaploidMarkerData::Initialize(HaploidMarkerData& dmd){
	EvolutionaryGeneticsData::Initialize(dmd);
	nlociHap_=dmd.nlociHap_;
	alleleLen_=dmd.alleleLen_;
	allHapLoci_=dmd.allHapLoci_;
	locusNumHap_=dmd.locusNumHap_; 
	skippedLocusNumHap_=dmd.skippedLocusNumHap_;
	variableLocusNumHap_=dmd.variableLocusNumHap_;
	allelesPresentHap_=dmd.allelesPresentHap_;
	alleleFreqHap_=dmd.alleleFreqHap_;
	alleleFreqHapGlobal_=dmd.alleleFreqHapGlobal_;
	nByLocusHap_=dmd.nByLocusHap_;
	allelesCurrentHap_=dmd.allelesCurrentHap_;
	varLociCurrentHap_=dmd.varLociCurrentHap_;
	alleleFreqsCurrentHap_=dmd.alleleFreqsCurrentHap_;
	}//Initialize(HaploidMarkerData&)
	

void  HaploidMarkerData::Initialize(HaploidMarkerData& parentData, aList<int>& popSubsetNums){
	EvolutionaryGeneticsData::Initialize(parentData,popSubsetNums);
	nlociHap_=parentData.nlociHap_;
	alleleLen_=parentData.alleleLen_;
	allHapLoci_=parentData.allHapLoci_;
	locusNumHap_=parentData.locusNumHap_;
	skippedLocusNumHap_=parentData.skippedLocusNumHap_;
	CheckForNewAlleles();
	UpdateVariableLociHap();
	}//end Initialize(HaploidMarkerData&, aList<int>&)


	
HaploidMarkerData&  HaploidMarkerData::operator=(HaploidMarkerData& dmd){
	EvolutionaryGeneticsData::operator=(dmd);
	nlociHap_=dmd.nlociHap_;
	alleleLen_=dmd.alleleLen_;
	allHapLoci_=dmd.allHapLoci_;
	locusNumHap_=dmd.locusNumHap_; 
	skippedLocusNumHap_=dmd.skippedLocusNumHap_;
	variableLocusNumHap_=dmd.variableLocusNumHap_;
	allelesPresentHap_=dmd.allelesPresentHap_;
	alleleFreqHap_=dmd.alleleFreqHap_;
	alleleFreqHapGlobal_=dmd.alleleFreqHapGlobal_;
	nByLocusHap_=dmd.nByLocusHap_;
	allelesCurrentHap_=dmd.allelesCurrentHap_;
	varLociCurrentHap_=dmd.varLociCurrentHap_;
	alleleFreqsCurrentHap_=dmd.alleleFreqsCurrentHap_;
	return *this;
	}//operator=(HaploidMarkerData&)




void HaploidMarkerData::CalculateHaplotypeFreqs (void){
	//This routine counts marker frequencies, taking into account
	//markers left blank in the data set.
	if (alleleFreqsCurrentHap_){return;}
	CheckForNewAlleles();
	long pop, i, loc, marker;
	long double freq;
//	char blank = ' ', tab='\t';	//don't count these
	aString missingDataFlags = ".?-*";//characters used to signify missing data
	aString endCharList = " \t,";//characters used to separate data fields
	alleleFreqHap_.Clear();
	nByLocusHap_.Clear();
	for (pop=0;pop< npop_;++pop){ 
		//set up a frequency list for this population
		alleleFreqHap_.Append( NEW ObjList<aList<long double> >());
		nByLocusHap_.Append(NEW aList<int>());
		for (loc=0;loc<nlociHap_;++loc){//loops through dominant loci
			//first set up a blank array of frequencies for each possible phenotype
			alleleFreqHap_[pop].Append(NEW aList<long double>());
			StringList& markerNamesList = allelesPresentHap_[loc];
			freq = (long double) 0.0;
			for (marker=0;marker<markerNamesList.len();++marker){
				alleleFreqHap_[pop][loc].Append(freq);}//initialize

			//next look at the phenotype of each individual and tally the frequency
			long nSampled = 0;	//the number of individuals
			ObjList<TextDataList>& phenotypesList = rawTraits_[pop];
			for (i=0;i< phenotypesList.len();++i){
				TextDatum& phene = phenotypesList[i][locusNumHap_[loc]];
				if (phene.OK()) {//if not, it's missing data
					//increment the frequency & sample size -- divide later
					marker = markerNamesList.Loc(&phene);
					alleleFreqHap_[pop][loc][marker]++;
					nSampled++;
					}//end if legal data
				}//end for i


			//now go through and divide by the sample size to restore the frequencies
			if(nSampled>0){
				for (marker=0;marker<markerNamesList.len();++marker){
					alleleFreqHap_[pop][loc][marker] /= (long double) nSampled;
					}
				}
			//store the sample size for this locus
			nByLocusHap_[pop].Append((int)nSampled);
			}//end for loc
		}//end for pop
	alleleFreqHapGlobal_.Clear();
	for(loc=0;loc<nlociHap_;++loc){
		alleleFreqHapGlobal_.Append(NEW aList<long double>());
		for(marker=0;marker<allelesPresentHap_[loc].len();++marker){
			freq=(long double)0.0;
			for(pop=0;pop<npop_;++pop){
				freq += alleleFreqHap_[pop][loc][marker];}
			alleleFreqHapGlobal_[loc].Append(freq/(long double)npop_);
			}}
	alleleFreqsCurrentHap_=1;
	}//end HaploidMarkerData::CalculatePhenotypeFreqs



ObjList<aList<long double> >* HaploidMarkerData::CalculateHaplotypeFreqsByPopSet(aList<int>& popSet, aList<long double>& nList){
 	//returns list of loci, each with list of marker freqs, using total marker set
 	CalculateHaplotypeFreqs();
 	nList.Clear();
 	ObjList<aList<long double> >* popSetFreqsPtr = NEW ObjList<aList<long double> >(alleleFreqHap_[0]);//copies structure
 	ObjList<aList<long double> >& popSetFreqs = *popSetFreqsPtr;//notational convenience
 	for(int loc=0;loc<nlociHap_;++loc){
 		int markers = GetNumAllelesHap(loc);
 		long double samSizeSet = (long double) 0.0;
 		for(int m=0;m<markers;++m){
 			long double nCopies = (long double) 0.0;
 			for(int popSetPop=0;popSetPop<popSet.len();++popSetPop){
 				int pop = popSet[popSetPop];
 				long double nSample = (long double) (nByLocusHap_[pop][loc]);
 				if(m==0){samSizeSet += nSample;}
 				nCopies += nSample * GetFreqHap(pop,loc,m);
 				}
 			if(m==0){nList.Append(samSizeSet);}
 			if(nCopies==0.0){
 					popSetFreqs[loc][m]=(long double) 0.0;}
 				else if(nCopies==samSizeSet){
 					popSetFreqs[loc][m]=(long double) 1.0;}
 				else{
 					popSetFreqs[loc][m] = nCopies / samSizeSet;
 					}
 			}//end for m
 		}//end for loc
 	return popSetFreqsPtr;
	}//end HaploidMarkerData::CalculateHaplotypeFreqsByPopSet



void HaploidMarkerData::CheckForNewAlleles (void){
	if (allelesCurrentHap_){return;}
	allelesPresentHap_.Clear();
	for (int loc=0;loc<nlociHap_;++loc){//initialize
		allelesPresentHap_.Append(NEW StringList());}
	for (int pop=0;pop<npop_;++pop){
		ObjList<TextDataList>& ptypeList = rawTraits_[pop];
		for(int i=0;i<ptypeList.len();++i){
			for (int loc=0;loc<nlociHap_;++loc){
				TextDatum& phene = ptypeList[i][locusNumHap_[loc]];
				if(phene.OK()){//ignore missing data
					StringList& alleleList = allelesPresentHap_[loc];
					if(!(alleleList.Includes(&phene))){
						alleleList.Append(NEW aString(phene));}
					}//OK
				}//end for loc
			}//end for i
		}//end for pop
	for (int loc=0;loc<nlociHap_;++loc){
		allelesPresentHap_[loc].Sort();}
	allelesCurrentHap_=true;
	UpdateVariableLociHap();
	}//end HaploidMarkerData::CheckForNewAlleles



void HaploidMarkerData::Clear(void){
	EvolutionaryGeneticsData::Clear();
	allHapLoci_.Clear();
	locusNumHap_.Clear();
	skippedLocusNumHap_.Clear();
	variableLocusNumHap_.Clear();
	allelesPresentHap_.Clear();
	alleleFreqHap_.Clear();
	alleleFreqHapGlobal_.Clear();
	nByLocusHap_.Clear();
	nlociHap_=allelesCurrentHap_=varLociCurrentHap_=alleleFreqsCurrentHap_=0;
	}//end HaploidMarkerData::Clear()





void  HaploidMarkerData::CreateBootstrapData(HaploidMarkerData& origData, Ran3& ran){
	EvolutionaryGeneticsData::CreateBootstrapData(origData,ran);
	CheckForNewAlleles();	
	}//end  HaploidMarkerData::CreateBootstrapData(EvolutionaryGeneticsData&)



void  HaploidMarkerData::CreateBootstrapDataConstPop(HaploidMarkerData& origData, Ran3& ran){
	EvolutionaryGeneticsData::CreateBootstrapDataConstPop(origData,ran);
	CheckForNewAlleles();	
	}//end  HaploidMarkerData::CreateBootstrapDataConstPop(EvolutionaryGeneticsData&)




void HaploidMarkerData::CreateRandomData(
							const long double Fst, 
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	Clear();
	title_ = "Randomly generated haploid data set with parametric Fst=";
	title_.cat(Fst);  
	title_.cat(" and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numPopsSampled;
	nlociHap_ = numLociSampled;
	ngroups_ = 1;
	traitName_.Append(NEW aString("ID"));
	int locusCount=0;
	traitNum_.Append(locusCount++);
	for(int loc = 0;loc<nlociHap_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',Digits(nlociHap_));
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(locusCount++);
		allHapLoci_.Append(loc+1);
		locusNumHap_.Append(loc+1);
		}
	ntraits_ = traitNum_.len();

	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
	GeneFrequencyDistribution parametricFreqDistrib(Fst, pBar, 1);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	aString genotype;
	ObjList<StatList<long double> > locusFrequenciesByPop;
	for(int pop=0;pop<npop_;++pop){
		StatList<long double>* locusFrequencies = NEW StatList<long double>();
		locusFrequenciesByPop.Append(locusFrequencies);
		for(int loc=0;loc<nlociHap_;++loc){
			long double pf = parametricFreqDistrib.RandomFrequency(ran);
			locusFrequencies->Append(pf);
			}//end for loc
		}//end for pop

	for (int pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("randomly generated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
		for (int i=0;i<numIndivsPerPop;++i){
			rawTraits_[pop].Append(NEW TextDataList());
			aString indiv(i+1);
			indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));
			}//end for i
		popSize_.Append(numIndivsPerPop);

		//create and append genotypes
		for(int loc=0;loc<nlociHap_;++loc){
			StatList<long double>& locusFrequencies = locusFrequenciesByPop[pop];
			long double pp = locusFrequencies[loc];
			for(int i=0;i<numIndivsPerPop;++i){
				if (ran()<pp){genotype = "A";}
					else {genotype = "B";}
				rawTraits_[pop][i].Append(NEW TextDatum(genotype));
				}//end for i
			}//end for loc
		}//end for pop	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end HaploidMarkerData::CreateRandomData(...)


void HaploidMarkerData::CreateRandomData(
							const long double Fst, 
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
	//multiple alleles per locus, variable sample sizes per pop
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated haploid data set with parametric Fst=";
	title_.cat(Fst); 
	npop_ = numIndivsPerPopLoc.len();
	nlociHap_ = pBar.len();
	ngroups_ = 1;
	traitName_.Append(NEW aString("ID"));
	int locusCount=0;
	traitNum_.Append(locusCount++);
	for(int loc = 0;loc<nlociHap_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',Digits(nlociHap_));
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(locusCount++);
		allHapLoci_.Append(loc+1);
		locusNumHap_.Append(loc+1);
		}
	ntraits_ = traitNum_.len();

	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();

	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	ObjList<ObjList<StatList<long double> > > frequenciesByPopLocusAllele;
	for(int pop=0;pop<npop_;++pop){
		frequenciesByPopLocusAllele.Append(NEW ObjList<StatList<long double> >());}
	for(int loc=0;loc<nlociHap_;++loc){
		if(pBar[loc].len()>1){//locus is variable
				GeneFrequencyDistMultiAllele parametricFreqDistrib(Fst, pBar[loc], 1);//new version sends ploidy
				for(int pop=0;pop<npop_;++pop){
					aList<long double>* pp = parametricFreqDistrib.RandomFrequencies(ran);
					frequenciesByPopLocusAllele[pop].Append(NEW StatList<long double>(*pp));//copy as StatList
					delete pp;pp=NULL;
					}//end for pop
				}
			else{//locus is monomorphic
				for(int pop=0;pop<npop_;++pop){
					frequenciesByPopLocusAllele[pop].Append(NEW StatList<long double>(pBar[loc]));//copy as StatList
					}//end for pop
				}				
		}//end for loc

	for (int pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("randomly generated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		int maxIndivPop = 0;
		for (int loc=0;loc<numIndivsPerPopLoc[pop].len();++loc){
			maxIndivPop = MAX(maxIndivPop,numIndivsPerPopLoc[pop][loc]);}

		//create id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
		for (int i=0;i<maxIndivPop;++i){
			rawTraits_[pop].Append(NEW TextDataList());
			aString indiv(i+1);//first create id numbers for each individual
			indiv.PrependCharToLen(' ', MAX(4,Digits(maxIndivPop)));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));
			}//end for i
		popSize_.Append(maxIndivPop);

		for(int loc=0;loc<nlociHap_;++loc){
			StatList<long double>& pp = frequenciesByPopLocusAllele[pop][loc];
			if(pp.len()>1){//sampling required
					for(int i=0;i<numIndivsPerPopLoc[pop][loc];++i){
						long gtypeVal = ran.weighted(pp);
						char genotype = char('A' + (int)gtypeVal);
						rawTraits_[pop][i].Append(NEW TextDatum(genotype));
						}//end for i
					for(int i=numIndivsPerPopLoc[pop][loc];i<maxIndivPop;++i){//unsampled individuals blank
						rawTraits_[pop][i].Append(NEW TextDatum());//adds missing data
						}//end for i
					}
				else{//monomorphic locus
					for(int i=0;i<numIndivsPerPopLoc[pop][loc];++i){
						rawTraits_[pop][i].Append(NEW TextDatum("A"));
						}//end for i
					for(int i=numIndivsPerPopLoc[pop][loc];i<maxIndivPop;++i){//unsampled individuals blank
						rawTraits_[pop][i].Append(NEW TextDatum());//add missing data
						}//end for i
					}					
			}//end for loc
		}//end for pop	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end HaploidMarkerData::CreateRandomData(...)



void HaploidMarkerData::CullHapLoci (StringList& lociToCull){
	if(lociToCull.len()==0){return;}//none to cull
	aList<int> cullListByNum;
	for(int tr=0;tr<lociToCull.len();++tr){
		aString cTraitname = lociToCull[tr];
		int item = traitName_.Loc(&cTraitname);
		if((item>=0)&&(!(skippedTraitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already skipped
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	CullHapLoci(cullListByNum);
	}//end procedure HaploidMarkerData::CullHapLoci(StringList&)


void HaploidMarkerData::CullHapLoci (aList<int>& lociToCull){
	//This function shifts trait numbers from from traitNums_ to the skippedTraitNums_ list
	if(lociToCull.len()==0){return;}//none to cull
	for(int tr=0;tr<lociToCull.len();++tr){
		int cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(traitNum_.Includes(cLoc) && cLoc>=0 && cLoc<traitName_.len()){
			long item = traitNum_.Loc(cLoc);
			skippedTraitNum_.Append(cLoc);
			skippedLocusNumHap_.Append(cLoc);
			traitNum_.Delete(item);
			item = locusNumHap_.Loc(cLoc);
			locusNumHap_.Delete(item);
			}//if item not skipped already
		}//tr
	nlociHap_ = locusNumHap_.len();
	skippedTraitNum_.Sort();
	skippedLocusNumHap_.Sort();
	ResetFlags();
	CheckForNewAlleles();
	}//HaploidMarkerData::CullHapLoci (aList<int>&)



void HaploidMarkerData::CullHapLocus (char* locusToCull){
	aString locToCull(locusToCull);
	CullHapLocus(locToCull);
	}//end HaploidMarkerData::CullHapLocus (char* )


void HaploidMarkerData::CullHapLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	CullHapLoci(lociToCull);
	}//end HaploidMarkerData::CullHapLocus (aString& )


void HaploidMarkerData::CullMonomorphicLociHap (void){	
	//This function culls the monomorphic loci from each genotype data string
	//and includes them among the skipped loci
	UpdateVariableLociHap();
	if(variableLocusNumHap_.len()==locusNumHap_.len()){return;}//none monomorphic
	aList<int> monomorphicLoci(locusNumHap_);
	for(int vLoc=0; vLoc<variableLocusNumHap_.len();++vLoc){
		if(monomorphicLoci.Includes(variableLocusNumHap_[vLoc])){//polymorphic
			int item = monomorphicLoci.Loc(variableLocusNumHap_[vLoc]);
			monomorphicLoci.Delete(item);}
		}//vLoc
	CullHapLoci(monomorphicLoci);
	}//end procedure HaploidMarkerData::CullMonomorphicLociHap



void HaploidMarkerData::ExpungeLoci (StringList& lociToCull){
	if(lociToCull.len()==0){return;}//none to cull
	aList<int> cullListByNum;
	for(int tr=0;tr<lociToCull.len();++tr){
		aString cTraitname = lociToCull[tr];
		int item = traitName_.Loc(&cTraitname);
		if(item>=0){//the trait exists in the rawlist
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	ExpungeLoci(cullListByNum);
	}//end procedure HaploidMarkerData::ExpungeLoci(StringList&)


void HaploidMarkerData::ExpungeLoci (aList<int>& lociToCull){
	//This function removes trait numbers from the haploid data set; called during reclassification
	if(lociToCull.len()==0){return;}//none to cull
	int tr,cLoc, item;
	for(tr=0;tr<lociToCull.len();++tr){
		cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(allHapLoci_.Includes(cLoc)){
			item = allHapLoci_.Loc(cLoc);
			allHapLoci_.Delete(item);}
		if(locusNumHap_.Includes(cLoc)){
			item = locusNumHap_.Loc(cLoc);
			locusNumHap_.Delete(item);}
		if(skippedLocusNumHap_.Includes(cLoc)){
			item = skippedLocusNumHap_.Loc(cLoc);
			skippedLocusNumHap_.Delete(item);}
		}//tr
	nlociHap_ = locusNumHap_.len();
	ResetFlags();
	HaploidMarkerData::CheckForNewAlleles();
	}//HaploidMarkerData::ExpungeLoci (aList<int>&)



void HaploidMarkerData::ExpungeLocus (char* locusToCull){
	aString locToCull(locusToCull);
	ExpungeLocus(locToCull);
	}//end HaploidMarkerData::ExpungeLocus (char* )


void HaploidMarkerData::ExpungeLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	ExpungeLoci(lociToCull);
	}//end HaploidMarkerData::ExpungeLocus (aString& )



long double HaploidMarkerData::GetExpectedHeterozygosity (int pop, int loc, int allele){
	long double freq = GetFreqHap(pop,loc,allele);
	return freq * ((long double)1.0 - freq);
	}//HaploidMarkerData::GetExpectedHeterozygosity (int, int, int)
	
	
long double HaploidMarkerData::GetExpectedHeterozygosity (int pop, int loc){
	long double freq, homosum=(long double)0.0;
	for(int allele=0;allele<allelesPresentHap_[loc].len();++allele){
		freq = GetFreqHap(pop,loc,allele);
		homosum += freq*freq;}
	return ((long double)1.0 - homosum);
	}//HaploidMarkerData::GetExpectedHeterozygosity (int, int)



long double HaploidMarkerData::GetFreqHap (int pop, int loc, int marker){
	if (!alleleFreqsCurrentHap_){CalculateHaplotypeFreqs();}
	return alleleFreqHap_[pop][loc][marker];
	}//end HaploidMarkerData::GetFreqHap



long double HaploidMarkerData::GetFreqHap (int loc, int allele){
	long double numAlleleCopies = 0.0;
	long double numThisAllele = 0.0;
	for(int pop=0;pop<npop_;++pop){
		long double n = (long double)GetLocusSampleSizeHap(pop,loc);
		numAlleleCopies += n;
		numThisAllele += n * GetFreqHap(pop,loc,allele);
		}
	if(numAlleleCopies==0.0){return (long double)0.0;}
	return numThisAllele/numAlleleCopies;
	}//end HaploidMarkerData::GetFreq



long double HaploidMarkerData::GetFreqMostCommonAllele (int pop, int loc){
	long double mostCommon = 0.0;
	for(int allele=0;allele<GetNumAllelesHap(loc);++allele){
		long double freq = GetFreqHap(pop,loc,allele);
		if (freq>(long double)0.5) {return freq;}
		if (mostCommon<freq){mostCommon = freq;}
		}//and for allele
	return mostCommon;
	}//end HaploidMarkerData::GetFreqMostCommonAllele


void HaploidMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
	//uploads data to pass to genetic analysis classes
	pgcList.Clear();
	int pop,i, dt;
	//initialize
	aList<int> lociByType;
	for(dt=0;dt<MultiInheritanceModeContainer<int>::inheritanceModes_;++dt){
		lociByType.Append(0);}
	lociByType[MultiInheritanceModeContainer<int>::haploid]=nlociHap_;
	for(pop=0;pop<npop_;++pop){
		pgcList.Append(NEW PopulationGeneContainer<int>(popSize(pop),lociByType));}
	
	//fill up
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<popSize(pop);++i){
			GetGenome(pgcList[pop][i], pop, i);}
		}
	}//HaploidMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >&)



void HaploidMarkerData::GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID){
	//uploads this indiv's data to HaplotypeContainer<int>[locus]
	HaplotypeContainer<int>* hc = (HaplotypeContainer<int>*) &gc;
	hc->SetSize(HaploidMarkerData::loci());
	if(setID){
		hc->SetID(1+GetIndivNumInDataSequence(pop,indiv));}
	for(int loc=0;loc<nlociHap_;++loc){
		if(rawTraits_[pop][indiv][locusNumHap_[loc]].missing()){
				hc->Clear(); return;}
			else{
				hc->setHaplotype(loc,HaploidMarkerData::alleleID(pop,indiv,loc));}
		}	
	}//HaploidMarkerData::GetGenome(GeneContainerCore&, int, int)



long HaploidMarkerData::GetIndividualsSampledHap (int pop, int loc){
	// loc refers to haploid loci only
	long nn=0;
	for(long i=0;i<individuals(pop);++i){
		if(GetLocusSampleSizeHap (pop, i, loc)>0){
			++nn;}}
	return nn;
	}//end  HaploidMarkerData::GetIndividualsSampledHap


long HaploidMarkerData::GetLocusSampleSizeHap (int loc){
	if(! alleleFreqsCurrentHap_){
		CalculateHaplotypeFreqs();}//calculates sample size
	long numAlleleCopies = 0;
	for(int pop=0;pop<npop_;++pop){
		numAlleleCopies += nByLocusHap_[pop][loc];
		}
	return numAlleleCopies;
	}//end  HaploidMarkerData::GetLocusSampleSize


long HaploidMarkerData::GetLocusSampleSizeHap (int pop, int loc){
	// loc refers to haploid loci only
	return nByLocusHap_[pop][loc];
	}//end  HaploidMarkerData::GetLocusSampleSizeHap


long HaploidMarkerData::GetLocusSampleSizeHap (int pop, int indiv, int loc){
	// loc refers to haploid loci only
	TextDatum& score = GenotypicVal(pop,indiv,loc);
	return score.OK();//false=0 if missing, true=1 if present
	}//end  HaploidMarkerData::GetLocusSampleSizeHap


int HaploidMarkerData::GetNumAllelesHap (int loc){
	// loc refers to haploid loci only
	//total in data set
	if(! allelesCurrentHap_){CheckForNewAlleles();}
	return allelesPresentHap_[loc].len();
	}//end  HaploidMarkerData::GetNumAllelesHap(loc)


int HaploidMarkerData::GetNumAllelesHap (int pop, int loc){
	// loc refers to dominant loci only
	//extant alleles in population
	if(! alleleFreqsCurrentHap_){CalculateHaplotypeFreqs();}
	aList<long double>& locList = alleleFreqHap_[pop][loc];
	int count=0;
	for(int a=0;a<locList.len();++a){
		if (locList[a] >(long double) 0.0){++count;}}
	return count;
	}//end  HaploidMarkerData::GetNumAllelesHap(pop,loc)



aString* HaploidMarkerData::GetTraitString(int pop, int indiv, int loc){
	//loc indexed as in locusNumCodom_ 
	aString *aStr = NEW aString();
	int tr = locusNumHap_[loc];
	aStr->cat(rawTraits_[pop][indiv][tr][0]);
	return aStr;
	}//HaploidMarkerData::GetTraitString


aString* HaploidMarkerData::GetTraitString(int pop, int indiv, int loc, int /*allele*/){
	return GetTraitString(pop, indiv, loc);
	}//HaploidMarkerData::GetTraitString


	


int HaploidMarkerData::individuals(int pop, int loc){
	int indivs=0;
	for(int i=0;i<individuals(pop);++i){
		if (GetLocusSampleSizeHap ( pop, i, loc)>0){++indivs;}}
	return indivs;
	}//end  HaploidMarkerData::individuals(int,int)



long HaploidMarkerData::maxall(void){
	//maximum alleles per locus in data set
	CheckForNewAlleles();
	long maxalleles=0;
	int loc;
	for(loc=0;loc<nlociHap_;++loc){
		maxalleles = MAX(maxalleles,allelesPresentHap_[loc].len());}
	return maxalleles;
	}//HaploidMarkerData::maxall()




/*
void  EvolutionaryGeneticsData::PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of allelic frequencies for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculateHaplotypeFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied


	outFile<<title<<endl<<endl<<endl;
	outFile<<"ALLELE FREQUENCY DATA"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop<<endl;
	outFile<<"NUMBER OF LOCI SURVEYED = ";outFile.width(3);outFile<<nloci<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (lociSkipped == 0){
			outFile<<"none";}
		else{
			for (int loc=0;loc< lociSkipped;++loc){
				aString& locName = skippedLocusName[loc];
				outFile<<locName;
				if (loc < lociSkipped-1){outFile<<", ";}
				}//end for loc
		}//end if lociSkipped
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop / ncols;
	int npop0 = npop;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Allele frequencies for each population."<<endl;
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;

		for (int loc=0;loc<nloci;++loc) {//print the locus and locus sample size
			aString& locWord = locusName[loc];
			outFile<< locWord;
			if (asciiFormat){outFile.width(11 + (5 - locWord.len()));}else{outFile<<colSpacer;}
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = GetLocusSampleSize(pop, loc) / 2;
				outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
				if (pop < endPop){
					if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
					}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = GetFreq(pop, loc, allele);
					aString freqStr(freq,0,4);
					if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									if (asciiFormat){
											aString blanks(' ',12);
											if (pop==x) {
												blanks.PrependCharToLen(' ',17);}
											outFile<<blanks;
											}
										else{outFile<<colSpacer;}
									}
								else{
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end EvolutionaryGeneticsData::PrintAlleleFreqs




void  EvolutionaryGeneticsData::PrintAlleleFreqsByPopList (ostream& outFile, 
											ObjList<ObjList<aList<double> > >& popFreqList, 
											ObjList<aList<double> >& popTwoNList, 
											int ncols){
	//popFreqList must contain the entire set of alleles in the data set
	//Sends a table of allelic frequencies to the output file.
	//ncols indexes the number of population columns per page

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = popFreqList.len() / ncols;
	int npop0 = popFreqList.len();
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)popFreqList.len(), (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			outFile.width(12);outFile<<pop+1;}
		outFile<<endl;

		for (int loc=0;loc<nloci;++loc) {//print the locus and locus sample size
			aString& locWord = locusName[loc];
			outFile<< locWord;outFile.width(11 + (5 - locWord.len()));
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = popTwoNList[pop][loc] / 2;
				outFile<<"[";outFile.width(2);outFile<<samSize<<"]";
				if (pop < endPop){outFile.width(9);}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = popFreqList[pop][loc][allele];
					aString freqStr(freq,0,4);
					freqStr.PrependCharToLen(' ',12);
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									aString blanks(' ',12);
									if (pop==x) {
										blanks.PrependCharToLen(' ',17);}
									outFile<<blanks;}
								else{
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end EvolutionaryGeneticsData::PrintAlleleFreqsByPopList
*/


void	 HaploidMarkerData::PrintHapMarkerFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintHapMarkerFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end HaploidMarkerData::PrintHapMarkerFile(aString&)



void HaploidMarkerData::PrintHapMarkerFile (ostream& outFile){
	aString dataLine = title_;
	dataLine.cat(":  skipped loci omitted");
	outFile<<dataLine<<endl;
	dataLine = "NPOP=";
	dataLine.cat(npop_);
	dataLine.cat(", NLOCI=");
	dataLine.cat(nlociHap_);
	long maxall_=0;
	for(int loc=0;loc<allelesPresentHap_.len();++loc){
		maxall_=MAX(maxall_,allelesPresentHap_[loc].len());}
	dataLine.cat(", MAXALL=");
	dataLine.cat(maxall_);
	dataLine.cat(", NGROUPS=");
	dataLine.cat(ngroups_);
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int loc=0;loc<nlociHap_;++loc){
		dataLine.cat(locusNameHap(loc));
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < nlociHap_){
				dataLine.cat("\t");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	dataLine = "SKIP=0";
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int pop=0;pop<npop_;++pop){
		outFile<<popName_[pop]<<endl;
		//cobble together a phenotype list for haploid markers only
		for (int i=0;i<individuals(pop);++i){
			StringList idata;
			idata.Append(NEW TextDatum(rawTraits_[pop][i][0]));
			for(int dloc=0;dloc<nlociHap_;++dloc){
				TextDatum& score=rawTraits_[pop][i][locusNumHap_[dloc]];
				if(score.OK()){
						idata.Append(NEW aString(score));}
					else{//record as missing data
						idata.Append(NEW aString(rawTraits_[pop][i].missingDataStr_));}
				}//dLoc
			aString outStr;
			for(int j=0;j<idata.len();++j){
				outStr+idata[j];
				if(j<idata.len()-1){
					outStr+" ";}}
			outFile<<outStr<<endl;
			}//end for i
		outFile<<"NEXT"<<endl;
		}//end for pop
	outFile<<"END"<<endl;
	}//end HaploidMarkerData::PrintDomMarkerFile(ostream&)



void	 HaploidMarkerData::PrintNEXUSFile(aString& outFileName, StringList& textToAppend){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintNEXUSFile(dataFile,textToAppend);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end HaploidMarkerData::PrintNEXUSFile(aString&,StringList&)



void HaploidMarkerData::PrintNEXUSFile (ostream& outFile, StringList& textToAppend){
	outFile<<"#NEXUS"<<endl;
	aString dataLine = title_;
	dataLine.prepend("[ ! ");
	outFile<<dataLine<<endl<<"]"<<endl<<endl;
	dataLine = "begin alleles;";
	outFile<<dataLine<<endl;
	dataLine = "\tdimensions newpops nloci=";
	dataLine.cat(nlociHap_);
	dataLine.cat(" npops=");
	dataLine.cat(npop_);
	dataLine.cat(";");
	outFile<<dataLine<<endl;
//	dataLine = "\tdominant all:1;";
//	outFile<<dataLine<<endl;
	dataLine = "\tformat labels missing=? ;";
	outFile<<dataLine<<endl;
	dataLine = "\tmatrix";
	outFile<<dataLine<<endl;
	dataLine.clear();
	/* need code for locus names
	for (int loc=0;loc<nlociHap_;++loc){
		dataLine.cat(locusNameDom_(loc));
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < nlociHap_){
				dataLine.cat("\t");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	*/
	aString endCharList("\t, ");
	for (int pop=0;pop<npop_;++pop){
		aString pname = popName_[pop];
		pname.replaceAll(' ','_');
		outFile<<'\t'<<pname<<":"<<endl;
		for (int i=0;i<individuals(pop);++i){
			StringList idata;
			idata.Append(NEW TextDatum(rawTraits_[pop][i][0]));
			idata[0].PrependCharToLen('0',3);//tweak the format of the ID code
			idata[0].prepend('_');
			idata[0].cat("_\t");
			for(int dloc=0;dloc<nlociHap_;++dloc){
				TextDatum& score=rawTraits_[pop][i][locusNumHap_[dloc]];
				if(score.OK()){
						idata.Append(NEW aString(score));}
					else{//record as missing data
						idata.Append(NEW aString("?"));}
				}//dLoc
			aString outStr;
			for(int j=0;j<idata.len();++j){
				outStr+idata[j];
				if(j<idata.len()-1){
					outStr+"\t";}}
			outFile<<"\t\t"<<outStr<<endl;

			}//end for i
		if(pop==npop_-1){
				outFile<<";"<<endl;}
			else{
				outFile<<","<<endl;}
		}//end for pop
	outFile<<"end;"<<endl<<endl;
	
	for(int i=0;i<textToAppend.len();++i){
		outFile<<textToAppend[i]<<endl;}
	outFile<<endl;
	}//end HaploidMarkerData::PrintNEXUSFile(ostream&, StringList&)



void HaploidMarkerData::PrintLociHap (ostream& outFile, int lociPerLine){//file assumed open
	int lineSpot = 0;
	for (int loc=0;loc<nlociHap_;++loc){ 
		aString& locName = locusNameHap(loc);
		outFile<<locName;
		++lineSpot;
		if ((loc == nlociHap_-1) || (lineSpot == lociPerLine)) {
				outFile<<endl;}
			else{
				outFile<<"\t";}
		}//end for loc
	outFile<<endl;
	}//end HaploidMarkerData::PrintLociHap


void HaploidMarkerData::PrintHaplotypeFreqs(aString& outFileName, int ncols, char colSpacer){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintHaplotypeFreqs(dataFile, ncols, colSpacer);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end HaploidMarkerData::PrintHaplotypeFreqs(aString&, int, char)



void  HaploidMarkerData::PrintHaplotypeFreqs (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of phenotypic frequencies for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculateHaplotypeFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied
	int significantDigits = 4;

	outFile<<title_<<endl<<endl<<endl;
	outFile<<"HAPLOTYPE FREQUENCY DATA"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop_<<endl;
	outFile<<"NUMBER OF MARKERS SURVEYED = ";outFile.width(3);outFile<<nlociHap_<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (skippedLocusNumHap_.len() == 0){
			outFile<<colSpacer<<"none";}
		else{
			cout<<colSpacer;
			for (int loc=0;loc< skippedLocusNumHap_.len();++loc){
				aString& locName = traitName_[skippedLocusNumHap_[loc]];
				outFile<<locName;
				if(MOD(loc+1,20)==0){outFile<<endl;}
					else if (loc < skippedLocusNumHap_.len()-1){outFile<<colSpacer;}
					else{}
				}//end for loc
		}//end if lociSkippedDom
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop_ / ncols;
	int npop0 = npop_;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Allele frequencies of HAPLOID markers for each population."<<endl;
				outFile<<"Number of alleles sampled in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Haplotype frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop_, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;

		for (int loc=0;loc<nlociHap_;++loc) {//print the locus and locus sample size
			aString& locWord = locusNameHap(loc);
			outFile<< locWord;
			if (asciiFormat){outFile.width(11 + (5 - locWord.len()));}else{outFile<<colSpacer;}
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = nByLocusHap_[pop][loc];
				outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
				if (pop < endPop){
					if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
					}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& markerNameList = allelesPresentHap_[loc];
			for (int marker=0; marker<markerNameList.len();++marker){
				aString& markerName=markerNameList[marker];
				aString markerNameCopy(markerName);
				markerNameCopy.PrependCharToLen(' ',3);
				outFile<<markerNameCopy;
				// print the frequency of the marker for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = alleleFreqHap_[pop][loc][marker];
					aString freqStr(freq,0,significantDigits);
					if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
										}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									if (asciiFormat){
											aString blanks(' ',12);
											if (pop==x) {
												blanks.PrependCharToLen(' ',17);}
											outFile<<blanks;
											}
										else{outFile<<colSpacer;}
									}
								else{
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end EvolutionaryGeneticsData::PrintHaplotypeFreqs




int HaploidMarkerData::ReadHaploidDataFile (aString& dataFileName){
	Clear();
	fstream dataFile;
	aString endCharList(" \t,\r"), missingDataListElems("?. -_");
	char endchar = '\n';	
	StringList missingDataList;
	for(int s=0;s<missingDataListElems.len();++s){
		missingDataList.Append(NEW aString(missingDataListElems[s]));}
	char *dataLine=NEW char[10001];dataLine[0]='\0';
	char* filename=dataFileName.raw();
	dataFile.open(filename,ios::in);
	int dataFileOpen = dataFile.is_open();
	if(! dataFileOpen){
		aString errstr("couldn\'t open the file ");
		errstr.cat(filename);
		aError(errstr);
		return 0;}

	dataFile.getline(dataLine, 10000,endchar);
	title_ = dataLine;

	dataFile.getline(dataLine, 10000,endchar); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	npop_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 6);
	nlociHap_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 7);
	int maxall_ = paramWord->toInt();//now ignored
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	ngroups_ =  paramWord->toInt();
	delete paramWord; paramWord=0;


	//get locus information
	int moreLoci = 1;//true
	long loc = 0;
	StringList rawLocusNameHap,skippedLocusNameHap;//temporary
	while (moreLoci){
		dataFile.getline(dataLine, 10000,endchar); 
		aString locusLine(dataLine);
		while (locusLine.len() > 0){
			++loc;
			aString *locName = locusLine.ChopOffFirstWord(endCharList);
			if(locName->len()>0){//there could be trailing blanks on locusLine
				aString *s = NEW aString(locName);
				rawLocusNameHap.Append(s);}//append a copy
			delete locName; locName=0;
			}//end while locusLine.len()
		if (loc >= nlociHap_){//data may be on more than one line
			moreLoci = 0;}//false
		}//end while moreLoci
	//check for duplicates	
	StringList uniqueNames;
	for(long i=0;i<rawLocusNameHap.len();++i){
		if(!(uniqueNames.Includes(&(rawLocusNameHap[i])))){
			uniqueNames.Append(NEW aString(rawLocusNameHap[i]));}}

	long numuni=uniqueNames.len();
	if(rawLocusNameHap.len()>numuni){
		//figure out the names and positions of duplicated loci & report
		StringList dupNames(rawLocusNameHap);
		for(loc=0;loc<uniqueNames.len();++loc){
			long uItem=dupNames.Loc(&(uniqueNames[loc]));
			dupNames.Delete(uItem);
			}//loc
		ObjList<aList<long> > dupPositions;
		for(loc=0;loc<dupNames.len();++loc){
			StringList locNamesCopy(rawLocusNameHap);//work with a copy
			dupPositions.Append(NEW aList<long>());
			long dupcount=0;
			long dupitem = locNamesCopy.Loc(&(dupNames[loc]));
			while (dupitem>=0){
				locNamesCopy.Delete(dupitem);
				dupPositions[loc].Append(dupitem+dupcount);//because the locNamesCopy list shrinks from the bottom
				++dupcount;
				dupitem = locNamesCopy.Loc(&(dupNames[loc]));
				}//while
			}//loc
		cout<<"You have duplicated locus names in your data set."<<endl;
		cout<<"Please check that these are not mistakes, and give all your loci unique names."<<endl;
		cout<<"duplicated loci:"<<endl;
		dataFile<<"You have duplicated locus names in your data set."<<endl;
		dataFile<<"Please check that these are not mistakes, and give all your loci unique names."<<endl;
		dataFile<<"duplicated loci:"<<endl;
		for(loc=0;loc<dupNames.len();++loc){
			aString dupLine(dupNames[loc]);
			dupLine.cat(" at positions ");
			for(int i=0;i<dupPositions[loc].len();++i){
				dupLine.cat(dupPositions[loc][i]+1);
				if(i<dupPositions[loc].len()-1){
					dupLine.cat(", ");}
				}//i
			cout<<dupLine<<endl;
			dataFile<<dupLine<<endl;}
		aError("The duplicated names are in your output file. Please edit your data file and start again from the beginning.");
		return 2;
		}//unique names
	//read in the number of loci to be skipped in the analyses
	dataFile.getline(dataLine, 10000,endchar); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	int lociSkippedHap = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped loci, if any
	loc = 0;
	moreLoci = 0;//false
	if (lociSkippedHap > 0){
			moreLoci = 1;}//true
	while (moreLoci){
		while (skipLine.len() > 0){
			++loc;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				aString *s = NEW aString(skipWord);
				skippedLocusNameHap.Append(s);//append a copy
				}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (loc >= lociSkippedHap){
				moreLoci = 0;}//false
			else	{//more loci on the next line
				dataFile.getline(dataLine, 10000,endchar); 
				skipLine = dataLine;}
		}//end while moreLoci

	traitName_=rawLocusNameHap;
	for(int tr=0;tr<traitName_.len();++tr){
		allHapLoci_.Append(tr);}
	locusNumHap_=allHapLoci_;
		
	//read in population data
	aString popLine;
	aString beast;
	ObjList<StringList> rawHaplotypes;
	for (int pop=0;pop<npop_;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 10000,endchar); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		aString *s = NEW aString(popLine);
		popName_.Append(s);//append a copy

		//make a new sublist for the genotypes of this population
		rawHaplotypes.Append(NEW StringList());
		// BEAST is the string for a particular individual
		beast.clear();
		while (beast != "NEXT"){ 
			dataFile.getline(dataLine, 10000,endchar); 
			beast = dataLine;
			beast.CullTrailingChar('\t');
			beast.CullTrailingChar(' ');
			beast.CullTrailingChar(',');
			if (beast != "NEXT"){ 
					aString *s = NEW aString(beast);
					rawHaplotypes[pop].Append(s);
					s=0;}//appends a copy
			}//end while beast..else
		}//end for pop
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	//Now parse the data by trait
	for(int pop=0;pop<npop_;++pop){	
		rawTraits_.Append(NEW ObjList<TextDataList>());
		for(int i=0;i<rawHaplotypes[pop].len();++i){
			aString dataStrCopy(rawHaplotypes[pop][i]);
			if(dataStrCopy.len()==0) continue;
			rawTraits_[pop].Append(NEW TextDataList("?"));
			aString *codeWord = dataStrCopy.ChopOffFirstWord(endCharList);
			rawTraits_[pop][i].Append(NEW TextDatum(*codeWord));
			for(int loc=0;loc<nlociHap_;++loc){
				int posPtypeWord = 1 + ((alleleLen_+1) * loc);
				if(posPtypeWord>=dataStrCopy.len()){
						rawTraits_[pop][i].Append(NEW TextDatum("",true));
						continue;}
				aString gene = aString(dataStrCopy, posPtypeWord, posPtypeWord+alleleLen_-1);
				if(missingDataList.Includes(&gene)){
						rawTraits_[pop][i].Append(NEW TextDatum("",true));}
					else{
						rawTraits_[pop][i].Append(NEW TextDatum(gene));}
				}//loc
			delete codeWord;codeWord=0;
			}//i		
		}//pop		
	CullHapLoci(skippedLocusNameHap);
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();
	return 1;
	}//end HaploidMarkerData::ReadHaploidDataFile


/*

void HaploidMarkerData::ReadAndAppendHaploidDataFile (aString& dataFileName){//not yet converted or debugged
	fstream dataFile;	
	char endchar = '\n';	
	char *dataLine=NEW char[10001];dataLine[0]='\0';
	aString endCharList(" \t,\r");
	char* filename=dataFileName.raw();
	dataFile.open(filename,ios::in);

	dataFile.getline(dataLine, 10000,endchar);
	title_.prepend("Combined data: [");
	title_.cat("] & [");
	title_.cat(dataLine);
	title_.cat("]");

	dataFile.getline(dataLine, 10000,endchar); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	int npopThisFile = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 6);
	int nlociDomThisFile = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 7);
	long maxallThisFile = paramWord->toInt();
	delete paramWord; paramWord=0;
	maxall = MAX(maxall, maxallThisFile);

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	int ngroupsThisFile =  paramWord->toInt();
	delete paramWord; paramWord=0;


	//get locus information
	int moreLoci = 1;//true
	int loc = 0;
	while (moreLoci){
		dataFile.getline(dataLine, 10000,endchar); 
		aString locusLine(dataLine);
		while (locusLine.len() > 0){
			++loc;
			aString *locName = locusLine.ChopOffFirstWord(endCharList);
			if(locName->len()>0){//there could be trailing blanks on locusLine
				if(!(rawLocusNameDom.Includes(locName))){//appended data can repeat loci
					aString *s = NEW aString(locName);
					rawLocusNameDom.Append(s);}//append a copy
				}
			delete locName; locName=0;
			}//end while locusLine.len()
		if (loc >= nlociDomThisFile){//data may be on more than one line
			moreLoci = 0;}//false
		}//end while moreLoci
	nlociHap_ = rawLocusNameDom.len();
	//read in the number of loci to be skipped in the analyses
	dataFile.getline(dataLine, 10000,endchar); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	int lociSkippedDomThisFile = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped loci, if any
	loc = 0;
	moreLoci = 0;//false
	if (lociSkippedDom > 0){
			moreLoci = 1;}//true
	while (moreLoci){
		while (skipLine.len() > 0){
			++loc;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				if(!(skippedLocusNameDom.Includes(skipWord))){
					aString *s = NEW aString(skipWord);//skipped loci can be repeated
					skippedLocusNameDom.Append(s);//append a copy
					int toSkip = rawLocusNameDom.Loc(s);
					skippedLocusNumDom.Append(toSkip);}
				}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (loc >= lociSkippedDom){
				moreLoci = 0;}//false
			else	{//more loci on the next line
				dataFile.getline(dataLine, 10000,endchar); 
				skipLine = dataLine;}
		}//end while moreLoci
	lociSkippedDom = skippedLocusNameDom.len();

	locusNameDom.Copy(rawLocusNameDom);
	for (loc=0;loc<skippedLocusNumDom.len();++loc){
		aString& toSkip = skippedLocusNameDom[loc];
		int location = locusNameDom.Loc(&toSkip);
		locusNameDom.Delete(location);
		}//end for loc
	nlociHap_ -= lociSkippedDom;

	//read in population data
	aString popLine;
	aString beast;
	for (int pop=0;pop<npopThisFile;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 10000,endchar); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		int thepop = pop;
		if(!(popName.Includes(&popLine))){
				aString *s = NEW aString(popLine);
				popName.Append(s);//append a copy
				thepop = popName.len()-1;//the last pop in list
				//make a new sublist for the genotypes of this population
				rawPhenotypes.Append(NEW StringList());				
				}
			else{
				thepop = popName.Loc(&popLine);}


		// BEAST is the string for a particular individual
		beast.clear();
		while (beast != "NEXT"){ 
			dataFile.getline(dataLine, 10000,endchar); 
			beast = dataLine;
			beast.CullTrailingChar('\t');
			beast.CullTrailingChar(' ');
			beast.CullTrailingChar(',');
			if (beast != "NEXT"){ 
				//check to see if this individual already has data
				aString beastCopy(beast);
				aString* IDbeast = beastCopy.ChopOffFirstWord(endCharList);
				int hasData	= 0, indivNum=0;
				for(int i=0;i<rawPhenotypes[thepop].len();++i){
					aString indivData(rawPhenotypes[thepop][i]);//copies
					aString *IDindiv = indivData.ChopOffFirstWord(endCharList);
					if(*IDbeast == *IDindiv){//already in data set
						hasData = 1;//true
						indivNum = i;
						delete IDindiv;IDindiv=0;
						break;//escape for i loop
						}
					delete IDindiv;IDindiv=0;
					}//end for i
				if(hasData){//append to end of this individual -- buggy if loci repeated
						beastCopy.prepend('\t');
						rawPhenotypes[thepop][indivNum].cat(beastCopy);
						}
					else{
						aString *s = NEW aString(beast);
						rawPhenotypes[thepop].Append(s);
						s=0;}//appends a copy
				delete IDbeast; IDbeast=0;
				}//end if beast != NEXT
			}//end while beast..else


		}//end for pop
	npop_ = rawPhenotypes.len();
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	ResetFlags();
	GroupMembership();
	LocalityMembership();
	CullSkippedLociDom();
	CheckForNewAlleles();
	}//end HaploidMarkerData::ReadAndAppendAFLPDataFile
*/




void HaploidMarkerData::RestoreLociHap (aList<int>& lociToRestore){
	//This function restores the designated loci to the analysis.
	//It shifts trait numbers from from skippedTraitNums_ to the traitNums_ list
	//and keps track of the changes in the locusNumHap_ and skippedLocusNumHap_ lists
	if(lociToRestore.len()==0){return;}//none to cull
	ResetFlags();
	for(int tr=0;tr<lociToRestore.len();++tr){
		int rLoc = lociToRestore[tr];//make sure it's a valid locus to restore
		if(skippedTraitNum_.Includes(rLoc) && rLoc>=0 && rLoc<traitName_.len()){
			traitNum_.Append(rLoc);
			locusNumHap_.Append(rLoc);
			long item = skippedTraitNum_.Loc(rLoc);
			skippedTraitNum_.Delete(item);
			item = skippedLocusNumHap_.Loc(rLoc);
			skippedLocusNumHap_.Delete(item);
			}//if item not skipped already
		}//tr
	traitNum_.Sort();
	locusNumHap_.Sort();
	CheckForNewAlleles();
	}//end  HaploidMarkerData::RestoreLociHap(aList<int>&)


void HaploidMarkerData::RestoreLociHap (StringList& lociToRestore){
	//This function restores the designated loci to the analysis
	if(lociToRestore.len()==0){return;}//none to cull
	//convert names to numbers and cull
	aList<int> restoreListByNum;
	for(int tr=0;tr<lociToRestore.len();++tr){
		aString rTraitname = lociToRestore[tr];
		int item = traitName_.Loc(&rTraitname);
		if((item>=0)&&(!(traitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already included
			restoreListByNum.Append(item);
			}//if item exists
		}//tr
	RestoreLociHap(restoreListByNum);
	}//end HaploidMarkerData::RestoreLociHap(StringList&)


void HaploidMarkerData::RestoreLocusHap (aString& locusToRestore){
	StringList lociToRestore;
	lociToRestore.Append(new aString(locusToRestore));
	RestoreLociHap(lociToRestore);
	}//end HaploidMarkerData::RestoreLocusHap (aString& )



void HaploidMarkerData::SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
	//records all data, passed along from genetic analysis classes
	//empties old data and creates new data structures
	Clear();
	title_ = "Simulated data set";
	npop_ = pgcList.len();
	nlociHap_ = pgcList[0].loci(MultiInheritanceModeContainer<int>::haploid);
	ngroups_ = 1;
	SetAlleleLength(1);//bug if maxall>9
	traitName_.Append(NEW aString("ID"));
	int locusCount=0;
	int pop,i, loc;
	traitNum_.Append(locusCount++);
	for(loc = 0;loc<nlociHap_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',Digits(nlociHap_));
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(locusCount++);
		allHapLoci_.Append(loc+1);
		locusNumHap_.Append(loc+1);
		}
	ntraits_ = traitNum_.len();
	
	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("simulated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create dummy id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	

		for (i=0;i<pgcList[pop].popSize();++i){
			rawTraits_[pop].Append(NEW TextDataList());
			rawTraits_[pop][i].Append(NEW TextDatum());}//empty ID
		popSize_.Append(pgcList[pop].popSize());


		//create and append dummy genotypes
		for(loc=0;loc<nlociHap_;++loc){
			for(i=0;i<pgcList[pop].popSize();++i){
				rawTraits_[pop][i].Append(NEW TextDatum(" "));
				}//end for i
			}//end for loc
		}//end for pop	

	//Fill up with data
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<popSize(pop);++i){
			SetGenome(pgcList[pop][i], pop, i);}
		}
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();		
	}//HaploidMarkerData::SetGenome(ObjList<PopulationGeneContainer<int> >&)



void HaploidMarkerData::SetGenome(GeneContainerCore& gc, int pop, int indiv){
	//records data from HaplotypeContainer<int>
	//assumes blank data records already created in EvolutionaryGeneticsData
	HaplotypeContainer<int>* hc = (HaplotypeContainer<int>*) &gc;
	if(hc->loci() != nlociHap_){
		aError("ERROR: HaploidMarkerData::SetGenome(): size mismatch between data classes");
		return;}
	ResetFlags();
	rawTraits_[pop][indiv][0] = hc->ID();
	int loc;
	for(loc=0;loc<nlociHap_;++loc){
		if(hc->missing()){//for whatever reason
				rawTraits_[pop][indiv][locusNumHap_[loc]].clear();}
			else{
				rawTraits_[pop][indiv][locusNumHap_[loc]] = hc->haplotype(loc);}
		}	
	}//HaploidMarkerData::GetGenome(GeneContainerCore&, int, int)



void HaploidMarkerData::SetLoci(aList<int>& lociToInclude){
	if(lociToInclude.len()==0) return;
	HaploidMarkerData::ResetGeneticFlags();
	allHapLoci_=lociToInclude;
	locusNumHap_=lociToInclude;
	skippedLocusNumHap_.Clear();
	int tr,cLoc,item;
	for(tr=0;tr<lociToInclude.len();++tr){
		cLoc = lociToInclude[tr];
		if(skippedTraitNum_.Includes(cLoc)){
			item = locusNumHap_.Loc(cLoc);
			if(item>=0){
				locusNumHap_.Delete(item);}
			if(! skippedLocusNumHap_.Includes(cLoc)){
				skippedLocusNumHap_.Append(cLoc);}
			}
		}//tr
	skippedLocusNumHap_.Sort();
	nlociHap_ = locusNumHap_.len();
	HaploidMarkerData::CheckForNewAlleles();
	}//HaploidMarkerData::SetLoci(aList<int>& )



void HaploidMarkerData::UpdateVariableLociHap(void){
	if (varLociCurrentHap_){return;}
	CheckForNewAlleles();
	variableLocusNumHap_.Clear();
	int loc;
	for(loc=0;loc<nlociHap_;++loc){
		if (allelesPresentHap_[loc].len()>1){
			variableLocusNumHap_.Append(loc);}}
	varLociCurrentHap_=1;
	}//end HaploidMarkerData::UpdateVariableLociHap


	
#endif





