//unit PopulationGeneticData.cpp


#ifndef PopulationGeneticDatacpp
#define PopulationGeneticDatacpp = 1

#include "PopulationGeneticData.h"
//#include "Element.cpp"
//#include "aList.cpp"
//#include "StringList.cpp"
//#include "StatList.cpp"
//#include "aVector.cpp"
//#include "aMatrix.cpp"
//#include "PartitionedMatrix.cpp"



PopulationGeneticData::PopulationGeneticData(PopulationGeneticData& parentData):
			title_(parentData.title_), 
			npop_(parentData.npop_), 
			nloci(parentData.nloci), 
			maxall(parentData.maxall), 
			ploidy_(parentData.ploidy_), 
			alleleLen_(parentData.alleleLen_), 
			ngroups(parentData.ngroups),
			variableLoci(parentData.variableLoci),
			popSize(parentData.popSize),
			groupName(parentData.groupName),
			popName(parentData.popName),
			localityName(parentData.localityName),
			rawLocusName(parentData.rawLocusName),
			locusName(parentData.locusName),
			skippedLocusName(parentData.skippedLocusName),
			groupHierarchy(parentData.groupHierarchy),
			popsByLocality(parentData.popsByLocality),
			varLociCurrent_(parentData.varLociCurrent_), 
			skippedLocusNum(parentData.skippedLocusNum),
			variableLoci_(parentData.variableLoci_),
			lociSkipped(parentData.lociSkipped),
			rawGtypes(parentData.rawGtypes),
			gtypes(parentData.gtypes),
			gtypesCurrent_(parentData.gtypesCurrent_),
			gtypesPopIndLocAll(parentData.gtypesPopIndLocAll),
			allelesCurrent_(parentData.allelesCurrent_),
			allelesPresent(parentData.allelesPresent),
			alleleFreqCurrent_(parentData.alleleFreqCurrent_),
			alleleFreq(parentData.alleleFreq),
			twoN(parentData.twoN),
			HetObsCurrent_(parentData.HetObsCurrent_), 
			HetObsByAllele(parentData.HetObsByAllele),
			HetObsByLocus(parentData.HetObsByLocus),
			diseqD(parentData.diseqD),
			locusD(parentData.locusD)
			{	
		}//end ctor PopulationGeneticData(PopulationGeneticData&)


PopulationGeneticData::PopulationGeneticData(PopulationGeneticData& parentData, aList<int>& popSubsetNums):
			title_(parentData.title_), ngroups(0),npop_(popSubsetNums.len()), nloci(parentData.nloci), 
			maxall(0), ploidy_(parentData.ploidy_), alleleLen_(parentData.alleleLen_), variableLoci(0),
			rawLocusName(parentData.rawLocusName),
			skippedLocusNum(parentData.skippedLocusNum),
			skippedLocusName(parentData.skippedLocusName),
			locusName(parentData.locusName), lociSkipped(parentData.lociSkipped),
			diseqD(parentData.diseqD),
			varLociCurrent_(0), allelesCurrent_(0), 
			alleleFreqCurrent_(0), HetObsCurrent_(0) {
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	popSubsetNums.Sort();
	for(int p=0;p<popSubsetNums.len();++p){
		int pp = popSubsetNums[p];//population num in parentData
		aString* pn = NEW aString(parentData.popName[pp]);
		popName.Append(pn);
		popSize.Append(parentData.popSize[pp]);
		StringList* rawG = NEW StringList(parentData.rawGtypes[pp]);
		rawGtypes.Append(rawG);
		StringList* newgtype = NEW StringList(parentData.gtypes[pp]);
		gtypes.Append(newgtype);
		gtypesPopIndLocAll.Append(NEW ObjList<ObjList<StringList> >());
		for(int i=0;i<parentData.gtypesPopIndLocAll[pp].len();++i){
			gtypesPopIndLocAll[p].Append(NEW ObjList<StringList>(parentData.gtypesPopIndLocAll[pp][i]));
			for(int a=0;a<parentData.gtypesPopIndLocAll[pp][i].len();++a){
				gtypesPopIndLocAll[p][i].Append(NEW StringList(parentData.gtypesPopIndLocAll[pp][i][a]));}
			}//i
		}//end for p
	gtypesCurrent_=1;
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();
	}//end ctor PopulationGeneticData(PopulationGeneticData&, aList<int>&)


void  PopulationGeneticData::Initialize(void){
	PopulationGeneticData::Clear();
	}//Initialize()
	
	
void  PopulationGeneticData::Initialize(aString& s){
	PopulationGeneticData::Clear();
	title_=s; 
	}//Initialize(aString&)
	
	
void  PopulationGeneticData::Initialize(PopulationGeneticData& pgd){
	title_=pgd.title_; 
	npop_=pgd.npop_; 
	nloci=pgd.nloci; 
	maxall=pgd.maxall; 
	ploidy_=pgd.ploidy_; 
	alleleLen_=pgd.alleleLen_; 
	ngroups=pgd.ngroups;
	variableLoci=pgd.variableLoci;
	popSize=pgd.popSize;
	groupName=pgd.groupName;
	popName=pgd.popName;
	localityName=pgd.localityName;
	lociSkipped=pgd.lociSkipped;
	rawLocusName=pgd.rawLocusName;
	locusName=pgd.locusName;
	skippedLocusName=pgd.skippedLocusName;
	groupHierarchy=pgd.groupHierarchy;
	popsByLocality=pgd.popsByLocality;
	varLociCurrent_=pgd.varLociCurrent_; 
	skippedLocusNum=pgd.skippedLocusNum;
	variableLoci_=pgd.variableLoci_;
	lociSkipped=pgd.lociSkipped;
	rawGtypes=pgd.rawGtypes;
	gtypes=pgd.gtypes;
	gtypesCurrent_=pgd.gtypesCurrent_;
	gtypesPopIndLocAll=pgd.gtypesPopIndLocAll;
	allelesCurrent_=pgd.allelesCurrent_;
	allelesPresent=pgd.allelesPresent;
	alleleFreqCurrent_=pgd.alleleFreqCurrent_;
	alleleFreq=pgd.alleleFreq;
	twoN=pgd.twoN;
	HetObsCurrent_=pgd.HetObsCurrent_; 
	HetObsByAllele=pgd.HetObsByAllele;
	HetObsByLocus=pgd.HetObsByLocus;
	diseqD=pgd.diseqD;
	locusD=pgd.locusD;
	}//Initialize(PopulationGeneticData&)
	
	
PopulationGeneticData&  PopulationGeneticData::operator=(PopulationGeneticData& pgd){
	title_=pgd.title_; 
	npop_=pgd.npop_; 
	nloci=pgd.nloci; 
	maxall=pgd.maxall; 
	ploidy_=pgd.ploidy_; 
	alleleLen_=pgd.alleleLen_; 
	ngroups=pgd.ngroups;
	variableLoci=pgd.variableLoci;
	popSize=pgd.popSize;
	groupName=pgd.groupName;
	popName=pgd.popName;
	localityName=pgd.localityName;
	rawLocusName=pgd.rawLocusName;
	locusName=pgd.locusName;
	lociSkipped=pgd.lociSkipped;
	skippedLocusName=pgd.skippedLocusName;
	groupHierarchy=pgd.groupHierarchy;
	popsByLocality=pgd.popsByLocality;
	varLociCurrent_=pgd.varLociCurrent_; 
	skippedLocusNum=pgd.skippedLocusNum;
	variableLoci_=pgd.variableLoci_;
	lociSkipped=pgd.lociSkipped;
	rawGtypes=pgd.rawGtypes;
	gtypes=pgd.gtypes;
	gtypesCurrent_=pgd.gtypesCurrent_;
	gtypesPopIndLocAll=pgd.gtypesPopIndLocAll;
	allelesCurrent_=pgd.allelesCurrent_;
	allelesPresent=pgd.allelesPresent;
	alleleFreqCurrent_=pgd.alleleFreqCurrent_;
	alleleFreq=pgd.alleleFreq;
	twoN=pgd.twoN;
	HetObsCurrent_=pgd.HetObsCurrent_; 
	HetObsByAllele=pgd.HetObsByAllele;
	HetObsByLocus=pgd.HetObsByLocus;
	diseqD=pgd.diseqD;
	locusD=pgd.locusD;
	return *this;
	}//operator=(PopulationGeneticData&)



void PopulationGeneticData::Calculate2LocusDisequilibrium (void){

	int doingHillsMethod = 0;//false
	//first make up an empty data array for locusD and D_
	long double initValue = 0.0;
	aString blank; blank.AppendCharToLen(' ',alleleLen_);

	diseqD.Clear();
	locusD.Clear();//locusD contains locusByLocusDiseqInPop as subLists
	for (int pop=0;pop<npop_;++pop){
		//locusByLocusDiseqInPop contains locusDiseqLists as sublists
		ObjList<aList<long double> >* locusByLocusDiseqInPop = NEW ObjList<aList<long double> >();
		locusD.Append(locusByLocusDiseqInPop);
		for (int loc1=0;loc1<nloci - 1;++loc1){
			//locusDiseqList contains D values
			aList<long double>* locusDiseqList = NEW aList<long double>();
			locusByLocusDiseqInPop->Append(locusDiseqList);
			for (int loc2=loc1 + 1;loc2<nloci;++loc2){
				locusDiseqList->Append(initValue);}
			}//end for loc1
		diseqD.Append(initValue);
		}//end for pop
	StatList<long double> Dlist;
	for (int pop=0;pop<npop_;++pop){
		for (int loc1=0;loc1<nloci - 1;++loc1){
			if (IsPolymorphic(pop,loc1)){ 
				StringList& alleleNamesList1=allelesPresent[loc1];
				for (int loc2 = (loc1 + 1); loc2<nloci;++loc2){
					if (IsPolymorphic(pop,loc2)){ 
						StringList& alleleNamesList2=allelesPresent[loc2];
						int kComb = 0;  long double diseq=0.0;
						for (int allele1=0;allele1<alleleNamesList1.len();++allele1){
							//get freq of allele1 (at loc1) in pop
							long double freq1 = GetFreq(pop, loc1, allele1);
							if ((freq1>(long double)0.0)&&(freq1<(long double)1.0)){
								aString& alleleName1 = alleleNamesList1[allele1];
								for (int allele2=0;allele2<alleleNamesList2.len();++allele2){
									int nSample = 0;/*set sample size to zero, to count for blanks*/
									/*get freq of allele2 (at loc2) in pop*/
									long double freq2 = GetFreq(pop, loc2, allele2);
									if ((freq2>(long double)0.0)&&(freq2<(long double)1.0)){
									aString& alleleName2 = alleleNamesList2[allele2];
									/*get genotype strings for each individual and  read joint frequencies*/
									long double gameteValue = 0.0;
									StringList& gtypeList = gtypes[pop];
									int bigN[3][3], bigX[2][2];
									for (int index1=0;index1<3;++index1){
										for (int index2=0;index2<3;++index2){
											bigN[index1][index2] = 0;}}
									for (int index1=0;index1<2;++index1){
										for (int index2=0;index2<2;++index2){
											bigX[index1][index2] = 0;}}
									for (int i=0;i<gtypeList.len();++i){//each individual
										StringList spot1 = Genotype(pop,i,loc1);
										/*' make sure it doesn't have any blanks*/
										int blankAt1 = 0;//false
										if (spot1.len() < 2){
												blankAt1 = 1;}//true
												//happens sometimes at end of data line with missing loci
											else{
												if ((spot1[0] == blank) || (spot1[1] == blank)) {blankAt1 = 1;}//true
											}//end if spot1.len...else
										if (!(blankAt1)){
											StringList spot2 = Genotype(pop,i,loc2);
											//make sure it doesn' t have any blanks
											int blankAt2 = 0;//false
											if (spot2.len() < 2) {
													blankAt2 = 1;}//true
												else{
													if ((spot2[0] == blank) || (spot2[1] == blank)) {blankAt2 = 1;}//true
												}//end if spot2.len...else...}
											if (!(blankAt2)){
												nSample++;
												//use this indiv in calculation
												//count instances of target alleles at each locus in this indiv
												long double count1 = 0;
												int index1 = 3;
												for(int chrom=0;chrom<2;++chrom){
													if (spot1[chrom] == alleleName1){
														count1 += 0.5;
														index1--;
													}}
												long double count2 = 0;
												int index2 = 3;
												for(int chrom=0;chrom<2;++chrom){
													if (spot2[chrom] == alleleName2){
														count2 += 0.5;
														index2--;
													}}
												//get a gametic value, which is the proportion
												//of gametes this individual will produce that
												//share target alleles at both loci.
												//Keep a sum of gametic values for the population
												gameteValue += count1 * count2;
												//this counts genotype freqs
												bigN[index1][index2]++;
												}//end if ! blankAt2
		//									delete spot2; spot2=0;
											}//end if ! blankAt1
		//								delete spot1; spot1=0;
										}//end for i
									//calculate frequency of joint genotype=gametic value/nsample}
									if (nSample == 0){
											gameteValue = (long double)-100;}
										else{
											gameteValue /= (long double)nSample;}
									//calculate Hill's (1974) intermediate variables & iterative method}
									long double HillsDiseq=0.0;
									if (doingHillsMethod){
										bigX[1][1] = 2 * bigN[1][1] + bigN[1][2] + bigN[2][1];
										bigX[2][1] = 2 * bigN[3][1] + bigN[2][1] + bigN[3][2];
										bigX[1][2] = 2 * bigN[1][3] + bigN[1][2] + bigN[2][3];
										bigX[2][2] = 2 * bigN[3][3] + bigN[2][3] + bigN[3][2];
										long double initialF11hat = (long double)((2 * bigX[1][1] + bigN[2][2]) / (2 * nSample)) - freq1 * freq2;
										long double diff = 10.0; long double f11hat=0.0;
										long iteration = 0;
										while (diff > 0.00001){
											//calculate in steps since it's long
											iteration++;
											f11hat = (long double)bigN[2][2] * initialF11hat * ((long double)1.0 - freq1 - freq2 + initialF11hat);
											f11hat /= (initialF11hat * ((long double)1.0 - freq1 - freq2 + initialF11hat) + (freq1 - initialF11hat) * (freq2 - initialF11hat));
											f11hat = ((long double)bigX[1][1] + f11hat) / (long double)(2 * nSample);
											diff = ABS(f11hat - initialF11hat);
											initialF11hat = f11hat;
											if (iteration > 40){
												initialF11hat *= 1.01;
												iteration = 0;}
											}//end while
										HillsDiseq = (long double)nSample * (f11hat - freq1 * freq2) / (long double)(nSample - 1);
										}//end if doingHillsMethod
									//calculate D
									diseq = gameteValue - freq1 * freq2;
									//was gameteValue -2* freq1 * freq2
									kComb++;
									ObjList<aList<long double> >& locusByLocusDiseqInPop = locusD[pop];
									aList<long double>& locusDiseqList = locusByLocusDiseqInPop[loc1];
									if (doingHillsMethod){
											diseq = HillsDiseq + locusDiseqList[loc2 - loc1];}
										else{
											diseq += locusDiseqList[loc2 - loc1];}
									locusDiseqList.Replace(diseq, loc2 - loc1);
									}//end if freq2>0 &&freq2<1
								}//end for allele2
							}//end if freq1>0 && freq1<1
						}//end for allele1
					ObjList<aList<long double> >& locusByLocusDiseqInPop = locusD[pop];
					aList<long double>& locusDiseqList = locusByLocusDiseqInPop[loc1];
					diseq += locusDiseqList.Nth(loc2 - loc1) / (long double)kComb;
					locusDiseqList.Replace(diseq, loc2 - loc1);
					Dlist.Append(diseq);
					}//end if IsPolymorphic(poploc2)
				}//end for loc2
			}//end if IsPolymorphic(pop,loc1)
		}//end for loc1
	if (diseqD.len() > 0){diseqD.Append(Dlist.mean());} else{diseqD.Append(0.0);}
	Dlist.Clear();
	}//end for pop
}//end PopulationGeneticData::Calculate2LocusDisequilibrium






void PopulationGeneticData::CalculateAlleleFreqs (void){
	//This routine counts allelic frequencies, taking into account
	//alleles left blank in the data set.
	if (alleleFreqCurrent_){return;}
	CheckForNewAlleles();
	long i, chrom, loc, allele;
//	aString dataStrCopy, codeWord, gtypeWord, gene;
//	char blank = ' ';	//don't count these
	aString blank; blank.AppendCharToLen(' ',alleleLen_);	//don't count blanks
	alleleFreq.Clear();//clear allele frequencies to start over
	twoN.Clear();//clear locus sample sizes to start over
	for (long pop=0;pop< npop_;++pop){ 
		//set up a frequency list for this population
		ObjList<aList<long double> > *popAlleleFreqs = NEW ObjList<aList<long double> >();
		alleleFreq.Append(popAlleleFreqs);
		aList<int> *locList = NEW aList<int>();
		twoN.Append(locList);
		for (loc=0;loc<nloci;++loc){
			//first set up a blank array of frequencies for each possible allele
			aList<long double> *locAlleleFreqs = NEW aList<long double>();
			popAlleleFreqs->Append(locAlleleFreqs);
			StringList& alleleNamesList = allelesPresent[loc];
			long double freq = 0.0;
			for (allele=0;allele<alleleNamesList.len();++allele){
				locAlleleFreqs->Append(freq);}

			//next look at the genotype of each individual and tally the frequency
			long allelesSampled = 0;	//the number of alleles, not individuals
			StringList& genotypesList = gtypes[pop];
			for (i=0;i< genotypesList.len();++i){
				aString dataStr = genotypesList[i];
				aString dataStrCopy(dataStr);
				//first snip off the individual' s designation code}
				aString *codeWord = dataStrCopy.ChopOffFirstWord();
				int codeLen = codeWord->len();
				dataStrCopy = dataStr;
				int posGtypeWord = dataStrCopy.pos(*codeWord) + codeLen + 1 + (loc * GenotypeSize() + 1);
				aString gtypeWord = aString(dataStrCopy, posGtypeWord, posGtypeWord+GenotypeSize()-1);
				for (chrom=0; chrom<ploidy_;++chrom){
					aString gene = aString(gtypeWord, chrom*alleleLen_+1, (chrom+1)*alleleLen_);
					if (! (gene == blank)) {//increment the frequency & sample size -- divide later}
						allele = alleleNamesList.Loc(&gene);
						freq = locAlleleFreqs->Nth(allele) + (long double)1.0;
						locAlleleFreqs->Replace(freq, allele);
						allelesSampled++;
						}//end if not blank
					}//end for chrom
				delete codeWord; codeWord=0;
				}//end for i


			//now go through and divide by the sample size to restore the frequencies
			if(allelesSampled>0){
				for (allele=0;allele<alleleNamesList.len();++allele){
					freq = locAlleleFreqs->Nth(allele) / (long double)allelesSampled;
					locAlleleFreqs->Replace(freq, allele) ;
					}
				}
			//store the sample size for this locus
			locList->Append(allelesSampled);
			}//end for loc}
		}//end for pop
	alleleFreqCurrent_=1;
	}//end PopulationGeneticData::CalculateAlleleFreqs



ObjList<aList<long double> >* PopulationGeneticData::CalculateAlleleFreqsByPopSet(aList<int>& popSet, aList<double>& twoNList){
 	//returns lost of loci, each with list of allele freqs, using total allele set
 	CalculateAlleleFreqs();
 	twoNList.Clear();
 	ObjList<aList<long double> >* popSetFreqsPtr = NEW ObjList<aList<long double> >(alleleFreq[0]);//copies structure
 	ObjList<aList<long double> >& popSetFreqs = *popSetFreqsPtr;//notational convenience
 	for(int loc=0;loc<nloci;++loc){
 		int alleles = GetNumAlleles(loc);
 		long double samSizeSet = 0;
 		for(int a=0;a<alleles;++a){
 			long double nCopies = 0.0;
 			for(int popSetPop=0;popSetPop<popSet.len();++popSetPop){
 				int pop = popSet[popSetPop];
 				long double nSample = (long double)twoN[pop][loc];
 				if(a==0){samSizeSet += nSample;}
 				nCopies += nSample * GetFreq(pop,loc,a);
 				}
 			if(a==0){twoNList.Append(samSizeSet);}
 			if(nCopies==0.0){
 					popSetFreqs[loc][a]=0.0;}
 				else if(nCopies==samSizeSet){
 					popSetFreqs[loc][a]=1.0;}
 				else{
 					popSetFreqs[loc][a] = nCopies / samSizeSet;
 					}
 			}//end for a
 		}//end for loc
 	return popSetFreqsPtr;
	}//end PopulationGeneticData::CalculateAlleleFreqsByPopSet



void PopulationGeneticData::CalculateObservedHeterozygosity (void){
	//Counts observed heterozygosity values for each population, locus & allele
	//directly from the genotype data strings for each individual.  
//	long i, codeLen, chrom, loc, allele, posGtypeWord, nSample;
	long i, loc, allele;
	aString dataStrCopy, codeWord, gtypeWord;
	StringList gene;
	long double hetsByAllele, hetsByLoc;
	char blank = ' ';	//don't count these
	//clear observed hets to start over
	HetObsByAllele.Clear();
	HetObsByLocus.Clear();
	for (int pop=0;pop<npop_;++pop){
		//set up a frequency list for this population
		ObjList<aList<long double> > *popHetObsAll = NEW ObjList<aList<long double> >();
		HetObsByAllele.Append(popHetObsAll);
		aList<long double> *popHetObsLoc = NEW aList<long double>();
		HetObsByLocus.Append(popHetObsLoc);
		for (loc=0;loc<nloci;++loc){
			//first set up a blank array of frequencies for each possible allele
			aList<long double> *locHetObs = NEW aList<long double>();
			popHetObsAll->Append(locHetObs);
			StringList& alleleNamesList = allelesPresent[loc];
			hetsByAllele = hetsByLoc = 0.0;
			popHetObsLoc->Append(hetsByLoc);
			for (allele=0;allele<alleleNamesList.len();++allele){
				locHetObs->Append(hetsByAllele);}
			//next look at the genotype of each individual and tally the frequency
			StringList& genotypesList = gtypes[pop];
			for (i=0;i<genotypesList.len();++i){
				aString dataStr = genotypesList[i];
				aString dataStrCopy(dataStr);
				//first snip off the individual' s designation code
				aString *codeWord = dataStrCopy.ChopOffFirstWord();
				int codeLen = codeWord->len();
				dataStrCopy = dataStr;
				int posGtypeWord =  dataStrCopy.pos(*codeWord) + codeLen + 1 + (3 * loc);
				aString gtypeWord = aString(dataStrCopy, posGtypeWord, posGtypeWord+1);
				aString gene1=aString(gtypeWord, 1, 1);		
				aString gene2=aString(gtypeWord, 2, 2);		
				if (!(gene1 == gene2) && (!(gene1 == blank) && !(gene2 == blank))) {// it's a heterozygote
					//increment the het frequencies of both alleles -- divide later
					++hetsByLoc;
					allele = alleleNamesList.Loc(&gene1);
					hetsByAllele = locHetObs->Nth(allele) + 1.0;
					locHetObs->Replace(hetsByAllele, allele);
					allele = alleleNamesList.Loc(&gene2);
					hetsByAllele = locHetObs->Nth(allele) + 1.0;
					locHetObs->Replace(hetsByAllele, allele);
					}//end if match & no blanks
				delete codeWord; codeWord=0;
				}//end for i
			//now go through and divide by the sample size to restore the frequencies
			long nSample = long(GetLocusSampleSize(pop, loc));
			if (nSample > 0){
				popHetObsLoc->Replace(hetsByLoc /((long double)nSample / (long double)2.0), loc);
				for (allele=0;allele<alleleNamesList.len();++allele){
					hetsByAllele = locHetObs->Nth(allele) / ((long double)nSample / (long double)2.0);
					locHetObs->Replace(hetsByAllele, allele);
					}//end for allele
				}//end if nSample
			}//end for loc
		}//end for pop
	HetObsCurrent_=1;
	}//end PopulationGeneticData::CalculateObservedHeterozygosity



void PopulationGeneticData::CalculateTwoN (void){
	//makes table of sample sizes for each pop, locus
	twoN.Clear();
	for(int pop=0;pop<npop_;++pop){
		twoN.Append(NEW aList<int>());
		for(int loc=0;loc<nloci;++loc){
			twoN[pop].Append(0);
			for(int i=0;i<popSize[pop];++i){
				twoN[pop][loc] += GetLocusSampleSize(pop, i, loc);
				}//end for i
			}//end for loc
		}//end for pop
	}//end PopulationGeneticData::CalculateTwoN



void PopulationGeneticData::CheckForNewAlleles (void){
	if (allelesCurrent_){return;}
	allelesPresent.Clear();
	for (int loc=0;loc<nloci;++loc){
		StringList *alleleList = NEW StringList();
		allelesPresent.Append(alleleList);}
	for (int pop=0;pop<npop_;++pop){
		StringList& gtypeList = gtypes[pop];
		for(int i=0;i<gtypeList.len();++i){
			aString& dataStr = gtypeList[i];
			char blank = ' ';//don't count these
			aString dataStrCopy(dataStr);
			//first snip off the individual' s designation code
			aString *codeWord = dataStrCopy.ChopOffFirstWord();
			int codeLen = codeWord->len();
			dataStrCopy = dataStr;
			for (int loc=0;loc<nloci;++loc){
				int posGtypeWord = dataStrCopy.pos(*codeWord) + codeLen + 1 + (3 * loc);
				aString gtypeWord(dataStrCopy, posGtypeWord, posGtypeWord+1);
				StringList& alleleList = allelesPresent[loc];
				for (int chrom=1;chrom<=2;++chrom){
					aString gene(gtypeWord, chrom, chrom);
					if (!(gene == blank)&&!(alleleList.Includes(&gene))) {
						aString *newGene = NEW aString(gene);
						alleleList.Append(newGene);}
					}//end for chrom
				}//end for loc
			delete codeWord; codeWord=0;
			}//end for i
		}//end for pop
	for (int loc=0;loc<nloci;++loc){
		StringList& alleleList = allelesPresent[loc];
		alleleList.Sort();}
	allelesCurrent_=1;
	UpdateVariableLoci();
	}//end PopulationGeneticData::CheckForNewAlleles


void PopulationGeneticData::CheckForNewHaplotypes (void){
	//use for haploid data sets only
	//not yet handling mixed haploid/diploid data sets 
	if (allelesCurrent_){return;}
	allelesPresent.Clear();
//	char blank = ' ';//don't count these
	aString endCharList(" \t,"), blank(' ');
	for (int loc=0;loc<nloci;++loc){
		allelesPresent.Append(NEW StringList());}
	for (int pop=0;pop<npop_;++pop){
		StringList& gtypeList = gtypes[pop];
		for(int i=0;i<gtypeList.len();++i){
			aString& dataStr = gtypeList[i];
			aString dataStrCopy(dataStr);
			//first snip off the individual' s designation code
			aString *codeWord = dataStrCopy.ChopOffFirstWord(endCharList);
			for (int loc=0;loc<nloci;++loc){
				aString *gtypeWord = dataStrCopy.ChopOffFirstWord(endCharList);
				StringList& alleleList = allelesPresent[loc];
				if (!(*gtypeWord == blank)&&!(alleleList.Includes(gtypeWord))) {
						alleleList.Append(gtypeWord);}
					else{
						delete gtypeWord;}
				gtypeWord=0;
				}//end for loc
			delete codeWord; codeWord=0;
			}//end for i
		}//end for pop
	for (int loc=0;loc<nloci;++loc){
		StringList& alleleList = allelesPresent[loc];
		alleleList.Sort();}
	allelesCurrent_=1;
	UpdateVariableHaploidLoci();
	}//end PopulationGeneticData::CheckForNewHaplotypes



void PopulationGeneticData::Clear(void){
	popSize.Clear();
	groupName.Clear();
	localityName.Clear();
	popName.Clear();
	locusName.Clear();
	rawLocusName.Clear();
	skippedLocusName.Clear();
	groupHierarchy.Clear();
	popsByLocality.Clear();
	skippedLocusNum.Clear();
	variableLoci_.Clear();
	rawGtypes.Clear();
	gtypes.Clear();
	gtypesPopIndLocAll.Clear();
	allelesPresent.Clear();
	alleleFreq.Clear();
	twoN.Clear();
	HetObsByAllele.Clear();
	HetObsByLocus.Clear();
	diseqD.Clear();
	locusD.Clear();
	ResetFlags();
	title_.clear();	//title_ of the data set from the first data line
	ngroups = npop_ = nloci = maxall = variableLoci = lociSkipped = 0;
	alleleLen_=1;
	ploidy_=2;
	}//end PopulationGeneticData::Clear()




void  PopulationGeneticData::CreateBootstrapData(PopulationGeneticData& origData, Ran3& ran){
	//individuals resampled
	Clear();
	title_ = origData.title_;	//title_ of the data set from the first data line
	title_.cat(" bootstrap replicate");
	ngroups = origData.ngroups;
	npop_ = origData.npop_;
	nloci = origData.nloci;
	maxall = origData.maxall;
	popSize = origData.popSize;
	lociSkipped = origData.lociSkipped;
	groupName = origData.groupName;
	localityName = origData.localityName;
	popName = origData.popName;
	rawLocusName = origData.rawLocusName;
	locusName = origData.locusName;
	skippedLocusName = origData.skippedLocusName;
	skippedLocusNum = origData.skippedLocusNum;
	groupHierarchy = origData.groupHierarchy;
	popsByLocality = origData.popsByLocality;
	

	
	for(int pop=0;pop<npop_;++pop){//randomly resamples within populations
		rawGtypes.Append(NEW StringList());
		gtypes.Append(NEW StringList());
		int newpop = ran.longBetween(0,npop_-1);
		int popsize = origData.rawGtypes[newpop].len();
		for (int i=0;i<popsize;++i){
			int ranIndiv = ran.longBetween(0,popsize-1);
			aString* ranGenotype = NEW aString(origData.rawGtypes[newpop][ranIndiv]);
			rawGtypes[pop].Append(ranGenotype);
			ranGenotype = NULL;//forget
			ranGenotype = NEW aString(origData.gtypes[newpop][ranIndiv]);
			gtypes[pop].Append(ranGenotype);
		}}
/*	//sample within populations only
	for(int pop=0;pop<npop_;++pop){//randomly resamples within populations
		rawGtypes.Append(NEW StringList());
		gtypes.Append(NEW StringList());
		int popsize = origData.rawGtypes[pop].len();
		for (int i=0;i<popsize;++i){
			int ranIndiv = ran.longBetween(0,popsize-1);
			aString* ranGenotype = NEW aString(origData.rawGtypes[pop][ranIndiv]);
			rawGtypes[pop].Append(ranGenotype);
			ranGenotype = NULL;//forget
			ranGenotype = NEW aString(origData.gtypes[pop][ranIndiv]);
			gtypes[pop].Append(ranGenotype);
		}}
*/
//	GroupMembership();
//	LocalityMembership();
//	CullSkippedLoci();
	CheckForNewAlleles();	
	}//end  PopulationGeneticData::CreateBootstrapData(PopulationGeneticData&)


void PopulationGeneticData::CreateGtypeList(void){
	gtypesPopIndLocAll.Clear();
	aString endcharlist(" \t\r\n,");
	for(int pop=0;pop<npop_;++pop){
		gtypesPopIndLocAll.Append(NEW ObjList<ObjList<StringList> >() );
		for(int i=0;i<gtypes[pop].len();++i){
			gtypesPopIndLocAll[pop].Append(NEW ObjList<StringList>());
			aString indiv(gtypes[pop][i]);
			aString* code = indiv.ChopOffFirstWord(endcharlist);
			delete code;code=0;
			for(int loc=0;loc<nloci;++loc){
				gtypesPopIndLocAll[pop][i].Append(NEW StringList());
//				int posGtypeWord = 1 + (2*alleleLen_+1) * loc;
				int posGtypeWord = 1 + (ploidy_*alleleLen_+1) * loc;
//				aString *gTypeString = NEW aString(indiv,posGtypeWord,posGtypeWord + ploidy_*alleleLen_ - 1);
				aString gTypeString(indiv,posGtypeWord,posGtypeWord + ploidy_*alleleLen_ - 1);
				for(int a=0;a<ploidy_;++a){
//					aString *alleleString = NEW aString(*gTypeString,a*ploidy_,a*ploidy_+alleleLen_-1);
//					aString *alleleString = NEW aString(gTypeString,a*ploidy_,a*ploidy_+alleleLen_-1);
//					aString alleleString(gTypeString,a*ploidy_,a*ploidy_+alleleLen_-1);
					aString alleleString(gTypeString,a*alleleLen_+1,(a+1)*alleleLen_);
					gtypesPopIndLocAll[pop][i][loc].Append(NEW aString(alleleString));}
//				delete gTypeString;gTypeString=0;
				}//loc
			}//i
		}//pop
	gtypesCurrent_=1;
	}//end PopulationGeneticData::CreateGtypeList()


void PopulationGeneticData::CreateRandomData(
							const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Fst=";
	title_.cat(Fst); title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numPopsSampled;
	nloci = numLociSampled;
	maxall = 2;
	ngroups = 1;
	for(int loc = 0;loc<nloci;++loc){
		aString locName("LOC");
		locName.cat(loc+1);
		aString *s = NEW aString(locName);
		rawLocusName.Append(s);}//append a copy
	lociSkipped=0;
	locusName.Copy(rawLocusName);


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
//	GeneFrequencyDistribution parametricFreqDistrib(Fst, pBar, 1000);//old version sends dp
	GeneFrequencyDistribution parametricFreqDistrib(Fst, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();

	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	ObjList<StatList<long double> > locusFrequenciesByPop;
	for(int pop=0;pop<npop_;++pop){
		StatList<long double>* locusFrequencies = NEW StatList<long double>();
		locusFrequenciesByPop.Append(locusFrequencies);
		for(int loc=0;loc<nloci;++loc){
			long double p = parametricFreqDistrib.RandomFrequency(ran);
			locusFrequencies->Append(p);
			}//end for loc
		}//end for pop

	for (int pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("randomly generated population # ");
		pName.cat(pop+1);pName.cat(" group #1");
		aString *s = NEW aString(pName);
		popName.Append(s);//append a copy

		StringList *rawGList = NEW StringList();
		rawGtypes.Append(rawGList);				
		for (int i=0;i<numIndivsPerPop;++i){
			aString indiv(i+1);//first create id numbers for each individual
			indiv.PrependCharToLen(' ', 4);
			aString *s = NEW aString(indiv);
			rawGList->Append(s);
			}//end for i
		popSize.Append(numIndivsPerPop);
		twoN.Append(NEW aList<int>());

		for(int loc=0;loc<nloci;++loc){
			StatList<long double>& locusFrequencies = locusFrequenciesByPop[pop];
			long double p = locusFrequencies[loc];
			GenotypeFreqDistribution popGenotypes(p, Fis);
			for(int i=0;i<numIndivsPerPop;++i){
				aString& indiv = (*rawGList)[i];
				indiv.cat(' ');
				aString genotype = '\0';
				int gtype = popGenotypes.RandomGenotype(ran);
				if (gtype==0){genotype = "AA";}
					else if(gtype==1){genotype = "AB";}
					else if(gtype==2){genotype = "BB";}			
				indiv.cat(genotype);
				}//end for i
			twoN[pop].Append(2*numIndivsPerPop);
			}//end for loc
		rawGList=0;//forget this information
		}//end for pop	
	GroupMembership();
	LocalityMembership();
//	CullSkippedLoci();
	gtypes = rawGtypes;
	CheckForNewAlleles();	
	}//end PopulationGeneticData::CreateRandomData



void PopulationGeneticData::CreateRandomData(
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Fst=";
	title_.cat(Fst); title_.cat(", Fis="); title_.cat(Fis); 
	npop_ = numIndivsPerPopLoc.len();
	nloci = pBar.len();
	maxall = 0;
	for(int loc=0;loc<nloci;++loc){
		maxall = MAX(maxall,pBar[loc].len());}
	ngroups = 1;
	for(int loc = 0;loc<nloci;++loc){
		aString locName("LOC");
		locName.cat(loc+1);
		aString *s = NEW aString(locName);
		rawLocusName.Append(s);}//append a copy
	lociSkipped=0;
	locusName.Copy(rawLocusName);


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();

	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	ObjList<ObjList<StatList<long double> > > frequenciesByPopLocusAllele;
	for(int pop=0;pop<npop_;++pop){
		frequenciesByPopLocusAllele.Append(NEW ObjList<StatList<long double> >());}
	for(int loc=0;loc<nloci;++loc){
		if(pBar[loc].len()>1){//locus is variable
//				GeneFrequencyDistMultiAllele parametricFreqDistrib(Fst, pBar[loc], 1000);//old version sends dp
				GeneFrequencyDistMultiAllele parametricFreqDistrib(Fst, pBar[loc], 2);//new version sends ploidy
				for(int pop=0;pop<npop_;++pop){
					aList<long double>* p = parametricFreqDistrib.RandomFrequencies(ran);
					frequenciesByPopLocusAllele[pop].Append(NEW StatList<long double>(*p));//copy as StatList
					delete p;p=NULL;
					}//end for pop
				}
			else{//locus is monomorphic
				for(int pop=0;pop<npop_;++pop){
					frequenciesByPopLocusAllele[pop].Append(NEW StatList<long double>(pBar[loc]));//copy as StatList
					}//end for pop
				}				
		}//end for loc

	for (int pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("randomly generated population # ");
		pName.cat(pop+1);pName.cat(" group #1");
		aString *s = NEW aString(pName);
		popName.Append(s);//append a copy

		StringList *rawGList = NEW StringList();
		rawGtypes.Append(rawGList);
		int maxIndivPop = 0;
		for (int loc=0;loc<numIndivsPerPopLoc[pop].len();++loc){
			maxIndivPop = MAX(maxIndivPop,numIndivsPerPopLoc[pop][loc]);}
						
		for (int i=0;i<maxIndivPop;++i){
			aString indiv(i+1);//first create id numbers for each individual
			indiv.PrependCharToLen(' ', 4);
			aString *s = NEW aString(indiv);
			rawGList->Append(s);
			}//end for i
		popSize.Append(maxIndivPop);
		twoN.Append(NEW aList<int>());

		for(int loc=0;loc<nloci;++loc){
			StatList<long double>& p = frequenciesByPopLocusAllele[pop][loc];
//			for(int a=0;a<p.len();++a){
//				cout<<"pop:  "<<pop+1<<"   "<<"loc:  "<<loc+1<<"   "<<"allele:  "<<a+1<<"   "<<p[a]<<endl;}
			if(p.len()>1){//sampling required
					GenotypeFreqMultiAlleleDist popGenotypes(p, Fis);
//					popGenotypes.PrintToScreen();
					for(int i=0;i<numIndivsPerPopLoc[pop][loc];++i){
						aString& indiv = (*rawGList)[i];
						indiv.cat(' ');
						aString* genotype = popGenotypes.RandomGenotypeString(ran);
						indiv.cat(genotype);
						delete genotype; genotype=NULL;
						}//end for i
					for(int i=numIndivsPerPopLoc[pop][loc];i<maxIndivPop;++i){//unsampled individuals blank
						aString& indiv = (*rawGList)[i];
						indiv.cat("   ");
						}//end for i
					}
				else{//monomorphic locus
					for(int i=0;i<numIndivsPerPopLoc[pop][loc];++i){
						aString& indiv = (*rawGList)[i];
						indiv.cat(" AA");
						}//end for i
					for(int i=numIndivsPerPopLoc[pop][loc];i<maxIndivPop;++i){//unsampled individuals blank
						aString& indiv = (*rawGList)[i];
						indiv.cat("   ");
						}//end for i
					}					
			twoN[pop].Append(2.0*numIndivsPerPopLoc[pop][loc]);
			}//end for loc
		rawGList=0;//forget this information
		}//end for pop	
	GroupMembership();
	LocalityMembership();
//	CullSkippedLoci();
	gtypes = rawGtypes;
	CheckForNewAlleles();	
	}//end PopulationGeneticData::CreateRandomData



void PopulationGeneticData::CreateRandomData(
							const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Flt=";
	title_.cat(Flt); title_.cat(", Fspt="); title_.cat(Fspt);  title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numSpeciesSampled * numLocalitiesSampled;
	nloci = numLociSampled;
	maxall = 2;
	ngroups = numSpeciesSampled;
	for(int loc = 0;loc<nloci;++loc){
		aString locName("LOC");
		locName.cat(loc+1);
		aString *s = NEW aString(locName);
		rawLocusName.Append(s);}//append a copy
	lociSkipped=0;
	locusName.Copy(rawLocusName);


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE SPECIES
//	GeneFrequencyDistribution parametricFreqDistribSP(Fspt, pBar, 1000);//old version sends dp
	GeneFrequencyDistribution parametricFreqDistribSP(Fspt, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<aList<long double> > pBarBySpeciesAndLocus;
	for(int sp0=0;sp0<numSpeciesSampled;++sp0){
		aList<long double>* locusPList = NEW aList<long double>();
		pBarBySpeciesAndLocus.Append(locusPList);
		for(int locus0=0;locus0<nloci;++locus0){
			long double p0 = parametricFreqDistribSP.RandomFrequency(ran);
			locusPList->Append(p0);
			}//end for locus0
		}//end for sp0
		
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	for(int sp1=0; sp1<numSpeciesSampled;++sp1){
		//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG populations within the species
		ObjList<StatList<long double> > locusFrequenciesByLocality;
		for(int locality1=0;locality1<numLocalitiesSampled; ++locality1){
			StatList<long double>* locusFrequencies = NEW StatList<long double>();
			locusFrequenciesByLocality.Append(locusFrequencies);
			for(int loc=0;loc<nloci;++loc){
				GeneFrequencyDistribution parametricFreqDistrib(Flt, pBarBySpeciesAndLocus[sp1][loc], 1000);
				long double p = parametricFreqDistrib.RandomFrequency(ran);
				locusFrequencies->Append(p);
				}//end for loc
			}//end for locality1

		for (int locality1=0;locality1<numLocalitiesSampled; ++locality1){
			// NAME THE POPULATION
			aString pName("LOCALITY");pName.cat(locality1+1);
			pName.cat(" randomly generated population of SPECIES"); pName.cat(sp1+1);
			aString *s = NEW aString(pName);
			popName.Append(s);//append a copy

			StringList *rawGList = NEW StringList();
			rawGtypes.Append(rawGList);				
			for (int i=0;i<numIndivsPerPop;++i){
				aString indiv(i+1);//first create id numbers for each individual
				indiv.PrependCharToLen(' ', 4);
				aString *s = NEW aString(indiv);
				rawGList->Append(s);
				}//end for i
			popSize.Append(numIndivsPerPop);
			twoN.Append(NEW aList<int>());

			for(int loc=0;loc<nloci;++loc){
				StatList<long double>& locusFrequencies = locusFrequenciesByLocality[locality1];
				double p = locusFrequencies[loc];
//				cout<<"pop:  "<<pop+1<<"   "<<"loc:  "<<loc+1<<"   "<<p<<endl;
				GenotypeFreqDistribution popGenotypes(p, Fis);
//				popGenotypes.PrintToScreen();
				for(int i=0;i<numIndivsPerPop;++i){
					aString& indiv = (*rawGList)[i];
					indiv.cat(' ');
					aString genotype = '\0';
					int gtype = popGenotypes.RandomGenotype(ran);
					if (gtype==0){genotype = "AA";}
						else if(gtype==1){genotype = "AB";}
						else if(gtype==2){genotype = "BB";}			
					indiv.cat(genotype);
//					cout<<pop+1<<"   "<<indiv<<endl;
					}//end for i
				(twoN.Last()).Append(2*numIndivsPerPop);
				}//end for loc
			rawGList=0;//forget this information
			}//end for locality1
		}//end for sp1	
	GroupMembership();
	LocalityMembership();
//	CullSkippedLoci();
	gtypes = rawGtypes;
	CheckForNewAlleles();	
	}//end PopulationGeneticData::CreateRandomData




void PopulationGeneticData::CreateRandomData(//not debugged
							const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Flt=";
	title_.cat(Flt); title_.cat(", Fspt="); title_.cat(Fspt);  
	title_.cat(", Flspt="); title_.cat(Flspt); title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numSpeciesSampled * numLocalitiesSampled;
	nloci = numLociSampled;
	maxall = 2;
	ngroups = numSpeciesSampled;
	for(int loc = 0;loc<nloci;++loc){
		aString locName("LOC");
		locName.cat(loc+1);
		aString *s = NEW aString(locName);
		rawLocusName.Append(s);}//append a copy
	lociSkipped=0;
	locusName.Copy(rawLocusName);


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE SPECIES
//	GeneFrequencyDistribution parametricFreqDistribSP(Fspt, pBar, 1000);//old version sends dp
	GeneFrequencyDistribution parametricFreqDistribSP(Fspt, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<aList<long double> > pBarBySpeciesAndLocus;
	for(int sp0=0;sp0<numSpeciesSampled;++sp0){
		aList<long double>* locusPList = NEW aList<long double>();
		pBarBySpeciesAndLocus.Append(locusPList);
		for(int locus0=0;locus0<nloci;++locus0){
			long double p0 = parametricFreqDistribSP.RandomFrequency(ran);
			locusPList->Append(p0);
			}//end for locus0
		}//end for sp0
		
	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE LOCALITIES
	GeneFrequencyDistribution parametricFreqDistribLocality(Flt, pBar, 1000);
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<aList<long double> > pBarByLocalityAndLocus;
	for(int L0=0;L0<numLocalitiesSampled;++L0){
		aList<long double>* locusPList = NEW aList<long double>();
		pBarByLocalityAndLocus.Append(locusPList);
		for(int locus0=0;locus0<nloci;++locus0){
			long double p0 = parametricFreqDistribLocality.RandomFrequency(ran);
			locusPList->Append(p0);
			}//end for locus0
		}//end for L0
		
	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE SUBPOPULATIONS
//	GeneFrequencyDistribution parametricFreqDistribSubpop(Flspt, pBar, 1000);//old version sends dp
	GeneFrequencyDistribution parametricFreqDistribSubpop(Flspt, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<ObjList<aList<long double> > > pBarBySpeciesLocalityAndLocus;
	for(int sp1=0;sp1<numSpeciesSampled;++sp1){
		pBarBySpeciesLocalityAndLocus.Append(NEW ObjList<aList<long double> >());
		for(int L1=0;L1<numLocalitiesSampled;++L1){
			aList<long double>* locusPList = NEW aList<long double>();
			pBarBySpeciesLocalityAndLocus[sp1].Append(locusPList);
			for(int locus0=0;locus0<nloci;++locus0){
				long double p0 = parametricFreqDistribSubpop.RandomFrequency(ran);
				long double devLocality = (pBarByLocalityAndLocus[L1][locus0] - pBar);
				long double devSpecies = (pBarBySpeciesAndLocus[sp1][locus0] - pBar);
				long double p = p0 + devLocality + devSpecies;
				locusPList->Append(p);
				}//end for locus0
			}//end for L0
		}//end for sp0
		
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	for(int sp1=0; sp1<numSpeciesSampled;++sp1){
		//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG populations within the species
/*		ObjList<StatList<double> > locusFrequenciesByLocality;
		for(int locality1=0;locality1<numLocalitiesSampled; ++locality1){
			StatList<double>* locusFrequencies = new StatList<double>();
			locusFrequenciesByLocality.Append(locusFrequencies);
			for(int loc=0;loc<nloci;++loc){
				GeneFrequencyDistribution parametricFreqDistrib(Flspt, pBarBySpeciesLocalityAndLocus[sp1][locality1][loc], 1000);
				double p = parametricFreqDistrib.RandomFrequency(ran);
				locusFrequencies->Append(p);
				}//end for loc
			}//end for locality1
*/
		for (int locality1=0;locality1<numLocalitiesSampled; ++locality1){
			// NAME THE POPULATION
			aString pName("LOCALITY");pName.cat(locality1+1);
			pName.cat(" randomly generated population of SPECIES"); pName.cat(sp1+1);
			aString *s = NEW aString(pName);
			popName.Append(s);//append a copy

			StringList *rawGList = NEW StringList();
			rawGtypes.Append(rawGList);				
			for (int i=0;i<numIndivsPerPop;++i){
				aString indiv(i+1);//first create id numbers for each individual
				indiv.PrependCharToLen(' ', 4);
				aString *s = NEW aString(indiv);
				rawGList->Append(s);
				}//end for i
			popSize.Append(numIndivsPerPop);
			twoN.Append(NEW aList<int>());

			for(int loc=0;loc<nloci;++loc){
//				StatList<double>& locusFrequencies = locusFrequenciesByLocality[locality1];
				long double p = pBarBySpeciesLocalityAndLocus[sp1][locality1][loc];
//				cout<<"pop:  "<<pop+1<<"   "<<"loc:  "<<loc+1<<"   "<<p<<endl;
				GenotypeFreqDistribution popGenotypes(p, Fis);
//				popGenotypes.PrintToScreen();
				for(int i=0;i<numIndivsPerPop;++i){
					aString& indiv = (*rawGList)[i];
					indiv.cat(' ');
					aString genotype = '\0';
					int gtype = popGenotypes.RandomGenotype(ran);
					if (gtype==0){genotype = "AA";}
						else if(gtype==1){genotype = "AB";}
						else if(gtype==2){genotype = "BB";}			
					indiv.cat(genotype);
//					cout<<pop+1<<"   "<<indiv<<endl;
					}//end for i
				(twoN.Last()).Append(2*numIndivsPerPop);
				}//end for loc
			rawGList=0;//forget this information
			}//end for locality1
		}//end for sp1	
	GroupMembership();
	LocalityMembership();
//	CullSkippedLoci();
	gtypes = rawGtypes;
	CheckForNewAlleles();	
	}//end PopulationGeneticData::CreateRandomData


void PopulationGeneticData::CreateRandomDataLocFirst(
							const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Flt=";
	title_.cat(Flt); title_.cat(", Fspt="); title_.cat(Fspt);  title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numSpeciesSampled * numLocalitiesSampled;
	nloci = numLociSampled;
	maxall = 2;
	ngroups = numSpeciesSampled;
	for(int loc = 0;loc<nloci;++loc){
		aString locName("LOC");
		locName.cat(loc+1);
		aString *s = NEW aString(locName);
		rawLocusName.Append(s);}//append a copy
	lociSkipped=0;
	locusName.Copy(rawLocusName);


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE LOCALITIES
//	GeneFrequencyDistribution parametricFreqDistribL(Flt, pBar, 1000);//old version sends dp
	GeneFrequencyDistribution parametricFreqDistribL(Flt, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	aList<long double> pBarByLocality;
	for(int L0=0;L0<numLocalitiesSampled;++L0){
		long double p0 = parametricFreqDistribL.RandomFrequency(ran);
		pBarByLocality.Append(p0);
		}//end for L0
		
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	for(int L1=0; L1<numLocalitiesSampled;++L1){
		//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG SPECIES WITHIN THE LOCALITIES
//		GeneFrequencyDistribution parametricFreqDistrib(Fspt, pBarByLocality[L1], 1000);//old version sends dp
		GeneFrequencyDistribution parametricFreqDistrib(Fspt, pBarByLocality[L1], 2);//new version sends ploidy
		ObjList<StatList<long double> > locusFrequenciesBySpecies;
		for(int species1=0;species1<numSpeciesSampled; ++species1){
			StatList<long double>* locusFrequencies = NEW StatList<long double>();
			locusFrequenciesBySpecies.Append(locusFrequencies);
			for(int loc=0;loc<nloci;++loc){
				long double p = parametricFreqDistrib.RandomFrequency(ran);
				locusFrequencies->Append(p);
				}//end for loc
			}//end for species1

		for (int species1=0;species1<numSpeciesSampled; ++species1){
			// NAME THE POPULATION
			aString pName("LOCALITY");pName.cat(L1+1);
			pName.cat(" randomly generated population of SPECIES"); pName.cat(species1+1);
			aString *s = NEW aString(pName);
			popName.Append(s);//append a copy

			StringList *rawGList = NEW StringList();
			rawGtypes.Append(rawGList);				
			for (int i=0;i<numIndivsPerPop;++i){
				aString indiv(i+1);//first create id numbers for each individual
				indiv.PrependCharToLen(' ', 4);
				aString *s = NEW aString(indiv);
				rawGList->Append(s);
				}//end for i
			popSize.Append(numIndivsPerPop);
			twoN.Append(NEW aList<int>());

			for(int loc=0;loc<nloci;++loc){
				StatList<long double>& locusFrequencies = locusFrequenciesBySpecies[species1];
				long double p = locusFrequencies[loc];
				GenotypeFreqDistribution popGenotypes(p, Fis);
				for(int i=0;i<numIndivsPerPop;++i){
					aString& indiv = (*rawGList)[i];
					indiv.cat(' ');
					aString genotype = '\0';
					int gtype = popGenotypes.RandomGenotype(ran);
					if (gtype==0){genotype = "AA";}
						else if(gtype==1){genotype = "AB";}
						else if(gtype==2){genotype = "BB";}			
					indiv.cat(genotype);
					}//end for i
				(twoN.Last()).Append(2*numIndivsPerPop);
				}//end for loc
			rawGList=0;//forget this information
			}//end for species1
		}//end for L1	
	GroupMembership();
	LocalityMembership();
//	CullSkippedLoci();
	gtypes = rawGtypes;
	CheckForNewAlleles();	
	}//end PopulationGeneticData::CreateRandomDataLocFirst


void PopulationGeneticData::CullLoci (StringList& lociToCull){
	//This function culls the designated loci from each genotype data string
	//and includes them among the skipped loci
	if(lociToCull.len()==0){return;}//none to cull
	for(int cLoc=0;cLoc<lociToCull.len();++cLoc){
		aString cLocusname = lociToCull[cLoc];
		int item = rawLocusName.Loc(&cLocusname);
		if((item>=0)&&(!(skippedLocusName.Includes(&cLocusname)))){
				//the locus exists in the rawlist && isn't already skipped
			skippedLocusName.Append(NEW aString(lociToCull[cLoc]));
			skippedLocusNum.Append(item);
			item = locusName.Loc(&cLocusname);
			locusName.Delete(item);
			lociSkipped++;--nloci;
			}//if item exists
		}//cLoc
	ResetFlags();
	CullSkippedLoci();
	CheckForNewAlleles();
	}//end procedure PopulationGeneticData::CullLoci(StringList&)



void PopulationGeneticData::CullLocus (char* locusToCull){
	aString locToCull(locusToCull);
	CullLocus(locToCull);
	}//end PopulationGeneticData::CullLocus (char* )


void PopulationGeneticData::CullLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	CullLoci(lociToCull);
	}//end PopulationGeneticData::CullLocus (aString& )


void PopulationGeneticData::CullMonomorphicLoci (void){	
	//This function culls the monomorphic loci from each genotype data string
	//and includes them among the skipped loci
	UpdateVariableLoci();
	if(variableLoci_.len()==locusName.len()){return;}//none monomorphic
	StringList monomorphicLoci(locusName);
	for(int vLoc=0; vLoc<variableLoci_.len();++vLoc){
		aString vLocusname = locusName[variableLoci_[vLoc]];
		int item = monomorphicLoci.Loc(&vLocusname);
		monomorphicLoci.Delete(item);
		}//vLoc
	CullLoci(monomorphicLoci);
	}//end procedure PopulationGeneticData::CullMonomorphicLoci



void PopulationGeneticData::CullSkippedLoci (void){	
	//This subprogram culls the skipped loci from each genotype data string
	gtypes.Clear();
	aList<int> skippedLociCopy = skippedLocusNum;
	skippedLociCopy.Sort();
	for(int pop=0;pop<npop_;++pop){
		StringList& popRawGtypes = rawGtypes[pop];
		StringList *popGtypes = NEW StringList();
		gtypes.Append(popGtypes);
		for(int i=0;i<popRawGtypes.len();++i){
			aString& dataStr = popRawGtypes[i];
			aString *dataStrCopy = NEW aString(dataStr);
			//first snip off the individual' s designation code}
			aString *codeWord = dataStrCopy->ChopOffFirstWord();
			int codeLen = codeWord->len();
			delete dataStrCopy; dataStrCopy=0;
			dataStrCopy=NEW aString(dataStr);
			popGtypes->Append(dataStrCopy);
			//then remove the 3 characters - 2 alleles and a blank - for the skipped locus}
			//work from last locus to first, to avoid problems with miscounting}
			for (int locToSkip=skippedLociCopy.len()-1;locToSkip>=0;--locToSkip){
				int loc = skippedLociCopy[locToSkip];
				int posLocToSkip = dataStrCopy->pos(*codeWord) + codeLen + (3 * loc);
				dataStrCopy->cut(posLocToSkip, posLocToSkip+2);
				}//	end for locToSkip
			delete codeWord; codeWord=0;
			}//end for i
		}//end for pop
	}//end procedure PopulationGeneticData::CullSkippedLoci




PopulationGeneticData* 	PopulationGeneticData::CullToOneGroup (int groupNum){
	//makes a subset data structure from the specified group
	//individuals resampled
	aString newTitle = title_;
	newTitle.cat(" -- ");
	newTitle.cat(groupName[groupNum]);
	newTitle.cat(" only");
	PopulationGeneticData *subsetData = NEW PopulationGeneticData(newTitle);

	subsetData->ngroups = 1;
	subsetData->npop_ = groupHierarchy[groupNum].len();
	subsetData->nloci = nloci;
	subsetData->maxall = maxall;
	subsetData->lociSkipped = lociSkipped;
	subsetData->groupName.Append(NEW aString(groupName[groupNum]));
	subsetData->localityName = localityName;
	for(int pop=0;pop<subsetData->npop_;++pop){
		aString *newName = NEW aString(popName[groupHierarchy[groupNum][pop]]);
		subsetData->popName.Append(newName);
		}
	subsetData->rawLocusName = rawLocusName;
	subsetData->locusName = locusName;
	subsetData->skippedLocusName = skippedLocusName;
	subsetData->skippedLocusNum = skippedLocusNum;
	
	for(int pop=0;pop<subsetData->npop_;++pop){
		int theRightPop = groupHierarchy[groupNum][pop];
		StringList *theRightRawList = (StringList*) rawGtypes[theRightPop].Duplicate();
		subsetData->rawGtypes.Append(theRightRawList);
		StringList *theRightList =(StringList*)  gtypes[theRightPop].Duplicate();
		subsetData->gtypes.Append(theRightList);
		}
	
	
	subsetData->GroupMembership();
	subsetData->LocalityMembership();
//	subsetData->CullSkippedLoci();
	subsetData->CheckForNewAlleles();
	return subsetData;
	}//end PopulationGeneticData::CullToOneGroup (int)



StringList& PopulationGeneticData::Genotype(int pop, int indiv, int loc){
	if(! gtypesCurrent_){ CreateGtypeList();}
	return gtypesPopIndLocAll[pop][indiv][loc];
	}//end  PopulationGeneticData::Genotype(int,int,int)


aString& PopulationGeneticData::Genotype(int pop, int indiv, int loc, int allele){
	if(! gtypesCurrent_){ CreateGtypeList();}
	return gtypesPopIndLocAll[pop][indiv][loc][allele];
	}//end  PopulationGeneticData::Genotype(int,int,int,int)



void PopulationGeneticData::GenotypeTable(ObjList<MatrixList>& gtypeTableList){
	//stored pops within loci; square matrices of genotypes
	//ASSUMES DIPLOID GENOTYPES ONLY
	//above and below diagonal carry same data
	gtypeTableList.Clear();
	for(int loc=0;loc<nloci;++loc){
		gtypeTableList.Append(NEW MatrixList());}
	for(int loc=0;loc<nloci;++loc){
		int alleles = GetNumAlleles(loc);//in whole data set
		for(int pop=0;pop<npop_;++pop){
			gtypeTableList[loc].Append(NEW aMatrix<long double>((long double)0.0,alleles,alleles));}
		}//end for loc

	for(int pop=0;pop<npop_;++pop){
		for(int loc=0;loc<nloci;++loc){
			for(int i=0;i<gtypes[pop].len();++i){//each individual
				aString* igtype0 = GetGenotypeString(pop, i, loc, 0);
				aString* igtype1 = GetGenotypeString(pop, i, loc, 1);
				int aPos0 = allelesPresent[loc].Loc(igtype0);//-1 returned if not in list
				int aPos1 = allelesPresent[loc].Loc(igtype1);
				if((aPos0>=0)&&(aPos1>=0)){//alleles exist
					gtypeTableList[loc][pop][aPos0][aPos1]++;
					if(aPos0 != aPos1){//heterozygous; increment matrix symmetrically
						gtypeTableList[loc][pop][aPos1][aPos0]++;}
					}
				delete igtype0;igtype0=NULL;
				delete igtype1;igtype1=NULL;
				}//end for i
			}//end for loc
		}//end for pop	
	}//end PopulationGeneticData::GenotypeTable(ObjList<MatrixList>& )




long double PopulationGeneticData::GetExpectedHeterozygosity (int pop, int loc, int allele){
	long double freq = GetFreq(pop, loc, allele);
	return ((long double)2.0 * freq * ((long double)1.0 - freq));
	}//end PopulationGeneticData::GetExpectedHeterozygosity(pop,loc,allele)



long double PopulationGeneticData::GetExpectedHeterozygosity (int pop, int loc){
	long double expHomoFreq = 0.0;
	for (int allele=0;allele<GetNumAlleles(loc);++allele){
		long double freq = GetFreq(pop, loc, allele);
		expHomoFreq += freq*freq;}
	return ((long double)1.0 - expHomoFreq);
	}//end PopulationGeneticData::GetExpectedHeterozygosity(pop,loc)



long double PopulationGeneticData::GetExpectedHeterozygosity (int pop){
	//returns mean Hexp over loci
	StatList<long double> hetExp;
	for (int loc=0;loc<nloci;++loc){
		hetExp.Append(GetExpectedHeterozygosity(pop,loc));}
	return hetExp.mean();
	}//end PopulationGeneticData::GetExpectedHeterozygosity(pop)



long double PopulationGeneticData::GetFreq (int loc, int allele){
	long double numAlleleCopies = 0.0;
	long double numThisAllele = 0.0;
	for(int pop=0;pop<npop_;++pop){
		long double n = (long double)GetLocusSampleSize(pop,loc);
		numAlleleCopies += n;
		numThisAllele += n * GetFreq(pop,loc,allele);
		}
	if(numAlleleCopies==0.0){return (long double)0.0;}
	return numThisAllele/numAlleleCopies;
	}//end PopulationGeneticData::GetFreq


long double PopulationGeneticData::GetFreq (int pop, int loc, int allele){
	if (!alleleFreqCurrent_){CalculateAlleleFreqs();}
	return alleleFreq[pop][loc][allele];
	}//end PopulationGeneticData::GetFreq


long double PopulationGeneticData::GetFreqMostCommonAllele (int pop, int loc){
	long double mostCommon = 0.0;
	for(int allele=0;allele<GetNumAlleles(loc);++allele){
		long double freq = GetFreq(pop,loc,allele);
		if (freq>(long double)0.5) {return freq;}
		if (mostCommon<freq){mostCommon = freq;}
		}//and for allele
	return mostCommon;
	}//end PopulationGeneticData::GetFreqMostCommonAllele


aString* PopulationGeneticData::GetGenotypeString(int pop, int indiv){
	StringList& genotypesList=gtypes[pop];
	aString& dataStr = genotypesList[indiv];
	aString *gtypeWord = NEW aString(dataStr);
	aString *codeWord = gtypeWord->ChopOffFirstWord();
	delete codeWord; codeWord=0;
	return gtypeWord;
	}//end PopulationGeneticData::GetGenotypeString(pop,indiv)



aString* PopulationGeneticData::GetGenotypeString(int pop, int indiv, int loc){
	aString *gtypeWord=GetGenotypeString(pop, indiv);
	int posGtypeWord = 1 + (2+1) * loc;
	aString *gTypeString = NEW aString(*gtypeWord,posGtypeWord,posGtypeWord +1);
	delete gtypeWord;gtypeWord=0;//this was created in GetGenotypeString(pop, indiv);
	return gTypeString;
	}//end PopulationGeneticData::GetGenotypeString(pop,indiv,loc)


aString* PopulationGeneticData::GetGenotypeString(int pop, int indiv, int loc, int chrom){
//	aString *gtypeString=GetGenotypeString(pop, indiv, loc);
			//	aString *gTypeChrom = NEW aString(*gtypeString,chrom,chrom);
//	aString *gTypeChrom = NEW aString( (*gtypeString)[chrom]);
//	delete gtypeString;gtypeString=0;//this was created in GetGenotypeString(pop, indiv, loc);
//	return gTypeChrom;

	StringList gtypeString=Genotype(pop, indiv, loc);
	aString *gTypeChrom = NEW aString( gtypeString[chrom]);
	return gTypeChrom;
	}//end PopulationGeneticData::GetGenotypeString(pop,indiv,loc,chrom)


long PopulationGeneticData::GetIndividualsSampled (int pop, int loc){
	long n=0;
	for(long i=0;i<individuals(pop);++i){
		if(GetLocusSampleSize (pop, i, loc)>0){
			++n;}}
	return n;
	}//end  PopulationGeneticData::GetIndividualsSampled


int PopulationGeneticData::GetLocality (int pop){
	for(int L=0;L<popsByLocality.len();++L){
		if (popsByLocality[L].Includes(pop)){
			return L;}}
	return (-1);//default if failure
	}//end  PopulationGeneticData::GetLocality


long PopulationGeneticData::GetLocusSampleSize (int loc){
	double numAlleleCopies = 0.0;
	for(int pop=0;pop<npop_;++pop){
		numAlleleCopies += twoN[pop][loc];
		}
	return numAlleleCopies;
	}//end  PopulationGeneticData::GetLocusSampleSize


long PopulationGeneticData::GetLocusSampleSize (int pop, int loc){
	return twoN[pop][loc];
	}//end  PopulationGeneticData::GetLocusSampleSize


int PopulationGeneticData::GetLocusSampleSize (int pop, int indiv, int loc){
	StringList g(Genotype(pop,indiv,loc));
	return g.len();//assumes that missing data have been culled
	}//end  PopulationGeneticData::GetLocusSampleSize


long double PopulationGeneticData::GetMeanAllelesPerLocus (int pop){
	long double meanAlleles = 0.0;
	for(int loc=0;loc<nloci;++loc){
		meanAlleles += (long double)GetNumAlleles(pop,loc);}
	meanAlleles /= (long double)nloci;
	return meanAlleles;
	}//end  PopulationGeneticData::GetMeanAllelesPerLocus


int PopulationGeneticData::GetNumAlleles (int loc){
	//total in data set
	if(! allelesCurrent_){CheckForNewAlleles();}
	StringList& locList = allelesPresent[loc];
	return locList.len();
	}//end  PopulationGeneticData::GetNumAlleles(loc)


int PopulationGeneticData::GetNumAlleles (int pop, int loc){
	//extant alleles in population
	if(! alleleFreqCurrent_){CalculateAlleleFreqs();}
	ObjList<aList<long double> >& popList = alleleFreq[pop];
	aList<long double>& locList = popList[loc];
	int count=0;
	for(int a=0;a<locList.len();++a){
		if (locList[a] >0.0){++count;}}
	return count;
	}//end  PopulationGeneticData::GetNumAlleles(pop,loc)


long double PopulationGeneticData::GetObservedHeterozygosity(int pop, int loc, int allele){
	if (!HetObsCurrent_){CalculateObservedHeterozygosity();}
	ObjList<aList<long double> >& popList = HetObsByAllele[pop];
	aList<long double>& locList = popList[loc];
	return locList[allele];
	}//end PopulationGeneticData::GetObservedHeterozygosity(pop,loc,allele)


long double PopulationGeneticData::GetObservedHeterozygosity(int pop, int loc){
	if (!HetObsCurrent_){CalculateObservedHeterozygosity();}
	aList<long double>& popList = HetObsByLocus[pop];
	return popList[loc];
	}//end PopulationGeneticData::GetObservedHeterozygosity(pop,loc)


long double PopulationGeneticData::GetObservedHeterozygosity (int pop){
	//returns mean Hobs over loci
	StatList<long double> hetObs;
	for (int loc=0;loc<nloci;++loc){
		hetObs.Append(GetObservedHeterozygosity(pop,loc));}
	return hetObs.mean();
	}//end PopulationGeneticData::GetObservedHeterozygosity(pop)



long double PopulationGeneticData::GetPercentPolymorphicLoci(int pop, double criterion){
	//default criterion is 100.0% = any polymorphism
	long double percentP = 0.0;
	criterion /= 100.0; 
	for(int loc=0;loc<nloci;++loc){
		long double mostCommon = GetFreqMostCommonAllele(pop,loc);
		if(mostCommon<(long double)criterion){++percentP;}}
	return (percentP/(long double)nloci) *(long double)100.0;
	}//end PopulationGeneticData::GetPercentPolymorphicLoci(pop,loc)


long double PopulationGeneticData::GetVarJackAllelesPerLocus (int pop){
	//returns jackknife variance of means, dropping loci
	StatList<long double> allelesPerLoc;
	for (int loc=0;loc<nloci;++loc){
		allelesPerLoc.Append((long double)GetNumAlleles(pop,loc));}
	StatList<long double> jackAllelesPerLoc;
	for (int loc=0;loc<nloci;++loc){
		long double storeValue = allelesPerLoc.First();
		allelesPerLoc.DeleteFirst();
		jackAllelesPerLoc.Append(allelesPerLoc.mean());
		allelesPerLoc.Append(storeValue);
		}
	return jackAllelesPerLoc.var();
	}//end PopulationGeneticData::GetVarJackAllelesPerLocus(pop)



long double PopulationGeneticData::GetVarJackExpectedHeterozygosity (int pop){
	//returns jackknife variance of means, dropping loci
	StatList<long double> hetExpLoc;
	for (int loc=0;loc<nloci;++loc){
		hetExpLoc.Append(GetExpectedHeterozygosity(pop, loc));}
	StatList<long double> jackHetExpLoc;
	for (int loc=0;loc<nloci;++loc){
		long double storeValue = hetExpLoc.First();
		hetExpLoc.DeleteFirst();
		jackHetExpLoc.Append(hetExpLoc.mean());
		hetExpLoc.Append(storeValue);
		}
	return jackHetExpLoc.var();
	}//end PopulationGeneticData::GetVarJackExpectedHeterozygosity(pop)



long double PopulationGeneticData::GetVarJackObservedHeterozygosity (int pop){
	//returns jackknife variance of means, dropping loci
	StatList<long double> hetObsLoc;
	for (int loc=0;loc<nloci;++loc){
		hetObsLoc.Append(GetObservedHeterozygosity(pop, loc));}
	StatList<long double> jackHetObsLoc;
	for (int loc=0;loc<nloci;++loc){
		long double storeValue = hetObsLoc.First();
		hetObsLoc.DeleteFirst();
		jackHetObsLoc.Append(hetObsLoc.mean());
		hetObsLoc.Append(storeValue);
		}
	return jackHetObsLoc.var();
	}//end PopulationGeneticData::GetVarJackObservedHeterozygosity(pop)



long double PopulationGeneticData::GetVarJackPercentPolyLoci (int pop, double criterion){
	//returns jackknife variance of means, dropping loci
	StatList<long double> jackPercentP;
	for (int loc=0;loc<nloci;++loc){
		jackPercentP.Append(GetPercentPolymorphicLoci(pop,criterion));}
	for (int loc=0;loc<nloci;++loc){
		double long temp = (jackPercentP[loc]/100.0)*nloci;
		if(IsPolymorphic(pop,loc,criterion)){--temp;}
		jackPercentP.Replace((temp/(long double)(nloci-1))*(long double)100.0,loc);
		}
	return jackPercentP.var();
	}//end PopulationGeneticData::GetVarJackPercentPolyLoci(pop,criterion)



void PopulationGeneticData::GroupMembership (void){
	//This routine determines group membership by the last full word in
	//the population title_.  The first population determines the name of the 
	//first group, the next different population determines the name of the secondgroup, etc.

	//groupName saves the names of the groups, in the order encountered
	//groupHierarchy saves the population numbers from the
	//	original population series, so they can be retrieved from group membership data.
	//groupSize saves the number of populations in a group, indexed by group number
	groupName.Clear();
	groupHierarchy.Clear();
	if (ngroups<=1){//all populations belong to the same group
			aString *thisGName = NEW aString("pooled populations");
			groupName.Append(thisGName);
			aList<int> *groupList = NEW aList<int>();
			groupHierarchy.Append(groupList);
			}
	int groupNum=0;
	for (int pop=0;pop<npop_;++pop){
		if (ngroups>1){
			// there is more than one group		
			// first get the name of the group from the population name string
			aString& popString = popName[pop];
			aString popStringCopy(popString);
			aString *thisGName = popStringCopy.ChopOffLastWord();

			//does this group already exist?
			if(groupName.Includes(thisGName)){
					groupNum = groupName.Loc(thisGName);}
				else{//add it to the list & add slots to corresponding lists
					aString *nameToAdd = NEW aString(thisGName);
					groupNum = groupName.len();
					groupName.Append(nameToAdd);
					aList<int> *groupList = NEW aList<int>();
					groupHierarchy.Append(groupList);
					}
			delete thisGName; thisGName=0;
			}//end if ngroups>1
		aList<int>& groupList = groupHierarchy[groupNum];
		groupList.Append(pop);
		}//end for pop
	}//end  PopulationGeneticData::GroupMembership


int PopulationGeneticData::GroupMembership(int pop){
	if(groupHierarchy.len()==0){
		GroupMembership();}
	for(int g=0;g<groupHierarchy.len();++g){
		if (groupHierarchy[g].Includes(pop)){
			return g;}}
	aString err("ERROR!  Population # ");err.cat(pop);err.cat(" not in any group.  group=-1 returned.");
	aError(err);
	return -1;
	}//end  PopulationGeneticData::GroupMembership(int)


int PopulationGeneticData::individuals(int pop, int loc){
	int indivs=0;
	for(int i=0;i<individuals(pop);++i){
		if (GetLocusSampleSize ( pop, i, loc)>0){++indivs;}}
	return indivs;
	}//end  PopulationGeneticData::individuals(int,int)
	


int  PopulationGeneticData::IsPolymorphic(int pop, int loc){//boolean
	double freq = GetFreqMostCommonAllele(pop,loc);
	if (freq==0.0){return 0;}
	if (freq<1.0){return 1;}else {return 0;}
	}// end PopulationGeneticData::IsPolymorphic


int  PopulationGeneticData::IsPolymorphic(int pop, int loc, double criterion){//boolean
	double freq = GetFreqMostCommonAllele(pop,loc);
	if (freq==0.0){return 0;}
	if (freq<(criterion/100.0)){return 1;}else {return 0;}
	}// end PopulationGeneticData::IsPolymorphic


void PopulationGeneticData::LocalityMembership (void){
	//This routine determines locality membership by the first full word in
	//the population title_.  The first population determines the name of the 
	//first locality, the next different population determines the name of the second locality, etc.

	//localityName saves the names of the localities, in the order encountered
	//popsByLocality saves the population numbers from the
	//	original population series, so they can be retrieved from locality membership data.
	localityName.Clear();
	popsByLocality.Clear();
	aString endCharList(" \t,;:\"(){}[]!?");
	int localityNum=0;
	for (int pop=0;pop<npop_;++pop){
		// there is more than one group		
		// first get the name of the group from the population name string
		aString& popString = popName[pop];
		aString popStringCopy(popString);
		aString *thisLName = popStringCopy.ChopOffFirstWord(endCharList);

		//does this locality already exist?
		if(localityName.Includes(thisLName)){
				localityNum = localityName.Loc(thisLName);}
			else{//add it to the list & add slots to corresponding lists
				aString *nameToAdd = NEW aString(thisLName);
				localityNum = localityName.len();
				localityName.Append(nameToAdd);
				aList<int> *localityList0 = NEW aList<int>();
				popsByLocality.Append(localityList0);
				}
		delete thisLName; thisLName=0;
		aList<int>& localityList1 = popsByLocality[localityNum];
		localityList1.Append(pop);
		}//end for pop
	}//end  PopulationGeneticData::LocalityMembership


int PopulationGeneticData::LocalityMembership(int pop){
	if(popsByLocality.len()==0){
		LocalityMembership();}
	for(int L=0;L<popsByLocality.len();++L){
		if (popsByLocality[L].Includes(pop)){
			return L;}}
	aString err("ERROR!  Population # ");err.cat(pop);err.cat(" not in any locality.  Locality=-1 returned.");
	aError(err);
	return -1;
	}//end  PopulationGeneticData::LocalityMembership(int)






void  PopulationGeneticData::PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of allelic frequencies for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculateAlleleFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied


	outFile<<title_<<endl<<endl<<endl;
	outFile<<"ALLELE FREQUENCY DATA"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop_<<endl;
	outFile<<"NUMBER OF LOCI SURVEYED = ";outFile.width(3);outFile<<nloci<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (lociSkipped == 0){
			outFile<<"none";}
		else{
			for (int loc=0;loc< lociSkipped;++loc){
				aString& locName = skippedLocusName[loc];
				outFile<<locName;
				if (loc < lociSkipped-1){outFile<<", ";}
				}//end for loc
		}//end if lociSkipped
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop_ / ncols;
	int npop0 = npop_;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Allele frequencies for each population."<<endl;
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop_, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;

		for (int loc=0;loc<nloci;++loc) {//print the locus and locus sample size
			aString& locWord = locusName[loc];
			outFile<< locWord;
			if (asciiFormat){outFile.width(11 + (5 - locWord.len()));}else{outFile<<colSpacer;}
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = GetLocusSampleSize(pop, loc) / 2;
				outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
				if (pop < endPop){
					if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
					}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					long double freq = GetFreq(pop, loc, allele);
					aString freqStr(freq,0,4);
					if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									if (asciiFormat){
											aString blanks(' ',12);
											if (pop==x) {
												blanks.PrependCharToLen(' ',17);}
											outFile<<blanks;
											}
										else{outFile<<colSpacer;}
									}
								else{
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end PopulationGeneticData::PrintAlleleFreqs




void  PopulationGeneticData::PrintAlleleFreqsByPopList (ostream& outFile, 
											ObjList<ObjList<aList<double> > >& popFreqList, 
											ObjList<aList<double> >& popTwoNList, 
											int ncols){
	//popFreqList must contain the entire set of alleles in the data set
	//Sends a table of allelic frequencies to the output file.
	//ncols indexes the number of population columns per page

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = popFreqList.len() / ncols;
	int npop0 = popFreqList.len();
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)popFreqList.len(), (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			outFile.width(12);outFile<<pop+1;}
		outFile<<endl;

		for (int loc=0;loc<nloci;++loc) {//print the locus and locus sample size
			aString& locWord = locusName[loc];
			outFile<< locWord;outFile.width(11 + (5 - locWord.len()));
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = popTwoNList[pop][loc] / 2;
				outFile<<"[";outFile.width(2);outFile<<samSize<<"]";
				if (pop < endPop){outFile.width(9);}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = popFreqList[pop][loc][allele];
					aString freqStr(freq,0,4);
					freqStr.PrependCharToLen(' ',12);
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									aString blanks(' ',12);
									if (pop==x) {
										blanks.PrependCharToLen(' ',17);}
									outFile<<blanks;}
								else{
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end PopulationGeneticData::PrintAlleleFreqsByPopList



void	 PopulationGeneticData::PrintGelJockFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintGelJockFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end PopulationGeneticData::PrintGelJockFile(aString&)



void	 PopulationGeneticData::PrintArlequinFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintArlequinFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end PopulationGeneticData::PrintArlequinFile(aString&)



void PopulationGeneticData::PrintGelJockFile (ostream& outFile){
	//		theGenotypes: ListStrStructureTypePtr;
	aString dataLine = title_;
	dataLine.cat(":  skipped loci omitted");
	outFile<<dataLine<<endl;
	dataLine = "NPOP=";
	dataLine.cat(npop_);
	dataLine.cat(", NLOCI=");
	dataLine.cat(nloci);
	dataLine.cat(", MAXALL=");
	dataLine.cat(maxall);
	dataLine.cat(", NGROUPS=");
	dataLine.cat(ngroups);
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int loc=0;loc<locusName.len();++loc){
		dataLine.cat(locusName[loc]);
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < locusName.len()){
				dataLine.cat(", ");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	dataLine = "SKIP=0";
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int pop=0;pop<npop_;++pop){
		outFile<<popName[pop]<<endl;
//		theSize := trunc(GetValueAt(popSize^, pop));
		StringList& theGenotypes = gtypes[pop];
		for (int i=0;i<theGenotypes.len();++i){
			dataLine = theGenotypes[i];
			outFile<<dataLine<<endl;
			}//end for i
		outFile<<"NEXT"<<endl;
		}//end for pop
	outFile<<"END"<<endl;
	}//end PopulationGeneticData::PrintGelJockFile


void PopulationGeneticData::PrintArlequinFile (ostream& outFile){
	//		theGenotypes: ListStrStructureTypePtr;
	outFile<<"[Profile]"<<endl;
	aString dataLine = "\tTitle=\"";
	dataLine.cat(title_);
	dataLine.cat('\"');
	outFile<<dataLine<<endl;
	dataLine = "\t\tNbSamples=";
	dataLine.cat(npop_);
	outFile<<dataLine<<endl;
	dataLine = "\t\tGenotypicData=1";
	outFile<<dataLine<<endl;
	dataLine = "\t\tGameticPhase=0";
	outFile<<dataLine<<endl;
	dataLine = "\t\tRecessiveData=0";
	outFile<<dataLine<<endl;
	dataLine = "\t\tIndividualLevel=1";
	outFile<<dataLine<<endl;
	dataLine = "\t\tMissingData=\"\?\"";
	outFile<<dataLine<<endl;
	dataLine = "\t\tCompDistMatrix=1";
	outFile<<dataLine<<endl<<endl;

	dataLine = "\t\tDataType=STANDARD";
	outFile<<dataLine<<endl;
	dataLine = "\t\tFrequency=ABS";
	outFile<<dataLine<<endl;
	dataLine = "\t\tLocusSeparator=WHITESPACE";
	outFile<<dataLine<<endl<<endl;
		
	dataLine = "[Data]";
	outFile<<dataLine<<endl<<endl;
	dataLine = "#Locus names:";
	outFile<<dataLine<<endl;

	dataLine= "\t# ";
	for (int loc=0;loc<locusName.len();++loc){
		dataLine.cat(locusName[loc]);
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine= "\t# ";}
			else if (loc+1 < locusName.len()){
				dataLine.cat("\t");}
			else{
				outFile<<dataLine<<endl;
				dataLine= "\t# ";}
		}//end for loc
	outFile<<endl<<endl;
	dataLine = "[[Samples]]";
	outFile<<dataLine<<endl<<endl;
	dataLine.clear();
	for (int pop=0;pop<npop_;++pop){
		dataLine = "\t\tSampleName=\"";
		dataLine.cat(popName[pop]);
		dataLine.cat('\"');
		outFile<<dataLine<<endl;
		StringList& theGenotypes = gtypes[pop];
		dataLine = "\t\tSampleSize=";
		dataLine.cat(theGenotypes.len());
		outFile<<dataLine<<endl;
		dataLine = "\t\tSampleData={";
		outFile<<dataLine<<endl;
		for (int i=0;i<theGenotypes.len();++i){
			//split each genotype into 2 parts, = "pseudohaplotypes"
			aString g(theGenotypes[i]);
			aString endCharList=" \t";
			aString *phaplo0 = g.ChopOffFirstWord(endCharList);
			phaplo0->cat(" 1 ");
			aString  phaplo1;phaplo1.PrependCharToLen(' ',phaplo0->len());
			outFile<<phaplo1<<endl;
			for(int loc=0;loc<loci();++loc){
				aString *gLoc0 = GetGenotypeString(pop,i,loc,0);
				aString *gLoc1 = GetGenotypeString(pop,i,loc,1);
				if(*gLoc0==' '){*gLoc0='\?';}
				if(*gLoc1==' '){*gLoc1='\?';}
				phaplo0->cat(*gLoc0);phaplo0->cat(' ');
				phaplo1.cat(*gLoc1);phaplo1.cat(' ');
				delete gLoc0; delete gLoc1;
				}
			outFile<<*phaplo0<<endl;
			outFile<< phaplo1<<endl;
			delete phaplo0;
			}//end for i
		outFile<<"\t\t}"<<endl<<endl;
		}//end for pop
	outFile<<endl<<endl;
	dataLine = "[[Structure]]";
	outFile<<dataLine<<endl<<endl;

	dataLine = "StructureName = \"Structure from population titles\"";
	outFile<<dataLine<<endl;
	dataLine = "NbGroups = 1";
	outFile<<dataLine<<endl;
	dataLine = "IndividualLevel = 1";
	outFile<<dataLine<<endl;
	for(int gr=0;gr<ngroups;++gr){
		dataLine = "\t#";
		dataLine.cat(groupName[gr]);
		outFile<<dataLine<<endl;
		dataLine = "\tGroup =  {";
		outFile<<dataLine<<endl;
		for(int pop=0;pop<groupHierarchy[gr].len();++pop){
			outFile<<"\t\t\""<<popName[groupHierarchy[gr][pop]]<<'\"'<<endl;
			}
		outFile<<"\t\t}"<<endl;
	}


	outFile<<endl;
	}//end PopulationGeneticData::PrintArlequinFile





void PopulationGeneticData::PrintHeterozygosity (ostream& outFile, const int hetType, int ncols){
	//file assumed open
	//Sends a table of the designated heterozygosities for each population to the output file.
	//ncols indexes the number of population columns per page
	outFile<<title_<<endl<<endl<<endl;
	if (hetType == observed) {
			outFile<<"OBSERVED HETEROZYGOSITY TABLE";}
		else{
			outFile<<"EXPECTED HETEROZYGOSITY TABLE";}
	outFile<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop_<<endl;
	outFile<<"NUMBER OF LOCI SURVEYED = ";outFile.width(3);outFile<<nloci<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (lociSkipped == 0){
			outFile<<"none";}
		else{
			for (int loc=0;loc< lociSkipped;++loc){
				aString& locName = skippedLocusName[loc];
				outFile<<locName;
				if (loc < lociSkipped-1){outFile<<", ";}
				}//end for loc
		}//end if lociSkipped
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop_ / ncols;
	int npop0 = npop_;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				if (hetType == expected) {
						outFile<<"Expected ";}
					else{
						outFile<<"Observed ";}
				outFile<<"heterozygosities for each allele in each population."<<endl;
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				if (hetType == expected) {
						outFile<<"Expected ";}
					else{
						outFile<<"Observed ";}
				outFile<<"heterozygosity table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop_, (ncols * (block+1)))-1;
		outFile<<"pop. ";		
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			outFile.width(12);outFile<<pop+1;}
		outFile<<endl;

		for (int loc=0;loc<nloci;++loc) {//print the locus and locus sample size
			aString& locWord = locusName[loc];
			outFile<< locWord;outFile.width(11 + (5 - locWord.len()));
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				int samSize = GetLocusSampleSize(pop, loc) / 2;
				outFile<<"[";outFile.width(2);outFile<<samSize<<"]";
				if (pop < endPop){
					outFile.width(9);}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresent[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double hets=0.0;
					if (hetType == observed) {
							hets = GetObservedHeterozygosity(pop, loc, allele);}
						else{
							hets = GetExpectedHeterozygosity(pop, loc, allele);}
					aString hetStr(hets,0,4);
					hetStr.PrependCharToLen(' ',12);
					if (pop == endPop){//just write the freq if >0
							if (hets > 0.0){
									if (pop==x) {
										hetStr.PrependCharToLen(' ',17);}
								 	outFile<<hetStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if hets=0}
							if (hets == 0.0){
									aString blanks(' ',12);
									if (pop==x) {
										blanks.PrependCharToLen(' ',17);}
									outFile<<blanks;}
								else{
									if (pop==x) {
										hetStr.PrependCharToLen(' ',17);}
									outFile<<hetStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block
	}//end PopulationGeneticData::PrintHeterozygosity



void PopulationGeneticData::PrintLoci (ostream& outFile, int lociPerLine){//file assumed open
	int lineSpot = 0;
	for (int loc=0;loc<locusName.len();++loc){ 
		aString& locName = locusName[loc];
		outFile<<locName;
		++lineSpot;
		if ((loc == locusName.len()-1) || (lineSpot == lociPerLine)) {
				outFile<<endl;}
			else{
				outFile<<", ";}
		}//end for loc
	outFile<<endl;
	}//end PopulationGeneticData::PrintLoci




void PopulationGeneticData::PrintPopKey (ostream& outFile, char colSpacer){//file assumed open
	//Makes a key to the populations studied, for use in
	//interpreting some of the data output matrices.
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied
	char c = colSpacer;

	outFile<<title_<<endl<<endl;
	outFile<<"Key to populations"<<endl<<endl;
	outFile<<"Number";
	if(asciiFormat){outFile.width(14);}else{outFile<<c;}
	outFile<<"Name"<<endl;
	aString blanks = aString(' ',7);
	for (int pop=0;pop<npop_;++pop){
		aString& popTitle = popName[pop];
		if(asciiFormat){outFile.width(3);outFile<< pop+1<<blanks<<popTitle<<endl;}
			else{outFile<< pop+1<<c<<popTitle<<endl;}
		}
	outFile<<endl<<endl;
	if (ngroups > 1){
		outFile<<"Key to group membership for hierarchical analyses"<<endl<<endl;
		for (int group=0;group<ngroups;++group){
			aString& groupTitle = groupName[group];
			if(asciiFormat){outFile.width(2);outFile<<group+1<<".  "<<groupTitle<<endl;}
				else{outFile<<group+1<<"."<<c<<groupTitle<<endl;}
			aList<int>& groupList = groupHierarchy[group];
			for (int gpop=0;gpop<groupList.len();++gpop){
				int pop = groupList[gpop];
				aString& popTitle = popName[pop];
				if(asciiFormat){outFile<<blanks<<popTitle<<endl;}
					else{outFile<<c<<popTitle<<endl;}
				}//end for gpop
			outFile<<endl;
			}//end for group
		outFile<<endl<<endl;
		}//end if ngroups
	}//end PopulationGeneticData::PrintPopKey



void	 PopulationGeneticData::PrintSASFormatData(ostream& outFile){
	//prints a data set that can be read using SAS
	//missing data represented by '.', columns separated by spaces
	outFile<<"pop indiv gamete ";//this and next loop comprise the column titles
	for(int loc=0;loc<nloci;++loc){
		aString& locName=locusName[loc];
		StringList& alleleList=allelesPresent[loc];
		for(int allele=0;allele<alleleList.len();++allele){
			aString alleleTitle(locName);
			alleleTitle.cat('_');
			alleleTitle.cat(alleleList[allele]);
			alleleTitle.cat(' ');
			outFile<<alleleTitle;}
		}//end for loc
	outFile<<endl;
	for(int pop=0;pop<npop_;++pop){
		aString popStr(pop+1);
		popStr.cat(' ');
		for(long i=0;i<popSize[pop];++i){
			aString indivStr(popStr);
			indivStr.cat(i+1);
			indivStr.cat(' ');
			for (int gamete=1;gamete<=2;++gamete){
				aString genotypeStr(indivStr);
				genotypeStr.cat(gamete);
				genotypeStr.cat(' ');
				for(int loc=0;loc<nloci;++loc){
					StringList& alleleList=allelesPresent[loc];
					for(int allele=0;allele<alleleList.len();++allele){
						aString& alleleName = alleleList[allele];
						aString* gene =	GetGenotypeString(pop, i, loc, gamete);
						if(*gene==alleleName){
								genotypeStr.cat('1');}
							else if(*gene==' '){
								genotypeStr.cat('.');}
							else{
								genotypeStr.cat('0');}
						genotypeStr.cat(' ');
						delete gene; gene=0;
						}//end for allele
					}//end for loc
				outFile<<genotypeStr<<endl;
				}//end for gamete
			}//end for i
		}//end for pop
	outFile<<endl;		
	}//end  PopulationGeneticData::PrintSASFormatData(ostream&)
	
	
void	 PopulationGeneticData::PrintSASFormatData(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintSASFormatData(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end PopulationGeneticData::PrintSASFormatData(aString&)



void PopulationGeneticData::PrintVariabilityScores(aString& outFileName, char sepChar){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintVariabilityScores(dataFile, sepChar);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end PopulationGeneticData::PrintVariabilityScores(aString&, char)



void PopulationGeneticData::PrintVariabilityScores(ostream& outFile, char sepChar){
	//This calls for subroutines which calculate standard genetic variability measures:
	//>  "POLYLOCI" =  % polymorphic loci and maan number of alleles per locus
	//>  "HETEXPECTED" = heterozygosity expected from Hardy-Weinberg ratios,
	//         with a jackknife variance calculation
	// >  "HETOBSERVED" = heterozygosity obtained directly from genotype frequencies,
	//         with jackknife variance calculation
	// >  And a summary table of these values for each population.

	// The following is the summary table
	// The populations are printed out by number and name, with
	// only the first 20 characters of the name used.
    	char eolChar = endChr;
	if (outFile == cout){eolChar = '\n';}
	int charsInPrintedPopName = 10;

	// title_ and column headings
	outFile<<eolChar<<eolChar;
	outFile<<"Genetic variability scores for all populations."<<eolChar;
	outFile<<"(Standard errors in parentheses.)"<<endChr<<eolChar;
	aString headings = " # ";
	headings.separator(1, sepChar);                 					 //   5 characters
	headings.cat("population"); headings.separator(4, sepChar);            // 13 characters
	headings.cat("alleles/loc"); headings.separator(4, sepChar);           // 13 characters
	headings.cat("%P(all)"); headings.separator(7, sepChar);            	 // 13 characters
	headings.cat("%P(99%)"); headings.separator(7, sepChar);            // 13 characters
	headings.cat("%P(95%)"); headings.separator(8, sepChar);            // 13 characters
	headings.cat("H (obs)"); headings.separator(9, sepChar);            	 // 16 characters
	headings.cat("H (exp)");                                				 // 16 characters
	outFile<<headings<<eolChar;
	for(int pop=0;pop<npop_;++pop){
		// population number
		aString outLine(pop+1);
		outLine.PrependCharToLen(' ', MAX(2,Digits(npop_)));
    		outLine.separator(1, sepChar); 

       		aString popStr(popName[pop]);
        	popStr.trunc(charsInPrintedPopName);
		popStr.separator(charsInPrintedPopName - popStr.len() + 3, sepChar);

		//mean number of alleles per locus
		aString meanAllPerLoc(GetMeanAllelesPerLocus(pop),2,2);
		popStr.cat(meanAllPerLoc);
		popStr.cat(" (");
		double stErr = sqrt(GetVarJackAllelesPerLocus(pop));
		aString varAllPerLoc(stErr,1,2);
		popStr.cat(varAllPerLoc);
		popStr.cat(")");popStr.separator(1, sepChar); 

		// %P detectable
		double p = GetPercentPolymorphicLoci(pop, 100.0);
		aString percentP(p,3,1);
		popStr.cat(percentP); popStr.cat(" (");
		stErr = sqrt(GetVarJackPercentPolyLoci(pop, 100.0) );
		aString varPercentP(stErr,2,1);
		popStr.cat(varPercentP);
		popStr.cat(")");popStr.separator(2, sepChar); 

		// %P at 99%
		p = GetPercentPolymorphicLoci(pop, 99.0);
		aString percentP99(p,3,1);
		popStr.cat(percentP99); popStr.cat(" (");
		stErr = sqrt(GetVarJackPercentPolyLoci(pop, 99.0) );
		aString varPercentP99(stErr,2,1);
		popStr.cat(varPercentP99);
		popStr.cat(")");popStr.separator(3, sepChar); 

		// %P at 95%
		p = GetPercentPolymorphicLoci(pop, 95.0);
		aString percentP95(p,3,1);
		popStr.cat(percentP95); popStr.cat(" (");
		stErr = sqrt(GetVarJackPercentPolyLoci(pop, 95.0) );
		aString varPercentP95(stErr,2,1);
		popStr.cat(varPercentP95);
		popStr.cat(")");popStr.separator(3, sepChar); 

		// Hobs with standard error in parentheses
		aString meanHetObs(GetObservedHeterozygosity(pop),1,3);
		popStr.cat(meanHetObs);
		popStr.cat(" (");
		stErr = sqrt(GetVarJackObservedHeterozygosity(pop));
		aString varHetObs(stErr,1,3);
		popStr.cat(varHetObs);
		popStr.cat(")");popStr.separator(3, sepChar); 

		// Hexp with standard error in parentheses
		aString meanHetExp(GetExpectedHeterozygosity(pop),1,3);
		popStr.cat(meanHetExp);
		popStr.cat(" (");
		stErr = sqrt(GetVarJackExpectedHeterozygosity(pop));
		aString varHetExp(stErr,1,3);
		popStr.cat(varHetExp);
		popStr.cat(")");

		outLine.cat(popStr);
		outFile<<outLine<<eolChar;
		}// end for pop

	outFile<<eolChar<<"Variances computed using a jackknife algorithm, dropping loci."<<eolChar<<eolChar;
	}//end  PopulationGeneticData::PrintVariabilityScores



void PopulationGeneticData::ReadArlequinDataFile (aString* dataFileName){
	fstream dataFile;	
	char *dataLine=NEW char[10001];dataLine[0]='\0';
//	aString beast, popLine;
	aString endCharList(" \t,:\"{}[]!?"), eq("="), parameterLine;
	char* filename=dataFileName->raw();
	dataFile.open(filename,ios::in);

	StringList rawArlequinData0;//read in everything as strings, then sort and process them
	ObjList<StringList> arlequinDataBySection;//sections and subsections
	int nArlSections=6;
	enum arlSections {profile=0,haplotype=1,distmatrices=2,samples=3,genStructure=4,mantelTests=5};
	aList<int> arlSectionStartLines, arlSectionEndLines;
	for(int sec0=0;sec0<nArlSections;++sec0){//initialize
		arlequinDataBySection.Append(NEW StringList());
		arlSectionStartLines.Append(-1);//flag
		arlSectionEndLines.Append(-1);//flag
		}

	aString rawDataLine;
	int lineNum=-1;//stores current line of valid data
	while(!dataFile.eof()){
		dataFile.getline(dataLine, 10000);
		rawDataLine = dataLine;
		SnipArlequinComment(rawDataLine);//removes comments
		rawDataLine.CullLeadingWhiteSpace();
		rawDataLine.CullTrailingWhiteSpace();
		if(rawDataLine.len()>0){//it's not blank
			rawArlequinData0.Append(NEW aString(rawDataLine));
			++lineNum;
			if(rawDataLine[0]=='['){//assume it's a section heading line
				aString linecopy(rawDataLine);//work with a copy
				linecopy.CullLeadingChar('[');
				linecopy.CullTrailingChar(']');
				linecopy.CullLeadingWhiteSpace();//in case of sloppy text
				linecopy.CullTrailingWhiteSpace();
				if(linecopy=="Profile" || linecopy=="profile" || linecopy=="PROFILE"){
						arlSectionStartLines[profile]=lineNum;
						arlSectionEndLines[profile]=lineNum-1;//end of previous section
						}
					else if(linecopy=="HaplotypeDefinition" || 
							linecopy=="haplotypedefinition" || 
							linecopy=="HAPLOTYPEDEFINITION"){
						arlSectionStartLines[haplotype]=lineNum;
						arlSectionEndLines[haplotype]=lineNum-1;//end of previous section
						}
					else if(linecopy=="DistanceMatrix" || 
							linecopy=="distancematrix" || 
							linecopy=="DISTANCEMATRIX"){
						arlSectionStartLines[distmatrices]=lineNum;
						arlSectionEndLines[distmatrices]=lineNum-1;//end of previous section
						}
					else if(linecopy=="Samples" || 
							linecopy=="samples" || 
							linecopy=="SAMPLES"){
						arlSectionStartLines[samples]=lineNum;
						arlSectionEndLines[samples]=lineNum-1;//end of previous section
						}
					else if(linecopy=="Structure" || 
							linecopy=="structure" || 
							linecopy=="STRUCTURE"){
						arlSectionStartLines[genStructure]=lineNum;
						arlSectionEndLines[genStructure]=lineNum-1;//end of previous section
						}
					else if(linecopy=="Mantel" || 
							linecopy=="mantel" || 
							linecopy=="MANTEL"){
						arlSectionStartLines[mantelTests]=lineNum;
						arlSectionEndLines[mantelTests]=lineNum-1;//end of previous section
						}
					else{}
				}//if '['
			}//len>0
		}//end while !eof
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;

	aList<int> sortedStartLines(arlSectionStartLines);
	aList<int> tempStartLines,tempEndLines;
	for(int sec2=0;sec2<arlSectionEndLines.len();++sec2){
		sortedStartLines.Append(arlSectionEndLines[sec2]);
		tempStartLines.Append(0);tempEndLines.Append(0);
		}
	sortedStartLines.Append(rawArlequinData0.len()-1);
	sortedStartLines.Sort();
	for(int sec3=0;sec3<arlSectionStartLines.len();++sec3){
		int posStart = sortedStartLines.Loc(arlSectionStartLines[sec3]);
		int endSection = sortedStartLines[posStart+1];
		arlSectionEndLines[sec3] = endSection;
		}//end for sec3
	
	if(arlSectionStartLines.Last() == -1){
		arlSectionStartLines.Last() = sortedStartLines.Last();
		arlSectionEndLines.Last() = sortedStartLines.Last();}
	for(int sec4 = nArlSections-2;sec4>=0;--sec4){
		if(arlSectionStartLines[sec4] == -1){
			arlSectionStartLines[sec4] = arlSectionStartLines[sec4+1];
			arlSectionEndLines[sec4] = arlSectionStartLines[sec4];}
		}//sec4
//	arlSectionStartLines.Print(cout," ");
//	arlSectionEndLines.Print(cout," ");
	
	//finish putting raw data into sections
	for(int sec5 = 0;sec5<nArlSections;++sec5){
		int start = arlSectionStartLines[sec5], end = arlSectionEndLines[sec5];
		for(int i=start+1; i<=end;++i){
			aString& processedDataLine = rawArlequinData0[i];
			arlequinDataBySection[sec5].Append(NEW aString(processedDataLine));
			}//i
//		arlequinDataBySection[sec5].Print(cout);
		}//sec5
		
	//now process each section separately
	// "[Profile]"//title_
	aString *tword=0, pline;
	for(int p0=0;p0<arlequinDataBySection[profile].len();++p0){
		pline = arlequinDataBySection[profile][p0];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "title_" || *tword == "Title" || *tword == "TITLE" ){
			pline.CullLeadingChar('\"');
			pline.CullTrailingChar('\"');
			title_ = pline;
			arlequinDataBySection[profile].Delete(p0);//no longer needed
			delete tword;tword=0;
			break;
			}//if==title_
		delete tword;tword=0;
		}

	// "[Profile]"//NbSamples
	for(int p1=0;p1<arlequinDataBySection[profile].len();++p1){
		pline = arlequinDataBySection[profile][p1];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "NbSamples" || *tword == "nbsamples" || *tword == "NBSAMPLES"){
			pline.CullLeadingWhiteSpace();
			npop_ = pline.toInt();
			arlequinDataBySection[profile].Delete(p1);//no longer needed
			delete tword;tword=0;
			break;
			}//if==NbSamples
		delete tword;tword=0;
		}

	// "[Profile]"//GenotypicData
	for(int p2=0;p2<arlequinDataBySection[profile].len();++p2){
		pline = arlequinDataBySection[profile][p2];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "GenotypicData" || *tword == "genotypicdata" || *tword == "GENOTYPICDATA"){
			pline.CullLeadingWhiteSpace();
			int gtypedata = pline.toInt();
			if(gtypedata != 1){
				aString err("Can't read in this type of data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return;
				}
			arlequinDataBySection[profile].Delete(p2);//no longer needed
			delete tword;tword=0;
			break;
			}//if==GenotypicData
		delete tword;tword=0;
		}

	// "[Profile]"//GameticPhase
	for(int p3=0;p3<arlequinDataBySection[profile].len();++p3){
		pline = arlequinDataBySection[profile][p3];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "GameticPhase" || *tword == "gameticphase" || *tword == "GAMETICPHASE"){
			pline.CullLeadingWhiteSpace();
			int gphasedata = pline.toInt();
			if(gphasedata != 0){
				aString err("Can't read this type of data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return;
				}
			arlequinDataBySection[profile].Delete(p3);//no longer needed
			delete tword;tword=0;
			break;
			}//if==GameticPhase
		delete tword;tword=0;
		}

	// "[Profile]"//RecessiveData
	for(int p4=0;p4<arlequinDataBySection[profile].len();++p4){
		pline = arlequinDataBySection[profile][p4];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "RecessiveData" || *tword == "recessivedata" || *tword == "RECESSIVEDATA"){
			pline.CullLeadingWhiteSpace();
			int recdata = pline.toInt();
			if(recdata != 0){
				aString err("Can't read this type of data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return;
				}
			arlequinDataBySection[profile].Delete(p4);//no longer needed
			delete tword;tword=0;
			break;
			}//if==RecessiveData
		delete tword;tword=0;
		}

	// "[Profile]"//DataType
	for(int p5=0;p5<arlequinDataBySection[profile].len();++p5){
		pline = arlequinDataBySection[profile][p5];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "DataType" || *tword == "datatype" || *tword == "DATATYPE"){
			pline.CullLeadingWhiteSpace();
			if(pline != "STANDARD"){
				aString err("Can only read DataType=STANDARD data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return;
				}
			arlequinDataBySection[profile].Delete(p5);//no longer needed
			delete tword;tword=0;
			break;
			}//if==DataType
		delete tword;tword=0;
		}

	// "[Profile]"//MissingData
	aString missingDataFlag("?");//default
	for(int p6=0;p6<arlequinDataBySection[profile].len();++p6){
		pline = arlequinDataBySection[profile][p6];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "MissingData" || *tword == "missingdata" || *tword == "MISSINGDATA"){
			pline.CullLeadingWhiteSpace();
			pline.CullLeadingChar('\"');//can be single or double quotes
			pline.CullTrailingChar('\"');
			pline.CullLeadingChar('\'');
			pline.CullTrailingChar('\'');
			missingDataFlag = pline;
			if(missingDataFlag.len() != 1){
				aString err("Missing data must be denoted by a single character.  Default used.");
				aError(err);
				missingDataFlag="?";
				}
			arlequinDataBySection[profile].Delete(p6);//no longer needed
			delete tword;tword=0;
			break;
			}//if==IndividualLevel
		delete tword;tword=0;
		}

	// "[Profile]"//RecessiveAllele
	aString RecessiveAlleleFlag("?");//default
	for(int p7=0;p7<arlequinDataBySection[profile].len();++p7){
		pline = arlequinDataBySection[profile][p7];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "RecessiveAllele" || *tword == "recessiveallele" || *tword == "RECESSIVEALLELE"){
			pline.CullLeadingWhiteSpace();
			pline.CullLeadingChar('\"');//can be single or double quotes
			pline.CullTrailingChar('\"');
			pline.CullLeadingChar('\'');
			pline.CullTrailingChar('\'');
			RecessiveAlleleFlag = pline;
			if(RecessiveAlleleFlag.len() == 0){
				aString err("Recessive allele (homozygote) not identified.  Default used.");
				aError(err);
				RecessiveAlleleFlag="null";
				}
			arlequinDataBySection[profile].Delete(p7);//no longer needed
			delete tword;tword=0;
			break;
			}//if==IndividualLevel
		delete tword;tword=0;
		}

	// "[Profile]"//Frequency
	for(int p8=0;p8<arlequinDataBySection[profile].len();++p8){
		pline = arlequinDataBySection[profile][p8];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "Frequency" || *tword == "frequency" || *tword == "FREQUENCY"){
			pline.CullLeadingWhiteSpace();
			if(!(pline == "ABS" || pline == "abs")){
				aString err("Can only read Frequency=ABS data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return;
				}
			arlequinDataBySection[profile].Delete(p8);//no longer needed
			delete tword;tword=0;
			break;
			}//if==DataType
		delete tword;tword=0;
		}


	// "[Profile]"//LocusSeparator
	aString locSep(" \t");//default
	for(int p9=0;p9<arlequinDataBySection[profile].len();++p9){
		pline = arlequinDataBySection[profile][p9];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "LocusSeparator" || *tword == "locusseparator" || *tword == "LOCUSSEPARATOR"){
			pline.CullLeadingWhiteSpace();
			if(pline == "WHITESPACE" || pline == "whitespace"){
					locSep = " \t";}
				else if(pline == "TAB" || pline == "tab"){
					locSep = "\t";}
				else if(pline == "COMMA" || pline == "comma"){
					locSep = ",";}
				else if(pline == "NONE" || pline == "none"){
					aString err("Need locus separator to read data from the Arlequin file");
					aError(err);
					delete tword;tword=0;
					return;}
				else{
					locSep = pline;
					if(locSep.len()==0){
						aString err("LocusSeparator not specified correctly.  Default used.");
						aError(err);
						locSep = " \t";}
					}
			arlequinDataBySection[profile].Delete(p9);//no longer needed
			delete tword;tword=0;
			break;
			}//if==DataType
		delete tword;tword=0;
		}

	//[Profile] //CompDistMatrix=  ignored
	//[Profile] //FrequencyThreshold=  ignored
	//[Profile] //EpsilonValue=  ignored

	//section [Data]

	//[Data] [[HaplotypeDesignation]]  ignored
	//[Data] [[DistanceMatrix]]  ignored


	//[Data] [[Sample]] 
	//read in population data
	nloci=0;
	aString popLine;
	ObjList<ObjList<ObjList<StringList> > > rawArlequinData;//temp storage [pop][i][loc][a]
	ObjList<StringList> rawIDnames;//temp storage [pop][id]
	ObjList<aList<long> > popGenotypeFreqs;//temp storage [pop][id]
	long dline=0;//data line counter
	for (int pop=0;pop<npop_;++pop){
		rawArlequinData.Append(NEW ObjList<ObjList<StringList> >());//set up raw data 
		rawIDnames.Append(NEW StringList());//set up raw data 
		popGenotypeFreqs.Append(NEW aList<long>());//set up raw data 
		
		//get the title_ of this population
		popLine = arlequinDataBySection[samples][dline++];
		aString* junk = popLine.ChopOffFirstWord(eq);
		delete junk; junk=0;
		popLine.CullLeadingWhiteSpace();
		popLine.CullLeadingChar('\"');
		popLine.CullTrailingChar('\"');
		aString *s = NEW aString(popLine);
		popName.Append(s);//append a copy

		//read sample size to expect
		popLine = arlequinDataBySection[samples][dline++];
		junk = popLine.ChopOffFirstWord(eq);
		delete junk; junk=0;
		popLine.CullLeadingWhiteSpace();
		long indivs=popLine.toLong();
		long i=(-1);//a counter
		popLine.clear();
		popLine = arlequinDataBySection[samples][dline++];
		popLine.CullTrailingWhiteSpace();
		if(popLine[popLine.len()-1] != '{'){
			aString err("Expected \'SampleData={\' at ");err.cat(*s);//adds pop title_
			aError(err);}
		bool thisPopDone = false;
		while (!thisPopDone){//LOOK AT EACH INDIVIDUAL
			popLine.clear();
			popLine = arlequinDataBySection[samples][dline++];
			if(popLine[0] == '}'){
				thisPopDone = true;
				continue;}//goes to end of while !thisPopDone

			rawArlequinData[pop].Append(NEW ObjList<StringList>());//set up raw data
			i++; 
			
			//individuals come as 2 consecutive lines in diploid data
			//the first line contains an ID number and a frequency
			aString indivline1(popLine);
			aString *id=indivline1.ChopOffFirstWord();
			aString *fr=indivline1.ChopOffFirstWord();
			long gfreq = fr->toLong();
			delete fr; fr=0;
			rawIDnames[pop].Append(id);
			popGenotypeFreqs[pop].Append(gfreq);

			aString indivline2(arlequinDataBySection[samples][dline]);//sneak a peek ahead to check ploidy
			bool diploid = false;
			if(indivline2[0]=='}'){
					thisPopDone = true; break;}
				else if(indivline2.len() == indivline1.len()){//assume diploid based on symmetry in data file
					diploid=true;
					dline++;//increment the counter
					}
			long loc0=0;
			aString *gene=0;			
			while(indivline1.len()>0){
				rawArlequinData[pop][i].Append(NEW StringList());//new for each locus
				gene = indivline1.ChopOffFirstWord(locSep);
				rawArlequinData[pop][i][loc0].Append(gene);
				++loc0;
				}//end while
			nloci = MAX(loc0,nloci);
			if(diploid){
				long loc01=0;
				while(indivline2.len()>0){
					gene = indivline2.ChopOffFirstWord(locSep);
					rawArlequinData[pop][i][loc01].Append(gene);
					++loc01;
					}//end while
				nloci = MAX(loc01,nloci);
				}//if diploid
			}//end while !thisPopDone
		}//end for pop

	//make up some locus names
	ObjList<StringList> alleleNames;
	int maxloclen = Digits(nloci);
	for(int loc=0;loc<nloci;++loc){
		alleleNames.Append(NEW StringList());
		aString *lname = NEW aString("loc");
		aString lnum(loc+1);lnum.PrependCharToLen('0',maxloclen);
		lname->cat(lnum);
		rawLocusName.Append(lname);
		}//for loc
	locusName.Copy(rawLocusName);

	//recode data, using a list of alleles/locus
	for(int pop1=0;pop1<npop_;++pop1){
		for(int i=0;i<rawArlequinData[pop1].len();++i){
			for(int loc1=0;loc1<nloci;++loc1){
				for(int h=0;h<rawArlequinData[pop1][i][loc1].len();++h){
					aString gene(rawArlequinData[pop1][i][loc1][h]);//work with copy
					if(! (alleleNames[loc1].Includes(&gene))){
						alleleNames[loc1].Append(NEW aString(gene));}
					}//end h
				}//loc1
			}//i
		}//pop1
	for(int loc1=0;loc1<nloci;++loc1){//sort and clear missing data flags
		alleleNames[loc1].Sort();
		if(alleleNames[loc1].Includes(&missingDataFlag)){
			int site = alleleNames[loc1].Loc(&missingDataFlag);
			alleleNames[loc1].Delete(site);}
		}//loc1
			
	//finish recoding
	for(int pop2=0;pop2<npop_;++pop2){
		//make a new sublist for the genotypes of this population
		StringList *rawGList = NEW StringList();
		rawGtypes.Append(rawGList);				
		for(int i2=0;i2<rawArlequinData[pop2].len();++i2){
			aString gtyp(rawIDnames[pop2][i2]);//work with copy of id
			for(int loc2=0;loc2<nloci;++loc2){
				gtyp.cat(' ');
				for(int h2=0;h2<rawArlequinData[pop2][i2][loc2].len();++h2){
					aString gene2(rawArlequinData[pop2][i2][loc2][h2]);//work with copy
					if(gene2 == missingDataFlag){
							gtyp.cat(' ');}
						else{
							int a = alleleNames[loc2].Loc(&gene2);
							gtyp.cat(a + 'A');}//converts to alphabetic
					}//end h2
				}//loc2
			for(long n=0;n<popGenotypeFreqs[pop2][i2];++n){//use frequency info here
				rawGtypes[pop2].Append(NEW aString(gtyp));}
			}//i	
		popSize.Append(rawGtypes[pop2].len());
		}//pop2
		



	//[Data] [[Structure]] 
	//read in group information, if any
	bool readGroupInfo = false;
	if(arlequinDataBySection[genStructure].len() > 0){
		readGroupInfo=true;}

	if(! readGroupInfo){
			ngroups=1;
			GroupMembership();}
		else{//read group info explicitly
			for(int s0=0;s0<arlequinDataBySection[genStructure].len();++s0){
				parameterLine = arlequinDataBySection[genStructure][s0];
				aString *pword=parameterLine.ChopOffFirstWord(eq);
				pword->CullTrailingWhiteSpace();
				if(*pword == "StructureName" || *pword == "structurename" || *pword == "STRUCTURENAME"){
					//just ignore
					arlequinDataBySection[genStructure].Delete(s0);//no longer needed
					delete pword;pword=0;
					break;
					}
				delete pword;pword=0;
				}//s0

			// "[Structure]"//NbGroups
			for(int s1=0;s1<arlequinDataBySection[genStructure].len();++s1){
				parameterLine = arlequinDataBySection[genStructure][s1];
				tword=parameterLine.ChopOffFirstWord(eq);
				tword->CullTrailingWhiteSpace();
				if(*tword == "NbGroups" || *tword == "nbgroups" || *tword == "NBGROUPS"){
					parameterLine.CullLeadingWhiteSpace();
					ngroups = parameterLine.toInt();
					arlequinDataBySection[genStructure].Delete(s1);//no longer needed
					delete tword;tword=0;
					break;
					}//if==NbGroups
				delete tword;tword=0;
				}//s1

			// "[Structure]"//IndividualLevel
			for(int s2=0;s2<arlequinDataBySection[genStructure].len();++s2){
				parameterLine = arlequinDataBySection[genStructure][s2];
				tword=parameterLine.ChopOffFirstWord(eq);
				tword->CullTrailingWhiteSpace();
				if(*tword == "IndividualLevel" || *tword == "individuallevel" || *tword == "INDIVIDUALLEVEL"){
					parameterLine.CullLeadingWhiteSpace();
					int ilevel = parameterLine.toInt();
					if(ilevel != 1){
						aString err("In [[Structure]], must set IndividualLevel=1 in order to read Arlequin data");
						aError(err);
						delete tword; tword=0;
						return;
						}
					arlequinDataBySection[genStructure].Delete(s2);//no longer needed
					delete tword;tword=0;
					break;
					}//if==IndividualLevel
				delete tword;tword=0;
				}//s2

			groupName.Clear();
			groupHierarchy.Clear();
			long sline=0;
			for(int gr=0;gr<ngroups;++gr){
				aString gpnum(gr+1);
				gpnum.PrependCharToLen('0',Digits(ngroups));
				gpnum.prepend("group");
				groupName.Append(NEW aString(gpnum));
				groupHierarchy.Append(NEW aList<int>());
				//read group membership intro lines
				parameterLine = arlequinDataBySection[genStructure][sline++];
				if(parameterLine[parameterLine.len()-1] != '{'){
						aString err("Expected \"Group={\" in [[Structure]]");
						aError(err);
						continue;}
					else{//read names in group
						bool thisGpDone = false;
						while (!thisGpDone){//LOOK AT EACH title_
							aString gpLine = arlequinDataBySection[genStructure][sline++];
							if(gpLine[0] == '}'){
								thisGpDone = true;
								continue;}//goes to end of while !thisGpDone
			
							gpLine.CullLeadingChar('\"');
							gpLine.CullTrailingChar('\"');
							int popnum = popName.Loc(&gpLine);
							groupHierarchy[gr].Append(popnum);
							}//end while !thisGpDone
						}//end else
				}//for gr
		}//end if readGroupInfo
		
		
	LocalityMembership();
	CullSkippedLoci();
	CheckForNewAlleles();
	}//end PopulationGeneticData::ReadArlequinDataFile



void PopulationGeneticData::SnipArlequinComment(aString& arqStr){
	const char pound='#';
	int compos = arqStr.pos(pound);
	if(compos==0 && arqStr[0] != pound){//no comment found
		return;}
	if(arqStr.len()==1 && arqStr[0] == pound){
		arqStr.clear();return;
		}
	arqStr.trunc(compos-1);
	}//end PopulationGeneticData::SnipArlequinComment(aString&)





void PopulationGeneticData::ReadGelJockDataFile (aString* dataFileName){
	fstream dataFile;	
	char *dataLine=NEW char[1001];dataLine[0]='\0';
//	aString beast, popLine;
	aString endCharList(" \t,:\"{}[]!?");
	char* filename=dataFileName->raw();
	
//	char endChar = '\r';
	char endChar = endchr;//defined in StringClass.h
	dataFile.open(filename,ios::in);
	int dataFileOpen = dataFile.is_open();
	if(! dataFileOpen){
		aString errstr("couldn\'t open the file ");
		errstr.cat(filename);
		aError(errstr);}
//	dataFile.getline(dataLine, 1000);
	dataFile.getline(dataLine, 1000, endChar);
	title_ = dataLine;

	dataFile.getline(dataLine, 1000, endChar); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	npop_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 6);
	nloci = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 7);
	maxall = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	ngroups =  paramWord->toInt();
	delete paramWord; paramWord=0;

	if(endCharList.len()>11){//ALLELECHAR may be missing 
		paramWord = parameterLine.ChopOffFirstWord(endCharList);
		paramWord->cut(0, 11);
		alleleLen_ =  paramWord->toInt();
		delete paramWord; paramWord=0;
		}


	//get locus information
	int moreLoci = 1;//true
	int loc = 0;
	while (moreLoci){
		dataFile.getline(dataLine, 1000, endChar); 
		aString locusLine(dataLine);
		while (locusLine.len() > 0){
			++loc;
			aString *locName = locusLine.ChopOffFirstWord(endCharList);
			if(locName->len()>0){//there could be trailing blanks on locusLine
				aString *s = NEW aString(locName);
				rawLocusName.Append(s);}//append a copy
			delete locName; locName=0;
			}//end while locusLine.len()
		if (loc >= nloci){//data may be on more than one line
			moreLoci = 0;}//false
		}//end while moreLoci
	//read in the number of loci to be skipped in the analyses
	dataFile.getline(dataLine, 1000, endChar); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	lociSkipped = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped loci, if any
	loc = 0;
	moreLoci = 0;//false
	if (lociSkipped > 0){
			moreLoci = 1;}//true
	while (moreLoci){
		while (skipLine.len() > 0){
			++loc;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				aString *s = NEW aString(skipWord);
				skippedLocusName.Append(s);//append a copy
				int toSkip = rawLocusName.Loc(s);
				skippedLocusNum.Append(toSkip);}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (loc >= lociSkipped){
				moreLoci = 0;}//false
			else	{//more loci on the next line
				dataFile.getline(dataLine, 1000); 
				skipLine = dataLine;}
		}//end while moreLoci

	locusName.Copy(rawLocusName);
	for (loc=0;loc<skippedLocusNum.len();++loc){
		aString& toSkip = skippedLocusName[loc];
		int location = locusName.Loc(&toSkip);
		locusName.Delete(location);
		}//end for loc
	nloci -= lociSkipped;

	//read in population data
	aString popLine;
	aString beast, beastTest;
	for (int pop=0;pop<npop_;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 1000, endChar); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		aString *s = NEW aString(popLine);
		popName.Append(s);//append a copy

		//make a new sublist for the genotypes of this population
		StringList *rawGList = NEW StringList();
		rawGtypes.Append(rawGList);				

		// BEAST is the string for a particular individual
		beast.clear(); beastTest.clear();
		while (beastTest != "NEXT"){ 
			dataFile.getline(dataLine, 1000, endChar); 
			beastTest = beast = dataLine;
			beastTest.CullLeadingWhiteSpace();
			beastTest.CullTrailingWhiteSpace();
			if (beastTest != "NEXT"){ 
					aString *s = NEW aString(beast);
					rawGList->Append(s);
					s=0;}//append a copy
				else{
					popSize.Append(rawGList->len());}
			}//end while beast..else
		rawGList=NULL;
		}//end for pop
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	GroupMembership();
	LocalityMembership();
	CullSkippedLoci();
	CheckForNewAlleles();
	}//end PopulationGeneticData::ReadGelJockDataFile


void PopulationGeneticData::ReadGelJockmtDNADataFile (aString* dataFileName){
	fstream dataFile;	
	char *dataLine=NEW char[1001];dataLine[0]='\0';
//	aString beast, popLine;
	aString endCharList(" \t,:\"{}[]!?");
	char* filename=dataFileName->raw();
	dataFile.open(filename,ios::in);

	dataFile.getline(dataLine, 1000);
	title_ = dataLine;

	dataFile.getline(dataLine, 1000); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	npop_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 6);
	nloci = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 7);
	maxall = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	ngroups =  paramWord->toInt();
	delete paramWord; paramWord=0;

	if(endCharList.len()>11){//ALLELECHAR may be missing 
		paramWord = parameterLine.ChopOffFirstWord(endCharList);
		paramWord->cut(0, 11);
		alleleLen_ =  paramWord->toInt();
		delete paramWord; paramWord=0;
		}

	//get locus information
	int moreLoci = 1;//true
	int loc = 0;
	while (moreLoci){
		dataFile.getline(dataLine, 1000); 
		aString locusLine(dataLine);
		while (locusLine.len() > 0){
			++loc;
			aString *locName = locusLine.ChopOffFirstWord(endCharList);
			if(locName->len()>0){//there could be trailing blanks on locusLine
				aString *s = NEW aString(locName);
				rawLocusName.Append(s);}//append a copy
			delete locName; locName=0;
			}//end while locusLine.len()
		if (loc >= nloci){//data may be on more than one line
			moreLoci = 0;}//false
		}//end while moreLoci
	//read in the number of loci to be skipped in the analyses
	dataFile.getline(dataLine, 1000); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	lociSkipped = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped loci, if any
	loc = 0;
	moreLoci = 0;//false
	if (lociSkipped > 0){
			moreLoci = 1;}//true
	while (moreLoci){
		while (skipLine.len() > 0){
			++loc;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				aString *s = NEW aString(skipWord);
				skippedLocusName.Append(s);//append a copy
				int toSkip = rawLocusName.Loc(s);
				skippedLocusNum.Append(toSkip);}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (loc >= lociSkipped){
				moreLoci = 0;}//false
			else	{//more loci on the next line
				dataFile.getline(dataLine, 1000); 
				skipLine = dataLine;}
		}//end while moreLoci

	locusName.Copy(rawLocusName);
	for (loc=0;loc<skippedLocusNum.len();++loc){
		aString& toSkip = skippedLocusName[loc];
		int location = locusName.Loc(&toSkip);
		locusName.Delete(location);
		}//end for loc
	nloci -= lociSkipped;

	//read in population data
	aString popLine;
	aString beast, beastTest;
	for (int pop=0;pop<npop_;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 1000); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		aString *s = NEW aString(popLine);
		popName.Append(s);//append a copy

		//make a new sublist for the genotypes of this population
		StringList *rawGList = NEW StringList();
		rawGtypes.Append(rawGList);				

		// BEAST is the string for a particular individual
		beast.clear();beastTest.clear();
		while (beastTest != "NEXT"){ 
			dataFile.getline(dataLine, 1000); 
			beastTest = beast = dataLine;
			beastTest.CullLeadingWhiteSpace();
			beastTest.CullTrailingWhiteSpace();
			if (beastTest != "NEXT"){ 
					aString *s = NEW aString(beast);
					rawGList->Append(s);
					s=0;}//append a copy
				else{
					popSize.Append(rawGList->len());}
			}//end while beast..else
		rawGList=NULL;
		}//end for pop
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	GroupMembership();
	LocalityMembership();
	CullSkippedLoci();//assumes diploid markers
	CheckForNewHaplotypes();
	}//end PopulationGeneticData::ReadGelJockmtDNADataFile


void PopulationGeneticData::RestoreLoci (StringList& lociToRestore){
	//This function restores the designated loci to each genotype data string
	//and removes them from the skipped loci
	if(lociToRestore.len()==0){return;}//none to cull
	for(int rLoc=0;rLoc<lociToRestore.len();++rLoc){
		aString rLocusname = lociToRestore[rLoc];
		int item = rawLocusName.Loc(&rLocusname);
		if(item>=0){//the locus exists in the list
			//retore to original location
			int inserted = 0;
			for(int j=locusName.len()-1;j>=0;--j){
				//compare to locations of currently used loci
				aString& currLocusname = locusName[j];
				int currItemRaw =  rawLocusName.Loc(&currLocusname);
				if(item > currItemRaw){//insert here
//					if(j==locusName.len()-1){
//							locusName.Append(NEW aString(rLocusname));}
//						else{
							locusName.Insert(NEW aString(rLocusname), j+1);
//							}
					inserted = 1;
					break;
					}
				}//j		
			if(! inserted){//goes at location 0
				locusName.Prepend(NEW aString(rLocusname));}
							
			int item1 = skippedLocusNum.Loc(item);
			if(item1>=0){//it's in the skipped list
				skippedLocusNum.Delete(item1);
				int item2 = skippedLocusName.Loc(&rLocusname);
				skippedLocusName.Delete(item2);
				--lociSkipped;++nloci;
				}//if item exists in skipped list
			}//if item exists
		}//cLoc
//	for(int i=0;i<rawLocusName.len();++i){
//		cout<<rawLocusName[i]<<",";}
//	cout<<endl<<endl;
//	for(int i=0;i<locusName.len();++i){
//		cout<<locusName[i]<<",";}
//	cout<<endl;
	ResetFlags();
	CullSkippedLoci();
	CheckForNewAlleles();
	}//end procedure PopulationGeneticData::RestoreLoci



void PopulationGeneticData::RestoreLocus (aString& locusToRestore){
	StringList lociToRestore;
	lociToRestore.Append(NEW aString(locusToRestore));
	RestoreLoci(lociToRestore);
	}//end PopulationGeneticData::RestoreLocus (aString& )


void PopulationGeneticData::RestoreLocus (char* locusToRestore){
	StringList lociToRestore;
	lociToRestore.Append(NEW aString(locusToRestore));
	RestoreLoci(lociToRestore);
	}//end PopulationGeneticData::RestoreLocus (char* )


void PopulationGeneticData::UpdateVariableLoci(void){
	if (varLociCurrent_){return;}
	CheckForNewAlleles();
	variableLoci_.Clear();
	for(int loc=0;loc<nloci;++loc){
		StringList& locList = allelesPresent[loc];
		if (locList.len()>1){
			variableLoci_.Append(loc);}}
	variableLoci = variableLoci_.len();
	varLociCurrent_=1;
	}//end PopulationGeneticData::UpdateVariableLoci



void PopulationGeneticData::UpdateVariableHaploidLoci(void){
	if (varLociCurrent_){return;}
	CheckForNewHaplotypes();
	variableLoci_.Clear();
	for(int loc=0;loc<nloci;++loc){
		StringList& locList = allelesPresent[loc];
		if (locList.len()>1){
			variableLoci_.Append(loc);}}
	variableLoci = variableLoci_.len();
	varLociCurrent_=1;
	}//end PopulationGeneticData::UpdateVariableHaploidLoci



#endif







