//unit PopulationPhenotypicData.cpp


#ifndef PopulationPhenotypicDatacpp
#define PopulationPhenotypicDatacpp = 1

#include "PopulationPhenotypicData.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "aVector.cpp"





PopulationPhenotypicData::PopulationPhenotypicData(PopulationPhenotypicData& parentData, aList<int>& popSubsetNums):
			title(parentData.title), ngroups(0),npop_(popSubsetNums.len()), ntraits(parentData.ntraits), 
			rawTraitName(parentData.rawTraitName),
			skippedTraitNum(parentData.skippedTraitNum),
			skippedTraitName(parentData.skippedTraitName),
			traitName(parentData.traitName), traitsSkipped(parentData.traitsSkipped){
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	popSubsetNums.Sort();
	for(int p=0;p<popSubsetNums.len();++p){
		aString* pn = NEW aString(popName[p]);
		popName.Append(pn);
		popSize.Append(parentData.popSize[p]);
		StringList* rawPh = NEW StringList(parentData.rawTraits[p]);
		rawTraits.Append(rawPh);
		StringList* newphtype = NEW StringList(parentData.traitStrs[p]);
		traitStrs.Append(newphtype);
		ObjList<aList<long double> >* popTr = NEW ObjList<aList<long double> >(parentData.traits_[p]);
		traits_.Append(popTr);
		ObjList<aList<int> >* popTrSc = NEW ObjList<aList<int> >(parentData.traitsScored[p]);
		traitsScored.Append(popTrSc);
		ObjList<StatList<long double> >* popTrSt = NEW ObjList<StatList<long double> >(parentData.popTraitStats[p]);
		popTraitStats.Append(popTrSt);
		}//end for p
	GroupMembership();
	LocalityMembership();
	}//end ctor PopulationPhenotypicData(PopulationPhenotypicData&, aList<int>&)





void PopulationPhenotypicData::Clear(void){
	popSize.Clear();
	groupName.Clear();
	localityName.Clear();
	popName.Clear();
	traitName.Clear();
	rawTraitName.Clear();
	skippedTraitName.Clear();
	groupHierarchy.Clear();
	popsByLocality.Clear();
	skippedTraitNum.Clear();
	rawTraits.Clear();
	traitStrs.Clear();
	traits_.Clear();
	traitsScored.Clear();
	popTraitStats.Clear();
	title.clear();	//title of the data set from the first data line
	ngroups = npop_ = ntraits = 0;
	}//end PopulationPhenotypicData::Clear()



void PopulationPhenotypicData::CullTraits (StringList& traitsToCull){
	//This function culls the designated traits from each phenotype data string
	//and includes them among the skipped traits
	if(traitsToCull.len()==0){return;}//none to cull
	for(int tr=0;tr<traitsToCull.len();++tr){
		aString cTraitname = traitsToCull[tr];
		int item = rawTraitName.Loc(&cTraitname);
		if((item>=0)&&(!(skippedTraitName.Includes(&cTraitname)))){
				//the locus exists in the rawlist && isn't already skipped
			skippedTraitName.Append(NEW aString(traitsToCull[tr]));
			skippedTraitNum.Append(item);
			item = traitName.Loc(&cTraitname);
			traitName.Delete(item);
			traitsSkipped++;--ntraits;
			}//if item exists
		}//tr
	ResetFlags();
	CullSkippedTraits();
	}//end procedure PopulationPhenotypicData::CullTraits(StringList&)



void PopulationPhenotypicData::CullTrait (char* traitToCull){
	aString trToCull(traitToCull);
	CullTrait(trToCull);
	}//end PopulationPhenotypicData::CullTrait (char* )


void PopulationPhenotypicData::CullTrait (aString& traitToCull){
	StringList traitsToCull;
	traitsToCull.Append(NEW aString(traitToCull));
	CullTraits(traitsToCull);
	}//end PopulationPhenotypicData::CullTrait (aString& )





void PopulationPhenotypicData::CullSkippedTraits (void){	
	//This subprogram culls the skipped traits from each phenotype data string
	aString endCharList(" \t,;:\"(){}[]!?");
	traitStrs.Clear();
	aList<int> skippedTraitsCopy = skippedTraitNum;
	skippedTraitsCopy.Sort();
	for(int pop=0;pop<npop_;++pop){
		StringList& popRawPhtypes = rawTraits[pop];
		StringList *popPhtypes = NEW StringList();
		traitStrs.Append(popPhtypes);
		for(int i=0;i<popRawPhtypes.len();++i){
			aString& dataStr = popRawPhtypes[i];
			aString* dataStrCopy = NEW aString(dataStr);
			//first snip off the individual' s designation code
			aString *processedStr = dataStrCopy->ChopOffFirstWord();//start with codeWord
			for (int tr=0;tr<ntraits+traitsSkipped;++tr){
			//this assumes missing data coded with '.' or some other non-whitespace 
				aString *aTrait = dataStrCopy->ChopOffFirstWord(endCharList);
				if(!(skippedTraitsCopy.Includes(tr))){
					//copy this trait to the processed string
					processedStr->cat(' ');
					processedStr->cat(*aTrait);
					}
				delete aTrait; aTrait = NULL;
				}//	end for tr
			popPhtypes->Append(processedStr);
			delete dataStrCopy; dataStrCopy=0;
			}//end for i
		}//end for pop
	}//end procedure PopulationPhenotypicData::CullSkippedTraits



aString* PopulationPhenotypicData::GetPhenotypeString(int pop, int indiv){
	StringList& phenotypesList=traitStrs[pop];
	aString& dataStr = phenotypesList[indiv];
	aString *phtypeWord = NEW aString(dataStr);
	aString *codeWord = phtypeWord->ChopOffFirstWord();
	delete codeWord; codeWord=NULL;
	return phtypeWord;
	}//end PopulationPhenotypicData::GetPhenotypeString(pop,indiv)



aString* PopulationPhenotypicData::GetPhenotypeString(int pop, int indiv, int trait){
	aString *phtypeWord=GetPhenotypeString(pop, indiv);
	aString endCharList(" \t,;:\"(){}[]!?");
	for(int loc0=0;loc0<trait-1;++loc0){
		aString* locWord = phtypeWord->ChopOffFirstWord(endCharList);
		delete locWord; locWord=NULL;
		}
	aString *phTypeString = phtypeWord->ChopOffFirstWord(endCharList);
	delete phtypeWord;phtypeWord=NULL;//this was created in GetPhenotypeString(pop, indiv);
	return phTypeString;
	}//end PopulationPhenotypicData::GetPhenotypeString(pop,indiv,loc)


int PopulationPhenotypicData::GetLocality (int pop){
	for(int L=0;L<popsByLocality.len();++L){
		if (popsByLocality[L].Includes(pop)){
			return L;}}
	return (-1);//default if failure
	}//end  PopulationPhenotypicData::GetLocality


long PopulationPhenotypicData::GetTraitSampleSize (int pop, int trait){
	return popTraitStats[pop][trait].len();
	}//end  PopulationPhenotypicData::GetTraitSampleSize



void PopulationPhenotypicData::GroupMembership (void){
	//This routine determines group membership by the last full word in
	//the population title.  The first population determines the name of the 
	//first group, the next different population determines the name of the secondgroup, etc.

	//groupName saves the names of the groups, in the order encountered
	//groupHierarchy saves the population numbers from the
	//	original population series, so they can be retrieved from group membership data.
	//groupSize saves the number of populations in a group, indexed by group number
	groupName.Clear();
	if (ngroups<=1){//all populations belong to the same group
		aString *thisGName = NEW aString("pooled populations");
		groupName.Append(thisGName);
		aList<int> *groupList = NEW aList<int>();
		groupHierarchy.Append(groupList);
		}
	int groupNum=0;
	for (int pop=0;pop<npop_;++pop){
		if (ngroups>1){
			// there is more than one group		
			// first get the name of the group from the population name string
			aString& popString = popName[pop];
			aString popStringCopy(popString);
			aString *thisGName = popStringCopy.ChopOffLastWord();

			//does this group already exist?
			if(groupName.Includes(thisGName)){
					groupNum = groupName.Loc(thisGName);}
				else{//add it to the list & add slots to corresponding lists
					aString *nameToAdd = NEW aString(thisGName);
					groupNum = groupName.len();
					groupName.Append(nameToAdd);
					aList<int> *groupList = NEW aList<int>();
					groupHierarchy.Append(groupList);
					}
			delete thisGName; thisGName=0;
			}//end if ngroups>1
		aList<int>& groupList = groupHierarchy[groupNum];
		groupList.Append(pop);
		}//end for pop
	}//end  PopulationPhenotypicData::GroupMembership


int PopulationPhenotypicData::GroupMembership(int pop){
	if(groupHierarchy.len()==0){
		GroupMembership();}
	for(int g=0;g<groupHierarchy.len();++g){
		if (groupHierarchy[g].Includes(pop)){
			return g;}}
	cout<<"ERROR!  Population # "<<pop<<" not in any group.  group=-1 returned.";
	return -1;
	}//end  PopulationPhenotypicData::GroupMembership(int)


int PopulationPhenotypicData::individuals(int pop, int trait){
	int indivs=0;
	for(int i=0;i<individuals(pop);++i){
		aString* traitStr = GetTraitStr(pop,i,trait);
		if (*traitStr != "."){++indivs;}
		delete traitStr; traitStr = NULL;
		}
	return indivs;
	}//end  PopulationPhenotypicData::individuals(int,int)


aString* PopulationPhenotypicData::GetTraitStr(int pop, int indiv, int trait){
	aString endCharList(" \t,;:\"(){}[]!?");
	aString& indivStr = traitStrs[pop][indiv];
	aString indivStrCopy(indivStr);
	for(int tr=-1;tr<trait;++tr){
		aString* junk = indivStrCopy.ChopOffFirstWord(endCharList);
		delete junk; junk = NULL;}
	return indivStrCopy.ChopOffFirstWord(endCharList);
	}//end  PopulationPhenotypicData::GetTraitStr(int, int, int)



void PopulationPhenotypicData::LocalityMembership (void){
	//This routine determines locality membership by the first full word in
	//the population title.  The first population determines the name of the 
	//first locality, the next different population determines the name of the second locality, etc.

	//localityName saves the names of the localities, in the order encountered
	//popsByLocality saves the population numbers from the
	//	original population series, so they can be retrieved from locality membership data.
	aString endCharList(" \t,;:\"(){}[]!?");
	localityName.Clear();
	popsByLocality.Clear();
	int localityNum=0;
	for (int pop=0;pop<npop_;++pop){
		// there is more than one group		
		// first get the name of the group from the population name string
		aString& popString = popName[pop];
		aString popStringCopy(popString);
		aString *thisLName = popStringCopy.ChopOffFirstWord(endCharList);

		//does this locality already exist?
		if(localityName.Includes(thisLName)){
				localityNum = localityName.Loc(thisLName);}
			else{//add it to the list & add slots to corresponding lists
				aString *nameToAdd = NEW aString(thisLName);
				localityNum = localityName.len();
				localityName.Append(nameToAdd);
				aList<int> *localityList0 = NEW aList<int>();
				popsByLocality.Append(localityList0);
				}
		delete thisLName; thisLName=0;
		aList<int>& localityList1 = popsByLocality[localityNum];
		localityList1.Append(pop);
		}//end for pop
	}//end  PopulationPhenotypicData::LocalityMembership


int PopulationPhenotypicData::LocalityMembership(int pop){
	if(popsByLocality.len()==0){
		LocalityMembership();}
	for(int L=0;L<popsByLocality.len();++L){
		if (popsByLocality[L].Includes(pop)){
			return L;}}
	cout<<"ERROR!  Population # "<<pop<<" not in any locality.  Locality=-1 returned.";
	return -1;
	}//end  PopulationPhenotypicData::LocalityMembership(int)




void	 PopulationPhenotypicData::PrintPhenotypeFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out);
	PrintPhenotypeFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end PopulationPhenotypicData::PrintPhenotypeFile(aString&)



void PopulationPhenotypicData::PrintPhenotypeFile (ostream& outFile){
	aString dataLine = title;
	dataLine.cat(":  skipped loci omitted");
	outFile<<dataLine<<endl;
	dataLine = "NPOP=";
	dataLine.cat(npop_);
	dataLine.cat(", NTRAITS=");
	dataLine.cat(ntraits);
	dataLine.cat(", NGROUPS=");
	dataLine.cat(ngroups);
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int t=0;t<traitName.len();++t){
		dataLine.cat(traitName[t]);
		if( ( ((double)(t+1)) / 5.0) - ((t+1) / 5) == 0){//new line for every 5 traits
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (t+1 < traitName.len()){
				dataLine.cat(", ");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for t
	dataLine = "SKIP=0";
	outFile<<dataLine<<endl;
	dataLine.clear();
	for (int pop=0;pop<npop_;++pop){
		outFile<<popName[pop]<<endl;
		StringList& thePhenotypes = traitStrs[pop];
		for (int i=0;i<thePhenotypes.len();++i){
			dataLine = thePhenotypes[i];
			outFile<<dataLine<<endl;
			}//end for i
		outFile<<"NEXT"<<endl;
		}//end for pop
	outFile<<"END"<<endl;
	}//end PopulationPhenotypicData::PrintPhenotypeFile





void PopulationPhenotypicData::PrintTraits (ostream& outFile, int traitsPerLine){//file assumed open
	int lineSpot = 0;
	for (int t=0;t<traitName.len();++t){ 
		aString& trName = traitName[t];
		outFile<<trName;
		++lineSpot;
		if ((t == traitName.len()-1) || (lineSpot == traitsPerLine)) {
				outFile<<endl;}
			else{
				outFile<<", ";}
		}//end for t
	outFile<<endl;
	}//end PopulationPhenotypicData::PrintTraits




void PopulationPhenotypicData::PrintPopKey (ostream& outFile){//file assumed open
	//Makes a key to the populations studied, for use in
	//interpreting some of the data output matrices.
	outFile<<title<<endl<<endl;
	outFile<<"Key to populations"<<endl<<endl;
	outFile<<"Number";outFile.width(14);outFile<<"Name"<<endl;
	aString blanks = aString(' ',7);
	for (int pop=0;pop<npop_;++pop){
		aString& popTitle = popName[pop];
		outFile.width(3);outFile<< pop+1<<blanks<<popTitle<<endl;}
	outFile<<endl<<endl;
	if (ngroups > 1){
		outFile<<"Key to group membership \
				for hierarchical analyses"<<endl<<endl;
		for (int group=0;group<ngroups;++group){
			aString& groupTitle = groupName[group];
			outFile.width(2);outFile<<group+1<<".  "<<groupTitle<<endl;
			aList<int>& groupList = groupHierarchy[group];
			for (int gpop=0;gpop<groupList.len();++gpop){
				int pop = groupList[gpop];
				aString& popTitle = popName[pop];
				outFile<<blanks<<popTitle<<endl;
				}//end for gpop
			outFile<<endl;
			}//end for group
		outFile<<endl<<endl;
		}//end if ngroups
	}//end PopulationPhenotypicData::PrintPopKey



void	 PopulationPhenotypicData::PrintSASFormatData(ostream& outFile){
	//prints a data set that can be read using SAS
	//missing data represented by '.', columns separated by spaces
	outFile<<"pop\tindiv\t";//this and next loop comprise the column titles
	aString titleLine;
	for(int t=0;t<ntraits-1;++t){
		titleLine.cat(traitName[t]);
		titleLine.cat('\t');
		}//end for loc
	titleLine.cat(traitName[ntraits-1]);
	outFile<<titleLine<<endl;
	for(int pop=0;pop<npop_;++pop){
		aString popStr(pop+1);
		popStr.cat('\t');
		for(long i=0;i<popSize[pop];++i){
			aString indivStr(popStr);
			indivStr.cat(i+1);
			indivStr.cat('\t');
			int nthTrait=-1;
			for(int t=0;t<ntraits;++t){
				if(traitsScored[pop][i].Includes(t)){
						nthTrait++;
						indivStr.cat(traits_[pop][i][nthTrait]);
						indivStr.cat('\t');}
					else{//trait is missing in this indiv
						indivStr.cat(".\t");}					
				}//end for t
				outFile<<indivStr<<endl;
			}//end for i
		}//end for pop
	outFile<<endl;		
	}//end  PopulationPhenotypicData::PrintSASFormatData(ostream&)
	
	
void	 PopulationPhenotypicData::PrintSASFormatData(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintSASFormatData(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end PopulationPhenotypicData::PrintSASFormatData(aString&)



void PopulationPhenotypicData::PrintVariabilityScores(aString& outFileName, char sepChar){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out | ios::app /*,filebuf::openprot*/);
	PrintVariabilityScores(dataFile, sepChar);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end PopulationPhenotypicData::PrintVariabilityScores(aString&, char)



void PopulationPhenotypicData::PrintVariabilityScores(ostream& outFile, char sepChar){
	// The following is the summary table
	// The populations are printed out by number and name, with
	// only the first 20 characters of the name used.
    	char eolChar = '\r';
	if (outFile == cout){eolChar = '\n';}
	int charsInPrintedPopName = 20;

	// title and column headings
	outFile<<eolChar<<eolChar;
	outFile<<title<<eolChar<<eolChar;
	outFile<<"Phenotypic variability scores for all populations."<<eolChar;
	outFile<<"(Standard errors in parentheses.)"<<eolChar<<eolChar;
	aString headings = " # ";
	headings.separator(1, sepChar);                 					 //   5 characters
	headings.cat("population"); headings.separator(4, sepChar);            // 13 characters
	headings.cat("trait"); headings.separator(4, sepChar);            // 13 characters
	headings.cat("traitName"); headings.separator(4, sepChar);            // 13 characters
	headings.cat("n"); headings.separator(4, sepChar);           // 13 characters
	headings.cat("mean"); headings.separator(7, sepChar);            	 // 13 characters
	headings.cat("var"); headings.separator(7, sepChar);            // 13 characters
	headings.cat("std dev"); headings.separator(8, sepChar);            // 13 characters
	headings.cat("std err"); headings.separator(9, sepChar);            	 // 16 characters
	headings.cat("max"); headings.separator(9, sepChar);            	 // 16 characters
	headings.cat("min"); headings.separator(9, sepChar);            	 // 16 characters
	outFile<<headings<<eolChar;
	for(int pop=0;pop<npop_;++pop){
		// population number
		aString popLine(pop+1);
		popLine.PrependCharToLen(' ', MAX(2,Digits(npop_)));
    	popLine.separator(1, sepChar); 

       	aString popStr(popName[pop]);
//       	popStr.cat(popLine);
        popStr.trunc(charsInPrintedPopName);
		popStr.separator(charsInPrintedPopName - popStr.len() + 3, sepChar);

		for(int t=0;t<ntraits;++t){
			aString outLine(popStr);
			//trait & name
			aString trNumStr(t+1);
			trNumStr.PrependCharToLen(' ', MAX(2,Digits(ntraits)));
			trNumStr.cat(sepChar);
			outLine.cat(trNumStr);
			aString trStr(traitName[t]);
			trStr.separator(10 - trStr.len(), sepChar);
			outLine.cat(trStr);

			StatList<long double>& thePopTraitStats = popTraitStats[pop][t];
			outLine.cat(thePopTraitStats.n());
			outLine.separator(1,sepChar);
			
			if(thePopTraitStats.n()>0){
				outLine.cat(thePopTraitStats.mean());
				outLine.separator(1,sepChar);
			
				outLine.cat(thePopTraitStats.var());
				outLine.separator(1,sepChar);
			
				outLine.cat(thePopTraitStats.stdev());
				outLine.separator(1,sepChar);
			
				outLine.cat(thePopTraitStats.stdErrMean());
				outLine.separator(1,sepChar);
			
				outLine.cat(thePopTraitStats.max());
				outLine.separator(1,sepChar);
			
				outLine.cat(thePopTraitStats.min());
				outLine.separator(1,sepChar);
				}//end if n>0

			outFile<<outLine<<endl;
			}// end for t
		}// end for pop

	outFile<<endl<<endl;
	}//end  PopulationPhenotypicData::PrintVariabilityScores



void PopulationPhenotypicData::ReadDataFile (aString* dataFileName){
	fstream dataFile;	
	aString endCharList(" \t,;:");
	char *dataLine=NEW char[1001];dataLine[0]='\0';
//	aString beast, popLine;
	char* filename=dataFileName->raw();
	dataFile.open(filename);

	dataFile.getline(dataLine, 1000);
	title = dataLine;

	dataFile.getline(dataLine, 1000); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	npop_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	ntraits = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	ngroups =  paramWord->toInt();
	delete paramWord; paramWord=0;


	//get trait information
	int moreTraits = 1;//true
	int t = 0;
	while (moreTraits){
		dataFile.getline(dataLine, 1000); 
		aString traitLine(dataLine);
		while (traitLine.len() > 0){
			++t;
			aString *trName = traitLine.ChopOffFirstWord(endCharList);
			if(trName->len()>0){//there could be trailing blanks on locusLine
				aString *s = NEW aString(trName);
				rawTraitName.Append(s);}//append a copy
			delete trName; trName=0;
			}//end while TraitLine.len()
		if (t >= ntraits){//data may be on more than one line
			moreTraits = 0;}//false
		}//end while moreTraits
	//read in the number of traits to be skipped in the analyses
	dataFile.getline(dataLine, 1000); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	traitsSkipped = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped traits, if any
	t = 0;
	moreTraits = 0;//false
	if (traitsSkipped > 0){
			moreTraits = 1;}//true
	while (moreTraits){
		while (skipLine.len() > 0){
			++t;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				aString *s = NEW aString(skipWord);
				skippedTraitName.Append(s);//append a copy
				int toSkip = rawTraitName.Loc(s);
				skippedTraitNum.Append(toSkip);}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (t >= traitsSkipped){
				moreTraits = 0;}//false
			else	{//more traits on the next line
				dataFile.getline(dataLine, 1000); 
				skipLine = dataLine;}
		}//end while moreTraits

	traitName.Copy(rawTraitName);
	for (int t=0;t<skippedTraitNum.len();++t){
		aString& toSkip = skippedTraitName[t];
		int location = traitName.Loc(&toSkip);
		traitName.Delete(location);
		}//end for t
	ntraits -= traitsSkipped;

	//read in population data
	aString popLine;
	aString beast;
	for (int pop=0;pop<npop_;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 1000); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		aString *s = NEW aString(popLine);
		popName.Append(s);//append a copy

		//make a new sublist for the genotypes of this population
		StringList *rawTrList = NEW StringList();
		rawTraits.Append(rawTrList);				

		// BEAST is the string for a particular individual
		beast.clear();
		while (beast != "NEXT"){ 
			dataFile.getline(dataLine, 1000); 
			beast = dataLine;
			if (beast != "NEXT"){ 
					aString *s = NEW aString(beast);
					rawTrList->Append(s);
					s=0;}//append a copy
				else{
					popSize.Append(rawTrList->len());}
			}//end while beast..else
		rawTrList=NULL;
		}//end for pop
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	GroupMembership();
	LocalityMembership();
	CullSkippedTraits();
	ReadTraitStrings();
	}//end PopulationPhenotypicData::ReadDataFile


void PopulationPhenotypicData::ReadTraitStrings(void){
//			ObjList<ObjList<aList<double> > > traits;	
					//list of pops, each with list of individuals, each with list of trait values
//			ObjList<ObjList<aList<int> > > traitsScored;//to handle missing data
					//list of pops, each with list of individuals, each with list of traits scored
//			ObjList<ObjList<StatList<double> > > popTraitStats;	
					//list of pops, with list of traits, each with StatList of scores
	aString endCharList(" \t,;:\"(){}[]!?");
	traits_.Clear();
	traitsScored.Clear();
	popTraitStats.Clear();
	for(int pop=0;pop<npop_;++pop){//initialize
		traits_.Append(NEW ObjList<aList<long double> >());
		traitsScored.Append(NEW ObjList<aList<int> >());
		popTraitStats.Append(NEW ObjList<StatList<long double> >());
		for(int i=0;i<popSize[pop];++i){//initialize
			traits_[pop].Append(NEW aList<long double>());
			traitsScored[pop].Append(NEW aList<int>());
			}//end for i
		for(int tr=0;tr<traitName.len();++tr){
			popTraitStats[pop].Append(NEW StatList<long double>());
			}//end for tr
		for(int i=0;i<popSize[pop];++i){
			aString& iTraits = traitStrs[pop][i];
			aString iTrCopy(iTraits);//work with a copy
			aString* codeWord = iTrCopy.ChopOffFirstWord(endCharList);
			delete codeWord; codeWord=NULL;
			int tr= -1;
			while(iTrCopy.len()>0){
				tr++;
				aString* numStr = iTrCopy.ChopOffFirstWord(endCharList);
				if(*numStr != "."){//if not missing value
					traitsScored[pop][i].Append(tr);
//					double val = numStr->toDouble();
					long double val = numStr->toLongDouble();
					traits_[pop][i].Append(val);
					popTraitStats[pop][tr].Append(val);
					}
				delete numStr; numStr=NULL;
				}//end while iTrCopy.len()
			}//end for i
		}//end for pop
	
	}//end PopulationPhenotypicData::ReadTraitStrings
 	


void PopulationPhenotypicData::RestoreTraits (StringList& traitsToRestore){
	//This function restores the designated traits to each phenotype data string
	//and removes them from the skipped traits
	if(traitsToRestore.len()==0){return;}//none to cull
	for(int rTr=0;rTr<traitsToRestore.len();++rTr){
		aString rTraitname = traitsToRestore[rTr];
		int item = rawTraitName.Loc(&rTraitname);
		if(item>=0){//the trait exists in the list
			//retore to original location
			int inserted = 0;
			for(int j=traitName.len()-1;j>=0;--j){
				//compare to locations of currently used traits
				aString& currTraitname = traitName[j];
				int currItemRaw =  rawTraitName.Loc(&currTraitname);
				if(item > currItemRaw){//insert here
						traitName.Insert(NEW aString(rTraitname), j+1);
					inserted = 1;
					break;
					}
				}//j		
			if(! inserted){//goes at location 0
				traitName.Prepend(NEW aString(rTraitname));}
							
			int item1 = skippedTraitNum.Loc(item);
			if(item1>=0){//it's in the skipped list
				skippedTraitNum.Delete(item1);
				int item2 = skippedTraitName.Loc(&rTraitname);
				skippedTraitName.Delete(item2);
				--traitsSkipped;++ntraits;
				}//if item exists in skipped list
			}//if item exists
		}//rTr
	ResetFlags();
	CullSkippedTraits();
	}//end procedure PopulationPhenotypicData::RestoreTraits(StringList&)



void PopulationPhenotypicData::RestoreTrait (aString& traitToRestore){
	StringList traitsToRestore;
	traitsToRestore.Append(NEW aString(traitToRestore));
	RestoreTraits(traitsToRestore);
	}//end PopulationPhenotypicData::RestoreTrait (aString& )


void PopulationPhenotypicData::RestoreTrait (char* traitToRestore){
	StringList traitsToRestore;
	traitsToRestore.Append(NEW aString(traitToRestore));
	RestoreTraits(traitsToRestore);
	}//end PopulationPhenotypicData::RestoreTrait (char* )



aVector<long double>* PopulationPhenotypicData::traitVector(int pop, int indiv){
	//vector of scores for each trait
	aVector<long double>* zz = NEW aVector<long double>((long double)0.0,ntraits);
	aVector<long double>& z = *zz;
	for(int tr=0;tr<ntraits;++tr){
		z[tr] = traits_[pop][indiv][tr];}
	return zz;
	}//PopulationPhenotypicData::traitVector(int, int)




#endif



