//unit SexLinkedCodomMarkerData.cpp

#ifndef SexLinkedCodomMarkerDatacpp
#define SexLinkedCodomMarkerDatacpp = 1

#include "SexLinkedCodomMarkerData.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "aVector.cpp"
#include "aMatrix.cpp"



SexLinkedCodomMarkerData::SexLinkedCodomMarkerData(SexLinkedCodomMarkerData& parentData){	
		Initialize(parentData);
		}//end ctor SexLinkedCodomMarkerData(SexLinkedCodomMarkerData&)


SexLinkedCodomMarkerData::SexLinkedCodomMarkerData(SexLinkedCodomMarkerData& parentData, aList<int>& popSubsetNums):
			EvolutionaryGeneticsData(parentData,popSubsetNums), nlociSLCodom_(parentData.nlociSLCodom_), 
			 ploidy_(parentData.ploidy_), alleleLen_(parentData.alleleLen_), 
			allSLCodomLoci_(parentData.allSLCodomLoci_),locusNumSLCodom_(parentData.locusNumSLCodom_),
			skippedLocusNumSLCodom_(parentData.skippedLocusNumSLCodom_){
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	int p,pp;
	for(p=0;p<popSubsetNums.len();++p){
		pp = popSubsetNums[p];//population num in parentData
		rawSLCodomGtypes_.Append(parentData.rawSLCodomGtypes_[pp].Duplicate());
		heterogameticIndivs_.Append(parentData.heterogameticIndivs_[pp].Duplicate());
		homogameticIndivs_.Append(parentData.homogameticIndivs_[pp].Duplicate());
		}//end for p
	ResetFlags();
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();
	}//end ctor SexLinkedCodomMarkerData(SexLinkedCodomMarkerData&, aList<int>&)


void  SexLinkedCodomMarkerData::Initialize(void){
	SexLinkedCodomMarkerData::Clear();
	}//Initialize()
	
	
void  SexLinkedCodomMarkerData::Initialize(aString& s){
	SexLinkedCodomMarkerData::Clear();
	EvolutionaryGeneticsData::Initialize(s); 
	}//Initialize(aString&)
	
	
void  SexLinkedCodomMarkerData::Initialize(SexLinkedCodomMarkerData& cmd){
	SexLinkedData::Initialize(cmd); 
	nlociSLCodom_=cmd.nlociSLCodom_; 
	ploidy_=cmd.ploidy_; 
	alleleLen_=cmd.alleleLen_;
	allSLCodomLoci_=cmd.allSLCodomLoci_; 
	locusNumSLCodom_=cmd.locusNumSLCodom_;
	skippedLocusNumSLCodom_=cmd.skippedLocusNumSLCodom_;
	variableLocusNumSLCodom_=cmd.variableLocusNumSLCodom_;
	rawSLCodomGtypes_=cmd.rawSLCodomGtypes_;
	allelesPresentSLCodom_=cmd.allelesPresentSLCodom_;
	alleleFreqSLCodom_=cmd.alleleFreqSLCodom_;
	alleleFreqSLCodomGlobal_=cmd.alleleFreqSLCodomGlobal_;
	alleleFreqHeterogametic_=cmd.alleleFreqHeterogametic_;
	alleleFreqHomogametic_=cmd.alleleFreqHomogametic_;

	genotypeCountHomogametic_=cmd.genotypeCountHomogametic_;
	genotypeCountHomogameticGlobal_=cmd.genotypeCountHomogameticGlobal_;
	genotypeCountHeterogametic_=cmd.genotypeCountHeterogametic_;
	genotypeCountHeterogameticGlobal_=cmd.genotypeCountHeterogameticGlobal_;

	genotypeFreqHomogametic_=cmd.genotypeFreqHomogametic_;
	genotypeFreqHomogameticGlobal_=cmd.genotypeFreqHomogameticGlobal_;
	twoNSLCodom_=cmd.twoNSLCodom_;
	NheterogameticPopLoc_=cmd.NheterogameticPopLoc_;
	twoNhomogameticPopLoc_=cmd.twoNhomogameticPopLoc_;
	HetObsByAlleleSL_=cmd.HetObsByAlleleSL_;
	HetObsByLocusSL_=cmd.HetObsByLocusSL_;
	diseqSLCodomD_=cmd.diseqSLCodomD_;
	locusSLCodomD_=cmd.locusSLCodomD_;
	allelesCurrentSLCodom_=cmd.allelesCurrentSLCodom_;
	alleleFreqCurrentSLCodom_=cmd.alleleFreqCurrentSLCodom_;
	genotypeFreqCurrentSLCodom_=cmd.genotypeFreqCurrentSLCodom_;
	HetObsCurrentSL_=cmd.HetObsCurrentSL_; 
	varLociCurrentSLCodom_=cmd.varLociCurrentSLCodom_; 
	DCurrentSLCodom_=cmd.DCurrentSLCodom_; 
	}//Initialize(SexLinkedCodomMarkerData& )
	
	
	
void  SexLinkedCodomMarkerData::Initialize(SexLinkedCodomMarkerData& parentData, aList<int>& popSubsetNums){
	Clear();
	SexLinkedData::Initialize(parentData,popSubsetNums);
	nlociSLCodom_=parentData.nlociSLCodom_; 
	ploidy_=parentData.ploidy_; 
	alleleLen_=parentData.alleleLen_;
	heterogameticSex_=parentData.heterogameticSex_;
	sexTrait_=parentData.sexTrait_;
	primarySexRatio_=parentData.primarySexRatio_;
	allSLCodomLoci_=parentData.allSLCodomLoci_; 
	locusNumSLCodom_=parentData.locusNumSLCodom_;
	skippedLocusNumSLCodom_=parentData.skippedLocusNumSLCodom_;
	int p, pp;
	for(p=0;p<popSubsetNums.len();++p){
		pp = popSubsetNums[p];//population num in parentData
		rawSLCodomGtypes_.Append(parentData.rawSLCodomGtypes_[pp].Duplicate());
		}//end for p
	CheckForNewAlleles();
	UpdateVariableLoci();
	}//end Initialize(DominantMarkerData2&, aList<int>&)


	
	
	
SexLinkedCodomMarkerData&  SexLinkedCodomMarkerData::operator=(SexLinkedCodomMarkerData& cmd){
	SexLinkedData::operator=(cmd); 
	nlociSLCodom_=cmd.nlociSLCodom_; 
	ploidy_=cmd.ploidy_; 
	alleleLen_=cmd.alleleLen_;
	allSLCodomLoci_=cmd.allSLCodomLoci_; 
	locusNumSLCodom_=cmd.locusNumSLCodom_;
	skippedLocusNumSLCodom_=cmd.skippedLocusNumSLCodom_;
	variableLocusNumSLCodom_=cmd.variableLocusNumSLCodom_;
	rawSLCodomGtypes_=cmd.rawSLCodomGtypes_;
	allelesPresentSLCodom_=cmd.allelesPresentSLCodom_;
	alleleFreqSLCodom_=cmd.alleleFreqSLCodom_;
	alleleFreqSLCodomGlobal_=cmd.alleleFreqSLCodomGlobal_;
	alleleFreqHeterogametic_=cmd.alleleFreqHeterogametic_;
	alleleFreqHomogametic_=cmd.alleleFreqHomogametic_;
	genotypeCountHomogametic_=cmd.genotypeCountHomogametic_;
	genotypeCountHomogameticGlobal_=cmd.genotypeCountHomogameticGlobal_;
	genotypeCountHeterogametic_=cmd.genotypeCountHeterogametic_;
	genotypeCountHeterogameticGlobal_=cmd.genotypeCountHeterogameticGlobal_;
	genotypeFreqHomogametic_=cmd.genotypeFreqHomogametic_;
	genotypeFreqHomogameticGlobal_=cmd.genotypeFreqHomogameticGlobal_;
	twoNSLCodom_=cmd.twoNSLCodom_;
	NheterogameticPopLoc_=cmd.NheterogameticPopLoc_;
	twoNhomogameticPopLoc_=cmd.twoNhomogameticPopLoc_;
	HetObsByAlleleSL_=cmd.HetObsByAlleleSL_;
	HetObsByLocusSL_=cmd.HetObsByLocusSL_;
	diseqSLCodomD_=cmd.diseqSLCodomD_;
	locusSLCodomD_=cmd.locusSLCodomD_;
	allelesCurrentSLCodom_=cmd.allelesCurrentSLCodom_;
	alleleFreqCurrentSLCodom_=cmd.alleleFreqCurrentSLCodom_;
	genotypeFreqCurrentSLCodom_=cmd.genotypeFreqCurrentSLCodom_;
	HetObsCurrentSL_=cmd.HetObsCurrentSL_; 
	varLociCurrentSLCodom_=cmd.varLociCurrentSLCodom_; 
	DCurrentSLCodom_=cmd.DCurrentSLCodom_; 
	return *this;
	}//operator=(SexLinkedCodomMarkerData&)


/*
void SexLinkedCodomMarkerData::Calculate2LocusDisequilibrium (void){
	if(DCurrentSLCodom_) return;
	if(ploidy_ != 2){
		aString err("ERROR: SexLinkedCodomMarkerData::Calculate2LocusDisequilibrium(): can\'t calculate D when ploidy=");
		err+ploidy_;
		aError(err);
		return;}
	int doingHillsMethod = 0;//false
	//first make up an empty data array for locusSLCodomD_ and D_
	long double initValue = 0.0;
//	aString blank; blank.AppendCharToLen(' ',alleleLen_);

	diseqSLCodomD_.Clear();
	locusSLCodomD_.Clear();//locusSLCodomD_ contains locusByLocusDiseqInPop as subLists
	for (int pop=0;pop<npop_;++pop){
		//locusByLocusDiseqInPop contains locusDiseqLists as sublists
		ObjList<aList<long double> >* locusByLocusDiseqInPop = NEW ObjList<aList<long double> >();
		locusSLCodomD_.Append(locusByLocusDiseqInPop);
		for (int loc1=0;loc1<nlociSLCodom_ - 1;++loc1){
			//locusDiseqList contains D values
			aList<long double>* locusDiseqList = NEW aList<long double>();
			locusByLocusDiseqInPop->Append(locusDiseqList);
			for (int loc2=loc1 + 1;loc2<nlociSLCodom_;++loc2){
				locusDiseqList->Append(initValue);}
			}//end for loc1
		diseqSLCodomD_.Append(initValue);
		}//end for pop
	StatList<long double> Dlist;
	for (int pop=0;pop<npop_;++pop){
		for (int loc1=0;loc1<nlociSLCodom_ - 1;++loc1){
			if (IsPolymorphic(pop,loc1)){ 
				StringList& alleleNamesList1=allelesPresentSLCodom_[loc1];
				for (int loc2 = (loc1 + 1); loc2<nlociSLCodom_;++loc2){
					if (IsPolymorphic(pop,loc2)){ 
						StringList& alleleNamesList2=allelesPresentSLCodom_[loc2];
						int kComb = 0;  long double diseq=0.0;
						for (int allele1=0;allele1<alleleNamesList1.len();++allele1){
							//get freq of allele1 (at loc1) in pop
							long double freq1 = GetFreq(pop, loc1, allele1);
							if ((freq1>(long double)0.0)&&(freq1<(long double)1.0)){
								aString& alleleName1 = alleleNamesList1[allele1];
								for (int allele2=0;allele2<alleleNamesList2.len();++allele2){
									int nSample = 0;//set sample size to zero, to count for blanks
									//get freq of allele2 (at loc2) in pop
									long double freq2 = GetFreq(pop, loc2, allele2);
									if ((freq2>(long double)0.0)&&(freq2<(long double)1.0)){
									aString& alleleName2 = alleleNamesList2[allele2];
									//get genotype strings for each individual and  read joint frequencies
									long double gameteValue = 0.0;
									int bigN[3][3], bigX[2][2];
									for (int index1=0;index1<3;++index1){
										for (int index2=0;index2<3;++index2){
											bigN[index1][index2] = 0;}}
									for (int index1=0;index1<2;++index1){
										for (int index2=0;index2<2;++index2){
											bigX[index1][index2] = 0;}}
									for (int i=0;i<individuals(pop);++i){//each individual
										TextDataList& spot1 = Genotype(pop,i,loc1);
										//' make sure it doesn't have any blanks
										int blankAt1 = 0;//false
										if (spot1[0].missing() || spot1[1].missing()) {blankAt1 = 1;}//true
										if (!(blankAt1)){
											TextDataList& spot2 = Genotype(pop,i,loc2);
											//make sure it doesn' t have any blanks
											int blankAt2 = 0;//false
											if (spot2[0].missing() || spot2[1].missing()) {blankAt2 = 1;}//true
											if (!(blankAt2)){
												nSample++;
												//use this indiv in calculation
												//count instances of target alleles at each locus in this indiv
												long double count1 = 0;
												int index1 = 3;
												for(int chrom=0;chrom<2;++chrom){
													if (spot1[chrom] == alleleName1){
														count1 += 0.5;
														index1--;
													}}
												long double count2 = 0;
												int index2 = 3;
												for(int chrom=0;chrom<2;++chrom){
													if (spot2[chrom] == alleleName2){
														count2 += 0.5;
														index2--;
													}}
												//get a gametic value, which is the proportion
												//of gametes this individual will produce that
												//share target alleles at both loci.
												//Keep a sum of gametic values for the population
												gameteValue += count1 * count2;
												//this counts genotype freqs
												bigN[index1][index2]++;
												}//end if ! blankAt2
											}//end if ! blankAt1
										}//end for i
									//calculate frequency of joint genotype=gametic value/nsample}
									if (nSample == 0){
											gameteValue = (long double)-100;}
										else{
											gameteValue /= (long double)nSample;}
									//calculate Hill's (1974) intermediate variables & iterative method}
									long double HillsDiseq=0.0;
									if (doingHillsMethod){
										bigX[1][1] = 2 * bigN[1][1] + bigN[1][2] + bigN[2][1];
										bigX[2][1] = 2 * bigN[3][1] + bigN[2][1] + bigN[3][2];
										bigX[1][2] = 2 * bigN[1][3] + bigN[1][2] + bigN[2][3];
										bigX[2][2] = 2 * bigN[3][3] + bigN[2][3] + bigN[3][2];
										long double initialF11hat = (long double)((2 * bigX[1][1] + bigN[2][2]) / (2 * nSample)) - freq1 * freq2;
										long double diff = 10.0; long double f11hat=0.0;
										long iteration = 0;
										while (diff > 0.00001){
											//calculate in steps since it's long
											iteration++;
											f11hat = (long double)bigN[2][2] * initialF11hat * ((long double)1.0 - freq1 - freq2 + initialF11hat);
											f11hat /= (initialF11hat * ((long double)1.0 - freq1 - freq2 + initialF11hat) + (freq1 - initialF11hat) * (freq2 - initialF11hat));
											f11hat = ((long double)bigX[1][1] + f11hat) / (long double)(2 * nSample);
											diff = ABS(f11hat - initialF11hat);
											initialF11hat = f11hat;
											if (iteration > 40){
												initialF11hat *= 1.01;
												iteration = 0;}
											}//end while
										HillsDiseq = (long double)nSample * (f11hat - freq1 * freq2) / (long double)(nSample - 1);
										}//end if doingHillsMethod
									//calculate D
									diseq = gameteValue - freq1 * freq2;
									//was gameteValue -2* freq1 * freq2
									kComb++;
									ObjList<aList<long double> >& locusByLocusDiseqInPop = locusSLCodomD_[pop];
									aList<long double>& locusDiseqList = locusByLocusDiseqInPop[loc1];
									if (doingHillsMethod){
											diseq = HillsDiseq + locusDiseqList[loc2 - loc1];}
										else{
											diseq += locusDiseqList[loc2 - loc1];}
									locusDiseqList.Replace(diseq, loc2 - loc1);
									}//end if freq2>0 &&freq2<1
								}//end for allele2
							}//end if freq1>0 && freq1<1
						}//end for allele1
					ObjList<aList<long double> >& locusByLocusDiseqInPop = locusSLCodomD_[pop];
					aList<long double>& locusDiseqList = locusByLocusDiseqInPop[loc1];
					diseq += locusDiseqList.Nth(loc2 - loc1) / (long double)kComb;
					locusDiseqList.Replace(diseq, loc2 - loc1);
					Dlist.Append(diseq);
					}//end if IsPolymorphic(poploc2)
				}//end for loc2
			}//end if IsPolymorphic(pop,loc1)
		}//end for loc1
	if (diseqSLCodomD_.len() > 0){diseqSLCodomD_.Append(Dlist.mean());} else{diseqSLCodomD_.Append(0.0);}
	Dlist.Clear();
	}//end for pop
	DCurrentSLCodom_=true;
	}//end SexLinkedCodomMarkerData::Calculate2LocusDisequilibrium
*/





void SexLinkedCodomMarkerData::CalculateAlleleFreqs (void){
	//This routine counts allelic frequencies, taking into account
	//alleles left blank in the data set.
	if (alleleFreqCurrentSLCodom_){return;}
	SortBySex();
	CheckForNewAlleles();
	long pop, i, chrom, loc, allele;
	alleleFreqSLCodom_.Clear();//clear allele frequencies to start over
	twoNSLCodom_.Clear();//clear locus sample sizes to start over
	alleleFreqHeterogametic_.Clear();
	alleleFreqHomogametic_.Clear();
	NheterogameticPopLoc_.Clear();
	twoNhomogameticPopLoc_.Clear();
	long double freq;
	int indivIsHomogametic,tr;
	long allelesSampled = 0, allelesSampledHetero=0, allelesSampledHomo=0;	//the number of alleles, not individuals
	for (pop=0;pop< npop_;++pop){ 
		//set up a frequency list for this population
		alleleFreqSLCodom_.Append(NEW ObjList<aList<long double> >());
		alleleFreqHeterogametic_.Append(NEW ObjList<aList<long double> >());
		alleleFreqHomogametic_.Append(NEW ObjList<aList<long double> >());
		twoNSLCodom_.Append(NEW aList<int>());
		NheterogameticPopLoc_.Append(NEW aList<int>());
		twoNhomogameticPopLoc_.Append(NEW aList<int>());
		for (loc=0;loc<nlociSLCodom_;++loc){
			tr=locusNumSLCodom_[loc];
			//first set up a blank array of frequencies for each possible allele
			alleleFreqSLCodom_[pop].Append(NEW aList<long double>());
			alleleFreqHeterogametic_[pop].Append(NEW aList<long double>());
			alleleFreqHomogametic_[pop].Append(NEW aList<long double>());
			StringList& alleleNamesList = allelesPresentSLCodom_[loc];
			freq = zero;
			for (allele=0;allele<alleleNamesList.len();++allele){
				alleleFreqSLCodom_[pop][loc].Append(freq);
				alleleFreqHeterogametic_[pop][loc].Append(freq);
				alleleFreqHomogametic_[pop][loc].Append(freq);
				}

			//next look at the genotype of each individual and tally the frequency
			allelesSampled = allelesSampledHetero=allelesSampledHomo=0;	//the number of alleles, not individuals
			for (i=0;i< individuals(pop);++i){
				indivIsHomogametic = homogametic(pop,i);
				for (chrom=0; chrom<ploidy(pop,i);++chrom){
//					TextDatum& gene = GenotypicVal(pop,i,locusNumSLCodom_[loc],chrom);
					TextDatum& gene = SexLinkedCodomMarkerData::GenotypicVal(pop,i,tr,chrom);
					if (gene.OK()) {//increment the frequency & sample size -- divide later}
						allele = alleleNamesList.Loc(&gene);
						alleleFreqSLCodom_[pop][loc][allele]++;
						++allelesSampled;
						if(indivIsHomogametic){
								alleleFreqHomogametic_[pop][loc][allele]++;
								++allelesSampledHomo;
								}
							else{
								alleleFreqHeterogametic_[pop][loc][allele]++;
								++allelesSampledHetero;
								}
						}//end if not missing data
					}//end for chrom
				}//end for i

			//now go through and divide by the sample size to restore the frequencies
			if(allelesSampled>0){
				for (allele=0;allele<alleleNamesList.len();++allele){
					alleleFreqSLCodom_[pop][loc][allele] /= (long double)allelesSampled;
					}}
			if(allelesSampledHomo>0){
				for (allele=0;allele<alleleNamesList.len();++allele){
					alleleFreqHomogametic_[pop][loc][allele] /= (long double)allelesSampledHomo;
					}}
			if(allelesSampledHetero>0){
				for (allele=0;allele<alleleNamesList.len();++allele){
					alleleFreqHeterogametic_[pop][loc][allele] /= (long double)allelesSampledHetero;
					}}
			//store the sample size for this locus
			twoNSLCodom_[pop].Append(allelesSampled);
			NheterogameticPopLoc_[pop].Append(allelesSampledHetero);
			twoNhomogameticPopLoc_[pop].Append(allelesSampledHomo);
			}//end for loc
		}//end for pop
	alleleFreqSLCodomGlobal_.Clear();
	alleleFreqHeterogameticGlobal_.Clear();
	alleleFreqHomogameticGlobal_.Clear();
	for(loc=0;loc<nlociSLCodom_;++loc){
		alleleFreqSLCodomGlobal_.Append(NEW aList<long double>());
		alleleFreqHeterogameticGlobal_.Append(NEW aList<long double>());
		alleleFreqHomogameticGlobal_.Append(NEW aList<long double>());
		for (allele=0;allele<allelesPresentSLCodom_[loc].len();++allele){
			freq = zero;
			for(pop=0;pop<npop_;++pop){
				freq += alleleFreqSLCodom_[pop][loc][allele];}
			alleleFreqSLCodomGlobal_[loc].Append(freq/(long double)npop_);

			freq = zero;
			for(pop=0;pop<npop_;++pop){
				freq += alleleFreqHomogametic_[pop][loc][allele];}
			alleleFreqHomogameticGlobal_[loc].Append(freq/(long double)npop_);

			freq = zero;
			for(pop=0;pop<npop_;++pop){
				freq += alleleFreqHeterogametic_[pop][loc][allele];}
			alleleFreqHeterogameticGlobal_[loc].Append(freq/(long double)npop_);
			}//allele
		}//loc		
	alleleFreqCurrentSLCodom_=1;
	}//end SexLinkedCodomMarkerData::CalculateAlleleFreqs



ObjList<aList<long double> >* SexLinkedCodomMarkerData::CalculateAlleleFreqsByPopSet(aList<int>& popSet, aList<long double>& twoNList){
 	//returns lost of loci, each with list of allele freqs, using total allele set
 	CalculateAlleleFreqs();
 	twoNList.Clear();
 	ObjList<aList<long double> >* popSetFreqsPtr = NEW ObjList<aList<long double> >(alleleFreqSLCodom_[0]);//copies structure
 	ObjList<aList<long double> >& popSetFreqs = *popSetFreqsPtr;//notational convenience
 	int loc,a, alleles,pop,popSetPop;
 	long double samSizeSet,nSample,nCopies;
 	for(loc=0;loc<nlociSLCodom_;++loc){
 		alleles = GetNumAlleles(loc);
 		samSizeSet = 0;
 		for(a=0;a<alleles;++a){
 			nCopies = zero;
 			for(popSetPop=0;popSetPop<popSet.len();++popSetPop){
 				pop = popSet[popSetPop];
 				nSample = (long double)twoNSLCodom_[pop][loc];
 				if(a==0){samSizeSet += nSample;}
 				nCopies += nSample * GetFreq(pop,loc,a);
 				}
 			if(a==0){twoNList.Append(samSizeSet);}
 			if(nCopies==zero){
 					popSetFreqs[loc][a]=zero;}
 				else if(nCopies==samSizeSet){
 					popSetFreqs[loc][a]=one;}
 				else{
 					popSetFreqs[loc][a] = nCopies / samSizeSet;
 					}
 			}//end for a
 		}//end for loc
 	return popSetFreqsPtr;
	}//end SexLinkedCodomMarkerData::CalculateAlleleFreqsByPopSet



void SexLinkedCodomMarkerData::CalculateGenotypeFreqs (void){
	//This routine counts allelic frequencies, taking into account
	//alleles left blank in the data set.
	if (genotypeFreqCurrentSLCodom_){return;}
	SortBySex();
	CalculateAlleleFreqs();
	long pop, i, loc, allele1, allele2,tr,sampledHomo;
	long double freq;
	genotypeFreqHomogametic_.Clear();
	genotypeCountHomogametic_.Clear();
	genotypeCountHeterogametic_.Clear();
	for (pop=0;pop< npop_;++pop){ 
		//set up a frequency list for this population
		genotypeFreqHomogametic_.Append(NEW ObjList<ObjList<aList<long double> > >());
		genotypeCountHomogametic_.Append(NEW ObjList<ObjList<aList<int> > >());
		genotypeCountHeterogametic_.Append(NEW ObjList<aList<int> >());
		for (loc=0;loc<nlociSLCodom_;++loc){
			tr=locusNumSLCodom_[loc];
			//first set up a blank array of frequencies for each possible allele
			genotypeFreqHomogametic_[pop].Append(NEW ObjList<aList<long double> >());
			genotypeCountHomogametic_[pop].Append(NEW ObjList<aList<int> >());
			genotypeCountHeterogametic_[pop].Append(NEW aList<int>());
			StringList& alleleNamesList = allelesPresentSLCodom_[loc];
			freq =  zero;
			for (allele1=0;allele1<alleleNamesList.len();++allele1){
				genotypeFreqHomogametic_[pop][loc].Append(NEW aList<long double>());
				genotypeCountHomogametic_[pop][loc].Append(NEW aList<int>());
				genotypeCountHeterogametic_[pop][loc].Append(0);
				for (allele2=0;allele2<alleleNamesList.len();++allele2){
					genotypeFreqHomogametic_[pop][loc][allele1].Append(zero);
					genotypeCountHomogametic_[pop][loc][allele1].Append(0);
					}}

			//next look at the genotype of each individual and tally the frequency
			sampledHomo=0;	//the number of individuals
			for (i=0;i< individuals(pop);++i){
				if(homogametic(pop,i)){
//						TextDatum& gene1 = SexLinkedCodomMarkerData::GenotypicVal(pop,i,loc,0);
//						TextDatum& gene2 = SexLinkedCodomMarkerData::GenotypicVal(pop,i,loc,1);
						TextDatum& gene1 = SexLinkedCodomMarkerData::GenotypicVal(pop,i,tr,0);
						TextDatum& gene2 = SexLinkedCodomMarkerData::GenotypicVal(pop,i,tr,1);
						if (gene1.OK() && gene2.OK()) {//increment the frequency & sample size -- divide later
							allele1 = alleleNamesList.Loc(&gene1);
							allele2 = alleleNamesList.Loc(&gene2);
							if(allele1>allele2){SWAP(allele1,allele2);}
							genotypeCountHomogametic_[pop][loc][allele1][allele2]++;
							genotypeFreqHomogametic_[pop][loc][allele1][allele2]++;
							++sampledHomo;
							}//end if not missing data
						}//end homogametic
					else{
//						TextDatum& gene = SexLinkedCodomMarkerData::GenotypicVal(pop,i,loc,0);
						TextDatum& gene = SexLinkedCodomMarkerData::GenotypicVal(pop,i,tr,0);
						if (gene.OK()) {
							allele1 = alleleNamesList.Loc(&gene);
							genotypeCountHeterogametic_[pop][loc][allele1]++;
							}//OK
						}//heterogametic
				}//end for i

			//now go through and divide by the sample size to restore the frequencies
			if(sampledHomo>0){
				for (allele1=0;allele1<alleleNamesList.len();++allele1){
					for (allele2=0;allele2<alleleNamesList.len();++allele2){
						genotypeFreqHomogametic_[pop][loc][allele1][allele2] /= (long double)sampledHomo;
						}}}
			}//end for loc
		}//end for pop
	genotypeFreqHomogameticGlobal_.Clear();
	genotypeCountHomogameticGlobal_.Clear();
	genotypeCountHeterogameticGlobal_.Clear();
	long summHom, summHet;
	for(loc=0;loc<nlociSLCodom_;++loc){
		genotypeCountHomogameticGlobal_.Append(NEW ObjList<aList<int> >());
		genotypeFreqHomogameticGlobal_.Append(NEW ObjList<aList<long double> >());
		genotypeCountHeterogameticGlobal_.Append(NEW aList<int>());
		for(allele1=0;allele1<allelesPresentSLCodom_[loc].len();++allele1){
			genotypeCountHomogameticGlobal_[loc].Append(NEW aList<int>());
			genotypeFreqHomogameticGlobal_[loc].Append(NEW aList<long double>());
			for(allele2=0;allele2<allelesPresentSLCodom_[loc].len();++allele2){
				freq = zero;
				summHom = 0;
				for(pop=0;pop<npop_;++pop){
					summHom += genotypeCountHomogametic_[pop][loc][allele1][allele2];
					freq += genotypeFreqHomogametic_[pop][loc][allele1][allele2];
					}
				genotypeCountHomogameticGlobal_[loc][allele1].Append(summHom);
				genotypeFreqHomogameticGlobal_[loc][allele1].Append(freq/(long double)npop_);
				}//allele2
			summHet=0;
			for(pop=0;pop<npop_;++pop){
				summHet += genotypeCountHeterogametic_[pop][loc][allele1];}
			genotypeCountHeterogameticGlobal_[loc].Append(summHet);
			}}
	genotypeFreqCurrentSLCodom_=1;
	}//end SexLinkedCodomMarkerData::CalculateAlleleFreqs



void SexLinkedCodomMarkerData::CalculateObservedHeterozygosity (void){
	//only calculated for homogametic sex
	//Counts observed heterozygosity values for each population, locus & allele
	//directly from the genotype data strings for each individual.  
//	long i, codeLen, chrom, loc, allele, posGtypeWord, nSample;
	if(ploidy_ != 2){
		aString err("ERROR: SexLinkedCodomMarkerData::CalculateObservedHeterozygosity(): can\'t calculate when ploidy=");
		err+ploidy_;
		aError(err);
		return;}
	long i, loc, allele,tr,pop;
	long double hetsByAllele, hetsByLoc,nSample;
	//clear observed hets to start over
	HetObsByAlleleSL_.Clear();
	HetObsByLocusSL_.Clear();
	ObjList<aList<int> > homogameticSampleSize,heterogameticSampleSize;//[pop][loc]
	for (pop=0;pop<npop_;++pop){
		homogameticSampleSize.Append(NEW aList<int>());
		//set up a frequency list for this population
		ObjList<aList<long double> > *popHetObsAll = NEW ObjList<aList<long double> >();
		HetObsByAlleleSL_.Append(popHetObsAll);
		aList<long double> *popHetObsLoc = NEW aList<long double>();
		HetObsByLocusSL_.Append(popHetObsLoc);
		for (loc=0;loc<nlociSLCodom_;++loc){
			tr=locusNumSLCodom_[loc];
			homogameticSampleSize[pop].Append(0);
			//first set up a blank array of frequencies for each possible allele
			aList<long double> *locHetObs = NEW aList<long double>();
			popHetObsAll->Append(locHetObs);
			StringList& alleleNamesList = allelesPresentSLCodom_[loc];
			hetsByAllele = hetsByLoc = 0.0;
			popHetObsLoc->Append(hetsByLoc);
			for (allele=0;allele<alleleNamesList.len();++allele){
				locHetObs->Append(hetsByAllele);}
			//next look at the genotype of each individual and tally the frequency
			for (i=0;i<individuals(pop);++i){
				if(heterogametic(pop,i)) continue;//only calculated for homogametic sex
//				TextDatum& gene1 = SexLinkedCodomMarkerData::GenotypicVal(pop,i,loc,0);		
//				TextDatum& gene2 = SexLinkedCodomMarkerData::GenotypicVal(pop,i,loc,1);
				TextDatum& gene1 = SexLinkedCodomMarkerData::GenotypicVal(pop,i,tr,0);		
				TextDatum& gene2 = SexLinkedCodomMarkerData::GenotypicVal(pop,i,tr,1);
				if(gene1.OK() && gene2.OK()){//it's scorable
					homogameticSampleSize[pop][loc]++;
					if (gene1 != gene2) {// it's a heterozygote
						//increment the het frequencies of both alleles -- divide later
						++hetsByLoc;
						allele = alleleNamesList.Loc(&gene1);
						hetsByAllele = locHetObs->Nth(allele) + 1.0;
						locHetObs->Replace(hetsByAllele, allele);
						allele = alleleNamesList.Loc(&gene2);
						hetsByAllele = locHetObs->Nth(allele) + 1.0;
						locHetObs->Replace(hetsByAllele, allele);
						}//end if match & no blanks
					}//scorable
				}//end for i
			//now go through and divide by the sample size to restore the frequencies
			nSample = homogameticSampleSize[pop][loc];
			if (nSample > 0){
//				popHetObsLoc->Replace(hetsByLoc /((long double)nSample / (long double)2.0), loc);
				popHetObsLoc->Replace(hetsByLoc /((long double)nSample), loc);
				for (allele=0;allele<alleleNamesList.len();++allele){
//					hetsByAllele = locHetObs->Nth(allele) / ((long double)nSample / (long double)2.0);
					hetsByAllele = locHetObs->Nth(allele) / ((long double)nSample);
					locHetObs->Replace(hetsByAllele, allele);
					}//end for allele
				}//end if nSample
			}//end for loc
		}//end for pop
	HetObsCurrentSL_=1;
	}//end SexLinkedCodomMarkerData::CalculateObservedHeterozygosity



void SexLinkedCodomMarkerData::CalculateTwoN (void){
	//makes table of sample sizes for each pop, locus
	twoNSLCodom_.Clear();
	NheterogameticPopLoc_.Clear();
	twoNhomogameticPopLoc_.Clear();
	int pop,loc,i,n;
	for(pop=0;pop<npop_;++pop){
		twoNSLCodom_.Append(NEW aList<int>());
		NheterogameticPopLoc_.Append(NEW aList<int>());
		twoNhomogameticPopLoc_.Append(NEW aList<int>());
		for(loc=0;loc<nlociSLCodom_;++loc){
			twoNSLCodom_[pop].Append(0);
			for(i=0;i<popSize_[pop];++i){
				n = GetLocusSampleSize(pop, i, loc);
				twoNSLCodom_[pop][loc] += n;
				if(heterogametic(pop,i)){
						NheterogameticPopLoc_[pop][loc] += n;}
					else{
						twoNhomogameticPopLoc_[pop][loc] += n;}
				}//end for i
			}//end for loc
		}//end for pop
	}//end SexLinkedCodomMarkerData::CalculateTwoN



void SexLinkedCodomMarkerData::CheckForNewAlleles (void){
	if (allelesCurrentSLCodom_){return;}
	allelesPresentSLCodom_.Clear();
	int pop,i,loc,chrom,tr;
	for (loc=0;loc<nlociSLCodom_;++loc){
		allelesPresentSLCodom_.Append(NEW StringList());}
	for (pop=0;pop<npop_;++pop){
		for(i=0;i<individuals(pop);++i){
			for (loc=0;loc<nlociSLCodom_;++loc){
				tr=locusNumSLCodom_[loc];
				StringList& alleleList = allelesPresentSLCodom_[loc];
				for (chrom=0;chrom<SexLinkedCodomMarkerData::Ploidy(pop,i,loc);++chrom){
//					TextDatum& gene = SexLinkedCodomMarkerData::GenotypicVal(pop,i,loc,chrom);
					TextDatum& gene = SexLinkedCodomMarkerData::GenotypicVal(pop,i,tr,chrom);
					if (gene.OK() &&!(alleleList.Includes(&gene))) {
						aString *newGene = NEW aString(gene);
						alleleList.Append(newGene);}
					}//end for chrom
				}//end for loc
			}//end for i
		}//end for pop
	for (loc=0;loc<nlociSLCodom_;++loc){
		allelesPresentSLCodom_[loc].Sort();}
	allelesCurrentSLCodom_=1;
	SexLinkedCodomMarkerData::UpdateVariableLoci();
	}//end SexLinkedCodomMarkerData::CheckForNewAlleles



void SexLinkedCodomMarkerData::Clear(void){
	EvolutionaryGeneticsData::Clear();
	allSLCodomLoci_.Clear();
	locusNumSLCodom_.Clear();
	skippedLocusNumSLCodom_.Clear();
	variableLocusNumSLCodom_.Clear();
	rawSLCodomGtypes_.Clear();
	heterogameticIndivs_.Clear();
	homogameticIndivs_.Clear();
	allelesPresentSLCodom_.Clear();
	alleleFreqSLCodom_.Clear();
	alleleFreqHeterogametic_.Clear();
	alleleFreqHomogametic_.Clear();

	genotypeCountHomogametic_.Clear();
	genotypeCountHomogameticGlobal_.Clear();
	genotypeCountHeterogametic_.Clear();
	genotypeCountHeterogameticGlobal_.Clear();
	genotypeFreqHomogametic_.Clear();
	genotypeFreqHomogameticGlobal_.Clear();
	
	twoNSLCodom_.Clear();
	NheterogameticPopLoc_.Clear();
	twoNhomogameticPopLoc_.Clear();
	HetObsByAlleleSL_.Clear();
	HetObsByLocusSL_.Clear();
	diseqSLCodomD_.Clear();
	locusSLCodomD_.Clear();
	ResetFlags();
	nlociSLCodom_= 0;
	alleleLen_=1;
	ploidy_=2;
	sexTrait_=-1;
	primarySexRatio_=half;
	heterogameticSex_=M;
	}//end SexLinkedCodomMarkerData::Clear()




void  SexLinkedCodomMarkerData::CreateBootstrapData(SexLinkedCodomMarkerData& origData, Ran3& ran){
	//individuals resampled
//	EvolutionaryGeneticsData::CreateBootstrapData(origData,ran);//faster to do it all here
	Clear();
	title_ = origData.title_;	//title_ of the data set from the first data line
	title_.cat(" bootstrap replicate");
	ngroups_ = origData.ngroups_;
	npop_ = origData.npop_;
	ntraits_ = origData.ntraits_;
	popName_ = origData.popName_;
	groupName_ = origData.groupName_;
	localityName_ = origData.localityName_;
	traitName_ = origData.traitName_;
	traitNum_ = origData.traitNum_;
	skippedTraitNum_ = origData.skippedTraitNum_;
	groupHierarchy_ = origData.groupHierarchy_;
	popsByLocality_ = origData.popsByLocality_;
	localityCoordinates_ = origData.localityCoordinates_;
	missing_ = origData.missing_;

	nlociSLCodom_ = origData.nlociSLCodom_;
	ploidy_ = origData.ploidy_;
	alleleLen_ = origData.alleleLen_;
	heterogameticSex_=origData.heterogameticSex_;
	sexTrait_=origData.sexTrait_;
	primarySexRatio_=origData.primarySexRatio_;
	allSLCodomLoci_ = origData.allSLCodomLoci_;
	locusNumSLCodom_ = origData.locusNumSLCodom_;
	skippedLocusNumSLCodom_ = origData.skippedLocusNumSLCodom_;
	variableLocusNumSLCodom_ = origData.variableLocusNumSLCodom_;	
	
	int pop,newpop,newpopsize,i, ranIndivID;
	for(pop=0;pop<npop_;++pop){//randomly resamples within populations
		rawTraits_.Append(NEW ObjList<TextDataList>());
		rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
		newpop = ran.longBetween(0,npop_-1);
		newpopsize = origData.rawTraits_[newpop].len();
		popSize_.Append(newpopsize);
		for (i=0;i<newpopsize;++i){
			ranIndivID = ran.longBetween(0,newpopsize-1);
			rawTraits_[pop].Append((TextDataList*)origData.rawTraits_[newpop][ranIndivID].Duplicate());
			rawSLCodomGtypes_[pop].Append(origData.rawSLCodomGtypes_[newpop][ranIndivID].Duplicate());
		}	}
	SortBySex();
	CheckForNewAlleles();	
	}//end  SexLinkedCodomMarkerData::CreateBootstrapData(SexLinkedCodomMarkerData&)




void SexLinkedCodomMarkerData::CreatePerfectData(const int numPopsSampled,
							const int numIndivsPerPop,//choose a multiple of 4
							const int numLociSampled, Ran3& ran){
	//	first set up initial variables
	Clear();
	title_ = "Data set with Fst=0, Fis=0, allele frequency p=0.5, sex ratio 1:1 for all loci";
	npop_ = numPopsSampled;
	nlociSLCodom_ = numLociSampled;
	ngroups_ = 1;
	int traitcount=0;
	traitName_.Append(NEW aString("ID"));
	traitNum_.Append(traitcount++);
	traitName_.Append(NEW aString("sex"));
	traitNum_.Append(traitcount++);
	sexTrait_=1;
	int pop,loc,loc0,i, item;
	for(loc = 0;loc<nlociSLCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociSLCodom_);
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(traitcount++);
		allSLCodomLoci_.Append(loc+2);
		locusNumSLCodom_.Append(loc+2);
		}
	ntraits_ =traitNum_.len();

	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
	StringList sexList, genotypeListM, genotypeListF;
	int lowThresholdF = numIndivsPerPop/8, highThresholdF=3*numIndivsPerPop/8;
	int thresholdM = numIndivsPerPop/4;
	for(i=0;i<numIndivsPerPop;++i){
		if(i<numIndivsPerPop/2){
				sexList.Append(NEW aString("M"));}
			else{
				sexList.Append(NEW aString("F"));}
		}//i
	for(i=0;i<numIndivsPerPop/2;++i){
		if(i<thresholdM){
				genotypeListM.Append(NEW aString("A"));}
			else{genotypeListM.Append(NEW aString("B"));}
		if(i<lowThresholdF){
				genotypeListF.Append(NEW aString("AA"));}
			else if(i>=highThresholdF){
				genotypeListF.Append(NEW aString("BB"));}
			else{
				genotypeListF.Append(NEW aString("AB"));}
		}//i
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	aString genotype;
	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("precisely generated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create id numbers & genders for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
		rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());	
		StringList sexListCopy(sexList);
		for (i=0;i<numIndivsPerPop;++i){
			rawTraits_[pop].Append(NEW TextDataList());
			aString indiv(i+1);
			indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));
			//choose a gender from list without replacement
			item = ran.longBetween(0,sexListCopy.len()-1);
			rawTraits_[pop][i].Append(NEW TextDatum(sexListCopy[item]));
			sexListCopy.Delete(item);
			
			rawSLCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
			for(loc=0;loc<nlociSLCodom_;++loc){
				rawSLCodomGtypes_[pop][i].Append(NEW TextDataList());}					
			}//end for i
		popSize_.Append(numIndivsPerPop);

		//create and append genotypes
		for(loc0=0;loc0<nlociSLCodom_;++loc0){
			StringList genotypeListFcopy(genotypeListF), genotypeListMcopy(genotypeListM);//work with copy
			loc = allSLCodomLoci_[loc0];
			for(i=0;i<numIndivsPerPop;++i){//choose genotypes randomly without replacement
				if(heterogametic(pop,i)){//heterogametic=male
						item = ran.longBetween(0,genotypeListMcopy.len()-1);
						rawTraits_[pop][i].Append(NEW TextDatum(genotypeListMcopy[item]));
						rawSLCodomGtypes_[pop][i][loc0].Append(NEW TextDatum(genotypeListMcopy[item]));		
						genotypeListMcopy.Delete(item);
						}
					else{//homogametic=female
						item = ran.longBetween(0,genotypeListFcopy.len()-1);
						rawTraits_[pop][i].Append(NEW TextDatum(genotypeListFcopy[item]));								
						for(int c=0;c<2;++c){
							rawSLCodomGtypes_[pop][i][loc0].Append(NEW TextDatum(genotypeListFcopy[item][c]));	
							}
						genotypeListFcopy.Delete(item);
						}	
				}//end for i
			}//end for loc
		}//end for pop
	SortBySex();
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end SexLinkedCodomMarkerData::CreatePerfectData



void SexLinkedCodomMarkerData::CreateRandomData(
							const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Fst=";
	title_.cat(Fst); title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numPopsSampled;
	nlociSLCodom_ = numLociSampled;
	ngroups_ = 1;
	int traitcount=0;
	traitName_.Append(NEW aString("ID"));
	traitNum_.Append(traitcount++);
	traitName_.Append(NEW aString("sex"));
	traitNum_.Append(traitcount++);
	sexTrait_=1;
	int pop,loc,loc0,i;
	for(loc = 0;loc<nlociSLCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociSLCodom_);
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(traitcount++);
		allSLCodomLoci_.Append(loc+2);
		locusNumSLCodom_.Append(loc+2);
		}
	ntraits_ =traitNum_.len();
	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
	GeneFrequencyDistribution parametricFreqDistrib(Fst, pBar, PloidyEffective());
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	aString genotype;
	ObjList<StatList<long double> > locusFrequenciesByPop;
	for(pop=0;pop<npop_;++pop){
		locusFrequenciesByPop.Append(NEW StatList<long double>());
		for(loc=0;loc<nlociSLCodom_;++loc){
			long double pf = parametricFreqDistrib.RandomFrequency(ran);
			locusFrequenciesByPop[pop].Append(pf);
			}//end for loc
		}//end for pop

	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("randomly generated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create id numbers & genders for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
		rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());	
		for (i=0;i<numIndivsPerPop;++i){
			rawTraits_[pop].Append(NEW TextDataList());
			aString indiv(i+1);
			indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));
			if(ran()<primarySexRatio()){//choose a gender
					rawTraits_[pop][i].Append(NEW TextDatum('F'));}
				else{
					rawTraits_[pop][i].Append(NEW TextDatum('M'));}
			rawSLCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
			for(loc=0;loc<nlociSLCodom_;++loc){
				rawSLCodomGtypes_[pop][i].Append(NEW TextDataList());}					
			}//end for i
		popSize_.Append(numIndivsPerPop);

		//create and append genotypes
		for(loc0=0;loc0<nlociSLCodom_;++loc0){
			loc = allSLCodomLoci_[loc0];
			StatList<long double>& locusFrequencies = locusFrequenciesByPop[pop];
			long double p = locusFrequencies[loc0];
			GenotypeFreqDistribution popGenotypesHomogametic(p, Fis);
			for(i=0;i<numIndivsPerPop;++i){
				int gtype=0;
				if(heterogametic(pop,i)){//heterogametic - choose based on allele frequency
						if(ran()>=p){gtype=1;}
						if (gtype==0){genotype = "A";}
							else if(gtype==1){genotype = "B";}
						rawTraits_[pop][i].Append(NEW TextDatum(genotype));
						rawSLCodomGtypes_[pop][i][loc0].Append(NEW TextDatum(genotype));		
						}
					else{//homogametic - choose based on genotype frequency
						gtype = popGenotypesHomogametic.RandomGenotype(ran);
						if (gtype==0){genotype = "AA";}
							else if(gtype==1){genotype = "AB";}
							else if(gtype==2){genotype = "BB";}	
						rawTraits_[pop][i].Append(NEW TextDatum(genotype));
						for(int c=0;c<ploidy_;++c){
							rawSLCodomGtypes_[pop][i][loc0].Append(NEW TextDatum(genotype[c]));		
							}
						}	
				}//end for i
			}//end for loc
		}//end for pop	\
	SortBySex();
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end SexLinkedCodomMarkerData::CreateRandomData



void SexLinkedCodomMarkerData::CreateRandomData(
							const long double Fst, 
							const long double Fis,
							ObjList<aList<long double> >& pBar, //loci by alleles
							ObjList<aList<int> >& numIndivsPerPopLoc,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Fst=";
	title_.cat(Fst); title_.cat(", Fis="); title_.cat(Fis); 
	npop_ = numIndivsPerPopLoc.len();
	nlociSLCodom_ = pBar.len();
	ngroups_ = 1;
	int traitcount=0;
	traitName_.Append(NEW aString("ID"));
	traitNum_.Append(traitcount++);
	traitName_.Append(NEW aString("sex"));
	traitNum_.Append(traitcount++);
	sexTrait_=1;
	for(int loc = 0;loc<nlociSLCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociSLCodom_);
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(traitcount++);
		allSLCodomLoci_.Append(loc+2);
		locusNumSLCodom_.Append(loc+2);
		}
	ntraits_ =traitNum_.len();


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();

	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	ObjList<ObjList<StatList<long double> > > frequenciesByPopLocusAllele;
	for(int pop=0;pop<npop_;++pop){
		frequenciesByPopLocusAllele.Append(NEW ObjList<StatList<long double> >());}
	for(int loc=0;loc<nlociSLCodom_;++loc){
		if(pBar[loc].len()>1){//locus is variable
				GeneFrequencyDistMultiAllele parametricFreqDistrib(Fst, pBar[loc], PloidyEffective());
				for(int pop=0;pop<npop_;++pop){
					aList<long double>* p = parametricFreqDistrib.RandomFrequencies(ran);
					frequenciesByPopLocusAllele[pop].Append(NEW StatList<long double>(*p));//copy as StatList
					delete p;p=NULL;
					}//end for pop
				}
			else{//locus is monomorphic
				for(int pop=0;pop<npop_;++pop){
					frequenciesByPopLocusAllele[pop].Append(NEW StatList<long double>(pBar[loc]));//copy as StatList
					}//end for pop
				}				
		}//end for loc

	for (int pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("randomly generated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		int maxIndivPop = 0;
		for (int loc=0;loc<numIndivsPerPopLoc[pop].len();++loc){
			maxIndivPop = MAX(maxIndivPop,numIndivsPerPopLoc[pop][loc]);}

		//create id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
		rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());							
		for (int i=0;i<maxIndivPop;++i){
			rawTraits_[pop].Append(NEW TextDataList());
			aString indiv(i+1);//first create id numbers for each individual
			indiv.PrependCharToLen(' ', MAX(4,Digits(maxIndivPop)));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));
			if(ran()<primarySexRatio()){//choose a gender
					rawTraits_[pop][i].Append(NEW TextDatum('F'));}
				else{
					rawTraits_[pop][i].Append(NEW TextDatum('M'));}
			rawSLCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
			for(int loc=0;loc<nlociSLCodom_;++loc){
				rawSLCodomGtypes_[pop][i].Append(NEW TextDataList());}					
			}//end for i
		popSize_.Append(maxIndivPop);

		for(int loc0=0;loc0<nlociSLCodom_;++loc0){
			int loc = allSLCodomLoci_[loc0];
			StatList<long double>& p = frequenciesByPopLocusAllele[pop][loc0];
//			for(int a=0;a<p.len();++a){
//				cout<<"pop:  "<<pop+1<<"   "<<"loc:  "<<loc+1<<"   "<<"allele:  "<<a+1<<"   "<<p[a]<<endl;}
				if(p.len()>1){//polymorphic:  sampling required
						GenotypeFreqMultiAlleleDist popGenotypes(p, Fis);
						for(int i=0;i<numIndivsPerPopLoc[pop][loc0];++i){
							if(heterogametic(pop,i)){//heterogametic -- sample based on allele frequency				
									char g = char(ran.weighted(p) + 'A');
									rawTraits_[pop][i].Append(NEW TextDatum(g));
									rawSLCodomGtypes_[pop][i][loc0].Append(NEW TextDatum(g));		
									}
								else{//homogametic: sample based on genotype frequency
									aString* genotype = popGenotypes.RandomGenotypeString(ran);
									rawTraits_[pop][i].Append(NEW TextDatum(*genotype));
									for(int c=0;c<ploidy_;++c){
										rawSLCodomGtypes_[pop][i][loc0].Append(NEW TextDatum((*genotype)[c]));	
										}		
									delete genotype; genotype=NULL;
									}
							}//end for i
						for(int i=numIndivsPerPopLoc[pop][loc0];i<maxIndivPop;++i){//unsampled individuals blank
							
							rawTraits_[pop][i].Append(NEW TextDatum());//add missing data
							for(int c=0;c<Ploidy(pop,i,loc);++c){
								rawSLCodomGtypes_[pop][i][loc0].Append(NEW TextDatum());//add missing data
								}	
							}//end for i
						}//polymorphic
					else{//monomorphic locus
						for(int i=0;i<numIndivsPerPopLoc[pop][loc0];++i){
							rawTraits_[pop][i].Append(NEW TextDatum("A"));
							for(int c=0;c<Ploidy(pop,i,loc);++c){
								rawSLCodomGtypes_[pop][i][loc0].Append(NEW TextDatum("A"));}	
							}//end for i
						for(int i=numIndivsPerPopLoc[pop][loc0];i<maxIndivPop;++i){//unsampled individuals blank
							rawTraits_[pop][i].Append(NEW TextDatum());//add missing data
							for(int c=0;c<Ploidy(pop,i,loc);++c){
								rawSLCodomGtypes_[pop][i][loc0].Append(NEW TextDatum());}//add missing data	
							}//end for i
						}//monomorphic
			}//end for loc
		}//end for pop	
	SortBySex();
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end SexLinkedCodomMarkerData::CreateRandomData



void SexLinkedCodomMarkerData::CreateRandomData(
							const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Flt=";
	title_.cat(Flt); title_.cat(", Fspt="); title_.cat(Fspt);  title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numSpeciesSampled * numLocalitiesSampled;
	nlociSLCodom_ = numLociSampled;
	ngroups_ = numSpeciesSampled;
	int traitcount=0;
	traitName_.Append(NEW aString("ID"));
	traitNum_.Append(traitcount++);
	traitName_.Append(NEW aString("sex"));
	traitNum_.Append(traitcount++);
	sexTrait_=1;
	for(int loc = 0;loc<nlociSLCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociSLCodom_);
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(traitcount++);
		allSLCodomLoci_.Append(loc+2);
		locusNumSLCodom_.Append(loc+2);
		}
	ntraits_ =traitNum_.len();


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE SPECIES
	GeneFrequencyDistribution parametricFreqDistribSP(Fspt, pBar, PloidyEffective());
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<aList<long double> > pBarBySpeciesAndLocus;
	for(int sp0=0;sp0<numSpeciesSampled;++sp0){
		aList<long double>* locusPList = NEW aList<long double>();
		pBarBySpeciesAndLocus.Append(locusPList);
		for(int locus0=0;locus0<nlociSLCodom_;++locus0){
			long double p0 = parametricFreqDistribSP.RandomFrequency(ran);
			locusPList->Append(p0);
			}//end for locus0
		}//end for sp0
		
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	for(int sp1=0; sp1<numSpeciesSampled;++sp1){
		//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG populations within the species
		ObjList<StatList<long double> > locusFrequenciesByLocality;
		for(int locality1=0;locality1<numLocalitiesSampled; ++locality1){
			StatList<long double>* locusFrequencies = NEW StatList<long double>();
			locusFrequenciesByLocality.Append(locusFrequencies);
			for(int loc=0;loc<nlociSLCodom_;++loc){
				GeneFrequencyDistribution parametricFreqDistrib(Flt, pBarBySpeciesAndLocus[sp1][loc], 1000);
				long double p = parametricFreqDistrib.RandomFrequency(ran);
				locusFrequencies->Append(p);
				}//end for loc
			}//end for locality1

		for (int locality1=0;locality1<numLocalitiesSampled; ++locality1){
			// NAME THE POPULATION
			aString pName("LOCALITY");pName.cat(locality1+1);pName.cat(" (");pName.cat(locality1+1);
			pName.cat(") randomly generated population of SPECIES"); pName.cat(sp1+1);
			popName_.Append(NEW aString(pName));//append a copy
			int pop = popName_.len()-1;
			//create id numbers for each individual
			rawTraits_.Append(NEW ObjList<TextDataList>());	
			rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());	
			for (int i=0;i<numIndivsPerPop;++i){
				rawTraits_[pop].Append(NEW TextDataList());
				aString indiv(i+1);//first create id numbers for each individual
				indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
				rawTraits_[pop][i].Append(NEW TextDatum(indiv));
				if(ran()<primarySexRatio()){//choose a gender
						rawTraits_[pop][i].Append(NEW TextDatum('F'));}
					else{
						rawTraits_[pop][i].Append(NEW TextDatum('M'));}
				rawSLCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				for(int loc=0;loc<nlociSLCodom_;++loc){
					rawSLCodomGtypes_[pop][i].Append(NEW TextDataList());}					
				}//end for i
			popSize_.Append(numIndivsPerPop);

			for(int loc=0;loc<nlociSLCodom_;++loc){
				StatList<long double>& locusFrequencies = locusFrequenciesByLocality[locality1];
				double p = locusFrequencies[loc];
				GenotypeFreqDistribution popGenotypes(p, Fis);
				for(int i=0;i<numIndivsPerPop;++i){
					aString genotype = "A";
					if(heterogametic(pop,i)){//heterogematic; choose based on allele frequency
							if(ran()>=p){genotype = "B";}
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype));								
							}
						else{//homogametic" choose based on genotype frequency
							int gtype = popGenotypes.RandomGenotype(ran);
							if (gtype==0){genotype = "AA";}
								else if(gtype==1){genotype = "AB";}
								else if(gtype==2){genotype = "BB";}			
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							for(int c=0;c<ploidy_;++c){
								rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[c]));		
								}
							}
					}//end for i
				}//end for loc
			}//end for locality1
		}//end for sp1	
	SortBySex();
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end SexLinkedCodomMarkerData::CreateRandomData




void SexLinkedCodomMarkerData::CreateRandomData(//not debugged
							const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Flt=";
	title_.cat(Flt); title_.cat(", Fspt="); title_.cat(Fspt);  
	title_.cat(", Flspt="); title_.cat(Flspt); title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numSpeciesSampled * numLocalitiesSampled;
	nlociSLCodom_ = numLociSampled;
	ngroups_ = numSpeciesSampled;
	int traitcount=0;
	traitName_.Append(NEW aString("ID"));
	traitNum_.Append(traitcount++);
	traitName_.Append(NEW aString("sex"));
	traitNum_.Append(traitcount++);
	sexTrait_=1;
	for(int loc = 0;loc<nlociSLCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociSLCodom_);
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(traitcount++);
		allSLCodomLoci_.Append(loc+2);
		locusNumSLCodom_.Append(loc+2);
		}
	ntraits_ =traitNum_.len();


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE SPECIES
	GeneFrequencyDistribution parametricFreqDistribSP(Fspt, pBar, PloidyEffective());
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<aList<long double> > pBarBySpeciesAndLocus;
	for(int sp0=0;sp0<numSpeciesSampled;++sp0){
		aList<long double>* locusPList = NEW aList<long double>();
		pBarBySpeciesAndLocus.Append(locusPList);
		for(int locus0=0;locus0<nlociSLCodom_;++locus0){
			long double p0 = parametricFreqDistribSP.RandomFrequency(ran);
			locusPList->Append(p0);
			}//end for locus0
		}//end for sp0
		
	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE LOCALITIES
	GeneFrequencyDistribution parametricFreqDistribLocality(Flt, pBar, 1000);
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<aList<long double> > pBarByLocalityAndLocus;
	for(int L0=0;L0<numLocalitiesSampled;++L0){
		aList<long double>* locusPList = NEW aList<long double>();
		pBarByLocalityAndLocus.Append(locusPList);
		for(int locus0=0;locus0<nlociSLCodom_;++locus0){
			long double p0 = parametricFreqDistribLocality.RandomFrequency(ran);
			locusPList->Append(p0);
			}//end for locus0
		}//end for L0
		
	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE SUBPOPULATIONS
//	GeneFrequencyDistribution parametricFreqDistribSubpop(Flspt, pBar, 1000);//old version sends dp
	GeneFrequencyDistribution parametricFreqDistribSubpop(Flspt, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<ObjList<aList<long double> > > pBarBySpeciesLocalityAndLocus;
	for(int sp1=0;sp1<numSpeciesSampled;++sp1){
		pBarBySpeciesLocalityAndLocus.Append(NEW ObjList<aList<long double> >());
		for(int L1=0;L1<numLocalitiesSampled;++L1){
			aList<long double>* locusPList = NEW aList<long double>();
			pBarBySpeciesLocalityAndLocus[sp1].Append(locusPList);
			for(int locus0=0;locus0<nlociSLCodom_;++locus0){
				long double p0 = parametricFreqDistribSubpop.RandomFrequency(ran);
				long double devLocality = (pBarByLocalityAndLocus[L1][locus0] - pBar);
				long double devSpecies = (pBarBySpeciesAndLocus[sp1][locus0] - pBar);
				long double p = p0 + devLocality + devSpecies;
				locusPList->Append(p);
				}//end for locus0
			}//end for L0
		}//end for sp0
		
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	for(int sp1=0; sp1<numSpeciesSampled;++sp1){
		//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG populations within the species
//		ObjList<StatList<double> > locusFrequenciesByLocality;
//		for(int locality1=0;locality1<numLocalitiesSampled; ++locality1){
//			StatList<double>* locusFrequencies = new StatList<double>();
//			locusFrequenciesByLocality.Append(locusFrequencies);
//			for(int loc=0;loc<nloci;++loc){
//				GeneFrequencyDistribution parametricFreqDistrib(Flspt, pBarBySpeciesLocalityAndLocus[sp1][locality1][loc], 1000);
//				double p = parametricFreqDistrib.RandomFrequency(ran);
//				locusFrequencies->Append(p);
//				}//end for loc
//			}//end for locality1

		for (int locality1=0;locality1<numLocalitiesSampled; ++locality1){
			// NAME THE POPULATION
			aString pName("LOCALITY");pName.cat(locality1+1);pName.cat(" (");pName.cat(locality1+1);
			pName.cat(") randomly generated population of SPECIES"); pName.cat(sp1+1);
			popName_.Append(NEW aString(pName));//append a copy
			int pop = popName_.len()-1;
			//create id numbers for each individual
			rawTraits_.Append(NEW ObjList<TextDataList>());	
			rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());	
			for (int i=0;i<numIndivsPerPop;++i){
				rawTraits_[pop].Append(NEW TextDataList());
				aString indiv(i+1);//first create id numbers for each individual
				indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
				rawTraits_[pop][i].Append(NEW TextDatum(indiv));
				if(ran()<primarySexRatio()){//choose a gender
						rawTraits_[pop][i].Append(NEW TextDatum('F'));}
					else{
						rawTraits_[pop][i].Append(NEW TextDatum('M'));}
				rawSLCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				for(int loc=0;loc<nlociSLCodom_;++loc){
					rawSLCodomGtypes_[pop][i].Append(NEW TextDataList());}					
				}//end for i
			popSize_.Append(numIndivsPerPop);

			for(int loc=0;loc<nlociSLCodom_;++loc){
				long double p = pBarBySpeciesLocalityAndLocus[sp1][locality1][loc];
				GenotypeFreqDistribution popGenotypes(p, Fis);
				for(int i=0;i<numIndivsPerPop;++i){
					aString genotype = "A";
					if(heterogametic(pop,i)){//heterogametic: choose based on allele frequency
							if(ran()>=p){genotype="B";}
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype));		
							}
						else{//homogametic: choose based on genotype frequency
							int gtype = popGenotypes.RandomGenotype(ran);
							if (gtype==0){genotype = "AA";}
								else if(gtype==1){genotype = "AB";}
								else if(gtype==2){genotype = "BB";}			
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							for(int c=0;c<ploidy_;++c){
								rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[c]));		
								}
							}		
					}//end for i
				}//end for loc
			}//end for locality1
		}//end for sp1	
	SortBySex();
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end SexLinkedCodomMarkerData::CreateRandomData


void SexLinkedCodomMarkerData::CreateRandomDataLocFirst(
							const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							const int numLociSampled,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Flt=";
	title_.cat(Flt); title_.cat(", Fspt="); title_.cat(Fspt);  title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numSpeciesSampled * numLocalitiesSampled;
	nlociSLCodom_ = numLociSampled;
	ngroups_ = numSpeciesSampled;
	int traitcount=0;
	traitName_.Append(NEW aString("ID"));
	traitNum_.Append(traitcount++);
	traitName_.Append(NEW aString("sex"));
	traitNum_.Append(traitcount++);
	sexTrait_=1;
	for(int loc = 0;loc<nlociSLCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociSLCodom_);
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(traitcount++);
		allSLCodomLoci_.Append(loc+2);
		locusNumSLCodom_.Append(loc+2);
		}
	ntraits_ =traitNum_.len();


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE LOCALITIES
	GeneFrequencyDistribution parametricFreqDistribL(Flt, pBar, PloidyEffective());//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	aList<long double> pBarByLocality;
	for(int L0=0;L0<numLocalitiesSampled;++L0){
		long double p0 = parametricFreqDistribL.RandomFrequency(ran);
		pBarByLocality.Append(p0);
		}//end for L0
		
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	for(int L1=0; L1<numLocalitiesSampled;++L1){
		//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG SPECIES WITHIN THE LOCALITIES
		GeneFrequencyDistribution parametricFreqDistrib(Fspt, pBarByLocality[L1], PloidyEffective());//new version sends ploidy
		ObjList<StatList<long double> > locusFrequenciesBySpecies;
		for(int species1=0;species1<numSpeciesSampled; ++species1){
			StatList<long double>* locusFrequencies = NEW StatList<long double>();
			locusFrequenciesBySpecies.Append(locusFrequencies);
			for(int loc=0;loc<nlociSLCodom_;++loc){
				long double p = parametricFreqDistrib.RandomFrequency(ran);
				locusFrequencies->Append(p);
				}//end for loc
			}//end for species1

		for (int species1=0;species1<numSpeciesSampled; ++species1){
			// NAME THE POPULATION
			aString pName("LOCALITY");pName.cat(L1+1);pName.cat(" (");pName.cat(L1+1);
			pName.cat(") randomly generated population of SPECIES"); pName.cat(species1+1);
			popName_.Append(NEW aString(pName));//append a copy
			int pop = popName_.len()-1;
			//create id numbers for each individual
			rawTraits_.Append(NEW ObjList<TextDataList>());	
			rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());	
			for (int i=0;i<numIndivsPerPop;++i){
				rawTraits_[pop].Append(NEW TextDataList());
				aString indiv(i+1);//first create id numbers for each individual
				indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
				rawTraits_[pop][i].Append(NEW TextDatum(indiv));
				if(ran()<primarySexRatio()){//choose a gender
						rawTraits_[pop][i].Append(NEW TextDatum('F'));}
					else{
						rawTraits_[pop][i].Append(NEW TextDatum('M'));}
				rawSLCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				for(int loc=0;loc<nlociSLCodom_;++loc){
					rawSLCodomGtypes_[pop][i].Append(NEW TextDataList());}					
				}//end for i
			popSize_.Append(numIndivsPerPop);

			for(int loc=0;loc<nlociSLCodom_;++loc){
				StatList<long double>& locusFrequencies = locusFrequenciesBySpecies[species1];
				long double p = locusFrequencies[loc];
				GenotypeFreqDistribution popGenotypes(p, Fis);
				for(int i=0;i<numIndivsPerPop;++i){
					aString genotype = "A";
					if(heterogametic(pop,i)){//heterogametic: choose based on allele frequency
							if(ran()<=p){genotype="B";}
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype));	
							}
						else{//homogametic:  choose based on genotype frequency
							int gtype = popGenotypes.RandomGenotype(ran);
							if (gtype==0){genotype = "AA";}
								else if(gtype==1){genotype = "AB";}
								else if(gtype==2){genotype = "BB";}			
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							for(int c=0;c<ploidy_;+c){
								rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[c]));		
								}
							}		
					}//end for i
				}//end for loc
			}//end for species1
		}//end for L1	
	SortBySex();
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end SexLinkedCodomMarkerData::CreateRandomDataLocFirst




void SexLinkedCodomMarkerData::CullLoci (aList<int>& lociToCull){
	//This function shifts trait numbers from from traitNums_ to the skippedTraitNums_ list
	if(lociToCull.len()==0){return;}//none to cull
	int tr,cLoc,item;
	for(tr=0;tr<lociToCull.len();++tr){
		cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(traitNum_.Includes(cLoc) && cLoc>=0 && cLoc<traitName_.len()){
			item = traitNum_.Loc(cLoc);
			skippedTraitNum_.Append(cLoc);
			skippedLocusNumSLCodom_.Append(cLoc);
			traitNum_.Delete(item);
			item = locusNumSLCodom_.Loc(cLoc);
			locusNumSLCodom_.Delete(item);
			}//if item not skipped already
		}//tr
	nlociSLCodom_ = locusNumSLCodom_.len();
	skippedTraitNum_.Sort();
	skippedLocusNumSLCodom_.Sort();
	ResetFlags();
	CheckForNewAlleles();
	}//end procedure SexLinkedCodomMarkerData::CullLoci(aList<int>&)


void SexLinkedCodomMarkerData::CullLoci (StringList& lociToCull){
	//This function culls the designated loci from each genotype data string
	//and includes them among the skipped loci
	if(lociToCull.len()==0){return;}//none to cull
	//convert names to numbers and cull
	aList<int> cullListByNum;
	int tr,item;
	for(tr=0;tr<lociToCull.len();++tr){
		aString cTraitname = lociToCull[tr];
		item = traitName_.Loc(&cTraitname);
		if((item>=0)&&(!(skippedTraitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already skipped
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	CullLoci(cullListByNum);
	}//end procedure SexLinkedCodomMarkerData::CullLoci(StringList&)



void SexLinkedCodomMarkerData::CullLocus (char* locusToCull){
	aString locToCull(locusToCull);
	CullLocus(locToCull);
	}//end SexLinkedCodomMarkerData::CullLocus (char* )


void SexLinkedCodomMarkerData::CullLocus (int locusToCull){
	aList<int> lociToCull;
	lociToCull.Append(locusToCull);
	CullLoci(lociToCull);
	}//end SexLinkedCodomMarkerData::CullLocus (int )


void SexLinkedCodomMarkerData::CullLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	CullLoci(lociToCull);
	}//end SexLinkedCodomMarkerData::CullLocus (aString& )


void SexLinkedCodomMarkerData::CullMonomorphicLoci (void){	
	//This function culls the monomorphic loci
	//and includes them among the skipped loci
	UpdateVariableLoci();
	if(variableLocusNumSLCodom_.len()==locusNumSLCodom_.len()){return;}//none monomorphic
	aList<int> monomorphicLoci(locusNumSLCodom_);
	int vLoc,vLocusID,item;
	for(vLoc=variableLocusNumSLCodom_.len()-1; vLoc>=0;--vLoc){
		vLocusID = variableLocusNumSLCodom_[vLoc];
		item = monomorphicLoci.Loc(vLocusID);
		monomorphicLoci.Delete(item);
		}//vLoc
	CullLoci(monomorphicLoci);
	}//end procedure SexLinkedCodomMarkerData::CullMonomorphicLoci




void SexLinkedCodomMarkerData::CullToOneGroup (EvolutionaryGeneticsData& parentData,int groupNum){
	//makes a subset data structure from the specified group
	//individuals resampled
	EvolutionaryGeneticsData::CullToOneGroup(parentData,groupNum);
	EvolutionaryGeneticsData* parentDataPtr =  &parentData;
	unsigned long parentDataUSL = (unsigned long) parentDataPtr;
	SexLinkedCodomMarkerData* parentDataCodomPtr = (SexLinkedCodomMarkerData*) parentDataUSL;
	SexLinkedCodomMarkerData& parentDataC = *parentDataCodomPtr;

	nlociSLCodom_ = parentDataC.nlociSLCodom_;
	ploidy_ = parentDataC.ploidy_;
	alleleLen_ = parentDataC.alleleLen_;
	allSLCodomLoci_ = parentDataC.allSLCodomLoci_;
	locusNumSLCodom_ = parentDataC.locusNumSLCodom_;
	skippedLocusNumSLCodom_ = parentDataC.skippedLocusNumSLCodom_;
	
	int pop, theRightPop;
	for(pop=0;pop<npop_;++pop){
		theRightPop = parentDataC.groupHierarchy_[groupNum][pop];
		rawSLCodomGtypes_.Append(parentDataC.rawSLCodomGtypes_[theRightPop].Duplicate());
		}
	SortBySex();
	CheckForNewAlleles();
	}//end SexLinkedCodomMarkerData::CullToOneGroup (int)



void SexLinkedCodomMarkerData::CullToOneSex (SexLinkedCodomMarkerData& parentData,genderSet sexVal){
	//makes a subset data structure from the specified group
	aList<int> sexValList;
	sexValList.Append((int)sexVal);
	CullToTheseSexes(parentData,sexValList);
	}//end SexLinkedCodomMarkerData::CullToOneSex (SexLinkedCodomMarkerData& , genderSet )


void SexLinkedCodomMarkerData::CullToTheseSexes (SexLinkedCodomMarkerData& parentData,aList<int>& sexValList){
	//makes a subset data structure from the specified group
	Clear();
	if(sexValList.len()==0) return;
	title_ = parentData.title_;
	title_.cat(" -- ");
	int s;
	for(s=0;s<sexValList.len();++s){
		genderSet sexVal = (genderSet) sexValList[s];
		switch(sexVal){
			case A:
				title_.cat("asexuals"); break;
			case B:
				title_.cat("hermaphrodites"); break;
			case F:
				title_.cat("females"); break;
			case M:
				title_.cat("males"); break;
			default: break;
			}
		if(s<sexValList.len()-2){
				title_.cat(", ");}
			else if (s == sexValList.len()-2){
				title_.cat(" & ");}
			else{}
		}
	title_.cat(" only");


	npop_ = parentData.npop_;
	ngroups_ = parentData.ngroups_;
	ntraits_ = parentData.ntraits_;
	groupName_ = parentData.groupName_;
	localityName_ = parentData.localityName_;
	traitName_ = parentData.traitName_;
	traitNum_ = parentData.traitNum_;
	skippedTraitNum_ = parentData.skippedTraitNum_;
	missing_ = parentData.missing_;
	popName_ = parentData.popName_;
	localityCoordinates_ = parentData.localityCoordinates_;
	missing_ = parentData.missing_;
	
	heterogameticSex_ = parentData.heterogameticSex_;
	sexTrait_ = parentData.sexTrait_;
	primarySexRatio_ = parentData.primarySexRatio_;

	nlociSLCodom_ = parentData.nlociSLCodom_;
	locusNumSLCodom_ = parentData.locusNumSLCodom_;
	allSLCodomLoci_ = parentData.allSLCodomLoci_;
	skippedLocusNumSLCodom_ = parentData.skippedLocusNumSLCodom_;
	
	genderSet thisSexVal=B;
	int pop, i, thisSexValInt;
	for(pop=0;pop<parentData.npop_;++pop){
		popSize_.Append(0);
		rawTraits_.Append(NEW ObjList<TextDataList>());
		rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
		for(i=0;i<parentData.popSize(pop);++i){
			TextDatum& thisSex = parentData.rawTraits_[pop][i][sexTrait_];
			thisSexVal = genderCode(thisSex);
			thisSexValInt = (int)thisSexVal;
			if(sexValList.Includes(thisSexValInt)){
				rawTraits_[pop].Append((TextDataList*)parentData.rawTraits_[pop][i].Duplicate());
				rawSLCodomGtypes_[pop].Append(parentData.rawSLCodomGtypes_[pop][i].Duplicate());
				popSize_[pop]++;}
			}//i
		}//pop
	
	//cull empty populations from data
	for(pop=npop_-1;pop>=0;--pop){
		if(parentData.rawTraits_[pop].len()==0){
			parentData.rawTraits_.Delete(pop);
			popName_.Delete(pop);
			popSize_.Delete(pop);
			}
		}//pop
	npop_ = rawTraits_.len();
		
	ResetFlags();
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();
	}//end SexLinkedCodomMarkerData::CullToTheseSexes (SexLinkedCodomMarkerData& ,aList<int>& )


void SexLinkedCodomMarkerData::ExpungeLoci (StringList& lociToCull){
	if(lociToCull.len()==0){return;}//none to cull
	aList<int> cullListByNum;
	int tr,item;
	for(tr=0;tr<lociToCull.len();++tr){
		aString cTraitname = lociToCull[tr];
		item = traitName_.Loc(&cTraitname);
		if(item>=0){//the trait exists in the rawlist
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	ExpungeLoci(cullListByNum);
	}//end  SexLinkedCodomMarkerData::ExpungeLoci(StringList&)


void SexLinkedCodomMarkerData::ExpungeLoci (aList<int>& lociToCull){
	//This function removes trait numbers from the haploid data set; called during reclassification
	if(lociToCull.len()==0){return;}//none to cull
	int tr,cLoc, item;
	for(tr=0;tr<lociToCull.len();++tr){
		cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(allSLCodomLoci_.Includes(cLoc)){
			item = allSLCodomLoci_.Loc(cLoc);
			allSLCodomLoci_.Delete(item);}
		if(locusNumSLCodom_.Includes(cLoc)){
			item = locusNumSLCodom_.Loc(cLoc);
			locusNumSLCodom_.Delete(item);}
		if(skippedLocusNumSLCodom_.Includes(cLoc)){
			item = skippedLocusNumSLCodom_.Loc(cLoc);
			skippedLocusNumSLCodom_.Delete(item);}
		}//tr
	nlociSLCodom_ = locusNumSLCodom_.len();
	ResetFlags();
//	SexLinkedCodomMarkerData::ParseGenotypicData(allSLCodomLoci_,0);
	SexLinkedCodomMarkerData::ParseGenotypicData(0);
	SexLinkedCodomMarkerData::CheckForNewAlleles();
	}//SexLinkedCodomMarkerData::ExpungeLoci (aList<int>&)



void SexLinkedCodomMarkerData::ExpungeLocus (char* locusToCull){
	aString locToCull(locusToCull);
	ExpungeLocus(locToCull);
	}//end SexLinkedCodomMarkerData::ExpungeLocus (char* )


void SexLinkedCodomMarkerData::ExpungeLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	ExpungeLoci(lociToCull);
	}//end SexLinkedCodomMarkerData::ExpungeLocus (aString& )


TextDataList& SexLinkedCodomMarkerData::GenotypicVal(int pop, int indiv, int trait){
	//does not notice skipped loci
//	return rawSLCodomGtypes_[pop][indiv][locusNumSLCodom_[loc]];
//	return rawSLCodomGtypes_[pop][indiv][loc];
//	int ordinalLoc = locusNumSLCodom_.Loc(loc);
//	return rawSLCodomGtypes_[pop][indiv][ordinalLoc];
//	int correctLoc = allSLCodomLoci_.Loc(locusNumSLCodom_[loc]);
	int correctLoc = allSLCodomLoci_.Loc(trait);
	return rawSLCodomGtypes_[pop][indiv][correctLoc];
	}//end  SexLinkedCodomMarkerData::GenotypicVal(int,int,int)


TextDatum& SexLinkedCodomMarkerData::GenotypicVal(int pop, int indiv, int trait, int allele){
	//does not notice skipped loci
//	return rawSLCodomGtypes_[pop][indiv][locusNumSLCodom_[loc]][allele];
//	return rawSLCodomGtypes_[pop][indiv][loc][allele];
//	int ordinalLoc = locusNumSLCodom_.Loc(loc);
//	return rawSLCodomGtypes_[pop][indiv][ordinalLoc][allele];
//	int correctLoc = allSLCodomLoci_.Loc(locusNumSLCodom_[loc]);
	int correctLoc = allSLCodomLoci_.Loc(trait);
	return rawSLCodomGtypes_[pop][indiv][correctLoc][allele];
	}//end  SexLinkedCodomMarkerData::GenotypicVal(int,int,int,int)




long double SexLinkedCodomMarkerData::GetExpectedHeterozygosity (int pop, int loc, int allele){
	//calculated from allele freqs of both sexes
	long double freq = GetFreq(pop, loc, allele);
	return (two * freq * (one - freq));
	}//end SexLinkedCodomMarkerData::GetExpectedHeterozygosity(pop,loc,allele)



long double SexLinkedCodomMarkerData::GetExpectedHeterozygosity (int pop, int loc){
	long double freq,expHomoFreq = zero;
	int allele;
	for (allele=0;allele<GetNumAlleles(loc);++allele){
		freq = GetFreq(pop, loc, allele);
		expHomoFreq += freq*freq;}
	return (one - expHomoFreq);
	}//end SexLinkedCodomMarkerData::GetExpectedHeterozygosity(pop,loc)



long double SexLinkedCodomMarkerData::GetExpectedHeterozygosity (int pop){
	//returns mean Hexp over loci
	StatList<long double> hetExp;
	int loc;
	for (loc=0;loc<nlociSLCodom_;++loc){
		hetExp.Append(GetExpectedHeterozygosity(pop,loc));}
	return hetExp.mean();
	}//end SexLinkedCodomMarkerData::GetExpectedHeterozygosity(pop)



long double SexLinkedCodomMarkerData::GetFreq (int loc, int allele){
	long double n, numAlleleCopies = zero;
	long double numThisAllele = zero;
	int pop;
	for(pop=0;pop<npop_;++pop){
		n = (long double)GetLocusSampleSize(pop,loc);
		numAlleleCopies += n;
		numThisAllele += n * GetFreq(pop,loc,allele);
		}
	if(numAlleleCopies==0.0){return zero;}
	return numThisAllele/numAlleleCopies;
	}//end SexLinkedCodomMarkerData::GetFreq


long double SexLinkedCodomMarkerData::GetFreq (int pop, int loc, int allele){
	if (!alleleFreqCurrentSLCodom_){CalculateAlleleFreqs();}
	return alleleFreqSLCodom_[pop][loc][allele];
	}//end SexLinkedCodomMarkerData::GetFreq


long double SexLinkedCodomMarkerData::GetFreqHeterogametic (int loc, int allele){
	long double n, numAlleleCopies = zero;
	long double numThisAllele = zero;
	for(int pop=0;pop<npop_;++pop){
		n = (long double)GetLocusSampleSizeHeterogametic(pop,loc);
		numAlleleCopies += n;
		numThisAllele += n * GetFreqHeterogametic(pop,loc,allele);
		}
	if(numAlleleCopies==zero){return zero;}
	return numThisAllele/numAlleleCopies;
	}//end SexLinkedCodomMarkerData::GetFreqHeterogametic


long double SexLinkedCodomMarkerData::GetFreqHeterogametic (int pop, int loc, int allele){
	if (!alleleFreqCurrentSLCodom_){CalculateAlleleFreqs();}
	return alleleFreqHeterogametic_[pop][loc][allele];
	}//end SexLinkedCodomMarkerData::GetFreqHeterogametic


long double SexLinkedCodomMarkerData::GetFreqHomogametic (int loc, int allele){
	long double n,numAlleleCopies = zero;
	long double numThisAllele = zero;
	int pop;
	for(pop=0;pop<npop_;++pop){
		n = (long double)GetLocusSampleSizeHomogametic(pop,loc);
		numAlleleCopies += n;
		numThisAllele += n * GetFreqHomogametic(pop,loc,allele);
		}
	if(numAlleleCopies==zero){return zero;}
	return numThisAllele/numAlleleCopies;
	}//end SexLinkedCodomMarkerData::GetFreqHomogametic


long double SexLinkedCodomMarkerData::GetFreqHomogametic (int pop, int loc, int allele){
	if (!alleleFreqCurrentSLCodom_){CalculateAlleleFreqs();}
	return alleleFreqHomogametic_[pop][loc][allele];
	}//end SexLinkedCodomMarkerData::GetFreqHomogametic


long double SexLinkedCodomMarkerData::GetFreqMostCommonAllele (int pop, int loc){
	long double freq, mostCommon = zero;
	int allele;
	for(allele=0;allele<GetNumAlleles(loc);++allele){
		freq = GetFreq(pop,loc,allele);
		if (freq>half) {return freq;}
		if (mostCommon<freq){mostCommon = freq;}
		}//and for allele
	return mostCommon;
	}//end SexLinkedCodomMarkerData::GetFreqMostCommonAllele



long double SexLinkedCodomMarkerData::GetFreqMostCommonAllele (int loc){
	long double freq,mostCommon = zero;
	int allele;
	for(allele=0;allele<GetNumAlleles(loc);++allele){
		freq = GetFreq(loc,allele);
		if (freq>half) {return freq;}
		if (mostCommon<freq){mostCommon = freq;}
		}//and for allele
	return mostCommon;
	}//end SexLinkedCodomMarkerData::GetFreqMostCommonAllele



void SexLinkedCodomMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
	//uploads data to pass to genetic analysis classes
	pgcList.Clear();
	int pop,i, dt;
	//initialize
	aList<int> lociByType;
	for(dt=0;dt<MultiInheritanceModeContainer<int>::inheritanceModes_;++dt){
		lociByType.Append(0);}
	lociByType[MultiInheritanceModeContainer<int>::sexLink]=nlociSLCodom_;
	for(pop=0;pop<npop_;++pop){
		pgcList.Append(NEW PopulationGeneContainer<int>(popSize(pop),lociByType));}
	
	//fill up
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<popSize(pop);++i){
			GetGenome(pgcList[pop][i], pop, i);}
		}
	}//SexLinkedCodomMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >&)



void SexLinkedCodomMarkerData::GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID){
	//uploads this indiv's data to MultiHaplotypeContainer<int>
	MultiHaplotypeContainer<int>* mhc = (MultiHaplotypeContainer<int>*) &gc;
	int thisPloidy = ploidy(pop,indiv);
	int loc,tr,c;
	if(heterogametic(pop,indiv)) {thisPloidy=1;}
	mhc->SetSize(thisPloidy,nlociSLCodom_);
	if(setID){
		mhc->SetID(1+GetIndivNumInDataSequence(pop,indiv));}
	for(loc=0;loc<nlociSLCodom_;++loc){
		tr=locusNumSLCodom_[loc];
		for(c=0;c<thisPloidy;++c){
//			TextDatum& gval = SexLinkedCodomMarkerData::GenotypicVal(pop,indiv,loc,c);
			TextDatum& gval = SexLinkedCodomMarkerData::GenotypicVal(pop,indiv,tr,c);
			if(gval.missing()){
					mhc->SetMissing(c);}
				else{
					mhc->setHaplotype(loc,c,SexLinkedCodomMarkerData::alleleID(loc,gval));}
			}}	
	}//SexLinkedCodomMarkerData::GetGenome(GeneContainerCore&, int, int)




/* see how this is used before debugging
aString* SexLinkedCodomMarkerData::GetGenotypeString(int pop, int indiv){
	StringList& genotypesList=gtypes[pop];
	aString& dataStr = genotypesList[indiv];
	aString *gtypeWord = NEW aString(dataStr);
	aString *codeWord = gtypeWord->ChopOffFirstWord();
	delete codeWord; codeWord=0;
	return gtypeWord;
	}//end SexLinkedCodomMarkerData::GetGenotypeString(pop,indiv)



aString* SexLinkedCodomMarkerData::GetGenotypeString(int pop, int indiv, int loc){
	aString *gtypeWord=GetGenotypeString(pop, indiv);
	int posGtypeWord = 1 + (2+1) * loc;
	aString *gTypeString = NEW aString(*gtypeWord,posGtypeWord,posGtypeWord +1);
	delete gtypeWord;gtypeWord=0;//this was created in GetGenotypeString(pop, indiv);
	return gTypeString;
	}//end SexLinkedCodomMarkerData::GetGenotypeString(pop,indiv,loc)


aString* SexLinkedCodomMarkerData::GetGenotypeString(int pop, int indiv, int loc, int chrom){
//	aString *gtypeString=GetGenotypeString(pop, indiv, loc);
			//	aString *gTypeChrom = NEW aString(*gtypeString,chrom,chrom);
//	aString *gTypeChrom = NEW aString( (*gtypeString)[chrom]);
//	delete gtypeString;gtypeString=0;//this was created in GetGenotypeString(pop, indiv, loc);
//	return gTypeChrom;

	StringList gtypeString=Genotype(pop, indiv, loc);
	aString *gTypeChrom = NEW aString( gtypeString[chrom]);
	return gTypeChrom;
	}//end SexLinkedCodomMarkerData::GetGenotypeString(pop,indiv,loc,chrom)
*/



long SexLinkedCodomMarkerData::GetIndividualsSampled (int pop, int loc){
	long i,n=0;
	for(i=0;i<individuals(pop);++i){
		if(GetLocusSampleSize (pop, i, loc)>0){
			++n;}}
	return n;
	}//end  SexLinkedCodomMarkerData::GetIndividualsSampled(int, int)


long SexLinkedCodomMarkerData::GetLocusSampleSize (int loc){
	if(! alleleFreqCurrentSLCodom_){
		CalculateAlleleFreqs();}//calculates twoNSLCodom_
	long double numAlleleCopies = zero;
	int pop;
	for(pop=0;pop<npop_;++pop){
		numAlleleCopies += twoNSLCodom_[pop][loc];
		}
	return numAlleleCopies;
	}//end  SexLinkedCodomMarkerData::GetLocusSampleSize


long SexLinkedCodomMarkerData::GetLocusSampleSize (int pop, int loc){
	if(! alleleFreqCurrentSLCodom_){
		CalculateAlleleFreqs();}//calculates twoNSLCodom_
	return twoNSLCodom_[pop][loc];
	}//end  SexLinkedCodomMarkerData::GetLocusSampleSize


int SexLinkedCodomMarkerData::GetLocusSampleSize (int pop, int indiv, int loc){
//	TextDataList& g = SexLinkedCodomMarkerData::GenotypicVal(pop,indiv,loc);
	int tr = locusNumSLCodom_[loc];
	TextDataList& g = SexLinkedCodomMarkerData::GenotypicVal(pop,indiv,tr);
	int c, n=0;
	for(c=0;c<ploidy(pop,indiv);++c){
		if(g[c].OK()) {++n;}}
	return n;
	}//end  SexLinkedCodomMarkerData::GetLocusSampleSize



long SexLinkedCodomMarkerData::GetLocusSampleSizeHeterogametic (int loc){//pooling populations
	if(! alleleFreqCurrentSLCodom_){
		CalculateAlleleFreqs();}//calculates NheterogameticPopLoc_
	long pop, numAlleleCopies = 0;
	for(pop=0;pop<npop_;++pop){
		numAlleleCopies += NheterogameticPopLoc_[pop][loc];
		}
	return numAlleleCopies;
	}//end  SexLinkedCodomMarkerData::GetLocusSampleSizeHeterogametic(int)


long SexLinkedCodomMarkerData::GetLocusSampleSizeHeterogametic (int pop, int loc){
	//using unskipped loci
	if(! alleleFreqCurrentSLCodom_){
		CalculateAlleleFreqs();}//calculates NheterogameticPopLoc_
	return NheterogameticPopLoc_[pop][loc];
	}//end  SexLinkedCodomMarkerData::GetLocusSampleSizeHeterogametic(int, int)




int	 SexLinkedCodomMarkerData::GetLocusSampleSizeHeterogametic (int pop, int indiv, int loc){
	//returns ploidy//using unskipped loci
	return GetLocusSampleSize(pop,indiv,loc);
	}//SexLinkedCodomMarkerData::GetLocusSampleSizeHeterogametic (int, int, int)



long SexLinkedCodomMarkerData::GetLocusSampleSizeHomogametic (int loc){//pooling populations
	if(! alleleFreqCurrentSLCodom_){
		CalculateAlleleFreqs();}//calculates twoNhomogameticPopLoc_
	long pop, numAlleleCopies = 0;
	for(pop=0;pop<npop_;++pop){
		numAlleleCopies += twoNhomogameticPopLoc_[pop][loc];
		}
	return numAlleleCopies;
	}//end  SexLinkedCodomMarkerData::GetLocusSampleSizeHomogametic(int)



long SexLinkedCodomMarkerData::GetLocusSampleSizeHomogametic (int pop, int loc){
	//using unskipped loci
	if(! alleleFreqCurrentSLCodom_){
		CalculateAlleleFreqs();}//calculates twoNhomogameticPopLoc_
	return twoNhomogameticPopLoc_[pop][loc];
	}//end  SexLinkedCodomMarkerData::GetLocusSampleSizeHomogametic(int, int)


int	 SexLinkedCodomMarkerData::GetLocusSampleSizeHomogametic (int pop, int indiv, int loc){
	//returns ploidy//using unskipped loci
	return GetLocusSampleSize(pop,indiv,loc);
	}//SexLinkedCodomMarkerData::GetLocusSampleSizeHomogametic (int, int, int)
	
	
	
long double SexLinkedCodomMarkerData::GetMeanAllelesPerLocus (int pop){
	long double meanAlleles = zero;
	int loc;
	for(loc=0;loc<nlociSLCodom_;++loc){
		meanAlleles += (long double)GetNumAlleles(pop,loc);}
	meanAlleles /= (long double)nlociSLCodom_;
	return meanAlleles;
	}//end  SexLinkedCodomMarkerData::GetMeanAllelesPerLocus


int SexLinkedCodomMarkerData::GetNumAlleles (int loc){
	//total in data set
	if(! allelesCurrentSLCodom_){CheckForNewAlleles();}
	StringList& locList = allelesPresentSLCodom_[loc];
	return locList.len();
	}//end  SexLinkedCodomMarkerData::GetNumAlleles(loc)


int SexLinkedCodomMarkerData::GetNumAlleles (int pop, int loc){
	//extant alleles in population
	if(! alleleFreqCurrentSLCodom_){CalculateAlleleFreqs();}
	ObjList<aList<long double> >& popList = alleleFreqSLCodom_[pop];
	aList<long double>& locList = popList[loc];
	int a,count=0;
	for(a=0;a<locList.len();++a){
		if (locList[a] >0.0){++count;}}
	return count;
	}//end  SexLinkedCodomMarkerData::GetNumAlleles(pop,loc)


long double SexLinkedCodomMarkerData::GetObservedHeterozygosity(int pop, int loc, int allele){
	if (!HetObsCurrentSL_){CalculateObservedHeterozygosity();}
	ObjList<aList<long double> >& popList = HetObsByAlleleSL_[pop];
	aList<long double>& locList = popList[loc];
	return locList[allele];
	}//end SexLinkedCodomMarkerData::GetObservedHeterozygosity(pop,loc,allele)


long double SexLinkedCodomMarkerData::GetObservedHeterozygosity(int pop, int loc){
	if (!HetObsCurrentSL_){CalculateObservedHeterozygosity();}
	aList<long double>& popList = HetObsByLocusSL_[pop];
	return popList[loc];
	}//end SexLinkedCodomMarkerData::GetObservedHeterozygosity(pop,loc)


long double SexLinkedCodomMarkerData::GetObservedHeterozygosity (int pop){
	//returns mean Hobs over loci
	StatList<long double> hetObs;
	int loc;
	for (loc=0;loc<nlociSLCodom_;++loc){
		hetObs.Append(GetObservedHeterozygosity(pop,loc));}
	return hetObs.mean();
	}//end SexLinkedCodomMarkerData::GetObservedHeterozygosity(pop)



long double SexLinkedCodomMarkerData::GetPercentPolymorphicLoci(int pop, long double criterion){
	//default criterion is 100.0% = any polymorphism
	long double mostCommon,percentP = zero;
	criterion /= (long double)100.0; 
	for(int loc=0;loc<nlociSLCodom_;++loc){
		mostCommon = GetFreqMostCommonAllele(pop,loc);
		if(mostCommon<criterion){++percentP;}}
	return (percentP/(long double)nlociSLCodom_) *(long double)100.0;
	}//end SexLinkedCodomMarkerData::GetPercentPolymorphicLoci(pop,loc)



aString* SexLinkedCodomMarkerData::GetTraitString(int pop, int indiv, int loc, int allele){
	//loc indexed as in locusNumCYfemale_ 
	aString *aStr = NEW aString();
	int tr = locusNumSLCodom_[loc];
	aStr->cat(rawTraits_[pop][indiv][tr][allele]);
	return aStr;
	}//CytoplasmicFemaleMarkerData::GetTraitString

long double SexLinkedCodomMarkerData::GetVarJackAllelesPerLocus (int pop){
	//returns jackknife variance of means, dropping loci
	StatList<long double> allelesPerLoc,jackAllelesPerLoc;
	int loc;
	long double storeValue;
	for (loc=0;loc<nlociSLCodom_;++loc){
		allelesPerLoc.Append((long double)GetNumAlleles(pop,loc));}
	for (loc=0;loc<nlociSLCodom_;++loc){
		storeValue = allelesPerLoc.First();
		allelesPerLoc.DeleteFirst();
		jackAllelesPerLoc.Append(allelesPerLoc.mean());
		allelesPerLoc.Append(storeValue);
		}
	return jackAllelesPerLoc.var();
	}//end SexLinkedCodomMarkerData::GetVarJackAllelesPerLocus(pop)



long double SexLinkedCodomMarkerData::GetVarJackExpectedHeterozygosity (int pop){
	//returns jackknife variance of means, dropping loci
	StatList<long double> hetExpLoc,jackHetExpLoc;
	int loc;
	long double storeValue;
	for (loc=0;loc<nlociSLCodom_;++loc){
		hetExpLoc.Append(GetExpectedHeterozygosity(pop, loc));}
	for (loc=0;loc<nlociSLCodom_;++loc){
		storeValue = hetExpLoc.First();
		hetExpLoc.DeleteFirst();
		jackHetExpLoc.Append(hetExpLoc.mean());
		hetExpLoc.Append(storeValue);
		}
	return jackHetExpLoc.var();
	}//end SexLinkedCodomMarkerData::GetVarJackExpectedHeterozygosity(pop)



long double SexLinkedCodomMarkerData::GetVarJackObservedHeterozygosity (int pop){
	//returns jackknife variance of means, dropping loci
	StatList<long double> hetObsLoc,jackHetObsLoc;
	int loc;
	long double storeValue;
	for (loc=0;loc<nlociSLCodom_;++loc){
		hetObsLoc.Append(GetObservedHeterozygosity(pop, loc));}
	for (loc=0;loc<nlociSLCodom_;++loc){
		storeValue = hetObsLoc.First();
		hetObsLoc.DeleteFirst();
		jackHetObsLoc.Append(hetObsLoc.mean());
		hetObsLoc.Append(storeValue);
		}
	return jackHetObsLoc.var();
	}//end SexLinkedCodomMarkerData::GetVarJackObservedHeterozygosity(pop)



long double SexLinkedCodomMarkerData::GetVarJackPercentPolyLoci (int pop, long double criterion){
	//returns jackknife variance of means, dropping loci
	StatList<long double> jackPercentP;
	int loc;
	double long temp;
	for (loc=0;loc<nlociSLCodom_;++loc){
		jackPercentP.Append(GetPercentPolymorphicLoci(pop,criterion));}
	for (loc=0;loc<nlociSLCodom_;++loc){
		temp = (jackPercentP[loc]/100.0)*nlociSLCodom_;
		if(IsPolymorphic(pop,loc,criterion)){--temp;}
		jackPercentP.Replace((temp/(long double)(nlociSLCodom_-1))*(long double)100.0,loc);
		}
	return jackPercentP.var();
	}//end SexLinkedCodomMarkerData::GetVarJackPercentPolyLoci(pop,criterion)




int SexLinkedCodomMarkerData::individuals(int pop, int loc){
	int i,indivs=0;
	for(i=0;i<individuals(pop);++i){
		if (GetLocusSampleSize ( pop, i, loc)>0){++indivs;}}
	return indivs;
	}//end  SexLinkedCodomMarkerData::individuals(int,int)
	


int  SexLinkedCodomMarkerData::IsPolymorphic(int pop, int loc){//boolean
	long double freq = GetFreqMostCommonAllele(pop,loc);
	if (freq==zero){return 0;}
	if (freq<one){return 1;}else {return 0;}
	}// end SexLinkedCodomMarkerData::IsPolymorphic


int  SexLinkedCodomMarkerData::IsPolymorphic(int pop, int loc, long double criterion){//boolean
	long double freq = GetFreqMostCommonAllele(pop,loc);
	if (freq==zero){return 0;}
	if (freq<(criterion/(long double)100.0)){return 1;}else {return 0;}
	}// end SexLinkedCodomMarkerData::IsPolymorphic


int  SexLinkedCodomMarkerData::IsPolymorphic(int loc){//boolean
	if(! allelesCurrentSLCodom_) {CheckForNewAlleles();}
	if (allelesPresentSLCodom_[loc].len()>1){ return 1;}
	return 0;
	}// end SexLinkedCodomMarkerData::IsPolymorphic


int  SexLinkedCodomMarkerData::IsPolymorphic(int loc, long double criterion){//boolean
	long double freq = GetFreqMostCommonAllele(loc);
	if (freq<(criterion/(long double)100.0)){return 1;}else {return 0;}
	}// end SexLinkedCodomMarkerData::IsPolymorphic


long SexLinkedCodomMarkerData::maxall(void){
	//maximum alleles per locus in data set
	CheckForNewAlleles();
	long loc, maxalleles=0;
	for(loc=0;loc<nlociSLCodom_;++loc){
		maxalleles = MAX(maxalleles,allelesPresentSLCodom_[loc].len());}
	return maxalleles;
	}//SexLinkedCodomMarkerData::maxall()


void SexLinkedCodomMarkerData::ParseGenotypicData(int /*startingLocus*/){
//void SexLinkedCodomMarkerData::ParseGenotypicData(aList<int>& codominantLoci,int startingLocus){
	//convert from raw trait data
	//assumes 
	rawSLCodomGtypes_.Clear();
//	if(codominantLoci.len()==0) return;
	ResetFlags();
	int pop, loc, i, c, sLoc, item,tr;
//	codominantLoci.Sort();
//	for(loc=0;loc<codominantLoci.len();++loc){
//		codominantLoci[loc]+=startingLocus;}
//	allSLCodomLoci_=locusNumSLCodom_=codominantLoci;
//	nlociSLCodom_=codominantLoci.len();
	
	for(pop=0;pop<npop_;++pop){//parse skipped loci too
		rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
		for(i=0;i<individuals(pop);++i){
			rawSLCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
			for(loc=0;loc<nlociSLCodom_;++loc){
				tr = locusNumSLCodom_[loc];
				TextDatum& genotype = rawTraits_[pop][i][tr];
				rawSLCodomGtypes_[pop][i].Append(NEW TextDataList());
				for(c=0;c<ploidy_;++c){
					if(genotype.missing() || c>=genotype.len()){//append all missing data flags
							rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum());}
						else{//parse the genotype and handle each allele copy separately
							aString gene(genotype,c*alleleLen_+1,(c+1)*alleleLen_);//check this
							aString genecopy(gene);
							gene.CullLeadingWhiteSpace();
							gene.CullTrailingWhiteSpace();
							if(gene.len()==0 || missing_.Includes(&gene)){
									rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genecopy,true));}//save verbatim but score as missing
								else{
									rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genecopy));}
							}
					}//c
				}//loc
			}//i
		}//pop
	for(sLoc=0;sLoc<skippedTraitNum_.len();++sLoc){
		if(locusNumSLCodom_.Includes(skippedTraitNum_[sLoc])){
			item = locusNumSLCodom_.Loc(skippedTraitNum_[sLoc]);
			locusNumSLCodom_.Delete(item);
			--nlociSLCodom_;
			if(! skippedLocusNumSLCodom_.Includes(skippedTraitNum_[sLoc])){
				skippedLocusNumSLCodom_.Append(skippedTraitNum_[sLoc]);}
			}
		}//sLoc
	skippedLocusNumSLCodom_.Sort();
	CheckForNewAlleles();
	}//SexLinkedCodomMarkerData::ParseGenotypicData(aList<int>& )
	


int SexLinkedCodomMarkerData::Ploidy(int pop, int indiv, int /*loc*/){
	if(heterogametic(pop,indiv)) {return ploidy_/2;}
	return ploidy_;
	}//SexLinkedCodomMarkerData::Ploidy(int, int, int)



void  SexLinkedCodomMarkerData::PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of allelic frequencies for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculateAlleleFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied


	outFile<<title_<<endl<<endl<<endl;
	outFile<<"ALLELE FREQUENCY DATA"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop_<<endl;
	outFile<<"NUMBER OF LOCI SURVEYED = ";outFile.width(3);outFile<<nlociSLCodom_<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (skippedLocusNumSLCodom_.len() == 0){
			outFile<<colSpacer<<"none";}
		else{
			outFile<<colSpacer;
			for (int loc=0;loc<skippedLocusNumSLCodom_.len();++loc){
				aString& locName = traitName_[skippedLocusNumSLCodom_[loc]];
				outFile<<locName;
				if (loc < skippedLocusNumSLCodom_.len()-1){outFile<<colSpacer;}
				}//end for loc
		}//end if lociSkipped
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop_ / ncols;
	int npop0 = npop_;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Allele frequencies for SEX-LINKED CODOMINANT markers for each population."<<endl;
				outFile<<"Number of alleles sampled in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop_, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;

		for (int loc=0;loc<nlociSLCodom_;++loc) {//print the locus and locus sample size
			aString& locWord = traitName_[locusNumSLCodom_[loc]];
			outFile<< locWord;
			if (asciiFormat){outFile.width(11 + (5 - locWord.len()));}else{outFile<<colSpacer;}
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
//				int samSize = GetLocusSampleSize(pop, loc) / ploidy_;
				int samSize = GetLocusSampleSize(pop, loc);
				outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
				if (pop < endPop){
					if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
					}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresentSLCodom_[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					long double freq = GetFreq(pop, loc, allele);
					aString freqStr(freq,0,4);
					if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									if (asciiFormat){
											aString blanks(' ',12);
											if (pop==x) {
												blanks.PrependCharToLen(' ',17);}
											outFile<<blanks;
											}
										else{outFile<<colSpacer;}
									}
								else{
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end SexLinkedCodomMarkerData::PrintAlleleFreqs


void  SexLinkedCodomMarkerData::PrintAlleleFreqsBySex (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of allelic frequencies by sex for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculateAlleleFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied


	outFile<<title_<<endl<<endl<<endl;
	outFile<<"ALLELE FREQUENCY DATA SORTED BY SEX"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop_<<endl;
	outFile<<"NUMBER OF LOCI SURVEYED = ";outFile.width(3);outFile<<nlociSLCodom_<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (lociSkipped() == 0){
			outFile<<"none";}
		else{
			for (int loc=0;loc<skippedLocusNumSLCodom_.len();++loc){
				aString& locName = traitName_[skippedLocusNumSLCodom_[loc]];
				outFile<<locName;
				if (loc < skippedLocusNumSLCodom_.len()-1){outFile<<", ";}
				}//end for loc
		}//end if lociSkipped
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop_ / ncols;
	int npop0 = npop_;
	if (MOD(npop0,ncols) > 0){++pFit;}
	int samSize;
	long double freq;
	aString mWord("M"),fWord("F"), sexWord;
	
	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Allele frequencies by sex for SEX-LINKED CODOMINANT markers for each population."<<endl;
				outFile<<"Number of alleles sampled in brackets for each locus and sex."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop_, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;
		for (int loc=0;loc<nlociSLCodom_;++loc) {//print the locus, and locus sample size BY SEX
			aString& locWord = traitName_[locusNumSLCodom_[loc]];
			outFile<< locWord<<endl;
			for(int sex=0;sex<2;++sex){
				if(sex==0) {sexWord=fWord;}else{sexWord=mWord;}
				outFile<<sexWord;
				if (asciiFormat){outFile.width(11 + (5 - sexWord.len()));}else{outFile<<colSpacer;}
				for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
					samSize=0;
					if(sex==0){//F
							if(heterogameticSex()==F){
									samSize = GetLocusSampleSizeHeterogametic(pop, loc);}
								else{
									samSize = GetLocusSampleSizeHomogametic(pop, loc);}
							}
						else{//M
							if(heterogameticSex()==M){
									samSize = GetLocusSampleSizeHeterogametic(pop, loc);}
								else{
									samSize = GetLocusSampleSizeHomogametic(pop, loc);}
							}
					outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
					if (pop < endPop){
						if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
						}
					}//end for pop
				outFile<<endl;

				//print the allelic letter designation on the left side of the page
				StringList& alleleNameList = allelesPresentSLCodom_[loc];
				for (int allele=0; allele<alleleNameList.len();++allele){
					aString& allName=alleleNameList[allele];
					aString allNameCopy(allName);
					allNameCopy.PrependCharToLen(' ',3);
					outFile<<allNameCopy;
					// print the frequency of the allele for this locus, or
					//a blank space if the frequency is zero
					for (int pop=x;pop<=endPop;++pop){
						freq=(long double)0.0;
						if(sex==0){//F
								if(heterogameticSex()==F){
										freq = GetFreqHeterogametic(pop, loc, allele);}
									else{
										freq = GetFreqHomogametic(pop, loc, allele);}
								}
							else{//M
								if(heterogameticSex()==M){
										freq = GetFreqHeterogametic(pop, loc, allele);}
									else{
										freq = GetFreqHomogametic(pop, loc, allele);}
								}
						aString freqStr(freq,0,4);
						if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
						if (pop == endPop){//just write the freq if >0
								if (freq > 0.0){
										if (pop==x) {
											if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
												//else{freqStr.prepend(colSpacer);}
											}
									 	outFile<<freqStr<<endl;}
									else{outFile<<endl;}
								}
							else	{//pop<endpop; include spaces if freq=0}
								if (freq == 0.0){
										if (asciiFormat){
												aString blanks(' ',12);
												if (pop==x) {
													blanks.PrependCharToLen(' ',17);}
												outFile<<blanks;
												}
											else{outFile<<colSpacer;}
										}
									else{
										if (pop==x) {
											if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
												//else{freqStr.prepend(colSpacer);}
											}
										outFile<<freqStr;}
								}//end if pop...else
						}//end for pop
					}//end for allele
				}//sex
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end SexLinkedCodomMarkerData::PrintAlleleFreqsBySex




void  SexLinkedCodomMarkerData::PrintAlleleFreqsByPopList (ostream& outFile, 
											ObjList<ObjList<aList<double> > >& popFreqList, 
											ObjList<aList<double> >& popTwoNList, 
											int ncols){
	//popFreqList must contain the entire set of alleles in the data set
	//Sends a table of allelic frequencies to the output file.
	//ncols indexes the number of population columns per page

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = popFreqList.len() / ncols;
	int npop0 = popFreqList.len();
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Number of alleles sampled in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Allele frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)popFreqList.len(), (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			outFile.width(12);outFile<<pop+1;}
		outFile<<endl;

		for (int loc=0;loc<nlociSLCodom_;++loc) {//print the locus and locus sample size
			aString& locWord = traitName_[locusNumSLCodom_[loc]];
			outFile<< locWord;outFile.width(11 + (5 - locWord.len()));
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
//				int samSize = popTwoNList[pop][loc] / ploidy_;
				int samSize = popTwoNList[pop][loc];
				outFile<<"[";outFile.width(2);outFile<<samSize<<"]";
				if (pop < endPop){outFile.width(9);}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresentSLCodom_[loc];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double freq = popFreqList[pop][loc][allele];
					aString freqStr(freq,0,4);
					freqStr.PrependCharToLen(' ',12);
					if (pop == endPop){//just write the freq if >0
							if (freq > 0.0){
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (freq == 0.0){
									aString blanks(' ',12);
									if (pop==x) {
										blanks.PrependCharToLen(' ',17);}
									outFile<<blanks;}
								else{
									if (pop==x) {
										freqStr.PrependCharToLen(' ',17);}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end SexLinkedCodomMarkerData::PrintAlleleFreqsByPopList



void	 SexLinkedCodomMarkerData::PrintGelJockFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintGelJockFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end SexLinkedCodomMarkerData::PrintGelJockFile(aString&)



void SexLinkedCodomMarkerData::PrintGelJockFile (ostream& outFile){
	aString dataLine = title_;
	dataLine.cat(":  skipped loci omitted");
	outFile<<dataLine<<endl;
	dataLine = "NPOP=";
	dataLine.cat(npop_);
	dataLine.cat(", NLOCI=");
	dataLine.cat(nlociSLCodom_);
	dataLine.cat(", MAXALL=");
	dataLine.cat(maxall());
	dataLine.cat(", NGROUPS=");
	dataLine.cat(ngroups_);
	outFile<<dataLine<<endl;
	dataLine.clear();
	int pop,i,loc,c,tr;
	for (loc=0;loc<locusNumSLCodom_.len();++loc){
		dataLine.cat(traitName_[locusNumSLCodom_[loc]]);
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (loc+1 < locusNumSLCodom_.len()){
				dataLine.cat(", ");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	dataLine = "SKIP=0";
	outFile<<dataLine<<endl;
	for (pop=0;pop<npop_;++pop){
		outFile<<popName_[pop]<<endl;
		for (i=0;i<individuals(pop);++i){
			dataLine.clear();
			dataLine.cat(rawTraits_[pop][i][0]);//individual's ID code is the first trait
			dataLine+" ";
			dataLine + rawTraits_[pop][i][sexTrait_];//individual's gender
			for(loc=0;loc<nlociSLCodom_;++loc){
				tr = locusNumSLCodom_[loc];
				dataLine+' ';
				for(c=0;c<ploidy_/2;++c){
//					dataLine + rawSLCodomGtypes_[pop][i][loc][c];
					dataLine + SexLinkedCodomMarkerData::GenotypicVal(pop,i,tr,c);
					}
				for(c=ploidy_/2;c<ploidy_;++c){
					if(homogametic(pop,i)){
//							dataLine + rawSLCodomGtypes_[pop][i][loc][c];
							dataLine + SexLinkedCodomMarkerData::GenotypicVal(pop,i,tr,c);
							}
						else{
							dataLine+" ";}//spacers
					}//c
				if(loc<nlociSLCodom_-1){
					dataLine+" ";}
				}//loc
			outFile<<dataLine<<endl;
			}//end for i
		outFile<<"NEXT"<<endl;
		}//end for pop
	outFile<<"END"<<endl<<endl;
	}//end SexLinkedCodomMarkerData::PrintGelJockFile


void	 SexLinkedCodomMarkerData::PrintArlequinFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintArlequinFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end SexLinkedCodomMarkerData::PrintArlequinFile(aString&)



void SexLinkedCodomMarkerData::PrintArlequinFile (ostream& outFile){
	//		theGenotypes: ListStrStructureTypePtr;
	outFile<<"[Profile]"<<endl;
	aString dataLine = "\tTitle=\"";
	dataLine.cat(title_);
	dataLine.cat('\"');
	outFile<<dataLine<<endl;
	dataLine = "\t\tNbSamples=";
	dataLine.cat(npop_);
	outFile<<dataLine<<endl;
	dataLine = "\t\tGenotypicData=1";
	outFile<<dataLine<<endl;
	dataLine = "\t\tGameticPhase=0";
	outFile<<dataLine<<endl;
	dataLine = "\t\tRecessiveData=0";
	outFile<<dataLine<<endl;
	dataLine = "\t\tIndividualLevel=1";
	outFile<<dataLine<<endl;
	dataLine = "\t\tMissingData=\"\?\"";
	outFile<<dataLine<<endl;
	dataLine = "\t\tCompDistMatrix=1";
	outFile<<dataLine<<endl<<endl;

	dataLine = "\t\tDataType=STANDARD";
	outFile<<dataLine<<endl;
	dataLine = "\t\tFrequency=ABS";
	outFile<<dataLine<<endl;
	dataLine = "\t\tLocusSeparator=WHITESPACE";
	outFile<<dataLine<<endl<<endl;
		
	dataLine = "[Data]";
	outFile<<dataLine<<endl<<endl;
	dataLine = "#Locus names:";
	outFile<<dataLine<<endl;

	dataLine= "\t# ";
	for (int loc=0;loc<locusNumSLCodom_.len();++loc){
		dataLine.cat(traitName_[locusNumSLCodom_[loc]]);
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine= "\t# ";}
			else if (loc+1 < locusNumSLCodom_.len()){
				dataLine.cat("\t");}
			else{
				outFile<<dataLine<<endl;
				dataLine= "\t# ";}
		}//end for loc
	outFile<<endl<<endl;
	dataLine = "[[Samples]]";
	outFile<<dataLine<<endl<<endl;
	dataLine.clear();
	int pop,i,loc,tr,c,gr;
	for (pop=0;pop<npop_;++pop){
		dataLine = "\t\tSampleName=\"";
		dataLine.cat(popName_[pop]);
		dataLine.cat('\"');
		outFile<<dataLine<<endl;
		dataLine = "\t\tSampleSize=";
		dataLine.cat(individuals(pop));
		outFile<<dataLine<<endl;
		dataLine = "\t\tSampleData={";
		outFile<<dataLine<<endl;
		StringList pseudohaps;
		for (i=0;i<individuals(pop);++i){
			pseudohaps.Clear();
			//split each genotype into ploidy_ pieces, = "pseudohaplotypes", on separate rows
			pseudohaps.Append(NEW aString(rawTraits_[pop][i][0]));//ID code
			pseudohaps[0].cat(" 1 ");//frequency of this genotype
			for(c=1;c<ploidy(pop,i);++c){
				pseudohaps.Append(NEW aString());
				pseudohaps[c].PrependCharToLen(' ',pseudohaps[0].len());}
			for(loc=0;loc<nlociSLCodom_;++loc){
				tr=locusNumSLCodom_[loc];
				for(c=0;c<ploidy(pop,i);++c){
//					TextDatum& allele = rawSLCodomGtypes_[pop][i][locusNumSLCodom_[loc]][c];
					TextDatum& allele = SexLinkedCodomMarkerData::GenotypicVal(pop,i,tr,c);
					if(allele.OK()){
							pseudohaps[c].cat(allele);}
						else{//missing data 
							pseudohaps[c].cat('\?');}
					pseudohaps[c].cat(' ');
					}//c
				}//loc
			pseudohaps.Print(outFile);
			}//end for i
		outFile<<"\t\t}"<<endl<<endl;
		}//end for pop
	outFile<<endl<<endl;
	dataLine = "[[Structure]]";
	outFile<<dataLine<<endl<<endl;

	dataLine = "StructureName = \"Structure from population titles\"";
	outFile<<dataLine<<endl;
	dataLine = "NbGroups = 1";
	outFile<<dataLine<<endl;
	dataLine = "IndividualLevel = 1";
	outFile<<dataLine<<endl;
	for(gr=0;gr<ngroups_;++gr){
		dataLine = "\t#";
		dataLine.cat(groupName_[gr]);
		outFile<<dataLine<<endl;
		dataLine = "\tGroup =  {";
		outFile<<dataLine<<endl;
		for(pop=0;pop<groupHierarchy_[gr].len();++pop){
			outFile<<"\t\t\""<<popName_[groupHierarchy_[gr][pop]]<<'\"'<<endl;
			}
		outFile<<"\t\t}"<<endl;
	}


	outFile<<endl;
	}//end SexLinkedCodomMarkerData::PrintArlequinFile





void  SexLinkedCodomMarkerData::PrintGenotypeCounts (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of genotypic counts for each population to the output file.
	//nCols indexes the number of population columns per page
	CalculateGenotypeFreqs();
	int asciiFormat = 1;//default: align columns with font of constant width
	if (colSpacer != ' '){
		asciiFormat = 0;}//separate columns using the spacer supplied


	outFile<<title_<<endl<<endl<<endl;
	outFile<<"GENOTYPE FREQUENCY DATA"<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop_<<endl;
	outFile<<"NUMBER OF LOCI SURVEYED = ";outFile.width(3);outFile<<nlociSLCodom_<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (lociSkipped() == 0){
			outFile<<"none";}
		else{
			for (int loc=0;loc<skippedLocusNumSLCodom_.len();++loc){
				aString& locName = traitName_[skippedLocusNumSLCodom_[loc]];
				outFile<<locName;
				if (loc < skippedLocusNumSLCodom_.len()-1){outFile<<", ";}
				}//end for loc
		}//end if lociSkipped
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop_ / ncols;
	int npop0 = npop_;
	if (MOD(npop0,ncols) > 0){++pFit;}
	int pop,loc,allele1,allele2;
	int countfreq;
	
	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				outFile<<"Genotype frequencies for SEX-LINKED CODOMINANT markers for each population."<<endl;
//				outFile<<"Number of alleles sampled in brackets for each locus."<<endl;
				}
			else{
				outFile<<"Genotype frequency table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop_, (ncols * (block+1)))-1;
		outFile<<"pop. ";
		for (pop=x;pop<=endPop;++pop){ //indexes the population number
			if (asciiFormat){outFile.width(12);} else{outFile<<colSpacer;}
			outFile<<pop+1;
			}
		outFile<<endl;

		for (loc=0;loc<nlociSLCodom_;++loc) {//print the locus and locus sample size
			aString& locWord = traitName_[locusNumSLCodom_[loc]];
			outFile<< locWord;
//			if (asciiFormat){outFile.width(11 + (5 - locWord.len()));}else{outFile<<colSpacer;}
//			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
//				int samSize = GetLocusSampleSize(pop, loc);
//				outFile<<"[";if (asciiFormat){outFile.width(2);}outFile<<samSize<<"]";
//				if (pop < endPop){
//					if (asciiFormat){outFile.width(9);}else{outFile<<colSpacer;}
//					}
//				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresentSLCodom_[loc];
			for (allele1=0; allele1<alleleNameList.len();++allele1){//heterogametic genotypes first
				aString& allName1=alleleNameList[allele1];
				aString allName1Copy(allName1);
				allName1Copy.PrependCharToLen(' ',3);
				outFile<<allName1Copy;
				for (pop=x;pop<=endPop;++pop){
					countfreq = GetGenotypeCountHeterogametic(pop, loc, allele1);
					aString freqStr(countfreq);
					if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
					if (pop == endPop){//just write the freq if >0
							if (countfreq > 0){
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
								 	outFile<<freqStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if freq=0}
							if (countfreq == 0){
									if (asciiFormat){
											aString blanks(' ',12);
											if (pop==x) {
												blanks.PrependCharToLen(' ',17);}
											outFile<<blanks;
											}
										else{outFile<<colSpacer;}
									}
								else{
									if (pop==x) {
										if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
											//else{freqStr.prepend(colSpacer);}
										}
									outFile<<freqStr;}
							}//end if pop...else
					}//end for pop
				}//allele1

			for (allele1=0; allele1<alleleNameList.len();++allele1){//homogametic genotypes next
				aString& allName1=alleleNameList[allele1];
				aString allName1Copy(allName1);
				allName1Copy.PrependCharToLen(' ',3);
				for (allele2=allele1; allele2<alleleNameList.len();++allele2){
					aString& allName2=alleleNameList[allele2];
					aString gtypeName(allName1Copy);
					gtypeName +=allName2;
					// print the frequency of the allele for this locus, or
					//a blank space if the frequency is zero
					outFile<<gtypeName;
					for (int pop=x;pop<=endPop;++pop){
						countfreq = GetGenotypeCountHomogametic(pop, loc, allele1, allele2);
						aString freqStr(countfreq);
						if (asciiFormat){freqStr.PrependCharToLen(' ',12);}else{freqStr.prepend(colSpacer);}
						if (pop == endPop){//just write the freq if >0
								if (countfreq > 0){
										if (pop==x) {
											if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
												//else{freqStr.prepend(colSpacer);}
											}
									 	outFile<<freqStr<<endl;}
									else{outFile<<endl;}
								}
							else	{//pop<endpop; include spaces if freq=0}
								if (countfreq == 0){
										if (asciiFormat){
												aString blanks(' ',12);
												if (pop==x) {
													blanks.PrependCharToLen(' ',17);}
												outFile<<blanks;
												}
											else{outFile<<colSpacer;}
										}
									else{
										if (pop==x) {
											if (asciiFormat){freqStr.PrependCharToLen(' ',17);}
												//else{freqStr.prepend(colSpacer);}
											}
										outFile<<freqStr;}
								}//end if pop...else
						}//end for pop
					}//end for allele2
				}//allele1
			}//end for loc
		outFile<<endl<<endl;
		}//end for block

	}//end SexLinkedCodomMarkerData::PrintGenotypeCounts




void SexLinkedCodomMarkerData::PrintHeterozygosity (ostream& outFile, const int hetType, int ncols){
	//file assumed open
	//Sends a table of the designated heterozygosities for each population to the output file.
	//ncols indexes the number of population columns per page
	outFile<<title_<<endl<<endl<<endl;
	if (hetType == observed) {
			aString titlestr("OBSERVED HETEROZYGOSITY TABLE: based on ");
			if(heterogameticSex_==M) {titlestr+"females only";}
				else {titlestr+"males only";}
			outFile<<titlestr;}
		else{
			outFile<<"EXPECTED HETEROZYGOSITY TABLE: based on allele frequencies from both sexes";}
	outFile<<endl<<endl;
	outFile<<"NUMBER OF POPULATIONS SURVEYED = ";outFile.width(3);outFile<<npop_<<endl;
	outFile<<"NUMBER OF LOCI SURVEYED = ";outFile.width(3);outFile<<nlociSLCodom_<<endl;
	outFile<<"LOCI SKIPPED : ";
	if (lociSkipped() == 0){
			outFile<<"none";}
		else{
			for (int loc=0;loc<lociSkipped();++loc){
				aString& locName = traitName_[skippedLocusNumSLCodom_[loc]];
				outFile<<locName;
				if (loc < lociSkipped()-1){outFile<<", ";}
				}//end for loc
		}//end if lociSkipped
	outFile<<endl;

	//print the table with populations as columns, loci and alleles as rows

	//PFIT it the total number of population blocks the table is divided into
	int pFit = npop_ / ncols;
	int npop0 = npop_;
	if (MOD(npop0,ncols) > 0){++pFit;}

	for (int block=0;block<pFit;++block){
		//M indexes the particular population block
		if (block == 0){ 
				if (hetType == expected) {
						outFile<<"Expected ";}
					else{
						outFile<<"Observed ";}
				outFile<<"heterozygosities for each allele in each population."<<endl;
				outFile<<"Sample sizes in brackets for each locus."<<endl;
				}
			else{
				if (hetType == expected) {
						outFile<<"Expected ";}
					else{
						outFile<<"Observed ";}
				outFile<<"heterozygosity table, continued."<<endl<<endl;}
		int x =  (ncols * (block));//x is the population that starts this row
		int endPop = MIN((int)npop_, (ncols * (block+1)))-1;
		outFile<<"pop. ";		
		for (int pop=x;pop<=endPop;++pop){ //indexes the population number
			outFile.width(12);outFile<<pop+1;}
		outFile<<endl;

		for (int loc=0;loc<nlociSLCodom_;++loc) {//print the locus and locus sample size
			aString& locWord = traitName_[locusNumSLCodom_[loc]];
			outFile<< locWord;outFile.width(11 + (5 - locWord.len()));
			int samSize;
			for (int pop=x;pop<=endPop;++pop){//print the locus sample sizes in brackets
				if(hetType==observed){
						samSize = GetLocusSampleSizeHomogametic(pop, loc) / ploidy_;}
					else{
						samSize = GetLocusSampleSize(pop, loc) / ploidy_;}
				outFile<<"[";outFile.width(2);outFile<<samSize<<"]";
				if (pop < endPop){
					outFile.width(9);}
				}//end for pop
			outFile<<endl;

			//print the allelic letter designation on the left side of the page
			StringList& alleleNameList = allelesPresentSLCodom_[locusNumSLCodom_[loc]];
			for (int allele=0; allele<alleleNameList.len();++allele){
				aString& allName=alleleNameList[allele];
				aString allNameCopy(allName);
				allNameCopy.PrependCharToLen(' ',3);
				outFile<<allNameCopy;
				// print the frequency of the allele for this locus, or
				//a blank space if the frequency is zero
				for (int pop=x;pop<=endPop;++pop){
					double hets=0.0;
					if (hetType == observed) {
							hets = GetObservedHeterozygosity(pop, loc, allele);}
						else{
							hets = GetExpectedHeterozygosity(pop, loc, allele);}
					aString hetStr(hets,0,4);
					hetStr.PrependCharToLen(' ',12);
					if (pop == endPop){//just write the freq if >0
							if (hets > 0.0){
									if (pop==x) {
										hetStr.PrependCharToLen(' ',17);}
								 	outFile<<hetStr<<endl;}
								else{outFile<<endl;}
							}
						else	{//pop<endpop; include spaces if hets=0}
							if (hets == 0.0){
									aString blanks(' ',12);
									if (pop==x) {
										blanks.PrependCharToLen(' ',17);}
									outFile<<blanks;}
								else{
									if (pop==x) {
										hetStr.PrependCharToLen(' ',17);}
									outFile<<hetStr;}
							}//end if pop...else
					}//end for pop
				}//end for allele
			}//end for loc
		outFile<<endl<<endl;
		}//end for block
	}//end SexLinkedCodomMarkerData::PrintHeterozygosity



void SexLinkedCodomMarkerData::PrintLoci (ostream& outFile, int lociPerLine){//file assumed open
	int loc,lineSpot = 0;
	for (loc=0;loc<locusNumSLCodom_.len();++loc){ 
		aString& locName = traitName_[locusNumSLCodom_[loc]];
		outFile<<locName;
		++lineSpot;
		if ((loc == locusNumSLCodom_.len()-1) || (lineSpot == lociPerLine)) {
				outFile<<endl;}
			else{
				outFile<<", ";}
		}//end for loc
	outFile<<endl;
	}//end SexLinkedCodomMarkerData::PrintLoci





void	 SexLinkedCodomMarkerData::PrintSASFormatData(ostream& outFile, char colSpacer){
	//prints a data set that can be read using SAS
	//missing data represented by '.', columns separated by spaces
	//possible alleles for each locus in separate columns

	int loc,pop,i,gamete,allele,tr;
	//this and next loop comprise the column titles
	outFile<<"pop"<<colSpacer<<"indiv"<<colSpacer<<"gamete"<<colSpacer;
	for(loc=0;loc<nlociSLCodom_;++loc){
		aString& locName=traitName_[locusNumSLCodom_[loc]];
		StringList& alleleList=allelesPresentSLCodom_[loc];
		for(allele=0;allele<alleleList.len();++allele){
			aString alleleTitle(locName);
			alleleTitle.cat('_');
			alleleTitle.cat(alleleList[allele]);
			alleleTitle.cat(colSpacer);
			outFile<<alleleTitle;}
		}//end for loc
	outFile<<endl;
	for(pop=0;pop<npop_;++pop){
		aString popStr(pop+1);
		popStr.cat(colSpacer);
		for(i=0;i<individuals(pop);++i){
			aString indivStr(popStr);
			indivStr.cat(i+1);
			indivStr.cat(colSpacer);
			for (gamete=0;gamete<ploidy_;++gamete){
				aString genotypeStr(indivStr);
				genotypeStr.cat(gamete+1);
				genotypeStr.cat(colSpacer);
				for(loc=0;loc<nlociSLCodom_;++loc){
					tr=locusNumSLCodom_[loc];
					StringList& alleleList=allelesPresentSLCodom_[loc];
					for(allele=0;allele<alleleList.len();++allele){
						aString& alleleName = alleleList[allele];
						if(homogametic(pop,i) || gamete<ploidy_/2 ){
//								TextDatum& gene = SexLinkedCodomMarkerData::GenotypicVal(pop, i, loc, gamete);
								TextDatum& gene = SexLinkedCodomMarkerData::GenotypicVal(pop, i, tr, gamete);
								if(gene.missing()){
										genotypeStr.cat('.');}
									else if(gene==alleleName){
										genotypeStr.cat('1');}
									else{
										genotypeStr.cat('0');}
								}
							else{//missing for heterogametic sex
								genotypeStr.cat('.');}
						genotypeStr.cat(colSpacer);
						}//end for allele
					}//end for loc
				outFile<<genotypeStr<<endl;
				}//end for gamete
			}//end for i
		}//end for pop
	outFile<<endl;		
	}//end  SexLinkedCodomMarkerData::PrintSASFormatData(ostream&,char)
	
	
void	 SexLinkedCodomMarkerData::PrintSASFormatData(aString& outFileName, char colSpacer){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintSASFormatData(dataFile,colSpacer);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end SexLinkedCodomMarkerData::PrintSASFormatData(aString&,char)



void SexLinkedCodomMarkerData::PrintVariabilityScores(aString& outFileName, char sepChar){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintVariabilityScores(dataFile, sepChar);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end SexLinkedCodomMarkerData::PrintVariabilityScores(aString&, char)



void SexLinkedCodomMarkerData::PrintVariabilityScores(ostream& outFile, char sepChar){
	//This calls for subroutines which calculate standard genetic variability measures:
	//>  "POLYLOCI" =  % polymorphic loci and maan number of alleles per locus
	//>  "HETEXPECTED" = heterozygosity expected from Hardy-Weinberg ratios,
	//         with a jackknife variance calculation
	// >  "HETOBSERVED" = heterozygosity obtained directly from genotype frequencies,
	//         with jackknife variance calculation
	// >  And a summary table of these values for each population.

	// The following is the summary table
	// The populations are printed out by number and name, with
	// only the first 20 characters of the name used.
	int pop,charsInPrintedPopName = 10;
	long double p,stErr, ninety5=(long double)95.0,ninety9=(long double)99.0,hundred=(long double)100.0;
	// title_ and column headings
	outFile<<endl<<endl;
	outFile<<"Genetic variability scores for all populations."<<endl;
	outFile<<"(Standard errors in parentheses.)"<<endChr<<endl;
	outFile<<"Observed heterozygosties are based on homogematic sex only."<<endChr<<endl;
	aString headings = " # ";
	headings.separator(1, sepChar);                 					 //   5 characters
	headings.cat("population"); headings.separator(4, sepChar);            // 13 characters
	headings.cat("alleles/loc"); headings.separator(4, sepChar);           // 13 characters
	headings.cat("%P(all)"); headings.separator(7, sepChar);            	 // 13 characters
	headings.cat("%P(99%)"); headings.separator(7, sepChar);            // 13 characters
	headings.cat("%P(95%)"); headings.separator(8, sepChar);            // 13 characters
	headings.cat("H (obs)"); headings.separator(9, sepChar);            	 // 16 characters
	headings.cat("H (exp)");                                				 // 16 characters
	outFile<<headings<<endl;
	for(pop=0;pop<npop_;++pop){
		// population number
		aString outLine(pop+1);
		outLine.PrependCharToLen(' ', MAX(2,Digits(npop_)));
    		outLine.separator(1, sepChar); 

       		aString popStr(popName_[pop]);
        	popStr.trunc(charsInPrintedPopName);
		popStr.separator(charsInPrintedPopName - popStr.len() + 3, sepChar);

		//mean number of alleles per locus
		aString meanAllPerLoc(GetMeanAllelesPerLocus(pop),2,2);
		popStr.cat(meanAllPerLoc);
		popStr.cat(" (");
		stErr = sqrt(GetVarJackAllelesPerLocus(pop));
		aString varAllPerLoc(stErr,1,2);
		popStr.cat(varAllPerLoc);
		popStr.cat(")");popStr.separator(1, sepChar); 

		// %P detectable
		p = GetPercentPolymorphicLoci(pop, hundred);
		aString percentP(p,3,1);
		popStr.cat(percentP); popStr.cat(" (");
		stErr = sqrt(GetVarJackPercentPolyLoci(pop, hundred) );
		aString varPercentP(stErr,2,1);
		popStr.cat(varPercentP);
		popStr.cat(")");popStr.separator(2, sepChar); 

		// %P at 99%
		p = GetPercentPolymorphicLoci(pop, ninety9);
		aString percentP99(p,3,1);
		popStr.cat(percentP99); popStr.cat(" (");
		stErr = sqrt(GetVarJackPercentPolyLoci(pop, ninety9) );
		aString varPercentP99(stErr,2,1);
		popStr.cat(varPercentP99);
		popStr.cat(")");popStr.separator(3, sepChar); 

		// %P at 95%
		p = GetPercentPolymorphicLoci(pop, ninety5);
		aString percentP95(p,3,1);
		popStr.cat(percentP95); popStr.cat(" (");
		stErr = sqrt(GetVarJackPercentPolyLoci(pop,ninety5) );
		aString varPercentP95(stErr,2,1);
		popStr.cat(varPercentP95);
		popStr.cat(")");popStr.separator(3, sepChar); 

		// Hobs with standard error in parentheses
		aString meanHetObs(GetObservedHeterozygosity(pop),1,3);
		popStr.cat(meanHetObs);
		popStr.cat(" (");
		stErr = sqrt(GetVarJackObservedHeterozygosity(pop));
		aString varHetObs(stErr,1,3);
		popStr.cat(varHetObs);
		popStr.cat(")");popStr.separator(3, sepChar); 

		// Hexp with standard error in parentheses
		aString meanHetExp(GetExpectedHeterozygosity(pop),1,3);
		popStr.cat(meanHetExp);
		popStr.cat(" (");
		stErr = sqrt(GetVarJackExpectedHeterozygosity(pop));
		aString varHetExp(stErr,1,3);
		popStr.cat(varHetExp);
		popStr.cat(")");

		outLine.cat(popStr);
		outFile<<outLine<<endl;
		}// end for pop

	outFile<<endl<<"Variances computed using a jackknife algorithm, dropping loci."<<endl<<endl;
	}//end  SexLinkedCodomMarkerData::PrintVariabilityScores



int SexLinkedCodomMarkerData::ReadArlequinDataFile (aString* dataFileName){
	fstream dataFile;	
	char *dataLine=NEW char[10001];dataLine[0]='\0';
//	aString beast, popLine;
	aString endCharList(" \t,:\"{}[]!?\n\r"), eq("="), parameterLine;
	char* filename=dataFileName->raw();
	dataFile.open(filename,ios::in);
	int dataFileOpen = dataFile.is_open();
	if(! dataFileOpen){
		aString errstr("couldn\'t open the file ");
		errstr.cat(filename);
		aError(errstr);
		return 0;}

	StringList rawArlequinData0;//read in everything as strings, then sort and process them
	ObjList<StringList> arlequinDataBySection;//sections and subsections
	int nArlSections=6;
	enum arlSections {profile=0,haplotype=1,distmatrices=2,samples=3,genStructure=4,mantelTests=5};
	aList<int> arlSectionStartLines, arlSectionEndLines;
	for(int sec0=0;sec0<nArlSections;++sec0){//initialize
		arlequinDataBySection.Append(NEW StringList());
		arlSectionStartLines.Append(-1);//flag
		arlSectionEndLines.Append(-1);//flag
		}

	aString rawDataLine;
	int lineNum=-1;//stores current line of valid data
	while(!dataFile.eof()){
		dataFile.getline(dataLine, 10000);
		rawDataLine = dataLine;
		SnipArlequinComment(rawDataLine);//removes comments
		rawDataLine.CullLeadingWhiteSpace();
		rawDataLine.CullTrailingWhiteSpace();
		if(rawDataLine.len()>0){//it's not blank
			rawArlequinData0.Append(NEW aString(rawDataLine));
			++lineNum;
			if(rawDataLine[0]=='['){//assume it's a section heading line
				aString linecopy(rawDataLine);//work with a copy
				linecopy.CullLeadingChar('[');
				linecopy.CullTrailingChar(']');
				linecopy.CullLeadingWhiteSpace();//in case of sloppy text
				linecopy.CullTrailingWhiteSpace();
				if(linecopy=="Profile" || linecopy=="profile" || linecopy=="PROFILE"){
						arlSectionStartLines[profile]=lineNum;
						arlSectionEndLines[profile]=lineNum-1;//end of previous section
						}
					else if(linecopy=="HaplotypeDefinition" || 
							linecopy=="haplotypedefinition" || 
							linecopy=="HAPLOTYPEDEFINITION"){
						arlSectionStartLines[haplotype]=lineNum;
						arlSectionEndLines[haplotype]=lineNum-1;//end of previous section
						}
					else if(linecopy=="DistanceMatrix" || 
							linecopy=="distancematrix" || 
							linecopy=="DISTANCEMATRIX"){
						arlSectionStartLines[distmatrices]=lineNum;
						arlSectionEndLines[distmatrices]=lineNum-1;//end of previous section
						}
					else if(linecopy=="Samples" || 
							linecopy=="samples" || 
							linecopy=="SAMPLES"){
						arlSectionStartLines[samples]=lineNum;
						arlSectionEndLines[samples]=lineNum-1;//end of previous section
						}
					else if(linecopy=="Structure" || 
							linecopy=="structure" || 
							linecopy=="STRUCTURE"){
						arlSectionStartLines[genStructure]=lineNum;
						arlSectionEndLines[genStructure]=lineNum-1;//end of previous section
						}
					else if(linecopy=="Mantel" || 
							linecopy=="mantel" || 
							linecopy=="MANTEL"){
						arlSectionStartLines[mantelTests]=lineNum;
						arlSectionEndLines[mantelTests]=lineNum-1;//end of previous section
						}
					else{}
				}//if '['
			}//len>0
		}//end while !eof
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;

	aList<int> sortedStartLines(arlSectionStartLines);
	aList<int> tempStartLines,tempEndLines;
	for(int sec2=0;sec2<arlSectionEndLines.len();++sec2){
		sortedStartLines.Append(arlSectionEndLines[sec2]);
		tempStartLines.Append(0);tempEndLines.Append(0);
		}
	sortedStartLines.Append(rawArlequinData0.len()-1);
	sortedStartLines.Sort();
	for(int sec3=0;sec3<arlSectionStartLines.len();++sec3){
		int posStart = sortedStartLines.Loc(arlSectionStartLines[sec3]);
		int endSection = sortedStartLines[posStart+1];
		arlSectionEndLines[sec3] = endSection;
		}//end for sec3
	
	if(arlSectionStartLines.Last() == -1){
		arlSectionStartLines.Last() = sortedStartLines.Last();
		arlSectionEndLines.Last() = sortedStartLines.Last();}
	for(int sec4 = nArlSections-2;sec4>=0;--sec4){
		if(arlSectionStartLines[sec4] == -1){
			arlSectionStartLines[sec4] = arlSectionStartLines[sec4+1];
			arlSectionEndLines[sec4] = arlSectionStartLines[sec4];}
		}//sec4
//	arlSectionStartLines.Print(cout," ");
//	arlSectionEndLines.Print(cout," ");
	
	//finish putting raw data into sections
	for(int sec5 = 0;sec5<nArlSections;++sec5){
		int start = arlSectionStartLines[sec5], end = arlSectionEndLines[sec5];
		for(int i=start+1; i<=end;++i){
			aString& processedDataLine = rawArlequinData0[i];
			arlequinDataBySection[sec5].Append(NEW aString(processedDataLine));
			}//i
//		arlequinDataBySection[sec5].Print(cout);
		}//sec5
		
	//now process each section separately
	// "[Profile]"//title_
	aString *tword=0, pline;
	for(int p0=0;p0<arlequinDataBySection[profile].len();++p0){
		pline = arlequinDataBySection[profile][p0];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "title_" || *tword == "Title" || *tword == "TITLE" ){
			pline.CullLeadingChar('\"');
			pline.CullTrailingChar('\"');
			title_ = pline;
			arlequinDataBySection[profile].Delete(p0);//no longer needed
			delete tword;tword=0;
			break;
			}//if==title_
		delete tword;tword=0;
		}

	// "[Profile]"//NbSamples
	for(int p1=0;p1<arlequinDataBySection[profile].len();++p1){
		pline = arlequinDataBySection[profile][p1];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "NbSamples" || *tword == "nbsamples" || *tword == "NBSAMPLES"){
			pline.CullLeadingWhiteSpace();
			npop_ = pline.toInt();
			arlequinDataBySection[profile].Delete(p1);//no longer needed
			delete tword;tword=0;
			break;
			}//if==NbSamples
		delete tword;tword=0;
		}

	// "[Profile]"//GenotypicData
	for(int p2=0;p2<arlequinDataBySection[profile].len();++p2){
		pline = arlequinDataBySection[profile][p2];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "GenotypicData" || *tword == "genotypicdata" || *tword == "GENOTYPICDATA"){
			pline.CullLeadingWhiteSpace();
			int gtypedata = pline.toInt();
			if(gtypedata != 1){
				aString err("Can't read in this type of data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p2);//no longer needed
			delete tword;tword=0;
			break;
			}//if==GenotypicData
		delete tword;tword=0;
		}

	// "[Profile]"//GameticPhase
	for(int p3=0;p3<arlequinDataBySection[profile].len();++p3){
		pline = arlequinDataBySection[profile][p3];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "GameticPhase" || *tword == "gameticphase" || *tword == "GAMETICPHASE"){
			pline.CullLeadingWhiteSpace();
			int gphasedata = pline.toInt();
			if(gphasedata != 0){
				aString err("Can't read this type of data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p3);//no longer needed
			delete tword;tword=0;
			break;
			}//if==GameticPhase
		delete tword;tword=0;
		}

	// "[Profile]"//RecessiveData
	for(int p4=0;p4<arlequinDataBySection[profile].len();++p4){
		pline = arlequinDataBySection[profile][p4];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "RecessiveData" || *tword == "recessivedata" || *tword == "RECESSIVEDATA"){
			pline.CullLeadingWhiteSpace();
			int recdata = pline.toInt();
			if(recdata != 0){
				aString err("Can't read this type of data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p4);//no longer needed
			delete tword;tword=0;
			break;
			}//if==RecessiveData
		delete tword;tword=0;
		}

	// "[Profile]"//DataType
	for(int p5=0;p5<arlequinDataBySection[profile].len();++p5){
		pline = arlequinDataBySection[profile][p5];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "DataType" || *tword == "datatype" || *tword == "DATATYPE"){
			pline.CullLeadingWhiteSpace();
			if(pline != "STANDARD"){
				aString err("Can only read DataType=STANDARD data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p5);//no longer needed
			delete tword;tword=0;
			break;
			}//if==DataType
		delete tword;tword=0;
		}

	// "[Profile]"//MissingData
	aString missingDataFlag("?");//default
	for(int p6=0;p6<arlequinDataBySection[profile].len();++p6){
		pline = arlequinDataBySection[profile][p6];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "MissingData" || *tword == "missingdata" || *tword == "MISSINGDATA"){
			pline.CullLeadingWhiteSpace();
			pline.CullLeadingChar('\"');//can be single or double quotes
			pline.CullTrailingChar('\"');
			pline.CullLeadingChar('\'');
			pline.CullTrailingChar('\'');
			missingDataFlag = pline;
			if(missingDataFlag.len() != 1){
				aString err("Missing data must be denoted by a single character.  Default used.");
				aError(err);
				missingDataFlag="?";
				}
			arlequinDataBySection[profile].Delete(p6);//no longer needed
			delete tword;tword=0;
			break;
			}//if==IndividualLevel
		delete tword;tword=0;
		}

	// "[Profile]"//RecessiveAllele
	aString RecessiveAlleleFlag("?");//default
	for(int p7=0;p7<arlequinDataBySection[profile].len();++p7){
		pline = arlequinDataBySection[profile][p7];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "RecessiveAllele" || *tword == "recessiveallele" || *tword == "RECESSIVEALLELE"){
			pline.CullLeadingWhiteSpace();
			pline.CullLeadingChar('\"');//can be single or double quotes
			pline.CullTrailingChar('\"');
			pline.CullLeadingChar('\'');
			pline.CullTrailingChar('\'');
			RecessiveAlleleFlag = pline;
			if(RecessiveAlleleFlag.len() == 0){
				aString err("Recessive allele (homozygote) not identified.  Default used.");
				aError(err);
				RecessiveAlleleFlag="null";
				}
			arlequinDataBySection[profile].Delete(p7);//no longer needed
			delete tword;tword=0;
			break;
			}//if==IndividualLevel
		delete tword;tword=0;
		}

	// "[Profile]"//Frequency
	for(int p8=0;p8<arlequinDataBySection[profile].len();++p8){
		pline = arlequinDataBySection[profile][p8];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "Frequency" || *tword == "frequency" || *tword == "FREQUENCY"){
			pline.CullLeadingWhiteSpace();
			if(!(pline == "ABS" || pline == "abs")){
				aString err("Can only read Frequency=ABS data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p8);//no longer needed
			delete tword;tword=0;
			break;
			}//if==DataType
		delete tword;tword=0;
		}


	// "[Profile]"//LocusSeparator
	aString locSep(" \t");//default
	for(int p9=0;p9<arlequinDataBySection[profile].len();++p9){
		pline = arlequinDataBySection[profile][p9];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "LocusSeparator" || *tword == "locusseparator" || *tword == "LOCUSSEPARATOR"){
			pline.CullLeadingWhiteSpace();
			if(pline == "WHITESPACE" || pline == "whitespace"){
					locSep = " \t";}
				else if(pline == "TAB" || pline == "tab"){
					locSep = "\t";}
				else if(pline == "COMMA" || pline == "comma"){
					locSep = ",";}
				else if(pline == "NONE" || pline == "none"){
					aString err("Need locus separator to read data from the Arlequin file");
					aError(err);
					delete tword;tword=0;
					return 2;}
				else{
					locSep = pline;
					if(locSep.len()==0){
						aString err("LocusSeparator not specified correctly.  Default used.");
						aError(err);
						locSep = " \t";}
					}
			arlequinDataBySection[profile].Delete(p9);//no longer needed
			delete tword;tword=0;
			break;
			}//if==DataType
		delete tword;tword=0;
		}

	//[Profile] //CompDistMatrix=  ignored
	//[Profile] //FrequencyThreshold=  ignored
	//[Profile] //EpsilonValue=  ignored

	//section [Data]

	//[Data] [[HaplotypeDesignation]]  ignored
	//[Data] [[DistanceMatrix]]  ignored


	//[Data] [[Sample]] 
	//read in population data
	long nloci=0;
	unsigned int alleleLength=1;
	aString popLine;
	ObjList<ObjList<ObjList<StringList> > > rawArlequinData;//temp storage [pop][i][loc][a]
	ObjList<StringList> rawIDnames;//temp storage [pop][id]
	ObjList<aList<long> > popGenotypeFreqs;//temp storage [pop][id]
	long dline=0;//data line counter
	for (int pop=0;pop<npop_;++pop){
		rawArlequinData.Append(NEW ObjList<ObjList<StringList> >());//set up raw data 
		rawIDnames.Append(NEW StringList());//set up raw data 
		popGenotypeFreqs.Append(NEW aList<long>());//set up raw data 
		
		//get the title_ of this population
		popLine = arlequinDataBySection[samples][dline++];
		aString* junk = popLine.ChopOffFirstWord(eq);
		delete junk; junk=0;
		popLine.CullLeadingWhiteSpace();
		popLine.CullLeadingChar('\"');
		popLine.CullTrailingChar('\"');
		aString *s = NEW aString(popLine);
		popName_.Append(s);//append a copy

		//read sample size to expect
		popLine = arlequinDataBySection[samples][dline++];
		junk = popLine.ChopOffFirstWord(eq);
		delete junk; junk=0;
		popLine.CullLeadingWhiteSpace();
		long indivs=popLine.toLong();
		long i=(-1);//a counter
		popLine.clear();
		popLine = arlequinDataBySection[samples][dline++];
		popLine.CullTrailingWhiteSpace();
		if(popLine[popLine.len()-1] != '{'){
			aString err("Expected \'SampleData={\' at ");err.cat(*s);//adds pop title_
			aError(err);}
		bool thisPopDone = false;
		while (!thisPopDone){//LOOK AT EACH INDIVIDUAL
			popLine.clear();
			popLine = arlequinDataBySection[samples][dline++];
			if(popLine[0] == '}'){
				thisPopDone = true;
				continue;}//goes to end of while !thisPopDone

			rawArlequinData[pop].Append(NEW ObjList<StringList>());//set up raw data
			i++; 
			
			//individuals come as 2 consecutive lines in diploid data
			//the first line contains an ID number and a frequency
			aString indivline1(popLine);
			aString *id=indivline1.ChopOffFirstWord();
			aString *fr=indivline1.ChopOffFirstWord();
			long gfreq = fr->toLong();
			delete fr; fr=0;
			rawIDnames[pop].Append(id);
			popGenotypeFreqs[pop].Append(gfreq);

			aString indivline2(arlequinDataBySection[samples][dline]);//sneak a peek ahead to check ploidy
			bool diploid = false;
			if(indivline2[0]=='}'){
					thisPopDone = true; break;}
				else if(indivline2.len() == indivline1.len()){//assume diploid based on symmetry in data file
					diploid=true;
					dline++;//increment the counter
					}
			long loc0=0;
			aString *gene=0;			
			while(indivline1.len()>0){
				rawArlequinData[pop][i].Append(NEW StringList());//new for each locus
				gene = indivline1.ChopOffFirstWord(locSep);
				alleleLength = MAX(alleleLength,gene->len());
				rawArlequinData[pop][i][loc0].Append(gene);
				++loc0;
				}//end while
			nloci = MAX(loc0,nloci);
			if(diploid){
				long loc01=0;
				while(indivline2.len()>0){
					gene = indivline2.ChopOffFirstWord(locSep);
					alleleLength = MAX(alleleLength,gene->len());
					rawArlequinData[pop][i][loc01].Append(gene);
					++loc01;
					}//end while
				nloci = MAX(loc01,nloci);
				}//if diploid
			}//end while !thisPopDone
		}//end for pop

	//make up some locus names
	traitName_.Clear();
	traitName_.Append(NEW aString("id"));
	ObjList<StringList> alleleNames;
	int maxloclen = Digits(nloci);
	for(int loc=0;loc<nloci;++loc){
		alleleNames.Append(NEW StringList());
		aString *lname = NEW aString("loc");
		aString lnum(loc+1);lnum.PrependCharToLen('0',maxloclen);
		lname->cat(lnum);
		traitName_.Append(lname);
		}//for loc

	//recode data, using a list of alleles/locus
	for(int pop1=0;pop1<npop_;++pop1){
		for(int i=0;i<rawArlequinData[pop1].len();++i){
			for(int loc1=0;loc1<nloci;++loc1){
				for(int h=0;h<rawArlequinData[pop1][i][loc1].len();++h){
					aString gene(rawArlequinData[pop1][i][loc1][h]);//work with copy
					if(! (alleleNames[loc1].Includes(&gene))){
						alleleNames[loc1].Append(NEW aString(gene));}
					}//end h
				}//loc1
			}//i
		}//pop1
	for(int loc1=0;loc1<nloci;++loc1){//sort; clear missing data flags
		alleleNames[loc1].Sort();
		if(alleleNames[loc1].Includes(&missingDataFlag)){
			int site = alleleNames[loc1].Loc(&missingDataFlag);
			alleleNames[loc1].Delete(site);}
		}//loc1
			
	//finish recoding
	alleleLen_=alleleLength;
	ploidy_=2;//assumed for now -- may be able to infer this from data
	rawTraits_.Clear();
	nlociSLCodom_ = nloci;
	aString missingDataToPrint(missingDataFlag);
	missingDataToPrint.PrependCharToLen(' ',alleleLen_);
	for(int pop2=0;pop2<npop_;++pop2){
		rawTraits_.Append(NEW ObjList<TextDataList>());				
		for(int i2=0;i2<rawArlequinData[pop2].len();++i2){
			TextDataList gtyp(missingDataFlag);
			gtyp.Append(NEW TextDatum(rawIDnames[pop2][i2]));
			for(int loc2=0;loc2<nloci;++loc2){
				bool datapresent=false;
				TextDatum *gtypLoc = NEW TextDatum();//starts out as missing
				for(int h2=0;h2<rawArlequinData[pop2][i2][loc2].len();++h2){
					aString gene2(rawArlequinData[pop2][i2][loc2][h2]);//work with copy
					if(gene2 == missingDataFlag){
							gtypLoc->cat(missingDataToPrint);}
						else{
							datapresent=true;
							gtypLoc->missing(false);
							gene2.PrependCharToLen(' ',alleleLen_);//standard size for all loci
							gtypLoc->cat(gene2);}
					}//end h2
				gtyp.Append(gtypLoc);
				}//loc2
			for(long n=0;n<popGenotypeFreqs[pop2][i2];++n){//use frequency info here
				rawTraits_[pop2].Append(NEW TextDataList(gtyp));}
			}//i	
		popSize_.Append(rawTraits_[pop2].len());
		}//pop2
		



	//[Data] [[Structure]] 
	//read in group information, if any
	bool readGroupInfo = false;
	if(arlequinDataBySection[genStructure].len() > 0){
		readGroupInfo=true;}

	if(! readGroupInfo){
			ngroups_=1;
			GroupMembership();}
		else{//read group info explicitly
			for(int s0=0;s0<arlequinDataBySection[genStructure].len();++s0){
				parameterLine = arlequinDataBySection[genStructure][s0];
				aString *pword=parameterLine.ChopOffFirstWord(eq);
				pword->CullTrailingWhiteSpace();
				if(*pword == "StructureName" || *pword == "structurename" || *pword == "STRUCTURENAME"){
					//just ignore
					arlequinDataBySection[genStructure].Delete(s0);//no longer needed
					delete pword;pword=0;
					break;
					}
				delete pword;pword=0;
				}//s0

			// "[Structure]"//NbGroups
			for(int s1=0;s1<arlequinDataBySection[genStructure].len();++s1){
				parameterLine = arlequinDataBySection[genStructure][s1];
				tword=parameterLine.ChopOffFirstWord(eq);
				tword->CullTrailingWhiteSpace();
				if(*tword == "NbGroups" || *tword == "nbgroups" || *tword == "NBGROUPS"){
					parameterLine.CullLeadingWhiteSpace();
					ngroups_ = parameterLine.toInt();
					arlequinDataBySection[genStructure].Delete(s1);//no longer needed
					delete tword;tword=0;
					break;
					}//if==NbGroups
				delete tword;tword=0;
				}//s1

			// "[Structure]"//IndividualLevel
			for(int s2=0;s2<arlequinDataBySection[genStructure].len();++s2){
				parameterLine = arlequinDataBySection[genStructure][s2];
				tword=parameterLine.ChopOffFirstWord(eq);
				tword->CullTrailingWhiteSpace();
				if(*tword == "IndividualLevel" || *tword == "individuallevel" || *tword == "INDIVIDUALLEVEL"){
					parameterLine.CullLeadingWhiteSpace();
					int ilevel = parameterLine.toInt();
					if(ilevel != 1){
						aString err("In [[Structure]], must set IndividualLevel=1 in order to read Arlequin data");
						aError(err);
						delete tword; tword=0;
						return 2;
						}
					arlequinDataBySection[genStructure].Delete(s2);//no longer needed
					delete tword;tword=0;
					break;
					}//if==IndividualLevel
				delete tword;tword=0;
				}//s2

			groupName_.Clear();
			groupHierarchy_.Clear();
			long sline=0;
			for(int gr=0;gr<ngroups_;++gr){
				aString gpnum(gr+1);
				gpnum.PrependCharToLen('0',Digits(ngroups_));
				gpnum.prepend("group");
				groupName_.Append(NEW aString(gpnum));
				groupHierarchy_.Append(NEW aList<int>());
				//read group membership intro lines
				parameterLine = arlequinDataBySection[genStructure][sline++];
				if(parameterLine[parameterLine.len()-1] != '{'){
						aString err("Expected \"Group={\" in [[Structure]]");
						aError(err);
						continue;}
					else{//read names in group
						bool thisGpDone = false;
						while (!thisGpDone){//LOOK AT EACH title_
							aString gpLine = arlequinDataBySection[genStructure][sline++];
							if(gpLine[0] == '}'){
								thisGpDone = true;
								continue;}//goes to end of while !thisGpDone
			
							gpLine.CullLeadingChar('\"');
							gpLine.CullTrailingChar('\"');
							int popnum = popName_.Loc(&gpLine);
							groupHierarchy_[gr].Append(popnum);
							}//end while !thisGpDone
						}//end else
				}//for gr
		}//end if readGroupInfo
		
	LocalityMembership();

	aList<int> codomLoci;
	for(int loc=0;loc<nlociSLCodom_;++loc){
		codomLoci.Append(loc);}//all loci are assumed codominant  - may adjust this later
//	ParseGenotypicData(codomLoci);
	ParseGenotypicData();
	CheckForNewAlleles();
	SortBySex();
	return 1;
	}//end SexLinkedCodomMarkerData::ReadArlequinDataFile



void SexLinkedCodomMarkerData::SnipArlequinComment(aString& arqStr){
	const char pound='#';
	int compos = arqStr.pos(pound);
	if(compos==0 && arqStr[0] != pound){//no comment found
		return;}
	if(arqStr.len()==1 && arqStr[0] == pound){
		arqStr.clear();return;
		}
	arqStr.trunc(compos-1);
	}//end SexLinkedCodomMarkerData::SnipArlequinComment(aString&)



int SexLinkedCodomMarkerData::ReadGelJockDataFile (aString* dataFileName){
	fstream dataFile;	
	char *dataLine=NEW char[1001];dataLine[0]='\0';
//	aString beast, popLine;
	aString endCharList(" \t,:\"{}[]!?\n\r");
	char* filename=dataFileName->raw();
	
//	char endChar = '\r';
	char endChar = endchr;//defined in StringClass.h
	dataFile.open(filename,ios::in);
	int dataFileOpen = dataFile.is_open();
	if(! dataFileOpen){
		aString errstr("couldn\'t open the file ");
		errstr.cat(filename);
		aError(errstr);
		return 0;}
//	dataFile.getline(dataLine, 1000);
	dataFile.getline(dataLine, 1000, endChar);
	title_ = dataLine;

	dataFile.getline(dataLine, 1000, endChar); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 5);
	npop_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 6);
	long nloci = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 7);
	int maxall_ = paramWord->toInt();
	delete paramWord; paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramWord->cut(0, 8);
	ngroups_ =  paramWord->toInt();
	delete paramWord; paramWord=0;

	if(endCharList.len()>11){//ALLELECHAR may be missing 
		paramWord = parameterLine.ChopOffFirstWord(endCharList);
		paramWord->cut(0, 11);
		alleleLen_ =  paramWord->toInt();
		delete paramWord; paramWord=0;
		}


	//get locus information
	traitName_.Clear();
	traitName_.Append(NEW aString("id"));
	int moreLoci = 1;//true
	int pop,j,loc = 0;
	while (moreLoci){
		dataFile.getline(dataLine, 1000, endChar); 
		aString locusLine(dataLine);
		while (locusLine.len() > 0){
			++loc;
			aString *locName = locusLine.ChopOffFirstWord(endCharList);
			if(locName->len()>0){//there could be trailing blanks on locusLine
				aString *s = NEW aString(locName);
				traitName_.Append(s);//append a copy
				traitNum_.Append(loc);}
			delete locName; locName=0;
			}//end while locusLine.len()
		if (loc >= nloci){//data may be on more than one line
			moreLoci = 0;}//false
		}//end while moreLoci
	//read in the number of loci to be skipped in the analyses
	dataFile.getline(dataLine, 1000, endChar); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	skipWord->cut(0,5);
	int i,blankLines,toSkip,item,lociSkipped = skipWord->toInt();
	delete skipWord; skipWord=0;

	//and read in the names of skipped loci, if any
	loc = 0;
	moreLoci = 0;//false
	if (lociSkipped > 0){
			moreLoci = 1;}//true
	StringList skippedLocusName;
	while (moreLoci){
		while (skipLine.len() > 0){
			++loc;
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				aString *s = NEW aString(skipWord);
				skippedLocusName.Append(s);//append a copy
				toSkip = traitName_.Loc(s);
				skippedTraitNum_.Append(toSkip);
				item = traitNum_.Loc(toSkip);
				traitNum_.Delete(item);
				}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (loc >= lociSkipped){
				moreLoci = 0;}//false
			else	{//more loci on the next line
				dataFile.getline(dataLine, 1000); 
				skipLine = dataLine;}
		}//end while moreLoci
	skippedTraitNum_.Sort();
	traitNum_.Sort();

	nloci = traitNum_.len();

	//read in population data
	aString popLine;
	aString beast, beastTest;
	for (pop=0;pop<npop_;++pop){
		//skip blank lines before pops in the data file
		blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 1000, endChar); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		aString *s = NEW aString(popLine);
		popName_.Append(s);//append a copy

		//make a new sublist for the genotypes of this population
		rawTraits_.Append(NEW ObjList<TextDataList>());	

		// BEAST is the string for a particular individual
		i=0;
		beast.clear(); beastTest.clear();
		while (beastTest != "NEXT"){ 
			dataFile.getline(dataLine, 1000, endChar); 
			beastTest = beast = dataLine;
			beastTest.CullLeadingWhiteSpace();
			beastTest.CullTrailingWhiteSpace();
			if (beastTest != "NEXT"){
					rawTraits_[pop].Append(NEW TextDataList("?"));//sets missingFlag=?
					aString s(beast);//work on a copy
					aString *idval=s.ChopOffFirstWord();
					rawTraits_[pop][i].Append(NEW TextDatum(*idval));
					delete idval;idval=0;
					//parse the genotype string
					for(loc=0;loc<nloci;++loc){
						aString g(s,loc*alleleLen_*ploidy_,(loc+1)*alleleLen_*ploidy_);//check this
						bool allmissing=true;
						for(j=0;j<alleleLen_*ploidy_;++j){
							if(g[j] != ' '){
								allmissing=false;break;}}
						if(allmissing){
								rawTraits_[pop][i].Append(NEW TextDatum());}
							else{
								rawTraits_[pop][i].Append(NEW TextDatum(g));}
						}//loc	
					++i;}
				else{
					popSize_.Append(rawTraits_[pop].len());}
			}//end while beast..else
		}//end for pop
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	GroupMembership();
	LocalityMembership();
	nlociSLCodom_=nloci;//by assumption -- modify this in other classes
	aList<int> codominantloci = traitNum_;
//	ParseGenotypicData(codominantloci);
	ParseGenotypicData();
	CheckForNewAlleles();
	return 1;
	}//end SexLinkedCodomMarkerData::ReadGelJockDataFile




void SexLinkedCodomMarkerData::RestoreLoci (aList<int>& lociToRestore){
	//This function restores the designated loci to the analysis.
	//It shifts trait numbers from from skippedTraitNums_ to the traitNums_ list
	//and keps track of the changes in the locusNumSLCodom_ and skippedLocusNumSLCodom_ lists
	if(lociToRestore.len()==0){return;}//none to cull
	ResetFlags();
	int tr,rLoc,item;
	for(tr=0;tr<lociToRestore.len();++tr){
		rLoc = lociToRestore[tr];//make sure it's a valid locus to restore
		if(skippedTraitNum_.Includes(rLoc) && rLoc>=0 && rLoc<traitName_.len()){
			traitNum_.Append(rLoc);
			locusNumSLCodom_.Append(rLoc);
			item = skippedTraitNum_.Loc(rLoc);
			skippedTraitNum_.Delete(item);
			item = skippedLocusNumSLCodom_.Loc(rLoc);
			skippedLocusNumSLCodom_.Delete(item);
			}//if item not skipped already
		}//tr
	traitNum_.Sort();
	locusNumSLCodom_.Sort();
	CheckForNewAlleles();
	SortBySex();
	}//end procedure SexLinkedCodomMarkerData::RestoreLoci(aList<int>&)


void SexLinkedCodomMarkerData::RestoreLoci (StringList& lociToRestore){
	//This function restores the designated loci to the analysis
	if(lociToRestore.len()==0){return;}//none to cull
	//convert names to numbers and cull
	aList<int> restoreListByNum;
	int tr,item;
	for(tr=0;tr<lociToRestore.len();++tr){
		aString rTraitname = lociToRestore[tr];
		item = traitName_.Loc(&rTraitname);
		if((item>=0)&&(!(traitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already included
			restoreListByNum.Append(item);
			}//if item exists
		}//tr
	RestoreLoci(restoreListByNum);
	}//end procedure SexLinkedCodomMarkerData::RestoreLoci(StringList&)



void SexLinkedCodomMarkerData::RestoreLocus (aString& locusToRestore){
	StringList lociToRestore;
	lociToRestore.Append(NEW aString(locusToRestore));
	RestoreLoci(lociToRestore);
	}//end SexLinkedCodomMarkerData::RestoreLocus (aString& )


void SexLinkedCodomMarkerData::RestoreLocus (char* locusToRestore){
	StringList lociToRestore;
	lociToRestore.Append(NEW aString(locusToRestore));
	RestoreLoci(lociToRestore);
	}//end SexLinkedCodomMarkerData::RestoreLocus (char* )


void SexLinkedCodomMarkerData::RestoreLocus (int locusToRestore){
	aList<int> lociToRestore;
	lociToRestore.Append(locusToRestore);
	RestoreLoci(lociToRestore);
	}//end SexLinkedCodomMarkerData::RestoreLocus (int )


void SexLinkedCodomMarkerData::SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
	//records all data, passed along from genetic analysis classes
	//empties old data and creates new data structures
	Clear();
	title_ = "Simulated sex-linked codominant data set";
	npop_ = pgcList.len();
	nlociSLCodom_ = pgcList[0].loci(MultiInheritanceModeContainer<int>::sexLink);
	ngroups_ = 1;
	SetAlleleLength(1);//bug if maxall>9
	traitName_.Append(NEW aString("ID"));
	int unspecfiedTraits=1, runningTraitNum=0;
	traitNum_.Append(runningTraitNum++);//ID
	if(pgcList[0].sexLinkedDataIncluded()){
		traitName_.Append(NEW aString("sex"));
		traitNum_.Append(runningTraitNum++);
		unspecfiedTraits++;
		SetSexTrait();		
		}
	int locusCount=0;
	int pop,i, loc;
	for(loc = 0;loc<nlociSLCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',Digits(nlociSLCodom_));
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(runningTraitNum++);
		allSLCodomLoci_.Append(traitNum_.Last());
		locusNumSLCodom_.Append(traitNum_.Last());
		}
	ntraits_ = traitNum_.len();
	
	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("simulated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create dummy id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
//		rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());

		for (i=0;i<pgcList[pop].popSize();++i){
			rawTraits_[pop].Append(NEW TextDataList());
			rawTraits_[pop][i].Append(NEW TextDatum());//empty ID
//			for(loc=0;loc<nlociCodom_;++loc){
//				rawCodomGtypes_[pop][i].Append(NEW TextDataList());}
			}
		popSize_.Append(pgcList[pop].popSize());


		//create and append dummy genotypes
		for(i=0;i<pgcList[pop].popSize();++i){
			//to determine sex
			if(pgcList[pop].sexLinkedDataIncluded()){
				rawTraits_[pop][i].Append(NEW TextDatum(" "));}//empty sexTrait
			for(loc=0;loc<nlociSLCodom_;++loc){
				rawTraits_[pop][i].Append(NEW TextDatum("  "));
				}//end for loc
			}//end for i
		}//end for pop	

	//Fill up with data
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<popSize(pop);++i){
			SetGenome(pgcList[pop][i], pop, i);}
		}
	GroupMembership();
	LocalityMembership();
	aList<int> lociToParse = allSLCodomLoci_;
	CheckForNewAlleles();		
	}//SexLinkedCodomMarkerData::SetGenome(ObjList<PopulationGeneContainer<int> >&)


void SexLinkedCodomMarkerData::SetGenome(GeneContainerCore& gc, int pop, int indiv){
	//records data to MultiHaplotypeContainer<int>
	MultiHaplotypeContainer<int>* mhc = (MultiHaplotypeContainer<int>*) &gc;
	if(mhc->loci() != nlociSLCodom_){
		aError("ERROR: SexLinkedCodomMarkerData::SetGenome(): size mismatch between data classes");
		return;}
	ResetFlags();
	rawTraits_[pop][indiv][0] = mhc->ID();
	int thisPloidy = mhc->ploidy();
	int loc,c;
	if(mhc->missing()){//entirely missing for whatever reason
		for(loc=0;loc<nlociSLCodom_;++loc){
			rawTraits_[pop][indiv][locusNumSLCodom_[loc]].clear();}
		return;}
	for(loc=0;loc<nlociSLCodom_;++loc){
		aString gvals;//initializes as missing
		for(c=0;c<thisPloidy;++c){
			if(mhc->missing(c)){
				gvals.AppendCharToLen(' ',alleleLen());}//missing value
			else{
				gvals.cat(mhc->haplotype(loc,c));}
			}	
		for(c=thisPloidy;c<ploidy();++c){//store as missing values
				gvals.AppendCharToLen(' ',alleleLen());}
		rawTraits_[pop][indiv][locusNumSLCodom_[loc]] = gvals;
		}//loc
	}//SexLinkedCodomMarkerData::SetGenome(GeneContainerCore&, int, int)


void SexLinkedCodomMarkerData::SetLoci(aList<int>& lociToInclude){
	if(lociToInclude.len()==0) return;
	SexLinkedCodomMarkerData::ResetGeneticFlags();
	allSLCodomLoci_=lociToInclude;
	locusNumSLCodom_=lociToInclude;
	int tr,cLoc,item;
	for(tr=0;tr<lociToInclude.len();++tr){
		cLoc = lociToInclude[tr];
		if(skippedTraitNum_.Includes(cLoc)){
			item = locusNumSLCodom_.Loc(cLoc);
			if(item>=0){
				locusNumSLCodom_.Delete(item);}
			if(! skippedLocusNumSLCodom_.Includes(cLoc)){
				skippedLocusNumSLCodom_.Append(cLoc);}
			}
		}//tr
	skippedLocusNumSLCodom_.Sort();
	nlociSLCodom_ = locusNumSLCodom_.len();
//	SexLinkedCodomMarkerData::ParseGenotypicData(allSLCodomLoci_,0);
	SexLinkedCodomMarkerData::ParseGenotypicData(0);
	SexLinkedCodomMarkerData::CheckForNewAlleles();
	}//SexLinkedCodomMarkerData::SetLoci(aList<int>& )



void SexLinkedCodomMarkerData::UpdateVariableLoci(void){
	if (varLociCurrentSLCodom_){return;}
	CheckForNewAlleles();
	variableLocusNumSLCodom_.Clear();
	int loc;
	for(loc=0;loc<nlociSLCodom_;++loc){
		StringList& locList = allelesPresentSLCodom_[loc];
		if (locList.len()>1){
//			variableLocusNumSLCodom_.Append(locusNumSLCodom_[loc]);}}
			variableLocusNumSLCodom_.Append(loc);}}
	varLociCurrentSLCodom_=true;
	}//end SexLinkedCodomMarkerData::UpdateVariableLoci



#endif







