/*****
GenotypeFreqDistSexLinkDom.cpp

Sex-linked dominant markers: p_ is the (dominant) allele frequency, 
beta_ is the dominant marker frequency over males and females, 
and  the class holds marker frequencies in the order {dominant,recessive}.  

The constructor should be set up like this:

GenotypeFreqDistribution someDist(dominantMarkerFreq, Fis, //SLdom=// true, //heterogameticSexIsMale=// true, 
			sexratio);

*****/

#ifndef GenotypeFreqDistSexLinkDomcpp
#define GenotypeFreqDistSexLinkDomcpp = 1


#include "GenotypeFreqDistSexLinkDom.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"




void GenotypeFreqDistSexLinkDom::CreateDistributionSexLinkedDominantMarker(void){
	//0-1 data, sampled from Fis, sex ratio & marker freq
	//because males and females have different frequencies
	//heterogametic sex first
	
	//calculate allele frequency from marker frequency
	long double one = (long double)1.0;
	long double two = one + one;
	long double four = two * two;
	p_ = - sqrt((one+sr_*(one-Fis_))*(one+sr_*(one-Fis_)) - four * beta_ * sr_ *(one-Fis_) );
	p_ += one + sr_ *(one + Fis_);
	p_ /= two * sr_ * (one-Fis_);
	betaHeterogametic_ = p_;
	betaHomogametic_ = p_ * p_ + Fis_ * p_ *(one-p_) + two * (one - Fis_) * p_ *(one - p_);
	
	if(! heterogameticSexIsMale_){
		SWAP(betaHeterogametic_,betaHomogametic_);}
	aList<long double> pp;//marker frequencies
	Append(betaHomogametic_);
	Append(one - betaHomogametic_);
	}//GenotypeFreqDistSexLinkDom::CreateDistributionSexLinkedCodominantMarker()




int GenotypeFreqDistSexLinkDom::RandomGenotype(Ran3List& ran, bool homogametic){
	aList<long double> sublist;
	if(homogametic){
			if(ran()<betaHomogametic_) return 0;}
		else{
			if(ran()<betaHeterogametic_) return 0;}
	return 1;
	}//GenotypeFreqDistSexLinkDom::RandomGenotype(Ran3List&, bool){



#endif