//GenotypeFreqMultiAlleleDist.cpp


#ifndef GenotypeFreqMultiAlleleDistcpp
#define GenotypeFreqMultiAlleleDistcpp = 1


#include "GenotypeFreqMultiAlleleDist.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"


void GenotypeFreqMultiAlleleDist::CreateDistribution(void){
	Clear();
	GenotypeFreqMultiAlleleDist& gFreqs = *this;//notational convenience
	ObjList<aList<long double> > gFreqArray;
	aList<long double> homoFreqs;
	for(int a0=0;a0<Alleles();++a0){
		gFreqArray.Append(NEW aList<long double>());
		homoFreqs.Append(p_[a0]*p_[a0]);
		for(int b0=0;b0<Alleles();++b0){
			gFreqArray[a0].Append((long double)2.0*p_[a0]*p_[b0]);
			}}
	for(int a=0;a<Alleles();++a){
		for(int b=a+1;b<Alleles();++b){
//			double Hexp = gFreqArray[a][b];
			if ((Fis_ != (long double)0.0) &&(p_[a] > (long double)0.0) && (p_[b] > (long double)0.0)){
				//since the heterozygotes are comprised of 50% of each allele, 
				//the deviation in heterozygotes described by Fis_
				//must be made up of equal parts contributions to/from the homozygote classes
				gFreqArray[a][b] *= (long double)1.0 - Fis_;
				}//end if Fis && polymorphic
			}
		if(Fis_ !=(long double)0.0) {
			homoFreqs[a] += Fis_ * p_[a] * ((long double)1.0 - p_[a]);}
		}
	for(int a1=0;a1<Alleles();++a1){
		gFreqArray[a1][a1] = homoFreqs[a1];}

	for(int a2=0;a2<Alleles();++a2){
		for(int b2=a2;b2<Alleles();++b2){
			aString* gt = NEW aString((char)(a2+'A'));
			gt->cat((char)(b2+'A'));
			gTypes_.Append(gt);
			gFreqs.Append(gFreqArray[a2][b2]);			
			}
		}
	long double summ = sum();
	}//end GenotypeFreqMultiAlleleDist::CreateDistribution
	


void GenotypeFreqMultiAlleleDist::CreateDistribution2(void){
	//notation follows Porter 2003 Molecular Ecology
	int a0,a1, alleles = p_.len();//a0= rows, a1= columns
	//first sort p_ into ascending order, saving the original order of alleles
	aList<int> freqorder, currentOrder, originalOrder, sortedOrder;
	for(a0=0;a0<alleles;++a0){
		currentOrder.Append(a0);
		originalOrder.Append(a0);
		}
	aList<long double> freqs, frqcopy(p_);
	while(frqcopy.len()>0){
		long double lowFreq=(long double)2.0;
		int item;
		for(int a3=0;a3<frqcopy.len();++a3){
			if(frqcopy[a3]<lowFreq){//find lowest
				lowFreq=frqcopy[a3];
				item=a3;}
			}
		freqs.Append(lowFreq);
		freqorder.Append(currentOrder[item]);
		frqcopy.Delete(item);
		currentOrder.Delete(item);
		}//end while 
	for(a0=0;a0<alleles;++a0){
		sortedOrder.Append(freqorder.Loc(originalOrder[a0]));}
//	for(a0=0;a0<alleles;++a0){
//		cout<<freqorder[a0]<<":"<<frq[freqorder[a0]]<<'\t';}cout<<endl;
//	for(a0=0;a0<alleles;++a0){
//		cout<<sortedOrder[a0]<<":"<<freqs[sortedOrder[a0]]<<'\t';}cout<<endl;
				
	//work on sorted allele frequencies
	ObjList<aList<long double> > gFreq;//notational convenience
	for(a0=0;a0<alleles;++a0){//initialize matrix  
		gFreq.Append(NEW aList<long double>());
		for(a1=0;a1<alleles;++a1){
			gFreq[a0].Append((long double)0.0);
			}}
	ObjList<aList<long double> > p, pt;//p[h][j]=pj^(h) in algebra notation, h<j
	for(a0=0;a0<alleles;++a0){
		p.Append(NEW aList<long double>());
		pt.Append(NEW aList<long double>());
		for(a1=0;a1<alleles;++a1){
			p[a0].Append((long double)0.0);
			pt[a0].Append((long double)0.0);
			}}//a0, a1
	for(a0=0;a0<alleles;++a0){//fill in values for p
		int& h = a0;//algebraic notation
		for(a1=0;a1<alleles;++a1){
			int& j=a1;//algebraic notation
			if(h==0){
					p[h][j] = freqs[j];}
				else if(j<h){
					p[h][j] = -1.0;}//error code for debugging
				else{
					long double sum_pi=(long double)0.0;
					for(int i=0;i<h;++i){
						sum_pi += freqs[i];}
					p[h][j] = freqs[j]/((long double)1.0-sum_pi);
					}
			}//a1=j
		}//a0=h
	aList<long double> lambda, Fmin, F;
	int t=0;//t = index of threshold frequency
	for(a0=0;a0<alleles;++a0){
		if(p[a0][a0] >= (long double)1.0){
				Fmin.Append((long double)0.0);}//handle divide-by-zero errors -- Fmin = 0 is arbitrary
			else{
				Fmin.Append(- p[a0][a0]/((long double)1.0 - p[a0][a0]));}
		if(a0==0){lambda.Append((long double)1.0);}
			else{
				lambda.Append(lambda[a0-1] * ((long double)1.0 - (long double)2.0 * p[a0-1][a0-1]));
				}
		F.Append(MAX((long double)Fis_, Fmin[a0]));
		if(Fis_ <= (float) Fmin[a0]){//find threshold; type conversion handles rounding error in Fmin calc
				t = a0;}
			else if((t==a0-1) && (a0==alleles-1)){//last allele is threshold
				t = a0;}
		}//a0
	if(Fis_ <= Fmin[0]){ t++;}//threshold defined as the allele above the cutoff, if Fis qualifies
	for(a0=0;a0<alleles;++a0){//fill in values for pt
		int& h = a0;//algebraic notation
		for(a1=0;a1<alleles;++a1){
			int& j=a1;//algebraic notation
			if(h==0){
					pt[h][j] = p[h][j];}
				else if(t > h-1){
					pt[h][j] = p[h][j];}//error code for debugging
				else{
					pt[h][j] = pt[h-1][j];}
			}//a1=j
		}//a0=h
	p = pt;
	for(a0=0;a0<alleles;++a0){//cols-- fill in matrix below diagonal
		for(a1=a0;a1<alleles;++a1){//rows
			int tt = MAX(t-1,0);
			if(a1==a0){//homozygotes
					if(t > a0){
							gFreq[a0][a0] = lambda[a0] * (p[a0][a0]*p[a0][a0] + F[a0] * p[a0][a0] * ((long double)1.0 - p[a0][a0]));}//accessing wrong p if 2 alleles
						else{
//							gFreq[a0][a0] = lambda[t] * (p[t][a0]*p[t][a0] + Fis_ * p[t][a0] * ((long double)1.0 - p[t][a0]));}
							gFreq[a0][a0] = lambda[tt] * (p[tt][a0]*p[tt][a0] + Fis_ * p[tt][a0] * ((long double)1.0 - p[tt][a0]));}
					}//end homozygotes
				else{//heterozygotes
					if(t > a0){
							gFreq[a1][a0] = gFreq[a0][a1] = (long double)2.0 * lambda[a0] * ((long double)1.0 - F[a0]) * p[a0][a0] * p[a0][a1];}
						else{
//							gFreq[a1][a0] = gFreq[a0][a1] = (long double)2.0 * lambda[t] * ((long double)1.0 - Fis_) * p[t][a0] *  p[t][a1];}
							gFreq[a1][a0] = gFreq[a0][a1] = (long double)2.0 * lambda[tt] * ((long double)1.0 - Fis_) * p[tt][a0] *  p[tt][a1];}
				}//end heterozygotes
			}}//a1,a0

	//restore genotypes to original order of allele frequencies
	ObjList<aList<long double> > sortedgFreqOrder(gFreq);//copies these
	for(a0=0;a0<alleles;++a0){
		for(a1=0;a1<=a0;++a1){
			gFreq[a0][a1] = sortedgFreqOrder[sortedOrder[a0]][sortedOrder[a1]];
			}}//a0,a1
	for(a0=0;a0<alleles;++a0){
		for(a1=0;a1<=a0;++a1){
			if(ABS(gFreq[a0][a1])<10.0e-15){//chop off rounding errors near zero
				gFreq[a0][a1] = (long double)0.0;}
			}//a1
		for(a1=a0+1;a1<alleles;++a1){
			gFreq[a0][a1] = (long double)0.0;}
		}//a0

	//convert to list notation
	GenotypeFreqMultiAlleleDist& gFreqList = *this;//notational convenience
	for(int a2=0;a2<alleles;++a2){
		for(int b2=a2;b2<alleles;++b2){
			aString* gt = NEW aString((char)(b2+'A'));
			gt->prepend((char)(a2+'A'));
			gTypes_.Append(gt);
			gFreqList.Append(gFreq[b2][a2]);			
			}
		}
	long double summ = sum();
	}//end GenotypeFreqMultiAlleleDist::CreateDistribution2
	



void GenotypeFreqMultiAlleleDist::PrintToScreen(void){
	for (int x=0;x<len();++x){
		long double p = Nth(x);
		if(gTypes_[x][0]<=gTypes_[x][1]){
			cout<<gTypes_[x]<<"  "<<p<<endl;}
	}//end for x
}//end GenotypeFreqMultiAlleleDist::PrintToScreen


aString* GenotypeFreqMultiAlleleDist::RandomGenotypeString(Ran3List& ran){
	int g = RandomGenotype(ran);
	aString* gt = NEW aString(gTypes_[g]);
	return gt;
	}//end GenotypeFreqMultiAlleleDist::RandomGenotypeString



#endif



