/*****************************************************
*
*	Fis_Fit_Q_Likelihood.cpp
*
*	These functions calculate the ln-likelihood of a Fis, Fit and allele frequencies
*	by applying a multinomial distribution.  Functions are available for dominant, codominant,
*	sex-linked dominant and sex-linked codominant markers.
*
******************************************************/

#ifndef Fis_Fit_Q_Likelihoodcp
#define Fis_Fit_Q_Likelihoodcpp = 1

#include "Fis_Fit_Q_Likelihood.h"
#include "Element.cpp"
#include "aList.cpp"



long double Fis_Fit_Q_Likelihood_Codominant(ParamRec& p, MetropolisHastingsData* data){
	//assumes multiple loci
	//handles constraints on minimum Fis and Fit
//	CodominantMarkerMCMCData* theData = (CodominantMarkerMCMCData*) data;
//	CodominantMarkerMCMCData* theData = (CodominantMarkerMCMCData*) (data->pointerToDerivedSelf_);//derived class copying trick
	CodominantMarkerMCMCData* theData = (CodominantMarkerMCMCData*) (data->GetPointerToDerivedSelf());//derived class copying trick
	Fis_Fit_Q_ParamRec* mcmc = (Fis_Fit_Q_ParamRec*) p.GetPointerToDerivedSelf();
	long double lnL = (long double) 0.0,lnLloc, lnLlocGlobal, p_i, P_i, p_j, P_j, homosum, gii, gij, Gii, Gij, Fprime_i, Fprime_ij, Fmin_i, Fmin_j;
	long double dq = (long double)1.0/(long double)2000.0;//assumes N=1000 -> this could be parameterized
	mcmc->lnL_Fis_ = mcmc->lnL_Fit_ = (long double)0.0;
//	long double sr=mcmc->SexRatio();
//	long double sr_m = (long double)1.0 - sr;
	int pop,loc,allele1, allele2;
	for(loc=0;loc<theData->nlociCodom_;++loc){
		if(theData->IsPolymorphic(loc)){//Fst & Nm undefined if not globally polymorphic
			//here simply ignore their likelihood if not polymorphic
			lnLloc = lnLlocGlobal = (long double) 0.0;
			//go through likelihood equation accumulating terms
				//Fis calcs
			for(pop=0;pop<theData->npop_;++pop){
				for(allele1=0;allele1<theData->allelesPresent_[loc].len();++allele1){
					gii = (long double) (theData->GetGenotypeCount(pop,loc,allele1,allele1));//homozgote of allele
					p_i = mcmc->q(pop,loc,allele1);
					Fmin_i = (long double)0.0;
					homosum = p_i * p_i;
					if(p_i > (long double)0.0){
							if(p_i <(long double)1.0){
								Fmin_i = MAX(-p_i/((long double)1.0 - p_i), -((long double)1.0 - p_i)/p_i);}
							Fprime_i = MAX(mcmc->Fis(), Fmin_i);
							for(allele2=0;allele2<theData->allelesPresent_[loc].len();++allele2){
								if(allele1==allele2) continue;
								p_j = mcmc->q(pop,loc,allele2);
								Fmin_j = Fprime_ij =(long double)0.0;
								if(p_j>(long double)0.0){
									Fmin_j = MAX(-p_j/((long double)1.0 - p_j), -((long double)1.0 - p_j)/p_j);
									Fprime_ij = MAX(Fmin_i, Fmin_i);}								
								homosum += Fprime_ij * p_i * p_j;
								}//allele2
							lnLloc += gii * ln(homosum);
							for(allele2=allele1+1;allele2<theData->allelesPresent_[loc].len();++allele2){
								p_j = mcmc->q(pop,loc,allele2);
								gij = (long double) (theData->GetGenotypeCount(pop,loc,allele1,allele2));//heterozgote of allele
								Fmin_j = Fprime_ij =(long double)0.0;
								if(p_j>(long double)0.0){
									Fmin_j = MAX(-p_j/((long double)1.0 - p_j), -((long double)1.0 - p_j)/p_j);
									Fprime_ij = MAX(Fmin_i, Fmin_i);}								
								if(p_j>(long double)0.0 && Fprime_ij<(long double)1.0){
										lnLloc += gij * (ln(p_i) + ln(p_j) + ln((long double)1.0 - Fprime_ij));}
									else{//ln[0] encountered
										if(gij>(long double)0.0){//else if 0ln0 do nothing
											lnLloc += (long double) -10.0e12;}//low likelihood so that algorithm will reject
										}
								}//allele2
							}
						else{//ln[0] undefined
							if(gii>(long double)0.0){
								lnLloc += (long double) -10e12;}//large number that algorithm will reject
								//else if 0ln[0] do nothing
							}								
					}//allele1
				
				}//end for pop

			//Fit calcs
			for(allele1=0;allele1<theData->allelesPresent_[loc].len();++allele1){
				Gii = (long double) (theData->GetGenotypeCount(loc,allele1,allele1));//homozgote of allele
				P_i = mcmc->Q(loc,allele1);
				Fmin_i = (long double)0.0;
				homosum = P_i * P_i;
				Fmin_i = MAX(-P_i/((long double)1.0 - P_i), -((long double)1.0 - P_i)/P_i);
				Fprime_i = MAX(mcmc->Fit(), Fmin_i);
				for(allele2=0;allele2<theData->allelesPresent_[loc].len();++allele2){
					if(allele1==allele2) continue;
					P_j = mcmc->Q(loc,allele2);
					Fmin_j = Fprime_ij =(long double)0.0;
					Fmin_j = MAX(-P_j/((long double)1.0 - P_j), -((long double)1.0 - P_j)/P_j);
					Fprime_ij = MAX(Fmin_i, Fmin_i);								
					homosum += Fprime_ij * P_i * P_j;
					}//allele2
				lnLloc += Gii * ln(homosum);
				for(allele2=allele1+1;allele2<theData->allelesPresent_[loc].len();++allele2){
					P_j = mcmc->Q(loc,allele2);
					Gij = (long double) (theData->GetGenotypeCount(loc,allele1,allele2));//heterozgote of allele
					Fmin_j = Fprime_ij =(long double)0.0;
					Fmin_j = MAX(-P_j/((long double)1.0 - P_j), -((long double)1.0 - P_j)/P_j);
					Fprime_ij = MAX(Fmin_i, Fmin_i);								
					if(Fprime_ij<(long double)1.0){
							lnLloc += Gij * (ln(P_i) + ln(P_j) + ln((long double)1.0 - Fprime_ij));}
						else{//ln[0] encountered
							if(Gij>(long double)0.0){//else if 0ln0 do nothing
								lnLloc += (long double) -10.0e12;}//low likelihood so that algorithm will reject
							}
					}//allele2
				}//allele1
			lnL += lnLloc + lnLlocGlobal;
			mcmc->lnL_Fis_ += lnLloc;	
			mcmc->lnL_Fit_ += lnLlocGlobal;	
			}//end if global polymorphism
		}//end for loc
	mcmc->lnL(lnL);
	return lnL;
	}//end Fis_Fit_Q_Likelihood_Codominant(ParamRec&, MetropolisHastingsData*)





long double Fis_Fit_Q_Likelihood_SexLinkedCodominant(ParamRec& p, MetropolisHastingsData* data){
	//permits multiple alleles
	//sex ratio may vary or not
//	SexLinkedCodominantMarkerMCMCData* theData = (SexLinkedCodominantMarkerMCMCData*) (data->pointerToDerivedSelf_);//derived class copying trick
	SexLinkedCodominantMarkerMCMCData* theData = (SexLinkedCodominantMarkerMCMCData*) (data->GetPointerToDerivedSelf());//derived class copying trick
	Fis_Fit_Q_ParamRec* mcmc = (Fis_Fit_Q_ParamRec*) p.GetPointerToDerivedSelf();
	long double lnL = (long double) 0.0,lnLloc, lnLlocGlobal, lnLtemp,lnLsr, p_i, P_i, p_j, P_j, homoval,
										gii, gij, gi, Gii, Gij, Gi, hetsum, Fprime_i, Fprime_ij, Fmin_i, Fmin_j;
	mcmc->lnL_Fis_ = mcmc->lnL_Fit_ = (long double)0.0;
	long double sr=mcmc->SexRatio();
	long double sr_m = (long double)1.0 - sr;
	int allele1, allele2, loc, pop, legal;
	for(loc=0;loc<theData->nlociSLCodom_;++loc){
		if(theData->IsPolymorphic(loc)){//Fstats undefined if not globally polymorphic
			//here simply ignore their likelihood if not polymorphic
			lnLloc = lnLlocGlobal = (long double) 0.0;
			//go through likelihood equation accumulating terms
				//Fis calcs
			for(pop=0;pop<theData->npop_;++pop){
				legal=true;
				lnLtemp = (long double)0.0;
				for(allele1=0;allele1<theData->allelesPresentSLCodom_[loc].len();++allele1){
					gii = (long double)(theData->GetGenotypeCountHomogametic(pop,loc,allele1,allele1));//homozgote of allele
					gi  = (long double)(theData->GetGenotypeCountHeterogametic(pop,loc,allele1));
					p_i = mcmc->q(pop,loc,allele1);
					Fmin_i = hetsum = (long double)0.0;										
					if(p_i > (long double)0.0){
							Fmin_i = MAX(-p_i/((long double)1.0 - p_i), -((long double)1.0 - p_i)/p_i);
							Fprime_i = MAX(mcmc->Fis(), Fmin_i);
							for(allele2=0;allele2<theData->allelesPresentSLCodom_[loc].len();++allele2){
								if(allele1==allele2)continue;
								p_j = mcmc->q(pop,loc,allele2);
								Fmin_j = (long double)0.0;
								if(p_j > (long double)0.0 && p_j<(long double)1.0){
									Fmin_j = MAX(-p_j/((long double)1.0 - p_j), -((long double)1.0 - p_j)/p_j);}
								Fprime_ij = MAX(Fprime_i, Fmin_j);
								hetsum += Fprime_ij * p_i * p_j;
								}//allele2
							lnLtemp += gi*ln(p_i);
							homoval = MAX((long double)1.0e-6, MIN((long double)1.0, p_i * p_i + hetsum));//handles rounding error & ln(0) issues
							lnLtemp += gii * ln(homoval);
							
							for(allele2=allele1+1;allele2<theData->allelesPresentSLCodom_[loc].len();++allele2){
								gij = (long double)(theData->GetGenotypeCountHomogametic(pop,loc,allele1,allele2));//heterozgote of allele
								p_j = mcmc->q(pop,loc,allele2);
								Fmin_j = (long double)0.0;
								if(p_j > (long double)0.0){
										if(p_j<(long double)1.0){
											Fmin_j = MAX(-p_j/((long double)1.0 - p_j), -((long double)1.0 - p_j)/p_j);}
										Fprime_ij = MAX(Fprime_i, Fmin_j);
										if(Fprime_ij<(long double)1.0){
												lnLtemp += gij * ln((long double)2.0 * p_i * p_j * ((long double)1.0 - Fprime_ij));}
											else{
												if(gij>(long double)0.0){
													legal = false;}
												}//Fprime_ij
										}
									else{
										if(gij>(long double)0.0){
											legal = false;}
										}//p_j
								}//allele2
							}//p_i
						
						else{//p_i == 0 so ln[0] undefined
							gij=(long double)0.0;//check if any allele copies observed
							for(allele2=allele1+1;allele2<theData->allelesPresentSLCodom_[loc].len();++allele2){
								gij += (long double)(theData->GetGenotypeCountHomogametic(pop,loc,allele1,allele2));}//heterozgote of allele
							if(gii + gij + gi>(long double)0.0){
								legal=false;}
								//else if 0ln[0] do nothing
							}//p_i							
					}//allele1
				if(legal){
						lnLloc += lnLtemp;}
					else{//large number that algorithm will reject
						lnLloc += (long double)-10.0e12;}
				}//end for pop

			//Fit calcs
			legal=true;
			lnLtemp = (long double)0.0;
			for(allele1=0;allele1<theData->allelesPresentSLCodom_[loc].len();++allele1){
				Gii = (long double)(theData->GetGenotypeCountHomogametic(loc,allele1,allele1));//homozgote of allele
				Gi  = (long double)(theData->GetGenotypeCountHeterogametic(loc,allele1));
				P_i = mcmc->Q(loc,allele1);
				Fmin_i = MAX(-P_i/((long double)1.0 - P_i), -((long double)1.0 - P_i)/P_i);
				Fprime_i = MAX(mcmc->Fit(), Fmin_i);
				hetsum = (long double)0.0;										
				for(allele2=0;allele2<theData->allelesPresentSLCodom_[loc].len();++allele2){
					if(allele1==allele2)continue;
					P_j = mcmc->Q(loc,allele2);
					Fmin_j = MAX(-P_j/((long double)1.0 - P_j), -((long double)1.0 - P_j)/P_j);
					Fprime_ij = MAX(Fprime_i, Fmin_j);
					hetsum += Fprime_ij * P_i * P_j;
					}//allele2
				lnLtemp += Gi*ln(P_i);
				homoval = MAX((long double)1.0e-6, MIN((long double)1.0, P_i * P_i + hetsum));//handles rounding error & ln(0) issues
				lnLtemp += Gii * ln(homoval);
				
				for(allele2=allele1+1;allele2<theData->allelesPresentSLCodom_[loc].len();++allele2){
					Gij = (long double)(theData->GetGenotypeCountHomogametic(loc,allele1,allele2));//heterozgote of allele
					P_j = mcmc->Q(loc,allele2);
					Fmin_j = (long double)0.0;
					if(P_j<(long double)1.0){
						Fmin_j = MAX(-P_j/((long double)1.0 - P_j), -((long double)1.0 - P_j)/P_j);}
					Fprime_ij = MAX(Fprime_i, Fmin_j);
					if(Fprime_ij<(long double)1.0){
							lnLtemp += Gij * ln((long double)2.0 * P_i * P_j * ((long double)1.0 - Fprime_ij));}
						else{
							if(Gij>(long double)0.0){
								legal = false;}
							}//Fprime_ij
					}//allele2
				}//allele1
			if(legal){
					lnLlocGlobal += lnLtemp;}
				else{//large number that algorithm will reject
					lnLlocGlobal += (long double)-10.0e12;}
				

			lnL += lnLloc + lnLlocGlobal;
			mcmc->lnL_Fis_ += lnLloc;	
			mcmc->lnL_Fit_ += lnLlocGlobal;	
			}//end if global polymorphism
		}//end for loc

	if(mcmc->SexRatioVaries()){
		int females,males;
		lnLsr = (long double)0.0;
		for(pop=0;pop<theData->npop_;++pop){
			females=males=0;
			if(theData->heterogameticSex()=='F'){
					females = theData->SampleSizeHeterogametic(pop);
					males = theData->SampleSizeHomogametic(pop);}
				else{
					males = theData->SampleSizeHeterogametic(pop);
					females = theData->SampleSizeHomogametic(pop);}
			lnLsr += females * ln(sr) + males * ln(sr_m);}
		lnL += lnLsr;
		}

	mcmc->lnL(lnL);
	return lnL;
	}//end Fis_Fit_Q_Likelihood_SexLinkedCodominant(ParamRec&, MetropolisHastingsData*)



/* Old version


long double Fis_Fit_Q_Likelihood_SexLinkedCodominant(ParamRec& p, MetropolisHastingsData* data){
	//permits multiple alleles
	//sex ratio may vary or not
	SexLinkedCodominantMarkerMCMCData* theData = (SexLinkedCodominantMarkerMCMCData*) (data->pointerToDerivedSelf_);//derived class copying trick
	Fis_Fit_Q_ParamRec* mcmc = (Fis_Fit_Q_ParamRec*) p.pointerToDerivedSelf;
	long double lnL = (long double) 0.0,lnLloc, lnLlocGlobal, lnLtemp,lnLsr, p_i, P_i, p_j, P_j, gAA, gAa, gA, GAA, GAa, GA,  Fprime_i, Fprime_ij, Fmin_i, Fmin_j;
	mcmc->lnL_Fis_ = mcmc->lnL_Fit_ = (long double)0.0;
	long double sr=mcmc->SexRatio();
	long double sr_m = (long double)1.0 - sr;
	int allele1, allele2, loc, pop, legal;
	for(loc=0;loc<theData->nlociSLCodom_;++loc){
		if(theData->IsPolymorphic(loc)){//Fstats undefined if not globally polymorphic
			//here simply ignore their likelihood if not polymorphic
			lnLloc = lnLlocGlobal = (long double) 0.0;
			//go through likelihood equation accumulating terms
				//Fis calcs
			for(pop=0;pop<theData->npop_;++pop){
				legal=true;
				lnLtemp = (long double)0.0;
				for(allele1=0;allele1<theData->allelesPresentSLCodom_[loc].len();++allele1){
					gAA = (long double)(theData->GetGenotypeCountHomogametic(pop,loc,allele1,allele1));//homozgote of allele
					gA  = (long double)(theData->GetGenotypeCountHeterogametic(pop,loc,allele1));
					p_i = mcmc->q(pop,loc,allele1);
					Fmin_i = (long double)0.0;					
					if(p_i > (long double)0.0){
							lnLtemp += ((long double)2.0 * gAA + gA) * ln(p_i);
							if(p_i<(long double)1.0){
								Fmin_i = MAX(-p_i/((long double)1.0 - p_i), -((long double)1.0 - p_i)/p_i);}
							}
						else{//ln[0] undefined
							if(gAA + gA>(long double)0.0){
								legal=false;}
								//else if 0ln[0] do nothing
							}								
					Fprime_i = MAX(mcmc->Fis(), Fmin_i);
					if(Fprime_i>(long double)-1.0){
							lnLtemp += gAA * ln((long double)1.0 + Fprime_i);}						
						else{//ln[0] undefined
							if(gAA>(long double)0.0){
								legal=false;}}
								//else if 0ln[0] do nothing

					for(allele2=allele1+1;allele2<theData->allelesPresentSLCodom_[loc].len();++allele2){
						gAa = (long double)(theData->GetGenotypeCountHomogametic(pop,loc,allele1, allele2));//heterozygotes with allele
						p_j = mcmc->q(pop,loc,allele2);
						Fmin_j = (long double)0.0;
						if(p_i > (long double)0.0){
								lnLtemp += gAa * ln(p_i);
								}
							else{//ln[0] undefined
								if(gAa>(long double)0.0){
									legal=false;}
									//else if 0ln[0] do nothing
								}						
						if(p_j > (long double)0.0){
								lnLtemp += gAa * ln(p_j);
								if(p_j<(long double)1.0){
									Fmin_j = MAX(-p_j/((long double)1.0 - p_j), -((long double)1.0 - p_j)/p_j);}
								}
							else{//ln[0] undefined
								if(gAa>(long double)0.0){
									legal=false;}
									//else if 0ln[0] do nothing
								}						
						Fprime_ij = MAX(Fprime_i, Fmin_j);
						if(Fprime_ij<(long double)1.0){
								lnLtemp += gAa * ln((long double)1.0 -Fprime_ij);}						
							else{
								if(gAa>(long double)0.0){//else if 0ln[0] do nothing
									legal=false;}}
						}//allele2
					
					}//allele1
				if(legal){
						lnLloc += lnLtemp;}
					else{//large number that algorithm will reject
						lnLloc += (long double)-10.0e12;}
				}//end for pop

			//Fit calcs
			legal=true;
			lnLtemp=(long double)0.0;
			for(allele1=0;allele1<theData->allelesPresentSLCodom_[loc].len();++allele1){
				GAA = (long double)(theData->GetGenotypeCountHomogametic(loc,allele1,allele1));//homozygote
				GA  = (long double)(theData->GetGenotypeCountHeterogametic(loc,allele1));
				P_i = mcmc->Q(loc,allele1);//never fixed
				if(P_i > (long double)0.0){//ln[0] undefined
						lnLtemp += ((long double)2.0 * GAA + GA) * ln(P_i);
						Fmin_i = MAX(-P_i/((long double)1.0 - P_i), -((long double)1.0 - P_i)/P_i);
						}
					else{//shouldn't get here
						if(GAA>(long double)0.0){
							legal=false;}
							//else if 0ln[0] do nothing
						}
				
				Fprime_i = MAX(mcmc->Fit(), Fmin_i);
				if(Fprime_i>(long double)-1.0){
						lnLtemp += GAA * ln((long double)1.0 + Fprime_i);}						
					else{//ln[0] undefined
						if(GAA>(long double)0.0){
							legal=false;}}
							//else if 0ln[0] do nothing

				for(allele2=allele1+1;allele2<theData->allelesPresentSLCodom_[loc].len();++allele2){
//					if(allele1 == allele2) continue;//skip
					P_j = mcmc->Q(loc,allele2);
					GAa = (long double)(theData->GetGenotypeCountHeterogametic(loc,allele1, allele2));//heterozygote
					if(P_i > (long double)0.0){//ln[0] undefined
							lnLtemp += GAa * ln(P_i);}
						else{//shouldn't get here
							if(GAa>(long double)0.0){
								legal=false;}
								//else if 0ln[0] do nothing
							}
					if(P_j > (long double)0.0){//ln[0] undefined
							lnLtemp += GAa * ln(P_j);}
						else{//shouldn't get here
							if(GAa>(long double)0.0){
								legal=false;}
								//else if 0ln[0] do nothing
							}
					Fmin_j = MAX(-P_j/((long double)1.0 - P_j), -((long double)1.0 - P_j)/P_j);//P_j can't be 1 or 0
					Fprime_ij = MAX(Fprime_i, Fmin_j);
					if(Fprime_ij<(long double)1.0){
							lnLtemp += GAa * ln((long double)1.0 - Fprime_ij);}						
						else{
							if(GAa>(long double)0.0){
								legal=false;}}
								//else if 0ln[0] do nothing
					}//allele2
				}//allele1
			if(legal){
					lnLlocGlobal += lnLtemp;}
				else{//large number that algorithm will reject
					lnLlocGlobal += (long double)-10.0e12;}
				

			lnL += lnLloc + lnLlocGlobal;
			mcmc->lnL_Fis_ += lnLloc;	
			mcmc->lnL_Fit_ += lnLlocGlobal;	
			}//end if global polymorphism
		}//end for loc

	if(mcmc->SexRatioVaries()){
		lnLsr = (long double)0.0;
		for(pop=0;pop<theData->npop_;++pop){
			int females = theData->SampleSizeHeterogametic(pop);
			int males = theData->SampleSizeHomogametic(pop);
			lnLsr += females * ln(sr) + males * ln(sr_m);}
		lnL += lnLsr;
		}

	mcmc->lnL(lnL);
	return lnL;
	}//end Fis_Fit_Q_Likelihood_SexLinkedCodominant(ParamRec&, MetropolisHastingsData*)

*/



long double Fis_Fit_Q_Likelihood_Dominant(ParamRec& p, MetropolisHastingsData* data){
	//assumes multiple loci
	//handles constraints on minimum Fis and Fit
//	DominantMarkerMCMCData* theData = (DominantMarkerMCMCData*) (data->pointerToDerivedSelf_);//derived class copying trick
	DominantMarkerMCMCData* theData = (DominantMarkerMCMCData*) (data->GetPointerToDerivedSelf());//derived class copying trick
	Fis_Fit_Q_ParamRec* mcmc = (Fis_Fit_Q_ParamRec*) p.GetPointerToDerivedSelf();
	long double lnL = (long double) 0.0,lnLloc, lnLlocGlobal, p_i, P_i, gAAgAa, gaa, GAAGAa, Gaa, val, Fprime_i, Fmin_i;
	long double dq = (long double)1.0/(long double)2000.0;//assumes N=1000 -> this could be parameterized
	mcmc->lnL_Fis_ = mcmc->lnL_Fit_ = (long double)0.0;
	int pop,loc;
	for(loc=0;loc<theData->nlociDom_;++loc){
		if(theData->IsPolymorphic(loc)){//Fst & Nm undefined if not globally polymorphic
			//here simply ignore their likelihood if not polymorphic
			lnLloc = lnLlocGlobal = (long double) 0.0;
			//go through likelihood equation accumulating terms
				//Fis calcs
			for(pop=0;pop<theData->npop_;++pop){
				gAAgAa = (long double)(theData->GetCountDom(pop,loc,1));//dominant marker
				gaa = (long double)(theData->GetCountDom(pop,loc,0));//dominant marker
				p_i = mcmc->q(pop,loc,1);
				Fmin_i = (long double)0.0;
//				if(p_i > (long double)0.0 && p_i < (long double)1.0){
//					Fmin_i = MAX(-p_i/((long double)1.0 - p_i), -((long double)1.0 - p_i)/p_i);}
				if(p_i == (long double)0.0 || p_i == (long double)1.0){
					continue;}//lnL=0 when the marker is fixed or absent
				Fmin_i = MAX(-p_i/((long double)1.0 - p_i), -((long double)1.0 - p_i)/p_i);
				Fprime_i = MAX(mcmc->Fis(), Fmin_i);
				val = (long double)1.0 - ((long double)1.0 - p_i)*((long double)1.0 - p_i)*((long double)1.0 + Fprime_i);
				if(val>(long double)0.0){
						lnLloc += gAAgAa * ln(val);}
					else{//ln[0] undefined
						if(gAAgAa>(long double)0.0){
							lnLloc += (long double) -10e10;}//large number that algorithm will reject
							//else if 0ln[0] do nothing
						}								
				if(p_i<(long double)1.0){
						lnLloc += (long double)2.0* gaa * ln((long double)1.0 - p_i);}
					else{//ln[0] undefined
						if(gaa>(long double)0.0){
							lnLloc += (long double) -10e10;}//large number that algorithm will reject
							//else if 0ln[0] do nothing
						}								

				if(Fprime_i>(long double)-1.0){
						lnLloc += gaa * ln((long double)1.0 + Fprime_i);}						
					else{//ln[0] undefined
						if(gaa>(long double)0.0){
							lnLloc += (long double) -10e10;}}//large number that algorithm will reject
							//else if 0ln[0] do nothing			
				}//end for pop

			//Fit calcs
			GAAGAa = (long double)(theData->GetCountDom(loc,1));//dominant marker
			Gaa = (long double)(theData->GetCountDom(loc,0));//dominant marker
			P_i = mcmc->Q(loc,1);
			if(P_i ==(long double)0.0 || P_i == (long double)1.0){
				continue;}//shouldn'tget here, but if so, lnL=0 when the marker is golbally fixed or absent
			Fmin_i = MAX(-P_i/((long double)1.0 - P_i), -((long double)1.0 - P_i)/P_i);
			Fprime_i = MAX(mcmc->Fit(), Fmin_i);
			val = (long double)1.0 - ((long double)1.0 - P_i)*((long double)1.0 - P_i)*((long double)1.0 + Fprime_i);
			if(val>(long double)0.0){
					lnLlocGlobal += GAAGAa * ln(val);}
				else{//ln[0] undefined
					if(GAAGAa>(long double)0.0){
						lnLlocGlobal += (long double) -10e10;}//large number that algorithm will reject
						//else if 0ln[0] do nothing
					}								
			lnLloc += (long double)2.0* Gaa * ln((long double)1.0 - P_i);

			if(Fprime_i>(long double)-1.0){
					lnLlocGlobal += Gaa * ln((long double)1.0 + Fprime_i);}						
				else{//ln[0] undefined
					if(Gaa>(long double)0.0){
						lnLlocGlobal += (long double) -10e10;}}//large number that algorithm will reject
						//else if 0ln[0] do nothing			
				

			lnL += lnLloc + lnLlocGlobal;
			mcmc->lnL_Fis_ += lnLloc;	
			mcmc->lnL_Fit_ += lnLlocGlobal;	
			}//end if global polymorphism
		}//end for loc
	mcmc->lnL(lnL);
	return lnL;
	}//end Fis_Fit_Q_Likelihood_Dominant(ParamRec&, MetropolisHastingsData*)







long double Fis_Fit_Q_Likelihood_SexLinkedDominant(ParamRec& p, MetropolisHastingsData* data){
	//assumes 2 markers per locus only
	//assumes constant sex ratio
//	CodominantMarkerMCMCData* theData = (CodominantMarkerMCMCData*) data;
//	SexLinkedDominantMarkerMCMCData* theData = (SexLinkedDominantMarkerMCMCData*) (data->pointerToDerivedSelf_);//derived class copying trick
	SexLinkedDominantMarkerMCMCData* theData = (SexLinkedDominantMarkerMCMCData*) (data->GetPointerToDerivedSelf());//derived class copying trick
	Fis_Fit_Q_DomParamRec* mcmc = (Fis_Fit_Q_DomParamRec*) p.GetPointerToDerivedSelf();
	long double lnL = (long double) 0.0,lnLloc, lnLlocGlobal, p_i, P_i, gAAgAa, gaa, gA, ga, GAAGAa, Gaa, GA, Ga, Fprime, Fmin, val1,val2;
	long double dq = (long double)1.0/(long double)2000.0;//assumes N=1000 -> this could be parameterized
	mcmc->lnL_Fis_ = mcmc->lnL_Fit_ = (long double)0.0;
	int loc, pop;
	for(loc=0;loc<theData->nlociSLDom_;++loc){
		if(theData->IsPolymorphic(loc)){//Fst & Nm undefined if not globally polymorphic
			//here simply ignore their likelihood if not polymorphic
			lnLloc = lnLlocGlobal = (long double) 0.0;
			//go through likelihood equation accumulating terms
				//Fis calcs
			for(pop=0;pop<theData->npop_;++pop){
				gAAgAa = (long double)(theData->GetCountDomHomogametic(pop,loc,1));//dominant marker
				gaa = (long double)(theData->GetCountDomHomogametic(pop,loc,0));
				gA  = (long double)(theData->GetCountDomHeterogametic(pop,loc,1));
				ga  = (long double)(theData->GetCountDomHeterogametic(pop,loc,0));
				p_i = mcmc->q_hetero(pop,loc,1);//allele frequency estimate
				Fmin = (long double)0.0;
				if(p_i > (long double)0.0 && p_i < (long double)1.0){
					Fmin = MAX(-p_i/((long double)1.0 - p_i), -((long double)1.0 - p_i)/p_i);}
//				if(p_i == (long double)0.0 || p_i == (long double)1.0){
//					continue;}//lnL=0 when the marker is fixed
//				Fmin = MAX(-p_i/((long double)1.0 - p_i), -((long double)1.0 - p_i)/p_i);
				Fprime = MAX(mcmc->Fis(), Fmin);
				val1 = (long double)1.0 + ((long double)1.0 - p_i)*((long double)1.0 - Fprime);
//				val1 = (long double)1.0 + ((long double)1.0 - p_i)*( Fprime);
				if(val1>(long double)0.0){
						lnLloc += gAAgAa * ln(val1);}
					else{//ln[0] undefined
						if(gAAgAa>(long double)0.0){
							lnLloc += (long double) -10e12;}//large number that algorithm will reject
							//else if 0ln[0] do nothing
						}								
				val2 = (long double)1.0 - p_i*((long double)1.0 - Fprime);
//				val2 = (long double)1.0 - p_i*(Fprime);//backwards but it works!!!
//				if(val2>(long double)0.0){//getting bug with val2=-0.0 not being recognized here
				if(val2>(long double)0.0 && val2 !=(long double)-0.0){
						lnLloc += gaa * ln(val2);}
					else{//ln[0] undefined
						if(gaa>(long double)0.0){
							lnLloc += (long double) -10e12;}//large number that algorithm will reject
							//else if 0ln[0] do nothing
						}								
				if(p_i>(long double)0.0){
						lnLloc += (gAAgAa + gA) * ln(p_i);}
					else{//ln[0] undefined
						if(gA>(long double)0.0){
							lnLloc += (long double) -10e10;}//large number that algorithm will reject
							//else if 0ln[0] do nothing
						}								
				if(p_i<(long double)1.0){
						lnLloc += (gaa + ga) * ln((long double)1.0 - p_i);}
					else{//ln[0] undefined
						if((gaa+ga)>(long double)0.0){
							lnLloc += (long double) -10e12;}//large number that algorithm will reject
							//else if 0ln[0] do nothing
						}								

				}//end for pop

			//Fit calcs
			GAAGAa = (long double)(theData->GetCountDomHomogametic(loc,1));//dominant marker
			Gaa = (long double)(theData->GetCountDomHomogametic(loc,0));//recessive marker
			GA = (long double)(theData->GetCountDomHeterogametic(loc,1));//dominant marker
			Ga = (long double)(theData->GetCountDomHeterogametic(loc,0));//recessive marker
			P_i = mcmc->Q_hetero(loc,1);//allele frequency estimate
			if(P_i == (long double)0.0 || P_i == (long double)1.0){
				continue;}//shouldn't happen, but if so, lnL=0 when the marker is globally fixed
			Fmin = MAX(-P_i/((long double)1.0 - P_i), -((long double)1.0 - P_i)/P_i);
			Fprime = MAX(mcmc->Fit(), Fmin);
			val1 = (long double)1.0 + ((long double)1.0 - P_i)*((long double)1.0 - Fprime);
//			val1 = (long double)1.0 + ((long double)1.0 - P_i)*(Fprime);
			if(val1>(long double)0.0){
					lnLlocGlobal += GAAGAa * ln(val1);}
				else{//ln[0] undefined
					if(GAAGAa>(long double)0.0){
						lnLlocGlobal += (long double) -10e12;}//large number that algorithm will reject
						//else if 0ln[0] do nothing
					}								
			val2 = (long double)1.0 - P_i*((long double)1.0 - Fprime);
//			val2 = (long double)1.0 - P_i*( Fprime);
			if(val2>(long double)0.0){
					lnLlocGlobal += Gaa * ln(val2);}
				else{//ln[0] undefined
					if(Gaa>(long double)0.0){
						lnLlocGlobal += (long double) -10e12;}//large number that algorithm will reject
						//else if 0ln[0] do nothing
					}								
			lnLlocGlobal += (GAAGAa + GA) * ln(P_i);
			lnLlocGlobal += (Gaa + Ga) * ln((long double)1.0 - P_i);
				

			lnL += lnLloc + lnLlocGlobal;
			mcmc->lnL_Fis_ += lnLloc;	
			mcmc->lnL_Fit_ += lnLlocGlobal;	
			}//end if global polymorphism
		}//end for loc

	if(mcmc->SexRatioVaries()){
		long double lnLsr;
		long double sr=mcmc->SexRatio();
		long double sr_m = (long double)1.0 - sr;
		int females,males;
		lnLsr = (long double)0.0;
		for(pop=0;pop<theData->npop_;++pop){
			females=males=0;
			if(theData->heterogameticSex()=='F'){
					females = theData->SampleSizeHeterogametic(pop);
					males = theData->SampleSizeHomogametic(pop);}
				else{
					males = theData->SampleSizeHeterogametic(pop);
					females = theData->SampleSizeHomogametic(pop);}
			lnLsr += females * ln(sr) + males * ln(sr_m);}
		lnL += lnLsr;
		}

	mcmc->lnL(lnL);
	return lnL;
	}//end Fis_Fit_Q_Likelihood_SexLinkedDominant(ParamRec&, MetropolisHastingsData*)





#endif
