/*

PopPhenoLandScapeInterface.cpp

*/

#ifndef PopPhenoLandScapeInterfacecpp
#define PopPhenoLandScapeInterfacecpp = 1

#include "PopPhenoLandScapeInterface.h"
#include "StringList.cpp"
#include "Element.cpp"
#include "aList.cpp"



void PopPhenoLandScapeInterface::DeleteIndividualsWithStates(
				StringList& undesirableStateNames, ObjList<StringList>& undesirableStates){
			//removes individuals with the listed state values
			//Uses AND as sorting criterion when multiple state names given
			//states are attributes (id, age, sex, etc.) of Individual and derived classes,
			//not traits stored in class Phenotype
	aList<int> stateNameLoc;
	for(int s=0;s<undesirableStateNames.len();++s){
		aString& stname = undesirableStateNames[s];
		int stnameLoc = indivStateNames.Loc(&stname);
		stateNameLoc.Append(stnameLoc);
		}
	for(int pop=0;pop<npop_;++pop){
		for(int i=popSize[pop]-1;i>=0;--i){
			int deleteThisIndiv = 1;//delete by default
			StringList& thisIndivStates = indivStateScores[pop][i];
			for(int st=0;st<stateNameLoc.len();++st){
				int locInList = stateNameLoc[st];
				for(int stScore=0;stScore<undesirableStates[st].len();++stScore){
					aString& uState = undesirableStates[st][stScore];
					if(!(thisIndivStates.Includes(&uState))){
						deleteThisIndiv = 0;
						break;
						}
					}//end for stScore
				if (!(deleteThisIndiv)){
					break;
					}
				}//end for st
			if(deleteThisIndiv){
				rawTraits[pop].Delete(i);
				traitStrs[pop].Delete(i);
				popSize[pop]--;
				}//end if deleteThisIndiv
			}//end for i
		}//end for pop
	ReadTraitStrings();
	}//end PopPhenoLandScapeInterface::DeleteIndividualsWithStates()


/*
void PopPhenoLandScapeInterface::DeleteIndividualsWithStates(
				StringList& undesirableStateNames, ObjList<StringList>& undesirableStates){
			//removes individuals with the listed state values
			//Uses AND as sorting criterion when multiple state names given
			//states are attributes (id, age, sex, etc.) of Individual and derived classes,
			//not traits stored in class Phenotype
	aList<int> stateNameLoc;
	for(int s=0;s<undesirableStateNames.len();++s){
		aString& stname = undesirableStateNames[s];
		int stnameLoc = indivStateNames.Loc(&stname);
		stateNameLoc.Append(stnameLoc);
		}
	for(int pop=0;pop<npop;++pop){
		aList<int> scoredIndivsKept;
		for(int tr=0;tr<popTraitStats[pop].len();++tr){
			scoredIndivsKept.Append(0);}
		for(int i=popSize[pop]-1;i>=0;--i){
			int deleteThisIndiv = 1;//delete by default
			StringList& thisIndivStates = indivStateScores[pop][i];
			for(int st=0;st<stateNameLoc.len();++st){
				int locInList = stateNameLoc[st];
				for(int stScore=0;stScore<undesirableStates[st].len();++stScore){
					aString& uState = undesirableStates[st][stScore];
					if(!(thisIndivStates.Includes(&uState))){
						deleteThisIndiv = 0;
						break;
						}
					}//end for stScore
				if (!(deleteThisIndiv)){
					break;
					}
				}//end for st
			if(deleteThisIndiv){
					indivStateScores[pop].Delete(i);
					rawTraits[pop].Delete(i);
					traitStrs[pop].Delete(i);
					traits[pop].Delete(i);
					for(int tr=traitsScored[pop][i].len()-1;tr>=0;--tr){
						if(traitsScored[pop][i][tr] != 0){//it was scored and should be deleted
							int itemToDelete = popTraitStats[pop][tr].len() -1 - scoredIndivsKept[tr];
							popTraitStats[pop][tr].Delete(itemToDelete);
							}//end if
						}//end for tr
					traitsScored[pop].Delete(i);
					}//end if deleteThisIndiv
				else{
					for(int tr=0;tr<popTraitStats[pop].len();++tr){
						int itemToIncrement = popTraitStats[pop][tr].len() -1 - scoredIndivsKept[tr];
						if(traitsScored[pop][itemToIncrement][tr] != 0){
							scoredIndivsKept[tr]++;}
						}//end for tr
					}//end else
			}//end for i
		}//end for pop
	}//end PopPhenoLandScapeInterface::DeleteIndividuals()

*/

void PopPhenoLandScapeInterface::KeepIndividualsWithStates(
				StringList& desirableStateNames,ObjList<StringList>& desirableStates){
			//removes individuals EXCEPT those with the listed state values
			//Uses AND as sorting criterion when multiple state names given
			//states are attributes (id, age, sex, etc.) of Individual and derived classes,
			//not traits stored in class Phenotype
	aList<int> stateNameLoc;
	for(int s=0;s<desirableStateNames.len();++s){
		aString& stname = desirableStateNames[s];
		int stnameLoc = indivStateNames.Loc(&stname);
		stateNameLoc.Append(stnameLoc);
		}
	for(int pop=0;pop<npop_;++pop){
		for(int i=popSize[pop]-1;i>=0;--i){
			int keepThisIndiv = 1;//keep by default
			StringList& thisIndivStates = indivStateScores[pop][i];
			for(int st=0;st<stateNameLoc.len();++st){//compare state by state
				int locInList = stateNameLoc[st];
				aString& itsState = thisIndivStates[locInList];
				if(!( desirableStates[st].Includes(&itsState))){
					keepThisIndiv = 0;//this individual has none of the possibly desirable traits
					break;
					}
				}//end for st
			if(!(keepThisIndiv)){
				rawTraits[pop].Delete(i);
				traitStrs[pop].Delete(i);
				popSize[pop]--;
				}//end if deleteThisIndiv
			}//end for i
		}//end for pop
	ReadTraitStrings();
	}//end PopPhenoLandScapeInterface::KeepIndividualsWithStates()
	
	
	
/*
void PopPhenoLandScapeInterface::KeepIndividualsWithStates(
				StringList& desirableStateNames,ObjList<StringList>& desirableStates){
			//removes individuals EXCEPT those with the listed state values
			//Uses AND as sorting criterion when multiple state names given
			//states are attributes (id, age, sex, etc.) of Individual and derived classes,
			//not traits stored in class Phenotype
	aList<int> stateNameLoc;
	for(int s=0;s<desirableStateNames.len();++s){
		aString& stname = desirableStateNames[s];
		int stnameLoc = indivStateNames.Loc(&stname);
		stateNameLoc.Append(stnameLoc);
		}
	for(int pop=0;pop<npop;++pop){
		aList<int> scoredIndivsKept;
		for(int tr=0;tr<popTraitStats[pop].len();++tr){
			scoredIndivsKept.Append(0);}
		for(int i=popSize[pop]-1;i>=0;--i){
			int keepThisIndiv = 1;//keep by default
			StringList& thisIndivStates = indivStateScores[pop][i];
			for(int st=0;st<stateNameLoc.len();++st){
				int locInList = stateNameLoc[st];
				for(int stScore=0;stScore<desirableStates[st].len();++stScore){
					aString& uState = desirableStates[st][stScore];
					if(!(thisIndivStates.Includes(&uState))){
						keepThisIndiv = 0;
						break;
						}
					}//end for stScore
				if (!(keepThisIndiv)){
					break;
					}
				}//end for st
			if(!(keepThisIndiv)){
					indivStateScores[pop].Delete(i);
					rawTraits[pop].Delete(i);
					traitStrs[pop].Delete(i);
					traits[pop].Delete(i);
					for(int tr=traitsScored[pop][i].len()-1;tr>=0;--tr){
						if(traitsScored[pop][i][tr] != 0){//it was scored and should be deleted
							int itemToDelete = popTraitStats[pop][tr].len() -1 - scoredIndivsKept[tr];
							popTraitStats[pop][tr].Delete(itemToDelete);
							}//end if
						}//end for tr
					traitsScored[pop].Delete(i);
					}//end if deleteThisIndiv
				else{
					for(int tr=0;tr<popTraitStats[pop].len();++tr){
						int itemToIncrement = popTraitStats[pop][tr].len() -1 - scoredIndivsKept[tr];
						if(traitsScored[pop][itemToIncrement][tr] != 0){
							scoredIndivsKept[tr]++;}
						}//end for tr
					}//end else
			}//end for i
		}//end for pop
	}//end PopPhenoLandScapeInterface::KeepIndividuals()

*/


/*
void PopPhenoLandScapeInterface::PrintSASFormatData(ostream& outFile){
	//prints a data set that can be read using SAS
	//missing data represented by '.', columns separated by spaces
	outFile<<"pop indiv ";//this and next loop comprise the column titles
	aString titleLine;
	for(int t=0;t<ntraits-1;++t){
		titleLine.cat(traitName[t]);
		titleLine.cat(' ');
		}//end for loc
	titleLine.cat(traitName[ntraits-1]);
	outFile<<titleLine<<endl;
	for(int pop=0;pop<npop;++pop){
		aString popStr(pop+1);
		popStr.cat(' ');
		for(long i=0;i<popSize[pop];++i){
			aString indivStr(popStr);
			indivStr.cat(i+1);
			indivStr.cat(' ');
			int nthTrait=-1;
			for(int t=0;t<ntraits;++t){
				if(traitsScored[pop][i].Includes(t)){
						nthTrait++;
						indivStr.cat(traits[pop][i][nthTrait]);
						indivStr.cat(' ');}
					else{//trait is missing in this indiv
						indivStr.cat(" . ");}					
				}//end for t
				outFile<<indivStr<<endl;
			}//end for i
		}//end for pop
	outFile<<endl;		
	}//end  PopulationPhenotypicData::PrintSASFormatData(ostream&)
*/	
	


void PopPhenoLandScapeInterface::ReadGeneticLandScape(GeneticLandScape& theLandScape, 
				int species, aString& simTitle, aList<int>& skippedTraitNumbers, int includeGenotype){
	int traitsToSortThrough = theLandScape.traits(species);
	title = simTitle;

	npop_ = theLandScape.area();
	ntraits = theLandScape.traits(species);
//	maxall= 0;
	ngroups = 1;//default value

	//read individual state titles and scores
	indivStateNames.Clear();//id, sex, age, etc. traits not stored in Phenotype class
	indivStateScores.Clear();//listed for each population


	aString endCharList(" \t,;:\"(){}[]!?");

	//make trait names
	for(int t=0;t<ntraits;++t){
		aString *trName = new aString("TRAIT_");
		aString trNum(t+1);
		trNum.PrependCharToLen('0', Digits(ntraits));
		trName->cat(trNum);
		rawTraitName.Append(trName);
		}//end for t
	//read in the number of traits to be skipped in the analyses
	traitsSkipped = skippedTraitNumbers.len();
	skippedTraitNum = skippedTraitNumbers;
	//and read in the names of skipped traits, if any
	for(int t=0;t<traitsSkipped;++t){
		aString *skipName = new aString("TRAIT_");
		aString skipNum(skippedTraitNumbers[t]+1);
		skipNum.PrependCharToLen('0', Digits(ntraits));
		skipName->cat(skipNum);
		skippedTraitName.Append(skipName);
		}//end for t

	traitName.Copy(rawTraitName);
	aList<int> actualTraitNumbers;
	for(int t=0;t<ntraits;++t){actualTraitNumbers.Append(t);}
	for (int t=0;t<skippedTraitNumbers.len();++t){
		aString& toSkip = skippedTraitName[t];
		int location = traitName.Loc(&toSkip);
		traitName.Delete(location);
		actualTraitNumbers.Delete(location);
		}//end for t
	ntraits -= traitsSkipped;
	//read in population data
	aList<int> popID;
	int count=0;
	for (int pop=0;pop<theLandScape.area();++pop){
//	for (int pop=0;pop<popsBySpecies[species];++pop){
			popID.Append(0);}
	for (int pop=0;pop<theLandScape.area();++pop){
//	for (int pop=0;pop<popsBySpecies[species];++pop){
		for(int s=0;s<theLandScape.species();++s){
			popID[pop]=count++;}}
	ntraits++;//adding fitness as a trait
	aString *fitStr=new aString("fitness");
	rawTraitName.Append(fitStr);
	fitStr=new aString("fitness");//twice
	traitName.Append(fitStr);
	fitStr=NULL;//forget
	if(includeGenotype){//write locus names
		int nloci = theLandScape.loci(species);
		CoordinateType xy(-1,-1);
		theLandScape.GetCoordinates(popID[0],xy);
		Population& thePopP = theLandScape.GetPopulation(species, xy);
		GeneticPopulation* thePopG = (GeneticPopulation*) &thePopP;
		for (int loc=0;loc<nloci;++loc){
			aString locName("LOC_");locName.cat(loc+1);
			for(int a=0;a<theLandScape.ploidy(species, loc);++a){
				aString* gstr = thePopG->GenotypeString(0,loc,a," ");
				char count='A';
				while(gstr->len()>0){
					aString* gtWord = gstr->ChopOffFirstWord(endCharList);
					gtWord->CullChar(' ');
					if(gtWord->len()>0){
							*gtWord=locName;
							gtWord->cat('_');gtWord->cat(count);
							traitName.Append(gtWord);
							ntraits++;//added a trait
							count++;
							gtWord=NULL;//forget
							}
						else{
							delete gtWord; gtWord=NULL;}
					}
				delete gstr; gstr=NULL;
				}//end for a
			}//end for loc
		}//end if includeGenotype

	//read in population data
	int emptyPops = 0;
	for (int pop=0;pop<npop_;++pop){
		CoordinateType xy(-1,-1);
		theLandScape.GetCoordinates(species,pop,xy);
		Population& thePopP = theLandScape.GetPopulation(species, xy);
		GeneticPopulation* thePopG = (GeneticPopulation*) &thePopP;
		GeneticPopulation& thePop = *thePopG;
		if(thePop.size()==0){
				++emptyPops;}
			else{
				//put the {x,y} coordinates of the population onto its name
				aString *s = new aString("Population ");
				s->cat(pop+1);
				s->cat(" y= "); s->cat(xy[1]);
				s->cat(" x= "); s->cat(xy[0]);
				popName.Append(s);
				
				if(pop==0){
					Individual& indiv = thePop.GetIndividual(0);
					StringList* statenames = indiv.stateNames();
					indivStateNames = *statenames;
					delete statenames; statenames=NULL;
					}
				indivStateScores.Append (new ObjList<StringList>());
				
				
				//make a new sublist for the phenotypes of this population
				StringList *rawPhList = new StringList();
				rawTraits.Append(rawPhList);				
				popSize.Append(thePop.size());

				// BEAST is the string for a particular individual
				for(int i=0;i<thePop.size();++i){
					aString beastID(i+1);
					Individual& indiv = thePop.GetIndividual(i);
					GeneticIndividual* gIndiv = (GeneticIndividual*) &indiv;
					indivStateScores[pop].Append(gIndiv->states());
					aString* beast = new aString();
					gIndiv->SetGenotypeString(*beast);
					beast->prepend(beastID);
					//now get its fitness
					double fitness = gIndiv->fitness();
					beast->cat(' ');
					beast->cat(fitness);
						if(includeGenotype){
							beast->cat(' ');
							aString* g = gIndiv->GenotypeString(" ");
							beast->cat(*g);
							delete g; g=NULL;
							}//end if includeGenotype
					rawPhList->Append(beast);
					beast=NULL;//forget
					}//end for i
				rawPhList=NULL;
			}//end if size()...else
		}//end for pop
	npop_ -= emptyPops;

	GroupMembership();
	CullSkippedTraits();
	ReadTraitStrings();
	 }//end PopPhenoLandScapeInterface::ReadGeneticLandScape



void PopPhenoLandScapeInterface::ReadGeneticLandScape(GeneticLandScape& theLandScape, 
				aString& simTitle, aList<int>& skippedTraitNumbers, int includeGenotype){
		//all species; can't handle haplodiploid or sex-linked loci
	Clear();
	title = simTitle;
	aString endCharList(" \t,;:\"(){}[]!?");
	npop_=ntraits=0; ngroups=theLandScape.species();
	aList<int> popsBySpecies;
	aList<int> totalLociBySpecies;
	for(int s=0;s<theLandScape.species();++s){
		npop_ += theLandScape.area();
		popsBySpecies.Append(theLandScape.area());
		int spTraits = theLandScape.traits(s); //for this species
		totalLociBySpecies.Append(spTraits);
		ntraits = MAX(ntraits, (long)spTraits);

		//make locus names
		for(int tr=0;tr<spTraits;++tr){
			aString *trName = new aString("TRAIT_");
			aString trNum(tr+1);
			trNum.PrependCharToLen('0', Digits(ntraits));
			trName->cat(trNum);
			if(rawTraitName.Includes(trName)){
					delete trName;}
				else{
					rawTraitName.Append(trName);}
			 trName=NULL;
			}//end for tr
		//read in the number of traits to be skipped in the analyses
		traitsSkipped = skippedTraitNumbers.len();
		skippedTraitNum = skippedTraitNumbers;
		//and read in the names of skipped loci, if any
		for(int tr=0;tr<traitsSkipped;++tr){
			aString *skipName = new aString("TRAIT_");
			aString skipNum(skippedTraitNumbers[tr]+1);
			skipNum.PrependCharToLen('0', Digits(ntraits));
			skipName->cat(skipNum);
			if(skippedTraitName.Includes(skipName)){
					delete skipName;}
				else{
					skippedTraitName.Append(skipName);}
			 skipName=NULL;
			}//end for tr
		}//end for s

	traitName.Copy(rawTraitName);
	aList<int> actualTraitNumbers;
	for(int tr=0;tr<ntraits;++tr){actualTraitNumbers.Append(tr);}
	for (int tr=0;tr<skippedTraitNumbers.len();++tr){
		aString& toSkip = skippedTraitName[tr];
		int location = traitName.Loc(&toSkip);
		traitName.Delete(location);
		actualTraitNumbers.Delete(location);
		}//end for tr
	ntraits -= traitsSkipped;
	//read in population data
	ObjList<aList<int> > popID;
	int count=0;
	for(int s=0;s<theLandScape.species();++s){
		aList<int>* theList = new aList<int>();
		popID.Append(theList);
		for (int pop=0;pop<popsBySpecies[s];++pop){
			theList->Append(0);}}
	for (int pop=0;pop<popsBySpecies[0];++pop){
		for(int s=0;s<theLandScape.species();++s){
			popID[s][pop]=count++;}}
	ntraits++;//adding fitness as a trait
	aString *fitStr=new aString("fitness");
	rawTraitName.Append(fitStr);
	fitStr=new aString("fitness");//twice
	traitName.Append(fitStr);
	fitStr=NULL;//forget
	if(includeGenotype){//write locus names
		for(int s=0;s<theLandScape.species();++s){
		int nloci = theLandScape.loci(s);
		CoordinateType xy(-1,-1);
		theLandScape.GetCoordinates(popID[s][0],xy);
		Population& thePopP = theLandScape.GetPopulation(s, xy);
		GeneticPopulation* thePopG = (GeneticPopulation*) &thePopP;
		for (int loc=0;loc<nloci;++loc){
			aString locName("LOC_");locName.cat(loc+1);
			for(int a=0;a<theLandScape.ploidy(s, loc);++a){
				aString* gstr = thePopG->GenotypeString(0,loc,a," ");
				char count='A';
				while(gstr->len()>0){
					aString* gtWord = gstr->ChopOffFirstWord(endCharList);
					gtWord->CullChar(' ');
					if(gtWord->len()>0){
							*gtWord=locName;
							gtWord->cat('_');gtWord->cat(count);
							traitName.Append(gtWord);
							ntraits++;//added a trait
							count++;
							gtWord=NULL;//forget
							}
						else{
							delete gtWord; gtWord=NULL;}
					}
				delete gstr; gstr=NULL;
				}//end for a
			}//end for loc
		}//end for s
		}//end if includeGenotype


			
	int emptyPops = 0;
	for(int s=0;s<theLandScape.species();++s){
		int nloci = theLandScape.loci(s);
		for (int pop=0;pop<popsBySpecies[s];++pop){
			CoordinateType xy(-1,-1);
			theLandScape.GetCoordinates(popID[s][pop],xy);
			Population& thePopP = theLandScape.GetPopulation(s, xy);
			GeneticPopulation* thePopG = (GeneticPopulation*) &thePopP;
			GeneticPopulation& thePop = *thePopG;
			
			if(thePop.size()==0){
					++emptyPops;}
				else{
					//put the {x,y} coordinates of the population onto its name
					aString *str = new aString("Population_");//locality first as one word
//					str->cat(pop+1);
					str->cat("y="); str->cat(xy[1]);
					str->cat("_x="); str->cat(xy[0]);
					str->cat(" of species_"); str->cat(s);//species last, as one word
					popName.Append(str);

					if(pop==0){
						Individual& indiv = thePop.GetIndividual(0);
						StringList* statenames = indiv.stateNames();
						indivStateNames = *statenames;
						delete statenames; statenames=NULL;
						}
					indivStateScores.Append (new ObjList<StringList>());
				
				
					//make a new sublist for the phenotypes of this population
					StringList *rawPhList = new StringList();
					rawTraits.Append(rawPhList);				
					popSize.Append(thePop.size());
					// BEAST is the string for a particular individual
					for(int i=0;i<thePop.size();++i){
						Individual& indiv = thePop.GetIndividual(i);
						indivStateScores[pop].Append(indiv.states());
						aString* beast = thePop.GetPhenotypeString(i," ");
						aString id(i+1);
						id.PrependCharToLen('0',4);id.cat(' ');
						beast->prepend(id);
						double fitness = thePop.fitness(i);
						beast->cat(' ');
						beast->cat(fitness);
						if(includeGenotype){
							beast->cat(' ');
//							aString* g = thePop.GenotypeString(i,' ');
//							beast->cat(*g);
//							delete g; g=NULL;
							for (int loc=0;loc<nloci;++loc){
								for(int a=0;a<theLandScape.ploidy(s, loc);++a){
									aString* gstr = thePopG->GenotypeString(i,loc,a," ");
									while(gstr->len()>0){
										aString* gtWord = gstr->ChopOffFirstWord(endCharList);
										gtWord->CullChar(' ');
										if(gtWord->len()>0){
											beast->cat(' ');
											beast->cat(*gtWord);
											}
										delete gtWord; gtWord=NULL;
										}
									delete gstr; gstr=NULL;
									}//end for a
								}//end for loc
							}//end if includeGenotype
						rawPhList->Append(beast);
						beast=NULL;
						}//end for i
					rawPhList=NULL;
				}//end if size()...else
			}//end for pop
		}//end for species
	npop_ -= emptyPops;

	GroupMembership();
	LocalityMembership();
	CullSkippedTraits();
	ReadTraitStrings();
	 }//end PopPhenoLandScapeInterface::ReadGeneticLandScape


void PopPhenoLandScapeInterface::ReadListOfIndividuals(ObjList<Individual>& indivList, 
				aString& simTitle, aList<int>& skippedTraitNumbers, int includeGenotype){
	GeneticIndividual *indiv1 = (GeneticIndividual*) &(indivList[0]);
	int traitsToSortThrough = indiv1->traits();
	title = simTitle;
	aString endCharList(" \t,;:\"(){}[]!?");

	npop_ = 1;
	ntraits = traitsToSortThrough;
	ngroups = 1;
	
	indivStateNames.Clear();
	indivStateScores.Clear();

	//make trait names
	for(int t=0;t<ntraits;++t){
		aString *trName = new aString("TRAIT_");
		aString trNum(t+1);
		trNum.PrependCharToLen('0', Digits(ntraits));
		trName->cat(trNum);
		rawTraitName.Append(trName);
		}//end for t
	//read in the number of traits to be skipped in the analyses
	traitsSkipped = skippedTraitNumbers.len();
	skippedTraitNum = skippedTraitNumbers;
	//and read in the names of skipped traits, if any
	for(int t=0;t<traitsSkipped;++t){
		aString *skipName = new aString("TRAIT_");
		aString skipNum(skippedTraitNumbers[t]+1);
		skipNum.PrependCharToLen('0', Digits(ntraits));
		skipName->cat(skipNum);
		skippedTraitName.Append(skipName);
		}//end for t

	traitName.Copy(rawTraitName);
	aList<int> actualTraitNumbers;
	for(int t=0;t<ntraits;++t){actualTraitNumbers.Append(t);}
	for (int t=0;t<skippedTraitNumbers.len();++t){
		aString& toSkip = skippedTraitName[t];
		long location = traitName.Loc(&toSkip);
		traitName.Delete(location);
		actualTraitNumbers.Delete(location);
		}//end for t
	ntraits -= traitsSkipped;

	ntraits++;//adding fitness as a trait
	aString *fitStr=new aString("fitness");
	rawTraitName.Append(fitStr);
	fitStr=new aString("fitness");//twice
	traitName.Append(fitStr);
	fitStr=NULL;//forget

	int nloci = indiv1->loci();
	if(includeGenotype){//write locus names
		for (int loc=0;loc<nloci;++loc){
			aString locName("LOC_");locName.cat(loc+1);
			for(int a=0;a<indiv1->ploidy(loc);++a){
				aString* gstr = indiv1->GenotypeString(loc,a," ");
				char count='A';
				while(gstr->len()>0){
					aString* gtWord = gstr->ChopOffFirstWord(endCharList);
					gtWord->CullChar(' ');
					if(gtWord->len()>0){
							*gtWord=locName;
							gtWord->cat('_');gtWord->cat(count);
							traitName.Append(gtWord);
							ntraits++;//added a trait
							count++;
							gtWord=NULL;//forget
							}
						else{
							delete gtWord; gtWord=NULL;}
					}
				delete gstr; gstr=NULL;
				}//end for a
			}//end for loc
		}//end if includeGenotype

	//read in data
	aString *s = new aString("Population 1");
	popName.Append(s);

	indivStateNames.Clear();
	indivStateScores.Clear();
	indivStateScores.Append(new ObjList<StringList>());

	//make a new sublist for the phenotypes of this population
	StringList *rawPhList = new StringList();
	rawTraits.Append(rawPhList);				
	popSize.Append(indivList.len());

	// BEAST is the string for a particular individual
	for(int i=0;i<indivList.len();++i){		
		aString beastID(i+1);beastID.cat(' ');
		Individual& indiv = indivList[i];
		if(i==0){
			StringList* statenames = indiv.stateNames();
			indivStateNames = *statenames;
			delete statenames; statenames=NULL;
			}
		indivStateScores.Last().Append(indiv.states());
 
 		GeneticIndividual* gIndiv = (GeneticIndividual*) &indiv;
		aString* beast = gIndiv->GetPhenotypeString(" ");
//		aString* beast = new aString();
//		gIndiv->SetGenotypeString(*beast);
		beast->prepend(beastID);
		//now get its fitness
		double fitness = gIndiv->fitness();
		beast->cat(' ');
		beast->cat(fitness);
		if(includeGenotype){
			beast->cat(' ');
			for (int loc=0;loc<nloci;++loc){
				for(int a=0;a<indiv1->ploidy(loc);++a){
					aString* gstr = gIndiv->GenotypeString(loc,a," ");
					while(gstr->len()>0){
						aString* gtWord = gstr->ChopOffFirstWord(endCharList);
						gtWord->CullChar(' ');
						if(gtWord->len()>0){
							beast->cat(' ');
							beast->cat(*gtWord);
							}
						delete gtWord; gtWord=NULL;
						}
					delete gstr; gstr=NULL;
					}//end for a
				}//end for loc
			}//end if includeGenotype
		rawPhList->Append(beast);
		beast=NULL;//forget
		}//end for i
	rawPhList=NULL;

	GroupMembership();
	CullSkippedTraits();
	ReadTraitStrings();
	 }//end PopPhenoLandScapeInterface::ReadListOfIndividuals





#endif







