// HybridIndexData.h

//This class holds a list of hybrid index

#ifndef HybridIndexDatah
#define HybridIndexDatah = 1

#include <aList.h>
//definitions for
class HybridIndex;
class HybridIndexByLocation;
class HybridIndexByLocData;


class HybridIndex{
	//stores an average of some set of values measured on an individual
	protected:
		double p_;//index value
		int items_;//the number of items that went into the calculation of the index
	public:
		HybridIndex(void): p_(0.0), items_(0) {};
		HybridIndex(double p, int items): p_(p), items_(items) {};
		
		HybridIndex(HybridIndex& hi):  p_(hi.p_), items_(hi.items_) {};
		HybridIndex(HybridIndex* hi):  p_(hi->p_), items_(hi->items_) {};
	
	virtual ~HybridIndex(void){};
	
	
		virtual double p(void) {return p_;}
		virtual int items(void) {return items_;}
		
		friend int operator== (HybridIndex& a, HybridIndex& b){
			if ((a.p==b.p)&&(a.items==b.items)){
				return 1;}else{return 0;}
			}
		friend int operator!= (HybridIndex& a, HybridIndex& b){
			return (!(a==b));}

	};//end class HybridIndex





class HybridIndexByLocation: public HybridIndex {
	//stores an average of some set of values measured on an individual
	protected:
		double x_;//location value
		int weight_;//weight to give this sample
	public:
		HybridIndexByLocation(void): x_(0.0), weight_(1.0) {};
		HybridIndexByLocation(double x, double weight=1.0): x_(x), weight_(weight) {};
		HybridIndexByLocation(double p, int items,double x, double weight=1.0): 
					HybridIndex(p,items), x_(x), weight_(weight) {};
		
		HybridIndexByLocation(HybridIndexByLocation& hi):  
				HybridIndex(hi), x_(hi.x_), weight_(hi.weight_) {};
		HybridIndexByLocation(HybridIndexByLocation* hi):  
				HybridIndex(hi), x_(hi->x_), weight_(hi->weight_) {};
	
	virtual ~HybridIndexByLocation(void){};
	
	
	
		virtual double 	x(void) {return x_;}
		virtual int 		weight(void) {return weight_;}
		
		friend int operator== (HybridIndexByLocation& a, HybridIndexByLocation& b){
			if ((a.p==b.p)&&(a.items==b.items)&&(a.x==b.x)&&(a.weight==b.weight)){
				return 1;}else{return 0;}
			}
		friend int operator!= (HybridIndexByLocation& a, HybridIndexByLocation& b){
			return (!(a==b));}

	};//end class HybridIndexByLocation






class HybridIndexByLocData: public ObjList<HybridIndexByLocation>{
	public:
			HybridIndexByLocData(void){};
			
			HybridIndexByLocData(HybridIndexByLocData& a):
					ObjList<HybridIndexByLocation>(a) {};
			HybridIndexByLocData(HybridIndexByLocData* a): 
					ObjList<HybridIndexByLocation>(a) {};
			
	virtual 	~HybridIndexByLocData(void){};
				
	virtual void	Append(double pVal, int itemsVal, double xVal, double weightVal=1.0){
				HybridIndexByLocation *hi = new HybridIndexByLocation(pVal,itemsVal,xVal,weightVal);
				ObjList<HybridIndexByLocation>::Append(hi);}

	virtual double	x(long loc){
						HybridIndexByLocation* hi = Nth(loc);
						return hi->x();}
	virtual double	p(long loc){
						HybridIndexByLocation* hi = Nth(loc);
						return hi->p();}
	virtual int		items(long loc){
						HybridIndexByLocation* hi = Nth(loc);
						return hi->items();}
	virtual double	weight(long loc){
						HybridIndexByLocation* hi = Nth(loc);
						return hi->weight();}
	
	};//end class HybridIndexByLocData
	
#endif
