//AbstractLikelihoodList.h


#ifndef AbstractLikelihoodListh
#define AbstractLikelihoodListh = 1

#include <aList.h>
#include <Boolean.h>
#include <Shape2ParamRec.h>
#include "AllozymeClineGlobals.h"


class AbstractLikelihoodList{
	//For each set of center populations found in the cline-fitting iterations.
	//this class stores the center populations themselves, and
	//the list of acdeptable parameters for those populations.
	//The best-fitting set of parameters is always the 0th element,
	//and the current set of parameters is the last element.
	//The intervening elements in the list are unsorted.
	public:
		likelihoodType compare;
		int saving_;//True if saving, False if not saving in listOfParamLists
		long totalSize;//the total number of parameters stored in all lists
		ObjList<ClinePopList> listOfCenterPopLists;
		aList<double> samplingAcuity;//multiplier for variance in resampling process
		const double initAcuity;
		aList<int> thisPopCombAnnealed;// boolean for disequilibrium estimation
				// when a new pop combination is found, then disequilibrium must be annealed

	public:
		AbstractLikelihoodList(likelihoodType z): compare(z), saving_(False), totalSize(0),initAcuity(0.1) {};
		AbstractLikelihoodList(AbstractLikelihoodList& c): 
				compare(c.compare), saving_(c.saving()),
				totalSize(c.totalSize),
				listOfCenterPopLists(c.listOfCenterPopLists), 
				samplingAcuity(c.samplingAcuity),
				initAcuity(0.1),
				thisPopCombAnnealed(c.thisPopCombAnnealed) {};
		AbstractLikelihoodList(AbstractLikelihoodList* c): 
				compare(c->compare), saving_(c->saving()),
				totalSize(c->totalSize),
				listOfCenterPopLists(c->listOfCenterPopLists), 
				samplingAcuity(c->samplingAcuity),
				initAcuity(c->initAcuity),
				thisPopCombAnnealed(c->thisPopCombAnnealed) {};
		virtual ~AbstractLikelihoodList(void){
					listOfCenterPopLists.Clear();
					samplingAcuity.Clear();
					thisPopCombAnnealed.Clear();
					}
		
	virtual double	Acuity(ClinePopList& popCombList){
						int loc = listOfCenterPopLists.Loc(&popCombList);
						return samplingAcuity[loc];}
	virtual int		Annealed(ClinePopList& popCombList){
						int loc = listOfCenterPopLists.Loc(&popCombList);
						return thisPopCombAnnealed[loc];}
//	virtual double 	bestL(void)=0;
	virtual void 		Clear(void){
						listOfCenterPopLists.Clear();
						samplingAcuity.Clear();
						thisPopCombAnnealed.Clear();
						totalSize=0; saving_ = False;
						}
	virtual void 		Clear(ClinePopList& popCombList){
						int loc = listOfCenterPopLists.Loc(&popCombList);
						listOfCenterPopLists.Delete(loc);
						samplingAcuity.Delete(loc);
						thisPopCombAnnealed.Delete(loc);
						}
	virtual int		IncludesPopComb(ClinePopList& popCombList){
						return listOfCenterPopLists.Includes(&popCombList);}
	virtual void 		PurgeUnlikelyParameters(void)=0;
						//removes ParamRec's that are unlikely relative to overall best
	virtual void 		PurgeUnlikelyParameters(int loc)=0;
						//removes paramRec's that are unlikely for centerPopList at loc
	virtual double 	SamplingAcuity(ClinePopList& popCombList){
						int loc = listOfCenterPopLists.Loc(&popCombList);
						return samplingAcuity[loc];}
	virtual void		SetAcuity(ClinePopList& popCombList, double acuity){
						int loc = listOfCenterPopLists.Loc(&popCombList);
						samplingAcuity.Replace(acuity, loc);}
	virtual void		SetAnnealed(ClinePopList& popCombList, int TorF){
						int loc = listOfCenterPopLists.Loc(&popCombList);
						thisPopCombAnnealed.Replace(TorF, loc);}
	virtual int		saving(void) {return saving_;}
	virtual void		SetSave(int TorF){saving_ = TorF;}
		};//end class AbstractLikelihoodList

#endif

