// AllozymeClineShapeData3.cp;
//This program calculates cline shape by fitting data in the form (x,y) to
//a composite sigmoid function.  The theory is in Szymura & Barton 1986.
//The program reads in a data set of allozymes and calculates the hybrid
//index and gametic disequilibria for each population.
#pragma once
#include "AllozymeClineShapeData3.h"



void AllozymeClineShapeData3::AnnealClineShape(const long repsBetweenAnnealSteps, 
						const double startTemp, const double endTemp, 
						Ran3& ran){
	int paramChanged = 0, unsuccessfulTries = 0;
	int savingNow = False,  allDone = False, annealing = True, switchParams; //boolean flags
	long rep=0;
	double temperatureNow = startTemp;//anneal from a wide variance
	do{//here is the annealing loop
		if(rep > repsBetweenAnnealSteps){//anneal the temperature, which narrows the sampling variance
			temperatureNow /= 10.0;
			rep = 0;
			*paramsInUse = *bestParams;
			}//end if rep
		DoOneShapeIteration(temperatureNow, annealing, ran, rep, switchParams);
		if (! switchParams){	//catches occasional dead-end traps when 
			//an unlikely parameter combination is accepted
				++unsuccessfulTries;
				if(unsuccessfulTries>=25){
					//this is a particularly bad parameter combination
					unsuccessfulTries=0;
					paramsInUse = bestParams;}
				}
			else{unsuccessfulTries=0;}
		if (temperatureNow < endTemp){
				allDone = True;}
		}while (! allDone);//ends the annealing loop
	}//AllozymeClineShapeData3::AnnealClineShape




double AllozymeClineShapeData3::DiseqLikelihood (Shape3ParamRec& paramSet, ClinePopList& centerPopList){
	//sample-size based weighting removed
	long N[5][5];//Hill's intermediate variable counting individuals by 2-locus genotype, modified to permit blanks
	long X[2][2];//Hill's intermediate variable counting individuals by 2-locus genotype, modified to permit blanks
	int index1, index2;
	double  f[2][2];
	double logL = 0.0, adjustedD = 0.0;
	String blank(" ");
	for (int popNum=0;popNum<centerPopList.len();++popNum){
		int pop = centerPopList[popNum];
		long sizePop = popSize[pop];
		double thisPopLocation = popLocation[pop];
		double pAtCline = paramSet.PonCline(thisPopLocation, maxScore);
		if ((pAtCline == 0.0) || (pAtCline == 1.0)) {
				adjustedD = 1.0;} //was = paramSet.D
			else{
				adjustedD = paramSet.D() / (pAtCline * (1.0 - pAtCline));}
		if (adjustedD > 1.0){
			adjustedD = 1.0;}
		for (int loc1=0;loc1<nloci - 1;++loc1){
			StringList* alleleNamesList1 = allelesPresent[loc1];
			for (int loc2=loc1+1;loc2<nloci;++loc2){
				StringList* alleleNamesList2 = allelesPresent[loc2];
				for (int allele1=0;allele1<alleleNamesList1->len();++allele1){
					//get freq of allele1 (at loc1) in pop
					double freq1 = GetFreq(pop, loc1, allele1);
					if ((freq1 > 0.0) && (freq1 < 1.0)) {
						String* alleleName1 = alleleNamesList1->Nth(allele1);
						for (int allele2=0;allele2<alleleNamesList2->len();++allele2){
							long nSample = 0;//set sample size to zero, to count for blanks
							//get freq of allele2 (at loc2) in pop
							double freq2 = GetFreq( pop, loc2, allele2);
							if ((freq2 > 0.0) && (freq2 < 1.0)) {
								f[0][0] = freq1 * freq2 + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
								f[0][1] = freq1 * (1.0 - freq2) + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
								f[1][0] = (1.0 - freq1) * freq2 + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
								f[1][1] = (1.0 - freq1) * (1.0 - freq2) + adjustedD * sqrt(freq1 * freq2 * (1.0 - freq1) * (1.0 - freq2));
								//Sometimes these values can be less than zero due to sampling error,
								//implying that it is infinitely improbable that an observed value
								//could occur under the model.
								//Therefore, values <=0 are set to a very small number, and will
								//give a very low finite likelihood
								for(index1=0;index1<2;++index1){
									for (index2=0;index2<2;++index2){
										if (f[index1][index2] <= 0.0){
											f[index1][index2] = 1.0e-50;}}}

								for(index1=0;index1<5;++index1){
									for (index2=0;index2<5;++index2){
										N[index1][index2] = 0;}}
								for(index1=0;index1<2;++index1){
									for (index2=0;index2<2;++index2){
										X[index1][index2] = 0;}}
								//get genotype strings for each individual and  read joint frequencies
								String* alleleName2 = alleleNamesList1->Nth(allele2);
								for (long i=0;i<sizePop;++i){//count up genotypes & store in matrix N
									String* spot1 = GetGenotypeString(pop,i,loc1);//spot1 created in function call
									//make sure it has been scored
									if (((*spot1)[0] == blank) && ((*spot1)[1] == blank)) {//not scored
										delete spot1; spot1=0;
										break;}//go to next individual
									String* spot2 = GetGenotypeString(pop,i,loc2);//spot2 created in function call
									if (((*spot2)[0] == blank) && ((*spot2)[1] == blank)) {//not scored
										delete spot1; spot1=0;
										delete spot2; spot2=0;
										break;}//go to next individual
									++nSample;//use this indiv in calculation

									//count instances of target alleles at each locus in this indiv
									if (((*spot1)[0] == blank) || ((*spot1)[1] == blank)) {//haploid individual or sex-linked locus
											index1 = 4;}
										else{index1=2;}//diploid individual
									for (int chrom=0;chrom<2;++chrom){
										if (*alleleName1==(*spot1)[chrom]){
											--index1;}}
									if (((*spot2)[0] == blank) || ((*spot2)[1] == blank)) {//haploid individual or sex-linked locus
											index2 = 4;}
										else{index2=2;}//diploid individual
									for (chrom=0;chrom<2;++chrom){
										if (*alleleName2==(*spot2)[chrom]){
											--index2;}}
									++N[index1][index2];
									delete spot1; spot1=0;
									delete spot2; spot2=0;
									}//end for i
								//calculate Hill's (1974) intermediate variables, modified for haploid & sex-linked cases
								X[0][0] = 2 * N[0][0] + N[0][1] + N[1][0] + N[0][3] + N[3][0] + N[3][3];
								X[1][0] = 2 * N[2][0] + N[1][0] + N[2][1] + N[2][3] + N[4][0] + N[4][3];
								X[0][1] = 2 * N[0][2] + N[0][1] + N[1][2] + N[0][4] + N[3][2] + N[3][4];
								X[1][1] = 2 * N[2][2] + N[1][2] + N[2][1] + N[2][4] + N[4][2] + N[4][4];
								for(index1=0;index1<2;++index1){
									for (index2=0;index2<2;++index2){
										if (f[index1][index2] > 0.0){
											logL += X[index1][index2] * ln(f[index1][index2]);
											logL += N[1][1] * ln(f[0][0] * f[1][1] + f[0][1] * f[1][0]);
											}}}//end if & for both indexes
								}//end if freq2>0
							}//end for allele2
						}//end if freq1>0
					}//end for allele1
				}//end for loc2
			}//end for loc1
		}//end for popNum
	//finally, standardize by the proportion of populations within the center region, 
	//as this will vary with the model parameters.  This will decrease logL.
	//if (centerPopList.len() > 0){
	//logL /= centerPopList.len() / npop;}
	paramSet.Ldiseq_ = logL;
	return logL;
	}//end AllozymeClineShapeData3::DiseqLikelihood







int AllozymeClineShapeData3::DoOneShapeIteration(const double temperature, 
									const int annealing, Ran3& ran, long& rep, int& switchParams){
	int allDone = False, paramChanged = 0;
	int centerExists = False;
	ClinePopList *centerPopList = new ClinePopList();
	do{//only accept if a population falls in the central region
		delete paramsToTry; paramsToTry=0;
		paramsToTry = paramsInUse->ChangeParams(*paramDeviations, ran, 
										paramChanged, temperature,annealing);
		centerPopList->Clear();
		centerExists = paramsToTry->FindCenterPopulations(*centerPopList, popLocation);
		}while(! centerExists);
	double LnewDiseq = DiseqLikelihood(*paramsToTry, centerPopList);
	double LnewCline = MaxLikeDevFromCline(*paramsToTry, maxScore);
	paramsToTry->SetTotal();
	paramsToTry->SetClineLikelihoodRatios(LmaxCline);
	++allTries;
	totalTries.Increment(paramChanged-1);
	switchParams = False;//if the fit is better, then save the new parameters

	if (paramsToTry->LtoLmaxCline() < 1.0){
			//if it's worse, then only save them with a probability proportional to their L ratio
			if (ran() < paramsToTry->LtoLmaxCline()) {
					switchParams = True;}}
												
		else{//new is better, so always switch
			switchParams = True;
			if (LnewCline > LmaxCline){//save new bests in appropriate places
				LmaxCline = LnewCline;
				paramsToTry->SetNewBest();//adjusts Lratios
				*bestParams = *paramsToTry;
				//replace initial values for diseq and total parameter estimates
				if (annealing){
					LcurrentList.Clear();
					LcurrentDiseqList.Clear();
					LcurrentTotalList.Clear();
					}//abolish all old information
				bestParams->PrintToScreen(allTries, harMeanRecomb);
				}//end if LnewCline > LmaxCline
		}//end if LtoLmaxCline... else...
	if(switchParams){
		acceptedChanges.Increment(paramChanged-1);
		*paramsInUse = *paramsToTry;
		LcurrentCline = LnewCline;
		allDone = LcurrentList.Append(paramsToTry, centerPopList);
		}//end if switchParams
	centerPopList->Clear();
	delete centerPopList; centerPopList=0;
	delete paramsToTry; paramsToTry=0;
	++rep;
	return allDone;
	}//end AllozymeClineShapeData3::DoOneShapeIteration
	
	
	
	
void AllozymeClineShapeData3::Find2unitParameterList(Ran3& ran){
	cout<<endl<<"Beginning cline shape error estimation"<<endl<<endl;
	double temperatureNow = temperature * 2.0; 
	int allDone = False;
	LcurrentList.SetSave(True);
	LcurrentDiseqList.SetSave(True);
	LcurrentTotalList.SetSave(True);

	LcurrentCline = bestParams->Lcline();
	*paramsInUse = *bestParams;
	int unsuccessfulTries = 0;
	InitializeCounters(params);

	long rep=0;
	do{//random walk sampling-error iteration loop
		//change cline shape parameters first
		int switchParams = False, annealing = False;
		allDone = DoOneShapeIteration(temperatureNow, annealing, ran, rep, switchParams);
			
		if (! switchParams)	{//catches occasional dead-end traps when 
				//an unlikely parameter combination is accepted
				++unsuccessfulTries;
				if(unsuccessfulTries>=25){
					//this is a particularly bad parameter combination
					unsuccessfulTries=0;
					paramsInUse = bestParams;}
				}
			else{//new shape parameters chosen
				unsuccessfulTries = 0;
				//test a new disequilibrium value
				}//end if ! switchParams...else...
		++rep;
		allDone = True;
		long lenSum = LcurrentList.totalSize;//compare SavedReps to the total of all pop combination lists}
		if (lenSum < savedReps){
			allDone = False;}
		}while (! allDone);//end error-sampling iteration
	cout<<endl<<"Done with cline shape error estimation"<<endl<<endl;
	}//end AllozymeClineShapeData3::Find2unitParameterList



void AllozymeClineShapeData3::FitAverageAllozymeCline(String& dataFileName, Ran3& ran){
	PrepareDataForClineAnalysis(dataFileName);
	InitializeCounters(params);

	paramsInUse->SetStartingValues();
	paramDeviations->SetDeviationValues();

	//calculate a maximum likelihood fit function
	ClinePopList centerPopList;
	paramsInUse->FindCenterPopulations(centerPopList, popLocation);
	LcurrentDiseq = DiseqLikelihood(*paramsInUse, centerPopList);
	LcurrentCline = MaxLikeDevFromCline(*paramsInUse, maxScore);
	paramsInUse->SetTotal();
	paramsInUse->Shape2ParamRec::SetNewBest();
	paramsInUse->DiseqParamRec::SetNewBest();
	paramsInUse->SetNewBest();
	LcurrentTotal = LcurrentDiseq + LcurrentCline;
	LcurrentList.Append(*paramsInUse, centerPopList);
	LcurrentDiseqList.Append(*paramsInUse, centerPopList);
	LcurrentTotalList.Append(*paramsInUse, centerPopList);
	centerPopList.Clear();


	LmaxCline = LcurrentCline;
	LmaxDiseq = LcurrentDiseq;
	LmaxTotal = LcurrentTotal;
	SetCurrentAsBest();
	bestParams->PrintToScreen(allTries, harMeanRecomb);




	//ANNEAL TO FIND THE MAXIMUM LIKELIHOOD ESTIMATE OF CLINE SHAPE

	AnnealClineShape(annealAfterRep,temperature * 100.0, lowTemp, ran);



	//NOW DO THE RANDOM-WALK ERROR ESTIMATION 
	Find2unitParameterList(ran);
	LcurrentList.Find2UnitSupportLimits (harMeanRecomb);

/*
	PrintBestParameters(acceptedChanges, totalTries, LmaxCline, bestParams, lowerCIset, upperCIset, bestParamsDiseq, bestParamsTotal, popCombListDiseq, popCombListTotal, Output, False);
	WriteLn(Output, 'chromosomes: ', numChromosomes : 6 : 3, '   harmonic mean recombination rate: ', harMeanRecomb : 8 : 5);
	popListFileName := Concat(outputFileName1, '.sum');
	open(outputData2, popListFileName);
	PrintBestParameters(acceptedChanges, totalTries, LmaxCline, bestParams, lowerCIset, upperCIset, bestParamsDiseq, bestParamsTotal, popCombListDiseq, popCombListTotal, outputData2, False);
	WriteLn(outputData2, 'chromosomes: ', numChromosomes : 6 : 3, '   harmonic mean recombination rate: ', harMeanRecomb : 8 : 5);
	close(outputData2);
	PrintSavedParameters(bestParams, savedParamList, outputData, outputFileName1);
	if alloData^^.nloci > 1 then	{provided there are enough loci}
		begin
			popListFileName := Concat(outputFileName2, '.D pops');
			open(outputData2, popListFileName);
			writeLn(outputData2, 'file, population set');
			for j := 1 to popCombListDiseq.len do
				begin
					outName := Concat('.', IntegerToString(j));
					outName := Concat(outputFileName2, outName);
					PrintSavedParameters(bestParamsDiseq^[j], diseqSavedParamList^[j], outputData, outName);
					centerPopList := ListStructureTypePtr(GetSubListAt(popCombListDiseq, j));
					writeLn(Output, 'In data file ', outName, ':');
					PrintPopComblist(centerPopList^, Output);
					writeLn(outputData2, outName, ':');
					PrintPopComblist(centerPopList^, outputData2);
				end;	{end for j}
			close(outputData2);
		end;	{end if nloci}
	if alloData^^.nloci > 1 then	{provided there are enough loci}
		begin
			popListFileName := Concat(outputFileName3, '.T pops');
			open(outputData2, popListFileName);
			writeLn(outputData2, 'file, population set');
			for j := 1 to popCombListTotal.len do
				begin
					outName := Concat('.', IntegerToString(j));
					outName := Concat(outputFileName3, outName);
					PrintSavedParameters(bestParamsTotal^[j], totalSavedParamList^[j], outputData, outName);
					centerPopList := ListStructureTypePtr(GetSubListAt(popCombListTotal, j));
					writeLn(Output, 'In data file ', outName, ':');
					PrintPopComblist(centerPopList^, Output);
					writeLn(outputData2, outName, ',');
					PrintPopComblist(centerPopList^, outputData2);
					centerPopList := nil;	{forget this value}
				end;	{end for j}
			close(outputData2);
		end; {end if nloci>1}




{check for differential introgression by locus}
	if introgressionBreakdownByLocus then
		if alloData^^.nloci > 1 then
			begin
				New(dummyPopList);
				InitializeListOfLists(dummyPopList^, primaryList);
				New(tempParamList);
				New(tempBestParams);
				for loc := 1 to alloData^^.nloci do
					begin
						RescaleLtoLmaxRatios(bestParamsLoc^[loc], savedParamListLocDev^[loc], 1);	{uses Lcline as criterion}
						tempParamList^[1] := savedParamListLocDev^[loc];
						tempBestParams^[1] := bestParamsLoc^[loc];
						Find2UnitSupportLimits(bestParamsLoc^[loc], lowerCIset, upperCIset, tempBestParams, tempBestParams, 1, 1, savedParamListLocDev^[loc], tempParamList, tempParamList);
						PrintBestParameters(acceptedChanges, totalTries, bestParamsLoc^[loc].LCline, bestParamsLoc^[loc], lowerCIset, upperCIset, tempBestParams, tempBestParams, dummyPopList^, dummyPopList^, Output, True);
						WriteLn(Output, 'chromosomes: ', numChromosomes : 6 : 3, '   harmonic mean recombination rate: ', harMeanRecomb : 8 : 5);
						popListFileName := Concat(outputFileName1, '.');
						popListFileName := Concat(outputFileName1, GetStrValueAt(alloData^^.locusName^, loc));
						popListFileName := Concat(popListFileName, '.sum');
						open(outputData2, popListFileName);
						PrintBestParameters(acceptedChanges, totalTries, bestParamsLoc^[loc].LCline, bestParamsLoc^[loc], lowerCIset, upperCIset, tempBestParams, tempBestParams, dummyPopList^, dummyPopList^, outputData2, True);
						WriteLn(outputData2, 'chromosomes: ', numChromosomes : 6 : 3, '   harmonic mean recombination rate: ', harMeanRecomb : 8 : 5);
						close(outputData2);
						popListFileName := Concat(outputFileName1, GetStrValueAt(alloData^^.locusName^, loc));
						PrintSavedParameters(bestParamsLoc^[loc], savedParamListLocDev^[loc], outputData2, popListFileName);
					end;	{end for loc}
				Dispose(dummyPopList);
				Dispose(tempParamList);
				Dispose(tempBestParams);

{for each locus's data, calculate the likelihood of the average and other loci's best fits}
				popListFileName := Concat(outputFileName1, '.L1overL2');
				open(outputData2, popListFileName);
				write(Output, 'loc,  ave,');
				write(outputData2, 'loc,  ave,');
				for loc1 := 1 to alloData^^.nloci do
					begin
						write(Output, Spaces(3), ', ', loc1 : 2);
						write(outputData2, Spaces(3), ', ', loc1 : 2);
					end;
				writeLn(Output);
				writeLn(outputData2);
				for loc1 := 1 to alloData^^.nloci do
					begin
						write(Output, loc1 : 2, Spaces(3), ', ');
						write(outputData2, loc1 : 2, Spaces(3), ', ');
						theRatio := exp(bestParamsLoc^[loc1].Lcline - bestParams.Lcline);
						write(Output, theRatio : 15 : 10);
						write(outputData2, theRatio : 15 : 10);
						for loc2 := 1 to alloData^^.nloci do
							begin
								write(Output, Spaces(3), ', ');
								write(outputData2, Spaces(3), ', ');
								theRatio := exp(bestParamsLoc^[loc1].Lcline - bestParamsLoc^[loc2].Lcline);
								write(Output, theRatio : 15 : 10);
								write(outputData2, theRatio : 15 : 10);
							end;	{end for loc2}
						writeLn(Output);
						writeLn(outputData2);
					end;	{end for loc1}
				close(outputData2);


			end;	{end if introgressionBreakdownByLocus & if nloci>1]}

{dispose everything}
	DisposeHybridIndexListsWithPtrs;
	DisposeSavedParamSet(savedParamList);
	for i := 1 to PopCombinations do
		DisposeSavedParamSet(totalSavedParamList^[i]);
	Dispose(totalSavedParamList);
	for i := 1 to PopCombinations do
		DisposeSavedParamSet(diseqSavedParamList^[i]);
	Dispose(diseqSavedParamList);
	while popCombListTotal.len > 0 do
		DisposeFirstSublist(popCombListTotal, primaryList);
	while popCombListDiseq.len > 0 do
		DisposeFirstSublist(popCombListDiseq, primaryList);
	Dispose(bestParamsLoc);
	for loc := 1 to maxLoci do
		DisposeSavedParamSet(savedParamListLocDev^[loc]);
	Dispose(savedParamListLocDev);
	DisposeAlloListsWithPointers;
	DisposeList(popLocation);
	Dispose(bestParamsDiseq);
	Dispose(bestParamsTotal);
*/
}//end AllozymeClineShapeData3::FitAverageAllozymeCline



double AllozymeClineShapeData3::MaxLikeDevFromCline (Shape3ParamRec& paramSet, double highScore){
	double runningTotal = 0.0;
	for (int loc=0;loc<nloci;++loc){
			runningTotal += MaxLikeDevFromCline(paramSet, loc, highScore);}
	paramSet.Lcline_ = runningTotal;
	if(runningTotal == 0.0){
			int dummyToForceDebugStop = 1;}
	return runningTotal;
	}//end AllozymeClineShapeData3::MaxLikeDevFromCline  (Shape3ParamRec&, double)



double AllozymeClineShapeData3::MaxLikeDevFromCline (Shape3ParamRec& paramSet, int loc, double highScore){
	//sample-size weights removed from this analysis
	// obs, expected, logL, temp, score: extended;
	double logL = 0.0;
	HybridIndexByLocData* hiData = hybridIndexByLocus[loc];
	for (long i=0;i<hiData->len();++i){
		double obs = hiData->p(i);
		double expected = paramSet.PonCline( hiData->x(i), highScore);
		if ((obs > 0.0) && (expected > 0.0)) {
			//this effectively sets obs*ln(E)=0 when obs=0 or exp=0
			logL +=  obs * ln(expected);}// * hiData->weight(i);
		if ((obs < 1.0) && (expected < 1.0)) {
			logL +=  (1.0 - obs) * ln(1.0 - expected);}// * hiData->weight(i);
		}//end for i
	paramSet.Lcline_ = logL;
	return logL;
	}//end AllozymeClineShapeData3::MaxLikeDevFromCline (Shape3ParamRec&, int, double)



void AllozymeClineShapeData3::PrepareDataForClineAnalysis(String& dataFileName){
	PoolDiagnosticAlleles();

	String convertFileName(dataFileName);
	convertFileName.cat(".pooled");
	PrintGelJockFile(convertFileName);

	CalculateAlleleFreqs();

	CalculateHybridIndex();//popLocation created
	CalculateHybridIndexByLocus();//popLocation re-created
	convertFileName = dataFileName;
	convertFileName.cat(".hyb index data");
	PrintHybridIndexFile(convertFileName);
}//end AllozymeClineShapeData3::PrepareDataForClineAnalysis

