// AllozymeClineShapeOnly.h

//This class calculates cline shape by fitting data in the form (x,y) to
//a 2-parameter tanh function.  The theory is in Szymura & Barton 1986.
//The program reads in a data set of allozymes and calculates the hybrid
//index and gametic disequilibria for each population.

//derived classes implement the appropriate cline shape functions
#ifndef AllozymeClineShapeOnlyh
#define AllozymeClineShapeOnlyh = 1

#include <iostream.h>
//#include <Shape2ParamRec.h>
#include <ClinePopList.h>
#include <AllozymeClineShapeData.h>





template <class T> class AllozymeClineShapeOnly: virtual public AllozymeClineShapeData{
		public://uses 2-, 4-, 6-, and 8-parameter shape estimation models
			int params;
			T *bestParams, *paramsInUse, *paramDeviations, *paramsToTry;			
			LikelihoodList<T> LcurrentList;
			
		public:
			AllozymeClineShapeOnly(double harmonicMeanRecomb): 
						AllozymeClineShapeData(harmonicMeanRecomb),
						LcurrentList(cline), params(0),
						bestParams(0), paramsInUse(0), paramDeviations(0), paramsToTry(0) {
							IShapeData();}
			AllozymeClineShapeOnly(aString& s,double harmonicMeanRecomb): 
						AllozymeClineShapeData(s, harmonicMeanRecomb),
						LcurrentList(cline), params(0),
						bestParams(0), paramsInUse(0), paramDeviations(0), paramsToTry(0) {
							IShapeData();}
			AllozymeClineShapeOnly(double numChromosomes, double mapDistBetweenLoci): 
						AllozymeClineShapeData(numChromosomes, mapDistBetweenLoci),
						LcurrentList(cline), params(0),
						bestParams(0), paramsInUse(0), paramDeviations(0), paramsToTry(0) {
							IShapeData();}
			AllozymeClineShapeOnly(aString& s, double numChromosomes, double mapDistBetweenLoci): 
						AllozymeClineShapeData(s, numChromosomes, mapDistBetweenLoci),
						LcurrentList(cline), params(0),
						bestParams(0), paramsInUse(0), paramDeviations(0), paramsToTry(0) {
							IShapeData();}
			~AllozymeClineShapeOnly(void){
				delete bestParams; bestParams=0;
				delete paramsInUse; paramsInUse=0;
				delete paramDeviations; paramDeviations=0;
				delete paramsToTry; paramsToTry=0;
				}
			
	//VIRTUAL FUNCTIONS
	virtual void 		AnnealClineShape(const long repsBetweenAnnealSteps, 
						const double startTemp, const double endTemp, 
						Ran3& ran, long& savedReps, long& annealAfterRep);
	virtual int 		DoOneShapeIteration(const double temperature,  
							const int annealing, Ran3& ran, long& rep, 
							int& switchParams, long& savedReps);
	virtual void 		Find2unitParameterList(Ran3& ran, long& savedReps);//should first call AnnealClineShape
	virtual void		FitAverageAllozymeCline(aString& dataFileName, Ran3& ran, 
						long& savedReps, long& annealAfterRep);//averaged across loci
	virtual void 		IShapeData(void ){
						delete bestParams; bestParams= new T();
						delete paramsInUse; paramsInUse= new T();
						delete paramDeviations; paramDeviations= new T();
						delete paramsToTry; paramsToTry= new T();
						}//end IShapeData		
	virtual double	MaxLikeDevFromCline (T& paramSet, double highScore);
	virtual double 	MaxLikeDevFromCline (T& paramSet, int loc, double highScore);
	virtual void 		PrepareDataForClineAnalysis(aString& dataFileName);
	virtual void		SetCurrentAsBest(void){
						delete bestParams; bestParams = 0;
						bestParams = new T(*paramsInUse);
						}//end SetCurrentAsBest
		};//end class AllozymeClineShapeOnly
		
		
#endif
