// AllozymeClineShapePlus.h

//This class calculates cline shape by fitting data in the form (x,y) to
//a 2-parameter tanh function.  The theory is in Szymura & Barton 1986.
//The program reads in a data set of allozymes and calculates the hybrid
//index and gametic disequilibria for each population.

//derived classes implement the appropriate cline shape functions
#ifndef AllozymeClineShapePlush
#define AllozymeClineShapePlush = 1

#include <iostream.h>
#include <Shape3ParamRec.h>
#include <ClinePopList.h>
#include <LikelihoodList.h>
#include <LikelihoodListDiseq.h>
#include <LikelihoodListTotal.h>
#include <AllozymeClineShapeData.h>





template<class T> class AllozymeClineShapePlus: virtual public AllozymeClineShapeData{
		public://analyzes 3-, 5-, 7-, and 9-parameter cline shape models
			int params;
			T *bestParams, *paramsInUse, *paramDeviations, *paramsToTry;			
			LikelihoodList<T> LcurrentList;
			LikelihoodListDiseq<T> LcurrentDiseqList;
			LikelihoodListTotal<T> LcurrentTotalList;
			double LcurrentDiseq, LmaxDiseq;
			double LcurrentTotal, LmaxTotal;


			
		public:
			AllozymeClineShapePlus(double harmonicMeanRecomb): 
						AllozymeClineShapeData(harmonicMeanRecomb),
						LcurrentList(cline), LcurrentDiseqList(disequilibrium), LcurrentTotalList(total), 
						params(0), bestParams(0), paramsInUse(0), paramDeviations(0), paramsToTry(0) {
							IShapeData();}
			AllozymeClineShapePlus(double numChromosomes, double mapDistBetweenLoci): 
						AllozymeClineShapeData(numChromosomes, mapDistBetweenLoci),
						LcurrentList(cline), LcurrentDiseqList(disequilibrium), LcurrentTotalList(total), 
						params(0), bestParams(0), paramsInUse(0), paramDeviations(0), paramsToTry(0) {
							IShapeData();}
			AllozymeClineShapePlus(aString& s,double harmonicMeanRecomb): 
						AllozymeClineShapeData(s, harmonicMeanRecomb),
						LcurrentList(cline), LcurrentDiseqList(disequilibrium), LcurrentTotalList(total), 
						params(0), bestParams(0), paramsInUse(0), paramDeviations(0), paramsToTry(0) {
							IShapeData();}
			AllozymeClineShapePlus(aString& s,double numChromosomes, double mapDistBetweenLoci): 
						AllozymeClineShapeData(s, numChromosomes, mapDistBetweenLoci),
						LcurrentList(cline), LcurrentDiseqList(disequilibrium), LcurrentTotalList(total), 
						params(0), bestParams(0), paramsInUse(0), paramDeviations(0), paramsToTry(0) {
							IShapeData();}
			~AllozymeClineShapePlus(void){
				delete bestParams; bestParams=0;
				delete paramsInUse; paramsInUse=0;
				delete paramDeviations; paramDeviations=0;
				delete paramsToTry; paramsToTry=0;
				}
			
	//VIRTUAL FUNCTIONS
	virtual void 		AnnealClineShape(const long repsBetweenAnnealSteps, 
							const double startTemp, const double endTemp, 
							Ran3& ran, long& savedReps);
	virtual void 		AnnealDiseq(const long repsBetweenAnnealSteps, 
							const double startTemp, const double endTemp, 
							Ran3& ran, T* defaultParams, long& savedReps);
	virtual double 	DiseqLikelihood (T& paramSet, ClinePopList& centerPopList);
	virtual int 		DoOneDiseqIteration(const double temperature, const int annealing, 
							Ran3& ran, long& rep, int& switchParams,
							T* paramsToUse, T* localBestParams, long& savedReps);
	virtual int 		DoOneShapeIteration(const double temperature,  
							const int annealing, Ran3& ran, long& rep, 
							int& switchParams, long& savedReps);
	virtual void 		Find2unitParameterList(Ran3& ran, long& savedReps, long& annealAfterRep);
							//should first call AnnealClineShape
	virtual int		FitAverageAllozymeCline(aString& dataFileName, 
								Ran3& ran, long& savedReps, long& annealAfterRep);
							//averaged across loci
							//returns 0 if unsuccessful
	virtual void 		IShapeData(void ){
						delete bestParams; bestParams= new T();
						delete paramsInUse; paramsInUse= new T();
						delete paramDeviations; paramDeviations= new T();
						delete paramsToTry; paramsToTry= new T();
						}//end IShapeData		
	virtual double	MaxLikeDevFromCline (T& paramSet, double highScore);
	virtual double 	MaxLikeDevFromCline (T& paramSet, int loc, double highScore);
	virtual void  		PrintBestParameters (aString& fileName);
	virtual void  		PrintBestParameters (ostream& outFile);
	virtual int 		PrepareDataForClineAnalysis(aString& dataFileName);//returns 0 if unsuccessful
	virtual void		SetCurrentAsBest(void){
						delete bestParams; bestParams = 0;
						bestParams = new T(*paramsInUse);
						}//end SetCurrentAsBest
	virtual int		VaryDiseq(const double temperature, Ran3& ran, long& rep, 
								int& switchParams, long& savedReps, long& annealAfterRep);
		};//end class AllozymeClineShapePlus
		
		
#endif
