//CurrentLikelihoodList.cp


#include <CurrentLikelihoodList.h>




int CurrentLikelihoodList::Append(ParamRec* x, ClinePopList& centerPopList){
	//first see if it compares to the very best cline shape found so far
	//this is needed when assessing disequilibrium and total likelihood estimates
	if( (overallBest.Lcline ==0.0) || (x->Lcline > overallBest.Lcline) ){
		overallBest = *x;//it might be a new high record
		PurgeUnlikelyParameters();}
	//now find the right location to store the paramRec
	if (listOfCenterPopLists.Includes(&centerPopList)){
			int loc=listOfCenterPopLists.Loc(&centerPopList);
			ParamRec* newCurrent = new ParamRec(x);
			currentParamList.Replace(newCurrent,loc);
			ParamRec* currentBest = bestParamList[loc];
			//check to see if it is a new high record for this centerPopList
			if( ((compare==cline)&&((currentBest->Lcline == 0.0) ||  (x->Lcline > currentBest->Lcline)) ) ||
			      ((compare==disequilibrium)&&((currentBest->Ldiseq == 0.0) ||  (x->Ldiseq > currentBest->Ldiseq))) ||
			      ((compare==total)&&((currentBest->Ltotal() == 0.0) ||  (x->Ltotal() > currentBest->Ltotal() )))  ){
			      		currentBest=0;
					ParamRec* newBest = new ParamRec(x);
					bestParamList.Replace(newBest,loc);
					PurgeUnlikelyParameters(loc);}
			// save if it's within Lmax-2 of the local best
			currentBest = bestParamList[loc];
			if(saving &&  ( ((compare==cline)&&(x->Lcline > currentBest->Lcline - 2.0)) ||
			      ((compare==disequilibrium)&&(x->Ldiseq > currentBest->Ldiseq - 2.0)) ||
			      ((compare==total)&&(x->Ltotal() > currentBest->Ltotal() - 2.0 ))  ) ){
			      		ParamRec* xCopy = new ParamRec(x);
					ObjList<ParamRec>* paramList = listOfParamLists[loc];
					paramList->Append(xCopy);
					++totalSize;}
			}
		else{
			ClinePopList* newList = new ClinePopList(centerPopList);
			listOfCenterPopLists.Append(newList);
			ParamRec* newCurrent=new ParamRec(x);
			currentParamList.Append(newCurrent);
			ParamRec* newBest=new ParamRec(x);
			bestParamList.Append(newBest);
			ObjList<ParamRec>* paramList = new ObjList<ParamRec>();
			listOfParamLists.Append(paramList);//at least append an empty list
			samplingAcuity.Append(initAcuity);
			thisPopCombAnnealed.Append(False);//change in later call, if necessary
			if (saving){
				ParamRec* xCopy = new ParamRec(x);
				paramList->Append(xCopy);
				++totalSize;}
			}
	if (totalSize>=savedReps){
		SetSave(False);}
	return saving;
	}//end CurrentLikelihoodList::Append



ParamRec* CurrentLikelihoodList::best(ClinePopList& centerPopList){
	if (listOfCenterPopLists.Includes(&centerPopList)){
			int loc=listOfCenterPopLists.Loc(&centerPopList);
			ParamRec* currentBest=0;
			return currentBest = bestParamList[loc];}
		else{return 0;}
	}//end  CurrentLikelihoodList::best
			
		

ParamRec* CurrentLikelihoodList::current(ClinePopList& centerPopList){
	if (listOfCenterPopLists.Includes(&centerPopList)){
			int loc=listOfCenterPopLists.Loc(&centerPopList);
			ObjList<ParamRec>* paramList = listOfParamLists[loc];
			return paramList->Last();}
		else{return 0;}
	}//end  CurrentLikelihoodList::current
			
		
void CurrentLikelihoodList::PurgeUnlikelyParameters(void){
	//removes paramRec's that are unlikely relative to overall best
	for(int set=listOfParamLists.len()-1;set>=0;--set){//backwards down the list
		ParamRec* localBest = bestParamList[set];
		if (localBest->Lcline < overallBest.Lcline - 2.0){
				//then none of the saved parameters remains valid
				bestParamList.Delete(set);
				currentParamList.Delete(set);
				ObjList<ParamRec>* paramList = listOfParamLists[set];
				totalSize -=paramList->len(); paramList=0;
				listOfParamLists.Delete(set);
				listOfCenterPopLists.Delete(set);}
			else{//there may still be some invalid ones in the saved lists		
				ObjList<ParamRec>* paramList = listOfParamLists[set];
				for(long rec=paramList->len()-1;rec>=0;--rec){
					ParamRec* paramToTest=paramList->Nth(rec);
					if (paramToTest->Lcline < overallBest.Lcline - 2.0){
						paramList->Delete(rec);
						--totalSize;}
					}//end for rec
				}//end if...else...
		}//end for set
	}//end CurrentLikelihoodList::PurgeUnlikelyParameters()


void CurrentLikelihoodList::PurgeUnlikelyParameters(int loc){
	//removes paramRec's that are unlikely for centerPopList at loc
	ParamRec* localBest = bestParamList[loc];
	ObjList<ParamRec>* paramList = listOfParamLists[loc];
	for(long rec=paramList->len()-1;rec>=0;--rec){
		ParamRec* paramToTest=paramList->Nth(rec);
		if(   ((compare==cline)&&(paramToTest->Lcline < localBest->Lcline - 2.0))  ||
 			((compare==disequilibrium)&&(paramToTest->Ldiseq < localBest->Ldiseq - 2.0))  ||
 			((compare==total)&&(paramToTest->Ltotal() < localBest->Ltotal() - 2.0))  ){
 				paramList->Delete(rec);
 				--totalSize;}
		}//end for rec
	}//end  CurrentLikelihoodList::PurgeUnlikelyParameters(int)



void CurrentLikelihoodList::SetBest(ClinePopList& popCombList, ParamRec* newBest){
	if(listOfCenterPopLists.Includes(&popCombList)){
			int loc = listOfCenterPopLists.Loc(&popCombList);
			ParamRec* oldBest = bestParamList[loc];
			*oldBest = *newBest;}
		else{
			ParamRec* bestToAdd = new ParamRec(newBest);
			bestParamList.Append(bestToAdd);}
	}//end CurrentLikelihoodList::SetBest



