//DiseqParamRec.cp


#pragma once
#include <DiseqParamRec.h>
#include <FitMultilocusCline.h>


DiseqParamRec* DiseqParamRec::ChangeParams (DiseqParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t, const int annealingShape){
		int OK=True;
		thisOne = 1;//annealingShape ignored
		DiseqParamRec* paramToChange = new DiseqParamRec(this);//calls copy constructor
		do	{
			OK = True;
			paramToChange->D_ = ran.NormDev(D(), paramDeviations.D() * t);
			OK = paramToChange->MeetsParamCriteria();
			}while (! OK);
		return paramToChange;
	}//	end DiseqParamRec::ChangeParams



int DiseqParamRec:: MeetsParamCriteria (void){
		int OK = True;
		if ((D() < diseqBottom) || (D() > diseqTop)) {
			OK = False;}
		return OK;
	}//end  DiseqParamRec:: MeetsParamCriteria



void DiseqParamRec::PrintDiseqWithCI(const double successRate, 
							DiseqParamRec& lowerCI, 
							DiseqParamRec& upperCI, 
							ostream& outFile){//called by the best parameter set only		
		outFile<<endl;
		outFile<<"diseq L max: ";outFile.precision(25);outFile<< Ldiseq()<<endl;
		outFile<<"D: ";outFile.precision(10);outFile<<D()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.D()<<" - ";
					      outFile.precision(10);outFile<<upperCI.D()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate<<endl;
	}//end DiseqParamRec::PrintDiseqWithCI



void DiseqParamRec::PrintToScreen (const long tries){
	cout.width(5);cout<<tries<<"   diseq Lmax: ";
	cout.precision(6);cout<< Ldiseq()<<endl;
	cout<<"         D: ";cout.precision(6); cout<<D();
	cout<<endl;
	}//end  DiseqParamRec::PrintToScreen



void DiseqParamRec::SetDiseqLikelihoodRatios(const double LmaxDiseq){
	LtoLmaxDiseq_ = exp(Ldiseq() - LmaxDiseq);//this form gives the ratio of the true likelihoods
	LmaxMinusLDiseq_ = LmaxDiseq - Ldiseq();
	}//end  DiseqParamRec::SetDiseqLikelihoodRatios(double)



void	 DiseqParamRec::SetStartingValues(void){
	// plausible arbitrary values that start the iteration process
	// these are now constants defined in AllozymeClineGlobals.h
	D_ = Idiseq;
	}//end DiseqParamRec::SetStartingValues

	
	
	
void	 DiseqParamRec::SetDeviationValues(void){
	//set initial parameter deviation values (standard deviations)
	//parameters deviated by random variables with these standard deviations
	// these are now constants defined in AllozymeClineGlobals.h
	D_ = diseqDev;
	}//end DiseqParamRec::SetDeviationValues




