// DiseqParamRec.h

#ifndef DiesqParamRech
#define DiseqParamRech = 1

#include <aList.h>
#include <Boolean.h>
#include <math.h>
#include <Hyperbolics.h>
#include <InlineTemplates.h>
#include <Randomness.h>
//#include <Shape2ParamRec.h>
#include "AllozymeClineGlobals.h"

class DiseqParamRec;

class DiseqParamRec{//for maximum likelihood estimation of disequilibrium
	    public:
		double D_;//disequilibrium
		double Ldiseq_;	//likelihood of this disequilibrium value
		double LtoLmaxDiseq_;
		double LmaxMinusLDiseq_;
	public:
		DiseqParamRec(void):
			Ldiseq_(0.0), LtoLmaxDiseq_(0.0), LmaxMinusLDiseq_(0.0), D_(0.0) {}
		DiseqParamRec(DiseqParamRec& x):	//copy constructor
				Ldiseq_(x.Ldiseq()), LtoLmaxDiseq_(x.LtoLmaxDiseq()), 
				LmaxMinusLDiseq_(x.LmaxMinusLDiseq()),
				D_(x.D())  {}
		DiseqParamRec(DiseqParamRec* x):	//copy constructor
				Ldiseq_(x->Ldiseq()), LtoLmaxDiseq_(x->LtoLmaxDiseq()), 
				LmaxMinusLDiseq_(x->LmaxMinusLDiseq()),
				D_(x->D()) {}

		virtual ~DiseqParamRec(void){}


	virtual double 	D(void){return D_;}
	virtual double 	Ldiseq(void){return Ldiseq_;}
	virtual double 	LtoLmaxDiseq(void){return LtoLmaxDiseq_;}
	virtual double 	LmaxMinusLDiseq(void){return LmaxMinusLDiseq_;}
	virtual void	 	D(double d){D_ = d;}
	virtual void	 	Ldiseq(double L){Ldiseq_ = L;}
	virtual void	 	LtoLmaxDiseq(double ratio){LtoLmaxDiseq_ = ratio;}
	virtual void	 	LmaxMinusLDiseq(double diff){LmaxMinusLDiseq_ = diff;}

	DiseqParamRec* 	ChangeDiseqParam (DiseqParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t){
						 return ChangeParams(paramDeviations, ran, thisOne, t, False);}
	DiseqParamRec* 	ChangeParams (DiseqParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t, const int annealingShape);
	virtual void		Clear(void){
						D_=Ldiseq_=LtoLmaxDiseq_=LmaxMinusLDiseq_=0.0;}
	virtual void		DerivedValues(aList<double>& derivedList){//returns a list of derived variables
									derivedList.Clear();//none calculated at this level
									}
	virtual double 	GetDiseqSuccess(aList<double>& successRate){
						return successRate[0];}//D is only parameter varied in ChangeParams
	virtual int		HasHigherLikelihood(DiseqParamRec& param, 
									likelihoodType compare, 
									double ci = 0.0){//boolean
						//always compares Ldiseq; always greater if param.Ldiseq() == 0
						// for overall best, ci ==0.0
						//for test against confidence limits, use ci = 2.0 (for 95% ci)
						return ((param.Ldiseq() == 0.0) ||  (Ldiseq() > param.Ldiseq() - ci ));}
	virtual void		max(DiseqParamRec& param){//sets D to the maximum of this.D & param.D
							if(param.D()>D()){D_=param.D();}
							Ldiseq_=LtoLmaxDiseq_=LmaxMinusLDiseq_=0.0;}
	virtual int 		MeetsParamCriteria (void);//boolean for legal parameters
	virtual void		min(DiseqParamRec& param){//sets D to the minimum of this.D & param.D
							if(param.D()<D()){D_=param.D();}
							Ldiseq_=LtoLmaxDiseq_=LmaxMinusLDiseq_=0.0;}
	virtual void 		PrintDiseqWithCI(const double successRate, 
							DiseqParamRec& lowerCI, 
							DiseqParamRec& upperCI, 
							ostream& outFile);//called by the best parameter set only		
	virtual void		PrintToScreen(const long tries);
	virtual void 		SetDiseqLikelihoodRatios(const double LmaxDiseq);
	virtual void		SetDeviationValues(void);
	virtual void 		SetNewBest(void){LtoLmaxDiseq_ = 1.0; LmaxMinusLDiseq_ = 0.0;}
	virtual void 		SetNewBestDiseq(void){SetNewBest();}
	virtual void		SetStartingValues(void);
	virtual void 		operator=(DiseqParamRec& x){
							Ldiseq_=x.Ldiseq(); LtoLmaxDiseq_=x.LtoLmaxDiseq(); 
							LmaxMinusLDiseq_=x.LmaxMinusLDiseq();
							D_=x.D();
							}

	friend int 		operator== (DiseqParamRec& a, DiseqParamRec& b){
							if(	(a.D()==b.D()))		 {return 1;} else { return 0;}
							}//end operator ==

	friend int 		operator!= (DiseqParamRec& a, DiseqParamRec& b){
									return (!(a==b));}

	};//end class DiseqParamRec

#endif

