//LikelihoodList.cp


#include <LikelihoodList.h>




template<class T> int LikelihoodList<T>::Append(T* x, ClinePopList& centerPopList, long& savedReps){
	//first see if it compares to the very best cline shape found so far
	//this is needed when assessing disequilibrium and total likelihood estimates
	if( (overallBest.Lcline() ==0.0) || (x->Lcline() > overallBest.Lcline()) ){
		overallBest = *x;//it might be a new high record
		PurgeUnlikelyParameters();}
	//now find the right location to store the paramRec
	if (listOfCenterPopLists.Includes(&centerPopList)){
			int loc=listOfCenterPopLists.Loc(&centerPopList);
			T* newCurrent = new T(x);
			currentParamList.Replace(newCurrent,loc);
			T* currentBest = bestParamList[loc];
			//check to see if it is a new high record for this centerPopList
			if(x->HasHigherLikelihood(*currentBest, compare)){
//				T* newBestForPopList = new T(x);
//				bestParamList.Replace(newBestForPopList, loc);
//				currentBest; currentBest = 0;
//				currentBest =  new T(x);
				*currentBest =  *x;
				PurgeUnlikelyParameters(loc);}

			// save if it's within Lmax-2 of the local best
			currentBest = bestParamList[loc];
			if(saving() &&  (x->HasHigherLikelihood(*currentBest, compare, 2.0) )){
			      		T* xCopy3 = new T(x);
					ObjList<T>* paramList = listOfParamLists[loc];
					paramList->Append(xCopy3);
					++totalSize;}
			}
		else{
			ClinePopList* newList = new ClinePopList(centerPopList);
			listOfCenterPopLists.Append(newList);
			T* newCurrent = new T(x);
			currentParamList.Append(newCurrent);
			T* newBest = new T(x);
			bestParamList.Append(newBest);
			ObjList<T>* paramList = new ObjList<T>();
			listOfParamLists.Append(paramList);//at least append an empty list
			samplingAcuity.Append(initAcuity);
			thisPopCombAnnealed.Append(False);//change in later call, if necessary
//			if (saving()){
			T* xCopy4 = new T(x);
			paramList->Append(xCopy4);
			++totalSize;
//				}
			}
	if (totalSize>=savedReps){
		SetSave(False);}
	return saving();
	}//end LikelihoodList<T>::Append



template<class T> T* LikelihoodList<T>::best(ClinePopList& centerPopList){
	if (listOfCenterPopLists.Includes(&centerPopList)){
			int loc=listOfCenterPopLists.Loc(&centerPopList);
			T* currentBest = 0;
			return currentBest = bestParamList[loc];}
		else{return 0;}
	}//end  LikelihoodList<T>::best
			
			
			

template<class T> double LikelihoodList<T>::bestL(ClinePopList& popCombList){
	T* bestParam = best(popCombList);
	if (bestParam == 0){ return 0.0;}
		else{ return bestParam->Lcline();}
	}//end LikelihoodList<T>::bestL(ClinePopList&)


		

template<class T> T* LikelihoodList<T>::current(ClinePopList& centerPopList){
	if (listOfCenterPopLists.Includes(&centerPopList)){
			int loc=listOfCenterPopLists.Loc(&centerPopList);
			ObjList<T>* paramList = listOfParamLists[loc];
			return paramList->Last();}
		else{return 0;}
	}//end  LikelihoodList<T>::current


template<class T> void LikelihoodList<T>::Find2UnitSupportLimits (const double recombinationRate){
	lowerCI = overallBest;//convenient starting points
	upperCI = overallBest;

	bestDerived.Clear();lowerDerivedCI.Clear();upperDerivedCI.Clear();
	overallBest.DerivedValues(bestDerived,recombinationRate);
	lowerDerivedCI.Copy(bestDerived);
	upperDerivedCI.Copy(bestDerived);

	for(int popSet=0;popSet<listOfParamLists.len();++popSet){
		ObjList<T>* paramList = listOfParamLists[popSet];
		for(long i=0;i<paramList->len();++i){
			T* rec = paramList->Nth(i);
			upperCI.max(*rec);
			lowerCI.min(*rec);
			aList<double> recDerived;
			rec->DerivedValues(recDerived,recombinationRate);
			for(int j=0;j<recDerived.len();++j){
				double derivedVal = recDerived[j];
				double lowDerivedVal = lowerDerivedCI[j];
				double highDerivedVal = upperDerivedCI[j];
				if(derivedVal<lowDerivedVal){
					lowerDerivedCI.Replace(derivedVal,j);}
				if(derivedVal>highDerivedVal){
					upperDerivedCI.Replace(derivedVal,j);}
				}//end for j; checking of derived CI
			}//end for i
		}//end for popSet
	}//end  LikelihoodList<T>::Find2UnitSupportLimits




template<class T> void LikelihoodList<T>::PrintBestParameters(
										long& allTries,
										aList<long>& acceptedChanges, 
										aList<long>& totalTries,
										ostream& outFile){
	long sumA = 0, sumN = 0; 
	aList<double> successRate; double overallSuccessRate = 0.0;
	for (int i=0;i<totalTries.len();++i){
		successRate.Append(0.0);}
	for (i=0;i<successRate.len();++i){
		if (totalTries[i] > 0){				
			successRate.Replace((double) acceptedChanges[i] / (double) totalTries[i], i);
			sumA += acceptedChanges[i];
			sumN += totalTries[i];
			}//end if totalTries
		}//end for i
	if (sumN > 0){
		overallSuccessRate = (double) sumA / (double) sumN;}

	overallBest.PrintShapeWithCI(allTries, overallSuccessRate, successRate, 
						lowerCI, upperCI, bestDerived, lowerDerivedCI, upperDerivedCI, outFile);
						
	}//end  LikelihoodList<T>::PrintBestParameters


	
			
		
template<class T> void LikelihoodList<T>::PurgeUnlikelyParameters(void){
	//removes paramRec's that are unlikely relative to overall best
	for(int set=listOfParamLists.len()-1;set>=0;--set){//backwards down the list
		T* localBest = bestParamList[set];
		if (localBest->Lcline() < overallBest.Lcline() - 2.0){
				//then none of the saved parameters remains valid
				bestParamList.Delete(set);
				currentParamList.Delete(set);
				ObjList<T>* paramList = listOfParamLists[set];
				totalSize -= paramList->len(); paramList=0;
				listOfParamLists.Delete(set);
				listOfCenterPopLists.Delete(set);}
			else{//there may still be some invalid ones in the saved lists		
				ObjList<T>* paramList = listOfParamLists[set];
				for(long rec=paramList->len()-1;rec>=0;--rec){
					T* paramToTest = paramList->Nth(rec);
					if (paramToTest->Lcline() < overallBest.Lcline() - 2.0){
						paramList->Delete(rec);
						--totalSize;}
					}//end for rec
				}//end if...else...
		}//end for set
	}//end LikelihoodList<T>::PurgeUnlikelyParameters()



template<class T> void LikelihoodList<T>::PurgeUnlikelyParameters(int loc){
	//removes T's that are unlikely for centerPopList at loc
	T* localBest = bestParamList[loc];
	ObjList<T>* paramList = listOfParamLists[loc];
	for(long rec=paramList->len()-1;rec>=0;--rec){
		T* paramToTest = paramList->Nth(rec);
		if( paramToTest->HasHigherLikelihood(*localBest, compare, 2.0) ){
 			paramList->Delete(rec);
 			--totalSize;}
		}//end for rec
	}//end  LikelihoodList<T>::PurgeUnlikelyParameters(int)



template<class T>void LikelihoodList<T>::SetBest(ClinePopList& popCombList, T* newBest){
	if(listOfCenterPopLists.Includes(&popCombList)){
			int loc = listOfCenterPopLists.Loc(&popCombList);
			T* oldBest = bestParamList[loc];
			*oldBest = *newBest;}
		else{
			T* bestToAdd = new T(newBest);
			bestParamList.Append(bestToAdd);}
	}//end LikelihoodList<T>::SetBest



