//Shape2ParamRec.cp


#pragma once
#include <Shape2ParamRec.h>
#include <ClinePopList.h>
#include <FitMultilocusCline.h>


Shape2ParamRec* Shape2ParamRec::ChangeParams (Shape2ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t, const int annealingShape){
	int OK=True;
	thisOne = ChooseParameter(ran, annealingShape);
	Shape2ParamRec* paramToChange = new Shape2ParamRec(this);//calls copy constructor
	do	{
		OK = True;
		if (thisOne == 1){//change center point by a random amount
				paramToChange->c_ = ran.NormDev(center(), paramDeviations.center() * t);}
			else if (thisOne == 2){//change width by random amount
				paramToChange->w_ = ran.NormDev(w(), paramDeviations.w() * t);}
		OK = paramToChange->SetShape();
		}while (! OK);
	return paramToChange;
	}//	end Shape2ParamRec::ChangeParams



int Shape2ParamRec::ChooseParameter(Ran3& ran, const int annealingShape){
		//randomly chooses a parameter to vary
		return ran.longBetween(1,2);//randomly choose center (1) or width (2)
		}//end Shape2ParamRec::ChooseParameter



int 	Shape2ParamRec::FindCenterPopulations(ClinePopList& centerPopList, ClinePopList& popLocation){
	return centerPopList.FindCenterPopulations(*this, popLocation);}



int Shape2ParamRec:: MeetsParamCriteria (void){
		int OK = True;
		if ((center() < leftEnd) || (center() > rightEnd)) {
				OK = False;}
			else if (w() <= 0.0) {
				OK = False;}
		return OK;
	}//end  Shape2ParamRec:: MeetsParamCriteria



double Shape2ParamRec::PonCline(double x, double score){
		//returns the value of p on the cline itself from the cline shape
		return  (score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (x - center()) / w())) / 2.0));
	}// end Shape2ParamRec::PonCline



void Shape2ParamRec::PrintShapeWithCI(const long allTries, 
							const double overallSuccessRate, 
							aList<double>& successRate,
							Shape2ParamRec& lowerCI, 
							Shape2ParamRec& upperCI, 
							aList<double>& bestDerived, 
							aList<double>& lowerDerivedCI, 
							aList<double>& upperDerivedCI, 
							ostream& outFile){//called by the best parameter set only
		outFile<<endl;
		outFile<<"cline L max: ";outFile.precision(25);outFile<< Lcline()<<endl;
		outFile<<"total replicates tried: "<<allTries<<endl;
		outFile<<"overall acceptance rate: "; outFile.precision(5);outFile<<overallSuccessRate<<endl;

		outFile<<"center: ";outFile.precision(10);outFile<<center()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.center()<<" - ";
					      outFile.precision(10);outFile<<upperCI.center()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[0]<<endl;
		outFile<<"width: ";outFile.precision(10);outFile<<w()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.w()<<" - ";
					      outFile.precision(10);outFile<<upperCI.w()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[1]<<endl;
		outFile<<"max slope: ";outFile.precision(10);outFile<<1.0/w()<<"   (";
					      outFile.precision(10);outFile<<1.0/lowerCI.w()<<" - ";
					      outFile.precision(10);outFile<<1.0/upperCI.w()<<")";
					      outFile<<endl;
		outFile<<"bottom L: ";outFile.precision(10);outFile<<bottomL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.bottomL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.bottomL()<<")";
					      outFile<<endl;
		outFile<<"top R: ";outFile.precision(10);outFile<<topR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.topR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.topR()<<")";
					      outFile<<endl;

		}//end  Shape2ParamRec::PrintShapeWithCI



void Shape2ParamRec::PrintToScreen (const long tries){
	cout.width(5);cout<<tries<<"   cline Lmax: ";
	cout.precision(6);cout<< Lcline()<<endl;
	cout<<"         c: ";cout.precision(3); cout<<center();
	cout<<"         w: ";cout.precision(3); cout<<w();
	cout<<endl;
	cout<<"   bottomL: ";cout.precision(6); cout<< bottomL();
	cout<<"      topR: ";cout.precision(6); cout<< topR();
	cout<<endl<<endl;
	}//end  Shape2ParamRec::PrintToScreen


void Shape2ParamRec::PrintToScreen (const long tries, const double sigma){
	PrintToScreen(tries);
	cout<<"     hets.s*: ";cout.precision(6); cout<< sVsHets(sigma);
	cout<<"     eco.s*: ";cout.precision(6); cout<< sStarEcotone(sigma);
	cout<<endl;
	cout<<"Dom.eco.s*: ";cout.precision(6); cout<< sStarDomEcotone(sigma);
	cout<<endl;
	}//end  Shape2ParamRec::PrintToScreen


void	 Shape2ParamRec::SetDeviationValues(void){
	//set initial parameter deviation values (standard deviations)
	//parameters deviated by random variables with these standard deviations
	// these are now constants defined in AllozymeClineGlobals.h
	c_ = centerDev;
	w_ = wDev;
	}//end Shape2ParamRec::SetDeviationValues



void Shape2ParamRec::SetClineLikelihoodRatios(double LmaxCline){
	LtoLmaxCline_ = exp(Lcline() - LmaxCline);//this form gives the ratio of the true likelihoods
	LmaxMinusLcline_ = LmaxCline - Lcline();
	}//end  Shape2ParamRec::SetClineLikelihoodRatios(double)



int Shape2ParamRec::SetShape(void){
		//see if shape is legal
		int OK = MeetsParamCriteria();
		return OK;
		}	//end Shape2ParamRec::SetShape



void	 Shape2ParamRec::SetStartingValues(void){
	// plausible arbitrary values that start the iteration process
	// these are now constants defined in AllozymeClineGlobals.h
	c_ = Icenter;
	w_ = Iw;
	}//end Shape2ParamRec::SetStartingValues

	
	
	

