//Shape3ParamRec.cp


#pragma once
#include <Shape3ParamRec.h>
#include <ClinePopList.h>
#include <FitMultilocusCline.h>




Shape3ParamRec* Shape3ParamRec::ChangeDiseqParam (Shape3ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t){
	int OK=True;
	thisOne = 3;
	Shape3ParamRec* paramToChange = new Shape3ParamRec(*this);//calls copy constructor
	do	{
		OK = True;
		//change disequilibrium by random amount
		paramToChange->D_ = ran.NormDev(D(), paramDeviations.D() * t);
		if ((D() < diseqBottom) || (D() > diseqTop)) {
			OK = False;}
		}while (! OK);
	return paramToChange;
	}//	end Shape3ParamRec::ChangeParams



Shape3ParamRec* Shape3ParamRec::ChangeParams (Shape3ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t, const int annealingShape){
	int OK=True;
	thisOne = ChooseParameter(ran, annealingShape);
	Shape3ParamRec* paramToChange = new Shape3ParamRec(*this);//calls copy constructor
	do	{
		OK = True;
		if (thisOne == 1){//change center point by a random amount
				paramToChange->c_ = ran.NormDev(center(), paramDeviations.center() * t);}
			else if (thisOne == 2){//change width by random amount
				paramToChange->w_ = ran.NormDev(w(), paramDeviations.w() * t);}
			else if (thisOne == 3){//change disequilibrium by random amount
				paramToChange->D_ = ran.NormDev(D(), paramDeviations.D() * t);}
		OK = paramToChange->SetShape();
		}while (! OK);
	return paramToChange;
	}//	end Shape3ParamRec::ChangeParams

void Shape3ParamRec::Clear(void){
	c_=w_=Lcline_=LtoLmaxCline_=LmaxMinusLcline_=0.0;
	D_=Ldiseq_=LtoLmaxDiseq_=LmaxMinusLDiseq_=0.0;
	Ltotal_=LtoLmaxTotal_=LmaxMinusLTotal_=0.0;
	}//	end Shape3ParamRec::Clear


Shape3ParamRec* 	Shape3ParamRec::Copy(void){//must be deleted elsewhere
						return new Shape3ParamRec(*this);}


int Shape3ParamRec::ChooseParameter(Ran3& ran, const int annealingShape){
	//randomly chooses a parameter to vary
	//randomly choose center (1), width (2), or disequilibrium (3)
	if (annealingShape){
			return ran.longBetween(1,2);}
		else {
			return ran.longBetween(1,3);}
	}//end Shape3ParamRec::ChooseParameter


void	 Shape3ParamRec::DerivedValues(aList<double>& derivedList, double recombinationRate){
	derivedList.Clear();
	double r = recombinationRate;
	derivedList.Append(sigma(r));
	derivedList.Append(sVsHets(r));
	derivedList.Append(sStarEcotone(r));
	derivedList.Append(sStarDomEcotone(r));
	}//end Shape3ParamRec::DerivedValues



int Shape3ParamRec::FindCenterPopulations(ClinePopList& centerPopList, ClinePopList& popLocation){
	return centerPopList.FindCenterPopulations(*this, popLocation);}


double Shape3ParamRec::GetDiseqSuccess(aList<double>& successRate){
	return successRate[2];}//D is third parameter varied in ChangeParams


int Shape3ParamRec::HasHigherLikelihood(Shape3ParamRec& param, likelihoodType compare, const double ci){//boolean
	//always greater if param.L == 0
	if (compare == cline){
			return ((param.Lcline() == 0.0) ||  (Lcline() > param.Lcline() - ci ));}
		else if (compare ==disequilibrium){
			return ((param.Ldiseq() == 0.0) ||  (Ldiseq() > param.Ldiseq() - ci ));}
		else {// (compare ==total)
			return ((param.Ltotal() == 0.0) ||  (Ltotal() > param.Ltotal() - ci ));}
	}//end HasHigherLikelihood



void Shape3ParamRec::max(Shape3ParamRec& param){//sets parameters to the maximum of this & param
	if(param.center()>center()){c_=param.center();}
	if(param.w()>w()){w_=param.w();}
	Lcline_=LtoLmaxCline_=LmaxMinusLcline_=0.0;
	if(param.D()>D()){D_=param.D();}
	Ldiseq_=LtoLmaxDiseq_=LmaxMinusLDiseq_=0.0;
	Ltotal_=LtoLmaxTotal_=LmaxMinusLTotal_=0.0;
	}//end Shape3ParamRec::max



int Shape3ParamRec:: MeetsParamCriteria (void){
	int OK = True;
	if ((center() < leftEnd) || (center() > rightEnd)) {
			OK = False;}
		else if (w() <= 0.0) {
			OK = False;}
		else if ((D() < diseqBottom) || (D() > diseqTop)) {
			OK = False;}
	return OK;
	}//end  Shape3ParamRec:: MeetsParamCriteria

void Shape3ParamRec::min(Shape3ParamRec& param){//sets parameters to the minimum of this & param
	if(param.center()<center()){c_=param.center();}
	if(param.w()<w()){w_=param.w();}
	Lcline_=LtoLmaxCline_=LmaxMinusLcline_=0.0;
	if(param.D()<D()){D_=param.D();}
	Ldiseq_=LtoLmaxDiseq_=LmaxMinusLDiseq_=0.0;
	Ltotal_=LtoLmaxTotal_=LmaxMinusLTotal_=0.0;
	}//end  Shape3ParamRec:: min


double Shape3ParamRec::PonCline(double x, double score){
		//returns the value of p on the cline itself from the cline shape
		return  (score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (x - center()) / w())) / 2.0));
	}// end Shape3ParamRec::PonCline


void Shape3ParamRec::PrintDerivedWithCI(Shape3ParamRec& lowerCI, 
								   Shape3ParamRec& upperCI, 
								   aList<double>& bestDerived,
								   aList<double>& lowerDerivedCI, 
								   aList<double>& upperDerivedCI, 
								   ostream& outFile){//called by the best total parameter set only
/*
				if popCombListDiseq.len > 0 then
					writeLn(outputData, 'Population combinations in disequilibrium list:', popCombListDiseq.len : 3);
				if popCombListTotal.len > 0 then
					writeLn(outputData, 'Population combinations in totals list:', popCombListTotal.len : 3);
				if popCombListDiseq.len > 0 then
					writeln(outputData, '(Support limits taken from all population combinations pooled.)');
				for i := 1 to popCombListDiseq.len do
					begin
						centerPopList := ListStructureTypePtr(GetSubListAt(popCombListDiseq, i));
						writeln(outputData, 'Populations included in this disequilibrium estimate:');
						PrintPopCombList(centerPopList^, outputData);
						writeln(outputData, 'diseq Lmax: ', bestparamsDiseq^[i].Ldiseq : 20 : 12);
						writeln(outputData, 'D: ', bestparamsDiseq^[i].D : 15 : 10, Spaces(3), '(', lowerCIset.D : 15 : 10, ' - ', upperCIset.D : 15 : 10, ')');
						centerPopList := ListStructureTypePtr(GetSubListAt(popCombListTotal, i));
						writeln(outputData, 'Populations included in these estimates of derived cline parameters:');
						PrintPopCombList(centerPopList^, outputData);
*/
		outFile<<endl;
		outFile<<"total L max: ";outFile.precision(25);outFile<< Lcline()<<endl;
		outFile<<"sigma: ";outFile.precision(10);outFile<<bestDerived[2]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[2]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[2]<<")";
					      outFile<<endl;
		outFile<<"s* vs. hets: ";outFile.precision(10);outFile<<bestDerived[6]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[6]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[6]<<")";
					      outFile<<endl;
		outFile<<"s* ecotone: ";outFile.precision(10);outFile<<bestDerived[8]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[8]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[8]<<")";
					      outFile<<endl;
		outFile<<"s* ecotone w/ dominance: ";outFile.precision(10);outFile<<bestDerived[9]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[9]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[9]<<")";
					      outFile<<endl;
		outFile<<endl;
		}//end Shape3ParamRec::PrintDerivedWithCI


void Shape3ParamRec::PrintDiseqWithCI(const double successRate, 
							Shape3ParamRec& lowerCI, 
							Shape3ParamRec& upperCI, 
							ostream& outFile){//called by the best parameter set only		
		outFile<<endl;
		outFile<<"diseq L max: ";outFile.precision(25);outFile<< Ldiseq()<<endl;
		outFile<<"D: ";outFile.precision(10);outFile<<D()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.D()<<" - ";
					      outFile.precision(10);outFile<<upperCI.D()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate<<endl;
	}//end Shape3ParamRec::PrintDiseqWithCI



void Shape3ParamRec::PrintShapeWithCI(const long allTries, 
							const double overallSuccessRate, 
							aList<double>& successRate,
							Shape3ParamRec& lowerCI, 
							Shape3ParamRec& upperCI, 
							aList<double>& bestDerived, 
							aList<double>& lowerDerivedCI, 
							aList<double>& upperDerivedCI, 
							ostream& outFile){//called by the best parameter set only
		outFile<<endl;
		outFile<<"cline L max: ";outFile.precision(25);outFile<< Lcline()<<endl;
		outFile<<"total replicates tried: "<<allTries<<endl;
		outFile<<"overall acceptance rate: "; outFile.precision(5);outFile<<overallSuccessRate<<endl;

		outFile<<"center: ";outFile.precision(10);outFile<<center()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.center()<<" - ";
					      outFile.precision(10);outFile<<upperCI.center()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[0]<<endl;
		outFile<<"width: ";outFile.precision(10);outFile<<w()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.w()<<" - ";
					      outFile.precision(10);outFile<<upperCI.w()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[1]<<endl;
		outFile<<"max slope: ";outFile.precision(10);outFile<<1.0/w()<<"   (";
					      outFile.precision(10);outFile<<1.0/lowerCI.w()<<" - ";
					      outFile.precision(10);outFile<<1.0/upperCI.w()<<")";
					      outFile<<endl;
		outFile<<"bottom L: ";outFile.precision(10);outFile<<bottomL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.bottomL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.bottomL()<<")";
					      outFile<<endl;
		outFile<<"top R: ";outFile.precision(10);outFile<<topR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.topR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.topR()<<")";
					      outFile<<endl;

		}//end  Shape3ParamRec::PrintShapeWithCI



void Shape3ParamRec::PrintToScreen (const long tries, const double recombinationRate){
	cout.width(5);cout<<tries<<"   cline Lmax: ";
	cout.precision(6);cout<< Lcline()<<endl;
	cout<<"         c: ";cout.precision(3); cout<<center();
	cout<<"         w: ";cout.precision(3); cout<<w();
	cout<<endl;
	cout<<"   bottomL: ";cout.precision(6); cout<< bottomL();
	cout<<"      topR: ";cout.precision(6); cout<< topR();
	cout<<endl;
	cout<<"   diseq Lmax: "; cout.precision(6); cout<< Ldiseq()<<endl;
	cout<<"         D: ";cout.precision(6); cout<<D();
	cout<<"     sigma: ";cout.precision(2); cout<< sigma(recombinationRate);
	cout<<endl;
	cout<<"     hets.s*: ";cout.precision(6); cout<< sVsHets(sigma(recombinationRate));
	cout<<"     eco.s*: ";cout.precision(6); cout<< sStarEcotone(sigma(recombinationRate));
	cout<<endl;
	cout<<"Dom.eco.s*: ";cout.precision(6); cout<< sStarDomEcotone(sigma(recombinationRate));
	cout<<endl<<endl;
	}//end  Shape3ParamRec::PrintToScreen


void Shape3ParamRec::SetClineLikelihoodRatios(double LmaxCline){
	LtoLmaxCline_ = exp(Lcline() - LmaxCline);//this form gives the ratio of the true likelihoods
	LmaxMinusLcline_ = LmaxCline - Lcline();
	}//end  Shape3ParamRec::SetClineLikelihoodRatios(double)


void	 Shape3ParamRec::SetDeviationValues(void){
	//set initial parameter deviation values (standard deviations)
	//parameters deviated by random variables with these standard deviations
	// these are now constants defined in AllozymeClineGlobals.h
	c_ = centerDev;
	w_ = wDev;
	D_ = diseqDev;
	}//end Shape3ParamRec::SetDeviationValues


void Shape3ParamRec::SetDiseqLikelihoodRatios(const double LmaxDiseq){
	LtoLmaxDiseq_ = exp(Ldiseq() - LmaxDiseq);//this form gives the ratio of the true likelihoods
	LmaxMinusLDiseq_ = LmaxDiseq - Ldiseq();
	}//end  Shape3ParamRec::SetDiseqLikelihoodRatios(double)


void Shape3ParamRec::SetNewBest(void){
	LtoLmaxCline_ = 1.0; LmaxMinusLcline_ = 0.0;}


void Shape3ParamRec::SetNewBestDiseq(void){
	LtoLmaxDiseq_ = 1.0; LmaxMinusLDiseq_ = 0.0;}


void Shape3ParamRec::SetNewBestTotal(void){
	LtoLmaxTotal_ = 1.0; LmaxMinusLTotal_ = 0.0;}


void Shape3ParamRec::SetTotal(void){
	Ltotal_ = Lcline() + Ldiseq();}


void Shape3ParamRec::SetTotalLikelihoodRatios(const double LmaxTotal){
	LtoLmaxTotal_ = exp(Ltotal() - LmaxTotal);//this form gives the ratio of the true likelihoods
	LmaxMinusLTotal_ = LmaxTotal - Ltotal();
	}//end  Shape3ParamRec::SetTotalLikelihoodRatios(double)



int Shape3ParamRec::SetShape(void){
		//see if shape is legal
		int OK = MeetsParamCriteria();
		return OK;
		}	//end Shape3ParamRec::SetShape


void	 Shape3ParamRec::SetStartingValues(void){
	// plausible arbitrary values that start the iteration process
	// these are now constants defined in AllozymeClineGlobals.h
	c_ = Icenter;
	w_ = Iw;
	D_ = Idiseq;
	}//end Shape3ParamRec::SetStartingValues

void Shape3ParamRec::operator=(Shape3ParamRec& x){
							c_= x.center();
							w_= x.w();
							D_= x.D();
							Lcline_= x.Lcline();
							LtoLmaxCline_= x.LtoLmaxCline();
							LmaxMinusLcline_= x.LmaxMinusLcline();
							Ldiseq_= x.Ldiseq();
							LtoLmaxDiseq_= x.LtoLmaxDiseq();
							LmaxMinusLDiseq_= x.LmaxMinusLDiseq();
							Ltotal_ = x.Ltotal();
							LtoLmaxTotal_ = x.LtoLmaxTotal();
							LmaxMinusLTotal_ = x.LmaxMinusLTotal();
							}

	
	
int operator== (Shape3ParamRec& a, Shape3ParamRec& b){
	if(	(a.center()==b.center())&&			
		(a.w()==b.w())	&&
		(a.D()==b.D())
	    )		 {return 1;}else{ return 0;}
	}//end operator ==


int operator!= (Shape3ParamRec& a, Shape3ParamRec& b){
							return (!(a==b));}


