// Shape3ParamRec.h

#ifndef Shape3ParamRech
#define Shape3ParamRech = 1

#include "AllozymeClineGlobals.h"

class ClinePopList;
class Shape3ParamRec;

class Shape3ParamRec{
		//a set of cline shape parameters and their derived parameters
		//for only width, center location, and disequilibrium
	    public:
		double c_;//when fitting, these are randomly chosen
		double w_;		//width
		double Lcline_;	//likelihood of this parameter set
		double LtoLmaxCline_;
		double LmaxMinusLcline_;
		double D_;//disequilibrium
		double Ldiseq_;	//likelihood of this disequilibrium value
		double LtoLmaxDiseq_;
		double LmaxMinusLDiseq_;
		double Ltotal_;	//likelihood of this parameter set
		double LtoLmaxTotal_;
		double LmaxMinusLTotal_;
	public:
		Shape3ParamRec(void): Lcline_(0.0), LtoLmaxCline_(0.0), LmaxMinusLcline_(0.0),
			c_(0.0), w_(0.0), Ldiseq_(0.0), LtoLmaxDiseq_(0.0), LmaxMinusLDiseq_(0.0), D_(0.0),
			Ltotal_(0.0), LtoLmaxTotal_(0.0), LmaxMinusLTotal_(0.0)
			{}
		Shape3ParamRec(Shape3ParamRec& x):	//copy constructor
					c_(x.center()),
					w_(x.w()),
					D_(x.D()),
					Lcline_(x.Lcline()),
					LtoLmaxCline_(x.LtoLmaxCline()),
					LmaxMinusLcline_(x.LmaxMinusLcline()),
					Ldiseq_(x.Ldiseq()),
					LtoLmaxDiseq_(x.LtoLmaxDiseq()),
					LmaxMinusLDiseq_(x.LmaxMinusLDiseq()),
					Ltotal_(x.Ltotal()),
					LtoLmaxTotal_(x.LtoLmaxTotal()),
					LmaxMinusLTotal_(x.LmaxMinusLTotal())
					{}
		Shape3ParamRec(Shape3ParamRec* x):	//copy constructor
					c_(x->center()),
					w_(x->w()),
					D_(x->D()),
					Lcline_(x->Lcline()),
					LtoLmaxCline_(x->LtoLmaxCline()),
					LmaxMinusLcline_(x->LmaxMinusLcline()),
					Ldiseq_(x->Ldiseq()),
					LtoLmaxDiseq_(x->LtoLmaxDiseq()),
					LmaxMinusLDiseq_(x->LmaxMinusLDiseq()),
					Ltotal_(x->Ltotal()),
					LtoLmaxTotal_(x->LtoLmaxTotal()),
					LmaxMinusLTotal_(x->LmaxMinusLTotal())
					{}
		virtual ~Shape3ParamRec(void){}



	virtual double 	center(void){return c_;}
	virtual double 	w(void) {return w_;}	
	virtual double 	bottomL(void){return 0.0;}//average allele freq on left side
	virtual double 	topR(void){return 1.0;} //average allele freq on right side
	virtual double 	Lcline(void){return Lcline_;}
	virtual double 	LtoLmaxCline(void) {return LtoLmaxCline_;}	
	virtual double 	LmaxMinusLcline(void){return LmaxMinusLcline_;}
	virtual double 	D(void){return D_;}
	virtual double 	Ldiseq(void){return Ldiseq_;}
	virtual double 	LtoLmaxDiseq(void){return LtoLmaxDiseq_;}
	virtual double 	LmaxMinusLDiseq(void){return LmaxMinusLDiseq_;}
	virtual double 	Ltotal(void){return Ltotal_;}
	virtual double 	LtoLmaxTotal(void) {return LtoLmaxTotal_;}	
	virtual double 	LmaxMinusLTotal(void){return LmaxMinusLTotal_;}
	virtual void	 	center(double c){c_ = c;}
	virtual void	 	w(double w) {w_ = w;}	
	virtual void	 	Lcline(double L){Lcline_ = L;}
	virtual void	 	LtoLmaxCline(double ratio) {LtoLmaxCline_ = ratio;}	
	virtual void	 	LmaxMinusLcline(double diff){LmaxMinusLcline_ = diff;}
	virtual void	 	D(double d){D_ = d;}
	virtual void	 	Ldiseq(double L){Ldiseq_ = L;}
	virtual void	 	LtoLmaxDiseq(double ratio){LtoLmaxDiseq_ = ratio;}
	virtual void	 	LmaxMinusLDiseq(double diff){LmaxMinusLDiseq_ = diff;}
	virtual void	 	Ltotal(double L){Ltotal_ = L;}
	virtual void	 	LtoLmaxTotal(double ratio) {LtoLmaxTotal_ = ratio;}	
	virtual void	 	LmaxMinusLTotal(double diff){LmaxMinusLTotal_ = diff;}

	//DERIVED VARIABLES
	virtual double sigma (const double recombinationRate){		
					if (D() < 0.0) {
							return 0.0;}
						else{
							return sqrt((D() * recombinationRate * w() * w()) / (1.0 + recombinationRate));}
					}//end sigma

	virtual double sVsHets (const double recombinationRate){
						double s = sigma(recombinationRate);
						return (2.0*s / w())*(2.0*s / w());}
	virtual double sStarEcotone (const double recombinationRate){	//selection in an ecotone, =(1.732*sigma/w)^2
						double s = sigma(recombinationRate);
						return (1.732*s / w())*(1.732*s / w());}
	virtual double sStarDomEcotone (const double recombinationRate){//selection in an ecotone vs dominant locus, =(1.782*sigma/w)^2
						double s = sigma(recombinationRate);
						return (1.782*s / w())*(1.782*s / w());}


	//STANDARD FUNCTIONS
	virtual Shape3ParamRec* 	
					ChangeDiseqParam (Shape3ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t);
	virtual Shape3ParamRec* 	
					ChangeParams (Shape3ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t, const int annealingShape);
	virtual int		ChooseParameter(Ran3& ran, const int annealingShape=0);
	virtual void		Clear(void);
	virtual Shape3ParamRec* 	Copy(void);//must be deleted elsewhere
	virtual void		DerivedValues(aList<double>& derivedList, double recombinationRate);
								//listed in order of calls above
	virtual int 		FindCenterPopulations(ClinePopList& centerPopList, ClinePopList& popLocation);
	virtual double 	GetDiseqSuccess(aList<double>& successRate);
							//D is third parameter varied in ChangeParams
	virtual int		HasHigherLikelihood(Shape3ParamRec& param, likelihoodType compare, const double ci = 0.0);//boolean
						//always greater if param.L == 0
	virtual void		max(Shape3ParamRec& param);//sets parameters to the maximum of this & param
	virtual int 		MeetsParamCriteria (void);//boolean for legal parameters
	virtual void		min(Shape3ParamRec& param);//sets parameters to the minimum of this & param
	virtual double 	PonCline(double x, double score);
	virtual void 		PrintDerivedWithCI(Shape3ParamRec& lowerCI, 
								   Shape3ParamRec& upperCI, 
								   aList<double>& bestDerived,
								   aList<double>& lowerDerivedCI, 
								   aList<double>& upperDerivedCI, 
								   ostream& outFile);//called by the best total parameter set only
	virtual void 		PrintDiseqWithCI(const double successRate, 
							Shape3ParamRec& lowerCI, 
							Shape3ParamRec& upperCI, 
							ostream& outFile);//called by the best parameter set only		
	virtual void 		PrintShapeWithCI(const long allTries, 
							const double overallSuccessRate, 
							aList<double>& successRate,
							Shape3ParamRec& lowerCI, 
							Shape3ParamRec& upperCI, 
							aList<double>& bestDerived, 
							aList<double>& lowerDerivedCI, 
							aList<double>& upperDerivedCI, 
							ostream& outFile);//called by the best parameter set only
	virtual void		PrintToScreen(const long tries, const double recombinationRate);
	virtual void 		SetClineLikelihoodRatios(double LmaxCline);
	virtual void 		SetDiseqLikelihoodRatios(const double LmaxDiseq);
	virtual void 		SetTotal(void);
	virtual void 		SetTotalLikelihoodRatios(const double LmaxTotal);
	virtual void		SetDeviationValues(void);
	virtual void 		SetNewBest(void);
	virtual void 		SetNewBestDiseq(void);
	virtual void 		SetNewBestTotal(void);
	virtual int    		SetShape(void);//returns boolean false if illegal shape
	virtual void		SetStartingValues(void);
	virtual void 		operator=(Shape3ParamRec& x);

	friend int 		operator== (Shape3ParamRec& a, Shape3ParamRec& b);

	friend int 		operator!= (Shape3ParamRec& a, Shape3ParamRec& b);

	};//end class Shape3ParamRec

#endif
