// Shape4ParamRec.h

#ifndef Shape4ParamRech
#define Shape4ParamRech = 1

#include <Shape2ParamRec.h>

class ClinePopList;
class Shape4ParamRec;

class Shape4ParamRec: public Shape2ParamRec{//a set of cline shape parameters and their derived parameters
		//for width, center location, and top and bottom allele frequencies
	    public:
		double bottomL_;	//average allele freq on left side
		double topR_;		//average allele freq on right side
	public:
		Shape4ParamRec(void):
			bottomL_(0.0), topR_(0.0) {}
		Shape4ParamRec(Shape4ParamRec& x){	//copy constructor
				c_= x.center();
				w_= x.w();
				bottomL_ = x.bottomL();
				topR_ = x.topR(); 
				Lcline_= x.Lcline();
				LtoLmaxCline_= x.LtoLmaxCline();
				LmaxMinusLcline_= x.LmaxMinusLcline();
				 }
		Shape4ParamRec(Shape4ParamRec* x){	//copy constructor
				c_= x->center();
				w_= x->w();
				bottomL_ = x->bottomL();
				topR_ = x->topR();
				Lcline_= x->Lcline();
				LtoLmaxCline_= x->LtoLmaxCline();
				LmaxMinusLcline_= x->LmaxMinusLcline();
				}
		virtual ~Shape4ParamRec(void){}

		virtual double bottomL(void){return bottomL_;}//average allele freq on left side
		virtual double topR(void){return topR_;} //average allele freq on right side
		virtual void	 bottomL(double p){bottomL_ = p;}//average allele freq on left side
		virtual void	 topR(double p){topR_ = p;} //average allele freq on right side


	virtual Shape4ParamRec* 	
					ChangeParams(Shape2ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t, const int annealingShape);
	virtual int		ChooseParameter(Ran3& ran, const int annealingShape=0);
	virtual void		Clear(void){
						Shape2ParamRec::Clear();
						bottomL_=topR_=0.0;}
	virtual Shape4ParamRec* 	Copy(void){//must be deleted elsewhere
						return new Shape4ParamRec(*this);}
	virtual void		DerivedValues(aList<double>& derivedList, double recombinationRate){
								derivedList.Clear();}//no derived variables at this level
	virtual int 		FindCenterPopulations(ClinePopList& centerPopList, ClinePopList& popLocation);
	virtual void		max(Shape4ParamRec& param){//sets parameters to the maximum of this & param
							Shape2ParamRec::max(param);
							if(param.bottomL()>bottomL()){bottomL_=param.bottomL();}
							if(param.topR()>topR()){topR_=param.topR();}
							}
	virtual int 		MeetsParamCriteria (void);//boolean for legal parameters
	virtual void		min(Shape4ParamRec& param){//sets parameters to the minimum of this & param
							Shape2ParamRec::min(param);
							if(param.bottomL()<bottomL()){bottomL_=param.bottomL();}
							if(param.topR()<topR()){topR_=param.topR();}
							}
	virtual void 		PrintShapeWithCI(const long allTries, 
							const double overallSuccessRate, 
							aList<double>& successRate,
							Shape4ParamRec& lowerCI, 
							Shape4ParamRec& upperCI, 
							aList<double>& bestDerived, 
							aList<double>& lowerDerivedCI, 
							aList<double>& upperDerivedCI, 
							ostream& outFile){//called by the best parameter set only
								Shape2ParamRec::PrintShapeWithCI(allTries, overallSuccessRate,
															successRate, lowerCI, upperCI, 
															bestDerived, lowerDerivedCI, 
															upperDerivedCI, outFile);}
	virtual void		SetDeviationValues(void);
	virtual void		SetStartingValues(void);
	virtual void 		operator=(Shape4ParamRec& x){
						c_= x.center();
						w_= x.w();
						bottomL_ = x.bottomL();
						topR_ = x.topR(); 
						Lcline_= x.Lcline();
						LtoLmaxCline_= x.LtoLmaxCline();
						LmaxMinusLcline_= x.LmaxMinusLcline();
						}

	friend int operator== (Shape4ParamRec& a, Shape4ParamRec& b){
		if(	((Shape2ParamRec) a == (Shape2ParamRec) b)&&
			 (a.bottomL()==b.bottomL())&&			(a.topR()==b.topR())
		    )		 {return 1;}else{ return 0;}
		}//end operator ==

	friend int operator!= (Shape4ParamRec& a, Shape4ParamRec& b){
		return (!(a==b));}

	};//end class Shape4ParamRec

#endif

