//Shape5ParamRec.cp



#include <Shape5ParamRec.h>
#include <ClinePopList.h>
//#include <FitMultilocusCline.h>



Shape5ParamRec* Shape5ParamRec::ChangeDiseqParam (Shape5ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t){
	int OK=True;
	thisOne = 5;
	Shape5ParamRec* paramToChange = new Shape5ParamRec(*this);//calls copy constructor
	do	{
		OK = True;
		//change disequilibrium by random amount
		paramToChange->D_ = ran.NormDev(D(), paramDeviations.D() * t);
		if ((paramToChange->D() < diseqBottom) || (paramToChange->D() > diseqTop)) {
			OK = False;}
		}while (! OK);
	return paramToChange;
	}//	end Shape5ParamRec::ChangeParams



Shape5ParamRec* Shape5ParamRec::ChangeParams(Shape5ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t, const int annealingShape){
	int OK=True;
	thisOne = ChooseParameter(ran, annealingShape);
	Shape5ParamRec* paramToChange = new Shape5ParamRec(*this);//calls copy constructor
	do	{
		OK = True;
		if (thisOne == 1){//change center point by a random amount
				paramToChange->c_ = ran.NormDev(center(), paramDeviations.center() * t);}
			else if (thisOne == 2){//change width by random amount
				paramToChange->w_ = ran.NormDev(w(), paramDeviations.w() * t);}
			else if (thisOne == 3){//change bottomL by random amount
				paramToChange->bottomL_ = ran.NormDev(bottomL(), paramDeviations.bottomL() * t);}
			else if (thisOne == 4){//change topR by random amount
				paramToChange->topR_ = ran.NormDev(topR(), paramDeviations.topR() * t);}
			else if (thisOne == 5){//change disequilibrium by random amount
				paramToChange->D_ = ran.NormDev(D(), paramDeviations.D() * t);}
		OK = paramToChange->SetShape();
		}while (! OK);
	return paramToChange;
	}//	end Shape5ParamRec::ChangeParams



int Shape5ParamRec::ChooseParameter(Ran3& ran, const int annealingShape){
	//randomly chooses a parameter to vary
	//randomly choose center (1), width (2), bottomL (3), topR (4), or disequilibrium (5)
	if (annealingShape){
			return ran.longBetween(1,4);}
		else {
			return ran.longBetween(1,5);}
	}//end Shape5ParamRec::ChooseParameter



void	 Shape5ParamRec::DerivedValues(aList<double>& derivedList, double recombinationRate){
	derivedList.Clear();
	double r = recombinationRate;
	derivedList.Append(sigma(r));
	derivedList.Append(sVsHets(r));
	derivedList.Append(sStarEcotone(r));
	derivedList.Append(sStarDomEcotone(r));
	}//end Shape5ParamRec::DerivedValues



int Shape5ParamRec::FindCenterPopulations(ClinePopList& centerPopList, ClinePopList& popLocation){
	return centerPopList.FindCenterPopulations(*this, popLocation);}



int Shape5ParamRec:: MeetsParamCriteria (void){
	int OK = True;
	if ((center() < leftEnd) || (center() > rightEnd)) {
			OK = False;}
		else if (w() <= 0.0) {
			OK = False;}
		else if ((D() < diseqBottom) || (D() > diseqTop)) {
			OK = False;}
		else if ((bottomL() < minScoreAtL) || (bottomL() > (minScoreAtL + maxScoreAtR) / 2.0)) {
			OK = False;}
		else if ((topR() > maxScoreAtR) || (topR() < (minScoreAtL + maxScoreAtR) / 2.0)) {
			OK = False;}
	return OK;
	}//end  Shape5ParamRec:: MeetsParamCriteria



double Shape5ParamRec::PonCline(const double x, const double score){
		//returns the value of p on the cline itself from the cline shape
		return  (score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (x - center()) / w())) / 2.0));
	}// end Shape5ParamRec::PonCline



void Shape5ParamRec::PrintDerivedWithCI(Shape5ParamRec& lowerCI, 
								   Shape5ParamRec& upperCI, 
								   aList<double>& bestDerived,
								   aList<double>& lowerDerivedCI, 
								   aList<double>& upperDerivedCI, 
								   ostream& outFile){//called by the best total parameter set only

		outFile<<endl;
		outFile<<"total L max: ";outFile.precision(25);outFile<< Lcline()<<endl;
		outFile<<"sigma: ";outFile.precision(10);outFile<<bestDerived[2]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[2]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[2]<<")";
					      outFile<<endl;
		outFile<<"s* vs. hets: ";outFile.precision(10);outFile<<bestDerived[6]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[6]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[6]<<")";
					      outFile<<endl;
		outFile<<"s* ecotone: ";outFile.precision(10);outFile<<bestDerived[8]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[8]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[8]<<")";
					      outFile<<endl;
		outFile<<"s* ecotone w/ dominance: ";outFile.precision(10);outFile<<bestDerived[9]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[9]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[9]<<")";
					      outFile<<endl;
		outFile<<endl;
		}//end Shape5ParamRec::PrintDerivedWithCI


void Shape5ParamRec::PrintDiseqWithCI(const double successRate, 
							Shape5ParamRec& lowerCI, 
							Shape5ParamRec& upperCI, 
							ostream& outFile){//called by the best parameter set only		
		outFile<<endl;
		outFile<<"diseq L max: ";outFile.precision(25);outFile<< Ldiseq()<<endl;
		outFile<<"D: ";outFile.precision(10);outFile<<D()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.D()<<" - ";
					      outFile.precision(10);outFile<<upperCI.D()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate<<endl;
	}//end Shape5ParamRec::PrintDiseqWithCI



void Shape5ParamRec::PrintShapeWithCI(const long allTries, 
							const double overallSuccessRate, 
							aList<double>& successRate,
							Shape5ParamRec& lowerCI, 
							Shape5ParamRec& upperCI, 
							aList<double>& bestDerived, 
							aList<double>& lowerDerivedCI, 
							aList<double>& upperDerivedCI, 
							ostream& outFile){//called by the best parameter set only
		outFile<<endl;
		outFile<<"cline L max: ";outFile.precision(25);outFile<< Lcline()<<endl;
		outFile<<"total replicates tried: "<<allTries<<endl;
		outFile<<"overall acceptance rate: "; outFile.precision(5);outFile<<overallSuccessRate<<endl;

		outFile<<"center: ";outFile.precision(10);outFile<<center()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.center()<<" - ";
					      outFile.precision(10);outFile<<upperCI.center()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[0]<<endl;
		outFile<<"width: ";outFile.precision(10);outFile<<w()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.w()<<" - ";
					      outFile.precision(10);outFile<<upperCI.w()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[1]<<endl;
		outFile<<"max slope: ";outFile.precision(10);outFile<<1.0/w()<<"   (";
					      outFile.precision(10);outFile<<1.0/lowerCI.w()<<" - ";
					      outFile.precision(10);outFile<<1.0/upperCI.w()<<")";
					      outFile<<endl;
		outFile<<"bottom L: ";outFile.precision(10);outFile<<bottomL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.bottomL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.bottomL()<<")";
					      outFile<<endl;
		outFile<<"top R: ";outFile.precision(10);outFile<<topR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.topR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.topR()<<")";
					      outFile<<endl;

		}//end  Shape5ParamRec::PrintShapeWithCI


void Shape5ParamRec::PrintToScreen (const long tries, const double recombinationRate){
	cout.width(5);cout<<tries<<"   cline Lmax: ";
	cout.precision(6);cout<< Lcline()<<endl;
	cout<<"         c: ";cout.precision(3); cout<<center();
	cout<<"         w: ";cout.precision(3); cout<<w();
	cout<<endl;
	cout<<"   bottomL: ";cout.precision(6); cout<< bottomL();
	cout<<"      topR: ";cout.precision(6); cout<< topR();
	cout<<endl;
	cout<<"   diseq Lmax: "; cout.precision(6); cout<< Ldiseq()<<endl;
	cout<<"         D: ";cout.precision(6); cout<<D();
	cout<<"     sigma: ";cout.precision(2); cout<< sigma(recombinationRate);
	cout<<endl;
	cout<<"     hets.s*: ";cout.precision(6); cout<< sVsHets(sigma(recombinationRate));
	cout<<"     eco.s*: ";cout.precision(6); cout<< sStarEcotone(sigma(recombinationRate));
	cout<<endl;
	cout<<"Dom.eco.s*: ";cout.precision(6); cout<< sStarDomEcotone(sigma(recombinationRate));
	cout<<endl<<endl;
	}//end  Shape5ParamRec::PrintToScreen




void Shape5ParamRec::SetClineLikelihoodRatios(double LmaxCline){
	LtoLmaxCline_ = exp(Lcline() - LmaxCline);//this form gives the ratio of the true likelihoods
	LmaxMinusLcline_ = LmaxCline - Lcline();
	}//end  Shape5ParamRec::SetClineLikelihoodRatios(double)


void	 Shape5ParamRec::SetDeviationValues(void){
	//set initial parameter deviation values (standard deviations)
	//parameters deviated by random variables with these standard deviations
	// these are now constants defined in AllozymeClineGlobals.h
	c_ = centerDev;
	w_ = wDev;
	D_ = diseqDev;
	bottomL_ = bottomTopDev;
	topR_ = bottomTopDev;
	}//end Shape5ParamRec::SetDeviationValues



void Shape5ParamRec::SetDiseqLikelihoodRatios(const double LmaxDiseq){
	LtoLmaxDiseq_ = exp(Ldiseq() - LmaxDiseq);//this form gives the ratio of the true likelihoods
	LmaxMinusLDiseq_ = LmaxDiseq - Ldiseq();
	}//end  Shape5ParamRec::SetDiseqLikelihoodRatios(double)



void	 Shape5ParamRec::SetStartingValues(void){
	// plausible arbitrary values that start the iteration process
	// these are now constants defined in AllozymeClineGlobals.h
	c_ = Icenter;
	w_ = Iw;
	D_ = Idiseq;
	bottomL_ = minScoreAtL;
	topR_ = maxScoreAtR;
	}//end Shape5ParamRec::SetStartingValues

	

void Shape5ParamRec::SetTotalLikelihoodRatios(const double LmaxTotal){
	LtoLmaxTotal_ = exp(Ltotal() - LmaxTotal);//this form gives the ratio of the true likelihoods
	LmaxMinusLTotal_ = LmaxTotal - Ltotal();
	}//end  Shape5ParamRec::SetTotalLikelihoodRatios(double)


int operator== (Shape5ParamRec& a, Shape5ParamRec& b){
	if(	(a.center()==b.center())&&			
		(a.w()==b.w())	&&
		(a.D()==b.D()) &&
		(a.bottomL()==b.bottomL())  &&
		(a.topR()==b.topR())
	    )	{return 1;}else{ return 0;}
	}//end operator ==

	


