//Shape6ParamRec.cp


#pragma once
#include <Shape6ParamRec.h>
#include <ClinePopList.h>
#include <FitMultilocusCline.h>




double Shape6ParamRec::bigS (const double recombinationRate){
	double wHP=WhybridOverWpure(recombinationRate);
	if (wHP == 0.0) {
			return 200.0;}	//an arbirtarily high number
		else{
			return -2.0 * ln(wHP);}
	}//end Shape6ParamRec::bigS




Shape6ParamRec* Shape6ParamRec::ChangeParams(Shape2ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t, const int annealingShape){
	int OK=True;
	thisOne = ChooseParameter(ran, annealingShape);
	Shape6ParamRec* paramToChange = new Shape6ParamRec(*this);//calls copy constructor
	do	{
		OK = True;
		if (thisOne == 1){//change center point by a random amount
				paramToChange->c_ = ran.NormDev(center(), paramDeviations.center() * t);}
			else if (thisOne == 2){//change width by random amount
				paramToChange->w_ = ran.NormDev(w(), paramDeviations.w() * t);}
			else if (thisOne == 3){//change thetaL by random amount
				paramToChange->thetaL_ = ran.NormDev(thetaL(), paramDeviations.thetaL() * t);}
			else if (thisOne == 4){//change thetaR by random amount
				paramToChange->thetaR_ = ran.NormDev(thetaR(), paramDeviations.thetaR() * t);}
			else if (thisOne == 5){//change asymptoteL by random amount
				paramToChange->asymptoteL_ = ran.NormDev(asymptoteL(),  paramDeviations.asymptoteL() * t);}
			else if (thisOne == 6){//change asymptoteR by random amount
				paramToChange->asymptoteR_ = ran.NormDev(asymptoteR(), paramDeviations.asymptoteR() * t);}
		OK = paramToChange->SetShape();
		}while (! OK);
	return paramToChange;
	}//	end Shape6ParamRec::ChangeParams



int Shape6ParamRec::ChooseParameter(Ran3& ran, const int annealingShape){ 
	//randomly choose center (1), width (2), thetaL (3), thetaR(4), asymptoteL (5), or asymptoteR (6)
	return ran.longBetween(1,6);
	}//end Shape6ParamRec::ChooseParameter


int Shape6ParamRec::FindCenterPopulations(ClinePopList& centerPopList, ClinePopList& popLocation){
	return centerPopList.FindCenterPopulations(*this, popLocation);}



int Shape6ParamRec::LJunction (const double score){
	//finds the point where the left and center functions converge
	double x1=center(), x2, xLogP, xTanhP, diff2=0.0,incrementPower=0.0;
	int exitLoop=False, maxedOut=False, junctionFound=False;
	// They might not ever intersect, so 
	//strategy is to iterate down the curves and see where the difference converges.
	//Start at the center and find the difference between curves.
	xLogP = score * (bottomL() + (topR() - bottomL()) * exp(4.0 * (x1 - (center() + asymptoteL())) * sqrt(thetaL()) / w()));
	xTanhP = score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (x1 - center()) / w())) / 2.0);
	double diff1 = xTanhP - xLogP;
	if (diff1 > 0.0) {//continue only if left function is below the center function
		do{//get function values at 2 locations and compare them
			maxedOut = False;
			x2 = x1 - exp(incrementPower * ln(10.0));	//=1*10^incrementPower
			xLogP = score * (bottomL() + (topR() - bottomL()) * exp(4.0 * (x2 - (center() + asymptoteL())) * sqrt(thetaL()) / w()));
			if (abs(2.0 * (x2 - center()) / w()) >= 6000.0){	//maximum double
					xTanhP = score * (bottomL() + (topR() - bottomL()) * 0.5);
					maxedOut = True;}
				else{
					xTanhP = score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (x2 - center()) / w())) / 2.0);}
			diff2 = xTanhP - xLogP;
			if (diff2 <= 0.0){//convergence point surrounded; check for convergence in middle
					incrementPower--;}
				else{//   if  diff2 <= diff1 then	probably converging to a point
					//move to the left one step
					diff1 = diff2;
					if (! maxedOut){
							x1 = x2;}
						else{
							incrementPower--;}
					}//	end if diff2...else...
			if (incrementPower < -3.0){//close enough
					exitLoop = True;
					if (x2 <= leftEnd){
							xL_ = leftEnd;
							betaL_ = score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (xL() - center()) / w())) / 2.0);}
							else{
							xL_=x2;
							betaL_ = (xLogP + xTanhP) / 2.0;}//use the average
					junctionFound = True;
					}//end if incrementPower
			}while (!exitLoop);
		}////end if diff1>0
	return junctionFound;
	}//	end Shape6ParamRec::LJunction



int Shape6ParamRec:: MeetsParamCriteria (void){
	int OK = Shape2ParamRec:: MeetsParamCriteria();
	if ((xL() < leftEnd) || (xL() > rightEnd)) {
			OK = False;}
		else if ((xR() < leftEnd) || (xR() > rightEnd)) {
			OK = False;}
		else if ((asymptoteL() < asymptoteBottom) || (asymptoteL() > asymptoteTop)) {
			OK = False;}
		else if ((asymptoteR() < asymptoteBottom) || (asymptoteR() > asymptoteTop)) {
			OK = False;}
		else if ((thetaR() < thetaBottom) || (thetaR() > thetaTop)) {
			OK = False;}
		else if ((thetaL() < thetaBottom) || (thetaL() > thetaTop)) {
			OK = False;}
	return OK;
	}//end  Shape6ParamRec:: MeetsParamCriteria



double Shape6ParamRec::PonCline(const double x, const double score){
	//returns the value of p on the cline itself from the cline shape
	if (x < xL()) {
			return score * (bottomL() + (topR() - bottomL()) * exp(4.0 * (x - (center() + asymptoteL())) * sqrt(thetaL()) / w()));
			}
		else if (x > xR()) {
			return score * (topR() - (topR() - bottomL()) * exp(-4.0 * (x - (center() - asymptoteR())) * sqrt(thetaR()) / w()));
			}
		else{
			return Shape2ParamRec::PonCline(x,score);
			}
	}// end Shape6ParamRec::PonCline


void Shape6ParamRec::PrintShapeWithCI(const long allTries, 
							const double overallSuccessRate, 
							aList<double>& successRate,
							Shape6ParamRec& lowerCI, 
							Shape6ParamRec& upperCI, 
							aList<double>& bestDerived, 
							aList<double>& lowerDerivedCI, 
							aList<double>& upperDerivedCI, 
							ostream& outFile){//called by the best parameter set only
		outFile<<endl;
		outFile<<"cline L max: ";outFile.precision(25);outFile<< Lcline()<<endl;
		outFile<<"total replicates tried: "<<allTries<<endl;
		outFile<<"overall acceptance rate: "; outFile.precision(5);outFile<<overallSuccessRate<<endl;

		outFile<<"center: ";outFile.precision(10);outFile<<center()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.center()<<" - ";
					      outFile.precision(10);outFile<<upperCI.center()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[0]<<endl;
		outFile<<"width: ";outFile.precision(10);outFile<<w()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.w()<<" - ";
					      outFile.precision(10);outFile<<upperCI.w()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[1]<<endl;
		outFile<<"max slope: ";outFile.precision(10);outFile<<1.0/w()<<"   (";
					      outFile.precision(10);outFile<<1.0/lowerCI.w()<<" - ";
					      outFile.precision(10);outFile<<1.0/upperCI.w()<<")";
					      outFile<<endl;
		outFile<<"bottom L: ";outFile.precision(10);outFile<<bottomL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.bottomL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.bottomL()<<")";
					      outFile<<endl;
		outFile<<"top R: ";outFile.precision(10);outFile<<topR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.topR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.topR()<<")";
					      outFile<<endl;
		outFile<<"theta L: ";outFile.precision(10);outFile<<thetaL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.thetaL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.thetaL()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[2]<<endl;
		outFile<<"theta R: ";outFile.precision(10);outFile<<thetaR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.thetaR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.thetaR()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[3]<<endl;
		outFile<<"asymptote L: ";outFile.precision(10);outFile<<asymptoteL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.asymptoteL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.asymptoteL()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[4]<<endl;
		outFile<<"asymptote R: ";outFile.precision(10);outFile<<asymptoteR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.asymptoteR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.asymptoteR()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[5]<<endl;
		outFile<<"xL: ";outFile.precision(10);outFile<<xL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.xL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.xL()<<")";
					      outFile<<endl;
		outFile<<"xR: ";outFile.precision(10);outFile<<xR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.xR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.xR()<<")";
					      outFile<<endl;
		outFile<<"beta L: ";outFile.precision(10);outFile<<betaL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.betaL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.betaL()<<")";
					      outFile<<endl;
		outFile<<"beta R: ";outFile.precision(10);outFile<<betaR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.betaR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.betaR()<<")";
					      outFile<<endl;
		outFile<<"Wh/Wp: ";outFile.precision(10);outFile<<bestDerived[0]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[0]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[0]<<")";
					      outFile<<endl;
		outFile<<"S: ";outFile.precision(10);outFile<<bestDerived[1]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[1]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[1]<<")";
					      outFile<<endl;
		}//end  Shape6ParamRec::PrintShapeWithCI



void Shape6ParamRec::PrintToScreen (const long tries, const double recombinationRate){
	cout.width(5);cout<<tries<<"   cline Lmax: ";
	cout.precision(6);cout<< Lcline()<<endl;
	cout<<"         c: ";cout.precision(3); cout<<center();
	cout<<"         w: ";cout.precision(3); cout<<w();
	cout<<endl;
	cout<<"   bottomL: ";cout.precision(6); cout<< bottomL();
	cout<<"      topR: ";cout.precision(6); cout<< topR();
	cout<<endl;
	cout<<"      thL: ";cout.precision(6); cout<<thetaL();
	cout<<"     thR: ";cout.precision(6); cout<<thetaR();
	cout<<endl;
	cout<<"  asymptoteL: ";cout.precision(5); cout<<asymptoteL();
	cout<<"  asymptoteR: ";cout.precision(5); cout<< asymptoteR();
	cout<<endl;
	cout<<"       xL: ";cout.precision(6); cout<<xL();
	cout<<"      xR: ";cout.precision(6); cout<<xR();
	cout<<endl;
	cout<<"     betaL: ";cout.precision(6); cout<<betaL();
	cout<<"     betaR: ";cout.precision(6); cout<<betaR();
	cout<<endl<<endl;
	}//end  Shape6ParamRec::PrintToScreen







int Shape6ParamRec::RJunction (const double score){
	//finds the point where the right and center functions converge
	double x1=center(), x2, xLogP, xTanhP, diff2, incrementPower=0.0;
	int exitLoop=False, maxedOut, junctionFound=False;
	//They might not ever intersect, so 
	//strategy is to iterate down the curves and see where the difference converges.
	//Start at the center and find the difference between curves.
	xLogP = score * (topR() - (topR() - bottomL()) * exp(-4.0 * (x1 - (center() - asymptoteR())) * sqrt(thetaR()) / w()));
	xTanhP = score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (x1 - (center())) / w())) / 2.0);
	double diff1 = xTanhP - xLogP;
	if (diff1 < 0.0){//continue only if right function is below the center function
		do{//get function values at 2 locations and compare them
			maxedOut = False;
			x2 = x1 + exp(incrementPower * ln(10.0));//=1*10^incrementPower
			xLogP = score * (topR() - (topR() - bottomL()) * exp(-4.0 * (x2 - (center() - asymptoteR())) * sqrt(thetaR()) / w()));
			if (abs(2.0 * (x2 - center()) / w()) >= 3200){//maximum extended
					xTanhP = score * (bottomL() + (topR() - bottomL()) * 0.5);
					maxedOut = True;}
				else{
					xTanhP = score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (x2 - center()) / w())) / 2.0);}
			diff2 = xTanhP - xLogP;
			if (diff2 >= 0.0){//convergence point surrounded; check for convergence in middle
					incrementPower--;}
				else {//if diff2 >= diff1 then probably converging to a point
					//move to the left one step
					diff1 = diff2;
					if (! maxedOut){
							x1 = x2;}
						else{
							incrementPower--;}
					}//end if diff2...else...
			if (incrementPower < -3.0) {//close enough
					exitLoop = True;
					if (x2 >= rightEnd){
							xR_ = rightEnd;
							betaR_ = score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (xR() - center()) / w())) / 2.0);}
						else{
							xR_ = x2;
							betaR_ = (xLogP + xTanhP) / 2.0;}
					betaR_ /= score;
					junctionFound = True;}
			}while (!exitLoop);
		}//end if diff1<0
	return junctionFound;
	}//end Shape6ParamRec::RJunction



void Shape6ParamRec::SetBarrierStrength(void){
	double dPLdXatxL=0.0, dPRdXatxR=0.0;
	double rVal = PonCline(xR(), maxScore);
	double lVal = PonCline(xL(), maxScore);
	double deltaP = abs(rVal- lVal);

	if (4.0 * sqrt(thetaL()) * (xL() - (center() + asymptoteL())) / w() <= -100.0) {
			dPLdXatxL = exp(-100.0);}	//derivatives found using Mathematica
		else if (4.0 * sqrt(thetaL()) * (xL() - (center() +asymptoteL())) / w() <= 100.0) {
			dPLdXatxL = 4.0 * exp(4.0 * sqrt(thetaL()) * (xL() - (center() + asymptoteL())) / w()) * sqrt(thetaL()) / w();}
		else	{
			dPLdXatxL = exp(100.0);}

	if ((4.0 * sqrt(thetaR()) * (xR() - (center() - asymptoteR())) / w()) <= -100.0) {
			dPRdXatxR = exp(100.0);}
		else if ((4.0 * sqrt(thetaR()) * (xR() - (center() - asymptoteR())) / w()) <= 100.0) {
			dPRdXatxR = 4.0 * sqrt(thetaR()) / (w() * exp(4.0 * sqrt(thetaR()) * (xR() - (center() - asymptoteR())) / w()));}
		else{
			dPRdXatxR = exp(-100.0);}

	if (dPLdXatxL == 0.0) {
			betaL_ = exp(100.0);}
		else{
			betaL_ = deltaP / dPLdXatxL;}
	if (dPRdXatxR == 0.0){
			betaR_ = exp(100.0);}
		else{
			betaR_ = deltaP / dPRdXatxR;}
	}	//end Shape6ParamRec::SetBarrierStrength



void	 Shape6ParamRec::SetDeviationValues(void){
	//set initial parameter deviation values (standard deviations)
	//parameters deviated by random variables with these standard deviations
	// these are now constants defined in AllozymeClineGlobals.h
	Shape2ParamRec::SetDeviationValues();
	thetaL_ = thetaDev;
	thetaR_ = thetaDev;
	asymptoteL_ = asymptoteDev;
	asymptoteR_ = asymptoteDev;
	}//end Shape6ParamRec::SetDeviationValues



int Shape6ParamRec::SetShape(void){
	//see if shape is legal, and calculate xL, xR, beta1, beta2
	int OK = MeetsParamCriteria();
	if (OK==False) {return OK;}
	OK = LJunction( maxScore);
	if (OK){
		OK = RJunction(maxScore);}
	if (OK){
		OK = MeetsParamCriteria();}
	return OK;
	}	//end ParamRec::SetShape




void	 Shape6ParamRec::SetStartingValues(void){
	// plausible arbitrary values that start the iteration process
	// these are now constants defined in AllozymeClineGlobals.h
	Shape2ParamRec::SetStartingValues();
	thetaL_ = IthetaL;
	thetaR_ = IthetaR;
	asymptoteL_ = IAsymptoteL;
	asymptoteR_ = IAsymptoteR;
	LJunction(maxScore);
	RJunction(maxScore);
	}//end Shape6ParamRec::SetStartingValues

	
	
double Shape6ParamRec::WhybridOverWpure (const double recombinationRate){
	double aveBeta =(betaL() + betaR()) / 2.0;
	double aveBetaToW =aveBeta / w();
	if ((aveBetaToW > 3000.0) || (aveBetaToW == 0.0)) {
			return 0.0;}
		else{
			return  exp(-recombinationRate * ln(aveBetaToW));}
	}//end Shape6ParamRec::WhybridOverWpure


int operator== (Shape6ParamRec& a, Shape6ParamRec& b){
		if(	((Shape2ParamRec) a == (Shape2ParamRec) b) &&
			(a.thetaL()==b.thetaL()) &&			(a.thetaR()==b.thetaR()) &&
			(a.asymptoteL()==b.asymptoteL()) &&	(a.asymptoteR()==b.asymptoteR()) &&
			(a.xL()==b.xL()) &&					(a.xR()==b.xR()) &&
			(a.betaL()==b.betaL()) &&				(a.betaR()==b.betaR())
		    )		 {return 1;}else{ return 0;}
		}//end operator ==


