//Shape7ParamRec.cp


#pragma once
#include <Shape7ParamRec.h>
#include <ClinePopList.h>
#include <FitMultilocusCline.h>


double Shape7ParamRec::bigS (const double recombinationRate){
	double wHP=WhybridOverWpure(recombinationRate);
	if (wHP == 0.0) {
			return 200.0;}	//an arbirtarily high number
		else{
			return -2.0 * ln(wHP);}
	}//end Shape7ParamRec::bigS


Shape7ParamRec* Shape7ParamRec::ChangeDiseqParam (Shape7ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t){
	int OK=True;
	thisOne = 7;
	Shape7ParamRec* paramToChange = new Shape7ParamRec(*this);//calls copy constructor
	do	{
		OK = True;
		//change disequilibrium by random amount
		paramToChange->D_ = ran.NormDev(D(), paramDeviations.D() * t);
		if ((paramToChange->D() < diseqBottom) || (paramToChange->D() > diseqTop)) {
			OK = False;}
		}while (! OK);
	return paramToChange;
	}//	end Shape7ParamRec::ChangeParams



Shape7ParamRec* Shape7ParamRec::ChangeParams(Shape7ParamRec& paramDeviations,
						  Ran3& ran, int& thisOne, const double t, const int annealingShape){
	int OK=True;
	thisOne = ChooseParameter(ran, annealingShape);
	Shape7ParamRec* paramToChange = new Shape7ParamRec(*this);//calls copy constructor
	do	{
		OK = True;
		if (thisOne == 1){//change center point by a random amount
				paramToChange->c_ = ran.NormDev(center(), paramDeviations.center() * t);}
			else if (thisOne == 2){//change width by random amount
				paramToChange->w_ = ran.NormDev(w(), paramDeviations.w() * t);}
			else if (thisOne == 3){//change thetaL by random amount
				paramToChange->thetaL_ = ran.NormDev(thetaL(), paramDeviations.thetaL() * t);}
			else if (thisOne == 4){//change thetaR by random amount
				paramToChange->thetaR_ = ran.NormDev(thetaR(), paramDeviations.thetaR() * t);}
			else if (thisOne == 5){//change asymptoteL by random amount
				paramToChange->asymptoteL_ = ran.NormDev(asymptoteL(),paramDeviations.asymptoteL() * t);}
			else if (thisOne == 6){//change asymptoteR by random amount
				paramToChange->asymptoteR_ = ran.NormDev(asymptoteR(),  paramDeviations.asymptoteR() * t);}
			else if (thisOne == 7){//change disequilibrium by random amount
				paramToChange->D_ = ran.NormDev(D(), paramDeviations.D() * t);}
		OK = paramToChange->SetShape();
		}while (! OK);
	return paramToChange;
	}//	end Shape7ParamRec::ChangeParams



int Shape7ParamRec::ChooseParameter(Ran3& ran, const int annealingShape){ 
	//randomly choose center (1), width (2), thetaL (3), thetaR(4), asymptoteL (5), asymptoteR (6), or disequilibrium (7)
	if (annealingShape){
			return ran.longBetween(1,6);}
		else {
			return ran.longBetween(1,7);}
	}//end Shape7ParamRec::ChooseParameter


void	 Shape7ParamRec::DerivedValues(aList<double>& derivedList, double recombinationRate){
	derivedList.Clear();
	const double r = recombinationRate;
	derivedList.Append(WhybridOverWpure(recombinationRate));
	derivedList.Append(bigS(recombinationRate));
	derivedList.Append(sigma(r));//order dependency; do not change these!
	derivedList.Append(nLocSel(r));
	derivedList.Append(sPerMarkerL(r));
	derivedList.Append(sPerMarkerR(r));
	derivedList.Append(sVsHets(r));
	derivedList.Append(sVsHetsRI(r));
	derivedList.Append(sStarEcotone(r));
	derivedList.Append(sStarDomEcotone(r));
	derivedList.Append(timeL(r));
	derivedList.Append(timeR(r));
	}//end Shape7ParamRec::DerivedValues



int Shape7ParamRec::FindCenterPopulations(ClinePopList& centerPopList, ClinePopList& popLocation){
	return centerPopList.FindCenterPopulations(*this, popLocation);}


int Shape7ParamRec::HasHigherLikelihood(Shape7ParamRec& param, likelihoodType compare, const double ci){//boolean
	//always greater if param.L == 0
	if (compare == cline){
			return ((param.Lcline() == 0.0) ||  (Lcline() > param.Lcline() - ci ));}
		else if (compare ==disequilibrium){
			return ((param.Ldiseq() == 0.0) ||  (Ldiseq() > param.Ldiseq() - ci ));}
		else {// (compare ==total)
			return ((param.Ltotal() == 0.0) ||  (Ltotal() > param.Ltotal() - ci ));}
	}//end Shape7ParamRec::HasHigherLikelihood



int Shape7ParamRec::LJunction (const double score){
	//finds the point where the left and center functions converge
	double x1=center(), x2, xLogP, xTanhP, diff2=0.0,incrementPower=0.0;
	int exitLoop=False, maxedOut=False, junctionFound=False;
	// They might not ever intersect, so 
	//strategy is to iterate down the curves and see where the difference converges.
	//Start at the center and find the difference between curves.
	xLogP = score * (bottomL() + (topR() - bottomL()) * exp(4.0 * (x1 - (center() + asymptoteL())) * sqrt(thetaL()) / w()));
	xTanhP = score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (x1 - center()) / w())) / 2.0);
	double diff1 = xTanhP - xLogP;
	if (diff1 > 0.0) {//continue only if left function is below the center function
		do{//get function values at 2 locations and compare them
			maxedOut = False;
			x2 = x1 - exp(incrementPower * ln(10.0));	//=1*10^incrementPower
			xLogP = score * (bottomL() + (topR() - bottomL()) * exp(4.0 * (x2 - (center() + asymptoteL())) * sqrt(thetaL()) / w()));
			if (abs(2.0 * (x2 - center()) / w()) >= 6000.0){	//maximum double
					xTanhP = score * (bottomL() + (topR() - bottomL()) * 0.5);
					maxedOut = True;}
				else{
					xTanhP = score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (x2 - center()) / w())) / 2.0);}
			diff2 = xTanhP - xLogP;
			if (diff2 <= 0.0){//convergence point surrounded; check for convergence in middle
					incrementPower--;}
				else{//   if  diff2 <= diff1 then	probably converging to a point
					//move to the left one step
					diff1 = diff2;
					if (! maxedOut){
							x1 = x2;}
						else{
							incrementPower--;}
					}//	end if diff2...else...
			if (incrementPower < -3.0){//close enough
					exitLoop = True;
					if (x2 <= leftEnd){
							xL_ = leftEnd;
							betaL_ = score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (xL() - center()) / w())) / 2.0);}
							else{
							xL_=x2;
							betaL_ = (xLogP + xTanhP) / 2.0;}//use the average
					junctionFound = True;
					}//end if incrementPower
			}while (!exitLoop);
		}////end if diff1>0
	return junctionFound;
	}//	end Shape7ParamRec::LJunction




int Shape7ParamRec:: MeetsParamCriteria (void){
	int OK = True;
	if ((xL() < leftEnd) || (xL() > rightEnd)) {
			OK = False;}
		else if ((xR() < leftEnd) || (xR() > rightEnd)) {
			OK = False;}
		else if ((asymptoteL() < asymptoteBottom) || (asymptoteL() > asymptoteTop)) {
			OK = False;}
		else if ((asymptoteR() < asymptoteBottom) || (asymptoteR() > asymptoteTop)) {
			OK = False;}
		else if ((thetaR() < thetaBottom) || (thetaR() > thetaTop)) {
			OK = False;}
		else if ((thetaL() < thetaBottom) || (thetaL() > thetaTop)) {
			OK = False;}
		else if ((D() < diseqBottom) || (D() > diseqTop)) {
			OK = False;}
	return OK;
	}//end  Shape7ParamRec:: MeetsParamCriteria



double Shape7ParamRec:: nLocSel (const double recombinationRate){//number of loci under selection
	double sVH=sVsHets(recombinationRate);
	double wHP=WhybridOverWpure(recombinationRate);
	if (sVH == 0.0){
			return 0.0;}		//no loci under selection if no selection
		else if ((sVH >= 1.0) || (wHP <= 0.0)) {
			return 1.0e20;}		//a really high number if selection=0
		else{
			return 2.0 * ln(wHP) / ln(1.0 - sVH);}
	}//end  Shape7ParamRec:: nLocSel



double Shape7ParamRec::PonCline(const double x, const double score){
	//returns the value of p on the cline itself from the cline shape
	if (x < xL()) {
			return score * (bottomL() + (topR() - bottomL()) * exp(4.0 * (x - (center() + asymptoteL())) * sqrt(thetaL()) / w()));
			}
		else if (x > xR()) {
			return score * (topR() - (topR() - bottomL()) * exp(-4.0 * (x - (center() - asymptoteR())) * sqrt(thetaR()) / w()));
			}
		else{
			return  (score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (x - center()) / w())) / 2.0));
			}
	}// end Shape7ParamRec::PonCline




void Shape7ParamRec::PrintDerivedWithCI(Shape7ParamRec& lowerCI, 
								   Shape7ParamRec& upperCI, 
								   aList<double>& bestDerived,
								   aList<double>& lowerDerivedCI, 
								   aList<double>& upperDerivedCI, 
								   ostream& outFile){//called by the best total parameter set only
		outFile<<endl;
		outFile<<"total L max: ";outFile.precision(25);outFile<< Lcline()<<endl;
		outFile<<"sigma: ";outFile.precision(10);outFile<<bestDerived[2]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[2]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[2]<<")";
					      outFile<<endl;
		outFile<<"timeL: ";outFile.precision(3);outFile<<bestDerived[10]<<"   (";
					      outFile.precision(3);outFile<<lowerDerivedCI[10]<<" - ";
					      outFile.precision(3);outFile<<upperDerivedCI[10]<<")";
					      outFile<<endl;
		outFile<<"timeR: ";outFile.precision(3);outFile<<bestDerived[11]<<"   (";
					      outFile.precision(3);outFile<<lowerDerivedCI[11]<<" - ";
					      outFile.precision(3);outFile<<upperDerivedCI[11]<<")";
					      outFile<<endl;
		outFile<<"s* vs. hets: ";outFile.precision(10);outFile<<bestDerived[6]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[6]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[6]<<")";
					      outFile<<endl;
		outFile<<"s* ecotone: ";outFile.precision(10);outFile<<bestDerived[8]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[8]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[8]<<")";
					      outFile<<endl;
		outFile<<"s* ecotone w/ dominance: ";outFile.precision(10);outFile<<bestDerived[9]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[9]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[9]<<")";
					      outFile<<endl;
		outFile<<"num. Loci selected: ";outFile.precision(3);outFile<<bestDerived[3]<<"   (";
					      outFile.precision(3);outFile<<lowerDerivedCI[3]<<" - ";
					      outFile.precision(3);outFile<<upperDerivedCI[3]<<")";
					      outFile<<endl;
		outFile<<"sel. per marker L: ";outFile.precision(10);outFile<<bestDerived[4]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[4]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[4]<<")";
					      outFile<<endl;
		outFile<<"sel. per marker R: ";outFile.precision(10);outFile<<bestDerived[5]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[5]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[5]<<")";
					      outFile<<endl;
		outFile<<"sel. per genetic isolation locus: ";outFile.precision(10);outFile<<bestDerived[7]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[7]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[7]<<")";
					      outFile<<endl;

		outFile<<endl;
		}//end Shape7ParamRec::PrintDerivedWithCI



void Shape7ParamRec::PrintDiseqWithCI(const double successRate, 
							Shape7ParamRec& lowerCI, 
							Shape7ParamRec& upperCI, 
							ostream& outFile){//called by the best parameter set only		
		outFile<<endl;
		outFile<<"diseq L max: ";outFile.precision(25);outFile<< Ldiseq()<<endl;
		outFile<<"D: ";outFile.precision(10);outFile<<D()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.D()<<" - ";
					      outFile.precision(10);outFile<<upperCI.D()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate<<endl;
	}//end Shape7ParamRec::PrintDiseqWithCI



void Shape7ParamRec::PrintShapeWithCI(const long allTries, 
							const double overallSuccessRate, 
							aList<double>& successRate,
							Shape7ParamRec& lowerCI, 
							Shape7ParamRec& upperCI, 
							aList<double>& bestDerived, 
							aList<double>& lowerDerivedCI, 
							aList<double>& upperDerivedCI, 
							ostream& outFile){//called by the best parameter set only
		outFile<<endl;
		outFile<<"cline L max: ";outFile.precision(25);outFile<< Lcline()<<endl;
		outFile<<"total replicates tried: "<<allTries<<endl;
		outFile<<"overall acceptance rate: "; outFile.precision(5);outFile<<overallSuccessRate<<endl;

		outFile<<"center: ";outFile.precision(10);outFile<<center()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.center()<<" - ";
					      outFile.precision(10);outFile<<upperCI.center()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[0]<<endl;
		outFile<<"width: ";outFile.precision(10);outFile<<w()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.w()<<" - ";
					      outFile.precision(10);outFile<<upperCI.w()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[1]<<endl;
		outFile<<"max slope: ";outFile.precision(10);outFile<<1.0/w()<<"   (";
					      outFile.precision(10);outFile<<1.0/lowerCI.w()<<" - ";
					      outFile.precision(10);outFile<<1.0/upperCI.w()<<")";
					      outFile<<endl;
		outFile<<"bottom L: ";outFile.precision(10);outFile<<bottomL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.bottomL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.bottomL()<<")";
					      outFile<<endl;
		outFile<<"top R: ";outFile.precision(10);outFile<<topR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.topR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.topR()<<")";
					      outFile<<endl;
		outFile<<"theta L: ";outFile.precision(10);outFile<<thetaL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.thetaL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.thetaL()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[2]<<endl;
		outFile<<"theta R: ";outFile.precision(10);outFile<<thetaR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.thetaR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.thetaR()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[3]<<endl;
		outFile<<"asymptote L: ";outFile.precision(10);outFile<<asymptoteL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.asymptoteL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.asymptoteL()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[4]<<endl;
		outFile<<"asymptote R: ";outFile.precision(10);outFile<<asymptoteR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.asymptoteR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.asymptoteR()<<")   rate : ";
					      outFile.precision(5);outFile<<successRate[5]<<endl;
		outFile<<"xL: ";outFile.precision(10);outFile<<xL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.xL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.xL()<<")";
					      outFile<<endl;
		outFile<<"xR: ";outFile.precision(10);outFile<<xR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.xR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.xR()<<")";
					      outFile<<endl;
		outFile<<"beta L: ";outFile.precision(10);outFile<<betaL()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.betaL()<<" - ";
					      outFile.precision(10);outFile<<upperCI.betaL()<<")";
					      outFile<<endl;
		outFile<<"beta R: ";outFile.precision(10);outFile<<betaR()<<"   (";
					      outFile.precision(10);outFile<<lowerCI.betaR()<<" - ";
					      outFile.precision(10);outFile<<upperCI.betaR()<<")";
					      outFile<<endl;
		outFile<<"Wh/Wp: ";outFile.precision(10);outFile<<bestDerived[0]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[0]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[0]<<")";
					      outFile<<endl;
		outFile<<"S: ";outFile.precision(10);outFile<<bestDerived[1]<<"   (";
					      outFile.precision(10);outFile<<lowerDerivedCI[1]<<" - ";
					      outFile.precision(10);outFile<<upperDerivedCI[1]<<")";
					      outFile<<endl;
	}//end  Shape7ParamRec::PrintShapeWithCI





void Shape7ParamRec::PrintToScreen (const long tries, const double recombinationRate){
	double r = recombinationRate;
	cout.width(5);cout<<tries<<"   cline Lmax: ";
	cout.precision(6);cout<< Lcline()<<endl;
	cout<<"         c: ";cout.precision(3); cout<<center();
	cout<<"         w: ";cout.precision(3); cout<<w();
	cout<<endl;
	cout<<"   bottomL: ";cout.precision(6); cout<< bottomL();
	cout<<"      topR: ";cout.precision(6); cout<< topR();
	cout<<endl;
	cout<<"      thL: ";cout.precision(6); cout<<thetaL();
	cout<<"     thR: ";cout.precision(6); cout<<thetaR();
	cout<<endl;
	cout<<"  asymptoteL: ";cout.precision(5); cout<<asymptoteL();
	cout<<"  asymptoteR: ";cout.precision(5); cout<< asymptoteR();
	cout<<endl;
	cout<<"       xL: ";cout.precision(6); cout<<xL();
	cout<<"      xR: ";cout.precision(6); cout<<xR();
	cout<<endl;
	cout<<"     betaL: ";cout.precision(6); cout<<betaL();
	cout<<"     betaR: ";cout.precision(6); cout<<betaR();
	cout<<endl;
	cout<<"diseq Lmax: "; cout.precision(6); cout<< Ldiseq();
	cout<<"   total Lmax: "; cout.precision(6); cout<< Ltotal()<<endl;
	cout<<"         D: ";cout.precision(6); cout<<D();
	cout<<"     sigma: ";cout.precision(2); cout<< sigma(r);
	cout<<endl;
	cout<<"     hets.s*: ";cout.precision(6); cout<< sVsHets(r);
	cout<<"     eco.s*: ";cout.precision(6); cout<< sStarEcotone(r);
	cout<<endl;
	cout<<"Dom.eco.s*: ";cout.precision(6); cout<< sStarDomEcotone(r);
	cout<<endl;
	cout<<"     timeL: ";cout.precision(1); cout<< timeL(r);
	cout<<"     timeR: ";cout.precision(1); cout<< timeR(r);
	cout<<endl;
	cout<<"     Wh/Wp: ";cout.precision(6); cout<< WhybridOverWpure(r);
	cout<<"     S: ";cout.precision(6); cout<< bigS(r);
	cout<<"         n: ";cout.precision(1); cout<< nLocSel(r);
	cout<<endl;
	cout<<"       s L: ";cout.precision(6); cout<< sPerMarkerL(r);
	cout<<"       s R: ";cout.precision(6); cout<< sPerMarkerR(r);
	cout<<endl;
	cout<<"  sel. per genetic isolation locus: ";cout.precision(6); cout<< sVsHetsRI(r);
	cout<<endl<<endl;
	}//end  Shape7ParamRec::PrintToScreen


int Shape7ParamRec::RJunction (const double score){
	//finds the point where the right and center functions converge
	double x1=center(), x2, xLogP, xTanhP, diff2, incrementPower=0.0;
	int exitLoop=False, maxedOut, junctionFound=False;
	//They might not ever intersect, so 
	//strategy is to iterate down the curves and see where the difference converges.
	//Start at the center and find the difference between curves.
	xLogP = score * (topR() - (topR() - bottomL()) * exp(-4.0 * (x1 - (center() - asymptoteR())) * sqrt(thetaR()) / w()));
	xTanhP = score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (x1 - (center())) / w())) / 2.0);
	double diff1 = xTanhP - xLogP;
	if (diff1 < 0.0){//continue only if right function is below the center function
		do{//get function values at 2 locations and compare them
			maxedOut = False;
			x2 = x1 + exp(incrementPower * ln(10.0));//=1*10^incrementPower
			xLogP = score * (topR() - (topR() - bottomL()) * exp(-4.0 * (x2 - (center() - asymptoteR())) * sqrt(thetaR()) / w()));
			if (abs(2.0 * (x2 - center()) / w()) >= 3200){//maximum extended
					xTanhP = score * (bottomL() + (topR() - bottomL()) * 0.5);
					maxedOut = True;}
				else{
					xTanhP = score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (x2 - center()) / w())) / 2.0);}
			diff2 = xTanhP - xLogP;
			if (diff2 >= 0.0){//convergence point surrounded; check for convergence in middle
					incrementPower--;}
				else {//if diff2 >= diff1 then probably converging to a point
					//move to the left one step
					diff1 = diff2;
					if (! maxedOut){
							x1 = x2;}
						else{
							incrementPower--;}
					}//end if diff2...else...
			if (incrementPower < -3.0) {//close enough
					exitLoop = True;
					if (x2 >= rightEnd){
							xR_ = rightEnd;
							betaR_ = score * (bottomL() + (topR() - bottomL()) * (1.0 + Tanh(2.0 * (xR() - center()) / w())) / 2.0);}
						else{
							xR_ = x2;
							betaR_ = (xLogP + xTanhP) / 2.0;}
					betaR_ /= score;
					junctionFound = True;}
			}while (!exitLoop);
		}//end if diff1<0
	return junctionFound;
	}//end Shape7ParamRec::RJunction



void Shape7ParamRec::SetBarrierStrength(void){
	double dPLdXatxL=0.0, dPRdXatxR=0.0;
	double rVal = PonCline(xR(), maxScore);
	double lVal = PonCline(xL(), maxScore);
	double deltaP = abs(rVal- lVal);

	if (4.0 * sqrt(thetaL()) * (xL() - (center() + asymptoteL())) / w() <= -100.0) {
			dPLdXatxL = exp(-100.0);}	//derivatives found using Mathematica
		else if (4.0 * sqrt(thetaL()) * (xL() - (center() +asymptoteL())) / w() <= 100.0) {
			dPLdXatxL = 4.0 * exp(4.0 * sqrt(thetaL()) * (xL() - (center() + asymptoteL())) / w()) * sqrt(thetaL()) / w();}
		else	{
			dPLdXatxL = exp(100.0);}

	if ((4.0 * sqrt(thetaR()) * (xR() - (center() - asymptoteR())) / w()) <= -100.0) {
			dPRdXatxR = exp(100.0);}
		else if ((4.0 * sqrt(thetaR()) * (xR() - (center() - asymptoteR())) / w()) <= 100.0) {
			dPRdXatxR = 4.0 * sqrt(thetaR()) / (w() * exp(4.0 * sqrt(thetaR()) * (xR() - (center() - asymptoteR())) / w()));}
		else{
			dPRdXatxR = exp(-100.0);}

	if (dPLdXatxL == 0.0) {
			betaL_ = exp(100.0);}
		else{
			betaL_ = deltaP / dPLdXatxL;}
	if (dPRdXatxR == 0.0){
			betaR_ = exp(100.0);}
		else{
			betaR_ = deltaP / dPRdXatxR;}
	}	//end Shape7ParamRec::SetBarrierStrength



void Shape7ParamRec::SetClineLikelihoodRatios(double LmaxCline){
	LtoLmaxCline_ = exp(Lcline() - LmaxCline);//this form gives the ratio of the true likelihoods
	LmaxMinusLcline_ = LmaxCline - Lcline();
	}//end  Shape7ParamRec::SetClineLikelihoodRatios(double)


void	 Shape7ParamRec::SetDeviationValues(void){
	//set initial parameter deviation values (standard deviations)
	//parameters deviated by random variables with these standard deviations
	// these are now constants defined in AllozymeClineGlobals.h
	c_ = centerDev;
	w_ = wDev;
	thetaL_ = thetaDev;
	thetaR_ = thetaDev;
	asymptoteL_ = asymptoteDev;
	asymptoteR_ = asymptoteDev;
	D_ = diseqDev;
	}//end Shape7ParamRec::SetDeviationValues


void Shape7ParamRec::SetDiseqLikelihoodRatios(const double LmaxDiseq){
	LtoLmaxDiseq_ = exp(Ldiseq() - LmaxDiseq);//this form gives the ratio of the true likelihoods
	LmaxMinusLDiseq_ = LmaxDiseq - Ldiseq();
	}//end  Shape7ParamRec::SetDiseqLikelihoodRatios(double)


int Shape7ParamRec::SetShape(void){
	//see if shape is legal, and calculate xL, xR, beta1, beta2
	int OK = MeetsParamCriteria();
	if (OK==False) {return OK;}
	OK = LJunction( maxScore);
	if (OK){
		OK = RJunction(maxScore);}
	if (OK){
		OK = MeetsParamCriteria();}
	return OK;
	}	//end Shape7ParamRec::SetShape




void	 Shape7ParamRec::SetStartingValues(void){
	// plausible arbitrary values that start the iteration process
	// these are now constants defined in AllozymeClineGlobals.h
	c_ = Icenter;
	w_ = Iw;
	thetaL_ = IthetaL;
	thetaR_ = IthetaR;
	asymptoteL_ = IAsymptoteL;
	asymptoteR_ = IAsymptoteR;
	D_ = Idiseq;
	LJunction(maxScore);
	RJunction(maxScore);
	}//end Shape7ParamRec::SetStartingValues


void Shape7ParamRec::SetTotal(void){
	Ltotal_ = Lcline() + Ldiseq();}


void Shape7ParamRec::SetTotalLikelihoodRatios(const double LmaxTotal){
	LtoLmaxTotal_ = exp(Ltotal() - LmaxTotal);//this form gives the ratio of the true likelihoods
	LmaxMinusLTotal_ = LmaxTotal - Ltotal();
	}//end  Shape7ParamRec::SetTotalLikelihoodRatios(double)


double Shape7ParamRec::sVsHetsRI (const double recombinationRate){
	double nL = nLocSel(recombinationRate);
	if (nL == 0.0){
			return 0.0;}
		else{
			return bigS(recombinationRate) / nL;}
	}//Shape7ParamRec::sVsHetsRI

	
	
double Shape7ParamRec::timeL (const double recombinationRate){
	double s = sigma(recombinationRate);
	if ((s == 0.0) || (betaL() / s > 1.0e40)) {
			return 1.0e20;}	//basically forever if no dispersal
		else{
			return sqr(betaL() / s);
			}
	}//end Shape7ParamRec::timeL



 double Shape7ParamRec::timeR (const double recombinationRate){
	double s = sigma(recombinationRate);
	if ((s == 0.0) || (betaR() / s > 1.0e40)) {
			return 1.0e20;}	//basically forever if no dispersal
		else{
			return sqr(betaR() / s);
			}
	}//end  Shape7ParamRec::timeR



double Shape7ParamRec::WhybridOverWpure (const double recombinationRate){
	double aveBeta =(betaL() + betaR()) / 2.0;
	double aveBetaToW =aveBeta / w();
	if ((aveBetaToW > 3000.0) || (aveBetaToW == 0.0)) {
			return 0.0;}
		else{
			return  exp(-recombinationRate * ln(aveBetaToW));}
	}//end Shape7ParamRec::WhybridOverWpure




int operator== (Shape7ParamRec& a, Shape7ParamRec& b){
	if(	(a.center()==b.center())&&			
		(a.w()==b.w())	&&
		(a.D()==b.D())  &&
		(a.thetaL()==b.thetaL()) &&			
		(a.thetaR()==b.thetaR()) &&
		(a.asymptoteL()==b.asymptoteL()) &&	
		(a.asymptoteR()==b.asymptoteR()) &&
		(a.xL()==b.xL()) &&					
		(a.xR()==b.xR()) &&
		(a.betaL()==b.betaL()) &&				
		(a.betaR()==b.betaR())
	    )		 {return 1;}else{ return 0;}
	}//end operator ==



