/*******************************************************
*
*	GridCell.cpp
*
*
*	This class defines habitat types and keeps a list of populations
*	of different species.

*******************************************************/
#ifndef GridCellcp
#define GridCellcp = 1



#include "GridCell.h"
#include "Population.h"
#include "LandScape.h"
#include "CoordinateClass.cpp"
#include "Element.cpp"
#include "aList.cpp"
#include "RandomList.h"



GridCell::~GridCell(void) {
  			pops_.Clear();
  			region_=NULL;
  			neighbors_.Detach();
  			}//end GridCell::~GridCel

GridCell* GridCell::Duplicate(LandScape* r, CoordinateType coords, int habitat){
	//virtual constructor
	return NEW GridCell(r,coords,habitat);}
	
	
GridCell* GridCell::Duplicate(LandScape* r, CoordinateType coords){
		//virtual constructor
	return NEW GridCell(r,coords);}


GridCell* GridCell::Duplicate(void){
		//virtual copy constructor
	GridCell* g = NEW GridCell(*this);
	g->neighbors_ = neighbors_;
	g->region_ = region_;
	return g;}



//void GridCell::AdultDispersal(int species, Ran3List& ran, 
//										double m, double kurtosisFactor){
//			pops_[species].AdultDispersal(ran, m, kurtosisFactor);}

void GridCell::AdultDispersal(int species, Ran3List& ran,aList<long double>& dispersalInfo){
			pops_[species].AdultDispersal(ran, dispersalInfo);}

long GridCell::adults(int species){
			return pops_[species].adults();}
			
			
void GridCell::AgeOneStep(int species, int maxage){
			pops_[species].AgeOneStep(maxage);}


void GridCell::Append(Population* p) {
				pops_.Append(p);}


void GridCell::AssignMates (int species, Ran3List& ran,aList<long double>& reproductionInfo){
			pops_[species].AssignMates(ran,reproductionInfo);}


long GridCell::babies(int species){
			return pops_[species].babies();}


void GridCell::BuryTheDead(int species){
			pops_[species].BuryTheDead();}
			
			
long GridCell::carryingCapacity(int species){
			return pops_[species].carryingCapacity();}
			
			
void GridCell::ClearOffspringLists(int species){
			pops_[species].ClearOffspringLists();}


//void GridCell::Dispersal(int species, Ran3List& ran, double m, double kurtosisFactor){
//			pops_[species].Dispersal(ran, m, kurtosisFactor);}


void GridCell::Dispersal(int species, Ran3List& ran, aList<long double>& dispersalInfo){
			pops_[species].Dispersal(ran, dispersalInfo);}


void	 GridCell::DropNeighbor(GridCell* oldNeighbor){
			if (neighbors_.Includes(oldNeighbor)){
				long i=neighbors_.Loc(oldNeighbor);
				neighbors_.Replace(NULL,i);}
			}//end GridCell::DropNeighbor


long double GridCell::Fecundity(int species, int age){
			if(region_!=NULL) return region_->Fecundity(species,age);
			return zero;}


void GridCell::GetAgeDistribution(int species, aList<int>& ageDist){
			pops_[species].GetAgeDistribution(ageDist);}


void GridCell::InitializeANewGeneration(int species){
			pops_[species].InitializeANewGeneration();}


long GridCell::dead(int species){
			return pops_[species].dead();}


int GridCell::extinct(int species){
			return pops_[species].extinct();}



void GridCell::Extract (int species, Individual* indiv){
			pops_[species].Extract(indiv);}


void GridCell::GameteDispersal(int species, Ran3List& ran,aList<long double>& dispersalInfo){
			pops_[species].GameteDispersal(ran, dispersalInfo);}


long double GridCell::gridScale(void){
			if(region_!=NULL) return region_->gridScale();
			return -one;}


void GridCell::ImmatureDispersal(int species, Ran3List& ran,aList<long double>& dispersalInfo){
			pops_[species].ImmatureDispersal(ran, dispersalInfo);}


long GridCell::immigrants (int species){//number of immigrants
			return pops_[species].immigrants();}


void GridCell::Install (int species, Individual* indiv){
			pops_[species].Install(indiv);}

long double GridCell::intrinsicSexRatio(int species){
			return region_->intrinsicSexRatio(species);}



void GridCell::IPopulation (int species, CoordinateType loc, long k, Ran3List& ran){
			pops_[species].IPopulation(loc,k,ran);}



int GridCell::IsAllOneSex(int species){
			return pops_[species].IsAllOneSex();}


migrationModelSet GridCell::migrationModel(void) {
			if(region_!=NULL) return region_->migrationModel();
			return (migrationModelSet)0;}


long double GridCell::MigrationProb(int species, int ageClass, int currentHabitat, int potentialHabitat){
			if(region_!=NULL) return region_->MigrationProb(species,ageClass,currentHabitat,potentialHabitat);
			return zero;}
	


long double GridCell::moveRate(int species, genderSet sex, int age){
			if(region_!=NULL) return region_->moveRate(species,habitat(),sex,age);
			return zero;}

GridCell* GridCell::neighbor(int direction){//can return null pointer
			return &(neighbors_[direction]);}


Population* GridCell::neighbor(int species, int direction){
	//can return null pointer
  			GridCell* g=neighbor(direction);
  			if(g==NULL){return NULL;}
  			return &(g->pop(species));}


aList<int>* GridCell::NeighboringHabitats(void){
	//delete this list elsewhere
	aList<int>* habitatList = NEW aList<int>();
	int n;
	for(n=0;n<neighbors_.len();++n){
		GridCell& g = neighbors_[n];
		if (&g==NULL){
				habitatList->Append(-1);}
			else{
				habitatList->Append(g.habitat());}
		}//end for n
	return habitatList;
	}//end GridCell::NeighboringHabitats


int	 GridCell::NoInconsistentSettings(int species){
			//returns an error code if settings are inconsistent
			return pops_[species].NoInconsistentSettings();}


Population& GridCell::pop(int species) {
			return pops_[species];}



void GridCell::PrintAgeDistribution(ostream& outFile, int species, int maxage){
			pops_[species].PrintAgeDistribution(outFile, maxage);}


void GridCell::PrintPopHeading(ostream& outFile, int species){
			pops_[species].PrintPopHeading(outFile);}


void GridCell::Recruit(int species, Ran3List& ran){
			pops_[species].Recruit(ran);}


void GridCell::Recruit(int species, long nthBaby){
			pops_[species].Recruit(nthBaby);}


LandScape* GridCell::region(void){
			return region_;}
  	
  	
void GridCell::ReplaceTheDead (int species, Ran3List& ran){
			pops_[species].ReplaceTheDead(ran);}


void GridCell::Reproduce (int species, long& idCount, Ran3List& ran, aList<long double>& specialInfo){
			pops_[species].Reproduce(idCount, ran, specialInfo);}


void GridCell::ReproduceBySyngamy (int species, long& idCount, Ran3List& ran, aList<long double>& specialInfo){
			pops_[species].ReproduceBySyngamy(idCount, ran, specialInfo);}


void GridCell::SetID (int species, int IDnum){
			pops_[species].SetID(IDnum);}


void	 GridCell::SetNeighbor(GridCell* newNeighbor){
			if ((newNeighbor==NULL)||(!(neighbors_.Includes(newNeighbor)))){
				neighbors_.Append(newNeighbor);}
			}//end GridCell::SetNeighbor(GridCell*)


void	 GridCell::SetNeighbor(GridCell* newNeighbor, long direction){
			neighbors_.Replace(newNeighbor, direction);
			}//end GridCell::SetNeighbor(GridCell*, long)


void GridCell::SetReproductiveStatus(int species){
			pops_[species].SetReproductiveStatus();}


long GridCell::size(int species){
			return pops_[species].size();}


void GridCell::SomeAdultsDie (int species, aList<long double>& fSurvivalSchedule,
						aList<long double>& mSurvivalSchedule,Ran3List& ran){
			pops_[species].SomeAdultsDie(fSurvivalSchedule,
												mSurvivalSchedule,ran);}

void GridCell::SomeImmaturesDie (int species, aList<long double>& fSurvivalSchedule,
						aList<long double>& mSurvivalSchedule,Ran3List& ran){
			pops_[species].SomeImmaturesDie(fSurvivalSchedule,
												mSurvivalSchedule,ran);}

void GridCell::SomeGametesDie (int species, aList<long double>& fSurvivalSchedule,
						aList<long double>& mSurvivalSchedule,Ran3List& ran){
			pops_[species].SomeGametesDie(fSurvivalSchedule,
												mSurvivalSchedule,ran);}

long GridCell::time(void){
			if(region_!=NULL) return region_->time();
			return 0;}


void GridCell::ViabilitySelection(int species, Ran3& ran){
			pops_[species].ViabilitySelection(ran);}


Population& GridCell::operator[] (int p) {
			return pop(p);}






#endif


