/***************************************************
*
*	DioeciousGeneticIndividual.cpp
*
*	Handles processes involving sexually reproducing genetic individuals within populations
*	Genotypes are held in a container class.
*	All are ideal hermaphrodites by default.
*
****************************************************/

#include "DioeciousGeneticIndividual.h"
#include "LandScape.h"
#include "Element.cpp"
#include "aList.cpp"
	
	
//CONSTRUCTORS AND DESTRUCTORS
DioeciousGeneticIndividual::DioeciousGeneticIndividual(Phenotype& newGenes, 
				genderSet gender, long id, Population* site):
		 	SexualGeneticIndividual(newGenes,id,site) {
	sex_=newGenes.sex(gender);	
	}


Individual* DioeciousGeneticIndividual::Duplicate (long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	return  new DioeciousGeneticIndividual(id, site);
	}//end  DioeciousGeneticIndividual::Duplicate


GeneticIndividual* DioeciousGeneticIndividual::Duplicate (Phenotype& newGenes, long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	return  new DioeciousGeneticIndividual(newGenes, sex(), id, site);
	}//end  DioeciousGeneticIndividual::Duplicate


void DioeciousGeneticIndividual::Copy (SexualGeneticIndividual* it){
	//assumes 'it' is SexualGeneticIndividual
	DioeciousGeneticIndividual* it1 = (DioeciousGeneticIndividual*) it;
	DioeciousGeneticIndividual::Copy(it1);
	}//end DioeciousGeneticIndividual::Copy

void DioeciousGeneticIndividual::Copy (GeneticIndividual* it){
	//assumes 'it' is SexualGeneticIndividual
	DioeciousGeneticIndividual* it1 = (DioeciousGeneticIndividual*) it;
	DioeciousGeneticIndividual::Copy(it1);
	}//end DioeciousGeneticIndividual::Copy

void DioeciousGeneticIndividual::Copy (Individual* it){
	//assumes 'it' is SexualGeneticIndividual
	DioeciousGeneticIndividual* it1 = (DioeciousGeneticIndividual*) it;
	DioeciousGeneticIndividual::Copy(it1);
	}//end DioeciousGeneticIndividual::Copy


void DioeciousGeneticIndividual::Copy (DioeciousGeneticIndividual* it){
	SexualGeneticIndividual::Copy((SexualGeneticIndividual*) it);
	}//end DioeciousGeneticIndividual::Copy


/*

//Could override to check for mates in population
int DioeciousGeneticIndividual::CanMate (int maxThisAgeClass){
	//override for mate-choice subclasses
	return ((alive())&&(lifeStage_==adult)&&
				(maxThisAgeClass>=matingsThisAgeClass_));
	}//end DioeciousGeneticIndividual::CanMate
*/
	
void DioeciousGeneticIndividual::Clone (long& idCount)	{
	//installs a new baby in the population
	DioeciousGeneticIndividual* baby = new DioeciousGeneticIndividual(++idCount);
	baby->age_ = 1;
	baby->alive_ = 1;
	baby->lifeStage_ = zygote;
	baby->moveRate_ = moveRate();
	baby->genes_ = genes_->Duplicate();
	baby->sex_ = baby->genes_->sex(sex());
	baby->matings_=0; 
	baby->matingsThisAgeClass_=0;
	pop_->Install(baby);
	}//end DioeciousGeneticIndividual::Clone
	


void DioeciousGeneticIndividual::MakeABaby (long& idCount, Ran3List& ran,double mutationRate, double mutationStdev){
	//OVERRIDE IN SUBCLASSES FOR DIFFERENT MATING SYSTEMS
	//THIS IMPLEMENTS SEXUAL REPRODUCTION WITH COMPLETE LAST-MATE SPERM PRECEDENCE
	//does nothing if no sperm available
	//installs a new baby in the population
	if (mates()){
		DioeciousGeneticIndividual* baby = new DioeciousGeneticIndividual(++idCount);
		baby->age_ = 1;
		baby->alive_ = 1;
		baby->lifeStage_ = zygote;
		baby->genes_ = genes_->Duplicate();
		//get the parents genotypes and send to Genotype for meiosis & fertilization	
		CoreGenome& momGenes = GetGenotype();
		CoreGenome& dadGenes = GetMate(ran);
		CoreGenome* g = (CoreGenome*) baby->genes_;
		g->SetSexualZygote(&momGenes, &dadGenes, ran, mutationRate, mutationStdev);
		baby->sex_ = baby->genes_->sex((ran.next()<intrinsicSexRatio()) ? F:M);//genetic sex determination koverrides random number
		if (baby->sex()==F){baby->moveRate(moveRate());}
				else {baby->moveRate(moveRate());}
		baby->SetPhenotype();
		baby->matings_=0; 
		baby->matingsThisAgeClass_=0;
		pop_->Install(baby);
		}// end if mates
	}  //end DioeciousGeneticIndividual::MakeABaby


void DioeciousGeneticIndividual::Reproduce (long& idCount, Ran3List& ran,double mutationRate, double mutationStdev)	{
	//installs new babies in the population
	//each female reproduces
	//according to its individual fecundity
	//derived classes handle special mating systems-- this is the closest to theoretical ideal
	//ALL OFFSPRING HAVE NEWLY CHOSEN FATHERS
	if(sex_==M) return;
	int b,f=Fecundity(ran);
	for (b=0;b<f;++b){
		ClearMates();
		FindMate(ran,1,1);//each offspring has newly chosen father
		MakeABaby(idCount,ran, mutationRate, mutationStdev);}
	}//end DioeciousGeneticIndividual::Reproduce








