/************************************************
* 	GeneticIndividual.cpp
* 
* 
* 
* 	This class handles processes within individuals that have genes
*************************************************/
#ifndef GeneticIndividualcp
#define GeneticIndividualcp = 1

# include "GeneticIndividual.h"
# include "GeneticPopulation.h"
# include "LandScape.h"
#include "Element.cpp"
#include "aList.cpp"
#include "RandomList.h"
  

GeneticIndividual::GeneticIndividual(Phenotype& newGenes, long id, Population* site){
	/*done this way to permit multiple inheritance for derived genetic individuals*/
	id_=id;
	pop_=site;
	genes_ = newGenes.Duplicate();
	SetPhenotype();
		/*cout<<" create GeneticIndividual #"<<id_<<endl;*/
	}/*end constructor GeneticIndividual::GeneticIndividual*/


		
GeneticIndividual::GeneticIndividual(long id, Population* site){
	//must add genes -- called during reproduction
	id_=id;
	pop_=site;
	genes_=NULL;
		//cout<<" create GeneticIndividual #"<<id_<<endl;
	}//end constructor GeneticIndividual::GeneticIndividual
	

GeneticIndividual::GeneticIndividual(void){
	//must add genes -- called during reproduction
	id_=0;
	pop_=NULL;
	genes_=NULL;
		//cout<<" create GeneticIndividual #"<<id_<<endl;
	}//end constructor GeneticIndividual::GeneticIndividual


GeneticIndividual::GeneticIndividual(GeneticIndividual& indiv){
	Individual::Copy((Individual*) &indiv);
//	genes_.Copy(indiv.genes_);
	genes_ = indiv.genes_->Duplicate();
	SetPhenotype();
	}//end constructor GeneticIndividual::GeneticIndividual
							
							
GeneticIndividual::GeneticIndividual(GeneticIndividual* indiv){
	Individual::Copy((Individual*) indiv);
//	genes_.Copy(indiv->genes_);
	genes_ = indiv->genes_->Duplicate();
	SetPhenotype();
	}//end constructor GeneticIndividual::GeneticIndividual


		
GeneticIndividual::~GeneticIndividual(void) {
		delete genes_; genes_ = NULL;
		//cout<<" destroy GeneticIndividual #"<<id_<<endl;
		}//end destructor GeneticIndividual::GeneticIndividual


Individual* GeneticIndividual::Duplicate (void){
	//returns a duplicate, used for virtual copy construction
	return  NEW GeneticIndividual();
	}//end  GeneticIndividual::Duplicate


Individual* GeneticIndividual::Duplicate (long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	return  NEW GeneticIndividual(id, site);
	}//end  GeneticIndividual::Duplicate


GeneticIndividual* GeneticIndividual::Duplicate (Phenotype& newGenes, long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	GeneticIndividual* gi =  NEW GeneticIndividual(id, site);
	delete gi->genes_;gi->genes_=NULL;
	gi->genes_ = newGenes.Duplicate();
	return gi;
	}//end  GeneticIndividual::Duplicate










int	GeneticIndividual::AllelePresent (int loc, aString& alleleVal){
	CoreGenome* g = (CoreGenome*) genes_;
	return g->AllelePresent(loc, alleleVal);
	}


//void GeneticIndividual::Clone (long& idCount, Ran3& ran, double mutationRate, double sigma)	{
void GeneticIndividual::Clone (long& idCount, Ran3List& ran, aList<long double>& specialInfo)	{
	//installs a new baby in the population
	GeneticIndividual* baby = NEW GeneticIndividual(++idCount);
	baby->age_ = 1;
	baby->alive_ = 1;
	baby->lifeStage_ = zygote;
	baby->moveRate(moveRate());
	baby->genes_ = genes_->Duplicate();
//	double  mutationRate = specialInfo[0];
//	double sigma = specialInfo[1];
	baby->genes_->Mutate(ran,specialInfo);
	baby->SetPhenotype();
	pop()->Install( (Individual*) baby);
	}//end GeneticIndividual::Clone
	

//void GeneticIndividual::Clone (Individual* mom, Ran3& ran, double mutationRate, double sigma)	{
void GeneticIndividual::Clone (Individual* mom, Ran3List& ran, aList<long double>& specialInfo)	{
	//copies mom as a new baby
	//obtain all information directly from mom
	Individual::Clone(mom,ran,specialInfo);// copies location etc. details
	//mom must be a GeneticIndividual too
	GeneticIndividual* gMom = (GeneticIndividual*) mom;
	CoreGenome& momGenes = gMom->GetGenotype();
	genes_ = momGenes.Duplicate();
//	double  mutationRate = specialInfo[0];
//	double sigma = specialInfo[1];
	genes_->Mutate(ran,specialInfo);
	SetPhenotype();
	}//end GeneticIndividual::Clone



int	GeneticIndividual::CountAlleles (int loc, aString& alleleVal){
	CoreGenome* g = (CoreGenome*) genes_;
	return g->CountAlleles(loc, alleleVal);
	}//end GeneticIndividual::CountAlleles


long double GeneticIndividual::fitness(void){
	int aliveStatus = alive_;
	long double fitness=zero;
	aList<long double> selParams;
	if(pop_ != NULL){//assumes viability selection independent of setting
		GeneticPopulation* gPop = (GeneticPopulation*) pop_;
		gPop->SetSelectionParams(selParams);
		}
	SetSelectionParams(selParams);
	Ran3 ran(-1);//dummy
	SurvivesViabilitySelection(selParams, ran,fitness);
	alive_=aliveStatus;//restores
	return fitness;
	}//end 	GeneticIndividual::fitness
	


CoreGenome&	GeneticIndividual::GetGenotype (void) {
	Phenotype& ph = *genes_;
	CoreGenome* gn = (CoreGenome*) &ph;
	return *gn;
	}//end GeneticIndividual::GetGenotype



aString* GeneticIndividual::GenotypeString(char* separator){
	CoreGenome* g = (CoreGenome*) genes_;
	return g->GenotypeString(separator);
	}//end GeneticIndividual::GenotypeString
	


aString* GeneticIndividual::GenotypeString(int locus, char* separator){
	CoreGenome* g = (CoreGenome*) genes_;
	return g->GenotypeString(locus, separator);
	}//end GeneticIndividual::GenotypeString
	
	
	
aString* GeneticIndividual::GenotypeString(int locus, int alleleNum, char* separator){
	CoreGenome* g = (CoreGenome*) genes_;
	aString* gstr = g->GenotypeString(locus, alleleNum, separator);
	return gstr;
	}//end GeneticIndividual::GenotypeString



/*void GeneticIndividual::IIIndividual (genderSet gender, lifeStageType lifeStage, 
								double rate, Ran3List& ran, LandScapeInitializer00& init){
	Individual::IIndividual(gender,lifeStage,rate, ran,init);
	}//end GeneticIndividual::IIndividual
*/

void GeneticIndividual::IIIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran, LandScapeInitializer01& init, aList<long double>& extraInfo){
	Individual::IIndividual(gender,lifeStage,rate, ran,init,extraInfo);
//	Individual::IIndividual(gender,lifeStage,rate, ran);
	SetPhenotype();
	}//end GeneticIndividual::IIndividual


int GeneticIndividual::IsHeterozygote (int loc){//returns false=0 if haploid
	CoreGenome* g = (CoreGenome*) genes_;
	return g->IsHeterozygote(loc);
	}//end GeneticIndividual::IsHeterozygote(int,int)
	
	
int GeneticIndividual::IsHeterozygote (int loc, aString& alleleVal){
	//returns false=0 if haploid
	CoreGenome* g = (CoreGenome*) genes_;
	return (g->IsHeterozygote(loc) && g->AllelePresent(loc,alleleVal));
	}//end GeneticIndividual::IsHeterozygote(int,int,int)



int GeneticIndividual::loci(void){
	CoreGenome* g = (CoreGenome*) genes_;
	return g->loci();
	}//end GeneticIndividual::loci()

/*
//void GeneticIndividual::MakeABaby (long& idCount, Ran3List& ran, double mutationRate, double sigma){
void GeneticIndividual::MakeABaby (long& idCount, Ran3List& ran, aList<double>& specialInfo){
	//OVERRIDE IN DERIVED CLASSES FOR DIFFERENT MATING SYSTEMS
	//installs a new baby in the population
//	Clone(idCount, ran, mutationRate, sigma);
	if(sex_==M) return;
	Clone(idCount, ran, specialInfo);
	}  //end GeneticIndividual::MakeABaby
*/


//void GeneticIndividual::Mutate (Ran3& ran, double mutationRate, double sigma){
void GeneticIndividual::Mutate (Ran3& ran, aList<long double>& specialInfo){
	//mutate an allele at loc to a value between minVal & maxVal
	CoreGenome* g = (CoreGenome*) genes_;
//	int locus = (int) ran.longBetween(0,g->loci()-1);
//	g->Mutate(ran,mutationRate,sigma);
	g->Mutate(ran,specialInfo);
	}//end GeneticIndividual::Mutate()

long double GeneticIndividual::phenotype(int i){
	CoreGenome* g = (CoreGenome*) genes_;
	return g->phenotype(i);
	}//end GeneticIndividual::phenotype(int)


int GeneticIndividual::ploidy(int loc){
	CoreGenome* g = (CoreGenome*) genes_;
	return g->ploidy(loc);
	}//end GeneticIndividual::ploidy(int)


void GeneticIndividual::PrintGenes(ostream& outFile, int alleleLen, int codeLen) {
	//create geneStr before calling this function
	aString geneStr;
	CoreGenome* g = (CoreGenome*) genes_;
	g->SetGenotypeString(geneStr, alleleLen);
	aString idStr(id_);
	idStr.PrependCharToLen(' ',codeLen);
	idStr.cat(' ');
	idStr.cat(geneStr);
	outFile<<idStr<<endl;
	}//end GeneticIndividual::PrintGenes

/*
//void GeneticIndividual::Reproduce (long& idCount, Ran3List& ran, double mutationRate, double sigma)	{
void GeneticIndividual::Reproduce (long& idCount, Ran3List& ran, aList<double>& specialInfo)	{
	//installs new babies in the population
	//each hermaphrodite reproduces
	//according to its individual fecundity
	//derived classes handle separate genders, genetics, etc.
	//IDEAL HERMAPHRODITES, SO ALL OFFSPRING HAVE NEWLY CHOSEN FATHERS
	if(sex_==M) return;
	int f=Fecundity(ran);
	for (int b=0;b<f;++b){
//		MakeABaby(idCount,ran,mutationRate,sigma);}
		MakeABaby(idCount,ran,specialInfo);}
	}//end GeneticIndividual::Reproduce
*/


void GeneticIndividual::SetGenotypeString(aString& geneStr,int alleleLen) {
	//create geneStr before calling this function
	CoreGenome* g = (CoreGenome*) genes_;
	g->SetGenotypeString(geneStr, alleleLen);
	SetPhenotype();
	}//end GeneticIndividual::SetGenotypeString


void GeneticIndividual::SetPhenotype(void){
	CoreGenome* g = (CoreGenome*) genes_;
	g->SetPhenotype();
	}//end GeneticIndividual::SetPhenotype


void GeneticIndividual::SetPhenotype(Ran3& ran, aList<long double>& heritabilities){
	CoreGenome* g = (CoreGenome*) genes_;
	g->SetPhenotype(ran, heritabilities);
	}//end GeneticIndividual::SetPhenotype



void GeneticIndividual::SetSelectionParams  (aList<long double>& selParams) {
	/*0=dead; 1=alive; used in natural selection*/
	long double theSex = (long double)sex_;
	selParams.Append(theSex);
	genes_->SetSelectionParams(selParams);
	}/*end GeneticIndividual::SetSelectionParams*/


int GeneticIndividual::SurvivesViabilitySelection  (aList<long double>& selParams, Ran3& ran, long double& fitness) {
	/*0=dead; 1=alive; used in natural selection*/
	if (! alive_) {return 0;}
	alive_=genes_->SurvivesViabilitySelection(selParams, *this, ran, fitness);
	return alive_;
	}/*end GeneticIndividual::SurvivesViabilitySelection*/






#endif

