/************************************************
* 	Individual.cpp
*
* 
* 
* 	This class handles processes within individuals 
*************************************************/


#ifndef Individualcpp
#define Individualcpp = 1

#include "Individual.h"
#include "Population.h"
#include "GridCell.h"
#include "LandScape.h"
#include "StringList.h"
#include "Element.cpp"
#include "aList.cpp"
#include "CoordinateClass.cpp"
#include "GeneContainers.cpp"


//CONSTRUCTORS AND DESTRUCTORS, + INITIALIZERS


Individual::Individual(long id, Population* site){	
	id_=id;//must have one
	age_=0;//not born
	alive_=0;//no
	causeOfDeath_=-1;//irrelevant
	lifeStage_=zygote;//
	moveRate_=0;//can't move
	sex_=A;//asexual
	movedThisGen_=0;
	pop_=site;//assigned here, but not copied
	//	cout<<" create individual #"<<id_<<endl;
	}//end constructor Individual::Individual



Individual::Individual(void){	
	id_=0;//none
	age_=1;//not born
	alive_=1;//yes
	causeOfDeath_=-1;//irrelevant
	lifeStage_=adult;
	moveRate_=0;//can't move
	sex_=A;//asexual
	movedThisGen_=0;
	pop_=NULL;//noplace
	//	cout<<" create individual #"<<id_<<endl;
	}//end constructor Individual::Individual


Individual::Individual(Individual& indiv){
	id_=indiv.id();//all these are duplicated
	age_=indiv.age();
	alive_=indiv.alive();
	causeOfDeath_=indiv.causeOfDeath_;
	lifeStage_=indiv.lifeStage_;
	moveRate_=indiv.moveRate_;
	sex_=indiv.sex();
	movedThisGen_=indiv.movedThisGen_;
	pop_=indiv.pop();//but this is assigned not copied
	}//end copy constructor Individual:Individual(Individual&)


Individual::Individual(Individual* indiv){
	id_=indiv->id();//all these are duplicated
	age_=indiv->age();
	alive_=indiv->alive();
	causeOfDeath_=indiv->causeOfDeath_;
	lifeStage_=indiv->lifeStage_;
	moveRate_=indiv->moveRate_;
	sex_=indiv->sex();
	movedThisGen_=indiv->movedThisGen_;
	pop_=indiv->pop();//but this is assigned not copied
	}//end copy constructor Individual:Individual(Individual&)




		
Individual::~Individual(void) {
		PrepareForBurial();
		pop_=NULL;//de-allocated, but not destroyed
	//	cout<<" destroy individual #"<<id_<<endl;
		}//end destructor Individual::Individual


Individual* Individual::Duplicate (long id, Population* site){
	//a virtual constructor
	return NEW Individual(id, site);
	}//end  Individual::Duplicate


Individual* Individual::Duplicate (void){
	//a virtual constructor
	return NEW Individual();
	}//end  Individual::Duplicate




void Individual::IIndividual (genderSet gender, lifeStageType ageClass, 
								long double rate, Ran3List&)	{
	//called only when starting the whole simulation replicate
	//MakeABaby does initialization within the simulation
	sex_ = gender;
	age_ = 1;
	alive_ = 1;
	lifeStage_ = ageClass;
	moveRate(rate);
	}//end Individual::IIndividual


/*void Individual::IIndividual (genderSet gender, lifeStageType ageClass, 
								double rate, Ran3List& ran, LandScapeInitializer00&)	{
	//called only when starting the whole simulation replicate
	//MakeABaby does initialization within the simulation
	Individual::IIndividual (gender, ageClass, rate, ran);//init does things in derived classes
	}//end Individual::IIndividual
*/

void Individual::IIndividual (genderSet gender, lifeStageType ageClass, 
								long double rate, Ran3List& ran, LandScapeInitializer01& /*init*/, aList<long double>& /*extraInfo*/)	{
	//called only when starting the whole simulation replicate
	//MakeABaby does initialization within the simulation
	Individual::IIndividual (gender, ageClass, rate, ran);//init does things in derived classes
	}//end Individual::IIndividual




//FUNCTIONS


void  Individual::AdultMortality (aList<long double>& survivalSchedule, Ran3List& ran){
	if (alive() ){
		if (ran.next()>=survivalSchedule[age()]){
			alive_ = 0;}
		}
} //end  Individual::AdultMortality



void Individual::Birthday (int maxage)	{
	++age_;
	movedThisGen_ = 0;
	if (age_ > maxage) {alive_ = 0;causeOfDeath_=0;} //death of old age
	//memory will be freed during step ReplaceTheDead
}	//end  Individual::Birthday


void Individual::Clone (Individual* mom, Ran3List& /*ran*/, aList<long double>& /*specialInfo*/)	{
	//copies mom as a new baby
	Copy(mom);
	age_ = 0;
	alive_ = 1;
	causeOfDeath_=-1;
	lifeStage_ = zygote;
	movedThisGen_=0;
	}//end Individual::Clone
	
	
void Individual::Clone (long& idCount, Ran3List& /*ran*/, aList<long double>& /*specialInfo*/)	{
	//installs a new baby in the population
	Individual* baby = NEW Individual(++idCount);
	baby->age_ = 1;
	baby->alive_ = 1;
	baby->lifeStage_ = zygote;
	pop_->Install(baby);
	baby->moveRate(moveRate());
	}//end Individual::Clone
	
	
CoordinateType& Individual::coords(void){
	return pop_->coords();}
	
	
void Individual::Copy (Individual* it)	{
	sex_ = it->sex();
	age_ = it->age();
	alive_ = it->alive();
	causeOfDeath_=it->causeOfDeath();
	lifeStage_ = it->lifeStage();
	pop_=it->pop();
	moveRate_=it->moveRate();
	movedThisGen_=it->movedThisGen();
	}//end Individual::Copy


	
int	Individual::Fecundity(Ran3List& ran){
	//returns number of babies to produce--converted from double
	// override for fecundity selection
	//override for habitat-specific reproduction
	if ((!alive())|| !(lifeStage_==adult)) {return 0;}
	long double rawF = pop_->Fecundity(age());//males have fecundity in derived classes that allow sperm production
	int f = (int) rawF;
	long double rem = rawF - f;
	if (rem==zero){ return f;}
	if (ran()<rem) {++f;}
	return f; 
	}//end Individual::Fecundity



char* Individual::GetGenderChar(void){
	if (sex()==B){ return "B";}
		else if (sex()==F){ return "F";}
		else if (sex()==M){ return "M";}
		else if (sex()==A){ return "A";}
		else {return "?";}//default
	} //end Individual::GetGenderChar


long double Individual::gridScale(void){
	if(pop_ != NULL) return pop_->gridScale();
	return -one;
	}


int	Individual::habitat(void){
	if (pop_==NULL) {return (-1);} 
		else{ return pop_->habitat();}
	}//end Individual::habitat


void  Individual::ImmatureMortality (aList<long double>& survivalSchedule, Ran3List& ran){
	AdultMortality(survivalSchedule,ran);
} //end  Individual::AdultMortality


long double  Individual::intrinsicSexRatio(void){
	if(pop_!=NULL) return pop_->intrinsicSexRatio();
	return -one;}


migrationModelSet Individual::migrationModel(void){
	if(pop_!=NULL) return pop_->migrationModel();
	return (migrationModelSet)0;}


long double Individual::MigrationProb(int currentHabitat, int potentialHabitat){
	if(pop_!=NULL) return pop_->MigrationProb(age(), currentHabitat,potentialHabitat);
	return zero;}



CoordinateType& Individual::MoveRandomly (Ran3List& ran, aList<long double>& dispersalInfo){
	/*the coordinates returned are the final location
	override in a derived class for habitat-determined movement rules
	or rules about consistency of direction each step*/
	if (alive() && !movedThisGen()) {
		if(migrationModel()==diffusion){
				return MoveRandomlyDiffusion(ran,dispersalInfo);
				}
			else if (migrationModel()==steppingStone){
				/*only some individuals move*/
				return MoveRandomlyStepStone(ran,dispersalInfo);
				}
			else if (migrationModel()==island){
				/*only some individuals move*/
				return MoveRandomlyIsland(ran,dispersalInfo);
				}
			else if (migrationModel()==kurtotic){
				/*movement is diffusion with random sigma
				randomly determine a new distance in cells to traverse
				travel in steps in random directions until total cells reached
				lose a step if a habitat(landscape) edge crossing is attempted*/

//				return MoveRandomlyKurtoticDiffusion(ran,kurtosisFactor);
				}
			else {/*error; don't go anywhere*/
					}
		}/*end if alive*/
	return coords();/*wherever it ends up*/
	}/*end Individual::MoveRandomly*/

CoordinateType& Individual::MoveRandomlyDiffusion (Ran3List& ran, aList<long double>& /*dispersalInfo*/){
	/*the coordinates returned are the final location
	override in a derived class for habitat-determined movement rules
	or rules about consistency of direction each step*/

	/*randomly determine a new distance in cells to traverse
	travel in steps in random directions until total cells reached
	lose a step if a habitat(landscape) edge crossing is attempted*/
	long double prob, mean=zero, stDev = zero;
	stDev = moveRate() / gridScale();
	long dist=(long) ran.NormDev(mean, stDev);
	dist=ABS(dist);
	GridCell& currentSite=site();/*current grid cell*/
	GridCell* targetSite= &currentSite;
	int n,potentialHabitat,direction;
	while (dist>0){
		int currentHabitat = targetSite->habitat();
		aList<int>* neighborHabitats = targetSite->NeighboringHabitats();
		aList<long double> migrationProbs;/*probabilities of moving a given direction*/
		for (n=0;n<neighborHabitats->len();++n){
			potentialHabitat = neighborHabitats->Nth(n);
			prob=zero;
			if (potentialHabitat != -1){/*-1 indicates absolute boundary*/
				prob = MigrationProb(currentHabitat, potentialHabitat);}
			migrationProbs.Append(prob);
			}/*end for n*/
		direction=ran.weighted(migrationProbs);
		if (WillEnterHabitat(targetSite->neighbor(direction),dist,ran)){
			targetSite=targetSite->neighbor(direction);
			movedThisGen(1);
			//dist -=1;/*uncomment to avoid skipping a turn for non-move*/
			}/*not-skipping will run forever if no acceptable cell available*/
		dist -=1;/*uncomment to skip a turn for non-move*/
		delete neighborHabitats; neighborHabitats=0;
		}/*end while dist*/
	if(targetSite != &currentSite){
		int s=species();
		currentSite.Extract(s,this);
		targetSite->Install(s,this);
		}//end if
	return coords();/*wherever it ends up*/
	}/*end Individual::MoveRandomlyDiffusion*/





CoordinateType& Individual::MoveRandomlyStepStone (Ran3List& ran, aList<long double>& /*dispersalInfo*/){
	long double mprob,prob;
	/*only some individuals move*/
	mprob = moveRate();
	if(ran()>=mprob) {
		return coords();}//don't migrate
	/*only move to adjacent grid cells
	randomly determine a new direction
	lose a step if a habitat(landscape) edge crossing is attempted*/
	GridCell& currentSite=site();//current grid cell
	GridCell* targetSite=&currentSite;
	int n, potentialHabitat,currentHabitat = targetSite->habitat();
	aList<int>* neighborHabitats = targetSite->NeighboringHabitats();
	aList<double> migrationProbs;/*probabilities of moving a given direction*/
	for (n=0;n<neighborHabitats->len();++n){
		potentialHabitat = neighborHabitats->Nth(n);
		prob=zero;
		if (potentialHabitat != -1){/*-1 indicates absolute boundary*/
			prob = MigrationProb(currentHabitat, potentialHabitat);}
		migrationProbs.Append(prob);
		}/*end for n*/
	int direction=ran.weighted(migrationProbs);
	long dist=1;
	while (dist>0){
		if (WillEnterHabitat(targetSite->neighbor(direction),dist,ran)){
			targetSite=targetSite->neighbor(direction);
			movedThisGen(1);
			//dist -=1;/*uncomment to avoid skipping a turn for non-move*/
			}/*not-skipping will run forever if no acceptable cell available*/
		dist -=1;/*uncomment to skip a turn for non-move*/
		}/*end while dist*/
	if(targetSite != &currentSite){
		int s=species();
		currentSite.Extract(s,this);
		targetSite->Install(s,this);
		}/*end if*/
	delete neighborHabitats;neighborHabitats=0;
	return coords();/*wherever it ends up*/
	}/*end Individual::MoveRandomlyStepStone*/



CoordinateType& Individual::MoveRandomlyIsland (Ran3List& ran, aList<long double>& /*dispersalInfo*/){
	//only some individuals move
	long double prob, mprob = moveRate();
	if(ran()>=mprob) {
		return coords();}//don't migrate
	//neighbors list contains all populations
	GridCell& currentSite=site();//current grid cell
	GridCell* targetSite=&currentSite;
	int r,rr,  c, potentialHabitat, currentHabitat = currentSite.habitat();
	LandScape* myRegion = currentSite.region();
	ObjList<aList<int> >* neighborHabitatsPtr = myRegion->habitatGrid();
	ObjList<aList<int> >& neighborHabitats = *neighborHabitatsPtr;
	aList<long double> migrationProbs;/*probabilities of moving a given direction*/
	for (r=0;r<neighborHabitats.len();++r){
		for (c=0;c<neighborHabitats[r].len();++c){
			potentialHabitat = neighborHabitats[r][c];
			prob=zero;
			if (potentialHabitat != -1){/*-1 indicates absolute boundary*/
				prob = MigrationProb(currentHabitat, potentialHabitat);}
			migrationProbs.Append(prob);
			}}//end for r & c
	int newPopNum=ran.weighted(migrationProbs);
	CoordinateType newCoords;
	for (rr=0;rr<neighborHabitats.len();++rr){
		if (newPopNum >= (rr+1) * neighborHabitats[rr].len()){
				newCoords[0]++;}
			else{
				newCoords[1] = newPopNum - (rr * neighborHabitats[rr].len());
				break;}
		}//end for rr
	
	targetSite = &(currentSite.region()->GetGridCell(newCoords));
	movedThisGen(1);
	if(targetSite != &currentSite){
		int s=species();
		currentSite.Extract(s,this);
		targetSite->Install(s,this);
		}//end if
	delete neighborHabitatsPtr;neighborHabitatsPtr=0;
	return coords();//wherever it ends up
	}//end Individual::MoveRandomlyIsland




CoordinateType& Individual::MoveRandomlyKurtoticDiffusion (Ran3List& ran, long double kurtosisFactor){
	/*movement is diffusion with random sigma
	randomly determine a new distance in cells to traverse
	travel in steps in random directions until total cells reached
	lose a step if a habitat(landscape) edge crossing is attempted*/
	long double mean=zero, stDev=zero, prob;
	stDev = moveRate() / gridScale();
	stDev=ran.NormDev(stDev, kurtosisFactor);
	stDev=abs(stDev);
	long dist=(long) ran.NormDev(mean, stDev);
	dist=abs(dist);
	GridCell& currentSite=site();/*current grid cell*/
	GridCell* targetSite = &currentSite;
	int n,potentialHabitat,direction;
	while (dist>0){
		int currentHabitat = targetSite->habitat();
		aList<int>* neighborHabitats = targetSite->NeighboringHabitats();
		aList<long double> migrationProbs;/*probabilities of moving a given direction*/
		for (n=0;n<neighborHabitats->len();++n){
			potentialHabitat = neighborHabitats->Nth(n);
			prob=zero;
			if (potentialHabitat != -1){/*-1 indicates absolute boundary*/
				prob = MigrationProb(currentHabitat, potentialHabitat);}
			migrationProbs.Append(prob);
			}/*end for n*/
		direction=ran.weighted(migrationProbs);
		GridCell* targetNeighbor = targetSite->neighbor(direction);
		if (WillEnterHabitat(targetNeighbor,dist,ran)){
			targetSite=targetNeighbor;
			movedThisGen(1);
			//dist -=1;/*uncomment to avoid skipping a turn for non-move*/
			}/*not-skipping will run forever if no acceptable cell available*/
		dist -=1;/*uncomment to skip a turn for non-move*/
		delete neighborHabitats;neighborHabitats=0;
		}/*end while dist*/
	if(targetSite != &currentSite){
		int s=species();
		currentSite.Extract(s,this);
		targetSite->Install(s,this);
		}//end if
	return coords();/*wherever it ends up*/
	}/*end Individual::MoveRandomlyKurtoticDiffusion*/


long double	Individual::moveRate(void) {
	if(pop_!=NULL) return pop_->moveRate(sex(),age());
	return zero;
	}//end Individual::moveRate(void)


Population* Individual::pop (void) {
	return pop_;}

void	 Individual::pop (Population* p){
	pop_=p;}


void Individual::PrepareForBurial(void){}//derive this 

void Individual::Print(ostream& outFile, char sepChar){
	aString outStr("ID: "),stage;  outStr+sepChar;
	outStr.cat(id_); outStr+sepChar;
	char sexCh = (sex_==M) ? 'M':'F';
	(outStr+sexCh)+sepChar;
	(outStr+"age:")+sepChar; 
	outStr.cat(age_);outStr+sepChar;
	switch(lifeStage_){
		case gamete:
			stage="gam"; break;
		case zygote:
			stage="zyg"; break;
		case juvenile:
			stage="juv"; break;
		case adult:
			stage="adt"; break;
		default:
			stage="???"; break;
		}
	(((outStr+"stage:")+sepChar)+stage)+sepChar;
	if(!alive_){
		(outStr+"death:")+sepChar; 
		outStr.cat(causeOfDeath_); outStr+sepChar;}
	(outStr+"popID:")+sepChar;
	if(pop_ == NULL){
			outStr+"none";}
		else{
			outStr.cat(pop_->id());}
	outFile<<outStr<<endl;
	}//Individual::Print(ostream&, char)



void Individual::Reproduce (long& idCount, Ran3List& ran, aList<long double>& specialInfo)	{
	//installs a new baby in the population
	if(sex_==M) return;//derived classes may allow males to produce gametes
	int b,f=Fecundity(ran);
	for (b=0;b<f;++b){
		MakeABaby(idCount,ran,specialInfo);}
	}//end Individual::Reproduce
	
	
GridCell&	Individual::site (void) {
	return pop_->site();}


void Individual::sex (char genderChar) {
	switch (genderChar){
		case 'A':
			sex_=A; break;
		case 'B':
			sex_=B; break;
		case 'F':
			sex_=F; break;
		case 'M':
			sex_=M; break;
		default:
			sex_=A; break;
		}//switch
	}//Individual::sex(char)


int Individual::species(void){
	if(pop_!=NULL) return pop_->species();
	return 0;}
 
StringList* Individual::stateNames(void){
	//column heading info for states() below
	StringList* line = NEW StringList();
	line->Append(NEW aString("id"));
	line->Append(NEW aString("age")); 
	line->Append(NEW aString("sex")); 
	line->Append(NEW aString("alive")); 
	line->Append(NEW aString("lifeStage")); 
	line->Append(NEW aString("habitat"));
	return line;
	}//end Individual::stateNames


StringList* Individual::states(void){
	// indiv values for stateNames() above
	StringList* line = NEW StringList();
	line->Append(NEW aString(id())); 
	line->Append(NEW aString(age())); 
	line->Append(NEW aString(sex()));
	line->Append(NEW aString(alive())); 
	line->Append(NEW aString(lifeStage())); 
	line->Append(NEW aString(habitat()));
	return line;
	}//end Individual::states


long Individual::time(void){
	if(pop_!=NULL) return pop_->time();
	return 0;}

void Individual::ToNewLoc (long, long){
	//records the necessary changes when an individual moves on the grid
//	presentLoc_.set(xdev, ydev);
	movedThisGen(1);
	}	// end Individual::ToNewLoc



int Individual::WillEnterHabitat(GridCell* g, long&, Ran3List&){
	if (NULL==g){return 0;}
	//rules for moving into a new gridcell
	//OVERRIDE IN DERIVED CLASSES FOR SPECIAL RULES!!!
		//overrides can change the distance to move next
		//or add stochasticity using random number decisions
		//most will evaluate habitat via a call to g->habitat()
		//may also inflict death here

	//here, it will always enter unless carrying capacity is exceeded for adults
//	Population* pop=g->pop(species());
//	if (pop->adults() >= pop->carryingCapacity()) {return 0;}
	return 1;
	}//end Individual::WillEnterHabitat




#endif
