/************************
*
*
*	Individual2.cpp

	****   MOVED TO Individual.cpp ***********
*
*
*************************/

#ifndef Individual2cp
#define Individual2cp = 1

#include "Individual.h"
#include "Population.h"
#include "GridCell.h"
#include "LandScape.h"


//CoordinateType& Individual::MoveRandomly (Ran3List& ran, double /*kurtosisFactor*/){
CoordinateType& Individual::MoveRandomly (Ran3List& ran, aList<long double>& dispersalInfo){
	/*the coordinates returned are the final location
	override in a derived class for habitat-determined movement rules
	or rules about consistency of direction each step*/
	if (alive() && !movedThisGen()) {
		if(migrationModel()==diffusion){
				return MoveRandomlyDiffusion(ran,dispersalInfo);
				}
			else if (migrationModel()==steppingStone){
				/*only some individuals move*/
				return MoveRandomlyStepStone(ran,dispersalInfo);
				}
			else if (migrationModel()==island){
				/*only some individuals move*/
				return MoveRandomlyIsland(ran,dispersalInfo);
				}
			else if (migrationModel()==kurtotic){
				/*movement is diffusion with random sigma
				randomly determine a new distance in cells to traverse
				travel in steps in random directions until total cells reached
				lose a step if a habitat(landscape) edge crossing is attempted*/

//				return MoveRandomlyKurtoticDiffusion(ran,kurtosisFactor);
				}
			else {/*error; don't go anywhere*/
					}
		}/*end if alive*/
	return coords();/*wherever it ends up*/
	}/*end Individual::MoveRandomly*/



CoordinateType& Individual::MoveRandomlyDiffusion (Ran3List& ran, aList<long double>& /*dispersalInfo*/){
	/*the coordinates returned are the final location
	override in a derived class for habitat-determined movement rules
	or rules about consistency of direction each step*/

	/*randomly determine a new distance in cells to traverse
	travel in steps in random directions until total cells reached
	lose a step if a habitat(landscape) edge crossing is attempted*/
	long double mean=zero, stDev = zero, prob;
	stDev = moveRate() / gridScale();
	long dist=(long) ran.NormDev(mean, stDev);
	dist=ABS(dist);
	GridCell& currentSite=site();/*current grid cell*/
	GridCell* targetSite= &currentSite;
	int n, potentialHabitat, direction;
	while (dist>0){
		int currentHabitat = targetSite->habitat();
		aList<int>* neighborHabitats = targetSite->NeighboringHabitats();
		aList<long double> migrationProbs;/*probabilities of moving a given direction*/
		for (n=0;n<neighborHabitats->len();++n){
			potentialHabitat = neighborHabitats->Nth(n);
			prob=zero;
			if (potentialHabitat != -1){/*-1 indicates absolute boundary*/
				prob = MigrationProb(currentHabitat, potentialHabitat);}
			migrationProbs.Append(prob);
			}/*end for n*/
		direction=ran.weighted(migrationProbs);
		if (WillEnterHabitat(targetSite->neighbor(direction),dist,ran)){
			targetSite=targetSite->neighbor(direction);
			movedThisGen(1);
			//dist -=1;/*uncomment to avoid skipping a turn for non-move*/
			}/*not-skipping will run forever if no acceptable cell available*/
		dist -=1;/*uncomment to skip a turn for non-move*/
		delete neighborHabitats; neighborHabitats=0;
		}/*end while dist*/
	if(targetSite != &currentSite){
		int s=species();
		currentSite.Extract(s,this);
		targetSite->Install(s,this);
		}//end if
	return coords();/*wherever it ends up*/
	}/*end Individual::MoveRandomlyDiffusion*/





CoordinateType& Individual::MoveRandomlyStepStone (Ran3List& ran, aList<long double>& /*dispersalInfo*/){
	long double mprob, prob;
	/*only some individuals move*/
	mprob = moveRate();
	if(ran()>=mprob) {
		return coords();}//don't migrate
	/*only move to adjacent grid cells
	randomly determine a new direction
	lose a step if a habitat(landscape) edge crossing is attempted*/
	GridCell& currentSite=site();//current grid cell
	GridCell* targetSite=&currentSite;
	int n, potentialHabitat, currentHabitat = targetSite->habitat();
	aList<int>* neighborHabitats = targetSite->NeighboringHabitats();
	aList<double> migrationProbs;/*probabilities of moving a given direction*/
	for (n=0;n<neighborHabitats->len();++n){
		potentialHabitat = neighborHabitats->Nth(n);
		prob=zero;
		if (potentialHabitat != -1){/*-1 indicates absolute boundary*/
			prob = MigrationProb(currentHabitat, potentialHabitat);}
		migrationProbs.Append(prob);
		}/*end for n*/
	int direction=ran.weighted(migrationProbs);
	long dist=1;
	while (dist>0){
		if (WillEnterHabitat(targetSite->neighbor(direction),dist,ran)){
			targetSite=targetSite->neighbor(direction);
			movedThisGen(1);
			//dist -=1;/*uncomment to avoid skipping a turn for non-move*/
			}/*not-skipping will run forever if no acceptable cell available*/
		dist -=1;/*uncomment to skip a turn for non-move*/
		}/*end while dist*/
	if(targetSite != &currentSite){
		int s=species();
		currentSite.Extract(s,this);
		targetSite->Install(s,this);
		}/*end if*/
	delete neighborHabitats;neighborHabitats=0;
	return coords();/*wherever it ends up*/
	}/*end Individual::MoveRandomlyStepStone*/



CoordinateType& Individual::MoveRandomlyIsland (Ran3List& ran, aList<long double>& /*dispersalInfo*/){
	//only some individuals move
	long double mprob = moveRate(), prob;
	if(ran()>=mprob) {
		return coords();}//don't migrate
	//neighbors list contains all populations
	GridCell& currentSite=site();//current grid cell
	GridCell* targetSite=&currentSite;
	int r,c,rr,potentialHabitat,currentHabitat = currentSite.habitat();
	ObjList<aList<int> >* neighborHabitatsPtr = currentSite.region()->habitatGrid();
	ObjList<aList<int> >& neighborHabitats = *neighborHabitatsPtr;
	aList<long double> migrationProbs;/*probabilities of moving a given direction*/
	for (r=0;r<neighborHabitats.len();++r){
		for (c=0;c<neighborHabitats[r].len();++c){
			potentialHabitat = neighborHabitats[r][c];
			prob=zero;
			if (potentialHabitat != -1){/*-1 indicates absolute boundary*/
				prob = MigrationProb(currentHabitat, potentialHabitat);}
			migrationProbs.Append(prob);
			}}//end for r & c
	int newPopNum=ran.weighted(migrationProbs);
	CoordinateType newCoords;
	for (rr=0;rr<neighborHabitats.len();++rr){
		if (newPopNum >= (rr+1) * neighborHabitats[rr].len()){
				newCoords[0]++;}
			else{
				newCoords[1] = newPopNum - (rr * neighborHabitats[rr].len());
				break;}
		}//end for rr
	
	targetSite = &(currentSite.region()->GetGridCell(newCoords));
	movedThisGen(1);
	if(targetSite != &currentSite){
		int s=species();
		currentSite.Extract(s,this);
		targetSite->Install(s,this);
		}//end if
	delete neighborHabitatsPtr;neighborHabitatsPtr=0;
	return coords();//wherever it ends up
	}//end Individual::MoveRandomlyIsland



#endif
