/***************************************************
*																														}
*	QuickAutoSexLinkSibmateIndiv.cpp

	Base Class: QuickDiplSexGenIndividual																					}
*																														}
*	Handles processes involving individuals with autosomal and X-linked genetic information.
*	Genotype is an array of short integers, stored separately for sex-linked genes.
*	Sexual reproduction, but mate identity not stored; must mate at 
*	site of reproduction.  Loci autosomal or X-linked, but assumed to be unlinked.
*																														}
****************************************************/
#ifndef QuickAutoSexLinkSibmateIndivcpp
#define QuickAutoSexLinkSibmateIndivcpp = 1

# include "QuickAutoSexLinkSibmateIndiv.h"
#include "Element.cpp"
#include "aList.cpp"
#include "RandomList.h"
#include "GeneContainers.cpp"


		
QuickAutoSexLinkSibmateIndiv::QuickAutoSexLinkSibmateIndiv(const int nlociA, 
					const int nlociX, const int ploidyX, int* newGenesA, int* newGenesX, 
					long double sexratio, long idVal, Population* popSite){
	//done this way to permit multiple inheritance for derived genetic individuals
	genes_=NULL;
	genesX_=NULL;
	id_=idVal;
	pop_=popSite;
	nloci_=nlociA;
	genes_= NEW int[2*nlociA]();
	int loc, c;
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			genes_[loc*2+c] = newGenesA[loc*2+c];
			}}
	genesX_= NEW int[ploidyX*nlociX]();
	for(loc=0;loc<nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			genesX_[loc*ploidyX+c] = newGenesX[loc*ploidyX+c];
			}}
	mom_=dad_=NULL;
	sexRatio_=sexratio;
	if(ploidyX==1){sex_=M;}else{sex_=F;}
	sibMatingProb_ = one/(sexRatio_*(long double)( popSite->adults())-one);
	}//end constructor QuickAutoSexLinkSibmateIndiv::QuickAutoSexLinkSibmateIndiv


		
QuickAutoSexLinkSibmateIndiv::QuickAutoSexLinkSibmateIndiv(const int nlociA, 
					const int nlociX, const int ploidyX, int* newGenesA, int* newGenesX, long idVal, Population* popSite):
		mom_(NULL), dad_(NULL){
	//done this way to permit multiple inheritance for derived genetic individuals
	sexRatio_=half;//default
	genes_=NULL;
	genesX_=NULL;
	id_=idVal;
	pop_=popSite;
	nloci_=nlociA;
	genes_= NEW int[2*nlociA]();
	int loc, c;
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			genes_[loc*2+c] = newGenesA[loc*2+c];
			}}
	genesX_= NEW int[ploidyX*nlociX]();
	for(loc=0;loc<nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			genesX_[loc*ploidyX+c] = newGenesX[loc*ploidyX+c];
			}}
	if(ploidyX==1){sex_=M;}else{sex_=F;}
	sibMatingProb_ = one/(sexRatio_*(long double) (popSite->adults())-one);
	}//end constructor QuickAutoSexLinkSibmateIndiv::QuickAutoSexLinkSibmateIndiv


		
QuickAutoSexLinkSibmateIndiv::QuickAutoSexLinkSibmateIndiv(long id, Population* site):
		QuickAutosomeSexChromIndividual(), mom_(NULL), dad_(NULL), sexRatio_(half), sibMatingProb_(zero){
	//must add genes -- called during reproduction
	id_=id;
	pop_=site;
	}//end constructor QuickAutosomeSexChromIndividual::QuickAutosomeSexChromIndividual
	

QuickAutoSexLinkSibmateIndiv::QuickAutoSexLinkSibmateIndiv(void):
		QuickAutosomeSexChromIndividual(), mom_(NULL), dad_(NULL), sexRatio_(half), sibMatingProb_(zero){
	//must add genes -- called during reproduction
	id_=0;
	pop_=NULL;
	}//end constructor QuickAutoSexLinkSibmateIndiv::QuickAutoSexLinkSibmateIndiv


QuickAutoSexLinkSibmateIndiv::QuickAutoSexLinkSibmateIndiv(QuickAutoSexLinkSibmateIndiv& indiv){
	QuickAutosomeSexChromIndividual::Copy((QuickAutosomeSexChromIndividual*) &indiv);
	mom_=dad_=NULL;
	sexRatio_=indiv.sexRatio_;
	sibMatingProb_=indiv.sibMatingProb_;
	}//end constructor QuickAutoSexLinkSibmateIndiv::QuickAutoSexLinkSibmateIndiv
							
							
QuickAutoSexLinkSibmateIndiv::QuickAutoSexLinkSibmateIndiv(QuickAutoSexLinkSibmateIndiv* indiv){
	QuickAutosomeSexChromIndividual::Copy((QuickAutosomeSexChromIndividual*) indiv);
	mom_=dad_=NULL;
	sexRatio_=half;
	sibMatingProb_=zero;
	}//end constructor QuickAutoSexLinkSibmateIndiv::QuickAutoSexLinkSibmateIndiv


		
QuickAutoSexLinkSibmateIndiv::~QuickAutoSexLinkSibmateIndiv(void) {
		int i;//inform relatives of your demise
		for(i=0;i<sibsF_.len();++i){
			sibsF_[i].RemoveSibF(this);}
		for(i=0;i<sibsM_.len();++i){
			sibsM_[i].RemoveSibF(this);}
		for(i=0;i<offspringF_.len();++i){
			offspringF_[i].RemoveParent(this);
			}
		for(i=0;i<offspringM_.len();++i){
			offspringM_[i].RemoveParent(this);}
		if(mom_!=NULL){mom_->RemoveOffspring(this);}
		if(dad_!=NULL){dad_->RemoveOffspring(this);}
		//forget all relatives
		sibsF_.Detach();
		sibsM_.Detach();
		offspringF_.Detach();
		offspringM_.Detach();
		mom_=dad_=NULL;
		}//end destructor QuickAutoSexLinkSibmateIndiv::~QuickAutoSexLinkSibmateIndiv


Individual* QuickAutoSexLinkSibmateIndiv::Duplicate (void){
	//returns a duplicate that doesn't have family info, used for virtual copy construction
	return  NEW QuickAutoSexLinkSibmateIndiv(*this);
	}//end  QuickAutoSexLinkSibmateIndiv::Duplicate


Individual* QuickAutoSexLinkSibmateIndiv::Duplicate (long id, Population* site){
	//returns a duplicate in type, without family info, used for virtual copy construction
	return  NEW QuickAutoSexLinkSibmateIndiv(id, site);
	}//end  QuickAutoSexLinkSibmateIndiv::Duplicate


QuickAutoSexLinkSibmateIndiv* QuickAutoSexLinkSibmateIndiv::Duplicate (int* newGenesA, int* newGenesX, const int ploidyX, long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	QuickAutoSexLinkSibmateIndiv* gi =  NEW QuickAutoSexLinkSibmateIndiv(id, site);
	gi->nloci_ = nloci_;
	gi->genes_ = NEW int[2*nloci_]();
	int loc, c;
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			gi->genes_[2*loc+c] = newGenesA[2*loc+c];
			}}
	gi->nlociX_ = nlociX_;
	gi->genes_ = NEW int[ploidyX*nlociX_]();
	for(loc=0;loc<nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			gi->genesX_[ploidyX*loc+c] = newGenesX[ploidyX*loc+c];
			}}
	return gi;
	}//end  QuickAutosomeSexChromIndividual::Duplicate


QuickAutoSexLinkSibmateIndiv* QuickAutoSexLinkSibmateIndiv::Duplicate (int* newGenesA, int* newGenesX, const int ploidyX, 
			long double sexratio, long double sibmatingprob, long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	QuickAutoSexLinkSibmateIndiv* gi =  NEW QuickAutoSexLinkSibmateIndiv(id, site);
	gi->nloci_ = nloci_;
	gi->genes_ = NEW int[2*nloci_]();
	int loc, c;
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			gi->genes_[2*loc+c] = newGenesA[2*loc+c];
			}}
	gi->nlociX_ = nlociX_;
	gi->genes_ = NEW int[ploidyX*nlociX_]();
	for(loc=0;loc<nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			gi->genesX_[ploidyX*loc+c] = newGenesX[ploidyX*loc+c];
			}}
	sexRatio_=sexratio;
	sibMatingProb_=sibmatingprob;
	return gi;
	}//end  QuickAutosomeSexChromIndividual::Duplicate


QuickAutoSexLinkSibmateIndiv* QuickAutoSexLinkSibmateIndiv::Duplicate(int* /*newGenesA*/, long /*id*/, 
										Population* /*site=NULL*/){
	aError("ERROR: can\'t use the construction QuickAutoSexLinkSibmateIndiv:Duplicate(int*, long,Population*)");
//	return NEW QuickAutoSexLinkSibmateIndiv(*this);
	QuickAutoSexLinkSibmateIndiv* doppelganger = NEW QuickAutoSexLinkSibmateIndiv(*this);
	return doppelganger;
	}//QuickAutosomeSexChromIndividual::Duplicate
	
	

void QuickAutoSexLinkSibmateIndiv::Copy (Individual* it){
	QuickAutosomeSexChromIndividual::Copy(it);
	mom_=dad_=NULL;//clear this info
	sibsF_.Detach();
	sibsM_.Detach();
	offspringF_.Detach();
	offspringM_.Detach();
	sexRatio_=half;
	sibMatingProb_=zero;
	}//end QuickAutoSexLinkSibmateIndiv::Copy 

void QuickAutoSexLinkSibmateIndiv::Copy (QuickDiplSexGenIndividual* it){
	QuickDiplSexGenIndividual::Copy((QuickDiplSexGenIndividual*) it);
	mom_=dad_=NULL;//clear this info
	sibsF_.Detach();
	sibsM_.Detach();
	offspringF_.Detach();
	offspringM_.Detach();
	sexRatio_=half;
	sibMatingProb_=zero;
	}//end QuickAutoSexLinkSibmateIndiv::Copy 


void QuickAutoSexLinkSibmateIndiv::Copy (QuickAutosomeSexChromIndividual* it){
	QuickAutosomeSexChromIndividual::Copy((QuickAutosomeSexChromIndividual*) it);
	mom_=dad_=NULL;//clear this info
	sibsF_.Detach();
	sibsM_.Detach();
	offspringF_.Detach();
	offspringM_.Detach();
	sexRatio_=half;
	sibMatingProb_=zero;
	}//end QuickAutoSexLinkSibmateIndiv::Copy 


void QuickAutoSexLinkSibmateIndiv::Copy (QuickAutoSexLinkSibmateIndiv* it){
	Copy((QuickAutosomeSexChromIndividual*) it);
	mom_=it->mom_;
	dad_=it->dad_;
	sibsF_=it->sibsF_;
	sibsM_=it->sibsM_;
	offspringF_=it->offspringF_;
	offspringM_=it->offspringM_;
	sexRatio_=it->sexRatio_;
	sibMatingProb_=it->sibMatingProb_;
	}//end QuickAutoSexLinkSibmateIndiv::Copy 








void QuickAutoSexLinkSibmateIndiv::Clone (long& idCount, Ran3List& ran, aList<long double>& specialInfo)	{
	//installs a new baby in the population
	if(sex_==M){
		aError("ERROR: males cannot clone themselves (QuickAutosomeSexChromIndividual::Clone(long&,Ran3&,double,double)).");
		return;}
	int loc, c, ploidyX=2;
	QuickAutoSexLinkSibmateIndiv* baby = NEW QuickAutoSexLinkSibmateIndiv(++idCount);
	baby->sex_ = sex_;
	baby->age_ = 1;
	baby->alive_ = 1;
	baby->lifeStage_ = zygote;
	baby->moveRate_ = moveRate_;
	baby->nloci_ = nloci_;
	baby->genes_ = NEW int[2*nloci_]();
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			baby->genes_[loc*2+c] = genes_[loc*2+c];
			}}
	if(sex_==M){ ploidyX=1;}
	baby->nlociX_ = nlociX_;
	baby->genesX_ = NEW int[ploidyX*nlociX_]();
	for(loc=0;loc<nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			baby->genesX_[loc*ploidyX+c] = genesX_[loc*ploidyX+c];
			}}
	baby->Mutate(ran,specialInfo);
	pop()->Install( (Individual*) baby);
	baby->mom_=mom_;
	baby->dad_=NULL;
	baby->sibsF_ = offspringF_;
	baby->sibsM_ = offspringM_;
	baby->sexRatio_=sexRatio_;
	baby->sibMatingProb_=sibMatingProb_;
	}//end QuickAutoSexLinkSibmateIndiv::Clone
	

void QuickAutoSexLinkSibmateIndiv::Clone (Individual* mom, Ran3List& ran, aList<long double>& specialInfo)	{
	//copies mom as a new baby *this
	//obtain all information directly from mom
	Individual::Clone(mom, ran, specialInfo);// copies location etc. details
	moveRate_ = mom->moveRate_;
	//mom must be a QuickAutoSexLinkSibmateIndiv too
	QuickAutoSexLinkSibmateIndiv* gMom = (QuickAutoSexLinkSibmateIndiv*) mom->GetPointerToDerivedSelf();
	int loc, c, ploidyX=2;
	if(gMom->sex_==M){ ploidyX=1;}
	nloci_ = gMom->nloci_;
	genes_ = NEW int[2*nloci_]();
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			genes_[loc*2+c] = gMom->genes_[loc*2+c];
			}}
	nlociX_ = gMom->nlociX_;
	genesX_ = NEW int[ploidyX*nlociX_]();
	for(loc=0;loc<nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			genesX_[loc*ploidyX+c] = gMom->genesX_[loc*ploidyX+c];
			}}
	Mutate(ran,specialInfo);
	mom_=gMom;
	dad_=NULL;
	sibsF_=gMom->offspringF_;
	sibsM_=gMom->offspringM_;
	sexRatio_=gMom->sexRatio_;
	sibMatingProb_=gMom->sibMatingProb_;
	}//end QuickAutoSexLinkSibmateIndiv::Clone



void QuickAutoSexLinkSibmateIndiv::IIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran, LandScapeInitializer01& init, aList<long double>& extraInfo){
	QuickAutosomeSexChromIndividual::IIndividual(gender,lifeStage,rate, ran,init,extraInfo);
	mom_=dad_=NULL;//clear family info if necessary
	sibsF_.Detach();
	sibsM_.Detach();
	offspringF_.Detach();
	offspringM_.Detach();
	sexRatio_=half;
	sibMatingProb_=zero;
	}//end QuickAutoSexLinkSibmateIndiv::IIndividual



void QuickAutoSexLinkSibmateIndiv::MakeABaby (long& idCount, Ran3List& ran, aList<long double>& specialInfo){
	//called by mother -- installs a new baby in the population
	if(sex_==M) return;
	if (mates()){
		QuickAutoSexLinkSibmateIndiv* baby = NEW QuickAutoSexLinkSibmateIndiv(++idCount);
		baby->age_ = 1;
		baby->alive_ = 1;
		baby->lifeStage_ = zygote;
		baby->sexRatio_=sexRatio_;
		baby->sibMatingProb_=sibMatingProb_;
		aList<long double> sexProbs;
//		double femaleProb = pop_->sexRatio();
//		long double femaleProb = sexRatio_;//for debugging
//		int bsex = ran() < one-femaleProb;
		int babysex = ran() < sexRatio_;//0 or 1
		baby->sex_ = (genderSet)(2 + babysex);//uses function in terms of boys
		bool matesWithSib=false, hasBrother=false;
		if(sibsM_.len()>0){hasBrother=true;}
		long dadnum;
		QuickAutoSexLinkSibmateIndiv* dad=NULL;
		if(hasBrother && ran()<sibMatingProb_){
				//choose a brother
				dadnum=ran.longBetween(0,sibsM_.len()-1);
				dad = &(sibsM_[dadnum]);
				}
			else{//choose randomly
				dadnum = ran.longBetween(0,pop_->fathers()-1);
				Individual& idad = pop_->father(dadnum);
				dad = (QuickAutoSexLinkSibmateIndiv*) idad.GetPointerToDerivedSelf();
				}
#ifdef DEBUG_REPRODUCTION
		Print(cout,' ');
		dad->Print(cout,' ');
#endif
		int i, loc, m, d, ploidyX=2;
		baby->sibMatingProb_=sibMatingProb_;
		baby->mom_=this;
		baby->dad_=dad;
		baby->sibsF_=offspringF_;
		baby->sibsM_=offspringM_;
		for(i=0;i<dad->offspringF_.len();++i){
			baby->sibsF_.Append(&(dad->offspringF_[i]));}
		for(i=0;i<dad->offspringM_.len();++i){
			baby->sibsM_.Append(&(dad->offspringM_[i]));}
		if(baby->sex_==M){
				baby->moveRate_ = dad->moveRate_;}
			else{
				baby->moveRate_ = moveRate_;}

		if(baby->sex_==M) ploidyX=1;
		baby->nloci_ = nloci_;
		baby->genes_ = NEW int[2*nloci_]();
		for(loc=0;loc<nloci_;++loc){//loci unlinked
			m=ran.longBetween(0,1);//egg meiosis
			baby->genes_[loc*2+0] = genes_[loc*2+m];
			d=ran.longBetween(0,1);//sperm meiosis
			baby->genes_[loc*2+1] = dad->genes_[loc*2+d];
			}
		baby->nlociX_ = nlociX_;
		baby->genesX_ = NEW int[ploidyX*nlociX_]();
		for(loc=0;loc<nlociX_;++loc){//loci unlinked
			m=ran.longBetween(0,1);//egg meiosis
			baby->genesX_[loc*ploidyX+0] = genesX_[loc*2+m];//mom is diploid
			if(baby->sex_==F){//male contribution too
				baby->genesX_[loc*ploidyX+1] = dad->genesX_[loc];//dad is heterogametic
				}
			}
		baby->Mutate(ran, specialInfo);
		pop_->Install(baby);
#ifdef DEBUG_REPRODUCTION
		baby->Print(cout,' ');cout<<endl;
#endif
		}// end if mates
	}  //end QuickAutoSexLinkSibmateIndiv::MakeABaby



void QuickAutoSexLinkSibmateIndiv::RemoveParent(QuickAutoSexLinkSibmateIndiv* parent){
	//forgets identity -- must delete elsewhere
	if(long(mom_)==long(parent)){//compare pointers
		mom_=NULL;}
	if(long(dad_)==long(parent)){//compare pointers
		dad_=NULL;}
	}//QuickAutoSexLinkSibmateIndiv::RemoveParent()
	
	
void QuickAutoSexLinkSibmateIndiv::RemoveOffspring(QuickAutoSexLinkSibmateIndiv* child){
	//forgets identity -- must delete elsewhere
	if(offspringF_.Includes(child)){
			offspringF_.Detach(child);}
		else if(offspringM_.Includes(child)){
			offspringM_.Detach(child);}
		else{}
	}//QuickAutoSexLinkSibmateIndiv::RemoveOffspring()
	
	
void QuickAutoSexLinkSibmateIndiv::RemoveSibF(QuickAutoSexLinkSibmateIndiv* sib){
	//forgets identity -- must delete elsewhere
	if(sibsF_.Includes(sib)){
		sibsF_.Detach(sib);}
	}//QuickAutoSexLinkSibmateIndiv::RemoveSibF()


void QuickAutoSexLinkSibmateIndiv::RemoveSibM(QuickAutoSexLinkSibmateIndiv* sib){
	//forgets identity -- must delete elsewhere
	if(sibsM_.Includes(sib)){
		sibsM_.Detach(sib);}
	}//QuickAutoSexLinkSibmateIndiv::RemoveSibM()


void QuickAutoSexLinkSibmateIndiv::SetOffspring(QuickAutoSexLinkSibmateIndiv* child){
	if(child->sex_==F && !(offspringF_.Includes(child))){
		offspringF_.Append(child);}
	if(child->sex_==M && !(offspringM_.Includes(child))){
		offspringM_.Append(child);}
	}//QuickAutoSexLinkSibmateIndiv::SetOffspring()
		
		
void QuickAutoSexLinkSibmateIndiv::SetSib(QuickAutoSexLinkSibmateIndiv* sib){
	if(sib->sex_==F && !(offspringF_.Includes(sib))){
		offspringF_.Append(sib);}
	if(sib->sex_==M && !(offspringM_.Includes(sib))){
		offspringM_.Append(sib);}
	}//QuickAutoSexLinkSibmateIndiv::SetSib()




#endif

