/***************************************************
*																														}
*	QuickAutosomeSexChromIndividual.cpp

	Base Class: QuickDiplSexGenIndividual																					}
*																														}
*	Handles processes involving individuals with autosomal and X-linked genetic information.
*	Genotype is an array of short integers, stored separately for sex-linked genes.
*	Sexual reproduction, but mate identity not stored; must mate at 
*	site of reproduction.  Loci autosomal or X-linked, but assumed to be unlinked.
*																														}
****************************************************/
#ifndef QuickAutosomeSexChromIndividualcpp
#define QuickAutosomeSexChromIndividualcpp = 1

# include "QuickAutosomeSexChromIndividual.h"
#include "Element.cpp"
#include "aList.cpp"
#include "RandomList.h"
#include "GeneContainers.cpp"


		
QuickAutosomeSexChromIndividual::QuickAutosomeSexChromIndividual(const int nlociA, 
					const int nlociX, const int ploidyX, int* newGenesA, int* newGenesX, long idVal, Population* popSite):
		nlociX_(nlociX), genesX_(0){
	//done this way to permit multiple inheritance for derived genetic individuals
	id_=idVal;
	pop_=popSite;
	nloci_=nlociA;
	genes_= NEW int[2*nlociA]();
	int loc, c;
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			genes_[loc*2+c] = newGenesA[loc*2+c];
			}}
	genesX_= NEW int[ploidyX*nlociX]();
	for(loc=0;loc<nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			genesX_[loc*ploidyX+c] = newGenesX[loc*ploidyX+c];
			}}
	if(ploidyX==1){sex_=M;}else{sex_=F;}
	}//end constructor QuickAutosomeSexChromIndividual::QuickAutosomeSexChromIndividual


		
QuickAutosomeSexChromIndividual::QuickAutosomeSexChromIndividual(long id, Population* site):
		QuickDiplSexGenIndividual(),nlociX_(0), genesX_(0){
	//must add genes -- called during reproduction
	id_=id;
	pop_=site;
	}//end constructor QuickAutosomeSexChromIndividual::QuickAutosomeSexChromIndividual
	

QuickAutosomeSexChromIndividual::QuickAutosomeSexChromIndividual(void):
		QuickDiplSexGenIndividual(),nlociX_(), genesX_(0){
	//must add genes -- called during reproduction
	id_=0;
	pop_=NULL;
	}//end constructor QuickAutosomeSexChromIndividual::QuickAutosomeSexChromIndividual


QuickAutosomeSexChromIndividual::QuickAutosomeSexChromIndividual(QuickAutosomeSexChromIndividual& indiv){
	QuickDiplSexGenIndividual::Copy((QuickDiplSexGenIndividual*) &indiv);
	nlociX_ = indiv.nlociX_;
	int ploidyX=2;
	if(indiv.sex_==M){ploidyX=1;}
	genesX_ = NEW int[ploidyX*nlociX_];
	for(int loc=0;loc<nlociX_;loc++){
		for(int c=0;c<ploidyX;++c){
			genesX_[loc*ploidyX+c] = indiv.genesX_[loc*ploidyX+c];
			}}
	}//end constructor QuickAutosomeSexChromIndividual::QuickAutosomeSexChromIndividual
							
							
QuickAutosomeSexChromIndividual::QuickAutosomeSexChromIndividual(QuickAutosomeSexChromIndividual* indiv){
	QuickDiplSexGenIndividual::Copy((QuickDiplSexGenIndividual*) indiv);
	nlociX_ = indiv->nlociX_;
	int loc, c, ploidyX=2;
	if(indiv->sex_==M){ploidyX=1;}
	genesX_ = NEW int[ploidyX*nlociX_];
	for(loc=0;loc<nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			genesX_[loc*ploidyX+c] = indiv->genesX_[loc*ploidyX+c];
			}}
	}//end constructor QuickAutosomeSexChromIndividual::QuickAutosomeSexChromIndividual


		
QuickAutosomeSexChromIndividual::~QuickAutosomeSexChromIndividual(void) {
		delete [] genesX_; genesX_ = NULL;
		}//end destructor QuickAutosomeSexChromIndividual::~QuickAutosomeSexChromIndividual


Individual* QuickAutosomeSexChromIndividual::Duplicate (void){
	//returns a duplicate, used for virtual copy construction
	return  NEW QuickAutosomeSexChromIndividual(*this);
	}//end  QuickAutosomeSexChromIndividual::Duplicate


Individual* QuickAutosomeSexChromIndividual::Duplicate (long id, Population* site){
	//returns a duplicate in type, used for virtual copy construction
	return  NEW QuickAutosomeSexChromIndividual(id, site);
	}//end  QuickAutosomeSexChromIndividual::Duplicate


QuickAutosomeSexChromIndividual* QuickAutosomeSexChromIndividual::Duplicate (int* newGenesA, int* newGenesX, const int ploidyX, long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	QuickAutosomeSexChromIndividual* gi =  NEW QuickAutosomeSexChromIndividual(id, site);
	gi->nloci_ = nloci_;
	gi->genes_ = NEW int[2*nloci_]();
	int loc, c;
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			gi->genes_[2*loc+c] = newGenesA[2*loc+c];
			}}
	gi->nlociX_ = nlociX_;
	gi->genes_ = NEW int[ploidyX*nlociX_]();
	for(loc=0;loc<nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			gi->genesX_[ploidyX*loc+c] = newGenesX[ploidyX*loc+c];
			}}
	return gi;
	}//end  QuickAutosomeSexChromIndividual::Duplicate


void QuickAutosomeSexChromIndividual::Copy (Individual* it){
//	Individual::Copy(it);
	QuickDiplSexGenIndividual::Copy(it);
	moveRate_ = it->moveRate_;
	delete [] genes_; genes_ = NULL; nloci_=0;
	delete [] genesX_; genesX_ = NULL; nlociX_=0;
	}//end QuickAutosomeSexChromIndividual::Copy 

void QuickAutosomeSexChromIndividual::Copy (QuickDiplSexGenIndividual* it){
	QuickDiplSexGenIndividual::Copy((QuickDiplSexGenIndividual*) it);
	moveRate_ = it->moveRate_;
	nlociX_=0;
	delete [] genesX_; genesX_ = NULL;
	}//end QuickAutosomeSexChromIndividual::Copy 



void QuickAutosomeSexChromIndividual::Copy (QuickAutosomeSexChromIndividual* it){
	genderSet originalSex = sex_;
	Individual::Copy((Individual*) it);
	moveRate_ = it->moveRate_;
	int ploidyX=2,loc,c;
	if(it->sex_==M){ploidyX=1;}
	if(it->nloci_ != nloci_){
		delete [] genes_; genes_ = NULL;
		nloci_ = it->nloci_;
		genes_ = NEW int[2*nloci_]();
		}
	if(it->nlociX_ != nlociX_ || originalSex != it->sex_){//must change genesX_ size
		delete [] genesX_; genesX_ = NULL;
		nlociX_ = it->nlociX_;
		genesX_ = NEW int[ploidyX*nlociX_]();
		}
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			genes_[loc*2+c] = it->genes_[loc*2+c];}};
	for(loc=0;loc<nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			genesX_[loc*ploidyX+c] = it->genesX_[loc*ploidyX+c];}};
	}//end QuickAutosomeSexChromIndividual::Copy 








int	QuickAutosomeSexChromIndividual::AllelePresent (int loc, aString& alleleVal){
	if(loc<nloci_){
		return QuickDiplSexGenIndividual::AllelePresent (loc, alleleVal);}
	int a = alleleVal.toInt(), locX = loc-nloci_;
	if(sex_==M){
		if ((genesX_[locX] == a)) {return 1;} else {return 0;}}
	if ((genesX_[locX*2+0] == a) || (genesX_[locX*2+1] == a)) return 1;
	return 0;			
	}


//void QuickAutosomeSexChromIndividual::Clone (long& idCount, Ran3& ran, double mutationRate, double sigma)	{
void QuickAutosomeSexChromIndividual::Clone (long& idCount, Ran3List& ran, aList<long double>& specialInfo)	{
	//installs a new baby in the population
	if(sex_==M){
		aError("ERROR: males cannot clone themselves (QuickAutosomeSexChromIndividual::Clone(long&,Ran3&,double,double)).");
		return;}
	int loc, c, ploidyX=2;
	QuickAutosomeSexChromIndividual* baby = NEW QuickAutosomeSexChromIndividual(++idCount);
	baby->sex_ = sex_;
	baby->age_ = 1;
	baby->alive_ = 1;
	baby->lifeStage_ = zygote;
	baby->moveRate_ = moveRate_;
	baby->nloci_ = nloci_;
	baby->genes_ = NEW int[2*nloci_]();
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			baby->genes_[loc*2+c] = genes_[loc*2+c];
			}}
	if(sex_==M){ ploidyX=1;}
	baby->nlociX_ = nlociX_;
	baby->genesX_ = NEW int[ploidyX*nlociX_]();
	for(loc=0;loc<nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			baby->genesX_[loc*ploidyX+c] = genesX_[loc*ploidyX+c];
			}}
//	baby->Mutate(ran,mutationRate,sigma);
	baby->Mutate(ran,specialInfo);
	pop()->Install( (Individual*) baby);
	}//end QuickAutosomeSexChromIndividual::Clone
	

//void QuickAutosomeSexChromIndividual::Clone (Individual* mom, Ran3& ran, double mutationRate, double sigma)	{
void QuickAutosomeSexChromIndividual::Clone (Individual* mom, Ran3List& ran, aList<long double>& specialInfo)	{
	//copies mom as a new baby *this
	//obtain all information directly from mom
//	Individual::Clone(mom);// copies location etc. details
	Individual::Clone(mom, ran, specialInfo);// copies location etc. details
	moveRate_ = mom->moveRate_;
	//mom must be a QuickAutosomeSexChromIndividual too
//	QuickAutosomeSexChromIndividual* gMom = (QuickAutosomeSexChromIndividual*) mom;
	QuickAutosomeSexChromIndividual* gMom = (QuickAutosomeSexChromIndividual*) mom->GetPointerToDerivedSelf();
	int loc, c, ploidyX=2;
	if(gMom->sex_==M){ ploidyX=1;}
	nloci_ = gMom->nloci_;
	genes_ = NEW int[2*nloci_]();
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			genes_[loc*2+c] = gMom->genes_[loc*2+c];
			}}
	nlociX_ = gMom->nlociX_;
	genesX_ = NEW int[ploidyX*nlociX_]();
	for(loc=0;loc<nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			genesX_[loc*ploidyX+c] = gMom->genesX_[loc*ploidyX+c];
			}}
//	Mutate(ran,mutationRate,sigma);
	Mutate(ran,specialInfo);
	}//end QuickAutosomeSexChromIndividual::Clone



int	QuickAutosomeSexChromIndividual::CountAlleles (int loc, aString& alleleVal){
	if(loc<nloci_){
		return QuickDiplSexGenIndividual::CountAlleles (loc, alleleVal);}
	int c, a = alleleVal.toInt(), ploidyX=2, locX=loc-nloci_, nAlleles=0;
	if(sex_==M){ploidyX=1;}
	for(c=0;c<ploidyX;+c){
		if (genes_[loc*ploidyX+c] == a) nAlleles++;}
	return nAlleles;
	}//end QuickAutosomeSexChromIndividual::CountAlleles




void QuickAutosomeSexChromIndividual::GetGenome(GeneContainerCore& gc){
	//refills container class as a MultiInheritanceModeContainer<int>[mode][ploidy][locus]
	MultiInheritanceModeContainer<int>* mimc = (MultiInheritanceModeContainer<int>*) &gc;
	mimc->Clear();
//	for(int i=0;i<MultiInheritanceModeContainer<int>::inheritanceModes_;++i){
//		mimc->inheritanceSet_.Append(NEW MultiHaplotypeContainer<int>());}
//	QuickDiplSexGenIndividual::GetGenome(mimc->diploidData());
	QuickDiplSexGenIndividual::GetGenome(*mimc);
	mimc->sex_ = sex_;
	mimc->SetSexLinkLoci(nlociX_);
	int c,loc;
	MultiHaplotypeContainer<int>& sexLinkedGenes = mimc->sexLinkedData();
	for(c=0;c<sexLinkedGenes.ploidy_;++c){
		for(loc = 0;loc<nlociX_;++loc){
			sexLinkedGenes[c][loc] = genesX_[loc*sexLinkedGenes.ploidy_+c];}
		}//c
	mimc->SetID(id_);
	}//Genotype::QuickAutosomeSexChromIndividual(GeneContainerCore& )



int* QuickAutosomeSexChromIndividual::GetGenotype (void) {
	//returns a copy that must be disposed of elsewhere
	int ploidyX=2, loc, c;
	if(sex_==M) ploidyX=1;
	int *genesCopy = NEW int[2*nloci_+ploidyX*nlociX_]();
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			genesCopy[loc*2+c] = genes_[loc*2+c];
			}}
	for(loc=2*nloci_;loc<2*nloci_+ploidyX*nlociX_;loc++){
		for(c=0;c<ploidyX;++c){
			genesCopy[2*nloci_+loc*ploidyX+c] = genesX_[loc*ploidyX+c];
			}}
	return genesCopy;
	}//end QuickAutosomeSexChromIndividual::GetGenotype



aString* QuickAutosomeSexChromIndividual::GenotypeString(char* separator){
	aString* gStr = NEW aString();
	int loc;
	for(loc=0;loc<nloci_+nlociX_;++loc){
		aString* lgStr = GenotypeString(loc, separator);
		if(loc>0){lgStr->cat(separator);}
		if(loc>nloci_ && sex_==M){lgStr->cat(separator);}//line up males and females into columns
		gStr->cat(*lgStr);
		delete lgStr;}
	return gStr;
	}//end QuickAutosomeSexChromIndividual::GenotypeString
	


aString* QuickAutosomeSexChromIndividual::GenotypeString(int locus, char* separator){
	//alleles separated by parentheses; e.g. (1)(1) or (1)(0)
	if(locus<nloci_) return QuickDiplSexGenIndividual::GenotypeString(locus, separator);
	int c, ploidyX=2, loc=locus-nloci_;
	if(sex_==M) ploidyX=1;
	aString* gStr = NEW aString();
	for(c=0;c<ploidyX;++c){
		aString cgStr("(");
		cgStr.cat(genesX_[loc*ploidyX+c]);
		cgStr.cat(")");
		gStr->cat(cgStr);
		}
	return gStr;
	}//end QuickAutosomeSexChromIndividual::GenotypeString
	
	
	
aString* QuickAutosomeSexChromIndividual::GenotypeString(int locus, int alleleNum, char* separator){
	if(locus<nloci_) return QuickDiplSexGenIndividual::GenotypeString(locus, alleleNum, separator);
	int loc=locus-nloci_, ploidyX=2;
	if(sex_==M) ploidyX=1;
	return NEW aString(genesX_[loc*ploidyX+alleleNum]);
	}//end QuickAutosomeSexChromIndividual::GenotypeString



void QuickAutosomeSexChromIndividual::IIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran, LandScapeInitializer01& init, aList<long double>& extraInfo){
	QuickDiplSexGenIndividual::IIndividual(gender,lifeStage,rate, ran,init,extraInfo);
	int species = (int)extraInfo[0];
	int pop =     (int)extraInfo[1];
	int indiv =   (int)extraInfo[2];
	MultiInheritanceModeContainer<int>* mimc = (MultiInheritanceModeContainer<int>*) &(init.initMultiInheritanceGenotypes_[species][pop][indiv]);
	QuickAutosomeSexChromIndividual::SetSexLinkedGenome(*mimc);
	}//end QuickAutosomeSexChromIndividual::IIndividual


int QuickAutosomeSexChromIndividual::IsHeterozygote (int loc){
	if(loc<nloci_) return QuickDiplSexGenIndividual::IsHeterozygote(loc);
	if(sex_==M) {return 0;}
	int locX = loc-nloci_;
	if (genesX_[locX*2+0] == genesX_[locX*2+1] ) return 0;
	return 1;
	}//end QuickAutosomeSexChromIndividual::IsHeterozygote(int)
	
	
int QuickAutosomeSexChromIndividual::IsHeterozygote (int loc, aString& alleleVal){
	if(loc<nloci_) return QuickDiplSexGenIndividual::IsHeterozygote(loc,alleleVal);
	if(sex_==M) {return 0;}
	int locX = loc-nloci_;
	int a = alleleVal.toInt();
	if ((genesX_[locX*2+0] == a) && (genesX_[locX*2+1] != a)) return 1;
	if ((genesX_[locX*2+0] != a) && (genesX_[locX*2+1] == a)) return 1;
	return 0;
	}//end QuickAutosomeSexChromIndividual::IsHeterozygote(int,aString&)



int QuickAutosomeSexChromIndividual::loci(void){
	return nloci_+nlociX_;
	}//end QuickAutosomeSexChromIndividual::loci()


//void QuickAutosomeSexChromIndividual::MakeABaby (long& idCount, Ran3List& ran, double mutationRate, double sigma){
void QuickAutosomeSexChromIndividual::MakeABaby (long& idCount, Ran3List& ran, aList<long double>& specialInfo){
	//assumes baby has the same moveRate_ rate as the calling parent, probably female
	//installs a new baby in the population
	if(sex_==M) return;
	if (mates()){
		QuickAutosomeSexChromIndividual* baby = NEW QuickAutosomeSexChromIndividual(++idCount);
		baby->age_ = 1;
		baby->alive_ = 1;
		baby->lifeStage_ = zygote;
		baby->moveRate_ = moveRate_;
		aList<long double> sexProbs;
//		double femaleProb = pop_->sexRatio();
		long double femaleProb = half;//for debugging
//		long double femaleProb = specialInfo[2];
//		sexProbs.Append(femaleProb);
//		sexProbs.Append(one - femaleProb);
//		baby->sex_ = (genderSet)(2 + ran.weighted(sexProbs));
		int bsex = ran() < one-femaleProb;
		baby->sex_ = (genderSet)(2 +bsex);
		long dadnum = ran.longBetween(0,pop_->fathers()-1);
		Individual& idad = pop_->father(dadnum);
		QuickAutosomeSexChromIndividual* dad = (QuickAutosomeSexChromIndividual*) idad.GetPointerToDerivedSelf();
#ifdef DEBUG_REPRODUCTION
		Print(cout,' ');
		dad->Print(cout,' ');
#endif
		int loc, m, d, ploidyX=2;
		if(baby->sex_==M) ploidyX=1;
		baby->nloci_ = nloci_;
		baby->genes_ = NEW int[2*nloci_]();
		for(loc=0;loc<nloci_;++loc){//loci unlinked
			m=ran.longBetween(0,1);//egg meiosis
			baby->genes_[loc*2+0] = genes_[loc*2+m];
			d=ran.longBetween(0,1);//sperm meiosis
			baby->genes_[loc*2+1] = dad->genes_[loc*2+d];
			}
		baby->nlociX_ = nlociX_;
		baby->genesX_ = NEW int[ploidyX*nlociX_]();
		for(loc=0;loc<nlociX_;++loc){//loci unlinked
			m=ran.longBetween(0,1);//egg meiosis
			baby->genesX_[loc*ploidyX+0] = genesX_[loc*2+m];//mom is diploid
			if(baby->sex_==F){//male contribution too
				baby->genesX_[loc*ploidyX+1] = dad->genesX_[loc];//dad is heterogametic
				}
			}
//		baby->Mutate(ran, mutationRate, sigma);
		baby->Mutate(ran, specialInfo);
		pop_->Install(baby);
#ifdef DEBUG_REPRODUCTION
		baby->Print(cout,' ');cout<<endl;
#endif
		}// end if mates
	}  //end QuickAutosomeSexChromIndividual::MakeABaby


int QuickAutosomeSexChromIndividual::mates(void){
	//asks if potential mates are available in this population
	if(pop_==NULL) return false;
	if(sex_==M){//find females
			if(pop_->females() > 0){
				return true;}
			}
		else if(sex_==F){
			if(pop_->males() > 0){
				return true;}
			}
		else{}
	return false;
	}//QuickAutosomeSexChromIndividual::mates()
		

//void QuickAutosomeSexChromIndividual::Mutate (Ran3& ran, double mutationRate, double /*sigma*/){
void QuickAutosomeSexChromIndividual::Mutate (Ran3& ran, aList<long double>& specialInfo){
							//specialInfo[0] = mutationRate
							//specialInfo[1] = 2-allele model (0), infinite-allele model (1)
							//specialInfo[2] = infinite allele counter for locus 1 (increment for each new mutation)
							//specialInfo[3] =    " locus 2, etc.
	//mutate the genome if warranted
	if(specialInfo.len()==0) return;
	if(specialInfo[0] == zero) return;
	QuickDiplSexGenIndividual::Mutate(ran,specialInfo);
	long double probMutant, mutationRate = specialInfo[0], ranNum = ran();
	bool biallelic = true;
	if(specialInfo.len()>1){
		if(specialInfo[1]!=0){
			biallelic=false;//infinite allele model
			}}
	//faster method: find out if any mutations exist by using the probability of no mutations overall
		//quit if no mutation turns up
		//if a mutation turns up, repeat for the remaining gene copies
	int nmutations=0;
	long alleleCopy,m,mutantCopy,mutantCopyLocus, runningcopy=0,allelecopies=1;
	long double allelecopiesLD = (long double)nloci_;
	if(sex_==F){
		allelecopiesLD*=two;
		allelecopies++;}
	for(alleleCopy = 0;alleleCopy<allelecopies;++alleleCopy){//all gene copies --> one-indexed
		if(alleleCopy==0){//binomial
				probMutant = exp(allelecopiesLD * ln(one - mutationRate));}
			else if (alleleCopy==1){
				probMutant += allelecopiesLD * mutationRate * exp((allelecopiesLD - one) * ln(one - mutationRate));
				}
			else{//really really rare unless mutationRate is really high
				probMutant = BinomialDistributionCDF(allelecopies,alleleCopy,mutationRate);
				}
		if(ranNum>=probMutant){
				nmutations++;}
			else{break;}
		}//alleleCopy
	if(nmutations==1){
			mutantCopy = ran.longBetween(0,allelecopies-1);
				if(biallelic){
						if(genes_[mutantCopy]==0){genes_[mutantCopy]=1;}else{genes_[mutantCopy]=0;}
						}
					else{//infinite allele
						mutantCopyLocus = mutantCopy/allelecopies;
						genesX_[mutantCopy] = (long) ++specialInfo[2+mutantCopyLocus];
						}
			}
		else if (nmutations>1){
			aList<int> mutatedAlleleCopies;//ordered 
			for(m=nmutations-1;m>=0;--m){//change a random allele without replacement
				mutantCopy=-1;
				do{
					mutantCopy=ran.longBetween(0,allelecopies-1);
					}while(mutatedAlleleCopies.Includes(mutantCopy));
				if(biallelic){
						if(genesX_[mutantCopy]==0){genesX_[mutantCopy]=1;}else{genesX_[mutantCopy]=0;}
						}
					else{//infinite allele
						mutantCopyLocus = mutantCopy/allelecopies;
						genesX_[mutantCopy] = (long) ++specialInfo[2+mutantCopyLocus];
						}
				if(m>0){
					mutatedAlleleCopies.Append(mutantCopy);}
				}//m
			}
		else{}//skip if no mutations
	}//end QuickAutosomeSexChromIndividual::Mutate()


long double QuickAutosomeSexChromIndividual::phenotype(int loc){
	//assign tha allele state 1 to dominance and return the pattern
	if(loc<nloci_) return QuickDiplSexGenIndividual::phenotype(loc);
	loc -= nloci_;
	if(sex_==F){if((genesX_[loc*2+0]==0)&&(genesX_[loc*2+1]==0)) return zero;}
	if(sex_==M){if(genesX_[loc+0]==0) return zero;}
	return one;
	}//end QuickAutosomeSexChromIndividual::phenotype(int)


int QuickAutosomeSexChromIndividual::ploidy(int loc){
	if(loc<nloci_) return QuickDiplSexGenIndividual::ploidy(loc);
	if(sex_==M) return 1;
	return 2;
	}//end QuickAutosomeSexChromIndividual::ploidy(int)



void QuickAutosomeSexChromIndividual::Print(ostream& outFile, char sepChar){
	aString outStr("ID: "); outStr.cat(id_); outStr+sepChar;
	char sexCh = (sex_==M) ? 'M':'F';
	int ploidyX = (sex_==M) ? 1:2;
	(outStr+sexCh)+sepChar;
	outFile<<outStr<<endl;
	outStr=sepChar; (outStr+"A:")+sepChar;
	int loc,c;
	for(loc=0;loc<nloci_;++loc){
		for(c=0;c<2;++c){
			outStr.cat(genes_[loc*2+c]);}
		if(loc<nloci_-1) outStr+sepChar;
		}
	outFile<<outStr<<endl;
	outStr=sepChar; (outStr+"X:")+sepChar;
	for(loc=0;loc<nlociX_;++loc){
		for(c=0;c<ploidyX;++c){
			outStr.cat(genesX_[loc*ploidyX+c]);}
		if(ploidyX==1) outStr+' ';
		if(loc<nlociX_-1) outStr+sepChar;
		}
	outFile<<outStr<<endl;	
	}//QuickAutosomeSexChromIndividual::Print(ostream&, char)


void QuickAutosomeSexChromIndividual::SetGenome(GeneContainerCore& gc){
	//initializes genome using MultiInheritanceModeContainer<int>[mode][ploidy][locus]
	MultiInheritanceModeContainer<int>* mimc = (MultiInheritanceModeContainer<int>*) &gc;
	QuickDiplSexGenIndividual::SetGenome(mimc->diploidData());
	SetSexLinkedGenome(gc);
	}//QuickAutosomeSexChromIndividual::SetGenome(GeneContainerCore& )


void QuickAutosomeSexChromIndividual::SetSexLinkedGenome(GeneContainerCore& gc){
	//only sets the sex-linked part
	//initializes genome using MultiInheritanceModeContainer<int>[mode][ploidy][locus]
	MultiInheritanceModeContainer<int>* mimc = (MultiInheritanceModeContainer<int>*) &gc;
	genderSet oldsex = sex_;
	sex_= mimc->sex_;
	int loc,c,ploidylevelX = (sex_==M) ? 1:2;
	bool sexChange = (oldsex==sex_) ? false:true;
	MultiHaplotypeContainer<int>& sexLinkedGenes = mimc->sexLinkedData();
	if(sexChange || nlociX_ != sexLinkedGenes.loci()){
		nlociX_ = sexLinkedGenes.loci();
		delete genesX_;//replace if different size is needed
		genesX_ = NEW int[ploidylevelX*nlociX_]();
		}
	for(loc = 0;loc<nlociX_;++loc){
		for(c=0;c<ploidylevelX;++c){
			genesX_[loc*ploidylevelX+c] = sexLinkedGenes.haplotype(loc,c);}
		}//loc
	}//QuickAutosomeSexChromIndividual::SetGenome(GeneContainerCore& )


void QuickAutosomeSexChromIndividual::SetGenotypeX(int loc, int allele, int genotype){
	int ploidyX=1;
	if(sex_==F){++ploidyX;}
	if(sex_==M){allele=0;}//haploid mals
	if(loc<nlociX_ && loc>=0 && allele<2 && allele>=0){
		genesX_[loc*ploidyX+allele] = genotype;}
	}




#endif

