/***************************************************
*																														}
*	QuickDiplSexGenIndividual.cpp

	Base Class: Individual																					}
*																														}
*	Handles processes involving individuals with genetic information.
*	Genotype is an array of short integers.
*	Sexual reproduction, but mate identity not stored; must mate at 
*	site of reproduction.
*																														}
****************************************************/
#ifndef QuickDiplSexGenIndividualcp
#define QuickDiplSexGenIndividualcp = 1

# include "QuickDiplSexGenIndividual.h"
#include "Element.cpp"
#include "aList.cpp"
#include "RandomList.h"
#include "GeneContainers.cpp"


		
QuickDiplSexGenIndividual::QuickDiplSexGenIndividual(const int nloci, int* newGenes, long id, Population* site):
		nloci_(nloci), genes_(0){
	//done this way to permit multiple inheritance for derived genetic individuals
	id_=id;
	pop_=site;
	genes_= NEW int[2*nloci_]();
	int loc,c;
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			genes_[loc*2+c] = newGenes[loc*2+c];
			}}
	if(pop_ != NULL){moveRate_=pop_->moveRate(B,1);} else {moveRate_=zero;}
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	sex_=B;
	}//end constructor QuickDiplSexGenIndividual::QuickDiplSexGenIndividual


		
QuickDiplSexGenIndividual::QuickDiplSexGenIndividual(long id, Population* site):
		nloci_(0), genes_(0){
	//must add genes -- called during reproduction
	id_=id;
	pop_=site;
	if(pop_ != NULL){moveRate_=pop_->moveRate(B,1);} else {moveRate_=zero;}
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	sex_=B;
	}//end constructor QuickDiplSexGenIndividual::QuickDiplSexGenIndividual
	

QuickDiplSexGenIndividual::QuickDiplSexGenIndividual(void):
		nloci_(), genes_(0){
	//must add genes -- called during reproduction
	id_=0;
	pop_=NULL;
	moveRate_=zero;
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	sex_=B;
	}//end constructor QuickDiplSexGenIndividual::QuickDiplSexGenIndividual


QuickDiplSexGenIndividual::QuickDiplSexGenIndividual(QuickDiplSexGenIndividual& indiv){
	Individual::Copy((Individual*) &indiv);
	moveRate_ = indiv.moveRate_;
	nloci_ = indiv.loci();
	genes_ = NEW int[2*nloci_];
	int loc,c;
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			genes_[loc*2+c] = indiv.genes_[loc*2+c];
			}}
	moveRate_=indiv.moveRate_;
	alive_=indiv.alive_;
	age_=indiv.age_;
	causeOfDeath_=indiv.causeOfDeath_;
	movedThisGen_=indiv.movedThisGen_;
	lifeStage_=indiv.lifeStage_;
	sex_=indiv.sex_;
	}//end constructor QuickDiplSexGenIndividual::QuickDiplSexGenIndividual
							
							
QuickDiplSexGenIndividual::QuickDiplSexGenIndividual(QuickDiplSexGenIndividual* indiv){
	Individual::Copy((Individual*) indiv);
	moveRate_ = indiv->moveRate_;
	nloci_ = indiv->loci();
	genes_ = NEW int[2*nloci_];
	int loc,c;
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			genes_[loc*2+c] = indiv->genes_[loc*2+c];
			}}
	moveRate_=indiv->moveRate_;
	alive_=indiv->alive_;
	age_=indiv->age_;
	causeOfDeath_=indiv->causeOfDeath_;
	movedThisGen_=indiv->movedThisGen_;
	lifeStage_=indiv->lifeStage_;
	sex_=indiv->sex_;
	}//end constructor QuickDiplSexGenIndividual::QuickDiplSexGenIndividual


		
QuickDiplSexGenIndividual::~QuickDiplSexGenIndividual(void) {
		delete [] genes_; genes_ = NULL;
		}//end destructor QuickDiplSexGenIndividual::QuickDiplSexGenIndividual


Individual* QuickDiplSexGenIndividual::Duplicate (void){
	//returns a duplicate, used for virtual copy construction
	return  NEW QuickDiplSexGenIndividual(*this);
	}//end  QuickDiplSexGenIndividual::Duplicate


Individual* QuickDiplSexGenIndividual::Duplicate (long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	return  NEW QuickDiplSexGenIndividual(id, site);
	}//end  QuickDiplSexGenIndividual::Duplicate


QuickDiplSexGenIndividual* QuickDiplSexGenIndividual::Duplicate (int* newGenes, long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	QuickDiplSexGenIndividual* gi =  NEW QuickDiplSexGenIndividual(id, site);
	gi->nloci_ = nloci_;
	delete [] genes_;
	gi->genes_ = NEW int[2*nloci_]();
	int loc,c;
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			gi->genes_[2*loc+c] = newGenes[2*loc+c];
			}}
	gi->moveRate_=moveRate_;
	gi->alive_=alive_;
	age_=age_;
	gi->causeOfDeath_=causeOfDeath_;
	gi->movedThisGen_=movedThisGen_;
	gi->lifeStage_=lifeStage_;
	return gi;
	}//end  QuickDiplSexGenIndividual::Duplicate


void QuickDiplSexGenIndividual::Copy (Individual* it){
	Individual::Copy(it);
	moveRate_ = it->moveRate_;
	delete [] genes_; genes_ = NULL; nloci_=0;
	}//end QuickDiplSexGenIndividual::Copy 

void QuickDiplSexGenIndividual::Copy (QuickDiplSexGenIndividual* it){
	Individual::Copy((Individual*) it);
	moveRate_ = it->moveRate_;
	int c,loc;
	if(nloci_ != it->nloci_){
		nloci_=it->nloci_;
		delete [] genes_;
		genes_=NEW int[2*nloci_]();}
	for(c=0;c<2;++c){
		for(loc=0;loc<nloci_;loc++){
			genes_[loc*2+c] = it->genes_[loc*2+c];}};
	}//end QuickDiplSexGenIndividual::Copy 



void QuickDiplSexGenIndividual::Initialize(const int nloci, int* newGenes, long id, Population* site){
	//done this way to permit multiple inheritance for derived genetic individuals
	id_=id;
	pop_=site;
	if(nloci_!=nloci){
		nloci_=nloci;
		delete [] genes_;
		genes_= NEW int[2*nloci]();}
	int c,loc;
	for(c=0;c<2;++c){
		for(loc=0;loc<nloci_;loc++){
			genes_[loc*2+c] = newGenes[loc*2+c];}};
	if(pop_ != NULL){moveRate_=pop_->moveRate(B,1);} else {moveRate_=zero;}
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	moveRate_=zero;
	lifeStage_=zygote;
	sex_=B;
	}//end QuickDiplSexGenIndividual::Initialize



void QuickDiplSexGenIndividual::Initialize(const int nloci, int* newGenes){
	//done this way to permit multiple inheritance for derived genetic individuals
	id_=-1;
	pop_=NULL;
	if(nloci_!=nloci){
		nloci_=nloci;
		delete [] genes_;
		genes_= NEW int[2*nloci]();}
	int c,loc;
	for(c=0;c<2;++c){
		for(loc=0;loc<nloci_;loc++){
			genes_[loc*2+c] = newGenes[loc*2+c];}};
	moveRate_=zero;
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	sex_=B;
	}//end QuickDiplSexGenIndividual::Initialize







int	QuickDiplSexGenIndividual::AllelePresent (int loc, aString& alleleVal){
	int a = alleleVal.toInt();
	if ((genes_[loc*2+0] == a) || (genes_[loc*2+1] == a)) return 1;
	return 0;
	}


void QuickDiplSexGenIndividual::Clone (long& idCount, Ran3List& ran, aList<long double>& specialInfo)	{
	//installs a new baby in the population
//	QuickDiplSexGenIndividual* baby = NEW QuickDiplSexGenIndividual(++idCount);
	QuickDiplSexGenIndividual* baby = NEW QuickDiplSexGenIndividual(*this);
	baby->id_ = ++idCount;
	baby->pop_ = pop_;
	baby->age_ = 1;
	baby->alive_ = 1;
	baby->lifeStage_ = zygote;
	baby->moveRate_=moveRate_;
//	baby->nloci_ = nloci_;
//	baby->genes_ = NEW int[2*nloci_]();
//	int loc,c;
//	for(loc=0;loc<nloci_;loc++){
//		for(c=0;c<2;++c){
//			baby->genes_[loc*2+c] = genes_[loc*2+c];
//			}}
	baby->Mutate(ran,specialInfo);
	baby->causeOfDeath_=0;
	baby->movedThisGen_=0;
	pop_->Install( (Individual*) baby);
	}//end QuickDiplSexGenIndividual::Clone
	

void QuickDiplSexGenIndividual::Clone (Individual* mom, Ran3List& ran, aList<long double>& specialInfo)	{
	//copies mom as a new baby *this
	//obtain all information directly from mom
	Individual::Clone(mom,ran,specialInfo);// copies location etc. details
	//mom must be a QuickDiplSexGenIndividual too
	QuickDiplSexGenIndividual* gMom = (QuickDiplSexGenIndividual*) mom->GetPointerToDerivedSelf();
	nloci_ = gMom->nloci_;
	delete [] genes_;
	genes_ = NEW int[2*nloci_]();
	int loc,c;
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			genes_[loc*2+c] = gMom->genes_[loc*2+c];
			}}
//	Mutate(ran,mutationRate,sigma);
	Mutate(ran,specialInfo);
	moveRate_=gMom->moveRate_;
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	pop_->Install( (Individual*) this);
	}//end QuickDiplSexGenIndividual::Clone



int	QuickDiplSexGenIndividual::CountAlleles (int loc, aString& alleleVal){
	int c, a = alleleVal.toInt();
	int nAlleles=0;
	for(c=0;c<2;+c){
		if (genes_[loc*2+c] == a) nAlleles++;}
	return nAlleles;
	}//end QuickDiplSexGenIndividual::CountAlleles




aString* QuickDiplSexGenIndividual::GenotypeString(char* separator){
	aString* gStr = NEW aString(), *lgStr;
	int loc;
	for(loc=0;loc<nloci_;++loc){
		lgStr = GenotypeString(loc, separator);
		if(loc>0){lgStr->cat(separator);}
		gStr->cat(*lgStr);
		delete lgStr;}
	return gStr;
	}//end QuickDiplSexGenIndividual::GenotypeString
	


aString* QuickDiplSexGenIndividual::GenotypeString(int locus, char* /*separator*/){
	//alleles separated by parentheses; e.g. (1)(1) or (1)(0)
	aString* gStr = NEW aString();
	int c;
	for(c=0;c<2;++c){
		aString cgStr("(");
		cgStr.cat(genes_[locus*2+c]);
		cgStr.cat(")");
		gStr->cat(cgStr);
		}
	return gStr;
	}//end QuickDiplSexGenIndividual::GenotypeString
	
	
	
aString* QuickDiplSexGenIndividual::GenotypeString(int locus, int alleleNum, char* /*separator*/){
	return NEW aString(genes_[locus*2+alleleNum]);
	}//end QuickDiplSexGenIndividual::GenotypeString





void QuickDiplSexGenIndividual::GetGenome(GeneContainerCore& gc){
	//refills container class as a MultiInheritanceModeContainer<int>[mode=1][ploidy][locus]
	MultiInheritanceModeContainer<int>* mimc = (MultiInheritanceModeContainer<int>*) &gc;
	mimc->Clear();
	mimc->sex_ = B;
	MultiHaplotypeContainer<int>& mhc = mimc->diploidData();
	int c,loc;
	mhc.ploidy_ = QuickDiplSexGenIndividual::ploidy();
	for(c=0;c<QuickDiplSexGenIndividual::ploidy();++c){
		mhc.geneSet_.Append(NEW HaplotypeContainer<int>(ID(),QuickDiplSexGenIndividual::loci()));
		for(loc = 0;loc<QuickDiplSexGenIndividual::loci();++loc){
			mhc[c][loc] = genes_[loc*QuickDiplSexGenIndividual::ploidy()+c];}
		}//c
	mhc.SetID(id_);
	}//QuickDiplSexGenIndividual::GetGenome(GeneContainerCore& gc)



int* QuickDiplSexGenIndividual::GetGenotype (void) {
	//returns a copy that must be disposed of elsewhere
	int *genesCopy = NEW int[2*nloci_]();
	int loc,c;
	for(loc=0;loc<nloci_;loc++){
		for(c=0;c<2;++c){
			genesCopy[loc*2+c] = genes_[loc*2+c];
			}}
	return genesCopy;
	}//end QuickDiplSexGenIndividual::GetGenotype


int QuickDiplSexGenIndividual::GetGenotype (int locus, int allele) {
	if(locus>=nloci_ || allele>=ploidy(locus)) return -1;
	return genes_[locus*ploidy(locus)+allele];
	}//end QuickDiplAsexualGenIndividual::GetGenotype(int,int)






void QuickDiplSexGenIndividual::IIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran, LandScapeInitializer01& init, aList<long double>& extraInfo){
	Individual::IIndividual(gender,lifeStage,rate, ran,init,extraInfo);
	int species = (int)extraInfo[0];
	int pop =     (int)extraInfo[1];
	int indiv =   (int)extraInfo[2];
	MultiInheritanceModeContainer<int>* mimc = (MultiInheritanceModeContainer<int>*) &(init.initMultiInheritanceGenotypes_[species][pop][indiv]);
	MultiHaplotypeContainer<int>& mhc = mimc->diploidData();
	QuickDiplSexGenIndividual::SetGenome(mhc);
	}//end QuickDiplSexGenIndividual::IIndividual


int QuickDiplSexGenIndividual::IsHeterozygote (int loc){
	if (genes_[loc*2+0] == genes_[loc*2+1] ) return 0;
	return 1;
	}//end QuickDiplSexGenIndividual::IsHeterozygote(int)
	
	
int QuickDiplSexGenIndividual::IsHeterozygote (int loc, aString& alleleVal){
	int a = alleleVal.toInt();
	if ((genes_[loc*2+0] == a) && (genes_[loc*2+1] != a)) return 1;
	if ((genes_[loc*2+0] != a) && (genes_[loc*2+1] == a)) return 1;
	return 0;
	}//end QuickDiplSexGenIndividual::IsHeterozygote(int,aString&)



int QuickDiplSexGenIndividual::loci(void){
	return nloci_;
	}//end QuickDiplSexGenIndividual::loci()


void QuickDiplSexGenIndividual::MakeABaby (long& idCount, Ran3List& ran, aList<long double>& specialInfo){
	//installs a new baby in the population
	QuickDiplSexGenIndividual* baby = NEW QuickDiplSexGenIndividual(++idCount);
	baby->age_ = 1;
	baby->alive_ = 1;
	baby->sex_=B;
	baby->lifeStage_ = zygote;
	baby->moveRate_ = moveRate_;
	long dadnum = ran.longBetween(0,pop_->fathers()-1);
	Individual& idad = pop_->father(dadnum);
	QuickDiplSexGenIndividual* dad = (QuickDiplSexGenIndividual*) idad.GetPointerToDerivedSelf();
	baby->nloci_ = nloci_;
	baby->genes_ = NEW int[2*nloci_]();
	int loc,m,d;
	for(loc=0;loc<nloci_;++loc){//loci unlinked
		m=ran.longBetween(0,1);//egg meiosis
		baby->genes_[loc*2+0] = genes_[loc*2+m];
		d=ran.longBetween(0,1);//sperm meiosis
		baby->genes_[loc*2+1] = dad->genes_[loc*2+d];
		}
	baby->Mutate(ran, specialInfo);
	pop_->Install(baby);
	}  //end QuickDiplSexGenIndividual::MakeABaby




void QuickDiplSexGenIndividual::Mutate (Ran3& ran, aList<long double>& specialInfo){
							//specialInfo[0] = mutationRate
							//specialInfo[1] = 2-allele model (0), infinite-allele model (1)
							//specialInfo[2] = infinite allele counter for locus 1 (increment for each new mutation)
							//specialInfo[3] =    " locus 2, etc.
	//mutate the genome if warranted
	if(specialInfo.len()==0) return;
	if(specialInfo[0] == zero) return;
	long double probMutant, mutationRate = specialInfo[0], ranNum = ran();
	bool biallelic = true;
	if(specialInfo.len()>1){
		if(specialInfo[1]!=0){
			biallelic=false;//infinite allele model
			}}
	//faster method: find out if any mutations exist by using the probability of no mutations overall
		//quit if no mutation turns up
		//if a mutation turns up, repeat for the remaining gene copies
	int nmutations=0;
	long alleleCopy,m,mutantCopy,mutantCopyLocus, runningcopy=0;
	long double allelecopies = two * (long double)nloci_;
	for(alleleCopy = 0;alleleCopy<2*nloci_;++alleleCopy){//all gene copies --> one-indexed
		if(alleleCopy==0){//binomial
				probMutant = exp(allelecopies * ln(one - mutationRate));}
			else if (alleleCopy==1){
				probMutant += allelecopies * mutationRate * exp((allelecopies - one) * ln(one - mutationRate));
				}
			else{//really really rare unless mutationRate is really high
				probMutant = BinomialDistributionCDF(2*nloci_,alleleCopy,mutationRate);
				}
		if(ranNum>=probMutant){
				nmutations++;}
			else{break;}
		}//alleleCopy
	if(nmutations==1){
			mutantCopy = ran.longBetween(0,2*nloci_-1);
				if(biallelic){
						if(genes_[mutantCopy]==0){genes_[mutantCopy]=1;}else{genes_[mutantCopy]=0;}
						}
					else{//infinite allele
						mutantCopyLocus = mutantCopy/2;
						genes_[mutantCopy] = (long) ++specialInfo[2+mutantCopyLocus];
						}
			}
		else if (nmutations>1){
			aList<int> mutatedAlleleCopies;//ordered 
			for(m=nmutations-1;m>=0;--m){//change a random allele without replacement
				mutantCopy=-1;
				do{
					mutantCopy=ran.longBetween(0,2*nloci_-1);
					}while(mutatedAlleleCopies.Includes(mutantCopy));
				if(biallelic){
						if(genes_[mutantCopy]==0){genes_[mutantCopy]=1;}else{genes_[mutantCopy]=0;}
						}
					else{//infinite allele
						mutantCopyLocus = mutantCopy/2;
						genes_[mutantCopy] = (long) ++specialInfo[2+mutantCopyLocus];
						}
				if(m>0){
					mutatedAlleleCopies.Append(mutantCopy);}
				}//m
			}
		else{}//skip if no mutations
	}//end QuickDiplSexGenIndividual::Mutate()


long double QuickDiplSexGenIndividual::phenotype(int i){
	//assign tha allele state 1 to dominance and return the pattern
	if((genes_[i*2+0]==0)&&(genes_[i*2+1]==0)) return zero;
	return one;
	}//end QuickDiplSexGenIndividual::phenotype(int)


int QuickDiplSexGenIndividual::ploidy(void){
	return 2;
	}//end QuickDiplSexGenIndividual::ploidy(int)


void QuickDiplSexGenIndividual::PrintGenes(ostream& outFile, int /*alleleLen*/, int codeLen) {
	//create geneStr before calling this function
	aString *geneStr = GenotypeString(" ");
	aString idStr(id_);
	idStr.PrependCharToLen(' ',codeLen);
	idStr.cat(' ');
	idStr.cat(*geneStr);
	outFile<<idStr<<endl;
	delete geneStr;
	}//end QuickDiplSexGenIndividual::PrintGenes


/*
//void QuickDiplSexGenIndividual::Reproduce (long& idCount, Ran3List& ran, double mutationRate, double sigma)	{
void QuickDiplSexGenIndividual::Reproduce (long& idCount, Ran3List& ran, aList<double>& reproductionInfo)	{
	//installs new babies in the population
	//each hermaphrodite reproduces
	//according to its individual fecundity
	//derived classes handle separate genders, genetics, etc.
	//IDEAL HERMAPHRODITES, SO ALL OFFSPRING HAVE NEWLY CHOSEN FATHERS
	if(sex_==M) return;
	int f=Fecundity(ran);
	for (int b=0;b<f;++b){
//		MakeABaby(idCount,ran,mutationRate,sigma);}
		MakeABaby(idCount,ran,		MakeABaby(idCount,ran,mutationRate,sigma);}
);}
	}//end QuickDiplSexGenIndividual::Reproduce
*/

void QuickDiplSexGenIndividual::SetGenome(GeneContainerCore& gc){
	//initializes genome using MultiHaplotypeContainer<int>[ploidy][locus]
	MultiHaplotypeContainer<int>* mhc = (MultiHaplotypeContainer<int>*) &gc;
	int loc,c,ploidylevel = mhc->ploidy();
	if(ploidylevel != 2){
		aError("Error using QuickDiplSexGenIndividual::SetGenome(GeneContainerCore&): must be diploid");
		return;}
	if(nloci_ != mhc->loci()){
		nloci_ = mhc->loci();
		delete genes_;
		genes_ = NEW int[ploidylevel*nloci_]();
		}
	for(loc = 0;loc<nloci_;++loc){
		for(c=0;c<ploidylevel;++c){
			genes_[loc*ploidylevel+c] = mhc->haplotype(loc,c);}
		}//loc
	}//QuickDiplSexGenIndividual::SetGenome(GeneContainerCore& )


void QuickDiplSexGenIndividual::SetGenotype(int loc, int allele, int genotype){
	if(loc<nloci_ && loc>=0 && allele<2 && allele>=0){
		genes_[loc*2+allele] = genotype;}
	}//QuickDiplSexGenIndividual::SetGenotype(int, int, int)


void QuickDiplSexGenIndividual::SetGenotype(int *genes, int nloci){//genes must be the same size array or hell breaks loose!
	if(nloci != nloci_){
		aError("ERROR: setting a genotype of the wrong size in QuickDiplSexGenIndividual");}
	int loc;
	for(loc=0;loc<nloci_;++loc){
		genes_[loc*2] = genes[loc*2];
		genes_[loc*2+1] = genes[loc*2+1];
		}
	}//QuickDiplSexGenIndividual::SetGenotype(int*, int)


void QuickDiplSexGenIndividual::SetGenotype(ObjList<aList<int> >& genes){//genes must be the same size array or hell breaks loose!
	if(genes.len() != nloci_){
		aError("ERROR: setting a genotype of the wrong size in QuickDiplSexGenIndividual");}
	int loc, c;
	for(loc=0;loc<nloci_;++loc){
		for(c=0;c<2;++c){
			genes_[loc*2+c] = genes[loc][c];
			}}
	}//QuickDiplSexGenIndividual::SetGenotype(int*, int)



#endif

