/***************************************************
*																														}
*	QuickHaplSexGenIndividual.cpp

	Base Class: Individual																					}
*																														}
*	Handles processes involving individuals with genetic information.
*	Genotype is an array of short integers.
*	Sexual reproduction, but mate identity not stored; must mate at 
*	site of reproduction.
*																														}
****************************************************/
#ifndef QuickHaplSexGenIndividualcp
#define QuickHaplSexGenIndividualcp = 1

# include "QuickHaplSexGenIndividual.h"
#include "Element.cpp"
#include "aList.cpp"
#include "RandomList.h"
#include "GeneContainers.cpp"


		
QuickHaplSexGenIndividual::QuickHaplSexGenIndividual(const int nloci, int* newGenes, long id, Population* site):
		nloci_(nloci), genes_(0){
	//done this way to permit multiple inheritance for derived genetic individuals
	id_=id;
	pop_=site;
	genes_= NEW int[nloci_]();
	int loc;
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = newGenes[loc];}
	if(pop_ != NULL){moveRate_=pop_->moveRate(B,1);} else {moveRate_=zero;}
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	}//end constructor QuickHaplSexGenIndividual::QuickHaplSexGenIndividual


		
QuickHaplSexGenIndividual::QuickHaplSexGenIndividual(long id, Population* site):
		nloci_(0), genes_(0){
	//must add genes -- called during reproduction
	id_=id;
	pop_=site;
	if(pop_ != NULL){moveRate_=pop_->moveRate(B,1);} else {moveRate_=zero;}
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	}//end constructor QuickHaplSexGenIndividual::QuickHaplSexGenIndividual
	

QuickHaplSexGenIndividual::QuickHaplSexGenIndividual(void):
		nloci_(), genes_(0){
	//must add genes -- called during reproduction
	id_=0;
	pop_=NULL;
	moveRate_=zero;
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	}//end constructor QuickHaplSexGenIndividual::QuickHaplSexGenIndividual


QuickHaplSexGenIndividual::QuickHaplSexGenIndividual(QuickHaplSexGenIndividual& indiv){
	Individual::Copy((Individual*) &indiv);
	nloci_ = indiv.loci();
	genes_ = NEW int[nloci_];
	int loc;
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = indiv.genes_[loc];}
	moveRate_=indiv.moveRate_;
	alive_=indiv.alive_;
	age_=indiv.age_;
	causeOfDeath_=indiv.causeOfDeath_;
	movedThisGen_=indiv.movedThisGen_;
	lifeStage_=indiv.lifeStage_;
	}//end constructor QuickHaplSexGenIndividual::QuickHaplSexGenIndividual
							
							
QuickHaplSexGenIndividual::QuickHaplSexGenIndividual(QuickHaplSexGenIndividual* indiv){
	Individual::Copy((Individual*) indiv);
	nloci_ = indiv->loci();
	genes_ = NEW int[nloci_];
	int loc;
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = indiv->genes_[loc];}
	moveRate_=indiv->moveRate_;
	alive_=indiv->alive_;
	age_=indiv->age_;
	causeOfDeath_=indiv->causeOfDeath_;
	movedThisGen_=indiv->movedThisGen_;
	lifeStage_=indiv->lifeStage_;
	}//end constructor QuickHaplSexGenIndividual::QuickHaplSexGenIndividual


		
QuickHaplSexGenIndividual::~QuickHaplSexGenIndividual(void) {
		delete [] genes_; genes_ = NULL;
		}//end destructor QuickHaplSexGenIndividual::QuickHaplSexGenIndividual


Individual* QuickHaplSexGenIndividual::Duplicate (void){
	//returns a duplicate, used for virtual copy construction
	return  NEW QuickHaplSexGenIndividual();
	}//end  QuickHaplSexGenIndividual::Duplicate


Individual* QuickHaplSexGenIndividual::Duplicate (long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	return  NEW QuickHaplSexGenIndividual(id, site);
	}//end  QuickHaplSexGenIndividual::Duplicate


QuickHaplSexGenIndividual* QuickHaplSexGenIndividual::Duplicate (int* newGenes, long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	QuickHaplSexGenIndividual* gi =  NEW QuickHaplSexGenIndividual(id, site);
	gi->nloci_ = nloci_;
	delete [] genes_;
	gi->genes_ = NEW int[nloci_]();
	int loc;
	for(loc=0;loc<nloci_;loc++){
		gi->genes_[loc] = newGenes[loc];}
	gi->moveRate_=moveRate_;
	gi->alive_=alive_;
	age_=age_;
	gi->causeOfDeath_=causeOfDeath_;
	gi->movedThisGen_=movedThisGen_;
	gi->lifeStage_=lifeStage_;
	return gi;
	}//end  QuickHaplSexGenIndividual::Duplicate


void QuickHaplSexGenIndividual::Copy (Individual* it){
	Individual::Copy(it);
	delete [] genes_; genes_ = NULL; nloci_=0;
	}//end QuickHaplSexGenIndividual::Copy 

void QuickHaplSexGenIndividual::Copy (QuickHaplSexGenIndividual* it){
	Individual::Copy((Individual*) it);
	int loc;
	if(nloci_ != it->nloci_){
		nloci_=it->nloci_;
		delete [] genes_;
		genes_=NEW int[nloci_]();}
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = it->genes_[loc];}
	}//end QuickHaplSexGenIndividual::Copy 



void QuickHaplSexGenIndividual::Initialize(const int nloci, int* newGenes, long id, Population* site){
	//done this way to permit multiple inheritance for derived genetic individuals
	id_=id;
	pop_=site;
	if(nloci_!=nloci){
		nloci_=nloci;
		delete [] genes_;
		genes_= NEW int[nloci]();}
	int loc;
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = newGenes[loc];}
	if(pop_ != NULL){moveRate_=pop_->moveRate(B,1);} else {moveRate_=zero;}
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	}//end QuickHaplSexGenIndividual::Initialize



void QuickHaplSexGenIndividual::Initialize(const int nloci, int* newGenes){
	//done this way to permit multiple inheritance for derived genetic individuals
	id_=-1;
	pop_=NULL;
	if(nloci_!=nloci){
		nloci_=nloci;
		delete [] genes_;
		genes_= NEW int[nloci]();}
	int loc;
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = newGenes[loc];}
	moveRate_=zero;
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	}//end QuickHaplSexGenIndividual::Initialize







int	QuickHaplSexGenIndividual::AllelePresent (int loc, aString& alleleVal){
	int a = alleleVal.toInt();
	if ((genes_[loc] == a) || (genes_[loc] == a)) return 1;
	return 0;
	}


void QuickHaplSexGenIndividual::Clone (long& idCount, Ran3List& ran, aList<long double>& specialInfo)	{
	//installs a new baby in the population
	QuickHaplSexGenIndividual* baby = NEW QuickHaplSexGenIndividual(++idCount);
	baby->id_ = ++idCount;
	baby->pop_ = pop_;
	baby->age_ = 1;
	baby->alive_ = 1;
	baby->lifeStage_ = zygote;
	baby->moveRate_=moveRate_;
	baby->nloci_ = nloci_;
	baby->genes_ = NEW int[nloci_]();
	int loc;
	for(loc=0;loc<nloci_;loc++){
			baby->genes_[loc] = genes_[loc];}
	baby->Mutate(ran,specialInfo);
	baby->causeOfDeath_=0;
	baby->movedThisGen_=0;
	pop_->Install( (Individual*) baby);
	}//end QuickHaplSexGenIndividual::Clone
	

void QuickHaplSexGenIndividual::Clone (Individual* mom, Ran3List& ran, aList<long double>& specialInfo)	{
	//copies mom as a new baby *this
	//obtain all information directly from mom
	Individual::Clone(mom,ran,specialInfo);// copies location etc. details
	//mom must be a QuickHaplSexGenIndividual too
	QuickHaplSexGenIndividual* gMom = (QuickHaplSexGenIndividual*) mom->GetPointerToDerivedSelf();
	nloci_ = gMom->nloci_;
	delete [] genes_;
	genes_ = NEW int[nloci_]();
	int loc;
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = gMom->genes_[loc];}
//	Mutate(ran,mutationRate,sigma);
	Mutate(ran,specialInfo);
	moveRate_=gMom->moveRate_;
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	pop_->Install( (Individual*) this);
	}//end QuickHaplSexGenIndividual::Clone



int	QuickHaplSexGenIndividual::CountAlleles (int loc, aString& alleleVal){
	int a = alleleVal.toInt();
	int nAlleles=0;
	if (genes_[loc] == a) nAlleles++;
	return nAlleles;
	}//end QuickHaplSexGenIndividual::CountAlleles




aString* QuickHaplSexGenIndividual::GenotypeString(char* separator){
	aString* gStr = NEW aString(), *lgStr;
	int loc;
	for(loc=0;loc<nloci_;++loc){
		lgStr = GenotypeString(loc, separator);
		if(loc>0){lgStr->cat(separator);}
		gStr->cat(*lgStr);
		delete lgStr;}
	return gStr;
	}//end QuickHaplSexGenIndividual::GenotypeString
	


aString* QuickHaplSexGenIndividual::GenotypeString(int locus, char* /*separator*/){
	//alleles separated by parentheses; e.g. (1)(1) or (1)(0)
	aString* gStr = NEW aString();
	aString cgStr("(");
	cgStr.cat(genes_[locus]);
	cgStr.cat(")");
	gStr->cat(cgStr);
	return gStr;
	}//end QuickHaplSexGenIndividual::GenotypeString
	
	
	
aString* QuickHaplSexGenIndividual::GenotypeString(int locus, int /*alleleNum*/, char* /*separator*/){
	return NEW aString(genes_[locus]);
	}//end QuickHaplSexGenIndividual::GenotypeString





void QuickHaplSexGenIndividual::GetGenome(GeneContainerCore& gc){
	//refills container class as a MultiHaplotypeContainer<int>[ploidy][locus]
	HaplotypeContainer<int>* hc = (HaplotypeContainer<int>*) &gc;
	hc->Clear();
//	mhc->ploidy_ = QuickHaplSexGenIndividual::ploidy();
	int loc;
	hc->haplotype_=NEW int[nloci_];
	for(loc = 0;loc<nloci_;++loc){
		(*hc)[loc] = genes_[loc];}
	hc->SetID(id_);
	}//QuickHaplSexGenIndividual::GetGenome(GeneContainerCore& gc)



int* QuickHaplSexGenIndividual::GetGenotype (void) {
	//returns a copy that must be disposed of elsewhere
	int *genesCopy = NEW int[nloci_]();
	int loc;
	for(loc=0;loc<nloci_;loc++){
		genesCopy[loc] = genes_[loc];}
	return genesCopy;
	}//end QuickHaplSexGenIndividual::GetGenotype



int QuickHaplSexGenIndividual::GetGenotype (int locus, int /*allele*/) {
	if(locus>=nloci_) return -1;
	return genes_[locus];
	}//end QuickHaplSexGenIndividual::GetGenotype




void QuickHaplSexGenIndividual::IIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran, LandScapeInitializer01& init, aList<long double>& extraInfo){
	Individual::IIndividual(gender,lifeStage,rate, ran,init,extraInfo);
	int species = (int)extraInfo[0];
	int pop =     (int)extraInfo[1];
	int indiv =   (int)extraInfo[2];
	MultiInheritanceModeContainer<int>* mimc = (MultiInheritanceModeContainer<int>*) &(init.initMultiInheritanceGenotypes_[species][pop][indiv]);
	MultiHaplotypeContainer<int>& mhc = mimc->haploidData();
	QuickHaplSexGenIndividual::SetGenome(mhc);
	}//end QuickHaplSexGenIndividual::IIndividual


int QuickHaplSexGenIndividual::loci(void){
	return nloci_;
	}//end QuickHaplSexGenIndividual::loci()


void QuickHaplSexGenIndividual::MakeABaby (long& idCount, Ran3List& ran, aList<long double>& specialInfo){
	//installs a new baby in the population
	//sexual reproduction -- all loci unlinked
	QuickHaplSexGenIndividual* baby = NEW QuickHaplSexGenIndividual(++idCount);
	baby->age_ = 1;
	baby->alive_ = 1;
	baby->sex_=B;
	baby->lifeStage_ = zygote;
	baby->moveRate_ = moveRate();
	long dadnum = ran.longBetween(0,pop_->fathers()-1);
	Individual& idad = pop_->father(dadnum);
	QuickHaplSexGenIndividual* dad = (QuickHaplSexGenIndividual*) idad.GetPointerToDerivedSelf();
	baby->nloci_ = nloci_;
	baby->genes_ = NEW int[nloci_]();
	int loc;
	for(loc=0;loc<nloci_;++loc){//loci unlinked
		if(ran()<half){//mom's gene
				baby->genes_[loc] = genes_[loc];}
			else{
				baby->genes_[loc] = dad->genes_[loc];}
		}
	baby->Mutate(ran, specialInfo);
	pop_->Install(baby);
	}  //end QuickHaplSexGenIndividual::MakeABaby




void QuickHaplSexGenIndividual::Mutate (Ran3& ran, aList<long double>& specialInfo){
	//mutate the genome if warranted
	if(specialInfo.len()==0) return;
	if(specialInfo[0] == zero) return;
	long double probnMutations, mutationRate = specialInfo[0], ranNum = ran();
	int nmutations=0;
	int loc,m,mloc;
	for(loc = nloci_;loc>0;--loc){
		probnMutations = exp(loc*ln(one - mutationRate));
		if(ranNum>probnMutations){
				nmutations++;}
			else{break;}
		}
	for(m=0;m<nmutations;++m){//change a random allele
		mloc=ran.longBetween(0,nloci_-1);
		if(genes_[mloc]==0){genes_[mloc]=1;}else{genes_[mloc]=0;}
		}
	}//end QuickHaplSexGenIndividual::Mutate()


long double QuickHaplSexGenIndividual::phenotype(int i){
	//assign tha allele state 1 to dominance and return the pattern
	if(genes_[i]==0) return zero;
	return one;
	}//end QuickHaplSexGenIndividual::phenotype(int)


int QuickHaplSexGenIndividual::ploidy(void){
	return 1;
	}//end QuickHaplSexGenIndividual::ploidy(int)


void QuickHaplSexGenIndividual::PrintGenes(ostream& outFile, int /*alleleLen*/, int codeLen) {
	//create geneStr before calling this function
	aString *geneStr = GenotypeString(' ');
	aString idStr(id_);
	idStr.PrependCharToLen(' ',codeLen);
	idStr.cat(' ');
	idStr.cat(*geneStr);
	outFile<<idStr<<endl;
	delete geneStr;
	}//end QuickHaplSexGenIndividual::PrintGenes



void QuickHaplSexGenIndividual::SetGenome(GeneContainerCore& gc){
	//initializes genome using MultiHaplotypeContainer<int>[ploidy][locus]
	HaplotypeContainer<int>* hc = (HaplotypeContainer<int>*) &gc;
	int loc;
	if(nloci_ != hc->loci()){
		nloci_ = hc->loci();
		delete genes_;
		genes_ = NEW int[nloci_]();
		}
	for(loc = 0;loc<nloci_;++loc){
		genes_[loc] = hc->haplotype(loc);}
	}//QuickHaplSexGenIndividual::SetGenome(GeneContainerCore& )





#endif

