/***************************************************
*																														
*	QuickHaplSexGenIndividual_LD.cpp

	Base Class: Individual																					
*																														
*	Handles processes involving individuals with genetic information.
*	Genotype is an array of short integers.
*	Sexual reproduction, but mate identity not stored; must mate at 
*	site of reproduction.
*																														
****************************************************/
#ifndef QuickHaplSexGenIndividual_LDcp
#define QuickHaplSexGenIndividual_LDcp = 1

# include "QuickHaplSexGenIndividual_LD.h"
#include "Element.cpp"
#include "aList.cpp"
#include "RandomList.h"
#include "GeneContainers.cpp"


void PhenotypeOneToOne(QuickHaplSexGenIndividual_LD& hapGuy, aList<long double>& ){
	//one gene = one trait - default phenotye function
	int loc;
	if(hapGuy.ntraits_!=hapGuy.nloci_){
		aError("ERROR: PhenotypeOneToOne(): ntraits != nloci");}
	for(loc=0;loc<hapGuy.nloci_;++loc){
		hapGuy.traits_[loc] = hapGuy.genes_[loc];}
	hapGuy.phenotypeSet_=true;
	}//PhenotypeOneToOne





		
QuickHaplSexGenIndividual_LD::QuickHaplSexGenIndividual_LD(const int nloci, long double* newGenes, 
													 const int ntraits, long double* newTraits, PhenotypeFunc thePhenotypeFunction, long id, 
													 Population* site):
		nloci_(nloci), genes_(0),ntraits_(ntraits), traits_(0), phenotypeSet_(false){
	//done this way to permit multiple inheritance for derived genetic individuals
	id_=id;
	pop_=site;
	genes_= NEW long double[nloci_]();
	traits_= NEW long double[ntraits_]();
	int loc,ph;
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = newGenes[loc];}
	for(ph=0;ph<ntraits_;ph++){
		traits_[ph] = newTraits[ph];}
	if(pop_ != NULL){moveRate_=pop_->moveRate(B,1);} else {moveRate_=zero;}
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	SetPhenotypeFunction(thePhenotypeFunction);
	}//end constructor QuickHaplSexGenIndividual_LD::QuickHaplSexGenIndividual_LD



		
QuickHaplSexGenIndividual_LD::QuickHaplSexGenIndividual_LD(long id, Population* site):
		nloci_(0), genes_(0), ntraits_(0), traits_(0), phenotypeSet_(false){
	//must add genes -- called during reproduction
	id_=id;
	pop_=site;
	if(pop_ != NULL){moveRate_=pop_->moveRate(B,1);} else {moveRate_=zero;}
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	SetPhenotypeFunction(PhenotypeOneToOne);
	}//end constructor QuickHaplSexGenIndividual_LD::QuickHaplSexGenIndividual_LD
	

QuickHaplSexGenIndividual_LD::QuickHaplSexGenIndividual_LD(void):
		nloci_(), genes_(0), ntraits_(), traits_(0), phenotypeSet_(false){
	//must add genes -- called during reproduction
	id_=0;
	pop_=NULL;
	moveRate_=zero;
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	SetPhenotypeFunction(PhenotypeOneToOne);
	}//end constructor QuickHaplSexGenIndividual_LD::QuickHaplSexGenIndividual_LD


QuickHaplSexGenIndividual_LD::QuickHaplSexGenIndividual_LD(QuickHaplSexGenIndividual_LD& indiv){
	Individual::Copy((Individual*) &indiv);
	nloci_ = indiv.loci();
	genes_ = NEW long double[nloci_];
	ntraits_ = indiv.ntraits_;
	traits_ = NEW long double[ntraits_];
	int loc,tr;
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = indiv.genes_[loc];}
	for(tr=0;tr<ntraits_;tr++){
		traits_[tr] = indiv.traits_[tr];}
	phenotypeSet_ = indiv.phenotypeSet_;
	moveRate_=indiv.moveRate_;
	alive_=indiv.alive_;
	age_=indiv.age_;
	causeOfDeath_=indiv.causeOfDeath_;
	movedThisGen_=indiv.movedThisGen_;
	lifeStage_=indiv.lifeStage_;
	SetPhenotypeFunction(indiv.SetTraits);
	}//end constructor QuickHaplSexGenIndividual_LD::QuickHaplSexGenIndividual_LD
							
							
QuickHaplSexGenIndividual_LD::QuickHaplSexGenIndividual_LD(QuickHaplSexGenIndividual_LD* indiv){
	Individual::Copy((Individual*) indiv);
	nloci_ = indiv->loci();
	genes_ = NEW long double[nloci_];
	ntraits_ = indiv->ntraits_;
	traits_ = NEW long double[ntraits_];
	int loc,tr;
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = indiv->genes_[loc];}
	for(tr=0;tr<ntraits_;tr++){
		traits_[tr] = indiv->traits_[tr];}
	phenotypeSet_ = indiv->phenotypeSet_;
	moveRate_=indiv->moveRate_;
	alive_=indiv->alive_;
	age_=indiv->age_;
	causeOfDeath_=indiv->causeOfDeath_;
	movedThisGen_=indiv->movedThisGen_;
	lifeStage_=indiv->lifeStage_;
	SetPhenotypeFunction(indiv->SetTraits);
	}//end constructor QuickHaplSexGenIndividual_LD::QuickHaplSexGenIndividual_LD


		
QuickHaplSexGenIndividual_LD::~QuickHaplSexGenIndividual_LD(void) {
		delete [] genes_; genes_ = NULL;
		delete [] traits_; traits_ = NULL;
		}//end destructor QuickHaplSexGenIndividual_LD::QuickHaplSexGenIndividual_LD


Individual* QuickHaplSexGenIndividual_LD::Duplicate (void){
	//returns a duplicate, used for virtual copy construction
	return  NEW QuickHaplSexGenIndividual_LD();
	}//end  QuickHaplSexGenIndividual_LD::Duplicate


Individual* QuickHaplSexGenIndividual_LD::Duplicate (long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	return  NEW QuickHaplSexGenIndividual_LD(id, site);
	}//end  QuickHaplSexGenIndividual_LD::Duplicate


QuickHaplSexGenIndividual_LD* QuickHaplSexGenIndividual_LD::Duplicate (long double* newGenes, long double* newTraits, long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	QuickHaplSexGenIndividual_LD* gi =  NEW QuickHaplSexGenIndividual_LD(id, site);
	gi->nloci_ = nloci_;
	delete [] genes_;
	gi->genes_ = NEW long double[nloci_]();
	gi->ntraits_ = ntraits_;
	delete [] traits_;
	gi->traits_ = NEW long double[ntraits_]();
	int loc,tr;
	for(loc=0;loc<nloci_;loc++){
		gi->genes_[loc] = newGenes[loc];}
	for(tr=0;tr<ntraits_;tr++){
		gi->traits_[tr] = newTraits[tr];}
	gi->phenotypeSet_=false;//in case new data included
	gi->moveRate_=moveRate_;
	gi->alive_=alive_;
	gi->age_=age_;
	gi->causeOfDeath_=causeOfDeath_;
	gi->movedThisGen_=movedThisGen_;
	gi->lifeStage_=lifeStage_;
	gi->SetPhenotypeFunction(SetTraits);

	return gi;
	}//end  QuickHaplSexGenIndividual_LD::Duplicate


void QuickHaplSexGenIndividual_LD::Copy (Individual* it){
	Individual::Copy(it);
	delete [] genes_; genes_ = NULL; nloci_=0;
	delete [] traits_; traits_ = NULL; ntraits_=0;
	phenotypeSet_=false;
	}//end QuickHaplSexGenIndividual_LD::Copy 

void QuickHaplSexGenIndividual_LD::Copy (QuickHaplSexGenIndividual_LD* it){
	Individual::Copy((Individual*) it);
	int loc,tr;
	if(nloci_ != it->nloci_){
		nloci_=it->nloci_;
		delete [] genes_;
		genes_=NEW long double[nloci_]();}
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = it->genes_[loc];}
	if(ntraits_ != it->ntraits_){
		ntraits_=it->ntraits_;
		delete [] traits_;
		traits_=NEW long double[ntraits_]();}
	for(tr=0;tr<ntraits_;tr++){
		traits_[tr] = it->traits_[tr];}
	phenotypeSet_=it->phenotypeSet_;
	SetPhenotypeFunction(it->SetTraits);
	}//end QuickHaplSexGenIndividual_LD::Copy 



void QuickHaplSexGenIndividual_LD::Initialize(const int nloci, long double* newGenes, const int ntraits, 
 												long double* newTraits, PhenotypeFunc thePhenotypeFunction, long id, Population* site){
	//done this way to permit multiple inheritance for derived genetic individuals
	id_=id;
	pop_=site;
	if(nloci_!=nloci){
		nloci_=nloci;
		delete [] genes_;
		genes_= NEW long double[nloci]();}
	if(ntraits_!=ntraits){
		ntraits_=ntraits;
		delete [] traits_;
		traits_= NEW long double[ntraits_]();}
	int loc,tr;
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = newGenes[loc];}
	for(tr=0;tr<ntraits_;tr++){
		traits_[tr] = newTraits[tr];}
	phenotypeSet_=false;//recalculate just in case
	if(pop_ != NULL){moveRate_=pop_->moveRate(B,1);} else {moveRate_=zero;}
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	SetPhenotypeFunction(thePhenotypeFunction);
	}//end QuickHaplSexGenIndividual_LD::Initialize



void QuickHaplSexGenIndividual_LD::Initialize(const int nloci, long double* newGenes, const int ntraits, long double* newTraits,
														PhenotypeFunc thePhenotypeFunction){
	//done this way to permit multiple inheritance for derived genetic individuals
	id_=-1;
	pop_=NULL;
	if(nloci_!=nloci){
		nloci_=nloci;
		delete [] genes_;
		genes_= NEW long double[nloci]();}
	if(ntraits_!=ntraits){
		ntraits_=ntraits;
		delete [] traits_;
		traits_= NEW long double[ntraits_]();}
	int loc,tr;
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = newGenes[loc];}
	for(tr=0;tr<ntraits_;tr++){
		traits_[tr] = newTraits[tr];}
	phenotypeSet_=false;//just in case
	moveRate_=zero;
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	SetPhenotypeFunction(thePhenotypeFunction);
	}//end QuickHaplSexGenIndividual_LD::Initialize







int	QuickHaplSexGenIndividual_LD::AllelePresent (int loc, aString& alleleVal){
	int a = alleleVal.toInt();
	if ((genes_[loc] == a) || (genes_[loc] == a)) return 1;
	return 0;
	}


void QuickHaplSexGenIndividual_LD::Clone (long& idCount, Ran3List& ran, aList<long double>& specialInfo)	{
	//installs a new baby in the population
	QuickHaplSexGenIndividual_LD* baby = NEW QuickHaplSexGenIndividual_LD(++idCount);
	baby->id_ = ++idCount;
	baby->pop_ = pop_;
	baby->age_ = 1;
	baby->alive_ = 1;
	baby->lifeStage_ = zygote;
	baby->moveRate_=moveRate_;
	baby->nloci_ = nloci_;
	baby->genes_ = NEW long double[nloci_]();
	baby->ntraits_ = ntraits_;
	baby->traits_ = NEW long double[ntraits_]();
	int loc,tr;
	for(loc=0;loc<nloci_;loc++){
			baby->genes_[loc] = genes_[loc];}
	for(tr=0;tr<ntraits_;tr++){
			baby->traits_[tr] = traits_[tr];}
	phenotypeSet_=false;//may be age-related
	baby->Mutate(ran,specialInfo);
	baby->causeOfDeath_=0;
	baby->movedThisGen_=0;
	baby->SetPhenotypeFunction(SetTraits);
	pop_->Install( (Individual*) baby);
	}//end QuickHaplSexGenIndividual_LD::Clone
	

void QuickHaplSexGenIndividual_LD::Clone (Individual* mom, Ran3List& ran, aList<long double>& specialInfo)	{
	//copies mom as a new baby *this
	//obtain all information directly from mom
	Individual::Clone(mom,ran,specialInfo);// copies location etc. details
	//mom must be a QuickHaplSexGenIndividual_LD too
	QuickHaplSexGenIndividual_LD* gMom = (QuickHaplSexGenIndividual_LD*) mom->GetPointerToDerivedSelf();
	nloci_ = gMom->nloci_;
	delete [] genes_;
	genes_ = NEW long double[nloci_]();
	ntraits_ = gMom->ntraits_;
	delete [] traits_;
	traits_ = NEW long double[ntraits_]();
	int loc,tr;
	for(loc=0;loc<nloci_;loc++){
		genes_[loc] = gMom->genes_[loc];}
	for(tr=0;tr<ntraits_;tr++){
		traits_[tr] = gMom->traits_[tr];}
	phenotypeSet_=false;//may be age-related
	Mutate(ran,specialInfo);
	moveRate_=gMom->moveRate_;
	alive_=age_=1;
	causeOfDeath_=movedThisGen_=0;
	lifeStage_=zygote;
	SetPhenotypeFunction(gMom->SetTraits);
	pop_->Install( (Individual*) this);
	}//end QuickHaplSexGenIndividual_LD::Clone



int	QuickHaplSexGenIndividual_LD::CountAlleles (int loc, aString& alleleVal){
	int a = alleleVal.toInt();
	int nAlleles=0;
	if (genes_[loc] == a) nAlleles++;
	return nAlleles;
	}//end QuickHaplSexGenIndividual_LD::CountAlleles




aString* QuickHaplSexGenIndividual_LD::GenotypeString(char* separator){
	aString* gStr = NEW aString(), *lgStr;
	int loc;
	for(loc=0;loc<nloci_;++loc){
		lgStr = GenotypeString(loc, separator);
		if(loc>0){lgStr->cat(separator);}
		gStr->cat(*lgStr);
		delete lgStr;}
	return gStr;
	}//end QuickHaplSexGenIndividual_LD::GenotypeString
	


aString* QuickHaplSexGenIndividual_LD::GenotypeString(int locus, char* /*separator*/){
	//alleles separated by parentheses; e.g. (1)(1) or (1)(0)
	aString* gStr = NEW aString();
	aString cgStr("(");
	cgStr.cat(genes_[locus]);
	cgStr.cat(")");
	gStr->cat(cgStr);
	return gStr;
	}//end QuickHaplSexGenIndividual_LD::GenotypeString
	
	
	
aString* QuickHaplSexGenIndividual_LD::GenotypeString(int locus, int /*alleleNum*/, char* /*separator*/){
	return NEW aString(genes_[locus]);
	}//end QuickHaplSexGenIndividual_LD::GenotypeString





void QuickHaplSexGenIndividual_LD::GetGenome(GeneContainerCore& gc){
	//refills container class as a MultiHaplotypeContainer<int>[ploidy][locus]
	HaplotypeContainer<long double>* hc = (HaplotypeContainer<long double>*) &gc;
	hc->Clear();
//	mhc->ploidy_ = QuickHaplSexGenIndividual_LD::ploidy();
	int loc;
	hc->haplotype_=NEW long double[nloci_];
	for(loc = 0;loc<nloci_;++loc){
		(*hc)[loc] = genes_[loc];}
	hc->SetID(id_);
	}//QuickHaplSexGenIndividual_LD::GetGenome(GeneContainerCore& gc)



long double* QuickHaplSexGenIndividual_LD::GetGenotype (void) {
	//returns a copy that must be disposed of elsewhere
	long double *genesCopy = NEW long double[nloci_]();
	int loc;
	for(loc=0;loc<nloci_;loc++){
		genesCopy[loc] = genes_[loc];}
	return genesCopy;
	}//end QuickHaplSexGenIndividual_LD::GetGenotype



long double QuickHaplSexGenIndividual_LD::GetGenotype (int locus, int /*allele*/) {
	if(locus>=nloci_) return -1;
	return genes_[locus];
	}//end QuickHaplSexGenIndividual_LD::GetGenotype




void QuickHaplSexGenIndividual_LD::IIndividual (genderSet gender, lifeStageType lifeStage, 
								long double rate, Ran3List& ran, LandScapeInitializer01& init, aList<long double>& extraInfo){
	Individual::IIndividual(gender,lifeStage,rate, ran,init,extraInfo);
	int species = (int)extraInfo[0];
	int pop =     (int)extraInfo[1];
	int indiv =   (int)extraInfo[2];
	MultiHaplotypeContainer<long double>* mhc = (MultiHaplotypeContainer<long double>*) &(init.initMultiInheritanceGenotypes_[species][pop][indiv]);
	QuickHaplSexGenIndividual_LD::SetGenome(*mhc);
	}//end QuickHaplSexGenIndividual_LD::IIndividual


int QuickHaplSexGenIndividual_LD::loci(void){
	return nloci_;
	}//end QuickHaplSexGenIndividual_LD::loci()


void QuickHaplSexGenIndividual_LD::MakeABaby (long& idCount, Ran3List& ran, aList<long double>& specialInfo){
	//installs a new baby in the population
	//sexual reproduction -- all loci unlinked
	QuickHaplSexGenIndividual_LD* baby = NEW QuickHaplSexGenIndividual_LD(++idCount);
	baby->age_ = 1;
	baby->alive_ = 1;
	baby->sex_=B;
	baby->lifeStage_ = zygote;
	baby->moveRate_ = moveRate();
	long dadnum = ran.longBetween(0,pop_->fathers()-1);
	Individual& idad = pop_->father(dadnum);
	QuickHaplSexGenIndividual_LD* dad = (QuickHaplSexGenIndividual_LD*) idad.GetPointerToDerivedSelf();
	baby->nloci_ = nloci_;
	baby->genes_ = NEW long double[nloci_]();
	int loc;
	for(loc=0;loc<nloci_;++loc){//loci unlinked
		if(ran()<half){//mom's gene
				baby->genes_[loc] = genes_[loc];}
			else{
				baby->genes_[loc] = dad->genes_[loc];}
		}
	baby->ntraits_ =ntraits_;
	baby->traits_ = NEW long double[ntraits_]();
	baby->Mutate(ran, specialInfo);
	baby->phenotypeSet_=false;
	baby->SetPhenotypeFunction(SetTraits);
	baby->SetPhenotype();
	pop_->Install(baby);
	}  //end QuickHaplSexGenIndividual_LD::MakeABaby




void QuickHaplSexGenIndividual_LD::Mutate (Ran3& ran, aList<long double>& specialInfo){
	//mutate the genome if warranted
	if(specialInfo.len()==0) return;
	if(specialInfo[0] == zero) return;
	long double probnMutations, mutationRate = specialInfo[0], ranNum = ran();
	int nmutations=0;
	int loc,m,mloc;
	for(loc = nloci_;loc>0;--loc){
		probnMutations = exp(loc*ln(one - mutationRate));
		if(ranNum>probnMutations){
				nmutations++;}
			else{break;}
		}
	for(m=0;m<nmutations;++m){//change a random allele
		mloc=ran.longBetween(0,nloci_-1);
		if(genes_[mloc]==0){genes_[mloc]=1;}else{genes_[mloc]=0;}
		}
	if(nmutations>0){
		phenotypeSet_=false;
		SetPhenotype();}
	}//end QuickHaplSexGenIndividual_LD::Mutate()


long double QuickHaplSexGenIndividual_LD::phenotype(int trait){
	if(trait<0 || trait>=ntraits_-1) return zero;
	if(! phenotypeSet_) SetPhenotype(specialInfo);
	return traits_[trait];
	}//end QuickHaplSexGenIndividual_LD::phenotype(int)


int QuickHaplSexGenIndividual_LD::ploidy(void){
	return 1;
	}//end QuickHaplSexGenIndividual_LD::ploidy(int)


aString* QuickHaplSexGenIndividual_LD::PhenotypeString(char* separator){
	aString* gStr = NEW aString(), *lgStr;
	int tr;
	for(tr=0;tr<ntraits_;++tr){
		lgStr = PhenotypeString(tr, separator);
		if(tr>0){lgStr->cat(separator);}
		gStr->cat(*lgStr);
		delete lgStr;}
	return gStr;
	}//end QuickHaplSexGenIndividual_LD::PhenotypeString
	


aString* QuickHaplSexGenIndividual_LD::PhenotypeString(int trait, char* /*separator*/){
	//alleles separated by commas
	if(trait<0 || trait>=ntraits_-1){
		aError("ERROR: PhenotypeString(): trait out of range");}
	if(!phenotypeSet_){
		aError("ERROR: calculate the phenotype before printing it");}
	aString* gStr = NEW aString();
	aString cgStr(" ");
	cgStr.cat(traits_[trait]);
	cgStr.cat(",");
	gStr->cat(cgStr);
	return gStr;
	}//end QuickHaplSexGenIndividual_LD::PhenotypeString
	
	
	
aString* QuickHaplSexGenIndividual_LD::PhenotypeString(int trait, int /*alleleNum*/, char* /*separator*/){
	return NEW aString(traits_[trait]);
	}//end QuickHaplSexGenIndividual_LD::PhenotypeString




void QuickHaplSexGenIndividual_LD::PrintGenes(ostream& outFile, int /*alleleLen*/, int codeLen) {
	//create geneStr before calling this function
	aString *geneStr = GenotypeString(' ');
	aString idStr(id_);
	idStr.PrependCharToLen(' ',codeLen);
	idStr.cat(' ');
	idStr.cat(*geneStr);
	outFile<<idStr<<endl;
	delete geneStr;
	}//end QuickHaplSexGenIndividual_LD::PrintGenes



void QuickHaplSexGenIndividual_LD::PrintPhenotypes(ostream& outFile, int /*alleleLen*/, int codeLen) {
	//create geneStr before calling this function
	aString *traitStr = PhenotypeString(' ');
	aString idStr(id_);
	idStr.PrependCharToLen(' ',codeLen);
	idStr.cat(' ');
	idStr.cat(*traitStr);
	outFile<<idStr<<endl;
	delete traitStr;
	}//end QuickHaplSexGenIndividual_LD::PrintGenes



void QuickHaplSexGenIndividual_LD::SetGenome(GeneContainerCore& gc){
	//initializes genome using MultiHaplotypeContainer<int>[ploidy][locus]
	MultiInheritanceModeContainer<long double>* mhc = (MultiInheritanceModeContainer<long double>*) &gc;
	int loc,tr,haploid=0, quantitative=5;
	if(nloci_ != mhc->loci(haploid)){
		nloci_ = mhc->loci(haploid);
		delete genes_;
		genes_ = NEW long double[nloci_]();
		}
	for(loc = 0;loc<nloci_;++loc){
		genes_[loc] = mhc->haplotype(haploid,loc,0);}
	for(tr = 0;tr<ntraits_;++tr){
		traits_[tr] = mhc->haplotype(quantitative,tr,0);}
	}//QuickHaplSexGenIndividual_LD::SetGenome(GeneContainerCore& )




void QuickHaplSexGenIndividual_LD::SetPhenotype(aList<long double>& specialInfo){
	if(phenotypeSet_=true) return;
	SetTraits(*this,specialInfo);
	}//end QuickHaplSexGenIndividual_LD::loci()



int QuickHaplSexGenIndividual_LD::traits(void){
	return ntraits_;
	}//end QuickHaplSexGenIndividual_LD::loci()





#endif

