/***************************************************
*
*	SexChromIndividual.cpp
*
*	Handles processes involving individuals with genetic sex determination
*	Genotypes have sex chromosomes and are held in a container class, SexChromGenome
*
****************************************************/


#ifndef SexChromIndividualcpp
#define SexChromIndividualcpp = 1



#include "SexChromIndividual.h"
#include "LandScape.h"
#include "Element.cpp"
#include "aList.cpp"
	
	
//CONSTRUCTORS AND DESTRUCTORS
Individual* SexChromIndividual::Duplicate (long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	return  new SexChromIndividual(id, site);
	}//end  SexChromIndividual::Duplicate


GeneticIndividual* SexChromIndividual::Duplicate (Phenotype& newGenes, long id, Population* site){
	//returns a duplicate, used for virtual copy construction
	return  new SexChromIndividual(newGenes, id, site);
	}//end  SexChromIndividual::Duplicate



	
void SexChromIndividual::Clone (long& idCount)	{
	//installs a new baby in the population
	SexChromIndividual* baby = new SexChromIndividual(++idCount);
	baby->age_ = 1;
	baby->alive_ = 1;
	baby->lifeStage_ = zygote;
	baby->moveRate_ = moveRate();
	baby->genes_ = genes_->Duplicate();
	baby->matings_=0; 
	baby->matingsThisAgeClass_=0;
	pop_->Install(baby);
	}//end SexChromIndividual::Clone
	

void SexChromIndividual::Copy (SexChromIndividual* it){
	DioeciousGeneticIndividual::Copy((DioeciousGeneticIndividual*) it);
	SetGender();
	}//end SexChromIndividual::Copy



void SexChromIndividual::MakeABaby (long& idCount, Ran3List& ran, double mutationRate, double mutationStdev){
	//OVERRIDE IN SUBCLASSES FOR DIFFERENT MATING SYSTEMS
	//THIS IMPLEMENTS SEXUAL REPRODUCTION WITH COMPLETE LAST-MATE SPERM PRECEDENCE
	//does nothing if no sperm available
	//installs a new baby in the population
	if(sex_==M) return;
	if (mates()){
		SexChromIndividual* baby = new SexChromIndividual(++idCount);
		baby->age_ = 1;
		baby->alive_ = 1;
		baby->lifeStage_ = zygote;
		baby->genes_ = genes_->Duplicate();
		//get the parents genotypes and send to Genotype for meiosis & fertilization	
		CoreGenome& momGenes = GetGenotype();
		CoreGenome& dadGenes = GetMate(ran);
		CoreGenome* g = (CoreGenome*) baby->genes_;
		g->SetSexualZygote(&momGenes, &dadGenes, ran, mutationRate, mutationStdev);
		baby->SetGender();
		if (baby->sex()==F){baby->moveRate(moveRate());}
				else {baby->moveRate(moveRate());}
		baby->SetPhenotype(ran);
		baby->matings_=0; 
		baby->matingsThisAgeClass_=0;
		pop_->Install(baby);
		}// end if mates
	}  //end SexChromIndividual::MakeABaby


void SexChromIndividual::Reproduce (long& idCount, Ran3List& ran, double mutationRate, double mutationStdev){
	//installs new babies in the population
	//each female reproduces
	//according to its individual fecundity
	//derived classes handle special mating systems-- this is the closest to theoretical ideal
	//ALL OFFSPRING HAVE NEWLY CHOSEN FATHERS
	if(sex_==M) return;
	int b,f=Fecundity(ran);
	for (b=0;b<f;++b){
		ClearMates();
		FindMate(ran,1,1);//each offspring has newly chosen father
		MakeABaby(idCount,ran, mutationRate, mutationStdev);}
	}//end SexChromIndividual::Reproduce


void SexChromIndividual::SetGender(void){
	//finds the gender from the sex chromosome and 
	//uses it to set the individual's gender
//	CoreSexChromGenome* theGenes = (CoreSexChromGenome*) genes_;
	sex_ = (genderSet) genes_->sex();
	}//end SexChromIndividual::SetGender



#endif

