/******************************************
*
*	DioeciousLandScape.cpp
*
*   This defines functions for the derived LandScape class
*	that forms the basis for individuals that must mate to reproduce.
*	It seeds the landscape with base-class hermaphroditic individuals,
*	and presumes constant habitat quality.  Derived
*	classes add features and/or take data.
******************************************/


#ifndef DioeciousLandScapecp
#define DioeciousLandScapecp = 1

#include "DioeciousLandScape.h"
#include "RandomList.cpp"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"


void DioeciousLandScape::ILandScape(Ran3List& ran, int seeded){
	//only called just after constructors. Exists to avoid code duplication
	int row,col;
	CoordinateType coords;
	for(row=0;row<len_;++row){
		ObjList<GridCell>*  gridList = NEW ObjList<GridCell>();
		grid_.Append(gridList);
		for(col=0;col<width_;++col){
			coords.set(row,col);
			GridCell* g = NEW GridCell(this, coords,0);//habitat=0
			gridList->Append(g);
			for (int s=0;s<species_;++s){
				DioeciousPopulation* pop= 
					NEW DioeciousPopulation(totalPopsCreated_++,s,g,maxCapacity_[s]);
				g->Append(pop);
				}//end for species
			}//end for cols
		}// end for rows	
	CreateMigrationMatrix();
	cout<<len_<<" x "<<width_<<" landscape created"<<endl;
	if(seeded){
		for (int s=0;s<species_;++s){
			SeedLandScape(s, initSize_[s], ran);}}
	}//end DioeciousLandScape::ILandScape


void DioeciousLandScape::ILandScape(LandScapeInitializer01& init, Ran3List& ran, int seeded){
	//only called just after constructors. Exists to avoid code duplication
	LandScape::ILandScape(init, ran,seeded);
	}//end DioeciousLandScape::ILandScape





void DioeciousLandScape::SeedLandScape (int species, long initSize, Ran3List& ran){
	//puts individuals randomly into an empty landscape
	//make and initialize each individual and install it in a specified (presently random) grid location
	CoordinateType loc;
	for (long i=0;i<initSize;++i){
		loc.RandomLoc(0,len_-1,0,width_-1,ran);
		genderSet sex = (ran.next()<intrinsicSexRatio(species)) ? F:M;
		DioeciousIndividual* indiv = NEW DioeciousIndividual(++totalIndivsCreated_);
		indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran);//age=1
		InstallInGrid(species, indiv, loc);}
	}//	end DioeciousLandScape::SeedLandScape 



void DioeciousLandScape::SeedLandScape (LandScapeInitializer01& init, int species, long initSize, Ran3List& ran){
	//puts individuals randomly into an empty landscape
	//make and initialize each individual and install it in a specified (presently random) grid location
	LandScape::SeedLandScape (init, species, initSize,ran);//explicit for clarity
	}//	end DioeciousLandScape::SeedLandScape 

#endif



