/******************************************
*
*	GeneticLandScape.cpp
*
*   This header defines the derived LandScape class
*	that forms the basis for individuals with genetic information.
*	It seeds the landscape with base-class genetic individuals,
*	and presumes constant habitat quality.  Derived
*	classes add features and/or take data.
******************************************/

#ifndef GeneticLandScapecp
#define GeneticLandScapecp = 1

#include "GeneticLandScape.h"
#include "RandomList.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "GeneContainers.cpp"



//	CONSTRUCTORS AND DESTRUCTORS
/*
GeneticLandScape<T>::GeneticLandScape(LandScapeInitializer00& init):
	//general purpose initializer
			LandScape(init),
			seededGenotypes_(init.initGenes_),
			seededGenotypeFreqs_(init.initGenotypeFreqs_){
#ifdef MemoryLeakDebug
	memID_=++memIDCount_;
	memIDList_.Append(memID_);
#endif
				
	for(int s=0;s<species_;++s){
		ObjList<ObjList<Phenotype > >& gengenList = seededGenotypes_[s];
		ObjList<Phenotype >&  genList = gengenList[0];
		Phenotype& gen = genList.First();
		StatList<int>* chromList = NEW StatList<int>();
		lociPerChromosome_.Append(chromList);
		for(int ch = 0; ch<gen.chroms1N();++ch){
			chromList->Append(gen.loci(ch) );}
		
  		ObjList<aList<T> >* speciesAlleleList = NEW ObjList<aList<T> >();
  		allelesInScape_.Append(speciesAlleleList);
		allelesCurrent_.Append(0);//flags
		allFreqCurrent_.Append(0);//flags
		hetExpDataCurrent_.Append(0);//flags
		hetObsDataCurrent_.Append(0);//flags
		}
	}//end constructor GeneticLandScape::GeneticLandScape(LandScapeInitializer00&)
*/

GeneticLandScape::GeneticLandScape(LandScapeInitializer01& init):
	/*general purpose initializer*/
			LandScape(init),
			/*seededGenotypes_(init.initPhenes_),*/
			seededGenotypeFreqs_(init.initPhenotypeFreqs_){
//	ObjList<ObjList<ObjList<Genome> > > seededGenotypes_;
	bool landScapeHandlesGeneticInitialization=false;
	for(int i=0;i<init.initPhenes_.len();++i){
		landScapeHandlesGeneticInitialization=true;
		seededGenotypes_.Append(NEW ObjList<ObjList<Phenotype > >());
		for(int j=0;j<init.initPhenes_[i].len();++j){
			seededGenotypes_[i].Append(NEW ObjList<Phenotype >());
			for(int k=0;k<init.initPhenes_[i][j].len();++k){
				Phenotype* p = init.initPhenes_[i][j][k].Duplicate();
				CoreGenome* pp = (CoreGenome*) p;
				pp->SetPhenotype();
//				seededGenotypes_[i][j].Append((Phenotype*) pp);
				seededGenotypes_[i][j].Append(pp);
				}
			}
		}
				
	for(int s=0;s<species_;++s){
		if(landScapeHandlesGeneticInitialization){
			ObjList<ObjList<Phenotype > >& gengenList = seededGenotypes_[s];
			ObjList<Phenotype >&  genList = gengenList[0];
			Phenotype& gen = genList.First();
			}
		StatList<int>* chromList = NEW StatList<int>();
		lociPerChromosome_.Append(chromList);
  		ObjList<StringList>* speciesAlleleList = NEW ObjList<StringList>();
  		allelesInScape_.Append(speciesAlleleList);
		allelesCurrent_.Append(0);//flags
		allFreqCurrent_.Append(0);//flags
		hetExpDataCurrent_.Append(0);//flags
		hetObsDataCurrent_.Append(0);//flags
		}
	}//end constructor GeneticLandScape::GeneticLandScape(LandScapeInitializer01&)


GeneticLandScape::GeneticLandScape(int len, //ONE SPECIES
					 int w, 	//default habitat=0, override ILandScape otherwise
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 long double fSurvivalRate, //CONSTANT THROUGH LIFE
  					 long double fFecundityRate, //CONSTANT THROUGH LIFE
  					 long double femaleMovementRate,
//  					 selectionSourceSet sSource,
  					 ObjList<Phenotype >& initGenes,
  					 aList<long double>& initGenotypeFreqs
					 ):
		LandScape(len,w,migrationModel,initSize,maxCapacity,maxage,
				  fSurvivalRate,fFecundityRate,femaleMovementRate/*, sSource*/){
	ObjList<Phenotype >* iGenes = NEW ObjList<Phenotype >(initGenes);
	ObjList<ObjList<Phenotype > >* ihGenes = NEW ObjList<ObjList<Phenotype > >();
	ihGenes->Append(iGenes);
	seededGenotypes_.Append(ihGenes);

	aList<long double>* iFreqs = NEW aList<long double>(initGenotypeFreqs);
	ObjList<aList<long double> >* ihFreqs = NEW ObjList<aList<long double> >();
	ihFreqs->Append(iFreqs);
	seededGenotypeFreqs_.Append(ihFreqs);
	
	Phenotype& gen = initGenes.First();
	StatList<int>* chromList = NEW StatList<int>();
	lociPerChromosome_.Append(chromList);
//	for(int ch = 0; ch<gen.chroms1N();++ch){
//		chromList->Append(gen.loci(ch) );}

//  	ObjList<aList<T> >* speciesAlleleList = NEW ObjList<aList<T> >();
  	ObjList<StringList>* speciesAlleleList = NEW ObjList<StringList>();
  	allelesInScape_.Append(speciesAlleleList);
	allelesCurrent_.Append(0);//flags
	allFreqCurrent_.Append(0);//flags
	hetExpDataCurrent_.Append(0);//flags
	hetObsDataCurrent_.Append(0);//flags
	}//end constructor GeneticLandScape::GeneticLandScape


GeneticLandScape::GeneticLandScape(int len, //ONE SPECIES, 1 HABITAT
					 int w, 	//default habitat=0, override ILandScape otherwise
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 aList<long double>* fSurvivalSchedule, //VARIABLE SURVIVAL
  					 aList<long double>* fFecunditySchedule, //VARIABLE FECUNDITY
 					 ObjList<aList<long double> >* femaleMovementRate,
// 					 selectionSourceSet sSource,
 					 ObjList<Phenotype >& initGenes,
  					 aList<long double>& initGenotypeFreqs
					 ):
		LandScape(len,w,migrationModel,initSize,maxCapacity,maxage,
				  fSurvivalSchedule,fFecunditySchedule,femaleMovementRate/*,sSource*/){
	ObjList<Phenotype >* iGenes = NEW ObjList<Phenotype >(initGenes);
	ObjList<ObjList<Phenotype > >* ihGenes = NEW ObjList<ObjList<Phenotype > >();
	ihGenes->Append(iGenes);
	seededGenotypes_.Append(ihGenes);

	aList<long double>* iFreqs = NEW aList<long double>(initGenotypeFreqs);
	ObjList<aList<long double> >* ihFreqs = NEW ObjList<aList<long double> >();
	ihFreqs->Append(iFreqs);
	seededGenotypeFreqs_.Append(ihFreqs);

	Phenotype& gen = initGenes.First();
	StatList<int>* chromList = NEW StatList<int>();
	lociPerChromosome_.Append(chromList);
//	for(int ch = 0; ch<gen.chroms1N();++ch){
//		chromList->Append(gen.loci(ch) );}

//  	ObjList<aList<T> >* speciesAlleleList = NEW ObjList<aList<T> >();
  	ObjList<StringList>* speciesAlleleList = NEW ObjList<StringList>();
  	allelesInScape_.Append(speciesAlleleList);
	allelesCurrent_.Append(0);//flags
	allFreqCurrent_.Append(0);//flags
	hetExpDataCurrent_.Append(0);//flags
	hetObsDataCurrent_.Append(0);//flags
	}//end constructor GeneticLandScape::GeneticLandScape


GeneticLandScape::GeneticLandScape(int len, //ONE OR MORE SPECIES, 1 HABITAT
					 int w, 	//default habitat=0, override ILandScape otherwise
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<long double> >* fSurvivalSchedule,
  					 ObjList<aList<long double> >* fFecunditySchedule,
 					 ObjList<ObjList<aList<long double> > >* femaleMovementRate,
// 					 selectionSourceSet sSource,
   					 ObjList<ObjList<Phenotype > >& initGenes,
  					 ObjList<aList<long double> >& initGenotypeFreqs
					 ):
		LandScape(len,w,migrationModel,species,initSize,maxCapacity,maxage,
				  fSurvivalSchedule,fFecunditySchedule,femaleMovementRate/*, sSource*/){
	ObjList<ObjList<Phenotype > >* newGenes = NEW ObjList<ObjList<Phenotype > >(initGenes);
	seededGenotypes_.Append(newGenes);
	ObjList<aList<long double> >* newGenotypeFreqs = NEW ObjList<aList<long double> >(initGenotypeFreqs);
	seededGenotypeFreqs_.Append(newGenotypeFreqs);

	ObjList<Phenotype >& genList = initGenes.First();
	Phenotype& gen = genList.First();
	StatList<int>* chromList = NEW StatList<int>();
	lociPerChromosome_.Append(chromList);
//	for(int ch = 0; ch<gen.chroms1N();++ch){
//		chromList->Append(gen.loci(ch) );}

	for(int s=0;s<species_;++s){
//  		ObjList<aList<T> >* speciesAlleleList = NEW ObjList<aList<T> >();
  		ObjList<StringList>* speciesAlleleList = NEW ObjList<StringList>();
  		allelesInScape_.Append(speciesAlleleList);
		allelesCurrent_.Append(0);//flags
		allFreqCurrent_.Append(0);//flags
		hetExpDataCurrent_.Append(0);//flags
		hetObsDataCurrent_.Append(0);//flags
		}
	}//end constructor GeneticLandScape::GeneticLandScape
					 

GeneticLandScape::GeneticLandScape(int len, /*ONE OR MORE SPECIES, 1 HABITAT*/
					 int w, 
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<long double> >* fSurvivalSchedule,
  					 ObjList<aList<long double> >* fFecunditySchedule,
 					 ObjList<ObjList<aList<long double> > >* femaleMovementRate,
// 					 selectionSourceSet sSource,
 					 ObjList<ObjList<ObjList<Phenotype > > >& initGenes,
					 ObjList<ObjList<aList<long double> > >& initGenotypeFreqs
					 ):
		LandScape(len,w,migrationModel,species,initSize,maxCapacity,maxage,
				  fSurvivalSchedule,fFecunditySchedule,femaleMovementRate/*, sSource*/){
	seededGenotypes_.Copy(initGenes);
	seededGenotypeFreqs_.Copy(initGenotypeFreqs);

	ObjList<ObjList<Phenotype > >& gengenList = initGenes.First();
	ObjList<Phenotype >&  genList = gengenList.First();
	Phenotype& gen = genList.First();
	StatList<int>* chromList = NEW StatList<int>();
	lociPerChromosome_.Append(chromList);
//	for(int ch = 0; ch<gen.chroms1N();++ch){
//		chromList->Append(gen.loci(ch) );}

	for(int s=0;s<species_;++s){
//  		ObjList<aList<T> >* speciesAlleleList = NEW ObjList<aList<T> >();
  		ObjList<StringList>* speciesAlleleList = NEW ObjList<StringList>();
  		allelesInScape_.Append(speciesAlleleList);
		allelesCurrent_.Append(0);//flags
		allFreqCurrent_.Append(0);//flags
		hetExpDataCurrent_.Append(0);//flags
		hetObsDataCurrent_.Append(0);//flags
		}
	}//end constructor GeneticLandScape::GeneticLandScape

					 
					 
GeneticLandScape::~GeneticLandScape(void){
  	allelesInScape_.Clear();
	allFreq_.Clear();
	allelesCurrent_.Clear();
	allFreqCurrent_.Clear();
	hetObsLoc_.Clear();
	hetExpLoc_.Clear();
	hetObs_.Clear();
	hetExp_.Clear();
	hetExpDataCurrent_.Clear();
	hetObsDataCurrent_.Clear();
	seededGenotypes_.Clear();
	seededGenotypeFreqs_.Clear();
	}//end destructor GeneticLandScape::~GeneticLandScape					 




// OVERRIDDEN AND NEW FUNCTIONS


void GeneticLandScape::AgeOneStep(int species){
	long old=dead(species);
	LandScape::AgeOneStep(species);
	if (old != dead(species)){//reset flags
		allelesCurrent_.Replace(0,species);
		allFreqCurrent_.Replace(0,species);
		hetExpDataCurrent_.Replace(0,species);
		hetObsDataCurrent_.Replace(0,species);
		}
	}//end GeneticLandScape::AgeOneStep
	

/*		//handled in base class now
void GeneticLandScape::DoOneCycle (long gen, Ran3List& ran, ObjList<aList<long double> >& dispersalInfo,
							 ObjList<aList<long double> >& reproductionInfo){
	//PROBABLY BEST TO OVERRIDE THIS IN DERIVED CLASSES
	TimeIncrement();
	int s;
	for(s=0;s<species();++s){
		ViabilitySelection(s, ran);}
	for(s=0;s<species();++s){
		AdultDispersal(s, ran,dispersalInfo[s]);}
	for(s=0;s<species();++s){
		MateOnGrid(s, ran);}
	for(s=0;s<species();++s){
		Reproduce(s, ran, reproductionInfo[s]);}
	for(s=0;s<species();++s){
		AgeOneStep(s);}
	for(s=0;s<species();++s){
		SomeAdultsDie(s, ran);}
	for(s=0;s<species();++s){
		ReplaceTheDead(s, ran);}

	//DERIVED should take data from dead individuals before calling BuryTheDead
	for(s=0;s<species();++s){
		BuryTheDead(s);}
	
	}//end GeneticLandScape::DoOneCycle
	

void GeneticLandScape::DoOneCycle (int species, long gen, Ran3List& ran, ObjList<aList<long double> >& dispersalInfo,
							 ObjList<aList<long double> >& reproductionInfo){
	//PROBABLY BEST TO OVERRIDE THIS IN DERIVED CLASSES
	TimeIncrement();
	ViabilitySelection(species, ran);
	AdultDispersal(species, ran, dispersalInfo[species]);
	MateOnGrid(species, ran);
	Reproduce(species, ran, reproductionInfo[species]);
	AgeOneStep(species);
	SomeAdultsDie(species, ran);
	ReplaceTheDead(species, ran);

	//DERIVED should take data from dead individuals before calling BuryTheDead
	BuryTheDead(species);
	
	}//end GeneticLandScape::DoOneCycle
*/

	

void GeneticLandScape::ILandScape (Ran3List& ran, int seeded){
	//only called just after constructors. Exists to avoid code duplication
	int row,col;
	CoordinateType coords;
	for(row=0;row<len_;++row){
		ObjList<GridCell>*  gridList = NEW ObjList<GridCell>();
		grid_.Append(gridList);
		for(col=0;col<width_;++col){
			coords.set(row,col);
			GeneticGridCell* g = NEW GeneticGridCell(this, coords,0);//habitat=0
			gridList->Append(g);
			for (int s=0;s<species_;++s){
				GeneticPopulation* pop= NEW GeneticPopulation(
								totalPopsCreated_++,s,g,maxCapacity_[s]);
				g->Append(pop);
				}//end for species
			}//end for cols
		}// end for rows	
	CreateMigrationMatrix();	
	cout<<len_<<" x "<<width_<<" landscape created"<<endl;
	if(seeded){
		for (int s=0;s<species_;++s){
			SeedLandScape(s, initSize_[s], ran);}
		}
	}//end GeneticLandScape::ILandScape

/*
void GeneticLandScape::ILandScape (LandScapeInitializer00& init, Ran3List& ran){
	//only called just after constructors. Exists to avoid code duplication
	LandScape::ILandScape (init, ran);//explicit for clarity
	}//end GeneticLandScape::ILandScape
*/

void GeneticLandScape::ILandScape (LandScapeInitializer01& init, Ran3List& ran, int seeded){
	//only called just after constructors. Exists to avoid code duplication
	LandScape::ILandScape (init, ran, seeded);//explicit for clarity
	if(init.initPhenes_.len()>0){
		SetLociByChromosome(init);}
	}//end GeneticLandScape::ILandScape


void GeneticLandScape::Reproduce (int species, Ran3List& ran, aList<long double>& specialInfo){
				//specialInfo[0]=mutationRate, specialInfo[1]= sigma
	long x,y,old = babies(species);
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
//			g.Reproduce(species, totalIndivsCreated_, ran, mutationRate, sigma);}}
			g.Reproduce(species, totalIndivsCreated_, ran, specialInfo);}}
	if (old != babies(species)){//reset flags
		allelesCurrent_.Replace(0,species);
		allFreqCurrent_.Replace(0,species);
		hetExpDataCurrent_.Replace(0,species);
		hetObsDataCurrent_.Replace(0,species);
		}
	}//end GeneticLandScape::Reproduce


	

void GeneticLandScape::SomeAdultsDie (int species, Ran3List& ran){
	long old=dead(species);
	LandScape::SomeAdultsDie(species, ran);
	if (old != dead(species)){//reset flags
		allelesCurrent_.Replace(0,species);
		allFreqCurrent_.Replace(0,species);
		hetExpDataCurrent_.Replace(0,species);
		hetObsDataCurrent_.Replace(0,species);
		}
	}//end GeneticLandScape::SomeAdultsDie






int  GeneticLandScape::AllelePosition (int species, int loc, aString& alleleVal){
	//returns allele's position in allelesInScape_
	if (! allelesCurrent_[species]){
		UpdateAlleleList(species);}
  	StringList& locList = allelesInScape_[species][loc];
	return locList.Loc(&alleleVal);
	}//end  GeneticLandScape::AllelePosition(int,int,aString)


int  GeneticLandScape::AllelePresent (int species, int loc, aString& alleleVal){
	for (long x=0;x<len_;++x){
		for (long y=0;y<width_;++y){
			GeneticGridCell* g= (GeneticGridCell*) &(grid_[x][y]);
			if(g->AllelePresent(species, loc, alleleVal)){return 1;}
			}
		}
	return 0;
	}//end GeneticLandScape::AllelePresent
	
aString&  GeneticLandScape::AlleleAtLoc (int species, int loc, int alleleNum){
	//returns allele's Value
	if (! allelesCurrent_[species]){
		UpdateAlleleList(species);}
  	StringList& locList = allelesInScape_[species][loc];
	return locList[alleleNum];
	}//end  GeneticLandScape::AlleleAtLoc(int,int,int)



int  GeneticLandScape::AllelesAtLoc (int species, int loc){
	//returns number
	if (! allelesCurrent_[species]){
		UpdateAlleleList(species);}
  	StringList& locList = allelesInScape_[species][loc];
	return locList.len();
	}//end  GeneticLandScape::AllelesAtLoc(int,int)




void GeneticLandScape::AssignMutations(int /*species*/){}//not programmed yet
		//derive this
		
		
void GeneticLandScape::CalcAlleleFrequencies(int species){
	if (allelesCurrent_[species]==0){UpdateAlleleList(species);}
	ObjList<aList<long double> >& freqs= allFreq_[species];
	ObjList<StringList>& alleleList= allelesInScape_[species];
	freqs.Clear();
	long x,y,a,j;
	for (j=0;j<loci(species);++j){
		StringList& locList=alleleList[j];
		aList<long double>* locFreqs=NEW aList<long double>();
		freqs.Append(locFreqs);
		for (x=0;x<len_;++x){
			for (y=0;y<width_;++y){
				GeneticGridCell* g= (GeneticGridCell*) &( grid_[x][y]);
				for (a=0;a<locList.len();++a){
					aString& alleleVal=locList[a];
					long double newFreq = g->AlleleFrequency(species,j,alleleVal);
					long double samSize = (long double)  g->SampleSize(species,j);
					locFreqs->Append(newFreq/samSize);
					}//end for a
				}//end for y
			}//end for x
		}//end for j
	allFreqCurrent_[species]=1;
	};//end GeneticLandScape::CalcAlleleFrequencies


void GeneticLandScape::CoordsOfNthLocus(int species, int nthLoc, int& onChromNum, int& locNumOnChrom){
	int sum=0;/*nthLoc is 0-indexed; returns zero-indexed coordinates*/
	StatList<int>& speciesLociPerChromList = lociPerChromosome_[species];	
	for(int chrom=0; chrom<speciesLociPerChromList.len();++chrom){
		int n = speciesLociPerChromList[chrom];
		if(nthLoc +1 <= sum + n){
				locNumOnChrom = nthLoc - sum;
				onChromNum = chrom;
				return;}
			else{
				sum+= n;}
		}/*end for chrom*/
	}/*end GeneticPopulation::CoordsOfNthLocus*/



long GeneticLandScape::CountAlleles (int species,int loc, aString& alleleVal){
	long sum=0,x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GeneticGridCell* g= (GeneticGridCell*) &(grid_[x][y]);
			sum+=g->CountAlleles(species, loc, alleleVal);
			}
		}
	return sum;
	}//end GeneticLandScape::CountAlleles
	


long GeneticLandScape::CountHets (int species,int loc, aString& alleleVal){
	long sum=0,x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GeneticGridCell* g= (GeneticGridCell*) &(grid_[x][y]);
			sum+=g->CountHets(species, loc, alleleVal);
			}
		}
	return sum;
	}//end GeneticLandScape::CountHets
	

long GeneticLandScape::CountHets (int species, int loc){
	long sum=0,x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GeneticGridCell* g= (GeneticGridCell*) &(grid_[x][y]);
			sum+=g->CountHets(species, loc);
			}
		}
	return sum;
	}//end GeneticLandScape::CountHets




void GeneticLandScape::GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList, int species){
	pgcList.Clear();
	CoordinateType coords;
	int popCount=0;
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			++popCount;
			pgcList.Append(NEW PopulationGeneContainer<int>());
			coords.set(x,y);
			Population& pop = GetPopulation (species, coords);
			GeneticPopulation* gpop = (GeneticPopulation*) &pop;
			gpop->GetGenome(pgcList[popCount]);
			}
		}
	}//end GeneticLandScape::GetGenome


	

void GeneticLandScape::GetGenome(PopulationGeneContainer<int>& pgc, int species, int popID){
	CoordinateType coords;
	GetCoordinates (species, popID, coords);
	Population& pop = GetPopulation (species, coords);
	GeneticPopulation* gpop = (GeneticPopulation*) &pop;
	return gpop->GetGenome(pgc);
	}//end GeneticLandScape::GetGenome


	

void GeneticLandScape::GetGenome(MultiInheritanceModeContainer<int>& mimc, int species, int popID, long indiv){
	CoordinateType coords;
	GetCoordinates (species, popID, coords);
	Population& pop = GetPopulation (species, coords);
	GeneticPopulation* gpop = (GeneticPopulation*) &pop;
	return gpop->GetGenome(mimc, indiv);
	}//end GeneticLandScape::GetGenome


	

aString* GeneticLandScape::GetPhenotypeString(int species, int popID, long indiv, char* separator){
	CoordinateType coords;
	GetCoordinates (species, popID, coords);
	Population& pop = GetPopulation (species, coords);
	GeneticPopulation* gpop = (GeneticPopulation*) &pop;
	return gpop->GetPhenotypeString(indiv, separator);
	}//end GeneticLandScape::GetPhenotypeString


	

void GeneticLandScape::ILandScapeByHabitat (Ran3List& ran,  LandScapeInitializer01& init, int seeded){
	int row,col;
	CoordinateType coords;
	for(row=0;row<len_;++row){
		ObjList<GridCell>*  gridList = NEW ObjList<GridCell>();
		grid_.Append(gridList);
		for(col=0;col<width_;++col){
			coords.set(row,col);
			GridCell* g = init.gridcellOfCorrectType->Duplicate(this, coords,0);/*habitat=0*/
			gridList->Append(g);
			for (int s=0;s<species_;++s){
				Population* pop= init.popOfCorrectType->Duplicate(
								totalPopsCreated_++,s,g,maxCapacity_[s]);
				g->Append(pop);
				}/*end for species*/
			}/*end for cols*/
		}/* end for rows	*/
	SetHabitatTypes(init.habitatGrid_);
	CreateMigrationMatrix();	
	cout<<len_<<" x "<<width_<<
			" landscape created"<<endl;
	if(seeded){
		for (int s=0;s<species_;++s){
			SeedLandScapeByHabitat(init, s, initSize_[s], ran);}
		SetLociByChromosome(init);
		}
	}/*end GeneticLandScape::ILandScapeByHabitat*/



void GeneticLandScape::ILandScapeByHabitat (Ran3List& ran, ObjList<aList<int> >& habitatGrid,
					ObjList<ObjList<ObjList<Phenotype > > >& genesByFreqByHabitatBySpecies,
					ObjList<ObjList<aList<long double> > >& freqsByHabitatBySpecies){
	/*only called just after constructors. Exists to avoid code duplication*/
	CoordinateType coords;
	for(long row=0;row<len_;++row){
		ObjList<GridCell>*  gridList = NEW ObjList<GridCell>();
		grid_.Append(gridList);
		for(long col=0;col<width_;++col){
			coords.set(row,col);
			GeneticGridCell* g = NEW GeneticGridCell(this, coords,0);/*habitat=0*/
			gridList->Append(g);
			for (int s=0;s<species_;++s){
				GeneticPopulation* pop= NEW GeneticPopulation(
								totalPopsCreated_++,s,g,maxCapacity_[s]);
				g->Append(pop);
				}/*end for species*/
			}/*end for cols*/
		}/* end for rows	*/
	SetHabitatTypes(habitatGrid);
	CreateMigrationMatrix();	
	cout<<len_<<" x "<<width_<<
			" landscape created"<<endl;
	for (int s=0;s<species_;++s){
				SeedLandScapeByHabitat(s, initSize_[s], ran,
					genesByFreqByHabitatBySpecies,freqsByHabitatBySpecies);}
	}/*end GeneticLandScape::ILandScapeByHabitat*/






//int GeneticLandScape::inheritanceType(int species, int locus){
//	//returns value matching inheritanceTypes in MultiInheritanceModeContainer<T> (GeneContainers.h)
//	}


int GeneticLandScape::loci(int species){
	int sum=0;
//	for(int c=0;c<lociPerChromosome_[species].len();++c){
		sum += lociPerChromosome_[species].sum();//}
	return sum;
	}//end GeneticLandScape::loci


int GeneticLandScape::NthLocus(int species, int chrom, int locOnChrom){//0 indexed
	//returns the sequential locus number given locus coordinates
	int sum = 0;
	StatList<int>& speciesLociPerChromList = lociPerChromosome_[species];	
	for(int ch=0; ch<chrom;++ch){
		sum+= speciesLociPerChromList[ch];}//counts loci
	return sum + locOnChrom;
	}//end GeneticPopulation::NthLocus
	

int GeneticLandScape::ploidy(int species, int locus){
	ObjList<Phenotype >& geneList=SeededGeneList(species,0);
	Phenotype& genes=geneList.First();
	int chrom, locOnChrom;
	CoordsOfNthLocus(species,locus, chrom,locOnChrom);
	return genes.ploidy(locus);
	}//end GeneticLandScape::ploidy


void GeneticLandScape::PrintAlleleFreqs(ostream&, int /*species*/){
	}//not programmed yet


void GeneticLandScape::PrintAlleleFreqs(ostream&, int /*species*/,int /*locus*/){
	}//not programmed yet


void GeneticLandScape::PrintAlleleFreqs(ostream& outFile, int species,int locus, aString& alleleVal){
	double freq=0.0;
	for (long y=0;y<width_;++y){
		aString line;
		for (long x=0;x<len_;++x){
			GeneticGridCell* g= (GeneticGridCell*) &(grid_[x][y]);
			freq = g->AlleleFrequency(species, locus, alleleVal);
			line.cat(freq,1,4);
			if(x<len_-1){line.cat(",");}
			}//end for x
		outFile<<line;
		if (outFile == cout){outFile<<endl;}
			else{outFile<<endchr;}
		line.clear();
		}//end for y
	if (outFile == cout){outFile<<endl;}
		else{outFile<<endchr;}
	}//end GeneticLandScape::PrintAlleleFreqs(outFile, species, locus, alleleVal)


void GeneticLandScape::PrintAlleleList(ostream&, int /*species*/){}//not programmed yet


void GeneticLandScape::PrintAlleleList(ostream&, int /*species*/,int /*locus*/){}//not programmed yet


void GeneticLandScape::PrintGeneStrings(ostream& outFile, int species,int alleleLen, int codeLen){
	for (long x=0;x<len_;++x){
		for (long y=0;y<width_;++y){
			GeneticGridCell* g= (GeneticGridCell*) &(grid_[x][y]);
			g->PrintPopHeading(outFile, species);
			g->PrintGeneStrings(outFile, species, alleleLen, codeLen);
			outFile<<endl;
			}
		}
	}//end GeneticLandScape::PrintGeneStrings


void GeneticLandScape::PrintHetExp(ostream&, int /*species*/){}//not programmed yet


void GeneticLandScape::PrintHetExp(ostream&, int /*species*/,int /*loc*/){}//not programmed yet


void GeneticLandScape::PrintHetObs(ostream&, int /*species*/){}//not programmed yet


 void GeneticLandScape::PrintHetObs(ostream&, int /*species*/,int /*loc*/){}//not programmed yet


long GeneticLandScape::SampleSize (int species,int loc){
	long sum=0;		//handles ploidy differences
	for (long x=0;x<len_;++x){
		for (long y=0;y<width_;++y){
			GeneticGridCell* g= (GeneticGridCell*) &(grid_[x][y]);
			sum+=g->SampleSize(species, loc);
			}
		}
	return sum;
	}//end GeneticLandScape::SampleSize


ObjList<Phenotype >&	/*genotypes seeded into the habitat, by species*/
	GeneticLandScape::SeededGeneList(int species, int habitat){
		ObjList<ObjList<Phenotype > >& speciesList = seededGenotypes_[species];
		return speciesList[habitat];
		}/*end GeneticLandScape::SeededGeneList*/
	
	
aList<long double>&	/*freqs of genotypes seeded into the habitat, by species*/
	GeneticLandScape::SeededGeneFreqList(int species, int habitat){
		ObjList<aList<long double> >& speciesList = seededGenotypeFreqs_[species];
		return speciesList[habitat];
		}/*end GeneticLandScape::SeededGeneFreqList*/




void GeneticLandScape::SeedLandScape (int species, long initSize, Ran3List& ran){
	//puts genetic individuals randomly into an empty landscape
	//inherit this for separate sexes etc
	genderSet sex = A;//default asexual
	CoordinateType loc;
	//make a weighted aList for genotypes that will be seeded
	StatList<long> geneWeights;
	ObjList<Phenotype >& geneList=SeededGeneList(species,0);
	aList<long double>& freqs=SeededGeneFreqList(species,0);
	for(long i=0;i<geneList.len();++i){
		long b= (long) (initSize * (freqs[i]));
		geneWeights.Append(b);
		}//end for i
	while (geneWeights.sum()<initSize){//in case frequencies don't divide exactly into initSize
			geneWeights.Increment(geneWeights.len()-1);//increment the last one
			}//end while
		
	//make and initialize each individual and install it in a specified (presently random) grid location	
	for (long i=0;i<initSize;++i){
		loc.RandomLoc(0,len_-1,0,width_-1,ran);
		long x=ran.weighted(geneWeights);
		geneWeights.Decrement(x);//samples without replacement
//		Genotype* newGenes=geneList->Nth(x);
		Phenotype& newGenes=geneList[x];
		GeneticIndividual* indiv = NEW GeneticIndividual(newGenes,++totalIndivsCreated_);
		InstallInGrid(species, indiv, loc);
		indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran);//age=1
		}//end for i
	}//	end GeneticLandScape::SeedLandScape 


/*
void GeneticLandScape::SeedLandScape (LandScapeInitializer00& init, int species, long initSize, Ran3List& ran){
	//puts genetic individuals randomly into an empty landscape
	//inherit this for separate sexes etc
	CoordinateType loc;
	//make a weighted aList for genotypes that will be seeded
	StatList<long> geneWeights;
	ObjList<Phenotype >& geneList=SeededGeneList(species,0);
	aList<double>& freqs=SeededGeneFreqList(species,0);
	for(long i=0;i<geneList.len();++i){
		long b= (long) (initSize * (freqs[i]));
		geneWeights.Append(b);
		}//end for i
	while (geneWeights.sum()<initSize){//in case frequencies don't divide exactly into initSize
			geneWeights.increment(geneWeights.len()-1);//increment the last one
			}//end while
		
	//make and initialize each individual and install it in a specified (presently random) grid location	
	for (long i=0;i<initSize;++i){
		loc.RandomLoc(0,len_-1,0,width_-1,ran);
		long x=ran.weighted(geneWeights);
		geneWeights.decrement(x);//samples without replacement
		Phenotype& newGenes=geneList[x];
		GeneticIndividual<T>* indivType = (GeneticIndividual<T>*) init.indivOfCorrectType;//casting assumes data are correct
		GeneticIndividual<T>* indiv = indivType->Duplicate(newGenes,++totalIndivsCreated_);
		double sexRatio = init.sexRatio(species);
		genderSet sex = indiv->RealizedSexOfClassMember(sexRatio, ran);
		InstallInGrid(species, indiv, loc);
		indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran, init);//age=1
		}//end for i
	}//	end GeneticLandScape::SeedLandScape 
*/


void GeneticLandScape::SeedLandScape (LandScapeInitializer01& init, int species, long initSize, Ran3List& ran){
	//puts genetic individuals randomly into an empty landscape
	//inherit this for separate sexes etc
	CoordinateType loc;
	//make a weighted aList for genotypes that will be seeded
	StatList<long> geneWeights;
	ObjList<Phenotype >& geneList=SeededGeneList(species,0);
	aList<long double>& freqs=SeededGeneFreqList(species,0);
	aList<long double> extraInfo;
	extraInfo.Append(zero);
	extraInfo.Append(zero);
	for(long i=0;i<geneList.len();++i){
		long b= (long) (initSize * (freqs[i]));
		geneWeights.Append(b);
		}//end for i
	while (geneWeights.sum()<initSize){//in case frequencies don't divide exactly into initSize
			geneWeights.Increment(geneWeights.len()-1);//increment the last one
			}//end while
		
	//make and initialize each individual and install it in a specified (presently random) grid location	
	for (long i=0;i<initSize;++i){
		loc.RandomLoc(0,len_-1,0,width_-1,ran);
		long x=ran.weighted(geneWeights);
		geneWeights.Decrement(x);//samples without replacement
		Phenotype& newGenes=geneList[x];
		GeneticIndividual* indivType = (GeneticIndividual*) init.indivOfCorrectType;//casting assumes data are correct
		GeneticIndividual* indiv = indivType->Duplicate(newGenes,++totalIndivsCreated_);
		long double sexRatio = init.sexRatio(species);
		genderSet sex = indiv->RealizedSexOfClassMember(sexRatio, ran);
		InstallInGrid(species, indiv, loc);
		extraInfo[0]=(long double)(indiv->pop()->id());
		extraInfo[1]=(long double)i;
		indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran, init, extraInfo);//age=1
		}//end for i
	}//	end GeneticLandScape::SeedLandScape 



void GeneticLandScape::SeedLandScapeByHabitat (int species, long initSize, Ran3List& ran, 
					ObjList<ObjList<ObjList<Phenotype > > >& genesByFreqByHabitatBySpecies,
					ObjList<ObjList<aList<long double> > >& freqsByHabitatBySpecies){
	//puts genetic individuals into an empty landscape
	//according to their genotype and habitat
	genderSet sex = A;//default idealHermaphrodite
	//make a weighted aList for genotypes that will be seeded
	ObjList<ObjList<Phenotype > >& genesByFreqByHabitat = genesByFreqByHabitatBySpecies[species];
	ObjList<aList<long double> >& freqsByHabitat = freqsByHabitatBySpecies[species];
	//make up a list of gridcells of each habitat
	ObjList<ObjList<GridCell> > gridcellsByHabitat;
	for(int hh=0;hh<freqsByHabitat.len();++hh){
		ObjList<GridCell>* cellsWithHabitat = NEW ObjList<GridCell>();
		gridcellsByHabitat.Append(cellsWithHabitat);}
	for (long x=0;x<len_;++x){
		for (long y=0;y<width_;++y){
			GridCell& g = grid_[x][y];			
			ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[g.habitat()];
			cellsWithHabitat.Append(&g);
			}}	

	for(long h=0;h<freqsByHabitat.len();++h){
		StatList<long> geneWeights;
		aList<long double>& genomeFreqs = freqsByHabitat[h];
		ObjList<Phenotype >& genesInHabitat = genesByFreqByHabitat[h];
		ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[h];
		long indivsInHabitat	= (long)( initSize *( ((long double) (cellsWithHabitat.len() )) / ((long double) ( len_ * width_))  ) );
		for(int gg=0;gg<genesInHabitat.len();++gg){			
			long n= (long) (indivsInHabitat * (genomeFreqs[gg]));
			geneWeights.Append(n);}
		while (geneWeights.sum()<indivsInHabitat){//in case frequencies don't divide exactly into initSize
			geneWeights.Increment(geneWeights.len()-1);//increment the last one
			}//end while

		//make and initialize each individual and install it in a specified (presently random) grid location
		//that has the proper habitat type
		for (long i=0;i<indivsInHabitat;++i){
			int cellNum = ran.longBetween(0,cellsWithHabitat.len()-1);
			GridCell& g = cellsWithHabitat[cellNum];
			CoordinateType loc = g.coords();
			long x=ran.weighted(geneWeights);
			geneWeights.Decrement(x);//samples without replacement
			Phenotype& newGenes=genesInHabitat[x];
			GeneticIndividual* indiv = NEW GeneticIndividual(newGenes,++totalIndivsCreated_);
			InstallInGrid(species, indiv, loc);
			indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran);//age=1
			}//end for i
		
		}//end for h
	
	//clean up
	for(int h=0;h<gridcellsByHabitat.len();++h){
		ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[h];
		cellsWithHabitat.Detach();}
	gridcellsByHabitat.Clear();

	}//	end GeneticLandScape<T>::SeedLandScapeByHabitat 



int GeneticLandScape::traits(int species){
	ObjList<Phenotype >& geneList=SeededGeneList(species,0);
	Phenotype& genes=geneList.First();

	return genes.traits();
	}//end GeneticLandScape::traits



/*
 void GeneticLandScape::SeedLandScapeByHabitat (LandScapeInitializer00& init, 
					int species, long initSize, Ran3List& ran){
	//puts genetic individuals into an empty landscape
	//according to their genotype and habitat
	ObjList<ObjList<ObjList<Phenotype > > >& genesByFreqByHabitatBySpecies = init.initGenes_;
	ObjList<ObjList<aList<double> > >& freqsByHabitatBySpecies = init.initGenotypeFreqs_;
	//make a weighted aList for genotypes that will be seeded
	ObjList<ObjList<Phenotype > >& genesByFreqByHabitat = genesByFreqByHabitatBySpecies[species];
	ObjList<aList<double> >& freqsByHabitat = freqsByHabitatBySpecies[species];
	//make up a list of gridcells of each habitat
	ObjList<ObjList<GridCell> > gridcellsByHabitat;
	for(int hh=0;hh<freqsByHabitat.len();++hh){
		ObjList<GridCell>* cellsWithHabitat = NEW ObjList<GridCell>();
		gridcellsByHabitat.Append(cellsWithHabitat);}
	for (long x=0;x<len_;++x){
		for (long y=0;y<width_;++y){
			GridCell& g = grid_[x][y];			
			ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[g.habitat()];
			cellsWithHabitat.Append(&g);
			}}	

	for(long h=0;h<freqsByHabitat.len();++h){
		StatList<long> geneWeights;
		aList<double>& genomeFreqs = freqsByHabitat[h];
		ObjList<Phenotype >& genesInHabitat = genesByFreqByHabitat[h];
		ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[h];
		long indivsInHabitat	= (long)( initSize *( ((double) (cellsWithHabitat.len() )) / ((double) ( len_ * width_))  ) );
		for(int gg=0;gg<genesInHabitat.len();++gg){			
			long n= (long) (indivsInHabitat * (genomeFreqs[gg]));
			geneWeights.Append(n);}
		while (geneWeights.sum()<indivsInHabitat){//in case frequencies don't divide exactly into initSize
			geneWeights.increment(geneWeights.len()-1);//increment the last one
			}//end while

		//make and initialize each individual and install it in a specified (presently random) grid location
		//that has the proper habitat type
		for (long i=0;i<indivsInHabitat;++i){
			int cellNum = ran.longBetween(0,cellsWithHabitat.len()-1);
			GridCell& g = cellsWithHabitat[cellNum];
			CoordinateType loc = g.coords();
			long x=ran.weighted(geneWeights);
			geneWeights.decrement(x);//samples without replacement
			Phenotype & newGenes=genesInHabitat[x];
			GeneticIndividual<T>* indivType = (GeneticIndividual<T>*) init.indivOfCorrectType;//casting assumes data are correct
			GeneticIndividual<T>* indiv = indivType->Duplicate(newGenes,++totalIndivsCreated_);
			double sexRatio = init.sexRatio(species);
			genderSet sex = indiv->RealizedSexOfClassMember(sexRatio, ran);
			InstallInGrid(species, indiv, loc);
			indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran);//age=1
			}//end for i
		
		}//end for h
	
	//clean up
	for(int h=0;h<gridcellsByHabitat.len();++h){
		ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[h];
		cellsWithHabitat.Detach();}
	gridcellsByHabitat.Clear();
	}//	end GeneticLandScape::SeedLandScapeByHabitat 
*/

void GeneticLandScape::SeedLandScapeByHabitat (LandScapeInitializer01& init, 
					int species, long initSize, Ran3List& ran){
	//puts genetic individuals into an empty landscape
	//according to their genotype and habitat
	ObjList<ObjList<ObjList<Phenotype> > >& genesByFreqByHabitatBySpecies = init.initPhenes_;
	ObjList<ObjList<aList<long double> > >& freqsByHabitatBySpecies = init.initPhenotypeFreqs_;
	//make a weighted aList for genotypes that will be seeded
	ObjList<ObjList<Phenotype> >& genesByFreqByHabitat = genesByFreqByHabitatBySpecies[species];
	ObjList<aList<long double> >& freqsByHabitat = freqsByHabitatBySpecies[species];
	//make up a list of gridcells of each habitat
	ObjList<ObjList<GridCell> > gridcellsByHabitat;
	for(int hh=0;hh<freqsByHabitat.len();++hh){
		ObjList<GridCell>* cellsWithHabitat = NEW ObjList<GridCell>();
		gridcellsByHabitat.Append(cellsWithHabitat);}
	for (long x=0;x<len_;++x){
		for (long y=0;y<width_;++y){
			GridCell& g = grid_[x][y];			
			ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[g.habitat()];
			cellsWithHabitat.Append(&g);
			}}	

	for(long h=0;h<freqsByHabitat.len();++h){
		StatList<long> geneWeights;
		aList<long double>& genomeFreqs = freqsByHabitat[h];
		ObjList<Phenotype>& genesInHabitat = genesByFreqByHabitat[h];
		ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[h];
		long indivsInHabitat	= (long)( initSize *( ((long double) (cellsWithHabitat.len() )) / ((long double) ( len_ * width_))  ) );
		for(int gg=0;gg<genesInHabitat.len();++gg){			
			long n= (long) (indivsInHabitat * (genomeFreqs[gg]));
			geneWeights.Append(n);}
		while (geneWeights.sum()<indivsInHabitat){//in case frequencies don't divide exactly into initSize
			geneWeights.Increment(geneWeights.len()-1);//increment the last one
			}//end while

		//make and initialize each individual and install it in a specified (presently random) grid location
		//that has the proper habitat type
		for (long i=0;i<indivsInHabitat;++i){
			int cellNum = ran.longBetween(0,cellsWithHabitat.len()-1);
			GridCell& g = cellsWithHabitat[cellNum];
			CoordinateType loc = g.coords();
			long x=ran.weighted(geneWeights);
			geneWeights.Decrement(x);//samples without replacement
			Phenotype& newPhenes=genesInHabitat[x];
			Phenotype* newGenes = (Phenotype*) &newPhenes;
			GeneticIndividual* indivType = (GeneticIndividual*) init.indivOfCorrectType;//casting assumes data are correct
			GeneticIndividual* indiv = indivType->Duplicate(*newGenes,++totalIndivsCreated_);
			long double sexRatio = init.sexRatio(species);
			genderSet sex = indiv->RealizedSexOfClassMember(sexRatio, ran);
			InstallInGrid(species, indiv, loc);
			indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran);//age=1
			}//end for i
		
		}//end for h
	
	//clean up
	for(int h=0;h<gridcellsByHabitat.len();++h){
		ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[h];
		cellsWithHabitat.Detach();}
	gridcellsByHabitat.Clear();
	}//	end GeneticLandScape::SeedLandScapeByHabitat 



void GeneticLandScape::SetLociByChromosome(LandScapeInitializer01& init){
	lociPerChromosome_.Clear();/*i=species, j=chromosome; val[i,j]=loci*/
	for(int s=0;s<species();++s){
		lociPerChromosome_.Append(NEW StatList<int>());/*i=species, j=chromosome; val[i,j]=loci*/
		CoreGenome* theGenes;
		int numchroms=0;
		Phenotype& thePhene = init.initPhenes_[s][0][0];//species, habitat, list of phenotypes
		//cannot assume all have the same chromosomal structure
		theGenes = (CoreGenome*) &thePhene;
		for(int pop=0;pop<init.initPhenes_[s].len();++pop){
			for(int i=0; i<init.initPhenes_[s][pop].len();++i){
				if(theGenes->chroms1N() > numchroms){
					numchroms = theGenes->chroms1N();
					theGenes = (CoreGenome*) &init.initPhenes_[s][pop][i];
					}//end if
				}}
		for(int j=0;j<numchroms;++j){
			lociPerChromosome_[s].Append(theGenes->loci(j));
			}
		}
	}//end GeneticLandScape::SetLociByChromosome


void GeneticLandScape::UpdateAlleleList(void){
	for(int s=0;s<species_;++s){UpdateAlleleList(s);}
	}//end GeneticLandScape::UpdateAlleleList




void GeneticLandScape::UpdateAlleleList(int species){
//	ObjList<aList<T> >& allelesInPop= allelesInScape_[species];
	ObjList<StringList>& allelesInPop= allelesInScape_[species];
	allelesInPop.Clear();
	int nloci=loci(species);
	for (int j=0;j<nloci;++j){
//		aList<T>* locList=NEW aList<T>();
		StringList* locList=NEW StringList();
		allelesInPop.Append(locList);
		for (long x=0;x<len_;++x){
			for (long y=0;y<width_;++y){
				GeneticGridCell* g= (GeneticGridCell*) &(grid_[x][y]);
				long nAlleles=g->Alleles(species,j);
				for (int a=0;a<nAlleles;++a){
//					T alleleVal=g->AlleleValue(species,j,a);
					aString& alleleVal=g->AlleleValue(species,j,a);
					if (!(locList->Includes(&alleleVal))){
						locList->Append(NEW aString(alleleVal));}
//						else{
//							delete alleleVal;}
//					alleleVal = NULL;//forget
					}//end for a
				}//end for y
			}//end for x
		locList->Sort();
		}//end for j
//	allelesCurrent_.Replace(1,species);
	allelesCurrent_[species]=1;
	}//end GeneticLandScape::UpdateAlleleList

#endif
