/*******************************************************
*
*	LandScape.cpp
*
*																							
*
*	This class handles processes among grid cells
*
*	Use derived classes for data collection.					
********************************************************/

#ifndef LandScapecpp
#define LandScapecpp = 1


//#include <iostream.h>
//#include <fstream.h>
#include "LandScape.h"
#include "Element.cpp"
#include "aList.cpp"
#include "CoordinateClass.cpp"
  
  

/*
LandScape::LandScape(LandScapeInitializer00& init):
			len_(init.length()), 
			width_(init.width()), 
			gridScale_(init.gridScale()), 
			migrationModel_(init.migrationModel()), 
			species_(init.species()),
			totalIndivsCreated_(0),
			totalPopsCreated_(0),
			selectionSource_(init.selectionSource()),
			maxage_(init.maxage_), 
			initSize_(init.initSize_),
			maxCapacity_(init.maxCapacity_),
			fSurvivalSchedule_(init.fSurvivalSchedule_),
			mSurvivalSchedule_(init.mSurvivalSchedule_),
			fFecunditySchedule_(init.fFecunditySchedule_),
			femaleMovementRate_(init.femaleMovementRate_),
			maleMovementRate_(init.maleMovementRate_)
			{
		}
*/

	
LandScape::LandScape(LandScapeInitializer01& init):
			len_(init.length()), 
			width_(init.width()), 
			gridScale_(init.gridScale()), 
			migrationModel_(init.migrationModel()), 
			species_(init.species()),
			totalIndivsCreated_(0),
			totalPopsCreated_(0),
//			selectionSource_(init.selectionSource()),
			globalTime_(init.globalTime()),
			localTime_(init.localTime()),
			maxage_(init.maxage_), 
			initSize_(init.initSize_),
			maxCapacity_(init.maxCapacity_),
			fSurvivalSchedule_(init.fSurvivalSchedule_),
			mSurvivalSchedule_(init.mSurvivalSchedule_),
			fFecunditySchedule_(init.fFecunditySchedule_),
			femaleMovementRate_(init.femaleMovementRate_),
			maleMovementRate_(init.maleMovementRate_)
			{
		}
	

LandScape::LandScape(int len, //default constructor for only one species
					 int w, 	//default habitat=0, override ILandScape otherwise
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 long double fSurvivalRate,//constant per age class
  					 long double fFecundityRate,//constant per age class
 					 long double femaleMovementRate//constant across habitats, sexes & age classes
// 					 selectionSourceSet sSource
					 ):
			len_(len), width_(w), gridScale_(one), migrationModel_(migrationModel), species_(1),
			totalIndivsCreated_(0),totalPopsCreated_(0),
			globalTime_(0),localTime_(0)/*,
			selectionSource_(sSource) */{
	initSize_.Append(initSize);
	maxage_.Append(maxage);
	maxCapacity_.Append(maxCapacity);
	aList<long double>* fSurvival=NEW aList<long double>();
	aList<long double>* mSurvival=NEW aList<long double>();
	aList<long double>* fecundity=NEW aList<long double>();
	fSurvivalSchedule_.Append(fSurvival);
	mSurvivalSchedule_.Append(mSurvival);
	fFecunditySchedule_.Append(fecundity);
	ObjList<aList<long double> >* fHabitatList=NEW ObjList<aList<long double> >();
	ObjList<aList<long double> >* mHabitatList=NEW ObjList<aList<long double> >();
	femaleMovementRate_.Append(fHabitatList);
	maleMovementRate_.Append(mHabitatList);
	aList<long double>* fDispersalList=NEW aList<long double>();
	aList<long double>* mDispersalList=NEW aList<long double>();
	fHabitatList->Append(fDispersalList);
	mHabitatList->Append(mDispersalList);
	int i;
	for (i=0;i<=maxage;++i){//<= means that age class 0 (zygotes) is represented
		fSurvival->Append(fSurvivalRate);
		mSurvival->Append(fSurvivalRate);//copies female by default
					//clear and recopy when constructing relevant derived classes
		fecundity->Append(fFecundityRate);
		fDispersalList->Append(femaleMovementRate);
		mDispersalList->Append(femaleMovementRate);		
		}
	}// end constructor LandScape::LandScape(one species)



LandScape::LandScape(int len, //default constructor for only one species
					 int w, 	//default habitat=0, override ILandScape otherwise
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 aList<long double>* fSurvivalSchedule,//be sure to include age class 0
  					 aList<long double>* fFecunditySchedule,//be sure to include age class 0
 					 ObjList<aList<long double> >* femaleMovementRate //be sure to include age class 0
// 					 selectionSourceSet sSource
					 ):
			len_(len), width_(w), gridScale_(one), migrationModel_(migrationModel), species_(1),
			totalIndivsCreated_(0),totalPopsCreated_(0),
			globalTime_(0),localTime_(0)/*,
			selectionSource_(sSource)*/ {
	initSize_.Append(initSize);
	maxage_.Append(maxage);
	maxCapacity_.Append(maxCapacity);
	aList<long double>* fSurvival=NEW aList<long double>(fSurvivalSchedule);
	fSurvivalSchedule_.Append(fSurvival);
	aList<long double>* mSurvival=NEW aList<long double>(fSurvivalSchedule);
	mSurvivalSchedule_.Append(mSurvival);//copies female by default
					//clear and recopy when constructing relevant derived classes
	aList<long double>* fFecundity=NEW aList<long double>(fFecunditySchedule);
	fFecunditySchedule_.Append(fFecundity);
	ObjList<aList<long double> >* fMovement=NEW ObjList<aList<long double> >(femaleMovementRate);
	femaleMovementRate_.Append(fMovement);
	ObjList<aList<long double> >* mMovement=NEW ObjList<aList<long double> >(femaleMovementRate);
	maleMovementRate_.Append(mMovement);//male rates equal female rates by default
	}// end constructor LandScape::LandScape(one species)




LandScape::LandScape(int len, //constructor for one or multiple species
					 int w, 	//default habitat=0, override ILandScape otherwise
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<long double> >* fSurvivalSchedule,//be sure to include age class 0
  					 ObjList<aList<long double> >* fFecunditySchedule,//be sure to include age class 0
 					 ObjList<ObjList<aList<long double> > >* femaleMovementRate //be sure to include age class 0
// 					 selectionSourceSet sSource
					 ):
		len_(len), width_(w), gridScale_(one), migrationModel_(migrationModel), species_(species), 
		maxage_(maxage), 
		initSize_(initSize),
		maxCapacity_(maxCapacity),
		globalTime_(0), localTime_(0),
		fSurvivalSchedule_(fSurvivalSchedule),
		mSurvivalSchedule_(fSurvivalSchedule),//copies female by default
						//clear and recopy when constructing relevant derived classes
		fFecunditySchedule_(fFecunditySchedule),
		femaleMovementRate_(femaleMovementRate),
		maleMovementRate_(femaleMovementRate),//default male rates equal female rates
		totalIndivsCreated_(0),totalPopsCreated_(0)/*,
		selectionSource_(sSource)*/{

//	ILandScape(ran);
	}// end constructor LandScape::LandScape(multiple species)



LandScape::~LandScape(void){
		grid_.Clear();
		fSurvivalSchedule_.Clear();
		mSurvivalSchedule_.Clear();
		fFecunditySchedule_.Clear();
		maxage_.Clear();
		initSize_.Clear();
		maxCapacity_.Clear();
		migrationMatrix_.Clear();
		cout<<"landscape destroyed"<<endl;
		};//end destructor LandScape::~LandScape



//FUNCTIONS  



void LandScape::AdultDispersal(int species, Ran3List& ran, aList<long double>& dispersalInfo){
	int x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g= grid_[x][y];
			g.AdultDispersal(species, ran,dispersalInfo);}}
	}//end LandScape::AdultDispersal
	


long LandScape::adults(int species){
	long tot=0, x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			tot+=g.adults(species);}}
	return tot;
	}//end LandScape::adults
	

void LandScape::AgeOneStep(int species){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.AgeOneStep(species, maxage_[species]);}}
	}//end LandScape::AgeOneStep
	

long LandScape::babies(int species){
	long tot=0,x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			tot+=g.babies(species);}}
	return tot;
	}//end LandScape::babies
	

void LandScape::BuryTheDead(int species){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.BuryTheDead(species);}}
	}//end LandScape:BuryTheDead



long LandScape::capacity(int species){
	long sum=0,x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			sum+=g.carryingCapacity(species);}}
	return sum;
	}//end LandScape::capacity


void LandScape::ClearOffspringLists(int species){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.ClearOffspringLists(species);}}
	}// end LandScape::ClearOffspringLists



void LandScape::CreateMigrationMatrix(void){
	/*creates a default migration matrix
	 //ObjList<ObjList<ObjList<List<double> > > > migrationMatrix_;
 		probability of moving from one habitat type to another
 		by species, age class, originating habitat, and target habitat
 		this need not be symmetrical, but the default has only one habitat with migration  prob = 1.0*/
	migrationMatrix_.Clear();
	int s,a;
	for(s=0;s<species_;++s){
		ObjList<ObjList<aList<long double> > >* speciesMigrationMatrix = NEW ObjList<ObjList<aList<long double> > >();
		migrationMatrix_.Append(speciesMigrationMatrix);
		for(a=0;a<maxage_[s]; ++a){
			ObjList<aList<long double> >* ageMigrationMatrix = NEW ObjList<aList<long double> >();
			speciesMigrationMatrix->Append(ageMigrationMatrix);
			aList<long double>* sourceMigrationVector = NEW aList<long double>();
			ageMigrationMatrix->Append(sourceMigrationVector);
			sourceMigrationVector->Append(one);//always move by default
			}//end for a
		}/*end for s*/
	}/*end LandScape::CreateMigrationMatrix*/


void LandScape::CreateMigrationMatrix(LandScapeInitializer00& init){
	CreateMigrationMatrix(init.migrationMatrix_);
	}//end LandScape::CreateMigrationMatrix



void LandScape::CreateMigrationMatrix(LandScapeInitializer01& init){
	CreateMigrationMatrix(init.migrationMatrix_);
	}/*end LandScape::CreateMigrationMatrix*/




void LandScape::CreateMigrationMatrix(ObjList<ObjList<ObjList<aList<long double> > > >& migrationMatrix){
	migrationMatrix_.Clear();
	int s,a,hab1,hab2;
	for(s=0;s<species_;++s){
		ObjList<ObjList<aList<long double> > >* speciesMigrationMatrix = NEW ObjList<ObjList<aList<long double> > >();
		ObjList<ObjList<aList<long double> > >& spMigrationMatrix = migrationMatrix[s];
		migrationMatrix_.Append(speciesMigrationMatrix);
//		for(a=0;a<maxage_[s]; ++a){
		for(a=0;a<=maxage_[s]; ++a){
			ObjList<aList<long double> >* ageMigrationMatrix = NEW ObjList<aList<long double> >();
			ObjList<aList<long double> >& aMigrationMatrix =spMigrationMatrix[a];
			speciesMigrationMatrix->Append(ageMigrationMatrix);
			for (hab1=0;hab1<aMigrationMatrix.len();++hab1){
				aList<long double>* sourceMigrationVector = NEW aList<long double>();
				aList<long double>& sMigrationVector = aMigrationMatrix[hab1];
				ageMigrationMatrix->Append(sourceMigrationVector);
				for(hab2=0;hab2<sMigrationVector.len();++hab2){
					sourceMigrationVector->Append(sMigrationVector[hab2]);//always move by default
					}//end hab2
				}//end hab1
			}//end for a
		}//end for s
	}//end LandScape::CreateMigrationMatrix(ObjL<<<<dbl>>>>)


long LandScape::dead(int species){
	long tot=0,x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			tot+=g.dead(species);}}
	return tot;
	}//end LandScape::dead
	



//void LandScape::Dispersal(int species, Ran3List& ran, double m,double kurtosisFactor){
void LandScape::Dispersal(int species, Ran3List& ran, aList<long double>& dispersalInfo){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g= grid_[x][y];
//			g.Dispersal(species, ran,m,kurtosisFactor);}}
			g.Dispersal(species, ran,dispersalInfo);}}
	}//end LandScape::Dispersal
	


void LandScape::DoOneCycle (aList<int>& cycleSequence, int species, Ran3List& ran, ObjList<aList<long double> >& dispersalInfo,
							 ObjList<aList<long double> >& reproductionInfo){
			//dispersalInfo and reproductionInfo lists indexed by species
			//contents depend on derived classes
			//contents passed to indviduals as they behave
	TimeIncrement();
	int c;
	//age, disperse, mate, reproduce, viabilitySelection, replaceDead, buryDead
	for(c=0;c<cycleSequence.len();++c){
		lifeCycleSteps step = (lifeCycleSteps) cycleSequence[c];
		switch(step){
			case age:
				AgeOneStep(species);
				break;
			case disperse:
				AdultDispersal(species, ran, dispersalInfo[species]);/*use m<1 in island model*/
				ImmatureDispersal(species, ran, dispersalInfo[species]);/*use m<1 in island model*/
				GameteDispersal(species, ran, dispersalInfo[species]);/*use m<1 in island model*/
				break;
			case adultsDisperse:
				AdultDispersal(species, ran, dispersalInfo[species]);/*use m<1 in island model*/
				break;
			case babiesDisperse:
				ImmatureDispersal(species, ran, dispersalInfo[species]);/*use m<1 in island model*/
				break;
			case gametesDisperse:
				GameteDispersal(species, ran, dispersalInfo[species]);/*use m<1 in island model*/
				break;
			case mate:
				MateOnGrid(species, ran, reproductionInfo[species]);
				break;
			case recruitToAdult:
				Recruit(species, ran);
				break;
			case reproduce:
				Reproduce(species, ran, reproductionInfo[species]);
				break;
			case reproduceSyngamy:
				ReproduceBySyngamy(species, ran, reproductionInfo[species]);
				break;
			case viabilitySelection:
				ViabilitySelection(species,ran);
				break;
			case adultMortality:
				SomeAdultsDie(species, ran);
				break;
			case immatureMortality:
				SomeImmaturesDie(species, ran);
				break;
			case gameteMortality:
				SomeGametesDie(species, ran);
				break;
			case clearOffspringLists:
				ClearOffspringLists(species);
				break;
			case replaceDead:
				ReplaceTheDead(species, ran);
				break;
			case buryDead://take data from dead individuals before calling BuryTheDead
				BuryTheDead(species);
				break;
			default:
				break;
			}
		}//c
	}/*end LandScape::DoOneCycle*/


void LandScape::DoOneCycle (aList<int>& cycleSequence, Ran3List& ran,
							 ObjList<aList<long double> >& dispersalInfo,
							 ObjList<aList<long double> >& reproductionInfo){
	/*PROBABLY BEST TO OVERRIDE THIS IN DERIVED CLASSES*/
	TimeIncrement();
	int c,s;
	//age, disperse, mate, reproduce, viabilitySelection, replaceDead, buryDead
	for(c=0;c<cycleSequence.len();++c){
		lifeCycleSteps step = (lifeCycleSteps) cycleSequence[c];
		switch(step){
			case age:
				for(s=0;s<species();++s){
					AgeOneStep(s);}
				break;
			case disperse:
				for(s=0;s<species();++s){
					AdultDispersal(s, ran, dispersalInfo[s]);/*use m<1 in island model*/
					ImmatureDispersal(s, ran, dispersalInfo[s]);/*use m<1 in island model*/
					GameteDispersal(s, ran, dispersalInfo[s]);/*use m<1 in island model*/
					}
				break;
			case adultsDisperse:
				for(s=0;s<species();++s){
					AdultDispersal(s, ran, dispersalInfo[s]);}/*use m<1 in island model*/
				break;
			case babiesDisperse:
				for(s=0;s<species();++s){
					ImmatureDispersal(s, ran, dispersalInfo[s]);}/*use m<1 in island model*/
				break;
			case gametesDisperse:
				for(s=0;s<species();++s){
					GameteDispersal(s, ran, dispersalInfo[s]);}/*use m<1 in island model*/
				break;
			case mate:
				for(s=0;s<species();++s){
					MateOnGrid(s, ran, reproductionInfo[s]);}
				break;
			case recruitToAdult:
				for(s=0;s<species();++s){
					Recruit(s, ran);}
				break;
			case reproduce:
				for(s=0;s<species();++s){
					Reproduce(s, ran, reproductionInfo[s]);}
				break;
			case reproduceSyngamy:
				for(s=0;s<species();++s){
					ReproduceBySyngamy(s, ran, reproductionInfo[s]);}
				break;
			case viabilitySelection:
				for(s=0;s<species();++s){
					ViabilitySelection(s,ran);}
				break;
			case adultMortality:
				for(s=0;s<species();++s){
					SomeAdultsDie(s, ran);}
				break;
			case immatureMortality:
				for(s=0;s<species();++s){
					SomeImmaturesDie(s, ran);}
				break;
			case clearOffspringLists:
				for(s=0;s<species();++s){
					ClearOffspringLists(s);}
				break;
			case replaceDead:
				for(s=0;s<species();++s){
					ReplaceTheDead(s, ran);}
				break;
			case buryDead://take data from dead individuals before calling BuryTheDead
				for(s=0;s<species();++s){
					BuryTheDead(s);}
				break;
			default:
				break;
			}
		}//c
	}/*end LandScape::DoOneCycle*/


void LandScape::DropNeighbors(GridCell* g1, GridCell* g2){
	g1->DropNeighbor(g2);
	g2->DropNeighbor(g1);
	}//end LandScape::dropNeighbors


long	LandScape::extantPops(int species){
	long sum=0,x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			if (!(g.extinct(species))) {++sum;}
			}}
	return sum;
	}//end LandScape::extantPops



int LandScape::extinct(int species){
	int answer=1;//true
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			if (!(g.extinct(species))) {return 0;}//false
			}}
	return answer;
	}//end LandScape::extinct


void LandScape::ExtractFromGrid (int species, CoordinateType loc, Individual* indiv){
//	ObjList<GridCell>* row=grid_[loc[0]];
//	GridCell* g= row->Nth(loc[1]);
	GridCell& g=grid_[loc[0]][loc[1]];
	g.Extract(species, indiv);
	}//end LandScape::ExtractFromGrid 
	
	
long double LandScape::Fecundity(int species, int age){
	//returns the fecundity schedule value
	return fFecunditySchedule_[species][age];
	}//end LandScape::Fecundity

	
	
void LandScape::GameteDispersal(int species, Ran3List& ran, aList<long double>& dispersalInfo){
	int x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g= grid_[x][y];
			g.GameteDispersal(species, ran,dispersalInfo);}}
	}//end LandScape::ImmatureDispersal
	


void LandScape::GetAgeDistribution(int species, aList<int>& ageDist){
	long oldLen=ageDist.len(),i,x,y;
	ageDist.Clear();
	for(i=0;i<oldLen;++i){ageDist.Append(0);}
	aList<int> popAgeDist;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			popAgeDist.Clear();
			for (i=0;i<oldLen;++i){popAgeDist.Append(0);}
			g.GetAgeDistribution(species, popAgeDist);
			for (i=0;i<ageDist.len();++i){
				ageDist.Replace(ageDist[i]+popAgeDist[i],i);}
			}
			}//end for x,y
	}//end LandScape::GetAgeDistribution


void LandScape::GetCoordinates (int species, int popID, CoordinateType& coords){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			Population& pop = g.pop(species);
			if (pop.id() == popID){
				coords = pop.coords();
				return;}
			}}
	}/*end LandScape::GetCoordinates*/



void LandScape::GetCoordinates (int popID, CoordinateType& coords){
	long x,y,s;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			for(s=0;s<species();++s){
				Population& pop = g.pop(s);
				if (pop.id() == popID){
					coords = pop.coords();
					return;}
			}}}
	}/*end LandScape::GetCoordinates*/




CoordinateType LandScape::GetNewLocCoordinates (CoordinateType oldLoc,
												long xdev, long ydev){
	//handles dispersal across the torus edges
	CoordinateType newLoc;
	newLoc[0] = oldLoc[0] + xdev;
	if (newLoc[0] <= 0){ 
			while (newLoc[0] <= 0) {newLoc[0] += len_;}}
		else if (newLoc[0] > len_){ 
			while (newLoc[0] > len_) {newLoc[0] -= len_;}}
	newLoc[1] = oldLoc[1] + ydev;
	if (newLoc[1] <= 0) {
			while (newLoc[1] <= 0){newLoc[1] += width_;}}
		else if (newLoc[1] > width_){
			while (newLoc[1] > width_){newLoc[1] -= width_;}}
	return(newLoc);
	}//	end LandScape::GetNewLocCoordinates 


GridCell& LandScape::GetGridCell (CoordinateType loc){
//	ObjList<GridCell>* row=grid_[loc[0]];
	return grid_[loc[0]][loc[1]];
//	return row->Nth(loc[1]);
	}//end LandScape::GetGridCell


Population& LandScape::GetPopulation (int species, CoordinateType loc){
	GridCell& g= GetGridCell(loc);
	return g.pop(species);
	}//end LandScape::GetPopulation
	
	
int LandScape::habitat(CoordinateType loc){
	GridCell& g= GetGridCell(loc);
	return g.habitat();
	}//end LandScape::habitat


long LandScape::habitats(void){
	ObjList<aList<long double> >& ratesByHabitat= femaleMovementRate_[0];
	return ratesByHabitat.len();
	}//end LandScape::habitats


ObjList<aList<int> >* LandScape::habitatGrid(void){
	//returns grid of habitat scores; must be destroyed elsewhere
	ObjList<aList<int> >* habgridPtr = NEW ObjList<aList<int> >();
	ObjList<aList<int> >& habgrid = *habgridPtr;
	int row,col;
	for(row=0;row<len_;++row){
		habgrid.Append(NEW aList<int>());
		for(col=0;col<width_;++col){
			habgrid[row].Append(grid_[row][col].habitat());
			}}
	return habgridPtr;	
	}//end LandScape::habitatGrid



void LandScape::ILandScape(Ran3List& ran,int seeded){
	//only called just after constructors. Exists to avoid code duplication
	int row,col,s;
	CoordinateType coords;
	for(row=0;row<len_;++row){
		ObjList<GridCell>*  gridList = NEW ObjList<GridCell>();
		grid_.Append(gridList);
		for(col=0;col<width_;++col){
			coords.set(row,col);
			GridCell* g = NEW GridCell(this, coords,0);//habitat=0
			gridList->Append(g);
			for (s=0;s<species_;++s){
				Population* pop= NEW 
						Population(totalPopsCreated_++,s,g,maxCapacity_[s]);
				g->Append(pop);
				}//end for species
			}//end for cols
		}// end for rows	
	CreateMigrationMatrix();
	cout<<len_<<" x "<<width_<<" landscape created"<<endl;
	if(seeded){
		for (int s=0;s<species_;++s){
			SeedLandScape(s, initSize_[s], ran);}}
	}//end LandScape::ILandScape

/*
void LandScape::ILandScape(LandScapeInitializer00& init, Ran3List& ran){
	//only called just after constructors. Exists to avoid code duplication
	int row,col;
	CoordinateType coords;
	for(row=0;row<len_;++row){
		ObjList<GridCell>*  gridList = NEW ObjList<GridCell>();
		grid_.Append(gridList);
		for(col=0;col<width_;++col){
			coords.set(row,col);
//			GridCell* g = NEW GridCell(this, coords,0);//habitat=0
			GridCell* g =init.gridcellOfCorrectType->Duplicate(this, coords,0);//habitat=0
			gridList->Append(g);
			for (int s=0;s<species_;++s){
				Population* pop= init.popOfCorrectType->Duplicate(totalPopsCreated_++,s,g,maxCapacity_[s]);
				g->Append(pop);
				}//end for species
			}//end for cols
		}// end for rows	
	CreateMigrationMatrix();
	cout<<len_<<" x "<<width_<<" landscape created"<<endl;
	for (int s=0;s<species_;++s){
		SeedLandScape(init, s, initSize_[s], ran);}
	}//end LandScape::ILandScape
*/

void LandScape::ILandScape(LandScapeInitializer01& init, Ran3List& ran, int seeded){
	//only called just after constructors. Exists to avoid code duplication
	int row,col,s;
	CoordinateType coords;
	for(row=0;row<len_;++row){
		ObjList<GridCell>*  gridList = NEW ObjList<GridCell>();
		grid_.Append(gridList);
		for(col=0;col<width_;++col){
			coords.set(row,col);
//			GridCell* g = NEW GridCell(this, coords,0);//habitat=0
			GridCell* g =init.gridcellOfCorrectType->Duplicate(this, coords,0);//habitat=0
			gridList->Append(g);
			for (s=0;s<species_;++s){
				Population* pop= init.popOfCorrectType->Duplicate(totalPopsCreated_++,s,g,maxCapacity_[s]);
				g->Append(pop);
				}//end for species
			}//end for cols
		}// end for rows	
	CreateMigrationMatrix(init.migrationMatrix_);
	cout<<len_<<" x "<<width_<<" landscape created"<<endl;
	if(seeded){
		for (s=0;s<species_;++s){
			SeedLandScape(init, s, initSize_[s], ran);}}
	}//end LandScape::ILandScape

/*
void LandScape::ILandScapeByHabitat (Ran3List& ran, LandScapeInitializer00& init){
	ILandScapeByHabitat(ran,init.habitatGrid_);
	}//end LandScape::ILandScapeByHabitat
*/

void LandScape::ILandScapeByHabitat (Ran3List& ran, LandScapeInitializer01& init,int seeded){
	ILandScapeByHabitat(ran,init.habitatGrid_,seeded);
	}/*end LandScape::ILandScapeByHabitat*/



void LandScape::ILandScapeByHabitat (Ran3List& ran, ObjList<aList<int> >& habitatGrid,int seeded){
	SetHabitatTypes(habitatGrid);
	ILandScape(ran,seeded);
	}/*end CLandScape::ILandScapeByHabitat*/

/*
void LandScape::ILandScapeByHabitat (LandScapeInitializer00& init, Ran3List& ran, ObjList<aList<int> >& habitatGrid){
	SetHabitatTypes(habitatGrid);
	ILandScape(init, ran);
	}//end CLandScape::ILandScapeByHabitat
*/

/*
void LandScape::ILandScapeByHabitat (LandScapeInitializer01& init, Ran3List& ran, ObjList<aList<int> >& habitatGrid){
	SetHabitatTypes(habitatGrid);
	ILandScape(init, ran);
	}//end CLandScape::ILandScapeByHabitat
*/


void LandScape::ImmatureDispersal(int species, Ran3List& ran, aList<long double>& dispersalInfo){
	int x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g= grid_[x][y];
			g.ImmatureDispersal(species, ran,dispersalInfo);}}
	}//end LandScape::ImmatureDispersal
	


long LandScape::immigrants(int species){//number of new migrants arriving
	long imm=0,x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			imm += g.immigrants(species);}}
	return imm;
	}/*end LandScape::immigrants*/



void LandScape::InstallInGrid (int species, Individual* indiv, CoordinateType loc){
	//use an independent procedure to change the individual's data
//	ObjList<GridCell>* row=grid_[loc[0]];
//	GridCell* g= row->Nth(loc[1]);
	GridCell& g=grid_[loc[0]][loc[1]];
	g.Install(species, indiv);
	} //end LandScape::InstallInGrid






void LandScape::MateOnGrid (int species, Ran3List& ran, aList<long double>& reproductionInfo){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.AssignMates(species, ran, reproductionInfo);}}
	}/*end LandScape::MateOnGrid*/


long double LandScape::MigrationProb(int species, int ageClass, int currentHabitat, int potentialHabitat){
//	 ObjList<ObjList<ObjList<List<double> > > > migrationMatrix_;
//	 0th age class cannot migrate, so it is eliminated from the list and ageClass-1 is used
//above coment obsolete -> individuals with gamete classes can migrate now
	 ObjList<ObjList<aList<long double> > >& spMigrationMatrix = migrationMatrix_[species];
//	 ObjList<aList<long double> >& ageMigrationMatrix = spMigrationMatrix[ageClass-1];
	 ObjList<aList<long double> >& ageMigrationMatrix = spMigrationMatrix[ageClass];
	 aList<long double>& currentHabitatMigrationVector = ageMigrationMatrix[currentHabitat];
	 return currentHabitatMigrationVector[potentialHabitat];
	}/*end LandScape::MigrationProb*/



long double	LandScape::moveRate(int species, int habitat, genderSet sex, int age){
	if (sex==M){
			ObjList<aList<long double> >& speciesList = maleMovementRate_[species];
			aList<long double>&  habitatList = speciesList[habitat];
			return habitatList[age];
			} 
		else{//sex = A, B, or F
			ObjList<aList<long double> >& speciesList = femaleMovementRate_[species];
			aList<long double>&  habitatList = speciesList[habitat];
			return habitatList[age];
			} 
}/*end LandScape::moveRate*/

/*
void LandScape::NaturalSelection(int species, Ran3List& ran){
	for (long x=0;x<len_;++x){
		for (long y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.NaturalSelection(species, ran, selectionSource());}}
	}//end LandScape::NaturalSelection
*/


ObjList<aList<int> >* LandScape::PopIDGrid(int species){
	//returns grid of pop IDs; must be destroyed elsewhere
	ObjList<aList<int> >* popIDgridPtr = NEW ObjList<aList<int> >();
	ObjList<aList<int> >& popIDgrid = *popIDgridPtr;
	int row,col;
	for(row=0;row<len_;++row){
		popIDgrid.Append(NEW aList<int>());
		for(col=0;col<width_;++col){
			Population& p = grid_[row][col].pop(species);
			popIDgrid[row].Append(p.id());
			}}
	return popIDgridPtr;	
	}//end LandScape::PopIDGrid



void LandScape::PrintAgeDistribution(ostream& outFile, int species, aString* msg){
	if(msg!=NULL){outFile<<*msg<<endl;}
	aList<int> ageDist;
	int i,j,a;
	for(i=0;i<=maxage_[species];++i){ageDist.Append(0);}/*age class 0 is for zygotes*/
	GetAgeDistribution(species, ageDist);
	outFile<<".";
	for (j=0;j<ageDist.len();++j){
		a=outFile.width(4);
		outFile<<ageDist[j]; outFile<<" ";}/*4 digits*/
	outFile<<endl;
	}/*end LandScape::PrintAgeDistribution*/



void LandScape::PrintGridConnections(ostream& outFile, aString* msg){
	/*prints out the current connections by gridcell*/
	if(msg!=NULL){outFile<<*msg<<endl;}
	long x,y,n;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			CoordinateType gLoc=g.coords();
			aString* line=NEW aString("cell ");
			line->cat("[");line->cat(gLoc[0]);line->cat(",");
			line->cat(gLoc[1]);line->cat("]");
			line->cat(" has neighbors: ");
			for (n=0;n<g.neighbors();++n){
				GridCell* adjacent=g.neighbor(n);
				if (adjacent==NULL){line->cat("none ");}
					else{
						CoordinateType nLoc=adjacent->coords();
						line->cat("(");line->cat(nLoc[0]);
						line->cat(",");line->cat(nLoc[1]);line->cat(")");
						}//end if..else
				if (n<g.neighbors()-1){line->cat(", ");}
				}//end for n
			outFile<<*line<<endl;
			delete line;
			}}//end for x & y
	outFile<<endl;
	}//end LandScape::PrintGridConnections
						
			



void LandScape::PrintGridOccupancy (ostream& outFile, int species, aString* msg){
	if(msg!=NULL){outFile<<*msg<<endl;}
	long num, sum=0,x,y;
	int a, old=outFile.width(3);
	for (y=0;y<width_;++y){
		for (x=0;x<len_;++x){
			GridCell& g = grid_[x][y];
			num = g.size(species);
			a=outFile.width(3);
			outFile<<num<<" ";// : 3
			sum += num;}//end for x
		outFile<<endl;}//end for y
	outFile<<"total: ";
	a=outFile.width(4);
	outFile<<sum<<endl;
	outFile<<endl<<endl;
	a=outFile.width(old);
	}//end  LandScape::PrintGridOccupancy





void LandScape::Recruit (int species, Ran3List& ran){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.Recruit(species, ran);}}
	}// end LandScape::ReplaceTheDead
	


void LandScape::ReplaceTheDead (int species, Ran3List& ran){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.ReplaceTheDead(species, ran);}}
	}// end LandScape::ReplaceTheDead
	



void LandScape::ReplaceTheDeadUsingLandscapeWideCarryingCapacity (int species, Ran3List& ran){
	//Use for continuous dispersal models where landscape  represents deme area and the grid describes location within a deme
	/* randomly chooses new recruits from babies on the landScape*/
	long recruits=capacity(species) - adults(species);
	if (recruits<0) {recruits=0;}
	long nthBabyInPop,tot=0,num=-1,rowNum,colNum,x,y,b,vacancies,p,i;
	unsigned int nthpop;
	if (recruits>0){
		/*first make a list of babies available
		remembering that local carrying capacity can't be exceeded*/
		aList<int> babiesByPop;
		for (x=0;x<len_;++x){
			for (y=0;y<width_;++y){
				GridCell& g=grid_[x][y];
				b=g.babies(species);
				vacancies=g.carryingCapacity(species) - g.adults(species);
				if (vacancies<0) {vacancies=0;}
				babiesByPop.Append(MIN(vacancies,b));
				tot+=MIN(vacancies,b);}}
		recruits=MIN(tot,recruits);
		for(i=0;i<recruits;++i){
			p = ran.weighted(babiesByPop);/*a random pop & baby*/
			nthpop = p+1;
			babiesByPop.Replace(babiesByPop[p]-1,p);/*decrement weights*/
			/*get the population and recruit the baby*/
			rowNum=DIV(nthpop,width_);
			if (MOD(nthpop,width_)==0){--rowNum;}
//			ObjList<GridCell>* row=grid_[rowNum];
			colNum=nthpop - rowNum*width_ -1;
//			GridCell* g= row->Nth(colNum);
			GridCell& g = grid_[rowNum][colNum];
			nthBabyInPop=ran.longBetween(0,g.babies(species)-1);/*tells index of baby in pop*/
			g.Recruit(species, nthBabyInPop);
			
			}/*end for i...recruits*/
		}/*end if recruits*/
	}/* end  LandScape::ReplaceTheDeadUsingLandscapeWideCarryingCapacity*/



void LandScape::Reproduce (int species, Ran3List& ran, aList<long double>& reproductionInfo){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.Reproduce(species, totalIndivsCreated_, ran, reproductionInfo);}}
	}// end LandScape::Reproduce
	


void LandScape::ReproduceBySyngamy (int species, Ran3List& ran, aList<long double>& reproductionInfo){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.ReproduceBySyngamy(species, totalIndivsCreated_, ran, reproductionInfo);}}
	}// end LandScape::ReproduceBySyngamy
	



void LandScape::SeedLandScape (int species, long initSize, Ran3List& ran){
	//puts individuals randomly into an empty landscape
	//inherit this for separate sexes etc
	genderSet sex = A;//default asexual
	//make and initialize each individual and install it in a specified (presently random) grid location
	CoordinateType loc;
	long i;
	for (i=0;i<initSize;++i){
		loc.RandomLoc(0,len_-1,0,width_-1,ran);
		Individual* indiv = NEW Individual(++totalIndivsCreated_);
		InstallInGrid(species, indiv, loc);
		indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran);//age=1
		}
	}//	end LandScape::SeedLandScape 



void LandScape::SeedLandScape (int species, Individual* indiv, Ran3List& ran){
	//puts designated individual randomly into the landscape
	//inherit this for separate sexes etc
	//make and initialize each individual and install it in a specified (presently random) grid location
	CoordinateType loc;
	loc.RandomLoc(0,len_-1,0,width_-1,ran);
	InstallInGrid(species, indiv, loc);
	}//	end LandScape::SeedLandScape 



/*
void LandScape::SeedLandScape (LandScapeInitializer00& init, int species, long initSize, Ran3List& ran){
	//puts individuals of the correct type randomly into an empty landscape
	//inherit this for separate sexes etc
	//make and initialize each individual and install it in a specified (presently random) grid location
	CoordinateType loc;
	for (long i=0;i<initSize;++i){
		loc.RandomLoc(0,len_-1,0,width_-1,ran);
		Individual* indiv = init.indivOfCorrectType->Duplicate(++totalIndivsCreated_);
		InstallInGrid(species, indiv, loc);
		double sexRatio = init.sexRatio(species);
		genderSet sex = indiv->RealizedSexOfClassMember(sexRatio,ran);//default asexual
		indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran, init);//age=1
		}//end for i
	}//	end LandScape::SeedLandScape 
*/


void LandScape::SeedLandScape (LandScapeInitializer01& init, int species, long initSize, Ran3List& ran){
	//puts individuals of the correct type randomly into an empty landscape
	//inherit this for separate sexes etc
	//make and initialize each individual and install it in a specified (presently random) grid location
	CoordinateType loc;
	aList<long double> extraInfo;
	extraInfo.Append(zero);
	extraInfo.Append(zero);
	long i;
	for (i=0;i<initSize;++i){
		loc.RandomLoc(0,len_-1,0,width_-1,ran);
		extraInfo[0]=(long double) GetPopulation(species,loc).id();
		extraInfo[1]=(long double)i;
		Individual* indiv = init.indivOfCorrectType->Duplicate(++totalIndivsCreated_);
		InstallInGrid(species, indiv, loc);
		long double sexRatio = init.sexRatio(species);
		genderSet sex = indiv->RealizedSexOfClassMember(sexRatio,ran);//default asexual
		indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran, init, extraInfo);//age=1
		}//end for i
	}//	end LandScape::SeedLandScape 



void LandScape::SetCapacity (long N, int species){
		maxCapacity_[species] = N;}



void LandScape::SetDispersalMatrix(ObjList<ObjList<aList<long double> > >& dispersalMatrix){
	/*ObjList<ObjList<List<double> > > femaleMovementRate_;//by species, habitat, and age class
 	//ObjList<ObjList<List<double> > > maleMovementRate_;//by species, habitat, and age class*/
	femaleMovementRate_.Clear();
	maleMovementRate_.Clear();
	int s,hab,a;
	for(s=0;s<dispersalMatrix.len();++s){
		ObjList<aList<long double> >* fSpeciesList = NEW ObjList<aList<long double> >();
		ObjList<aList<long double> >* mSpeciesList = NEW ObjList<aList<long double> >();
		femaleMovementRate_.Append(fSpeciesList);
		maleMovementRate_.Append(mSpeciesList);
		ObjList<aList<long double> >& sourceSpeciesList = dispersalMatrix[s];
		for(hab=0;hab<sourceSpeciesList.len();++hab){
			aList<long double>* fHabitatList = NEW aList<long double>();
			aList<long double>* mHabitatList = NEW aList<long double>();
			fSpeciesList->Append(fHabitatList);
			mSpeciesList->Append(mHabitatList);
			aList<long double>& sourceHabitatList = sourceSpeciesList[hab];
			for (a=0;a<sourceHabitatList.len();++a){
				fHabitatList->Append(sourceHabitatList[a]);
				mHabitatList->Append(sourceHabitatList[a]);
				}/*end for a*/
			}/*end for hab*/
		}/*end for s*/
	}/*end LandScape::SetDispersalMatrix*/


void LandScape::SetNeighbors(GridCell& g1, GridCell& g2){
	/*directionless */
	g1.SetNeighbor(&g2);
	g2.SetNeighbor(&g1);
	}/*end LandScape::setNeighbors*/
	
	
void LandScape::SetIslandGridNeighbors(void){
	//any shape habitats ok
	//all grid cells connect to all others except self
	// directions:  arbitrary and not consistently set
	long x1,x2,y1,y2;
	for (x1=0;x1<len_;++x1){
//		ObjList<GridCell>* row1=grid_[x1];
		for (y1=0;y1<width_;++y1){
			GridCell& g1=grid_[x1][y1];
			for (x2=0;x2<len_;++x2){
				for (y2=0;y2<width_;++y2){
					GridCell& g2=grid_[x2][y2];
					if (g1!=g2){//excludes self
						g1.SetNeighbor(&g2);}
					}//end for y2
				}//end for x2
			}//end for y1
		}//end for x1
	}//end LandScape::SetIslandGridNeighbors
		
	
void LandScape::SetGridNeighbors(void){
	//4-edged, square grid cells; 2-D or linear habitats ok
	//no loop or torus structure
	// directions:  0:north; 1:east; 2:south; 3:west
	// where no neighbor exists, a NULL pointer sits
	GridCell* dummy=NULL;//make fully NULL list first
	long x,y,d;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			for(d=0;d<4;++d){g.SetNeighbor(dummy);}
			}
		}
	//replace NULLs with appropriate cells
	for (x=0;x<len_;++x){
//		ObjList<GridCell>* row=grid_[x];
		for (y=0;y<width_-1;++y){
			GridCell& g1=grid_[x][y];
			GridCell& g2=grid_[x][y+1];
			g1.SetNeighbor(&g2,1);
			g2.SetNeighbor(&g1,3);
			}//end for y
		}//end for x
	for (y=0;y<width_;++y){
		for (x=0;x<len_-1;++x){
			GridCell& g1=grid_[x][y];
			GridCell& g2=grid_[x+1][y];
			g1.SetNeighbor(&g2,2);
			g2.SetNeighbor(&g1,0);
			}//end for x
		}//end for y
	}//end LandScape::SetGridNeighbors
		
	
void LandScape::SetHabitatTypes(void){/*all default habitat type = 0*/
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.habitat(0);}}
	}/*end LandScape::SetHabitatTypes(void)*/


void LandScape::SetHabitatTypes(ObjList<aList<int> >& habitatGrid){
	if (grid_.len() != habitatGrid.len()){
		cout<<"Habitat grid must be the same size and"<<
			" shape as the population grid."<<endl;
		cout<<"This has a habitat grid of "<<habitatGrid.len()<<
			" rows and a pop grid of "<< grid_.len()<<" rows."<<endl;
		return;}
	long x,y;
	for (x=0;x<len_;++x){
		ObjList<GridCell>& row=grid_[x];
		aList<int>& habitatRow=habitatGrid[x];
		if (habitatRow.len() != row.len()){
			cout<<"Habitat grid must be the same size and "<<
				"shape as the population grid."<<endl;
			cout<<"This has a habitat grid of "<<habitatRow.len()<<
				" cols and a pop grid of "<< row.len()<<" cols."<<endl;
			return;}
		for (y=0;y<width_;++y){
			GridCell& g= row[y];
			g.habitat( habitatRow[y]);}
		}/*end for x*/
	}/*end LandScape::SetHabitatTypes(ObjList<aList<int>>&)*/


void LandScape::SetHabitatTypes(aString& dataFileName){
	fstream dataFile;	
	aString endCharList(" \t,;:\"(){}[]!?");
	char *dataLine=NEW char[1001];dataLine[0]='\0';
//	aString beast, popLine;
	char* filename=dataFileName.raw();
	dataFile.open(filename);
	ObjList<aList<int> > habitatGrid;
	while (! dataFile.eof()){
		dataFile.getline(dataLine, 1000);
		aString oneLineOfData(dataLine);
		if(oneLineOfData.len()>0){
			aList<int>* habitatRow = NEW aList<int>();
			habitatGrid.Append(habitatRow);
			while(oneLineOfData.len()>0){
				aString* habStr = oneLineOfData.ChopOffFirstWord(endCharList);
				if (habStr->len()>0){
					int habNum = habStr->toInt();
					habitatRow->Append(habNum);
					}
				delete habStr; habStr=0;
				}//end while len(0>0
			}//end if dataLine.len
		}//end while !eof
	dataFile.close();
	delete [] dataLine; dataLine = 0;
	SetHabitatTypes(habitatGrid);
	habitatGrid.Clear();
	}/*end LandScape::SetHabitatTypes(ObjList<List<int>>&)*/


void LandScape::SetHexGridNeighbors(void){
	//6-edged, hexagonal grid cells; use SetGridNeighbors for 1-D linear structures
	//no loop or torus structure
	// directions:  0:northeast; 1:east; 2:southeast; 3:southwest; 4:west; 5:northwest
	// where no neighbor exists, a NULL pointer sits
	GridCell* dummy=NULL;//make fully NULL list first
	long x,y,d;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			for(d=0;d<6;++d){g.SetNeighbor(dummy);}
			}
		}
	//replace NULLs with appropriate cells
	//e.g., (1,0) joins to (0,1), (1,1), (2,1), (2,0), NULL, & (0,0)
	//note directional structure by even and odd rows:
	//		(0,0)		(0,1)		(0,2)
	//
	//			  (1,0)		  (1,1)		  (1,2)
	//
	//		(2,0)		(2,1)		(2,2)
	// etc.
	//does all except across edges
	for (x=0;x<len_;++x){
		//first the within-row connections
//		ObjList<GridCell>* row=grid_[x];
		for (y=0;y<width_-1;++y){
			GridCell& g1=grid_[x][y];
			GridCell& g2=grid_[x][y+1];
//			GridCell* g1= row->Nth(y);
//			GridCell* g2= row->Nth(y+1);
			g1.SetNeighbor(&g2,1);
			g2.SetNeighbor(&g1,4);
			}//end for y
		}//end for x
	//then the connections to the next row
	for (y=0;y<width_;++y){
		for (x=0;x<len_-1;++x){
//			ObjList<GridCell>* row1=grid_[x];
//			ObjList<GridCell>* row2=grid_[x+1];
//			GridCell* g1= row1->Nth(y);
//			GridCell* g2= row2->Nth(y);
			GridCell& g1=grid_[x][y];
			GridCell& g2=grid_[x+1][y];
			if (even(x)){
					g1.SetNeighbor(&g2,2);
					g2.SetNeighbor(&g1,5);}
				else{
					g1.SetNeighbor(&g2,3);
					g2.SetNeighbor(&g1,0);}
			}//end for x
		}//end for y
	}//end LandScape::SetGridNeighbors
		
	
void LandScape::SetLoopHabitat(void){
	//assumes 4-edged, square grid cells
	//ends link in only the longer of 2 directions
	// directions:  0:north; 1:east; 2:south; 3:west
	SetGridNeighbors();
	long x,y;
	if (len_<=width_){
			for (x=0;x<len_;++x){
//				ObjList<GridCell>* row=grid_[x];
//				GridCell* g1= row->Nth(0);
//				GridCell* g2= row->Nth(width_-1);
				GridCell& g1=grid_[x][0];
				GridCell& g2=grid_[x][width_-1];
				g1.SetNeighbor(&g2,3);
				g2.SetNeighbor(&g1,1);
				}}//end for x
		else{
//			ObjList<GridCell>* row1=grid_[0];
//			ObjList<GridCell>* row2=grid_[len_-1];
			for (y=0;y<width_;++y){
//				GridCell* g1= row1->Nth(y);
//				GridCell* g2= row2->Nth(y);
				GridCell& g1=grid_[0][y];
				GridCell& g2=grid_[len_-1][y];
				g1.SetNeighbor(&g2,0);
				g2.SetNeighbor(&g1,2);
				}//end for y
			}//end if..else			
	}//end LandScape::SetLoopHabitat


void LandScape::SetLoopHabitatLongway(void){
	//assumes 4-edged, square grid cells
	//ends link in only the shorter of 2 directions
	// directions:  0:north; 1:east; 2:south; 3:west
	SetGridNeighbors();
	long x,y;
	if (width_<=len_){
			for (x=0;x<len_;++x){
//				ObjList<GridCell>* row=grid_[x];
//				GridCell* g1= row->Nth(0);
//				GridCell* g2= row->Nth(width_-1);
				GridCell& g1=grid_[x][0];
				GridCell& g2=grid_[x][width_-1];
				g1.SetNeighbor(&g2,3);
				g2.SetNeighbor(&g1,1);
				}}//end for x
		else{
//			ObjList<GridCell>* row1=grid_[0];
//			ObjList<GridCell>* row2=grid_[len_-1];
			for (y=0;y<width_;++y){
//				GridCell* g1= row1->Nth(y);
//				GridCell* g2= row2->Nth(y);
				GridCell& g1=grid_[0][y];
				GridCell& g2=grid_[len_-1][y];
				g1.SetNeighbor(&g2,0);
				g2.SetNeighbor(&g1,2);
				}//end for y
			}//end if..else			
	}//end LandScape::SetLoopHabitat



void LandScape::SetHexTorusHabitat(void){
	//edges link in all directions
	//e.g., (1,0) joins to (0,1), (1,1), (2,1), (2,0), far edge at (1,2), & (0,0)
	//note directional structure by even and odd rows:
	//		(0,0)		(0,1)		(0,2)
	//
	//			  (1,0)		  (1,1)		  (1,2)
	//
	//		(2,0)		(2,1)		(2,2)
	//
	//			  (3,0)		  (3,1)		  (3,2)
	// etc.
	// directions:  0:northeast; 1:east; 2:southeast; 3:southwest; 4:west; 5:northwest
	SetHexGridNeighbors();
	//set the E-W connections
	long x,y;
	for (x=0;x<len_;++x){
//		ObjList<GridCell>* row1=grid_[x];
//		GridCell* g1= row1->Nth(0);
//		GridCell* g2= row1->Nth(width_-1);
		GridCell& g1=grid_[x][0];
		GridCell& g2=grid_[x][width_-1];
		g1.SetNeighbor(&g2,4);
		g2.SetNeighbor(&g1,1);
		if (x-1>=0){//connect to previous row
//			ObjList<GridCell>* row2=grid_[x-1];
//			GridCell* g3= row2->Nth(0);
//			GridCell* g4= row2->Nth(width_-1);
			GridCell& g3=grid_[x-1][0];
			GridCell& g4=grid_[x-1][width_-1];
			if (even(x)){
					g1.SetNeighbor(&g3,0);
					g3.SetNeighbor(&g1,3);
					g2.SetNeighbor(&g4,0);
					g4.SetNeighbor(&g2,3);}
				else{
					g1.SetNeighbor(&g3,5);
					g3.SetNeighbor(&g1,2);
					g2.SetNeighbor(&g4,5);
					g4.SetNeighbor(&g2,2);}
			}//end if x-1
		}//end for x
	//set the N-S connections along the top & bottom edges
//	ObjList<GridCell>* row1=grid_[0];
//	ObjList<GridCell>* row2=grid_[len_-1];//(len_-1) must be an odd number
	for (y=0;y<width_;++y){//connect SW-to-NE
//		GridCell* g1= row1->Nth(y);
//		GridCell* g2= row2->Nth(y);
		GridCell& g1=grid_[0][y];
		GridCell& g2=grid_[len_-1][y];
		g1.SetNeighbor(&g2,0);
		g2.SetNeighbor(&g1,3);
		if (y-1>=0){//connect to previous column
//			GridCell* g3= row2->Nth(y-1);
			GridCell& g3=grid_[len_-1][y-1];
			g1.SetNeighbor(&g3,5);
			g3.SetNeighbor(&g1,2);
			}//end if y-1
		}//end for y
	//set links across corners
//	row1=grid_[0];
//	row2=grid_[len_-1];//(len_-1) must be an odd number
//	GridCell* g1= row1->Nth(0);
//	GridCell* g2= row2->Nth(width_-1);
	GridCell& g1=grid_[0][0];
	GridCell& g2=grid_[len_-1][width_-1];
	g1.SetNeighbor(&g2,5);
	g2.SetNeighbor(&g1,2);
//	g1= row1->Nth(width_-1);
//	g2= row2->Nth(0);
	GridCell& g11=grid_[0][width_-1];
	GridCell& g22=grid_[len_-1][0];
	g11.SetNeighbor(&g22,0);
	g22.SetNeighbor(&g11,3);
	}//end LandScape::SetHexTorusHabitat



void LandScape::SetTorusHabitat(void){
	//assumes 4-edged, square grid cells
	// directions:  0:north; 1:east; 2:south; 3:west
	//edges link in all directions
	SetGridNeighbors();
	long x,y;
	for (x=0;x<len_;++x){
//		ObjList<GridCell>* row=grid_[x];
//		GridCell* g1= row->Nth(0);
//		GridCell* g2= row->Nth(width_-1);
		GridCell& g1=grid_[x][0];
		GridCell& g2=grid_[x][width_-1];
		g1.SetNeighbor(&g2,3);
		g2.SetNeighbor(&g1,1);
		}//end for x
//	ObjList<GridCell>* row1=grid_[0];
//	ObjList<GridCell>* row2=grid_[len_-1];
	for (y=0;y<width_;++y){
//		GridCell* g1= row1->Nth(y);
//		GridCell* g2= row2->Nth(y);
		GridCell& g1=grid_[0][y];
		GridCell& g2=grid_[len_-1][y];
		g1.SetNeighbor(&g2,0);
		g2.SetNeighbor(&g1,2);
		}//end for y
	}//end LandScape::SetTorusHabitat
	
	


void LandScape::SomeAdultsDie (int species, Ran3List& ran){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.SomeAdultsDie(species,fSurvivalSchedule_[species],
							 mSurvivalSchedule_[species],ran);}}
	}//end LandScape::SomeAdultsDie


void LandScape::SomeImmaturesDie (int species, Ran3List& ran){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.SomeImmaturesDie(species,fSurvivalSchedule_[species],
							 mSurvivalSchedule_[species],ran);}}
	}//end LandScape::SomeImmaturesDie


void LandScape::SomeGametesDie (int species, Ran3List& ran){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.SomeGametesDie(species,fSurvivalSchedule_[species],
							 mSurvivalSchedule_[species],ran);}}
	}//end LandScape::SomeGAmetesDie


void LandScape::ViabilitySelection(Ran3& ran){
	int s;
	for(s=0;s<species_;++s){
		ViabilitySelection(s, ran);}
	}//end LandScape::ViabilitySelection(int, Ran3&)


void LandScape::ViabilitySelection(int species, Ran3& ran){
	long x,y;
	for (x=0;x<len_;++x){
		for (y=0;y<width_;++y){
			GridCell& g=grid_[x][y];
			g.ViabilitySelection(species, ran);}}
	}//end LandScape::ViabilitySelection(int, Ran3&)



#endif
