/******************************************
*
*	LandScapeInitializer00.cpp
*
*   This header holds all the variables that may be used to
*	initialize a LandScape-based simulation.  Derived classes
*	may be created to ad to this list, to initialize classes newly derived
*	from LandScape simulation bases.  Use sequential numbers to
*	represent derived classes.
*
******************************************/


#ifndef LandScapeInitializer00cpp
#define LandScapeInitializer00cpp

#include "LandScapeInitializer00.h"
#include "Element.cpp"
#include "aList.cpp"


LandScapeInitializer00:: ~LandScapeInitializer00(void){
	initSize_.Clear();
	maxage_.Clear();
	maxCapacity_.Clear();
	fSurvivalSchedule_.Clear();
	mSurvivalSchedule_.Clear();
	fFecunditySchedule_.Clear();
	femaleMovementRate_.Clear();
	maleMovementRate_.Clear();
	migrationMatrix_.Clear();
	intrinSexRatio_.Clear();
	initGenes_.Clear();
	initGenotypeFreqs_.Clear();
	delete indivOfCorrectType; indivOfCorrectType=NULL;
	delete popOfCorrectType; popOfCorrectType=NULL;
	}//end LandScapeInitializer00::~LandScapeInitializer00




void LandScapeInitializer00::AppendGenes(Genome& genes, double freq, int species){
	//all habitats the same
	if(initGenes_.len()==0){//construct it
		for(int s=0;s<species_;++s){
			ObjList<ObjList<Genome> >* spGList = new ObjList<ObjList<Genome> >();
			initGenes_.Append(spGList);
			ObjList<aList<double> >* spFList = new ObjList<aList<double> >();
			initGenotypeFreqs_.Append(spFList);
			}}
	ObjList<ObjList<Genome> >& spGList = initGenes_[species];
	ObjList<aList<double> >& spFList = initGenotypeFreqs_[species];
	if (spGList.len()==0){//construct it
		for(int h=0;h<habitats_;++h){
			ObjList<Genome>* hGList = new ObjList<Genome>();
			spGList.Append(hGList);
			aList<double>* hFList = new aList<double>();
			spFList.Append(hFList);
			}}
	for(int h=0;h<habitats_;++h){
		AppendGenes(genes, freq, species, h);}
	}//end LandScapeInitializer00::AppendGenes(Genome&, double&, int)


void LandScapeInitializer00::AppendGenes(Genome& genes, double freq, int species, int habitat){
	if(initGenes_.len()==0){//construct it
		for(int s=0;s<species_;++s){
			ObjList<ObjList<Genome> >* spGList = new ObjList<ObjList<Genome> >();
			initGenes_.Append(spGList);
			ObjList<aList<double> >* spFList = new ObjList<aList<double> >();
			initGenotypeFreqs_.Append(spFList);
			}}
	ObjList<ObjList<Genome> >& spGList = initGenes_[species];
	ObjList<aList<double> >& spFList = initGenotypeFreqs_[species];
	if (spGList.len()==0){//construct it
		for(int h=0;h<habitats_;++h){
			ObjList<Genome>* hGList = new ObjList<Genome>();
			spGList.Append(hGList);
			aList<double>* hFList = new aList<double>();
			spFList.Append(hFList);
			}}
	ObjList<Genome>& hGList = spGList[habitat];
	Genome* copyGenes = (Genome*) (genes.Duplicate());
	hGList.Append(copyGenes);
	aList<double>& hFList =spFList[habitat];
	hFList.Append(freq);
	}//end LandScapeInitializer00::AppendGenes(Genome&, double&, int, int)


void LandScapeInitializer00::AssortativeMatingProb(double prob){
	//all species the same
	assortMatingProb_.Clear();
	for(int s=0;s<species();++s){
		assortMatingProb_.Append(prob);}
	}//end LandScapeInitializer00::AssortativeMatingProb(double)


void LandScapeInitializer00::AssortativeMatingProb(double prob, int species){
	if(assortMatingProb_.len()==0){
		for(int s=0;s<species_;++s){
			assortMatingProb_.Append(0.0);}}
	assortMatingProb_[species]=prob;
	}//end LandScapeInitializer00::AssortativeMatingProb(double, int)




void LandScapeInitializer00::femaleFecundity(double b){//all species & ages the same
	/*set species_ and maxage_ first*/
	fFecunditySchedule_.Clear();
	for(int s=0;s<species_;++s){
		aList<double>* spList = new aList<double>();
		fFecunditySchedule_.Append(spList);
		femaleFecundity(b, s);}
	}//end LandScapeInitializer00::femaleFecundity(double)

			
void LandScapeInitializer00::femaleFecundity(double b, int species){//all ages the same
	/*set species_ and maxage_ first*/
	if(fFecunditySchedule_.len()==0){//construct a list
		for(int s=0;s<species_;++s){
			aList<double>* spList = new aList<double>();
			fFecunditySchedule_.Append(spList);}}
	aList<double>& spList = fFecunditySchedule_[species];
	spList.Clear();//create a new list
	for(int a=0;a<=maxage_[species];++a){//includes ageclass 0
		spList.Append(0.0);}
	for(int a=1;a<=maxage_[species];++a){//skips ageclass 0
		femaleFecundity(b,species,a);}//beause neonates don't reproduce
	}//end LandScapeInitializer00::femaleFecundity(double,int)

		
void LandScapeInitializer00::femaleFecundity(double b, int species, int ageClass){
	/*set species_ and maxage_ first*/
	if(fFecunditySchedule_.len()==0){//construct a list
		for(int s=0;s<species_;++s){
			aList<double>* spList = new aList<double>();
			fFecunditySchedule_.Append(spList);}}
	aList<double>& spList = fFecunditySchedule_[species];
	if(spList.len()==0){//construct a list of zeros
		for (int a=0;a<=maxage_[species];++a){//<= implies neonates are an implicit age class
			spList.Append(0.0);}}
	//the actual procedure
	spList.Replace(b,ageClass);
	}//end LandScapeInitializer00::femaleFecundity(double,int,int)


void LandScapeInitializer00::femaleMovementRate(double m){
	//all species, habitats & ages the same
	/*set species_ and maxage_ first*/
	femaleMovementRate_.Clear();
	for(int s=0;s<species_;++s){
		ObjList<aList<double> >* spList = new ObjList<aList<double> >();
		femaleMovementRate_.Append(spList);
		femaleMovementRate(m, s);}
	}//end LandScapeInitializer00::femaleMovementRate(double)

			
void LandScapeInitializer00::femaleMovementRate(double m, int species){
	//all habitats & ages the same
	/*set species_, habitats_ and maxage_ first*/
	if(femaleMovementRate_.len()==0){//construct a list
		for(int s=0;s<species_;++s){
			ObjList<aList<double> >* spList = new ObjList<aList<double> >();
			femaleMovementRate_.Append(spList);}}
	ObjList<aList<double> >& spList = femaleMovementRate_[species];
	spList.Clear();
	for(int h=0;h<habitats_;++h){
		aList<double>* hList = new aList<double>();
		spList.Append(hList);}
	for(int a=0;a<=maxage_[species];++a){
		femaleMovementRate(m,species,a);}
	}//end LandScapeInitializer00::femaleMovementRate(double,int)

		
void LandScapeInitializer00::femaleMovementRate(double m, int species, int ageClass){
	/*set species_, habitats_ and maxage_ first*/
	if(femaleMovementRate_.len()==0){//construct a list
		for(int s=0;s<species_;++s){
			ObjList<aList<double> >* spList = new ObjList<aList<double> >();
			femaleMovementRate_.Append(spList);}}
	ObjList<aList<double> >& spList = femaleMovementRate_[species];
	if(spList.len()==0){//construct a list
		for(int h=0;h<habitats_;++h){
			aList<double>* hList = new aList<double>();
			spList.Append(hList);}}
	for(int h=0;h<habitats_;++h){
		aList<double>& hList = spList[h];
		femaleMovementRate(m,species,h,ageClass);}
	}//end LandScapeInitializer00::femaleMovementRate(double,int,int)


void LandScapeInitializer00::femaleMovementRate(double m, int species, int habitat, int ageClass){
	/*set species_, habitats_ and maxage_ first*/
	if(femaleMovementRate_.len()==0){//construct a list
		for(int s=0;s<species_;++s){
			ObjList<aList<double> >* spList = new ObjList<aList<double> >();
			femaleMovementRate_.Append(spList);}}
	ObjList<aList<double> >& spList = femaleMovementRate_[species];
	if(spList.len()==0){//construct a list
		for(int h=0;h<habitats_;++h){
			aList<double>* hList = new aList<double>();
			spList.Append(hList);}}
	aList<double>& hList = spList[habitat];
	if(hList.len() ==0){
		for(int a=0;a<=maxage_[species];++a){
			hList.Append(0.0);}}//implies neonates cannot move
	//the actual work requested
	hList.Replace(m,ageClass);
	}//end LandScapeInitializer00::femaleMovementRate(double,int,int,int)


void LandScapeInitializer00::femaleSurvival(double m){//all species & ages the same
	/*set species_ and maxage_ first*/
	fSurvivalSchedule_.Clear();
	for(int s=0;s<species_;++s){
		aList<double>* spList = new aList<double>();
		fSurvivalSchedule_.Append(spList);
		femaleSurvival(m, s);}
	}//end LandScapeInitializer00::femaleSurvival(double)

			
void LandScapeInitializer00::femaleSurvival(double m, int species){//all ages the same
	/*set species_ and maxage_ first*/
	aList<double>& spList = fSurvivalSchedule_[species];
	for(int a=0;a<maxage_[species];++a){
		femaleSurvival(m,species,a);}
	}//end LandScapeInitializer00::femaleSurvival(double,int)

		
void LandScapeInitializer00::femaleSurvival(double m, int species, int ageClass){
	/*set species_ and maxage_ first*/
	aList<double>& spList = fSurvivalSchedule_[species];
	if(spList.len() < maxage_[species]){
			spList.Append(m);}
		else{spList.Replace(m,ageClass);}
	}//end LandScapeInitializer00::femaleSurvival(double,int,int)


void LandScapeInitializer00::initSize(long n){//all species the same
	initSize_.Clear();
	for(int s=0;s<species_;++s){
		initSize_.Append(n);}
	}//end LandScapeInitializer00::initSize(long)


void LandScapeInitializer00::initSize(long n, int species){
	if(initSize_.len() < species+1){
			initSize_.Append(n);}
		else{initSize_.Replace(n,species);}
	}//end LandScapeInitializer00::initSize(long,int)


void LandScapeInitializer00::maleMovementRate(double m){
	//all species, habitats & ages the same
	/*set species_ and maxage_ first*/
	maleMovementRate_.Clear();
	for(int s=0;s<species_;++s){
		ObjList<aList<double> >* spList = new ObjList<aList<double> >();
		maleMovementRate_.Append(spList);
		maleMovementRate(m, s);}
	}//end LandScapeInitializer00::maleMovementRate(double)

			
void LandScapeInitializer00::maleMovementRate(double m, int species){
	//all habitats & ages the same
	/*set species_, habitats_ and maxage_ first*/
	if(maleMovementRate_.len()==0){//construct a list
		for(int s=0;s<species_;++s){
			ObjList<aList<double> >* spList = new ObjList<aList<double> >();
			maleMovementRate_.Append(spList);}}
	ObjList<aList<double> >& spList = maleMovementRate_[species];
	spList.Clear();
	for(int h=0;h<habitats_;++h){
		aList<double>* hList = new aList<double>();
		spList.Append(hList);}
	for(int a=0;a<=maxage_[species];++a){
		maleMovementRate(m,species,a);}
	}//end LandScapeInitializer00::maleMovementRate(double,int)

		
void LandScapeInitializer00::maleMovementRate(double m, int species, int ageClass){
	/*set species_, habitats_ and maxage_ first*/
	if(maleMovementRate_.len()==0){//construct a list
		for(int s=0;s<species_;++s){
			ObjList<aList<double> >* spList = new ObjList<aList<double> >();
			maleMovementRate_.Append(spList);}}
	ObjList<aList<double> >& spList = maleMovementRate_[species];
	if(spList.len()==0){//construct a list
		for(int h=0;h<habitats_;++h){
			aList<double>* hList = new aList<double>();
			spList.Append(hList);}}
	for(int h=0;h<habitats_;++h){
		aList<double>& hList = spList[h];
		maleMovementRate(m,species,h,ageClass);}
	}//end LandScapeInitializer00::maleMovementRate(double,int,int)


void LandScapeInitializer00::maleMovementRate(double m, int species, int habitat, int ageClass){
	/*set species_, habitats_ and maxage_ first*/
	if(maleMovementRate_.len()==0){//construct a list
		for(int s=0;s<species_;++s){
			ObjList<aList<double> >* spList = new ObjList<aList<double> >();
			maleMovementRate_.Append(spList);}}
	ObjList<aList<double> >& spList = maleMovementRate_[species];
	if(spList.len()==0){//construct a list
		for(int h=0;h<habitats_;++h){
			aList<double>* hList = new aList<double>();
			spList.Append(hList);}}
	aList<double>& hList = spList[habitat];
	if(hList.len() ==0){
		for(int a=0;a<=maxage_[species];++a){
			hList.Append(0.0);}}//implies neonates cannot move
	//the actual work requested
	hList.Replace(m,ageClass);
	}//end LandScapeInitializer00::maleMovementRate(double,int,int,int)


void LandScapeInitializer00::maleSurvival(double m){//all species & ages the same
	/*set species_ and maxage_ first*/
	mSurvivalSchedule_.Clear();
	for(int s=0;s<species_;++s){
		aList<double>* spList = new aList<double>();
		mSurvivalSchedule_.Append(spList);
		maleSurvival(m, s);}
	}//end LandScapeInitializer00::maleSurvival(double)

			
void LandScapeInitializer00::maleSurvival(double m, int species){//all ages the same
	/*set species_ and maxage_ first*/
	aList<double>& spList = mSurvivalSchedule_[species];
	for(int a=0;a<maxage_[species];++a){
		maleSurvival(m,species,a);}
	}//end LandScapeInitializer00::maleSurvival(double,int)

		
void LandScapeInitializer00::maleSurvival(double m, int species, int ageClass){
	/*set species_ and maxage_ first*/
	aList<double>& spList = mSurvivalSchedule_[species];
	if(spList.len() < maxage_[species]){
			spList.Append(m);}
		else{spList.Replace(m,ageClass);}
	}//end LandScapeInitializer00::maleSurvival(double,int,int)


void LandScapeInitializer00::maxage(int a){//all species the same
	maxage_.Clear();
	for(int s=0;s<species_;++s){
		maxage_.Append(a);}
	}//LandScapeInitializer00::
	
	
void LandScapeInitializer00::maxage(int a, int species){
	if(maxage_.len() < species+1){
			maxage_.Append(a);}
		else{maxage_.Replace(a,species);}
	}//end LandScapeInitializer00::maxage(int,int)


void LandScapeInitializer00::maxCapacity(long n){//all species the same
	maxCapacity_.Clear();
	for(int s=0;s<species_;++s){
		maxCapacity_.Append(n);}
	}//LandScapeInitializer00::maxCapacity(long)


void LandScapeInitializer00::maxCapacity(long n, int species){
	if(maxCapacity_.len() < species+1){
			maxCapacity_.Append(n);}
		else{maxCapacity_.Replace(n,species);}
	}//end LandScapeInitializer00::maxCapacity(long,int)


void LandScapeInitializer00::migrationProb(double m){
	//all species, habitats & ages the same
	/*set species_, habitats_ and maxage_ first*/
	migrationMatrix_.Clear();
	for(int s=0;s<species_;++s){
		ObjList<ObjList<aList<double> > >* spList = new ObjList<ObjList<aList<double> > >();
		migrationMatrix_.Append(spList);
		migrationProb(m, s);}
	}//end LandScapeInitializer00::migrationProb(double)

			
void LandScapeInitializer00::migrationProb(double m, int species){
	//all habitats & ages the same
	/*set species_, habitats_ and maxage_ first*/
	ObjList<ObjList<aList<double> > >& spList = migrationMatrix_[species];
	spList.Clear();
	for(int a=0;a<maxage_[species];++a){
		ObjList<aList<double> >* ageList = new ObjList<aList<double> >();
		spList.Append(ageList);
		migrationProb(m,species,a);}
	}//end LandScapeInitializer00::migrationProb(double,int)

		
void LandScapeInitializer00::migrationProb(double m, int species, int ageClass){
	/*set species_, habitats_ and maxage_ first*/
	ObjList<ObjList<aList<double> > >& spList = migrationMatrix_[species];
	ObjList<aList<double> >& ageList = spList[ageClass];
	ageList.Clear();
	for(int h1=0;h1<habitats_;++h1){
		aList<double>* h1List = new aList<double>();
		ageList.Append(h1List);
		for(int h2=0;h2<habitats_;++h2){
			migrationProb(m,species,ageClass,h1,h2);}
		}//end for h1
	}//end LandScapeInitializer00::migrationProb(double,int,int)


void LandScapeInitializer00::migrationProb(double m, int species, int fromHabitat, int toHabitat){
	/*set species_, habitats_ and maxage_ first*/
	if(migrationMatrix_.len()==0){//construct one
		for(int s=0;s<species_;++s){
			ObjList<ObjList<aList<double> > >* spList = new ObjList<ObjList<aList<double> > >();
			migrationMatrix_.Append(spList);}}
	ObjList<ObjList<aList<double> > >& spList = migrationMatrix_[species];
	if(spList.len()==0){//construct one
		for(int a=0;a<maxage_[species];++a){
			ObjList<aList<double> >* ageList = new ObjList<aList<double> >();
			spList.Append(ageList);}}
	for(int a=0;a<maxage_[species];++a){//replace for all ages
		ObjList<aList<double> >& ageList = spList[a];
		if(ageList.len()==0){//construct one
			for(int h1=0;h1<habitats_;++h1){
				aList<double>* h1List = new aList<double>();
				ageList.Append(h1List);}}
		aList<double>& h1List = ageList[fromHabitat];
		if(h1List.len()==0){//construct one & fill with zeros
			for(int h2=0;h2<habitats_;++h2){
			h1List.Append(0.0);}}
		h1List.Replace(m,toHabitat);
		}
	}//end LandScapeInitializer00::migrationProb(double,int,int,int)


void LandScapeInitializer00::migrationProb(double m, int species, int ageClass, int fromHabitat, int toHabitat){
	/*set species_, habitats_ and maxage_ first*/
	ObjList<ObjList<aList<double> > >& spList = migrationMatrix_[species];
	ObjList<aList<double> >& ageList = spList[ageClass];
	aList<double>& h1List = ageList[fromHabitat];
	if(h1List.len() < habitats_){
			h1List.Append(m);}
		else{h1List.Replace(m,toHabitat);}
	}//end LandScapeInitializer00::migrationProb(double,int,int,int,int)



void LandScapeInitializer00::movementRate(double m){
	femaleMovementRate(m);
	maleMovementRate(m);
	}//end LandScapeInitializer00::movementRate(double)


void LandScapeInitializer00::movementRate(double m, int species){
	femaleMovementRate(m, species);
	maleMovementRate(m, species);
	}//end LandScapeInitializer00::movementRate(double,int)


void LandScapeInitializer00::movementRate(double m, int species, int ageClass){
	femaleMovementRate(m, species, ageClass);
	maleMovementRate(m, species, ageClass);
	}//end LandScapeInitializer00::movementRate(double,int,int)


void LandScapeInitializer00::movementRate(double m, int species, int habitat, int ageClass){
	femaleMovementRate(m, species, habitat, ageClass);
	maleMovementRate(m, species, habitat, ageClass);
	}//end LandScapeInitializer00::movementRate(double,int,int,int)


void LandScapeInitializer00::setHabitat(int habitatValue, int row, int column){
	if (habitatGrid_.len()==0){//construct grid
		for(int r=0;r<length();++r){
			aList<int>* h1List = new aList<int>();
			habitatGrid_.Append(h1List);
			for(int col=0;col<width();++col){
				h1List->Append(0);}}}
	aList<int>& h1List = habitatGrid_[row];
	h1List.Replace(habitatValue,column);
}//end LandScapeInitializer00::setHabitat


void LandScapeInitializer00::SetIndividual(Individual* indiv){
	delete indivOfCorrectType;
	indivOfCorrectType = indiv;
	}//end LandScapeInitializer00::SetIndividual


void LandScapeInitializer00::SetGridCell(GridCell* g){
	delete gridcellOfCorrectType;
	gridcellOfCorrectType = g;
	}//end LandScapeInitializer00::SetGridCell



void LandScapeInitializer00::SetPopulation(Population* pop){
	delete popOfCorrectType;
	popOfCorrectType = pop;
	}//end LandScapeInitializer00::SetPopulation


void LandScapeInitializer00::sexRatio(double x){//all species the same
	intrinSexRatio_.Clear();
	for(int s=0;s<species_;++s){
		intrinSexRatio_.Append(x);}
	}//LandScapeInitializer00::sexRatio(double)


void LandScapeInitializer00::sexRatio(double x, int species){
	if(intrinSexRatio_.len() < species+1){
			intrinSexRatio_.Append(x);}
		else{intrinSexRatio_.Replace(x,species);}
	}//end LandScapeInitializer00::sexRatio(double,int)


void LandScapeInitializer00::survival(double m){
	femaleSurvival(m);
	maleSurvival(m);
	}//end LandScapeInitializer00::survival(double)


void LandScapeInitializer00::survival(double m, int species){
	femaleSurvival(m, species);
	maleSurvival(m, species);
	}//end LandScapeInitializer00::survival(double,int)


void LandScapeInitializer00::survival(double m, int species, int ageClass){
	femaleSurvival(m, species, ageClass);
	maleSurvival(m, species, ageClass);
	}//end LandScapeInitializer00::survival(double,int,int)





#endif



