/******************************************
*
*	LandScapeInitializer01.cpp
*
*   This header holds all the variables that may be used to
*	initialize a LandScape-based simulation.  Derived classes
*	may be created to ad to this list, to initialize classes newly derived
*	from LandScape simulation bases.  Use sequential numbers to
*	represent derived classes.
*
******************************************/


#ifndef LandScapeInitializer01cpp
#define LandScapeInitializer01cpp = 1


#include "LandScapeInitializer01.h"
#include "Element.cpp"
#include "aList.cpp"
#include "GeneContainers.cpp"


LandScapeInitializer01:: ~LandScapeInitializer01(void){
	initSize_.Clear();
	maxage_.Clear();
	maxCapacity_.Clear();
	fSurvivalSchedule_.Clear();
	mSurvivalSchedule_.Clear();
	fFecunditySchedule_.Clear();
	femaleMovementRate_.Clear();
	maleMovementRate_.Clear();
	migrationMatrix_.Clear();
	intrinSexRatio_.Clear();
	initPhenes_.Clear();
	initPhenotypeFreqs_.Clear();
	delete indivOfCorrectType; indivOfCorrectType=NULL;
	delete popOfCorrectType; popOfCorrectType=NULL;
	delete gridcellOfCorrectType; gridcellOfCorrectType=NULL;
	}//end LandScapeInitializer01::~LandScapeInitializer01




//void LandScapeInitializer01::AppendGenotypes(ObjList<PopulationGeneContainer<int> >* genesByPop, int species){
void LandScapeInitializer01::AppendGenotypes(ObjList<PopulationGeneContainer<long double> >* genesByPop, int species){
	if(initMultiInheritanceGenotypes_.len()-1<species){
		int sp;
		for(sp=0;sp<species-1;++sp){
//			initMultiInheritanceGenotypes_.Append(NEW ObjList<PopulationGeneContainer<int> >());}
			initMultiInheritanceGenotypes_.Append(NEW ObjList<PopulationGeneContainer<long double> >());}
		initMultiInheritanceGenotypes_.Append(genesByPop);
		return;
		}//if
	initMultiInheritanceGenotypes_.Replace(genesByPop,species);
	}//LandScapeInitializer01::AppendGenotypes(ObjList<PopulationGeneContainer<int>*, int)
		


void LandScapeInitializer01::AppendPhenes(Phenotype& phenes, long double freq, int species){
	//all habitats the same
	int s,h;
	if(initPhenes_.len()==0){//construct it
		for(s=0;s<species_;++s){
			ObjList<ObjList<Phenotype> >* spPhList = NEW ObjList<ObjList<Phenotype> >();
			initPhenes_.Append(spPhList);
			ObjList<aList<long double> >* spFList = NEW ObjList<aList<long double> >();
			initPhenotypeFreqs_.Append(spFList);
			}}
	ObjList<ObjList<Phenotype> >& spPhList = initPhenes_[species];
	ObjList<aList<long double> >& spFList = initPhenotypeFreqs_[species];
	if (spPhList.len()==0){//construct it
		for(h=0;h<habitats_;++h){
			ObjList<Phenotype>* hPhList = NEW ObjList<Phenotype>();
			spPhList.Append(hPhList);
			aList<long double>* hFList = NEW aList<long double>();
			spFList.Append(hFList);
			}}
	for(h=0;h<habitats_;++h){
		AppendPhenes(phenes, freq, species, h);}
	}//end LandScapeInitializer01::AppendPhenes(Phenotype&, double&, int)


void LandScapeInitializer01::AppendPhenes(Phenotype& phenes, long double freq, int species, int habitat){
	int s,h;
	if(initPhenes_.len()==0){//construct it
		for(s=0;s<species_;++s){
			ObjList<ObjList<Phenotype> >* spPhList = NEW ObjList<ObjList<Phenotype> >();
			initPhenes_.Append(spPhList);
			ObjList<aList<long double> >* spFList = NEW ObjList<aList<long double> >();
			initPhenotypeFreqs_.Append(spFList);
			}}
	ObjList<ObjList<Phenotype> >& spPhList = initPhenes_[species];
	ObjList<aList<long double> >& spFList = initPhenotypeFreqs_[species];
	if (spPhList.len()==0){//construct it
		for(h=0;h<habitats_;++h){
			ObjList<Phenotype>* hPhList = NEW ObjList<Phenotype>();
			spPhList.Append(hPhList);
			aList<long double>* hFList = NEW aList<long double>();
			spFList.Append(hFList);
			}}
	ObjList<Phenotype>& hPhList = spPhList[habitat];
	Phenotype* copyPhenes = phenes.Duplicate();
	hPhList.Append(copyPhenes);
	aList<long double>& hFList =spFList[habitat];
	hFList.Append(freq);
	}//end LandScapeInitializer00::AppendPhenes(Phenotype&, double&, int, int)


void LandScapeInitializer01::AssortativeMatingProb(long double prob){
	//all species the same
	assortMatingProb_.Clear();
	for(int s=0;s<species();++s){
		assortMatingProb_.Append(prob);}
	}//end LandScapeInitializer01::AssortativeMatingProb(double)


void LandScapeInitializer01::AssortativeMatingProb(long double prob, int species){
	if(assortMatingProb_.len()==0){
		int s;
		for(s=0;s<species_;++s){
			assortMatingProb_.Append(zero);}}
	assortMatingProb_[species]=prob;
	}//end LandScapeInitializer01::AssortativeMatingProb(double, int)




void LandScapeInitializer01::femaleFecundity(long double b){//all species & ages the same
	/*set species_ and maxage_ first*/
	fFecunditySchedule_.Clear();
	int s;
	for(s=0;s<species_;++s){
		aList<long double>* spList = NEW aList<long double>();
		fFecunditySchedule_.Append(spList);
		femaleFecundity(b, s);}
	}//end LandScapeInitializer01::femaleFecundity(double)

			
void LandScapeInitializer01::femaleFecundity(long double b, int species){//all ages the same
	/*set species_ and maxage_ first*/
	int s,a;
	if(fFecunditySchedule_.len()==0){//construct a list
		for(s=0;s<species_;++s){
			aList<long double>* spList = NEW aList<long double>();
			fFecunditySchedule_.Append(spList);}}
	aList<long double>& spList = fFecunditySchedule_[species];
	spList.Clear();//create a new list
	for(a=0;a<=maxage_[species];++a){//includes ageclass 0
		spList.Append(zero);}
	for(a=1;a<=maxage_[species];++a){//skips ageclass 0
		femaleFecundity(b,species,a);}//beause neonates don't reproduce
	}//end LandScapeInitializer01::femaleFecundity(double,int)

		
void LandScapeInitializer01::femaleFecundity(long double b, int species, int ageClass){
	//set species_ and maxage_ first
	int s,a;
	if(fFecunditySchedule_.len()==0){//construct a list
		for(s=0;s<species_;++s){
			aList<long double>* spList = NEW aList<long double>();
			fFecunditySchedule_.Append(spList);}}
	aList<long double>& spList = fFecunditySchedule_[species];
	if(spList.len()==0){//construct a list of zeros
		for (a=0;a<=maxage_[species];++a){//<= implies neonates are an implicit age class
			spList.Append(zero);}}
	//the actual procedure
	spList.Replace(b,ageClass);
	}//end LandScapeInitializer01::femaleFecundity(double,int,int)


void LandScapeInitializer01::femaleMovementRate(long double m){
	//all species, habitats & ages the same
	/*set species_ and maxage_ first*/
	femaleMovementRate_.Clear();
	int s;
	for(s=0;s<species_;++s){
		femaleMovementRate_.Append(NEW ObjList<aList<long  double> >());
		femaleMovementRate(m, s);}
	}//end LandScapeInitializer01::femaleMovementRate(double)

			
void LandScapeInitializer01::femaleMovementRate(long double m, int species){
	//all habitats & ages the same
	/*set species_, habitats_ and maxage_ first*/
	int s,h,a;
	if(femaleMovementRate_.len()==0){//construct a list
		for(s=0;s<species_;++s){
			femaleMovementRate_.Append(NEW ObjList<aList<long double> >());}}
	ObjList<aList<long double> >& spList = femaleMovementRate_[species];
	spList.Clear();
	for(h=0;h<habitats_;++h){
		spList.Append(NEW aList<long double>());}
	for(a=0;a<=maxage_[species];++a){
		femaleMovementRate(m,species,a);}
	}//end LandScapeInitializer01::femaleMovementRate(double,int)

		
void LandScapeInitializer01::femaleMovementRate(long double m, int species, int ageClass){
	/*set species_, habitats_ and maxage_ first*/
	int s,h;
	if(femaleMovementRate_.len()==0){//construct a list
		for(s=0;s<species_;++s){
			femaleMovementRate_.Append(NEW ObjList<aList<long double> >());}}
	ObjList<aList<long double> >& spList = femaleMovementRate_[species];
	if(spList.len()==0){//construct a list
		for(h=0;h<habitats_;++h){
			spList.Append( NEW aList<long double>());}}
	for(h=0;h<habitats_;++h){
//		aList<long double>& hList = spList[h];
		femaleMovementRate(m,species,h,ageClass);}
	}//end LandScapeInitializer01::femaleMovementRate(double,int,int)


void LandScapeInitializer01::femaleMovementRate(long double m, int species, int habitat, int ageClass){
	/*set species_, habitats_ and maxage_ first*/
	int s,h,a;
	if(femaleMovementRate_.len()==0){//construct a list
		for(s=0;s<species_;++s){
			femaleMovementRate_.Append(NEW ObjList<aList<long double> >());}}
	ObjList<aList<long double> >& spList = femaleMovementRate_[species];
	if(spList.len()==0){//construct a list
		for(h=0;h<habitats_;++h){
			spList.Append(NEW aList<long double>());}}
	aList<long double>& hList = spList[habitat];
	if(hList.len() ==0){
		for(a=0;a<=maxage_[species];++a){
			hList.Append(zero);}}//implies neonates cannot move
	//the actual work requested
	hList.Replace(m,ageClass);
	}//end LandScapeInitializer01::femaleMovementRate(double,int,int,int)


void LandScapeInitializer01::femaleSurvival(long double m){//all species & ages the same
	/*set species_ and maxage_ first*/
	fSurvivalSchedule_.Clear();
	int s;
	for(s=0;s<species_;++s){
		aList<long double>* spList = NEW aList<long double>();
		fSurvivalSchedule_.Append(spList);
		femaleSurvival(m, s);}
	}//end LandScapeInitializer01::femaleSurvival(double)

			
void LandScapeInitializer01::femaleSurvival(long double m, int species){//all ages the same
	/*set species_ and maxage_ first*/
	while(fSurvivalSchedule_.len()<species_){
		fSurvivalSchedule_.Append(NEW aList<long double>());}
	aList<long double>& spList = fSurvivalSchedule_[species];
//	for(int a=0;a<maxage_[species];++a){
	for(int a=0;a<=maxage_[species];++a){
		femaleSurvival(m,species,a);}
	}//end LandScapeInitializer01::femaleSurvival(double,int)

		
void LandScapeInitializer01::femaleSurvival(long double m, int species, int ageClass){
	/*set species_ and maxage_ first*/
	while(fSurvivalSchedule_.len()<species_){
		fSurvivalSchedule_.Append(NEW aList<long double>());}
	aList<long double>& spList = fSurvivalSchedule_[species];
//	if(spList.len() < maxage_[species]){
	if(spList.len() <= maxage_[species]){
			spList.Append(m);}
		else{spList.Replace(m,ageClass);}
	}//end LandScapeInitializer01::femaleSurvival(double,int,int)


void LandScapeInitializer01::initSize(long n){//all species the same
	initSize_.Clear();
	int s;
	for(s=0;s<species_;++s){
		initSize_.Append(n);}
	}//end LandScapeInitializer01::initSize(long)


void LandScapeInitializer01::initSize(long n, int species){
	if(initSize_.len() < species+1){
			initSize_.Append(n);}
		else{initSize_.Replace(n,species);}
	}//end LandScapeInitializer01::initSize(long,int)


void LandScapeInitializer01::maleMovementRate(long double m){
	//all species, habitats & ages the same
	/*set species_ and maxage_ first*/
	maleMovementRate_.Clear();
	int s;
	for(s=0;s<species_;++s){
		ObjList<aList<long double> >* spList = NEW ObjList<aList<long double> >();
		maleMovementRate_.Append(spList);
		maleMovementRate(m, s);}
	}//end LandScapeInitializer01::maleMovementRate(double)

			
void LandScapeInitializer01::maleMovementRate(long double m, int species){
	//all habitats & ages the same
	/*set species_, habitats_ and maxage_ first*/
	int s,h,a;
	if(maleMovementRate_.len()==0){//construct a list
		for(s=0;s<species_;++s){
			maleMovementRate_.Append(NEW ObjList<aList<long double> >());}}
	ObjList<aList<long double> >& spList = maleMovementRate_[species];
	spList.Clear();
	for(h=0;h<habitats_;++h){
		aList<long double>* hList = NEW aList<long double>();
		spList.Append(hList);}
	for(a=0;a<=maxage_[species];++a){
		maleMovementRate(m,species,a);}
	}//end LandScapeInitializer01::maleMovementRate(double,int)

		
void LandScapeInitializer01::maleMovementRate(long double m, int species, int ageClass){
	/*set species_, habitats_ and maxage_ first*/
	int s,h;
	if(maleMovementRate_.len()==0){//construct a list
		for(s=0;s<species_;++s){
			maleMovementRate_.Append(NEW ObjList<aList<long double> >());}}
	ObjList<aList<long double> >& spList = maleMovementRate_[species];
	if(spList.len()==0){//construct a list
		for(h=0;h<habitats_;++h){
			spList.Append(NEW aList<long double>());}}
	for(h=0;h<habitats_;++h){
//		aList<long double>& hList = spList[h];
		maleMovementRate(m,species,h,ageClass);}
	}//end LandScapeInitializer01::maleMovementRate(double,int,int)


void LandScapeInitializer01::maleMovementRate(long double m, int species, int habitat, int ageClass){
	/*set species_, habitats_ and maxage_ first*/
	int s,h,a;
	if(maleMovementRate_.len()==0){//construct a list
		for(s=0;s<species_;++s){
			maleMovementRate_.Append(NEW ObjList<aList<long double> >());}}
	ObjList<aList<long double> >& spList = maleMovementRate_[species];
	if(spList.len()==0){//construct a list
		for(h=0;h<habitats_;++h){
			spList.Append(NEW aList<long double>());}}
	aList<long double>& hList = spList[habitat];
	if(hList.len() ==0){
		for(a=0;a<=maxage_[species];++a){
			hList.Append(zero);}}//implies neonates cannot move
	//the actual work requested
	hList.Replace(m,ageClass);
	}//end LandScapeInitializer01::maleMovementRate(double,int,int,int)


void LandScapeInitializer01::maleSurvival(long double m){//all species & ages the same
	/*set species_ and maxage_ first*/
	mSurvivalSchedule_.Clear();
	int s;
	for(s=0;s<species_;++s){
		aList<long double>* spList = NEW aList<long double>();
		mSurvivalSchedule_.Append(spList);
		maleSurvival(m, s);}
	}//end LandScapeInitializer01::maleSurvival(double)

			
void LandScapeInitializer01::maleSurvival(long double m, int species){//all ages the same
	/*set species_ and maxage_ first*/
	int a;
	while(mSurvivalSchedule_.len()<species_){
		mSurvivalSchedule_.Append(NEW aList<long double>());}
	aList<long double>& spList = mSurvivalSchedule_[species];
//	for(a=0;a<maxage_[species];++a){
	for(a=0;a<=maxage_[species];++a){
		maleSurvival(m,species,a);}
	}//end LandScapeInitializer01::maleSurvival(double,int)

		
void LandScapeInitializer01::maleSurvival(long double m, int species, int ageClass){
	/*set species_ and maxage_ first*/
	while(mSurvivalSchedule_.len()<species_){
		mSurvivalSchedule_.Append(NEW aList<long double>());}
	aList<long double>& spList = mSurvivalSchedule_[species];
//	if(spList.len() < maxage_[species]){
	if(spList.len() <= maxage_[species]){
			spList.Append(m);}
		else{spList.Replace(m,ageClass);}
	}//end LandScapeInitializer01::maleSurvival(double,int,int)


void LandScapeInitializer01::maxage(int a){//all species the same
	maxage_.Clear();
	int s;
	for(s=0;s<species_;++s){
		maxage_.Append(a);}
	}//LandScapeInitializer01::maxage(int)
	
	
void LandScapeInitializer01::maxage(int a, int species){
	if(maxage_.len() < species+1){
			maxage_.Append(a);}
		else{maxage_.Replace(a,species);}
	}//end LandScapeInitializer01::maxage(int,int)


void LandScapeInitializer01::maxCapacity(long n){//all species the same
	maxCapacity_.Clear();
	int s;
	for(s=0;s<species_;++s){
		maxCapacity_.Append(n);}
	}//LandScapeInitializer01::maxCapacity(long)


void LandScapeInitializer01::maxCapacity(long n, int species){
	if(maxCapacity_.len() < species+1){
			maxCapacity_.Append(n);}
		else{maxCapacity_.Replace(n,species);}
	}//end LandScapeInitializer01::maxCapacity(long,int)


void LandScapeInitializer01::migrationProb(long double m){
	//all species, habitats & ages the same
	/*set species_, habitats_ and maxage_ first*/
	migrationMatrix_.Clear();
	int s;
	for(s=0;s<species_;++s){
		ObjList<ObjList<aList<long double> > >* spList = NEW ObjList<ObjList<aList<long double> > >();
		migrationMatrix_.Append(spList);
		migrationProb(m, s);}
	}//end LandScapeInitializer01::migrationProb(double)

			
void LandScapeInitializer01::migrationProb(long double m, int species){
	//all habitats & ages the same
	/*set species_, habitats_ and maxage_ first*/
	int a,s;
	if(migrationMatrix_.len()==0){//construct one
		for(s=0;s<species_;++s){
			migrationMatrix_.Append(NEW ObjList<ObjList<aList<long double> > >());}}
	ObjList<ObjList<aList<long double> > >& spList = migrationMatrix_[species];
	spList.Clear();
//	for(a=0;a<maxage_[species];++a){
	for(a=0;a<=maxage_[species];++a){
		ObjList<aList<long double> >* ageList = NEW ObjList<aList<long double> >();
		spList.Append(ageList);
		migrationProb(m,species,a);}
	}//end LandScapeInitializer01::migrationProb(double,int)

		
void LandScapeInitializer01::migrationProb(long double m, int species, int ageClass){
	/*set species_, habitats_ and maxage_ first*/
	int h1,h2,s,a;
	if(migrationMatrix_.len()==0){//construct one
		for(s=0;s<species_;++s){
			migrationMatrix_.Append(NEW ObjList<ObjList<aList<long double> > >());}}
	ObjList<ObjList<aList<long double> > >& spList = migrationMatrix_[species];
	if(spList.len()==0){//construct one
		for(a=0;a<=maxage_[species];++a){
			spList.Append(NEW ObjList<aList<long double> >());}}
	ObjList<aList<long double> >& ageList = spList[ageClass];
	ageList.Clear();
	for(h1=0;h1<habitats_;++h1){
		ageList.Append(NEW aList<long double>());
		for(h2=0;h2<habitats_;++h2){
			migrationProb(m,species,ageClass,h1,h2);}
		}//end for h1
	}//end LandScapeInitializer01::migrationProb(double,int,int)


void LandScapeInitializer01::migrationProb(long double m, int species, int fromHabitat, int toHabitat){
	/*set species_, habitats_ and maxage_ first*/
	int s,a,h1,h2;
	if(migrationMatrix_.len()==0){//construct one
		for(s=0;s<species_;++s){
			migrationMatrix_.Append(NEW ObjList<ObjList<aList<long double> > >());}}
	ObjList<ObjList<aList<long double> > >& spList = migrationMatrix_[species];
	if(spList.len()==0){//construct one
		for(a=0;a<=maxage_[species];++a){
			spList.Append(NEW ObjList<aList<long double> >());}}
//	for(a=0;a<maxage_[species];++a){//replace for all ages
	for(a=0;a<=maxage_[species];++a){//replace for all ages
		ObjList<aList<long double> >& ageList = spList[a];
		if(ageList.len()==0){//construct one
			for(h1=0;h1<habitats_;++h1){
				ageList.Append(NEW aList<long double>());}}
		aList<long double>& h1List = ageList[fromHabitat];
		if(h1List.len()==0){//construct one & fill with zeros
			for(h2=0;h2<habitats_;++h2){
			h1List.Append(0.0);}}
		h1List.Replace(m,toHabitat);
		}
	}//end LandScapeInitializer01::migrationProb(double,int,int,int)


void LandScapeInitializer01::migrationProb(long double m, int species, int ageClass, int fromHabitat, int toHabitat){
	/*set species_, habitats_ and maxage_ first*/
	ObjList<ObjList<aList<long double> > >& spList = migrationMatrix_[species];
	ObjList<aList<long double> >& ageList = spList[ageClass];
	aList<long double>& h1List = ageList[fromHabitat];
	if(h1List.len() < habitats_){
			h1List.Append(m);}
		else{h1List.Replace(m,toHabitat);}
	}//end LandScapeInitializer01::migrationProb(double,int,int,int,int)



void LandScapeInitializer01::movementRate(long double m){
	femaleMovementRate(m);
	maleMovementRate(m);
	}//end LandScapeInitializer01::movementRate(double)


void LandScapeInitializer01::movementRate(long double m, int species){
	femaleMovementRate(m, species);
	maleMovementRate(m, species);
	}//end LandScapeInitializer01::movementRate(double,int)


void LandScapeInitializer01::movementRate(long double m, int species, int ageClass){
	femaleMovementRate(m, species, ageClass);
	maleMovementRate(m, species, ageClass);
	}//end LandScapeInitializer01::movementRate(double,int,int)


void LandScapeInitializer01::movementRate(long double m, int species, int habitat, int ageClass){
	femaleMovementRate(m, species, habitat, ageClass);
	maleMovementRate(m, species, habitat, ageClass);
	}//end LandScapeInitializer01::movementRate(double,int,int,int)


void LandScapeInitializer01::setHabitat(int habitatValue, int row, int column){
	if (habitatGrid_.len()==0){//construct grid
		int r,col;
		for(r=0;r<length();++r){
			aList<int>* h1List = NEW aList<int>();
			habitatGrid_.Append(h1List);
			for(col=0;col<width();++col){
				h1List->Append(0);}}}
	aList<int>& h1List = habitatGrid_[row];
	h1List.Replace(habitatValue,column);
}//end LandScapeInitializer01::setHabitat


void LandScapeInitializer01::SetIndividual(Individual* indiv){
	delete indivOfCorrectType;
	indivOfCorrectType = indiv;
	}//end LandScapeInitializer01::SetIndividual


void LandScapeInitializer01::SetGridCell(GridCell* g){
	delete gridcellOfCorrectType;
	gridcellOfCorrectType = g;
	}//end LandScapeInitializer01::SetGridCell



void LandScapeInitializer01::SetPopulation(Population* pop){
	delete popOfCorrectType;
	popOfCorrectType = pop;
	}//end LandScapeInitializer01::SetPopulation


void LandScapeInitializer01::sexRatio(long double x){//all species the same
	intrinSexRatio_.Clear();
	int s;
	for(s=0;s<species_;++s){
		intrinSexRatio_.Append(x);}
	}//LandScapeInitializer01::sexRatio(double)


void LandScapeInitializer01::sexRatio(long double x, int species){
	if(intrinSexRatio_.len() < species+1){
			intrinSexRatio_.Append(x);}
		else{intrinSexRatio_.Replace(x,species);}
	}//end LandScapeInitializer01::sexRatio(double,int)


void LandScapeInitializer01::survival(long double m){
	femaleSurvival(m);
	maleSurvival(m);
	}//end LandScapeInitializer01::survival(double)


void LandScapeInitializer01::survival(long double m, int species){
	femaleSurvival(m, species);
	maleSurvival(m, species);
	}//end LandScapeInitializer01::survival(double,int)


void LandScapeInitializer01::survival(long double m, int species, int ageClass){
	femaleSurvival(m, species, ageClass);
	maleSurvival(m, species, ageClass);
	}//end LandScapeInitializer01::survival(double,int,int)



#endif





