/******************************************
*
*	SexualGeneticLandScape.cpp
*
*   This header defines the derived LandScape class
*	that forms the basis for individuals with genetic information.
*	It seeds the landscape with base-class genetic individuals,
*	and presumes constant habitat quality.  Derived
*	classes add features and/or take data.
******************************************/

#ifndef SexualGeneticLandScapecp
#define SexualGeneticLandScapecp = 1


#include "SexualGeneticLandScape.h"
#include "RandomList.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"



/*	CONSTRUCTORS AND DESTRUCTORS*/
/*
SexualGeneticLandScape::SexualGeneticLandScape(LandScapeInitializer00& init):
	//general purpose initializer
	GeneticLandScape(init){}
*/


SexualGeneticLandScape::SexualGeneticLandScape(LandScapeInitializer01& init):
	/*general purpose initializer*/
	GeneticLandScape(init){}

SexualGeneticLandScape::SexualGeneticLandScape(int len, /*ONE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 long double fSurvivalRate, /*CONSTANT THROUGH LIFE*/
  					 long double fFecundityRate, /*CONSTANT THROUGH LIFE*/
   					 long double femaleMovementRate,
//   					 selectionSourceSet sSource,
 					 ObjList<Phenotype >& initGenes,
  					 aList<long double>& initGenotypeFreqs
					 ):
		GeneticLandScape(len,w,migrationModel,initSize, maxCapacity, maxage, 
					 fSurvivalRate, fFecundityRate, femaleMovementRate,
					 /*sSource,*/initGenes,initGenotypeFreqs){
	}/*end constructor SexualGeneticLandScape::SexualGeneticLandScape*/


SexualGeneticLandScape::SexualGeneticLandScape(int len, /*ONE SPECIES*/
					 int w, 	/*default habitat=0, override ILandScape otherwise*/
					 migrationModelSet migrationModel,
					 long initSize, 
					 long maxCapacity, 
					 int maxage, 
					 aList<long double>* fSurvivalSchedule, /*VARIABLE SURVIVAL*/
  					 aList<long double>* fFecunditySchedule, /*VARIABLE FECUNDITY*/
 					 ObjList<aList<long double> >* femaleMovementRate,
// 					 selectionSourceSet sSource,
  					 ObjList<Phenotype >& initGenes,
  					 aList<long double>& initGenotypeFreqs
					 ):
		GeneticLandScape(len,w,migrationModel,initSize,maxCapacity,maxage, 
					 fSurvivalSchedule, fFecunditySchedule, femaleMovementRate,
					 /*sSource,*/initGenes,initGenotypeFreqs){
	}/*end constructor SexualGeneticLandScape::SexualGeneticLandScape*/


SexualGeneticLandScape::SexualGeneticLandScape(int len, /*ONE OR MORE SPECIES, 1 HABITAT*/
					 int w, 	
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<long double> >* fSurvivalSchedule,
  					 ObjList<aList<long double> >* fFecunditySchedule,
 					 ObjList<ObjList<aList<long double> > >* femaleMovementRate,
// 					 selectionSourceSet sSource,
   					 ObjList<ObjList<Phenotype > >& initGenes,
  					 ObjList<aList<long double> >& initGenotypeFreqs
					 ):
		GeneticLandScape(len,w,migrationModel,species,initSize, maxCapacity,maxage, 
					 fSurvivalSchedule,fFecunditySchedule,femaleMovementRate,
					 /*sSource,*/initGenes,initGenotypeFreqs){
	}/*end constructor SexualGeneticLandScape::SexualGeneticLandScape*/


SexualGeneticLandScape::SexualGeneticLandScape(int len, /*ONE OR MORE SPECIES, 1 HABITAT*/
					 int w, 
					 migrationModelSet migrationModel,
					 int species, 
					 aList<long>* initSize,
					 aList<long>* maxCapacity, 
					 aList<int>* maxage, 
					 ObjList<aList<long double> >* fSurvivalSchedule,
  					 ObjList<aList<long double> >* fFecunditySchedule,
 					 ObjList<ObjList<aList<long double> > >* femaleMovementRate,
// 					 selectionSourceSet sSource,
   					 ObjList<ObjList<ObjList<Phenotype > > >& initGenes,
					 ObjList<ObjList<aList<long double> > >& initGenotypeFreqs
					 ):
		GeneticLandScape(len,w,migrationModel,species,initSize, maxCapacity,maxage, 
					 fSurvivalSchedule,fFecunditySchedule,femaleMovementRate,
					 /*sSource,*/initGenes,initGenotypeFreqs){
	}/*end constructor SexualGeneticLandScape::SexualGeneticLandScape*/
					 
					 

SexualGeneticLandScape::~SexualGeneticLandScape(void){
	}/*end destructor GeneticLandScape<T>::~GeneticLandScape		*/			 




/* OVERRIDDEN AND NEW FUNCTIONS*/


/* handled by base class now
void SexualGeneticLandScape::DoOneCycle (long gen, Ran3List& ran, ObjList<aList<long double> >& dispersalInfo,
							 ObjList<aList<long double> >& reproductionInfo){
	TimeIncrement();
	int s;
	for(s=0;s<species();++s){
		ViabilitySelection(s, ran);}
	for(s=0;s<species();++s){
		AdultDispersal(s, ran,dispersalInfo[s]);
		}
	for(s=0;s<species();++s){
		MateOnGrid(s, ran);}
	for(s=0;s<species();++s){
		Reproduce(s, ran, reproductionInfo[s]);}
	for(s=0;s<species();++s){
		AgeOneStep(s);}
	for(s=0;s<species();++s){
		SomeAdultsDie(s, ran);}
	for(s=0;s<species();++s){
		ReplaceTheDead(s, ran);}

	//DERIVED should take data from dead individuals before calling BuryTheDead
	for(s=0;s<species();++s){
		BuryTheDead(s);}
	
	}//end SexualGeneticLandScape::DoOneCycle
	

void SexualGeneticLandScape::DoOneCycle (int species, long gen, 
							Ran3List& ran, ObjList<aList<long double> >& dispersalInfo,
							ObjList<aList<long double> >& reproductionInfo){
	//PROBABLY BEST TO OVERRIDE THIS IN DERIVED CLASSES
	TimeIncrement();
	ViabilitySelection(species, ran);
//	AdultDispersal(species, ran, m);
	AdultDispersal(species, ran, dispersalInfo[species]);
//	MateOnGrid(species, ran);
//	Reproduce(species, ran, mutationRate, sigma);//mating done here for ideal hermaphrodite reproduction
	Reproduce(species, ran, reproductionInfo[species]);
	AgeOneStep(species);
	SomeAdultsDie(species, ran);
	ReplaceTheDead(species, ran);

	//DERIVED should take data from dead individuals before calling BuryTheDead
	BuryTheDead(species);
	
	}//end SexualGeneticLandScape::DoOneCycle
*/



void SexualGeneticLandScape::ILandScape (Ran3List& ran, int seeded){
	/*only called just after constructors. Exists to avoid code duplication*/
	int row,col;
	CoordinateType coords;
	for(row=0;row<len_;++row){
		ObjList<GridCell>*  gridList = NEW ObjList<GridCell>();
		grid_.Append(gridList);
		for(col=0;col<width_;++col){
			coords.set(row,col);
			GeneticGridCell* g = NEW GeneticGridCell(this, coords,0);/*habitat=0*/
			gridList->Append(g);
			for (int s=0;s<species_;++s){
				SexualGeneticPopulation* pop= NEW SexualGeneticPopulation(
								totalPopsCreated_++,s,g,maxCapacity_[s]);
				g->Append(pop);
				}/*end for species*/
			}/*end for cols*/
		}/* end for rows	*/
	CreateMigrationMatrix();	
	cout<<len_<<" x "<<width_<<
			" landscape created"<<endl;
	if(seeded){
		for (int s=0;s<species_;++s){
			SeedLandScape(s, initSize_[s], ran);}
		}
	}/*end GeneticLandScape<T>::ILandScape*/


/*
void SexualGeneticLandScape::ILandScape (LandScapeInitializer00& init, Ran3List& ran){
	//explicit here for clarity only
	GeneticLandScape::ILandScape (init, ran);
	}//end SexualGeneticLandScape::ILandScape
*/

void SexualGeneticLandScape::ILandScape (LandScapeInitializer01& init, Ran3List& ran, int seeded){
	/*explicit here for clarity only*/
	GeneticLandScape::ILandScape (init, ran, seeded);
	}/*end SexualGeneticLandScape::ILandScape*/



void SexualGeneticLandScape::ILandScapeByHabitat (Ran3List& ran, ObjList<aList<int> >& habitatGrid,
					ObjList<ObjList<ObjList<Phenotype > > >& genesByFreqByHabitatBySpecies,
					ObjList<ObjList<aList<long double> > >& freqsByHabitatBySpecies){
	/*only called just after constructors. Exists to avoid code duplication*/
	CoordinateType coords;
	for(long row=0;row<len_;++row){
		ObjList<GridCell>*  gridList = NEW ObjList<GridCell>();
		grid_.Append(gridList);
		for(long col=0;col<width_;++col){
			coords.set(row,col);
			GeneticGridCell* g = NEW GeneticGridCell(this, coords,0);/*habitat=0*/
			gridList->Append(g);
			for (int s=0;s<species_;++s){
				SexualGeneticPopulation* pop= NEW SexualGeneticPopulation(
								totalPopsCreated_++,s,g,maxCapacity_[s]);
				g->Append(pop);
				}/*end for species*/
			}/*end for cols*/
		}/* end for rows	*/
	SetHabitatTypes(habitatGrid);
	CreateMigrationMatrix();	
	cout<<len_<<" x "<<width_<<
			" landscape created"<<endl;
	for (int s=0;s<species_;++s){
				SeedLandScapeByHabitat(s, initSize_[s], ran,
					genesByFreqByHabitatBySpecies,freqsByHabitatBySpecies);}
	}/*end SexualGeneticLandScape::ILandScapeByHabitat*/

/*
void SexualGeneticLandScape::ILandScapeByHabitat (Ran3List& ran, LandScapeInitializer00& init){
	//explicit here only for clarity
	GeneticLandScape<T>::ILandScapeByHabitat(ran,init);
	}//end SexualGeneticLandScape::ILandScapeByHabitat
*/


void SexualGeneticLandScape::ILandScapeByHabitat (Ran3List& ran, LandScapeInitializer01& init, int seeded){
	/*explicit here only for clarity*/
	GeneticLandScape::ILandScapeByHabitat(ran,init, seeded);
	}/*end SexualGeneticLandScape::ILandScapeByHabitat*/




void SexualGeneticLandScape::SeedLandScape (int species, long initSize, Ran3List& ran){
	/*puts sexual genetic individuals randomly into an empty landscape*/
	/*derive this for separate sexes etc*/
	genderSet sex = B;/*default idealHermaphrodite*/
	CoordinateType loc;
	/*make a weighted aList for genotypes that will be seeded*/
	StatList<long> geneWeights;
	ObjList<Phenotype >& geneList = SeededGeneList(species,0);
	aList<long double>& freqs = SeededGeneFreqList(species,0);
	for(long i=0;i<geneList.len();++i){
		long b= (long) (initSize * (freqs[i]));
		geneWeights.Append(b);
		}/*end for i*/
	while (geneWeights.sum()<initSize){/*in case frequencies don't divide exactly into initSize*/
			geneWeights.Increment(geneWeights.len()-1);/*increment the last one*/
			}/*end while*/
		
	/*make and initialize each individual and install it in a specified (presently random) grid location	*/
	for (long i=0;i<initSize;++i){
		loc.RandomLoc(0,len_-1,0,width_-1,ran);
		long x=ran.weighted(geneWeights);
		geneWeights.Decrement(x);/*samples without replacement*/
		Phenotype& newGenes=geneList[x];
		SexualGeneticIndividual* indiv = NEW SexualGeneticIndividual(newGenes,++totalIndivsCreated_);
		indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran);/*age=1*/
		InstallInGrid(species, indiv, loc);}
	}/*	end SexualGeneticLandScape::SeedLandScape */


/*
void SexualGeneticLandScape::SeedLandScape (LandScapeInitializer00& init, int species, long initSize, Ran3List& ran){
	//explict here for clarity only
	GeneticLandScape<T>::SeedLandScape (init, species, initSize, ran);
	}//	end SexualGeneticLandScape::SeedLandScape 
*/

void SexualGeneticLandScape::SeedLandScape (LandScapeInitializer01& init, int species, long initSize, Ran3List& ran){
	/*explict here for clarity only*/
	GeneticLandScape::SeedLandScape (init, species, initSize, ran);
	}/*	end SexualGeneticLandScape::SeedLandScape */



void SexualGeneticLandScape::SeedLandScapeByHabitat (int species, long initSize, Ran3List& ran, 
					ObjList<ObjList<ObjList<Phenotype > > >& genesByFreqByHabitatBySpecies,
					ObjList<ObjList<aList<long double> > >& freqsByHabitatBySpecies){
	//puts genetic individuals into an empty landscape
	//according to their genotype and habitat
	genderSet sex = B;//default idealHermaphrodite
	//make a weighted aList for genotypes that will be seeded
	ObjList<ObjList<Phenotype > >& genesByFreqByHabitat=genesByFreqByHabitatBySpecies[species];
	ObjList<aList<long double> >& freqsByHabitat=freqsByHabitatBySpecies[species];
	//make up a list of gridcells of each habitat
	ObjList<ObjList<GridCell> > gridcellsByHabitat;
	for(int hh=0;hh<freqsByHabitat.len();++hh){
		ObjList<GridCell>* cellsWithHabitat = NEW ObjList<GridCell>();
		gridcellsByHabitat.Append(cellsWithHabitat);}
	for (long x=0;x<len_;++x){
		for (long y=0;y<width_;++y){
			GridCell& g= grid_[x][y];	
			ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[g.habitat()];
			cellsWithHabitat.Append(&g);
			}}	

	for(long h=0;h<freqsByHabitat.len();++h){
		StatList<long> geneWeights;
		aList<long double>& genomeFreqs = freqsByHabitat[h];
		ObjList<Phenotype >& genesInHabitat = genesByFreqByHabitat[h];
		ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[h];
		long indivsInHabitat	= (long)( initSize *( ((long double) (cellsWithHabitat.len() )) / ((long double) ( len_ * width_))  ) );
		for(int gg=0;gg<genesInHabitat.len();++gg){			
			long n= (long) (indivsInHabitat * (genomeFreqs[gg]));
			geneWeights.Append(n);}
		while (geneWeights.sum()<indivsInHabitat){//in case frequencies don't divide exactly into initSize
			geneWeights.Increment(geneWeights.len()-1);//increment the last one
			}//end while

		//make and initialize each individual and install it in a specified (presently random) grid location
		//that has the proper habitat type
		for (long i=0;i<indivsInHabitat;++i){
			int cellNum = ran.longBetween(0,cellsWithHabitat.len()-1);
			GridCell& g = cellsWithHabitat[cellNum];
			CoordinateType loc = g.coords();
			long x=ran.weighted(geneWeights);
			geneWeights.Decrement(x);//samples without replacement
			Phenotype& newGenes = genesInHabitat[x];
			SexualGeneticIndividual* indiv = NEW SexualGeneticIndividual(newGenes,++totalIndivsCreated_);
			indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran);//age=1
			InstallInGrid(species, indiv, loc);
			}//end for i
		
		}//end for h
	
	//clean up
	for(int h=0;h<gridcellsByHabitat.len();++h){
		ObjList<GridCell>& cellsWithHabitat = gridcellsByHabitat[h];
		cellsWithHabitat.Detach();}
	gridcellsByHabitat.Clear();

	}//	end SexualGeneticLandScape::SeedLandScapeByHabitat 


/*
void SexualGeneticLandScape::SeedLandScapeByHabitat(LandScapeInitializer00& init, 
							int species, long initSize, Ran3List& ran){
	//explicit here only for clarity
	GeneticLandScape::SeedLandScapeByHabitat(init, species, initSize, ran);
	}//	end SexualGeneticLandScape::SeedLandScapeByHabitat 
*/

void SexualGeneticLandScape::SeedLandScapeByHabitat(LandScapeInitializer01& init, 
							int species, long initSize, Ran3List& ran){
	/*explicit here only for clarity*/
	GeneticLandScape::SeedLandScapeByHabitat(init, species, initSize, ran);
	}//	end SexualGeneticLandScape::SeedLandScapeByHabitat 



#endif

