/******************************************
*
*	SexualLandScape.cpp
*
*   This defines functions for the derived LandScape class
*	that forms the basis for individuals that must mate to reproduce.
*	It seeds the landscape with base-class hermaphroditic individuals,
*	and presumes constant habitat quality.  Derived
*	classes add features and/or take data.
******************************************/


#ifndef SexualLandScapecp
#define SexualLandScapecp = 1

#include "SexualLandScape.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "RandomList.cpp"


void SexualLandScape::ILandScape(Ran3List& ran,int seeded){
	//only called just after constructors. Exists to avoid code duplication
	int row,col;
	CoordinateType coords;
	for(row=0;row<len_;++row){
		ObjList<GridCell>*  gridList = NEW ObjList<GridCell>();
		grid_.Append(gridList);
		for(col=0;col<width_;++col){
			coords.set(row,col);
			GridCell* g = new GridCell(this, coords,0);//habitat=0
			gridList->Append(g);
			for (int s=0;s<species_;++s){
				SexualPopulation* pop= NEW SexualPopulation(
								totalPopsCreated_++,s,g,maxCapacity_[s]);
				g->Append(pop);
				}//end for species
			}//end for cols
		}// end for rows	
	CreateMigrationMatrix();
	cout<<len_<<" x "<<width_<<" landscape created"<<endl;
	if(seeded){
		for (int s=0;s<species_;++s){
			SeedLandScape(s, initSize_[s], ran);}}
	}//end SexualLandScape::ILandScape


void SexualLandScape::ILandScape(LandScapeInitializer01& init, Ran3List& ran, int seeded){
	//only called just after constructors. Exists to avoid code duplication
	LandScape::ILandScape(init, ran, seeded);//made explicit for clarity
	}//end SexualLandScape::ILandScape





void SexualLandScape::SeedLandScape (int species, long initSize, Ran3List& ran){
	//puts individuals randomly into an empty landscape
	//make and initialize each individual and install it in a specified (presently random) grid location
	CoordinateType loc;
	for (long i=0;i<initSize;++i){
		loc.RandomLoc(0,len_-1,0,width_-1,ran);
		genderSet sex = B;//default hermaphrodite
		SexualIndividual* indiv = NEW SexualIndividual(++totalIndivsCreated_);
		indiv->IIndividual(sex, adult, moveRate(species,habitat(loc),sex,1), ran);//age=1
		InstallInGrid(species, indiv, loc);}
	}//	end SexualLandScape::SeedLandScape 




void SexualLandScape::SeedLandScape (LandScapeInitializer01& init, int species, long initSize, Ran3List& ran){
	//puts individuals randomly into an empty landscape
	//inherit this for separate sexes etc
	LandScape::SeedLandScape (init, species, initSize, ran);//made explicit for clarity
	}//	end SexualLandScape::SeedLandScape 

#endif

