/*******************************************
*
*
*	MatrixList.cpp
*
*	A class that implements a list of matrices of long doubles
*
*
*
************************************************/

#ifndef MatrixListcpp
#define matrixListcpp = 1


#include "MatrixList.h"
#include "Element.cpp"
#include "aList.cpp"
#include "aVector.cpp"
#include "aMatrix.cpp"



aMatrix<long double>* MatrixList::ConcatCols(void){
	//returns new matrix concatenated by adding columns
	//rows must all match or null matrix returned
	MatrixList& x = *this;
	aMatrix<long double>* m=NULL;
	long r, c, colSum=x[0].cols(), cBasis=0, mrows=x[0].rows();
	for(int i=1;i<len();++i){
		colSum +=x[i].cols();
		if(x[0].rows() != x[i].rows()){
			cout<<"ERROR:  Can't do MatrixList::ConcatCols if row numbers don't match.  Null matrix returned."<<endl;
			return m;}}
	m = NEW aMatrix<long double>(zero, mrows, colSum);
	for(int i=0;i<len();++i){
		long xcols = x[i].cols();
		for(c=0;c<xcols;++c){
			for(r=0;c<mrows;++r){
				long double a = x[i][r][c];
				(*m)[r][c+cBasis] = a;}}
		cBasis += x[i].cols();}
	return m;
	}//end MatrixList::ConcatCols
		
		
		
aMatrix<long double>* MatrixList::ConcatRows(void){
	//returns new matrix concatenated by adding rows
	//cols must all match or null matrix returned
	MatrixList& x = *this;
	aMatrix<long double>* m=NULL;
	long r, c, rowSum=x[0].rows(), rBasis=0, mcols=x[0].cols();
	for(int i=1;i<len();++i){
		rowSum +=x[i].rows();
		if(x[0].cols() != x[i].cols()){
			cout<<"ERROR:  Can't do MatrixList::ConcatRows if column numbers don't match.  Null matrix returned."<<endl;
			return m;}}
	m = NEW aMatrix<long double>(zero, rowSum, mcols);
	for(int i=0;i<len();++i){
		long xrows = x[i].rows();
		for(r=0;r<xrows;++r){
			for(c=0;c<mcols;++c){
				long double a =  x[i][r][c];
				(*m)[ r+rBasis][c] =a;}}
		rBasis += xrows;}
	return m;
	}//end MatrixList::ConcatRows
		
		
		

	
aMatrix<long double>*  MatrixList::DirectSum (void){
	MatrixList& x = *this;//notational convenience
	long colSum=0, rowSum=0, r,c;
	for(int i=0;i<len();++i){
		colSum += x[i].cols();
		rowSum += x[i].rows();}

	aMatrix<long double>* m = NEW aMatrix<long double>(0.0, rowSum, colSum);
	aMatrix<long double>& mat = *m;//notational convenience
	long rBasis=0, cBasis=0;
	for(int i=0;i<len();++i){
		aMatrix<long double>& component = x[i];
		for (r=0; r<component.rows(); ++r){
			for(c=0;  c<component.cols(); ++c){
				mat[r+rBasis][c+cBasis]= component[r][c];
			}}
		cBasis += component.cols();
		rBasis += component.rows();
		}
	return m;
	}//end MatrixList::DirectSum



aMatrix<long double>* MatrixList::Sum(void){
	//sums the matrices in the list
	if(len_==0) {return NEW aMatrix<long double>();}//returns empty matrix
	aMatrix<long double>& firstItem = First();
	for(int i=1;i<len_;++i){
		aMatrix<long double>& item0 = Nth(i);
		if (! firstItem.CanBeAdded(item0)){//incompatible list
			return NEW aMatrix<long double>();}}//returns empty matrix
	aMatrix<long double>* sum = NEW aMatrix<long double>(First());//copies first matrix
	aMatrix<long double>* newSum = NULL;
	for(int i=1; i<len_;++i){
		aMatrix<long double>& item1 = Nth(i);
		 newSum = sum->Add(item1);
		delete sum; sum=NULL;
		sum = newSum;  newSum = NULL;}
	return sum;
	}//end MatrixList::Sum
		
	
int operator== (MatrixList& mx1, MatrixList& mx2){
		ObjList<aMatrix<long double> >& mmx1 = mx1;
		ObjList<aMatrix<long double> >& mmx2 = mx2;
		return (mmx1 == mmx2);}

int operator!= (MatrixList& mx1,  MatrixList& mx2){
		return (! (mx1 == mx2) );}



#endif


