/*******************************************
*
*
*	aMatrix.cpp
*
*	A class that implements vectors, matrices, and does matrix algebra
*
*	Matrices can be converted from  ObjList<aList<T>>
*
*
************************************************/

#ifndef aMatrixcp
#define aMatrixcp = 1


#include "aMatrix.h"
//#include <fstream.h>
#include "Element.cpp"
#include "aList.cpp"
#include "aVector.cpp"






template<class T > aMatrix<T>::aMatrix(T x,   unsigned long r, unsigned long c)
		: rows_(r), cols_(c), nullity_(-1)
		{
	for(long rr=0;rr<rows_;++rr){
		aVector<T>* newVec = NEW aVector<T>(x, c);
		if(newVec==NULL){
			aString err("Failed 'new' in aMatrix<T>::aMatrix(T x, r,c)");BadOperation(err);}
		data.Append(newVec);
		newVec=NULL;}
	}//end constructor aMatrix<T>::aMatrix(T,unsigned long, unsigned long )



template<class T > aMatrix<T>::aMatrix(unsigned long r, unsigned long c)
		: rows_(r), cols_(c), nullity_(-1)
		{
	if(r<=0){rows_= cols_=0;}//no columns if no rows
		else if(c<=0){cols_ = 1;}//can't make a 0-length vector
	for(long rr=0;rr<rows_;++rr){
		aVector<T>* newVec = NEW aVector<T>((T)0.0, c);
		if(newVec==NULL){
			aString err("Failed 'new' in aMatrix<T>::aMatrix(r,c)");BadOperation(err);}
		data.Append(newVec);
		newVec=NULL;}
	}//end constructor aMatrix<T>::aMatrix(unsigned long, unsigned long)



template<class T > aMatrix<T>::aMatrix(aMatrix<T>& m)
		: rows_(m.rows_), cols_(m.cols_), nullity_(m.nullity_)
		{
	for(long rr=0;rr<rows_;++rr){
		aVector<T>& oldVec = m[rr];
		aVector<T>* newVec = NEW aVector<T>(oldVec);
		if(newVec==NULL){
			aString err("Failed 'new' in aMatrix<T>::aMatrix(aMatrix<T>&)");BadOperation(err);}
		data.Append(newVec);
		newVec=NULL;}
	}//end constructor aMatrix<T>::aMatrix(aMatrix<T>& m)



template<class T > aMatrix<T>::aMatrix(aMatrix<T>* m)
		: rows_(m->rows_), cols_(m->cols_), nullity_(m->nullity_)
		{
	for(long rr=0;rr<rows_;++rr){
		aVector<T>& oldVec = (*m)[rr];
		aVector<T>* newVec = NEW aVector<T>(oldVec);
		if(newVec==NULL){
			aString err("Failed 'new' in aMatrix<T>::aMatrix(aMatrix<T>*)");BadOperation(err);}
		data.Append(newVec);
		newVec=NULL;}
	}//end constructor aMatrix<T>::aMatrix(aMatrix<T>* m)



template<class T > aMatrix<T>::~aMatrix(void){
	data.Clear();
	}//end constructor aMatrix<T>::~aMatrix(void)



template<class T > aMatrix<T>* aMatrix<T>::Add (aMatrix<T>& m){
	//adds m to this and returns result in a new matrix
	//can return a null pointer if matrices not matched in size
	if(!CanBeAdded(m)){
		aString a="Error:  can't add matrices of different sizes!";
		BadOperation(a);
		return NULL;}
	aMatrix<T>* sumMatrix = NEW aMatrix<T>(this) ; 
	if(sumMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::Add(aMatrix<T>&)");BadOperation(err);}
	for (long r=0;r<rows_;++r){
		for (long c=0;c<cols_;++c){
			(*sumMatrix)[r][c] += m[r][c];}}
	return sumMatrix;
	}//end aMatrix<T>::Add (aMatrix<T>&)



template<class T > void aMatrix<T>::CalcNullity (void){
	//calculates the nullity of the matrix,
	//which is the number of columns (rows) derived from other columns (rows)
	aMatrix<long double> *Vmatrix = ToLongDouble();
	matrixHandle<long double> VmatrixH;
	VmatrixH.Bind(Vmatrix);Vmatrix=NULL;//deleted with handle at exit
	aVector<long double> w(cols());
	aMatrix<long double> *U = SVDecomposition(VmatrixH,w);
	delete U; U=NULL;
	
	long double wMax = 0.0;//zero the low w's
	for (long j=0;j<cols();++j){
		long double wj = w[j];
		if (wj > wMax){
			wMax = wj;}
		}//end for j
	long double wMin = wMax * (long double)1.0e-6;
	for (long j=0;j<cols();++j){
		long double wj = w[j];
		if (wj < wMin){
			w[j]=0.0;}
		}//end for j
	
	nullity_=0;
	for(long j=0;j<w.len();++j){
		if(w[j] == 0.0){nullity_++;}}
	
	}//end aMatrix<T>::CalcNullity



template<class T > void aMatrix<T>::cdiv(long double xr, long double  xi, 
										 long double yr, long double  yi,
										 long double cdivr, long double cdivi) {
	   // Complex scalar division.
	   //used in eigenvalue calculations
      long double r,d;
      if (ABS(yr) > ABS(yi)) {
	         r = yi/yr;
	         d = yr + r*yi;
	         cdivr = (xr + r*xi)/d;
	         cdivi = (xi - r*xr)/d;
	     	 } 
      else {
	         r = yr/yi;
	         d = yi + r*yr;
	         cdivr = (r*xr + xi)/d;
	         cdivi = (r*xi - xr)/d;
	      	}
   }//aMatrix<T>::cdiv()


template<class T > void aMatrix<T>::Chop (void){
	//round to zero values less than 10^-6 times abs(Max)
	aMatrix<T>& m = *this;
	T high = Max();
	T low = Min();
	T tops = max(abs(high), abs(low));
	double threshold = ((double)tops) * 10E-6;
	for(long r=0;r<rows_;++r){
		for (long c=0;c<cols_;++c){
			if((abs(m[r][c])<threshold) || (m[r][c] == T(-0.0))){
				m[r][c]=T(0.0);}}}
	}//end  aMatrix<T>::Chop




template<class T > aMatrix<T>*  aMatrix<T>::Concat (aMatrix<T>& m){
	//appends m to this by adding columns and returns result in a new matrix
	//rows_ must match or NULL pointer returned
	if(rows() != m.rows()){
		aString a="Error:  can't concatenate matrices of different row numbers!";
		BadOperation(a);
		return NULL;}
	aMatrix<T>* newMatrix = NEW aMatrix<T>(rows(), cols()+m.cols()); 
	if(newMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::Concat(aMatrix<T>&)");BadOperation(err);}
	for(long r=0;r<rows_;++r){
		for(long c=0;c<cols_;++c){
			(*newMatrix)[r][c]=data[r][c];}}
	for(long r=0;r<rows_;++r){
		for(long c=cols_;c<cols_+m.cols_;++c){
			(*newMatrix)[r][c]=m[r][c-cols_];}}
	return newMatrix;
	}//end aMatrix<T>::Concat (aMatrix<T>&)



template<class T > void aMatrix<T>::ConcatCol (aVector<T>& v){
	//appends column v to this 
	//rows_ must match len_ or NULL pointer returned
	if((rows()>0)&&(rows()!=v.len())){
		aString a="Error:  can't concatenate a matrix and vector of different row numbers!";
		BadOperation(a);
		return ;}
	if(rows_==0){
			rows_=v.len();//add this column
			cols_=1;
			for(long r=0;r<rows_;++r){
				aVector<T>* newRow = NEW aVector<T>(v[r]);
				if(newRow==NULL){
					aString err("Failed 'new' in aMatrix<T>::ConcatCol(aVector<T>&)");BadOperation(err);}
				data.Append(newRow);
				newRow=NULL;}
			}
		else{
			for(long r=0;r<rows_;++r){
				aVector<long double>& rowVec =data[r];
				rowVec.Append(v[r]);}
			++cols_;
			}
	}//end aMatrix<T>::ConcatCol (aVector<T>&)
	
	
	
template<class T > void aMatrix<T>::ConcatRow (aVector<T>& v){
	//appends row v to this 
	//cols_ must match len_ or NULL pointer returned
	if((cols()>0)&&(cols()!=v.len())){//an empty matrix can take any size vector
		aString a="Error:  can't concatenate a matrix and vector of different column numbers!";
		BadOperation(a);
		return ;}
	aVector<T>* newRow = NEW aVector<T>(v);
	if(newRow==NULL){
		aString err("Failed 'new' in aMatrix<T>::ConcatRow(aVector<T>&)");BadOperation(err);}
	data.Append(newRow);
	newRow=NULL;
	++rows_;
	if (cols_==0){
		cols_ = v.len();}
	}//end aMatrix<T>::ConcatRow (aVector<T>&)



template<class T > aVector<T>*  aMatrix<T>::Column (long col){
	//returns the column in a new vector
	if((col<0)||(col>=cols())){
		aString a("Error:  asked for column ");a.cat(col);a.cat(" in a matrix with columns 0 to ");a.cat((long)cols()-1);
		BadOperation(a);
		return NULL;}
	aVector<T>* copiedColVec = NEW aVector<T>(rows_);
	if(copiedColVec==NULL){
		aString err("Failed 'new' in aMatrix<T>::Column(long)");BadOperation(err);}
	for(long r=0;r<rows_;++r){
		(*copiedColVec)[r]=data[r][col];}
	return copiedColVec;
	}//end  aMatrix<T>::Column 



template<class T > void aMatrix<T>::Copy(aMatrix<T>& m){
	Clear();
	for(long rr=0;rr<m.rows_;++rr){
		aVector<T>& oldVec = m[rr];
		aVector<T>* newVec = NEW aVector<T>(oldVec);
		if(newVec==NULL){
			aString err("Failed 'new' in aMatrix<T>::Copy(aMatrix<T>&)");BadOperation(err);}
		data.Append(newVec);
		newVec=NULL;}
	rows_=m.rows_;
	cols_=m.cols_;
	nullity_=m.nullity_;
	}//end aMatrix<T>::Copy(aMatrix<T>& m)


template<class T > void aMatrix<T>::DeleteCol (long col){
	//deletes col
	if((col<0)||(col>=cols())){
		aString a("Error:  tried to delete column ");a.cat(col);a.cat(" in a matrix with columns 0 to ");a.cat((long)cols()-1);
		BadOperation(a);
		return;}
	for(long r=0;r<rows_;++r){
		aVector<T>& vec = data[r];
		vec.Delete(col);
		}//end for r
	--cols_;
	}//end aMatrix<T>::DeleteCol
		


template<class T > aMatrix<T>* aMatrix<T>::DeleteColOfCopy (long col){
	//deletes col and returns result in a new matrix
	if((col<0)||(col>=cols())){
		aString a("Error:  tried to delete column ");a.cat(col);a.cat(" in a matrix with columns 0 to ");a.cat((long)cols()-1);
		BadOperation(a);
		return NULL;}
	aMatrix<T>* newMatrix = NEW aMatrix<T>(rows(),cols()-1);
	if(newMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::DeleteCol(long)");BadOperation(err);}
	for(long r=0;r<rows_;++r){
		for(long c=0;c<col;++c){
			(*newMatrix)[r][c]=data[r][c];}
		for(long c=col;c<cols_-1;++c){
			(*newMatrix)[r][c]=data[r][c+1];}
		}//end for r
	return newMatrix;
	}//end aMatrix<T>::DeleteColOfCopy
		


template<class T > void aMatrix<T>::DeleteRow (long row){
	//deletes row 
	if((row<0)||(row>=rows())){
		aString a("Error:  tried to delete row ");a.cat(row);a.cat(" in a matrix with rows 0 to ");a.cat((long)rows()-1);
		BadOperation(a);
		return;}
	data.Delete(row);
	--rows_;
	}//end aMatrix<T>::DeleteRow
		


template<class T > aMatrix<T>* aMatrix<T>::DeleteRowOfCopy (long row){
	//deletes row and returns result in a new matrix
	if((row<0)||(row>=rows())){
		aString a("Error:  tried to delete row ");a.cat(row);a.cat(" in a matrix with rows 0 to ");a.cat((long)rows()-1);
		BadOperation(a);
		return NULL;}
	aMatrix<T>* newMatrix = NEW aMatrix<T>(rows()-1,cols());
	if(newMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::DeleteRow(long)");BadOperation(err);}
	for(long c=0;c<cols_;++c){
		for(long r=0;r<row;++r){
			(*newMatrix)[r][c]=data[r][c];}
		for(long r=row;r<rows_-1;++r){
			(*newMatrix)[r][c]=data[r+1][c];}
		}//end for c
	return newMatrix;
	}//end aMatrix<T>::DeleteRowOfCopy
		


template<class T > long double aMatrix<T>::Determinant (void){// can easily overflow
	//follows Press et al p.47
	if (!Square()){
		aString a("Error:  can't calculate the determinant of a non-square matrix!");
		BadOperation(a);
		return 0.0;}
	if (rows() == 0){
		aString a("Error:  can't calculate the determinant of an empty matrix!");
		BadOperation(a);
		return 0.0;}
	if (rows() == 1){//determinant is just the value in the matrix
		return data[0][0];}
	if (rows() == 2){//determinant is the difference of the cross products
		long double diffCrossProducts = data[0][0] * data[1][1] - data[1][0] * data[0][1];
		return diffCrossProducts;}
	aMatrix<T> dup(*this);//work on a copy
	aVector<long> indx(0,rows());
	int sign=0;
	dup.LUDecomposition(indx,sign);
	long double det = sign;
	for (long c=0; c<cols_;++c){
		det *= dup.data[c][c];//product of diagonal elements; can easily overflow
		}//end for c
	if(det==(-0.0)){det=0.0;}
	return det;
	}//end aMatrix<T>::Determinant


/*
template<class T > double aMatrix<T>::Determinant (void){
	//recursively called
	double det = 0.0;
	if (!Square()){
		aString a("Error:  can't calculate the determinant of a non-square matrix!");
		BadOperation(a);
		return 0.0;}
	if (rows() == 0){
		aString a("Error:  can't calculate the determinant of an empty matrix!");
		BadOperation(a);
		return 0.0;}
	if (rows() == 1){//determinant is just the value in the matrix
		return data[0][0];}
	if (rows() == 2){//determinant is the difference of the cross products
		double diffCrossProducts = data[0][0] * data[1][1] - data[1][0] * data[0][1];
		return diffCrossProducts;}
	for (long c=0; c<cols_;++c){
		double val = data[0][c];
		if (val !=0.0){
			int sign = GetCofactorSign(c+ 1);
			aMatrix<T>* minor = Minor(0, c);
			double minorDet = minor->Determinant();
			det += val * sign * minorDet;
			delete minor; minor = NULL;
			}//end if val
		}//end for c
	if(det==(-0.0)){det=0.0;}
	return det;
	}//end aMatrix<T>::Determinant
*/


template<class T > int aMatrix<T>::EigenCalcs(aVector<long double>& eigenValues, //returns correct eigenvalues
							   aMatrix<long double>& eigenVectorCols){//returns corresponding eigenvectors
	//returns 0 if successful, 1 if not
	if(rows()==0 || rows() != cols()) {
		aError("Error: The matrix is not square so the eigenvectors contain complex numbers");
		return 1;}
	int c, errQ;
	long n_ = eigenValues.len();
	for(c=0;c<n_;++c){
		eigenValues[c]=(long double)0.0;}
	aVector<long double>& diagonalElements = eigenValues;
	aVector<long double> imaginaryElements(0.0,cols());
	aMatrix<long double> *V = ToLongDouble();//work with a copy
	if(symmetric()){
			errQ = tred2(diagonalElements, imaginaryElements, *V);
			if(errQ) {
				delete V; V = NULL;
				return errQ;}
			diagonalElements.Print(cout);
			imaginaryElements.Print(cout);
			V->Print(cout);
			errQ = tql2(diagonalElements, imaginaryElements, *V);
			if(errQ) {
				delete V; V = NULL;
				return errQ;}
			eigenVectorCols = *V;//copy contents to output
			}
		else{
//				aMatrix<long double> *H = ToLongDouble();//scratch pad
				aMatrix<long double> H(*V);//scratch pad
				aVector<long double> ort((long double)0.0,n_);//scratch pad
		         		   
		         // Reduce to Hessenberg form.
		         orthes(*V, ort, H);
		   
		         // Reduce Hessenberg to real Schur form.
		         hqr2(diagonalElements, imaginaryElements, *V, H);
//				delete H; H = NULL;
      			}
	delete V; V = NULL;
	return 0;
	}//aMatrix<T>::EigenCalcs()


template<class T > int aMatrix<T>::GetCofactorSign (long num){
	//recursive; calculates (-1)^(num)
	//used in calculating determinants
	int val= -1;
	if (num > 1){
		val = - GetCofactorSign(num - 1);}
	return val;
	}//end aMatrix<T>::GetCofactorSign


template<class T > aMatrix<T>* 	aMatrix<T>::HorizontalDirectProduct(aMatrix<T>& m){
	//returns direct product of row vectors
	//often used for design matrix construction
	if (rows() !=m.rows()){
		aError("HorizontalDirectProduct() requires matrices with the same number of rows.");
		return NULL;}
	aMatrix<T>& a = *this;//convenience
	aMatrix<T> *hdpp = NEW aMatrix<T>(T(0.0),rows(),cols()*m.cols());
	aMatrix<T>& hdp = *hdpp;
	for(long r=0;r<rows();++r){
		for(long c1=0;c1<cols();++c1){
			T val1 = a[r][c1];
			for(long c2=0;c2<m.cols();++c2){
				long hdpCol = c1*m.cols()+c2;
				T val2 = m[r][c2];
				hdp[r][hdpCol] = val1 * val2;
				}//c2
			}//c1
		}//r
	return hdpp;
	}//HorizontalDirectProduct(aMatrix<T>&)
							
					


template<class T > int aMatrix<T>::hqr2(aVector<long double>& diagonalElements,
										 aVector<long double>& imaginaryElements,
										 aMatrix<long double>& V,
										 aMatrix<long double>& H){
	//version incorporated from JAMA library, in jama_eig.h
	long nn = diagonalElements.len();
	aVector<long double>&  d = diagonalElements;//shorthand
	aVector<long double>&  e = imaginaryElements;
      //  This is derived from the Algol procedure hqr2,
      //  by Martin and Wilkinson, Handbook for Auto. Comp.,
      //  Vol.ii-Linear Algebra, and the corresponding
      //  Fortran subroutine in EISPACK.
   
      // Initialize   
      long n_ = nn-1;
      long low = 0;
      long high = nn-1;
      long double eps = pow((long double)2.0,(long double)-52.0);
      long double exshift = (long double)0.0;
      long double p,q,r,s,z,t,w,x,y,cdivr,cdivi;
   	  p=q=r=s=z=t=w=x=y=cdivr=cdivi=(long double)0.0;
      // Store roots isolated by balance and compute matrix norm
   
      long double norm = (long double)0.0;
      for (long i = 0; i < nn; i++) {
         if ((i < low) || (i > high)) {
            d[i] = H[i][i];
            e[i] = (long double)0.0;
         }
         for (long j = MAX(i-1,(long)0); j < nn; j++) {
            norm += ABS(H[i][j]);
         }
      }
   
      // Outer loop over eigenvalue index
   
      int iter = 0;
      while (n_ >= low) {
   
         // Look for single small sub-diagonal element
   
         long l = n_;
         while (l > low) {
            s = ABS(H[l-1][l-1]) + ABS(H[l][l]);
            if (s == (long double)0.0) {
               s = norm;
            }
            if (ABS(H[l][l-1]) < eps * s) {
               break;
            }
            l--;
         }
       
         // Check for convergence
         // One root found
   
         if (l == n_) {
            H[n_][n_] += exshift;
            d[n_] = H[n_][n_];
            e[n_] = (long double)0.0;
            n_--;
            iter = 0;
   
         // Two roots found
   
         } else if (l == n_-1) {
            w = H[n_][n_-1] * H[n_-1][n_];
            p = (H[n_-1][n_-1] - H[n_][n_]) / (long double)2.0;
            q = p * p + w;
            z = sqrt(ABS(q));
            H[n_][n_] += exshift;
            H[n_-1][n_-1] += exshift;
            x = H[n_][n_];
   
            // Real pair
   
            if (q >= (long double)0.0) {
               if (p >= (long double)0.0) {
                  z = p + z;
               } else {
                  z = p - z;
               }
               d[n_-1] = x + z;
               d[n_] = d[n_-1];
               if (z != (long double)0.0) {
                  d[n_] = x - w / z;
               }
               e[n_-1] = (long double)0.0;
               e[n_] = (long double)0.0;
               x = H[n_][n_-1];
               s = ABS(x) + ABS(z);
               p = x / s;
               q = z / s;
               r = sqrt(p * p + q * q);
               p = p / r;
               q = q / r;
   
               // Row modification
   
               for (int j = n_-1; j < nn; j++) {
                  z = H[n_-1][j];
                  H[n_-1][j] = q * z + p * H[n_][j];
                  H[n_][j] = q * H[n_][j] - p * z;
               }
   
               // Column modification
   
               for (int i = 0; i <= n_; i++) {
                  z = H[i][n_-1];
                  H[i][n_-1] = q * z + p * H[i][n_];
                  H[i][n_] = q * H[i][n_] - p * z;
               }
   
               // Accumulate transformations
   
               for (int i = low; i <= high; i++) {
                  z = V[i][n_-1];
                  V[i][n_-1] = q * z + p * V[i][n_];
                  V[i][n_] = q * V[i][n_] - p * z;
               }
   
            // Complex pair
   
            } else {
               d[n_-1] = x + p;
               d[n_] = x + p;
               e[n_-1] = z;
               e[n_] = -z;
            }
            n_ = n_ - 2;
            iter = 0;
   
         // No convergence yet
   
         } else {
   
            // Form shift
   
            x = H[n_][n_];
            y = (long double)0.0;
            w = (long double)0.0;
            if (l < n_) {
               y = H[n_-1][n_-1];
               w = H[n_][n_-1] * H[n_-1][n_];
            }
   
            // Wilkinson's original ad hoc shift
   
            if (iter == 10) {
               exshift += x;
               for (int i = low; i <= n_; i++) {
                  H[i][i] -= x;
               }
               s = ABS(H[n_][n_-1]) + ABS(H[n_-1][n_-2]);
               x = y = (long double)0.75 * s;
               w = (long double)-0.4375 * s * s;
            }

            // MATLAB's new ad hoc shift

            if (iter == 30) {
                s = (y - x) / (long double)2.0;
                s = s * s + w;
                if (s > (long double)0.0) {
                    s = sqrt(s);
                    if (y < x) {
                       s = -s;
                    }
                    s = x - w / ((y - x) / (long double)2.0 + s);
                    for (int i = low; i <= n_; i++) {
                       H[i][i] -= s;
                    }
                    exshift += s;
                    x = y = w = (long double)0.964;
                }
            }
   
            iter = iter + 1;   // (Could check iteration count here.)
   
            // Look for two consecutive small sub-diagonal elements
   
            int m = n_-2;
            while (m >= l) {
               z = H[m][m];
               r = x - z;
               s = y - z;
               p = (r * s - w) / H[m+1][m] + H[m][m+1];
               q = H[m+1][m+1] - z - r - s;
               r = H[m+2][m+1];
               s = ABS(p) + ABS(q) + ABS(r);
               p = p / s;
               q = q / s;
               r = r / s;
               if (m == l) {
                  break;
               }
               if (ABS(H[m][m-1]) * (ABS(q) + ABS(r)) <
                  eps * (ABS(p) * (ABS(H[m-1][m-1]) + ABS(z) + ABS(H[m+1][m+1])))) {
                     break;
               }
               m--;
            }
   
            for (int i = m+2; i <= n_; i++) {
               H[i][i-2] = (long double)0.0;
               if (i > m+2) {
                  H[i][i-3] = (long double)0.0;
               }
            }
   
            // Double QR step involving rows l:n and columns m:n
   
            for (long k = m; k <= n_-1; k++) {
               int notlast = (k != n_-1);
               if (k != m) {
                  p = H[k][k-1];
                  q = H[k+1][k-1];
                  r = (notlast ? H[k+2][k-1] : (long double)0.0);
                  x = ABS(p) + ABS(q) + ABS(r);
                  if (x != (long double)0.0) {
                     p = p / x;
                     q = q / x;
                     r = r / x;
                  }
               }
               if (x == (long double)0.0) {
                  break;
               }
               s = sqrt(p * p + q * q + r * r);
               if (p < (long double)0.0) {
                  s = -s;
               }
               if (s != (long double)0.0) {
                  if (k != m) {
                     H[k][k-1] = -s * x;
                  } else if (l != m) {
                     H[k][k-1] = -H[k][k-1];
                  }
                  p = p + s;
                  x = p / s;
                  y = q / s;
                  z = r / s;
                  q = q / p;
                  r = r / p;
   
                  // Row modification
   
                  for (int j = k; j < nn; j++) {
                     p = H[k][j] + q * H[k+1][j];
                     if (notlast) {
                        p = p + r * H[k+2][j];
                        H[k+2][j] = H[k+2][j] - p * z;
                     }
                     H[k][j]   -= p * x;
                     H[k+1][j] -= p * y;
                  }
   
                  // Column modification
   
                  for (long i = 0; i <= MIN(n_,(long)(k+3)); i++) {
                     p = x * H[i][k] + y * H[i][k+1];
                     if (notlast) {
                        p += z * H[i][k+2];
                        H[i][k+2] -= p * r;
                     }
                     H[i][k]   -= p;
                     H[i][k+1] -= p * q;
                  }
   
                  // Accumulate transformations
   
                  for (long i = low; i <= high; i++) {
                     p = x * V[i][k] + y * V[i][k+1];
                     if (notlast) {
                        p = p + z * V[i][k+2];
                        V[i][k+2] -= p * r;
                     }
                     V[i][k]   -= p;
                     V[i][k+1] -= p * q;
                  }
               }  // (s != 0)
            }  // k loop
         }  // check convergence
      }  // while (n >= low)
      
      // Backsubstitute to find vectors of upper triangular form

      if (norm == (long double)0.0) {
         return 0;
      }
   
      for (n_ = nn-1; n_ >= 0; n_--) {
         p = d[n_];
         q = e[n_];
   
         // Real vector
   
         if (q == (long double)0) {
            int l = n_;
            H[n_][n_] = 1.0;
            for (long i = n_-1; i >= 0; i--) {
               w = H[i][i] - p;
               r = (long double)0.0;
               for (long j = l; j <= n_; j++) {
                  r += H[i][j] * H[j][n_];
               }
               if (e[i] < (long double)0.0) {
                  z = w;
                  s = r;
               } else {
                  l = i;
                  if (e[i] == (long double)0.0) {
                     if (w != (long double)0.0) {
                        H[i][n_] = -r / w;
                     } else {
                        H[i][n_] = -r / (eps * norm);
                     }
   
                  // Solve real equations
   
                  } else {
                     x = H[i][i+1];
                     y = H[i+1][i];
                     q = (d[i] - p) * (d[i] - p) + e[i] * e[i];
                     t = (x * s - z * r) / q;
                     H[i][n_] = t;
                     if (ABS(x) > ABS(z)) {
                        H[i+1][n_] = (-r - w * t) / x;
                     } else {
                        H[i+1][n_] = (-s - y * t) / z;
                     }
                  }
   
                  // Overflow control
   
                  t = ABS(H[i][n_]);
                  if ((eps * t) * t > 1) {
                     for (long j = i; j <= n_; j++) {
                        H[j][n_] = H[j][n_] / t;
                     }
                  }
               }
            }
   
         // Complex vector
   
         } else if (q < 0) {
            int l = n_-1;

            // Last vector component imaginary so matrix is triangular
   
            if (ABS(H[n_][n_-1]) > ABS(H[n_-1][n_])) {
               H[n_-1][n_-1] = q / H[n_][n_-1];
               H[n_-1][n_] = -(H[n_][n_] - p) / H[n_][n_-1];
            } else {
               cdiv((long double)0.0,-H[n_-1][n_],H[n_-1][n_-1]-p,q,cdivr,cdivi);
               H[n_-1][n_-1] = cdivr;
               H[n_-1][n_] = cdivi;
            }
            H[n_][n_-1] = (long double)0.0;
            H[n_][n_] = (long double)1.0;
            for (long i = n_-2; i >= 0; i--) {
               long double ra,sa,vr,vi;
               ra = sa = (long double)0.0;
               for (long j = l; j <= n_; j++) {
                  ra +=  H[i][j] * H[j][n_-1];
                  sa +=  H[i][j] * H[j][n_];
               }
               w = H[i][i] - p;
   
               if (e[i] < (long double)0.0) {
                  z = w;
                  r = ra;
                  s = sa;
               } else {
                  l = i;
                  if (e[i] == (long double)0.0) {
                     cdiv(-ra,-sa,w,q,cdivr,cdivi);
                     H[i][n_-1] = cdivr;
                     H[i][n_] = cdivi;
                  } else {
   
                     // Solve complex equations
   
                     x = H[i][i+1];
                     y = H[i+1][i];
                     vr = (d[i] - p) * (d[i] - p) + e[i] * e[i] - q * q;
                     vi = (d[i] - p) * (long double)2.0 * q;
                     if ((vr == (long double)0.0) && (vi == (long double)0.0)) {
                        vr = eps * norm * (ABS(w) + ABS(q) + ABS(x) + ABS(y) + ABS(z));
                     }
                     cdiv(x*r-z*ra+q*sa,x*s-z*sa-q*ra,vr,vi,cdivr,cdivi);
                     H[i][n_-1] = cdivr;
                     H[i][n_] = cdivi;
                     if (ABS(x) > (ABS(z) + ABS(q))) {
                        H[i+1][n_-1] = (-ra - w * H[i][n_-1] + q * H[i][n_]) / x;
                        H[i+1][n_] = (-sa - w * H[i][n_] - q * H[i][n_-1]) / x;
                     } else {
                        cdiv(-r-y*H[i][n_-1],-s-y*H[i][n_],z,q,cdivr,cdivi);
                        H[i+1][n_-1] = cdivr;
                        H[i+1][n_] = cdivi;
                     }
                  }
   
                  // Overflow control

                  t = MAX(ABS(H[i][n_-1]),ABS(H[i][n_]));
                  if ((eps * t) * t > (long double)1.0) {
                     for (int j = i; j <= n_; j++) {
                        H[j][n_-1] = H[j][n_-1] / t;
                        H[j][n_] = H[j][n_] / t;
                     }
                  }
               }
            }
         }
      }
   
      // Vectors of isolated roots
   
      for (long i = 0; i < nn; i++) {
         if (i < low || i > high) {
            for (int j = i; j < nn; j++) {
               V[i][j] = H[i][j];
            }
         }
      }
   
      // Back transformation to get eigenvectors of original matrix
   
      for (long j = nn-1; j >= low; j--) {
         for (long i = low; i <= high; i++) {
            z = (long double)0.0;
            for (long k = low; k <= MIN(j,(long)high); k++) {
               z = z + V[i][k] * H[k][j];
            }
            V[i][j] = z;
         }
      }
   
	return 0;
	}//end aMatrix<T>::hqr2



template<class T > void aMatrix<T>::InsertColumn (aVector<T>& v, int colNum){
	//inserts copy of column v into this 
	//rows_ must match len_ or error returned
	if((rows()>0)&&(rows()!=v.len())){
		aString a="Error:  can't combine a matrix and column vector of different row numbers!";
		BadOperation(a);
		return ;}
	if(rows_==0){
			rows_=v.len();//add this column
			cols_=1;
			for(long r=0;r<rows_;++r){
				aVector<T>* newRow = NEW aVector<T>(v[r]);
				if(newRow==NULL){
					aString err("Failed 'new' in aMatrix<T>::InsertCol(aVector<T>&, int)");BadOperation(err);}
				data.Append(newRow);
				newRow=NULL;}
			}
		else{
			for(long r=0;r<rows_;++r){
				aVector<long double>& rowVec =data[r];
				rowVec.Insert(v[r], colNum);}
			++cols_;
			}
	}//end aMatrix<T>::InsertColumn (aVector<T>&, int)
	
	
	
template<class T > void aMatrix<T>::InsertRow (aVector<T>& v, int rowNum){
	//inserts copy of row v into this 
	//cols_ must match len_ or error returned
	if((cols()>0)&&(cols()!=v.len())){//an empty matrix can take any size vector
		aString a="Error:  can't combine a matrix and row vector of different column numbers!";
		BadOperation(a);
		return ;}
	aVector<T>* newRow = NEW aVector<T>(v);
	if(newRow==NULL){
		aString err("Failed 'new' in aMatrix<T>::InsertRow(aVector<T>&, int)");BadOperation(err);}
	data.Insert(newRow, rowNum);
	newRow=NULL;
	++rows_;
	if (cols_==0){
		cols_ = v.len();}
	}//end aMatrix<T>::InsertRow (aVector<T>&, int)



template<class T > aMatrix<long double>* aMatrix<T>::InverseByMinors (void){
	//using minors method; long computing time; result in new matrix
	//only does square matrices
	if (!Square()){
		aString a("Error:  can't presently find the inverse of a non-square matrix!");
		BadOperation(a);
		return NULL;}
	aMatrix<long double>* invMatrix = NEW aMatrix<long double>(rows_, cols_);
	if(invMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::InverseByMinors()");BadOperation(err);}
	aMatrix<long double>* m = ToDouble();//copies this to type 'double'
	for (long r=0;r<rows_;++r){
		for(long c=0;c<cols_;++c){
			aMatrix<T>* minor = Minor(c, r);//reversing automatically transposes invMatrix
			long double minorDet = minor->Determinant();
			int sign = GetCofactorSign((r+1) + (c+1));
			long double val = sign * minorDet;
			if(val==(long double)(-0.0)){
				val=(long double)0.0;}
			(*invMatrix)[r][c]=val;
			delete minor;  minor = NULL;
			}}//end for r & c
	aVector<long double>* rowVec = invMatrix->Row(0);//a copy
	aVector<long double>* firstCol = m->Column(0);//a copy
	long double det = rowVec->Multiply(*firstCol);//the determinant
	if(det==(long double)(-0.0)){
		det=(long double)0.0;}
	delete rowVec; rowVec=NULL;
	delete firstCol; firstCol=NULL;
	if (det != (long double)0.0){
			invMatrix->Multiply((long double)1.0 / det);}
		else{
			aString a("Error: zero determinant encountered:  can't invert matrix.");
			BadOperation(a);
			delete invMatrix; invMatrix=NULL;}
	delete m; m=NULL;
	return invMatrix;
	}//end  aMatrix<T>::Inverse


template<class T > void aMatrix<T>::LUBackSubstitution (aVector<long double>& b, aVector<long>& indx, int sign){
	//debug first;  overwrites the original matrix
	//for square matrices; see notes in LUDecomposition
	long ii = 0;
	for(long c=0;c<cols_;++c){
		long ip = indx[c];
		long double sum = b[ip];
		 b[ip] = b[c];
		if (ii > 0){
				for (long j = ii;j< c - 1;++j){
					sum -= ((long double) data[c][j]) * b[j];}}
			else if (sum > (long double)0.0){
				ii = c;}
		b[c]=sum;
		}//end for c
	for (c=cols()-1;c>=0;--c){
		long double sum = b[c];
		for (long j = c + 1;j<cols();++j){
			sum -= ((long double) data[c][j]) *b[j];}
		b[c] = sum / (long double) data[c][c];
		}//end for c downto
	}//end aMatrix<T>::LUBackSubstitution



template<class T > void aMatrix<T>::LUDecomposition (aVector<long>& indx, int& sign){
	//debug first;  overwrites the original matrix
	//faster version for inversion; from Press et al 1992, p. 46}
	//indx holds the order of the original row numbers
	//i=row, iMax is a row counter, j=col, k is either row or column
	sign = 1; double big;
	int nonSingularMatrix = 1;//true
	long n = cols();//=rows too since square matrices are inverted
	aVector<double> vv(0.0,n);//holds implicit scaling of rows
	for(long i=0;i<n;++i){
		indx[i]=i;}// rows start in correct order
	for(long i=0;i<n;++i){
		long double big = 0.0;
		for(long j=0;j<n;++j){
			long double temp = ABS(data[i][j]);
			if (temp > big){
				big = temp;}
			}//end for j
		if (big == (long double)0.0){
				nonSingularMatrix = 1;// false
				aString a("Error:  can't decompose a singular matrix");
				BadOperation(a);
				return;}
			else{
				vv[i] = (long double)1.0 / big;}
		}//end for i
	long iMax=0;
	for(long j=0;j<n;++j){
//		for(long i=0;i<j - 1;++i){ 
		for(long i=0;i<j;++i){ 
			long double sum = data[i][j];
			for (long k=0;k<i;++k){
				sum -= ((long double) data[i][k]) * ((long double) data[k][j]);}
			data[i][j] = sum;
			}//end for i
		big = 0.0;
		for(long i=j;i<n;++i){
			long double sum = data[i][j];
			for (long k=0;k<j;++k){
				sum -= ((long double) data[i][k]) * ((long double) data[k][j]);}
			data[i][j] = sum;
			long double dum = vv[i] * ABS(sum);
			if (dum >= big){
				big = dum;
				iMax = i;
				}//end if dum>big
			}//end for i
		if (j != iMax){
			for (long k=0;k<n;++k){
				//switch the row values in a
				T dum = data[iMax][k];
				data[iMax][k]=data[j][k];
				data[j][k]=dum;
				}//end for k
			sign = - sign;
			vv[iMax]=vv[j];
			}//end if j != iMax
		indx[j]=iMax;
		if(data[j][j] == (long double)0.0){
			data[j][j] = (long double)1.0e-20;}//set to non-zero small number to dodge a singularity problem
		if (j != n-1){//if not last element
			long double dum = (long double)1.0 / (long double)data[j][j];
			for (long i= j + 1;i<n;++i){
				data[i][j] *= dum;}
			}//end if j
		}//end for j
	}//end aMatrix<T>::LUDecomposition (aVector<long>&, int)



template<class T > aMatrix<T>*  aMatrix<T>::LUInverse (void){
	//debug first;  returns result in a new matrix
	if (!Square()){
		aString a("Error:  can't presently find the inverse of a non-square matrix!");
		BadOperation(a);
		return NULL;}
	aMatrix<T>* invMatrix = NEW aMatrix<T>(rows(),cols());
	if(invMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::LUInverse()");BadOperation(err);}
	aMatrix<T>* coeffMatrixCopy = NEW aMatrix<T>(this);//operate on the copy
	if(coeffMatrixCopy==NULL){
		aString err("Failed 'new' in aMatrix<T>::LUInverse()");BadOperation(err);}
	aVector<long> rowArrangement(rows());
	int sign=0;
	coeffMatrixCopy->LUDecomposition(rowArrangement, sign);
	for (long c=0;c<cols();++c){
		aVector<double> colVec(0.0,rows());
		colVec[c]=1.0;
		coeffMatrixCopy->LUBackSubstitution(colVec, rowArrangement, sign);
		for(long r=0;r<rows();++r){
			(*invMatrix)[r][c] = colVec[r];}
		}//end for c
	//toss temporary vectors & matrices
	delete coeffMatrixCopy; coeffMatrixCopy=NULL;
	return invMatrix;
	}//end aMatrix<T>::LUInverse



template<class T > aMatrix<long double>* aMatrix<T>::MatrixPlusForm(void){
	//returns [Inverse(X'.X)].X; NULL if inversion fails
	aMatrix<long double> *X = ToLongDouble();
	aMatrix<long double>* Xprime = X->Transpose();
	aMatrix<long double>* XprimeX = Xprime->Multiply(*X);
	aMatrix<long double>* invXprimeX = XprimeX->Inverse();
	delete X; X=NULL;
	delete XprimeX; XprimeX=NULL;
	if(invXprimeX==NULL){
		cout<<"ERROR:  in MatrixPlusForm(); can't invert the matrix"<<endl;
		delete Xprime; Xprime=NULL;
		return NULL;}
	aMatrix<long double>* invXprimeXX = invXprimeX->Multiply(*Xprime);
	delete invXprimeX; invXprimeX=NULL;
	delete Xprime; Xprime=NULL;
	return invXprimeXX;
	}//end  aMatrix<T>::MatrixPlusForm()



template<class T > aMatrix<long double>* aMatrix<T>::MatrixPlusForm(aMatrix<long double>& invXprimeX){
	//returns [Inverse(X'.X)].X
	aMatrix<long double> *X = ToDouble();
	aMatrix<long double>* Xprime = X->Transpose();
	aMatrix<long double>* invXprimeXX = invXprimeX.Multiply(*Xprime);
	delete X; X=NULL;
	delete Xprime; Xprime=NULL;
	return invXprimeXX;
	}//end  aMatrix<T>::MatrixPlusForm()



/*
template<class T > aMatrix<double>* aMatrix<T>::MatrixPlusForm(void){
	//returns Moore-Penrose inverse; X'.[Inverse(X.X')].X'.X.[Inverse(X'.X)].X'; NULL if inversion fails
	aMatrix<double> *X = ToDouble();
	aMatrix<double>* Xprime = X->Transpose();
	aMatrix<double>* XprimeX = Xprime->Multiply(*X);
	aMatrix<double>* invXprimeX = XprimeX->Inverse();
	delete XprimeX; XprimeX=NULL;
	if(invXprimeX==NULL){
		cout<<"ERROR:  in MatrixPlusForm(); can't invert the matrix"<<endl;
		delete X; X=NULL;
		delete Xprime; Xprime=NULL;
		return NULL;}
	aMatrix<double>* XXprime = X->Multiply(*Xprime);
	aMatrix<double>* invXXprime = XXprime->Inverse();
	delete XXprime; XXprime=NULL;
	if(invXXprime==NULL){
		cout<<"ERROR:  in MatrixPlusForm(); can't invert the matrix"<<endl;
		delete X; X=NULL;
		delete Xprime; Xprime=NULL;
		return NULL;}
	aMatrix<double>* XprInvXXpr = Xprime->Multiply(*invXXprime);
	delete invXXprime; invXXprime=NULL;
	aMatrix<double>* XprInvXXprX = XprInvXXpr->Multiply(*X);
	delete X; X=NULL;
	aMatrix<double>* XprInvXXprXInvXprX = XprInvXXprX->Multiply(*invXprimeX);
	delete XprInvXXprX; XprInvXXprX=NULL;
	delete XprInvXXpr; XprInvXXpr=NULL;
	delete invXprimeX;invXprimeX=NULL;
	aMatrix<double>* XprInvXXprXInvXprXXpr = XprInvXXprXInvXprX->Multiply(*Xprime);
	delete XprInvXXprXInvXprX; XprInvXXprXInvXprX=NULL;
	delete Xprime; Xprime=NULL;
	return XprInvXXprXInvXprXXpr;
	}//end  aMatrix<T>::MatrixPlusForm()



template<class T > aMatrix<double>* aMatrix<T>::MatrixPlusForm(aMatrix<double>& invXprimeX){
	//returns Moore-Penrose inverse; X'.[Inverse(X.X')].X'.X.[Inverse(X'.X)].X'; NULL if inversion fails
	aMatrix<double> *X = ToDouble();
	aMatrix<double>* Xprime = X->Transpose();
	aMatrix<double>* XXprime = X->Multiply(*Xprime);
	aMatrix<double>* invXXprime = XXprime->Inverse();
	delete XXprime; XXprime=NULL;
	if(invXXprime==NULL){
		cout<<"ERROR:  in MatrixPlusForm(); can't invert the matrix"<<endl;
		delete X; X=NULL;
		delete Xprime; Xprime=NULL;
		return NULL;}
	aMatrix<double>* XprInvXXpr = Xprime->Multiply(*invXXprime);
	aMatrix<double>* XprInvXXprX = XprInvXXpr->Multiply(*X);
	delete X; X=NULL;
	aMatrix<double>* XprInvXXprXInvXprX = XprInvXXprX->Multiply(invXprimeX);
	delete XprInvXXpr; XprInvXXpr=NULL;
	aMatrix<double>* XprInvXXprXInvXprXXpr = XprInvXXprXInvXprX->Multiply(*Xprime);
	delete XprInvXXprXInvXprX; XprInvXXprXInvXprX=NULL;
	delete Xprime; Xprime=NULL;
	return XprInvXXprXInvXprXXpr;
	}//end  aMatrix<T>::MatrixPlusForm()

*/

template<class T > T  aMatrix<T>::Max (void){
	//highest value in matrix
	aMatrix<T>& m = *this;
	T high = m[0][0];
	for(long r=0;r<rows_;++r){
		for(long c=0;c<cols_;++c){
			high = max(high,m[r][c]);}}
	return high;
	}//end  aMatrix<T>::Max


template<class T > T  aMatrix<T>::Min (void){
	//lowest value in matrix
	aMatrix<T>& m = *this;
	T low = m[0][0];
	for(long r=0;r<rows_;++r){
		for(long c=0;c<cols_;++c){
			low = min(low,m[r][c]);}}
	return low;
	}//end  aMatrix<T>::Min




template<class T > aMatrix<T>*  aMatrix<T>::Minor (long  row, long col){
	//returns result in a new matrix
	aMatrix<T>*newMatrix1 = DeleteRowOfCopy(row);
	aMatrix<T>*newMatrix2 = newMatrix1->DeleteColOfCopy(col);
	delete newMatrix1; newMatrix1 = NULL;	
	return newMatrix2;
	}//end aMatrix<T>::Minor 



template<class T > aMatrix<long double>* aMatrix<T>::Multiply(aMatrix<T>& m){
	//premultiplies m to this and returns result in a new matrix
	//returns the product matrix; a null pointer if calling error occurs
	if(!CanBeMultiplied(m)){
		aString a="Error:  can't multiply matrices of unmatched sizes!";
		BadOperation(a);
		return NULL;}
	aMatrix<long double>* productMatrix = NEW aMatrix<long double>(rows(), m.cols());
	if(productMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::Multiply(aMatrix<T>&)");BadOperation(err);}
	aMatrix<T>& a = *this;//convenience
	for (long r=0;r<rows();++r){
//		aVector<T>* rowVec1 = Row(r);
		for (long c=0;c<m.cols();++c){
//			aVector<T>* colVec2 = m.Column(c);
//			long double val = rowVec1->Multiply(*colVec2);
//			if(val==(long double)(-0.0)){
//				val=(long double)0.0;}
//			(*productMatrix)[r][c]=val;
//			delete colVec2; colVec2=NULL;
			
			long double sum_=(long double)0.0;
			for(long i=0;i<cols();++i){//i=length of the row
					sum_ += (long double)a[r][i] * (long double) m[i][c];}
			if(sum_ ==(long double)(-0.0)){
				sum_ =(long double)0.0;}
			(*productMatrix)[r][c]=sum_;
			}//end for c
//		delete rowVec1; rowVec1=NULL;
		}//end for r
	return productMatrix;
	}//end aMatrix<T>::Multiply(aMatrix<T>&)


template<class T > aMatrix<long double>* aMatrix<T>::Multiply(aVector<T>& v){
	//premultiplies v to this and returns result in a new matrix
	//returns the product matrix; a null pointer if calling error occurs
	if(!CanBeMultiplied(v)){
		aString a="Error:  can't multiply a matrix & vector of unmatched sizes!";
		BadOperation(a);
		return NULL;}
	aMatrix<long double>* productMatrix = NEW aMatrix<long double>(rows(), v.len());
	if(productMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::Multiply(aVector<T>&)");BadOperation(err);}
	aMatrix<T>& a = *this;//convenience
	for (long r=0;r<rows();++r){
//		aVector<T>* rowVec1 = Row(r);
//		long double val = rowVec1->Multiply(v);
//		if(val==(long double)(-0.0)){
//			val=(long double)0.0;}
//		(*productMatrix)[r][0]=val;
//		delete rowVec1; rowVec1=NULL;

		long double sum_=(long double)0.0;
		for(long i=0;i<cols();++i){//i=length of the row
				sum_ += (long double)a[r][i] * (long double) v[i];}
		if(sum_ ==(long double)(-0.0)){
			sum_ =(long double)0.0;}
		(*productMatrix)[r][0]=sum_;

		}//end for r
	return productMatrix;
	}//end aMatrix<T>::Multiply(aMatrix<T>&)


template<class T > void aMatrix<T>::Multiply (T scalar){
	if(scalar == (T)1.0){return;}
	for(long r=0;r<rows_;++r){
		for(long c=0;c<cols_;++c){
			T val = data[r][c] * scalar;
			if(val == (T)(-0.0)){
				val = (T) 0.0;}
			data[r][c]=val;}}
	}//end aMatrix<T>::Multiply(T)



template<class T > int aMatrix<T>::orthes(aMatrix<long double>& V,
										 aVector<long double>& ort,
										 aMatrix<long double>& H){
	//version incorporated from JAMA library, in jama_eig.h
	//used to get eigenvalues && eigenvectors from non-symmetric matrices
	long n_ = ort.len();
	
      //  This is derived from the Algol procedures orthes and ortran,
      //  by Martin and Wilkinson, Handbook for Auto. Comp.,
      //  Vol.ii-Linear Algebra, and the corresponding
      //  Fortran subroutines in EISPACK.
   
      int low = 0;
      int high = n_-1;
   
      for (int m = low+1; m <= high-1; m++) {
   
         // Scale column.
   
         long double scale = 0.0;
         for (int i = m; i <= high; i++) {
            scale = scale + ABS(H[i][m-1]);
         }
         if (scale != (long double) 0.0) {
   
            // Compute Householder transformation.
   
            long double h = 0.0;
            for (int i = high; i >= m; i--) {
               ort[i] = H[i][m-1]/scale;
               h += ort[i] * ort[i];
            }
            long double g = sqrt(h);
            if (ort[m] > (long double) 0.0) {
               g = -g;
            }
            h -= ort[m] * g;
            ort[m] -= g;
   
            // Apply Householder similarity transformation
            // H = (I-u*u'/h)*H*(I-u*u')/h)
   
            for (int j = m; j < n_; j++) {
               long double f = (long double) 0.0;
               for (int i = high; i >= m; i--) {
                  f += ort[i]*H[i][j];
               }
               f = f/h;
               for (int i = m; i <= high; i++) {
                  H[i][j] -= f*ort[i];
               }
           }
   
           for (int i = 0; i <= high; i++) {
               long double f = (long double) 0.0;
               for (int j = high; j >= m; j--) {
                  f += ort[j]*H[i][j];
               }
               f = f/h;
               for (int j = m; j <= high; j++) {
                  H[i][j] -= f*ort[j];
               }
            }
            ort[m] *= scale;
            H[m][m-1] = scale*g;
         }
      }
   
      // Accumulate transformations (Algol's ortran).

      for (int i = 0; i < n_; i++) {
         for (int j = 0; j < n_; j++) {
            V[i][j] = (i == j ? (long double) 1.0 : (long double) 0.0);
         }
      }

      for (int m = high-1; m >= low+1; m--) {
         if (H[m][m-1] != (long double) 0.0) {
            for (int i = m+1; i <= high; i++) {
               ort[i] = H[i][m-1];
            }
            for (int j = m; j <= high; j++) {
               long double g = (long double) 0.0;
               for (int i = m; i <= high; i++) {
                  g += ort[i] * V[i][j];
               }
               // Double division avoids possible underflow
               g = (g / ort[m]) / H[m][m-1];
               for (int i = m; i <= high; i++) {
                  V[i][j] += g * ort[i];
               }
            }
         }
      }
   
	return 0;
	}//end aMatrix<T>::orthes



template<class T > void  aMatrix<T>::Print (aString& message, char* outFileName, char separator){//file assumed closed
	ofstream outFile;
	outFile.open(outFileName,ios::out |ios::app);
	Print(message, outFile, separator);
	outFile<<endl;
	outFile.close();
	}//end aMatrix<T>::Print



template<class T > void  aMatrix<T>::Print (aString& message, ostream& outFile, char separator){
	//file assumed open
	if(message != ""){outFile<<message<<endl;}
	for(long r=0;r<rows_;++r){
		for(long c=0;c<cols_-1;++c){
			outFile<<data[r][c]<<separator;}
		outFile<<data[r][cols_-1]<<endl;
		}//end for r
	outFile<<endl;
	}//end aMatrix<T>::Print



template<class T > double  aMatrix<T>::Pythag (double a, double b){
	//gives back (a^2 +b^2)^(.5) without overflow problems; Press et al p.70
	double absa = ABS(a);
	double absb = ABS(b);
	double val=0.0;
	if (absa > absb){
			double ratio = absb / absa;
			val = absa * sqrt(1.0 + sqr(ratio));}
		else if (absb != 0.0){
			double ratio = absa / absb;
			val = absb * sqrt(1.0 + sqr(ratio));}
	return val;
	}//end aMatrix<T>::Pythag


template<class T > long double  aMatrix<T>::Pythag (long double a, long double b){
	//gives back (a^2 +b^2)^(.5) without overflow problems; Press et al p.70
	long double absa = ABS(a);
	long double absb = ABS(b);
	long double val=(long double)0.0;
	if (absa > absb){
			long double ratio = absb / absa;
			val = absa * sqrt((long double)1.0 + sqr(ratio));}
		else if (absb != (long double)0.0){
			long double ratio = absa / absb;
			val = absb * sqrt((long double)1.0 + sqr(ratio));}
	return val;
	}//end aMatrix<T>::Pythag


template<class T > void  aMatrix<T>::RandomPermutation (Ran3& ran){
	//randomizes the rows and columns, keeping them associated
	aList<int> oldrcnums, newrcnums;
	for (long rc=0;rc<rows_;++rc){oldrcnums.Append(rc);}//original sequence
	aList<int> workrcnums(oldrcnums);
	for (long rc=rows_-1;rc>=0;--rc){//choose new sequence
		int item = ran.longBetween(0, oldrcnums.len()-1);
		newrcnums.Append(oldrcnums[item]);
		oldrcnums.Delete(item);}
//	newrcnums.Print(cout, " ");
//	workrcnums.Print(cout, " ");
	//permute to the new sequence
	for (long rc=0;rc<rows_;++rc){
		int temploc = workrcnums.Loc(newrcnums[rc]);
		SwapRowsCols(rc,temploc);
//		Print();
		//keep track of swaps in the working sequence
		int temp = workrcnums[temploc];
		workrcnums[temploc] = workrcnums[rc];
		workrcnums[rc] = temp;
//		workrcnums.Print(cout, " ");
		}
	}//end aMatrix<T>::RandomPermutation



template<class T > aVector<T>*  aMatrix<T>::Row (long row){
	//returns the row in a new vector
	if((row<0)||(row>=rows())){
		aString a("Error:  asked for row ");a.cat(row); a.cat(" in a matrix with rows 0 to "); a.cat((long)rows()-1); 
		BadOperation(a);
		return NULL;}
	aVector<T>* copiedRowVec = NEW aVector<T>(cols_);
	if(copiedRowVec==NULL){
		aString err("Failed 'new' in aMatrix<T>::Row(long)");BadOperation(err);}
	for(long c=0;c<cols_;++c){
		(*copiedRowVec)[c]=data[row][c];}
	return copiedRowVec;
	}//end  aMatrix<T>::Row 


template<class T > T  aMatrix<T>::sesq (void){
	//returns sum of squared elements of matrix
	T sum=(T)0.0;
	aMatrix<T>& x = *this;
	for(long r=0;r<rows_;++r){
		for(long c=0;c<cols_;++c){
			sum += x[r][c] * x[r][c];}}
	return sum;
	}//end  aMatrix<T>::sesq 
	



template<class T > double aMatrix<T>::SignProc (double a, double b){
	if (b >= 0.0){return ABS(a);}
		else{return -ABS(a);}
	}//end aMatrix<T>::SignProc


template<class T > long double aMatrix<T>::SignProc (long double a, long double b){
	if (b >= (long double)0.0){return ABS(a);}
		else{return -ABS(a);}
	}//end aMatrix<T>::SignProc


template<class T > aVector<long double>* aMatrix<T>::SolutionOfLinearEquations (aMatrix<T>& coefficients, aVector<T>& rightHandSide){
	//debug first;  returns result in a new vector
	//gives back a new vector of the solutions without destroying CoefficientMatrix or rightHandSide vector
	int sign=0;
	aVector<long double>* solutionVector = NEW aVector<long double>(rightHandSide);
	if(solutionVector==NULL){
		aString err("Failed 'new' in aMatrix<T>::SolutionOfLinearEquations(...)");BadOperation(err);}
	aMatrix<T>* coeffMatrixCopy = NEW aMatrix<T>(coefficients);//operate on the copy
	if(coeffMatrixCopy==NULL){
		aString err("Failed 'new' in aMatrix<T>::SolutionOfLinearEquations(...)");BadOperation(err);}
	aVector<long> rowArrangement(rightHandSide.len());
	coeffMatrixCopy->LUDecomposition(rowArrangement, sign);
	coeffMatrixCopy->LUBackSubstitution(*solutionVector, rowArrangement, sign);
	//toss temporary matrix
	delete coeffMatrixCopy; coeffMatrixCopy=NULL;
	return solutionVector;
	}//end aMatrix<T>::SolutionOfLinearEquations


template<class T > aMatrix<T>* aMatrix<T>::submatrix(long rStart, long cStart, long rEnd, long cEnd){
	//returns new submatrix
	aMatrix<T>* m=NULL;
	if((rStart<0)||(rEnd>=rows_)||(cStart<0)||(cEnd>=cols_)){
		aString e("ERROR: can't create a submatrix of the given dimensions.");BadOperation(e);
		return m;}
	aMatrix<T>& x = *this;
	m = NEW aMatrix<T>(T(0.0), rEnd-rStart+1, cEnd-cStart+1);
	for(long r=0;r<rEnd-rStart+1;++r){
		for(long c=0;c<cEnd-cStart+1;++c){
			(*m)[r][c] = x[rStart+r][cStart+c];}}
	return m;
	}//end aMatrix<T>::submatrix ()



template<class T > aMatrix<T>* aMatrix<T>::Subtract (aMatrix<T>& m){
	//subtracts m from this and returns result in a new matrix
	//can return a null pointer if matrices not matched in size
	if(!CanBeAdded(m)){
		aString a="Error:  can't subtract matrices of different sizes!";
		BadOperation(a);
		return NULL;}
	aMatrix<T>* diffMatrix = NEW aMatrix<T>(this) ; 
	if(diffMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::Subtract(aMatrix<T>&)");BadOperation(err);}
	for (long r=0;r<rows_;++r){
		for (long c=0;c<cols_;++c){
			(*diffMatrix)[r][c] -= m[r][c];}}
	return diffMatrix;
	}//end aMatrix<T>::Subtract (aMatrix<T>&)



template<class T > aVector<long double>*  aMatrix<T>::SVBackSubstitution (matrixHandle<long double>& VmatrixH, 
														aVector<long double>&  b, aVector<long double>& w){
	//debug first;  returns result in a new vector
	//see notes in SVDecomposition; b is the vector of right-hand side values, and it get changed
		//into the vector of solutions.  w, this & Vmatrix are unchanged.
	//'this', the calling matrix, is Umatrix in Pascal versions
	unsigned long m = rows();
	unsigned long n = cols();
	aVector<long double> temp(n);
	aVector<long double>* x = NEW aVector<long double>(n);
	if(x==NULL){
		aString err("Failed 'new' in aMatrix<T>::SVBackSubstitution(...)");BadOperation(err);}
	aMatrix<long double>* Vmatrix = VmatrixH.mat;
	for (long j=0;j<n;++j){
		long double s = (long double)0.0;
		long double wj = w[j];
		if (wj != (long double)0.0){
			for (long i=0;i<m;++i){
				s += data[i][j] * b[i];}
			s /= wj;
			}//end if w[j] != 0
		temp[j]=s; 
		}//end for j
	for (long j=0;j<n;++j){
		long double s = (long double)0.0;
		for (long jj=0;jj<n;++jj){
			s += (*Vmatrix)[j][jj] * temp[jj];}
		(*x)[j]=s;
		}//end for j
	Vmatrix=NULL;//forget; returns with handle
	return x;
	}//end aMatrix<T>::SVBackSubstitution

/* moved to aMatrix2.cpp
template<class T > aMatrix<double>* aMatrix<T>::SVDecomposition (matrixHandle<double>& VmatrixH, aVector<double>& w){
	//singularValue decomposition of a matrix returned as Umatrix & Vmatrix
	//Vmatrix is entered as 0 and created here; Umatrix should be a copy of the calling matrix, since
	//it is changed here.  The vector w is cleared and recreated here too.
	
	//The entire Press et al. routine is 1-indexed, and subtraction is used here in each array element to achieve 0-indexing
	aMatrix<double>* Umatrix = ToDouble();
	unsigned long n = cols();
	unsigned long m = rows();
	aVector<double> rv1(n);
	w.Initialize(0.0);
	aMatrix<double>* Vmatrix = VmatrixH.mat;
	VmatrixH.Detach();
	delete Vmatrix; Vmatrix=NULL;
	Vmatrix = NEW aMatrix<double>(n, n);
	if(Vmatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::SVDecomposition(...)");BadOperation(err);}
	aMatrix<double>& v = *Vmatrix;
	aMatrix<double>& a = *Umatrix;
	double g = 0.0, scale = 0.0, anorm = 0.0, s = 0.0, val = 0.0, scaleg = 0.0;
	double c = 0.0, f = 0.0, h = 0.0, x = 0.0, y = 0.0, z = 0.0;
	long L=0, i, j, k, nm;
	
	//householder reduction to bidiagonal form
	for (i=0;i<n;i++){
		L = i + 1;
		scaleg = scale*g;  if(scaleg==(-0.0)){scaleg = 0.0;}
		rv1[i] = scaleg;
		g = s = scale = 0.0;
		if (i <= m){//0-indexed
			for( k=i;k<m;++k){
				scale += abs(a[ k][i]);}
			if (scale != 0.0){
				for( k=i;k<m;++k){
					val = a[k][i] / scale; if(val==(-0.0)){val = 0.0;}
					a[k][i]=val;
					s += val * val;
					}//end for k
				f = a[i][i];
				g = - SignProc(sqrt(s), f); if(g==(-0.0)){g = 0.0;}
				h = f * g - s; if(h==(-0.0)){h = 0.0;}
				a[i][i] = f - g;
				for (j=L;j<n;++j){//L not 1
					s = 0.0;
					for( k=i;k<m;++k){
						s +=a[k][i] *  a[k][j]; if(s==(-0.0)){s = 0.0;}
						}
					f = s / h; if(f==(-0.0)){f = 0.0;}
					for( k=i;k<m;++k){
						val = a[k][j] + f *  a[k][i]; if(val==(-0.0)){val = 0.0;}
						a[k][j]=val;
						}//end for k
					}//end for j
				for( k=i;k<m;++k){
					a[k][i] *= scale; if(a[k][i]==(-0.0)){a[k][i] = 0.0;}
					}
				}//end if scale != 0
			}//end if i< m
		w[i] = scale * g; if(w[i]==(-0.0)){w[i] = 0.0;}
		g = s = scale = 0.0;
		if ((i < m) && (i != n-1)) {// 0-indexed
			for( k=L;k<n;++k){
				scale += abs(a[i][k]);}
			if (scale != 0.0) {
				for(k=L;k<n;++k){
					val = a[i][k]/scale; if(val==(-0.0)){val = 0.0;}
					a[i][k]=val;
					s += val * val;
					}//end for k
				f =  a[i][L];
				g = -SignProc(sqrt(s), f); if(g==(-0.0)){g = 0.0;}
				h = f * g - s; if(h==(-0.0)){h = 0.0;}
				a[i][L] = f - g;
				for(k=L;k<n;++k){
					rv1[k]= a[i][k]  / h; if(rv1[k]==(-0.0)){rv1[k] = 0.0;}
					}
				for( j=L;j<m;++j){
					s = 0.0;
					for(k=L;k<n;++k){
						s += a[j][k] *  a[i][k]; if(s==(-0.0)){s = 0.0;}
						}
					for(k=L;k<n;++k){
						 a[j][k]  += s * (rv1[k]); if( a[j][k]==(-0.0)){ a[j][k] = 0.0;}
						 }
					}//end for j
				for(k=L;k<n;++k){
					 a[i][k]  *= scale;if( a[i][k] ==(-0.0)){ a[i][k]  = 0.0;}
					 }
				}//end if scale != 0
			}//end if i<m && i != n-1
		val = abs(w[i]) + abs(rv1[i]);
		if (val > anorm){
			anorm = val;}
		}//end for i

	//accumulation of right-hand transforms
	for (i = n-1;i>=0;--i){
		if (i < n-1){
			if (g != 0.0){
				for ( j=L;j<n;++j){
					v[j][i]= (  a[i][j] / a[i][L]  ) / g;  if(v[j][i]==(-0.0)){v[j][i] = 0.0;}
					}
				for ( j=L;j<n;++j){
					s = 0.0;
					for ( k=L;k<n;++k){
						s += a[i][k] *  v[k][j];  if(s==(-0.0)){s = 0.0;}
						}
					for ( k=L;k<n;++k){
						v[k][j] += s * v[k][i];  if(v[k][j]==(-0.0)){v[k][j] = 0.0;}
						}
					}//end for j
				}//end if g<>0
			for ( j=L;j<n;++j){
				v[i][j] = 0.0;
				v[j][i] = 0.0;
				}//end for j
			}//end if i<n-1
		v[i][i] = 1.0;
		g = rv1[i];
		L = i;
		}//end for i downto

	//accumulation of left-hand transforms
	long mnMin = n;
	if (m < n){
		mnMin = m;}
	for (i = mnMin-1;i>=0;--i){
		L = i + 1;
		g = w[i];
		for ( j = L;j<n;++j){
			a[i][j]=0.0;}
		if (g != 0.0){
				g = 1.0 / g;
				for ( j = L;j<n;++j){
					s = 0.0;
					for ( k = L;k<m;++k){
						s +=a[k][i] *a[k][j];  if(s==(-0.0)){s = 0.0;}
						}
					f = (s /a[i][i] ) * g;  if(f==(-0.0)){f = 0.0;}
					for ( k = i;k<m;++k){
						a[k][j] +=  f * a[k][i];  if(a[k][j]==(-0.0)){a[k][j] = 0.0;}
						}
					}//end for j
				for ( j = i;j<m;++j){
					a[j][i] *= g;  if(a[j][i]==(-0.0)){a[j][i] = 0.0;}
					}
				}//end
			else{
				for ( j = i;j<m;++j){
					a[j][i] = 0.0;
					}	
				}//end if g<>0 ... else ...}
		a[i][i]++;
		}//end for i downto

	//diagonalization of the bidiagonal form
	for ( k = n-1;k>=0;--k){  //loop over singular values
		for(int its=0;its<30;++its){	//loop over allowed iterations
			int flag = 1;
			for(L=k;L>=0;--L){	//test for splitting;             				use k not k-1
				nm = L - 1;
				if (abs(rv1[L]) + anorm == anorm){
					flag = 0;
					break;
					}//end if anorm
				if (abs(w[nm]) + anorm == anorm){
					break;}
				}//end for L 
			if (flag != 0){		//LABEL 1 in Pascal code;  cancellation of rv1[L], if L>1
				c = 0.0;
				s = 1.0;
				for (i=L; i<=k;++i){									//use  <= not <
					f = s* rv1[i];  if(f==(-0.0)){f = 0.0;}
					rv1[i] *= c;  if(rv1[i]==(-0.0)){rv1[i] = 0.0;}
					if (abs(f) + anorm == anorm){
						break;}
					g = w[i];
					h = Pythag(f, g);  if(h==(-0.0)){h = 0.0;}
					w[i] = h;
					h = 1.0 / h;
					c = g * h;  if(c==(-0.0)){c = 0.0;}
					s = -f * h;  if(s==(-0.0)){s = 0.0;}
					for ( j=0;j<m;++j){
						y = a[j][nm];
						z = a[j][i];
						a[j][nm] = (y * c) + (z * s);  if(a[j][nm]==(-0.0)){a[j][nm] = 0.0;}
						a[j][i] = (z * c) - (y * s);  if( a[j][i]==(-0.0)){ a[j][i] = 0.0;}
						}//end for j
					}//end for i
				}//end if flag<>0
			z = w[k];
			if (L == k){//convergence
					if (z < 0.0){//singular value is made non-negative
						w[k] = -z;
						for ( j=0;j<n;++j){
							v[j][k] *= (-1.0);  if( v[j][k]==(-0.0)){ v[j][k] = 0.0;}
							 }
						}//end if z<0
					break;
					}//end
				else if (its == 29){ 
					cout<<"no convergence after 30 iterations of SVDecomposition"<<endl;
					break;}
			x = w[L];		//shift from bottom 2-by-2 minor:
			nm = k - 1;
			y = w[nm];
			g = rv1[nm];
			h = rv1[k];
			f = ((y - z) * (y + z) + (g - h) * (g + h)) / (2.0 * h * y);  if(f==(-0.0)){f = 0.0;}
			g = Pythag(f, 1.0);  if(g==(-0.0)){g = 0.0;}
			f = ((x - z) * (x + z) + h * ((y / (f + SignProc(g, f))) - h)) / x;  if(f==(-0.0)){f = 0.0;}

			c = s = 1.0;	//next QR transformation:    
			for ( j=L;j<=nm;++j){//<?
				i = j + 1;
				g = rv1[i];
				y = w[i];
				h = s * g;  if(h==(-0.0)){h = 0.0;}
				g *= c;  if(g==(-0.0)){g = 0.0;}
				z = Pythag(f, h);  if(z==(-0.0)){z = 0.0;}
				rv1[j]=z;
				c = f / z;  if(c==(-0.0)){c = 0.0;}
				s = h / z;  if(s==(-0.0)){s = 0.0;}
				f = (x * c) + (g * s);  if(f==(-0.0)){f = 0.0;}
				g = (g * c) - (x * s);  if(g==(-0.0)){g = 0.0;}
				h = y * s;  if(h==(-0.0)){h = 0.0;}
				y *= c;  if(y==(-0.0)){y = 0.0;}
				for (long jj=0;jj<n;++jj){
					x = v[jj][j];
					z =  v[jj][i];
					v[jj][j] = (x * c) +( z * s);  if(v[jj][j]==(-0.0)){v[jj][j] = 0.0;}
					v[jj][i] = (z * c) - (x * s);  if(v[jj][i]==(-0.0)){v[jj][i] = 0.0;}
					}//end for jj
				z = Pythag(f, h);  if(z==(-0.0)){z = 0.0;}
				w[j] = z;
				if (z != 0.0){//rotation can be arbitraty if z==0.0
					z = 1.0 / z;
					c = f * z;  if(c==(-0.0)){c = 0.0;}
					s = h * z;  if(s==(-0.0)){s = 0.0;}
					}//end if z != 0.0
				f = (c * g) + (s * y);  if(f==(-0.0)){f = 0.0;}
				x = (c * y) - (s * g);  if(x==(-0.0)){x = 0.0;}
				for (long  jj=0;jj<m;++jj){
					y = a[jj][j];
					z = a[jj][i];
					a[jj][j] = (y * c) + (z * s);  if(a[jj][j]==(-0.0)){a[jj][j] = 0.0;}
					a[jj][i] = (z * c) - (y * s);  if(a[jj][i]==(-0.0)){a[jj][i] = 0.0;}
					}//end for jj
				}//end for j
			 rv1[L] = 0.0;
			 rv1[k] =f;
			 w[k] = x;
			}//end for its 
		}//end;	{end for k downto}
	VmatrixH.Bind(Vmatrix);
	return Umatrix;
	}//end aMatrix<T>::SVDecomposition

*/


template<class T > aMatrix<long double>* aMatrix<T>::SVDecomposition (matrixHandle<long double>& VmatrixH, aVector<long double>& w){
	//singularValue decomposition of a matrix returned as Umatrix & Vmatrix
	//Vmatrix is entered as 0 and created here; Umatrix should be a copy of the calling matrix, since
	//it is changed here.  The vector w is cleared and recreated here too.
	
	//The entire Press et al. routine is 1-indexed, and subtraction is used here in each array element to achieve 0-indexing
	aMatrix<long double>* Umatrix = ToLongDouble();
	unsigned long n = cols();
	unsigned long m = rows();
	aVector<long double> rv1(n);
	w.Initialize((long double)0.0);
	aMatrix<long double>* Vmatrix = VmatrixH.mat;
	VmatrixH.Detach();
	delete Vmatrix; Vmatrix=NULL;
	Vmatrix = NEW aMatrix<long double>((long double)0.0,n, n);
	if(Vmatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::SVDecomposition(...)");BadOperation(err);}
	aMatrix<long double>& v = *Vmatrix;
	aMatrix<long double>& a = *Umatrix;
	long double g = (long double)0.0, scale = (long double)0.0;
	long double anorm = (long double)0.0, s = (long double)0.0, val = (long double)0.0, scaleg = (long double)0.0;
	long double c = (long double)0.0, f = (long double)0.0, h = (long double)0.0, x = 0.0, y = (long double)0.0, z = (long double)0.0;
	long L=0, i, j, k, nm;
	
	//householder reduction to bidiagonal form
	for (i=0;i<n;i++){
		L = i + 1;
		scaleg = scale*g;  if(scaleg==((long double)-0.0)){scaleg = (long double)0.0;}
		rv1[i] = scaleg;
		g = s = scale = (long double)0.0;
		if (i <= m){//0-indexed
			for( k=i;k<m;++k){
				scale += ABS(a[ k][i]);}
			if (scale != (long double)0.0){
				for( k=i;k<m;++k){
					val = a[k][i] / scale; if(val==((long double)-0.0)){val = (long double)0.0;}
					a[k][i]=val;
					s += val * val;
					}//end for k
				f = a[i][i];
				g = - SignProc(sqrt(s), f); if(g==((long double)-0.0)){g = (long double)0.0;}
				h = f * g - s; if(h==((long double)-0.0)){h = (long double)0.0;}
				a[i][i] = f - g;
				for (j=L;j<n;++j){//L not 1
					s = (long double)0.0;
					for( k=i;k<m;++k){
						s +=a[k][i] *  a[k][j]; if(s==((long double)-0.0)){s = (long double)0.0;}
						}
					f = s / h; if(f==((long double)-0.0)){f = (long double)0.0;}
					for( k=i;k<m;++k){
						val = a[k][j] + f *  a[k][i]; if(val==(long double)(-0.0)){val = (long double)0.0;}
						a[k][j]=val;
						}//end for k
					}//end for j
				for( k=i;k<m;++k){
					a[k][i] *= scale; if(a[k][i]==((long double)-0.0)){a[k][i] = (long double)0.0;}
					}
				}//end if scale != 0
			}//end if i< m
		w[i] = scale * g; if(w[i]==((long double)-0.0)){w[i] = (long double)0.0;}
		g = s = scale = (long double)0.0;
		if ((i < m) && (i != n-1)) {// 0-indexed
			for( k=L;k<n;++k){
				scale += ABS(a[i][k]);}
			if (scale != (long double)0.0) {
				for(k=L;k<n;++k){
					val = a[i][k]/scale; if(val==(long double)(-0.0)){val = (long double)0.0;}
					a[i][k]=val;
					s += val * val;
					}//end for k
				f =  a[i][L];
				g = -SignProc(sqrt(s), f); if(g==(long double)(-0.0)){g = (long double)0.0;}
				h = f * g - s; if(h==(long double)(-0.0)){h = (long double)0.0;}
				a[i][L] = f - g;
				for(k=L;k<n;++k){
					rv1[k]= a[i][k]  / h; if(rv1[k]==(long double)(-0.0)){rv1[k] = (long double)0.0;}
					}
				for( j=L;j<m;++j){
					s = (long double)0.0;
					for(k=L;k<n;++k){
						s += a[j][k] *  a[i][k]; if(s==((long double)-0.0)){s = (long double)0.0;}
						}
					for(k=L;k<n;++k){
						 a[j][k]  += s * (rv1[k]); if( a[j][k]==(long double)(-0.0)){ a[j][k] = (long double)0.0;}
						 }
					}//end for j
				for(k=L;k<n;++k){
					 a[i][k]  *= scale;if( a[i][k] ==(long double)(-0.0)){ a[i][k]  = (long double)0.0;}
					 }
				}//end if scale != 0
			}//end if i<m && i != n-1
		val = ABS(w[i]) + ABS(rv1[i]);
		if (val > anorm){
			anorm = val;}
		}//end for i

	//accumulation of right-hand transforms
	for (i = n-1;i>=0;--i){
		if (i < n-1){
			if (g != (long double)0.0){
				for ( j=L;j<n;++j){
					v[j][i]= (  a[i][j] / a[i][L]  ) / g;  if(v[j][i]==(long double)(-0.0)){v[j][i] = (long double)0.0;}
					}
				for ( j=L;j<n;++j){
					s = (long double)0.0;
					for ( k=L;k<n;++k){
						s += a[i][k] *  v[k][j];  if(s==(long double)(-0.0)){s = (long double)0.0;}
						}
					for ( k=L;k<n;++k){
						v[k][j] += s * v[k][i];  if(v[k][j]==(long double)(-0.0)){v[k][j] = (long double)0.0;}
						}
					}//end for j
				}//end if g<>0
			for ( j=L;j<n;++j){
				v[i][j] = (long double)0.0;
				v[j][i] = (long double)0.0;
				}//end for j
			}//end if i<n-1
		v[i][i] = (long double)1.0;
		g = rv1[i];
		L = i;
		}//end for i downto

	//accumulation of left-hand transforms
	long mnMin = n;
	if (m < n){
		mnMin = m;}
	for (i = mnMin-1;i>=0;--i){
		L = i + 1;
		g = w[i];
		for ( j = L;j<n;++j){
			a[i][j]=(long double)0.0;}
		if (g != (long double)0.0){
				g = (long double)1.0 / g;
				for ( j = L;j<n;++j){
					s = (long double)0.0;
					for ( k = L;k<m;++k){
						s +=a[k][i] *a[k][j];  if(s==(long double)(-0.0)){s = (long double)0.0;}
						}
					f = (s /a[i][i] ) * g;  if(f==(long double)(-0.0)){f = (long double)0.0;}
					for ( k = i;k<m;++k){
						a[k][j] +=  f * a[k][i];  if(a[k][j]==(long double)(-0.0)){a[k][j] = (long double)0.0;}
						}
					}//end for j
				for ( j = i;j<m;++j){
					a[j][i] *= g;  if(a[j][i]==(long double)(-0.0)){a[j][i] = (long double)0.0;}
					}
				}//end
			else{
				for ( j = i;j<m;++j){
					a[j][i] = (long double)0.0;
					}	
				}//end if g<>0 ... else ...}
		a[i][i]++;
		}//end for i downto

	//diagonalization of the bidiagonal form
	for ( k = n-1;k>=0;--k){  //loop over singular values
		for(int its=0;its<30;++its){	//loop over allowed iterations
			int flag = 1;
			for(L=k;L>=0;--L){	//test for splitting;             				use k not k-1
				nm = L - 1;
				if (ABS(rv1[L]) + anorm == anorm){
					flag = 0;
					break;
					}//end if anorm
				if (ABS(w[nm]) + anorm == anorm){
					break;}
				}//end for L 
			if (flag != 0){		//LABEL 1 in Pascal code;  cancellation of rv1[L], if L>1
				c = (long double)0.0;
				s = (long double)1.0;
				for (i=L; i<=k;++i){									//use  <= not <
					f = s* rv1[i];  if(f==(long double)(-0.0)){f = (long double)0.0;}
					rv1[i] *= c;  if(rv1[i]==(long double)(-0.0)){rv1[i] = (long double)0.0;}
					if (ABS(f) + anorm == anorm){
						break;}
					g = w[i];
					h = Pythag(f, g);  if(h==(long double)(-0.0)){h = (long double)0.0;}
					w[i] = h;
					h = (long double)1.0 / h;
					c = g * h;  if(c==(long double)(-0.0)){c = (long double)0.0;}
					s = -f * h;  if(s==(long double)(-0.0)){s = (long double)0.0;}
					for ( j=0;j<m;++j){
						y = a[j][nm];
						z = a[j][i];
						a[j][nm] = (y * c) + (z * s);  if(a[j][nm]==(long double)(-0.0)){a[j][nm] = (long double)0.0;}
						a[j][i] = (z * c) - (y * s);  if( a[j][i]==(long double)(-0.0)){ a[j][i] = (long double)0.0;}
						}//end for j
					}//end for i
				}//end if flag<>0
			z = w[k];
			if (L == k){//convergence
					if (z < (long double)0.0){//singular value is made non-negative
						w[k] = -z;
						for ( j=0;j<n;++j){
							v[j][k] *= (long double)(-1.0);  if( v[j][k]==(long double)(-0.0)){ v[j][k] = (long double)0.0;}
							 }
						}//end if z<0
					break;
					}//end
				else if (its == 29){ 
					cout<<"no convergence after 30 iterations of SVDecomposition"<<endl;
					break;}
			x = w[L];		//shift from bottom 2-by-2 minor:
			nm = k - 1;
			y = w[nm];
			g = rv1[nm];
			h = rv1[k];
			f = ((y - z) * (y + z) + (g - h) * (g + h)) / ((long double)2.0 * h * y);  if(f==(long double)(-0.0)){f =(long double) 0.0;}
			g = Pythag(f, (long double)1.0);  if(g==(long double)(-0.0)){g = (long double)0.0;}
			f = ((x - z) * (x + z) + h * ((y / (f + SignProc(g, f))) - h)) / x;  if(f==(long double)(-0.0)){f = (long double)0.0;}

			c = s = (long double)1.0;	//next QR transformation:    
			for ( j=L;j<=nm;++j){//<?
				i = j + 1;
				g = rv1[i];
				y = w[i];
				h = s * g;  if(h==(long double)(-0.0)){h = (long double)0.0;}
				g *= c;  if(g==(long double)(-0.0)){g = (long double)0.0;}
				z = Pythag(f, h);  if(z==(long double)(-0.0)){z = (long double)0.0;}
				rv1[j]=z;
				c = f / z;  if(c==(long double)(-0.0)){c = (long double)0.0;}
				s = h / z;  if(s==(long double)(-0.0)){s = (long double)0.0;}
				f = (x * c) + (g * s);  if(f==(long double)(-0.0)){f =(long double) 0.0;}
				g = (g * c) - (x * s);  if(g==(long double)(-0.0)){g = (long double)0.0;}
				h = y * s;  if(h==(long double)(-0.0)){h = (long double)0.0;}
				y *= c;  if(y==(long double)(-0.0)){y = (long double)0.0;}
				for (long jj=0;jj<n;++jj){
					x = v[jj][j];
					z =  v[jj][i];
					v[jj][j] = (x * c) +( z * s);  if(v[jj][j]==(long double)(-0.0)){v[jj][j] = (long double)0.0;}
					v[jj][i] = (z * c) - (x * s);  if(v[jj][i]==(long double)(-0.0)){v[jj][i] = (long double)0.0;}
					}//end for jj
				z = Pythag(f, h);  if(z==(long double)(-0.0)){z = (long double)0.0;}
				w[j] = z;
				if (z != (long double)0.0){//rotation can be arbitraty if z==0.0
					z = (long double)1.0 / z;
					c = f * z;  if(c==(long double)(-0.0)){c = (long double)0.0;}
					s = h * z;  if(s==(long double)(-0.0)){s = (long double)0.0;}
					}//end if z != 0.0
				f = (c * g) + (s * y);  if(f==(long double)(-0.0)){f =(long double)0.0;}
				x = (c * y) - (s * g);  if(x==(long double)(-0.0)){x =(long double) 0.0;}
				for (long  jj=0;jj<m;++jj){
					y = a[jj][j];
					z = a[jj][i];
					a[jj][j] = (y * c) + (z * s);  if(a[jj][j]==(long double)(-0.0)){a[jj][j] = (long double)0.0;}
					a[jj][i] = (z * c) - (y * s);  if(a[jj][i]==(long double)(-0.0)){a[jj][i] = (long double)0.0;}
					}//end for jj
				}//end for j
			 rv1[L] = (long double)0.0;
			 rv1[k] =f;
			 w[k] = x;
			}//end for its 
		}//end;	{end for k downto}
	VmatrixH.Bind(Vmatrix);
	return Umatrix;
	}//end aMatrix<T>::SVDecomposition



template<class T > aMatrix<long double>* aMatrix<T>::SVInverse (void){	
	//debug first;  returns result in a new matrix
	if (!Square()){
		aString a("Error:  can't presently find the inverse of a non-square matrix!");
		BadOperation(a);
		return NULL;}
	aMatrix<long double>* invMatrix = NEW aMatrix<long double>(rows(),cols());
	if(invMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::SVInverse()");BadOperation(err);}

	matrixHandle<long double> VmatrixH;//send Vmatrix as a null pointer
	aVector<long double> w((long double)0.0,cols());//sent as 0 vector
	aMatrix<long double>* Umatrix = SVDecomposition(VmatrixH, w);
	
	
	long double wMax = (long double)0.0;//zero the low w's
	for (long j=0;j<cols();++j){
		long double wj = w[j];
		if (wj > wMax){
			wMax = wj;}
		}//end for j
	long double wMin = wMax * (long double)1.0e-6;
	for (long j=0;j<cols();++j){
		long double wj = w[j];
		if (wj < wMin){
			w[j]=(long double)0.0;}
		}//end for j
	
	//a quick calculation of nullity is useful
	nullity_=0;
	for(long j=0;j<w.len();++j){
		if(w[j] == (long double)0.0){nullity_++;}}


	//now complete the inversion
	aVector<long double> colVec(rows());
	for (long c=0;c<cols_;++c){
		for (long cc=0;cc<cols_;++cc){
			colVec[cc]=(long double)0.0;}//reinitialize
		colVec[c]=(long double)1.0;
		aVector<long double>* solVec = Umatrix->SVBackSubstitution(VmatrixH, colVec, w);
		for (long r=0;r<invMatrix->rows();++r){
			(*invMatrix)[r][c] = (*solVec)[r];}
		delete solVec; solVec = NULL;
		}//end for c
	//toss temporary vectors
	delete Umatrix; Umatrix=NULL;
	invMatrix->nullity_=nullity_;
	return invMatrix;
	}//end aMatrix<T>::SVInverse


template<class T > void  aMatrix<T>::SwapCols(long c1, long c2){
	if (c1==c2){return;}
	for(long r=0;r<rows_;++r){
		T temp = (*this)[r][c1];
		(*this)[r][c1] = (*this)[r][c2];
		(*this)[r][c2] = temp;}
	}//end aMatrix<T>::SwapCols


template<class T > void  aMatrix<T>::SwapRows(long r1, long r2){
	if (r1==r2){return;}
	for(long c=0;c<cols_;++c){
		T temp = (*this)[r1][c];
		(*this)[r1][c] = (*this)[r2][c];
		(*this)[r2][c] = temp;}
	}//end aMatrix<T>::SwapRows



template<class T > int aMatrix<T>::symmetric(void){
	// 0 x 0 and 1 x 1 matrices are trivially symmetric
	if(rows() != cols()) return 0;
	for(long r=1;r<rows();++r){
		for(long c=0;c<r;++c){
			if(data[r][c] != data[c][r]) return 0;
			}}
	return 1;
	}//end amatrix<T>::symmetric()



template<class T > aMatrix<T>*  aMatrix<T>::Transpose (void){//returns result in a new matrix
	aMatrix<T>*  transMatrix =NEW aMatrix<T>(cols(), rows());
	if(transMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::Transpose()");BadOperation(err);return NULL;}
	for(long r=0;r<rows_;++r){
		for(long c=0;c<cols_;++c){
			(*transMatrix)[c][r] = data[r][c];}}
	transMatrix->nullity_=nullity_;
	return transMatrix;
	}//end  aMatrix<T>::Transpose



template<class T > aMatrix<double>* aMatrix<T>::ToDouble(void){
	//converts all values to type 'double' and returns result as a new matrix
	aMatrix<double>* newMatrix = NEW aMatrix<double>(rows_,cols_);
	if(newMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::ToDouble()");BadOperation(err);}
	for(long r=0;r<rows_;++r){
		for(long c=0;c<cols_;++c){
			(*newMatrix).data[r][c]=(double) data[r][c];}}
	return newMatrix;
	}//end  newMatrix<T>::ToDouble


template<class T > aMatrix<long double>* aMatrix<T>::ToLongDouble(void){
	//converts all values to type 'double' and returns result as a new matrix
	aMatrix<long double>* newMatrix = NEW aMatrix<long double>(rows_,cols_);
	if(newMatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::ToDouble()");BadOperation(err);}
	for(long r=0;r<rows_;++r){
		for(long c=0;c<cols_;++c){
			(*newMatrix).data[r][c]=(long double) data[r][c];}}
	return newMatrix;
	}//end  newMatrix<T>::ToLongDouble


template<class T > T aMatrix<T>::Trace(void){
	//returns the sum of the diagonal elements if a square matrix
	if(! Square()){
		aString err("ERROR:  Can't take the trace of a non-square matrix!");BadOperation(err);}
	T sum = (T) 0.0;
	for(long r=0;r<rows_;++r){
		sum += data[r][r];}
	return sum;
	}//end  newMatrix<T>::Trace




template<class T > int aMatrix<T>::tql2(aVector<long double>& diagonalElements,
										 aVector<long double>& imaginaryElements,
										 aMatrix<long double>& eigenVectorsInCols){
	//version incorporated from JAMA library, in jama_eig.h
	long n_ = diagonalElements.len();
	aVector<long double>&  d = diagonalElements;//shorthand
	aVector<long double>&  e = imaginaryElements;
	aMatrix<long double>&  V = eigenVectorsInCols;

   //  This is derived from the Algol procedures tql2, by
   //  Bowdler, Martin, Reinsch, and Wilkinson, Handbook for
   //  Auto. Comp., Vol.ii-Linear Algebra, and the corresponding
   //  Fortran subroutine in EISPACK.
   
      for (int i = 1; i < n_; i++) {
         e[i-1] = e[i];
      }
      e[n_-1] = (long double) 0.0;
   
      long double f = (long double) 0.0;
      long double tst1 = (long double) 0.0;
      long double eps = pow((long double) 2.0,(long double) -52.0);
      for (int l = 0; l < n_; l++) {

         // Find small subdiagonal element
   
         tst1 = MAX(tst1,ABS(d[l]) + ABS(e[l]));
         int m = l;

        // Original while-loop from Java code
         while (m < n_) {
            if (ABS(e[m]) <= eps*tst1) {
               break;
            }
            m++;
         }

   
         // If m == l, d[l] is an eigenvalue,
         // otherwise, iterate.
   
         if (m > l) {
            int iter = 0;
            do {
			   if(iter++ == 30){
					aError("Error: Too many iterations in aMatrix<T>::tqli() while solving for eigenvalues"); 
					return 1;}
   
               // Compute implicit shift
   
               long double g = d[l];
               long double p = (d[l+1] - g) / ((long double) 2.0 * e[l]);
               long double r = Pythag(p,(long double) 1.0);
               if (p < (long double) 0.0) {
                  r = -r;
               }
               d[l] = e[l] / (p + r);
               d[l+1] = e[l] * (p + r);
               long double dl1 = d[l+1];
               long double h = g - d[l];
               for (int i = l+2; i < n_; i++) {
                  d[i] -= h;
               }
               f = f + h;
   
               // Implicit QL transformation.
   
               p = d[m];
               long double c = (long double) 1.0;
               long double c2 = c;
               long double c3 = c;
               long double el1 = e[l+1];
               long double s = (long double) 0.0;
               long double s2 = (long double) 0.0;
               for (int i = m-1; i >= l; i--) {
                  c3 = c2;
                  c2 = c;
                  s2 = s;
                  g = c * e[i];
                  h = c * p;
                  r = Pythag(p,e[i]);
                  e[i+1] = s * r;
                  s = e[i] / r;
                  c = p / r;
                  p = c * d[i] - s * g;
                  d[i+1] = h + s * (c * g + s * d[i]);
   
                  // Accumulate transformation.
   
                  for (int k = 0; k < n_; k++) {
                     h = V[k][i+1];
                     V[k][i+1] = s * V[k][i] + c * h;
                     V[k][i] = c * V[k][i] - s * h;
                  }
               }
               p = -s * s2 * c3 * el1 * e[l] / dl1;
               e[l] = s * p;
               d[l] = c * p;
   
               // Check for convergence.
   
            } while (ABS(e[l]) > eps*tst1);
         }
         d[l] = d[l] + f;
         e[l] = 0.0;
      }
     
      // Sort eigenvalues and corresponding vectors.
   
      for (int i = 0; i < n_-1; i++) {
         int k = i;
         long double p = d[i];
         for (int j = i+1; j < n_; j++) {
            if (d[j] < p) {
               k = j;
               p = d[j];
            }
         }
         if (k != i) {
            d[k] = d[i];
            d[i] = p;
            for (int j = 0; j < n_; j++) {
               p = V[j][i];
               V[j][i] = V[j][k];
               V[j][k] = p;
            }
         }
      }
     return 0;
	}//aMatrix<T>::tql2()




/*  //old buggy version
template<class T > int aMatrix<T>::tqli(aVector<long double>& diagonalElements,
										 aVector<long double>& offdiagonalElements,
										 aMatrix<long double>& eigenVectorCols, int getEigenVectors){
	//called by EigenCalcs()
	//returns 0 if successful, error code otherwise
	//modified from 'Numerical Recipes in Pascal'
	int m,j,iter,i,k;
	long n_ = diagonalElements.len();
	long double s,r,p,g,f,dd,c,b;
	aVector<long double>&  d = diagonalElements;//shorthand
	aVector<long double>&  e = offdiagonalElements;
	aMatrix<long double>&  z = eigenVectorCols;
	for(i=1;i<n_;++i) {
		e[i-1] = e[i];}
	e[n_-1]=(long double)0.0;
	for(j=0;j<n_;++j){
		iter=0;
		do{
			for(m=j;m<n_-1;++m){
				dd = ABS(d[m]) + ABS(d[m+1]);
				if((long double)(ABS(e[m])+dd) == dd) break;
				}//m
			if(m != j){
				if(iter++ == 30){
					aError("Error: Too many iterations in aMatrix<T>::tqli() while solving for eigenvalues"); 
					return 1;}
				g = (d[j+1] - d[j])/((long double)2.0 * e[j]);
				r = Pythag(g, (long double)1.0);
				g = d[m] - d[j] + e[j]/(g + SignProc(r,g));
				s = c = (long double) 1.0;
				p = (long double) 0.0;
				for(i=m-1;i>=j;--i){
					f = s * e[i];
					b = c * e[i];
					e[i+1] = (r = Pythag(f,g));
					if(r == (long double)0.0){
						d[i+1] -= p;
						e[m] = (long double)0.0;
						break;}
					s = f/r;
					c = g/r;
					g = d[i+1] - p;
					r = (d[i] - g)* s + (long double)2.0 * c * b;
					d[i+1] = g + (p = s*r);
					g = c * r - b;
					if(getEigenVectors){
						for(k=0;k<n_;++k){
							f = z[k][i+1];
							z[k][i+1] = s * z[k][i] + c * f;
							z[k][i] = c * z[k][i] - s * f;
							}//k
						}//getEigenVectors
					}//i
				if(r == (long double)0.0 && i >=j) continue;
				d[j] -= p;
				e[j] = g;
				e[m] = (long double) 0.0;
				}//m != j
			}while(m != j);
		}//j
	return 0;
	}//aMatrix<T>::tqli()
*/


template<class T > int aMatrix<T>::	tred2(aVector<long double>& diagonalElements,
									 	  aVector<long double>& imaginaryElements, 
										  aMatrix<long double>& V){
	//version incorporated from JAMA library, in jama_eig.h
	long n_ = diagonalElements.len();
	aVector<long double>&  d = diagonalElements;//shorthand
	aVector<long double>&  e = imaginaryElements;

   //  This is derived from the Algol procedures tred2 by
   //  Bowdler, Martin, Reinsch, and Wilkinson, Handbook for
   //  Auto. Comp., Vol.ii-Linear Algebra, and the corresponding
   //  Fortran subroutine in EISPACK.

      for (int j = 0; j < n_; j++) {
         d[j] = V[n_-1][j];
      }

      // Householder reduction to tridiagonal form.
   
      for (int i = n_-1; i > 0; i--) {
   
         // Scale to avoid under/overflow.
   
         long double scale = (long double) 0.0;
         long double h = (long double) 0.0;
         for (int k = 0; k < i; k++) {
            scale += ABS(d[k]);
         }
         if (scale == (long double) 0.0) {
            e[i] = d[i-1];
            for (int j = 0; j < i; j++) {
               d[j] = V[i-1][j];
               V[i][j] = (long double) 0.0;
               V[j][i] = (long double) 0.0;
            }
         } else {
   
            // Generate Householder vector.
   
            for (int k = 0; k < i; k++) {
               d[k] /= scale;
               h += d[k] * d[k];
            }
            long double f = d[i-1];
            long double g = sqrt(h);
            if (f > (long double) 0.0) {
               g = -g;
            }
            e[i] = scale * g;
            h = h - f * g;
            d[i-1] = f - g;
            for (int j = 0; j < i; j++) {
               e[j] = (long double) 0.0;
            }
   
            // Apply similarity transformation to remaining columns.
   
            for (int j = 0; j < i; j++) {
               f = d[j];
               V[j][i] = f;
               g = e[j] + V[j][j] * f;
               for (int k = j+1; k <= i-1; k++) {
                  g += V[k][j] * d[k];
                  e[k] += V[k][j] * f;
               }
               e[j] = g;
            }
            f = (long double) 0.0;
            for (int j = 0; j < i; j++) {
               e[j] /= h;
               f += e[j] * d[j];
            }
            long double hh = f / (h + h);
            for (int j = 0; j < i; j++) {
               e[j] -= hh * d[j];
            }
            for (int j = 0; j < i; j++) {
               f = d[j];
               g = e[j];
               for (int k = j; k <= i-1; k++) {
                  V[k][j] -= (f * e[k] + g * d[k]);
               }
               d[j] = V[i-1][j];
               V[i][j] = (long double) 0.0;
            }
         }
         d[i] = h;
      }
   
      // Accumulate transformations.
   
      for (int i = 0; i < n_-1; i++) {
         V[n_-1][i] = V[i][i];
         V[i][i] = (long double) 1.0;
         long double h = d[i+1];
         if (h != (long double) 0.0) {
            for (int k = 0; k <= i; k++) {
               d[k] = V[k][i+1] / h;
            }
            for (int j = 0; j <= i; j++) {
               long double g = (long double) 0.0;
               for (int k = 0; k <= i; k++) {
                  g += V[k][i+1] * V[k][j];
               }
               for (int k = 0; k <= i; k++) {
                  V[k][j] -= g * d[k];
               }
            }
         }
         for (int k = 0; k <= i; k++) {
            V[k][i+1] = (long double) 0.0;
         }
      }
      for (int j = 0; j < n_; j++) {
         d[j] = V[n_-1][j];
         V[n_-1][j] = (long double) 0.0;
      }
      V[n_-1][n_-1] = (long double) 1.0;
      e[0] = (long double) 0.0;
   return 0;
	}//aMatrix<T>::tred2()



/*  //old buggy version
template<class T > int aMatrix<T>::	tred2(aVector<long double>& diagonalElements,
									 	  aVector<long double>& offdiagonalElements, 
										  aMatrix<long double>& a, int getEigenVectors){
	//called by EigenCalcs()
	//returns 0 if successful, error code otherwise
	//modified from 'Numerical Recipes in C'
	int i,j,k,m;
	long n_ = diagonalElements.len();
	aVector<long double>&  d = diagonalElements;//shorthand
	aVector<long double>&  e = offdiagonalElements;
	long double scale,hh,h,g,f;
	for(i = n_-1;i>=1;--i){
		m = i-1;
		h = scale = (long double)0.0;
		if(m>0){
				for(k=0;k<=m;++k){// < or <= ?  -->  used < in pascal code; <= in Java code 
					scale += ABS(a[i][k]);}
				if(scale == (long double)0.0){
						e[i] = a[i][m];}
					else{
						for(k=0;k<=m;++k){// < or <= ?  -->  used < in pascal code
							a[i][k] /= scale;
							h += a[i][k] * a[i][k];
							}//k
						f = a[i][m];
						g = (f >= (long double)0.0 ? -sqrt(h) : sqrt(h) );
						e[i] = scale * g;
						h -= f*g;
						a[i][m] = f - g;
						f = (long double) 0.0;
						for( j=0;j<=m;++j){// < or <= ?  -->  used < in pascal code
							if(getEigenVectors) a[j][i] = a[i][j]/h;
							g = (long double) 0.0;
							for(k=0;k<j;++k){
								g += a[j][k] * a[i][k];}
							for(k=j+1;k<=m;++k){
								g += a[k][j] * a[i][k];}
							e[j] = g/h;
							f += e[j] * a[i][j];
							}//j
						hh = f/(h+h);
						for(j=0;j<=m;++j){
							f = a[i][j];
							e[j] = g = e[j] - hh*f;
							for(k=0;k<j;++k){
								a[j][k] -= (f * e[k] + g * a[i][k]);}
							}//j
						}//else (scale != 0.0)
					}
			else{// m==0
				e[i] = a[i][m];}
		d[i] = h;
		}//i
	if(getEigenVectors) {
		d[0] = (long double)0.0;}
	e[0] = (long double)0.0;
	for(i=0;i<n_;++i){
		if(getEigenVectors){
			m = i-1;
			if(d[i] > (long double)0.0){
				for(j=0;j<m;++j){
					g = (long double)0.0;
					for(k=0;k<m;++k){
						g += a[i][k] * a[k][j];}
					for(k=0;k<m;++k){
						a[k][j] -= g * a[k][i];}
					}//j
				}//if d[i]
			}//if getEigenVectors
		d[i] = a[i][i];
		if(getEigenVectors){
			a[i][i] = (long double)1.0;
			for(j=0;j<m;++j){
				a[j][i] = a[i][j] = (long double)0.0;}
			}//if getEigenVectors
		}//i
	return 0;
	}//aMatrix<T>::tred2()
*/

template <class T > 
	int operator== (aMatrix< T >& matrix1, aMatrix< T >& matrix2){
		if ((matrix1.rows() != matrix2.rows()) || ((matrix1.cols() != matrix2.cols()))) {return 0;}//false
		for(long r=0;r<matrix1.rows();++r){
			for(long c=0;c<matrix1.cols();++c){
				if(matrix1[r][c] != matrix2[r][c]){return 0;}}}//false
		return 1;//true
	}//end operator == (aMatrix<T>&,aMatrix<T>&)


template <class T > int operator!= (aMatrix< T >& matrix1, aMatrix< T >& matrix2){
		return !(matrix1 == matrix2);}













#endif



