/*********************
*
*	aMatrix2.cpp
*

		*****  MOVED TO aMatrix.cpp   ******
*
**********************/

#ifndef aMatrix2cp
#define aMatrix2cp = 1

#include "aMatrix.h"




template<class T > aMatrix<long double>* aMatrix<T>::SVDecomposition (matrixHandle<long double>& VmatrixH, aVector<long double>& w){
	//singularValue decomposition of a matrix returned as Umatrix & Vmatrix
	//Vmatrix is entered as 0 and created here; Umatrix should be a copy of the calling matrix, since
	//it is changed here.  The vector w is cleared and recreated here too.
	
	//The entire Press et al. routine is 1-indexed, and subtraction is used here in each array element to achieve 0-indexing
	aMatrix<long double>* Umatrix = ToLongDouble();
	unsigned long n = cols();
	unsigned long m = rows();
	aVector<long double> rv1(n);
	w.Initialize((long double)0.0);
	aMatrix<long double>* Vmatrix = VmatrixH.mat;
	VmatrixH.Detach();
	delete Vmatrix; Vmatrix=NULL;
	Vmatrix = NEW aMatrix<long double>((long double)0.0,n, n);
	if(Vmatrix==NULL){
		aString err("Failed 'new' in aMatrix<T>::SVDecomposition(...)");BadOperation(err);}
	aMatrix<long double>& v = *Vmatrix;
	aMatrix<long double>& a = *Umatrix;
	long double g = (long double)0.0, scale = (long double)0.0;
	long double anorm = (long double)0.0, s = (long double)0.0, val = (long double)0.0, scaleg = (long double)0.0;
	long double c = (long double)0.0, f = (long double)0.0, h = (long double)0.0, x = 0.0, y = (long double)0.0, z = (long double)0.0;
	long L=0, i, j, k, nm;
	
	//householder reduction to bidiagonal form
	for (i=0;i<n;i++){
		L = i + 1;
		scaleg = scale*g;  if(scaleg==((long double)-0.0)){scaleg = (long double)0.0;}
		rv1[i] = scaleg;
		g = s = scale = (long double)0.0;
		if (i <= m){//0-indexed
			for( k=i;k<m;++k){
				scale += ABS(a[ k][i]);}
			if (scale != (long double)0.0){
				for( k=i;k<m;++k){
					val = a[k][i] / scale; if(val==((long double)-0.0)){val = (long double)0.0;}
					a[k][i]=val;
					s += val * val;
					}//end for k
				f = a[i][i];
				g = - SignProc(sqrt(s), f); if(g==((long double)-0.0)){g = (long double)0.0;}
				h = f * g - s; if(h==((long double)-0.0)){h = (long double)0.0;}
				a[i][i] = f - g;
				for (j=L;j<n;++j){//L not 1
					s = (long double)0.0;
					for( k=i;k<m;++k){
						s +=a[k][i] *  a[k][j]; if(s==((long double)-0.0)){s = (long double)0.0;}
						}
					f = s / h; if(f==((long double)-0.0)){f = (long double)0.0;}
					for( k=i;k<m;++k){
						val = a[k][j] + f *  a[k][i]; if(val==(long double)(-0.0)){val = (long double)0.0;}
						a[k][j]=val;
						}//end for k
					}//end for j
				for( k=i;k<m;++k){
					a[k][i] *= scale; if(a[k][i]==((long double)-0.0)){a[k][i] = (long double)0.0;}
					}
				}//end if scale != 0
			}//end if i< m
		w[i] = scale * g; if(w[i]==((long double)-0.0)){w[i] = (long double)0.0;}
		g = s = scale = (long double)0.0;
		if ((i < m) && (i != n-1)) {// 0-indexed
			for( k=L;k<n;++k){
				scale += ABS(a[i][k]);}
			if (scale != (long double)0.0) {
				for(k=L;k<n;++k){
					val = a[i][k]/scale; if(val==(long double)(-0.0)){val = (long double)0.0;}
					a[i][k]=val;
					s += val * val;
					}//end for k
				f =  a[i][L];
				g = -SignProc(sqrt(s), f); if(g==(long double)(-0.0)){g = (long double)0.0;}
				h = f * g - s; if(h==(long double)(-0.0)){h = (long double)0.0;}
				a[i][L] = f - g;
				for(k=L;k<n;++k){
					rv1[k]= a[i][k]  / h; if(rv1[k]==(long double)(-0.0)){rv1[k] = (long double)0.0;}
					}
				for( j=L;j<m;++j){
					s = (long double)0.0;
					for(k=L;k<n;++k){
						s += a[j][k] *  a[i][k]; if(s==((long double)-0.0)){s = (long double)0.0;}
						}
					for(k=L;k<n;++k){
						 a[j][k]  += s * (rv1[k]); if( a[j][k]==(long double)(-0.0)){ a[j][k] = (long double)0.0;}
						 }
					}//end for j
				for(k=L;k<n;++k){
					 a[i][k]  *= scale;if( a[i][k] ==(long double)(-0.0)){ a[i][k]  = (long double)0.0;}
					 }
				}//end if scale != 0
			}//end if i<m && i != n-1
		val = ABS(w[i]) + ABS(rv1[i]);
		if (val > anorm){
			anorm = val;}
		}//end for i

	//accumulation of right-hand transforms
	for (i = n-1;i>=0;--i){
		if (i < n-1){
			if (g != (long double)0.0){
				for ( j=L;j<n;++j){
					v[j][i]= (  a[i][j] / a[i][L]  ) / g;  if(v[j][i]==(long double)(-0.0)){v[j][i] = (long double)0.0;}
					}
				for ( j=L;j<n;++j){
					s = (long double)0.0;
					for ( k=L;k<n;++k){
						s += a[i][k] *  v[k][j];  if(s==(long double)(-0.0)){s = (long double)0.0;}
						}
					for ( k=L;k<n;++k){
						v[k][j] += s * v[k][i];  if(v[k][j]==(long double)(-0.0)){v[k][j] = (long double)0.0;}
						}
					}//end for j
				}//end if g<>0
			for ( j=L;j<n;++j){
				v[i][j] = (long double)0.0;
				v[j][i] = (long double)0.0;
				}//end for j
			}//end if i<n-1
		v[i][i] = (long double)1.0;
		g = rv1[i];
		L = i;
		}//end for i downto

	//accumulation of left-hand transforms
	long mnMin = n;
	if (m < n){
		mnMin = m;}
	for (i = mnMin-1;i>=0;--i){
		L = i + 1;
		g = w[i];
		for ( j = L;j<n;++j){
			a[i][j]=(long double)0.0;}
		if (g != (long double)0.0){
				g = (long double)1.0 / g;
				for ( j = L;j<n;++j){
					s = (long double)0.0;
					for ( k = L;k<m;++k){
						s +=a[k][i] *a[k][j];  if(s==(long double)(-0.0)){s = (long double)0.0;}
						}
					f = (s /a[i][i] ) * g;  if(f==(long double)(-0.0)){f = (long double)0.0;}
					for ( k = i;k<m;++k){
						a[k][j] +=  f * a[k][i];  if(a[k][j]==(long double)(-0.0)){a[k][j] = (long double)0.0;}
						}
					}//end for j
				for ( j = i;j<m;++j){
					a[j][i] *= g;  if(a[j][i]==(long double)(-0.0)){a[j][i] = (long double)0.0;}
					}
				}//end
			else{
				for ( j = i;j<m;++j){
					a[j][i] = (long double)0.0;
					}	
				}//end if g<>0 ... else ...}
		a[i][i]++;
		}//end for i downto

	//diagonalization of the bidiagonal form
	for ( k = n-1;k>=0;--k){  //loop over singular values
		for(int its=0;its<30;++its){	//loop over allowed iterations
			int flag = 1;
			for(L=k;L>=0;--L){	//test for splitting;             				use k not k-1
				nm = L - 1;
				if (ABS(rv1[L]) + anorm == anorm){
					flag = 0;
					break;
					}//end if anorm
				if (ABS(w[nm]) + anorm == anorm){
					break;}
				}//end for L 
			if (flag != 0){		//LABEL 1 in Pascal code;  cancellation of rv1[L], if L>1
				c = (long double)0.0;
				s = (long double)1.0;
				for (i=L; i<=k;++i){									//use  <= not <
					f = s* rv1[i];  if(f==(long double)(-0.0)){f = (long double)0.0;}
					rv1[i] *= c;  if(rv1[i]==(long double)(-0.0)){rv1[i] = (long double)0.0;}
					if (ABS(f) + anorm == anorm){
						break;}
					g = w[i];
					h = Pythag(f, g);  if(h==(long double)(-0.0)){h = (long double)0.0;}
					w[i] = h;
					h = (long double)1.0 / h;
					c = g * h;  if(c==(long double)(-0.0)){c = (long double)0.0;}
					s = -f * h;  if(s==(long double)(-0.0)){s = (long double)0.0;}
					for ( j=0;j<m;++j){
						y = a[j][nm];
						z = a[j][i];
						a[j][nm] = (y * c) + (z * s);  if(a[j][nm]==(long double)(-0.0)){a[j][nm] = (long double)0.0;}
						a[j][i] = (z * c) - (y * s);  if( a[j][i]==(long double)(-0.0)){ a[j][i] = (long double)0.0;}
						}//end for j
					}//end for i
				}//end if flag<>0
			z = w[k];
			if (L == k){//convergence
					if (z < (long double)0.0){//singular value is made non-negative
						w[k] = -z;
						for ( j=0;j<n;++j){
							v[j][k] *= (long double)(-1.0);  if( v[j][k]==(long double)(-0.0)){ v[j][k] = (long double)0.0;}
							 }
						}//end if z<0
					break;
					}//end
				else if (its == 29){ 
					cout<<"no convergence after 30 iterations of SVDecomposition"<<endl;
					break;}
			x = w[L];		//shift from bottom 2-by-2 minor:
			nm = k - 1;
			y = w[nm];
			g = rv1[nm];
			h = rv1[k];
			f = ((y - z) * (y + z) + (g - h) * (g + h)) / ((long double)2.0 * h * y);  if(f==(long double)(-0.0)){f =(long double) 0.0;}
			g = Pythag(f, (long double)1.0);  if(g==(long double)(-0.0)){g = (long double)0.0;}
			f = ((x - z) * (x + z) + h * ((y / (f + SignProc(g, f))) - h)) / x;  if(f==(long double)(-0.0)){f = (long double)0.0;}

			c = s = (long double)1.0;	//next QR transformation:    
			for ( j=L;j<=nm;++j){//<?
				i = j + 1;
				g = rv1[i];
				y = w[i];
				h = s * g;  if(h==(long double)(-0.0)){h = (long double)0.0;}
				g *= c;  if(g==(long double)(-0.0)){g = (long double)0.0;}
				z = Pythag(f, h);  if(z==(long double)(-0.0)){z = (long double)0.0;}
				rv1[j]=z;
				c = f / z;  if(c==(long double)(-0.0)){c = (long double)0.0;}
				s = h / z;  if(s==(long double)(-0.0)){s = (long double)0.0;}
				f = (x * c) + (g * s);  if(f==(long double)(-0.0)){f =(long double) 0.0;}
				g = (g * c) - (x * s);  if(g==(long double)(-0.0)){g = (long double)0.0;}
				h = y * s;  if(h==(long double)(-0.0)){h = (long double)0.0;}
				y *= c;  if(y==(long double)(-0.0)){y = (long double)0.0;}
				for (long jj=0;jj<n;++jj){
					x = v[jj][j];
					z =  v[jj][i];
					v[jj][j] = (x * c) +( z * s);  if(v[jj][j]==(long double)(-0.0)){v[jj][j] = (long double)0.0;}
					v[jj][i] = (z * c) - (x * s);  if(v[jj][i]==(long double)(-0.0)){v[jj][i] = (long double)0.0;}
					}//end for jj
				z = Pythag(f, h);  if(z==(long double)(-0.0)){z = (long double)0.0;}
				w[j] = z;
				if (z != (long double)0.0){//rotation can be arbitraty if z==0.0
					z = (long double)1.0 / z;
					c = f * z;  if(c==(long double)(-0.0)){c = (long double)0.0;}
					s = h * z;  if(s==(long double)(-0.0)){s = (long double)0.0;}
					}//end if z != 0.0
				f = (c * g) + (s * y);  if(f==(long double)(-0.0)){f =(long double)0.0;}
				x = (c * y) - (s * g);  if(x==(long double)(-0.0)){x =(long double) 0.0;}
				for (long  jj=0;jj<m;++jj){
					y = a[jj][j];
					z = a[jj][i];
					a[jj][j] = (y * c) + (z * s);  if(a[jj][j]==(long double)(-0.0)){a[jj][j] = (long double)0.0;}
					a[jj][i] = (z * c) - (y * s);  if(a[jj][i]==(long double)(-0.0)){a[jj][i] = (long double)0.0;}
					}//end for jj
				}//end for j
			 rv1[L] = (long double)0.0;
			 rv1[k] =f;
			 w[k] = x;
			}//end for its 
		}//end;	{end for k downto}
	VmatrixH.Bind(Vmatrix);
	return Umatrix;
	}//end aMatrix<T>::SVDecomposition

#endif