/*******************************************
*
*
*	aVector.cpp
*
*	A class that implements vectors, and does vector algebra
*
*	Vectors can be converted from    aList<T>
*
*
************************************************/

#ifndef aVectorcp
#define aVectorcp = 1

#include "aVector.h"
#include "Element.cpp"
#include "aList.cpp"



template<class T >  aVector<T>::aVector(T x, unsigned long n): len_(n), data(NEW T[n]){
	if(data==NULL){
		aString err("Failed 'new' in aVector<T>::aVector(T x, n)");BadOperation(err);}
	for(long i=0;i<len_;++i){
		data[i]=x;}
	}


template<class T >  aVector<T>::aVector(unsigned long n): len_(n), data(NEW T[n]){
	if(data==NULL){
		aString err("Failed 'new' in aVector<T>::aVector(n)");BadOperation(err);}
	for(long i=0;i<len_;++i){
		data[i]=(T)0.0;}
	}


template<class T >  aVector<T>::aVector(void): len_(0), data(NULL){
	}


template<class T >  aVector<T>::aVector(aList<T>& x): len_(x.len()), data(NEW T[x.len()]){
	if(data==NULL){
		aString err("Failed 'new' in aVector<T>::aVector(aList<T>)");BadOperation(err);}
	for(long i=0;i<len_;++i){
		data[i]=x[i];}
	}


template<class T >  aVector<T>::aVector(aVector<T>& v):len_(v.len_), data(NEW T[v.len_]){
	if(data==NULL){
		aString err("Failed 'new' in aVector<T>::aVector(aVector<T>&)");BadOperation(err);}
	for(long i=0;i<len_;++i){
		data[i]=v.data[i];}
	}


template<class T >  aVector<T>::aVector(aVector<T>* v):len_(v->len_), data(NEW T[v->len_]){
	if(data==NULL){
		aString err("Failed 'new' in aVector<T>::aVector(aVector<T>*)");BadOperation(err);}
	for(long i=0;i<len_;++i){
		data[i]=v->data[i];}
	}


template<class T >  aVector<T>::~aVector(void){
	delete [] data; data=0;
	}//end aVector<T>:: ~aVector


template<class T >  void aVector<T>::BadOperation(aString& err/*, ostream& outFile*/){
			aError(err);
		//	outFile<<err<<endl;
			}



template<class T > aVector<T>* aVector<T>::Add(aVector<T>& v){
	//adds this to v and returns result as a new vector
	//can return a nil pointer if sizes mismatch}
	if(!Compatible(v)){
		aString a="Error:  can't add vectors of different sizes!";
		BadOperation(a);
		return NULL;}
	aVector<T>* sumVector = NEW aVector<T>(this);//copies this
	if(sumVector==NULL){
		aString err("Failed 'new' in aVector<T>::Add");BadOperation(err);}
	for(long i=0;i<len_;++i){
		(*sumVector)[i] += v[i];}
	return  sumVector;
	}//aVector<T>::Add(aVector<T>& v)



template<class T > void aVector<T>::Append (T val){
	//lengthens the current vector
	T* newData = NEW T [len_+1];
	if(newData==NULL){
		aString err("Failed 'new' in aVector<T>::Append(T)");BadOperation(err);}
	for(long i=0;i<len_;++i){ newData[i]=data[i];}
	newData[len_]=val;
	delete[] data;
	data=newData;
	++len_;
	newData=NULL;
	}//end aVector<T>::Append (T)


template<class T > void aVector<T>::Append (aVector<T>& v){
	//lengthens the current vector; v unchanged
	T* newData = NEW T [len_+v.len_];
	if(newData==NULL){
		aString err("Failed 'new' in aVector<T>::Append(aVector<T>&)");BadOperation(err);}
	for(long i=0;i<len_;++i){ newData[i]=data[i];}
	for(long i=len_;i<len_+v.len_;++i){ newData[i]=v.data[i-len_];}
	delete[] data;
	data=newData;
	len_+=v.len_;
	newData=NULL;
	}//end aVector<T>::Append (T)


template<class T >  void aVector<T>::Clear(void){
	delete [] data; data=0;
	len_=0;
	}//end aVector<T>::Clear


template<class T > int aVector<T>::Compatible(aVector<T>& v){//can be added, multiplied, etc.
					if(len()==v.len()){
						return 1;}
					return 0;
					}


template<class T > void aVector<T>::Delete (long loc){//shortens the vector
	T* newData = NEW T [len_-1];
	if(newData==NULL){
		aString err("Failed 'new' in aVector<T>::Delete(long)");BadOperation(err);}
	for(long i=0;i<loc;++i){ newData[i]=data[i];}
	for(long i=loc;i<len_-1;++i){ newData[i]=data[i+1];}
	delete[] data;
	data=newData;
	--len_;
	newData=NULL;
	}//end aVector<T>::Delete 


template<class T > void aVector<T>::Initialize(T val){
					for(long i=0;i<len();++i){ 
						data[i]=val;}}


template<class T > void aVector<T>::Insert (T val, const long loc){
	//lengthens the current vector
	T* newData = NEW T [len_+1];
	if(newData==NULL){
		aString err("Failed 'new' in aVector<T>::Append(T)");BadOperation(err);}
	for(long i=0;i<loc;++i){ newData[i]=data[i];}
	newData[loc]=val;
	for(long i=loc;i<len_;++i){ newData[i+1]=data[i];}
	delete[] data;
	data=newData;
	++len_;
	newData=NULL;
	}//end aVector<T>::Insert (T, const int)


template<class T > unsigned long  aVector<T>::len(void){
				return len_;}


template<class T > void aVector<T>::Multiply (T scalar){
	if(scalar == (T)1.0){return;}
	for(long i=0;i<len_;++i){
		data[i] *= scalar;}
	}//end aVector<T>::Multiply (T scalar)


template<class T > long double aVector<T>::Multiply (aVector<T>& v){
	//multiplies this by v and returns result as a double
	if(!Compatible(v)){
		aString a="Error:  can't multiply vectors of different sizes!";
		BadOperation(a);
		return 0.0;}
	long double sum=0.0;
	for(long i=0;i<len_;++i){
		sum += (long double) data[i] * (long double)v.data[i];}
	return sum;
	}//end aVector<T>::Multiply (aVector<T>&)



template<class T > void aVector<T>::Print (ostream& outFile, int columnVector, char separator){
	//default 1 is column vector, 0 is row vector
	//outFile assumed open
	for(long i=0;i<len_;++i){
		outFile<<data[i];
		if (columnVector){outFile<<endl;}
			else{outFile<<separator;}
		}
	outFile<<endl;
	}//end aVector<T>::Print 



template<class T > T aVector<T>::Sum (void){
	T sum = (T)0.0;
	for(long i=0;i<len_;++i){
		sum += data[i];}
	return sum;
	}//end aVector<T>::Sum


template<class T > aVector<double>* aVector<T>::ToDouble(void){
	aVector<double>* newVector = NEW aVector<double>(len_);
	if(newVector==NULL){
		aString err("Failed 'new' in aVector<T>::ToDouble()");BadOperation(err);}
	for(long i=0;i<len_;++i){
		(*newVector).data[i]=(double) data[i];}
	return newVector;
	}//end  aVector<T>::ToDouble


template<class T > aVector<long double>* aVector<T>::ToLongDouble(void){
	aVector<long double>* newVector = NEW aVector<long double>(len_);
	if(newVector==NULL){
		aString err("Failed 'new' in aVector<T>::ToDouble()");BadOperation(err);}
	for(long i=0;i<len_;++i){
		(*newVector).data[i]=(long double) data[i];}
	return newVector;
	}//end  aVector<T>::ToLongDouble


template<class T > void aVector<T>::Replace (T val, long loc){
	data[loc]=val;}


template<class T > T& aVector<T>::operator[](unsigned long i) {
	if(i>=len_){
			aString err("Tried to access element ");err.cat((long)i);err.cat(" in a vector indexed from 0 to ");err.cat((long)len_-1);
			BadOperation(err);
			return data[0];}
		else{
			return data[i];}
	}//end operator[]


template<class T > aVector<T>*  aVector<T>::operator+(aVector<T>& v){
					return Add(v);}


template<class T > aVector<T>*  aVector<T>:: operator-(aVector<T>& v){
					aVector<T> vCopy(v);
					vCopy.Multiply(-1);
					aVector<T>* answer=Add(vCopy);
					return answer;}



template <class T > 
	int operator== (aVector< T >& vector1, aVector< T >& vector2){
		if (vector1.len() != vector2.len()){return 0;}//false
		for(long i=0;i<vector1.len();++i){
			if(vector1[i] != vector2[i]){return 0;}}//false
		return 1;//true
	}//end operator == (aVector<T>&,aVector<T>&)


template <class T > 
	int operator!= (aVector< T >& vector1, aVector< T >& vector2){
		return !(vector1 == vector2);}



#endif

