/***************************************************************
*
*
*	BootList.cpp
*
*	base class:   StatList<T>
*
*	Sorts a distribution and finds approximate P values
*
***************************************************************/


#ifndef BootListcp
#define BootListcp = 1

#include "BootList.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"


template <class T> long double BootList<T>::Get1TailP (T obsValue, int tailEnd){
	if(len() == 0){
		return one;}
	Sort();
	if (First() == Last()){//all are identical
		return one;}//100% probability of this result
	if (obsValue < First()){ 
			if (tailEnd == 0){//it's off the bottom of the scale
					return one;}//0% probability of this result
				else{
					return one;}//100% probability of this result
			}
		else if (obsValue > Last()){ 
			if (tailEnd == 0){//it's off the top of the scale
					return one;}//100% probability of this result
				else{
					return one;}//0% probability of this result
			}
		else{//obsValue is in the middle of the list somewhere
			if (tailEnd == 0){//count percentile from bottom
					for(long rep=0;rep<len();++rep){
						if (obsValue < Nth(rep)) {
							return (long double) rep/(long double)len();}
					}}
				else {//tailEnd==1 so count from the top down
					for(long rep=len()-1;rep>=0;--rep){
						if (obsValue > Nth(rep)){ 
							return (long double) (len()-rep)/(long double)len();}
					}}
		}//end if obsValue<else...
	return one;//default
	}//end  BootList<T>::Get1TailP


template <class T> long double BootList<T>::Get2TailP (T obsValue){
	if(len() == 0){
		return one;}
	Sort();
	if (First() == Last()){//all are identical
		if(obsValue == First()){
				return one;}//100% probability of this result
			else{
				return zero;}
			}
//	T median = Nth(DIV(len(), (long) 2));
	T median;
	if(len()==1){
			median = First();}
		else if(len()==2){
			median = T((First() + Last())/two);}
		else if(odd(len())){
			median = Nth(DIV(len(), (long) 2));}
		else{
			long midpt = DIV(len(), (long) 2);
			median =  T((long double)(Nth(midpt) + Nth(midpt+1))/two);}
	if (obsValue < First()){ 
			return zero;}//0% probability of this result
		else if (obsValue > Last()){ 
			return zero;}//0% probability of this result
		else if (obsValue <= median){//count percentile from bottom
			for(long rep=0;rep<len();++rep){
				if (obsValue < Nth(rep)) {
//					double Pval1tail =   (double) (rep)/(double)len();
					long double Pval1tail =   (long double) (rep+1)/(long double)len();
					Pval1tail = MIN(half,Pval1tail);
					return two * Pval1tail;}
				}}
		else {//median<obsValue<=Last()
			//count percentile from top down
			for(long rep=len()-1;rep>=0;--rep){
				if (obsValue > Nth(rep)){ 
//					double Pval1tail =  1.0 - (double) (rep)/(double)len();
					long double Pval1tail =  one - (long double) (rep+1)/(long double)len();
					Pval1tail = MIN(half,Pval1tail);
					return two * Pval1tail;}
				}}
	return one;//default
	}//end BootList<T>::Get2TailP


template <class T> T BootList<T>::Percentile (double percent){
	//returns the value at percent
	Sort();
	int loc = (percent/100.0)*len();
	return Nth(loc);
	}//end BootList<T>::Percentile (double )


#endif

