/******************************************
*
*	SortList.cpp
*
*	Base class:  ObjList<T>
*
*	User-defined objects that can be sorted
*	i.e., they have definitions for operators <, >, =
*
*******************************************/

#ifndef SortListcpp
#define SortListcpp=1

#include "SortList.h"
//#include <iostream.h>
#include "Element.cpp"
#include "aList.cpp"



template<class T> SortList<T>::SortList(void):ObjList<T>(), sorted_(0){
    	//cout<<"create SortList"<<endl;
    	}//empty list


template<class T> SortList<T>::SortList (SortList<T>& toCopy):
		ObjList<T>(toCopy), sorted_(toCopy.sorted_) {//constucts a new copy
	}//end copy constructor &


template<class T> SortList<T>::SortList (SortList<T>* toCopy):
		ObjList<T>(toCopy), sorted_(toCopy->sorted_) {//constucts a new copy
	}//end copy constructor *



template<class T> SortList<T>::SortList (ObjList<T>& toCopy):
		ObjList<T>(toCopy), sorted_(0) {//copies from basic ObjList
	}//end copy constructor &


template<class T> SortList<T>::SortList (ObjList<T>* toCopy):
		ObjList<T>(toCopy), sorted_(0) {//copies from basic ObjList
	}//end copy constructor *




template<class T > void SortList< T >::Append (T* x) {
	ObjList< T >::Append (x);
	sorted_=0;
	}	//end SortList< T >::Append


template<class T > void SortList< T >::Clear (void) {
	ObjList< T >::Clear ();
	sorted_=0;
	}	// end SortList< T >::Clear


template<class T > void SortList< T >::Delete (long loc) {
	//range of loc=0 (=first) to len_-1 (=last)
	//deletes an element from a list & frees it's memory
	//use DetachNth if you want to save contents from destruction
	if(((loc>0) && (loc<len_-1)) ||(len_==1)){sorted_=0;}
	ObjList< T >::Delete (loc);
	} 	//end SortList< T >::Delete



template<class T > void SortList< T >::Detach (long loc) {
	//range of loc=0 (=first) to len_-1 (=last)
	// removes element at loc, but does not free it's memory
	// memory must be freed elsewhere
	if(((loc>0) && (loc<len_-1)) ||(len_==1)){sorted_=0;}
	ObjList< T >::Delete (loc);
	} //end SortList< T >::Detach(long)


template<class T > void SortList< T >::Detach (void) {
	// deletes each element, but does not free it's memory
	ObjList< T >::Detach ();
	sorted_=0;
	}	// end SortList< T >::Detach(void)


template <class T>  ObjList<T>*  SortList<T>::Duplicate(void){
	return NEW SortList<T>(*this);
	} //end SortList< T >::Duplicate


template<class T > void SortList< T >::Insert (T* x, long loc) {
	//x inserted at loc; elements  loc are shifted
	//x simply Appended if loclen_
	ObjList< T >::Insert (x, loc);
	sorted_=0;
	}	//end SortList< T >::Insert




template<class T > void SortList< T >::Prepend (T* x) {
	//attaches x to the beginning of the list
	ObjList< T >::Prepend (x);
	sorted_=0;
	}	//end SortList< T >::Prepend 



template<class T > void SortList< T >::Replace (T* x, long loc) {
	//range of loc=0 (=first) to len()-1 (=last)
	//the old element must be freed elsewhere from memory
	ObjList< T >::Replace (x, loc);
	sorted_=0;
	}	// SortList< T >::Replace


template<class T> void SortList<T>::Sort(void){
	if (len_<2){return;}
	Sort(0,len_-1);
	sorted_=1;
	}// SortList<T>::Sort

	
	
template<class T> void SortList<T>::Sort(long start, long finish){
	//implements QuickSort
	//patched to accomodate 0-indexing
	if (len_<2) {return;}
	long left= start+1, right= finish+1;
	long two = 2;
	T startValue = Nth(DIV((left + right),two)-1);
	do{
			while (Nth(left-1) < startValue) {++left;}
			while (startValue < Nth(right-1)) {--right;}
			if (left <= right) {
				T *temp = &(Nth(left-1));
				Replace(&(Nth(right-1)),left-1);
				Replace(temp,right-1);
				++left;
				--right;
				}//end if
		}while(right > left);
	if (start < right-1) {
		Sort(start, right-1);}
	if (left-1 < finish) {
		Sort(left-1, finish);}
	}//end  SortList<T>::Sort



template<class T> void SortList<T>::SortDescending(void){
	if (len_<2){return;}
	SortDescending(0,len_-1);
	sorted_=1;
	}// SortList<T>::SortDescending()

	
	
template<class T> void SortList<T>::SortDescending(long start, long finish){
	//implements QuickSort
	//patched to accomodate 0-indexing
	if (len_<2) {return;}
	long left= start+1, right= finish+1;
	long two = 2;
	T startValue = Nth(DIV((left + right),two)-1);
	do{
			while (Nth(left-1) > startValue) {++left;}
			while (startValue > Nth(right-1)) {--right;}
			if (left <= right) {
				T *temp = &(Nth(left-1));
				Replace(&(Nth(right-1)),left-1);
				Replace(temp,right-1);
				++left;
				--right;
				}//end if
		}while(right > left);
	if (start < right-1) {
		SortDescending(start, right-1);}
	if (left-1 < finish) {
		SortDescending(left-1, finish);}
	}//end  SortList<T>::SortDescending




template<class T > SortList< T >& SortList< T >::operator= (SortList< T >& z){
    //copies list z element by element
    Clear();
    for (long i=0;i<z.len_;++i){
    	T& x = z.Nth(i);
    	T* y = (T*)(x.Duplicate());//virtual-copies the object
    	Append(y);}
    sorted_ = z.sorted_;
    return *this;
    }// ObjList< T >::operator=



template <class T > int operator== (SortList< T >& list1, SortList< T >& list2){
	if (list1.len_ != list2.len_){return 0;}
	for (long i=0;i<list1.len_;++i){
		if(list1[i] != list2[i]){return 0;}}
	return 1;
	}//end friend operator==

template <class T > 
	int operator!= (SortList< T >& list1, SortList< T >& list2){
		return !(list1==list2);}



#endif

