/******************************************
*
*	aList.cpp
*
*	This implements a template container class
*	for growable lists of unknown size.
*
*	Lists of numbers and characters are stored in class aList.
*	
*	Lists of class objects are stored in class ObjList.
*
*******************************************/

#ifndef alistcp
#define alistcp = 1

#include <iostream.h>
#include "Element.cpp"
#include "aList.h"
#include "StringClass.h"


template<class T> aList<T>::aList(void): 
	first(0), last(0), recent(0), len_(0), recentLoc_(0) {
    	//cout<<"create aList"<<endl;//debug coode
    	}//empty list



template<class T> aList<T>::aList (aList<T>& toCopy):
		first(NULL), last(NULL), recent(NULL), len_(0), recentLoc_(0) {//constucts a new copy
	for(long i=0;i<toCopy.len();++i){
		T x=toCopy[i];
		Append(x);
		}
	}//end copy constructor &


template<class T> aList<T>::aList (aList<T>* toCopy):
		first(NULL), last(NULL), recent(NULL), len_(0), recentLoc_(0) {//constucts a new copy
	for(long i=0;i<toCopy->len();++i){
		T x=toCopy->Nth(i);
		Append(x);
		}
	}//end copy constructor *

template<class T> aList<T>::~aList(void) {
	Clear();//deletes remaining contents
	}//end destructor






template<class T> void aList<T>::Append (T x) {try{
	//attaches x to the end of the list
	NumElem<T>	*elem=NEW NumElem<T>(x);
	if (len_ == 0) {first = elem;}
		else {		//connect it to the end of the list
			elem->prev = last;
			last->next = elem;}
	elem->next = NULL;	//there is no next element
	last = recent = elem; //resets the last element
	recentLoc_ = len_;//the most recent item manipulated
	++len_;
	}catch(std::bad_alloc){
		CatchBadAlloc("Append");}
	}	//end aList::Append


template<class T> int aList<T>::BadIndex(long loc){
	//prints error message if index is bad
	if ((len_<0)||(loc<0)||(loc>=len_)){
//		cout<<"Bad index: sought item "<<loc<<" in a 0-indexed aList of len "<<len_<<endl;
		aString mess("Bad index: sought item ");mess.cat(loc);
		aString m2(" in a 0-indexed ObjList of len ");mess.cat(m2);mess.cat(len_);
		aError(mess);
		return 1;}
	return 0;
	}//end aList<T>::BadIndex

	
template<class T> int aList<T>::CatchBadAlloc(const char* fname){
	aString mess("memory error:  operator new() failed in function ");
	mess.cat(fname);
	aError(mess);
	return 1;
	}//end aList<T>::CatchBadAlloc(const char*)


template<class T> void aList<T>::Clear (void) {
	// deletes each element & frees it's memory
	while (len_ > 0) {
		DeleteLast();}
	}	// end aList<T>::Clear


template<class T> void aList<T>::Copy (aList<T>& toCopy) {
	// copies each element 
	Clear();
    for (long i=0;i<toCopy.len();++i){
    	Append(toCopy.Nth(i));}
	}	// end aList<T>::Copy


template<class T> void aList<T>::Copy (aList<T>* toCopy) {
	// copies each element 
	Clear();
    for (long i=0;i<toCopy->len();++i){
    	Append(toCopy->Nth(i));}
	}	// end aList<T>::Copy


template<class T> void aList<T>::Decrement (long loc) {
	//range of loc=0 (=first) to len()-1 (=last)
	//adds 1 to the value in list
	NumElem<T> *elem = GoToElem(loc);
	if(elem !=NULL){(elem->data_)-- ;}
	} //end aList<T>::Decrement


template<class T> void aList<T>::DeleteFirst (void) {
	// deletes first element from a list & frees it's memory
	//use DetachFirst if you want to save it from destruction
	if (len_ > 0) {
		NumElem<T> *elem = first;
		if (len_ == 1) {//no first or last elements will be left
				first = recent = last = NULL;}
			else if (len_ == 2) {
				//the first & last elements are now the same
				first = elem->next;
				first->prev = NULL;
				last = recent = first;}
			else {//the 2nd element is now the first
				elem->next->prev = NULL;
				first = recent = elem->next;
				}
		--len_ ;
		recentLoc_=0;
		delete elem;elem=0;
		} //end if len>0
		else {BadIndex(0);}
	}	// end aList<T>::DeleteFirst


template<class T> void aList<T>::DeleteLast (void) {
	//deletes an element from a list & frees it's memory
	//use DetachLast if you want to save it from destruction
	if (len_ > 0) {
		NumElem<T> *elem = last;
		if (len_ == 1) {//no first or last elements will be left
				first = recent = last =NULL;
				recentLoc_ = len_ = 0;
				delete elem;elem=0;
				return;}
			else if (len_ == 2) {//the first & last elements are now the same
				last = elem->prev;
				last->next = NULL;
				first = recent = last;}
			else {//the 2nd-last element is now the last
				elem->prev->next = NULL;
				last = recent = elem->prev;}
		--len_;
		recentLoc_ = len_ - 1;
		delete elem;elem=0;
		} //end if len>0
		else {BadIndex(0);}
	}	// end aList<T>::DeleteLast
	

template<class T> void aList<T>::Delete (long loc) {
	//range of loc=0 (=first) to len_-1 (=last)
	//deletes an element from a list & frees it's memory
	//use DetachNth if you want to save contents from destruction
	if (!(BadIndex(loc))&&(len_ > 0) && (loc >=0) && (loc<len_ )) {
		if (loc == len_-1) {
				DeleteLast();}
			else if (loc == 0) {
				DeleteFirst();}
			else {  //delete an internal element
				NumElem<T> *elem = GoToElem(loc);
				elem->prev->next = elem->next;
				elem->next->prev = elem->prev;
				--len_;
				 recent = elem->prev;//recent drops to one earlier in list
				--recentLoc_;
				delete elem;elem=0;
				}
		} //end if len_>0
	} 	//end aList<T>::Delete



template <class T>  aList<T>*  aList<T>::Duplicate(void){
	aList<T> *dup = 0;
	try{
		dup = NEW aList<T>(*this);
	}catch(std::bad_alloc){
		CatchBadAlloc("Duplicate");}
	return dup;
	} //end aList< T >::Duplicate



template<class T> T& aList<T>::First(void) {
	//returns a copy of the item, and leaves original in list
	//returns unitialized reference if the index is no good
	BadIndex(0);
	recent = first; recentLoc_=0;
	return first->data();
	}	//end aList<T>::First


template<class T> NumElem<T>* aList<T>::GoToElem (long loc) {
	if (!(BadIndex(loc))&&(len() > 0)) {
			if (recentLoc_==loc) {return recent;}
			NumElem<T> *elem = NULL;
			if(ABS(recentLoc_-loc)<loc){//if loc is closer to recentLoc_ than to zero
					elem = recent;
					if(recentLoc_<loc){//count up to loc
							for(long i=recentLoc_;i<loc;++i){elem = elem->next;}
							}
						else{//count down to loc
							for(long i=recentLoc_;i>loc;--i){elem = elem->prev;}
							}
					}
				else{//count from zero
					elem = first;
					for(long i=0;i<loc;++i){elem = elem->next;}
					}
			recentLoc_=loc;
			recent = elem;
			return elem;}
		else{//default returns the first item
			recentLoc_=0;
			recent = first;
			return first;}
	}//end aList<T>::GoToElem 


template<class T> void aList<T>::Insert (T x, long loc) {
	//x inserted after loc; elements > loc are shifted
	//x simply Appended if loclen_
/*  if (!(BadIndex(loc))&&(0<=loc)&&(loc<len_)){
	if (loc == 0) {
			Prepend(x);}
		else if (loc == len()-1) {
			Append(x);}
		else {	//insert in the middle of the list
			NumElem<T> *elem = GoToElem(loc);
			NumElem<T> *newElem = NEW NumElem<T>(x);
			newElem->next = elem->next;
			elem->next->prev = newElem;
			elem->next = newElem;
			newElem->prev = elem;
			++len_;
			recent = newElem;
			recentLoc_=loc+1;
			}
	  } //end if 0loc<len_
*/
	//x inserted at loc; elements  loc are shifted up
	//x simply Appended if loc==len_
  if ((loc < 0) ||(loc > len_)){
  		aString mess("Bad index: tried to insert object at ");mess.cat(loc);
		aString m2(" in a 0-indexed ObjList of len ");mess.cat(m2);mess.cat(len_);
		aError(mess);
		return;
		}
	try{
	if (loc == 0) {
			Prepend(x);}
//		else if (loc == len_-1) {
		else if (loc == len_) {
			Append(x);}
		else {	//insert in the middle of the list
			NumElem< T > *elem = GoToElem(loc);
			NumElem< T > *newElem = NEW NumElem< T >(x);
			newElem->next = elem;
			newElem->prev = elem->prev;
			elem->prev->next = newElem;
			elem->prev = newElem;
			++len_;
			recent = newElem;
			recentLoc_=loc;
//			newElem->next = elem->next;
//			elem->next->prev = newElem;
//			elem->next = newElem;
//			newElem->prev = elem;
//			++len_;
//			recent = newElem;
//			recentLoc_=loc+1;
			}
//	  } //end if 0loc<len_
	}catch(std::bad_alloc){
		CatchBadAlloc("Insert");}
	}	//end aList<T>::Insert



template<class T> int aList<T>::Includes (T x) {
	// true if x is in the list
	if (len_ > 0) {
		NumElem<T> *elem = first;
		for(long i=0;i<len_;++i){
			if (elem->data() == x) {
				recent = elem;
				recentLoc_=i;
				return 1;}
			elem = elem->next;}
		}
	recent = first;
	recentLoc_=0;
	return 0;
	}	//end aList<T>::Includes


template<class T> void aList<T>::Increment (long loc) {
	//range of loc=0 (=first) to len_-1 (=last)
	//adds 1 to the value in list at loc
	NumElem<T> *elem = GoToElem(loc);
	if(elem !=NULL){(elem->data_)++ ;}
	} //end aList<T>::Increment


template<class T> T& aList<T>::Last (void) {
	//returns a reference the item, and leaves original in list
	//returns uninitialized reference if the index is no good
	BadIndex(len_-1);
	recent = last;
	recentLoc_=len_-1;
	return last->data();
	}	//end aList<T>::Last



template<class T> long aList<T>::len(void) {
	return len_;}



template<class T> long aList<T>::Loc (T x) {
	//returns the location if the object in the list
	//returns -1 if not present, so be careful!
	if (len_ > 0) {
		NumElem<T> *elem = first;
		for(long i=0;i<len_;++i) {
			if (elem->data() == x) {
				recent = elem;
				recentLoc_=i;
				return i;}
			elem = elem->next;
			}
		} //end if len_>0
	recent = first;
	recentLoc_=0;
	return (-1);
	}	//end aList<T>::Loc



template<class T> T& aList<T>::Nth (long loc) {
	//range of loc=0 (=first) to len_-1 (=last)
	//returns a reference to the item at loc, and leaves original in list
	//returns uninitialized reference if the index is no good
	NumElem<T> *elem = GoToElem(loc);
	return elem->data();
	} //end aList<T>::Nth


template<class T> void aList<T>::Prepend (T x) {try{
	//attaches x to the beginning of the list
	NumElem <T> *elem = NEW NumElem<T>(x);
	if (len_ == 0) {
			last = elem;}
		else { //connect it to the beginning of the list
			elem->next = first;
			first->prev = elem;}
	first = recent = elem; //reset the val of the first element
	++len_;
	recentLoc_=0;
	}catch(std::bad_alloc){
		CatchBadAlloc("Prepend");}
	}	//end aList<T>::Prepend 

template <class T> void aList<T>::Print(ostream& outfile, char* seperator){
	for(long i=0;i<len_;++i){
		T& x = Nth(i);
		outfile<<x<<seperator;}
	outfile<<endl;
	}


template<class T> void aList<T>::Replace (T x, long loc) {
	//range of loc=0 (=first) to len_-1 (=last)
	//the old element must be freed elsewhere from memory
	NumElem<T> *elem = GoToElem(loc);
	if(elem !=NULL){elem->data_ = x;}
	}	// aList<T>::Replace


template<class T> void aList<T>::Sort(void){
	if (len_<2){return;}
	Sort(0,len_-1);
	}// aList<T>::Sort

	
	
template<class T> void aList<T>::Sort(long start, long finish){
	//implements QuickSort
	//patched to accomodate 0-indexing
	if (len_<2) {return;}
	long left= start+1, right= finish+1;
//	T startValue = Nth(div((start + finish),(long) 2));
	long two = 2;
	T startValue = Nth(DIV((left + right),two)-1);
	do{
			while (Nth(left-1) < startValue) {++left;}
			while (startValue < Nth(right-1)) {--right;}
			if (left <= right) {
				T temp = Nth(left-1);
				Replace(Nth(right-1),left-1);
				Replace(temp,right-1);
				++left;
				--right;
				}//end if
		}while(right > left);//Pascal:  repeat ... until right <=left
	if (start < right-1) {
		Sort(start, right-1);}
	if (left-1 < finish) {
		Sort(left-1, finish);}
	}//end  aList<T>::Sort



template<class T > aList< T >& aList< T >::operator= (aList< T >& z){
    //copies list z element by element
    Clear();
    for (long i=0;i<z.len_;++i){
    	Append(z.Nth(i));}
    return *this;
    }// aList< T >::operator=


template<class T > T& aList< T >::operator[] (const long loc) {
	return Nth(loc);
    }// aList< T >::operator[]


template <class T > int operator== (aList< T >& list1, aList< T >& list2){
	if (list1.len_ != list2.len_){return 0;}
	for (long i=0;i<list1.len_;++i){
		if(list1[i] != list2[i]){return 0;}}
	return 1;
	}//end friend operator==

template <class T > int operator!= (aList< T >& list1, aList< T >& list2){
	return !(list1==list2);}






// HERE ARE THE DEFINITIONS FOR CLASS ObjList



template<class T> ObjList<T>::ObjList(void): 
	first(NULL), last(NULL), recent(NULL), len_(0), recentLoc_(0){
    	//cout<<"create ObjList"<<endl;
    	}//empty list


template<class T> ObjList<T>::ObjList (ObjList<T>& toCopy):
		first(NULL), last(NULL), recent(NULL), len_(0), recentLoc_(0) {//constucts a new copy
	for(long i=0;i<toCopy.len();++i){
		T& x=toCopy[i];
//		T* y=NEW T(x);//copies the object
		T* y=(T*)(x.Duplicate());//virtual-copies the object
		Append(y);
		}
	}//end copy constructor &


template<class T> ObjList<T>::ObjList (ObjList<T>* toCopy):
		first(NULL), last(NULL), recent(NULL), len_(0), recentLoc_(0) {//constucts a new copy
	for(long i=0;i<toCopy->len();++i){
		T& x=toCopy->Nth(i);
//		T* y=NEW T(x);//copies the object
		T* y=(T*)(x.Duplicate());//virtual-copies the object
		Append(y);
		}
	}//end copy constructor *


template<class T> ObjList<T>::~ObjList(void) {
    	Clear();//deletes remaining contents
    	}//end destructor





template<class T > void ObjList< T >::Append (T* x) {try{
	//attaches x to the end of the list
	ObjElem< T >	*elem=NEW ObjElem< T >(x);
	if (len_ == 0) {first = elem;}
		else {		//connect it to the end of the list
			elem->prev = last;
			last->next = elem;}
	elem->next = NULL;	//there is no next element
	last = recent = elem; //resets the last element
	recentLoc_ = len_;//the most recent item manipulated
	++len_;
	}catch(std::bad_alloc){
		CatchBadAlloc("Append");}
	}	//end ObjList< T >::Append


template<class T> int ObjList<T>::BadIndex(long loc){
	//prints error message if index is bad
	if ((len_<=0)||(loc<0)||(loc>=len_)){
//		cout<<"Bad index: sought item "<<loc<<" in a 0-indexed ObjList of len "<<len_<<endl;
		aString mess("Bad index: sought item ");mess.cat(loc);
		aString m2(" in a 0-indexed ObjList of len ");mess.cat(m2);mess.cat(len_);
		aError(mess);
		return 1;}
	return 0;
	}//end ObjList<T>::BadIndex
	

template<class T> int ObjList<T>::CatchBadAlloc(const char* fname){
	aString mess("memory error:  operator new() failed in function ");
	mess.cat(fname);
	aError(mess);
	return 1;
	}//end ObjList<T>::CatchBadAlloc(const char*)


template<class T > void ObjList< T >::Clear (void) {
	// deletes each element & frees it's memory
	while (len_ > 0) {
		DeleteLast();}
	}	// end ObjList< T >::Clear


template<class T> void ObjList< T >::Copy (ObjList<T>& toCopy) {
	// copies each element 
	Clear();
	for(long i=0;i<toCopy.len();++i){
		T& x=toCopy[i];
//		T* y=NEW T(x);//copies the object
		T* y=(T*)(x.Duplicate());//virtual-copies the object
		Append(y);
		}
	}	// end ObjList<T>::Copy


template<class T> void ObjList< T >::Copy (ObjList<T>* toCopy) {
	// copies each element 
	Clear();
	for(long i=0;i<toCopy->len();++i){
		T& x=toCopy->Nth(i);
//		T* y=NEW T(x);//copies the object
		T* y= (T*)(x.Duplicate());//virtual-copies the object
		Append(y);
		}
	}	// end ObjList<T>::Copy


template<class T > void ObjList< T >::DeleteFirst (void) {
	// deletes first element from a list & frees it's memory
	//use DetachFirst if you want to save it from destruction
	if ((!BadIndex(0))&&(len_ > 0)) {
		ObjElem< T > *elem = first;
		if (len_ == 1) {//no first or last elements will be left
				first = recent = last = NULL;}
			else if (len_ == 2) {
				//the first & last elements are now the same
				first = elem->next;
				first->prev = NULL;
				last = recent = first;}
			else {//the 2nd element is now the first
				elem->next->prev = NULL;
				first = recent = elem->next;
				}
		--len_ ;
		recentLoc_=0;
		delete elem->data_; elem->data_=NULL;
		delete elem;elem=0;
		} //end if len>0
	}	// end ObjList< T >::DeleteFirst


template<class T > void ObjList< T >::DeleteLast (void) {
	//deletes an element from a list & frees it's memory
	//use DetachLast if you want to save it from destruction
	if ((!BadIndex(len_-1))&&(len_ > 0)) {
		ObjElem< T > *elem = last;
		if (len_ == 1) {//no first or last elements will be left
				first = recent = last = NULL;
				recentLoc_ = len_ = 0;
				delete elem->data_; elem->data_=NULL;
				delete elem;elem=0;
				return;}
			else if (len_ == 2) {//the first & last elements are now the same
				last = elem->prev;
				last->next = NULL;
				first = recent = last;}
			else {//the 2nd-last element is now the last
				elem->prev->next = NULL;
				last = recent = elem->prev;}
		--len_; 
		recentLoc_ = len_ - 1;
		delete elem->data_; elem->data_=NULL;
		delete elem;elem=0;
		} //end if len>0
	}	// end aList< T >::DeleteLast
	

template<class T > void ObjList< T >::Delete (long loc) {
	//range of loc=0 (=first) to len_-1 (=last)
	//deletes an element from a list & frees it's memory
	//use DetachNth if you want to save contents from destruction
	if ((!BadIndex(loc))&&(len_ > 0) && (loc >=0) && (loc<len_ )) {
		if (loc == len_-1) {
				DeleteLast();}
			else if (loc == 0) {
				DeleteFirst();}
			else {  //delete an internal element
				ObjElem< T > *elem = GoToElem(loc);
				elem->prev->next = elem->next;
				elem->next->prev = elem->prev;
				--len_;
				 recent = elem->prev;//recent drops to one earlier in list
				--recentLoc_;
				delete elem->data_; elem->data_=NULL;
				delete elem; elem=0;
				}
		} //end if len_>0
	} 	//end aList< T >::Delete


template<class T > void ObjList< T >::DetachFirst (void) {
	// removes first element, but does not free it's memory
	// memory must be freed elsewhere
	if ((!BadIndex(0))&&(len_ > 0)) {
		ObjElem< T > *elem = first;
		if (len_ == 1) {//no first or last elements will be left
				first = recent = last = NULL;}
			else if (len_ == 2) {
				// the first & last elements are now the same
				first = elem->next;
				first->prev = NULL;
				last = recent = first;}
			else {	//the 2nd element is now the first
				elem->next->prev = NULL;
				first = recent = elem->next;}
		--len_;
		recentLoc_=0;
		elem->data_=NULL;//leaks memory if not freed elsewhere!
		delete elem;elem=0;
		} //end if len_>0
	}	//end ObjList< T >::DetachFirst


template<class T > void ObjList< T >::DetachLast (void) {
	// removes last element, but does not free it's memory
	// memory must be freed elsewhere
	if ((!BadIndex(len_-1))&&(len_ > 0)) {
		ObjElem< T > *elem = last;
		if (len_ == 1) { //no first or last elements will be left}
				first = recent = last = NULL;
				recentLoc_ = len_ = 0;
				elem->data_ = NULL;//leaks memory if not freed elsewhere!
				delete elem;elem=0;
				return;}
			else if (len_ == 2) {
				// the first & last elements are now the same
				last = elem->prev;
				last->next = NULL;
				first = recent = last;}
			else {	//the 2nd-last element is now the last
				elem->prev->next = NULL;
				last = recent = elem->prev;}
		--len_;
		recentLoc_ = len_ -1;
		elem->data_=NULL;//leaks memory if not freed elsewhere!
		delete elem;elem=0;
		} //end if len()>0
	} //end ObjList< T >::DetachLast



template<class T > void ObjList< T >::Detach (long loc) {
	//range of loc=0 (=first) to len_-1 (=last)
	// removes element at loc, but does not free it's memory
	// memory must be freed elsewhere
	if ((!BadIndex(loc))&&(len_ > 0) && (loc >=0) && (loc<len_ )) {
		if (loc == len_-1)  {
				DetachLast();}
			else if (loc == 0) {
				DetachFirst();}
			else { //detach an internal element
				ObjElem<T> *elem = GoToElem(loc);
				elem->prev->next = elem->next;
				elem->next->prev = elem->prev;
				--len_;
				 recent = elem->prev;//recent drops to one earlier in list
				--recentLoc_;
				elem->data_ = NULL;//leaks memory if not freed elsewhere!
				delete elem;elem=0;
				}
		} //end if len_>0
	} //end ObjList< T >::Detach


template<class T > void ObjList< T >::Detach (void) {
	// deletes each element, but does not free it's memory
	while (len_ > 0) {
		DetachLast();}
	}	// end ObjList< T >::Detach(void)


template<class T > void ObjList< T >::Detach (T* x) {
	// removes element x, but does not free it's memory
	// memory must be freed elsewhere
	// does nothing if x is not in the list
	Detach(Loc(x));
	} //end ObjList< T >::Detach


template <class T>  ObjList<T>*  ObjList<T>::Duplicate(void){
	ObjList<T> *dup=0;
	try{
		dup = NEW ObjList<T>(*this);
	}catch(std::bad_alloc){
		CatchBadAlloc("Duplicate");}
	return dup;
	} //end ObjList< T >::Duplicate


template<class T > T& ObjList< T >::First(void) {
	//returns a pointer to the item, and leaves original in list
	//returns uninitialized pointer if the index is no good
	BadIndex(0);
	recent = first; recentLoc_=0;
	return first->data();
	}	//end ObjList< T >::First


template<class T> ObjElem<T>* ObjList<T>::GoToElem (long loc) {
	if (!(BadIndex(loc))&&(len() > 0)) {
			if (recentLoc_==loc) {return recent;}
			ObjElem<T> *elem = NULL;
			if(ABS(recentLoc_-loc)<loc){//if loc is closer to recentLoc_ than to zero
					elem = recent;
					if(recentLoc_<loc){//count up to loc
							for(long i=recentLoc_;i<loc;++i){elem = elem->next;}
							}
						else{//count down to loc
							for(long i=recentLoc_;i>loc;--i){elem = elem->prev;}
							}
					}
				else{//count from zero
					elem = first;
					for(long i=0;i<loc;++i){elem = elem->next;}
					}
			recentLoc_=loc;
			recent = elem;
			return elem;}
		else{//default returns the first item
			recentLoc_=0;
			recent = first;
			return first;}
	}//end ObjList<T>::GoToElem 


template<class T > void ObjList< T >::Insert (T* x, long loc) {
	//x inserted at loc; elements  loc are shifted up
	//x simply Appended if loc==len_
//  if ((!BadIndex(loc))&&(0<=loc)&&(loc<len_)){
  if ((loc < 0) ||(loc > len_)){
  		aString mess("Bad index: tried to insert object at ");mess.cat(loc);
		aString m2(" in a 0-indexed ObjList of len ");mess.cat(m2);mess.cat(len_);
		aError(mess);
		return;
		}
	try{
	if (loc == 0) {
			Prepend(x);}
//		else if (loc == len_-1) {
		else if (loc == len_) {
			Append(x);}
		else {	//insert in the middle of the list
			ObjElem< T > *elem = GoToElem(loc);
			ObjElem< T > *newElem = NEW ObjElem< T >(x);
			newElem->next = elem;
			newElem->prev = elem->prev;
			elem->prev->next = newElem;
			elem->prev = newElem;
			++len_;
			recent = newElem;
			recentLoc_=loc;
//			newElem->next = elem->next;
//			elem->next->prev = newElem;
//			elem->next = newElem;
//			newElem->prev = elem;
//			++len_;
//			recent = newElem;
//			recentLoc_=loc+1;
			}
//	  } //end if 0loc<len_
	}catch(std::bad_alloc){
		CatchBadAlloc("Insert");}
	}	//end ObjList< T >::Insert



template<class T > int ObjList< T >::Includes (T* x) {
	// true if x is in the list
	// can crash if == is not defined for T
	if (len_ > 0) {
		ObjElem< T > *elem = first;
		for(long i=0;i<len_;++i){
			if (*(elem->data_) == *x) {
				recent = elem;
				recentLoc_=i;
				return 1;}
			elem = elem->next;}
		}
	recent = first;
	recentLoc_=0;
	return 0;
	}	//end ObjList< T >::Includes


template<class T > T& ObjList< T >::Last (void) {
	//returns a pointer the item, and leaves original in list
	//returns uninitialized pointer if the index is no good
	BadIndex(len_-1);
	recent = last;
	recentLoc_=len_-1;
	return  last->data();
	}	//end ObjList< T >::Last


template<class T >  const long ObjList< T >::len(void) {
	return len_;
	}	//end ObjList< T >::len


template<class T > long ObjList< T >::Loc (T* x) {
	//returns the location if the object in the list
	//returns -1 if not present, so be careful!
	if (len_ > 0) {
		ObjElem< T > *elem = first;
		for(long i=0;i<len_;++i) {
			if (*(elem->data_) == *x) {
				recent = elem;
				recentLoc_=i;
				return i;}
			elem = elem->next;
			}
		} //end if len_>0
	recent = first;
	recentLoc_=0;
	return (-1);
	}	//end ObjList< T >::Loc



template<class T > T& ObjList< T >::Nth (long loc) {
	//range of loc=0 (=first) to len_-1 (=last)
	//returns a pointer to the item at loc, and leaves original in list
	//returns uninitialized pointer if the index is no good
	ObjElem<T> *elem = GoToElem(loc);
	return elem->data();
	} //end ObjList< T >::Nth


template<class T > void ObjList< T >::Prepend (T* x) {try{
	//attaches x to the beginning of the list
	ObjElem < T > *elem = NEW ObjElem< T >(x);
	if (len_ == 0) {
			last = elem;}
		else { //connect it to the beginning of the list
			elem->next = first;
			first->prev = elem;}
	first = recent = elem; //reset the val of the first element
	++len_;
	recentLoc_=0;
	}catch(std::bad_alloc){
		CatchBadAlloc("Prepend");}
	}	//end ObjList< T >::Prepend 



template<class T > void ObjList< T >::Replace (T* x, long loc) {
	//range of loc=0 (=first) to len()-1 (=last)
	//the old element must be freed elsewhere from memory
	ObjElem<T> *elem = GoToElem(loc);
	if(elem !=NULL){elem->data_ = x;}
	}	// ObjList< T >::Replace



template<class T > ObjList< T >& ObjList< T >::operator= (ObjList< T >& z){
    //copies list z element by element
    Clear();
    for (long i=0;i<z.len_;++i){
    	T& x = z.Nth(i);
//    T* y = NEW T(x);
    	T* y = (T*)(x.Duplicate());//virtual-copies the object
    	Append(y);}
    return *this;
    }// ObjList< T >::operator=


template<class T > T&    ObjList< T >::operator[] (const long loc) {
	return Nth(loc);
	}	// ObjList< T >::operator[]


template <class T > int operator== (ObjList< T >& list1, ObjList< T >& list2){
	if (list1.len_ != list2.len_){return 0;}
	for (long i=0;i<list1.len_;++i){
		if(list1[i] != list2[i]){return 0;}}
	return 1;
	}//end friend operator==

template <class T > 
	int operator!= (ObjList< T >& list1, ObjList< T >& list2){
		return !(list1==list2);}



#endif

