/*********************************************
*
*
*		BetaDistribution.cpp
*
*
*		Functions for calculating beta-distribution statistics
*
*
**********************************************/


#ifndef BetaDistributioncpp
#define BetaDistributioncpp = 1

#include "BetaDistribution.h"
#include "Element.cpp"
#include "aList.cpp"


long double Beta(long double alpha, long double beta){
	long double density = zero;
	if (alpha <= zero || beta <= zero){//the density is infinite
			aString err("lnBetaDistribution(");
			err.cat(alpha);
			err.cat(",");
			err.cat(beta);
			err.cat(") is undefined.  Beta=0  returned.");
			aError(err);
			}
		else{
			density = exp(lnBeta(alpha,beta));
			}
	return density;
}//end Beta(long double, long double)


long double BinomialDistributionCDF(long n, long r, long double p){
		//n = sample size, r = #found, p = probability
		//calculated using the incomplete beta function
	long double pr=zero;
	if(p>zero && n>0){
		if (p==one && r==n){
				pr=one;}
			else if(p<one){
				pr = BetaIncomplete((long double)r,(long double)(n-r+1),p);
				}//p<1
			else{}
		}//p>0
	return pr;
	}//BinomialDistributionCDF(long double, long double, long double)
		

long double lnBeta(long double alpha, long double beta){
	long double lnval = one;
	if (alpha <= zero || beta <= zero){
			aString err("Beta(");
			err.cat(alpha);
			err.cat(",");
			err.cat(beta);
			err.cat(") is undefined.  lnBeta=1 returned.");
			aError(err);
			}
		else{
			long double lngammaAlpha = gammaln(alpha);
			long double lngammaBeta = gammaln(beta);
			long double lngammaAlphaBeta = gammaln(alpha+beta);
			lnval = lngammaAlpha + lngammaBeta - lngammaAlphaBeta;
			}
	return lnval;
	}//end lnBeta(long double, long double)


long double BetaDistributionPDF(long double alpha, long double beta, long double x){
	return exp(lnBetaDistributionPDF(alpha,beta,x));
	}//end BetaDistributionPDF(long double, long double, long double)



long double lnBetaDistributionPDF(long double alpha, long double beta, long double x){
	//probability density function at x
	long double lndensity = one;
	if (alpha <= zero || beta <= zero || x <=zero || x>= one){
			if((alpha>one && beta>one) && (x==zero || x==one) ){
					}//density is zero
				else{//x is out of range, or the density is infinite
					aString err("lnBetaDistribution(");
					err.cat(alpha);
					err.cat(",");
					err.cat(beta);
					err.cat(",");
					err.cat(x);
					err.cat(") is undefined.  ln[density] = 1  returned.");
					aError(err);
					}
			}
		else{
			lndensity = (alpha-one)*log(x) + (beta - one)*log(one-x) - lnBeta(alpha,beta);
			}
	return lndensity;
}//end lnBetaDistributionPDF(long double, long double, long double)


long double lnBetaDistributionPDF(long double alpha, long double beta, long double x, long double dx){
	//probability density function at x scaled by dx
	long double answer = lnBetaDistributionPDF(alpha,beta,x) + ln(dx);
//	long double answer = lnBetaDistributionPDF(alpha,beta,x);
	return answer;
	}//lnBetaDistributionPDF(long double, long double, long double, long double)



long double BetaDistributionCDF(long double alpha, long double beta, long double x){
	return BetaRegularized(alpha,beta,x);
	}//BetaDistributionPDF(long double, long double, long double)


long double BetaRegularized(long double alpha, long double beta, long double x){
	return BetaIncomplete(alpha,beta,x)/Beta(alpha,beta);
	}//BetaRegularized(long double, long double, long double)


	
long double BetaIncomplete(long double alpha, long double beta, long double x){
	//Beta function integrated from 0 to  x (0x1)
	//from Press et al.
	long double bi = zero;
	long double pdf = BetaDistributionPDF(alpha, beta,x);
	if(x<(alpha+one)/(alpha+beta + two)){
			bi = pdf * BetaContinuedFraction(alpha, beta, x) / alpha;
			}
		else{
			bi = one - pdf *  BetaContinuedFraction(beta, alpha, one-x) / beta;
			}
	return bi;
	}//BetaIncomplete(long double, long double, long double)



long double BetaContinuedFraction(long double alpha, long double beta, long double x){
	//used to calculate BetaIncomplete
	int m, itmax = 100;
	long double eps = (long double) 1.0E-7;
	long double tam,qap,qam,qab,em,d,bz,bpp,bp,bm,az,app,am,aold,ap;
	am = bm = az = one;
	qab = alpha + beta;
	qap = alpha + one;
	qam = alpha - one;
	bz = one - qab * x/qap;
	for(m=0;m<itmax;++m){
		em=(long double) m;
		tam = two*em;
		d = em*(beta-em)*x/((alpha+tam)*(qap+tam));
		ap = az + d*am;
		bp = bp + d*bm;
		d = -(alpha+em)*(qab+em)*x/((alpha+tam)*(qap+tam));		
		app = ap + d * az;
		bpp = bp + d * bz;
		aold = az;
		am = ap/bpp;
		bm = bp/bpp;
		az = app/bpp;
		bz = one;
		if(ABS(az-aold) < eps*ABS(az)) break;
		}
	return az;
	}//BetaContinuedFraction(long double, long double, long double)


/*
long double RandomBeta(long double alpha, long double beta, Ran3& ran){
	long double x1 = iRandomGamma(alpha,one);
	long double x2 = iRandomGamma(beta,one);
	return x1/(x1+x2);
	}//end RandomBeta(long double, long double, Ran3&)

*/





#endif

