/***************************************************************
*
*
*		Bisection_GoldenRatioFns.cpp
*
*		Bisection narrows in on a value along a line, including root-finding in 1 dimension.
*		GoldenRatio finds the minimum or maximum of a function in 1 dimension.
*
*		From Press et al. 1992 Numerical recipes in C
*
****************************************************************/



#ifndef Bisection_GoldenRatioFnscpp
#define Bisection_GoldenRatioFnscpp

#include "Bisection_GoldenRatioFns.h"
#include "Element.cpp"
#include "aList.cpp"


long double Bisection_GoldenRatioFns::FindRootViaBisection(long double x1, long double x2, aList<long double>& variables){
		//the function returns a point between x1 and x2 to tolerance tol, using variables in variableList
	int j, JMAX=40;
	long double dx, y, ymid, xmid, rtb=zero;
	y = func(x1, variables);
	ymid = func(x2, variables);
	if(y*ymid>=zero){
			aError("ERROR: FindRootViaBisection(...): Root not properly bracketed for bisection.");}
		else{
			if(y<zero){
					dx = x2-x1;
					rtb=x1;}
				else{
					dx=x1-x2;
					rtb=x2;}
			for(j=0;j<JMAX;++j){
				dx *= half;
				xmid = rtb+dx;
				ymid = func(xmid,variables);
				if(ymid<=zero){
					rtb=xmid;}
				if(ABS(dx)<zero || ymid ==zero){
					break;}
				}//j
			}//no bracketing error
	return rtb;
	}//end FindPointViaBisection




long double Bisection_GoldenRatioFns::FindPointViaBisection(long double yTarget, long double x1, long double x2, aList<long double>& variables){
		//the function returns a point between x1 and x2 to tolerance tol, using variables in variableList
		//before calling, it is critical identify which section of the curve to investigate, as y may occure at more than one x vbalue n a curve
	int j, JMAX=40;
	long double dx, y, ymid, xmid, rtb=zero;
	y = func(x1, variables);
	ymid = func(x2, variables);
//	if(y*ymid>=zero){
//			aError("ERROR: FindPointViaBisection(...): Point not properly bracketed for bisection.");}
//		else{
			if(y<yTarget){
					dx = x2-x1;
					rtb=x1;}
				else{
					dx=x1-x2;
					rtb=x2;}
			for(j=0;j<JMAX;++j){
				dx *= half;
				xmid = rtb+dx;
				ymid = func(xmid,variables);
				if(ymid<=yTarget){
					rtb=xmid;}
				if(ABS(dx)<zero || ymid ==yTarget){
					break;}
				}//j
//			}//no bracketing error
	return rtb;
	}//end FindPointViaBisection




	//   !!!!!    NOT WORKING SATISFACTORALLY YET    !!!!!     
long double Bisection_GoldenRatioFns::FindMinMaxViaGoldenRatio(bool minimize, long double ax, long double bx, long double cx, 
										long double tol, long double& xmin, aList<long double>& variables){
					//minimize=true to minimize the function, minimize=false to maximize the function
		//the function returns the min/max between ax and cx, with ax<bx<cx, using variables in variableList
	long double y1,y2,y1Prime, y2Prime, x0=ax,x1,x2,x3=cx, ymin, f_sign;
	minimize ? f_sign=one : f_sign= -one;
	long double R = GoldenRatio()-one;
	long double C = one - R;
	
	if(ABS(cx-bx)>ABS(bx-ax)){
			x1=bx;
			x2 = bx + C*(cx-bx);}
		else{
			x2=bx;
			x1 = bx - C*(bx-ax);}
//	y1 = func(x1, variables);
//	y2 = func(x2,variables);
	y1 = f_sign * func(x1, variables);
	y2 = f_sign * func(x2, variables);
	while(ABS(x3-x0) > tol*(ABS(x1)+ABS(x2))){
		if(y2 < y1){
				Shift3(x0,x1,x2,R*x1 + C*x3);
//				y2Prime = func(x2,variables);
				y2Prime = f_sign * func(x2,variables);
				Shift2(y1,y2,y2Prime);
				}
			else{
				Shift3(x3,x2,x1,R*x2 + C*x0);
//				y1Prime = func(x1,variables);
				y1Prime = f_sign * func(x1,variables);
				Shift2(y2,y1,y1Prime);
				}
		}//while
	if(y1<y2){
			xmin = x1;
			ymin = y1;}
		else{
			xmin = x2;
			ymin = y2;}
	return f_sign * ymin;
	}// end Bisection_GoldenRatioFns::FindMinMaxViaGoldenRatio












#endif