/*********************************************
*
*
*		ChiSqDistribution.cpp
*
*
*		Functions for calculating Chi-squared-distribution statistics
*
*
**********************************************/


#ifndef ChiSqDistributioncpp
#define ChiSqDistributioncpp = 1

#include "ChiSqDistribution.h"


long double ChiSqDistribution(	const long double x, 
							const long double mu, 
							const long double sigma){
		//calculates the height of N(mu,sigma) at point x
//		const long double pi = 3.141592653589793238462643383;
		long double sqrt2pi = sigma * sqrt( two * PI());
		if(x == mu){return one/sqrt2pi;}
		long double dev = (x - mu) / sigma;
		if(dev*dev/two==zero){return one/sqrt2pi;}
		long double exponent = - (dev * dev / two);
		long double f = exp(exponent);
//		long double s = (sigma * sqrt2pi);
		long double s = sqrt2pi;
		return f/s;
		}


long double ChiSqProbDensityFunction(const int df, const long double x){
	//height of function at x
	long double pdf = zero;
	if(x>zero){
		long double dfdbl = (long double) df;
		long double lnpdf = (dfdbl/two - one)*log(x) - x/two - (dfdbl/two)*log(two) - gammaln(dfdbl/two);
		pdf = exp(lnpdf);
		}
	return pdf;
	}//ChiSqCumDensityFunction(const int, const long double)


long double ChiSqCumDensityFunction(const int df, const long double x){
	//integral from 0 to x
	long double dfdbl = (long double) df;
	long double cdf = GammaRegularized(dfdbl/two, zero, x/two);
	return cdf;
	}//ChiSqCumDensityFunction(const int, const long double)



long double ChiSqDistributionPvalue(const int df, const long double x){
	//integral from x to infinity
	long double pval = one - ChiSqProbDensityFunction(df,x);
	return pval;
	}//ChiSqProbDensityFunction(const int, const long double)


long double ChiSquareRandomValue(const int df, Ran3& ran){
	long double ranval = zero;
	long double dfdbl = (long double)df, a;
	if(df == 1){
	          ranval = two* gsGamma(dfdbl/two, df/two/E(), ran);
	          }
		else if(df == 2){
		  		ranval = two *  (-log(ran()));
				}
	    else if(df < 5){
	          a = dfdbl/two - one;
	          ranval = two * a * gkm1Gamma((dfdbl/two - one/(three * dfdbl))/a, two/a, ran);
	          }
	    else{
	          a = dfdbl/two - one;
	          ranval = two * a * gkm2Gamma(one/sqrt(dfdbl/two), (dfdbl/two - one/(three * dfdbl))/a, two/a, ran);
	          }
	return ranval;
	}


long double gsGamma(const long double alpha, const long double r, Ran3& ran){
	long double x = one, t = one;
	long double q = (one + r) * ran();
	long double gsg = zero;
	if(q < one){
			x = exp(- alpha * log(q));  
			t = exp(-x);}
		else{
             x = one - log(one + (one-q)/r);
             t = exp((alpha - one)*log(x));
             }
     while(ran() > t){
		q = (one + r) * ran();
		if(q < one){
				x = exp(-alpha*log(q));  
				t = exp(-x);}
			else{
				x = one - log(one + (one-q)/r);
				t = exp((alpha-one)*log(x));
				}
		}//while
	return x;
	}
        

long double gkm1Gamma(const long double b, const long double c, Ran3& ran){
   long double u1 = ran(), u2 = ran();
   long double w = b * u1/u2;
   while( (c* u2 - c - two + w + one/w > zero) && (c * log(u2) - log(w) + w - one >= zero) ){
		u1 = ran();  
		u2 =ran();  
		w = b * u1/u2;
		}//while
	return w;
	}
   

long double gkm2Gamma(const long double k, const long double b, const long double c, Ran3& ran){
	long double u1 = ran();
	long double u2 = u1 + k * (one - (long double)1.86 * ran());
	while(!(zero < u2 && u2 < one)){
	    u1 = ran();  
	    u2 = u1 + k * (one - (long double)1.86 * ran());
	    }
    long double w = b * u1/u2;
	while((c * u2 - c - two + w + one/w > zero) && (c * log(u2) - log(w) + w - one >= zero)){
        u1 = ran();
        u2 = u1 + k * (one - (long double)1.86 * ran());
        while(!(zero < u2 && u2 < one) ){
              u1 = ran();
              u2 = u1 + k * (one - (long double)1.86 * ran());
              }
        w = b * u1/u2;
        }
	return w;
	}


#endif

