/******************************************************
*
*	CubicBezierSpline.cpp
*
*	Parameters for a normal distribution defined by mean and stdev.
*
*	Use this as a base class for derived classes that add parameters.
*
******************************************************/
#ifndef CubicBezierSplinecpp
#define CubicBezierSplinecpp = 1

#include "CubicBezierSpline.h"
#include "Element.cpp"
#include "aList.cpp"
#include "CoordinateClass.cpp"
#include "dataColumnRecNum.cpp"


CubicBezierSpline::CubicBezierSpline(bool deleteContentsOnDisposal): usingStored_x_(false), //curveAtKnown_x_Current(false),
																	deleteContentsOnDisposal_(deleteContentsOnDisposal){}


CubicBezierSpline::CubicBezierSpline(int numNodes, bool deleteContentsOnDisposal)://empty nodes & sections
  					xL_(zero,zero,zero),xR_(zero,zero,zero), usingStored_x_(false), //curveAtKnown_x_Current(false),
  					deleteContentsOnDisposal_(deleteContentsOnDisposal){
  				int i;
  				for(i=0;i<numNodes;++i){
  					nodeList_.Append(NEW BezierNode());}
  				for(i=0;i<numNodes-1;++i){
  					sectionList_.Append(NEW CubicBezierSplineSection(&(nodeList_[i]), &(nodeList_[i+1]), i));}
  				SetSplineConnections();
  				}


CubicBezierSpline::CubicBezierSpline(ObjList<BezierNode>& sortedNodes, bool deleteContentsOnDisposal):
  					xL_(sortedNodes[0].coords_),xR_(sortedNodes.Last().coords_), 
  					usingStored_x_(false), //curveAtKnown_x_Current(false), 
  					deleteContentsOnDisposal_(deleteContentsOnDisposal){
  				int i;
  				nodeList_ = sortedNodes;
  				for(i=0;i<nodeList_.len()-1;++i){
  					sectionList_.Append(NEW CubicBezierSplineSection(&(nodeList_[i]), &(nodeList_[i+1]), i));}
  				SetSplineConnections();
  				}


CubicBezierSpline::CubicBezierSpline(ObjList<BezierNode>& sortedNodes,
							 aList<long double>& x_values, 
							 bool deleteContentsOnDisposal):
  					xL_(sortedNodes[0].coords_),xR_(sortedNodes.Last().coords_), 
  					usingStored_x_(true), //curveAtKnown_x_Current(false), 
  					deleteContentsOnDisposal_(deleteContentsOnDisposal){
  				//x_values--> indexed [ordered datum] ->holds x 
  				nodeList_ = sortedNodes;
  				int i, j, sec=0;
  				long double x;
  				for(i=0;i<nodeList_.len()-1-1;++i){
  					sectionList_.Append(NEW CubicBezierSplineSection(&(nodeList_[i]), &(nodeList_[i+1]), i));
  					aList<long double> theseXvals;
  					for(j=0;j<x_values.len();++j){
  						x = x_values[j];
  						if(sectionList_[i].xIsInSection(x) && !(theseXvals.Includes(x))){
  							theseXvals.Append(x);}
  						}//j
  					theseXvals.Sort();
  					sectionList_[i].SetXvalues(theseXvals);
  					}//
  				SetSplineConnections();
//				long double x = xL_[0];
//				for(i=0;i<sectionList_.len();++i){
//					curveAtKnown_x.Append(NEW ObjList<CoordinateClass<long double> >());}
			//	curveAtKnown_x[0].Append(NEW CoordinateClass<long double>(x,zero));
//				for(i=0;i<x_values.len();++i){
//					sec=SectionAt_x(x_values[i]);
//					curveAtKnown_x[sec].Append(NEW CoordinateClass<long double>(x_values[i],zero));
//  					}//i
  				SetStoredValues();
  				}




CubicBezierSpline::CubicBezierSpline(CubicBezierSpline& spline, bool deleteContentsOnDisposal){
				Copy(spline, deleteContentsOnDisposal);
				}



CubicBezierSpline::~CubicBezierSpline(void){
  				int sec;
  				for(sec=0;sec<sectionList_.len();++sec){
  					sectionList_[sec].SetNodeLeft(NULL);
  					sectionList_[sec].SetNodeRight(NULL);
  					}
  				sectionList_.Clear();//order critical -- otherwise disposes of nodes twice
//  				if(deleteContentsOnDisposal_){
  						nodeList_.Clear();
//  						}
//  					else{
//  						nodeList_.Detach();
//  						}
  				}//CubicBezierSpline::~CubicBezierSpline(



CubicBezierSpline* CubicBezierSpline::Duplicate(void){
	return NEW CubicBezierSpline(*this, true);
	}//end CubicBezierSpline::Duplicate()




void CubicBezierSpline::operator=(CubicBezierSpline& spline){
	//special copy of contents because of node-section dependency
	Copy(spline, true);
	}//end  CubicBezierSpline::operator=(ParamRec&)



void CubicBezierSpline::Copy(CubicBezierSpline& spline, bool deleteContentsOnDisposal){
	//special copy of contents because of node-section dependency
	//need to acknowledge that a memory leak can occur, and code to eliminate it
	int i, sec;
	deleteContentsOnDisposal_=deleteContentsOnDisposal;
	for(sec=0;sec<sectionList_.len();++sec){
		sectionList_[sec].SetNodeLeft(NULL);
		sectionList_[sec].SetNodeRight(NULL);
		}
	sectionList_.Clear();//order critical -- otherwise disposes of nodes twice
	nodeList_.Clear();
	for(i=0;i<spline.nodeList_.len();++i){
		nodeList_.Append(NEW BezierNode(spline.nodeList_[i]));}
	for(i=0;i<nodeList_.len()-1;++i){
		sectionList_.Append(NEW CubicBezierSplineSection(&(nodeList_[i]), &(nodeList_[i+1]),deleteContentsOnDisposal));}
  	xL_ = spline.xL_;
  	xR_ = spline.xR_;
  	usingStored_x_ = spline.usingStored_x_;
//  	curveAtKnown_x_Current = spline.curveAtKnown_x_Current;
//  	curveAtKnown_x = spline.curveAtKnown_x;
	}//end  CubicBezierSpline::operator=(ParamRec&)



void CubicBezierSpline::ChangeSplineParam(int node, int paramNum, long double newVal){
	nodeList_[node].SetParamValue(paramNum,newVal);
	}//end CubicBezierSpline::ChangeSplineParam(int)





long double CubicBezierSpline::ExpectedValue(aList<long double>& coords, long double toleranceFactor){
	return FunctionValue(coords[0],toleranceFactor);
	}//end CubicBezierSpline::ExpectedValue(aList<double>&)


long double CubicBezierSpline::FunctionValue(long double x, long double toleranceFactor){
	int sec = SectionAt_x(x);
	long double yPrime=zero;
	CoordinateClass<long double> pointOnCurve(x,zero,zero);
	GetCoords(sec,x,pointOnCurve,toleranceFactor);
	yPrime = pointOnCurve[1];
	return yPrime;
	}//CubicBezierSpline::FunctionValue(long double, long double)



void CubicBezierSpline::GetCoords(int section, long double x, CoordinateClass<long double>& pointOnCurve, long double toleranceFactor){
	//sets pointOnCurve
	pointOnCurve[0]=x;
	sectionList_[section].bezierValue(x,pointOnCurve,toleranceFactor);
	}//CubicBezierSpline::GetStoredCoords(int,long double, CoordinateClass<long double>&)



void CubicBezierSpline::MultiplyParam(int node, int paramAtNode, long double factor){
	//use to rescale a param deviation
	nodeList_[node].MultiplyParam(paramAtNode,factor);
	}//CubicBezierSpline::MultiplyParam(int, int, double)


int CubicBezierSpline::NumParams(void){
	//number of parameters -- default as some may be held constant
	int num= 5 * nodeList_.len() - 2;//2 distances past ends not relevant
	return num;
	}//CubicBezierSpline::NumParams()


aString* CubicBezierSpline::ParamName(int node, int paramAtNode){
	aString *returnStr=nodeList_[node].ParamName(paramAtNode);
	returnStr->prepend(".");
	returnStr->prepend(node);
	returnStr->prepend("node_");
	return returnStr;
	}//end CubicBezierSpline::ParamName(int, int)


void CubicBezierSpline::ParamName(int node, int paramAtNode, aString& appendToThis){
	nodeList_[node].ParamName(paramAtNode,appendToThis);
	}//end CubicBezierSpline::ParamName(int, int, aString&)



void CubicBezierSpline::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	//appended to dataline
	int node, pp;
	for(node=0; node<nodeList_.len();++node){
		for(pp=0;pp<nodeList_[node].NumParams();++pp){
			if(!(node==0 && pp==3)  && !(node==nodeList_.len()-1 && pp==4)){
				dataline.cat(sepchar);
				dataline.cat("node_");
				dataline.cat(node);
				dataline.cat(".");
				nodeList_[node].ParamName(pp,dataline);
				}//if
			}//pp
		}//node
	}//CubicBezierSpline::ParamString(aString&, char)


long double CubicBezierSpline::ParamValue(int node, int paramAtNode){
	//returns value of parameter	
	long double val = nodeList_[node].ParamValue(paramAtNode);
	return val;
	}//end CubicBezierSpline::ParamValue(int, int )



void CubicBezierSpline::Print(ostream& outFile, aString& message, char* sepchar){
	aString outStr;
	outFile<<message<<endl;
	int node,pp, params = nodeList_[0].NumParams();
	for(node=0;node<nodeList_.len();++node){
		outStr.clear();
		outStr.cat(node);
		for(pp=0;pp<params;++pp){
			outStr+sepchar;
			outStr+nodeList_[node].ParamValue(pp);}
		outFile<<outStr<<endl;
		}
	}//CubicBezierSpline::Print(ostream&, aString&, char*)


void CubicBezierSpline::RecalculateYvalues(int section){
	sectionList_[section].SetSplineYforXinDataCurrent(false);
	}
	
	
void CubicBezierSpline::RecalculateYvalues(aList<int>& sectionsAffected){
	int s;
	for(s=0;s<sectionsAffected.len();++s){
		RecalculateYvalues(s);}
	}



int	CubicBezierSpline::SectionAt_x(long double x){
	int sec= -1, i;
	if(x<xL_[0] || x>xR_[0]){
			aError("ERROR: CubicBezierSplineParamRec::SectionAt_x() -- x is out of range");}
		else{
			for(i=0;i<sectionList_.len();++i){
				if(sectionList_[i].xIsInSection(x)){
					sec=i;
					break;}
				}
			}
	return sec;
	}//CubicBezierSpline::SectionAt_x
	
	
void CubicBezierSpline::SectionsAtNodes(aList<int>& nodePositions, aList<int>& newSectionList){
	int nn,sec;
	newSectionList.Clear();
	for(nn=0;nn<nodePositions.len();++nn){
		BezierNode& node = nodeList_[nodePositions[nn]];
		if(node.section0_ != NULL){
			sec = sectionList_.Loc(node.section0_);
			if(! newSectionList.Includes(sec)){
				newSectionList.Append(sec);}
			}
		if(node.section1_ != NULL){
			sec = sectionList_.Loc(node.section1_);
			if(! newSectionList.Includes(sec)){
				newSectionList.Append(sec);}
			}
		}//nn
	}//CubicBezierSpline::SectionsAtNodes(aList<int>&, aList<int>&)
		



void CubicBezierSpline::SetMathematicaFormString(aString& stringToFill){
	//used in a Mathematica notebook to print a spline graphic
	aString nodeString;
	stringToFill = "{";
	int node;
	nodeList_[0].SetMathematicaFormString(nodeString);
	stringToFill.cat(nodeString);
	for(node=1;node<nodeList_.len();++node){
		nodeList_[node].SetMathematicaFormString(nodeString);
		stringToFill.cat(", ");
		stringToFill.cat(nodeString);
		}
	stringToFill.cat("}");
	}//CubicBezierSpline::SetMathematicaFormString(aString& )



void CubicBezierSpline::SetMathematicaFormStringList(StringList& stringListToFill){
	//used in a Mathematica notebook to print a spline graphic
	stringListToFill.Clear();
	int node;
//	nodeList_[0].SetMathematicaFormString(stringListToFill);
//	stringListToFill.cat(nodeList_[0]);
	for(node=0;node<nodeList_.len();++node){
		stringListToFill.Append(NEW aString());
		if(node==0){
			stringListToFill[node].cat("{");}
		nodeList_[node].SetMathematicaFormString(stringListToFill[node]);
		if(node==nodeList_.len()-1){
				stringListToFill[node].cat("}");}
			else{
				stringListToFill[node].cat(",");}
		}//node
	}//CubicBezierSpline::SetMathematicaFormString(aString& )



void CubicBezierSpline::SetParamValue(int node, int paramAtNode, long double newVal){
	//set value of parameter
	nodeList_[node].SetParamValue(paramAtNode,newVal);
	}//end CubicBezierSpline::SetParamValue(int,double)



void CubicBezierSpline::SetParamValueAllNodes(int paramAtNode, long double newVal){//use for setting deviations & upper/lower limits
	//set value of parameter
	int node;
	for(node=0;node<nodeList_.len();++node){
		nodeList_[node].SetParamValue(paramAtNode,newVal);}			
	}//end CubicBezierSplineParamRec::SetParamValue(int,double)



void CubicBezierSpline::SetSplineConnections(void){
	//hooks up sections & set terminal nodes
	nodeList_.First().SetTerminalLeft(true);
	nodeList_.Last().SetTerminalRight(true);
	int sec;
	for(sec=0;sec<sectionList_.len();++sec){
		sectionList_[sec].SetSpline(this);}
	sectionList_.First().SetSectionLeft(NULL);
	sectionList_.Last().SetSectionRight(NULL);
	for(sec=0;sec<sectionList_.len()-1;++sec){
		CubicBezierSplineSection& nextsec = sectionList_[sec+1];
		sectionList_[sec].SetSectionRight(&nextsec);
		}
	for(sec=1;sec<sectionList_.len();++sec){
		CubicBezierSplineSection& nextsec = sectionList_[sec-1];
		sectionList_[sec].SetSectionLeft(&nextsec);
		}
	}//CubicBezierSpline::SetSplineConnections()



void CubicBezierSpline::SetStoredValues(void){//when parameters change
	int s;
	aList<int> sectionsAffected;
	for(s=0;s<sectionList_.len();++s){
		sectionsAffected.Append(s);}
	SetStoredValues(sectionsAffected);
	}//CubicBezierSpline::SetStoredValues()
	
	
void CubicBezierSpline::SetStoredValues(int node){//when parameters change at that node
	aList<int> nodes,sections;
	nodes.Append(node);
	SectionsAtNodes(nodes,sections);
	SetStoredValues(sections);
	}

void CubicBezierSpline::SetStoredValues(BezierNode& node){//when parameters change at that node
	SetStoredValues(nodeList_.Loc(&node));
	}
	
	
void CubicBezierSpline::SetStoredValues(aList<int>& sectionsAffected){
	int s, sec;
	for(s=0;s<sectionsAffected.len();++s){
		sec = sectionsAffected[s];
		sectionList_[sec].SetSplineYforXinDataCurrent(false);
		}
	}



void CubicBezierSpline::SetStoredValuesLists(aList<long double>& xValues){
	int i,sec;
	long double x;
	ObjList<aList<long double> > xBySection;
	for(sec=0;sec<sectionList_.len();++sec){
		xBySection.Append(NEW aList<long double>());}
	for(i=0;i<xValues.len();++i){
		x = xValues[i];
		sec = SectionAt_x(x);
		if(! (xBySection[sec].Includes(x))){
			xBySection[sec].Append(x);}
		}
	
	for(sec=0;sec<sectionList_.len();++sec){
		xBySection[sec].Sort();
		sectionList_[sec].SetXvalues(xBySection[sec]);
		}
	}//CubicBezierSpline::SetStoredValuesLists(aList<long double>&)
	
	
	
void CubicBezierSpline::SetStoredValuesLists(dataColumnRecNum<long double>& xValues){
	int i,sec;
	long double x;
	ObjList<aList<long double> > xBySection;
	for(sec=0;sec<sectionList_.len();++sec){
		xBySection.Append(NEW aList<long double>());}
	for(i=0;i<xValues.len();++i){
		x = xValues[i];
		sec = SectionAt_x(x);
		if(! (xBySection[sec].Includes(x))){
			xBySection[sec].Append(x);}
		}
	
	for(sec=0;sec<sectionList_.len();++sec){
		xBySection[sec].Sort();
		sectionList_[sec].SetXvalues(xBySection[sec]);
		}
	}//CubicBezierSpline::SetStoredValuesLists(dataSetRecNum<long double>&)
	
	



void CubicBezierSpline::SetToleranceFactor(long double newFactor){
	int s;
	for(s=0;s<sectionList_.len();++s){
		sectionList_[s].CheckToleranceFactor(newFactor);
		}
	}

void CubicBezierSpline::SetYminMax(long double ymin, long double ymax){
	int sec;
	for(sec=0;sec<sectionList_.len();++sec){
		sectionList_[sec].SetYminMax(ymin,ymax);}
	}


void CubicBezierSpline::SetYminMax(aList<long double>& ymin, aList<long double>& ymax){
	int sec;
	if(ymin.len() != sectionList_.len() || ymax.len() != sectionList_.len()){
		aError("ERROR: CubicBezierSpline::SetYminMax: ymin & ymax lists must match number of sections.");}
	for(sec=0;sec<sectionList_.len();++sec){
		sectionList_[sec].SetYminMax(ymin[sec],ymax[sec]);}
	}


/*
void CubicBezierSpline::SetStoredValues(ObjList<CubicBezierSplineSection>& sectionsAffected){
	//when parameters change in these sections
	if(usingStored_x_){
		int s,sec,d;
		long double tolerance = (long double)0.001;
		for(s=0;s<sectionsAffected.len();++s){
			CubicBezierSplineSection& section = sectionsAffected[s];
			sec = sectionList_.Loc(&section);
			ObjList<CoordinateClass<long double> >& sectionCurve = curveAtKnown_x[sec];
			for(d=0;d<sectionCurve.len();++d){
				section.t_from_x(sectionCurve[d][0], sectionCurve[d],tolerance);
				}//d
			}//s
		}
	}//CubicBezierSpline::SetStoredValues(ObjList<CubicBezierSplineSection>& )
*/				




int	operator==(CubicBezierSpline& p, CubicBezierSpline& q){
	return ((p.sectionList_==q.sectionList_) && (p.nodeList_==q.nodeList_)
				&& (p.usingStored_x_==q.usingStored_x_));
	}//end operator==(CubicBezierSpline&, CubicBezierSpline&)


int	operator!=(CubicBezierSpline& p, CubicBezierSpline& q){
	return !(p==q);
	}//end operator!=(CubicBezierSpline&, CubicBezierSpline&)



#endif


