/******************************************************
*
*	CubicBezierSplineSection.cpp
*
*	A class holding the parameters and functions required to describe a section between 2 nodes of cubic Bezier curve,
*
*	Use use with a container class that holds a separate list of nodes, such as CubicBezierSplineParamRec
*
******************************************************/

#ifndef CubicBezierSplineSectioncpp
#define CubicBezierSplineSectioncpp = 1


#include "CubicBezierSplineSection.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "CoordinateClass.cpp"

	
CubicBezierSplineSection::CubicBezierSplineSection(int sectionID,bool deleteContentsOnDisposal):node0_(NULL), node1_(NULL), sectionPrev_(NULL), spline_(NULL), 
					sectionNext_(NULL), bezierParamsCurrent_(false),ID_(sectionID),
					splineYforXinDataCurrent_(false), toleranceFactor_((long double)0.001), yMin_(-one), yMax_(one),
  					//lnL_section_(one), lnLmax_section_(one),lnL_current_(false), 
  					deleteContentsOnDisposal_(deleteContentsOnDisposal)//,ymin_(zero),ymax_(zero),yMinMaxCurrent_(false)
  					{
  		int p;
  		for(p=0;p<4;++p){
  			cp_.Append(NEW CoordinateClass<long double>());}
  		}

CubicBezierSplineSection::CubicBezierSplineSection(BezierNode *node0, BezierNode *node1, 
							int sectionID, bool deleteContentsOnDisposal):
  					node0_(node0), node1_(node1), sectionPrev_(NULL), sectionNext_(NULL),  spline_(NULL), 
  					bezierParamsCurrent_(false), ID_(sectionID),
  					splineYforXinDataCurrent_(false), toleranceFactor_((long double)0.001), yMin_(-one), yMax_(one),
  					deleteContentsOnDisposal_(deleteContentsOnDisposal)//,ymin_(zero),ymax_(zero),yMinMaxCurrent_(false)
  					//lnL_section_(one), lnLmax_section_(one),lnL_current_(false)
  					{
  		int p;
  		for(p=0;p<4;++p){
  			cp_.Append(NEW CoordinateClass<long double>());}
  		if(node0 != NULL){
  			node0_->SetSectionRight(this);}
  		if(node1 != NULL){
  			node1_->SetSectionLeft(this);}
  		if(node0_ != NULL  && node1_ != NULL){
  			SetBezierParams();}
  		}


CubicBezierSplineSection::CubicBezierSplineSection(BezierNode *node0, BezierNode *node1,
  								CubicBezierSplineSection *section0, CubicBezierSplineSection *section1, 
  								int sectionID, bool deleteContentsOnDisposal):
  					node0_(node0), node1_(node1), sectionPrev_(section0), sectionNext_(section1),  spline_(NULL), 
  					yMin_(-one), yMax_(one), deleteContentsOnDisposal_(deleteContentsOnDisposal), 
  					//lnL_section_(one), lnLmax_section_(one),lnL_current_(false)
  					ID_(sectionID), splineYforXinDataCurrent_(false), toleranceFactor_((long double)0.001)//,
  					//ymin_(zero),ymax_(zero),yMinMaxCurrent_(false)
  					{
  		int p;
  		for(p=0;p<4;++p){
  			cp_.Append(NEW CoordinateClass<long double>());}
  		if(node0_ != NULL  && node1_ != NULL){
  			SetBezierParams();}
  		}


CubicBezierSplineSection::CubicBezierSplineSection(CubicBezierSplineSection& cbss, bool deleteContentsOnDisposal){
				Copy(cbss, deleteContentsOnDisposal);
				}

CubicBezierSplineSection::~CubicBezierSplineSection(void){
				if(deleteContentsOnDisposal_){
		  				delete node0_;
		  				delete node1_;
		  				node0_ = node1_ = NULL;
		  				}
		  			else{
		  				node0_ = node1_ = NULL;}//forget
  				sectionPrev_ = sectionNext_ = NULL;//forget
  				spline_ = NULL;
  				cp_.Clear();
  				realizedRange_.Clear();
  				}
  				
  				
CubicBezierSplineSection* CubicBezierSplineSection::Duplicate(void){
	return NEW CubicBezierSplineSection(*this, true);}
	
	
void CubicBezierSplineSection::operator=(CubicBezierSplineSection& cbss){
			Copy(cbss,true);
			}


void CubicBezierSplineSection::Copy(CubicBezierSplineSection& cbss, bool deleteContentsOnDisposal){
	deleteContentsOnDisposal_ = deleteContentsOnDisposal;
	splineYforXinDataCurrent_ = cbss.splineYforXinDataCurrent_;
	toleranceFactor_ = cbss.splineYforXinDataCurrent_;
	splineXvals_ = cbss.splineXvals_;
	splineYforXinData_ = cbss.splineYforXinData_;
	yMin_ = cbss.yMin_;
	yMax_ = cbss.yMax_;
	xL_=cbss.xL_;
	xR_=cbss.xR_;
	cp_=cbss.cp_;
	realizedRange_=cbss.realizedRange_;
	realizedRangeCurrent_ = cbss.realizedRangeCurrent_;
	toleranceFactor_ = cbss.toleranceFactor_;
	ID_=cbss.ID_;
//	ymin_=cbss.ymin_;
//	ymax_=cbss.ymax_;
//	yMinMaxCurrent_=cbss.yMinMaxCurrent_;
	quadraticSolutions_ = cbss.quadraticSolutions_;
	if(deleteContentsOnDisposal){
			node0_= cbss.node0_->Duplicate();
			node1_= cbss.node1_->Duplicate();
			sectionPrev_= NULL;
			sectionNext_= NULL;
			node0_->SetSectionLeft(this);
			node1_->SetSectionRight(this);
			spline_= NULL;
			bezierParamsCurrent_= false;
			}
		else{
			node0_=cbss.node0_;
			node1_=cbss.node1_;
			sectionPrev_=cbss.sectionPrev_;
			sectionNext_=cbss.sectionNext_;
			spline_=cbss.spline_;
			bezierParamsCurrent_ = cbss.bezierParamsCurrent_;
			}
	}//CubicBezierSplineSection::Copy(CubicBezierSplineSection&, bool ) 	


void CubicBezierSplineSection::CopyDataFieldsOnly(CubicBezierSplineSection& cbss){
	splineYforXinDataCurrent_ = cbss.splineYforXinDataCurrent_;
	toleranceFactor_ = cbss.splineYforXinDataCurrent_;
	splineXvals_ = cbss.splineXvals_;
	splineYforXinData_ = cbss.splineYforXinData_;
	yMin_ = cbss.yMin_;
	yMax_ = cbss.yMax_;
	xL_=cbss.xL_;
	xR_=cbss.xR_;
	cp_=cbss.cp_;
	realizedRange_=cbss.realizedRange_;
	realizedRangeCurrent_ = cbss.realizedRangeCurrent_;
	toleranceFactor_ = cbss.toleranceFactor_;
	ID_=cbss.ID_;
	quadraticSolutions_ = cbss.quadraticSolutions_;
	}//CubicBezierSplineSection::Copy(CubicBezierSplineSection&, bool ) 	



void CubicBezierSplineSection::DetachPointers(void){
	//detaches but does not destroy nodes, spline, & adjacent sections
	DetachSpline();
	DetachNodes();
	DetachSections();
	}


void CubicBezierSplineSection::DetachSpline(void){
	//detaches but does not destroy spline only
	spline_ = NULL;
	}
	
	
void CubicBezierSplineSection::DetachNodes(void){
	//detaches but does not destroy nodes only
	node0_ = node1_ = NULL;
	}
	
	
void CubicBezierSplineSection::DetachSections(void){
	//detaches but does not destroy adjacent sections only
	sectionPrev_ = sectionNext_ = NULL;
	}








long double CubicBezierSplineSection::bezierValue(long double t, long double a, long double b, long double c, long double d){
	//determines x or y at position t on the curve, depending on values in a, b, c, d
	long double val;
	val = a*t*t*t + b*t*t + c*t + d;
	return val;
	}//CubicBezierSplineSection::bezierValue(long double, long double, long double, long double, long double)



long double CubicBezierSplineSection::bezierValue(long double t, aList<long double>& abcd){
	//determines x or y at position t on the curve, depending on values in abcd
	long double val = zero;
	if(abcd.len()>=4){
			val = bezierValue(t,abcd[0],abcd[1],abcd[2],abcd[3]);}
		else{
			aError("ERROR: CubicBezierSplineSection::bezierValue(t,abcd) needs  4 variables in its parameter list");}
	return val;
	}//CubicBezierSplineSection::bezierValue(long double , aList<long double>& )





long double CubicBezierSplineSection::bezierValue(long double x, CoordinateClass<long double>& coordsOnCurve, long double toleranceFactor){
	//determines {x,y} at position x on the curve; returns vals in coordsOnCurve
	long double y = cp_[0][1];//dummy
	coordsOnCurve[0] = x;
	if(xIsInSection(x)){
		CheckToleranceFactor(toleranceFactor);
		if(! splineYforXinDataCurrent_){
			CalculateBezierValues(toleranceFactor);}
		if(splineXvals_.Includes(x)){
				y = GetTabledYfromX(x);}
			else{	
				y=CalculateBezierValue(x, toleranceFactor);}	
		}
	coordsOnCurve[1] = y;
	return y;
	}//CubicBezierSplineSection::bezierValue(long double , CoordinateClass<long double>& )
	

	
	
/*
bool CubicBezierSplineSection::bezierValuesInRange(void){
	//checks derivatives to see if all portions of the curve are in range
	//THIS METHOD FAILS
	bool allLegal=true,legal1=false,legal2=false, ignore1=false, ignore2=false;
	long double x1, x2, t1=zero, t2=zero, y1, y2, ax, bx, cx, ay, by, cy;
	realizedRange_.Clear();
	cx = three * (cp_[1][0] - cp_[0][0]);
	bx = three * (cp_[2][0] - cp_[1][0]) - cx;
	ax = cp_[3][0] - cp_[0][0] - bx - cx;
	cy = three * (cp_[1][1] - cp_[0][1]);
	by = three * (cp_[2][1] - cp_[1][1]) - cy;
	ay = cp_[3][1] - cp_[0][1] - by - cy;
	if(ay != zero){
		t1 = (-by - sqrt(by*by - three * ay * cy))/(three*ay);//t at derivative = 0 on y axis
		t2 = (-by + sqrt(by*by - three * ay * cy))/(three*ay);//t at derivative = 0 on y axis
		}
	y1 = ay*t1*t1*t1 + by*t1*t1 + cy*t1 + cp_[0][1];//corresponding y
	y2 = ay*t2*t2*t2 + by*t2*t2 + cy*t2 + cp_[0][1];
	x1 = ax*t1*t1*t1 + bx*t1*t1 + cx*t1 + cp_[0][0];//corresponding x
	x2 = ax*t2*t2*t2 + bx*t2*t2 + cx*t2 + cp_[0][0];
	if(y1<=yMax_ && y1>=yMin_){
		if(xIsInSection(x1)){
				realizedRange_.Append(y1);
				legal1=true;}
			else{ 
				ignore1=true;}
		}
	if(y2<=yMax_ && y2>=yMin_){
		if(xIsInSection(x2)){
				realizedRange_.Append(y2);
				legal2=true;}
			else{ 
				ignore2=true;}
		}

	if(ignore1 && ignore2){}//none legal
		else if(ignore1 && !ignore2){
			if(legal2){
				allLegal=true;}
			}
		else if(ignore2 && !ignore1){
			if(legal1){
				allLegal=true;}
			}
		else{
			if(legal1 && legal2){
				allLegal=true;}
			}
	realizedRangeCurrent_=true;
	if(realizedRange_.len()==0){
		allLegal=false;}
	return allLegal;
	}//CubicBezierSplineSection::bezierValuesInRange(long double, long double)
*/

	
bool CubicBezierSplineSection::bezierValuesInRange(void){
	//checks derivatives to see if all portions of the curve are in range
	//work along t-axis to maximum & minimum using Press et al.'s Golden Section method
	bool allLegal=true;
		if(node0_!=NULL && node1_!=NULL){
				bool downLeft, downCenter, downRight, flatLeft, flatRight, upLeft, upCenter, upRight, doingLeft;
				long double t=zero, y=zero, yL=zero, yR=zero, t0, t1, a, b, c, d, y0, y1, y2, y3;
				long double R = GoldenRatio() - one;
				long double C = one - R;
				aList<long double> abcd, abc;// quadraticSolutions;
				Bisection_GoldenRatioFns minmaxfns(bezierValueFn);
				downLeft=downCenter=downRight=flatLeft=flatRight=upLeft=upRight=upCenter=doingLeft=false;
				realizedRange_.Clear();
				d = cp_[3][1];
				c = three * (cp_[1][1] - cp_[0][1]);
				b = three * (cp_[2][1] - cp_[1][1]) - c;
				a = d - cp_[0][1] - b - c;
				
				//check the nodes for the shape of the curve
				
				if(node0_->HandleDistanceRight()==zero || node0_->Angle()==zero){
						flatLeft=true;}
	//				else{
	//					if(node0_->Angle()>zero){
	//							upLeft=true;}
	//						else{
	//							downLeft=true;}
	//					}
				if(node1_->HandleDistanceLeft()==zero || node1_->Angle()==zero){
						flatRight=true;}
	//				else{
	//					if(node0_->Angle()<zero){
	//							upRight=true;}
	//						else{
	//							downRight=true;}
	//					}
				y1 = bezierValue(sqrt(toleranceFactor_),a,b,c,d);
				y2 = bezierValue(half,a,b,c,d);
				y3 = bezierValue(one-sqrt(toleranceFactor_),a,b,c,d);
				if(flatLeft || flatRight){// || y1*y2>=zero || y3*y2>=zero){
//						allLegal=true;
						realizedRange_.Append(zero);
						realizedRange_.Append(zero);
						}
					else{//a curve exsts
						//c + 2 b t + 3 a t^2 ==> derivative of the cubic spline
						abc.Append(three*a);
						abc.Append(two*b);
						abc.Append(c);
						SolveQuadratic(abc,quadraticSolutions_);//in y
						if(quadraticSolutions_.len()==0){
//								allLegal=false;
								realizedRange_.Append(zero);
								realizedRange_.Append(zero);
								}
							else{//quadratic solutions exist
								t0=quadraticSolutions_[0];
								if(zero <= t0 && t0 <= one){
										y0 = bezierValue(t0,a,b,c,d);
										realizedRange_.Append(y0);
//										if(y0>=yMin_ && y0<=yMax_){
//												allLegal=true;}
//											else{
//												allLegal=false;}
										}
									else{
										realizedRange_.Append(zero);}
									t1=quadraticSolutions_[1];
									if(zero <= t1 && t1 <= one){
											y1 = bezierValue(t1,a,b,c,d);
											realizedRange_.Append(y1);
//											if(y1>=yMin_ && y1<=yMax_){
//													allLegal=true;}
//												else{
//													allLegal=false;}
											}//if t1
										else{
											realizedRange_.Append(zero);}
								}//quadratic solutions exist
						}//a curve exists
								
				}
			else{
				aError("ERROR: CubicBezierSplineSection::bezierValuesInRange(): nodes unspecified.");
				allLegal=false;}
//	yMinMaxCurrent_=true;
	realizedRangeCurrent_=true;
	int i;
	for(i=0;i<realizedRange_.len();++i){
		if(realizedRange_[i]<yMin_ || yMax_<realizedRange_[i]){
			allLegal=false;
			break;
			}
		}//i
	return allLegal;
	}//CubicBezierSplineSection::bezierValuesInRange(long double, long double)
	


long double CubicBezierSplineSection::CalculateBezierValue(long double x, long double toleranceFactor,
										aList<long double>& abcd_x, aList<long double>& abcd_y){
	long double t, tolerance, y = cp_[0][1];//dummy value 
	if(xIsInSection(x)){
		CheckToleranceFactor(toleranceFactor);
		Bisection_GoldenRatioFns minmaxfns(bezierValueFn);
		bool withinTolerance=false;
//		long double xmid, tmid, t, dt, zeropoint;
//		long double a_x,b_x,c_x,d_x,a_y,b_y,c_y,d_y, tolerance;
		tolerance = toleranceFactor;
		
		t = minmaxfns.FindPointViaBisection(x, (long double)zero, (long double) one, abcd_x);
		
		y = bezierValueFn(t, abcd_y);
		if(y < yMin_ || y> yMax_){
			aError("ERROR: CubicBezierSplineSection::CalculateBezierValue: y value out of range.");
			t = minmaxfns.FindPointViaBisection(x, (long double)zero, (long double) one, abcd_x);
			y = bezierValueFn(t, abcd_y);
			}
		}//x in section
	return y;
	}//CubicBezierSplineSection::CalculateBezierValue(long double, CoordinateClass<long double>&, long double)





long double CubicBezierSplineSection::CalculateBezierValue(long double x, long double toleranceFactor){
	//goal here is to
	long double y = cp_[0][1];//dummy value 
	if(xIsInSection(x)){
		CheckToleranceFactor(toleranceFactor);
		Bisection_GoldenRatioFns minmaxfns(bezierValueFn);
		bool withinTolerance=false;
//		long double xIteration, xTemp,tmid,dt,zeropoint,xmid;
		long double a_x,b_x,c_x,d_x,a_y,b_y,c_y,d_y, t, tolerance;
//		x0 = cp_[0][0];
//		x2 = cp_[3][0];
//		xIteration = xTemp = x2;//initial value
		tolerance = toleranceFactor;
		long double cpx1=cp_[0][0],cpx2=cp_[1][0],cpx3=cp_[2][0],cpx4=cp_[3][0];
		long double cpy1=cp_[0][1],cpy2=cp_[1][1],cpy3=cp_[2][1],cpy4=cp_[3][1];
		
		d_x = cp_[0][0];
		c_x = three * (cp_[1][0] - cp_[0][0]);
		b_x = three * (cp_[2][0] - cp_[1][0]) - c_x;
		a_x = cp_[3][0] - cp_[0][0] - b_x - c_x;
		aList<long double> abcd_x;
		abcd_x.Append(a_x);
		abcd_x.Append(b_x);
		abcd_x.Append(c_x);
		abcd_x.Append(d_x);

		d_y = cp_[0][1];
		c_y = three * (cp_[1][1] - cp_[0][1]);
		b_y = three * (cp_[2][1] - cp_[1][1]) - c_y;
		a_y = cp_[3][1] - cp_[0][1] - b_y - c_y;

		//first need to bracket x within the appropriate the t values
//		zeropoint = minmaxfns.FindRootViaBisection(t+sqrt(toleranceFactor_),t1-sqrt(toleranceFactor_), abcd_x);
		t = minmaxfns.FindPointViaBisection(x, zero, one, abcd_x);
		
/*		int iteration=0;
		xmid = a_x + b_x + c_x + cp_[0][0];//t=1
		if(x==xL_[0]){
				t = zero;}
			else if(x==xR_[0]){
				t = one;}
			else{
				t = zero;
				dt = one;
				while(ABS(x-xmid)>toleranceFactor){
					++iteration;
					dt *= half;					
					tmid = t + dt;
					xmid = a_x*tmid*tmid*tmid + b_x*tmid*tmid + c_x*tmid + cp_[0][0];
					if(xmid<=x){
						t = tmid;}
					}//while				
				}
*/
		y = a_y*t*t*t + b_y*t*t + c_y*t + d_y;
		if(y < yMin_ || y> yMax_){
			aError("ERROR: CubicBezierSplineSection::CalculateBezierValue: y value out of range.");
			}
		}//x in section
	return y;
	}//CubicBezierSplineSection::CalculateBezierValue(long double, CoordinateClass<long double>&, long double)
	
	

void CubicBezierSplineSection::CalculateBezierValues(long double toleranceFactor){
	splineYforXinDataCurrent_=false;
	SetBezierParams();
	int i;
	long double x, y, xConst = splineXvals_.min()-one;
	long double a_x,b_x,c_x,d_x,a_y,b_y,c_y,d_y;
		long double cpx1=cp_[0][0],cpx2=cp_[1][0],cpx3=cp_[2][0],cpx4=cp_[3][0];
		long double cpy1=cp_[0][1],cpy2=cp_[1][1],cpy3=cp_[2][1],cpy4=cp_[3][1];
	aList<long double> abcd_x, abcd_y;
	d_x = cp_[0][0];
	c_x = three * (cp_[1][0] - cp_[0][0]);
	b_x = three * (cp_[2][0] - cp_[1][0]) - c_x;
	a_x = cp_[3][0] - cp_[0][0] - b_x - c_x;
	abcd_x.Append(a_x);
	abcd_x.Append(b_x);
	abcd_x.Append(c_x);
	abcd_x.Append(d_x);

	d_y = cp_[0][1];
	c_y = three * (cp_[1][1] - cp_[0][1]);
	b_y = three * (cp_[2][1] - cp_[1][1]) - c_y;
	a_y = cp_[3][1] - cp_[0][1] - b_y - c_y;
	abcd_y.Append(a_y);
	abcd_y.Append(b_y);
	abcd_y.Append(c_y);
	abcd_y.Append(d_y);

	for(i=0;i<splineXvals_.len();++i){
		x = splineXvals_[i];
		if(x == xL_[0] || x == xR_[0]){
				splineYforXinData_[i] = y = zero;}
			else if(i!=0 && x==xConst){
				splineYforXinData_[i] = y;}
			else{
				splineYforXinData_[i] = y = CalculateBezierValue(x,toleranceFactor,abcd_x, abcd_y);}
		xConst=x;
		}
	splineYforXinDataCurrent_=true;
	}//CubicBezierSplineSection::CalculateBezierValues(long double, long double )



void CubicBezierSplineSection::CheckToleranceFactor(long double toleranceFactorToCheck){
	if(toleranceFactor_ <= zero || toleranceFactorToCheck < toleranceFactor_){
		SetSplineYforXinDataCurrent(false);
		toleranceFactor_ = toleranceFactorToCheck;
		}
	}



/*
void CubicBezierSplineSection::bezierValueAtT(long double t, CoordinateClass<long double>& coordsOnCurve){
	//determines {x,y} at position x on the curve; returns vals in coordsOnCurve
	SetBezierParams();
	long double a_x,b_x,c_x,a_y,b_y,c_y,t2,t3;
	
	c_x = (long double)3.0 * (cp_[1][0] - cp_[0][0]);
	b_x = (long double)3.0 * (cp_[2][0] - cp_[1][0]) - c_x;
	a_x = cp_[3][0] - cp_[0][0] - b_x - c_x;
	

	c_y = (long double)3.0 * (cp_[1][1] - cp_[0][1]);
	b_y = (long double)3.0 * (cp_[2][1] - cp_[1][1]) - c_y;
	a_y = cp_[3][1] - cp_[0][1] - b_y - c_y;
	
	t2=t*t;
	t3=t2*t;
	
	coordsOnCurve[0] = a_x*t3 + b_x*t2 + c_x*t + cp_[0][0];
	coordsOnCurve[1] = a_y*t3 + b_y*t2 + c_y*t + cp_[0][1];
	}//CubicBezierSplineSection::bezierValue(long double , CoordinateClass<long double>& )
*/


/*
void CubicBezierSplineSection::FindCubicRoots(aList<long double>& roots){
	roots.Clear();
	long double a,b,c,p,q,u1,u2,quadterm, sqrtterm, cubeterm1, cubeterm2;
	bool legalu1=false, legalu2=false;
	c = three * (cp_[1][0] - cp_[0][0]);
	b = three * (cp_[2][0] - cp_[1][0]) - c;
	a = cp_[3][0] - cp_[0][0] - b - c;
	p = b - a*a/three;
	q = c + (two*a*a*a - (long double)9.0*a*b)/long double(27.0);
	quadterm = q*q/four + p*p*p/(long double)27.0;
	if(quadterm>=zero){
		sqrtterm = sqrt(quadterm);
		cubeterm1 = q/two + sqrtterm;
		cubeterm2 = q/two - sqrtterm;
		if(cubeterm1!=zero){
			legalu1=true;
			u1 = cuberoot(abs(cubeterm1));
			if(cubeterm1<zero){
				u1 = -u1;}
			
			}
		if(cubeterm2 != zero){
			legalu2=true;
			u2 = cuberoot(abs(cubeterm2));
			if(cubeterm2<zero){
				u2 = -u2;}
			}
		}//quadterm
			
			
	}//CubicBezierSplineSection::FindCubicRoots(aList<long double>&
*/		
		

	

long double CubicBezierSplineSection::GetTabledYfromX(long double x){
	long double y = cp_[0][1];//default dummy value
	if(xIsInSection(x)){
		int posX = splineXvals_.Loc(x);
		y = splineYforXinData_[posX];
		}
	return y;
	}//CubicBezierSplineSection::GetTabledYfromX(long double)


void CubicBezierSplineSection::handleCoordsL(CoordinateClass<long double>& coordsToGet){
	return node0_->HandleCartesianRight(coordsToGet);}//yep, the right handle of the left node

	
void CubicBezierSplineSection::handleCoordsR(CoordinateClass<long double>& coordsToGet){
	return node1_->HandleCartesianLeft(coordsToGet);}//yep, the left handle of the right node

	
void CubicBezierSplineSection::nodeCoordsL(CoordinateClass<long double>& coordsToGet){
	return node0_->NodeCartesian(coordsToGet);}

	
void CubicBezierSplineSection::nodeCoordsR(CoordinateClass<long double>& coordsToGet){
	return node1_->NodeCartesian(coordsToGet);}

	
long double CubicBezierSplineSection::range_x(void){//along x-axis
	 SetBezierParams();
	 return xR_[0] - xL_[0];
	 }


long double CubicBezierSplineSection::ResidualAt_x(CoordinateClass<long double>& datum, long double toleranceFactor){
	long double resval=zero, x = datum[0], y = datum[1], yPrime;
	if(xIsInSection(x)){
		CheckToleranceFactor(toleranceFactor);
		if(splineYforXinDataCurrent_){
			CoordinateClass<long double> dummy(x,zero);
			yPrime = bezierValue(x,dummy,toleranceFactor);
			resval = y - yPrime;
			}
		}
	return resval;
	}


//void CubicBezierSplineSection::lnLNowInvalid(void){
//	lnL_current_=false;}
	

/*
long double CubicBezierSplineSection::ResidualAt_t(long double t, long double yPrime){
	//on t-axis scale -- t out of range returns error message
	long double val=zero;
	if(t < zero || t > one){
			aString err("ERROR: The value t=");
			err.cat(t); err.cat(" is outside the range 0 - 1");
			aError(err);
			}
		else{
			val = yPrime - ValueAt_t(t);}
	return val;
	}
*/

		
void CubicBezierSplineSection::SetBezierParams(void){
	if(!bezierParamsCurrent_){
		SetEndpoints();
		SetControlPoints();
		bezierParamsCurrent_=true;}
	}
	
	

void CubicBezierSplineSection::SetBezierParamsCurrent(bool tf){
	bezierParamsCurrent_=tf;
//	SetBezierParams();
	if(tf==false){
			SetSplineYforXinDataCurrent(false);
//			lnLNowInvalid();
			}
	}

void CubicBezierSplineSection::SetEndpoints(void){
	if(node0_ != NULL){
		xL_=node0_->coords_;
		SetBezierParamsCurrent(false);
		}
	if(node1_ != NULL){
		xR_=node1_->coords_;
		SetBezierParamsCurrent(false);
		}
	}//CubicBezierSplineSection::SetEndpoints()
	


void CubicBezierSplineSection::SetControlPoints(void){
	nodeCoordsL(cp_[0]);
	handleCoordsL(cp_[1]);
	handleCoordsR(cp_[2]);
	nodeCoordsR(cp_[3]);
	SetBezierParamsCurrent(false);
	}//CubicBezierSplineSection::controlPoint(int, CoordinateClass<long double>&)



//void CubicBezierSplineSection::SetLnL(long double lnLval){
//	lnL_section_=lnLval;
//	if(lnLmax_section_ > zero || lnLval > lnLmax_section_){
//		lnLmax_section_ = lnLval;}
//	lnL_current_=true;
//	}
		

void CubicBezierSplineSection::SetNodeLeft(BezierNode *node){
	node0_ = node;
	if(node != NULL && sectionPrev_ == NULL){
		node0_->SetTerminalLeft(true);}
	bezierParamsCurrent_=false;
	 SetBezierParams();
	}

void CubicBezierSplineSection::SetNodeRight(BezierNode *node){
	node1_ = node;
	if(node != NULL && sectionNext_ == NULL){
		node1_->SetTerminalRight(true);}
	bezierParamsCurrent_=false;
	 SetBezierParams();
	}


void CubicBezierSplineSection::SetSectionID(int id){
	ID_=id;}

void CubicBezierSplineSection::SetSectionLeft(CubicBezierSplineSection *section){
	sectionPrev_ = section;
	if(section==NULL){
		node0_->SetTerminalLeft(true);}
		else{
			node0_->SetTerminalLeft(false);
			node0_->SetSectionRight(this);
			}
	bezierParamsCurrent_=false;
	 SetBezierParams();
	}

void CubicBezierSplineSection::SetSectionRight(CubicBezierSplineSection *section){
	sectionNext_ = section;
	if(section==NULL){
			node1_->SetTerminalRight(true);}
		else{
			node1_->SetTerminalRight(false);
			node1_->SetSectionLeft(this);
			}
	bezierParamsCurrent_=false;
	 SetBezierParams();
	}


void CubicBezierSplineSection::SetSpline(CubicBezierSpline *spline){
	spline_=spline;
//	if(spline != NULL){
//		lnLNowInvalid();
//		}
	}



void CubicBezierSplineSection::SetXvalues(aList<long double>& localBinCenters){
	//sets first value in splineYforXinData_
	splineYforXinData_.Clear();
	splineXvals_.Clear();
	splineYforXinDataCurrent_=false;
	int b;
	for(b=0;b<localBinCenters.len();++b){
		splineXvals_.Append(localBinCenters[b]);
		splineYforXinData_.Append(zero);
		}
	}// CubicBezierSplineSection::SetXvalues(aList<long double>& )



void CubicBezierSplineSection::SetSplineYforXinDataCurrent(bool tf){
	splineYforXinDataCurrent_=tf;
	}

void CubicBezierSplineSection::SetYminMax(long double ymin, long double ymax){
	yMin_ = ymin;
	yMax_ = ymax;
	}


/*	
long double CubicBezierSplineSection::t_from_x(long double x, CoordinateClass<long double>& finalCoords, 
													long double tolerance){
	long double tval= -one;//default nonsense
	if(xIsInSection(x)){
		bool withinTolerance=false;
		long double x0,x1,x2,t0,t1,t2;
		CoordinateClass<long double> x1coords(zero,zero,zero);
		t0=zero;
		t1=one/two;
		t2=one;
		x0=cp_[0][0];
		x2=cp_[3][0];
		if((x-x0)<=tolerance){
				tval=zero;
				finalCoords=cp_[0];
				}
			else if((x2-x)<=tolerance){
				tval=one;
				finalCoords=cp_[3];
			}
			else{
				do{
					bezierValueAtT(t1,x1coords);
					x1=x1coords[0];
					if(abs(x-x1)<=tolerance){
							withinTolerance=true;}
						else{
							if(x>x1){
									t0=t1;
									t1 += (t2-t1)/two;
									}
								else{
									t2=t1;
									t1 -= (t1-t0)/two;
									}
							}						
					}while(! withinTolerance);		
				tval=t1;
				finalCoords = x1coords;
				}
			}//xIsInSection
	return tval;
	}//CubicBezierSplineSection::t_from_x(long double, CoordinateClass<long double>&, long double)
	
	
long double CubicBezierSplineSection::t_from_x(long double x, bool monotonic_x){
	long double tval= -one;//default nonsense
	if(xIsInSection(x) && monotonic_x){
		tval = (x-xL_[0])/(xR_[0]-xL_[0]);
		}
	return tval;
	}//CubicBezierSplineSection::t_from_x(long double, CoordinateClass<long double>&, long double)
*/	
	
	

bool CubicBezierSplineSection::xIsInSection(long double x){
	 SetBezierParams();
	return ((x >= xL_[0]) && (x <= xR_[0]));
	}


long double CubicBezierSplineSection::xL(void){//left end of range
	 SetBezierParams();
	return xL_[0];}


long double CubicBezierSplineSection::xR(void){//right end of range
	 SetBezierParams();
	return xR_[0];}


long double CubicBezierSplineSection::yL(void){//left end of range
	 SetBezierParams();
	return xL_[1];}


long double CubicBezierSplineSection::yR(void){//right end of range
	 SetBezierParams();
	return xR_[1];}






int	operator==(CubicBezierSplineSection& p, CubicBezierSplineSection& q){
	return ((p.node0_ == q.node0_) && (p.node1_ == q.node1_) && (p.sectionPrev_ == q.sectionPrev_) && (p.sectionNext_ == q.sectionNext_)
				 && (p.xL_ == q.xL_) && (p.xR_ == q.xR_) && (p.node1_ == q.node1_));
	}


int	operator!=(CubicBezierSplineSection& p, CubicBezierSplineSection& q){
	return !(p==q);}



#endif














