//Gamma.cpp

#ifndef Gammacp
#define Gammacp = 1


#include "Gamma.h"
//#include "InlineTemplates.h"
#include <math.h>


long double gammaln(const long double xx){
	//directly from Numerical Recipes in C; changed doubles --> long doubles
	long double x,y,tmp,ser;
	static long double cof[6] = {76.18009172947146, - 86.50532032941677,
								 24.01409824083091, - 1.231739572450155,
								 0.1208650973866179e-2, -0.5395239384953e-5};
	int j;
	y=x=xx;
	tmp = x + (long double)5.5;
	tmp -= (x + half) * ln(tmp);//log
	ser = 1.000000000190015;
	for(j=0;j<=5;++j){
		ser += cof[j]/++y;}
	long double answer = -tmp + ln(2.5066282746310005 * ser / x);
	return answer;
	}//end gammaln

	//old algorithm translated form numerical recipes in Pascal
/*long double gammaln(const long double xx){
	const long double stp = 2.50662827465;
	long double x = xx - 1.0;
	long double tmp = x + 5.5;
	tmp = (x + 0.5) * ln(tmp) - tmp;
//	tmp = (x + 0.5) * log(tmp) - tmp;
	long double ser = 1.0 + 76.18009173 / (x + 1.0) - 86.50532033 / (x+ 2.0)
				    + 24.01409822 / (x+3.0) - 1.231739516 / (x+ 4.0)
				    + 0.120858003e-2 / (x + 5.0) - 0.536382e-5 / (x+ 6.0);
	long double stpSer = stp + ser;
	return tmp + ln(stpSer);
//	return tmp + log(stpSer);
	}//end gammaln
*/	
	
long double Gamma(const long double xx){
	return exp(gammaln(xx));}
	
long double Gamma(const long double a, const long double x){
	return gammaInc(a,x);}

inline long double Gamma(const long double a, const long double x0, const long double x1){
	return gammaGeneralizedInc(a,x0,x1);}


long double Erf(const long double x){
	//error function -- integral 0 to x 
	if(x<zero){
		return (- gammaInc(half, x*x));}
	return gammaInc(half, x*x);
	}//Erf(const long double)


long double Erfc(const long double x){
	//complementary error function -- integral x to infinity with shape parameter a
	if(x<zero){
		return (one + gammaInc(half, x*x));}
	return gammaIncComp(half, x*x);
	}//Erfc(const long double)


long double gammaInc(const long double a, const long double x){
	//incomplete gamma function -- integral 0 to x with shape parameter a
	if(x<zero){
			aError("ERROR: cannot call the function gammaInc with x<0");
			return zero;}
		else if (a <= zero){
			aError("ERROR: cannot call the function gammaInc with a0");
			return zero;}
		else{}
	long double gam_a=zero;//dummy
	if(x<a+one){
		return gammaSer(a,x,gam_a);
		}
	return one - gammaCF(a,x,gam_a);
	}// gammaInc(const long double, const long double)



long double gammaGeneralizedInc(const long double a, const long double x0, const long double x1){
	//incomplete gamma function -- integral x0 to x1 with shape parameter a 
	long double ggi = Gamma(a,x0) - Gamma(a,x1);
	return ggi;
	}// gammaGeneralizedInc(const long double, const long double, const long double)





long double gammaIncComp(const long double a, const long double x){
	//complement of incomplete gamma function -- integral x to infiity with shape parameter a 
	if(x<zero){
			aError("ERROR: cannot call the function gammaIncComp with x<0");
			return zero;}
		else if (a <= zero){
			aError("ERROR: cannot call the function gammaIncComp with a0");
			return zero;}
		else{}
	long double gam_a = zero;//dummy
	if(x<a+one){
		return one - gammaSer(a,x,gam_a);
		}
	return gammaCF(a,x,gam_a);
	}//gammaIncComp(const long double, const long double)


long double gammaSer(const long double a, const long double x, long double gammalna){
	//series method for evaluating incomplete gamma function 
	gammalna = gammaln(a);
	if(x==zero){
		return zero;}
	if(x<zero){
		aError("ERROR: cannot call the function gammaSer with x<0");
		return zero;}
		else if (a <= zero){
			aError("ERROR: cannot call the function gammaSer with a0");
			return zero;}
	const int itmax = 500;
	const long double eps = 3.0e-7;
	long double ap = a;
	long double summ = one / a;
	long double del = summ;
	for (int i=0;i<itmax;++i){
		ap += one;
		del *= x/ap;
		summ += del;
		if(ABS(del) < ABS(summ)*eps) {//close enough
			return summ*exp( -x + a*ln(x) - gammalna);}
		}//i
	//shouldn't get here
		aError("ERROR: function gammaSer didn't converge on an answer within its acceptable error margin");
	return summ*exp( -x + a*ln(x) - gammalna);	//return best guess
	}//gammaSer((const long double, const long double, long double)
	
	
long double gammaCF(const long double a, const long double x, long double gammalna){
	//continued-fraction method for evaluating incomplete gamma function 
	gammalna = gammaln(a);
	if(x==zero){
		return zero;}
	if(x<zero){
		aError("ERROR: cannot call the function gammaCF with x<0");
		return zero;}
		else if (a <= zero){
			aError("ERROR: cannot call the function gammaCF with a0");
			return zero;}
	const int itmax = 500;
	const long double eps = 3.0e-7;
	long double gold = zero;
	long double a0 = one;
	long double a1 = x;
	long double b0 = zero;
	long double b1 = one;
	long double fac = one;
	long double an, ana, anf, g;
	for (int i=0;i<itmax;++i){
		an = (long double)i;
		ana = an - a;
		a0 = (a1 + a0 * ana) * fac;
		b0 = (b1 + b0 * ana) * fac;
		anf = an * fac;
		a1 = x * a0 + anf * a1;
		b1 = x * b0 + anf * b1;
		if(a1 != zero){//renormalize
			fac = one / a1;
			g = b1 * fac;
			if(ABS((g - gold)/g) < eps) {//converged
				return g * exp( -x + a*ln(x) - gammalna);}
			gold = g;
			}
		}//i
	//shouldn't get here
		aError("ERROR: function gammaCF didn't converge on an answer within its acceptable error margin");
	return g * exp( -x + a*ln(x) - gammalna);	//return best guess
	}//gammaCF((const long double, const long double, long double)



long double GammaRegularized(const long double a, const long double x){
	long double gr = gammaInc(a,x) / Gamma(a);
	return gr;
	}//GammaRegularized(const long double, const long double)
	
	
	
	
long double GammaRegularized(const long double a, const long double x0, const long double x1){
	long double gr = gammaGeneralizedInc(a,x0,x1) / Gamma(a);
	return gr;
	}//GammaRegularized(const long double, const long double, const long double)
	


/*
long double RandomGamma(const long double a, const long double x, Ran3& ran){
	//a random number from a gamma distribution
	long double rg=zero, aa = a-one;
	if(a<one){
			rg = x * gsGamma(a,a/E());
			}
		else if(a==one){
			rg = -log(ran())/x;
			}
		else if(a < (long double)2.5){
			rg = x * aa * gkm1gamma((a-one/(two*three*a))/a, two/a);
			}
		else{
			rg = x * aa * gkm2gamma(one/sqrt(a), (a-one/(two*three*a))/a, two/a);
			}
	return rg;
	}//RandomGamma(const long double, const long double, Ran3&)
	
exponential = Compile[{{lambda, _Real}, {q, _Real}}, -Log[q]/lambda]

long double gsGamma(const long double a, const long double r, Ran3& ran){
	//used in calculation of random gamma
	long double x = one, t = one, q = (one + r)*ran();
	if(q < one){
			x = exp(-alpha*log(q));
			t = exp(-x);}
		else{
			x = one + log(one+(one-q)/r);
			t = exp((a-one)*log(x));}
	while(ran()>t){
		q = (one+r) * ran();
		if(q<one){
				x = exp(-alpha*log(q));
				t = exp(-x);}
			else{
				x = one + log(one+(one-q)/r);
				t = exp((a-one)*log(x));}
		}//while
	return x;
	}//gsGamma(const long double, const long double, Ran3&)
	

long double gkm1Gamma(const long double b, const long double c, Ran3& ran){
	//used in calculation of random gamma
	long double u1 = ran(), u2 = ran(), w;
	w = b*u1/u2;
	while((c*u2 - c - two + w + one/w > zero) && (c*log(u2) - log(w) + w - one >= zero){
		u1 = ran();  
		u2 = ran();
		w = b*u1/u2;
		}//while
	return w;
	}//gkm1Gamma(const long double, const long double, Ran3&)



long double gkm2Gamma(const long double k, const long double b, const long double c, Ran3& ran){
	//used in calculation of random gamma
	long double u1 = ran(), u2, w;
	u2 = u1 + k*(one - (long double)1.86*ran());
	while(!(zero < u2 && u2 < one)){
		u1 = ran();
		u2 = u1 + k*(one - (long double)1.86*ran());
		}//while
	w = b*u1/u2;
	while((c*u2 - c - two + w + one/w > zero) && (c*log(u2) - log(w) + w - one >= zero){
		u1 = ran();
		u2 = u1 + k*(one - (long double)1.86*ran());
		while(!(zero < u2 && u2 < one)){
			u1 = ran();
			u2 = u1 + k*(one - (long double)1.86*ran());
			}//while
		w = b*u1/u2;
		}//while
	return w;
	}////gkm2Gamma(const long double, const long double, const long double, Ran3&)
	

*/


#endif







