/**********************
*
*	MantelTest.cpp
*
*	Does a Mantel test to compare two same-sized matrices for 
*	correlations among cells
*
**********************/

#ifndef MantelTestcp
#define MantelTestcp = 1

#include "MantelTest.h"
#include "Element.cpp"
#include "aList.cpp"
#include "aVector.cpp"
#include "aMatrix.cpp"
#include "BootList.cpp"

long double MantelTestr(aMatrix<long double>& m1, aMatrix<long double>& m2){
	if(! m1.CanBeAdded(m2)){
		cout<<"ERROR:  In MantelTestr, matrices must be the same size!"<<endl;
		return zero;}
	long double sum1=zero, sum2=zero, mean1=zero, mean2=zero;
	int count=0;
	for(long c1=1;c1<m1.cols();++c1){
		for(long r1=0;r1<c1;++r1){
			sum1 += m1[r1][c1];
			sum2 += m2[r1][c1];
			++count;}}
	mean1=sum1/count;
	mean2=sum2/count;
	long double numerator=zero, v1=zero, v2=zero;
	for(long c1=1;c1<m1.cols();++c1){
		for(long r1=0;r1<c1;++r1){
			numerator += (m1[r1][c1] - mean1) * (m2[r1][c1] - mean2);
			v1 += m1[r1][c1] * m1[r1][c1];
			v2 += m2[r1][c1] * m2[r1][c1];
			}}
	long double r = numerator/(sqrt(v1) * sqrt(v2));
	return r;
	}//end MantelTestr



long double MantelTest(aMatrix<long double>& m1, aMatrix<long double>& m2,
					long double& r, long double& P, int reps, Ran3& ran){
	r = MantelTestr(m1, m2);//observed value

	BootList<long double> rList;
	for(int rep=0;rep<reps;++rep){
		aMatrix<long double> workingm1(m1);//use a copy
		workingm1.RandomPermutation(ran);
		long double newr = MantelTestr(workingm1, m2);
		rList.Append(newr);
		}
	rList.Sort();
	P = rList.Get2TailP(r);
	return P;
	}//end MantelTest



#endif



