/******************************************************
*
*	CubicBezierSplineParamRec.cpp
*
*	Parameters for a normal distribution defined by mean and stdev.
*
*	Use this as a base class for derived classes that add parameters.
*
******************************************************/
#ifndef CubicBezierSplineParamReccpp
#define CubicBezierSplineParamReccpp = 1

#include "CubicBezierSplineParamRec.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"
#include "CoordinateClass.cpp"


CubicBezierSplineParamRec::CubicBezierSplineParamRec(void):ParamRec(), sigma_((long double)1.0),
  					spline_(NULL), variableParamLookupTable_(NULL), //lnLbySection_(NULL),
  					//sectionsToRecalculate_lnL_(NULL),
  					dataLocationsBySection_(NULL),
  					toleranceFactor_((long double)0.001) {
  				SetPointerToDerivedSelf();
  				}


CubicBezierSplineParamRec::CubicBezierSplineParamRec(long double sig, CubicBezierSpline *spline,
													 ObjList<aList<int> > *variableParamLookupTable,
													 StatList<long double>& lnLbySection ,aList<int>& sectionsToRecalculate_lnL,
													 ObjList<aList<int> > *dataLocationsBySection
													 ):
					ParamRec(), sigma_(sig),
  					spline_(spline), variableParamLookupTable_(variableParamLookupTable),
  					lnLbySection_(lnLbySection), sectionsToRecalculate_lnL_(sectionsToRecalculate_lnL),
  					dataLocationsBySection_(dataLocationsBySection),
  					toleranceFactor_((long double)0.001){
  				SetPointerToDerivedSelf();
  				}


CubicBezierSplineParamRec::CubicBezierSplineParamRec(CubicBezierSplineParamRec& p){
  				SetPointerToDerivedSelf();
				operator=(p);
				}



CubicBezierSplineParamRec::~CubicBezierSplineParamRec(void){
  				pointerToDerivedSelf = NULL;//forget all these
  				spline_ = NULL;
  				variableParamLookupTable_ = NULL;
  				//lnLbySection_ = NULL;
  				//sectionsToRecalculate_lnL_ = NULL;
  				dataLocationsBySection_ = NULL;
  				}



ParamRec* CubicBezierSplineParamRec::Duplicate(void){
	CubicBezierSplineParamRec* copy = NEW CubicBezierSplineParamRec(*this);
	return copy;
	}//end CubicBezierSplineParamRec::Duplicate()



CubicBezierSplineParamRec* CubicBezierSplineParamRec::_Duplicate(void){
	return NEW CubicBezierSplineParamRec(*this);
	}//end CubicBezierSplineParamRec::_Duplicate()



void CubicBezierSplineParamRec::SetPointerToDerivedSelf(void){
  	pointerToDerivedSelf = (long) this;
  	}//end

long CubicBezierSplineParamRec::GetPointerToDerivedSelf(void){//handle case where OS moves pointers to new memory locations
  	long pdsf = (long) this;
  	return pdsf;
  	}//end


void CubicBezierSplineParamRec::operator=(ParamRec& p){
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class CubicBezierSplineParamRec
	CubicBezierSplineParamRec* pp = (CubicBezierSplineParamRec*) p.GetPointerToDerivedSelf();
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
	sigma_ = pp->sigma_;
	spline_ = pp->spline_;//copies pointer
  	variableParamLookupTable_ = pp->variableParamLookupTable_;//copies pointer
  	lnLbySection_ = pp->lnLbySection_;
  	sectionsToRecalculate_lnL_ = pp->sectionsToRecalculate_lnL_;
  	dataLocationsBySection_ = pp->dataLocationsBySection_;
  	toleranceFactor_ = pp->toleranceFactor_;
	}//end  CubicBezierSplineParamRec::operator=(ParamRec&)



int CubicBezierSplineParamRec::ChangeBezierParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, long double /*weight*/){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	CubicBezierSplineParamRec* deviations = (CubicBezierSplineParamRec*) paramDeviations->GetPointerToDerivedSelf();
	CubicBezierSplineParamRec* lowLim = (CubicBezierSplineParamRec*) lowerLimits->GetPointerToDerivedSelf();
	CubicBezierSplineParamRec* highLim = (CubicBezierSplineParamRec*) upperLimits->GetPointerToDerivedSelf();
	CubicBezierSpline *spline = spline_;
	int returnval,sec,s,node,paramAtNode;
	bool inRange = true;
	node =(*variableParamLookupTable_)[paramNum][0];
	paramAtNode =(*variableParamLookupTable_)[paramNum][1];
	BezierNode& primaryNode = spline_->nodeList_[node];
	BezierNode primaryNodeCopy(primaryNode);
	BezierNode& devNode = deviations->spline_->nodeList_[node];
	BezierNode& lowNode = lowLim->spline_->nodeList_[node];
	BezierNode& highNode = highLim->spline_->nodeList_[node];
	aList<int> tempSectionsToRecalcLnL;
	returnval = primaryNode.ChangeNodeParam(paramAtNode, ran, devNode, lowNode, highNode);
	if(returnval==1){//check for range compatibility
//		spline_->ChangeSplineParam(node,paramAtNode,
		for(s=2;s<(*variableParamLookupTable_)[paramNum].len();++s){
			sec = (*variableParamLookupTable_)[paramNum][s];
//			(*sectionsToRecalculate_lnL_)[sec]=true;
			CubicBezierSplineSection sectionCopy(spline_->sectionList_[sec],true);
			if(spline_->sectionList_[sec].node0_ == &primaryNode){
					primaryNodeCopy.SetSectionLeft(&sectionCopy);}
				else{
					primaryNodeCopy.SetSectionRight(&sectionCopy);}
			sectionCopy.SetSplineYforXinDataCurrent(false);
//			spline_->RecalculateYvalues(sec);//only sets the flag
//			inRange = spline_->sectionList_[sec].bezierValuesInRange(-one/two, one/two, toleranceFactor_);
//			inRange = sectionCopy.bezierValuesInRange(-one/two, one/two, toleranceFactor_);
//			inRange = sectionCopy.bezierValuesInRange(-one/two, one/two);
			inRange = sectionCopy.bezierValuesInRange();
			if(! inRange){
				returnval=0;
				break;}
			tempSectionsToRecalcLnL.Append(sec);
			}//s
		}
	if(returnval==0){//restore originals
//	if(returnval==1){
		CubicBezierSplineSection *leftsection = primaryNode.section0_;
		CubicBezierSplineSection *rightsection = primaryNode.section1_;
		primaryNode=primaryNodeCopy;
		primaryNode.SetSectionLeft(leftsection);
		primaryNode.SetSectionRight(rightsection);
//		primaryNode.SetSpline(spline);
		leftsection = NULL;
		rightsection = NULL;
		for(s=0;s<tempSectionsToRecalcLnL.len();++s){
			sec = tempSectionsToRecalcLnL[s];
			sectionsToRecalculate_lnL_[sec]=true;
			}
		}
	spline=NULL;
	return returnval;
	}//end CubicBezierSplineParamRec::ChangeParam(int)




int CubicBezierSplineParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits,long double weight){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	bool returnval;
	if(paramNum<ErrorDistParams()){
			CubicBezierSplineParamRec* deviations = (CubicBezierSplineParamRec*) paramDeviations->GetPointerToDerivedSelf();
		 	returnval= ChangeSigma(ran, deviations->sigma()*weight,lowerLimits,upperLimits);
		 	}
		 else{
//		 	returnval= ChangeBezierParam(paramNum - ErrorDistParams(), ran, paramDeviations, lowerLimits, upperLimits);}
		 	returnval= ChangeBezierParam(paramNum, ran, paramDeviations, lowerLimits, upperLimits);}
	
	return (int)returnval;
	}//end CubicBezierSplineParamRec::ChangeParam(int)




int CubicBezierSplineParamRec::ChangeParams(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits, 
							aList<int>& paramChoices, int& paramsToChooseAmong, 
							bool sequentialParamVariation, long double weight){
	int paramToChange;
	paramToChange = ParamRec::ChangeParams(ran, paramDeviations, lowerLimits, upperLimits, 
							paramChoices, paramsToChooseAmong, sequentialParamVariation, weight);
/*	if(paramToChange<ErrorDistParams()){
			for(sec=0;sec<(*sectionsToRecalculate_lnL_).len();++sec){//sections affected
				(*sectionsToRecalculate_lnL_)[sec]=true;
				}
			}
		else{
			aList<int>& paramData = (*variableParamLookupTable_)[paramToChange];
			for(i=2;i<paramData.len();++i){//sections affected
				sec=paramData[i];
				(*sectionsToRecalculate_lnL_)[sec]=true;
				}
			}
*/
	return paramToChange;
	}//end ParamRec::ChangeParams(Ran3&, ParamRec*, ParamRec*, ParamRec*, aList<int>&, long double, int&, bool)





int CubicBezierSplineParamRec::ChangeSigma(Ran3& ran, long double sigmaForNewSigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int sec, validChange=false;
	long double oldSigma = sigma_;
	do{
		sigma_ = ran.NormDev(oldSigma, sigmaForNewSigma);
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	if(validChange){
		for(sec=0;sec<sectionsToRecalculate_lnL_.len();++sec){
			sectionsToRecalculate_lnL_[sec] = true;
			}
		}
	return validChange;
	}//end CubicBezierSplineParamRec::ChangeSigma(Ran3&, long double, ParamRec*, ParamRec*)



void CubicBezierSplineParamRec::CheckToleranceFactor(long double newFactor){
	if(newFactor != toleranceFactor_){
		SetToleranceFactor(newFactor);}
	}
	


long double CubicBezierSplineParamRec::ExpectedValue(aList<long double>& coords){
	return ExpectedValue(coords,toleranceFactor_);
	}//end CubicBezierSplineParamRec::ExpectedValue(aList<double>&)



long double CubicBezierSplineParamRec::ExpectedValue(aList<long double>& coords, long double toleranceFactor){
	CheckToleranceFactor(toleranceFactor);
	return spline_->FunctionValue(coords[0],toleranceFactor);
	}//end CubicBezierSplineParamRec::ExpectedValue(aList<double>&)




int CubicBezierSplineParamRec::MeetsParamCriteria (ParamRec* /*lowerLimits*/, ParamRec* /*upperLimits*/){//handled at parameter selection stage
//	//use type conversion that circumvents usual virtual base class restrictions
//	CubicBezierSplineParamRec* lower = (CubicBezierSplineParamRec*) lowerLimits->pointerToDerivedSelf;
//	CubicBezierSplineParamRec* upper = (CubicBezierSplineParamRec*) upperLimits->pointerToDerivedSelf;
	int returnval=1;
	if(sigma_ <= zero){returnval = 0;}//must be positive
	return returnval;
	}//end CubicBezierSplineParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void CubicBezierSplineParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	if(paramNum==0){
	 		 sigma_ *= factor;}
	 	 else{
			int node =(*variableParamLookupTable_)[paramNum][0];
			int paramAtNode =(*variableParamLookupTable_)[paramNum][1];
			BezierNode& primaryNode = spline_->nodeList_[node];
			primaryNode.MultiplyParam(paramAtNode,factor);
			}
	}//CubicBezierSplineParamRec::MultiplyParam(int, double)



int CubicBezierSplineParamRec::NumParams(void){
	//number of parameters that vary
//	return ErrorDistParams() + variableParamLookupTable_->len();
	int num=0;
	if(variableParamLookupTable_ != NULL){
		num=variableParamLookupTable_->len();}
	return num;
	}//end CubicBezierSplineParamRec::NumParams()


int CubicBezierSplineParamRec::NumDerivedParams(void){
	//number of parameters that are functions of varying parameters 
	//override in derived classes
	return 0;
	}//ParamRec::NumDerivedParams()



aString* CubicBezierSplineParamRec::ParamName(int paramNum){
	aString *returnStr=NEW aString();
	if(paramNum==0){
		 	*returnStr = "stdev";}
		 else{
			int node = (*variableParamLookupTable_)[paramNum][0];
			int paramAtNode = (*variableParamLookupTable_)[paramNum][1];
			*returnStr=spline_->ParamName(node, paramAtNode);
			}
	return returnStr;
	}//end CubicBezierSplineParamRec::ParamName(int )



void CubicBezierSplineParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::ParamString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat(sigma_,6,10);
	int pp, node, paramAtNode;
	for(pp=ErrorDistParams();pp<variableParamLookupTable_->len();++pp){
		dataline.cat(sepchar);
		node = (*variableParamLookupTable_)[pp][0];
		paramAtNode = (*variableParamLookupTable_)[pp][1];
		dataline.cat("node_");dataline.cat(node);dataline.cat(".");
		spline_->nodeList_[node].ParamName(paramAtNode, dataline);
		}//pp
	}//end CubicBezierSplineParamRec::ParamString(aString&, char)



long double CubicBezierSplineParamRec::ParamValue(int paramNum){
	//returns value of parameter	
	long double val;
	if(paramNum==0){ 
			val=sigma();}
		else{
			int node=(*variableParamLookupTable_)[paramNum][0];
			int paramAtNode = (*variableParamLookupTable_)[paramNum][1];
			val = spline_->nodeList_[node].ParamValue(paramAtNode);
			}
	return val;
	}//end CubicBezierSplineParamRec::ParamValue(int )


void CubicBezierSplineParamRec::Print(ostream& outFile, long rep, aString& message){
	ParamRec::Print(outFile, rep, message);
	aString outStr("stdev: ");
	outStr.cat(sigma(),6,8);
	outFile<<outStr;
	outStr.clear();
	int pp,node, paramAtNode,prevNode=(*variableParamLookupTable_)[0][0];;
	for(pp=ErrorDistParams();pp<variableParamLookupTable_->len();++pp){
		node = (*variableParamLookupTable_)[pp][0];
		paramAtNode = (*variableParamLookupTable_)[pp][1];
		if(node != prevNode){
			outFile<<outStr<<endl;
			prevNode=node;
			outStr = "node: "; outStr.cat(node);
			}
		outStr.cat("\t");
		spline_->ParamName(node,paramAtNode,outStr);
		outStr.cat(": "); 
		outStr.cat(spline_->ParamValue(node,paramAtNode)); 
		}//pp	
	outFile<<outStr<<endl;
	}//CubicBezierSplineParamRec::Print



int CubicBezierSplineParamRec::SectionAt_x(long double x){
	int sec=0;
	if(spline_ != NULL){
		sec = spline_->SectionAt_x(x);}
	return sec;
	}//CubicBezierSplineParamRec::SectionAt_x(long double )
	
	
void CubicBezierSplineParamRec::SectionsAtNodes(aList<int>& nodelist, aList<int>& newSectionList){
	if(spline_ != NULL){
		spline_->SectionsAtNodes(nodelist,newSectionList);
		}
	}//CubicBezierSplineParamRec::SectionAt_x(long double )
	
	

void CubicBezierSplineParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter
	if(paramNum==0){
			sigma(newVal);}
	 	else{
	 		int node, paramAtNode;
			node = (*variableParamLookupTable_)[paramNum][0];
			paramAtNode = (*variableParamLookupTable_)[paramNum][1];
			spline_->SetParamValue(node,paramAtNode,newVal);
			}//else
	}//end CubicBezierSplineParamRec::SetParamValue(int,double)




void CubicBezierSplineParamRec::SetParamValueAllNodes(int paramNumAtNode, long double newVal){//use for setting deviations & upper/lower limits
	//set value of parameter
	if(spline_ != NULL){
 			spline_->SetParamValueAllNodes(paramNumAtNode,newVal);		
		}
	}//end CubicBezierSplineParamRec::SetParamValue(int,double)


void CubicBezierSplineParamRec::SetToleranceFactor(long double newFactor){
	toleranceFactor_=newFactor;
	if(spline_ != NULL){
		spline_->SetToleranceFactor(newFactor);
		}
	}

void CubicBezierSplineParamRec::SetVariableParamLookupTable(ObjList<aList<int> > *vplt){
	variableParamLookupTable_ = vplt;
	}//CubicBezierSplineParamRec::SetVariableParamLookupTable(ObjList<aList<int> >*)



long double CubicBezierSplineParamRec::sigma(void){
	return sigma_;}	


void CubicBezierSplineParamRec::sigma(long double s){
	sigma_ = s;}	



void CubicBezierSplineParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::TitleString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat("sigma");
	int pp,node,paramAtNode;
	for(pp=ErrorDistParams();pp<variableParamLookupTable_->len();++pp){
		node = (*variableParamLookupTable_)[pp][0];
		paramAtNode = (*variableParamLookupTable_)[pp][1];
		dataline.cat(sepchar);
		dataline.cat("node_");
		dataline.cat((*variableParamLookupTable_)[pp][0]);		
		dataline.cat(".");
		spline_->ParamName(node, paramAtNode, dataline);
		}//pp	
	}//end CubicBezierSplineParamRec::TitleString(aString&, char)







int	operator==(CubicBezierSplineParamRec& p, CubicBezierSplineParamRec& q){
	ParamRec& a = p;
	ParamRec& b = q;
	return ((a == b) && (p.sigma_ == q.sigma_) && (p.spline_==q.spline_) 
				&& (p.variableParamLookupTable_==q.variableParamLookupTable_));
	}//end operator==(CubicBezierSplineParamRec&, CubicBezierSplineParamRec&)


int	operator!=(CubicBezierSplineParamRec& p, CubicBezierSplineParamRec& q){
	return !(p==q);
	}//end operator!=(CubicBezierSplineParamRec&, CubicBezierSplineParamRec&)



#endif


