/******************************************************
*
*	CumNormalDistParamRec.cpp
*
*	Parameters for a normal distribution defined by mean and stdev.
*
*	Use this as a base class for derived classes that add parameters.
*
******************************************************/
#ifndef CumNormalDistParamReccpp
#define CumNormalDistParamReccpp = 1

#include "CumNormalDistParamRec.h"
#include "CumulativeNormalLikelihood.h"
#include "Element.cpp"
#include "aList.cpp"

	
ParamRec* CumNormalDistParamRec::Duplicate(void){
	CumNormalDistParamRec* copy = NEW CumNormalDistParamRec(*this);
	return copy;
	}//end CumNormalDistParamRec::Duplicate()


CumNormalDistParamRec* CumNormalDistParamRec::_Duplicate(void){
	CumNormalDistParamRec* copy = NEW CumNormalDistParamRec(*this);
	return copy;
	}//end CumNormalDistParamRec::_Duplicate()


void CumNormalDistParamRec::SetPointerToDerivedSelf(void){
  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in CumNormalDistParamRec"<<endl;
  	}//end



long double CumNormalDistParamRec::ExpectedValue(aList<long double>& coords){
	return exp(CumulativeNormalProbabilityAtX(coords[0]));
	}//end CumNormalDistParamRec::ExpectedValue(aList<double>&)



long double CumNormalDistParamRec::CumulativeNormalProbabilityAtX(long double x){
	if (x == mu()) return ln(half);
	long double a = x,b = mu();
	if(x>mu()){
		SWAP(a,b);}
	//approximate integration by summation
	const long double dx = ABS(a-b)/(long double)100.0;//integration interval
	long double t, prob, probDensity, intervalDensity = zero;
	for(t=a;t<b;t += dx){
		prob = NormalDistrib(t, mu(), sigma());
		intervalDensity +=  prob * dx;}
	intervalDensity /= sigma();
	if(x<mu()){
			probDensity = half - intervalDensity;}
		else{
			probDensity = half + intervalDensity;}
	if(probDensity == zero){//return value for an arbitrarily small probability, 10^-10
		return ln((long double)1.e-10);}
	return ln(probDensity);	
	}//end CumulativeNormalProbability(double)



long double CumNormalDistParamRec::XforCumulativeNormalProbability(long double prob){
	if (prob == half) {return mu();}
		else if(prob<zero){
			aString err("In XforCumulativeNormalProbability: Can't have probability < 0");
			aError(err);}
		else if(prob>one){
			aString err("In XforCumulativeNormalProbability: Can't have probability > 1");
			aError(err);}
		else{}
	//approximate integration by summation
	long double probDensity = half, intervalDensity = zero, x = mu();
	const long double dx = sigma()/(long double)100.0;//integration interval
	if(prob<half){
		while (probDensity > prob){
			x -= dx;
			long double p_x = NormalDistrib(x, mu(), sigma());
			intervalDensity =  p_x * dx;
			probDensity -= intervalDensity/sigma();
			}
		return x+dx;
		}//end if prob<0.5

	if(prob>half){
		while (probDensity < prob){
			x += dx;
			long double p_x = NormalDistrib(x, mu(), sigma());
			intervalDensity =  p_x * dx;
			probDensity += intervalDensity/sigma();
			}
		return x;
		}
	return zero;//default
	}//end XforCumulativeNormalProbability(double)






void CumNormalDistParamRec::operator=(ParamRec& p){
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class CumNormalDistParamRec
	CumNormalDistParamRec* pp = (CumNormalDistParamRec*) p.pointerToDerivedSelf;
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
	mu_= pp->mu_;
	sigma_ = pp->sigma_;
	}//end  CumNormalDistParamRec::operator=(ParamRec&)





int	operator==(CumNormalDistParamRec& p, CumNormalDistParamRec& q){
	NormalDistParamRec& a = p;
	NormalDistParamRec& b = q;
	return (a == b);
	}//end operator==(CumNormalDistParamRec&, CumNormalDistParamRec&)


int	operator!=(CumNormalDistParamRec& p, CumNormalDistParamRec& q){
	return !(p==q);
	}//end operator!=(CumNormalDistParamRec&, CumNormalDistParamRec&)



#endif


