/*****************************************************
*
*	CumulativeNormalLikelihood.cpp
*
*	This function calculates the ln-likelihood of data point x
*	on a cumulative normal distribution with mean mu and 
*	standard deviation sigma.
*
******************************************************/

#ifndef CumulativeNormalLikelihoodcpp
#define CumulativeNormalLikelihoodcpp = 1

#include "CumulativeNormalLikelihood.h"
#include "CumulativeNormalData.h"
#include "Element.cpp"
#include "aList.cpp"



long double CumulativeNormalLikelihood(long double x, long double mu, long double sigma){
	if (x == mu) return ln(half);
	long double a = x,b = mu;
	if(x>mu){
		SWAP(a,b);}
	//approximate integration by summation
	const long double dx = ABS(a-b)/(long double)5000.0;//integration interval
	long double t, prob, probDensity, intervalDensity = zero;
	for(t=a;t<b;t += dx){
		prob = NormalDistrib(t, mu, sigma);
		intervalDensity +=  prob * dx;}
	intervalDensity /= sigma;
	if(x<mu){
			probDensity = half - intervalDensity;}
		else{
			probDensity = half + intervalDensity;}
	if(probDensity == zero){//return value for an arbitrarily small probability, 10^-10
		return ln((long double)1.0e-10);}
	return ln(probDensity);	
	}//end CumulativeNormalLikelihood


long double CumulativeNormalLikelihood(ParamRec& p, MetropolisHastingsData* data){
	CumulativeNormalData* theData = (CumulativeNormalData*) data;
	long double lnL = zero;
	long double a, d, sumAlive=zero, sumDead=zero;
	for(int i=0;i<theData->len();++i){
    	aList<long double> coords;  coords.Append(theData->dose[i]);
    	long double expected = p.ExpectedValue(coords);
    	expected = MAX(expected, (long double)10.0e-10);//bounds lower limit to logit transform equivalent
    	expected = MIN(expected, one-(long double)10.0e-10);//bounds upper limit to logit transform equivalent
//     	lnL += theData->alive[i] * ln(expected);
//     	lnL += theData->dead[i] * ln(1.0 - expected);
		sumAlive += theData->alive[i];
		sumDead += theData->dead[i];
//     	lnL +=  sumAlive * expected;
//     	lnL += sumDead * (1.0 - expected);
		a = theData->alive[i];
		d = theData->dead[i];
     	lnL += d * ln(expected);
     	lnL += a * ln(one - expected);
   		}//end for i
	p.lnL(lnL);
	return lnL;
	}//end CumulativeNormalLikelihood(ParamRec&, MetropolisHastingsData*)













#endif
