/*************************************************************
*
*	DoseRespClineCumNormParamRec.h
*
*	This fits a set of cumulative normal dose response curves to a Barton cline model.
*
*************************************************************/
#ifndef DoseRespClineCumNormParamReccpp
#define DoseRespClineCumNormParamReccpp = 1

#include "DoseRespClineCumNormParamRec.h"

	
DoseRespClineCumNormParamRec::DoseRespClineCumNormParamRec(void):ClineShapeCWTBParamRec(){
  	SetPointerToDerivedSelf();
  	cout<<"constructing DoseRespClineCumNormParamRec()"<<endl;
  	}


DoseRespClineCumNormParamRec::DoseRespClineCumNormParamRec(double c, double w, 
  					aList<double>& low, aList<double>& high, 
  					DoseResponseParamRec& p, aList<double>& locations
  					):
  				ClineShapeCWTBParamRec(c,w,low,high),
  				doseResponseCurve_(),
  				location_(locations)//,
  				{
  	SetPointerToDerivedSelf();
	c_= c;
	w = w_;
	leftSideScore_ = low;
	rightSideScore_ = high;
  	for(int i=0;i<locations.len();++i){
  		doseResponseCurve_.Append(new DoseResponseParamRec(p));}
//  cout<<"constructing DoseRespClineCumNormParamRec(double...)"<<endl;
  	}  	


DoseRespClineCumNormParamRec::DoseRespClineCumNormParamRec(DoseRespClineCumNormParamRec& p):
  				ClineShapeCWTBParamRec(p),
  				doseResponseCurve_(p.doseResponseCurve_),
  				location_(p.location_)
  				{
  	SetPointerToDerivedSelf();
	lnL_= p.lnL_;
	lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;	
	c_= p.c_;
	w_ = p.w_;
	leftSideScore_= p.leftSideScore_;
	rightSideScore_= p.rightSideScore_;
//  cout<<"constructing DoseRespClineCumNormParamRec"<<endl;
  	}  	


DoseRespClineCumNormParamRec::DoseRespClineCumNormParamRec(DoseRespClineCumNormParamRec* p):
  				ClineShapeCWTBParamRec(*p),
  				doseResponseCurve_(p->doseResponseCurve_),
  				location_(p->location_)
  				{
  	SetPointerToDerivedSelf();
	lnL_= p->lnL_;
	lnLmaxMinusLnL_ = p->lnLmaxMinusLnL_;	
	c_= p->c_;
	w_ = p->w_;
	leftSideScore_= p->leftSideScore_;
	rightSideScore_= p->rightSideScore_;
//  cout<<"constructing DoseRespClineCumNormParamRec"<<endl;
	}



ParamRec* DoseRespClineCumNormParamRec::Duplicate(void){
	DoseRespClineCumNormParamRec* copy = new DoseRespClineCumNormParamRec(*this);
	return copy;
	}//end DoseRespClineCumNormParamRec::Duplicate()


DoseRespClineCumNormParamRec* DoseRespClineCumNormParamRec::_Duplicate(void){
	DoseRespClineCumNormParamRec* copy = new DoseRespClineCumNormParamRec(*this);
	return copy;
	}//end DoseRespClineCumNormParamRec::_Duplicate()


void DoseRespClineCumNormParamRec::SetPointerToDerivedSelf(void){
  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in DoseRespClineCumNormParamRec"<<endl;
  	}//end DoseRespClineCumNormParamRec::SetPointerToDerivedSelf()




int DoseRespClineCumNormParamRec::ChangeDoseMean(int pop, Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//intercepts are estimated separately for each location
	DoseRespClineCumNormParamRec* deviations = (DoseRespClineCumNormParamRec*) paramDeviations->pointerToDerivedSelf;
	DoseRespClineCumNormParamRec* lower = (DoseRespClineCumNormParamRec*) lowerLimits->pointerToDerivedSelf;
	DoseRespClineCumNormParamRec* upper = (DoseRespClineCumNormParamRec*) upperLimits->pointerToDerivedSelf;
	DoseResponseParamRec* devs = (DoseResponseParamRec*) deviations->doseResponseCurve_[pop].pointerToDerivedSelf;
	ParamRec& low = lower->doseResponseCurve_[pop];
	ParamRec& high = upper->doseResponseCurve_[pop];
	DoseResponseParamRec* popCurve = (DoseResponseParamRec*) doseResponseCurve_[pop].pointerToDerivedSelf;
	int validChange = popCurve->ChangeMu(ran, devs->mu(), &low, &high);
	return validChange;
	}//end DoseRespClineCumNormParamRec::ChangeDoseMean(int, Ran3&, ParamRec*,ParamRec*, ParamRec*)



int DoseRespClineCumNormParamRec::ChangeDoseSlope(int pop, Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//slopes are estimated separately for each location
	DoseRespClineCumNormParamRec* deviations = (DoseRespClineCumNormParamRec*) paramDeviations->pointerToDerivedSelf;
	DoseRespClineCumNormParamRec* lower = (DoseRespClineCumNormParamRec*) lowerLimits->pointerToDerivedSelf;
	DoseRespClineCumNormParamRec* upper = (DoseRespClineCumNormParamRec*) upperLimits->pointerToDerivedSelf;
	DoseResponseParamRec* devs = (DoseResponseParamRec*) deviations->doseResponseCurve_[pop].pointerToDerivedSelf;
	ParamRec& low = lower->doseResponseCurve_[pop];
	ParamRec& high = upper->doseResponseCurve_[pop];
	DoseResponseParamRec* popCurve = (DoseResponseParamRec*) doseResponseCurve_[pop].pointerToDerivedSelf;
	int validChange = popCurve->ChangeSigma(ran, devs->sigma(), &low, &high);
	return validChange;
	}//end DoseRespClineCumNormParamRec::ChangeDoseSlope(int, Ran3&, ParamRec*,ParamRec*, ParamRec*)




int DoseRespClineCumNormParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	int n = ClineShapeCWTBParamRec::NumParams();
	if(paramNum<n){//change a shape parameter
		return ClineShapeCWTBParamRec::ChangeParam(paramNum,ran, 
							paramDeviations, lowerLimits, upperLimits);
		}
	paramNum -= n;
	if(paramNum<doseResponseCurve_.len()){//change a mean
		return ChangeDoseMean(paramNum, ran, paramDeviations, lowerLimits, upperLimits);
		}
	paramNum -= doseResponseCurve_.len();
	if(paramNum<doseResponseCurve_.len()){//change a slope
		return ChangeDoseSlope(paramNum, ran, paramDeviations, lowerLimits, upperLimits);
		}
	//default
	aString errstr("bug: can't change param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");
	errstr.cat(NumParams()+1);
	errstr.cat("-parameter dose-response cline model");
	aError(errstr);
	return 0;
	}//end DoseRespClineCumNormParamRec::ChangeParam(int)



void DoseRespClineCumNormParamRec::Clear(void){
	doseResponseCurve_.Clear(); 
  	ClineShapeCWTBParamRec::Clear();
  	}//end DoseRespClineCumNormParamRec::Clear()



double DoseRespClineCumNormParamRec::ExpectedValue(aList<double>& coords){
	return ClineShapeCWTBParamRec::ExpectedValue(coords);
	}//end DoseRespClineCumNormParamRec::ExpectedValue(aList<double>&)



int DoseRespClineCumNormParamRec::locations(void){
	//average limiting score on left side
	return location_.len();
	}//end DoseRespClineCumNormParamRec::locations()



int DoseRespClineCumNormParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//use type conversion that circumvents usual virtual base class restrictions
//	if (sigmaLD50AroundCline_ <= 0.0){return 0;}
	DoseRespClineCumNormParamRec* lower = (DoseRespClineCumNormParamRec*) lowerLimits->pointerToDerivedSelf;
	DoseRespClineCumNormParamRec* upper = (DoseRespClineCumNormParamRec*) upperLimits->pointerToDerivedSelf;
	for(int loc=0;loc<locations();++loc){
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[loc].pointerToDerivedSelf;
		ParamRec& low = lower->doseResponseCurve_[loc];
		ParamRec& high = upper->doseResponseCurve_[loc];
		if(0 == actual->MeetsParamCriteria(&low, &high)){return 0;}
		}
	return ClineShapeCWTBParamRec::MeetsParamCriteria (lowerLimits, upperLimits);
	}//end DoseRespClineCumNormParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)



void DoseRespClineCumNormParamRec::MultiplyParam(int paramNum, double factor){
	//use to rescale a param deviation
	int n = ClineShapeCWTBParamRec::NumParams();
	if(paramNum<n){
		ClineShapeCWTBParamRec::MultiplyParam(paramNum,factor);
		return;
		}
	paramNum -= n;
	if(paramNum < locations()){//change a slope
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[paramNum].pointerToDerivedSelf;
		double mean = actual->mu();
		actual->mu(mean * factor);
		return;
		}
	paramNum -= locations();
	if(paramNum < locations()){//change an intercept
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[paramNum].pointerToDerivedSelf;
		double stdev = actual->sigma();
		actual->sigma(stdev * factor);
		return;
		}	
	aString errstr("bug: can't rescale param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");
	errstr.cat(NumParams()+1);
	errstr.cat("-parameter dose-response cline model");
	aError(errstr);
	}//DoseRespClineCumNormParamRec::MultiplyParam(int, double)



int DoseRespClineCumNormParamRec::NumParams(void){//parameters that vary
	int n = ClineShapeCWTBParamRec::NumParams();
	n += 2 * locations();//each dose-response slope & mean estimated separately
	return n;
	}//end DoseRespClineCumNormParamRec::NumParams()


int DoseRespClineCumNormParamRec::NumDerivedParams(void){
	//number of parameters that are functions of varying parameters 
	return 0;
	}//end DoseRespClineCumNormParamRec::NumDerivedParams()



aString* DoseRespClineCumNormParamRec::ParamName(int paramNum){
	int n = ClineShapeCWTBParamRec::NumParams();
	if(paramNum<n){
		return ClineShapeCWTBParamRec::ParamName(paramNum);
		}
	paramNum -= n;
	aString* newStr = new aString();
	if(paramNum<doseResponseCurve_.len()){
		newStr->cat("LD50_x=");
		newStr->cat(location_[paramNum],6,2);
		newStr->CullChar(' ');
		return newStr;
		}
	paramNum-=doseResponseCurve_.len();
	if(paramNum<doseResponseCurve_.len()){
			newStr->cat("sigma_x=");
			newStr->cat(location_[paramNum],6,2);
			newStr->CullChar(' ');
			}
		else{
			//default
			aString errstr("bug: can't return name of param #");
			errstr.cat(paramNum+1);
			errstr.cat(" in a ");
			errstr.cat(NumParams()+1);
			errstr.cat("-parameter dose-response cline model");
			aError(errstr);
			}
	return newStr;
	}//end DoseRespClineCumNormParamRec::ParamName(int )



void DoseRespClineCumNormParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ClineShapeCWTBParamRec::ParamString(dataline, sepchar);
	for(int loc=0;loc<locations();++loc){
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[loc].pointerToDerivedSelf;
		aString doseStr(sepchar);
		doseStr.cat(actual->lnL(),6,9);
		doseStr.cat(sepchar);
		doseStr.cat((double)actual->LD50(),6,9);
		doseStr.cat(sepchar);
		doseStr.cat((double)actual->sigma(),6,9);
		dataline.cat(doseStr);
		}
	}//end DoseRespClineCumNormParamRec::ParamString(aString&, char)



double DoseRespClineCumNormParamRec::ParamValue(int paramNum){
	int n = ClineShapeCWTBParamRec::NumParams();
	if(paramNum<n){
		return ClineShapeCWTBParamRec::ParamValue(paramNum);
		}
	paramNum -= n;
	if(paramNum<doseResponseCurve_.len()){
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[paramNum].pointerToDerivedSelf;
		return actual->mu();
		}
	paramNum-=doseResponseCurve_.len();
	if(paramNum<doseResponseCurve_.len()){
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[paramNum].pointerToDerivedSelf;
		return actual->sigma();
		}
	//default
	aString errstr("bug: can't return value of param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");
	errstr.cat(NumParams()+1);
	errstr.cat("-parameter dose-response cline model");
	aError(errstr);
	return 0.0;
	}//end DoseRespClineCumNormParamRec::ParamValue(int )


void DoseRespClineCumNormParamRec::Print(ostream& outFile, long rep, aString& message){
	ClineShapeCWTBParamRec::Print(outFile, rep, message);
	
	for(int loc=0;loc<locations();++loc){
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[loc].pointerToDerivedSelf;
		aString outStr("loc:");
		outStr.cat(location_[loc],4,2);
		outStr.separator(1);
		outStr.cat("LD50:");
		outStr.cat((double)actual->LD50(),6,9);
		outStr.separator(2);
		outStr.cat("sigma:");
		outStr.cat((double)actual->sigma(),6,9);
		outFile<<outStr<<endl;
		}//end for loc
	}//DoseRespClineCumNormParamRec::Print



void DoseRespClineCumNormParamRec::SetParamValue(int paramNum, long double newVal){
	int n = ClineShapeCWTBParamRec::NumParams();
	if(paramNum<n){
		ClineShapeCWTBParamRec::SetParamValue(paramNum,newVal);
		return;
		}
	paramNum -= n;
	if(paramNum < locations()){
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[paramNum].pointerToDerivedSelf;
		actual->mu(newVal);
		return;
		}
	paramNum -= locations();
	if(paramNum < locations()){
			DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[paramNum].pointerToDerivedSelf;
			actual->sigma(newVal);
			}
		else{
			aString errstr("bug: can't set value of param #");
			errstr.cat(paramNum+1);
			errstr.cat(" in a ");
			errstr.cat(NumParams()+1);
			errstr.cat("-parameter dose-response cline model");
			aError(errstr);
			}
	}//end DoseRespClineCumNormParamRec::SetParamValue(int,double)



void DoseRespClineCumNormParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ClineShapeCWTBParamRec::TitleString(dataline, sepchar);
	for(int loc=0;loc<location_.len();++loc){
		aString doseStr(sepchar);
		doseStr.cat("LnL_x=");
		doseStr.cat(location_[loc],6,2);
		doseStr.cat(sepchar);
		doseStr.cat("LD50_x=");
		doseStr.cat(location_[loc],6,2);
		doseStr.cat(sepchar);
		doseStr.cat("sigma_x=");
		doseStr.cat(location_[loc],6,2);
		if(sepchar != ' '){doseStr.CullChar(' ');}
		dataline.cat(doseStr);
		}
	}//end DoseRespClineCumNormParamRec::TitleString(aString&, char)



int DoseRespClineCumNormParamRec::TotalParams(void){
	return NumParams() + NumDerivedParams();
	}//end DoseRespClineCumNormParamRec::TotalParams()




void DoseRespClineCumNormParamRec::operator=(ParamRec& p){
//	ClineShapeCWTBParamRec::operator=(p);
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means pp must actually point to an object of class DoseRespClineLogitParamRec
	DoseRespClineCumNormParamRec* pp = (DoseRespClineCumNormParamRec*) p.pointerToDerivedSelf;
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
	c_= pp->c_;
	w_ = pp->w_;
	leftSideScore_= pp->leftSideScore_;
	rightSideScore_ = pp->rightSideScore_;
	doseResponseCurve_= pp->doseResponseCurve_;
	location_= pp->location_;
	}//end  DoseRespClineCumNormParamRec::operator=(ParamRec&)





int	operator==(DoseRespClineCumNormParamRec& p, DoseRespClineCumNormParamRec& q){
	ClineShapeCWTBParamRec& a = p;
	ClineShapeCWTBParamRec& b = q;
	return ((a == b) && 
			(p.doseResponseCurve_ == q.doseResponseCurve_) &&
			(p.location_ == q.location_)
			);
	}//end operator==(DoseRespClineCumNormParamRec&, DoseRespClineCumNormParamRec&)


int	operator!=(DoseRespClineCumNormParamRec& p, DoseRespClineCumNormParamRec& q){
	return !(p==q);
	}//end operator!=(DoseRespClineCumNormParamRec&, DoseRespClineCumNormParamRec&)



#endif


