/*************************************************************
*
*	DoseRespHeritabilityParamRec.h
*
*	This fits a set of cumulative normal dose response curves to a Barton cline model.
*
*************************************************************/
#ifndef DoseRespHeritabilityParamReccpp
#define DoseRespHeritabilityParamReccpp = 1

#include "DoseRespHeritabilityParamRec.h"

	
DoseRespHeritabilityParamRec::DoseRespHeritabilityParamRec(void): slope_(0.0),
  	intercept_(0.0){
  	SetPointerToDerivedSelf();
  	cout<<"constructing DoseRespHeritabilityParamRec()"<<endl;
  	}


DoseRespHeritabilityParamRec::DoseRespHeritabilityParamRec(double a, double b, 
  					DoseResponseParamRec& p, aList<double>& midparentList
  					):
  				slope_(b),
  				intercept_(a),
  				doseResponseCurve_(),
  				midparent_(midparentList)//,
  				{
  	SetPointerToDerivedSelf();
  	for(int i=0;i<midparents();++i){
  		doseResponseCurve_.Append(new DoseResponseParamRec(p));}
//  cout<<"constructing DoseRespHeritabilityParamRec(double...)"<<endl;
  	}  	


DoseRespHeritabilityParamRec::DoseRespHeritabilityParamRec(DoseRespHeritabilityParamRec& p):
				slope_(p.slope_),
				intercept_(p.intercept_),
  				doseResponseCurve_(p.doseResponseCurve_),
  				midparent_(p.midparent_)
  				{
  	SetPointerToDerivedSelf();
	lnL_= p.lnL_;
	lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;	
//  cout<<"constructing DoseRespHeritabilityParamRec"<<endl;
  	}  	


DoseRespHeritabilityParamRec::DoseRespHeritabilityParamRec(DoseRespHeritabilityParamRec* p):
				slope_(p->slope_),
				intercept_(p->intercept_),
  				doseResponseCurve_(p->doseResponseCurve_),
  				midparent_(p->midparent_)
  				{
  	SetPointerToDerivedSelf();
	lnL_= p->lnL_;
	lnLmaxMinusLnL_ = p->lnLmaxMinusLnL_;	
//  cout<<"constructing DoseRespHeritabilityParamRec"<<endl;
	}



ParamRec* DoseRespHeritabilityParamRec::Duplicate(void){
	DoseRespHeritabilityParamRec* copy = new DoseRespHeritabilityParamRec(*this);
	return copy;
	}//end DoseRespHeritabilityParamRec::Duplicate()


DoseRespHeritabilityParamRec* DoseRespHeritabilityParamRec::_Duplicate(void){
	DoseRespHeritabilityParamRec* copy = new DoseRespHeritabilityParamRec(*this);
	return copy;
	}//end DoseRespHeritabilityParamRec::_Duplicate()


void DoseRespHeritabilityParamRec::SetPointerToDerivedSelf(void){
  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in DoseRespHeritabilityParamRec"<<endl;
  	}//end DoseRespHeritabilityParamRec::SetPointerToDerivedSelf()




int DoseRespHeritabilityParamRec::ChangeDoseMean(int pop, Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//intercepts are estimated separately for each location
	DoseRespHeritabilityParamRec* deviations = (DoseRespHeritabilityParamRec*) paramDeviations->pointerToDerivedSelf;
	DoseRespHeritabilityParamRec* lower = (DoseRespHeritabilityParamRec*) lowerLimits->pointerToDerivedSelf;
	DoseRespHeritabilityParamRec* upper = (DoseRespHeritabilityParamRec*) upperLimits->pointerToDerivedSelf;
	DoseResponseParamRec* devs = (DoseResponseParamRec*) deviations->doseResponseCurve_[pop].pointerToDerivedSelf;
	ParamRec& low = lower->doseResponseCurve_[pop];
	ParamRec& high = upper->doseResponseCurve_[pop];
	DoseResponseParamRec* popCurve = (DoseResponseParamRec*) doseResponseCurve_[pop].pointerToDerivedSelf;
	int validChange = popCurve->ChangeMu(ran, devs->mu(), &low, &high);
	return validChange;
	}//end DoseRespHeritabilityParamRec::ChangeDoseMean(int, Ran3&, ParamRec*,ParamRec*, ParamRec*)



int DoseRespHeritabilityParamRec::ChangeDoseSlope(int pop, Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//slopes are estimated separately for each location
	DoseRespHeritabilityParamRec* deviations = (DoseRespHeritabilityParamRec*) paramDeviations->pointerToDerivedSelf;
	DoseRespHeritabilityParamRec* lower = (DoseRespHeritabilityParamRec*) lowerLimits->pointerToDerivedSelf;
	DoseRespHeritabilityParamRec* upper = (DoseRespHeritabilityParamRec*) upperLimits->pointerToDerivedSelf;
	DoseResponseParamRec* devs = (DoseResponseParamRec*) deviations->doseResponseCurve_[pop].pointerToDerivedSelf;
	ParamRec& low = lower->doseResponseCurve_[pop];
	ParamRec& high = upper->doseResponseCurve_[pop];
	DoseResponseParamRec* popCurve = (DoseResponseParamRec*) doseResponseCurve_[pop].pointerToDerivedSelf;
	int validChange = popCurve->ChangeSigma(ran, devs->sigma(), &low, &high);
	return validChange;
	}//end DoseRespHeritabilityParamRec::ChangeDoseSlope(int, Ran3&, ParamRec*,ParamRec*, ParamRec*)



int DoseRespHeritabilityParamRec::ChangeIntercept(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	DoseRespHeritabilityParamRec* dev = (DoseRespHeritabilityParamRec*) paramDeviations->pointerToDerivedSelf;
	int validChange=false;
	double oldIntercept = intercept_;
	do{
		intercept_ = ran.NormDev(oldIntercept, dev->intercept());
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end DoseRespHeritabilityParamRec::ChangeIntercept(Ran3&, ParamRec*, ParamRec*, ParamRec*)



int DoseRespHeritabilityParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
	if(paramNum==0){//change a linear fit parameter
		return ChangeIntercept(ran, paramDeviations,lowerLimits, upperLimits);
		}
	if(paramNum==1){//change a linear fit parameter
		return ChangeSlope(ran, paramDeviations,lowerLimits, upperLimits);
		}
	paramNum -= 2;
	if(paramNum<doseResponseCurve_.len()){//change a mean
		return ChangeDoseMean(paramNum, ran, paramDeviations, lowerLimits, upperLimits);
		}
	paramNum -= doseResponseCurve_.len();
	if(paramNum<doseResponseCurve_.len()){//change a slope
		return ChangeDoseSlope(paramNum, ran, paramDeviations, lowerLimits, upperLimits);
		}
	//default
	aString errstr("bug: can't change param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");
	errstr.cat(NumParams()+1);
	errstr.cat("-parameter dose-response heritability model");
	aError(errstr);
	return 0;
	}//end DoseRespHeritabilityParamRec::ChangeParam(int)



int DoseRespHeritabilityParamRec::ChangeSlope(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	DoseRespHeritabilityParamRec* dev = (DoseRespHeritabilityParamRec*) paramDeviations->pointerToDerivedSelf;
	int validChange=false;
	double oldSlope = slope_;
	do{
		slope_ = ran.NormDev(oldSlope, dev->slope());
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end DoseRespHeritabilityParamRec::ChangeSlope(Ran3&, ParamRec*, ParamRec*, ParamRec*)




void DoseRespHeritabilityParamRec::Clear(void){
	doseResponseCurve_.Clear();
	midparent_.Clear(); 
  	}//end DoseRespHeritabilityParamRec::Clear()



double DoseRespHeritabilityParamRec::ExpectedValue(aList<double>& coords){
	double x = coords[0];
	double y = intercept_ + slope_ * x;
	return y;
	}//end DoseRespHeritabilityParamRec::ExpectedValue(aList<double>&)



double DoseRespHeritabilityParamRec::heritability(void){
	return slope_;//change this for different regression conditions
	}//end DoseRespHeritabilityParamRec::heritability()


double DoseRespHeritabilityParamRec::intercept(void){
	return intercept_;
	}//end DoseRespHeritabilityParamRec::intercept()


void DoseRespHeritabilityParamRec::intercept(double a){
	intercept_ = a;
	}//end DoseRespHeritabilityParamRec::intercept(double )


int DoseRespHeritabilityParamRec::midparents(void){
	//average limiting score on left side
	return midparent_.len();
	}//end DoseRespHeritabilityParamRec::locations()



int DoseRespHeritabilityParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//use type conversion that circumvents usual virtual base class restrictions
	if (ABS(slope_) > 1.0e10){return 0;}//not too steep
	DoseRespHeritabilityParamRec* lower = (DoseRespHeritabilityParamRec*) lowerLimits->pointerToDerivedSelf;
	DoseRespHeritabilityParamRec* upper = (DoseRespHeritabilityParamRec*) upperLimits->pointerToDerivedSelf;
	for(int m=0;m<midparents();++m){
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[m].pointerToDerivedSelf;
		ParamRec& low = lower->doseResponseCurve_[m];
		ParamRec& high = upper->doseResponseCurve_[m];
		if(0 == actual->MeetsParamCriteria(&low, &high)){return 0;}
		}
	return 1;
	}//end DoseRespHeritabilityParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)



void DoseRespHeritabilityParamRec::MultiplyParam(int paramNum, double factor){
	//use to rescale a param deviation
	if(paramNum==0){
		intercept_ *= factor;
		return;}
	if(paramNum==1){
		slope_ *= factor;
		return;}
	paramNum -= 2;
	if(paramNum < midparents()){//change a slope
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[paramNum].pointerToDerivedSelf;
		double mean = actual->mu();
		actual->mu(mean * factor);
		return;
		}
	paramNum -= midparents();
	if(paramNum < midparents()){//change an intercept
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[paramNum].pointerToDerivedSelf;
		double stdev = actual->sigma();
		actual->sigma(stdev * factor);
		return;
		}	
	aString errstr("bug: can't rescale param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");
	errstr.cat(NumParams()+1);
	errstr.cat("-parameter dose-response heritability model");
	aError(errstr);
	}//DoseRespHeritabilityParamRec::MultiplyParam(int, double)



int DoseRespHeritabilityParamRec::NumParams(void){//parameters that vary
	int n = 2 + 2 * midparents();//each dose-response slope & mean estimated separately
	return n;
	}//end DoseRespHeritabilityParamRec::NumParams()


int DoseRespHeritabilityParamRec::NumDerivedParams(void){
	//number of parameters that are functions of varying parameters 
	return 1;//heritability 
	}//end DoseRespHeritabilityParamRec::NumDerivedParams()



aString* DoseRespHeritabilityParamRec::ParamName(int paramNum){
	if(paramNum==0){
		return new aString("intercept");
		}
	if(paramNum==1){
		return new aString("slope");
		}
	paramNum -= 2;
	aString* newStr = new aString();
	if(paramNum<doseResponseCurve_.len()){
		newStr->cat("LD50_m=");
		newStr->cat(midparent_[paramNum],6,2);
		newStr->CullChar(' ');
		return newStr;
		}
	paramNum-=doseResponseCurve_.len();
	if(paramNum<doseResponseCurve_.len()){
		newStr->cat("sigma_m=");
		newStr->cat(midparent_[paramNum],6,2);
		newStr->CullChar(' ');
		return newStr;
		}
	paramNum-=doseResponseCurve_.len();
	if(paramNum==0){
		return new aString("heritability");
		}
	//default
	aString errstr("bug: can't return name of param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");
	errstr.cat(NumParams()+1);
	errstr.cat("-parameter dose-response heritability model");
	aError(errstr);
	return newStr;
	}//end DoseRespHeritabilityParamRec::ParamName(int )



void DoseRespHeritabilityParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::ParamString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat("heritability");
	dataline.cat(sepchar);
	dataline.cat("slope");
	dataline.cat(sepchar);
	dataline.cat("intercept");
	for(int m=0;m<midparents();++m){
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[m].pointerToDerivedSelf;
		aString doseStr(sepchar);
		doseStr.cat(actual->lnL(),6,9);
		doseStr.cat(sepchar);
		doseStr.cat((double)actual->LD50(),6,9);
		doseStr.cat(sepchar);
		doseStr.cat((double)actual->sigma(),6,9);
		dataline.cat(doseStr);
		}
	}//end DoseRespHeritabilityParamRec::ParamString(aString&, char)



double DoseRespHeritabilityParamRec::ParamValue(int paramNum){
	if(paramNum==0){
		return intercept();
		}
	if(paramNum==1){
		return slope();
		}
	paramNum -= 2;
	if(paramNum<doseResponseCurve_.len()){
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[paramNum].pointerToDerivedSelf;
		return actual->mu();
		}
	paramNum-=doseResponseCurve_.len();
	if(paramNum<doseResponseCurve_.len()){
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[paramNum].pointerToDerivedSelf;
		return actual->sigma();
		}
	paramNum-=doseResponseCurve_.len();
	if(paramNum==0){
		return heritability();
		}
	//default
	aString errstr("bug: can't return value of param #");
	errstr.cat(paramNum+1);
	errstr.cat(" in a ");
	errstr.cat(NumParams()+1);
	errstr.cat("-parameter dose-response heritability model");
	aError(errstr);
	return 0.0;
	}//end DoseRespHeritabilityParamRec::ParamValue(int )



void DoseRespHeritabilityParamRec::Print(ostream& outFile, long rep, aString& message){
	ParamRec::Print(outFile, rep, message);
	aString outline("h^2: ");
	outline.cat((double) heritability(),2,6);
	outline.separator(2);
	outline.cat("slope: ");
	outline.cat((double) slope(),2,6);
	outline.separator(2);
	outline.cat("intercept: ");
	outline.cat((double) intercept(),6,6);
	outFile<<outline<<endl;
	
	for(int m=0;m<midparents();++m){
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[m].pointerToDerivedSelf;
		aString outStr("midparents:");
		outStr.cat(midparent_[m],4,2);
		outStr.separator(1);
		outStr.cat("LD50:");
		outStr.cat((double)actual->LD50(),6,6);
		outStr.separator(2);
		outStr.cat("sigma:");
		outStr.cat((double)actual->sigma(),6,6);
		outFile<<outStr<<endl;
		}//end for loc
	}//DoseRespHeritabilityParamRec::Print



void DoseRespHeritabilityParamRec::SetParamValue(int paramNum, long double newVal){
	if(paramNum==0){
		intercept(newVal);
		}
	if(paramNum==1){
		slope(newVal);
		}
	paramNum -= 2;
	if(paramNum < midparents()){
		DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[paramNum].pointerToDerivedSelf;
		actual->mu(newVal);
		return;
		}
	paramNum -= midparents();
	if(paramNum < midparents()){
			DoseResponseParamRec* actual = (DoseResponseParamRec*) doseResponseCurve_[paramNum].pointerToDerivedSelf;
			actual->sigma(newVal);
			}
		else{
			aString errstr("bug: can't set value of param #");
			errstr.cat(paramNum+1);
			errstr.cat(" in a ");
			errstr.cat(NumParams()+1);
			errstr.cat("-parameter dose-response heritability model");
			aError(errstr);
			}
	}//end DoseRespHeritabilityParamRec::SetParamValue(int,double)



double DoseRespHeritabilityParamRec::slope(void){
	return slope_;
	}//end DoseRespHeritabilityParamRec::slope()


void DoseRespHeritabilityParamRec::slope(double b){
	slope_ = b;
	}//end DoseRespHeritabilityParamRec::slope(double )


void DoseRespHeritabilityParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::TitleString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat((double)heritability(),2,9);
	dataline.cat(sepchar);
	dataline.cat((double)slope(),2,9);
	dataline.cat(sepchar);
	dataline.cat((double)intercept(),2,9);
	for(int m=0;m<midparent_.len();++m){
		aString doseStr(sepchar);
		doseStr.cat("LnL_m=");
		doseStr.cat(midparent_[m],6,2);
		doseStr.cat(sepchar);
		doseStr.cat("LD50_m=");
		doseStr.cat(midparent_[m],6,2);
		doseStr.cat(sepchar);
		doseStr.cat("sigma_m=");
		doseStr.cat(midparent_[m],6,2);
		if(sepchar != ' '){doseStr.CullChar(' ');}
		dataline.cat(doseStr);
		}
	}//end DoseRespHeritabilityParamRec::TitleString(aString&, char)



int DoseRespHeritabilityParamRec::TotalParams(void){
	return NumParams() + NumDerivedParams();
	}//end DoseRespHeritabilityParamRec::TotalParams()




void DoseRespHeritabilityParamRec::operator=(ParamRec& p){
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means pp must actually point to an object of class DoseRespClineLogitParamRec
	DoseRespHeritabilityParamRec* pp = (DoseRespHeritabilityParamRec*) p.pointerToDerivedSelf;
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
	slope_= pp->slope_;
	intercept_ = pp->intercept_;
	doseResponseCurve_= pp->doseResponseCurve_;
	midparent_= pp->midparent_;
	}//end  DoseRespHeritabilityParamRec::operator=(ParamRec&)





int	operator==(DoseRespHeritabilityParamRec& p, DoseRespHeritabilityParamRec& q){
	return ((p.slope_ == q.slope_) && 
			(p.intercept_ == q.intercept_) && 
			(p.doseResponseCurve_ == q.doseResponseCurve_) &&
			(p.midparent_ == q.midparent_)
			);
	}//end operator==(DoseRespHeritabilityParamRec&, DoseRespHeritabilityParamRec&)


int	operator!=(DoseRespHeritabilityParamRec& p, DoseRespHeritabilityParamRec& q){
	return !(p==q);
	}//end operator!=(DoseRespHeritabilityParamRec&, DoseRespHeritabilityParamRec&)



#endif


