/******************************************************
*
*	NormalDistParamRec.cpp
*
*	Parameters for a normal distribution defined by mean and stdev.
*
*	Use this as a base class for derived classes that add parameters.
*
******************************************************/
#ifndef NormalDistParamReccpp
#define NormalDistParamReccpp = 1

#include "NormalDistParamRec.h"
#include "CumulativeNormalLikelihood.h"
#include "Element.cpp"
#include "aList.cpp"


ParamRec* NormalDistParamRec::Duplicate(void){
	NormalDistParamRec* copy = NEW NormalDistParamRec(*this);
	return copy;
	}//end NormalDistParamRec::Duplicate()


NormalDistParamRec* NormalDistParamRec::_Duplicate(void){
	NormalDistParamRec* copy = NEW NormalDistParamRec(*this);
	return copy;
	}//end NormalDistParamRec::_Duplicate()


long NormalDistParamRec::GetPointerToDerivedSelf(void){
  	return (long) this;
  	}//end

//void NormalDistParamRec::SetPointerToDerivedSelf(void){
//  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in NormalDistParamRec"<<endl;
//  	}//end


long double NormalDistParamRec::mu(void){
	return mu_;}


void NormalDistParamRec::mu(long double m){
	mu_ = m;}


int NormalDistParamRec::ChangeMu(Ran3& ran, double sigmaForNewMu, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false;
	long double oldMu = mu_;
	do{
		mu_ = ran.NormDev(oldMu, sigmaForNewMu);
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end NormalDistParamRec::ChangeCenter(Ran3&, double, ParamRec*, ParamRec*)


int NormalDistParamRec::ChangeParam(int paramNum, Ran3& ran, 
							ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits,long double weight){
	//return 0 for failure, 1 for success
	//use type conversion that circumvents usual virtual base class restrictions
//	NormalDistParamRec* deviations = (NormalDistParamRec*) paramDeviations->pointerToDerivedSelf;
	NormalDistParamRec* deviations = (NormalDistParamRec*) paramDeviations->GetPointerToDerivedSelf();
	switch (paramNum){
	 case 0:
	 	 return ChangeMu(ran, deviations->mu()*weight, lowerLimits, upperLimits);
	 	 break;
	 case 1:
	 	 return ChangeSigma(ran, deviations->sigma()*weight, lowerLimits, upperLimits);
	 	 break;
	 default:
		aString errstr("bug: can't change normal distribution param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 2-parameter distribution");
		aError(errstr);
	 	return 0;
	 	}
	 return 0;
	}//end NormalDistParamRec::ChangeParam(int)

/*
int NormalDistParamRec::ChangeParams(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	int validchange = false;
	int paramToChange = RandomParameter(ran);
	do{
		validchange = ChangeParam(paramToChange, ran, 
										paramDeviations, lowerLimits, upperLimits);
		}while(!(validchange));
	return paramToChange;
	}//end NormalDistParamRec::ChangeParams(Ran3&, ParamRec*, ParamRec*, ParamRec*)
*/

int NormalDistParamRec::ChangeSigma(Ran3& ran, double sigmaForNewSigma, 
							ParamRec* lowerLimits, ParamRec* upperLimits){
	//use normal distribution to vary center
	//return 0 for failure, 1 for success
	int validChange=false;
	long double oldSigma = sigma_;
	do{
		sigma_ = ran.NormDev(oldSigma, sigmaForNewSigma);
		validChange = MeetsParamCriteria(lowerLimits, upperLimits);
		}while(!(validChange));
	return validChange;
	}//end NormalDistParamRec::ChangeWidth(Ran3&, double, ParamRec*, ParamRec*)


long double NormalDistParamRec::ExpectedValue(aList<long double>& coords){
	return NormalDistrib(coords[0],mu_,sigma_);
	}//end NormalDistParamRec::ExpectedValue(aList<double>&)




int NormalDistParamRec::MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits){
	//use type conversion that circumvents usual virtual base class restrictions
//	NormalDistParamRec* lower = (NormalDistParamRec*) lowerLimits->pointerToDerivedSelf;
//	NormalDistParamRec* upper = (NormalDistParamRec*) upperLimits->pointerToDerivedSelf;
	NormalDistParamRec* lower = (NormalDistParamRec*) lowerLimits->GetPointerToDerivedSelf();
	NormalDistParamRec* upper = (NormalDistParamRec*) upperLimits->GetPointerToDerivedSelf();
	if(mu_ <lower->mu_){return 0;}
		else if (mu_ > upper->mu_){return 0;}
		else {}
	if(sigma_ <= 0.0){return 0;}//must be positive
	return 1;
	}//end NormalDistParamRec::MeetsParamCriteria(ParamRec*, ParamRec*)


void NormalDistParamRec::MultiplyParam(int paramNum, long double factor){
	//use to rescale a param deviation
	switch (paramNum){
	 case 0:
	 	 mu_ *= factor;
	 	 break;
	 case 1:
	 	 sigma_ *= factor;
	 	 break;
	 default:
		aString errstr("bug: can't rescale param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 2-parameter normal distribution");
		aError(errstr);
	 	}
	}//NormalDistParamRec::MultiplyParam(int, double)


int NormalDistParamRec::NumParams(void){
	//number of parameters that vary
	return 2;
	}//end NormalDistParamRec::NumParams()


int NormalDistParamRec::NumDerivedParams(void){
	//number of parameters that are functions of varying parameters 
	//override in derived classes
	return 0;
	}//ParamRec::NumDerivedParams()



aString* NormalDistParamRec::ParamName(int paramNum){
	switch (paramNum){
	 case 0:
	 	 return NEW aString("mean");
	 	 break;
	 case 1:
	 	 return NEW aString("stdev");
	 	 break;
	 default:
		aString errstr("bug: can't return name of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 2-parameter normal distribution");
		aError(errstr);
	 	}
	return NEW aString();
	}//end NormalDistParamRec::ParamName(int )



void NormalDistParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::ParamString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat(mu_,6,15);
	dataline.cat(sepchar);
	dataline.cat(sigma_,6,15);
	}//end NormalDistParamRec::ParamString(aString&, char)



long double NormalDistParamRec::ParamValue(int paramNum){
	//returns value of parameter
	switch (paramNum){
	 case 0:
	 	 return mu();
	 	 break;
	 case 1:
	 	 return sigma();
	 	 break;
	 default:
		aString errstr("bug: can't return value of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 2-parameter normal distribution");
		aError(errstr);
	 	}
	return 0.0;
	}//end NormalDistParamRec::ParamValue(int )


void NormalDistParamRec::Print(ostream& outFile, long rep, aString& message){
	ParamRec::Print(outFile, rep, message);
	aString outStr("mean: ");
	outStr.cat(mu(),6,8);
	outStr.separator(5);
	outStr.cat("stdev: ");
	outStr.cat(sigma(),6,8);
	outFile<<outStr<<endl;
	}//NormalDistParamRec::Print


void NormalDistParamRec::SetParamValue(int paramNum, long double newVal){
	//set value of parameter
	switch (paramNum){
	 case 0:
	 	 mu(newVal);
	 	 break;
	 case 1:
	 	 sigma(newVal);
	 	 break;
	 default:
		aString errstr("bug: can't set value of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 2-parameter normal distribution");
		aError(errstr);
	 	}
	}//end NormalDistParamRec::SetParamValue(int,double)



void NormalDistParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::TitleString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat("mean");
	dataline.cat(sepchar);
	dataline.cat("stdev");
	}//end NormalDistParamRec::TitleString(aString&, char)




long double NormalDistParamRec::sigma(void){
	return sigma_;}	


void NormalDistParamRec::sigma(long double s){
	sigma_ = s;}	




void NormalDistParamRec::operator=(ParamRec& p){
	//uses type conversion that circumvents usual virtual base class restrictions
	//This means p must actually point to an object of class NormalDistParamRec
//	NormalDistParamRec* pp = (NormalDistParamRec*) p.pointerToDerivedSelf;
	NormalDistParamRec* pp = (NormalDistParamRec*) p.GetPointerToDerivedSelf();
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
	mu_= pp->mu_;
	sigma_ = pp->sigma_;
	}//end  NormalDistParamRec::operator=(ParamRec&)





int	operator==(NormalDistParamRec& p, NormalDistParamRec& q){
	ParamRec& a = p;
	ParamRec& b = q;
	return ((a == b) && (p.mu_ == q.mu_) && (p.sigma_ == q.sigma_));
	}//end operator==(NormalDistParamRec&, NormalDistParamRec&)


int	operator!=(NormalDistParamRec& p, NormalDistParamRec& q){
	return !(p==q);
	}//end operator!=(NormalDistParamRec&, NormalDistParamRec&)



#endif


