/***************************************************

	 GeneticPopulation.cpp

	Handles population processes for genetic individuals	

***************************************************/

#ifndef GeneticPopulationcp
#define GeneticPopulationcp = 1

#include "GeneticPopulation.h"
#include "GeneticGridCell.h"
#include "Element.cpp"
#include "aList.cpp"
#include "RandomList.h"
#include "StatList.cpp"
#include "GeneContainers.cpp"


//CONSTRUCTORS & DESTRUCTORS

GeneticPopulation::GeneticPopulation(long id, int species, GridCell* g, long k):
		Population(id,species,g,k), hetObs_(0), hetExp_(0), allelesCurrent_(0), 
		sampleSizeCurrent_(0), allFreqCurrent_(0), hetExpDataCurrent_(0), 
		hetObsDataCurrent_(0){}
		
		
GeneticPopulation::GeneticPopulation(long id):
		Population(id), hetObs_(0), hetExp_(0), allelesCurrent_(0), 
		sampleSizeCurrent_(0), allFreqCurrent_(0), hetExpDataCurrent_(0), 
		hetObsDataCurrent_(0){}


GeneticPopulation::GeneticPopulation(GeneticPopulation& p)://copy constructor
		Population(p),//copies from Population
		allelesInPop_(p.allelesInPop_),
		allFreq_(p.allFreq_),
		hetObsLoc_(p.hetObsLoc_),
		hetExpLoc_(p.hetExpLoc_),
		hetObs_(p.hetObs_),
		hetExp_(p.hetExp_),
	    allelesCurrent_(p.allelesCurrent_),
	    sampleSizeCurrent_(p.sampleSizeCurrent_),
	    allFreqCurrent_(p.allFreqCurrent_),
	    hetExpDataCurrent_(p.hetExpDataCurrent_),
	    hetObsDataCurrent_(p.hetObsDataCurrent_){
	}//end GeneticPopulation::GeneticPopulation(GeneticPopulation&)

			
GeneticPopulation::GeneticPopulation(GeneticPopulation* p)://copy constructor
		Population(p),//copies from Population
		allelesInPop_(p->allelesInPop_),
		allFreq_(p->allFreq_),
		hetObsLoc_(p->hetObsLoc_),
		hetExpLoc_(p->hetExpLoc_),
		hetObs_(p->hetObs_),
		hetExp_(p->hetExp_),
	    allelesCurrent_(p->allelesCurrent_),
	    sampleSizeCurrent_(p->sampleSizeCurrent_),
		allFreqCurrent_(p->allFreqCurrent_),
	    hetExpDataCurrent_(p->hetExpDataCurrent_),
	    hetObsDataCurrent_(p->hetObsDataCurrent_){
	}//end GeneticPopulation::GeneticPopulation(GeneticPopulation*)

			
GeneticPopulation::~GeneticPopulation(void){
	allelesInPop_.Clear();
	allFreq_.Clear();
	hetObsLoc_.Clear();
	hetExpLoc_.Clear();
	//		cout<<"destroy GeneticPopulation #"<<id_<<endl;
	}//end destructor GeneticPopulation::~GeneticPopulation






//FUNCTIONS


void GeneticPopulation::AgeOneStep(int maxage){
	//flags change if anyone dies
	long old=dead();
	Population::AgeOneStep(maxage);
	if (old != dead()){//frequencies may change
		allelesCurrent_=sampleSizeCurrent_=0;
		allFreqCurrent_=hetExpDataCurrent_=hetObsDataCurrent_=0;}
	}/*end GeneticPopulation::AgeOneStep*/




long double GeneticPopulation::AlleleFrequency (int loc, aString& alleleVal){
	if (AllelePresent(loc,alleleVal)){
			if (!allFreqCurrent_){CalcAlleleFrequencies();}
			aList<long double>& locFreqList=allFreq_[loc];
//			aList<T>& locList=allelesInPop_[loc];
			StringList& locList=allelesInPop_[loc];
			int j=locList.Loc(&alleleVal);
			return locFreqList[j];
			}
		else{ return (long double)0.0;}
	}/*end GeneticPopulation::AlleleFrequency*/
	

int GeneticPopulation::AllelePresent (int loc, aString& alleleVal){
	if (!allelesCurrent_){UpdateAlleleList();}
//	aList<T>& locList=allelesInPop_[loc];
	StringList& locList=allelesInPop_[loc];
	return locList.Includes(&alleleVal);
	}	/*end GeneticPopulation::AllelePresent*/


int GeneticPopulation::AllelePresent (ObjList<Individual>& theList, int loc, aString& alleleVal){
	if (theList.len() > 0) {
		int chrom, locOnChrom;
		CoordsOfNthLocus(loc,chrom, locOnChrom);
		for(long i=0;i<theList.len();++i){
			GeneticIndividual* indiv = (GeneticIndividual*) &(theList[i]);
			if (indiv->AllelePresent(loc,alleleVal)){return 1;}
			}/*end for i*/
		}/*end if len()*/
	return 0;
	}	/*end GeneticPopulation::AllelePresent*/


long GeneticPopulation::Alleles (int loc){
	if (!allelesCurrent_){UpdateAlleleList();}
	return allelesInPop_[loc].len();
	}//end GeneticPopulation::Alleles 

aString& GeneticPopulation::AlleleValue (int loc, int alleleNum){
	if (!allelesCurrent_){UpdateAlleleList();}
	return allelesInPop_[loc][alleleNum];
	}//end GeneticPopulation::AlleleValue 
	



void GeneticPopulation::AssignMutations(void){}//derive this


void GeneticPopulation::CalcAlleleFrequencies(void){
	//This routine counts allelic frequencies, accounting for different ploidies among individuals
	//clear allele frequencies to start over
	if (!allelesCurrent_){UpdateAlleleList();}	
	allFreq_.Clear();
	//set up a frequency aList for this population
	int nloci=loci();
	for (int j=0;j<nloci;++j) {
		//first set up a blank array of frequencies for each possible allele
		aList<long double>* locAlleleFreqs = NEW aList<long double>();
		allFreq_.Append(locAlleleFreqs);
		StringList& locAlleles = allelesInPop_[j];
		for (int a=0;a<locAlleles.len();++a){locAlleleFreqs->Append(0.0);}
		}//end for j

	if (extinct()){allFreqCurrent_=1;return;}
	//next look at the genotype of each individual and tally the frequency
	//this little function occurs within the scope of the CalcAlleleFrequencies function
	TallyAlleleFrequencies(asexuals_);
	TallyAlleleFrequencies(herms_);
	TallyAlleleFrequencies(females_);
	TallyAlleleFrequencies(males_);
	TallyAlleleFrequencies(asexualBabies_);
	TallyAlleleFrequencies(hermBabies_);
	TallyAlleleFrequencies(femaleBabies_);
	TallyAlleleFrequencies(maleBabies_);
		//now go through and divide by the sample size to restore the frequencies
	for (int j=0;j<nloci;++j){
		if (SampleSize(j) > 0){
			StringList& locAlleles = allelesInPop_[j];
			aList<long double>& locAlleleFreqs = allFreq_[j];
			for (int a=0;a<locAlleles.len();++a){ 
				long double freq = locAlleleFreqs[a] / (long double) SampleSize(j);
				locAlleleFreqs.Replace(freq,a);
				}//end for a
			}//end if >0
		}//end for j
	allFreqCurrent_=1;
	}//end GeneticPopulation::CalculateAlleleFrequencies



void GeneticPopulation::CalcExpHeterozygosity(void){
	if (!allFreqCurrent_){CalcAlleleFrequencies();}
	//clear heterozygosities to start over
	hetExpLoc_.Clear();
	int nloci=loci();
	for (int j=0;j<nloci;++j) {
		aList<long double>& locFreqList=allFreq_[j];
		long double homoSqd=0.0;
		for (int a=0;a<locFreqList.len();++a){
			long double q=locFreqList[a];
			homoSqd += q*q;
			}//end for a
		hetExpLoc_.Append((long double)1.0 - homoSqd);
		}//end for j
	hetExp_=hetExpLoc_.mean();
	hetExpDataCurrent_=1;	
	};//end GeneticPopulation::CalcExpHeterozygosity


void GeneticPopulation::CalcObsHeterozygosity(void){
	//This routine counts observed heterozygosities, accounting for different ploidies among individuals
	//haploid individuals are included in the total sample size, along with homozygous individuals with ploidy>1
	//clear allele frequencies to start over
	hetObsLoc_.Clear();
	long double n=(long double) size();
	int nloci=loci();
	for (int j=0;j<nloci;++j){
		long double hets=(long double) CountHets(j);
		if (n>0.0){
				hetObsLoc_.Append(hets/n);}
			else{
				hetObsLoc_.Append(0.0);}
		}//end for j	
	hetObs_=hetObsLoc_.mean();
	hetObsDataCurrent_=1;
	};//end GeneticPopulation::CalcObsHeterozygosity


void GeneticPopulation::CoordsOfNthLocus(int nthLoc, int& onChromNum, int& locNumOnChrom){
	GeneticGridCell* theSite = (GeneticGridCell*) site_;
	theSite->CoordsOfNthLocus(species(), nthLoc, onChromNum, locNumOnChrom);
	}//end GeneticPopulation::CoordsOfNthLocus


long GeneticPopulation::CountAlleles (int loc, aString& alleleVal){
	//considers adults and offspring
	long totalAlleles=CountAlleles(asexuals_, loc, alleleVal);
	totalAlleles+=CountAlleles(asexualBabies_, loc, alleleVal);
	totalAlleles+=CountAlleles(herms_, loc, alleleVal);
	totalAlleles+=CountAlleles(hermBabies_, loc, alleleVal);
	totalAlleles+=CountAlleles(females_, loc, alleleVal);
	totalAlleles+=CountAlleles(femaleBabies_, loc, alleleVal);
	totalAlleles+=CountAlleles(males_, loc, alleleVal);
	totalAlleles+=CountAlleles(maleBabies_, loc, alleleVal);
	return(totalAlleles);
	}	//end GeneticPopulation::CountAlleles


long GeneticPopulation::CountAlleles (ObjList<Individual>& theList, int loc, aString& alleleVal){
	long sum=0;
	if (theList.len() > 0) {
		int chrom, locOnChrom;
		CoordsOfNthLocus(loc,chrom, locOnChrom);
		for(long i=0;i<theList.len();++i){
			GeneticIndividual* indiv = (GeneticIndividual*) &(theList[i]);
			sum+=indiv->CountAlleles(loc,alleleVal);
			}//end for i
		}//end if len()
	return sum;
	}	//end GeneticPopulation::CountAlleles


long GeneticPopulation::CountHets (int loc, aString& alleleVal){
	//considers adults and offspring
	long hets=CountHets(asexuals_, loc, alleleVal);
	hets+=CountHets(asexualBabies_, loc, alleleVal);
	hets+=CountHets(herms_, loc, alleleVal);
	hets+=CountHets(hermBabies_, loc, alleleVal);
	hets+=CountHets(females_, loc, alleleVal);
	hets+=CountHets(femaleBabies_, loc, alleleVal);
	hets+=CountHets(males_, loc, alleleVal);
	hets+=CountHets(maleBabies_, loc, alleleVal);
	return(hets);
	}	//end GeneticPopulation::CountHets


long GeneticPopulation::CountHets (ObjList<Individual>& theList, int loc, aString& alleleVal){
	long sum=0;
	if (theList.len() > 0) {
		int chrom, locOnChrom;
		CoordsOfNthLocus(loc,chrom, locOnChrom);
		for(long i=0;i<theList.len();++i){
			GeneticIndividual* indiv = (GeneticIndividual*) &(theList[i]);
			if (indiv->IsHeterozygote(loc,alleleVal)) {++sum;}
			}//end for i
		}//end if len()
	return sum;
	}	//end GeneticPopulation::CountHets



long GeneticPopulation::CountHets (int loc){
	//considers adults and offspring
	long hets=CountHets(asexuals_, loc);
	hets+=CountHets(asexualBabies_, loc);
	hets+=CountHets(herms_, loc);
	hets+=CountHets(hermBabies_, loc);
	hets+=CountHets(females_, loc);
	hets+=CountHets(femaleBabies_, loc);
	hets+=CountHets(males_, loc);
	hets+=CountHets(maleBabies_, loc);
	return(hets);
	}	//end GeneticPopulation::CountHets


long GeneticPopulation::CountHets (ObjList<Individual>& theList, int loc){
	long sum=0;
	if (theList.len() > 0) {
		int chrom, locOnChrom;
		CoordsOfNthLocus(loc,chrom, locOnChrom);
		for(long i=0;i<theList.len();++i){
			GeneticIndividual* indiv = (GeneticIndividual*) &(theList[i]);
			if (indiv->IsHeterozygote(loc)) {++sum;}
			}//end for i
		}//end if len()
	return sum;
	}	//end GeneticPopulation::CountHets


void GeneticPopulation::Extract (Individual* indiv){
	Population::Extract(indiv);
	allelesCurrent_=allFreqCurrent_=hetExpDataCurrent_=hetObsDataCurrent_=0;
	} //end GeneticPopulation::Extract


double GeneticPopulation::fitness(int indiv){
	Individual& indivI = GetIndividual(indiv);
	GeneticIndividual* in = (GeneticIndividual*) &indivI;
	return in->fitness();
	}//GeneticPopulation::fitness


aString*	GeneticPopulation::GenotypeString(long indiv, char* separator){
	Individual& indivI = GetIndividual(indiv);
	GeneticIndividual* in = (GeneticIndividual*) (&indivI);
	return in->GenotypeString(separator);
	} //end GeneticPopulation::GenotypeString(long, char)




aString*	GeneticPopulation::GenotypeString(long indiv, int locus, char* separator){
	Individual& indivI = GetIndividual(indiv);
	GeneticIndividual* in = (GeneticIndividual*) (&indivI);
	return in->GenotypeString(locus, separator);
	} //end GeneticPopulation::GenotypeString(long, int, char)




aString*	GeneticPopulation::GenotypeString(long indiv, int locus, int alleleNum, char* separator){
	Individual& indivI = GetIndividual(indiv);
	GeneticIndividual* in = (GeneticIndividual*) (&indivI);
	return in->GenotypeString(locus, alleleNum, separator);
	} //end GeneticPopulation::GenotypeString(long, int, int, char)




void GeneticPopulation::GetGenome(PopulationGeneContainer<int>& pgc){
	pgc.Clear();
	long i;
	for(i=0;i<size();++i){
		pgc.indivs_.Append(NEW MultiInheritanceModeContainer<int>());}
	for(i=0;i<size();++i){
		Individual& indivI = GetIndividual(i);
		indivI.GetGenome(pgc.indivs_[i]);}
	}//end GeneticLandScape::GetGenome


void GeneticPopulation::GetGenome(MultiInheritanceModeContainer<int>& mimc, long indiv){
	Individual& indivI = GetIndividual(indiv);
	return indivI.GetGenome(mimc);
	}//end GeneticPopulation::GetGenome


	
aString*	GeneticPopulation::GetPhenotypeString(long indiv, char* separator){
	Individual& indivI = GetIndividual(indiv);
	GeneticIndividual* in = (GeneticIndividual*) (&indivI);
	return in->GetPhenotypeString(separator);
	} //end GeneticPopulation::GetPhenotypeString





long double GeneticPopulation::hetExp (int loc, aString& alleleVal){
	if (!allFreqCurrent_){CalcAlleleFrequencies();}
	StringList& locList=allelesInPop_[loc];
	if (locList.Includes(&alleleVal)){
			aList<long double>& locFreqList=allFreq_[loc];
			int j=locList.Loc(&alleleVal);
			long double q= locFreqList[j];
			return (long double)2.0*q*((long double)1.0-q);
			}
		else{ return (long double)0.0;}
	}//end GeneticPopulation::AlleleFrequency
	

long double GeneticPopulation::hetExp (int loc){
	if (!hetExpDataCurrent_){CalcExpHeterozygosity();}
	return hetExpLoc_[loc];
}//end GeneticPopulation::hetExp


long double GeneticPopulation::hetObs (int loc, aString& alleleVal){
	long hets=CountHets(asexuals_, loc,alleleVal);
	hets+=CountHets(asexualBabies_, loc,alleleVal);
	hets+=CountHets(herms_, loc,alleleVal);
	hets+=CountHets(hermBabies_, loc,alleleVal);
	hets+=CountHets(females_, loc,alleleVal);
	hets+=CountHets(femaleBabies_, loc,alleleVal);
	hets+=CountHets(males_, loc,alleleVal);
	hets+=CountHets(maleBabies_, loc,alleleVal);
	long double n=((long double) SampleSize(loc))/((long double) ploidy(loc));
	if (n>(long double)0.0){return( ((long double) hets)/n);}
		else {return (long double)0.0;}
}//end GeneticPopulation::hetObs


long double GeneticPopulation::hetObs (int loc){
	if (!hetObsDataCurrent_){CalcObsHeterozygosity();}
	return hetObsLoc_[loc];
}//end GeneticPopulation::hetObs



void GeneticPopulation::Install (Individual* indiv){
	Population::Install(indiv);
	allelesCurrent_=allFreqCurrent_=hetExpDataCurrent_=hetObsDataCurrent_=0;
	} //end GeneticPopulation::Install

int GeneticPopulation::loci(void){
		GeneticGridCell* theSite = (GeneticGridCell*) site_;
		return theSite->loci(species());}


int GeneticPopulation::NoInconsistentSettings(void){
		return 1;}//not yet programmed
				//returns an error code if settings are inconsistent


int GeneticPopulation::NthLocus(int chrom, int locOnChrom){//0 indexed
	//returns the sequential locus number given locus coordinates
	GeneticGridCell* theSite = (GeneticGridCell*) site_;
	return theSite->NthLocus(species(),chrom,locOnChrom);
	}//end GeneticPopulation::NthLocus
	


int GeneticPopulation::ploidy(int loc) {
	GeneticGridCell* theSite = (GeneticGridCell*) site_;
	return theSite->ploidy(species(),loc);}


void GeneticPopulation::PrintAlleleFreqs(ostream& ){}//not yet programmed


void GeneticPopulation::PrintAlleleFreqs(ostream& , int /*locus*/){}//not yet programmed


void GeneticPopulation::PrintAlleleFreqs(ostream& , int /*locus*/, aString& /*alleleVal*/){
	// not programmed yet
	}	// end GeneticPopulation::PrintAlleleFreqs



void GeneticPopulation::PrintAlleleList(ostream&){
	// not programmed yet
	}	//end GeneticPopulation::PrintAlleleList




void GeneticPopulation::PrintAlleleList(ostream&, int /*locus*/){
	// not programmed yet
	}	//end GeneticPopulation::PrintAlleleList



void GeneticPopulation::PrintGeneStrings(ostream& outFile, int alleleLen, int codeLen){
	if (! extinct()){
			PrintGeneStrings(outFile, asexuals_,alleleLen,codeLen);
			PrintGeneStrings(outFile, herms_,alleleLen,codeLen);
			PrintGeneStrings(outFile, females_,alleleLen,codeLen);
			PrintGeneStrings(outFile, males_,alleleLen,codeLen);
			PrintGeneStrings(outFile, asexualBabies_,alleleLen,codeLen);
			PrintGeneStrings(outFile, hermBabies_,alleleLen,codeLen);
			PrintGeneStrings(outFile, femaleBabies_,alleleLen,codeLen);
			PrintGeneStrings(outFile, maleBabies_,alleleLen,codeLen);
			outFile<<endl;
			}
		else{outFile<<"         extinct"<<endl<<endl;}//end if size()
	}//end GeneticPopulation::PrintGeneStrings


void GeneticPopulation::PrintGeneStrings(ostream& outFile, ObjList<Individual>& theList, int alleleLen, int codeLen){
	if (theList.len() > 0) {
		for(long i=0;i<theList.len();++i){
			GeneticIndividual* indiv = (GeneticIndividual*) &(theList[i]);
			aString indivStr(indiv->GetGenderChar());
			indivStr.cat(' ');
			outFile<<indivStr;
			indiv->PrintGenes(outFile, alleleLen,codeLen);
			}//end for i
		}//end if len()
	}	//end GeneticPopulation::PrintGeneStrings



//void GeneticPopulation::Reproduce (long& idCount, Ran3List& ran, double mutationRate, double sigma){
void GeneticPopulation::Reproduce (long& idCount, Ran3List& ran, aList<long double>& specialInfo){
	//clones randomly chosen asexual individuals in the population
	//copies up to num of herms in population
	//derived classes handle separate genders, sexual reproduction, etc.
	long old=babies();
//	for (int i=0;i<asexuals();++i){
//		GeneticIndividual* asexu = (GeneticIndividual*) &(asexual(i));
//		asexu->Reproduce(idCount,ran,mutationRate,sigma);}
	int i;
	for (i=0;i<adults();++i){
		Adult(i).Reproduce(idCount,ran,specialInfo);}
	if (old != babies()){//frequencies may change
		allelesCurrent_=sampleSizeCurrent_=0;
		allFreqCurrent_=hetExpDataCurrent_=hetObsDataCurrent_=0;}
	} //end GeneticPopulation::Reproduce 


void GeneticPopulation::SampleSize (void){//handles ploidy differences
	sampleSizeCurrent_=(-1);
	sampleSize_.Clear();
	int j,nloci=loci();
	for (j=0;j<nloci;++j) {
		sampleSize_.Append(SampleSize(j));
		}//end for j		
	sampleSizeCurrent_=1;
	}//end GeneticPopulation::SampleSize 


long GeneticPopulation::SampleSize (int loc){//handles ploidy differences
	if (sampleSizeCurrent_>0){
			return sampleSize_[loc];}
		else if (sampleSizeCurrent_==0){
			SampleSize();//redo the whole thing
			return sampleSize_[loc];
			}
		else{
			long sum=0;
			if (! extinct()){
				sum+=SampleSize(loc,asexuals_);
				sum+=SampleSize(loc,herms_);
				sum+=SampleSize(loc,females_);
				sum+=SampleSize(loc,males_);
				sum+=SampleSize(loc,asexualBabies_);
				sum+=SampleSize(loc,hermBabies_);
				sum+=SampleSize(loc,femaleBabies_);
				sum+=SampleSize(loc,maleBabies_);
				}//end if Extinct()
			return sum;
			}//end else
	}//end GeneticPopulation::SampleSize


long GeneticPopulation::SampleSize (int loc, ObjList<Individual>& theList){
	long sum=0;
	if (theList.len() > 0) {
		int chrom, locOnChrom;
		CoordsOfNthLocus(loc,chrom, locOnChrom);
		for(long i=0;i<theList.len();++i){
			GeneticIndividual* indiv = (GeneticIndividual*) &(theList[i]);
//			sum+=indiv->PloidyAtChrom(chrom);
			sum+=indiv->ploidy(loc);
			}//end for i
		}//end if len()
	return sum;
	}	//end GeneticPopulation::SampleSize




void GeneticPopulation::TallyAlleleFrequencies(ObjList<Individual>& theList){
	//only call from within CalcAlleleFrequencies
	for (long i=0;i<theList.len();++i){
		GeneticIndividual* indiv = (GeneticIndividual*) &(theList[i]);
		for (int j=0;j<loci();++j){
			int chrom, locOnChrom;
			CoordsOfNthLocus(j,chrom, locOnChrom);
//			aList<T>& locAlleles=allelesInPop_[j];
			StringList& locAlleles=allelesInPop_[j];
			aList<long double>& locAlleleFreqs = allFreq_[j];
			for(int a=0;a<locAlleles.len();++a){
//				T alleleValToCheck=locAlleles[a];
				aString& alleleValToCheck=locAlleles[a];
				for(int alleleNum=0;alleleNum<indiv->ploidy(j);++alleleNum){					
//				for(int alleleNum=0;alleleNum<indiv->PloidyAtChrom(chrom);++alleleNum){					
//					T alleleVal=indiv->GetGene(chrom, locOnChrom,alleleNum);
					aString* alleleVal=indiv->GenotypeString(j,alleleNum, " ");
					if (alleleVal==alleleValToCheck){
						int alleleInList=locAlleles.Loc(alleleVal);
						locAlleleFreqs.Replace(locAlleleFreqs[alleleInList]+(long double)1.0,alleleInList);//increments
						}
					delete alleleVal; alleleVal=NULL;
					}//end for alleleNum
				}//end for a
			}//end for j
		}//end for i
	}//end GeneticPopulation::TallyAlleleFrequencies(ObjList<Individual>&)



void GeneticPopulation::UpdateAlleleList(void){
	allelesInPop_.Clear();
	int nloci=loci();
	for (int j=0;j<nloci;++j) {
//		aList<T>* locAlleles=NEW aList<T>();
		StringList* locAlleles=NEW StringList();
		allelesInPop_.Append(locAlleles);
		}//end for j		
	UpdateAlleleList(asexuals_);
	UpdateAlleleList(herms_);
	UpdateAlleleList(females_);
	UpdateAlleleList(males_);
	UpdateAlleleList(asexualBabies_);
	UpdateAlleleList(hermBabies_);
	UpdateAlleleList(femaleBabies_);
	UpdateAlleleList(maleBabies_);
	for (int j=0;j<nloci;++j) {
//		aList<T>& locAlleles=allelesInPop_[j];
		StringList& locAlleles=allelesInPop_[j];
		locAlleles.Sort();
		}//end for j		
	allelesCurrent_=1;
	}//end  GeneticPopulation::UpdateAlleleList



void GeneticPopulation::UpdateAlleleList(ObjList<Individual>& theList){
	for (long i=0;i<theList.len();++i){
		UpdateAlleleList(theList[i]);
		}//end for i
	}//end GeneticPopulation::UpdateAlleleList(ObjList<Individual>& theList)


void GeneticPopulation::UpdateAlleleList(Individual& indiv){
	//indiv must be a GeneticIndividual
	GeneticIndividual* genIndiv=(GeneticIndividual*) &indiv;
		for (int j=0;j<loci();++j){
			int chrom, locOnChrom;
			CoordsOfNthLocus(j,chrom, locOnChrom);
			StringList& locAlleles=allelesInPop_[j];
			for(int a=0;a<genIndiv->ploidy(j);++a){
//			for(int a=0;a<genIndiv->PloidyAtChrom(chrom);++a){
				aString* alleleVal = genIndiv->GenotypeString(j,a," ");
				if (!(locAlleles.Includes(alleleVal))){
						locAlleles.Append(alleleVal);
						}
					else{
						delete alleleVal;}
				alleleVal=NULL;
				}//end for a
			}//end for j
	}// end GeneticPopulation::UpdateAlleleList


void GeneticPopulation::ViabilitySelection (Ran3& ran){
	ViabilitySelection(asexuals_, ran);
	ViabilitySelection(herms_, ran);
	ViabilitySelection(females_, ran);
	ViabilitySelection(males_, ran);
	ViabilitySelection(asexualBabies_, ran);
	ViabilitySelection(hermBabies_, ran);
	ViabilitySelection(femaleBabies_, ran);
	ViabilitySelection(maleBabies_, ran);
	}//end GeneticPopulation::ViabilitySelection(Ran3& ran)


void GeneticPopulation::ViabilitySelection (ObjList<Individual>& theList, Ran3& ran){
	long double fitness;//dummy variable
	for (long i=theList.len()-1;i>=0;--i){
//		selParams.Clear();
//		double hab = (double) habitat();
//		selParams.Append(hab);
		aList<long double> selParams;
		Individual* indivI = &(theList[i]);
		GeneticIndividual* indiv = (GeneticIndividual*) (indivI);
		SetSelectionParams(selParams, *indiv);
		int survives = indiv->SurvivesViabilitySelection(selParams, ran, fitness);
		if (! survives ){
			theList.Detach(i);
			deadList_.Append(indiv);}
		}//end for i
	}//end GeneticPopulation::ViabilitySelection(ObjList<Individual>&, Ran3&)


void GeneticPopulation::SetSelectionParams (aList<long double>& selParams){
	selParams.Clear();
	long double hab = (double) habitat();
	selParams.Append(hab);
	}//end GeneticPopulation::SetSelectionParams (aList<double>&)

void GeneticPopulation::SetSelectionParams (aList<long double>& selParams, GeneticIndividual& indiv){
	SetSelectionParams (selParams);
	indiv.SetSelectionParams(selParams);
	}//end GeneticPopulation::SetSelectionParams (aList<double>&, GeneticIndividual&)


#endif

