/*******************************************************
*
*	Population.cpp
*
*	Superclass:  none
*
*	This class handles processes within populations
********************************************************/

#ifndef Populationhcpp
#define Populationcpp = 1



#include "Population.h"
#include "GridCell.h"
#include "Element.cpp"
#include "aList.cpp"
#include "CoordinateClass.cpp"


Population::Population(long id, int species, GridCell* g,long k){
	id_=id;
	species_=species;
	site_=g; 
	carryingCapacity_=k;
		//		cout<<"create population #"<<id_<<endl;
	}//end constructor Population::Population
			
			
Population::Population(void):id_(0),carryingCapacity_(0),species_(0) {
	site_=NULL;
	}//end constructor Population::Population


Population::Population(long id):id_(id),carryingCapacity_(0),species_(0) {
	site_=NULL;
	//cout<<"create population #"<<id_<<endl;
	}//end constructor Population::Population



Population::Population(Population& p)://copy constructor
		id_(p.id_),//all these are duplicated
		species_(p.species_),
		carryingCapacity_(p.carryingCapacity_),
		asexuals_(p.asexuals_),
		females_(p.females_),
		males_(p.males_),
		herms_(p.herms_),
		asexualBabies_(p.asexualBabies_),
		maleBabies_(p.maleBabies_),
		femaleBabies_(p.femaleBabies_),
		hermBabies_(p.hermBabies_),
		maleGametes_(p.maleGametes_),
		femaleGametes_(p.femaleGametes_),
		hermGametes_(p.hermGametes_),
		deadList_(p.deadList_)  {
		site_=p.site_;//but this is assigned not copied
	}//end Population::Population(Population&)

			
Population::Population(Population* p)://copy constructor
		id_(p->id_),//all these are duplicated
		species_(p->species_),
		carryingCapacity_(p->carryingCapacity_),
		asexuals_(p->asexuals_),
		females_(p->females_),
		males_(p->males_),
		herms_(p->herms_),
		asexualBabies_(p->asexualBabies_),
		maleBabies_(p->maleBabies_),
		femaleBabies_(p->femaleBabies_),
		hermBabies_(p->hermBabies_),
		maleGametes_(p->maleGametes_),
		femaleGametes_(p->femaleGametes_),
		hermGametes_(p->hermGametes_),
		deadList_(p->deadList_)  {
		site_=p->site_;//but this is assigned not copied
	}//end Population::Population(Population&)

			
Population::~Population(void){
			asexuals_.Clear();
			females_.Clear();
			males_.Clear();
			herms_.Clear();
			ClearGameteLists();
			ClearOffspringLists();
			deadList_.Clear();
			site_=NULL;//detach but do not delete; destroyed elsewhere
	//		cout<<"destroy population #"<<id_<<endl;
			}//end destructor Population::~Population

Population* Population::Duplicate(long id, int species, GridCell* g,long k){
	//a virtual constructor
	return NEW Population(id,species,g,k);
	}//end Population::Duplicate

Population* Population::Duplicate(long id){
	//a virtual constructor
	return NEW Population(id);
	}//end Population::Duplicate


Population* Population::Duplicate(void){
	//a virtual copy constructor
	return NEW Population(*this);
	}//end Population::Duplicate


//FUNCTIONS


Individual& Population::Adult(long i){
	//returns an adult, order asexuals, herms, females, males
	if(i<asexuals_.len()){
		return asexuals_[i];}
	i -= asexuals_.len();
	if(i<herms_.len()){
		return herms_[i];}
	i -= herms_.len();
	if(i<females_.len()){
		return females_[i];}
	i -= females_.len();
	if(i<males_.len()){
		return males_[i];}
	aString err("looking for adult[");err.cat(i);err.cat("] of a total of ");
	err.cat(adults());err.cat(" adults.");
	aError(err);
	return asexuals_[0];
	}//end Population::adult(long)


long Population::adults(void){
	return (asexuals() + herms() + females() + males());
	}//end  Population::adults



/*void Population::AdultDispersal(Ran3List& ran, double m, double kurtosisFactor){
	Dispersal(asexuals_,ran,m,kurtosisFactor);	
	Dispersal(herms_,ran,m,kurtosisFactor);	
	Dispersal(females_,ran,m,kurtosisFactor);	
	Dispersal(males_,ran,m,kurtosisFactor);	
	}//end Population::AdultDispersal
*/

void Population::AdultDispersal(Ran3List& ran, aList<long double>& dispersalInfo){
	Dispersal(asexuals_,ran,dispersalInfo);	
	Dispersal(herms_,ran,dispersalInfo);	
	Dispersal(females_,ran,dispersalInfo);	
	Dispersal(males_,ran,dispersalInfo);	
	}//end Population::AdultDispersal


void Population::AgeOneStep(int maxage){
	//process adults but not babies here
	AgeOneStep(asexuals_,maxage);
	AgeOneStep(herms_,maxage);
	AgeOneStep(females_,maxage);
	AgeOneStep(males_,maxage);
	}//end Population::AgeOneStep


void Population::AgeOneStep(ObjList<Individual>& theList, int maxage){
	//process males & females, but not babies here
	Individual* indiv; int i;
	for (i=theList.len()-1;i>=0;--i){
		indiv = &(theList[i]);
		indiv->Birthday(maxage);
		if (!(indiv->alive() )){
			theList.Detach(i);
			deadList_.Append(indiv);}
		}
	}//end Population::AgeOneStep


void Population::AssignMates (Ran3List&, aList<long double>&, int, int){}
	//does nothing since default is asexual


long Population::babies(void){
	return (asexualBabies_.len() + femaleBabies_.len() + 
						maleBabies_.len() + hermBabies_.len());
	}//end  Population::babies


Individual& Population::Baby(long i){
	//returns an adult, order asexuals, herms, females, males
	if(i<asexualBabies_.len()){
		return asexualBabies_[i];}
	i -= asexualBabies_.len();
	if(i<hermBabies_.len()){
		return hermBabies_[i];}
	i -= hermBabies_.len();
	if(i<femaleBabies_.len()){
		return femaleBabies_[i];}
	i -= femaleBabies_.len();
	if(i<males_.len()){
		return maleBabies_[i];}
	aString err("looking for baby[");err.cat(i);err.cat("] of a total of ");
	err.cat(adults());err.cat(" babies.");
	aError(err);
	return asexualBabies_[0];
	}//end Population::Baby(long)



void Population::BuryTheDead(void){
	//This is called to erase dead individuals from memory, after all
	//	their uses have been exhausted (as mate info, data source, etc).
	//	must be called after ReplaceTheDead
	long i;
	for (i=0;i<dead();++i){//prepare the individual for burial
		Individual& deadIndiv = dead(i);
		deadIndiv.PrepareForBurial();	//removes associated lists, etc
		}
	deadList_.Clear();
	//also remove any babies not recruited
	ClearOffspringLists();
	}	//end Population::BuryTheDead
	

void Population::ClearGameteLists (void){
	hermGametes_.Clear();
	maleGametes_.Clear();
	femaleGametes_.Clear();
	}//end Population::ClearOffspringLists
	
	
void Population::ClearOffspringLists (void){
	asexualBabies_.Clear();
	maleBabies_.Clear();
	femaleBabies_.Clear();
	hermBabies_.Clear();
	}//end Population::ClearOffspringLists
	
	
CoordinateType& Population::coords(void){
	if (site_ != NULL){
			return site_->coords();}
		else{//should not get here
			aError("Popupation::coords() not available -- the population is not on the grid.");
			CoordinateType* c=NEW CoordinateType;
			c->set(-1,-1);
			return *c;}//will introduce a memory leak
	}//end Population::coords(void)



void Population::InitializeANewGeneration(void){//empty function to be programmed
	}//end Population::InitializeANewGeneration



long Population::dead(void){
	return (deadList_.len());
	}//end  Population::dead


void Population::dead(Individual* deadguy){
	//under some simulation problems, individuals can die in other contexts
	if(deadguy->lifeStage()==gamete){
		dead(deadguy,deadguy->gameteType());
		return;
		}
	genderSet deadguysSex = deadguy->sex_;
	switch (deadguysSex){
		case A:
			if(asexuals_.Includes(deadguy)){
					asexuals_.Detach(deadguy);}
				else if(asexualBabies_.Includes(deadguy)){
					asexualBabies_.Detach(deadguy);}
				else{}
			break;
		case B:
			if(herms_.Includes(deadguy)){
					herms_.Detach(deadguy);}
				else if(hermBabies_.Includes(deadguy)){
					hermBabies_.Detach(deadguy);}
				else{}
			break;
		case F:
			if(females_.Includes(deadguy)){
					females_.Detach(deadguy);}
				else if(femaleBabies_.Includes(deadguy)){
					femaleBabies_.Detach(deadguy);}
				else{}
			break;
		case M:
			if(males_.Includes(deadguy)){
					males_.Detach(deadguy);}
				else if(maleBabies_.Includes(deadguy)){
					maleBabies_.Detach(deadguy);}
				else{}
			break;
		default:
			break;
		}
	deadList_.Append(deadguy);
	}//end  Population::dead


void Population::dead(Individual* deadgamete, genderSet typeOfGamete){
	//under some simulation problems, individuals can die in other contexts
	if(deadgamete->lifeStage()!=gamete){
		dead(deadgamete);
		return;
		}
	switch (typeOfGamete){
		case B:
			if(hermGametes_.Includes(deadgamete)){
					hermGametes_.Detach(deadgamete);}
			break;
		case F:
			if(femaleGametes_.Includes(deadgamete)){
					femaleGametes_.Detach(deadgamete);}
			break;
		case M:
			if(maleGametes_.Includes(deadgamete)){
					maleGametes_.Detach(deadgamete);}
			break;
		default:
			break;
		}
	deadList_.Append(deadgamete);
	}//end  Population::dead


/*
void Population::Dispersal(Ran3List& ran, double m, double kurtosisFactor){
	AdultDispersal(ran,m,kurtosisFactor);	
	JuvenileDispersal(ran,m,kurtosisFactor);	
	}//end Population::Dispersal
*/

void Population::Dispersal(Ran3List& ran, aList<long double>& dispersalInfo){
	AdultDispersal(ran,dispersalInfo);	
	JuvenileDispersal(ran,dispersalInfo);	
	}//end Population::Dispersal


//void Population::Dispersal(ObjList<Individual>& theList, Ran3List& ran, 
//							double m, double kurtosisFactor){
void Population::Dispersal(ObjList<Individual>& theList, Ran3List& ran, 
							aList<long double>& dispersalInfo){
	//send m<1 for island model
	long double diff, m = dispersalInfo[0];
	long i, index, Nm= (long) ( m * theList.len());
	if ((migrationModel()==island) || (migrationModel()==steppingStone)) {//each individual has a chance to disperse
//		for (long i=Nm-1;i>=0;--i){
		for (i=theList.len()-1;i>=0;--i){//individuals decide whether to move
			Individual& indiv = theList[i];
//			indiv.MoveRandomly(ran, kurtosisFactor);
			indiv.MoveRandomly(ran, dispersalInfo);
			}//end for i
		return;}
	if (m<one ) {//randomly choose individuals to disperse
		aList<long> dispersers, indivs;
		for (i=0;i<theList.len();++i){indivs.Append(i);}
		diff=( m * theList.len()) - (long double)Nm;
		if (ran.next()<diff){++Nm;}
		for (i=0;i<Nm;++i){
			index=ran.longBetween(0,indivs.len());
			dispersers.Append(indivs[index]);
			indivs.Delete(index);//samples without replacement
			}//end for i
		dispersers.Sort();
		for (i=Nm-1;i>=0;--i){
			Individual& indiv = theList[dispersers[i]];
//			indiv.MoveRandomly(ran, kurtosisFactor);
			indiv.MoveRandomly(ran, dispersalInfo);
			}//end for i
		return;
		}//end if m<1.0
	//go here if m==1.0
	for (i=Nm-1;i>=0;--i){
		Individual& indiv = theList[i];
//		indiv.MoveRandomly(ran, kurtosisFactor);
		indiv.MoveRandomly(ran, dispersalInfo);
		}
	}//end Population::Dispersal



void Population::Extract (Individual* indiv){
	//use an independent procedure to change the individual's data
	if(indiv->lifeStage_==gamete){
			Extract(indiv,indiv->gameteType());}
		else{
			if (indiv->sex() == A){
					if(indiv->lifeStage()==adult){asexuals_.Detach(indiv);}
						else {asexualBabies_.Detach(indiv);}
					}
				else if (indiv->sex() == B){
					if(indiv->lifeStage()==adult){herms_.Detach(indiv);}
						else {hermBabies_.Detach(indiv);}
					}
				else if (indiv->sex() == F){
					if(indiv->lifeStage()==adult){females_.Detach(indiv);}
						else {femaleBabies_.Detach(indiv);}
					}
				else {//indiv.sex=M
					if(indiv->lifeStage()==adult){males_.Detach(indiv);}
						else {maleBabies_.Detach(indiv);}
						}
				}
	indiv->pop(NULL);
	} /*end Population::Extract*/
	
	
void Population::Extract (Individual* indiv, genderSet parentGender){
	//use an independent procedure to change the individual's data
	if(indiv->lifeStage_!=gamete){
		Extract(indiv);
		return;}
	if (parentGender == B){
		hermGametes_.Detach(indiv);}
	else if (parentGender == F){
		femaleGametes_.Detach(indiv);}
	else if (parentGender == M){
		maleGametes_.Detach(indiv);}
	else{}
	indiv->pop(NULL);
	} /*end Population::Extract*/
	
	
Individual& Population::father(long i){
	long sum = 0; //Individual* indiv = NULL;
	if(i< sum+herms()){
		Individual& indiv = herm(i-sum);
		return indiv; }
	sum += herms();
	if(i< sum+males()){
		Individual& indiv = male(i-sum);
		return indiv; }
	return herm(0); //default
	}//end Population::father


long double	Population::Fecundity(int age){/*from landscape-level fecundity schedule*/
	if(site_!=NULL) return site_->Fecundity(species(), age);
	return zero;}


Individual& Population::Gamete(long i){
	//returns an adult, order herms, females, males
	if(i<hermGametes_.len()){
		return hermGametes_[i];}
	i -= hermGametes_.len();
	if(i<femaleGametes_.len()){
		return femaleGametes_[i];}
	i -= femaleGametes_.len();
	if(i<maleGametes_.len()){
		return maleGametes_[i];}
	aString err("looking for gamete[");err.cat(i);err.cat("] of a total of ");
	err.cat(gametes());err.cat(" gametes.");
	aError(err);
	return hermGametes_[0];
	}//end Population::adult(long)


long Population::gametes(void){
	return hermGametes_.len()+femaleGametes_.len()+maleGametes_.len();
	}


void Population::GetAgeDistribution(aList<int>& ageDist){
	long oldLen=ageDist.len();
	ageDist.Clear();
	int j;
	for (j=0; j<oldLen;++j){ageDist.Append(0);}
	GetAgeDistribution(asexuals_, ageDist);
	GetAgeDistribution(herms_, ageDist);
	GetAgeDistribution(females_, ageDist);
	GetAgeDistribution(males_, ageDist);
	ageDist.Replace(babies(),0);
		} /* end Population::GetAgeDistribution*/


void Population::GetAgeDistribution(ObjList<Individual>& theList, 
									 aList<int>& ageDist){
	int i;
	for (i=0;i<theList.len();++i){
		Individual& indiv = theList[i];
		ageDist.Replace(ageDist[indiv.age()]+1,indiv.age() );}
		} /* end Population::GetAgeDistribution*/



Individual& Population::GetAdult(long i){
	long sum = 0; //Individual* indiv = NULL;
	if(i< sum+asexuals()){
		Individual& indiv = asexual(i-sum);
		return indiv; }
	sum += asexuals();
	if(i< sum+herms()){
		Individual& indiv = herm(i-sum);
		return indiv; }
	sum += herms();
	if(i< sum+females()){
		Individual& indiv = female(i-sum);
		return indiv; }
	sum += females();
	if(i< sum+males()){
		Individual& indiv = male(i-sum);
		return indiv; }
	return asexual(0); //default
	}//end Population::GetAdult


Individual& Population::GetBaby(long i){
	long sum = 0; //Individual* indiv = NULL;
	if(i< sum+asexualBabies()){
		Individual& baby = asexualBaby(i-sum);
		return baby; }
	sum += asexualBabies();
	if(i< sum+hermBabies()){
		Individual& baby = hermBaby(i-sum);
		return baby; }
	sum += hermBabies();
	if(i< sum+femaleBabies()){
		Individual& baby = femaleBaby(i-sum);
		return baby; }
	sum += femaleBabies();
	if(i< sum+maleBabies()){
		Individual& baby = maleBaby(i-sum);
		return baby; }
	return asexual(0); //default
	}//end Population::GetBaby


Individual& Population::GetGamete(long i){
	long sum = 0; //Individual* indiv = NULL;
	if(i< sum+hermGametes()){
		Individual& gam = hermGamete(i-sum);
		return gam; }
	sum += hermGametes();
	if(i< sum+femaleGametes()){
		Individual& gam = femaleGamete(i-sum);
		return gam; }
	sum += femaleGametes();
	if(i< sum+maleGametes()){
		Individual& gam = maleGamete(i-sum);
		return gam; }
	return hermGamete(0); //default
	}//end Population::GetGamete


/*REPLACED BY GetAdult()
Individual& Population::GetIndividual(long i){
	long ii = i; //Individual* indiv = NULL;
	if(ii< adults()){
		return GetAdult(ii); }
	ii -= adults();
	if(ii< babies()){
		return GetBaby(ii); }
	ii -= babies();
	if(ii< gametes()){
		return GetGamete(ii); }
	return Adult(0); //default
	}//end Population::GetIndividual
*/




Individual& Population::GetIndividual(long indiv){
	long double f=zero;
	if(indiv<adults()){
			return Adult(indiv);}
		else if(indiv<adults()+babies()){
			return Baby(indiv-adults());}
		else if (indiv<adults()+babies()+gametes()){
			return Gamete(indiv-adults()-babies());}
		else{}
	aString estr="ERROR: can't return individual ";
	estr.cat(indiv); estr.cat(" in a population of size");
	estr.cat(adults()+babies()+gametes());
	aError(estr);
	return Adult(0);//default
	}//	RangeExpansionPopulation::Fecundity(int indiv)
			
	

long double Population::gridScale(void){
	if(site_!=NULL) return site_->gridScale();
	return -one;}


int	 Population::habitat(void){
	if(site_!=NULL) return site_->habitat();
	return 0;}



void Population::ImmatureDispersal(Ran3List& ran, aList<long double>& dispersalInfo){
	Dispersal(asexualBabies_,ran,dispersalInfo);	
	Dispersal(hermBabies_,ran,dispersalInfo);	
	Dispersal(femaleBabies_,ran,dispersalInfo);	
	Dispersal(maleBabies_,ran,dispersalInfo);	
	}//end Population::ImmatureDispersal


void Population::GameteDispersal(Ran3List& ran, aList<long double>& dispersalInfo){
	Dispersal(hermGametes_,ran,dispersalInfo);	
	Dispersal(femaleGametes_,ran,dispersalInfo);	
	Dispersal(maleGametes_,ran,dispersalInfo);	
	}//end Population::ImmatureDispersal



long Population::immigrants(void){//number of new immigrants
	return immigrantAdults()+immigrantImmatures()+immigrantGametes();
	}//end Population::immigrantAdults



long Population::immigrantAdults(void){//number of new immigrants
	long i,imm=0;
	for(i=0;i<adults();++i){
		if(Adult(i).movedThisGen()){
			imm++;}
		}//end for i
	return imm;
	}//end Population::immigrantAdults



long Population::immigrantImmatures(void){//number of new immigrants
	long i,imm=0;
	for(i=0;i<babies();++i){
		if(Baby(i).movedThisGen()){
			imm++;}
		}//end for i
	return imm;
	}//end Population::immigrantInmmatures



long Population::immigrantGametes(void){//number of new immigrants
	long i,imm=0;
	for(i=0;i<gametes();++i){
		if(Gamete(i).movedThisGen()){
			imm++;}
		}//end for i
	return imm;
	}//end Population::immigrantGametes



void Population::Install (Individual* indiv){
	//use an independent procedure to change the individual's data
	if(indiv->lifeStage()==gamete){
		Install(indiv,indiv->gameteType());
		return;
		}
	if (indiv->sex() == A){
			if(indiv->lifeStage()==adult){asexuals_.Append(indiv);}
				else {asexualBabies_.Append(indiv);}
			}
		else if (indiv->sex() == B){
			if(indiv->lifeStage()==adult){herms_.Append(indiv);}
				else if(indiv->lifeStage()==gamete){hermGametes_.Append(indiv);}
				else {hermBabies_.Append(indiv);}
			}
		else if (indiv->sex() == F){
			if(indiv->lifeStage()==adult){females_.Append(indiv);}
				else if(indiv->lifeStage()==gamete){femaleGametes_.Append(indiv);}
				else {femaleBabies_.Append(indiv);}
			}
		else {//indiv.sex=M
			if(indiv->lifeStage()==adult){males_.Append(indiv);}
				else if(indiv->lifeStage()==gamete){maleGametes_.Append(indiv);}
				else {maleBabies_.Append(indiv);}
				}
	indiv->pop(this);
	} //end Population::Install


void Population::Install (Individual* indiv, genderSet parentGender){
	//use an independent procedure to change the individual's data
	if(indiv->lifeStage_ != gamete){ 
		Install(indiv);
		return;}
	int i=1,j=3;
	if(indiv->gameteType() != parentGender){
		i=j;//debugging nonsense
		}
	if (parentGender == F){
		femaleGametes_.Append(indiv);
		}
		else if (parentGender == M){
			maleGametes_.Append(indiv);
			}
		else if (parentGender == B){
			hermGametes_.Append(indiv);
			}
		else{}
	indiv->pop(this);
	} //end Population::Install


long double	Population::intrinsicSexRatio(void){
	if(site_!=NULL) return site_->intrinsicSexRatio(species());
	return -one;}



void Population::IPopulation (CoordinateType /*loc*/, long k, Ran3List& /* ran*/){
	carryingCapacity_ = k;	
	}  /*end Population::IPopulation*/
	
	
int Population::IsAllOneSex(void){
	if ((herms()>0) || (hermBabies()>0))  {return 0;} 
		else if (((males() > 0) || (maleBabies() > 0)) && 
				 ((females() > 0) || (femaleBabies() > 0))) {
						return 0;}
	return 1;
	}	//end Population::IsAllOneSex


/*
void Population::JuvenileDispersal(Ran3List& ran, double m, double kurtosisFactor){
	Dispersal(asexualBabies_,ran,m,kurtosisFactor);	
	Dispersal(hermBabies_,ran,m,kurtosisFactor);	
	Dispersal(femaleBabies_,ran,m,kurtosisFactor);	
	Dispersal(maleBabies_,ran,m,kurtosisFactor);	
	}//end Population::JuvenileDispersal
*/


void Population::JuvenileDispersal(Ran3List& ran, aList<long double>& dispersalInfo){
	Dispersal(asexualBabies_,ran,dispersalInfo);	
	Dispersal(hermBabies_,ran,dispersalInfo);	
	Dispersal(femaleBabies_,ran,dispersalInfo);	
	Dispersal(maleBabies_,ran,dispersalInfo);	
	}//end Population::JuvenileDispersal



migrationModelSet Population::migrationModel(void) {
	if(site_!=NULL) return site_->migrationModel();
	return (migrationModelSet)0;}


long double Population::MigrationProb(int ageClass, int currentHabitat, int potentialHabitat){
	if(site_!=NULL) return site_->MigrationProb(species(), ageClass, currentHabitat, potentialHabitat);
	return zero;}


Individual& Population::mother(long i){
	long sum = 0; //Individual* indiv = NULL;
	if(i< sum+asexuals()){
		Individual& indiv = asexual(i-sum);
		return indiv; }
	sum += asexuals();
	if(i< sum+herms()){
		Individual& indiv = herm(i-sum);
		return indiv; }
	sum += herms();
	if(i< sum+females()){
		Individual& indiv = female(i-sum);
		return indiv; }
	return asexual(0); //default
	}//end Population::GetIndividual


long double Population::moveRate(genderSet sex, int age){
	if(site_!= NULL) {return site_->moveRate(species(),sex,age);}
	return zero;}

/*
void Population::NaturalSelection(Ran3List& ran, selectionSourceSet sSource){
	NaturalSelection(asexuals_, ran, sSource);
	NaturalSelection(herms_,  ran, sSource);
	NaturalSelection(females_,  ran, sSource);
	NaturalSelection(males_,  ran, sSource);
//	NaturalSelection(babies(), ran, sSource);
	} // end Population::NaturalSelection




void Population::NaturalSelection (ObjList<Individual>& theList, Ran3List& ran, selectionSourceSet sSource){
	for(long i=0;i<theList.len();++i){
		Individual* indiv = &(theList[i]);
		indiv->NaturalSelection(ran, sSource);
		if (!(indiv->alive() )){
			theList.Detach(i);
			deadList_.Append(indiv);}		
		}
	}//  end Population::NaturalSelection

*/


int Population::NoInconsistentSettings(void){//not programmed yet
	//returns an error code if settings are inconsistent
	return 1;//true
	}//end  Population::NoInconsistentSettings(void)


void Population::PrintAgeDistribution(ostream& outFile, int maxage){
	aList<int> ageDist;
	int i;
	for (i=0; i<maxage;++i){ageDist.Append(0);}
	GetAgeDistribution(ageDist);
	for (i=0;i<ageDist.len();++i){ 
		outFile<<ageDist[i];outFile<< " ";}//set to 4 places
	outFile<<endl;
	} /* end Population::PrintAgeDistribution*/



void Population::PrintPopHeading(ostream& outFile){
	/*just starting this pop, so print a heading*/
	outFile<<endl;	// set to 2 digits
	CoordinateType loc=coords();
	outFile<<"species "<<species_<<", pop: ("<<
			loc[0]<<", "<<loc[1]<< ")"<<endl;
	} /*end Population::PrintPopHeading*/


void Population::RandomDeathOfAdult (Ran3List& ran, int numToDie){
	//adults randomly die, with equal probability
	aList<int> listlens;
	listlens.Append(asexuals());
	listlens.Append(herms());
	listlens.Append(females());
	listlens.Append(males());
	if (numToDie>adults()) {
		numToDie=adults();}
	Individual* theAdult = NULL;
	int n,listNum,i;
	for(n=0;n<numToDie;++n){
		listNum = ran.weighted(listlens);
		if (listNum==0){
				i = ran.longBetween(0,asexuals()-1);
				theAdult= &(asexual(i));
				asexuals_.Detach(i);}
			else if (listNum==1){
				i = ran.longBetween(0,herms()-1);
				theAdult= &(herm(i));
				herms_.Detach(i);}
			else if (listNum==2){
				i = ran.longBetween(0,females()-1);
				theAdult= &(female(i));
				females_.Detach(i);}
			else{// (listNum==3)
				i = ran.longBetween(0,males()-1);
				theAdult= &(male(i));
				males_.Detach(i);}
		theAdult->Dies();
		deadList_.Append(theAdult);
		theAdult = NULL;}
	}//end Population::RandomDeathOfAdult



void Population::Recruit (long babyIndex){
	//nthBaby is the baby's index
	//finds the nth baby and promotes to an adult
	//does nothing if nthBaby is a bad index
	long babyNum=babyIndex;
	if ((!(babies()==0))&&(babyNum<=babies())){
		int done=0;
		//first check asexuals
		if (babyNum<asexualBabies()){
				Individual* baby= &(asexualBaby(babyNum));
				baby->lifeStage_ = adult;
				asexualBabies_.Detach(babyNum);
				Install(baby);
				return;}
			else{
				babyNum-=asexualBabies();}
		if (babyNum<hermBabies()){
				Individual* baby= &(hermBaby(babyNum));
				baby->lifeStage_ = adult;
				Install(baby);
				hermBabies_.Detach(babyNum);
				return;}
			else{
				babyNum-=hermBabies();}
		if (babyNum<femaleBabies()){
				Individual* baby= &(femaleBaby(babyNum));
				baby->lifeStage_ = adult;
				Install(baby);
				femaleBabies_.Detach(babyNum);
				return;}
			else{
				babyNum-=femaleBabies();}
		if (babyNum<maleBabies()){
				Individual* baby= &(maleBaby(babyNum));
				baby->lifeStage_ = adult;
				Install(baby);
				maleBabies_.Detach(babyNum);
				}
		}//end if bad index
	}//end Population::Recruit



void Population::Recruit (Ran3List& ran){
	//promotes babies to adults until carrying capacity is reached
	long num;
	bool listfound;
	Individual* baby;
	while ((adults() < carryingCapacity()) &&(babies() > 0)) {
		//recruit an offspring
		listfound=false;
		num = ran.longBetween(0,babies()-1);
		//get a baby from the asexual baby aList
		if (num<asexualBabies()){
				baby = &(asexualBaby(num));
				asexualBabies_.Detach(num); //detach the baby from that aList, since it is being recruited
				listfound=true;
				}
			else {num-=asexualBabies();}
		if (!listfound){
			if(num<hermBabies()){
					baby = &(hermBaby(num));
					hermBabies_.Detach(num);
					listfound=true;
					}
				else {num-=hermBabies();}
			}
		if (!listfound){
			if (num<femaleBabies()){
					baby = &(femaleBaby(num));
					femaleBabies_.Detach(num); 
					listfound=true;
					}
				else {num-=femaleBabies();}
			}
		if (!listfound){
			if (num<maleBabies()){
					baby = &(maleBaby(num));
					maleBabies_.Detach(num);
					listfound=true;
					}
			}
		baby->lifeStage_ = adult;
		Install(baby);	//put in the baby
		}//end while babies and slots available
	}//end Population::Recruit




void Population::ReplaceTheDead (Ran3List& ran){
	// If an adult is present, then as many new recruits
	// are produced as the number in the deadList.
	long i, num;int done;
	for (i=0;i<dead();++i){
		if ((adults() < carryingCapacity()) &&(babies() > 0)) {
			//recruit an offspring
			num = ran.longBetween(0,babies()-1); done=0;
			//get a baby from the asexual baby aList
			if (num<asexualBabies()){
					Individual* baby = &(asexualBaby(num));
					asexualBabies_.Detach(num); //delete the baby from that aList, since it is being recruited
					baby->lifeStage_ = adult;
					Install(baby);	//put in the baby
					return;
					}
				else {num-=asexualBabies();}
			if (num<hermBabies()){
					Individual* baby = &(hermBaby(num));
					hermBabies_.Detach(num); //delete the baby from that aList, since it is being recruited
					baby->lifeStage_ = adult;
					Install(baby);	//put in the baby
					return;
					}
				else {num-=hermBabies();}
			if (num<femaleBabies()){
					Individual* baby = &(femaleBaby(num));
					femaleBabies_.Detach(num); 
							//detach the baby from that aList, since it is being recruited
					baby->lifeStage_ = adult;
					Install(baby);	//put in the baby
					return;
					}
				else {num-=femaleBabies();}
			if (num<maleBabies()){
					Individual* baby = &(maleBaby(num));
					maleBabies_.Detach(num);
							//detach the baby from that aList, since it is being recruited
					baby->lifeStage_ = adult;
					Install(baby);	//put in the baby
					}
			} else {break;}	//end if babies available
		}	//end for i
	}	//end Population::ReplaceTheDead




void Population::Reproduce (long& idCount, Ran3List& ran, aList<long double>& specialInfo){
//void Population::Reproduce (long& idCount, Ran3List& ran){
	//individuals reproduce according to their individual fecundities
	//specialInfo holds data relevant to derived Indvidual classes
	int i;
	for (i=0;i<asexuals();++i){
		asexual(i).Reproduce(idCount,ran,specialInfo);}
	for (i=0;i<females();++i){
		female(i).Reproduce(idCount,ran,specialInfo);}
	for (i=0;i<herms();++i){
		herm(i).Reproduce(idCount,ran,specialInfo);}
	for (i=0;i<males();++i){//when meiosis produces gametes
		male(i).Reproduce(idCount,ran,specialInfo);}
	} //end Population::Reproduce 



void Population::ReproduceBySyngamy (long& idCount, Ran3List& ran, aList<long double>& specialInfo){
//void Population::Reproduce (long& idCount, Ran3List& ran){
	//individuals reproduce according to their individual fecundities
	//specialInfo holds data relevant to derived Indvidual classes
	//since gametes are combined upon recruitment to zygote stage, must work with decreasing lists
	long i;
	while(femaleGametes() + hermGametes() >1){
		i=ran.longBetween(0,femaleGametes() + hermGametes()-1);
		Gamete(i).Reproduce(idCount,ran,specialInfo);
		}
	ClearGameteLists();//if extras exist due to initial odd numbers
	} //end Population::ReproducebySyngamy



long Population::size(void){
	return adults() + babies() + gametes();
	}//end  Population::size


void Population::SetReproductiveStatus(void){};

GridCell& Population::site(void) {
	return *site_;}


void Population::SomeAdultsDie (aList<long double>& fSurvivalSchedule,
								 aList<long double>& mSurvivalSchedule,Ran3List& ran){
	int i;
	for (i=asexuals()-1;i>=0;--i){
		Individual* theAdult = &(asexual(i));
		theAdult->AdultMortality(fSurvivalSchedule,ran);
		if (!(theAdult->alive() )){
			asexuals_.Detach(i);
			deadList_.Append(theAdult);}
		}
	for (i=herms()-1;i>=0;--i){
		Individual* theAdult = &(herm(i));
		theAdult->AdultMortality(fSurvivalSchedule,ran);
		if (!(theAdult->alive() )){
			herms_.Detach(i);
			deadList_.Append(theAdult);}
		}
	for (i=females()-1;i>=0;--i){
		Individual* theAdult = &(female(i));
		theAdult->AdultMortality(fSurvivalSchedule,ran);
		if (!(theAdult->alive() )){
			females_.Detach(i);
			deadList_.Append(theAdult);}
		}
	for (i=males()-1;i>=0;--i){
		Individual* theAdult = &(male(i));
		theAdult->AdultMortality(mSurvivalSchedule,ran);
		if (!(theAdult->alive() )){
			males_.Detach(i);
			deadList_.Append(theAdult);}
		}
	}//end Population::SomeAdultsDie


void Population::SomeImmaturesDie (aList<long double>& fSurvivalSchedule,
								 aList<long double>& mSurvivalSchedule,Ran3List& ran){
	int i;
	Individual* theBaby;
	for (i=asexualBabies()-1;i>=0;--i){
		theBaby = &(asexualBaby(i));
		theBaby->AdultMortality(fSurvivalSchedule,ran);
		if (!(theBaby->alive() )){
			asexualBabies_.Detach(i);
			deadList_.Append(theBaby);}
		}
	for (i=hermBabies()-1;i>=0;--i){
		theBaby = &(hermBaby(i));
		theBaby->AdultMortality(fSurvivalSchedule,ran);
		if (!(theBaby->alive() )){
			hermBabies_.Detach(i);
			deadList_.Append(theBaby);}
		}
	for (i=femaleBabies()-1;i>=0;--i){
		theBaby = &(femaleBaby(i));
		theBaby->AdultMortality(fSurvivalSchedule,ran);
		if (!(theBaby->alive() )){
			femaleBabies_.Detach(i);
			deadList_.Append(theBaby);}
		}
	for (i=maleBabies()-1;i>=0;--i){
		theBaby = &(maleBaby(i));
		theBaby->AdultMortality(mSurvivalSchedule,ran);
		if (!(theBaby->alive() )){
			maleBabies_.Detach(i);
			deadList_.Append(theBaby);}
		}
	}//end Population::SomeImmaturesDie


void Population::SomeGametesDie (aList<long double>& fSurvivalSchedule,
								 aList<long double>& mSurvivalSchedule,Ran3List& ran){
	int i;
	Individual* theGamete;
	for (i=hermGametes()-1;i>=0;--i){
		theGamete = &(hermGamete(i));
		theGamete->AdultMortality(fSurvivalSchedule,ran);
		if (!(theGamete->alive() )){
			hermGametes_.Detach(i);
			deadList_.Append(theGamete);}
		}
	for (i=femaleGametes()-1;i>=0;--i){
		theGamete = &(femaleGamete(i));
		theGamete->AdultMortality(fSurvivalSchedule,ran);
		if (!(theGamete->alive() )){
			femaleGametes_.Detach(i);
			deadList_.Append(theGamete);}
		}
	for (i=maleGametes()-1;i>=0;--i){
		theGamete = &(maleGamete(i));
		theGamete->AdultMortality(mSurvivalSchedule,ran);
		if (!(theGamete->alive() )){
			maleBabies_.Detach(i);
			deadList_.Append(theGamete);}
		}
	}//end Population::SomeGametesDie


long Population::time(void){
		if(site_!=NULL) return site_->time();
		return 0;}


#endif

