/*******************************************************
*
*	SexualGeneticPopulation.cpp															
*
*	Superclass:  GeneticPopulation																							
*
*	This class handles processes within sexually reproducing genetic populations					
********************************************************/

#ifndef SexualGeneticPopulationcp
#define SexualGeneticPopulationcp = 1

#include "GeneticPopulation.h"
#include "SexualGeneticPopulation.h"
#include "RandomList.h"
#include "Element.cpp"
#include "aList.cpp"



void SexualGeneticPopulation::AssignMates (Ran3List& ran, aList<long double>& reproductionInfo, int tries, int maxThisAgeClass){
	for (long i=0;i<herms();++i){
		herm(i).FindMate(ran,reproductionInfo,tries,maxThisAgeClass);
		}//end for i
	for (long i=0;i<females();++i){
		female(i).FindMate(ran,reproductionInfo,tries,maxThisAgeClass);
		}//end for i
	}//end SexualGeneticPopulation::AssignMates


//void SexualGeneticPopulation::Reproduce (long& idCount, Ran3List& ran, double mutationRate, double sigma){
void SexualGeneticPopulation::Reproduce (long& idCount, Ran3List& ran, aList<long double>& specialInfo){
	//each hermaphrodite reproduces
	//according to its individual fecundity
	//derived classes handle separate genders, genetics, etc.
	//IDEAL HERMAPHRODITES, SO ALL OFFSPRING HAVE NEWLY CHOSEN FATHERS
	long old=babies();
//	GeneticPopulation::Reproduce (idCount,ran, mutationRate, sigma);//asexuals reproduce too
	GeneticPopulation::Reproduce (idCount,ran, specialInfo);//asexuals reproduce too
	for (int i=0;i<herms();++i){//hermaphrodites reproduce
		GeneticIndividual* gi = (GeneticIndividual*) &(herm(i));
//		gi->Reproduce(idCount,ran, mutationRate, sigma);
		gi->Reproduce(idCount,ran, specialInfo);
		}
	for (int i=0;i<females();++i){//females reproduce
		GeneticIndividual* gi = (GeneticIndividual*) &(female(i));
//		gi->Reproduce(idCount,ran, mutationRate, sigma);
		gi->Reproduce(idCount,ran, specialInfo);
		}
	if (old != babies()){//frequencies may change
		allFreqCurrent_=sampleSizeCurrent_=0;
		allelesCurrent_=hetExpDataCurrent_=hetObsDataCurrent_=0;}
	} //end SexualGeneticPopulation::Reproduce 



ObjList<Individual>* SexualGeneticPopulation::SampleNBabies(int nBabies, Ran3List& ran, 
															 SexualGeneticPopulation& theOtherPop){
	//samples babies from random matings with otherPop
	ObjList<Individual>* babyList = NEW ObjList<Individual>();
	//work with copies so that original remains undisturbed
	SexualGeneticPopulation thisPop(*this), otherPop(theOtherPop);
	thisPop.ClearOffspringLists();
	otherPop.ClearOffspringLists();
	long i;
	for(i=0;i<thisPop.adults(); ++i){//clear mates
		Individual& indiv = thisPop.GetIndividual(i);
		SexualGeneticIndividual* sgIndiv = (SexualGeneticIndividual*) &indiv;
		sgIndiv->ClearMates();
		}//end for i
	for(i=0;i<otherPop.adults(); ++i){//clear mates
		Individual& indiv = otherPop.GetIndividual(i);
		SexualGeneticIndividual* sgIndiv = (SexualGeneticIndividual*) &indiv;
		sgIndiv->ClearMates();
		}//end for i
	long babyID=0;
	aList<long double> dummyMutationParams;
	dummyMutationParams.Append(zero);
	dummyMutationParams.Append(zero);
	for(i=0;i<nBabies;++i){
		SexualGeneticIndividual *mom,*dad; 
		mom=dad=NULL;
		int i1 = ran.longBetween(0,thisPop.mothers()-1);
		int i2 = ran.longBetween(0,otherPop.fathers()-1);
		Individual& indiv1 = thisPop.mother(i1);
		Individual& indiv2 = otherPop.father(i2);
		mom = (SexualGeneticIndividual*) &indiv1;
		dad = (SexualGeneticIndividual*) &indiv2;
		mom->SetMate(dad,0);//0 is a dummy variable
//		const double zero=0.0;
//		mom->MakeABaby(babyID,ran,zero,zero);//no mutation
		mom->MakeABaby(babyID,ran,dummyMutationParams);//no mutation
		Individual& baby = mom->pop_->GetBaby(0);
		mom->pop_->Extract(&baby);
		babyList->Append(&baby);
		mom->ClearMates();
		dad->ClearMates();
		mom=dad=NULL;//forget
		}//end for i
	return babyList;
	} //end SexualGeneticPopulation::SampleNBabies 





#endif


