/******************************************
*
*	Randomness.cpp
*
*	handles 4-bit random number classes
*
******************************************/

#ifndef Randomnesscpp
#define Randomnesscpp = 1

#include "Randomness.h"




void Ran3::IRan3(void){
	//initializes 
	long double mj = mseed_ + idum_;
	if (!(mj < zero)) {
			mj -= mbig_ * long(mj / mbig_);}
		else {
//			mj = mbig_ - abs(mj) + mbig_ * long(abs(mj) / mbig_);}//this is the 2-bit version
//			mj = mod(mj, mbig_);//this is the 4-bit version
			mj = long(mj - mbig_ * long(mj/mbig_));//same, without mod call
			}
	ran3ma_[54] = mj;
	long double mk = one;
	long ii;
	for (long i=1;i<55;++i){// 1 to 54 in pascal
//		ii =  mod(21 *(i), 55);//fix operator mod long( i - j * long(i/j))
		long temp = 21*(i);
		ii =  long( temp - 55 * long(temp/55));//same without mod call 
		ran3ma_[ii-1] = mk;
		mk = mj - mk;
		if (mk < mz_) {mk += mbig_;}
		mj = ran3ma_[ii-1];
		}//	end for i
	for (int k=0;k<4;++k){
		for (int i=0;i<55;++i){ 
//			ii=(1 + mod((i + 30), 55))-1;
			long temp = (i + 30);
			ii=1+long( temp - 55 * long(temp/55))-1;//same without mod call 
			ran3ma_[i] -= ran3ma_[ii];
			if (ran3ma_[i] < mz_) {ran3ma_[i] += mbig_;}
			}
		}
	ran3Inext_ = 0;
	ran3Inextp_ = 31;
	idum_ = 1;
	}//end IRan3
	
long double Ran3::next(void){
	//returns a random number
	++ran3Inext_;
	if (ran3Inext_ == 56) {ran3Inext_ = 1;}
	++ran3Inextp_;
	if (ran3Inextp_ == 56) {ran3Inextp_ = 1;}
	long double mj = ran3ma_[ran3Inext_-1] - ran3ma_[ran3Inextp_-1];
	if (mj < mz_){
			mj += mbig_;}
	ran3ma_[ran3Inext_-1] = mj;
	lastReturned_ = mj * fac_;
	return lastReturned_;
	}// end Ran3::next
	
	




int Ran3::BernoulliDev(const long double p){//0p1
	//returns 0 if a random number p and 1 otherwise
	if(p<zero || p>one){
		aError("Bernoulli distribution parameter out of range");
		return 0;}
	int num=0;
	if(next()>p){num=1;}
	lastReturned_=num;
	return lastReturned_;
	}//Ran3::BernoulliDev()



/*	Random[BetaDistribution[p_, q_]] := 
  	With[{x1 = Random[GammaDistribution[p, 1]]},
  		x1/(x1 + Random[GammaDistribution[q, 1]])]
*/
long double Ran3::BetaDev(const long double a, const long double b){ 
	long double x1 = GammaDev(a, one);
	long double x2 = GammaDev(b, one);
	lastReturned_ = x1/(x1 + x2);
	return lastReturned_;
	}//end BetaDev(const double, const double)


long double Ran3::CauchyDev(void){
	//returns a random number from a Cauchy Distribiution
	long double ran2=GasDev(), ran1 = GasDev();
	while(ran2==zero){//eliminate zeros here
		ran2 = GasDev();}
	lastReturned_ = ran1/ran2;
	return lastReturned_;
	}//end CauchyDev()
	

long double Ran3::ChiSquareDev(const long double v){
	//returns a random number from a Chi-Square distribution with degrees of freedom = v
	return GammaDev(v/two,half);
	}//Ran3::ChiSquareDev()


/*	Random[GammaDistribution[alpha_, beta_]] :=
 		 Which[alpha < 1,
		  	beta gsGamma[alpha, alpha/E],
		alpha < 2.5,
	  		a = alpha-1;
	  		beta a gkm1Gamma[(alpha - 1/(6 alpha))/a, 2/a],  
		True,
	 		a = alpha-1;
	  		beta a gkm2Gamma[1/Sqrt[alpha], (alpha - 1/(6 alpha))/a, 2/a] 
  		]
*/
long double Ran3::GammaDev(const long double alpha, const long double beta){
	if(alpha < one){
		const double baseNaturalLog = 2.7182818284590452354;
		lastReturned_ = beta * gsGamma(alpha, alpha/baseNaturalLog);
		return lastReturned_;
		}
	long double a = alpha-one;
	if(alpha < (long double)2.5){
		lastReturned_ = beta * a * gkm1Gamma((alpha - one/(two*three*alpha))/a, two/a);
		return lastReturned_;
		} 
	lastReturned_ = beta * a * gkm2Gamma(one/sqrt(alpha), (alpha - one/(two*three*alpha))/a, two/a);
	return lastReturned_;
	}//end Ran3::GammaDev(const double, const double)


long double Ran3::GasDev (void) {
	//returns a random deviate from a standard normal distribution
	//adapted from Pascal Recipes p 225
	long double f, r, v1, v2;
	if (gasdevIset_ == 0){
			do{
				v1 = two * next() - one;
				v2 = two * next() - one;
				r = (v1*v1) + (v2*v2);
				}while ((r <= zero) || (r >= one));
			f = sqrt(-two * log(r) / r);
			gasdevGset_ = v1 * f;
			gasdevIset_ = 1;
			lastReturned_ = v2 * f;
			return lastReturned_;
			}
		else{
			gasdevIset_ = 0;
			lastReturned_ = gasdevGset_;
			return lastReturned_;
			}
	}//end Ran3::GasDev




long double Ran3::gkm1Gamma(const long double b, const long double c){
	long double u1 = next();
	long double u2 = next();
	long double w = b * u1 / u2;
	while(!(c*u2 - c - two + w + one/w <= zero) && 
		  !(c*ln(u2) - ln(w) + w - one < zero)){
		u1 = next();
		u2 = next();
		w = b * u1 / u2;
		}//end while
	lastReturned_ = w;
	return lastReturned_;
	}//end Ran3::gkm1Gamma(const double, const double)
	




long double Ran3::gkm2Gamma(const long double k, const long double b, const long double c){
	long double u1 = next();
	long double u2 = u1 + k*(one - (long double)1.86 * next());
	while((u2<=zero)||(u2>=one)){//while	!(0.0 < u2 < 1.0)
	   u1 = next();  //chooses u2 betw. 0 and 1
	   u2 = u1 + k*(one - (long double)1.86 * next());
	   }//end while
	long double w = b * u1 / u2;
	while(!(c*u2 - c - two + w + one/w <=zero) &&
           !(c*ln(u2) - ln(w) + w - one < zero)){
		u1 = next();
		u2 = u1 + k*(one - (long double)1.86 * next());
		while((u2<=zero)||(u2>=one)){//while !(0 < u2 < 1)
			u1 = next();
			u2 = u1 + k*(one - (long double)1.86 * next());
			}//end while !(0 < u2 < 1)
		w = b * u1 / u2;
		}//end while
	lastReturned_ = w;
	return lastReturned_;
	}//end Ran3::gkm2Gamma(const double k, const double b, const double c, Ran3& ran)



long double	Ran3::gsGamma(const long double alpha, const long double r){
	long double x = 1.0e0, t = 1.0e0;
	long double q = (one + r) * next();
	if(q < one){
    		x = exp((one/alpha)*ln(q));//q^(1/alpha)
    		t = exp(-x);}
    	else{
			x = one - ln(one + (one-q)/r);
			t = exp((alpha - one)*ln(x));//x^(alpha-1)
			}
	while(next() > t){
		q = (one + r) * next();
		if(q < one){
    			x = exp((one/alpha)*ln(q));//q^(1/alpha)
    			t = exp(-x);}
			else{
				x = one - ln(one + (one-q)/r);
				t = exp((alpha - one)*ln(x));//x^(alpha-1)
				}	
		}//end while
	lastReturned_ = x;
	return lastReturned_;
	}//end Ran3::gsGamma(const double, const double)


long Ran3::longBetween (long bottom, long top){
	long range = top - bottom + 1;
	long answer = long((next() * range) + bottom);
	lastReturned_ = answer;//type conversion
	return  lastReturned_;
	}//end Ran3::longBetween


double Ran3::doubleBetween (double bottom, double top){
	double range = top - bottom;
	lastReturned_ = (next() * range) + bottom;
	return  lastReturned_;
	}//end Ran3::doubleBetween


long double Ran3::longdoubleBetween (long double bottom, long double top){
	long double range = top - bottom;
	lastReturned_ = (next() * range) + bottom;
	return  lastReturned_;
	}//end Ran3::doubleBetween


long double Ran3::LogNormalDev(const long double mu, const long double sigma){
	//returns a random number from a log-normal distribution
	//with mean = mu and st.dev = sigma
	long double dev = GasDev();
	lastReturned_ = exp(mu + sigma * dev);
	return lastReturned_;
	}//Ran3::LogNormalDev



long double Ran3::NormDev (long double mu, long double sigma){
	//returns a random normal deviate 
	//from dist. of mean=mu and st.dev=sigma}
	lastReturned_ = (sigma * GasDev() + mu);
	return  lastReturned_;
	}


int Ran3::PoissonDev(const long double mu){
	long double answer = zero;
	if(mu<zero){
	 	aError("Can\'t draw from a poisson distribution with mean < 0");}
	int legal = true;
	 if(mu>zero){
		//returns a random integer from a Poisson distribution with mean mu
		//mu<= 25, so accumulate a product of random numbers
		if(mu<=(long double)25.0){
				long double threshold = exp(- mu);
				int num=0;
				long double product = one;
				while(legal){
					product *= next();
					if(product <= threshold){
						answer = num;
						}
					++num;
					if(product<10.0e-20){
						legal = false;}
					}
				}//mu<=25
			else{//if(mu>25.0){//approximate using Normal distribution
				long double sigma = sqrt(mu);
				while(legal){
					legal = true;
					long double z = NormDev(mu,sqrt(mu));
					answer = mu + (sigma * z) - half;
					if (answer<zero){legal = false;}
					}//while legal
				}//mu>25
		}//mu>0
	lastReturned_= (int) answer;	
	return (int) answer;//returns integer portion
	}
		
		
long double Ran3::WeibullDev(long double kappa, long double lambda, long double theta){
	long double dev = next(), answer = theta;
	if(theta>=zero && kappa>=zero && dev >= theta){
		long double val1 = - ln(dev);
		long double val2 = pow(val1,one/kappa);
		answer = theta + lambda * val2;
		}
	return answer;
	}// Ran3::WeibullDev(long double, long double, long double)



#endif

