/******************************************************
*
*	Fis_Fit_Q_DomParamRec.h
*
*	Parameters for estimating Fis or Fit and allele frequencies Q using a binomial distribution.
*	Use for diploid sex-linked/haplodiploid markers only
*
******************************************************/

#ifndef Fis_Fit_Q_DomParamRech
#define Fis_Fit_Q_DomParamRech = 1


#include "Element.h"
#include "aList.h"
#include "Randomness.h"
#include "ParamRec.h"
#include "Gamma.h"
#include "Fis_Fit_Q_ParamRec.h"
#include "InlineTemplates.h"
#include "MemoryManager.h"

class Fis_Fit_Q_DomParamRec;
class Fis_Fit_Q_DomParamRec: virtual public Fis_Fit_Q_ParamRec,virtual public ParamRec{
  public:
//	long double Fis_;//base-class variables
//	long double Fit_;
//	long double Fst_;
//	ObjList<ObjList<aList<long double> > > q_estimated_;//[pop][loc][allele] - base class variable --> use for homogametic sex
	ObjList<ObjList<aList<long double> > > q_hetero_estimated_;//[pop][loc][allele] - use for heterogametic sex
	int numqHeteroParams_, numIndependentqHeteroParams_;
//	ObjList<aList<long double> > Q_estimated_;//[loc][allele] - base class variable --> use for homogametic sex
	ObjList<aList<long double> > QHetero_estimated_;//[loc][allele]
	int numQHeteroParams_, numIndependentQHeteroParams_;
  	
  public:
  	Fis_Fit_Q_DomParamRec(void);
  	Fis_Fit_Q_DomParamRec(long double Fis_val, long double Fit_val, long double sr,
								 ObjList<ObjList<aList<long double> > >& init_q_estimated,//both sexes
								 ObjList<ObjList<aList<long double> > >& init_qHeterogametic_estimated);
  	Fis_Fit_Q_DomParamRec(long double Fis_val, long double Fit_val, long double sr,
								 ObjList<ObjList<aList<long double> > >& init_q_estimated,//both sexes
								 ObjList<ObjList<aList<long double> > >& init_qHeterogametic_estimated,
								 ObjList<aList<long double> >& init_QHetero_estimated_);
  	Fis_Fit_Q_DomParamRec(Fis_Fit_Q_DomParamRec& p);
  	virtual ~Fis_Fit_Q_DomParamRec(void);

  	virtual ParamRec* 				Duplicate(void);
  	Fis_Fit_Q_DomParamRec* 			_Duplicate(void);
	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
  	


	virtual int 		NumqHeteroParams(void){return numqHeteroParams_;}
	virtual int 		NumIndependentqHeteroParams(void){return numIndependentqHeteroParams_;}
	virtual int 		NumQHeteroParams(void){return numQHeteroParams_;}
	virtual int 		NumIndependentQHeteroParams(void){return numIndependentQHeteroParams_;}
	long double 		Q_hetero(int loc, int allele);//returns global estimated frequency
	long double 		q_hetero(int pop, int loc, int allele);//returns local estimated frequency

	virtual int 		Change_q_hetero(int paramNum, Ran3& ran, Fis_Fit_Q_DomParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		Change_Q_hetero(int paramNum, Ran3& ran, Fis_Fit_Q_DomParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double /*weight=1.0*/);
//	virtual int 	 	ChangeParams(Ran3& ran, //returns number of parameter changed
//									ParamRec* paramDeviations, 
//									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 	 	ChangeAnnealingParams(Ran3& ran, //returns number of parameter changed
									ParamRec* paramDeviations, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
			void		InitializeQ(void);
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumDerivedParams(void);
	virtual int 		NumParams(void);//number of parameters
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double 		ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
//	virtual int 		RandomParameter(Ran3& ran);
	virtual void		ResetParamChoices(void){
							Fis_Fit_Q_ParamRec::ResetParamChoices();
							paramsToChoose_ += numIndependentqHeteroParams_ + numIndependentQHeteroParams_;//num params not yet chosen
							}
	void				SetDependentAllele_q_hetero(int pop, int loc){
							long double summ=zero;
							int a,last_a = q_hetero_estimated_[pop][loc].len()-1;
							for(a=0;a<last_a;++a){
								summ += q_hetero_estimated_[pop][loc][a];}
							q_hetero_estimated_[pop][loc][last_a] = one - summ;
							}
	void				SetDependentAllele_Q_hetero(int loc){
							long double summ=zero;
							int a,last_a = QHetero_estimated_[loc].len()-1;
							for(a=0;a<last_a;++a){
								summ += QHetero_estimated_[loc][a];}
							QHetero_estimated_[loc][last_a] = one - summ;
							}
			void 		SetNumqParams(void);
			void 		SetNumQParams(void);
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
	virtual int 		TotalParams(void);//number of parameters that could vary, plus derived paramerters

	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(Fis_Fit_Q_DomParamRec& p, Fis_Fit_Q_DomParamRec& q);
	friend int	operator!=(Fis_Fit_Q_DomParamRec& p, Fis_Fit_Q_DomParamRec& q);

	};//end class Fis_Fit_Q_DomParamRec



#endif