/******************************************************
*
*	BartonHZModelGenotype.h
*
*	This class implements an AdditiveGenotype, and a selected genotype.
*	It uses a harmonic mean recombination rate between
*	the individual loci and the selected genotype.
*
*	Base class:  AdditiveGenotype
*
*******************************************************/

#ifndef BartonHZModelGenotypeh
#define BartonHZModelGenotypeh = 1
#include <iostream.h>
#include "Element.cpp"
#include "aList.cpp"
#include "Allele.cpp"
#include "Randomness.h"
#include "AdditiveGenotype.h"

class BartonHZModelGenotype: public AdditiveGenotype {
  protected:
  	ObjList<ObjList<Allele<int> > > loci_;
  public:
  	Genotype(void):loci_(){};//creates empty list
  	Genotype(int numLoci, int ploidy, int alleleVal);
  			//creates a 'null' homozygous individual at all loci
	Genotype(Genotype&);//copies the Genotype during construction
	Genotype(Genotype*);//copies the Genotype during construction
virtual ~Genotype(void){loci_.Clear();}//destroys the list
  	
virtual int	AllelePresent (int loc, int alleleVal);
				//counts the absolute frequency of alleleVal 

virtual int	CountAlleles (int loc, int alleleVal);
				//counts the absolute frequency of alleleVal 

virtual void	Copy (Genotype&);//copies to an existing genotype
virtual void	Copy (Genotype*);//copies to an existing genotype
	
virtual void	Fertilize (Genotype* momGamete, Genotype* dadGamete);

virtual void	Gamete (Genotype* parentGenes, Ran3& ran);

virtual int	GetGene (int locus, int alleleNum);
				//returns the value of the allele

virtual int	IsHeterozygote (int locus);
				//returns false=0 if haploid

virtual int	loci(void){return loci_.len();}
  	
virtual void	Mutate(int locus, int alleleNum, int newAlleleVal){
  				set(locus,alleleNum,newAlleleVal);}
  				//changes the value of an allele
  		
virtual int	PloidyAtLoc (int locus);
				//returns the number of alleles

virtual void	Print(ostream& outFile, int alleleLen=4);

virtual int	RandomAllele(int locus, Ran3& ran);
  				//randomly picks an allele and returns its value

virtual void	set(int locus, int alleleNum, int newAlleleVal);
  				//changes the value of an allele
  				//use when doing reproduction, etc.
  				//same as mutate, but term is not so process-laden
  				//USES ZERO-BASED INDICES FOR locus AND alleleNum
  		
virtual void	SetGenotypeString(aString& geneStr, int alleleLen=4);
	
virtual void	SetSexualZygote(Genotype* momGenes, Genotype* dadGenes, Ran3& ran);
				//uses mom's and dads genes to seed the genotype
	

friend int operator== (Genotype& g1, Genotype& g2);
		
friend int operator!= (Genotype& g1, Genotype& g2);
  	
  	};//end class Genotype
  	
  	
int operator== (Genotype& g1, Genotype& g2);
		
int operator!= (Genotype& g1, Genotype& g2);

#endif
