/******************************************************
*
*	Chromosome.h
*
*	This class implements a list of linked loci and
*	alleles.  It handles
*	cloning and recombination reproduction.
*
*	Base class:  ObjList<Allele<T> >
*
*******************************************************/

#ifndef Chromosomeh
#define Chromosomeh = 1

#include <iostream.h>
#include "Randomness.h"
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "Allele.h"
#include "MemoryManager.h"

template <class T> class Chromosome: public ObjList<Allele<T> > {
  protected:
  	aList<float> mapDist_;/*the map distance between loci
  				assumes transitivity in map distances*/
  public:
  	Chromosome(void):ObjList<Allele<T> >(), mapDist_(){
		}/*creates empty list*/
  	Chromosome(int numLoci, T alleleVal, float mapDist);
  			/*creates a 'null' chromosome with same allele value at all loci,
  			and same map distance between loci
  			 mapDist*(numLoci-1) must be 100*/
  	Chromosome(int numLoci, Allele<T>& alleleVal, float mapDist);
  			/*creates a 'null' chromosome with same allele at all loci,
  			and same map distance between loci
  			 mapDist*(numLoci-1) must be 100*/
  	Chromosome( aList<T>& alleleVal, aList<float>& mapDist);
  	Chromosome( ObjList<Allele<T> >& alleleVal, aList<float>& mapDist);
	Chromosome(Chromosome<T>& old);/*copies the Chromosome during construction*/
	Chromosome(Chromosome<T>* old);/*copies the Chromosome during construction*/
virtual ~Chromosome(void){
		Clear();
		}/*destroys the list*/
  	
//virtual int	AllelePresent (int loc, T alleleVal);
virtual int	AllelePresent (int loc, aString& alleleVal);
virtual int	AllelePresent (int loc, Allele<T>& a);

void			Clear (void){
				mapDist_.Clear();
				ObjList<Allele<T> >::Clear();
				}/*end Clear*/


virtual void	Copy (Chromosome<T>&);/*copies to an existing Chromosome*/
virtual void	Copy (Chromosome<T>*);/*copies to an existing Chromosome*/
	
virtual Allele<T>&		GetAllele (int locus);
				/*returns the value of the allele*/
virtual T		GetGene (int locus);
				/*returns the value of the allele*/

virtual aString* GenotypeString(char* separator = " ");
virtual aString* GenotypeString(int allele, char* separator = " ");

virtual int 	IsX(void){return 0;}//default is not X chromosome
virtual int 	IsY(void){return 0;}//default is not Y chromosome

virtual int		loci(void){return len();}
  	
virtual float	mapDist(int loc){return mapDist_[loc];}
  	
virtual void	Mutate(Ran3& ran, double mutationRate, double sigma);
  				/*changes the value of an allele*/
  		
virtual void	Mutate(int locus, T newAlleleVal){
  				set(locus,newAlleleVal);}
  				/*changes the value of an allele*/
  		
virtual void	Print(ostream& outFile, int alleleLen=4);/*outFile assumed open*/

virtual void	Recombine(Chromosome<T>& c1, Chromosome<T>& c2, Ran3& ran);
				/*makes a recombinant Chromosome from c1 & c2 
				based on map distances among loci */

virtual void	Recombine(Chromosome<T>* c1, Chromosome<T>* c2, Ran3& ran){
					Recombine(*c1, *c2, ran);}

virtual void	RecombineWith(Chromosome<T>& c2, Ran3& ran);
				/*makes 2 recombinant Chromosomes from this & c2 
				based on map distances among loci */

virtual void	RecombineWith(Chromosome<T>* c2, Ran3& ran){
					RecombineWith(*c2, ran);}

virtual void	set(int locus, T newAlleleVal);
  				/*changes the value of an allele
  				use when doing reproduction, etc.
  				same as mutate, but term is not so process-laden
  				USES ZERO-BASED INDICES FOR locus 	*/

virtual void	SetGenotypeString(aString& geneStr, int alleleLen=4);

Chromosome<T>& operator= (Chromosome<T>& c2){
					Clear();
					mapDist_ = c2.mapDist_;
					for(int i=0;i<c2.len();++i){
						Allele<T>& x = c2[i];
						Allele<T>* y = NEW Allele<T>(x);
						Append(y);}
					return *this;
					}//end operator=
	
Chromosome<T>& operator= (Chromosome<T>* c2){
					return operator= (*c2);
					}//end operator=
	
friend int operator== (Chromosome<T>& c1, Chromosome<T>& c2);
		
friend int operator!= (Chromosome<T>& c1, Chromosome<T>& c2);
  	
friend int operator< (Chromosome<T>& c1, Chromosome<T>& c2);/*compares by allele value, for sorting*/
  	
friend int operator> (Chromosome<T>& c1, Chromosome<T>& c2);/*compares by allele value, for sorting*/
  	
friend int operator<= (Chromosome<T>& c1, Chromosome<T>& c2);/*compares by allele value, for sorting*/
  	
friend int operator>= (Chromosome<T>& c1, Chromosome<T>& c2);/*compares by allele value, for sorting*/
  	

  	};/*end class Chromosome<T>*/
  	




  	
template <class T> int operator== (Chromosome<T>& g1, Chromosome<T>& g2);
		
template <class T> int operator!= (Chromosome<T>& g1, Chromosome<T>& g2);

template <class T> int operator< (Chromosome<T>& c1, Chromosome<T>& c2);/*compares by allele value, for sorting*/
  	
template <class T> int operator> (Chromosome<T>& c1, Chromosome<T>& c2);/*compares by allele value, for sorting*/
  	
template <class T> int operator<= (Chromosome<T>& c1, Chromosome<T>& c2);/*compares by allele value, for sorting*/
  	
template <class T> int operator>= (Chromosome<T>& c1, Chromosome<T>& c2);/*compares by allele value, for sorting*/
  	

#endif
