/*************************************************************
*
*	DoseRespClineCumNormParamRec.h
*
*	This fits a set of cumulative normal dose response curves to a Barton cline model.
*
*************************************************************/


#ifndef DoseRespClineCumNormParamRech
#define DoseRespClineCumNormParamRech = 1


#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include "InlineTemplates.h"
#include "StringClass.h"
#include "ParamRec.h"
#include "Element.cpp"
#include "aList.cpp"
#include "ClineShapeCWTBParamRec.h"
#include "DoseResponseParamRec.h"

class DoseRespClineCumNormParamRec;
class DoseRespClineCumNormParamRec: public ClineShapeCWTBParamRec{
  public:
  	ObjList<ParamRec> doseResponseCurve_;//always holds DoseResponseParamRec
  	aList<double> location_;

  	DoseRespClineCumNormParamRec(void);
  	DoseRespClineCumNormParamRec(double c, double w, 
  					aList<double>& low, aList<double>& high, 
  					DoseResponseParamRec& p, aList<double>& locations
  					);
  	DoseRespClineCumNormParamRec(DoseRespClineCumNormParamRec& p);
  	DoseRespClineCumNormParamRec(DoseRespClineCumNormParamRec* p);
  	virtual ~DoseRespClineCumNormParamRec(void){
  			pointerToDerivedSelf = 0;//forget this
  			Clear();
  			}
  	
  	//OVERRIDE THESE IN ALL DERIVED CLASSES
  	virtual ParamRec* 				Duplicate(void);
  	virtual DoseRespClineCumNormParamRec* 	_Duplicate(void);
	virtual void 					SetPointerToDerivedSelf(void);
	
  	
	virtual int 		ChangeDoseMean(int pop, Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
							//intercepts are estimated separately for each location
	virtual int 		ChangeDoseSlope(int pop, Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
							//slopes are estimated separately for each location
	virtual int 		ChangeParam(int paramNum, Ran3& ran, 
									ParamRec* paramVarationValues, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
							//returns 0 for failure, 1 for success
	virtual void 		Clear(void);
	virtual double 		ExpectedValue(aList<double>& coords);
	virtual int 		locations(void);//number of locations
	virtual int 		MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, double factor);//use to rescale a param deviation
	virtual int 		NumParams(void);//number of parameters that could vary
	virtual int 		NumDerivedParams(void);//number of parameters that are functions of varying parameters 
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual double 		ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
	virtual int 		TotalParams(void);//number of parameters that could vary, plus derived paramerters
	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

			friend int	operator==(DoseRespClineCumNormParamRec& p, DoseRespClineCumNormParamRec& q);
			friend int	operator!=(DoseRespClineCumNormParamRec& p, DoseRespClineCumNormParamRec& q);

	};//end class DoseRespClineCumNormParamRec


#endif
