/******************************************************
*
*	CoreGenome.h
*
*	This virtual base class implements genotypic functions.  
*	It handles cloning and sexual reproduction.
*
*	Base class:  Phenotype
*
*******************************************************/

#ifndef CoreGenomeh
#define CoreGenomeh = 1

#include <math.h>
#include <iostream.h>
#include "Randomness.h"
#include "RandomList.h"
#include "StringClass.h"
#include "Element.cpp"
#include "aList.cpp"
#include "Phenotype.h"


class CoreGenome: public Phenotype {
  public:
  	CoreGenome(void):Phenotype(){
			}//creates empty list

  	CoreGenome(ViabilitySelectionFunc viabSelection):Phenotype(viabSelection){
			}/*creates empty list*/

  	CoreGenome(CoreGenome& g):Phenotype(g.traits(), g.selectionFunc){
			}/*creates empty list*/

  	CoreGenome(CoreGenome* g):Phenotype(g->traits(), g->selectionFunc){
			}/*creates empty list*/

virtual ~CoreGenome(void){
			}//end destructor
		virtual long GetPointerToDerivedSelf(void){
					return (long) this;}


virtual long double  additiveRelativePhenotype(void)=0;//derive for an additive phenotype

virtual int		AllelePresent (int /*loc*/, aString& /*alleleVal*/)=0;

virtual int		chroms1N(void);//default is unlinked loci
					
virtual int		CountAlleles (int /*loc*/, aString& /*alleleVal*/)=0;
				/*counts the absolute frequency of alleleVal */

virtual void	Fertilize (CoreGenome* /*momGamete*/, CoreGenome* /*dadGamete*/){}

//virtual void	Gamete (CoreGenome* /*parentGenes*/, Ran3List& /*ran*/, double /*mutationRate*/, double /*sigma*/){}
virtual void	Gamete (CoreGenome* /*parentGenes*/, Ran3List& /*ran*/, aList<long double>& /*specialInfo*/){}

virtual aString* 	GenotypeString(char* /*separator=" "*/)=0;
					//returns all ploidy copies separated by parentheses (A)(A) at each locus
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(int /*locus*/, char* /*separator=" "*/)=0;
					//finds nth overall locus and returns all ploidy copies separated by parentheses (A)(A)
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(int /*locus*/, int /*alleleNum*/,  char* /*separator=" "*/)=0;
					//finds nth overall locus and returns the alleleNum_th copy

virtual void	GetGenome(GeneContainerCore& /*gc*/){//refills container class at appropriate level
						aError("Error: base class CoreGenome has no explicit genome");}

virtual void	SetGenome(GeneContainerCore& /*gc*/){//initializes genome using appropriate level of container class
						aError("Error: base class CoreGenome has no explicit genome");}

virtual int 	HasY(void){return 0;}//0 if false, 1 if Y chromosome present

virtual int		IsHeterozygote (int /*locus*/)=0;
				/*returns false=0 if haploid*/

virtual int		loci(void){return 0;}//default is nonsense

virtual int		loci(int /*chrom*/){if(loci()>0){return 1;}else{return 0;}}//default is unlinked loci

//virtual void	Mutate(Ran3& /*ran*/, double/* mutationRate*/, double /*sigma=0.0*/)=0;
virtual void	Mutate(Ran3& /*ran*/, aList<long double>& /*specialInfo*/)=0;

  			
virtual long double	phenotype(int i=0){
					return trait(i);}

virtual int 	ploidy (int /*locus=0*/)=0;
  		
virtual void	Print(ostream& /*outFile*/){}

virtual void	Print(ostream& /*outFile*/, int /*alleleLen=4*/){}

virtual void	SetGenotypeString(aString& /*geneStr*/, int /*alleleLen=4*/)=0;
	
virtual void	SetPhenotype(void){}
				//default is no phenotype
				
//virtual void	SetPhenotype(Ran3& /*ran*/, double /*heritability = 1.0*/){}
virtual void	SetPhenotype(Ran3& /*ran*/, aList<long double>& /*specialInfo*/){}
				//default is no phenotype

//virtual void 	SetSexualZygote(CoreGenome* /*momGenes*/, CoreGenome* /*dadGenes*/, 
//								Ran3List& /*ran*/, double /*mutationRate*/, double /*sigma*/){}
virtual void 	SetSexualZygote(CoreGenome* /*momGenes*/, CoreGenome* /*dadGenes*/, 
								Ran3List& /*ran*/, aList<long double>& /*specialInfo*/){}
					//actually defined in sexual classes
				

friend int operator== (CoreGenome& g1, CoreGenome& g2);
		
friend int operator!= (CoreGenome& g1, CoreGenome& g2);
  	
  	};//end class CoreGenome





#endif
