/*************************************************************
*
*	DoseRespHeritabilityParamRec.h
*
*	This fits a set of cumulative normal dose response curves to a linear model,
*	using midparents of each dose curve.
*
*************************************************************/


#ifndef DoseRespHeritabilityParamRech
#define DoseRespHeritabilityParamRech = 1


#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include "InlineTemplates.h"
#include "StringClass.h"
#include "ParamRec.h"
#include "Element.cpp"
#include "aList.cpp"
#include "DoseResponseParamRec.h"

class DoseRespHeritabilityParamRec;
class DoseRespHeritabilityParamRec: virtual public ParamRec{
  public:
  	long double slope_;//linear regression
  	long double intercept_;//linear regression
  	ObjList<ParamRec> doseResponseCurve_;//always holds DoseResponseParamRec
  	aList<long double> midparent_;

  	DoseRespHeritabilityParamRec(void);
	DoseRespHeritabilityParamRec(long double a, long double b, 
  					DoseResponseParamRec& p, aList<long double>& midparentList);
  	DoseRespHeritabilityParamRec(DoseRespHeritabilityParamRec& p);
  	DoseRespHeritabilityParamRec(DoseRespHeritabilityParamRec* p);
  	virtual ~DoseRespHeritabilityParamRec(void){
  			pointerToDerivedSelf = 0;//forget this
  			Clear();
  			}
  	
  	//OVERRIDE THESE IN ALL DERIVED CLASSES
  	virtual ParamRec* 				Duplicate(void);
  	virtual DoseRespHeritabilityParamRec* 	_Duplicate(void);
	virtual void 					SetPointerToDerivedSelf(void);
	
  	
	virtual int 		ChangeDoseMean(int pop, Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
							//intercepts are estimated separately for each location
	virtual int 		ChangeDoseSlope(int pop, Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
							//slopes are estimated separately for each location
	virtual int 		ChangeIntercept(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, 
									ParamRec* paramVarationValues, 
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=one);
	virtual int 		ChangeSlope(Ran3& ran, ParamRec* paramDeviations, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
							//returns 0 for failure, 1 for success
	virtual void 		Clear(void);
	virtual long double 		ExpectedValue(aList<long double>& coords);
	virtual long double 		heritability(void);//returns value
	virtual long double 		intercept(void);//returns value
	virtual void 		intercept(long double a);//sets value
	virtual int 		midparents(void);//number of locations
	virtual int 		MeetsParamCriteria (ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumParams(void);//number of parameters that could vary
	virtual int 		NumDerivedParams(void);//number of parameters that are functions of varying parameters 
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double 	ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual long double 		slope(void);//returns value
	virtual void 		slope(long double b);//sets value
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
	virtual int 		TotalParams(void);//number of parameters that could vary, plus derived paramerters
	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

			friend int	operator==(DoseRespHeritabilityParamRec& p, DoseRespHeritabilityParamRec& q);
			friend int	operator!=(DoseRespHeritabilityParamRec& p, DoseRespHeritabilityParamRec& q);

	};//end class DoseRespHeritabilityParamRec


#endif
