/******************************************************
*
*	DoseResponseParamRec.h
*
*	Parameters for fitting dose-response curves by maximum likelihood.
*	It's really just a version of a Cumulative Normal Distribution
*	
*
******************************************************/

#ifndef DoseResponseParamRech
#define DoseResponseParamRech = 1


//#include "ParamRec.h"
//#include "NormalDistParamRec.h"
#include "CumNormalDistParamRec.h"

class DoseResponseParamRec;
class DoseResponseParamRec: public CumNormalDistParamRec{
  public:
  	DoseResponseParamRec(void);
  	DoseResponseParamRec(double m, double s);
  	DoseResponseParamRec(DoseResponseParamRec& p);
  	virtual ~DoseResponseParamRec(void);
  	virtual ParamRec* 				Duplicate(void);
  	virtual DoseResponseParamRec* 	_Duplicate(void);
	virtual void 					SetPointerToDerivedSelf(void);//always override in derived

	virtual double 		LD50(void);//LD 50 value
	virtual double 		DoseAtMortality(double mortalityFraction);
	virtual double 		LD(double mortalityPercent);
	virtual double 		MortalityAtDose(double dose);
	virtual double 		naturalResponse(void);//fitted mortzality at dose zero
	virtual double 		slope(void);// = 1/(sigma*sqrt(2))

	
	virtual int 		NumDerivedParams(void);//number of parameters that are functions of varying parameters 
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double 	ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file
	virtual int 		TotalParams(void);//5 returned typically


	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(DoseResponseParamRec& p, DoseResponseParamRec& q);
	friend int	operator!=(DoseResponseParamRec& p, DoseResponseParamRec& q);

	};//end class DoseResponseParamRec



#endif