/******************************************************
*
*	NormalDistParamRec.h
*
*	Parameters for a normal distribution defined by mean and stdev.
*
*	Use this as a base class for derived classes that add parameters.
*
******************************************************/

#ifndef NormalDistParamRech
#define NormalDistParamRech = 1


#include "InlineTemplates.h"
#include "Element.h"
#include "aList.h"
#include "ParamRec.h"
#include "Randomness.h"
#include "Stringclass.h"
#include "NormalDistrib.h"

class NormalDistParamRec;
class NormalDistParamRec: virtual public ParamRec{
  public:
	long double mu_;//mean
	long double sigma_;//stdev
  	
  public:
  	NormalDistParamRec(void):ParamRec(), mu_((long double) 0.0), sigma_((long double)1.0){
//  				SetPointerToDerivedSelf();
  				}
  	NormalDistParamRec(double m, double s):ParamRec(), mu_(m), sigma_(s){
//  				SetPointerToDerivedSelf();
  				}
  	NormalDistParamRec(NormalDistParamRec& p): 
  					mu_(p.mu_), 
  					sigma_(p.sigma_){
//  				SetPointerToDerivedSelf();
				lnL_= p.lnL_;
				lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;	
				}
  	virtual ~NormalDistParamRec(void){
//  				pointerToDerivedSelf = NULL;//forget
  				}

  	virtual ParamRec* 				Duplicate(void);
  	virtual NormalDistParamRec* 	_Duplicate(void);
//	virtual void 					SetPointerToDerivedSelf(void);//always override in derived
	virtual long 					GetPointerToDerivedSelf(void);//always override in derived
  	


			long double 		mu(void);//returns mu_
			void 		mu(long double m);//sets mu_
			long double 		sigma(void);//returns sigma_
			void 		sigma(long double s);//sets sigma_	
			int 		ChangeMu(Ran3& ran, double sigmaForNewMu, 
							ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual int 		ChangeParam(int paramNum, Ran3& ran, //returns 0 for failure, 1 for success
									ParamRec* paramDeviations,  
									ParamRec* lowerLimits, ParamRec* upperLimits, long double weight=1.0);
//	virtual int 	 	ChangeParams(Ran3& ran, //returns number of parameter changed
//									ParamRec* paramDeviations, 
//									ParamRec* lowerLimits, ParamRec* upperLimits);
			int 		ChangeSigma(Ran3& ran, double sigmaForNewSigma, 
									ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual long double ExpectedValue(aList<long double>& coords);
	virtual int 		MeetsParamCriteria(ParamRec* lowerLimits, ParamRec* upperLimits);
	virtual void 		MultiplyParam(int paramNum, long double factor);//use to rescale a param deviation
	virtual int 		NumDerivedParams(void);//number of parameters that are functions of varying parameters 
	virtual int 		NumParams(void);//number of parameters
	virtual aString* 	ParamName(int paramNum);//returns 'new' name of parameter; must be destroyed elsewhere
	virtual void 		ParamString(aString& dataline, char sepchar='\t');//a list of the values that will be printed to a big file
	virtual long double ParamValue(int paramNum);//returns value of parameter
	virtual void 		Print(ostream& outFile, long rep, aString& message);
	virtual void 		SetParamValue(int paramNum, long double newVal);//sets value of parameter
	virtual void 		TitleString(aString& dataline, char sepchar='\t');//a list of the column headings that will be printed to a big file

	virtual void 		operator=(ParamRec& p);
							//derived classes with ParamRec as a virtual base class use
							// operator=() in a tricky way.  So, ONLY use operator=() between
							//objects of the same class.

	friend int	operator==(NormalDistParamRec& p, NormalDistParamRec& q);
	friend int	operator!=(NormalDistParamRec& p, NormalDistParamRec& q);

	};//end class NormalDistParamRec



#endif