/**********************************************
*
*	EpistaticGenome.h
*
*	This class implements an n-locus Genome with arbitrary
*	linkage relationships, where the phenotype is arbitrarily non-additive.
*	To determine phenotype and fitness, general reference lists are maintained.
* 	The reference lists must be initialized separately.
*
*	Base Class: Genome
*
**********************************************/

#ifndef EpistaticGenomeh

#define EpistaticGenomeh = 1

#include <iostream.h>
#include "Element.h"
#include "aList.h"
#include "Genome.h"
#include "ChromosomeList.h"

template <class T> class EpistaticGenome : public Genome<T>{
  public:
	static ObjList<ObjList<ObjList<aList<T> > > > possibleGenotypes_;
			//list of general implementations of this class, each
			//arranged as lists of possible individuals, 
			//each with lists of loci, these with sorted, ploidy-length allele combinations
	static ObjList<aList<double> > possiblePhenotypes_;
	static ObjList<aList<double> > possibleFitnesses_;
			//arranged in same order as genotypes above
	int kind_;//which kind of EpistaticGenome when there are more than one
  public:
  	EpistaticGenome(int kind=0):Genome<T>(),kind_(kind){}/*creates empty list*/
  	EpistaticGenome(ViabilitySelectionFunc viabSelection, int kind=0):
  				Genome<T>(viabSelection),kind_(kind){}/*creates empty list*/
  	EpistaticGenome(int numLociPerChrom, int num1NChroms, int ploidy, T alleleVal, int kind=0):
  				Genome<T>(numLociPerChrom, num1NChroms, ploidy, alleleVal),kind_(kind){}
  			/*creates a 'null' homozygous individual at all loci*/
  	EpistaticGenome(ObjList<ChromosomeList<T> >& c, int kind=0):  Genome<T>(c),kind_(kind) {}
  	EpistaticGenome(int numLociPerChrom, int num1NChroms, int ploidy, T alleleVal, 
  			ViabilitySelectionFunc viabSelection, int kind=0):
  				Genome<T>(numLociPerChrom, num1NChroms, ploidy, alleleVal, viabSelection),kind_(kind){}
  			/*creates a 'null' homozygous individual at all loci*/
  	EpistaticGenome(ObjList<ChromosomeList<T> >& c, ViabilitySelectionFunc viabSelection, int kind=0):
  	  Genome<T>(c, viabSelection),kind_(kind) {}
	EpistaticGenome(EpistaticGenome<T>& old):Genome<T>(old), kind_(old.kind_){}
	EpistaticGenome(EpistaticGenome<T>* old):Genome<T>(old), kind_(old->kind_){}
virtual ~EpistaticGenome(void){}

virtual Phenotype* new_Phenotype(void){
					return new EpistaticGenome<T>(kind_);}

virtual Phenotype* Duplicate(void);//virtual copy constructor
	


  public:
    ObjList<aList<T> >* genotypeSummary(void);
  	double rawPhenotype(void);//returns the correct possiblePhenotype_ value
  	double rawFitness(void);//returns the correct possibleFitnesses_ score
  		//both the above may later be modified by other factors by Individual, 
  		//where phenotype and fitness are assessed


};//end class EpistaticGenome


#endif
