
	//GeneContainers.h

	//Use these classes to swap data between specialized simulation and data-analysis classes
	//that may store data in arbitrary ways

#ifndef GeneContainersh
#define GeneContainersh = 1


#include "aError.h"
#include "Element.h"
#include "aList.h"
#include "PopulationIntf.h"
#include <iostream.h>
#include "StringClass.h"
#include "MemoryManager.h"
#include "GeneContainerCore.h"

template <class T> class HaplotypeContainer;
//template <class T> class MultiHaplotypeContainer;
//template <class T> class MultiInheritanceModeContainer;
//template <class T> class PopulationGeneContainer;


template <class T> class HaplotypeContainer: public GeneContainerCore{
		//stores copies using operator=; not for unique objects that cannot be copied!
	public:
		T *haplotype_;
		int nloci_;
		bool missing_;//dummy for missing data
	public:
		HaplotypeContainer(void):nloci_(0),haplotype_(NULL), missing_(true){}

		HaplotypeContainer(long IDval, int numLoci):nloci_(numLoci), missing_(false),haplotype_(NULL){
			id_=IDval;
			if(nloci_==0){
				missing_=true; return;}
			haplotype_ = NEW T[nloci_]();
			int i;
			for(i=0;i<nloci_;++i){
				haplotype_[i] = 0;}
			}
		HaplotypeContainer(long IDval, int numLoci, T* geneVals):nloci_(numLoci),missing_(false),haplotype_(NULL){
			id_=IDval;
			if(nloci_==0){
				missing_=true; return;}
			haplotype_ = NEW T[nloci_]();
			int loc;
			for(loc=0;loc<nloci_;++loc){
				haplotype_[loc] = geneVals[loc];}
			}
		HaplotypeContainer(long IDval, aList<T>& geneVals):nloci_(geneVals.len()),missing_(false),haplotype_(NULL){
			id_=IDval;
			if(nloci_==0){
				missing_=true; return;}
			haplotype_ = NEW T[nloci_]();
			int loc;
			for(loc=0;loc<nloci_;++loc){
				haplotype_[loc] = geneVals[loc];}
			}
		HaplotypeContainer(long IDval, ObjList<T>& geneVals):nloci_(geneVals.len()),missing_(false),haplotype_(NULL){
			id_=IDval;
			if(nloci_==0){
				missing=true; return;}
			haplotype_ = NEW T[nloci_]();
			int loc;
			for(loc=0;loc<nloci_;++loc){
				haplotype_[loc] = geneVals[loc];}
			}
		HaplotypeContainer(HaplotypeContainer<T>& hc):nloci_(hc.nloci_),haplotype_(NULL),missing_(hc.missing_){
			id_=hc.id_;
			if(nloci_==0){
				missing_=true; return;}
			haplotype_ = NEW T[nloci_]();
			int i;
			for(i=0;i<nloci_;++i){
				haplotype_[i] = hc.haplotype_[i];}
			}
		HaplotypeContainer(HaplotypeContainer<T>* hc):nloci_(hc->nloci_),haplotype_(NULL),missing_(hc->missing_){
			id_=hc->id_;
			if(nloci_==0){
				missing_=true; return;}
			haplotype_ = NEW T[nloci_]();
			int i;
			for(i=0;i<nloci_;++i){
				haplotype_[i] = hc->haplotype_[i];}
			}
		~HaplotypeContainer(void){
				delete [] haplotype_; haplotype_=NULL;}
		
		virtual GeneContainerCore* Duplicate(void){//virtual copy
					return NEW HaplotypeContainer<T>(*this);}
		virtual long GetPointerToDerivedSelf(void){
					return (long) this;}
		
		virtual void Clear(void){
					GeneContainerCore:Clear();
					nloci_=0;
					delete [] haplotype_; haplotype_=NULL;
					missing_=true;
					}
		
		virtual int EqualQ(GeneContainerCore& gc2){//virtual operator==
					HaplotypeContainer* hc2 = (HaplotypeContainer*) gc2.GetPointerToDerivedSelf();
					if(missing_ == hc2->missing_){return GeneContainerCore::EqualQ(gc2);}
					if(nloci_ != hc2->nloci_){return 0;}
					int loc;
					for(loc=0;loc<nloci_;++loc){
						if(haplotype_[loc] != hc2->haplotype_[loc]){return 0;}}
					return GeneContainerCore::EqualQ(gc2);
					}

		T& haplotype(int loc){
				if(loc<0 || loc >=nloci_ || missing_ || haplotype_==NULL){
					aError("ERROR: HaplotypeContainer<T>: tried to reference a value out of range.");}
				return haplotype_[loc];}
		
		int loci(void){return nloci_;}
		
		bool missing(void){return missing_;}
		
		int ploidy(void){return 1;}
		
		virtual void Print(ostream& output, char sepChar){
					aString outStr("ID:"); 
					outStr.cat(id_); outStr+sepChar;
					if(missing_ || haplotype_==NULL){
							outStr+"missing";}
						else{
							int loc;
							for(loc=0;loc<nloci_-1;++loc){
								outStr.cat(haplotype_[loc]); outStr+sepChar;}
							outStr.cat(haplotype_[nloci_-1]);
							}
					output<<outStr<<endl;
					}
		
		void setHaplotype(int loc, T hapValue){
				if(loc<0 || loc>= nloci_ || haplotype_==NULL) return;
				missing_=false;
				haplotype_[loc] = hapValue;}
		
		virtual void SetSize(int newNloci){
			if(nloci_ != newNloci){
				nloci_ = newNloci;
				delete [] haplotype_;
				haplotype_=NULL;
				if(nloci_>0){
					missing_=false;
					haplotype_ = NEW T[nloci_]();
					}
				int i;
				for(i=0;i<nloci_;++i){
					haplotype_[i] = 0;}
				}
			}
				
		HaplotypeContainer& operator=(HaplotypeContainer<T>& hc){
			GeneContainerCore::operator=(hc);
			missing_=hc.missing_;
			if(nloci_ != hc.nloci_){
				delete [] haplotype_;haplotype=NULL;
				nloci_ = hc.nloci_;
				if(nloci_==0){
					missing=true; return;}
				haplotype_ = NEW T[nloci_]();
				}
			int i;
			for(i=0;i<nloci_;++i){
				haplotype_[i] = hc.haplotype_[i];}
			return *this;
			}
		
		T& operator[](int loc){
				if(loc<0 || loc >=nloci_ || missing_ || haplotype_==NULL){
					aError("ERROR: HaplotypeContainer<T>: tried to reference a value out of range.");}
				T& hapval =  haplotype_[loc];
				return hapval;}
		
		friend int operator==(HaplotypeContainer<T>& hc1, HaplotypeContainer<T>& hc2);
		friend int operator!=(HaplotypeContainer<T>& hc1, HaplotypeContainer<T>& hc2);
		};//end template <class T> class HaplotypeContainer<T>

template <class T> int operator==(HaplotypeContainer<T>& hc1, HaplotypeContainer<T>& hc2);
template <class T> int operator!=(HaplotypeContainer<T>& hc1, HaplotypeContainer<T>& hc2);


/*
template <class T> class HaplotypeContainer: public GeneContainerCore{
		//stores copies using operator=; not for unique objects that cannot be copied!
	public:
		T *haplotype_;
		int nloci_;
		bool missing_;//dummy for missing data
	public:
		HaplotypeContainer(void):nloci_(0),haplotype_(NULL), missing_(true){}

		HaplotypeContainer(long IDval, int numLoci):nloci_(numLoci), missing_(false),haplotype_(NULL){
			id_=IDval;
			if(nloci_==0){
				missing_=true; return;}
			haplotype_ = NEW T[nloci_]();
			for(int i=0;i<nloci_;++i){
				haplotype_[i] = 0;}
			}
		HaplotypeContainer(long IDval, int numLoci, T* geneVals):nloci_(numLoci),missing_(false),haplotype_(NULL){
			id_=IDval;
			if(nloci_==0){
				missing_=true; return;}
			haplotype_ = NEW T[nloci_]();
			for(int loc=0;loc<nloci_;++loc){
				haplotype_[loc] = geneVals[loc];}
			}
		HaplotypeContainer(long IDval, aList<T>& geneVals):nloci_(geneVals.len()),missing_(false),haplotype_(NULL){
			id_=IDval;
			if(nloci_==0){
				missing_=true; return;}
			haplotype_ = NEW T[nloci_]();
			for(int loc=0;loc<nloci_;++loc){
				haplotype_[loc] = geneVals[loc];}
			}
		HaplotypeContainer(long IDval, ObjList<T>& geneVals):nloci_(geneVals.len()),missing_(false),haplotype_(NULL){
			id_=IDval;
			if(nloci_==0){
				missing=true; return;}
			haplotype_ = NEW T[nloci_]();
			for(int loc=0;loc<nloci_;++loc){
				haplotype_[loc] = geneVals[loc];}
			}
		HaplotypeContainer(HaplotypeContainer<T>& hc):nloci_(hc.nloci_),haplotype_(NULL),missing_(hc.missing_){
			id_=hc.id_;
			if(nloci_==0){
				missing_=true; return;}
			haplotype_ = NEW T[nloci_]();
			for(int i=0;i<nloci_;++i){
				haplotype_[i] = hc.haplotype_[i];}
			}
		HaplotypeContainer(HaplotypeContainer<T>* hc):nloci_(hc->nloci_),haplotype_(NULL),missing_(hc->missing_){
			id_=hc->id_;
			if(nloci_==0){
				missing_=true; return;}
			haplotype_ = NEW T[nloci_]();
			for(int i=0;i<nloci_;++i){
				haplotype_[i] = hc->haplotype_[i];}
			}
		~HaplotypeContainer(void){
				delete [] haplotype_; haplotype_=NULL;}
		
		virtual GeneContainerCore* Duplicate(void){//virtual copy
					return NEW HaplotypeContainer<T>(*this);}
		virtual long GetPointerToDerivedSelf(void){
					return (long) this;}
		
		virtual void Clear(void){
					GeneContainerCore:Clear();
					nloci_=0;
					delete [] haplotype_; haplotype_=NULL;
					missing_=true;
					}
		
		virtual int EqualQ(GeneContainerCore& gc2){//virtual operator==
					HaplotypeContainer* hc2 = (HaplotypeContainer*) &gc2;
					if(missing_ == hc2->missing_){return GeneContainerCore::EqualQ(gc2);}
					if(nloci_ != hc2->nloci_){return 0;}
					for(int loc=0;loc<nloci_;++loc){
						if(haplotype_[loc] != hc2->haplotype_[loc]){return 0;}}
					return GeneContainerCore::EqualQ(gc2);
					}

		T& haplotype(int loc){
				return haplotype_[loc];}
		
		int loci(void){return nloci_;}
		
		bool missing(void){return missing_;}
		
		int ploidy(void){return 1;}
		
		virtual void Print(ostream& output, char sepChar){
					aString outStr("ID:"); 
					outStr.cat(id_); outStr+sepChar;
					if(missing_){
							outStr+"missing";}
						else{
							for(int loc=0;loc<nloci_-1;++loc){
								outStr.cat(haplotype_[loc]); outStr+sepChar;}
							outStr.cat(haplotype_[nloci_-1]);
							}
					output<<outStr<<endl;
					}
		
		void setHaplotype(int loc, T hapValue){
				missing_=false;
				haplotype_[loc] = hapValue;}
		
		virtual void SetSize(int newNloci){
			if(nloci_ != newNloci){
				nloci_ = newNloci;
				missing_=false;
				delete [] haplotype_;
				haplotype_ = NEW T[nloci_]();
				for(int i=0;i<nloci_;++i){
					haplotype_[i] = 0;}
				}
			}
				
		HaplotypeContainer& operator=(HaplotypeContainer<T>& hc){
			GeneContainerCore::operator=(hc);
			missing_=hc.missing_;
			if(nloci_ != hc.nloci_){
				delete [] haplotype_;haplotype=NULL;
				nloci_ = hc.nloci_;
				if(nloci_==0){
					missing=true; return;}
				haplotype_ = NEW T[nloci_]();
				}
			for(int i=0;i<nloci_;++i){
				haplotype_[i] = hc.haplotype_[i];}
			return *this;
			}
		
		T& operator[](int loc){
				return haplotype_[loc];}
		
		friend int operator==(HaplotypeContainer<T>& hc1, HaplotypeContainer<T>& hc2);
		friend int operator!=(HaplotypeContainer<T>& hc1, HaplotypeContainer<T>& hc2);
		};//end template <class T> class HaplotypeContainer<T>
*/





template <class T> class MultiHaplotypeContainer: public GeneContainerCore{
		//each list item is one haploid copy of the genome
		//haploid genome stores copies using operator=; not for unique objects that cannot be copied!
	public:
		ObjList<HaplotypeContainer<T> > geneSet_;//[chromosome][locus]
		int ploidy_;
	public:
		MultiHaplotypeContainer(void):ploidy_(0){}
		MultiHaplotypeContainer(long IDval):ploidy_(0){
			id_=IDval;
			}
		MultiHaplotypeContainer(long IDval, int ploidyLevel, int numLoci):ploidy_(ploidyLevel){
				int c;
				for(c=0;c<ploidy_;++c){
					geneSet_.Append(NEW HaplotypeContainer<T>(IDval,numLoci));}
				}
		MultiHaplotypeContainer(MultiHaplotypeContainer<T>& mhc):ploidy_(mhc.ploidy_),geneSet_(mhc.geneSet_){
				id_=mhc.ID();}
		MultiHaplotypeContainer(MultiHaplotypeContainer<T>* mhc):ploidy_(mhc->ploidy_),geneSet_(mhc->geneSet_){
				id_=mhc->ID();}
		~MultiHaplotypeContainer(void){
				geneSet_.Clear();}
		
		virtual GeneContainerCore* Duplicate(void){//virtual copy
					return NEW MultiHaplotypeContainer<T>(*this);}
		virtual long GetPointerToDerivedSelf(void){
					return (long) this;}
		
		virtual int EqualQ(GeneContainerCore& gc2){//virtual operator==
					MultiHaplotypeContainer<T>* mhc2 = (MultiHaplotypeContainer<T>*) &gc2;
					if(ploidy_ != mhc2->ploidy_){return 0;}
					if(loci() != mhc2->loci()){return 0;}
					for(int c=0;c<ploidy_;++c){
						for(int loc=0;loc<loci();++loc){
							if(geneSet_[c][loc] != mhc2->geneSet_[c][loc]){return 0;}}}
					return GeneContainerCore::EqualQ(gc2);
					}

		T& haplotype(int loc, int chromosome){
				return geneSet_[chromosome][loc];}
		
		int loci(void){
				if(geneSet_.len()==0){ return 0;}
				return geneSet_[0].loci();}

		bool missing(void){//false if any values are present
				for(int c=0;c<geneSet_.len();++c){
					if(!(geneSet_[c].missing())){
						return false;}}
				return true;
				}
				
		bool missing(int chromosome){
				return geneSet_[chromosome].missing();}
				
		int ploidy(void){return ploidy_;}
		
		void ploidy(int ploidyType){ploidy_ = ploidyType;}
		
		virtual void Print(ostream& output, char sepChar){
					aString outStr("ID:");
					outStr.cat(id_); outStr+sepChar;
					if(missing()){
							outStr+"missing";}
						else{
							for(int loc=0;loc<loci();++loc){
								for(int c=0;c<ploidy_;++c){
									outStr.cat(geneSet_[c][loc]);}
								if(loc < loci()-1) {
									outStr+sepChar;}
								}
							}
					output<<outStr<<endl;
					}

		void setHaplotype(int loc, int chromosome, T hapValue){
				geneSet_[chromosome].setHaplotype(loc,hapValue);}
		
		virtual void SetID(long IDval){
					for(int c=0;c<ploidy_;++c){
						geneSet_[c].SetID(IDval);}
					GeneContainerCore::SetID(IDval);
					}

		void SetMissing(int chromosome){
				geneSet_[chromosome].Clear();}
				
		virtual void SetSize(int newPloidy, int newNloci){
			if(ploidy_ != newPloidy){
				ploidy_ = newPloidy;
				geneSet_.Clear();
				for(int c=0;c<ploidy_;++c){
					geneSet_.Append(NEW HaplotypeContainer<T>(ID(),newNloci));}
				}
			if(newNloci != loci()){
				for(int c=0;c<ploidy_;++c){
					geneSet_[c].SetSize(newNloci);}
				}
				else{}
			}


		MultiHaplotypeContainer<T>& operator=(MultiHaplotypeContainer<T>& mhc){
			GeneContainerCore::operator=(mhc);
			ploidy_=mhc.ploidy_;
			geneSet_=mhc.geneSet_;
			return *this;
			}

		HaplotypeContainer<T>& operator[](int chromosome){
				return geneSet_[chromosome];}

		virtual void Clear(void){
			GeneContainerCore::Clear();
			geneSet_.Clear();
			ploidy_=0;}
		
		friend int operator==(MultiHaplotypeContainer<T>& mhc1, MultiHaplotypeContainer<T>& mhc2);
		friend int operator!=(MultiHaplotypeContainer<T>& mhc1, MultiHaplotypeContainer<T>& mhc2);
		};//end template <class T> class MultiHaplotypeContainer<T>

template <class T> int operator==(MultiHaplotypeContainer<T>& mhc1, MultiHaplotypeContainer<T>& mhc2);
template <class T> int operator!=(MultiHaplotypeContainer<T>& mhc1, MultiHaplotypeContainer<T>& mhc2);






template <class T> class MultiInheritanceModeContainer: public GeneContainerCore{
		//each list item is one inheritance mode:  ordered
				//haploid=0, diploid=1, haplodiploid/sex-linked=2, cytoplasmic female=3, cytoplasmic male=4, other=5)
		//core haploid genome elements store copies using operator=; not for unique objects that cannot be copied!
	public:
		static const int inheritanceModes_ = 6;//in above sequence
		static enum inheritanceMode {haploid=0,diploid=1,sexLink=2,cytoFemale=3,cytoMale=4,otherMode=5};
		
	public:
		ObjList<MultiHaplotypeContainer<T> > inheritanceSet_;//[mode]
		genderSet sex_;
		genderSet heterogameticSex_;//default M is heterogametic
	public:
		MultiInheritanceModeContainer(void):sex_(B),heterogameticSex_(M){//default hermaphrodite
				id_=-1;
				int i;
				for(i=0;i<inheritanceModes_;++i){ 
					inheritanceSet_.Append(NEW MultiHaplotypeContainer<T>());}
				}
		MultiInheritanceModeContainer(long IDval, genderSet gender, genderSet heterogameticSexVal=M):
						sex_(gender),heterogameticSex_(heterogameticSexVal){
				id_=IDval;
				int i;
				for(i=0;i<inheritanceModes_;++i){ 
					inheritanceSet_.Append(NEW MultiHaplotypeContainer<T>());}
				}
		MultiInheritanceModeContainer(aList<int>& lociByType,genderSet sexVal=B,genderSet heterogameticSexVal=M):
					sex_(sexVal),heterogameticSex_(heterogameticSexVal){
				id_=-1;						
				int thisPloidy, i;
				for(i=0;i<inheritanceModes_;++i){ 
					thisPloidy = (i==1 || i==2) ? 2:1;
					if(i==2 && sex_==heterogameticSex_) {thisPloidy=1;}//override
					inheritanceSet_.Append(NEW MultiHaplotypeContainer<T>(id_,thisPloidy,lociByType[i]));}
				}
				
		MultiInheritanceModeContainer(MultiInheritanceModeContainer<T>& mimc):
				sex_(mimc.sex_), inheritanceSet_(mimc.inheritanceSet_),heterogameticSex_(mimc.heterogameticSex_){}
		MultiInheritanceModeContainer(MultiInheritanceModeContainer<T>* mimc):
				sex_(mimc->sex_),inheritanceSet_(mimc->inheritanceSet_),heterogameticSex_(mimc->heterogameticSex_){}
		~MultiInheritanceModeContainer(void){
				inheritanceSet_.Clear();
				}
		
		virtual GeneContainerCore* Duplicate(void){//virtual copy
					return NEW MultiInheritanceModeContainer<T>(*this);}
		virtual long GetPointerToDerivedSelf(void){
					return (long) this;}
		
		virtual void Clear(void){
			GeneContainerCore::Clear();
			int i;
			for(i=0;i<inheritanceModes_;++i){//keep list structure intact, but empty substructural contents
				inheritanceSet_[i].Clear();}
			sex_=B;//default hermaphrodite
			}
		
		virtual void Reset(void){
			GeneContainerCore::Clear();
			int i;
			for(i=0;i<inheritanceModes_;++i){//keep list structure intact, but empty substructural contents
				inheritanceSet_[i].Clear();}
			sex_=B;//default hermaphrodite
			}
		

		bool codominantDataIncluded(void){
				return diploidDataIncluded();}

		MultiHaplotypeContainer<T>& codomData(void){
				return inheritanceSet_[diploid];}
		
		bool cytoFemaleDataIncluded(void){
				return (inheritanceSet_[cytoFemale].loci()>0) ? true:false;}

		MultiHaplotypeContainer<T>& cytoFemaleData(void){
				return inheritanceSet_[cytoFemale];}
		
		bool cytoMaleDataIncluded(void){
				return (inheritanceSet_[cytoMale].loci()>0) ? true:false;}

		MultiHaplotypeContainer<T>& cytoMaleData(void){
				return inheritanceSet_[cytoMale];}
		
		bool diploidDataIncluded(void){
				return (inheritanceSet_[diploid].loci()>0) ? true:false;}

		MultiHaplotypeContainer<T>& diploidData(void){
				return inheritanceSet_[diploid];}
		
		virtual int EqualQ(GeneContainerCore& gc2){//virtual operator==
					MultiInheritanceModeContainer<T>* mimc2 = (MultiInheritanceModeContainer<T>*) &gc2;
					if (sex_ != mimc2->sex_) return 0;
					return inheritanceSet_ == mimc2->inheritanceSet_;
					}

		T& haplotype(int inheritanceType, int loc, int chromosome){
				return inheritanceSet_[inheritanceType][chromosome][loc];}
		
		bool haploidDataIncluded(void){
				return (inheritanceSet_[haploid].loci()>0) ? true:false;}

		MultiHaplotypeContainer<T>& haploidData(void){
				return inheritanceSet_[haploid];}
		
		genderSet heterogameticSex(){
				return heterogameticSex_;}

		char heterogameticSexChar(){
				return (heterogameticSex_==M)? 'M':'F';}

		int inheritanceType(int locus){
				int loc=locus, mode;
				for(mode=0;mode<inheritanceModes_;++mode){
					if(loc<inheritanceSet_[mode].loci()){
							return mode;}
						else{
							loc -= inheritanceSet_[mode].loci();}
					}
				aString err("Error using MultiInheritanceModeContainer<T>:  Locus ");
				err.cat(locus);err.cat(" doesn\'t exist.");
				aError(err);
				return -1;
				}

		bool IsHeterogameticSex(void){
				return sex_ == heterogameticSex_;}
		
		int loci(void){
				int nloci=0, i;
				for(i=0;i<inheritanceModes_;++i){
					nloci += inheritanceSet_[i].loci();}
				return nloci;}

		
		int loci(int dataType){
				return inheritanceSet_[dataType].loci();
				}

		int loci(MultiInheritanceModeContainer<T>::inheritanceMode im){
				int dataType = (int) im;
				return inheritanceSet_[dataType].loci();
				}

		bool missing(int dataType, int chromosome){
				return inheritanceSet_[dataType].missing(chromosome);}
				
		bool missing(MultiInheritanceModeContainer<T>::inheritanceMode im, int chromosome){
				int dataType = (int) im;
				return inheritanceSet_[dataType].missing(chromosome);}
				
		virtual void Print(ostream& output, char sepChar){
					aString outStr("ID:");
					outStr.cat(id_);outStr+sepChar;
					char sexCh; 
					switch(sex_){
						case A:
							sexCh='A'; break;
						case B:
							sexCh='B'; break;
						case F:
							sexCh='F'; break;
						case M:
							sexCh='M'; break;
						default:
							sexCh='B'; break;
						}
					(outStr+sexCh)+sepChar;		
					int mode, loc, c;			
					for(mode=0;mode<inheritanceModes_;++mode){
						if(inheritanceSet_[mode].loci()>0){
							//inheritanceMode {haploid=0,diploid=1,sexLink=2,cytoFemale=3,cytoMale=4,otherMode=5};
							switch((MultiInheritanceModeContainer<T>::inheritanceMode) mode){
								case haploid:
									(outStr+"H:")+sepChar; break;
								case diploid:
									(outStr+"A:")+sepChar; break;
								case sexLink:
									(outStr+"X:")+sepChar; break;
								case cytoFemale:
									(outStr+"F:")+sepChar; break;
								case cytoMale:
									(outStr+"M:")+sepChar; break;
								default: break;
								}}
						for(loc=0;loc<inheritanceSet_[mode].loci();++loc){
							for(c=0;c<inheritanceSet_[mode].ploidy();++c){
								outStr.cat(inheritanceSet_[mode][c][loc]);}
							outStr+sepChar;
							}//loc
						}//mode
					output<<outStr<<endl;
					}

		virtual void SetID(long IDval){
					int i;
					for(i=0;i<inheritanceModes_;++i){
						inheritanceSet_[i].SetID(IDval);}
					GeneContainerCore::SetID(IDval);
					}

		virtual void SetSize(int dataType, int newPloidy, int newNloci){
					MultiInheritanceModeContainer<T>::inheritanceMode im = 
								(MultiInheritanceModeContainer<T>::inheritanceMode) dataType;
					inheritanceSet_[dataType].SetSize(newPloidy,newNloci);
					}
			
		virtual void SetSize(MultiInheritanceModeContainer<T>::inheritanceMode im, int newPloidy, int newNloci){
					int dataType = (int) im;
					inheritanceSet_[dataType].SetSize(newPloidy,newNloci);
					}
			
		void SetHeterogameticSex(genderSet sx){
					heterogameticSex_=sx;}

		void SetHeterogameticSex(char sxChar){
					switch(sxChar){
					//	case 'A': heterogameticSex_=A;break;//impossible
						case 'F': heterogameticSex_=F;break;
						case 'M': heterogameticSex_=M;break;
						default:  heterogameticSex_=B;break;//hermaphroditic by default
						}
					}

		void SetSex(genderSet sx){
				sex_ = sx;
				SetSexLinkPloidy();}
				
		void SetSex(char sx){
			switch (sx){
				case 'M':
					sex_=M; break;
				case 'm':
					sex_=M; break;
				case 'F':
					sex_=F; break;
				case 'f':
					sex_=F; break;
				case 'A':
					sex_=A; break;
				case 'a':
					sex_=A; break;
				case 'B':
					sex_=B; break;
				case 'b':
					sex_=B; break;
				default:
					sex_=B; break;
				}
			SetSexLinkPloidy();
			}
		
		void SetSexLinkPloidy(void){
				int SLloci=	sexLinkedData().loci();
				SetSexLinkLoci(SLloci);
				}
			
		void SetSexLinkLoci(int sexLinkLoci){//changes genome size as well
				if(IsHeterogameticSex()){
						sexLinkedData().SetSize(1,sexLinkLoci);}
					else{
						sexLinkedData().SetSize(2,sexLinkLoci);}
				}
			
		genderSet sex(void){
				return sex_;}
				
		char sexChar(void){
				char sx;
				switch(sex_){
					case A: sx='A';break;
					case M: sx='M';break;
					case F: sx='F';break;
					default: sx='B';break;
					}
				return sx;}
				
		bool sexLinkedDataIncluded(void){
				return (inheritanceSet_[sexLink].loci()>0) ? true:false;}
				
		MultiHaplotypeContainer<T>& sexLinkedData(void){
				return inheritanceSet_[sexLink];}
		
		bool sexDataIncluded(void){
				return (sexLinkedDataIncluded() || cytoFemaleDataIncluded() || cytoMaleDataIncluded());}
				
				
		MultiInheritanceModeContainer<T>& operator=(MultiInheritanceModeContainer<T>& mimc){
			GeneContainerCore::operator=(mimc);
			inheritanceSet_ = mimc.inheritanceSet_;
			return *this;
			}

		MultiHaplotypeContainer<T>& operator[](int dataType){
				return inheritanceSet_[dataType];}

		MultiHaplotypeContainer<T>& operator[](MultiInheritanceModeContainer<T>::inheritanceMode im){
				int dataType = (int) im;
				return inheritanceSet_[dataType];}

		friend int operator==(MultiInheritanceModeContainer<T>& mhc1, MultiInheritanceModeContainer<T>& mhc2);
		friend int operator!=(MultiInheritanceModeContainer<T>& mhc1, MultiInheritanceModeContainer<T>& mhc2);
		};//end template <class T> class MultiInheritanceModeContainer<T>

template <class T> int operator==(MultiInheritanceModeContainer<T>& mhc1, MultiInheritanceModeContainer<T>& mhc2);
template <class T> int operator!=(MultiInheritanceModeContainer<T>& mhc1, MultiInheritanceModeContainer<T>& mhc2);





template <class T> class PopulationGeneContainer{
		//each list item is one inheritance mode:  ordered
				//haploid=0, diploid=1, haplodiploid/sex-linked=3, cytoplasmic female=4, cytoplasmic male=5, other=6)
		//core haploid genome elements store copies using operator=; not for unique objects that cannot be copied!
	public:
		ObjList<MultiInheritanceModeContainer<T> > indivs_;
	
	public:
		PopulationGeneContainer(void){}
		PopulationGeneContainer(int numIndivs, aList<int>& lociByType){
				for(int i=0;i<numIndivs;++i){
					indivs_.Append(NEW MultiInheritanceModeContainer<T>(lociByType));}
				}
		PopulationGeneContainer(PopulationGeneContainer<T>& pgc):indivs_(pgc.indivs_){}
		PopulationGeneContainer(PopulationGeneContainer<T>* pgc):indivs_(pgc->indivs_){}
		~PopulationGeneContainer(void){
				Clear();}

		virtual PopulationGeneContainer<T>* Duplicate(void){//virtual copy constructor
				return NEW PopulationGeneContainer<T>(*this);}

		virtual void Clear(void){
				indivs_.Clear();}
		virtual long GetPointerToDerivedSelf(void){
					return (long) this;}
				
		bool codominantDataIncluded(void){
				if(indivs_.len()==0) return false;
				return indivs_[0].codominantDataIncluded();}

		bool cytoFemaleDataIncluded(void){
				if(indivs_.len()==0) return false;
				return indivs_[0].cytoFemaleDataIncluded();}

		bool cytoMaleDataIncluded(void){
				if(indivs_.len()==0) return false;
				return indivs_[0].cytoMaleDataIncluded();}

		bool diploidDataIncluded(void){
				if(indivs_.len()==0) return false;
				return indivs_[0].diploidDataIncluded();}

		virtual int EqualQ(PopulationGeneContainer<T>& pgc2){//virtual operator==
					return indivs_ == pgc2.indivs_;
					}

		bool haploidDataIncluded(void){
				if(indivs_.len()==0) return false;
				return indivs_[0].haploidDataIncluded();
				}

		bool heterogametic(int indiv){
				if(indivs_.len()==0) return false;
				return indivs_[indiv].heterogametic();
				}

		genderSet heterogameticSex(){
				if(indivs_.len()==0) return M;
				return indivs_[0].heterogameticSex();
				}

		char heterogameticSexChar(){
				if(indivs_.len()==0) return 'M';
				return indivs_[0].heterogameticSexChar();
				}

		int inheritanceType(int locus){
				if(indivs_.len()==0) return 0;
				return indivs_[0].inheritanceType(locus);
				}

		int loci(void){
				if(indivs_.len()==0) return 0;
				return indivs_[0].loci();
				}

		int loci(MultiInheritanceModeContainer<T>::inheritanceMode dataType){
				if(indivs_.len()==0) return 0;
				return indivs_[0].loci(dataType);
				}

		int popSize(void){return indivs_.len();}

		virtual void Print(ostream& output, char sepChar){
					for(int i=0;i<indivs_.len();++i){
						indivs_[i].Print(output,sepChar);}
					}

		void SetHeterogameticSex(genderSet sx){
				for(int i=0;i<indivs_.len();++i){
					indivs_[i].SetHeterogameticSex(sx);}
				}

		void SetHeterogameticSex(char sxChar){
				for(int i=0;i<indivs_.len();++i){
					indivs_[i].SetHeterogameticSex(sxChar);}
				}

		bool sexLinkedDataIncluded(void){
				if(indivs_.len()==0) return false;
				return indivs_[0].sexLinkedDataIncluded();}
				
				
		bool sexDataIncluded(void){
				if(indivs_.len()==0) return false;
				return indivs_[0].sexDataIncluded();}
				
		PopulationGeneContainer<T>& operator=(PopulationGeneContainer<T>& pgc){
				indivs_ = pgc.indivs_;
				return *this;}
		
		MultiInheritanceModeContainer<T>& operator[](int i){
				return indivs_[i];}
		
		friend int operator==(PopulationGeneContainer<T>& pgc1, PopulationGeneContainer<T>& pgc2);
		friend int operator!=(PopulationGeneContainer<T>& pgc1, PopulationGeneContainer<T>& pgc2);
		};//end template <class T> class PopulationGeneContainer<T>

template <class T> int operator==(PopulationGeneContainer<T>& pgc1, PopulationGeneContainer<T>& pgc2);
template <class T> int operator!=(PopulationGeneContainer<T>& pgc1, PopulationGeneContainer<T>& pgc2);





#endif
