//unit FstatisticAnalysisCodom.cpp


#ifndef FstatisticAnalysisCodomcpp
#define FstatisticAnalysisCodomcpp = 1

#include "FstatisticAnalysisCodom.h"
#include "Element.cpp"
#include "aList.cpp"
#include "StatList.cpp"



FstatisticAnalysisCodom::FstatisticAnalysisCodom(FstatisticAnalysisCodom& parentData){
			CodominantMarkerData::Initialize(parentData);
			FstatisticAnalysisCore::Initialize(parentData);
		}//end ctor FstatisticAnalysisCodom(FstatisticAnalysisCodom&)


FstatisticAnalysisCodom::FstatisticAnalysisCodom(CodominantMarkerData& parentData, aList<int>& popSubsetNums) {
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	CodominantMarkerData::Initialize(parentData,popSubsetNums);
	FstatisticAnalysisCore::Initialize();
	}//end ctor FstatisticAnalysisCodom(CodominantMarkerData&, aList<int>&)


FstatisticAnalysisCodom::FstatisticAnalysisCodom(FstatisticAnalysisCodom& parentData, aList<int>& popSubsetNums){
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	CodominantMarkerData::Initialize(parentData,popSubsetNums);
	FstatisticAnalysisCore::Initialize();
	}//end ctor FstatisticAnalysisCodom(FstatisticAnalysisCodom&, aList<int>&)


void  FstatisticAnalysisCodom::Initialize(void){
	CodominantMarkerData::Initialize();
	FstatisticAnalysisCore::Initialize();
	}//Initialize()
	
	
void  FstatisticAnalysisCodom::Initialize(aString& s){
	FstatisticAnalysisCodom::Initialize();
	EvolutionaryGeneticsData::Initialize(s); 
	}//Initialize(aString&)
	
	
void  FstatisticAnalysisCodom::Initialize(FstatisticAnalysisCodom& pgd){
	CodominantMarkerData::Initialize(pgd);
	FstatisticAnalysisCore::Initialize(pgd);
	}//Initialize(FstatisticAnalysisCodom&)
	
	
FstatisticAnalysisCodom&  FstatisticAnalysisCodom::operator=(FstatisticAnalysisCodom& pgd){
	CodominantMarkerData::operator=(pgd);
	FstatisticAnalysisCore::operator=(pgd);
	return *this;
	}//operator=(FstatisticAnalysisCodom&)


void FstatisticAnalysisCodom::ResetFlags(void){
	FstatisticAnalysisCore::ResetFlags();
	CodominantMarkerData::ResetFlags();
	}



void FstatisticAnalysisCodom::CalculateFactorialVarCompCoeffs(GLM& lm /*, int loc*/){
				//calculate variance component coefficients
				//assumes data previously sorted by species, then locality
				long double n___=0.0;//total genes in locus
				long double d__=0.0;//total individuals
				StatList<long double> n_j_;
				for(int locality=0;locality<popsByLocality_.len();++locality){
					n_j_.Append(0.0);
					for(int popL=0;popL<popsByLocality_[locality].len();++popL){
						n_j_[locality] += GetLocusSampleSize(popsByLocality_[locality][popL], 0);
						d__ += popSize_[popsByLocality_[locality][popL] ];
						}//uses first locus, all are the same
					n___+=n_j_[locality];}
				StatList<long double> ni__;
				ObjList<StatList<long double> > nij_;
				ObjList<ObjList<StatList<long double> > > nijk;
				ObjList<StatList<long double> > sum_k_nijkSquared;
				StatList<long double> sum_j_nij_Squared;
				StatList<long double> sum_jk_nijkSquared;
				
				for(int sp0=0;sp0<groupHierarchy_.len();++sp0){
					ni__.Append(0.0);
					nij_.Append(NEW StatList<long double>());
					nijk.Append(NEW ObjList<StatList<long double> >());
					sum_k_nijkSquared.Append(NEW StatList<long double>());
					sum_j_nij_Squared.Append(0.0);
					sum_jk_nijkSquared.Append(0.0);
					for(int L0=0;L0<popsByLocality_.len();++L0){//some localities may be empty for a given species
						nij_[sp0].Append(0.0);
						nijk[sp0].Append(NEW StatList<long double>());
						sum_k_nijkSquared[sp0].Append(0.0);
						}
					}//end for sp0
				long double sum_i_Sumjk_nijkSquared_ni__=0.0;
				long double sum_i_sumj_nijSquared_ni__ = 0.0;
				long double sum_ijk_nijkSquared=0.0;
				long double sum_ij_nijSquared = 0.0;
				long double sum_ni__Squared = 0.0;
				for(int sp1=0;sp1<groupHierarchy_.len();++sp1){
					for(int L1=0;L1<groupHierarchy_[sp1].len();++L1){//some localities may be empty for a given species
						int L = LocalityMembership(groupHierarchy_[sp1][L1]);//handles missing pops correctly
						nij_[sp1][L] += GetLocusSampleSize(groupHierarchy_[sp1][L1], 0);
						for(int ind=0;ind<popSize_[groupHierarchy_[sp1][L1] ];++ind){
							nijk[sp1][L].Append(GetLocusSampleSize(groupHierarchy_[sp1][L1],ind,0));
							sum_k_nijkSquared[sp1][L] += nijk[sp1][L][ind] * nijk[sp1][L][ind];
							}
						sum_j_nij_Squared[sp1] += nij_[sp1][L] *  nij_[sp1][L];
						}
					sum_jk_nijkSquared[sp1] += sum_k_nijkSquared[sp1].sum();
					ni__[sp1] += nij_[sp1].sum();
					sum_i_Sumjk_nijkSquared_ni__ += sum_jk_nijkSquared[sp1] /ni__[sp1] ;
					sum_i_sumj_nijSquared_ni__ += sum_j_nij_Squared[sp1] /ni__[sp1] ;
					sum_ijk_nijkSquared += sum_jk_nijkSquared[sp1];
					sum_ij_nijSquared += sum_j_nij_Squared[sp1];
					sum_ni__Squared += ni__[sp1] * ni__[sp1];
					}
				long double sum_j_n_j_Squared = 0.0;
				for(int L3=0;L3<popsByLocality_.len();++L3){
					sum_j_n_j_Squared += n_j_[L3]*n_j_[L3];}
				aMatrix<long double> C((long double)0.0,popsByLocality_.len(), popsByLocality_.len());
				for(int jr0=0;jr0<C.rows();++jr0){
					for(int jc0=0;jc0<C.cols();++jc0){
						long double sumnijnijprime=0.0;
						for(int sp3=0;sp3<groupHierarchy_.len();++sp3){
							sumnijnijprime += nij_[sp3][jr0]*nij_[sp3][jc0] / ni__[sp3];}
						sumnijnijprime *= (-1.0);
						C[jr0][jc0] = sumnijnijprime;
						if(jr0==jc0){
							C[jr0][jc0] += n_j_[jr0];}
						}}//end for jr0 && jc0
				MatrixList  Dmx, Gmx, Hmx, Umx;
				for(int sp4=0;sp4<groupHierarchy_.len();++sp4){
					Dmx.Append(NEW aMatrix<long double>((long double)0.0,popsByLocality_.len(), popsByLocality_.len()));
					Gmx.Append(NEW aMatrix<long double>((long double)0.0,popsByLocality_.len(), popsByLocality_.len()));
					Hmx.Append(NEW aMatrix<long double>((long double)0.0,popsByLocality_.len(), popsByLocality_.len()));
					Umx.Append(NEW aMatrix<long double>((long double)0.0,popsByLocality_.len(), popsByLocality_.len()));
					for(int jr1=0;jr1<popsByLocality_.len();++jr1){
						for(int jc1=0;jc1<popsByLocality_.len();++jc1){
							if(jr1==jc1){
									Dmx[sp4][jr1][jc1] = nij_[sp4][jr1] * (1.0 - nij_[sp4][jr1] /ni__[sp4]);
									Gmx[sp4][jr1][jc1] = sum_j_nij_Squared[sp4]/ni__[sp4] -2.0*nij_[sp4][jr1] + ni__[sp4];
									Gmx[sp4][jr1][jc1] *=( nij_[sp4][jr1] * nij_[sp4][jr1]) / ni__[sp4];
									Hmx[sp4][jr1][jc1] = (sum_jk_nijkSquared[sp4]/ni__[sp4]) *nij_[sp4][jr1] * nij_[sp4][jr1]
													+ (ni__[sp4] - 2.0*nij_[sp4][jr1]) * sum_k_nijkSquared[sp4][jr1];
									Hmx[sp4][jr1][jc1] /= ni__[sp4];
									Umx[sp4][jr1][jc1] =  (nij_[sp4][jr1] * nij_[sp4][jr1]* sum_jk_nijkSquared[sp4]/ni__[sp4]) 
													+ (ni__[sp4] - 2.0*nij_[sp4][jr1]) * sum_k_nijkSquared[sp4][jr1];
									Umx[sp4][jr1][jc1] /= ni__[sp4];
									}
								else{
									Dmx[sp4][jr1][jc1] = (-1.0)*(nij_[sp4][jr1] *  nij_[sp4][jc1]) /ni__[sp4];
									Gmx[sp4][jr1][jc1] = sum_j_nij_Squared[sp4]/ni__[sp4] -nij_[sp4][jr1] + -nij_[sp4][jc1];
									Gmx[sp4][jr1][jc1] *= ( nij_[sp4][jr1] * nij_[sp4][jc1]) / ni__[sp4];
									Hmx[sp4][jr1][jc1] = (sum_jk_nijkSquared[sp4]/ni__[sp4]) *nij_[sp4][jr1] * nij_[sp4][jc1]
													- nij_[sp4][jr1] * sum_k_nijkSquared[sp4][jc1]
													- nij_[sp4][jc1] * sum_k_nijkSquared[sp4][jr1];
									Hmx[sp4][jr1][jc1] /= ni__[sp4];
									Umx[sp4][jr1][jc1] =  (nij_[sp4][jr1] * nij_[sp4][jc1]* sum_jk_nijkSquared[sp4]/ni__[sp4]) 
													-nij_[sp4][jr1] * sum_k_nijkSquared[sp4][jc1]
													-nij_[sp4][jc1] * sum_k_nijkSquared[sp4][jr1];
									Umx[sp4][jr1][jc1] /= ni__[sp4];
									}
							}}//end for jr1&&jc1
					}//end for sp4
				aMatrix<long double>* invC = C.Inverse();
				aMatrix<long double>* Hsum = Hmx.Sum();
				aMatrix<long double>* invCHsum = invC->Multiply(*Hsum);
				aMatrix<long double>* Gsum = Gmx.Sum();
				aMatrix<long double>* invCGsum = invC->Multiply(*Gsum);
				MatrixList DsformMx, UsformMx;
				for(int sp5=0;sp5<groupHierarchy_.len();++sp5){
					PartitionedMatrix Dpm, Upm;
					DsformMx.Append(Dpm.sForm(Dmx[sp5], sp5+1, groupHierarchy_.len()+1));
					UsformMx.Append(Upm.sForm(Umx[sp5], sp5+1, groupHierarchy_.len()+1));
					}
				aMatrix<long double>* DsFormSum = DsformMx.Sum();
				aMatrix<long double>* UsFormSum = UsformMx.Sum();
				aMatrix<long double>* invDsFormSum = DsFormSum->Inverse();
				aMatrix<long double>* invDsFormSum_UsFormSum = invDsFormSum->Multiply(*UsFormSum);
				
				long double n00,n01,n02,n03,n04,n05,n06,n07,n08,n09,n10,n11,n12,n13,n14;
				 n00=n01=n02=n03=n04=n05=n06=n07=n08=n09=n10=n11=n12=n13=n14=0.0;
				 n00 = n___ - d__;
				 n01 = d__ - (long double) npop_;  
				 //npop_ is the number of populations by localities, accounting for missing populaitons in factorial array
				 n02 = n___ - sum_i_Sumjk_nijkSquared_ni__ - invDsFormSum_UsFormSum->Trace();
				 n03 =  (long double) npop_ - (long double)  groupHierarchy_.len() - (long double)  popsByLocality_.len() + 1.0;
				 n04 = invDsFormSum_UsFormSum->Trace() - invCHsum->Trace();
				 n05 = n___ - sum_i_sumj_nijSquared_ni__ - invCGsum->Trace();
				 n06 = (long double)  popsByLocality_.len() - 1.0;
				 n07 =  invCHsum->Trace();
				 n08 = invCGsum->Trace();
				 n09 = n___ - sum_i_sumj_nijSquared_ni__;
				 n10 = (long double) groupHierarchy_.len() - 1.0;
				 n11 = sum_i_Sumjk_nijkSquared_ni__ - (sum_ijk_nijkSquared / n___);
				 n12 = sum_i_sumj_nijSquared_ni__ - (sum_ij_nijSquared / n___);
				 n13 = sum_i_sumj_nijSquared_ni__ - (sum_j_n_j_Squared / n___);
				 n14 = n___ - (sum_ni__Squared / n___);
				 
				 lm.varCompCoefficients.Clear();
				 aList<long double>* sigmaGCoeffs = NEW aList<long double>();
				 lm.varCompCoefficients.Prepend(sigmaGCoeffs);
				 aList<long double>* sigmaICoeffs = NEW aList<long double>();
				 lm.varCompCoefficients.Prepend(sigmaICoeffs);
				 aList<long double>* sigmaSLCoeffs = NEW aList<long double>();
				 lm.varCompCoefficients.Prepend(sigmaSLCoeffs);
				 aList<long double>* sigmaLCoeffs = NEW aList<long double>();
				 lm.varCompCoefficients.Prepend(sigmaLCoeffs);
				 aList<long double>* sigmaSCoeffs = NEW aList<long double>();
				 lm.varCompCoefficients.Prepend(sigmaSCoeffs);
				 sigmaGCoeffs->Append(n00);
				 sigmaICoeffs->Append(n01);
				 sigmaICoeffs->Append(n02);
				 sigmaSLCoeffs->Append(n04);
				 sigmaSLCoeffs->Append(n03);
				 sigmaSLCoeffs->Append(n05);
				 sigmaLCoeffs->Append(n08);
				 sigmaLCoeffs->Append(n07);
				 sigmaLCoeffs->Append(n06);
				 sigmaLCoeffs->Append(n09);
				 sigmaSCoeffs->Append(n13);
				 sigmaSCoeffs->Append(n12);
				 sigmaSCoeffs->Append(n11);
				 sigmaSCoeffs->Append(n10);
				 sigmaSCoeffs->Append(n14);
				 lm.varCompCoeffsSupplied(1);//true
				 sigmaGCoeffs=sigmaICoeffs=sigmaSLCoeffs=sigmaLCoeffs=sigmaSCoeffs=NULL;//forget these
				 delete invDsFormSum_UsFormSum; invDsFormSum_UsFormSum=NULL;
				 delete invDsFormSum; invDsFormSum=NULL;
				 delete UsFormSum; UsFormSum=NULL;
				 delete DsFormSum; DsFormSum=NULL;
				 delete invCGsum; invCGsum=NULL;
				 delete Gsum; Gsum=NULL;
				 delete invCHsum; invCHsum=NULL;
				 delete Hsum; Hsum=NULL;
				 delete invC; invC=NULL;
				 delete invDsFormSum_UsFormSum; invDsFormSum_UsFormSum=NULL;
	}//end FstatisticAnalysisCodom::CalculateFactorialVarCompCoeffs



void FstatisticAnalysisCodom::CalculateFactorialVarCompCoeffsII(GLM& lm /*, int loc*/){
				//calculate variance component coefficients
				//assumes data previously sorted by species, then locality
				long double n___=0.0;//total genes in locus
				long double d__=0.0;//total individuals
				StatList<long double> n_j_;
				for(int sp00=0;sp00<groupHierarchy_.len();++sp00){
					n_j_.Append(0.0);
					for(int popL=0;popL<groupHierarchy_[sp00].len();++popL){
						n_j_[sp00] += GetLocusSampleSize(groupHierarchy_[sp00][popL], 0);
						d__ += popSize_[groupHierarchy_[sp00][popL] ];
						}//uses first locus, all are the same
					n___+=n_j_[sp00];}
				StatList<long double> ni__;
				ObjList<StatList<long double> > nij_;
				ObjList<ObjList<StatList<long double> > > nijk;
				ObjList<StatList<long double> > sum_k_nijkSquared;
				StatList<long double> sum_j_nij_Squared;
				StatList<long double> sum_jk_nijkSquared;
				
				for(int L0=0;L0<popsByLocality_.len();++L0){
					ni__.Append(0.0);
					nij_.Append(NEW StatList<long double>());
					nijk.Append(NEW ObjList<StatList<long double> >());
					sum_k_nijkSquared.Append(NEW StatList<long double>());
					sum_j_nij_Squared.Append(0.0);
					sum_jk_nijkSquared.Append(0.0);
					for(int sp0=0;sp0<groupHierarchy_.len();++sp0){//some localities may be empty for a given species
						nij_[L0].Append(0.0);
						nijk[L0].Append(NEW StatList<long double>());
						sum_k_nijkSquared[L0].Append(0.0);
						}
					}//end for L0
				long double sum_i_Sumjk_nijkSquared_ni__=0.0;
				long double sum_i_sumj_nijSquared_ni__ = 0.0;
				long double sum_ijk_nijkSquared=0.0;
				long double sum_ij_nijSquared = 0.0;
				long double sum_ni__Squared = 0.0;
				for(int L1=0;L1<popsByLocality_.len();++L1){
					for(int sp1=0;sp1<popsByLocality_[L1].len();++sp1){//some localities may be empty for a given species
						int sp = GroupMembership(popsByLocality_[L1][sp1]);//handles missing pops correctly
						nij_[L1][sp] += GetLocusSampleSize(popsByLocality_[L1][sp1], 0);
						for(int ind=0;ind<popSize_[popsByLocality_[L1][sp1] ];++ind){
							nijk[L1][sp].Append(GetLocusSampleSize(popsByLocality_[L1][sp1],ind,0));
							sum_k_nijkSquared[L1][sp] += nijk[L1][sp][ind] * nijk[L1][sp][ind];
							}
						sum_j_nij_Squared[L1] += nij_[L1][sp] *  nij_[L1][sp];
						}
					sum_jk_nijkSquared[L1] += sum_k_nijkSquared[L1].sum();
					ni__[L1] += nij_[L1].sum();
					sum_i_Sumjk_nijkSquared_ni__ += sum_jk_nijkSquared[L1] /ni__[L1] ;
					sum_i_sumj_nijSquared_ni__ += sum_j_nij_Squared[L1] /ni__[L1] ;
					sum_ijk_nijkSquared += sum_jk_nijkSquared[L1];
					sum_ij_nijSquared += sum_j_nij_Squared[L1];
					sum_ni__Squared += ni__[L1] * ni__[L1];
					}
				long double sum_j_n_j_Squared = 0.0;
				for(int sp3=0;sp3<groupHierarchy_.len();++sp3){
					sum_j_n_j_Squared += n_j_[sp3]*n_j_[sp3];}
				aMatrix<long double> C((long double)0.0,groupHierarchy_.len(), groupHierarchy_.len());
				for(int jr0=0;jr0<C.rows();++jr0){
					for(int jc0=0;jc0<C.cols();++jc0){
						long double sumnijnijprime=0.0;
						for(int L3=0;L3<popsByLocality_.len();++L3){
							sumnijnijprime += nij_[L3][jr0]*nij_[L3][jc0] / ni__[L3];}
						sumnijnijprime *= (-1.0);
						C[jr0][jc0] = sumnijnijprime;
						if(jr0==jc0){
							C[jr0][jc0] += n_j_[jr0];}
						}}//end for jr0 && jc0
				MatrixList  Dmx, Gmx, Hmx, Umx;
				for(int L4=0;L4<popsByLocality_.len();++L4){
					Dmx.Append(NEW aMatrix<long double>((long double)0.0,groupHierarchy_.len(), groupHierarchy_.len()));
					Gmx.Append(NEW aMatrix<long double>((long double)0.0,groupHierarchy_.len(), groupHierarchy_.len()));
					Hmx.Append(NEW aMatrix<long double>((long double)0.0,groupHierarchy_.len(), groupHierarchy_.len()));
					Umx.Append(NEW aMatrix<long double>((long double)0.0,groupHierarchy_.len(), groupHierarchy_.len()));
					for(int jr1=0;jr1<groupHierarchy_.len();++jr1){
						for(int jc1=0;jc1<groupHierarchy_.len();++jc1){
							if(jr1==jc1){
									Dmx[L4][jr1][jc1] = nij_[L4][jr1] * (1.0 - nij_[L4][jr1] /ni__[L4]);
									Gmx[L4][jr1][jc1] = sum_j_nij_Squared[L4]/ni__[L4] -2.0*nij_[L4][jr1] + ni__[L4];
									Gmx[L4][jr1][jc1] *=( nij_[L4][jr1] * nij_[L4][jr1]) / ni__[L4];
									Hmx[L4][jr1][jc1] = (sum_jk_nijkSquared[L4]/ni__[L4]) *nij_[L4][jr1] * nij_[L4][jr1]
													+ (ni__[L4] - 2.0*nij_[L4][jr1]) * sum_k_nijkSquared[L4][jr1];
									Hmx[L4][jr1][jc1] /= ni__[L4];
									Umx[L4][jr1][jc1] =  (nij_[L4][jr1] * nij_[L4][jr1]* sum_jk_nijkSquared[L4]/ni__[L4]) 
													+ (ni__[L4] - 2.0*nij_[L4][jr1]) * sum_k_nijkSquared[L4][jr1];
									Umx[L4][jr1][jc1] /= ni__[L4];
									}
								else{
									Dmx[L4][jr1][jc1] = (-1.0)*(nij_[L4][jr1] *  nij_[L4][jc1]) /ni__[L4];
									Gmx[L4][jr1][jc1] = sum_j_nij_Squared[L4]/ni__[L4] -nij_[L4][jr1] + -nij_[L4][jc1];
									Gmx[L4][jr1][jc1] *= ( nij_[L4][jr1] * nij_[L4][jc1]) / ni__[L4];
									Hmx[L4][jr1][jc1] = (sum_jk_nijkSquared[L4]/ni__[L4]) *nij_[L4][jr1] * nij_[L4][jc1]
													- nij_[L4][jr1] * sum_k_nijkSquared[L4][jc1]
													- nij_[L4][jc1] * sum_k_nijkSquared[L4][jr1];
									Hmx[L4][jr1][jc1] /= ni__[L4];
									Umx[L4][jr1][jc1] =  (nij_[L4][jr1] * nij_[L4][jc1]* sum_jk_nijkSquared[L4]/ni__[L4]) 
													-nij_[L4][jr1] * sum_k_nijkSquared[L4][jc1]
													-nij_[L4][jc1] * sum_k_nijkSquared[L4][jr1];
									Umx[L4][jr1][jc1] /= ni__[L4];
									}
							}}//end for jr1&&jc1
					}//end for sp4
				aMatrix<long double>* invC = C.Inverse();
				aMatrix<long double>* Hsum = Hmx.Sum();
				aMatrix<long double>* invCHsum = invC->Multiply(*Hsum);
				aMatrix<long double>* Gsum = Gmx.Sum();
				aMatrix<long double>* invCGsum = invC->Multiply(*Gsum);
				MatrixList DsformMx, UsformMx;
				for(int L5=0;L5<popsByLocality_.len();++L5){
					PartitionedMatrix Dpm, Upm;
					DsformMx.Append(Dpm.sForm(Dmx[L5], L5+1, popsByLocality_.len()+1));
					UsformMx.Append(Upm.sForm(Umx[L5], L5+1, popsByLocality_.len()+1));
					}
				aMatrix<long double>* DsFormSum = DsformMx.Sum();
				aMatrix<long double>* UsFormSum = UsformMx.Sum();
				aMatrix<long double>* invDsFormSum = DsFormSum->Inverse();
				aMatrix<long double>* invDsFormSum_UsFormSum = invDsFormSum->Multiply(*UsFormSum);
				
				long double n00,n01,n02,n03,n04,n05,n06,n07,n08,n09,n10,n11,n12,n13,n14;
				 n00=n01=n02=n03=n04=n05=n06=n07=n08=n09=n10=n11=n12=n13=n14=0.0;
				 n00 = n___ - d__;
				 n01 = d__ - (long double) npop_;  
				 //npop_ is the number of populations by localities, accounting for missing populations in factorial array
				 n02 = n___ - sum_i_Sumjk_nijkSquared_ni__ - invDsFormSum_UsFormSum->Trace();
				 n03 =  (long double) npop_ - (long double)  groupHierarchy_.len() - (long double)  popsByLocality_.len() + 1.0;
				 n04 = invDsFormSum_UsFormSum->Trace() - invCHsum->Trace();
				 n05 = n___ - sum_i_sumj_nijSquared_ni__ - invCGsum->Trace();
				 n06 = (long double)  popsByLocality_.len() - 1.0;
				 n07 =  invCHsum->Trace();
				 n08 = invCGsum->Trace();
				 n09 = n___ - sum_i_sumj_nijSquared_ni__;
				 n10 = (long double) groupHierarchy_.len() - 1.0;
				 n11 = sum_i_Sumjk_nijkSquared_ni__ - (sum_ijk_nijkSquared / n___);
				 n12 = sum_i_sumj_nijSquared_ni__ - (sum_ij_nijSquared / n___);
				 n13 = sum_i_sumj_nijSquared_ni__ - (sum_j_n_j_Squared / n___);
				 n14 = n___ - (sum_ni__Squared / n___);
				 
				 lm.varCompCoefficients.Clear();
				 aList<long double>* sigmaGCoeffs = NEW aList<long double>();
				 lm.varCompCoefficients.Prepend(sigmaGCoeffs);
				 aList<long double>* sigmaICoeffs = NEW aList<long double>();
				 lm.varCompCoefficients.Prepend(sigmaICoeffs);
				 aList<long double>* sigmaSLCoeffs = NEW aList<long double>();
				 lm.varCompCoefficients.Prepend(sigmaSLCoeffs);
				 aList<long double>* sigmaLCoeffs = NEW aList<long double>();
				 lm.varCompCoefficients.Prepend(sigmaLCoeffs);
				 aList<long double>* sigmaSCoeffs = NEW aList<long double>();
				 lm.varCompCoefficients.Prepend(sigmaSCoeffs);
				 sigmaGCoeffs->Append(n00);
				 sigmaICoeffs->Append(n01);
				 sigmaICoeffs->Append(n02);
				 sigmaSLCoeffs->Append(n04);
				 sigmaSLCoeffs->Append(n03);
				 sigmaSLCoeffs->Append(n05);
				 sigmaLCoeffs->Append(n08);
				 sigmaLCoeffs->Append(n07);
				 sigmaLCoeffs->Append(n06);
				 sigmaLCoeffs->Append(n09);
				 sigmaSCoeffs->Append(n13);
				 sigmaSCoeffs->Append(n12);
				 sigmaSCoeffs->Append(n11);
				 sigmaSCoeffs->Append(n10);
				 sigmaSCoeffs->Append(n14);
				 lm.varCompCoeffsSupplied(1);//true
				 sigmaGCoeffs=sigmaICoeffs=sigmaSLCoeffs=sigmaLCoeffs=sigmaSCoeffs=NULL;//forget these
				 delete invDsFormSum_UsFormSum; invDsFormSum_UsFormSum=NULL;
				 delete invDsFormSum; invDsFormSum=NULL;
				 delete UsFormSum; UsFormSum=NULL;
				 delete DsFormSum; DsFormSum=NULL;
				 delete invCGsum; invCGsum=NULL;
				 delete Gsum; Gsum=NULL;
				 delete invCHsum; invCHsum=NULL;
				 delete Hsum; Hsum=NULL;
				 delete invC; invC=NULL;
				 delete invDsFormSum_UsFormSum; invDsFormSum_UsFormSum=NULL;
	}//end FstatisticAnalysisCodom::CalculateFactorialVarCompCoeffsII



void FstatisticAnalysisCodom::CalculateFactorialVarCompCoeffsIII(aList<long double>& nList, aList<long double>& dfList /*, int loc*/){
	//calculates variance component coefficients for 3 estimation methods at once
	//assumes data previously sorted by species, then locality
	long double n___=0.0;//total genes in locus
	long double d__=0.0;//total individuals
	StatList<long double> n_j_;
	for(int locality=0;locality<popsByLocality_.len();++locality){
		n_j_.Append(0.0);
		for(int popL=0;popL<popsByLocality_[locality].len();++popL){
			n_j_[locality] += GetLocusSampleSize(popsByLocality_[locality][popL], 0);
			d__ += popSize_[popsByLocality_[locality][popL] ];
			}//uses first locus, all are the same
		n___+=n_j_[locality];}
	StatList<long double> ni__;
	ObjList<StatList<long double> > nij_;
	ObjList<ObjList<StatList<long double> > > nijk;
	ObjList<StatList<long double> > sum_k_nijkSquared;
	StatList<long double> sum_j_nij_Squared, sum_i_nij_Squared;
	StatList<long double> sum_jk_nijkSquared, sum_ik_nijkSquared;
				
	for(int sp0=0;sp0<groupHierarchy_.len();++sp0){
		ni__.Append(0.0);
		nij_.Append(NEW StatList<long double>());
		nijk.Append(NEW ObjList<StatList<long double> >());
		sum_k_nijkSquared.Append(NEW StatList<long double>());
		sum_j_nij_Squared.Append(0.0);
		sum_jk_nijkSquared.Append(0.0);
		for(int L0=0;L0<popsByLocality_.len();++L0){//some localities may be empty for a given species
			nij_[sp0].Append(0.0);
			nijk[sp0].Append(NEW StatList<long double>());
			sum_k_nijkSquared[sp0].Append(0.0);
			}
		}//end for sp0
	for(int L00=0;L00<popsByLocality_.len();++L00){
		sum_i_nij_Squared.Append(0.0);
		sum_ik_nijkSquared.Append(0.0);
		}//end for L00
	long double sum_i_Sumjk_nijkSquared_ni__=0.0;
	long double sum_i_sumj_nijSquared_ni__ = 0.0;
	long double sum_j_Sumik_nijkSquared_n_j_=0.0;
	long double sum_j_sumi_nijSquared_n_j_ = 0.0;
	long double sum_ijk_nijkSquared=0.0;
	long double sum_ij_nijSquared = 0.0;
	long double sum_ni__Squared = 0.0;
	for(int sp1=0;sp1<groupHierarchy_.len();++sp1){
		for(int L1=0;L1<groupHierarchy_[sp1].len();++L1){//some localities may be empty for a given species
			int L = LocalityMembership(groupHierarchy_[sp1][L1]);//handles missing pops correctly
			nij_[sp1][L] += GetLocusSampleSize(groupHierarchy_[sp1][L1], 0);
			for(int ind=0;ind<popSize_[groupHierarchy_[sp1][L1] ];++ind){
				nijk[sp1][L].Append(GetLocusSampleSize(groupHierarchy_[sp1][L1],ind,0));
				sum_k_nijkSquared[sp1][L] += nijk[sp1][L][ind] * nijk[sp1][L][ind];
				}
			sum_j_nij_Squared[sp1] += nij_[sp1][L] *  nij_[sp1][L];
			}
		sum_jk_nijkSquared[sp1] += sum_k_nijkSquared[sp1].sum();
		ni__[sp1] += nij_[sp1].sum();
		sum_i_Sumjk_nijkSquared_ni__ += sum_jk_nijkSquared[sp1] /ni__[sp1] ;
		sum_i_sumj_nijSquared_ni__ += sum_j_nij_Squared[sp1] /ni__[sp1] ;
		sum_ijk_nijkSquared += sum_jk_nijkSquared[sp1];
		sum_ij_nijSquared += sum_j_nij_Squared[sp1];
		sum_ni__Squared += ni__[sp1] * ni__[sp1];
		}
	long double sum_j_n_j_Squared = 0.0;
	for(int L3=0;L3<popsByLocality_.len();++L3){
		for(int sp1=0;sp1<groupHierarchy_.len();++sp1){
			sum_i_nij_Squared[L3] += nij_[sp1][L3] *  nij_[sp1][L3];
			sum_ik_nijkSquared[L3] += sum_k_nijkSquared[sp1][L3];
			}
		sum_j_Sumik_nijkSquared_n_j_ += sum_ik_nijkSquared[L3] /n_j_[L3] ;
		sum_j_sumi_nijSquared_n_j_ += sum_i_nij_Squared[L3] /n_j_[L3] ;
		sum_j_n_j_Squared += n_j_[L3]*n_j_[L3];
		}
	aMatrix<long double> C((long double) 0.0,popsByLocality_.len(), popsByLocality_.len());
	for(int jr0=0;jr0<C.rows();++jr0){
		for(int jc0=0;jc0<C.cols();++jc0){
			long double sumnijnijprime=0.0;
			for(int sp3=0;sp3<groupHierarchy_.len();++sp3){
				sumnijnijprime += nij_[sp3][jr0]*nij_[sp3][jc0] / ni__[sp3];}
			sumnijnijprime *= (-1.0);
			C[jr0][jc0] = sumnijnijprime;
			if(jr0==jc0){
				C[jr0][jc0] += n_j_[jr0];}
			}}//end for jr0 && jc0
	MatrixList  Dmx, Gmx, Hmx, Umx;
	for(int sp4=0;sp4<groupHierarchy_.len();++sp4){
		Dmx.Append(NEW aMatrix<long double>((long double) 0.0,popsByLocality_.len(), popsByLocality_.len()));
		Gmx.Append(NEW aMatrix<long double>((long double) 0.0,popsByLocality_.len(), popsByLocality_.len()));
		Hmx.Append(NEW aMatrix<long double>((long double) 0.0,popsByLocality_.len(), popsByLocality_.len()));
		Umx.Append(NEW aMatrix<long double>((long double) 0.0,popsByLocality_.len(), popsByLocality_.len()));
		for(int jr1=0;jr1<popsByLocality_.len();++jr1){
			for(int jc1=0;jc1<popsByLocality_.len();++jc1){
				if(jr1==jc1){
						Dmx[sp4][jr1][jc1] = nij_[sp4][jr1] * (1.0 - nij_[sp4][jr1] /ni__[sp4]);
						Gmx[sp4][jr1][jc1] = sum_j_nij_Squared[sp4]/ni__[sp4] -2.0*nij_[sp4][jr1] + ni__[sp4];
						Gmx[sp4][jr1][jc1] *=( nij_[sp4][jr1] * nij_[sp4][jr1]) / ni__[sp4];
						Hmx[sp4][jr1][jc1] = (sum_jk_nijkSquared[sp4]/ni__[sp4]) *nij_[sp4][jr1] * nij_[sp4][jr1]
										+ (ni__[sp4] - 2.0*nij_[sp4][jr1]) * sum_k_nijkSquared[sp4][jr1];
						Hmx[sp4][jr1][jc1] /= ni__[sp4];
						Umx[sp4][jr1][jc1] =  (nij_[sp4][jr1] * nij_[sp4][jr1]* sum_jk_nijkSquared[sp4]/ni__[sp4]) 
										+ (ni__[sp4] - 2.0*nij_[sp4][jr1]) * sum_k_nijkSquared[sp4][jr1];
						Umx[sp4][jr1][jc1] /= ni__[sp4];
						}
					else{
						Dmx[sp4][jr1][jc1] = (-1.0)*(nij_[sp4][jr1] *  nij_[sp4][jc1]) /ni__[sp4];
						Gmx[sp4][jr1][jc1] = sum_j_nij_Squared[sp4]/ni__[sp4] -nij_[sp4][jr1] + -nij_[sp4][jc1];
						Gmx[sp4][jr1][jc1] *= ( nij_[sp4][jr1] * nij_[sp4][jc1]) / ni__[sp4];
						Hmx[sp4][jr1][jc1] = (sum_jk_nijkSquared[sp4]/ni__[sp4]) *nij_[sp4][jr1] * nij_[sp4][jc1]
										- nij_[sp4][jr1] * sum_k_nijkSquared[sp4][jc1]
										- nij_[sp4][jc1] * sum_k_nijkSquared[sp4][jr1];
						Hmx[sp4][jr1][jc1] /= ni__[sp4];
						Umx[sp4][jr1][jc1] =  (nij_[sp4][jr1] * nij_[sp4][jc1]* sum_jk_nijkSquared[sp4]/ni__[sp4]) 
										-nij_[sp4][jr1] * sum_k_nijkSquared[sp4][jc1]
										-nij_[sp4][jc1] * sum_k_nijkSquared[sp4][jr1];
						Umx[sp4][jr1][jc1] /= ni__[sp4];
						}
				}}//end for jr1&&jc1
		}//end for sp4
	aMatrix<long double>* invC = C.Inverse();
	aMatrix<long double>* Hsum = Hmx.Sum();
	aMatrix<long double>* invCHsum = invC->Multiply(*Hsum);
	aMatrix<long double>* Gsum = Gmx.Sum();
	aMatrix<long double>* invCGsum = invC->Multiply(*Gsum);
	MatrixList DsformMx, UsformMx;
	for(int sp5=0;sp5<groupHierarchy_.len();++sp5){
		PartitionedMatrix Dpm, Upm;
		DsformMx.Append(Dpm.sForm(Dmx[sp5], sp5+1, groupHierarchy_.len()+1));
		UsformMx.Append(Upm.sForm(Umx[sp5], sp5+1, groupHierarchy_.len()+1));
		}
	aMatrix<long double>* DsFormSum = DsformMx.Sum();
	aMatrix<long double>* UsFormSum = UsformMx.Sum();
	aMatrix<long double>* invDsFormSum = DsFormSum->Inverse();
	aMatrix<long double>* invDsFormSum_UsFormSum = invDsFormSum->Multiply(*UsFormSum);

//check this section for order of ij
	aMatrix<long double> CC((long double) 0.0,groupHierarchy_.len(), groupHierarchy_.len());
	for(int jr0=0;jr0<CC.rows();++jr0){
		for(int jc0=0;jc0<CC.cols();++jc0){
			long double sumnijnijprime=0.0;
			for(int L3=0;L3<popsByLocality_.len();++L3){
				sumnijnijprime += nij_[jr0][L3]*nij_[jc0][L3] / n_j_[L3];}
			sumnijnijprime *= (-1.0);
			CC[jr0][jc0] = sumnijnijprime;
			if(jr0==jc0){
				CC[jr0][jc0] += ni__[jr0];}
			}}//end for jr0 && jc0
	Dmx.Clear(); Gmx.Clear();  Hmx.Clear();  Umx.Clear(); //reuse these
	for(int L4=0;L4<popsByLocality_.len();++L4){
		Gmx.Append(NEW aMatrix<long double>((long double) 0.0,groupHierarchy_.len(), groupHierarchy_.len()));
		Hmx.Append(NEW aMatrix<long double>((long double) 0.0,groupHierarchy_.len(), groupHierarchy_.len()));
		for(int jr1=0;jr1<groupHierarchy_.len();++jr1){
			for(int jc1=0;jc1<groupHierarchy_.len();++jc1){
				if(jr1==jc1){
						Gmx[L4][jr1][jc1] = sum_i_nij_Squared[L4]/n_j_[L4] -2.0*nij_[jr1][L4] + n_j_[L4];
						Gmx[L4][jr1][jc1] *=( nij_[jr1][L4] * nij_[jr1][L4]) / n_j_[L4];
						Hmx[L4][jr1][jc1] = (sum_ik_nijkSquared[L4]/n_j_[L4]) *nij_[jr1][L4] * nij_[jr1][L4]
										+ (n_j_[L4] - 2.0*nij_[jr1][L4]) * sum_k_nijkSquared[jr1][L4];
						Hmx[L4][jr1][jc1] /= n_j_[L4];
						}
					else{
						Gmx[L4][jr1][jc1] = sum_i_nij_Squared[L4]/n_j_[L4] - nij_[jr1][L4]  - nij_[jc1][L4];
						Gmx[L4][jr1][jc1] *= ( nij_[jr1][L4] * nij_[jc1][L4]) / n_j_[L4];
						Hmx[L4][jr1][jc1] = (sum_ik_nijkSquared[L4]/n_j_[L4]) *nij_[jr1][L4] * nij_[jc1][L4]
										- nij_[jr1][L4] * sum_k_nijkSquared[jc1][L4]
										- nij_[jc1][L4] * sum_k_nijkSquared[jr1][L4];
						Hmx[L4][jr1][jc1] /= n_j_[L4];
						}
				}}//end for jr1&&jc1
		}//end for L4
	aMatrix<long double>* invCC = CC.Inverse();
	aMatrix<long double>* HHsum = Hmx.Sum();
	aMatrix<long double>* invCCHHsum = invCC->Multiply(*HHsum);
	aMatrix<long double>* GGsum = Gmx.Sum();
	aMatrix<long double>* invCCGGsum = invCC->Multiply(*GGsum);
				
	long double n01,n02,n03,n04,n05,n06,n07,n08,n09,n10,n11,n12,n13,n14,n15,n16,n17;
	 n01=n02=n03=n04=n05=n06=n07=n08=n09=n10=n11=n12=n13=n14=n15=n16=n17=0.0;
	 //npop_ is the number of populations by localities, accounting for missing populations in factorial array
	nList.Clear();
	for(int i=0;i<17;++i){nList.Append(0.0);}
	nList[0] = n01 = n___ - sum_i_Sumjk_nijkSquared_ni__ - invDsFormSum_UsFormSum->Trace();
	nList[1] = n02 = invDsFormSum_UsFormSum->Trace() - invCHsum->Trace();
	nList[2] = n03 = n___ - sum_i_sumj_nijSquared_ni__ - invCGsum->Trace();
	nList[3] = n04 =  invCHsum->Trace();
	nList[4] = n05 = invCGsum->Trace();
	nList[5] = n06 = n___ - sum_i_sumj_nijSquared_ni__;
	nList[6] = n07 = sum_i_Sumjk_nijkSquared_ni__ - (sum_ijk_nijkSquared / n___);
	nList[7] = n08 = sum_i_sumj_nijSquared_ni__ - (sum_ij_nijSquared / n___);
	nList[8] = n09 = sum_i_sumj_nijSquared_ni__ - (sum_j_n_j_Squared / n___);
	nList[9] = n10 = n___ - (sum_ni__Squared / n___);
	nList[10] = n11 =  invCCHHsum->Trace();
	nList[11] = n12 = invCCGGsum->Trace();
	nList[12] = n13 = n___ - sum_j_sumi_nijSquared_n_j_;
	nList[13] = n14 = sum_j_Sumik_nijkSquared_n_j_  - (sum_ijk_nijkSquared / n___);
	nList[14] = n15 = sum_j_sumi_nijSquared_n_j_ - (sum_ij_nijSquared / n___);
	nList[15] = n16 = sum_j_sumi_nijSquared_n_j_ - (sum_ni__Squared / n___);
	nList[16] = n17 = n___ - (sum_j_n_j_Squared / n___);
	dfList.Clear();
	for(int i=0;i<=5;++i){dfList.Append(0.0);}
	dfList[0] =(long double) ngroups_ - 1.0;//species
	dfList[1] =(long double) popsByLocality_.len() - 1.0;//localities
	dfList[2] = (long double) npop_ - (long double) ngroups_ - (long double) popsByLocality_.len() + 1.0;//populations
	dfList[3] = d__ - (long double) npop_;//individuals
	dfList[4] = n___ - d__;//genes
	dfList[5] = n___ - 1.0;//total
	 delete invDsFormSum_UsFormSum; invDsFormSum_UsFormSum=NULL;
	 delete invDsFormSum; invDsFormSum=NULL;
	 delete UsFormSum; UsFormSum=NULL;
	 delete DsFormSum; DsFormSum=NULL;
	 delete invCGsum; invCGsum=NULL;
	 delete Gsum; Gsum=NULL;
	 delete invCHsum; invCHsum=NULL;
	 delete Hsum; Hsum=NULL;
	 delete invC; invC=NULL;
	 delete invCCGGsum; invCCGGsum=NULL;
	 delete GGsum; GGsum=NULL;
	 delete invCCHHsum; invCCHHsum=NULL;
	 delete HHsum; HHsum=NULL;
	 delete invCC; invCC=NULL;
	 delete invDsFormSum_UsFormSum; invDsFormSum_UsFormSum=NULL;
	}//end FstatisticAnalysisCodom::CalculateFactorialVarCompCoeffsIII


void FstatisticAnalysisCodom::CalculateFactorialVarCompCoeffsIII(aList<long double>& nList, 
													aList<long double>& dfList, int loc){
	//calculates variance component coefficients for 3 estimation methods at once
	//assumes data previously sorted by species, then locality
	long double n___=0.0;//total genes sampled in locus
	long double d__=0.0;//total individuals sampled at this locus
	StatList<long double> n_j_;
	for(int locality=0;locality<popsByLocality_.len();++locality){
		n_j_.Append(0.0);
		for(int popL=0;popL<popsByLocality_[locality].len();++popL){
			n_j_[locality] += GetLocusSampleSize(popsByLocality_[locality][popL], loc);
			d__ +=  GetIndividualsSampled(popsByLocality_[locality][popL], loc);
			}//uses first locus, all are the same
		n___+=n_j_[locality];}
	StatList<long double> ni__;
	ObjList<StatList<long double> > nij_;
	ObjList<ObjList<StatList<long double> > > nijk;
	ObjList<StatList<long double> > sum_k_nijkSquared;
	StatList<long double> sum_j_nij_Squared, sum_i_nij_Squared;
	StatList<long double> sum_jk_nijkSquared, sum_ik_nijkSquared;
				
	for(int sp0=0;sp0<groupHierarchy_.len();++sp0){
		ni__.Append(0.0);
		nij_.Append(NEW StatList<long double>());
		nijk.Append(NEW ObjList<StatList<long double> >());
		sum_k_nijkSquared.Append(NEW StatList<long double>());
		sum_j_nij_Squared.Append(0.0);
		sum_jk_nijkSquared.Append(0.0);
		for(int L0=0;L0<popsByLocality_.len();++L0){//some localities may be empty for a given species
			nij_[sp0].Append(0.0);
			nijk[sp0].Append(NEW StatList<long double>());
			sum_k_nijkSquared[sp0].Append(0.0);
			}
		}//end for sp0
	for(int L00=0;L00<popsByLocality_.len();++L00){
		sum_i_nij_Squared.Append(0.0);
		sum_ik_nijkSquared.Append(0.0);
		}//end for L00
	long double sum_i_Sumjk_nijkSquared_ni__=0.0;
	long double sum_i_sumj_nijSquared_ni__ = 0.0;
	long double sum_j_Sumik_nijkSquared_n_j_=0.0;
	long double sum_j_sumi_nijSquared_n_j_ = 0.0;
	long double sum_ijk_nijkSquared=0.0;
	long double sum_ij_nijSquared = 0.0;
	long double sum_ni__Squared = 0.0;
	for(int sp1=0;sp1<groupHierarchy_.len();++sp1){
		for(int L1=0;L1<groupHierarchy_[sp1].len();++L1){//some localities may be empty for a given species
			int L = LocalityMembership(groupHierarchy_[sp1][L1]);//handles missing pops correctly
			nij_[sp1][L] += GetLocusSampleSize(groupHierarchy_[sp1][L1], loc);
			for(int ind=0;ind<popSize_[groupHierarchy_[sp1][L1] ];++ind){
				int ploidy = GetLocusSampleSize(groupHierarchy_[sp1][L1],ind,loc);
				if(ploidy>0){//skips missing data
					nijk[sp1][L].Append(ploidy);
					sum_k_nijkSquared[sp1][L] += nijk[sp1][L].Last() * nijk[sp1][L].Last();
					}
				}
			sum_j_nij_Squared[sp1] += nij_[sp1][L] *  nij_[sp1][L];
			}
		sum_jk_nijkSquared[sp1] += sum_k_nijkSquared[sp1].sum();
		ni__[sp1] += nij_[sp1].sum();
		sum_i_Sumjk_nijkSquared_ni__ += sum_jk_nijkSquared[sp1] /ni__[sp1] ;
		sum_i_sumj_nijSquared_ni__ += sum_j_nij_Squared[sp1] /ni__[sp1] ;
		sum_ijk_nijkSquared += sum_jk_nijkSquared[sp1];
		sum_ij_nijSquared += sum_j_nij_Squared[sp1];
		sum_ni__Squared += ni__[sp1] * ni__[sp1];
		}
	long double sum_j_n_j_Squared = 0.0;
	for(int L3=0;L3<popsByLocality_.len();++L3){
		for(int sp1=0;sp1<groupHierarchy_.len();++sp1){
			sum_i_nij_Squared[L3] += nij_[sp1][L3] *  nij_[sp1][L3];
			sum_ik_nijkSquared[L3] += sum_k_nijkSquared[sp1][L3];
			}
		sum_j_Sumik_nijkSquared_n_j_ += sum_ik_nijkSquared[L3] /n_j_[L3] ;
		sum_j_sumi_nijSquared_n_j_ += sum_i_nij_Squared[L3] /n_j_[L3] ;
		sum_j_n_j_Squared += n_j_[L3]*n_j_[L3];
		}
	aMatrix<long double> C((long double)0.0,popsByLocality_.len(), popsByLocality_.len());
	for(int jr0=0;jr0<C.rows();++jr0){
		for(int jc0=0;jc0<C.cols();++jc0){
			long double sumnijnijprime=0.0;
			for(int sp3=0;sp3<groupHierarchy_.len();++sp3){
				sumnijnijprime += nij_[sp3][jr0]*nij_[sp3][jc0] / ni__[sp3];}
			sumnijnijprime *= (-1.0);
			C[jr0][jc0] = sumnijnijprime;
			if(jr0==jc0){
				C[jr0][jc0] += n_j_[jr0];}
			}}//end for jr0 && jc0
	MatrixList  Dmx, Gmx, Hmx, Umx;
	for(int sp4=0;sp4<groupHierarchy_.len();++sp4){
		Dmx.Append(NEW aMatrix<long double>((long double)0.0,popsByLocality_.len(), popsByLocality_.len()));
		Gmx.Append(NEW aMatrix<long double>((long double)0.0,popsByLocality_.len(), popsByLocality_.len()));
		Hmx.Append(NEW aMatrix<long double>((long double)0.0,popsByLocality_.len(), popsByLocality_.len()));
		Umx.Append(NEW aMatrix<long double>((long double)0.0,popsByLocality_.len(), popsByLocality_.len()));
		for(int jr1=0;jr1<popsByLocality_.len();++jr1){
			for(int jc1=0;jc1<popsByLocality_.len();++jc1){
				if(jr1==jc1){
						Dmx[sp4][jr1][jc1] = nij_[sp4][jr1] * (1.0 - nij_[sp4][jr1] /ni__[sp4]);
						Gmx[sp4][jr1][jc1] = sum_j_nij_Squared[sp4]/ni__[sp4] -2.0*nij_[sp4][jr1] + ni__[sp4];
						Gmx[sp4][jr1][jc1] *=( nij_[sp4][jr1] * nij_[sp4][jr1]) / ni__[sp4];
						Hmx[sp4][jr1][jc1] = (sum_jk_nijkSquared[sp4]/ni__[sp4]) *nij_[sp4][jr1] * nij_[sp4][jr1]
										+ (ni__[sp4] - 2.0*nij_[sp4][jr1]) * sum_k_nijkSquared[sp4][jr1];
						Hmx[sp4][jr1][jc1] /= ni__[sp4];
						Umx[sp4][jr1][jc1] =  (nij_[sp4][jr1] * nij_[sp4][jr1]* sum_jk_nijkSquared[sp4]/ni__[sp4]) 
										+ (ni__[sp4] - 2.0*nij_[sp4][jr1]) * sum_k_nijkSquared[sp4][jr1];
						Umx[sp4][jr1][jc1] /= ni__[sp4];
						}
					else{
						Dmx[sp4][jr1][jc1] = (-1.0)*(nij_[sp4][jr1] *  nij_[sp4][jc1]) /ni__[sp4];
						Gmx[sp4][jr1][jc1] = sum_j_nij_Squared[sp4]/ni__[sp4] -nij_[sp4][jr1] + -nij_[sp4][jc1];
						Gmx[sp4][jr1][jc1] *= ( nij_[sp4][jr1] * nij_[sp4][jc1]) / ni__[sp4];
						Hmx[sp4][jr1][jc1] = (sum_jk_nijkSquared[sp4]/ni__[sp4]) *nij_[sp4][jr1] * nij_[sp4][jc1]
										- nij_[sp4][jr1] * sum_k_nijkSquared[sp4][jc1]
										- nij_[sp4][jc1] * sum_k_nijkSquared[sp4][jr1];
						Hmx[sp4][jr1][jc1] /= ni__[sp4];
						Umx[sp4][jr1][jc1] =  (nij_[sp4][jr1] * nij_[sp4][jc1]* sum_jk_nijkSquared[sp4]/ni__[sp4]) 
										-nij_[sp4][jr1] * sum_k_nijkSquared[sp4][jc1]
										-nij_[sp4][jc1] * sum_k_nijkSquared[sp4][jr1];
						Umx[sp4][jr1][jc1] /= ni__[sp4];
						}
				}}//end for jr1&&jc1
		}//end for sp4
	aMatrix<long double>* invC = C.Inverse();
	aMatrix<long double>* Hsum = Hmx.Sum();
	aMatrix<long double>* invCHsum = invC->Multiply(*Hsum);
	aMatrix<long double>* Gsum = Gmx.Sum();
	aMatrix<long double>* invCGsum = invC->Multiply(*Gsum);
	MatrixList DsformMx, UsformMx;
	for(int sp5=0;sp5<groupHierarchy_.len();++sp5){
		PartitionedMatrix Dpm, Upm;
		DsformMx.Append(Dpm.sForm(Dmx[sp5], sp5+1, groupHierarchy_.len()+1));
		UsformMx.Append(Upm.sForm(Umx[sp5], sp5+1, groupHierarchy_.len()+1));
		}
	aMatrix<long double>* DsFormSum = DsformMx.Sum();
	aMatrix<long double>* UsFormSum = UsformMx.Sum();
	aMatrix<long double>* invDsFormSum = DsFormSum->Inverse();
	aMatrix<long double>* invDsFormSum_UsFormSum = invDsFormSum->Multiply(*UsFormSum);

//check this section for order of ij
	aMatrix<long double> CC((long double)0.0,groupHierarchy_.len(), groupHierarchy_.len());
	for(int jr0=0;jr0<CC.rows();++jr0){
		for(int jc0=0;jc0<CC.cols();++jc0){
			long double sumnijnijprime=0.0;
			for(int L3=0;L3<popsByLocality_.len();++L3){
				sumnijnijprime += nij_[jr0][L3]*nij_[jc0][L3] / n_j_[L3];}
			sumnijnijprime *= (-1.0);
			CC[jr0][jc0] = sumnijnijprime;
			if(jr0==jc0){
				CC[jr0][jc0] += ni__[jr0];}
			}}//end for jr0 && jc0
	Dmx.Clear(); Gmx.Clear();  Hmx.Clear();  Umx.Clear(); //reuse these
	for(int L4=0;L4<popsByLocality_.len();++L4){
		Gmx.Append(NEW aMatrix<long double>((long double)0.0,groupHierarchy_.len(), groupHierarchy_.len()));
		Hmx.Append(NEW aMatrix<long double>((long double)0.0,groupHierarchy_.len(), groupHierarchy_.len()));
		for(int jr1=0;jr1<groupHierarchy_.len();++jr1){
			for(int jc1=0;jc1<groupHierarchy_.len();++jc1){
				if(jr1==jc1){
						Gmx[L4][jr1][jc1] = sum_i_nij_Squared[L4]/n_j_[L4] -2.0*nij_[jr1][L4] + n_j_[L4];
						Gmx[L4][jr1][jc1] *=( nij_[jr1][L4] * nij_[jr1][L4]) / n_j_[L4];
						Hmx[L4][jr1][jc1] = (sum_ik_nijkSquared[L4]/n_j_[L4]) *nij_[jr1][L4] * nij_[jr1][L4]
										+ (n_j_[L4] - 2.0*nij_[jr1][L4]) * sum_k_nijkSquared[jr1][L4];
						Hmx[L4][jr1][jc1] /= n_j_[L4];
						}
					else{
						Gmx[L4][jr1][jc1] = sum_i_nij_Squared[L4]/n_j_[L4] - nij_[jr1][L4]  - nij_[jc1][L4];
						Gmx[L4][jr1][jc1] *= ( nij_[jr1][L4] * nij_[jc1][L4]) / n_j_[L4];
						Hmx[L4][jr1][jc1] = (sum_ik_nijkSquared[L4]/n_j_[L4]) *nij_[jr1][L4] * nij_[jc1][L4]
										- nij_[jr1][L4] * sum_k_nijkSquared[jc1][L4]
										- nij_[jc1][L4] * sum_k_nijkSquared[jr1][L4];
						Hmx[L4][jr1][jc1] /= n_j_[L4];
						}
				}}//end for jr1&&jc1
		}//end for L4
	aMatrix<long double>* invCC = CC.Inverse();
	aMatrix<long double>* HHsum = Hmx.Sum();
	aMatrix<long double>* invCCHHsum = invCC->Multiply(*HHsum);
	aMatrix<long double>* GGsum = Gmx.Sum();
	aMatrix<long double>* invCCGGsum = invCC->Multiply(*GGsum);
				
	long double n01,n02,n03,n04,n05,n06,n07,n08,n09,n10,n11,n12,n13,n14,n15,n16,n17;
	 n01=n02=n03=n04=n05=n06=n07=n08=n09=n10=n11=n12=n13=n14=n15=n16=n17=0.0;
	 //npop_ is the number of populations by localities, accounting for missing populations in factorial array
	nList.Clear();
	for(int i=0;i<17;++i){nList.Append(0.0);}
	nList[0] = n01 = n___ - sum_i_Sumjk_nijkSquared_ni__ - invDsFormSum_UsFormSum->Trace();
	nList[1] = n02 = invDsFormSum_UsFormSum->Trace() - invCHsum->Trace();
	nList[2] = n03 = n___ - sum_i_sumj_nijSquared_ni__ - invCGsum->Trace();
	nList[3] = n04 =  invCHsum->Trace();
	nList[4] = n05 = invCGsum->Trace();
	nList[5] = n06 = n___ - sum_i_sumj_nijSquared_ni__;
	nList[6] = n07 = sum_i_Sumjk_nijkSquared_ni__ - (sum_ijk_nijkSquared / n___);
	nList[7] = n08 = sum_i_sumj_nijSquared_ni__ - (sum_ij_nijSquared / n___);
	nList[8] = n09 = sum_i_sumj_nijSquared_ni__ - (sum_j_n_j_Squared / n___);
	nList[9] = n10 = n___ - (sum_ni__Squared / n___);
	nList[10] = n11 =  invCCHHsum->Trace();
	nList[11] = n12 = invCCGGsum->Trace();
	nList[12] = n13 = n___ - sum_j_sumi_nijSquared_n_j_;
	nList[13] = n14 = sum_j_Sumik_nijkSquared_n_j_  - (sum_ijk_nijkSquared / n___);
	nList[14] = n15 = sum_j_sumi_nijSquared_n_j_ - (sum_ij_nijSquared / n___);
	nList[15] = n16 = sum_j_sumi_nijSquared_n_j_ - (sum_ni__Squared / n___);
	nList[16] = n17 = n___ - (sum_j_n_j_Squared / n___);
	dfList.Clear();
	for(int i=0;i<=5;++i){dfList.Append(0.0);}
	dfList[0] =(long double) ngroups_ - 1.0;//species
	dfList[1] =(long double) popsByLocality_.len() - 1.0;//localities
	dfList[2] = (long double) npop_ - (long double) ngroups_ - (long double) popsByLocality_.len() + 1.0;//populations
	dfList[3] = d__ - (long double) npop_;//individuals
	dfList[4] = n___ - d__;//genes
	dfList[5] = n___ - 1.0;//total
	 delete invDsFormSum_UsFormSum; invDsFormSum_UsFormSum=NULL;
	 delete invDsFormSum; invDsFormSum=NULL;
	 delete UsFormSum; UsFormSum=NULL;
	 delete DsFormSum; DsFormSum=NULL;
	 delete invCGsum; invCGsum=NULL;
	 delete Gsum; Gsum=NULL;
	 delete invCHsum; invCHsum=NULL;
	 delete Hsum; Hsum=NULL;
	 delete invC; invC=NULL;
	 delete invCCGGsum; invCCGGsum=NULL;
	 delete GGsum; GGsum=NULL;
	 delete invCCHHsum; invCCHHsum=NULL;
	 delete HHsum; HHsum=NULL;
	 delete invCC; invCC=NULL;
	 delete invDsFormSum_UsFormSum; invDsFormSum_UsFormSum=NULL;
	}//end FstatisticAnalysisCodom::CalculateFactorialVarCompCoeffsIII



void FstatisticAnalysisCodom::CalculateStdVarCompCoeffs(aList<long double>& nList, aList<long double>& dfList){
	//calculates variance component coefficients for standard F-stats using linear algebra
	long double n___=0.0;//total genes in locus
	long double d__=0.0;//total individuals
	StatList<long double> n_j_;
	for(int locality=0;locality<npop_;++locality){
		n_j_.Append(0.0);
		n_j_[locality] += GetLocusSampleSize(popsByLocality_[locality][0], 0);
		d__ += popSize_[popsByLocality_[locality][0] ];
		n___+=n_j_[locality];
		}
	StatList<long double> ni__;
	ObjList<StatList<long double> > nij_;
	ObjList<ObjList<StatList<long double> > > nijk;
	ObjList<StatList<long double> > sum_k_nijkSquared;
	StatList<long double> sum_j_nij_Squared, sum_i_nij_Squared;
	StatList<long double> sum_jk_nijkSquared, sum_ik_nijkSquared;
				
	ni__.Append(0.0);
	nij_.Append(NEW StatList<long double>());
	nijk.Append(NEW ObjList<StatList<long double> >());
	sum_k_nijkSquared.Append(NEW StatList<long double>());
	sum_j_nij_Squared.Append(0.0);
	sum_jk_nijkSquared.Append(0.0);
	for(int L0=0;L0<npop_;++L0){//some localities may be empty for a given species
		nij_[0].Append(0.0);
		nijk[0].Append(NEW StatList<long double>());
		sum_k_nijkSquared[0].Append(0.0);
		}

	for(int L00=0;L00<npop_;++L00){
		sum_i_nij_Squared.Append(0.0);
		sum_ik_nijkSquared.Append(0.0);
		}//end for L00
	long double sum_i_Sumjk_nijkSquared_ni__=0.0;
	long double sum_i_sumj_nijSquared_ni__ = 0.0;
	long double sum_j_Sumik_nijkSquared_n_j_=0.0;
	long double sum_j_sumi_nijSquared_n_j_ = 0.0;
	long double sum_ijk_nijkSquared=0.0;
	long double sum_ij_nijSquared = 0.0;
	long double sum_ni__Squared = 0.0;

	for(int L=0;L<npop_;++L){
		nij_[0][L] += GetLocusSampleSize(L, 0);
		for(int ind=0;ind<popSize_[L];++ind){
			nijk[0][L].Append(GetLocusSampleSize(L,ind,0));
			sum_k_nijkSquared[0][L] += nijk[0][L][ind] * nijk[0][L][ind];
			}
		sum_j_nij_Squared[0] += nij_[0][L] *  nij_[0][L];
		}
	sum_jk_nijkSquared[0] += sum_k_nijkSquared[0].sum();
	ni__[0] += nij_[0].sum();
	sum_i_Sumjk_nijkSquared_ni__ += sum_jk_nijkSquared[0] /ni__[0] ;
	sum_i_sumj_nijSquared_ni__ += sum_j_nij_Squared[0] /ni__[0] ;
	sum_ijk_nijkSquared += sum_jk_nijkSquared[0];
	sum_ij_nijSquared += sum_j_nij_Squared[0];
	sum_ni__Squared += ni__[0] * ni__[0];

	long double sum_j_n_j_Squared = 0.0;
	for(int L3=0;L3<npop_;++L3){
		sum_i_nij_Squared[L3] += nij_[0][L3] *  nij_[0][L3];
		sum_ik_nijkSquared[L3] += sum_k_nijkSquared[0][L3];
		sum_j_Sumik_nijkSquared_n_j_ += sum_ik_nijkSquared[L3] /n_j_[L3] ;
		sum_j_sumi_nijSquared_n_j_ += sum_i_nij_Squared[L3] /n_j_[L3] ;
		sum_j_n_j_Squared += n_j_[L3]*n_j_[L3];
		}//L3
	aMatrix<long double> C((long double)0.0,npop_, npop_);
	for(int jr0=0;jr0<C.rows();++jr0){
		for(int jc0=0;jc0<C.cols();++jc0){
			long double sumnijnijprime=0.0;
			sumnijnijprime += nij_[0][jr0]*nij_[0][jc0] / ni__[0];
			sumnijnijprime *= (-1.0);
			C[jr0][jc0] = sumnijnijprime;
			if(jr0==jc0){
				C[jr0][jc0] += n_j_[jr0];}
			}}//end for jr0 && jc0

	MatrixList  Dmx, Gmx, Hmx, Umx;
	Dmx.Append(NEW aMatrix<long double>((long double)0.0,npop_,npop_));
	Gmx.Append(NEW aMatrix<long double>((long double)0.0,npop_,npop_));
	Hmx.Append(NEW aMatrix<long double>((long double)0.0,npop_,npop_));
	Umx.Append(NEW aMatrix<long double>((long double)0.0,npop_,npop_));
	for(int jr1=0;jr1<npop_;++jr1){
		for(int jc1=0;jc1<npop_;++jc1){
			if(jr1==jc1){
					Dmx[0][jr1][jc1] = nij_[0][jr1] * (1.0 - nij_[0][jr1] /ni__[0]);
					Gmx[0][jr1][jc1] = sum_j_nij_Squared[0]/ni__[0] -2.0*nij_[0][jr1] + ni__[0];
					Gmx[0][jr1][jc1] *=( nij_[0][jr1] * nij_[0][jr1]) / ni__[0];
					Hmx[0][jr1][jc1] = (sum_jk_nijkSquared[0]/ni__[0]) *nij_[0][jr1] * nij_[0][jr1]
									+ (ni__[0] - 2.0*nij_[0][jr1]) * sum_k_nijkSquared[0][jr1];
					Hmx[0][jr1][jc1] /= ni__[0];
					Umx[0][jr1][jc1] =  (nij_[0][jr1] * nij_[0][jr1]* sum_jk_nijkSquared[0]/ni__[0]) 
									+ (ni__[0] - 2.0*nij_[0][jr1]) * sum_k_nijkSquared[0][jr1];
					Umx[0][jr1][jc1] /= ni__[0];
					}
				else{
					Dmx[0][jr1][jc1] = (-1.0)*(nij_[0][jr1] *  nij_[0][jc1]) /ni__[0];
					Gmx[0][jr1][jc1] = sum_j_nij_Squared[0]/ni__[0] -nij_[0][jr1] + -nij_[0][jc1];
					Gmx[0][jr1][jc1] *= ( nij_[0][jr1] * nij_[0][jc1]) / ni__[0];
					Hmx[0][jr1][jc1] = (sum_jk_nijkSquared[0]/ni__[0]) *nij_[0][jr1] * nij_[0][jc1]
									- nij_[0][jr1] * sum_k_nijkSquared[0][jc1]
									- nij_[0][jc1] * sum_k_nijkSquared[0][jr1];
					Hmx[0][jr1][jc1] /= ni__[0];
					Umx[0][jr1][jc1] =  (nij_[0][jr1] * nij_[0][jc1]* sum_jk_nijkSquared[0]/ni__[0]) 
									-nij_[0][jr1] * sum_k_nijkSquared[0][jc1]
									-nij_[0][jc1] * sum_k_nijkSquared[0][jr1];
					Umx[0][jr1][jc1] /= ni__[0];
					}
			}}//end for jr1&&jc1
	aMatrix<long double>* invC = C.Inverse();
	aMatrix<long double>* Hsum = Hmx.Sum();
	aMatrix<long double>* invCHsum = invC->Multiply(*Hsum);
	aMatrix<long double>* Gsum = Gmx.Sum();
	aMatrix<long double>* invCGsum = invC->Multiply(*Gsum);

	MatrixList DsformMx, UsformMx;
	PartitionedMatrix Dpm, Upm;
	DsformMx.Append(Dpm.sForm(Dmx[0], 1, 2));
	UsformMx.Append(Upm.sForm(Umx[0], 1, 2));
	aMatrix<long double>* DsFormSum = DsformMx.Sum();
	aMatrix<long double>* UsFormSum = UsformMx.Sum();
	aMatrix<long double>* invDsFormSum = DsFormSum->Inverse();
	aMatrix<long double>* invDsFormSum_UsFormSum = invDsFormSum->Multiply(*UsFormSum);

//check this section for order of ij
	long double n01,n02,n03,n04;
	 n01=n02=n03=n04=0.0;
	 //npop_ is the number of populations by localities, accounting for missing populations in factorial array
	nList.Clear();
	for(int i=0;i<3;++i){nList.Append(0.0);}
	nList[0] = n01 = n___ - sum_i_Sumjk_nijkSquared_ni__ - invDsFormSum_UsFormSum->Trace();
	nList[1] = n02 = invDsFormSum_UsFormSum->Trace() - invCHsum->Trace();
	nList[2] = n03 = n___ - sum_i_sumj_nijSquared_ni__ - invCGsum->Trace();
	nList[3] = n04 =  invCHsum->Trace();

	dfList.Clear();
	for(int i=0;i<=3;++i){dfList.Append(0.0);}
	dfList[0] = (long double) npop_ - 1.0;//populations
	dfList[1] = d__ - (long double) npop_;//individuals
	dfList[2] = n___ - d__;//genes
	dfList[3] = n___ - 1.0;//total
	 delete invDsFormSum_UsFormSum; invDsFormSum_UsFormSum=NULL;
	 delete invDsFormSum; invDsFormSum=NULL;
	 delete UsFormSum; UsFormSum=NULL;
	 delete DsFormSum; DsFormSum=NULL;
	 delete invCGsum; invCGsum=NULL;
	 delete Gsum; Gsum=NULL;
	 delete invCHsum; invCHsum=NULL;
	 delete Hsum; Hsum=NULL;
	 delete invC; invC=NULL;
	}//end FstatisticAnalysisCodom::CalculateStdVarCompCoeffs



void FstatisticAnalysisCodom::CalculateIBDestimates(Ran3& ran,int sampleSize){//sampled with replacement, per locus
	if(! IBDcurrent_){
			ClearIBDs();}
		else if(IBDmethod_.Includes((int)IBDestimCodom)) {return;}
		else{}
	CalculateAlleleFreqs();
	if(nlociCodom_==0){
		IBDcurrent_=true;
		return;}

	//within individuals
	long i,pop,pop1,pop2,loc,a1,a2,i1,i2,rep;
	StatList<int> IBDwithinIndivByLocCounter;
	long double IBDval;
	long double IBD_inIndiv_, IBD_betwIndivInPop_, IBD_betwIndivBetwPops_;
	StatList<long double> 	IBD_inIndiv_ByLoc_, IBD_betwIndivInPop_ByLoc_, IBD_betwIndivBetwPops_ByLoc_;
	StatList<long double> 	IBD_inIndiv_ByLoc_Num, IBD_betwIndivInPop_ByLoc_Num, IBD_betwIndivBetwPops_ByLoc_Num, IBD_Denom;

	jackIBDvalsLoc_.Append(NEW ObjList<aList<long double> >());
	for(loc=0;loc<nlociCodom_;++loc){
		jackIBDvalsLoc_.Last().Append(NEW aList<long double>());
		jackIBDvalsLoc_.Last().Last().Append(zero);//IBD within indiv
		jackIBDvalsLoc_.Last().Last().Append(zero);//IBD within pop
		jackIBDvalsLoc_.Last().Last().Append(zero);//IBD betw pops
		IBDwithinIndivByLocCounter.Append(0);
		IBD_inIndiv_ByLoc_.Append(zero);
		IBD_betwIndivInPop_ByLoc_.Append(zero);
		IBD_betwIndivBetwPops_ByLoc_.Append(zero);
		IBD_inIndiv_ByLoc_Num.Append(zero);
		IBD_betwIndivInPop_ByLoc_Num.Append(zero);
		IBD_betwIndivBetwPops_ByLoc_Num.Append(zero);
		IBD_Denom.Append(zero);
		}//loc

	for(loc=0;loc<nlociCodom_;++loc){
		for(rep=0;rep<sampleSize;++rep){// correct sample size search for missing data
			IBD_inIndiv_ByLoc_.Append(zero);
			pop = ran.longBetween(0,npop_-1);
			i = ran.longBetween(0,popSize(pop)-1);
			TextDatum& g0_1 = GenotypicVal(pop, i, locusNumCodom_[loc], 0);
			TextDatum& g1_1 = GenotypicVal(pop, i, locusNumCodom_[loc], 1);
			if(g0_1 == g1_1 && g0_1.OK()){
				IBDwithinIndivByLocCounter[loc]++;
				}
			}//rep
		IBDval = (long double)IBDwithinIndivByLocCounter[loc] / (long double) sampleSize;
//		IBD_inIndiv_ByLoc_.Append(IBDval);
		IBD_inIndiv_ByLoc_[loc]=IBDval;
		IBD_inIndiv_ByLoc_Num[loc] = (long double)IBDwithinIndivByLocCounter[loc];
		}//loc
	IBD_inIndiv_ = (long double)IBDwithinIndivByLocCounter.sum() / (long double) (nlociCodom_ * sampleSize);//accounts for sample size diffs among loci
	
	//between individuals
	StatList<int> IBDbetwIndivInPopByLocCounter;
	for(loc=0;loc<nlociCodom_;++loc){
		IBDbetwIndivInPopByLocCounter.Append(0);
		}//loc

	for(loc=0;loc<nlociCodom_;++loc){
		for(rep=0;rep<sampleSize;++rep){
			do{
				pop = ran.longBetween(0,npop_-1);
				}while(popSize(pop)<2);
			i1=ran.longBetween(0,popSize(pop)-1);
			do{
				i2=ran.longBetween(0,popSize(pop)-1);
				}while(i1==i2);
			a1= ran.longBetween(0,1);
			a2 = ran.longBetween(0,1);
			TextDatum& g0_2 = GenotypicVal(pop, i1, locusNumCodom_[loc], a1);
			TextDatum& g1_2 = GenotypicVal(pop, i2, locusNumCodom_[loc], a2);
			if(g0_2 == g1_2 && g0_2.OK()){
				IBDbetwIndivInPopByLocCounter[loc]++;}
			}//rep
		IBDval = (long double)IBDbetwIndivInPopByLocCounter[loc] /(long double) sampleSize;
		IBD_betwIndivInPop_ByLoc_[loc]=IBDval;
		IBD_betwIndivInPop_ByLoc_Num[loc] = (long double)IBDbetwIndivInPopByLocCounter[loc];
		}//loc
	IBD_betwIndivInPop_ = (long double)IBDbetwIndivInPopByLocCounter.sum() /(long double) (sampleSize * nlociCodom_);
	
	
	StatList<int> IBDbetwIndivBetwPopsByLocCounter;
	for(loc=0;loc<nlociCodom_;++loc){
		IBDbetwIndivBetwPopsByLocCounter.Append(0);
		}//loc

	for(loc=0;loc<nlociCodom_;++loc){
		for(rep=0;rep<sampleSize;++rep){
			do{
				pop1 = ran.longBetween(0,npop_-1);
				pop2 = ran.longBetween(0,npop_-1);
				}while(pop1==pop2 || popSize(pop1)<2 || popSize(pop2)<2);
			i1=ran.longBetween(0,popSize(pop1)-1);
			i2=ran.longBetween(0,popSize(pop2)-1);
			a1= ran.longBetween(0,1);
			a2 = ran.longBetween(0,1);
			TextDatum& g0_2 = GenotypicVal(pop1, i1, locusNumCodom_[loc], a1);
			TextDatum& g1_2 = GenotypicVal(pop2, i2, locusNumCodom_[loc], a2);
			if(g0_2 == g1_2 && g0_2.OK()){
				IBDbetwIndivBetwPopsByLocCounter[loc]++;
				}
			}//rep
		IBDval = (long double)IBDbetwIndivBetwPopsByLocCounter[loc] /(long double) sampleSize;
		IBD_betwIndivBetwPops_ByLoc_[loc]=IBDval;
		IBD_betwIndivBetwPops_ByLoc_Num[loc] = (long double)IBDbetwIndivBetwPopsByLocCounter[loc];
		}//loc
	IBD_betwIndivBetwPops_ = (long double)IBDbetwIndivBetwPopsByLocCounter.sum() /(long double) (sampleSize * nlociCodom_);
				

	//transfer to general data structures
	IBDmethod_.Append(IBDestimCodom);
	IBDlevels_.Append( NEW StringList());
	IBDlevels_.Last().Append(NEW aString("withinIndivs"));
	IBDlevels_.Last().Append(NEW aString("betwIndivsInPops"));
	IBDlevels_.Last().Append(NEW aString("betwIndivsBetwPops"));
	IBDvals_.Append(NEW aList<long double>());
	IBDvals_.Last().Append(IBD_inIndiv_);
	IBDvals_.Last().Append(IBD_betwIndivInPop_);
	IBDvals_.Last().Append(IBD_betwIndivBetwPops_);
	IBDvalsByLoc_.Append(NEW ObjList<aList<long double> >());
	ObjList<aList<long double> >& IBDByLoc = IBDvalsByLoc_.Last();
	ObjList<aList<long double> >& jackIBDByLoc = jackIBDvalsLoc_.Last();
	for(loc=0;loc<nlociCodom_;++loc){
		IBDByLoc.Append(NEW aList<long double>());
		IBDByLoc[loc].Append(IBD_inIndiv_ByLoc_[loc]);
		IBDByLoc[loc].Append(IBD_betwIndivInPop_ByLoc_[loc]);
		IBDByLoc[loc].Append(IBD_betwIndivBetwPops_ByLoc_[loc]);
		if(nlociCodom_>1){
			jackIBDByLoc[loc][0] = (IBD_inIndiv_ByLoc_Num.sum()-IBD_inIndiv_ByLoc_Num[loc]) / (long double)((nlociCodom_-1)*sampleSize);
			jackIBDByLoc[loc][1] = (IBD_betwIndivInPop_ByLoc_Num.sum()-IBD_betwIndivInPop_ByLoc_Num[loc]) / (long double)((nlociCodom_-1)*sampleSize);
			jackIBDByLoc[loc][2] = (IBD_betwIndivBetwPops_ByLoc_Num.sum()-IBD_betwIndivBetwPops_ByLoc_Num[loc]) / (long double)((nlociCodom_-1)*sampleSize);
			}
		}//loc
	IBDcurrent_=true;

	//update F-stats
	long double Fisval, Fstval, Fitval;
	if(! FstatsCurrent_){
		ClearFstats();}
	Fmethod_.Append(IBDestimCodom);
	Flevels_.Append( NEW StringList());
	Flevels_.Last().Append(NEW aString("Fis"));
	Flevels_.Last().Append(NEW aString("Fst"));
	Flevels_.Last().Append(NEW aString("Fit"));
	Fstats_.Append(NEW aList<long double>());
	Fisval=zero;
	Fstval=Fitval=one;
	if(IBD_betwIndivInPop_ != one){
		Fisval = (IBD_inIndiv_ - IBD_betwIndivInPop_) / (one - IBD_betwIndivInPop_);}
	if(IBD_betwIndivBetwPops_ != one){
		Fstval = (IBD_betwIndivInPop_ - IBD_betwIndivBetwPops_) / (one - IBD_betwIndivBetwPops_);
		Fitval = (IBD_inIndiv_ - IBD_betwIndivBetwPops_) / (one - IBD_betwIndivBetwPops_);
		}
	Fstats_.Last().Append(Fisval);
	Fstats_.Last().Append(Fstval);
	Fstats_.Last().Append(Fitval);
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//[Fmethod][loc][Flevel]
	ObjList<aList<long double> >& FByLoc = FstatsLoc_.Last();
	ObjList<aList<long double> >& jackFByLoc = jackFstatsLoc_.Last();
	StatList<long double> jackFisByLocNum, jackFstByLocNum, jackFitByLocNum, jackFisByLocDenom, jackFstByLocDenom;
	long double FisNum, FstNum, FitNum, FisDenom, FstDenom;
	for(loc=0;loc<nlociCodom_;++loc){
		FisNum=FstNum=FitNum=FisDenom=FstDenom=Fisval=zero;
		Fitval=Fstval=one;
		FByLoc.Append(NEW aList<long double>());
		if(IBD_betwIndivInPop_ByLoc_[loc] != one){
			FisNum = (IBD_inIndiv_ByLoc_[loc] - IBD_betwIndivInPop_ByLoc_[loc]);
			FisDenom = (one - IBD_betwIndivInPop_ByLoc_[loc]);
			Fisval = FisNum/FisDenom;
			}
		if(IBD_betwIndivBetwPops_ByLoc_[loc] != one){
			FstNum = (IBD_betwIndivInPop_ByLoc_[loc] - IBD_betwIndivBetwPops_ByLoc_[loc]);
			FstDenom = (one - IBD_betwIndivBetwPops_ByLoc_[loc]);
			FitNum = (IBD_inIndiv_ByLoc_[loc] - IBD_betwIndivBetwPops_ByLoc_[loc]);
			Fstval = FstNum/FstDenom;
			Fitval = FitNum/FstDenom;
			}
		FByLoc[loc].Append(Fisval);
		FByLoc[loc].Append(Fstval);
		FByLoc[loc].Append(Fitval);
		jackFisByLocNum.Append(FisNum);
		jackFstByLocNum.Append(FstNum);
		jackFitByLocNum.Append(FitNum);
		jackFisByLocDenom.Append(FisDenom);
		jackFstByLocDenom.Append(FstDenom);
		}//loc
	for(loc=0;loc<nlociCodom_;++loc){
		Fisval=FisNum=FstNum=FitNum=FisDenom=FstDenom=zero;
		Fstval=Fitval=one;
		jackFByLoc.Append(NEW aList<long double>());
		FisDenom = jackFisByLocDenom.sum()-jackFisByLocDenom[loc];
		FstDenom = jackFstByLocDenom.sum()-jackFstByLocDenom[loc];
		if(FisDenom != zero){
			Fisval = (jackFisByLocNum.sum()-jackFisByLocNum[loc])/FisDenom;}
		if(FstDenom != zero){
			Fstval = (jackFstByLocNum.sum()-jackFstByLocNum[loc])/FstDenom;
			Fitval = (jackFitByLocNum.sum()-jackFitByLocNum[loc])/FstDenom;
			}
		jackFByLoc[loc].Append(Fisval);
		jackFByLoc[loc].Append(Fstval);//Fst
		jackFByLoc[loc].Append(Fitval);//Fit
		}//loc
	FstatsCurrent_=true;
	}//FstatisticAnalysisCodom::CalculateIBDestimates(Ran3&,int)



void FstatisticAnalysisCodom::CalculateIBDexhaustive(void){//only do this for very small population samples
	if(! IBDcurrent_){
			ClearIBDs();}
		else if(IBDmethod_.Includes((int)IBDexhaustCodom)) {return;}
		else{}
	CalculateAlleleFreqs();
	if(nlociCodom_==0){
		IBDcurrent_=true;
		return;}

	//within individuals
	long i,pop,pop1,pop2,loc,a1,a2,i1,i2;
	StatList<int> IBDwithinIndivByLocCounter;
	long double IBDval,IBDdenom, IBDrunningDenom=0;
	long double IBD_inIndiv_, IBD_betwIndivInPop_, IBD_betwIndivBetwPops_;
	StatList<long double> 	IBD_inIndiv_ByLoc_, IBD_betwIndivInPop_ByLoc_, IBD_betwIndivBetwPops_ByLoc_;
	StatList<long double> 	IBD_inIndiv_ByLoc_Num, IBD_betwIndivInPop_ByLoc_Num, IBD_betwIndivBetwPops_ByLoc_Num, IBD_Denom;
	StatList<int> allIndivsByLoc;

	jackIBDvalsLoc_.Append(NEW ObjList<aList<long double> >());
	for(loc=0;loc<nlociCodom_;++loc){
		jackIBDvalsLoc_.Last().Append(NEW aList<long double>());
		jackIBDvalsLoc_.Last()[loc].Append(zero);//IBD within indiv
		jackIBDvalsLoc_.Last().Last().Append(zero);//IBD within pop
		jackIBDvalsLoc_.Last().Last().Append(zero);//IBD betw pops
		IBDwithinIndivByLocCounter.Append(0);
		allIndivsByLoc.Append(0);
		IBD_inIndiv_ByLoc_.Append(zero);
		IBD_betwIndivInPop_ByLoc_.Append(zero);
		IBD_betwIndivBetwPops_ByLoc_.Append(zero);
		IBD_inIndiv_ByLoc_Num.Append(zero);
		IBD_betwIndivInPop_ByLoc_Num.Append(zero);
		IBD_betwIndivBetwPops_ByLoc_Num.Append(zero);
		IBD_Denom.Append(zero);
		for(pop=0;pop<npop_;++pop){
//			allIndivsByLoc[loc] += GetIndividualsSampled(pop,locusNumCodom_[loc]);
			allIndivsByLoc[loc] += GetIndividualsSampled(pop,loc);
			}//pop
		}//loc

	int nCounter, nCountOverLoci=0;
	for(loc=0;loc<nlociCodom_;++loc){
//		IBD_inIndiv_ByLoc_.Append(zero);
		nCounter=0;
		for(pop=0;pop<npop_;++pop){
			for(i=0;i<popSize(pop);++i){
				TextDatum& g0_1 = GenotypicVal(pop, i, locusNumCodom_[loc], 0);
				TextDatum& g1_1 = GenotypicVal(pop, i, locusNumCodom_[loc], 1);
				if(g0_1.OK() && g1_1.OK()){
					++nCounter;
					if(g0_1 == g1_1 && g0_1.OK()){
						IBDwithinIndivByLocCounter[loc]++;
						}
					}
				}//i
			}//pop
		IBDval = (long double)IBDwithinIndivByLocCounter[loc];
//		IBDdenom = ((long double) allIndivsByLoc[loc] * two) * ((long double) allIndivsByLoc[loc] * two - one) / two;
		IBDdenom = (long double)nCounter;
		IBD_inIndiv_ByLoc_[loc] = IBDval/IBDdenom;
		IBD_inIndiv_ByLoc_Num[loc] = IBDval;
		IBD_Denom[loc] = IBDdenom;
		nCountOverLoci += nCounter;
		}//loc
	IBDval = (long double)IBDwithinIndivByLocCounter.sum();
//	IBDdenom = ((long double) allIndivsByLoc.sum() * two) * ((long double) allIndivsByLoc.sum() * two - one) / two;
	IBDdenom = (long double) nCountOverLoci;
	IBD_inIndiv_ = IBDval/IBDdenom;
	
	//between individuals
	StatList<int> IBDbetwIndivInPopByLocCounter;
	for(loc=0;loc<nlociCodom_;++loc){
		IBDbetwIndivInPopByLocCounter.Append(0);
		}//loc
	nCountOverLoci=0;
	for(loc=0;loc<nlociCodom_;++loc){
		nCounter=0;
		for(pop=0;pop<npop_;++pop){
			for(i1=0;i1<popSize(pop)-1;++i1){
				for(i2=i1+1;i2<popSize(pop);++i2){
					for(a1=0;a1<2;++a1){
						for(a2=0;a2<2;++a2){
							TextDatum& g0_2 = GenotypicVal(pop, i1, locusNumCodom_[loc], a1);
							TextDatum& g1_2 = GenotypicVal(pop, i2, locusNumCodom_[loc], a2);
							if(g0_2.OK() && g1_2.OK()){
								++nCounter;
								if(g0_2 == g1_2){
									IBDbetwIndivInPopByLocCounter[loc]++;}
								}
							}//a2
						}//a1
					}//i2
				}//i1
			}//pop
		IBDval = (long double)IBDbetwIndivInPopByLocCounter[loc];
//		IBDdenom = ( (long double) allIndivsByLoc[loc] * two)*( (long double) allIndivsByLoc[loc] * two - one) /two;
		IBDdenom = (long double) nCounter;
		IBD_betwIndivInPop_ByLoc_Num[loc]=IBDval;
//		IBD_betwIndivInPop_ByLoc_.Append(IBDval/IBDdenom);
		IBD_betwIndivInPop_ByLoc_[loc] = IBDval/IBDdenom;
		nCountOverLoci+=nCounter;
		}//loc
//	IBDval = (long double)IBDbetwIndivInPopByLocCounter.sum() /( (long double) allIndivsByLoc.sum() * four);
	IBDval = (long double)IBDbetwIndivInPopByLocCounter.sum();
//	IBDdenom = ((long double) allIndivsByLoc.sum() * two) * ((long double) allIndivsByLoc.sum() * two - one) / two;
	IBDdenom = (long double) nCountOverLoci;
	IBD_betwIndivInPop_ = IBDval/IBDdenom;
	
	StatList<int> IBDbetwIndivBetwPopsByLocCounter;
	for(loc=0;loc<nlociCodom_;++loc){
		IBDbetwIndivBetwPopsByLocCounter.Append(0);
		}//loc

	//Between individuals between populations
	
	nCountOverLoci=0;
	for(loc=0;loc<nlociCodom_;++loc){
		nCounter=0;
		for(pop1=0;pop1<npop_-1;++pop1){
			for(pop2=pop1+1;pop2<npop_;++pop2){
				for(i1=0;i1<popSize(pop1)-1;++i1){
					for(i2=0;i2<popSize(pop2);++i2){
						for(a1=0;a1<2;++a1){
							for(a2=0;a2<2;++a2){
								TextDatum& g0_2 = GenotypicVal(pop1, i1, locusNumCodom_[loc], a1);
								TextDatum& g1_2 = GenotypicVal(pop2, i2, locusNumCodom_[loc], a2);
								if(g0_2.OK() && g1_2.OK()){
									++nCounter;
									if(g0_2 == g1_2){
										IBDbetwIndivBetwPopsByLocCounter[loc]++;
										}
									}//OK
								}//a2
							}//a1
						}//i2
					}//i1
				}//pop2
			}//pop1
		IBDval = (long double)IBDbetwIndivBetwPopsByLocCounter[loc];
//		IBDdenom = ( (long double) allIndivsByLoc[loc] * two)*( (long double) allIndivsByLoc[loc] * two - one) /two;
		IBDdenom = (long double) nCounter;
		IBDrunningDenom += IBDdenom;
		IBD_betwIndivBetwPops_ByLoc_[loc]=IBDval/IBDdenom;
		IBD_betwIndivBetwPops_ByLoc_Num[loc] = IBDval;
		nCountOverLoci+=nCounter;
		}//loc
	IBDval = (long double)IBDbetwIndivBetwPopsByLocCounter.sum();
//	IBDdenom = ((long double) allIndivsByLoc.sum() * two) * ((long double) allIndivsByLoc.sum() * two - one) / two;
	IBDdenom = (long double) nCountOverLoci;
	IBD_betwIndivBetwPops_ = IBDval/IBDdenom;
			
	//transfer to general data structures
	IBDmethod_.Append(IBDexhaustCodom);
	IBDlevels_.Append( NEW StringList());
	IBDlevels_.Last().Append(NEW aString("withinIndivs"));
	IBDlevels_.Last().Append(NEW aString("betwIndivsInPops"));
	IBDlevels_.Last().Append(NEW aString("betwIndivsBetwPops"));
	IBDvals_.Append(NEW aList<long double>());
	IBDvals_.Last().Append(IBD_inIndiv_);
	IBDvals_.Last().Append(IBD_betwIndivInPop_);
	IBDvals_.Last().Append(IBD_betwIndivBetwPops_);
	IBDvalsByLoc_.Append(NEW ObjList<aList<long double> >());
	ObjList<aList<long double> >& IBDByLoc = IBDvalsByLoc_.Last();
	ObjList<aList<long double> >& jackIBDByLoc = jackIBDvalsLoc_.Last();
	for(loc=0;loc<nlociCodom_;++loc){
		IBDByLoc.Append(NEW aList<long double>());
		IBDByLoc[loc].Append(IBD_inIndiv_ByLoc_[loc]);
		IBDByLoc[loc].Append(IBD_betwIndivInPop_ByLoc_[loc]);
		IBDByLoc[loc].Append(IBD_betwIndivBetwPops_ByLoc_[loc]);
		if(IBD_Denom.sum()>IBD_Denom[loc]){
			jackIBDByLoc[loc][0] = (IBD_inIndiv_ByLoc_Num.sum() - IBD_inIndiv_ByLoc_Num[loc]) / (IBD_Denom.sum()-IBD_Denom[loc]);
			jackIBDByLoc[loc][1] = (IBD_betwIndivInPop_ByLoc_Num.sum() - IBD_betwIndivInPop_ByLoc_Num[loc]) / (IBD_Denom.sum()-IBD_Denom[loc]);
			jackIBDByLoc[loc][2] = (IBD_betwIndivBetwPops_ByLoc_Num.sum() - IBD_betwIndivBetwPops_ByLoc_Num[loc]) / (IBD_Denom.sum()-IBD_Denom[loc]);
			}
		}//loc
	IBDcurrent_=true;
	
	//update F-stats
	if(! FstatsCurrent_){
		ClearFstats();}
	Fmethod_.Append(IBDexhaustCodom);
	Flevels_.Append( NEW StringList());
	Flevels_.Last().Append(NEW aString("Fis"));
	Flevels_.Last().Append(NEW aString("Fst"));
	Flevels_.Last().Append(NEW aString("Fit"));
	Fstats_.Append(NEW aList<long double>());
	long double Fisval, Fstval, Fitval;
	Fisval=zero;
	Fstval=Fitval=one;
	if(IBD_betwIndivInPop_ != one){
		Fisval = (IBD_inIndiv_ - IBD_betwIndivInPop_) / (one - IBD_betwIndivInPop_);}
	if(IBD_betwIndivBetwPops_ != one){
		Fstval = (IBD_betwIndivInPop_ - IBD_betwIndivBetwPops_) / (one - IBD_betwIndivBetwPops_);
		Fitval = (IBD_inIndiv_ - IBD_betwIndivBetwPops_) / (one - IBD_betwIndivBetwPops_);
		}
	Fstats_.Last().Append(Fisval);
	Fstats_.Last().Append(Fstval);
	Fstats_.Last().Append(Fitval);
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//[Fmethod][loc][Flevel]
	ObjList<aList<long double> >& FByLoc = FstatsLoc_.Last();
	ObjList<aList<long double> >& jackFByLoc = jackFstatsLoc_.Last();
	StatList<long double> jackFisByLocNum, jackFstByLocNum, jackFitByLocNum, jackFisByLocDenom, jackFstByLocDenom;
	long double FisNum, FstNum, FitNum, FisDenom, FstDenom;
	for(loc=0;loc<nlociCodom_;++loc){
		FByLoc.Append(NEW aList<long double>());
		Fisval=FisNum=FstNum=FitNum=FisDenom=FstDenom=zero;
		Fstval=Fitval=one;
		if(IBD_betwIndivInPop_ByLoc_[loc] != one){
			FisNum = (IBD_inIndiv_ByLoc_[loc] - IBD_betwIndivInPop_ByLoc_[loc]);
			FisDenom = (one - IBD_betwIndivInPop_ByLoc_[loc]);
			Fisval = FisNum/FisDenom;
			}
		if(IBD_betwIndivBetwPops_ByLoc_[loc] != one){
			FstNum = (IBD_betwIndivInPop_ByLoc_[loc] - IBD_betwIndivBetwPops_ByLoc_[loc]);
			FstDenom = (one - IBD_betwIndivBetwPops_ByLoc_[loc]);
			FitNum = (IBD_inIndiv_ByLoc_[loc] - IBD_betwIndivBetwPops_ByLoc_[loc]);
			Fstval = FstNum/FstDenom;
			Fitval = FitNum/FstDenom;
			}
		FByLoc[loc].Append(Fisval);
		FByLoc[loc].Append(Fstval);
		FByLoc[loc].Append(Fitval);
		jackFisByLocNum.Append(FisNum);
		jackFstByLocNum.Append(FstNum);
		jackFitByLocNum.Append(FitNum);
		jackFisByLocDenom.Append(FisDenom);
		jackFstByLocDenom.Append(FstDenom);
		}//loc
	for(loc=0;loc<nlociCodom_;++loc){
		Fisval=FisNum=FstNum=FitNum=FisDenom=FstDenom=zero;
		Fstval=Fitval=one;
		jackFByLoc.Append(NEW aList<long double>());
		FisDenom = jackFisByLocDenom.sum()-jackFisByLocDenom[loc];
		FstDenom = jackFstByLocDenom.sum()-jackFstByLocDenom[loc];
		if(FisDenom != zero){
			Fisval = (jackFisByLocNum.sum()-jackFisByLocNum[loc])/FisDenom;}
		if(FstDenom != zero){
			Fstval = (jackFstByLocNum.sum()-jackFstByLocNum[loc])/FstDenom;
			Fitval = (jackFitByLocNum.sum()-jackFitByLocNum[loc])/FstDenom;
			}
		jackFByLoc[loc].Append(Fisval);
		jackFByLoc[loc].Append(Fstval);//Fst
		jackFByLoc[loc].Append(Fitval);//Fit
		}//loc

	FstatsCurrent_=true;	
	}//FstatisticAnalysisCodom::CalculateIBDexhaustive()





void FstatisticAnalysisCodom::Clear(void){
	FstatisticAnalysisCore::Clear();
	CodominantMarkerData::Clear();
	}//end FstatisticAnalysisCodom::Clear()



void FstatisticAnalysisCodom::ClearIBDs(void){
		//derived classes pick and delete Fstats calculated from IBD
	FstatisticAnalysisCore::ClearIBDs();
	if(IBDmethod_.Includes((int)IBDexhaustCodom)){
		int m = IBDmethod_.Loc((int)IBDexhaustCodom);
		IBDlevels_.Delete(m);
		IBDvalsByLoc_.Delete(m);
		IBDmethod_.Delete(m);
		}
	if(IBDmethod_.Includes((int)IBDestimCodom)){
		int m = IBDmethod_.Loc((int)IBDestimCodom);
		IBDlevels_.Delete(m);
		IBDvalsByLoc_.Delete(m);
		IBDmethod_.Delete(m);
		}
	}//end FstatisticAnalysisCodom::ClearIBDs()



long double FstatisticAnalysisCodom::DiploidFstFromNm(long double Nm, int numTrueDemes, bool zygoteMigration){
	long double Fstval = zero, scale=four;
	if(zygoteMigration){scale=eight;}
	if(numTrueDemes<2){numTrueDemes=0;}
		else{numTrueDemes=MAX((int)npop_,numTrueDemes);}
	if(numTrueDemes>1){
		long double d =(long double)numTrueDemes;
		scale *= d*d/((d-one)*(d-one));
		}
	Fstval = one/(scale*Nm + one);
	return Fstval;
	}//FstatisticAnalysisCodom::DiploidFstFromNm(long double,int,bool)




void FstatisticAnalysisCodom::FstatsLinearAlg(void){
	//uses linear algebra to get variance components for F-statistics
	ObjList<GLM> LinearModelList;
	FstatsLinearAlg(LinearModelList);
	}//end  FstatisticAnalysisCodom::FstatsLinearAlg



void FstatisticAnalysisCodom::FstatsLinearAlg(ObjList<GLM>& LinearModelList){
	//uses linear algebra to get variance components for F-statistics
	//clear out any previous calculations
  	if(! FstatsCurrent_){
  		ClearFstats();
  		}
  	if((Fmethod_.Includes((int) linearAlg))){return;}
	FstatsCurrent_=vcCurrent_=0;
	Fmethod_.Append((int) linearAlg);
	int m = Fmethod_.len()-1;
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, F-level
	Fstats_.Append(NEW aList<long double>());//F-level
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	Flevels_.Append(NEW StringList());//F-level

	vcMethod_.Append((int) linearAlg);
	vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
	vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
	vc_.Append(NEW aList<long double>());//vc-level
	vcLevels_.Append(NEW StringList());//vc-level

	//set up all the F-stat arrays
	// first the array describing the Fstat structure:
	aString* FWord=NEW aString("Fit");
	Flevels_[m].Append(FWord);
	FWord=NEW aString("Fis");
	Flevels_[m].Append(FWord);
	FWord=NEW aString("Fst");
	Flevels_[m].Append(FWord);

	aString* vcWord=NEW aString("var(p)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(i)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(g)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(t)");
	vcLevels_[m].Append(vcWord);
	//make and initialize the jackknife & by-locus Fstat lists
	for (int Flevel=0;Flevel<3;++Flevel){Fstats_[m].Append(0.0);}
	for (int vclevel=0;vclevel<4;++vclevel){vc_[m].Append(0.0);}
	//some intermediate variables
	aList<long double> ALoc, BLoc, CLoc, BCLoc, ABCLoc;
	CheckForNewAlleles();
	UpdateVariableLoci();
	for (int j=0;j<CodominantMarkerData::loci();++j){
		//make and initialize the jackknife & by-locus Fstat lists for this locus 
		aList<long double>* locFstat = NEW aList<long double>();
		FstatsLoc_[m].Append(locFstat);
		aList<long double>* locJackFstat = NEW aList<long double>();
		jackFstatsLoc_[m].Append(locJackFstat);
		for (int Flevel=0;Flevel<3;++Flevel){
			locFstat->Append(0.0);
			locJackFstat->Append(0.0);
			}
		//make and initialize the jackknife & by-allele Fstat lists for this locus 
		ObjList<aList<long double> >* locAlleleFstat = NEW ObjList<aList<long double> >();
		FstatsAll_[m].Append(locAlleleFstat);
		//initialize variables
		ALoc.Append(0.0);
		BLoc.Append(0.0);
		CLoc.Append(0.0);
		BCLoc.Append(0.0);
		ABCLoc.Append(0.0);

		//make and initialize the by-allele F-stat list for each allele and append to this locus list
		StringList& locAlleleList = allelesPresent_[j];
		for (int a=0;a<locAlleleList.len();++a){
			aList<long double>* alleleFstat =NEW aList<long double>();
			locAlleleFstat->Append(alleleFstat);
			for (int Flevel=0;Flevel<3;++Flevel){alleleFstat->Append(0.0);}
			alleleFstat=NULL;
			}//end for a
		locFstat=locJackFstat=NULL;
		locAlleleFstat=NULL;	//forget						

		//make and initialize the by-allele var comp list
		vcLoc_[m].Append(NEW aList<long double>());
		for (int vclevel=0;vclevel<4;++vclevel){vcLoc_[m][j].Append(0.0);}
		vcAll_[m].Append(NEW ObjList<aList<long double> >());
		for (int a=0;a<locAlleleList.len();++a){
			vcAll_[m][j].Append(NEW aList<long double>());
			for (int vclevel=0;vclevel<4;++vclevel){vcAll_[m][j][a].Append(0.0);}
			}//end for a							
		}//end for j

	//the main body of the calculations
	
	long double Asum,Bsum,Csum,BCsum,ABCsum;
	Asum=Bsum=Csum=BCsum=ABCsum=0.0;
	ANOVAGenetics theTable;
	StringList locusAlleleNames;
	StringList factorList;
	aString* popStr = NEW aString("pop$");
	aString* indivinpopSr = NEW aString("indiv$(pop$)");
	factorList.Append(popStr); 
	factorList.Append(indivinpopSr); 
	Ran3 ran(-1);//dummy
	//need to loop over missing data classes
	//this assumes no missing data


	dataSetRec* rawAnovaData =theTable.CreateRawANOVADataFormat(*this, locusAlleleNames);
//	rawAnovaData->Print(cout);
	ObjList<aList<long double> >* varianceComponents = NULL;
	if (! rawAnovaData->MissingData()){
		if(LinearModelList.len()==0){//first time through
			GLM* lm = NEW GLM();
			lm->SetToReuseDesign(1);//true; reuse for each locus and allele
			LinearModelList.Append(lm);
			lm=NULL;//forget
			}
		GLM& linearModel = LinearModelList[0];
			varianceComponents =theTable.CalculateVarianceComponents(/* *this, */*rawAnovaData, 
											factorList, locusAlleleNames,/* 0, ran,*/ linearModel);
			}//end if no missing data
		else{//must dissect rawANOVAData into separate loci
			//and run individually
		if(LinearModelList.len()==0){//first time through
			for(int vloc00=0;vloc00<variableLocusNumCodom_.len();++vloc00){
				GLM* lm = NEW GLM();
				lm->SetToReuseDesign(1);//true; reuse for each allele of this locus
				LinearModelList.Append(lm);}}
			varianceComponents = NEW ObjList<aList<long double> >();
			int locallelecount0=3;//column of rawANOVAData holding correct locus and allele
			for(int vloc0=0;vloc0<variableLocusNumCodom_.len();++vloc0){
				dataSetRec locusANOVAData;//work with copies of columns from rawANOVAData
				dataColumnRec& grCol0 = rawAnovaData->First();
				dataColumnRec& popCol0 = rawAnovaData->Nth(1);
				dataColumnRec& indivCol0 = rawAnovaData->Nth(2);
				dataColumnRec* grCol1 = grCol0.Duplicate();
				dataColumnRec* popCol1 = popCol0.Duplicate();
				dataColumnRec* indivCol1 = indivCol0.Duplicate();
				locusANOVAData.Append(grCol1);
				locusANOVAData.Append(popCol1);
				locusANOVAData.Append(indivCol1);
				int loc0 = variableLocusNumCodom_[vloc0];
				StringList& locAlleleList0 = allelesPresent_[loc0];
				for(int a0=0;a0<locAlleleList0.len();++a0){
					dataColumnRec& locAllCol0 = rawAnovaData->Nth(locallelecount0);
					dataColumnRec* locAllCol1 = locAllCol0.Duplicate();
					locusANOVAData.Append(locAllCol1);
					locallelecount0++;
					}//end for a0
				StringList thisLocusAlleleNames;
				GLM& linearModel0 = LinearModelList[vloc0];
				ObjList<aList<long double> >* locVarianceComponents =theTable.CalculateVarianceComponents(/* *this, */locusANOVAData, 
											factorList, thisLocusAlleleNames,/* 0, ran,*/ linearModel0);
				//now convert to variance components
				
				//transfer locus data to full list
				for(int vc0=0;vc0<locVarianceComponents->len();++vc0){
					aList<long double>* ABCvarList00 = &(*locVarianceComponents)[vc0];
					varianceComponents->Append(ABCvarList00);//now in both lists
					}//end for vc0
				locVarianceComponents->Detach();//removed from this list without destruction
				delete locVarianceComponents; locVarianceComponents = NULL;
				}//end for vloc0
		}//end if no missing data...else

	//store away the data and calculations
	int locallelecount=0;
	for(int vloc=0;vloc<variableLocusNumCodom_.len();++vloc){
		int loc = variableLocusNumCodom_[vloc];
		StringList& locAlleleList = allelesPresent_[loc];
		for(int a=0;a<locAlleleList.len();++a){
			long double A = (*varianceComponents)[0][locallelecount];
			long double B = (*varianceComponents)[1][locallelecount];
			long double C = (*varianceComponents)[2][locallelecount];

			vcAll_[m][loc][a][0] = A;
			vcAll_[m][loc][a][1] = B;
			vcAll_[m][loc][a][2] = C;
			vcAll_[m][loc][a][3] = A + B + C;
			ObjList<aList<long double> >& locAlleleFstat=FstatsAll_[m][loc];
			aList<long double>& alleleFstat =locAlleleFstat[a];
			if ((A + B + C) != 0.0) {
				aString levelName="Fst";
				int Flevel = Flevels_[m].Loc(&levelName);
				alleleFstat[Flevel]=A / (A + B + C);
				levelName="Fit";
				Flevel = Flevels_[m].Loc(&levelName);
				alleleFstat[Flevel]=1.0 - (C / (A + B + C));
				}//end if A+B+C
			if ((B + C) != 0.0) {
				aString levelName="Fis";
				int Flevel = Flevels_[m].Loc(&levelName);
				alleleFstat[Flevel]=1.0 - (C / (B + C));
				}//end if (B+C)
			//save these in an array for a quicker calculation of the jackknife variance estimates
			ALoc[loc] += A;
			BLoc[loc] += B;
			CLoc[loc] += C;
			BCLoc[loc]+= B+C;
			ABCLoc[loc] +=A+B+C;

			Asum += A;
			Bsum += B;
			Csum += C;
			BCsum += B + C;
			ABCsum += A + B + C;
			locallelecount++;
			}//end for a

		vcLoc_[m][loc][0] = ALoc[loc];
		vcLoc_[m][loc][1] = BLoc[loc];
		vcLoc_[m][loc][2] = CLoc[loc];
		vcLoc_[m][loc][3] = ABCLoc[loc];
		aList<long double>& locFstat = FstatsLoc_[m][loc];
		long double temp = ABCLoc[loc];
		if (temp != 0.0) {
			aString levelName="Fst";
			int Flevel = Flevels_[m].Loc(&levelName);
			locFstat[Flevel]=ALoc[loc] / temp;
			levelName="Fit";
			Flevel = Flevels_[m].Loc(&levelName);
			locFstat[Flevel]=1.0 - (CLoc[loc] / temp);
			}//end if temp
		temp= BCLoc[loc];
		if (temp != 0.0){
			aString levelName="Fis";
			int Flevel = Flevels_[m].Loc(&levelName);
			locFstat[Flevel]=1.0 - (CLoc[loc] / temp);
			}//end if temp
		}//end for j

	vc_[m][0] = Asum;
	vc_[m][1] = Bsum;
	vc_[m][2] = Csum;
	vc_[m][3] = ABCsum;
	if (ABCsum != 0.0){
		aString levelName="Fst";
		int Flevel = Flevels_[m].Loc(&levelName);
		Fstats_[m][Flevel]=(Asum / ABCsum);
		levelName="Fit";
		Flevel = Flevels_[m].Loc(&levelName);
		Fstats_[m][Flevel]=(1.0 - (Csum / ABCsum));
		}//end if ABCsum
	if (BCsum != 0.0){
		aString levelName="Fis";
		int Flevel = Flevels_[m].Loc(&levelName);
		Fstats_[m][Flevel]=(1.0 - (Csum / BCsum));
		}//end if BCSum

	//now do the jackknife calculations, over loci
	for (int j=0;j<variableLocusNumCodom_.len();++j){
		int loc = variableLocusNumCodom_[j];//the actual locus		
		aList<long double>& locJackFstat = jackFstatsLoc_[m][loc];
		long double temp = ABCLoc[loc];
		if ((ABCsum - temp) != 0.0) {
			aString levelName="Fst";
			int Flevel = Flevels_[m].Loc(&levelName);
			locJackFstat[Flevel]=((Asum - ALoc[loc]) / (ABCsum - temp));
			levelName="Fit";
			Flevel = Flevels_[m].Loc(&levelName);
			locJackFstat[Flevel]=1.0 - (Csum - CLoc[loc]) / (ABCsum - temp);
			}//end if ABCsum
		temp = BCLoc[loc];
		if (BCsum - temp != 0.0) {
			aString levelName="Fis";
			int Flevel = Flevels_[m].Loc(&levelName);
			locJackFstat[Flevel]=1.0 - (Csum - CLoc[loc]) / (BCsum - temp);
			}//end if BCSum
		}//end for j


	delete rawAnovaData; rawAnovaData=NULL;
	delete varianceComponents; varianceComponents=NULL;
	popStr=NULL; indivinpopSr=NULL;
	FstatsCurrent_=vcCurrent_=1;
	}//end  FstatisticAnalysisCodom::FstatsLinearAlg




void FstatisticAnalysisCodom::FstatsLinearAlgII(void){
	//uses linear algebra to get variance components for F-statistics
	//in a standard  nested design.  Coefficients of variance components estimated using simpler formulae
	
  	if(! FstatsCurrent_){
  		ClearFstats();
  		}
 	if((Fmethod_.Includes((int) linearAlgII))  ){return;}
	FstatsCurrent_=vcCurrent_=0;
	Fmethod_.Append((int) linearAlgII);
	int m = Fmethod_.len();
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, F-level
	Fstats_.Append(NEW aList<long double>());//F-level
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	Flevels_.Append(NEW StringList());//F-level

	vcMethod_.Append((int) linearAlgII);
	vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
	vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
	vc_.Append(NEW aList<long double>());//vc-level
	vcLevels_.Append(NEW StringList());//vc-level

	int lev = Flevels_.len()-1;
	CheckForNewAlleles();
	UpdateVariableLoci();
	//set up all the F-stat arrays
	// first the array describing the Fstat structure:
	aString *FitWord, *FisWord, *FstWord;
	FitWord=NEW aString("Fit");
	Flevels_[lev].Append(FitWord);
	FisWord=NEW aString("Fis");
	Flevels_[lev].Append(FisWord);
	FstWord=NEW aString("Fst");
	Flevels_[lev].Append(FstWord);

	aString* vcWord=NEW aString("var(p)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(i)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(g)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(t)");
	vcLevels_[m].Append(vcWord);
	//make and initialize the jackknife & by-locus Fstat lists
	for (int Flevel=0;Flevel<Flevels_[lev].len();++Flevel){Fstats_[lev].Append(0.0);}
	for (int vclevel=0;vclevel<4;++vclevel){vc_[m].Append(0.0);}
	//some intermediate variables
	for (int j=0;j<CodominantMarkerData::loci();++j){//make for all loci; replace values of variable ones later
		//make and initialize the jackknife & by-locus Fstat lists for this locus 
		aList<long double>* locFstat = NEW aList<long double>();
		FstatsLoc_[lev].Append(locFstat);
		aList<long double>* locJackFstat = NEW aList<long double>();
		jackFstatsLoc_[lev].Append(locJackFstat);
		for (int Flevel=0;Flevel<Flevels_[lev].len();++Flevel){
			locFstat->Append(0.0);
			locJackFstat->Append(0.0);
			}
		//make and initialize the jackknife & by-allele Fstat lists for this locus 
		ObjList<aList<long double> >* locAlleleFstat = NEW ObjList<aList<long double> >();
		FstatsAll_[lev].Append(locAlleleFstat);
		//initialize variables

		//make and initialize the by-allele F-stat list for each allele and append to this locus list
		StringList& locAlleleList = allelesPresent_[j];
		for (int a=0;a<locAlleleList.len();++a){
			aList<long double>* alleleFstat =NEW aList<long double>();
			locAlleleFstat->Append(alleleFstat);
			for (int Flevel=0;Flevel<Flevels_[lev].len();++Flevel){alleleFstat->Append(0.0);}
			alleleFstat = NULL;
			}//end for a	
		locFstat = locJackFstat= NULL;
		locAlleleFstat= NULL;			

		//make and initialize the by-allele var comp list
		vcLoc_[m].Append(NEW aList<long double>());
		for (int vclevel=0;vclevel<4;++vclevel){vcLoc_[m][j].Append(0.0);}
		vcAll_[m].Append(NEW ObjList<aList<long double> >());
		for (int a=0;a<locAlleleList.len();++a){
			vcAll_[m][j].Append(NEW aList<long double>());
			for (int vclevel=0;vclevel<4;++vclevel){vcAll_[m][j][a].Append(0.0);}
			}//end for a							
		}//end for j

	//the main body of the calculations
	
	ANOVAGenetics theTable;
	StringList locusAlleleNames;
	StringList factorList;
	aString* popStr = NEW aString("pop$");
	aString* indivinPopStr = NEW aString("indiv$ (pop$)");
	factorList.Append(popStr); 
	factorList.Append(indivinPopStr); 
	Ran3 ran(-1);//dummy
	//need to loop over missing data classes
	//this assumes no missing data
		


	dataSetRec* rawAnovaData =theTable.CreateRawANOVADataFormat(*this, locusAlleleNames);
	ObjList<aList<long double> > varianceComponents;
	MatrixList Zmatrices;
	if (! rawAnovaData->MissingData()){
		int loc = 0;//first locus=0; all loci the same
		aList<long double> nList, dfList;//these hold coefficients and degrees of freedom
		CalculateStdVarCompCoeffs(nList, dfList );//loc=0 and all have the same sample size
		//calculate Z matrices and then remainder sums of squares
		MatrixList ZLList0, ZIList;
		ObjList<MatrixList> ZSLList0, ZSIList;
		ObjList<ObjList<MatrixList> > ZSLIList;
		ZSLList0.Append(NEW MatrixList);
		ZSIList.Append(NEW MatrixList);
		ZSLIList.Append(NEW ObjList<MatrixList>());
		ZSLIList[0].Append(NEW MatrixList());
		int n0=0;
		for(int p=0;p<npop_;++p){
			int n00 = GetLocusSampleSize(p,loc);
			n0 += n00;
			ZSLList0[0].Append(OneMatrix((long double)1.0,n00,1));
			for(int i0=0;i0<individuals(p);++i0){
				int n000 = GetLocusSampleSize(p,i0,loc);
				if(n000>0){
					ZSLIList[0][p].Append(OneMatrix((long double)1.0,n000,1));}
				}//end for i0	
			ZSIList[0].Append(ZSLIList[0][p].DirectSum());
			}//end for p
		ZLList0.Append(ZSLList0[0].DirectSum());		
		ZIList.Append(ZSIList[0].DirectSum());		


		aMatrix<long double>* ZSL = ZLList0.DirectSum();
		Zmatrices.Append(ZSL);ZSL=NULL;
		aMatrix<long double>* ZI = ZIList.DirectSum();
		Zmatrices.Append(ZI);ZI=NULL;
		ZLList0.Clear();//free memory
		ZSLList0.Clear();//free memory
		ZSIList.Clear();//free memory
		ZSLIList.Clear();//free memory
		Zmatrices.Prepend(OneMatrix((long double)1.0,dfList.Last()+1,1));//one matrix of length n___

	//replicate and consolidate steps in GeneralLinearModel::varcompsANOVA
		matrixHandle<long double> invZprimeZH(NEW aMatrix<long double>());//initially empty; filled at first calculation
		matrixHandle<long double> invZ1primeZ1H(NEW aMatrix<long double>());//initially empty; filled at first calculation
		matrixHandle<long double> invZ1SprimeZ1SH(NEW aMatrix<long double>());//initially empty; filled at first calculation
		GLM* lm = NEW GLM();
		aList<int> omitItems;//when determining reduced models
		MatrixList* R_muab_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
		aMatrix<long double>* Z = lm->MakeDesignMatrix(*R_muab_DesignComponents, 0);//= full model
		delete R_muab_DesignComponents; R_muab_DesignComponents=NULL;
		MatrixList* R_mua_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
		omitItems.Append(2);//individuals omitted
		aMatrix<long double>* Z1S = lm->MakeDesignMatrix(*R_mua_DesignComponents, omitItems);//= reduced model
		delete R_mua_DesignComponents; R_mua_DesignComponents=NULL;
		MatrixList* R_mu_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
		omitItems.Append(1);//populations and individuals omitted
		aMatrix<long double>* Z1 = lm->MakeDesignMatrix(*R_mu_DesignComponents, omitItems);//= reduced model
		delete R_mu_DesignComponents; R_mu_DesignComponents=NULL;
		for(int locallele=0;locallele<locusAlleleNames.len();++locallele){
			aString& colTitle0 = locusAlleleNames[locallele];
			dataColumnRec& depVarColRec0 = rawAnovaData->GetDataColumn(colTitle0);//type conversion
			dataColumnRecNum<long double>* depVarColPtr0 = (dataColumnRecNum<long double>*) &depVarColRec0;
			dataColumnRecNum<long double>& depVarCol0 = *depVarColPtr0;
			aList<long double>& varList0 = depVarCol0.dataList();
			aMatrix<long double> Pmatrix;
			aVector<long double> aVecPtr0(varList0);
			Pmatrix.ConcatCol(aVecPtr0);

			long double R_mu, R_mua, R_muab;//sums of squares from including model factors
			R_mu = R_mua = R_muab = 0.0;
			long double SSE;  //dummy variable
			long double PprimeP;//total sum of squares
			//R_muab
			lm->RegressionSS(Z, &Pmatrix, R_muab, SSE, PprimeP, invZprimeZH);//invZprimeZH replaced if empty
			//R_mua
			lm->RegressionSS(Z1S, &Pmatrix, R_mua, SSE, PprimeP, invZ1SprimeZ1SH);//invZ1SprimeZ1SH replaced if empty
			//R_mu
			lm->RegressionSS(Z1, &Pmatrix, R_mu, SSE, PprimeP, invZ1primeZ1H);//invZ1primeZ1H replaced if empty
		
			long double SSLSP = R_mua - R_mu;
			long double SSI = R_muab - R_mua;
			long double SSG = PprimeP - R_muab;
		
			long double sigmaSqG = SSG/dfList[4];
			long double sigmaSqI = (  SSI - (dfList[3]*sigmaSqG)  ) / nList[0];
			long double sigmaSqSL = (  SSLSP - (nList[1] * sigmaSqI) - (dfList[2]*sigmaSqG)  ) / nList[2];
			long double sigmaSqT = PprimeP / dfList[5];//empirical total variance
			
			aList<long double>* vc = NEW aList<long double>();
			vc->Append(sigmaSqG);
			vc->Append(sigmaSqI);
			vc->Append(sigmaSqSL);
			vc->Append(sigmaSqT);
			
			varianceComponents.Append(vc);
			vc=NULL;//forget this
			}//end for locallele		


		delete Z; Z=NULL;//clear unused memory
		delete Z1S; Z1S=NULL;//clear unused memory
		delete Z1; Z1=NULL;//clear unused memory
		delete lm; lm=NULL;
			}//end if no missing data
		else{//must dissect rawANOVAData into separate loci
			//and run individually
			//make the above into a subfunction of this function and feed it locus lists of appropriately culled data
			//it should spit back variance component lists that can be concatenated
		}//end if no missing data...else


	//store away the data and calculations
	aList<long double> sigmaSL_Loc, sigmaI_Loc, sigmaG_Loc, sigmaIG_Loc, sigmaT_Loc;
	for(int loc0=0;loc0<CodominantMarkerData::loci();++loc0){
		sigmaSL_Loc.Append(0.0);
		sigmaI_Loc.Append(0.0);
		sigmaG_Loc.Append(0.0);
		sigmaIG_Loc.Append(0.0);
		sigmaT_Loc.Append(0.0);
		}//end for loc0
	long double sigmaSL_sum, sigmaI_sum, sigmaG_sum, sigmaIG_sum, sigmaT_sum;
	sigmaSL_sum = sigmaI_sum = sigmaG_sum = sigmaIG_sum = sigmaT_sum = 0.0;

	int locallelecount=0, Flevel;
	for(int vloc=0;vloc<variableLocusNumCodom_.len();++vloc){
		int loc = variableLocusNumCodom_[vloc];
		StringList& locAlleleList = allelesPresent_[loc];
		for(int a=0;a<locAlleleList.len();++a){
			long double sigmaG = varianceComponents[locallelecount][0];
			long double sigmaI = varianceComponents[locallelecount][1];
			long double sigmaLSP = varianceComponents[locallelecount][2];
			long double sigmaT = varianceComponents[locallelecount][3];//empirical value 
			vcAll_[m][loc][a][0] = sigmaLSP;
			vcAll_[m][loc][a][1] = sigmaI;
			vcAll_[m][loc][a][2] = sigmaG;
			vcAll_[m][loc][a][3] = sigmaT;
			if (sigmaT != 0.0) {
				Flevel = Flevels_[lev].Loc(FstWord);//Fst
				FstatsAll_[lev][loc][a][Flevel] = sigmaLSP /sigmaT;
				Flevel = Flevels_[lev].Loc(FitWord);//Fit
				FstatsAll_[lev][loc][a][Flevel]=1.0 - (sigmaG / sigmaT);
				}//end if sigmaT
			if ((sigmaI + sigmaG) != 0.0) {
				Flevel = Flevels_[lev].Loc(FisWord);//Fis
				FstatsAll_[lev][loc][a][Flevel] = 1.0 - (sigmaG / (sigmaI + sigmaG));
				}//end if (B+C)

			//save these in an array for a quicker calculation of the jackknife variance estimates
			sigmaSL_Loc[loc] += sigmaLSP;
			sigmaI_Loc[loc] += sigmaI;
			sigmaG_Loc[loc] += sigmaG;
			sigmaIG_Loc[loc] += sigmaI + sigmaG;
			sigmaT_Loc[loc] += sigmaT;

			sigmaSL_sum += sigmaLSP;
			sigmaI_sum += sigmaI;
			sigmaG_sum += sigmaG;
			sigmaIG_sum += sigmaI + sigmaG;
			sigmaT_sum += sigmaT;

			locallelecount++;
			}//end for a

		vcLoc_[m][loc][0] = sigmaSL_Loc[loc];
		vcLoc_[m][loc][1] = sigmaI_Loc[loc];
		vcLoc_[m][loc][2] = sigmaG_Loc[loc];
		vcLoc_[m][loc][3] = sigmaT_Loc[loc];
		if (sigmaT_Loc[loc] != 0.0) {
			Flevel = Flevels_[lev].Loc(FstWord);//Fst
			 FstatsLoc_[lev][loc][Flevel] = sigmaSL_Loc[loc] /sigmaT_Loc[loc];
			Flevel = Flevels_[lev].Loc(FitWord);//Fit
			 FstatsLoc_[lev][loc][Flevel]=1.0 - (sigmaG_Loc[loc] / sigmaT_Loc[loc]);
			}//end if sigmaT_Loc
		if (sigmaIG_Loc[loc] != 0.0){
			Flevel = Flevels_[lev].Loc(FisWord);//Fis
			 FstatsLoc_[lev][loc][Flevel] = 1.0 - (sigmaG_Loc[loc] /sigmaIG_Loc[loc]);
			}//end if sigmaIG_Loc
		}//end for j

	vc_[m][0] = sigmaSL_sum;
	vc_[m][1] = sigmaI_sum;
	vc_[m][2] = sigmaIG_sum;
	vc_[m][3] = sigmaT_sum;
	if (sigmaT_sum != 0.0){
		Flevel = Flevels_[lev].Loc(FstWord);//Fst
		Fstats_[lev][Flevel] = sigmaSL_sum /sigmaT_sum;
		Flevel = Flevels_[lev].Loc(FitWord);//Fit
		Fstats_[lev][Flevel]=1.0 - (sigmaG_sum / sigmaT_sum);
		}//end if sigmaT_sum
	if (sigmaIG_sum != 0.0){
		Flevel = Flevels_[lev].Loc(FisWord);//Fis
		Fstats_[lev][Flevel] = 1.0 - (sigmaG_sum /sigmaIG_sum);
		}//end if sigmaIG_sum

	//now do the jackknife calculations, over loci
	for (int j=0;j<variableLocusNumCodom_.len();++j){//skip monomorphic loci
		int loc = variableLocusNumCodom_[j];//the actual locus		
		if ((sigmaT_sum - sigmaT_Loc[loc]) != 0.0) {
			Flevel = Flevels_[lev].Loc(FstWord);//Fst
			jackFstatsLoc_[lev][loc][Flevel] = (sigmaSL_sum - sigmaSL_Loc[loc]) /(sigmaT_sum - sigmaT_Loc[loc]);
			Flevel = Flevels_[lev].Loc(FitWord);//Fit
			jackFstatsLoc_[lev][loc][Flevel]=1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) / (sigmaT_sum - sigmaT_Loc[loc]));
			}//end if sigmaT_sum
		if (sigmaIG_sum - sigmaIG_Loc[loc] != 0.0) {
			Flevel = Flevels_[lev].Loc(FisWord);//Fis
			jackFstatsLoc_[lev][loc][Flevel] = 1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) /(sigmaIG_sum - sigmaIG_Loc[loc]));
			}//end if sigmaIG_sum
		}//end for j

	FstatsCurrent_=vcCurrent_=1;
	FitWord=FstWord=FisWord=NULL;//forget these; deleted elsewhere
	
	popStr = indivinPopStr = NULL;
	delete rawAnovaData; rawAnovaData=NULL;
	popStr = indivinPopStr = NULL;
	}//end  FstatisticAnalysisCodom::FstatsLinearAlgII



void FstatisticAnalysisCodom::FstatsIntrogression(ObjList<GLM>& LinearModelList, int includeInteraction){
	//uses linear algebra to get variance components for F-statistics
	//in a factorial, nested design
	//clear out any previous calculations
  	if(! FstatsCurrent_){
  		ClearFstats();}
	if((Fmethod_.Includes((int) linearIntrog))&&(includeInteraction==1)){return;}
	if((Fmethod_.Includes((int) linearIntrogNoInt))&&(includeInteraction==0)){return;}
	if(includeInteraction==0){Fmethod_.Append((int) linearIntrogNoInt);}
	if(includeInteraction==1){Fmethod_.Append((int) linearIntrog);}
	FstatsCurrent_=vcCurrent_=0;
	int m = Fmethod_.len()-1;
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, F-level
	Fstats_.Append(NEW aList<long double>());//F-level
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	Flevels_.Append(NEW StringList());//F-level

	if(includeInteraction==0){vcMethod_.Append((int) linearIntrogNoInt);}
	if(includeInteraction==1){vcMethod_.Append((int) linearIntrog);}
	vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
	vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
	vc_.Append(NEW aList<long double>());//vc-level
	vcLevels_.Append(NEW StringList());//vc-level

	//set up all the F-stat arrays
	// first the array describing the Fstat structure:
	aString* FitWord=NEW aString("Fit");
	Flevels_[m].Append(FitWord);
	aString* FisWord=NEW aString("Fis");
	Flevels_[m].Append(FisWord);
	aString* FstWord=NEW aString("Fst");
	Flevels_[m].Append(FstWord);
	aString* FltWord=NEW aString("Flt");
	Flevels_[m].Append(FltWord);
	aString* FsptWord=NEW aString("Fspt");
	Flevels_[m].Append(FsptWord);
	aString* FlsptWord=NEW aString("Flspt");//make, but only use if needed
	if (includeInteraction){
		Flevels_[m].Append(FlsptWord);}

	aString* vcWord=NEW aString("var(Sp)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(L)");
	vcLevels_[m].Append(vcWord);
	aString* varLSpWord=NEW aString("var(LSp)");//make, but only use if needed
	if (includeInteraction){
		vcLevels_[m].Append(varLSpWord);}
	vcWord=NEW aString("var(i)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(g)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(t)");
	vcLevels_[m].Append(vcWord);
	int vclevels=vcLevels_[m].len();
	//make and initialize the jackknife & by-locus Fstat lists
	for (int Flevel=0;Flevel<Flevels_.len();++Flevel){Fstats_[m].Append(0.0);}
	for (int vclevel=0;vclevel<vclevels;++vclevel){vc_[m].Append(0.0);}
	//some intermediate variables
	aList<long double> sigmaL_Loc, sigmaS_Loc, sigmaSL_Loc, sigmaI_Loc, sigmaG_Loc, sigmaIG_Loc, sigmaT_Loc;
	CheckForNewAlleles();
	UpdateVariableLoci();
//	for (int jj=0;jj<variableLoci_.len();++jj){
	for (int j=0;j<CodominantMarkerData::loci();++j){//make for all loci; replace values of variable ones later
//		int j=variableLoci_[jj];
		//make and initialize the jackknife & by-locus Fstat lists for this locus 
		aList<long double>* locFstat = NEW aList<long double>();
		FstatsLoc_[m].Append(locFstat);
		aList<long double>* locJackFstat = NEW aList<long double>();
		jackFstatsLoc_[m].Append(locJackFstat);
		for (int Flevel=0;Flevel<Flevels_.len();++Flevel){
			locFstat->Append(0.0);
			locJackFstat->Append(0.0);
			}
		//make and initialize the jackknife & by-allele Fstat lists for this locus 
		ObjList<aList<long double> >* locAlleleFstat = NEW ObjList<aList<long double> >();
		FstatsAll_[m].Append(locAlleleFstat);
		//initialize variables
		sigmaS_Loc.Append(0.0);
		sigmaL_Loc.Append(0.0);
		sigmaSL_Loc.Append(0.0);
		sigmaI_Loc.Append(0.0);
		sigmaG_Loc.Append(0.0);
		sigmaIG_Loc.Append(0.0);
		sigmaT_Loc.Append(0.0);

		//make and initialize the by-allele F-stat list for each allele and append to this locus list
		StringList& locAlleleList = allelesPresent_[j];
		for (int a=0;a<locAlleleList.len();++a){
			aList<long double>* alleleFstat =NEW aList<long double>();
			locAlleleFstat->Append(alleleFstat);
			for (int Flevel=0;Flevel<Flevels_.len();++Flevel){alleleFstat->Append(0.0);}
			alleleFstat = NULL;
			}//end for a	
		locFstat = locJackFstat= NULL;
		locAlleleFstat= NULL;			

		//make and initialize the by-allele var comp list
		vcLoc_[m].Append(NEW aList<long double>());
		for (int vclevel=0;vclevel<vclevels;++vclevel){vcLoc_[m][j].Append(0.0);}
		vcAll_[m].Append(NEW ObjList<aList<long double> >());
		for (int a=0;a<locAlleleList.len();++a){
			vcAll_[m][j].Append(NEW aList<long double>());
			for (int vclevel=0;vclevel<vclevels;++vclevel){vcAll_[m][j][a].Append(0.0);}
			}//end for a							
		}//end for j

	//the main body of the calculations
	
	long double sigmaL_sum, sigmaS_sum, sigmaSL_sum, sigmaI_sum, sigmaG_sum, sigmaIG_sum, sigmaT_sum;
	sigmaL_sum = sigmaS_sum = sigmaSL_sum = sigmaI_sum = sigmaG_sum = sigmaIG_sum = sigmaT_sum = 0.0;
	ANOVAGenetics theTable;
	StringList locusAlleleNames;
	StringList factorList;
	aString* speciesStr = NEW aString("group$");
	aString* localityStr = NEW aString("locality$");
	aString* popStr = NEW aString("group$ x locality$");
	aString* indivinPopStr = NEW aString("indiv$ (group$ x locality$)");
	factorList.Append(speciesStr); //don't switch this order!!
	factorList.Append(localityStr); 
	if(includeInteraction){
		factorList.Append(popStr); }
	factorList.Append(indivinPopStr); 
	Ran3 ran(-1);//dummy
	//need to loop over missing data classes
	//this assumes no missing data


	dataSetRec* rawAnovaData =theTable.CreateRawANOVADataFormat(*this, locusAlleleNames);
	aString checkDataName("Ymatrices");
	rawAnovaData->Print(checkDataName);
	ObjList<aList<long double> >* varianceComponents = NULL;
	if (! rawAnovaData->MissingData()){
			if(LinearModelList.len()==0){//first time through
				GLM* lm = NEW GLM();
				lm->SetToReuseDesign(1);//true; reuse for each locus and allele
				LinearModelList.Append(lm);
				lm = NULL;//forget
				}
			GLM& linearModel = LinearModelList[0];
			varianceComponents =theTable.CalculateVarianceComponents(/**this, */*rawAnovaData, 
											factorList, locusAlleleNames,/* 0, ran,*/ linearModel);

			}//end if no missing data
		else{//must dissect rawANOVAData into separate loci
			//and run individually
/*			varianceComponents = new ObjList<aList<double> >();
			for (int f=0;f<=factorList.len();++f){
				aList<double>* ABCvarList = NEW aList<double>();
				meanSquares->Append(ABCvarList);	}
			int locallelecount0=3;//column of rawANOVAData holding correct locus and allele
			for(int vloc0=0;vloc0<variableLoci_.len();++vloc0){
				dataSetRec locusANOVAData;//work with copies of columns from rawANOVAData
				dataColumnRec& grCol0 = rawAnovaData->First();
				dataColumnRec& popCol0 = rawAnovaData->Nth(1);
				dataColumnRec& indivCol0 = rawAnovaData->Nth(2);
				dataColumnRec* grCol1 = grCol0.Clone();
				dataColumnRec* popCol1 = popCol0.Clone();
				dataColumnRec* indivCol1 = indivCol0.Clone();
				locusANOVAData.Append(grCol1);
				locusANOVAData.Append(popCol1);
				locusANOVAData.Append(indivCol1);
				int loc0 = variableLoci_[vloc0];
				StringList& locAlleleList0 = allelesPresent_[loc0];
				for(int a0=0;a0<locAlleleList0.len();++a0){
					dataColumnRec& locAllCol0 = rawAnovaData->Nth(locallelecount0);
					dataColumnRec* locAllCol1 = locAllCol0.Clone();
					locusANOVAData.Append(locAllCol1);
					locallelecount0++;
					}//end for a0
				StringList thisLocusAlleleNames;
				ObjList<aList<double> >* locMeanSquares =theTable.CalculateMeanSquares(*this, locusANOVAData, 
											factorList, thisLocusAlleleNames, 0, ran);
				//now convert to variance components
				
				//transfer locus data to full list
				for(int a00=0;a00<thisLocusAlleleNames.len();++a00){
					aString* locAllNam00 = NEW aString(thisLocusAlleleNames[a00]);
					locusAlleleNames.Append(locAllNam00);
					for(int vc0=0;vc0<meanSquares->len();++vc0){
						aList<double>& ABCvarList00 = (*meanSquares)[vc0];
						aList<double>& ABCvarList01 = (*locMeanSquares)[vc0];
						ABCvarList00.Append(ABCvarList01[a00]);}
					}//end for a00
				delete locMeanSquares; locMeanSquares = NULL;
				}//end for vloc0
*/
		}//end if no missing data...else


	//store away the data and calculations
	int locallelecount=0;
	for(int vloc=0;vloc<variableLocusNumCodom_.len();++vloc){
		int loc = variableLocusNumCodom_[vloc];
		StringList& locAlleleList = allelesPresent_[loc];
		for(int a=0;a<locAlleleList.len();++a){
			long double sigmaSP = (*varianceComponents)[0][locallelecount];
			long double sigmaL = (*varianceComponents)[1][locallelecount];
			long double sigmaLSP = 0.0, sigmaI, sigmaG,sigmaT;
			if(includeInteraction){
					sigmaLSP = (*varianceComponents)[2][locallelecount];
					sigmaI = (*varianceComponents)[3][locallelecount];
					sigmaG = (*varianceComponents)[4][locallelecount];
					sigmaT = sigmaSP+sigmaL+sigmaLSP+sigmaI+sigmaG;
					}
				else{
					sigmaI = (*varianceComponents)[2][locallelecount];
					sigmaG = (*varianceComponents)[3][locallelecount];
					sigmaT = sigmaSP+sigmaL+sigmaLSP+sigmaI+sigmaG;
					}

			vcAll_[m][loc][a][0] = sigmaSP;
			vcAll_[m][loc][a][1] = sigmaL;
			if(includeInteraction){
					vcAll_[m][loc][a][2] = sigmaLSP;
					vcAll_[m][loc][a][3] = sigmaI;
					vcAll_[m][loc][a][4] = sigmaG;
					vcAll_[m][loc][a][5] = sigmaT;
					}
				else{
					vcAll_[m][loc][a][2] = sigmaI;
					vcAll_[m][loc][a][3] = sigmaG;
					vcAll_[m][loc][a][4] = sigmaT;
					}			
			ObjList<aList<long double> >& locAlleleFstat=FstatsAll_[m][loc];
			aList<long double>& alleleFstat =locAlleleFstat[a];
			if (sigmaT != 0.0) {
				int Flevel = Flevels_[m].Loc(FsptWord);//Fspt
				alleleFstat[Flevel] = sigmaSP / sigmaT;
				Flevel = Flevels_[m].Loc(FltWord);//Flt
				alleleFstat[Flevel] = sigmaL / sigmaT;
				if(includeInteraction){
					Flevel = Flevels_[m].Loc(FlsptWord);//Fst
					alleleFstat[Flevel] = sigmaLSP /sigmaT;}
				Flevel = Flevels_[m].Loc(FitWord);//Fit
				alleleFstat[Flevel]=1.0 - (sigmaG / sigmaT);
				Flevel = Flevels_[m].Loc(FstWord);//Fst
				alleleFstat[Flevel]= (sigmaSP + sigmaL +  sigmaLSP)/ sigmaT;
				}//end if sigmaT
			if ((sigmaI + sigmaG) != 0.0) {
				int Flevel = Flevels_[m].Loc(FisWord);//Fis
				alleleFstat[Flevel] = 1.0 - (sigmaG / (sigmaI + sigmaG));
				}//end if (B+C)

			//save these in an array for a quicker calculation of the jackknife variance estimates
			sigmaS_Loc[loc] += sigmaSP;
			sigmaL_Loc[loc] += sigmaL;
			sigmaSL_Loc[loc] += sigmaLSP;
			sigmaI_Loc[loc] += sigmaI;
			sigmaG_Loc[loc] += sigmaG;
			sigmaIG_Loc[loc] += sigmaI + sigmaG;
			sigmaT_Loc[loc] += sigmaT;

			sigmaS_sum += sigmaSP;
			sigmaL_sum += sigmaL;
			sigmaSL_sum += sigmaLSP;
			sigmaI_sum += sigmaI;
			sigmaG_sum += sigmaG;
			sigmaIG_sum += sigmaI + sigmaG;
			sigmaT_sum += sigmaT;

			locallelecount++;
			}//end for a

		vcLoc_[m][loc][0] = sigmaS_Loc[loc];
		vcLoc_[m][loc][1] = sigmaL_Loc[loc];
		if(includeInteraction){
				vcLoc_[m][loc][2] = sigmaSL_Loc[loc];
				vcLoc_[m][loc][3] = sigmaI_Loc[loc];
				vcLoc_[m][loc][4] = sigmaG_Loc[loc];
				vcLoc_[m][loc][5] = sigmaT_Loc[loc];
				}
			else{
				vcLoc_[m][loc][2] = sigmaI_Loc[loc];
				vcLoc_[m][loc][3] = sigmaG_Loc[loc];
				vcLoc_[m][loc][4] = sigmaT_Loc[loc];
				}			
		aList<long double>& locFstat = FstatsLoc_[m][loc];
		if (sigmaT_Loc[loc] != 0.0) {
			int Flevel = Flevels_[m].Loc(FsptWord);//Fspt
			locFstat[Flevel] =sigmaS_Loc[loc]/ sigmaT_Loc[loc];
			Flevel = Flevels_[m].Loc(FltWord);//Flt
			locFstat[Flevel] = sigmaL_Loc[loc] / sigmaT_Loc[loc];
			if(includeInteraction){
				Flevel = Flevels_[m].Loc(FlsptWord);//Fst
				locFstat[Flevel] = sigmaSL_Loc[loc] /sigmaT_Loc[loc];}
			Flevel = Flevels_[m].Loc(FitWord);//Fit
			locFstat[Flevel]=1.0 - (sigmaG_Loc[loc] / sigmaT_Loc[loc]);
			Flevel = Flevels_[m].Loc(FstWord);//Fst
			locFstat[Flevel]=(sigmaS_Loc[loc] + sigmaL_Loc[loc] + sigmaSL_Loc[loc])/ sigmaT_Loc[loc];
			}//end if sigmaT_Loc
		if (sigmaIG_Loc[loc] != 0.0){
			int Flevel = Flevels_[m].Loc(FisWord);//Fis
			locFstat[Flevel] = 1.0 - (sigmaG_Loc[loc] /sigmaIG_Loc[loc]);
			}//end if sigmaIG_Loc
		}//end for j

	vc_[m][0] = sigmaS_sum;
	vc_[m][1] = sigmaL_sum;
	if(includeInteraction){
			vc_[m][2] = sigmaSL_sum;
			vc_[m][3] = sigmaI_sum;
			vc_[m][4] = sigmaG_sum;
			vc_[m][5] = sigmaT_sum;
			}
		else{
			vc_[m][2] = sigmaI_sum;
			vc_[m][3] = sigmaG_sum;
			vc_[m][4] = sigmaT_sum;
			}			
	if (sigmaT_sum != 0.0){
		int Flevel = Flevels_[m].Loc(FsptWord);//Fspt
		Fstats_[m][Flevel] = sigmaS_sum / sigmaT_sum;
		Flevel = Flevels_[m].Loc(FltWord);//Flt
		Fstats_[m][Flevel] = sigmaL_sum / sigmaT_sum;
		if(includeInteraction){
			Flevel = Flevels_[m].Loc(FlsptWord);//Fst
			Fstats_[m][Flevel] = sigmaSL_sum /sigmaT_sum;}
		Flevel = Flevels_[m].Loc(FitWord);//Fit
		Fstats_[m][Flevel]=1.0 - (sigmaG_sum / sigmaT_sum);
		Flevel = Flevels_[m].Loc(FstWord);//Fst
		Fstats_[m][Flevel]=(sigmaS_sum + sigmaL_sum + sigmaSL_sum) / sigmaT_sum;
		}//end if sigmaT_sum
	if (sigmaIG_sum != 0.0){
		int Flevel = Flevels_[m].Loc(FisWord);//Fis
		Fstats_[m][Flevel] = 1.0 - (sigmaG_sum /sigmaIG_sum);
		}//end if sigmaIG_sum

	//now do the jackknife calculations, over loci
	for (int j=0;j<variableLocusNumCodom_.len();++j){//skip monomorphic loci
		int loc = variableLocusNumCodom_[j];//the actual locus		
		aList<long double>& locJackFstat = jackFstatsLoc_[m][loc];
		if ((sigmaT_sum - sigmaT_Loc[loc]) != 0.0) {
			int Flevel = Flevels_[m].Loc(FsptWord);//Fspt
			locJackFstat[Flevel] =(sigmaS_sum - sigmaS_Loc[loc])/ (sigmaT_sum - sigmaT_Loc[loc]);
			Flevel = Flevels_[m].Loc(FltWord);//Flt
			locJackFstat[Flevel] = (sigmaL_sum - sigmaL_Loc[loc])/ (sigmaT_sum - sigmaT_Loc[loc]);
			if(includeInteraction){
				Flevel = Flevels_[m].Loc(FlsptWord);//Flspt
				locJackFstat[Flevel] = (sigmaSL_sum - sigmaSL_Loc[loc]) /(sigmaT_sum - sigmaT_Loc[loc]);}
			Flevel = Flevels_[m].Loc(FitWord);//Fit
			locJackFstat[Flevel]=1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) / (sigmaT_sum - sigmaT_Loc[loc]));
			Flevel = Flevels_[m].Loc(FstWord);//Fst
			locJackFstat[Flevel]=((sigmaS_sum+sigmaL_sum+sigmaSL_sum) - (sigmaS_Loc[loc]+sigmaL_Loc[loc]+sigmaSL_Loc[loc]))
							 / (sigmaT_sum - sigmaT_Loc[loc]);
			}//end if sigmaT_sum
		if (sigmaIG_sum - sigmaIG_Loc[loc] != 0.0) {
			int Flevel = Flevels_[m].Loc(FisWord);//Fis
			locJackFstat[Flevel] = 1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) /(sigmaIG_sum - sigmaIG_Loc[loc]));
			}//end if BCSum
		}//end for j

	FstatsCurrent_=vcCurrent_=1;
	FsptWord=FltWord=FlsptWord=FitWord=FstWord=FisWord=NULL;//forget these; deleted elsewhere
	
	speciesStr = localityStr = popStr = indivinPopStr = NULL;
	delete rawAnovaData; rawAnovaData=NULL;
	delete varianceComponents; varianceComponents=NULL;
	speciesStr = localityStr = popStr = indivinPopStr = NULL;
	}//end  FstatisticAnalysisCodom::FstatsIntrogression




void FstatisticAnalysisCodom::FstatsIntrogression(int includeInteraction){
	//uses linear algebra to get variance components for F-statistics
	//in a factorial, nested design
	ObjList<GLM> LinearModelList;
	FstatsIntrogression(LinearModelList, includeInteraction);
	}//end  FstatisticAnalysisCodom::FstatsIntrogression




void FstatisticAnalysisCodom::FactorialFstats(void){
	//uses linear algebra to get variance components for F-statistics
	//in a factorial, nested design
	ObjList<GLM> LinearModelList;
	FactorialFstats(LinearModelList, 1);
	}//end  FstatisticAnalysisCodom::FactorialFstats()




void FstatisticAnalysisCodom::FactorialFstats(ObjList<GLM>& LinearModelList, int includeInteraction){
	//uses linear algebra to get variance components for F-statistics
	//in a factorial, nested design.  Coefficients of variance components estimated using simpler formulae
	//clear out any previous calculations
	
	//assumes data previously sorted by species, then locality
  	if(! FstatsCurrent_){
  		ClearFstats();}
  	if((Fmethod_.Includes((int) factorialFstats))){return;}
	FstatsCurrent_=vcCurrent_=0;
	Fmethod_.Append((int) factorialFstats);
	int m = Fmethod_.len()-1;
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, F-level
	Fstats_.Append(NEW aList<long double>());//F-level
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	Flevels_.Append(NEW StringList());//F-level

	vcMethod_.Append((int) factorialFstats);
	vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
	vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
	vc_.Append(NEW aList<long double>());//vc-level
	vcLevels_.Append(NEW StringList());//vc-level

	//set up all the F-stat arrays
	// first the array describing the Fstat structure:
	aString* FitWord=NEW aString("Fit");
	Flevels_[m].Append(FitWord);
	aString* FisWord=NEW aString("Fis");
	Flevels_[m].Append(FisWord);
	aString* FstWord=NEW aString("Fst");
	Flevels_[m].Append(FstWord);
	aString* FltWord=NEW aString("Flt");
	Flevels_[m].Append(FltWord);
	aString* FsptWord=NEW aString("Fspt");
	Flevels_[m].Append(FsptWord);
	aString* FlsptWord=NEW aString("Flspt");
	Flevels_[m].Append(FlsptWord);

	aString* vcWord=NEW aString("var(Sp)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(L)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(LSp)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(i)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(g)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(t)");
	vcLevels_[m].Append(vcWord);
	int vclevels=vcLevels_[m].len();
	//make and initialize the jackknife & by-locus Fstat lists
	for (int Flevel=0;Flevel<Flevels_.len();++Flevel){Fstats_[m].Append(0.0);}
	for (int vclevel=0;vclevel<vclevels;++vclevel){vc_[m].Append(0.0);}
	//some intermediate variables
	aList<long double> sigmaL_Loc, sigmaS_Loc, sigmaSL_Loc, sigmaI_Loc, sigmaG_Loc, sigmaIG_Loc, sigmaT_Loc;
	CheckForNewAlleles();
	UpdateVariableLoci();
	for (int j=0;j<CodominantMarkerData::loci();++j){//make for all loci; replace values of variable ones later
		//make and initialize the jackknife & by-locus Fstat lists for this locus 
		aList<long double>* locFstat = NEW aList<long double>();
		FstatsLoc_[m].Append(locFstat);
		aList<long double>* locJackFstat = NEW aList<long double>();
		jackFstatsLoc_[m].Append(locJackFstat);
		for (int Flevel=0;Flevel<Flevels_.len();++Flevel){
			locFstat->Append(0.0);
			locJackFstat->Append(0.0);
			}
		//make and initialize the jackknife & by-allele Fstat lists for this locus 
		ObjList<aList<long double> >* locAlleleFstat = NEW ObjList<aList<long double> >();
		FstatsAll_[m].Append(locAlleleFstat);
		//initialize variables
		sigmaS_Loc.Append(0.0);
		sigmaL_Loc.Append(0.0);
		sigmaSL_Loc.Append(0.0);
		sigmaI_Loc.Append(0.0);
		sigmaG_Loc.Append(0.0);
		sigmaIG_Loc.Append(0.0);
		sigmaT_Loc.Append(0.0);

		//make and initialize the by-allele F-stat list for each allele and append to this locus list
		StringList& locAlleleList = allelesPresent_[j];
		for (int a=0;a<locAlleleList.len();++a){
			aList<long double>* alleleFstat =NEW aList<long double>();
			locAlleleFstat->Append(alleleFstat);
			for (int Flevel=0;Flevel<Flevels_.len();++Flevel){alleleFstat->Append(0.0);}
			alleleFstat = NULL;
			}//end for a	
		locFstat = locJackFstat= NULL;
		locAlleleFstat= NULL;			

		//make and initialize the by-allele var comp list
		vcLoc_[m].Append(NEW aList<long double>());
		for (int vclevel=0;vclevel<vclevels;++vclevel){vcLoc_[m][j].Append(0.0);}
		vcAll_[m].Append(NEW ObjList<aList<long double> >());
		for (int a=0;a<locAlleleList.len();++a){
			vcAll_[m][j].Append(NEW aList<long double>());
			for (int vclevel=0;vclevel<vclevels;++vclevel){vcAll_[m][j][a].Append(0.0);}
			}//end for a							
		}//end for j

	//the main body of the calculations
	
	long double sigmaL_sum, sigmaS_sum, sigmaSL_sum, sigmaI_sum, sigmaG_sum, sigmaIG_sum, sigmaT_sum;
	sigmaL_sum = sigmaS_sum = sigmaSL_sum = sigmaI_sum = sigmaG_sum = sigmaIG_sum = sigmaT_sum = 0.0;
	ANOVAGenetics theTable;
	StringList locusAlleleNames;
	StringList factorList;
	aString* speciesStr = NEW aString("group$");
	aString* localityStr = NEW aString("locality$");
	aString* popStr = NEW aString("group$ x locality$");
	aString* indivinPopStr = NEW aString("indiv$ (group$ x locality$)");
	factorList.Append(speciesStr); //don't switch this order!!
	factorList.Append(localityStr); 
	if(includeInteraction){
		factorList.Append(popStr); }
	factorList.Append(indivinPopStr); 
	Ran3 ran(-1);//dummy
	//need to loop over missing data classes
	//this assumes no missing data


	dataSetRec* rawAnovaData =theTable.CreateRawANOVADataFormat(*this, locusAlleleNames);
	aString checkDataName("Ymatrices");
	rawAnovaData->Print(checkDataName);
	ObjList<aList<long double> >* varianceComponents = NULL;
	if (! rawAnovaData->MissingData()){
			if(LinearModelList.len()==0){//first time through
				GLM* lm = NEW GLM();
				lm->SetToReuseDesign(1);//true; reuse for each locus and allele
				LinearModelList.Append(lm);
				CalculateFactorialVarCompCoeffs(*lm/*,0*/);
				lm = NULL;//forget
				}//end if first time through
			GLM& linearModel = LinearModelList[0];
			varianceComponents =theTable.CalculateVarianceComponents(/* *this,*/ *rawAnovaData, 
											factorList, locusAlleleNames, /*0, ran,*/ linearModel);

			}//end if no missing data
		else{//must dissect rawANOVAData into separate loci
			//and run individually
/*			varianceComponents = NEW ObjList<aList<double> >();
			for (int f=0;f<=factorList.len();++f){
				aList<double>* ABCvarList = NEW aList<double>();
				meanSquares->Append(ABCvarList);	}
			int locallelecount0=3;//column of rawANOVAData holding correct locus and allele
			for(int vloc0=0;vloc0<variableLoci_.len();++vloc0){
				dataSetRec locusANOVAData;//work with copies of columns from rawANOVAData
				dataColumnRec& grCol0 = rawAnovaData->First();
				dataColumnRec& popCol0 = rawAnovaData->Nth(1);
				dataColumnRec& indivCol0 = rawAnovaData->Nth(2);
				dataColumnRec* grCol1 = grCol0.Clone();
				dataColumnRec* popCol1 = popCol0.Clone();
				dataColumnRec* indivCol1 = indivCol0.Clone();
				locusANOVAData.Append(grCol1);
				locusANOVAData.Append(popCol1);
				locusANOVAData.Append(indivCol1);
				int loc0 = variableLoci_[vloc0];
				StringList& locAlleleList0 = allelesPresent_[loc0];
				for(int a0=0;a0<locAlleleList0.len();++a0){
					dataColumnRec& locAllCol0 = rawAnovaData->Nth(locallelecount0);
					dataColumnRec* locAllCol1 = locAllCol0.Clone();
					locusANOVAData.Append(locAllCol1);
					locallelecount0++;
					}//end for a0
				StringList thisLocusAlleleNames;
				ObjList<aList<double> >* locMeanSquares =theTable.CalculateMeanSquares(*this, locusANOVAData, 
											factorList, thisLocusAlleleNames, 0, ran);
				//now convert to variance components
				
				//transfer locus data to full list
				for(int a00=0;a00<thisLocusAlleleNames.len();++a00){
					aString* locAllNam00 = NEW aString(thisLocusAlleleNames[a00]);
					locusAlleleNames.Append(locAllNam00);
					for(int vc0=0;vc0<meanSquares->len();++vc0){
						aList<double>& ABCvarList00 = (*meanSquares)[vc0];
						aList<double>& ABCvarList01 = (*locMeanSquares)[vc0];
						ABCvarList00.Append(ABCvarList01[a00]);}
					}//end for a00
				delete locMeanSquares; locMeanSquares = NULL;
				}//end for vloc0
*/
		}//end if no missing data...else


	//store away the data and calculations
	int locallelecount=0;
	for(int vloc=0;vloc<variableLocusNumCodom_.len();++vloc){
		int loc = variableLocusNumCodom_[vloc];
		StringList& locAlleleList = allelesPresent_[loc];
		for(int a=0;a<locAlleleList.len();++a){
			long double sigmaSP = (*varianceComponents)[0][locallelecount];
			long double sigmaL = (*varianceComponents)[1][locallelecount];
			long double sigmaLSP = 0.0, sigmaI, sigmaG,sigmaT;
			if(includeInteraction){
					sigmaLSP = (*varianceComponents)[2][locallelecount];
					sigmaI = (*varianceComponents)[3][locallelecount];
					sigmaG = (*varianceComponents)[4][locallelecount];
					sigmaT = sigmaSP+sigmaL+sigmaLSP+sigmaI+sigmaG;
					}
				else{
					sigmaI = (*varianceComponents)[2][locallelecount];
					sigmaG = (*varianceComponents)[3][locallelecount];
					sigmaT = sigmaSP+sigmaL+sigmaLSP+sigmaI+sigmaG;
					}
			vcAll_[m][loc][a][0] = sigmaSP;
			vcAll_[m][loc][a][1] = sigmaL;
			vcAll_[m][loc][a][2] = sigmaLSP;
			vcAll_[m][loc][a][3] = sigmaI;
			vcAll_[m][loc][a][4] = sigmaG;
			vcAll_[m][loc][a][5] = sigmaT;

			ObjList<aList<long double> >& locAlleleFstat=FstatsAll_[m][loc];
			aList<long double>& alleleFstat =locAlleleFstat[a];
			if (sigmaT != 0.0) {
				int Flevel = Flevels_[m].Loc(FsptWord);//Fspt
				alleleFstat[Flevel] = sigmaSP / sigmaT;
				Flevel = Flevels_[m].Loc(FltWord);//Flt
				alleleFstat[Flevel] = sigmaL / sigmaT;
				if(includeInteraction){
					Flevel = Flevels_[m].Loc(FlsptWord);//Fst
					alleleFstat[Flevel] = sigmaLSP /sigmaT;}
				Flevel = Flevels_[m].Loc(FitWord);//Fit
				alleleFstat[Flevel]=1.0 - (sigmaG / sigmaT);
				Flevel = Flevels_[m].Loc(FstWord);//Fst
				alleleFstat[Flevel]= (sigmaSP + sigmaL +  sigmaLSP)/ sigmaT;
				}//end if sigmaT
			if ((sigmaI + sigmaG) != 0.0) {
				int Flevel = Flevels_[m].Loc(FisWord);//Fis
				alleleFstat[Flevel] = 1.0 - (sigmaG / (sigmaI + sigmaG));
				}//end if (B+C)

			//save these in an array for a quicker calculation of the jackknife variance estimates
			sigmaS_Loc[loc] += sigmaSP;
			sigmaL_Loc[loc] += sigmaL;
			sigmaSL_Loc[loc] += sigmaLSP;
			sigmaI_Loc[loc] += sigmaI;
			sigmaG_Loc[loc] += sigmaG;
			sigmaIG_Loc[loc] += sigmaI + sigmaG;
			sigmaT_Loc[loc] += sigmaT;

			sigmaS_sum += sigmaSP;
			sigmaL_sum += sigmaL;
			sigmaSL_sum += sigmaLSP;
			sigmaI_sum += sigmaI;
			sigmaG_sum += sigmaG;
			sigmaIG_sum += sigmaI + sigmaG;
			sigmaT_sum += sigmaT;

			locallelecount++;
			}//end for a

		vcLoc_[m][loc][0] = sigmaS_Loc[loc];
		vcLoc_[m][loc][1] = sigmaL_Loc[loc];
		vcLoc_[m][loc][2] = sigmaSL_Loc[loc];
		vcLoc_[m][loc][3] = sigmaI_Loc[loc];
		vcLoc_[m][loc][4] = sigmaG_Loc[loc];
		vcLoc_[m][loc][5] = sigmaT_Loc[loc];
		aList<long double>& locFstat = FstatsLoc_[m][loc];
		if (sigmaT_Loc[loc] != 0.0) {
			int Flevel = Flevels_[m].Loc(FsptWord);//Fspt
			locFstat[Flevel] =sigmaS_Loc[loc]/ sigmaT_Loc[loc];
			Flevel = Flevels_[m].Loc(FltWord);//Flt
			locFstat[Flevel] = sigmaL_Loc[loc] / sigmaT_Loc[loc];
			if(includeInteraction){
				Flevel = Flevels_[m].Loc(FlsptWord);//Fst
				locFstat[Flevel] = sigmaSL_Loc[loc] /sigmaT_Loc[loc];}
			Flevel = Flevels_[m].Loc(FitWord);//Fit
			locFstat[Flevel]=1.0 - (sigmaG_Loc[loc] / sigmaT_Loc[loc]);
			Flevel = Flevels_[m].Loc(FstWord);//Fst
			locFstat[Flevel]=(sigmaS_Loc[loc] + sigmaL_Loc[loc] + sigmaSL_Loc[loc])/ sigmaT_Loc[loc];
			}//end if sigmaT_Loc
		if (sigmaIG_Loc[loc] != 0.0){
			int Flevel = Flevels_[m].Loc(FisWord);//Fis
			locFstat[Flevel] = 1.0 - (sigmaG_Loc[loc] /sigmaIG_Loc[loc]);
			}//end if sigmaIG_Loc
		}//end for j

	vc_[m][0] = sigmaS_sum;
	vc_[m][1] = sigmaL_sum;
	vc_[m][2] = sigmaSL_sum;
	vc_[m][3] = sigmaI_sum;
	vc_[m][4] = sigmaG_sum;
	vc_[m][5] = sigmaT_sum;
	if (sigmaT_sum != 0.0){
		int Flevel = Flevels_[m].Loc(FsptWord);//Fspt
		Fstats_[m][Flevel] = sigmaS_sum / sigmaT_sum;
		Flevel = Flevels_[m].Loc(FltWord);//Flt
		Fstats_[m][Flevel] = sigmaL_sum / sigmaT_sum;
		if(includeInteraction){
			Flevel = Flevels_[m].Loc(FlsptWord);//Fst
			Fstats_[m][Flevel] = sigmaSL_sum /sigmaT_sum;}
		Flevel = Flevels_[m].Loc(FitWord);//Fit
		Fstats_[m][Flevel]=1.0 - (sigmaG_sum / sigmaT_sum);
		Flevel = Flevels_[m].Loc(FstWord);//Fst
		Fstats_[m][Flevel]=(sigmaS_sum + sigmaL_sum + sigmaSL_sum) / sigmaT_sum;
		}//end if sigmaT_sum
	if (sigmaIG_sum != 0.0){
		int Flevel = Flevels_[m].Loc(FisWord);//Fis
		Fstats_[m][Flevel] = 1.0 - (sigmaG_sum /sigmaIG_sum);
		}//end if sigmaIG_sum

	//now do the jackknife calculations, over loci
	for (int j=0;j<variableLocusNumCodom_.len();++j){//skip monomorphic loci
		int loc = variableLocusNumCodom_[j];//the actual locus		
		aList<long double>& locJackFstat = jackFstatsLoc_[m][loc];
		if ((sigmaT_sum - sigmaT_Loc[loc]) != 0.0) {
			int Flevel = Flevels_[m].Loc(FsptWord);//Fspt
			locJackFstat[Flevel] =(sigmaS_sum - sigmaS_Loc[loc])/ (sigmaT_sum - sigmaT_Loc[loc]);
			Flevel = Flevels_[m].Loc(FltWord);//Flt
			locJackFstat[Flevel] = (sigmaL_sum - sigmaL_Loc[loc])/ (sigmaT_sum - sigmaT_Loc[loc]);
			if(includeInteraction){
				Flevel = Flevels_[m].Loc(FlsptWord);//Flspt
				locJackFstat[Flevel] = (sigmaSL_sum - sigmaSL_Loc[loc]) /(sigmaT_sum - sigmaT_Loc[loc]);}
			Flevel = Flevels_[m].Loc(FitWord);//Fit
			locJackFstat[Flevel]=1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) / (sigmaT_sum - sigmaT_Loc[loc]));
			Flevel = Flevels_[m].Loc(FstWord);//Fst
			locJackFstat[Flevel]=((sigmaS_sum+sigmaL_sum+sigmaSL_sum) - (sigmaS_Loc[loc]+sigmaL_Loc[loc]+sigmaSL_Loc[loc]))
							 / (sigmaT_sum - sigmaT_Loc[loc]);
			}//end if sigmaT_sum
		if (sigmaIG_sum - sigmaIG_Loc[loc] != 0.0) {
			int Flevel = Flevels_[m].Loc(FisWord);//Fis
			locJackFstat[Flevel] = 1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) /(sigmaIG_sum - sigmaIG_Loc[loc]));
			}//end if BCSum
		}//end for j

	FstatsCurrent_=vcCurrent_=1;
	FsptWord=FltWord=FlsptWord=FitWord=FstWord=FisWord=NULL;//forget these; deleted elsewhere
	
	speciesStr = localityStr = popStr = indivinPopStr = NULL;
	delete rawAnovaData; rawAnovaData=NULL;
	delete varianceComponents; varianceComponents=NULL;
	speciesStr = localityStr = popStr = indivinPopStr = NULL;
	}//end  FstatisticAnalysisCodom::FactorialFstats




void FstatisticAnalysisCodom::FactorialFstatsII(void){
	//uses linear algebra to get variance components for F-statistics
	//in a factorial, nested design
	ObjList<GLM> LinearModelList;
	FactorialFstatsII(LinearModelList, 1);
	}//end  FstatisticAnalysisCodom::FactorialFstatsII()




void FstatisticAnalysisCodom::FactorialFstatsII(ObjList<GLM>& LinearModelList, int includeInteraction){
	//uses linear algebra to get variance components for F-statistics
	//in a factorial, nested design.  Coefficients of variance components estimated using simpler formulae
	//clear out any previous calculations
	//this method estimates locality after species variance component
	
	//assumes data previously sorted by species, then locality
  	if(! FstatsCurrent_){
  		ClearFstats();}
  	if((Fmethod_.Includes((int) factorialFstatsII))){return;}
	FstatsCurrent_=vcCurrent_=0;
	Fmethod_.Append((int) factorialFstatsII);
	int m = Fmethod_.len()-1;
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, F-level
	Fstats_.Append(NEW aList<long double>());//F-level
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	Flevels_.Append(NEW StringList());//F-level

	vcMethod_.Append((int) factorialFstatsII);
	vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
	vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
	vc_.Append(NEW aList<long double>());//vc-level
	vcLevels_.Append(NEW StringList());//vc-level

	//set up all the F-stat arrays
	// first the array describing the Fstat structure:
	aString* FitWord=NEW aString("Fit");
	Flevels_[m].Append(FitWord);
	aString* FisWord=NEW aString("Fis");
	Flevels_[m].Append(FisWord);
	aString* FstWord=NEW aString("Fst");
	Flevels_[m].Append(FstWord);
	aString* FltWord=NEW aString("Flt");
	Flevels_[m].Append(FltWord);
	aString* FsptWord=NEW aString("Fspt");
	Flevels_[m].Append(FsptWord);
	aString* FlsptWord=NEW aString("Flspt");
	Flevels_[m].Append(FlsptWord);

	aString* vcWord=NEW aString("var(Sp)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(L)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(LSp)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(i)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(g)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(t)");
	vcLevels_[m].Append(vcWord);
	int vclevels=vcLevels_[m].len();
	//make and initialize the jackknife & by-locus Fstat lists
	for (int Flevel=0;Flevel<Flevels_.len();++Flevel){Fstats_[m].Append(0.0);}
	for (int vclevel=0;vclevel<vclevels;++vclevel){vc_[m].Append(0.0);}
	//some intermediate variables
	aList<long double> sigmaL_Loc, sigmaS_Loc, sigmaSL_Loc, sigmaI_Loc, sigmaG_Loc, sigmaIG_Loc, sigmaT_Loc;
	CheckForNewAlleles();
	UpdateVariableLoci();
	for (int j=0;j<CodominantMarkerData::loci();++j){//make for all loci; replace values of variable ones later
		//make and initialize the jackknife & by-locus Fstat lists for this locus 
		aList<long double>* locFstat = NEW aList<long double>();
		FstatsLoc_[m].Append(locFstat);
		aList<long double>* locJackFstat = NEW aList<long double>();
		jackFstatsLoc_[m].Append(locJackFstat);
		for (int Flevel=0;Flevel<Flevels_.len();++Flevel){
			locFstat->Append(0.0);
			locJackFstat->Append(0.0);
			}
		//make and initialize the jackknife & by-allele Fstat lists for this locus 
		ObjList<aList<long double> >* locAlleleFstat = NEW ObjList<aList<long double> >();
		FstatsAll_[m].Append(locAlleleFstat);
		//initialize variables
		sigmaS_Loc.Append(0.0);
		sigmaL_Loc.Append(0.0);
		sigmaSL_Loc.Append(0.0);
		sigmaI_Loc.Append(0.0);
		sigmaG_Loc.Append(0.0);
		sigmaIG_Loc.Append(0.0);
		sigmaT_Loc.Append(0.0);

		//make and initialize the by-allele F-stat list for each allele and append to this locus list
		StringList& locAlleleList = allelesPresent_[j];
		for (int a=0;a<locAlleleList.len();++a){
			aList<long double>* alleleFstat =NEW aList<long double>();
			locAlleleFstat->Append(alleleFstat);
			for (int Flevel=0;Flevel<Flevels_.len();++Flevel){alleleFstat->Append(0.0);}
			alleleFstat = NULL;
			}//end for a	
		locFstat = locJackFstat= NULL;
		locAlleleFstat= NULL;			

		//make and initialize the by-allele var comp list
		vcLoc_[m].Append(NEW aList<long double>());
		for (int vclevel=0;vclevel<vclevels;++vclevel){vcLoc_[m][j].Append(0.0);}
		vcAll_[m].Append(NEW ObjList<aList<long double> >());
		for (int a=0;a<locAlleleList.len();++a){
			vcAll_[m][j].Append(NEW aList<long double>());
			for (int vclevel=0;vclevel<vclevels;++vclevel){vcAll_[m][j][a].Append(0.0);}
			}//end for a							
		}//end for j

	//the main body of the calculations
	
	long double sigmaL_sum, sigmaS_sum, sigmaSL_sum, sigmaI_sum, sigmaG_sum, sigmaIG_sum, sigmaT_sum;
	sigmaL_sum = sigmaS_sum = sigmaSL_sum = sigmaI_sum = sigmaG_sum = sigmaIG_sum = sigmaT_sum = 0.0;
	ANOVAGenetics theTable;
	StringList locusAlleleNames;
	StringList factorList;
	aString* speciesStr = NEW aString("group$");
	aString* localityStr = NEW aString("locality$");
	aString* popStr = NEW aString("group$ x locality$");
	aString* indivinPopStr = NEW aString("indiv$ (group$ x locality$)");
	factorList.Append(localityStr); //don't switch this order!!
	factorList.Append(speciesStr); 
	if(includeInteraction){
		factorList.Append(popStr); }
	factorList.Append(indivinPopStr); 
	Ran3 ran(-1);//dummy
	//need to loop over missing data classes
	//this assumes no missing data


	dataSetRec* rawAnovaData =theTable.CreateRawANOVADataFormat(*this, locusAlleleNames);
	aString checkDataName("Ymatrices");
	rawAnovaData->Print(checkDataName);
	ObjList<aList<long double> >* varianceComponents = NULL;
	if (! rawAnovaData->MissingData()){
			if(LinearModelList.len()==0){//first time through
				GLM* lm = NEW GLM();
				lm->SetToReuseDesign(1);//true; reuse for each locus and allele
				LinearModelList.Append(lm);
				CalculateFactorialVarCompCoeffsII(*lm /*,0*/);
				lm = NULL;//forget
				}//end if first time through
			GLM& linearModel = LinearModelList[0];
			varianceComponents =theTable.CalculateVarianceComponents(/* *this, */ *rawAnovaData, 
											factorList, locusAlleleNames, /*0, ran, */ linearModel);

			}//end if no missing data
		else{//must dissect rawANOVAData into separate loci
			//and run individually
/*			varianceComponents = NEW ObjList<aList<double> >();
			for (int f=0;f<=factorList.len();++f){
				aList<double>* ABCvarList = NEW aList<double>();
				meanSquares->Append(ABCvarList);	}
			int locallelecount0=3;//column of rawANOVAData holding correct locus and allele
			for(int vloc0=0;vloc0<variableLoci_.len();++vloc0){
				dataSetRec locusANOVAData;//work with copies of columns from rawANOVAData
				dataColumnRec& grCol0 = rawAnovaData->First();
				dataColumnRec& popCol0 = rawAnovaData->Nth(1);
				dataColumnRec& indivCol0 = rawAnovaData->Nth(2);
				dataColumnRec* grCol1 = grCol0.Clone();
				dataColumnRec* popCol1 = popCol0.Clone();
				dataColumnRec* indivCol1 = indivCol0.Clone();
				locusANOVAData.Append(grCol1);
				locusANOVAData.Append(popCol1);
				locusANOVAData.Append(indivCol1);
				int loc0 = variableLoci_[vloc0];
				StringList& locAlleleList0 = allelesPresent_[loc0];
				for(int a0=0;a0<locAlleleList0.len();++a0){
					dataColumnRec& locAllCol0 = rawAnovaData->Nth(locallelecount0);
					dataColumnRec* locAllCol1 = locAllCol0.Clone();
					locusANOVAData.Append(locAllCol1);
					locallelecount0++;
					}//end for a0
				StringList thisLocusAlleleNames;
				ObjList<aList<double> >* locMeanSquares =theTable.CalculateMeanSquares(*this, locusANOVAData, 
											factorList, thisLocusAlleleNames, 0, ran);
				//now convert to variance components
				
				//transfer locus data to full list
				for(int a00=0;a00<thisLocusAlleleNames.len();++a00){
					aString* locAllNam00 = NEW aString(thisLocusAlleleNames[a00]);
					locusAlleleNames.Append(locAllNam00);
					for(int vc0=0;vc0<meanSquares->len();++vc0){
						aList<double>& ABCvarList00 = (*meanSquares)[vc0];
						aList<double>& ABCvarList01 = (*locMeanSquares)[vc0];
						ABCvarList00.Append(ABCvarList01[a00]);}
					}//end for a00
				delete locMeanSquares; locMeanSquares = NULL;
				}//end for vloc0
*/
		}//end if no missing data...else


	//store away the data and calculations
	int locallelecount=0;
	for(int vloc=0;vloc<variableLocusNumCodom_.len();++vloc){
		int loc = variableLocusNumCodom_[vloc];
		StringList& locAlleleList = allelesPresent_[loc];
		for(int a=0;a<locAlleleList.len();++a){
			long double sigmaL = (*varianceComponents)[0][locallelecount];//note order
			long double sigmaSP = (*varianceComponents)[1][locallelecount];
			long double sigmaLSP = 0.0, sigmaI, sigmaG,sigmaT;
			if(includeInteraction){
					sigmaLSP = (*varianceComponents)[2][locallelecount];
					sigmaI = (*varianceComponents)[3][locallelecount];
					sigmaG = (*varianceComponents)[4][locallelecount];
					sigmaT = sigmaSP+sigmaL+sigmaLSP+sigmaI+sigmaG;
					}
				else{
					sigmaI = (*varianceComponents)[2][locallelecount];
					sigmaG = (*varianceComponents)[3][locallelecount];
					sigmaT = sigmaSP+sigmaL+sigmaLSP+sigmaI+sigmaG;
					}
			vcAll_[m][loc][a][0] = sigmaSP;
			vcAll_[m][loc][a][1] = sigmaL;
			vcAll_[m][loc][a][2] = sigmaLSP;//can be 0 if no interaction used
			vcAll_[m][loc][a][3] = sigmaI;
			vcAll_[m][loc][a][4] = sigmaG;
			vcAll_[m][loc][a][5] = sigmaT;
			ObjList<aList<long double> >& locAlleleFstat=FstatsAll_[m][loc];
			aList<long double>& alleleFstat =locAlleleFstat[a];
			if (sigmaT != 0.0) {
				int Flevel = Flevels_[m].Loc(FsptWord);//Fspt
				alleleFstat[Flevel] = sigmaSP / sigmaT;
				Flevel = Flevels_[m].Loc(FltWord);//Flt
				alleleFstat[Flevel] = sigmaL / sigmaT;
				if(includeInteraction){
					Flevel = Flevels_[m].Loc(FlsptWord);//Fst
					alleleFstat[Flevel] = sigmaLSP /sigmaT;}
				Flevel = Flevels_[m].Loc(FitWord);//Fit
				alleleFstat[Flevel]=1.0 - (sigmaG / sigmaT);
				Flevel = Flevels_[m].Loc(FstWord);//Fst
				alleleFstat[Flevel]= (sigmaSP + sigmaL +  sigmaLSP)/ sigmaT;
				}//end if sigmaT
			if ((sigmaI + sigmaG) != 0.0) {
				int Flevel = Flevels_[m].Loc(FisWord);//Fis
				alleleFstat[Flevel] = 1.0 - (sigmaG / (sigmaI + sigmaG));
				}//end if (B+C)

			//save these in an array for a quicker calculation of the jackknife variance estimates
			sigmaS_Loc[loc] += sigmaSP;
			sigmaL_Loc[loc] += sigmaL;
			sigmaSL_Loc[loc] += sigmaLSP;
			sigmaI_Loc[loc] += sigmaI;
			sigmaG_Loc[loc] += sigmaG;
			sigmaIG_Loc[loc] += sigmaI + sigmaG;
			sigmaT_Loc[loc] += sigmaT;

			sigmaS_sum += sigmaSP;
			sigmaL_sum += sigmaL;
			sigmaSL_sum += sigmaLSP;
			sigmaI_sum += sigmaI;
			sigmaG_sum += sigmaG;
			sigmaIG_sum += sigmaI + sigmaG;
			sigmaT_sum += sigmaT;

			locallelecount++;
			}//end for a

		vcLoc_[m][loc][0] = sigmaS_Loc[loc];
		vcLoc_[m][loc][1] = sigmaL_Loc[loc];
		vcLoc_[m][loc][2] = sigmaSL_Loc[loc];
		vcLoc_[m][loc][3] = sigmaI_Loc[loc];
		vcLoc_[m][loc][4] = sigmaG_Loc[loc];
		vcLoc_[m][loc][5] = sigmaT_Loc[loc];
		aList<long double>& locFstat = FstatsLoc_[m][loc];
		if (sigmaT_Loc[loc] != 0.0) {
			int Flevel = Flevels_[m].Loc(FsptWord);//Fspt
			locFstat[Flevel] =sigmaS_Loc[loc]/ sigmaT_Loc[loc];
			Flevel = Flevels_[m].Loc(FltWord);//Flt
			locFstat[Flevel] = sigmaL_Loc[loc] / sigmaT_Loc[loc];
			if(includeInteraction){
				Flevel = Flevels_[m].Loc(FlsptWord);//Fst
				locFstat[Flevel] = sigmaSL_Loc[loc] /sigmaT_Loc[loc];}
			Flevel = Flevels_[m].Loc(FitWord);//Fit
			locFstat[Flevel]=1.0 - (sigmaG_Loc[loc] / sigmaT_Loc[loc]);
			Flevel = Flevels_[m].Loc(FstWord);//Fst
			locFstat[Flevel]=(sigmaS_Loc[loc] + sigmaL_Loc[loc] + sigmaSL_Loc[loc])/ sigmaT_Loc[loc];
			}//end if sigmaT_Loc
		if (sigmaIG_Loc[loc] != 0.0){
			int Flevel = Flevels_[m].Loc(FisWord);//Fis
			locFstat[Flevel] = 1.0 - (sigmaG_Loc[loc] /sigmaIG_Loc[loc]);
			}//end if sigmaIG_Loc
		}//end for j

	vc_[m][0] = sigmaS_sum;
	vc_[m][1] = sigmaL_sum;
	vc_[m][2] = sigmaSL_sum;
	vc_[m][3] = sigmaI_sum;
	vc_[m][4] = sigmaG_sum;
	vc_[m][5] = sigmaT_sum;
	if (sigmaT_sum != 0.0){
		int Flevel = Flevels_[m].Loc(FsptWord);//Fspt
		Fstats_[m][Flevel] = sigmaS_sum / sigmaT_sum;
		Flevel = Flevels_[m].Loc(FltWord);//Flt
		Fstats_[m][Flevel] = sigmaL_sum / sigmaT_sum;
		if(includeInteraction){
			Flevel = Flevels_[m].Loc(FlsptWord);//Fst
			Fstats_[m][Flevel] = sigmaSL_sum /sigmaT_sum;}
		Flevel = Flevels_[m].Loc(FitWord);//Fit
		Fstats_[m][Flevel]=1.0 - (sigmaG_sum / sigmaT_sum);
		Flevel = Flevels_[m].Loc(FstWord);//Fst
		Fstats_[m][Flevel]=(sigmaS_sum + sigmaL_sum + sigmaSL_sum) / sigmaT_sum;
		}//end if sigmaT_sum
	if (sigmaIG_sum != 0.0){
		int Flevel = Flevels_[m].Loc(FisWord);//Fis
		Fstats_[m][Flevel] = 1.0 - (sigmaG_sum /sigmaIG_sum);
		}//end if sigmaIG_sum

	//now do the jackknife calculations, over loci
	for (int j=0;j<variableLocusNumCodom_.len();++j){//skip monomorphic loci
		int loc = variableLocusNumCodom_[j];//the actual locus		
		aList<long double>& locJackFstat = jackFstatsLoc_[m][loc];
		if ((sigmaT_sum - sigmaT_Loc[loc]) != 0.0) {
			int Flevel = Flevels_[m].Loc(FsptWord);//Fspt
			locJackFstat[Flevel] =(sigmaS_sum - sigmaS_Loc[loc])/ (sigmaT_sum - sigmaT_Loc[loc]);
			Flevel = Flevels_[m].Loc(FltWord);//Flt
			locJackFstat[Flevel] = (sigmaL_sum - sigmaL_Loc[loc])/ (sigmaT_sum - sigmaT_Loc[loc]);
			if(includeInteraction){
				Flevel = Flevels_[m].Loc(FlsptWord);//Flspt
				locJackFstat[Flevel] = (sigmaSL_sum - sigmaSL_Loc[loc]) /(sigmaT_sum - sigmaT_Loc[loc]);}
			Flevel = Flevels_[m].Loc(FitWord);//Fit
			locJackFstat[Flevel]=1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) / (sigmaT_sum - sigmaT_Loc[loc]));
			Flevel = Flevels_[m].Loc(FstWord);//Fst
			locJackFstat[Flevel]=((sigmaS_sum+sigmaL_sum+sigmaSL_sum) - (sigmaS_Loc[loc]+sigmaL_Loc[loc]+sigmaSL_Loc[loc]))
							 / (sigmaT_sum - sigmaT_Loc[loc]);
			}//end if sigmaT_sum
		if (sigmaIG_sum - sigmaIG_Loc[loc] != 0.0) {
			int Flevel = Flevels_[m].Loc(FisWord);//Fis
			locJackFstat[Flevel] = 1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) /(sigmaIG_sum - sigmaIG_Loc[loc]));
			}//end if BCSum
		}//end for j

	FstatsCurrent_=vcCurrent_=1;
	FsptWord=FltWord=FlsptWord=FitWord=FstWord=FisWord=NULL;//forget these; deleted elsewhere
	
	speciesStr = localityStr = popStr = indivinPopStr = NULL;
	delete rawAnovaData; rawAnovaData=NULL;
	delete varianceComponents; varianceComponents=NULL;
	speciesStr = localityStr = popStr = indivinPopStr = NULL;
	}//end  FstatisticAnalysisCodom::FactorialFstatsII





void FstatisticAnalysisCodom::FactorialFstatsIII(void){
	//uses linear algebra to get variance components for F-statistics
	//in a factorial, nested design.  Coefficients of variance components estimated using simpler formulae
	
	//this Fmethod calculates all 3 versions at once, in the most streamlined fashion devised to date
	
	//assumes data previously sorted by species, then locality -- this is critical for success
  	if(! FstatsCurrent_){
  		ClearFstats();}
 	if((Fmethod_.Includes((int) factorialFstats))
  			&&(Fmethod_.Includes((int) factorialFstatsII))
  			&&(Fmethod_.Includes((int) factorialFstatsIII))){return;}
	FstatsCurrent_=vcCurrent_=0;
	//remove any extant factorialFstats to avoid duplication
	for(int f=0;f<3;++f){
		int g;
		if((Fmethod_.Includes((int) factorialFstats))){
				g=Fmethod_.Loc((int) factorialFstats);}
			else if((Fmethod_.Includes((int) factorialFstatsII))){
				g=Fmethod_.Loc((int) factorialFstatsII);}
			else if((Fmethod_.Includes((int) factorialFstatsIII))){
				g=Fmethod_.Loc((int) factorialFstatsIII);}
			else{break;}
		Fmethod_.Delete(g);
		Flevels_.Delete(g);
		Fstats_.Delete(g);
		FstatsLoc_.Delete(g);
		jackFstatsLoc_.Delete(g);
		FstatsAll_.Delete(g);

		vcMethod_.Delete(g);
		vcLevels_.Delete(g);
		vc_.Delete(g);
		vcLoc_.Delete(g);
		vcAll_.Delete(g);
		}//end for f
	Fmethod_.Append((int) factorialFstats);
	int mI = Fmethod_.len()-1;
	Fmethod_.Append((int) factorialFstatsII);
	int mII = Fmethod_.len()-1;
	Fmethod_.Append((int) factorialFstatsIII);
	int mIII = Fmethod_.len()-1;
	for(int f=0;f<3;++f){
		FstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
		FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, F-level
		Fstats_.Append(NEW aList<long double>());//F-level
		jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
		Flevels_.Append(NEW StringList());//F-level
		}//end for f

	vcMethod_.Append((int) factorialFstats);
	vcMethod_.Append((int) factorialFstatsII);
	vcMethod_.Append((int) factorialFstatsIII);
	for(int f=0;f<3;++f){
		vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
		vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
		vc_.Append(NEW aList<long double>());//vc-level
		vcLevels_.Append(NEW StringList());//vc-level
		}//end for f


	CheckForNewAlleles();
	UpdateVariableLoci();
	CalculateAlleleFreqs();
	//set up all the F-stat arrays
	// first the array describing the Fstat structure:
	aString *FitWord, *FisWord, *FstWord, *FltWord, *FsptWord, *FlsptWord, *FslWord, *FsspWord, *FsinltWord, *FsinsptWord;
	for(int m=mI;m<=mIII;++m){
		FitWord=NEW aString("Fit");
		Flevels_[m].Append(FitWord);
		FisWord=NEW aString("Fis");
		Flevels_[m].Append(FisWord);
		FstWord=NEW aString("Fst");
		Flevels_[m].Append(FstWord);
		FltWord=NEW aString("Flt");
		Flevels_[m].Append(FltWord);
		FsptWord=NEW aString("Fspt");
		Flevels_[m].Append(FsptWord);
		FlsptWord=NEW aString("Flspt");
		Flevels_[m].Append(FlsptWord);
		FslWord=NEW aString("Fsl");
		Flevels_[m].Append(FslWord);
		FsspWord=NEW aString("Fssp");
		Flevels_[m].Append(FsspWord);
		FsinltWord=NEW aString("Fsinlt");
		Flevels_[m].Append(FsinltWord);
		FsinsptWord=NEW aString("Fsinspt");
		Flevels_[m].Append(FsinsptWord);

		aString* vcWord=NEW aString("var(Sp)");
		vcLevels_[m].Append(vcWord);
		vcWord=NEW aString("var(L)");
		vcLevels_[m].Append(vcWord);
		vcWord=NEW aString("var(LSp)");
		vcLevels_[m].Append(vcWord);
		vcWord=NEW aString("var(i)");
		vcLevels_[m].Append(vcWord);
		vcWord=NEW aString("var(g)");
		vcLevels_[m].Append(vcWord);
		vcWord=NEW aString("var(t)");
		vcLevels_[m].Append(vcWord);
		int vclevels=vcLevels_[mI].len();
		//make and initialize the jackknife & by-locus Fstat lists
		for (int Flevel=0;Flevel<Flevels_[m].len();++Flevel){Fstats_[m].Append(0.0);}
		for (int vclevel=0;vclevel<vclevels;++vclevel){vc_[m].Append(0.0);}
		//some intermediate variables
		for (int j=0;j<CodominantMarkerData::loci();++j){//make for all loci; replace values of variable ones later
			//make and initialize the jackknife & by-locus Fstat lists for this locus 
			aList<long double>* locFstat = NEW aList<long double>();
			FstatsLoc_[m].Append(locFstat);
			aList<long double>* locJackFstat = NEW aList<long double>();
			jackFstatsLoc_[m].Append(locJackFstat);
			for (int Flevel=0;Flevel<Flevels_[m].len();++Flevel){
				locFstat->Append(0.0);
				locJackFstat->Append(0.0);
				}
			//make and initialize the jackknife & by-allele Fstat lists for this locus 
			ObjList<aList<long double> >* locAlleleFstat = NEW ObjList<aList<long double> >();
			FstatsAll_[m].Append(locAlleleFstat);
			//initialize variables

			//make and initialize the by-allele F-stat list for each allele and append to this locus list
			StringList& locAlleleList = allelesPresent_[j];
			for (int a=0;a<locAlleleList.len();++a){
				aList<long double>* alleleFstat =NEW aList<long double>();
				locAlleleFstat->Append(alleleFstat);
				for (int Flevel=0;Flevel<Flevels_[m].len();++Flevel){alleleFstat->Append(0.0);}
				alleleFstat = NULL;
				}//end for a	
			locFstat = locJackFstat= NULL;
			locAlleleFstat= NULL;			

			//make and initialize the by-allele var comp list
			vcLoc_[m].Append(NEW aList<long double>());
			for (int vclevel=0;vclevel<vclevels;++vclevel){vcLoc_[m][j].Append(0.0);}
			vcAll_[m].Append(NEW ObjList<aList<long double> >());
			for (int a=0;a<locAlleleList.len();++a){
				vcAll_[m][j].Append(NEW aList<long double>());
				for (int vclevel=0;vclevel<vclevels;++vclevel){vcAll_[m][j][a].Append(0.0);}
			}//end for a							

			}//end for j
		}//end for m

	//the main body of the calculations
	
	ANOVAGenetics theTable;
	StringList locusAlleleNames;
	StringList factorList;
	aString* speciesStr = NEW aString("group$");
	aString* localityStr = NEW aString("locality$");
	aString* popStr = NEW aString("group$ x locality$");
	aString* indivinPopStr = NEW aString("indiv$ (group$ x locality$)");
	factorList.Append(speciesStr); //don't switch this order!!
	factorList.Append(localityStr); 
	factorList.Append(popStr); 
	factorList.Append(indivinPopStr); 
	Ran3 ran(-1);//dummy
	//need to loop over missing data classes
	//this assumes no missing data
		


	dataSetRec* rawAnovaData =theTable.CreateRawANOVADataFormat(*this, locusAlleleNames);
	ObjList<aList<long double> > varianceComponents;
	aList<int> locusOrder;
	if (! rawAnovaData->MissingData()){
		FactorialFstatsIIIGuts(*rawAnovaData, varianceComponents, variableLocusNumCodom_, locusAlleleNames);
		locusOrder = variableLocusNumCodom_;
/*		MatrixList Zmatrices;
		int loc = 0;//first locus=0; all loci the same
		aList<double> nList, dfList;//these hold coefficients and degrees of freedom
		CalculateFactorialVarCompCoeffsIII(nList, dfList, loc);//loc=0 and all have the same sample size
		//calculate Z matrices and then remainder sums of squares
		MatrixList ZSList, ZLList0, ZLList1, ZIList;
		ObjList<MatrixList> ZSLList0,  ZSLList1, ZSIList;
		ObjList<ObjList<MatrixList> > ZSLIList;
		for(int sp00=0;sp00<ngroups_;++sp00){
			ZSLList0.Append(NEW MatrixList);
			ZSLList1.Append(NEW MatrixList);
			ZSIList.Append(NEW MatrixList);
			ZSLIList.Append(NEW ObjList<MatrixList>());
			for(int p00=0;p00<groupHierarchy_[sp00].len();++p00){
				ZSLIList[sp00].Append(NEW MatrixList());}
			}
		for(int sp0=0;sp0<ngroups_;++sp0){
			int nSp0=0;
			for(int p0=0;p0<groupHierarchy_[sp0].len();++p0){
				int p = groupHierarchy_[sp0][p0];
				int n00 = GetLocusSampleSize(p,loc);
				nSp0 += n00;
				ZSLList0[sp0].Append(OneMatrix(1.0,n00,1));//relies on properly sorted data
				for(int i0=0;i0<individuals(p);++i0){
					int n000 = GetLocusSampleSize(p,i0,loc);
					if(n000>0){
						ZSLIList[sp0][p0].Append(OneMatrix(1.0,n000,1));}
					}//end for i0	
				ZSIList[sp0].Append(ZSLIList[sp0][p0].DirectSum());
				}//end for sp0
			ZSList.Append(OneMatrix(1.0,nSp0,1));	
			ZLList0.Append(ZSLList0[sp0].DirectSum());		
			ZIList.Append(ZSIList[sp0].DirectSum());		
			}//end for sp0
		//special handling for ZL so that missing columns get zeros
		ObjList<aList<int> > popNums;//this sorts populatons by number in factorial array
		for(int sp1=0;sp1<groupHierarchy_.len();++sp1){
			popNums.Append(NEW aList<int>());
			for(int L1=0;L1<localities();++L1){
				popNums[sp1].Append(-1);//-1 indicates empty cell
				}}
		for(int pop00=0;pop00<npop_;++pop00){
			int gr=GroupMembership(pop00);
			int L01=GetLocality(pop00);
			popNums[gr][L01] = pop00;
			}
		for( sp1=0;sp1<ngroups_;++sp1){
			aList<int> emptyLocalities;
			int nSp1=0;
			for(int p1=0;p1<localities();++p1){
				int pp = popNums[sp1][p1];
				if (pp==-1){//pop is empty 
						emptyLocalities.Append(p1);}
					else{
						int n00L = GetLocusSampleSize(pp,loc);
						nSp1 += n00L;
						ZSLList1[sp1].Append(OneMatrix(1.0,n00L,1));}//relies on properly sorted data
				}//end for p1
			matrix<double>* ZLsp = ZSLList1[sp1].DirectSum();
			for(int missing=0;missing<emptyLocalities.len();++missing){
				vector<double> zeroCol(0.0, nSp1);
				ZLsp->InsertColumn(zeroCol, emptyLocalities[missing]);}
			ZLList1.Append(ZLsp);		
			}//end for sp1
		matrix<double>* ZS = ZSList.DirectSum();
		Zmatrices.Append(ZS); ZS=NULL;
		matrix<double>* ZL = ZLList1.ConcatRows();
		Zmatrices.Append(ZL);ZL=NULL;
		matrix<double>* ZSL = ZLList0.DirectSum();
		Zmatrices.Append(ZSL);ZSL=NULL;
		matrix<double>* ZI = ZIList.DirectSum();
		Zmatrices.Append(ZI);ZI=NULL;
		ZSList.Clear();//free memory
		ZLList0.Clear();//free memory
		ZLList1.Clear();//free memory
		ZSLList0.Clear();//free memory
		ZSLList1.Clear();//free memory
		ZSIList.Clear();//free memory
		ZSLIList.Clear();//free memory
		Zmatrices.Prepend(OneMatrix(1.0,dfList.Last()+1,1));//one matrix of length n___

	//replicate and consolidate steps in GeneralLinearModel::varcompsANOVA
		matrixHandle<double> invZprimeZH(NEW matrix<double>());//initially empty; filled at first calculation
		matrixHandle<double> invZ1primeZ1H(NEW matrix<double>());//initially empty; filled at first calculation
		matrixHandle<double> invZ1SprimeZ1SH(NEW matrix<double>());//initially empty; filled at first calculation
		matrixHandle<double> invZ1LprimeZ1LH(NEW matrix<double>());//initially empty; filled at first calculation
		matrixHandle<double> invZ1SLprimeZ1SLH(NEW matrix<double>());//initially empty; filled at first calculation
		matrixHandle<double> invZ1SLSLprimeZ1SLSLH(NEW matrix<double>());//initially empty; filled at first calculation
		GLM* lm = NEW GLM();
		aList<int> omitItems;//when determining reduced models
		MatrixList* R_muabcd_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
		matrix<double>* Z = lm->MakeDesignMatrix(*R_muabcd_DesignComponents, 0);//= full model
		delete R_muabcd_DesignComponents; R_muabcd_DesignComponents=NULL;
		MatrixList* R_muabc_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
		omitItems.Append(4);//individuals omitted
		matrix<double>* Z1SLSL = lm->MakeDesignMatrix(*R_muabc_DesignComponents, omitItems);//= reduced model
		delete R_muabc_DesignComponents; R_muabc_DesignComponents=NULL;
		MatrixList* R_muab_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
		omitItems.Append(3);//populations and individuals omitted
		matrix<double>* Z1SL = lm->MakeDesignMatrix(*R_muab_DesignComponents, omitItems);//= reduced model
		delete R_muab_DesignComponents; R_muab_DesignComponents=NULL;
		MatrixList* R_mua_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
		omitItems.Append(2);//localities, populations and individuals omitted
		matrix<double>* Z1S = lm->MakeDesignMatrix(*R_mua_DesignComponents, omitItems);//= reduced model
		delete R_mua_DesignComponents; R_mua_DesignComponents=NULL;
		MatrixList* R_mu_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
		omitItems.Append(1);//species, localities, populations and individuals omitted
		matrix<double>* Z1 = lm->MakeDesignMatrix(*R_mu_DesignComponents, omitItems);//= reduced model
		delete R_mu_DesignComponents; R_mu_DesignComponents=NULL;
		MatrixList* R_mub_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
		omitItems.Delete(2);//species, populations and individuals omitted
		matrix<double>* Z1L = lm->MakeDesignMatrix(*R_mub_DesignComponents, omitItems);//= reduced model
		delete R_mub_DesignComponents; R_mub_DesignComponents=NULL;
		for(int locallele=0;locallele<locusAlleleNames.len();++locallele){
			aString& colTitle0 = locusAlleleNames[locallele];
			dataColumnRecNum<double>& depVarCol0 = (dataColumnRecNum<double>) rawAnovaData->GetDataColumn(colTitle0);
			aList<double>& varList0 = depVarCol0.dataList();
			matrix<double> Pmatrix;
			vector<double> aVecPtr0(varList0);
			Pmatrix.ConcatCol(aVecPtr0);

			double R_mu, R_mua, R_mub, R_muab, R_muabc, R_muabcd;//sums of squares from including model factors
			R_mu = R_mua = R_mub = R_muab = R_muabc = R_muabcd = 0.0;
			double SSE;  //dummy variable
			double PprimeP;//total sum of squares
			//R_muabcd
			lm->RegressionSS(Z, &Pmatrix, R_muabcd, SSE, PprimeP, invZprimeZH);//invZprimeZH replaced if empty
			//R_muabc
			lm->RegressionSS(Z1SLSL, &Pmatrix, R_muabc, SSE, PprimeP, invZ1SLSLprimeZ1SLSLH);//invZ1SLSLprimeZ1SLSLH replaced if empty
			//R_muab
			lm->RegressionSS(Z1SL, &Pmatrix, R_muab, SSE, PprimeP, invZ1SLprimeZ1SLH);//invZ1SLprimeZ1SLH replaced if empty
			//R_mua
			lm->RegressionSS(Z1S, &Pmatrix, R_mua, SSE, PprimeP, invZ1SprimeZ1SH);//invZ1SprimeZ1SH replaced if empty
			//R_mu
			lm->RegressionSS(Z1, &Pmatrix, R_mu, SSE, PprimeP, invZ1primeZ1H);//invZ1primeZ1H replaced if empty
			//R_mub
			lm->RegressionSS(Z1L, &Pmatrix, R_mub, SSE, PprimeP, invZ1LprimeZ1LH);//invZ1primeZ1H replaced if empty
		
			double SSSP1 = R_mua - R_mu;
			double SSL1 = R_muab - R_mua;
			double SSSP2 = R_muab - R_mub;
			double SSL2 = R_mub - R_mu;
			double SSLSP = R_muabc - R_muab;
			double SSI = R_muabcd - R_muabc;
			double SSG = PprimeP - R_muabcd;
		
			double sigmaSqG = SSG/dfList[4];
			double sigmaSqI = (  SSI - (dfList[3]*sigmaSqG)  ) / nList[0];
			double sigmaSqSL = (  SSLSP - (nList[1] * sigmaSqI) - (dfList[2]*sigmaSqG)  ) / nList[2];
			double sigmaSqL1 = (  SSL1 - (nList[4] * sigmaSqSL) - (nList[3] * sigmaSqI) - (dfList[1]*sigmaSqG)  ) / nList[5];
			double sigmaSqS1 = (  SSSP1 - (nList[8] * sigmaSqL1) - (nList[7] * sigmaSqSL) - (nList[6] * sigmaSqI) - (dfList[0]*sigmaSqG)  ) / nList[9];
			double sigmaSqS2 = (  SSSP1 - (nList[11] * sigmaSqSL) - (nList[10] * sigmaSqI) - (dfList[0]*sigmaSqG)  ) / nList[12];
			double sigmaSqL2 = (  SSL2 - (nList[15] * sigmaSqS2) - (nList[14] * sigmaSqSL) - (nList[13] * sigmaSqI) - (dfList[1]*sigmaSqG)  ) / nList[16];
			double sigmaSqT = PprimeP / dfList[5];//empirical total variance
			
			double sigmaSqSinSp1 = sigmaSqT - sigmaSqS1 - sigmaSqI - sigmaSqG;
			double sigmaSqSinSp2 = sigmaSqT - sigmaSqS2 - sigmaSqI - sigmaSqG;
			double sigmaSqSinL1 = sigmaSqT - sigmaSqL1 - sigmaSqI - sigmaSqG;
			double sigmaSqSinL2 = sigmaSqT - sigmaSqL2 - sigmaSqI - sigmaSqG;

			aList<double>* vc = NEW aList<double>();
			vc->Append(sigmaSqG);
			vc->Append(sigmaSqI);
			vc->Append(sigmaSqSL);
			vc->Append(sigmaSqL1);
			vc->Append(sigmaSqS1);
			vc->Append(sigmaSqL2);
			vc->Append(sigmaSqS2);
			vc->Append(sigmaSqT);
			vc->Append(sigmaSqSinSp1);
			vc->Append(sigmaSqSinSp2);
			vc->Append(sigmaSqSinL1);
			vc->Append(sigmaSqSinL2);
			
			varianceComponents.Append(vc);
			vc=NULL;//forget this
			}//end for locallele		


		delete Z; Z=NULL;//clear unused memory
		delete Z1SLSL; Z1SLSL=NULL;//clear unused memory
		delete Z1SL; Z1SL=NULL;//clear unused memory
		delete Z1S; Z1S=NULL;//clear unused memory
		delete Z1; Z1=NULL;//clear unused memory
		delete Z1L; Z1L=NULL;//clear unused memory
		delete lm; lm=NULL;
*/
			}//end if no missing data
		else{//must dissect rawANOVAData into separate loci
			//and run individually
			
			//first determine the sets of loci that have identical sampling regimes
			ObjList<StringList> locusNamesBySamplingRegime;
			ObjList<aList<int> > lociBySamplingRegime;
			
			//2nd dissect rawANOVAData into these pieces and cull missing data
			ObjList<dataSetRec>* parsedDataPtr = rawAnovaData->ParseMissingDataIntoSets(locusAlleleNames,'\0',-1);
			ObjList<dataSetRec>& parsedData = *parsedDataPtr;
			ObjList<StringList> locusAlleleNamesByRegime;
			for(int d=0;d<parsedData.len();++d){
				locusAlleleNamesByRegime.Append(NEW StringList());
				locusNamesBySamplingRegime.Append(NEW StringList());
				lociBySamplingRegime.Append(NEW aList<int>());
				for(int dc=0; dc<parsedData[d].len();++dc){
					aString& coltitle = parsedData[d][dc].title();
					if(locusAlleleNames.Includes(&coltitle)){
						locusAlleleNamesByRegime[d].Append(NEW aString(coltitle));
						aString locName(coltitle);
						locName.cut(locName.pos('_'),locName.len());
						if(!(locusNamesBySamplingRegime[d].Includes(&locName))){
							locusNamesBySamplingRegime[d].Append(NEW aString(locName));}
						}
					}//enmdfor dc
				}//end for d
			for(int d=0;d<locusNamesBySamplingRegime.len();++d){
				for(int dc=0; dc<locusNamesBySamplingRegime[d].len();++dc){
					aString& locName = locusNamesBySamplingRegime[d][dc];
					int locNum = traitName_.Loc(&locName);
					lociBySamplingRegime[d].Append(locNum);
					locusOrder.Append(locNum);//for correct reordering of variance components by locus
					}}
			
			//3rd run each piece through factoriaFstatsIIIGuts()
			for(int d=0;d<parsedData.len();++d){
				parsedData[d].CullMissingData('\0',-1);
				FactorialFstatsIIIGuts(parsedData[d], varianceComponents, lociBySamplingRegime[d], locusAlleleNamesByRegime[d]);
				}//end for d
			

			//clean up
			delete parsedDataPtr; parsedDataPtr=NULL;
			}//end if no missing data...else


	//store away the data and calculations
	aList<long double> sigmaS1_Loc, sigmaS2_Loc, sigmaL1_Loc, sigmaL2_Loc, sigmaSL_Loc, sigmaI_Loc, sigmaG_Loc, sigmaIG_Loc,
			 sigmaT0_Loc, sigmaT1_Loc, sigmaT2_Loc, sigmaT3_Loc,sigmaSinSp1_Loc, sigmaSinSp2_Loc, sigmaSinL1_Loc, sigmaSinL2_Loc;
	for(int loc0=0;loc0<CodominantMarkerData::loci();++loc0){
		sigmaS1_Loc.Append(0.0);
		sigmaS2_Loc.Append(0.0);
		sigmaL1_Loc.Append(0.0);
		sigmaL2_Loc.Append(0.0);
		sigmaSL_Loc.Append(0.0);
		sigmaI_Loc.Append(0.0);
		sigmaG_Loc.Append(0.0);
		sigmaIG_Loc.Append(0.0);
		sigmaT0_Loc.Append(0.0);
		sigmaT1_Loc.Append(0.0);
		sigmaT2_Loc.Append(0.0);
		sigmaT3_Loc.Append(0.0);
		sigmaSinSp1_Loc.Append(0.0);
		sigmaSinSp2_Loc.Append(0.0);
		sigmaSinL1_Loc.Append(0.0);
		sigmaSinL2_Loc.Append(0.0);
		}//end for loc0
	long double sigmaS1_sum, sigmaS2_sum, sigmaL1_sum, sigmaL2_sum, sigmaSL_sum, sigmaI_sum, sigmaG_sum, sigmaIG_sum, 
			sigmaT0_sum, sigmaT1_sum, sigmaT2_sum, sigmaT3_sum,  sigmaSinSp1_sum, sigmaSinSp2_sum, sigmaSinL1_sum, sigmaSinL2_sum;
	sigmaS1_sum = sigmaS2_sum = sigmaL1_sum = sigmaL2_sum = sigmaSL_sum = sigmaI_sum = sigmaG_sum = sigmaIG_sum = 
			sigmaT0_sum = sigmaT1_sum = sigmaT2_sum = sigmaT3_sum =  sigmaSinSp1_sum = sigmaSinSp2_sum = 
			sigmaSinL1_sum = sigmaSinL2_sum = 0.0;

	int locallelecount=0, Flevel;
	for(int vloc=0;vloc<locusOrder.len();++vloc){
		int loc = locusOrder[vloc];
		StringList& locAlleleList = allelesPresent_[loc];
		for(int a=0;a<locAlleleList.len();++a){
			long double sigmaG = varianceComponents[locallelecount][0];
			long double sigmaI = varianceComponents[locallelecount][1];
			long double sigmaLSP = varianceComponents[locallelecount][2];
			long double sigmaL1 = varianceComponents[locallelecount][3];
			long double sigmaS1 = varianceComponents[locallelecount][4];
			long double sigmaL2 = varianceComponents[locallelecount][5];
			long double sigmaS2 = varianceComponents[locallelecount][6];
			long double sigmaT0 = varianceComponents[locallelecount][7];//empirical value 
			long double sigmaT1 = sigmaT0;//most justifiable method, = p(1-p)
			long double sigmaT2 = sigmaT0;
			long double sigmaT3 = sigmaT0;
			long double sigmaSinSp1 = varianceComponents[locallelecount][8];
			long double sigmaSinSp2 = varianceComponents[locallelecount][9];
			long double sigmaSinL1 = varianceComponents[locallelecount][10];
			long double sigmaSinL2 = varianceComponents[locallelecount][11];
//			sigmaT1 = sigmaS1+sigmaL1+sigmaLSP+sigmaI+sigmaG;//these not equal when design unbalanced
//			sigmaT2 = sigmaS2+sigmaL2+sigmaLSP+sigmaI+sigmaG;
//			sigmaT3 = sigmaS2+sigmaL1+sigmaLSP+sigmaI+sigmaG;
			vcAll_[mI][loc][a][0] = sigmaS1;//Fmethod I
			vcAll_[mI][loc][a][1] = sigmaL1;
			vcAll_[mI][loc][a][2] = sigmaLSP;
			vcAll_[mI][loc][a][3] = sigmaI;
			vcAll_[mI][loc][a][4] = sigmaG;
			vcAll_[mI][loc][a][5] = sigmaT1;
			vcAll_[mII][loc][a][0] = sigmaS2;//Fmethod II
			vcAll_[mII][loc][a][1] = sigmaL2;
			vcAll_[mII][loc][a][2] = sigmaLSP;
			vcAll_[mII][loc][a][3] = sigmaI;
			vcAll_[mII][loc][a][4] = sigmaG;
			vcAll_[mII][loc][a][5] = sigmaT2;
			vcAll_[mIII][loc][a][0] = sigmaS2;//Fmethod III
			vcAll_[mIII][loc][a][1] = sigmaL1;
			vcAll_[mIII][loc][a][2] = sigmaLSP;
			vcAll_[mIII][loc][a][3] = sigmaI;
			vcAll_[mIII][loc][a][4] = sigmaG;
			vcAll_[mIII][loc][a][5] = sigmaT3;
			if (sigmaT1 != 0.0) {
				Flevel = Flevels_[mI].Loc(FsptWord);//Fspt
				FstatsAll_[mI][loc][a][Flevel] = sigmaS1 / sigmaT1;
				Flevel = Flevels_[mI].Loc(FltWord);//Flt
				FstatsAll_[mI][loc][a][Flevel] = sigmaL1 / sigmaT1;
				Flevel = Flevels_[mI].Loc(FlsptWord);//Fst
				FstatsAll_[mI][loc][a][Flevel] = sigmaLSP /sigmaT1;
				Flevel = Flevels_[mI].Loc(FitWord);//Fit
				FstatsAll_[mI][loc][a][Flevel]=1.0 - (sigmaG / sigmaT1);
				Flevel = Flevels_[mI].Loc(FstWord);//Fst
				FstatsAll_[mI][loc][a][Flevel]= (sigmaS1 + sigmaL1 +  sigmaLSP)/ sigmaT1;
				Flevel = Flevels_[mI].Loc(FsinltWord);//Fsinlt
				FstatsAll_[mI][loc][a][Flevel]= sigmaSinL1/ sigmaT1;
				Flevel = Flevels_[mI].Loc(FsinsptWord);//Fsinspt
				FstatsAll_[mI][loc][a][Flevel]= sigmaSinSp1/ sigmaT1;
				}//end if sigmaT1
			if( (sigmaT1 - sigmaL1) != 0.0){
				Flevel = Flevels_[mI].Loc(FslWord);//Fsl
				FstatsAll_[mI][loc][a][Flevel]= sigmaSinL1/ (sigmaT1 - sigmaL1);
				}
			if( (sigmaT1 - sigmaS1) != 0.0){
				Flevel = Flevels_[mI].Loc(FsspWord);//Fssp
				FstatsAll_[mI][loc][a][Flevel]= sigmaSinSp1/ (sigmaT1 - sigmaS1);
				}
			if (sigmaT2 != 0.0) {
				Flevel = Flevels_[mII].Loc(FsptWord);//Fspt
				FstatsAll_[mII][loc][a][Flevel] = sigmaS2 / sigmaT2;
				Flevel = Flevels_[mII].Loc(FltWord);//Flt
				FstatsAll_[mII][loc][a][Flevel] = sigmaL2 / sigmaT2;
				Flevel = Flevels_[mII].Loc(FlsptWord);//Fst
				FstatsAll_[mII][loc][a][Flevel] = sigmaLSP /sigmaT2;
				Flevel = Flevels_[mII].Loc(FitWord);//Fit
				FstatsAll_[mII][loc][a][Flevel]=1.0 - (sigmaG / sigmaT2);
				Flevel = Flevels_[mII].Loc(FstWord);//Fst
				FstatsAll_[mII][loc][a][Flevel]= (sigmaS2 + sigmaL2 +  sigmaLSP)/ sigmaT2;
				Flevel = Flevels_[mII].Loc(FsinltWord);//Fsinlt
				FstatsAll_[mII][loc][a][Flevel]= sigmaSinL2/ sigmaT2;
				Flevel = Flevels_[mII].Loc(FsinsptWord);//Fsinspt
				FstatsAll_[mII][loc][a][Flevel]= sigmaSinSp2/ sigmaT2;
				}//end if sigmaT2
			if( (sigmaT2 - sigmaL2) != 0.0){
				Flevel = Flevels_[mII].Loc(FslWord);//Fsl
				FstatsAll_[mII][loc][a][Flevel]= sigmaSinL2/ (sigmaT2 - sigmaL2);
				}
			if( (sigmaT2 - sigmaS2) != 0.0){
				Flevel = Flevels_[mII].Loc(FsspWord);//Fssp
				FstatsAll_[mII][loc][a][Flevel]= sigmaSinSp2/ (sigmaT2 - sigmaS2);
				}
			if (sigmaT3 != 0.0) {
				Flevel = Flevels_[mIII].Loc(FsptWord);//Fspt
				FstatsAll_[mIII][loc][a][Flevel] = sigmaS2 / sigmaT3;
				Flevel = Flevels_[mIII].Loc(FltWord);//Flt
				FstatsAll_[mIII][loc][a][Flevel] = sigmaL1 / sigmaT3;
				Flevel = Flevels_[mIII].Loc(FlsptWord);//Fst
				FstatsAll_[mIII][loc][a][Flevel] = sigmaLSP /sigmaT3;
				Flevel = Flevels_[mIII].Loc(FitWord);//Fit
				FstatsAll_[mIII][loc][a][Flevel]=1.0 - (sigmaG / sigmaT3);
				Flevel = Flevels_[mIII].Loc(FstWord);//Fst
				FstatsAll_[mIII][loc][a][Flevel]= (sigmaS2 + sigmaL1 +  sigmaLSP)/ sigmaT3;
				Flevel = Flevels_[mIII].Loc(FsinltWord);//Fsinlt
				FstatsAll_[mIII][loc][a][Flevel]= sigmaSinL1/ sigmaT3;
				Flevel = Flevels_[mIII].Loc(FsinsptWord);//Fsinspt
				FstatsAll_[mIII][loc][a][Flevel]= sigmaSinSp2/ sigmaT3;
				}//end if sigmaT3
			if( (sigmaT3 - sigmaL1) != 0.0){
				Flevel = Flevels_[mIII].Loc(FslWord);//Fsl
				FstatsAll_[mIII][loc][a][Flevel]= sigmaSinL1/ (sigmaT3 - sigmaL1);
				}
			if( (sigmaT3 - sigmaS2) != 0.0){
				Flevel = Flevels_[mIII].Loc(FsspWord);//Fssp
				FstatsAll_[mIII][loc][a][Flevel]= sigmaSinSp2/ (sigmaT3 - sigmaS2);
				}
			if ((sigmaI + sigmaG) != 0.0) {
				Flevel = Flevels_[mI].Loc(FisWord);//Fis
				FstatsAll_[mI][loc][a][Flevel] = 1.0 - (sigmaG / (sigmaI + sigmaG));
				FstatsAll_[mII][loc][a][Flevel] = 1.0 - (sigmaG / (sigmaI + sigmaG));
				FstatsAll_[mIII][loc][a][Flevel] = 1.0 - (sigmaG / (sigmaI + sigmaG));
				}//end if (B+C)

			//save these in an array for a quicker calculation of the jackknife variance estimates
			sigmaS1_Loc[loc] += sigmaS1;
			sigmaS2_Loc[loc] += sigmaS2;
			sigmaL1_Loc[loc] += sigmaL1;
			sigmaL2_Loc[loc] += sigmaL2;
			sigmaSL_Loc[loc] += sigmaLSP;
			sigmaI_Loc[loc] += sigmaI;
			sigmaG_Loc[loc] += sigmaG;
			sigmaIG_Loc[loc] += sigmaI + sigmaG;
			sigmaT0_Loc[loc] += sigmaT0;
			sigmaT1_Loc[loc] += sigmaT1;
			sigmaT2_Loc[loc] += sigmaT2;
			sigmaT3_Loc[loc] += sigmaT3;
			sigmaSinSp1_Loc[loc] += sigmaSinSp1;
			sigmaSinSp2_Loc[loc] += sigmaSinSp2;
			sigmaSinL1_Loc[loc] += sigmaSinL1;
			sigmaSinL2_Loc[loc] += sigmaSinL2;

			sigmaS1_sum += sigmaS1;
			sigmaS2_sum += sigmaS2;
			sigmaL1_sum += sigmaL1;
			sigmaL2_sum += sigmaL2;
			sigmaSL_sum += sigmaLSP;
			sigmaI_sum += sigmaI;
			sigmaG_sum += sigmaG;
			sigmaIG_sum += sigmaI + sigmaG;
			sigmaT0_sum += sigmaT0;
			sigmaT1_sum += sigmaT1;
			sigmaT2_sum += sigmaT2;
			sigmaT3_sum += sigmaT3;
			sigmaSinSp1_sum += sigmaSinSp1;
			sigmaSinSp2_sum += sigmaSinSp2;
			sigmaSinL1_sum += sigmaSinL1;
			sigmaSinL2_sum += sigmaSinL2;

			locallelecount++;
			}//end for a

		vcLoc_[mI][loc][0] = sigmaS1_Loc[loc];//Fmethod I
		vcLoc_[mI][loc][1] = sigmaL1_Loc[loc];
		vcLoc_[mI][loc][2] = sigmaSL_Loc[loc];
		vcLoc_[mI][loc][3] = sigmaI_Loc[loc];
		vcLoc_[mI][loc][4] = sigmaG_Loc[loc];
		vcLoc_[mI][loc][5] = sigmaT1_Loc[loc];
		vcLoc_[mII][loc][0] = sigmaS2_Loc[loc];//Fmethod II
		vcLoc_[mII][loc][1] = sigmaL2_Loc[loc];
		vcLoc_[mII][loc][2] = sigmaSL_Loc[loc];
		vcLoc_[mII][loc][3] = sigmaI_Loc[loc];
		vcLoc_[mII][loc][4] = sigmaG_Loc[loc];
		vcLoc_[mII][loc][5] = sigmaT2_Loc[loc];
		vcLoc_[mIII][loc][0] = sigmaS2_Loc[loc];//Fmethod III
		vcLoc_[mIII][loc][1] = sigmaL1_Loc[loc];
		vcLoc_[mIII][loc][2] = sigmaSL_Loc[loc];
		vcLoc_[mIII][loc][3] = sigmaI_Loc[loc];
		vcLoc_[mIII][loc][4] = sigmaG_Loc[loc];
		vcLoc_[mIII][loc][5] = sigmaT3_Loc[loc];
		if (sigmaT1_Loc[loc] != 0.0) {
			Flevel = Flevels_[mI].Loc(FsptWord);//Fspt
			 FstatsLoc_[mI][loc][Flevel] =sigmaS1_Loc[loc]/ sigmaT1_Loc[loc];
			Flevel = Flevels_[mI].Loc(FltWord);//Flt
			 FstatsLoc_[mI][loc][Flevel] = sigmaL1_Loc[loc] / sigmaT1_Loc[loc];
			Flevel = Flevels_[mI].Loc(FlsptWord);//Fst
			 FstatsLoc_[mI][loc][Flevel] = sigmaSL_Loc[loc] /sigmaT1_Loc[loc];
			Flevel = Flevels_[mI].Loc(FitWord);//Fit
			 FstatsLoc_[mI][loc][Flevel]=1.0 - (sigmaG_Loc[loc] / sigmaT1_Loc[loc]);
			Flevel = Flevels_[mI].Loc(FstWord);//Fst
			 FstatsLoc_[mI][loc][Flevel]=(sigmaS1_Loc[loc] + sigmaL1_Loc[loc] + sigmaSL_Loc[loc])/ sigmaT1_Loc[loc];
			Flevel = Flevels_[mI].Loc(FsinltWord);//Fsinlt
			FstatsLoc_[mI][loc][Flevel] = sigmaSinL1_Loc[loc]/ sigmaT1_Loc[loc];
			Flevel = Flevels_[mI].Loc(FsinsptWord);//Fsinspt
			FstatsLoc_[mI][loc][Flevel] = sigmaSinSp1_Loc[loc]/ sigmaT1_Loc[loc];
			}//end if sigmaT1_Loc
		if( (sigmaT1_Loc[loc] - sigmaL1_Loc[loc]) != 0.0){
			Flevel = Flevels_[mI].Loc(FslWord);//Fsl
			FstatsLoc_[mI][loc][Flevel]= sigmaSinL1_Loc[loc]/ (sigmaT1_Loc[loc] - sigmaL1_Loc[loc]);
			}
		if( (sigmaT1_Loc[loc] - sigmaS1_Loc[loc]) != 0.0){
			Flevel = Flevels_[mI].Loc(FsspWord);//Fssp
			FstatsLoc_[mI][loc][Flevel]= sigmaSinSp1_Loc[loc]/ (sigmaT1_Loc[loc] - sigmaS1_Loc[loc]);
			}
		if (sigmaT2_Loc[loc] != 0.0) {
			Flevel = Flevels_[mII].Loc(FsptWord);//Fspt
			 FstatsLoc_[mII][loc][Flevel] =sigmaS2_Loc[loc]/ sigmaT2_Loc[loc];
			Flevel = Flevels_[mII].Loc(FltWord);//Flt
			 FstatsLoc_[mII][loc][Flevel] = sigmaL2_Loc[loc] / sigmaT2_Loc[loc];
			Flevel = Flevels_[mII].Loc(FlsptWord);//Fst
			 FstatsLoc_[mII][loc][Flevel] = sigmaSL_Loc[loc] /sigmaT2_Loc[loc];
			Flevel = Flevels_[mII].Loc(FitWord);//Fit
			 FstatsLoc_[mII][loc][Flevel]=1.0 - (sigmaG_Loc[loc] / sigmaT2_Loc[loc]);
			Flevel = Flevels_[mII].Loc(FstWord);//Fst
			 FstatsLoc_[mII][loc][Flevel]=(sigmaS2_Loc[loc] + sigmaL2_Loc[loc] + sigmaSL_Loc[loc])/ sigmaT2_Loc[loc];
			Flevel = Flevels_[mII].Loc(FsinltWord);//Fsinlt
			FstatsLoc_[mII][loc][Flevel] = sigmaSinL2_Loc[loc]/ sigmaT2_Loc[loc];
			Flevel = Flevels_[mII].Loc(FsinsptWord);//Fsinspt
			FstatsLoc_[mII][loc][Flevel] = sigmaSinSp2_Loc[loc]/ sigmaT2_Loc[loc];
			}//end if sigmaT2_Loc
		if( ( sigmaT2_Loc[loc] - sigmaL2_Loc[loc]) != 0.0){
			Flevel = Flevels_[mII].Loc(FslWord);//Fsl
			FstatsLoc_[mII][loc][Flevel]= sigmaSinL2_Loc[loc]/ ( sigmaT2_Loc[loc] - sigmaL2_Loc[loc]);
			}
		if( ( sigmaT2_Loc[loc] - sigmaS2_Loc[loc]) != 0.0){
			Flevel = Flevels_[mII].Loc(FsspWord);//Fssp
			FstatsLoc_[mII][loc][Flevel]= sigmaSinSp2_Loc[loc]/ ( sigmaT2_Loc[loc] - sigmaS2_Loc[loc]);
			}
		if (sigmaT3_Loc[loc] != 0.0) {
			Flevel = Flevels_[mIII].Loc(FsptWord);//Fspt
			 FstatsLoc_[mIII][loc][Flevel] =sigmaS2_Loc[loc]/ sigmaT3_Loc[loc];
			Flevel = Flevels_[mIII].Loc(FltWord);//Flt
			 FstatsLoc_[mIII][loc][Flevel] = sigmaL1_Loc[loc] / sigmaT3_Loc[loc];
			Flevel = Flevels_[mIII].Loc(FlsptWord);//Fst
			 FstatsLoc_[mIII][loc][Flevel] = sigmaSL_Loc[loc] /sigmaT3_Loc[loc];
			Flevel = Flevels_[mIII].Loc(FitWord);//Fit
			 FstatsLoc_[mIII][loc][Flevel]=1.0 - (sigmaG_Loc[loc] / sigmaT3_Loc[loc]);
			Flevel = Flevels_[mIII].Loc(FstWord);//Fst
			 FstatsLoc_[mIII][loc][Flevel]=(sigmaS1_Loc[loc] + sigmaL1_Loc[loc] + sigmaSL_Loc[loc])/ sigmaT3_Loc[loc];
			Flevel = Flevels_[mIII].Loc(FsinltWord);//Fsinlt
			FstatsLoc_[mIII][loc][Flevel] = sigmaSinL1_Loc[loc]/ sigmaT3_Loc[loc];
			Flevel = Flevels_[mIII].Loc(FsinsptWord);//Fsinspt
			FstatsLoc_[mIII][loc][Flevel] = sigmaSinSp2_Loc[loc]/ sigmaT3_Loc[loc];
			}//end if sigmaT3_Loc
		if( (sigmaT3_Loc[loc] - sigmaL1_Loc[loc]) != 0.0){
			Flevel = Flevels_[mIII].Loc(FslWord);//Fsl
			FstatsLoc_[mIII][loc][Flevel]= sigmaSinL1_Loc[loc]/ (sigmaT3_Loc[loc] - sigmaL1_Loc[loc]);
			}
		if( (sigmaT3_Loc[loc] - sigmaS2_Loc[loc]) != 0.0){
			Flevel = Flevels_[mIII].Loc(FsspWord);//Fssp
			FstatsLoc_[mIII][loc][Flevel]= sigmaSinSp2_Loc[loc]/ (sigmaT3_Loc[loc] - sigmaS2_Loc[loc]);
			}
		if (sigmaIG_Loc[loc] != 0.0){
			Flevel = Flevels_[mI].Loc(FisWord);//Fis
			 FstatsLoc_[mI][loc][Flevel] = 1.0 - (sigmaG_Loc[loc] /sigmaIG_Loc[loc]);
			 FstatsLoc_[mII][loc][Flevel] = 1.0 - (sigmaG_Loc[loc] /sigmaIG_Loc[loc]);
			 FstatsLoc_[mIII][loc][Flevel] = 1.0 - (sigmaG_Loc[loc] /sigmaIG_Loc[loc]);
			}//end if sigmaIG_Loc
		}//end for j

	vc_[mI][0] = sigmaS1_sum;//Fmethod I
	vc_[mI][1] = sigmaL1_sum;
	vc_[mI][2] = sigmaSL_sum;
	vc_[mI][3] = sigmaI_sum;
	vc_[mI][4] = sigmaG_sum;
	vc_[mI][5] = sigmaT1_sum;
	vc_[mII][0] = sigmaS2_sum;//Fmethod II
	vc_[mII][1] = sigmaL2_sum;
	vc_[mII][2] = sigmaSL_sum;
	vc_[mII][3] = sigmaI_sum;
	vc_[mII][4] = sigmaG_sum;
	vc_[mII][5] = sigmaT2_sum;
	vc_[mIII][0] = sigmaS2_sum;//Fmethod III
	vc_[mIII][1] = sigmaL1_sum;
	vc_[mIII][2] = sigmaSL_sum;
	vc_[mIII][3] = sigmaI_sum;
	vc_[mIII][4] = sigmaG_sum;
	vc_[mIII][5] = sigmaT3_sum;
	if (sigmaT1_sum != 0.0){
		Flevel = Flevels_[mI].Loc(FsptWord);//Fspt
		Fstats_[mI][Flevel] = sigmaS1_sum / sigmaT1_sum;
		Flevel = Flevels_[mI].Loc(FltWord);//Flt
		Fstats_[mI][Flevel] = sigmaL1_sum / sigmaT1_sum;
		Flevel = Flevels_[mI].Loc(FlsptWord);//Fst
		Fstats_[mI][Flevel] = sigmaSL_sum /sigmaT1_sum;
		Flevel = Flevels_[mI].Loc(FitWord);//Fit
		Fstats_[mI][Flevel]=1.0 - (sigmaG_sum / sigmaT1_sum);
		Flevel = Flevels_[mI].Loc(FstWord);//Fst
		Fstats_[mI][Flevel]=(sigmaS1_sum + sigmaL1_sum + sigmaSL_sum) / sigmaT1_sum;
		Flevel = Flevels_[mI].Loc(FsinltWord);//Fsinlt
		Fstats_[mI][Flevel] = sigmaSinL1_sum/ sigmaT1_sum;
		Flevel = Flevels_[mI].Loc(FsinsptWord);//Fsinspt
		Fstats_[mI][Flevel] = sigmaSinSp1_sum/ sigmaT1_sum;
		}//end if sigmaT1_sum
	if( (sigmaT1_sum - sigmaL1_sum) != 0.0){
		Flevel = Flevels_[mI].Loc(FslWord);//Fsl
		Fstats_[mI][Flevel]= sigmaSinL1_sum/ (sigmaT1_sum - sigmaL1_sum);
		}
	if( (sigmaT1_sum - sigmaS1_sum) != 0.0){
		Flevel = Flevels_[mI].Loc(FsspWord);//Fssp
		Fstats_[mI][Flevel]= sigmaSinSp1_sum/ (sigmaT1_sum - sigmaS1_sum);
		}
	if (sigmaT2_sum != 0.0){
		Flevel = Flevels_[mII].Loc(FsptWord);//Fspt
		Fstats_[mII][Flevel] = sigmaS2_sum / sigmaT2_sum;
		Flevel = Flevels_[mII].Loc(FltWord);//Flt
		Fstats_[mII][Flevel] = sigmaL2_sum / sigmaT2_sum;
		Flevel = Flevels_[mII].Loc(FlsptWord);//Fst
		Fstats_[mII][Flevel] = sigmaSL_sum /sigmaT2_sum;
		Flevel = Flevels_[mII].Loc(FitWord);//Fit
		Fstats_[mII][Flevel]=1.0 - (sigmaG_sum / sigmaT2_sum);
		Flevel = Flevels_[mII].Loc(FstWord);//Fst
		Fstats_[mII][Flevel]=(sigmaS2_sum + sigmaL2_sum + sigmaSL_sum) / sigmaT2_sum;
		Flevel = Flevels_[mII].Loc(FsinltWord);//Fsinlt
		Fstats_[mII][Flevel] = sigmaSinL2_sum/ sigmaT2_sum;
		Flevel = Flevels_[mII].Loc(FsinsptWord);//Fsinspt
		Fstats_[mII][Flevel] = sigmaSinSp2_sum/ sigmaT2_sum;
		}//end if sigmaT2_sum
	if( ( sigmaT2_sum - sigmaL2_sum) != 0.0){
		Flevel = Flevels_[mII].Loc(FslWord);//Fsl
		Fstats_[mII][Flevel]= sigmaSinL2_sum/ ( sigmaT2_sum - sigmaL2_sum);
		}
	if( ( sigmaT2_sum - sigmaS2_sum) != 0.0){
		Flevel = Flevels_[mII].Loc(FsspWord);//Fssp
		Fstats_[mII][Flevel]= sigmaSinSp2_sum/ ( sigmaT2_sum - sigmaS2_sum);
		}
	if (sigmaT3_sum != 0.0){
		Flevel = Flevels_[mIII].Loc(FsptWord);//Fspt
		Fstats_[mIII][Flevel] = sigmaS2_sum / sigmaT3_sum;
		Flevel = Flevels_[mIII].Loc(FltWord);//Flt
		Fstats_[mIII][Flevel] = sigmaL1_sum / sigmaT3_sum;
		Flevel = Flevels_[mIII].Loc(FlsptWord);//Fst
		Fstats_[mIII][Flevel] = sigmaSL_sum /sigmaT3_sum;
		Flevel = Flevels_[mIII].Loc(FitWord);//Fit
		Fstats_[mIII][Flevel]=1.0 - (sigmaG_sum / sigmaT3_sum);
		Flevel = Flevels_[mIII].Loc(FstWord);//Fst
		Fstats_[mIII][Flevel]=(sigmaS2_sum + sigmaL1_sum + sigmaSL_sum) / sigmaT3_sum;
		Flevel = Flevels_[mIII].Loc(FsinltWord);//Fsinlt
		Fstats_[mIII][Flevel] = sigmaSinL1_sum/ sigmaT3_sum;
		Flevel = Flevels_[mIII].Loc(FsinsptWord);//Fsinspt
		Fstats_[mIII][Flevel] = sigmaSinSp2_sum/ sigmaT3_sum;
		}//end if sigmaT3_sum
	if( (sigmaT3_sum - sigmaL1_sum) != 0.0){
		Flevel = Flevels_[mIII].Loc(FslWord);//Fsl
		Fstats_[mIII][Flevel]= sigmaSinL1_sum/ (sigmaT3_sum - sigmaL1_sum);
		}
	if( (sigmaT3_sum - sigmaS2_sum) != 0.0){
		Flevel = Flevels_[mIII].Loc(FsspWord);//Fssp
		Fstats_[mIII][Flevel]= sigmaSinSp2_sum/ (sigmaT3_sum - sigmaS2_sum);
		}
	if (sigmaIG_sum != 0.0){
		Flevel = Flevels_[mI].Loc(FisWord);//Fis
		Fstats_[mI][Flevel] = 1.0 - (sigmaG_sum /sigmaIG_sum);
		Fstats_[mII][Flevel] = 1.0 - (sigmaG_sum /sigmaIG_sum);
		Fstats_[mIII][Flevel] = 1.0 - (sigmaG_sum /sigmaIG_sum);
		}//end if sigmaIG_sum

	//now do the jackknife calculations, over loci
	for (int j=0;j<variableLocusNumCodom_.len();++j){//skip monomorphic loci
		int loc = variableLocusNumCodom_[j];//the actual locus		
		if ((sigmaT1_sum - sigmaT1_Loc[loc]) != 0.0) {
			Flevel = Flevels_[mI].Loc(FsptWord);//Fspt
			jackFstatsLoc_[mI][loc][Flevel] =(sigmaS1_sum - sigmaS1_Loc[loc])/ (sigmaT1_sum - sigmaT1_Loc[loc]);
			Flevel = Flevels_[mI].Loc(FltWord);//Flt
			jackFstatsLoc_[mI][loc][Flevel] = (sigmaL1_sum - sigmaL1_Loc[loc])/ (sigmaT1_sum - sigmaT1_Loc[loc]);
			Flevel = Flevels_[mI].Loc(FlsptWord);//Flspt
			jackFstatsLoc_[mI][loc][Flevel] = (sigmaSL_sum - sigmaSL_Loc[loc]) /(sigmaT1_sum - sigmaT1_Loc[loc]);
			Flevel = Flevels_[mI].Loc(FitWord);//Fit
			jackFstatsLoc_[mI][loc][Flevel]=1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) / (sigmaT1_sum - sigmaT1_Loc[loc]));
			Flevel = Flevels_[mI].Loc(FstWord);//Fst
			jackFstatsLoc_[mI][loc][Flevel]=((sigmaS1_sum+sigmaL1_sum+sigmaSL_sum) - (sigmaS1_Loc[loc]+sigmaL1_Loc[loc]+sigmaSL_Loc[loc]))
							 / (sigmaT1_sum - sigmaT1_Loc[loc]);
			Flevel = Flevels_[mI].Loc(FsinltWord);//Fsinlt
			jackFstatsLoc_[mI][loc][Flevel] = (sigmaSinL1_sum - sigmaSinL1_Loc[loc])/ (sigmaT1_sum - sigmaT1_Loc[loc]);
			Flevel = Flevels_[mI].Loc(FsinsptWord);//Fsinspt
			jackFstatsLoc_[mI][loc][Flevel] = (sigmaSinSp1_sum - sigmaSinSp1_Loc[loc])/  (sigmaT1_sum - sigmaT1_Loc[loc]);
			}//end if sigmaT1_sum
		if( (sigmaT1_sum - sigmaT1_Loc[loc]) - (sigmaL1_sum - sigmaL1_Loc[loc]) != 0.0){
			Flevel = Flevels_[mI].Loc(FslWord);//Fsl
			jackFstatsLoc_[mI][loc][Flevel]= (sigmaSinL1_sum - sigmaSinL1_Loc[loc])
					/ (sigmaT1_sum - sigmaT1_Loc[loc]) - (sigmaL1_sum - sigmaL1_Loc[loc]);
			}
		if(  (sigmaT1_sum - sigmaT1_Loc[loc]) - (sigmaS1_sum - sigmaS1_Loc[loc]) != 0.0){
			Flevel = Flevels_[mI].Loc(FsspWord);//Fssp
			jackFstatsLoc_[mI][loc][Flevel]= (sigmaSinSp1_sum - sigmaSinSp1_Loc[loc])
					/  (sigmaT1_sum - sigmaT1_Loc[loc]) - (sigmaS1_sum - sigmaS1_Loc[loc]);
			}
		if ((sigmaT2_sum - sigmaT2_Loc[loc]) != 0.0) {
			Flevel = Flevels_[mII].Loc(FsptWord);//Fspt
			jackFstatsLoc_[mII][loc][Flevel] =(sigmaS2_sum - sigmaS2_Loc[loc])/ (sigmaT2_sum - sigmaT2_Loc[loc]);
			Flevel = Flevels_[mII].Loc(FltWord);//Flt
			jackFstatsLoc_[mII][loc][Flevel] = (sigmaL2_sum - sigmaL2_Loc[loc])/ (sigmaT2_sum - sigmaT2_Loc[loc]);
			Flevel = Flevels_[mII].Loc(FlsptWord);//Flspt
			jackFstatsLoc_[mII][loc][Flevel] = (sigmaSL_sum - sigmaSL_Loc[loc]) /(sigmaT2_sum - sigmaT2_Loc[loc]);
			Flevel = Flevels_[mII].Loc(FitWord);//Fit
			jackFstatsLoc_[mII][loc][Flevel]=1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) / (sigmaT2_sum - sigmaT2_Loc[loc]));
			Flevel = Flevels_[mII].Loc(FstWord);//Fst
			jackFstatsLoc_[mII][loc][Flevel]=((sigmaS2_sum+sigmaL2_sum+sigmaSL_sum) - (sigmaS2_Loc[loc]+sigmaL2_Loc[loc]+sigmaSL_Loc[loc]))
							 / (sigmaT2_sum - sigmaT2_Loc[loc]);
			Flevel = Flevels_[mII].Loc(FsinltWord);//Fsinlt
			jackFstatsLoc_[mII][loc][Flevel] = (sigmaSinL2_sum - sigmaSinL2_Loc[loc])/ (sigmaT2_sum - sigmaT2_Loc[loc]);
			Flevel = Flevels_[mII].Loc(FsinsptWord);//Fsinspt
			jackFstatsLoc_[mII][loc][Flevel] = (sigmaSinSp2_sum - sigmaSinSp2_Loc[loc])/  (sigmaT2_sum - sigmaT2_Loc[loc]);
			}//end if sigmaT2_sum
		if( (sigmaT2_sum - sigmaT2_Loc[loc]) - (sigmaL2_sum - sigmaL2_Loc[loc]) != 0.0){
			Flevel = Flevels_[mII].Loc(FslWord);//Fsl
			jackFstatsLoc_[mII][loc][Flevel]= (sigmaSinL2_sum - sigmaSinL2_Loc[loc])
					/ (sigmaT2_sum - sigmaT2_Loc[loc]) - (sigmaL2_sum - sigmaL2_Loc[loc]);
			}
		if(  (sigmaT2_sum - sigmaT2_Loc[loc]) - (sigmaS2_sum - sigmaS2_Loc[loc]) != 0.0){
			Flevel = Flevels_[mII].Loc(FsspWord);//Fssp
			jackFstatsLoc_[mII][loc][Flevel]= (sigmaSinSp2_sum - sigmaSinSp2_Loc[loc])
					/  (sigmaT2_sum - sigmaT2_Loc[loc]) - (sigmaS2_sum - sigmaS2_Loc[loc]);
			}
		if ((sigmaT3_sum - sigmaT3_Loc[loc]) != 0.0) {
			Flevel = Flevels_[mIII].Loc(FsptWord);//Fspt
			jackFstatsLoc_[mIII][loc][Flevel] =(sigmaS2_sum - sigmaS2_Loc[loc])/ (sigmaT3_sum - sigmaT3_Loc[loc]);
			Flevel = Flevels_[mIII].Loc(FltWord);//Flt
			jackFstatsLoc_[mIII][loc][Flevel] = (sigmaL1_sum - sigmaL1_Loc[loc])/ (sigmaT3_sum - sigmaT3_Loc[loc]);
			Flevel = Flevels_[mIII].Loc(FlsptWord);//Flspt
			jackFstatsLoc_[mIII][loc][Flevel] = (sigmaSL_sum - sigmaSL_Loc[loc]) /(sigmaT3_sum - sigmaT3_Loc[loc]);
			Flevel = Flevels_[mIII].Loc(FitWord);//Fit
			jackFstatsLoc_[mIII][loc][Flevel]=1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) / (sigmaT3_sum - sigmaT3_Loc[loc]));
			Flevel = Flevels_[mIII].Loc(FstWord);//Fst
			jackFstatsLoc_[mIII][loc][Flevel]=((sigmaS2_sum+sigmaL1_sum+sigmaSL_sum) - (sigmaS2_Loc[loc]+sigmaL1_Loc[loc]+sigmaSL_Loc[loc]))
							 / (sigmaT3_sum - sigmaT3_Loc[loc]);
			Flevel = Flevels_[mIII].Loc(FsinltWord);//Fsinlt
			jackFstatsLoc_[mIII][loc][Flevel] = (sigmaSinL1_sum - sigmaSinL1_Loc[loc])/ (sigmaT3_sum - sigmaT3_Loc[loc]);
			Flevel = Flevels_[mIII].Loc(FsinsptWord);//Fsinspt
			jackFstatsLoc_[mIII][loc][Flevel] = (sigmaSinSp2_sum - sigmaSinSp2_Loc[loc])/  (sigmaT3_sum - sigmaT3_Loc[loc]);
			}//end if sigmaT3_sum
		if( (sigmaT3_sum - sigmaT3_Loc[loc]) - (sigmaL1_sum - sigmaL1_Loc[loc]) != 0.0){
			Flevel = Flevels_[mIII].Loc(FslWord);//Fsl
			jackFstatsLoc_[mIII][loc][Flevel]= (sigmaSinL1_sum - sigmaSinL1_Loc[loc])
					/ (sigmaT3_sum - sigmaT3_Loc[loc]) - (sigmaL1_sum - sigmaL1_Loc[loc]);
			}
		if(  (sigmaT3_sum - sigmaT3_Loc[loc]) - (sigmaS2_sum - sigmaS2_Loc[loc]) != 0.0){
			Flevel = Flevels_[mIII].Loc(FsspWord);//Fssp
			jackFstatsLoc_[mIII][loc][Flevel]= (sigmaSinSp2_sum - sigmaSinSp2_Loc[loc])
					/  (sigmaT3_sum - sigmaT3_Loc[loc]) - (sigmaS2_sum - sigmaS2_Loc[loc]);
			}
		if (sigmaIG_sum - sigmaIG_Loc[loc] != 0.0) {
			Flevel = Flevels_[mI].Loc(FisWord);//Fis
			jackFstatsLoc_[mI][loc][Flevel] = 1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) /(sigmaIG_sum - sigmaIG_Loc[loc]));
			jackFstatsLoc_[mII][loc][Flevel] = 1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) /(sigmaIG_sum - sigmaIG_Loc[loc]));
			jackFstatsLoc_[mIII][loc][Flevel] = 1.0 - ((sigmaG_sum - sigmaG_Loc[loc]) /(sigmaIG_sum - sigmaIG_Loc[loc]));
			}//end if sigmaIG_sum
		}//end for j

	FstatsCurrent_=vcCurrent_=1;
	FsptWord=FltWord=FlsptWord=FitWord=FstWord=FisWord=FslWord=FsspWord=FsinltWord=FsinsptWord=NULL;//forget these; deleted elsewhere
	
	speciesStr = localityStr = popStr = indivinPopStr = NULL;
	delete rawAnovaData; rawAnovaData=NULL;
	speciesStr = localityStr = popStr = indivinPopStr = NULL;
	}//end  FstatisticAnalysisCodom::FactorialFstatsIII




void FstatisticAnalysisCodom::FactorialFstatsIIIGuts(dataSetRec& rawAnovaData, ObjList<aList<long double> >& varianceComponents, aList<int>& locusNums, StringList& locusByAlleleNames){
	//gets variance components for loci having the same sampling regimes
	MatrixList Zmatrices;
	int loc = locusNums[0];//first locus=0; all loci the same samle size
	aList<long double> nList, dfList;//these hold coefficients and degrees of freedom
	CalculateFactorialVarCompCoeffsIII(nList, dfList, loc);//loc=first and all have the same sample size
	//calculate Z matrices and then remainder sums of squares
	MatrixList ZSList, ZLList0, ZLList1, ZIList;
	ObjList<MatrixList> ZSLList0,  ZSLList1, ZSIList;
	ObjList<ObjList<MatrixList> > ZSLIList;
	for(int sp00=0;sp00<ngroups_;++sp00){
		ZSLList0.Append(NEW MatrixList);
		ZSLList1.Append(NEW MatrixList);
		ZSIList.Append(NEW MatrixList);
		ZSLIList.Append(NEW ObjList<MatrixList>());
		for(int p00=0;p00<groupHierarchy_[sp00].len();++p00){
			ZSLIList[sp00].Append(NEW MatrixList());}
		}
	for(int sp0=0;sp0<ngroups_;++sp0){
		int nSp0=0;
		for(int p0=0;p0<groupHierarchy_[sp0].len();++p0){
			int p = groupHierarchy_[sp0][p0];
			int n00 = GetLocusSampleSize(p,loc);
			nSp0 += n00;
			ZSLList0[sp0].Append(OneMatrix((long double)1.0,n00,1));//relies on properly sorted data
			for(int i0=0;i0<individuals(p);++i0){
				int n000 = GetLocusSampleSize(p,i0,loc);
				if(n000>0){
					ZSLIList[sp0][p0].Append(OneMatrix((long double)1.0,n000,1));}
				}//end for i0	
			ZSIList[sp0].Append(ZSLIList[sp0][p0].DirectSum());
			}//end for sp0
		ZSList.Append(OneMatrix((long double)1.0,nSp0,1));	
		ZLList0.Append(ZSLList0[sp0].DirectSum());		
		ZIList.Append(ZSIList[sp0].DirectSum());		
		}//end for sp0
	//special handling for ZL so that missing columns get zeros
	ObjList<aList<int> > popNums;//this sorts populations by number in factorial array
	for(int sp1=0;sp1<groupHierarchy_.len();++sp1){
		popNums.Append(NEW aList<int>());
		for(int L1=0;L1<localities();++L1){
			popNums[sp1].Append(-1);//-1 indicates empty cell
			}}
	for(int pop00=0;pop00<npop_;++pop00){
		int gr=GroupMembership(pop00);
		int L01=GetLocality(pop00);
		popNums[gr][L01] = pop00;
		}
	for(int sp1=0;sp1<ngroups_;++sp1){
		aList<int> emptyLocalities;
		int nSp1=0;
		for(int p1=0;p1<localities();++p1){
			int pp = popNums[sp1][p1];
			if (pp==-1){//pop is empty for all loci
					emptyLocalities.Append(p1);}
				else{
					int n00L = GetLocusSampleSize(pp,loc);
					if(n00L==0){//pop is empty for this locus
							emptyLocalities.Append(p1);}
						else{
							nSp1 += n00L;
							ZSLList1[sp1].Append(OneMatrix((long double)1.0,n00L,1));}}//relies on properly sorted data
			}//end for p1
		aMatrix<long double>* ZLsp = ZSLList1[sp1].DirectSum();
		for(int missing=0;missing<emptyLocalities.len();++missing){
			aVector<long double> zeroCol((long double)0.0, nSp1);
			ZLsp->InsertColumn(zeroCol, emptyLocalities[missing]);}
		ZLList1.Append(ZLsp);		
		}//end for sp1
	aMatrix<long double>* ZS = ZSList.DirectSum();
	Zmatrices.Append(ZS); ZS=NULL;
	aMatrix<long double>* ZL = ZLList1.ConcatRows();
	Zmatrices.Append(ZL);ZL=NULL;
	aMatrix<long double>* ZSL = ZLList0.DirectSum();
	Zmatrices.Append(ZSL);ZSL=NULL;
	aMatrix<long double>* ZI = ZIList.DirectSum();
	Zmatrices.Append(ZI);ZI=NULL;
	ZSList.Clear();//free memory
	ZLList0.Clear();//free memory
	ZLList1.Clear();//free memory
	ZSLList0.Clear();//free memory
	ZSLList1.Clear();//free memory
	ZSIList.Clear();//free memory
	ZSLIList.Clear();//free memory
	Zmatrices.Prepend(OneMatrix((long double)1.0,dfList.Last()+1,1));//one matrix of length n___

	//replicate and consolidate steps in GeneralLinearModel::varcompsANOVA
	matrixHandle<long double> invZprimeZH(NEW aMatrix<long double>());//initially empty; filled at first calculation
	matrixHandle<long double> invZ1primeZ1H(NEW aMatrix<long double>());//initially empty; filled at first calculation
	matrixHandle<long double> invZ1SprimeZ1SH(NEW aMatrix<long double>());//initially empty; filled at first calculation
	matrixHandle<long double> invZ1LprimeZ1LH(NEW aMatrix<long double>());//initially empty; filled at first calculation
	matrixHandle<long double> invZ1SLprimeZ1SLH(NEW aMatrix<long double>());//initially empty; filled at first calculation
	matrixHandle<long double> invZ1SLSLprimeZ1SLSLH(NEW aMatrix<long double>());//initially empty; filled at first calculation
	GLM* lm = NEW GLM();
	aList<int> omitItems;//when determining reduced models
	MatrixList* R_muabcd_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
	aMatrix<long double>* Z = lm->MakeDesignMatrix(*R_muabcd_DesignComponents, 0);//= full model
	delete R_muabcd_DesignComponents; R_muabcd_DesignComponents=NULL;
	MatrixList* R_muabc_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
	omitItems.Append(4);//individuals omitted
	aMatrix<long double>* Z1SLSL = lm->MakeDesignMatrix(*R_muabc_DesignComponents, omitItems);//= reduced model
	delete R_muabc_DesignComponents; R_muabc_DesignComponents=NULL;
	MatrixList* R_muab_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
	omitItems.Append(3);//populations and individuals omitted
	aMatrix<long double>* Z1SL = lm->MakeDesignMatrix(*R_muab_DesignComponents, omitItems);//= reduced model
	delete R_muab_DesignComponents; R_muab_DesignComponents=NULL;
	MatrixList* R_mua_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
	omitItems.Append(2);//localities, populations and individuals omitted
	aMatrix<long double>* Z1S = lm->MakeDesignMatrix(*R_mua_DesignComponents, omitItems);//= reduced model
	delete R_mua_DesignComponents; R_mua_DesignComponents=NULL;
	MatrixList* R_mu_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
	omitItems.Append(1);//species, localities, populations and individuals omitted
	aMatrix<long double>* Z1 = lm->MakeDesignMatrix(*R_mu_DesignComponents, omitItems);//= reduced model
	delete R_mu_DesignComponents; R_mu_DesignComponents=NULL;
	MatrixList* R_mub_DesignComponents = NEW MatrixList(Zmatrices);//work with copy
	omitItems.Delete(2);//species, populations and individuals omitted
	aMatrix<long double>* Z1L = lm->MakeDesignMatrix(*R_mub_DesignComponents, omitItems);//= reduced model
	delete R_mub_DesignComponents; R_mub_DesignComponents=NULL;
	for(int locallele=0;locallele<locusByAlleleNames.len();++locallele){
		aString& colTitle0 = locusByAlleleNames[locallele];
		dataColumnRecNum<long double>* depVarCol0 = (dataColumnRecNum<long double>*) &(rawAnovaData.GetDataColumn(colTitle0));
		aList<long double>& varList0 = depVarCol0->dataList();
		aMatrix<long double> Pmatrix;
		aVector<long double> aVecPtr0(varList0);
		Pmatrix.ConcatCol(aVecPtr0);

		long double R_mu, R_mua, R_mub, R_muab, R_muabc, R_muabcd;//sums of squares from including model factors
		R_mu = R_mua = R_mub = R_muab = R_muabc = R_muabcd = 0.0;
		long double SSE;  //dummy variable
		long double PprimeP;//total sum of squares
		//R_muabcd
		lm->RegressionSS(Z, &Pmatrix, R_muabcd, SSE, PprimeP, invZprimeZH);//invZprimeZH replaced if empty
		//R_muabc
		lm->RegressionSS(Z1SLSL, &Pmatrix, R_muabc, SSE, PprimeP, invZ1SLSLprimeZ1SLSLH);//invZ1SLSLprimeZ1SLSLH replaced if empty
		//R_muab
		lm->RegressionSS(Z1SL, &Pmatrix, R_muab, SSE, PprimeP, invZ1SLprimeZ1SLH);//invZ1SLprimeZ1SLH replaced if empty
		//R_mua
		lm->RegressionSS(Z1S, &Pmatrix, R_mua, SSE, PprimeP, invZ1SprimeZ1SH);//invZ1SprimeZ1SH replaced if empty
		//R_mu
		lm->RegressionSS(Z1, &Pmatrix, R_mu, SSE, PprimeP, invZ1primeZ1H);//invZ1primeZ1H replaced if empty
		//R_mub
		lm->RegressionSS(Z1L, &Pmatrix, R_mub, SSE, PprimeP, invZ1LprimeZ1LH);//invZ1primeZ1H replaced if empty
		
		long double SSSP1 = R_mua - R_mu;
		long double SSL1 = R_muab - R_mua;
		long double SSSP2 = R_muab - R_mub;
		long double SSL2 = R_mub - R_mu;
		long double SSLSP = R_muabc - R_muab;
		long double SSI = R_muabcd - R_muabc;
		long double SSG = PprimeP - R_muabcd;
		
		long double sigmaSqG = SSG/dfList[4];
		long double sigmaSqI = (  SSI - (dfList[3]*sigmaSqG)  ) / nList[0];
		long double sigmaSqSL = (  SSLSP - (nList[1] * sigmaSqI) - (dfList[2]*sigmaSqG)  ) / nList[2];
		long double sigmaSqL1 = (  SSL1 - (nList[4] * sigmaSqSL) - (nList[3] * sigmaSqI) - (dfList[1]*sigmaSqG)  ) / nList[5];
		long double sigmaSqS1 = (  SSSP1 - (nList[8] * sigmaSqL1) - (nList[7] * sigmaSqSL) - (nList[6] * sigmaSqI) - (dfList[0]*sigmaSqG)  ) / nList[9];
		long double sigmaSqS2 = (  SSSP1 - (nList[11] * sigmaSqSL) - (nList[10] * sigmaSqI) - (dfList[0]*sigmaSqG)  ) / nList[12];
		long double sigmaSqL2 = (  SSL2 - (nList[15] * sigmaSqS2) - (nList[14] * sigmaSqSL) - (nList[13] * sigmaSqI) - (dfList[1]*sigmaSqG)  ) / nList[16];
		long double sigmaSqT = PprimeP / dfList[5];//empirical total variance
			
		long double sigmaSqSinSp1 = sigmaSqT - sigmaSqS1 - sigmaSqI - sigmaSqG;
		long double sigmaSqSinSp2 = sigmaSqT - sigmaSqS2 - sigmaSqI - sigmaSqG;
		long double sigmaSqSinL1 = sigmaSqT - sigmaSqL1 - sigmaSqI - sigmaSqG;
		long double sigmaSqSinL2 = sigmaSqT - sigmaSqL2 - sigmaSqI - sigmaSqG;

		aList<long double>* vc = NEW aList<long double>();
		vc->Append(sigmaSqG);
		vc->Append(sigmaSqI);
		vc->Append(sigmaSqSL);
		vc->Append(sigmaSqL1);
		vc->Append(sigmaSqS1);
		vc->Append(sigmaSqL2);
		vc->Append(sigmaSqS2);
		vc->Append(sigmaSqT);
		vc->Append(sigmaSqSinSp1);
		vc->Append(sigmaSqSinSp2);
		vc->Append(sigmaSqSinL1);
		vc->Append(sigmaSqSinL2);
			
		varianceComponents.Append(vc);
		vc=NULL;//forget this
		}//end for locallele		


	delete Z; Z=NULL;//clear unused memory
	delete Z1SLSL; Z1SLSL=NULL;//clear unused memory
	delete Z1SL; Z1SL=NULL;//clear unused memory
	delete Z1S; Z1S=NULL;//clear unused memory
	delete Z1; Z1=NULL;//clear unused memory
	delete Z1L; Z1L=NULL;//clear unused memory
	delete lm; lm=NULL;
	}//end FstatisticAnalysisCodom::factorialFstatsIIIGuts


void FstatisticAnalysisCodom::PrintFStatisticsByLocus(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer){
	StringList namesOfLoci;
	CodominantMarkerData::GetLocusNames(namesOfLoci);
	FstatisticAnalysisCore::PrintFStatisticsByLocus(outFile, Fmethod, titleStr, namesOfLoci, colSpacer);
	}
	
	
void FstatisticAnalysisCodom::PrintFStatisticsByAllele(ostream& outFile, const int Fmethod, aString& titleStr, char colSpacer){
	StringList namesOfLoci;
	CodominantMarkerData::GetLocusNames(namesOfLoci);
	FstatisticAnalysisCore::PrintFStatisticsByAllele(outFile, Fmethod, titleStr, namesOfLoci, allelesPresent_, colSpacer);
	}





void  FstatisticAnalysisCodom::HierarchicalWeirTheta(void){
    //this routine calculates Weir & Cockerham's hierarchical
    // F-statistics among groups of populations
    //Notation follows Wier & Cockerham 1984, p.1365
    
    //can't handle anything but diploids with all loci scored
    
  	if(! FstatsCurrent_){
  		ClearFstats();
  		}
  	if((Fmethod_.Includes((int) hierWandC))){return;}
	if(nlociCodom_==0){return;}
	FstatsCurrent_=vcCurrent_=0;
	Fmethod_.Append((int) hierWandC);
	int m = Fmethod_.len()-1;
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, F-level
	Fstats_.Append(NEW aList<long double>());//F-level
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	Flevels_.Append(NEW StringList());//F-level
  
	vcMethod_.Append((int) hierWandC);
	vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
	vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
	vc_.Append(NEW aList<long double>());//vc-level
	vcLevels_.Append(NEW StringList());//vc-level


	//set up all the F-stat arrays
	// first the array describing the Fstat structure:
	aString* FWord=NEW aString("Fit"); Flevels_[m].Append(FWord);
	FWord=NEW aString("Fis"); Flevels_[m].Append(FWord);
	FWord=NEW aString("Fig"); Flevels_[m].Append(FWord);
	FWord=NEW aString("Fsg"); Flevels_[m].Append(FWord);
	FWord=NEW aString("Fst"); Flevels_[m].Append(FWord);
	FWord=NEW aString("Fgt"); Flevels_[m].Append(FWord);

	aString* vcWord=NEW aString("var(gr)");vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(p)");vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(i)");vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(g)");vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(t)");vcLevels_[m].Append(vcWord);

	//make and initialize the jackknife & by-locus Fstat lists
	for (int Flevel=0;Flevel<6;++Flevel){Fstats_[m].Append(0.0);}
	for (int vclevel=0;vclevel<5;++vclevel){vc_[m].Append(0.0);}
	//some intermediate variables
	aList<long double>* ALoc=NEW aList<long double>();
	aList<long double>* B1Loc=NEW aList<long double>();
	aList<long double>* B2Loc=NEW aList<long double>();
	aList<long double>* CLoc=NEW aList<long double>();
	aList<long double>* ABLoc=NEW aList<long double>();
	aList<long double>* BCLoc=NEW aList<long double>();
	aList<long double>* BBCLoc=NEW aList<long double>();
	aList<long double>* ABBCLoc=NEW aList<long double>();
	CheckForNewAlleles();
	UpdateVariableLoci();
	for (int j=0;j<variableLocusNumCodom_.len();++j){
		ALoc->Append(0.0);
		B1Loc->Append(0.0);
		B2Loc->Append(0.0);
		CLoc->Append(0.0);
		ABLoc->Append(0.0);
		BCLoc->Append(0.0);
		BBCLoc->Append(0.0);
		ABBCLoc->Append(0.0);
		aList<long double>* locFstats = NEW aList<long double>();
		FstatsLoc_[m].Append(locFstats);//loc, F-level
		aList<long double>* jackLocFstats = NEW aList<long double>();
		jackFstatsLoc_[m].Append(jackLocFstats);//loc, F-level
		for (int Flevel=0;Flevel<6;++Flevel){
			locFstats->Append(0.0);
			jackLocFstats->Append(0.0);
			}//end for Flevel
		ObjList<aList<long double> >* alleleFstats = NEW ObjList<aList<long double> >();
		FstatsAll_[m].Append(alleleFstats);//loc,allele, F-level
		StringList& locAlleleList=allelesPresent_[j];
		for (int a=0;a<locAlleleList.len();++a){
			aList<long double>* locAlleleFstats = NEW aList<long double>();
			alleleFstats->Append(locAlleleFstats);
			for (int Flevel=0;Flevel<6;++Flevel){locAlleleFstats->Append(0.0);}
			}//end for a
		//make and initialize the by-allele var comp list
		vcLoc_[m].Append(NEW aList<long double>());
		for (int vclevel=0;vclevel<5;++vclevel){vcLoc_[m][j].Append(0.0);}
		vcAll_[m].Append(NEW ObjList<aList<long double> >());
		for (int a=0;a<locAlleleList.len();++a){
			vcAll_[m][j].Append(NEW aList<long double>());
			for (int vclevel=0;vclevel<5;++vclevel){vcAll_[m][j][a].Append(0.0);}
			}//end for a							
		}//end for j
		
	long double Asum,B1sum,B2sum,Csum,ABsum,BCsum,BBCsum,ABBCsum;
	Asum=B1sum=B2sum=Csum=ABsum=BCsum=BBCsum=ABBCsum=0.0;
	for (int j=0;j<variableLocusNumCodom_.len();++j){
		int loc=variableLocusNumCodom_[j];
		StringList& locAlleleList=allelesPresent_[loc];
		for(int a=0;a<locAlleleList.len();++a){
			StatList<long double> qDot, nDot, nSample;
			ObjList<StatList<long double> > allFreqs;//group, pop
			long double mDot,qDotDot,nDotDot,MSP,MSI,MSG,MSD,n1,n2,n3,A,B1,B2,C;
			mDot=qDotDot=nDotDot=MSP=MSI=MSG=MSD=n1=n2=n3=A=B1=B2=C=0.0;
			for (int g=0;g<groupHierarchy_.len();++g){//FOR GROUP=1 TO NGROUPS
				aList<int>& pops = groupHierarchy_[g];
				mDot += (long double) pops.len();// += SIZE(GROUP)
				qDot.Append(0.0);//PDOT!(GROUP)=0
				nDot.Append(0.0);//NDOT!(GROUP)=0
				StatList<long double>* q=NEW StatList<long double>();
				allFreqs.Append(q);
				}//end for g
			//Calculate for the whole population set:
			for (int g=0;g<groupHierarchy_.len();++g){//FOR GROUP=1 TO NGROUPS
				aList<int>& pops = groupHierarchy_[g];
				StatList<long double>& q = allFreqs[g];
				for(int p=0;p<pops.len();++p){//FOR GPOP=1 TO SIZE(GROUP)
					int pop=pops[p];
					long double n=(long double) (GetLocusSampleSize(pop,loc));
					nSample.Append(n/(long double)2.0);//nSample is for 1N sample size, not 2N
					long double freq=GetFreq(pop,loc,a);
					q.Append(freq);
					//Average allelic frequency (PDOTDOT!) across groups
					qDotDot += n/(long double)(2.0)*freq;

					//Average allelic frequency within this group (PDOT!(GROUP))
					qDot.Replace(qDot[g]+n*freq, g);

					//Total sample size per locus (NDOTDOT)
					nDotDot += n;

					//Sample size per locus within this group
					nDot.Replace(nDot[g]+n, g);
					}//next p
				if (nDot[g]>0.0){
					qDot.Replace(qDot[g]/nDot[g], g);}
				}//end for g
			qDotDot /= nDotDot;

			//Calculate intermediate variables:
			//if all the naturally occurring groups are being used, then use R=NGROUPS
			//if just a sample of natural groups are used, use R=(NGROUPS-1)
			long double r = (long double)groupHierarchy_.len();
			for (int g=0;g<groupHierarchy_.len();++g){//FOR GROUP=1 TO NGROUPS
				long double NSQ=0.0;
				MSP += nDot[g] * (qDot[g] - qDotDot) * (qDot[g] - qDotDot);
				aList<int>& pops = groupHierarchy_[g];
				StatList<long double>& q = allFreqs[g];
				for(int p=0;p<pops.len();++p){//FOR GPOP=1 TO SIZE(GROUP)
					int pop=pops[p];
					long double n=nSample[g];
					MSD += n*(q[p]-qDot[g])*(q[p]-qDot[g]);
					//Be sure to divide heterozygosity by 2 when summing over alleles.
					long double Hexp = 2.0*q[p]*(1.0-q[p]);
					long double Hobs = GetObservedHeterozygosity(pop,loc,a);
					MSI += n*Hexp;
					MSI -= 0.5*n*Hobs; if(MSI==(-0.0)){ MSI=0.0;}
					MSG += n*Hobs;
					NSQ += n*n;
					}//end for p
				n1 += ((nDotDot-nDot[g])*NSQ)/(nDot[g]*nDotDot);
				n2 += nDot[g]*nDot[g];
				n3 +=  NSQ/nDot[g];
				}//end for g
			MSP *= (long double)2.0/(r-1.0);
			MSD *= (long double)2.0/(mDot-r); if(MSD==(long double)(-0.0)){ MSD=(long double)0.0;}
			MSI /= (nDotDot - mDot);if(MSI==(-0.0)){ MSI=(long double)0.0;}
			MSG /= (long double)2.0*nDotDot;
			n1 /= (r-1.0);if(n1==(long double)(-0.0)){ n1=(long double)0.0;}
			n2 = (nDotDot - n2/nDotDot)/(r-1.0);if(n2==(long double)(-0.0)){ n2=(long double)0.0;}
			n3 = (nDotDot - n3)/(mDot-r);if(n3==(long double)(-0.0)){ n3=(long double)0.0;}

			//Calculate final variables:
			A = (n3*MSP - n1*MSD - MSI*(n3-n1))/((long double)2.0*n2*n3); if(A==(long double)(-0.0)){ A=(long double)0.0;}
			B1 = (MSI - MSG)/(long double)2.0; if(B1==(long double)(-0.0)){ B1=(long double)0.0;}
			B2 = (MSD - MSI)/((long double)2.0*n3); if(B2==(long double)(-0.0)){ B2=(long double)0.0;}
			C = MSG; if(C==(long double)(-0.0)){ C=(long double)0.0;}

			//store Fstat values for each allele:
			ObjList<aList<long double> >& alleleFstats =FstatsAll_[m][loc];
			aList<long double>& locAlleleFstats = alleleFstats[a];
			vcAll_[m][loc][a][0] = A;
			vcAll_[m][loc][a][1] = B1;
			vcAll_[m][loc][a][2] = B2;
			vcAll_[m][loc][a][3] = C;
			vcAll_[m][loc][a][3] = A + B1 + B2 + C;
			if ((A+B1+B2+C)!=0.0) {
				aString levelName="Fgt"; 
				int Flevel = Flevels_[m].Loc(&levelName);
				locAlleleFstats.Replace(A/(A+B1+B2+C), Flevel);//WEIRFgtALL!(L,M)= A/(A+B1+B2+C)
				levelName="Fst"; 
				Flevel = Flevels_[m].Loc(&levelName);
				locAlleleFstats.Replace((A+B2)/(A+B1+B2+C), Flevel);//WEIRFstALL!(L,M)=(A+B2)/(A+B1+B2+C)
				levelName="Fit"; 
				Flevel = Flevels_[m].Loc(&levelName);
				locAlleleFstats.Replace(1.0-C/(A+B1+B2+C), Flevel);//WEIRFitALL!(L,M)=1.0-C/(A+B1+B2+C)
				}//end if A+B1+B2+C
			if ((B1+B2+C)!=0.0) {
				aString levelName="Fig"; 
				int Flevel = Flevels_[m].Loc(&levelName);
				locAlleleFstats.Replace(1.0-C/(B1+B2+C), Flevel);//WEIRFigALL!(L,M)=1.0-C/(B1+B2+C)
				levelName="Fsg"; Flevel = Flevels_[m].Loc(&levelName);
				locAlleleFstats.Replace(1.0-(B1+C)/(B1+B2+C), Flevel);//WEIRFsgALL!(L,M)=1.0-(B1+C)/(B1+B2+C)
				}//end if (B1+B2+C)
			if ((B1+C)!=0.0) {
				aString levelName="Fis"; 
				int Flevel = Flevels_[m].Loc(&levelName);
				locAlleleFstats.Replace(1.0-C/(B1+C), Flevel);//WEIRFisALL!(L,M)=1.0-C/(B1+C)
				}//end if (B1+C)

			//accumulate locus sums for weighted averages & jackknife
			ALoc->Replace(ALoc->Nth(loc) + A,loc);
			B1Loc->Replace(B1Loc->Nth(loc) + B1,loc);
			B2Loc->Replace(B2Loc->Nth(loc) + B2,loc);
			CLoc->Replace(CLoc->Nth(loc) + C,loc);
			ABLoc->Replace(ABLoc->Nth(loc) + A+B2,loc);
			BCLoc->Replace(BCLoc->Nth(loc) + B1+C,loc);
			BBCLoc->Replace(BBCLoc->Nth(loc) + B1+B2+C,loc);
			ABBCLoc->Replace(ABBCLoc->Nth(loc) + A+B1+B2+C,loc);

			//accumulate overall sums for weighted averages
			Asum += A;
			B1sum += B1;
			B2sum += B2;
			Csum += C;
			ABsum += A+B2;
			BCsum += BCsum+B1+C;
			BBCsum += BBCsum+B1+B2+C;
			ABBCsum += A+B1+B2+C;
			}//end for a

		vcLoc_[m][loc][0] = (*ALoc)[loc];
		vcLoc_[m][loc][1] = (*B1Loc)[loc];
		vcLoc_[m][loc][2] = (*B2Loc)[loc];
		vcLoc_[m][loc][3] = (*CLoc)[loc];
		vcLoc_[m][loc][4] = (*ABBCLoc)[loc];
		//store Fstat values for each locus:
		aList<long double>& locFstats = FstatsLoc_[m][loc];//loc, F-level
		if (ABBCLoc->Nth(loc) != 0.0) {
			aString levelName="Fgt"; 
			int Flevel = Flevels_[m].Loc(&levelName);
			locFstats.Replace(ALoc->Nth(loc)/ABBCLoc->Nth(loc), Flevel);//WEIRFgtLOC!(L)=ALOC!(L)/ABBCLOC!(L)
			levelName="Fst"; Flevel = Flevels_[m].Loc(&levelName);
			locFstats.Replace(ABLoc->Nth(loc)/ABBCLoc->Nth(loc), Flevel);//WEIRFstLOC!(L)=ABLOC!(L)/ABBCLOC!(L)
			levelName="Fit"; Flevel = Flevels_[m].Loc(&levelName);
			locFstats.Replace(1.0-CLoc->Nth(loc)/ABBCLoc->Nth(loc), Flevel);//WEIRFitLOC!(L)=1-(CLOC!(L)/ABBCLOC!(L))
			}//end if ABBCLoc->Nth(loc)
		if (BBCLoc->Nth(loc) != 0.0) {
			aString levelName="Fig"; 
			int Flevel = Flevels_[m].Loc(&levelName);
			locFstats.Replace(1.0-CLoc->Nth(loc)/BBCLoc->Nth(loc), Flevel);//WEIRFigLOC!(L)=1-(CLOC!(L)/BBCLOC!(L))
			levelName="Fsg"; Flevel = Flevels_[m].Loc(&levelName);
			locFstats.Replace(1.0-BCLoc->Nth(loc)/BBCLoc->Nth(loc), Flevel);//WEIRFsgLOC!(L)=1-(BCLOC!(L)/BBCLOC!(L))
			}//end if BBCLoc->Nth(loc)
		if (BCLoc->Nth(loc) != 0.0) {
			aString levelName="Fis"; 
			int Flevel = Flevels_[m].Loc(&levelName);
			locFstats.Replace(1.0-CLoc->Nth(loc)/BCLoc->Nth(loc), Flevel);//WEIRFisLOC!(L)=1-(CLOC!(L)/BCLOC!(L))
			}//end if BCLoc->Nth(loc)
		}//next j

	vc_[m][0] = Asum;
	vc_[m][1] = B1sum;
	vc_[m][2] = B2sum;
	vc_[m][3] = Csum;
	vc_[m][4] = ABBCsum;
	//store overall Fstat values:
	if (ABBCsum != 0.0) {
		aString levelName="Fgt"; 
		int Flevel = Flevels_[m].Loc(&levelName);
		Fstats_[m].Replace(Asum/ABBCsum, Flevel);//WEIRFgt!=ASUM!/ABBCSUM!
		levelName="Fst"; Flevel = Flevels_[m].Loc(&levelName);
		Fstats_[m].Replace(ABsum/ABBCsum, Flevel);//WEIRFst!=ABSUM!/ABBCSUM!
		levelName="Fit"; Flevel = Flevels_[m].Loc(&levelName);
		Fstats_[m].Replace(1.0-Csum/ABBCsum, Flevel);//WEIRFit!=1-(CSUM!/ABBCSUM!)
		}//end if ABBCsum
	if (BBCsum != 0.0) {
		aString levelName="Fig"; 
		int Flevel = Flevels_[m].Loc(&levelName);
		Fstats_[m].Replace(1.0-Csum/BBCsum, Flevel);// WEIRFig!=1-(CSUM!/BBCSUM!)
		levelName="Fsg"; Flevel = Flevels_[m].Loc(&levelName);
		Fstats_[m].Replace(1.0-BCsum/BBCsum, Flevel);//WEIRFsg!=1-(BCSUM!/BBCSUM!)
		}//end if BBCsum
	if (BCsum != 0.0) {
		aString levelName="Fis"; 
		int Flevel = Flevels_[m].Loc(&levelName);
		Fstats_[m].Replace(1.0-Csum/BCsum, Flevel);//WEIRFis!=1-(CSUM!/BCSUM!)
		}//end if BCsum



	//for jackknife variance estimates
	for (int j=0;j<variableLocusNumCodom_.len();++j){
		int loc=variableLocusNumCodom_[j];
		aList<long double>& jackLocFstats = jackFstatsLoc_[m][loc];//loc, F-level
		if (ABBCsum !=ABBCLoc->Nth(loc)){
			aString levelName="Fgt"; int Flevel = Flevels_[m].Loc(&levelName);
			long double val=(Asum-ALoc->Nth(loc))/(ABBCsum-ABBCLoc->Nth(loc));
			jackLocFstats.Replace(val, Flevel);
			levelName="Fst"; Flevel = Flevels_[m].Loc(&levelName);
			val=(ABsum-ABLoc->Nth(loc))/(ABBCsum-ABBCLoc->Nth(loc));
			jackLocFstats.Replace(val, Flevel);
			levelName="Fit"; Flevel = Flevels_[m].Loc(&levelName);
			val=1.0-(Csum-CLoc->Nth(loc))/(ABBCsum-ABBCLoc->Nth(loc));
			jackLocFstats.Replace(val, Flevel);//WEIRFit!=1-(CSUM!/ABBCSUM!)
			}//end if ABBCsum
		if (BBCsum != BBCLoc->Nth(loc)) {
			aString levelName="Fig"; int Flevel = Flevels_[m].Loc(&levelName);
			long double val=1.0 - (Csum-CLoc->Nth(loc))/(BBCsum-BBCLoc->Nth(loc));
			jackLocFstats.Replace(val, Flevel);
			levelName="Fsg"; Flevel = Flevels_[m].Loc(&levelName);
			val=1.0-(BCsum-BCLoc->Nth(loc))/(BBCsum-BBCLoc->Nth(loc));
			jackLocFstats.Replace(val, Flevel);
			}//end if BBCsum
		if (BCsum != BCLoc->Nth(loc)) {
			aString levelName="Fis"; int Flevel = Flevels_[m].Loc(&levelName);
			long double val=1.0 - (Csum-CLoc->Nth(loc))/(BCsum-BCLoc->Nth(loc));
			jackLocFstats.Replace(val, Flevel);
			}//end if BCsum
		}//end for j
	FstatsCurrent_=vcCurrent_=1;
	delete ALoc;ALoc=0;
	delete B1Loc;B1Loc=0;
	delete B2Loc;B2Loc=0;
	delete CLoc;CLoc=0;
	delete ABLoc;ABLoc=0;
	delete BCLoc;BCLoc=0;
	delete BBCLoc;BBCLoc=0;
	delete ABBCLoc;ABBCLoc=0;
	}//end  FstatisticAnalysisCodom::HierarchicalWeirTheta



long double FstatisticAnalysisCodom::Nm(long double FstDip, int /*Fmethod*/, int numTrueDemes, bool zygoteMigration){
	//assumes diploid inheritance
	if(FstDip<=zero) return (long double)1.0e6;
	if(FstDip>=one) return zero;
	long double Nmval=zero, scale = four;
	if(zygoteMigration){scale = eight;}
	if(numTrueDemes<2){numTrueDemes=0;}
		else{numTrueDemes=MAX((int)npop_,numTrueDemes);}
	if(numTrueDemes>1){
		long double d = (long double)numTrueDemes;
		scale = (d-one)*(d-one)*(d*d);}
	Nmval = scale * (one - FstDip)/(four*FstDip);
	return Nmval;
	}//FstatisticAnalysisCodom::Nm(long double, int, bool, bool)



long double FstatisticAnalysisCodom::NmFromDiploidFst(long double FstDip, int /*Fmethod*/, int numTrueDemes, bool zygoteMigration){
	//assumes diploid inheritance
	if(FstDip<=zero) return (long double)1.0e6;
	if(FstDip>=one) return zero;
	long double Nmval=zero;
	long double scale=one;
	if(numTrueDemes<2){numTrueDemes=0;}
		else{numTrueDemes=MAX((int)npop_,numTrueDemes);}
	if(numTrueDemes>1){
		long double d=(long double)numTrueDemes;
		scale = (one-d)*(one-d)/(d*d);}
	if(zygoteMigration){
			Nmval = scale * (one - FstDip)/(eight*FstDip);}
		else{
			Nmval = scale * (one - FstDip)/(four*FstDip);}
	return Nmval;
	}//FstatisticAnalysisCodom::NmFromDiploidFst(long double, int, bool, bool)




void FstatisticAnalysisCodom::WeirTheta (void){
	//calculates Weir & Cockerham's  F-statistics among populations,
	//to be used when there is only one group of pops.
	//Notation follows Wier & Cockerham 1984, eqns. 1-4, p.1359-60

	//clear out any previous calculations
  	if(! FstatsCurrent_){
  		ClearFstats();}
	if((Fmethod_.Includes((int) WandC))){return;}
	if(nlociCodom_==0){return;}
	FstatsCurrent_=vcCurrent_=0;
	Fmethod_.Append((int) WandC);
	int m = Fmethod_.len()-1;
	FstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	FstatsAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, F-level
	Fstats_.Append(NEW aList<long double>());//F-level
	jackFstatsLoc_.Append(NEW ObjList<aList<long double> >());//loc, F-level
	Flevels_.Append(NEW StringList());//F-level

	vcMethod_.Append((int) WandC);
	vcLoc_.Append(NEW ObjList<aList<long double> >());//loc, vc-level
	vcAll_.Append(NEW ObjList<ObjList<aList<long double> > >());//loc,allele, vc-level
	vc_.Append(NEW aList<long double>());//vc-level
	vcLevels_.Append(NEW StringList());//vc-level

	//set up all the F-stat arrays
	// first the array describing the Fstat structure:
	aString* FWord=NEW aString("Fst");
	Flevels_[m].Append(FWord);
	FWord=NEW aString("Fis");
	Flevels_[m].Append(FWord);
	FWord=NEW aString("Fit");
	Flevels_[m].Append(FWord);

	aString* vcWord=NEW aString("var(p)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(i)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(g)");
	vcLevels_[m].Append(vcWord);
	vcWord=NEW aString("var(t)");
	vcLevels_[m].Append(vcWord);
	//make and initialize the jackknife & by-locus Fstat lists
	int Flevel,vclevel;
	for (Flevel=0;Flevel<3;++Flevel){Fstats_[m].Append(zero);}
	for (vclevel=0;vclevel<4;++vclevel){vc_[m].Append(zero);}
	//some intermediate variables
	aList<long double>* ALoc=NEW aList<long double>();
	aList<long double>* BLoc=NEW aList<long double>();
	aList<long double>* CLoc=NEW aList<long double>();
	aList<long double>* BCLoc=NEW aList<long double>();
	aList<long double>* ABCLoc=NEW aList<long double>();
	CheckForNewAlleles();
	UpdateVariableLoci();
	int j,a,loc,pop;
	for (j=0;j<CodominantMarkerData::loci();++j){
		//make and initialize the jackknife & by-locus Fstat lists for this locus 
		aList<long double>* locFstat = NEW aList<long double>();
		FstatsLoc_[m].Append(locFstat);
		aList<long double>* locJackFstat = NEW aList<long double>();
		jackFstatsLoc_[m].Append(locJackFstat);
		for (Flevel=0;Flevel<3;++Flevel){
			locFstat->Append(zero);
			locJackFstat->Append(zero);
			}
		//make and initialize the jackknife & by-allele Fstat lists for this locus 
		ObjList<aList<long double> >* locAlleleFstat = NEW ObjList<aList<long double> >();
		FstatsAll_[m].Append(locAlleleFstat);
		//initialize variables
		ALoc->Append(zero);
		BLoc->Append(zero);
		CLoc->Append(zero);
		BCLoc->Append(zero);
		ABCLoc->Append(zero);

		//make and initialize the by-allele F-stat list for each allele and append to this locus list
		StringList& locAlleleList = allelesPresent_[j];
		for (a=0;a<locAlleleList.len();++a){
			aList<long double>* alleleFstat =NEW aList<long double>();
			locAlleleFstat->Append(alleleFstat);
			for (Flevel=0;Flevel<3;++Flevel){alleleFstat->Append(zero);}
			}//end for a							

		//make and initialize the by-allele var comp list
		vcLoc_[m].Append(NEW aList<long double>());
		for (vclevel=0;vclevel<4;++vclevel){vcLoc_[m][j].Append(zero);}
		vcAll_[m].Append(NEW ObjList<aList<long double> >());
		for (a=0;a<locAlleleList.len();++a){
			vcAll_[m][j].Append(NEW aList<long double>());
			for (vclevel=0;vclevel<4;++vclevel){vcAll_[m][j][a].Append(zero);}
			}//end for a							
		}//end for j

	//the main body of the calculations	
//	ObjList<GeneticPopulation>* pops=popList_[group];
	long double r = (long double) (npop_), nSample,freq,hObs;
	long double Asum,Bsum,Csum,BCsum,ABCsum;
	long double Nsq,Nc,Nbar,pBar,ssq,Hbar,A,B,C;
	aString levelName;
	Asum=Bsum=Csum=BCsum=ABCsum=zero;
	for (j=0;j<variableLocusNumCodom_.len();++j){
		loc = variableLocusNumCodom_[j];//the actual locus
		StringList& locAlleleList = allelesPresent_[loc];
		for (a=0;a<locAlleleList.len();++a){
			Nsq=Nc=Nbar=pBar=ssq=Hbar=A=B=C=zero;
			aList<long double> q;//allele frequencies
			//Calculate for the whole population set:
			for (pop=0;pop<npop_;++pop){
				q.Append(GetFreq(pop,loc, a));
				nSample = (long double) (GetLocusSampleSize(pop,loc));//  /2.0
				//average sample size among pops:
				Nbar += nSample;
				//preliminary variable for term accounting for the variance in sample sizes:
				Nsq += nSample * nSample;
				// Average allelic frequency (=pBar)
				pBar += nSample * (long double)q[pop];
				//go get the observed heterozygosity for this allele and locus
				hObs = (long double)GetObservedHeterozygosity(pop,loc,a);
				Hbar += (nSample * hObs);
				}//end for pop

			if ((r > one)&&(Nbar>zero)) {//this weeds out zero denominators
				Nbar /= r;
				Nc = (r * Nbar - Nsq / (r * Nbar)) / (r - one);
				pBar /= (r * Nbar);
				Hbar /= (r * Nbar);

				//Now get the term accounting for variance among allelic frequencies
				for (pop=0;pop<npop_;++pop){
					nSample = (long double) (GetLocusSampleSize(pop,loc));
					freq=(long double)q[pop];
					ssq += (nSample * (freq - pBar) * (freq - pBar));
					}//end for pop
				ssq /= ((r - one) * Nbar);

				//Calculate final variables in steps, because the equations are long:
				if ((Nc != zero)&&(Nbar != one)){//this weeds out zero denominators
					A = (pBar * (one - pBar)) - (ssq * ((r - one) / r)) - (Hbar /four);
					A = (Nbar / Nc) * (ssq - (A / (Nbar - one)));
					}//end if Nc & Nbar

				if (Nbar != one){//this weeds out zero denominators
					B = Hbar * ((two * Nbar - one) / (four * Nbar));
					B = (pBar * (one - pBar)) - (ssq * ((r - one) / r)) - B;
					B *= (Nbar / (Nbar - one));
					}//end if Nbar!=1
				C = Hbar / two;
				}//end if r<>0, r<>1, Nbar<>0

			vcAll_[m][loc][a][0] = A;
			vcAll_[m][loc][a][1] = B;
			vcAll_[m][loc][a][2] = C;
			vcAll_[m][loc][a][3] = A + B + C;
			//A, B, &C remain zero if their components have any zero denominators
			ObjList<aList<long double> >& locAlleleFstat=FstatsAll_[m][loc];
			aList<long double>& alleleFstat =locAlleleFstat[a];
			if ((A + B + C) != zero) {
				levelName="Fst";
				Flevel = Flevels_[m].Loc(&levelName);
				alleleFstat.Replace(A / (A + B + C), Flevel);
				levelName="Fit";
				Flevel = Flevels_[m].Loc(&levelName);
				alleleFstat.Replace(one - (C / (A + B + C)),Flevel);
				}//end if A+B+C
			if ((B + C) != zero) {
				levelName="Fis";
				Flevel = Flevels_[m].Loc(&levelName);
				alleleFstat.Replace(one - (C / (B + C)), Flevel);
				}//end if (B+C)
			//save these in an array for a quicker calculation of the jackknife variance estimates
			ALoc->Replace(ALoc->Nth(loc)+A,loc);
			BLoc->Replace(BLoc->Nth(loc)+B,loc);
			CLoc->Replace(CLoc->Nth(loc)+C,loc);
			BCLoc->Replace(BCLoc->Nth(loc)+B+C,loc);
			ABCLoc->Replace(ABCLoc->Nth(loc)+A+B+C,loc);

			Asum += A;
			Bsum += B;
			Csum += C;
			BCsum += B + C;
			ABCsum += A + B + C;
			
			}//end for a
		vcLoc_[m][loc][0] = (*ALoc)[loc];
		vcLoc_[m][loc][1] = (*BLoc)[loc];
		vcLoc_[m][loc][2] = (*CLoc)[loc];
		vcLoc_[m][loc][3] = (*ABCLoc)[loc];
		aList<long double>& locFstat = FstatsLoc_[m][loc];
		long double temp = ABCLoc->Nth(loc);
		if (temp != zero) {
			levelName="Fst";
			Flevel = Flevels_[m].Loc(&levelName);
			locFstat.Replace(ALoc->Nth(loc) / temp, Flevel);
			levelName="Fit";
			Flevel = Flevels_[m].Loc(&levelName);
			locFstat.Replace(one - (CLoc->Nth(loc) / temp), Flevel);
			}//end if temp
		temp= BCLoc->Nth(loc);
		if (temp != zero){
			levelName="Fis";
			Flevel = Flevels_[m].Loc(&levelName);
			locFstat.Replace(one - (CLoc->Nth(loc) / temp), Flevel);
			}//end if temp
		}//end for j

	vc_[m][0] = Asum;
	vc_[m][1] = Bsum;
	vc_[m][2] = Csum;
	vc_[m][3] = ABCsum;
	if (ABCsum != zero){
		levelName="Fst";
		Flevel = Flevels_[m].Loc(&levelName);
		Fstats_[m].Replace((Asum / ABCsum), Flevel);
		levelName="Fit";
		Flevel = Flevels_[m].Loc(&levelName);
		Fstats_[m].Replace((one - (Csum / ABCsum)), Flevel);
		}//end if ABCsum
	if (BCsum != zero){
		levelName="Fis";
		Flevel = Flevels_[m].Loc(&levelName);
		Fstats_[m].Replace((one - (Csum / BCsum)), Flevel);
		}//end if BCSum

	//now do the jackknife calculations, over loci
	for (j=0;j<variableLocusNumCodom_.len();++j){
		loc = variableLocusNumCodom_[j];//the actual locus		
		aList<long double>& locJackFstat = jackFstatsLoc_[m][loc];
		long double temp = ABCLoc->Nth(loc);
		if ((ABCsum - temp) != zero) {
			levelName="Fst";
			Flevel = Flevels_[m].Loc(&levelName);
			locJackFstat.Replace(((Asum - ALoc->Nth(loc)) / (ABCsum - temp)), Flevel);
			levelName="Fit";
			Flevel = Flevels_[m].Loc(&levelName);
			locJackFstat.Replace(one - (Csum - CLoc->Nth(loc)) / (ABCsum - temp), Flevel);
			}//end if ABCsum
		temp = BCLoc->Nth(loc);
		if (BCsum - temp != zero) {
			levelName="Fis";
			Flevel = Flevels_[m].Loc(&levelName);
			locJackFstat.Replace(one - (Csum - CLoc->Nth(loc)) / (BCsum - temp), Flevel);
			}//end if BCSum
		}//end for j
	delete ALoc;ALoc=0;
	delete BLoc;BLoc=0;
	delete CLoc;CLoc=0;
	delete BCLoc;BCLoc=0;
	delete ABCLoc;ABCLoc=0;
	FstatsCurrent_=vcCurrent_=1;
	}//end  FstatisticAnalysisCodom::WeirTheta





#endif


