/******************************************************
*
*	Genome.h
*
*	This class implements a list of chromosomes.  
*	It handles cloning and sexual reproduction.
*
*	Base class:  Phenotype
*
*******************************************************/

#ifndef Genomeh
#define Genomeh = 1

#include <math.h>
#include <iostream.h>
#include "InlineTemplates.h"
#include "Randomness.h"
#include "RandomList.h"
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "Allele.h"
#include "Chromosome.h"
#include "CoreGenome.h"
#include "Phenotype.h"
#include "MemoryManager.h"

template <class T> class ChromosomeList;
//template <class T> class Genome;

template <class T> class ChromosomeList: public ObjList<Chromosome<T> >{
    public:/*holds copies of the same chromosome, to handle ploidy*/
	ChromosomeList(void): ObjList<Chromosome<T> >() {
			}
	ChromosomeList(ChromosomeList<T>& clist):
			ObjList<Chromosome<T> >(clist){
			}
	ChromosomeList(ChromosomeList<T>* clist):
			ObjList<Chromosome<T> >(clist){
			}
	virtual ~ChromosomeList(void){
			Clear();
			}


	virtual void Mutate(Ran3& ran, double mutationRate, double sigma){	
		for(int i=0;i<len();++i){	
			Chromosome<T>& chrT = Nth(i);
			chrT.Mutate(ran, mutationRate, sigma);
			}
		}
	virtual void Sort (void);/*a QuickSort algorithm*/
	virtual void Sort (long start, long finish);/*a QuickSort algorithm*/
	ChromosomeList<T>& operator=(ChromosomeList<T>& c){
		ObjList<Chromosome<T> >& cRef =	ObjList<Chromosome<T> >::operator=((ObjList<Chromosome<T> >) c);
		ChromosomeList<T>* cPtr =	(ChromosomeList<T>*) &cRef;
		return *cPtr;}
	};/*end class ChromosomeList*/



template <class T> class Genome: public CoreGenome {
//template <class T> class Genome: public Phenotype {
  protected:
  	ObjList<ChromosomeList<T> > genome_;
  public:
  	Genome(void): CoreGenome(), genome_(){
//  	Genome(void):Phenotype(), genome_(){
			}/*creates empty list*/
  	Genome(int numLociPerChrom, int num1NChroms, int ploidy, T alleleVal);
  			/*creates a 'null' homozygous individual at all loci*/
  	Genome(ObjList<ChromosomeList<T> >& c):  CoreGenome(), genome_(c) {
//  	Genome(ObjList<ChromosomeList<T> >& c):  Phenotype(), genome_(c) {
			}
//  	Genome(ViabilitySelectionFunc viabSelection):Phenotype(viabSelection),genome_(){
  	Genome(ViabilitySelectionFunc viabSelection):CoreGenome(viabSelection),genome_(){
			}/*creates empty list*/
  	Genome(int numLociPerChrom, int num1NChroms, int ploidy, T alleleVal, ViabilitySelectionFunc viabSelection);
  			/*creates a 'null' homozygous individual at all loci*/
  	Genome(ObjList<ChromosomeList<T> >& c,ViabilitySelectionFunc viabSelection):
  					CoreGenome(viabSelection), genome_(c) {
//  					Phenotype(viabSelection), genome_(c) {
			}
	Genome(Genome<T>&);/*copies the Genome during construction*/
	Genome(Genome<T>*);/*copies the Genome during construction*/
virtual ~Genome(void){/*destroys the list*/
				genome_.Clear();
			}//end destructor

virtual double 	additiveRelativePhenotype(void);
  	
//virtual int		AllelePresent (int chrom, int loc, T alleleVal);
virtual int		AllelePresent (int locus, aString& alleleVal);
virtual int		AllelePresent (int chrom, int loc, aString& alleleVal);
virtual int		AllelePresent (int chrom, int loc, Allele<T>& a);

virtual int		chroms1N(void){
				return genome_.len();}

//virtual int		CountAlleles (int chrom, int loc, T alleleVal);
virtual int		CountAlleles (int locus, aString& alleleVal);
virtual int		CountAlleles (int chrom, int loc, aString& alleleVal);
virtual int		CountAlleles (int chrom, int loc, Allele<T>& a);
				/*counts the absolute frequency of alleleVal */

void			Copy (Genome<T>&);/*copies to an existing genotype*/

void			Copy (Genome<T>*);/*copies to an existing genotype*/

virtual void	Detach(void);/*release but don't destroy chromosomes*/

virtual Phenotype* new_Phenotype(void){
					return NEW Genome<T>();}
	
virtual Phenotype* Duplicate(void){
				Genome<T>* g = NEW Genome<T>();
				g->Copy(*this);
				return g;}
	
virtual void	Fertilize (CoreGenome* momGamete, CoreGenome* dadGamete);

virtual void	Gamete (CoreGenome* theParentGenes, Ran3List& ran, double mutationRate, double sigma);

virtual aString* 	GenotypeString(char separator=' ');
					//returns all ploidy copies separated by parentheses (A)(A) at each locus
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(char* separator);
					//returns all ploidy copies separated by parentheses (A)(A) at each locus
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(int locus, char* separator);
					//finds nth overall locus and returns all ploidy copies separated by parentheses (A)(A)
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(int locus, int alleleNum, char* separator);
					//finds nth overall locus and returns the alleleNum_th copy

virtual aString* 	GenotypeString(int chrom, int locus, int alleleNum, char* separator);
					//finds nth locus on the chrom_th chromosome and returns the alleleNum_th copy

virtual Allele<T>&	GetAllele (int chrom, int locus, int alleleNum);
				/*returns the reference of the allele*/

virtual Allele<T>&	GetAllele (int locus, int alleleNum);
				/*returns the reference of the allele*/

virtual T		GetGene (int chrom, int locus, int alleleNum);
				/*returns the value of the allele*/

virtual T		GetGene (int locus, int alleleNum);
				/*returns the value of the allele*/

virtual int		IsHeterozygote (int chrom, int locus);
				/*returns false=0 if haploid*/

virtual int		IsHeterozygote (int locus);
				/*returns false=0 if haploid*/

virtual int		loci(void);

virtual int		loci(int chrom);
  	
virtual void	Mutate(Ran3& ran, double mutationRate, double sigma);


virtual void	Mutate(int chrom, int locus, int alleleNum, T newAlleleVal){
  				set(chrom, locus,alleleNum,newAlleleVal);}
  				/*changes the value of an allele*/
  			
virtual double	phenotype(int i=0){
					return trait(i);}

virtual int 	ploidy (int locus=0);
  		
virtual int		PloidyAtChrom (int chrom);
				/*returns the number of alleles*/

virtual void	Print(ostream& outFile, int alleleLen=4);

//virtual T		RandomAllele(int chrom, int locus, Ran3& ran);
//  				/*randomly picks an allele and returns its value*/
virtual Allele<T>&		
			RandomAllele(int chrom, int locus, Ran3& ran);
  				/*randomly picks an allele and returns its reference*/

virtual void	set(int chrom, int locus, int alleleNum, T newAlleleVal);
  				/*changes the value of an allele
  				use when doing reproduction, etc.
  				same as mutate, but term is not so process-laden
  				USES ZERO-BASED INDICES FOR locus AND alleleNum*/
  		
virtual void	SetGenotypeString(aString& geneStr, int alleleLen=4);
	
virtual void	SetPhenotype(void);
				/*default is additive phenotype*/
				
virtual void	SetPhenotype(Ran3& ran, double heritability = 1.0);
				/*default is additive phenotype*/
				
virtual void 	SetSelectionParams(aList<double>& selParams);
					//copies all allele values to selParams

virtual void	SetSexualZygote(CoreGenome* momGenes, CoreGenome* dadGenes, Ran3List& ran, double mutationRate, double sigma);
				/*uses mom's and dads genes to seed the genotype*/

Genome<T>& operator=(Genome<T>& g){
		selectionFunc = g.selectionFunc;
  		trait_.Clear();
		for(int i=0;i<g.trait_.len();++i){
			trait_.Append(g.trait_[i]);}
		genome_.Clear();
		for(int i=0;i<g.genome_.len();++i){
			ChromosomeList<T>& x = g.genome_[i];
			ChromosomeList<T>* y = NEW ChromosomeList<T>(x);
			genome_.Append(y);}
		return *this;}
	

friend int operator== (Genome<T>& g1, Genome<T>& g2);
		
friend int operator!= (Genome<T>& g1, Genome<T>& g2);
  	
#ifdef MemoryLeakDebug
	static void PrintIDList(ostream& outFile=cout);
	static void PrintIDListLen(ostream& outFile=cout);
#endif
  	};//end class Genome
  	
  	
template <class T> int operator== (Genome<T>& g1, Genome<T>& g2);
		
template <class T> int operator!= (Genome<T>& g1, Genome<T>& g2);


#endif
