//unit MultiformatMarkerData.cpp

#ifndef MultiformatMarkerDatacpp
#define MultiformatMarkerDatacpp = 1

#include "MultiformatMarkerData.h"
#include "FstatisticAnalysisCodom.h"
#include "FstatisticAnalysisHap.h"
#include "FstatisticAnalysisDom2.h"
#include "FstatisticAnalysisCodomSexLink.h"
#include "FstatisticAnalysisDomSexLink.h"
#include "FstatisticAnalysisCytoFemale.h"
#include "FstatisticAnalysisCytoMale.h"
#include "Element.cpp"
#include "aList.cpp"
#include "GeneContainers.cpp"
//#include "TextDataList.cpp"
#include "StatList.cpp"
#include "aVector.cpp"
#include "aMatrix.cpp"
//#include "MatrixList.cpp"
//#include "PartitionedMatrix.cpp"



MultiformatMarkerData::MultiformatMarkerData(MultiformatMarkerData& parentData){	
	Initialize(parentData);
	}//end ctor MultiformatMarkerData(MultiformatMarkerData&)

MultiformatMarkerData::MultiformatMarkerData(HaploidMarkerData& parentData){	
	Initialize();
	HaploidMarkerData::Initialize(parentData);
	traitTypesIncluded_[hap]=true;
	traitType_.Append(0);//ID
	int loc;
	for(loc=0;loc<nlociHap_;++loc){
		traitType_.Append(hap);}
	}//end ctor MultiformatMarkerData(MultiformatMarkerData&)

MultiformatMarkerData::MultiformatMarkerData(CodominantMarkerData& parentData){	
	Initialize();
	CodominantMarkerData::Initialize(parentData);
	traitTypesIncluded_[codom]=true;
	traitType_.Append(0);//ID
	int loc;
	for(loc=0;loc<nlociCodom_;++loc){
		traitType_.Append(codom);}
	ParseGenotypicData();
	}//end ctor MultiformatMarkerData(MultiformatMarkerData&)

MultiformatMarkerData::MultiformatMarkerData(DominantMarkerData2& parentData){	
	Initialize();
	DominantMarkerData2::Initialize(parentData);
	traitTypesIncluded_[dom]=true;
	traitType_.Append(0);//ID
	int loc;
	for(loc=0;loc<nlociDom_;++loc){
		traitType_.Append(dom);}
	}//end ctor MultiformatMarkerData(MultiformatMarkerData&)

MultiformatMarkerData::MultiformatMarkerData(SexLinkedCodomMarkerData& parentData){	
	Initialize();
	SexLinkedCodomMarkerData::Initialize(parentData);
	traitTypesIncluded_[SLcodom]=true;
	traitType_.Append(0);//ID
	int loc;
	for(loc=0;loc<nlociSLCodom_+1;++loc){//initialize
		traitType_.Append(SLcodom);}
	traitType_[parentData.sexTrait_]=1;//sex
	ParseGenotypicData();
	}//end ctor MultiformatMarkerData(MultiformatMarkerData&)

MultiformatMarkerData::MultiformatMarkerData(SexLinkedDomMarkerData2& parentData){	
	Initialize();
	SexLinkedDomMarkerData2::Initialize(parentData);
	traitTypesIncluded_[SLdom]=true;
	traitType_.Append(0);//ID
	int loc;
	for(loc=0;loc<nlociSLDom_+1;++loc){//initialize
		traitType_.Append(SLdom);}
	traitType_[parentData.sexTrait_]=1;//sex
	}//end ctor MultiformatMarkerData(MultiformatMarkerData&)

MultiformatMarkerData::MultiformatMarkerData(CytoplasmicFemaleMarkerData& parentData){	
	Initialize();
	int loc;
	CytoplasmicFemaleMarkerData::Initialize(parentData);
	if(parentData.traits(CYfemale)>0){
		traitTypesIncluded_[CYfemale]=true;}
	if(parentData.traits(CYfemaleOnly)>0){
		traitTypesIncluded_[CYfemaleOnly]=true;}
	traitType_.Append(ID);//ID
	for(loc=0;loc<nlociCYfemale_+1;++loc){//initialize
		traitType_.Append(CYfemale);}
	for(loc=0;loc<allPresentInFemalesOnly_.len();++loc){//re-initialize
		traitType_[allPresentInFemalesOnly_[loc]+1] = CYfemaleOnly;}
	traitType_[parentData.sexTrait_]=sex;//sex
	DetermineFemaleOnlyTraits();
	}//end ctor MultiformatMarkerData(MultiformatMarkerData&)

MultiformatMarkerData::MultiformatMarkerData(CytoplasmicMaleMarkerData& parentData){	
	Initialize();
	int loc;
	CytoplasmicMaleMarkerData::Initialize(parentData);
	if(parentData.traits(CYmale)>0){
		traitTypesIncluded_[CYmale]=true;}
	if(parentData.traits(CYmaleOnly)>0){
		traitTypesIncluded_[CYmaleOnly]=true;}
	traitType_.Append(ID);//ID
	for(loc=0;loc<nlociCYmale_+1;++loc){//initialize
		traitType_.Append(CYmale);}
	for(loc=0;loc<allPresentInMalesOnly_.len();++loc){//re-initialize
		traitType_[allPresentInMalesOnly_[loc]+1] = CYmaleOnly;}
	traitType_[parentData.sexTrait_]=sex;//sex
	DetermineMaleOnlyTraits();
	}//end ctor MultiformatMarkerData(MultiformatMarkerData&)


MultiformatMarkerData::MultiformatMarkerData(MultiformatMarkerData& parentData, aList<int>& popSubsetNums):
			EvolutionaryGeneticsData(parentData,popSubsetNums), 
			traitTypesIncluded_(parentData.traitTypesIncluded_),traitType_(parentData.traitType_){
	//makes a copy of the original data using a subset of populations
	//useful for separate analyses of data subsets
	ResetGeneticFlags();
	Initialize(parentData,popSubsetNums); 	
//	traitTypeCurrent_=parentData.traitTypeCurrent_;//restore this
//	for(int p=0;p<popSubsetNums.len();++p){
//		int pp = popSubsetNums[p];//population num in parentData
//		if(codominantDataIncluded()){
//			rawCodomGtypes_.Append(parentData.rawCodomGtypes_[pp].Duplicate());}
//		if(codominantSexLinkedDataIncluded()){
//			rawSLCodomGtypes_.Append(parentData.rawSLCodomGtypes_[pp].Duplicate());}
//		}//end for p
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();
	}//end ctor MultiformatMarkerData(MultiformatMarkerData&, aList<int>&)


void  MultiformatMarkerData::Initialize(void){
	MultiformatMarkerData::Clear();
	int t;
	for(t=0;t<numTraitTypes_;++t){
		traitTypesIncluded_.Append(false);}
	}//Initialize()
	
	
void  MultiformatMarkerData::Initialize(aString& s){
	MultiformatMarkerData::Clear();
	EvolutionaryGeneticsData::Initialize(s); 
	int t;
	for(t=0;t<numTraitTypes_;++t){
		traitTypesIncluded_.Append(false);}
	}//Initialize(aString&)
	
	
void  MultiformatMarkerData::Initialize(MultiformatMarkerData& cmd){
	HaploidMarkerData::Initialize(cmd); 
	CodominantMarkerData::Initialize(cmd); 
	DominantMarkerData2::Initialize(cmd); 
	SexLinkedCodomMarkerData::Initialize(cmd); 
	SexLinkedDomMarkerData2::Initialize(cmd); 
	CytoplasmicFemaleMarkerData::Initialize(cmd); 
	CytoplasmicMaleMarkerData::Initialize(cmd); 
	traitType_=cmd.traitType_;
	traitTypesIncluded_=cmd.traitTypesIncluded_;
	traitTypeCurrent_=cmd.traitTypeCurrent_;
	}//Initialize(MultiformatMarkerData& )
	
	
	
void  MultiformatMarkerData::Initialize(MultiformatMarkerData& parentData, aList<int>& popSubsetNums){
	//a lot of redundancy here that could be cleared up
	EvolutionaryGeneticsData::Initialize(parentData,popSubsetNums);
	HaploidMarkerData::Initialize(parentData,popSubsetNums); 
	CodominantMarkerData::Initialize(parentData,popSubsetNums); 
	DominantMarkerData2::Initialize(parentData,popSubsetNums); 
	SexLinkedCodomMarkerData::Initialize(parentData,popSubsetNums); 
	SexLinkedDomMarkerData2::Initialize(parentData,popSubsetNums); 
	CytoplasmicFemaleMarkerData::Initialize(parentData,popSubsetNums); 
	CytoplasmicMaleMarkerData::Initialize(parentData,popSubsetNums); 
	traitTypesIncluded_=parentData.traitTypesIncluded_;//this could introduce a bug, if not all pops scored in all trait types
	traitType_=parentData.traitType_;
	traitTypeCurrent_=parentData.traitTypeCurrent_;
	}//end Initialize(DominantMarkerData2&, aList<int>&)


	
	
	
MultiformatMarkerData&  MultiformatMarkerData::operator=(MultiformatMarkerData& cmd){
	Initialize(cmd); 
	return *this;
	}//operator=(MultiformatMarkerData&)


/*
void MultiformatMarkerData::Calculate2LocusDisequilibrium (void){
	if(DCurrent_) return;
	if(ploidy_ != 2){
		aString err("ERROR: MultiformatMarkerData::Calculate2LocusDisequilibrium(): can\'t calculate D when ploidy=");
		err+ploidy_;
		aError(err);
		return;}
	int doingHillsMethod = 0;//false
	//first make up an empty data array for locusD_ and D_
	long double initValue = 0.0;
//	aString blank; blank.AppendCharToLen(' ',alleleLen_);

	diseqD_.Clear();
	locusD_.Clear();//locusD_ contains locusByLocusDiseqInPop as subLists
	for (int pop=0;pop<npop_;++pop){
		//locusByLocusDiseqInPop contains locusDiseqLists as sublists
		ObjList<aList<long double> >* locusByLocusDiseqInPop = NEW ObjList<aList<long double> >();
		locusD_.Append(locusByLocusDiseqInPop);
		for (int loc1=0;loc1<nlociCodom_ - 1;++loc1){
			//locusDiseqList contains D values
			aList<long double>* locusDiseqList = NEW aList<long double>();
			locusByLocusDiseqInPop->Append(locusDiseqList);
			for (int loc2=loc1 + 1;loc2<nlociCodom_;++loc2){
				locusDiseqList->Append(initValue);}
			}//end for loc1
		diseqD_.Append(initValue);
		}//end for pop
	StatList<long double> Dlist;
	for (int pop=0;pop<npop_;++pop){
		for (int loc1=0;loc1<nlociCodom_ - 1;++loc1){
			if (IsPolymorphic(pop,loc1)){ 
				StringList& alleleNamesList1=allelesPresent_[loc1];
				for (int loc2 = (loc1 + 1); loc2<nlociCodom_;++loc2){
					if (IsPolymorphic(pop,loc2)){ 
						StringList& alleleNamesList2=allelesPresent_[loc2];
						int kComb = 0;  long double diseq=0.0;
						for (int allele1=0;allele1<alleleNamesList1.len();++allele1){
							//get freq of allele1 (at loc1) in pop
							long double freq1 = GetFreq(pop, loc1, allele1);
							if ((freq1>(long double)0.0)&&(freq1<(long double)1.0)){
								aString& alleleName1 = alleleNamesList1[allele1];
								for (int allele2=0;allele2<alleleNamesList2.len();++allele2){
									int nSample = 0;//set sample size to zero, to count for blanks
									//get freq of allele2 (at loc2) in pop
									long double freq2 = GetFreq(pop, loc2, allele2);
									if ((freq2>(long double)0.0)&&(freq2<(long double)1.0)){
									aString& alleleName2 = alleleNamesList2[allele2];
									//get genotype strings for each individual and  read joint frequencies
									long double gameteValue = 0.0;
									int bigN[3][3], bigX[2][2];
									for (int index1=0;index1<3;++index1){
										for (int index2=0;index2<3;++index2){
											bigN[index1][index2] = 0;}}
									for (int index1=0;index1<2;++index1){
										for (int index2=0;index2<2;++index2){
											bigX[index1][index2] = 0;}}
									for (int i=0;i<individuals(pop);++i){//each individual
										TextDataList& spot1 = Genotype(pop,i,loc1);
										//' make sure it doesn't have any blanks
										int blankAt1 = 0;//false
										if (spot1[0].missing() || spot1[1].missing()) {blankAt1 = 1;}//true
										if (!(blankAt1)){
											TextDataList& spot2 = Genotype(pop,i,loc2);
											//make sure it doesn' t have any blanks
											int blankAt2 = 0;//false
											if (spot2[0].missing() || spot2[1].missing()) {blankAt2 = 1;}//true
											if (!(blankAt2)){
												nSample++;
												//use this indiv in calculation
												//count instances of target alleles at each locus in this indiv
												long double count1 = 0;
												int index1 = 3;
												for(int chrom=0;chrom<2;++chrom){
													if (spot1[chrom] == alleleName1){
														count1 += 0.5;
														index1--;
													}}
												long double count2 = 0;
												int index2 = 3;
												for(int chrom=0;chrom<2;++chrom){
													if (spot2[chrom] == alleleName2){
														count2 += 0.5;
														index2--;
													}}
												//get a gametic value, which is the proportion
												//of gametes this individual will produce that
												//share target alleles at both loci.
												//Keep a sum of gametic values for the population
												gameteValue += count1 * count2;
												//this counts genotype freqs
												bigN[index1][index2]++;
												}//end if ! blankAt2
											}//end if ! blankAt1
										}//end for i
									//calculate frequency of joint genotype=gametic value/nsample}
									if (nSample == 0){
											gameteValue = (long double)-100;}
										else{
											gameteValue /= (long double)nSample;}
									//calculate Hill's (1974) intermediate variables & iterative method}
									long double HillsDiseq=0.0;
									if (doingHillsMethod){
										bigX[1][1] = 2 * bigN[1][1] + bigN[1][2] + bigN[2][1];
										bigX[2][1] = 2 * bigN[3][1] + bigN[2][1] + bigN[3][2];
										bigX[1][2] = 2 * bigN[1][3] + bigN[1][2] + bigN[2][3];
										bigX[2][2] = 2 * bigN[3][3] + bigN[2][3] + bigN[3][2];
										long double initialF11hat = (long double)((2 * bigX[1][1] + bigN[2][2]) / (2 * nSample)) - freq1 * freq2;
										long double diff = 10.0; long double f11hat=0.0;
										long iteration = 0;
										while (diff > 0.00001){
											//calculate in steps since it's long
											iteration++;
											f11hat = (long double)bigN[2][2] * initialF11hat * ((long double)1.0 - freq1 - freq2 + initialF11hat);
											f11hat /= (initialF11hat * ((long double)1.0 - freq1 - freq2 + initialF11hat) + (freq1 - initialF11hat) * (freq2 - initialF11hat));
											f11hat = ((long double)bigX[1][1] + f11hat) / (long double)(2 * nSample);
											diff = ABS(f11hat - initialF11hat);
											initialF11hat = f11hat;
											if (iteration > 40){
												initialF11hat *= 1.01;
												iteration = 0;}
											}//end while
										HillsDiseq = (long double)nSample * (f11hat - freq1 * freq2) / (long double)(nSample - 1);
										}//end if doingHillsMethod
									//calculate D
									diseq = gameteValue - freq1 * freq2;
									//was gameteValue -2* freq1 * freq2
									kComb++;
									ObjList<aList<long double> >& locusByLocusDiseqInPop = locusD_[pop];
									aList<long double>& locusDiseqList = locusByLocusDiseqInPop[loc1];
									if (doingHillsMethod){
											diseq = HillsDiseq + locusDiseqList[loc2 - loc1];}
										else{
											diseq += locusDiseqList[loc2 - loc1];}
									locusDiseqList.Replace(diseq, loc2 - loc1);
									}//end if freq2>0 &&freq2<1
								}//end for allele2
							}//end if freq1>0 && freq1<1
						}//end for allele1
					ObjList<aList<long double> >& locusByLocusDiseqInPop = locusD_[pop];
					aList<long double>& locusDiseqList = locusByLocusDiseqInPop[loc1];
					diseq += locusDiseqList.Nth(loc2 - loc1) / (long double)kComb;
					locusDiseqList.Replace(diseq, loc2 - loc1);
					Dlist.Append(diseq);
					}//end if IsPolymorphic(poploc2)
				}//end for loc2
			}//end if IsPolymorphic(pop,loc1)
		}//end for loc1
	if (diseqD_.len() > 0){diseqD_.Append(Dlist.mean());} else{diseqD_.Append(0.0);}
	Dlist.Clear();
	}//end for pop
	DCurrent_=true;
	}//end MultiformatMarkerData::Calculate2LocusDisequilibrium
*/




void MultiformatMarkerData::AddTraits(int traitType, aList<int>& traitNumbers){//handle sex and ID separately
	if(traitNumbers.len()==0 || traitType <=1) return;
	int trloc, trnum, tr;
	for(tr=0;tr<traitNumbers.len();++tr){
		trnum = traitNumbers[tr];
		traitType_[trnum]=traitType;
		switch(traitType){
			case hap:
				if(! allHapLoci_.Includes(trnum)){
					allelesCurrentHap_=false;
					traitTypesIncluded_[traitType]=true;
					allHapLoci_.Append(trnum);
					if(skippedTraitNum_.Includes(trnum)){
							skippedLocusNumHap_.Append(trnum);}
						else{
							locusNumHap_.Append(trnum);}
					nlociHap_ = locusNumHap_.len();}					
				break;
			case codom: 
				if(! allCodomLoci_.Includes(trnum)){
					allelesCurrent_=false;
					traitTypesIncluded_[traitType]=true;
					allCodomLoci_.Append(trnum);
					if(skippedTraitNum_.Includes(trnum)){
							skippedLocusNumCodom_.Append(trnum);}
						else{
							locusNumCodom_.Append(trnum);}
					nlociCodom_ = locusNumCodom_.len();}					
				break;
			case dom: 
				if(! allDomLoci_.Includes(trnum)){
					allelesCurrentDom_=false;
					traitTypesIncluded_[traitType]=true;
					allDomLoci_.Append(trnum);
					if(skippedTraitNum_.Includes(trnum)){
							skippedLocusNumDom_.Append(trnum);}
						else{
							locusNumDom_.Append(trnum);}
					nlociDom_ = locusNumDom_.len();}					
				break;
			case SLcodom: 
				if(! allSLCodomLoci_.Includes(trnum)){
					allelesCurrentSLCodom_=false;
					traitTypesIncluded_[traitType]=true;
					allSLCodomLoci_.Append(trnum);
					if(skippedTraitNum_.Includes(trnum)){
							skippedLocusNumSLCodom_.Append(trnum);}
						else{
							locusNumSLCodom_.Append(trnum);}
					nlociSLCodom_ = locusNumSLCodom_.len();}					
				break;
			case SLdom: 
				if(! allSLDomLoci_.Includes(trnum)){
					allelesCurrentSLDom_=false;
					traitTypesIncluded_[traitType]=true;
					allSLDomLoci_.Append(trnum);
					if(skippedTraitNum_.Includes(trnum)){
							skippedLocusNumSLDom_.Append(trnum);}
						else{
							locusNumSLDom_.Append(trnum);}
					nlociSLDom_ = locusNumSLDom_.len();}					
				break;
			case CYfemale: 
				traitTypesIncluded_[traitType]=true;
				traitType_[trnum]=CYfemale;
				if(! allCYfemaleLoci_.Includes(trnum)){
					allelesCurrentCYfemale_=false;
					allCYfemaleLoci_.Append(trnum);
					if(skippedTraitNum_.Includes(trnum)){
							skippedLocusNumCYfemale_.Append(trnum);}
						else{
							locusNumCYfemale_.Append(trnum);}
					nlociCYfemale_ = locusNumCYfemale_.len();}					
				if(allPresentInFemalesOnly_.Includes(trnum)){
					trloc = allPresentInFemalesOnly_.Loc(trnum);
					allPresentInFemalesOnly_.Delete(trloc);
					}
				if(presentInFemalesOnly_.Includes(trnum)){
					trloc = presentInFemalesOnly_.Loc(trnum);
					presentInFemalesOnly_.Delete(trloc);
					}
				if(! CytoplasmicFemaleMarkerData::allPresentInBothSexes_.Includes(trnum)){
					CytoplasmicFemaleMarkerData::allPresentInBothSexes_.Append(trnum);}
				if(! CytoplasmicFemaleMarkerData::presentInBothSexes_.Includes(trnum)){
					CytoplasmicFemaleMarkerData::presentInBothSexes_.Append(trnum);}
				break;
			case CYfemaleOnly: 
				traitTypesIncluded_[CYfemaleOnly]=true;
				traitType_[trnum]=CYfemaleOnly;
				if(! allCYfemaleLoci_.Includes(trnum)){
					allelesCurrentCYfemale_=false;
					allCYfemaleLoci_.Append(trnum);
					}
				if(! allPresentInFemalesOnly_.Includes(trnum)){
					allPresentInFemalesOnly_.Append(trnum);}
				if(skippedTraitNum_.Includes(trnum)){
						skippedLocusNumCYfemale_.Append(trnum);}
					else{
						locusNumCYfemale_.Append(trnum);
						if(! presentInFemalesOnly_.Includes(trnum)){
							presentInFemalesOnly_.Append(trnum);}
						}
				nlociCYfemale_ = locusNumCYfemale_.len();
				break;
			case CYmale: 
				traitTypesIncluded_[traitType]=true;
				traitType_[trnum]=CYmale;
				if(! allCYmaleLoci_.Includes(trnum)){
					allelesCurrentCYmale_=false;
					allCYmaleLoci_.Append(trnum);
					if(skippedTraitNum_.Includes(trnum)){
							skippedLocusNumCYmale_.Append(trnum);}
						else{
							locusNumCYmale_.Append(trnum);}
					nlociCYmale_ = locusNumCYmale_.len();}					
				if(allPresentInMalesOnly_.Includes(trnum)){
					trloc = allPresentInMalesOnly_.Loc(trnum);
					allPresentInMalesOnly_.Delete(trloc);
					}
				if(presentInMalesOnly_.Includes(trnum)){
					trloc = presentInMalesOnly_.Loc(trnum);
					presentInMalesOnly_.Delete(trloc);
					}
				if(! CytoplasmicMaleMarkerData::allPresentInBothSexes_.Includes(trnum)){
					CytoplasmicMaleMarkerData::allPresentInBothSexes_.Append(trnum);}
				if(! CytoplasmicMaleMarkerData::presentInBothSexes_.Includes(trnum)){
					CytoplasmicMaleMarkerData::presentInBothSexes_.Append(trnum);}
				break;
			case CYmaleOnly: 
				traitTypesIncluded_[CYmaleOnly]=true;
				traitType_[trnum]=CYmaleOnly;
				if(! allCYmaleLoci_.Includes(trnum)){
					allelesCurrentCYmale_=false;
					allCYmaleLoci_.Append(trnum);
					}
				if(! allPresentInMalesOnly_.Includes(trnum)){
					allPresentInMalesOnly_.Append(trnum);}
				if(skippedTraitNum_.Includes(trnum)){
						skippedLocusNumCYmale_.Append(trnum);}
					else{//only add to active lists if not skipped
						locusNumCYmale_.Append(trnum);
						if(! presentInMalesOnly_.Includes(trnum)){
							presentInMalesOnly_.Append(trnum);}
						}
				nlociCYmale_ = locusNumCYmale_.len();			
				break;
			default: break;
			}//switch
		}//tr
	locusNumHap_.Sort();
	locusNumCodom_.Sort();
	locusNumDom_.Sort();
	locusNumSLCodom_.Sort();
	locusNumSLDom_.Sort();
	locusNumCYfemale_.Sort();
	locusNumCYmale_.Sort();
	allPresentInFemalesOnly_.Sort();
	presentInFemalesOnly_.Sort();
	allPresentInMalesOnly_.Sort();
	presentInMalesOnly_.Sort();
	}//MultiformatMarkerData::AddTraits(int , aList<int>& )



int MultiformatMarkerData::alleleID(int loc, TextDatum& alleleVal){
	int locAdj=loc, nloc = HaploidMarkerData::loci();
	if(locAdj<nloc){
		return HaploidMarkerData::alleleID(locAdj, alleleVal);}
		else{
			locAdj -= nloc;}
	nloc = CodominantMarkerData::loci();
	if(locAdj<nloc){
		return CodominantMarkerData::alleleID(locAdj, alleleVal);}
		else{
			locAdj -= nloc;}
		nloc = DominantMarkerData2::loci();
	if(locAdj<nloc){
		return DominantMarkerData2::alleleID(locAdj, alleleVal);}
		else{
			locAdj -= nloc;}
		nloc = SexLinkedCodomMarkerData::loci();
	if(locAdj<nloc){
		return SexLinkedCodomMarkerData::alleleID(locAdj, alleleVal);}
		else{
			locAdj -= nloc;}
		nloc = SexLinkedDomMarkerData2::loci();
	if(locAdj<nloc){
		return SexLinkedDomMarkerData2::alleleID(locAdj, alleleVal);}
		else{
			locAdj -= nloc;}
		nloc = CytoplasmicFemaleMarkerData::loci();
	if(locAdj<nloc){
		return CytoplasmicFemaleMarkerData::alleleID(locAdj, alleleVal);}
		else{
			locAdj -= nloc;}
		nloc = CytoplasmicMaleMarkerData::loci();
	if(locAdj<nloc){
		return CytoplasmicMaleMarkerData::alleleID(locAdj, alleleVal);}
	aString msg("Error:  cannot return the allele ID # for locus #");
	msg.cat(loc+1); msg.cat(" when there are only ");
	msg.cat(loci()+1); msg.cat(" loci in the data set");
	aError(msg);
	return 0;//default
	}//MultiformatMarkerData::alleles(int, TextDatum&)



int MultiformatMarkerData::alleles(int loc){
	int locAdj=loc, nloc = HaploidMarkerData::loci();
	if(locAdj<nloc){
		return HaploidMarkerData::alleles(locAdj);}
		else{
			locAdj -= nloc;}
	nloc = CodominantMarkerData::loci();
	if(locAdj<nloc){
		return CodominantMarkerData::alleles(locAdj);}
		else{
			locAdj -= nloc;}
		nloc = DominantMarkerData2::loci();
	if(locAdj<nloc){
		return DominantMarkerData2::alleles(locAdj);}
		else{
			locAdj -= nloc;}
		nloc = SexLinkedCodomMarkerData::loci();
	if(locAdj<nloc){
		return SexLinkedCodomMarkerData::alleles(locAdj);}
		else{
			locAdj -= nloc;}
		nloc = SexLinkedDomMarkerData2::loci();
	if(locAdj<nloc){
		return SexLinkedDomMarkerData2::alleles(locAdj);}
		else{
			locAdj -= nloc;}
		nloc = CytoplasmicFemaleMarkerData::loci();
	if(locAdj<nloc){
		return CytoplasmicFemaleMarkerData::alleles(locAdj);}
		else{
			locAdj -= nloc;}
		nloc = CytoplasmicMaleMarkerData::loci();
	if(locAdj<nloc){
		return CytoplasmicMaleMarkerData::alleles(locAdj);}
	aString msg("Error:  cannot return the number of alleles for locus #");
	msg.cat(loc+1); msg.cat(" when there are only ");
	msg.cat(loci()+1); msg.cat(" loci in the data set");
	aError(msg);
	return 0;//default
	}//MultiformatMarkerData::alleles(int )





void MultiformatMarkerData::CalculateAlleleFreqs (void){
	if(haploidDataIncluded()){
		HaploidMarkerData::CalculateHaplotypeFreqs();}
	if(codominantDataIncluded()){
		CodominantMarkerData::CalculateAlleleFreqs();}
	if(codominantSexLinkedDataIncluded()){
		SexLinkedCodomMarkerData::CalculateAlleleFreqs();}
	if(dominantDataIncluded()){
		DominantMarkerData2::CalculatePhenotypeFreqs();}
	if(dominantSexLinkedDataIncluded()){
		SexLinkedDomMarkerData2::CalculatePhenotypeFreqs();}
	if(cytoplasmicFemaleMarkersIncluded()){
		CytoplasmicFemaleMarkerData::CalculateGenotypeFreqs();}
	if(cytoplasmicMaleMarkersIncluded()){
		CytoplasmicMaleMarkerData::CalculateGenotypeFreqs();}
	}//end MultiformatMarkerData::CalculateAlleleFreqs



ObjList<ObjList<aList<long double> > >* MultiformatMarkerData::CalculateMarkerFreqsByPopSet(aList<int>& popSet, 
													ObjList<aList<long double> >& sampleSizeListbyTypeAndLoc, bool splitCytotypeBySex){
 	//returns list of trait types, each with list of loci, each with list of allele freqs, using total allele set
 	//includes trait types 0 & 1, with no loci
 	CalculateAlleleFreqs();
 	sampleSizeListbyTypeAndLoc.Clear();
 	int trtype;
 	aList<long double> hapNlist,codomNlist, domNlist, SLcodomNlist,SLdomNlist,cytoFNlist,cytoFONlist,cytoMNList,cytoMONList;
 	ObjList<ObjList<aList<long double> > >* freqSetPtr = NEW ObjList<ObjList<aList<long double> > >();//[trtype][pop][loc][allele]
 	ObjList<ObjList<aList<long double> > >& freqSet = *freqSetPtr;//notational convenience
	bool found, CYmaleDone=false, CYfemaleDone=false;
 	for(trtype=0;trtype<numTraitTypes_;++trtype){
		found=false;
		sampleSizeListbyTypeAndLoc.Append(NEW aList<long double>());
 		switch(trtype){
		  case hap://haploid loci
			if(haploidDataIncluded()){
				found=true;
				freqSet.Append(HaploidMarkerData::CalculateHaplotypeFreqsByPopSet(popSet,hapNlist));
				sampleSizeListbyTypeAndLoc[trtype] = hapNlist;}
			break;
		  case codom://codominant loci
			if(codominantDataIncluded()){
				found=true;
				freqSet.Append(CodominantMarkerData::CalculateAlleleFreqsByPopSet(popSet,codomNlist));
				sampleSizeListbyTypeAndLoc[trtype] = codomNlist;}
			break;
		  case dom://dominant loci
			if(dominantDataIncluded()){
				found=true;
				freqSet.Append(DominantMarkerData2::CalculatePhenotypeFreqsByPopSet(popSet,domNlist));
				sampleSizeListbyTypeAndLoc[trtype] = domNlist;}
			break;
		  case SLcodom://codominant sex-linked loci
			if(codominantSexLinkedDataIncluded()){
				found=true;
				freqSet.Append(SexLinkedCodomMarkerData::CalculateAlleleFreqsByPopSet(popSet,SLcodomNlist));
				sampleSizeListbyTypeAndLoc[trtype] = SLcodomNlist;}
			break;
		  case SLdom://dominant sex-linked loci
			if(dominantSexLinkedDataIncluded()){
				found=true;
				freqSet.Append(SexLinkedDomMarkerData2::CalculatePhenotypeFreqsByPopSet(popSet,SLdomNlist));
				sampleSizeListbyTypeAndLoc[trtype] = SLdomNlist;}
			break;
		  case CYfemale://cytoplasmic female loci
			if(cytoplasmicFemaleMarkersIncluded()){
				found=true;
				freqSet.Append(CytoplasmicFemaleMarkerData::CalculateGenotypeFreqsByPopSet(popSet,cytoFNlist,CytoplasmicFemaleMarkerData::presentBothSexes));
				sampleSizeListbyTypeAndLoc[trtype] = cytoFNlist;}
			break;
		  case CYfemaleOnly://cytoplasmic female loci
			if(cytoplasmicFemaleMarkersIncluded()){
				found=true;
				freqSet.Append(CytoplasmicFemaleMarkerData::CalculateGenotypeFreqsByPopSet(popSet,cytoFONlist,presentFonly));
				sampleSizeListbyTypeAndLoc[trtype] = cytoFNlist;}
			break;
		  case CYmale://cytoplasmic male loci
			if(cytoplasmicMaleMarkersIncluded()){
				found=true;
				freqSet.Append(CytoplasmicMaleMarkerData::CalculateGenotypeFreqsByPopSet(popSet,cytoMNList,CytoplasmicMaleMarkerData::presentBothSexes));
				sampleSizeListbyTypeAndLoc[trtype] = cytoMNList;}
			break;
		  case CYmaleOnly://cytoplasmic male loci
			if(cytoplasmicMaleMarkersIncluded()){
				found=true;
				freqSet.Append(CytoplasmicMaleMarkerData::CalculateGenotypeFreqsByPopSet(popSet,cytoMONList,presentMonly));
				sampleSizeListbyTypeAndLoc[trtype] = cytoMNList;}
			break;
		  default: break;
		 }//switch
		 if(!found){//append empty list for this trait type
		  	freqSet.Append(NEW ObjList<aList<long double> >());}

		 }//trtype
	if(!splitCytotypeBySex){
		int loc;
		for(loc=0;loc<freqSet[CYfemaleOnly].len();++loc){
			freqSet[CYfemale].Append(NEW aList<long double>(freqSet[CYfemaleOnly][loc]));}
		for(loc=freqSet[CYfemaleOnly].len()-1;loc>=0;--loc){
			freqSet[CYfemaleOnly].Delete(loc);}
		for(loc=0;loc<freqSet[CYmaleOnly].len();++loc){
			freqSet[CYmale].Append(NEW aList<long double>(freqSet[CYmaleOnly][loc]));}
		for(loc=freqSet[CYmaleOnly].len()-1;loc>=0;--loc){
			freqSet[CYmaleOnly].Delete(loc);}
		freqSet.Delete(CYmaleOnly);
		freqSet.Delete(CYfemaleOnly);
		}// !splitCytotypeBySex
   	return freqSetPtr;
	}//end MultiformatMarkerData::CalculateAlleleFreqsByPopSet






void MultiformatMarkerData::CalculateObservedHeterozygosity (void){
	if(codominantDataIncluded()){
		CodominantMarkerData::CalculateObservedHeterozygosity();}
	if(codominantSexLinkedDataIncluded()){
		SexLinkedCodomMarkerData::CalculateObservedHeterozygosity();}
	}//end MultiformatMarkerData::CalculateObservedHeterozygosity



void MultiformatMarkerData::CalculateTwoN (void){//doesn't do 1N for haploid markers
	//makes table of sample sizes for each pop, locus
	if(codominantDataIncluded()){
		CodominantMarkerData::CalculateTwoN();}
	if(codominantSexLinkedDataIncluded()){
		SexLinkedCodomMarkerData::CalculateTwoN();}
	}//end MultiformatMarkerData::CalculateTwoN



void MultiformatMarkerData::CheckForNewAlleles (void){
	if(haploidDataIncluded()){
		HaploidMarkerData::CheckForNewAlleles();}
	if(codominantDataIncluded()){
		CodominantMarkerData::CheckForNewAlleles();}
	if(codominantSexLinkedDataIncluded()){
		SexLinkedCodomMarkerData::CheckForNewAlleles();}
	if(dominantDataIncluded()){
		DominantMarkerData2::CheckForNewAlleles();}
	if(dominantSexLinkedDataIncluded()){
		SexLinkedDomMarkerData2::CheckForNewAlleles();}
	if(cytoplasmicFemaleMarkersIncluded()){
		CytoplasmicFemaleMarkerData::CheckForNewAlleles();}
	if(cytoplasmicMaleMarkersIncluded()){
		CytoplasmicMaleMarkerData::CheckForNewAlleles();}
	}//end MultiformatMarkerData::CheckForNewAlleles



void MultiformatMarkerData::Clear(void){
	HaploidMarkerData::Clear();
	CodominantMarkerData::Clear();
	DominantMarkerData2::Clear();
	SexLinkedCodomMarkerData::Clear();
	SexLinkedDomMarkerData2::Clear();
	CytoplasmicFemaleMarkerData::Clear();
	CytoplasmicMaleMarkerData::Clear();
	traitType_.Clear();
	traitTypesIncluded_.Clear();
	ResetFlags();
	}//end MultiformatMarkerData::Clear()


void MultiformatMarkerData::ConvertCodominantData(MultiformatMarkerData& mfData){
	//never use self-reference mfData=*this
	//converts extant codominant data into dominant marker phenotypes; skipped loci omitted
	//first allele is recessive to second allele; missing data treated as unscored
	mfData.CheckForNewAlleles();	
	int cloc, slcloc,trt;
	bool tooManyAlleles=false;
	for(cloc=0;cloc<mfData.nlociCodom_;++cloc){
		if(mfData.allelesPresent_[cloc].len() >2){
			tooManyAlleles=true; break;}}
	for(slcloc=0;slcloc<mfData.nlociSLCodom_;++slcloc){
		if(mfData.allelesPresentSLCodom_[slcloc].len() >2){
			tooManyAlleles=true; break;}}
	if(tooManyAlleles){
		aError("ERROR: MultiformatMarkerData::ConvertCodominantData only works when there are 1-2 allele copies/locus in the data");
		return;}
	Clear();
	for(trt=0;trt<numTraitTypes_;++trt){
		traitTypesIncluded_.Append(false);}

	title_=mfData.title_ + "--converted from codominant data";
	npop_=mfData.npop_;
	ngroups_=mfData.ngroups_;
	popSize_=mfData.popSize_;
	popName_=mfData.popName_;
	groupName_=mfData.groupName_;
	localityName_=mfData.localityName_;
	groupHierarchy_=mfData.groupHierarchy_;
	popsByLocality_=mfData.popsByLocality_;
	localityCoordinates_=mfData.localityCoordinates_;
	SetPloidy(mfData.ploidy());
	SetAlleleLength(mfData.alleleLen());
	heterogameticSex_=mfData.heterogameticSex_;
	sexTrait_=mfData.sexTrait_;
	int runningTraitNum=0;
	traitName_.Append(NEW aString("ID"));
	traitNum_.Append(runningTraitNum++);
	traitTypesIncluded_[0]=true;
	traitType_.Append(ID);
	if(mfData.nlociSLCodom_>0){
		traitName_.Append(NEW aString("sex"));
		traitNum_.Append(runningTraitNum++);
		sexTrait_=1;
		traitType_.Append(sex);
		traitTypesIncluded_[1]=true;
		heterogameticIndivs_=mfData.heterogameticIndivs_;//just copy
		homogameticIndivs_=mfData.homogameticIndivs_;
		}
	if(mfData.nlociCodom_>0){
		nlociDom_ = mfData.nlociCodom_;
		traitTypesIncluded_[dom]=true;
		for(cloc=0;cloc<mfData.nlociCodom_;++cloc){
			traitName_.Append(NEW aString(mfData.traitName_[mfData.locusNumCodom_[cloc]]));
			traitNum_.Append(runningTraitNum++);
			allDomLoci_.Append(traitNum_.Last());//last one created
			locusNumDom_.Append(traitNum_.Last());//last one created
			traitType_.Append(dom);
			}
		}//nlociCodom_
	if(mfData.nlociSLCodom_>0){
		nlociSLDom_ = mfData.nlociSLCodom_;
		traitTypesIncluded_[SLdom]=true;
		for(slcloc=0;slcloc<mfData.nlociSLCodom_;++slcloc){
			traitName_.Append(NEW aString(mfData.traitName_[mfData.locusNumSLCodom_[slcloc]]));
			traitNum_.Append(runningTraitNum++);
			allSLDomLoci_.Append(traitNum_.Last());//last one created
			locusNumSLDom_.Append(traitNum_.Last());//last one created
			traitType_.Append(SLdom);
			}
		}//nlociSLCodom_
	int pop,loc,i,p,pp,tr;
	bool recessive, missingflag;
	for(pop=0;pop<npop_;++pop){
		rawTraits_.Append(NEW ObjList<TextDataList>());//{0,1} presence-absence data as read from data file
		if(nlociDom_>0){
			nByLocusDom_.Append(NEW aList<int>());}
		for(loc=0;loc<nlociDom_;++loc){
			nByLocusDom_[pop].Append(0);}//initialize		
		if(nlociSLDom_>0){
			nByLocusSLDom_.Append(NEW aList<int>());}
		for(loc=0;loc<nlociSLDom_;++loc){
			nByLocusSLDom_[pop].Append(0);}//initialize		
		for(i=0;i<mfData.individuals(pop);++i){
			rawTraits_[pop].Append(NEW TextDataList());
			rawTraits_[pop][i].Append(NEW TextDatum(mfData.Trait(pop,i,0)));//ID code
			if(nlociSLDom_>0){
				rawTraits_[pop][i].Append(NEW TextDatum(mfData.Trait(pop,i,mfData.sexTrait_)));}//sex code
			for(cloc=0;cloc<mfData.nlociCodom_;++cloc){
				tr=mfData.locusNumCodom_[cloc];
//				TextDataList codomGenes(mfData.rawCodomGtypes_[pop][i][cloc]);//a copy
				TextDataList codomGenes(mfData.CodominantMarkerData::GenotypicVal(pop,i,tr));//a copy
				missingflag=false;
				for(p=0;p<ploidy();++p){
					if(codomGenes[p].missing()){missingflag=true;break;}}
				if(missingflag){//treat as unscored data
					rawTraits_[pop][i].Append(NEW TextDatum("",true));//store as missing
					continue;
					}
				codomGenes.Sort();//to order of codomData.allelesPresent_
				recessive=true;
				for(pp=0;pp<ploidy();++pp){
					if(codomGenes[pp] != mfData.allelesPresent_[cloc][0]){//recessive allele A present
						recessive=false;
						break;
						}}
				if (recessive){
						rawTraits_[pop][i].Append(NEW TextDatum("0",false));}
					else {
						rawTraits_[pop][i].Append(NEW TextDatum("1",false));}
				nByLocusDom_[pop][cloc]++;
				}//end for cloc
			for(slcloc=0;slcloc<mfData.nlociSLCodom_;++slcloc){
				tr=mfData.locusNumSLCodom_[slcloc];
//				TextDataList codomGenesSL(mfData.rawSLCodomGtypes_[pop][i][slcloc]);//a copy
				TextDataList codomGenesSL(mfData.SexLinkedCodomMarkerData::GenotypicVal(pop,i,tr));//a copy
				missingflag=false;
				int ploidyToUse = 2;
//				if(heterogametic(pop,i)){ploidyToUse=1;}
				if(mfData.heterogametic(pop,i)){ploidyToUse=1;}
				for(p=0;p<ploidyToUse;++p){
					if(codomGenesSL[p].missing()){missingflag=true;break;}}
				if(missingflag){//treat as unscored data
					rawTraits_[pop][i].Append(NEW TextDatum());//store as missing
					continue;
					}
				codomGenesSL.Sort();//to order of codomData.allelesPresent_
				recessive=true;
				for(pp=0;pp<ploidyToUse;++pp){
					if(codomGenesSL[pp] != mfData.allelesPresentSLCodom_[slcloc][0]){//recessive allele A present
						recessive=false;
						break;
						}}
				if (recessive){
						rawTraits_[pop][i].Append(NEW TextDatum("0"));}
					else {
						rawTraits_[pop][i].Append(NEW TextDatum("1"));}
				nByLocusSLDom_[pop][slcloc]++;
				}//end for slcloc
			}//end for i
		}//end for pop
	CheckForNewAlleles();	
	}//end MultiformatMarkerData::ConvertCodominantData(MultiformatMarkerData&)



void  MultiformatMarkerData::CreateBootstrapData(MultiformatMarkerData& origData, Ran3& ran){
	//individuals resampled
//	EvolutionaryGeneticsData::CreateBootstrapData(origData,ran);//faster to do it all here
	Clear();
	ResetFlags();
	title_ = origData.title_;	//title_ of the data set from the first data line
	title_.cat(" bootstrap replicate");
	ngroups_ = origData.ngroups_;
	npop_ = origData.npop_;
	ntraits_ = origData.ntraits_;
	popName_ = origData.popName_;
	groupName_ = origData.groupName_;
	localityName_ = origData.localityName_;
	traitName_ = origData.traitName_;
	traitNum_ = origData.traitNum_;
	skippedTraitNum_ = origData.skippedTraitNum_;
	groupHierarchy_ = origData.groupHierarchy_;
	popsByLocality_ = origData.popsByLocality_;
	localityCoordinates_ = origData.localityCoordinates_;
	missing_ = origData.missing_;
	sexTrait_ = origData.sexTrait_;
	primarySexRatio_ = origData.primarySexRatio_;
	heterogameticSex_ = origData.heterogameticSex_;
	CytoplasmicFemaleMarkerData::allPresentInBothSexes_ = origData.CytoplasmicFemaleMarkerData::allPresentInBothSexes_;
	CytoplasmicFemaleMarkerData::presentInBothSexes_ = origData.CytoplasmicFemaleMarkerData::presentInBothSexes_;
	allPresentInFemalesOnly_ = origData.allPresentInFemalesOnly_;
	presentInFemalesOnly_ = origData.presentInFemalesOnly_;
	CytoplasmicMaleMarkerData::allPresentInBothSexes_ = origData.CytoplasmicMaleMarkerData::allPresentInBothSexes_;
	CytoplasmicMaleMarkerData::presentInBothSexes_ = origData.CytoplasmicMaleMarkerData::presentInBothSexes_;
	allPresentInMalesOnly_ = origData.allPresentInMalesOnly_;
	presentInMalesOnly_ = origData.presentInMalesOnly_;

	SetPloidy(origData.ploidy());
	SetAlleleLength(origData.alleleLen());
	traitTypesIncluded_ = origData.traitTypesIncluded_;
	traitType_ = origData.traitType_;
	traitTypeCurrent_ = origData.traitTypeCurrent_;

	if(origData.haploidDataIncluded()){
		nlociHap_=origData.nlociHap_;
		allHapLoci_=origData.allHapLoci_;
		locusNumHap_=origData.locusNumHap_; 
		skippedLocusNumHap_=origData.skippedLocusNumHap_;
		}
	if(origData.codominantDataIncluded()){
		nlociCodom_ = origData.nlociCodom_;
		allCodomLoci_ = origData.allCodomLoci_;
		locusNumCodom_ = origData.locusNumCodom_;
		skippedLocusNumCodom_ = origData.skippedLocusNumCodom_;
		}
	if(origData.dominantDataIncluded()){
		nlociDom_=origData.nlociDom_;
		allDomLoci_=origData.allDomLoci_;
		locusNumDom_=origData.locusNumDom_; 
		skippedLocusNumDom_=origData.skippedLocusNumDom_;
		}
	if(origData.codominantSexLinkedDataIncluded()){
		nlociSLCodom_=origData.nlociSLCodom_; 
		allSLCodomLoci_=origData.allSLCodomLoci_; 
		locusNumSLCodom_=origData.locusNumSLCodom_;
		skippedLocusNumSLCodom_=origData.skippedLocusNumSLCodom_;
		}
	if(origData.dominantSexLinkedDataIncluded()){
		nlociSLDom_=origData.nlociSLDom_;
		allSLDomLoci_=origData.allSLDomLoci_;
		locusNumSLDom_=origData.locusNumSLDom_; 
		skippedLocusNumSLDom_=origData.skippedLocusNumSLDom_;
		}
	if(origData.cytoplasmicFemaleMarkersIncluded()){
		nlociCYfemale_=origData.nlociCYfemale_;
		allCYfemaleLoci_=origData.allCYfemaleLoci_;
		locusNumCYfemale_=origData.locusNumCYfemale_; 
		skippedLocusNumCYfemale_=origData.skippedLocusNumCYfemale_;
		allPresentInFemalesOnly_=origData.allPresentInFemalesOnly_;
		presentInFemalesOnly_=origData.presentInFemalesOnly_;
		}
	if(origData.cytoplasmicMaleMarkersIncluded()){
		nlociCYmale_=origData.nlociCYmale_;
		allCYmaleLoci_=origData.allCYmaleLoci_;
		locusNumCYmale_=origData.locusNumCYmale_; 
		skippedLocusNumCYmale_=origData.skippedLocusNumCYmale_;
		allPresentInMalesOnly_=origData.allPresentInMalesOnly_;
		presentInMalesOnly_=origData.presentInMalesOnly_;
		}
	
	int pop,i;
	for(pop=0;pop<npop_;++pop){//randomly resamples within populations
		rawTraits_.Append(NEW ObjList<TextDataList>());
		if(codominantDataIncluded()){
			rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());}
		if(codominantSexLinkedDataIncluded()){
			rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());}
		
		int newpop = ran.longBetween(0,npop_-1);
		int newpopsize = origData.rawTraits_[newpop].len();
		popSize_.Append(newpopsize);
		for (i=0;i<newpopsize;++i){
			int ranIndivID = ran.longBetween(0,newpopsize-1);
			rawTraits_[pop].Append((TextDataList*)origData.rawTraits_[newpop][ranIndivID].Duplicate());
			if(codominantDataIncluded()){
				rawCodomGtypes_[pop].Append(origData.rawCodomGtypes_[newpop][ranIndivID].Duplicate());}
			if(codominantSexLinkedDataIncluded()){
				rawSLCodomGtypes_[pop].Append(origData.rawSLCodomGtypes_[newpop][ranIndivID].Duplicate());}
		}}

	SortBySex();
	CheckForNewAlleles();	
	}//end  MultiformatMarkerData::CreateBootstrapData(MultiformatMarkerData&)




void MultiformatMarkerData::CreateRandomData(
							const long double Fst, 
							const long double Fis,
							const long double pBar, 
							const int numPopsSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,
							long double sexratio,
							int numTrueDemes,
							bool zygoteMigration){
	//	first set up initial variables
	Clear();
	ResetFlags();
	title_ = "Randomly generated data set with parametric Fst=";
	title_.cat(Fst,1,Digits(Fst)); title_.cat(", Fis="); title_.cat(Fis,1,Digits(Fis)); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar,1,Digits(pBar)); title_.cat(" among pops for all loci, ");
	if(numTrueDemes<2){
			title_.cat("infinite island model, ");}
		else{
			title_.cat("finite island model with ");
			title_.cat(numTrueDemes);
			title_.cat(" demes, ");
			}
	title_.cat("and migration by ");
	if(zygoteMigration){
			title_.cat("zygotes");}
		else{title_.cat("gametes");}
	npop_ = numPopsSampled;
	ngroups_ = 1;
	SetAlleleLength(1);
//	traitType_=numLociSampledByDataType;
	int tr, trt, trtype, pop, loc, i, gtype;
	for(trt=0;trt<numLociSampledByDataType.len();++trt){
		traitTypesIncluded_.Append(false);
		if(numLociSampledByDataType[trt]>0){
			traitTypesIncluded_[trt]=true;}}
	ntraits_ = numLociSampledByDataType.sum()+1;//ID
	traitName_.Append(NEW aString("ID"));
	traitType_.Append(ID);
	int runningTraitNum=0, unspecfiedTraits=1;
	traitNum_.Append(runningTraitNum++);
	traitTypesIncluded_[0]=true;

	if(sexLinkedDataIncluded()){
		ntraits_++;
		traitName_.Append(NEW aString("sex"));
		traitType_.Append(sex);
		traitNum_.Append(runningTraitNum++);
		traitTypesIncluded_[1]=true;
		unspecfiedTraits++;
		SetSexTrait();		
		for(pop=0;pop<npop_;++pop){
			homogameticIndivs_.Append(NEW aList<int>());
			heterogameticIndivs_.Append(NEW aList<int>());}
		}
		
	for(tr = 0;tr<ntraits_-unspecfiedTraits;++tr){
		aString locName("LOC"),trStr(tr+1);
		trStr.PrependCharToLen('0',Digits(ntraits_-unspecfiedTraits));
		locName.cat(trStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(runningTraitNum++);
		}
	for(trt=hap;trt<numLociSampledByDataType.len();++trt){
		for(tr=0;tr<numLociSampledByDataType[trt];++tr){
			traitType_.Append(trt);
			}}
	
	nlociHap_=nlociCodom_=nlociDom_=nlociSLCodom_=nlociSLDom_=nlociCYfemale_=nlociCYmale_=0;
	int locuscount=0;
	for(trtype=hap;trtype<numLociSampledByDataType.len();++trtype){//set up cross-referencing lists
		for(loc=0;loc<numLociSampledByDataType[trtype];++loc){
			switch(trtype){
			  case hap://haploid loci
				allHapLoci_.Append(locuscount+unspecfiedTraits);
				locusNumHap_.Append(locuscount+unspecfiedTraits);
				nlociHap_++;
				break;
			  case codom://codominant loci
				allCodomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCodom_.Append(locuscount+unspecfiedTraits);
				nlociCodom_++;
				break;
			  case dom://dominant loci
				allDomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumDom_.Append(locuscount+unspecfiedTraits);
				nlociDom_++;
				break;
			  case SLcodom://codominant sex-linked loci
				allSLCodomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumSLCodom_.Append(locuscount+unspecfiedTraits);
				nlociSLCodom_++;
				break;
			  case SLdom://dominant sex-linked loci
				allSLDomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumSLDom_.Append(locuscount+unspecfiedTraits);
				nlociSLDom_++;
				break;
			  case CYfemale://cytoplasmic female loci
				allCYfemaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYfemale_.Append(locuscount+unspecfiedTraits);
				CytoplasmicFemaleMarkerData::allPresentInBothSexes_.Append(locuscount+unspecfiedTraits);
				CytoplasmicFemaleMarkerData::presentInBothSexes_.Append(locuscount+unspecfiedTraits);
				nlociCYfemale_++;
				break;
			  case CYfemaleOnly://cytoplasmic female loci
				allCYfemaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYfemale_.Append(locuscount+unspecfiedTraits);
				allPresentInFemalesOnly_.Append(locuscount+unspecfiedTraits);
				presentInFemalesOnly_.Append(locuscount+unspecfiedTraits);
				nlociCYfemale_++;
				break;
			  case CYmale://cytoplasmic male loci
				allCYmaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYmale_.Append(locuscount+unspecfiedTraits);
				CytoplasmicMaleMarkerData::allPresentInBothSexes_.Append(locuscount+unspecfiedTraits);
				CytoplasmicMaleMarkerData::presentInBothSexes_.Append(locuscount+unspecfiedTraits);
				nlociCYmale_++;
				break;
			  case CYmaleOnly://cytoplasmic male loci
				allCYmaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYmale_.Append(locuscount+unspecfiedTraits);
				allPresentInMalesOnly_.Append(locuscount+unspecfiedTraits);
				presentInMalesOnly_.Append(locuscount+unspecfiedTraits);
				nlociCYmale_++;
				break;
			  default: break;
			  }//switch
 			 ++locuscount;
			 }//loc
			}//trtype

	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
	FstatisticAnalysisCodom fstCodomClass;
	FstatisticAnalysisHap fstHapClass;
	FstatisticAnalysisDom2 fstDomClass;
	FstatisticAnalysisCodomSexLink fstSLCodomClass;
	FstatisticAnalysisDomSexLink fstSLDomClass;
	FstatisticAnalysisCytoFemale fstCytoFemaleClass;
	FstatisticAnalysisCytoMale fstCytoMaleClass;

	long double NmEffective = fstCodomClass.NmFromDiploidFst(Fst,numTrueDemes,zygoteMigration);
	long double FstHapEffective = fstHapClass.HaploidFstFromNm(NmEffective,numTrueDemes);
	long double FstDomEffective = fstDomClass.DomFstFromNm(NmEffective,numTrueDemes,zygoteMigration);
	long double FstSLCodomEffective = fstSLCodomClass.SexLinkedCodomFstFromNm(NmEffective,numTrueDemes,zygoteMigration);
	long double FstSLDomEffective = fstSLDomClass.SexLinkedDomFstFromNm(NmEffective,numTrueDemes,zygoteMigration);
	long double FstCytoFemaleEffective = fstCytoFemaleClass.CytoplasmicFstFromNm(NmEffective,numTrueDemes);
	long double FstCytoMaleEffective = fstCytoMaleClass.CytoplasmicFstFromNm(NmEffective,numTrueDemes);

	GeneFrequencyDistribution parametricFreqDistrib1N(FstHapEffective, pBar, 1);//haploid
	GeneFrequencyDistribution parametricFreqDistrib2N(Fst, pBar, 2);//diploid codominant
	GeneFrequencyDistribution parametricFreqDistrib2NDom(FstDomEffective, pBar, 1);//diploid dominant -- marker freq
	GeneFrequencyDistribution parametricFreqDistribSL(FstSLCodomEffective, pBar, PloidyEffective());//sex-linked codominant
	GeneFrequencyDistribution parametricFreqDistribSLDom(FstSLDomEffective, pBar, 1);//sex-linked dominant -- marker freq
	GeneFrequencyDistribution parametricFreqDistribCytoF(FstCytoFemaleEffective, pBar,1);//cytoplasmic female
	GeneFrequencyDistribution parametricFreqDistribCytoM(FstCytoMaleEffective, pBar,1);//cytoplasmic male
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	TextDatum genotype;
	long double pf;
	ObjList<StatList<long double> > locusFrequenciesByPop;
	for(pop=0;pop<npop_;++pop){
		locusFrequenciesByPop.Append(NEW StatList<long double>());
		for(trtype=hap;trtype<numLociSampledByDataType.len();++trtype){
			for(loc=0;loc<numLociSampledByDataType[trtype];++loc){
				switch(trtype){
				  case hap://haploid loci
					pf = parametricFreqDistrib1N.RandomFrequency(ran);
					break;
				  case codom://codominant loci
					pf = parametricFreqDistrib2NDom.RandomFrequency(ran);
					break;
				  case dom://dominant loci
//					pf = parametricFreqDistrib1N.RandomFrequency(ran);
					pf = parametricFreqDistrib2N.RandomFrequency(ran);
					break;
				  case SLcodom://codominant sex-linked loci
					pf = parametricFreqDistribSL.RandomFrequency(ran);
					break;
				  case SLdom://dominant sex-linked loci
					pf = parametricFreqDistribSLDom.RandomFrequency(ran);
					break;
				  case CYfemale://cytoplasmic female loci
				  case CYfemaleOnly://cytoplasmic female loci
					pf = parametricFreqDistribCytoF.RandomFrequency(ran);
					break;
				  case CYmale://cytoplasmic male loci
				  case CYmaleOnly://cytoplasmic male loci
					pf = parametricFreqDistribCytoM.RandomFrequency(ran);
					break;
				  default: break;
				 }//switch
 			 locusFrequenciesByPop[pop].Append(pf);
			 ++locuscount;
			 }//loc
			}//trtype
		}//pop

	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("POP");
		aString pdigit(pop+1);
		pdigit.PrependCharToLen('0',Digits(npop_));
		pName.cat(pdigit);
		pName.cat(": randomly generated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		popName_.Append( NEW aString(pName));//append a copy

		//create id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	
		for (i=0;i<numIndivsPerPop;++i){
			rawTraits_[pop].Append(NEW TextDataList());
			aString indiv(i+1);
			indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));
			}//end for i
		popSize_.Append(numIndivsPerPop);

		if(numLociSampledByDataType[codom]>0){//codominant loci
			rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
			for (i=0;i<numIndivsPerPop;++i){
				rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				}//i
			}
		if(numLociSampledByDataType[SLcodom]>0){//sex-linked codominant loci
			rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
			for (i=0;i<numIndivsPerPop;++i){
				rawSLCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				}//i
			}

		//determine sex, if sex-linked
		if(sexLinkedDataIncluded()){
			char sexchar;
			for (i=0;i<numIndivsPerPop;++i){
				sexchar='M';
				if(ran()<sexratio){
					sexchar='F';}
				rawTraits_[pop][i].Append(NEW TextDatum(sexchar));
				if(sexchar==F && heterogameticSex()==F){
						heterogameticIndivs_[pop].Append(i);}
					else if(sexchar==F && heterogameticSex()==M){
						homogameticIndivs_[pop].Append(i);}
					else if(sexchar==M && heterogameticSex()==M){
						heterogameticIndivs_[pop].Append(i);}
					else if(sexchar==M && heterogameticSex()==F){
						homogameticIndivs_[pop].Append(i);}
					else{}
				}//i
			}
		
		//create and append genotypes
		locuscount=0;
		bool makeSexLinkedCodom;
		long double sr = primarySexRatio();
		GenotypeFreqDistribution popGenotypes;
//		GenotypeFreqDistDom popGenotypesDom;//pp == beta == marker frequency
		GenotypeFreqDistDom popGenotypesDom;//pp == genotype (not marker) frequency -- convert from codominant
		GenotypeFreqDistSexLinkDom popGenotypesSLdom;//pp == betaBar == marker frequendcy
		for(trtype=hap;trtype<numLociSampledByDataType.len();++trtype){//set up cross-referencing lists
			if(trtype==SLcodom){ makeSexLinkedCodom=true;}
				else{ makeSexLinkedCodom=false;}
			for(loc=0;loc<numLociSampledByDataType[trtype];++loc){
				long double pp = locusFrequenciesByPop[pop][locuscount];
					switch(trtype){
					  case hap://haploid loci
						for(i=0;i<numIndivsPerPop;++i){
							if(pp==zero){
									genotype="A";}
								else if(pp==one){
									genotype="B";}
								else if(ran()<pp){
									genotype="A";}
								else{genotype="B";}
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  case codom://codominant loci
						popGenotypes.Initialize(pp, Fis);
						for(i=0;i<numIndivsPerPop;++i){
							if(pp==zero){
									gtype=0;}
								else if(pp==one){
									gtype=2;}
								else{
									gtype = popGenotypes.RandomGenotype(ran);}
							if (gtype==0){genotype = "AA";}
								else if(gtype==1){genotype = "AB";}
								else if(gtype==2){genotype = "BB";}	
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							rawCodomGtypes_[pop][i].Append(NEW TextDataList());
							rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[0]));
							rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[1]));
							}//i
						break;
					  case dom://dominant loci
						popGenotypesDom.Initialize(pp, Fis);//pp == betaBar == marker frequency
						for(i=0;i<numIndivsPerPop;++i){
							if(pp==zero){
									gtype=0;}
								else{
//									gtype = popGenotypesDom.RandomGenotype(ran);}//this no longer confounds marker frequency with allele frequency
									gtype = popGenotypesDom.RandomGenotype(ran);}//this no longer confounds marker frequency with allele frequency
//							if (gtype==0){genotype = "1";}
//								else {genotype = "0";}// (gtype==1)
							if (gtype==0){genotype = "1";}//dominant homozygote
								else if (gtype==1) {genotype = "1";}//heterozygote 
								else {genotype = "0";}//recessive homozygote
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  case SLcodom://codominant sex-linked loci
						for(i=0;i<numIndivsPerPop;++i){
							if(rawTraits_[pop][i][1]==heterogameticSex()){//usually M
									if(pp==zero){
											genotype="A";}
										else if(pp==one){
											genotype="B";}
										else if(ran()<pp){
											genotype="A";}
										else{genotype="B";}
									}
								else{//homogametic
									if(pp==zero){
											gtype=0;}
										else if(pp==one){
											gtype=2;}
										else{
											gtype = popGenotypes.RandomGenotype(ran);}
									if (gtype==0){genotype = "AA";}
										else if(gtype==1){genotype = "AB";}
										else if(gtype==2){genotype = "BB";}
									}	
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							rawSLCodomGtypes_[pop][i].Append(NEW TextDataList());
							rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[0]));
							if(genotype.len()<2){
									rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum());}//missing data
								else{
									rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[1]));}
							}//i
						break;
					  case SLdom://dominant sex-linked loci
						popGenotypesSLdom.Initialize(pp, Fis,true,sr);//pp == betaBar == marker frequency
						for(i=0;i<numIndivsPerPop;++i){//this confounds marker freq with allele freq
							if(rawTraits_[pop][i][1]==heterogameticSex()){//usually M
									if(pp==zero){
											genotype="1";}
										else if(pp==one){
											genotype="0";}
										else if(ran()<pp){
											genotype="1";}
										else{genotype="0";}
									}
								else{//homogametic -- no longer confounds allele frequency with marker frequency
									if(pp==zero){
											gtype=0;}
										else{
											gtype = popGenotypesSLdom.RandomGenotype(ran,true);}//homogametic==true
									if (gtype==0){genotype = "1";}
										else{genotype = "0";}//(gtype==1)
									}	
								rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  case CYfemale://cytoplasmic female loci
						for(i=0;i<numIndivsPerPop;++i){
							if(pp==zero){
									genotype="1";}
								else if(pp==one){
									genotype="0";}
								else if(ran()<pp){
									genotype="1";}
								else{genotype="0";}
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  case CYfemaleOnly://cytoplasmic female loci
						for(i=0;i<numIndivsPerPop;++i){
							if(rawTraits_[pop][i][1]=='F' || rawTraits_[pop][i][1]=='f'){
									if(pp==zero){
											genotype="1";}
										else if(pp==one){
											genotype="0";}
										else if(ran()<pp){
											genotype="1";}
										else{genotype="0";}
									}
								else{//male
									genotype.missing(true);}
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  case CYmale://cytoplasmic male loci
						for(i=0;i<numIndivsPerPop;++i){
							if(pp==zero){
									genotype="1";}
								else if(pp==one){
									genotype="0";}
								else if(ran()<pp){
									genotype="1";}
								else{genotype="0";}
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  case CYmaleOnly://cytoplasmic male loci
						for(i=0;i<numIndivsPerPop;++i){
							if(rawTraits_[pop][i][1]=='M' || rawTraits_[pop][i][1]=='m'){
								if(pp==zero){
										genotype="1";}
									else if(pp==one){
										genotype="0";}
									else if(ran()<pp){
										genotype="1";}
									else{genotype="0";}
									}
								else{//female
									genotype.missing(true);}
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  default: break;
					  }//switch
				++locuscount;
				 }//loc
				}//trtype
		}//end for pop	
	GroupMembership();
	LocalityMembership();
	SortBySex();
	CheckForNewAlleles();
	}//end MultiformatMarkerData::CreateRandomData



void MultiformatMarkerData::CreateRandomData(
							const long double FstEffective, 
							const long double Fis,
							ObjList<ObjList<aList<long double> > >& pBar,//[datatype][loc][allele]
							ObjList<ObjList<aList<int> > >& numIndivsPerPopDataTypeLoc,//[pop][datatype][loc]
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,
							long double sexratio,
							int numTrueDemes,
							bool zygoteMigration){
	//	first set up initial variables
	Clear();
	ResetFlags();
	title_ = "Randomly generated data set with parametric Fst=";title_.cat(FstEffective,1,Digits(FstEffective)); 
	title_.cat(" and Fis="); title_.cat(Fis,1,Digits(Fis)); title_.cat(" among pops for all loci, ");
	if(numTrueDemes<2){
			title_.cat("infinite island model, ");}
		else{
			title_.cat("finite island model with ");
			title_.cat(numTrueDemes);
			title_.cat(" demes, ");
			}
	title_.cat("and migration by ");
	if(zygoteMigration){
			title_.cat("zygotes");}
		else{title_.cat("gametes");}
	npop_ = numIndivsPerPopDataTypeLoc.len();
	ngroups_ = 1;
	SetAlleleLength(1);
	int tr, trt, trtype, pop, loc, i, gtype;
	int unspecfiedTraits=0;
	for(trt=0;trt<numLociSampledByDataType.len();++trt){
		traitTypesIncluded_.Append(false);
		if(numLociSampledByDataType[trt]>0){
			traitTypesIncluded_[trt]=true;}}
	ntraits_ = numLociSampledByDataType.sum()+1;
	traitName_.Append(NEW aString("ID"));
	traitType_.Append(ID);
	int runningTraitNum=0;
	traitNum_.Append(runningTraitNum++);
	traitTypesIncluded_[0]=true;
	unspecfiedTraits=1;
	if(numTrueDemes<2) {numTrueDemes=0;}//infinite island model
	 else{numTrueDemes = MAX((int)npop_,numTrueDemes);}
	
	if(sexLinkedDataIncluded()){
		ntraits_++;
		traitName_.Append(NEW aString("sex"));
		traitType_.Append(sex);
		traitNum_.Append(runningTraitNum++);
		traitTypesIncluded_[0]=true;
		unspecfiedTraits++;
		SetSexTrait();		
		for(pop=0;pop<npop_;++pop){
			homogameticIndivs_.Append(NEW aList<int>());
			heterogameticIndivs_.Append(NEW aList<int>());}
		}
		
	for(tr = 0;tr<ntraits_-unspecfiedTraits;++tr){
		aString locName("LOC"),trStr(tr+1);
		trStr.PrependCharToLen('0',Digits(ntraits_-unspecfiedTraits));
		locName.cat(trStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(runningTraitNum++);
		}
	for(trt=hap;trt<numLociSampledByDataType.len();++trt){
		for(tr=0;tr<numLociSampledByDataType[trt];++tr){
			traitType_.Append(trt);
			}}
	
	int locuscount=0;
	nlociHap_=nlociCodom_=nlociDom_=nlociSLCodom_=nlociSLDom_=nlociCYfemale_=nlociCYmale_=0;
	for(trtype=2;trtype<numLociSampledByDataType.len();++trtype){//set up cross-referencing lists
		for(loc=0;loc<numLociSampledByDataType[trtype];++loc){
			switch(trtype){
			  case hap://haploid loci
				allHapLoci_.Append(locuscount+unspecfiedTraits);
				locusNumHap_.Append(locuscount+unspecfiedTraits);
				nlociHap_++;
				break;
			  case codom://codominant loci
				allCodomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCodom_.Append(locuscount+unspecfiedTraits);
				nlociCodom_++;
				break;
			  case dom://dominant loci
				allDomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumDom_.Append(locuscount+unspecfiedTraits);
				nlociDom_++;
				break;
			  case SLcodom://codominant sex-linked loci
				allSLCodomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumSLCodom_.Append(locuscount+unspecfiedTraits);
				nlociSLCodom_++;
				break;
			  case SLdom://dominant sex-linked loci
				allSLDomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumSLDom_.Append(locuscount+unspecfiedTraits);
				nlociSLDom_++;
				break;
			  case CYfemale://cytoplasmic female loci
				allCYfemaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYfemale_.Append(locuscount+unspecfiedTraits);
				CytoplasmicFemaleMarkerData::allPresentInBothSexes_.Append(locuscount+unspecfiedTraits);
				CytoplasmicFemaleMarkerData::presentInBothSexes_.Append(locuscount+unspecfiedTraits);
				nlociCYfemale_++;
				break;
			  case CYfemaleOnly://cytoplasmic female loci
				allCYfemaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYfemale_.Append(locuscount+unspecfiedTraits);
				allPresentInFemalesOnly_.Append(locuscount+unspecfiedTraits);
				presentInFemalesOnly_.Append(locuscount+unspecfiedTraits);
				nlociCYfemale_++;
				break;
			  case CYmale://cytoplasmic male loci
				allCYmaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYmale_.Append(locuscount+unspecfiedTraits);
				CytoplasmicMaleMarkerData::allPresentInBothSexes_.Append(locuscount+unspecfiedTraits);
				CytoplasmicMaleMarkerData::presentInBothSexes_.Append(locuscount+unspecfiedTraits);
				nlociCYmale_++;
				break;
			  case CYmaleOnly://cytoplasmic male loci
				allCYmaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYmale_.Append(locuscount+unspecfiedTraits);
				allPresentInMalesOnly_.Append(locuscount+unspecfiedTraits);
				presentInMalesOnly_.Append(locuscount+unspecfiedTraits);
				nlociCYmale_++;
				break;
			  default: break;
			  }//switch
			 ++locuscount;
			 }//loc
			}//trtype

	int dt,dloc,aa,item;
	aList<int> numIndivsPerPop;//max for each pop
	ObjList<aList<int> > individualIDsbyPop;
	for(pop=0;pop<numIndivsPerPopDataTypeLoc.len();++pop){
		numIndivsPerPop.Append(0);
		for(dt=0;dt<numIndivsPerPopDataTypeLoc[pop].len();++dt){
			for(dloc=0;dloc<numIndivsPerPopDataTypeLoc[pop][dt].len();++dloc){
				numIndivsPerPop[pop] = MAX(numIndivsPerPop[pop],numIndivsPerPopDataTypeLoc[pop][dt][dloc]);}}
		individualIDsbyPop.Append(NEW aList<int>());
		for(i=0;i<numIndivsPerPop[pop];++i){
			individualIDsbyPop[pop].Append(i);}
		}//pop

	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE UNIVERSE OF POPULATIONS
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	FstatisticAnalysisCodom fstCodomClass;
	FstatisticAnalysisHap fstHapClass;
	FstatisticAnalysisDom2 fstDomClass;
	FstatisticAnalysisCodomSexLink fstSLCodomClass;
	FstatisticAnalysisDomSexLink fstSLDomClass;
	FstatisticAnalysisCytoFemale fstCytoFemaleClass;//separate distributions for sexes in case sex ratio not 1:1
	FstatisticAnalysisCytoMale fstCytoMaleClass;
	long double Nm = fstCodomClass.NmFromDiploidFst(FstEffective,numTrueDemes,zygoteMigration);
	long double FstHapEquiv = fstHapClass.HaploidFstFromNm(Nm,numTrueDemes);
	long double FstDomEquiv = fstDomClass.DomFstFromNm(Nm,numTrueDemes,zygoteMigration);
	long double FstSLCodomEquiv = fstSLCodomClass.SexLinkedCodomFstFromNm(Nm,numTrueDemes,zygoteMigration);
	long double FstSLDomEquiv = fstSLDomClass.SexLinkedDomFstFromNm(Nm,numTrueDemes,zygoteMigration);
	long double FstCytoFemaleEquiv = fstCytoFemaleClass.CytoplasmicFstFromNm(Nm,numTrueDemes);
	long double FstCytoMaleEquiv = fstCytoMaleClass.CytoplasmicFstFromNm(Nm,numTrueDemes);
	TextDatum genotype;
	aList<long double> *pf=NULL;
	ObjList<StatList<long double> > locusFrequenciesByPop;
	GeneFrequencyDistMultiAllele parametricFreqDistrib1N(one);//haploid
	GeneFrequencyDistMultiAllele parametricFreqDistrib2N(two);//diploid
	GeneFrequencyDistMultiAllele parametricFreqDistribDom(one);//diploid dominant
	GeneFrequencyDistMultiAllele parametricFreqDistribSL(SexLinkedCodomMarkerData::PloidyEffective());//sex-linked
	GeneFrequencyDistMultiAllele parametricFreqDistribSLDom(one);//sex-linked dom
	GeneFrequencyDistMultiAllele parametricFreqDistribCytoFemale(CytoplasmicFemaleMarkerData::PloidyEffective());//cytoplasmic passed on by one sex
	GeneFrequencyDistMultiAllele parametricFreqDistribCytoMale(CytoplasmicMaleMarkerData::PloidyEffective());//cytoplasmic passed on by one sex
	for(pop=0;pop<npop_;++pop){
		locusFrequenciesByPop.Append(NEW StatList<long double>());
//		locuscount=0;
		for(trtype=hap;trtype<numLociSampledByDataType.len();++trtype){
			for(loc=0;loc<numLociSampledByDataType[trtype];++loc){
				switch(trtype){
				  case hap://haploid loci
					parametricFreqDistrib1N.Initialize(FstHapEquiv, pBar[trtype][loc]);//haploid
					pf = parametricFreqDistrib1N.RandomFrequencies(ran);
					break;
				  case codom://codominant loci
					parametricFreqDistrib2N.Initialize(FstEffective, pBar[trtype][loc]);//diploid
					pf = parametricFreqDistrib2N.RandomFrequencies(ran);
					break;
				  case dom://dominant loci
					parametricFreqDistribDom.Initialize(FstDomEquiv, pBar[trtype][loc]);//diploid dominant
					pf = parametricFreqDistribDom.RandomFrequencies(ran);
					break;
				  case SLcodom://codominant sex-linked loci
					parametricFreqDistribSL.Initialize(FstSLCodomEquiv, pBar[trtype][loc]);//sex-linked
					pf = parametricFreqDistribSL.RandomFrequencies(ran);
					break;
				  case SLdom://dominant sex-linked loci
					parametricFreqDistribSLDom.Initialize(FstSLDomEquiv, pBar[trtype][loc]);//sex-linked dom
					pf = parametricFreqDistribSLDom.RandomFrequencies(ran);
					break;
				  case CYfemale://cytoplasmic female loci
					parametricFreqDistribCytoFemale.Initialize(FstCytoFemaleEquiv, pBar[CYfemale][loc]);//cytoplasmic passed on by one sex
					pf = parametricFreqDistribCytoFemale.RandomFrequencies(ran);
					break;
				  case CYfemaleOnly://cytoplasmic female loci
					parametricFreqDistribCytoFemale.Initialize(FstCytoFemaleEquiv, pBar[CYfemaleOnly][loc]);//cytoplasmic passed on by one sex
					pf = parametricFreqDistribCytoFemale.RandomFrequencies(ran);
					break;
				  case CYmale://cytoplasmic male loci
					parametricFreqDistribCytoMale.Initialize(FstCytoMaleEquiv, pBar[CYmale][loc]);//cytoplasmic passed on by one sex
					pf = parametricFreqDistribCytoMale.RandomFrequencies(ran);
					break;
				  case CYmaleOnly://cytoplasmic male loci
					parametricFreqDistribCytoMale.Initialize(FstCytoMaleEquiv, pBar[CYmaleOnly][loc]);//cytoplasmic passed on by one sex
					pf = parametricFreqDistribCytoMale.RandomFrequencies(ran);
					break;
				  default: break;
				 }//switch
				 for(aa=0;aa<pf->len();++aa){
				 	locusFrequenciesByPop[pop].Append((*pf)[aa]);}
				 delete pf;pf=NULL;
//			++locuscount;
			 }//loc
			}//trtype
		}//pop

	ObjList<aList<int> >maleID, femaleID; //[pop][i] -- used if sex-linked data needed
	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("POP");
		aString pdigit(pop+1);
		pdigit.PrependCharToLen('0',Digits(npop_));
		pName.cat(pdigit);
		pName.cat(": randomly generated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	

		for (i=0;i<numIndivsPerPop[pop];++i){
			rawTraits_[pop].Append(NEW TextDataList());
			aString indiv(i+1);
			indiv.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop[pop])));
			rawTraits_[pop][i].Append(NEW TextDatum(indiv));}
		popSize_.Append(numIndivsPerPop[pop]);

		//determine sex, if sex-linked
		if(sexLinkedDataIncluded()){
			maleID.Append(NEW aList<int>());
			femaleID.Append(NEW aList<int>());
			char sexchar;
			for (i=0;i<numIndivsPerPop[pop];++i){
				sexchar='M';//randomly allocate sex according to sex ratio info
				if(ran()<sexratio){
					sexchar='F';}
				rawTraits_[pop][i].Append(NEW TextDatum(sexchar));				
				if(sexchar==F && heterogameticSex()==F){
						heterogameticIndivs_[pop].Append(i);}
					else if(sexchar==F && heterogameticSex()==M){
						homogameticIndivs_[pop].Append(i);}
					else if(sexchar==M && heterogameticSex()==M){
						heterogameticIndivs_[pop].Append(i);}
					else if(sexchar==M && heterogameticSex()==F){
						homogameticIndivs_[pop].Append(i);}
					else{}
				if(sexchar==F){
						femaleID[pop].Append(i);}
					else{
						maleID[pop].Append(i);}
				}//i
			}

		if(numLociSampledByDataType[codom]>0){//codominant loci
			rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
			for (i=0;i<numIndivsPerPop[pop];++i){
				rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				}//i
			}
		if(numLociSampledByDataType[SLcodom]>0){//sex-linked codominant loci
			rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
			for (i=0;i<numIndivsPerPop[pop];++i){
				rawSLCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				}//i
			}

		
		//create and append genotypes
		locuscount=0;
		GenotypeFreqDistribution popGenotypes;
		GenotypeFreqDistDom popGenotypesDom;
		GenotypeFreqDistSexLinkDom popGenotypesSLdom;
		aList<int> listToSampleFrom;// listOfAllIndivs;//use to sample randomly if not all individuals carry gene copies
		for(trtype=hap;trtype<numLociSampledByDataType.len();++trtype){//set up cross-referencing lists
			for(loc=0;loc<numLociSampledByDataType[trtype];++loc){
				long double pp = locusFrequenciesByPop[pop][locuscount];
				switch(trtype){
				  case hap://haploid loci
				  	listToSampleFrom = individualIDsbyPop[pop];//sample without replacement
					for(i=0;i<numIndivsPerPopDataTypeLoc[pop][trtype][loc];++i){
						if(pp==zero){
								genotype="A";}
							else if(pp==one){
								genotype="B";}
							else if(ran()<pp){
								genotype="A";}
							else{genotype="B";}
						item=ran.longBetween(0,listToSampleFrom.len()-1);
						rawTraits_[pop][listToSampleFrom[item]].Append(NEW TextDatum(genotype));
						listToSampleFrom.Delete(item);
						}//i
					break;
				  case codom://codominant loci
					popGenotypes.Initialize(pp, Fis);
				  	listToSampleFrom = individualIDsbyPop[pop];//sample without replacement
					for(i=0;i<numIndivsPerPopDataTypeLoc[pop][trtype][loc];++i){
						if(pp==zero){
								gtype=0;}
							else if(pp==one){
								gtype=2;}
							else{
								gtype = popGenotypes.RandomGenotype(ran);}
						if (gtype==0){genotype = "AA";}
							else if(gtype==1){genotype = "AB";}
							else if(gtype==2){genotype = "BB";}	
						item=ran.longBetween(0,listToSampleFrom.len()-1);
						rawTraits_[pop][listToSampleFrom[item]].Append(NEW TextDatum(genotype));
						listToSampleFrom.Delete(item);
						}//i
					break;
				  case dom://dominant loci
					popGenotypesDom.Initialize(pp, Fis);//pp == beta == marker frequency
				  	listToSampleFrom = individualIDsbyPop[pop];//sample without replacement
					for(i=0;i<numIndivsPerPopDataTypeLoc[pop][trtype][loc];++i){
						if(pp==zero){
								gtype=0;}
							else{
								gtype = popGenotypesDom.RandomGenotype(ran);}//this no longer confounds marker frequency with allele frequency
						if (gtype==0){genotype = "1";}
							else {genotype = "0";}//(gtype==1)
						item=ran.longBetween(0,listToSampleFrom.len()-1);
						rawTraits_[pop][listToSampleFrom[item]].Append(NEW TextDatum(genotype));
						listToSampleFrom.Delete(item);
						}//i
					break;
				  case SLcodom://codominant sex-linked loci
					popGenotypes.Initialize(pp, Fis);
				  	listToSampleFrom = individualIDsbyPop[pop];//sample without replacement
					for(i=0;i<numIndivsPerPopDataTypeLoc[pop][trtype][loc];++i){
						item=ran.longBetween(0,listToSampleFrom.len()-1);
						if(rawTraits_[pop][listToSampleFrom[item]][1]==heterogameticSex()){//usually M
								if(pp==zero){
										genotype="A";}
									else if(pp==one){
										genotype="B";}
									else if(ran()<pp){
										genotype="A";}
									else{genotype="B";}
								}
							else{//homogametic
								if(pp==zero){
										gtype=0;}
									else if(pp==one){
										gtype=2;}
									else{
										gtype = popGenotypes.RandomGenotype(ran);}
								if (gtype==0){genotype = "AA";}
									else if(gtype==1){genotype = "AB";}
									else if(gtype==2){genotype = "BB";}
								}	
						rawTraits_[pop][listToSampleFrom[item]].Append(NEW TextDatum(genotype));
						listToSampleFrom.Delete(item);
						}//i
					break;
				  case SLdom://dominant sex-linked loci
					popGenotypesSLdom.Initialize(pp, Fis,true,0.5);//pp == betaBar == marker frequency
				  	listToSampleFrom = individualIDsbyPop[pop];//sample without replacement
					for(i=0;i<numIndivsPerPopDataTypeLoc[pop][trtype][loc];++i){
						item=ran.longBetween(0,listToSampleFrom.len()-1);
						if(rawTraits_[pop][listToSampleFrom[item]][1]==heterogameticSex()){//usually M
								if(pp==zero){
										genotype="1";}
									else if(pp==one){
										genotype="0";}
									else if(ran()<pp){
										genotype="1";}
									else{genotype="0";}
								}
							else{//homogametic -- no longer confounds allele frequency with marker frequency
								if(pp==zero){
										gtype=0;}
									else{
										gtype = popGenotypesSLdom.RandomGenotype(ran,true);}//homogametic==true
								if (gtype==0){genotype = "1";}
									else{genotype = "0";}//(gtype==1)
								}	
						rawTraits_[pop][listToSampleFrom[item]].Append(NEW TextDatum(genotype));
						listToSampleFrom.Delete(item);
						}//i
					break;
				  case CYfemale://cytoplasmic female loci
				  	listToSampleFrom = individualIDsbyPop[pop];//sample without replacement
					for(i=0;i<numIndivsPerPopDataTypeLoc[pop][trtype][loc];++i){
						if(pp==zero){
								genotype="1";}
							else if (pp==one){
								genotype="0";}
							else if (ran()<pp){genotype = "1";}
							else {genotype = "0";}
						item=ran.longBetween(0,listToSampleFrom.len()-1);
						rawTraits_[pop][listToSampleFrom[item]].Append(NEW TextDatum(genotype));
						listToSampleFrom.Delete(item);
						}//i
					break;
				  case CYfemaleOnly://cytoplasmic female loci
				  	listToSampleFrom = femaleID[pop];//sample without replacement
					int fsampled = MIN((int)listToSampleFrom.len(),numIndivsPerPopDataTypeLoc[pop][trtype][loc]);
					for(i=0;i<fsampled;++i){
						item=ran.longBetween(0,listToSampleFrom.len()-1);
						if(rawTraits_[pop][listToSampleFrom[item]][1]=='F' || rawTraits_[pop][listToSampleFrom[item]][1]=='f'){
								if(pp==zero){genotype="1";}
									else if(pp==one){genotype="0";}
									else if(ran()<pp){
										genotype="1";}
									else{genotype="0";}
								}
							else{//male
								genotype.missing(true);}
						rawTraits_[pop][listToSampleFrom[item]].Append(NEW TextDatum(genotype));
						listToSampleFrom.Delete(item);
						}//i
					for(i=0;i<maleID[pop].len();++i){
						rawTraits_[pop][maleID[pop][i]].Append(NEW TextDatum());
						}//i
					break;
				  case CYmale://cytoplasmic male loci
				  	listToSampleFrom = individualIDsbyPop[pop];//sample without replacement
					for(i=0;i<numIndivsPerPopDataTypeLoc[pop][trtype][loc];++i){
						if(pp==zero){genotype="1";}
							else if (pp==one){genotype="0";}
							else if (ran()<pp){genotype = "1";}
							else {genotype = "0";}
						item=ran.longBetween(0,listToSampleFrom.len()-1);
						rawTraits_[pop][listToSampleFrom[item]].Append(NEW TextDatum(genotype));
						listToSampleFrom.Delete(item);
						}//i
					break;
				  case CYmaleOnly://cytoplasmic male loci
				  	listToSampleFrom = maleID[pop];//sample without replacement
//				  	listOfAllIndivs = individualIDsbyPop[pop];
					int msampled = MIN((int)listToSampleFrom.len(),numIndivsPerPopDataTypeLoc[pop][trtype][loc]);
					for(i=0;i<msampled;++i){
						item = ran.longBetween(0,listToSampleFrom.len()-1);
						if(rawTraits_[pop][listToSampleFrom[item]][1]=='M' || rawTraits_[pop][listToSampleFrom[item]][1]=='m'){
							if(pp==zero){genotype="1";}
								else if (pp==one){genotype="0";}
								else if(ran()<pp){
										genotype="1";}
								else{genotype="0";}
								}
							else{//female
								genotype.missing(true);}
						rawTraits_[pop][listToSampleFrom[item]].Append(NEW TextDatum(genotype));
//						int itemAll = listOfAllIndivs.Loc(listToSampleFrom[item]);
						listToSampleFrom.Delete(item);
//						listOfAllIndivs.Delete(itemAll);
						}//i
					for(i=0;i<femaleID[pop].len();++i){
						rawTraits_[pop][femaleID[pop][i]].Append(NEW TextDatum());
						}//i
					break;
				  default: break;
				  }//switch
				//treat remainder as missing data
				for(i=0;i<listToSampleFrom.len();++i){
					rawTraits_[pop][listToSampleFrom[i]].Append(NEW TextDatum());
					}//i
				++locuscount;
				}//loc
			}//trtype
		}//end for pop	
	GroupMembership();
	LocalityMembership();
	SortBySex();
	ParseGenotypicData();
	CheckForNewAlleles();	
//	aString datadump("datadump.txt");
//	PrintGelJockFile(datadump);
	}//end MultiformatMarkerData::CreateRandomData



void MultiformatMarkerData::CreateRandomData(
							const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran,
							long double sexratio,
							int numTrueDemesWithinSpecies,
							int numTrueDemesBetweenSpecies,
							bool zygoteMigration){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated multi-species data set with parametric Flt=";
	title_.cat(Flt,1,Digits(Flt)); title_.cat(", Fspt="); title_.cat(Fspt,1,Digits(Fspt));  title_.cat(", Fis="); title_.cat(Fis,Digits(Fis)); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar,1,Digits(pBar)); title_.cat(" among pops for all loci, ");
	if(numTrueDemesWithinSpecies<2){
			title_.cat("infinite");}
		else{title_.cat(numTrueDemesWithinSpecies);}
	title_.cat(" demes within species, ");
	if(numTrueDemesBetweenSpecies<2){
			title_.cat("infinite");}
		else{title_.cat(numTrueDemesBetweenSpecies);}
	title_.cat(" species, and migration by ");
	if(zygoteMigration){
			title_.cat("zygotes");}
		else{title_.cat("gametes");}
	
	npop_ = numSpeciesSampled * numLocalitiesSampled;
	ngroups_ = numSpeciesSampled;
	SetAlleleLength(1);
	int tr, trt, trtype, pop, loc, i, gtype;
	int unspecfiedTraits=0;
	for(trt=0;trt<numLociSampledByDataType.len();++trt){
		traitTypesIncluded_.Append(false);
		if(numLociSampledByDataType[trt]>0){
			traitTypesIncluded_[trt]=true;}}
	ntraits_ = numLociSampledByDataType.sum()+1;
	traitName_.Append(NEW aString("ID"));
	int runningTraitNum=0;
	traitNum_.Append(runningTraitNum++);
	traitTypesIncluded_[0]=true;
	unspecfiedTraits++;

	if(sexLinkedDataIncluded()){
		ntraits_++;
		traitName_.Append(NEW aString("sex"));
		traitNum_.Append(runningTraitNum++);
		traitTypesIncluded_[0]=true;
		unspecfiedTraits++;
		SetSexTrait();		
		for(pop=0;pop<npop_;++pop){
			homogameticIndivs_.Append(NEW aList<int>());
			heterogameticIndivs_.Append(NEW aList<int>());}
		}
		
	for(tr = 0;tr<ntraits_-unspecfiedTraits;++tr){
		aString locName("LOC"),trStr(tr+1);
		trStr.PrependCharToLen('0',Digits(ntraits_-unspecfiedTraits));
		locName.cat(trStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(runningTraitNum++);
		}
	for(trt=hap;trt<numLociSampledByDataType.len();++trt){
		for(tr=0;tr<numLociSampledByDataType[trt];++tr){
			traitType_.Append(trt);
			}}
	

	int locuscount=0;
	nlociHap_=nlociCodom_=nlociDom_=nlociSLCodom_=nlociSLDom_=nlociCYfemale_=nlociCYmale_=0;
	for(trtype=hap;trtype<numLociSampledByDataType.len();++trtype){//set up cross-referencing lists
		for(loc=0;loc<numLociSampledByDataType[trtype];++loc){
			switch(trtype){
			  case hap://haploid loci
				allHapLoci_.Append(locuscount+unspecfiedTraits);
				locusNumHap_.Append(locuscount+unspecfiedTraits);
				nlociHap_++;
				break;
			  case codom://codominant loci
				allCodomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCodom_.Append(locuscount+unspecfiedTraits);
				nlociCodom_++;
				break;
			  case dom://dominant loci
				allDomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumDom_.Append(locuscount+unspecfiedTraits);
				nlociDom_++;
				break;
			  case SLcodom://codominant sex-linked loci
				allSLCodomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumSLCodom_.Append(locuscount+unspecfiedTraits);
				nlociSLCodom_++;
				break;
			  case SLdom://dominant sex-linked loci
				allSLDomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumSLDom_.Append(locuscount+unspecfiedTraits);
				nlociSLDom_++;
				break;
			  case CYfemale://cytoplasmic female loci
				allCYfemaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYfemale_.Append(locuscount+unspecfiedTraits);
				CytoplasmicFemaleMarkerData::allPresentInBothSexes_.Append(locuscount+unspecfiedTraits);
				CytoplasmicFemaleMarkerData::presentInBothSexes_.Append(locuscount+unspecfiedTraits);
				nlociCYfemale_++;
				break;
			  case CYfemaleOnly://cytoplasmic female loci
				allCYfemaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYfemale_.Append(locuscount+unspecfiedTraits);
				allPresentInFemalesOnly_.Append(locuscount+unspecfiedTraits);
				presentInFemalesOnly_.Append(locuscount+unspecfiedTraits);
				nlociCYfemale_++;
				break;
			  case CYmale://cytoplasmic male loci
				allCYmaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYmale_.Append(locuscount+unspecfiedTraits);
				CytoplasmicMaleMarkerData::allPresentInBothSexes_.Append(locuscount+unspecfiedTraits);
				CytoplasmicMaleMarkerData::presentInBothSexes_.Append(locuscount+unspecfiedTraits);
				nlociCYmale_++;
				break;
			  case CYmaleOnly://cytoplasmic male loci
				allCYmaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYmale_.Append(locuscount+unspecfiedTraits);
				allPresentInMalesOnly_.Append(locuscount+unspecfiedTraits);
				presentInMalesOnly_.Append(locuscount+unspecfiedTraits);
				nlociCYmale_++;
				break;
			  default: break;
			  }//switch
			 ++locuscount;
			 }//loc
			}//trtype
	int loci_=locuscount;

	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE SPECIES
	FstatisticAnalysisCodom fstCodomClass;
	FstatisticAnalysisHap fstHapClass;
	FstatisticAnalysisDom2 fstDomClass;
	FstatisticAnalysisCodomSexLink fstSLCodomClass;
	FstatisticAnalysisDomSexLink fstSLDomClass;
	FstatisticAnalysisCytoFemale fstCytoFemaleClass;
	FstatisticAnalysisCytoMale fstCytoMaleClass;

	long double NmspEffective = fstCodomClass.NmFromDiploidFst(Fspt,numTrueDemesWithinSpecies,zygoteMigration);
	long double FsptHapEffective = fstHapClass.HaploidFstFromNm(NmspEffective,numTrueDemesWithinSpecies);
	long double FsptDomEffective = fstDomClass.DomFstFromNm(NmspEffective,numTrueDemesWithinSpecies,zygoteMigration);
	long double FsptSLCodomEffective = fstSLCodomClass.SexLinkedCodomFstFromNm(NmspEffective,numTrueDemesWithinSpecies,zygoteMigration);
	long double FsptSLDomEffective = fstSLDomClass.SexLinkedDomFstFromNm(NmspEffective,numTrueDemesWithinSpecies,zygoteMigration);
	long double FsptCytoFemaleEffective = fstCytoFemaleClass.CytoplasmicFstFromNm(NmspEffective,numTrueDemesWithinSpecies);
	long double FsptCytoMaleEffective = fstCytoMaleClass.CytoplasmicFstFromNm(NmspEffective,numTrueDemesWithinSpecies);

	long double NmltEffective = fstCodomClass.NmFromDiploidFst(Flt,numTrueDemesBetweenSpecies,zygoteMigration);
	long double FltHapEffective = fstHapClass.HaploidFstFromNm(NmltEffective,numTrueDemesBetweenSpecies);
	long double FltDomEffective = fstDomClass.DomFstFromNm(NmltEffective,numTrueDemesBetweenSpecies,zygoteMigration);
	long double FltSLCodomEffective = fstSLCodomClass.SexLinkedCodomFstFromNm(NmltEffective,numTrueDemesBetweenSpecies,zygoteMigration);
	long double FltSLDomEffective = fstSLDomClass.SexLinkedDomFstFromNm(NmltEffective,numTrueDemesBetweenSpecies,zygoteMigration);
	long double FltCytoFemaleEffective = fstCytoFemaleClass.CytoplasmicFstFromNm(NmltEffective,numTrueDemesBetweenSpecies);
	long double FltCytoMaleEffective = fstCytoMaleClass.CytoplasmicFstFromNm(NmltEffective,numTrueDemesBetweenSpecies);

	long double NmlsptEffective = fstCodomClass.NmFromDiploidFst(Flspt,numTrueDemesWithinSpecies*numTrueDemesBetweenSpecies,zygoteMigration);
	long double FlsptHapEffective = fstHapClass.HaploidFstFromNm(NmlsptEffective,numTrueDemesWithinSpecies*numTrueDemesBetweenSpecies);
	long double FlsptDomEffective = fstDomClass.DomFstFromNm(NmlsptEffective,numTrueDemesWithinSpecies*numTrueDemesBetweenSpecies,zygoteMigration);
	long double FlsptSLCodomEffective = fstSLCodomClass.SexLinkedCodomFstFromNm(NmlsptEffective,numTrueDemesWithinSpecies*numTrueDemesBetweenSpecies,zygoteMigration);
	long double FlsptSLDomEffective = fstSLDomClass.SexLinkedDomFstFromNm(NmlsptEffective,numTrueDemesWithinSpecies*numTrueDemesBetweenSpecies,zygoteMigration);
	long double FlsptCytoFemaleEffective = fstCytoFemaleClass.CytoplasmicFstFromNm(NmlsptEffective,numTrueDemesWithinSpecies*numTrueDemesBetweenSpecies);
	long double FlsptCytoMaleEffective = fstCytoMaleClass.CytoplasmicFstFromNm(NmlsptEffective,numTrueDemesWithinSpecies*numTrueDemesBetweenSpecies);

	GeneFrequencyDistribution parametricFreqDistribSP_1N(FsptHapEffective, pBar, 1);//haploid
	GeneFrequencyDistribution parametricFreqDistribSP_2N(Fspt, pBar, 2);//diploid
	GeneFrequencyDistribution parametricFreqDistribSP_Dom(FsptDomEffective, pBar, 1);//diploid dominant
	GeneFrequencyDistribution parametricFreqDistribSP_SL(FsptSLCodomEffective, pBar, PloidyEffective());//sex-linked
	GeneFrequencyDistribution parametricFreqDistribSP_SLDom(FsptSLDomEffective, pBar, 1);//sex-linked dominant
	GeneFrequencyDistribution parametricFreqDistribCytoSP_CYF(FsptCytoFemaleEffective, pBar, 1);//cytoplasmic passed on by one sex
	GeneFrequencyDistribution parametricFreqDistribCytoSP_CYM(FsptCytoMaleEffective, pBar, 1);//cytoplasmic passed on by one sex

	GeneFrequencyDistribution parametricFreqDistribL_1N(FltHapEffective, pBar, 1);//haploid
	GeneFrequencyDistribution parametricFreqDistribL_2N(Flt, pBar, 2);//diploid
	GeneFrequencyDistribution parametricFreqDistribL_Dom(FltDomEffective, pBar, 1);//diploid dominant
	GeneFrequencyDistribution parametricFreqDistribL_SL(FltSLCodomEffective, pBar, PloidyEffective());//sex-linked
	GeneFrequencyDistribution parametricFreqDistribL_SLDom(FltSLDomEffective, pBar, 1);//sex-linked dom
	GeneFrequencyDistribution parametricFreqDistribCytoL_CYF(FltCytoFemaleEffective, pBar,1);//cytoplasmic passed on by one sex
	GeneFrequencyDistribution parametricFreqDistribCytoL_CYM(FltCytoMaleEffective, pBar, 1);//cytoplasmic passed on by one sex

	GeneFrequencyDistribution parametricFreqDistribLSP_1N(FlsptHapEffective, pBar, 1);//haploid
	GeneFrequencyDistribution parametricFreqDistribLSP_2N(Flspt, pBar, 2);//diploid
	GeneFrequencyDistribution parametricFreqDistribLSP_Dom(FlsptDomEffective, pBar, 1);//diploid dominant
	GeneFrequencyDistribution parametricFreqDistribLSP_SL(FlsptSLCodomEffective, pBar, PloidyEffective());//sex-linked
	GeneFrequencyDistribution parametricFreqDistribLSP_SLDom(FlsptSLDomEffective, pBar, 1);//sex-linked dom
	GeneFrequencyDistribution parametricFreqDistribCytoLSP_CYF(FlsptCytoFemaleEffective, pBar, 1);//cytoplasmic passed on by one sex
	GeneFrequencyDistribution parametricFreqDistribCytoLSP_CYM(FlsptCytoMaleEffective, pBar, 1);//cytoplasmic passed on by one sex
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	TextDatum genotype;
	long double pf;
	ObjList<aList<long double> > pBarBySpeciesAndLocus, pBarByLocalityAndLocus;
	int sp0;
	for(sp0=0;sp0<numSpeciesSampled;++sp0){
		locuscount=0;
		pBarBySpeciesAndLocus.Append(NEW aList<long double>());
		for(trtype=hap;trtype<numLociSampledByDataType.len();++trtype){
			for(loc=0;loc<numLociSampledByDataType[trtype];++loc){
				++locuscount;
				switch(trtype){
				  case hap://haploid loci
					pf = parametricFreqDistribSP_1N.RandomFrequency(ran);
					break;
				  case codom://codominant loci
					pf = parametricFreqDistribSP_2N.RandomFrequency(ran);
					break;
				  case dom://dominant loci
					pf = parametricFreqDistribSP_Dom.RandomFrequency(ran);
					break;
				  case SLcodom://codominant sex-linked loci
					pf = parametricFreqDistribSP_SL.RandomFrequency(ran);
					break;
				  case SLdom://dominant sex-linked loci
					pf = parametricFreqDistribSP_SLDom.RandomFrequency(ran);
					break;
				  case CYfemale://cytoplasmic female loci
				  case CYfemaleOnly://cytoplasmic female loci
					pf = parametricFreqDistribCytoSP_CYF.RandomFrequency(ran);
					break;
				  case CYmale://cytoplasmic male loci
				  case CYmaleOnly://cytoplasmic male loci
					pf = parametricFreqDistribCytoSP_CYM.RandomFrequency(ran);
					break;
				  default: break;
				 }//switch
 				 pBarBySpeciesAndLocus[sp0].Append(pf);
			 }//loc
			}//trtype
		}//end for sp0

	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG localities
	int locale0;
	for(locale0=0;locale0<numLocalitiesSampled;++locale0){
		locuscount=0;
		pBarByLocalityAndLocus.Append(NEW aList<long double>());
		for(trtype=hap;trtype<numLociSampledByDataType.len();++trtype){
			for(loc=0;loc<numLociSampledByDataType[trtype];++loc){
				switch(trtype){
				  case hap://haploid loci
					pf = parametricFreqDistribL_1N.RandomFrequency(ran);
					break;
				  case codom://codominant loci
					pf = parametricFreqDistribL_2N.RandomFrequency(ran);
					break;
				  case dom://dominant loci
					pf = parametricFreqDistribL_Dom.RandomFrequency(ran);
					break;
				  case SLcodom://codominant sex-linked loci
					pf = parametricFreqDistribL_SL.RandomFrequency(ran);
					break;
				  case SLdom://dominant sex-linked loci
					pf = parametricFreqDistribL_SLDom.RandomFrequency(ran);
					break;
				  case CYfemale://cytoplasmic female loci
				  case CYfemaleOnly://cytoplasmic female loci
					pf = parametricFreqDistribCytoL_CYF.RandomFrequency(ran);
					break;
				  case CYmale://cytoplasmic male loci
				  case CYmaleOnly://cytoplasmic male loci
					pf = parametricFreqDistribCytoL_CYM.RandomFrequency(ran);
					break;
				  default: break;
				 }//switch
 				 pBarByLocalityAndLocus[locale0].Append(pf);
 			 ++locuscount;
			 }//loc
			}//trtype
		}//end for locale0
		
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	int sp1,locale1;
	for(sp1=0; sp1<numSpeciesSampled;++sp1){
		for (locale1=0;locale1<numLocalitiesSampled; ++locale1){
			// NAME THE POPULATION
			aString loca(locale1+1);
			loca.PrependCharToLen('0',Digits(numSpeciesSampled*numLocalitiesSampled));
			aString pName("LOCALITY");pName.cat(loca);pName.cat(" (");pName.cat(locale1+1);
			pName.cat(") randomly generated population of SPECIES"); pName.cat(sp1+1);
			popName_.Append(NEW aString(pName));//append a copy
			pop = popName_.len()-1;
			//SET UP APPROPRIATE DATA ARRAYS
			//create id numbers for each individual
			rawTraits_.Append(NEW ObjList<TextDataList>());	
			for (i=0;i<numIndivsPerPop;++i){
				rawTraits_[pop].Append(NEW TextDataList());
				aString indivID(i+1);
				indivID.PrependCharToLen(' ', MAX(4,Digits(numIndivsPerPop)));
				rawTraits_[pop][i].Append(NEW TextDatum(indivID));
				}//end for i
			popSize_.Append(numIndivsPerPop);

			if(numLociSampledByDataType[codom]>0){//codominant loci
				rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
				for (i=0;i<numIndivsPerPop;++i){
					rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
//					for(int loc=0;loc<numLociSampledByDataType[2];++loc){
//						rawCodomGtypes_[pop][i].Append(NEW TextDataList());}					
					}//i
				}
			if(numLociSampledByDataType[SLcodom]>0){//sex-linked codominant loci
				rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
				for (i=0;i<numIndivsPerPop;++i){
					rawSLCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
//					for(int loc=0;loc<numLociSampledByDataType[4];++loc){
//						rawSLCodomGtypes_[pop][i].Append(NEW TextDataList());}					
					}//i
				}

			//determine sex, if sex-linked
			if(sexLinkedDataIncluded()){
				char sexchar;
				for (i=0;i<numIndivsPerPop;++i){
					sexchar='M';
					if(ran()<sexratio){
						sexchar='F';}
					rawTraits_[pop][i].Append(NEW TextDatum(sexchar));			
					if(sexchar==F && heterogameticSex()==F){
							heterogameticIndivs_[pop].Append(i);}
						else if(sexchar==F && heterogameticSex()==M){
							homogameticIndivs_[pop].Append(i);}
						else if(sexchar==M && heterogameticSex()==M){
							heterogameticIndivs_[pop].Append(i);}
						else if(sexchar==M && heterogameticSex()==F){
							homogameticIndivs_[pop].Append(i);}
						else{}
					}//i
				}

			//create and append genotypes
			locuscount=0;
			GenotypeFreqDistribution popGenotypes;
			GenotypeFreqDistribution popGenotypesSL;
			GenotypeFreqDistDom popGenotypesDom;//marker frequency
			GenotypeFreqDistSexLinkDom popGenotypesSLdom;//marker frequency
			for(trtype=hap;trtype<numLociSampledByDataType.len();++trtype){//set up cross-referencing lists
				for(loc=0;loc<numLociSampledByDataType[trtype];++loc){
					long double pp_1N = parametricFreqDistribLSP_1N.RandomFrequency(ran);//removing species and locality main effects
					long double pp_2N = parametricFreqDistribLSP_2N.RandomFrequency(ran);
					long double beta_Dom = parametricFreqDistribLSP_Dom.RandomFrequency(ran);
					long double pp_SL = parametricFreqDistribLSP_SL.RandomFrequency(ran);
					long double betaBar_SLDom = parametricFreqDistribLSP_SLDom.RandomFrequency(ran);
					long double pp_CYF = parametricFreqDistribCytoLSP_CYF.RandomFrequency(ran);
					long double pp_CYM = parametricFreqDistribCytoLSP_CYM.RandomFrequency(ran);
					pp_1N += (pBarBySpeciesAndLocus[sp1][locuscount] - pBar) + (pBarByLocalityAndLocus[locale1][locuscount] - pBar);//adjust for species and locality
					pp_2N += (pBarBySpeciesAndLocus[sp1][locuscount] - pBar) + (pBarByLocalityAndLocus[locale1][locuscount] - pBar);
					beta_Dom += (pBarBySpeciesAndLocus[sp1][locuscount] - pBar) + (pBarByLocalityAndLocus[locale1][locuscount] - pBar);
					pp_SL += (pBarBySpeciesAndLocus[sp1][locuscount] - pBar) + (pBarByLocalityAndLocus[locale1][locuscount] - pBar);
					betaBar_SLDom += (pBarBySpeciesAndLocus[sp1][locuscount] - pBar) + (pBarByLocalityAndLocus[locale1][locuscount] - pBar);
					pp_CYF += (pBarBySpeciesAndLocus[sp1][locuscount] - pBar) + (pBarByLocalityAndLocus[locale1][locuscount] - pBar);
					pp_CYM += (pBarBySpeciesAndLocus[sp1][locuscount] - pBar) + (pBarByLocalityAndLocus[locale1][locuscount] - pBar);
					pp_1N = MAX(pp_1N,(long double)0.0);pp_1N = MIN(pp_1N,(long double)1.0);//rescale to bounds
					pp_2N = MAX(pp_2N,(long double)0.0);pp_2N = MIN(pp_2N,(long double)1.0);//rescale to bounds
					beta_Dom = MAX(beta_Dom,(long double)0.0);beta_Dom = MIN(beta_Dom,(long double)1.0);//rescale to bounds
					pp_SL = MAX(pp_SL,(long double)0.0);pp_SL = MIN(pp_SL,(long double)1.0);//rescale to bounds
					betaBar_SLDom = MAX(betaBar_SLDom,(long double)0.0);betaBar_SLDom = MIN(betaBar_SLDom,(long double)1.0);//rescale to bounds
					pp_CYF = MAX(pp_CYF,(long double)0.0);pp_CYF = MIN(pp_CYF,(long double)1.0);//rescale to bounds
					pp_CYM = MAX(pp_CYM,(long double)0.0);pp_CYM = MIN(pp_CYM,(long double)1.0);//rescale to bounds
					
					switch(trtype){
					  case hap://haploid loci
						for(i=0;i<numIndivsPerPop;++i){
							if(pp_1N==zero){
									genotype="A";}
								else if(pp_1N==one){
									genotype="B";}
								if(ran()<pp_1N){
										genotype="A";}
									else{genotype="B";}								
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  case codom://codominant loci
					  	popGenotypes.Initialize(pp_2N, Fis);
						for(i=0;i<numIndivsPerPop;++i){
							if(pp_2N==zero){
									gtype=0;}
								else if(pp_2N==one){
									gtype=2;}
								else{
									gtype = popGenotypes.RandomGenotype(ran);}
							if (gtype==0){genotype = "AA";}
								else if(gtype==1){genotype = "AB";}
								else if(gtype==2){genotype = "BB";}	
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							rawCodomGtypes_[pop][i].Append(NEW TextDataList());
							rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[0]));
							rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[1]));
							}//i
						break;
					  case dom://dominant loci
						popGenotypesDom.Initialize(beta_Dom, Fis);
						for(i=0;i<numIndivsPerPop;++i){
							if(beta_Dom==zero){
									gtype=0;}
								else{
									gtype = popGenotypesDom.RandomGenotype(ran);}//this no longer confounds marker frequency with allele frequency
							if (gtype==0){genotype = "1";}
								else if(gtype==1){genotype = "1";}
								else if(gtype==2){genotype = "0";}	
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  case SLcodom://codominant sex-linked loci
						popGenotypesSL.Initialize(pp_SL, Fis);
						for(i=0;i<numIndivsPerPop;++i){
							if(rawTraits_[pop][i][1]==heterogameticSex()){//usually M
									if(pp_SL==zero){
											genotype="A";}
										else if(pp_SL==one){
											genotype="B";}
										else if(ran()<pp_SL){
											genotype="A";}
										else{genotype="B";}
									}
								else{//homogametic
									if(pp_SL==zero){
											gtype=0;}
										else if(pp_SL==one){
											gtype=2;}
										else{
											gtype = popGenotypesSL.RandomGenotype(ran);}
									if (gtype==0){genotype = "AA";}
										else if(gtype==1){genotype = "AB";}
										else if(gtype==2){genotype = "BB";}
									}	

							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							rawSLCodomGtypes_[pop][i].Append(NEW TextDataList());
							rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[0]));
							if(genotype.len()<2){
									rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum());}//missing data
								else{
									rawSLCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[1]));}
							}//i
						break;
					  case SLdom://dominant sex-linked loci
						popGenotypesSLdom.Initialize(betaBar_SLDom, Fis,true,0.5);//pp == betaBar == marker frequency
						for(i=0;i<numIndivsPerPop;++i){
							if(rawTraits_[pop][i][1]==heterogameticSex()){//usually M
									if(pp_1N==zero){
											genotype="1";}
										else if(pp_1N==one){
											genotype="0";}
										else if(ran()<pp_1N){
											genotype="1";}
										else{genotype="0";}
									}
								else{//homogametic -- no longer confounds allele frequency with marker frequency
									if(betaBar_SLDom==zero){
											gtype=0;}
										else if(betaBar_SLDom==one){
											gtype=2;}
										else{
											gtype = popGenotypesSLdom.RandomGenotype(ran,true);}//homogametic==true
									if (gtype==0){genotype = "1";}
										else if(gtype==1){genotype = "1";}
										else if(gtype==2){genotype = "0";}
									}	
								rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  case CYfemale://cytoplasmic female loci
						for(i=0;i<numIndivsPerPop;++i){
							if(ran()<pp_CYF){
									genotype="1";}
								else{genotype="0";}
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  case CYfemaleOnly://cytoplasmic female loci
						for(i=0;i<numIndivsPerPop;++i){
							if(rawTraits_[pop][i][1]=='F' || rawTraits_[pop][i][1]=='m'){
									if(ran()<pp_CYF){
											genotype="1";}
										else{genotype="0";}
									}
								else{//male
									genotype.missing(true);}
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  case CYmale://cytoplasmic male loci
						for(i=0;i<numIndivsPerPop;++i){
							if(ran()<pp_CYM){
									genotype="1";}
								else{genotype="0";}
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  case CYmaleOnly://cytoplasmic male loci
						for(i=0;i<numIndivsPerPop;++i){
							if(rawTraits_[pop][i][1]=='M' || rawTraits_[pop][i][1]=='m'){
									if(ran()<pp_CYM){
											genotype="1";}
										else{genotype="0";}
									}
								else{//female
									genotype.missing(true);}
							rawTraits_[pop][i].Append(NEW TextDatum(genotype));
							}//i
						break;
					  default: break;
					  }//switch
					++locuscount;
					 }//loc
					}//trtype


			}//end for locality1
		}//end for sp1	
	GroupMembership();
	LocalityMembership();
	SortBySex();
	CheckForNewAlleles();	
	}//end MultiformatMarkerData::CreateRandomData



/*
void MultiformatMarkerData::CreateRandomData(//not debugged
							const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Flspt, //differentiation among subpops after means accounted for
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Flt=";
	title_.cat(Flt); title_.cat(", Fspt="); title_.cat(Fspt);  
	title_.cat(", Flspt="); title_.cat(Flspt); title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numSpeciesSampled * numLocalitiesSampled;
	nlociCodom_ = numLociSampled;
	ngroups_ = numSpeciesSampled;
	traitName_.Append(NEW aString("ID"));
	for(int loc = 0;loc<nlociCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociCodom_);
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		allCodomLoci_.Append(loc+1);
		locusNumCodom_.Append(loc+1);
		}


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE SPECIES
	GeneFrequencyDistribution parametricFreqDistribSP(Fspt, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<aList<long double> > pBarBySpeciesAndLocus;
	for(int sp0=0;sp0<numSpeciesSampled;++sp0){
		aList<long double>* locusPList = NEW aList<long double>();
		pBarBySpeciesAndLocus.Append(locusPList);
		for(int locus0=0;locus0<nlociCodom_;++locus0){
			long double p0 = parametricFreqDistribSP.RandomFrequency(ran);
			locusPList->Append(p0);
			}//end for locus0
		}//end for sp0
		
	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE LOCALITIES
	GeneFrequencyDistribution parametricFreqDistribLocality(Flt, pBar, 2);
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<aList<long double> > pBarByLocalityAndLocus;
	for(int L0=0;L0<numLocalitiesSampled;++L0){
		aList<long double>* locusPList = NEW aList<long double>();
		pBarByLocalityAndLocus.Append(locusPList);
		for(int locus0=0;locus0<nlociCodom_;++locus0){
			long double p0 = parametricFreqDistribLocality.RandomFrequency(ran);
			locusPList->Append(p0);
			}//end for locus0
		}//end for L0
		
	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE SUBPOPULATIONS
//	GeneFrequencyDistribution parametricFreqDistribSubpop(Flspt, pBar, 1000);//old version sends dp
	GeneFrequencyDistribution parametricFreqDistribSubpop(Flspt, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	ObjList<ObjList<aList<long double> > > pBarBySpeciesLocalityAndLocus;
	for(int sp1=0;sp1<numSpeciesSampled;++sp1){
		pBarBySpeciesLocalityAndLocus.Append(NEW ObjList<aList<long double> >());
		for(int L1=0;L1<numLocalitiesSampled;++L1){
			aList<long double>* locusPList = NEW aList<long double>();
			pBarBySpeciesLocalityAndLocus[sp1].Append(locusPList);
			for(int locus0=0;locus0<nlociCodom_;++locus0){
				long double p0 = parametricFreqDistribSubpop.RandomFrequency(ran);
				long double devLocality = (pBarByLocalityAndLocus[L1][locus0] - pBar);
				long double devSpecies = (pBarBySpeciesAndLocus[sp1][locus0] - pBar);
				long double p = p0 + devLocality + devSpecies;
				locusPList->Append(p);
				}//end for locus0
			}//end for L0
		}//end for sp0
		
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	for(int sp1=0; sp1<numSpeciesSampled;++sp1){
		//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG populations within the species
//		ObjList<StatList<double> > locusFrequenciesByLocality;
//		for(int locality1=0;locality1<numLocalitiesSampled; ++locality1){
//			StatList<double>* locusFrequencies = new StatList<double>();
//			locusFrequenciesByLocality.Append(locusFrequencies);
//			for(int loc=0;loc<nloci;++loc){
//				GeneFrequencyDistribution parametricFreqDistrib(Flspt, pBarBySpeciesLocalityAndLocus[sp1][locality1][loc], 1000);
//				double p = parametricFreqDistrib.RandomFrequency(ran);
//				locusFrequencies->Append(p);
//				}//end for loc
//			}//end for locality1
//
		for (int locality1=0;locality1<numLocalitiesSampled; ++locality1){
			// NAME THE POPULATION
			aString pName("LOCALITY");pName.cat(locality1+1);pName.cat(" (");pName.cat(locality1+1);
			pName.cat(") randomly generated population of SPECIES"); pName.cat(sp1+1);
			popName_.Append(NEW aString(pName));//append a copy
			int pop = popName_.len()-1;
			//create id numbers for each individual
			rawTraits_.Append(NEW ObjList<TextDataList>());	
			rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());	
			for (int i=0;i<numIndivsPerPop;++i){
				rawTraits_[pop].Append(NEW TextDataList());
				aString indiv(i+1);//first create id numbers for each individual
				indiv.PrependCharToLen(' ', MAX(4,numIndivsPerPop));
				rawTraits_[pop][i].Append(NEW TextDatum(indiv));
				rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				for(int loc=0;loc<nlociCodom_;++loc){
					rawCodomGtypes_[pop][i].Append(NEW TextDataList());}					
				}//end for i
			popSize_.Append(numIndivsPerPop);

			for(int loc=0;loc<nlociCodom_;++loc){
				long double p = pBarBySpeciesLocalityAndLocus[sp1][locality1][loc];
				GenotypeFreqDistribution popGenotypes(p, Fis);
				for(int i=0;i<numIndivsPerPop;++i){
					aString genotype = '\0';
					int gtype = popGenotypes.RandomGenotype(ran);
					if (gtype==0){genotype = "AA";}
						else if(gtype==1){genotype = "AB";}
						else if(gtype==2){genotype = "BB";}			
					rawTraits_[pop][i].Append(NEW TextDatum(genotype));
					rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[0]));		
					rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[1]));		
					}//end for i
				}//end for loc
			}//end for locality1
		}//end for sp1	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end MultiformatMarkerData::CreateRandomData


void MultiformatMarkerData::CreateRandomDataLocFirst(
							const long double Fspt,//differentation among species
							const long double Flt, //differentiation among localities
							const long double Fis,
							const long double pBar, //grand mean
							const int numSpeciesSampled,
							const int numLocalitiesSampled,
							const int numIndivsPerPop,
							StatList<int>& numLociSampledByDataType,
							Ran3List& ran){
	//	first set up initial variables
	Clear();
	title_ = "Randomly generated data set with parametric Flt=";
	title_.cat(Flt); title_.cat(", Fspt="); title_.cat(Fspt);  title_.cat(", Fis="); title_.cat(Fis); 
	title_.cat(", and mean frequency of p="); title_.cat(pBar); title_.cat(" among pops for all loci");
	npop_ = numSpeciesSampled * numLocalitiesSampled;
	nlociCodom_ = numLociSampled;
	ngroups_ = numSpeciesSampled;
	traitName_.Append(NEW aString("ID"));
	for(int loc = 0;loc<nlociCodom_;++loc){
		aString locName("LOC"),locStr(loc+1);
		locStr.PrependCharToLen('0',nlociCodom_);
		locName.cat(locStr);
		traitName_.Append(NEW aString(locName));//append a copy
		allCodomLoci_.Append(loc+1);
		locusNumCodom_.Append(loc+1);
		}


	//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG THE LOCALITIES
//	GeneFrequencyDistribution parametricFreqDistribL(Flt, pBar, 1000);//old version sends dp
	GeneFrequencyDistribution parametricFreqDistribL(Flt, pBar, 2);//new version sends ploidy
		//THIS IMPLEMENTS WRIGHT'S EQUILIBRIUM DISTRIBUTION, SEE VOL. 2, EQN 13.48, PAGE 362
	//testDistribution.PrintToScreen();
	aList<long double> pBarByLocality;
	for(int L0=0;L0<numLocalitiesSampled;++L0){
		long double p0 = parametricFreqDistribL.RandomFrequency(ran);
		pBarByLocality.Append(p0);
		}//end for L0
		
	//SAMPLE PARAMETRIC GENE FREQUENCIES OF POPULATIONS FROM THE UNIVERSE OF POPULATIONS
		//each locus in each population is sampled independently
	for(int L1=0; L1<numLocalitiesSampled;++L1){
		//GENERATE THE DISTRIBUTION OF PARAMETRIC GENE FREQUENCIES AMONG SPECIES WITHIN THE LOCALITIES
//		GeneFrequencyDistribution parametricFreqDistrib(Fspt, pBarByLocality[L1], 1000);//old version sends dp
		GeneFrequencyDistribution parametricFreqDistrib(Fspt, pBarByLocality[L1], 2);//new version sends ploidy
		ObjList<StatList<long double> > locusFrequenciesBySpecies;
		for(int species1=0;species1<numSpeciesSampled; ++species1){
			StatList<long double>* locusFrequencies = NEW StatList<long double>();
			locusFrequenciesBySpecies.Append(locusFrequencies);
			for(int loc=0;loc<nlociCodom_;++loc){
				long double p = parametricFreqDistrib.RandomFrequency(ran);
				locusFrequencies->Append(p);
				}//end for loc
			}//end for species1

		for (int species1=0;species1<numSpeciesSampled; ++species1){
			// NAME THE POPULATION
			aString pName("LOCALITY");pName.cat(L1+1);pName.cat(" (");pName.cat(L1+1);
			pName.cat(") randomly generated population of SPECIES"); pName.cat(species1+1);
			popName_.Append(NEW aString(pName));//append a copy
			int pop = popName_.len()-1;
			//create id numbers for each individual
			rawTraits_.Append(NEW ObjList<TextDataList>());	
			rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());	
			for (int i=0;i<numIndivsPerPop;++i){
				rawTraits_[pop].Append(NEW TextDataList());
				aString indiv(i+1);//first create id numbers for each individual
				indiv.PrependCharToLen(' ', MAX(4,numIndivsPerPop));
				rawTraits_[pop][i].Append(NEW TextDatum(indiv));
				rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				for(int loc=0;loc<nlociCodom_;++loc){
					rawCodomGtypes_[pop][i].Append(NEW TextDataList());}					
				}//end for i
			popSize_.Append(numIndivsPerPop);

			for(int loc=0;loc<nlociCodom_;++loc){
				StatList<long double>& locusFrequencies = locusFrequenciesBySpecies[species1];
				long double p = locusFrequencies[loc];
				GenotypeFreqDistribution popGenotypes(p, Fis);
				for(int i=0;i<numIndivsPerPop;++i){
					aString genotype = '\0';
					int gtype = popGenotypes.RandomGenotype(ran);
					if (gtype==0){genotype = "AA";}
						else if(gtype==1){genotype = "AB";}
						else if(gtype==2){genotype = "BB";}			
					rawTraits_[pop][i].Append(NEW TextDatum(genotype));
					rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[0]));		
					rawCodomGtypes_[pop][i][loc].Append(NEW TextDatum(genotype[1]));		
					}//end for i
				}//end for loc
			}//end for species1
		}//end for L1	
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();	
	}//end MultiformatMarkerData::CreateRandomDataLocFirst
*/



void MultiformatMarkerData::CullLoci (aList<int>& lociToCull){
	//This function shifts trait numbers from from traitNums_ to the relevant skippedTraitNums_ lists
	if(lociToCull.len()==0){return;}//none to cull
	traitTypeCurrent_=false;
	lociToCull.Sort();
	int tr, cLoc,datasettype;
	long item;
	//cull them from traitNum_, then cull from subclasses' lists
	for(tr=lociToCull.len()-1;tr>=0;--tr){
		cLoc = lociToCull[tr];//make sure it's a valid cull step
		if(traitNum_.Includes(cLoc) && cLoc>=0 && cLoc<traitName_.len() && (! skippedTraitNum_.Includes(cLoc))){
			item = traitNum_.Loc(cLoc);
			skippedTraitNum_.Append(cLoc);
			traitNum_.Delete(item);
			datasettype = traitType_[cLoc];
			switch(datasettype){
			  case hap://haploid
					if(! skippedLocusNumHap_.Includes(cLoc)){
						skippedLocusNumHap_.Append(cLoc);}
					item = locusNumHap_.Loc(cLoc);
					if(item>=0){
						locusNumHap_.Delete(item);}
					break;
			  case codom://codominant
					if(! skippedLocusNumCodom_.Includes(cLoc)){
						skippedLocusNumCodom_.Append(cLoc);}
					item = locusNumCodom_.Loc(cLoc);
					if(item>=0){
						locusNumCodom_.Delete(item);}
					break;
			  case dom://dominant
					if(! skippedLocusNumDom_.Includes(cLoc)){
						skippedLocusNumDom_.Append(cLoc);}
					item = locusNumDom_.Loc(cLoc);
					if(item>=0){
						locusNumDom_.Delete(item);}
					break;
			  case SLcodom://sex-linked codominant
					if(! skippedLocusNumSLCodom_.Includes(cLoc)){
						skippedLocusNumSLCodom_.Append(cLoc);}
					item = locusNumSLCodom_.Loc(cLoc);
					if(item>=0){
						locusNumSLCodom_.Delete(item);}
					break;
			  case SLdom://sex-linked dominant
					if(! skippedLocusNumSLDom_.Includes(cLoc)){
						skippedLocusNumSLDom_.Append(cLoc);}
					item = locusNumSLDom_.Loc(cLoc);
					if(item>=0){
						locusNumSLDom_.Delete(item);}
					break;
			  case CYfemale://cytoplasmic female 
					if(! skippedLocusNumCYfemale_.Includes(cLoc)){
						skippedLocusNumCYfemale_.Append(cLoc);}
					item = locusNumCYfemale_.Loc(cLoc);
					if(item>=0){
						locusNumCYfemale_.Delete(item);}
					break;
			  case CYfemaleOnly://cytoplasmic female 
					if(! skippedLocusNumCYfemale_.Includes(cLoc)){
						skippedLocusNumCYfemale_.Append(cLoc);}
					item = locusNumCYfemale_.Loc(cLoc);
					if(item>=0){
						locusNumCYfemale_.Delete(item);}
					item = presentInFemalesOnly_.Loc(cLoc);
					if(item>=0){
						presentInFemalesOnly_.Delete(item);}
					break;
			  case CYmale://cytoplasmic male
					if(! skippedLocusNumCYmale_.Includes(cLoc)){
						skippedLocusNumCYmale_.Append(cLoc);}
					item = locusNumCYmale_.Loc(cLoc);
					if(item>=0){
						locusNumCYmale_.Delete(item);}
					break;
			  case CYmaleOnly://cytoplasmic male
					if(! skippedLocusNumCYmale_.Includes(cLoc)){
						skippedLocusNumCYmale_.Append(cLoc);}
					item = locusNumCYmale_.Loc(cLoc);
					if(item>=0){
						locusNumCYmale_.Delete(item);}
					item = presentInMalesOnly_.Loc(cLoc);
					if(item>=0){
						presentInMalesOnly_.Delete(item);}
					break;
			  default://not in a sublist
			  		break;
			  }//switch
			}//if item not skipped already
		}//tr
	nlociHap_ = locusNumHap_.len();
	nlociCodom_ = locusNumCodom_.len();
	nlociDom_ = locusNumDom_.len();
	nlociSLCodom_ = locusNumSLCodom_.len();
	nlociSLDom_ = locusNumSLDom_.len();
	nlociCYfemale_ = locusNumCYfemale_.len();
	nlociCYmale_ = locusNumCYmale_.len();
	skippedTraitNum_.Sort();
	skippedLocusNumHap_.Sort();
	skippedLocusNumCodom_.Sort();
	skippedLocusNumDom_.Sort();
	skippedLocusNumSLCodom_.Sort();
	skippedLocusNumSLDom_.Sort();
	skippedLocusNumCYfemale_.Sort();
	skippedLocusNumCYmale_.Sort();
	allPresentInFemalesOnly_.Sort();
	presentInFemalesOnly_.Sort();
	allPresentInMalesOnly_.Sort();
	presentInMalesOnly_.Sort();

	ResetGeneticFlags();
	traitTypeCurrent_=true;
	SortBySex();
	CheckForNewAlleles();
	}//end procedure MultiformatMarkerData::CullLoci(aList<int>&)


void MultiformatMarkerData::CullLoci (StringList& lociToCull){
	//This function culls the designated loci from each genotype data string
	//and includes them among the skipped loci
	if(lociToCull.len()==0){return;}//none to cull
	//convert names to numbers and cull
	aList<int> cullListByNum;
	int tr, item;
	for(tr=0;tr<lociToCull.len();++tr){
		aString cTraitname = lociToCull[tr];
		item = traitName_.Loc(&cTraitname);
		if((item>=0)&&(!(skippedTraitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already skipped
			cullListByNum.Append(item);
			}//if item exists
		}//tr
	CullLoci(cullListByNum);
	}//end procedure MultiformatMarkerData::CullLoci(StringList&)



void MultiformatMarkerData::CullLocus (char* locusToCull){
	aString locToCull(locusToCull);
	CullLocus(locToCull);
	}//end MultiformatMarkerData::CullLocus (char* )


void MultiformatMarkerData::CullLocus (int locusToCull){
	aList<int> lociToCull;
	lociToCull.Append(locusToCull);
	CullLoci(lociToCull);
	}//end MultiformatMarkerData::CullLocus (int )


void MultiformatMarkerData::CullLocus (aString& locusToCull){
	StringList lociToCull;
	lociToCull.Append(NEW aString(locusToCull));
	CullLoci(lociToCull);
	}//end MultiformatMarkerData::CullLocus (aString& )


void MultiformatMarkerData::CullMonomorphicLoci (void){	
	if(codominantDataIncluded()){
		CodominantMarkerData::CullMonomorphicLoci();}
	if(codominantSexLinkedDataIncluded()){
		SexLinkedCodomMarkerData::CullMonomorphicLoci();}
	if(dominantDataIncluded()){
		DominantMarkerData2::CullMonomorphicLociDom();}
	if(dominantSexLinkedDataIncluded()){
		SexLinkedCodomMarkerData::CullMonomorphicLoci();}
	if(cytoplasmicFemaleMarkersIncluded()){
		CytoplasmicFemaleMarkerData::CullMonomorphicLoci();}
	if(cytoplasmicMaleMarkersIncluded()){
		CytoplasmicMaleMarkerData::CullMonomorphicLoci();}
	}//end procedure MultiformatMarkerData::CullMonomorphicLoci




void MultiformatMarkerData::CullToOneGroup (EvolutionaryGeneticsData& parentData,int groupNum){
	//makes a subset data structure from the specified group
	//individuals resampled
	EvolutionaryGeneticsData* parentDataPtr =  &parentData;
	unsigned long parentDataUSL = (unsigned long) parentDataPtr;
	MultiformatMarkerData* parentDataCodomPtr = (MultiformatMarkerData*) parentDataUSL;
	MultiformatMarkerData& parentDataC = *parentDataCodomPtr;

	Initialize(parentDataC,parentDataC.groupHierarchy_[groupNum]);

	CheckForNewAlleles();
	}//end MultiformatMarkerData::CullToOneGroup (int)



void MultiformatMarkerData::CullToOneSex (MultiformatMarkerData& parentData,genderSet sexVal){
	//makes a subset data structure from the specified group
	aList<int> sexValList;
	sexValList.Append((int)sexVal);
	CullToTheseSexes(parentData,sexValList);
	}//end MultiformatMarkerData::CullToOneSex (MultiformatMarkerData& , genderSet )


void MultiformatMarkerData::CullToTheseSexes (MultiformatMarkerData& parentData,aList<int>& sexValList){
	//makes a subset data structure from the specified group
	Clear();
	if(sexValList.len()==0) return;
	title_ = parentData.title_;
	title_.cat(" -- ");
	int s;
	for(s=0;s<sexValList.len();++s){
		genderSet sexVal = (genderSet) sexValList[s];
		switch(sexVal){
			case A:
				title_.cat("asexuals"); break;
			case B:
				title_.cat("hermaphrodites"); break;
			case F:
				title_.cat("females"); break;
			case M:
				title_.cat("males"); break;
			default: break;
			}
		if(s<sexValList.len()-2){
				title_.cat(", ");}
			else if (s == sexValList.len()-2){
				title_.cat(" & ");}
			else{}
		}
	title_.cat(" only");


	npop_ = parentData.npop_;
	ngroups_ = parentData.ngroups_;
	ntraits_ = parentData.ntraits_;
	groupName_ = parentData.groupName_;
	localityName_ = parentData.localityName_;
	traitName_ = parentData.traitName_;
	traitNum_ = parentData.traitNum_;
	skippedTraitNum_ = parentData.skippedTraitNum_;
	missing_ = parentData.missing_;
	popName_ = parentData.popName_;
	localityCoordinates_ = parentData.localityCoordinates_;
	missing_ = parentData.missing_;
	
	heterogameticSex_ = parentData.heterogameticSex_;
	sexTrait_ = parentData.sexTrait_;
	primarySexRatio_ = parentData.primarySexRatio_;
	
	nlociHap_ = parentData.nlociHap_;
	nlociCodom_ = parentData.nlociCodom_;
	nlociDom_ = parentData.nlociDom_;
	nlociSLCodom_ = parentData.nlociSLCodom_;
	nlociSLDom_ = parentData.nlociSLDom_;
	nlociCYfemale_ = parentData.nlociCYfemale_;
	nlociCYmale_ = parentData.nlociCYmale_;

	locusNumHap_ = parentData.locusNumHap_;
	locusNumCodom_ = parentData.locusNumCodom_;
	locusNumDom_ = parentData.locusNumDom_;
	locusNumSLCodom_ = parentData.locusNumSLCodom_;
	locusNumSLDom_ = parentData.locusNumSLDom_;
	locusNumCYfemale_ = parentData.locusNumCYfemale_;
	locusNumCYmale_ = parentData.locusNumCYmale_;

	allHapLoci_ = parentData.allHapLoci_;
	allCodomLoci_ = parentData.allCodomLoci_;
	allDomLoci_ = parentData.allDomLoci_;
	allSLCodomLoci_ = parentData.allSLCodomLoci_;
	allSLDomLoci_ = parentData.allSLDomLoci_;
	allCYfemaleLoci_ = parentData.allCYfemaleLoci_;
	allCYmaleLoci_ = parentData.allCYmaleLoci_;

	allPresentInFemalesOnly_ = parentData.allPresentInFemalesOnly_;
	presentInFemalesOnly_ = parentData.presentInFemalesOnly_;
	allPresentInMalesOnly_ = parentData.allPresentInMalesOnly_;
	presentInMalesOnly_ = parentData.presentInMalesOnly_;

	skippedLocusNumHap_ = parentData.skippedLocusNumHap_;
	skippedLocusNumCodom_ = parentData.skippedLocusNumCodom_;
	skippedLocusNumDom_ = parentData.skippedLocusNumDom_;
	skippedLocusNumSLCodom_ = parentData.skippedLocusNumSLCodom_;
	skippedLocusNumSLDom_ = parentData.skippedLocusNumSLDom_;
	skippedLocusNumCYfemale_ = parentData.skippedLocusNumCYfemale_;
	skippedLocusNumCYmale_ = parentData.skippedLocusNumCYmale_;

	traitTypesIncluded_ = parentData.traitTypesIncluded_;
	traitType_ = parentData.traitType_;
	traitTypeCurrent_ = parentData.traitTypeCurrent_;
	dummytextDatum_ = parentData.dummytextDatum_;
	dummytextDataList_ = parentData.dummytextDataList_;
	
	genderSet thisSexVal=B;
	int pop, i, thisSexValInt;
	for(pop=0;pop<parentData.npop_;++pop){
		popSize_.Append(0);
		rawTraits_.Append(NEW ObjList<TextDataList>());
		rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
		rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
		for(i=0;i<parentData.popSize(pop);++i){
			TextDatum& thisSex = parentData.rawTraits_[pop][i][sexTrait_];
			thisSexVal = genderCode(thisSex);
			thisSexValInt = (int)thisSexVal;
			if(sexValList.Includes(thisSexValInt)){
				rawTraits_[pop].Append((TextDataList*)parentData.rawTraits_[pop][i].Duplicate());
				rawCodomGtypes_[pop].Append(parentData.rawCodomGtypes_[pop][i].Duplicate());
				rawSLCodomGtypes_[pop].Append(parentData.rawSLCodomGtypes_[pop][i].Duplicate());
				popSize_[pop]++;}
			}//i
		}//pop
	
	//cull empty populations from data
	for(pop=npop_-1;pop>=0;--pop){
		if(parentData.rawTraits_[pop].len()==0){
			parentData.rawTraits_.Delete(pop);
			popName_.Delete(pop);
			popSize_.Delete(pop);
			}
		}//pop
	npop_ = rawTraits_.len();
		
	ResetFlags();
	GroupMembership();
	LocalityMembership();
	CheckForNewAlleles();
	}//end MultiformatMarkerData::CullToTheseSexes (MultiformatMarkerData& ,aList<int>& )




void MultiformatMarkerData::DetermineFemaleOnlyTraits(void){
	if(allCYfemaleLoci_.len() == 0) return;
	int tr, pop, i, loc;
	bool inMale;
	aList<int> presentInMales;
	for(loc=0;loc<locusNumCYfemale_.len();++loc){
		tr=locusNumCYfemale_[loc];
		inMale=false;
		for(pop=0;pop<npop_;++pop){
			for(i=0;i<individuals(pop);++i){
				if((rawTraits_[pop][i][sexTrait_]=='M' || rawTraits_[pop][i][sexTrait_]=='m') 
				   && ! (rawTraits_[pop][i][tr].missing()) ){
				   	inMale=true; break;}
				}//i
			if(inMale){
				presentInMales.Append(tr);
				break;}
			}//pop
		}//loc
	for(loc=0;loc<locusNumCYfemale_.len();++loc){
		tr=locusNumCYfemale_[loc];
		if(! presentInMales.Includes(tr)){
			traitType_[tr]=CYfemaleOnly;
			traitTypesIncluded_[CYfemaleOnly]=true;
			if(!presentInFemalesOnly_.Includes(tr)){
				presentInFemalesOnly_.Append(tr);}
			}
		}//loc
	presentInFemalesOnly_.Sort();
	}//MultiformatMarkerData::DetermineFemaleOnlyTraits()



void MultiformatMarkerData::DetermineMaleOnlyTraits(void){
	if(allCYmaleLoci_.len() == 0) return;
	int tr, pop, i, loc;
	bool inFemale;
	aList<int> presentInFemales;
	for(loc=0;loc<locusNumCYmale_.len();++loc){
		tr=locusNumCYmale_[loc];
		inFemale=false;
		for(pop=0;pop<npop_;++pop){
			for(i=0;i<individuals(pop);++i){
				if((rawTraits_[pop][i][sexTrait_]=='F' || rawTraits_[pop][i][sexTrait_]=='f') 
				   && ! (rawTraits_[pop][i][tr].missing()) ){
				   	inFemale=true; break;}
				}//i
			if(inFemale){
				presentInFemales.Append(tr);
				break;}
			}//pop
		}//loc
	for(loc=0;loc<locusNumCYmale_.len();++loc){
		tr=locusNumCYmale_[loc];
		if(! presentInFemales.Includes(tr)){
			traitType_[tr]=CYmaleOnly;
			traitTypesIncluded_[CYmaleOnly]=true;
			if(!presentInMalesOnly_.Includes(tr)){
				presentInMalesOnly_.Append(tr);}
			}
		}//loc
	presentInMalesOnly_.Sort();
	}//MultiformatMarkerData::DetermineMaleOnlyTraits()


TextDataList& MultiformatMarkerData::GenotypicVal(int pop, int indiv, int trait){
	//does not notice skipped loci
	//locus refers to trait num in the entire data set, not in the subset like in the derived classes
	int datasettype = traitType_[trait];
	switch(datasettype){
	  case codom://codominant
//	  		correctLoc=locusNumCodom_.Loc(loc);
//			return rawCodomGtypes_[pop][indiv][correctLoc];
			return CodominantMarkerData::GenotypicVal(pop,indiv,trait);
			break;
	  case SLcodom://sex-linked codominant
//	  		correctLoc=locusNumSLCodom_.Loc(loc);
//			return rawSLCodomGtypes_[pop][indiv][locusNumSLCodom_[loc]];
			return SexLinkedCodomMarkerData::GenotypicVal(pop,indiv,trait);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 aError("Error:  Couldn't find the proper Genotype in MultiformatMarkerData::GenotypicVal(int,int,int)");
	 return dummytextDataList_;
	}//end  MultiformatMarkerData::GenotypicVal(int,int,int)



TextDatum& MultiformatMarkerData::GenotypicVal(int pop, int indiv, int trait, int allele){
	//does not notice skipped loci
	//******locus refers to trait num in the entire data set, not in the subset like in the derived classes******
	int correctLoc,datasettype = traitType_[trait];
	switch(datasettype){
	  case hap://haploid
//	  		correctLoc=locusNumHap_.Loc(trait);
	  		correctLoc=allHapLoci_.Loc(trait);
			return rawTraits_[pop][indiv][correctLoc];
			break;
	  case codom://codominant
	  		if(locusNumCodom_.Includes(trait)){//ignores skipped
//	  			correctLoc=allCodomLoci_.Loc(trait);
//				return rawCodomGtypes_[pop][indiv][correctLoc][allele];
				return CodominantMarkerData::GenotypicVal(pop,indiv,trait,allele);
				}
			break;
	  case SLcodom://sex-linked codominant
	  		if(locusNumSLCodom_.Includes(trait)){//ignores skipped
//	  			correctLoc=allSLCodomLoci_.Loc(trait);
//				return rawSLCodomGtypes_[pop][indiv][correctLoc][allele];
				return SexLinkedCodomMarkerData::GenotypicVal(pop,indiv,trait,allele);
				}
			break;
	  case CYfemale:
	  case CYfemaleOnly:
//	  		correctLoc=locusNumCYfemale_.Loc(trait);
	  		correctLoc=allCYfemaleLoci_.Loc(trait);
			return rawTraits_[pop][indiv][correctLoc];
			break;
	  case CYmale:
	  case CYmaleOnly:
//	  		correctLoc=locusNumCYmale_.Loc(trait);
	  		correctLoc=allCYmaleLoci_.Loc(trait);
			return rawTraits_[pop][indiv][correctLoc];
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 aError("Error:  Couldn't find the proper Genotype in MultiformatMarkerData::GenotypicVal(int,int,int,int)");
	 return dummytextDatum_;
	}//end  MultiformatMarkerData::GenotypicVal(int,int,int,int)



int MultiformatMarkerData::GenotypeSize(int loc){
	int correctLoc, datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GenotypeSize ();
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GenotypeSize ();
			break;
	  case dom://dominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return DominantMarkerData2::GenotypeSize ();
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GenotypeSize ();
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GenotypeSize ();
			break;
	  case CYfemale://female cytoplasmic
	  case CYfemaleOnly://female cytoplasmic
	  		correctLoc=locusNumCYfemale_.Loc(loc);
			return CytoplasmicFemaleMarkerData::GenotypeSize ();
			break;
	  case CYmale://male cytoplasmic
	  case CYmaleOnly://male cytoplasmic
	  		correctLoc=locusNumCYmale_.Loc(loc);
			return CytoplasmicMaleMarkerData::GenotypeSize ();
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 aError("Error:  Couldn't find the proper value in MultiformatMarkerData::GenotypeSize()");
	 return (long double)0.0;
}



long double MultiformatMarkerData::GetExpectedHeterozygosity (int pop, int loc, int allele){
	int correctLoc, datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GetExpectedHeterozygosity (pop, correctLoc, allele);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetExpectedHeterozygosity (pop, correctLoc, allele);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetExpectedHeterozygosity (pop, correctLoc, allele);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GetExpectedHeterozygosity (pop, correctLoc, allele);
			break;
	  case CYfemale://female cytoplasmic
	  case CYfemaleOnly://female cytoplasmic
	  		correctLoc=locusNumCYfemale_.Loc(loc);
			return CytoplasmicFemaleMarkerData::GetExpectedHeterozygosity (pop, correctLoc, allele);
			break;
	  case CYmale://male cytoplasmic
	  case CYmaleOnly://male cytoplasmic
	  		correctLoc=locusNumCYmale_.Loc(loc);
			return CytoplasmicMaleMarkerData::GetExpectedHeterozygosity (pop, correctLoc, allele);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 aError("Error:  Couldn't find the proper heterozygosity in MultiformatMarkerData::GetExpectedHeterozygosity(int,int,int)");
	 return (long double)0.0;
	}//end MultiformatMarkerData::GetExpectedHeterozygosity(pop,loc,allele)



long double MultiformatMarkerData::GetExpectedHeterozygosity (int pop, int loc){
	int correctLoc, datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GetExpectedHeterozygosity (pop, correctLoc);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetExpectedHeterozygosity (pop, correctLoc);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetExpectedHeterozygosity (pop, correctLoc);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GetExpectedHeterozygosity (pop, correctLoc);
			break;
	  case CYfemale://female cytoplasmic
	  case CYfemaleOnly://female cytoplasmic
	  		correctLoc=locusNumCYfemale_.Loc(loc);
			return CytoplasmicFemaleMarkerData::GetExpectedHeterozygosity (pop, correctLoc);
			break;
	  case CYmale://male cytoplasmic
	  case CYmaleOnly://male cytoplasmic
	  		correctLoc=locusNumCYmale_.Loc(loc);
			return CytoplasmicMaleMarkerData::GetExpectedHeterozygosity (pop, correctLoc);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 aError("Error:  Couldn't find the proper heterozygosity in MultiformatMarkerData::GetExpectedHeterozygosity(int,int)");
	 return (long double)0.0;
	}//end MultiformatMarkerData::GetExpectedHeterozygosity(pop,loc)



long double MultiformatMarkerData::GetExpectedHeterozygosity (int pop){
	//returns mean Hexp over loci
	StatList<long double> hetExp;
	int loc,correctLoc,datasettype;
	for (loc=0;loc<ntraits_;++loc){
		datasettype = traitType_[loc];
		switch(datasettype){
		  case hap://haploid
		  		correctLoc=locusNumHap_.Loc(loc);
				hetExp.Append(HaploidMarkerData::GetExpectedHeterozygosity (pop, correctLoc));
				break;
		  case codom://codominant
		  		correctLoc=locusNumCodom_.Loc(loc);
				hetExp.Append(CodominantMarkerData::GetExpectedHeterozygosity (pop, correctLoc));
				break;
		  case SLcodom://sex-linked codominant
		  		correctLoc=locusNumSLCodom_.Loc(loc);
				hetExp.Append(SexLinkedCodomMarkerData::GetExpectedHeterozygosity (pop, correctLoc));
				break;
		  case SLdom://sex-linked dominant
		  		correctLoc=locusNumSLDom_.Loc(loc);
				hetExp.Append(SexLinkedDomMarkerData2::GetExpectedHeterozygosity (pop, correctLoc));
				break;
		  case CYfemale://female cytoplasmic
		  case CYfemaleOnly://female cytoplasmic
		  		correctLoc=locusNumCYfemale_.Loc(loc);
				hetExp.Append(CytoplasmicFemaleMarkerData::GetExpectedHeterozygosity (pop, correctLoc));
				break;
		  case CYmale://male cytoplasmic
		  case CYmaleOnly://male cytoplasmic
		  		correctLoc=locusNumCYmale_.Loc(loc);
				hetExp.Append(CytoplasmicMaleMarkerData::GetExpectedHeterozygosity (pop, correctLoc));
				break;
		  default://not in a sublist
		  		break;
		  }//switch
		}
	return hetExp.mean();
	}//end MultiformatMarkerData::GetExpectedHeterozygosity(pop)



long double MultiformatMarkerData::GetFreq (int loc, int allele){
	int correctLoc,datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GetFreqHap (correctLoc, allele);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetFreq (correctLoc, allele);
			break;
	  case dom://dominant
	  		correctLoc=locusNumDom_.Loc(loc);
			return DominantMarkerData2::GetFreqDom (correctLoc, allele);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetFreq (correctLoc, allele);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GetFreqDom (correctLoc, allele);
			break;
	  case CYfemale://female cytoplasmic
	  case CYfemaleOnly://female cytoplasmic
	  		correctLoc=locusNumCYfemale_.Loc(loc);
			return CytoplasmicFemaleMarkerData::GetFreqCYfemale (correctLoc, allele);
			break;
	  case CYmale://male cytoplasmic
	  case CYmaleOnly://male cytoplasmic
	  		correctLoc=locusNumCYmale_.Loc(loc);
			return CytoplasmicMaleMarkerData::GetFreqCYmale (correctLoc, allele);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 return (long double)0.0;
	}//end MultiformatMarkerData::GetFreq


long double MultiformatMarkerData::GetFreq (int pop, int loc, int allele){
	int correctLoc,datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GetFreqHap (pop, correctLoc, allele);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetFreq (pop, correctLoc, allele);
			break;
	  case dom://dominant
	  		correctLoc=locusNumDom_.Loc(loc);
			return DominantMarkerData2::GetFreqDom (pop, correctLoc, allele);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetFreq (pop, correctLoc, allele);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GetFreqDom (pop, correctLoc, allele);
			break;
	  case CYfemale://female cytoplasmic
	  case CYfemaleOnly://female cytoplasmic
	  		correctLoc=locusNumCYfemale_.Loc(loc);
			return CytoplasmicFemaleMarkerData::GetFreqCYfemale (pop, correctLoc, allele);
			break;
	  case CYmale://male cytoplasmic
	  case CYmaleOnly://male cytoplasmic
	  		correctLoc=locusNumCYmale_.Loc(loc);
			return CytoplasmicMaleMarkerData::GetFreqCYmale (pop, correctLoc, allele);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 return (long double)0.0;
	}//end MultiformatMarkerData::GetFreq


long double MultiformatMarkerData::GetFreqMostCommonAllele (int pop, int loc){
	int correctLoc,datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GetFreqMostCommonAllele (pop, correctLoc);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetFreqMostCommonAllele (pop, correctLoc);
			break;
	  case dom://dominant
	  		correctLoc=locusNumDom_.Loc(loc);
			return DominantMarkerData2::GetFreqMostCommonAllele (pop, correctLoc);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetFreqMostCommonAllele (pop, correctLoc);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GetFreqMostCommonAllele (pop, correctLoc);
			break;
	  case CYfemale://female cytoplasmic
	  case CYfemaleOnly://female cytoplasmic
	  		correctLoc=locusNumCYfemale_.Loc(loc);
			return CytoplasmicFemaleMarkerData::GetFreqMostCommonAllele (pop, correctLoc);
			break;
	  case CYmale://male cytoplasmic
	  case CYmaleOnly://male cytoplasmic
	  		correctLoc=locusNumCYmale_.Loc(loc);
			return CytoplasmicMaleMarkerData::GetFreqMostCommonAllele (pop, correctLoc);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 return (long double)0.0;
	}//end MultiformatMarkerData::GetFreqMostCommonAllele



void MultiformatMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
	//uploads data to pass to genetic analysis classes
	pgcList.Clear();
	int pop,i, dt;
	//initialize
	aList<int> lociByType;
	for(dt=0;dt<MultiInheritanceModeContainer<int>::inheritanceModes_;++dt){
		lociByType.Append(0);}
	lociByType[MultiInheritanceModeContainer<int>::haploid]=nlociHap_;
	lociByType[MultiInheritanceModeContainer<int>::diploid]=nlociCodom_;
	lociByType[MultiInheritanceModeContainer<int>::sexLink]=nlociSLCodom_;
	lociByType[MultiInheritanceModeContainer<int>::cytoFemale]=nlociCYfemale_;
	lociByType[MultiInheritanceModeContainer<int>::cytoMale]=nlociCYmale_;
	for(pop=0;pop<npop_;++pop){
		pgcList.Append(NEW PopulationGeneContainer<int>(popSize(pop),lociByType));}
	
	//fill up
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<popSize(pop);++i){
			GetGenome(pgcList[pop][i], pop, i, true);}
		}
	}//MultiformatMarkerData::GetGenome(ObjList<PopulationGeneContainer<int> >&)



void MultiformatMarkerData::GetGenome(GeneContainerCore& gc, int pop, int indiv, bool setID){
	//uploads data to MultiInheritanceModeContainer<int>
	MultiInheritanceModeContainer<int>* mimc = (MultiInheritanceModeContainer<int>*) &gc;
	mimc->Clear();
	if(setID){
		mimc->SetID(1+GetIndivNumInDataSequence(pop,indiv));}//ID's should not be 0-indexed
	if(haploidDataIncluded()){
		HaploidMarkerData::GetGenome(mimc->inheritanceSet_[MultiInheritanceModeContainer<int>::haploid],pop,indiv,false);}
	if(codominantDataIncluded()){
		CodominantMarkerData::GetGenome(mimc->inheritanceSet_[MultiInheritanceModeContainer<int>::diploid],pop,indiv,false);}
	if(sexLinkedDataIncluded()){
		SexLinkedData::GetGenome(gc,pop,indiv,false);
		mimc->SetHeterogameticSex(heterogameticSex());}
	if(cytoplasmicFemaleMarkersIncluded()){
		CytoplasmicFemaleMarkerData::GetGenome(mimc->inheritanceSet_[MultiInheritanceModeContainer<int>::cytoFemale],pop,indiv,false);}
	if(cytoplasmicMaleMarkersIncluded()){
		CytoplasmicMaleMarkerData::GetGenome(mimc->inheritanceSet_[MultiInheritanceModeContainer<int>::cytoMale],pop,indiv,false);}
	if(codominantSexLinkedDataIncluded()){
		SexLinkedCodomMarkerData::GetGenome(mimc->inheritanceSet_[MultiInheritanceModeContainer<int>::sexLink],pop,indiv,false);}
	}//MultiformatMarkerData::GetGenome(GeneContainerCore&, int, int)





/* see how this is used before debugging
aString* MultiformatMarkerData::GetGenotypeString(int pop, int indiv){
	StringList& genotypesList=gtypes[pop];
	aString& dataStr = genotypesList[indiv];
	aString *gtypeWord = NEW aString(dataStr);
	aString *codeWord = gtypeWord->ChopOffFirstWord();
	delete codeWord; codeWord=0;
	return gtypeWord;
	}//end MultiformatMarkerData::GetGenotypeString(pop,indiv)
*/



/*
aString* MultiformatMarkerData::GetGenotypeString(int pop, int indiv, int loc, int chrom){
//	aString *gtypeString=GetGenotypeString(pop, indiv, loc);
			//	aString *gTypeChrom = NEW aString(*gtypeString,chrom,chrom);
//	aString *gTypeChrom = NEW aString( (*gtypeString)[chrom]);
//	delete gtypeString;gtypeString=0;//this was created in GetGenotypeString(pop, indiv, loc);
//	return gTypeChrom;

	StringList gtypeString=Genotype(pop, indiv, loc);
	aString *gTypeChrom = NEW aString( gtypeString[chrom]);
	return gTypeChrom;
	}//end MultiformatMarkerData::GetGenotypeString(pop,indiv,loc,chrom)
*/

long MultiformatMarkerData::GetIndividualsSampled (int pop, int loc){
	int correctLoc, realLoc, datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GetIndividualsSampled (pop, correctLoc);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetIndividualsSampled (pop, correctLoc);
			break;
	  case dom://dominant
	  		correctLoc=locusNumDom_.Loc(loc);
			return DominantMarkerData2::GetIndividualsSampled (pop, correctLoc);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetIndividualsSampled (pop, correctLoc);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GetIndividualsSampled (pop, correctLoc);
			break;
	  case CYfemale://female cytoplasmic
	  		if(CytoplasmicFemaleMarkerData::presentInBothSexes_.len()>0){
	  		realLoc = CytoplasmicFemaleMarkerData::presentInBothSexes_[loc];
	  		if(realLoc>=0){
		  		correctLoc = CytoplasmicFemaleMarkerData::presentInBothSexes_.Loc(realLoc);
				return CytoplasmicFemaleMarkerData::GetIndividualsSampled (pop, correctLoc);}
				}
			break;
	  case CYfemaleOnly://female cytoplasmic
	  		if(presentInFemalesOnly_.len()>0){
		  		realLoc = CytoplasmicFemaleMarkerData::presentInFemalesOnly_[loc];
		  		if(realLoc>=0){
			  		correctLoc = CytoplasmicFemaleMarkerData::presentInFemalesOnly_.Loc(realLoc);
					return CytoplasmicFemaleMarkerData::GetIndividualsSampled (pop, correctLoc);}
				}
			break;
	  case CYmale://male cytoplasmic
	  		if(CytoplasmicMaleMarkerData::presentInBothSexes_.len()>0){
		  		realLoc = CytoplasmicMaleMarkerData::presentInBothSexes_[loc];
		  		if(realLoc>=0){
			  		correctLoc = CytoplasmicMaleMarkerData::presentInBothSexes_.Loc(realLoc);
					return CytoplasmicMaleMarkerData::GetIndividualsSampled (pop, correctLoc);}
				}
			break;
	  case CYmaleOnly://male cytoplasmic
	  		if(presentInMalesOnly_.len()>0){
		  		realLoc = CytoplasmicMaleMarkerData::presentInMalesOnly_[loc];
		  		if(realLoc>=0){
			  		correctLoc = CytoplasmicMaleMarkerData::presentInMalesOnly_.Loc(realLoc);
					return CytoplasmicMaleMarkerData::GetIndividualsSampled (pop, correctLoc);}
				}
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 return (long)0.0;
	}//end  MultiformatMarkerData::GetIndividualsSampled


long MultiformatMarkerData::GetIndividualsSampled (int traitType, int pop, int loc){//locus within trait type
	int correctLoc;//, realLoc;
	switch(traitType){
	  case hap://haploid
			return HaploidMarkerData::GetIndividualsSampled (pop, loc);
			break;
	  case codom://codominant
			return CodominantMarkerData::GetIndividualsSampled (pop, loc);
			break;
	  case dom://dominant
			return DominantMarkerData2::GetIndividualsSampled (pop, loc);
			break;
	  case SLcodom://sex-linked codominant
			return SexLinkedCodomMarkerData::GetIndividualsSampled (pop, loc);
			break;
	  case SLdom://sex-linked dominant
			return SexLinkedDomMarkerData2::GetIndividualsSampled (pop, loc);
			break;
	  case CYfemale://female cytoplasmic
	  		if(CytoplasmicFemaleMarkerData::presentInBothSexes_.len()>0){
//		  		realLoc = CytoplasmicFemaleMarkerData::presentInBothSexes_[loc];
//		  		if(realLoc>=0){
//			  		correctLoc = CytoplasmicFemaleMarkerData::presentInBothSexes_.Loc(loc);
			  		correctLoc = CytoplasmicFemaleMarkerData::presentInBothSexes_[loc];
					return CytoplasmicFemaleMarkerData::GetIndividualsSampled (pop, correctLoc);
//					}
				}
			break;
	  case CYfemaleOnly://female cytoplasmic
	  		if(presentInFemalesOnly_.len()>0){
//		  		realLoc = CytoplasmicFemaleMarkerData::presentInFemalesOnly_[loc];
//		  		if(realLoc>=0){
			  		correctLoc = CytoplasmicFemaleMarkerData::presentInFemalesOnly_[loc];
					return CytoplasmicFemaleMarkerData::GetIndividualsSampled (pop, correctLoc);
//					}
				}
			break;
	  case CYmale://male cytoplasmic
	  		if(CytoplasmicMaleMarkerData::presentInBothSexes_.len()>0){
//		  		realLoc = CytoplasmicMaleMarkerData::presentInBothSexes_[loc];
//		  		if(realLoc>=0){
//			  		correctLoc = CytoplasmicMaleMarkerData::presentInBothSexes_.Loc(loc);
			  		correctLoc = CytoplasmicMaleMarkerData::presentInBothSexes_[loc];
					return CytoplasmicMaleMarkerData::GetIndividualsSampled (pop, correctLoc);
//					}
				}
			break;
	  case CYmaleOnly://male cytoplasmic
	  		if(presentInMalesOnly_.len()>0){
//		  		realLoc = CytoplasmicMaleMarkerData::presentInMalesOnly_[loc];
//		  		if(realLoc>=0){
//			  		correctLoc = CytoplasmicMaleMarkerData::presentInMalesOnly_.Loc(loc);
			  		correctLoc = CytoplasmicMaleMarkerData::presentInMalesOnly_[loc];
					return CytoplasmicMaleMarkerData::GetIndividualsSampled (pop, correctLoc);
//					}
				}
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 return (long)0.0;
	}//end  MultiformatMarkerData::GetIndividualsSampled


long MultiformatMarkerData::GetLocusSampleSize (int loc){
	int correctLoc,realLoc, datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GetLocusSampleSizeHap (correctLoc);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetLocusSampleSize (correctLoc);
			break;
	  case dom://dominant
	  		correctLoc=locusNumDom_.Loc(loc);
			return DominantMarkerData2::GetLocusSampleSizeDom (correctLoc);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetLocusSampleSize (correctLoc);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GetLocusSampleSizeDom (correctLoc);
			break;
	  case CYfemale://female cytoplasmic
	  		if(CytoplasmicFemaleMarkerData::presentInBothSexes_.len()>0){
		  		realLoc = CytoplasmicFemaleMarkerData::presentInBothSexes_[loc];
		  		if(realLoc>=0){
			  		correctLoc = CytoplasmicFemaleMarkerData::presentInBothSexes_.Loc(realLoc);
					return CytoplasmicFemaleMarkerData::GetLocusSampleSizeCYfemale (correctLoc);}
				}
			break;
	  case CYfemaleOnly://female cytoplasmic
	  		if(presentInFemalesOnly_.len()>0){
		  		realLoc = CytoplasmicFemaleMarkerData::presentInFemalesOnly_[loc];
		  		if(realLoc>=0){
			  		correctLoc = CytoplasmicFemaleMarkerData::presentInFemalesOnly_.Loc(realLoc);
					return CytoplasmicFemaleMarkerData::GetLocusSampleSizeCYfemale (correctLoc);}
				}
			break;
	  case CYmale://male cytoplasmic
	  		if(CytoplasmicMaleMarkerData::presentInBothSexes_.len()>0){
		  		realLoc = CytoplasmicMaleMarkerData::presentInBothSexes_[loc];
		  		if(realLoc>=0){
			  		correctLoc = CytoplasmicMaleMarkerData::presentInBothSexes_.Loc(realLoc);
					return CytoplasmicMaleMarkerData::GetLocusSampleSizeCYmale (correctLoc);}
				}
			break;
	  case CYmaleOnly://male cytoplasmic
	  		if(presentInMalesOnly_.len()>0){
		  		realLoc = CytoplasmicMaleMarkerData::presentInMalesOnly_[loc];
		  		if(realLoc>=0){
			  		correctLoc = CytoplasmicMaleMarkerData::presentInMalesOnly_.Loc(realLoc);
					return CytoplasmicMaleMarkerData::GetLocusSampleSizeCYmale (correctLoc);}
				}
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 return (long)0.0;
	}//end  MultiformatMarkerData::GetLocusSampleSize


long MultiformatMarkerData::GetLocusSampleSize (int pop, int loc){
	int correctLoc, datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GetLocusSampleSizeHap (pop, correctLoc);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetLocusSampleSize (pop, correctLoc);
			break;
	  case dom://dominant
	  		correctLoc=locusNumDom_.Loc(loc);
			return DominantMarkerData2::GetLocusSampleSizeDom (pop, correctLoc);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetLocusSampleSize (pop, correctLoc);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GetLocusSampleSizeDom (pop, correctLoc);
			break;
	  case CYfemale://female cytoplasmic
	  case CYfemaleOnly://female cytoplasmic
	  		correctLoc=locusNumCYfemale_.Loc(loc);
			return CytoplasmicFemaleMarkerData::GetLocusSampleSizeCYfemale (pop, correctLoc);
			break;
	  case CYmale://male cytoplasmic
	  case CYmaleOnly://male cytoplasmic
	  		correctLoc=locusNumCYmale_.Loc(loc);
			return CytoplasmicMaleMarkerData::GetLocusSampleSizeCYmale (pop, correctLoc);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 return (long)0.0;
	}//end  MultiformatMarkerData::GetLocusSampleSize


int MultiformatMarkerData::GetLocusSampleSize (int pop, int indiv, int loc){
	int correctLoc,datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GetLocusSampleSizeHap (pop, indiv, correctLoc);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetLocusSampleSize (pop, indiv, correctLoc);
			break;
	  case dom://dominant
	  		correctLoc=locusNumDom_.Loc(loc);
			return DominantMarkerData2::GetLocusSampleSizeDom (pop, indiv, correctLoc);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetLocusSampleSize (pop, indiv, correctLoc);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GetLocusSampleSizeDom (pop, indiv, correctLoc);
			break;
	  case CYfemale://female cytoplasmic
	  case CYfemaleOnly://female cytoplasmic
	  		correctLoc=locusNumCYfemale_.Loc(loc);
			return CytoplasmicFemaleMarkerData::GetLocusSampleSizeCYfemale (pop, indiv, correctLoc);
			break;
	  case CYmale://male cytoplasmic
	  case CYmaleOnly://male cytoplasmic
	  		correctLoc=locusNumCYmale_.Loc(loc);
			return CytoplasmicMaleMarkerData::GetLocusSampleSizeCYmale (pop, indiv, correctLoc);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 return (long)0.0;
	}//end  MultiformatMarkerData::GetLocusSampleSize



/*
long double MultiformatMarkerData::GetMeanAllelesPerLocus (int pop){
	int correctLoc,datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GetMeanAllelesPerLocus (pop);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetMeanAllelesPerLocus (pop);
			break;
	  case dom://dominant
	  		correctLoc=locusNumDom_.Loc(loc);
			return DominantMarkerData2::GetMeanAllelesPerLocus (pop);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetMeanAllelesPerLocus (pop);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GetMeanAllelesPerLocus (pop);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 return (long double)0.0;
	}//end  MultiformatMarkerData::GetMeanAllelesPerLocus
*/

int MultiformatMarkerData::GetNumAlleles (int loc){
	//total in data set
	int correctLoc,datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GetNumAllelesHap (loc);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetNumAlleles (loc);
			break;
	  case dom://dominant
	  		correctLoc=locusNumDom_.Loc(loc);
			return DominantMarkerData2::GetNumAllelesDom (loc);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetNumAlleles (loc);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GetNumAllelesDom (loc);
			break;
	  case CYfemale://female cytoplasmic
	  case CYfemaleOnly://female cytoplasmic
	  		correctLoc=locusNumCYfemale_.Loc(loc);
			return CytoplasmicFemaleMarkerData::GetNumAllelesCYfemale (loc);
			break;
	  case CYmale://male cytoplasmic
	  case CYmaleOnly://male cytoplasmic
	  		correctLoc=locusNumCYmale_.Loc(loc);
			return CytoplasmicMaleMarkerData::GetNumAllelesCYmale (loc);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
//	 return (int)0.0;
	 return 0;
	}//end  MultiformatMarkerData::GetNumAlleles(loc)


int MultiformatMarkerData::GetNumAlleles (int pop, int loc){
	//extant alleles in population
	int correctLoc,datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::GetNumAllelesHap (pop, loc);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetNumAlleles (pop, loc);
			break;
	  case dom://dominant
	  		correctLoc=locusNumDom_.Loc(loc);
			return DominantMarkerData2::GetNumAllelesDom (pop, loc);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetNumAlleles (pop, loc);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::GetNumAllelesDom (pop, loc);
			break;
	  case CYfemale://female cytoplasmic
	  case CYfemaleOnly://female cytoplasmic
	  		correctLoc=locusNumCYfemale_.Loc(loc);
			return CytoplasmicFemaleMarkerData::GetNumAllelesCYfemale (pop, loc);
			break;
	  case CYmale://male cytoplasmic
	  case CYmaleOnly://male cytoplasmic
	  		correctLoc=locusNumCYmale_.Loc(loc);
			return CytoplasmicMaleMarkerData::GetNumAllelesCYmale (pop, loc);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
//	 return (int)0.0;
	 return 0;
	}//end  MultiformatMarkerData::GetNumAlleles(pop,loc)


long double MultiformatMarkerData::GetObservedHeterozygosity(int pop, int loc, int allele){
	int correctLoc,datasettype = traitType_[loc];
	switch(datasettype){
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetObservedHeterozygosity (pop, correctLoc, allele);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetObservedHeterozygosity (pop, correctLoc, allele);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 aError("Error:  Couldn't find the proper heterozygosity in MultiformatMarkerData::GetObservedHeterozygosity(int,int)");
//	 return (long double)0.0;
	 return zero;
	}//end MultiformatMarkerData::GetObservedHeterozygosity(pop,loc,allele)


long double MultiformatMarkerData::GetObservedHeterozygosity(int pop, int loc){
	int correctLoc,datasettype = traitType_[loc];
	switch(datasettype){
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::GetObservedHeterozygosity (pop, correctLoc);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::GetObservedHeterozygosity (pop, correctLoc);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 aError("Error:  Couldn't find the proper heterozygosity in MultiformatMarkerData::GetObservededHeterozygosity(int,int)");
	 return zero;
	}//end MultiformatMarkerData::GetObservedHeterozygosity(pop,loc)


long double MultiformatMarkerData::GetObservedHeterozygosity (int pop){
	//returns mean Hobs over loci
	StatList<long double> hetObs;
	int loc,correctLoc,datasettype;
	for (loc=0;loc<ntraits_;++loc){
		datasettype = traitType_[loc];
		switch(datasettype){
		  case codom://codominant
		  		correctLoc=locusNumCodom_.Loc(loc);
				hetObs.Append(CodominantMarkerData::GetExpectedHeterozygosity (pop, correctLoc));
				break;
		  case SLcodom://sex-linked codominant
		  		correctLoc=locusNumSLCodom_.Loc(loc);
				hetObs.Append(SexLinkedCodomMarkerData::GetExpectedHeterozygosity (pop, correctLoc));
				break;
		  default://not in a sublist
		  		break;
		  }//switch
		}
	return hetObs.mean();
	}//end MultiformatMarkerData::GetObservedHeterozygosity(pop)



long double MultiformatMarkerData::GetPercentPolymorphicLoci(int pop, double criterion){
	//default criterion is 100.0% = any polymorphism
	long double percentP = zero;
	criterion /= 100.0; 
	int loc;
	for(loc=0;loc<nlociCodom_;++loc){
		long double mostCommon = GetFreqMostCommonAllele(pop,loc);
		if(mostCommon<(long double)criterion){++percentP;}}
	return (percentP/(long double)nlociCodom_) *(long double)100.0;
	}//end MultiformatMarkerData::GetPercentPolymorphicLoci(pop,loc)



aString*	MultiformatMarkerData::GetTraitName(int traitType, int trNumWithinType){
	//ignores skipped traits
	if(trNumWithinType<0){return NEW aString();}
	switch(traitType){
		case ID:
			return NEW aString();
			break;
		case sex:
			return NEW aString(traitName_[sexTrait_]);
			break;
		case hap:
			if(trNumWithinType<locusNumHap_.len()){
				return NEW aString(traitName_[HaploidMarkerData::locusNumHap_[trNumWithinType]]);};
			break;
		case codom:
			if(trNumWithinType<locusNumCodom_.len()){
				return NEW aString(traitName_[CodominantMarkerData::locusNumCodom_[trNumWithinType]]);};
			break;
		case dom:
			if(trNumWithinType<locusNumDom_.len()){
				return NEW aString(traitName_[DominantMarkerData2::locusNumDom_[trNumWithinType]]);};
			break;
		case SLcodom:
			if(trNumWithinType<locusNumSLCodom_.len()){
				return NEW aString(traitName_[SexLinkedCodomMarkerData::locusNumSLCodom_[trNumWithinType]]);};
			break;
		case SLdom:
			if(trNumWithinType<locusNumSLDom_.len()){
				return NEW aString(traitName_[SexLinkedDomMarkerData2::locusNumSLDom_[trNumWithinType]]);};
			break;
		case CYfemale:
		case CYfemaleOnly:
			if(trNumWithinType<locusNumCYfemale_.len()){
				return NEW aString(traitName_[CytoplasmicFemaleMarkerData::locusNumCYfemale_[trNumWithinType]]);};
			break;
		case CYmale:
		case CYmaleOnly:
			if(trNumWithinType<locusNumCYmale_.len()){
				return NEW aString(traitName_[CytoplasmicMaleMarkerData::locusNumCYmale_[trNumWithinType]]);};
			break;
		default:
		break;
		}
	return NEW aString();//dummy value for missing trait
	
	}//end MultiformatMarkerData::GetTraitName(traitType, trNumWithinType)
					


int	MultiformatMarkerData::GetTraitNum(int traitType, int trNumWithinType){
	//ignores skipped traits
	if(trNumWithinType<0){return -1;}
	switch(traitType){
		case ID:
			return 0;
			break;
		case sex:
			return sexTrait_;
			break;
		case hap:
			if(trNumWithinType<locusNumHap_.len()){
				return HaploidMarkerData::locusNumHap_[trNumWithinType];};
			break;
		case codom:
			if(trNumWithinType<locusNumCodom_.len()){
				return CodominantMarkerData::locusNumCodom_[trNumWithinType];};
			break;
		case dom:
			if(trNumWithinType<locusNumDom_.len()){
				return DominantMarkerData2::locusNumDom_[trNumWithinType];};
			break;
		case SLcodom:
			if(trNumWithinType<locusNumSLCodom_.len()){
				return SexLinkedCodomMarkerData::locusNumSLCodom_[trNumWithinType];}
			break;
		case SLdom:
			if(trNumWithinType<locusNumSLDom_.len()){
				return SexLinkedDomMarkerData2::locusNumSLDom_[trNumWithinType];};
			break;
		case CYfemale:
			if(trNumWithinType<CytoplasmicFemaleMarkerData::presentInBothSexes_.len()){
				return CytoplasmicFemaleMarkerData::presentInBothSexes_[trNumWithinType];}
			break;
		case CYfemaleOnly:
			if(trNumWithinType<CytoplasmicFemaleMarkerData::presentInFemalesOnly_.len()){
				return CytoplasmicFemaleMarkerData::presentInFemalesOnly_[trNumWithinType];}
			break;
		case CYmale:
			if(trNumWithinType<CytoplasmicMaleMarkerData::presentInBothSexes_.len()){
				return CytoplasmicMaleMarkerData::presentInBothSexes_[trNumWithinType];}
			break;
		case CYmaleOnly:
			if(trNumWithinType<CytoplasmicMaleMarkerData::presentInMalesOnly_.len()){
				return CytoplasmicMaleMarkerData::presentInMalesOnly_[trNumWithinType];}
			break;
		default:
		break;
		}//switch
	return -1;//dummy value for missing trait
	
	}//end MultiformatMarkerData::GetTraitNumGlobal(traitType, trNumWithinType)
					


aString* MultiformatMarkerData::GetTraitString(int pop, int indiv, int trait, int allele){
	//find the right loc and call the subclass routine
	int locInList=0;
	switch(traitType_[trait]){
		case ID:
			return NEW aString(rawTraits_[pop][indiv][0]);
			break;
		case sex:
			return NEW aString(rawTraits_[pop][indiv][sexTrait_]);
			break;
		case hap:
			locInList = locusNumHap_.Loc(trait);
			return HaploidMarkerData::GetTraitString(pop,indiv,locInList);
			break;
		case codom:
			locInList = locusNumCodom_.Loc(trait);
			return CodominantMarkerData::GetTraitString(pop,indiv,locInList, allele);
			break;
		case dom:
			locInList = locusNumDom_.Loc(trait);
			return DominantMarkerData2::GetTraitString(pop,indiv,locInList);
			break;
		case SLcodom:
			locInList = locusNumSLCodom_.Loc(trait);
			return SexLinkedCodomMarkerData::GetTraitString(pop,indiv,locInList,allele);
			break;
		case SLdom:
			locInList = locusNumSLDom_.Loc(trait);
			return SexLinkedDomMarkerData2::GetTraitString(pop,indiv,locInList);
			break;
		case CYfemale:
		case CYfemaleOnly:
			locInList = locusNumCYfemale_.Loc(trait);
			return CytoplasmicFemaleMarkerData::GetTraitString(pop,indiv,locInList);
			break;
		case CYmale:
		case CYmaleOnly:
			locInList = locusNumCYmale_.Loc(trait);
			return CytoplasmicMaleMarkerData::GetTraitString(pop,indiv,locInList);
			break;
		default:
		break;
		}
	return NEW aString();//empty
	}//end MultiformatMarkerData::GetTraitString(pop,indiv,loc)


/*
long double MultiformatMarkerData::GetVarJackAllelesPerLocus (int pop){
	//returns jackknife variance of means, dropping loci
	StatList<long double> allelesPerLoc;
	for (int loc=0;loc<nlociCodom_;++loc){
		allelesPerLoc.Append((long double)GetNumAlleles(pop,loc));}
	StatList<long double> jackAllelesPerLoc;
	for (int loc=0;loc<nlociCodom_;++loc){
		long double storeValue = allelesPerLoc.First();
		allelesPerLoc.DeleteFirst();
		jackAllelesPerLoc.Append(allelesPerLoc.mean());
		allelesPerLoc.Append(storeValue);
		}
	return jackAllelesPerLoc.var();
	}//end MultiformatMarkerData::GetVarJackAllelesPerLocus(pop)



long double MultiformatMarkerData::GetVarJackExpectedHeterozygosity (int pop){
	//returns jackknife variance of means, dropping loci
	StatList<long double> hetExpLoc;
	for (int loc=0;loc<nlociCodom_;++loc){
		hetExpLoc.Append(GetExpectedHeterozygosity(pop, loc));}
	StatList<long double> jackHetExpLoc;
	for (int loc=0;loc<nlociCodom_;++loc){
		long double storeValue = hetExpLoc.First();
		hetExpLoc.DeleteFirst();
		jackHetExpLoc.Append(hetExpLoc.mean());
		hetExpLoc.Append(storeValue);
		}
	return jackHetExpLoc.var();
	}//end MultiformatMarkerData::GetVarJackExpectedHeterozygosity(pop)



long double MultiformatMarkerData::GetVarJackObservedHeterozygosity (int pop){
	//returns jackknife variance of means, dropping loci
	StatList<long double> hetObsLoc;
	for (int loc=0;loc<nlociCodom_;++loc){
		hetObsLoc.Append(GetObservedHeterozygosity(pop, loc));}
	StatList<long double> jackHetObsLoc;
	for (int loc=0;loc<nlociCodom_;++loc){
		long double storeValue = hetObsLoc.First();
		hetObsLoc.DeleteFirst();
		jackHetObsLoc.Append(hetObsLoc.mean());
		hetObsLoc.Append(storeValue);
		}
	return jackHetObsLoc.var();
	}//end MultiformatMarkerData::GetVarJackObservedHeterozygosity(pop)



long double MultiformatMarkerData::GetVarJackPercentPolyLoci (int pop, double criterion){
	//returns jackknife variance of means, dropping loci
	StatList<long double> jackPercentP;
	for (int loc=0;loc<nlociCodom_;++loc){
		jackPercentP.Append(GetPercentPolymorphicLoci(pop,criterion));}
	for (int loc=0;loc<nlociCodom_;++loc){
		double long temp = (jackPercentP[loc]/100.0)*nlociCodom_;
		if(IsPolymorphic(pop,loc,criterion)){--temp;}
		jackPercentP.Replace((temp/(long double)(nlociCodom_-1))*(long double)100.0,loc);
		}
	return jackPercentP.var();
	}//end MultiformatMarkerData::GetVarJackPercentPolyLoci(pop,criterion)
*/



int MultiformatMarkerData::individuals(int pop, int loc){
	int indivs=0;
	int i;
	for(i=0;i<individuals(pop);++i){
		if (GetLocusSampleSize ( pop, i, loc)>0){++indivs;}}
	return indivs;
	}//end  MultiformatMarkerData::individuals(int,int)
	


int  MultiformatMarkerData::IsPolymorphic(int pop, int loc){//boolean
	long double freq = GetFreqMostCommonAllele(pop,loc);
	if (freq==zero){return 0;}
	if (freq<one){return 1;}
	return 0;
	}// end MultiformatMarkerData::IsPolymorphic


int  MultiformatMarkerData::IsPolymorphic(int pop, int loc, double criterion){//boolean
	long double freq = GetFreqMostCommonAllele(pop,loc);
	if (freq==zero){return 0;}
	if (freq<(criterion/100.0)){return 1;}
	return 0;
	}// end MultiformatMarkerData::IsPolymorphic



int MultiformatMarkerData::loci(void){// unskipped loci
	int locsum = HaploidMarkerData::loci();
	locsum += CodominantMarkerData::loci();
	locsum += DominantMarkerData2::loci();
	locsum += SexLinkedCodomMarkerData::loci();
	locsum += SexLinkedDomMarkerData2::loci();
	locsum += CytoplasmicFemaleMarkerData::loci();
	locsum += CytoplasmicMaleMarkerData::loci();	
	return locsum;
	}//MultiformatMarkerData::loci()


int MultiformatMarkerData::loci(int traitType){// unskipped loci
	switch(traitType){
		case hap:  
			return HaploidMarkerData::loci(); break;
		case codom:  
			return CodominantMarkerData::loci(); break;
		case dom:  
			return DominantMarkerData2::loci(); break;
		case SLcodom: 
			return SexLinkedCodomMarkerData::loci(); break;
		case SLdom: 
			return SexLinkedDomMarkerData2::loci(); break;
		case CYfemale: 
			return CytoplasmicFemaleMarkerData::presentInBothSexes_.len(); break;
		case CYfemaleOnly: 
			return CytoplasmicFemaleMarkerData::presentInFemalesOnly_.len(); break;
		case CYmale: 
			return CytoplasmicMaleMarkerData::presentInBothSexes_.len(); break;
		case CYmaleOnly: 
			return CytoplasmicMaleMarkerData::presentInMalesOnly_.len(); break;
		default: break;
		}//switch
	return 0;
	}//MultiformatMarkerData::loci(int)



int MultiformatMarkerData::lociSkipped(void){
	int locsum = HaploidMarkerData::lociSkipped();
	locsum += CodominantMarkerData::lociSkipped();
	locsum += DominantMarkerData2::lociSkipped();
	locsum += SexLinkedCodomMarkerData::lociSkipped();
	locsum += SexLinkedDomMarkerData2::lociSkipped();
	locsum += CytoplasmicFemaleMarkerData::lociSkipped();
	locsum += CytoplasmicMaleMarkerData::lociSkipped();	
	return locsum;
	}



aString& MultiformatMarkerData::locusName(int loc){
	int locAdj=loc, nloc = HaploidMarkerData::loci();
	if(locAdj<nloc){
		return traitName_[locusNumHap_[locAdj]];}
		else{
			locAdj -= nloc;}
	nloc = CodominantMarkerData::loci();
	if(locAdj<nloc){
		return traitName_[locusNumCodom_[locAdj]];}
		else{
			locAdj -= nloc;}
	nloc = DominantMarkerData2::loci();
	if(locAdj<nloc){
		return traitName_[locusNumDom_[locAdj]];}
		else{
			locAdj -= nloc;}
	nloc = SexLinkedCodomMarkerData::loci();
	if(locAdj<nloc){
		return traitName_[locusNumSLCodom_[locAdj]];}
		else{
			locAdj -= nloc;}
	nloc = SexLinkedDomMarkerData2::loci();
	if(locAdj<nloc){
		return traitName_[locusNumSLDom_[locAdj]];}
		else{
			locAdj -= nloc;}
	nloc = CytoplasmicFemaleMarkerData::loci();
	if(locAdj<nloc){
		return traitName_[locusNumCYfemale_[locAdj]];}
		else{
			locAdj -= nloc;}
	nloc = CytoplasmicMaleMarkerData::loci();
	if(locAdj<nloc){
		return traitName_[locusNumCYmale_[locAdj]];}
	aString msg("Error: tried to access the name of locus #");
	msg.cat(loc+1); msg.cat(" when there are only ");
	msg.cat(loci()+1); msg.cat(" loci in the data set");
	aError(msg);
	return traitName_[0];//default
	}


long MultiformatMarkerData::maxall(void){
	//maximum alleles per locus in data set
	ParseGenotypicData();
	CheckForNewAlleles();
	long maxalleles=0;
	maxalleles = MAX(maxalleles,HaploidMarkerData::maxall());
	maxalleles = MAX(maxalleles,CodominantMarkerData::maxall());
	maxalleles = MAX(maxalleles,DominantMarkerData2::maxall());
	maxalleles = MAX(maxalleles,SexLinkedCodomMarkerData::maxall());
	maxalleles = MAX(maxalleles,SexLinkedDomMarkerData2::maxall());
	maxalleles = MAX(maxalleles,CytoplasmicFemaleMarkerData::maxall());
	maxalleles = MAX(maxalleles,CytoplasmicMaleMarkerData::maxall());
	return maxalleles;
	}//MultiformatMarkerData::maxall()




void MultiformatMarkerData::MergeDataSets(MultiformatMarkerData& dataToAppend){
	//sorts by population name and ID number; omits skipped loci
	MultiformatMarkerData temp(*this);//save a copy to work with
	aString merger("Merged data:  (1) ");
	merger += title_;
	merger += "; (2) ";
	merger += dataToAppend.title_;
	title_ = merger;
	//check for unique populations & loci
	int pop,pop2,pop2loc=0,i1,i2,i1loc,i2loc,tr,trt;
	aList<int> popsOnlyInList2, traitsOnlyInList2;//holds location in list2
	ObjList<aList<int> > indivsOnlyInList1;
	 
	for(pop=0;pop<dataToAppend.npop_;++pop){
		if(! popName_.Includes(&(dataToAppend.popName_[pop]))){
			popsOnlyInList2.Append(pop);}
		indivsOnlyInList1.Append(NEW aList<int>());
		for(i1=0;i1<individuals(pop);++i1){ 
			indivsOnlyInList1[pop].Append(i1);}
		for(i2=0;i2<dataToAppend.individuals(pop);++i2){ 
			TextDatum& idum2 = dataToAppend.rawTraits_[pop][i2][0];
			//match this ID in pop2
			i2loc= -1;
			for(i1=0;i1<indivsOnlyInList1[pop].len();++i1){
				TextDatum& idum1 =  rawTraits_[pop2loc][indivsOnlyInList1[pop][i1]][0];
				if (idum2 == idum1){
					i1loc = indivsOnlyInList1[pop][i1];
					indivsOnlyInList1[pop].Delete(i1);
					break;}}
			}//i2
		}//pop

	for(tr=0;tr<dataToAppend.traitName_.len();++tr){
		if(! traitName_.Includes(&(dataToAppend.traitName_[tr]))){
			traitsOnlyInList2.Append(tr);}}
	//enlarge the data bases for traits
	int totalTraits = ntraits_ + traitsOnlyInList2.len();
	
	
	for(pop=0;pop<npop_;++pop){//do list1 pops first
		pop2loc=-1;
		for(pop2=0;pop2<dataToAppend.npop_;++pop2){
			if(popName_[pop] == dataToAppend.popName_[pop2]){
				pop2loc=pop2;break;}}
		if(pop2loc==-1) continue;//not in list flag
		aList<int> pop2indivsRemainingToCheck;
		for(i2=0;i2<dataToAppend.individuals(pop2loc);++i2){
			pop2indivsRemainingToCheck.Append(i2);}
		for(i1=0;i1<individuals(pop);++i1){ 
			TextDatum& ID1 = rawTraits_[pop][i1][0];
			//match this ID in pop2
			i2loc= -1;
			for(i2=0;i2<pop2indivsRemainingToCheck.len();++i2){
				TextDatum& ID2 =  dataToAppend.rawTraits_[pop2loc][pop2indivsRemainingToCheck[i2]][0];
				if (ID1 == ID2){
					i2loc = pop2indivsRemainingToCheck[i2];
					pop2indivsRemainingToCheck.Delete(i2);
					break;}}
			if(i2loc == -1) continue;//skips to next i1
			
			//merge this individual
			int traitToGet;
			if(indivsOnlyInList1[pop].Includes(i1)){
					for(tr=0;tr<traitsOnlyInList2.len();++tr){//fill trait 2 slots with empty data
						traitToGet = traitsOnlyInList2[tr];
						TextDatum newData(dataToAppend.rawTraits_[pop2loc][0][traitToGet]);
						newData.clear();
						rawTraits_[pop][i1].Append(NEW TextDatum(newData));
						}//tr
					}
				else{
					for(tr=0;tr<traitsOnlyInList2.len();++tr){
						traitToGet = traitsOnlyInList2[tr];
						TextDatum& traitSetToAppend = dataToAppend.rawTraits_[pop2loc][i2loc][traitToGet];
						rawTraits_[pop][i1].Append(NEW TextDatum(traitSetToAppend));
						}//tr
					}
			}//i1
		for(i2=0;i2<pop2indivsRemainingToCheck.len();++i2){
			//tack on these individuals
			rawTraits_[pop].Append(NEW TextDataList());
			rawTraits_[pop].Last().Append(NEW TextDatum(dataToAppend.rawTraits_[pop2loc][i2loc][0]));//ID
			for(tr=1;tr<temp.traitName_.len();++tr){//empty list for most traits
				if(tr==sexTrait_){
						int sxtr2 = dataToAppend.sexTrait_;
						rawTraits_[pop].Last().Append(NEW TextDatum(dataToAppend.rawTraits_[pop2loc][i2loc][sxtr2]));}
					else{
						rawTraits_[pop].Last().Append(NEW TextDatum());}
				}//tr
			for(tr=0;tr<traitsOnlyInList2.len();++tr){
				int traitToGet2 = traitsOnlyInList2[tr];
				TextDatum& traitSetToAppend2 = dataToAppend.rawTraits_[pop2loc][i2loc][traitToGet2];
				rawTraits_[pop].Last().Append(NEW TextDatum(traitSetToAppend2));
				}//tr
			}//i2
		}//pop
	//now update the trait type info
	for(tr=0;tr<traitsOnlyInList2.len();++tr){
		traitType_.Append(dataToAppend.traitType_[traitsOnlyInList2[tr]]);}
	for(trt=0;trt<traitTypesIncluded_.len();++trt){
		if(dataToAppend.traitTypesIncluded_[trt]==true){
			traitTypesIncluded_[trt]=true;}}

	//update the sublists of trait names
	for(tr=0;tr<dataToAppend.skippedTraitNum_.len();++tr){
		if(! skippedTraitNum_.Includes(dataToAppend.skippedTraitNum_[tr])){
			skippedTraitNum_.Append(dataToAppend.skippedTraitNum_[tr]);}
		}
	for(tr=0;tr<traitsOnlyInList2.len();++tr){
		trt = dataToAppend.traitType_[traitsOnlyInList2[tr]];
		traitName_.Append(NEW aString(dataToAppend.traitName_[traitsOnlyInList2[tr]]));
		int thisTrNum = traitNum_.len();
		traitNum_.Append(thisTrNum);
		int skipThisTrait=skippedTraitNum_.Includes(thisTrNum);
		if(!skipThisTrait) ntraits_++;
		switch(trt){
		  case hap://haploid loci
			allHapLoci_.Append(thisTrNum);
			if(skipThisTrait){
					skippedLocusNumHap_.Append(thisTrNum);}
				else{
					locusNumHap_.Append(thisTrNum);
					nlociHap_++;}
			break;
		  case codom://codominant loci
			allCodomLoci_.Append(thisTrNum);
			if(skipThisTrait){
					skippedLocusNumCodom_.Append(thisTrNum);}
				else{
					locusNumCodom_.Append(thisTrNum);
					nlociCodom_++;}
			break;
		  case dom://dominant loci
			allDomLoci_.Append(thisTrNum);
			if(skipThisTrait){
					skippedLocusNumDom_.Append(thisTrNum);}
				else{
					locusNumDom_.Append(thisTrNum);
					nlociDom_++;}
			break;
		  case SLcodom://codominant sex-linked loci
			allSLCodomLoci_.Append(thisTrNum);
			if(skipThisTrait){
					skippedLocusNumSLCodom_.Append(thisTrNum);}
				else{
					locusNumSLCodom_.Append(thisTrNum);
					nlociSLCodom_++;}
			break;
		  case SLdom://dominant sex-linked loci
			allSLDomLoci_.Append(thisTrNum);
			if(skipThisTrait){
					skippedLocusNumSLDom_.Append(thisTrNum);}
				else{
					locusNumSLDom_.Append(thisTrNum);
					nlociSLDom_++;}
			break;
		  case CYfemale://cytoplasmic female loci
			allCYfemaleLoci_.Append(thisTrNum);
			if(skipThisTrait){
					skippedLocusNumCYfemale_.Append(thisTrNum);}
				else{
					locusNumCYfemale_.Append(thisTrNum);
					nlociCYfemale_++;}
			break;
		  case CYfemaleOnly://cytoplasmic female loci
			allCYfemaleLoci_.Append(thisTrNum);
			if(! allPresentInFemalesOnly_.Includes(thisTrNum)){
				allPresentInFemalesOnly_.Append(thisTrNum);}
			if(skipThisTrait){
					skippedLocusNumCYfemale_.Append(thisTrNum);}
				else{
					locusNumCYfemale_.Append(thisTrNum);
					nlociCYfemale_++;
					presentInFemalesOnly_.Append(thisTrNum);
					}
			break;
		  case CYmale://cytoplasmic male loci
			allCYmaleLoci_.Append(thisTrNum);
			if(skipThisTrait){
					skippedLocusNumCYmale_.Append(thisTrNum);}
				else{
					locusNumCYmale_.Append(thisTrNum);
					nlociCYmale_++;}
			break;
		  case CYmaleOnly://cytoplasmic male loci
			allCYmaleLoci_.Append(thisTrNum);
			if(! allPresentInMalesOnly_.Includes(thisTrNum)){
				allPresentInMalesOnly_.Append(thisTrNum);}
			if(skipThisTrait){
					skippedLocusNumCYmale_.Append(thisTrNum);
					}
				else{
					locusNumCYmale_.Append(thisTrNum);
					nlociCYmale_++;
					presentInMalesOnly_.Append(thisTrNum);
					}
			break;
		  default: break;
		  }//switch
		}//tr
	npop_ += popsOnlyInList2.len();
	ResetFlags();
	GroupMembership();
	LocalityMembership();
	ParseGenotypicData();
	}//end MultiformatMarkerData::MergeDataSets(MultiformatMarkerData&)





void MultiformatMarkerData::ParseGenotypicData(){
	ObjList<aList<int> > sortedTraits;
	int trt,tr;
	for(trt=0;trt<traitTypesIncluded_.len();++trt){
		sortedTraits.Append(NEW aList<int>());}
	for(tr=0;tr<traitType_.len();++tr){
		sortedTraits[traitType_[tr]].Append(tr);}
//	CodominantMarkerData::ParseGenotypicData(sortedTraits[codom],0);
//	SexLinkedCodomMarkerData::ParseGenotypicData(sortedTraits[SLcodom],0);
	SortTraits(codom,sortedTraits[codom]);
	SortTraits(SLcodom,sortedTraits[SLcodom]);
	CodominantMarkerData::ParseGenotypicData(0);
	SexLinkedCodomMarkerData::ParseGenotypicData(0);
	}//MultiformatMarkerData::ParseGenotypicData(aList<int>& )


int MultiformatMarkerData::Ploidy(int loc){
	int datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
			return 1;
			break;
	  case codom://codominant
			return CodominantMarkerData::ploidy_;
			break;
	  case dom://dominant
			return DominantMarkerData2::ploidy_;
			break;
	  case SLcodom://sex-linked codominant
			return SexLinkedCodomMarkerData::ploidy_;
			break;
	  case SLdom://sex-linked dominant
			return SexLinkedDomMarkerData2::ploidy_;
			break;
	  case CYfemale://female cytoplasmic
	  case CYfemaleOnly://female cytoplasmic
			return CytoplasmicFemaleMarkerData::ploidy_;
			break;
	  case CYmale://male cytoplasmic
	  case CYmaleOnly://male cytoplasmic
			return CytoplasmicMaleMarkerData::ploidy_;
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 return 0;
	}//MultiformatMarkerData::Ploidy(int)


int MultiformatMarkerData::Ploidy(int pop, int indiv, int loc){
	int correctLoc, datasettype = traitType_[loc];
	switch(datasettype){
	  case hap://haploid
	  		correctLoc=locusNumHap_.Loc(loc);
			return HaploidMarkerData::Ploidy (pop,indiv,loc);
			break;
	  case codom://codominant
	  		correctLoc=locusNumCodom_.Loc(loc);
			return CodominantMarkerData::Ploidy (pop,indiv,loc);
			break;
	  case dom://dominant
	  		correctLoc=locusNumDom_.Loc(loc);
			return DominantMarkerData2::Ploidy (pop,indiv,loc);
			break;
	  case SLcodom://sex-linked codominant
	  		correctLoc=locusNumSLCodom_.Loc(loc);
			return SexLinkedCodomMarkerData::Ploidy (pop,indiv,loc);
			break;
	  case SLdom://sex-linked dominant
	  		correctLoc=locusNumSLDom_.Loc(loc);
			return SexLinkedDomMarkerData2::Ploidy (pop,indiv,loc);
			break;
	  case CYfemale://female cytoplasmic
	  case CYfemaleOnly://female cytoplasmic
	  		correctLoc=locusNumCYfemale_.Loc(loc);
			return CytoplasmicFemaleMarkerData::Ploidy (pop,indiv,loc);
			break;
	  case CYmale://male cytoplasmic
	  case CYmaleOnly://male cytoplasmic
	  		correctLoc=locusNumCYmale_.Loc(loc);
			return CytoplasmicMaleMarkerData::Ploidy (pop,indiv,loc);
			break;
	  default://not in a sublist
	  		break;
	  }//switch
	 return zero;
	}//MultiformatMarkerData::Ploidy(int, int, int)



void  MultiformatMarkerData::PrintAlleleFreqs (ostream& outFile, int ncols, char colSpacer){
	//Sends a table of allelic frequencies for each population to the output file.
	if(haploidDataUnskipped()){
		HaploidMarkerData::PrintHaplotypeFreqs(outFile, ncols, colSpacer);}
	if(codominantDataUnskipped()){
		CodominantMarkerData::PrintAlleleFreqs(outFile, ncols, colSpacer);}
	if(dominantDataUnskipped()){
		DominantMarkerData2::PrintPhenotypeFreqs(outFile, ncols, colSpacer);}
	if(codominantSexLinkedDataUnskipped()){
		SexLinkedCodomMarkerData::PrintAlleleFreqs(outFile, ncols, colSpacer);}
	if(dominantSexLinkedDataUnskipped()){
		SexLinkedDomMarkerData2::PrintPhenotypeFreqs(outFile, ncols, colSpacer);}
	if(cytoplasmicFemaleMarkersUnskipped()){
		CytoplasmicFemaleMarkerData::PrintGenotypeFreqs(outFile, ncols, colSpacer);}
	if(cytoplasmicMaleMarkersUnskipped()){
		CytoplasmicMaleMarkerData::PrintGenotypeFreqs(outFile, ncols, colSpacer);}
	}//end MultiformatMarkerData::PrintAlleleFreqs



/*
void  MultiformatMarkerData::PrintAlleleFreqsByPopList (ostream& outFile, 
											ObjList<ObjList<aList<double> > >& popFreqList, 
											ObjList<aList<double> >& popTwoNList, 
											int ncols){
	//popFreqList must contain the entire set of alleles in the data set
	//Sends a table of allelic frequencies to the output file.
	if(haploidDataIncluded()){
		HaploidMarkerData::PrintAlleleFreqsByPopList(outFile, popFreqList, popTwoNList, ncols);}
	if(codominantDataIncluded()){
		CodominantMarkerData::PrintAlleleFreqsByPopList(outFile, popFreqList, popTwoNList, ncols);}
	if(dominantDataIncluded()){
		CodominantMarkerData::PrintAlleleFreqsByPopList(outFile, popFreqList, popTwoNList, ncols);}
	if(codominantSexLinkedDataIncluded()){
		SexLinkedCodomMarkerData::PrintAlleleFreqsByPopList(outFile, popFreqList, popTwoNList, ncols);}
	if(dominantSexLinkedDataIncluded()){
		SexLinkedDomMarkerData2::PrintAlleleFreqsByPopList(outFile, popFreqList, popTwoNList, ncols);}
	}//end MultiformatMarkerData::PrintAlleleFreqsByPopList
*/


void	 MultiformatMarkerData::PrintArlequinFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintArlequinFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end MultiformatMarkerData::PrintArlequinFile(aString&)



void MultiformatMarkerData::PrintArlequinFile (ostream& outFile){
	//		theGenotypes: ListStrStructureTypePtr;
	outFile<<"[Profile]"<<endl;
	aString dataLine = "\tTitle=\"";
	dataLine.cat(title_);
	dataLine.cat('\"');
	outFile<<dataLine<<endl;
	dataLine = "\t\tNbSamples=";
	dataLine.cat(npop_);
	outFile<<dataLine<<endl;
	dataLine = "\t\tGenotypicData=1";
	outFile<<dataLine<<endl;
	dataLine = "\t\tGameticPhase=0";
	outFile<<dataLine<<endl;
	dataLine = "\t\tRecessiveData=0";
	outFile<<dataLine<<endl;
	dataLine = "\t\tIndividualLevel=1";
	outFile<<dataLine<<endl;
	dataLine = "\t\tMissingData=\"\?\"";
	outFile<<dataLine<<endl;
	dataLine = "\t\tCompDistMatrix=1";
	outFile<<dataLine<<endl<<endl;

	dataLine = "\t\tDataType=STANDARD";
	outFile<<dataLine<<endl;
	dataLine = "\t\tFrequency=ABS";
	outFile<<dataLine<<endl;
	dataLine = "\t\tLocusSeparator=WHITESPACE";
	outFile<<dataLine<<endl<<endl;
		
	dataLine = "[Data]";
	outFile<<dataLine<<endl<<endl;
	dataLine = "#Locus names:";
	outFile<<dataLine<<endl;

	dataLine= "\t# ";
	int pop,loc,tr,i,c,gr;
	for (loc=0;loc<locusNumCodom_.len();++loc){
		dataLine.cat(traitName_[locusNumCodom_[loc]]);
		if( ( ((double)(loc+1)) / 5.0) - ((loc+1) / 5) == 0){//new line for every 5 loci
				outFile<<dataLine<<endl;
				dataLine= "\t# ";}
			else if (loc+1 < locusNumCodom_.len()){
				dataLine.cat("\t");}
			else{
				outFile<<dataLine<<endl;
				dataLine= "\t# ";}
		}//end for loc
	outFile<<endl<<endl;
	dataLine = "[[Samples]]";
	outFile<<dataLine<<endl<<endl;
	dataLine.clear();
	StringList pseudohaps;
	for (pop=0;pop<npop_;++pop){
		dataLine = "\t\tSampleName=\"";
		dataLine.cat(popName_[pop]);
		dataLine.cat('\"');
		outFile<<dataLine<<endl;
//		StringList& theGenotypes = gtypes[pop];
		dataLine = "\t\tSampleSize=";
		dataLine.cat(individuals(pop));
		outFile<<dataLine<<endl;
		dataLine = "\t\tSampleData={";
		outFile<<dataLine<<endl;
		for (i=0;i<individuals(pop);++i){
			pseudohaps.Clear();
			//split each genotype into ploidy_ pieces, = "pseudohaplotypes", on separate rows
			pseudohaps.Append(NEW aString(rawTraits_[pop][i][0]));//ID code
			pseudohaps[0].cat(" 1 ");//frequency of this genotype
//			aString g(theGenotypes[i]);
//			aString endCharList=" \t";
//			aString *phaplo0 = g.ChopOffFirstWord(endCharList);//ID code
//			phaplo0->cat(" 1 ");
			for(c=1;c<ploidy();++c){
				pseudohaps.Append(NEW aString());
				pseudohaps[c].PrependCharToLen(' ',pseudohaps[0].len());}
//			aString  phaplo1;phaplo1.PrependCharToLen(' ',phaplo0->len());
//			outFile<<phaplo1<<endl;
			for(loc=0;loc<nlociCodom_;++loc){
				tr=locusNumCodom_[loc];
				for(c=0;c<ploidy();++c){
//					TextDatum& allele = rawCodomGtypes_[pop][i][locusNumCodom_[loc]][c];
					TextDatum& allele = CodominantMarkerData::GenotypicVal(pop,i,loc,c);
					if(allele.OK()){
							pseudohaps[c].cat(allele);}
						else{//missing data 
							pseudohaps[c].cat('\?');}
					pseudohaps[c].cat(' ');
					}//c
/*				aString *gLoc0 = GetGenotypeString(pop,i,loc,0);
				aString *gLoc1 = GetGenotypeString(pop,i,loc,1);
				if(*gLoc0==' '){*gLoc0='\?';}
				if(*gLoc1==' '){*gLoc1='\?';}
				phaplo0->cat(*gLoc0);phaplo0->cat(' ');
				phaplo1.cat(*gLoc1);phaplo1.cat(' ');
				delete gLoc0; delete gLoc1;
*/				}//loc
			pseudohaps.Print(outFile);
//			outFile<<*phaplo0<<endl;
//			outFile<< phaplo1<<endl;
//			delete phaplo0;
			}//end for i
		outFile<<"\t\t}"<<endl<<endl;
		}//end for pop
	outFile<<endl<<endl;
	dataLine = "[[Structure]]";
	outFile<<dataLine<<endl<<endl;

	dataLine = "StructureName = \"Structure from population titles\"";
	outFile<<dataLine<<endl;
	dataLine = "NbGroups = 1";
	outFile<<dataLine<<endl;
	dataLine = "IndividualLevel = 1";
	outFile<<dataLine<<endl;
	for(gr=0;gr<ngroups_;++gr){
		dataLine = "\t#";
		dataLine.cat(groupName_[gr]);
		outFile<<dataLine<<endl;
		dataLine = "\tGroup =  {";
		outFile<<dataLine<<endl;
		for(pop=0;pop<groupHierarchy_[gr].len();++pop){
			outFile<<"\t\t\""<<popName_[groupHierarchy_[gr][pop]]<<'\"'<<endl;
			}
		outFile<<"\t\t}"<<endl;
	}


	outFile<<endl;
	}//end MultiformatMarkerData::PrintArlequinFile




void	 MultiformatMarkerData::PrintGelJockFile(aString& outFileName){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintGelJockFile(dataFile);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end MultiformatMarkerData::PrintGelJockFile(aString&)



void MultiformatMarkerData::PrintGelJockFile (ostream& outFile){
	//omits skipped traits
	aString dataLine = title_;
	dataLine.cat(":  skipped traits omitted");
	outFile<<dataLine<<endl;
	dataLine = "NPOP=";
	dataLine.cat(npop_);
	dataLine.cat(", NTRAITS=");
	dataLine.cat(ntraits_-skippedTraitNum_.len());
	dataLine.cat(", MAXALL=");
	dataLine.cat(maxall());
	dataLine.cat(", NGROUPS=");
	dataLine.cat(ngroups_);
	dataLine.cat(", ALLELELEN=");
	dataLine.cat(alleleLen());
	outFile<<dataLine<<endl;
	dataLine.clear();
	int tr, trt, loc, a, pop, i;
	for (trt=0;trt<traitNum_.len();++trt){
		tr=traitNum_[trt];
		dataLine.cat(traitName_[tr]);
		if( ( ((double)(trt+1)) / 10.0) - ((trt+1) / 10) == 0){//new line for every 10 traits
				outFile<<dataLine<<endl;
				dataLine.clear();}
			else if (trt+1 < traitNum_.len()){
				dataLine.cat(", ");}
			else{
				outFile<<dataLine<<endl;
				dataLine.clear();}
		}//end for loc
	dataLine = "SKIP=0";
	outFile<<dataLine<<endl;
	dataLine.clear();
	aString blankStr;
	for(a=0;a<alleleLen();++a){
		blankStr+=' ';}
	int maxpopsize=0, trtype;
	for (pop=0;pop<npop_;++pop){
		maxpopsize = MAX(maxpopsize,individuals(pop));}
	maxpopsize = Digits(maxpopsize);//formatting constant
	for (pop=0;pop<npop_;++pop){
		outFile<<popName_[pop]<<endl;
		for (i=0;i<individuals(pop);++i){
			dataLine.clear();
			for(trt=0;trt<traitNum_.len();++trt){
				tr=traitNum_[trt];
				if(skippedTraitNum_.Includes(tr)) continue;//can't, actually
				if(trt>0){
					dataLine += ' ';}//spacer
				trtype = traitType_[tr];
				if(trtype==codom){//codominant --work from parsed genotypes
						TextDataList& traitSet = CodominantMarkerData::GenotypicVal(pop,i,tr);
						for(a=0;a<traitSet.len();++a){
							TextDatum geneC = traitSet[a];
						
							if(geneC.OK()){
									geneC.PrependCharToLen('0',alleleLen());
									dataLine += geneC;}
								else{
									dataLine += blankStr;}}
						}
					else if(trtype==SLcodom){//sex-linked codominant --work from parsed genotypes
						loc = locusNumSLCodom_.Loc(tr);
						TextDataList& traitSet = SexLinkedCodomMarkerData::GenotypicVal(pop,i,tr);
						for(a=0;a<traitSet.len();++a){
							TextDatum geneSL = traitSet[a];
							if(geneSL.OK()){
									geneSL.PrependCharToLen('0',alleleLen());
									dataLine += geneSL;}
								else{
									dataLine += blankStr;}}
						}
					else if(trtype==hap){
						loc = locusNumHap_.Loc(tr);
						TextDatum geneH = HaploidMarkerData::GenotypicVal(pop,i,loc);
//						TextDatum& geneH = HaploidMarkerData::GenotypicVal(pop,i,tr);
						if(geneH.OK()){
								geneH.PrependCharToLen('0',alleleLen());
								dataLine += geneH;}
							else{
								dataLine += blankStr;}
						}
					else if(trtype==dom){
						loc = locusNumDom_.Loc(tr);
						TextDatum pheneD = DominantMarkerData2::PhenotypicVal(pop,i,loc);
//						TextDatum pheneD = DominantMarkerData2::PhenotypicVal(pop,i,tr);
						if(pheneD.OK()){
								pheneD.PrependCharToLen('0',alleleLen());
								dataLine += pheneD;}
							else{
								dataLine += blankStr;}
						}
					else if(trtype==SLdom){
						loc = locusNumSLDom_.Loc(tr);
						TextDatum pheneSLD = SexLinkedDomMarkerData2::PhenotypicVal(pop,i,loc);
						if(pheneSLD.OK()){
								pheneSLD.PrependCharToLen('0',alleleLen());
								dataLine += pheneSLD;}
							else{
								dataLine += blankStr;}
						}
					else if(trtype==CYfemale || trtype == CYfemaleOnly){
						loc = locusNumCYfemale_.Loc(tr);
						TextDatum geneCYF = CytoplasmicFemaleMarkerData::GenotypicVal(pop,i,loc,0);
//						TextDatum geneCYF = CytoplasmicFemaleMarkerData::GenotypicVal(pop,i,tr,0);
						if(geneCYF.OK()){
								geneCYF.PrependCharToLen('0',alleleLen());
								dataLine += geneCYF;}
							else{
								dataLine += blankStr;}
						}
					else if(trtype==CYmale || trtype == CYmaleOnly){
						loc = locusNumCYmale_.Loc(tr);
						TextDatum geneCYM = CytoplasmicMaleMarkerData::GenotypicVal(pop,i,loc,0);
//						TextDatum geneCYM = CytoplasmicMaleMarkerData::GenotypicVal(pop,i,tr,0);
						if(geneCYM.OK()){
								geneCYM.PrependCharToLen('0',alleleLen());
								dataLine += geneCYM;}
							else{
								dataLine += blankStr;}
						}
					else{//work from raw traits
						TextDatum& trait = rawTraits_[pop][i][tr];
						if(trait.OK()){
								if(trt==0){//aligns text for easier editing
										aString IDstr(trait);
										IDstr.PrependCharToLen(' ',maxpopsize);
										dataLine += IDstr;}
									else{
										dataLine += trait;}
								}
							else{ dataLine += blankStr;}
						}
				}//trt
			outFile<<dataLine<<endl;
			}//end for i
		outFile<<"NEXT"<<endl;
		}//end for pop
	outFile<<"END"<<endl;
	}//end MultiformatMarkerData::PrintGelJockFile




void MultiformatMarkerData::PrintHeterozygosity (ostream& outFile, const int hetType, int ncols){
	//file assumed open
	//Sends a table of the designated heterozygosities for each population to the output file.
//	if(haploidDataIncluded()){
//		HaploidMarkerData::PrintHeterozygosity(outFile, hetType, ncols);}
	if(codominantDataIncluded()){
		CodominantMarkerData::PrintHeterozygosity(outFile, hetType, ncols);}
	if(codominantSexLinkedDataIncluded()){
		SexLinkedCodomMarkerData::PrintHeterozygosity(outFile, hetType, ncols);}
//	if(dominantSexLinkedDataIncluded()){
//		SexLinkedDomMarkerData2::PrintHeterozygosity(outFile, hetType, ncols);}
	}//end MultiformatMarkerData::PrintHeterozygosity



/*
void MultiformatMarkerData::PrintLoci (ostream& outFile, int lociPerLine){//file assumed open
	int lineSpot = 0;
	for (int loc=0;loc<locusNumCodom_.len();++loc){ 
		aString& locName = traitName_[locusNumCodom_[loc]];
		outFile<<locName;
		++lineSpot;
		if ((loc == locusNumCodom_.len()-1) || (lineSpot == lociPerLine)) {
				outFile<<endl;}
			else{
				outFile<<", ";}
		}//end for loc
	outFile<<endl;
	}//end MultiformatMarkerData::PrintLoci
*/




void	 MultiformatMarkerData::PrintSASFormatData(ostream& outFile, char colSpacer){
	//prints a data set that can be read using SAS
	//missing data represented by '.', columns separated by spaces
	//possible alleles for each locus in separate columns
	if(haploidDataIncluded()){
		HaploidMarkerData::PrintSASFormatData(outFile, colSpacer);}
	if(codominantDataIncluded()){
		CodominantMarkerData::PrintSASFormatData(outFile, colSpacer);}
	if(dominantDataIncluded()){
		DominantMarkerData2::PrintSASFormatData(outFile, colSpacer);}
	if(codominantSexLinkedDataIncluded()){
		SexLinkedCodomMarkerData::PrintSASFormatData(outFile, colSpacer);}
	if(dominantSexLinkedDataIncluded()){
		SexLinkedDomMarkerData2::PrintSASFormatData(outFile, colSpacer);}
	if(cytoplasmicFemaleMarkersIncluded()){
		CytoplasmicFemaleMarkerData::PrintSASFormatData(outFile, colSpacer);}
	if(cytoplasmicMaleMarkersIncluded()){
		CytoplasmicMaleMarkerData::PrintSASFormatData(outFile, colSpacer);}
	}//end  MultiformatMarkerData::PrintSASFormatData(ostream&,char)
	
	
void	 MultiformatMarkerData::PrintSASFormatData(aString& outFileName, char colSpacer){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintSASFormatData(dataFile,colSpacer);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end MultiformatMarkerData::PrintSASFormatData(aString&,char)



void MultiformatMarkerData::PrintVariabilityScores(aString& outFileName, char sepChar){
	fstream dataFile;
	char* fileName = outFileName.raw();
	dataFile.open(fileName, ios::out /*,filebuf::openprot*/);
	PrintVariabilityScores(dataFile, sepChar);
	dataFile.close();
	delete [] fileName;fileName=0;
	}//end MultiformatMarkerData::PrintVariabilityScores(aString&, char)



void MultiformatMarkerData::PrintVariabilityScores(ostream& outFile, char sepChar){
//	if(haploidDataIncluded()){
//		HaploidMarkerData::PrintVariabilityScores(outFile, sepChar);}
	if(codominantDataIncluded()){
		CodominantMarkerData::PrintVariabilityScores(outFile, sepChar);}
//	if(dominantDataIncluded()){
//		DominantMarkerData2::PrintVariabilityScores(outFile, sepChar);}
	if(codominantSexLinkedDataIncluded()){
		SexLinkedCodomMarkerData::PrintVariabilityScores(outFile, sepChar);}
//	if(dominantSexLinkedDataIncluded()){
//		SexLinkedDomMarkerData2::PrintVariabilityScores(outFile, sepChar);}
	}//end  MultiformatMarkerData::PrintVariabilityScores



int MultiformatMarkerData::ReadArlequinDataFile (aString* dataFileName){
	fstream dataFile;	
	char *dataLine=NEW char[10001];dataLine[0]='\0';
//	aString beast, popLine;
	aString endCharList(" \t,:\"{}[]!?\n\r"), eq("="), parameterLine;
	char* filename=dataFileName->raw();
	dataFile.open(filename,ios::in);
	int dataFileOpen = dataFile.is_open();
	if(! dataFileOpen){
		aString errstr("couldn\'t open the file ");
		errstr.cat(filename);
		aError(errstr);
		return 0;}

	StringList rawArlequinData0;//read in everything as strings, then sort and process them
	ObjList<StringList> arlequinDataBySection;//sections and subsections
	int nArlSections=6;
	enum arlSections {profile=0,haplotype=1,distmatrices=2,samples=3,genStructure=4,mantelTests=5};
	aList<int> arlSectionStartLines, arlSectionEndLines;
	int sec0;
	for(sec0=0;sec0<nArlSections;++sec0){//initialize
		arlequinDataBySection.Append(NEW StringList());
		arlSectionStartLines.Append(-1);//flag
		arlSectionEndLines.Append(-1);//flag
		}

	aString rawDataLine;
	int lineNum=-1;//stores current line of valid data
	while(!dataFile.eof()){
		dataFile.getline(dataLine, 10000);
		rawDataLine = dataLine;
		SnipArlequinComment(rawDataLine);//removes comments
		rawDataLine.CullLeadingWhiteSpace();
		rawDataLine.CullTrailingWhiteSpace();
		if(rawDataLine.len()>0){//it's not blank
			rawArlequinData0.Append(NEW aString(rawDataLine));
			++lineNum;
			if(rawDataLine[0]=='['){//assume it's a section heading line
				aString linecopy(rawDataLine);//work with a copy
				linecopy.CullLeadingChar('[');
				linecopy.CullTrailingChar(']');
				linecopy.CullLeadingWhiteSpace();//in case of sloppy text
				linecopy.CullTrailingWhiteSpace();
				if(linecopy=="Profile" || linecopy=="profile" || linecopy=="PROFILE"){
						arlSectionStartLines[profile]=lineNum;
						arlSectionEndLines[profile]=lineNum-1;//end of previous section
						}
					else if(linecopy=="HaplotypeDefinition" || 
							linecopy=="haplotypedefinition" || 
							linecopy=="HAPLOTYPEDEFINITION"){
						arlSectionStartLines[haplotype]=lineNum;
						arlSectionEndLines[haplotype]=lineNum-1;//end of previous section
						}
					else if(linecopy=="DistanceMatrix" || 
							linecopy=="distancematrix" || 
							linecopy=="DISTANCEMATRIX"){
						arlSectionStartLines[distmatrices]=lineNum;
						arlSectionEndLines[distmatrices]=lineNum-1;//end of previous section
						}
					else if(linecopy=="Samples" || 
							linecopy=="samples" || 
							linecopy=="SAMPLES"){
						arlSectionStartLines[samples]=lineNum;
						arlSectionEndLines[samples]=lineNum-1;//end of previous section
						}
					else if(linecopy=="Structure" || 
							linecopy=="structure" || 
							linecopy=="STRUCTURE"){
						arlSectionStartLines[genStructure]=lineNum;
						arlSectionEndLines[genStructure]=lineNum-1;//end of previous section
						}
					else if(linecopy=="Mantel" || 
							linecopy=="mantel" || 
							linecopy=="MANTEL"){
						arlSectionStartLines[mantelTests]=lineNum;
						arlSectionEndLines[mantelTests]=lineNum-1;//end of previous section
						}
					else{}
				}//if '['
			}//len>0
		}//end while !eof
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;

	aList<int> sortedStartLines(arlSectionStartLines);
	aList<int> tempStartLines,tempEndLines;
	int sec2,sec3,sec4,sec5,i,p0,p1,p2,p3,p4,p5,p6,p7,p8,p9;
	for(sec2=0;sec2<arlSectionEndLines.len();++sec2){
		sortedStartLines.Append(arlSectionEndLines[sec2]);
		tempStartLines.Append(0);tempEndLines.Append(0);
		}
	sortedStartLines.Append(rawArlequinData0.len()-1);
	sortedStartLines.Sort();
	for(sec3=0;sec3<arlSectionStartLines.len();++sec3){
		int posStart = sortedStartLines.Loc(arlSectionStartLines[sec3]);
		int endSection = sortedStartLines[posStart+1];
		arlSectionEndLines[sec3] = endSection;
		}//end for sec3
	
	if(arlSectionStartLines.Last() == -1){
		arlSectionStartLines.Last() = sortedStartLines.Last();
		arlSectionEndLines.Last() = sortedStartLines.Last();}
	for(sec4 = nArlSections-2;sec4>=0;--sec4){
		if(arlSectionStartLines[sec4] == -1){
			arlSectionStartLines[sec4] = arlSectionStartLines[sec4+1];
			arlSectionEndLines[sec4] = arlSectionStartLines[sec4];}
		}//sec4
//	arlSectionStartLines.Print(cout," ");
//	arlSectionEndLines.Print(cout," ");
	
	//finish putting raw data into sections
	for(sec5 = 0;sec5<nArlSections;++sec5){
		int start = arlSectionStartLines[sec5], end = arlSectionEndLines[sec5];
		for(i=start+1; i<=end;++i){
			aString& processedDataLine = rawArlequinData0[i];
			arlequinDataBySection[sec5].Append(NEW aString(processedDataLine));
			}//i
//		arlequinDataBySection[sec5].Print(cout);
		}//sec5
		
	//now process each section separately
	// "[Profile]"//title_
	aString *tword=0, pline;
	for(p0=0;p0<arlequinDataBySection[profile].len();++p0){
		pline = arlequinDataBySection[profile][p0];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "title_" || *tword == "Title" || *tword == "TITLE" ){
			pline.CullLeadingChar('\"');
			pline.CullTrailingChar('\"');
			title_ = pline;
			arlequinDataBySection[profile].Delete(p0);//no longer needed
			delete tword;tword=0;
			break;
			}//if==title_
		delete tword;tword=0;
		}

	// "[Profile]"//NbSamples
	for(p1=0;p1<arlequinDataBySection[profile].len();++p1){
		pline = arlequinDataBySection[profile][p1];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "NbSamples" || *tword == "nbsamples" || *tword == "NBSAMPLES"){
			pline.CullLeadingWhiteSpace();
			npop_ = pline.toInt();
			arlequinDataBySection[profile].Delete(p1);//no longer needed
			delete tword;tword=0;
			break;
			}//if==NbSamples
		delete tword;tword=0;
		}

	// "[Profile]"//GenotypicData
	for(p2=0;p2<arlequinDataBySection[profile].len();++p2){
		pline = arlequinDataBySection[profile][p2];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "GenotypicData" || *tword == "genotypicdata" || *tword == "GENOTYPICDATA"){
			pline.CullLeadingWhiteSpace();
			int gtypedata = pline.toInt();
			if(gtypedata != 1){
				aString err("Can't read in this type of data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p2);//no longer needed
			delete tword;tword=0;
			break;
			}//if==GenotypicData
		delete tword;tword=0;
		}

	// "[Profile]"//GameticPhase
	for(p3=0;p3<arlequinDataBySection[profile].len();++p3){
		pline = arlequinDataBySection[profile][p3];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "GameticPhase" || *tword == "gameticphase" || *tword == "GAMETICPHASE"){
			pline.CullLeadingWhiteSpace();
			int gphasedata = pline.toInt();
			if(gphasedata != 0){
				aString err("Can't read this type of data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p3);//no longer needed
			delete tword;tword=0;
			break;
			}//if==GameticPhase
		delete tword;tword=0;
		}

	// "[Profile]"//RecessiveData
	for(p4=0;p4<arlequinDataBySection[profile].len();++p4){
		pline = arlequinDataBySection[profile][p4];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "RecessiveData" || *tword == "recessivedata" || *tword == "RECESSIVEDATA"){
			pline.CullLeadingWhiteSpace();
			int recdata = pline.toInt();
			if(recdata != 0){
				aString err("Can't read this type of data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p4);//no longer needed
			delete tword;tword=0;
			break;
			}//if==RecessiveData
		delete tword;tword=0;
		}

	// "[Profile]"//DataType
	for(p5=0;p5<arlequinDataBySection[profile].len();++p5){
		pline = arlequinDataBySection[profile][p5];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "DataType" || *tword == "datatype" || *tword == "DATATYPE"){
			pline.CullLeadingWhiteSpace();
			if(pline != "STANDARD"){
				aString err("Can only read DataType=STANDARD data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p5);//no longer needed
			delete tword;tword=0;
			break;
			}//if==DataType
		delete tword;tword=0;
		}

	// "[Profile]"//MissingData
	aString missingDataFlag("?");//default
	for(p6=0;p6<arlequinDataBySection[profile].len();++p6){
		pline = arlequinDataBySection[profile][p6];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "MissingData" || *tword == "missingdata" || *tword == "MISSINGDATA"){
			pline.CullLeadingWhiteSpace();
			pline.CullLeadingChar('\"');//can be single or double quotes
			pline.CullTrailingChar('\"');
			pline.CullLeadingChar('\'');
			pline.CullTrailingChar('\'');
			missingDataFlag = pline;
			if(missingDataFlag.len() != 1){
				aString err("Missing data must be denoted by a single character.  Default used.");
				aError(err);
				missingDataFlag="?";
				}
			arlequinDataBySection[profile].Delete(p6);//no longer needed
			delete tword;tword=0;
			break;
			}//if==IndividualLevel
		delete tword;tword=0;
		}

	// "[Profile]"//RecessiveAllele
	aString RecessiveAlleleFlag("?");//default
	for(p7=0;p7<arlequinDataBySection[profile].len();++p7){
		pline = arlequinDataBySection[profile][p7];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "RecessiveAllele" || *tword == "recessiveallele" || *tword == "RECESSIVEALLELE"){
			pline.CullLeadingWhiteSpace();
			pline.CullLeadingChar('\"');//can be single or double quotes
			pline.CullTrailingChar('\"');
			pline.CullLeadingChar('\'');
			pline.CullTrailingChar('\'');
			RecessiveAlleleFlag = pline;
			if(RecessiveAlleleFlag.len() == 0){
				aString err("Recessive allele (homozygote) not identified.  Default used.");
				aError(err);
				RecessiveAlleleFlag="null";
				}
			arlequinDataBySection[profile].Delete(p7);//no longer needed
			delete tword;tword=0;
			break;
			}//if==IndividualLevel
		delete tword;tword=0;
		}

	// "[Profile]"//Frequency
	for(p8=0;p8<arlequinDataBySection[profile].len();++p8){
		pline = arlequinDataBySection[profile][p8];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "Frequency" || *tword == "frequency" || *tword == "FREQUENCY"){
			pline.CullLeadingWhiteSpace();
			if(!(pline == "ABS" || pline == "abs")){
				aString err("Can only read Frequency=ABS data from the Arlequin file");
				aError(err);
				delete tword;tword=0;
				return 2;
				}
			arlequinDataBySection[profile].Delete(p8);//no longer needed
			delete tword;tword=0;
			break;
			}//if==DataType
		delete tword;tword=0;
		}


	// "[Profile]"//LocusSeparator
	aString locSep(" \t");//default
	for(p9=0;p9<arlequinDataBySection[profile].len();++p9){
		pline = arlequinDataBySection[profile][p9];
		tword=pline.ChopOffFirstWord(eq);
		tword->CullTrailingWhiteSpace();
		if(*tword == "LocusSeparator" || *tword == "locusseparator" || *tword == "LOCUSSEPARATOR"){
			pline.CullLeadingWhiteSpace();
			if(pline == "WHITESPACE" || pline == "whitespace"){
					locSep = " \t";}
				else if(pline == "TAB" || pline == "tab"){
					locSep = "\t";}
				else if(pline == "COMMA" || pline == "comma"){
					locSep = ",";}
				else if(pline == "NONE" || pline == "none"){
					aString err("Need locus separator to read data from the Arlequin file");
					aError(err);
					delete tword;tword=0;
					return 2;}
				else{
					locSep = pline;
					if(locSep.len()==0){
						aString err("LocusSeparator not specified correctly.  Default used.");
						aError(err);
						locSep = " \t";}
					}
			arlequinDataBySection[profile].Delete(p9);//no longer needed
			delete tword;tword=0;
			break;
			}//if==DataType
		delete tword;tword=0;
		}

	//[Profile] //CompDistMatrix=  ignored
	//[Profile] //FrequencyThreshold=  ignored
	//[Profile] //EpsilonValue=  ignored

	//section [Data]

	//[Data] [[HaplotypeDesignation]]  ignored
	//[Data] [[DistanceMatrix]]  ignored


	//[Data] [[Sample]] 
	//read in population data
	long nloci=0;
	unsigned int alleleLength=1;
	aString popLine;
	ObjList<ObjList<ObjList<StringList> > > rawArlequinData;//temp storage [pop][i][loc][a]
	ObjList<StringList> rawIDnames;//temp storage [pop][id]
	ObjList<aList<long> > popGenotypeFreqs;//temp storage [pop][id]
	long dline=0;//data line counter
	int pop;
	for (pop=0;pop<npop_;++pop){
		rawArlequinData.Append(NEW ObjList<ObjList<StringList> >());//set up raw data 
		rawIDnames.Append(NEW StringList());//set up raw data 
		popGenotypeFreqs.Append(NEW aList<long>());//set up raw data 
		
		//get the title_ of this population
		popLine = arlequinDataBySection[samples][dline++];
		aString* junk = popLine.ChopOffFirstWord(eq);
		delete junk; junk=0;
		popLine.CullLeadingWhiteSpace();
		popLine.CullLeadingChar('\"');
		popLine.CullTrailingChar('\"');
		aString *s = NEW aString(popLine);
		popName_.Append(s);//append a copy

		//read sample size to expect
		popLine = arlequinDataBySection[samples][dline++];
		junk = popLine.ChopOffFirstWord(eq);
		delete junk; junk=0;
		popLine.CullLeadingWhiteSpace();
		long indivs=popLine.toLong();
		long i=(-1);//a counter
		popLine.clear();
		popLine = arlequinDataBySection[samples][dline++];
		popLine.CullTrailingWhiteSpace();
		if(popLine[popLine.len()-1] != '{'){
			aString err("Expected \'SampleData={\' at ");err.cat(*s);//adds pop title_
			aError(err);}
		bool thisPopDone = false;
		while (!thisPopDone){//LOOK AT EACH INDIVIDUAL
			popLine.clear();
			popLine = arlequinDataBySection[samples][dline++];
			if(popLine[0] == '}'){
				thisPopDone = true;
				continue;}//goes to end of while !thisPopDone

			rawArlequinData[pop].Append(NEW ObjList<StringList>());//set up raw data
			i++; 
			
			//individuals come as 2 consecutive lines in diploid data
			//the first line contains an ID number and a frequency
			aString indivline1(popLine);
			aString *id=indivline1.ChopOffFirstWord();
			aString *fr=indivline1.ChopOffFirstWord();
			long gfreq = fr->toLong();
			delete fr; fr=0;
			rawIDnames[pop].Append(id);
			popGenotypeFreqs[pop].Append(gfreq);

			aString indivline2(arlequinDataBySection[samples][dline]);//sneak a peek ahead to check ploidy
			bool diploid = false;
			if(indivline2[0]=='}'){
					thisPopDone = true; break;}
				else if(indivline2.len() == indivline1.len()){//assume diploid based on symmetry in data file
					diploid=true;
					dline++;//increment the counter
					}
			long loc0=0;
			aString *gene=0;			
			while(indivline1.len()>0){
				rawArlequinData[pop][i].Append(NEW StringList());//new for each locus
				gene = indivline1.ChopOffFirstWord(locSep);
				alleleLength = MAX(alleleLength,gene->len());
				rawArlequinData[pop][i][loc0].Append(gene);
				++loc0;
				}//end while
			nloci = MAX(loc0,nloci);
			if(diploid){
				long loc01=0;
				while(indivline2.len()>0){
					gene = indivline2.ChopOffFirstWord(locSep);
					alleleLength = MAX(alleleLength,gene->len());
					rawArlequinData[pop][i][loc01].Append(gene);
					++loc01;
					}//end while
				nloci = MAX(loc01,nloci);
				}//if diploid
			}//end while !thisPopDone
		}//end for pop

	//make up some locus names
	traitName_.Clear();
	traitName_.Append(NEW aString("id"));
	ObjList<StringList> alleleNames;
	int maxloclen = Digits(nloci), loc,pop1,h,loc1;
	for(loc=0;loc<nloci;++loc){
		alleleNames.Append(NEW StringList());
		aString *lname = NEW aString("loc");
		aString lnum(loc+1);lnum.PrependCharToLen('0',maxloclen);
		lname->cat(lnum);
		traitName_.Append(lname);
		}//for loc

	//recode data, using a list of alleles/locus
	for(pop1=0;pop1<npop_;++pop1){
		for(i=0;i<rawArlequinData[pop1].len();++i){
			for(loc1=0;loc1<nloci;++loc1){
				for(h=0;h<rawArlequinData[pop1][i][loc1].len();++h){
					aString gene(rawArlequinData[pop1][i][loc1][h]);//work with copy
					if(! (alleleNames[loc1].Includes(&gene))){
						alleleNames[loc1].Append(NEW aString(gene));}
					}//end h
				}//loc1
			}//i
		}//pop1
	for(loc1=0;loc1<nloci;++loc1){//sort; clear missing data flags
		alleleNames[loc1].Sort();
		if(alleleNames[loc1].Includes(&missingDataFlag)){
			int site = alleleNames[loc1].Loc(&missingDataFlag);
			alleleNames[loc1].Delete(site);}
		}//loc1
			
	//finish recoding
	SetAlleleLength(alleleLength);
	SetPloidy(2);//assumed for now -- may be able to infer this from data
	rawTraits_.Clear();
	nlociCodom_ = nloci;
	aString missingDataToPrint(missingDataFlag);
	missingDataToPrint.PrependCharToLen(' ',alleleLength);
	int pop2,i2,loc2,h2,n;
	for(pop2=0;pop2<npop_;++pop2){
		rawTraits_.Append(NEW ObjList<TextDataList>());				
		for(i2=0;i2<rawArlequinData[pop2].len();++i2){
			TextDataList gtyp(missingDataFlag);
			gtyp.Append(NEW TextDatum(rawIDnames[pop2][i2]));
			for(loc2=0;loc2<nloci;++loc2){
				bool datapresent=false;
				TextDatum *gtypLoc = NEW TextDatum();//starts out as missing
				for(h2=0;h2<rawArlequinData[pop2][i2][loc2].len();++h2){
					aString gene2(rawArlequinData[pop2][i2][loc2][h2]);//work with copy
					if(gene2 == missingDataFlag){
							gtypLoc->cat(missingDataToPrint);}
						else{
							datapresent=true;
							gtypLoc->missing(false);
							gene2.PrependCharToLen(' ',alleleLen());//standard size for all loci
							gtypLoc->cat(gene2);}
					}//end h2
				gtyp.Append(gtypLoc);
				}//loc2
			for(n=0;n<popGenotypeFreqs[pop2][i2];++n){//use frequency info here
				rawTraits_[pop2].Append(NEW TextDataList(gtyp));}
			}//i	
		popSize_.Append(rawTraits_[pop2].len());
		}//pop2
		



	//[Data] [[Structure]] 
	//read in group information, if any
	bool readGroupInfo = false;
	if(arlequinDataBySection[genStructure].len() > 0){
		readGroupInfo=true;}
	int s0,s1,s2,gr;
	if(! readGroupInfo){
			ngroups_=1;
			GroupMembership();}
		else{//read group info explicitly
			for(s0=0;s0<arlequinDataBySection[genStructure].len();++s0){
				parameterLine = arlequinDataBySection[genStructure][s0];
				aString *pword=parameterLine.ChopOffFirstWord(eq);
				pword->CullTrailingWhiteSpace();
				if(*pword == "StructureName" || *pword == "structurename" || *pword == "STRUCTURENAME"){
					//just ignore
					arlequinDataBySection[genStructure].Delete(s0);//no longer needed
					delete pword;pword=0;
					break;
					}
				delete pword;pword=0;
				}//s0

			// "[Structure]"//NbGroups
			for(s1=0;s1<arlequinDataBySection[genStructure].len();++s1){
				parameterLine = arlequinDataBySection[genStructure][s1];
				tword=parameterLine.ChopOffFirstWord(eq);
				tword->CullTrailingWhiteSpace();
				if(*tword == "NbGroups" || *tword == "nbgroups" || *tword == "NBGROUPS"){
					parameterLine.CullLeadingWhiteSpace();
					ngroups_ = parameterLine.toInt();
					arlequinDataBySection[genStructure].Delete(s1);//no longer needed
					delete tword;tword=0;
					break;
					}//if==NbGroups
				delete tword;tword=0;
				}//s1

			// "[Structure]"//IndividualLevel
			for(s2=0;s2<arlequinDataBySection[genStructure].len();++s2){
				parameterLine = arlequinDataBySection[genStructure][s2];
				tword=parameterLine.ChopOffFirstWord(eq);
				tword->CullTrailingWhiteSpace();
				if(*tword == "IndividualLevel" || *tword == "individuallevel" || *tword == "INDIVIDUALLEVEL"){
					parameterLine.CullLeadingWhiteSpace();
					int ilevel = parameterLine.toInt();
					if(ilevel != 1){
						aString err("In [[Structure]], must set IndividualLevel=1 in order to read Arlequin data");
						aError(err);
						delete tword; tword=0;
						return 2;
						}
					arlequinDataBySection[genStructure].Delete(s2);//no longer needed
					delete tword;tword=0;
					break;
					}//if==IndividualLevel
				delete tword;tword=0;
				}//s2

			groupName_.Clear();
			groupHierarchy_.Clear();
			long sline=0;
			for(gr=0;gr<ngroups_;++gr){
				aString gpnum(gr+1);
				gpnum.PrependCharToLen('0',Digits(ngroups_));
				gpnum.prepend("group");
				groupName_.Append(NEW aString(gpnum));
				groupHierarchy_.Append(NEW aList<int>());
				//read group membership intro lines
				parameterLine = arlequinDataBySection[genStructure][sline++];
				if(parameterLine[parameterLine.len()-1] != '{'){
						aString err("Expected \"Group={\" in [[Structure]]");
						aError(err);
						continue;}
					else{//read names in group
						bool thisGpDone = false;
						while (!thisGpDone){//LOOK AT EACH title_
							aString gpLine = arlequinDataBySection[genStructure][sline++];
							if(gpLine[0] == '}'){
								thisGpDone = true;
								continue;}//goes to end of while !thisGpDone
			
							gpLine.CullLeadingChar('\"');
							gpLine.CullTrailingChar('\"');
							int popnum = popName_.Loc(&gpLine);
							groupHierarchy_[gr].Append(popnum);
							}//end while !thisGpDone
						}//end else
				}//for gr
		}//end if readGroupInfo
		
	LocalityMembership();

	traitType_.Clear();
	traitTypesIncluded_.Clear();
	int trt;
	for(trt=0;trt<6;++trt){
		traitTypesIncluded_.Append(0);}
	traitTypesIncluded_[2]=true;
	traitType_.Append(ID);//id
	aList<int> codomLoci;
	for(loc=0;loc<nlociCodom_;++loc){
		traitType_.Append(codom);
		codomLoci.Append(loc+1);}//all loci are assumed codominant  - may adjust this later
	CodominantMarkerData::SetLoci(codomLoci);
	ParseGenotypicData();
	CheckForNewAlleles();
	return 1;
	}//end MultiformatMarkerData::ReadArlequinDataFile



int MultiformatMarkerData::ReadGelJockDataFile (aString* dataFileName){
	//set up data file first
	Clear();
	int pop, trt, i, j;
	for(trt=0;trt<numTraitTypes_;++trt){
		traitTypesIncluded_.Append(false);}
	fstream dataFile;	
	char *dataLine=NEW char[1001];dataLine[0]='\0';
	aString endCharList(" \t,:\"\'\n\r");
	aString endCharList2("=");
	char* filename=dataFileName->raw();
	
	char endChar = endchr;//defined in StringClass.h
	dataFile.open(filename,ios::in);
	int dataFileOpen = dataFile.is_open();
	if(! dataFileOpen){
		aString errstr("couldn\'t open the file ");
		errstr.cat(filename);
		aError(errstr);
		return 0;}
	dataFile.getline(dataLine, 1000, endChar);
	title_ = dataLine;

	dataFile.getline(dataLine, 1000, endChar); 
	aString parameterLine(dataLine);
	aString *paramWord = parameterLine.ChopOffFirstWord(endCharList);
	aString *paramValWord = paramWord->ChopOffLastWord(endCharList2);
//	paramWord->cut(0, 5);
	npop_ = paramValWord->toInt();
	delete paramWord; delete paramValWord; paramValWord=paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
//	paramWord->cut(0, 6);
	paramValWord = paramWord->ChopOffLastWord(endCharList2);
//	long nloci = paramValWord->toInt();
	ntraits_ = paramValWord->toInt();
	delete paramWord; delete paramValWord; paramValWord=paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
//	paramWord->cut(0, 7);
	paramValWord = paramWord->ChopOffLastWord(endCharList2);
	int maxall_ = paramValWord->toInt();
	delete paramWord; delete paramValWord; paramValWord=paramWord=0;

	paramWord = parameterLine.ChopOffFirstWord(endCharList);
	paramValWord = paramWord->ChopOffLastWord(endCharList2);
//	paramWord->cut(0, 8);
	ngroups_ =  paramValWord->toInt();
	delete paramWord; delete paramValWord; paramValWord=paramWord=0;

	if(parameterLine.len()>11){//ALLELECHAR may be missing 
		paramWord = parameterLine.ChopOffFirstWord(endCharList);
		paramValWord = paramWord->ChopOffLastWord(endCharList2);
//		paramWord->cut(0, 11);
		int alleleLength =  paramValWord->toInt();
		delete paramWord; delete paramValWord; paramValWord=paramWord=0;
		SetAlleleLength(alleleLength);
		}


	//get locus information
	traitName_.Clear();
//	traitName_.Append(NEW aString("ID"));//this is now in the data set
//	ntraits_++;
//	traitType_.Append(0);//for the ID number 
	bool moreTraits = true;
	trt = 0;
	while (moreTraits){
		dataFile.getline(dataLine, 1000, endChar); 
		aString traitLine(dataLine);
		while (traitLine.len() > 0){
			aString *trtName = traitLine.ChopOffFirstWord(endCharList);
			if(trtName->len()>0){//there could be trailing blanks on locusLine
				aString *s = NEW aString(trtName);
				traitName_.Append(s);//append a copy
				traitNum_.Append(trt);
				traitType_.Append(0);//store all as unclassified traits initially
//				ntraits_++;
				++trt;
				}
			delete trtName; trtName=0;
			}//end while locusLine.len()
		if (trt >= ntraits_){//data may be on more than one line
			moreTraits = false;}
		}//end while moreTraits
	if(traitName_.len()>0){ traitTypesIncluded_[0]=true;}
	//read in the number of traits to be skipped in the analyses
	dataFile.getline(dataLine, 1000, endChar); 
	aString skipLine(dataLine);
	aString *skipWord = skipLine.ChopOffFirstWord(endCharList);
	aString *skipWordVal = skipWord->ChopOffLastWord(endCharList2);
//	skipWord->cut(0,5);
	int traitsSkipped = skipWordVal->toInt();
	delete skipWord; delete skipWordVal; skipWordVal=skipWord=0;

	//and read in the names of skipped loci, if any
	trt = 0;
	moreTraits = 0;//false
	if (traitsSkipped > 0){
			moreTraits = true;}
	StringList skippedTraitName;
	while (moreTraits){
		while (skipLine.len() > 0){
			skipWord = skipLine.ChopOffFirstWord(endCharList);
			if (skipWord->len()>0){//there could be trailing blanks
				aString *s = NEW aString(skipWord);
				skippedTraitName.Append(s);//append a copy
				int toSkip = traitName_.Loc(s);
				skippedTraitNum_.Append(toSkip);
				int item = traitNum_.Loc(toSkip);
				traitNum_.Delete(item);
				++trt;
				}
			delete skipWord; skipWord=0;
			}//end while .len()
		if (trt >= traitsSkipped){
				moreTraits = false;}
			else	{//more traits on the next line
				dataFile.getline(dataLine, 1000); 
				skipLine = dataLine;}
		}//end while moreTraits
	skippedTraitNum_.Sort();
	traitNum_.Sort();

	ntraits_=traitNum_.len();//includes ID
//	nloci = ntraits_ - 1;

	//read in population data
	aString popLine;
	aString beast, beastTest;
	ObjList<StringList> rawTraitData;//for internal processing
	unsigned maxTraitStringLen=0;
	for (pop=0;pop<npop_;++pop){
		//skip blank lines before pops in the data file
		int blankLines = 0;
		popLine.clear();
		while (popLine.len() == 0){
			dataFile.getline(dataLine, 1000, endChar); 
			popLine = dataLine;
			++blankLines;
			if (blankLines > 20){
				cout<<"A lot of blank lines in the data set.  Did you set NPOP too high ?"<<endl;}
			}//end while .len()
		aString *s = NEW aString(popLine);
		popName_.Append(s);//append a copy

		//make a new sublist for the genotypes of this population
		rawTraits_.Append(NEW ObjList<TextDataList>());	
		rawTraitData.Append(NEW StringList());
		
		// BEAST is the string for a particular individual
		i=0;
		beast.clear(); beastTest.clear();
		while (beastTest != "NEXT"){ 
			dataFile.getline(dataLine, 1000, endChar); 
			beastTest = beast = dataLine;
			beastTest.CullLeadingWhiteSpace();
			beastTest.CullTrailingWhiteSpace();
			if (beastTest != "NEXT"){
					rawTraits_[pop].Append(NEW TextDataList("?"));//sets missingFlag=?
					aString s(beast);//work on a copy
					aString *idval=s.ChopOffFirstWord();//deletes the between-words space character too
					rawTraits_[pop][i].Append(NEW TextDatum(*idval));
					delete idval;idval=0;
					s.CullTrailingSpaces();//to handle trait-sorting properly below
					rawTraitData[pop].Append(NEW aString(s));//first character is not a space, it's part of the first trait
					++i;
					maxTraitStringLen = MAX(maxTraitStringLen,s.len());//for later
					//parse the genotype string
//					for(int loc=0;loc<nloci;++loc){
//						int ploid = Ploidy(loc);//got cart before horse here -- ploidy depends on classification of inheritance pattern!
//						aString g(s,loc*alleleLen()*ploid,(loc+1)*alleleLen()*ploid);//check this
//						bool allmissing=true;
//						for(int j=0;j<alleleLen()*ploid;++j){
//							if(g[j] != ' '){
//								allmissing=false;break;}}
//						if(allmissing){
//								rawTraits_[pop][i].Append(NEW TextDatum());}
//							else{
//								rawTraits_[pop][i].Append(NEW TextDatum(g));}
//						}//loc	
					}
				else{
					popSize_.Append(rawTraits_[pop].len());}
			}//end while beast..else
		}//end for pop
	dataFile.close();
	delete [] dataLine;dataLine=0;
	delete [] filename;filename=0;
	
	//now process the raw trait data
	//1. get the distribution of non-blank characters along the data strings
	int c;//character along the data string
	aList<int> dataStringCharFrequency;//[site] distribution of number of strings that have a data character at [site]
	for(c=0;c<maxTraitStringLen;++c){
		dataStringCharFrequency.Append(0);}
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<rawTraitData[pop].len();++i){
			for(c=0;c<rawTraitData[pop][i].len();++c){
				char haveALook = rawTraitData[pop][i][c];//handles space-, tab-, or comma-delimited text
				if(!(rawTraitData[pop][i][c]==' ' || rawTraitData[pop][i][c]=='\t'|| rawTraitData[pop][i][c]==',')){
					dataStringCharFrequency[c]++;}
				}}}
	
	//2. use the distribution to identify
	//	a. misaligned data
	aList<int> blankLocations;
	for(c=0;c<maxTraitStringLen;++c){
		if(dataStringCharFrequency[c]==0){
			blankLocations.Append(c);}}
	if(blankLocations.len()<traitName_.len()-2){//no first space, no last space, & id trait not included in blankLocations tally
		cout<<"It looks like there may be misaligned data (or a trait without data). "<<endl;
		cout<<"Starting after the ID numbers, the cumulative distribution of data characters along the data string is "<<endl;
		aString outlinestr;
		for(c=0;c<dataStringCharFrequency.len();++c){
			if(dataStringCharFrequency[c]==0){outlinestr+'0';}
			else{outlinestr+'1';}}
		cout<<endl<<outlinestr<<endl<<endl;
		cout<<"where 0 represents a blank and 1 represents any data character.  A sequence of 1\'s that is too long shows where \
you should look for misalignment.  A sequence of 0\'s is problematic except at the end of the data line.  In a text-editing program, \
use a same-width font like Courier to find the problem(s).  Also check that NTRAITS= matches the list of trait names.  Remember \
to save as a text file when finished editing. "<<endl;
		
		return 0;}
	
	//	b. the trait size of each locus
	blankLocations.Prepend(0);//algorithmic convenience
	aList<int> traitSizes;
//	for(trt=1;trt<traitType_.len();++trt){
	for(trt=0;trt<traitType_.len();++trt){
		traitSizes.Append(0);}//always ignore first trait's size because it varies, but keep place in list to avoid hassles with indexing
	int prevTraitEndPoint=0;
	for(trt=0;trt<traitType_.len();++trt){
//		if(trt>=traitSizes.len()-1) break;//skip trailing spaces
//		traitSizes[trt+1]=blankLocations[trt]-prevTraitEndPoint;//skip the ID trait
		if(trt<blankLocations.len()){
				traitSizes[trt]=blankLocations[trt]-prevTraitEndPoint;//skip the ID trait
				if(trt>0) prevTraitEndPoint=blankLocations[trt]+1;
				}
			else{//last trait
				//blankLocations can be longer if additional data are on the string, but if not thiw works
				traitSizes[trt]=maxTraitStringLen-prevTraitEndPoint;}
		}
	//3. use the blank locations and trait sizes to copy out genotypes
	int startcopy,endcopy;
	bool allmissing;
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<rawTraitData[pop].len();++i){
			aString& sToParse = rawTraitData[pop][i];
			startcopy=0;
			for(trt=1;trt<traitSizes.len();++trt){//skips ID trait in this section
				endcopy = startcopy + traitSizes[trt];
				allmissing=true;
				if(startcopy < sToParse.len()){
						aString g(sToParse,startcopy+1,endcopy);//check this
						int ploid = traitSizes[trt]/alleleLen();
						aString blankstr;
						for(j=0;j<alleleLen();++j){
							blankstr+' ';}
						int startcopy2=0;
						int endcopy2=alleleLen();
						for(j=0;j<ploid;++j){
							aString thisAllele(g,startcopy2+1,endcopy2);//check this
							if(thisAllele != blankstr){
								allmissing=false;break;}
							startcopy2+=alleleLen();
							endcopy2+=alleleLen();
							}
						if(allmissing){
								rawTraits_[pop][i].Append(NEW TextDatum());}
							else{
								rawTraits_[pop][i].Append(NEW TextDatum(g));}
						}
					else{
						rawTraits_[pop][i].Append(NEW TextDatum());
						}
					
				startcopy += traitSizes[trt]+1;//check this
				}//trt	
			}//i
		}//pop
	//4. use the trait size to identify potential haplodiploid/sex-linked loci
	
	GroupMembership();
	LocalityMembership();
	TryToSortTraits();
//	SortTraitsInteractively();
	ParseGenotypicData();
	CheckForNewAlleles();
	return 1;
	}//end MultiformatMarkerData::ReadGelJockDataFile



void MultiformatMarkerData::ReclassifyTraits(int oldTraitType, int newTraitType){
	if(traitTypesIncluded_[oldTraitType]==false) return;
	aList<int> traitsToChange;
	int tr;
	for(tr=0;tr<traitType_.len();++tr){
		if(traitType_[tr]==oldTraitType || traitType_[tr]==newTraitType){
			traitsToChange.Append(tr);}
		}//tr
	SortTraits(newTraitType, traitsToChange);//SortTraits takes all traits of the type, not just the new ones
	}// MultiformatMarkerData::ReclassifyTraits(int , int )


void MultiformatMarkerData::RestoreLoci (aList<int>& lociToRestore){
	//This function shifts trait numbers from from traitNums_ to the relevant skippedTraitNums_ lists
	if(lociToRestore.len()==0){return;}//none to cull
	traitTypeCurrent_=false;
	lociToRestore.Sort();
	int tr,cLoc,datasettype;
	long item;
	//restore them to traitNum_, then to subclasses' lists
	for(tr=lociToRestore.len()-1;tr>=0;--tr){
		cLoc = lociToRestore[tr];//make sure it's a valid step
		if((! traitNum_.Includes(cLoc)) && cLoc>=0 && cLoc<traitName_.len() && skippedTraitNum_.Includes(cLoc)){
			item = skippedTraitNum_.Loc(cLoc);
			traitNum_.Append(cLoc);
			skippedTraitNum_.Delete(item);
			datasettype = traitType_[cLoc];
			switch(datasettype){
			  case hap://haploid
					if(! locusNumHap_.Includes(cLoc)){
						locusNumHap_.Append(cLoc);}
					item = skippedLocusNumHap_.Loc(cLoc);
					if(item>=0){
						skippedLocusNumHap_.Delete(item);}
					break;
			  case codom://codominant
					if(! locusNumCodom_.Includes(cLoc)){
						locusNumCodom_.Append(cLoc);}
					item = skippedLocusNumCodom_.Loc(cLoc);
					if(item>=0){
						skippedLocusNumCodom_.Delete(item);}
					break;
			  case dom://dominant
					if(! locusNumDom_.Includes(cLoc)){
						locusNumDom_.Append(cLoc);}
					item = skippedLocusNumDom_.Loc(cLoc);
					if(item>=0){
						skippedLocusNumDom_.Delete(item);}
					break;
			  case SLcodom://sex-linked codominant
					if(! locusNumSLCodom_.Includes(cLoc)){
						locusNumSLCodom_.Append(cLoc);}
					item = skippedLocusNumSLCodom_.Loc(cLoc);
					if(item>=0){
						skippedLocusNumSLCodom_.Delete(item);}
					break;
			  case SLdom://sex-linked dominant
					if(! locusNumSLDom_.Includes(cLoc)){
						locusNumSLDom_.Append(cLoc);}
					item = skippedLocusNumSLDom_.Loc(cLoc);
					if(item>=0){
						skippedLocusNumSLDom_.Delete(item);}
					break;
			  case CYfemale://cytoplasmic female 
					if(! locusNumCYfemale_.Includes(cLoc)){
						locusNumCYfemale_.Append(cLoc);}
					item = skippedLocusNumCYfemale_.Loc(cLoc);
					if(item>=0){
						skippedLocusNumCYfemale_.Delete(item);}
					if(! CytoplasmicFemaleMarkerData::allPresentInBothSexes_.Includes(cLoc)){
						CytoplasmicFemaleMarkerData::allPresentInBothSexes_.Append(cLoc);}						
					if(! CytoplasmicFemaleMarkerData::presentInBothSexes_.Includes(cLoc)){
						CytoplasmicFemaleMarkerData::presentInBothSexes_.Append(cLoc);}						
					break;
			  case CYfemaleOnly://cytoplasmic female only
					if(! locusNumCYfemale_.Includes(cLoc)){
						locusNumCYfemale_.Append(cLoc);}
					item = skippedLocusNumCYfemale_.Loc(cLoc);
					if(item>=0){
						skippedLocusNumCYfemale_.Delete(item);}
					if(! allPresentInFemalesOnly_.Includes(cLoc)){
						allPresentInFemalesOnly_.Append(cLoc);}						
					if(! presentInFemalesOnly_.Includes(cLoc)){
						presentInFemalesOnly_.Append(cLoc);}						
					break;
			  case CYmale://cytoplasmic male
					if(! locusNumCYmale_.Includes(cLoc)){
						locusNumCYmale_.Append(cLoc);}
					item = skippedLocusNumCYmale_.Loc(cLoc);
					if(item>=0){
						skippedLocusNumCYmale_.Delete(item);}
					if(! CytoplasmicMaleMarkerData::allPresentInBothSexes_.Includes(cLoc)){
						CytoplasmicMaleMarkerData::allPresentInBothSexes_.Append(cLoc);}						
					if(! CytoplasmicMaleMarkerData::presentInBothSexes_.Includes(cLoc)){
						CytoplasmicMaleMarkerData::presentInBothSexes_.Append(cLoc);}						
					break;
			  case CYmaleOnly://cytoplasmic male only
					if(! locusNumCYmale_.Includes(cLoc)){
						locusNumCYmale_.Append(cLoc);}
					item = skippedLocusNumCYmale_.Loc(cLoc);
					if(item>=0){
						skippedLocusNumCYmale_.Delete(item);}
					if(! allPresentInMalesOnly_.Includes(cLoc)){
						allPresentInMalesOnly_.Append(cLoc);}						
					if(! presentInMalesOnly_.Includes(cLoc)){
						presentInMalesOnly_.Append(cLoc);}						
					break;
			  default://not in a sublist
			  		break;
			  }//switch
			}//if item not skipped already
		}//tr
	nlociHap_ = locusNumHap_.len();
	nlociCodom_ = locusNumCodom_.len();
	nlociDom_ = locusNumDom_.len();
	nlociSLCodom_ = locusNumSLCodom_.len();
	nlociSLDom_ = locusNumSLDom_.len();
	nlociCYfemale_ = locusNumCYfemale_.len();
	nlociCYmale_ = locusNumCYmale_.len();
	traitNum_.Sort();
	locusNumHap_.Sort();
	locusNumCodom_.Sort();
	locusNumDom_.Sort();
	locusNumSLCodom_.Sort();
	locusNumSLDom_.Sort();
	locusNumCYfemale_.Sort();
	locusNumCYmale_.Sort();
	allPresentInFemalesOnly_.Sort();
	presentInFemalesOnly_.Sort();
	allPresentInMalesOnly_.Sort();
	presentInMalesOnly_.Sort();
	CytoplasmicFemaleMarkerData::allPresentInBothSexes_.Sort();
	CytoplasmicFemaleMarkerData::presentInBothSexes_.Sort();
	CytoplasmicMaleMarkerData::allPresentInBothSexes_.Sort();
	CytoplasmicMaleMarkerData::presentInBothSexes_.Sort();
	ResetGeneticFlags();
	traitTypeCurrent_=true;
	SortBySex();
	CheckForNewAlleles();
	}//end procedure MultiformatMarkerData::RestoreLoci(aList<int>&)


void MultiformatMarkerData::RestoreLoci (StringList& lociToRestore){
	//This function restores the designated loci to the analysis
	if(lociToRestore.len()==0){return;}//none to cull
	//convert names to numbers and cull
	aList<int> restoreListByNum;
	int tr;
	for(tr=0;tr<lociToRestore.len();++tr){
		aString rTraitname = lociToRestore[tr];
		int item = traitName_.Loc(&rTraitname);
		if((item>=0)&&(!(traitNum_.Includes(item)))){
			//the trait exists in the rawlist && isn't already included
			restoreListByNum.Append(item);
			}//if item exists
		}//tr
	RestoreLoci(restoreListByNum);
	}//end procedure MultiformatMarkerData::RestoreLoci(StringList&)



void MultiformatMarkerData::RestoreLocus (aString& locusToRestore){
	StringList lociToRestore;
	lociToRestore.Append(NEW aString(locusToRestore));
	RestoreLoci(lociToRestore);
	}//end MultiformatMarkerData::RestoreLocus (aString& )


void MultiformatMarkerData::RestoreLocus (char* locusToRestore){
	StringList lociToRestore;
	lociToRestore.Append(NEW aString(locusToRestore));
	RestoreLoci(lociToRestore);
	}//end MultiformatMarkerData::RestoreLocus (char* )


void MultiformatMarkerData::RestoreLocus (int locusToRestore){
	aList<int> lociToRestore;
	lociToRestore.Append(locusToRestore);
	RestoreLoci(lociToRestore);
	}//end MultiformatMarkerData::RestoreLocus (int )




void MultiformatMarkerData::SetGenome(ObjList<PopulationGeneContainer<int> >& pgcList){
	//records all data, passed along from genetic analysis classes
	//empties old data and creates new data structures
	Clear();
	ResetFlags();
	title_ = "Simulated data set";
	npop_ = pgcList.len();
	ngroups_ = 1;
	SetAlleleLength(1);//bug if maxall>9
	int pop,i, trt, tr, trtype, loc;
	int unspecfiedTraits=0;
	StatList<int> numLociSampledByDataType;
	for(trt=0;trt<numTraitTypes_;++trt){
		traitTypesIncluded_.Append(false);
		numLociSampledByDataType.Append(0);
		}
	if(pgcList[0].loci(MultiInheritanceModeContainer<int>::haploid)>0){
		traitTypesIncluded_[hap]=true;
		numLociSampledByDataType[hap]=pgcList[0].loci(MultiInheritanceModeContainer<int>::haploid);
		}
	if(pgcList[0].loci(MultiInheritanceModeContainer<int>::diploid)>0){
		traitTypesIncluded_[codom]=true;
		numLociSampledByDataType[codom]=pgcList[0].loci(MultiInheritanceModeContainer<int>::diploid);
		}
	if(pgcList[0].loci(MultiInheritanceModeContainer<int>::sexLink)>0){
		traitTypesIncluded_[SLcodom]=true;
		numLociSampledByDataType[SLcodom]=pgcList[0].loci(MultiInheritanceModeContainer<int>::sexLink);
		}
	if(pgcList[0].loci(MultiInheritanceModeContainer<int>::cytoFemale)>0){
		traitTypesIncluded_[CYfemale]=true;
		numLociSampledByDataType[CYfemale]=pgcList[0].loci(MultiInheritanceModeContainer<int>::cytoFemale);
		}
	if(pgcList[0].loci(MultiInheritanceModeContainer<int>::cytoMale)>0){
		traitTypesIncluded_[CYmale]=true;
		numLociSampledByDataType[CYmale]=pgcList[0].loci(MultiInheritanceModeContainer<int>::cytoMale);
		}

	ntraits_ = numLociSampledByDataType.sum()+1;
	traitName_.Append(NEW aString("ID"));
	traitType_.Append(ID);
	int runningTraitNum=0;
	traitNum_.Append(runningTraitNum++);
	traitTypesIncluded_[0]=true;
	unspecfiedTraits=1;
	
	if(pgcList[0].sexLinkedDataIncluded()){
		traitTypesIncluded_[sex]=true;
		ntraits_++;
		traitName_.Append(NEW aString("sex"));
		traitType_.Append(sex);
		traitNum_.Append(runningTraitNum++);
		traitTypesIncluded_[0]=true;
		unspecfiedTraits++;
		SetSexTrait();		
		for(pop=0;pop<npop_;++pop){
			homogameticIndivs_.Append(NEW aList<int>());
			heterogameticIndivs_.Append(NEW aList<int>());}
		}
		
	for(tr = 0;tr<ntraits_-unspecfiedTraits;++tr){
		aString locName("LOC"),trStr(tr+1);
		trStr.PrependCharToLen('0',Digits(ntraits_-unspecfiedTraits));
		locName.cat(trStr);
		traitName_.Append(NEW aString(locName));//append a copy
		traitNum_.Append(runningTraitNum++);
		}
	for(trt=hap;trt<numLociSampledByDataType.len();++trt){
		for(tr=0;tr<numLociSampledByDataType[trt];++tr){
			traitType_.Append(trt);
			}}
	
	int locuscount=0;
	nlociHap_=nlociCodom_=nlociDom_=nlociSLCodom_=nlociSLDom_=nlociCYfemale_=nlociCYmale_=0;
	for(trtype=2;trtype<numLociSampledByDataType.len();++trtype){//set up cross-referencing lists
		for(loc=0;loc<numLociSampledByDataType[trtype];++loc){
			switch(trtype){
			  case hap://haploid loci
				allHapLoci_.Append(locuscount+unspecfiedTraits);
				locusNumHap_.Append(locuscount+unspecfiedTraits);
				nlociHap_++;
				break;
			  case codom://codominant loci
				allCodomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCodom_.Append(locuscount+unspecfiedTraits);
				nlociCodom_++;
				break;
			  case dom://dominant loci
				allDomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumDom_.Append(locuscount+unspecfiedTraits);
				nlociDom_++;
				break;
			  case SLcodom://codominant sex-linked loci
				allSLCodomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumSLCodom_.Append(locuscount+unspecfiedTraits);
				nlociSLCodom_++;
				break;
			  case SLdom://dominant sex-linked loci
				allSLDomLoci_.Append(locuscount+unspecfiedTraits);
				locusNumSLDom_.Append(locuscount+unspecfiedTraits);
				nlociSLDom_++;
				break;
			  case CYfemale://cytoplasmic female loci
				allCYfemaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYfemale_.Append(locuscount+unspecfiedTraits);
				nlociCYfemale_++;
				CytoplasmicFemaleMarkerData::allPresentInBothSexes_.Append(locuscount+unspecfiedTraits);
				CytoplasmicFemaleMarkerData::presentInBothSexes_.Append(locuscount+unspecfiedTraits);
				break;
			  case CYfemaleOnly://cytoplasmic female loci
				allCYfemaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYfemale_.Append(locuscount+unspecfiedTraits);
				allPresentInFemalesOnly_.Append(locuscount+unspecfiedTraits);
				presentInFemalesOnly_.Append(locuscount+unspecfiedTraits);
				nlociCYfemale_++;
				break;
			  case CYmale://cytoplasmic male loci
				allCYmaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYmale_.Append(locuscount+unspecfiedTraits);
				nlociCYmale_++;
				CytoplasmicMaleMarkerData::allPresentInBothSexes_.Append(locuscount+unspecfiedTraits);
				CytoplasmicMaleMarkerData::presentInBothSexes_.Append(locuscount+unspecfiedTraits);
				break;
			  case CYmaleOnly://cytoplasmic male loci
				allCYmaleLoci_.Append(locuscount+unspecfiedTraits);
				locusNumCYmale_.Append(locuscount+unspecfiedTraits);
				allPresentInMalesOnly_.Append(locuscount+unspecfiedTraits);
				presentInMalesOnly_.Append(locuscount+unspecfiedTraits);
				nlociCYmale_++;
				break;
			  default: break;
			  }//switch
			 ++locuscount;
			 }//loc
			}//trtype

	//initialize data field
	
	for (pop=0;pop<npop_; ++pop){
		// NAME THE POPULATION
		aString pName("POP");
		aString pdigit(pop+1);
		pdigit.PrependCharToLen('0',Digits(npop_));
		pName.cat(pdigit);
		pName.cat(": randomly generated population # ");
		pName.cat(pop+1);pName.cat("(");pName.cat(pop+1);
		pName.cat(") group #1");
		aString *s = NEW aString(pName);
		popName_.Append(s);//append a copy

		//create dummy id numbers for each individual
		rawTraits_.Append(NEW ObjList<TextDataList>());	

		for (i=0;i<pgcList[pop].popSize();++i){
			rawTraits_[pop].Append(NEW TextDataList());
			rawTraits_[pop][i].Append(NEW TextDatum());}//empty ID
		popSize_.Append(pgcList[pop].popSize());

		//determine sex, if sex-linked
		if(pgcList[pop].sexLinkedDataIncluded()){
			rawTraits_[pop][i].Append(NEW TextDatum(" "));}//empty sexTrait

/*	handled in ParseGenotypicData()
		if(numLociSampledByDataType[codom]>0){//codominant loci
			rawCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
			for (i=0;i<pgcList[pop].popSize();++i){
				rawCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				}//i
			}
		if(numLociSampledByDataType[SLcodom]>0){//sex-linked codominant loci
			rawSLCodomGtypes_.Append(NEW ObjList<ObjList<TextDataList> >());
			for (i=0;i<pgcList[pop].popSize();++i){
				rawSLCodomGtypes_[pop].Append(NEW ObjList<TextDataList>());
				}//i
			}
*/
		
		//create and append dummy genotypes
		locuscount=0;
		for(trtype=hap;trtype<numLociSampledByDataType.len();++trtype){//set up cross-referencing lists
			for(loc=0;loc<numLociSampledByDataType[trtype];++loc){
					switch(trtype){
					  case hap://haploid loci
						for(i=0;i<pgcList[pop].popSize();++i){
							rawTraits_[pop][i].Append(NEW TextDatum(" "));
							}//i
						break;
					  case codom://codominant loci
						for(i=0;i<pgcList[pop].popSize();++i){
							rawTraits_[pop][i].Append(NEW TextDatum("  "));
							}//i
						break;
					  case SLcodom://codominant sex-linked loci
						for(i=0;i<pgcList[pop].popSize();++i){
							rawTraits_[pop][i].Append(NEW TextDatum("  "));//even heterogametics are scored here with missing data slots
							}//i
						break;
					  case CYfemale://cytoplasmic female loci
					  case CYfemaleOnly://cytoplasmic female loci
						for(i=0;i<pgcList[pop].popSize();++i){
							rawTraits_[pop][i].Append(NEW TextDatum(" "));
							}//i
						break;
					  case CYmale://cytoplasmic male loci
					  case CYmaleOnly://cytoplasmic male loci
						for(i=0;i<pgcList[pop].popSize();++i){
							rawTraits_[pop][i].Append(NEW TextDatum(" "));
							}//i
						break;
					  default: break;
					  }//switch
				++locuscount;
				 }//loc
			}//trtype
		}//end for pop	
	
	//Fill up with data
	for(pop=0;pop<npop_;++pop){
		for(i=0;i<popSize(pop);++i){
			SetGenome(pgcList[pop][i], pop, i);}
		}
	DetermineFemaleOnlyTraits();
	DetermineMaleOnlyTraits();
	GroupMembership();
	LocalityMembership();
	ParseGenotypicData();
	CheckForNewAlleles();	
	}//MultiformatMarkerData::SetGenome(ObjList<PopulationGeneContainer<int> >&)



void MultiformatMarkerData::SetGenome(GeneContainerCore& gc, int pop, int indiv){
	//records data from MultiInheritanceModeContainer<int>
	MultiInheritanceModeContainer<int>* mimc = (MultiInheritanceModeContainer<int>*) &gc;
	rawTraits_[pop][indiv][0] = mimc->ID();
	if(mimc->haploidDataIncluded()){
		HaploidMarkerData::SetGenome(mimc->inheritanceSet_[MultiInheritanceModeContainer<int>::haploid],pop,indiv);}
	if(mimc->codominantDataIncluded()){
		CodominantMarkerData::SetGenome(mimc->inheritanceSet_[MultiInheritanceModeContainer<int>::diploid],pop,indiv);}
	if(mimc->sexDataIncluded()){
		SexLinkedData::SetGenome(gc,pop,indiv);}
	if(mimc->cytoFemaleDataIncluded()){
		CytoplasmicFemaleMarkerData::SetGenome(mimc->inheritanceSet_[MultiInheritanceModeContainer<int>::cytoFemale],pop,indiv);}
	if(mimc->cytoMaleDataIncluded()){
		CytoplasmicMaleMarkerData::SetGenome(mimc->inheritanceSet_[MultiInheritanceModeContainer<int>::cytoMale],pop,indiv);}
	if(mimc->sexLinkedDataIncluded()){
		SexLinkedCodomMarkerData::SetGenome(mimc->inheritanceSet_[MultiInheritanceModeContainer<int>::sexLink],pop,indiv);}
	DetermineFemaleOnlyTraits();
	DetermineMaleOnlyTraits();
	}//MultiformatMarkerData::SetGenome(GeneContainerCore&, int, int)



void MultiformatMarkerData::SnipArlequinComment(aString& arqStr){
	const char pound='#';
	int compos = arqStr.pos(pound);
	if(compos==0 && arqStr[0] != pound){//no comment found
		return;}
	if(arqStr.len()==1 && arqStr[0] == pound){
		arqStr.clear();return;
		}
	arqStr.trunc(compos-1);
	}//end MultiformatMarkerData::SnipArlequinComment(aString&)




void MultiformatMarkerData::SortTraits(int traitType, aList<int>& traitList){
	//listed traits are assigned to traitType

	int tr, trt;
	aList<int> traitListCopy(traitList);//work with a copy
	aList<int> oldTraitType(traitType_),traitCountList;
	ObjList<aList<int> > sortedLoci, lociToClear;

	for(tr=traitList.len()-1;tr>=0;--tr){//work backwards
		if(traitType_[traitList[tr]] == traitType){
				traitListCopy.Delete(tr);//already has the correct trait type
				}//already has the correct trait type
			else{
				traitType_[traitList[tr]] = traitType;//change the type
				}
		}//tr
	if(traitListCopy.len()==0){ return;}//all have corrrect type
	//some must have changed; re-sort all
	ResetFlags();
	for(trt=0;trt<traitTypesIncluded_.len();++trt){//set up arrays
		traitCountList.Append(0);
		sortedLoci.Append(NEW aList<int>());
		lociToClear.Append(NEW aList<int>());
		}
	for(tr=0;tr<traitType_.len();++tr){//count traits
		traitCountList[traitType_[tr]]++;//counts by type
		sortedLoci[traitType_[tr]].Append(tr);//holds traits by their type
		if(traitType_[tr] != oldTraitType[tr]){
			lociToClear[oldTraitType[tr]].Append(tr);
			}
		}
	for(trt=0;trt<traitTypesIncluded_.len();++trt){
		traitTypesIncluded_[trt]=false;
		if(traitCountList[trt]>0){
			traitTypesIncluded_[trt]=true;}
		}//trt

	//go through and resort traits in subclasses
//	traitList.Print(cout," ");
	for(trt=0;trt<traitTypesIncluded_.len();++trt){
		switch(trt){
			case hap://haploid
//				lociToClear[trt].Print(cout," ");
//				sortedLoci[trt].Print(cout," ");
				HaploidMarkerData::ExpungeLoci(lociToClear[trt]);
				HaploidMarkerData::SetLoci(sortedLoci[trt]);
				break;
			case codom://codominant
				CodominantMarkerData::ExpungeLoci(lociToClear[trt]);
				CodominantMarkerData::SetLoci(sortedLoci[trt]);
				break;
			case dom://dominant
				DominantMarkerData2::ExpungeLoci(lociToClear[trt]);
				DominantMarkerData2::SetLoci(sortedLoci[trt]);
				break;
			case SLcodom://sex-linked codominant						
				SexLinkedCodomMarkerData::ExpungeLoci(lociToClear[trt]);
				SexLinkedCodomMarkerData::SetLoci(sortedLoci[trt]);
				break;
			case SLdom://sex-linked dominant
				SexLinkedDomMarkerData2::ExpungeLoci(lociToClear[trt]);
				SexLinkedDomMarkerData2::SetLoci(sortedLoci[trt]);
				break;
			case CYfemale://female cytoplasmic
			case CYfemaleOnly://female cytoplasmic
				CytoplasmicFemaleMarkerData::ExpungeLoci(lociToClear[trt]);
				CytoplasmicFemaleMarkerData::SetLoci(sortedLoci[trt]);
				break;
			case CYmale://male cytoplasmic
			case CYmaleOnly://male cytoplasmic
				CytoplasmicMaleMarkerData::ExpungeLoci(lociToClear[trt]);
				CytoplasmicMaleMarkerData::SetLoci(sortedLoci[trt]);
				break;
			default:
				break;
			}//switch
		}//trt	
	}//MultiformatMarkerData::SortTraits(int, aList<int>&)



void MultiformatMarkerData::SortTraitsInteractively(ObjList<aList<int> >& presortedTraits){
	//confirms previous classification with user
	//presortedTraits in order of 0=haploid+cytoplasmic female+cytoplasmic male+dominant+sex-linked dominant;  1=codominant; 2=sex-linked codominant; 3=cytoplasmic male-inherited
	bool OK_=false, keepTrying=true, badEntry=false, sortingTraits=true;
//	char yn;
	int pop, i, tr, trt, trInList,/*loc,*/ startnum, endnum;
	char *inputline = NEW char[1001];inputline[0]='\0';
	aString inData;
	bool editByName=true,editByNumber=true;//default number
	ObjList<aList<int> > traitTypeClassification = presortedTraits;
	aList<int> lociPresentInFemalesOnly, lociPresentInMalesOnly;
	cout<<endl<<"From the arrangement of the data and some guesswork, traits have been preliminarily sorted.  In this step, you have a chance to edit these categories."<<endl;
	while(! OK_){
		cout<<"Do you prefer to display categories by trait name, or by trait number (following the order in the data file)?"<<endl;
		cout<<"Edit by ?=   [name, number, both]"<<endl;
		cin.getline(inputline,1001);
		inputline[1000]='\0';
		inData=inputline;
		if (inData == "NAME" || inData == "name"||inData == "NA" || inData == "na"){
				editByName=OK_=true;
				editByNumber=false;}
			else if (inData == "NUMBER" || inData == "number" || inData == "NO" || inData == "no" || inData == "No"
										|| inData == "NU" || inData == "nu" || inData == "Nu"){
				editByName=false; 
				editByNumber=OK_=true;}
//			else if (inData == "BOTH" || inData == "both" ||inData=="b"||inData=="B"){
			else if (inData[0] == 'B' || inData[0] == 'b'){
				editByName=editByNumber=OK_=true;}
			else{
				cout<<"You answered "<<inData<<".  Please answer \'name\', \'number\', or \both\'."<<endl<<endl; OK_=false;}
		}
	OK_=false;
//	int sexcolumn;
	aString inputParse1(" ,\t");
	aString inputParse2("-");
	aString choiceTag;
	StringList reclassifyList;
	aList<int> reclassifyListByNum;
	bool maleHeterogametic=true, dataConflict=false, firstone=true, dataScanDone=false;
	int ctrt;

//	const int CYfemaleOnly=numTraitTypes_, CYmaleOnly=numTraitTypes_+1;
	while( ! OK_){
		while(sortingTraits){
			badEntry=false;
			cout<<"Your data are sorted into the following categories.  To edit, choose the category and enter new values.  Resize the window if \
needed to see the whole list."<<endl<<endl;
			for(trt=0;trt<traitTypeClassification.len();++trt){//print the listing
				switch(trt){
					case ID:
						cout<<" 0. ID and unclassified traits:"<<endl; 
						break;
					case sex:
						cout<<" 1. Sex identification trait:"<<endl; 
						break;
					case hap:
						cout<<" 2. Haploid inheritance (nuclear loci):"<<endl; 
						break;
					case codom:
						cout<<" 3. Codominant inheritance:"<<endl; 
						break;
					case dom:
						cout<<" 4. Dominant inheritance:"<<endl; 
						break;
					case SLcodom:
						cout<<" 5. Codominant sex-linked or haplodiploid inheritance:"<<endl; 
						break;
					case SLdom:
						cout<<" 6. Dominant sex-linked or haplodiploid inheritance:"<<endl; 
						break;
					case CYfemale:
						cout<<" 7. Cytoplasmic female-transmitted inheritance (but found in both sexes):"<<endl; 
						break;
					case CYmale:
						cout<<" 8. Cytoplasmic male-transmitted inheritance (but found in both sexes):"<<endl; 
						break;
					case CYfemaleOnly:
						cout<<" 9. Cytoplasmic female-transmitted inheritance (found in females only):"<<endl; 
						break;
					case CYmaleOnly:
						cout<<"10. Cytoplasmic male-transmitted inheritance (found in males only):"<<endl; 
						break;
		//			case 11:
		//				cout<<"11. Quantitative (continuous) traits"<<endl; 
		//				break;
		//			case 12:
		//				cout<<"12. Quantitative (continuous) threshold traits"<<endl; 
		//				break;
					default:
						break;
					}//switch trt
				//print the traits
				if(editByName && ! editByNumber){
						for(tr=0;tr<traitTypeClassification[trt].len();++tr){
							if(tr>0) {cout<<", ";}else{cout<<"    ";}
							cout<<traitName_[traitTypeClassification[trt][tr]];}
						}
					else if(editByNumber && ! editByName){
						for(tr=0;tr<traitTypeClassification[trt].len();++tr){
							if(tr>0) {cout<<", ";}else{cout<<"    ";}//add code for sequences like 5-10
							cout<<traitTypeClassification[trt][tr]+1;}
						}
					else {//both
						for(tr=0;tr<traitTypeClassification[trt].len();++tr){
							if(tr>0) {cout<<", ";}else{cout<<"    ";}//add code for sequences like 5-10
							cout<<traitTypeClassification[trt][tr]+1<<"("<<traitName_[traitTypeClassification[trt][tr]]<<")";}
						}
				if(traitTypeClassification[trt].len()==0){
					cout<<"    none";}
				cout<<endl;
				}//trt  print out
			//pick a location to move to
			cout<<"Location to sort a trait into =?  [0-"<<traitTypeClassification.len()-1<<", done]"<<endl;
			cin.getline(inputline,1001);
			inputline[1000]='\0';
			inData=inputline;
			inData.CullLeadingWhiteSpace();
			inData.CullTrailingWhiteSpace();
			if(inData=="DONE" || inData=="done"||inData=="d"||inData=="D"){
				sortingTraits=false;
				continue;}
			ResetFlags();
			traitTypeCurrent_=false;
			int choice=-1; 			
			if(inData.isNumber()){
					choice = inData.toInt();
					if(choice<0 || choice>CYmaleOnly){
						badEntry=true;}
					}
				else{//illegal value
					OK_=false; 
					badEntry=true;
					continue;}
			if(badEntry){
				cout<<"ERROR: You entered \""<<inData<<"\", which is not one of the choices.  Please try again."<<endl<<endl;
				continue;}


				cout<<"You can type in the names or the ordinal numbers of these traits. Type in names exactly as they appear in the data file. \
If entering more than one name, separate them with commas or white space (e.g., \"trait1,trait3\").  If \
entering numbers, separate them with commas or white space too; you can enter a sequence of numbers using a hyphen with no white space \
(e.g., \"1-3,5,8\").  Don\'t mix trait names and numbers."<<endl<<endl;
			reclassifyListByNum.Clear();
			reclassifyList.Clear();
			if(choice>=0){
				cout<<"Sorting traits into:  ("<<choice<<") ";
				switch (choice){
					case ID:
						choiceTag="ID and unclassified traits"; 
						break;
					case sex:
						choiceTag="Sex identification trait"; 
						break;
					case hap:
						choiceTag="Haploid inheritance (nuclear loci)";
						break;
					case codom:
						choiceTag="Codominant inheritance";
						break;
					case dom:
						choiceTag="Dominant inheritance";
						break;
					case SLcodom:
						choiceTag="Codominant sex-linked or haplodiploid inheritance";
						break;
					case SLdom:
						choiceTag="Dominant sex-linked or haplodiploid inheritance";
						break;
					case CYfemale:
						choiceTag="Cytoplasmic female-transmitted inheritance (but found in both sexes)";
						break;
					case CYmale:
						choiceTag="Cytoplasmic male-transmitted inheritance (but found in both sexes)"; 
						break;
					case CYfemaleOnly:
						choiceTag="Cytoplasmic female-transmitted inheritance (found in females only)";
						break;
					case CYmaleOnly:
						choiceTag="Cytoplasmic male-transmitted inheritance (found in males only)"; 
						break;
		//			case 11:
		//				choiceTag="Quantitative (continuous) traits";
		//				break;
		//			case 12:
		//				choiceTag="Quantitative (continuous) threshold traits";
		//				break;
					default:
						badEntry=true;
						break;
					}//switch choice
				cout<<choiceTag<<endl;
				cout<<"? trait names or numbers = "<<endl;
				cin.getline(inputline,1001);
				inputline[1000]='\0';
				aString traitList(inputline);
				while(traitList.len()>0){
					aString *traitToReclassify = traitList.ChopOffFirstWord(inputParse1);
					//determine if this is a number or a name
					aString testval((*traitToReclassify)[0]);
					if(testval.isNumber()){
							startnum=0,endnum=0;
							//check for hyphenated values
							if(traitToReclassify->pos(inputParse2)>0){
								aString *endNumStr = traitToReclassify->ChopOffLastWord(inputParse2);
								endnum = endNumStr->toInt()-1;
//								endnum = endNumStr->toInt();
								delete endNumStr; endNumStr=0;
								}//0-indexing
							startnum=traitToReclassify->toInt()-1;//0-indexing
//							startnum=traitToReclassify->toInt();
							endnum=MAX(startnum,endnum);
						 	for(ctrt=startnum;ctrt<=endnum;++ctrt){
								reclassifyListByNum.Append(ctrt);}
							}//culling by number
						else{//reclassify by name
							if(traitName_.Includes(traitToReclassify)){
									if(!(reclassifyList.Includes(traitToReclassify))){
											reclassifyList.Append(NEW aString(traitToReclassify));
											int traitLoc = traitName_.Loc(traitToReclassify);
											reclassifyListByNum.Append(traitLoc);
											}
										else{}}
								else{
									cout<<"ERROR:  The trait named "<<traitToReclassify<<" isn\'t in the current set of traits.  Maybe you misspelled it?"<<endl;}
							}//reclassify by name
					delete traitToReclassify; traitToReclassify=0;
					}//while traitList.len()
				//put it in the new location
				//remove it from the input list
				reclassifyListByNum.Sort();
				for(tr=reclassifyListByNum.len()-1;tr>=0;tr--){
					//check to make sure it's movable and not already there
					if(reclassifyListByNum[tr]==0){
							cout<<"ERROR:  The trait \'ID\' has to remain in category 0."<<endl<<endl;}
						else{//not moving ID trait
							if(!(traitTypeClassification[choice].Includes(reclassifyListByNum[tr]))){//skip if already there
								for(trt=0;trt<traitTypeClassification.len();++trt){
									if(trt==choice) continue;
									if(traitTypeClassification[trt].Includes(reclassifyListByNum[tr])){
										int tr3=traitTypeClassification[trt].Loc(reclassifyListByNum[tr]);
										traitTypeClassification[trt].Delete(tr3);
										if(choice==1){//sex trait allowed only one entry, so remove the others
											for(tr3=traitTypeClassification[choice].len()-1;tr3>=0;tr3--){
												traitTypeClassification[0].Append(traitTypeClassification[choice][tr3]);
												traitTypeClassification[choice].DeleteLast();}}
										traitTypeClassification[choice].Append(reclassifyListByNum[tr]);
										break;
										}}
								}//! includes
							}//not moving ID trait
					reclassifyListByNum.DeleteLast();
					traitTypeClassification[choice].Sort();
					}//tr
				}//choice>=0
			}//sortingTraits

		//scan for heterogametic sex info
		TextDatum geneset;
		if(traitTypeClassification[1].len()==0){
				OK_=true;}
			else{//it has a sex trait
				sexTrait_=traitTypeClassification[1][0];
				for(trInList=0;trInList<traitTypeClassification[5].len();++trInList){//scan genotypes
				maleHeterogametic= firstone=true;
				dataConflict = dataScanDone=false;
				tr = traitTypeClassification[5][trInList];
				for(pop=0;pop<npop_;++pop){
					for(i=0;i<individuals(pop);++i){
						TextDatum& sexslc = rawTraits_[pop][i][sexTrait_];
						if(sexslc.OK()){//skips unscored individuals
							geneset = rawTraits_[pop][i][tr];//copy
							if(geneset.OK()){
								dataScanDone=true;
								geneset.CullLeadingSpaces();
								geneset.CullTrailingSpaces();
								if(geneset.len()==SexLinkedCodomMarkerData::alleleLen()){//it's heterogametic
										if(sexslc=="M" || sexslc=="m"){
												if(! maleHeterogametic){
													dataConflict=true;break;}
												}//m
											else if(sexslc=="F" || sexslc=="f"){
												if(firstone){
														maleHeterogametic=false;}
													else if (maleHeterogametic){
														dataConflict=true;break;}
													else{}
												}//f
										}//heterogametic
									else{//it's homogametic
										if(sexslc=="M" || sexslc=="m"){
												if(firstone){
														maleHeterogametic=false;}
													else if(maleHeterogametic){
														dataConflict=true;break;}
													else{}
												}//m
											else if(sexslc=="F" || sexslc=="f"){
												if(!maleHeterogametic){
													dataConflict=true; break;}
												}//f
										}//homogametic
								firstone=false;
								}//gene OK
							}//sex OK
						}//i
					if(dataConflict) break;
					}//pop
					
					if(dataConflict){
							cout<<"Trait #"<<traitTypeClassification[SLcodom][trInList]<<" ("<<traitName_[traitTypeClassification[SLcodom][trInList]]<<") is classified as \
	sex-linked (haplodiploid) codominant, but the genotypes of some individuals show the wrong ploidy level.  Please sort this out before analysis."<<endl;}
					}//trInList
				//cytoplasmic female
				for(trInList=0;trInList<traitTypeClassification[CYfemaleOnly].len();++trInList){//scan genotypes
					tr = traitTypeClassification[CYfemaleOnly][trInList];
					for(pop=0;pop<npop_;++pop){
						for(i=0;i<individuals(pop);++i){
							TextDatum& sexcyf = rawTraits_[pop][i][sexTrait_];
							if(sexcyf.OK()){//skips unscored individuals
								geneset = rawTraits_[pop][i][tr];//copy
								if((sexcyf=="M" || sexcyf=="m") && geneset.OK()){//present in a male
									dataConflict=true;break;}
								firstone=false;
								}//sex OK
							}//i
						if(dataConflict) break;
						}//pop
					
					if(dataConflict){
							cout<<"Trait #"<<traitTypeClassification[CYfemaleOnly][trInList]<<" ("<<traitName_[traitTypeClassification[CYfemaleOnly][trInList]]<<") is classified as \
	cytoplasmic & female-only, but the genotypes of some individuals are present in males.  Please sort this out before analysis."<<endl;}
					}//trInList
				//cytoplasmic male
				maleHeterogametic= firstone=true;
				dataConflict = dataScanDone=false;
				for(trInList=0;trInList<traitTypeClassification[CYmaleOnly].len();++trInList){//scan genotypes
					tr = traitTypeClassification[CYmaleOnly][trInList];
					for(pop=0;pop<npop_;++pop){
						for(i=0;i<individuals(pop);++i){
							TextDatum& sexcym = rawTraits_[pop][i][sexTrait_];
							if(sexcym.OK()){//skips unscored individuals
								geneset = rawTraits_[pop][i][tr];//copy
								if((sexcym=="F" || sexcym=="f") && geneset.OK()){//present in a female
									dataConflict=true;break;}
								firstone=false;
								}//sex OK
							}//i
						if(dataConflict) break;
						}//pop
					
					if(dataConflict){
							cout<<"Trait #"<<traitTypeClassification[CYmaleOnly][trInList]<<" ("<<traitName_[traitTypeClassification[CYmaleOnly][trInList]]<<") is classified as \
	cytoplasmic & male-only, but the genotypes of some individuals are present in females.  Please sort this out before analysis."<<endl;}
					}//trInList
				if(! dataConflict) {OK_=true;}
				if(dataScanDone){
						cout<<"From sifting through the sex-linked (haplodiploid) codominant data, the heterogametic sex appears to be ";
						if(maleHeterogametic){cout<<"MALE.";}else{cout<<"FEMALE.";}
						cout<<endl<<"If this is wrong, please check for scoring inconsistencies before analysis."<<endl<<endl;}
					else{//just have to specify it
						bool specifyingHetSex=true;
						while(specifyingHetSex){
							cout<<"You need to specify the heterogametic sex."<<endl;
							cout<<"  heterogametic sex =?  [M or F]"<<endl;
							cin.getline(inputline,1001);
							inputline[1]='\0';//truncate
							inData=inputline;
							if(inData=="M" || inData=="m"){
									maleHeterogametic=true;
									specifyingHetSex=false;}
								else if(inData=="F" || inData=="f"){
									maleHeterogametic=specifyingHetSex=false;}
								else{
									specifyingHetSex=false;
									cout<<"ERROR: you answered "<<inData<<", which is not M or F.  Please try again."<<endl<<endl;}
							}//specifyingHetSex
						}
				if(maleHeterogametic){
						heterogameticSex_= M;}
					else{
						heterogameticSex_= F;}
				}//has a sex trait
								
		}//while ! OK
	//now take this information and update the datalists
	int trtype2;
	for(trt=hap;trt<traitTypeClassification.len();++trt){
		for(trtype2=hap;trtype2<traitTypeClassification.len();++trtype2){
			RemoveTraits(trt,traitTypeClassification[trtype2]);
			}}
	for(trt=hap;trt<traitTypeClassification.len();++trt){
		AddTraits(trt,traitTypeClassification[trt]);}
	traitTypeCurrent_=true;
	
/*	nlociHap_ = traitTypeClassification[hap].len();
	allHapLoci_ = locusNumHap_ = traitTypeClassification[hap];
	nlociCodom_ = traitTypeClassification[codom].len();
	allCodomLoci_ = locusNumCodom_ = traitTypeClassification[codom];
	nlociDom_ = traitTypeClassification[dom].len();
	allDomLoci_ = locusNumDom_ = traitTypeClassification[dom];
	nlociSLCodom_ = traitTypeClassification[SLcodom].len();
	allSLCodomLoci_ = locusNumSLCodom_ = traitTypeClassification[SLcodom];
	nlociSLDom_ = traitTypeClassification[SLdom].len();
	allSLDomLoci_ = locusNumSLDom_ = traitTypeClassification[SLdom];
	nlociCYfemale_ = traitTypeClassification[CYfemale].len() + traitTypeClassification[CYfemaleOnly].len();
	allCYfemaleLoci_ = locusNumCYfemale_ = traitTypeClassification[CYfemale];
	for(loc=0;loc<traitTypeClassification[CYfemaleOnly].len();++loc){
		allCYfemaleLoci_.Append(traitTypeClassification[CYfemaleOnly][loc]);
		locusNumCYfemale_.Append(traitTypeClassification[CYfemaleOnly][loc]);
		}
	nlociCYmale_ = traitTypeClassification[CYmale].len() + traitTypeClassification[CYmaleOnly].len();
	allCYmaleLoci_ = locusNumCYmale_ = traitTypeClassification[CYmale];
	for(loc=0;loc<traitTypeClassification[CYmaleOnly].len();++loc){
		allCYmaleLoci_.Append(traitTypeClassification[CYmaleOnly][loc]);
		locusNumCYmale_.Append(traitTypeClassification[CYmaleOnly][loc]);
		}
	presentInFemalesOnly_.Clear();
	presentInMalesOnly_.Clear();
	for(trt=0;trt<traitTypeClassification.len();++trt){
		if(trt<traitTypeClassification.len()-2){//not sex-limited
				if(traitTypeClassification[trt].len()>0){
						traitTypesIncluded_[trt]=true;}
					else{ traitTypesIncluded_[trt]=false;}
				for(tr=0;tr<traitTypeClassification[trt].len();++tr){
					traitType_[traitTypeClassification[trt][tr]] = trt;}
				}
			else{//sex-limited traits handled differently
				if(traitTypeClassification[trt].len()>0){
						traitTypesIncluded_[trt-2]=true;
						}
				for(tr=0;tr<traitTypeClassification[trt].len();++tr){
					traitType_[traitTypeClassification[trt][tr]] = trt-2;
					if(trt==CYfemaleOnly){
						presentInFemalesOnly_.Append(traitTypeClassification[trt][tr]);}
					if(trt==CYmaleOnly){
						presentInMalesOnly_.Append(traitTypeClassification[trt][tr]);}
					}
				}
			}//trt
*/
	delete [] inputline; inputline=0;
	
	}//MultiformatMarkerData::SortTraitsInteractively()



void MultiformatMarkerData::RemoveTraits(int traitType, aList<int>& traitNumbers){//handle sex and ID separately
//	const int CYfemaleOnly=numTraitTypes_, CYmaleOnly=numTraitTypes_+1;
	if(traitNumbers.len()==0) return;
	int tr,trnum,trloc;
	for(tr=0;tr<traitNumbers.len();++tr){
		trnum = traitNumbers[tr];
		traitType_[trnum]=0;//unclassified by default
		switch(traitType){
			case hap:
				if(allHapLoci_.Includes(trnum)){
					allelesCurrentHap_=false;
					trloc = allHapLoci_.Loc(trnum);
					allHapLoci_.Delete(trloc);
					if(skippedTraitNum_.Includes(trnum)){
							trloc = skippedLocusNumHap_.Loc(trnum);
							skippedLocusNumHap_.Delete(trloc);}
						else{
							trloc = locusNumHap_.Loc(trnum);
							locusNumHap_.Delete(trloc);}
					nlociHap_ = locusNumHap_.len();
					if(allHapLoci_.len()==0){
						traitTypesIncluded_[hap]=false;}
					}
				break;
			case codom: 
				if(allCodomLoci_.Includes(trnum)){
					allelesCurrent_=false;
					trloc = allCodomLoci_.Loc(trnum);
					allCodomLoci_.Delete(trloc);
					if(skippedTraitNum_.Includes(trnum)){
							trloc = skippedLocusNumCodom_.Loc(trnum);
							skippedLocusNumCodom_.Delete(trloc);}
						else{
							trloc = locusNumCodom_.Loc(trnum);
							locusNumCodom_.Delete(trloc);}
					nlociCodom_ = locusNumCodom_.len();					
					if(allCodomLoci_.len()==0){
						traitTypesIncluded_[codom]=false;}
					}
				break;
			case dom: 
				if(allDomLoci_.Includes(trnum)){
					allelesCurrentDom_=false;
					trloc = allDomLoci_.Loc(trnum);
					allDomLoci_.Delete(trloc);
					if(skippedTraitNum_.Includes(trnum)){
							trloc = skippedLocusNumDom_.Loc(trnum);
							skippedLocusNumDom_.Delete(trloc);}
						else{
							trloc = locusNumDom_.Loc(trnum);
							locusNumDom_.Delete(trloc);}
					nlociDom_ = locusNumDom_.len();					
					if(allDomLoci_.len()==0){
						traitTypesIncluded_[dom]=false;}
					}
				break;
			case SLcodom: 
				if(allSLCodomLoci_.Includes(trnum)){
					allelesCurrentSLCodom_=false;
					trloc = allSLCodomLoci_.Loc(trnum);
					allSLCodomLoci_.Delete(trloc);
					if(skippedTraitNum_.Includes(trnum)){
							trloc = skippedLocusNumSLCodom_.Loc(trnum);
							skippedLocusNumSLCodom_.Delete(trloc);}
						else{
							trloc = locusNumSLCodom_.Loc(trnum);
							locusNumSLCodom_.Delete(trloc);}
					nlociSLCodom_ = locusNumSLCodom_.len();					
					if(allSLCodomLoci_.len()==0){
						traitTypesIncluded_[SLcodom]=false;}
					}
				break;
			case SLdom: 
				if(allSLDomLoci_.Includes(trnum)){
					allelesCurrentSLDom_=false;
					trloc = allSLDomLoci_.Loc(trnum);
					allSLDomLoci_.Delete(trloc);
					if(skippedTraitNum_.Includes(trnum)){
							trloc = skippedLocusNumSLDom_.Loc(trnum);
							skippedLocusNumSLDom_.Delete(trloc);}
						else{
							trloc = locusNumSLDom_.Loc(trnum);
							locusNumSLDom_.Delete(trloc);}
					nlociSLDom_ = locusNumSLDom_.len();					
					if(allSLDomLoci_.len()==0){
						traitTypesIncluded_[SLdom]=false;}
					}
				break;
			case CYfemale: 
			case CYfemaleOnly: 
				if(allCYfemaleLoci_.Includes(trnum)){
					allelesCurrentCYfemale_=false;
					trloc = allCYfemaleLoci_.Loc(trnum);
					allCYfemaleLoci_.Delete(trloc);
					if(skippedTraitNum_.Includes(trnum)){
							trloc = skippedLocusNumCYfemale_.Loc(trnum);
							skippedLocusNumCYfemale_.Delete(trloc);}
						else{
							trloc = locusNumCYfemale_.Loc(trnum);
							locusNumCYfemale_.Delete(trloc);}
					nlociCYfemale_ = locusNumCYfemale_.len();}
				if(allPresentInFemalesOnly_.Includes(trnum)){
					trloc = allPresentInFemalesOnly_.Loc(trnum);
					allPresentInFemalesOnly_.Delete(trloc);}
				if(presentInFemalesOnly_.Includes(trnum)){
					trloc = presentInFemalesOnly_.Loc(trnum);
					presentInFemalesOnly_.Delete(trloc);}
				if(CytoplasmicFemaleMarkerData::allPresentInBothSexes_.Includes(trnum)){
					trloc = CytoplasmicFemaleMarkerData::allPresentInBothSexes_.Loc(trnum);
					CytoplasmicFemaleMarkerData::allPresentInBothSexes_.Delete(trloc);}
				if(CytoplasmicFemaleMarkerData::presentInBothSexes_.Includes(trnum)){
					trloc = CytoplasmicFemaleMarkerData::presentInBothSexes_.Loc(trnum);
					CytoplasmicFemaleMarkerData::presentInBothSexes_.Delete(trloc);}
				if(allCYfemaleLoci_.len()==0){
						traitTypesIncluded_[CYfemale]=false;
						traitTypesIncluded_[CYfemaleOnly]=false;}
					else{//check them all
						int countCYF=0,countCYFonly=0;
						for(tr=0;tr<traitType_.len();++tr){
							if(traitType_[tr]==CYfemale){
									++countCYF;}				
								else if(traitType_[tr]==CYfemaleOnly){
									++countCYFonly;}
								else{}
							}//tr
						if(countCYF==0){
								traitTypesIncluded_[CYfemale]=false;}				
							else if(countCYFonly==0){
								traitTypesIncluded_[CYfemaleOnly]=false;}
							else{}
						}//done checking			
				break;
			case CYmale: 
			case CYmaleOnly: 
				if(allCYmaleLoci_.Includes(trnum)){
					allelesCurrentCYmale_=false;
					trloc = allCYmaleLoci_.Loc(trnum);
					allCYmaleLoci_.Delete(trloc);
					if(skippedTraitNum_.Includes(trnum)){
							trloc = skippedLocusNumCYmale_.Loc(trnum);
							skippedLocusNumCYmale_.Delete(trloc);}
						else{
							trloc = locusNumCYmale_.Loc(trnum);
							locusNumCYmale_.Delete(trloc);}
					nlociCYmale_ = locusNumCYmale_.len();}					
				if(allPresentInMalesOnly_.Includes(trnum)){
					trloc = allPresentInMalesOnly_.Loc(trnum);
					allPresentInMalesOnly_.Delete(trloc);}
				if(presentInMalesOnly_.Includes(trnum)){
					trloc = presentInMalesOnly_.Loc(trnum);
					presentInMalesOnly_.Delete(trloc);}
				if(CytoplasmicMaleMarkerData::allPresentInBothSexes_.Includes(trnum)){
					trloc = CytoplasmicMaleMarkerData::allPresentInBothSexes_.Loc(trnum);
					CytoplasmicMaleMarkerData::allPresentInBothSexes_.Delete(trloc);}
				if(CytoplasmicMaleMarkerData::presentInBothSexes_.Includes(trnum)){
					trloc = CytoplasmicMaleMarkerData::presentInBothSexes_.Loc(trnum);
					CytoplasmicMaleMarkerData::presentInBothSexes_.Delete(trloc);}
				if(allCYmaleLoci_.len()==0){
						traitTypesIncluded_[CYmale]=false;
						traitTypesIncluded_[CYmaleOnly]=false;}	
					else{//check them all
						int countCYM=0,countCYMonly=0;
						for(tr=0;tr<traitType_.len();++tr){
							if(traitType_[tr]==CYmale){
									++countCYM;}				
								else if(traitType_[tr]==CYmaleOnly){
									++countCYMonly;}
								else{}
							}//tr
						if(countCYM==0){
								traitTypesIncluded_[CYmale]=false;}				
							else if(countCYMonly==0){
								traitTypesIncluded_[CYmaleOnly]=false;}
							else{}
						}//done checking			
				break;
			default: break;
			}//switch
		}//tr
	}//MultiformatMarkerData::RemoveTraits(int , aList<int>& )



void MultiformatMarkerData::TryToSortTraits(void){
	if(traitTypeCurrent_) return;
	//search for sex-describing traits first
	int pop, i, tr, trt,traitToConsider;
	aList<int> possibleSexTraits,possibleCYfemaleOnlyTraits,possibleCYmaleOnlyTraits;
	aString trait;
	TextDatum traitCopy;
	for(trt=0;trt<traitNum_.len();++trt){ //ignores skipped traits
		tr = traitNum_[trt];
		if(tr==0) continue;//skip ID trait
		traitToConsider=-1;
		for(pop=0;pop<npop_;++pop){
			traitCopy = rawTraits_[pop][0][tr];//trait in first individual
			traitCopy.CullLeadingSpaces();//handles cases where alleleLen()>1
			traitCopy.CullTrailingSpaces();
			if(traitCopy == 'M' || traitCopy == 'm' || traitCopy == 'F' || traitCopy == 'f' || traitCopy.len() == 0 ){
					traitToConsider = tr;}
				else {continue;}//skips to next trait
			for(i=1;i<individuals(pop);++i){
				traitCopy = rawTraits_[pop][i][tr];
				traitCopy.CullLeadingSpaces();
				traitCopy.CullTrailingSpaces();
				if(!(traitCopy == 'M' || traitCopy == 'm' || traitCopy == 'F' || traitCopy == 'f' || traitCopy.len() == 0 )){
						traitToConsider = -1;
						break;}
				}//i
			}//pop
		if(traitToConsider==tr){//got a candidate
			possibleSexTraits.Append(tr);}
		}//trt
	
	bool traitEliminated=false;
	//now check for female-limited cytoplasmic traits
	if(possibleSexTraits.len()>0){
		for(trt=0;trt<traitNum_.len();++trt){ //skip ID trait
			tr = traitNum_[trt];
			if(tr==0 || possibleCYfemaleOnlyTraits.Includes(tr)) continue;//skip these
			traitToConsider=-1;
			traitEliminated=false;
			for(pop=0;pop<npop_;++pop){
				TextDatum& trait0cyf = rawTraits_[pop][0][tr];//trait in first individual
				TextDatum& sexval0cyf = rawTraits_[pop][0][possibleSexTraits[0]];
				if(trait0cyf.len() == CytoplasmicFemaleMarkerData::alleleLen() || trait0cyf.missing() || sexval0cyf.missing() || sexval0cyf == 'F' || sexval0cyf == 'f'){
						traitToConsider = tr;}
					else {
						traitEliminated=true;
						break;}//skips to next pop
				for(i=1;i<individuals(pop);++i){
					TextDatum& traitCyf = rawTraits_[pop][i][tr];
					TextDatum& sexvalCyf = rawTraits_[pop][i][possibleSexTraits[0]];
					if((  traitCyf.len() != CytoplasmicFemaleMarkerData::alleleLen() && traitCyf.OK()  ) || sexvalCyf == 'M' || sexvalCyf == 'm'){
							traitToConsider = -1;
							break;}
					}//i
				if(traitEliminated) continue;//skips to next trait
				}//pop
			if(traitToConsider==tr){//got a candidate
				possibleCYfemaleOnlyTraits.Append(tr);}
			}//trt
		//now check for male-limited cytoplasmic traits
		for(trt=0;trt<traitNum_.len();++trt){ //skip ID trait
			tr = traitNum_[trt];
			if(tr==0 || possibleCYmaleOnlyTraits.Includes(tr) || possibleCYfemaleOnlyTraits.Includes(tr)) continue;//skip these
			traitToConsider=-1;
			traitEliminated=false;
			for(pop=0;pop<npop_;++pop){
				TextDatum& trait0cym = rawTraits_[pop][0][tr];//trait in first individual
				TextDatum& sexval0cym = rawTraits_[pop][0][possibleSexTraits[0]];
				if(trait0cym.len() == CytoplasmicMaleMarkerData::alleleLen() || trait0cym.missing() || sexval0cym.missing() || sexval0cym == 'M' || sexval0cym == 'm'){
						traitToConsider = tr;}
					else {
						traitEliminated=true;
						break;}//skips to next pop
				for(i=1;i<individuals(pop);++i){
					TextDatum& traitCym = rawTraits_[pop][i][tr];
					TextDatum& sexvalCym = rawTraits_[pop][i][possibleSexTraits[0]];
					if((  traitCym.len() != CytoplasmicMaleMarkerData::alleleLen() && traitCym.OK()  ) || sexvalCym == 'F' || sexvalCym == 'f'){
							traitToConsider = -1;
							break;}
					}//i
				if(traitEliminated) continue;//skips to next trait
				}//pop
			if(traitToConsider==tr){//got a candidate
				possibleCYmaleOnlyTraits.Append(tr);}
			}//trt
		}//sex trait available
	//now check for haploid, dominant, sex-linked dominant & cytoplasmic traits
	aList<int> possibleHapDomTraits;
	traitEliminated=false;
	for(trt=0;trt<traitNum_.len();++trt){ //skip ID trait
		tr = traitNum_[trt];
		if(tr==0 || possibleSexTraits.Includes(tr) || possibleCYmaleOnlyTraits.Includes(tr)
				 || possibleCYfemaleOnlyTraits.Includes(tr)) continue;//skip these
		traitToConsider=-1;
		traitEliminated=false;
		for(pop=0;pop<npop_;++pop){
			TextDatum& trait0Hd = rawTraits_[pop][0][tr];//trait in first individual
			if(trait0Hd.len() == HaploidMarkerData::alleleLen() || trait0Hd.missing() ){//assumes haploid and dominant data have same alleleLen_
					traitToConsider = tr;}
				else {
					traitEliminated=true;
					break;}//skips to next pop
			for(i=1;i<individuals(pop);++i){
				TextDatum& traitHd = rawTraits_[pop][i][tr];
				if(traitHd.len() != HaploidMarkerData::alleleLen() && traitHd.OK()){
						traitToConsider = -1;
						break;}
				}//i
			if(traitEliminated) continue;//skips to next trait
			}//pop
		if(traitToConsider==tr){//got a candidate
			possibleHapDomTraits.Append(tr);}
		}//trt
	
	//now check for sex-linked traits
	aList<int> possibleSexLinkCodomTraits, possibleCodomTraits;
	bool traitIdentified = false;
	for(trt=0;trt<traitNum_.len();++trt){ //skip ID trait
		tr = traitNum_[trt];
		if(tr==0 || possibleSexTraits.Includes(tr) || possibleHapDomTraits.Includes(tr) 
				 || possibleCYmaleOnlyTraits.Includes(tr) || possibleCYfemaleOnlyTraits.Includes(tr)) continue;//skip these
		traitToConsider=-1;
		traitIdentified = false;
		for(pop=0;pop<npop_;++pop){
			for(i=1;i<individuals(pop);++i){
				traitCopy = rawTraits_[pop][i][tr];
				traitCopy.CullLeadingSpaces();
				traitCopy.CullTrailingSpaces();
				if(traitCopy.len() == HaploidMarkerData::alleleLen()){
					possibleSexLinkCodomTraits.Append(tr);//any such trait will be sex-linked, since hap|dom traits are already swept out
					traitIdentified = true;
					break;}
				}//i
			if(traitIdentified) break;//skip to next trait
			}//pop
		if(!traitIdentified){
			possibleCodomTraits.Append(tr);}
		}//tr
	ObjList<aList<int> > presortedTraits;
	for(trt=0;trt<markerTypes();++trt){//empty list
		presortedTraits.Append(NEW aList<int>());}
	presortedTraits[ID].Append(0);//ID trait
	presortedTraits[sex].Append(possibleSexTraits[0]);//sex trait
//	presortedTraits.Append(NEW aList<int>(possibleCodomTraits));//3
//	presortedTraits.Append(NEW aList<int>(possibleHapDomTraits));//4 dom
//	presortedTraits.Append(NEW aList<int>(possibleSexLinkCodomTraits));//5
	//clean up
	for(tr=1;tr<possibleSexTraits.len();++tr){//extra sex? traits go with ID as unsorted
		presortedTraits[0].Append(possibleSexTraits[tr]);}
	for(tr=0;tr<possibleCodomTraits.len();++tr){
		presortedTraits[codom].Append(possibleCodomTraits[tr]);}
	for(tr=0;tr<possibleHapDomTraits.len();++tr){
		presortedTraits[dom].Append(possibleHapDomTraits[tr]);}
	for(tr=0;tr<possibleSexLinkCodomTraits.len();++tr){
		presortedTraits[SLcodom].Append(possibleSexLinkCodomTraits[tr]);}
	for(tr=0;tr<possibleCYfemaleOnlyTraits.len();++tr){
		presortedTraits[CYfemaleOnly].Append(possibleSexLinkCodomTraits[tr]);}
	for(tr=0;tr<possibleCYmaleOnlyTraits.len();++tr){
		presortedTraits[CYmaleOnly].Append(possibleSexLinkCodomTraits[tr]);}
	bool found;
	for(tr=0;tr<traitNum_.len();++tr){//any holdouts
		found=false;
		int tr2=traitNum_[tr];
		for(trt=0;trt<presortedTraits.len();++trt){
			if(presortedTraits[trt].Includes(tr2)){
				found=true; break;}
			if (found) {break;}
			}
		if(!found){
			presortedTraits[ID].Append(tr2);}//extra unsorted traits go with ID
		}//tr
		
		
	SortTraitsInteractively(presortedTraits);
	}//MultiformatMarkerData::TryToSortTraits ( )




int MultiformatMarkerData::traits(int traitType){
	switch(traitType){
		case hap:
			return HaploidMarkerData::traits();	
			break;
		case codom:
			return CodominantMarkerData::traits();	
			break;
		case dom:
			return DominantMarkerData2::traits();	
			break;
		case SLcodom:
			return SexLinkedCodomMarkerData::traits();	
			break;
		case SLdom:
			return SexLinkedDomMarkerData2::traits();	
			break;
		case CYfemale:
			return CytoplasmicFemaleMarkerData::presentInBothSexes_.len();	
			break;
		case CYmale:
			return CytoplasmicMaleMarkerData::presentInBothSexes_.len();	
			break;
		case CYfemaleOnly:
			return CytoplasmicFemaleMarkerData::presentInFemalesOnly_.len();	
			break;
		case CYmaleOnly:
			return CytoplasmicMaleMarkerData::presentInMalesOnly_.len();	
			break;
		default: break;
		}
	return EvolutionaryGeneticsData::traits();	
	}//traits()}



void MultiformatMarkerData::UpdateVariableLoci(void){
/*	if (varLociCurrent_){return;}
	CheckForNewAlleles();
	variableLocusNumCodom_.Clear();
	for(int loc=0;loc<nlociCodom_;++loc){
		StringList& locList = allelesPresent_[loc];
		if (locList.len()>1){
			variableLocusNumCodom_.Append(locusNumCodom_[loc]);}}
	varLociCurrent_=true;
*/
	UpdateVariableLociHap();
	CodominantMarkerData::UpdateVariableLoci();
	DominantMarkerData2::UpdateVariableLociDom();
	SexLinkedCodomMarkerData::UpdateVariableLoci();
	SexLinkedDomMarkerData2::UpdateVariableLociDom();
	CytoplasmicFemaleMarkerData::UpdateVariableLociCYfemale();
	CytoplasmicMaleMarkerData::UpdateVariableLociCYmale();
	}//end MultiformatMarkerData::UpdateVariableLoci




#endif






