/******************************************************
*
*	Genotype.h
*
*	This class implements a list of loci and
*	alleles.  All loci unlinked.  It handles
*	cloning and sexual reproduction.
*
*	Base class:  Phenotype
*
*******************************************************/

#ifndef Genotypeh
#define Genotypeh = 1


#include <iostream.h>
#include "Randomness.h"
#include "StringClass.h"
#include "aError.h"
#include "Element.h"
#include "aList.h"
#include "Allele.h"
#include "InlineTemplates.h"
#include "Phenotype.h"
#include "CoreGenome.h"
#include "GeneContainers.h"
#include "MemoryManager.h"

class Genotype;

class Genotype: public CoreGenome {
//class Genotype: public Phenotype {
  protected:
  	ObjList<ObjList<Allele<int> > > loci_;//[loc][allele]
  			//empty lists can occur at sex-limited loci, e.g., females have no gene at Y-linked loci
  public:
  	Genotype(void):loci_(){};//creates empty list
  	Genotype(int numLoci, int ploidy, int alleleVal);
  			//creates a 'null' homozygous individual at all loci
  	Genotype(aList<int>& ploidyList, int alleleVal);
  			//creates a 'null' homozygous individual at all loci, with loci varying in ploidy
	Genotype(Genotype&);//copies the Genotype during construction
	Genotype(Genotype*);//copies the Genotype during construction
virtual ~Genotype(void){loci_.Clear();}//destroys the list
  	
virtual Phenotype* Duplicate(){
				return NEW Genotype(*this);}
				//counts the absolute frequency of alleleVal 
virtual long GetPointerToDerivedSelf(void){
				return (long) this;}

virtual long double  additiveRelativePhenotype(void);//derive for an additive phenotype


virtual int		AllelePresent (int loc, aString& alleleVal);
virtual int		AllelePresent (int loc, int alleleVal);
				//counts the absolute frequency of alleleVal 

virtual int		CountAlleles (int loc, aString& alleleVal);
virtual int		CountAlleles (int loc, int alleleVal);
				//counts the absolute frequency of alleleVal 

//virtual void	Copy (Genotype&);//copies to an existing genotype
//virtual void	Copy (Genotype*);//copies to an existing genotype
virtual void	Copy (Phenotype&);//copies to an existing genotype
virtual void	Copy (Phenotype*);//copies to an existing genotype
	
//virtual void	Fertilize (Genotype* momGamete, Genotype* dadGamete);
virtual void	Fertilize (CoreGenome* momGamete, CoreGenome* dadGamete);

//virtual void	Gamete (Genotype* parentGenes, Ran3& ran);
//virtual void	Gamete (CoreGenome* parentGenes, Ran3List& ran, double /*mutationRate*/, double /*sigma*/);
virtual void	Gamete (CoreGenome* parentGenes, Ran3List& ran, aList<long double>& specialInfo);

virtual int		GetGene (int locus, int alleleNum);
				//returns the value of the allele

virtual void	GetGenome(GeneContainerCore& gc, long IDval){
					gc.SetID(IDval);
					GetGenome(gc);}
				//refills container class as a MultiHaplotypeContainer<int>[ploidy][locus]
virtual void	GetGenome(GeneContainerCore& gc);
				//refills container class as a MultiHaplotypeContainer<int>[ploidy][locus]
			
virtual aString* 	GenotypeString(char* separator=" ");
					//returns all ploidy copies separated by parentheses (A)(A) at each locus
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(int locus, char* /*separator=" "*/);
					//finds nth overall locus and returns all ploidy copies separated by parentheses (A)(A)
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(int locus, int alleleNum,  char* /*separator=" "*/);
					//finds nth overall locus and returns the alleleNum_th copy

virtual int		IsHeterozygote (int locus);
				//returns false=0 if haploid

virtual int		loci(void){return loci_.len();}

virtual void 	MakeNull(void);//sets existing genotype to all zeros
  	
virtual void	Mutate(int locus, int alleleNum, int newAlleleVal){
  						set(locus,alleleNum,newAlleleVal);}
  				//changes the value of an allele
  		
//virtual void	Mutate(Ran3& /*ran*/, double /*mutationRate*/, double /*sigma*/){
virtual void	Mutate(Ran3& /*ran*/, aList<long double>& /*mutationInfo*/){
  						aString errstr("Error: tried to mutate the whole genotype when you want to mutate the allele");
  						aError(errstr);
  				}//changes the value of an allele
  		
virtual int 	ploidy (int locus=0){
					return PloidyAtLoc(locus);}
  		
virtual int		PloidyAtLoc (int locus);
				//returns the number of alleles

virtual void	Print(ostream& outFile, int alleleLen=4);

virtual int		RandomAllele(int locus, Ran3& ran);
  				//randomly picks an allele and returns its value

virtual void	set(int locus, int alleleNum, int newAlleleVal);
  				//changes the value of an allele
  				//use when doing reproduction, etc.
  				//same as mutate, but term is not so process-laden
  				//USES ZERO-BASED INDICES FOR locus AND alleleNum
  		
virtual void	SetGenome(GeneContainerCore& gc);
				//initializes genome using MultiHaplotypeContainer<int>[ploidy][locus]

virtual void	SetGenotypeString(aString& geneStr, int alleleLen=4);
	
//virtual void	SetSexualZygote(Genotype* momGenes, Genotype* dadGenes, Ran3& ran, 
//virtual void	SetSexualZygote(CoreGenome* momGenes, CoreGenome* dadGenes, Ran3List& ran, 
//										double mutationRate, double sigma);
virtual void	SetSexualZygote(CoreGenome* momGenes, CoreGenome* dadGenes, Ran3List& ran, 
										aList<long double>& specialInfo);
				//uses mom's and dads genes to seed the genotype
	

friend int operator== (Genotype& g1, Genotype& g2);
		
friend int operator!= (Genotype& g1, Genotype& g2);
  	
  	};//end class Genotype
  	
  	
int operator== (Genotype& g1, Genotype& g2);
		
int operator!= (Genotype& g1, Genotype& g2);

#endif
