/******************************************************
*
*	DoseResponseParamRec.h
*
*	Parameters for fitting dose-response curves by maximum likelihood.
*	It's really just a version of a Cumulative Normal Distribution
*	
*
******************************************************/

#ifndef DoseResponseParamReccpp
#define DoseResponseParamReccpp = 1

#include "DoseResponseParamRec.h"
#include "CumulativeNormalLikelihood.h"

DoseResponseParamRec::DoseResponseParamRec(void){
  	SetPointerToDerivedSelf();
  	}


DoseResponseParamRec::DoseResponseParamRec(double m, double s){
	SetPointerToDerivedSelf();
	mu_= m;
	sigma_ = s;
	}


DoseResponseParamRec::DoseResponseParamRec(DoseResponseParamRec& p):
  				CumNormalDistParamRec(p){
  	SetPointerToDerivedSelf();
	lnL_= p.lnL_;
	lnLmaxMinusLnL_ = p.lnLmaxMinusLnL_;	
	mu_= p.mu_;
	sigma_ = p.sigma_;
	}


DoseResponseParamRec::~DoseResponseParamRec(void){
  	pointerToDerivedSelf = NULL;//forget
  	}


	
ParamRec* DoseResponseParamRec::Duplicate(void){
	DoseResponseParamRec* copy = new DoseResponseParamRec(*this);
	return copy;
	}//end DoseResponseParamRec::Duplicate()


DoseResponseParamRec* DoseResponseParamRec::_Duplicate(void){
	DoseResponseParamRec* copy = new DoseResponseParamRec(*this);
	return copy;
	}//end DoseResponseParamRec::_Duplicate()


void DoseResponseParamRec::SetPointerToDerivedSelf(void){
  	pointerToDerivedSelf = (long) this;
//  	cout<<"setting in DoseResponseParamRec"<<endl;
  	}//end




double DoseResponseParamRec::DoseAtMortality(double mortalityFraction){
	return XforCumulativeNormalProbability(mortalityFraction);
	}//end DoseResponseParamRec::DoseAtMortality(double)
	

double DoseResponseParamRec::LD50(void){
	//LD 50 value
	return mu();
	}//DoseResponseParamRec::LD50()


double DoseResponseParamRec::LD(double mortalityPercent){
	return DoseAtMortality(mortalityPercent/100.0);
	}//DoseResponseParamRec::LD(double)


	
double DoseResponseParamRec::MortalityAtDose(double dose){
	return exp(CumulativeNormalProbabilityAtX(dose));
	}//end DoseResponseParamRec::MortalityAtDose(double)


double DoseResponseParamRec::naturalResponse(void){
	//mortality at dose zero
	return MortalityAtDose(0.0);
	}//end DoseResponseParamRec::naturalResponse()


int DoseResponseParamRec::NumDerivedParams(void){
	//number of parameters that are functions of varying parameters 
	//override in derived classes
	return 4;
	}//ParamRec::NumDerivedParams()


aString* DoseResponseParamRec::ParamName(int paramNum){
	switch (paramNum){
	 case 0:
	 	 return new aString("LD50");
	 	 break;
	 case 1:
	 	 return new aString("sigma");
	 	 break;
	 case 2:
	 	 return new aString("slope");
	 	 break;
	 case 3:
	 	 return new aString("natural response");
	 	 break;
	 case 4:
	 	 return new aString("LD10");
	 	 break;
	 case 5:
	 	 return new aString("LD90");
	 	 break;
	 default:
		aString errstr("bug: can't return name of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 5-parameter dose response analysis");
		aError(errstr);
	 	}
	return new aString();
	}//end NormalDistParamRec::ParamName(int )


void DoseResponseParamRec::ParamString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::ParamString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat(LD50(),6,9);
	dataline.cat(sepchar);
	dataline.cat(sigma(),6,9);
	dataline.cat(sepchar);
	dataline.cat(slope(),6,9);
	dataline.cat(sepchar);
	dataline.cat(naturalResponse(),6,9);
	dataline.cat(sepchar);
	dataline.cat(LD(10.0),6,9);
	dataline.cat(sepchar);
	dataline.cat(LD(90.0),6,9);
	}//end DoseResponseParamRec::ParamString(aString&, char)



double DoseResponseParamRec::ParamValue(int paramNum){
	//returns value of parameter
	switch (paramNum){
	 case 0:
	 	 return LD50();
	 	 break;
	 case 1:
	 	 return sigma();
	 	 break;
	 case 2:
	 	 return slope();
	 	 break;
	 case 3:
	 	 return naturalResponse();
	 	 break;
	 case 4:
	 	 return LD(10.0);
	 	 break;
	 case 5:
	 	 return LD(90.0);
	 	 break;
	 default:
		aString errstr("bug: can't return value of param #");
		errstr.cat(paramNum+1);
		errstr.cat(" in a 5-parameter dose response analysis");
		aError(errstr);
	 	}
	return 0.0;
	}//end DoseResponseParamRec::ParamValue(int )


void DoseResponseParamRec::Print(ostream& outFile, long rep, aString& message){
	ParamRec::Print(outFile, rep, message);
	aString outStr("LD50: ");
	outStr.cat(LD50(),6,8);
	outStr.separator(5);
	outStr.cat("sigma: ");
	outStr.cat(sigma(),6,8);
	outStr.separator(5);
	outStr.cat("slope: ");
	outStr.cat(slope(),6,8);
	outFile<<outStr<<endl;
	outStr="natural response: ";
	outStr.cat(naturalResponse(),6,8);
	outFile<<outStr<<endl;
	outStr="LD10: ";
	outStr.cat(LD(10.0),6,8);
	outStr.separator(5);
	outStr.cat("LD90: ");
	outStr.cat(LD(90.0),6,8);
	outFile<<outStr<<endl;
	}//DoseResponseParamRec::Print


double DoseResponseParamRec::slope(void){
	double b = 1.0/(sigma()*sqrt(2.0));
	return b;
	}//end DoseResponseParamRec::slope(void)
	


void DoseResponseParamRec::TitleString(aString& dataline, char sepchar){
	//a list of the values that will be printed to a big file
	ParamRec::TitleString(dataline, sepchar);
	dataline.cat(sepchar);
	dataline.cat("LD50");
	dataline.cat(sepchar);
	dataline.cat("sigma");
	dataline.cat(sepchar);
	dataline.cat("slope");
	dataline.cat(sepchar);
	dataline.cat("natural response");
	dataline.cat(sepchar);
	dataline.cat("LD10");
	dataline.cat(sepchar);
	dataline.cat("LD90");
	}//end DoseResponseParamRec::TitleString(aString&, char)


int DoseResponseParamRec::TotalParams(void){
//	return 6;
	return ParamRec::TotalParams();
	}



void DoseResponseParamRec::operator=(ParamRec& p){
	DoseResponseParamRec* pp = (DoseResponseParamRec*) p.pointerToDerivedSelf;
	lnL_= pp->lnL_;
	lnLmaxMinusLnL_ = pp->lnLmaxMinusLnL_;	
	mu_= pp->mu_;
	sigma_ = pp->sigma_;
	}//end  DoseResponseParamRec::operator=(ParamRec&)





int	operator==(DoseResponseParamRec& p, DoseResponseParamRec& q){
	CumNormalDistParamRec& a = p;
	CumNormalDistParamRec& b = q;
	return (a == b);
	}//end operator==(DoseResponseParamRec&, DoseResponseParamRec&)


int	operator!=(DoseResponseParamRec& p, DoseResponseParamRec& q){
	return !(p==q);
	}//end operator!=(DoseResponseParamRec&, DoseResponseParamRec&)



#endif


