/********************************************
*
*
*		Phenotype.h
*
*		This is the base class for phenotypic scores
*
*********************************************/

#ifndef Phenotypeh
#define Phenotypeh = 1

#include "Element.cpp"
#include "aList.cpp"
#include "StringClass.h"
#include "PopulationIntf.h"
#include "SelectionIntf.h"
#include "GeneContainerCore.h"
#include "InlineTemplates.h"
#include "MemoryManager.h"

typedef int (*ViabilitySelectionFunc)(aList<long double>&, Individual&, Ran3&, long double&);

/*default selection function*/
int NeutralToSelection(aList<long double>&, Individual&, Ran3&, long double& fitness);//{
//	fitness = one;
//	return 1;}/*always alive*/


 class Phenotype {
  	protected:
  		aList<long double> trait_;/*list of phenotypic values*/
  	public:
  		ViabilitySelectionFunc selectionFunc;
  			/*attaches a function that does viability selection*/
  			/*default is always neutrality*/

  	
  	public:
  		Phenotype(void): trait_()/*, selectionFunc(NULL)*/ {
 			(*selectionFunc) = NeutralToSelection;
 			}//default is neutrality
  		Phenotype(long double z): trait_()/*, selectionFunc(NULL)*/ {
			(*selectionFunc) = NeutralToSelection;//default is neutrality
  			trait_.Append(z);}
  		Phenotype(aList<long double> z): trait_(z)/*, selectionFunc(NULL)*/ {
			(*selectionFunc) = NeutralToSelection;}//default is neutrality
  		Phenotype(ViabilitySelectionFunc viabSelection): trait_()/*, selectionFunc(NULL)*/ {
 			(*selectionFunc) = viabSelection;}
  		Phenotype(long double z, ViabilitySelectionFunc viabSelection): trait_()/*, selectionFunc(NULL)*/ {
			(*selectionFunc) = viabSelection;
  			trait_.Append(z);}
  		Phenotype(aList<long double> z, ViabilitySelectionFunc viabSelection): trait_(z)/*, selectionFunc(NULL)*/ {
			(*selectionFunc) = viabSelection;}

  		virtual ~Phenotype(void){
  			trait_.Clear();
  			selectionFunc=NULL;//forget but do not delete
  			}

	virtual Phenotype* new_Phenotype(void){
				return NEW Phenotype();}
	
	virtual Phenotype* Duplicate(void){
				Phenotype* p = new_Phenotype();
				p->Copy(*this);
				return p;}
	
	virtual long GetPointerToDerivedSelf(void){
				return (long) this;}

	void 		Copy(Phenotype& p);  		
	void 		Copy(Phenotype* p){Copy(*p);}	

	virtual long double 	trait(void); //returns trait 0
	virtual long double 	trait(int i);
	virtual long 	traits(void){return trait_.len();}
	virtual void	GetGenome(GeneContainerCore& /*gc*/, long /*IDval*/){//refills container class at appropriate level
							aError("Error: base class Phenotype has no genome");}
	virtual void	GetGenome(GeneContainerCore& /*gc*/){//refills container class at appropriate level
							aError("Error: base class Phenotype has no genome");}
	virtual aString* GetPhenotypeString(char* separator=" ");
	virtual void 	MakeNull(void);//fills with 0s and NeutraltoSelection
//	virtual void 	Mutate(Ran3& ran, double mutationRate, double sigma);
	virtual void 	Mutate(Ran3& ran, aList<long double>& mutationInfo);
	virtual void    Print(ostream& ){}
	virtual void    Print(ostream&,int ){}
	void 			SetSelectionFunction (ViabilitySelectionFunc viabSelection){
					(*selectionFunc) = viabSelection;}
	virtual void 	SetSelectionParams(aList<long double>& selParams);
					/*default copies traits to selParams*/
	virtual int 	SurvivesViabilitySelection(aList<long double>& selParams, Individual& indiv, Ran3& ran, long double& fitness){
					/*0=dead; 1=alive; used in natural selection*/
//							SetSelectionParams(selParams);
							return (*selectionFunc)(selParams, indiv, ran, fitness);}
	virtual void	SetGenome(GeneContainerCore& /*gc*/){//initializes genome using appropriate level of container class
							aError("Error: base class Phenotype has no genome");}
  	virtual void 	SetTrait(long double z, int i = 0);
  	virtual genderSet 	sex(genderSet gender){return gender;}//override for sex determined genomes
  	virtual long double 	operator[] (int i){
  						return trait(i);}
  	
  	virtual int 	ploidy(int /*locus=0*/){return 0;}
  						

friend int operator== (Phenotype& p1, Phenotype& p2);
		
friend int operator!= (Phenotype& p1, Phenotype& p2);
  	
  		
  	};/*end class Phenotype*/

int operator== (Phenotype& p1, Phenotype& p2);
		
int operator!= (Phenotype& p1, Phenotype& p2);
  	




#endif

