/*****************************************************
*
*
*	RegulatedAllele.h
*
*	Holds the cis-regulatory sites of an allele
*
******************************************************/
#ifndef RegulatedAlleleh
#define RegulatedAlleleh = 1
#include "Element.h"
#include "aList.h"
#include "StringClass.h"
#include "Randomness.h"
#include "RandomList.h"
#include "Allele.h"

enum locusComponents {unspecifiedSite, productSite, promoterSite, inhibitorSite};





template <class T> class RegulatedAllele: public Allele<T> {
  private:
  	T promoter_;//value that upregulates
  	T inhibitor_;//value that downregulates
  public:
  	RegulatedAllele(T x, T p, T i): Allele<T>(x), promoter_(p), inhibitor_(i){
			}
	RegulatedAllele(RegulatedAllele<T>& a): Allele(a.val()), 
											promoter_(a.promoter()), 
											inhibitor_(a.inhibitor()){
			}
	RegulatedAllele(RegulatedAllele<T>* a): Allele<T>(a->val()), 
											promoter_(a->promoter()), 
											inhibitor_(a->inhibitor()){
			}
virtual ~RegulatedAllele(void){
			}

virtual Allele<T>& operator= (RegulatedAllele<T>& a){
  	data_=a.val(); 
  	promoter_=a.promoter(); 
  	inhibitor_=a.inhibitor(); 
  	return *this;
  	}//end RegulatedAllele<T>::operator=
  	
virtual Allele<T>* 	Duplicate(void);
  		T 			inhibitor(void) {return inhibitor_;}
  		T 			promoter(void) {return promoter_;}
virtual void 		mutateTo(T x){
						mutateTo(x,unspecifiedSite);}
virtual void 		mutateTo(T x, locusComponents site);
virtual void 		Mutate(Ran3& ran, double sigma);
virtual void 		Mutate(Ran3& ran, double sigma=1.0, locusComponents site=unspecifiedSite);
virtual aString* 	GenotypeString(char* separator=" "){
						return GenotypeString(separator, unspecifiedSite);}
virtual aString* 	GenotypeString(char* separator, locusComponents omitSite);
		
  	
  	friend int operator== (RegulatedAllele<T>& a, RegulatedAllele<T>& b);
  	friend int operator!= (RegulatedAllele<T>& a, RegulatedAllele<T>& b);

  	};//end RegulatedAllele


template<class T> int operator== (RegulatedAllele<T>& a, RegulatedAllele<T>& b){
	return ( (a.data_==b.data_) && (a.promoter_==b.promoter_) && (a.inhibitor_==b.inhibitor_));}

template<class T> int operator!= (RegulatedAllele<T>& a, RegulatedAllele<T>& b){
	return !(a==b);}



#endif
