/******************************************************
*
*	RegulatedGenome.h
*
*	This class implements a Genome of  RegulatedAlleles.  
*	It handles cloning and sexual reproduction.
*
*	Base class:  Phenotype
*
*******************************************************/

#ifndef RegulatedGenomeh
#define RegulatedGenomeh = 1

#include <iostream.h>
#include "Randomness.h"
#include "RandomList.h"
#include "StringClass.h"
#include "Element.h"
#include "aList.hp"
#include "RegulatedAllele.h"
#include "Chromosome.h"
#include "Phenotype.h"
#include "Genome.h"
#include <math.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"

template <class T> class RegulatedGenome: virtual public Genome<T> {
  public:
    ObjList<aList<int> > devPath_;//stores developmental pathways as locus names
    	//order in which loci influence one another
    	//negative values inhibit, positive values promote
    	
  public:
  	RegulatedGenome(void):Genome<T>(){
 		devPath_.Append(new aList<int>());
  		}
  	RegulatedGenome(int numLociPerChrom, int num1NChroms, int ploidy, T alleleVal, T promoValue, T inhibValue);
  			/*creates a 'null' homozygous individual at all loci*/
  	RegulatedGenome(ObjList<ChromosomeList<T> >& c): Genome<T>(c) {
  		devPath_.Append(new aList<int>();
  		}
  	RegulatedGenome(ObjList<ChromosomeList<T> >& c,ObjList<aList<int> >& path): Genome<T>(c), devPath_(path) {
			}
  	RegulatedGenome(ViabilitySelectionFunc viabSelection):Phenotype(viabSelection),genome_(){
  		devPath_.Append(new aList<int>();
  		}/*creates empty list*/
  	RegulatedGenome(int numLociPerChrom, int num1NChroms, int ploidy, 
  					T alleleVal, T promoValue, T inhibValue, ViabilitySelectionFunc viabSelection);
  			/*creates a 'null' homozygous individual at all loci*/
  	RegulatedGenome(ObjList<ChromosomeList<T> >& c,ViabilitySelectionFunc viabSelection):
  					Genome<T>(c, viabSelection) {
  		devPath_.Append(new aList<int>();
  		}
  	RegulatedGenome(ObjList<ChromosomeList<T> >& c,ObjList<aList<int> >& path,
  					ViabilitySelectionFunc viabSelection):
  					Genome<T>(c, viabSelection), devPath_(path) {
		}
	RegulatedGenome(RegulatedGenome<T>&);/*copies the RegulatedGenome during construction*/
	RegulatedGenome(RegulatedGenome<T>*);/*copies the RegulatedGenome during construction*/
virtual ~RegulatedGenome(void){
						devPath_.Clear();
						}//end destructor


virtual Phenotype* new_Phenotype(void){
					return NEW RegulatedGenome<T>();}
	
virtual Phenotype* Duplicate(void){
				RegulatedGenome<T>* g = NEW RegulatedGenome<T>(*this);
//				RegulatedGenome<T>* g = NEW RegulatedGenome<T>();
//				g->Copy(*this);
//				g->devPath_.Copy(devPath_);
				return g;}
	

virtual void	Mutate(Ran3& ran, double mutationRate, double sigma){
					Genome<T>::Mutate(ran,mutationRate,sigma);}



virtual void	Mutate(int chrom, int locus, int alleleNum, T newAlleleVal, locusComponents site){
  					set(chrom, locus,alleleNum,newAlleleVal, site);}
  					/*changes the value of an allele*/
  			
virtual void	Mutate(int /*chrom*/, int /*locus*/, int /*alleleNum*/, T /*newAlleleVal*/){
  					cout<<"ERROR:  Shouldn't arrive here.  "<<endl; 
  					cout<<"Call RegulatedGenome<T>::Mutate(int,int,int,T,locusComponents) instead."<<endl;
  					}
  				
  			
virtual void	set(int chrom, int locus, int alleleNum, T newAlleleVal, locusComponents site);
  				/*changes the value of an allele
  				use when doing reproduction, etc.
  				same as mutate, but term is not so process-laden
  				USES ZERO-BASED INDICES FOR locus AND alleleNum*/
  				
virtual void	set(int /*chrom*/, int /*locus*/, int /*alleleNum*/, T /*newAlleleVal*/){
  					cout<<"ERROR:  Shouldn't arrive here.  "<<endl; 
  					cout<<"Call RegulatedGenome<T>::set(int,int,int,T,locusComponents) instead."<<endl;
  					}
  				
  			
virtual void	SetDevPath(ObjList<aList<int> >& path);
  		
virtual void	SetGenotypeString(aString& geneStr, int /* alleleLen=4 */);
	
virtual void	SetPhenotype(void);
virtual void	SetPhenotype(Ran3& ran, double heritability);
				
  	
  	};//end class RegulatedGenome
  	
  	


#endif
