/******************************************************
*
*	RegulatedGenome2.h
*
*	This class implements a regulated genetic pathway that has 
*	a promoter and product at each locus, and 2 states for promoters & products.
*	All loci are unlinked, and the chromosmoe number is diploid  
*	It handles cloning and sexual reproduction.
*
*	Base class:  Phenotype
*
*******************************************************/

#ifndef RegulatedGenome2h
#define RegulatedGenome2h = 1

#include <iostream.h>
#include "RandomList.h"
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "Phenotype.h"
#include "CoreGenome.h"
#include "RegulatedAllele2.h"
#include <math.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"

class RegulatedGenome2: public CoreGenome {
  public:
	ObjList<ObjList<RegulatedAllele2> > genes_;//locus first, then ploidy; all unlinked here

    ObjList<aList<int> > devPath_;//stores developmental pathways as locus names
    	//order in which loci influence one another
    	
  public:
  	RegulatedGenome2(void){
  		genes_.Append(NEW ObjList<RegulatedAllele2>());
  		devPath_.Append(NEW aList<int>());
  		}
  	RegulatedGenome2(int numLoci, int ploidy, float* alleleVal, float* promoValue);
  			//creates a 'null' homozygous individual at all loci

  	RegulatedGenome2(RegulatedGenome2& g): CoreGenome(g), 
  											genes_(g.genes_), devPath_(g.devPath_){
  		}
  	RegulatedGenome2(RegulatedGenome2* g): CoreGenome(g), 
  											genes_(g->genes_), devPath_(g->devPath_){
  		}
  	RegulatedGenome2(ViabilitySelectionFunc viabSelection):CoreGenome(viabSelection){
  		genes_.Append(NEW ObjList<RegulatedAllele2>());
  		devPath_.Append(NEW aList<int>());
  		}//creates empty list

  	RegulatedGenome2(int numLoci, int ploidy, 
  					float* alleleVal, float* promoValue, ViabilitySelectionFunc viabSelection);
  			//creates a 'null' homozygous individual at all loci
  	RegulatedGenome2(RegulatedGenome2& g,ViabilitySelectionFunc viabSelection):
  					CoreGenome(g), genes_(g.genes_), devPath_(g.devPath_) {
  		SetSelectionFunction(viabSelection);
  		}
virtual ~RegulatedGenome2(void){
						genes_.Clear();
						devPath_.Clear();
						}//end destructor


virtual Phenotype* new_Phenotype(void){
					return NEW RegulatedGenome2();}
	
virtual Phenotype* Duplicate(void){
					return NEW RegulatedGenome2(*this);}
	

virtual double  additiveRelativePhenotype(void);//additive phenotype

virtual int		AllelePresent (int loc, aString& alleleVal);

virtual int		chroms1N(void);//last are the sex chromosomes

virtual int		CountAlleles (int loc, aString& alleleVal);
				//counts the absolute frequency of alleleVal

virtual void	Fertilize (CoreGenome* momGamete, CoreGenome* dadGamete);

virtual void	Gamete (CoreGenome* theparentGenes, Ran3List& ran, double mutationRate, double sigma);

virtual int 	GenePresent(int locus, int alleleNum){
					return ( locus<loci() && alleleNum < ploidy() );}

virtual aString* 	GenotypeString(char* separator=" ");
					//returns all ploidy copies separated by parentheses (A)(A) at each locus
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(int locus, char* separator=" ");
					//finds nth overall locus and returns all ploidy copies separated by parentheses (A)(A)
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(int locus, int alleleNum,  char* separator=" ");
					//finds nth overall locus and returns the alleleNum_th copy

virtual RegulatedAllele2&
					GetGene(int locus, int alleleNum){
						return genes_[locus][alleleNum];}

virtual int		IsHeterozygote (int locus);
				/*returns false=0 if haploid*/

virtual int 	loci(void);

virtual void	Mutate(Ran3& ran, double mutationRate, double sigma=0.0);//sigma is s.d. of mutation effect size

virtual double	phenotype(int i=0){
					return trait(i);}

virtual int 	ploidy(int locus = 0){
					return genes_[locus].len();}


virtual void	Print(ostream& outFile, int alleleLen=4);


virtual void	set(int locus, int alleleNum, int allelePart, float newAlleleVal);
  				/*changes the value of an allele
  				use when doing reproduction, etc.
  				same as mutate, but term is not so process-laden
  				USES ZERO-BASED INDICES FOR locus AND alleleNum.
  				allelePart is the product or promoter part of the allele*/
  				
  			
virtual void	SetDevPath(ObjList<aList<int> >& path);
  		
virtual void	SetGenotypeString(aString& geneStr, int /* alleleLen=4 */);
	
virtual void	SetPhenotype(void);
virtual void	SetPhenotype(Ran3& ran, double heritability);

virtual void 	SetSexualZygote(CoreGenome* momGenes, CoreGenome* dadGenes, Ran3List& ran, double mutationRate, double sigma);

	
				/*uses mom's and dads genes to seed the genotype*/









RegulatedGenome2& operator=(RegulatedGenome2& c);

friend int operator==(RegulatedGenome2& a, RegulatedGenome2& b);
friend int operator!=(RegulatedGenome2& a, RegulatedGenome2& b);
	
				
  	
  	};//end class RegulatedGenome2
  	



#endif
