/******************************************************
*
*	RegulatedSexChrom1.h
*
*	This Chromosome is either X or Y
* 	It holds a list of RegulatedAllele1 objects
*
*******************************************************/

#ifndef RegulatedSexChrom1h
#define RegulatedSexChrom1h = 1

#include <iostream.h>
#include "Randomness.h"
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "InlineTemplates.h"
#include "RegulatedAllele1.h"
#include "MemoryManager.h"

class RegulatedSexChromosome1{
  protected:
  	int XorY_;//X=0 or Y=1 chromosome
  	ObjList<RegulatedAllele1> genes_;
  	aList<float> mapDist_;//assumes transitivity: AC = AB+BC
  public:
  	RegulatedSexChromosome1(int XorY):XorY_(XorY){
		}/*creates empty list*/

  	RegulatedSexChromosome1(int numLoci, float alleleVal, float promoVal, float mapDist, int XorY);
  			/*creates a 'null' chromosome with same allele value at all loci,
  			and same map distance between loci
  			 mapDist*(numLoci-1) must be 100*/

  	RegulatedSexChromosome1(aList<float>& alleleVal, aList<float>& promoVal, aList<float>& mapDist, int XorY);
  	RegulatedSexChromosome1(int numLoci, RegulatedAllele1& allele, float mapDist, int XorY);
  			/*creates a 'null' chromosome with same allele at all loci,
  			and same map distance between loci
  			 mapDist*(numLoci-1) must be 100*/
	RegulatedSexChromosome1(RegulatedSexChromosome1& old);/*copies the RegulatedSexChromosome1 during construction*/
	RegulatedSexChromosome1(RegulatedSexChromosome1* old);/*copies the RegulatedSexChromosome1 during construction*/
virtual ~RegulatedSexChromosome1(void){
		genes_.Clear();
		mapDist_.Clear();
		}/*destroys the list*/
  	
virtual RegulatedSexChromosome1*	Duplicate (void){
			return NEW RegulatedSexChromosome1(*this);}



virtual void	Copy (RegulatedSexChromosome1&);/*copies to an existing RegulatedSexChromosome1*/
virtual void	Copy (RegulatedSexChromosome1*);/*copies to an existing RegulatedSexChromosome1*/
	
virtual aString* GenotypeString(char* separator);
virtual aString* GenotypeString(int allele, char* separator);

RegulatedAllele1& GetAllele (int locus);
virtual float 	GetAlleleValue (int locus);
virtual float 	GetPromoValue (int locus);

virtual int 	IsX(void){return ! XorY_;}//is X chromosome
virtual int 	IsY(void){return XorY_;}//is X chromosome

virtual int 	loci(void){return genes_.len();}

virtual void 	Mutate(Ran3& ran, double mutationRate, double sigma);

virtual void 	Print(ostream& outFile, char* separator);

virtual void	Recombine(RegulatedSexChromosome1& c1, RegulatedSexChromosome1& c2, Ran3& ran);
					//makes a recombinant RegulatedSexChromosome2 from c1 & c2 
					//based on map distances among loci 
					//c1 and c2 emptied of alleles

virtual void	Recombine(RegulatedSexChromosome1* c1, RegulatedSexChromosome1* c2, Ran3& ran){
					//c1 and c2 emptied of alleles
					Recombine(*c1, *c2, ran);}

virtual void	RecombineWith(RegulatedSexChromosome1& c2, Ran3& ran);
				/*makes 2 recombinant RegulatedSexChromosome2 from this & c2 
				based on map distances among loci */

virtual void	RecombineWith(RegulatedSexChromosome1* c2, Ran3& ran){
					RecombineWith(*c2, ran);}

virtual void 			set(int locus, float newValue, int allelePart);
				//changes the value of an allele



virtual int				XorY(void){
					return XorY_;}

RegulatedSexChromosome1& operator= (RegulatedSexChromosome1& c2);

RegulatedAllele1& operator[] (int locus){
						return GetAllele(locus);}

	


friend int operator== (RegulatedSexChromosome1& c1, RegulatedSexChromosome1& c2);
		
friend int operator!= (RegulatedSexChromosome1& c1, RegulatedSexChromosome1& c2);

  	};/*end class RegulatedSexChromosome1*/
  	
  	
int operator== (RegulatedSexChromosome1& g1, RegulatedSexChromosome1& g2);
		
int operator!= (RegulatedSexChromosome1& g1, RegulatedSexChromosome1& g2);



#endif
