/******************************************************
*
*	RegulatedSexChrom2.h
*
*	This Chromosome is either X or Y
* 	It holds a list of RegulatedAllele2 objects
*
*******************************************************/

#ifndef RegulatedSexChrom2h
#define RegulatedSexChrom2h = 1

#include <iostream.h>
#include "Randomness.h"
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "InlineTemplates.h"
#include "RegulatedAllele2.h"
#include "MemoryManager.h"

class RegulatedSexChromosome2{
  protected:
  	int XorY_;//X=0 or Y=1 chromosome
  	ObjList<RegulatedAllele2> genes_;
  	aList<float> mapDist_;//assumes transitivity: AC = AB+BC
  public:
  	RegulatedSexChromosome2(int XorY):XorY_(XorY){
		}/*creates empty list*/

  	RegulatedSexChromosome2(int numLoci, float alleleVal, float promoVal, float mapDist, int XorY);
  			/*creates a 'null' chromosome with same allele value at all loci,
  			and same map distance between loci
  			 mapDist*(numLoci-1) must be 100*/

  	RegulatedSexChromosome2(aList<float>& alleleVal, aList<float>& promoVal, aList<float>& mapDist, int XorY);
  	RegulatedSexChromosome2(int numLoci, RegulatedAllele2& allele, float mapDist, int XorY);
  			/*creates a 'null' chromosome with same allele at all loci,
  			and same map distance between loci
  			 mapDist*(numLoci-1) must be 100*/
	RegulatedSexChromosome2(RegulatedSexChromosome2& old);/*copies the RegulatedSexChromosome2 during construction*/
	RegulatedSexChromosome2(RegulatedSexChromosome2* old);/*copies the RegulatedSexChromosome2 during construction*/
virtual ~RegulatedSexChromosome2(void){
		genes_.Clear();
		mapDist_.Clear();
		}/*destroys the list*/
  	
virtual RegulatedSexChromosome2*	Duplicate (void){
			return NEW RegulatedSexChromosome2(*this);}



virtual void	Copy (RegulatedSexChromosome2&);/*copies to an existing RegulatedSexChromosome2*/
virtual void	Copy (RegulatedSexChromosome2*);/*copies to an existing RegulatedSexChromosome2*/
	
virtual aString* GenotypeString(char* separator);
virtual aString* GenotypeString(int allele, char* separator);

RegulatedAllele2& GetAllele (int locus);
virtual float 	GetAlleleValue (int locus, int dimension);
virtual float 	GetPromoValue (int locus, int dimension);

virtual int 	IsX(void){return ! XorY_;}//is X chromosome
virtual int 	IsY(void){return XorY_;}//is X chromosome

virtual int 	loci(void){return genes_.len();}

virtual void 	Mutate(Ran3& ran, double mutationRate, double sigma);

virtual void 	Print(ostream& outFile, char* separator);

virtual void	Recombine(RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2, Ran3& ran);
					//makes a recombinant RegulatedSexChromosome2 from c1 & c2 
					//based on map distances among loci 
					//c1 and c2 emptied of alleles

virtual void	Recombine(RegulatedSexChromosome2* c1, RegulatedSexChromosome2* c2, Ran3& ran){
					//c1 and c2 emptied of alleles
					Recombine(*c1, *c2, ran);}

virtual void	RecombineWith(RegulatedSexChromosome2& c2, Ran3& ran);
				/*makes 2 recombinant RegulatedSexChromosome2 from this & c2 
				based on map distances among loci */

virtual void	RecombineWith(RegulatedSexChromosome2* c2, Ran3& ran){
					RecombineWith(*c2, ran);}

virtual void 			set(int locus, float newValue, int allelePart);
				//changes the value of an allele



virtual int				XorY(void){
					return XorY_;}

RegulatedSexChromosome2& operator= (RegulatedSexChromosome2& c2);

RegulatedAllele2& operator[] (int locus){
						return GetAllele(locus);}

	


friend int operator== (RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2);
		
friend int operator!= (RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2);
/*  	
friend int operator< (RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2);
  	
friend int operator> (RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2);
  	
friend int operator<= (RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2);
  	
friend int operator>= (RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2);
*/  	



  	};/*end class RegulatedSexChromosome2*/
  	
  	
int operator== (RegulatedSexChromosome2& g1, RegulatedSexChromosome2& g2);
		
int operator!= (RegulatedSexChromosome2& g1, RegulatedSexChromosome2& g2);
/*
int operator> (RegulatedSexChromosome2& g1, RegulatedSexChromosome2& g2);

int operator< (RegulatedSexChromosome2& g1, RegulatedSexChromosome2& g2);

int operator>= (RegulatedSexChromosome2& g1, RegulatedSexChromosome2& g2);

int operator<= (RegulatedSexChromosome2& g1, RegulatedSexChromosome2& g2);
*/

#endif
