/******************************************************
*
*	RegulatedSexGenome1.h
*
*	This class implements a regulated genetic pathway that has 
*	a promoter and product at each locus, and 1 state for promoter & product.
*	All loci are unlinked, and the chromosome number is diploid  
*	It handles cloning and sexual reproduction.
*
*	It includes a separate sex chromosome, X or Y
*	Here, XX is female and XY is male

	In chromosome designations, the sex-chromosome is the LAST chromosome.
*
*	Base class:  RegulatedGenome2(Phenotype)
*
*******************************************************/

#ifndef RegulatedSexGenome1h
#define RegulatedSexGenome2h = 1

#include <iostream.h>
#include "RandomList.h"
#include "StringClass.h"
#include "Element.cpp"
#include "aList.cpp"
#include "Phenotype.h"
#include "CoreGenome.h"
#include "RegulatedAllele1.h"
#include "RegulatedGenome1.h"
#include "RegulatedSexChrom1.h"
#include <math.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"

class RegulatedSexGenome1: public RegulatedGenome1{
  public:
	ObjList<RegulatedSexChromosome1> sexGenes_;//Sex-linked genes here
		//can be XX, XO, or XY
		//loci numbered first on the autosomes,
		//	then on the X-chromosome,		
		//		then on the Y-chromosome (if any)
  public:

  	RegulatedSexGenome1(void);
  		//creates empty genome

  	RegulatedSexGenome1(int numLoci, int ploidy, 
  			float alleleVal, float promoValue, int sex, 
  			int numX_genes, int numY_genes);
		//creates a 'null' homozygous individual at all loci
  		
  	

  	RegulatedSexGenome1(RegulatedSexGenome1& g);
  		//copy constructor

  	RegulatedSexGenome1(RegulatedSexGenome1* g);
  		//copy constructor

  	RegulatedSexGenome1(ViabilitySelectionFunc viabSelection);
		//creates empty genome

  	RegulatedSexGenome1(int numLoci, int ploidy, 
  			float alleleVal, float promoValue, int sex, 
  			int numX_genes, int numY_genes, 
  			ViabilitySelectionFunc viabSelection);
  		//creates a 'null' homozygous individual at all loci

  	RegulatedSexGenome1(RegulatedSexGenome1& g,ViabilitySelectionFunc viabSelection);
  		//copy constructor

virtual ~RegulatedSexGenome1(void);


virtual Phenotype* new_Phenotype(void){
					return NEW RegulatedSexGenome1();}
	
virtual Phenotype* Duplicate(void){
					return NEW RegulatedSexGenome1(*this);}
	

virtual double  additiveRelativePhenotype(void);//additive phenotype

virtual int		AllelePresent (int loc, aString& alleleVal);

virtual int		CountAlleles (int loc, aString& alleleVal);
				//counts the absolute frequency of alleleVal

virtual void	Fertilize (CoreGenome* momGamete, CoreGenome* dadGamete);
					//virtual-copies parent genes to new zygote (=*this) during reproduction
					//gametes remain intact

virtual void	Gamete (CoreGenome* theparentGenes, Ran3List& ran, double mutationRate, double sigma);

virtual int 	GenePresent(int locus, int alleleNum);

virtual aString* 	GenotypeString(char* separator=" ");
					//returns all ploidy copies separated by parentheses (A)(A) at each locus
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(int locus, char* separator=" ");
					//finds nth overall locus and returns all ploidy copies separated by parentheses (A)(A)
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(int locus, int alleleNum,  char* separator=" ");
					//finds nth overall locus and returns the alleleNum_th copy

virtual RegulatedAllele2&
					GetGene(int locus, int alleleNum);
					
					
virtual int		IsHeterozygote (int locus);
				/*returns false=0 if haploid*/

virtual int 	loci(void);

virtual void	Mutate(Ran3& ran, double mutationRate, double sigma=0.0);//sigma is s.d. of mutation effect size

virtual int 	ploidy(int locus = 0);


virtual void	Print(ostream& outFile, int alleleLen=4);


virtual void	set(int locus, int alleleNum, int allelePart, float newAlleleVal);
  				/*changes the value of an allele
  				use when doing reproduction, etc.
  				same as mutate, but term is not so process-laden
  				USES ZERO-BASED INDICES FOR locus AND alleleNum.
  				allelePart is the product or promoter part of the allele*/
  				
  			
virtual void	SetGenotypeString(aString& geneStr, int /* alleleLen=4 */);
	
virtual void	SetPhenotype(void);
virtual void	SetPhenotype(Ran3& ran, double heritability);

virtual void 	SetSexualZygote(CoreGenome* momGenes, CoreGenome* dadGenes, Ran3List& ran, double mutationRate, double sigma);

virtual int sex(void);//returns the gender from the genome

	
RegulatedSexGenome1& operator=(RegulatedSexGenome1& c);

friend int operator==(RegulatedSexGenome1& a, RegulatedSexGenome1& b);
friend int operator!=(RegulatedSexGenome1& a, RegulatedSexGenome1& b);
	
				
  	
  	};//end class RegulatedSexGenome1
  	



#endif
