/******************************************************
*
*	TwoStateGenome.h
*
*	This class implements a simple genome that has 
*	unlinked diploid loci, each with allelic states of 0 & 1.
*	It handles cloning and sexual reproduction.
*
*	Base class:  Phenotype, CoreGenome
*
*******************************************************/

#ifndef TwoStateGenomeh
#define TwoStateGenomeh = 1

#include <iostream.h>
#include "Randomness.h"
#include "RandomList.h"
#include "StringClass.h"
#include "Element.h"
#include "aList.h"
#include "Phenotype.h"
#include "CoreGenome.h"
#include "TwoStateAllele.h"
#include <math.h>
#include "InlineTemplates.h"
#include "MemoryManager.h"

class TwoStateGenome: public CoreGenome {
  public:
//	ObjList<ObjList<TwoStateAllele> > genes_;//ploidy first, then locus; all unlinked here
  private:
  	unsigned loci_, ploidy_;/*ORDER DEPENDENCY, DON'T SWITCH THESE*/
  	TwoStateAllele *genes_;

  public:
  	TwoStateGenome(void):loci_(0),ploidy_(0),genes_(NULL){
  	TwoStateGenome(int numLoci, int ploidy, int alleleVal=0);
  			//creates a 'null' homozygous individual at all loci

  	TwoStateGenome(TwoStateGenome& g);

  	TwoStateGenome(TwoStateGenome* g);

  	TwoStateGenome(ViabilitySelectionFunc viabSelection):
  			CoreGenome(viabSelection),loci_(0),ploidy_(0),genes_(NULL){
  		}//creates empty list

  	TwoStateGenome(int numLoci, int ploidy, 
  					int alleleVal, ViabilitySelectionFunc viabSelection);
  			//creates a 'null' homozygous individual at all loci
  	TwoStateGenome(TwoStateGenome& g,ViabilitySelectionFunc viabSelection):
  					CoreGenome(g),loci_(g.loci_),ploidy_(g.ploidy_), genes_(g.genes_) {
  		SetSelectionFunction(viabSelection);
  		}
virtual ~TwoStateGenome(void){
						delete [] genes_;
						}//end destructor


virtual Phenotype* new_Phenotype(void){
					return NEW TwoStateGenome();}
	
virtual Phenotype* Duplicate(void){
					return NEW TwoStateGenome(*this);}
	

virtual double 	additiveRelativePhenotype(void);//returns additive phenotype

virtual int		AllelePresent (int loc, aString& alleleVal);

virtual int		CountAlleles (int loc, aString& alleleVal);
				//counts the absolute frequency of alleleVal

virtual void	Fertilize (CoreGenome* momGamete, CoreGenome* dadGamete);

virtual void	Gamete (CoreGenome* theparentGenes, Ran3List& ran, double mutationRate, double /*sigma*/);

virtual aString* 	GenotypeString(char* separator=" ");
					//returns all ploidy copies separated by parentheses (A)(A) at each locus
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(int locus, char* separator=" ");
					//finds nth overall locus and returns all ploidy copies separated by parentheses (A)(A)
					//this permits complex alleles to be represented easily

virtual aString* 	GenotypeString(int locus, int alleleNum,  char* separator=" ");
					//finds nth overall locus and returns the alleleNum_th copy

virtual int		IsHeterozygote (int locus);
				/*returns false=0 if haploid*/

virtual int 	loci(void);

virtual void	Mutate(Ran3& ran, double mutationRate, double /*sigma=0.0*/);//sigma is s.d. of mutation effect size

virtual double	phenotype(int i=0){
					return trait(i);}

virtual int 	ploidy(int /*locus = 0*/){
					return ploidy_;}


virtual void	Print(ostream& outFile, int alleleLen=4);


virtual void	set(int locus, int alleleNum, int newAlleleVal);
  				/*changes the value of an allele
  				use when doing reproduction, etc.
  				same as mutate, but term is not so process-laden
  				USES ZERO-BASED INDICES FOR locus AND alleleNum.
  				allelePart is the product or promoter part of the allele*/
  				
  			
virtual void	SetGenotypeString(aString& geneStr, int /* alleleLen=4 */);
	
virtual void	SetPhenotype(void);
virtual void	SetPhenotype(Ran3& ran, double heritability);

virtual void 	SetSexualZygote(CoreGenome* momGenes, CoreGenome* dadGenes, Ran3List& ran, double mutationRate, double /*sigma*/);

	
				/*uses mom's and dads genes to seed the genotype*/


TwoStateGenome& operator=(TwoStateGenome& c);

friend int operator==(TwoStateGenome& a, TwoStateGenome& b);
friend int operator!=(TwoStateGenome& a, TwoStateGenome& b);
	
				
  	
  	};//end class TwoStateGenome
  	



#endif
