/******************************************************
*
*	Chromosome<T>.cpp
*
*	This class implements a list of linked loci and
*	alleles.  It handles
*	cloning and recombination reproduction.
*
*	Base class:  ObjList<Allele<int> >
*
*******************************************************/
#ifndef Chromosomecp
#define Chromosomecp = 1

#include "Chromosome.h"
#include "Element.cpp"
#include "aList.cpp"
#include "Allele.cpp"



  	
  	
//template <class T> int Chromosome<T>::AllelePresent (int loc, T alleleVal){
template <class T> int Chromosome<T>::AllelePresent (int loc, aString& alleleVal){
	Allele<T>& allele = Nth(loc);
	aString* alleleStr = allele.GenotypeString(" ");
	int answer=0;
//	if (alleleVal == allele.val() ) {return 1;}
	if (alleleVal == *alleleStr ) {answer=1;}
	delete alleleStr; alleleStr=NULL;
	return answer;
	}	//end Chromosome<T>::AllelePresent


template <class T> int Chromosome<T>::AllelePresent (int loc, Allele<T>& a){
	Allele<T>& allele = Nth(loc);
	if (a == allele ) {return 1;}
	return 0;
	}	//end Chromosome<T>::AllelePresent(int, Allele<T>&)


template <class T> void Chromosome<T>::Copy(Chromosome<T>& old){
	//copies the Chromosome
	Clear();  mapDist_.Clear(); 
	for (int j=0;j<old.loci();++j){
  		Allele<T>& oldAllele = old[j];
  		Allele<T>* allele = NEW Allele<T>(oldAllele);
  		Append(allele);
  		}//end for j
	for (int j=0;j<old.loci()-1;++j){
  		mapDist_.Append(old.mapDist(j));
  		}//end for j
  	}//end Chromosome<T>::Copy(Chromosome<T>&)
		

template <class T> void Chromosome<T>::Copy(Chromosome<T>* old){
	//copies the Chromosome
	Clear();  mapDist_.Clear(); 
	for (int j=0;j<old->loci();++j){
  		Allele<T>& oldAllele = old->Nth(j);
  		Allele<T>* allele = NEW Allele<T>(oldAllele);
  		Append(allele);
  		}//end for j
	for (int j=0;j<old->loci()-1;++j){
  		mapDist_.Append(old->mapDist(j));
  		}//end for j
  	}//end Chromosome<T>::Copy(Chromosome<T>*)
		
  	


template <class T> Chromosome<T>::Chromosome(int numLoci, T alleleVal, float mapDist){
  	//creates a 'null' homozygous individual at all loci
  	for(int j=0;j<numLoci;++j){
  		Allele<T>* a = NEW Allele<T>(alleleVal);
  		Append(a);
  		}//end for loci
  	for(int j=0;j<numLoci-1;++j){
  		mapDist_.Append(mapDist);
  		}//end for loci-1
  	}//end constructor Chromosome<T>::Chromosome(int,int,float)


template <class T> Chromosome<T>::Chromosome(int numLoci, Allele<T>& a, float mapDist){
  	//creates a 'null' homozygous individual at all loci
  	for(int j=0;j<numLoci;++j){
//  		Allele<T>* aa = NEW Allele<T>(a);
  		Allele<T>* aa = a.Duplicate();
  		Append(aa);
  		}//end for loci
  	for(int j=0;j<numLoci-1;++j){
  		mapDist_.Append(mapDist);
  		}//end for loci-1
  	}//end constructor Chromosome<T>::Chromosome(int,Allele<T>&,float)


template <class T> Chromosome<T>::Chromosome( aList<T>& alleleVal, aList<float>& mapDist):
  				mapDist_(mapDist){
  	for(int a=0;a<alleleVal.len();++a){
//  		Allele<T>* allele = NEW Allele<T>(alleleVal[a]);
  		Allele<T>* allele = NEW Allele<T>(alleleVal[a]);
  		Append(allele);
  		}//end for a
  	}//end constructor Chromosome<T>::Chromosome(aList<T>&, List<float>&)
  			
  			
template <class T> Chromosome<T>::Chromosome( ObjList<Allele<T> >& alleleList, aList<float>& mapDist):
  				mapDist_(mapDist){
  	for(int a=0;a<alleleList.len();++a){
//  		Allele<T>* allele = NEW Allele<T>(alleleList[a]);
  		Allele<T>* allele = alleleList[a].Duplicate();
  		Append(allele);
  		}//end for a
  	}//end constructor Chromosome<T>::Chromosome( ObjList<Allele<T>>&, List<float>&)
  			
  			
template <class T> Chromosome<T>::Chromosome(Chromosome<T>& old):  //copies the Chromosome during construction
  				ObjList<Allele<T> >(old), 
  				mapDist_(old.mapDist_){
				}
		
template <class T> Chromosome<T>::Chromosome(Chromosome<T>* old)://copies the Chromosome during construction
  				ObjList<Allele<T> >(*old), 
  				mapDist_(old->mapDist_){
				}



template <class T> Allele<T>& Chromosome<T>::GetAllele (int locus){ //returns the reference of the allele
	Allele<T>& a = Nth(locus);
	return a;
	}//end Chromosome<T>::GetAllele


template <class T> T Chromosome<T>::GetGene (int locus){ //returns the value of the allele
	Allele<T>& a = Nth(locus);
	return a.val();
	}//end Chromosome<T>::GetGene


template <class T>  aString* Chromosome<T>::GenotypeString(char* separator){
	aString* gstr= NEW aString();
	for(int j=0;j<loci();++j){
		Allele<T>& a = GetAllele(j);
	  	aString* allStr = a.GenotypeString(separator);
	  	gstr->cat(*allStr);
	  	if(j<loci()-1){gstr->cat(separator);}
	  	delete allStr; allStr=NULL;
	  	}//end for j
	return gstr;
  	}//end Chromosome<T>::GenotypeString(char)


template <class T>  aString* Chromosome<T>::GenotypeString(int allele, char* separator){
	Allele<T>& a = GetAllele(allele);
	return a.GenotypeString(separator);
  	}//end Chromosome<T>::GenotypeString(int, char)


template <class T> void Chromosome<T>::Mutate(Ran3& ran, double mutationRate, double sigma){
	for(int loc=0;loc<loci();++loc){
		if(ran()<mutationRate){
			Allele<T>& a = Nth(loc);
  			a.Mutate(ran,sigma);}
  		}
  	}//end Chromosome<T>::Mutate

		
template <class T> void Chromosome<T>::Print(ostream& outFile, int alleleLen){
	aString geneStr;
	SetGenotypeString(geneStr,alleleLen);
	outFile<<geneStr<<endl;
	}//end Chromosome<T>::Print


template <class T> void Chromosome<T>::Recombine(Chromosome<T>& c1, Chromosome<T>& c2, Ran3& ran){
	//makes a recombinant Chromosome from c1 & c2 
	//based on map distances among loci 
	mapDist_ = c1.mapDist_;
	//first locus can come from either c1 or c2
	Chromosome<T> presentChrom(c1);
	Chromosome<T> otherChrom(c2);
	if(ran()<0.5){SWAP(presentChrom, otherChrom);}
	Append(&(presentChrom[0]));
	for(int j=0;j<c1.loci()-1;++j){
		if(ran()<mapDist(j)){SWAP(presentChrom, otherChrom);}
		Append(&(presentChrom[j+1]));
		}//end for j
	}//end Chromosome<T>::Recombine



template <class T> void Chromosome<T>::RecombineWith(Chromosome<T>& c2, Ran3& ran){
	//makes 2 recombinant Chromosomes from this & c2 
	//based on map distances among loci 
	//first locus can come from either c1 or c2
	//ASSUMES IDENTICAL MAP DISTANCES AMONG LOCI
	Chromosome<T>* presentChrom = NEW Chromosome<T>(*this);
	Chromosome<T>* otherChrom = NEW Chromosome<T>(c2);
	if(ran()<0.5){
//		swap(presentChrom, otherChrom);
		Chromosome<T>* temp = presentChrom;
		presentChrom = otherChrom;
		otherChrom = temp;
		temp = NULL;
		}
	Clear();  c2.Clear();
	mapDist_ = presentChrom->mapDist_;
	c2.mapDist_ = otherChrom->mapDist_;
	Append(&(presentChrom->First()));
	c2.Append(&(otherChrom->First()));
	for(int j=0;j<presentChrom->loci()-1;++j){
		if(ran()<mapDist(j)){
	//		swap(presentChrom, otherChrom);
			Chromosome<T>* temp = presentChrom;
			presentChrom = otherChrom;
			otherChrom = temp;
			temp = NULL;
			}
		Append(&(presentChrom->Nth(j+1)) );
		c2.Append(&(otherChrom->Nth(j+1)) );
		}//end for j
	presentChrom->Detach();
	otherChrom->Detach();
	delete presentChrom; presentChrom=NULL;
	delete otherChrom; otherChrom=NULL;
	}//end Chromosome<T>::RecombineWith



template <class T> void Chromosome<T>::set(int locus, T newAlleleVal){
	//changes the value of an allele
  	Allele<T>& allele = Nth(locus);
  	allele.mutateTo(newAlleleVal);
  	}//end Chromosome<T>::set
	
	
template <class T> void Chromosome<T>::SetGenotypeString(aString& geneStr, int alleleLen){
	geneStr.clear();
	for(int j=0;j<loci();++j){
	  	aString allStr(GetGene(j));
	  	allStr.PrependCharToLen('0',alleleLen);
	  	geneStr.cat(allStr);
	  	if (j!=loci()-1){geneStr.cat('.');}
	  	}//end for j
	  }//end Chromosome<T>::SetGenotypeString
	
	

template <class T> int operator== (Chromosome<T>& c1, Chromosome<T>& c2){
	if(c1.loci()!=c2.loci()){return 0;}
	for (int j=0;j<c1.loci();++j){
		Allele<T>& a1=c1[j];
		Allele<T>& a2=c2[j];
		if(a1 != a2){return 0;}}
	for (int j=0;j<c1.loci()-1;++j){
		float d1=c1.mapDist(j);
		float d2=c2.mapDist(j);
		if(d1!=d2){return 0;}}
	return 1;
	}//end friend operator==
		
template <class T> int operator!= (Chromosome<T>& g1, Chromosome<T>& g2){
  	return !(g1==g2);}
  	

template <class T> int operator< (Chromosome<T>& c1, Chromosome<T>& c2){//compares by allele value, for sorting
	int shortLen = min(c1.loci(), c2.loci() );
	for (int j=0;j<shortLen;++j){
		Allele<T>& a1=c1[j];
		Allele<T>& a2=c2[j];
		if(a1 < a2){return 1;}}
	if(c1.loci() < c2.loci()){return 1;}
	return 0;//default returns false
	}//end friend operator<
  	
template <class T> int operator> (Chromosome<T>& c1, Chromosome<T>& c2){//compares by allele value, for sorting
	int shortLen = min(c1.loci(), c2.loci() );
	for (int j=0;j<shortLen;++j){
		Allele<T>& a1=c1[j];
		Allele<T>& a2=c2[j];
		if(a1 > a2){return 1;}}
	if(c1.loci() > c2.loci()){return 1;}
	return 0;//default returns false
	}//end friend operator>
  	
template <class T> int operator<= (Chromosome<T>& c1, Chromosome<T>& c2){//compares by allele value, for sorting
	return !(c1 > c2);}
  	
template <class T> int operator>= (Chromosome<T>& c1, Chromosome<T>& c2){//compares by allele value, for sorting
	return !(c1 < c2);}
  	
#endif









