/**********************************************
*
*	EpistaticGenome.cpp
*
*	This class implements an n-locus Genome with arbitrary
*	linkage relationships, where the phenotype is arbitrarily non-additive.
*	To determine phenotype and fitness, general reference lists are maintained.
* 	The reference lists must be initialized separately.
*
*	Base Class: Genome
*
**********************************************/

#ifndef EpistaticGenomecp

#define EpistaticGenomecp = 1

#include "EpistaticGenome"
#include "Element.cpp"
#include "aList.cpp"
#include "Genome.cpp"
#include "ChromosomeList.cpp"

template <class T> ObjList<ObjList<ObjList<aList<T> > > >	
						EpistaticGenome<T>::possibleGenotypes_ = ObjList<ObjList<ObjList<aList<T> > > >();
template <class T> ObjList<aList<double> > EpistaticGenome<T>::possiblePhenotypes_ = ObjList<aList<double> >();
template <class T> ObjList<aList<double> > EpistaticGenome<T>::possibleFitnesses_ = ObjList<aList<double> >();


template <class T> Phenotype* EpistaticGenome<T>::Duplicate(void){
	Phenotype* p = new_Phenotype();
	Genome<T>* g = (Genome<T>*) p;
	g->Copy(*this);
	return g;
	}//end EpistaticGenome<T>::Duplicate




template <class T> ObjList<aList<T> >* EpistaticGenome<T>::genotypeSummary(void){
	//arranged as lists of loci, these with sorted, ploidy-length allele combinations
	ObjList<aList<T> >* summary = new ObjList<aList<T> >();
	for(int c=0;c<chroms1N();++c){
		for(int loc=0;loc<loci(c);++loc){
			summary->Append(new aList<T>());
			for(int p=0;p<PloidyAtChrom(c);++p){
				(summary->Last()).Append(GetGene(c,loc,p));}
			(summary->Last()).Sort();
			}}
	return summary;
	}//end class EpistaticGenome<T>::genotypeSummary





template <class T> double EpistaticGenome<T>::rawPhenotype(void){
	//returns the correct possiblePhenotype_ value
  	//may later be modified by other factors by Individual, where phenotype and fitness are assessed
  	ObjList<aList<T> >* summary = genotypeSummary();
  	int genotypeNum = possibleGenotypes_[kind_].Loc(summary);
  	delete summary; summary=NULL;
  	if (genotypeNum==-1){
  		cout<<"ERROR: this genotype not tabled in EpistaticGenome<T>::genotypeSummary_"<<endl;
  		return 0.0;}
 	return possiblePhenotypes_[kind_][genotypeNum]; 	 
	}//end class EpistaticGenome<T>::rawPhenotype




template <class T> double EpistaticGenome<T>::rawFitness(void){
	//returns the correct possibleFitnesses_ score
  	//may later be modified by other factors by Individual, where phenotype and fitness are assessed
  	ObjList<aList<T> >* summary = genotypeSummary();
  	int genotypeNum = possibleGenotypes_[kind_].Loc(summary);
  	delete summary; summary=NULL;
  	if (genotypeNum==-1){
  		cout<<"ERROR: this genotype not tabled in EpistaticGenome::genotypeSummary_"<<endl;
  		return 0.0;}
 	return possibleFitnesses_[kind_][genotypeNum]; 	 
	}//end class EpistaticGenome<T>::rawFitness


#endif
