/******************************************************
*
*	Genome<T>.cpp
*
*	This class implements a list of chromosomes.  
*	It handles cloning and sexual reproduction.
*
*	Base class:  Phenotype
*
*******************************************************/

#ifndef Genomecp
#define Genomecp = 1

#include "Genome.h"
#include "Element.cpp"
#include "aList.cpp"
#include "Allele.cpp"
#include "Chromosome.cpp"




template <class T> double Genome<T>::additiveRelativePhenotype(void){
	double sum=0.0, n=0.0;
	for(int ch=0;ch<chroms1N();++ch){
		for(int loc=0;loc<loci(ch);++loc){
			for(int k=0;k<PloidyAtChrom(ch);++k){
				sum += GetGene(ch,loc,k);//assumes numerical gene value
				++n;
				}}}
	return sum/n;
	}/*end Genome<T>::additiveRelativePhenotype*/
  	
  	
  	
//template <class T> int Genome<T>::AllelePresent (int chrom, int loc, T alleleVal){
template <class T> int Genome<T>::AllelePresent (int chrom, int loc, aString& alleleVal){
	ChromosomeList<T>& chromList = genome_[chrom];
	for (int ch=0;ch<chromList.len();++ch){
		Chromosome<T>& c = chromList[ch];
		if (c.AllelePresent(loc,alleleVal) ) {return 1;}}
	return 0;
	}	//end Genome<T>::AllelePresent(int,int,int)


template <class T> int Genome<T>::AllelePresent (int loc, aString& alleleVal){
	int lociSoFar = 0;
	for (int ch=0;ch<chroms1N();++ch){
		if (loc < lociSoFar + loci(ch) ){
			return AllelePresent(ch, loc - lociSoFar,alleleVal);}
		lociSoFar += loci(ch);}
	return 0;//default
	}	//end Genome<T>::AllelePresent


template <class T> int Genome<T>::AllelePresent (int chrom, int loc, Allele<T>& a){
	ChromosomeList<T>& chromList = genome_[chrom];
	for (int ch=0;ch<chromList.len();++ch){
		Chromosome<T>& c = chromList[ch];
		if (c.AllelePresent(loc,a) ) {return 1;}}
	return 0;
	}	//end Genome<T>::AllelePresent(int,int,Allele<T>&)



template <class T> void Genome<T>::Copy(Genome<T>& old){
	Phenotype::Copy(old);
	//copies the Genome
	genome_.Clear(); 
	for (int chrom=0;chrom<old.chroms1N();++chrom){
  		ChromosomeList<T>* chromList = new ChromosomeList<T>();
  		genome_.Append(chromList);
  		ChromosomeList<T>& oldChromList = old.genome_[chrom];
  		for(int ch=0;ch<oldChromList.len();++ch){
  			Chromosome<T>& oldChrom = oldChromList[ch];
  			Chromosome<T>* c = new Chromosome<T>(oldChrom);
  			chromList->Append(c);
  			}//end for ch
  		}//end for chrom
  	}//end Genome<T>::Copy(Genome<T>&)
		

template <class T> void Genome<T>::Copy(Genome<T>* old){
	Copy(*old);
  	}//end Genome<T>::Copy(Genome<T>*)


  	
template <class T> int Genome<T>::CountAlleles (int loc, aString& alleleVal){
	int lociSoFar = 0;
	for (int ch=0;ch<chroms1N();++ch){
		if (loc < lociSoFar + loci(ch) ){
			return CountAlleles(ch, loc - lociSoFar,alleleVal);}
		lociSoFar += loci(ch);}
	return 0;//default
	}	//end Genome<T>::CountAlleles


template <class T> int Genome<T>::CountAlleles (int chrom, int loc, aString& alleleVal){
	ChromosomeList<T>& chromList = genome_[chrom];
	int count=0;
	for (int ch=0;ch<chromList.len();++ch){
		Chromosome<T>& c = chromList[ch];
		if (c.AllelePresent(loc,alleleVal) ) {++count;}}
	return(count);
	}	//end Genome<T>::CountAlleles


template <class T> int Genome<T>::CountAlleles (int chrom, int loc, Allele<T>& a){
	ChromosomeList<T>& chromList = genome_[chrom];
	int count=0;
	for (int ch=0;ch<chromList.len();++ch){
		Chromosome<T>& c = chromList[ch];
		if (c.AllelePresent(loc,a) ) {++count;}}
	return(count);
	}	//end Genome<T>::CountAlleles


template <class T> void Genome<T>::Detach(void){//release but don't destroy chromosomes
	for(int chrom=0;chrom<chroms1N();++chrom){
		ChromosomeList<T>& chromList = genome_[chrom];
		chromList.Detach();
		}//end for chrom
	genome_.Clear();
	}//end Genome<T>::Detach()



template <class T> void Genome<T>::Fertilize(CoreGenome* momGamete, CoreGenome* dadGamete){
	//fuses two gametes
	//transfers the alleles to the new zygotic genotype
	//detaches alleles from momGamete and dadGamete
//	Genome<T>* mommyGamete = (Genome<T>*) momGamete;
//	Genome<T>* daddyGamete = (Genome<T>*) dadGamete;
	Genome<T>* mommyGamete = (Genome<T>*) momGamete->Duplicate();
	Genome<T>* daddyGamete = (Genome<T>*) dadGamete->Duplicate();
	genome_.Clear();
	for (int chrom=0;chrom<mommyGamete->chroms1N();++chrom){
		ChromosomeList<T>* chromList = new ChromosomeList<T>();
		genome_.Append(chromList);
		ChromosomeList<T>& momChromList = mommyGamete->genome_[chrom];
		ChromosomeList<T>& dadChromList = daddyGamete->genome_[chrom];
		for (int ch=0;ch<momChromList.len();++ch){
			chromList->Append(&(momChromList[ch]));}
		for (int ch=0;ch<dadChromList.len();++ch){
			chromList->Append(&(dadChromList[ch]));}
		momChromList.Detach();
		dadChromList.Detach();
		}//end for chrom
	}//end Genome<T>::Fertilize



template <class T> void Genome<T>::Gamete(CoreGenome* theParentGenes, Ran3List& ran, double mutationRate, double sigma){
	//does meiosis to turn the genotype into a gamete of parentGenes
	//robust to haplodiploidy, genetic sex determination of XY or XO type, and polyploidy
	genome_.Clear();
	Genome<T>* parentGenes = (Genome<T>*) theParentGenes;
	Genome<T>* oldGenes = (Genome<T>*) theParentGenes->Duplicate();
	//work with a new copy to sample without replacement
	//and to avoid permanent recombination
	//determine if all loci haploid --robust to genetic sex determination

	for (int chrom=0;chrom<parentGenes->chroms1N();++chrom){
  		ChromosomeList<T>* chromList = new ChromosomeList<T>();
  		genome_.Append(chromList);
  		ChromosomeList<T>& parentChromList = oldGenes->genome_[chrom];
  		int ploidy = parentGenes->PloidyAtChrom(chrom);
  		int chromsToMake = ploidy;
  		if (ploidy>1){
  			chromsToMake = ploidy/2;
  			double pl = (double)ploidy, two = 2.0;
  			if ((MOD(pl, two) != 0.0) && (ran.next() < 0.5)){
  				++chromsToMake;}//odd chromosome may or may not join
  				}
  		if(ploidy == 2){//recombination in higher ploidy not implemented
  			//check for recombination
  			Chromosome<T>& chrom1 = parentChromList[0];
  			Chromosome<T>& chrom2 = parentChromList[1];
  			chrom1.RecombineWith(&chrom2, ran);
  			}//end if ploidy==2
  				
  		for(int ch=0;ch<chromsToMake;++ch){
  			//samples oldGenes without replacement for proper treatment of polyploids
 			long num=ran.longBetween(0,parentChromList.len()-1);
  			Chromosome<T>* parentChrom = &(parentChromList[num]);
 			chromList->Append(parentChrom);
 			parentChromList.Detach(num);
 			if (mutationRate>0.0){
 				parentChrom->Mutate(ran, mutationRate, sigma);}
  			}//end for ch
  		}//end for chrom
	delete oldGenes;oldGenes=0;
	}//end Genome<T>::Gamete




template <class T> Genome<T>::Genome(int numLociPerChrom, int num1NChroms, int ploidy, T alleleVal): Phenotype(){
  	//creates a 'null' homozygous individual at all loci
  	float mapDist = 0.0;
  	if(numLociPerChrom>1){mapDist = 1.0/((float) num1NChroms-1.0);}
  	for(int chrom=0;chrom<num1NChroms;++chrom){
  		ChromosomeList<T>* chromList = new ChromosomeList<T>();
  		genome_.Append(chromList);
  		for (int k=0;k<ploidy;++k){
  			Chromosome<T>* ch = new Chromosome<T>(numLociPerChrom,alleleVal, mapDist);
  			chromList->Append(ch);
  			}//end for k
  		}//end for chrom
  	}//end constructor Genome<T>::Genome(int,int,int,T)



  			
template <class T> Genome<T>::Genome(int numLociPerChrom, int num1NChroms, int ploidy, T alleleVal, ViabilitySelectionFunc viabSelection):
		Phenotype(viabSelection){
  	//creates a 'null' homozygous individual at all loci
  	float mapDist = 0.0;
  	if(numLociPerChrom>1){mapDist = 1.0/((float) num1NChroms-1.0);}
  	for(int chrom=0;chrom<num1NChroms;++chrom){
  		ChromosomeList<T>* chromList = new ChromosomeList<T>();
  		genome_.Append(chromList);
  		for (int k=0;k<ploidy;++k){
  			Chromosome<T>* ch = new Chromosome<T>(numLociPerChrom,alleleVal, mapDist);
  			chromList->Append(ch);
  			}//end for k
  		}//end for chrom
  	}//end constructor Genome::Genome(int,int,int,T,ViabilitySelectionFunc)



  			
template <class T> Genome<T>::Genome(Genome<T>& old){
	//copy the phenotype
	selectionFunc = old.selectionFunc;
	for (int t=0;t<old.trait_.len();++t){
		trait_.Append(old.trait_[t]);}
	//copy the Genome 
	for (int chrom=0;chrom<old.chroms1N();++chrom){
  		ChromosomeList<T>* chromList = new ChromosomeList<T>();
  		genome_.Append(chromList);
  		ChromosomeList<T>& oldChromList = old.genome_[chrom];
  		for(int ch=0;ch<oldChromList.len();++ch){
  			Chromosome<T>& oldChrom = oldChromList[ch];
  			Chromosome<T>* c = new Chromosome<T>(oldChrom);
  			chromList->Append(c);
  			}//end for ch
  		}//end for chrom
  	}//end constructor Genome<T>::Genome(Genome<T>&)
		
		
		
template <class T> Genome<T>::Genome(Genome<T>* old){
	//copy the phenotype
	selectionFunc = old->selectionFunc;
	for (int t=0;t<old->trait_.len();++t){
		trait_.Append(old->trait_[t]);}
	//copy the Genome 
	for (int chrom=0;chrom<old->chroms1N();++chrom){
  		ChromosomeList<T>* chromList = new ChromosomeList<T>();
  		genome_.Append(chromList);
  		ChromosomeList<T>& oldChromList = old->genome_[chrom];
  		for(int ch=0;ch<oldChromList.len();++ch){
  			Chromosome<T>& oldChrom = oldChromList[ch];
  			Chromosome<T>* c = new Chromosome<T>(oldChrom);
  			chromList->Append(c);
  			}//end for ch
  		}//end for chrom
  	}//end constructor Genome<T>::Genome(Genome<T>*)
		
		
		
template <class T> aString* Genome<T>::GenotypeString(char separator){
		char* sepChar = &separator;
		return GenotypeString(sepChar);
		}//end Genome<T>::GenotypeString


template <class T> aString* Genome<T>::GenotypeString(char* separator){
	//returns all ploidy copies separated by parentheses (A)(A) at each locus
	//this permits complex alleles to be represented easily
	aString* gtstr = new aString();
	for (int locus=0;locus<loci();++locus){
		aString* gstr = GenotypeString(locus,separator);
		gtstr->cat(*gstr);
		gtstr->cat(separator);
		delete gstr;gstr=NULL;
		}//end for locus
	return gtstr;
  	}//end Genome<T>::GenotypeString
  	

template <class T> aString* Genome<T>::GenotypeString(int locus, char* separator){
	//returns the string of an allele
	aString* gtstr = new aString();
	int lociSoFar = 0;
	char spar='(', epar=')';
	for (int ch=0;ch<chroms1N();++ch){
		if (locus < lociSoFar + loci(ch) ){
			for(int alleleNum=0;alleleNum<PloidyAtChrom(ch);++alleleNum){
				aString* gstr = GenotypeString(ch, locus - lociSoFar,alleleNum,separator);
				gstr->prepend(spar);gstr->cat(epar);
				gtstr->cat(*gstr);
				delete gstr;gstr=NULL;
				}//end for alleleNum
			return gtstr;
			}//end for locus
		lociSoFar += loci(ch);}
	return gtstr;//default
  	}//end Genome<T>::GenotypeString
  	

template <class T> aString* Genome<T>::GenotypeString(int locus, int alleleNum, char* separator){
	//returns the string of an allele
	int lociSoFar = 0;
	for (int ch=0;ch<chroms1N();++ch){
		if (locus < lociSoFar + loci(ch) ){
			return GenotypeString(ch, locus - lociSoFar,alleleNum,separator);
			}//end for ch
		lociSoFar += loci(ch);}
	return new aString();//default
  	}//end Genome<T>::GenotypeString
  	

template <class T> aString* Genome<T>::GenotypeString(int chrom, int locus, int alleleNum, char* separator){
	//returns the string of an allele
  	ChromosomeList<T>& chromList = genome_[chrom];
  	Chromosome<T>& c = chromList[alleleNum];
  	return c.GenotypeString(locus,separator);
  	}//end Genome<T>::GenotypeString
  	

template <class T> Allele<T>& Genome<T>::GetAllele(int chrom, int locus, int alleleNum){
	//returns the value of an allele
  	ChromosomeList<T>& chromList = genome_[chrom];
  	Chromosome<T>& c = chromList[alleleNum];
  	return c.GetAllele(locus);
  	}//end Genome<T>::GetAllele
  	

template <class T> Allele<T>& Genome<T>::GetAllele(int locus, int alleleNum){
	//returns the value of an allele
	int lociSoFar = 0;
	for (int ch=0;ch<chroms1N();++ch){
		if (locus < lociSoFar + loci(ch) ){
			return GetAllele(ch, locus - lociSoFar,alleleNum);}
		lociSoFar += loci(ch);}
	return  GetAllele(0,0);//default
  	}//end Genome<T>::GetAllele
  	

template <class T> T Genome<T>::GetGene(int chrom, int locus, int alleleNum){
	//returns the value of an allele
  	ChromosomeList<T>& chromList = genome_[chrom];
  	Chromosome<T>& c = chromList[alleleNum];
  	return c.GetGene(locus);
  	}//end Genome<T>::GetGene
  	

template <class T> T Genome<T>::GetGene(int locus, int alleleNum){
	//returns the value of an allele
	int lociSoFar = 0;
	for (int ch=0;ch<chroms1N();++ch){
		if (locus < lociSoFar + loci(ch) ){
			return GetGene(ch, locus - lociSoFar,alleleNum);}
		lociSoFar += loci(ch);}
	return 0;//default
  	}//end Genome<T>::GetGene
  	

template <class T> int Genome<T>::IsHeterozygote (int locus){
	//returns FALSE=0 if no genotype available, or if haploid
  	int lociSoFar = 0;
	for (int ch=0;ch<chroms1N();++ch){
		if (locus < lociSoFar + loci(ch) ){
			return IsHeterozygote(ch, locus - lociSoFar);}
		lociSoFar += loci(ch);}
	return 0;//default
	}//end Genome<T>::IsHeterozygote


template <class T> int Genome<T>::IsHeterozygote (int chrom, int locus){
	//returns FALSE=0 if no genotype available, or if haploid
  	ChromosomeList<T>& chromList = genome_[chrom];
  	if(chromList.len()<2){return 0;}//it's haploid 
  	Chromosome<T>& c1 = chromList[0];
  	Allele<T>& a1=c1.GetAllele(locus);
  	for(int ch=1;ch<chromList.len();++ch){//handles polyploidy
  		Chromosome<T>& c2 = chromList[ch];
  		Allele<T>& a2=c2.GetAllele(locus);
  		if(a1 != a2){return 1;}//it's a heterozygote
  		}//end for ch
  	//it must be a homozygote
  	return 0;
	}//end Genome<T>::IsHeterozygote


template <class T> int Genome<T>::loci(void){
	int n=0;
	for(int chrom=0;chrom<chroms1N();++chrom){
		n += loci(chrom);}
	return n;
	}//end Genome<T>::loci
  	

template <class T> int Genome<T>::loci(int chrom){
	ChromosomeList<T>& chromList = genome_[chrom];
	Chromosome<T>& ch = chromList[0];
	return ch.loci();
	}//end Genome<T>::loci(int)
  	


template <class T> void Genome<T>::Mutate(Ran3& ran, double mutationRate, double sigma){
	if (mutationRate<=0.0){return;}
	for(int c=0;c<genome_.len();++c){
		genome_[c].Mutate(ran,mutationRate,sigma);
		}
	}//end Genome<T>::Mutate(Ran3&, double, double)



template <class T> int Genome<T>::ploidy (int locus){
	//sex-linked & haplodiploid loci have variable ploidy
	int sum=0;
	for(int chrom=0;chrom<chroms1N();++chrom){
		int n=loci(chrom);
		if(sum + n >= locus){//the right chromosome
  				ChromosomeList<T>& chromList = genome_[chrom];
 				return chromList.len();
				}
			else{
				sum += n;}		
		}
 	return -1;//error value
	}	//end Genome<T>::ploidy(int)
  	
 
template <class T> int Genome<T>::PloidyAtChrom (int chrom){
	//sex-linked & haplodiploid loci have variable ploidy
  	ChromosomeList<T>& chromList = genome_[chrom];
 	return chromList.len();
	}	//end Genome<T>::PloidyAtChrom
  	
 
template <class T> void Genome<T>::Print(ostream& outFile, int alleleLen){
	aString geneStr;//dummy starter value will be cleared anyway
	SetGenotypeString(geneStr,alleleLen);
	outFile<<geneStr<<endl;
	}//end Genome<T>::Print

  	
    		
template <class T> Allele<T>&	Genome<T>::RandomAllele(int chrom, int locus, Ran3& ran){
  	//randomly picks an allele and returns its value
  	ChromosomeList<T>& chromList = genome_[chrom];
  	long num=ran.longBetween(0,chromList.len()-1);
  	Chromosome<T>& c = chromList[num];
	return c.GetAllele(locus);
	}//end Genome<T>::RandomAllele
  	


template <class T> void Genome<T>::set(int chrom, int locus, int alleleNum, T newAlleleVal){
	//changes the value of an allele
  	ChromosomeList<T>& chromList = genome_[chrom];
  	Chromosome<T>& c = chromList[alleleNum];
  	c.set(locus, newAlleleVal);
  	}//end Genome::set
	
	
template <class T> void Genome<T>::SetGenotypeString(aString& geneStr, int alleleLen){
	geneStr.clear();
	for(int chrom=0;chrom<chroms1N();++chrom){
  		ChromosomeList<T>& chromList = genome_[chrom];
  		Chromosome<T>& cBase = chromList[0];
	  	for(int ch=0;ch<chromList.len();++ch){
  			Chromosome<T>& c = chromList[ch];
  			for(int loc=0;loc<cBase.loci();++loc){
	  			aString allStr(c.GetGene(loc));
	  			allStr.PrependCharToLen('0',alleleLen);
	  			geneStr.cat(allStr);
	  			if(loc != cBase.loci()-1){
	  				geneStr.cat('_');}
	  			}//end for loc
	  		if (ch == chromList.len()-1){
	  				geneStr.cat(' ');}
	  			else{
	  				geneStr.cat("||");}
	  		}//end for ch
	  	}//end for chrom
	  }//end Genome<T>::SetGenotypeString


template <class T> void Genome<T>::SetPhenotype(void){
	/*default is additive phenotype*/
	int additive=0;//set to 0 for adding log of gene scores, 1 otherwise
	double sum=0.0;
	for(int ch=0;ch<chroms1N();++ch){
		for(int loc=0;loc<loci(ch);++loc){
			for(int k=0;k<PloidyAtChrom(ch);++k){
				if(additive){
						sum += GetGene(ch,loc,k);}
					else{//add log scores
 						sum += log(GetGene(ch,loc,k));}
				}}}
	if(! additive){
		sum = exp(sum);}
	SetTrait(sum,0);
	}/*end Genome<T>::SetPhenotype(void)*/



template <class T> void Genome<T>::SetPhenotype(Ran3& ran, double heritability){
	/*default is additive phenotype*/
	SetPhenotype();
	if (heritability <1.0){
		double sum=trait();
		sum += ran.NormDev(0.0,1.0-heritability);
		SetTrait(sum,0);
		}
	}/*end Genome<T>::SetPhenotype(ran,double)*/



template <class T> void Genome<T>::SetSelectionParams(aList<double>& selParams){
	for(int c = 0;c<chroms1N();++c){
		for(int loc=0;loc<loci(c);++loc){
			for(int a=0;a<PloidyAtChrom(c);++a){
				double t = 0.0 + GetGene(c,loc,a);
				selParams.Append(t);}}}
	}//end Genome<T>::SetSelectionParams(aList<double>&)


template <class T> void Genome<T>::SetSexualZygote(CoreGenome* momGenes, CoreGenome* dadGenes, Ran3List& ran, double mutationRate, double sigma){
	//uses mom's and dad's genes to seed the genotype
//	Genome<T>* mommyGenes = (Genome<T>*) momGenes;
//	Genome<T>* daddyGenes = (Genome<T>*) dadGenes;
//	Genome<T>* momGamete = (Genome<T>*) mommyGenes->Duplicate();
//	Genome<T>* dadGamete = (Genome<T>*) daddyGenes->Duplicate();
	Genome<T>* momGamete = (Genome<T>*) momGenes->Duplicate();
	Genome<T>* dadGamete = (Genome<T>*) dadGenes->Duplicate();
	momGamete->Gamete(momGenes,ran,mutationRate,sigma);//genes copied to gamete
	dadGamete->Gamete(dadGenes,ran,mutationRate,sigma);
	Fertilize(momGamete, dadGamete);//momGamete & dadGamete are emptied in this step
	delete momGamete; momGamete=NULL;
	delete dadGamete; dadGamete=NULL;
	}//end Genome<T>::SetSexualZygote



	

template <class T> int operator== (Genome<T>& g1, Genome<T>& g2){
	//true if genotypes and phenotypes are identical
	if ((Phenotype) g1 != (Phenotype) g2) {return 0;}
	
	if(g1.chroms1N()!=g2.chroms1N()){return 0;}
	for (int chrom=0;chrom<g1.chroms1N();++chrom){
		ChromosomeList<T>& chromList1=g1.genome_[chrom];
		ChromosomeList<T>& chromList2 = g2.genome_[chrom];
		if(chromList1.len() != chromList2.len()){return 0;}
		chromList1.Sort();  chromList2.Sort();
		for(int ch=0;ch<chromList1.len();++ch){
			Chromosome<T>& c1=chromList1[ch];
			Chromosome<T>& c2=chromList2[ch];
			if(c1 != c2){return 0;}
			}//end for ch
			}//end for chrom
	return 1;//default returns true
	}//end friend operator==
		
template <class T> int operator!= (Genome<T>& g1, Genome<T>& g2){
  	return !(g1==g2);}
  	
  	
template <class T> void ChromosomeList<T>::Sort(void){
	Sort(0,len()-1);}


template <class T> void ChromosomeList<T>::Sort(long start, long finish){
	//implements QuickSort
	//patched to accomodate 0-indexing
	if (len()<2) {return;}
	long left= start+1, right= finish+1;
	long leftplusright = left + right, two = 2;
	Chromosome<T>& startValue = Nth(DIV(leftplusright,two)-1);
	do{
			while (Nth(left-1) < startValue) {++left;}
			while (startValue < Nth(right-1)) {--right;}
			if (left <= right) {
				Chromosome<T>* temp = &(Nth(left-1));
				Replace(&(Nth(right-1)),left-1);
				Replace(temp,right-1);
				++left;
				--right;
				}//end if
		}while(right > left);
	if (start < right-1) {
		Sort(start, right-1);}
	if (left-1 < finish) {
		Sort(left-1, finish);}
	}//end  ChromosomeList<T>::Sort

#endif




