/******************************************************
*
*	Genotype.cpp
*
*	This class implements a list of loci and
*	alleles.  All loci unlinked.  It handles
*	cloning and sexual reproduction.
*
*
*
*******************************************************/

#include "Genotype.h"
#include "Element.cpp"
#include "aList.cpp"
#include "Allele.cpp"
#include "GeneContainers.cpp"


long double Genotype::additiveRelativePhenotype(void){
	long double sum=zero;
	int loc,a;
	for (loc=0;loc<loci_.len();++loc){
		for (a=0;a<loci_[loc].len();++a){
			sum += loci_[loc][a].val();
			}}
	return sum;
	}	//end Genotype::additiveRelativePhenotype
	
  	
  	
int Genotype::AllelePresent (int loc, aString& alleleVal){
	int aVal = alleleVal.toInt();
	return AllelePresent(loc,aVal);
	}	//end Genotype::AllelePresent


int Genotype::AllelePresent (int loc, int alleleVal){
	ObjList<Allele<int> >& locList = loci_[loc];
	int a;
	for (a=0;a<locList.len();++a){
		Allele<int>& allele = locList[a];
		if (alleleVal == allele.val() ) {return 1;}}
	return 0;
	}	//end Genotype::AllelePresent


void Genotype::Copy(Phenotype& oldp){
	Genotype* oldpp = (Genotype*) &oldp;
	Genotype& old = *oldpp;
	//copies the Genotype
	loci_.Clear(); 
	int j, a;
	for (j=0;j<old.loci();++j){
  		ObjList<Allele<int> >* locusList = NEW ObjList<Allele<int> >();
  		loci_.Append(locusList);
  		ObjList<Allele<int> >& oldLocusList = old.loci_[j];
  		for(a=0;a<oldLocusList.len();++a){
  			Allele<int>& oldAllele = oldLocusList[a];
  			Allele<int>* allele = NEW Allele<int>(oldAllele);
  			locusList->Append(allele);
  			}//end for a
  		}//end for j
  	}//end Genotype::Copy(Genotype&)
		

//void Genotype::Copy(Genotype* old){
void Genotype::Copy(Phenotype* oldp){
	//copies the Genotype
	Genotype* old = (Genotype*) oldp;
	loci_.Clear(); 
	int j,a;
	for (j=0;j<old->loci();++j){
  		ObjList<Allele<int> >* locusList = NEW ObjList<Allele<int> >();
  		loci_.Append(locusList);
  		ObjList<Allele<int> >& oldLocusList = old->loci_[j];
  		for(a=0;a<oldLocusList.len();++a){
  			Allele<int>& oldAllele = oldLocusList[a];
  			Allele<int>* allele = NEW Allele<int>(oldAllele);
  			locusList->Append(allele);
  			}//end for a
  		}//end for j
  	}//end Genotype::Copy(Genotype*)


  	
int Genotype::CountAlleles (int loc, aString& alleleVal){
	int aVal = alleleVal.toInt();
	return CountAlleles(loc,aVal);
	}	//end Genotype::CountAlleles


int Genotype::CountAlleles (int loc, int alleleVal){
	ObjList<Allele<int> >& locList = loci_[loc];
	int a, count=0;
	for (a=0;a<locList.len();++a){
		Allele<int>& allele = locList[a];
		if (alleleVal == allele.val() ) {count++;}}
	return(count);
	}	//end Genotype::CountAllelesAtLoc

    		

void Genotype::Fertilize(CoreGenome* momGamete, CoreGenome* dadGamete){
	//fuses two gametes
	//transfers the alleles to the new zygotic genotype
	//detaches alleles from the momGamete and dadGamete
	Genotype* mommyGamete = (Genotype*) momGamete;
	Genotype* daddyGamete = (Genotype*) dadGamete;
	loci_.Clear();
	int j,a;
	for (j=0;j<momGamete->loci();++j){
		ObjList<Allele<int> >* locusList = NEW ObjList<Allele<int> >();
		loci_.Append(locusList);
		ObjList<Allele<int> >& momLocusList = mommyGamete->loci_[j];
		ObjList<Allele<int> >& dadLocusList = daddyGamete->loci_[j];
		for (a=0;a<momLocusList.len();++a){
			locusList->Append(&(momLocusList[a]));}
		for (a=0;a<dadLocusList.len();++a){
			locusList->Append(&(dadLocusList[a]));}
		momLocusList.Detach();
		dadLocusList.Detach();
		}
	
	}//end Genotype::Fertilize
		

//void Genotype::Gamete(Genotype* parentGenes, Ran3& ran){
//void Genotype::Gamete(CoreGenome* parentGenes, Ran3List& ran, double /*mutationRate*/, double /*sigma*/){
void Genotype::Gamete(CoreGenome* parentGenes, Ran3List& ran, aList<long double>& /*specialInfo*/){
	//does meiosis to turn the genotype into a gamete of parentGenes
	//robust to haplodiploidy, genetic sex determination of XY or XO type, and polyploidy
	//ASSUMES UNLINKED LOCI
	//OVERRIDE FOR LINKAGE GENOTYPES
	Genotype* parentGenes_ = (Genotype*) parentGenes;
	loci_.Clear(); int genesToMake;
	Genotype* oldGenes=NEW Genotype(parentGenes_);//work with a new copy to sample without replacement
	//determine if all loci haploid --robust to genetic sex determination
	int j, a, ploidy,haploid=1;//true
	for (j=0;j<parentGenes_->loci();++j){
		if (parentGenes_->PloidyAtLoc(j)>1){haploid=0;break;}}//false

	for (j=0;j<parentGenes_->loci();++j){
  		ObjList<Allele<int> >* locusList = NEW ObjList<Allele<int> >();
  		loci_.Append(locusList);
  		ObjList<Allele<int> >& parentLocusList = oldGenes->loci_[j];
  		ploidy=parentGenes_->PloidyAtLoc(j);
  		if (haploid){genesToMake=1;}
  			else{
  				genesToMake=ploidy/2;
  				long double pl = (long double)ploidy;
  				if ((MOD(pl, two) != zero)&&(ran.next()<half)){
  					++genesToMake;}//odd chromosome may or may not join
  				}
  		for(a=0;a<genesToMake;++a){
  			//samples without replacement for proper treatment of polyploids
 			long num=ran.longBetween(0,parentLocusList.len()-1);
  			Allele<int>& parentAllele = parentLocusList[num];
 			locusList->Append(&(parentAllele));
 //insert mutation function here ?
 			parentLocusList.Detach(num);
  			}//end for a
  		}//end for j
	delete oldGenes;
	}//end Genotype::Gamete

	



Genotype::Genotype(int numLoci, int ploidy, int alleleVal){
  	//creates a 'null' homozygous individual at all loci
  	int j,a;
  	for(j=0;j<numLoci;++j){
  		ObjList<Allele<int> >* locusList = NEW ObjList<Allele<int> >();
  		loci_.Append(locusList);
  		for(a=0;a<ploidy;++a){
  			Allele<int>* allele = NEW Allele<int>(alleleVal);
  			locusList->Append(allele);
  			}//end for ploidy
  		}//end for loci
  	}//end constructor Genotype::Genotype(int,int,int)
  			
  			
  			
Genotype::Genotype(aList<int>& ploidyList, int alleleVal){
  	//creates a 'null' homozygous individual at all loci, with loci varying in ploidy
  	int j,a;
  	for(j=0;j<ploidyList.len();++j){
  		ObjList<Allele<int> >* locusList = NEW ObjList<Allele<int> >();
  		loci_.Append(locusList);
  		for(a=0;a<ploidyList[j];++a){
  			Allele<int>* allele = NEW Allele<int>(alleleVal);
  			locusList->Append(allele);
  			}//end for ploidy
  		}//end for loci
  	}//end constructor Genotype::Genotype(int,int,int)
  			
  			
  			
  			
Genotype::Genotype(Genotype& old){
	//copy the phenotype
	selectionFunc = old.selectionFunc;
	int t,j,a;
	for (t=0;t<old.trait_.len();++t){
		trait_.Append(old.trait_[t]);}
	//copy the Genotype 
	for (j=0;j<old.loci();++j){
  		ObjList<Allele<int> >* locusList = NEW ObjList<Allele<int> >();
  		loci_.Append(locusList);
  		ObjList<Allele<int> >& oldLocusList = old.loci_[j];
  		for(a=0;a<oldLocusList.len();++a){
  			Allele<int>& oldAllele = oldLocusList[a];
  			Allele<int>* allele = NEW Allele<int>(oldAllele);
  			locusList->Append(allele);
  			}//end for a
  		}//end for j
  	}//end constructor Genotype::Genotype(Genotype&)
		
		
		
Genotype::Genotype(Genotype* old){
	//copy the phenotype
	selectionFunc = old->selectionFunc;
	int t,j,a;
	for (t=0;t<old->trait_.len();++t){
		trait_.Append(old->trait_[t]);}
	//copy the Genotype 
	for (j=0;j<old->loci();++j){
  		ObjList<Allele<int> >* locusList = NEW ObjList<Allele<int> >();
  		loci_.Append(locusList);
  		ObjList<Allele<int> >& oldLocusList = old->loci_[j];
  		for(a=0;a<oldLocusList.len();++a){
  			Allele<int>& oldAllele = oldLocusList[a];
  			Allele<int>* allele = NEW Allele<int>(oldAllele);
  			locusList->Append(allele);
  			}//end for a
  		}//end for j
  	}//end constructor Genotype::Genotype(Genotype*)
		
		
		
int Genotype::GetGene(int locus, int alleleNum){
	//returns the value of an allele
  	ObjList<Allele<int> >& locusList = loci_[locus];
  	Allele<int>& allele = locusList[alleleNum];
  	return allele.val();
  	}//end Genotype::GetGene
  	

aString* Genotype::GenotypeString(char* separator){
	//returns all ploidy copies separated by parentheses (A)(A) at each locus
	//this permits complex alleles to be represented easily
	aString *gStr = NEW aString();
	int loc,a;
	for (loc=0;loc<loci_.len();++loc){
		for (a=0;a<loci_[loc].len();++a){
			gStr->cat('('); gStr->cat(loci_[loc][a].val()); gStr->cat(')');
			}//a
		gStr->cat(separator);
		}//loc
	return gStr;
	}//end Genotype::GenotypeString(char*)


aString* Genotype::GenotypeString(int locus, char* /*separator*/){
	//finds nth overall locus and returns all ploidy copies separated by parentheses (A)(A)
	//this permits complex alleles to be represented easily
	aString *gStr = NEW aString();
	int a;
	for (a=0;a<loci_[locus].len();++a){
		gStr->cat('('); gStr->cat(loci_[locus][a].val()); gStr->cat(')');
		}//a
	return gStr;
	}//end Genotype::GenotypeString(int, char*)


aString* Genotype::GenotypeString(int locus, int alleleNum,  char* /*separator*/){
	//finds nth overall locus and returns the alleleNum_th copy
	aString *gStr = NEW aString(loci_[locus][alleleNum].val());
	return gStr;
	}//end Genotype::GenotypeString(int, int, char*)



void Genotype::GetGenome(GeneContainerCore& gc){
	//refills container class as a MultiHaplotypeContainer<int>[ploidy][locus]
	MultiHaplotypeContainer<int>* mhc = (MultiHaplotypeContainer<int>*) &gc;
	mhc->Clear();
	mhc->ploidy_ = ploidy();
	int c,loc;
	for(c=0;c<ploidy();++c){
		mhc->geneSet_.Append(NEW HaplotypeContainer<int>(mhc->ID(),loci()));
		for(loc = 0;loc<loci();++loc){
			(*mhc)[c][loc] = GetGene(loc,c);}
		}//c
	}//Genotype::GetGenome(GeneContainerCore& )



int Genotype::IsHeterozygote (int locus){
	//returns FALSE=0 if no genotype available, or if haploid
	int answer=0,a,alleleVal;
	if (loci() > 0) {
		ObjList<Allele<int> >& locList = loci_[locus];
		for (a=0;a<locList.len();++a){
			Allele<int>& allele = locList[a];
			if (a == 0) {alleleVal = allele.val();}
				else {if (alleleVal != allele.val() ) {answer = 1;}}
			}}//end for a && end if it has genes
	return answer;
	}//end Genotype::IsHeterozygote

void Genotype::MakeNull(void){
	Phenotype::MakeNull();
	int loc,a;
	for (loc=0;loc<loci_.len();++loc){
		for (a=0;a<loci_[loc].len();++a){
			Allele<int>& allele = loci_[loc][a];
			allele.mutateTo(0);
			}//a
		}//loc
	}//end Genotype::MakeNull()



int Genotype::PloidyAtLoc (int locus){
	//sex-linked loci have variable ploidy
  	ObjList<Allele<int> >& locusList = loci_[locus];
 	return locusList.len();
	}	//end Genotype::PloidyAtLoc
  	
 
void Genotype::Print(ostream& outFile, int alleleLen){
	aString geneStr('.');//dummy starter value will be cleared anyway
	SetGenotypeString(geneStr,alleleLen);
	outFile<<geneStr<<endl;
	}//end Genotype::Print
	
    		
int	Genotype::RandomAllele(int locus, Ran3& ran){
  	//randomly picks an allele and returns its value
  	ObjList<Allele<int> >& locusList = loci_[locus];
  	long num=ran.longBetween(0,locusList.len()-1);
  	Allele<int>& allele = locusList[num];
	return allele.val();
	}//end Genotype::RandomAllele
  	


void Genotype::set(int locus, int alleleNum, int newAlleleVal){
	//changes the value of an allele
  	ObjList<Allele<int> >& locusList = loci_[locus];
  	Allele<int>& allele = locusList[alleleNum];
  	allele.mutateTo(newAlleleVal);
  	}//end Genotype::set
	
	
	
void Genotype::SetGenome(GeneContainerCore& gc){
	//initializes genome using MultiHaplotypeContainer<int>[ploidy][locus]
	loci_.Clear();
	MultiHaplotypeContainer<int>* mhc = (MultiHaplotypeContainer<int>*) &gc;
	int loc,c,nloci = mhc->loci();
	int ploidylevel = mhc->ploidy();
	for(loc = 0;loc<nloci;++loc){
		loci_.Append(NEW ObjList<Allele<int> >());
		for(c=0;c<ploidylevel;++c){
			loci_[loc].Append(NEW Allele<int>(mhc->haplotype(loc,c)));}
		}
	}//Genotype::SetGenome(GeneContainerCore& )


void Genotype::SetGenotypeString(aString& geneStr, int alleleLen){
	geneStr.clear();
	int j,a,b;
	for(j=0;j<loci();++j){
	  	ObjList<Allele<int> >& locusList = loci_[j];
	  	b=locusList.len();
	  	for(a=0;a<b;++a){
	  		aString allStr(GetGene(j,a));
	  		allStr.PrependCharToLen('0',alleleLen);
	  		geneStr.cat(allStr);
	  		if(a!=b-1){geneStr.cat('.');}
	  		}
	  	if (j!=loci()-1){geneStr.cat(' ');}
	  	}//end for j
	  }//end Genotype::SetGenotypeString
	


void Genotype::SetSexualZygote(CoreGenome* momGenes, CoreGenome* dadGenes, Ran3List& ran, 
					//					double mutationRate, double sigma){
										aList<long double>& specialInfo){
	//uses mom's and dad's genes to seed the genotype
	Genotype* mommyGenes = (Genotype*) momGenes;
	Genotype* daddyGenes = (Genotype*)  dadGenes;
	Genotype momGamete, dadGamete;
	momGamete.Gamete(mommyGenes,ran,specialInfo);//genes copied to gamete
	dadGamete.Gamete(daddyGenes,ran,specialInfo);
	Fertilize(&momGamete,&dadGamete);//momGamete & dadGamete are emptied in this step
	long double sigma,newValueDbl,mutationRate = specialInfo[0];
	if(ran()<mutationRate){
		int mutantLoc = ran.longBetween(0, loci()-1 );
		int mutantAllele = ran.longBetween(0,PloidyAtLoc(mutantLoc)-1);
		sigma = specialInfo[1];
		newValueDbl = ran.NormDev(zero,sigma);
		int newVal = newValueDbl;			
		Mutate(mutantLoc,mutantAllele,newVal);}
	}//end Genotype::SetSexualZygote



	

int operator== (Genotype& g1, Genotype& g2){
	if(g1.loci()!=g2.loci()){return 0;}
	int j,a;
	for (j=0;j<g1.loci_.len();++j){
		ObjList<Allele<int> >& locList1=g1.loci_[j];
		ObjList<Allele<int> >& locList2=g2.loci_[j];
		if(locList1.len()!=locList2.len()){return 0;}
		for(a=0;a<locList1.len();++a){
			Allele<int>& a1=locList1[a];
			Allele<int>& a2=locList2[a];
			if(a1!=a2){return 0;}}}
	return 1;
	}//end friend operator==
		
int operator!= (Genotype& g1, Genotype& g2){
  	return !(g1==g2);}
  	
  	






