/*****************************************************
*
*
*	RegulatedAllele.cpp
*
*	Holds the cis-regulatory sites of an allele
*
******************************************************/
#ifndef RegulatedAllelecp
#define RegulatedAllelecp = 1


#include "RegulatedAllele.h"
#include "Element.cpp"
#include "aList.cpp"
#include "RandomList.cpp"
#include "Allele.cpp"



template<class T > Allele<T>* RegulatedAllele<T>::Duplicate(void){
	return new RegulatedAllele<T>(*this);
	}//end RegulatedAllele<T>::Duplicate()


template<class T > aString* RegulatedAllele<T>::GenotypeString(char* separator, locusComponents site) {
  	locusComponents s=site;
  	aString* g;
  	if(site==productSite){
  			g = new aString(data_);}
  		else if (site == promoterSite){
  			g = new aString(promoter_);}
  		else if (site ==inhibitorSite){
  			g = new aString(inhibitor_;}
  		else{//(site == unspecifiedSite)
  			cout<<"ERROR: Don't call RegulatedAllele<T>::GenotypeString(char*,locusComponents)"<<endl;
  			cout<<"  without specifying a site."<<endl;
  			}
  	return g;
  	}//end RegulatedAllele<T>::mutateTo(T, locusComponents)




template<class T > void RegulatedAllele<T>::Mutate(Ran3& ran, double sigma){
	Mutate(ran, sigma, unspecifiedSite);
	}//end RegulatedAllele<T>::Mutate(Ran3&, double)



template<class T > void RegulatedAllele<T>::Mutate(Ran3& ran, double sigma, locusComponents site){
  	locusComponents s = site;
  	if (site == unspecifiedSite){
  		s = (locusComponents) ran.longBetween(1,3);}
  	if(s==productSite){
  			data_= (T) ran.NormDev((double)data_, sigma);}
  		else if (s == promoterSite){
  			promoter_ =  (T) ran.NormDev((double)promoter_, sigma);;}
  		else{//s ==inhibitorSite
  			inhibitor_ =  (T) ran.NormDev((double)inhibitor_, sigma);;}
  	}//end RegulatedAllele<T>::Mutate(Ran3&,double,locusComponents)


template<class T > void RegulatedAllele<T>::mutateTo(T x, locusComponents site){
  	if(site == productSite){
  			data_ = x;}
  		else if (site == promoterSite){
  			promoter_ =  x;}
  		else if( site ==inhibitorSite){
  			inhibitor_ = x;}
  		else{//(site == unspecifiedSite)
  			cout<<"ERROR: Don't call RegulatedAllele<T>::MutateTo(T,locusComponents)"<<endl;
  			cout<<"  without specifying a mutation site."<<endl;
  			}
  	}//end RegulatedAllele<T>::mutateTo(Ran3&,double,locusComponents)


template<class T > aString* RegulatedAllele<T>::GenotypeString(char* separator, locusComponents omitSite){
	aString* genotype = new aString();
	if (omitSite != promoterSite){
			genotype->cat(promoter_);
			genotype->cat(separator);}
	if (omitSite != inhibitorSite){
			genotype->cat(inhibitor_);
			genotype->cat(separator);}
	if (omitSite != productSite){
			genotype->cat(data_);
			genotype->cat(separator);}
	return genotype;
	}//end RegulatedAllele<T>::GenotypeString(char&,locusComponents)
		



#endif
