/*****************************************************
*
*
*	RegulatedAllele2.cpp
*
*	Holds the cis-regulatory sites of an allele
*
******************************************************/
#ifndef RegulatedAllele2cp
#define RegulatedAllele2cp = 1


#include "RegulatedAllele2.h"
#include "Element.cpp"
#include "aList.cpp"
#include "RandomList.cpp"
//#include <iostream.h>



RegulatedAllele2* RegulatedAllele2::Duplicate(void){
	return NEW RegulatedAllele2(*this);
	}//end RegulatedAllele2::Duplicate()


aString* RegulatedAllele2::GenotypeString(char* sepChar) {
  	aString* g;
  	g = NEW aString(promoter(0));
  	g->cat(sepChar);
  	g->cat(promoter(1));
  	g->cat(sepChar);
  	g->cat(product(0));
  	g->cat(sepChar);
  	g->cat(product(1));
  	return g;
  	}//end RegulatedAllele2::GenotypeString(char)


aString* RegulatedAllele2::GenotypeString(int allelePart, char* /*sepChar*/) {
	if(OutOfRange(allelePart)){return NEW aString();}
  	aString* g;
  	if(allelePart<2){//hits promoter
  			g = new aString(promoter(allelePart));}
  		else{//hits product
  			g = new aString(product(allelePart-2));}
  	return g;
  	}//end RegulatedAllele2::GenotypeString(int)




void RegulatedAllele2::Mutate(Ran3& ran, double sigma){
	//pick the part that mutates
  	int s =  ran.longBetween(0,3);  	
  	if(s<2){//hits promoter
  			double old = promoter_[s];
  			promoter_[s] = ran.NormDev(old, sigma);}
  		else{//hits product
  			double old = product_[s-2];
  			product_[s-2] = ran.NormDev(old, sigma);}
  	}//end RegulatedAllele2::Mutate(Ran3&,double)



void RegulatedAllele2::MutateTo(float newValue, int allelePart){
	if(OutOfRange(allelePart)){return;}
  	if(allelePart<2){//hits promoter
  			promoter_[allelePart] =newValue;}
  		else{//hits product
  			product_[allelePart-2] = newValue;}
  	}//end RegulatedAllele2::MutateTo(float, int)


void RegulatedAllele2::set(float newValue, int allelePart){
	MutateTo(newValue, allelePart);
  	}//end RegulatedAllele2::set(float, int)


int RegulatedAllele2::InRange(int allelePart){
	if( ( allelePart<0) || (allelePart>3) ){
		cout<<"allelePart "<<allelePart <<" out of range"<<endl;
		return 0;}
	return 1;
	}//end RegulatedAllele2::InRange(int)


int RegulatedAllele2::OutOfRange(int allelePart){
	return ! InRange(allelePart);
	}//end RegulatedAllele2::OutOfRange(int)


int operator== (RegulatedAllele2& a, RegulatedAllele2& b){
	return ( (a.promoter(0)==b.promoter(0) ) && 
			 (a.promoter(1)==b.promoter(1) ) && 
			 (a.product(0)==b.product(0) ) && 
			 (a.product(1)==b.product(1) ) );
			 }

int operator!= (RegulatedAllele2& a, RegulatedAllele2& b){
	return !(a==b);}




#endif
