/******************************************************
*
*	RegulatedGenome.cp
*
*	This class implements a Genome of  RegulatedAlleles.  
*	It handles cloning and sexual reproduction.
*
*	Base class:  Genome
*
*******************************************************/
#ifndef RegulatedGenomecp
#define RegulatedGenomecp = 1

#include "RegulatedGenome.h"
#include "Genome.cpp"
#include "Chromosome.cpp"
#include "RandomList.cpp"
#include "Element.cpp"
#include "aList.cpp"




template <class T> RegulatedGenome<T>::RegulatedGenome(int numLociPerChrom, int num1NChroms, int ploidy,
													 T alleleVal, T promoValue, T inhibValue): Genome(){
  	//creates a 'null' homozygous individual at all loci
  	devPath_.Append(NEW aList<int>();
   	float mapDist = 0.0;
  	if(numLociPerChrom>1){mapDist = 1.0/((float) num1NChroms-1.0);}
  	for(int chrom=0;chrom<num1NChroms;++chrom){
  		ChromosomeList<T>* chromList = NEW ChromosomeList<T>();
  		genome_.Append(chromList);
  		for (int k=0;k<ploidy;++k){
  			RegulatedAllele<T> a(alleleVal,promoValue,inhibValue);
  			Chromosome<T>* ch = NEW Chromosome<T>(numLociPerChrom,a, mapDist);
  			chromList->Append(ch);
  			}//end for k
  		}//end for chrom
  	}//end constructor RegulatedGenome<T>::RegulatedGenome(int,int,int,T,T,T)



  			
template <class T> RegulatedGenome<T>::RegulatedGenome(int numLociPerChrom, int num1NChroms, int ploidy,
						T alleleVal, T promoValue, T inhibValue, ViabilitySelectionFunc viabSelection):
		Genome(viabSelection){
  	//creates a 'null' homozygous individual at all loci
  	devPath_.Append(NEW aList<int>());
  	float mapDist = 0.0;
  	if(numLociPerChrom>1){mapDist = 1.0/((float) num1NChroms-1.0);}
  	for(int chrom=0;chrom<num1NChroms;++chrom){
  		ChromosomeList<T>* chromList = NEW ChromosomeList<T>();
  		genome_.Append(chromList);
  		for (int k=0;k<ploidy;++k){
  			RegulatedAllele<T> a(alleleVal,promoValue,inhibValue);
  			Chromosome<T>* ch = NEW Chromosome<T>(numLociPerChrom,a, mapDist);
  			chromList->Append(ch);
  			}//end for k
  		}//end for chrom
  	}//end constructor RegulatedGenome::RegulatedGenome(int,int,int,T,T,T,ViabilitySelectionFunc)



  			
template <class T> RegulatedGenome<T>::RegulatedGenome(RegulatedGenome<T>& old):
		  	devPath_(old.devPath_){
	//copy the phenotype
	selectionFunc = old.selectionFunc;
	for (int t=0;t<old.trait_.len();++t){
		trait_.Append(old.trait_[t]);}
	//copy the Genome 
	for (int chrom=0;chrom<old.chroms1N();++chrom){
  		ChromosomeList<T>* chromList = NEW ChromosomeList<T>();
  		genome_.Append(chromList);
  		ChromosomeList<T>& oldChromList = old.genome_[chrom];
  		for(int ch=0;ch<oldChromList.len();++ch){
  			Chromosome<T>& oldChrom = oldChromList[ch];
  			Chromosome<T>* c = NEW Chromosome<T>(oldChrom);
  			chromList->Append(c);
  			}//end for ch
  		}//end for chrom
  	}//end constructor RegulatedGenome<T>::RegulatedGenome(RegulatedGenome<T>&)
		
		
		
template <class T> RegulatedGenome<T>::RegulatedGenome(RegulatedGenome<T>* old):
		  	devPath_(old->devPath_){
	//copy the phenotype
	selectionFunc = old->selectionFunc;
	for (int t=0;t<old->trait_.len();++t){
		trait_.Append(old->trait_[t]);}
	//copy the Genome 
	for (int chrom=0;chrom<old->chroms1N();++chrom){
  		ChromosomeList<T>* chromList = NEW ChromosomeList<T>();
  		genome_.Append(chromList);
  		ChromosomeList<T>& oldChromList = old->genome_[chrom];
  		for(int ch=0;ch<oldChromList.len();++ch){
  			Chromosome<T>& oldChrom = oldChromList[ch];
  			Chromosome<T>* c = NEW Chromosome<T>(oldChrom);
  			chromList->Append(c);
  			}//end for ch
  		}//end for chrom
  	}//end constructor RegulatedGenome<T>::RegulatedGenome(RegulatedGenome<T>*)
		
		
		




    		
template <class T> void RegulatedGenome<T>::set(int chrom, int locus, int alleleNum, 
										T newAlleleVal, locusComponents site){
	//changes the value of an allele
  	Chromosome<T>& c = genome_[chrom][alleleNum];
  	Allele<T>& a = c.GetAllele(locus);
  	Allele<T>* aa = &a;
  	RegulatedAllele<T>* r = (RegulatedAllele<T>*) aa;
  	r->mutateTo(newAlleleVal,site);
  	}//end RegulatedGenome::set
	


template <class T> void RegulatedGenome<T>::SetDevPath(ObjList<aList<int> >& path){
		devPath_.Copy(path);
		}//end RegulatedGenome::SetDevPath


template <class T> void RegulatedGenome<T>::SetGenotypeString(aString& geneStr, int){
	geneStr.clear();
	char* sepChar = " ";
	for(int i=0;i<trait_.len();++i){
		geneStr.cat(trait(i));//the major phenotypes
		geneStr.cat(sepChar);}
	for(int chrom=0;chrom<chroms1N();++chrom){
		ObjList<Chromosome<T> >& chromList = genome_[chrom];//homologous copies in list
	  	for(int ch=0;ch<chromList.len();++ch){//go down chromosome copies
  			Chromosome<T>& c = chromList[ch];
  			for(int loc=0;loc<c.loci();++loc){
  				Allele<T> aa = c.GetAllele(loc);
  				RegulatedAllele<T>* a = (RegulatedAllele<T>*) &aa;
	  			aString* allStr= a->GenotypeString(sepChar);
//	  			allStr.PrependCharToLen('0',alleleLen);
	  			geneStr.cat(*allStr);
	  			delete allStr; allStr = NULL;
	  			a = NULL;//forget
	  			if(loc != c.loci()-1){
	  				geneStr.cat(sepChar);}
	  			}//end for loc
	  		if (ch == chromList.len()-1){
	  				geneStr.cat(sepChar);}
	  			else{
	  				geneStr.cat(" || ");}
	  		}//end for ch
	  	}//end for chrom
		}//end RegulatedGenome<T>::SetGenotypeString(aString&, int)


	
 template <class T> void RegulatedGenome<T>::SetPhenotype(void){
	Genome<T>::SetPhenotype();return;//uncomment for additive model
	double sum=0.0;//sum the different pathways for the phenotype
	for(int i=0;i<devPath_.len();++i){
		//average scores over allele values
		double product=1.0;
		for(int loc=0;loc<devPath_[i].len()-1;++loc){
			int locus1 = devPath_[i][loc];
			int sign1 = sign(locus1);
			locus1 = ABS(locus1);
			int locus2 = devPath_[i][loc+1];
			int sign2 = sign(locus2);//negative values bind inhibitors
			locus2 = ABS(locus2);
			//take average of values across allele copies
			double prodA=0.0,promB=0.0,inhibB=0.0;
			int ploc1 = ploidy(locus1);
			int ploc2 = ploidy(locus2);
			for(int ch=0;ch<ploc1;++ch){
				Allele<T>& aa = GetAllele(locus1, ch);
				RegulatedAllele<T>* a = (RegulatedAllele<T>*) &aa;
				prodA += (double)(a->val());
				}
			prodA /= (double)(ploc1);//gets average
			for(int ch=0;ch<ploc2;++ch){
				Allele<T>& bb = GetAllele(locus2, ch);
				RegulatedAllele<T>* b = (RegulatedAllele<T>*) &bb;
				promB += (double)(b->promoter());
				inhibB += (double)(b->inhibitor());
				}
			promB /= (double)(ploc2);//gets average
			inhibB /= (double)(ploc2);//gets average
			double coeff=0.0, coeff2=0.0;
			if(sign2==1){
					coeff = prodA - promB;//bind to promoter
					coeff2 = -coeff*coeff;
					product *= exp(coeff2);
					}
				else{//sign2== -1
					coeff = prodA - inhibB;//bind to inhibitor
					coeff2 = -coeff*coeff;
					product *= 1.0 - exp(coeff2);
					}
			}//end for loc

		//comment this to study only regulatory sites
		int loc = devPath_[i].Last();
		double prodave=0.0;
		int ploidlastloc=ploidy(loc);
		for(int ch=0;ch<ploidlastloc;++ch){
			Allele<T>& cc = GetAllele(loc, ch);			
			RegulatedAllele<T>* c = (RegulatedAllele<T>*) &cc;
			prodave += (double)(c->val());}
		prodave /= (double)ploidlastloc;
		product *= prodave;

		sum += product;
		}
	SetTrait(sum,0);
	}/*end RegulatedGenome<T>::SetPhenotype(void)*/



template <class T> void RegulatedGenome<T>::SetPhenotype(Ran3& ran, double heritability){
	/*default is additive phenotype*/
	SetPhenotype();
	if (heritability <1.0){
		double sum=trait();
		sum += ran.NormDev(0.0,1.0-heritability);
		SetTrait(sum,0);
		}
	}/*end RegulatedGenome<T>::SetPhenotype(ran,double)*/




	


#endif




