/******************************************************
*
*	RegulatedGenome1.cp
*
*	This class implements a Phenotype of RegulatedAlleles.  
*	It handles cloning and sexual reproduction.
*
*	Base class:  Phenotype
*
*******************************************************/
#ifndef RegulatedGenome1cp
#define RegulatedGenome1cp = 1

#include "RegulatedGenome1.h"
#include "RandomList.cpp"
#include "Element.cpp"
#include "aList.cpp"






RegulatedGenome1::RegulatedGenome1( int numLoci, int ploidy,
									float promoValue, float productValue){
  	//creates a 'null' homozygous individual identical at all loci
  	devPath_.Append(new aList<int>());
  	for(int i=0;i<numLoci;++i){
  		genes_.Append(new ObjList<RegulatedAllele1>());
  		for(int a=0;a<ploidy;++a){
  			genes_[i].Append(new RegulatedAllele1(promoValue,productValue));
  			}}
   	}//end constructor RegulatedGenome1::RegulatedGenome1(int,int,float[2],float[2])



  			
RegulatedGenome1::RegulatedGenome1(int numLoci, int ploidy,
									float promoValue, float productValue, 
									ViabilitySelectionFunc viabSelection):
		CoreGenome(viabSelection){
  	//creates a 'null' homozygous individual at all loci
  	devPath_.Append(new aList<int>());
  	for(int i=0;i<numLoci;++i){
  		genes_.Append(new ObjList<RegulatedAllele1>());
  		for(int a=0;a<ploidy;++a){
  			genes_[i].Append(new RegulatedAllele1(promoValue,productValue));
  			}}
 	}//end constructor RegulatedGenome1::RegulatedGenome1(int,int,float[2],float[2],ViabilitySelectionFunc)



double  RegulatedGenome1::additiveRelativePhenotype(void){//additive phenotype
	//uses product only
	double sum=0.0;
	for(int loc=0;loc<genes_.len();++loc){
		for(int a=0;a<ploidy(loc);++a){
			sum += genes_[loc][a].product();
			}}
	return sum;
	}//end RegulatedGenome1::additiveRelativePhenotype




    		
int	RegulatedGenome1::AllelePresent (int loc, aString& alleleVal){
	for(int a=0;a<ploidy(loc);++a){
		aString* val = genes_[loc][a].GenotypeString(" ");
		if(*val == alleleVal){
			delete val;val = NULL;
			return 1;}
		delete val;val = NULL;
			}
	return 0;
	}//end RegulatedGenome1::AllelePresent


int	RegulatedGenome1::CountAlleles (int loc, aString& alleleVal){
				//counts the absolute frequency of alleleVal
	int num = 0;
	for(int a=0;a<ploidy(loc);++a){
		aString* val = genes_[loc][a].GenotypeString(" ");
		if(*val == alleleVal){
			++num;}
		delete val;val = NULL;
			}
	return num;
	}//end RegulatedGenome1::CountAlleles



void RegulatedGenome1::Fertilize (CoreGenome* momGamete, CoreGenome* dadGamete){
	//transfers parent genes to new zygote (=*this) during reproduction
	genes_.Clear();
	RegulatedGenome1* mommyGamete = (RegulatedGenome1*) momGamete;
	RegulatedGenome1* daddyGamete = (RegulatedGenome1*) dadGamete;
	for (int loc=0;loc<mommyGamete->genes_.len();++loc){
		genes_.Append(new ObjList<RegulatedAllele1>() );
		for (int a=0;a<mommyGamete->ploidy(loc);++a){
//			genes_[loc].Append(new RegulatedAllele1(mommyGamete->genes_[loc][a]) );
			genes_[loc].Append(mommyGamete->genes_[loc][a].Duplicate() );
			}
		for (int a=0;a<daddyGamete->ploidy(loc);++a){
//			genes_[loc].Append(new RegulatedAllele1(daddyGamete->genes_[loc][a]) );
			genes_[loc].Append(daddyGamete->genes_[loc][a].Duplicate() );
			}
		}//end for loc
	
	}//RegulatedGenome1::Fertilize



void RegulatedGenome1::Gamete (CoreGenome* theParentGenes, Ran3List& ran, double mutationRate, double sigma){
	//does meiosis to turn the genotype into a gamete of parentGenes
	//robust to haplodiploidy, genetic sex determination of XY or XO type, and polyploidy
	genes_.Clear();
	RegulatedGenome1* parentGenes = (RegulatedGenome1*) theParentGenes;
	RegulatedGenome1* oldGenes = (RegulatedGenome1*) parentGenes->Duplicate();
	//work with a new copy to sample without replacement
	//and to avoid permanent recombination
	//determine if all loci haploid --robust to genetic sex determination

	for (int loc=0;loc<parentGenes->genes_.len();++loc){
  		genes_.Append(new ObjList<RegulatedAllele1>() );
  		int ploidy = parentGenes->ploidy(loc);
  		int chromsToMake = ploidy;
  		if (ploidy>1){
  			chromsToMake = ploidy/2;
  			double pl = (double)ploidy, two = 2.0;
  			if ((MOD(pl, 2.0) != 0.0) && (ran() < 0.5)){
  				++chromsToMake;}//odd chromosome may or may not join
  				}
  				
  		for(int ch=0;ch<chromsToMake;++ch){
  			//samples oldGenes without replacement for proper treatment of polyploids
 			long a=ran.longBetween(0,parentGenes->ploidy()-1);
// 			RegulatedAllele1& theAllele = oldGenes->genes_[loc][a];
// 			genes_[loc].Append(&theAllele);
// 			oldGenes->genes_[loc].Detach(a);
 			RegulatedAllele1* theAllele = oldGenes->genes_[loc][a].Duplicate();
 			genes_[loc].Append(theAllele);
 			if ((mutationRate>0.0) && (ran()<mutationRate)){
 				genes_[loc].Last().Mutate(ran, sigma);}
  			}//end for ch
  		}//end for loc
	delete oldGenes;oldGenes=0;
	}//end RegulatedGenome1::Gamete





aString*  RegulatedGenome1::GenotypeString(char* separator){
					//returns all ploidy copies separated by parentheses (A)(A) at each locus
					//this permits complex alleles to be represented easily
	//returns all ploidy copies separated by parentheses (A)(A) at each locus
	//this permits complex alleles to be represented easily
	aString* gtstr = new aString();
	for (int locus=0;locus<genes_.len();++locus){
		aString* gstr = GenotypeString(locus,separator);
		gtstr->cat(*gstr);
		gtstr->cat(separator);
		delete gstr;gstr=NULL;
		}//end for locus
	return gtstr;
  	}//end RegulatedGenome1::GenotypeString(char*)



aString* RegulatedGenome1::GenotypeString(int locus, char* separator){
					//finds nth overall locus and returns all ploidy copies separated by parentheses (A)(A)
					//this permits complex alleles to be represented easily
	aString* gtstr = new aString();
	char spar='(', epar=')';
	for(int a=0;a<ploidy(locus);++a){
		aString* gstr = GenotypeString(locus,a,separator);
		gstr->prepend(spar);gstr->cat(epar);
		gtstr->cat(*gstr);
		delete gstr;gstr=NULL;
		}//end for a
	return gtstr;
  	}//end RegulatedGenome1::GenotypeString
  	


aString* RegulatedGenome1::GenotypeString(int locus, int alleleNum,  char* separator){
	//finds nth overall locus and returns the alleleNum_th copy
	return genes_[locus][alleleNum].GenotypeString(separator);
  	}//end RegulatedGenome1::GenotypeString
  	



int	RegulatedGenome1::IsHeterozygote (int locus){
				//returns false=0 if haploid
	if(ploidy(locus)<2){return 0;}
	RegulatedAllele1& alleleVal = genes_[locus][0];
	for(int a=1;a<ploidy(locus);++a){
		if(alleleVal != genes_[locus][a]){
			return 1;}
		}
	return 0;
	}//end RegulatedGenome1::IsHeterozygote


int RegulatedGenome1::loci(void){
	return genes_.len();
	}//end RegulatedGenome1::loci




void RegulatedGenome1::Mutate(Ran3& ran, double mutationRate, double sigma){
	//sigma is s.d. of mutation effect size
	for(int loc=0;loc<genes_.len();loc++){
		for(int a=0;a<ploidy(loc);++a){
			if(ran()<mutationRate){
				genes_[loc][a].Mutate(ran,sigma);}
			}}
	}//end RegulatedGenome1::Mutate(Ran3&, double, double){



void RegulatedGenome1::Print(ostream& outFile, int alleleLen){
	aString geneStr;//dummy starter value will be cleared anyway
	SetGenotypeString(geneStr,alleleLen);
	outFile<<geneStr<<endl;
	}//end RegulatedGenome1::Print




void RegulatedGenome1::set(int locus, int alleleNum, int allelePart, float newAlleleVal){
	//changes the value of an allele
  	genes_[locus][alleleNum].MutateTo(newAlleleVal,allelePart);
  	}//end RegulatedGenome1::set
	


void RegulatedGenome1::SetDevPath(ObjList<aList<int> >& path){
	devPath_.Copy(path);
	}//end RegulatedGenome1::SetDevPath


void RegulatedGenome1::SetGenotypeString(aString& geneStr, int){
	geneStr.clear();
	char* sepChar = " ";
	for(int i=0;i<trait_.len();++i){
		geneStr.cat(trait(i));//the major phenotypes
		geneStr.cat(sepChar);}
	for(int loc=0;loc<genes_.len();++loc){
	  	for(int a=0;a<ploidy();++a){
  				RegulatedAllele1& r = genes_[loc][a];
	  			aString* allStr= r.GenotypeString(sepChar);
	  			geneStr.cat(*allStr);
	  			delete allStr; allStr = NULL;
	  			if(loc != loci()-1){
	  					geneStr.cat(sepChar);}
	  				else{
	  					geneStr.cat(" || ");}
	  		}//end for a
	  	}//end for loc
		}//end RegulatedGenome1::SetGenotypeString(aString&, int)


	
void RegulatedGenome1::SetPhenotype(void){
	//a Euclidean distance is used to bind product and promoter in parameter space
	for(int i=0;i<devPath_.len();++i){
		double product=1.0;
		//average scores over allele values
		for(int loc=0;loc<devPath_[i].len()-1;++loc){
			int locus1 = devPath_[i][loc];
			int locus2 = devPath_[i][loc+1];
			//take average of values across allele copies
			double prodA=0.0, promB=0.0;
			for(int a1=0;a1<ploidy(locus1);++a1){
//				RegulatedAllele1& alleleA = genes_[locus1][a1];
				RegulatedAllele1& alleleA = GetGene(locus1, a1);
				prodA += alleleA.product();
				}
			prodA /= (double)(ploidy(locus1));//gets average
			for(int a2=0;a2<ploidy(locus2);++a2){
//				RegulatedAllele1& alleleB = genes_[locus2][a2];
				RegulatedAllele1& alleleB = GetGene(locus2, a2);
				promB += alleleB.promoter();
				}
			promB /= (double)(ploidy(locus2));//gets average
			
			//bind to promoter
			 	//Gaussian binding function
			double coeff = prodA - promB;//bind to promoter
			double coeff2 = -coeff*coeff;
			product *= exp(coeff2);
			}//end for loc
		SetTrait(product,i);
		}//next step in pathway
	}//end RegulatedGenome1::SetPhenotype(void)



void RegulatedGenome1::SetPhenotype(Ran3& ran, double heritability){
	/*default is additive phenotype*/
	SetPhenotype();
	if (heritability <1.0){
		double sum=trait();
		sum += ran.NormDev(0.0,1.0-heritability);//assumes Vp=1.0  
		SetTrait(sum,0);
		}
	}//end RegulatedGenome1::SetPhenotype(ran,double)


void RegulatedGenome1::SetSexualZygote(CoreGenome* momGenes, CoreGenome* dadGenes, Ran3List& ran, double mutationRate, double sigma){
	//uses mom's and dad's genes to seed the genotype
	RegulatedGenome1* mommyGenes = (RegulatedGenome1*) momGenes;
	RegulatedGenome1* daddyGenes = (RegulatedGenome1*) dadGenes;
	RegulatedGenome1* momGamete = (RegulatedGenome1*) mommyGenes->Duplicate();
	RegulatedGenome1* dadGamete = (RegulatedGenome1*) daddyGenes->Duplicate();
	momGamete->Gamete(momGenes,ran,mutationRate,sigma);//genes copied to gamete
	dadGamete->Gamete(dadGenes,ran,mutationRate,sigma);
	Fertilize(momGamete, dadGamete);//momGamete & dadGamete are emptied in this step
	delete momGamete; momGamete=NULL;
	delete dadGamete; dadGamete=NULL;
	}//end RegulatedGenome1::SetSexualZygote




RegulatedGenome1& RegulatedGenome1::operator=(RegulatedGenome1& c){
	Phenotype::Copy(c);
	genes_.Copy(c.genes_);
	devPath_.Copy(c.devPath_);
	return *this;
	}//end RegulatedGenome1::operator=(RegulatedGenome& c)



int operator==(RegulatedGenome1& a, RegulatedGenome1& b){
	return ((a.traits() == b.traits()) &&
			(a.genes_ == b.genes_) &&
			(a.devPath_ == b.devPath_) );}



int operator!=(RegulatedGenome1& a, RegulatedGenome1& b){
			return ! (a==b);}
	


	


#endif




