/******************************************************
*
*	RegulatedSexChrom1.cpp
*
*	This Chromosome is either X or Y
* 	It holds a list of RegulatedAllele1 objects
*
*******************************************************/

#ifndef RegulatedSexChrom1cpp
#define RegulatedSexChrom1cpp = 1

#include "RegulatedSexChrom1.h"
#include "Element.cpp"
#include "aList.cpp"

  	
void RegulatedSexChromosome1::Copy(RegulatedSexChromosome1& old){
	//copies the RegulatedSexChromosome2
	genes_.Clear();  mapDist_.Clear(); 
	for (int j=0;j<old.loci();++j){
  		RegulatedAllele1& oldAllele = old.genes_[j];
  		RegulatedAllele1* allele = NEW RegulatedAllele1(oldAllele);
  		genes_.Append(allele);
  		}//end for j
	for (int j=0;j<old.mapDist_.len();++j){
  		mapDist_.Append(old.mapDist_[j]);
  		}//end for j
  	}//end RegulatedSexChromosome1::Copy(RegulatedSexChromosome1&)
		

void RegulatedSexChromosome1::Copy(RegulatedSexChromosome1* old){
	Copy(*old);
  	}//end RegulatedSexChromosome1>::Copy(RegulatedSexChromosome1*)
		
  	


RegulatedSexChromosome1::RegulatedSexChromosome1(
			int numLoci, float alleleVal, float promoVal, float mapDist, int XorY):
		XorY_(XorY){
  	//creates a 'null' homozygous individual at all loci
  	for(int j=0;j<numLoci;++j){
  		genes_.Append(NEW RegulatedAllele1(promoVal, alleleVal));
  		}//end for loci
  	for(int j=0;j<numLoci-1;++j){
  		mapDist_.Append(mapDist);
  		}//end for loci-1
  	}//end ctor RegulatedSexChromosome1::RegulatedSexChromosome1(int,float,float,float,int)


RegulatedSexChromosome1::RegulatedSexChromosome1(
			aList<float>& alleleVal, aList<float>& promoVal, 
			aList<float>& mapDist, int XorY):
		XorY_(XorY), mapDist_(mapDist){
  	//creates a 'null' homozygous individual at all loci
  	for(int j=0;j<alleleVal.len();++j){
  		genes_.Append(NEW RegulatedAllele1(promoVal[j], alleleVal[j]));
  		}//end for loci
  	}//end ctor RegulatedSexChromosome1::RegulatedSexChromosome1(aList<float>&,aList<float>&,aList<float>&,int)



RegulatedSexChromosome1::RegulatedSexChromosome1(
			int numLoci, RegulatedAllele1& allele, float mapDist, int XorY):
		XorY_(XorY){
  	//creates a 'null' chromosome with same allele at all loci,
  	//and same map distance between loci
  	//mapDist*(numLoci-1) must be 100
  	for(int a=0;a<numLoci;++a){
  		genes_.Append(NEW RegulatedAllele1(allele));
  		}//end for a
  	for(int j=0;j<numLoci-1;++j){
  		mapDist_.Append(mapDist);
  		}//end for loci-1
  	}//end ctor RegulatedSexChromosome1::RegulatedSexChromosome1(int,RegulatedAllele2&,float,int)
  			
  			
RegulatedSexChromosome1::RegulatedSexChromosome1(RegulatedSexChromosome1& old):
  			genes_(old.genes_), 
  			mapDist_(old.mapDist_),XorY_(old.XorY_){
	  //copies the RegulatedSexChromosome1 during construction
	}//end copy ctor
		
RegulatedSexChromosome1::RegulatedSexChromosome1(RegulatedSexChromosome1* old):
  			genes_(old->genes_), 
  			mapDist_(old->mapDist_),XorY_(old->XorY_){
	  //copies the RegulatedSexChromosome1 during construction
	}//end copy ctor
		



aString* RegulatedSexChromosome1::GenotypeString(char* separator){
	aString* gstr= NEW aString();
	for(int j=0;j<loci();++j){
		RegulatedAllele1& a = GetAllele(j);
	  	aString* allStr = a.GenotypeString(separator);
	  	gstr->cat(*allStr);
	  	delete allStr; allStr=NULL;
	  	if(j<loci()-1){gstr->cat(separator);}
	  	}//end for j
	return gstr;
  	}//end RegulatedSexChromosome1::GenotypeString(char)


aString* RegulatedSexChromosome1::GenotypeString(int allele, char* separator){
	RegulatedAllele1& a = GetAllele(allele);
	return a.GenotypeString(separator);
  	}//end RegulatedSexChromosome1::GenotypeString(int, char)


RegulatedAllele1& RegulatedSexChromosome1::GetAllele (int locus){ 
	//returns the reference of the allele
	return genes_[locus];
	}//end RegulatedSexChromosome1::GetAllele


float RegulatedSexChromosome1::GetAlleleValue (int locus){ 
	//returns the value of the allele asst the given dimension
	return genes_[locus].product();
	}//end RegulatedSexChromosome1::GetAlleleValue


float RegulatedSexChromosome1::GetPromoValue (int locus){ 
	//returns the value of the allele asst the given dimension
	return genes_[locus].promoter();
	}//end RegulatedSexChromosome1::GetPromoValue


void RegulatedSexChromosome1::Mutate(Ran3& ran, double mutationRate, double sigma){
	if(mutationRate<=0.0){return;}
	for(int loc=0;loc<loci();++loc){
		if(ran()<mutationRate){
			RegulatedAllele1& a = GetAllele(loc);
  			a.Mutate(ran,sigma);}
  		}
  	}//end RegulatedSexChromosome1::Mutate

		
void RegulatedSexChromosome1::Print(ostream& outFile, char* separator){
	aString* geneStr = GenotypeString(separator);
	outFile<<*geneStr<<endl;
	delete geneStr; geneStr=NULL;
	}//end RegulatedSexChromosome1::Print


void RegulatedSexChromosome1::Recombine(RegulatedSexChromosome1& c1, RegulatedSexChromosome1& c2, Ran3& ran){
	//makes a recombinant RegulatedSexChromosome2 from c1 & c2 
	//based on map distances among loci 
	if(c1.XorY() != c2.XorY()){return;}
	genes_.Clear();
	mapDist_ = c1.mapDist_;
	//first locus can come from either c1 or c2
	RegulatedSexChromosome1 presentChrom(c1);//use copy 
	RegulatedSexChromosome1 otherChrom(c2);//use copy 
	if(ran()<0.5){SWAP(presentChrom, otherChrom);}
	genes_.Append(&(presentChrom.genes_.Last()));//transfer
	presentChrom.genes_.DetachLast();
	otherChrom.genes_.DeleteLast();//delete allele not transferred
	for(int j=mapDist_.len()-1;j>=0;--j){
		if(ran()<mapDist_[j]){SWAP(presentChrom, otherChrom);}
		genes_.Append(&(presentChrom.genes_.Last()));
		presentChrom.genes_.DetachLast();
		otherChrom.genes_.DeleteLast();//delete allele not transferred
		}//end for j
	}//end RegulatedSexChromosome1::Recombine



void RegulatedSexChromosome1::RecombineWith(RegulatedSexChromosome1& c2, Ran3& ran){
	//makes 2 recombinant RegulatedSexChromosome1 from this & c2 
	//based on map distances among loci 
	//first locus can come from either c1 or c2
	//ASSUMES IDENTICAL MAP DISTANCES AMONG LOCI
	if(XorY() != c2.XorY()){return;}
	if(loci() ==1){
		SWAP(*this,c2);
		return;}
	RegulatedSexChromosome1* presentChrom = NEW RegulatedSexChromosome1(*this);
	RegulatedSexChromosome1* otherChrom = NEW RegulatedSexChromosome1(c2);
	if(ran()<0.5){
		RegulatedSexChromosome1* temp = presentChrom;
		presentChrom = otherChrom;
		otherChrom = temp;
		temp = NULL;
		}
	genes_.Clear();  c2.genes_.Clear();
	mapDist_ = presentChrom->mapDist_;
	c2.mapDist_ = otherChrom->mapDist_;
	genes_.Append(&(presentChrom->genes_.First()));
	c2.genes_.Append(&(otherChrom->genes_.First()));
	for(int j=0;j<presentChrom->loci()-1;++j){
		if(ran()<mapDist_[j]){
			RegulatedSexChromosome1* temp = presentChrom;
			presentChrom = otherChrom;
			otherChrom = temp;
			temp = NULL;
			}
		genes_.Append(&(presentChrom->genes_[j+1]) );
		c2.genes_.Append(&(otherChrom->genes_[j+1]) );
		}//end for j
	presentChrom->genes_.Detach();
	otherChrom->genes_.Detach();
	delete presentChrom; presentChrom=NULL;
	delete otherChrom; otherChrom=NULL;
	}//end RegulatedSexChromosome1::RecombineWith



void RegulatedSexChromosome1::set(int locus, float newValue, int allelePart){
	//changes the value of an allele
  	genes_[locus].set(newValue, allelePart);
  	}//end RegulatedSexChromosome1::set
	
	

RegulatedSexChromosome1& RegulatedSexChromosome1::operator= (RegulatedSexChromosome1& c2){
	XorY_ = c2.XorY();
	genes_ = c2.genes_;
	mapDist_=c2.mapDist_;
	return *this;
	}//end RegulatedSexChromosome1::operator=
	







int operator== (RegulatedSexChromosome1& c1, RegulatedSexChromosome1& c2){
	if(c1.XorY() != c2.XorY()){return 0;}
	if(c1.loci()!=c2.loci()){return 0;}
	for (int j=0;j<c1.loci();++j){
		if(c1.genes_[j] != c2.genes_[j]){return 0;}
		}
	for (int j=0;j<c1.loci()-1;++j){
		if(c1.mapDist_[j] != c2.mapDist_[j]){return 0;}
		}
	return 1;
	}//end friend operator==
		
int operator!= (RegulatedSexChromosome1& c1, RegulatedSexChromosome1& c2){
  	return !(c1==c2);}
  	


#endif









