/******************************************************
*
*	RegulatedSexChrom2.cpp
*
*	This Chromosome is either X or Y
* 	It holds a list of RegulatedAllele2 objects
*
*******************************************************/

#ifndef RegulatedSexChrom2cpp
#define RegulatedSexChrom2cpp = 1

#include "RegulatedSexChrom2.h"
#include "Element.cpp"
#include "aList.cpp"

  	
void RegulatedSexChromosome2::Copy(RegulatedSexChromosome2& old){
	//copies the RegulatedSexChromosome2
	genes_.Clear();  mapDist_.Clear(); 
	for (int j=0;j<old.loci();++j){
  		RegulatedAllele2& oldAllele = old.genes_[j];
  		RegulatedAllele2* allele = NEW RegulatedAllele2(oldAllele);
  		genes_.Append(allele);
  		}//end for j
	for (int j=0;j<old.mapDist_.len();++j){
  		mapDist_.Append(old.mapDist_[j]);
  		}//end for j
  	}//end RegulatedSexChromosome2::Copy(RegulatedSexChromosome2&)
		

void RegulatedSexChromosome2::Copy(RegulatedSexChromosome2* old){
	Copy(*old);
  	}//end RegulatedSexChromosome2>::Copy(RegulatedSexChromosome2*)
		
  	


RegulatedSexChromosome2::RegulatedSexChromosome2(
			int numLoci, float alleleVal, float promoVal, float mapDist, int XorY):
		XorY_(XorY){
  	//creates a 'null' homozygous individual at all loci
  	for(int j=0;j<numLoci;++j){
  		genes_.Append(NEW RegulatedAllele2(promoVal, promoVal, alleleVal, alleleVal));
  		}//end for loci
  	for(int j=0;j<numLoci-1;++j){
  		mapDist_.Append(mapDist);
  		}//end for loci-1
  	}//end ctor RegulatedSexChromosome2::RegulatedSexChromosome2(int,float,float,float,int)


RegulatedSexChromosome2::RegulatedSexChromosome2(
			aList<float>& alleleVal, aList<float>& promoVal, 
			aList<float>& mapDist, int XorY):
		XorY_(XorY), mapDist_(mapDist){
  	//creates a 'null' homozygous individual at all loci
 	for(int j=0;j<alleleVal.len();++j){
  		genes_.Append(NEW RegulatedAllele2(promoVal[j], promoVal[j], alleleVal[j], alleleVal[j]));
  		}//end for loci
  	}//end ctor RegulatedSexChromosome2::RegulatedSexChromosome2(aList<float>&,aList<float>&,aList<float>&,int)



RegulatedSexChromosome2::RegulatedSexChromosome2(
			int numLoci, RegulatedAllele2& allele, float mapDist, int XorY):
		XorY_(XorY){
  	//creates a 'null' chromosome with same allele at all loci,
  	//and same map distance between loci
  	//mapDist*(numLoci-1) must be 100
  	for(int a=0;a<numLoci;++a){
  		genes_.Append(NEW RegulatedAllele2(allele));
  		}//end for a
  	for(int j=0;j<numLoci-1;++j){
  		mapDist_.Append(mapDist);
  		}//end for loci-1
  	}//end ctor RegulatedSexChromosome2::RegulatedSexChromosome2(int,RegulatedAllele2&,float,int)
  			
  			
RegulatedSexChromosome2::RegulatedSexChromosome2(RegulatedSexChromosome2& old):
  			genes_(old.genes_), 
  			mapDist_(old.mapDist_),XorY_(old.XorY_){
	  //copies the RegulatedSexChromosome2 during construction
	}//end copy ctor
		
RegulatedSexChromosome2::RegulatedSexChromosome2(RegulatedSexChromosome2* old):
  			genes_(old->genes_), 
  			mapDist_(old->mapDist_),XorY_(old->XorY_){
	  //copies the RegulatedSexChromosome2 during construction
	}//end copy ctor
		



aString* RegulatedSexChromosome2::GenotypeString(char* separator){
	aString* gstr= NEW aString();
	for(int j=0;j<loci();++j){
		RegulatedAllele2& a = GetAllele(j);
	  	aString* allStr = a.GenotypeString(separator);
	  	gstr->cat(*allStr);
	  	delete allStr; allStr=NULL;
	  	if(j<loci()-1){gstr->cat(separator);}
	  	}//end for j
	return gstr;
  	}//end RegulatedSexChromosome2::GenotypeString(char)


aString* RegulatedSexChromosome2::GenotypeString(int allele, char* separator){
	RegulatedAllele2& a = GetAllele(allele);
	return a.GenotypeString(separator);
  	}//end RegulatedSexChromosome2::GenotypeString(int, char)


RegulatedAllele2& RegulatedSexChromosome2::GetAllele (int locus){ 
	//returns the reference of the allele
	return genes_[locus];
	}//end RegulatedSexChromosome2::GetAllele


float RegulatedSexChromosome2::GetAlleleValue (int locus, int dimension){ 
	//returns the value of the allele asst the given dimension
	return genes_[locus].product(dimension);
	}//end RegulatedSexChromosome2::GetAlleleValue


float RegulatedSexChromosome2::GetPromoValue (int locus, int dimension){ 
	//returns the value of the allele asst the given dimension
	return genes_[locus].promoter(dimension);
	}//end RegulatedSexChromosome2::GetPromoValue


void RegulatedSexChromosome2::Mutate(Ran3& ran, double mutationRate, double sigma){
	if(mutationRate<=0.0){return;}
	for(int loc=0;loc<loci();++loc){
		if(ran()<mutationRate){
			RegulatedAllele2& a = GetAllele(loc);
  			a.Mutate(ran,sigma);}
  		}
  	}//end RegulatedSexChromosome2::Mutate

		
void RegulatedSexChromosome2::Print(ostream& outFile, char* separator){
	aString* geneStr = GenotypeString(separator);
	outFile<<*geneStr<<endl;
	delete geneStr; geneStr=NULL;
	}//end RegulatedSexChromosome2::Print


void RegulatedSexChromosome2::Recombine(RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2, Ran3& ran){
	//makes a recombinant RegulatedSexChromosome2 from c1 & c2 
	//based on map distances among loci 
	if(c1.XorY() != c2.XorY()){return;}
	genes_.Clear();
	mapDist_ = c1.mapDist_;
	//first locus can come from either c1 or c2
	RegulatedSexChromosome2 presentChrom(c1);//use copy 
	RegulatedSexChromosome2 otherChrom(c2);//use copy 
	if(ran()<0.5){SWAP(presentChrom, otherChrom);}
	genes_.Append(&(presentChrom.genes_.Last()));//transfer
	presentChrom.genes_.DetachLast();
	otherChrom.genes_.DeleteLast();//delete allele not transferred
	for(int j=mapDist_.len()-1;j>=0;--j){
		if(ran()<mapDist_[j]){SWAP(presentChrom, otherChrom);}
		genes_.Append(&(presentChrom.genes_.Last()));
		presentChrom.genes_.DetachLast();
		otherChrom.genes_.DeleteLast();//delete allele not transferred
		}//end for j
	}//end RegulatedSexChromosome2::Recombine



void RegulatedSexChromosome2::RecombineWith(RegulatedSexChromosome2& c2, Ran3& ran){
	//makes 2 recombinant RegulatedSexChromosome2 from this & c2 
	//based on map distances among loci 
	//first locus can come from either c1 or c2
	//ASSUMES IDENTICAL MAP DISTANCES AMONG LOCI
	if(XorY() != c2.XorY()){return;}
	if(loci() ==1){
		SWAP(*this,c2);
		return;}
	RegulatedSexChromosome2* presentChrom = NEW RegulatedSexChromosome2(*this);
	RegulatedSexChromosome2* otherChrom = NEW RegulatedSexChromosome2(c2);
	if(ran()<0.5){
		RegulatedSexChromosome2* temp = presentChrom;
		presentChrom = otherChrom;
		otherChrom = temp;
		temp = NULL;
		}
	genes_.Clear();  c2.genes_.Clear();
	mapDist_ = presentChrom->mapDist_;
	c2.mapDist_ = otherChrom->mapDist_;
	genes_.Append(&(presentChrom->genes_.First()));
	c2.genes_.Append(&(otherChrom->genes_.First()));
	for(int j=0;j<presentChrom->loci()-1;++j){
		if(ran()<mapDist_[j]){
			RegulatedSexChromosome2* temp = presentChrom;
			presentChrom = otherChrom;
			otherChrom = temp;
			temp = NULL;
			}
		genes_.Append(&(presentChrom->genes_[j+1]) );
		c2.genes_.Append(&(otherChrom->genes_[j+1]) );
		}//end for j
	presentChrom->genes_.Detach();
	otherChrom->genes_.Detach();
	delete presentChrom; presentChrom=NULL;
	delete otherChrom; otherChrom=NULL;
	}//end RegulatedSexChromosome2::RecombineWith



void RegulatedSexChromosome2::set(int locus, float newValue, int allelePart){
	//changes the value of an allele
  	genes_[locus].set(newValue, allelePart);
  	}//end RegulatedSexChromosome2::set
	
	

RegulatedSexChromosome2& RegulatedSexChromosome2::operator= (RegulatedSexChromosome2& c2){
	XorY_ = c2.XorY();
	genes_ = c2.genes_;
	mapDist_=c2.mapDist_;
	return *this;
	}//end RegulatedSexChromosome2::operator=
	







int operator== (RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2){
	if(c1.XorY() != c2.XorY()){return 0;}
	if(c1.loci()!=c2.loci()){return 0;}
	for (int j=0;j<c1.loci();++j){
		if(c1.genes_[j] != c2.genes_[j]){return 0;}
		}
	for (int j=0;j<c1.loci()-1;++j){
		if(c1.mapDist_[j] != c2.mapDist_[j]){return 0;}
		}
	return 1;
	}//end friend operator==
		
int operator!= (RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2){
  	return !(c1==c2);}
  	
/*
int operator< (RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2){
	//compares by allele value, for sorting
	int shortLen = min(c1.loci(), c2.loci() );
	for (int j=0;j<shortLen;++j){
		if(c1.genes_[j] < c2.genes_[j]){return 1;}
		}
	if(c1.loci() < c2.loci()){return 1;}
	return 0;//default returns false
	}//end friend operator<
  	
int operator> (RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2){
	//compares by allele value, for sorting
	int shortLen = min(c1.loci(), c2.loci() );
	for (int j=0;j<shortLen;++j){
		if(c1.genes_[j] < c2.genes_[j]){return 1;}
		}
	if(c1.loci() > c2.loci()){return 1;}
	return 0;//default returns false
	}//end friend operator<
  	
int operator<= (RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2){
	//compares by allele value, for sorting
	return !(c1 > c2);}
  	
int operator>= (RegulatedSexChromosome2& c1, RegulatedSexChromosome2& c2){
	//compares by allele value, for sorting
	return !(c1 < c2);}


  */


#endif









