/******************************************************
*
*	RegulatedSexGenome2.cpp
*
*	This class implements a regulated genetic pathway that has 
*	a promoter and product at each locus, and 2 states for promoters & products.
*	All loci are unlinked, and the chromosome number is diploid  
*	It handles cloning and sexual reproduction.
*
*	It includes a separate sex chromosome, X or Y
*	Here, XX is female and XY is male

	In chromosome designations, the sex-chromosome is the LAST chromosome.
*
*	Base class:  RegulatedGenome2(Phenotype)
*
*******************************************************/
#ifndef RegulatedSexGenome2cpp
#define RegulatedSexGenome2cpp = 1

#include "RegulatedSexGenome2.h"
#include "RandomList.cpp"
#include "Element.cpp"
#include "aList.cpp"




RegulatedSexGenome2::RegulatedSexGenome2(void):RegulatedGenome2(){
  		//creates empty genome
//  		sexGenes_.Append(new RegulatedSexChromosome2());		
  		}


RegulatedSexGenome2::RegulatedSexGenome2(
	int numLoci, int ploidy, float alleleVal, float promoValue, 
	genderSet sex,int numX_genes, int numY_genes):
  			RegulatedGenome2(numLoci,ploidy,&alleleVal,&promoValue){
			//creates a 'null' homozygous individual at all loci
		float mapX = 1.0/(float)numX_genes;
		float mapY=0.0;
		if(numY_genes){mapY = 1.0/(float)numY_genes;}
  		sexGenes_.Append(NEW RegulatedSexChromosome2(numX_genes,alleleVal,promoValue,mapX, 0));
  			//one X-chromosome		
  		if(ploidy==2){//diploid; haplodiploid handled here too by default
  			if(sex==F){//female
  					sexGenes_.Append(NEW RegulatedSexChromosome2(numX_genes,alleleVal,promoValue,mapX,0));//X-chromosome	
  					}
  				else if(numY_genes > 0){//	XO sex determination modeled if no y-linked loci used
  					sexGenes_.Append(NEW RegulatedSexChromosome2(numY_genes,alleleVal,promoValue,mapY,1));//X-chromosome	
  					}
  				else{}
  			}//end if ploidy
  		}//end ctor RegulatedSexGenome2
  		
  	
RegulatedSexGenome2::RegulatedSexGenome2(
	int numLoci, int ploidy, float alleleVal, float promoValue, 
	genderSet sex,int numX_genes, int numY_genes, 
  			ViabilitySelectionFunc viabSelection):
  			RegulatedGenome2(numLoci,ploidy,&alleleVal,&promoValue,viabSelection){
			//creates a 'null' homozygous individual at all loci
		float mapX = 1.0/(float)numX_genes;
		float mapY=0.0;
		if(numY_genes){mapX = 1.0/(float)numY_genes;}
  		sexGenes_.Append(NEW RegulatedSexChromosome2(numX_genes,alleleVal,promoValue,mapX,0));
  			//one X-chromosome		
  		if(ploidy==2){//diploid; haplodiploid handled here too by default
  			if(sex==F){//female
  					sexGenes_.Append(NEW RegulatedSexChromosome2(numX_genes,alleleVal,promoValue,mapX,0));//X-chromosome	
  					}
  				else if(numY_genes > 0){//	XO sex determination modeled if no y-linked loci used
  					sexGenes_.Append(NEW RegulatedSexChromosome2(numY_genes,alleleVal,promoValue,mapY,1));//X-chromosome	
  					}
  				else{}
  			}//end if ploidy
  		}//end ctor RegulatedSexGenome2
  		
  	
RegulatedSexGenome2::RegulatedSexGenome2(RegulatedSexGenome2& g): 
			RegulatedGenome2(g),sexGenes_(g.sexGenes_) {
  		//copy constructor
  		}

RegulatedSexGenome2::RegulatedSexGenome2(RegulatedSexGenome2* g): 
			RegulatedGenome2(*g),sexGenes_(g->sexGenes_) {
  		//copy constructor
  		}

RegulatedSexGenome2::RegulatedSexGenome2(ViabilitySelectionFunc viabSelection):
			RegulatedGenome2(viabSelection){
		//creates empty genome
  		}

RegulatedSexGenome2::RegulatedSexGenome2(RegulatedSexGenome2& g,
		ViabilitySelectionFunc viabSelection):
  			RegulatedGenome2(g,viabSelection), 
  			sexGenes_(g.sexGenes_) {
  		//copy constructor
  		}

RegulatedSexGenome2::~RegulatedSexGenome2(void){
		sexGenes_.Clear();
		}//end destructor









double  RegulatedSexGenome2::additiveRelativePhenotype(void){//additive phenotype
	//uses product only
	double sum=RegulatedGenome2::additiveRelativePhenotype();
	for(int ch=0;ch<sexGenes_.len();++ch){
		for(int loc=0;loc<sexGenes_[ch].loci();++loc){
			sum += sexGenes_[ch][loc].product(0);
			sum += sexGenes_[ch][loc].product(1);
			}}
	return sum;
	}//end RegulatedSexGenome2::additiveRelativePhenotype



    		
int	RegulatedSexGenome2::AllelePresent (int loc, aString& alleleVal){
	int autosomalLoci = RegulatedGenome2::loci();
	if(loc < autosomalLoci){ // use base class function for autosomes
		return RegulatedGenome2::AllelePresent(loc,alleleVal);}
	//the locus is on a sex chromosome
	int sexLocus = loc - autosomalLoci;
	int Xloci=0;//check if on the X
	for(int ch=0;ch<sexGenes_.len();++ch){
		if(sexGenes_[ch].IsX()){
			Xloci = sexGenes_[ch].loci();
			break;
		}}
	if (sexLocus < Xloci){//it would be on X chromosome(s)
			for(int ch=0;ch<sexGenes_.len();++ch){
				if(sexGenes_[ch].IsX()){
					aString* val = sexGenes_[ch][sexLocus].GenotypeString(" ");
					if(*val == alleleVal){
						delete val;val = NULL;
						return 1;//found a match
						}
					delete val;val = NULL;
					}//end if IsX
				}//end for ch
			return 0;//it's absent
			}
		else{//it would be on Y, if Y present
			sexLocus -= Xloci;
			for(int ch=0;ch<sexGenes_.len();++ch){
				if(sexGenes_[ch].IsY()){
					aString* val = sexGenes_[ch][sexLocus].GenotypeString(" ");
					if(*val == alleleVal){
						delete val;val = NULL;
						return 1;//found a match
						}
					delete val;val = NULL;
					}//end if IsY
				}//end for ch
			return 0;//it's absent
			}
	return 0;//default
	}//end RegulatedSexGenome2::AllelePresent



int	RegulatedSexGenome2::chroms1N(void){
	return genes_.len() + 1 + HasY();//last is the sex chromosome, and add 1 if Y chromosome is present
	}// end RegulatedSexGenome2::chroms1N


int	RegulatedSexGenome2::CountAlleles (int loc, aString& alleleVal){
				//counts the absolute frequency of alleleVal
	int autosomalLoci = RegulatedGenome2::loci();
	if(loc < autosomalLoci){ // use base class function for autosomes
		return RegulatedGenome2::CountAlleles(loc,alleleVal);}

	//the locus is on a sex chromosome
	int num = 0;
	int sexLocus = loc - autosomalLoci;
	int Xloci=0;//check if on the X
	for(int ch=0;ch<sexGenes_.len();++ch){
		if(sexGenes_[ch].IsX()){
			Xloci = sexGenes_[ch].loci();
			break;
		}}
	if (sexLocus < Xloci){//it would be on X chromosome(s)
			for(int ch=0;ch<sexGenes_.len();++ch){
				if(sexGenes_[ch].IsX()){
					aString* val = sexGenes_[ch][sexLocus].GenotypeString(" ");
					if(*val == alleleVal){
						++num;//found a match
						}
					delete val;val = NULL;
					}//end if IsX
				}//end for ch
			}
		else{//it would be on Y, if Y present
			sexLocus -= Xloci;
			for(int ch=0;ch<sexGenes_.len();++ch){
				if(sexGenes_[ch].IsY()){
					aString* val = sexGenes_[ch][sexLocus].GenotypeString(" ");
					if(*val == alleleVal){
						++num;//found a match
						}
					delete val;val = NULL;
					}//end if IsY
				}//end for ch
			}
	return num;
	}//end RegulatedSexGenome2::CountAlleles



void RegulatedSexGenome2::Fertilize (CoreGenome* momGamete, CoreGenome* dadGamete){
	//virtual-copies parent genes to new zygote (=*this) during reproduction
	//gametes remain intact
	RegulatedGenome2::Fertilize(momGamete,dadGamete);//does autosomes
	sexGenes_.Clear();
	RegulatedSexGenome2* mommyGamete = (RegulatedSexGenome2*) momGamete;
	RegulatedSexGenome2* daddyGamete = (RegulatedSexGenome2*) dadGamete;
	for (int ch=0;ch<mommyGamete->sexGenes_.len();++ch){
		sexGenes_.Append(mommyGamete->sexGenes_[ch].Duplicate() );
		}//end for ch
	for (int ch=0;ch<daddyGamete->sexGenes_.len();++ch){
		sexGenes_.Append(daddyGamete->sexGenes_[ch].Duplicate() );
		}//end for ch	
	}//RegulatedSexGenome2::Fertilize



void RegulatedSexGenome2::Gamete (CoreGenome* theParentGenes, Ran3List& ran, double mutationRate, double sigma){
	//does meiosis to turn the genotype into a gamete of parentGenes
	//robust to haplodiploidy, genetic sex determination of XY or XO type
	RegulatedGenome2::Gamete (theParentGenes,ran,mutationRate,sigma);//autosomes
	sexGenes_.Clear();
	RegulatedSexGenome2* parentGenes = (RegulatedSexGenome2*) theParentGenes;
	RegulatedSexGenome2* oldGenes = (RegulatedSexGenome2*) parentGenes->Duplicate();
	//work with a new copy to sample without replacement
	//and to avoid permanent recombination
	//determine if all loci haploid --robust to genetic sex determination
	for(int ch1=0;ch1<oldGenes->sexGenes_.len()-1;++ch1){
		//handles polyploid recombination
		//X and Y don't recombine in this function
		for(int ch2=ch1+1;ch2<sexGenes_.len();++ch2){
			oldGenes->sexGenes_[ch1].RecombineWith(oldGenes->sexGenes_[ch2],ran);
			}}
	int parentChroms = oldGenes->sexGenes_.len();
	int totalChromsToTransfer = parentChroms/2;
	if( odd(parentChroms) ){//if odd number of sex chroms
		if(ran()<0.5){totalChromsToTransfer++;}}//include one more 50% of time
	for(int ch=0;ch<totalChromsToTransfer;++ch){
		int chromToTransfer = ran.longBetween(0,oldGenes->sexGenes_.len()-1);
		sexGenes_.Append(oldGenes->sexGenes_[chromToTransfer].Duplicate());
		oldGenes->sexGenes_.Delete(chromToTransfer);
 		sexGenes_.Last().Mutate(ran, mutationRate, sigma);
		}
	delete oldGenes;oldGenes=0;
	}//end RegulatedSexGenome2::Gamete



int RegulatedSexGenome2::GenePresent(int locus, int alleleNum){
	int autosomalLoci = RegulatedGenome2::loci();
	if(locus<autosomalLoci){//it's autosomal
		return RegulatedGenome2::GenePresent(locus,alleleNum);}
	
	//the locus is on a sex chromosome
	if(alleleNum >= sexGenes_.len()){
		return 0;}
	int sexLocus = locus - autosomalLoci;
	int Xloci=0;//check if on the X
	for(int ch=0;ch<sexGenes_.len();++ch){
		if(sexGenes_[ch].IsX()){
			Xloci = sexGenes_[ch].loci();
			break;
		}}
	if (sexLocus < Xloci){//the locus is on the X chromosome
			if(sexGenes_[alleleNum].IsX()){
				return 1;
				}//end if IsX				
			}
		else{//the locus is on the Y, if Y present
			if(sexGenes_[alleleNum].IsY()){
				return 1;
				}//end if IsY
			}
	return 0;//default
	}//end RegulatedSexGenome2::GenePresent



aString*  RegulatedSexGenome2::GenotypeString(char* separator){
					//returns all ploidy copies separated by parentheses (A)(A) at each locus
					//this permits complex alleles to be represented easily
	//returns all ploidy copies separated by parentheses (A)(A) at each locus
	//this permits complex alleles to be represented easily
	aString* gtstr = RegulatedGenome2::GenotypeString(separator);
	int autosomalLoci = RegulatedGenome2::loci();
	for (int locus=autosomalLoci;locus<loci();++locus){
		aString* gstr = GenotypeString(locus,separator);
		gtstr->cat(*gstr);
		gtstr->cat(separator);
		delete gstr;gstr=NULL;
		}//end for locus
	return gtstr;
  	}//end RegulatedSexGenome2::GenotypeString(char*)



aString* RegulatedSexGenome2::GenotypeString(int locus, char* separator){
	//finds nth overall locus and returns all ploidy copies separated by parentheses (A)(A)
	//this permits complex alleles to be represented easily
	int autosomalLoci = RegulatedGenome2::loci();
	if(locus<autosomalLoci){//it's autosomal
		return RegulatedGenome2::GenotypeString(locus,separator);}
	//the locus is on a sex chromosome
	aString* gtstr = NEW aString();
	char spar='(', epar=')';

	int sexLocus = locus - autosomalLoci;
	int Xloci=0;//check if on the X
	for(int ch=0;ch<sexGenes_.len();++ch){
		if(sexGenes_[ch].IsX()){
			Xloci = sexGenes_[ch].loci();
			break;
		}}
	if (sexLocus < Xloci){//the locus is on the X chromosome
			for(int ch=0;ch<sexGenes_.len();++ch){
				if(sexGenes_[ch].IsX()){
					aString* gstr = sexGenes_[ch][sexLocus].GenotypeString(separator);
					gtstr->cat(*gstr);
					delete gstr;gstr=NULL;
					}//end if IsX
				}//end for ch
			return 0;//it's absent
			}
		else{//the locus is on the Y, if Y present
			sexLocus -= Xloci;
			for(int ch=0;ch<sexGenes_.len();++ch){
				if(sexGenes_[ch].IsY()){
					aString* gstr = sexGenes_[ch][sexLocus].GenotypeString(separator);
					gtstr->cat(*gstr);
					delete gstr;gstr=NULL;
					}//end if IsY
				}//end for ch
			}
	return gtstr;
  	}//end RegulatedSexGenome2::GenotypeString
  	


aString* RegulatedSexGenome2::GenotypeString(int locus, int alleleNum,  char* separator){
	//finds nth overall locus and returns the alleleNum_th copy
	//if the allelenum-th copy is of the wrong sex chromosome, a blank "_" is returned
	int autosomalLoci = RegulatedGenome2::loci();
	if(locus<autosomalLoci){//it's autosomal
		return RegulatedGenome2::GenotypeString(locus,alleleNum,separator);}

	//the locus is on a sex chromosome
	if (alleleNum >= sexGenes_.len()){//return a blank
		return NEW aString("_");}
	//find the right chromosome
	int sexLocus = locus - autosomalLoci;
	int Xloci=0;//check if on the X
	for(int ch=0;ch<sexGenes_.len();++ch){
		if(sexGenes_[ch].IsX()){
			Xloci = sexGenes_[ch].loci();
			break;
		}}
	if (sexLocus < Xloci){//the locus is on the X chromosome
			if(sexGenes_[alleleNum].IsX()){
				aString* gstr = sexGenes_[alleleNum][sexLocus].GenotypeString(separator);
				return gstr;
				}//end if IsX
			return NEW aString("_");//return blank if Y occupiies this spot
			}
		else{//the locus is on the Y, if Y present
			sexLocus -= Xloci;
			if(sexGenes_[alleleNum].IsY()){
				aString* gstr = sexGenes_[alleleNum][sexLocus].GenotypeString(separator);
				return gstr;
				}//end if IsY
			return NEW aString("_");//return blank if X occupiies this spot
			}
	return NEW aString("_");//default
  	}//end RegulatedSexGenome2::GenotypeString
  	


RegulatedAllele2& RegulatedSexGenome2::GetGene(int locus, int alleleNum){
	int autosomalLoci = RegulatedGenome2::loci();
	if(locus<autosomalLoci){//it's autosomal
		return RegulatedGenome2::GetGene(locus,alleleNum);}

	//the locus is on a sex chromosome
	int sexLocus = locus - autosomalLoci;
	int Xloci=0;//check if on the X
	for(int ch=0;ch<sexGenes_.len();++ch){
		if(sexGenes_[ch].IsX()){
			Xloci = sexGenes_[ch].loci();
			break;
		}}
	if (sexLocus < Xloci){//the locus is on the X chromosome
			if(sexGenes_[alleleNum].IsX()){
				return sexGenes_[alleleNum][sexLocus];
				}//end if IsX
			}
		else{//the locus is on the Y, if Y present
			sexLocus -= Xloci;
			if(sexGenes_[alleleNum].IsY()){
				return sexGenes_[alleleNum][sexLocus];
				}//end if IsY
			}
	aError("no gene found in call to RegulatedSexGenome2::GetGene.  Nonsense default returned.");
	return sexGenes_[0][0];
	}//end RegulatedSexGenome2::GetGene



int RegulatedSexGenome2::HasY(void){
	//0 if false, 1 if Y chromosome present
	for(int ch=0;ch<sexGenes_.len();++ch){
		if(sexGenes_[ch].IsY()){
			return 1;}
		}
	return 0;
	}//end  RegulatedSexGenome2::HasY



int	RegulatedSexGenome2::IsHeterozygote (int locus){
	//returns false=0 if haploid
	int autosomalLoci = RegulatedGenome2::loci();
	if(locus<autosomalLoci){//it's autosomal
		return RegulatedGenome2::IsHeterozygote(locus);}
	if(ploidy(locus)<2){return 0;}
	//find the right chromosome
	int sexLocus = locus - autosomalLoci;
	int Xloci=0;//check if on the X
	for(int ch=0;ch<sexGenes_.len();++ch){
		if(sexGenes_[ch].IsX()){
			Xloci = sexGenes_[ch].loci();
			break;
		}}
	if (sexLocus < Xloci){//the locus is on the X chromosome
			int foundFirstX=0;
			RegulatedAllele2* firstCopy;
			for(int ch=0;ch<sexGenes_.len();++ch){
				if(sexGenes_[ch].IsX()){
					if(foundFirstX==0){
							foundFirstX=1;
							firstCopy = &(sexGenes_[ch][sexLocus]);
							}
						else{
							if(*firstCopy != sexGenes_[ch][sexLocus]){
								return 1;//must be heterozygous
								}}
					}
				}//end for ch
			return 0;//it's homozygote or only one copy exists
			}
		else{//the locus is on the Y, if Y present
			sexLocus -= Xloci;
			int foundFirstY=0;
			RegulatedAllele2* firstCopy;
			for(int ch=0;ch<sexGenes_.len();++ch){
				if(sexGenes_[ch].IsY()){
					if(foundFirstY==0){
							foundFirstY=1;
							firstCopy = &(sexGenes_[ch][sexLocus]);
							}
						else{
							if(*firstCopy != sexGenes_[ch][sexLocus]){
								return 1;//must be heterozygous
								}}
					}
				}//end for ch
			return 0;//it's homozygote or only one copy exists
			}
	return 0;//default
	}//end RegulatedSexGenome2::IsHeterozygote


int RegulatedSexGenome2::loci(void){
	int autosomalLoci = RegulatedGenome2::loci();
	int Xloci=0;//count loci on X
	for(int ch=0;ch<sexGenes_.len();++ch){
		if(sexGenes_[ch].IsX()){
			Xloci = sexGenes_[ch].loci();
			break;
		}}
	int Yloci=0;//count loci on Y if present
	for(int ch=0;ch<sexGenes_.len();++ch){
		if(sexGenes_[ch].IsY()){
			Yloci = sexGenes_[ch].loci();
			break;
		}}
	return autosomalLoci + Xloci + Yloci;
	}//end RegulatedSexGenome2::loci




void RegulatedSexGenome2::Mutate(Ran3& ran, double mutationRate, double sigma){
	//sigma is s.d. of mutation effect size
	RegulatedGenome2::Mutate(ran, mutationRate, sigma);
	for(int loc=0;loc<sexGenes_.len();++loc){
		sexGenes_[loc].Mutate(ran,mutationRate,sigma);
		}
	}//end RegulatedSexGenome2::Mutate(Ran3&, double, double)



void RegulatedSexGenome2::Print(ostream& outFile, int /*alleleLen*/){
	aString* geneStr = GenotypeString(" ");//dummy starter value will be cleared anyway
	outFile<<*geneStr<<endl;
	delete geneStr; geneStr=NULL;
	}//end RegulatedSexGenome2::Print




void RegulatedSexGenome2::set(int locus, int alleleNum, int allelePart, float newAlleleVal){
	//changes the value of an allele
	int autosomalLoci = RegulatedGenome2::loci();
	if(locus<autosomalLoci){//it's autosomal
		RegulatedGenome2::set(locus,alleleNum,allelePart,newAlleleVal);
		return;
		}//end if autosomal

	//the locus is on a sex chromosome
	if (alleleNum >= sexGenes_.len()){//do nothing
		return;}
	//find the right chromosome
	int sexLocus = locus - autosomalLoci;
	int Xloci=0;//check if on the X
	for(int ch=0;ch<sexGenes_.len();++ch){
		if(sexGenes_[ch].IsX()){
			Xloci = sexGenes_[ch].loci();
			break;
		}}
	if (sexLocus < Xloci){//the locus is on the X chromosome
			if(sexGenes_[alleleNum].IsX()){
				sexGenes_[alleleNum].set(sexLocus,allelePart,newAlleleVal);
				}//end if IsX
			return;
			}
		else{//the locus is on the Y, if Y present
			sexLocus -= Xloci;
			if(sexGenes_[alleleNum].IsY()){
				sexGenes_[alleleNum].set(sexLocus,allelePart,newAlleleVal);
				}//end if IsY
			return;
			}
	return;//default
  	}//end RegulatedGenome2::set
	


void RegulatedSexGenome2::SetGenotypeString(aString& geneStr, int){
	//standaridization ignored
	aString* newStr = GenotypeString(" ");
	geneStr = *newStr;//copies
	delete newStr;newStr=NULL;
	}//end RegulatedSexGenome2::SetGenotypeString(aString&, int)




void RegulatedSexGenome2::SetSexualZygote(CoreGenome* momGenes, CoreGenome* dadGenes, Ran3List& ran, double mutationRate, double sigma){
	//uses mom's and dad's genes to seed the genotype
	RegulatedSexGenome2* mommyGenes = (RegulatedSexGenome2*) momGenes;
	RegulatedSexGenome2* daddyGenes = (RegulatedSexGenome2*) dadGenes;
	RegulatedSexGenome2* momGamete = (RegulatedSexGenome2*) mommyGenes->Duplicate();
	RegulatedSexGenome2* dadGamete = (RegulatedSexGenome2*) daddyGenes->Duplicate();
	momGamete->Gamete(momGenes,ran,mutationRate,sigma);//genes copied to gamete
	dadGamete->Gamete(dadGenes,ran,mutationRate,sigma);
	Fertilize(momGamete, dadGamete);//momGamete & dadGamete are emptied in this step
	delete momGamete; momGamete=NULL;
	delete dadGamete; dadGamete=NULL;
	}//end RegulatedSexGenome2::SetSexualZygote


genderSet RegulatedSexGenome2::sex(genderSet /*gender*/){//returns the gender from the genome
	genderSet theSex = M;//male by default (haplodiploid, etc.)
	int xchroms=0;
	for(int ch=0;ch<sexGenes_.len();++ch){
		if(sexGenes_[ch].IsX()){
			++xchroms;}
		}
	if(xchroms>1){theSex=F;}
	return theSex;
	}//end RegulatedSexGenome2::sex(genderSet)


RegulatedSexGenome2& RegulatedSexGenome2::operator=(RegulatedSexGenome2& c){
	RegulatedGenome2::Copy(c);
	sexGenes_.Copy(c.sexGenes_);
	return *this;
	}//end RegulatedSexGenome2::operator=(RegulatedSexGenome2& c)



int operator==(RegulatedSexGenome2& a, RegulatedSexGenome2& b){
	return (((RegulatedGenome2) a == (RegulatedGenome2) b) &&
			(a.sexGenes_ == b.sexGenes_));}



int operator!=(RegulatedSexGenome2& a, RegulatedSexGenome2& b){
			return ! (a==b);}
	


	


#endif




