/******************************************************
*
*	SexChromosome.h
*
*	This Chromosome is either X or Y
*
*	Base class:  Chromosome<T> 
*
*******************************************************/
#ifndef SexChromosomecp
#define SexChromosomecp = 1

#include "SexChromosome.h"
#include "Element.cpp"
#include "aList.cpp"
#include "Allele.cpp"
#include "Chromosome.cpp"


  	
  	

template <class T> void SexChromosome<T>::Copy(SexChromosome<T>& old){
	//copies the SexChromosome
	Chromosome<T>::Copy(old);
	XorY_ = old.XorY();	
  	}//end SexChromosome<T>::Copy(SexChromosome<T>&)
		

template <class T> void SexChromosome<T>::Copy(SexChromosome<T>* old){
	//copies the Chromosome
	Chromosome<T>::Copy(old);
	XorY_ = old->XorY();	
  	}//end SexChromosome<T>::Copy(SexChromosome<T>*)
		
  	


template <class T> SexChromosome<T>::SexChromosome(int numLoci, T alleleVal, float mapDist, int XorY):
			XorY_(XorY){
  	//creates a 'null' homozygous individual at all loci
  	for(int j=0;j<numLoci;++j){
  		Allele<T>* a = NEW Allele<T>(alleleVal);
  		Append(a);
  		}//end for loci
  	for(int j=0;j<numLoci-1;++j){
  		mapDist_.Append(mapDist);
  		}//end for loci-1
  	}//end constructor SexChromosome<T>::SexChromosome(int,int,float,int)


template <class T> SexChromosome<T>::SexChromosome(int numLoci, Allele<T>& a, float mapDist, int XorY):
			XorY_(XorY){
  	//creates a 'null' homozygous individual at all loci
  	for(int j=0;j<numLoci;++j){
//  		Allele<T>* aa = new Allele<T>(a);
  		Allele<T>* aa = a.Duplicate();
  		Append(aa);
  		}//end for loci
  	for(int j=0;j<numLoci-1;++j){
  		mapDist_.Append(mapDist);
  		}//end for loci-1
  	}//end constructor SexChromosome<T>::SexChromosome(int,Allele<T>&,float,int)


template <class T> SexChromosome<T>::SexChromosome( aList<T>& alleleVal, aList<float>& mapDist, int XorY):
			XorY_(XorY),mapDist_(mapDist){
  	for(int a=0;a<alleleVal.len();++a){
//  		Allele<T>* allele = new Allele<T>(alleleVal[a]);
  		Allele<T>* allele = NEW Allele<T>(alleleVal[a]);
  		Append(allele);
  		}//end for a
  	}//end constructor SexChromosome<T>::SexChromosome(aList<T>&, List<float>&, int)
  			
  			
template <class T> SexChromosome<T>::SexChromosome( ObjList<Allele<T> >& alleleList, aList<float>& mapDist, int XorY):
			XorY_(XorY), mapDist_(mapDist){
  	for(int a=0;a<alleleList.len();++a){
//  		Allele<T>* allele = new Allele<T>(alleleList[a]);
  		Allele<T>* allele = alleleList[a].Duplicate();
  		Append(allele);
  		}//end for a
  	}//end constructor SexChromosome<T>::SexChromosome( ObjList<Allele<T>>&, List<float>&, int)
  			
  			
template <class T> SexChromosome<T>::SexChromosome(SexChromosome<T>& old):  //copies the Chromosome during construction
  				ObjList<Allele<T> >(old), 
  				mapDist_(old.mapDist_),
  				XorY_(old.XorY()){
				}
		
template <class T> SexChromosome<T>::SexChromosome(SexChromosome<T>* old)://copies the Chromosome during construction
  				ObjList<Allele<T> >(*old), 
  				mapDist_(old->mapDist_),
  				XorY_(old->XorY()){
				}


template <class T> int SexChromosome<T>::IsX(void){
	if (XorY()==0){
		return 1;}
	return 0;
	}//default is not X chromosome


template <class T> int SexChromosome<T>::IsY(void){	
	if (XorY()==1){
		return 1;}
	return 0;
	}




template <class T> void SexChromosome<T>::Recombine(SexChromosome<T>& c1, SexChromosome<T>& c2, Ran3& ran){
	//makes a recombinant SexChromosome from c1 & c2 
	if(c1.IsX() && c2.IsX()){
		Chromosome<T>::Recombine(c1, c2, ran);}
	}//end SexChromosome<T>::Recombine



template <class T> void SexChromosome<T>::RecombineWith(SexChromosome<T>& c2, Ran3& ran){
	//makes 2 recombinant SexChromosomes from this & c2 
	//ASSUMES IDENTICAL MAP DISTANCES AMONG LOCI
		if(IsX() && c2.IsX()){
		Chromosome<T>::RecombineWith(c2, ran);}
	}//end SexChromosome<T>::RecombineWith




template <class T> int operator== (SexChromosome<T>& c1, SexChromosome<T>& c2){
	if(c1.XorY()!=c2.XorY()){return 0;}
	return ((Chromosome<T>&) c1 == (Chromosome<T>&) c2);
	}//end friend operator==
		
template <class T> int operator!= (SexChromosome<T>& g1, SexChromosome<T>& g2){
  	return !(g1==g2);}
  	

#endif









