/******************************************************
*
*	TwoStateGenome.cp
*
*	This class implements a Phenotype of TwoStateAlleles.  
*	It handles cloning and sexual reproduction.
*
*	Base class:  Phenotype
*
*******************************************************/
#ifndef TwoStateGenomecp
#define TwoStateGenomecp = 1

#include "TwoStateGenome.h"
#include "RandomList.cpp"
#include "Element.cpp"
#include "aList.cpp"






TwoStateGenome::TwoStateGenome( int numLoci, int ploidy,int alleleVal):
	loci_(numLoci),ploidy_(ploidy), genes_(NULL){
  	//creates a 'null' homozygous individual identical at all loci
	genes_ = NEW TwoStateAllele[numLoci*ploidy];
  	for(int loc=0;loc<numLoci;++loc){
  		for(int a=0;a<ploidy;++a){
  			int index = loc*ploidy + a;
  			genes_[index].set(alleleVal);
  			}}
   	}//end constructor TwoStateGenome::TwoStateGenome(int,int,int)



  			
TwoStateGenome::TwoStateGenome(int numLoci, int ploidy,
									int alleleVal, 
									ViabilitySelectionFunc viabSelection):
		CoreGenome(viabSelection),loci_(numLoci),ploidy_(ploidy), genes_(NULL){
  	//creates a 'null' homozygous individual at all loci
	genes_ = NEW TwoStateAllele[numLoci*ploidy];
  	for(int i=0;i<numLoci;++i){
  		for(int a=0;a<ploidy;++a){
  			int index = i*ploidy + a;
  			genes_[index].set(alleleVal);
  			}}
 	}//end constructor TwoStateGenome::TwoStateGenome(int,int,int,ViabilitySelectionFunc)

TwoStateGenome::TwoStateGenome(TwoStateGenome& g): 
  			CoreGenome(g),loci_(g.loci_),ploidy_(g.ploidy_),genes_(NULL){
	int alleleSlots =loci()*ploidy(0);
	genes_ = NEW TwoStateAllele[alleleSlots];
	for (int loc=0;loc<loci();++loc){
		for (int a=0;a<ploidy(loc);++a){
			int index = loc*ploidy(loc)+a;
			genes_[index].set(g.genes_[index].product());
			}
		}//end for loc
  		}//end constructor TwoStateGenome::TwoStateGenome(TwoStateGenome&)


TwoStateGenome::TwoStateGenome(TwoStateGenome* g): 
  			CoreGenome(g),loci_(g->loci_),ploidy_(g->ploidy_),genes_(NULL){
	int alleleSlots =loci()*ploidy(0);
	genes_ = NEW TwoStateAllele[alleleSlots];
	for (int loc=0;loc<loci();++loc){
		for (int a=0;a<ploidy(loc);++a){
			int index = loc*ploidy(loc)+a;
			genes_[index].set(g->genes_[index].product());
			}
		}//end for loc
		
  		}//end constructor TwoStateGenome::TwoStateGenome(TwoStateGenome*)



    		
double 	TwoStateGenome::additiveRelativePhenotype(void){//returns additive phenotype
	double sum=0.0;//sum the different pathways for the phenotype
	for(int loc=0;loc<loci();++loc){
		for(int a=0;a<ploidy(loc);++a){
			int index = loc*ploidy(loc)+a;
			sum += genes_[index].product();
			}}
	return sum;
	}//end TwoStateGenome::additiveRelativePhenotype(void)


int	TwoStateGenome::AllelePresent (int loc, aString& alleleVal){
	int theValue = alleleVal.toLong();
	for(int a=0;a<ploidy(loc);++a){
		int index = loc*ploidy(loc)+a;
		if(theValue == genes_[index].product()){
			return 1;}
		}
	return 0;
	}//end TwoStateGenome::AllelePresent



int	TwoStateGenome::CountAlleles (int loc, aString& alleleVal){
				//counts the absolute frequency of alleleVal
	int theValue = alleleVal.toLong();
	int num = 0;
	for(int a=0;a<ploidy(loc);++a){
		int index = loc*ploidy(loc)+a;
		if(theValue == genes_[index].product()){
			++num;}
		}
	return num;
	}//end TwoStateGenome::CountAlleles



void TwoStateGenome::Fertilize (CoreGenome* momGamete, CoreGenome* dadGamete){
	//transfers parent genes to new zygote (=*this) during reproduction
	delete [] genes_;
	TwoStateGenome* mommyGamete = (TwoStateGenome*) momGamete;
	TwoStateGenome* daddyGamete = (TwoStateGenome*) dadGamete;
	loci_ = mommyGamete->loci();
	ploidy_ = mommyGamete->ploidy(0)+daddyGamete->ploidy(0);
	int alleleSlots =loci()*ploidy(0);
	genes_ = NEW TwoStateAllele[alleleSlots];
	for (int loc=0;loc<mommyGamete->loci();++loc){
		int index1;
		for (int a=0;a<mommyGamete->ploidy(0);++a){
			int index0 = loc*mommyGamete->ploidy(0)+a;
			index1 = loc*ploidy(loc)+a;
			genes_[index1].set(mommyGamete->genes_[index0].product());
			}
		for (int a=0;a<daddyGamete->ploidy(0);++a){
			int index2 = loc*daddyGamete->ploidy(0)+a;
			int index3 = index1+a+1;
			genes_[index3].set(daddyGamete->genes_[index2].product());
			}
		}//end for loc
	
	}//TwoStateGenome::Fertilize



void TwoStateGenome::Gamete (CoreGenome* theParentGenes, Ran3List& ran, double mutationRate, double /*sigma*/){
	//does meiosis to turn the genotype into a gamete of parentGenes
	//robust to haplodiploidy, genetic sex determination of XY or XO type, and polyploidy
	delete [] genes_;
	TwoStateGenome* parentGenes = (TwoStateGenome*) theParentGenes;
//	TwoStateGenome* oldGenes = (TwoStateGenome*) parentGenes->Duplicate();
	loci_ = parentGenes->loci();
  	int chromsToMake = parentGenes->ploidy(0);
  	if (chromsToMake>1){//odd chromosome may or may not join
  			chromsToMake = parentGenes->ploidy(0)/2;
  			double pl = (double)parentGenes->ploidy(0);
  			if ((MOD(pl, 2.0) != 0.0) && (ran() < 0.5)){
  				++chromsToMake;}
  				}
  	ploidy_ = chromsToMake;
	//work with a new copy to sample without replacement
	//and to avoid permanent recombination
	//determine if all loci haploid --robust to genetic sex determination
	genes_ = NEW TwoStateAllele[loci_*ploidy_];
	for (int loc=0;loc<loci();++loc){
		aList<int> indices;
		for(int a=0;a<parentGenes->ploidy(loc);++a){
			indices.Append(loc*parentGenes->ploidy(loc) + a);}
   		for(int ch=0;ch<ploidy(loc);++ch){
  			//samples parentGenes without replacement for proper treatment of polyploids
 			long a=ran.longBetween(0,indices.len()-1);
 			TwoStateAllele& theAllele = parentGenes->genes_[indices[a]];
 			int index2 = loc*ploidy(loc) + ch;
 			genes_[index2] = theAllele;//copies
 			indices.Delete(a);
 			if ((mutationRate>0.0) && (ran()<mutationRate)){
 				genes_[index2].Mutate();}
  			}//end for ch
  		}//end for loc
//	delete oldGenes;oldGenes=0;
	}//end TwoStateGenome::Gamete





aString*  TwoStateGenome::GenotypeString(char* separator){
					//returns all ploidy copies separated by parentheses (A)(A) at each locus
					//this permits complex alleles to be represented easily
	//returns all ploidy copies separated by parentheses (A)(A) at each locus
	//this permits complex alleles to be represented easily
	aString* gtstr = NEW aString();
	for (int locus=0;locus<loci();++locus){
		aString* gstr = GenotypeString(locus,separator);
		gtstr->cat(*gstr);
		gtstr->cat(separator);
		delete gstr;gstr=NULL;
		}//end for locus
	return gtstr;
  	}//end TwoStateGenome::GenotypeString(char*)



aString* TwoStateGenome::GenotypeString(int locus, char* separator){
					//finds nth overall locus and returns all ploidy copies separated by parentheses (A)(A)
					//this permits complex alleles to be represented easily
	aString* gtstr = NEW aString();
	char spar='(', epar=')';
	for(int a=0;a<ploidy(locus);++a){
		aString* gstr = GenotypeString(locus,a,separator);
		gstr->prepend(spar);gstr->cat(epar);
		gtstr->cat(*gstr);
		delete gstr;gstr=NULL;
		}//end for a
	return gtstr;
  	}//end TwoStateGenome::GenotypeString
  	


aString* TwoStateGenome::GenotypeString(int locus, int alleleNum,  char* separator){
	//finds nth overall locus and returns the alleleNum_th copy
	int index = locus*ploidy(locus)+alleleNum;
	return genes_[index].GenotypeString(separator);
  	}//end TwoStateGenome::GenotypeString
  	



int	TwoStateGenome::IsHeterozygote (int locus){
				//returns false=0 if haploid
	if(ploidy(locus)<2){return 0;}
	int index = locus*ploidy(locus);
	TwoStateAllele& alleleVal = genes_[locus*ploidy(locus)];
	for(int a=1;a<ploidy(locus);++a){
		if(alleleVal != genes_[index+a]){
			return 1;}
		}
	return 0;
	}//end TwoStateGenome::IsHeterozygote


int TwoStateGenome::loci(void){
	return loci_;
	}//end TwoStateGenome::loci




void TwoStateGenome::Mutate(Ran3& ran, double mutationRate, double /*sigma*/){
	for(int loc=0;loc<loci();loc++){
		for(int a=0;a<ploidy(loc);++a){
			if(ran()<mutationRate){
				int index = loc*ploidy(loc)+a;
				genes_[index].Mutate();}
			}}
	}//end TwoStateGenome::Mutate(Ran3&, double, double){



void TwoStateGenome::Print(ostream& outFile, int alleleLen){
	aString geneStr;//dummy starter value will be cleared anyway
	SetGenotypeString(geneStr,alleleLen);
	outFile<<geneStr<<endl;
	}//end TwoStateGenome::Print




void TwoStateGenome::set(int locus, int alleleNum, int newAlleleVal){
	//changes the value of an allele
	int index = locus*ploidy(locus)+alleleNum;
  	genes_[index].MutateTo(newAlleleVal);
  	}//end TwoStateGenome::set
	


void TwoStateGenome::SetGenotypeString(aString& geneStr, int){
	geneStr.clear();
	char* sepChar = " ";
	for(int i=0;i<trait_.len();++i){
		geneStr.cat(trait(i));//the major phenotypes
		geneStr.cat(sepChar);}
	for(int loc=0;loc<loci();++loc){
	  	for(int a=0;a<ploidy(loc);++a){
	  		int index = loc*ploidy(loc)+a;
  			TwoStateAllele& r = genes_[index];
	  		aString* allStr= r.GenotypeString(sepChar);
	  		geneStr.cat(*allStr);
	  		delete allStr; allStr = NULL;
	  		if(loc != loci()-1){
	  				geneStr.cat(sepChar);}
	  			else{
	  				geneStr.cat(" || ");}
	  		}//end for a
	  	}//end for loc
	}//end TwoStateGenome::SetGenotypeString(aString&, int)


	
void TwoStateGenome::SetPhenotype(void){
	//an additive phenotype is used
	double sum=0.0;//sum the different pathways for the phenotype
	for(int loc=0;loc<loci();++loc){
		for(int a=0;a<ploidy(loc);++a){
			int index = loc*ploidy(loc) +a;
			sum += genes_[index].product();
			}}
	SetTrait(sum,0);
	}//end TwoStateGenome::SetPhenotype(void)



void TwoStateGenome::SetPhenotype(Ran3& ran, double heritability){
	/*default is additive phenotype*/
	SetPhenotype();
	if (heritability <1.0){
		double sum=trait();
		sum += ran.NormDev(0.0,1.0-heritability);
		SetTrait(sum,0);
		}
	}//end TwoStateGenome::SetPhenotype(ran,double)



void TwoStateGenome::SetSexualZygote(CoreGenome* momGenes, CoreGenome* dadGenes, Ran3List& ran, double mutationRate, double sigma){
	//uses mom's and dad's genes to seed the genotype
	TwoStateGenome* mommyGenes = (TwoStateGenome*) momGenes;
	TwoStateGenome* daddyGenes = (TwoStateGenome*) dadGenes;
	TwoStateGenome* momGamete = (TwoStateGenome*) mommyGenes->Duplicate();
	TwoStateGenome* dadGamete = (TwoStateGenome*) daddyGenes->Duplicate();
	momGamete->Gamete(momGenes,ran,mutationRate,sigma);//genes copied to gamete
	dadGamete->Gamete(dadGenes,ran,mutationRate,sigma);
	Fertilize(momGamete, dadGamete);//momGamete & dadGamete are emptied in this step
	delete momGamete; momGamete=NULL;
	delete dadGamete; dadGamete=NULL;
	}//end TwoStateGenome::SetSexualZygote




TwoStateGenome& TwoStateGenome::operator=(TwoStateGenome& c){
	Phenotype::Copy(c);	
	delete [] genes_;
	loci_ = c.loci();
	ploidy_ = c.ploidy(0);
	int alleleSlots =loci()*ploidy(0);
	genes_ = NEW TwoStateAllele[alleleSlots];
	for (int loc=0;loc<loci();++loc){
		for (int a=0;a<ploidy(loc);++a){
			int index = loc*ploidy(loc)+a;
			genes_[index].set(c.genes_[index].product());
			}
		}//end for loc
	return *this;
	}//end TwoStateGenome::operator=(TwoStateGenome& c)



int operator==(TwoStateGenome& a, TwoStateGenome& b){
	if((a.traits() != b.traits()) || (a.loci() != b.loci()) || (a.ploidy(0) != b.ploidy(0))){
		return 0;}//false
	for(int loc=0;loc<a.loci();++loc){
		for(int ch=0;ch<a.ploidy(loc);++ch){
			int index = loc*a.ploidy(loc)+ch;
			if(a.genes_[index] != b.genes_[index]){
				return 0;}//false
			}}
	return 1;//true
	}//end operator==(TwoStateGenome& a, TwoStateGenome& b)



int operator!=(TwoStateGenome& a, TwoStateGenome& b){
			return ! (a==b);}
	


	


#endif




